/* Automatically @generated by tree-sitter v0.25.5 */

#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 6504
#define LARGE_STATE_COUNT 1951
#define SYMBOL_COUNT 374
#define ALIAS_COUNT 5
#define TOKEN_COUNT 164
#define EXTERNAL_TOKEN_COUNT 14
#define FIELD_COUNT 35
#define MAX_ALIAS_SEQUENCE_LENGTH 12
#define MAX_RESERVED_WORD_SET_SIZE 0
#define PRODUCTION_ID_COUNT 625
#define SUPERTYPE_COUNT 0

enum ts_symbol_identifiers {
  sym_identifier = 1,
  anon_sym_SEMI = 2,
  anon_sym_macro = 3,
  anon_sym_LPAREN = 4,
  anon_sym_RPAREN = 5,
  anon_sym_LBRACE = 6,
  anon_sym_RBRACE = 7,
  anon_sym_COMMA = 8,
  anon_sym_EQ_GT = 9,
  anon_sym_macro_rules = 10,
  anon_sym_BANG = 11,
  anon_sym_LBRACK = 12,
  anon_sym_RBRACK = 13,
  anon_sym_COLON = 14,
  anon_sym_DOLLAR = 15,
  aux_sym_token_repetition_pattern_token1 = 16,
  anon_sym_PLUS = 17,
  anon_sym_STAR = 18,
  anon_sym_QMARK = 19,
  anon_sym_block = 20,
  anon_sym_expr = 21,
  anon_sym_expr_2021 = 22,
  anon_sym_ident = 23,
  anon_sym_item = 24,
  anon_sym_lifetime = 25,
  anon_sym_literal = 26,
  anon_sym_meta = 27,
  anon_sym_pat = 28,
  anon_sym_pat_param = 29,
  anon_sym_path = 30,
  anon_sym_stmt = 31,
  anon_sym_tt = 32,
  anon_sym_ty = 33,
  anon_sym_vis = 34,
  anon_sym_u8 = 35,
  anon_sym_i8 = 36,
  anon_sym_u16 = 37,
  anon_sym_i16 = 38,
  anon_sym_u32 = 39,
  anon_sym_i32 = 40,
  anon_sym_u64 = 41,
  anon_sym_i64 = 42,
  anon_sym_u128 = 43,
  anon_sym_i128 = 44,
  anon_sym_isize = 45,
  anon_sym_usize = 46,
  anon_sym_f32 = 47,
  anon_sym_f64 = 48,
  anon_sym_bool = 49,
  anon_sym_str = 50,
  anon_sym_char = 51,
  anon_sym_DASH = 52,
  anon_sym_SLASH = 53,
  anon_sym_PERCENT = 54,
  anon_sym_CARET = 55,
  anon_sym_AMP = 56,
  anon_sym_PIPE = 57,
  anon_sym_AMP_AMP = 58,
  anon_sym_PIPE_PIPE = 59,
  anon_sym_LT_LT = 60,
  anon_sym_GT_GT = 61,
  anon_sym_PLUS_EQ = 62,
  anon_sym_DASH_EQ = 63,
  anon_sym_STAR_EQ = 64,
  anon_sym_SLASH_EQ = 65,
  anon_sym_PERCENT_EQ = 66,
  anon_sym_CARET_EQ = 67,
  anon_sym_AMP_EQ = 68,
  anon_sym_PIPE_EQ = 69,
  anon_sym_LT_LT_EQ = 70,
  anon_sym_GT_GT_EQ = 71,
  anon_sym_EQ = 72,
  anon_sym_EQ_EQ = 73,
  anon_sym_BANG_EQ = 74,
  anon_sym_GT = 75,
  anon_sym_LT = 76,
  anon_sym_GT_EQ = 77,
  anon_sym_LT_EQ = 78,
  anon_sym_AT = 79,
  anon_sym__ = 80,
  anon_sym_DOT = 81,
  anon_sym_DOT_DOT = 82,
  anon_sym_DOT_DOT_DOT = 83,
  anon_sym_DOT_DOT_EQ = 84,
  anon_sym_COLON_COLON = 85,
  anon_sym_DASH_GT = 86,
  anon_sym_POUND = 87,
  anon_sym_SQUOTE = 88,
  anon_sym_as = 89,
  anon_sym_async = 90,
  anon_sym_auto = 91,
  anon_sym_await = 92,
  anon_sym_become = 93,
  anon_sym_break = 94,
  anon_sym_const = 95,
  anon_sym_continue = 96,
  anon_sym_default = 97,
  anon_sym_enum = 98,
  anon_sym_fn = 99,
  anon_sym_for = 100,
  anon_sym_gen = 101,
  anon_sym_if = 102,
  anon_sym_impl = 103,
  anon_sym_let = 104,
  anon_sym_loop = 105,
  anon_sym_match = 106,
  anon_sym_mod = 107,
  anon_sym_pub = 108,
  anon_sym_return = 109,
  anon_sym_raw = 110,
  anon_sym_static = 111,
  anon_sym_struct = 112,
  anon_sym_trait = 113,
  anon_sym_type = 114,
  anon_sym_union = 115,
  anon_sym_unsafe = 116,
  anon_sym_use = 117,
  anon_sym_where = 118,
  anon_sym_while = 119,
  anon_sym_extern = 120,
  anon_sym_safe = 121,
  anon_sym_ref = 122,
  anon_sym_TILDEconst = 123,
  anon_sym_else = 124,
  anon_sym_in = 125,
  anon_sym_LT2 = 126,
  anon_sym_dyn = 127,
  sym_mutable_specifier = 128,
  anon_sym_yield = 129,
  anon_sym_move = 130,
  anon_sym_try = 131,
  anon_sym_box = 132,
  sym_integer_literal = 133,
  aux_sym_string_literal_token1 = 134,
  anon_sym_DQUOTE = 135,
  sym_char_literal = 136,
  sym_escape_sequence = 137,
  anon_sym_true = 138,
  anon_sym_false = 139,
  anon_sym_SLASH_SLASH = 140,
  aux_sym_line_comment_token1 = 141,
  anon_sym_BANG2 = 142,
  anon_sym_STAR_SLASH = 143,
  anon_sym_SLASH_STAR = 144,
  sym_shebang = 145,
  sym_self = 146,
  sym_super = 147,
  sym_crate = 148,
  aux_sym_metavariable_token1 = 149,
  sym_string_content = 150,
  sym__raw_string_literal_start = 151,
  sym_raw_string_literal_content = 152,
  sym__raw_string_literal_end = 153,
  sym_float_literal = 154,
  sym__outer_block_doc_comment_start = 155,
  sym__outer_line_doc_comment_start = 156,
  sym__inner_block_doc_comment_marker = 157,
  sym__block_comment_content = 158,
  sym__line_doc_content = 159,
  sym__frontmatter_start = 160,
  sym_frontmatter_content = 161,
  sym__frontmatter_end = 162,
  sym__error_sentinel = 163,
  sym_source_file = 164,
  sym__statement = 165,
  sym_empty_statement = 166,
  sym_expression_statement = 167,
  sym__declaration_statement = 168,
  sym_macro_definition_v2 = 169,
  sym_macro_arguments_declaration = 170,
  sym_macro_body_v2 = 171,
  sym_macro_rules_v2 = 172,
  sym_macro_rule_v2 = 173,
  sym_macro_definition = 174,
  sym_macro_rule = 175,
  sym__token_pattern = 176,
  sym_token_tree_pattern = 177,
  sym_token_binding_pattern = 178,
  sym_token_repetition_pattern = 179,
  sym_fragment_specifier = 180,
  sym_token_tree = 181,
  sym_token_repetition = 182,
  sym_primitive_type = 183,
  sym_attribute_item = 184,
  sym_attributes = 185,
  sym_inner_attribute_item = 186,
  sym_attribute = 187,
  sym_mod_item = 188,
  sym_foreign_mod_item = 189,
  sym_declaration_list = 190,
  sym_struct_item = 191,
  sym_union_item = 192,
  sym_enum_item = 193,
  sym_enum_variant_list = 194,
  sym_enum_variant = 195,
  sym_field_declaration_list = 196,
  sym_field_declaration = 197,
  sym_ordered_field_declaration_list = 198,
  sym_extern_crate_declaration = 199,
  sym_const_item = 200,
  sym_static_item = 201,
  sym_type_item = 202,
  sym_function_item = 203,
  sym_function_signature_item = 204,
  sym_function_modifiers = 205,
  aux_sym__item_modifiers = 206,
  sym_where_clause = 207,
  sym_where_predicate = 208,
  sym_impl_item = 209,
  sym_trait_item = 210,
  sym_trait_bounds = 211,
  sym_higher_ranked_trait_bound = 212,
  sym_removed_trait_bound = 213,
  sym_type_parameters = 214,
  sym_const_parameter = 215,
  sym_type_parameter = 216,
  sym_lifetime_parameter = 217,
  sym_let_declaration = 218,
  sym_use_declaration = 219,
  sym__use_clause = 220,
  sym_scoped_use_list = 221,
  sym_use_list = 222,
  sym_use_as_clause = 223,
  sym_use_wildcard = 224,
  sym_parameters = 225,
  sym_self_parameter = 226,
  sym_variadic_parameter = 227,
  sym_parameter = 228,
  sym_extern_modifier = 229,
  sym_visibility_modifier = 230,
  sym__type = 231,
  sym_bracketed_type = 232,
  sym_qualified_type = 233,
  sym_lifetime = 234,
  sym_array_type = 235,
  sym_function_type = 236,
  sym_unsafe_bound_type = 237,
  sym_tuple_type = 238,
  sym_unit_type = 239,
  sym_generic_function = 240,
  sym_generic_type = 241,
  sym_generic_type_with_turbofish = 242,
  sym_bounded_type = 243,
  sym_use_bounds = 244,
  sym_type_arguments = 245,
  sym_type_binding = 246,
  sym_reference_type = 247,
  sym_pointer_type = 248,
  sym_never_type = 249,
  sym_abstract_type = 250,
  sym_dynamic_type = 251,
  sym__expression_except_range = 252,
  sym__expression = 253,
  sym_expression_with_attribute = 254,
  sym__expression_without_attribute = 255,
  sym_block_expression_with_attribute = 256,
  sym_macro_invocation_item = 257,
  sym_macro_invocation = 258,
  sym_delim_token_tree = 259,
  sym__delim_tokens = 260,
  sym__non_delim_token = 261,
  sym_scoped_identifier = 262,
  sym_scoped_type_identifier_in_expression_position = 263,
  sym_scoped_type_identifier = 264,
  sym_range_expression = 265,
  sym_unary_expression = 266,
  sym_try_expression = 267,
  sym_reference_expression = 268,
  sym_binary_expression = 269,
  sym_assignment_expression = 270,
  sym_compound_assignment_expr = 271,
  sym_type_cast_expression = 272,
  sym_return_expression = 273,
  sym_become_expression = 274,
  sym_yield_expression = 275,
  sym_call_expression = 276,
  sym_arguments = 277,
  sym_array_expression = 278,
  sym_parenthesized_expression = 279,
  sym_tuple_expression = 280,
  sym_unit_expression = 281,
  sym_struct_expression = 282,
  sym_field_initializer_list = 283,
  sym_shorthand_field_initializer = 284,
  sym_field_initializer = 285,
  sym_base_field_initializer = 286,
  sym_if_expression = 287,
  sym_let_condition = 288,
  sym__let_chain = 289,
  sym__condition = 290,
  sym_else_clause = 291,
  sym_match_expression = 292,
  sym_match_block = 293,
  sym_match_arm = 294,
  sym_last_match_arm = 295,
  sym_match_pattern = 296,
  sym_while_expression = 297,
  sym_loop_expression = 298,
  sym_for_expression = 299,
  sym_const_block = 300,
  sym_closure_expression = 301,
  sym_closure_parameters = 302,
  sym_label = 303,
  sym_break_expression = 304,
  sym_continue_expression = 305,
  sym_index_expression = 306,
  sym_await_expression = 307,
  sym_field_expression = 308,
  sym_unsafe_block = 309,
  sym_async_block = 310,
  sym_gen_block = 311,
  sym_try_block = 312,
  sym_block = 313,
  sym__pattern = 314,
  sym_generic_pattern = 315,
  sym_tuple_pattern = 316,
  sym_slice_pattern = 317,
  sym_tuple_struct_pattern = 318,
  sym_struct_pattern = 319,
  sym_field_pattern = 320,
  sym_remaining_field_pattern = 321,
  sym_mut_pattern = 322,
  sym_range_pattern = 323,
  sym_ref_pattern = 324,
  sym_box_pattern = 325,
  sym_captured_pattern = 326,
  sym_reference_pattern = 327,
  sym_or_pattern = 328,
  sym__literal = 329,
  sym__literal_pattern = 330,
  sym_negative_literal = 331,
  sym_string_literal = 332,
  sym_raw_string_literal = 333,
  sym_boolean_literal = 334,
  sym_line_comment = 335,
  sym__inner_line_doc_comment_marker = 336,
  sym_line_outer_doc_comment = 337,
  sym_block_outer_doc_comment = 338,
  sym_block_comment = 339,
  sym_frontmatter = 340,
  sym_metavariable = 341,
  aux_sym_source_file_repeat1 = 342,
  aux_sym_macro_arguments_declaration_repeat1 = 343,
  aux_sym_macro_body_v2_repeat1 = 344,
  aux_sym_macro_rules_v2_repeat1 = 345,
  aux_sym_macro_definition_repeat1 = 346,
  aux_sym__non_special_token_repeat1 = 347,
  aux_sym_attributes_repeat1 = 348,
  aux_sym_declaration_list_repeat1 = 349,
  aux_sym_enum_variant_list_repeat1 = 350,
  aux_sym_field_declaration_list_repeat1 = 351,
  aux_sym_ordered_field_declaration_list_repeat1 = 352,
  aux_sym_where_clause_repeat1 = 353,
  aux_sym_trait_bounds_repeat1 = 354,
  aux_sym_type_parameters_repeat1 = 355,
  aux_sym_use_list_repeat1 = 356,
  aux_sym_parameters_repeat1 = 357,
  aux_sym_unsafe_bound_type_repeat1 = 358,
  aux_sym_tuple_type_repeat1 = 359,
  aux_sym_use_bounds_repeat1 = 360,
  aux_sym_type_arguments_repeat1 = 361,
  aux_sym_delim_token_tree_repeat1 = 362,
  aux_sym_arguments_repeat1 = 363,
  aux_sym_array_expression_repeat1 = 364,
  aux_sym_tuple_expression_repeat1 = 365,
  aux_sym_field_initializer_list_repeat1 = 366,
  aux_sym_match_block_repeat1 = 367,
  aux_sym_match_block_repeat2 = 368,
  aux_sym_closure_parameters_repeat1 = 369,
  aux_sym_tuple_pattern_repeat1 = 370,
  aux_sym_slice_pattern_repeat1 = 371,
  aux_sym_struct_pattern_repeat1 = 372,
  aux_sym_string_literal_repeat1 = 373,
  alias_sym_field_ellipsis = 374,
  alias_sym_field_identifier = 375,
  alias_sym_let_chain = 376,
  alias_sym_shorthand_field_identifier = 377,
  alias_sym_type_identifier = 378,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [anon_sym_SEMI] = ";",
  [anon_sym_macro] = "macro",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_COMMA] = ",",
  [anon_sym_EQ_GT] = "=>",
  [anon_sym_macro_rules] = "macro_rules",
  [anon_sym_BANG] = "!",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_COLON] = ":",
  [anon_sym_DOLLAR] = "$",
  [aux_sym_token_repetition_pattern_token1] = "token_repetition_pattern_token1",
  [anon_sym_PLUS] = "+",
  [anon_sym_STAR] = "*",
  [anon_sym_QMARK] = "\?",
  [anon_sym_block] = "block",
  [anon_sym_expr] = "expr",
  [anon_sym_expr_2021] = "expr_2021",
  [anon_sym_ident] = "ident",
  [anon_sym_item] = "item",
  [anon_sym_lifetime] = "lifetime",
  [anon_sym_literal] = "literal",
  [anon_sym_meta] = "meta",
  [anon_sym_pat] = "pat",
  [anon_sym_pat_param] = "pat_param",
  [anon_sym_path] = "path",
  [anon_sym_stmt] = "stmt",
  [anon_sym_tt] = "tt",
  [anon_sym_ty] = "ty",
  [anon_sym_vis] = "vis",
  [anon_sym_u8] = "u8",
  [anon_sym_i8] = "i8",
  [anon_sym_u16] = "u16",
  [anon_sym_i16] = "i16",
  [anon_sym_u32] = "u32",
  [anon_sym_i32] = "i32",
  [anon_sym_u64] = "u64",
  [anon_sym_i64] = "i64",
  [anon_sym_u128] = "u128",
  [anon_sym_i128] = "i128",
  [anon_sym_isize] = "isize",
  [anon_sym_usize] = "usize",
  [anon_sym_f32] = "f32",
  [anon_sym_f64] = "f64",
  [anon_sym_bool] = "bool",
  [anon_sym_str] = "str",
  [anon_sym_char] = "char",
  [anon_sym_DASH] = "-",
  [anon_sym_SLASH] = "/",
  [anon_sym_PERCENT] = "%",
  [anon_sym_CARET] = "^",
  [anon_sym_AMP] = "&",
  [anon_sym_PIPE] = "|",
  [anon_sym_AMP_AMP] = "&&",
  [anon_sym_PIPE_PIPE] = "||",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_PLUS_EQ] = "+=",
  [anon_sym_DASH_EQ] = "-=",
  [anon_sym_STAR_EQ] = "*=",
  [anon_sym_SLASH_EQ] = "/=",
  [anon_sym_PERCENT_EQ] = "%=",
  [anon_sym_CARET_EQ] = "^=",
  [anon_sym_AMP_EQ] = "&=",
  [anon_sym_PIPE_EQ] = "|=",
  [anon_sym_LT_LT_EQ] = "<<=",
  [anon_sym_GT_GT_EQ] = ">>=",
  [anon_sym_EQ] = "=",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_GT] = ">",
  [anon_sym_LT] = "<",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_AT] = "@",
  [anon_sym__] = "_",
  [anon_sym_DOT] = ".",
  [anon_sym_DOT_DOT] = "..",
  [anon_sym_DOT_DOT_DOT] = "...",
  [anon_sym_DOT_DOT_EQ] = "..=",
  [anon_sym_COLON_COLON] = "::",
  [anon_sym_DASH_GT] = "->",
  [anon_sym_POUND] = "#",
  [anon_sym_SQUOTE] = "'",
  [anon_sym_as] = "as",
  [anon_sym_async] = "async",
  [anon_sym_auto] = "auto",
  [anon_sym_await] = "await",
  [anon_sym_become] = "become",
  [anon_sym_break] = "break",
  [anon_sym_const] = "const",
  [anon_sym_continue] = "continue",
  [anon_sym_default] = "default",
  [anon_sym_enum] = "enum",
  [anon_sym_fn] = "fn",
  [anon_sym_for] = "for",
  [anon_sym_gen] = "gen",
  [anon_sym_if] = "if",
  [anon_sym_impl] = "impl",
  [anon_sym_let] = "let",
  [anon_sym_loop] = "loop",
  [anon_sym_match] = "match",
  [anon_sym_mod] = "mod",
  [anon_sym_pub] = "pub",
  [anon_sym_return] = "return",
  [anon_sym_raw] = "raw",
  [anon_sym_static] = "static",
  [anon_sym_struct] = "struct",
  [anon_sym_trait] = "trait",
  [anon_sym_type] = "type",
  [anon_sym_union] = "union",
  [anon_sym_unsafe] = "unsafe",
  [anon_sym_use] = "use",
  [anon_sym_where] = "where",
  [anon_sym_while] = "while",
  [anon_sym_extern] = "extern",
  [anon_sym_safe] = "safe",
  [anon_sym_ref] = "ref",
  [anon_sym_TILDEconst] = "~const",
  [anon_sym_else] = "else",
  [anon_sym_in] = "in",
  [anon_sym_LT2] = "<",
  [anon_sym_dyn] = "dyn",
  [sym_mutable_specifier] = "mutable_specifier",
  [anon_sym_yield] = "yield",
  [anon_sym_move] = "move",
  [anon_sym_try] = "try",
  [anon_sym_box] = "box",
  [sym_integer_literal] = "integer_literal",
  [aux_sym_string_literal_token1] = "\"",
  [anon_sym_DQUOTE] = "\"",
  [sym_char_literal] = "char_literal",
  [sym_escape_sequence] = "escape_sequence",
  [anon_sym_true] = "true",
  [anon_sym_false] = "false",
  [anon_sym_SLASH_SLASH] = "//",
  [aux_sym_line_comment_token1] = "line_comment_token1",
  [anon_sym_BANG2] = "!",
  [anon_sym_STAR_SLASH] = "*/",
  [anon_sym_SLASH_STAR] = "/*",
  [sym_shebang] = "shebang",
  [sym_self] = "self",
  [sym_super] = "super",
  [sym_crate] = "crate",
  [aux_sym_metavariable_token1] = "metavariable_token1",
  [sym_string_content] = "string_content",
  [sym__raw_string_literal_start] = "_raw_string_literal_start",
  [sym_raw_string_literal_content] = "string_content",
  [sym__raw_string_literal_end] = "_raw_string_literal_end",
  [sym_float_literal] = "float_literal",
  [sym__outer_block_doc_comment_start] = "_outer_block_doc_comment_start",
  [sym__outer_line_doc_comment_start] = "_outer_line_doc_comment_start",
  [sym__inner_block_doc_comment_marker] = "inner_doc_comment_marker",
  [sym__block_comment_content] = "_block_comment_content",
  [sym__line_doc_content] = "doc_comment",
  [sym__frontmatter_start] = "_frontmatter_start",
  [sym_frontmatter_content] = "frontmatter_content",
  [sym__frontmatter_end] = "_frontmatter_end",
  [sym__error_sentinel] = "_error_sentinel",
  [sym_source_file] = "source_file",
  [sym__statement] = "_statement",
  [sym_empty_statement] = "empty_statement",
  [sym_expression_statement] = "expression_statement",
  [sym__declaration_statement] = "_declaration_statement",
  [sym_macro_definition_v2] = "macro_definition_v2",
  [sym_macro_arguments_declaration] = "macro_arguments_declaration",
  [sym_macro_body_v2] = "macro_body_v2",
  [sym_macro_rules_v2] = "macro_rules_v2",
  [sym_macro_rule_v2] = "macro_rule_v2",
  [sym_macro_definition] = "macro_definition",
  [sym_macro_rule] = "macro_rule",
  [sym__token_pattern] = "_token_pattern",
  [sym_token_tree_pattern] = "token_tree_pattern",
  [sym_token_binding_pattern] = "token_binding_pattern",
  [sym_token_repetition_pattern] = "token_repetition_pattern",
  [sym_fragment_specifier] = "fragment_specifier",
  [sym_token_tree] = "token_tree",
  [sym_token_repetition] = "token_repetition",
  [sym_primitive_type] = "primitive_type",
  [sym_attribute_item] = "attribute_item",
  [sym_attributes] = "attributes",
  [sym_inner_attribute_item] = "inner_attribute_item",
  [sym_attribute] = "attribute",
  [sym_mod_item] = "mod_item",
  [sym_foreign_mod_item] = "foreign_mod_item",
  [sym_declaration_list] = "declaration_list",
  [sym_struct_item] = "struct_item",
  [sym_union_item] = "union_item",
  [sym_enum_item] = "enum_item",
  [sym_enum_variant_list] = "enum_variant_list",
  [sym_enum_variant] = "enum_variant",
  [sym_field_declaration_list] = "field_declaration_list",
  [sym_field_declaration] = "field_declaration",
  [sym_ordered_field_declaration_list] = "ordered_field_declaration_list",
  [sym_extern_crate_declaration] = "extern_crate_declaration",
  [sym_const_item] = "const_item",
  [sym_static_item] = "static_item",
  [sym_type_item] = "type_item",
  [sym_function_item] = "function_item",
  [sym_function_signature_item] = "function_signature_item",
  [sym_function_modifiers] = "function_modifiers",
  [aux_sym__item_modifiers] = "_item_modifiers",
  [sym_where_clause] = "where_clause",
  [sym_where_predicate] = "where_predicate",
  [sym_impl_item] = "impl_item",
  [sym_trait_item] = "trait_item",
  [sym_trait_bounds] = "trait_bounds",
  [sym_higher_ranked_trait_bound] = "higher_ranked_trait_bound",
  [sym_removed_trait_bound] = "removed_trait_bound",
  [sym_type_parameters] = "type_parameters",
  [sym_const_parameter] = "const_parameter",
  [sym_type_parameter] = "type_parameter",
  [sym_lifetime_parameter] = "lifetime_parameter",
  [sym_let_declaration] = "let_declaration",
  [sym_use_declaration] = "use_declaration",
  [sym__use_clause] = "_use_clause",
  [sym_scoped_use_list] = "scoped_use_list",
  [sym_use_list] = "use_list",
  [sym_use_as_clause] = "use_as_clause",
  [sym_use_wildcard] = "use_wildcard",
  [sym_parameters] = "parameters",
  [sym_self_parameter] = "self_parameter",
  [sym_variadic_parameter] = "variadic_parameter",
  [sym_parameter] = "parameter",
  [sym_extern_modifier] = "extern_modifier",
  [sym_visibility_modifier] = "visibility_modifier",
  [sym__type] = "_type",
  [sym_bracketed_type] = "bracketed_type",
  [sym_qualified_type] = "qualified_type",
  [sym_lifetime] = "lifetime",
  [sym_array_type] = "array_type",
  [sym_function_type] = "function_type",
  [sym_unsafe_bound_type] = "unsafe_bound_type",
  [sym_tuple_type] = "tuple_type",
  [sym_unit_type] = "unit_type",
  [sym_generic_function] = "generic_function",
  [sym_generic_type] = "generic_type",
  [sym_generic_type_with_turbofish] = "generic_type_with_turbofish",
  [sym_bounded_type] = "bounded_type",
  [sym_use_bounds] = "use_bounds",
  [sym_type_arguments] = "type_arguments",
  [sym_type_binding] = "type_binding",
  [sym_reference_type] = "reference_type",
  [sym_pointer_type] = "pointer_type",
  [sym_never_type] = "never_type",
  [sym_abstract_type] = "abstract_type",
  [sym_dynamic_type] = "dynamic_type",
  [sym__expression_except_range] = "_expression_except_range",
  [sym__expression] = "_expression",
  [sym_expression_with_attribute] = "expression_with_attribute",
  [sym__expression_without_attribute] = "_expression_without_attribute",
  [sym_block_expression_with_attribute] = "block_expression_with_attribute",
  [sym_macro_invocation_item] = "macro_invocation_item",
  [sym_macro_invocation] = "macro_invocation",
  [sym_delim_token_tree] = "token_tree",
  [sym__delim_tokens] = "_delim_tokens",
  [sym__non_delim_token] = "_non_delim_token",
  [sym_scoped_identifier] = "scoped_identifier",
  [sym_scoped_type_identifier_in_expression_position] = "scoped_type_identifier",
  [sym_scoped_type_identifier] = "scoped_type_identifier",
  [sym_range_expression] = "range_expression",
  [sym_unary_expression] = "unary_expression",
  [sym_try_expression] = "try_expression",
  [sym_reference_expression] = "reference_expression",
  [sym_binary_expression] = "binary_expression",
  [sym_assignment_expression] = "assignment_expression",
  [sym_compound_assignment_expr] = "compound_assignment_expr",
  [sym_type_cast_expression] = "type_cast_expression",
  [sym_return_expression] = "return_expression",
  [sym_become_expression] = "become_expression",
  [sym_yield_expression] = "yield_expression",
  [sym_call_expression] = "call_expression",
  [sym_arguments] = "arguments",
  [sym_array_expression] = "array_expression",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym_tuple_expression] = "tuple_expression",
  [sym_unit_expression] = "unit_expression",
  [sym_struct_expression] = "struct_expression",
  [sym_field_initializer_list] = "field_initializer_list",
  [sym_shorthand_field_initializer] = "shorthand_field_initializer",
  [sym_field_initializer] = "field_initializer",
  [sym_base_field_initializer] = "base_field_initializer",
  [sym_if_expression] = "if_expression",
  [sym_let_condition] = "let_condition",
  [sym__let_chain] = "_let_chain",
  [sym__condition] = "_condition",
  [sym_else_clause] = "else_clause",
  [sym_match_expression] = "match_expression",
  [sym_match_block] = "match_block",
  [sym_match_arm] = "match_arm",
  [sym_last_match_arm] = "match_arm",
  [sym_match_pattern] = "match_pattern",
  [sym_while_expression] = "while_expression",
  [sym_loop_expression] = "loop_expression",
  [sym_for_expression] = "for_expression",
  [sym_const_block] = "const_block",
  [sym_closure_expression] = "closure_expression",
  [sym_closure_parameters] = "closure_parameters",
  [sym_label] = "label",
  [sym_break_expression] = "break_expression",
  [sym_continue_expression] = "continue_expression",
  [sym_index_expression] = "index_expression",
  [sym_await_expression] = "await_expression",
  [sym_field_expression] = "field_expression",
  [sym_unsafe_block] = "unsafe_block",
  [sym_async_block] = "async_block",
  [sym_gen_block] = "gen_block",
  [sym_try_block] = "try_block",
  [sym_block] = "block",
  [sym__pattern] = "_pattern",
  [sym_generic_pattern] = "generic_pattern",
  [sym_tuple_pattern] = "tuple_pattern",
  [sym_slice_pattern] = "slice_pattern",
  [sym_tuple_struct_pattern] = "tuple_struct_pattern",
  [sym_struct_pattern] = "struct_pattern",
  [sym_field_pattern] = "field_pattern",
  [sym_remaining_field_pattern] = "remaining_field_pattern",
  [sym_mut_pattern] = "mut_pattern",
  [sym_range_pattern] = "range_pattern",
  [sym_ref_pattern] = "ref_pattern",
  [sym_box_pattern] = "box_pattern",
  [sym_captured_pattern] = "captured_pattern",
  [sym_reference_pattern] = "reference_pattern",
  [sym_or_pattern] = "or_pattern",
  [sym__literal] = "_literal",
  [sym__literal_pattern] = "_literal_pattern",
  [sym_negative_literal] = "negative_literal",
  [sym_string_literal] = "string_literal",
  [sym_raw_string_literal] = "raw_string_literal",
  [sym_boolean_literal] = "boolean_literal",
  [sym_line_comment] = "line_comment",
  [sym__inner_line_doc_comment_marker] = "inner_doc_comment_marker",
  [sym_line_outer_doc_comment] = "line_outer_doc_comment",
  [sym_block_outer_doc_comment] = "block_outer_doc_comment",
  [sym_block_comment] = "block_comment",
  [sym_frontmatter] = "frontmatter",
  [sym_metavariable] = "metavariable",
  [aux_sym_source_file_repeat1] = "source_file_repeat1",
  [aux_sym_macro_arguments_declaration_repeat1] = "macro_arguments_declaration_repeat1",
  [aux_sym_macro_body_v2_repeat1] = "macro_body_v2_repeat1",
  [aux_sym_macro_rules_v2_repeat1] = "macro_rules_v2_repeat1",
  [aux_sym_macro_definition_repeat1] = "macro_definition_repeat1",
  [aux_sym__non_special_token_repeat1] = "_non_special_token_repeat1",
  [aux_sym_attributes_repeat1] = "attributes_repeat1",
  [aux_sym_declaration_list_repeat1] = "declaration_list_repeat1",
  [aux_sym_enum_variant_list_repeat1] = "enum_variant_list_repeat1",
  [aux_sym_field_declaration_list_repeat1] = "field_declaration_list_repeat1",
  [aux_sym_ordered_field_declaration_list_repeat1] = "ordered_field_declaration_list_repeat1",
  [aux_sym_where_clause_repeat1] = "where_clause_repeat1",
  [aux_sym_trait_bounds_repeat1] = "trait_bounds_repeat1",
  [aux_sym_type_parameters_repeat1] = "type_parameters_repeat1",
  [aux_sym_use_list_repeat1] = "use_list_repeat1",
  [aux_sym_parameters_repeat1] = "parameters_repeat1",
  [aux_sym_unsafe_bound_type_repeat1] = "unsafe_bound_type_repeat1",
  [aux_sym_tuple_type_repeat1] = "tuple_type_repeat1",
  [aux_sym_use_bounds_repeat1] = "use_bounds_repeat1",
  [aux_sym_type_arguments_repeat1] = "type_arguments_repeat1",
  [aux_sym_delim_token_tree_repeat1] = "delim_token_tree_repeat1",
  [aux_sym_arguments_repeat1] = "arguments_repeat1",
  [aux_sym_array_expression_repeat1] = "array_expression_repeat1",
  [aux_sym_tuple_expression_repeat1] = "tuple_expression_repeat1",
  [aux_sym_field_initializer_list_repeat1] = "field_initializer_list_repeat1",
  [aux_sym_match_block_repeat1] = "match_block_repeat1",
  [aux_sym_match_block_repeat2] = "match_block_repeat2",
  [aux_sym_closure_parameters_repeat1] = "closure_parameters_repeat1",
  [aux_sym_tuple_pattern_repeat1] = "tuple_pattern_repeat1",
  [aux_sym_slice_pattern_repeat1] = "slice_pattern_repeat1",
  [aux_sym_struct_pattern_repeat1] = "struct_pattern_repeat1",
  [aux_sym_string_literal_repeat1] = "string_literal_repeat1",
  [alias_sym_field_ellipsis] = "field_ellipsis",
  [alias_sym_field_identifier] = "field_identifier",
  [alias_sym_let_chain] = "let_chain",
  [alias_sym_shorthand_field_identifier] = "shorthand_field_identifier",
  [alias_sym_type_identifier] = "type_identifier",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_macro] = anon_sym_macro,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_EQ_GT] = anon_sym_EQ_GT,
  [anon_sym_macro_rules] = anon_sym_macro_rules,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_DOLLAR] = anon_sym_DOLLAR,
  [aux_sym_token_repetition_pattern_token1] = aux_sym_token_repetition_pattern_token1,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_QMARK] = anon_sym_QMARK,
  [anon_sym_block] = anon_sym_block,
  [anon_sym_expr] = anon_sym_expr,
  [anon_sym_expr_2021] = anon_sym_expr_2021,
  [anon_sym_ident] = anon_sym_ident,
  [anon_sym_item] = anon_sym_item,
  [anon_sym_lifetime] = anon_sym_lifetime,
  [anon_sym_literal] = anon_sym_literal,
  [anon_sym_meta] = anon_sym_meta,
  [anon_sym_pat] = anon_sym_pat,
  [anon_sym_pat_param] = anon_sym_pat_param,
  [anon_sym_path] = anon_sym_path,
  [anon_sym_stmt] = anon_sym_stmt,
  [anon_sym_tt] = anon_sym_tt,
  [anon_sym_ty] = anon_sym_ty,
  [anon_sym_vis] = anon_sym_vis,
  [anon_sym_u8] = anon_sym_u8,
  [anon_sym_i8] = anon_sym_i8,
  [anon_sym_u16] = anon_sym_u16,
  [anon_sym_i16] = anon_sym_i16,
  [anon_sym_u32] = anon_sym_u32,
  [anon_sym_i32] = anon_sym_i32,
  [anon_sym_u64] = anon_sym_u64,
  [anon_sym_i64] = anon_sym_i64,
  [anon_sym_u128] = anon_sym_u128,
  [anon_sym_i128] = anon_sym_i128,
  [anon_sym_isize] = anon_sym_isize,
  [anon_sym_usize] = anon_sym_usize,
  [anon_sym_f32] = anon_sym_f32,
  [anon_sym_f64] = anon_sym_f64,
  [anon_sym_bool] = anon_sym_bool,
  [anon_sym_str] = anon_sym_str,
  [anon_sym_char] = anon_sym_char,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_AMP_AMP] = anon_sym_AMP_AMP,
  [anon_sym_PIPE_PIPE] = anon_sym_PIPE_PIPE,
  [anon_sym_LT_LT] = anon_sym_LT_LT,
  [anon_sym_GT_GT] = anon_sym_GT_GT,
  [anon_sym_PLUS_EQ] = anon_sym_PLUS_EQ,
  [anon_sym_DASH_EQ] = anon_sym_DASH_EQ,
  [anon_sym_STAR_EQ] = anon_sym_STAR_EQ,
  [anon_sym_SLASH_EQ] = anon_sym_SLASH_EQ,
  [anon_sym_PERCENT_EQ] = anon_sym_PERCENT_EQ,
  [anon_sym_CARET_EQ] = anon_sym_CARET_EQ,
  [anon_sym_AMP_EQ] = anon_sym_AMP_EQ,
  [anon_sym_PIPE_EQ] = anon_sym_PIPE_EQ,
  [anon_sym_LT_LT_EQ] = anon_sym_LT_LT_EQ,
  [anon_sym_GT_GT_EQ] = anon_sym_GT_GT_EQ,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [anon_sym_BANG_EQ] = anon_sym_BANG_EQ,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [anon_sym_AT] = anon_sym_AT,
  [anon_sym__] = anon_sym__,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_DOT_DOT] = anon_sym_DOT_DOT,
  [anon_sym_DOT_DOT_DOT] = anon_sym_DOT_DOT_DOT,
  [anon_sym_DOT_DOT_EQ] = anon_sym_DOT_DOT_EQ,
  [anon_sym_COLON_COLON] = anon_sym_COLON_COLON,
  [anon_sym_DASH_GT] = anon_sym_DASH_GT,
  [anon_sym_POUND] = anon_sym_POUND,
  [anon_sym_SQUOTE] = anon_sym_SQUOTE,
  [anon_sym_as] = anon_sym_as,
  [anon_sym_async] = anon_sym_async,
  [anon_sym_auto] = anon_sym_auto,
  [anon_sym_await] = anon_sym_await,
  [anon_sym_become] = anon_sym_become,
  [anon_sym_break] = anon_sym_break,
  [anon_sym_const] = anon_sym_const,
  [anon_sym_continue] = anon_sym_continue,
  [anon_sym_default] = anon_sym_default,
  [anon_sym_enum] = anon_sym_enum,
  [anon_sym_fn] = anon_sym_fn,
  [anon_sym_for] = anon_sym_for,
  [anon_sym_gen] = anon_sym_gen,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_impl] = anon_sym_impl,
  [anon_sym_let] = anon_sym_let,
  [anon_sym_loop] = anon_sym_loop,
  [anon_sym_match] = anon_sym_match,
  [anon_sym_mod] = anon_sym_mod,
  [anon_sym_pub] = anon_sym_pub,
  [anon_sym_return] = anon_sym_return,
  [anon_sym_raw] = anon_sym_raw,
  [anon_sym_static] = anon_sym_static,
  [anon_sym_struct] = anon_sym_struct,
  [anon_sym_trait] = anon_sym_trait,
  [anon_sym_type] = anon_sym_type,
  [anon_sym_union] = anon_sym_union,
  [anon_sym_unsafe] = anon_sym_unsafe,
  [anon_sym_use] = anon_sym_use,
  [anon_sym_where] = anon_sym_where,
  [anon_sym_while] = anon_sym_while,
  [anon_sym_extern] = anon_sym_extern,
  [anon_sym_safe] = anon_sym_safe,
  [anon_sym_ref] = anon_sym_ref,
  [anon_sym_TILDEconst] = anon_sym_TILDEconst,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_in] = anon_sym_in,
  [anon_sym_LT2] = anon_sym_LT,
  [anon_sym_dyn] = anon_sym_dyn,
  [sym_mutable_specifier] = sym_mutable_specifier,
  [anon_sym_yield] = anon_sym_yield,
  [anon_sym_move] = anon_sym_move,
  [anon_sym_try] = anon_sym_try,
  [anon_sym_box] = anon_sym_box,
  [sym_integer_literal] = sym_integer_literal,
  [aux_sym_string_literal_token1] = anon_sym_DQUOTE,
  [anon_sym_DQUOTE] = anon_sym_DQUOTE,
  [sym_char_literal] = sym_char_literal,
  [sym_escape_sequence] = sym_escape_sequence,
  [anon_sym_true] = anon_sym_true,
  [anon_sym_false] = anon_sym_false,
  [anon_sym_SLASH_SLASH] = anon_sym_SLASH_SLASH,
  [aux_sym_line_comment_token1] = aux_sym_line_comment_token1,
  [anon_sym_BANG2] = anon_sym_BANG,
  [anon_sym_STAR_SLASH] = anon_sym_STAR_SLASH,
  [anon_sym_SLASH_STAR] = anon_sym_SLASH_STAR,
  [sym_shebang] = sym_shebang,
  [sym_self] = sym_self,
  [sym_super] = sym_super,
  [sym_crate] = sym_crate,
  [aux_sym_metavariable_token1] = aux_sym_metavariable_token1,
  [sym_string_content] = sym_string_content,
  [sym__raw_string_literal_start] = sym__raw_string_literal_start,
  [sym_raw_string_literal_content] = sym_string_content,
  [sym__raw_string_literal_end] = sym__raw_string_literal_end,
  [sym_float_literal] = sym_float_literal,
  [sym__outer_block_doc_comment_start] = sym__outer_block_doc_comment_start,
  [sym__outer_line_doc_comment_start] = sym__outer_line_doc_comment_start,
  [sym__inner_block_doc_comment_marker] = sym__inner_block_doc_comment_marker,
  [sym__block_comment_content] = sym__block_comment_content,
  [sym__line_doc_content] = sym__line_doc_content,
  [sym__frontmatter_start] = sym__frontmatter_start,
  [sym_frontmatter_content] = sym_frontmatter_content,
  [sym__frontmatter_end] = sym__frontmatter_end,
  [sym__error_sentinel] = sym__error_sentinel,
  [sym_source_file] = sym_source_file,
  [sym__statement] = sym__statement,
  [sym_empty_statement] = sym_empty_statement,
  [sym_expression_statement] = sym_expression_statement,
  [sym__declaration_statement] = sym__declaration_statement,
  [sym_macro_definition_v2] = sym_macro_definition_v2,
  [sym_macro_arguments_declaration] = sym_macro_arguments_declaration,
  [sym_macro_body_v2] = sym_macro_body_v2,
  [sym_macro_rules_v2] = sym_macro_rules_v2,
  [sym_macro_rule_v2] = sym_macro_rule_v2,
  [sym_macro_definition] = sym_macro_definition,
  [sym_macro_rule] = sym_macro_rule,
  [sym__token_pattern] = sym__token_pattern,
  [sym_token_tree_pattern] = sym_token_tree_pattern,
  [sym_token_binding_pattern] = sym_token_binding_pattern,
  [sym_token_repetition_pattern] = sym_token_repetition_pattern,
  [sym_fragment_specifier] = sym_fragment_specifier,
  [sym_token_tree] = sym_token_tree,
  [sym_token_repetition] = sym_token_repetition,
  [sym_primitive_type] = sym_primitive_type,
  [sym_attribute_item] = sym_attribute_item,
  [sym_attributes] = sym_attributes,
  [sym_inner_attribute_item] = sym_inner_attribute_item,
  [sym_attribute] = sym_attribute,
  [sym_mod_item] = sym_mod_item,
  [sym_foreign_mod_item] = sym_foreign_mod_item,
  [sym_declaration_list] = sym_declaration_list,
  [sym_struct_item] = sym_struct_item,
  [sym_union_item] = sym_union_item,
  [sym_enum_item] = sym_enum_item,
  [sym_enum_variant_list] = sym_enum_variant_list,
  [sym_enum_variant] = sym_enum_variant,
  [sym_field_declaration_list] = sym_field_declaration_list,
  [sym_field_declaration] = sym_field_declaration,
  [sym_ordered_field_declaration_list] = sym_ordered_field_declaration_list,
  [sym_extern_crate_declaration] = sym_extern_crate_declaration,
  [sym_const_item] = sym_const_item,
  [sym_static_item] = sym_static_item,
  [sym_type_item] = sym_type_item,
  [sym_function_item] = sym_function_item,
  [sym_function_signature_item] = sym_function_signature_item,
  [sym_function_modifiers] = sym_function_modifiers,
  [aux_sym__item_modifiers] = aux_sym__item_modifiers,
  [sym_where_clause] = sym_where_clause,
  [sym_where_predicate] = sym_where_predicate,
  [sym_impl_item] = sym_impl_item,
  [sym_trait_item] = sym_trait_item,
  [sym_trait_bounds] = sym_trait_bounds,
  [sym_higher_ranked_trait_bound] = sym_higher_ranked_trait_bound,
  [sym_removed_trait_bound] = sym_removed_trait_bound,
  [sym_type_parameters] = sym_type_parameters,
  [sym_const_parameter] = sym_const_parameter,
  [sym_type_parameter] = sym_type_parameter,
  [sym_lifetime_parameter] = sym_lifetime_parameter,
  [sym_let_declaration] = sym_let_declaration,
  [sym_use_declaration] = sym_use_declaration,
  [sym__use_clause] = sym__use_clause,
  [sym_scoped_use_list] = sym_scoped_use_list,
  [sym_use_list] = sym_use_list,
  [sym_use_as_clause] = sym_use_as_clause,
  [sym_use_wildcard] = sym_use_wildcard,
  [sym_parameters] = sym_parameters,
  [sym_self_parameter] = sym_self_parameter,
  [sym_variadic_parameter] = sym_variadic_parameter,
  [sym_parameter] = sym_parameter,
  [sym_extern_modifier] = sym_extern_modifier,
  [sym_visibility_modifier] = sym_visibility_modifier,
  [sym__type] = sym__type,
  [sym_bracketed_type] = sym_bracketed_type,
  [sym_qualified_type] = sym_qualified_type,
  [sym_lifetime] = sym_lifetime,
  [sym_array_type] = sym_array_type,
  [sym_function_type] = sym_function_type,
  [sym_unsafe_bound_type] = sym_unsafe_bound_type,
  [sym_tuple_type] = sym_tuple_type,
  [sym_unit_type] = sym_unit_type,
  [sym_generic_function] = sym_generic_function,
  [sym_generic_type] = sym_generic_type,
  [sym_generic_type_with_turbofish] = sym_generic_type_with_turbofish,
  [sym_bounded_type] = sym_bounded_type,
  [sym_use_bounds] = sym_use_bounds,
  [sym_type_arguments] = sym_type_arguments,
  [sym_type_binding] = sym_type_binding,
  [sym_reference_type] = sym_reference_type,
  [sym_pointer_type] = sym_pointer_type,
  [sym_never_type] = sym_never_type,
  [sym_abstract_type] = sym_abstract_type,
  [sym_dynamic_type] = sym_dynamic_type,
  [sym__expression_except_range] = sym__expression_except_range,
  [sym__expression] = sym__expression,
  [sym_expression_with_attribute] = sym_expression_with_attribute,
  [sym__expression_without_attribute] = sym__expression_without_attribute,
  [sym_block_expression_with_attribute] = sym_block_expression_with_attribute,
  [sym_macro_invocation_item] = sym_macro_invocation_item,
  [sym_macro_invocation] = sym_macro_invocation,
  [sym_delim_token_tree] = sym_token_tree,
  [sym__delim_tokens] = sym__delim_tokens,
  [sym__non_delim_token] = sym__non_delim_token,
  [sym_scoped_identifier] = sym_scoped_identifier,
  [sym_scoped_type_identifier_in_expression_position] = sym_scoped_type_identifier,
  [sym_scoped_type_identifier] = sym_scoped_type_identifier,
  [sym_range_expression] = sym_range_expression,
  [sym_unary_expression] = sym_unary_expression,
  [sym_try_expression] = sym_try_expression,
  [sym_reference_expression] = sym_reference_expression,
  [sym_binary_expression] = sym_binary_expression,
  [sym_assignment_expression] = sym_assignment_expression,
  [sym_compound_assignment_expr] = sym_compound_assignment_expr,
  [sym_type_cast_expression] = sym_type_cast_expression,
  [sym_return_expression] = sym_return_expression,
  [sym_become_expression] = sym_become_expression,
  [sym_yield_expression] = sym_yield_expression,
  [sym_call_expression] = sym_call_expression,
  [sym_arguments] = sym_arguments,
  [sym_array_expression] = sym_array_expression,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym_tuple_expression] = sym_tuple_expression,
  [sym_unit_expression] = sym_unit_expression,
  [sym_struct_expression] = sym_struct_expression,
  [sym_field_initializer_list] = sym_field_initializer_list,
  [sym_shorthand_field_initializer] = sym_shorthand_field_initializer,
  [sym_field_initializer] = sym_field_initializer,
  [sym_base_field_initializer] = sym_base_field_initializer,
  [sym_if_expression] = sym_if_expression,
  [sym_let_condition] = sym_let_condition,
  [sym__let_chain] = sym__let_chain,
  [sym__condition] = sym__condition,
  [sym_else_clause] = sym_else_clause,
  [sym_match_expression] = sym_match_expression,
  [sym_match_block] = sym_match_block,
  [sym_match_arm] = sym_match_arm,
  [sym_last_match_arm] = sym_match_arm,
  [sym_match_pattern] = sym_match_pattern,
  [sym_while_expression] = sym_while_expression,
  [sym_loop_expression] = sym_loop_expression,
  [sym_for_expression] = sym_for_expression,
  [sym_const_block] = sym_const_block,
  [sym_closure_expression] = sym_closure_expression,
  [sym_closure_parameters] = sym_closure_parameters,
  [sym_label] = sym_label,
  [sym_break_expression] = sym_break_expression,
  [sym_continue_expression] = sym_continue_expression,
  [sym_index_expression] = sym_index_expression,
  [sym_await_expression] = sym_await_expression,
  [sym_field_expression] = sym_field_expression,
  [sym_unsafe_block] = sym_unsafe_block,
  [sym_async_block] = sym_async_block,
  [sym_gen_block] = sym_gen_block,
  [sym_try_block] = sym_try_block,
  [sym_block] = sym_block,
  [sym__pattern] = sym__pattern,
  [sym_generic_pattern] = sym_generic_pattern,
  [sym_tuple_pattern] = sym_tuple_pattern,
  [sym_slice_pattern] = sym_slice_pattern,
  [sym_tuple_struct_pattern] = sym_tuple_struct_pattern,
  [sym_struct_pattern] = sym_struct_pattern,
  [sym_field_pattern] = sym_field_pattern,
  [sym_remaining_field_pattern] = sym_remaining_field_pattern,
  [sym_mut_pattern] = sym_mut_pattern,
  [sym_range_pattern] = sym_range_pattern,
  [sym_ref_pattern] = sym_ref_pattern,
  [sym_box_pattern] = sym_box_pattern,
  [sym_captured_pattern] = sym_captured_pattern,
  [sym_reference_pattern] = sym_reference_pattern,
  [sym_or_pattern] = sym_or_pattern,
  [sym__literal] = sym__literal,
  [sym__literal_pattern] = sym__literal_pattern,
  [sym_negative_literal] = sym_negative_literal,
  [sym_string_literal] = sym_string_literal,
  [sym_raw_string_literal] = sym_raw_string_literal,
  [sym_boolean_literal] = sym_boolean_literal,
  [sym_line_comment] = sym_line_comment,
  [sym__inner_line_doc_comment_marker] = sym__inner_block_doc_comment_marker,
  [sym_line_outer_doc_comment] = sym_line_outer_doc_comment,
  [sym_block_outer_doc_comment] = sym_block_outer_doc_comment,
  [sym_block_comment] = sym_block_comment,
  [sym_frontmatter] = sym_frontmatter,
  [sym_metavariable] = sym_metavariable,
  [aux_sym_source_file_repeat1] = aux_sym_source_file_repeat1,
  [aux_sym_macro_arguments_declaration_repeat1] = aux_sym_macro_arguments_declaration_repeat1,
  [aux_sym_macro_body_v2_repeat1] = aux_sym_macro_body_v2_repeat1,
  [aux_sym_macro_rules_v2_repeat1] = aux_sym_macro_rules_v2_repeat1,
  [aux_sym_macro_definition_repeat1] = aux_sym_macro_definition_repeat1,
  [aux_sym__non_special_token_repeat1] = aux_sym__non_special_token_repeat1,
  [aux_sym_attributes_repeat1] = aux_sym_attributes_repeat1,
  [aux_sym_declaration_list_repeat1] = aux_sym_declaration_list_repeat1,
  [aux_sym_enum_variant_list_repeat1] = aux_sym_enum_variant_list_repeat1,
  [aux_sym_field_declaration_list_repeat1] = aux_sym_field_declaration_list_repeat1,
  [aux_sym_ordered_field_declaration_list_repeat1] = aux_sym_ordered_field_declaration_list_repeat1,
  [aux_sym_where_clause_repeat1] = aux_sym_where_clause_repeat1,
  [aux_sym_trait_bounds_repeat1] = aux_sym_trait_bounds_repeat1,
  [aux_sym_type_parameters_repeat1] = aux_sym_type_parameters_repeat1,
  [aux_sym_use_list_repeat1] = aux_sym_use_list_repeat1,
  [aux_sym_parameters_repeat1] = aux_sym_parameters_repeat1,
  [aux_sym_unsafe_bound_type_repeat1] = aux_sym_unsafe_bound_type_repeat1,
  [aux_sym_tuple_type_repeat1] = aux_sym_tuple_type_repeat1,
  [aux_sym_use_bounds_repeat1] = aux_sym_use_bounds_repeat1,
  [aux_sym_type_arguments_repeat1] = aux_sym_type_arguments_repeat1,
  [aux_sym_delim_token_tree_repeat1] = aux_sym_delim_token_tree_repeat1,
  [aux_sym_arguments_repeat1] = aux_sym_arguments_repeat1,
  [aux_sym_array_expression_repeat1] = aux_sym_array_expression_repeat1,
  [aux_sym_tuple_expression_repeat1] = aux_sym_tuple_expression_repeat1,
  [aux_sym_field_initializer_list_repeat1] = aux_sym_field_initializer_list_repeat1,
  [aux_sym_match_block_repeat1] = aux_sym_match_block_repeat1,
  [aux_sym_match_block_repeat2] = aux_sym_match_block_repeat2,
  [aux_sym_closure_parameters_repeat1] = aux_sym_closure_parameters_repeat1,
  [aux_sym_tuple_pattern_repeat1] = aux_sym_tuple_pattern_repeat1,
  [aux_sym_slice_pattern_repeat1] = aux_sym_slice_pattern_repeat1,
  [aux_sym_struct_pattern_repeat1] = aux_sym_struct_pattern_repeat1,
  [aux_sym_string_literal_repeat1] = aux_sym_string_literal_repeat1,
  [alias_sym_field_ellipsis] = alias_sym_field_ellipsis,
  [alias_sym_field_identifier] = alias_sym_field_identifier,
  [alias_sym_let_chain] = alias_sym_let_chain,
  [alias_sym_shorthand_field_identifier] = alias_sym_shorthand_field_identifier,
  [alias_sym_type_identifier] = alias_sym_type_identifier,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_macro] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_macro_rules] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOLLAR] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_token_repetition_pattern_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_block] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_expr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_expr_2021] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ident] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_item] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_lifetime] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_literal] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_meta] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pat] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pat_param] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_path] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_stmt] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_tt] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ty] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_vis] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u8] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_i8] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u16] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_i16] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u32] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_i32] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u64] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_i64] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u128] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_i128] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_isize] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_usize] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_f32] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_f64] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_bool] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_str] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_char] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUND] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_as] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_async] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_auto] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_await] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_become] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_const] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_continue] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_default] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_enum] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_fn] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_gen] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_impl] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_let] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_loop] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_match] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_mod] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pub] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_raw] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_static] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_struct] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_trait] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_type] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_union] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_unsafe] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_use] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_where] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_while] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_extern] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_safe] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ref] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDEconst] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_in] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_dyn] = {
    .visible = true,
    .named = false,
  },
  [sym_mutable_specifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_yield] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_move] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_try] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_box] = {
    .visible = true,
    .named = false,
  },
  [sym_integer_literal] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_string_literal_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [sym_char_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_escape_sequence] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_true] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_false] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_SLASH] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_line_comment_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_BANG2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_STAR] = {
    .visible = true,
    .named = false,
  },
  [sym_shebang] = {
    .visible = true,
    .named = true,
  },
  [sym_self] = {
    .visible = true,
    .named = true,
  },
  [sym_super] = {
    .visible = true,
    .named = true,
  },
  [sym_crate] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_metavariable_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_string_content] = {
    .visible = true,
    .named = true,
  },
  [sym__raw_string_literal_start] = {
    .visible = false,
    .named = true,
  },
  [sym_raw_string_literal_content] = {
    .visible = true,
    .named = true,
  },
  [sym__raw_string_literal_end] = {
    .visible = false,
    .named = true,
  },
  [sym_float_literal] = {
    .visible = true,
    .named = true,
  },
  [sym__outer_block_doc_comment_start] = {
    .visible = false,
    .named = true,
  },
  [sym__outer_line_doc_comment_start] = {
    .visible = false,
    .named = true,
  },
  [sym__inner_block_doc_comment_marker] = {
    .visible = true,
    .named = true,
  },
  [sym__block_comment_content] = {
    .visible = false,
    .named = true,
  },
  [sym__line_doc_content] = {
    .visible = true,
    .named = true,
  },
  [sym__frontmatter_start] = {
    .visible = false,
    .named = true,
  },
  [sym_frontmatter_content] = {
    .visible = true,
    .named = true,
  },
  [sym__frontmatter_end] = {
    .visible = false,
    .named = true,
  },
  [sym__error_sentinel] = {
    .visible = false,
    .named = true,
  },
  [sym_source_file] = {
    .visible = true,
    .named = true,
  },
  [sym__statement] = {
    .visible = false,
    .named = true,
  },
  [sym_empty_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__declaration_statement] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_macro_definition_v2] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_arguments_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_body_v2] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_rules_v2] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_rule_v2] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_rule] = {
    .visible = true,
    .named = true,
  },
  [sym__token_pattern] = {
    .visible = false,
    .named = true,
  },
  [sym_token_tree_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_token_binding_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_token_repetition_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_fragment_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_token_tree] = {
    .visible = true,
    .named = true,
  },
  [sym_token_repetition] = {
    .visible = true,
    .named = true,
  },
  [sym_primitive_type] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute_item] = {
    .visible = true,
    .named = true,
  },
  [sym_attributes] = {
    .visible = true,
    .named = true,
  },
  [sym_inner_attribute_item] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym_mod_item] = {
    .visible = true,
    .named = true,
  },
  [sym_foreign_mod_item] = {
    .visible = true,
    .named = true,
  },
  [sym_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_item] = {
    .visible = true,
    .named = true,
  },
  [sym_union_item] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_item] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_variant_list] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_variant] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_ordered_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_extern_crate_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_const_item] = {
    .visible = true,
    .named = true,
  },
  [sym_static_item] = {
    .visible = true,
    .named = true,
  },
  [sym_type_item] = {
    .visible = true,
    .named = true,
  },
  [sym_function_item] = {
    .visible = true,
    .named = true,
  },
  [sym_function_signature_item] = {
    .visible = true,
    .named = true,
  },
  [sym_function_modifiers] = {
    .visible = true,
    .named = true,
  },
  [aux_sym__item_modifiers] = {
    .visible = false,
    .named = false,
  },
  [sym_where_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_where_predicate] = {
    .visible = true,
    .named = true,
  },
  [sym_impl_item] = {
    .visible = true,
    .named = true,
  },
  [sym_trait_item] = {
    .visible = true,
    .named = true,
  },
  [sym_trait_bounds] = {
    .visible = true,
    .named = true,
  },
  [sym_higher_ranked_trait_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_removed_trait_bound] = {
    .visible = true,
    .named = true,
  },
  [sym_type_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_const_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_type_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_lifetime_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_let_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_use_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym__use_clause] = {
    .visible = false,
    .named = true,
  },
  [sym_scoped_use_list] = {
    .visible = true,
    .named = true,
  },
  [sym_use_list] = {
    .visible = true,
    .named = true,
  },
  [sym_use_as_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_use_wildcard] = {
    .visible = true,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_self_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_variadic_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_extern_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_visibility_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym__type] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_bracketed_type] = {
    .visible = true,
    .named = true,
  },
  [sym_qualified_type] = {
    .visible = true,
    .named = true,
  },
  [sym_lifetime] = {
    .visible = true,
    .named = true,
  },
  [sym_array_type] = {
    .visible = true,
    .named = true,
  },
  [sym_function_type] = {
    .visible = true,
    .named = true,
  },
  [sym_unsafe_bound_type] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_type] = {
    .visible = true,
    .named = true,
  },
  [sym_unit_type] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_function] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_type] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_type_with_turbofish] = {
    .visible = true,
    .named = true,
  },
  [sym_bounded_type] = {
    .visible = true,
    .named = true,
  },
  [sym_use_bounds] = {
    .visible = true,
    .named = true,
  },
  [sym_type_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_type_binding] = {
    .visible = true,
    .named = true,
  },
  [sym_reference_type] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_type] = {
    .visible = true,
    .named = true,
  },
  [sym_never_type] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_type] = {
    .visible = true,
    .named = true,
  },
  [sym_dynamic_type] = {
    .visible = true,
    .named = true,
  },
  [sym__expression_except_range] = {
    .visible = false,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_expression_with_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym__expression_without_attribute] = {
    .visible = false,
    .named = true,
  },
  [sym_block_expression_with_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_invocation_item] = {
    .visible = true,
    .named = true,
  },
  [sym_macro_invocation] = {
    .visible = true,
    .named = true,
  },
  [sym_delim_token_tree] = {
    .visible = true,
    .named = true,
  },
  [sym__delim_tokens] = {
    .visible = false,
    .named = true,
  },
  [sym__non_delim_token] = {
    .visible = false,
    .named = true,
  },
  [sym_scoped_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_scoped_type_identifier_in_expression_position] = {
    .visible = true,
    .named = true,
  },
  [sym_scoped_type_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_range_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_try_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_reference_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_compound_assignment_expr] = {
    .visible = true,
    .named = true,
  },
  [sym_type_cast_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_return_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_become_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_yield_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_array_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unit_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_field_initializer_list] = {
    .visible = true,
    .named = true,
  },
  [sym_shorthand_field_initializer] = {
    .visible = true,
    .named = true,
  },
  [sym_field_initializer] = {
    .visible = true,
    .named = true,
  },
  [sym_base_field_initializer] = {
    .visible = true,
    .named = true,
  },
  [sym_if_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_let_condition] = {
    .visible = true,
    .named = true,
  },
  [sym__let_chain] = {
    .visible = false,
    .named = true,
  },
  [sym__condition] = {
    .visible = false,
    .named = true,
  },
  [sym_else_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_match_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_match_block] = {
    .visible = true,
    .named = true,
  },
  [sym_match_arm] = {
    .visible = true,
    .named = true,
  },
  [sym_last_match_arm] = {
    .visible = true,
    .named = true,
  },
  [sym_match_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_while_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_loop_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_for_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_const_block] = {
    .visible = true,
    .named = true,
  },
  [sym_closure_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_closure_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_label] = {
    .visible = true,
    .named = true,
  },
  [sym_break_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_index_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_await_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_field_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unsafe_block] = {
    .visible = true,
    .named = true,
  },
  [sym_async_block] = {
    .visible = true,
    .named = true,
  },
  [sym_gen_block] = {
    .visible = true,
    .named = true,
  },
  [sym_try_block] = {
    .visible = true,
    .named = true,
  },
  [sym_block] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_generic_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_slice_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_struct_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_field_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_remaining_field_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_mut_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_range_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_ref_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_box_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_captured_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_reference_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_or_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__literal] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__literal_pattern] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_negative_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_string_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_raw_string_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_boolean_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_line_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__inner_line_doc_comment_marker] = {
    .visible = true,
    .named = true,
  },
  [sym_line_outer_doc_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_block_outer_doc_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_block_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_frontmatter] = {
    .visible = true,
    .named = true,
  },
  [sym_metavariable] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_source_file_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_macro_arguments_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_macro_body_v2_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_macro_rules_v2_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_macro_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__non_special_token_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_attributes_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_enum_variant_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_field_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_ordered_field_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_where_clause_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_trait_bounds_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_use_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_unsafe_bound_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_use_bounds_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_arguments_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_delim_token_tree_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_arguments_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_array_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_field_initializer_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_match_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_match_block_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_closure_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_tuple_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_slice_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_struct_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_literal_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_field_ellipsis] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_field_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_let_chain] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_shorthand_field_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_identifier] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_alias = 1,
  field_alias_body = 2,
  field_alternative = 3,
  field_argument = 4,
  field_arguments = 5,
  field_attributes = 6,
  field_body = 7,
  field_bounds = 8,
  field_condition = 9,
  field_consequence = 10,
  field_default_type = 11,
  field_doc = 12,
  field_element = 13,
  field_expression = 14,
  field_field = 15,
  field_function = 16,
  field_inner = 17,
  field_invocation = 18,
  field_left = 19,
  field_length = 20,
  field_list = 21,
  field_macro = 22,
  field_name = 23,
  field_operator = 24,
  field_parameters = 25,
  field_path = 26,
  field_pattern = 27,
  field_return_type = 28,
  field_right = 29,
  field_rules = 30,
  field_trait = 31,
  field_type = 32,
  field_type_arguments = 33,
  field_type_parameters = 34,
  field_value = 35,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_alias] = "alias",
  [field_alias_body] = "alias_body",
  [field_alternative] = "alternative",
  [field_argument] = "argument",
  [field_arguments] = "arguments",
  [field_attributes] = "attributes",
  [field_body] = "body",
  [field_bounds] = "bounds",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_default_type] = "default_type",
  [field_doc] = "doc",
  [field_element] = "element",
  [field_expression] = "expression",
  [field_field] = "field",
  [field_function] = "function",
  [field_inner] = "inner",
  [field_invocation] = "invocation",
  [field_left] = "left",
  [field_length] = "length",
  [field_list] = "list",
  [field_macro] = "macro",
  [field_name] = "name",
  [field_operator] = "operator",
  [field_parameters] = "parameters",
  [field_path] = "path",
  [field_pattern] = "pattern",
  [field_return_type] = "return_type",
  [field_right] = "right",
  [field_rules] = "rules",
  [field_trait] = "trait",
  [field_type] = "type",
  [field_type_arguments] = "type_arguments",
  [field_type_parameters] = "type_parameters",
  [field_value] = "value",
};

static const TSMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [2] = {.index = 0, .length = 1},
  [3] = {.index = 1, .length = 1},
  [4] = {.index = 2, .length = 1},
  [7] = {.index = 3, .length = 1},
  [8] = {.index = 3, .length = 1},
  [9] = {.index = 4, .length = 1},
  [11] = {.index = 5, .length = 2},
  [12] = {.index = 7, .length = 1},
  [13] = {.index = 8, .length = 2},
  [14] = {.index = 10, .length = 2},
  [15] = {.index = 5, .length = 2},
  [16] = {.index = 12, .length = 2},
  [17] = {.index = 14, .length = 2},
  [18] = {.index = 16, .length = 2},
  [19] = {.index = 18, .length = 1},
  [20] = {.index = 1, .length = 1},
  [21] = {.index = 19, .length = 2},
  [22] = {.index = 21, .length = 1},
  [23] = {.index = 22, .length = 2},
  [24] = {.index = 22, .length = 2},
  [25] = {.index = 24, .length = 1},
  [26] = {.index = 25, .length = 1},
  [27] = {.index = 26, .length = 2},
  [28] = {.index = 28, .length = 1},
  [29] = {.index = 28, .length = 1},
  [30] = {.index = 25, .length = 1},
  [31] = {.index = 26, .length = 2},
  [32] = {.index = 26, .length = 2},
  [33] = {.index = 29, .length = 1},
  [34] = {.index = 30, .length = 1},
  [35] = {.index = 31, .length = 1},
  [36] = {.index = 31, .length = 1},
  [37] = {.index = 32, .length = 2},
  [38] = {.index = 32, .length = 2},
  [39] = {.index = 34, .length = 2},
  [40] = {.index = 36, .length = 2},
  [41] = {.index = 38, .length = 2},
  [42] = {.index = 40, .length = 1},
  [43] = {.index = 40, .length = 1},
  [44] = {.index = 41, .length = 2},
  [45] = {.index = 43, .length = 1},
  [46] = {.index = 44, .length = 2},
  [47] = {.index = 36, .length = 2},
  [48] = {.index = 46, .length = 1},
  [49] = {.index = 47, .length = 1},
  [50] = {.index = 48, .length = 2},
  [51] = {.index = 21, .length = 1},
  [52] = {.index = 22, .length = 2},
  [53] = {.index = 22, .length = 2},
  [54] = {.index = 50, .length = 2},
  [55] = {.index = 52, .length = 2},
  [56] = {.index = 54, .length = 2},
  [57] = {.index = 56, .length = 1},
  [58] = {.index = 22, .length = 2},
  [59] = {.index = 22, .length = 2},
  [60] = {.index = 57, .length = 3},
  [61] = {.index = 60, .length = 2},
  [62] = {.index = 62, .length = 2},
  [63] = {.index = 62, .length = 2},
  [64] = {.index = 64, .length = 2},
  [65] = {.index = 52, .length = 2},
  [66] = {.index = 16, .length = 2},
  [67] = {.index = 66, .length = 3},
  [68] = {.index = 69, .length = 1},
  [69] = {.index = 60, .length = 2},
  [70] = {.index = 60, .length = 2},
  [71] = {.index = 70, .length = 1},
  [72] = {.index = 70, .length = 1},
  [73] = {.index = 40, .length = 1},
  [74] = {.index = 60, .length = 2},
  [75] = {.index = 71, .length = 1},
  [76] = {.index = 72, .length = 2},
  [77] = {.index = 70, .length = 1},
  [78] = {.index = 74, .length = 1},
  [79] = {.index = 75, .length = 1},
  [80] = {.index = 76, .length = 2},
  [81] = {.index = 78, .length = 1},
  [82] = {.index = 79, .length = 2},
  [83] = {.index = 81, .length = 1},
  [84] = {.index = 81, .length = 1},
  [85] = {.index = 82, .length = 2},
  [86] = {.index = 84, .length = 2},
  [87] = {.index = 86, .length = 2},
  [88] = {.index = 88, .length = 3},
  [89] = {.index = 91, .length = 2},
  [90] = {.index = 93, .length = 3},
  [91] = {.index = 96, .length = 3},
  [92] = {.index = 99, .length = 2},
  [93] = {.index = 99, .length = 2},
  [94] = {.index = 101, .length = 3},
  [95] = {.index = 104, .length = 2},
  [96] = {.index = 106, .length = 1},
  [97] = {.index = 107, .length = 2},
  [98] = {.index = 109, .length = 3},
  [99] = {.index = 112, .length = 2},
  [100] = {.index = 114, .length = 2},
  [101] = {.index = 116, .length = 2},
  [102] = {.index = 118, .length = 2},
  [103] = {.index = 116, .length = 2},
  [104] = {.index = 118, .length = 2},
  [105] = {.index = 120, .length = 1},
  [107] = {.index = 121, .length = 3},
  [108] = {.index = 124, .length = 3},
  [109] = {.index = 127, .length = 2},
  [110] = {.index = 129, .length = 3},
  [111] = {.index = 132, .length = 2},
  [112] = {.index = 134, .length = 2},
  [113] = {.index = 124, .length = 3},
  [114] = {.index = 134, .length = 2},
  [115] = {.index = 136, .length = 2},
  [116] = {.index = 138, .length = 2},
  [117] = {.index = 140, .length = 2},
  [118] = {.index = 84, .length = 2},
  [119] = {.index = 120, .length = 1},
  [120] = {.index = 142, .length = 1},
  [121] = {.index = 143, .length = 1},
  [122] = {.index = 144, .length = 3},
  [123] = {.index = 120, .length = 1},
  [124] = {.index = 147, .length = 1},
  [125] = {.index = 148, .length = 2},
  [126] = {.index = 3, .length = 1},
  [127] = {.index = 150, .length = 3},
  [128] = {.index = 153, .length = 1},
  [129] = {.index = 154, .length = 2},
  [130] = {.index = 156, .length = 1},
  [131] = {.index = 156, .length = 1},
  [133] = {.index = 157, .length = 3},
  [134] = {.index = 157, .length = 3},
  [135] = {.index = 24, .length = 1},
  [136] = {.index = 160, .length = 3},
  [137] = {.index = 163, .length = 2},
  [138] = {.index = 165, .length = 3},
  [139] = {.index = 168, .length = 3},
  [140] = {.index = 171, .length = 2},
  [141] = {.index = 173, .length = 2},
  [142] = {.index = 173, .length = 2},
  [143] = {.index = 175, .length = 3},
  [144] = {.index = 178, .length = 3},
  [145] = {.index = 181, .length = 4},
  [146] = {.index = 185, .length = 3},
  [147] = {.index = 188, .length = 3},
  [148] = {.index = 191, .length = 2},
  [149] = {.index = 193, .length = 2},
  [150] = {.index = 195, .length = 1},
  [151] = {.index = 196, .length = 1},
  [152] = {.index = 197, .length = 2},
  [153] = {.index = 199, .length = 2},
  [154] = {.index = 201, .length = 3},
  [155] = {.index = 204, .length = 3},
  [156] = {.index = 199, .length = 2},
  [157] = {.index = 201, .length = 3},
  [158] = {.index = 207, .length = 2},
  [160] = {.index = 209, .length = 2},
  [161] = {.index = 211, .length = 2},
  [162] = {.index = 213, .length = 3},
  [163] = {.index = 216, .length = 4},
  [164] = {.index = 171, .length = 2},
  [165] = {.index = 220, .length = 3},
  [166] = {.index = 223, .length = 2},
  [167] = {.index = 225, .length = 2},
  [168] = {.index = 227, .length = 3},
  [169] = {.index = 230, .length = 2},
  [170] = {.index = 232, .length = 2},
  [171] = {.index = 232, .length = 2},
  [172] = {.index = 234, .length = 4},
  [173] = {.index = 238, .length = 3},
  [174] = {.index = 241, .length = 3},
  [175] = {.index = 244, .length = 4},
  [176] = {.index = 248, .length = 3},
  [177] = {.index = 251, .length = 4},
  [178] = {.index = 255, .length = 3},
  [179] = {.index = 258, .length = 4},
  [180] = {.index = 262, .length = 3},
  [181] = {.index = 265, .length = 2},
  [182] = {.index = 267, .length = 3},
  [183] = {.index = 270, .length = 4},
  [184] = {.index = 274, .length = 3},
  [185] = {.index = 277, .length = 2},
  [186] = {.index = 279, .length = 3},
  [187] = {.index = 282, .length = 2},
  [188] = {.index = 284, .length = 3},
  [189] = {.index = 238, .length = 3},
  [190] = {.index = 282, .length = 2},
  [191] = {.index = 287, .length = 2},
  [192] = {.index = 289, .length = 2},
  [193] = {.index = 291, .length = 2},
  [194] = {.index = 293, .length = 3},
  [195] = {.index = 296, .length = 3},
  [196] = {.index = 299, .length = 2},
  [197] = {.index = 301, .length = 2},
  [198] = {.index = 303, .length = 1},
  [199] = {.index = 304, .length = 2},
  [200] = {.index = 306, .length = 2},
  [201] = {.index = 308, .length = 1},
  [202] = {.index = 306, .length = 2},
  [203] = {.index = 120, .length = 1},
  [204] = {.index = 309, .length = 2},
  [205] = {.index = 309, .length = 2},
  [206] = {.index = 311, .length = 3},
  [207] = {.index = 314, .length = 2},
  [208] = {.index = 316, .length = 1},
  [209] = {.index = 317, .length = 1},
  [211] = {.index = 318, .length = 2},
  [212] = {.index = 320, .length = 3},
  [213] = {.index = 323, .length = 2},
  [214] = {.index = 325, .length = 3},
  [215] = {.index = 328, .length = 3},
  [216] = {.index = 331, .length = 4},
  [217] = {.index = 335, .length = 2},
  [218] = {.index = 337, .length = 4},
  [219] = {.index = 341, .length = 3},
  [220] = {.index = 344, .length = 4},
  [221] = {.index = 348, .length = 2},
  [222] = {.index = 350, .length = 3},
  [223] = {.index = 348, .length = 2},
  [224] = {.index = 350, .length = 3},
  [225] = {.index = 353, .length = 3},
  [226] = {.index = 356, .length = 2},
  [227] = {.index = 358, .length = 3},
  [228] = {.index = 361, .length = 3},
  [229] = {.index = 364, .length = 4},
  [230] = {.index = 361, .length = 3},
  [231] = {.index = 364, .length = 4},
  [232] = {.index = 358, .length = 3},
  [233] = {.index = 368, .length = 2},
  [234] = {.index = 370, .length = 2},
  [235] = {.index = 372, .length = 2},
  [236] = {.index = 374, .length = 2},
  [237] = {.index = 376, .length = 1},
  [238] = {.index = 377, .length = 2},
  [239] = {.index = 379, .length = 2},
  [240] = {.index = 381, .length = 3},
  [241] = {.index = 384, .length = 4},
  [242] = {.index = 306, .length = 2},
  [243] = {.index = 388, .length = 2},
  [244] = {.index = 390, .length = 3},
  [245] = {.index = 393, .length = 3},
  [246] = {.index = 396, .length = 4},
  [247] = {.index = 400, .length = 2},
  [248] = {.index = 402, .length = 3},
  [249] = {.index = 405, .length = 3},
  [250] = {.index = 408, .length = 2},
  [251] = {.index = 410, .length = 3},
  [252] = {.index = 413, .length = 3},
  [253] = {.index = 416, .length = 3},
  [254] = {.index = 419, .length = 2},
  [255] = {.index = 421, .length = 2},
  [256] = {.index = 421, .length = 2},
  [257] = {.index = 282, .length = 2},
  [258] = {.index = 423, .length = 3},
  [259] = {.index = 426, .length = 4},
  [260] = {.index = 430, .length = 4},
  [261] = {.index = 434, .length = 3},
  [262] = {.index = 437, .length = 4},
  [263] = {.index = 441, .length = 4},
  [264] = {.index = 445, .length = 5},
  [265] = {.index = 450, .length = 4},
  [266] = {.index = 454, .length = 3},
  [267] = {.index = 457, .length = 4},
  [268] = {.index = 461, .length = 4},
  [269] = {.index = 454, .length = 3},
  [270] = {.index = 457, .length = 4},
  [271] = {.index = 465, .length = 3},
  [272] = {.index = 468, .length = 3},
  [273] = {.index = 471, .length = 3},
  [274] = {.index = 474, .length = 4},
  [275] = {.index = 478, .length = 5},
  [276] = {.index = 434, .length = 3},
  [277] = {.index = 483, .length = 4},
  [278] = {.index = 487, .length = 3},
  [279] = {.index = 490, .length = 3},
  [280] = {.index = 493, .length = 4},
  [281] = {.index = 497, .length = 3},
  [282] = {.index = 500, .length = 3},
  [283] = {.index = 503, .length = 4},
  [284] = {.index = 507, .length = 4},
  [285] = {.index = 511, .length = 3},
  [286] = {.index = 514, .length = 3},
  [287] = {.index = 517, .length = 2},
  [288] = {.index = 519, .length = 3},
  [289] = {.index = 522, .length = 4},
  [290] = {.index = 526, .length = 3},
  [291] = {.index = 377, .length = 2},
  [292] = {.index = 529, .length = 3},
  [293] = {.index = 532, .length = 2},
  [294] = {.index = 534, .length = 2},
  [295] = {.index = 536, .length = 3},
  [296] = {.index = 539, .length = 3},
  [297] = {.index = 542, .length = 2},
  [298] = {.index = 542, .length = 2},
  [299] = {.index = 544, .length = 2},
  [300] = {.index = 546, .length = 1},
  [301] = {.index = 547, .length = 2},
  [302] = {.index = 549, .length = 3},
  [303] = {.index = 552, .length = 3},
  [304] = {.index = 555, .length = 4},
  [305] = {.index = 559, .length = 3},
  [306] = {.index = 562, .length = 3},
  [307] = {.index = 565, .length = 2},
  [308] = {.index = 567, .length = 4},
  [309] = {.index = 571, .length = 5},
  [310] = {.index = 576, .length = 4},
  [311] = {.index = 580, .length = 3},
  [312] = {.index = 580, .length = 3},
  [313] = {.index = 583, .length = 2},
  [314] = {.index = 585, .length = 3},
  [315] = {.index = 583, .length = 2},
  [316] = {.index = 585, .length = 3},
  [317] = {.index = 588, .length = 3},
  [318] = {.index = 591, .length = 4},
  [319] = {.index = 588, .length = 3},
  [320] = {.index = 591, .length = 4},
  [321] = {.index = 595, .length = 4},
  [322] = {.index = 595, .length = 4},
  [323] = {.index = 599, .length = 3},
  [324] = {.index = 602, .length = 3},
  [325] = {.index = 605, .length = 3},
  [326] = {.index = 608, .length = 3},
  [327] = {.index = 611, .length = 2},
  [328] = {.index = 613, .length = 2},
  [329] = {.index = 615, .length = 4},
  [330] = {.index = 619, .length = 3},
  [331] = {.index = 171, .length = 2},
  [332] = {.index = 622, .length = 3},
  [333] = {.index = 625, .length = 3},
  [334] = {.index = 628, .length = 4},
  [335] = {.index = 632, .length = 3},
  [336] = {.index = 635, .length = 3},
  [337] = {.index = 638, .length = 4},
  [338] = {.index = 642, .length = 2},
  [339] = {.index = 644, .length = 3},
  [340] = {.index = 647, .length = 3},
  [341] = {.index = 650, .length = 4},
  [342] = {.index = 654, .length = 3},
  [343] = {.index = 657, .length = 4},
  [344] = {.index = 654, .length = 3},
  [345] = {.index = 657, .length = 4},
  [346] = {.index = 661, .length = 3},
  [347] = {.index = 664, .length = 4},
  [348] = {.index = 668, .length = 4},
  [349] = {.index = 672, .length = 5},
  [350] = {.index = 677, .length = 5},
  [351] = {.index = 682, .length = 4},
  [352] = {.index = 686, .length = 5},
  [353] = {.index = 691, .length = 3},
  [354] = {.index = 694, .length = 4},
  [355] = {.index = 691, .length = 3},
  [356] = {.index = 694, .length = 4},
  [357] = {.index = 698, .length = 4},
  [358] = {.index = 702, .length = 4},
  [359] = {.index = 706, .length = 5},
  [360] = {.index = 702, .length = 4},
  [361] = {.index = 706, .length = 5},
  [362] = {.index = 698, .length = 4},
  [363] = {.index = 711, .length = 3},
  [364] = {.index = 714, .length = 3},
  [365] = {.index = 717, .length = 3},
  [366] = {.index = 720, .length = 3},
  [367] = {.index = 723, .length = 3},
  [368] = {.index = 726, .length = 4},
  [369] = {.index = 730, .length = 4},
  [370] = {.index = 734, .length = 5},
  [371] = {.index = 739, .length = 3},
  [372] = {.index = 742, .length = 4},
  [373] = {.index = 746, .length = 4},
  [374] = {.index = 750, .length = 3},
  [375] = {.index = 753, .length = 4},
  [376] = {.index = 757, .length = 4},
  [377] = {.index = 761, .length = 4},
  [378] = {.index = 765, .length = 3},
  [379] = {.index = 768, .length = 4},
  [380] = {.index = 772, .length = 5},
  [381] = {.index = 777, .length = 4},
  [382] = {.index = 720, .length = 3},
  [383] = {.index = 781, .length = 4},
  [384] = {.index = 785, .length = 3},
  [385] = {.index = 788, .length = 3},
  [386] = {.index = 791, .length = 4},
  [387] = {.index = 795, .length = 4},
  [388] = {.index = 799, .length = 3},
  [389] = {.index = 802, .length = 4},
  [390] = {.index = 806, .length = 2},
  [391] = {.index = 808, .length = 3},
  [392] = {.index = 811, .length = 3},
  [393] = {.index = 814, .length = 2},
  [394] = {.index = 816, .length = 3},
  [395] = {.index = 819, .length = 3},
  [396] = {.index = 822, .length = 2},
  [397] = {.index = 824, .length = 3},
  [398] = {.index = 827, .length = 4},
  [399] = {.index = 831, .length = 3},
  [400] = {.index = 834, .length = 3},
  [401] = {.index = 837, .length = 2},
  [402] = {.index = 837, .length = 2},
  [403] = {.index = 839, .length = 1},
  [404] = {.index = 840, .length = 3},
  [405] = {.index = 843, .length = 3},
  [406] = {.index = 846, .length = 4},
  [407] = {.index = 850, .length = 3},
  [408] = {.index = 853, .length = 2},
  [409] = {.index = 855, .length = 5},
  [410] = {.index = 860, .length = 3},
  [411] = {.index = 860, .length = 3},
  [412] = {.index = 863, .length = 4},
  [413] = {.index = 863, .length = 4},
  [414] = {.index = 867, .length = 3},
  [415] = {.index = 870, .length = 4},
  [416] = {.index = 867, .length = 3},
  [417] = {.index = 870, .length = 4},
  [418] = {.index = 874, .length = 3},
  [419] = {.index = 877, .length = 3},
  [420] = {.index = 880, .length = 3},
  [421] = {.index = 883, .length = 3},
  [422] = {.index = 886, .length = 4},
  [423] = {.index = 890, .length = 4},
  [424] = {.index = 894, .length = 4},
  [425] = {.index = 898, .length = 2},
  [426] = {.index = 900, .length = 3},
  [427] = {.index = 903, .length = 3},
  [428] = {.index = 906, .length = 4},
  [429] = {.index = 910, .length = 2},
  [430] = {.index = 912, .length = 3},
  [431] = {.index = 910, .length = 2},
  [432] = {.index = 912, .length = 3},
  [433] = {.index = 915, .length = 3},
  [434] = {.index = 918, .length = 4},
  [435] = {.index = 915, .length = 3},
  [436] = {.index = 918, .length = 4},
  [437] = {.index = 922, .length = 4},
  [438] = {.index = 922, .length = 4},
  [439] = {.index = 926, .length = 2},
  [440] = {.index = 928, .length = 3},
  [441] = {.index = 931, .length = 4},
  [442] = {.index = 935, .length = 4},
  [443] = {.index = 939, .length = 5},
  [444] = {.index = 944, .length = 4},
  [445] = {.index = 948, .length = 5},
  [446] = {.index = 953, .length = 6},
  [447] = {.index = 959, .length = 5},
  [448] = {.index = 964, .length = 4},
  [449] = {.index = 964, .length = 4},
  [450] = {.index = 968, .length = 3},
  [451] = {.index = 971, .length = 4},
  [452] = {.index = 968, .length = 3},
  [453] = {.index = 971, .length = 4},
  [454] = {.index = 975, .length = 4},
  [455] = {.index = 979, .length = 5},
  [456] = {.index = 975, .length = 4},
  [457] = {.index = 979, .length = 5},
  [458] = {.index = 984, .length = 5},
  [459] = {.index = 984, .length = 5},
  [460] = {.index = 989, .length = 4},
  [461] = {.index = 993, .length = 4},
  [462] = {.index = 997, .length = 4},
  [463] = {.index = 1001, .length = 3},
  [464] = {.index = 1004, .length = 4},
  [465] = {.index = 1008, .length = 4},
  [466] = {.index = 1012, .length = 5},
  [467] = {.index = 1017, .length = 4},
  [468] = {.index = 1021, .length = 4},
  [469] = {.index = 1025, .length = 5},
  [470] = {.index = 1030, .length = 3},
  [471] = {.index = 1033, .length = 4},
  [472] = {.index = 1037, .length = 4},
  [473] = {.index = 1041, .length = 5},
  [474] = {.index = 1046, .length = 4},
  [475] = {.index = 1050, .length = 5},
  [476] = {.index = 1046, .length = 4},
  [477] = {.index = 1050, .length = 5},
  [478] = {.index = 1055, .length = 5},
  [479] = {.index = 1060, .length = 4},
  [480] = {.index = 1064, .length = 5},
  [481] = {.index = 1069, .length = 3},
  [482] = {.index = 1072, .length = 4},
  [483] = {.index = 1076, .length = 4},
  [484] = {.index = 1080, .length = 3},
  [485] = {.index = 1083, .length = 4},
  [486] = {.index = 1087, .length = 4},
  [487] = {.index = 1091, .length = 3},
  [488] = {.index = 1094, .length = 4},
  [489] = {.index = 1098, .length = 5},
  [490] = {.index = 1103, .length = 4},
  [491] = {.index = 1107, .length = 4},
  [492] = {.index = 1111, .length = 5},
  [493] = {.index = 1116, .length = 4},
  [494] = {.index = 1120, .length = 3},
  [495] = {.index = 1123, .length = 3},
  [496] = {.index = 1126, .length = 4},
  [497] = {.index = 1130, .length = 2},
  [498] = {.index = 1132, .length = 3},
  [499] = {.index = 1135, .length = 3},
  [500] = {.index = 1138, .length = 4},
  [501] = {.index = 1142, .length = 4},
  [502] = {.index = 1146, .length = 3},
  [503] = {.index = 1149, .length = 4},
  [504] = {.index = 1153, .length = 4},
  [505] = {.index = 1157, .length = 3},
  [506] = {.index = 1160, .length = 4},
  [507] = {.index = 1160, .length = 4},
  [508] = {.index = 1164, .length = 4},
  [509] = {.index = 1168, .length = 3},
  [510] = {.index = 1171, .length = 3},
  [511] = {.index = 1174, .length = 3},
  [512] = {.index = 1177, .length = 4},
  [513] = {.index = 1181, .length = 3},
  [514] = {.index = 1181, .length = 3},
  [515] = {.index = 1184, .length = 4},
  [516] = {.index = 1184, .length = 4},
  [517] = {.index = 1188, .length = 3},
  [518] = {.index = 1191, .length = 4},
  [519] = {.index = 1188, .length = 3},
  [520] = {.index = 1191, .length = 4},
  [521] = {.index = 1195, .length = 4},
  [522] = {.index = 1199, .length = 4},
  [523] = {.index = 1203, .length = 5},
  [524] = {.index = 1208, .length = 6},
  [525] = {.index = 1214, .length = 4},
  [526] = {.index = 1214, .length = 4},
  [527] = {.index = 1218, .length = 5},
  [528] = {.index = 1218, .length = 5},
  [529] = {.index = 1223, .length = 4},
  [530] = {.index = 1227, .length = 5},
  [531] = {.index = 1223, .length = 4},
  [532] = {.index = 1227, .length = 5},
  [533] = {.index = 1232, .length = 4},
  [534] = {.index = 1236, .length = 4},
  [535] = {.index = 1240, .length = 4},
  [536] = {.index = 1244, .length = 4},
  [537] = {.index = 1248, .length = 5},
  [538] = {.index = 1253, .length = 5},
  [539] = {.index = 1258, .length = 3},
  [540] = {.index = 1261, .length = 4},
  [541] = {.index = 1265, .length = 4},
  [542] = {.index = 1269, .length = 5},
  [543] = {.index = 1274, .length = 3},
  [544] = {.index = 1277, .length = 4},
  [545] = {.index = 1274, .length = 3},
  [546] = {.index = 1277, .length = 4},
  [547] = {.index = 1281, .length = 4},
  [548] = {.index = 1285, .length = 5},
  [549] = {.index = 1281, .length = 4},
  [550] = {.index = 1285, .length = 5},
  [551] = {.index = 1290, .length = 5},
  [552] = {.index = 1290, .length = 5},
  [553] = {.index = 1295, .length = 3},
  [554] = {.index = 1298, .length = 5},
  [555] = {.index = 1303, .length = 6},
  [556] = {.index = 1309, .length = 5},
  [557] = {.index = 1314, .length = 4},
  [558] = {.index = 1318, .length = 4},
  [559] = {.index = 1322, .length = 5},
  [560] = {.index = 1327, .length = 3},
  [561] = {.index = 1330, .length = 4},
  [562] = {.index = 1334, .length = 4},
  [563] = {.index = 1338, .length = 5},
  [564] = {.index = 1343, .length = 5},
  [565] = {.index = 1348, .length = 4},
  [566] = {.index = 1352, .length = 5},
  [567] = {.index = 1357, .length = 5},
  [568] = {.index = 1362, .length = 4},
  [569] = {.index = 1366, .length = 2},
  [570] = {.index = 1368, .length = 3},
  [571] = {.index = 1371, .length = 3},
  [572] = {.index = 1374, .length = 4},
  [573] = {.index = 1378, .length = 2},
  [574] = {.index = 1380, .length = 4},
  [575] = {.index = 1384, .length = 5},
  [576] = {.index = 1389, .length = 4},
  [577] = {.index = 1393, .length = 4},
  [578] = {.index = 1397, .length = 4},
  [579] = {.index = 1401, .length = 4},
  [580] = {.index = 1401, .length = 4},
  [581] = {.index = 1405, .length = 3},
  [582] = {.index = 1408, .length = 5},
  [583] = {.index = 1413, .length = 5},
  [584] = {.index = 1413, .length = 5},
  [585] = {.index = 1418, .length = 5},
  [586] = {.index = 1423, .length = 4},
  [587] = {.index = 1427, .length = 4},
  [588] = {.index = 1431, .length = 4},
  [589] = {.index = 1435, .length = 5},
  [590] = {.index = 1440, .length = 4},
  [591] = {.index = 1440, .length = 4},
  [592] = {.index = 1444, .length = 5},
  [593] = {.index = 1444, .length = 5},
  [594] = {.index = 1449, .length = 4},
  [595] = {.index = 1453, .length = 5},
  [596] = {.index = 1449, .length = 4},
  [597] = {.index = 1453, .length = 5},
  [598] = {.index = 1458, .length = 6},
  [599] = {.index = 1464, .length = 5},
  [600] = {.index = 1469, .length = 3},
  [601] = {.index = 1472, .length = 4},
  [602] = {.index = 1476, .length = 4},
  [603] = {.index = 1480, .length = 5},
  [604] = {.index = 1485, .length = 3},
  [605] = {.index = 1488, .length = 5},
  [606] = {.index = 1493, .length = 6},
  [607] = {.index = 1499, .length = 5},
  [608] = {.index = 1504, .length = 3},
  [609] = {.index = 1507, .length = 3},
  [610] = {.index = 1510, .length = 4},
  [611] = {.index = 1514, .length = 5},
  [612] = {.index = 1519, .length = 5},
  [613] = {.index = 1524, .length = 5},
  [614] = {.index = 1529, .length = 5},
  [615] = {.index = 1529, .length = 5},
  [616] = {.index = 1534, .length = 4},
  [617] = {.index = 1538, .length = 4},
  [618] = {.index = 1542, .length = 4},
  [619] = {.index = 1546, .length = 5},
  [620] = {.index = 1551, .length = 6},
  [621] = {.index = 1557, .length = 4},
  [622] = {.index = 1561, .length = 3},
  [623] = {.index = 1564, .length = 5},
  [624] = {.index = 1569, .length = 4},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_invocation, 0},
  [1] =
    {field_doc, 1},
  [2] =
    {field_value, 1},
  [3] =
    {field_name, 1},
  [4] =
    {field_body, 1},
  [5] =
    {field_body, 1},
    {field_name, 0},
  [7] =
    {field_attributes, 0},
  [8] =
    {field_attributes, 0},
    {field_expression, 1},
  [10] =
    {field_attributes, 0},
    {field_invocation, 1},
  [12] =
    {field_arguments, 1},
    {field_function, 0},
  [14] =
    {field_body, 1},
    {field_parameters, 0},
  [16] =
    {field_doc, 2},
    {field_inner, 1},
  [18] =
    {field_inner, 1},
  [19] =
    {field_name, 1},
    {field_rules, 2},
  [21] =
    {field_macro, 0},
  [22] =
    {field_name, 2},
    {field_path, 0},
  [24] =
    {field_value, 2},
  [25] =
    {field_left, 0},
  [26] =
    {field_type, 0},
    {field_type_arguments, 1},
  [28] =
    {field_right, 1},
  [29] =
    {field_type, 1},
  [30] =
    {field_parameters, 1},
  [31] =
    {field_trait, 1},
  [32] =
    {field_parameters, 1},
    {field_trait, 0},
  [34] =
    {field_body, 2},
    {field_parameters, 1},
  [36] =
    {field_body, 2},
    {field_name, 1},
  [38] =
    {field_condition, 1},
    {field_consequence, 2},
  [40] =
    {field_name, 0},
  [41] =
    {field_body, 2},
    {field_type, 1},
  [43] =
    {field_pattern, 1},
  [44] =
    {field_body, 2},
    {field_value, 1},
  [46] =
    {field_list, 1},
  [47] =
    {field_argument, 1},
  [48] =
    {field_body, 2},
    {field_condition, 1},
  [50] =
    {field_function, 0},
    {field_type_arguments, 2},
  [52] =
    {field_type, 0},
    {field_type_arguments, 2},
  [54] =
    {field_attributes, 0},
    {field_body, 2},
  [56] =
    {field_body, 2},
  [57] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [60] =
    {field_left, 0},
    {field_right, 2},
  [62] =
    {field_field, 2},
    {field_value, 0},
  [64] =
    {field_type, 2},
    {field_value, 0},
  [66] =
    {field_arguments, 2},
    {field_body, 3},
    {field_name, 1},
  [69] =
    {field_value, 3},
  [70] =
    {field_type, 0},
  [71] =
    {field_type_arguments, 2},
  [72] =
    {field_pattern, 0},
    {field_type, 2},
  [74] =
    {field_element, 1},
  [75] =
    {field_type, 2},
  [76] =
    {field_type, 2},
    {field_type_parameters, 1},
  [78] =
    {field_parameters, 2},
  [79] =
    {field_alias, 2},
    {field_type, 0},
  [81] =
    {field_arguments, 1},
  [82] =
    {field_body, 3},
    {field_parameters, 2},
  [84] =
    {field_body, 3},
    {field_name, 2},
  [86] =
    {field_body, 3},
    {field_name, 1},
  [88] =
    {field_body, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [91] =
    {field_name, 1},
    {field_parameters, 2},
  [93] =
    {field_body, 3},
    {field_name, 1},
    {field_parameters, 2},
  [96] =
    {field_alternative, 3},
    {field_condition, 1},
    {field_consequence, 2},
  [99] =
    {field_bounds, 1},
    {field_name, 0},
  [101] =
    {field_body, 3},
    {field_type, 2},
    {field_type_parameters, 1},
  [104] =
    {field_body, 3},
    {field_type, 1},
  [106] =
    {field_pattern, 2},
  [107] =
    {field_name, 1},
    {field_type_parameters, 2},
  [109] =
    {field_body, 3},
    {field_bounds, 2},
    {field_name, 1},
  [112] =
    {field_bounds, 2},
    {field_name, 1},
  [114] =
    {field_body, 3},
    {field_type, 2},
  [116] =
    {field_list, 2},
    {field_path, 0},
  [118] =
    {field_alias, 2},
    {field_path, 0},
  [120] =
    {field_name, 2},
  [121] =
    {field_attributes, 0},
    {field_name, 2},
    {field_rules, 3},
  [124] =
    {field_attributes, 0},
    {field_body, 3},
    {field_name, 2},
  [127] =
    {field_attributes, 0},
    {field_type, 2},
  [129] =
    {field_attributes, 0},
    {field_body, 3},
    {field_type, 2},
  [132] =
    {field_attributes, 0},
    {field_pattern, 2},
  [134] =
    {field_attributes, 0},
    {field_name, 2},
  [136] =
    {field_argument, 2},
    {field_attributes, 0},
  [138] =
    {field_attributes, 0},
    {field_body, 3},
  [140] =
    {field_name, 2},
    {field_rules, 3},
  [142] =
    {field_argument, 2},
  [143] =
    {field_body, 3},
  [144] =
    {field_body, 3},
    {field_parameters, 0},
    {field_return_type, 2},
  [147] =
    {field_length, 3},
  [148] =
    {field_pattern, 1},
    {field_type, 3},
  [150] =
    {field_attributes, 0},
    {field_pattern, 1},
    {field_type, 3},
  [153] =
    {field_type, 3},
  [154] =
    {field_parameters, 1},
    {field_return_type, 3},
  [156] =
    {field_trait, 3},
  [157] =
    {field_parameters, 1},
    {field_return_type, 3},
    {field_trait, 0},
  [160] =
    {field_body, 4},
    {field_parameters, 1},
    {field_return_type, 3},
  [163] =
    {field_body, 4},
    {field_name, 2},
  [165] =
    {field_body, 4},
    {field_bounds, 3},
    {field_name, 2},
  [168] =
    {field_body, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [171] =
    {field_name, 1},
    {field_type, 3},
  [173] =
    {field_bounds, 1},
    {field_left, 0},
  [175] =
    {field_body, 4},
    {field_name, 1},
    {field_type_parameters, 2},
  [178] =
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [181] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [185] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 2},
  [188] =
    {field_body, 4},
    {field_pattern, 1},
    {field_value, 3},
  [191] =
    {field_pattern, 1},
    {field_value, 3},
  [193] =
    {field_default_type, 2},
    {field_name, 0},
  [195] =
    {field_attributes, 1},
  [196] =
    {field_attributes, 2, .inherited = true},
  [197] =
    {field_attributes, 0, .inherited = true},
    {field_attributes, 1, .inherited = true},
  [199] =
    {field_trait, 1},
    {field_type, 3},
  [201] =
    {field_body, 4},
    {field_trait, 1},
    {field_type, 3},
  [204] =
    {field_body, 4},
    {field_type, 2},
    {field_type_parameters, 1},
  [207] =
    {field_alternative, 3},
    {field_pattern, 1},
  [209] =
    {field_body, 4},
    {field_parameters, 3},
  [211] =
    {field_alias_body, 3},
    {field_name, 1},
  [213] =
    {field_body, 4},
    {field_bounds, 2},
    {field_name, 1},
  [216] =
    {field_body, 4},
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [220] =
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [223] =
    {field_body, 4},
    {field_name, 3},
  [225] =
    {field_type, 3},
    {field_type_parameters, 2},
  [227] =
    {field_body, 4},
    {field_type, 3},
    {field_type_parameters, 2},
  [230] =
    {field_body, 4},
    {field_type, 2},
  [232] =
    {field_field, 0},
    {field_value, 2},
  [234] =
    {field_arguments, 3},
    {field_attributes, 0},
    {field_body, 4},
    {field_name, 2},
  [238] =
    {field_attributes, 0},
    {field_body, 4},
    {field_name, 3},
  [241] =
    {field_attributes, 0},
    {field_body, 4},
    {field_name, 2},
  [244] =
    {field_attributes, 0},
    {field_body, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [248] =
    {field_attributes, 0},
    {field_name, 2},
    {field_parameters, 3},
  [251] =
    {field_attributes, 0},
    {field_body, 4},
    {field_name, 2},
    {field_parameters, 3},
  [255] =
    {field_attributes, 0},
    {field_type, 3},
    {field_type_parameters, 2},
  [258] =
    {field_attributes, 0},
    {field_body, 4},
    {field_type, 3},
    {field_type_parameters, 2},
  [262] =
    {field_attributes, 0},
    {field_body, 4},
    {field_type, 2},
  [265] =
    {field_attributes, 0},
    {field_pattern, 3},
  [267] =
    {field_attributes, 0},
    {field_name, 2},
    {field_type_parameters, 3},
  [270] =
    {field_attributes, 0},
    {field_body, 4},
    {field_bounds, 3},
    {field_name, 2},
  [274] =
    {field_attributes, 0},
    {field_bounds, 3},
    {field_name, 2},
  [277] =
    {field_attributes, 0},
    {field_type, 3},
  [279] =
    {field_attributes, 0},
    {field_body, 4},
    {field_type, 3},
  [282] =
    {field_attributes, 0},
    {field_name, 3},
  [284] =
    {field_attributes, 0},
    {field_name, 3},
    {field_rules, 4},
  [287] =
    {field_argument, 3},
    {field_attributes, 0},
  [289] =
    {field_attributes, 0},
    {field_body, 4},
  [291] =
    {field_name, 2},
    {field_parameters, 3},
  [293] =
    {field_body, 4},
    {field_name, 2},
    {field_parameters, 3},
  [296] =
    {field_arguments, 3},
    {field_body, 4},
    {field_name, 2},
  [299] =
    {field_name, 2},
    {field_type_parameters, 3},
  [301] =
    {field_bounds, 3},
    {field_name, 2},
  [303] =
    {field_name, 3},
  [304] =
    {field_body, 4},
    {field_condition, 3},
  [306] =
    {field_name, 0},
    {field_type, 2},
  [308] =
    {field_length, 4},
  [309] =
    {field_name, 0},
    {field_pattern, 2},
  [311] =
    {field_attributes, 0},
    {field_pattern, 2},
    {field_type, 4},
  [314] =
    {field_element, 1},
    {field_length, 3},
  [316] =
    {field_pattern, 0},
  [317] =
    {field_type, 4},
  [318] =
    {field_parameters, 2},
    {field_return_type, 4},
  [320] =
    {field_body, 5},
    {field_parameters, 2},
    {field_return_type, 4},
  [323] =
    {field_alias_body, 4},
    {field_name, 2},
  [325] =
    {field_body, 5},
    {field_bounds, 3},
    {field_name, 2},
  [328] =
    {field_body, 5},
    {field_name, 2},
    {field_type_parameters, 3},
  [331] =
    {field_body, 5},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [335] =
    {field_name, 0},
    {field_value, 2},
  [337] =
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 3},
    {field_type_parameters, 2},
  [341] =
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [344] =
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [348] =
    {field_trait, 2},
    {field_type, 4},
  [350] =
    {field_body, 5},
    {field_trait, 2},
    {field_type, 4},
  [353] =
    {field_bounds, 1},
    {field_default_type, 3},
    {field_name, 0},
  [356] =
    {field_attributes, 1},
    {field_attributes, 3, .inherited = true},
  [358] =
    {field_body, 5},
    {field_trait, 1},
    {field_type, 3},
  [361] =
    {field_trait, 2},
    {field_type, 4},
    {field_type_parameters, 1},
  [364] =
    {field_body, 5},
    {field_trait, 2},
    {field_type, 4},
    {field_type_parameters, 1},
  [368] =
    {field_pattern, 2},
    {field_type, 4},
  [370] =
    {field_pattern, 2},
    {field_value, 4},
  [372] =
    {field_alternative, 4},
    {field_pattern, 2},
  [374] =
    {field_pattern, 0},
    {field_value, 2},
  [376] =
    {field_condition, 2},
  [377] =
    {field_name, 2},
    {field_type, 4},
  [379] =
    {field_attributes, 1},
    {field_type, 2},
  [381] =
    {field_attributes, 2, .inherited = true},
    {field_type, 1},
    {field_type, 2, .inherited = true},
  [384] =
    {field_attributes, 0, .inherited = true},
    {field_attributes, 1, .inherited = true},
    {field_type, 0, .inherited = true},
    {field_type, 1, .inherited = true},
  [388] =
    {field_alias_body, 4},
    {field_name, 1},
  [390] =
    {field_alias_body, 4},
    {field_bounds, 2},
    {field_name, 1},
  [393] =
    {field_alias_body, 4},
    {field_name, 1},
    {field_type_parameters, 2},
  [396] =
    {field_body, 5},
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [400] =
    {field_name, 1},
    {field_type, 4},
  [402] =
    {field_bounds, 2},
    {field_name, 1},
    {field_type, 4},
  [405] =
    {field_name, 1},
    {field_type, 4},
    {field_type_parameters, 2},
  [408] =
    {field_body, 5},
    {field_name, 3},
  [410] =
    {field_body, 5},
    {field_bounds, 4},
    {field_name, 3},
  [413] =
    {field_body, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [416] =
    {field_body, 5},
    {field_type, 3},
    {field_type_parameters, 2},
  [419] =
    {field_alias, 4},
    {field_name, 2},
  [421] =
    {field_field, 1},
    {field_value, 3},
  [423] =
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 3},
  [426] =
    {field_attributes, 0},
    {field_body, 5},
    {field_bounds, 4},
    {field_name, 3},
  [430] =
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [434] =
    {field_attributes, 0},
    {field_name, 2},
    {field_type, 4},
  [437] =
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 2},
    {field_type_parameters, 3},
  [441] =
    {field_attributes, 0},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [445] =
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [450] =
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 3},
  [454] =
    {field_attributes, 0},
    {field_trait, 2},
    {field_type, 4},
  [457] =
    {field_attributes, 0},
    {field_body, 5},
    {field_trait, 2},
    {field_type, 4},
  [461] =
    {field_attributes, 0},
    {field_body, 5},
    {field_type, 3},
    {field_type_parameters, 2},
  [465] =
    {field_attributes, 0},
    {field_pattern, 2},
    {field_value, 4},
  [468] =
    {field_alternative, 4},
    {field_attributes, 0},
    {field_pattern, 2},
  [471] =
    {field_alias_body, 4},
    {field_attributes, 0},
    {field_name, 2},
  [474] =
    {field_attributes, 0},
    {field_body, 5},
    {field_bounds, 3},
    {field_name, 2},
  [478] =
    {field_attributes, 0},
    {field_body, 5},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [483] =
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [487] =
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 4},
  [490] =
    {field_attributes, 0},
    {field_type, 4},
    {field_type_parameters, 3},
  [493] =
    {field_attributes, 0},
    {field_body, 5},
    {field_type, 4},
    {field_type_parameters, 3},
  [497] =
    {field_attributes, 0},
    {field_body, 5},
    {field_type, 3},
  [500] =
    {field_attributes, 0},
    {field_name, 3},
    {field_parameters, 4},
  [503] =
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 3},
    {field_parameters, 4},
  [507] =
    {field_arguments, 4},
    {field_attributes, 0},
    {field_body, 5},
    {field_name, 3},
  [511] =
    {field_attributes, 0},
    {field_name, 3},
    {field_type_parameters, 4},
  [514] =
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 3},
  [517] =
    {field_attributes, 0},
    {field_name, 4},
  [519] =
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [522] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [526] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 3},
  [529] =
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [532] =
    {field_body, 5},
    {field_name, 4},
  [534] =
    {field_name, 3},
    {field_parameters, 4},
  [536] =
    {field_body, 5},
    {field_name, 3},
    {field_parameters, 4},
  [539] =
    {field_name, 0},
    {field_type, 3},
    {field_type_arguments, 1},
  [542] =
    {field_name, 1},
    {field_pattern, 3},
  [544] =
    {field_attributes, 0},
    {field_pattern, 1},
  [546] =
    {field_type, 5},
  [547] =
    {field_alias_body, 5},
    {field_name, 2},
  [549] =
    {field_alias_body, 5},
    {field_bounds, 3},
    {field_name, 2},
  [552] =
    {field_alias_body, 5},
    {field_name, 2},
    {field_type_parameters, 3},
  [555] =
    {field_body, 6},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [559] =
    {field_name, 1},
    {field_type, 3},
    {field_value, 5},
  [562] =
    {field_body, 1},
    {field_name, 0},
    {field_value, 3},
  [565] =
    {field_name, 1},
    {field_value, 3},
  [567] =
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [571] =
    {field_body, 6},
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [576] =
    {field_body, 6},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [580] =
    {field_body, 6},
    {field_trait, 2},
    {field_type, 4},
  [583] =
    {field_trait, 3},
    {field_type, 5},
  [585] =
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
  [588] =
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 1},
  [591] =
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 1},
  [595] =
    {field_body, 6},
    {field_trait, 2},
    {field_type, 4},
    {field_type_parameters, 1},
  [599] =
    {field_pattern, 1},
    {field_type, 3},
    {field_value, 5},
  [602] =
    {field_alternative, 5},
    {field_pattern, 1},
    {field_type, 3},
  [605] =
    {field_alternative, 5},
    {field_pattern, 1},
    {field_value, 3},
  [608] =
    {field_body, 6},
    {field_parameters, 3},
    {field_return_type, 5},
  [611] =
    {field_name, 3},
    {field_type, 5},
  [613] =
    {field_attributes, 1},
    {field_type, 3},
  [615] =
    {field_attributes, 1},
    {field_attributes, 3, .inherited = true},
    {field_type, 2},
    {field_type, 3, .inherited = true},
  [619] =
    {field_attributes, 3, .inherited = true},
    {field_type, 2},
    {field_type, 3, .inherited = true},
  [622] =
    {field_alias_body, 5},
    {field_bounds, 2},
    {field_name, 1},
  [625] =
    {field_alias_body, 5},
    {field_name, 1},
    {field_type_parameters, 2},
  [628] =
    {field_alias_body, 5},
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [632] =
    {field_bounds, 2},
    {field_name, 1},
    {field_type, 5},
  [635] =
    {field_name, 1},
    {field_type, 5},
    {field_type_parameters, 2},
  [638] =
    {field_bounds, 3},
    {field_name, 1},
    {field_type, 5},
    {field_type_parameters, 2},
  [642] =
    {field_alias_body, 5},
    {field_name, 3},
  [644] =
    {field_body, 6},
    {field_bounds, 4},
    {field_name, 3},
  [647] =
    {field_body, 6},
    {field_name, 3},
    {field_type_parameters, 4},
  [650] =
    {field_body, 6},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [654] =
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [657] =
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [661] =
    {field_alias_body, 5},
    {field_attributes, 0},
    {field_name, 3},
  [664] =
    {field_attributes, 0},
    {field_body, 6},
    {field_bounds, 4},
    {field_name, 3},
  [668] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 3},
    {field_type_parameters, 4},
  [672] =
    {field_attributes, 0},
    {field_body, 6},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [677] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [682] =
    {field_attributes, 0},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [686] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [691] =
    {field_attributes, 0},
    {field_trait, 3},
    {field_type, 5},
  [694] =
    {field_attributes, 0},
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
  [698] =
    {field_attributes, 0},
    {field_body, 6},
    {field_trait, 2},
    {field_type, 4},
  [702] =
    {field_attributes, 0},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [706] =
    {field_attributes, 0},
    {field_body, 6},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [711] =
    {field_attributes, 0},
    {field_pattern, 3},
    {field_type, 5},
  [714] =
    {field_attributes, 0},
    {field_pattern, 3},
    {field_value, 5},
  [717] =
    {field_alternative, 5},
    {field_attributes, 0},
    {field_pattern, 3},
  [720] =
    {field_attributes, 0},
    {field_name, 3},
    {field_type, 5},
  [723] =
    {field_alias_body, 5},
    {field_attributes, 0},
    {field_name, 2},
  [726] =
    {field_alias_body, 5},
    {field_attributes, 0},
    {field_bounds, 3},
    {field_name, 2},
  [730] =
    {field_alias_body, 5},
    {field_attributes, 0},
    {field_name, 2},
    {field_type_parameters, 3},
  [734] =
    {field_attributes, 0},
    {field_body, 6},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [739] =
    {field_attributes, 0},
    {field_name, 2},
    {field_type, 5},
  [742] =
    {field_attributes, 0},
    {field_bounds, 3},
    {field_name, 2},
    {field_type, 5},
  [746] =
    {field_attributes, 0},
    {field_name, 2},
    {field_type, 5},
    {field_type_parameters, 3},
  [750] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 4},
  [753] =
    {field_attributes, 0},
    {field_body, 6},
    {field_bounds, 5},
    {field_name, 4},
  [757] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [761] =
    {field_attributes, 0},
    {field_body, 6},
    {field_type, 4},
    {field_type_parameters, 3},
  [765] =
    {field_alias, 5},
    {field_attributes, 0},
    {field_name, 3},
  [768] =
    {field_attributes, 0},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [772] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [777] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 4},
  [781] =
    {field_attributes, 0},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [785] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 5},
  [788] =
    {field_attributes, 0},
    {field_name, 4},
    {field_parameters, 5},
  [791] =
    {field_attributes, 0},
    {field_body, 6},
    {field_name, 4},
    {field_parameters, 5},
  [795] =
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 4},
    {field_type_parameters, 3},
  [799] =
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [802] =
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [806] =
    {field_name, 2},
    {field_type, 5},
  [808] =
    {field_bounds, 3},
    {field_name, 2},
    {field_type, 5},
  [811] =
    {field_name, 2},
    {field_type, 5},
    {field_type_parameters, 3},
  [814] =
    {field_body, 6},
    {field_name, 4},
  [816] =
    {field_body, 6},
    {field_bounds, 5},
    {field_name, 4},
  [819] =
    {field_body, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [822] =
    {field_alias, 5},
    {field_name, 3},
  [824] =
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [827] =
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [831] =
    {field_body, 6},
    {field_name, 3},
    {field_parameters, 4},
  [834] =
    {field_body, 6},
    {field_pattern, 3},
    {field_value, 5},
  [837] =
    {field_name, 2},
    {field_pattern, 4},
  [839] =
    {field_type, 6},
  [840] =
    {field_alias_body, 6},
    {field_bounds, 3},
    {field_name, 2},
  [843] =
    {field_alias_body, 6},
    {field_name, 2},
    {field_type_parameters, 3},
  [846] =
    {field_alias_body, 6},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [850] =
    {field_body, 2},
    {field_name, 1},
    {field_value, 4},
  [853] =
    {field_name, 2},
    {field_value, 4},
  [855] =
    {field_body, 7},
    {field_name, 1},
    {field_parameters, 3},
    {field_return_type, 5},
    {field_type_parameters, 2},
  [860] =
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
  [863] =
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 1},
  [867] =
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 1},
  [870] =
    {field_body, 7},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 1},
  [874] =
    {field_pattern, 2},
    {field_type, 4},
    {field_value, 6},
  [877] =
    {field_alternative, 6},
    {field_pattern, 2},
    {field_type, 4},
  [880] =
    {field_alternative, 6},
    {field_pattern, 2},
    {field_value, 4},
  [883] =
    {field_name, 2},
    {field_type, 4},
    {field_value, 6},
  [886] =
    {field_attributes, 1},
    {field_attributes, 4, .inherited = true},
    {field_type, 3},
    {field_type, 4, .inherited = true},
  [890] =
    {field_alias_body, 6},
    {field_bounds, 3},
    {field_name, 1},
    {field_type_parameters, 2},
  [894] =
    {field_bounds, 3},
    {field_name, 1},
    {field_type, 6},
    {field_type_parameters, 2},
  [898] =
    {field_alias_body, 6},
    {field_name, 3},
  [900] =
    {field_alias_body, 6},
    {field_bounds, 4},
    {field_name, 3},
  [903] =
    {field_alias_body, 6},
    {field_name, 3},
    {field_type_parameters, 4},
  [906] =
    {field_body, 7},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [910] =
    {field_trait, 4},
    {field_type, 6},
  [912] =
    {field_body, 7},
    {field_trait, 4},
    {field_type, 6},
  [915] =
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [918] =
    {field_body, 7},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [922] =
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [926] =
    {field_name, 4},
    {field_type, 6},
  [928] =
    {field_alias_body, 6},
    {field_attributes, 0},
    {field_name, 3},
  [931] =
    {field_alias_body, 6},
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 3},
  [935] =
    {field_alias_body, 6},
    {field_attributes, 0},
    {field_name, 3},
    {field_type_parameters, 4},
  [939] =
    {field_attributes, 0},
    {field_body, 7},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [944] =
    {field_attributes, 0},
    {field_name, 2},
    {field_type, 4},
    {field_value, 6},
  [948] =
    {field_attributes, 0},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [953] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [959] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [964] =
    {field_attributes, 0},
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
  [968] =
    {field_attributes, 0},
    {field_trait, 4},
    {field_type, 6},
  [971] =
    {field_attributes, 0},
    {field_body, 7},
    {field_trait, 4},
    {field_type, 6},
  [975] =
    {field_attributes, 0},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [979] =
    {field_attributes, 0},
    {field_body, 7},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [984] =
    {field_attributes, 0},
    {field_body, 7},
    {field_trait, 3},
    {field_type, 5},
    {field_type_parameters, 2},
  [989] =
    {field_attributes, 0},
    {field_pattern, 2},
    {field_type, 4},
    {field_value, 6},
  [993] =
    {field_alternative, 6},
    {field_attributes, 0},
    {field_pattern, 2},
    {field_type, 4},
  [997] =
    {field_alternative, 6},
    {field_attributes, 0},
    {field_pattern, 2},
    {field_value, 4},
  [1001] =
    {field_attributes, 0},
    {field_name, 4},
    {field_type, 6},
  [1004] =
    {field_alias_body, 6},
    {field_attributes, 0},
    {field_bounds, 3},
    {field_name, 2},
  [1008] =
    {field_alias_body, 6},
    {field_attributes, 0},
    {field_name, 2},
    {field_type_parameters, 3},
  [1012] =
    {field_alias_body, 6},
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [1017] =
    {field_attributes, 0},
    {field_bounds, 3},
    {field_name, 2},
    {field_type, 6},
  [1021] =
    {field_attributes, 0},
    {field_name, 2},
    {field_type, 6},
    {field_type_parameters, 3},
  [1025] =
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 2},
    {field_type, 6},
    {field_type_parameters, 3},
  [1030] =
    {field_alias_body, 6},
    {field_attributes, 0},
    {field_name, 4},
  [1033] =
    {field_attributes, 0},
    {field_body, 7},
    {field_bounds, 5},
    {field_name, 4},
  [1037] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 4},
    {field_type_parameters, 5},
  [1041] =
    {field_attributes, 0},
    {field_body, 7},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1046] =
    {field_attributes, 0},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 3},
  [1050] =
    {field_attributes, 0},
    {field_body, 7},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 3},
  [1055] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [1060] =
    {field_attributes, 0},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [1064] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [1069] =
    {field_attributes, 0},
    {field_name, 3},
    {field_type, 6},
  [1072] =
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 3},
    {field_type, 6},
  [1076] =
    {field_attributes, 0},
    {field_name, 3},
    {field_type, 6},
    {field_type_parameters, 4},
  [1080] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 5},
  [1083] =
    {field_attributes, 0},
    {field_body, 7},
    {field_bounds, 6},
    {field_name, 5},
  [1087] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 5},
    {field_type_parameters, 6},
  [1091] =
    {field_alias, 6},
    {field_attributes, 0},
    {field_name, 4},
  [1094] =
    {field_attributes, 0},
    {field_name, 4},
    {field_parameters, 6},
    {field_type_parameters, 5},
  [1098] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 4},
    {field_parameters, 6},
    {field_type_parameters, 5},
  [1103] =
    {field_attributes, 0},
    {field_body, 7},
    {field_name, 4},
    {field_parameters, 5},
  [1107] =
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [1111] =
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [1116] =
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [1120] =
    {field_bounds, 3},
    {field_name, 2},
    {field_type, 6},
  [1123] =
    {field_name, 2},
    {field_type, 6},
    {field_type_parameters, 3},
  [1126] =
    {field_bounds, 4},
    {field_name, 2},
    {field_type, 6},
    {field_type_parameters, 3},
  [1130] =
    {field_alias_body, 6},
    {field_name, 4},
  [1132] =
    {field_body, 7},
    {field_bounds, 5},
    {field_name, 4},
  [1135] =
    {field_body, 7},
    {field_name, 4},
    {field_type_parameters, 5},
  [1138] =
    {field_body, 7},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1142] =
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 5},
    {field_type_parameters, 4},
  [1146] =
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [1149] =
    {field_body, 7},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [1153] =
    {field_alias_body, 7},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [1157] =
    {field_body, 3},
    {field_name, 2},
    {field_value, 5},
  [1160] =
    {field_body, 8},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 1},
  [1164] =
    {field_alternative, 7},
    {field_pattern, 1},
    {field_type, 3},
    {field_value, 5},
  [1168] =
    {field_name, 3},
    {field_type, 5},
    {field_value, 7},
  [1171] =
    {field_alias_body, 7},
    {field_bounds, 4},
    {field_name, 3},
  [1174] =
    {field_alias_body, 7},
    {field_name, 3},
    {field_type_parameters, 4},
  [1177] =
    {field_alias_body, 7},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [1181] =
    {field_body, 8},
    {field_trait, 4},
    {field_type, 6},
  [1184] =
    {field_body, 8},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [1188] =
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 2},
  [1191] =
    {field_body, 8},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 2},
  [1195] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 3},
  [1199] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_name, 3},
    {field_type_parameters, 4},
  [1203] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [1208] =
    {field_attributes, 0},
    {field_body, 8},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [1214] =
    {field_attributes, 0},
    {field_body, 8},
    {field_trait, 4},
    {field_type, 6},
  [1218] =
    {field_attributes, 0},
    {field_body, 8},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 2},
  [1223] =
    {field_attributes, 0},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 2},
  [1227] =
    {field_attributes, 0},
    {field_body, 8},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 2},
  [1232] =
    {field_attributes, 0},
    {field_pattern, 3},
    {field_type, 5},
    {field_value, 7},
  [1236] =
    {field_alternative, 7},
    {field_attributes, 0},
    {field_pattern, 3},
    {field_type, 5},
  [1240] =
    {field_alternative, 7},
    {field_attributes, 0},
    {field_pattern, 3},
    {field_value, 5},
  [1244] =
    {field_attributes, 0},
    {field_name, 3},
    {field_type, 5},
    {field_value, 7},
  [1248] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 2},
    {field_type_parameters, 3},
  [1253] =
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 2},
    {field_type, 7},
    {field_type_parameters, 3},
  [1258] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_name, 4},
  [1261] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_bounds, 5},
    {field_name, 4},
  [1265] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_name, 4},
    {field_type_parameters, 5},
  [1269] =
    {field_attributes, 0},
    {field_body, 8},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1274] =
    {field_attributes, 0},
    {field_trait, 5},
    {field_type, 7},
  [1277] =
    {field_attributes, 0},
    {field_body, 8},
    {field_trait, 5},
    {field_type, 7},
  [1281] =
    {field_attributes, 0},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 3},
  [1285] =
    {field_attributes, 0},
    {field_body, 8},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 3},
  [1290] =
    {field_attributes, 0},
    {field_body, 8},
    {field_trait, 4},
    {field_type, 6},
    {field_type_parameters, 3},
  [1295] =
    {field_attributes, 0},
    {field_name, 5},
    {field_type, 7},
  [1298] =
    {field_attributes, 0},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [1303] =
    {field_attributes, 0},
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [1309] =
    {field_attributes, 0},
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [1314] =
    {field_attributes, 0},
    {field_bounds, 4},
    {field_name, 3},
    {field_type, 7},
  [1318] =
    {field_attributes, 0},
    {field_name, 3},
    {field_type, 7},
    {field_type_parameters, 4},
  [1322] =
    {field_attributes, 0},
    {field_bounds, 5},
    {field_name, 3},
    {field_type, 7},
    {field_type_parameters, 4},
  [1327] =
    {field_alias_body, 7},
    {field_attributes, 0},
    {field_name, 5},
  [1330] =
    {field_attributes, 0},
    {field_body, 8},
    {field_bounds, 6},
    {field_name, 5},
  [1334] =
    {field_attributes, 0},
    {field_body, 8},
    {field_name, 5},
    {field_type_parameters, 6},
  [1338] =
    {field_attributes, 0},
    {field_body, 8},
    {field_bounds, 7},
    {field_name, 5},
    {field_type_parameters, 6},
  [1343] =
    {field_attributes, 0},
    {field_body, 8},
    {field_name, 4},
    {field_parameters, 6},
    {field_type_parameters, 5},
  [1348] =
    {field_attributes, 0},
    {field_name, 4},
    {field_parameters, 5},
    {field_return_type, 7},
  [1352] =
    {field_attributes, 0},
    {field_body, 8},
    {field_name, 4},
    {field_parameters, 5},
    {field_return_type, 7},
  [1357] =
    {field_body, 8},
    {field_name, 2},
    {field_parameters, 4},
    {field_return_type, 6},
    {field_type_parameters, 3},
  [1362] =
    {field_bounds, 4},
    {field_name, 2},
    {field_type, 7},
    {field_type_parameters, 3},
  [1366] =
    {field_alias_body, 7},
    {field_name, 4},
  [1368] =
    {field_alias_body, 7},
    {field_bounds, 5},
    {field_name, 4},
  [1371] =
    {field_alias_body, 7},
    {field_name, 4},
    {field_type_parameters, 5},
  [1374] =
    {field_body, 8},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1378] =
    {field_name, 5},
    {field_type, 7},
  [1380] =
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [1384] =
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [1389] =
    {field_body, 8},
    {field_name, 3},
    {field_parameters, 4},
    {field_return_type, 6},
  [1393] =
    {field_alternative, 8},
    {field_pattern, 2},
    {field_type, 4},
    {field_value, 6},
  [1397] =
    {field_alias_body, 8},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [1401] =
    {field_body, 9},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 2},
  [1405] =
    {field_name, 4},
    {field_type, 6},
    {field_value, 8},
  [1408] =
    {field_alias_body, 8},
    {field_attributes, 0},
    {field_bounds, 5},
    {field_name, 3},
    {field_type_parameters, 4},
  [1413] =
    {field_attributes, 0},
    {field_body, 9},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 2},
  [1418] =
    {field_alternative, 8},
    {field_attributes, 0},
    {field_pattern, 2},
    {field_type, 4},
    {field_value, 6},
  [1423] =
    {field_attributes, 0},
    {field_name, 4},
    {field_type, 6},
    {field_value, 8},
  [1427] =
    {field_alias_body, 8},
    {field_attributes, 0},
    {field_bounds, 5},
    {field_name, 4},
  [1431] =
    {field_alias_body, 8},
    {field_attributes, 0},
    {field_name, 4},
    {field_type_parameters, 5},
  [1435] =
    {field_alias_body, 8},
    {field_attributes, 0},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1440] =
    {field_attributes, 0},
    {field_body, 9},
    {field_trait, 5},
    {field_type, 7},
  [1444] =
    {field_attributes, 0},
    {field_body, 9},
    {field_trait, 5},
    {field_type, 7},
    {field_type_parameters, 3},
  [1449] =
    {field_attributes, 0},
    {field_trait, 6},
    {field_type, 8},
    {field_type_parameters, 3},
  [1453] =
    {field_attributes, 0},
    {field_body, 9},
    {field_trait, 6},
    {field_type, 8},
    {field_type_parameters, 3},
  [1458] =
    {field_attributes, 0},
    {field_body, 9},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [1464] =
    {field_attributes, 0},
    {field_bounds, 5},
    {field_name, 3},
    {field_type, 8},
    {field_type_parameters, 4},
  [1469] =
    {field_alias_body, 8},
    {field_attributes, 0},
    {field_name, 5},
  [1472] =
    {field_alias_body, 8},
    {field_attributes, 0},
    {field_bounds, 6},
    {field_name, 5},
  [1476] =
    {field_alias_body, 8},
    {field_attributes, 0},
    {field_name, 5},
    {field_type_parameters, 6},
  [1480] =
    {field_attributes, 0},
    {field_body, 9},
    {field_bounds, 7},
    {field_name, 5},
    {field_type_parameters, 6},
  [1485] =
    {field_attributes, 0},
    {field_name, 6},
    {field_type, 8},
  [1488] =
    {field_attributes, 0},
    {field_name, 4},
    {field_parameters, 6},
    {field_return_type, 8},
    {field_type_parameters, 5},
  [1493] =
    {field_attributes, 0},
    {field_body, 9},
    {field_name, 4},
    {field_parameters, 6},
    {field_return_type, 8},
    {field_type_parameters, 5},
  [1499] =
    {field_attributes, 0},
    {field_body, 9},
    {field_name, 4},
    {field_parameters, 5},
    {field_return_type, 7},
  [1504] =
    {field_alias_body, 8},
    {field_bounds, 5},
    {field_name, 4},
  [1507] =
    {field_alias_body, 8},
    {field_name, 4},
    {field_type_parameters, 5},
  [1510] =
    {field_alias_body, 8},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1514] =
    {field_body, 9},
    {field_name, 3},
    {field_parameters, 5},
    {field_return_type, 7},
    {field_type_parameters, 4},
  [1519] =
    {field_alternative, 9},
    {field_attributes, 0},
    {field_pattern, 3},
    {field_type, 5},
    {field_value, 7},
  [1524] =
    {field_alias_body, 9},
    {field_attributes, 0},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1529] =
    {field_attributes, 0},
    {field_body, 10},
    {field_trait, 6},
    {field_type, 8},
    {field_type_parameters, 3},
  [1534] =
    {field_attributes, 0},
    {field_name, 5},
    {field_type, 7},
    {field_value, 9},
  [1538] =
    {field_alias_body, 9},
    {field_attributes, 0},
    {field_bounds, 6},
    {field_name, 5},
  [1542] =
    {field_alias_body, 9},
    {field_attributes, 0},
    {field_name, 5},
    {field_type_parameters, 6},
  [1546] =
    {field_alias_body, 9},
    {field_attributes, 0},
    {field_bounds, 7},
    {field_name, 5},
    {field_type_parameters, 6},
  [1551] =
    {field_attributes, 0},
    {field_body, 10},
    {field_name, 4},
    {field_parameters, 6},
    {field_return_type, 8},
    {field_type_parameters, 5},
  [1557] =
    {field_alias_body, 9},
    {field_bounds, 6},
    {field_name, 4},
    {field_type_parameters, 5},
  [1561] =
    {field_name, 5},
    {field_type, 7},
    {field_value, 9},
  [1564] =
    {field_alias_body, 10},
    {field_attributes, 0},
    {field_bounds, 7},
    {field_name, 5},
    {field_type_parameters, 6},
  [1569] =
    {field_attributes, 0},
    {field_name, 6},
    {field_type, 8},
    {field_value, 10},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [1] = {
    [0] = sym_identifier,
  },
  [5] = {
    [0] = sym_primitive_type,
  },
  [6] = {
    [0] = alias_sym_type_identifier,
  },
  [8] = {
    [1] = alias_sym_type_identifier,
  },
  [10] = {
    [0] = alias_sym_let_chain,
  },
  [11] = {
    [0] = alias_sym_type_identifier,
  },
  [20] = {
    [1] = sym__line_doc_content,
  },
  [22] = {
    [0] = sym_identifier,
  },
  [23] = {
    [0] = sym_identifier,
  },
  [24] = {
    [0] = sym_identifier,
    [2] = alias_sym_type_identifier,
  },
  [26] = {
    [0] = sym_identifier,
  },
  [27] = {
    [0] = sym_identifier,
  },
  [28] = {
    [1] = sym_identifier,
  },
  [31] = {
    [0] = alias_sym_type_identifier,
  },
  [35] = {
    [1] = alias_sym_type_identifier,
  },
  [37] = {
    [0] = alias_sym_type_identifier,
  },
  [40] = {
    [1] = alias_sym_type_identifier,
  },
  [42] = {
    [0] = alias_sym_type_identifier,
  },
  [53] = {
    [2] = alias_sym_type_identifier,
  },
  [55] = {
    [0] = alias_sym_type_identifier,
  },
  [58] = {
    [0] = sym_generic_type,
  },
  [59] = {
    [0] = sym_generic_type,
    [2] = alias_sym_type_identifier,
  },
  [63] = {
    [2] = alias_sym_field_identifier,
  },
  [66] = {
    [2] = sym__line_doc_content,
  },
  [69] = {
    [0] = sym_identifier,
    [2] = sym_identifier,
  },
  [70] = {
    [0] = sym_identifier,
  },
  [72] = {
    [0] = alias_sym_type_identifier,
  },
  [73] = {
    [0] = alias_sym_shorthand_field_identifier,
  },
  [74] = {
    [2] = sym_identifier,
  },
  [77] = {
    [0] = sym_generic_type,
  },
  [83] = {
    [0] = sym_identifier,
  },
  [86] = {
    [2] = alias_sym_type_identifier,
  },
  [87] = {
    [1] = alias_sym_type_identifier,
  },
  [88] = {
    [1] = alias_sym_type_identifier,
  },
  [92] = {
    [0] = alias_sym_type_identifier,
  },
  [97] = {
    [1] = alias_sym_type_identifier,
  },
  [98] = {
    [1] = alias_sym_type_identifier,
  },
  [99] = {
    [1] = alias_sym_type_identifier,
  },
  [101] = {
    [0] = sym_identifier,
  },
  [102] = {
    [0] = sym_identifier,
  },
  [106] = {
    [1] = alias_sym_field_ellipsis,
  },
  [108] = {
    [2] = alias_sym_type_identifier,
  },
  [114] = {
    [2] = alias_sym_type_identifier,
  },
  [119] = {
    [2] = alias_sym_type_identifier,
  },
  [123] = {
    [2] = sym_identifier,
  },
  [126] = {
    [1] = alias_sym_shorthand_field_identifier,
  },
  [130] = {
    [3] = alias_sym_type_identifier,
  },
  [132] = {
    [2] = alias_sym_type_identifier,
  },
  [133] = {
    [0] = alias_sym_type_identifier,
  },
  [135] = {
    [0] = sym_identifier,
  },
  [137] = {
    [2] = alias_sym_type_identifier,
  },
  [138] = {
    [2] = alias_sym_type_identifier,
  },
  [139] = {
    [2] = alias_sym_type_identifier,
  },
  [141] = {
    [0] = alias_sym_type_identifier,
  },
  [143] = {
    [1] = alias_sym_type_identifier,
  },
  [149] = {
    [0] = alias_sym_type_identifier,
  },
  [153] = {
    [1] = alias_sym_type_identifier,
  },
  [154] = {
    [1] = alias_sym_type_identifier,
  },
  [159] = {
    [3] = sym_identifier,
  },
  [161] = {
    [1] = alias_sym_type_identifier,
  },
  [162] = {
    [1] = alias_sym_type_identifier,
  },
  [163] = {
    [1] = alias_sym_type_identifier,
  },
  [164] = {
    [1] = alias_sym_type_identifier,
  },
  [165] = {
    [1] = alias_sym_type_identifier,
  },
  [166] = {
    [3] = alias_sym_type_identifier,
  },
  [171] = {
    [0] = alias_sym_field_identifier,
  },
  [173] = {
    [3] = alias_sym_type_identifier,
  },
  [174] = {
    [2] = alias_sym_type_identifier,
  },
  [175] = {
    [2] = alias_sym_type_identifier,
  },
  [182] = {
    [2] = alias_sym_type_identifier,
  },
  [183] = {
    [2] = alias_sym_type_identifier,
  },
  [184] = {
    [2] = alias_sym_type_identifier,
  },
  [190] = {
    [3] = alias_sym_type_identifier,
  },
  [196] = {
    [2] = alias_sym_type_identifier,
  },
  [197] = {
    [2] = alias_sym_type_identifier,
  },
  [202] = {
    [0] = alias_sym_type_identifier,
  },
  [203] = {
    [2] = alias_sym_shorthand_field_identifier,
  },
  [205] = {
    [0] = alias_sym_field_identifier,
  },
  [210] = {
    [1] = alias_sym_type_identifier,
  },
  [213] = {
    [2] = alias_sym_type_identifier,
  },
  [214] = {
    [2] = alias_sym_type_identifier,
  },
  [215] = {
    [2] = alias_sym_type_identifier,
  },
  [216] = {
    [2] = alias_sym_type_identifier,
  },
  [221] = {
    [2] = alias_sym_type_identifier,
  },
  [222] = {
    [2] = alias_sym_type_identifier,
  },
  [225] = {
    [0] = alias_sym_type_identifier,
  },
  [227] = {
    [1] = alias_sym_type_identifier,
  },
  [228] = {
    [2] = alias_sym_type_identifier,
  },
  [229] = {
    [2] = alias_sym_type_identifier,
  },
  [242] = {
    [0] = alias_sym_field_identifier,
  },
  [243] = {
    [1] = alias_sym_type_identifier,
  },
  [244] = {
    [1] = alias_sym_type_identifier,
  },
  [245] = {
    [1] = alias_sym_type_identifier,
  },
  [246] = {
    [1] = alias_sym_type_identifier,
  },
  [247] = {
    [1] = alias_sym_type_identifier,
  },
  [248] = {
    [1] = alias_sym_type_identifier,
  },
  [249] = {
    [1] = alias_sym_type_identifier,
  },
  [250] = {
    [3] = alias_sym_type_identifier,
  },
  [251] = {
    [3] = alias_sym_type_identifier,
  },
  [252] = {
    [3] = alias_sym_type_identifier,
  },
  [256] = {
    [1] = alias_sym_field_identifier,
  },
  [257] = {
    [3] = sym_identifier,
  },
  [258] = {
    [3] = alias_sym_type_identifier,
  },
  [259] = {
    [3] = alias_sym_type_identifier,
  },
  [260] = {
    [3] = alias_sym_type_identifier,
  },
  [262] = {
    [2] = alias_sym_type_identifier,
  },
  [266] = {
    [2] = alias_sym_type_identifier,
  },
  [267] = {
    [2] = alias_sym_type_identifier,
  },
  [273] = {
    [2] = alias_sym_type_identifier,
  },
  [274] = {
    [2] = alias_sym_type_identifier,
  },
  [275] = {
    [2] = alias_sym_type_identifier,
  },
  [276] = {
    [2] = alias_sym_type_identifier,
  },
  [277] = {
    [2] = alias_sym_type_identifier,
  },
  [278] = {
    [4] = alias_sym_type_identifier,
  },
  [285] = {
    [3] = alias_sym_type_identifier,
  },
  [286] = {
    [3] = alias_sym_type_identifier,
  },
  [291] = {
    [2] = alias_sym_type_identifier,
  },
  [292] = {
    [2] = alias_sym_type_identifier,
  },
  [293] = {
    [4] = alias_sym_type_identifier,
  },
  [296] = {
    [0] = alias_sym_type_identifier,
  },
  [298] = {
    [1] = alias_sym_field_identifier,
  },
  [301] = {
    [2] = alias_sym_type_identifier,
  },
  [302] = {
    [2] = alias_sym_type_identifier,
  },
  [303] = {
    [2] = alias_sym_type_identifier,
  },
  [304] = {
    [2] = alias_sym_type_identifier,
  },
  [311] = {
    [2] = alias_sym_type_identifier,
  },
  [313] = {
    [3] = alias_sym_type_identifier,
  },
  [314] = {
    [3] = alias_sym_type_identifier,
  },
  [317] = {
    [3] = alias_sym_type_identifier,
  },
  [318] = {
    [3] = alias_sym_type_identifier,
  },
  [321] = {
    [2] = alias_sym_type_identifier,
  },
  [331] = {
    [1] = alias_sym_field_identifier,
  },
  [332] = {
    [1] = alias_sym_type_identifier,
  },
  [333] = {
    [1] = alias_sym_type_identifier,
  },
  [334] = {
    [1] = alias_sym_type_identifier,
  },
  [335] = {
    [1] = alias_sym_type_identifier,
  },
  [336] = {
    [1] = alias_sym_type_identifier,
  },
  [337] = {
    [1] = alias_sym_type_identifier,
  },
  [338] = {
    [3] = alias_sym_type_identifier,
  },
  [339] = {
    [3] = alias_sym_type_identifier,
  },
  [340] = {
    [3] = alias_sym_type_identifier,
  },
  [341] = {
    [3] = alias_sym_type_identifier,
  },
  [342] = {
    [3] = alias_sym_type_identifier,
  },
  [343] = {
    [3] = alias_sym_type_identifier,
  },
  [346] = {
    [3] = alias_sym_type_identifier,
  },
  [347] = {
    [3] = alias_sym_type_identifier,
  },
  [348] = {
    [3] = alias_sym_type_identifier,
  },
  [349] = {
    [3] = alias_sym_type_identifier,
  },
  [353] = {
    [3] = alias_sym_type_identifier,
  },
  [354] = {
    [3] = alias_sym_type_identifier,
  },
  [357] = {
    [2] = alias_sym_type_identifier,
  },
  [358] = {
    [3] = alias_sym_type_identifier,
  },
  [359] = {
    [3] = alias_sym_type_identifier,
  },
  [367] = {
    [2] = alias_sym_type_identifier,
  },
  [368] = {
    [2] = alias_sym_type_identifier,
  },
  [369] = {
    [2] = alias_sym_type_identifier,
  },
  [370] = {
    [2] = alias_sym_type_identifier,
  },
  [371] = {
    [2] = alias_sym_type_identifier,
  },
  [372] = {
    [2] = alias_sym_type_identifier,
  },
  [373] = {
    [2] = alias_sym_type_identifier,
  },
  [374] = {
    [4] = alias_sym_type_identifier,
  },
  [375] = {
    [4] = alias_sym_type_identifier,
  },
  [376] = {
    [4] = alias_sym_type_identifier,
  },
  [382] = {
    [3] = alias_sym_type_identifier,
  },
  [383] = {
    [3] = alias_sym_type_identifier,
  },
  [384] = {
    [5] = alias_sym_type_identifier,
  },
  [390] = {
    [2] = alias_sym_type_identifier,
  },
  [391] = {
    [2] = alias_sym_type_identifier,
  },
  [392] = {
    [2] = alias_sym_type_identifier,
  },
  [393] = {
    [4] = alias_sym_type_identifier,
  },
  [394] = {
    [4] = alias_sym_type_identifier,
  },
  [395] = {
    [4] = alias_sym_type_identifier,
  },
  [402] = {
    [2] = alias_sym_field_identifier,
  },
  [404] = {
    [2] = alias_sym_type_identifier,
  },
  [405] = {
    [2] = alias_sym_type_identifier,
  },
  [406] = {
    [2] = alias_sym_type_identifier,
  },
  [410] = {
    [3] = alias_sym_type_identifier,
  },
  [412] = {
    [3] = alias_sym_type_identifier,
  },
  [414] = {
    [4] = alias_sym_type_identifier,
  },
  [415] = {
    [4] = alias_sym_type_identifier,
  },
  [423] = {
    [1] = alias_sym_type_identifier,
  },
  [424] = {
    [1] = alias_sym_type_identifier,
  },
  [425] = {
    [3] = alias_sym_type_identifier,
  },
  [426] = {
    [3] = alias_sym_type_identifier,
  },
  [427] = {
    [3] = alias_sym_type_identifier,
  },
  [428] = {
    [3] = alias_sym_type_identifier,
  },
  [429] = {
    [4] = alias_sym_type_identifier,
  },
  [430] = {
    [4] = alias_sym_type_identifier,
  },
  [433] = {
    [4] = alias_sym_type_identifier,
  },
  [434] = {
    [4] = alias_sym_type_identifier,
  },
  [437] = {
    [3] = alias_sym_type_identifier,
  },
  [440] = {
    [3] = alias_sym_type_identifier,
  },
  [441] = {
    [3] = alias_sym_type_identifier,
  },
  [442] = {
    [3] = alias_sym_type_identifier,
  },
  [443] = {
    [3] = alias_sym_type_identifier,
  },
  [448] = {
    [3] = alias_sym_type_identifier,
  },
  [450] = {
    [4] = alias_sym_type_identifier,
  },
  [451] = {
    [4] = alias_sym_type_identifier,
  },
  [454] = {
    [4] = alias_sym_type_identifier,
  },
  [455] = {
    [4] = alias_sym_type_identifier,
  },
  [458] = {
    [3] = alias_sym_type_identifier,
  },
  [464] = {
    [2] = alias_sym_type_identifier,
  },
  [465] = {
    [2] = alias_sym_type_identifier,
  },
  [466] = {
    [2] = alias_sym_type_identifier,
  },
  [467] = {
    [2] = alias_sym_type_identifier,
  },
  [468] = {
    [2] = alias_sym_type_identifier,
  },
  [469] = {
    [2] = alias_sym_type_identifier,
  },
  [470] = {
    [4] = alias_sym_type_identifier,
  },
  [471] = {
    [4] = alias_sym_type_identifier,
  },
  [472] = {
    [4] = alias_sym_type_identifier,
  },
  [473] = {
    [4] = alias_sym_type_identifier,
  },
  [474] = {
    [4] = alias_sym_type_identifier,
  },
  [475] = {
    [4] = alias_sym_type_identifier,
  },
  [481] = {
    [3] = alias_sym_type_identifier,
  },
  [482] = {
    [3] = alias_sym_type_identifier,
  },
  [483] = {
    [3] = alias_sym_type_identifier,
  },
  [484] = {
    [5] = alias_sym_type_identifier,
  },
  [485] = {
    [5] = alias_sym_type_identifier,
  },
  [486] = {
    [5] = alias_sym_type_identifier,
  },
  [494] = {
    [2] = alias_sym_type_identifier,
  },
  [495] = {
    [2] = alias_sym_type_identifier,
  },
  [496] = {
    [2] = alias_sym_type_identifier,
  },
  [497] = {
    [4] = alias_sym_type_identifier,
  },
  [498] = {
    [4] = alias_sym_type_identifier,
  },
  [499] = {
    [4] = alias_sym_type_identifier,
  },
  [500] = {
    [4] = alias_sym_type_identifier,
  },
  [504] = {
    [2] = alias_sym_type_identifier,
  },
  [506] = {
    [4] = alias_sym_type_identifier,
  },
  [510] = {
    [3] = alias_sym_type_identifier,
  },
  [511] = {
    [3] = alias_sym_type_identifier,
  },
  [512] = {
    [3] = alias_sym_type_identifier,
  },
  [513] = {
    [4] = alias_sym_type_identifier,
  },
  [515] = {
    [4] = alias_sym_type_identifier,
  },
  [517] = {
    [5] = alias_sym_type_identifier,
  },
  [518] = {
    [5] = alias_sym_type_identifier,
  },
  [521] = {
    [3] = alias_sym_type_identifier,
  },
  [522] = {
    [3] = alias_sym_type_identifier,
  },
  [523] = {
    [3] = alias_sym_type_identifier,
  },
  [525] = {
    [4] = alias_sym_type_identifier,
  },
  [527] = {
    [4] = alias_sym_type_identifier,
  },
  [529] = {
    [5] = alias_sym_type_identifier,
  },
  [530] = {
    [5] = alias_sym_type_identifier,
  },
  [537] = {
    [2] = alias_sym_type_identifier,
  },
  [538] = {
    [2] = alias_sym_type_identifier,
  },
  [539] = {
    [4] = alias_sym_type_identifier,
  },
  [540] = {
    [4] = alias_sym_type_identifier,
  },
  [541] = {
    [4] = alias_sym_type_identifier,
  },
  [542] = {
    [4] = alias_sym_type_identifier,
  },
  [543] = {
    [5] = alias_sym_type_identifier,
  },
  [544] = {
    [5] = alias_sym_type_identifier,
  },
  [547] = {
    [5] = alias_sym_type_identifier,
  },
  [548] = {
    [5] = alias_sym_type_identifier,
  },
  [551] = {
    [4] = alias_sym_type_identifier,
  },
  [557] = {
    [3] = alias_sym_type_identifier,
  },
  [558] = {
    [3] = alias_sym_type_identifier,
  },
  [559] = {
    [3] = alias_sym_type_identifier,
  },
  [560] = {
    [5] = alias_sym_type_identifier,
  },
  [561] = {
    [5] = alias_sym_type_identifier,
  },
  [562] = {
    [5] = alias_sym_type_identifier,
  },
  [563] = {
    [5] = alias_sym_type_identifier,
  },
  [568] = {
    [2] = alias_sym_type_identifier,
  },
  [569] = {
    [4] = alias_sym_type_identifier,
  },
  [570] = {
    [4] = alias_sym_type_identifier,
  },
  [571] = {
    [4] = alias_sym_type_identifier,
  },
  [572] = {
    [4] = alias_sym_type_identifier,
  },
  [578] = {
    [3] = alias_sym_type_identifier,
  },
  [579] = {
    [5] = alias_sym_type_identifier,
  },
  [582] = {
    [3] = alias_sym_type_identifier,
  },
  [583] = {
    [5] = alias_sym_type_identifier,
  },
  [587] = {
    [4] = alias_sym_type_identifier,
  },
  [588] = {
    [4] = alias_sym_type_identifier,
  },
  [589] = {
    [4] = alias_sym_type_identifier,
  },
  [590] = {
    [5] = alias_sym_type_identifier,
  },
  [592] = {
    [5] = alias_sym_type_identifier,
  },
  [594] = {
    [6] = alias_sym_type_identifier,
  },
  [595] = {
    [6] = alias_sym_type_identifier,
  },
  [599] = {
    [3] = alias_sym_type_identifier,
  },
  [600] = {
    [5] = alias_sym_type_identifier,
  },
  [601] = {
    [5] = alias_sym_type_identifier,
  },
  [602] = {
    [5] = alias_sym_type_identifier,
  },
  [603] = {
    [5] = alias_sym_type_identifier,
  },
  [608] = {
    [4] = alias_sym_type_identifier,
  },
  [609] = {
    [4] = alias_sym_type_identifier,
  },
  [610] = {
    [4] = alias_sym_type_identifier,
  },
  [613] = {
    [4] = alias_sym_type_identifier,
  },
  [614] = {
    [6] = alias_sym_type_identifier,
  },
  [617] = {
    [5] = alias_sym_type_identifier,
  },
  [618] = {
    [5] = alias_sym_type_identifier,
  },
  [619] = {
    [5] = alias_sym_type_identifier,
  },
  [621] = {
    [4] = alias_sym_type_identifier,
  },
  [623] = {
    [5] = alias_sym_type_identifier,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym_generic_type_with_turbofish, 2,
    sym_generic_type_with_turbofish,
    sym_generic_type,
  sym__let_chain, 2,
    sym__let_chain,
    alias_sym_let_chain,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 6,
  [7] = 7,
  [8] = 8,
  [9] = 9,
  [10] = 10,
  [11] = 11,
  [12] = 12,
  [13] = 4,
  [14] = 3,
  [15] = 10,
  [16] = 12,
  [17] = 4,
  [18] = 3,
  [19] = 7,
  [20] = 12,
  [21] = 4,
  [22] = 3,
  [23] = 7,
  [24] = 12,
  [25] = 4,
  [26] = 3,
  [27] = 7,
  [28] = 12,
  [29] = 4,
  [30] = 3,
  [31] = 7,
  [32] = 12,
  [33] = 4,
  [34] = 3,
  [35] = 7,
  [36] = 12,
  [37] = 4,
  [38] = 3,
  [39] = 7,
  [40] = 12,
  [41] = 7,
  [42] = 42,
  [43] = 43,
  [44] = 44,
  [45] = 45,
  [46] = 46,
  [47] = 47,
  [48] = 43,
  [49] = 45,
  [50] = 42,
  [51] = 45,
  [52] = 45,
  [53] = 43,
  [54] = 43,
  [55] = 42,
  [56] = 42,
  [57] = 44,
  [58] = 47,
  [59] = 46,
  [60] = 43,
  [61] = 46,
  [62] = 43,
  [63] = 44,
  [64] = 47,
  [65] = 45,
  [66] = 45,
  [67] = 43,
  [68] = 43,
  [69] = 45,
  [70] = 45,
  [71] = 44,
  [72] = 46,
  [73] = 47,
  [74] = 74,
  [75] = 74,
  [76] = 76,
  [77] = 77,
  [78] = 76,
  [79] = 79,
  [80] = 76,
  [81] = 81,
  [82] = 81,
  [83] = 77,
  [84] = 76,
  [85] = 81,
  [86] = 77,
  [87] = 81,
  [88] = 77,
  [89] = 89,
  [90] = 89,
  [91] = 91,
  [92] = 92,
  [93] = 93,
  [94] = 94,
  [95] = 95,
  [96] = 96,
  [97] = 92,
  [98] = 98,
  [99] = 99,
  [100] = 99,
  [101] = 93,
  [102] = 98,
  [103] = 103,
  [104] = 94,
  [105] = 93,
  [106] = 92,
  [107] = 107,
  [108] = 107,
  [109] = 109,
  [110] = 109,
  [111] = 111,
  [112] = 94,
  [113] = 103,
  [114] = 91,
  [115] = 111,
  [116] = 116,
  [117] = 117,
  [118] = 118,
  [119] = 118,
  [120] = 93,
  [121] = 118,
  [122] = 95,
  [123] = 94,
  [124] = 92,
  [125] = 96,
  [126] = 118,
  [127] = 93,
  [128] = 118,
  [129] = 94,
  [130] = 92,
  [131] = 131,
  [132] = 132,
  [133] = 133,
  [134] = 134,
  [135] = 135,
  [136] = 136,
  [137] = 137,
  [138] = 131,
  [139] = 139,
  [140] = 140,
  [141] = 141,
  [142] = 142,
  [143] = 143,
  [144] = 144,
  [145] = 145,
  [146] = 146,
  [147] = 145,
  [148] = 135,
  [149] = 136,
  [150] = 140,
  [151] = 139,
  [152] = 141,
  [153] = 133,
  [154] = 145,
  [155] = 146,
  [156] = 139,
  [157] = 145,
  [158] = 134,
  [159] = 136,
  [160] = 160,
  [161] = 161,
  [162] = 162,
  [163] = 163,
  [164] = 164,
  [165] = 165,
  [166] = 166,
  [167] = 167,
  [168] = 168,
  [169] = 169,
  [170] = 170,
  [171] = 171,
  [172] = 172,
  [173] = 173,
  [174] = 174,
  [175] = 175,
  [176] = 176,
  [177] = 177,
  [178] = 178,
  [179] = 161,
  [180] = 180,
  [181] = 181,
  [182] = 177,
  [183] = 160,
  [184] = 184,
  [185] = 185,
  [186] = 181,
  [187] = 185,
  [188] = 188,
  [189] = 189,
  [190] = 190,
  [191] = 191,
  [192] = 192,
  [193] = 193,
  [194] = 194,
  [195] = 195,
  [196] = 196,
  [197] = 188,
  [198] = 198,
  [199] = 164,
  [200] = 189,
  [201] = 190,
  [202] = 202,
  [203] = 177,
  [204] = 160,
  [205] = 181,
  [206] = 185,
  [207] = 188,
  [208] = 189,
  [209] = 190,
  [210] = 191,
  [211] = 192,
  [212] = 193,
  [213] = 194,
  [214] = 195,
  [215] = 196,
  [216] = 198,
  [217] = 164,
  [218] = 191,
  [219] = 192,
  [220] = 177,
  [221] = 221,
  [222] = 222,
  [223] = 181,
  [224] = 185,
  [225] = 188,
  [226] = 189,
  [227] = 190,
  [228] = 191,
  [229] = 192,
  [230] = 193,
  [231] = 194,
  [232] = 195,
  [233] = 196,
  [234] = 193,
  [235] = 198,
  [236] = 164,
  [237] = 237,
  [238] = 238,
  [239] = 177,
  [240] = 240,
  [241] = 241,
  [242] = 198,
  [243] = 202,
  [244] = 194,
  [245] = 245,
  [246] = 177,
  [247] = 195,
  [248] = 248,
  [249] = 249,
  [250] = 177,
  [251] = 251,
  [252] = 198,
  [253] = 253,
  [254] = 177,
  [255] = 196,
  [256] = 198,
  [257] = 175,
  [258] = 175,
  [259] = 178,
  [260] = 198,
  [261] = 160,
  [262] = 178,
  [263] = 263,
  [264] = 264,
  [265] = 264,
  [266] = 266,
  [267] = 267,
  [268] = 268,
  [269] = 269,
  [270] = 175,
  [271] = 178,
  [272] = 272,
  [273] = 273,
  [274] = 264,
  [275] = 275,
  [276] = 268,
  [277] = 277,
  [278] = 278,
  [279] = 248,
  [280] = 280,
  [281] = 268,
  [282] = 282,
  [283] = 283,
  [284] = 284,
  [285] = 221,
  [286] = 286,
  [287] = 176,
  [288] = 288,
  [289] = 180,
  [290] = 278,
  [291] = 280,
  [292] = 161,
  [293] = 184,
  [294] = 294,
  [295] = 222,
  [296] = 238,
  [297] = 245,
  [298] = 253,
  [299] = 266,
  [300] = 269,
  [301] = 272,
  [302] = 277,
  [303] = 248,
  [304] = 284,
  [305] = 288,
  [306] = 294,
  [307] = 307,
  [308] = 308,
  [309] = 309,
  [310] = 165,
  [311] = 166,
  [312] = 167,
  [313] = 168,
  [314] = 170,
  [315] = 171,
  [316] = 172,
  [317] = 173,
  [318] = 174,
  [319] = 307,
  [320] = 176,
  [321] = 308,
  [322] = 161,
  [323] = 309,
  [324] = 238,
  [325] = 248,
  [326] = 326,
  [327] = 176,
  [328] = 198,
  [329] = 329,
  [330] = 330,
  [331] = 331,
  [332] = 332,
  [333] = 333,
  [334] = 334,
  [335] = 330,
  [336] = 336,
  [337] = 331,
  [338] = 338,
  [339] = 339,
  [340] = 340,
  [341] = 336,
  [342] = 342,
  [343] = 342,
  [344] = 344,
  [345] = 340,
  [346] = 332,
  [347] = 347,
  [348] = 348,
  [349] = 349,
  [350] = 350,
  [351] = 351,
  [352] = 352,
  [353] = 353,
  [354] = 349,
  [355] = 355,
  [356] = 356,
  [357] = 357,
  [358] = 358,
  [359] = 351,
  [360] = 348,
  [361] = 356,
  [362] = 357,
  [363] = 358,
  [364] = 352,
  [365] = 353,
  [366] = 366,
  [367] = 366,
  [368] = 368,
  [369] = 366,
  [370] = 370,
  [371] = 371,
  [372] = 372,
  [373] = 368,
  [374] = 370,
  [375] = 375,
  [376] = 370,
  [377] = 368,
  [378] = 370,
  [379] = 371,
  [380] = 372,
  [381] = 371,
  [382] = 375,
  [383] = 366,
  [384] = 368,
  [385] = 372,
  [386] = 371,
  [387] = 375,
  [388] = 370,
  [389] = 371,
  [390] = 372,
  [391] = 370,
  [392] = 371,
  [393] = 366,
  [394] = 368,
  [395] = 370,
  [396] = 371,
  [397] = 372,
  [398] = 372,
  [399] = 375,
  [400] = 375,
  [401] = 375,
  [402] = 366,
  [403] = 375,
  [404] = 366,
  [405] = 368,
  [406] = 368,
  [407] = 372,
  [408] = 408,
  [409] = 408,
  [410] = 408,
  [411] = 411,
  [412] = 412,
  [413] = 413,
  [414] = 414,
  [415] = 415,
  [416] = 416,
  [417] = 417,
  [418] = 418,
  [419] = 419,
  [420] = 420,
  [421] = 421,
  [422] = 422,
  [423] = 423,
  [424] = 424,
  [425] = 425,
  [426] = 426,
  [427] = 427,
  [428] = 428,
  [429] = 429,
  [430] = 430,
  [431] = 431,
  [432] = 432,
  [433] = 433,
  [434] = 434,
  [435] = 412,
  [436] = 436,
  [437] = 437,
  [438] = 438,
  [439] = 439,
  [440] = 440,
  [441] = 441,
  [442] = 442,
  [443] = 443,
  [444] = 444,
  [445] = 445,
  [446] = 446,
  [447] = 447,
  [448] = 448,
  [449] = 449,
  [450] = 450,
  [451] = 451,
  [452] = 452,
  [453] = 453,
  [454] = 454,
  [455] = 455,
  [456] = 456,
  [457] = 428,
  [458] = 439,
  [459] = 459,
  [460] = 436,
  [461] = 448,
  [462] = 462,
  [463] = 422,
  [464] = 421,
  [465] = 441,
  [466] = 425,
  [467] = 467,
  [468] = 468,
  [469] = 469,
  [470] = 470,
  [471] = 471,
  [472] = 472,
  [473] = 462,
  [474] = 474,
  [475] = 475,
  [476] = 476,
  [477] = 477,
  [478] = 478,
  [479] = 479,
  [480] = 480,
  [481] = 481,
  [482] = 482,
  [483] = 483,
  [484] = 484,
  [485] = 485,
  [486] = 486,
  [487] = 487,
  [488] = 488,
  [489] = 489,
  [490] = 488,
  [491] = 491,
  [492] = 450,
  [493] = 470,
  [494] = 489,
  [495] = 495,
  [496] = 491,
  [497] = 488,
  [498] = 495,
  [499] = 489,
  [500] = 500,
  [501] = 501,
  [502] = 502,
  [503] = 503,
  [504] = 504,
  [505] = 500,
  [506] = 506,
  [507] = 507,
  [508] = 508,
  [509] = 507,
  [510] = 508,
  [511] = 507,
  [512] = 507,
  [513] = 506,
  [514] = 508,
  [515] = 506,
  [516] = 506,
  [517] = 508,
  [518] = 518,
  [519] = 518,
  [520] = 520,
  [521] = 520,
  [522] = 520,
  [523] = 523,
  [524] = 520,
  [525] = 518,
  [526] = 523,
  [527] = 518,
  [528] = 523,
  [529] = 523,
  [530] = 530,
  [531] = 531,
  [532] = 531,
  [533] = 533,
  [534] = 533,
  [535] = 533,
  [536] = 531,
  [537] = 533,
  [538] = 531,
  [539] = 539,
  [540] = 540,
  [541] = 541,
  [542] = 541,
  [543] = 543,
  [544] = 544,
  [545] = 545,
  [546] = 545,
  [547] = 547,
  [548] = 548,
  [549] = 549,
  [550] = 550,
  [551] = 414,
  [552] = 552,
  [553] = 553,
  [554] = 554,
  [555] = 555,
  [556] = 442,
  [557] = 426,
  [558] = 444,
  [559] = 559,
  [560] = 560,
  [561] = 561,
  [562] = 554,
  [563] = 563,
  [564] = 420,
  [565] = 429,
  [566] = 566,
  [567] = 567,
  [568] = 568,
  [569] = 569,
  [570] = 570,
  [571] = 571,
  [572] = 481,
  [573] = 483,
  [574] = 570,
  [575] = 575,
  [576] = 480,
  [577] = 575,
  [578] = 456,
  [579] = 459,
  [580] = 452,
  [581] = 484,
  [582] = 468,
  [583] = 469,
  [584] = 476,
  [585] = 451,
  [586] = 474,
  [587] = 477,
  [588] = 475,
  [589] = 479,
  [590] = 472,
  [591] = 449,
  [592] = 482,
  [593] = 478,
  [594] = 487,
  [595] = 471,
  [596] = 596,
  [597] = 597,
  [598] = 598,
  [599] = 599,
  [600] = 600,
  [601] = 601,
  [602] = 602,
  [603] = 603,
  [604] = 604,
  [605] = 605,
  [606] = 606,
  [607] = 607,
  [608] = 608,
  [609] = 609,
  [610] = 610,
  [611] = 611,
  [612] = 612,
  [613] = 613,
  [614] = 614,
  [615] = 615,
  [616] = 616,
  [617] = 617,
  [618] = 618,
  [619] = 619,
  [620] = 620,
  [621] = 621,
  [622] = 622,
  [623] = 623,
  [624] = 624,
  [625] = 625,
  [626] = 626,
  [627] = 627,
  [628] = 628,
  [629] = 629,
  [630] = 630,
  [631] = 631,
  [632] = 632,
  [633] = 633,
  [634] = 634,
  [635] = 635,
  [636] = 636,
  [637] = 637,
  [638] = 638,
  [639] = 639,
  [640] = 640,
  [641] = 641,
  [642] = 642,
  [643] = 643,
  [644] = 644,
  [645] = 645,
  [646] = 646,
  [647] = 647,
  [648] = 648,
  [649] = 649,
  [650] = 650,
  [651] = 651,
  [652] = 652,
  [653] = 653,
  [654] = 654,
  [655] = 655,
  [656] = 656,
  [657] = 657,
  [658] = 658,
  [659] = 659,
  [660] = 660,
  [661] = 661,
  [662] = 662,
  [663] = 663,
  [664] = 664,
  [665] = 665,
  [666] = 666,
  [667] = 667,
  [668] = 668,
  [669] = 669,
  [670] = 670,
  [671] = 671,
  [672] = 672,
  [673] = 673,
  [674] = 674,
  [675] = 675,
  [676] = 676,
  [677] = 677,
  [678] = 678,
  [679] = 679,
  [680] = 680,
  [681] = 681,
  [682] = 682,
  [683] = 683,
  [684] = 684,
  [685] = 685,
  [686] = 686,
  [687] = 687,
  [688] = 688,
  [689] = 689,
  [690] = 690,
  [691] = 691,
  [692] = 692,
  [693] = 693,
  [694] = 694,
  [695] = 695,
  [696] = 696,
  [697] = 697,
  [698] = 698,
  [699] = 699,
  [700] = 700,
  [701] = 701,
  [702] = 702,
  [703] = 703,
  [704] = 704,
  [705] = 705,
  [706] = 706,
  [707] = 707,
  [708] = 708,
  [709] = 709,
  [710] = 710,
  [711] = 711,
  [712] = 712,
  [713] = 713,
  [714] = 714,
  [715] = 715,
  [716] = 716,
  [717] = 717,
  [718] = 718,
  [719] = 719,
  [720] = 720,
  [721] = 721,
  [722] = 722,
  [723] = 723,
  [724] = 724,
  [725] = 725,
  [726] = 726,
  [727] = 727,
  [728] = 728,
  [729] = 729,
  [730] = 730,
  [731] = 731,
  [732] = 732,
  [733] = 733,
  [734] = 734,
  [735] = 735,
  [736] = 736,
  [737] = 737,
  [738] = 738,
  [739] = 739,
  [740] = 740,
  [741] = 741,
  [742] = 742,
  [743] = 743,
  [744] = 744,
  [745] = 745,
  [746] = 746,
  [747] = 747,
  [748] = 748,
  [749] = 749,
  [750] = 750,
  [751] = 751,
  [752] = 752,
  [753] = 753,
  [754] = 754,
  [755] = 755,
  [756] = 756,
  [757] = 757,
  [758] = 758,
  [759] = 759,
  [760] = 760,
  [761] = 761,
  [762] = 762,
  [763] = 763,
  [764] = 764,
  [765] = 765,
  [766] = 766,
  [767] = 767,
  [768] = 768,
  [769] = 769,
  [770] = 770,
  [771] = 771,
  [772] = 772,
  [773] = 773,
  [774] = 774,
  [775] = 775,
  [776] = 776,
  [777] = 777,
  [778] = 778,
  [779] = 779,
  [780] = 780,
  [781] = 781,
  [782] = 782,
  [783] = 783,
  [784] = 784,
  [785] = 785,
  [786] = 786,
  [787] = 787,
  [788] = 788,
  [789] = 789,
  [790] = 790,
  [791] = 791,
  [792] = 792,
  [793] = 793,
  [794] = 794,
  [795] = 795,
  [796] = 796,
  [797] = 797,
  [798] = 798,
  [799] = 799,
  [800] = 800,
  [801] = 801,
  [802] = 802,
  [803] = 803,
  [804] = 804,
  [805] = 805,
  [806] = 806,
  [807] = 807,
  [808] = 808,
  [809] = 809,
  [810] = 810,
  [811] = 811,
  [812] = 812,
  [813] = 813,
  [814] = 814,
  [815] = 815,
  [816] = 816,
  [817] = 817,
  [818] = 818,
  [819] = 819,
  [820] = 820,
  [821] = 821,
  [822] = 822,
  [823] = 823,
  [824] = 824,
  [825] = 825,
  [826] = 826,
  [827] = 827,
  [828] = 828,
  [829] = 829,
  [830] = 830,
  [831] = 831,
  [832] = 832,
  [833] = 833,
  [834] = 834,
  [835] = 835,
  [836] = 836,
  [837] = 837,
  [838] = 838,
  [839] = 839,
  [840] = 840,
  [841] = 841,
  [842] = 842,
  [843] = 843,
  [844] = 844,
  [845] = 845,
  [846] = 846,
  [847] = 847,
  [848] = 848,
  [849] = 849,
  [850] = 850,
  [851] = 851,
  [852] = 852,
  [853] = 853,
  [854] = 854,
  [855] = 855,
  [856] = 856,
  [857] = 857,
  [858] = 858,
  [859] = 859,
  [860] = 860,
  [861] = 861,
  [862] = 862,
  [863] = 863,
  [864] = 864,
  [865] = 865,
  [866] = 866,
  [867] = 867,
  [868] = 868,
  [869] = 869,
  [870] = 870,
  [871] = 871,
  [872] = 872,
  [873] = 873,
  [874] = 874,
  [875] = 875,
  [876] = 876,
  [877] = 877,
  [878] = 878,
  [879] = 879,
  [880] = 880,
  [881] = 881,
  [882] = 882,
  [883] = 883,
  [884] = 884,
  [885] = 885,
  [886] = 886,
  [887] = 887,
  [888] = 888,
  [889] = 889,
  [890] = 890,
  [891] = 891,
  [892] = 892,
  [893] = 893,
  [894] = 894,
  [895] = 895,
  [896] = 896,
  [897] = 897,
  [898] = 898,
  [899] = 899,
  [900] = 900,
  [901] = 901,
  [902] = 902,
  [903] = 903,
  [904] = 904,
  [905] = 905,
  [906] = 906,
  [907] = 907,
  [908] = 908,
  [909] = 909,
  [910] = 910,
  [911] = 911,
  [912] = 912,
  [913] = 913,
  [914] = 914,
  [915] = 915,
  [916] = 916,
  [917] = 917,
  [918] = 918,
  [919] = 919,
  [920] = 920,
  [921] = 921,
  [922] = 922,
  [923] = 923,
  [924] = 924,
  [925] = 925,
  [926] = 926,
  [927] = 927,
  [928] = 928,
  [929] = 929,
  [930] = 930,
  [931] = 931,
  [932] = 932,
  [933] = 933,
  [934] = 934,
  [935] = 935,
  [936] = 936,
  [937] = 937,
  [938] = 938,
  [939] = 939,
  [940] = 940,
  [941] = 941,
  [942] = 942,
  [943] = 943,
  [944] = 944,
  [945] = 945,
  [946] = 946,
  [947] = 947,
  [948] = 948,
  [949] = 949,
  [950] = 950,
  [951] = 951,
  [952] = 952,
  [953] = 953,
  [954] = 954,
  [955] = 955,
  [956] = 956,
  [957] = 957,
  [958] = 958,
  [959] = 959,
  [960] = 960,
  [961] = 961,
  [962] = 962,
  [963] = 963,
  [964] = 964,
  [965] = 965,
  [966] = 966,
  [967] = 967,
  [968] = 968,
  [969] = 969,
  [970] = 970,
  [971] = 971,
  [972] = 972,
  [973] = 973,
  [974] = 974,
  [975] = 975,
  [976] = 976,
  [977] = 977,
  [978] = 978,
  [979] = 979,
  [980] = 980,
  [981] = 981,
  [982] = 982,
  [983] = 983,
  [984] = 984,
  [985] = 985,
  [986] = 986,
  [987] = 987,
  [988] = 988,
  [989] = 989,
  [990] = 990,
  [991] = 991,
  [992] = 992,
  [993] = 993,
  [994] = 994,
  [995] = 995,
  [996] = 996,
  [997] = 997,
  [998] = 998,
  [999] = 999,
  [1000] = 1000,
  [1001] = 1001,
  [1002] = 1002,
  [1003] = 1003,
  [1004] = 1004,
  [1005] = 1005,
  [1006] = 1006,
  [1007] = 1007,
  [1008] = 1008,
  [1009] = 1009,
  [1010] = 1010,
  [1011] = 1011,
  [1012] = 1012,
  [1013] = 1013,
  [1014] = 1014,
  [1015] = 1015,
  [1016] = 1016,
  [1017] = 1017,
  [1018] = 1018,
  [1019] = 1019,
  [1020] = 1020,
  [1021] = 1021,
  [1022] = 1022,
  [1023] = 1023,
  [1024] = 1024,
  [1025] = 1025,
  [1026] = 1026,
  [1027] = 1027,
  [1028] = 1028,
  [1029] = 1029,
  [1030] = 1030,
  [1031] = 1031,
  [1032] = 1032,
  [1033] = 1033,
  [1034] = 1034,
  [1035] = 1035,
  [1036] = 1036,
  [1037] = 1037,
  [1038] = 1038,
  [1039] = 1039,
  [1040] = 1040,
  [1041] = 1041,
  [1042] = 1042,
  [1043] = 1043,
  [1044] = 1044,
  [1045] = 1045,
  [1046] = 1046,
  [1047] = 1047,
  [1048] = 1048,
  [1049] = 1049,
  [1050] = 1050,
  [1051] = 1051,
  [1052] = 1052,
  [1053] = 1053,
  [1054] = 1054,
  [1055] = 1055,
  [1056] = 1056,
  [1057] = 1057,
  [1058] = 1058,
  [1059] = 1059,
  [1060] = 1060,
  [1061] = 1061,
  [1062] = 1062,
  [1063] = 1063,
  [1064] = 1064,
  [1065] = 1065,
  [1066] = 1066,
  [1067] = 1067,
  [1068] = 1068,
  [1069] = 1069,
  [1070] = 1070,
  [1071] = 1071,
  [1072] = 1072,
  [1073] = 1073,
  [1074] = 1074,
  [1075] = 1075,
  [1076] = 1076,
  [1077] = 1077,
  [1078] = 1078,
  [1079] = 1079,
  [1080] = 1080,
  [1081] = 1081,
  [1082] = 1082,
  [1083] = 1083,
  [1084] = 1084,
  [1085] = 1085,
  [1086] = 1086,
  [1087] = 1087,
  [1088] = 1088,
  [1089] = 1089,
  [1090] = 1090,
  [1091] = 1091,
  [1092] = 1092,
  [1093] = 1093,
  [1094] = 1094,
  [1095] = 1095,
  [1096] = 1096,
  [1097] = 1097,
  [1098] = 1098,
  [1099] = 1099,
  [1100] = 1100,
  [1101] = 1101,
  [1102] = 1102,
  [1103] = 1103,
  [1104] = 1104,
  [1105] = 1105,
  [1106] = 1106,
  [1107] = 1107,
  [1108] = 1108,
  [1109] = 1109,
  [1110] = 1110,
  [1111] = 1111,
  [1112] = 1112,
  [1113] = 1113,
  [1114] = 1114,
  [1115] = 1115,
  [1116] = 1116,
  [1117] = 1117,
  [1118] = 1118,
  [1119] = 1119,
  [1120] = 1120,
  [1121] = 1121,
  [1122] = 1122,
  [1123] = 1123,
  [1124] = 1124,
  [1125] = 1125,
  [1126] = 1126,
  [1127] = 1127,
  [1128] = 1128,
  [1129] = 1129,
  [1130] = 1130,
  [1131] = 1131,
  [1132] = 1132,
  [1133] = 1133,
  [1134] = 1134,
  [1135] = 1135,
  [1136] = 1136,
  [1137] = 1137,
  [1138] = 1138,
  [1139] = 1139,
  [1140] = 1140,
  [1141] = 1141,
  [1142] = 1142,
  [1143] = 1143,
  [1144] = 1144,
  [1145] = 1145,
  [1146] = 420,
  [1147] = 1147,
  [1148] = 1148,
  [1149] = 1149,
  [1150] = 1150,
  [1151] = 1151,
  [1152] = 1152,
  [1153] = 1153,
  [1154] = 1154,
  [1155] = 1155,
  [1156] = 1156,
  [1157] = 1157,
  [1158] = 1158,
  [1159] = 1159,
  [1160] = 1160,
  [1161] = 1161,
  [1162] = 1162,
  [1163] = 1163,
  [1164] = 1164,
  [1165] = 1165,
  [1166] = 1166,
  [1167] = 429,
  [1168] = 1168,
  [1169] = 1169,
  [1170] = 1170,
  [1171] = 442,
  [1172] = 426,
  [1173] = 444,
  [1174] = 1174,
  [1175] = 1175,
  [1176] = 1176,
  [1177] = 1177,
  [1178] = 1178,
  [1179] = 1179,
  [1180] = 1180,
  [1181] = 1181,
  [1182] = 1182,
  [1183] = 1183,
  [1184] = 1184,
  [1185] = 1185,
  [1186] = 1186,
  [1187] = 1187,
  [1188] = 1188,
  [1189] = 1189,
  [1190] = 1190,
  [1191] = 1191,
  [1192] = 1192,
  [1193] = 1193,
  [1194] = 1194,
  [1195] = 1195,
  [1196] = 1196,
  [1197] = 1197,
  [1198] = 1198,
  [1199] = 1199,
  [1200] = 1200,
  [1201] = 1201,
  [1202] = 1202,
  [1203] = 1203,
  [1204] = 1204,
  [1205] = 1205,
  [1206] = 1206,
  [1207] = 1207,
  [1208] = 1208,
  [1209] = 1209,
  [1210] = 1210,
  [1211] = 1211,
  [1212] = 1212,
  [1213] = 1213,
  [1214] = 1214,
  [1215] = 1215,
  [1216] = 1216,
  [1217] = 1217,
  [1218] = 1218,
  [1219] = 1219,
  [1220] = 1220,
  [1221] = 1221,
  [1222] = 1222,
  [1223] = 1223,
  [1224] = 1224,
  [1225] = 1225,
  [1226] = 1226,
  [1227] = 1227,
  [1228] = 1228,
  [1229] = 1229,
  [1230] = 1230,
  [1231] = 1231,
  [1232] = 1232,
  [1233] = 1233,
  [1234] = 1234,
  [1235] = 1235,
  [1236] = 1236,
  [1237] = 1237,
  [1238] = 1238,
  [1239] = 1239,
  [1240] = 1240,
  [1241] = 1241,
  [1242] = 1242,
  [1243] = 1243,
  [1244] = 1244,
  [1245] = 1245,
  [1246] = 1246,
  [1247] = 1247,
  [1248] = 1248,
  [1249] = 1249,
  [1250] = 1250,
  [1251] = 1251,
  [1252] = 1252,
  [1253] = 1253,
  [1254] = 1254,
  [1255] = 1255,
  [1256] = 1256,
  [1257] = 1257,
  [1258] = 1258,
  [1259] = 1259,
  [1260] = 1260,
  [1261] = 1261,
  [1262] = 1262,
  [1263] = 1263,
  [1264] = 1264,
  [1265] = 1265,
  [1266] = 1266,
  [1267] = 1267,
  [1268] = 1268,
  [1269] = 1269,
  [1270] = 1270,
  [1271] = 1271,
  [1272] = 1272,
  [1273] = 1273,
  [1274] = 1274,
  [1275] = 1275,
  [1276] = 1276,
  [1277] = 1277,
  [1278] = 1278,
  [1279] = 1279,
  [1280] = 1280,
  [1281] = 1281,
  [1282] = 1282,
  [1283] = 1283,
  [1284] = 1284,
  [1285] = 1285,
  [1286] = 1286,
  [1287] = 1287,
  [1288] = 1288,
  [1289] = 1289,
  [1290] = 1290,
  [1291] = 1291,
  [1292] = 1292,
  [1293] = 1293,
  [1294] = 1294,
  [1295] = 1295,
  [1296] = 1296,
  [1297] = 1297,
  [1298] = 1298,
  [1299] = 1299,
  [1300] = 1300,
  [1301] = 1301,
  [1302] = 1302,
  [1303] = 1303,
  [1304] = 1304,
  [1305] = 1305,
  [1306] = 1306,
  [1307] = 1307,
  [1308] = 1308,
  [1309] = 1309,
  [1310] = 1310,
  [1311] = 1311,
  [1312] = 1312,
  [1313] = 1313,
  [1314] = 1314,
  [1315] = 1315,
  [1316] = 1316,
  [1317] = 1317,
  [1318] = 1318,
  [1319] = 1319,
  [1320] = 1320,
  [1321] = 1321,
  [1322] = 1322,
  [1323] = 1323,
  [1324] = 1324,
  [1325] = 1325,
  [1326] = 1326,
  [1327] = 1327,
  [1328] = 1328,
  [1329] = 1329,
  [1330] = 1330,
  [1331] = 1331,
  [1332] = 1332,
  [1333] = 1333,
  [1334] = 1334,
  [1335] = 1335,
  [1336] = 1336,
  [1337] = 1337,
  [1338] = 1338,
  [1339] = 1339,
  [1340] = 1340,
  [1341] = 1341,
  [1342] = 1342,
  [1343] = 1343,
  [1344] = 1344,
  [1345] = 1345,
  [1346] = 1346,
  [1347] = 1347,
  [1348] = 1348,
  [1349] = 1349,
  [1350] = 1350,
  [1351] = 1351,
  [1352] = 1352,
  [1353] = 1353,
  [1354] = 1354,
  [1355] = 1355,
  [1356] = 1356,
  [1357] = 1357,
  [1358] = 1358,
  [1359] = 1359,
  [1360] = 1360,
  [1361] = 1361,
  [1362] = 1362,
  [1363] = 1361,
  [1364] = 1364,
  [1365] = 1365,
  [1366] = 1366,
  [1367] = 1364,
  [1368] = 1368,
  [1369] = 1362,
  [1370] = 1370,
  [1371] = 1371,
  [1372] = 1372,
  [1373] = 1373,
  [1374] = 1374,
  [1375] = 1375,
  [1376] = 1376,
  [1377] = 1377,
  [1378] = 1377,
  [1379] = 1379,
  [1380] = 1380,
  [1381] = 1381,
  [1382] = 1382,
  [1383] = 1383,
  [1384] = 1384,
  [1385] = 1370,
  [1386] = 1380,
  [1387] = 1383,
  [1388] = 1388,
  [1389] = 1381,
  [1390] = 1382,
  [1391] = 1391,
  [1392] = 1392,
  [1393] = 1393,
  [1394] = 1394,
  [1395] = 1395,
  [1396] = 1396,
  [1397] = 1397,
  [1398] = 1398,
  [1399] = 1398,
  [1400] = 1400,
  [1401] = 1401,
  [1402] = 1392,
  [1403] = 1391,
  [1404] = 1404,
  [1405] = 1343,
  [1406] = 1406,
  [1407] = 1407,
  [1408] = 1408,
  [1409] = 1409,
  [1410] = 1410,
  [1411] = 1410,
  [1412] = 1412,
  [1413] = 1395,
  [1414] = 1397,
  [1415] = 1406,
  [1416] = 1416,
  [1417] = 1417,
  [1418] = 1412,
  [1419] = 1396,
  [1420] = 1420,
  [1421] = 1421,
  [1422] = 1408,
  [1423] = 1423,
  [1424] = 1401,
  [1425] = 1423,
  [1426] = 1400,
  [1427] = 1427,
  [1428] = 1427,
  [1429] = 1429,
  [1430] = 1430,
  [1431] = 1421,
  [1432] = 1427,
  [1433] = 1421,
  [1434] = 1427,
  [1435] = 1421,
  [1436] = 1436,
  [1437] = 1437,
  [1438] = 1437,
  [1439] = 1437,
  [1440] = 1440,
  [1441] = 1441,
  [1442] = 1437,
  [1443] = 1437,
  [1444] = 1444,
  [1445] = 1445,
  [1446] = 1446,
  [1447] = 1446,
  [1448] = 1448,
  [1449] = 1445,
  [1450] = 1450,
  [1451] = 1451,
  [1452] = 1452,
  [1453] = 1453,
  [1454] = 1453,
  [1455] = 1453,
  [1456] = 1456,
  [1457] = 1453,
  [1458] = 1446,
  [1459] = 1459,
  [1460] = 1451,
  [1461] = 1459,
  [1462] = 1462,
  [1463] = 1452,
  [1464] = 1459,
  [1465] = 1465,
  [1466] = 1452,
  [1467] = 1453,
  [1468] = 1459,
  [1469] = 1459,
  [1470] = 1470,
  [1471] = 1471,
  [1472] = 1472,
  [1473] = 1473,
  [1474] = 1473,
  [1475] = 1470,
  [1476] = 1476,
  [1477] = 1477,
  [1478] = 1478,
  [1479] = 1479,
  [1480] = 1480,
  [1481] = 1481,
  [1482] = 1482,
  [1483] = 1483,
  [1484] = 1484,
  [1485] = 1485,
  [1486] = 1486,
  [1487] = 1487,
  [1488] = 1488,
  [1489] = 1489,
  [1490] = 1490,
  [1491] = 1491,
  [1492] = 1492,
  [1493] = 1493,
  [1494] = 1494,
  [1495] = 1495,
  [1496] = 1496,
  [1497] = 1497,
  [1498] = 1498,
  [1499] = 1499,
  [1500] = 1500,
  [1501] = 1501,
  [1502] = 1502,
  [1503] = 1503,
  [1504] = 1504,
  [1505] = 1505,
  [1506] = 1506,
  [1507] = 1507,
  [1508] = 1508,
  [1509] = 1509,
  [1510] = 1510,
  [1511] = 1511,
  [1512] = 1512,
  [1513] = 1513,
  [1514] = 1514,
  [1515] = 1515,
  [1516] = 1516,
  [1517] = 1517,
  [1518] = 1518,
  [1519] = 1519,
  [1520] = 1520,
  [1521] = 1521,
  [1522] = 1522,
  [1523] = 1523,
  [1524] = 1524,
  [1525] = 1525,
  [1526] = 1526,
  [1527] = 1527,
  [1528] = 1528,
  [1529] = 1529,
  [1530] = 1530,
  [1531] = 1531,
  [1532] = 1532,
  [1533] = 1533,
  [1534] = 1534,
  [1535] = 1535,
  [1536] = 1536,
  [1537] = 1537,
  [1538] = 1538,
  [1539] = 1539,
  [1540] = 1540,
  [1541] = 1541,
  [1542] = 1542,
  [1543] = 1543,
  [1544] = 1544,
  [1545] = 1545,
  [1546] = 1546,
  [1547] = 1547,
  [1548] = 1548,
  [1549] = 1549,
  [1550] = 1550,
  [1551] = 1551,
  [1552] = 1552,
  [1553] = 1553,
  [1554] = 1554,
  [1555] = 1555,
  [1556] = 1556,
  [1557] = 1557,
  [1558] = 1558,
  [1559] = 1559,
  [1560] = 1560,
  [1561] = 1561,
  [1562] = 1562,
  [1563] = 1563,
  [1564] = 1564,
  [1565] = 1565,
  [1566] = 1566,
  [1567] = 1567,
  [1568] = 1568,
  [1569] = 1569,
  [1570] = 1570,
  [1571] = 1571,
  [1572] = 1572,
  [1573] = 1494,
  [1574] = 1497,
  [1575] = 1511,
  [1576] = 1523,
  [1577] = 1577,
  [1578] = 1578,
  [1579] = 1579,
  [1580] = 1580,
  [1581] = 1581,
  [1582] = 1582,
  [1583] = 1583,
  [1584] = 1584,
  [1585] = 1585,
  [1586] = 1570,
  [1587] = 1583,
  [1588] = 1584,
  [1589] = 1567,
  [1590] = 1568,
  [1591] = 1591,
  [1592] = 1592,
  [1593] = 1593,
  [1594] = 1594,
  [1595] = 1595,
  [1596] = 1494,
  [1597] = 1497,
  [1598] = 1511,
  [1599] = 1523,
  [1600] = 1600,
  [1601] = 1601,
  [1602] = 1577,
  [1603] = 1578,
  [1604] = 1579,
  [1605] = 1580,
  [1606] = 1581,
  [1607] = 1607,
  [1608] = 1583,
  [1609] = 1584,
  [1610] = 1610,
  [1611] = 1585,
  [1612] = 1612,
  [1613] = 1570,
  [1614] = 1614,
  [1615] = 1615,
  [1616] = 1616,
  [1617] = 1617,
  [1618] = 1618,
  [1619] = 1619,
  [1620] = 1620,
  [1621] = 1621,
  [1622] = 1622,
  [1623] = 1623,
  [1624] = 1624,
  [1625] = 1625,
  [1626] = 1626,
  [1627] = 1627,
  [1628] = 1628,
  [1629] = 1629,
  [1630] = 1630,
  [1631] = 1631,
  [1632] = 1632,
  [1633] = 1633,
  [1634] = 1634,
  [1635] = 1635,
  [1636] = 1636,
  [1637] = 1637,
  [1638] = 1638,
  [1639] = 1639,
  [1640] = 1640,
  [1641] = 1641,
  [1642] = 1642,
  [1643] = 1643,
  [1644] = 1644,
  [1645] = 1645,
  [1646] = 1646,
  [1647] = 1647,
  [1648] = 1648,
  [1649] = 1649,
  [1650] = 1494,
  [1651] = 1497,
  [1652] = 1652,
  [1653] = 1511,
  [1654] = 1518,
  [1655] = 1523,
  [1656] = 1656,
  [1657] = 1657,
  [1658] = 1658,
  [1659] = 1659,
  [1660] = 1660,
  [1661] = 1577,
  [1662] = 1578,
  [1663] = 1580,
  [1664] = 1581,
  [1665] = 1665,
  [1666] = 1666,
  [1667] = 1667,
  [1668] = 1668,
  [1669] = 1669,
  [1670] = 1670,
  [1671] = 1671,
  [1672] = 1672,
  [1673] = 1673,
  [1674] = 1674,
  [1675] = 1675,
  [1676] = 1571,
  [1677] = 1677,
  [1678] = 1583,
  [1679] = 1584,
  [1680] = 1680,
  [1681] = 1681,
  [1682] = 1682,
  [1683] = 1585,
  [1684] = 1570,
  [1685] = 1685,
  [1686] = 1686,
  [1687] = 1687,
  [1688] = 1494,
  [1689] = 1497,
  [1690] = 1690,
  [1691] = 1511,
  [1692] = 1518,
  [1693] = 1693,
  [1694] = 1694,
  [1695] = 1695,
  [1696] = 1696,
  [1697] = 1577,
  [1698] = 1578,
  [1699] = 1580,
  [1700] = 1581,
  [1701] = 1583,
  [1702] = 1584,
  [1703] = 1585,
  [1704] = 1570,
  [1705] = 1705,
  [1706] = 1706,
  [1707] = 1707,
  [1708] = 1577,
  [1709] = 1709,
  [1710] = 1578,
  [1711] = 1579,
  [1712] = 1712,
  [1713] = 1713,
  [1714] = 1580,
  [1715] = 1715,
  [1716] = 1716,
  [1717] = 1581,
  [1718] = 1718,
  [1719] = 1567,
  [1720] = 1720,
  [1721] = 1721,
  [1722] = 1722,
  [1723] = 1568,
  [1724] = 1718,
  [1725] = 1567,
  [1726] = 1568,
  [1727] = 1549,
  [1728] = 1680,
  [1729] = 1729,
  [1730] = 1533,
  [1731] = 1536,
  [1732] = 1732,
  [1733] = 1548,
  [1734] = 1551,
  [1735] = 1552,
  [1736] = 1557,
  [1737] = 1558,
  [1738] = 1563,
  [1739] = 1566,
  [1740] = 1582,
  [1741] = 1741,
  [1742] = 1742,
  [1743] = 1743,
  [1744] = 1579,
  [1745] = 1720,
  [1746] = 1721,
  [1747] = 1722,
  [1748] = 1476,
  [1749] = 1729,
  [1750] = 1750,
  [1751] = 1751,
  [1752] = 1752,
  [1753] = 1753,
  [1754] = 1682,
  [1755] = 1485,
  [1756] = 1487,
  [1757] = 1488,
  [1758] = 1489,
  [1759] = 1490,
  [1760] = 1491,
  [1761] = 1502,
  [1762] = 1509,
  [1763] = 1512,
  [1764] = 1515,
  [1765] = 1750,
  [1766] = 1521,
  [1767] = 1524,
  [1768] = 1528,
  [1769] = 1532,
  [1770] = 1535,
  [1771] = 1540,
  [1772] = 1591,
  [1773] = 1592,
  [1774] = 1593,
  [1775] = 1775,
  [1776] = 1594,
  [1777] = 1614,
  [1778] = 1616,
  [1779] = 1619,
  [1780] = 1620,
  [1781] = 1781,
  [1782] = 1626,
  [1783] = 1627,
  [1784] = 1632,
  [1785] = 1635,
  [1786] = 1638,
  [1787] = 1645,
  [1788] = 1646,
  [1789] = 1647,
  [1790] = 1648,
  [1791] = 1649,
  [1792] = 1652,
  [1793] = 1665,
  [1794] = 1668,
  [1795] = 1671,
  [1796] = 1673,
  [1797] = 1675,
  [1798] = 1677,
  [1799] = 1681,
  [1800] = 1800,
  [1801] = 1685,
  [1802] = 1686,
  [1803] = 1687,
  [1804] = 1693,
  [1805] = 1694,
  [1806] = 1696,
  [1807] = 1705,
  [1808] = 1707,
  [1809] = 1709,
  [1810] = 1712,
  [1811] = 1715,
  [1812] = 1716,
  [1813] = 1813,
  [1814] = 1814,
  [1815] = 1815,
  [1816] = 1480,
  [1817] = 1481,
  [1818] = 1818,
  [1819] = 1498,
  [1820] = 1499,
  [1821] = 1500,
  [1822] = 1503,
  [1823] = 1504,
  [1824] = 1505,
  [1825] = 1506,
  [1826] = 1507,
  [1827] = 1508,
  [1828] = 1510,
  [1829] = 1513,
  [1830] = 1514,
  [1831] = 1516,
  [1832] = 1522,
  [1833] = 1526,
  [1834] = 1527,
  [1835] = 1529,
  [1836] = 1530,
  [1837] = 1837,
  [1838] = 1534,
  [1839] = 1537,
  [1840] = 1538,
  [1841] = 1539,
  [1842] = 1541,
  [1843] = 1542,
  [1844] = 1543,
  [1845] = 1544,
  [1846] = 1545,
  [1847] = 1546,
  [1848] = 1547,
  [1849] = 1553,
  [1850] = 1554,
  [1851] = 1555,
  [1852] = 1556,
  [1853] = 1559,
  [1854] = 1560,
  [1855] = 1561,
  [1856] = 1562,
  [1857] = 1564,
  [1858] = 1572,
  [1859] = 1751,
  [1860] = 1610,
  [1861] = 1612,
  [1862] = 1615,
  [1863] = 1617,
  [1864] = 1618,
  [1865] = 1621,
  [1866] = 1622,
  [1867] = 1623,
  [1868] = 1624,
  [1869] = 1625,
  [1870] = 1630,
  [1871] = 1633,
  [1872] = 1634,
  [1873] = 1636,
  [1874] = 1637,
  [1875] = 1639,
  [1876] = 1640,
  [1877] = 1641,
  [1878] = 1642,
  [1879] = 1643,
  [1880] = 1644,
  [1881] = 1656,
  [1882] = 1657,
  [1883] = 1658,
  [1884] = 1659,
  [1885] = 1660,
  [1886] = 1666,
  [1887] = 1667,
  [1888] = 1669,
  [1889] = 1670,
  [1890] = 1672,
  [1891] = 1674,
  [1892] = 1706,
  [1893] = 1585,
  [1894] = 1713,
  [1895] = 1742,
  [1896] = 1743,
  [1897] = 1753,
  [1898] = 1775,
  [1899] = 1781,
  [1900] = 1900,
  [1901] = 1901,
  [1902] = 1902,
  [1903] = 1903,
  [1904] = 1904,
  [1905] = 1905,
  [1906] = 1482,
  [1907] = 1477,
  [1908] = 1478,
  [1909] = 1501,
  [1910] = 1517,
  [1911] = 1519,
  [1912] = 1520,
  [1913] = 1531,
  [1914] = 1900,
  [1915] = 1550,
  [1916] = 1901,
  [1917] = 1902,
  [1918] = 1903,
  [1919] = 1718,
  [1920] = 1904,
  [1921] = 1752,
  [1922] = 1813,
  [1923] = 1905,
  [1924] = 1814,
  [1925] = 1815,
  [1926] = 1800,
  [1927] = 596,
  [1928] = 597,
  [1929] = 1929,
  [1930] = 1376,
  [1931] = 1371,
  [1932] = 1932,
  [1933] = 1372,
  [1934] = 1375,
  [1935] = 1374,
  [1936] = 1936,
  [1937] = 1937,
  [1938] = 1938,
  [1939] = 1939,
  [1940] = 1938,
  [1941] = 1937,
  [1942] = 1939,
  [1943] = 1943,
  [1944] = 1943,
  [1945] = 1945,
  [1946] = 1946,
  [1947] = 597,
  [1948] = 596,
  [1949] = 597,
  [1950] = 596,
  [1951] = 1951,
  [1952] = 1952,
  [1953] = 1953,
  [1954] = 1954,
  [1955] = 596,
  [1956] = 597,
  [1957] = 597,
  [1958] = 1372,
  [1959] = 1372,
  [1960] = 1375,
  [1961] = 1376,
  [1962] = 1375,
  [1963] = 1374,
  [1964] = 1371,
  [1965] = 596,
  [1966] = 1376,
  [1967] = 1374,
  [1968] = 1371,
  [1969] = 1969,
  [1970] = 1969,
  [1971] = 1205,
  [1972] = 1969,
  [1973] = 1151,
  [1974] = 1969,
  [1975] = 1969,
  [1976] = 1976,
  [1977] = 1977,
  [1978] = 1978,
  [1979] = 1979,
  [1980] = 1980,
  [1981] = 1374,
  [1982] = 1372,
  [1983] = 1375,
  [1984] = 1984,
  [1985] = 1376,
  [1986] = 1371,
  [1987] = 1987,
  [1988] = 1988,
  [1989] = 671,
  [1990] = 618,
  [1991] = 619,
  [1992] = 620,
  [1993] = 621,
  [1994] = 622,
  [1995] = 623,
  [1996] = 624,
  [1997] = 625,
  [1998] = 626,
  [1999] = 627,
  [2000] = 628,
  [2001] = 629,
  [2002] = 630,
  [2003] = 631,
  [2004] = 632,
  [2005] = 633,
  [2006] = 634,
  [2007] = 635,
  [2008] = 636,
  [2009] = 639,
  [2010] = 640,
  [2011] = 647,
  [2012] = 648,
  [2013] = 649,
  [2014] = 650,
  [2015] = 651,
  [2016] = 652,
  [2017] = 653,
  [2018] = 654,
  [2019] = 655,
  [2020] = 656,
  [2021] = 657,
  [2022] = 658,
  [2023] = 659,
  [2024] = 660,
  [2025] = 661,
  [2026] = 662,
  [2027] = 663,
  [2028] = 664,
  [2029] = 665,
  [2030] = 666,
  [2031] = 667,
  [2032] = 668,
  [2033] = 669,
  [2034] = 670,
  [2035] = 607,
  [2036] = 672,
  [2037] = 673,
  [2038] = 674,
  [2039] = 675,
  [2040] = 676,
  [2041] = 677,
  [2042] = 678,
  [2043] = 679,
  [2044] = 680,
  [2045] = 608,
  [2046] = 682,
  [2047] = 683,
  [2048] = 684,
  [2049] = 685,
  [2050] = 686,
  [2051] = 687,
  [2052] = 689,
  [2053] = 690,
  [2054] = 691,
  [2055] = 693,
  [2056] = 694,
  [2057] = 695,
  [2058] = 696,
  [2059] = 697,
  [2060] = 698,
  [2061] = 699,
  [2062] = 700,
  [2063] = 701,
  [2064] = 702,
  [2065] = 703,
  [2066] = 704,
  [2067] = 705,
  [2068] = 706,
  [2069] = 707,
  [2070] = 708,
  [2071] = 709,
  [2072] = 710,
  [2073] = 711,
  [2074] = 712,
  [2075] = 713,
  [2076] = 714,
  [2077] = 715,
  [2078] = 717,
  [2079] = 718,
  [2080] = 719,
  [2081] = 720,
  [2082] = 721,
  [2083] = 722,
  [2084] = 723,
  [2085] = 724,
  [2086] = 725,
  [2087] = 726,
  [2088] = 727,
  [2089] = 728,
  [2090] = 729,
  [2091] = 730,
  [2092] = 731,
  [2093] = 732,
  [2094] = 733,
  [2095] = 734,
  [2096] = 735,
  [2097] = 736,
  [2098] = 737,
  [2099] = 738,
  [2100] = 739,
  [2101] = 740,
  [2102] = 741,
  [2103] = 742,
  [2104] = 743,
  [2105] = 744,
  [2106] = 745,
  [2107] = 746,
  [2108] = 747,
  [2109] = 748,
  [2110] = 749,
  [2111] = 750,
  [2112] = 752,
  [2113] = 753,
  [2114] = 754,
  [2115] = 755,
  [2116] = 756,
  [2117] = 757,
  [2118] = 758,
  [2119] = 759,
  [2120] = 760,
  [2121] = 761,
  [2122] = 762,
  [2123] = 763,
  [2124] = 764,
  [2125] = 765,
  [2126] = 766,
  [2127] = 767,
  [2128] = 768,
  [2129] = 769,
  [2130] = 770,
  [2131] = 771,
  [2132] = 772,
  [2133] = 773,
  [2134] = 774,
  [2135] = 778,
  [2136] = 779,
  [2137] = 780,
  [2138] = 781,
  [2139] = 782,
  [2140] = 783,
  [2141] = 784,
  [2142] = 785,
  [2143] = 786,
  [2144] = 787,
  [2145] = 788,
  [2146] = 789,
  [2147] = 790,
  [2148] = 791,
  [2149] = 792,
  [2150] = 793,
  [2151] = 794,
  [2152] = 795,
  [2153] = 796,
  [2154] = 797,
  [2155] = 798,
  [2156] = 799,
  [2157] = 801,
  [2158] = 802,
  [2159] = 803,
  [2160] = 804,
  [2161] = 805,
  [2162] = 806,
  [2163] = 807,
  [2164] = 808,
  [2165] = 809,
  [2166] = 810,
  [2167] = 811,
  [2168] = 812,
  [2169] = 813,
  [2170] = 814,
  [2171] = 815,
  [2172] = 816,
  [2173] = 817,
  [2174] = 818,
  [2175] = 819,
  [2176] = 820,
  [2177] = 821,
  [2178] = 822,
  [2179] = 823,
  [2180] = 824,
  [2181] = 825,
  [2182] = 826,
  [2183] = 827,
  [2184] = 828,
  [2185] = 829,
  [2186] = 830,
  [2187] = 831,
  [2188] = 833,
  [2189] = 834,
  [2190] = 835,
  [2191] = 836,
  [2192] = 837,
  [2193] = 838,
  [2194] = 839,
  [2195] = 840,
  [2196] = 841,
  [2197] = 842,
  [2198] = 843,
  [2199] = 844,
  [2200] = 845,
  [2201] = 846,
  [2202] = 847,
  [2203] = 848,
  [2204] = 849,
  [2205] = 850,
  [2206] = 851,
  [2207] = 852,
  [2208] = 853,
  [2209] = 854,
  [2210] = 855,
  [2211] = 856,
  [2212] = 857,
  [2213] = 858,
  [2214] = 859,
  [2215] = 860,
  [2216] = 861,
  [2217] = 862,
  [2218] = 863,
  [2219] = 864,
  [2220] = 865,
  [2221] = 866,
  [2222] = 867,
  [2223] = 868,
  [2224] = 869,
  [2225] = 870,
  [2226] = 871,
  [2227] = 872,
  [2228] = 873,
  [2229] = 874,
  [2230] = 875,
  [2231] = 876,
  [2232] = 877,
  [2233] = 878,
  [2234] = 879,
  [2235] = 880,
  [2236] = 881,
  [2237] = 882,
  [2238] = 883,
  [2239] = 884,
  [2240] = 885,
  [2241] = 886,
  [2242] = 887,
  [2243] = 888,
  [2244] = 889,
  [2245] = 890,
  [2246] = 891,
  [2247] = 894,
  [2248] = 895,
  [2249] = 896,
  [2250] = 897,
  [2251] = 898,
  [2252] = 899,
  [2253] = 900,
  [2254] = 901,
  [2255] = 902,
  [2256] = 903,
  [2257] = 904,
  [2258] = 905,
  [2259] = 906,
  [2260] = 907,
  [2261] = 908,
  [2262] = 909,
  [2263] = 910,
  [2264] = 911,
  [2265] = 912,
  [2266] = 913,
  [2267] = 914,
  [2268] = 915,
  [2269] = 916,
  [2270] = 917,
  [2271] = 918,
  [2272] = 919,
  [2273] = 920,
  [2274] = 921,
  [2275] = 922,
  [2276] = 923,
  [2277] = 924,
  [2278] = 925,
  [2279] = 926,
  [2280] = 927,
  [2281] = 928,
  [2282] = 929,
  [2283] = 930,
  [2284] = 931,
  [2285] = 932,
  [2286] = 933,
  [2287] = 934,
  [2288] = 935,
  [2289] = 936,
  [2290] = 937,
  [2291] = 938,
  [2292] = 939,
  [2293] = 940,
  [2294] = 941,
  [2295] = 942,
  [2296] = 943,
  [2297] = 944,
  [2298] = 945,
  [2299] = 946,
  [2300] = 947,
  [2301] = 948,
  [2302] = 949,
  [2303] = 950,
  [2304] = 951,
  [2305] = 952,
  [2306] = 953,
  [2307] = 954,
  [2308] = 955,
  [2309] = 956,
  [2310] = 957,
  [2311] = 958,
  [2312] = 959,
  [2313] = 960,
  [2314] = 961,
  [2315] = 963,
  [2316] = 964,
  [2317] = 965,
  [2318] = 966,
  [2319] = 967,
  [2320] = 968,
  [2321] = 969,
  [2322] = 970,
  [2323] = 971,
  [2324] = 972,
  [2325] = 973,
  [2326] = 974,
  [2327] = 975,
  [2328] = 976,
  [2329] = 977,
  [2330] = 978,
  [2331] = 979,
  [2332] = 980,
  [2333] = 981,
  [2334] = 982,
  [2335] = 983,
  [2336] = 984,
  [2337] = 985,
  [2338] = 986,
  [2339] = 987,
  [2340] = 988,
  [2341] = 989,
  [2342] = 990,
  [2343] = 991,
  [2344] = 992,
  [2345] = 993,
  [2346] = 995,
  [2347] = 996,
  [2348] = 997,
  [2349] = 998,
  [2350] = 999,
  [2351] = 1000,
  [2352] = 1001,
  [2353] = 1002,
  [2354] = 1003,
  [2355] = 1004,
  [2356] = 1005,
  [2357] = 1006,
  [2358] = 1007,
  [2359] = 1008,
  [2360] = 1009,
  [2361] = 1010,
  [2362] = 1011,
  [2363] = 1012,
  [2364] = 1013,
  [2365] = 1014,
  [2366] = 1015,
  [2367] = 1016,
  [2368] = 1017,
  [2369] = 1018,
  [2370] = 599,
  [2371] = 1020,
  [2372] = 1021,
  [2373] = 1022,
  [2374] = 1023,
  [2375] = 1024,
  [2376] = 1025,
  [2377] = 1360,
  [2378] = 1026,
  [2379] = 1027,
  [2380] = 1028,
  [2381] = 1029,
  [2382] = 1030,
  [2383] = 1031,
  [2384] = 1032,
  [2385] = 1033,
  [2386] = 1034,
  [2387] = 1035,
  [2388] = 1036,
  [2389] = 1037,
  [2390] = 1038,
  [2391] = 1040,
  [2392] = 1041,
  [2393] = 1042,
  [2394] = 1043,
  [2395] = 1044,
  [2396] = 1045,
  [2397] = 1046,
  [2398] = 1047,
  [2399] = 1048,
  [2400] = 1049,
  [2401] = 1050,
  [2402] = 1051,
  [2403] = 1052,
  [2404] = 1053,
  [2405] = 1054,
  [2406] = 1055,
  [2407] = 1056,
  [2408] = 1057,
  [2409] = 1058,
  [2410] = 1059,
  [2411] = 1060,
  [2412] = 1061,
  [2413] = 1062,
  [2414] = 1063,
  [2415] = 1064,
  [2416] = 1065,
  [2417] = 1066,
  [2418] = 609,
  [2419] = 610,
  [2420] = 611,
  [2421] = 613,
  [2422] = 2422,
  [2423] = 1279,
  [2424] = 601,
  [2425] = 612,
  [2426] = 2426,
  [2427] = 1092,
  [2428] = 614,
  [2429] = 615,
  [2430] = 1095,
  [2431] = 1245,
  [2432] = 1100,
  [2433] = 645,
  [2434] = 776,
  [2435] = 1193,
  [2436] = 1238,
  [2437] = 1242,
  [2438] = 1277,
  [2439] = 1337,
  [2440] = 467,
  [2441] = 1338,
  [2442] = 485,
  [2443] = 1339,
  [2444] = 1342,
  [2445] = 1344,
  [2446] = 1346,
  [2447] = 1347,
  [2448] = 1348,
  [2449] = 641,
  [2450] = 1076,
  [2451] = 1085,
  [2452] = 1091,
  [2453] = 1094,
  [2454] = 448,
  [2455] = 462,
  [2456] = 688,
  [2457] = 716,
  [2458] = 420,
  [2459] = 429,
  [2460] = 442,
  [2461] = 426,
  [2462] = 444,
  [2463] = 1019,
  [2464] = 1079,
  [2465] = 1080,
  [2466] = 1081,
  [2467] = 1082,
  [2468] = 1083,
  [2469] = 1084,
  [2470] = 1086,
  [2471] = 1087,
  [2472] = 1088,
  [2473] = 1089,
  [2474] = 1090,
  [2475] = 2475,
  [2476] = 1096,
  [2477] = 1097,
  [2478] = 1098,
  [2479] = 1099,
  [2480] = 1101,
  [2481] = 1107,
  [2482] = 1108,
  [2483] = 1109,
  [2484] = 1110,
  [2485] = 1111,
  [2486] = 1112,
  [2487] = 1113,
  [2488] = 1114,
  [2489] = 1115,
  [2490] = 1116,
  [2491] = 1117,
  [2492] = 1118,
  [2493] = 1119,
  [2494] = 1120,
  [2495] = 1121,
  [2496] = 1127,
  [2497] = 1140,
  [2498] = 1141,
  [2499] = 1142,
  [2500] = 1143,
  [2501] = 1145,
  [2502] = 1147,
  [2503] = 1148,
  [2504] = 1149,
  [2505] = 1150,
  [2506] = 1153,
  [2507] = 1154,
  [2508] = 1156,
  [2509] = 1157,
  [2510] = 1158,
  [2511] = 1159,
  [2512] = 1160,
  [2513] = 1161,
  [2514] = 1162,
  [2515] = 1163,
  [2516] = 1164,
  [2517] = 1165,
  [2518] = 1168,
  [2519] = 1185,
  [2520] = 1186,
  [2521] = 1190,
  [2522] = 1191,
  [2523] = 2523,
  [2524] = 962,
  [2525] = 1205,
  [2526] = 1206,
  [2527] = 1207,
  [2528] = 1208,
  [2529] = 1209,
  [2530] = 1212,
  [2531] = 616,
  [2532] = 1214,
  [2533] = 1215,
  [2534] = 1216,
  [2535] = 1217,
  [2536] = 1218,
  [2537] = 1221,
  [2538] = 1222,
  [2539] = 1223,
  [2540] = 1227,
  [2541] = 1228,
  [2542] = 1229,
  [2543] = 1230,
  [2544] = 1232,
  [2545] = 1233,
  [2546] = 1241,
  [2547] = 1247,
  [2548] = 1248,
  [2549] = 1249,
  [2550] = 1250,
  [2551] = 1254,
  [2552] = 1255,
  [2553] = 1257,
  [2554] = 1259,
  [2555] = 1260,
  [2556] = 1261,
  [2557] = 1262,
  [2558] = 1263,
  [2559] = 1266,
  [2560] = 1267,
  [2561] = 1268,
  [2562] = 1269,
  [2563] = 1271,
  [2564] = 1275,
  [2565] = 1276,
  [2566] = 1280,
  [2567] = 1281,
  [2568] = 1282,
  [2569] = 1283,
  [2570] = 1284,
  [2571] = 1285,
  [2572] = 1286,
  [2573] = 1287,
  [2574] = 1288,
  [2575] = 1289,
  [2576] = 1290,
  [2577] = 1292,
  [2578] = 1293,
  [2579] = 1294,
  [2580] = 1295,
  [2581] = 1296,
  [2582] = 1297,
  [2583] = 1298,
  [2584] = 1299,
  [2585] = 1300,
  [2586] = 1301,
  [2587] = 1302,
  [2588] = 1303,
  [2589] = 1304,
  [2590] = 1305,
  [2591] = 1306,
  [2592] = 1307,
  [2593] = 1308,
  [2594] = 1309,
  [2595] = 1310,
  [2596] = 1311,
  [2597] = 1312,
  [2598] = 1313,
  [2599] = 1314,
  [2600] = 1315,
  [2601] = 1316,
  [2602] = 1317,
  [2603] = 1318,
  [2604] = 1319,
  [2605] = 1320,
  [2606] = 1322,
  [2607] = 1323,
  [2608] = 1324,
  [2609] = 1326,
  [2610] = 1327,
  [2611] = 1328,
  [2612] = 1329,
  [2613] = 1330,
  [2614] = 1331,
  [2615] = 1332,
  [2616] = 1333,
  [2617] = 1334,
  [2618] = 1335,
  [2619] = 1336,
  [2620] = 1345,
  [2621] = 1353,
  [2622] = 1355,
  [2623] = 1356,
  [2624] = 604,
  [2625] = 605,
  [2626] = 606,
  [2627] = 637,
  [2628] = 642,
  [2629] = 643,
  [2630] = 644,
  [2631] = 692,
  [2632] = 751,
  [2633] = 775,
  [2634] = 800,
  [2635] = 832,
  [2636] = 892,
  [2637] = 1039,
  [2638] = 1067,
  [2639] = 1068,
  [2640] = 1069,
  [2641] = 1070,
  [2642] = 1071,
  [2643] = 1072,
  [2644] = 1073,
  [2645] = 1074,
  [2646] = 1077,
  [2647] = 1078,
  [2648] = 1093,
  [2649] = 1102,
  [2650] = 1106,
  [2651] = 1122,
  [2652] = 1123,
  [2653] = 1124,
  [2654] = 1125,
  [2655] = 1126,
  [2656] = 1128,
  [2657] = 1129,
  [2658] = 1130,
  [2659] = 1131,
  [2660] = 1132,
  [2661] = 1133,
  [2662] = 1134,
  [2663] = 1135,
  [2664] = 1136,
  [2665] = 1137,
  [2666] = 1138,
  [2667] = 1139,
  [2668] = 1144,
  [2669] = 1151,
  [2670] = 1152,
  [2671] = 1155,
  [2672] = 1166,
  [2673] = 1170,
  [2674] = 1174,
  [2675] = 1175,
  [2676] = 1176,
  [2677] = 1177,
  [2678] = 1178,
  [2679] = 1179,
  [2680] = 1180,
  [2681] = 1181,
  [2682] = 1182,
  [2683] = 1183,
  [2684] = 1184,
  [2685] = 1187,
  [2686] = 1188,
  [2687] = 1189,
  [2688] = 1192,
  [2689] = 1194,
  [2690] = 1195,
  [2691] = 1196,
  [2692] = 1197,
  [2693] = 1198,
  [2694] = 1199,
  [2695] = 1200,
  [2696] = 1201,
  [2697] = 1202,
  [2698] = 1203,
  [2699] = 1204,
  [2700] = 1210,
  [2701] = 1211,
  [2702] = 1213,
  [2703] = 1219,
  [2704] = 1220,
  [2705] = 1224,
  [2706] = 1225,
  [2707] = 1226,
  [2708] = 1231,
  [2709] = 1234,
  [2710] = 1235,
  [2711] = 1236,
  [2712] = 1237,
  [2713] = 1239,
  [2714] = 1240,
  [2715] = 1243,
  [2716] = 1244,
  [2717] = 1246,
  [2718] = 1252,
  [2719] = 1253,
  [2720] = 1256,
  [2721] = 1258,
  [2722] = 1264,
  [2723] = 1265,
  [2724] = 1270,
  [2725] = 1272,
  [2726] = 1273,
  [2727] = 1274,
  [2728] = 1278,
  [2729] = 1291,
  [2730] = 1321,
  [2731] = 1325,
  [2732] = 1340,
  [2733] = 1341,
  [2734] = 1349,
  [2735] = 1350,
  [2736] = 1351,
  [2737] = 1352,
  [2738] = 1354,
  [2739] = 1357,
  [2740] = 1358,
  [2741] = 1359,
  [2742] = 600,
  [2743] = 602,
  [2744] = 603,
  [2745] = 617,
  [2746] = 681,
  [2747] = 1951,
  [2748] = 1371,
  [2749] = 1376,
  [2750] = 1372,
  [2751] = 1374,
  [2752] = 1954,
  [2753] = 1375,
  [2754] = 1953,
  [2755] = 2755,
  [2756] = 2756,
  [2757] = 2757,
  [2758] = 2758,
  [2759] = 2759,
  [2760] = 2760,
  [2761] = 2761,
  [2762] = 2762,
  [2763] = 2763,
  [2764] = 2764,
  [2765] = 2765,
  [2766] = 2766,
  [2767] = 2767,
  [2768] = 2765,
  [2769] = 2769,
  [2770] = 2770,
  [2771] = 2771,
  [2772] = 2772,
  [2773] = 2770,
  [2774] = 2774,
  [2775] = 2772,
  [2776] = 2776,
  [2777] = 2777,
  [2778] = 2778,
  [2779] = 2779,
  [2780] = 2780,
  [2781] = 2781,
  [2782] = 2782,
  [2783] = 2783,
  [2784] = 2784,
  [2785] = 2785,
  [2786] = 2786,
  [2787] = 2787,
  [2788] = 2788,
  [2789] = 2789,
  [2790] = 2790,
  [2791] = 2791,
  [2792] = 2792,
  [2793] = 2793,
  [2794] = 2794,
  [2795] = 2795,
  [2796] = 2796,
  [2797] = 2797,
  [2798] = 2798,
  [2799] = 2799,
  [2800] = 2800,
  [2801] = 2801,
  [2802] = 2802,
  [2803] = 425,
  [2804] = 2804,
  [2805] = 2805,
  [2806] = 2806,
  [2807] = 441,
  [2808] = 2808,
  [2809] = 2809,
  [2810] = 2810,
  [2811] = 2811,
  [2812] = 2812,
  [2813] = 2813,
  [2814] = 2814,
  [2815] = 2815,
  [2816] = 2816,
  [2817] = 421,
  [2818] = 2818,
  [2819] = 2819,
  [2820] = 2820,
  [2821] = 2821,
  [2822] = 2822,
  [2823] = 2823,
  [2824] = 2824,
  [2825] = 2825,
  [2826] = 2826,
  [2827] = 2827,
  [2828] = 2828,
  [2829] = 2829,
  [2830] = 2830,
  [2831] = 2831,
  [2832] = 2832,
  [2833] = 2833,
  [2834] = 2774,
  [2835] = 2835,
  [2836] = 414,
  [2837] = 2837,
  [2838] = 2838,
  [2839] = 2839,
  [2840] = 2840,
  [2841] = 2841,
  [2842] = 2842,
  [2843] = 2843,
  [2844] = 2844,
  [2845] = 500,
  [2846] = 2846,
  [2847] = 1946,
  [2848] = 2848,
  [2849] = 415,
  [2850] = 2830,
  [2851] = 2824,
  [2852] = 2806,
  [2853] = 2842,
  [2854] = 2854,
  [2855] = 2814,
  [2856] = 2831,
  [2857] = 2838,
  [2858] = 2808,
  [2859] = 2859,
  [2860] = 469,
  [2861] = 449,
  [2862] = 442,
  [2863] = 448,
  [2864] = 2864,
  [2865] = 2865,
  [2866] = 2774,
  [2867] = 2867,
  [2868] = 2868,
  [2869] = 2869,
  [2870] = 2870,
  [2871] = 2871,
  [2872] = 2872,
  [2873] = 2873,
  [2874] = 2874,
  [2875] = 2875,
  [2876] = 2876,
  [2877] = 2877,
  [2878] = 2878,
  [2879] = 2879,
  [2880] = 2880,
  [2881] = 2881,
  [2882] = 422,
  [2883] = 2883,
  [2884] = 2884,
  [2885] = 456,
  [2886] = 2886,
  [2887] = 459,
  [2888] = 2888,
  [2889] = 2889,
  [2890] = 2890,
  [2891] = 2891,
  [2892] = 2892,
  [2893] = 2893,
  [2894] = 2894,
  [2895] = 420,
  [2896] = 2896,
  [2897] = 2897,
  [2898] = 2898,
  [2899] = 2899,
  [2900] = 2900,
  [2901] = 2901,
  [2902] = 2902,
  [2903] = 2903,
  [2904] = 444,
  [2905] = 2905,
  [2906] = 2906,
  [2907] = 2907,
  [2908] = 2908,
  [2909] = 2909,
  [2910] = 468,
  [2911] = 2911,
  [2912] = 2912,
  [2913] = 2913,
  [2914] = 462,
  [2915] = 436,
  [2916] = 2916,
  [2917] = 2917,
  [2918] = 2779,
  [2919] = 2919,
  [2920] = 2920,
  [2921] = 2921,
  [2922] = 2922,
  [2923] = 2923,
  [2924] = 2924,
  [2925] = 2925,
  [2926] = 2926,
  [2927] = 2927,
  [2928] = 487,
  [2929] = 2929,
  [2930] = 472,
  [2931] = 2931,
  [2932] = 478,
  [2933] = 480,
  [2934] = 484,
  [2935] = 2935,
  [2936] = 2936,
  [2937] = 476,
  [2938] = 477,
  [2939] = 2939,
  [2940] = 2940,
  [2941] = 2941,
  [2942] = 2942,
  [2943] = 479,
  [2944] = 2944,
  [2945] = 2945,
  [2946] = 2946,
  [2947] = 2947,
  [2948] = 2948,
  [2949] = 2949,
  [2950] = 2950,
  [2951] = 429,
  [2952] = 467,
  [2953] = 2953,
  [2954] = 2954,
  [2955] = 2955,
  [2956] = 2956,
  [2957] = 2957,
  [2958] = 2958,
  [2959] = 2959,
  [2960] = 2960,
  [2961] = 2961,
  [2962] = 474,
  [2963] = 2963,
  [2964] = 475,
  [2965] = 2965,
  [2966] = 2966,
  [2967] = 452,
  [2968] = 481,
  [2969] = 2969,
  [2970] = 2970,
  [2971] = 2779,
  [2972] = 426,
  [2973] = 482,
  [2974] = 2974,
  [2975] = 2975,
  [2976] = 483,
  [2977] = 2977,
  [2978] = 2978,
  [2979] = 2979,
  [2980] = 2980,
  [2981] = 2981,
  [2982] = 2982,
  [2983] = 2983,
  [2984] = 485,
  [2985] = 2985,
  [2986] = 2986,
  [2987] = 471,
  [2988] = 2988,
  [2989] = 2989,
  [2990] = 2990,
  [2991] = 2991,
  [2992] = 2992,
  [2993] = 2993,
  [2994] = 2994,
  [2995] = 2774,
  [2996] = 2996,
  [2997] = 2996,
  [2998] = 2998,
  [2999] = 2994,
  [3000] = 3000,
  [3001] = 3001,
  [3002] = 3002,
  [3003] = 3003,
  [3004] = 3004,
  [3005] = 2779,
  [3006] = 2774,
  [3007] = 3001,
  [3008] = 2998,
  [3009] = 3009,
  [3010] = 3000,
  [3011] = 3004,
  [3012] = 3003,
  [3013] = 2990,
  [3014] = 3002,
  [3015] = 2992,
  [3016] = 2989,
  [3017] = 2988,
  [3018] = 3018,
  [3019] = 3019,
  [3020] = 3020,
  [3021] = 3021,
  [3022] = 3022,
  [3023] = 3023,
  [3024] = 3024,
  [3025] = 3025,
  [3026] = 3026,
  [3027] = 3027,
  [3028] = 2779,
  [3029] = 3029,
  [3030] = 2797,
  [3031] = 2771,
  [3032] = 2769,
  [3033] = 3033,
  [3034] = 2779,
  [3035] = 2766,
  [3036] = 3036,
  [3037] = 3037,
  [3038] = 3038,
  [3039] = 3039,
  [3040] = 3040,
  [3041] = 3041,
  [3042] = 3042,
  [3043] = 2800,
  [3044] = 3044,
  [3045] = 3045,
  [3046] = 3046,
  [3047] = 2797,
  [3048] = 3048,
  [3049] = 2800,
  [3050] = 2780,
  [3051] = 2800,
  [3052] = 3052,
  [3053] = 2797,
  [3054] = 3054,
  [3055] = 2944,
  [3056] = 3056,
  [3057] = 3057,
  [3058] = 2787,
  [3059] = 2821,
  [3060] = 2837,
  [3061] = 2837,
  [3062] = 2790,
  [3063] = 2793,
  [3064] = 2792,
  [3065] = 2818,
  [3066] = 2944,
  [3067] = 2791,
  [3068] = 2821,
  [3069] = 2841,
  [3070] = 2789,
  [3071] = 2818,
  [3072] = 2946,
  [3073] = 2946,
  [3074] = 2786,
  [3075] = 2841,
  [3076] = 2788,
  [3077] = 2798,
  [3078] = 3078,
  [3079] = 3079,
  [3080] = 2946,
  [3081] = 414,
  [3082] = 2844,
  [3083] = 2813,
  [3084] = 3009,
  [3085] = 3085,
  [3086] = 2883,
  [3087] = 2794,
  [3088] = 3088,
  [3089] = 2813,
  [3090] = 2991,
  [3091] = 3091,
  [3092] = 3088,
  [3093] = 3093,
  [3094] = 3088,
  [3095] = 3095,
  [3096] = 2822,
  [3097] = 2799,
  [3098] = 3091,
  [3099] = 2965,
  [3100] = 2944,
  [3101] = 3085,
  [3102] = 2795,
  [3103] = 2965,
  [3104] = 3104,
  [3105] = 2796,
  [3106] = 2883,
  [3107] = 3107,
  [3108] = 2854,
  [3109] = 3109,
  [3110] = 3088,
  [3111] = 3111,
  [3112] = 3085,
  [3113] = 2846,
  [3114] = 3107,
  [3115] = 3109,
  [3116] = 3085,
  [3117] = 3079,
  [3118] = 3111,
  [3119] = 3093,
  [3120] = 3120,
  [3121] = 3078,
  [3122] = 3095,
  [3123] = 2804,
  [3124] = 3104,
  [3125] = 3120,
  [3126] = 3019,
  [3127] = 2832,
  [3128] = 3045,
  [3129] = 2833,
  [3130] = 2835,
  [3131] = 3033,
  [3132] = 2810,
  [3133] = 3024,
  [3134] = 2812,
  [3135] = 2811,
  [3136] = 2859,
  [3137] = 3137,
  [3138] = 3027,
  [3139] = 3139,
  [3140] = 2840,
  [3141] = 3018,
  [3142] = 426,
  [3143] = 3137,
  [3144] = 429,
  [3145] = 2820,
  [3146] = 2802,
  [3147] = 3022,
  [3148] = 3037,
  [3149] = 2823,
  [3150] = 2819,
  [3151] = 415,
  [3152] = 3041,
  [3153] = 3025,
  [3154] = 3038,
  [3155] = 3020,
  [3156] = 2827,
  [3157] = 2828,
  [3158] = 3029,
  [3159] = 2991,
  [3160] = 3039,
  [3161] = 444,
  [3162] = 2848,
  [3163] = 420,
  [3164] = 3040,
  [3165] = 2826,
  [3166] = 3042,
  [3167] = 2815,
  [3168] = 2801,
  [3169] = 2805,
  [3170] = 3021,
  [3171] = 3009,
  [3172] = 3139,
  [3173] = 3023,
  [3174] = 3026,
  [3175] = 2843,
  [3176] = 442,
  [3177] = 2839,
  [3178] = 2816,
  [3179] = 2809,
  [3180] = 3044,
  [3181] = 3036,
  [3182] = 2825,
  [3183] = 3183,
  [3184] = 2957,
  [3185] = 2983,
  [3186] = 449,
  [3187] = 2950,
  [3188] = 2986,
  [3189] = 3189,
  [3190] = 3190,
  [3191] = 2907,
  [3192] = 3192,
  [3193] = 487,
  [3194] = 452,
  [3195] = 2970,
  [3196] = 3196,
  [3197] = 3190,
  [3198] = 3033,
  [3199] = 3199,
  [3200] = 2873,
  [3201] = 3201,
  [3202] = 3202,
  [3203] = 436,
  [3204] = 3204,
  [3205] = 2878,
  [3206] = 3206,
  [3207] = 3207,
  [3208] = 3208,
  [3209] = 2980,
  [3210] = 2954,
  [3211] = 3211,
  [3212] = 422,
  [3213] = 2886,
  [3214] = 3214,
  [3215] = 3183,
  [3216] = 3216,
  [3217] = 3024,
  [3218] = 2981,
  [3219] = 3045,
  [3220] = 3033,
  [3221] = 3024,
  [3222] = 3222,
  [3223] = 3027,
  [3224] = 3037,
  [3225] = 3225,
  [3226] = 2940,
  [3227] = 2982,
  [3228] = 2941,
  [3229] = 3207,
  [3230] = 471,
  [3231] = 421,
  [3232] = 2891,
  [3233] = 3044,
  [3234] = 3234,
  [3235] = 2979,
  [3236] = 441,
  [3237] = 500,
  [3238] = 467,
  [3239] = 3239,
  [3240] = 2975,
  [3241] = 3025,
  [3242] = 3192,
  [3243] = 3027,
  [3244] = 425,
  [3245] = 485,
  [3246] = 3037,
  [3247] = 3183,
  [3248] = 3222,
  [3249] = 3225,
  [3250] = 2871,
  [3251] = 469,
  [3252] = 2884,
  [3253] = 3239,
  [3254] = 3192,
  [3255] = 3183,
  [3256] = 3225,
  [3257] = 3038,
  [3258] = 3239,
  [3259] = 3192,
  [3260] = 3183,
  [3261] = 3225,
  [3262] = 472,
  [3263] = 2893,
  [3264] = 2894,
  [3265] = 3018,
  [3266] = 3239,
  [3267] = 478,
  [3268] = 3192,
  [3269] = 3269,
  [3270] = 3183,
  [3271] = 3225,
  [3272] = 3225,
  [3273] = 2897,
  [3274] = 3192,
  [3275] = 480,
  [3276] = 3239,
  [3277] = 3192,
  [3278] = 484,
  [3279] = 2919,
  [3280] = 3225,
  [3281] = 3239,
  [3282] = 3192,
  [3283] = 2927,
  [3284] = 3208,
  [3285] = 3189,
  [3286] = 2876,
  [3287] = 3018,
  [3288] = 2877,
  [3289] = 3289,
  [3290] = 3196,
  [3291] = 3291,
  [3292] = 448,
  [3293] = 2908,
  [3294] = 462,
  [3295] = 3295,
  [3296] = 3289,
  [3297] = 2913,
  [3298] = 2909,
  [3299] = 2917,
  [3300] = 3300,
  [3301] = 2900,
  [3302] = 2880,
  [3303] = 2936,
  [3304] = 2925,
  [3305] = 2872,
  [3306] = 2911,
  [3307] = 2916,
  [3308] = 2903,
  [3309] = 2905,
  [3310] = 2945,
  [3311] = 2985,
  [3312] = 3312,
  [3313] = 3313,
  [3314] = 3044,
  [3315] = 3202,
  [3316] = 2921,
  [3317] = 3214,
  [3318] = 2942,
  [3319] = 2955,
  [3320] = 3201,
  [3321] = 2977,
  [3322] = 3295,
  [3323] = 2881,
  [3324] = 3324,
  [3325] = 3325,
  [3326] = 2892,
  [3327] = 2959,
  [3328] = 2899,
  [3329] = 3329,
  [3330] = 2974,
  [3331] = 474,
  [3332] = 3332,
  [3333] = 3222,
  [3334] = 3183,
  [3335] = 3025,
  [3336] = 3026,
  [3337] = 2939,
  [3338] = 475,
  [3339] = 2966,
  [3340] = 2978,
  [3341] = 481,
  [3342] = 2969,
  [3343] = 2947,
  [3344] = 2912,
  [3345] = 483,
  [3346] = 2898,
  [3347] = 2948,
  [3348] = 2926,
  [3349] = 2906,
  [3350] = 3222,
  [3351] = 3022,
  [3352] = 3041,
  [3353] = 3019,
  [3354] = 3020,
  [3355] = 2949,
  [3356] = 3183,
  [3357] = 2888,
  [3358] = 3206,
  [3359] = 2889,
  [3360] = 3225,
  [3361] = 3361,
  [3362] = 3029,
  [3363] = 3039,
  [3364] = 3040,
  [3365] = 3042,
  [3366] = 2935,
  [3367] = 3021,
  [3368] = 3044,
  [3369] = 3023,
  [3370] = 2896,
  [3371] = 2924,
  [3372] = 3036,
  [3373] = 2956,
  [3374] = 2958,
  [3375] = 3332,
  [3376] = 1946,
  [3377] = 2960,
  [3378] = 2961,
  [3379] = 2868,
  [3380] = 2870,
  [3381] = 2874,
  [3382] = 3025,
  [3383] = 3026,
  [3384] = 3384,
  [3385] = 3385,
  [3386] = 3386,
  [3387] = 3387,
  [3388] = 2890,
  [3389] = 3038,
  [3390] = 3022,
  [3391] = 3041,
  [3392] = 3019,
  [3393] = 3020,
  [3394] = 3029,
  [3395] = 3039,
  [3396] = 3040,
  [3397] = 3042,
  [3398] = 3021,
  [3399] = 3023,
  [3400] = 3036,
  [3401] = 3385,
  [3402] = 456,
  [3403] = 2864,
  [3404] = 3404,
  [3405] = 3405,
  [3406] = 2865,
  [3407] = 459,
  [3408] = 2867,
  [3409] = 2869,
  [3410] = 3289,
  [3411] = 3295,
  [3412] = 468,
  [3413] = 3214,
  [3414] = 2875,
  [3415] = 2901,
  [3416] = 3416,
  [3417] = 3289,
  [3418] = 2879,
  [3419] = 3419,
  [3420] = 2920,
  [3421] = 3289,
  [3422] = 3289,
  [3423] = 2922,
  [3424] = 2923,
  [3425] = 3416,
  [3426] = 2929,
  [3427] = 3038,
  [3428] = 3239,
  [3429] = 2931,
  [3430] = 476,
  [3431] = 477,
  [3432] = 479,
  [3433] = 3018,
  [3434] = 3384,
  [3435] = 3045,
  [3436] = 482,
  [3437] = 2953,
  [3438] = 3239,
  [3439] = 3439,
  [3440] = 3440,
  [3441] = 3441,
  [3442] = 3442,
  [3443] = 3443,
  [3444] = 3444,
  [3445] = 3445,
  [3446] = 3446,
  [3447] = 3446,
  [3448] = 3439,
  [3449] = 3386,
  [3450] = 3440,
  [3451] = 3451,
  [3452] = 3452,
  [3453] = 3453,
  [3454] = 3454,
  [3455] = 3455,
  [3456] = 3456,
  [3457] = 3455,
  [3458] = 3291,
  [3459] = 3459,
  [3460] = 3460,
  [3461] = 3461,
  [3462] = 3462,
  [3463] = 3460,
  [3464] = 3464,
  [3465] = 3465,
  [3466] = 3466,
  [3467] = 3467,
  [3468] = 3468,
  [3469] = 3461,
  [3470] = 3454,
  [3471] = 3462,
  [3472] = 3472,
  [3473] = 3466,
  [3474] = 3456,
  [3475] = 3465,
  [3476] = 3476,
  [3477] = 3441,
  [3478] = 3478,
  [3479] = 3464,
  [3480] = 3300,
  [3481] = 3445,
  [3482] = 3452,
  [3483] = 3441,
  [3484] = 3443,
  [3485] = 3442,
  [3486] = 3478,
  [3487] = 3459,
  [3488] = 3468,
  [3489] = 3489,
  [3490] = 3489,
  [3491] = 3489,
  [3492] = 3489,
  [3493] = 3489,
  [3494] = 3489,
  [3495] = 3489,
  [3496] = 3489,
  [3497] = 415,
  [3498] = 3498,
  [3499] = 3498,
  [3500] = 3500,
  [3501] = 3500,
  [3502] = 2781,
  [3503] = 2791,
  [3504] = 2789,
  [3505] = 2788,
  [3506] = 3506,
  [3507] = 2792,
  [3508] = 2792,
  [3509] = 2776,
  [3510] = 2854,
  [3511] = 2804,
  [3512] = 2789,
  [3513] = 2769,
  [3514] = 3514,
  [3515] = 2844,
  [3516] = 3516,
  [3517] = 2846,
  [3518] = 3518,
  [3519] = 2771,
  [3520] = 2766,
  [3521] = 2788,
  [3522] = 2822,
  [3523] = 2783,
  [3524] = 2791,
  [3525] = 2784,
  [3526] = 3526,
  [3527] = 3526,
  [3528] = 3528,
  [3529] = 3529,
  [3530] = 3528,
  [3531] = 3518,
  [3532] = 3532,
  [3533] = 2798,
  [3534] = 2795,
  [3535] = 2780,
  [3536] = 3536,
  [3537] = 3537,
  [3538] = 3538,
  [3539] = 2802,
  [3540] = 3538,
  [3541] = 2796,
  [3542] = 3536,
  [3543] = 3532,
  [3544] = 2799,
  [3545] = 3545,
  [3546] = 2794,
  [3547] = 3547,
  [3548] = 3548,
  [3549] = 3518,
  [3550] = 2786,
  [3551] = 2787,
  [3552] = 2790,
  [3553] = 2793,
  [3554] = 3547,
  [3555] = 2839,
  [3556] = 2809,
  [3557] = 3548,
  [3558] = 2843,
  [3559] = 3559,
  [3560] = 3560,
  [3561] = 2823,
  [3562] = 467,
  [3563] = 3563,
  [3564] = 3560,
  [3565] = 448,
  [3566] = 2805,
  [3567] = 2811,
  [3568] = 3568,
  [3569] = 3569,
  [3570] = 3570,
  [3571] = 2848,
  [3572] = 485,
  [3573] = 3573,
  [3574] = 3574,
  [3575] = 2810,
  [3576] = 3569,
  [3577] = 3569,
  [3578] = 3578,
  [3579] = 2833,
  [3580] = 462,
  [3581] = 3581,
  [3582] = 2819,
  [3583] = 2840,
  [3584] = 3574,
  [3585] = 3563,
  [3586] = 2835,
  [3587] = 2828,
  [3588] = 3581,
  [3589] = 3573,
  [3590] = 3590,
  [3591] = 3591,
  [3592] = 3592,
  [3593] = 3590,
  [3594] = 3568,
  [3595] = 2801,
  [3596] = 2825,
  [3597] = 2983,
  [3598] = 2907,
  [3599] = 2981,
  [3600] = 3600,
  [3601] = 2959,
  [3602] = 3602,
  [3603] = 2908,
  [3604] = 3604,
  [3605] = 2982,
  [3606] = 3606,
  [3607] = 3602,
  [3608] = 2978,
  [3609] = 2865,
  [3610] = 3610,
  [3611] = 3606,
  [3612] = 2958,
  [3613] = 2909,
  [3614] = 2868,
  [3615] = 3604,
  [3616] = 3616,
  [3617] = 2920,
  [3618] = 2961,
  [3619] = 2898,
  [3620] = 2970,
  [3621] = 2926,
  [3622] = 2879,
  [3623] = 2870,
  [3624] = 2927,
  [3625] = 2874,
  [3626] = 3626,
  [3627] = 2980,
  [3628] = 2923,
  [3629] = 3629,
  [3630] = 2925,
  [3631] = 2929,
  [3632] = 3632,
  [3633] = 3626,
  [3634] = 3600,
  [3635] = 2875,
  [3636] = 3632,
  [3637] = 2867,
  [3638] = 2931,
  [3639] = 2956,
  [3640] = 2977,
  [3641] = 2919,
  [3642] = 2960,
  [3643] = 2906,
  [3644] = 3606,
  [3645] = 3645,
  [3646] = 3646,
  [3647] = 3647,
  [3648] = 3648,
  [3649] = 421,
  [3650] = 3650,
  [3651] = 422,
  [3652] = 2832,
  [3653] = 2812,
  [3654] = 2859,
  [3655] = 3646,
  [3656] = 3656,
  [3657] = 3645,
  [3658] = 3658,
  [3659] = 3658,
  [3660] = 3648,
  [3661] = 441,
  [3662] = 3648,
  [3663] = 3656,
  [3664] = 3645,
  [3665] = 425,
  [3666] = 2827,
  [3667] = 3650,
  [3668] = 3668,
  [3669] = 3669,
  [3670] = 3656,
  [3671] = 3668,
  [3672] = 3672,
  [3673] = 3673,
  [3674] = 2788,
  [3675] = 2791,
  [3676] = 2791,
  [3677] = 3677,
  [3678] = 3678,
  [3679] = 2792,
  [3680] = 3680,
  [3681] = 2789,
  [3682] = 3682,
  [3683] = 2788,
  [3684] = 2792,
  [3685] = 2789,
  [3686] = 2791,
  [3687] = 3687,
  [3688] = 3688,
  [3689] = 2792,
  [3690] = 3690,
  [3691] = 3506,
  [3692] = 2789,
  [3693] = 2788,
  [3694] = 3694,
  [3695] = 3695,
  [3696] = 3694,
  [3697] = 3694,
  [3698] = 3694,
  [3699] = 3694,
  [3700] = 3700,
  [3701] = 3701,
  [3702] = 3702,
  [3703] = 3703,
  [3704] = 3704,
  [3705] = 3705,
  [3706] = 3706,
  [3707] = 2769,
  [3708] = 3708,
  [3709] = 3709,
  [3710] = 2771,
  [3711] = 420,
  [3712] = 429,
  [3713] = 442,
  [3714] = 3714,
  [3715] = 3715,
  [3716] = 3716,
  [3717] = 3717,
  [3718] = 426,
  [3719] = 444,
  [3720] = 3720,
  [3721] = 3721,
  [3722] = 3722,
  [3723] = 3723,
  [3724] = 3724,
  [3725] = 3725,
  [3726] = 3726,
  [3727] = 3727,
  [3728] = 3728,
  [3729] = 3729,
  [3730] = 2791,
  [3731] = 2788,
  [3732] = 2789,
  [3733] = 2792,
  [3734] = 3734,
  [3735] = 2766,
  [3736] = 3734,
  [3737] = 3734,
  [3738] = 3738,
  [3739] = 3739,
  [3740] = 3740,
  [3741] = 3741,
  [3742] = 3545,
  [3743] = 3743,
  [3744] = 3704,
  [3745] = 3739,
  [3746] = 3703,
  [3747] = 3705,
  [3748] = 3708,
  [3749] = 3738,
  [3750] = 3740,
  [3751] = 3741,
  [3752] = 3721,
  [3753] = 3743,
  [3754] = 3754,
  [3755] = 3755,
  [3756] = 3756,
  [3757] = 3757,
  [3758] = 3758,
  [3759] = 3759,
  [3760] = 3760,
  [3761] = 3761,
  [3762] = 3762,
  [3763] = 3763,
  [3764] = 3764,
  [3765] = 3765,
  [3766] = 3766,
  [3767] = 3767,
  [3768] = 3768,
  [3769] = 3769,
  [3770] = 3770,
  [3771] = 3771,
  [3772] = 3772,
  [3773] = 3773,
  [3774] = 3774,
  [3775] = 3775,
  [3776] = 3776,
  [3777] = 3777,
  [3778] = 3778,
  [3779] = 3779,
  [3780] = 3545,
  [3781] = 3781,
  [3782] = 3782,
  [3783] = 3783,
  [3784] = 3784,
  [3785] = 3785,
  [3786] = 3786,
  [3787] = 2780,
  [3788] = 3677,
  [3789] = 3688,
  [3790] = 3775,
  [3791] = 3791,
  [3792] = 3782,
  [3793] = 3761,
  [3794] = 3778,
  [3795] = 3795,
  [3796] = 3796,
  [3797] = 3797,
  [3798] = 3798,
  [3799] = 3799,
  [3800] = 3800,
  [3801] = 3592,
  [3802] = 3545,
  [3803] = 3803,
  [3804] = 471,
  [3805] = 3791,
  [3806] = 3806,
  [3807] = 3807,
  [3808] = 3775,
  [3809] = 3809,
  [3810] = 3810,
  [3811] = 3811,
  [3812] = 3812,
  [3813] = 3545,
  [3814] = 3814,
  [3815] = 3815,
  [3816] = 3816,
  [3817] = 3817,
  [3818] = 3818,
  [3819] = 3819,
  [3820] = 3820,
  [3821] = 3821,
  [3822] = 3822,
  [3823] = 3823,
  [3824] = 2790,
  [3825] = 3825,
  [3826] = 3826,
  [3827] = 2798,
  [3828] = 3828,
  [3829] = 3829,
  [3830] = 2787,
  [3831] = 3831,
  [3832] = 3832,
  [3833] = 3833,
  [3834] = 2793,
  [3835] = 3835,
  [3836] = 3836,
  [3837] = 3837,
  [3838] = 3838,
  [3839] = 3839,
  [3840] = 3840,
  [3841] = 3841,
  [3842] = 3818,
  [3843] = 3843,
  [3844] = 3814,
  [3845] = 3845,
  [3846] = 3846,
  [3847] = 3847,
  [3848] = 3841,
  [3849] = 3815,
  [3850] = 3821,
  [3851] = 3851,
  [3852] = 2786,
  [3853] = 3853,
  [3854] = 3854,
  [3855] = 3855,
  [3856] = 3856,
  [3857] = 3857,
  [3858] = 2799,
  [3859] = 2795,
  [3860] = 3860,
  [3861] = 3825,
  [3862] = 3862,
  [3863] = 3592,
  [3864] = 3592,
  [3865] = 3816,
  [3866] = 3825,
  [3867] = 2796,
  [3868] = 3835,
  [3869] = 3817,
  [3870] = 3820,
  [3871] = 3854,
  [3872] = 3872,
  [3873] = 3872,
  [3874] = 3823,
  [3875] = 3840,
  [3876] = 3819,
  [3877] = 3877,
  [3878] = 3846,
  [3879] = 3838,
  [3880] = 3853,
  [3881] = 3833,
  [3882] = 3862,
  [3883] = 3883,
  [3884] = 3856,
  [3885] = 3825,
  [3886] = 3825,
  [3887] = 3877,
  [3888] = 3857,
  [3889] = 3889,
  [3890] = 3831,
  [3891] = 3843,
  [3892] = 3826,
  [3893] = 3893,
  [3894] = 3894,
  [3895] = 3895,
  [3896] = 3896,
  [3897] = 3897,
  [3898] = 3898,
  [3899] = 3899,
  [3900] = 3900,
  [3901] = 3901,
  [3902] = 3610,
  [3903] = 3903,
  [3904] = 3904,
  [3905] = 3905,
  [3906] = 3906,
  [3907] = 3901,
  [3908] = 3908,
  [3909] = 3909,
  [3910] = 3910,
  [3911] = 3894,
  [3912] = 3912,
  [3913] = 3895,
  [3914] = 3914,
  [3915] = 3915,
  [3916] = 3916,
  [3917] = 3917,
  [3918] = 3918,
  [3919] = 3919,
  [3920] = 3897,
  [3921] = 3908,
  [3922] = 3903,
  [3923] = 3912,
  [3924] = 3924,
  [3925] = 3925,
  [3926] = 3926,
  [3927] = 3927,
  [3928] = 3928,
  [3929] = 3929,
  [3930] = 3930,
  [3931] = 3915,
  [3932] = 3932,
  [3933] = 3933,
  [3934] = 3934,
  [3935] = 3935,
  [3936] = 3905,
  [3937] = 3916,
  [3938] = 3929,
  [3939] = 3939,
  [3940] = 3940,
  [3941] = 3941,
  [3942] = 3592,
  [3943] = 3677,
  [3944] = 3924,
  [3945] = 3688,
  [3946] = 3904,
  [3947] = 3914,
  [3948] = 3948,
  [3949] = 3925,
  [3950] = 3909,
  [3951] = 3926,
  [3952] = 3927,
  [3953] = 3932,
  [3954] = 3928,
  [3955] = 3933,
  [3956] = 3906,
  [3957] = 3939,
  [3958] = 3948,
  [3959] = 3910,
  [3960] = 3930,
  [3961] = 3961,
  [3962] = 3919,
  [3963] = 3963,
  [3964] = 3917,
  [3965] = 3965,
  [3966] = 3965,
  [3967] = 3677,
  [3968] = 3688,
  [3969] = 3918,
  [3970] = 3961,
  [3971] = 3963,
  [3972] = 3972,
  [3973] = 3973,
  [3974] = 3974,
  [3975] = 3975,
  [3976] = 3976,
  [3977] = 3977,
  [3978] = 3978,
  [3979] = 3979,
  [3980] = 3980,
  [3981] = 3981,
  [3982] = 3973,
  [3983] = 3975,
  [3984] = 3984,
  [3985] = 3985,
  [3986] = 3986,
  [3987] = 3987,
  [3988] = 3974,
  [3989] = 3975,
  [3990] = 3990,
  [3991] = 3991,
  [3992] = 3992,
  [3993] = 3993,
  [3994] = 3994,
  [3995] = 3995,
  [3996] = 3996,
  [3997] = 3997,
  [3998] = 3998,
  [3999] = 3999,
  [4000] = 4000,
  [4001] = 4001,
  [4002] = 4002,
  [4003] = 4003,
  [4004] = 4003,
  [4005] = 4005,
  [4006] = 4006,
  [4007] = 3984,
  [4008] = 4008,
  [4009] = 3976,
  [4010] = 3978,
  [4011] = 3979,
  [4012] = 4002,
  [4013] = 4013,
  [4014] = 4014,
  [4015] = 4013,
  [4016] = 3977,
  [4017] = 3980,
  [4018] = 4018,
  [4019] = 4019,
  [4020] = 4020,
  [4021] = 4021,
  [4022] = 4022,
  [4023] = 3981,
  [4024] = 4024,
  [4025] = 3987,
  [4026] = 4001,
  [4027] = 4024,
  [4028] = 4028,
  [4029] = 3994,
  [4030] = 3996,
  [4031] = 3997,
  [4032] = 3999,
  [4033] = 3975,
  [4034] = 4028,
  [4035] = 4035,
  [4036] = 3995,
  [4037] = 4037,
  [4038] = 3998,
  [4039] = 4039,
  [4040] = 4040,
  [4041] = 4041,
  [4042] = 3975,
  [4043] = 4043,
  [4044] = 4044,
  [4045] = 4045,
  [4046] = 4014,
  [4047] = 4022,
  [4048] = 4048,
  [4049] = 4040,
  [4050] = 4041,
  [4051] = 4051,
  [4052] = 4037,
  [4053] = 4018,
  [4054] = 4021,
  [4055] = 4055,
  [4056] = 3985,
  [4057] = 4057,
  [4058] = 3972,
  [4059] = 4055,
  [4060] = 4019,
  [4061] = 4035,
  [4062] = 4005,
  [4063] = 4020,
  [4064] = 4064,
  [4065] = 4008,
  [4066] = 4066,
  [4067] = 4014,
  [4068] = 3990,
  [4069] = 4069,
  [4070] = 3986,
  [4071] = 4071,
  [4072] = 4072,
  [4073] = 4073,
  [4074] = 4074,
  [4075] = 4075,
  [4076] = 4076,
  [4077] = 4077,
  [4078] = 4078,
  [4079] = 4079,
  [4080] = 4080,
  [4081] = 4081,
  [4082] = 4082,
  [4083] = 4083,
  [4084] = 4084,
  [4085] = 4085,
  [4086] = 422,
  [4087] = 4074,
  [4088] = 4088,
  [4089] = 4089,
  [4090] = 4090,
  [4091] = 4091,
  [4092] = 4091,
  [4093] = 4093,
  [4094] = 4094,
  [4095] = 4095,
  [4096] = 4096,
  [4097] = 4097,
  [4098] = 4077,
  [4099] = 4093,
  [4100] = 4100,
  [4101] = 4088,
  [4102] = 4078,
  [4103] = 4103,
  [4104] = 3672,
  [4105] = 4080,
  [4106] = 4106,
  [4107] = 4107,
  [4108] = 4094,
  [4109] = 4109,
  [4110] = 4094,
  [4111] = 4079,
  [4112] = 2812,
  [4113] = 4077,
  [4114] = 4088,
  [4115] = 4115,
  [4116] = 4116,
  [4117] = 4094,
  [4118] = 4118,
  [4119] = 4094,
  [4120] = 2859,
  [4121] = 4121,
  [4122] = 4081,
  [4123] = 4123,
  [4124] = 4124,
  [4125] = 3669,
  [4126] = 4124,
  [4127] = 4127,
  [4128] = 3678,
  [4129] = 4094,
  [4130] = 4130,
  [4131] = 4131,
  [4132] = 421,
  [4133] = 4133,
  [4134] = 4116,
  [4135] = 4135,
  [4136] = 4136,
  [4137] = 4137,
  [4138] = 4076,
  [4139] = 4095,
  [4140] = 4140,
  [4141] = 4141,
  [4142] = 4142,
  [4143] = 4143,
  [4144] = 4144,
  [4145] = 4145,
  [4146] = 4146,
  [4147] = 4118,
  [4148] = 441,
  [4149] = 4149,
  [4150] = 4150,
  [4151] = 4082,
  [4152] = 4152,
  [4153] = 4153,
  [4154] = 4154,
  [4155] = 4083,
  [4156] = 4153,
  [4157] = 4157,
  [4158] = 4097,
  [4159] = 4159,
  [4160] = 4100,
  [4161] = 4161,
  [4162] = 2827,
  [4163] = 4163,
  [4164] = 4164,
  [4165] = 4165,
  [4166] = 4166,
  [4167] = 4084,
  [4168] = 4085,
  [4169] = 4169,
  [4170] = 4135,
  [4171] = 4171,
  [4172] = 4172,
  [4173] = 4154,
  [4174] = 4163,
  [4175] = 4175,
  [4176] = 4176,
  [4177] = 4150,
  [4178] = 4178,
  [4179] = 4179,
  [4180] = 4180,
  [4181] = 4181,
  [4182] = 4182,
  [4183] = 425,
  [4184] = 2832,
  [4185] = 4165,
  [4186] = 4178,
  [4187] = 4103,
  [4188] = 4188,
  [4189] = 4189,
  [4190] = 4172,
  [4191] = 4191,
  [4192] = 4166,
  [4193] = 4166,
  [4194] = 4180,
  [4195] = 4181,
  [4196] = 4121,
  [4197] = 4123,
  [4198] = 4136,
  [4199] = 4106,
  [4200] = 4152,
  [4201] = 4073,
  [4202] = 4089,
  [4203] = 4090,
  [4204] = 4127,
  [4205] = 4107,
  [4206] = 4206,
  [4207] = 4077,
  [4208] = 4140,
  [4209] = 4073,
  [4210] = 4210,
  [4211] = 4142,
  [4212] = 4152,
  [4213] = 4144,
  [4214] = 4145,
  [4215] = 4088,
  [4216] = 4109,
  [4217] = 4141,
  [4218] = 4096,
  [4219] = 4219,
  [4220] = 4220,
  [4221] = 4115,
  [4222] = 4157,
  [4223] = 4149,
  [4224] = 4224,
  [4225] = 4225,
  [4226] = 4161,
  [4227] = 4227,
  [4228] = 4228,
  [4229] = 4191,
  [4230] = 4220,
  [4231] = 4231,
  [4232] = 4219,
  [4233] = 4233,
  [4234] = 4234,
  [4235] = 4171,
  [4236] = 4133,
  [4237] = 4143,
  [4238] = 4146,
  [4239] = 4239,
  [4240] = 4159,
  [4241] = 4175,
  [4242] = 4242,
  [4243] = 4188,
  [4244] = 4182,
  [4245] = 4227,
  [4246] = 4206,
  [4247] = 4234,
  [4248] = 2854,
  [4249] = 4228,
  [4250] = 4071,
  [4251] = 4239,
  [4252] = 2780,
  [4253] = 4164,
  [4254] = 2804,
  [4255] = 4255,
  [4256] = 4242,
  [4257] = 2822,
  [4258] = 4225,
  [4259] = 2846,
  [4260] = 4260,
  [4261] = 4231,
  [4262] = 4233,
  [4263] = 4072,
  [4264] = 2844,
  [4265] = 4189,
  [4266] = 4266,
  [4267] = 4267,
  [4268] = 4268,
  [4269] = 4269,
  [4270] = 4270,
  [4271] = 4271,
  [4272] = 4272,
  [4273] = 2757,
  [4274] = 4274,
  [4275] = 4275,
  [4276] = 4276,
  [4277] = 4277,
  [4278] = 4278,
  [4279] = 4279,
  [4280] = 4280,
  [4281] = 4276,
  [4282] = 4282,
  [4283] = 4283,
  [4284] = 4284,
  [4285] = 4285,
  [4286] = 4286,
  [4287] = 4287,
  [4288] = 4286,
  [4289] = 4289,
  [4290] = 4137,
  [4291] = 4291,
  [4292] = 4289,
  [4293] = 4293,
  [4294] = 4294,
  [4295] = 4295,
  [4296] = 4296,
  [4297] = 4297,
  [4298] = 4298,
  [4299] = 4299,
  [4300] = 4179,
  [4301] = 4301,
  [4302] = 4284,
  [4303] = 4303,
  [4304] = 4304,
  [4305] = 4305,
  [4306] = 4306,
  [4307] = 4307,
  [4308] = 4308,
  [4309] = 4309,
  [4310] = 4310,
  [4311] = 4311,
  [4312] = 4291,
  [4313] = 4313,
  [4314] = 4131,
  [4315] = 4303,
  [4316] = 4304,
  [4317] = 4317,
  [4318] = 2762,
  [4319] = 4319,
  [4320] = 4320,
  [4321] = 4321,
  [4322] = 4322,
  [4323] = 4323,
  [4324] = 4324,
  [4325] = 4325,
  [4326] = 4326,
  [4327] = 4327,
  [4328] = 4328,
  [4329] = 4329,
  [4330] = 4330,
  [4331] = 4331,
  [4332] = 4332,
  [4333] = 4266,
  [4334] = 4332,
  [4335] = 4335,
  [4336] = 4336,
  [4337] = 4337,
  [4338] = 4338,
  [4339] = 4339,
  [4340] = 4306,
  [4341] = 4282,
  [4342] = 424,
  [4343] = 4343,
  [4344] = 4305,
  [4345] = 4309,
  [4346] = 4311,
  [4347] = 4313,
  [4348] = 4307,
  [4349] = 4323,
  [4350] = 4350,
  [4351] = 4351,
  [4352] = 4352,
  [4353] = 4353,
  [4354] = 4317,
  [4355] = 4355,
  [4356] = 4356,
  [4357] = 4278,
  [4358] = 4301,
  [4359] = 4359,
  [4360] = 4278,
  [4361] = 4361,
  [4362] = 4285,
  [4363] = 4287,
  [4364] = 4364,
  [4365] = 4365,
  [4366] = 4366,
  [4367] = 4367,
  [4368] = 4368,
  [4369] = 4364,
  [4370] = 4287,
  [4371] = 4343,
  [4372] = 4283,
  [4373] = 4285,
  [4374] = 4374,
  [4375] = 4375,
  [4376] = 2758,
  [4377] = 4319,
  [4378] = 4378,
  [4379] = 4379,
  [4380] = 4335,
  [4381] = 4381,
  [4382] = 4336,
  [4383] = 4383,
  [4384] = 4285,
  [4385] = 4350,
  [4386] = 4320,
  [4387] = 4387,
  [4388] = 4321,
  [4389] = 4308,
  [4390] = 4364,
  [4391] = 4293,
  [4392] = 4337,
  [4393] = 4393,
  [4394] = 4394,
  [4395] = 4395,
  [4396] = 2759,
  [4397] = 2760,
  [4398] = 4379,
  [4399] = 2763,
  [4400] = 4351,
  [4401] = 1946,
  [4402] = 4322,
  [4403] = 4393,
  [4404] = 4352,
  [4405] = 4405,
  [4406] = 4324,
  [4407] = 4407,
  [4408] = 4408,
  [4409] = 4338,
  [4410] = 4325,
  [4411] = 432,
  [4412] = 4339,
  [4413] = 4413,
  [4414] = 4414,
  [4415] = 4294,
  [4416] = 4280,
  [4417] = 4326,
  [4418] = 2756,
  [4419] = 2761,
  [4420] = 2755,
  [4421] = 4327,
  [4422] = 4295,
  [4423] = 4310,
  [4424] = 4379,
  [4425] = 4353,
  [4426] = 4394,
  [4427] = 4298,
  [4428] = 4428,
  [4429] = 4299,
  [4430] = 4430,
  [4431] = 4331,
  [4432] = 4432,
  [4433] = 4287,
  [4434] = 4274,
  [4435] = 2793,
  [4436] = 4328,
  [4437] = 4343,
  [4438] = 4438,
  [4439] = 4439,
  [4440] = 4440,
  [4441] = 4441,
  [4442] = 4442,
  [4443] = 4443,
  [4444] = 4444,
  [4445] = 4445,
  [4446] = 4446,
  [4447] = 4447,
  [4448] = 4448,
  [4449] = 4449,
  [4450] = 4450,
  [4451] = 4451,
  [4452] = 4451,
  [4453] = 4453,
  [4454] = 4454,
  [4455] = 4455,
  [4456] = 4456,
  [4457] = 4457,
  [4458] = 4458,
  [4459] = 4459,
  [4460] = 4460,
  [4461] = 4461,
  [4462] = 4462,
  [4463] = 4463,
  [4464] = 4464,
  [4465] = 4465,
  [4466] = 4466,
  [4467] = 4467,
  [4468] = 4468,
  [4469] = 4469,
  [4470] = 4470,
  [4471] = 4471,
  [4472] = 4472,
  [4473] = 4439,
  [4474] = 4474,
  [4475] = 4475,
  [4476] = 4476,
  [4477] = 4477,
  [4478] = 4478,
  [4479] = 4479,
  [4480] = 4441,
  [4481] = 4481,
  [4482] = 4482,
  [4483] = 4483,
  [4484] = 4484,
  [4485] = 4485,
  [4486] = 4486,
  [4487] = 4487,
  [4488] = 4488,
  [4489] = 4489,
  [4490] = 4490,
  [4491] = 4491,
  [4492] = 4492,
  [4493] = 4493,
  [4494] = 4494,
  [4495] = 4495,
  [4496] = 4496,
  [4497] = 4457,
  [4498] = 4498,
  [4499] = 4499,
  [4500] = 4500,
  [4501] = 4501,
  [4502] = 4502,
  [4503] = 4503,
  [4504] = 4504,
  [4505] = 4491,
  [4506] = 4506,
  [4507] = 4507,
  [4508] = 4508,
  [4509] = 4509,
  [4510] = 4510,
  [4511] = 4506,
  [4512] = 4475,
  [4513] = 4513,
  [4514] = 4514,
  [4515] = 4515,
  [4516] = 4516,
  [4517] = 4517,
  [4518] = 4448,
  [4519] = 4519,
  [4520] = 4449,
  [4521] = 4521,
  [4522] = 4522,
  [4523] = 4523,
  [4524] = 4507,
  [4525] = 4525,
  [4526] = 4526,
  [4527] = 4527,
  [4528] = 4528,
  [4529] = 4529,
  [4530] = 4530,
  [4531] = 4531,
  [4532] = 4532,
  [4533] = 4463,
  [4534] = 4534,
  [4535] = 4535,
  [4536] = 4536,
  [4537] = 4537,
  [4538] = 4443,
  [4539] = 4446,
  [4540] = 4540,
  [4541] = 4513,
  [4542] = 4542,
  [4543] = 4543,
  [4544] = 4525,
  [4545] = 4439,
  [4546] = 4487,
  [4547] = 4488,
  [4548] = 4548,
  [4549] = 4549,
  [4550] = 4535,
  [4551] = 4551,
  [4552] = 4465,
  [4553] = 4446,
  [4554] = 4513,
  [4555] = 4548,
  [4556] = 4556,
  [4557] = 4557,
  [4558] = 4558,
  [4559] = 4559,
  [4560] = 4560,
  [4561] = 4561,
  [4562] = 4562,
  [4563] = 4563,
  [4564] = 4485,
  [4565] = 4565,
  [4566] = 4521,
  [4567] = 4443,
  [4568] = 4543,
  [4569] = 4569,
  [4570] = 4570,
  [4571] = 4453,
  [4572] = 4454,
  [4573] = 4514,
  [4574] = 3714,
  [4575] = 3715,
  [4576] = 4576,
  [4577] = 3716,
  [4578] = 3717,
  [4579] = 4579,
  [4580] = 4536,
  [4581] = 4581,
  [4582] = 4582,
  [4583] = 4526,
  [4584] = 4530,
  [4585] = 4492,
  [4586] = 4586,
  [4587] = 3722,
  [4588] = 3723,
  [4589] = 3724,
  [4590] = 4590,
  [4591] = 4591,
  [4592] = 3725,
  [4593] = 4593,
  [4594] = 4508,
  [4595] = 4500,
  [4596] = 4448,
  [4597] = 4449,
  [4598] = 4598,
  [4599] = 4599,
  [4600] = 4600,
  [4601] = 4543,
  [4602] = 4525,
  [4603] = 4439,
  [4604] = 4487,
  [4605] = 4562,
  [4606] = 4439,
  [4607] = 4446,
  [4608] = 4513,
  [4609] = 4548,
  [4610] = 4509,
  [4611] = 4558,
  [4612] = 4559,
  [4613] = 4560,
  [4614] = 4474,
  [4615] = 4563,
  [4616] = 4558,
  [4617] = 4565,
  [4618] = 4521,
  [4619] = 4453,
  [4620] = 4454,
  [4621] = 4621,
  [4622] = 4514,
  [4623] = 4453,
  [4624] = 4624,
  [4625] = 4625,
  [4626] = 4510,
  [4627] = 4492,
  [4628] = 4459,
  [4629] = 4460,
  [4630] = 4525,
  [4631] = 4631,
  [4632] = 4632,
  [4633] = 4487,
  [4634] = 4634,
  [4635] = 4446,
  [4636] = 4559,
  [4637] = 4560,
  [4638] = 4638,
  [4639] = 4639,
  [4640] = 4461,
  [4641] = 4565,
  [4642] = 4521,
  [4643] = 4453,
  [4644] = 4454,
  [4645] = 4559,
  [4646] = 4525,
  [4647] = 4446,
  [4648] = 4559,
  [4649] = 4560,
  [4650] = 4454,
  [4651] = 4487,
  [4652] = 4453,
  [4653] = 4454,
  [4654] = 4654,
  [4655] = 4655,
  [4656] = 4656,
  [4657] = 4453,
  [4658] = 4454,
  [4659] = 4659,
  [4660] = 4660,
  [4661] = 4523,
  [4662] = 4453,
  [4663] = 4454,
  [4664] = 4664,
  [4665] = 4453,
  [4666] = 4454,
  [4667] = 4560,
  [4668] = 4557,
  [4669] = 4534,
  [4670] = 4504,
  [4671] = 4671,
  [4672] = 4672,
  [4673] = 4638,
  [4674] = 4486,
  [4675] = 4675,
  [4676] = 4492,
  [4677] = 4543,
  [4678] = 4488,
  [4679] = 4679,
  [4680] = 4672,
  [4681] = 3702,
  [4682] = 3727,
  [4683] = 4526,
  [4684] = 4569,
  [4685] = 4559,
  [4686] = 4686,
  [4687] = 4687,
  [4688] = 4537,
  [4689] = 4494,
  [4690] = 4690,
  [4691] = 4462,
  [4692] = 4530,
  [4693] = 4693,
  [4694] = 4468,
  [4695] = 4548,
  [4696] = 3728,
  [4697] = 4697,
  [4698] = 4531,
  [4699] = 4576,
  [4700] = 3729,
  [4701] = 4701,
  [4702] = 4702,
  [4703] = 4703,
  [4704] = 4704,
  [4705] = 4579,
  [4706] = 4706,
  [4707] = 4593,
  [4708] = 4708,
  [4709] = 4560,
  [4710] = 4501,
  [4711] = 4561,
  [4712] = 4600,
  [4713] = 4590,
  [4714] = 4591,
  [4715] = 4502,
  [4716] = 4561,
  [4717] = 4717,
  [4718] = 4718,
  [4719] = 4719,
  [4720] = 4720,
  [4721] = 4515,
  [4722] = 4516,
  [4723] = 4517,
  [4724] = 4634,
  [4725] = 4469,
  [4726] = 4470,
  [4727] = 4471,
  [4728] = 4503,
  [4729] = 4519,
  [4730] = 4472,
  [4731] = 4731,
  [4732] = 4732,
  [4733] = 4733,
  [4734] = 4734,
  [4735] = 4498,
  [4736] = 4736,
  [4737] = 4737,
  [4738] = 4624,
  [4739] = 4563,
  [4740] = 4740,
  [4741] = 4489,
  [4742] = 4742,
  [4743] = 4493,
  [4744] = 4499,
  [4745] = 4476,
  [4746] = 4746,
  [4747] = 4534,
  [4748] = 4599,
  [4749] = 4477,
  [4750] = 4478,
  [4751] = 4479,
  [4752] = 4440,
  [4753] = 4753,
  [4754] = 4590,
  [4755] = 4704,
  [4756] = 4706,
  [4757] = 4444,
  [4758] = 4758,
  [4759] = 4450,
  [4760] = 4675,
  [4761] = 4761,
  [4762] = 4762,
  [4763] = 4763,
  [4764] = 4764,
  [4765] = 4514,
  [4766] = 4525,
  [4767] = 4586,
  [4768] = 4740,
  [4769] = 4481,
  [4770] = 4482,
  [4771] = 4483,
  [4772] = 4718,
  [4773] = 4445,
  [4774] = 4458,
  [4775] = 4775,
  [4776] = 4484,
  [4777] = 4582,
  [4778] = 4591,
  [4779] = 4527,
  [4780] = 4780,
  [4781] = 4528,
  [4782] = 4490,
  [4783] = 4529,
  [4784] = 4780,
  [4785] = 4687,
  [4786] = 4786,
  [4787] = 4631,
  [4788] = 4788,
  [4789] = 4598,
  [4790] = 4790,
  [4791] = 4621,
  [4792] = 4792,
  [4793] = 4625,
  [4794] = 4565,
  [4795] = 4795,
  [4796] = 4581,
  [4797] = 4632,
  [4798] = 4535,
  [4799] = 4737,
  [4800] = 4535,
  [4801] = 4801,
  [4802] = 4534,
  [4803] = 4532,
  [4804] = 415,
  [4805] = 4672,
  [4806] = 4535,
  [4807] = 4679,
  [4808] = 4719,
  [4809] = 4809,
  [4810] = 4534,
  [4811] = 4697,
  [4812] = 4672,
  [4813] = 4639,
  [4814] = 4702,
  [4815] = 4815,
  [4816] = 4816,
  [4817] = 4817,
  [4818] = 4818,
  [4819] = 4708,
  [4820] = 4742,
  [4821] = 4731,
  [4822] = 4746,
  [4823] = 4758,
  [4824] = 4824,
  [4825] = 4761,
  [4826] = 4826,
  [4827] = 4764,
  [4828] = 4495,
  [4829] = 4540,
  [4830] = 4549,
  [4831] = 4753,
  [4832] = 4832,
  [4833] = 4693,
  [4834] = 4466,
  [4835] = 4455,
  [4836] = 4464,
  [4837] = 4542,
  [4838] = 4570,
  [4839] = 4521,
  [4840] = 4655,
  [4841] = 4686,
  [4842] = 4733,
  [4843] = 4734,
  [4844] = 4467,
  [4845] = 4720,
  [4846] = 4565,
  [4847] = 4809,
  [4848] = 4815,
  [4849] = 4660,
  [4850] = 4536,
  [4851] = 4786,
  [4852] = 4852,
  [4853] = 4816,
  [4854] = 4817,
  [4855] = 4664,
  [4856] = 4717,
  [4857] = 4857,
  [4858] = 4558,
  [4859] = 3760,
  [4860] = 4860,
  [4861] = 4861,
  [4862] = 4862,
  [4863] = 4863,
  [4864] = 4864,
  [4865] = 4865,
  [4866] = 4866,
  [4867] = 4867,
  [4868] = 4868,
  [4869] = 4869,
  [4870] = 4870,
  [4871] = 4871,
  [4872] = 4872,
  [4873] = 4873,
  [4874] = 4874,
  [4875] = 4874,
  [4876] = 4876,
  [4877] = 4877,
  [4878] = 4878,
  [4879] = 4879,
  [4880] = 4880,
  [4881] = 4881,
  [4882] = 4882,
  [4883] = 4883,
  [4884] = 4884,
  [4885] = 4885,
  [4886] = 4886,
  [4887] = 4887,
  [4888] = 3797,
  [4889] = 4889,
  [4890] = 4890,
  [4891] = 4891,
  [4892] = 4892,
  [4893] = 4893,
  [4894] = 4894,
  [4895] = 4895,
  [4896] = 4896,
  [4897] = 4897,
  [4898] = 4898,
  [4899] = 4884,
  [4900] = 4900,
  [4901] = 4901,
  [4902] = 4902,
  [4903] = 4903,
  [4904] = 4904,
  [4905] = 4905,
  [4906] = 3809,
  [4907] = 4907,
  [4908] = 4908,
  [4909] = 4909,
  [4910] = 4910,
  [4911] = 4911,
  [4912] = 4912,
  [4913] = 4913,
  [4914] = 3755,
  [4915] = 4915,
  [4916] = 4916,
  [4917] = 4917,
  [4918] = 4918,
  [4919] = 4919,
  [4920] = 4920,
  [4921] = 4873,
  [4922] = 4922,
  [4923] = 3756,
  [4924] = 3757,
  [4925] = 4925,
  [4926] = 4926,
  [4927] = 4927,
  [4928] = 4928,
  [4929] = 4929,
  [4930] = 3758,
  [4931] = 4931,
  [4932] = 4932,
  [4933] = 3759,
  [4934] = 4934,
  [4935] = 4935,
  [4936] = 4936,
  [4937] = 4937,
  [4938] = 4938,
  [4939] = 3798,
  [4940] = 4940,
  [4941] = 4941,
  [4942] = 4942,
  [4943] = 442,
  [4944] = 4860,
  [4945] = 4945,
  [4946] = 4898,
  [4947] = 3762,
  [4948] = 4901,
  [4949] = 4949,
  [4950] = 4950,
  [4951] = 4951,
  [4952] = 4952,
  [4953] = 3765,
  [4954] = 4879,
  [4955] = 4955,
  [4956] = 4956,
  [4957] = 3766,
  [4958] = 4896,
  [4959] = 4959,
  [4960] = 4960,
  [4961] = 3769,
  [4962] = 4962,
  [4963] = 4963,
  [4964] = 4964,
  [4965] = 4945,
  [4966] = 4912,
  [4967] = 4967,
  [4968] = 4968,
  [4969] = 4927,
  [4970] = 4970,
  [4971] = 4902,
  [4972] = 4972,
  [4973] = 4973,
  [4974] = 4916,
  [4975] = 4951,
  [4976] = 4874,
  [4977] = 4977,
  [4978] = 4978,
  [4979] = 4979,
  [4980] = 4980,
  [4981] = 4919,
  [4982] = 4982,
  [4983] = 4920,
  [4984] = 4873,
  [4985] = 4884,
  [4986] = 4891,
  [4987] = 4987,
  [4988] = 4988,
  [4989] = 4989,
  [4990] = 4891,
  [4991] = 4991,
  [4992] = 4992,
  [4993] = 4993,
  [4994] = 4994,
  [4995] = 4995,
  [4996] = 4931,
  [4997] = 4932,
  [4998] = 4998,
  [4999] = 4999,
  [5000] = 4951,
  [5001] = 5001,
  [5002] = 5002,
  [5003] = 5003,
  [5004] = 5004,
  [5005] = 5005,
  [5006] = 5006,
  [5007] = 776,
  [5008] = 5008,
  [5009] = 4951,
  [5010] = 4874,
  [5011] = 5011,
  [5012] = 4992,
  [5013] = 4900,
  [5014] = 5014,
  [5015] = 3768,
  [5016] = 5016,
  [5017] = 4905,
  [5018] = 3795,
  [5019] = 4879,
  [5020] = 426,
  [5021] = 5021,
  [5022] = 5022,
  [5023] = 5023,
  [5024] = 5024,
  [5025] = 4876,
  [5026] = 3770,
  [5027] = 4887,
  [5028] = 3786,
  [5029] = 5029,
  [5030] = 5014,
  [5031] = 1095,
  [5032] = 4918,
  [5033] = 5001,
  [5034] = 5034,
  [5035] = 4911,
  [5036] = 5036,
  [5037] = 4884,
  [5038] = 5038,
  [5039] = 4955,
  [5040] = 5040,
  [5041] = 5041,
  [5042] = 5042,
  [5043] = 5043,
  [5044] = 5044,
  [5045] = 5045,
  [5046] = 5022,
  [5047] = 5047,
  [5048] = 5048,
  [5049] = 4926,
  [5050] = 5050,
  [5051] = 5051,
  [5052] = 5034,
  [5053] = 4926,
  [5054] = 4862,
  [5055] = 5055,
  [5056] = 5056,
  [5057] = 5057,
  [5058] = 4896,
  [5059] = 5059,
  [5060] = 4936,
  [5061] = 4956,
  [5062] = 4959,
  [5063] = 5063,
  [5064] = 5056,
  [5065] = 436,
  [5066] = 5063,
  [5067] = 5067,
  [5068] = 5068,
  [5069] = 5069,
  [5070] = 5070,
  [5071] = 5071,
  [5072] = 5023,
  [5073] = 5073,
  [5074] = 5029,
  [5075] = 5075,
  [5076] = 2794,
  [5077] = 5077,
  [5078] = 5078,
  [5079] = 4967,
  [5080] = 5080,
  [5081] = 5081,
  [5082] = 5082,
  [5083] = 5083,
  [5084] = 5084,
  [5085] = 4935,
  [5086] = 4960,
  [5087] = 5068,
  [5088] = 5088,
  [5089] = 5069,
  [5090] = 5090,
  [5091] = 4962,
  [5092] = 5075,
  [5093] = 3771,
  [5094] = 4874,
  [5095] = 5095,
  [5096] = 3785,
  [5097] = 5097,
  [5098] = 5098,
  [5099] = 3803,
  [5100] = 3806,
  [5101] = 3810,
  [5102] = 3811,
  [5103] = 5103,
  [5104] = 5097,
  [5105] = 5070,
  [5106] = 5098,
  [5107] = 5107,
  [5108] = 5108,
  [5109] = 5108,
  [5110] = 5110,
  [5111] = 4880,
  [5112] = 5112,
  [5113] = 5113,
  [5114] = 4882,
  [5115] = 5059,
  [5116] = 4917,
  [5117] = 5117,
  [5118] = 5118,
  [5119] = 5119,
  [5120] = 5112,
  [5121] = 5121,
  [5122] = 3807,
  [5123] = 5123,
  [5124] = 4995,
  [5125] = 5125,
  [5126] = 5126,
  [5127] = 5127,
  [5128] = 5128,
  [5129] = 5129,
  [5130] = 5130,
  [5131] = 5131,
  [5132] = 5132,
  [5133] = 5133,
  [5134] = 5134,
  [5135] = 4919,
  [5136] = 5136,
  [5137] = 4863,
  [5138] = 5138,
  [5139] = 5139,
  [5140] = 5140,
  [5141] = 5117,
  [5142] = 5142,
  [5143] = 5143,
  [5144] = 5144,
  [5145] = 5145,
  [5146] = 5146,
  [5147] = 5147,
  [5148] = 5148,
  [5149] = 5149,
  [5150] = 5150,
  [5151] = 5151,
  [5152] = 5152,
  [5153] = 5153,
  [5154] = 5154,
  [5155] = 5155,
  [5156] = 5156,
  [5157] = 5121,
  [5158] = 5118,
  [5159] = 5159,
  [5160] = 1245,
  [5161] = 5067,
  [5162] = 5162,
  [5163] = 5048,
  [5164] = 5051,
  [5165] = 5165,
  [5166] = 5166,
  [5167] = 4897,
  [5168] = 5168,
  [5169] = 4938,
  [5170] = 4940,
  [5171] = 5171,
  [5172] = 5172,
  [5173] = 5173,
  [5174] = 5174,
  [5175] = 5175,
  [5176] = 5176,
  [5177] = 5177,
  [5178] = 5178,
  [5179] = 5179,
  [5180] = 444,
  [5181] = 5181,
  [5182] = 5182,
  [5183] = 3799,
  [5184] = 5125,
  [5185] = 5185,
  [5186] = 5186,
  [5187] = 5138,
  [5188] = 5188,
  [5189] = 5189,
  [5190] = 5190,
  [5191] = 4973,
  [5192] = 4991,
  [5193] = 5067,
  [5194] = 5194,
  [5195] = 5195,
  [5196] = 5128,
  [5197] = 645,
  [5198] = 5123,
  [5199] = 4950,
  [5200] = 4934,
  [5201] = 5201,
  [5202] = 5084,
  [5203] = 5203,
  [5204] = 5204,
  [5205] = 5205,
  [5206] = 5131,
  [5207] = 5153,
  [5208] = 5126,
  [5209] = 5130,
  [5210] = 5194,
  [5211] = 4879,
  [5212] = 5151,
  [5213] = 5152,
  [5214] = 5201,
  [5215] = 5215,
  [5216] = 5134,
  [5217] = 5136,
  [5218] = 5218,
  [5219] = 5219,
  [5220] = 5220,
  [5221] = 5189,
  [5222] = 5222,
  [5223] = 5223,
  [5224] = 5224,
  [5225] = 4938,
  [5226] = 5226,
  [5227] = 3800,
  [5228] = 5228,
  [5229] = 5229,
  [5230] = 4940,
  [5231] = 4977,
  [5232] = 5232,
  [5233] = 4978,
  [5234] = 5234,
  [5235] = 5095,
  [5236] = 5236,
  [5237] = 5226,
  [5238] = 5238,
  [5239] = 5239,
  [5240] = 5238,
  [5241] = 5241,
  [5242] = 5149,
  [5243] = 4952,
  [5244] = 4942,
  [5245] = 5011,
  [5246] = 5232,
  [5247] = 5247,
  [5248] = 3777,
  [5249] = 3781,
  [5250] = 3754,
  [5251] = 3812,
  [5252] = 3783,
  [5253] = 3784,
  [5254] = 5139,
  [5255] = 429,
  [5256] = 5256,
  [5257] = 5257,
  [5258] = 5258,
  [5259] = 5006,
  [5260] = 3763,
  [5261] = 5261,
  [5262] = 5140,
  [5263] = 5263,
  [5264] = 5042,
  [5265] = 5265,
  [5266] = 5266,
  [5267] = 5267,
  [5268] = 5241,
  [5269] = 5088,
  [5270] = 5150,
  [5271] = 3764,
  [5272] = 3767,
  [5273] = 5016,
  [5274] = 5274,
  [5275] = 4889,
  [5276] = 4860,
  [5277] = 5277,
  [5278] = 5144,
  [5279] = 5215,
  [5280] = 5146,
  [5281] = 5228,
  [5282] = 4898,
  [5283] = 5148,
  [5284] = 5284,
  [5285] = 4932,
  [5286] = 5286,
  [5287] = 5287,
  [5288] = 1100,
  [5289] = 5289,
  [5290] = 5290,
  [5291] = 4929,
  [5292] = 5147,
  [5293] = 5293,
  [5294] = 4869,
  [5295] = 4925,
  [5296] = 5293,
  [5297] = 5297,
  [5298] = 5077,
  [5299] = 5299,
  [5300] = 5071,
  [5301] = 5301,
  [5302] = 5302,
  [5303] = 5303,
  [5304] = 4861,
  [5305] = 4907,
  [5306] = 4994,
  [5307] = 4987,
  [5308] = 5308,
  [5309] = 5309,
  [5310] = 5310,
  [5311] = 5311,
  [5312] = 5004,
  [5313] = 3772,
  [5314] = 5119,
  [5315] = 4937,
  [5316] = 2794,
  [5317] = 5317,
  [5318] = 5318,
  [5319] = 5319,
  [5320] = 4988,
  [5321] = 4989,
  [5322] = 4919,
  [5323] = 5323,
  [5324] = 4938,
  [5325] = 4940,
  [5326] = 5326,
  [5327] = 4879,
  [5328] = 5328,
  [5329] = 3773,
  [5330] = 4878,
  [5331] = 4881,
  [5332] = 4886,
  [5333] = 4931,
  [5334] = 4932,
  [5335] = 5219,
  [5336] = 4964,
  [5337] = 5337,
  [5338] = 5338,
  [5339] = 5021,
  [5340] = 5340,
  [5341] = 5218,
  [5342] = 3774,
  [5343] = 4901,
  [5344] = 4979,
  [5345] = 5261,
  [5346] = 4972,
  [5347] = 5043,
  [5348] = 4912,
  [5349] = 4982,
  [5350] = 5350,
  [5351] = 5351,
  [5352] = 5185,
  [5353] = 5195,
  [5354] = 4864,
  [5355] = 5090,
  [5356] = 4866,
  [5357] = 5357,
  [5358] = 5203,
  [5359] = 4915,
  [5360] = 5267,
  [5361] = 5297,
  [5362] = 5107,
  [5363] = 5239,
  [5364] = 5110,
  [5365] = 4919,
  [5366] = 5366,
  [5367] = 5319,
  [5368] = 5368,
  [5369] = 5257,
  [5370] = 4865,
  [5371] = 5371,
  [5372] = 471,
  [5373] = 5222,
  [5374] = 5286,
  [5375] = 5234,
  [5376] = 4895,
  [5377] = 5377,
  [5378] = 5287,
  [5379] = 4904,
  [5380] = 467,
  [5381] = 4871,
  [5382] = 5382,
  [5383] = 5383,
  [5384] = 5220,
  [5385] = 5008,
  [5386] = 5080,
  [5387] = 5036,
  [5388] = 5083,
  [5389] = 5041,
  [5390] = 5390,
  [5391] = 5050,
  [5392] = 4909,
  [5393] = 5289,
  [5394] = 485,
  [5395] = 5337,
  [5396] = 5396,
  [5397] = 5038,
  [5398] = 5398,
  [5399] = 5113,
  [5400] = 5318,
  [5401] = 5188,
  [5402] = 5132,
  [5403] = 5190,
  [5404] = 5133,
  [5405] = 5129,
  [5406] = 4941,
  [5407] = 4998,
  [5408] = 4920,
  [5409] = 5382,
  [5410] = 4916,
  [5411] = 5290,
  [5412] = 5303,
  [5413] = 5057,
  [5414] = 4936,
  [5415] = 5415,
  [5416] = 5416,
  [5417] = 5258,
  [5418] = 5418,
  [5419] = 5205,
  [5420] = 5223,
  [5421] = 5274,
  [5422] = 5422,
  [5423] = 5368,
  [5424] = 5424,
  [5425] = 5171,
  [5426] = 5073,
  [5427] = 5173,
  [5428] = 4938,
  [5429] = 5178,
  [5430] = 5351,
  [5431] = 5159,
  [5432] = 5162,
  [5433] = 5179,
  [5434] = 448,
  [5435] = 462,
  [5436] = 5436,
  [5437] = 4940,
  [5438] = 5366,
  [5439] = 5390,
  [5440] = 4942,
  [5441] = 420,
  [5442] = 4931,
  [5443] = 5443,
  [5444] = 5444,
  [5445] = 5445,
  [5446] = 5446,
  [5447] = 5447,
  [5448] = 5448,
  [5449] = 5449,
  [5450] = 5450,
  [5451] = 5451,
  [5452] = 5452,
  [5453] = 5453,
  [5454] = 5454,
  [5455] = 5455,
  [5456] = 5456,
  [5457] = 5457,
  [5458] = 5458,
  [5459] = 5459,
  [5460] = 5460,
  [5461] = 5461,
  [5462] = 5462,
  [5463] = 5463,
  [5464] = 5464,
  [5465] = 5465,
  [5466] = 5466,
  [5467] = 5467,
  [5468] = 5468,
  [5469] = 5469,
  [5470] = 5470,
  [5471] = 5471,
  [5472] = 5472,
  [5473] = 5473,
  [5474] = 5474,
  [5475] = 5475,
  [5476] = 5476,
  [5477] = 5477,
  [5478] = 5478,
  [5479] = 5479,
  [5480] = 5448,
  [5481] = 5481,
  [5482] = 5482,
  [5483] = 5483,
  [5484] = 5484,
  [5485] = 5485,
  [5486] = 5486,
  [5487] = 5463,
  [5488] = 5488,
  [5489] = 5489,
  [5490] = 5458,
  [5491] = 5491,
  [5492] = 5492,
  [5493] = 5493,
  [5494] = 5494,
  [5495] = 5495,
  [5496] = 5496,
  [5497] = 5497,
  [5498] = 5498,
  [5499] = 5462,
  [5500] = 5500,
  [5501] = 5501,
  [5502] = 5502,
  [5503] = 5503,
  [5504] = 5504,
  [5505] = 5505,
  [5506] = 5506,
  [5507] = 5507,
  [5508] = 5508,
  [5509] = 5509,
  [5510] = 5451,
  [5511] = 5452,
  [5512] = 5453,
  [5513] = 5513,
  [5514] = 5514,
  [5515] = 5515,
  [5516] = 5516,
  [5517] = 5517,
  [5518] = 5518,
  [5519] = 5519,
  [5520] = 5508,
  [5521] = 5458,
  [5522] = 5455,
  [5523] = 5523,
  [5524] = 5465,
  [5525] = 5525,
  [5526] = 5523,
  [5527] = 5527,
  [5528] = 5503,
  [5529] = 5529,
  [5530] = 5530,
  [5531] = 5531,
  [5532] = 5457,
  [5533] = 5533,
  [5534] = 5534,
  [5535] = 5535,
  [5536] = 5458,
  [5537] = 5459,
  [5538] = 5462,
  [5539] = 5539,
  [5540] = 5458,
  [5541] = 5541,
  [5542] = 5542,
  [5543] = 5543,
  [5544] = 5467,
  [5545] = 5545,
  [5546] = 5546,
  [5547] = 5547,
  [5548] = 5548,
  [5549] = 5459,
  [5550] = 5481,
  [5551] = 5551,
  [5552] = 2959,
  [5553] = 5486,
  [5554] = 5458,
  [5555] = 5555,
  [5556] = 5493,
  [5557] = 5494,
  [5558] = 5558,
  [5559] = 5559,
  [5560] = 5560,
  [5561] = 5505,
  [5562] = 5562,
  [5563] = 5563,
  [5564] = 5502,
  [5565] = 5565,
  [5566] = 5566,
  [5567] = 5567,
  [5568] = 5568,
  [5569] = 5569,
  [5570] = 5459,
  [5571] = 5462,
  [5572] = 5572,
  [5573] = 5573,
  [5574] = 5574,
  [5575] = 5467,
  [5576] = 5576,
  [5577] = 5577,
  [5578] = 5481,
  [5579] = 5450,
  [5580] = 5573,
  [5581] = 5454,
  [5582] = 5477,
  [5583] = 5583,
  [5584] = 5458,
  [5585] = 5493,
  [5586] = 5494,
  [5587] = 5460,
  [5588] = 5588,
  [5589] = 5505,
  [5590] = 5464,
  [5591] = 5466,
  [5592] = 5592,
  [5593] = 5470,
  [5594] = 5472,
  [5595] = 5459,
  [5596] = 5596,
  [5597] = 5467,
  [5598] = 5598,
  [5599] = 5599,
  [5600] = 5600,
  [5601] = 5601,
  [5602] = 5602,
  [5603] = 5603,
  [5604] = 5458,
  [5605] = 5493,
  [5606] = 5494,
  [5607] = 5607,
  [5608] = 5459,
  [5609] = 5609,
  [5610] = 5610,
  [5611] = 5611,
  [5612] = 5473,
  [5613] = 5467,
  [5614] = 5460,
  [5615] = 5443,
  [5616] = 5458,
  [5617] = 5493,
  [5618] = 5494,
  [5619] = 5462,
  [5620] = 5620,
  [5621] = 5621,
  [5622] = 5622,
  [5623] = 5474,
  [5624] = 5624,
  [5625] = 5467,
  [5626] = 5626,
  [5627] = 5458,
  [5628] = 5493,
  [5629] = 5494,
  [5630] = 5630,
  [5631] = 5558,
  [5632] = 5467,
  [5633] = 5493,
  [5634] = 5494,
  [5635] = 5467,
  [5636] = 5493,
  [5637] = 5494,
  [5638] = 5467,
  [5639] = 5493,
  [5640] = 5494,
  [5641] = 5467,
  [5642] = 5493,
  [5643] = 5494,
  [5644] = 5644,
  [5645] = 5645,
  [5646] = 5646,
  [5647] = 5647,
  [5648] = 5648,
  [5649] = 5649,
  [5650] = 5650,
  [5651] = 5651,
  [5652] = 5444,
  [5653] = 5653,
  [5654] = 5654,
  [5655] = 5655,
  [5656] = 5465,
  [5657] = 5479,
  [5658] = 5471,
  [5659] = 5483,
  [5660] = 5493,
  [5661] = 5474,
  [5662] = 5662,
  [5663] = 5501,
  [5664] = 2811,
  [5665] = 5665,
  [5666] = 2840,
  [5667] = 5478,
  [5668] = 2868,
  [5669] = 5669,
  [5670] = 5467,
  [5671] = 2870,
  [5672] = 5468,
  [5673] = 5525,
  [5674] = 5674,
  [5675] = 5609,
  [5676] = 5471,
  [5677] = 5445,
  [5678] = 5473,
  [5679] = 5568,
  [5680] = 5574,
  [5681] = 5681,
  [5682] = 5476,
  [5683] = 4894,
  [5684] = 5474,
  [5685] = 5467,
  [5686] = 5686,
  [5687] = 2874,
  [5688] = 5688,
  [5689] = 5689,
  [5690] = 5690,
  [5691] = 5691,
  [5692] = 5024,
  [5693] = 5045,
  [5694] = 5694,
  [5695] = 5695,
  [5696] = 5696,
  [5697] = 5697,
  [5698] = 5698,
  [5699] = 5699,
  [5700] = 5700,
  [5701] = 5701,
  [5702] = 5481,
  [5703] = 5703,
  [5704] = 5506,
  [5705] = 5705,
  [5706] = 5686,
  [5707] = 5707,
  [5708] = 2920,
  [5709] = 5709,
  [5710] = 5484,
  [5711] = 5485,
  [5712] = 5555,
  [5713] = 5562,
  [5714] = 5563,
  [5715] = 2923,
  [5716] = 5716,
  [5717] = 5717,
  [5718] = 5718,
  [5719] = 5719,
  [5720] = 5592,
  [5721] = 5721,
  [5722] = 5722,
  [5723] = 5486,
  [5724] = 5463,
  [5725] = 5725,
  [5726] = 5726,
  [5727] = 5624,
  [5728] = 5728,
  [5729] = 5729,
  [5730] = 5489,
  [5731] = 5645,
  [5732] = 5513,
  [5733] = 5733,
  [5734] = 5649,
  [5735] = 2929,
  [5736] = 5497,
  [5737] = 2931,
  [5738] = 5484,
  [5739] = 5739,
  [5740] = 5740,
  [5741] = 5471,
  [5742] = 5742,
  [5743] = 5743,
  [5744] = 5744,
  [5745] = 5745,
  [5746] = 5717,
  [5747] = 5718,
  [5748] = 5721,
  [5749] = 5749,
  [5750] = 5458,
  [5751] = 5751,
  [5752] = 5752,
  [5753] = 5523,
  [5754] = 5754,
  [5755] = 5669,
  [5756] = 5493,
  [5757] = 5757,
  [5758] = 5758,
  [5759] = 5759,
  [5760] = 5494,
  [5761] = 5761,
  [5762] = 5495,
  [5763] = 5757,
  [5764] = 5529,
  [5765] = 5566,
  [5766] = 5569,
  [5767] = 5572,
  [5768] = 5576,
  [5769] = 2980,
  [5770] = 5770,
  [5771] = 2983,
  [5772] = 5498,
  [5773] = 5496,
  [5774] = 5719,
  [5775] = 4894,
  [5776] = 5622,
  [5777] = 5626,
  [5778] = 5502,
  [5779] = 5630,
  [5780] = 5647,
  [5781] = 5781,
  [5782] = 5770,
  [5783] = 5498,
  [5784] = 5784,
  [5785] = 5785,
  [5786] = 5786,
  [5787] = 5468,
  [5788] = 5788,
  [5789] = 5789,
  [5790] = 5790,
  [5791] = 2907,
  [5792] = 5792,
  [5793] = 5695,
  [5794] = 5716,
  [5795] = 5784,
  [5796] = 5796,
  [5797] = 5518,
  [5798] = 5654,
  [5799] = 5761,
  [5800] = 5531,
  [5801] = 5543,
  [5802] = 5596,
  [5803] = 5648,
  [5804] = 5804,
  [5805] = 5505,
  [5806] = 2970,
  [5807] = 5621,
  [5808] = 5533,
  [5809] = 5603,
  [5810] = 5655,
  [5811] = 5662,
  [5812] = 5709,
  [5813] = 5728,
  [5814] = 5458,
  [5815] = 5815,
  [5816] = 5665,
  [5817] = 5500,
  [5818] = 5517,
  [5819] = 5506,
  [5820] = 5745,
  [5821] = 5821,
  [5822] = 5796,
  [5823] = 5602,
  [5824] = 5646,
  [5825] = 5825,
  [5826] = 5507,
  [5827] = 5790,
  [5828] = 5828,
  [5829] = 5829,
  [5830] = 5830,
  [5831] = 5505,
  [5832] = 5832,
  [5833] = 5833,
  [5834] = 5471,
  [5835] = 5474,
  [5836] = 2811,
  [5837] = 5665,
  [5838] = 2840,
  [5839] = 5839,
  [5840] = 5840,
  [5841] = 2832,
  [5842] = 5476,
  [5843] = 4894,
  [5844] = 2812,
  [5845] = 2859,
  [5846] = 5846,
  [5847] = 5847,
  [5848] = 5848,
  [5849] = 5527,
  [5850] = 5850,
  [5851] = 2827,
  [5852] = 5486,
  [5853] = 5853,
  [5854] = 5509,
  [5855] = 5855,
  [5856] = 5856,
  [5857] = 5665,
  [5858] = 5786,
  [5859] = 5859,
  [5860] = 5788,
  [5861] = 5789,
  [5862] = 5739,
  [5863] = 5665,
  [5864] = 5864,
  [5865] = 5865,
  [5866] = 5461,
  [5867] = 5733,
  [5868] = 5868,
  [5869] = 5473,
  [5870] = 5870,
  [5871] = 5600,
  [5872] = 5872,
  [5873] = 5519,
  [5874] = 5598,
  [5875] = 5599,
  [5876] = 5601,
  [5877] = 5877,
  [5878] = 5498,
  [5879] = 5879,
  [5880] = 5880,
  [5881] = 5476,
  [5882] = 5502,
  [5883] = 5883,
  [5884] = 5754,
  [5885] = 5689,
  [5886] = 5698,
  [5887] = 5699,
  [5888] = 5644,
  [5889] = 5494,
  [5890] = 5890,
  [5891] = 5891,
  [5892] = 5456,
  [5893] = 5469,
  [5894] = 5815,
  [5895] = 5781,
  [5896] = 5855,
  [5897] = 5864,
  [5898] = 5872,
  [5899] = 5481,
  [5900] = 5829,
  [5901] = 5901,
  [5902] = 5691,
  [5903] = 5517,
  [5904] = 5904,
  [5905] = 5535,
  [5906] = 5446,
  [5907] = 5546,
  [5908] = 5700,
  [5909] = 5703,
  [5910] = 5447,
  [5911] = 5610,
  [5912] = 5491,
  [5913] = 5690,
  [5914] = 5914,
  [5915] = 5694,
  [5916] = 5749,
  [5917] = 5696,
  [5918] = 5752,
  [5919] = 5729,
  [5920] = 5877,
  [5921] = 5461,
  [5922] = 5473,
  [5923] = 5600,
  [5924] = 5476,
  [5925] = 5541,
  [5926] = 5502,
  [5927] = 5829,
  [5928] = 5600,
  [5929] = 5829,
  [5930] = 5542,
  [5931] = 5547,
  [5932] = 5879,
  [5933] = 5726,
  [5934] = 5934,
  [5935] = 5607,
  [5936] = 5725,
  [5937] = 5577,
  [5938] = 5938,
  [5939] = 5674,
  [5940] = 5759,
  [5941] = 5697,
  [5942] = 5740,
  [5943] = 5743,
  [5944] = 5744,
  [5945] = 5514,
  [5946] = 5651,
  [5947] = 5848,
  [5948] = 5449,
  [5949] = 5559,
  [5950] = 5870,
  [5951] = 5482,
  [5952] = 5461,
  [5953] = 5515,
  [5954] = 5833,
  [5955] = 5840,
  [5956] = 5850,
  [5957] = 5914,
  [5958] = 5958,
  [5959] = 5959,
  [5960] = 5960,
  [5961] = 5961,
  [5962] = 5962,
  [5963] = 5963,
  [5964] = 5964,
  [5965] = 5965,
  [5966] = 5966,
  [5967] = 5967,
  [5968] = 5968,
  [5969] = 5961,
  [5970] = 5970,
  [5971] = 5971,
  [5972] = 5972,
  [5973] = 5973,
  [5974] = 5974,
  [5975] = 5975,
  [5976] = 5976,
  [5977] = 5977,
  [5978] = 5978,
  [5979] = 5979,
  [5980] = 5980,
  [5981] = 5981,
  [5982] = 5982,
  [5983] = 5983,
  [5984] = 5984,
  [5985] = 5985,
  [5986] = 5986,
  [5987] = 5987,
  [5988] = 5988,
  [5989] = 5989,
  [5990] = 5990,
  [5991] = 5991,
  [5992] = 5992,
  [5993] = 5993,
  [5994] = 5975,
  [5995] = 5995,
  [5996] = 5996,
  [5997] = 5997,
  [5998] = 5966,
  [5999] = 5999,
  [6000] = 6000,
  [6001] = 6001,
  [6002] = 6002,
  [6003] = 6003,
  [6004] = 6004,
  [6005] = 6002,
  [6006] = 6006,
  [6007] = 6007,
  [6008] = 6008,
  [6009] = 6009,
  [6010] = 6010,
  [6011] = 6011,
  [6012] = 6012,
  [6013] = 6013,
  [6014] = 6014,
  [6015] = 6015,
  [6016] = 6016,
  [6017] = 6017,
  [6018] = 6018,
  [6019] = 5967,
  [6020] = 6020,
  [6021] = 6021,
  [6022] = 6022,
  [6023] = 6023,
  [6024] = 5988,
  [6025] = 6025,
  [6026] = 6026,
  [6027] = 6027,
  [6028] = 6028,
  [6029] = 6008,
  [6030] = 6030,
  [6031] = 6012,
  [6032] = 6032,
  [6033] = 6033,
  [6034] = 5972,
  [6035] = 6035,
  [6036] = 6036,
  [6037] = 6037,
  [6038] = 6035,
  [6039] = 6039,
  [6040] = 6040,
  [6041] = 5987,
  [6042] = 6042,
  [6043] = 6043,
  [6044] = 6040,
  [6045] = 6045,
  [6046] = 5982,
  [6047] = 6047,
  [6048] = 6048,
  [6049] = 6002,
  [6050] = 6050,
  [6051] = 6002,
  [6052] = 6052,
  [6053] = 6053,
  [6054] = 6054,
  [6055] = 6055,
  [6056] = 6056,
  [6057] = 6057,
  [6058] = 6058,
  [6059] = 6059,
  [6060] = 6033,
  [6061] = 6061,
  [6062] = 6048,
  [6063] = 6063,
  [6064] = 6064,
  [6065] = 6042,
  [6066] = 6066,
  [6067] = 6067,
  [6068] = 6068,
  [6069] = 6069,
  [6070] = 6026,
  [6071] = 6071,
  [6072] = 6014,
  [6073] = 6073,
  [6074] = 6074,
  [6075] = 6061,
  [6076] = 6039,
  [6077] = 6077,
  [6078] = 6078,
  [6079] = 6079,
  [6080] = 6055,
  [6081] = 5963,
  [6082] = 6082,
  [6083] = 6083,
  [6084] = 6084,
  [6085] = 6085,
  [6086] = 6086,
  [6087] = 6087,
  [6088] = 6088,
  [6089] = 6089,
  [6090] = 6090,
  [6091] = 6083,
  [6092] = 6092,
  [6093] = 6042,
  [6094] = 6094,
  [6095] = 6095,
  [6096] = 6096,
  [6097] = 6084,
  [6098] = 6098,
  [6099] = 6099,
  [6100] = 6100,
  [6101] = 6101,
  [6102] = 6102,
  [6103] = 5989,
  [6104] = 6104,
  [6105] = 6105,
  [6106] = 5982,
  [6107] = 6107,
  [6108] = 6108,
  [6109] = 5971,
  [6110] = 6110,
  [6111] = 6002,
  [6112] = 419,
  [6113] = 6113,
  [6114] = 6114,
  [6115] = 6068,
  [6116] = 6116,
  [6117] = 6117,
  [6118] = 6118,
  [6119] = 6068,
  [6120] = 6037,
  [6121] = 6121,
  [6122] = 6073,
  [6123] = 6102,
  [6124] = 6035,
  [6125] = 6028,
  [6126] = 6126,
  [6127] = 6127,
  [6128] = 6128,
  [6129] = 6129,
  [6130] = 6130,
  [6131] = 6131,
  [6132] = 6132,
  [6133] = 6133,
  [6134] = 6074,
  [6135] = 5995,
  [6136] = 6136,
  [6137] = 6137,
  [6138] = 6138,
  [6139] = 6139,
  [6140] = 6006,
  [6141] = 6007,
  [6142] = 6142,
  [6143] = 6068,
  [6144] = 6006,
  [6145] = 6077,
  [6146] = 6146,
  [6147] = 6007,
  [6148] = 6148,
  [6149] = 6079,
  [6150] = 6150,
  [6151] = 6151,
  [6152] = 5999,
  [6153] = 6153,
  [6154] = 6148,
  [6155] = 6155,
  [6156] = 6156,
  [6157] = 6157,
  [6158] = 6158,
  [6159] = 5963,
  [6160] = 6014,
  [6161] = 6161,
  [6162] = 6131,
  [6163] = 6163,
  [6164] = 6027,
  [6165] = 6087,
  [6166] = 6166,
  [6167] = 6108,
  [6168] = 6006,
  [6169] = 6169,
  [6170] = 6101,
  [6171] = 6171,
  [6172] = 5982,
  [6173] = 6173,
  [6174] = 6174,
  [6175] = 5982,
  [6176] = 6176,
  [6177] = 6030,
  [6178] = 6178,
  [6179] = 6169,
  [6180] = 6180,
  [6181] = 6181,
  [6182] = 6182,
  [6183] = 6183,
  [6184] = 6184,
  [6185] = 6185,
  [6186] = 6121,
  [6187] = 6095,
  [6188] = 6188,
  [6189] = 6036,
  [6190] = 6096,
  [6191] = 6191,
  [6192] = 6192,
  [6193] = 6107,
  [6194] = 6194,
  [6195] = 6151,
  [6196] = 6196,
  [6197] = 5985,
  [6198] = 6198,
  [6199] = 5983,
  [6200] = 6200,
  [6201] = 6059,
  [6202] = 5996,
  [6203] = 6178,
  [6204] = 6146,
  [6205] = 6205,
  [6206] = 6206,
  [6207] = 6207,
  [6208] = 6208,
  [6209] = 6209,
  [6210] = 6101,
  [6211] = 6211,
  [6212] = 6180,
  [6213] = 6213,
  [6214] = 6214,
  [6215] = 5968,
  [6216] = 6216,
  [6217] = 6058,
  [6218] = 6086,
  [6219] = 6133,
  [6220] = 6156,
  [6221] = 6171,
  [6222] = 5968,
  [6223] = 6018,
  [6224] = 6224,
  [6225] = 6225,
  [6226] = 6216,
  [6227] = 6227,
  [6228] = 6082,
  [6229] = 6229,
  [6230] = 6227,
  [6231] = 6231,
  [6232] = 6232,
  [6233] = 6207,
  [6234] = 6156,
  [6235] = 5962,
  [6236] = 6236,
  [6237] = 6237,
  [6238] = 6105,
  [6239] = 6229,
  [6240] = 6240,
  [6241] = 6241,
  [6242] = 6242,
  [6243] = 6243,
  [6244] = 6244,
  [6245] = 6245,
  [6246] = 6225,
  [6247] = 6176,
  [6248] = 6163,
  [6249] = 6064,
  [6250] = 6250,
  [6251] = 6078,
  [6252] = 6252,
  [6253] = 6227,
  [6254] = 6130,
  [6255] = 6255,
  [6256] = 5960,
  [6257] = 6082,
  [6258] = 6148,
  [6259] = 6259,
  [6260] = 6101,
  [6261] = 6261,
  [6262] = 6262,
  [6263] = 6263,
  [6264] = 6216,
  [6265] = 6069,
  [6266] = 6266,
  [6267] = 6156,
  [6268] = 6268,
  [6269] = 6006,
  [6270] = 5982,
  [6271] = 423,
  [6272] = 6105,
  [6273] = 6229,
  [6274] = 6243,
  [6275] = 6004,
  [6276] = 5977,
  [6277] = 6205,
  [6278] = 6278,
  [6279] = 6216,
  [6280] = 6101,
  [6281] = 6156,
  [6282] = 6282,
  [6283] = 6243,
  [6284] = 6243,
  [6285] = 6101,
  [6286] = 6156,
  [6287] = 6287,
  [6288] = 6243,
  [6289] = 6156,
  [6290] = 6243,
  [6291] = 6156,
  [6292] = 6243,
  [6293] = 6156,
  [6294] = 6156,
  [6295] = 6156,
  [6296] = 6156,
  [6297] = 6297,
  [6298] = 6298,
  [6299] = 6207,
  [6300] = 6099,
  [6301] = 6032,
  [6302] = 6241,
  [6303] = 6263,
  [6304] = 6137,
  [6305] = 6305,
  [6306] = 6306,
  [6307] = 6307,
  [6308] = 6297,
  [6309] = 6309,
  [6310] = 6310,
  [6311] = 6007,
  [6312] = 5992,
  [6313] = 6157,
  [6314] = 5978,
  [6315] = 6315,
  [6316] = 6316,
  [6317] = 6173,
  [6318] = 6183,
  [6319] = 6184,
  [6320] = 6240,
  [6321] = 6321,
  [6322] = 6078,
  [6323] = 6323,
  [6324] = 6310,
  [6325] = 6325,
  [6326] = 5986,
  [6327] = 6321,
  [6328] = 6015,
  [6329] = 6182,
  [6330] = 6330,
  [6331] = 6331,
  [6332] = 6332,
  [6333] = 6333,
  [6334] = 5958,
  [6335] = 6224,
  [6336] = 6158,
  [6337] = 6332,
  [6338] = 6166,
  [6339] = 6339,
  [6340] = 6014,
  [6341] = 6341,
  [6342] = 6342,
  [6343] = 6113,
  [6344] = 5993,
  [6345] = 6054,
  [6346] = 5990,
  [6347] = 6347,
  [6348] = 6232,
  [6349] = 6349,
  [6350] = 6350,
  [6351] = 6351,
  [6352] = 6347,
  [6353] = 6089,
  [6354] = 6104,
  [6355] = 5961,
  [6356] = 6356,
  [6357] = 6021,
  [6358] = 6358,
  [6359] = 6200,
  [6360] = 6209,
  [6361] = 6213,
  [6362] = 5961,
  [6363] = 6339,
  [6364] = 5964,
  [6365] = 6365,
  [6366] = 6136,
  [6367] = 6155,
  [6368] = 6078,
  [6369] = 6129,
  [6370] = 6181,
  [6371] = 6278,
  [6372] = 6306,
  [6373] = 6105,
  [6374] = 6229,
  [6375] = 5965,
  [6376] = 6191,
  [6377] = 5979,
  [6378] = 6148,
  [6379] = 6243,
  [6380] = 6333,
  [6381] = 6255,
  [6382] = 6325,
  [6383] = 6383,
  [6384] = 6323,
  [6385] = 6330,
  [6386] = 6016,
  [6387] = 6057,
  [6388] = 6388,
  [6389] = 6026,
  [6390] = 6390,
  [6391] = 6053,
  [6392] = 6211,
  [6393] = 6250,
  [6394] = 6013,
  [6395] = 6007,
  [6396] = 6315,
  [6397] = 6397,
  [6398] = 6398,
  [6399] = 6061,
  [6400] = 6011,
  [6401] = 6401,
  [6402] = 6084,
  [6403] = 6403,
  [6404] = 6117,
  [6405] = 6043,
  [6406] = 6161,
  [6407] = 6025,
  [6408] = 6398,
  [6409] = 6066,
  [6410] = 6063,
  [6411] = 6298,
  [6412] = 6099,
  [6413] = 6032,
  [6414] = 5970,
  [6415] = 6094,
  [6416] = 6307,
  [6417] = 6297,
  [6418] = 6307,
  [6419] = 6002,
  [6420] = 6136,
  [6421] = 6358,
  [6422] = 6138,
  [6423] = 6166,
  [6424] = 6342,
  [6425] = 6099,
  [6426] = 6032,
  [6427] = 6297,
  [6428] = 6331,
  [6429] = 6429,
  [6430] = 6032,
  [6431] = 6297,
  [6432] = 6032,
  [6433] = 6297,
  [6434] = 6297,
  [6435] = 6297,
  [6436] = 6436,
  [6437] = 6282,
  [6438] = 6438,
  [6439] = 6082,
  [6440] = 5959,
  [6441] = 6441,
  [6442] = 6442,
  [6443] = 6350,
  [6444] = 6429,
  [6445] = 6445,
  [6446] = 6050,
  [6447] = 6231,
  [6448] = 6316,
  [6449] = 5976,
  [6450] = 6006,
  [6451] = 6252,
  [6452] = 6452,
  [6453] = 6007,
  [6454] = 5974,
  [6455] = 6455,
  [6456] = 6342,
  [6457] = 6192,
  [6458] = 6090,
  [6459] = 6259,
  [6460] = 6460,
  [6461] = 6142,
  [6462] = 6261,
  [6463] = 6463,
  [6464] = 6388,
  [6465] = 6001,
  [6466] = 5973,
  [6467] = 5984,
  [6468] = 6445,
  [6469] = 6026,
  [6470] = 6397,
  [6471] = 6153,
  [6472] = 6017,
  [6473] = 6214,
  [6474] = 6349,
  [6475] = 6071,
  [6476] = 6452,
  [6477] = 6009,
  [6478] = 6478,
  [6479] = 6436,
  [6480] = 6071,
  [6481] = 6438,
  [6482] = 6442,
  [6483] = 6441,
  [6484] = 6266,
  [6485] = 6022,
  [6486] = 6196,
  [6487] = 6100,
  [6488] = 6351,
  [6489] = 6356,
  [6490] = 6206,
  [6491] = 6082,
  [6492] = 6208,
  [6493] = 5991,
  [6494] = 6084,
  [6495] = 6463,
  [6496] = 6020,
  [6497] = 6309,
  [6498] = 6498,
  [6499] = 6499,
  [6500] = 6500,
  [6501] = 6501,
  [6502] = 6502,
  [6503] = 6503,
};

static const TSCharacterRange sym_identifier_character_set_1[] = {
  {'A', 'Z'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xba, 0xba}, {0xc0, 0xd6}, {0xd8, 0xf6},
  {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x370, 0x374}, {0x376, 0x377}, {0x37b, 0x37d},
  {0x37f, 0x37f}, {0x386, 0x386}, {0x388, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5}, {0x3f7, 0x481}, {0x48a, 0x52f},
  {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x620, 0x64a}, {0x66e, 0x66f}, {0x671, 0x6d3},
  {0x6d5, 0x6d5}, {0x6e5, 0x6e6}, {0x6ee, 0x6ef}, {0x6fa, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x710}, {0x712, 0x72f}, {0x74d, 0x7a5},
  {0x7b1, 0x7b1}, {0x7ca, 0x7ea}, {0x7f4, 0x7f5}, {0x7fa, 0x7fa}, {0x800, 0x815}, {0x81a, 0x81a}, {0x824, 0x824}, {0x828, 0x828},
  {0x840, 0x858}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x8a0, 0x8c9}, {0x904, 0x939}, {0x93d, 0x93d}, {0x950, 0x950},
  {0x958, 0x961}, {0x971, 0x980}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0}, {0x9b2, 0x9b2}, {0x9b6, 0x9b9},
  {0x9bd, 0x9bd}, {0x9ce, 0x9ce}, {0x9dc, 0x9dd}, {0x9df, 0x9e1}, {0x9f0, 0x9f1}, {0x9fc, 0x9fc}, {0xa05, 0xa0a}, {0xa0f, 0xa10},
  {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39}, {0xa59, 0xa5c}, {0xa5e, 0xa5e}, {0xa72, 0xa74},
  {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9}, {0xabd, 0xabd}, {0xad0, 0xad0},
  {0xae0, 0xae1}, {0xaf9, 0xaf9}, {0xb05, 0xb0c}, {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30}, {0xb32, 0xb33}, {0xb35, 0xb39},
  {0xb3d, 0xb3d}, {0xb5c, 0xb5d}, {0xb5f, 0xb61}, {0xb71, 0xb71}, {0xb83, 0xb83}, {0xb85, 0xb8a}, {0xb8e, 0xb90}, {0xb92, 0xb95},
  {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9}, {0xbd0, 0xbd0}, {0xc05, 0xc0c},
  {0xc0e, 0xc10}, {0xc12, 0xc28}, {0xc2a, 0xc39}, {0xc3d, 0xc3d}, {0xc58, 0xc5a}, {0xc5d, 0xc5d}, {0xc60, 0xc61}, {0xc80, 0xc80},
  {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbd, 0xcbd}, {0xcdd, 0xcde}, {0xce0, 0xce1},
  {0xcf1, 0xcf2}, {0xd04, 0xd0c}, {0xd0e, 0xd10}, {0xd12, 0xd3a}, {0xd3d, 0xd3d}, {0xd4e, 0xd4e}, {0xd54, 0xd56}, {0xd5f, 0xd61},
  {0xd7a, 0xd7f}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6}, {0xe01, 0xe30}, {0xe32, 0xe32},
  {0xe40, 0xe46}, {0xe81, 0xe82}, {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5}, {0xea7, 0xeb0}, {0xeb2, 0xeb2},
  {0xebd, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf40, 0xf47}, {0xf49, 0xf6c}, {0xf88, 0xf8c},
  {0x1000, 0x102a}, {0x103f, 0x103f}, {0x1050, 0x1055}, {0x105a, 0x105d}, {0x1061, 0x1061}, {0x1065, 0x1066}, {0x106e, 0x1070}, {0x1075, 0x1081},
  {0x108e, 0x108e}, {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248}, {0x124a, 0x124d}, {0x1250, 0x1256},
  {0x1258, 0x1258}, {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5}, {0x12b8, 0x12be}, {0x12c0, 0x12c0},
  {0x12c2, 0x12c5}, {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x1380, 0x138f}, {0x13a0, 0x13f5}, {0x13f8, 0x13fd},
  {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1711}, {0x171f, 0x1731}, {0x1740, 0x1751},
  {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1780, 0x17b3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dc}, {0x1820, 0x1878}, {0x1880, 0x18a8}, {0x18aa, 0x18aa},
  {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1950, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9}, {0x1a00, 0x1a16}, {0x1a20, 0x1a54},
  {0x1aa7, 0x1aa7}, {0x1b05, 0x1b33}, {0x1b45, 0x1b4c}, {0x1b83, 0x1ba0}, {0x1bae, 0x1baf}, {0x1bba, 0x1be5}, {0x1c00, 0x1c23}, {0x1c4d, 0x1c4f},
  {0x1c5a, 0x1c7d}, {0x1c80, 0x1c8a}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1ce9, 0x1cec}, {0x1cee, 0x1cf3}, {0x1cf5, 0x1cf6}, {0x1cfa, 0x1cfa},
  {0x1d00, 0x1dbf}, {0x1e00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57}, {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b},
  {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4}, {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3},
  {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x2071, 0x2071}, {0x207f, 0x207f}, {0x2090, 0x209c}, {0x2102, 0x2102},
  {0x2107, 0x2107}, {0x210a, 0x2113}, {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126}, {0x2128, 0x2128}, {0x212a, 0x2139},
  {0x213c, 0x213f}, {0x2145, 0x2149}, {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cee}, {0x2cf2, 0x2cf3}, {0x2d00, 0x2d25},
  {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67}, {0x2d6f, 0x2d6f}, {0x2d80, 0x2d96}, {0x2da0, 0x2da6}, {0x2da8, 0x2dae}, {0x2db0, 0x2db6},
  {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce}, {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x3005, 0x3007}, {0x3021, 0x3029}, {0x3031, 0x3035},
  {0x3038, 0x303c}, {0x3041, 0x3096}, {0x309d, 0x309f}, {0x30a1, 0x30fa}, {0x30fc, 0x30ff}, {0x3105, 0x312f}, {0x3131, 0x318e}, {0x31a0, 0x31bf},
  {0x31f0, 0x31ff}, {0x3400, 0x4dbf}, {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa61f}, {0xa62a, 0xa62b}, {0xa640, 0xa66e},
  {0xa67f, 0xa69d}, {0xa6a0, 0xa6ef}, {0xa717, 0xa71f}, {0xa722, 0xa788}, {0xa78b, 0xa7cd}, {0xa7d0, 0xa7d1}, {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7dc},
  {0xa7f2, 0xa801}, {0xa803, 0xa805}, {0xa807, 0xa80a}, {0xa80c, 0xa822}, {0xa840, 0xa873}, {0xa882, 0xa8b3}, {0xa8f2, 0xa8f7}, {0xa8fb, 0xa8fb},
  {0xa8fd, 0xa8fe}, {0xa90a, 0xa925}, {0xa930, 0xa946}, {0xa960, 0xa97c}, {0xa984, 0xa9b2}, {0xa9cf, 0xa9cf}, {0xa9e0, 0xa9e4}, {0xa9e6, 0xa9ef},
  {0xa9fa, 0xa9fe}, {0xaa00, 0xaa28}, {0xaa40, 0xaa42}, {0xaa44, 0xaa4b}, {0xaa60, 0xaa76}, {0xaa7a, 0xaa7a}, {0xaa7e, 0xaaaf}, {0xaab1, 0xaab1},
  {0xaab5, 0xaab6}, {0xaab9, 0xaabd}, {0xaac0, 0xaac0}, {0xaac2, 0xaac2}, {0xaadb, 0xaadd}, {0xaae0, 0xaaea}, {0xaaf2, 0xaaf4}, {0xab01, 0xab06},
  {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a}, {0xab5c, 0xab69}, {0xab70, 0xabe2}, {0xac00, 0xd7a3},
  {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d}, {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17}, {0xfb1d, 0xfb1d}, {0xfb1f, 0xfb28},
  {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41}, {0xfb43, 0xfb44}, {0xfb46, 0xfbb1}, {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d},
  {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77}, {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b},
  {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff21, 0xff3a}, {0xff41, 0xff5a}, {0xff66, 0xff9d}, {0xffa0, 0xffbe}, {0xffc2, 0xffc7}, {0xffca, 0xffcf},
  {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d}, {0x1003f, 0x1004d}, {0x10050, 0x1005d},
  {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x10300, 0x1031f}, {0x1032d, 0x1034a}, {0x10350, 0x10375}, {0x10380, 0x1039d},
  {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104b0, 0x104d3}, {0x104d8, 0x104fb}, {0x10500, 0x10527}, {0x10530, 0x10563},
  {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595}, {0x10597, 0x105a1}, {0x105a3, 0x105b1}, {0x105b3, 0x105b9}, {0x105bb, 0x105bc},
  {0x105c0, 0x105f3}, {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767}, {0x10780, 0x10785}, {0x10787, 0x107b0}, {0x107b2, 0x107ba}, {0x10800, 0x10805},
  {0x10808, 0x10808}, {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c}, {0x1083f, 0x10855}, {0x10860, 0x10876}, {0x10880, 0x1089e}, {0x108e0, 0x108f2},
  {0x108f4, 0x108f5}, {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7}, {0x109be, 0x109bf}, {0x10a00, 0x10a00}, {0x10a10, 0x10a13}, {0x10a15, 0x10a17},
  {0x10a19, 0x10a35}, {0x10a60, 0x10a7c}, {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae4}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55}, {0x10b60, 0x10b72},
  {0x10b80, 0x10b91}, {0x10c00, 0x10c48}, {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d23}, {0x10d4a, 0x10d65}, {0x10d6f, 0x10d85}, {0x10e80, 0x10ea9},
  {0x10eb0, 0x10eb1}, {0x10ec2, 0x10ec4}, {0x10f00, 0x10f1c}, {0x10f27, 0x10f27}, {0x10f30, 0x10f45}, {0x10f70, 0x10f81}, {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6},
  {0x11003, 0x11037}, {0x11071, 0x11072}, {0x11075, 0x11075}, {0x11083, 0x110af}, {0x110d0, 0x110e8}, {0x11103, 0x11126}, {0x11144, 0x11144}, {0x11147, 0x11147},
  {0x11150, 0x11172}, {0x11176, 0x11176}, {0x11183, 0x111b2}, {0x111c1, 0x111c4}, {0x111da, 0x111da}, {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x1122b},
  {0x1123f, 0x11240}, {0x11280, 0x11286}, {0x11288, 0x11288}, {0x1128a, 0x1128d}, {0x1128f, 0x1129d}, {0x1129f, 0x112a8}, {0x112b0, 0x112de}, {0x11305, 0x1130c},
  {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330}, {0x11332, 0x11333}, {0x11335, 0x11339}, {0x1133d, 0x1133d}, {0x11350, 0x11350}, {0x1135d, 0x11361},
  {0x11380, 0x11389}, {0x1138b, 0x1138b}, {0x1138e, 0x1138e}, {0x11390, 0x113b5}, {0x113b7, 0x113b7}, {0x113d1, 0x113d1}, {0x113d3, 0x113d3}, {0x11400, 0x11434},
  {0x11447, 0x1144a}, {0x1145f, 0x11461}, {0x11480, 0x114af}, {0x114c4, 0x114c5}, {0x114c7, 0x114c7}, {0x11580, 0x115ae}, {0x115d8, 0x115db}, {0x11600, 0x1162f},
  {0x11644, 0x11644}, {0x11680, 0x116aa}, {0x116b8, 0x116b8}, {0x11700, 0x1171a}, {0x11740, 0x11746}, {0x11800, 0x1182b}, {0x118a0, 0x118df}, {0x118ff, 0x11906},
  {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916}, {0x11918, 0x1192f}, {0x1193f, 0x1193f}, {0x11941, 0x11941}, {0x119a0, 0x119a7}, {0x119aa, 0x119d0},
  {0x119e1, 0x119e1}, {0x119e3, 0x119e3}, {0x11a00, 0x11a00}, {0x11a0b, 0x11a32}, {0x11a3a, 0x11a3a}, {0x11a50, 0x11a50}, {0x11a5c, 0x11a89}, {0x11a9d, 0x11a9d},
  {0x11ab0, 0x11af8}, {0x11bc0, 0x11be0}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c2e}, {0x11c40, 0x11c40}, {0x11c72, 0x11c8f}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09},
  {0x11d0b, 0x11d30}, {0x11d46, 0x11d46}, {0x11d60, 0x11d65}, {0x11d67, 0x11d68}, {0x11d6a, 0x11d89}, {0x11d98, 0x11d98}, {0x11ee0, 0x11ef2}, {0x11f02, 0x11f02},
  {0x11f04, 0x11f10}, {0x11f12, 0x11f33}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399}, {0x12400, 0x1246e}, {0x12480, 0x12543}, {0x12f90, 0x12ff0}, {0x13000, 0x1342f},
  {0x13441, 0x13446}, {0x13460, 0x143fa}, {0x14400, 0x14646}, {0x16100, 0x1611d}, {0x16800, 0x16a38}, {0x16a40, 0x16a5e}, {0x16a70, 0x16abe}, {0x16ad0, 0x16aed},
  {0x16b00, 0x16b2f}, {0x16b40, 0x16b43}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f}, {0x16d40, 0x16d6c}, {0x16e40, 0x16e7f}, {0x16f00, 0x16f4a}, {0x16f50, 0x16f50},
  {0x16f93, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe3}, {0x17000, 0x187f7}, {0x18800, 0x18cd5}, {0x18cff, 0x18d08}, {0x1aff0, 0x1aff3}, {0x1aff5, 0x1affb},
  {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155}, {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb}, {0x1bc00, 0x1bc6a},
  {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1d400, 0x1d454}, {0x1d456, 0x1d49c}, {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2}, {0x1d4a5, 0x1d4a6},
  {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3}, {0x1d4c5, 0x1d505}, {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514}, {0x1d516, 0x1d51c},
  {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546}, {0x1d54a, 0x1d550}, {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0}, {0x1d6c2, 0x1d6da},
  {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e}, {0x1d750, 0x1d76e}, {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8}, {0x1d7aa, 0x1d7c2},
  {0x1d7c4, 0x1d7cb}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e030, 0x1e06d}, {0x1e100, 0x1e12c}, {0x1e137, 0x1e13d}, {0x1e14e, 0x1e14e}, {0x1e290, 0x1e2ad},
  {0x1e2c0, 0x1e2eb}, {0x1e4d0, 0x1e4eb}, {0x1e5d0, 0x1e5ed}, {0x1e5f0, 0x1e5f0}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb}, {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe},
  {0x1e800, 0x1e8c4}, {0x1e900, 0x1e943}, {0x1e94b, 0x1e94b}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24}, {0x1ee27, 0x1ee27},
  {0x1ee29, 0x1ee32}, {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49}, {0x1ee4b, 0x1ee4b},
  {0x1ee4d, 0x1ee4f}, {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d}, {0x1ee5f, 0x1ee5f},
  {0x1ee61, 0x1ee62}, {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e}, {0x1ee80, 0x1ee89},
  {0x1ee8b, 0x1ee9b}, {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739}, {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1},
  {0x2ceb0, 0x2ebe0}, {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af},
};

static const TSCharacterRange sym_identifier_character_set_3[] = {
  {'0', '9'}, {'A', 'Z'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xb7, 0xb7}, {0xba, 0xba},
  {0xc0, 0xd6}, {0xd8, 0xf6}, {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x300, 0x374},
  {0x376, 0x377}, {0x37b, 0x37d}, {0x37f, 0x37f}, {0x386, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5}, {0x3f7, 0x481},
  {0x483, 0x487}, {0x48a, 0x52f}, {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x591, 0x5bd}, {0x5bf, 0x5bf}, {0x5c1, 0x5c2},
  {0x5c4, 0x5c5}, {0x5c7, 0x5c7}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x610, 0x61a}, {0x620, 0x669}, {0x66e, 0x6d3}, {0x6d5, 0x6dc},
  {0x6df, 0x6e8}, {0x6ea, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x74a}, {0x74d, 0x7b1}, {0x7c0, 0x7f5}, {0x7fa, 0x7fa}, {0x7fd, 0x7fd},
  {0x800, 0x82d}, {0x840, 0x85b}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x897, 0x8e1}, {0x8e3, 0x963}, {0x966, 0x96f},
  {0x971, 0x983}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0}, {0x9b2, 0x9b2}, {0x9b6, 0x9b9}, {0x9bc, 0x9c4},
  {0x9c7, 0x9c8}, {0x9cb, 0x9ce}, {0x9d7, 0x9d7}, {0x9dc, 0x9dd}, {0x9df, 0x9e3}, {0x9e6, 0x9f1}, {0x9fc, 0x9fc}, {0x9fe, 0x9fe},
  {0xa01, 0xa03}, {0xa05, 0xa0a}, {0xa0f, 0xa10}, {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39},
  {0xa3c, 0xa3c}, {0xa3e, 0xa42}, {0xa47, 0xa48}, {0xa4b, 0xa4d}, {0xa51, 0xa51}, {0xa59, 0xa5c}, {0xa5e, 0xa5e}, {0xa66, 0xa75},
  {0xa81, 0xa83}, {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9}, {0xabc, 0xac5},
  {0xac7, 0xac9}, {0xacb, 0xacd}, {0xad0, 0xad0}, {0xae0, 0xae3}, {0xae6, 0xaef}, {0xaf9, 0xaff}, {0xb01, 0xb03}, {0xb05, 0xb0c},
  {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30}, {0xb32, 0xb33}, {0xb35, 0xb39}, {0xb3c, 0xb44}, {0xb47, 0xb48}, {0xb4b, 0xb4d},
  {0xb55, 0xb57}, {0xb5c, 0xb5d}, {0xb5f, 0xb63}, {0xb66, 0xb6f}, {0xb71, 0xb71}, {0xb82, 0xb83}, {0xb85, 0xb8a}, {0xb8e, 0xb90},
  {0xb92, 0xb95}, {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9}, {0xbbe, 0xbc2},
  {0xbc6, 0xbc8}, {0xbca, 0xbcd}, {0xbd0, 0xbd0}, {0xbd7, 0xbd7}, {0xbe6, 0xbef}, {0xc00, 0xc0c}, {0xc0e, 0xc10}, {0xc12, 0xc28},
  {0xc2a, 0xc39}, {0xc3c, 0xc44}, {0xc46, 0xc48}, {0xc4a, 0xc4d}, {0xc55, 0xc56}, {0xc58, 0xc5a}, {0xc5d, 0xc5d}, {0xc60, 0xc63},
  {0xc66, 0xc6f}, {0xc80, 0xc83}, {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbc, 0xcc4},
  {0xcc6, 0xcc8}, {0xcca, 0xccd}, {0xcd5, 0xcd6}, {0xcdd, 0xcde}, {0xce0, 0xce3}, {0xce6, 0xcef}, {0xcf1, 0xcf3}, {0xd00, 0xd0c},
  {0xd0e, 0xd10}, {0xd12, 0xd44}, {0xd46, 0xd48}, {0xd4a, 0xd4e}, {0xd54, 0xd57}, {0xd5f, 0xd63}, {0xd66, 0xd6f}, {0xd7a, 0xd7f},
  {0xd81, 0xd83}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6}, {0xdca, 0xdca}, {0xdcf, 0xdd4},
  {0xdd6, 0xdd6}, {0xdd8, 0xddf}, {0xde6, 0xdef}, {0xdf2, 0xdf3}, {0xe01, 0xe3a}, {0xe40, 0xe4e}, {0xe50, 0xe59}, {0xe81, 0xe82},
  {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5}, {0xea7, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xec8, 0xece},
  {0xed0, 0xed9}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf18, 0xf19}, {0xf20, 0xf29}, {0xf35, 0xf35}, {0xf37, 0xf37}, {0xf39, 0xf39},
  {0xf3e, 0xf47}, {0xf49, 0xf6c}, {0xf71, 0xf84}, {0xf86, 0xf97}, {0xf99, 0xfbc}, {0xfc6, 0xfc6}, {0x1000, 0x1049}, {0x1050, 0x109d},
  {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248}, {0x124a, 0x124d}, {0x1250, 0x1256}, {0x1258, 0x1258},
  {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5}, {0x12b8, 0x12be}, {0x12c0, 0x12c0}, {0x12c2, 0x12c5},
  {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x135d, 0x135f}, {0x1369, 0x1371}, {0x1380, 0x138f}, {0x13a0, 0x13f5},
  {0x13f8, 0x13fd}, {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1715}, {0x171f, 0x1734},
  {0x1740, 0x1753}, {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1772, 0x1773}, {0x1780, 0x17d3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dd}, {0x17e0, 0x17e9},
  {0x180b, 0x180d}, {0x180f, 0x1819}, {0x1820, 0x1878}, {0x1880, 0x18aa}, {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1920, 0x192b}, {0x1930, 0x193b},
  {0x1946, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9}, {0x19d0, 0x19da}, {0x1a00, 0x1a1b}, {0x1a20, 0x1a5e}, {0x1a60, 0x1a7c},
  {0x1a7f, 0x1a89}, {0x1a90, 0x1a99}, {0x1aa7, 0x1aa7}, {0x1ab0, 0x1abd}, {0x1abf, 0x1ace}, {0x1b00, 0x1b4c}, {0x1b50, 0x1b59}, {0x1b6b, 0x1b73},
  {0x1b80, 0x1bf3}, {0x1c00, 0x1c37}, {0x1c40, 0x1c49}, {0x1c4d, 0x1c7d}, {0x1c80, 0x1c8a}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1cd0, 0x1cd2},
  {0x1cd4, 0x1cfa}, {0x1d00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57}, {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b},
  {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4}, {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3},
  {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x200c, 0x200d}, {0x203f, 0x2040}, {0x2054, 0x2054}, {0x2071, 0x2071},
  {0x207f, 0x207f}, {0x2090, 0x209c}, {0x20d0, 0x20dc}, {0x20e1, 0x20e1}, {0x20e5, 0x20f0}, {0x2102, 0x2102}, {0x2107, 0x2107}, {0x210a, 0x2113},
  {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126}, {0x2128, 0x2128}, {0x212a, 0x2139}, {0x213c, 0x213f}, {0x2145, 0x2149},
  {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cf3}, {0x2d00, 0x2d25}, {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67},
  {0x2d6f, 0x2d6f}, {0x2d7f, 0x2d96}, {0x2da0, 0x2da6}, {0x2da8, 0x2dae}, {0x2db0, 0x2db6}, {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce},
  {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x2de0, 0x2dff}, {0x3005, 0x3007}, {0x3021, 0x302f}, {0x3031, 0x3035}, {0x3038, 0x303c}, {0x3041, 0x3096},
  {0x3099, 0x309a}, {0x309d, 0x309f}, {0x30a1, 0x30ff}, {0x3105, 0x312f}, {0x3131, 0x318e}, {0x31a0, 0x31bf}, {0x31f0, 0x31ff}, {0x3400, 0x4dbf},
  {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa62b}, {0xa640, 0xa66f}, {0xa674, 0xa67d}, {0xa67f, 0xa6f1}, {0xa717, 0xa71f},
  {0xa722, 0xa788}, {0xa78b, 0xa7cd}, {0xa7d0, 0xa7d1}, {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7dc}, {0xa7f2, 0xa827}, {0xa82c, 0xa82c}, {0xa840, 0xa873},
  {0xa880, 0xa8c5}, {0xa8d0, 0xa8d9}, {0xa8e0, 0xa8f7}, {0xa8fb, 0xa8fb}, {0xa8fd, 0xa92d}, {0xa930, 0xa953}, {0xa960, 0xa97c}, {0xa980, 0xa9c0},
  {0xa9cf, 0xa9d9}, {0xa9e0, 0xa9fe}, {0xaa00, 0xaa36}, {0xaa40, 0xaa4d}, {0xaa50, 0xaa59}, {0xaa60, 0xaa76}, {0xaa7a, 0xaac2}, {0xaadb, 0xaadd},
  {0xaae0, 0xaaef}, {0xaaf2, 0xaaf6}, {0xab01, 0xab06}, {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a},
  {0xab5c, 0xab69}, {0xab70, 0xabea}, {0xabec, 0xabed}, {0xabf0, 0xabf9}, {0xac00, 0xd7a3}, {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d},
  {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17}, {0xfb1d, 0xfb28}, {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41},
  {0xfb43, 0xfb44}, {0xfb46, 0xfbb1}, {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d}, {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe00, 0xfe0f},
  {0xfe20, 0xfe2f}, {0xfe33, 0xfe34}, {0xfe4d, 0xfe4f}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77}, {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b},
  {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff10, 0xff19}, {0xff21, 0xff3a}, {0xff3f, 0xff3f}, {0xff41, 0xff5a}, {0xff65, 0xffbe}, {0xffc2, 0xffc7},
  {0xffca, 0xffcf}, {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d}, {0x1003f, 0x1004d},
  {0x10050, 0x1005d}, {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x101fd, 0x101fd}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x102e0, 0x102e0}, {0x10300, 0x1031f},
  {0x1032d, 0x1034a}, {0x10350, 0x1037a}, {0x10380, 0x1039d}, {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104a0, 0x104a9},
  {0x104b0, 0x104d3}, {0x104d8, 0x104fb}, {0x10500, 0x10527}, {0x10530, 0x10563}, {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595},
  {0x10597, 0x105a1}, {0x105a3, 0x105b1}, {0x105b3, 0x105b9}, {0x105bb, 0x105bc}, {0x105c0, 0x105f3}, {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767},
  {0x10780, 0x10785}, {0x10787, 0x107b0}, {0x107b2, 0x107ba}, {0x10800, 0x10805}, {0x10808, 0x10808}, {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c},
  {0x1083f, 0x10855}, {0x10860, 0x10876}, {0x10880, 0x1089e}, {0x108e0, 0x108f2}, {0x108f4, 0x108f5}, {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7},
  {0x109be, 0x109bf}, {0x10a00, 0x10a03}, {0x10a05, 0x10a06}, {0x10a0c, 0x10a13}, {0x10a15, 0x10a17}, {0x10a19, 0x10a35}, {0x10a38, 0x10a3a}, {0x10a3f, 0x10a3f},
  {0x10a60, 0x10a7c}, {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae6}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55}, {0x10b60, 0x10b72}, {0x10b80, 0x10b91},
  {0x10c00, 0x10c48}, {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d27}, {0x10d30, 0x10d39}, {0x10d40, 0x10d65}, {0x10d69, 0x10d6d}, {0x10d6f, 0x10d85},
  {0x10e80, 0x10ea9}, {0x10eab, 0x10eac}, {0x10eb0, 0x10eb1}, {0x10ec2, 0x10ec4}, {0x10efc, 0x10f1c}, {0x10f27, 0x10f27}, {0x10f30, 0x10f50}, {0x10f70, 0x10f85},
  {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11000, 0x11046}, {0x11066, 0x11075}, {0x1107f, 0x110ba}, {0x110c2, 0x110c2}, {0x110d0, 0x110e8}, {0x110f0, 0x110f9},
  {0x11100, 0x11134}, {0x11136, 0x1113f}, {0x11144, 0x11147}, {0x11150, 0x11173}, {0x11176, 0x11176}, {0x11180, 0x111c4}, {0x111c9, 0x111cc}, {0x111ce, 0x111da},
  {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x11237}, {0x1123e, 0x11241}, {0x11280, 0x11286}, {0x11288, 0x11288}, {0x1128a, 0x1128d}, {0x1128f, 0x1129d},
  {0x1129f, 0x112a8}, {0x112b0, 0x112ea}, {0x112f0, 0x112f9}, {0x11300, 0x11303}, {0x11305, 0x1130c}, {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330},
  {0x11332, 0x11333}, {0x11335, 0x11339}, {0x1133b, 0x11344}, {0x11347, 0x11348}, {0x1134b, 0x1134d}, {0x11350, 0x11350}, {0x11357, 0x11357}, {0x1135d, 0x11363},
  {0x11366, 0x1136c}, {0x11370, 0x11374}, {0x11380, 0x11389}, {0x1138b, 0x1138b}, {0x1138e, 0x1138e}, {0x11390, 0x113b5}, {0x113b7, 0x113c0}, {0x113c2, 0x113c2},
  {0x113c5, 0x113c5}, {0x113c7, 0x113ca}, {0x113cc, 0x113d3}, {0x113e1, 0x113e2}, {0x11400, 0x1144a}, {0x11450, 0x11459}, {0x1145e, 0x11461}, {0x11480, 0x114c5},
  {0x114c7, 0x114c7}, {0x114d0, 0x114d9}, {0x11580, 0x115b5}, {0x115b8, 0x115c0}, {0x115d8, 0x115dd}, {0x11600, 0x11640}, {0x11644, 0x11644}, {0x11650, 0x11659},
  {0x11680, 0x116b8}, {0x116c0, 0x116c9}, {0x116d0, 0x116e3}, {0x11700, 0x1171a}, {0x1171d, 0x1172b}, {0x11730, 0x11739}, {0x11740, 0x11746}, {0x11800, 0x1183a},
  {0x118a0, 0x118e9}, {0x118ff, 0x11906}, {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916}, {0x11918, 0x11935}, {0x11937, 0x11938}, {0x1193b, 0x11943},
  {0x11950, 0x11959}, {0x119a0, 0x119a7}, {0x119aa, 0x119d7}, {0x119da, 0x119e1}, {0x119e3, 0x119e4}, {0x11a00, 0x11a3e}, {0x11a47, 0x11a47}, {0x11a50, 0x11a99},
  {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8}, {0x11bc0, 0x11be0}, {0x11bf0, 0x11bf9}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c36}, {0x11c38, 0x11c40}, {0x11c50, 0x11c59},
  {0x11c72, 0x11c8f}, {0x11c92, 0x11ca7}, {0x11ca9, 0x11cb6}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d36}, {0x11d3a, 0x11d3a}, {0x11d3c, 0x11d3d},
  {0x11d3f, 0x11d47}, {0x11d50, 0x11d59}, {0x11d60, 0x11d65}, {0x11d67, 0x11d68}, {0x11d6a, 0x11d8e}, {0x11d90, 0x11d91}, {0x11d93, 0x11d98}, {0x11da0, 0x11da9},
  {0x11ee0, 0x11ef6}, {0x11f00, 0x11f10}, {0x11f12, 0x11f3a}, {0x11f3e, 0x11f42}, {0x11f50, 0x11f5a}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399}, {0x12400, 0x1246e},
  {0x12480, 0x12543}, {0x12f90, 0x12ff0}, {0x13000, 0x1342f}, {0x13440, 0x13455}, {0x13460, 0x143fa}, {0x14400, 0x14646}, {0x16100, 0x16139}, {0x16800, 0x16a38},
  {0x16a40, 0x16a5e}, {0x16a60, 0x16a69}, {0x16a70, 0x16abe}, {0x16ac0, 0x16ac9}, {0x16ad0, 0x16aed}, {0x16af0, 0x16af4}, {0x16b00, 0x16b36}, {0x16b40, 0x16b43},
  {0x16b50, 0x16b59}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f}, {0x16d40, 0x16d6c}, {0x16d70, 0x16d79}, {0x16e40, 0x16e7f}, {0x16f00, 0x16f4a}, {0x16f4f, 0x16f87},
  {0x16f8f, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe4}, {0x16ff0, 0x16ff1}, {0x17000, 0x187f7}, {0x18800, 0x18cd5}, {0x18cff, 0x18d08}, {0x1aff0, 0x1aff3},
  {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155}, {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb},
  {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1bc9d, 0x1bc9e}, {0x1ccf0, 0x1ccf9}, {0x1cf00, 0x1cf2d}, {0x1cf30, 0x1cf46},
  {0x1d165, 0x1d169}, {0x1d16d, 0x1d172}, {0x1d17b, 0x1d182}, {0x1d185, 0x1d18b}, {0x1d1aa, 0x1d1ad}, {0x1d242, 0x1d244}, {0x1d400, 0x1d454}, {0x1d456, 0x1d49c},
  {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2}, {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3}, {0x1d4c5, 0x1d505},
  {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514}, {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546}, {0x1d54a, 0x1d550},
  {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0}, {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e}, {0x1d750, 0x1d76e},
  {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8}, {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1d7ce, 0x1d7ff}, {0x1da00, 0x1da36}, {0x1da3b, 0x1da6c}, {0x1da75, 0x1da75},
  {0x1da84, 0x1da84}, {0x1da9b, 0x1da9f}, {0x1daa1, 0x1daaf}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e000, 0x1e006}, {0x1e008, 0x1e018}, {0x1e01b, 0x1e021},
  {0x1e023, 0x1e024}, {0x1e026, 0x1e02a}, {0x1e030, 0x1e06d}, {0x1e08f, 0x1e08f}, {0x1e100, 0x1e12c}, {0x1e130, 0x1e13d}, {0x1e140, 0x1e149}, {0x1e14e, 0x1e14e},
  {0x1e290, 0x1e2ae}, {0x1e2c0, 0x1e2f9}, {0x1e4d0, 0x1e4f9}, {0x1e5d0, 0x1e5fa}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb}, {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe},
  {0x1e800, 0x1e8c4}, {0x1e8d0, 0x1e8d6}, {0x1e900, 0x1e94b}, {0x1e950, 0x1e959}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24},
  {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32}, {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49},
  {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f}, {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d},
  {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62}, {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e},
  {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b}, {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x1fbf0, 0x1fbf9}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739},
  {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0}, {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af}, {0xe0100, 0xe01ef},
};

static const TSCharacterRange aux_sym_metavariable_token1_character_set_1[] = {
  {'\t', '\r'}, {' ', ' '}, {'A', 'Z'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xba, 0xba},
  {0xc0, 0xd6}, {0xd8, 0xf6}, {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x370, 0x374},
  {0x376, 0x377}, {0x37b, 0x37d}, {0x37f, 0x37f}, {0x386, 0x386}, {0x388, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5},
  {0x3f7, 0x481}, {0x48a, 0x52f}, {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x620, 0x64a},
  {0x66e, 0x66f}, {0x671, 0x6d3}, {0x6d5, 0x6d5}, {0x6e5, 0x6e6}, {0x6ee, 0x6ef}, {0x6fa, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x710},
  {0x712, 0x72f}, {0x74d, 0x7a5}, {0x7b1, 0x7b1}, {0x7ca, 0x7ea}, {0x7f4, 0x7f5}, {0x7fa, 0x7fa}, {0x800, 0x815}, {0x81a, 0x81a},
  {0x824, 0x824}, {0x828, 0x828}, {0x840, 0x858}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x8a0, 0x8c9}, {0x904, 0x939},
  {0x93d, 0x93d}, {0x950, 0x950}, {0x958, 0x961}, {0x971, 0x980}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0},
  {0x9b2, 0x9b2}, {0x9b6, 0x9b9}, {0x9bd, 0x9bd}, {0x9ce, 0x9ce}, {0x9dc, 0x9dd}, {0x9df, 0x9e1}, {0x9f0, 0x9f1}, {0x9fc, 0x9fc},
  {0xa05, 0xa0a}, {0xa0f, 0xa10}, {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39}, {0xa59, 0xa5c},
  {0xa5e, 0xa5e}, {0xa72, 0xa74}, {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9},
  {0xabd, 0xabd}, {0xad0, 0xad0}, {0xae0, 0xae1}, {0xaf9, 0xaf9}, {0xb05, 0xb0c}, {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30},
  {0xb32, 0xb33}, {0xb35, 0xb39}, {0xb3d, 0xb3d}, {0xb5c, 0xb5d}, {0xb5f, 0xb61}, {0xb71, 0xb71}, {0xb83, 0xb83}, {0xb85, 0xb8a},
  {0xb8e, 0xb90}, {0xb92, 0xb95}, {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9},
  {0xbd0, 0xbd0}, {0xc05, 0xc0c}, {0xc0e, 0xc10}, {0xc12, 0xc28}, {0xc2a, 0xc39}, {0xc3d, 0xc3d}, {0xc58, 0xc5a}, {0xc5d, 0xc5d},
  {0xc60, 0xc61}, {0xc80, 0xc80}, {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbd, 0xcbd},
  {0xcdd, 0xcde}, {0xce0, 0xce1}, {0xcf1, 0xcf2}, {0xd04, 0xd0c}, {0xd0e, 0xd10}, {0xd12, 0xd3a}, {0xd3d, 0xd3d}, {0xd4e, 0xd4e},
  {0xd54, 0xd56}, {0xd5f, 0xd61}, {0xd7a, 0xd7f}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6},
  {0xe01, 0xe30}, {0xe32, 0xe32}, {0xe40, 0xe46}, {0xe81, 0xe82}, {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5},
  {0xea7, 0xeb0}, {0xeb2, 0xeb2}, {0xebd, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf40, 0xf47},
  {0xf49, 0xf6c}, {0xf88, 0xf8c}, {0x1000, 0x102a}, {0x103f, 0x103f}, {0x1050, 0x1055}, {0x105a, 0x105d}, {0x1061, 0x1061}, {0x1065, 0x1066},
  {0x106e, 0x1070}, {0x1075, 0x1081}, {0x108e, 0x108e}, {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248},
  {0x124a, 0x124d}, {0x1250, 0x1256}, {0x1258, 0x1258}, {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5},
  {0x12b8, 0x12be}, {0x12c0, 0x12c0}, {0x12c2, 0x12c5}, {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x1380, 0x138f},
  {0x13a0, 0x13f5}, {0x13f8, 0x13fd}, {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1711},
  {0x171f, 0x1731}, {0x1740, 0x1751}, {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1780, 0x17b3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dc}, {0x1820, 0x1878},
  {0x1880, 0x18a8}, {0x18aa, 0x18aa}, {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1950, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9},
  {0x1a00, 0x1a16}, {0x1a20, 0x1a54}, {0x1aa7, 0x1aa7}, {0x1b05, 0x1b33}, {0x1b45, 0x1b4c}, {0x1b83, 0x1ba0}, {0x1bae, 0x1baf}, {0x1bba, 0x1be5},
  {0x1c00, 0x1c23}, {0x1c4d, 0x1c4f}, {0x1c5a, 0x1c7d}, {0x1c80, 0x1c8a}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1ce9, 0x1cec}, {0x1cee, 0x1cf3},
  {0x1cf5, 0x1cf6}, {0x1cfa, 0x1cfa}, {0x1d00, 0x1dbf}, {0x1e00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57},
  {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b}, {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4},
  {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3}, {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x2071, 0x2071}, {0x207f, 0x207f},
  {0x2090, 0x209c}, {0x2102, 0x2102}, {0x2107, 0x2107}, {0x210a, 0x2113}, {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126},
  {0x2128, 0x2128}, {0x212a, 0x2139}, {0x213c, 0x213f}, {0x2145, 0x2149}, {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cee},
  {0x2cf2, 0x2cf3}, {0x2d00, 0x2d25}, {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67}, {0x2d6f, 0x2d6f}, {0x2d80, 0x2d96}, {0x2da0, 0x2da6},
  {0x2da8, 0x2dae}, {0x2db0, 0x2db6}, {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce}, {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x3005, 0x3007},
  {0x3021, 0x3029}, {0x3031, 0x3035}, {0x3038, 0x303c}, {0x3041, 0x3096}, {0x309d, 0x309f}, {0x30a1, 0x30fa}, {0x30fc, 0x30ff}, {0x3105, 0x312f},
  {0x3131, 0x318e}, {0x31a0, 0x31bf}, {0x31f0, 0x31ff}, {0x3400, 0x4dbf}, {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa61f},
  {0xa62a, 0xa62b}, {0xa640, 0xa66e}, {0xa67f, 0xa69d}, {0xa6a0, 0xa6ef}, {0xa717, 0xa71f}, {0xa722, 0xa788}, {0xa78b, 0xa7cd}, {0xa7d0, 0xa7d1},
  {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7dc}, {0xa7f2, 0xa801}, {0xa803, 0xa805}, {0xa807, 0xa80a}, {0xa80c, 0xa822}, {0xa840, 0xa873}, {0xa882, 0xa8b3},
  {0xa8f2, 0xa8f7}, {0xa8fb, 0xa8fb}, {0xa8fd, 0xa8fe}, {0xa90a, 0xa925}, {0xa930, 0xa946}, {0xa960, 0xa97c}, {0xa984, 0xa9b2}, {0xa9cf, 0xa9cf},
  {0xa9e0, 0xa9e4}, {0xa9e6, 0xa9ef}, {0xa9fa, 0xa9fe}, {0xaa00, 0xaa28}, {0xaa40, 0xaa42}, {0xaa44, 0xaa4b}, {0xaa60, 0xaa76}, {0xaa7a, 0xaa7a},
  {0xaa7e, 0xaaaf}, {0xaab1, 0xaab1}, {0xaab5, 0xaab6}, {0xaab9, 0xaabd}, {0xaac0, 0xaac0}, {0xaac2, 0xaac2}, {0xaadb, 0xaadd}, {0xaae0, 0xaaea},
  {0xaaf2, 0xaaf4}, {0xab01, 0xab06}, {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a}, {0xab5c, 0xab69},
  {0xab70, 0xabe2}, {0xac00, 0xd7a3}, {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d}, {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17},
  {0xfb1d, 0xfb1d}, {0xfb1f, 0xfb28}, {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41}, {0xfb43, 0xfb44}, {0xfb46, 0xfbb1},
  {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d}, {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77},
  {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b}, {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff21, 0xff3a}, {0xff41, 0xff5a}, {0xff66, 0xff9d}, {0xffa0, 0xffbe},
  {0xffc2, 0xffc7}, {0xffca, 0xffcf}, {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d},
  {0x1003f, 0x1004d}, {0x10050, 0x1005d}, {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x10300, 0x1031f}, {0x1032d, 0x1034a},
  {0x10350, 0x10375}, {0x10380, 0x1039d}, {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104b0, 0x104d3}, {0x104d8, 0x104fb},
  {0x10500, 0x10527}, {0x10530, 0x10563}, {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595}, {0x10597, 0x105a1}, {0x105a3, 0x105b1},
  {0x105b3, 0x105b9}, {0x105bb, 0x105bc}, {0x105c0, 0x105f3}, {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767}, {0x10780, 0x10785}, {0x10787, 0x107b0},
  {0x107b2, 0x107ba}, {0x10800, 0x10805}, {0x10808, 0x10808}, {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c}, {0x1083f, 0x10855}, {0x10860, 0x10876},
  {0x10880, 0x1089e}, {0x108e0, 0x108f2}, {0x108f4, 0x108f5}, {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7}, {0x109be, 0x109bf}, {0x10a00, 0x10a00},
  {0x10a10, 0x10a13}, {0x10a15, 0x10a17}, {0x10a19, 0x10a35}, {0x10a60, 0x10a7c}, {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae4}, {0x10b00, 0x10b35},
  {0x10b40, 0x10b55}, {0x10b60, 0x10b72}, {0x10b80, 0x10b91}, {0x10c00, 0x10c48}, {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d23}, {0x10d4a, 0x10d65},
  {0x10d6f, 0x10d85}, {0x10e80, 0x10ea9}, {0x10eb0, 0x10eb1}, {0x10ec2, 0x10ec4}, {0x10f00, 0x10f1c}, {0x10f27, 0x10f27}, {0x10f30, 0x10f45}, {0x10f70, 0x10f81},
  {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11003, 0x11037}, {0x11071, 0x11072}, {0x11075, 0x11075}, {0x11083, 0x110af}, {0x110d0, 0x110e8}, {0x11103, 0x11126},
  {0x11144, 0x11144}, {0x11147, 0x11147}, {0x11150, 0x11172}, {0x11176, 0x11176}, {0x11183, 0x111b2}, {0x111c1, 0x111c4}, {0x111da, 0x111da}, {0x111dc, 0x111dc},
  {0x11200, 0x11211}, {0x11213, 0x1122b}, {0x1123f, 0x11240}, {0x11280, 0x11286}, {0x11288, 0x11288}, {0x1128a, 0x1128d}, {0x1128f, 0x1129d}, {0x1129f, 0x112a8},
  {0x112b0, 0x112de}, {0x11305, 0x1130c}, {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330}, {0x11332, 0x11333}, {0x11335, 0x11339}, {0x1133d, 0x1133d},
  {0x11350, 0x11350}, {0x1135d, 0x11361}, {0x11380, 0x11389}, {0x1138b, 0x1138b}, {0x1138e, 0x1138e}, {0x11390, 0x113b5}, {0x113b7, 0x113b7}, {0x113d1, 0x113d1},
  {0x113d3, 0x113d3}, {0x11400, 0x11434}, {0x11447, 0x1144a}, {0x1145f, 0x11461}, {0x11480, 0x114af}, {0x114c4, 0x114c5}, {0x114c7, 0x114c7}, {0x11580, 0x115ae},
  {0x115d8, 0x115db}, {0x11600, 0x1162f}, {0x11644, 0x11644}, {0x11680, 0x116aa}, {0x116b8, 0x116b8}, {0x11700, 0x1171a}, {0x11740, 0x11746}, {0x11800, 0x1182b},
  {0x118a0, 0x118df}, {0x118ff, 0x11906}, {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916}, {0x11918, 0x1192f}, {0x1193f, 0x1193f}, {0x11941, 0x11941},
  {0x119a0, 0x119a7}, {0x119aa, 0x119d0}, {0x119e1, 0x119e1}, {0x119e3, 0x119e3}, {0x11a00, 0x11a00}, {0x11a0b, 0x11a32}, {0x11a3a, 0x11a3a}, {0x11a50, 0x11a50},
  {0x11a5c, 0x11a89}, {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8}, {0x11bc0, 0x11be0}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c2e}, {0x11c40, 0x11c40}, {0x11c72, 0x11c8f},
  {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d30}, {0x11d46, 0x11d46}, {0x11d60, 0x11d65}, {0x11d67, 0x11d68}, {0x11d6a, 0x11d89}, {0x11d98, 0x11d98},
  {0x11ee0, 0x11ef2}, {0x11f02, 0x11f02}, {0x11f04, 0x11f10}, {0x11f12, 0x11f33}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399}, {0x12400, 0x1246e}, {0x12480, 0x12543},
  {0x12f90, 0x12ff0}, {0x13000, 0x1342f}, {0x13441, 0x13446}, {0x13460, 0x143fa}, {0x14400, 0x14646}, {0x16100, 0x1611d}, {0x16800, 0x16a38}, {0x16a40, 0x16a5e},
  {0x16a70, 0x16abe}, {0x16ad0, 0x16aed}, {0x16b00, 0x16b2f}, {0x16b40, 0x16b43}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f}, {0x16d40, 0x16d6c}, {0x16e40, 0x16e7f},
  {0x16f00, 0x16f4a}, {0x16f50, 0x16f50}, {0x16f93, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe3}, {0x17000, 0x187f7}, {0x18800, 0x18cd5}, {0x18cff, 0x18d08},
  {0x1aff0, 0x1aff3}, {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155}, {0x1b164, 0x1b167},
  {0x1b170, 0x1b2fb}, {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1d400, 0x1d454}, {0x1d456, 0x1d49c}, {0x1d49e, 0x1d49f},
  {0x1d4a2, 0x1d4a2}, {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3}, {0x1d4c5, 0x1d505}, {0x1d507, 0x1d50a},
  {0x1d50d, 0x1d514}, {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546}, {0x1d54a, 0x1d550}, {0x1d552, 0x1d6a5},
  {0x1d6a8, 0x1d6c0}, {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e}, {0x1d750, 0x1d76e}, {0x1d770, 0x1d788},
  {0x1d78a, 0x1d7a8}, {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e030, 0x1e06d}, {0x1e100, 0x1e12c}, {0x1e137, 0x1e13d},
  {0x1e14e, 0x1e14e}, {0x1e290, 0x1e2ad}, {0x1e2c0, 0x1e2eb}, {0x1e4d0, 0x1e4eb}, {0x1e5d0, 0x1e5ed}, {0x1e5f0, 0x1e5f0}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb},
  {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe}, {0x1e800, 0x1e8c4}, {0x1e900, 0x1e943}, {0x1e94b, 0x1e94b}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22},
  {0x1ee24, 0x1ee24}, {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32}, {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47},
  {0x1ee49, 0x1ee49}, {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f}, {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b},
  {0x1ee5d, 0x1ee5d}, {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62}, {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c},
  {0x1ee7e, 0x1ee7e}, {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b}, {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739},
  {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0}, {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af},
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(74);
      ADVANCE_MAP(
        '!', 168,
        '"', 159,
        '#', 144,
        '$', 89,
        '%', 103,
        '&', 106,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 100,
        '.', 135,
        '/', 102,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 148,
        '=', 125,
        '>', 129,
        '?', 97,
        '@', 134,
        '[', 84,
        '\\', 50,
        ']', 85,
        '^', 104,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
        '~', 43,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(70);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 1:
      if (lookahead == '\n') ADVANCE(175);
      if (lookahead == '[') ADVANCE(3);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(1);
      if (lookahead != 0) ADVANCE(3);
      END_STATE();
    case 2:
      if (lookahead == '\n') ADVANCE(175);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(1);
      if (lookahead != 0 &&
          lookahead != '[') ADVANCE(3);
      END_STATE();
    case 3:
      if (lookahead == '\n') ADVANCE(175);
      if (lookahead != 0) ADVANCE(3);
      END_STATE();
    case 4:
      ADVANCE_MAP(
        '!', 83,
        '"', 158,
        '#', 143,
        '$', 89,
        '%', 103,
        '&', 106,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 100,
        '.', 135,
        '/', 102,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 131,
        '=', 125,
        '>', 129,
        '?', 97,
        '@', 134,
        '[', 84,
        ']', 85,
        '^', 104,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(4);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 5:
      ADVANCE_MAP(
        '!', 83,
        '"', 158,
        '#', 143,
        '$', 58,
        '%', 103,
        '&', 106,
        '\'', 146,
        '(', 76,
        '*', 96,
        '+', 94,
        '-', 99,
        '.', 135,
        '/', 102,
        '0', 153,
        ':', 87,
        '<', 131,
        '=', 125,
        '>', 129,
        '?', 97,
        '[', 84,
        '^', 104,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 108,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(5);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 6:
      ADVANCE_MAP(
        '!', 83,
        '"', 158,
        '#', 143,
        '$', 88,
        '%', 103,
        '&', 106,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 100,
        '.', 135,
        '/', 102,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 131,
        '=', 125,
        '>', 129,
        '?', 97,
        '@', 134,
        '[', 84,
        ']', 85,
        '^', 104,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(6);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 7:
      ADVANCE_MAP(
        '!', 83,
        '%', 103,
        '&', 106,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 99,
        '.', 135,
        '/', 102,
        ':', 39,
        ';', 75,
        '<', 148,
        '=', 124,
        '>', 129,
        '?', 97,
        '[', 84,
        ']', 85,
        '^', 104,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(7);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 8:
      ADVANCE_MAP(
        '!', 83,
        '%', 103,
        '&', 106,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 99,
        '.', 135,
        '/', 102,
        ':', 39,
        ';', 75,
        '<', 131,
        '=', 124,
        '>', 129,
        '?', 97,
        '[', 84,
        ']', 85,
        '^', 104,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(8);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 9:
      ADVANCE_MAP(
        '!', 83,
        '%', 103,
        '&', 106,
        '\'', 145,
        '(', 76,
        '*', 96,
        '+', 94,
        '-', 99,
        '.', 135,
        '/', 102,
        ':', 39,
        '<', 131,
        '=', 125,
        '>', 129,
        '?', 97,
        '[', 84,
        '^', 104,
        'r', 173,
        '{', 78,
        '|', 108,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(9);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 10:
      ADVANCE_MAP(
        '!', 83,
        '%', 103,
        '&', 106,
        '(', 76,
        '*', 96,
        '+', 94,
        '-', 99,
        '.', 135,
        '/', 102,
        ':', 39,
        '<', 148,
        '=', 125,
        '>', 129,
        '?', 97,
        '[', 84,
        '^', 104,
        'r', 173,
        '|', 108,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(10);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 11:
      ADVANCE_MAP(
        '!', 82,
        '"', 159,
        '#', 143,
        '$', 58,
        '&', 105,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 95,
        '+', 93,
        ',', 80,
        '-', 41,
        '.', 29,
        '/', 27,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 130,
        '=', 123,
        '>', 128,
        '?', 97,
        '[', 84,
        '\\', 50,
        ']', 85,
        'r', 173,
        '{', 78,
        '|', 107,
        '}', 79,
        '~', 43,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(16);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 12:
      ADVANCE_MAP(
        '!', 82,
        '"', 158,
        '#', 143,
        '$', 58,
        '&', 105,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 95,
        '+', 93,
        ',', 80,
        '-', 98,
        '.', 29,
        '/', 27,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 130,
        '?', 97,
        '[', 84,
        ']', 85,
        'b', 171,
        'c', 172,
        'r', 173,
        '|', 107,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(12);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 13:
      ADVANCE_MAP(
        '!', 82,
        '"', 158,
        '#', 143,
        '$', 58,
        '&', 105,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 95,
        ',', 80,
        '-', 98,
        '.', 31,
        '/', 27,
        '0', 153,
        ':', 39,
        '<', 130,
        '?', 97,
        '[', 84,
        ']', 85,
        'b', 171,
        'c', 172,
        'r', 173,
        '|', 107,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(13);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 14:
      ADVANCE_MAP(
        '!', 82,
        '"', 158,
        '#', 143,
        '$', 58,
        '&', 105,
        '\'', 146,
        '(', 76,
        '*', 95,
        '+', 93,
        ',', 80,
        '-', 101,
        '.', 30,
        '/', 27,
        '0', 153,
        ':', 39,
        ';', 75,
        '<', 130,
        '[', 84,
        ']', 85,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 107,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(14);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 15:
      ADVANCE_MAP(
        '!', 82,
        '"', 158,
        '#', 143,
        '$', 58,
        '&', 105,
        '\'', 146,
        '(', 76,
        '*', 95,
        '-', 101,
        '.', 31,
        '/', 27,
        '0', 153,
        ':', 39,
        '<', 130,
        '[', 84,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 107,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(15);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 16:
      ADVANCE_MAP(
        '!', 82,
        '#', 143,
        '$', 58,
        '&', 105,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 95,
        '+', 93,
        ',', 80,
        '-', 41,
        '.', 29,
        '/', 27,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 130,
        '=', 123,
        '>', 128,
        '?', 97,
        '[', 84,
        ']', 85,
        'r', 173,
        '{', 78,
        '|', 107,
        '}', 79,
        '~', 43,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(16);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 17:
      ADVANCE_MAP(
        '!', 82,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 95,
        '+', 93,
        ',', 80,
        '.', 29,
        '/', 27,
        ':', 87,
        ';', 75,
        '<', 148,
        '=', 123,
        '>', 128,
        '@', 134,
        ']', 85,
        'r', 173,
        '{', 78,
        '|', 107,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(17);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 18:
      ADVANCE_MAP(
        '!', 82,
        '(', 76,
        ')', 77,
        '+', 93,
        ',', 80,
        '.', 29,
        '/', 27,
        ':', 87,
        '<', 148,
        '=', 42,
        '@', 134,
        'r', 173,
        '{', 78,
        '|', 107,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(18);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 19:
      ADVANCE_MAP(
        '!', 40,
        '%', 103,
        '&', 106,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 100,
        '.', 135,
        '/', 102,
        ':', 86,
        ';', 75,
        '<', 131,
        '=', 124,
        '>', 129,
        '?', 97,
        '[', 84,
        ']', 85,
        '^', 104,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(19);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 20:
      ADVANCE_MAP(
        '!', 40,
        '%', 103,
        '&', 106,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 99,
        '.', 135,
        '/', 102,
        ':', 86,
        ';', 75,
        '<', 131,
        '=', 124,
        '>', 129,
        '?', 97,
        '[', 84,
        ']', 85,
        '^', 104,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(20);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 21:
      ADVANCE_MAP(
        '!', 40,
        '%', 103,
        '&', 106,
        '(', 76,
        '*', 96,
        '+', 94,
        '-', 100,
        '.', 135,
        '/', 102,
        '<', 131,
        '=', 125,
        '>', 129,
        '?', 97,
        '[', 84,
        '^', 104,
        'r', 173,
        '|', 108,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(21);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 22:
      if (lookahead == '"') ADVANCE(158);
      if (lookahead == '/') ADVANCE(27);
      if (lookahead == ';') ADVANCE(75);
      if (lookahead == 'r') ADVANCE(173);
      if (lookahead == '{') ADVANCE(78);
      if (lookahead == 'b' ||
          lookahead == 'c') ADVANCE(172);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(22);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 23:
      ADVANCE_MAP(
        '#', 143,
        '\'', 145,
        '(', 76,
        ')', 77,
        '*', 32,
        '+', 93,
        ',', 80,
        '-', 41,
        '.', 30,
        '/', 27,
        '0', 153,
        ':', 86,
        ';', 75,
        '<', 148,
        '=', 123,
        '>', 128,
        ']', 85,
        'r', 173,
        '{', 78,
        '|', 107,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(23);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 24:
      if (lookahead == '\'') ADVANCE(160);
      END_STATE();
    case 25:
      if (lookahead == '\'') ADVANCE(160);
      if (lookahead == '\\') ADVANCE(51);
      if (lookahead != 0) ADVANCE(24);
      END_STATE();
    case 26:
      ADVANCE_MAP(
        ')', 77,
        ',', 80,
        '.', 29,
        '/', 27,
        ':', 86,
        ';', 75,
        '<', 130,
        '=', 123,
        '>', 128,
        ']', 85,
        'r', 173,
        '{', 78,
        '|', 107,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(26);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 27:
      if (lookahead == '*') ADVANCE(170);
      if (lookahead == '/') ADVANCE(162);
      END_STATE();
    case 28:
      if (lookahead == '*') ADVANCE(95);
      if (lookahead == '+') ADVANCE(93);
      if (lookahead == '/') ADVANCE(90);
      if (lookahead == '?') ADVANCE(97);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(91);
      if (lookahead != 0) ADVANCE(92);
      END_STATE();
    case 29:
      if (lookahead == '.') ADVANCE(137);
      END_STATE();
    case 30:
      if (lookahead == '.') ADVANCE(136);
      END_STATE();
    case 31:
      if (lookahead == '.') ADVANCE(138);
      END_STATE();
    case 32:
      if (lookahead == '/') ADVANCE(169);
      END_STATE();
    case 33:
      if (lookahead == '1') ADVANCE(35);
      if (lookahead == '3') ADVANCE(34);
      if (lookahead == '6') ADVANCE(37);
      if (lookahead == '8') ADVANCE(149);
      if (lookahead == 's') ADVANCE(45);
      END_STATE();
    case 34:
      if (lookahead == '2') ADVANCE(149);
      END_STATE();
    case 35:
      if (lookahead == '2') ADVANCE(38);
      if (lookahead == '6') ADVANCE(149);
      END_STATE();
    case 36:
      if (lookahead == '3') ADVANCE(34);
      if (lookahead == '6') ADVANCE(37);
      END_STATE();
    case 37:
      if (lookahead == '4') ADVANCE(149);
      END_STATE();
    case 38:
      if (lookahead == '8') ADVANCE(149);
      END_STATE();
    case 39:
      if (lookahead == ':') ADVANCE(141);
      END_STATE();
    case 40:
      if (lookahead == '=') ADVANCE(127);
      END_STATE();
    case 41:
      if (lookahead == '>') ADVANCE(142);
      END_STATE();
    case 42:
      if (lookahead == '>') ADVANCE(81);
      END_STATE();
    case 43:
      if (lookahead == 'c') ADVANCE(47);
      END_STATE();
    case 44:
      if (lookahead == 'e') ADVANCE(149);
      END_STATE();
    case 45:
      if (lookahead == 'i') ADVANCE(52);
      END_STATE();
    case 46:
      if (lookahead == 'n') ADVANCE(48);
      END_STATE();
    case 47:
      if (lookahead == 'o') ADVANCE(46);
      END_STATE();
    case 48:
      if (lookahead == 's') ADVANCE(49);
      END_STATE();
    case 49:
      if (lookahead == 't') ADVANCE(147);
      END_STATE();
    case 50:
      if (lookahead == 'u') ADVANCE(53);
      if (lookahead == 'x') ADVANCE(63);
      if (lookahead != 0) ADVANCE(161);
      END_STATE();
    case 51:
      if (lookahead == 'u') ADVANCE(54);
      if (lookahead == 'x') ADVANCE(64);
      if (lookahead != 0) ADVANCE(24);
      END_STATE();
    case 52:
      if (lookahead == 'z') ADVANCE(44);
      END_STATE();
    case 53:
      if (lookahead == '{') ADVANCE(67);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(61);
      END_STATE();
    case 54:
      if (lookahead == '{') ADVANCE(68);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(65);
      END_STATE();
    case 55:
      if (lookahead == '}') ADVANCE(24);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(55);
      END_STATE();
    case 56:
      if (lookahead == '}') ADVANCE(161);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(56);
      END_STATE();
    case 57:
      if (lookahead == '0' ||
          lookahead == '1' ||
          lookahead == '_') ADVANCE(154);
      END_STATE();
    case 58:
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(58);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(176);
      END_STATE();
    case 59:
      if (('0' <= lookahead && lookahead <= '7') ||
          lookahead == '_') ADVANCE(155);
      END_STATE();
    case 60:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(24);
      END_STATE();
    case 61:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(63);
      END_STATE();
    case 62:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(161);
      END_STATE();
    case 63:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(62);
      END_STATE();
    case 64:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(60);
      END_STATE();
    case 65:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(64);
      END_STATE();
    case 66:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(157);
      END_STATE();
    case 67:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(56);
      END_STATE();
    case 68:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(55);
      END_STATE();
    case 69:
      if (set_contains(sym_identifier_character_set_1, 685, lookahead)) ADVANCE(174);
      END_STATE();
    case 70:
      if (eof) ADVANCE(74);
      ADVANCE_MAP(
        '!', 83,
        '"', 158,
        '#', 144,
        '$', 89,
        '%', 103,
        '&', 106,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 100,
        '.', 135,
        '/', 102,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 148,
        '=', 125,
        '>', 129,
        '?', 97,
        '@', 134,
        '[', 84,
        ']', 85,
        '^', 104,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
        '~', 43,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(70);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 71:
      if (eof) ADVANCE(74);
      ADVANCE_MAP(
        '!', 83,
        '"', 158,
        '#', 143,
        '$', 58,
        '%', 103,
        '&', 106,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 96,
        '+', 94,
        ',', 80,
        '-', 99,
        '.', 135,
        '/', 102,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 131,
        '=', 124,
        '>', 129,
        '?', 97,
        '[', 84,
        ']', 85,
        '^', 104,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 108,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(71);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 72:
      if (eof) ADVANCE(74);
      ADVANCE_MAP(
        '!', 82,
        '"', 158,
        '#', 144,
        '$', 58,
        '&', 105,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 95,
        '+', 93,
        ',', 80,
        '-', 98,
        '.', 30,
        '/', 27,
        '0', 153,
        ':', 87,
        ';', 75,
        '<', 130,
        '=', 123,
        '>', 128,
        '?', 97,
        '[', 84,
        ']', 85,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 107,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(72);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 73:
      if (eof) ADVANCE(74);
      ADVANCE_MAP(
        '!', 82,
        '"', 158,
        '#', 143,
        '$', 58,
        '&', 105,
        '\'', 146,
        '(', 76,
        ')', 77,
        '*', 95,
        ',', 80,
        '-', 98,
        '.', 30,
        '/', 27,
        '0', 153,
        ':', 39,
        ';', 75,
        '<', 130,
        '=', 42,
        '[', 84,
        ']', 85,
        'b', 171,
        'c', 172,
        'r', 173,
        '{', 78,
        '|', 107,
        '}', 79,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(73);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(156);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(174);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=') ADVANCE(127);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 85:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 87:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == ':') ADVANCE(141);
      END_STATE();
    case 88:
      ACCEPT_TOKEN(anon_sym_DOLLAR);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_DOLLAR);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(58);
      if (set_contains(aux_sym_metavariable_token1_character_set_1, 687, lookahead)) ADVANCE(176);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(aux_sym_token_repetition_pattern_token1);
      if (lookahead == '*') ADVANCE(170);
      if (lookahead == '/') ADVANCE(163);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(92);
      END_STATE();
    case 91:
      ACCEPT_TOKEN(aux_sym_token_repetition_pattern_token1);
      if (lookahead == '/') ADVANCE(90);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(91);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(92);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(aux_sym_token_repetition_pattern_token1);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(92);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 94:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '=') ADVANCE(113);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '=') ADVANCE(115);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(anon_sym_QMARK);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '=') ADVANCE(114);
      END_STATE();
    case 100:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '=') ADVANCE(114);
      if (lookahead == '>') ADVANCE(142);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '>') ADVANCE(142);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(170);
      if (lookahead == '/') ADVANCE(162);
      if (lookahead == '=') ADVANCE(116);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      if (lookahead == '=') ADVANCE(117);
      END_STATE();
    case 104:
      ACCEPT_TOKEN(anon_sym_CARET);
      if (lookahead == '=') ADVANCE(118);
      END_STATE();
    case 105:
      ACCEPT_TOKEN(anon_sym_AMP);
      END_STATE();
    case 106:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(109);
      if (lookahead == '=') ADVANCE(119);
      END_STATE();
    case 107:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '=') ADVANCE(120);
      if (lookahead == '|') ADVANCE(110);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 111:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      if (lookahead == '=') ADVANCE(121);
      END_STATE();
    case 112:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '=') ADVANCE(122);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_PLUS_EQ);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_DASH_EQ);
      END_STATE();
    case 115:
      ACCEPT_TOKEN(anon_sym_STAR_EQ);
      END_STATE();
    case 116:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(anon_sym_PERCENT_EQ);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_CARET_EQ);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_AMP_EQ);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_PIPE_EQ);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_LT_LT_EQ);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(anon_sym_GT_GT_EQ);
      END_STATE();
    case 123:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 124:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(126);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(126);
      if (lookahead == '>') ADVANCE(81);
      END_STATE();
    case 126:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_GT);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(132);
      if (lookahead == '>') ADVANCE(112);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_LT);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(111);
      if (lookahead == '=') ADVANCE(133);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 134:
      ACCEPT_TOKEN(anon_sym_AT);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(137);
      END_STATE();
    case 136:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      if (lookahead == '.') ADVANCE(139);
      if (lookahead == '=') ADVANCE(140);
      END_STATE();
    case 138:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      if (lookahead == '=') ADVANCE(140);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_DOT);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_EQ);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(anon_sym_COLON_COLON);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(anon_sym_DASH_GT);
      END_STATE();
    case 143:
      ACCEPT_TOKEN(anon_sym_POUND);
      END_STATE();
    case 144:
      ACCEPT_TOKEN(anon_sym_POUND);
      if (lookahead == '!') ADVANCE(2);
      END_STATE();
    case 145:
      ACCEPT_TOKEN(anon_sym_SQUOTE);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(anon_sym_SQUOTE);
      if (lookahead == '\'') ADVANCE(160);
      if (lookahead == '\\') ADVANCE(51);
      if (lookahead != 0) ADVANCE(24);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(anon_sym_TILDEconst);
      END_STATE();
    case 148:
      ACCEPT_TOKEN(anon_sym_LT2);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(sym_integer_literal);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == '2') ADVANCE(157);
      if (lookahead == 'f') ADVANCE(151);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(33);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(157);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == '3') ADVANCE(150);
      if (lookahead == '6') ADVANCE(152);
      if (lookahead == 'f') ADVANCE(151);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(33);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(157);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == '4') ADVANCE(157);
      if (lookahead == 'f') ADVANCE(151);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(33);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(157);
      END_STATE();
    case 153:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'b') ADVANCE(57);
      if (lookahead == 'f') ADVANCE(36);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'o') ADVANCE(59);
      if (lookahead == 'u') ADVANCE(33);
      if (lookahead == 'x') ADVANCE(66);
      if (('0' <= lookahead && lookahead <= '9') ||
          lookahead == '_') ADVANCE(156);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(36);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(33);
      if (lookahead == '0' ||
          lookahead == '1' ||
          lookahead == '_') ADVANCE(154);
      END_STATE();
    case 155:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(36);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(33);
      if (('0' <= lookahead && lookahead <= '7') ||
          lookahead == '_') ADVANCE(155);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(36);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(33);
      if (('0' <= lookahead && lookahead <= '9') ||
          lookahead == '_') ADVANCE(156);
      END_STATE();
    case 157:
      ACCEPT_TOKEN(sym_integer_literal);
      if (lookahead == 'f') ADVANCE(151);
      if (lookahead == 'i') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(33);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(157);
      END_STATE();
    case 158:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(anon_sym_DQUOTE);
      END_STATE();
    case 160:
      ACCEPT_TOKEN(sym_char_literal);
      END_STATE();
    case 161:
      ACCEPT_TOKEN(sym_escape_sequence);
      END_STATE();
    case 162:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      END_STATE();
    case 163:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      if (lookahead != 0 &&
          lookahead != '*' &&
          lookahead != '+' &&
          lookahead != '?') ADVANCE(92);
      END_STATE();
    case 164:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      if (lookahead == '!') ADVANCE(168);
      if (lookahead == '/') ADVANCE(165);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(166);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(167);
      END_STATE();
    case 165:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      if (lookahead == '*') ADVANCE(167);
      if (lookahead == '/') ADVANCE(167);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(167);
      END_STATE();
    case 166:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      if (lookahead == '/') ADVANCE(165);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(166);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead)) ADVANCE(167);
      END_STATE();
    case 167:
      ACCEPT_TOKEN(aux_sym_line_comment_token1);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(167);
      END_STATE();
    case 168:
      ACCEPT_TOKEN(anon_sym_BANG2);
      END_STATE();
    case 169:
      ACCEPT_TOKEN(anon_sym_STAR_SLASH);
      END_STATE();
    case 170:
      ACCEPT_TOKEN(anon_sym_SLASH_STAR);
      END_STATE();
    case 171:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(158);
      if (lookahead == '\'') ADVANCE(25);
      if (set_contains(sym_identifier_character_set_3, 800, lookahead)) ADVANCE(174);
      END_STATE();
    case 172:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(158);
      if (set_contains(sym_identifier_character_set_3, 800, lookahead)) ADVANCE(174);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '#') ADVANCE(69);
      if (set_contains(sym_identifier_character_set_3, 800, lookahead)) ADVANCE(174);
      END_STATE();
    case 174:
      ACCEPT_TOKEN(sym_identifier);
      if (set_contains(sym_identifier_character_set_3, 800, lookahead)) ADVANCE(174);
      END_STATE();
    case 175:
      ACCEPT_TOKEN(sym_shebang);
      END_STATE();
    case 176:
      ACCEPT_TOKEN(aux_sym_metavariable_token1);
      if (set_contains(sym_identifier_character_set_3, 800, lookahead)) ADVANCE(176);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      ADVANCE_MAP(
        '_', 1,
        'a', 2,
        'b', 3,
        'c', 4,
        'd', 5,
        'e', 6,
        'f', 7,
        'g', 8,
        'i', 9,
        'l', 10,
        'm', 11,
        'p', 12,
        'r', 13,
        's', 14,
        't', 15,
        'u', 16,
        'v', 17,
        'w', 18,
        'y', 19,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      END_STATE();
    case 1:
      ACCEPT_TOKEN(anon_sym__);
      END_STATE();
    case 2:
      if (lookahead == 's') ADVANCE(20);
      if (lookahead == 'u') ADVANCE(21);
      if (lookahead == 'w') ADVANCE(22);
      END_STATE();
    case 3:
      if (lookahead == 'e') ADVANCE(23);
      if (lookahead == 'l') ADVANCE(24);
      if (lookahead == 'o') ADVANCE(25);
      if (lookahead == 'r') ADVANCE(26);
      END_STATE();
    case 4:
      if (lookahead == 'h') ADVANCE(27);
      if (lookahead == 'o') ADVANCE(28);
      if (lookahead == 'r') ADVANCE(29);
      END_STATE();
    case 5:
      if (lookahead == 'e') ADVANCE(30);
      if (lookahead == 'y') ADVANCE(31);
      END_STATE();
    case 6:
      if (lookahead == 'l') ADVANCE(32);
      if (lookahead == 'n') ADVANCE(33);
      if (lookahead == 'x') ADVANCE(34);
      END_STATE();
    case 7:
      if (lookahead == '3') ADVANCE(35);
      if (lookahead == '6') ADVANCE(36);
      if (lookahead == 'a') ADVANCE(37);
      if (lookahead == 'n') ADVANCE(38);
      if (lookahead == 'o') ADVANCE(39);
      END_STATE();
    case 8:
      if (lookahead == 'e') ADVANCE(40);
      END_STATE();
    case 9:
      ADVANCE_MAP(
        '1', 41,
        '3', 42,
        '6', 43,
        '8', 44,
        'd', 45,
        'f', 46,
        'm', 47,
        'n', 48,
        's', 49,
        't', 50,
      );
      END_STATE();
    case 10:
      if (lookahead == 'e') ADVANCE(51);
      if (lookahead == 'i') ADVANCE(52);
      if (lookahead == 'o') ADVANCE(53);
      END_STATE();
    case 11:
      if (lookahead == 'a') ADVANCE(54);
      if (lookahead == 'e') ADVANCE(55);
      if (lookahead == 'o') ADVANCE(56);
      if (lookahead == 'u') ADVANCE(57);
      END_STATE();
    case 12:
      if (lookahead == 'a') ADVANCE(58);
      if (lookahead == 'u') ADVANCE(59);
      END_STATE();
    case 13:
      if (lookahead == 'a') ADVANCE(60);
      if (lookahead == 'e') ADVANCE(61);
      END_STATE();
    case 14:
      if (lookahead == 'a') ADVANCE(62);
      if (lookahead == 'e') ADVANCE(63);
      if (lookahead == 't') ADVANCE(64);
      if (lookahead == 'u') ADVANCE(65);
      END_STATE();
    case 15:
      if (lookahead == 'r') ADVANCE(66);
      if (lookahead == 't') ADVANCE(67);
      if (lookahead == 'y') ADVANCE(68);
      END_STATE();
    case 16:
      if (lookahead == '1') ADVANCE(69);
      if (lookahead == '3') ADVANCE(70);
      if (lookahead == '6') ADVANCE(71);
      if (lookahead == '8') ADVANCE(72);
      if (lookahead == 'n') ADVANCE(73);
      if (lookahead == 's') ADVANCE(74);
      END_STATE();
    case 17:
      if (lookahead == 'i') ADVANCE(75);
      END_STATE();
    case 18:
      if (lookahead == 'h') ADVANCE(76);
      END_STATE();
    case 19:
      if (lookahead == 'i') ADVANCE(77);
      END_STATE();
    case 20:
      ACCEPT_TOKEN(anon_sym_as);
      if (lookahead == 'y') ADVANCE(78);
      END_STATE();
    case 21:
      if (lookahead == 't') ADVANCE(79);
      END_STATE();
    case 22:
      if (lookahead == 'a') ADVANCE(80);
      END_STATE();
    case 23:
      if (lookahead == 'c') ADVANCE(81);
      END_STATE();
    case 24:
      if (lookahead == 'o') ADVANCE(82);
      END_STATE();
    case 25:
      if (lookahead == 'o') ADVANCE(83);
      if (lookahead == 'x') ADVANCE(84);
      END_STATE();
    case 26:
      if (lookahead == 'e') ADVANCE(85);
      END_STATE();
    case 27:
      if (lookahead == 'a') ADVANCE(86);
      END_STATE();
    case 28:
      if (lookahead == 'n') ADVANCE(87);
      END_STATE();
    case 29:
      if (lookahead == 'a') ADVANCE(88);
      END_STATE();
    case 30:
      if (lookahead == 'f') ADVANCE(89);
      END_STATE();
    case 31:
      if (lookahead == 'n') ADVANCE(90);
      END_STATE();
    case 32:
      if (lookahead == 's') ADVANCE(91);
      END_STATE();
    case 33:
      if (lookahead == 'u') ADVANCE(92);
      END_STATE();
    case 34:
      if (lookahead == 'p') ADVANCE(93);
      if (lookahead == 't') ADVANCE(94);
      END_STATE();
    case 35:
      if (lookahead == '2') ADVANCE(95);
      END_STATE();
    case 36:
      if (lookahead == '4') ADVANCE(96);
      END_STATE();
    case 37:
      if (lookahead == 'l') ADVANCE(97);
      END_STATE();
    case 38:
      ACCEPT_TOKEN(anon_sym_fn);
      END_STATE();
    case 39:
      if (lookahead == 'r') ADVANCE(98);
      END_STATE();
    case 40:
      if (lookahead == 'n') ADVANCE(99);
      END_STATE();
    case 41:
      if (lookahead == '2') ADVANCE(100);
      if (lookahead == '6') ADVANCE(101);
      END_STATE();
    case 42:
      if (lookahead == '2') ADVANCE(102);
      END_STATE();
    case 43:
      if (lookahead == '4') ADVANCE(103);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_i8);
      END_STATE();
    case 45:
      if (lookahead == 'e') ADVANCE(104);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 47:
      if (lookahead == 'p') ADVANCE(105);
      END_STATE();
    case 48:
      ACCEPT_TOKEN(anon_sym_in);
      END_STATE();
    case 49:
      if (lookahead == 'i') ADVANCE(106);
      END_STATE();
    case 50:
      if (lookahead == 'e') ADVANCE(107);
      END_STATE();
    case 51:
      if (lookahead == 't') ADVANCE(108);
      END_STATE();
    case 52:
      if (lookahead == 'f') ADVANCE(109);
      if (lookahead == 't') ADVANCE(110);
      END_STATE();
    case 53:
      if (lookahead == 'o') ADVANCE(111);
      END_STATE();
    case 54:
      if (lookahead == 'c') ADVANCE(112);
      if (lookahead == 't') ADVANCE(113);
      END_STATE();
    case 55:
      if (lookahead == 't') ADVANCE(114);
      END_STATE();
    case 56:
      if (lookahead == 'd') ADVANCE(115);
      if (lookahead == 'v') ADVANCE(116);
      END_STATE();
    case 57:
      if (lookahead == 't') ADVANCE(117);
      END_STATE();
    case 58:
      if (lookahead == 't') ADVANCE(118);
      END_STATE();
    case 59:
      if (lookahead == 'b') ADVANCE(119);
      END_STATE();
    case 60:
      if (lookahead == 'w') ADVANCE(120);
      END_STATE();
    case 61:
      if (lookahead == 'f') ADVANCE(121);
      if (lookahead == 't') ADVANCE(122);
      END_STATE();
    case 62:
      if (lookahead == 'f') ADVANCE(123);
      END_STATE();
    case 63:
      if (lookahead == 'l') ADVANCE(124);
      END_STATE();
    case 64:
      if (lookahead == 'a') ADVANCE(125);
      if (lookahead == 'm') ADVANCE(126);
      if (lookahead == 'r') ADVANCE(127);
      END_STATE();
    case 65:
      if (lookahead == 'p') ADVANCE(128);
      END_STATE();
    case 66:
      if (lookahead == 'a') ADVANCE(129);
      if (lookahead == 'u') ADVANCE(130);
      if (lookahead == 'y') ADVANCE(131);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_tt);
      END_STATE();
    case 68:
      ACCEPT_TOKEN(anon_sym_ty);
      if (lookahead == 'p') ADVANCE(132);
      END_STATE();
    case 69:
      if (lookahead == '2') ADVANCE(133);
      if (lookahead == '6') ADVANCE(134);
      END_STATE();
    case 70:
      if (lookahead == '2') ADVANCE(135);
      END_STATE();
    case 71:
      if (lookahead == '4') ADVANCE(136);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(anon_sym_u8);
      END_STATE();
    case 73:
      if (lookahead == 'i') ADVANCE(137);
      if (lookahead == 's') ADVANCE(138);
      END_STATE();
    case 74:
      if (lookahead == 'e') ADVANCE(139);
      if (lookahead == 'i') ADVANCE(140);
      END_STATE();
    case 75:
      if (lookahead == 's') ADVANCE(141);
      END_STATE();
    case 76:
      if (lookahead == 'e') ADVANCE(142);
      if (lookahead == 'i') ADVANCE(143);
      END_STATE();
    case 77:
      if (lookahead == 'e') ADVANCE(144);
      END_STATE();
    case 78:
      if (lookahead == 'n') ADVANCE(145);
      END_STATE();
    case 79:
      if (lookahead == 'o') ADVANCE(146);
      END_STATE();
    case 80:
      if (lookahead == 'i') ADVANCE(147);
      END_STATE();
    case 81:
      if (lookahead == 'o') ADVANCE(148);
      END_STATE();
    case 82:
      if (lookahead == 'c') ADVANCE(149);
      END_STATE();
    case 83:
      if (lookahead == 'l') ADVANCE(150);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_box);
      END_STATE();
    case 85:
      if (lookahead == 'a') ADVANCE(151);
      END_STATE();
    case 86:
      if (lookahead == 'r') ADVANCE(152);
      END_STATE();
    case 87:
      if (lookahead == 's') ADVANCE(153);
      if (lookahead == 't') ADVANCE(154);
      END_STATE();
    case 88:
      if (lookahead == 't') ADVANCE(155);
      END_STATE();
    case 89:
      if (lookahead == 'a') ADVANCE(156);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(anon_sym_dyn);
      END_STATE();
    case 91:
      if (lookahead == 'e') ADVANCE(157);
      END_STATE();
    case 92:
      if (lookahead == 'm') ADVANCE(158);
      END_STATE();
    case 93:
      if (lookahead == 'r') ADVANCE(159);
      END_STATE();
    case 94:
      if (lookahead == 'e') ADVANCE(160);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(anon_sym_f32);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_f64);
      END_STATE();
    case 97:
      if (lookahead == 's') ADVANCE(161);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_for);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_gen);
      END_STATE();
    case 100:
      if (lookahead == '8') ADVANCE(162);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(anon_sym_i16);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(anon_sym_i32);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(anon_sym_i64);
      END_STATE();
    case 104:
      if (lookahead == 'n') ADVANCE(163);
      END_STATE();
    case 105:
      if (lookahead == 'l') ADVANCE(164);
      END_STATE();
    case 106:
      if (lookahead == 'z') ADVANCE(165);
      END_STATE();
    case 107:
      if (lookahead == 'm') ADVANCE(166);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_let);
      END_STATE();
    case 109:
      if (lookahead == 'e') ADVANCE(167);
      END_STATE();
    case 110:
      if (lookahead == 'e') ADVANCE(168);
      END_STATE();
    case 111:
      if (lookahead == 'p') ADVANCE(169);
      END_STATE();
    case 112:
      if (lookahead == 'r') ADVANCE(170);
      END_STATE();
    case 113:
      if (lookahead == 'c') ADVANCE(171);
      END_STATE();
    case 114:
      if (lookahead == 'a') ADVANCE(172);
      END_STATE();
    case 115:
      ACCEPT_TOKEN(anon_sym_mod);
      END_STATE();
    case 116:
      if (lookahead == 'e') ADVANCE(173);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(sym_mutable_specifier);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_pat);
      if (lookahead == '_') ADVANCE(174);
      if (lookahead == 'h') ADVANCE(175);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_pub);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_raw);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_ref);
      END_STATE();
    case 122:
      if (lookahead == 'u') ADVANCE(176);
      END_STATE();
    case 123:
      if (lookahead == 'e') ADVANCE(177);
      END_STATE();
    case 124:
      if (lookahead == 'f') ADVANCE(178);
      END_STATE();
    case 125:
      if (lookahead == 't') ADVANCE(179);
      END_STATE();
    case 126:
      if (lookahead == 't') ADVANCE(180);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_str);
      if (lookahead == 'u') ADVANCE(181);
      END_STATE();
    case 128:
      if (lookahead == 'e') ADVANCE(182);
      END_STATE();
    case 129:
      if (lookahead == 'i') ADVANCE(183);
      END_STATE();
    case 130:
      if (lookahead == 'e') ADVANCE(184);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_try);
      END_STATE();
    case 132:
      if (lookahead == 'e') ADVANCE(185);
      END_STATE();
    case 133:
      if (lookahead == '8') ADVANCE(186);
      END_STATE();
    case 134:
      ACCEPT_TOKEN(anon_sym_u16);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(anon_sym_u32);
      END_STATE();
    case 136:
      ACCEPT_TOKEN(anon_sym_u64);
      END_STATE();
    case 137:
      if (lookahead == 'o') ADVANCE(187);
      END_STATE();
    case 138:
      if (lookahead == 'a') ADVANCE(188);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(anon_sym_use);
      END_STATE();
    case 140:
      if (lookahead == 'z') ADVANCE(189);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(anon_sym_vis);
      END_STATE();
    case 142:
      if (lookahead == 'r') ADVANCE(190);
      END_STATE();
    case 143:
      if (lookahead == 'l') ADVANCE(191);
      END_STATE();
    case 144:
      if (lookahead == 'l') ADVANCE(192);
      END_STATE();
    case 145:
      if (lookahead == 'c') ADVANCE(193);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(anon_sym_auto);
      END_STATE();
    case 147:
      if (lookahead == 't') ADVANCE(194);
      END_STATE();
    case 148:
      if (lookahead == 'm') ADVANCE(195);
      END_STATE();
    case 149:
      if (lookahead == 'k') ADVANCE(196);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(anon_sym_bool);
      END_STATE();
    case 151:
      if (lookahead == 'k') ADVANCE(197);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(anon_sym_char);
      END_STATE();
    case 153:
      if (lookahead == 't') ADVANCE(198);
      END_STATE();
    case 154:
      if (lookahead == 'i') ADVANCE(199);
      END_STATE();
    case 155:
      if (lookahead == 'e') ADVANCE(200);
      END_STATE();
    case 156:
      if (lookahead == 'u') ADVANCE(201);
      END_STATE();
    case 157:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 158:
      ACCEPT_TOKEN(anon_sym_enum);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(anon_sym_expr);
      if (lookahead == '_') ADVANCE(202);
      END_STATE();
    case 160:
      if (lookahead == 'r') ADVANCE(203);
      END_STATE();
    case 161:
      if (lookahead == 'e') ADVANCE(204);
      END_STATE();
    case 162:
      ACCEPT_TOKEN(anon_sym_i128);
      END_STATE();
    case 163:
      if (lookahead == 't') ADVANCE(205);
      END_STATE();
    case 164:
      ACCEPT_TOKEN(anon_sym_impl);
      END_STATE();
    case 165:
      if (lookahead == 'e') ADVANCE(206);
      END_STATE();
    case 166:
      ACCEPT_TOKEN(anon_sym_item);
      END_STATE();
    case 167:
      if (lookahead == 't') ADVANCE(207);
      END_STATE();
    case 168:
      if (lookahead == 'r') ADVANCE(208);
      END_STATE();
    case 169:
      ACCEPT_TOKEN(anon_sym_loop);
      END_STATE();
    case 170:
      if (lookahead == 'o') ADVANCE(209);
      END_STATE();
    case 171:
      if (lookahead == 'h') ADVANCE(210);
      END_STATE();
    case 172:
      ACCEPT_TOKEN(anon_sym_meta);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(anon_sym_move);
      END_STATE();
    case 174:
      if (lookahead == 'p') ADVANCE(211);
      END_STATE();
    case 175:
      ACCEPT_TOKEN(anon_sym_path);
      END_STATE();
    case 176:
      if (lookahead == 'r') ADVANCE(212);
      END_STATE();
    case 177:
      ACCEPT_TOKEN(anon_sym_safe);
      END_STATE();
    case 178:
      ACCEPT_TOKEN(sym_self);
      END_STATE();
    case 179:
      if (lookahead == 'i') ADVANCE(213);
      END_STATE();
    case 180:
      ACCEPT_TOKEN(anon_sym_stmt);
      END_STATE();
    case 181:
      if (lookahead == 'c') ADVANCE(214);
      END_STATE();
    case 182:
      if (lookahead == 'r') ADVANCE(215);
      END_STATE();
    case 183:
      if (lookahead == 't') ADVANCE(216);
      END_STATE();
    case 184:
      ACCEPT_TOKEN(anon_sym_true);
      END_STATE();
    case 185:
      ACCEPT_TOKEN(anon_sym_type);
      END_STATE();
    case 186:
      ACCEPT_TOKEN(anon_sym_u128);
      END_STATE();
    case 187:
      if (lookahead == 'n') ADVANCE(217);
      END_STATE();
    case 188:
      if (lookahead == 'f') ADVANCE(218);
      END_STATE();
    case 189:
      if (lookahead == 'e') ADVANCE(219);
      END_STATE();
    case 190:
      if (lookahead == 'e') ADVANCE(220);
      END_STATE();
    case 191:
      if (lookahead == 'e') ADVANCE(221);
      END_STATE();
    case 192:
      if (lookahead == 'd') ADVANCE(222);
      END_STATE();
    case 193:
      ACCEPT_TOKEN(anon_sym_async);
      END_STATE();
    case 194:
      ACCEPT_TOKEN(anon_sym_await);
      END_STATE();
    case 195:
      if (lookahead == 'e') ADVANCE(223);
      END_STATE();
    case 196:
      ACCEPT_TOKEN(anon_sym_block);
      END_STATE();
    case 197:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 198:
      ACCEPT_TOKEN(anon_sym_const);
      END_STATE();
    case 199:
      if (lookahead == 'n') ADVANCE(224);
      END_STATE();
    case 200:
      ACCEPT_TOKEN(sym_crate);
      END_STATE();
    case 201:
      if (lookahead == 'l') ADVANCE(225);
      END_STATE();
    case 202:
      if (lookahead == '2') ADVANCE(226);
      END_STATE();
    case 203:
      if (lookahead == 'n') ADVANCE(227);
      END_STATE();
    case 204:
      ACCEPT_TOKEN(anon_sym_false);
      END_STATE();
    case 205:
      ACCEPT_TOKEN(anon_sym_ident);
      END_STATE();
    case 206:
      ACCEPT_TOKEN(anon_sym_isize);
      END_STATE();
    case 207:
      if (lookahead == 'i') ADVANCE(228);
      END_STATE();
    case 208:
      if (lookahead == 'a') ADVANCE(229);
      END_STATE();
    case 209:
      ACCEPT_TOKEN(anon_sym_macro);
      if (lookahead == '_') ADVANCE(230);
      END_STATE();
    case 210:
      ACCEPT_TOKEN(anon_sym_match);
      END_STATE();
    case 211:
      if (lookahead == 'a') ADVANCE(231);
      END_STATE();
    case 212:
      if (lookahead == 'n') ADVANCE(232);
      END_STATE();
    case 213:
      if (lookahead == 'c') ADVANCE(233);
      END_STATE();
    case 214:
      if (lookahead == 't') ADVANCE(234);
      END_STATE();
    case 215:
      ACCEPT_TOKEN(sym_super);
      END_STATE();
    case 216:
      ACCEPT_TOKEN(anon_sym_trait);
      END_STATE();
    case 217:
      ACCEPT_TOKEN(anon_sym_union);
      END_STATE();
    case 218:
      if (lookahead == 'e') ADVANCE(235);
      END_STATE();
    case 219:
      ACCEPT_TOKEN(anon_sym_usize);
      END_STATE();
    case 220:
      ACCEPT_TOKEN(anon_sym_where);
      END_STATE();
    case 221:
      ACCEPT_TOKEN(anon_sym_while);
      END_STATE();
    case 222:
      ACCEPT_TOKEN(anon_sym_yield);
      END_STATE();
    case 223:
      ACCEPT_TOKEN(anon_sym_become);
      END_STATE();
    case 224:
      if (lookahead == 'u') ADVANCE(236);
      END_STATE();
    case 225:
      if (lookahead == 't') ADVANCE(237);
      END_STATE();
    case 226:
      if (lookahead == '0') ADVANCE(238);
      END_STATE();
    case 227:
      ACCEPT_TOKEN(anon_sym_extern);
      END_STATE();
    case 228:
      if (lookahead == 'm') ADVANCE(239);
      END_STATE();
    case 229:
      if (lookahead == 'l') ADVANCE(240);
      END_STATE();
    case 230:
      if (lookahead == 'r') ADVANCE(241);
      END_STATE();
    case 231:
      if (lookahead == 'r') ADVANCE(242);
      END_STATE();
    case 232:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 233:
      ACCEPT_TOKEN(anon_sym_static);
      END_STATE();
    case 234:
      ACCEPT_TOKEN(anon_sym_struct);
      END_STATE();
    case 235:
      ACCEPT_TOKEN(anon_sym_unsafe);
      END_STATE();
    case 236:
      if (lookahead == 'e') ADVANCE(243);
      END_STATE();
    case 237:
      ACCEPT_TOKEN(anon_sym_default);
      END_STATE();
    case 238:
      if (lookahead == '2') ADVANCE(244);
      END_STATE();
    case 239:
      if (lookahead == 'e') ADVANCE(245);
      END_STATE();
    case 240:
      ACCEPT_TOKEN(anon_sym_literal);
      END_STATE();
    case 241:
      if (lookahead == 'u') ADVANCE(246);
      END_STATE();
    case 242:
      if (lookahead == 'a') ADVANCE(247);
      END_STATE();
    case 243:
      ACCEPT_TOKEN(anon_sym_continue);
      END_STATE();
    case 244:
      if (lookahead == '1') ADVANCE(248);
      END_STATE();
    case 245:
      ACCEPT_TOKEN(anon_sym_lifetime);
      END_STATE();
    case 246:
      if (lookahead == 'l') ADVANCE(249);
      END_STATE();
    case 247:
      if (lookahead == 'm') ADVANCE(250);
      END_STATE();
    case 248:
      ACCEPT_TOKEN(anon_sym_expr_2021);
      END_STATE();
    case 249:
      if (lookahead == 'e') ADVANCE(251);
      END_STATE();
    case 250:
      ACCEPT_TOKEN(anon_sym_pat_param);
      END_STATE();
    case 251:
      if (lookahead == 's') ADVANCE(252);
      END_STATE();
    case 252:
      ACCEPT_TOKEN(anon_sym_macro_rules);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 72, .external_lex_state = 2},
  [2] = {.lex_state = 73, .external_lex_state = 2},
  [3] = {.lex_state = 73, .external_lex_state = 3},
  [4] = {.lex_state = 73, .external_lex_state = 3},
  [5] = {.lex_state = 73, .external_lex_state = 3},
  [6] = {.lex_state = 73, .external_lex_state = 3},
  [7] = {.lex_state = 73, .external_lex_state = 3},
  [8] = {.lex_state = 73, .external_lex_state = 3},
  [9] = {.lex_state = 73, .external_lex_state = 3},
  [10] = {.lex_state = 73, .external_lex_state = 3},
  [11] = {.lex_state = 73, .external_lex_state = 3},
  [12] = {.lex_state = 73, .external_lex_state = 3},
  [13] = {.lex_state = 73, .external_lex_state = 3},
  [14] = {.lex_state = 73, .external_lex_state = 3},
  [15] = {.lex_state = 73, .external_lex_state = 3},
  [16] = {.lex_state = 73, .external_lex_state = 3},
  [17] = {.lex_state = 73, .external_lex_state = 3},
  [18] = {.lex_state = 73, .external_lex_state = 3},
  [19] = {.lex_state = 73, .external_lex_state = 3},
  [20] = {.lex_state = 73, .external_lex_state = 3},
  [21] = {.lex_state = 73, .external_lex_state = 3},
  [22] = {.lex_state = 73, .external_lex_state = 3},
  [23] = {.lex_state = 73, .external_lex_state = 3},
  [24] = {.lex_state = 73, .external_lex_state = 3},
  [25] = {.lex_state = 73, .external_lex_state = 3},
  [26] = {.lex_state = 73, .external_lex_state = 3},
  [27] = {.lex_state = 73, .external_lex_state = 3},
  [28] = {.lex_state = 73, .external_lex_state = 3},
  [29] = {.lex_state = 73, .external_lex_state = 3},
  [30] = {.lex_state = 73, .external_lex_state = 3},
  [31] = {.lex_state = 73, .external_lex_state = 3},
  [32] = {.lex_state = 73, .external_lex_state = 3},
  [33] = {.lex_state = 73, .external_lex_state = 3},
  [34] = {.lex_state = 73, .external_lex_state = 3},
  [35] = {.lex_state = 73, .external_lex_state = 3},
  [36] = {.lex_state = 73, .external_lex_state = 3},
  [37] = {.lex_state = 73, .external_lex_state = 3},
  [38] = {.lex_state = 73, .external_lex_state = 3},
  [39] = {.lex_state = 73, .external_lex_state = 3},
  [40] = {.lex_state = 73, .external_lex_state = 3},
  [41] = {.lex_state = 73, .external_lex_state = 3},
  [42] = {.lex_state = 71, .external_lex_state = 3},
  [43] = {.lex_state = 71, .external_lex_state = 3},
  [44] = {.lex_state = 71, .external_lex_state = 3},
  [45] = {.lex_state = 71, .external_lex_state = 3},
  [46] = {.lex_state = 71, .external_lex_state = 3},
  [47] = {.lex_state = 71, .external_lex_state = 3},
  [48] = {.lex_state = 71, .external_lex_state = 3},
  [49] = {.lex_state = 71, .external_lex_state = 3},
  [50] = {.lex_state = 5, .external_lex_state = 3},
  [51] = {.lex_state = 5, .external_lex_state = 3},
  [52] = {.lex_state = 5, .external_lex_state = 3},
  [53] = {.lex_state = 5, .external_lex_state = 3},
  [54] = {.lex_state = 5, .external_lex_state = 3},
  [55] = {.lex_state = 71, .external_lex_state = 3},
  [56] = {.lex_state = 71, .external_lex_state = 3},
  [57] = {.lex_state = 5, .external_lex_state = 3},
  [58] = {.lex_state = 5, .external_lex_state = 3},
  [59] = {.lex_state = 5, .external_lex_state = 3},
  [60] = {.lex_state = 71, .external_lex_state = 3},
  [61] = {.lex_state = 71, .external_lex_state = 3},
  [62] = {.lex_state = 71, .external_lex_state = 3},
  [63] = {.lex_state = 71, .external_lex_state = 3},
  [64] = {.lex_state = 71, .external_lex_state = 3},
  [65] = {.lex_state = 71, .external_lex_state = 3},
  [66] = {.lex_state = 71, .external_lex_state = 3},
  [67] = {.lex_state = 71, .external_lex_state = 3},
  [68] = {.lex_state = 71, .external_lex_state = 3},
  [69] = {.lex_state = 71, .external_lex_state = 3},
  [70] = {.lex_state = 71, .external_lex_state = 3},
  [71] = {.lex_state = 71, .external_lex_state = 3},
  [72] = {.lex_state = 71, .external_lex_state = 3},
  [73] = {.lex_state = 71, .external_lex_state = 3},
  [74] = {.lex_state = 73, .external_lex_state = 3},
  [75] = {.lex_state = 73, .external_lex_state = 3},
  [76] = {.lex_state = 73, .external_lex_state = 3},
  [77] = {.lex_state = 73, .external_lex_state = 3},
  [78] = {.lex_state = 73, .external_lex_state = 3},
  [79] = {.lex_state = 73, .external_lex_state = 3},
  [80] = {.lex_state = 73, .external_lex_state = 3},
  [81] = {.lex_state = 73, .external_lex_state = 3},
  [82] = {.lex_state = 73, .external_lex_state = 3},
  [83] = {.lex_state = 73, .external_lex_state = 3},
  [84] = {.lex_state = 73, .external_lex_state = 3},
  [85] = {.lex_state = 73, .external_lex_state = 3},
  [86] = {.lex_state = 73, .external_lex_state = 3},
  [87] = {.lex_state = 73, .external_lex_state = 3},
  [88] = {.lex_state = 73, .external_lex_state = 3},
  [89] = {.lex_state = 73, .external_lex_state = 3},
  [90] = {.lex_state = 73, .external_lex_state = 3},
  [91] = {.lex_state = 73, .external_lex_state = 3},
  [92] = {.lex_state = 14, .external_lex_state = 3},
  [93] = {.lex_state = 14, .external_lex_state = 3},
  [94] = {.lex_state = 14, .external_lex_state = 3},
  [95] = {.lex_state = 73, .external_lex_state = 3},
  [96] = {.lex_state = 73, .external_lex_state = 3},
  [97] = {.lex_state = 14, .external_lex_state = 3},
  [98] = {.lex_state = 73, .external_lex_state = 3},
  [99] = {.lex_state = 73, .external_lex_state = 3},
  [100] = {.lex_state = 73, .external_lex_state = 3},
  [101] = {.lex_state = 14, .external_lex_state = 3},
  [102] = {.lex_state = 73, .external_lex_state = 3},
  [103] = {.lex_state = 73, .external_lex_state = 3},
  [104] = {.lex_state = 14, .external_lex_state = 3},
  [105] = {.lex_state = 14, .external_lex_state = 3},
  [106] = {.lex_state = 14, .external_lex_state = 3},
  [107] = {.lex_state = 73, .external_lex_state = 3},
  [108] = {.lex_state = 73, .external_lex_state = 3},
  [109] = {.lex_state = 73, .external_lex_state = 3},
  [110] = {.lex_state = 73, .external_lex_state = 3},
  [111] = {.lex_state = 73, .external_lex_state = 3},
  [112] = {.lex_state = 14, .external_lex_state = 3},
  [113] = {.lex_state = 73, .external_lex_state = 3},
  [114] = {.lex_state = 73, .external_lex_state = 3},
  [115] = {.lex_state = 73, .external_lex_state = 3},
  [116] = {.lex_state = 73, .external_lex_state = 3},
  [117] = {.lex_state = 73, .external_lex_state = 3},
  [118] = {.lex_state = 14, .external_lex_state = 3},
  [119] = {.lex_state = 14, .external_lex_state = 3},
  [120] = {.lex_state = 14, .external_lex_state = 3},
  [121] = {.lex_state = 14, .external_lex_state = 3},
  [122] = {.lex_state = 73, .external_lex_state = 3},
  [123] = {.lex_state = 14, .external_lex_state = 3},
  [124] = {.lex_state = 14, .external_lex_state = 3},
  [125] = {.lex_state = 73, .external_lex_state = 3},
  [126] = {.lex_state = 14, .external_lex_state = 3},
  [127] = {.lex_state = 14, .external_lex_state = 3},
  [128] = {.lex_state = 14, .external_lex_state = 3},
  [129] = {.lex_state = 14, .external_lex_state = 3},
  [130] = {.lex_state = 14, .external_lex_state = 3},
  [131] = {.lex_state = 73, .external_lex_state = 3},
  [132] = {.lex_state = 4, .external_lex_state = 4},
  [133] = {.lex_state = 73, .external_lex_state = 3},
  [134] = {.lex_state = 73, .external_lex_state = 3},
  [135] = {.lex_state = 73, .external_lex_state = 3},
  [136] = {.lex_state = 12, .external_lex_state = 3},
  [137] = {.lex_state = 73, .external_lex_state = 3},
  [138] = {.lex_state = 73, .external_lex_state = 3},
  [139] = {.lex_state = 12, .external_lex_state = 3},
  [140] = {.lex_state = 73, .external_lex_state = 3},
  [141] = {.lex_state = 73, .external_lex_state = 3},
  [142] = {.lex_state = 73, .external_lex_state = 3},
  [143] = {.lex_state = 73, .external_lex_state = 3},
  [144] = {.lex_state = 73, .external_lex_state = 3},
  [145] = {.lex_state = 73, .external_lex_state = 3},
  [146] = {.lex_state = 73, .external_lex_state = 3},
  [147] = {.lex_state = 73, .external_lex_state = 3},
  [148] = {.lex_state = 73, .external_lex_state = 3},
  [149] = {.lex_state = 12, .external_lex_state = 3},
  [150] = {.lex_state = 73, .external_lex_state = 3},
  [151] = {.lex_state = 12, .external_lex_state = 3},
  [152] = {.lex_state = 73, .external_lex_state = 3},
  [153] = {.lex_state = 73, .external_lex_state = 3},
  [154] = {.lex_state = 73, .external_lex_state = 3},
  [155] = {.lex_state = 73, .external_lex_state = 3},
  [156] = {.lex_state = 12, .external_lex_state = 3},
  [157] = {.lex_state = 73, .external_lex_state = 3},
  [158] = {.lex_state = 73, .external_lex_state = 3},
  [159] = {.lex_state = 12, .external_lex_state = 3},
  [160] = {.lex_state = 73, .external_lex_state = 3},
  [161] = {.lex_state = 73, .external_lex_state = 3},
  [162] = {.lex_state = 73, .external_lex_state = 3},
  [163] = {.lex_state = 73, .external_lex_state = 3},
  [164] = {.lex_state = 73, .external_lex_state = 3},
  [165] = {.lex_state = 73, .external_lex_state = 3},
  [166] = {.lex_state = 73, .external_lex_state = 3},
  [167] = {.lex_state = 73, .external_lex_state = 3},
  [168] = {.lex_state = 73, .external_lex_state = 3},
  [169] = {.lex_state = 73, .external_lex_state = 3},
  [170] = {.lex_state = 73, .external_lex_state = 3},
  [171] = {.lex_state = 73, .external_lex_state = 3},
  [172] = {.lex_state = 73, .external_lex_state = 3},
  [173] = {.lex_state = 73, .external_lex_state = 3},
  [174] = {.lex_state = 73, .external_lex_state = 3},
  [175] = {.lex_state = 73, .external_lex_state = 3},
  [176] = {.lex_state = 73, .external_lex_state = 3},
  [177] = {.lex_state = 73, .external_lex_state = 3},
  [178] = {.lex_state = 73, .external_lex_state = 3},
  [179] = {.lex_state = 73, .external_lex_state = 3},
  [180] = {.lex_state = 73, .external_lex_state = 3},
  [181] = {.lex_state = 73, .external_lex_state = 3},
  [182] = {.lex_state = 73, .external_lex_state = 3},
  [183] = {.lex_state = 73, .external_lex_state = 3},
  [184] = {.lex_state = 73, .external_lex_state = 3},
  [185] = {.lex_state = 73, .external_lex_state = 3},
  [186] = {.lex_state = 73, .external_lex_state = 3},
  [187] = {.lex_state = 73, .external_lex_state = 3},
  [188] = {.lex_state = 73, .external_lex_state = 3},
  [189] = {.lex_state = 73, .external_lex_state = 3},
  [190] = {.lex_state = 73, .external_lex_state = 3},
  [191] = {.lex_state = 73, .external_lex_state = 3},
  [192] = {.lex_state = 73, .external_lex_state = 3},
  [193] = {.lex_state = 73, .external_lex_state = 3},
  [194] = {.lex_state = 73, .external_lex_state = 3},
  [195] = {.lex_state = 73, .external_lex_state = 3},
  [196] = {.lex_state = 73, .external_lex_state = 3},
  [197] = {.lex_state = 73, .external_lex_state = 3},
  [198] = {.lex_state = 73, .external_lex_state = 3},
  [199] = {.lex_state = 73, .external_lex_state = 3},
  [200] = {.lex_state = 73, .external_lex_state = 3},
  [201] = {.lex_state = 73, .external_lex_state = 3},
  [202] = {.lex_state = 73, .external_lex_state = 3},
  [203] = {.lex_state = 73, .external_lex_state = 3},
  [204] = {.lex_state = 73, .external_lex_state = 3},
  [205] = {.lex_state = 73, .external_lex_state = 3},
  [206] = {.lex_state = 73, .external_lex_state = 3},
  [207] = {.lex_state = 73, .external_lex_state = 3},
  [208] = {.lex_state = 73, .external_lex_state = 3},
  [209] = {.lex_state = 73, .external_lex_state = 3},
  [210] = {.lex_state = 73, .external_lex_state = 3},
  [211] = {.lex_state = 73, .external_lex_state = 3},
  [212] = {.lex_state = 73, .external_lex_state = 3},
  [213] = {.lex_state = 73, .external_lex_state = 3},
  [214] = {.lex_state = 73, .external_lex_state = 3},
  [215] = {.lex_state = 73, .external_lex_state = 3},
  [216] = {.lex_state = 73, .external_lex_state = 3},
  [217] = {.lex_state = 73, .external_lex_state = 3},
  [218] = {.lex_state = 73, .external_lex_state = 3},
  [219] = {.lex_state = 73, .external_lex_state = 3},
  [220] = {.lex_state = 73, .external_lex_state = 3},
  [221] = {.lex_state = 73, .external_lex_state = 3},
  [222] = {.lex_state = 73, .external_lex_state = 3},
  [223] = {.lex_state = 73, .external_lex_state = 3},
  [224] = {.lex_state = 73, .external_lex_state = 3},
  [225] = {.lex_state = 73, .external_lex_state = 3},
  [226] = {.lex_state = 73, .external_lex_state = 3},
  [227] = {.lex_state = 73, .external_lex_state = 3},
  [228] = {.lex_state = 73, .external_lex_state = 3},
  [229] = {.lex_state = 73, .external_lex_state = 3},
  [230] = {.lex_state = 73, .external_lex_state = 3},
  [231] = {.lex_state = 73, .external_lex_state = 3},
  [232] = {.lex_state = 73, .external_lex_state = 3},
  [233] = {.lex_state = 73, .external_lex_state = 3},
  [234] = {.lex_state = 73, .external_lex_state = 3},
  [235] = {.lex_state = 73, .external_lex_state = 3},
  [236] = {.lex_state = 73, .external_lex_state = 3},
  [237] = {.lex_state = 73, .external_lex_state = 3},
  [238] = {.lex_state = 73, .external_lex_state = 3},
  [239] = {.lex_state = 73, .external_lex_state = 3},
  [240] = {.lex_state = 73, .external_lex_state = 3},
  [241] = {.lex_state = 73, .external_lex_state = 3},
  [242] = {.lex_state = 73, .external_lex_state = 3},
  [243] = {.lex_state = 73, .external_lex_state = 3},
  [244] = {.lex_state = 73, .external_lex_state = 3},
  [245] = {.lex_state = 73, .external_lex_state = 3},
  [246] = {.lex_state = 73, .external_lex_state = 3},
  [247] = {.lex_state = 73, .external_lex_state = 3},
  [248] = {.lex_state = 73, .external_lex_state = 3},
  [249] = {.lex_state = 73, .external_lex_state = 3},
  [250] = {.lex_state = 73, .external_lex_state = 3},
  [251] = {.lex_state = 73, .external_lex_state = 3},
  [252] = {.lex_state = 73, .external_lex_state = 3},
  [253] = {.lex_state = 73, .external_lex_state = 3},
  [254] = {.lex_state = 73, .external_lex_state = 3},
  [255] = {.lex_state = 73, .external_lex_state = 3},
  [256] = {.lex_state = 73, .external_lex_state = 3},
  [257] = {.lex_state = 73, .external_lex_state = 3},
  [258] = {.lex_state = 73, .external_lex_state = 3},
  [259] = {.lex_state = 73, .external_lex_state = 3},
  [260] = {.lex_state = 73, .external_lex_state = 3},
  [261] = {.lex_state = 73, .external_lex_state = 3},
  [262] = {.lex_state = 73, .external_lex_state = 3},
  [263] = {.lex_state = 73, .external_lex_state = 3},
  [264] = {.lex_state = 12, .external_lex_state = 3},
  [265] = {.lex_state = 12, .external_lex_state = 3},
  [266] = {.lex_state = 73, .external_lex_state = 3},
  [267] = {.lex_state = 73, .external_lex_state = 3},
  [268] = {.lex_state = 12, .external_lex_state = 3},
  [269] = {.lex_state = 73, .external_lex_state = 3},
  [270] = {.lex_state = 73, .external_lex_state = 3},
  [271] = {.lex_state = 73, .external_lex_state = 3},
  [272] = {.lex_state = 73, .external_lex_state = 3},
  [273] = {.lex_state = 73, .external_lex_state = 3},
  [274] = {.lex_state = 12, .external_lex_state = 3},
  [275] = {.lex_state = 73, .external_lex_state = 3},
  [276] = {.lex_state = 12, .external_lex_state = 3},
  [277] = {.lex_state = 73, .external_lex_state = 3},
  [278] = {.lex_state = 73, .external_lex_state = 3},
  [279] = {.lex_state = 73, .external_lex_state = 3},
  [280] = {.lex_state = 73, .external_lex_state = 3},
  [281] = {.lex_state = 12, .external_lex_state = 3},
  [282] = {.lex_state = 73, .external_lex_state = 3},
  [283] = {.lex_state = 73, .external_lex_state = 3},
  [284] = {.lex_state = 73, .external_lex_state = 3},
  [285] = {.lex_state = 73, .external_lex_state = 3},
  [286] = {.lex_state = 73, .external_lex_state = 3},
  [287] = {.lex_state = 73, .external_lex_state = 3},
  [288] = {.lex_state = 73, .external_lex_state = 3},
  [289] = {.lex_state = 73, .external_lex_state = 3},
  [290] = {.lex_state = 73, .external_lex_state = 3},
  [291] = {.lex_state = 73, .external_lex_state = 3},
  [292] = {.lex_state = 73, .external_lex_state = 3},
  [293] = {.lex_state = 73, .external_lex_state = 3},
  [294] = {.lex_state = 73, .external_lex_state = 3},
  [295] = {.lex_state = 73, .external_lex_state = 3},
  [296] = {.lex_state = 73, .external_lex_state = 3},
  [297] = {.lex_state = 73, .external_lex_state = 3},
  [298] = {.lex_state = 73, .external_lex_state = 3},
  [299] = {.lex_state = 73, .external_lex_state = 3},
  [300] = {.lex_state = 73, .external_lex_state = 3},
  [301] = {.lex_state = 73, .external_lex_state = 3},
  [302] = {.lex_state = 73, .external_lex_state = 3},
  [303] = {.lex_state = 73, .external_lex_state = 3},
  [304] = {.lex_state = 73, .external_lex_state = 3},
  [305] = {.lex_state = 73, .external_lex_state = 3},
  [306] = {.lex_state = 73, .external_lex_state = 3},
  [307] = {.lex_state = 73, .external_lex_state = 3},
  [308] = {.lex_state = 73, .external_lex_state = 3},
  [309] = {.lex_state = 73, .external_lex_state = 3},
  [310] = {.lex_state = 73, .external_lex_state = 3},
  [311] = {.lex_state = 73, .external_lex_state = 3},
  [312] = {.lex_state = 73, .external_lex_state = 3},
  [313] = {.lex_state = 73, .external_lex_state = 3},
  [314] = {.lex_state = 73, .external_lex_state = 3},
  [315] = {.lex_state = 73, .external_lex_state = 3},
  [316] = {.lex_state = 73, .external_lex_state = 3},
  [317] = {.lex_state = 73, .external_lex_state = 3},
  [318] = {.lex_state = 73, .external_lex_state = 3},
  [319] = {.lex_state = 73, .external_lex_state = 3},
  [320] = {.lex_state = 73, .external_lex_state = 3},
  [321] = {.lex_state = 73, .external_lex_state = 3},
  [322] = {.lex_state = 73, .external_lex_state = 3},
  [323] = {.lex_state = 73, .external_lex_state = 3},
  [324] = {.lex_state = 73, .external_lex_state = 3},
  [325] = {.lex_state = 73, .external_lex_state = 3},
  [326] = {.lex_state = 73, .external_lex_state = 3},
  [327] = {.lex_state = 73, .external_lex_state = 3},
  [328] = {.lex_state = 73, .external_lex_state = 3},
  [329] = {.lex_state = 12, .external_lex_state = 3},
  [330] = {.lex_state = 4, .external_lex_state = 4},
  [331] = {.lex_state = 4, .external_lex_state = 4},
  [332] = {.lex_state = 4, .external_lex_state = 4},
  [333] = {.lex_state = 4, .external_lex_state = 4},
  [334] = {.lex_state = 4, .external_lex_state = 4},
  [335] = {.lex_state = 4, .external_lex_state = 4},
  [336] = {.lex_state = 4, .external_lex_state = 4},
  [337] = {.lex_state = 4, .external_lex_state = 4},
  [338] = {.lex_state = 4, .external_lex_state = 4},
  [339] = {.lex_state = 4, .external_lex_state = 4},
  [340] = {.lex_state = 4, .external_lex_state = 4},
  [341] = {.lex_state = 4, .external_lex_state = 4},
  [342] = {.lex_state = 4, .external_lex_state = 4},
  [343] = {.lex_state = 4, .external_lex_state = 4},
  [344] = {.lex_state = 4, .external_lex_state = 4},
  [345] = {.lex_state = 4, .external_lex_state = 4},
  [346] = {.lex_state = 4, .external_lex_state = 4},
  [347] = {.lex_state = 6, .external_lex_state = 4},
  [348] = {.lex_state = 4, .external_lex_state = 4},
  [349] = {.lex_state = 4, .external_lex_state = 4},
  [350] = {.lex_state = 4, .external_lex_state = 4},
  [351] = {.lex_state = 4, .external_lex_state = 4},
  [352] = {.lex_state = 4, .external_lex_state = 4},
  [353] = {.lex_state = 4, .external_lex_state = 4},
  [354] = {.lex_state = 4, .external_lex_state = 4},
  [355] = {.lex_state = 4, .external_lex_state = 4},
  [356] = {.lex_state = 4, .external_lex_state = 4},
  [357] = {.lex_state = 4, .external_lex_state = 4},
  [358] = {.lex_state = 4, .external_lex_state = 4},
  [359] = {.lex_state = 4, .external_lex_state = 4},
  [360] = {.lex_state = 4, .external_lex_state = 4},
  [361] = {.lex_state = 4, .external_lex_state = 4},
  [362] = {.lex_state = 4, .external_lex_state = 4},
  [363] = {.lex_state = 4, .external_lex_state = 4},
  [364] = {.lex_state = 4, .external_lex_state = 4},
  [365] = {.lex_state = 4, .external_lex_state = 4},
  [366] = {.lex_state = 6, .external_lex_state = 4},
  [367] = {.lex_state = 6, .external_lex_state = 4},
  [368] = {.lex_state = 6, .external_lex_state = 4},
  [369] = {.lex_state = 6, .external_lex_state = 4},
  [370] = {.lex_state = 6, .external_lex_state = 4},
  [371] = {.lex_state = 6, .external_lex_state = 4},
  [372] = {.lex_state = 6, .external_lex_state = 4},
  [373] = {.lex_state = 6, .external_lex_state = 4},
  [374] = {.lex_state = 6, .external_lex_state = 4},
  [375] = {.lex_state = 6, .external_lex_state = 4},
  [376] = {.lex_state = 6, .external_lex_state = 4},
  [377] = {.lex_state = 6, .external_lex_state = 4},
  [378] = {.lex_state = 6, .external_lex_state = 4},
  [379] = {.lex_state = 6, .external_lex_state = 4},
  [380] = {.lex_state = 6, .external_lex_state = 4},
  [381] = {.lex_state = 6, .external_lex_state = 4},
  [382] = {.lex_state = 6, .external_lex_state = 4},
  [383] = {.lex_state = 6, .external_lex_state = 4},
  [384] = {.lex_state = 6, .external_lex_state = 4},
  [385] = {.lex_state = 6, .external_lex_state = 4},
  [386] = {.lex_state = 6, .external_lex_state = 4},
  [387] = {.lex_state = 6, .external_lex_state = 4},
  [388] = {.lex_state = 6, .external_lex_state = 4},
  [389] = {.lex_state = 6, .external_lex_state = 4},
  [390] = {.lex_state = 6, .external_lex_state = 4},
  [391] = {.lex_state = 6, .external_lex_state = 4},
  [392] = {.lex_state = 6, .external_lex_state = 4},
  [393] = {.lex_state = 6, .external_lex_state = 4},
  [394] = {.lex_state = 6, .external_lex_state = 4},
  [395] = {.lex_state = 6, .external_lex_state = 4},
  [396] = {.lex_state = 6, .external_lex_state = 4},
  [397] = {.lex_state = 6, .external_lex_state = 4},
  [398] = {.lex_state = 6, .external_lex_state = 4},
  [399] = {.lex_state = 6, .external_lex_state = 4},
  [400] = {.lex_state = 6, .external_lex_state = 4},
  [401] = {.lex_state = 6, .external_lex_state = 4},
  [402] = {.lex_state = 6, .external_lex_state = 4},
  [403] = {.lex_state = 6, .external_lex_state = 4},
  [404] = {.lex_state = 6, .external_lex_state = 4},
  [405] = {.lex_state = 6, .external_lex_state = 4},
  [406] = {.lex_state = 6, .external_lex_state = 4},
  [407] = {.lex_state = 6, .external_lex_state = 4},
  [408] = {.lex_state = 13, .external_lex_state = 4},
  [409] = {.lex_state = 13, .external_lex_state = 4},
  [410] = {.lex_state = 13, .external_lex_state = 4},
  [411] = {.lex_state = 4, .external_lex_state = 4},
  [412] = {.lex_state = 4, .external_lex_state = 4},
  [413] = {.lex_state = 4, .external_lex_state = 4},
  [414] = {.lex_state = 71, .external_lex_state = 3},
  [415] = {.lex_state = 4, .external_lex_state = 4},
  [416] = {.lex_state = 4, .external_lex_state = 4},
  [417] = {.lex_state = 6, .external_lex_state = 4},
  [418] = {.lex_state = 4, .external_lex_state = 4},
  [419] = {.lex_state = 4, .external_lex_state = 4},
  [420] = {.lex_state = 71, .external_lex_state = 3},
  [421] = {.lex_state = 4, .external_lex_state = 4},
  [422] = {.lex_state = 4, .external_lex_state = 4},
  [423] = {.lex_state = 4, .external_lex_state = 4},
  [424] = {.lex_state = 4, .external_lex_state = 4},
  [425] = {.lex_state = 4, .external_lex_state = 4},
  [426] = {.lex_state = 71, .external_lex_state = 3},
  [427] = {.lex_state = 4, .external_lex_state = 4},
  [428] = {.lex_state = 4, .external_lex_state = 4},
  [429] = {.lex_state = 71, .external_lex_state = 3},
  [430] = {.lex_state = 4, .external_lex_state = 4},
  [431] = {.lex_state = 4, .external_lex_state = 4},
  [432] = {.lex_state = 4, .external_lex_state = 4},
  [433] = {.lex_state = 4, .external_lex_state = 4},
  [434] = {.lex_state = 4, .external_lex_state = 4},
  [435] = {.lex_state = 6, .external_lex_state = 4},
  [436] = {.lex_state = 4, .external_lex_state = 4},
  [437] = {.lex_state = 4, .external_lex_state = 4},
  [438] = {.lex_state = 4, .external_lex_state = 4},
  [439] = {.lex_state = 4, .external_lex_state = 4},
  [440] = {.lex_state = 4, .external_lex_state = 4},
  [441] = {.lex_state = 4, .external_lex_state = 4},
  [442] = {.lex_state = 71, .external_lex_state = 3},
  [443] = {.lex_state = 13, .external_lex_state = 4},
  [444] = {.lex_state = 71, .external_lex_state = 3},
  [445] = {.lex_state = 4, .external_lex_state = 4},
  [446] = {.lex_state = 4, .external_lex_state = 4},
  [447] = {.lex_state = 4, .external_lex_state = 4},
  [448] = {.lex_state = 71, .external_lex_state = 3},
  [449] = {.lex_state = 71, .external_lex_state = 3},
  [450] = {.lex_state = 71, .external_lex_state = 3},
  [451] = {.lex_state = 71, .external_lex_state = 3},
  [452] = {.lex_state = 71, .external_lex_state = 3},
  [453] = {.lex_state = 6, .external_lex_state = 4},
  [454] = {.lex_state = 6, .external_lex_state = 4},
  [455] = {.lex_state = 6, .external_lex_state = 4},
  [456] = {.lex_state = 71, .external_lex_state = 3},
  [457] = {.lex_state = 6, .external_lex_state = 4},
  [458] = {.lex_state = 6, .external_lex_state = 4},
  [459] = {.lex_state = 71, .external_lex_state = 3},
  [460] = {.lex_state = 6, .external_lex_state = 4},
  [461] = {.lex_state = 6, .external_lex_state = 4},
  [462] = {.lex_state = 6, .external_lex_state = 4},
  [463] = {.lex_state = 6, .external_lex_state = 4},
  [464] = {.lex_state = 6, .external_lex_state = 4},
  [465] = {.lex_state = 6, .external_lex_state = 4},
  [466] = {.lex_state = 6, .external_lex_state = 4},
  [467] = {.lex_state = 71, .external_lex_state = 3},
  [468] = {.lex_state = 71, .external_lex_state = 3},
  [469] = {.lex_state = 71, .external_lex_state = 3},
  [470] = {.lex_state = 71, .external_lex_state = 3},
  [471] = {.lex_state = 71, .external_lex_state = 3},
  [472] = {.lex_state = 71, .external_lex_state = 3},
  [473] = {.lex_state = 71, .external_lex_state = 3},
  [474] = {.lex_state = 71, .external_lex_state = 3},
  [475] = {.lex_state = 71, .external_lex_state = 3},
  [476] = {.lex_state = 71, .external_lex_state = 3},
  [477] = {.lex_state = 71, .external_lex_state = 3},
  [478] = {.lex_state = 71, .external_lex_state = 3},
  [479] = {.lex_state = 71, .external_lex_state = 3},
  [480] = {.lex_state = 71, .external_lex_state = 3},
  [481] = {.lex_state = 71, .external_lex_state = 3},
  [482] = {.lex_state = 71, .external_lex_state = 3},
  [483] = {.lex_state = 71, .external_lex_state = 3},
  [484] = {.lex_state = 71, .external_lex_state = 3},
  [485] = {.lex_state = 71, .external_lex_state = 3},
  [486] = {.lex_state = 71, .external_lex_state = 3},
  [487] = {.lex_state = 71, .external_lex_state = 3},
  [488] = {.lex_state = 13, .external_lex_state = 4},
  [489] = {.lex_state = 13, .external_lex_state = 4},
  [490] = {.lex_state = 13, .external_lex_state = 4},
  [491] = {.lex_state = 13, .external_lex_state = 4},
  [492] = {.lex_state = 71, .external_lex_state = 3},
  [493] = {.lex_state = 71, .external_lex_state = 3},
  [494] = {.lex_state = 13, .external_lex_state = 4},
  [495] = {.lex_state = 13, .external_lex_state = 4},
  [496] = {.lex_state = 13, .external_lex_state = 4},
  [497] = {.lex_state = 13, .external_lex_state = 4},
  [498] = {.lex_state = 13, .external_lex_state = 4},
  [499] = {.lex_state = 13, .external_lex_state = 4},
  [500] = {.lex_state = 71, .external_lex_state = 3},
  [501] = {.lex_state = 15, .external_lex_state = 3},
  [502] = {.lex_state = 15, .external_lex_state = 3},
  [503] = {.lex_state = 15, .external_lex_state = 3},
  [504] = {.lex_state = 15, .external_lex_state = 3},
  [505] = {.lex_state = 5, .external_lex_state = 3},
  [506] = {.lex_state = 13, .external_lex_state = 3},
  [507] = {.lex_state = 13, .external_lex_state = 3},
  [508] = {.lex_state = 72, .external_lex_state = 4},
  [509] = {.lex_state = 13, .external_lex_state = 3},
  [510] = {.lex_state = 72, .external_lex_state = 4},
  [511] = {.lex_state = 13, .external_lex_state = 3},
  [512] = {.lex_state = 13, .external_lex_state = 3},
  [513] = {.lex_state = 13, .external_lex_state = 3},
  [514] = {.lex_state = 72, .external_lex_state = 4},
  [515] = {.lex_state = 13, .external_lex_state = 3},
  [516] = {.lex_state = 13, .external_lex_state = 3},
  [517] = {.lex_state = 72, .external_lex_state = 4},
  [518] = {.lex_state = 72, .external_lex_state = 4},
  [519] = {.lex_state = 72, .external_lex_state = 4},
  [520] = {.lex_state = 72, .external_lex_state = 4},
  [521] = {.lex_state = 72, .external_lex_state = 4},
  [522] = {.lex_state = 72, .external_lex_state = 4},
  [523] = {.lex_state = 72, .external_lex_state = 4},
  [524] = {.lex_state = 72, .external_lex_state = 4},
  [525] = {.lex_state = 72, .external_lex_state = 4},
  [526] = {.lex_state = 72, .external_lex_state = 4},
  [527] = {.lex_state = 72, .external_lex_state = 4},
  [528] = {.lex_state = 72, .external_lex_state = 4},
  [529] = {.lex_state = 72, .external_lex_state = 4},
  [530] = {.lex_state = 72, .external_lex_state = 4},
  [531] = {.lex_state = 13, .external_lex_state = 3},
  [532] = {.lex_state = 13, .external_lex_state = 3},
  [533] = {.lex_state = 13, .external_lex_state = 3},
  [534] = {.lex_state = 13, .external_lex_state = 3},
  [535] = {.lex_state = 13, .external_lex_state = 3},
  [536] = {.lex_state = 13, .external_lex_state = 3},
  [537] = {.lex_state = 13, .external_lex_state = 3},
  [538] = {.lex_state = 13, .external_lex_state = 3},
  [539] = {.lex_state = 13, .external_lex_state = 3},
  [540] = {.lex_state = 11, .external_lex_state = 5},
  [541] = {.lex_state = 11, .external_lex_state = 5},
  [542] = {.lex_state = 11, .external_lex_state = 5},
  [543] = {.lex_state = 13, .external_lex_state = 3},
  [544] = {.lex_state = 11, .external_lex_state = 5},
  [545] = {.lex_state = 11, .external_lex_state = 5},
  [546] = {.lex_state = 11, .external_lex_state = 5},
  [547] = {.lex_state = 13, .external_lex_state = 3},
  [548] = {.lex_state = 13, .external_lex_state = 3},
  [549] = {.lex_state = 13, .external_lex_state = 3},
  [550] = {.lex_state = 13, .external_lex_state = 3},
  [551] = {.lex_state = 71, .external_lex_state = 3},
  [552] = {.lex_state = 11, .external_lex_state = 5},
  [553] = {.lex_state = 11, .external_lex_state = 5},
  [554] = {.lex_state = 13, .external_lex_state = 4},
  [555] = {.lex_state = 11, .external_lex_state = 5},
  [556] = {.lex_state = 71, .external_lex_state = 3},
  [557] = {.lex_state = 71, .external_lex_state = 3},
  [558] = {.lex_state = 71, .external_lex_state = 3},
  [559] = {.lex_state = 11, .external_lex_state = 5},
  [560] = {.lex_state = 11, .external_lex_state = 5},
  [561] = {.lex_state = 11, .external_lex_state = 5},
  [562] = {.lex_state = 13, .external_lex_state = 4},
  [563] = {.lex_state = 11, .external_lex_state = 5},
  [564] = {.lex_state = 71, .external_lex_state = 3},
  [565] = {.lex_state = 71, .external_lex_state = 3},
  [566] = {.lex_state = 11, .external_lex_state = 5},
  [567] = {.lex_state = 11, .external_lex_state = 5},
  [568] = {.lex_state = 71, .external_lex_state = 3},
  [569] = {.lex_state = 71, .external_lex_state = 3},
  [570] = {.lex_state = 13, .external_lex_state = 4},
  [571] = {.lex_state = 11, .external_lex_state = 5},
  [572] = {.lex_state = 71, .external_lex_state = 3},
  [573] = {.lex_state = 71, .external_lex_state = 3},
  [574] = {.lex_state = 13, .external_lex_state = 4},
  [575] = {.lex_state = 13, .external_lex_state = 4},
  [576] = {.lex_state = 71, .external_lex_state = 3},
  [577] = {.lex_state = 13, .external_lex_state = 4},
  [578] = {.lex_state = 71, .external_lex_state = 3},
  [579] = {.lex_state = 71, .external_lex_state = 3},
  [580] = {.lex_state = 71, .external_lex_state = 3},
  [581] = {.lex_state = 71, .external_lex_state = 3},
  [582] = {.lex_state = 71, .external_lex_state = 3},
  [583] = {.lex_state = 71, .external_lex_state = 3},
  [584] = {.lex_state = 71, .external_lex_state = 3},
  [585] = {.lex_state = 71, .external_lex_state = 3},
  [586] = {.lex_state = 71, .external_lex_state = 3},
  [587] = {.lex_state = 71, .external_lex_state = 3},
  [588] = {.lex_state = 71, .external_lex_state = 3},
  [589] = {.lex_state = 71, .external_lex_state = 3},
  [590] = {.lex_state = 71, .external_lex_state = 3},
  [591] = {.lex_state = 71, .external_lex_state = 3},
  [592] = {.lex_state = 71, .external_lex_state = 3},
  [593] = {.lex_state = 71, .external_lex_state = 3},
  [594] = {.lex_state = 71, .external_lex_state = 3},
  [595] = {.lex_state = 71, .external_lex_state = 3},
  [596] = {.lex_state = 73, .external_lex_state = 3},
  [597] = {.lex_state = 73, .external_lex_state = 3},
  [598] = {.lex_state = 13, .external_lex_state = 4},
  [599] = {.lex_state = 73, .external_lex_state = 3},
  [600] = {.lex_state = 73, .external_lex_state = 3},
  [601] = {.lex_state = 73, .external_lex_state = 3},
  [602] = {.lex_state = 73, .external_lex_state = 3},
  [603] = {.lex_state = 73, .external_lex_state = 3},
  [604] = {.lex_state = 73, .external_lex_state = 3},
  [605] = {.lex_state = 73, .external_lex_state = 3},
  [606] = {.lex_state = 73, .external_lex_state = 3},
  [607] = {.lex_state = 73, .external_lex_state = 3},
  [608] = {.lex_state = 73, .external_lex_state = 3},
  [609] = {.lex_state = 73, .external_lex_state = 3},
  [610] = {.lex_state = 73, .external_lex_state = 3},
  [611] = {.lex_state = 73, .external_lex_state = 3},
  [612] = {.lex_state = 73, .external_lex_state = 3},
  [613] = {.lex_state = 73, .external_lex_state = 3},
  [614] = {.lex_state = 73, .external_lex_state = 3},
  [615] = {.lex_state = 73, .external_lex_state = 3},
  [616] = {.lex_state = 73, .external_lex_state = 3},
  [617] = {.lex_state = 73, .external_lex_state = 3},
  [618] = {.lex_state = 73, .external_lex_state = 3},
  [619] = {.lex_state = 73, .external_lex_state = 3},
  [620] = {.lex_state = 73, .external_lex_state = 3},
  [621] = {.lex_state = 73, .external_lex_state = 3},
  [622] = {.lex_state = 73, .external_lex_state = 3},
  [623] = {.lex_state = 73, .external_lex_state = 3},
  [624] = {.lex_state = 73, .external_lex_state = 3},
  [625] = {.lex_state = 73, .external_lex_state = 3},
  [626] = {.lex_state = 73, .external_lex_state = 3},
  [627] = {.lex_state = 73, .external_lex_state = 3},
  [628] = {.lex_state = 73, .external_lex_state = 3},
  [629] = {.lex_state = 73, .external_lex_state = 3},
  [630] = {.lex_state = 73, .external_lex_state = 3},
  [631] = {.lex_state = 73, .external_lex_state = 3},
  [632] = {.lex_state = 73, .external_lex_state = 3},
  [633] = {.lex_state = 73, .external_lex_state = 3},
  [634] = {.lex_state = 73, .external_lex_state = 3},
  [635] = {.lex_state = 73, .external_lex_state = 3},
  [636] = {.lex_state = 73, .external_lex_state = 3},
  [637] = {.lex_state = 73, .external_lex_state = 3},
  [638] = {.lex_state = 73, .external_lex_state = 3},
  [639] = {.lex_state = 73, .external_lex_state = 3},
  [640] = {.lex_state = 73, .external_lex_state = 3},
  [641] = {.lex_state = 73, .external_lex_state = 3},
  [642] = {.lex_state = 73, .external_lex_state = 3},
  [643] = {.lex_state = 73, .external_lex_state = 3},
  [644] = {.lex_state = 73, .external_lex_state = 3},
  [645] = {.lex_state = 73, .external_lex_state = 3},
  [646] = {.lex_state = 11},
  [647] = {.lex_state = 73, .external_lex_state = 3},
  [648] = {.lex_state = 73, .external_lex_state = 3},
  [649] = {.lex_state = 73, .external_lex_state = 3},
  [650] = {.lex_state = 73, .external_lex_state = 3},
  [651] = {.lex_state = 73, .external_lex_state = 3},
  [652] = {.lex_state = 73, .external_lex_state = 3},
  [653] = {.lex_state = 73, .external_lex_state = 3},
  [654] = {.lex_state = 73, .external_lex_state = 3},
  [655] = {.lex_state = 73, .external_lex_state = 3},
  [656] = {.lex_state = 73, .external_lex_state = 3},
  [657] = {.lex_state = 73, .external_lex_state = 3},
  [658] = {.lex_state = 73, .external_lex_state = 3},
  [659] = {.lex_state = 73, .external_lex_state = 3},
  [660] = {.lex_state = 73, .external_lex_state = 3},
  [661] = {.lex_state = 73, .external_lex_state = 3},
  [662] = {.lex_state = 73, .external_lex_state = 3},
  [663] = {.lex_state = 73, .external_lex_state = 3},
  [664] = {.lex_state = 73, .external_lex_state = 3},
  [665] = {.lex_state = 73, .external_lex_state = 3},
  [666] = {.lex_state = 73, .external_lex_state = 3},
  [667] = {.lex_state = 73, .external_lex_state = 3},
  [668] = {.lex_state = 73, .external_lex_state = 3},
  [669] = {.lex_state = 73, .external_lex_state = 3},
  [670] = {.lex_state = 73, .external_lex_state = 3},
  [671] = {.lex_state = 73, .external_lex_state = 3},
  [672] = {.lex_state = 73, .external_lex_state = 3},
  [673] = {.lex_state = 73, .external_lex_state = 3},
  [674] = {.lex_state = 73, .external_lex_state = 3},
  [675] = {.lex_state = 73, .external_lex_state = 3},
  [676] = {.lex_state = 73, .external_lex_state = 3},
  [677] = {.lex_state = 73, .external_lex_state = 3},
  [678] = {.lex_state = 73, .external_lex_state = 3},
  [679] = {.lex_state = 73, .external_lex_state = 3},
  [680] = {.lex_state = 73, .external_lex_state = 3},
  [681] = {.lex_state = 73, .external_lex_state = 3},
  [682] = {.lex_state = 73, .external_lex_state = 3},
  [683] = {.lex_state = 73, .external_lex_state = 3},
  [684] = {.lex_state = 73, .external_lex_state = 3},
  [685] = {.lex_state = 73, .external_lex_state = 3},
  [686] = {.lex_state = 73, .external_lex_state = 3},
  [687] = {.lex_state = 73, .external_lex_state = 3},
  [688] = {.lex_state = 73, .external_lex_state = 3},
  [689] = {.lex_state = 73, .external_lex_state = 3},
  [690] = {.lex_state = 73, .external_lex_state = 3},
  [691] = {.lex_state = 73, .external_lex_state = 3},
  [692] = {.lex_state = 73, .external_lex_state = 3},
  [693] = {.lex_state = 73, .external_lex_state = 3},
  [694] = {.lex_state = 73, .external_lex_state = 3},
  [695] = {.lex_state = 73, .external_lex_state = 3},
  [696] = {.lex_state = 73, .external_lex_state = 3},
  [697] = {.lex_state = 73, .external_lex_state = 3},
  [698] = {.lex_state = 73, .external_lex_state = 3},
  [699] = {.lex_state = 73, .external_lex_state = 3},
  [700] = {.lex_state = 73, .external_lex_state = 3},
  [701] = {.lex_state = 73, .external_lex_state = 3},
  [702] = {.lex_state = 73, .external_lex_state = 3},
  [703] = {.lex_state = 73, .external_lex_state = 3},
  [704] = {.lex_state = 73, .external_lex_state = 3},
  [705] = {.lex_state = 73, .external_lex_state = 3},
  [706] = {.lex_state = 73, .external_lex_state = 3},
  [707] = {.lex_state = 73, .external_lex_state = 3},
  [708] = {.lex_state = 73, .external_lex_state = 3},
  [709] = {.lex_state = 73, .external_lex_state = 3},
  [710] = {.lex_state = 73, .external_lex_state = 3},
  [711] = {.lex_state = 73, .external_lex_state = 3},
  [712] = {.lex_state = 73, .external_lex_state = 3},
  [713] = {.lex_state = 73, .external_lex_state = 3},
  [714] = {.lex_state = 73, .external_lex_state = 3},
  [715] = {.lex_state = 73, .external_lex_state = 3},
  [716] = {.lex_state = 73, .external_lex_state = 3},
  [717] = {.lex_state = 73, .external_lex_state = 3},
  [718] = {.lex_state = 73, .external_lex_state = 3},
  [719] = {.lex_state = 73, .external_lex_state = 3},
  [720] = {.lex_state = 73, .external_lex_state = 3},
  [721] = {.lex_state = 73, .external_lex_state = 3},
  [722] = {.lex_state = 73, .external_lex_state = 3},
  [723] = {.lex_state = 73, .external_lex_state = 3},
  [724] = {.lex_state = 73, .external_lex_state = 3},
  [725] = {.lex_state = 73, .external_lex_state = 3},
  [726] = {.lex_state = 73, .external_lex_state = 3},
  [727] = {.lex_state = 73, .external_lex_state = 3},
  [728] = {.lex_state = 73, .external_lex_state = 3},
  [729] = {.lex_state = 73, .external_lex_state = 3},
  [730] = {.lex_state = 73, .external_lex_state = 3},
  [731] = {.lex_state = 73, .external_lex_state = 3},
  [732] = {.lex_state = 73, .external_lex_state = 3},
  [733] = {.lex_state = 73, .external_lex_state = 3},
  [734] = {.lex_state = 73, .external_lex_state = 3},
  [735] = {.lex_state = 73, .external_lex_state = 3},
  [736] = {.lex_state = 73, .external_lex_state = 3},
  [737] = {.lex_state = 73, .external_lex_state = 3},
  [738] = {.lex_state = 73, .external_lex_state = 3},
  [739] = {.lex_state = 73, .external_lex_state = 3},
  [740] = {.lex_state = 73, .external_lex_state = 3},
  [741] = {.lex_state = 73, .external_lex_state = 3},
  [742] = {.lex_state = 73, .external_lex_state = 3},
  [743] = {.lex_state = 73, .external_lex_state = 3},
  [744] = {.lex_state = 73, .external_lex_state = 3},
  [745] = {.lex_state = 73, .external_lex_state = 3},
  [746] = {.lex_state = 73, .external_lex_state = 3},
  [747] = {.lex_state = 73, .external_lex_state = 3},
  [748] = {.lex_state = 73, .external_lex_state = 3},
  [749] = {.lex_state = 73, .external_lex_state = 3},
  [750] = {.lex_state = 73, .external_lex_state = 3},
  [751] = {.lex_state = 73, .external_lex_state = 3},
  [752] = {.lex_state = 73, .external_lex_state = 3},
  [753] = {.lex_state = 73, .external_lex_state = 3},
  [754] = {.lex_state = 73, .external_lex_state = 3},
  [755] = {.lex_state = 73, .external_lex_state = 3},
  [756] = {.lex_state = 73, .external_lex_state = 3},
  [757] = {.lex_state = 73, .external_lex_state = 3},
  [758] = {.lex_state = 73, .external_lex_state = 3},
  [759] = {.lex_state = 73, .external_lex_state = 3},
  [760] = {.lex_state = 73, .external_lex_state = 3},
  [761] = {.lex_state = 73, .external_lex_state = 3},
  [762] = {.lex_state = 73, .external_lex_state = 3},
  [763] = {.lex_state = 73, .external_lex_state = 3},
  [764] = {.lex_state = 73, .external_lex_state = 3},
  [765] = {.lex_state = 73, .external_lex_state = 3},
  [766] = {.lex_state = 73, .external_lex_state = 3},
  [767] = {.lex_state = 73, .external_lex_state = 3},
  [768] = {.lex_state = 73, .external_lex_state = 3},
  [769] = {.lex_state = 73, .external_lex_state = 3},
  [770] = {.lex_state = 73, .external_lex_state = 3},
  [771] = {.lex_state = 73, .external_lex_state = 3},
  [772] = {.lex_state = 73, .external_lex_state = 3},
  [773] = {.lex_state = 73, .external_lex_state = 3},
  [774] = {.lex_state = 73, .external_lex_state = 3},
  [775] = {.lex_state = 73, .external_lex_state = 3},
  [776] = {.lex_state = 73, .external_lex_state = 3},
  [777] = {.lex_state = 11},
  [778] = {.lex_state = 73, .external_lex_state = 3},
  [779] = {.lex_state = 73, .external_lex_state = 3},
  [780] = {.lex_state = 73, .external_lex_state = 3},
  [781] = {.lex_state = 73, .external_lex_state = 3},
  [782] = {.lex_state = 73, .external_lex_state = 3},
  [783] = {.lex_state = 73, .external_lex_state = 3},
  [784] = {.lex_state = 73, .external_lex_state = 3},
  [785] = {.lex_state = 73, .external_lex_state = 3},
  [786] = {.lex_state = 73, .external_lex_state = 3},
  [787] = {.lex_state = 73, .external_lex_state = 3},
  [788] = {.lex_state = 73, .external_lex_state = 3},
  [789] = {.lex_state = 73, .external_lex_state = 3},
  [790] = {.lex_state = 73, .external_lex_state = 3},
  [791] = {.lex_state = 73, .external_lex_state = 3},
  [792] = {.lex_state = 73, .external_lex_state = 3},
  [793] = {.lex_state = 73, .external_lex_state = 3},
  [794] = {.lex_state = 73, .external_lex_state = 3},
  [795] = {.lex_state = 73, .external_lex_state = 3},
  [796] = {.lex_state = 73, .external_lex_state = 3},
  [797] = {.lex_state = 73, .external_lex_state = 3},
  [798] = {.lex_state = 73, .external_lex_state = 3},
  [799] = {.lex_state = 73, .external_lex_state = 3},
  [800] = {.lex_state = 73, .external_lex_state = 3},
  [801] = {.lex_state = 73, .external_lex_state = 3},
  [802] = {.lex_state = 73, .external_lex_state = 3},
  [803] = {.lex_state = 73, .external_lex_state = 3},
  [804] = {.lex_state = 73, .external_lex_state = 3},
  [805] = {.lex_state = 73, .external_lex_state = 3},
  [806] = {.lex_state = 73, .external_lex_state = 3},
  [807] = {.lex_state = 73, .external_lex_state = 3},
  [808] = {.lex_state = 73, .external_lex_state = 3},
  [809] = {.lex_state = 73, .external_lex_state = 3},
  [810] = {.lex_state = 73, .external_lex_state = 3},
  [811] = {.lex_state = 73, .external_lex_state = 3},
  [812] = {.lex_state = 73, .external_lex_state = 3},
  [813] = {.lex_state = 73, .external_lex_state = 3},
  [814] = {.lex_state = 73, .external_lex_state = 3},
  [815] = {.lex_state = 73, .external_lex_state = 3},
  [816] = {.lex_state = 73, .external_lex_state = 3},
  [817] = {.lex_state = 73, .external_lex_state = 3},
  [818] = {.lex_state = 73, .external_lex_state = 3},
  [819] = {.lex_state = 73, .external_lex_state = 3},
  [820] = {.lex_state = 73, .external_lex_state = 3},
  [821] = {.lex_state = 73, .external_lex_state = 3},
  [822] = {.lex_state = 73, .external_lex_state = 3},
  [823] = {.lex_state = 73, .external_lex_state = 3},
  [824] = {.lex_state = 73, .external_lex_state = 3},
  [825] = {.lex_state = 73, .external_lex_state = 3},
  [826] = {.lex_state = 73, .external_lex_state = 3},
  [827] = {.lex_state = 73, .external_lex_state = 3},
  [828] = {.lex_state = 73, .external_lex_state = 3},
  [829] = {.lex_state = 73, .external_lex_state = 3},
  [830] = {.lex_state = 73, .external_lex_state = 3},
  [831] = {.lex_state = 73, .external_lex_state = 3},
  [832] = {.lex_state = 73, .external_lex_state = 3},
  [833] = {.lex_state = 73, .external_lex_state = 3},
  [834] = {.lex_state = 73, .external_lex_state = 3},
  [835] = {.lex_state = 73, .external_lex_state = 3},
  [836] = {.lex_state = 73, .external_lex_state = 3},
  [837] = {.lex_state = 73, .external_lex_state = 3},
  [838] = {.lex_state = 73, .external_lex_state = 3},
  [839] = {.lex_state = 73, .external_lex_state = 3},
  [840] = {.lex_state = 73, .external_lex_state = 3},
  [841] = {.lex_state = 73, .external_lex_state = 3},
  [842] = {.lex_state = 73, .external_lex_state = 3},
  [843] = {.lex_state = 73, .external_lex_state = 3},
  [844] = {.lex_state = 73, .external_lex_state = 3},
  [845] = {.lex_state = 73, .external_lex_state = 3},
  [846] = {.lex_state = 73, .external_lex_state = 3},
  [847] = {.lex_state = 73, .external_lex_state = 3},
  [848] = {.lex_state = 73, .external_lex_state = 3},
  [849] = {.lex_state = 73, .external_lex_state = 3},
  [850] = {.lex_state = 73, .external_lex_state = 3},
  [851] = {.lex_state = 73, .external_lex_state = 3},
  [852] = {.lex_state = 73, .external_lex_state = 3},
  [853] = {.lex_state = 73, .external_lex_state = 3},
  [854] = {.lex_state = 73, .external_lex_state = 3},
  [855] = {.lex_state = 73, .external_lex_state = 3},
  [856] = {.lex_state = 73, .external_lex_state = 3},
  [857] = {.lex_state = 73, .external_lex_state = 3},
  [858] = {.lex_state = 73, .external_lex_state = 3},
  [859] = {.lex_state = 73, .external_lex_state = 3},
  [860] = {.lex_state = 73, .external_lex_state = 3},
  [861] = {.lex_state = 73, .external_lex_state = 3},
  [862] = {.lex_state = 73, .external_lex_state = 3},
  [863] = {.lex_state = 73, .external_lex_state = 3},
  [864] = {.lex_state = 73, .external_lex_state = 3},
  [865] = {.lex_state = 73, .external_lex_state = 3},
  [866] = {.lex_state = 73, .external_lex_state = 3},
  [867] = {.lex_state = 73, .external_lex_state = 3},
  [868] = {.lex_state = 73, .external_lex_state = 3},
  [869] = {.lex_state = 73, .external_lex_state = 3},
  [870] = {.lex_state = 73, .external_lex_state = 3},
  [871] = {.lex_state = 73, .external_lex_state = 3},
  [872] = {.lex_state = 73, .external_lex_state = 3},
  [873] = {.lex_state = 73, .external_lex_state = 3},
  [874] = {.lex_state = 73, .external_lex_state = 3},
  [875] = {.lex_state = 73, .external_lex_state = 3},
  [876] = {.lex_state = 73, .external_lex_state = 3},
  [877] = {.lex_state = 73, .external_lex_state = 3},
  [878] = {.lex_state = 73, .external_lex_state = 3},
  [879] = {.lex_state = 73, .external_lex_state = 3},
  [880] = {.lex_state = 73, .external_lex_state = 3},
  [881] = {.lex_state = 73, .external_lex_state = 3},
  [882] = {.lex_state = 73, .external_lex_state = 3},
  [883] = {.lex_state = 73, .external_lex_state = 3},
  [884] = {.lex_state = 73, .external_lex_state = 3},
  [885] = {.lex_state = 73, .external_lex_state = 3},
  [886] = {.lex_state = 73, .external_lex_state = 3},
  [887] = {.lex_state = 73, .external_lex_state = 3},
  [888] = {.lex_state = 73, .external_lex_state = 3},
  [889] = {.lex_state = 73, .external_lex_state = 3},
  [890] = {.lex_state = 73, .external_lex_state = 3},
  [891] = {.lex_state = 73, .external_lex_state = 3},
  [892] = {.lex_state = 73, .external_lex_state = 3},
  [893] = {.lex_state = 73, .external_lex_state = 3},
  [894] = {.lex_state = 73, .external_lex_state = 3},
  [895] = {.lex_state = 73, .external_lex_state = 3},
  [896] = {.lex_state = 73, .external_lex_state = 3},
  [897] = {.lex_state = 73, .external_lex_state = 3},
  [898] = {.lex_state = 73, .external_lex_state = 3},
  [899] = {.lex_state = 73, .external_lex_state = 3},
  [900] = {.lex_state = 73, .external_lex_state = 3},
  [901] = {.lex_state = 73, .external_lex_state = 3},
  [902] = {.lex_state = 73, .external_lex_state = 3},
  [903] = {.lex_state = 73, .external_lex_state = 3},
  [904] = {.lex_state = 73, .external_lex_state = 3},
  [905] = {.lex_state = 73, .external_lex_state = 3},
  [906] = {.lex_state = 73, .external_lex_state = 3},
  [907] = {.lex_state = 73, .external_lex_state = 3},
  [908] = {.lex_state = 73, .external_lex_state = 3},
  [909] = {.lex_state = 73, .external_lex_state = 3},
  [910] = {.lex_state = 73, .external_lex_state = 3},
  [911] = {.lex_state = 73, .external_lex_state = 3},
  [912] = {.lex_state = 73, .external_lex_state = 3},
  [913] = {.lex_state = 73, .external_lex_state = 3},
  [914] = {.lex_state = 73, .external_lex_state = 3},
  [915] = {.lex_state = 73, .external_lex_state = 3},
  [916] = {.lex_state = 73, .external_lex_state = 3},
  [917] = {.lex_state = 73, .external_lex_state = 3},
  [918] = {.lex_state = 73, .external_lex_state = 3},
  [919] = {.lex_state = 73, .external_lex_state = 3},
  [920] = {.lex_state = 73, .external_lex_state = 3},
  [921] = {.lex_state = 73, .external_lex_state = 3},
  [922] = {.lex_state = 73, .external_lex_state = 3},
  [923] = {.lex_state = 73, .external_lex_state = 3},
  [924] = {.lex_state = 73, .external_lex_state = 3},
  [925] = {.lex_state = 73, .external_lex_state = 3},
  [926] = {.lex_state = 73, .external_lex_state = 3},
  [927] = {.lex_state = 73, .external_lex_state = 3},
  [928] = {.lex_state = 73, .external_lex_state = 3},
  [929] = {.lex_state = 73, .external_lex_state = 3},
  [930] = {.lex_state = 73, .external_lex_state = 3},
  [931] = {.lex_state = 73, .external_lex_state = 3},
  [932] = {.lex_state = 73, .external_lex_state = 3},
  [933] = {.lex_state = 73, .external_lex_state = 3},
  [934] = {.lex_state = 73, .external_lex_state = 3},
  [935] = {.lex_state = 73, .external_lex_state = 3},
  [936] = {.lex_state = 73, .external_lex_state = 3},
  [937] = {.lex_state = 73, .external_lex_state = 3},
  [938] = {.lex_state = 73, .external_lex_state = 3},
  [939] = {.lex_state = 73, .external_lex_state = 3},
  [940] = {.lex_state = 73, .external_lex_state = 3},
  [941] = {.lex_state = 73, .external_lex_state = 3},
  [942] = {.lex_state = 73, .external_lex_state = 3},
  [943] = {.lex_state = 73, .external_lex_state = 3},
  [944] = {.lex_state = 73, .external_lex_state = 3},
  [945] = {.lex_state = 73, .external_lex_state = 3},
  [946] = {.lex_state = 73, .external_lex_state = 3},
  [947] = {.lex_state = 73, .external_lex_state = 3},
  [948] = {.lex_state = 73, .external_lex_state = 3},
  [949] = {.lex_state = 73, .external_lex_state = 3},
  [950] = {.lex_state = 73, .external_lex_state = 3},
  [951] = {.lex_state = 73, .external_lex_state = 3},
  [952] = {.lex_state = 73, .external_lex_state = 3},
  [953] = {.lex_state = 73, .external_lex_state = 3},
  [954] = {.lex_state = 73, .external_lex_state = 3},
  [955] = {.lex_state = 73, .external_lex_state = 3},
  [956] = {.lex_state = 73, .external_lex_state = 3},
  [957] = {.lex_state = 73, .external_lex_state = 3},
  [958] = {.lex_state = 73, .external_lex_state = 3},
  [959] = {.lex_state = 73, .external_lex_state = 3},
  [960] = {.lex_state = 73, .external_lex_state = 3},
  [961] = {.lex_state = 73, .external_lex_state = 3},
  [962] = {.lex_state = 73, .external_lex_state = 3},
  [963] = {.lex_state = 73, .external_lex_state = 3},
  [964] = {.lex_state = 73, .external_lex_state = 3},
  [965] = {.lex_state = 73, .external_lex_state = 3},
  [966] = {.lex_state = 73, .external_lex_state = 3},
  [967] = {.lex_state = 73, .external_lex_state = 3},
  [968] = {.lex_state = 73, .external_lex_state = 3},
  [969] = {.lex_state = 73, .external_lex_state = 3},
  [970] = {.lex_state = 73, .external_lex_state = 3},
  [971] = {.lex_state = 73, .external_lex_state = 3},
  [972] = {.lex_state = 73, .external_lex_state = 3},
  [973] = {.lex_state = 73, .external_lex_state = 3},
  [974] = {.lex_state = 73, .external_lex_state = 3},
  [975] = {.lex_state = 73, .external_lex_state = 3},
  [976] = {.lex_state = 73, .external_lex_state = 3},
  [977] = {.lex_state = 73, .external_lex_state = 3},
  [978] = {.lex_state = 73, .external_lex_state = 3},
  [979] = {.lex_state = 73, .external_lex_state = 3},
  [980] = {.lex_state = 73, .external_lex_state = 3},
  [981] = {.lex_state = 73, .external_lex_state = 3},
  [982] = {.lex_state = 73, .external_lex_state = 3},
  [983] = {.lex_state = 73, .external_lex_state = 3},
  [984] = {.lex_state = 73, .external_lex_state = 3},
  [985] = {.lex_state = 73, .external_lex_state = 3},
  [986] = {.lex_state = 73, .external_lex_state = 3},
  [987] = {.lex_state = 73, .external_lex_state = 3},
  [988] = {.lex_state = 73, .external_lex_state = 3},
  [989] = {.lex_state = 73, .external_lex_state = 3},
  [990] = {.lex_state = 73, .external_lex_state = 3},
  [991] = {.lex_state = 73, .external_lex_state = 3},
  [992] = {.lex_state = 73, .external_lex_state = 3},
  [993] = {.lex_state = 73, .external_lex_state = 3},
  [994] = {.lex_state = 73, .external_lex_state = 3},
  [995] = {.lex_state = 73, .external_lex_state = 3},
  [996] = {.lex_state = 73, .external_lex_state = 3},
  [997] = {.lex_state = 73, .external_lex_state = 3},
  [998] = {.lex_state = 73, .external_lex_state = 3},
  [999] = {.lex_state = 73, .external_lex_state = 3},
  [1000] = {.lex_state = 73, .external_lex_state = 3},
  [1001] = {.lex_state = 73, .external_lex_state = 3},
  [1002] = {.lex_state = 73, .external_lex_state = 3},
  [1003] = {.lex_state = 73, .external_lex_state = 3},
  [1004] = {.lex_state = 73, .external_lex_state = 3},
  [1005] = {.lex_state = 73, .external_lex_state = 3},
  [1006] = {.lex_state = 73, .external_lex_state = 3},
  [1007] = {.lex_state = 73, .external_lex_state = 3},
  [1008] = {.lex_state = 73, .external_lex_state = 3},
  [1009] = {.lex_state = 73, .external_lex_state = 3},
  [1010] = {.lex_state = 73, .external_lex_state = 3},
  [1011] = {.lex_state = 73, .external_lex_state = 3},
  [1012] = {.lex_state = 73, .external_lex_state = 3},
  [1013] = {.lex_state = 73, .external_lex_state = 3},
  [1014] = {.lex_state = 73, .external_lex_state = 3},
  [1015] = {.lex_state = 73, .external_lex_state = 3},
  [1016] = {.lex_state = 73, .external_lex_state = 3},
  [1017] = {.lex_state = 73, .external_lex_state = 3},
  [1018] = {.lex_state = 73, .external_lex_state = 3},
  [1019] = {.lex_state = 73, .external_lex_state = 3},
  [1020] = {.lex_state = 73, .external_lex_state = 3},
  [1021] = {.lex_state = 73, .external_lex_state = 3},
  [1022] = {.lex_state = 73, .external_lex_state = 3},
  [1023] = {.lex_state = 73, .external_lex_state = 3},
  [1024] = {.lex_state = 73, .external_lex_state = 3},
  [1025] = {.lex_state = 73, .external_lex_state = 3},
  [1026] = {.lex_state = 73, .external_lex_state = 3},
  [1027] = {.lex_state = 73, .external_lex_state = 3},
  [1028] = {.lex_state = 73, .external_lex_state = 3},
  [1029] = {.lex_state = 73, .external_lex_state = 3},
  [1030] = {.lex_state = 73, .external_lex_state = 3},
  [1031] = {.lex_state = 73, .external_lex_state = 3},
  [1032] = {.lex_state = 73, .external_lex_state = 3},
  [1033] = {.lex_state = 73, .external_lex_state = 3},
  [1034] = {.lex_state = 73, .external_lex_state = 3},
  [1035] = {.lex_state = 73, .external_lex_state = 3},
  [1036] = {.lex_state = 73, .external_lex_state = 3},
  [1037] = {.lex_state = 73, .external_lex_state = 3},
  [1038] = {.lex_state = 73, .external_lex_state = 3},
  [1039] = {.lex_state = 73, .external_lex_state = 3},
  [1040] = {.lex_state = 73, .external_lex_state = 3},
  [1041] = {.lex_state = 73, .external_lex_state = 3},
  [1042] = {.lex_state = 73, .external_lex_state = 3},
  [1043] = {.lex_state = 73, .external_lex_state = 3},
  [1044] = {.lex_state = 73, .external_lex_state = 3},
  [1045] = {.lex_state = 73, .external_lex_state = 3},
  [1046] = {.lex_state = 73, .external_lex_state = 3},
  [1047] = {.lex_state = 73, .external_lex_state = 3},
  [1048] = {.lex_state = 73, .external_lex_state = 3},
  [1049] = {.lex_state = 73, .external_lex_state = 3},
  [1050] = {.lex_state = 73, .external_lex_state = 3},
  [1051] = {.lex_state = 73, .external_lex_state = 3},
  [1052] = {.lex_state = 73, .external_lex_state = 3},
  [1053] = {.lex_state = 73, .external_lex_state = 3},
  [1054] = {.lex_state = 73, .external_lex_state = 3},
  [1055] = {.lex_state = 73, .external_lex_state = 3},
  [1056] = {.lex_state = 73, .external_lex_state = 3},
  [1057] = {.lex_state = 73, .external_lex_state = 3},
  [1058] = {.lex_state = 73, .external_lex_state = 3},
  [1059] = {.lex_state = 73, .external_lex_state = 3},
  [1060] = {.lex_state = 73, .external_lex_state = 3},
  [1061] = {.lex_state = 73, .external_lex_state = 3},
  [1062] = {.lex_state = 73, .external_lex_state = 3},
  [1063] = {.lex_state = 73, .external_lex_state = 3},
  [1064] = {.lex_state = 73, .external_lex_state = 3},
  [1065] = {.lex_state = 73, .external_lex_state = 3},
  [1066] = {.lex_state = 73, .external_lex_state = 3},
  [1067] = {.lex_state = 73, .external_lex_state = 3},
  [1068] = {.lex_state = 73, .external_lex_state = 3},
  [1069] = {.lex_state = 73, .external_lex_state = 3},
  [1070] = {.lex_state = 73, .external_lex_state = 3},
  [1071] = {.lex_state = 73, .external_lex_state = 3},
  [1072] = {.lex_state = 73, .external_lex_state = 3},
  [1073] = {.lex_state = 73, .external_lex_state = 3},
  [1074] = {.lex_state = 73, .external_lex_state = 3},
  [1075] = {.lex_state = 12, .external_lex_state = 4},
  [1076] = {.lex_state = 73, .external_lex_state = 3},
  [1077] = {.lex_state = 73, .external_lex_state = 3},
  [1078] = {.lex_state = 73, .external_lex_state = 3},
  [1079] = {.lex_state = 73, .external_lex_state = 3},
  [1080] = {.lex_state = 73, .external_lex_state = 3},
  [1081] = {.lex_state = 73, .external_lex_state = 3},
  [1082] = {.lex_state = 73, .external_lex_state = 3},
  [1083] = {.lex_state = 73, .external_lex_state = 3},
  [1084] = {.lex_state = 73, .external_lex_state = 3},
  [1085] = {.lex_state = 73, .external_lex_state = 3},
  [1086] = {.lex_state = 73, .external_lex_state = 3},
  [1087] = {.lex_state = 73, .external_lex_state = 3},
  [1088] = {.lex_state = 73, .external_lex_state = 3},
  [1089] = {.lex_state = 73, .external_lex_state = 3},
  [1090] = {.lex_state = 73, .external_lex_state = 3},
  [1091] = {.lex_state = 73, .external_lex_state = 3},
  [1092] = {.lex_state = 73, .external_lex_state = 3},
  [1093] = {.lex_state = 73, .external_lex_state = 3},
  [1094] = {.lex_state = 73, .external_lex_state = 3},
  [1095] = {.lex_state = 73, .external_lex_state = 3},
  [1096] = {.lex_state = 73, .external_lex_state = 3},
  [1097] = {.lex_state = 73, .external_lex_state = 3},
  [1098] = {.lex_state = 73, .external_lex_state = 3},
  [1099] = {.lex_state = 73, .external_lex_state = 3},
  [1100] = {.lex_state = 73, .external_lex_state = 3},
  [1101] = {.lex_state = 73, .external_lex_state = 3},
  [1102] = {.lex_state = 73, .external_lex_state = 3},
  [1103] = {.lex_state = 11},
  [1104] = {.lex_state = 11},
  [1105] = {.lex_state = 11},
  [1106] = {.lex_state = 73, .external_lex_state = 3},
  [1107] = {.lex_state = 73, .external_lex_state = 3},
  [1108] = {.lex_state = 73, .external_lex_state = 3},
  [1109] = {.lex_state = 73, .external_lex_state = 3},
  [1110] = {.lex_state = 73, .external_lex_state = 3},
  [1111] = {.lex_state = 73, .external_lex_state = 3},
  [1112] = {.lex_state = 73, .external_lex_state = 3},
  [1113] = {.lex_state = 73, .external_lex_state = 3},
  [1114] = {.lex_state = 73, .external_lex_state = 3},
  [1115] = {.lex_state = 73, .external_lex_state = 3},
  [1116] = {.lex_state = 73, .external_lex_state = 3},
  [1117] = {.lex_state = 73, .external_lex_state = 3},
  [1118] = {.lex_state = 73, .external_lex_state = 3},
  [1119] = {.lex_state = 73, .external_lex_state = 3},
  [1120] = {.lex_state = 73, .external_lex_state = 3},
  [1121] = {.lex_state = 73, .external_lex_state = 3},
  [1122] = {.lex_state = 73, .external_lex_state = 3},
  [1123] = {.lex_state = 73, .external_lex_state = 3},
  [1124] = {.lex_state = 73, .external_lex_state = 3},
  [1125] = {.lex_state = 73, .external_lex_state = 3},
  [1126] = {.lex_state = 73, .external_lex_state = 3},
  [1127] = {.lex_state = 73, .external_lex_state = 3},
  [1128] = {.lex_state = 73, .external_lex_state = 3},
  [1129] = {.lex_state = 73, .external_lex_state = 3},
  [1130] = {.lex_state = 73, .external_lex_state = 3},
  [1131] = {.lex_state = 73, .external_lex_state = 3},
  [1132] = {.lex_state = 73, .external_lex_state = 3},
  [1133] = {.lex_state = 73, .external_lex_state = 3},
  [1134] = {.lex_state = 73, .external_lex_state = 3},
  [1135] = {.lex_state = 73, .external_lex_state = 3},
  [1136] = {.lex_state = 73, .external_lex_state = 3},
  [1137] = {.lex_state = 73, .external_lex_state = 3},
  [1138] = {.lex_state = 73, .external_lex_state = 3},
  [1139] = {.lex_state = 73, .external_lex_state = 3},
  [1140] = {.lex_state = 73, .external_lex_state = 3},
  [1141] = {.lex_state = 73, .external_lex_state = 3},
  [1142] = {.lex_state = 73, .external_lex_state = 3},
  [1143] = {.lex_state = 73, .external_lex_state = 3},
  [1144] = {.lex_state = 73, .external_lex_state = 3},
  [1145] = {.lex_state = 73, .external_lex_state = 3},
  [1146] = {.lex_state = 73, .external_lex_state = 3},
  [1147] = {.lex_state = 73, .external_lex_state = 3},
  [1148] = {.lex_state = 73, .external_lex_state = 3},
  [1149] = {.lex_state = 73, .external_lex_state = 3},
  [1150] = {.lex_state = 73, .external_lex_state = 3},
  [1151] = {.lex_state = 73, .external_lex_state = 3},
  [1152] = {.lex_state = 73, .external_lex_state = 3},
  [1153] = {.lex_state = 73, .external_lex_state = 3},
  [1154] = {.lex_state = 73, .external_lex_state = 3},
  [1155] = {.lex_state = 73, .external_lex_state = 3},
  [1156] = {.lex_state = 73, .external_lex_state = 3},
  [1157] = {.lex_state = 73, .external_lex_state = 3},
  [1158] = {.lex_state = 73, .external_lex_state = 3},
  [1159] = {.lex_state = 73, .external_lex_state = 3},
  [1160] = {.lex_state = 73, .external_lex_state = 3},
  [1161] = {.lex_state = 73, .external_lex_state = 3},
  [1162] = {.lex_state = 73, .external_lex_state = 3},
  [1163] = {.lex_state = 73, .external_lex_state = 3},
  [1164] = {.lex_state = 73, .external_lex_state = 3},
  [1165] = {.lex_state = 73, .external_lex_state = 3},
  [1166] = {.lex_state = 73, .external_lex_state = 3},
  [1167] = {.lex_state = 73, .external_lex_state = 3},
  [1168] = {.lex_state = 73, .external_lex_state = 3},
  [1169] = {.lex_state = 73, .external_lex_state = 3},
  [1170] = {.lex_state = 73, .external_lex_state = 3},
  [1171] = {.lex_state = 73, .external_lex_state = 3},
  [1172] = {.lex_state = 73, .external_lex_state = 3},
  [1173] = {.lex_state = 73, .external_lex_state = 3},
  [1174] = {.lex_state = 73, .external_lex_state = 3},
  [1175] = {.lex_state = 73, .external_lex_state = 3},
  [1176] = {.lex_state = 73, .external_lex_state = 3},
  [1177] = {.lex_state = 73, .external_lex_state = 3},
  [1178] = {.lex_state = 73, .external_lex_state = 3},
  [1179] = {.lex_state = 73, .external_lex_state = 3},
  [1180] = {.lex_state = 73, .external_lex_state = 3},
  [1181] = {.lex_state = 73, .external_lex_state = 3},
  [1182] = {.lex_state = 73, .external_lex_state = 3},
  [1183] = {.lex_state = 73, .external_lex_state = 3},
  [1184] = {.lex_state = 73, .external_lex_state = 3},
  [1185] = {.lex_state = 73, .external_lex_state = 3},
  [1186] = {.lex_state = 73, .external_lex_state = 3},
  [1187] = {.lex_state = 73, .external_lex_state = 3},
  [1188] = {.lex_state = 73, .external_lex_state = 3},
  [1189] = {.lex_state = 73, .external_lex_state = 3},
  [1190] = {.lex_state = 73, .external_lex_state = 3},
  [1191] = {.lex_state = 73, .external_lex_state = 3},
  [1192] = {.lex_state = 73, .external_lex_state = 3},
  [1193] = {.lex_state = 73, .external_lex_state = 3},
  [1194] = {.lex_state = 73, .external_lex_state = 3},
  [1195] = {.lex_state = 73, .external_lex_state = 3},
  [1196] = {.lex_state = 73, .external_lex_state = 3},
  [1197] = {.lex_state = 73, .external_lex_state = 3},
  [1198] = {.lex_state = 73, .external_lex_state = 3},
  [1199] = {.lex_state = 73, .external_lex_state = 3},
  [1200] = {.lex_state = 73, .external_lex_state = 3},
  [1201] = {.lex_state = 73, .external_lex_state = 3},
  [1202] = {.lex_state = 73, .external_lex_state = 3},
  [1203] = {.lex_state = 73, .external_lex_state = 3},
  [1204] = {.lex_state = 73, .external_lex_state = 3},
  [1205] = {.lex_state = 73, .external_lex_state = 3},
  [1206] = {.lex_state = 73, .external_lex_state = 3},
  [1207] = {.lex_state = 73, .external_lex_state = 3},
  [1208] = {.lex_state = 73, .external_lex_state = 3},
  [1209] = {.lex_state = 73, .external_lex_state = 3},
  [1210] = {.lex_state = 73, .external_lex_state = 3},
  [1211] = {.lex_state = 73, .external_lex_state = 3},
  [1212] = {.lex_state = 73, .external_lex_state = 3},
  [1213] = {.lex_state = 73, .external_lex_state = 3},
  [1214] = {.lex_state = 73, .external_lex_state = 3},
  [1215] = {.lex_state = 73, .external_lex_state = 3},
  [1216] = {.lex_state = 73, .external_lex_state = 3},
  [1217] = {.lex_state = 73, .external_lex_state = 3},
  [1218] = {.lex_state = 73, .external_lex_state = 3},
  [1219] = {.lex_state = 73, .external_lex_state = 3},
  [1220] = {.lex_state = 73, .external_lex_state = 3},
  [1221] = {.lex_state = 73, .external_lex_state = 3},
  [1222] = {.lex_state = 73, .external_lex_state = 3},
  [1223] = {.lex_state = 73, .external_lex_state = 3},
  [1224] = {.lex_state = 73, .external_lex_state = 3},
  [1225] = {.lex_state = 73, .external_lex_state = 3},
  [1226] = {.lex_state = 73, .external_lex_state = 3},
  [1227] = {.lex_state = 73, .external_lex_state = 3},
  [1228] = {.lex_state = 73, .external_lex_state = 3},
  [1229] = {.lex_state = 73, .external_lex_state = 3},
  [1230] = {.lex_state = 73, .external_lex_state = 3},
  [1231] = {.lex_state = 73, .external_lex_state = 3},
  [1232] = {.lex_state = 73, .external_lex_state = 3},
  [1233] = {.lex_state = 73, .external_lex_state = 3},
  [1234] = {.lex_state = 73, .external_lex_state = 3},
  [1235] = {.lex_state = 73, .external_lex_state = 3},
  [1236] = {.lex_state = 73, .external_lex_state = 3},
  [1237] = {.lex_state = 73, .external_lex_state = 3},
  [1238] = {.lex_state = 73, .external_lex_state = 3},
  [1239] = {.lex_state = 73, .external_lex_state = 3},
  [1240] = {.lex_state = 73, .external_lex_state = 3},
  [1241] = {.lex_state = 73, .external_lex_state = 3},
  [1242] = {.lex_state = 73, .external_lex_state = 3},
  [1243] = {.lex_state = 73, .external_lex_state = 3},
  [1244] = {.lex_state = 73, .external_lex_state = 3},
  [1245] = {.lex_state = 73, .external_lex_state = 3},
  [1246] = {.lex_state = 73, .external_lex_state = 3},
  [1247] = {.lex_state = 73, .external_lex_state = 3},
  [1248] = {.lex_state = 73, .external_lex_state = 3},
  [1249] = {.lex_state = 73, .external_lex_state = 3},
  [1250] = {.lex_state = 73, .external_lex_state = 3},
  [1251] = {.lex_state = 11},
  [1252] = {.lex_state = 73, .external_lex_state = 3},
  [1253] = {.lex_state = 73, .external_lex_state = 3},
  [1254] = {.lex_state = 73, .external_lex_state = 3},
  [1255] = {.lex_state = 73, .external_lex_state = 3},
  [1256] = {.lex_state = 73, .external_lex_state = 3},
  [1257] = {.lex_state = 73, .external_lex_state = 3},
  [1258] = {.lex_state = 73, .external_lex_state = 3},
  [1259] = {.lex_state = 73, .external_lex_state = 3},
  [1260] = {.lex_state = 73, .external_lex_state = 3},
  [1261] = {.lex_state = 73, .external_lex_state = 3},
  [1262] = {.lex_state = 73, .external_lex_state = 3},
  [1263] = {.lex_state = 73, .external_lex_state = 3},
  [1264] = {.lex_state = 73, .external_lex_state = 3},
  [1265] = {.lex_state = 73, .external_lex_state = 3},
  [1266] = {.lex_state = 73, .external_lex_state = 3},
  [1267] = {.lex_state = 73, .external_lex_state = 3},
  [1268] = {.lex_state = 73, .external_lex_state = 3},
  [1269] = {.lex_state = 73, .external_lex_state = 3},
  [1270] = {.lex_state = 73, .external_lex_state = 3},
  [1271] = {.lex_state = 73, .external_lex_state = 3},
  [1272] = {.lex_state = 73, .external_lex_state = 3},
  [1273] = {.lex_state = 73, .external_lex_state = 3},
  [1274] = {.lex_state = 73, .external_lex_state = 3},
  [1275] = {.lex_state = 73, .external_lex_state = 3},
  [1276] = {.lex_state = 73, .external_lex_state = 3},
  [1277] = {.lex_state = 73, .external_lex_state = 3},
  [1278] = {.lex_state = 73, .external_lex_state = 3},
  [1279] = {.lex_state = 73, .external_lex_state = 3},
  [1280] = {.lex_state = 73, .external_lex_state = 3},
  [1281] = {.lex_state = 73, .external_lex_state = 3},
  [1282] = {.lex_state = 73, .external_lex_state = 3},
  [1283] = {.lex_state = 73, .external_lex_state = 3},
  [1284] = {.lex_state = 73, .external_lex_state = 3},
  [1285] = {.lex_state = 73, .external_lex_state = 3},
  [1286] = {.lex_state = 73, .external_lex_state = 3},
  [1287] = {.lex_state = 73, .external_lex_state = 3},
  [1288] = {.lex_state = 73, .external_lex_state = 3},
  [1289] = {.lex_state = 73, .external_lex_state = 3},
  [1290] = {.lex_state = 73, .external_lex_state = 3},
  [1291] = {.lex_state = 73, .external_lex_state = 3},
  [1292] = {.lex_state = 73, .external_lex_state = 3},
  [1293] = {.lex_state = 73, .external_lex_state = 3},
  [1294] = {.lex_state = 73, .external_lex_state = 3},
  [1295] = {.lex_state = 73, .external_lex_state = 3},
  [1296] = {.lex_state = 73, .external_lex_state = 3},
  [1297] = {.lex_state = 73, .external_lex_state = 3},
  [1298] = {.lex_state = 73, .external_lex_state = 3},
  [1299] = {.lex_state = 73, .external_lex_state = 3},
  [1300] = {.lex_state = 73, .external_lex_state = 3},
  [1301] = {.lex_state = 73, .external_lex_state = 3},
  [1302] = {.lex_state = 73, .external_lex_state = 3},
  [1303] = {.lex_state = 73, .external_lex_state = 3},
  [1304] = {.lex_state = 73, .external_lex_state = 3},
  [1305] = {.lex_state = 73, .external_lex_state = 3},
  [1306] = {.lex_state = 73, .external_lex_state = 3},
  [1307] = {.lex_state = 73, .external_lex_state = 3},
  [1308] = {.lex_state = 73, .external_lex_state = 3},
  [1309] = {.lex_state = 73, .external_lex_state = 3},
  [1310] = {.lex_state = 73, .external_lex_state = 3},
  [1311] = {.lex_state = 73, .external_lex_state = 3},
  [1312] = {.lex_state = 73, .external_lex_state = 3},
  [1313] = {.lex_state = 73, .external_lex_state = 3},
  [1314] = {.lex_state = 73, .external_lex_state = 3},
  [1315] = {.lex_state = 73, .external_lex_state = 3},
  [1316] = {.lex_state = 73, .external_lex_state = 3},
  [1317] = {.lex_state = 73, .external_lex_state = 3},
  [1318] = {.lex_state = 73, .external_lex_state = 3},
  [1319] = {.lex_state = 73, .external_lex_state = 3},
  [1320] = {.lex_state = 73, .external_lex_state = 3},
  [1321] = {.lex_state = 73, .external_lex_state = 3},
  [1322] = {.lex_state = 73, .external_lex_state = 3},
  [1323] = {.lex_state = 73, .external_lex_state = 3},
  [1324] = {.lex_state = 73, .external_lex_state = 3},
  [1325] = {.lex_state = 73, .external_lex_state = 3},
  [1326] = {.lex_state = 73, .external_lex_state = 3},
  [1327] = {.lex_state = 73, .external_lex_state = 3},
  [1328] = {.lex_state = 73, .external_lex_state = 3},
  [1329] = {.lex_state = 73, .external_lex_state = 3},
  [1330] = {.lex_state = 73, .external_lex_state = 3},
  [1331] = {.lex_state = 73, .external_lex_state = 3},
  [1332] = {.lex_state = 73, .external_lex_state = 3},
  [1333] = {.lex_state = 73, .external_lex_state = 3},
  [1334] = {.lex_state = 73, .external_lex_state = 3},
  [1335] = {.lex_state = 73, .external_lex_state = 3},
  [1336] = {.lex_state = 73, .external_lex_state = 3},
  [1337] = {.lex_state = 73, .external_lex_state = 3},
  [1338] = {.lex_state = 73, .external_lex_state = 3},
  [1339] = {.lex_state = 73, .external_lex_state = 3},
  [1340] = {.lex_state = 73, .external_lex_state = 3},
  [1341] = {.lex_state = 73, .external_lex_state = 3},
  [1342] = {.lex_state = 73, .external_lex_state = 3},
  [1343] = {.lex_state = 11},
  [1344] = {.lex_state = 73, .external_lex_state = 3},
  [1345] = {.lex_state = 73, .external_lex_state = 3},
  [1346] = {.lex_state = 73, .external_lex_state = 3},
  [1347] = {.lex_state = 73, .external_lex_state = 3},
  [1348] = {.lex_state = 73, .external_lex_state = 3},
  [1349] = {.lex_state = 73, .external_lex_state = 3},
  [1350] = {.lex_state = 73, .external_lex_state = 3},
  [1351] = {.lex_state = 73, .external_lex_state = 3},
  [1352] = {.lex_state = 73, .external_lex_state = 3},
  [1353] = {.lex_state = 73, .external_lex_state = 3},
  [1354] = {.lex_state = 73, .external_lex_state = 3},
  [1355] = {.lex_state = 73, .external_lex_state = 3},
  [1356] = {.lex_state = 73, .external_lex_state = 3},
  [1357] = {.lex_state = 73, .external_lex_state = 3},
  [1358] = {.lex_state = 73, .external_lex_state = 3},
  [1359] = {.lex_state = 73, .external_lex_state = 3},
  [1360] = {.lex_state = 73, .external_lex_state = 3},
  [1361] = {.lex_state = 13, .external_lex_state = 4},
  [1362] = {.lex_state = 13, .external_lex_state = 4},
  [1363] = {.lex_state = 13, .external_lex_state = 4},
  [1364] = {.lex_state = 13, .external_lex_state = 4},
  [1365] = {.lex_state = 13, .external_lex_state = 4},
  [1366] = {.lex_state = 73, .external_lex_state = 3},
  [1367] = {.lex_state = 13, .external_lex_state = 4},
  [1368] = {.lex_state = 13, .external_lex_state = 4},
  [1369] = {.lex_state = 13, .external_lex_state = 4},
  [1370] = {.lex_state = 13, .external_lex_state = 4},
  [1371] = {.lex_state = 73, .external_lex_state = 3},
  [1372] = {.lex_state = 73, .external_lex_state = 3},
  [1373] = {.lex_state = 12, .external_lex_state = 4},
  [1374] = {.lex_state = 73, .external_lex_state = 3},
  [1375] = {.lex_state = 73, .external_lex_state = 3},
  [1376] = {.lex_state = 73, .external_lex_state = 3},
  [1377] = {.lex_state = 13, .external_lex_state = 4},
  [1378] = {.lex_state = 13, .external_lex_state = 4},
  [1379] = {.lex_state = 13, .external_lex_state = 4},
  [1380] = {.lex_state = 13, .external_lex_state = 4},
  [1381] = {.lex_state = 13, .external_lex_state = 4},
  [1382] = {.lex_state = 13, .external_lex_state = 4},
  [1383] = {.lex_state = 13, .external_lex_state = 4},
  [1384] = {.lex_state = 13, .external_lex_state = 4},
  [1385] = {.lex_state = 13, .external_lex_state = 4},
  [1386] = {.lex_state = 13, .external_lex_state = 4},
  [1387] = {.lex_state = 13, .external_lex_state = 4},
  [1388] = {.lex_state = 12, .external_lex_state = 4},
  [1389] = {.lex_state = 13, .external_lex_state = 4},
  [1390] = {.lex_state = 13, .external_lex_state = 4},
  [1391] = {.lex_state = 13, .external_lex_state = 4},
  [1392] = {.lex_state = 13, .external_lex_state = 4},
  [1393] = {.lex_state = 13, .external_lex_state = 4},
  [1394] = {.lex_state = 13, .external_lex_state = 4},
  [1395] = {.lex_state = 13, .external_lex_state = 4},
  [1396] = {.lex_state = 13, .external_lex_state = 4},
  [1397] = {.lex_state = 13, .external_lex_state = 4},
  [1398] = {.lex_state = 13, .external_lex_state = 4},
  [1399] = {.lex_state = 13, .external_lex_state = 4},
  [1400] = {.lex_state = 13, .external_lex_state = 4},
  [1401] = {.lex_state = 13, .external_lex_state = 4},
  [1402] = {.lex_state = 13, .external_lex_state = 4},
  [1403] = {.lex_state = 13, .external_lex_state = 4},
  [1404] = {.lex_state = 13, .external_lex_state = 4},
  [1405] = {.lex_state = 11},
  [1406] = {.lex_state = 13, .external_lex_state = 4},
  [1407] = {.lex_state = 13, .external_lex_state = 4},
  [1408] = {.lex_state = 13, .external_lex_state = 4},
  [1409] = {.lex_state = 13, .external_lex_state = 4},
  [1410] = {.lex_state = 13, .external_lex_state = 4},
  [1411] = {.lex_state = 13, .external_lex_state = 4},
  [1412] = {.lex_state = 13, .external_lex_state = 4},
  [1413] = {.lex_state = 13, .external_lex_state = 4},
  [1414] = {.lex_state = 13, .external_lex_state = 4},
  [1415] = {.lex_state = 13, .external_lex_state = 4},
  [1416] = {.lex_state = 13, .external_lex_state = 4},
  [1417] = {.lex_state = 13, .external_lex_state = 4},
  [1418] = {.lex_state = 13, .external_lex_state = 4},
  [1419] = {.lex_state = 13, .external_lex_state = 4},
  [1420] = {.lex_state = 13, .external_lex_state = 4},
  [1421] = {.lex_state = 13, .external_lex_state = 4},
  [1422] = {.lex_state = 13, .external_lex_state = 4},
  [1423] = {.lex_state = 13, .external_lex_state = 4},
  [1424] = {.lex_state = 13, .external_lex_state = 4},
  [1425] = {.lex_state = 13, .external_lex_state = 4},
  [1426] = {.lex_state = 13, .external_lex_state = 4},
  [1427] = {.lex_state = 13, .external_lex_state = 4},
  [1428] = {.lex_state = 13, .external_lex_state = 4},
  [1429] = {.lex_state = 13, .external_lex_state = 4},
  [1430] = {.lex_state = 13, .external_lex_state = 4},
  [1431] = {.lex_state = 13, .external_lex_state = 4},
  [1432] = {.lex_state = 13, .external_lex_state = 4},
  [1433] = {.lex_state = 13, .external_lex_state = 4},
  [1434] = {.lex_state = 13, .external_lex_state = 4},
  [1435] = {.lex_state = 13, .external_lex_state = 4},
  [1436] = {.lex_state = 13, .external_lex_state = 4},
  [1437] = {.lex_state = 11},
  [1438] = {.lex_state = 11},
  [1439] = {.lex_state = 11},
  [1440] = {.lex_state = 11},
  [1441] = {.lex_state = 11},
  [1442] = {.lex_state = 11},
  [1443] = {.lex_state = 11},
  [1444] = {.lex_state = 11},
  [1445] = {.lex_state = 11},
  [1446] = {.lex_state = 11},
  [1447] = {.lex_state = 11},
  [1448] = {.lex_state = 11},
  [1449] = {.lex_state = 11},
  [1450] = {.lex_state = 11},
  [1451] = {.lex_state = 11},
  [1452] = {.lex_state = 11},
  [1453] = {.lex_state = 11},
  [1454] = {.lex_state = 11},
  [1455] = {.lex_state = 11},
  [1456] = {.lex_state = 11},
  [1457] = {.lex_state = 11},
  [1458] = {.lex_state = 11},
  [1459] = {.lex_state = 11},
  [1460] = {.lex_state = 11},
  [1461] = {.lex_state = 11},
  [1462] = {.lex_state = 11},
  [1463] = {.lex_state = 11},
  [1464] = {.lex_state = 11},
  [1465] = {.lex_state = 11},
  [1466] = {.lex_state = 11},
  [1467] = {.lex_state = 11},
  [1468] = {.lex_state = 11},
  [1469] = {.lex_state = 11},
  [1470] = {.lex_state = 11},
  [1471] = {.lex_state = 11},
  [1472] = {.lex_state = 11},
  [1473] = {.lex_state = 11},
  [1474] = {.lex_state = 11},
  [1475] = {.lex_state = 11},
  [1476] = {.lex_state = 11},
  [1477] = {.lex_state = 11},
  [1478] = {.lex_state = 11},
  [1479] = {.lex_state = 11},
  [1480] = {.lex_state = 11},
  [1481] = {.lex_state = 11},
  [1482] = {.lex_state = 11},
  [1483] = {.lex_state = 11},
  [1484] = {.lex_state = 11},
  [1485] = {.lex_state = 11},
  [1486] = {.lex_state = 11},
  [1487] = {.lex_state = 11},
  [1488] = {.lex_state = 11},
  [1489] = {.lex_state = 11},
  [1490] = {.lex_state = 11},
  [1491] = {.lex_state = 11},
  [1492] = {.lex_state = 11},
  [1493] = {.lex_state = 11},
  [1494] = {.lex_state = 11},
  [1495] = {.lex_state = 11},
  [1496] = {.lex_state = 11},
  [1497] = {.lex_state = 11},
  [1498] = {.lex_state = 11},
  [1499] = {.lex_state = 11},
  [1500] = {.lex_state = 11},
  [1501] = {.lex_state = 11},
  [1502] = {.lex_state = 11},
  [1503] = {.lex_state = 11},
  [1504] = {.lex_state = 11},
  [1505] = {.lex_state = 11},
  [1506] = {.lex_state = 11},
  [1507] = {.lex_state = 11},
  [1508] = {.lex_state = 11},
  [1509] = {.lex_state = 11},
  [1510] = {.lex_state = 11},
  [1511] = {.lex_state = 11},
  [1512] = {.lex_state = 11},
  [1513] = {.lex_state = 11},
  [1514] = {.lex_state = 11},
  [1515] = {.lex_state = 11},
  [1516] = {.lex_state = 11},
  [1517] = {.lex_state = 11},
  [1518] = {.lex_state = 11},
  [1519] = {.lex_state = 11},
  [1520] = {.lex_state = 11},
  [1521] = {.lex_state = 11},
  [1522] = {.lex_state = 11},
  [1523] = {.lex_state = 11},
  [1524] = {.lex_state = 11},
  [1525] = {.lex_state = 11},
  [1526] = {.lex_state = 11},
  [1527] = {.lex_state = 11},
  [1528] = {.lex_state = 11},
  [1529] = {.lex_state = 11},
  [1530] = {.lex_state = 11},
  [1531] = {.lex_state = 11},
  [1532] = {.lex_state = 11},
  [1533] = {.lex_state = 11},
  [1534] = {.lex_state = 11},
  [1535] = {.lex_state = 11},
  [1536] = {.lex_state = 11},
  [1537] = {.lex_state = 11},
  [1538] = {.lex_state = 11},
  [1539] = {.lex_state = 11},
  [1540] = {.lex_state = 11},
  [1541] = {.lex_state = 11},
  [1542] = {.lex_state = 11},
  [1543] = {.lex_state = 11},
  [1544] = {.lex_state = 11},
  [1545] = {.lex_state = 11},
  [1546] = {.lex_state = 11},
  [1547] = {.lex_state = 11},
  [1548] = {.lex_state = 11},
  [1549] = {.lex_state = 11},
  [1550] = {.lex_state = 11},
  [1551] = {.lex_state = 11},
  [1552] = {.lex_state = 11},
  [1553] = {.lex_state = 11},
  [1554] = {.lex_state = 11},
  [1555] = {.lex_state = 11},
  [1556] = {.lex_state = 11},
  [1557] = {.lex_state = 11},
  [1558] = {.lex_state = 11},
  [1559] = {.lex_state = 11},
  [1560] = {.lex_state = 11},
  [1561] = {.lex_state = 11},
  [1562] = {.lex_state = 11},
  [1563] = {.lex_state = 11},
  [1564] = {.lex_state = 11},
  [1565] = {.lex_state = 11},
  [1566] = {.lex_state = 11},
  [1567] = {.lex_state = 11},
  [1568] = {.lex_state = 11},
  [1569] = {.lex_state = 11},
  [1570] = {.lex_state = 11},
  [1571] = {.lex_state = 11},
  [1572] = {.lex_state = 11},
  [1573] = {.lex_state = 11},
  [1574] = {.lex_state = 11},
  [1575] = {.lex_state = 11},
  [1576] = {.lex_state = 11},
  [1577] = {.lex_state = 11},
  [1578] = {.lex_state = 11},
  [1579] = {.lex_state = 11},
  [1580] = {.lex_state = 11},
  [1581] = {.lex_state = 11},
  [1582] = {.lex_state = 11},
  [1583] = {.lex_state = 11},
  [1584] = {.lex_state = 11},
  [1585] = {.lex_state = 11},
  [1586] = {.lex_state = 11},
  [1587] = {.lex_state = 11},
  [1588] = {.lex_state = 11},
  [1589] = {.lex_state = 11},
  [1590] = {.lex_state = 11},
  [1591] = {.lex_state = 11},
  [1592] = {.lex_state = 11},
  [1593] = {.lex_state = 11},
  [1594] = {.lex_state = 11},
  [1595] = {.lex_state = 11},
  [1596] = {.lex_state = 11},
  [1597] = {.lex_state = 11},
  [1598] = {.lex_state = 11},
  [1599] = {.lex_state = 11},
  [1600] = {.lex_state = 11},
  [1601] = {.lex_state = 11},
  [1602] = {.lex_state = 11},
  [1603] = {.lex_state = 11},
  [1604] = {.lex_state = 11},
  [1605] = {.lex_state = 11},
  [1606] = {.lex_state = 11},
  [1607] = {.lex_state = 11},
  [1608] = {.lex_state = 11},
  [1609] = {.lex_state = 11},
  [1610] = {.lex_state = 11},
  [1611] = {.lex_state = 11},
  [1612] = {.lex_state = 11},
  [1613] = {.lex_state = 11},
  [1614] = {.lex_state = 11},
  [1615] = {.lex_state = 11},
  [1616] = {.lex_state = 11},
  [1617] = {.lex_state = 11},
  [1618] = {.lex_state = 11},
  [1619] = {.lex_state = 11},
  [1620] = {.lex_state = 11},
  [1621] = {.lex_state = 11},
  [1622] = {.lex_state = 11},
  [1623] = {.lex_state = 11},
  [1624] = {.lex_state = 11},
  [1625] = {.lex_state = 11},
  [1626] = {.lex_state = 11},
  [1627] = {.lex_state = 11},
  [1628] = {.lex_state = 11},
  [1629] = {.lex_state = 11},
  [1630] = {.lex_state = 11},
  [1631] = {.lex_state = 11},
  [1632] = {.lex_state = 11},
  [1633] = {.lex_state = 11},
  [1634] = {.lex_state = 11},
  [1635] = {.lex_state = 11},
  [1636] = {.lex_state = 11},
  [1637] = {.lex_state = 11},
  [1638] = {.lex_state = 11},
  [1639] = {.lex_state = 11},
  [1640] = {.lex_state = 11},
  [1641] = {.lex_state = 11},
  [1642] = {.lex_state = 11},
  [1643] = {.lex_state = 11},
  [1644] = {.lex_state = 11},
  [1645] = {.lex_state = 11},
  [1646] = {.lex_state = 11},
  [1647] = {.lex_state = 11},
  [1648] = {.lex_state = 11},
  [1649] = {.lex_state = 11},
  [1650] = {.lex_state = 11},
  [1651] = {.lex_state = 11},
  [1652] = {.lex_state = 11},
  [1653] = {.lex_state = 11},
  [1654] = {.lex_state = 11},
  [1655] = {.lex_state = 11},
  [1656] = {.lex_state = 11},
  [1657] = {.lex_state = 11},
  [1658] = {.lex_state = 11},
  [1659] = {.lex_state = 11},
  [1660] = {.lex_state = 11},
  [1661] = {.lex_state = 11},
  [1662] = {.lex_state = 11},
  [1663] = {.lex_state = 11},
  [1664] = {.lex_state = 11},
  [1665] = {.lex_state = 11},
  [1666] = {.lex_state = 11},
  [1667] = {.lex_state = 11},
  [1668] = {.lex_state = 11},
  [1669] = {.lex_state = 11},
  [1670] = {.lex_state = 11},
  [1671] = {.lex_state = 11},
  [1672] = {.lex_state = 11},
  [1673] = {.lex_state = 11},
  [1674] = {.lex_state = 11},
  [1675] = {.lex_state = 11},
  [1676] = {.lex_state = 11},
  [1677] = {.lex_state = 11},
  [1678] = {.lex_state = 11},
  [1679] = {.lex_state = 11},
  [1680] = {.lex_state = 11},
  [1681] = {.lex_state = 11},
  [1682] = {.lex_state = 11},
  [1683] = {.lex_state = 11},
  [1684] = {.lex_state = 11},
  [1685] = {.lex_state = 11},
  [1686] = {.lex_state = 11},
  [1687] = {.lex_state = 11},
  [1688] = {.lex_state = 11},
  [1689] = {.lex_state = 11},
  [1690] = {.lex_state = 11},
  [1691] = {.lex_state = 11},
  [1692] = {.lex_state = 11},
  [1693] = {.lex_state = 11},
  [1694] = {.lex_state = 11},
  [1695] = {.lex_state = 11},
  [1696] = {.lex_state = 11},
  [1697] = {.lex_state = 11},
  [1698] = {.lex_state = 11},
  [1699] = {.lex_state = 11},
  [1700] = {.lex_state = 11},
  [1701] = {.lex_state = 11},
  [1702] = {.lex_state = 11},
  [1703] = {.lex_state = 11},
  [1704] = {.lex_state = 11},
  [1705] = {.lex_state = 11},
  [1706] = {.lex_state = 11},
  [1707] = {.lex_state = 11},
  [1708] = {.lex_state = 11},
  [1709] = {.lex_state = 11},
  [1710] = {.lex_state = 11},
  [1711] = {.lex_state = 11},
  [1712] = {.lex_state = 11},
  [1713] = {.lex_state = 11},
  [1714] = {.lex_state = 11},
  [1715] = {.lex_state = 11},
  [1716] = {.lex_state = 11},
  [1717] = {.lex_state = 11},
  [1718] = {.lex_state = 11},
  [1719] = {.lex_state = 11},
  [1720] = {.lex_state = 11},
  [1721] = {.lex_state = 11},
  [1722] = {.lex_state = 11},
  [1723] = {.lex_state = 11},
  [1724] = {.lex_state = 11},
  [1725] = {.lex_state = 11},
  [1726] = {.lex_state = 11},
  [1727] = {.lex_state = 11},
  [1728] = {.lex_state = 11},
  [1729] = {.lex_state = 11},
  [1730] = {.lex_state = 11},
  [1731] = {.lex_state = 11},
  [1732] = {.lex_state = 11},
  [1733] = {.lex_state = 11},
  [1734] = {.lex_state = 11},
  [1735] = {.lex_state = 11},
  [1736] = {.lex_state = 11},
  [1737] = {.lex_state = 11},
  [1738] = {.lex_state = 11},
  [1739] = {.lex_state = 11},
  [1740] = {.lex_state = 11},
  [1741] = {.lex_state = 11},
  [1742] = {.lex_state = 11},
  [1743] = {.lex_state = 11},
  [1744] = {.lex_state = 11},
  [1745] = {.lex_state = 11},
  [1746] = {.lex_state = 11},
  [1747] = {.lex_state = 11},
  [1748] = {.lex_state = 11},
  [1749] = {.lex_state = 11},
  [1750] = {.lex_state = 11},
  [1751] = {.lex_state = 11},
  [1752] = {.lex_state = 11},
  [1753] = {.lex_state = 11},
  [1754] = {.lex_state = 11},
  [1755] = {.lex_state = 11},
  [1756] = {.lex_state = 11},
  [1757] = {.lex_state = 11},
  [1758] = {.lex_state = 11},
  [1759] = {.lex_state = 11},
  [1760] = {.lex_state = 11},
  [1761] = {.lex_state = 11},
  [1762] = {.lex_state = 11},
  [1763] = {.lex_state = 11},
  [1764] = {.lex_state = 11},
  [1765] = {.lex_state = 11},
  [1766] = {.lex_state = 11},
  [1767] = {.lex_state = 11},
  [1768] = {.lex_state = 11},
  [1769] = {.lex_state = 11},
  [1770] = {.lex_state = 11},
  [1771] = {.lex_state = 11},
  [1772] = {.lex_state = 11},
  [1773] = {.lex_state = 11},
  [1774] = {.lex_state = 11},
  [1775] = {.lex_state = 11},
  [1776] = {.lex_state = 11},
  [1777] = {.lex_state = 11},
  [1778] = {.lex_state = 11},
  [1779] = {.lex_state = 11},
  [1780] = {.lex_state = 11},
  [1781] = {.lex_state = 11},
  [1782] = {.lex_state = 11},
  [1783] = {.lex_state = 11},
  [1784] = {.lex_state = 11},
  [1785] = {.lex_state = 11},
  [1786] = {.lex_state = 11},
  [1787] = {.lex_state = 11},
  [1788] = {.lex_state = 11},
  [1789] = {.lex_state = 11},
  [1790] = {.lex_state = 11},
  [1791] = {.lex_state = 11},
  [1792] = {.lex_state = 11},
  [1793] = {.lex_state = 11},
  [1794] = {.lex_state = 11},
  [1795] = {.lex_state = 11},
  [1796] = {.lex_state = 11},
  [1797] = {.lex_state = 11},
  [1798] = {.lex_state = 11},
  [1799] = {.lex_state = 11},
  [1800] = {.lex_state = 11},
  [1801] = {.lex_state = 11},
  [1802] = {.lex_state = 11},
  [1803] = {.lex_state = 11},
  [1804] = {.lex_state = 11},
  [1805] = {.lex_state = 11},
  [1806] = {.lex_state = 11},
  [1807] = {.lex_state = 11},
  [1808] = {.lex_state = 11},
  [1809] = {.lex_state = 11},
  [1810] = {.lex_state = 11},
  [1811] = {.lex_state = 11},
  [1812] = {.lex_state = 11},
  [1813] = {.lex_state = 11},
  [1814] = {.lex_state = 11},
  [1815] = {.lex_state = 11},
  [1816] = {.lex_state = 11},
  [1817] = {.lex_state = 11},
  [1818] = {.lex_state = 11},
  [1819] = {.lex_state = 11},
  [1820] = {.lex_state = 11},
  [1821] = {.lex_state = 11},
  [1822] = {.lex_state = 11},
  [1823] = {.lex_state = 11},
  [1824] = {.lex_state = 11},
  [1825] = {.lex_state = 11},
  [1826] = {.lex_state = 11},
  [1827] = {.lex_state = 11},
  [1828] = {.lex_state = 11},
  [1829] = {.lex_state = 11},
  [1830] = {.lex_state = 11},
  [1831] = {.lex_state = 11},
  [1832] = {.lex_state = 11},
  [1833] = {.lex_state = 11},
  [1834] = {.lex_state = 11},
  [1835] = {.lex_state = 11},
  [1836] = {.lex_state = 11},
  [1837] = {.lex_state = 11},
  [1838] = {.lex_state = 11},
  [1839] = {.lex_state = 11},
  [1840] = {.lex_state = 11},
  [1841] = {.lex_state = 11},
  [1842] = {.lex_state = 11},
  [1843] = {.lex_state = 11},
  [1844] = {.lex_state = 11},
  [1845] = {.lex_state = 11},
  [1846] = {.lex_state = 11},
  [1847] = {.lex_state = 11},
  [1848] = {.lex_state = 11},
  [1849] = {.lex_state = 11},
  [1850] = {.lex_state = 11},
  [1851] = {.lex_state = 11},
  [1852] = {.lex_state = 11},
  [1853] = {.lex_state = 11},
  [1854] = {.lex_state = 11},
  [1855] = {.lex_state = 11},
  [1856] = {.lex_state = 11},
  [1857] = {.lex_state = 11},
  [1858] = {.lex_state = 11},
  [1859] = {.lex_state = 11},
  [1860] = {.lex_state = 11},
  [1861] = {.lex_state = 11},
  [1862] = {.lex_state = 11},
  [1863] = {.lex_state = 11},
  [1864] = {.lex_state = 11},
  [1865] = {.lex_state = 11},
  [1866] = {.lex_state = 11},
  [1867] = {.lex_state = 11},
  [1868] = {.lex_state = 11},
  [1869] = {.lex_state = 11},
  [1870] = {.lex_state = 11},
  [1871] = {.lex_state = 11},
  [1872] = {.lex_state = 11},
  [1873] = {.lex_state = 11},
  [1874] = {.lex_state = 11},
  [1875] = {.lex_state = 11},
  [1876] = {.lex_state = 11},
  [1877] = {.lex_state = 11},
  [1878] = {.lex_state = 11},
  [1879] = {.lex_state = 11},
  [1880] = {.lex_state = 11},
  [1881] = {.lex_state = 11},
  [1882] = {.lex_state = 11},
  [1883] = {.lex_state = 11},
  [1884] = {.lex_state = 11},
  [1885] = {.lex_state = 11},
  [1886] = {.lex_state = 11},
  [1887] = {.lex_state = 11},
  [1888] = {.lex_state = 11},
  [1889] = {.lex_state = 11},
  [1890] = {.lex_state = 11},
  [1891] = {.lex_state = 11},
  [1892] = {.lex_state = 11},
  [1893] = {.lex_state = 11},
  [1894] = {.lex_state = 11},
  [1895] = {.lex_state = 11},
  [1896] = {.lex_state = 11},
  [1897] = {.lex_state = 11},
  [1898] = {.lex_state = 11},
  [1899] = {.lex_state = 11},
  [1900] = {.lex_state = 11},
  [1901] = {.lex_state = 11},
  [1902] = {.lex_state = 11},
  [1903] = {.lex_state = 11},
  [1904] = {.lex_state = 11},
  [1905] = {.lex_state = 11},
  [1906] = {.lex_state = 11},
  [1907] = {.lex_state = 11},
  [1908] = {.lex_state = 11},
  [1909] = {.lex_state = 11},
  [1910] = {.lex_state = 11},
  [1911] = {.lex_state = 11},
  [1912] = {.lex_state = 11},
  [1913] = {.lex_state = 11},
  [1914] = {.lex_state = 11},
  [1915] = {.lex_state = 11},
  [1916] = {.lex_state = 11},
  [1917] = {.lex_state = 11},
  [1918] = {.lex_state = 11},
  [1919] = {.lex_state = 11},
  [1920] = {.lex_state = 11},
  [1921] = {.lex_state = 11},
  [1922] = {.lex_state = 11},
  [1923] = {.lex_state = 11},
  [1924] = {.lex_state = 11},
  [1925] = {.lex_state = 11},
  [1926] = {.lex_state = 11},
  [1927] = {.lex_state = 73, .external_lex_state = 3},
  [1928] = {.lex_state = 73, .external_lex_state = 3},
  [1929] = {.lex_state = 14, .external_lex_state = 3},
  [1930] = {.lex_state = 73, .external_lex_state = 3},
  [1931] = {.lex_state = 73, .external_lex_state = 3},
  [1932] = {.lex_state = 14, .external_lex_state = 3},
  [1933] = {.lex_state = 73, .external_lex_state = 3},
  [1934] = {.lex_state = 73, .external_lex_state = 3},
  [1935] = {.lex_state = 73, .external_lex_state = 3},
  [1936] = {.lex_state = 73, .external_lex_state = 3},
  [1937] = {.lex_state = 11},
  [1938] = {.lex_state = 11},
  [1939] = {.lex_state = 11},
  [1940] = {.lex_state = 11},
  [1941] = {.lex_state = 11},
  [1942] = {.lex_state = 11},
  [1943] = {.lex_state = 11},
  [1944] = {.lex_state = 11},
  [1945] = {.lex_state = 13, .external_lex_state = 3},
  [1946] = {.lex_state = 11},
  [1947] = {.lex_state = 12, .external_lex_state = 3},
  [1948] = {.lex_state = 12, .external_lex_state = 3},
  [1949] = {.lex_state = 11, .external_lex_state = 5},
  [1950] = {.lex_state = 11, .external_lex_state = 5},
  [1951] = {.lex_state = 72, .external_lex_state = 4},
  [1952] = {.lex_state = 11},
  [1953] = {.lex_state = 72, .external_lex_state = 4},
  [1954] = {.lex_state = 72, .external_lex_state = 4},
  [1955] = {.lex_state = 13, .external_lex_state = 3},
  [1956] = {.lex_state = 13, .external_lex_state = 3},
  [1957] = {.lex_state = 11, .external_lex_state = 5},
  [1958] = {.lex_state = 12, .external_lex_state = 3},
  [1959] = {.lex_state = 11, .external_lex_state = 5},
  [1960] = {.lex_state = 11, .external_lex_state = 5},
  [1961] = {.lex_state = 12, .external_lex_state = 3},
  [1962] = {.lex_state = 12, .external_lex_state = 3},
  [1963] = {.lex_state = 12, .external_lex_state = 3},
  [1964] = {.lex_state = 12, .external_lex_state = 3},
  [1965] = {.lex_state = 11, .external_lex_state = 5},
  [1966] = {.lex_state = 11, .external_lex_state = 5},
  [1967] = {.lex_state = 11, .external_lex_state = 5},
  [1968] = {.lex_state = 11, .external_lex_state = 5},
  [1969] = {.lex_state = 11},
  [1970] = {.lex_state = 11},
  [1971] = {.lex_state = 13, .external_lex_state = 3},
  [1972] = {.lex_state = 11},
  [1973] = {.lex_state = 13, .external_lex_state = 3},
  [1974] = {.lex_state = 11},
  [1975] = {.lex_state = 11},
  [1976] = {.lex_state = 13, .external_lex_state = 3},
  [1977] = {.lex_state = 13, .external_lex_state = 3},
  [1978] = {.lex_state = 13, .external_lex_state = 3},
  [1979] = {.lex_state = 13, .external_lex_state = 3},
  [1980] = {.lex_state = 13, .external_lex_state = 3},
  [1981] = {.lex_state = 13, .external_lex_state = 3},
  [1982] = {.lex_state = 13, .external_lex_state = 3},
  [1983] = {.lex_state = 13, .external_lex_state = 3},
  [1984] = {.lex_state = 13, .external_lex_state = 3},
  [1985] = {.lex_state = 13, .external_lex_state = 3},
  [1986] = {.lex_state = 13, .external_lex_state = 3},
  [1987] = {.lex_state = 13, .external_lex_state = 3},
  [1988] = {.lex_state = 13, .external_lex_state = 3},
  [1989] = {.lex_state = 11, .external_lex_state = 5},
  [1990] = {.lex_state = 11, .external_lex_state = 5},
  [1991] = {.lex_state = 11, .external_lex_state = 5},
  [1992] = {.lex_state = 11, .external_lex_state = 5},
  [1993] = {.lex_state = 11, .external_lex_state = 5},
  [1994] = {.lex_state = 11, .external_lex_state = 5},
  [1995] = {.lex_state = 11, .external_lex_state = 5},
  [1996] = {.lex_state = 11, .external_lex_state = 5},
  [1997] = {.lex_state = 11, .external_lex_state = 5},
  [1998] = {.lex_state = 11, .external_lex_state = 5},
  [1999] = {.lex_state = 11, .external_lex_state = 5},
  [2000] = {.lex_state = 11, .external_lex_state = 5},
  [2001] = {.lex_state = 11, .external_lex_state = 5},
  [2002] = {.lex_state = 11, .external_lex_state = 5},
  [2003] = {.lex_state = 11, .external_lex_state = 5},
  [2004] = {.lex_state = 11, .external_lex_state = 5},
  [2005] = {.lex_state = 11, .external_lex_state = 5},
  [2006] = {.lex_state = 11, .external_lex_state = 5},
  [2007] = {.lex_state = 11, .external_lex_state = 5},
  [2008] = {.lex_state = 11, .external_lex_state = 5},
  [2009] = {.lex_state = 11, .external_lex_state = 5},
  [2010] = {.lex_state = 11, .external_lex_state = 5},
  [2011] = {.lex_state = 11, .external_lex_state = 5},
  [2012] = {.lex_state = 11, .external_lex_state = 5},
  [2013] = {.lex_state = 11, .external_lex_state = 5},
  [2014] = {.lex_state = 11, .external_lex_state = 5},
  [2015] = {.lex_state = 11, .external_lex_state = 5},
  [2016] = {.lex_state = 11, .external_lex_state = 5},
  [2017] = {.lex_state = 11, .external_lex_state = 5},
  [2018] = {.lex_state = 11, .external_lex_state = 5},
  [2019] = {.lex_state = 11, .external_lex_state = 5},
  [2020] = {.lex_state = 11, .external_lex_state = 5},
  [2021] = {.lex_state = 11, .external_lex_state = 5},
  [2022] = {.lex_state = 11, .external_lex_state = 5},
  [2023] = {.lex_state = 11, .external_lex_state = 5},
  [2024] = {.lex_state = 11, .external_lex_state = 5},
  [2025] = {.lex_state = 11, .external_lex_state = 5},
  [2026] = {.lex_state = 11, .external_lex_state = 5},
  [2027] = {.lex_state = 11, .external_lex_state = 5},
  [2028] = {.lex_state = 11, .external_lex_state = 5},
  [2029] = {.lex_state = 11, .external_lex_state = 5},
  [2030] = {.lex_state = 11, .external_lex_state = 5},
  [2031] = {.lex_state = 11, .external_lex_state = 5},
  [2032] = {.lex_state = 11, .external_lex_state = 5},
  [2033] = {.lex_state = 11, .external_lex_state = 5},
  [2034] = {.lex_state = 11, .external_lex_state = 5},
  [2035] = {.lex_state = 11, .external_lex_state = 5},
  [2036] = {.lex_state = 11, .external_lex_state = 5},
  [2037] = {.lex_state = 11, .external_lex_state = 5},
  [2038] = {.lex_state = 11, .external_lex_state = 5},
  [2039] = {.lex_state = 11, .external_lex_state = 5},
  [2040] = {.lex_state = 11, .external_lex_state = 5},
  [2041] = {.lex_state = 11, .external_lex_state = 5},
  [2042] = {.lex_state = 11, .external_lex_state = 5},
  [2043] = {.lex_state = 11, .external_lex_state = 5},
  [2044] = {.lex_state = 11, .external_lex_state = 5},
  [2045] = {.lex_state = 11, .external_lex_state = 5},
  [2046] = {.lex_state = 11, .external_lex_state = 5},
  [2047] = {.lex_state = 11, .external_lex_state = 5},
  [2048] = {.lex_state = 11, .external_lex_state = 5},
  [2049] = {.lex_state = 11, .external_lex_state = 5},
  [2050] = {.lex_state = 11, .external_lex_state = 5},
  [2051] = {.lex_state = 11, .external_lex_state = 5},
  [2052] = {.lex_state = 11, .external_lex_state = 5},
  [2053] = {.lex_state = 11, .external_lex_state = 5},
  [2054] = {.lex_state = 11, .external_lex_state = 5},
  [2055] = {.lex_state = 11, .external_lex_state = 5},
  [2056] = {.lex_state = 11, .external_lex_state = 5},
  [2057] = {.lex_state = 11, .external_lex_state = 5},
  [2058] = {.lex_state = 11, .external_lex_state = 5},
  [2059] = {.lex_state = 11, .external_lex_state = 5},
  [2060] = {.lex_state = 11, .external_lex_state = 5},
  [2061] = {.lex_state = 11, .external_lex_state = 5},
  [2062] = {.lex_state = 11, .external_lex_state = 5},
  [2063] = {.lex_state = 11, .external_lex_state = 5},
  [2064] = {.lex_state = 11, .external_lex_state = 5},
  [2065] = {.lex_state = 11, .external_lex_state = 5},
  [2066] = {.lex_state = 11, .external_lex_state = 5},
  [2067] = {.lex_state = 11, .external_lex_state = 5},
  [2068] = {.lex_state = 11, .external_lex_state = 5},
  [2069] = {.lex_state = 11, .external_lex_state = 5},
  [2070] = {.lex_state = 11, .external_lex_state = 5},
  [2071] = {.lex_state = 11, .external_lex_state = 5},
  [2072] = {.lex_state = 11, .external_lex_state = 5},
  [2073] = {.lex_state = 11, .external_lex_state = 5},
  [2074] = {.lex_state = 11, .external_lex_state = 5},
  [2075] = {.lex_state = 11, .external_lex_state = 5},
  [2076] = {.lex_state = 11, .external_lex_state = 5},
  [2077] = {.lex_state = 11, .external_lex_state = 5},
  [2078] = {.lex_state = 11, .external_lex_state = 5},
  [2079] = {.lex_state = 11, .external_lex_state = 5},
  [2080] = {.lex_state = 11, .external_lex_state = 5},
  [2081] = {.lex_state = 11, .external_lex_state = 5},
  [2082] = {.lex_state = 11, .external_lex_state = 5},
  [2083] = {.lex_state = 11, .external_lex_state = 5},
  [2084] = {.lex_state = 11, .external_lex_state = 5},
  [2085] = {.lex_state = 11, .external_lex_state = 5},
  [2086] = {.lex_state = 11, .external_lex_state = 5},
  [2087] = {.lex_state = 11, .external_lex_state = 5},
  [2088] = {.lex_state = 11, .external_lex_state = 5},
  [2089] = {.lex_state = 11, .external_lex_state = 5},
  [2090] = {.lex_state = 11, .external_lex_state = 5},
  [2091] = {.lex_state = 11, .external_lex_state = 5},
  [2092] = {.lex_state = 11, .external_lex_state = 5},
  [2093] = {.lex_state = 11, .external_lex_state = 5},
  [2094] = {.lex_state = 11, .external_lex_state = 5},
  [2095] = {.lex_state = 11, .external_lex_state = 5},
  [2096] = {.lex_state = 11, .external_lex_state = 5},
  [2097] = {.lex_state = 11, .external_lex_state = 5},
  [2098] = {.lex_state = 11, .external_lex_state = 5},
  [2099] = {.lex_state = 11, .external_lex_state = 5},
  [2100] = {.lex_state = 11, .external_lex_state = 5},
  [2101] = {.lex_state = 11, .external_lex_state = 5},
  [2102] = {.lex_state = 11, .external_lex_state = 5},
  [2103] = {.lex_state = 11, .external_lex_state = 5},
  [2104] = {.lex_state = 11, .external_lex_state = 5},
  [2105] = {.lex_state = 11, .external_lex_state = 5},
  [2106] = {.lex_state = 11, .external_lex_state = 5},
  [2107] = {.lex_state = 11, .external_lex_state = 5},
  [2108] = {.lex_state = 11, .external_lex_state = 5},
  [2109] = {.lex_state = 11, .external_lex_state = 5},
  [2110] = {.lex_state = 11, .external_lex_state = 5},
  [2111] = {.lex_state = 11, .external_lex_state = 5},
  [2112] = {.lex_state = 11, .external_lex_state = 5},
  [2113] = {.lex_state = 11, .external_lex_state = 5},
  [2114] = {.lex_state = 11, .external_lex_state = 5},
  [2115] = {.lex_state = 11, .external_lex_state = 5},
  [2116] = {.lex_state = 11, .external_lex_state = 5},
  [2117] = {.lex_state = 11, .external_lex_state = 5},
  [2118] = {.lex_state = 11, .external_lex_state = 5},
  [2119] = {.lex_state = 11, .external_lex_state = 5},
  [2120] = {.lex_state = 11, .external_lex_state = 5},
  [2121] = {.lex_state = 11, .external_lex_state = 5},
  [2122] = {.lex_state = 11, .external_lex_state = 5},
  [2123] = {.lex_state = 11, .external_lex_state = 5},
  [2124] = {.lex_state = 11, .external_lex_state = 5},
  [2125] = {.lex_state = 11, .external_lex_state = 5},
  [2126] = {.lex_state = 11, .external_lex_state = 5},
  [2127] = {.lex_state = 11, .external_lex_state = 5},
  [2128] = {.lex_state = 11, .external_lex_state = 5},
  [2129] = {.lex_state = 11, .external_lex_state = 5},
  [2130] = {.lex_state = 11, .external_lex_state = 5},
  [2131] = {.lex_state = 11, .external_lex_state = 5},
  [2132] = {.lex_state = 11, .external_lex_state = 5},
  [2133] = {.lex_state = 11, .external_lex_state = 5},
  [2134] = {.lex_state = 11, .external_lex_state = 5},
  [2135] = {.lex_state = 11, .external_lex_state = 5},
  [2136] = {.lex_state = 11, .external_lex_state = 5},
  [2137] = {.lex_state = 11, .external_lex_state = 5},
  [2138] = {.lex_state = 11, .external_lex_state = 5},
  [2139] = {.lex_state = 11, .external_lex_state = 5},
  [2140] = {.lex_state = 11, .external_lex_state = 5},
  [2141] = {.lex_state = 11, .external_lex_state = 5},
  [2142] = {.lex_state = 11, .external_lex_state = 5},
  [2143] = {.lex_state = 11, .external_lex_state = 5},
  [2144] = {.lex_state = 11, .external_lex_state = 5},
  [2145] = {.lex_state = 11, .external_lex_state = 5},
  [2146] = {.lex_state = 11, .external_lex_state = 5},
  [2147] = {.lex_state = 11, .external_lex_state = 5},
  [2148] = {.lex_state = 11, .external_lex_state = 5},
  [2149] = {.lex_state = 11, .external_lex_state = 5},
  [2150] = {.lex_state = 11, .external_lex_state = 5},
  [2151] = {.lex_state = 11, .external_lex_state = 5},
  [2152] = {.lex_state = 11, .external_lex_state = 5},
  [2153] = {.lex_state = 11, .external_lex_state = 5},
  [2154] = {.lex_state = 11, .external_lex_state = 5},
  [2155] = {.lex_state = 11, .external_lex_state = 5},
  [2156] = {.lex_state = 11, .external_lex_state = 5},
  [2157] = {.lex_state = 11, .external_lex_state = 5},
  [2158] = {.lex_state = 11, .external_lex_state = 5},
  [2159] = {.lex_state = 11, .external_lex_state = 5},
  [2160] = {.lex_state = 11, .external_lex_state = 5},
  [2161] = {.lex_state = 11, .external_lex_state = 5},
  [2162] = {.lex_state = 11, .external_lex_state = 5},
  [2163] = {.lex_state = 11, .external_lex_state = 5},
  [2164] = {.lex_state = 11, .external_lex_state = 5},
  [2165] = {.lex_state = 11, .external_lex_state = 5},
  [2166] = {.lex_state = 11, .external_lex_state = 5},
  [2167] = {.lex_state = 11, .external_lex_state = 5},
  [2168] = {.lex_state = 11, .external_lex_state = 5},
  [2169] = {.lex_state = 11, .external_lex_state = 5},
  [2170] = {.lex_state = 11, .external_lex_state = 5},
  [2171] = {.lex_state = 11, .external_lex_state = 5},
  [2172] = {.lex_state = 11, .external_lex_state = 5},
  [2173] = {.lex_state = 11, .external_lex_state = 5},
  [2174] = {.lex_state = 11, .external_lex_state = 5},
  [2175] = {.lex_state = 11, .external_lex_state = 5},
  [2176] = {.lex_state = 11, .external_lex_state = 5},
  [2177] = {.lex_state = 11, .external_lex_state = 5},
  [2178] = {.lex_state = 11, .external_lex_state = 5},
  [2179] = {.lex_state = 11, .external_lex_state = 5},
  [2180] = {.lex_state = 11, .external_lex_state = 5},
  [2181] = {.lex_state = 11, .external_lex_state = 5},
  [2182] = {.lex_state = 11, .external_lex_state = 5},
  [2183] = {.lex_state = 11, .external_lex_state = 5},
  [2184] = {.lex_state = 11, .external_lex_state = 5},
  [2185] = {.lex_state = 11, .external_lex_state = 5},
  [2186] = {.lex_state = 11, .external_lex_state = 5},
  [2187] = {.lex_state = 11, .external_lex_state = 5},
  [2188] = {.lex_state = 11, .external_lex_state = 5},
  [2189] = {.lex_state = 11, .external_lex_state = 5},
  [2190] = {.lex_state = 11, .external_lex_state = 5},
  [2191] = {.lex_state = 11, .external_lex_state = 5},
  [2192] = {.lex_state = 11, .external_lex_state = 5},
  [2193] = {.lex_state = 11, .external_lex_state = 5},
  [2194] = {.lex_state = 11, .external_lex_state = 5},
  [2195] = {.lex_state = 11, .external_lex_state = 5},
  [2196] = {.lex_state = 11, .external_lex_state = 5},
  [2197] = {.lex_state = 11, .external_lex_state = 5},
  [2198] = {.lex_state = 11, .external_lex_state = 5},
  [2199] = {.lex_state = 11, .external_lex_state = 5},
  [2200] = {.lex_state = 11, .external_lex_state = 5},
  [2201] = {.lex_state = 11, .external_lex_state = 5},
  [2202] = {.lex_state = 11, .external_lex_state = 5},
  [2203] = {.lex_state = 11, .external_lex_state = 5},
  [2204] = {.lex_state = 11, .external_lex_state = 5},
  [2205] = {.lex_state = 11, .external_lex_state = 5},
  [2206] = {.lex_state = 11, .external_lex_state = 5},
  [2207] = {.lex_state = 11, .external_lex_state = 5},
  [2208] = {.lex_state = 11, .external_lex_state = 5},
  [2209] = {.lex_state = 11, .external_lex_state = 5},
  [2210] = {.lex_state = 11, .external_lex_state = 5},
  [2211] = {.lex_state = 11, .external_lex_state = 5},
  [2212] = {.lex_state = 11, .external_lex_state = 5},
  [2213] = {.lex_state = 11, .external_lex_state = 5},
  [2214] = {.lex_state = 11, .external_lex_state = 5},
  [2215] = {.lex_state = 11, .external_lex_state = 5},
  [2216] = {.lex_state = 11, .external_lex_state = 5},
  [2217] = {.lex_state = 11, .external_lex_state = 5},
  [2218] = {.lex_state = 11, .external_lex_state = 5},
  [2219] = {.lex_state = 11, .external_lex_state = 5},
  [2220] = {.lex_state = 11, .external_lex_state = 5},
  [2221] = {.lex_state = 11, .external_lex_state = 5},
  [2222] = {.lex_state = 11, .external_lex_state = 5},
  [2223] = {.lex_state = 11, .external_lex_state = 5},
  [2224] = {.lex_state = 11, .external_lex_state = 5},
  [2225] = {.lex_state = 11, .external_lex_state = 5},
  [2226] = {.lex_state = 11, .external_lex_state = 5},
  [2227] = {.lex_state = 11, .external_lex_state = 5},
  [2228] = {.lex_state = 11, .external_lex_state = 5},
  [2229] = {.lex_state = 11, .external_lex_state = 5},
  [2230] = {.lex_state = 11, .external_lex_state = 5},
  [2231] = {.lex_state = 11, .external_lex_state = 5},
  [2232] = {.lex_state = 11, .external_lex_state = 5},
  [2233] = {.lex_state = 11, .external_lex_state = 5},
  [2234] = {.lex_state = 11, .external_lex_state = 5},
  [2235] = {.lex_state = 11, .external_lex_state = 5},
  [2236] = {.lex_state = 11, .external_lex_state = 5},
  [2237] = {.lex_state = 11, .external_lex_state = 5},
  [2238] = {.lex_state = 11, .external_lex_state = 5},
  [2239] = {.lex_state = 11, .external_lex_state = 5},
  [2240] = {.lex_state = 11, .external_lex_state = 5},
  [2241] = {.lex_state = 11, .external_lex_state = 5},
  [2242] = {.lex_state = 11, .external_lex_state = 5},
  [2243] = {.lex_state = 11, .external_lex_state = 5},
  [2244] = {.lex_state = 11, .external_lex_state = 5},
  [2245] = {.lex_state = 11, .external_lex_state = 5},
  [2246] = {.lex_state = 11, .external_lex_state = 5},
  [2247] = {.lex_state = 11, .external_lex_state = 5},
  [2248] = {.lex_state = 11, .external_lex_state = 5},
  [2249] = {.lex_state = 11, .external_lex_state = 5},
  [2250] = {.lex_state = 11, .external_lex_state = 5},
  [2251] = {.lex_state = 11, .external_lex_state = 5},
  [2252] = {.lex_state = 11, .external_lex_state = 5},
  [2253] = {.lex_state = 11, .external_lex_state = 5},
  [2254] = {.lex_state = 11, .external_lex_state = 5},
  [2255] = {.lex_state = 11, .external_lex_state = 5},
  [2256] = {.lex_state = 11, .external_lex_state = 5},
  [2257] = {.lex_state = 11, .external_lex_state = 5},
  [2258] = {.lex_state = 11, .external_lex_state = 5},
  [2259] = {.lex_state = 11, .external_lex_state = 5},
  [2260] = {.lex_state = 11, .external_lex_state = 5},
  [2261] = {.lex_state = 11, .external_lex_state = 5},
  [2262] = {.lex_state = 11, .external_lex_state = 5},
  [2263] = {.lex_state = 11, .external_lex_state = 5},
  [2264] = {.lex_state = 11, .external_lex_state = 5},
  [2265] = {.lex_state = 11, .external_lex_state = 5},
  [2266] = {.lex_state = 11, .external_lex_state = 5},
  [2267] = {.lex_state = 11, .external_lex_state = 5},
  [2268] = {.lex_state = 11, .external_lex_state = 5},
  [2269] = {.lex_state = 11, .external_lex_state = 5},
  [2270] = {.lex_state = 11, .external_lex_state = 5},
  [2271] = {.lex_state = 11, .external_lex_state = 5},
  [2272] = {.lex_state = 11, .external_lex_state = 5},
  [2273] = {.lex_state = 11, .external_lex_state = 5},
  [2274] = {.lex_state = 11, .external_lex_state = 5},
  [2275] = {.lex_state = 11, .external_lex_state = 5},
  [2276] = {.lex_state = 11, .external_lex_state = 5},
  [2277] = {.lex_state = 11, .external_lex_state = 5},
  [2278] = {.lex_state = 11, .external_lex_state = 5},
  [2279] = {.lex_state = 11, .external_lex_state = 5},
  [2280] = {.lex_state = 11, .external_lex_state = 5},
  [2281] = {.lex_state = 11, .external_lex_state = 5},
  [2282] = {.lex_state = 11, .external_lex_state = 5},
  [2283] = {.lex_state = 11, .external_lex_state = 5},
  [2284] = {.lex_state = 11, .external_lex_state = 5},
  [2285] = {.lex_state = 11, .external_lex_state = 5},
  [2286] = {.lex_state = 11, .external_lex_state = 5},
  [2287] = {.lex_state = 11, .external_lex_state = 5},
  [2288] = {.lex_state = 11, .external_lex_state = 5},
  [2289] = {.lex_state = 11, .external_lex_state = 5},
  [2290] = {.lex_state = 11, .external_lex_state = 5},
  [2291] = {.lex_state = 11, .external_lex_state = 5},
  [2292] = {.lex_state = 11, .external_lex_state = 5},
  [2293] = {.lex_state = 11, .external_lex_state = 5},
  [2294] = {.lex_state = 11, .external_lex_state = 5},
  [2295] = {.lex_state = 11, .external_lex_state = 5},
  [2296] = {.lex_state = 11, .external_lex_state = 5},
  [2297] = {.lex_state = 11, .external_lex_state = 5},
  [2298] = {.lex_state = 11, .external_lex_state = 5},
  [2299] = {.lex_state = 11, .external_lex_state = 5},
  [2300] = {.lex_state = 11, .external_lex_state = 5},
  [2301] = {.lex_state = 11, .external_lex_state = 5},
  [2302] = {.lex_state = 11, .external_lex_state = 5},
  [2303] = {.lex_state = 11, .external_lex_state = 5},
  [2304] = {.lex_state = 11, .external_lex_state = 5},
  [2305] = {.lex_state = 11, .external_lex_state = 5},
  [2306] = {.lex_state = 11, .external_lex_state = 5},
  [2307] = {.lex_state = 11, .external_lex_state = 5},
  [2308] = {.lex_state = 11, .external_lex_state = 5},
  [2309] = {.lex_state = 11, .external_lex_state = 5},
  [2310] = {.lex_state = 11, .external_lex_state = 5},
  [2311] = {.lex_state = 11, .external_lex_state = 5},
  [2312] = {.lex_state = 11, .external_lex_state = 5},
  [2313] = {.lex_state = 11, .external_lex_state = 5},
  [2314] = {.lex_state = 11, .external_lex_state = 5},
  [2315] = {.lex_state = 11, .external_lex_state = 5},
  [2316] = {.lex_state = 11, .external_lex_state = 5},
  [2317] = {.lex_state = 11, .external_lex_state = 5},
  [2318] = {.lex_state = 11, .external_lex_state = 5},
  [2319] = {.lex_state = 11, .external_lex_state = 5},
  [2320] = {.lex_state = 11, .external_lex_state = 5},
  [2321] = {.lex_state = 11, .external_lex_state = 5},
  [2322] = {.lex_state = 11, .external_lex_state = 5},
  [2323] = {.lex_state = 11, .external_lex_state = 5},
  [2324] = {.lex_state = 11, .external_lex_state = 5},
  [2325] = {.lex_state = 11, .external_lex_state = 5},
  [2326] = {.lex_state = 11, .external_lex_state = 5},
  [2327] = {.lex_state = 11, .external_lex_state = 5},
  [2328] = {.lex_state = 11, .external_lex_state = 5},
  [2329] = {.lex_state = 11, .external_lex_state = 5},
  [2330] = {.lex_state = 11, .external_lex_state = 5},
  [2331] = {.lex_state = 11, .external_lex_state = 5},
  [2332] = {.lex_state = 11, .external_lex_state = 5},
  [2333] = {.lex_state = 11, .external_lex_state = 5},
  [2334] = {.lex_state = 11, .external_lex_state = 5},
  [2335] = {.lex_state = 11, .external_lex_state = 5},
  [2336] = {.lex_state = 11, .external_lex_state = 5},
  [2337] = {.lex_state = 11, .external_lex_state = 5},
  [2338] = {.lex_state = 11, .external_lex_state = 5},
  [2339] = {.lex_state = 11, .external_lex_state = 5},
  [2340] = {.lex_state = 11, .external_lex_state = 5},
  [2341] = {.lex_state = 11, .external_lex_state = 5},
  [2342] = {.lex_state = 11, .external_lex_state = 5},
  [2343] = {.lex_state = 11, .external_lex_state = 5},
  [2344] = {.lex_state = 11, .external_lex_state = 5},
  [2345] = {.lex_state = 11, .external_lex_state = 5},
  [2346] = {.lex_state = 11, .external_lex_state = 5},
  [2347] = {.lex_state = 11, .external_lex_state = 5},
  [2348] = {.lex_state = 11, .external_lex_state = 5},
  [2349] = {.lex_state = 11, .external_lex_state = 5},
  [2350] = {.lex_state = 11, .external_lex_state = 5},
  [2351] = {.lex_state = 11, .external_lex_state = 5},
  [2352] = {.lex_state = 11, .external_lex_state = 5},
  [2353] = {.lex_state = 11, .external_lex_state = 5},
  [2354] = {.lex_state = 11, .external_lex_state = 5},
  [2355] = {.lex_state = 11, .external_lex_state = 5},
  [2356] = {.lex_state = 11, .external_lex_state = 5},
  [2357] = {.lex_state = 11, .external_lex_state = 5},
  [2358] = {.lex_state = 11, .external_lex_state = 5},
  [2359] = {.lex_state = 11, .external_lex_state = 5},
  [2360] = {.lex_state = 11, .external_lex_state = 5},
  [2361] = {.lex_state = 11, .external_lex_state = 5},
  [2362] = {.lex_state = 11, .external_lex_state = 5},
  [2363] = {.lex_state = 11, .external_lex_state = 5},
  [2364] = {.lex_state = 11, .external_lex_state = 5},
  [2365] = {.lex_state = 11, .external_lex_state = 5},
  [2366] = {.lex_state = 11, .external_lex_state = 5},
  [2367] = {.lex_state = 11, .external_lex_state = 5},
  [2368] = {.lex_state = 11, .external_lex_state = 5},
  [2369] = {.lex_state = 11, .external_lex_state = 5},
  [2370] = {.lex_state = 11, .external_lex_state = 5},
  [2371] = {.lex_state = 11, .external_lex_state = 5},
  [2372] = {.lex_state = 11, .external_lex_state = 5},
  [2373] = {.lex_state = 11, .external_lex_state = 5},
  [2374] = {.lex_state = 11, .external_lex_state = 5},
  [2375] = {.lex_state = 11, .external_lex_state = 5},
  [2376] = {.lex_state = 11, .external_lex_state = 5},
  [2377] = {.lex_state = 11, .external_lex_state = 5},
  [2378] = {.lex_state = 11, .external_lex_state = 5},
  [2379] = {.lex_state = 11, .external_lex_state = 5},
  [2380] = {.lex_state = 11, .external_lex_state = 5},
  [2381] = {.lex_state = 11, .external_lex_state = 5},
  [2382] = {.lex_state = 11, .external_lex_state = 5},
  [2383] = {.lex_state = 11, .external_lex_state = 5},
  [2384] = {.lex_state = 11, .external_lex_state = 5},
  [2385] = {.lex_state = 11, .external_lex_state = 5},
  [2386] = {.lex_state = 11, .external_lex_state = 5},
  [2387] = {.lex_state = 11, .external_lex_state = 5},
  [2388] = {.lex_state = 11, .external_lex_state = 5},
  [2389] = {.lex_state = 11, .external_lex_state = 5},
  [2390] = {.lex_state = 11, .external_lex_state = 5},
  [2391] = {.lex_state = 11, .external_lex_state = 5},
  [2392] = {.lex_state = 11, .external_lex_state = 5},
  [2393] = {.lex_state = 11, .external_lex_state = 5},
  [2394] = {.lex_state = 11, .external_lex_state = 5},
  [2395] = {.lex_state = 11, .external_lex_state = 5},
  [2396] = {.lex_state = 11, .external_lex_state = 5},
  [2397] = {.lex_state = 11, .external_lex_state = 5},
  [2398] = {.lex_state = 11, .external_lex_state = 5},
  [2399] = {.lex_state = 11, .external_lex_state = 5},
  [2400] = {.lex_state = 11, .external_lex_state = 5},
  [2401] = {.lex_state = 11, .external_lex_state = 5},
  [2402] = {.lex_state = 11, .external_lex_state = 5},
  [2403] = {.lex_state = 11, .external_lex_state = 5},
  [2404] = {.lex_state = 11, .external_lex_state = 5},
  [2405] = {.lex_state = 11, .external_lex_state = 5},
  [2406] = {.lex_state = 11, .external_lex_state = 5},
  [2407] = {.lex_state = 11, .external_lex_state = 5},
  [2408] = {.lex_state = 11, .external_lex_state = 5},
  [2409] = {.lex_state = 11, .external_lex_state = 5},
  [2410] = {.lex_state = 11, .external_lex_state = 5},
  [2411] = {.lex_state = 11, .external_lex_state = 5},
  [2412] = {.lex_state = 11, .external_lex_state = 5},
  [2413] = {.lex_state = 11, .external_lex_state = 5},
  [2414] = {.lex_state = 11, .external_lex_state = 5},
  [2415] = {.lex_state = 11, .external_lex_state = 5},
  [2416] = {.lex_state = 11, .external_lex_state = 5},
  [2417] = {.lex_state = 11, .external_lex_state = 5},
  [2418] = {.lex_state = 11, .external_lex_state = 5},
  [2419] = {.lex_state = 11, .external_lex_state = 5},
  [2420] = {.lex_state = 11, .external_lex_state = 5},
  [2421] = {.lex_state = 11, .external_lex_state = 5},
  [2422] = {.lex_state = 11, .external_lex_state = 5},
  [2423] = {.lex_state = 11, .external_lex_state = 5},
  [2424] = {.lex_state = 11, .external_lex_state = 5},
  [2425] = {.lex_state = 11, .external_lex_state = 5},
  [2426] = {.lex_state = 11, .external_lex_state = 5},
  [2427] = {.lex_state = 11, .external_lex_state = 5},
  [2428] = {.lex_state = 11, .external_lex_state = 5},
  [2429] = {.lex_state = 11, .external_lex_state = 5},
  [2430] = {.lex_state = 11, .external_lex_state = 5},
  [2431] = {.lex_state = 11, .external_lex_state = 5},
  [2432] = {.lex_state = 11, .external_lex_state = 5},
  [2433] = {.lex_state = 11, .external_lex_state = 5},
  [2434] = {.lex_state = 11, .external_lex_state = 5},
  [2435] = {.lex_state = 11, .external_lex_state = 5},
  [2436] = {.lex_state = 11, .external_lex_state = 5},
  [2437] = {.lex_state = 11, .external_lex_state = 5},
  [2438] = {.lex_state = 11, .external_lex_state = 5},
  [2439] = {.lex_state = 11, .external_lex_state = 5},
  [2440] = {.lex_state = 11, .external_lex_state = 5},
  [2441] = {.lex_state = 11, .external_lex_state = 5},
  [2442] = {.lex_state = 11, .external_lex_state = 5},
  [2443] = {.lex_state = 11, .external_lex_state = 5},
  [2444] = {.lex_state = 11, .external_lex_state = 5},
  [2445] = {.lex_state = 11, .external_lex_state = 5},
  [2446] = {.lex_state = 11, .external_lex_state = 5},
  [2447] = {.lex_state = 11, .external_lex_state = 5},
  [2448] = {.lex_state = 11, .external_lex_state = 5},
  [2449] = {.lex_state = 11, .external_lex_state = 5},
  [2450] = {.lex_state = 11, .external_lex_state = 5},
  [2451] = {.lex_state = 11, .external_lex_state = 5},
  [2452] = {.lex_state = 11, .external_lex_state = 5},
  [2453] = {.lex_state = 11, .external_lex_state = 5},
  [2454] = {.lex_state = 11, .external_lex_state = 5},
  [2455] = {.lex_state = 11, .external_lex_state = 5},
  [2456] = {.lex_state = 11, .external_lex_state = 5},
  [2457] = {.lex_state = 11, .external_lex_state = 5},
  [2458] = {.lex_state = 11, .external_lex_state = 5},
  [2459] = {.lex_state = 11, .external_lex_state = 5},
  [2460] = {.lex_state = 11, .external_lex_state = 5},
  [2461] = {.lex_state = 11, .external_lex_state = 5},
  [2462] = {.lex_state = 11, .external_lex_state = 5},
  [2463] = {.lex_state = 11, .external_lex_state = 5},
  [2464] = {.lex_state = 11, .external_lex_state = 5},
  [2465] = {.lex_state = 11, .external_lex_state = 5},
  [2466] = {.lex_state = 11, .external_lex_state = 5},
  [2467] = {.lex_state = 11, .external_lex_state = 5},
  [2468] = {.lex_state = 11, .external_lex_state = 5},
  [2469] = {.lex_state = 11, .external_lex_state = 5},
  [2470] = {.lex_state = 11, .external_lex_state = 5},
  [2471] = {.lex_state = 11, .external_lex_state = 5},
  [2472] = {.lex_state = 11, .external_lex_state = 5},
  [2473] = {.lex_state = 11, .external_lex_state = 5},
  [2474] = {.lex_state = 11, .external_lex_state = 5},
  [2475] = {.lex_state = 8},
  [2476] = {.lex_state = 11, .external_lex_state = 5},
  [2477] = {.lex_state = 11, .external_lex_state = 5},
  [2478] = {.lex_state = 11, .external_lex_state = 5},
  [2479] = {.lex_state = 11, .external_lex_state = 5},
  [2480] = {.lex_state = 11, .external_lex_state = 5},
  [2481] = {.lex_state = 11, .external_lex_state = 5},
  [2482] = {.lex_state = 11, .external_lex_state = 5},
  [2483] = {.lex_state = 11, .external_lex_state = 5},
  [2484] = {.lex_state = 11, .external_lex_state = 5},
  [2485] = {.lex_state = 11, .external_lex_state = 5},
  [2486] = {.lex_state = 11, .external_lex_state = 5},
  [2487] = {.lex_state = 11, .external_lex_state = 5},
  [2488] = {.lex_state = 11, .external_lex_state = 5},
  [2489] = {.lex_state = 11, .external_lex_state = 5},
  [2490] = {.lex_state = 11, .external_lex_state = 5},
  [2491] = {.lex_state = 11, .external_lex_state = 5},
  [2492] = {.lex_state = 11, .external_lex_state = 5},
  [2493] = {.lex_state = 11, .external_lex_state = 5},
  [2494] = {.lex_state = 11, .external_lex_state = 5},
  [2495] = {.lex_state = 11, .external_lex_state = 5},
  [2496] = {.lex_state = 11, .external_lex_state = 5},
  [2497] = {.lex_state = 11, .external_lex_state = 5},
  [2498] = {.lex_state = 11, .external_lex_state = 5},
  [2499] = {.lex_state = 11, .external_lex_state = 5},
  [2500] = {.lex_state = 11, .external_lex_state = 5},
  [2501] = {.lex_state = 11, .external_lex_state = 5},
  [2502] = {.lex_state = 11, .external_lex_state = 5},
  [2503] = {.lex_state = 11, .external_lex_state = 5},
  [2504] = {.lex_state = 11, .external_lex_state = 5},
  [2505] = {.lex_state = 11, .external_lex_state = 5},
  [2506] = {.lex_state = 11, .external_lex_state = 5},
  [2507] = {.lex_state = 11, .external_lex_state = 5},
  [2508] = {.lex_state = 11, .external_lex_state = 5},
  [2509] = {.lex_state = 11, .external_lex_state = 5},
  [2510] = {.lex_state = 11, .external_lex_state = 5},
  [2511] = {.lex_state = 11, .external_lex_state = 5},
  [2512] = {.lex_state = 11, .external_lex_state = 5},
  [2513] = {.lex_state = 11, .external_lex_state = 5},
  [2514] = {.lex_state = 11, .external_lex_state = 5},
  [2515] = {.lex_state = 11, .external_lex_state = 5},
  [2516] = {.lex_state = 11, .external_lex_state = 5},
  [2517] = {.lex_state = 11, .external_lex_state = 5},
  [2518] = {.lex_state = 11, .external_lex_state = 5},
  [2519] = {.lex_state = 11, .external_lex_state = 5},
  [2520] = {.lex_state = 11, .external_lex_state = 5},
  [2521] = {.lex_state = 11, .external_lex_state = 5},
  [2522] = {.lex_state = 11, .external_lex_state = 5},
  [2523] = {.lex_state = 11, .external_lex_state = 5},
  [2524] = {.lex_state = 11, .external_lex_state = 5},
  [2525] = {.lex_state = 11, .external_lex_state = 5},
  [2526] = {.lex_state = 11, .external_lex_state = 5},
  [2527] = {.lex_state = 11, .external_lex_state = 5},
  [2528] = {.lex_state = 11, .external_lex_state = 5},
  [2529] = {.lex_state = 11, .external_lex_state = 5},
  [2530] = {.lex_state = 11, .external_lex_state = 5},
  [2531] = {.lex_state = 11, .external_lex_state = 5},
  [2532] = {.lex_state = 11, .external_lex_state = 5},
  [2533] = {.lex_state = 11, .external_lex_state = 5},
  [2534] = {.lex_state = 11, .external_lex_state = 5},
  [2535] = {.lex_state = 11, .external_lex_state = 5},
  [2536] = {.lex_state = 11, .external_lex_state = 5},
  [2537] = {.lex_state = 11, .external_lex_state = 5},
  [2538] = {.lex_state = 11, .external_lex_state = 5},
  [2539] = {.lex_state = 11, .external_lex_state = 5},
  [2540] = {.lex_state = 11, .external_lex_state = 5},
  [2541] = {.lex_state = 11, .external_lex_state = 5},
  [2542] = {.lex_state = 11, .external_lex_state = 5},
  [2543] = {.lex_state = 11, .external_lex_state = 5},
  [2544] = {.lex_state = 11, .external_lex_state = 5},
  [2545] = {.lex_state = 11, .external_lex_state = 5},
  [2546] = {.lex_state = 11, .external_lex_state = 5},
  [2547] = {.lex_state = 11, .external_lex_state = 5},
  [2548] = {.lex_state = 11, .external_lex_state = 5},
  [2549] = {.lex_state = 11, .external_lex_state = 5},
  [2550] = {.lex_state = 11, .external_lex_state = 5},
  [2551] = {.lex_state = 11, .external_lex_state = 5},
  [2552] = {.lex_state = 11, .external_lex_state = 5},
  [2553] = {.lex_state = 11, .external_lex_state = 5},
  [2554] = {.lex_state = 11, .external_lex_state = 5},
  [2555] = {.lex_state = 11, .external_lex_state = 5},
  [2556] = {.lex_state = 11, .external_lex_state = 5},
  [2557] = {.lex_state = 11, .external_lex_state = 5},
  [2558] = {.lex_state = 11, .external_lex_state = 5},
  [2559] = {.lex_state = 11, .external_lex_state = 5},
  [2560] = {.lex_state = 11, .external_lex_state = 5},
  [2561] = {.lex_state = 11, .external_lex_state = 5},
  [2562] = {.lex_state = 11, .external_lex_state = 5},
  [2563] = {.lex_state = 11, .external_lex_state = 5},
  [2564] = {.lex_state = 11, .external_lex_state = 5},
  [2565] = {.lex_state = 11, .external_lex_state = 5},
  [2566] = {.lex_state = 11, .external_lex_state = 5},
  [2567] = {.lex_state = 11, .external_lex_state = 5},
  [2568] = {.lex_state = 11, .external_lex_state = 5},
  [2569] = {.lex_state = 11, .external_lex_state = 5},
  [2570] = {.lex_state = 11, .external_lex_state = 5},
  [2571] = {.lex_state = 11, .external_lex_state = 5},
  [2572] = {.lex_state = 11, .external_lex_state = 5},
  [2573] = {.lex_state = 11, .external_lex_state = 5},
  [2574] = {.lex_state = 11, .external_lex_state = 5},
  [2575] = {.lex_state = 11, .external_lex_state = 5},
  [2576] = {.lex_state = 11, .external_lex_state = 5},
  [2577] = {.lex_state = 11, .external_lex_state = 5},
  [2578] = {.lex_state = 11, .external_lex_state = 5},
  [2579] = {.lex_state = 11, .external_lex_state = 5},
  [2580] = {.lex_state = 11, .external_lex_state = 5},
  [2581] = {.lex_state = 11, .external_lex_state = 5},
  [2582] = {.lex_state = 11, .external_lex_state = 5},
  [2583] = {.lex_state = 11, .external_lex_state = 5},
  [2584] = {.lex_state = 11, .external_lex_state = 5},
  [2585] = {.lex_state = 11, .external_lex_state = 5},
  [2586] = {.lex_state = 11, .external_lex_state = 5},
  [2587] = {.lex_state = 11, .external_lex_state = 5},
  [2588] = {.lex_state = 11, .external_lex_state = 5},
  [2589] = {.lex_state = 11, .external_lex_state = 5},
  [2590] = {.lex_state = 11, .external_lex_state = 5},
  [2591] = {.lex_state = 11, .external_lex_state = 5},
  [2592] = {.lex_state = 11, .external_lex_state = 5},
  [2593] = {.lex_state = 11, .external_lex_state = 5},
  [2594] = {.lex_state = 11, .external_lex_state = 5},
  [2595] = {.lex_state = 11, .external_lex_state = 5},
  [2596] = {.lex_state = 11, .external_lex_state = 5},
  [2597] = {.lex_state = 11, .external_lex_state = 5},
  [2598] = {.lex_state = 11, .external_lex_state = 5},
  [2599] = {.lex_state = 11, .external_lex_state = 5},
  [2600] = {.lex_state = 11, .external_lex_state = 5},
  [2601] = {.lex_state = 11, .external_lex_state = 5},
  [2602] = {.lex_state = 11, .external_lex_state = 5},
  [2603] = {.lex_state = 11, .external_lex_state = 5},
  [2604] = {.lex_state = 11, .external_lex_state = 5},
  [2605] = {.lex_state = 11, .external_lex_state = 5},
  [2606] = {.lex_state = 11, .external_lex_state = 5},
  [2607] = {.lex_state = 11, .external_lex_state = 5},
  [2608] = {.lex_state = 11, .external_lex_state = 5},
  [2609] = {.lex_state = 11, .external_lex_state = 5},
  [2610] = {.lex_state = 11, .external_lex_state = 5},
  [2611] = {.lex_state = 11, .external_lex_state = 5},
  [2612] = {.lex_state = 11, .external_lex_state = 5},
  [2613] = {.lex_state = 11, .external_lex_state = 5},
  [2614] = {.lex_state = 11, .external_lex_state = 5},
  [2615] = {.lex_state = 11, .external_lex_state = 5},
  [2616] = {.lex_state = 11, .external_lex_state = 5},
  [2617] = {.lex_state = 11, .external_lex_state = 5},
  [2618] = {.lex_state = 11, .external_lex_state = 5},
  [2619] = {.lex_state = 11, .external_lex_state = 5},
  [2620] = {.lex_state = 11, .external_lex_state = 5},
  [2621] = {.lex_state = 11, .external_lex_state = 5},
  [2622] = {.lex_state = 11, .external_lex_state = 5},
  [2623] = {.lex_state = 11, .external_lex_state = 5},
  [2624] = {.lex_state = 11, .external_lex_state = 5},
  [2625] = {.lex_state = 11, .external_lex_state = 5},
  [2626] = {.lex_state = 11, .external_lex_state = 5},
  [2627] = {.lex_state = 11, .external_lex_state = 5},
  [2628] = {.lex_state = 11, .external_lex_state = 5},
  [2629] = {.lex_state = 11, .external_lex_state = 5},
  [2630] = {.lex_state = 11, .external_lex_state = 5},
  [2631] = {.lex_state = 11, .external_lex_state = 5},
  [2632] = {.lex_state = 11, .external_lex_state = 5},
  [2633] = {.lex_state = 11, .external_lex_state = 5},
  [2634] = {.lex_state = 11, .external_lex_state = 5},
  [2635] = {.lex_state = 11, .external_lex_state = 5},
  [2636] = {.lex_state = 11, .external_lex_state = 5},
  [2637] = {.lex_state = 11, .external_lex_state = 5},
  [2638] = {.lex_state = 11, .external_lex_state = 5},
  [2639] = {.lex_state = 11, .external_lex_state = 5},
  [2640] = {.lex_state = 11, .external_lex_state = 5},
  [2641] = {.lex_state = 11, .external_lex_state = 5},
  [2642] = {.lex_state = 11, .external_lex_state = 5},
  [2643] = {.lex_state = 11, .external_lex_state = 5},
  [2644] = {.lex_state = 11, .external_lex_state = 5},
  [2645] = {.lex_state = 11, .external_lex_state = 5},
  [2646] = {.lex_state = 11, .external_lex_state = 5},
  [2647] = {.lex_state = 11, .external_lex_state = 5},
  [2648] = {.lex_state = 11, .external_lex_state = 5},
  [2649] = {.lex_state = 11, .external_lex_state = 5},
  [2650] = {.lex_state = 11, .external_lex_state = 5},
  [2651] = {.lex_state = 11, .external_lex_state = 5},
  [2652] = {.lex_state = 11, .external_lex_state = 5},
  [2653] = {.lex_state = 11, .external_lex_state = 5},
  [2654] = {.lex_state = 11, .external_lex_state = 5},
  [2655] = {.lex_state = 11, .external_lex_state = 5},
  [2656] = {.lex_state = 11, .external_lex_state = 5},
  [2657] = {.lex_state = 11, .external_lex_state = 5},
  [2658] = {.lex_state = 11, .external_lex_state = 5},
  [2659] = {.lex_state = 11, .external_lex_state = 5},
  [2660] = {.lex_state = 11, .external_lex_state = 5},
  [2661] = {.lex_state = 11, .external_lex_state = 5},
  [2662] = {.lex_state = 11, .external_lex_state = 5},
  [2663] = {.lex_state = 11, .external_lex_state = 5},
  [2664] = {.lex_state = 11, .external_lex_state = 5},
  [2665] = {.lex_state = 11, .external_lex_state = 5},
  [2666] = {.lex_state = 11, .external_lex_state = 5},
  [2667] = {.lex_state = 11, .external_lex_state = 5},
  [2668] = {.lex_state = 11, .external_lex_state = 5},
  [2669] = {.lex_state = 11, .external_lex_state = 5},
  [2670] = {.lex_state = 11, .external_lex_state = 5},
  [2671] = {.lex_state = 11, .external_lex_state = 5},
  [2672] = {.lex_state = 11, .external_lex_state = 5},
  [2673] = {.lex_state = 11, .external_lex_state = 5},
  [2674] = {.lex_state = 11, .external_lex_state = 5},
  [2675] = {.lex_state = 11, .external_lex_state = 5},
  [2676] = {.lex_state = 11, .external_lex_state = 5},
  [2677] = {.lex_state = 11, .external_lex_state = 5},
  [2678] = {.lex_state = 11, .external_lex_state = 5},
  [2679] = {.lex_state = 11, .external_lex_state = 5},
  [2680] = {.lex_state = 11, .external_lex_state = 5},
  [2681] = {.lex_state = 11, .external_lex_state = 5},
  [2682] = {.lex_state = 11, .external_lex_state = 5},
  [2683] = {.lex_state = 11, .external_lex_state = 5},
  [2684] = {.lex_state = 11, .external_lex_state = 5},
  [2685] = {.lex_state = 11, .external_lex_state = 5},
  [2686] = {.lex_state = 11, .external_lex_state = 5},
  [2687] = {.lex_state = 11, .external_lex_state = 5},
  [2688] = {.lex_state = 11, .external_lex_state = 5},
  [2689] = {.lex_state = 11, .external_lex_state = 5},
  [2690] = {.lex_state = 11, .external_lex_state = 5},
  [2691] = {.lex_state = 11, .external_lex_state = 5},
  [2692] = {.lex_state = 11, .external_lex_state = 5},
  [2693] = {.lex_state = 11, .external_lex_state = 5},
  [2694] = {.lex_state = 11, .external_lex_state = 5},
  [2695] = {.lex_state = 11, .external_lex_state = 5},
  [2696] = {.lex_state = 11, .external_lex_state = 5},
  [2697] = {.lex_state = 11, .external_lex_state = 5},
  [2698] = {.lex_state = 11, .external_lex_state = 5},
  [2699] = {.lex_state = 11, .external_lex_state = 5},
  [2700] = {.lex_state = 11, .external_lex_state = 5},
  [2701] = {.lex_state = 11, .external_lex_state = 5},
  [2702] = {.lex_state = 11, .external_lex_state = 5},
  [2703] = {.lex_state = 11, .external_lex_state = 5},
  [2704] = {.lex_state = 11, .external_lex_state = 5},
  [2705] = {.lex_state = 11, .external_lex_state = 5},
  [2706] = {.lex_state = 11, .external_lex_state = 5},
  [2707] = {.lex_state = 11, .external_lex_state = 5},
  [2708] = {.lex_state = 11, .external_lex_state = 5},
  [2709] = {.lex_state = 11, .external_lex_state = 5},
  [2710] = {.lex_state = 11, .external_lex_state = 5},
  [2711] = {.lex_state = 11, .external_lex_state = 5},
  [2712] = {.lex_state = 11, .external_lex_state = 5},
  [2713] = {.lex_state = 11, .external_lex_state = 5},
  [2714] = {.lex_state = 11, .external_lex_state = 5},
  [2715] = {.lex_state = 11, .external_lex_state = 5},
  [2716] = {.lex_state = 11, .external_lex_state = 5},
  [2717] = {.lex_state = 11, .external_lex_state = 5},
  [2718] = {.lex_state = 11, .external_lex_state = 5},
  [2719] = {.lex_state = 11, .external_lex_state = 5},
  [2720] = {.lex_state = 11, .external_lex_state = 5},
  [2721] = {.lex_state = 11, .external_lex_state = 5},
  [2722] = {.lex_state = 11, .external_lex_state = 5},
  [2723] = {.lex_state = 11, .external_lex_state = 5},
  [2724] = {.lex_state = 11, .external_lex_state = 5},
  [2725] = {.lex_state = 11, .external_lex_state = 5},
  [2726] = {.lex_state = 11, .external_lex_state = 5},
  [2727] = {.lex_state = 11, .external_lex_state = 5},
  [2728] = {.lex_state = 11, .external_lex_state = 5},
  [2729] = {.lex_state = 11, .external_lex_state = 5},
  [2730] = {.lex_state = 11, .external_lex_state = 5},
  [2731] = {.lex_state = 11, .external_lex_state = 5},
  [2732] = {.lex_state = 11, .external_lex_state = 5},
  [2733] = {.lex_state = 11, .external_lex_state = 5},
  [2734] = {.lex_state = 11, .external_lex_state = 5},
  [2735] = {.lex_state = 11, .external_lex_state = 5},
  [2736] = {.lex_state = 11, .external_lex_state = 5},
  [2737] = {.lex_state = 11, .external_lex_state = 5},
  [2738] = {.lex_state = 11, .external_lex_state = 5},
  [2739] = {.lex_state = 11, .external_lex_state = 5},
  [2740] = {.lex_state = 11, .external_lex_state = 5},
  [2741] = {.lex_state = 11, .external_lex_state = 5},
  [2742] = {.lex_state = 11, .external_lex_state = 5},
  [2743] = {.lex_state = 11, .external_lex_state = 5},
  [2744] = {.lex_state = 11, .external_lex_state = 5},
  [2745] = {.lex_state = 11, .external_lex_state = 5},
  [2746] = {.lex_state = 11, .external_lex_state = 5},
  [2747] = {.lex_state = 73, .external_lex_state = 4},
  [2748] = {.lex_state = 11, .external_lex_state = 5},
  [2749] = {.lex_state = 11, .external_lex_state = 5},
  [2750] = {.lex_state = 11, .external_lex_state = 5},
  [2751] = {.lex_state = 11, .external_lex_state = 5},
  [2752] = {.lex_state = 73, .external_lex_state = 4},
  [2753] = {.lex_state = 11, .external_lex_state = 5},
  [2754] = {.lex_state = 73, .external_lex_state = 4},
  [2755] = {.lex_state = 11},
  [2756] = {.lex_state = 11},
  [2757] = {.lex_state = 11},
  [2758] = {.lex_state = 11},
  [2759] = {.lex_state = 11},
  [2760] = {.lex_state = 11},
  [2761] = {.lex_state = 11},
  [2762] = {.lex_state = 11},
  [2763] = {.lex_state = 11},
  [2764] = {.lex_state = 8},
  [2765] = {.lex_state = 72, .external_lex_state = 4},
  [2766] = {.lex_state = 7},
  [2767] = {.lex_state = 8},
  [2768] = {.lex_state = 72, .external_lex_state = 4},
  [2769] = {.lex_state = 7},
  [2770] = {.lex_state = 72, .external_lex_state = 4},
  [2771] = {.lex_state = 7},
  [2772] = {.lex_state = 72, .external_lex_state = 4},
  [2773] = {.lex_state = 72, .external_lex_state = 4},
  [2774] = {.lex_state = 8},
  [2775] = {.lex_state = 72, .external_lex_state = 4},
  [2776] = {.lex_state = 11},
  [2777] = {.lex_state = 11},
  [2778] = {.lex_state = 11},
  [2779] = {.lex_state = 8},
  [2780] = {.lex_state = 7},
  [2781] = {.lex_state = 11},
  [2782] = {.lex_state = 8},
  [2783] = {.lex_state = 11},
  [2784] = {.lex_state = 11},
  [2785] = {.lex_state = 11},
  [2786] = {.lex_state = 7},
  [2787] = {.lex_state = 7},
  [2788] = {.lex_state = 7},
  [2789] = {.lex_state = 7},
  [2790] = {.lex_state = 7},
  [2791] = {.lex_state = 7},
  [2792] = {.lex_state = 7},
  [2793] = {.lex_state = 7},
  [2794] = {.lex_state = 7},
  [2795] = {.lex_state = 8},
  [2796] = {.lex_state = 8},
  [2797] = {.lex_state = 8},
  [2798] = {.lex_state = 8},
  [2799] = {.lex_state = 8},
  [2800] = {.lex_state = 8},
  [2801] = {.lex_state = 8},
  [2802] = {.lex_state = 7},
  [2803] = {.lex_state = 8},
  [2804] = {.lex_state = 8},
  [2805] = {.lex_state = 19},
  [2806] = {.lex_state = 11},
  [2807] = {.lex_state = 8},
  [2808] = {.lex_state = 11},
  [2809] = {.lex_state = 8},
  [2810] = {.lex_state = 8},
  [2811] = {.lex_state = 8},
  [2812] = {.lex_state = 19},
  [2813] = {.lex_state = 8},
  [2814] = {.lex_state = 11},
  [2815] = {.lex_state = 8},
  [2816] = {.lex_state = 8},
  [2817] = {.lex_state = 8},
  [2818] = {.lex_state = 8},
  [2819] = {.lex_state = 19},
  [2820] = {.lex_state = 8},
  [2821] = {.lex_state = 8},
  [2822] = {.lex_state = 8},
  [2823] = {.lex_state = 19},
  [2824] = {.lex_state = 11},
  [2825] = {.lex_state = 8},
  [2826] = {.lex_state = 8},
  [2827] = {.lex_state = 19},
  [2828] = {.lex_state = 8},
  [2829] = {.lex_state = 11},
  [2830] = {.lex_state = 11},
  [2831] = {.lex_state = 11},
  [2832] = {.lex_state = 19},
  [2833] = {.lex_state = 8},
  [2834] = {.lex_state = 8},
  [2835] = {.lex_state = 8},
  [2836] = {.lex_state = 8},
  [2837] = {.lex_state = 8},
  [2838] = {.lex_state = 11},
  [2839] = {.lex_state = 8},
  [2840] = {.lex_state = 8},
  [2841] = {.lex_state = 8},
  [2842] = {.lex_state = 11},
  [2843] = {.lex_state = 8},
  [2844] = {.lex_state = 8},
  [2845] = {.lex_state = 20},
  [2846] = {.lex_state = 8},
  [2847] = {.lex_state = 20},
  [2848] = {.lex_state = 19},
  [2849] = {.lex_state = 8},
  [2850] = {.lex_state = 11},
  [2851] = {.lex_state = 11},
  [2852] = {.lex_state = 11},
  [2853] = {.lex_state = 11},
  [2854] = {.lex_state = 8},
  [2855] = {.lex_state = 11},
  [2856] = {.lex_state = 11},
  [2857] = {.lex_state = 11},
  [2858] = {.lex_state = 11},
  [2859] = {.lex_state = 19},
  [2860] = {.lex_state = 8},
  [2861] = {.lex_state = 8},
  [2862] = {.lex_state = 8},
  [2863] = {.lex_state = 8},
  [2864] = {.lex_state = 8},
  [2865] = {.lex_state = 8},
  [2866] = {.lex_state = 9},
  [2867] = {.lex_state = 8},
  [2868] = {.lex_state = 8},
  [2869] = {.lex_state = 8},
  [2870] = {.lex_state = 8},
  [2871] = {.lex_state = 8},
  [2872] = {.lex_state = 8},
  [2873] = {.lex_state = 8},
  [2874] = {.lex_state = 8},
  [2875] = {.lex_state = 8},
  [2876] = {.lex_state = 8},
  [2877] = {.lex_state = 8},
  [2878] = {.lex_state = 8},
  [2879] = {.lex_state = 8},
  [2880] = {.lex_state = 8},
  [2881] = {.lex_state = 8},
  [2882] = {.lex_state = 8},
  [2883] = {.lex_state = 8},
  [2884] = {.lex_state = 8},
  [2885] = {.lex_state = 8},
  [2886] = {.lex_state = 8},
  [2887] = {.lex_state = 8},
  [2888] = {.lex_state = 8},
  [2889] = {.lex_state = 8},
  [2890] = {.lex_state = 8},
  [2891] = {.lex_state = 8},
  [2892] = {.lex_state = 8},
  [2893] = {.lex_state = 8},
  [2894] = {.lex_state = 8},
  [2895] = {.lex_state = 8},
  [2896] = {.lex_state = 8},
  [2897] = {.lex_state = 8},
  [2898] = {.lex_state = 8},
  [2899] = {.lex_state = 8},
  [2900] = {.lex_state = 8},
  [2901] = {.lex_state = 8},
  [2902] = {.lex_state = 11},
  [2903] = {.lex_state = 8},
  [2904] = {.lex_state = 8},
  [2905] = {.lex_state = 8},
  [2906] = {.lex_state = 8},
  [2907] = {.lex_state = 8},
  [2908] = {.lex_state = 8},
  [2909] = {.lex_state = 8},
  [2910] = {.lex_state = 8},
  [2911] = {.lex_state = 8},
  [2912] = {.lex_state = 8},
  [2913] = {.lex_state = 8},
  [2914] = {.lex_state = 8},
  [2915] = {.lex_state = 8},
  [2916] = {.lex_state = 8},
  [2917] = {.lex_state = 8},
  [2918] = {.lex_state = 8},
  [2919] = {.lex_state = 8},
  [2920] = {.lex_state = 8},
  [2921] = {.lex_state = 8},
  [2922] = {.lex_state = 8},
  [2923] = {.lex_state = 8},
  [2924] = {.lex_state = 8},
  [2925] = {.lex_state = 8},
  [2926] = {.lex_state = 8},
  [2927] = {.lex_state = 8},
  [2928] = {.lex_state = 8},
  [2929] = {.lex_state = 8},
  [2930] = {.lex_state = 8},
  [2931] = {.lex_state = 8},
  [2932] = {.lex_state = 8},
  [2933] = {.lex_state = 8},
  [2934] = {.lex_state = 8},
  [2935] = {.lex_state = 8},
  [2936] = {.lex_state = 8},
  [2937] = {.lex_state = 8},
  [2938] = {.lex_state = 8},
  [2939] = {.lex_state = 8},
  [2940] = {.lex_state = 8},
  [2941] = {.lex_state = 8},
  [2942] = {.lex_state = 8},
  [2943] = {.lex_state = 8},
  [2944] = {.lex_state = 8},
  [2945] = {.lex_state = 8},
  [2946] = {.lex_state = 8},
  [2947] = {.lex_state = 8},
  [2948] = {.lex_state = 8},
  [2949] = {.lex_state = 8},
  [2950] = {.lex_state = 8},
  [2951] = {.lex_state = 8},
  [2952] = {.lex_state = 8},
  [2953] = {.lex_state = 8},
  [2954] = {.lex_state = 8},
  [2955] = {.lex_state = 8},
  [2956] = {.lex_state = 8},
  [2957] = {.lex_state = 8},
  [2958] = {.lex_state = 8},
  [2959] = {.lex_state = 8},
  [2960] = {.lex_state = 8},
  [2961] = {.lex_state = 8},
  [2962] = {.lex_state = 8},
  [2963] = {.lex_state = 11},
  [2964] = {.lex_state = 8},
  [2965] = {.lex_state = 8},
  [2966] = {.lex_state = 8},
  [2967] = {.lex_state = 8},
  [2968] = {.lex_state = 8},
  [2969] = {.lex_state = 8},
  [2970] = {.lex_state = 8},
  [2971] = {.lex_state = 8},
  [2972] = {.lex_state = 8},
  [2973] = {.lex_state = 8},
  [2974] = {.lex_state = 8},
  [2975] = {.lex_state = 8},
  [2976] = {.lex_state = 8},
  [2977] = {.lex_state = 8},
  [2978] = {.lex_state = 8},
  [2979] = {.lex_state = 8},
  [2980] = {.lex_state = 8},
  [2981] = {.lex_state = 8},
  [2982] = {.lex_state = 8},
  [2983] = {.lex_state = 8},
  [2984] = {.lex_state = 8},
  [2985] = {.lex_state = 8},
  [2986] = {.lex_state = 8},
  [2987] = {.lex_state = 8},
  [2988] = {.lex_state = 11},
  [2989] = {.lex_state = 11},
  [2990] = {.lex_state = 11},
  [2991] = {.lex_state = 8},
  [2992] = {.lex_state = 11},
  [2993] = {.lex_state = 11},
  [2994] = {.lex_state = 11},
  [2995] = {.lex_state = 8},
  [2996] = {.lex_state = 11},
  [2997] = {.lex_state = 11},
  [2998] = {.lex_state = 11},
  [2999] = {.lex_state = 11},
  [3000] = {.lex_state = 11},
  [3001] = {.lex_state = 11},
  [3002] = {.lex_state = 11},
  [3003] = {.lex_state = 11},
  [3004] = {.lex_state = 11},
  [3005] = {.lex_state = 9},
  [3006] = {.lex_state = 8},
  [3007] = {.lex_state = 11},
  [3008] = {.lex_state = 11},
  [3009] = {.lex_state = 8},
  [3010] = {.lex_state = 11},
  [3011] = {.lex_state = 11},
  [3012] = {.lex_state = 11},
  [3013] = {.lex_state = 11},
  [3014] = {.lex_state = 11},
  [3015] = {.lex_state = 11},
  [3016] = {.lex_state = 11},
  [3017] = {.lex_state = 11},
  [3018] = {.lex_state = 8},
  [3019] = {.lex_state = 8},
  [3020] = {.lex_state = 8},
  [3021] = {.lex_state = 8},
  [3022] = {.lex_state = 8},
  [3023] = {.lex_state = 8},
  [3024] = {.lex_state = 8},
  [3025] = {.lex_state = 8},
  [3026] = {.lex_state = 8},
  [3027] = {.lex_state = 8},
  [3028] = {.lex_state = 8},
  [3029] = {.lex_state = 8},
  [3030] = {.lex_state = 8},
  [3031] = {.lex_state = 10},
  [3032] = {.lex_state = 10},
  [3033] = {.lex_state = 8},
  [3034] = {.lex_state = 8},
  [3035] = {.lex_state = 10},
  [3036] = {.lex_state = 8},
  [3037] = {.lex_state = 8},
  [3038] = {.lex_state = 8},
  [3039] = {.lex_state = 8},
  [3040] = {.lex_state = 8},
  [3041] = {.lex_state = 8},
  [3042] = {.lex_state = 8},
  [3043] = {.lex_state = 8},
  [3044] = {.lex_state = 8},
  [3045] = {.lex_state = 8},
  [3046] = {.lex_state = 8},
  [3047] = {.lex_state = 8},
  [3048] = {.lex_state = 8},
  [3049] = {.lex_state = 8},
  [3050] = {.lex_state = 10},
  [3051] = {.lex_state = 9},
  [3052] = {.lex_state = 8},
  [3053] = {.lex_state = 9},
  [3054] = {.lex_state = 8},
  [3055] = {.lex_state = 8},
  [3056] = {.lex_state = 8},
  [3057] = {.lex_state = 8},
  [3058] = {.lex_state = 10},
  [3059] = {.lex_state = 8},
  [3060] = {.lex_state = 8},
  [3061] = {.lex_state = 9},
  [3062] = {.lex_state = 10},
  [3063] = {.lex_state = 10},
  [3064] = {.lex_state = 10},
  [3065] = {.lex_state = 8},
  [3066] = {.lex_state = 8},
  [3067] = {.lex_state = 10},
  [3068] = {.lex_state = 9},
  [3069] = {.lex_state = 8},
  [3070] = {.lex_state = 10},
  [3071] = {.lex_state = 9},
  [3072] = {.lex_state = 8},
  [3073] = {.lex_state = 8},
  [3074] = {.lex_state = 10},
  [3075] = {.lex_state = 9},
  [3076] = {.lex_state = 10},
  [3077] = {.lex_state = 9},
  [3078] = {.lex_state = 11},
  [3079] = {.lex_state = 11},
  [3080] = {.lex_state = 9},
  [3081] = {.lex_state = 9},
  [3082] = {.lex_state = 9},
  [3083] = {.lex_state = 9},
  [3084] = {.lex_state = 8},
  [3085] = {.lex_state = 8},
  [3086] = {.lex_state = 8},
  [3087] = {.lex_state = 10},
  [3088] = {.lex_state = 8},
  [3089] = {.lex_state = 8},
  [3090] = {.lex_state = 8},
  [3091] = {.lex_state = 8},
  [3092] = {.lex_state = 8},
  [3093] = {.lex_state = 8},
  [3094] = {.lex_state = 8},
  [3095] = {.lex_state = 11},
  [3096] = {.lex_state = 9},
  [3097] = {.lex_state = 9},
  [3098] = {.lex_state = 8},
  [3099] = {.lex_state = 8},
  [3100] = {.lex_state = 9},
  [3101] = {.lex_state = 8},
  [3102] = {.lex_state = 9},
  [3103] = {.lex_state = 9},
  [3104] = {.lex_state = 11},
  [3105] = {.lex_state = 9},
  [3106] = {.lex_state = 9},
  [3107] = {.lex_state = 11},
  [3108] = {.lex_state = 9},
  [3109] = {.lex_state = 11},
  [3110] = {.lex_state = 8},
  [3111] = {.lex_state = 11},
  [3112] = {.lex_state = 8},
  [3113] = {.lex_state = 9},
  [3114] = {.lex_state = 11},
  [3115] = {.lex_state = 11},
  [3116] = {.lex_state = 8},
  [3117] = {.lex_state = 11},
  [3118] = {.lex_state = 11},
  [3119] = {.lex_state = 8},
  [3120] = {.lex_state = 11},
  [3121] = {.lex_state = 11},
  [3122] = {.lex_state = 11},
  [3123] = {.lex_state = 9},
  [3124] = {.lex_state = 11},
  [3125] = {.lex_state = 11},
  [3126] = {.lex_state = 8},
  [3127] = {.lex_state = 21},
  [3128] = {.lex_state = 8},
  [3129] = {.lex_state = 9},
  [3130] = {.lex_state = 9},
  [3131] = {.lex_state = 8},
  [3132] = {.lex_state = 9},
  [3133] = {.lex_state = 8},
  [3134] = {.lex_state = 21},
  [3135] = {.lex_state = 9},
  [3136] = {.lex_state = 21},
  [3137] = {.lex_state = 8},
  [3138] = {.lex_state = 8},
  [3139] = {.lex_state = 8},
  [3140] = {.lex_state = 9},
  [3141] = {.lex_state = 8},
  [3142] = {.lex_state = 9},
  [3143] = {.lex_state = 8},
  [3144] = {.lex_state = 9},
  [3145] = {.lex_state = 9},
  [3146] = {.lex_state = 10},
  [3147] = {.lex_state = 8},
  [3148] = {.lex_state = 8},
  [3149] = {.lex_state = 21},
  [3150] = {.lex_state = 21},
  [3151] = {.lex_state = 9},
  [3152] = {.lex_state = 8},
  [3153] = {.lex_state = 8},
  [3154] = {.lex_state = 8},
  [3155] = {.lex_state = 8},
  [3156] = {.lex_state = 21},
  [3157] = {.lex_state = 9},
  [3158] = {.lex_state = 8},
  [3159] = {.lex_state = 9},
  [3160] = {.lex_state = 8},
  [3161] = {.lex_state = 9},
  [3162] = {.lex_state = 21},
  [3163] = {.lex_state = 9},
  [3164] = {.lex_state = 8},
  [3165] = {.lex_state = 9},
  [3166] = {.lex_state = 8},
  [3167] = {.lex_state = 9},
  [3168] = {.lex_state = 9},
  [3169] = {.lex_state = 21},
  [3170] = {.lex_state = 8},
  [3171] = {.lex_state = 9},
  [3172] = {.lex_state = 8},
  [3173] = {.lex_state = 8},
  [3174] = {.lex_state = 8},
  [3175] = {.lex_state = 9},
  [3176] = {.lex_state = 9},
  [3177] = {.lex_state = 9},
  [3178] = {.lex_state = 9},
  [3179] = {.lex_state = 9},
  [3180] = {.lex_state = 8},
  [3181] = {.lex_state = 8},
  [3182] = {.lex_state = 9},
  [3183] = {.lex_state = 8},
  [3184] = {.lex_state = 9},
  [3185] = {.lex_state = 9},
  [3186] = {.lex_state = 9},
  [3187] = {.lex_state = 9},
  [3188] = {.lex_state = 9},
  [3189] = {.lex_state = 8},
  [3190] = {.lex_state = 8},
  [3191] = {.lex_state = 9},
  [3192] = {.lex_state = 8},
  [3193] = {.lex_state = 9},
  [3194] = {.lex_state = 9},
  [3195] = {.lex_state = 9},
  [3196] = {.lex_state = 8},
  [3197] = {.lex_state = 8},
  [3198] = {.lex_state = 9},
  [3199] = {.lex_state = 8},
  [3200] = {.lex_state = 9},
  [3201] = {.lex_state = 8},
  [3202] = {.lex_state = 8},
  [3203] = {.lex_state = 9},
  [3204] = {.lex_state = 8},
  [3205] = {.lex_state = 9},
  [3206] = {.lex_state = 8},
  [3207] = {.lex_state = 8},
  [3208] = {.lex_state = 8},
  [3209] = {.lex_state = 9},
  [3210] = {.lex_state = 9},
  [3211] = {.lex_state = 8},
  [3212] = {.lex_state = 9},
  [3213] = {.lex_state = 9},
  [3214] = {.lex_state = 11},
  [3215] = {.lex_state = 8},
  [3216] = {.lex_state = 8},
  [3217] = {.lex_state = 9},
  [3218] = {.lex_state = 9},
  [3219] = {.lex_state = 8},
  [3220] = {.lex_state = 8},
  [3221] = {.lex_state = 8},
  [3222] = {.lex_state = 8},
  [3223] = {.lex_state = 8},
  [3224] = {.lex_state = 8},
  [3225] = {.lex_state = 8},
  [3226] = {.lex_state = 9},
  [3227] = {.lex_state = 9},
  [3228] = {.lex_state = 9},
  [3229] = {.lex_state = 8},
  [3230] = {.lex_state = 9},
  [3231] = {.lex_state = 9},
  [3232] = {.lex_state = 9},
  [3233] = {.lex_state = 9},
  [3234] = {.lex_state = 8},
  [3235] = {.lex_state = 9},
  [3236] = {.lex_state = 9},
  [3237] = {.lex_state = 9},
  [3238] = {.lex_state = 9},
  [3239] = {.lex_state = 8},
  [3240] = {.lex_state = 9},
  [3241] = {.lex_state = 9},
  [3242] = {.lex_state = 8},
  [3243] = {.lex_state = 9},
  [3244] = {.lex_state = 9},
  [3245] = {.lex_state = 9},
  [3246] = {.lex_state = 9},
  [3247] = {.lex_state = 8},
  [3248] = {.lex_state = 8},
  [3249] = {.lex_state = 8},
  [3250] = {.lex_state = 9},
  [3251] = {.lex_state = 9},
  [3252] = {.lex_state = 9},
  [3253] = {.lex_state = 8},
  [3254] = {.lex_state = 8},
  [3255] = {.lex_state = 8},
  [3256] = {.lex_state = 8},
  [3257] = {.lex_state = 9},
  [3258] = {.lex_state = 8},
  [3259] = {.lex_state = 8},
  [3260] = {.lex_state = 8},
  [3261] = {.lex_state = 8},
  [3262] = {.lex_state = 9},
  [3263] = {.lex_state = 9},
  [3264] = {.lex_state = 9},
  [3265] = {.lex_state = 9},
  [3266] = {.lex_state = 8},
  [3267] = {.lex_state = 9},
  [3268] = {.lex_state = 8},
  [3269] = {.lex_state = 8},
  [3270] = {.lex_state = 8},
  [3271] = {.lex_state = 8},
  [3272] = {.lex_state = 8},
  [3273] = {.lex_state = 9},
  [3274] = {.lex_state = 8},
  [3275] = {.lex_state = 9},
  [3276] = {.lex_state = 8},
  [3277] = {.lex_state = 8},
  [3278] = {.lex_state = 9},
  [3279] = {.lex_state = 9},
  [3280] = {.lex_state = 8},
  [3281] = {.lex_state = 8},
  [3282] = {.lex_state = 8},
  [3283] = {.lex_state = 9},
  [3284] = {.lex_state = 8},
  [3285] = {.lex_state = 8},
  [3286] = {.lex_state = 9},
  [3287] = {.lex_state = 8},
  [3288] = {.lex_state = 9},
  [3289] = {.lex_state = 11},
  [3290] = {.lex_state = 8},
  [3291] = {.lex_state = 8},
  [3292] = {.lex_state = 9},
  [3293] = {.lex_state = 9},
  [3294] = {.lex_state = 9},
  [3295] = {.lex_state = 11},
  [3296] = {.lex_state = 11},
  [3297] = {.lex_state = 9},
  [3298] = {.lex_state = 9},
  [3299] = {.lex_state = 9},
  [3300] = {.lex_state = 8},
  [3301] = {.lex_state = 9},
  [3302] = {.lex_state = 9},
  [3303] = {.lex_state = 9},
  [3304] = {.lex_state = 9},
  [3305] = {.lex_state = 9},
  [3306] = {.lex_state = 9},
  [3307] = {.lex_state = 9},
  [3308] = {.lex_state = 9},
  [3309] = {.lex_state = 9},
  [3310] = {.lex_state = 9},
  [3311] = {.lex_state = 9},
  [3312] = {.lex_state = 8},
  [3313] = {.lex_state = 8},
  [3314] = {.lex_state = 8},
  [3315] = {.lex_state = 8},
  [3316] = {.lex_state = 9},
  [3317] = {.lex_state = 11},
  [3318] = {.lex_state = 9},
  [3319] = {.lex_state = 9},
  [3320] = {.lex_state = 8},
  [3321] = {.lex_state = 9},
  [3322] = {.lex_state = 11},
  [3323] = {.lex_state = 9},
  [3324] = {.lex_state = 8},
  [3325] = {.lex_state = 8},
  [3326] = {.lex_state = 9},
  [3327] = {.lex_state = 9},
  [3328] = {.lex_state = 9},
  [3329] = {.lex_state = 8},
  [3330] = {.lex_state = 9},
  [3331] = {.lex_state = 9},
  [3332] = {.lex_state = 8},
  [3333] = {.lex_state = 8},
  [3334] = {.lex_state = 8},
  [3335] = {.lex_state = 8},
  [3336] = {.lex_state = 9},
  [3337] = {.lex_state = 9},
  [3338] = {.lex_state = 9},
  [3339] = {.lex_state = 9},
  [3340] = {.lex_state = 9},
  [3341] = {.lex_state = 9},
  [3342] = {.lex_state = 9},
  [3343] = {.lex_state = 9},
  [3344] = {.lex_state = 9},
  [3345] = {.lex_state = 9},
  [3346] = {.lex_state = 9},
  [3347] = {.lex_state = 9},
  [3348] = {.lex_state = 9},
  [3349] = {.lex_state = 9},
  [3350] = {.lex_state = 8},
  [3351] = {.lex_state = 9},
  [3352] = {.lex_state = 9},
  [3353] = {.lex_state = 9},
  [3354] = {.lex_state = 9},
  [3355] = {.lex_state = 9},
  [3356] = {.lex_state = 8},
  [3357] = {.lex_state = 9},
  [3358] = {.lex_state = 8},
  [3359] = {.lex_state = 9},
  [3360] = {.lex_state = 8},
  [3361] = {.lex_state = 8},
  [3362] = {.lex_state = 9},
  [3363] = {.lex_state = 9},
  [3364] = {.lex_state = 9},
  [3365] = {.lex_state = 9},
  [3366] = {.lex_state = 9},
  [3367] = {.lex_state = 9},
  [3368] = {.lex_state = 8},
  [3369] = {.lex_state = 9},
  [3370] = {.lex_state = 9},
  [3371] = {.lex_state = 9},
  [3372] = {.lex_state = 9},
  [3373] = {.lex_state = 9},
  [3374] = {.lex_state = 9},
  [3375] = {.lex_state = 8},
  [3376] = {.lex_state = 9},
  [3377] = {.lex_state = 9},
  [3378] = {.lex_state = 9},
  [3379] = {.lex_state = 9},
  [3380] = {.lex_state = 9},
  [3381] = {.lex_state = 9},
  [3382] = {.lex_state = 8},
  [3383] = {.lex_state = 8},
  [3384] = {.lex_state = 8},
  [3385] = {.lex_state = 8},
  [3386] = {.lex_state = 8},
  [3387] = {.lex_state = 8},
  [3388] = {.lex_state = 9},
  [3389] = {.lex_state = 8},
  [3390] = {.lex_state = 8},
  [3391] = {.lex_state = 8},
  [3392] = {.lex_state = 8},
  [3393] = {.lex_state = 8},
  [3394] = {.lex_state = 8},
  [3395] = {.lex_state = 8},
  [3396] = {.lex_state = 8},
  [3397] = {.lex_state = 8},
  [3398] = {.lex_state = 8},
  [3399] = {.lex_state = 8},
  [3400] = {.lex_state = 8},
  [3401] = {.lex_state = 8},
  [3402] = {.lex_state = 9},
  [3403] = {.lex_state = 9},
  [3404] = {.lex_state = 8},
  [3405] = {.lex_state = 8},
  [3406] = {.lex_state = 9},
  [3407] = {.lex_state = 9},
  [3408] = {.lex_state = 9},
  [3409] = {.lex_state = 9},
  [3410] = {.lex_state = 11},
  [3411] = {.lex_state = 11},
  [3412] = {.lex_state = 9},
  [3413] = {.lex_state = 11},
  [3414] = {.lex_state = 9},
  [3415] = {.lex_state = 9},
  [3416] = {.lex_state = 8},
  [3417] = {.lex_state = 11},
  [3418] = {.lex_state = 9},
  [3419] = {.lex_state = 8},
  [3420] = {.lex_state = 9},
  [3421] = {.lex_state = 11},
  [3422] = {.lex_state = 11},
  [3423] = {.lex_state = 9},
  [3424] = {.lex_state = 9},
  [3425] = {.lex_state = 8},
  [3426] = {.lex_state = 9},
  [3427] = {.lex_state = 8},
  [3428] = {.lex_state = 8},
  [3429] = {.lex_state = 9},
  [3430] = {.lex_state = 9},
  [3431] = {.lex_state = 9},
  [3432] = {.lex_state = 9},
  [3433] = {.lex_state = 8},
  [3434] = {.lex_state = 8},
  [3435] = {.lex_state = 9},
  [3436] = {.lex_state = 9},
  [3437] = {.lex_state = 9},
  [3438] = {.lex_state = 8},
  [3439] = {.lex_state = 8},
  [3440] = {.lex_state = 11},
  [3441] = {.lex_state = 8},
  [3442] = {.lex_state = 8},
  [3443] = {.lex_state = 8},
  [3444] = {.lex_state = 8},
  [3445] = {.lex_state = 8},
  [3446] = {.lex_state = 8},
  [3447] = {.lex_state = 8},
  [3448] = {.lex_state = 8},
  [3449] = {.lex_state = 9},
  [3450] = {.lex_state = 11},
  [3451] = {.lex_state = 8},
  [3452] = {.lex_state = 8},
  [3453] = {.lex_state = 8},
  [3454] = {.lex_state = 8},
  [3455] = {.lex_state = 8},
  [3456] = {.lex_state = 8},
  [3457] = {.lex_state = 8},
  [3458] = {.lex_state = 9},
  [3459] = {.lex_state = 8},
  [3460] = {.lex_state = 8},
  [3461] = {.lex_state = 8},
  [3462] = {.lex_state = 8},
  [3463] = {.lex_state = 8},
  [3464] = {.lex_state = 8},
  [3465] = {.lex_state = 8},
  [3466] = {.lex_state = 8},
  [3467] = {.lex_state = 8},
  [3468] = {.lex_state = 8},
  [3469] = {.lex_state = 8},
  [3470] = {.lex_state = 8},
  [3471] = {.lex_state = 8},
  [3472] = {.lex_state = 8},
  [3473] = {.lex_state = 8},
  [3474] = {.lex_state = 8},
  [3475] = {.lex_state = 8},
  [3476] = {.lex_state = 8},
  [3477] = {.lex_state = 8},
  [3478] = {.lex_state = 8},
  [3479] = {.lex_state = 8},
  [3480] = {.lex_state = 9},
  [3481] = {.lex_state = 8},
  [3482] = {.lex_state = 8},
  [3483] = {.lex_state = 8},
  [3484] = {.lex_state = 8},
  [3485] = {.lex_state = 8},
  [3486] = {.lex_state = 8},
  [3487] = {.lex_state = 8},
  [3488] = {.lex_state = 8},
  [3489] = {.lex_state = 8},
  [3490] = {.lex_state = 8},
  [3491] = {.lex_state = 8},
  [3492] = {.lex_state = 8},
  [3493] = {.lex_state = 8},
  [3494] = {.lex_state = 8},
  [3495] = {.lex_state = 8},
  [3496] = {.lex_state = 8},
  [3497] = {.lex_state = 11},
  [3498] = {.lex_state = 11},
  [3499] = {.lex_state = 11},
  [3500] = {.lex_state = 11},
  [3501] = {.lex_state = 11},
  [3502] = {.lex_state = 11},
  [3503] = {.lex_state = 17},
  [3504] = {.lex_state = 17},
  [3505] = {.lex_state = 17},
  [3506] = {.lex_state = 17},
  [3507] = {.lex_state = 17},
  [3508] = {.lex_state = 17},
  [3509] = {.lex_state = 11},
  [3510] = {.lex_state = 11},
  [3511] = {.lex_state = 11},
  [3512] = {.lex_state = 17},
  [3513] = {.lex_state = 17},
  [3514] = {.lex_state = 11},
  [3515] = {.lex_state = 11},
  [3516] = {.lex_state = 11},
  [3517] = {.lex_state = 11},
  [3518] = {.lex_state = 17},
  [3519] = {.lex_state = 17},
  [3520] = {.lex_state = 17},
  [3521] = {.lex_state = 17},
  [3522] = {.lex_state = 11},
  [3523] = {.lex_state = 11},
  [3524] = {.lex_state = 17},
  [3525] = {.lex_state = 11},
  [3526] = {.lex_state = 11, .external_lex_state = 5},
  [3527] = {.lex_state = 11, .external_lex_state = 5},
  [3528] = {.lex_state = 11, .external_lex_state = 5},
  [3529] = {.lex_state = 11, .external_lex_state = 5},
  [3530] = {.lex_state = 11, .external_lex_state = 5},
  [3531] = {.lex_state = 17},
  [3532] = {.lex_state = 11, .external_lex_state = 5},
  [3533] = {.lex_state = 11},
  [3534] = {.lex_state = 11},
  [3535] = {.lex_state = 17},
  [3536] = {.lex_state = 11, .external_lex_state = 5},
  [3537] = {.lex_state = 72, .external_lex_state = 4},
  [3538] = {.lex_state = 11, .external_lex_state = 5},
  [3539] = {.lex_state = 23},
  [3540] = {.lex_state = 11, .external_lex_state = 5},
  [3541] = {.lex_state = 11},
  [3542] = {.lex_state = 11, .external_lex_state = 5},
  [3543] = {.lex_state = 11, .external_lex_state = 5},
  [3544] = {.lex_state = 11},
  [3545] = {.lex_state = 17},
  [3546] = {.lex_state = 17},
  [3547] = {.lex_state = 23, .external_lex_state = 5},
  [3548] = {.lex_state = 23, .external_lex_state = 5},
  [3549] = {.lex_state = 17},
  [3550] = {.lex_state = 17},
  [3551] = {.lex_state = 17},
  [3552] = {.lex_state = 17},
  [3553] = {.lex_state = 17},
  [3554] = {.lex_state = 23, .external_lex_state = 5},
  [3555] = {.lex_state = 11},
  [3556] = {.lex_state = 11},
  [3557] = {.lex_state = 23, .external_lex_state = 5},
  [3558] = {.lex_state = 11},
  [3559] = {.lex_state = 11, .external_lex_state = 5},
  [3560] = {.lex_state = 23, .external_lex_state = 5},
  [3561] = {.lex_state = 23},
  [3562] = {.lex_state = 23},
  [3563] = {.lex_state = 23, .external_lex_state = 5},
  [3564] = {.lex_state = 23, .external_lex_state = 5},
  [3565] = {.lex_state = 23},
  [3566] = {.lex_state = 23},
  [3567] = {.lex_state = 11},
  [3568] = {.lex_state = 23, .external_lex_state = 5},
  [3569] = {.lex_state = 17},
  [3570] = {.lex_state = 11},
  [3571] = {.lex_state = 23},
  [3572] = {.lex_state = 23},
  [3573] = {.lex_state = 23, .external_lex_state = 5},
  [3574] = {.lex_state = 23, .external_lex_state = 5},
  [3575] = {.lex_state = 11},
  [3576] = {.lex_state = 17},
  [3577] = {.lex_state = 17},
  [3578] = {.lex_state = 11},
  [3579] = {.lex_state = 11},
  [3580] = {.lex_state = 23},
  [3581] = {.lex_state = 23, .external_lex_state = 5},
  [3582] = {.lex_state = 23},
  [3583] = {.lex_state = 11},
  [3584] = {.lex_state = 23, .external_lex_state = 5},
  [3585] = {.lex_state = 23, .external_lex_state = 5},
  [3586] = {.lex_state = 11},
  [3587] = {.lex_state = 11},
  [3588] = {.lex_state = 23, .external_lex_state = 5},
  [3589] = {.lex_state = 23, .external_lex_state = 5},
  [3590] = {.lex_state = 23, .external_lex_state = 5},
  [3591] = {.lex_state = 11},
  [3592] = {.lex_state = 11},
  [3593] = {.lex_state = 23, .external_lex_state = 5},
  [3594] = {.lex_state = 23, .external_lex_state = 5},
  [3595] = {.lex_state = 11},
  [3596] = {.lex_state = 11},
  [3597] = {.lex_state = 23},
  [3598] = {.lex_state = 23},
  [3599] = {.lex_state = 23},
  [3600] = {.lex_state = 11, .external_lex_state = 5},
  [3601] = {.lex_state = 23},
  [3602] = {.lex_state = 23, .external_lex_state = 5},
  [3603] = {.lex_state = 23},
  [3604] = {.lex_state = 23, .external_lex_state = 5},
  [3605] = {.lex_state = 23},
  [3606] = {.lex_state = 11, .external_lex_state = 5},
  [3607] = {.lex_state = 23, .external_lex_state = 5},
  [3608] = {.lex_state = 23},
  [3609] = {.lex_state = 23},
  [3610] = {.lex_state = 17},
  [3611] = {.lex_state = 11, .external_lex_state = 5},
  [3612] = {.lex_state = 23},
  [3613] = {.lex_state = 23},
  [3614] = {.lex_state = 23},
  [3615] = {.lex_state = 23, .external_lex_state = 5},
  [3616] = {.lex_state = 11},
  [3617] = {.lex_state = 23},
  [3618] = {.lex_state = 23},
  [3619] = {.lex_state = 23},
  [3620] = {.lex_state = 23},
  [3621] = {.lex_state = 23},
  [3622] = {.lex_state = 23},
  [3623] = {.lex_state = 23},
  [3624] = {.lex_state = 23},
  [3625] = {.lex_state = 23},
  [3626] = {.lex_state = 23, .external_lex_state = 5},
  [3627] = {.lex_state = 23},
  [3628] = {.lex_state = 23},
  [3629] = {.lex_state = 23, .external_lex_state = 5},
  [3630] = {.lex_state = 23},
  [3631] = {.lex_state = 23},
  [3632] = {.lex_state = 23, .external_lex_state = 5},
  [3633] = {.lex_state = 23, .external_lex_state = 5},
  [3634] = {.lex_state = 11, .external_lex_state = 5},
  [3635] = {.lex_state = 23},
  [3636] = {.lex_state = 23, .external_lex_state = 5},
  [3637] = {.lex_state = 23},
  [3638] = {.lex_state = 23},
  [3639] = {.lex_state = 23},
  [3640] = {.lex_state = 23},
  [3641] = {.lex_state = 23},
  [3642] = {.lex_state = 23},
  [3643] = {.lex_state = 23},
  [3644] = {.lex_state = 11, .external_lex_state = 5},
  [3645] = {.lex_state = 11, .external_lex_state = 5},
  [3646] = {.lex_state = 11},
  [3647] = {.lex_state = 23, .external_lex_state = 5},
  [3648] = {.lex_state = 11, .external_lex_state = 5},
  [3649] = {.lex_state = 26},
  [3650] = {.lex_state = 11},
  [3651] = {.lex_state = 26},
  [3652] = {.lex_state = 11},
  [3653] = {.lex_state = 11},
  [3654] = {.lex_state = 11},
  [3655] = {.lex_state = 11},
  [3656] = {.lex_state = 11, .external_lex_state = 5},
  [3657] = {.lex_state = 11, .external_lex_state = 5},
  [3658] = {.lex_state = 11, .external_lex_state = 5},
  [3659] = {.lex_state = 11, .external_lex_state = 5},
  [3660] = {.lex_state = 11, .external_lex_state = 5},
  [3661] = {.lex_state = 26},
  [3662] = {.lex_state = 11, .external_lex_state = 5},
  [3663] = {.lex_state = 11, .external_lex_state = 5},
  [3664] = {.lex_state = 11, .external_lex_state = 5},
  [3665] = {.lex_state = 26},
  [3666] = {.lex_state = 11},
  [3667] = {.lex_state = 11},
  [3668] = {.lex_state = 11, .external_lex_state = 5},
  [3669] = {.lex_state = 26},
  [3670] = {.lex_state = 11, .external_lex_state = 5},
  [3671] = {.lex_state = 11, .external_lex_state = 5},
  [3672] = {.lex_state = 26},
  [3673] = {.lex_state = 11},
  [3674] = {.lex_state = 17},
  [3675] = {.lex_state = 17},
  [3676] = {.lex_state = 17},
  [3677] = {.lex_state = 11},
  [3678] = {.lex_state = 26},
  [3679] = {.lex_state = 17},
  [3680] = {.lex_state = 11},
  [3681] = {.lex_state = 17},
  [3682] = {.lex_state = 17},
  [3683] = {.lex_state = 17},
  [3684] = {.lex_state = 17},
  [3685] = {.lex_state = 17},
  [3686] = {.lex_state = 17},
  [3687] = {.lex_state = 11, .external_lex_state = 5},
  [3688] = {.lex_state = 11},
  [3689] = {.lex_state = 17},
  [3690] = {.lex_state = 11, .external_lex_state = 5},
  [3691] = {.lex_state = 18},
  [3692] = {.lex_state = 17},
  [3693] = {.lex_state = 17},
  [3694] = {.lex_state = 17},
  [3695] = {.lex_state = 11},
  [3696] = {.lex_state = 17},
  [3697] = {.lex_state = 17},
  [3698] = {.lex_state = 17},
  [3699] = {.lex_state = 17},
  [3700] = {.lex_state = 11},
  [3701] = {.lex_state = 11},
  [3702] = {.lex_state = 11},
  [3703] = {.lex_state = 17},
  [3704] = {.lex_state = 11},
  [3705] = {.lex_state = 17},
  [3706] = {.lex_state = 11},
  [3707] = {.lex_state = 17},
  [3708] = {.lex_state = 11},
  [3709] = {.lex_state = 11},
  [3710] = {.lex_state = 17},
  [3711] = {.lex_state = 23},
  [3712] = {.lex_state = 23},
  [3713] = {.lex_state = 23},
  [3714] = {.lex_state = 11},
  [3715] = {.lex_state = 11},
  [3716] = {.lex_state = 11},
  [3717] = {.lex_state = 11},
  [3718] = {.lex_state = 23},
  [3719] = {.lex_state = 23},
  [3720] = {.lex_state = 17},
  [3721] = {.lex_state = 17},
  [3722] = {.lex_state = 11},
  [3723] = {.lex_state = 11},
  [3724] = {.lex_state = 11},
  [3725] = {.lex_state = 11},
  [3726] = {.lex_state = 11},
  [3727] = {.lex_state = 11},
  [3728] = {.lex_state = 11},
  [3729] = {.lex_state = 11},
  [3730] = {.lex_state = 18},
  [3731] = {.lex_state = 18},
  [3732] = {.lex_state = 18},
  [3733] = {.lex_state = 18},
  [3734] = {.lex_state = 7},
  [3735] = {.lex_state = 17},
  [3736] = {.lex_state = 7},
  [3737] = {.lex_state = 7},
  [3738] = {.lex_state = 17},
  [3739] = {.lex_state = 17},
  [3740] = {.lex_state = 17},
  [3741] = {.lex_state = 17},
  [3742] = {.lex_state = 17},
  [3743] = {.lex_state = 17},
  [3744] = {.lex_state = 11},
  [3745] = {.lex_state = 17},
  [3746] = {.lex_state = 17},
  [3747] = {.lex_state = 17},
  [3748] = {.lex_state = 11},
  [3749] = {.lex_state = 17},
  [3750] = {.lex_state = 17},
  [3751] = {.lex_state = 17},
  [3752] = {.lex_state = 17},
  [3753] = {.lex_state = 17},
  [3754] = {.lex_state = 23},
  [3755] = {.lex_state = 23},
  [3756] = {.lex_state = 23},
  [3757] = {.lex_state = 23},
  [3758] = {.lex_state = 23},
  [3759] = {.lex_state = 23},
  [3760] = {.lex_state = 23},
  [3761] = {.lex_state = 11},
  [3762] = {.lex_state = 23},
  [3763] = {.lex_state = 23},
  [3764] = {.lex_state = 23},
  [3765] = {.lex_state = 23},
  [3766] = {.lex_state = 23},
  [3767] = {.lex_state = 23},
  [3768] = {.lex_state = 23},
  [3769] = {.lex_state = 23},
  [3770] = {.lex_state = 23},
  [3771] = {.lex_state = 23},
  [3772] = {.lex_state = 23},
  [3773] = {.lex_state = 23},
  [3774] = {.lex_state = 23},
  [3775] = {.lex_state = 17},
  [3776] = {.lex_state = 17},
  [3777] = {.lex_state = 23},
  [3778] = {.lex_state = 11},
  [3779] = {.lex_state = 11},
  [3780] = {.lex_state = 18},
  [3781] = {.lex_state = 23},
  [3782] = {.lex_state = 11},
  [3783] = {.lex_state = 23},
  [3784] = {.lex_state = 23},
  [3785] = {.lex_state = 23},
  [3786] = {.lex_state = 23},
  [3787] = {.lex_state = 17},
  [3788] = {.lex_state = 18},
  [3789] = {.lex_state = 18},
  [3790] = {.lex_state = 17},
  [3791] = {.lex_state = 11},
  [3792] = {.lex_state = 11},
  [3793] = {.lex_state = 11},
  [3794] = {.lex_state = 11},
  [3795] = {.lex_state = 23},
  [3796] = {.lex_state = 11},
  [3797] = {.lex_state = 23},
  [3798] = {.lex_state = 23},
  [3799] = {.lex_state = 23},
  [3800] = {.lex_state = 23},
  [3801] = {.lex_state = 12},
  [3802] = {.lex_state = 17},
  [3803] = {.lex_state = 23},
  [3804] = {.lex_state = 23},
  [3805] = {.lex_state = 11},
  [3806] = {.lex_state = 23},
  [3807] = {.lex_state = 23},
  [3808] = {.lex_state = 17},
  [3809] = {.lex_state = 23},
  [3810] = {.lex_state = 23},
  [3811] = {.lex_state = 23},
  [3812] = {.lex_state = 23},
  [3813] = {.lex_state = 17},
  [3814] = {.lex_state = 26},
  [3815] = {.lex_state = 26},
  [3816] = {.lex_state = 11},
  [3817] = {.lex_state = 11},
  [3818] = {.lex_state = 26},
  [3819] = {.lex_state = 26},
  [3820] = {.lex_state = 26},
  [3821] = {.lex_state = 26},
  [3822] = {.lex_state = 11},
  [3823] = {.lex_state = 11},
  [3824] = {.lex_state = 23},
  [3825] = {.lex_state = 11},
  [3826] = {.lex_state = 11},
  [3827] = {.lex_state = 18},
  [3828] = {.lex_state = 11},
  [3829] = {.lex_state = 11},
  [3830] = {.lex_state = 23},
  [3831] = {.lex_state = 11},
  [3832] = {.lex_state = 11},
  [3833] = {.lex_state = 11},
  [3834] = {.lex_state = 23},
  [3835] = {.lex_state = 17},
  [3836] = {.lex_state = 11},
  [3837] = {.lex_state = 11},
  [3838] = {.lex_state = 26},
  [3839] = {.lex_state = 0},
  [3840] = {.lex_state = 17},
  [3841] = {.lex_state = 11},
  [3842] = {.lex_state = 26},
  [3843] = {.lex_state = 26},
  [3844] = {.lex_state = 26},
  [3845] = {.lex_state = 11},
  [3846] = {.lex_state = 17},
  [3847] = {.lex_state = 11},
  [3848] = {.lex_state = 11},
  [3849] = {.lex_state = 26},
  [3850] = {.lex_state = 26},
  [3851] = {.lex_state = 11},
  [3852] = {.lex_state = 23},
  [3853] = {.lex_state = 17},
  [3854] = {.lex_state = 17},
  [3855] = {.lex_state = 11},
  [3856] = {.lex_state = 17},
  [3857] = {.lex_state = 11},
  [3858] = {.lex_state = 18},
  [3859] = {.lex_state = 18},
  [3860] = {.lex_state = 11},
  [3861] = {.lex_state = 11},
  [3862] = {.lex_state = 17},
  [3863] = {.lex_state = 18},
  [3864] = {.lex_state = 12},
  [3865] = {.lex_state = 11},
  [3866] = {.lex_state = 11},
  [3867] = {.lex_state = 18},
  [3868] = {.lex_state = 17},
  [3869] = {.lex_state = 11},
  [3870] = {.lex_state = 26},
  [3871] = {.lex_state = 17},
  [3872] = {.lex_state = 17},
  [3873] = {.lex_state = 17},
  [3874] = {.lex_state = 11},
  [3875] = {.lex_state = 17},
  [3876] = {.lex_state = 26},
  [3877] = {.lex_state = 11},
  [3878] = {.lex_state = 17},
  [3879] = {.lex_state = 26},
  [3880] = {.lex_state = 17},
  [3881] = {.lex_state = 11},
  [3882] = {.lex_state = 17},
  [3883] = {.lex_state = 11},
  [3884] = {.lex_state = 17},
  [3885] = {.lex_state = 11},
  [3886] = {.lex_state = 11},
  [3887] = {.lex_state = 11},
  [3888] = {.lex_state = 11},
  [3889] = {.lex_state = 11},
  [3890] = {.lex_state = 11},
  [3891] = {.lex_state = 26},
  [3892] = {.lex_state = 11},
  [3893] = {.lex_state = 11},
  [3894] = {.lex_state = 11},
  [3895] = {.lex_state = 11},
  [3896] = {.lex_state = 11},
  [3897] = {.lex_state = 11},
  [3898] = {.lex_state = 11},
  [3899] = {.lex_state = 11},
  [3900] = {.lex_state = 11},
  [3901] = {.lex_state = 26},
  [3902] = {.lex_state = 18},
  [3903] = {.lex_state = 26},
  [3904] = {.lex_state = 11},
  [3905] = {.lex_state = 11},
  [3906] = {.lex_state = 26},
  [3907] = {.lex_state = 26},
  [3908] = {.lex_state = 11},
  [3909] = {.lex_state = 11},
  [3910] = {.lex_state = 26},
  [3911] = {.lex_state = 11},
  [3912] = {.lex_state = 11},
  [3913] = {.lex_state = 11},
  [3914] = {.lex_state = 11},
  [3915] = {.lex_state = 11},
  [3916] = {.lex_state = 11},
  [3917] = {.lex_state = 11},
  [3918] = {.lex_state = 11},
  [3919] = {.lex_state = 22, .external_lex_state = 6},
  [3920] = {.lex_state = 11},
  [3921] = {.lex_state = 11},
  [3922] = {.lex_state = 26},
  [3923] = {.lex_state = 11},
  [3924] = {.lex_state = 11},
  [3925] = {.lex_state = 11},
  [3926] = {.lex_state = 11},
  [3927] = {.lex_state = 11},
  [3928] = {.lex_state = 11},
  [3929] = {.lex_state = 11},
  [3930] = {.lex_state = 11},
  [3931] = {.lex_state = 11},
  [3932] = {.lex_state = 11},
  [3933] = {.lex_state = 11},
  [3934] = {.lex_state = 11},
  [3935] = {.lex_state = 11},
  [3936] = {.lex_state = 11},
  [3937] = {.lex_state = 11},
  [3938] = {.lex_state = 11},
  [3939] = {.lex_state = 11},
  [3940] = {.lex_state = 11},
  [3941] = {.lex_state = 11},
  [3942] = {.lex_state = 12},
  [3943] = {.lex_state = 72},
  [3944] = {.lex_state = 11},
  [3945] = {.lex_state = 72},
  [3946] = {.lex_state = 11},
  [3947] = {.lex_state = 11},
  [3948] = {.lex_state = 22, .external_lex_state = 6},
  [3949] = {.lex_state = 11},
  [3950] = {.lex_state = 11},
  [3951] = {.lex_state = 11},
  [3952] = {.lex_state = 11},
  [3953] = {.lex_state = 11},
  [3954] = {.lex_state = 11},
  [3955] = {.lex_state = 11},
  [3956] = {.lex_state = 26},
  [3957] = {.lex_state = 11},
  [3958] = {.lex_state = 22, .external_lex_state = 6},
  [3959] = {.lex_state = 26},
  [3960] = {.lex_state = 11},
  [3961] = {.lex_state = 22, .external_lex_state = 6},
  [3962] = {.lex_state = 22, .external_lex_state = 6},
  [3963] = {.lex_state = 11},
  [3964] = {.lex_state = 11},
  [3965] = {.lex_state = 22, .external_lex_state = 6},
  [3966] = {.lex_state = 22, .external_lex_state = 6},
  [3967] = {.lex_state = 72},
  [3968] = {.lex_state = 72},
  [3969] = {.lex_state = 11},
  [3970] = {.lex_state = 22, .external_lex_state = 6},
  [3971] = {.lex_state = 11},
  [3972] = {.lex_state = 0},
  [3973] = {.lex_state = 0},
  [3974] = {.lex_state = 0},
  [3975] = {.lex_state = 11},
  [3976] = {.lex_state = 0},
  [3977] = {.lex_state = 0},
  [3978] = {.lex_state = 0},
  [3979] = {.lex_state = 0},
  [3980] = {.lex_state = 0},
  [3981] = {.lex_state = 0},
  [3982] = {.lex_state = 0},
  [3983] = {.lex_state = 11},
  [3984] = {.lex_state = 23},
  [3985] = {.lex_state = 11},
  [3986] = {.lex_state = 0},
  [3987] = {.lex_state = 0},
  [3988] = {.lex_state = 0},
  [3989] = {.lex_state = 11},
  [3990] = {.lex_state = 0},
  [3991] = {.lex_state = 72},
  [3992] = {.lex_state = 72},
  [3993] = {.lex_state = 72},
  [3994] = {.lex_state = 0},
  [3995] = {.lex_state = 0},
  [3996] = {.lex_state = 0},
  [3997] = {.lex_state = 0},
  [3998] = {.lex_state = 0},
  [3999] = {.lex_state = 0},
  [4000] = {.lex_state = 72},
  [4001] = {.lex_state = 0},
  [4002] = {.lex_state = 0},
  [4003] = {.lex_state = 23},
  [4004] = {.lex_state = 23},
  [4005] = {.lex_state = 0},
  [4006] = {.lex_state = 22, .external_lex_state = 6},
  [4007] = {.lex_state = 23},
  [4008] = {.lex_state = 11},
  [4009] = {.lex_state = 0},
  [4010] = {.lex_state = 0},
  [4011] = {.lex_state = 0},
  [4012] = {.lex_state = 0},
  [4013] = {.lex_state = 0},
  [4014] = {.lex_state = 12},
  [4015] = {.lex_state = 0},
  [4016] = {.lex_state = 0},
  [4017] = {.lex_state = 0},
  [4018] = {.lex_state = 0},
  [4019] = {.lex_state = 0},
  [4020] = {.lex_state = 0},
  [4021] = {.lex_state = 0},
  [4022] = {.lex_state = 23},
  [4023] = {.lex_state = 0},
  [4024] = {.lex_state = 23},
  [4025] = {.lex_state = 0},
  [4026] = {.lex_state = 0},
  [4027] = {.lex_state = 23},
  [4028] = {.lex_state = 23},
  [4029] = {.lex_state = 0},
  [4030] = {.lex_state = 0},
  [4031] = {.lex_state = 0},
  [4032] = {.lex_state = 0},
  [4033] = {.lex_state = 11},
  [4034] = {.lex_state = 23},
  [4035] = {.lex_state = 23},
  [4036] = {.lex_state = 0},
  [4037] = {.lex_state = 23},
  [4038] = {.lex_state = 0},
  [4039] = {.lex_state = 11},
  [4040] = {.lex_state = 11},
  [4041] = {.lex_state = 23},
  [4042] = {.lex_state = 11},
  [4043] = {.lex_state = 0},
  [4044] = {.lex_state = 17},
  [4045] = {.lex_state = 17},
  [4046] = {.lex_state = 12},
  [4047] = {.lex_state = 23},
  [4048] = {.lex_state = 0},
  [4049] = {.lex_state = 11},
  [4050] = {.lex_state = 23},
  [4051] = {.lex_state = 11},
  [4052] = {.lex_state = 23},
  [4053] = {.lex_state = 0},
  [4054] = {.lex_state = 0},
  [4055] = {.lex_state = 11},
  [4056] = {.lex_state = 11},
  [4057] = {.lex_state = 72},
  [4058] = {.lex_state = 0},
  [4059] = {.lex_state = 11},
  [4060] = {.lex_state = 0},
  [4061] = {.lex_state = 23},
  [4062] = {.lex_state = 0},
  [4063] = {.lex_state = 0},
  [4064] = {.lex_state = 72},
  [4065] = {.lex_state = 11},
  [4066] = {.lex_state = 72},
  [4067] = {.lex_state = 12},
  [4068] = {.lex_state = 0},
  [4069] = {.lex_state = 0},
  [4070] = {.lex_state = 0},
  [4071] = {.lex_state = 11},
  [4072] = {.lex_state = 11},
  [4073] = {.lex_state = 23},
  [4074] = {.lex_state = 11},
  [4075] = {.lex_state = 72},
  [4076] = {.lex_state = 11},
  [4077] = {.lex_state = 23},
  [4078] = {.lex_state = 11},
  [4079] = {.lex_state = 11},
  [4080] = {.lex_state = 11},
  [4081] = {.lex_state = 11},
  [4082] = {.lex_state = 11},
  [4083] = {.lex_state = 11},
  [4084] = {.lex_state = 11},
  [4085] = {.lex_state = 11},
  [4086] = {.lex_state = 18},
  [4087] = {.lex_state = 11},
  [4088] = {.lex_state = 11},
  [4089] = {.lex_state = 11},
  [4090] = {.lex_state = 11},
  [4091] = {.lex_state = 11},
  [4092] = {.lex_state = 11},
  [4093] = {.lex_state = 11},
  [4094] = {.lex_state = 11},
  [4095] = {.lex_state = 11},
  [4096] = {.lex_state = 11},
  [4097] = {.lex_state = 11},
  [4098] = {.lex_state = 23},
  [4099] = {.lex_state = 11},
  [4100] = {.lex_state = 0},
  [4101] = {.lex_state = 11},
  [4102] = {.lex_state = 11},
  [4103] = {.lex_state = 11},
  [4104] = {.lex_state = 18},
  [4105] = {.lex_state = 11},
  [4106] = {.lex_state = 23},
  [4107] = {.lex_state = 11},
  [4108] = {.lex_state = 11},
  [4109] = {.lex_state = 11},
  [4110] = {.lex_state = 11},
  [4111] = {.lex_state = 11},
  [4112] = {.lex_state = 23},
  [4113] = {.lex_state = 23},
  [4114] = {.lex_state = 11},
  [4115] = {.lex_state = 11},
  [4116] = {.lex_state = 11},
  [4117] = {.lex_state = 11},
  [4118] = {.lex_state = 23},
  [4119] = {.lex_state = 11},
  [4120] = {.lex_state = 23},
  [4121] = {.lex_state = 11},
  [4122] = {.lex_state = 11},
  [4123] = {.lex_state = 11},
  [4124] = {.lex_state = 23},
  [4125] = {.lex_state = 18},
  [4126] = {.lex_state = 23},
  [4127] = {.lex_state = 23},
  [4128] = {.lex_state = 18},
  [4129] = {.lex_state = 11},
  [4130] = {.lex_state = 11},
  [4131] = {.lex_state = 11},
  [4132] = {.lex_state = 18},
  [4133] = {.lex_state = 11},
  [4134] = {.lex_state = 11},
  [4135] = {.lex_state = 11},
  [4136] = {.lex_state = 11},
  [4137] = {.lex_state = 11},
  [4138] = {.lex_state = 11},
  [4139] = {.lex_state = 11},
  [4140] = {.lex_state = 11},
  [4141] = {.lex_state = 11},
  [4142] = {.lex_state = 11},
  [4143] = {.lex_state = 11},
  [4144] = {.lex_state = 11},
  [4145] = {.lex_state = 11},
  [4146] = {.lex_state = 11},
  [4147] = {.lex_state = 23},
  [4148] = {.lex_state = 18},
  [4149] = {.lex_state = 11},
  [4150] = {.lex_state = 11},
  [4151] = {.lex_state = 11},
  [4152] = {.lex_state = 72},
  [4153] = {.lex_state = 11},
  [4154] = {.lex_state = 11},
  [4155] = {.lex_state = 11},
  [4156] = {.lex_state = 11},
  [4157] = {.lex_state = 11},
  [4158] = {.lex_state = 11},
  [4159] = {.lex_state = 11},
  [4160] = {.lex_state = 0},
  [4161] = {.lex_state = 11},
  [4162] = {.lex_state = 23},
  [4163] = {.lex_state = 0},
  [4164] = {.lex_state = 23},
  [4165] = {.lex_state = 0},
  [4166] = {.lex_state = 23},
  [4167] = {.lex_state = 11},
  [4168] = {.lex_state = 11},
  [4169] = {.lex_state = 0},
  [4170] = {.lex_state = 11},
  [4171] = {.lex_state = 11},
  [4172] = {.lex_state = 11},
  [4173] = {.lex_state = 11},
  [4174] = {.lex_state = 0},
  [4175] = {.lex_state = 11},
  [4176] = {.lex_state = 11},
  [4177] = {.lex_state = 11},
  [4178] = {.lex_state = 11},
  [4179] = {.lex_state = 11},
  [4180] = {.lex_state = 11},
  [4181] = {.lex_state = 11},
  [4182] = {.lex_state = 11},
  [4183] = {.lex_state = 18},
  [4184] = {.lex_state = 23},
  [4185] = {.lex_state = 0},
  [4186] = {.lex_state = 11},
  [4187] = {.lex_state = 11},
  [4188] = {.lex_state = 11},
  [4189] = {.lex_state = 11},
  [4190] = {.lex_state = 11},
  [4191] = {.lex_state = 11},
  [4192] = {.lex_state = 14},
  [4193] = {.lex_state = 23},
  [4194] = {.lex_state = 11},
  [4195] = {.lex_state = 11},
  [4196] = {.lex_state = 11},
  [4197] = {.lex_state = 11},
  [4198] = {.lex_state = 11},
  [4199] = {.lex_state = 23},
  [4200] = {.lex_state = 72},
  [4201] = {.lex_state = 14},
  [4202] = {.lex_state = 11},
  [4203] = {.lex_state = 11},
  [4204] = {.lex_state = 23},
  [4205] = {.lex_state = 11},
  [4206] = {.lex_state = 11},
  [4207] = {.lex_state = 23},
  [4208] = {.lex_state = 11},
  [4209] = {.lex_state = 23},
  [4210] = {.lex_state = 0},
  [4211] = {.lex_state = 11},
  [4212] = {.lex_state = 72},
  [4213] = {.lex_state = 11},
  [4214] = {.lex_state = 11},
  [4215] = {.lex_state = 11},
  [4216] = {.lex_state = 11},
  [4217] = {.lex_state = 11},
  [4218] = {.lex_state = 11},
  [4219] = {.lex_state = 11},
  [4220] = {.lex_state = 11},
  [4221] = {.lex_state = 11},
  [4222] = {.lex_state = 11},
  [4223] = {.lex_state = 11},
  [4224] = {.lex_state = 0},
  [4225] = {.lex_state = 11},
  [4226] = {.lex_state = 11},
  [4227] = {.lex_state = 11},
  [4228] = {.lex_state = 11},
  [4229] = {.lex_state = 11},
  [4230] = {.lex_state = 11},
  [4231] = {.lex_state = 11},
  [4232] = {.lex_state = 11},
  [4233] = {.lex_state = 11},
  [4234] = {.lex_state = 11},
  [4235] = {.lex_state = 11},
  [4236] = {.lex_state = 11},
  [4237] = {.lex_state = 11},
  [4238] = {.lex_state = 11},
  [4239] = {.lex_state = 11},
  [4240] = {.lex_state = 11},
  [4241] = {.lex_state = 11},
  [4242] = {.lex_state = 11},
  [4243] = {.lex_state = 11},
  [4244] = {.lex_state = 11},
  [4245] = {.lex_state = 11},
  [4246] = {.lex_state = 11},
  [4247] = {.lex_state = 11},
  [4248] = {.lex_state = 18},
  [4249] = {.lex_state = 11},
  [4250] = {.lex_state = 11},
  [4251] = {.lex_state = 11},
  [4252] = {.lex_state = 0},
  [4253] = {.lex_state = 23},
  [4254] = {.lex_state = 18},
  [4255] = {.lex_state = 11},
  [4256] = {.lex_state = 11},
  [4257] = {.lex_state = 18},
  [4258] = {.lex_state = 11},
  [4259] = {.lex_state = 18},
  [4260] = {.lex_state = 11},
  [4261] = {.lex_state = 11},
  [4262] = {.lex_state = 11},
  [4263] = {.lex_state = 11},
  [4264] = {.lex_state = 18},
  [4265] = {.lex_state = 11},
  [4266] = {.lex_state = 11},
  [4267] = {.lex_state = 11},
  [4268] = {.lex_state = 11},
  [4269] = {.lex_state = 11},
  [4270] = {.lex_state = 23},
  [4271] = {.lex_state = 23},
  [4272] = {.lex_state = 11},
  [4273] = {.lex_state = 23},
  [4274] = {.lex_state = 23},
  [4275] = {.lex_state = 11},
  [4276] = {.lex_state = 11},
  [4277] = {.lex_state = 0},
  [4278] = {.lex_state = 11},
  [4279] = {.lex_state = 23},
  [4280] = {.lex_state = 23},
  [4281] = {.lex_state = 11},
  [4282] = {.lex_state = 11},
  [4283] = {.lex_state = 23},
  [4284] = {.lex_state = 11},
  [4285] = {.lex_state = 23},
  [4286] = {.lex_state = 11},
  [4287] = {.lex_state = 23},
  [4288] = {.lex_state = 11},
  [4289] = {.lex_state = 11},
  [4290] = {.lex_state = 11},
  [4291] = {.lex_state = 11},
  [4292] = {.lex_state = 11},
  [4293] = {.lex_state = 11},
  [4294] = {.lex_state = 23},
  [4295] = {.lex_state = 11},
  [4296] = {.lex_state = 11},
  [4297] = {.lex_state = 11},
  [4298] = {.lex_state = 11},
  [4299] = {.lex_state = 11},
  [4300] = {.lex_state = 11},
  [4301] = {.lex_state = 11},
  [4302] = {.lex_state = 11},
  [4303] = {.lex_state = 11},
  [4304] = {.lex_state = 11},
  [4305] = {.lex_state = 11},
  [4306] = {.lex_state = 11},
  [4307] = {.lex_state = 11},
  [4308] = {.lex_state = 11},
  [4309] = {.lex_state = 11},
  [4310] = {.lex_state = 11},
  [4311] = {.lex_state = 11},
  [4312] = {.lex_state = 11},
  [4313] = {.lex_state = 11},
  [4314] = {.lex_state = 11},
  [4315] = {.lex_state = 11},
  [4316] = {.lex_state = 11},
  [4317] = {.lex_state = 11},
  [4318] = {.lex_state = 23},
  [4319] = {.lex_state = 11},
  [4320] = {.lex_state = 11},
  [4321] = {.lex_state = 11},
  [4322] = {.lex_state = 11},
  [4323] = {.lex_state = 11},
  [4324] = {.lex_state = 11},
  [4325] = {.lex_state = 11},
  [4326] = {.lex_state = 11},
  [4327] = {.lex_state = 11},
  [4328] = {.lex_state = 11},
  [4329] = {.lex_state = 11},
  [4330] = {.lex_state = 11},
  [4331] = {.lex_state = 11},
  [4332] = {.lex_state = 11},
  [4333] = {.lex_state = 11},
  [4334] = {.lex_state = 11},
  [4335] = {.lex_state = 11},
  [4336] = {.lex_state = 11},
  [4337] = {.lex_state = 11},
  [4338] = {.lex_state = 11},
  [4339] = {.lex_state = 11},
  [4340] = {.lex_state = 11},
  [4341] = {.lex_state = 11},
  [4342] = {.lex_state = 0},
  [4343] = {.lex_state = 72},
  [4344] = {.lex_state = 11},
  [4345] = {.lex_state = 11},
  [4346] = {.lex_state = 11},
  [4347] = {.lex_state = 11},
  [4348] = {.lex_state = 11},
  [4349] = {.lex_state = 11},
  [4350] = {.lex_state = 11},
  [4351] = {.lex_state = 11},
  [4352] = {.lex_state = 11},
  [4353] = {.lex_state = 11},
  [4354] = {.lex_state = 11},
  [4355] = {.lex_state = 11},
  [4356] = {.lex_state = 11},
  [4357] = {.lex_state = 11},
  [4358] = {.lex_state = 11},
  [4359] = {.lex_state = 11},
  [4360] = {.lex_state = 11},
  [4361] = {.lex_state = 11},
  [4362] = {.lex_state = 23},
  [4363] = {.lex_state = 23},
  [4364] = {.lex_state = 72},
  [4365] = {.lex_state = 11},
  [4366] = {.lex_state = 11},
  [4367] = {.lex_state = 23},
  [4368] = {.lex_state = 0},
  [4369] = {.lex_state = 23},
  [4370] = {.lex_state = 23},
  [4371] = {.lex_state = 23},
  [4372] = {.lex_state = 23},
  [4373] = {.lex_state = 23},
  [4374] = {.lex_state = 11},
  [4375] = {.lex_state = 11},
  [4376] = {.lex_state = 23},
  [4377] = {.lex_state = 11},
  [4378] = {.lex_state = 11},
  [4379] = {.lex_state = 11},
  [4380] = {.lex_state = 11},
  [4381] = {.lex_state = 11},
  [4382] = {.lex_state = 11},
  [4383] = {.lex_state = 11},
  [4384] = {.lex_state = 23},
  [4385] = {.lex_state = 11},
  [4386] = {.lex_state = 11},
  [4387] = {.lex_state = 11},
  [4388] = {.lex_state = 11},
  [4389] = {.lex_state = 11},
  [4390] = {.lex_state = 23},
  [4391] = {.lex_state = 11},
  [4392] = {.lex_state = 11},
  [4393] = {.lex_state = 11},
  [4394] = {.lex_state = 11},
  [4395] = {.lex_state = 11},
  [4396] = {.lex_state = 23},
  [4397] = {.lex_state = 23},
  [4398] = {.lex_state = 11},
  [4399] = {.lex_state = 23},
  [4400] = {.lex_state = 11},
  [4401] = {.lex_state = 23},
  [4402] = {.lex_state = 11},
  [4403] = {.lex_state = 11},
  [4404] = {.lex_state = 11},
  [4405] = {.lex_state = 0},
  [4406] = {.lex_state = 11},
  [4407] = {.lex_state = 11},
  [4408] = {.lex_state = 23},
  [4409] = {.lex_state = 11},
  [4410] = {.lex_state = 11},
  [4411] = {.lex_state = 0},
  [4412] = {.lex_state = 11},
  [4413] = {.lex_state = 11},
  [4414] = {.lex_state = 0},
  [4415] = {.lex_state = 23},
  [4416] = {.lex_state = 23},
  [4417] = {.lex_state = 11},
  [4418] = {.lex_state = 23},
  [4419] = {.lex_state = 23},
  [4420] = {.lex_state = 23},
  [4421] = {.lex_state = 11},
  [4422] = {.lex_state = 11},
  [4423] = {.lex_state = 11},
  [4424] = {.lex_state = 11},
  [4425] = {.lex_state = 11},
  [4426] = {.lex_state = 11},
  [4427] = {.lex_state = 11},
  [4428] = {.lex_state = 11},
  [4429] = {.lex_state = 11},
  [4430] = {.lex_state = 0},
  [4431] = {.lex_state = 11},
  [4432] = {.lex_state = 11},
  [4433] = {.lex_state = 23},
  [4434] = {.lex_state = 23},
  [4435] = {.lex_state = 23},
  [4436] = {.lex_state = 11},
  [4437] = {.lex_state = 23},
  [4438] = {.lex_state = 11},
  [4439] = {.lex_state = 11},
  [4440] = {.lex_state = 11},
  [4441] = {.lex_state = 11},
  [4442] = {.lex_state = 72},
  [4443] = {.lex_state = 72},
  [4444] = {.lex_state = 11},
  [4445] = {.lex_state = 11},
  [4446] = {.lex_state = 0},
  [4447] = {.lex_state = 23},
  [4448] = {.lex_state = 11},
  [4449] = {.lex_state = 11},
  [4450] = {.lex_state = 11},
  [4451] = {.lex_state = 11},
  [4452] = {.lex_state = 11},
  [4453] = {.lex_state = 7},
  [4454] = {.lex_state = 7},
  [4455] = {.lex_state = 7},
  [4456] = {.lex_state = 9},
  [4457] = {.lex_state = 0},
  [4458] = {.lex_state = 11},
  [4459] = {.lex_state = 11},
  [4460] = {.lex_state = 11},
  [4461] = {.lex_state = 11},
  [4462] = {.lex_state = 11},
  [4463] = {.lex_state = 11},
  [4464] = {.lex_state = 7},
  [4465] = {.lex_state = 0},
  [4466] = {.lex_state = 7},
  [4467] = {.lex_state = 7},
  [4468] = {.lex_state = 11},
  [4469] = {.lex_state = 11},
  [4470] = {.lex_state = 11},
  [4471] = {.lex_state = 11},
  [4472] = {.lex_state = 11},
  [4473] = {.lex_state = 11},
  [4474] = {.lex_state = 11},
  [4475] = {.lex_state = 11},
  [4476] = {.lex_state = 11},
  [4477] = {.lex_state = 11},
  [4478] = {.lex_state = 11},
  [4479] = {.lex_state = 11},
  [4480] = {.lex_state = 11},
  [4481] = {.lex_state = 11},
  [4482] = {.lex_state = 11},
  [4483] = {.lex_state = 11},
  [4484] = {.lex_state = 11},
  [4485] = {.lex_state = 0},
  [4486] = {.lex_state = 72},
  [4487] = {.lex_state = 11},
  [4488] = {.lex_state = 11},
  [4489] = {.lex_state = 11},
  [4490] = {.lex_state = 11},
  [4491] = {.lex_state = 11},
  [4492] = {.lex_state = 11},
  [4493] = {.lex_state = 11},
  [4494] = {.lex_state = 72},
  [4495] = {.lex_state = 11},
  [4496] = {.lex_state = 72},
  [4497] = {.lex_state = 0},
  [4498] = {.lex_state = 11},
  [4499] = {.lex_state = 11},
  [4500] = {.lex_state = 11},
  [4501] = {.lex_state = 11},
  [4502] = {.lex_state = 11},
  [4503] = {.lex_state = 11},
  [4504] = {.lex_state = 11},
  [4505] = {.lex_state = 11},
  [4506] = {.lex_state = 11},
  [4507] = {.lex_state = 11},
  [4508] = {.lex_state = 11},
  [4509] = {.lex_state = 11},
  [4510] = {.lex_state = 11},
  [4511] = {.lex_state = 11},
  [4512] = {.lex_state = 11},
  [4513] = {.lex_state = 11},
  [4514] = {.lex_state = 11},
  [4515] = {.lex_state = 11},
  [4516] = {.lex_state = 11},
  [4517] = {.lex_state = 11},
  [4518] = {.lex_state = 11},
  [4519] = {.lex_state = 11},
  [4520] = {.lex_state = 11},
  [4521] = {.lex_state = 7},
  [4522] = {.lex_state = 23},
  [4523] = {.lex_state = 11},
  [4524] = {.lex_state = 11},
  [4525] = {.lex_state = 11, .external_lex_state = 7},
  [4526] = {.lex_state = 7},
  [4527] = {.lex_state = 11},
  [4528] = {.lex_state = 11},
  [4529] = {.lex_state = 11},
  [4530] = {.lex_state = 7},
  [4531] = {.lex_state = 72},
  [4532] = {.lex_state = 72},
  [4533] = {.lex_state = 11},
  [4534] = {.lex_state = 0},
  [4535] = {.lex_state = 17},
  [4536] = {.lex_state = 72},
  [4537] = {.lex_state = 11},
  [4538] = {.lex_state = 72},
  [4539] = {.lex_state = 0},
  [4540] = {.lex_state = 7},
  [4541] = {.lex_state = 11},
  [4542] = {.lex_state = 7},
  [4543] = {.lex_state = 11},
  [4544] = {.lex_state = 11, .external_lex_state = 7},
  [4545] = {.lex_state = 11},
  [4546] = {.lex_state = 11},
  [4547] = {.lex_state = 11},
  [4548] = {.lex_state = 11},
  [4549] = {.lex_state = 7},
  [4550] = {.lex_state = 17},
  [4551] = {.lex_state = 72},
  [4552] = {.lex_state = 0},
  [4553] = {.lex_state = 0},
  [4554] = {.lex_state = 11},
  [4555] = {.lex_state = 11},
  [4556] = {.lex_state = 23},
  [4557] = {.lex_state = 11},
  [4558] = {.lex_state = 11},
  [4559] = {.lex_state = 11, .external_lex_state = 7},
  [4560] = {.lex_state = 0},
  [4561] = {.lex_state = 7},
  [4562] = {.lex_state = 11},
  [4563] = {.lex_state = 7},
  [4564] = {.lex_state = 0},
  [4565] = {.lex_state = 7},
  [4566] = {.lex_state = 7},
  [4567] = {.lex_state = 72},
  [4568] = {.lex_state = 11},
  [4569] = {.lex_state = 11},
  [4570] = {.lex_state = 7},
  [4571] = {.lex_state = 7},
  [4572] = {.lex_state = 7},
  [4573] = {.lex_state = 11},
  [4574] = {.lex_state = 18},
  [4575] = {.lex_state = 18},
  [4576] = {.lex_state = 11},
  [4577] = {.lex_state = 18},
  [4578] = {.lex_state = 18},
  [4579] = {.lex_state = 11},
  [4580] = {.lex_state = 72},
  [4581] = {.lex_state = 11},
  [4582] = {.lex_state = 72},
  [4583] = {.lex_state = 7},
  [4584] = {.lex_state = 7},
  [4585] = {.lex_state = 11},
  [4586] = {.lex_state = 11},
  [4587] = {.lex_state = 18},
  [4588] = {.lex_state = 18},
  [4589] = {.lex_state = 18},
  [4590] = {.lex_state = 11},
  [4591] = {.lex_state = 11},
  [4592] = {.lex_state = 18},
  [4593] = {.lex_state = 11},
  [4594] = {.lex_state = 11},
  [4595] = {.lex_state = 11},
  [4596] = {.lex_state = 11},
  [4597] = {.lex_state = 11},
  [4598] = {.lex_state = 11},
  [4599] = {.lex_state = 11},
  [4600] = {.lex_state = 11},
  [4601] = {.lex_state = 11},
  [4602] = {.lex_state = 11, .external_lex_state = 7},
  [4603] = {.lex_state = 11},
  [4604] = {.lex_state = 11},
  [4605] = {.lex_state = 11},
  [4606] = {.lex_state = 11},
  [4607] = {.lex_state = 0},
  [4608] = {.lex_state = 11},
  [4609] = {.lex_state = 11},
  [4610] = {.lex_state = 11},
  [4611] = {.lex_state = 11},
  [4612] = {.lex_state = 11, .external_lex_state = 7},
  [4613] = {.lex_state = 0},
  [4614] = {.lex_state = 11},
  [4615] = {.lex_state = 7},
  [4616] = {.lex_state = 11},
  [4617] = {.lex_state = 7},
  [4618] = {.lex_state = 7},
  [4619] = {.lex_state = 7},
  [4620] = {.lex_state = 7},
  [4621] = {.lex_state = 11},
  [4622] = {.lex_state = 11},
  [4623] = {.lex_state = 7},
  [4624] = {.lex_state = 11},
  [4625] = {.lex_state = 11},
  [4626] = {.lex_state = 11},
  [4627] = {.lex_state = 11},
  [4628] = {.lex_state = 11},
  [4629] = {.lex_state = 11},
  [4630] = {.lex_state = 11, .external_lex_state = 7},
  [4631] = {.lex_state = 11},
  [4632] = {.lex_state = 11},
  [4633] = {.lex_state = 11},
  [4634] = {.lex_state = 23},
  [4635] = {.lex_state = 0},
  [4636] = {.lex_state = 11, .external_lex_state = 7},
  [4637] = {.lex_state = 0},
  [4638] = {.lex_state = 11},
  [4639] = {.lex_state = 0},
  [4640] = {.lex_state = 11},
  [4641] = {.lex_state = 7},
  [4642] = {.lex_state = 7},
  [4643] = {.lex_state = 7},
  [4644] = {.lex_state = 7},
  [4645] = {.lex_state = 11, .external_lex_state = 7},
  [4646] = {.lex_state = 11, .external_lex_state = 7},
  [4647] = {.lex_state = 0},
  [4648] = {.lex_state = 11, .external_lex_state = 7},
  [4649] = {.lex_state = 0},
  [4650] = {.lex_state = 7},
  [4651] = {.lex_state = 11},
  [4652] = {.lex_state = 7},
  [4653] = {.lex_state = 7},
  [4654] = {.lex_state = 72},
  [4655] = {.lex_state = 7},
  [4656] = {.lex_state = 72},
  [4657] = {.lex_state = 7},
  [4658] = {.lex_state = 7},
  [4659] = {.lex_state = 72},
  [4660] = {.lex_state = 11},
  [4661] = {.lex_state = 11},
  [4662] = {.lex_state = 7},
  [4663] = {.lex_state = 7},
  [4664] = {.lex_state = 11},
  [4665] = {.lex_state = 7},
  [4666] = {.lex_state = 7},
  [4667] = {.lex_state = 0},
  [4668] = {.lex_state = 11},
  [4669] = {.lex_state = 0},
  [4670] = {.lex_state = 11},
  [4671] = {.lex_state = 23},
  [4672] = {.lex_state = 11},
  [4673] = {.lex_state = 11},
  [4674] = {.lex_state = 72},
  [4675] = {.lex_state = 11},
  [4676] = {.lex_state = 11},
  [4677] = {.lex_state = 11},
  [4678] = {.lex_state = 11},
  [4679] = {.lex_state = 11},
  [4680] = {.lex_state = 11},
  [4681] = {.lex_state = 18},
  [4682] = {.lex_state = 18},
  [4683] = {.lex_state = 7},
  [4684] = {.lex_state = 11},
  [4685] = {.lex_state = 11, .external_lex_state = 7},
  [4686] = {.lex_state = 7},
  [4687] = {.lex_state = 11},
  [4688] = {.lex_state = 11},
  [4689] = {.lex_state = 72},
  [4690] = {.lex_state = 23},
  [4691] = {.lex_state = 11},
  [4692] = {.lex_state = 7},
  [4693] = {.lex_state = 72},
  [4694] = {.lex_state = 11},
  [4695] = {.lex_state = 11},
  [4696] = {.lex_state = 18},
  [4697] = {.lex_state = 11},
  [4698] = {.lex_state = 72},
  [4699] = {.lex_state = 11},
  [4700] = {.lex_state = 18},
  [4701] = {.lex_state = 11},
  [4702] = {.lex_state = 11},
  [4703] = {.lex_state = 0},
  [4704] = {.lex_state = 11},
  [4705] = {.lex_state = 11},
  [4706] = {.lex_state = 11},
  [4707] = {.lex_state = 11},
  [4708] = {.lex_state = 11},
  [4709] = {.lex_state = 0},
  [4710] = {.lex_state = 11},
  [4711] = {.lex_state = 7},
  [4712] = {.lex_state = 11},
  [4713] = {.lex_state = 11},
  [4714] = {.lex_state = 11},
  [4715] = {.lex_state = 11},
  [4716] = {.lex_state = 7},
  [4717] = {.lex_state = 11},
  [4718] = {.lex_state = 11},
  [4719] = {.lex_state = 72},
  [4720] = {.lex_state = 72},
  [4721] = {.lex_state = 11},
  [4722] = {.lex_state = 11},
  [4723] = {.lex_state = 11},
  [4724] = {.lex_state = 23},
  [4725] = {.lex_state = 11},
  [4726] = {.lex_state = 11},
  [4727] = {.lex_state = 11},
  [4728] = {.lex_state = 11},
  [4729] = {.lex_state = 11},
  [4730] = {.lex_state = 11},
  [4731] = {.lex_state = 7},
  [4732] = {.lex_state = 72},
  [4733] = {.lex_state = 7},
  [4734] = {.lex_state = 7},
  [4735] = {.lex_state = 11},
  [4736] = {.lex_state = 72},
  [4737] = {.lex_state = 11},
  [4738] = {.lex_state = 11},
  [4739] = {.lex_state = 7},
  [4740] = {.lex_state = 72},
  [4741] = {.lex_state = 11},
  [4742] = {.lex_state = 11},
  [4743] = {.lex_state = 11},
  [4744] = {.lex_state = 11},
  [4745] = {.lex_state = 11},
  [4746] = {.lex_state = 11},
  [4747] = {.lex_state = 0},
  [4748] = {.lex_state = 11},
  [4749] = {.lex_state = 11},
  [4750] = {.lex_state = 11},
  [4751] = {.lex_state = 11},
  [4752] = {.lex_state = 11},
  [4753] = {.lex_state = 7},
  [4754] = {.lex_state = 11},
  [4755] = {.lex_state = 11},
  [4756] = {.lex_state = 11},
  [4757] = {.lex_state = 11},
  [4758] = {.lex_state = 11},
  [4759] = {.lex_state = 11},
  [4760] = {.lex_state = 11},
  [4761] = {.lex_state = 11},
  [4762] = {.lex_state = 0},
  [4763] = {.lex_state = 72},
  [4764] = {.lex_state = 11},
  [4765] = {.lex_state = 11},
  [4766] = {.lex_state = 11, .external_lex_state = 7},
  [4767] = {.lex_state = 11},
  [4768] = {.lex_state = 72},
  [4769] = {.lex_state = 11},
  [4770] = {.lex_state = 11},
  [4771] = {.lex_state = 11},
  [4772] = {.lex_state = 11},
  [4773] = {.lex_state = 11},
  [4774] = {.lex_state = 11},
  [4775] = {.lex_state = 72},
  [4776] = {.lex_state = 11},
  [4777] = {.lex_state = 72},
  [4778] = {.lex_state = 11},
  [4779] = {.lex_state = 11},
  [4780] = {.lex_state = 72},
  [4781] = {.lex_state = 11},
  [4782] = {.lex_state = 11},
  [4783] = {.lex_state = 11},
  [4784] = {.lex_state = 72},
  [4785] = {.lex_state = 11},
  [4786] = {.lex_state = 11},
  [4787] = {.lex_state = 11},
  [4788] = {.lex_state = 28},
  [4789] = {.lex_state = 11},
  [4790] = {.lex_state = 28},
  [4791] = {.lex_state = 11},
  [4792] = {.lex_state = 0},
  [4793] = {.lex_state = 11},
  [4794] = {.lex_state = 7},
  [4795] = {.lex_state = 23},
  [4796] = {.lex_state = 11},
  [4797] = {.lex_state = 11},
  [4798] = {.lex_state = 17},
  [4799] = {.lex_state = 11},
  [4800] = {.lex_state = 17},
  [4801] = {.lex_state = 28},
  [4802] = {.lex_state = 0},
  [4803] = {.lex_state = 72},
  [4804] = {.lex_state = 18},
  [4805] = {.lex_state = 11},
  [4806] = {.lex_state = 17},
  [4807] = {.lex_state = 11},
  [4808] = {.lex_state = 72},
  [4809] = {.lex_state = 11},
  [4810] = {.lex_state = 0},
  [4811] = {.lex_state = 11},
  [4812] = {.lex_state = 11},
  [4813] = {.lex_state = 0},
  [4814] = {.lex_state = 11},
  [4815] = {.lex_state = 11},
  [4816] = {.lex_state = 11},
  [4817] = {.lex_state = 11},
  [4818] = {.lex_state = 11, .external_lex_state = 7},
  [4819] = {.lex_state = 11},
  [4820] = {.lex_state = 11},
  [4821] = {.lex_state = 7},
  [4822] = {.lex_state = 11},
  [4823] = {.lex_state = 11},
  [4824] = {.lex_state = 23},
  [4825] = {.lex_state = 11},
  [4826] = {.lex_state = 28},
  [4827] = {.lex_state = 11},
  [4828] = {.lex_state = 11},
  [4829] = {.lex_state = 7},
  [4830] = {.lex_state = 7},
  [4831] = {.lex_state = 7},
  [4832] = {.lex_state = 23},
  [4833] = {.lex_state = 72},
  [4834] = {.lex_state = 7},
  [4835] = {.lex_state = 7},
  [4836] = {.lex_state = 7},
  [4837] = {.lex_state = 7},
  [4838] = {.lex_state = 7},
  [4839] = {.lex_state = 7},
  [4840] = {.lex_state = 7},
  [4841] = {.lex_state = 7},
  [4842] = {.lex_state = 7},
  [4843] = {.lex_state = 7},
  [4844] = {.lex_state = 7},
  [4845] = {.lex_state = 72},
  [4846] = {.lex_state = 7},
  [4847] = {.lex_state = 11},
  [4848] = {.lex_state = 11},
  [4849] = {.lex_state = 11},
  [4850] = {.lex_state = 72},
  [4851] = {.lex_state = 11},
  [4852] = {.lex_state = 0},
  [4853] = {.lex_state = 11},
  [4854] = {.lex_state = 11},
  [4855] = {.lex_state = 11},
  [4856] = {.lex_state = 11},
  [4857] = {.lex_state = 23},
  [4858] = {.lex_state = 11},
  [4859] = {.lex_state = 18},
  [4860] = {.lex_state = 72},
  [4861] = {.lex_state = 0},
  [4862] = {.lex_state = 72},
  [4863] = {.lex_state = 11},
  [4864] = {.lex_state = 72},
  [4865] = {.lex_state = 7},
  [4866] = {.lex_state = 72},
  [4867] = {.lex_state = 72},
  [4868] = {.lex_state = 0},
  [4869] = {.lex_state = 0},
  [4870] = {.lex_state = 0},
  [4871] = {.lex_state = 0},
  [4872] = {.lex_state = 72},
  [4873] = {.lex_state = 72},
  [4874] = {.lex_state = 11},
  [4875] = {.lex_state = 11},
  [4876] = {.lex_state = 0},
  [4877] = {.lex_state = 0},
  [4878] = {.lex_state = 0},
  [4879] = {.lex_state = 11},
  [4880] = {.lex_state = 72},
  [4881] = {.lex_state = 0},
  [4882] = {.lex_state = 0},
  [4883] = {.lex_state = 0},
  [4884] = {.lex_state = 0},
  [4885] = {.lex_state = 72},
  [4886] = {.lex_state = 72},
  [4887] = {.lex_state = 0},
  [4888] = {.lex_state = 18},
  [4889] = {.lex_state = 0},
  [4890] = {.lex_state = 72},
  [4891] = {.lex_state = 72},
  [4892] = {.lex_state = 0},
  [4893] = {.lex_state = 72},
  [4894] = {.lex_state = 0},
  [4895] = {.lex_state = 0},
  [4896] = {.lex_state = 0},
  [4897] = {.lex_state = 72},
  [4898] = {.lex_state = 0},
  [4899] = {.lex_state = 0},
  [4900] = {.lex_state = 0},
  [4901] = {.lex_state = 0},
  [4902] = {.lex_state = 72},
  [4903] = {.lex_state = 0},
  [4904] = {.lex_state = 0},
  [4905] = {.lex_state = 0},
  [4906] = {.lex_state = 18},
  [4907] = {.lex_state = 0},
  [4908] = {.lex_state = 72},
  [4909] = {.lex_state = 7},
  [4910] = {.lex_state = 72},
  [4911] = {.lex_state = 72},
  [4912] = {.lex_state = 0},
  [4913] = {.lex_state = 72},
  [4914] = {.lex_state = 18},
  [4915] = {.lex_state = 0},
  [4916] = {.lex_state = 0},
  [4917] = {.lex_state = 0},
  [4918] = {.lex_state = 0},
  [4919] = {.lex_state = 72},
  [4920] = {.lex_state = 72},
  [4921] = {.lex_state = 72},
  [4922] = {.lex_state = 0},
  [4923] = {.lex_state = 18},
  [4924] = {.lex_state = 18},
  [4925] = {.lex_state = 0},
  [4926] = {.lex_state = 0},
  [4927] = {.lex_state = 0},
  [4928] = {.lex_state = 19},
  [4929] = {.lex_state = 72},
  [4930] = {.lex_state = 18},
  [4931] = {.lex_state = 72},
  [4932] = {.lex_state = 72},
  [4933] = {.lex_state = 18},
  [4934] = {.lex_state = 0},
  [4935] = {.lex_state = 0},
  [4936] = {.lex_state = 0},
  [4937] = {.lex_state = 72},
  [4938] = {.lex_state = 11},
  [4939] = {.lex_state = 18},
  [4940] = {.lex_state = 72},
  [4941] = {.lex_state = 0},
  [4942] = {.lex_state = 72},
  [4943] = {.lex_state = 18},
  [4944] = {.lex_state = 72},
  [4945] = {.lex_state = 0},
  [4946] = {.lex_state = 0},
  [4947] = {.lex_state = 18},
  [4948] = {.lex_state = 0},
  [4949] = {.lex_state = 11},
  [4950] = {.lex_state = 0},
  [4951] = {.lex_state = 72},
  [4952] = {.lex_state = 72},
  [4953] = {.lex_state = 18},
  [4954] = {.lex_state = 11},
  [4955] = {.lex_state = 72},
  [4956] = {.lex_state = 11},
  [4957] = {.lex_state = 18},
  [4958] = {.lex_state = 0},
  [4959] = {.lex_state = 0},
  [4960] = {.lex_state = 72},
  [4961] = {.lex_state = 18},
  [4962] = {.lex_state = 72},
  [4963] = {.lex_state = 0},
  [4964] = {.lex_state = 0},
  [4965] = {.lex_state = 0},
  [4966] = {.lex_state = 0},
  [4967] = {.lex_state = 0},
  [4968] = {.lex_state = 0},
  [4969] = {.lex_state = 0},
  [4970] = {.lex_state = 72},
  [4971] = {.lex_state = 72},
  [4972] = {.lex_state = 72},
  [4973] = {.lex_state = 0},
  [4974] = {.lex_state = 0},
  [4975] = {.lex_state = 72},
  [4976] = {.lex_state = 11},
  [4977] = {.lex_state = 11},
  [4978] = {.lex_state = 0},
  [4979] = {.lex_state = 0},
  [4980] = {.lex_state = 72},
  [4981] = {.lex_state = 72},
  [4982] = {.lex_state = 72},
  [4983] = {.lex_state = 72},
  [4984] = {.lex_state = 72},
  [4985] = {.lex_state = 0},
  [4986] = {.lex_state = 72},
  [4987] = {.lex_state = 72},
  [4988] = {.lex_state = 11},
  [4989] = {.lex_state = 0},
  [4990] = {.lex_state = 72},
  [4991] = {.lex_state = 72},
  [4992] = {.lex_state = 72},
  [4993] = {.lex_state = 72},
  [4994] = {.lex_state = 0},
  [4995] = {.lex_state = 72},
  [4996] = {.lex_state = 72},
  [4997] = {.lex_state = 72},
  [4998] = {.lex_state = 0},
  [4999] = {.lex_state = 18},
  [5000] = {.lex_state = 72},
  [5001] = {.lex_state = 0},
  [5002] = {.lex_state = 72},
  [5003] = {.lex_state = 72},
  [5004] = {.lex_state = 0},
  [5005] = {.lex_state = 0},
  [5006] = {.lex_state = 0},
  [5007] = {.lex_state = 72},
  [5008] = {.lex_state = 0},
  [5009] = {.lex_state = 72},
  [5010] = {.lex_state = 11},
  [5011] = {.lex_state = 72},
  [5012] = {.lex_state = 72},
  [5013] = {.lex_state = 0},
  [5014] = {.lex_state = 0},
  [5015] = {.lex_state = 18},
  [5016] = {.lex_state = 72},
  [5017] = {.lex_state = 0},
  [5018] = {.lex_state = 18},
  [5019] = {.lex_state = 11},
  [5020] = {.lex_state = 18},
  [5021] = {.lex_state = 72},
  [5022] = {.lex_state = 72},
  [5023] = {.lex_state = 72},
  [5024] = {.lex_state = 8},
  [5025] = {.lex_state = 0},
  [5026] = {.lex_state = 18},
  [5027] = {.lex_state = 0},
  [5028] = {.lex_state = 18},
  [5029] = {.lex_state = 72},
  [5030] = {.lex_state = 0},
  [5031] = {.lex_state = 72},
  [5032] = {.lex_state = 0},
  [5033] = {.lex_state = 0},
  [5034] = {.lex_state = 72},
  [5035] = {.lex_state = 72},
  [5036] = {.lex_state = 0},
  [5037] = {.lex_state = 0},
  [5038] = {.lex_state = 7},
  [5039] = {.lex_state = 72},
  [5040] = {.lex_state = 0},
  [5041] = {.lex_state = 0},
  [5042] = {.lex_state = 72},
  [5043] = {.lex_state = 11},
  [5044] = {.lex_state = 0},
  [5045] = {.lex_state = 8},
  [5046] = {.lex_state = 72},
  [5047] = {.lex_state = 19},
  [5048] = {.lex_state = 0},
  [5049] = {.lex_state = 0},
  [5050] = {.lex_state = 0},
  [5051] = {.lex_state = 0},
  [5052] = {.lex_state = 72},
  [5053] = {.lex_state = 0},
  [5054] = {.lex_state = 72},
  [5055] = {.lex_state = 0},
  [5056] = {.lex_state = 11},
  [5057] = {.lex_state = 11},
  [5058] = {.lex_state = 0},
  [5059] = {.lex_state = 11},
  [5060] = {.lex_state = 0},
  [5061] = {.lex_state = 11},
  [5062] = {.lex_state = 0},
  [5063] = {.lex_state = 0},
  [5064] = {.lex_state = 11},
  [5065] = {.lex_state = 23},
  [5066] = {.lex_state = 0},
  [5067] = {.lex_state = 0},
  [5068] = {.lex_state = 72},
  [5069] = {.lex_state = 72},
  [5070] = {.lex_state = 72},
  [5071] = {.lex_state = 0},
  [5072] = {.lex_state = 72},
  [5073] = {.lex_state = 0},
  [5074] = {.lex_state = 72},
  [5075] = {.lex_state = 72},
  [5076] = {.lex_state = 17},
  [5077] = {.lex_state = 0},
  [5078] = {.lex_state = 11},
  [5079] = {.lex_state = 0},
  [5080] = {.lex_state = 7},
  [5081] = {.lex_state = 23, .external_lex_state = 8},
  [5082] = {.lex_state = 72},
  [5083] = {.lex_state = 7},
  [5084] = {.lex_state = 11},
  [5085] = {.lex_state = 0},
  [5086] = {.lex_state = 72},
  [5087] = {.lex_state = 72},
  [5088] = {.lex_state = 0},
  [5089] = {.lex_state = 72},
  [5090] = {.lex_state = 11},
  [5091] = {.lex_state = 72},
  [5092] = {.lex_state = 72},
  [5093] = {.lex_state = 18},
  [5094] = {.lex_state = 11},
  [5095] = {.lex_state = 0},
  [5096] = {.lex_state = 18},
  [5097] = {.lex_state = 0},
  [5098] = {.lex_state = 0},
  [5099] = {.lex_state = 18},
  [5100] = {.lex_state = 18},
  [5101] = {.lex_state = 18},
  [5102] = {.lex_state = 18},
  [5103] = {.lex_state = 11, .external_lex_state = 7},
  [5104] = {.lex_state = 0},
  [5105] = {.lex_state = 72},
  [5106] = {.lex_state = 0},
  [5107] = {.lex_state = 72},
  [5108] = {.lex_state = 0},
  [5109] = {.lex_state = 0},
  [5110] = {.lex_state = 72},
  [5111] = {.lex_state = 72},
  [5112] = {.lex_state = 0},
  [5113] = {.lex_state = 7},
  [5114] = {.lex_state = 0},
  [5115] = {.lex_state = 11},
  [5116] = {.lex_state = 0},
  [5117] = {.lex_state = 72},
  [5118] = {.lex_state = 72},
  [5119] = {.lex_state = 11},
  [5120] = {.lex_state = 0},
  [5121] = {.lex_state = 0},
  [5122] = {.lex_state = 18},
  [5123] = {.lex_state = 0},
  [5124] = {.lex_state = 72},
  [5125] = {.lex_state = 72},
  [5126] = {.lex_state = 0},
  [5127] = {.lex_state = 11},
  [5128] = {.lex_state = 0},
  [5129] = {.lex_state = 7},
  [5130] = {.lex_state = 72},
  [5131] = {.lex_state = 72},
  [5132] = {.lex_state = 72},
  [5133] = {.lex_state = 72},
  [5134] = {.lex_state = 11},
  [5135] = {.lex_state = 72},
  [5136] = {.lex_state = 0},
  [5137] = {.lex_state = 11},
  [5138] = {.lex_state = 0},
  [5139] = {.lex_state = 0},
  [5140] = {.lex_state = 0},
  [5141] = {.lex_state = 72},
  [5142] = {.lex_state = 72},
  [5143] = {.lex_state = 0},
  [5144] = {.lex_state = 11},
  [5145] = {.lex_state = 0},
  [5146] = {.lex_state = 0},
  [5147] = {.lex_state = 0},
  [5148] = {.lex_state = 72},
  [5149] = {.lex_state = 0},
  [5150] = {.lex_state = 72},
  [5151] = {.lex_state = 0},
  [5152] = {.lex_state = 0},
  [5153] = {.lex_state = 0},
  [5154] = {.lex_state = 11},
  [5155] = {.lex_state = 0},
  [5156] = {.lex_state = 164},
  [5157] = {.lex_state = 0},
  [5158] = {.lex_state = 72},
  [5159] = {.lex_state = 11},
  [5160] = {.lex_state = 72},
  [5161] = {.lex_state = 0},
  [5162] = {.lex_state = 0},
  [5163] = {.lex_state = 0},
  [5164] = {.lex_state = 0},
  [5165] = {.lex_state = 0},
  [5166] = {.lex_state = 0},
  [5167] = {.lex_state = 72},
  [5168] = {.lex_state = 0},
  [5169] = {.lex_state = 11},
  [5170] = {.lex_state = 72},
  [5171] = {.lex_state = 0},
  [5172] = {.lex_state = 0},
  [5173] = {.lex_state = 0},
  [5174] = {.lex_state = 0},
  [5175] = {.lex_state = 0},
  [5176] = {.lex_state = 0},
  [5177] = {.lex_state = 0},
  [5178] = {.lex_state = 0},
  [5179] = {.lex_state = 0},
  [5180] = {.lex_state = 18},
  [5181] = {.lex_state = 0},
  [5182] = {.lex_state = 0},
  [5183] = {.lex_state = 18},
  [5184] = {.lex_state = 72},
  [5185] = {.lex_state = 0},
  [5186] = {.lex_state = 19},
  [5187] = {.lex_state = 0},
  [5188] = {.lex_state = 7},
  [5189] = {.lex_state = 72},
  [5190] = {.lex_state = 7},
  [5191] = {.lex_state = 0},
  [5192] = {.lex_state = 72},
  [5193] = {.lex_state = 0},
  [5194] = {.lex_state = 0},
  [5195] = {.lex_state = 7},
  [5196] = {.lex_state = 0},
  [5197] = {.lex_state = 72},
  [5198] = {.lex_state = 0},
  [5199] = {.lex_state = 0},
  [5200] = {.lex_state = 0},
  [5201] = {.lex_state = 72},
  [5202] = {.lex_state = 11},
  [5203] = {.lex_state = 0},
  [5204] = {.lex_state = 19},
  [5205] = {.lex_state = 11},
  [5206] = {.lex_state = 72},
  [5207] = {.lex_state = 0},
  [5208] = {.lex_state = 0},
  [5209] = {.lex_state = 72},
  [5210] = {.lex_state = 0},
  [5211] = {.lex_state = 11},
  [5212] = {.lex_state = 0},
  [5213] = {.lex_state = 0},
  [5214] = {.lex_state = 72},
  [5215] = {.lex_state = 72},
  [5216] = {.lex_state = 11},
  [5217] = {.lex_state = 0},
  [5218] = {.lex_state = 72},
  [5219] = {.lex_state = 0},
  [5220] = {.lex_state = 7},
  [5221] = {.lex_state = 72},
  [5222] = {.lex_state = 0},
  [5223] = {.lex_state = 0},
  [5224] = {.lex_state = 0},
  [5225] = {.lex_state = 11},
  [5226] = {.lex_state = 72},
  [5227] = {.lex_state = 18},
  [5228] = {.lex_state = 72},
  [5229] = {.lex_state = 11},
  [5230] = {.lex_state = 72},
  [5231] = {.lex_state = 11},
  [5232] = {.lex_state = 0},
  [5233] = {.lex_state = 0},
  [5234] = {.lex_state = 7},
  [5235] = {.lex_state = 0},
  [5236] = {.lex_state = 72},
  [5237] = {.lex_state = 72},
  [5238] = {.lex_state = 11},
  [5239] = {.lex_state = 11},
  [5240] = {.lex_state = 11},
  [5241] = {.lex_state = 0},
  [5242] = {.lex_state = 0},
  [5243] = {.lex_state = 72},
  [5244] = {.lex_state = 72},
  [5245] = {.lex_state = 72},
  [5246] = {.lex_state = 0},
  [5247] = {.lex_state = 72},
  [5248] = {.lex_state = 18},
  [5249] = {.lex_state = 18},
  [5250] = {.lex_state = 18},
  [5251] = {.lex_state = 18},
  [5252] = {.lex_state = 18},
  [5253] = {.lex_state = 18},
  [5254] = {.lex_state = 0},
  [5255] = {.lex_state = 18},
  [5256] = {.lex_state = 19},
  [5257] = {.lex_state = 0},
  [5258] = {.lex_state = 0},
  [5259] = {.lex_state = 0},
  [5260] = {.lex_state = 18},
  [5261] = {.lex_state = 11},
  [5262] = {.lex_state = 0},
  [5263] = {.lex_state = 0},
  [5264] = {.lex_state = 72},
  [5265] = {.lex_state = 72},
  [5266] = {.lex_state = 0},
  [5267] = {.lex_state = 0},
  [5268] = {.lex_state = 0},
  [5269] = {.lex_state = 0},
  [5270] = {.lex_state = 72},
  [5271] = {.lex_state = 18},
  [5272] = {.lex_state = 18},
  [5273] = {.lex_state = 72},
  [5274] = {.lex_state = 11},
  [5275] = {.lex_state = 0},
  [5276] = {.lex_state = 72},
  [5277] = {.lex_state = 11},
  [5278] = {.lex_state = 11},
  [5279] = {.lex_state = 72},
  [5280] = {.lex_state = 0},
  [5281] = {.lex_state = 72},
  [5282] = {.lex_state = 0},
  [5283] = {.lex_state = 72},
  [5284] = {.lex_state = 72},
  [5285] = {.lex_state = 72},
  [5286] = {.lex_state = 0},
  [5287] = {.lex_state = 0},
  [5288] = {.lex_state = 72},
  [5289] = {.lex_state = 0},
  [5290] = {.lex_state = 0},
  [5291] = {.lex_state = 72},
  [5292] = {.lex_state = 0},
  [5293] = {.lex_state = 0},
  [5294] = {.lex_state = 0},
  [5295] = {.lex_state = 0},
  [5296] = {.lex_state = 0},
  [5297] = {.lex_state = 0},
  [5298] = {.lex_state = 0},
  [5299] = {.lex_state = 23},
  [5300] = {.lex_state = 0},
  [5301] = {.lex_state = 72},
  [5302] = {.lex_state = 72},
  [5303] = {.lex_state = 0},
  [5304] = {.lex_state = 0},
  [5305] = {.lex_state = 0},
  [5306] = {.lex_state = 0},
  [5307] = {.lex_state = 72},
  [5308] = {.lex_state = 72},
  [5309] = {.lex_state = 0},
  [5310] = {.lex_state = 72},
  [5311] = {.lex_state = 72},
  [5312] = {.lex_state = 0},
  [5313] = {.lex_state = 18},
  [5314] = {.lex_state = 11},
  [5315] = {.lex_state = 72},
  [5316] = {.lex_state = 17},
  [5317] = {.lex_state = 72},
  [5318] = {.lex_state = 72},
  [5319] = {.lex_state = 0},
  [5320] = {.lex_state = 11},
  [5321] = {.lex_state = 0},
  [5322] = {.lex_state = 72},
  [5323] = {.lex_state = 0},
  [5324] = {.lex_state = 11},
  [5325] = {.lex_state = 72},
  [5326] = {.lex_state = 11},
  [5327] = {.lex_state = 11},
  [5328] = {.lex_state = 72},
  [5329] = {.lex_state = 18},
  [5330] = {.lex_state = 0},
  [5331] = {.lex_state = 0},
  [5332] = {.lex_state = 72},
  [5333] = {.lex_state = 72},
  [5334] = {.lex_state = 72},
  [5335] = {.lex_state = 0},
  [5336] = {.lex_state = 0},
  [5337] = {.lex_state = 11},
  [5338] = {.lex_state = 72},
  [5339] = {.lex_state = 72},
  [5340] = {.lex_state = 72},
  [5341] = {.lex_state = 72},
  [5342] = {.lex_state = 18},
  [5343] = {.lex_state = 0},
  [5344] = {.lex_state = 0},
  [5345] = {.lex_state = 11},
  [5346] = {.lex_state = 72},
  [5347] = {.lex_state = 11},
  [5348] = {.lex_state = 0},
  [5349] = {.lex_state = 72},
  [5350] = {.lex_state = 72},
  [5351] = {.lex_state = 11},
  [5352] = {.lex_state = 0},
  [5353] = {.lex_state = 7},
  [5354] = {.lex_state = 72},
  [5355] = {.lex_state = 11},
  [5356] = {.lex_state = 72},
  [5357] = {.lex_state = 0},
  [5358] = {.lex_state = 0},
  [5359] = {.lex_state = 0},
  [5360] = {.lex_state = 0},
  [5361] = {.lex_state = 0},
  [5362] = {.lex_state = 72},
  [5363] = {.lex_state = 11},
  [5364] = {.lex_state = 72},
  [5365] = {.lex_state = 72},
  [5366] = {.lex_state = 11},
  [5367] = {.lex_state = 0},
  [5368] = {.lex_state = 7},
  [5369] = {.lex_state = 0},
  [5370] = {.lex_state = 7},
  [5371] = {.lex_state = 72},
  [5372] = {.lex_state = 18},
  [5373] = {.lex_state = 0},
  [5374] = {.lex_state = 0},
  [5375] = {.lex_state = 7},
  [5376] = {.lex_state = 0},
  [5377] = {.lex_state = 0},
  [5378] = {.lex_state = 0},
  [5379] = {.lex_state = 0},
  [5380] = {.lex_state = 18},
  [5381] = {.lex_state = 0},
  [5382] = {.lex_state = 11},
  [5383] = {.lex_state = 72},
  [5384] = {.lex_state = 7},
  [5385] = {.lex_state = 0},
  [5386] = {.lex_state = 7},
  [5387] = {.lex_state = 0},
  [5388] = {.lex_state = 7},
  [5389] = {.lex_state = 0},
  [5390] = {.lex_state = 7},
  [5391] = {.lex_state = 0},
  [5392] = {.lex_state = 7},
  [5393] = {.lex_state = 0},
  [5394] = {.lex_state = 18},
  [5395] = {.lex_state = 11},
  [5396] = {.lex_state = 11},
  [5397] = {.lex_state = 7},
  [5398] = {.lex_state = 0},
  [5399] = {.lex_state = 7},
  [5400] = {.lex_state = 72},
  [5401] = {.lex_state = 7},
  [5402] = {.lex_state = 72},
  [5403] = {.lex_state = 7},
  [5404] = {.lex_state = 72},
  [5405] = {.lex_state = 7},
  [5406] = {.lex_state = 0},
  [5407] = {.lex_state = 0},
  [5408] = {.lex_state = 72},
  [5409] = {.lex_state = 11},
  [5410] = {.lex_state = 0},
  [5411] = {.lex_state = 0},
  [5412] = {.lex_state = 0},
  [5413] = {.lex_state = 11},
  [5414] = {.lex_state = 0},
  [5415] = {.lex_state = 11},
  [5416] = {.lex_state = 11},
  [5417] = {.lex_state = 0},
  [5418] = {.lex_state = 11},
  [5419] = {.lex_state = 11},
  [5420] = {.lex_state = 0},
  [5421] = {.lex_state = 11},
  [5422] = {.lex_state = 11},
  [5423] = {.lex_state = 7},
  [5424] = {.lex_state = 72},
  [5425] = {.lex_state = 0},
  [5426] = {.lex_state = 0},
  [5427] = {.lex_state = 0},
  [5428] = {.lex_state = 11},
  [5429] = {.lex_state = 0},
  [5430] = {.lex_state = 11},
  [5431] = {.lex_state = 11},
  [5432] = {.lex_state = 0},
  [5433] = {.lex_state = 0},
  [5434] = {.lex_state = 18},
  [5435] = {.lex_state = 18},
  [5436] = {.lex_state = 72},
  [5437] = {.lex_state = 72},
  [5438] = {.lex_state = 11},
  [5439] = {.lex_state = 7},
  [5440] = {.lex_state = 72},
  [5441] = {.lex_state = 18},
  [5442] = {.lex_state = 72},
  [5443] = {.lex_state = 72},
  [5444] = {.lex_state = 0},
  [5445] = {.lex_state = 0},
  [5446] = {.lex_state = 72},
  [5447] = {.lex_state = 72},
  [5448] = {.lex_state = 72},
  [5449] = {.lex_state = 11},
  [5450] = {.lex_state = 72},
  [5451] = {.lex_state = 72},
  [5452] = {.lex_state = 72},
  [5453] = {.lex_state = 72},
  [5454] = {.lex_state = 72},
  [5455] = {.lex_state = 72},
  [5456] = {.lex_state = 11},
  [5457] = {.lex_state = 72},
  [5458] = {.lex_state = 11},
  [5459] = {.lex_state = 23, .external_lex_state = 9},
  [5460] = {.lex_state = 11},
  [5461] = {.lex_state = 72},
  [5462] = {.lex_state = 72},
  [5463] = {.lex_state = 11},
  [5464] = {.lex_state = 0},
  [5465] = {.lex_state = 11},
  [5466] = {.lex_state = 0},
  [5467] = {.lex_state = 11},
  [5468] = {.lex_state = 0},
  [5469] = {.lex_state = 11},
  [5470] = {.lex_state = 0},
  [5471] = {.lex_state = 11},
  [5472] = {.lex_state = 0},
  [5473] = {.lex_state = 0},
  [5474] = {.lex_state = 72},
  [5475] = {.lex_state = 72},
  [5476] = {.lex_state = 11},
  [5477] = {.lex_state = 72},
  [5478] = {.lex_state = 72},
  [5479] = {.lex_state = 0},
  [5480] = {.lex_state = 72},
  [5481] = {.lex_state = 72},
  [5482] = {.lex_state = 11},
  [5483] = {.lex_state = 11},
  [5484] = {.lex_state = 11},
  [5485] = {.lex_state = 0},
  [5486] = {.lex_state = 11},
  [5487] = {.lex_state = 11},
  [5488] = {.lex_state = 23},
  [5489] = {.lex_state = 0},
  [5490] = {.lex_state = 11},
  [5491] = {.lex_state = 72},
  [5492] = {.lex_state = 72},
  [5493] = {.lex_state = 11},
  [5494] = {.lex_state = 11},
  [5495] = {.lex_state = 0},
  [5496] = {.lex_state = 0},
  [5497] = {.lex_state = 0},
  [5498] = {.lex_state = 72},
  [5499] = {.lex_state = 72},
  [5500] = {.lex_state = 0},
  [5501] = {.lex_state = 0},
  [5502] = {.lex_state = 0},
  [5503] = {.lex_state = 0},
  [5504] = {.lex_state = 0},
  [5505] = {.lex_state = 72},
  [5506] = {.lex_state = 11},
  [5507] = {.lex_state = 0},
  [5508] = {.lex_state = 11},
  [5509] = {.lex_state = 11},
  [5510] = {.lex_state = 72},
  [5511] = {.lex_state = 72},
  [5512] = {.lex_state = 72},
  [5513] = {.lex_state = 0},
  [5514] = {.lex_state = 0},
  [5515] = {.lex_state = 0},
  [5516] = {.lex_state = 0},
  [5517] = {.lex_state = 11},
  [5518] = {.lex_state = 72},
  [5519] = {.lex_state = 0},
  [5520] = {.lex_state = 11},
  [5521] = {.lex_state = 11},
  [5522] = {.lex_state = 72},
  [5523] = {.lex_state = 11},
  [5524] = {.lex_state = 11},
  [5525] = {.lex_state = 0},
  [5526] = {.lex_state = 11},
  [5527] = {.lex_state = 0},
  [5528] = {.lex_state = 0},
  [5529] = {.lex_state = 0},
  [5530] = {.lex_state = 72},
  [5531] = {.lex_state = 0},
  [5532] = {.lex_state = 72},
  [5533] = {.lex_state = 0},
  [5534] = {.lex_state = 11},
  [5535] = {.lex_state = 11},
  [5536] = {.lex_state = 11},
  [5537] = {.lex_state = 23, .external_lex_state = 9},
  [5538] = {.lex_state = 72},
  [5539] = {.lex_state = 0},
  [5540] = {.lex_state = 11},
  [5541] = {.lex_state = 72},
  [5542] = {.lex_state = 72},
  [5543] = {.lex_state = 0},
  [5544] = {.lex_state = 11},
  [5545] = {.lex_state = 0},
  [5546] = {.lex_state = 11},
  [5547] = {.lex_state = 72},
  [5548] = {.lex_state = 0},
  [5549] = {.lex_state = 23, .external_lex_state = 9},
  [5550] = {.lex_state = 72},
  [5551] = {.lex_state = 0},
  [5552] = {.lex_state = 23},
  [5553] = {.lex_state = 11},
  [5554] = {.lex_state = 11},
  [5555] = {.lex_state = 72},
  [5556] = {.lex_state = 11},
  [5557] = {.lex_state = 11},
  [5558] = {.lex_state = 0},
  [5559] = {.lex_state = 72},
  [5560] = {.lex_state = 72},
  [5561] = {.lex_state = 72},
  [5562] = {.lex_state = 72},
  [5563] = {.lex_state = 72},
  [5564] = {.lex_state = 0},
  [5565] = {.lex_state = 0},
  [5566] = {.lex_state = 0},
  [5567] = {.lex_state = 0},
  [5568] = {.lex_state = 0},
  [5569] = {.lex_state = 0},
  [5570] = {.lex_state = 23, .external_lex_state = 9},
  [5571] = {.lex_state = 72},
  [5572] = {.lex_state = 0},
  [5573] = {.lex_state = 19},
  [5574] = {.lex_state = 0},
  [5575] = {.lex_state = 11},
  [5576] = {.lex_state = 0},
  [5577] = {.lex_state = 11},
  [5578] = {.lex_state = 72},
  [5579] = {.lex_state = 72},
  [5580] = {.lex_state = 19},
  [5581] = {.lex_state = 72},
  [5582] = {.lex_state = 72},
  [5583] = {.lex_state = 23},
  [5584] = {.lex_state = 11},
  [5585] = {.lex_state = 11},
  [5586] = {.lex_state = 11},
  [5587] = {.lex_state = 11},
  [5588] = {.lex_state = 0},
  [5589] = {.lex_state = 72},
  [5590] = {.lex_state = 0},
  [5591] = {.lex_state = 0},
  [5592] = {.lex_state = 72},
  [5593] = {.lex_state = 0},
  [5594] = {.lex_state = 0},
  [5595] = {.lex_state = 23, .external_lex_state = 9},
  [5596] = {.lex_state = 0},
  [5597] = {.lex_state = 11},
  [5598] = {.lex_state = 72},
  [5599] = {.lex_state = 72},
  [5600] = {.lex_state = 11},
  [5601] = {.lex_state = 72},
  [5602] = {.lex_state = 72},
  [5603] = {.lex_state = 0},
  [5604] = {.lex_state = 11},
  [5605] = {.lex_state = 11},
  [5606] = {.lex_state = 11},
  [5607] = {.lex_state = 72},
  [5608] = {.lex_state = 23, .external_lex_state = 9},
  [5609] = {.lex_state = 0},
  [5610] = {.lex_state = 11},
  [5611] = {.lex_state = 72},
  [5612] = {.lex_state = 0},
  [5613] = {.lex_state = 11},
  [5614] = {.lex_state = 11},
  [5615] = {.lex_state = 72},
  [5616] = {.lex_state = 11},
  [5617] = {.lex_state = 11},
  [5618] = {.lex_state = 11},
  [5619] = {.lex_state = 72},
  [5620] = {.lex_state = 72},
  [5621] = {.lex_state = 72},
  [5622] = {.lex_state = 0},
  [5623] = {.lex_state = 72},
  [5624] = {.lex_state = 0},
  [5625] = {.lex_state = 11},
  [5626] = {.lex_state = 0},
  [5627] = {.lex_state = 11},
  [5628] = {.lex_state = 11},
  [5629] = {.lex_state = 11},
  [5630] = {.lex_state = 0},
  [5631] = {.lex_state = 0},
  [5632] = {.lex_state = 11},
  [5633] = {.lex_state = 11},
  [5634] = {.lex_state = 11},
  [5635] = {.lex_state = 11},
  [5636] = {.lex_state = 11},
  [5637] = {.lex_state = 11},
  [5638] = {.lex_state = 11},
  [5639] = {.lex_state = 11},
  [5640] = {.lex_state = 11},
  [5641] = {.lex_state = 11},
  [5642] = {.lex_state = 11},
  [5643] = {.lex_state = 11},
  [5644] = {.lex_state = 0},
  [5645] = {.lex_state = 0},
  [5646] = {.lex_state = 72},
  [5647] = {.lex_state = 0},
  [5648] = {.lex_state = 0},
  [5649] = {.lex_state = 72},
  [5650] = {.lex_state = 0},
  [5651] = {.lex_state = 0},
  [5652] = {.lex_state = 0},
  [5653] = {.lex_state = 72},
  [5654] = {.lex_state = 72},
  [5655] = {.lex_state = 72},
  [5656] = {.lex_state = 11},
  [5657] = {.lex_state = 0},
  [5658] = {.lex_state = 11},
  [5659] = {.lex_state = 11},
  [5660] = {.lex_state = 11},
  [5661] = {.lex_state = 72},
  [5662] = {.lex_state = 72},
  [5663] = {.lex_state = 0},
  [5664] = {.lex_state = 72},
  [5665] = {.lex_state = 72},
  [5666] = {.lex_state = 72},
  [5667] = {.lex_state = 72},
  [5668] = {.lex_state = 23},
  [5669] = {.lex_state = 0, .external_lex_state = 10},
  [5670] = {.lex_state = 11},
  [5671] = {.lex_state = 23},
  [5672] = {.lex_state = 0},
  [5673] = {.lex_state = 0},
  [5674] = {.lex_state = 72},
  [5675] = {.lex_state = 0},
  [5676] = {.lex_state = 11},
  [5677] = {.lex_state = 0},
  [5678] = {.lex_state = 0},
  [5679] = {.lex_state = 0},
  [5680] = {.lex_state = 0},
  [5681] = {.lex_state = 72},
  [5682] = {.lex_state = 11},
  [5683] = {.lex_state = 0},
  [5684] = {.lex_state = 72},
  [5685] = {.lex_state = 11},
  [5686] = {.lex_state = 0},
  [5687] = {.lex_state = 23},
  [5688] = {.lex_state = 0},
  [5689] = {.lex_state = 72},
  [5690] = {.lex_state = 11},
  [5691] = {.lex_state = 11},
  [5692] = {.lex_state = 0},
  [5693] = {.lex_state = 0},
  [5694] = {.lex_state = 72},
  [5695] = {.lex_state = 0},
  [5696] = {.lex_state = 72},
  [5697] = {.lex_state = 72},
  [5698] = {.lex_state = 72},
  [5699] = {.lex_state = 72},
  [5700] = {.lex_state = 11},
  [5701] = {.lex_state = 11},
  [5702] = {.lex_state = 72},
  [5703] = {.lex_state = 11},
  [5704] = {.lex_state = 11},
  [5705] = {.lex_state = 23},
  [5706] = {.lex_state = 0},
  [5707] = {.lex_state = 0},
  [5708] = {.lex_state = 23},
  [5709] = {.lex_state = 72},
  [5710] = {.lex_state = 11},
  [5711] = {.lex_state = 0},
  [5712] = {.lex_state = 72},
  [5713] = {.lex_state = 72},
  [5714] = {.lex_state = 72},
  [5715] = {.lex_state = 23},
  [5716] = {.lex_state = 72},
  [5717] = {.lex_state = 72},
  [5718] = {.lex_state = 72},
  [5719] = {.lex_state = 0},
  [5720] = {.lex_state = 72},
  [5721] = {.lex_state = 72},
  [5722] = {.lex_state = 0},
  [5723] = {.lex_state = 11},
  [5724] = {.lex_state = 11},
  [5725] = {.lex_state = 11},
  [5726] = {.lex_state = 0},
  [5727] = {.lex_state = 0},
  [5728] = {.lex_state = 0},
  [5729] = {.lex_state = 0},
  [5730] = {.lex_state = 0},
  [5731] = {.lex_state = 0},
  [5732] = {.lex_state = 0},
  [5733] = {.lex_state = 0},
  [5734] = {.lex_state = 72},
  [5735] = {.lex_state = 23},
  [5736] = {.lex_state = 0},
  [5737] = {.lex_state = 23},
  [5738] = {.lex_state = 11},
  [5739] = {.lex_state = 72},
  [5740] = {.lex_state = 72},
  [5741] = {.lex_state = 11},
  [5742] = {.lex_state = 23},
  [5743] = {.lex_state = 72},
  [5744] = {.lex_state = 72},
  [5745] = {.lex_state = 11},
  [5746] = {.lex_state = 72},
  [5747] = {.lex_state = 72},
  [5748] = {.lex_state = 72},
  [5749] = {.lex_state = 11},
  [5750] = {.lex_state = 11},
  [5751] = {.lex_state = 72},
  [5752] = {.lex_state = 11},
  [5753] = {.lex_state = 11},
  [5754] = {.lex_state = 11},
  [5755] = {.lex_state = 0, .external_lex_state = 10},
  [5756] = {.lex_state = 11},
  [5757] = {.lex_state = 0},
  [5758] = {.lex_state = 0},
  [5759] = {.lex_state = 0},
  [5760] = {.lex_state = 11},
  [5761] = {.lex_state = 72},
  [5762] = {.lex_state = 0},
  [5763] = {.lex_state = 0},
  [5764] = {.lex_state = 0},
  [5765] = {.lex_state = 0},
  [5766] = {.lex_state = 0},
  [5767] = {.lex_state = 0},
  [5768] = {.lex_state = 0},
  [5769] = {.lex_state = 23},
  [5770] = {.lex_state = 0},
  [5771] = {.lex_state = 23},
  [5772] = {.lex_state = 72},
  [5773] = {.lex_state = 0},
  [5774] = {.lex_state = 0},
  [5775] = {.lex_state = 0},
  [5776] = {.lex_state = 0},
  [5777] = {.lex_state = 0},
  [5778] = {.lex_state = 0},
  [5779] = {.lex_state = 0},
  [5780] = {.lex_state = 0},
  [5781] = {.lex_state = 11},
  [5782] = {.lex_state = 0},
  [5783] = {.lex_state = 72},
  [5784] = {.lex_state = 0},
  [5785] = {.lex_state = 72},
  [5786] = {.lex_state = 72},
  [5787] = {.lex_state = 0},
  [5788] = {.lex_state = 72},
  [5789] = {.lex_state = 72},
  [5790] = {.lex_state = 72},
  [5791] = {.lex_state = 23},
  [5792] = {.lex_state = 11},
  [5793] = {.lex_state = 0},
  [5794] = {.lex_state = 72},
  [5795] = {.lex_state = 0},
  [5796] = {.lex_state = 72},
  [5797] = {.lex_state = 72},
  [5798] = {.lex_state = 72},
  [5799] = {.lex_state = 72},
  [5800] = {.lex_state = 0},
  [5801] = {.lex_state = 0},
  [5802] = {.lex_state = 0},
  [5803] = {.lex_state = 0},
  [5804] = {.lex_state = 0},
  [5805] = {.lex_state = 72},
  [5806] = {.lex_state = 23},
  [5807] = {.lex_state = 72},
  [5808] = {.lex_state = 0},
  [5809] = {.lex_state = 0},
  [5810] = {.lex_state = 72},
  [5811] = {.lex_state = 72},
  [5812] = {.lex_state = 72},
  [5813] = {.lex_state = 0},
  [5814] = {.lex_state = 11},
  [5815] = {.lex_state = 72},
  [5816] = {.lex_state = 72},
  [5817] = {.lex_state = 0},
  [5818] = {.lex_state = 11},
  [5819] = {.lex_state = 11},
  [5820] = {.lex_state = 11},
  [5821] = {.lex_state = 23},
  [5822] = {.lex_state = 72},
  [5823] = {.lex_state = 72},
  [5824] = {.lex_state = 72},
  [5825] = {.lex_state = 72},
  [5826] = {.lex_state = 0},
  [5827] = {.lex_state = 72},
  [5828] = {.lex_state = 0},
  [5829] = {.lex_state = 11},
  [5830] = {.lex_state = 11},
  [5831] = {.lex_state = 72},
  [5832] = {.lex_state = 0},
  [5833] = {.lex_state = 72},
  [5834] = {.lex_state = 11},
  [5835] = {.lex_state = 72},
  [5836] = {.lex_state = 72},
  [5837] = {.lex_state = 72},
  [5838] = {.lex_state = 72},
  [5839] = {.lex_state = 0},
  [5840] = {.lex_state = 72},
  [5841] = {.lex_state = 19},
  [5842] = {.lex_state = 11},
  [5843] = {.lex_state = 0},
  [5844] = {.lex_state = 19},
  [5845] = {.lex_state = 19},
  [5846] = {.lex_state = 0},
  [5847] = {.lex_state = 0},
  [5848] = {.lex_state = 11},
  [5849] = {.lex_state = 0},
  [5850] = {.lex_state = 72},
  [5851] = {.lex_state = 19},
  [5852] = {.lex_state = 11},
  [5853] = {.lex_state = 11},
  [5854] = {.lex_state = 11},
  [5855] = {.lex_state = 72},
  [5856] = {.lex_state = 23, .external_lex_state = 9},
  [5857] = {.lex_state = 72},
  [5858] = {.lex_state = 72},
  [5859] = {.lex_state = 0},
  [5860] = {.lex_state = 72},
  [5861] = {.lex_state = 72},
  [5862] = {.lex_state = 72},
  [5863] = {.lex_state = 72},
  [5864] = {.lex_state = 72},
  [5865] = {.lex_state = 0},
  [5866] = {.lex_state = 72},
  [5867] = {.lex_state = 0},
  [5868] = {.lex_state = 72},
  [5869] = {.lex_state = 0},
  [5870] = {.lex_state = 72},
  [5871] = {.lex_state = 11},
  [5872] = {.lex_state = 72},
  [5873] = {.lex_state = 0},
  [5874] = {.lex_state = 72},
  [5875] = {.lex_state = 72},
  [5876] = {.lex_state = 72},
  [5877] = {.lex_state = 11},
  [5878] = {.lex_state = 72},
  [5879] = {.lex_state = 11},
  [5880] = {.lex_state = 72},
  [5881] = {.lex_state = 11},
  [5882] = {.lex_state = 0},
  [5883] = {.lex_state = 72},
  [5884] = {.lex_state = 11},
  [5885] = {.lex_state = 72},
  [5886] = {.lex_state = 72},
  [5887] = {.lex_state = 72},
  [5888] = {.lex_state = 0},
  [5889] = {.lex_state = 11},
  [5890] = {.lex_state = 0},
  [5891] = {.lex_state = 11},
  [5892] = {.lex_state = 11},
  [5893] = {.lex_state = 11},
  [5894] = {.lex_state = 72},
  [5895] = {.lex_state = 11},
  [5896] = {.lex_state = 72},
  [5897] = {.lex_state = 72},
  [5898] = {.lex_state = 72},
  [5899] = {.lex_state = 72},
  [5900] = {.lex_state = 11},
  [5901] = {.lex_state = 0},
  [5902] = {.lex_state = 11},
  [5903] = {.lex_state = 11},
  [5904] = {.lex_state = 0},
  [5905] = {.lex_state = 11},
  [5906] = {.lex_state = 72},
  [5907] = {.lex_state = 11},
  [5908] = {.lex_state = 11},
  [5909] = {.lex_state = 11},
  [5910] = {.lex_state = 72},
  [5911] = {.lex_state = 11},
  [5912] = {.lex_state = 72},
  [5913] = {.lex_state = 11},
  [5914] = {.lex_state = 72},
  [5915] = {.lex_state = 72},
  [5916] = {.lex_state = 11},
  [5917] = {.lex_state = 72},
  [5918] = {.lex_state = 11},
  [5919] = {.lex_state = 0},
  [5920] = {.lex_state = 11},
  [5921] = {.lex_state = 72},
  [5922] = {.lex_state = 0},
  [5923] = {.lex_state = 11},
  [5924] = {.lex_state = 11},
  [5925] = {.lex_state = 72},
  [5926] = {.lex_state = 0},
  [5927] = {.lex_state = 11},
  [5928] = {.lex_state = 11},
  [5929] = {.lex_state = 11},
  [5930] = {.lex_state = 72},
  [5931] = {.lex_state = 72},
  [5932] = {.lex_state = 11},
  [5933] = {.lex_state = 0},
  [5934] = {.lex_state = 72},
  [5935] = {.lex_state = 72},
  [5936] = {.lex_state = 11},
  [5937] = {.lex_state = 11},
  [5938] = {.lex_state = 72},
  [5939] = {.lex_state = 72},
  [5940] = {.lex_state = 0},
  [5941] = {.lex_state = 72},
  [5942] = {.lex_state = 72},
  [5943] = {.lex_state = 72},
  [5944] = {.lex_state = 72},
  [5945] = {.lex_state = 0},
  [5946] = {.lex_state = 0},
  [5947] = {.lex_state = 11},
  [5948] = {.lex_state = 11},
  [5949] = {.lex_state = 72},
  [5950] = {.lex_state = 72},
  [5951] = {.lex_state = 11},
  [5952] = {.lex_state = 72},
  [5953] = {.lex_state = 0},
  [5954] = {.lex_state = 72},
  [5955] = {.lex_state = 72},
  [5956] = {.lex_state = 72},
  [5957] = {.lex_state = 72},
  [5958] = {.lex_state = 19},
  [5959] = {.lex_state = 0},
  [5960] = {.lex_state = 0},
  [5961] = {.lex_state = 0},
  [5962] = {.lex_state = 0},
  [5963] = {.lex_state = 0},
  [5964] = {.lex_state = 0},
  [5965] = {.lex_state = 0},
  [5966] = {.lex_state = 0},
  [5967] = {.lex_state = 0},
  [5968] = {.lex_state = 0},
  [5969] = {.lex_state = 0},
  [5970] = {.lex_state = 0},
  [5971] = {.lex_state = 0},
  [5972] = {.lex_state = 11},
  [5973] = {.lex_state = 0},
  [5974] = {.lex_state = 11},
  [5975] = {.lex_state = 0},
  [5976] = {.lex_state = 0},
  [5977] = {.lex_state = 11},
  [5978] = {.lex_state = 19},
  [5979] = {.lex_state = 0},
  [5980] = {.lex_state = 0, .external_lex_state = 11},
  [5981] = {.lex_state = 0, .external_lex_state = 12},
  [5982] = {.lex_state = 0},
  [5983] = {.lex_state = 11},
  [5984] = {.lex_state = 11},
  [5985] = {.lex_state = 0},
  [5986] = {.lex_state = 11},
  [5987] = {.lex_state = 0},
  [5988] = {.lex_state = 0},
  [5989] = {.lex_state = 0},
  [5990] = {.lex_state = 11},
  [5991] = {.lex_state = 0},
  [5992] = {.lex_state = 0},
  [5993] = {.lex_state = 11},
  [5994] = {.lex_state = 0},
  [5995] = {.lex_state = 0},
  [5996] = {.lex_state = 11},
  [5997] = {.lex_state = 0},
  [5998] = {.lex_state = 0},
  [5999] = {.lex_state = 0},
  [6000] = {.lex_state = 23},
  [6001] = {.lex_state = 0},
  [6002] = {.lex_state = 0, .external_lex_state = 11},
  [6003] = {.lex_state = 0},
  [6004] = {.lex_state = 0},
  [6005] = {.lex_state = 0, .external_lex_state = 11},
  [6006] = {.lex_state = 0, .external_lex_state = 13},
  [6007] = {.lex_state = 23},
  [6008] = {.lex_state = 0},
  [6009] = {.lex_state = 0},
  [6010] = {.lex_state = 0},
  [6011] = {.lex_state = 0},
  [6012] = {.lex_state = 0},
  [6013] = {.lex_state = 0},
  [6014] = {.lex_state = 11},
  [6015] = {.lex_state = 11},
  [6016] = {.lex_state = 0},
  [6017] = {.lex_state = 0},
  [6018] = {.lex_state = 0},
  [6019] = {.lex_state = 0},
  [6020] = {.lex_state = 0},
  [6021] = {.lex_state = 19},
  [6022] = {.lex_state = 11},
  [6023] = {.lex_state = 0},
  [6024] = {.lex_state = 0},
  [6025] = {.lex_state = 0},
  [6026] = {.lex_state = 0},
  [6027] = {.lex_state = 0},
  [6028] = {.lex_state = 0},
  [6029] = {.lex_state = 0},
  [6030] = {.lex_state = 0},
  [6031] = {.lex_state = 0},
  [6032] = {.lex_state = 0},
  [6033] = {.lex_state = 11},
  [6034] = {.lex_state = 11},
  [6035] = {.lex_state = 0},
  [6036] = {.lex_state = 11},
  [6037] = {.lex_state = 72},
  [6038] = {.lex_state = 0},
  [6039] = {.lex_state = 0},
  [6040] = {.lex_state = 0},
  [6041] = {.lex_state = 0},
  [6042] = {.lex_state = 11},
  [6043] = {.lex_state = 19},
  [6044] = {.lex_state = 0},
  [6045] = {.lex_state = 0},
  [6046] = {.lex_state = 0},
  [6047] = {.lex_state = 19},
  [6048] = {.lex_state = 0},
  [6049] = {.lex_state = 0, .external_lex_state = 11},
  [6050] = {.lex_state = 11},
  [6051] = {.lex_state = 0, .external_lex_state = 11},
  [6052] = {.lex_state = 0},
  [6053] = {.lex_state = 19},
  [6054] = {.lex_state = 11},
  [6055] = {.lex_state = 0},
  [6056] = {.lex_state = 11},
  [6057] = {.lex_state = 0},
  [6058] = {.lex_state = 0},
  [6059] = {.lex_state = 0},
  [6060] = {.lex_state = 11},
  [6061] = {.lex_state = 0},
  [6062] = {.lex_state = 0},
  [6063] = {.lex_state = 19},
  [6064] = {.lex_state = 0},
  [6065] = {.lex_state = 11},
  [6066] = {.lex_state = 0},
  [6067] = {.lex_state = 73},
  [6068] = {.lex_state = 11},
  [6069] = {.lex_state = 0},
  [6070] = {.lex_state = 0},
  [6071] = {.lex_state = 0},
  [6072] = {.lex_state = 11},
  [6073] = {.lex_state = 0},
  [6074] = {.lex_state = 0},
  [6075] = {.lex_state = 0},
  [6076] = {.lex_state = 0},
  [6077] = {.lex_state = 0},
  [6078] = {.lex_state = 72},
  [6079] = {.lex_state = 0},
  [6080] = {.lex_state = 0},
  [6081] = {.lex_state = 0},
  [6082] = {.lex_state = 11},
  [6083] = {.lex_state = 0},
  [6084] = {.lex_state = 0},
  [6085] = {.lex_state = 0},
  [6086] = {.lex_state = 0},
  [6087] = {.lex_state = 0},
  [6088] = {.lex_state = 72},
  [6089] = {.lex_state = 0},
  [6090] = {.lex_state = 11},
  [6091] = {.lex_state = 0},
  [6092] = {.lex_state = 19},
  [6093] = {.lex_state = 11},
  [6094] = {.lex_state = 0},
  [6095] = {.lex_state = 0},
  [6096] = {.lex_state = 0},
  [6097] = {.lex_state = 0},
  [6098] = {.lex_state = 11},
  [6099] = {.lex_state = 19},
  [6100] = {.lex_state = 11},
  [6101] = {.lex_state = 0, .external_lex_state = 14},
  [6102] = {.lex_state = 11},
  [6103] = {.lex_state = 0},
  [6104] = {.lex_state = 0},
  [6105] = {.lex_state = 0},
  [6106] = {.lex_state = 0},
  [6107] = {.lex_state = 11},
  [6108] = {.lex_state = 0},
  [6109] = {.lex_state = 0},
  [6110] = {.lex_state = 0, .external_lex_state = 11},
  [6111] = {.lex_state = 0, .external_lex_state = 11},
  [6112] = {.lex_state = 0},
  [6113] = {.lex_state = 19},
  [6114] = {.lex_state = 0},
  [6115] = {.lex_state = 11},
  [6116] = {.lex_state = 11},
  [6117] = {.lex_state = 11},
  [6118] = {.lex_state = 11},
  [6119] = {.lex_state = 11},
  [6120] = {.lex_state = 72},
  [6121] = {.lex_state = 0},
  [6122] = {.lex_state = 0},
  [6123] = {.lex_state = 11},
  [6124] = {.lex_state = 0},
  [6125] = {.lex_state = 0},
  [6126] = {.lex_state = 19},
  [6127] = {.lex_state = 11},
  [6128] = {.lex_state = 0},
  [6129] = {.lex_state = 0},
  [6130] = {.lex_state = 0},
  [6131] = {.lex_state = 0},
  [6132] = {.lex_state = 0},
  [6133] = {.lex_state = 0},
  [6134] = {.lex_state = 0},
  [6135] = {.lex_state = 0},
  [6136] = {.lex_state = 0},
  [6137] = {.lex_state = 11},
  [6138] = {.lex_state = 11},
  [6139] = {.lex_state = 72},
  [6140] = {.lex_state = 0, .external_lex_state = 13},
  [6141] = {.lex_state = 23},
  [6142] = {.lex_state = 11},
  [6143] = {.lex_state = 11},
  [6144] = {.lex_state = 0, .external_lex_state = 13},
  [6145] = {.lex_state = 0},
  [6146] = {.lex_state = 0},
  [6147] = {.lex_state = 23},
  [6148] = {.lex_state = 72},
  [6149] = {.lex_state = 0},
  [6150] = {.lex_state = 11},
  [6151] = {.lex_state = 11},
  [6152] = {.lex_state = 0},
  [6153] = {.lex_state = 11},
  [6154] = {.lex_state = 72},
  [6155] = {.lex_state = 19},
  [6156] = {.lex_state = 0},
  [6157] = {.lex_state = 11},
  [6158] = {.lex_state = 0},
  [6159] = {.lex_state = 0},
  [6160] = {.lex_state = 11},
  [6161] = {.lex_state = 19},
  [6162] = {.lex_state = 0},
  [6163] = {.lex_state = 0},
  [6164] = {.lex_state = 0},
  [6165] = {.lex_state = 0},
  [6166] = {.lex_state = 0},
  [6167] = {.lex_state = 0},
  [6168] = {.lex_state = 0, .external_lex_state = 13},
  [6169] = {.lex_state = 0},
  [6170] = {.lex_state = 0, .external_lex_state = 14},
  [6171] = {.lex_state = 0},
  [6172] = {.lex_state = 0},
  [6173] = {.lex_state = 11},
  [6174] = {.lex_state = 0},
  [6175] = {.lex_state = 0},
  [6176] = {.lex_state = 11},
  [6177] = {.lex_state = 0},
  [6178] = {.lex_state = 0},
  [6179] = {.lex_state = 0},
  [6180] = {.lex_state = 0},
  [6181] = {.lex_state = 0},
  [6182] = {.lex_state = 11},
  [6183] = {.lex_state = 11},
  [6184] = {.lex_state = 11},
  [6185] = {.lex_state = 11},
  [6186] = {.lex_state = 0},
  [6187] = {.lex_state = 0},
  [6188] = {.lex_state = 0},
  [6189] = {.lex_state = 11},
  [6190] = {.lex_state = 0},
  [6191] = {.lex_state = 0},
  [6192] = {.lex_state = 0},
  [6193] = {.lex_state = 11},
  [6194] = {.lex_state = 0},
  [6195] = {.lex_state = 11},
  [6196] = {.lex_state = 0},
  [6197] = {.lex_state = 0},
  [6198] = {.lex_state = 11},
  [6199] = {.lex_state = 11},
  [6200] = {.lex_state = 0},
  [6201] = {.lex_state = 0},
  [6202] = {.lex_state = 11},
  [6203] = {.lex_state = 0},
  [6204] = {.lex_state = 0},
  [6205] = {.lex_state = 0},
  [6206] = {.lex_state = 0},
  [6207] = {.lex_state = 0},
  [6208] = {.lex_state = 0},
  [6209] = {.lex_state = 0},
  [6210] = {.lex_state = 0, .external_lex_state = 14},
  [6211] = {.lex_state = 0},
  [6212] = {.lex_state = 0},
  [6213] = {.lex_state = 0},
  [6214] = {.lex_state = 0},
  [6215] = {.lex_state = 0},
  [6216] = {.lex_state = 0},
  [6217] = {.lex_state = 0},
  [6218] = {.lex_state = 0},
  [6219] = {.lex_state = 0},
  [6220] = {.lex_state = 0},
  [6221] = {.lex_state = 0},
  [6222] = {.lex_state = 0},
  [6223] = {.lex_state = 0},
  [6224] = {.lex_state = 11},
  [6225] = {.lex_state = 0},
  [6226] = {.lex_state = 0},
  [6227] = {.lex_state = 0},
  [6228] = {.lex_state = 11},
  [6229] = {.lex_state = 0},
  [6230] = {.lex_state = 0},
  [6231] = {.lex_state = 11},
  [6232] = {.lex_state = 19},
  [6233] = {.lex_state = 0},
  [6234] = {.lex_state = 0},
  [6235] = {.lex_state = 0},
  [6236] = {.lex_state = 11},
  [6237] = {.lex_state = 11},
  [6238] = {.lex_state = 0},
  [6239] = {.lex_state = 0},
  [6240] = {.lex_state = 0},
  [6241] = {.lex_state = 11},
  [6242] = {.lex_state = 11},
  [6243] = {.lex_state = 0},
  [6244] = {.lex_state = 0},
  [6245] = {.lex_state = 19},
  [6246] = {.lex_state = 0},
  [6247] = {.lex_state = 11},
  [6248] = {.lex_state = 0},
  [6249] = {.lex_state = 0},
  [6250] = {.lex_state = 0},
  [6251] = {.lex_state = 72},
  [6252] = {.lex_state = 11},
  [6253] = {.lex_state = 0},
  [6254] = {.lex_state = 0},
  [6255] = {.lex_state = 0},
  [6256] = {.lex_state = 0},
  [6257] = {.lex_state = 11},
  [6258] = {.lex_state = 72},
  [6259] = {.lex_state = 0},
  [6260] = {.lex_state = 0, .external_lex_state = 14},
  [6261] = {.lex_state = 0},
  [6262] = {.lex_state = 19},
  [6263] = {.lex_state = 0},
  [6264] = {.lex_state = 0},
  [6265] = {.lex_state = 0},
  [6266] = {.lex_state = 0},
  [6267] = {.lex_state = 0},
  [6268] = {.lex_state = 0},
  [6269] = {.lex_state = 0, .external_lex_state = 13},
  [6270] = {.lex_state = 0},
  [6271] = {.lex_state = 0},
  [6272] = {.lex_state = 0},
  [6273] = {.lex_state = 0},
  [6274] = {.lex_state = 0},
  [6275] = {.lex_state = 0},
  [6276] = {.lex_state = 11},
  [6277] = {.lex_state = 0},
  [6278] = {.lex_state = 19},
  [6279] = {.lex_state = 0},
  [6280] = {.lex_state = 0, .external_lex_state = 14},
  [6281] = {.lex_state = 0},
  [6282] = {.lex_state = 11},
  [6283] = {.lex_state = 0},
  [6284] = {.lex_state = 0},
  [6285] = {.lex_state = 0, .external_lex_state = 14},
  [6286] = {.lex_state = 0},
  [6287] = {.lex_state = 19},
  [6288] = {.lex_state = 0},
  [6289] = {.lex_state = 0},
  [6290] = {.lex_state = 0},
  [6291] = {.lex_state = 0},
  [6292] = {.lex_state = 0},
  [6293] = {.lex_state = 0},
  [6294] = {.lex_state = 0},
  [6295] = {.lex_state = 0},
  [6296] = {.lex_state = 0},
  [6297] = {.lex_state = 19},
  [6298] = {.lex_state = 11},
  [6299] = {.lex_state = 0},
  [6300] = {.lex_state = 19},
  [6301] = {.lex_state = 0},
  [6302] = {.lex_state = 11},
  [6303] = {.lex_state = 0},
  [6304] = {.lex_state = 11},
  [6305] = {.lex_state = 0},
  [6306] = {.lex_state = 11},
  [6307] = {.lex_state = 0},
  [6308] = {.lex_state = 19},
  [6309] = {.lex_state = 19},
  [6310] = {.lex_state = 11},
  [6311] = {.lex_state = 23},
  [6312] = {.lex_state = 0},
  [6313] = {.lex_state = 11},
  [6314] = {.lex_state = 19},
  [6315] = {.lex_state = 11},
  [6316] = {.lex_state = 0},
  [6317] = {.lex_state = 11},
  [6318] = {.lex_state = 11},
  [6319] = {.lex_state = 11},
  [6320] = {.lex_state = 0},
  [6321] = {.lex_state = 11},
  [6322] = {.lex_state = 72},
  [6323] = {.lex_state = 19},
  [6324] = {.lex_state = 11},
  [6325] = {.lex_state = 0},
  [6326] = {.lex_state = 11},
  [6327] = {.lex_state = 11},
  [6328] = {.lex_state = 11},
  [6329] = {.lex_state = 11},
  [6330] = {.lex_state = 0},
  [6331] = {.lex_state = 11},
  [6332] = {.lex_state = 0},
  [6333] = {.lex_state = 0},
  [6334] = {.lex_state = 19},
  [6335] = {.lex_state = 11},
  [6336] = {.lex_state = 0},
  [6337] = {.lex_state = 0},
  [6338] = {.lex_state = 0},
  [6339] = {.lex_state = 19},
  [6340] = {.lex_state = 11},
  [6341] = {.lex_state = 0},
  [6342] = {.lex_state = 72},
  [6343] = {.lex_state = 19},
  [6344] = {.lex_state = 11},
  [6345] = {.lex_state = 11},
  [6346] = {.lex_state = 11},
  [6347] = {.lex_state = 0},
  [6348] = {.lex_state = 19},
  [6349] = {.lex_state = 11},
  [6350] = {.lex_state = 11},
  [6351] = {.lex_state = 0},
  [6352] = {.lex_state = 0},
  [6353] = {.lex_state = 0},
  [6354] = {.lex_state = 0},
  [6355] = {.lex_state = 0},
  [6356] = {.lex_state = 0},
  [6357] = {.lex_state = 19},
  [6358] = {.lex_state = 0},
  [6359] = {.lex_state = 0},
  [6360] = {.lex_state = 0},
  [6361] = {.lex_state = 0},
  [6362] = {.lex_state = 0},
  [6363] = {.lex_state = 19},
  [6364] = {.lex_state = 0},
  [6365] = {.lex_state = 0},
  [6366] = {.lex_state = 0},
  [6367] = {.lex_state = 19},
  [6368] = {.lex_state = 72},
  [6369] = {.lex_state = 0},
  [6370] = {.lex_state = 0},
  [6371] = {.lex_state = 19},
  [6372] = {.lex_state = 11},
  [6373] = {.lex_state = 0},
  [6374] = {.lex_state = 0},
  [6375] = {.lex_state = 0},
  [6376] = {.lex_state = 0},
  [6377] = {.lex_state = 0},
  [6378] = {.lex_state = 72},
  [6379] = {.lex_state = 0},
  [6380] = {.lex_state = 0},
  [6381] = {.lex_state = 0},
  [6382] = {.lex_state = 0},
  [6383] = {.lex_state = 19},
  [6384] = {.lex_state = 19},
  [6385] = {.lex_state = 0},
  [6386] = {.lex_state = 0},
  [6387] = {.lex_state = 0},
  [6388] = {.lex_state = 0},
  [6389] = {.lex_state = 0},
  [6390] = {.lex_state = 19},
  [6391] = {.lex_state = 19},
  [6392] = {.lex_state = 0},
  [6393] = {.lex_state = 0},
  [6394] = {.lex_state = 0},
  [6395] = {.lex_state = 23},
  [6396] = {.lex_state = 11},
  [6397] = {.lex_state = 0},
  [6398] = {.lex_state = 0},
  [6399] = {.lex_state = 0},
  [6400] = {.lex_state = 0},
  [6401] = {.lex_state = 0, .external_lex_state = 15},
  [6402] = {.lex_state = 0},
  [6403] = {.lex_state = 11},
  [6404] = {.lex_state = 11},
  [6405] = {.lex_state = 19},
  [6406] = {.lex_state = 19},
  [6407] = {.lex_state = 0},
  [6408] = {.lex_state = 0},
  [6409] = {.lex_state = 0},
  [6410] = {.lex_state = 19},
  [6411] = {.lex_state = 11},
  [6412] = {.lex_state = 19},
  [6413] = {.lex_state = 0},
  [6414] = {.lex_state = 0},
  [6415] = {.lex_state = 0},
  [6416] = {.lex_state = 0},
  [6417] = {.lex_state = 19},
  [6418] = {.lex_state = 0},
  [6419] = {.lex_state = 0, .external_lex_state = 11},
  [6420] = {.lex_state = 0},
  [6421] = {.lex_state = 0},
  [6422] = {.lex_state = 11},
  [6423] = {.lex_state = 0},
  [6424] = {.lex_state = 72},
  [6425] = {.lex_state = 19},
  [6426] = {.lex_state = 0},
  [6427] = {.lex_state = 19},
  [6428] = {.lex_state = 11},
  [6429] = {.lex_state = 11},
  [6430] = {.lex_state = 0},
  [6431] = {.lex_state = 19},
  [6432] = {.lex_state = 0},
  [6433] = {.lex_state = 19},
  [6434] = {.lex_state = 19},
  [6435] = {.lex_state = 19},
  [6436] = {.lex_state = 11},
  [6437] = {.lex_state = 11},
  [6438] = {.lex_state = 11},
  [6439] = {.lex_state = 11},
  [6440] = {.lex_state = 0},
  [6441] = {.lex_state = 11},
  [6442] = {.lex_state = 11},
  [6443] = {.lex_state = 11},
  [6444] = {.lex_state = 11},
  [6445] = {.lex_state = 11},
  [6446] = {.lex_state = 11},
  [6447] = {.lex_state = 11},
  [6448] = {.lex_state = 0},
  [6449] = {.lex_state = 0},
  [6450] = {.lex_state = 0, .external_lex_state = 13},
  [6451] = {.lex_state = 11},
  [6452] = {.lex_state = 72},
  [6453] = {.lex_state = 23},
  [6454] = {.lex_state = 11},
  [6455] = {.lex_state = 23},
  [6456] = {.lex_state = 72},
  [6457] = {.lex_state = 0},
  [6458] = {.lex_state = 11},
  [6459] = {.lex_state = 0},
  [6460] = {.lex_state = 0},
  [6461] = {.lex_state = 11},
  [6462] = {.lex_state = 0},
  [6463] = {.lex_state = 0},
  [6464] = {.lex_state = 0},
  [6465] = {.lex_state = 0},
  [6466] = {.lex_state = 0},
  [6467] = {.lex_state = 11},
  [6468] = {.lex_state = 11},
  [6469] = {.lex_state = 0},
  [6470] = {.lex_state = 0},
  [6471] = {.lex_state = 11},
  [6472] = {.lex_state = 0},
  [6473] = {.lex_state = 0},
  [6474] = {.lex_state = 11},
  [6475] = {.lex_state = 0},
  [6476] = {.lex_state = 72},
  [6477] = {.lex_state = 0},
  [6478] = {.lex_state = 11},
  [6479] = {.lex_state = 11},
  [6480] = {.lex_state = 0},
  [6481] = {.lex_state = 11},
  [6482] = {.lex_state = 11},
  [6483] = {.lex_state = 11},
  [6484] = {.lex_state = 0},
  [6485] = {.lex_state = 11},
  [6486] = {.lex_state = 0},
  [6487] = {.lex_state = 11},
  [6488] = {.lex_state = 0},
  [6489] = {.lex_state = 0},
  [6490] = {.lex_state = 0},
  [6491] = {.lex_state = 11},
  [6492] = {.lex_state = 0},
  [6493] = {.lex_state = 0},
  [6494] = {.lex_state = 0},
  [6495] = {.lex_state = 0},
  [6496] = {.lex_state = 0},
  [6497] = {.lex_state = 19},
  [6498] = {(TSStateId)(-1),},
  [6499] = {(TSStateId)(-1),},
  [6500] = {(TSStateId)(-1),},
  [6501] = {(TSStateId)(-1),},
  [6502] = {(TSStateId)(-1),},
  [6503] = {(TSStateId)(-1),},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [STATE(0)] = {
    [sym_line_comment] = STATE(0),
    [sym_block_comment] = STATE(0),
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_macro] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_EQ_GT] = ACTIONS(1),
    [anon_sym_macro_rules] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_DOLLAR] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_QMARK] = ACTIONS(1),
    [anon_sym_block] = ACTIONS(1),
    [anon_sym_expr] = ACTIONS(1),
    [anon_sym_expr_2021] = ACTIONS(1),
    [anon_sym_ident] = ACTIONS(1),
    [anon_sym_item] = ACTIONS(1),
    [anon_sym_lifetime] = ACTIONS(1),
    [anon_sym_literal] = ACTIONS(1),
    [anon_sym_meta] = ACTIONS(1),
    [anon_sym_pat] = ACTIONS(1),
    [anon_sym_pat_param] = ACTIONS(1),
    [anon_sym_path] = ACTIONS(1),
    [anon_sym_stmt] = ACTIONS(1),
    [anon_sym_tt] = ACTIONS(1),
    [anon_sym_ty] = ACTIONS(1),
    [anon_sym_vis] = ACTIONS(1),
    [anon_sym_u8] = ACTIONS(1),
    [anon_sym_i8] = ACTIONS(1),
    [anon_sym_u16] = ACTIONS(1),
    [anon_sym_i16] = ACTIONS(1),
    [anon_sym_u32] = ACTIONS(1),
    [anon_sym_i32] = ACTIONS(1),
    [anon_sym_u64] = ACTIONS(1),
    [anon_sym_i64] = ACTIONS(1),
    [anon_sym_u128] = ACTIONS(1),
    [anon_sym_i128] = ACTIONS(1),
    [anon_sym_isize] = ACTIONS(1),
    [anon_sym_usize] = ACTIONS(1),
    [anon_sym_f32] = ACTIONS(1),
    [anon_sym_f64] = ACTIONS(1),
    [anon_sym_bool] = ACTIONS(1),
    [anon_sym_str] = ACTIONS(1),
    [anon_sym_char] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_PLUS_EQ] = ACTIONS(1),
    [anon_sym_DASH_EQ] = ACTIONS(1),
    [anon_sym_STAR_EQ] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [anon_sym_PERCENT_EQ] = ACTIONS(1),
    [anon_sym_CARET_EQ] = ACTIONS(1),
    [anon_sym_AMP_EQ] = ACTIONS(1),
    [anon_sym_PIPE_EQ] = ACTIONS(1),
    [anon_sym_LT_LT_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym__] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_DOT_DOT] = ACTIONS(1),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1),
    [anon_sym_COLON_COLON] = ACTIONS(1),
    [anon_sym_DASH_GT] = ACTIONS(1),
    [anon_sym_POUND] = ACTIONS(1),
    [anon_sym_SQUOTE] = ACTIONS(1),
    [anon_sym_as] = ACTIONS(1),
    [anon_sym_async] = ACTIONS(1),
    [anon_sym_auto] = ACTIONS(1),
    [anon_sym_await] = ACTIONS(1),
    [anon_sym_become] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_const] = ACTIONS(1),
    [anon_sym_continue] = ACTIONS(1),
    [anon_sym_default] = ACTIONS(1),
    [anon_sym_enum] = ACTIONS(1),
    [anon_sym_fn] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_gen] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_impl] = ACTIONS(1),
    [anon_sym_let] = ACTIONS(1),
    [anon_sym_loop] = ACTIONS(1),
    [anon_sym_match] = ACTIONS(1),
    [anon_sym_mod] = ACTIONS(1),
    [anon_sym_pub] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_raw] = ACTIONS(1),
    [anon_sym_static] = ACTIONS(1),
    [anon_sym_struct] = ACTIONS(1),
    [anon_sym_trait] = ACTIONS(1),
    [anon_sym_type] = ACTIONS(1),
    [anon_sym_union] = ACTIONS(1),
    [anon_sym_unsafe] = ACTIONS(1),
    [anon_sym_use] = ACTIONS(1),
    [anon_sym_where] = ACTIONS(1),
    [anon_sym_while] = ACTIONS(1),
    [anon_sym_extern] = ACTIONS(1),
    [anon_sym_safe] = ACTIONS(1),
    [anon_sym_ref] = ACTIONS(1),
    [anon_sym_TILDEconst] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_in] = ACTIONS(1),
    [anon_sym_LT2] = ACTIONS(1),
    [anon_sym_dyn] = ACTIONS(1),
    [sym_mutable_specifier] = ACTIONS(1),
    [anon_sym_yield] = ACTIONS(1),
    [anon_sym_move] = ACTIONS(1),
    [anon_sym_try] = ACTIONS(1),
    [anon_sym_box] = ACTIONS(1),
    [sym_integer_literal] = ACTIONS(1),
    [aux_sym_string_literal_token1] = ACTIONS(1),
    [anon_sym_DQUOTE] = ACTIONS(1),
    [sym_char_literal] = ACTIONS(1),
    [sym_escape_sequence] = ACTIONS(1),
    [anon_sym_true] = ACTIONS(1),
    [anon_sym_false] = ACTIONS(1),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_BANG2] = ACTIONS(1),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_shebang] = ACTIONS(1),
    [sym_self] = ACTIONS(1),
    [sym_super] = ACTIONS(1),
    [sym_crate] = ACTIONS(1),
    [aux_sym_metavariable_token1] = ACTIONS(1),
    [sym_string_content] = ACTIONS(1),
    [sym__raw_string_literal_start] = ACTIONS(1),
    [sym_raw_string_literal_content] = ACTIONS(1),
    [sym__raw_string_literal_end] = ACTIONS(1),
    [sym_float_literal] = ACTIONS(1),
    [sym__outer_block_doc_comment_start] = ACTIONS(1),
    [sym__outer_line_doc_comment_start] = ACTIONS(1),
    [sym__inner_block_doc_comment_marker] = ACTIONS(1),
    [sym__block_comment_content] = ACTIONS(1),
    [sym__line_doc_content] = ACTIONS(1),
    [sym__frontmatter_start] = ACTIONS(1),
    [sym_frontmatter_content] = ACTIONS(1),
    [sym__frontmatter_end] = ACTIONS(1),
    [sym__error_sentinel] = ACTIONS(1),
  },
  [STATE(1)] = {
    [sym_source_file] = STATE(6128),
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(1),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(1),
    [sym_frontmatter] = STATE(5),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(6),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(7),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(37),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_shebang] = ACTIONS(113),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
    [sym__frontmatter_start] = ACTIONS(129),
  },
  [STATE(2)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(2),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(2),
    [sym_frontmatter] = STATE(8),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(9),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(131),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
    [sym__frontmatter_start] = ACTIONS(129),
  },
  [STATE(3)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3356),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(3),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(3),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(7),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(135),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(4)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3282),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(4),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(4),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(137),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(5)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(5),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(5),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(9),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(131),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(6)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(6),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(6),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(10),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(131),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(7)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3271),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(7),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(7),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(139),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(8)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(8),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(8),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(11),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(141),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(9)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(9),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(9),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(10),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(141),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(10)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(10),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(10),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(10),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(143),
    [sym_identifier] = ACTIONS(145),
    [anon_sym_SEMI] = ACTIONS(148),
    [anon_sym_macro] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(154),
    [anon_sym_LBRACE] = ACTIONS(157),
    [anon_sym_macro_rules] = ACTIONS(160),
    [anon_sym_BANG] = ACTIONS(163),
    [anon_sym_LBRACK] = ACTIONS(166),
    [anon_sym_STAR] = ACTIONS(163),
    [anon_sym_u8] = ACTIONS(169),
    [anon_sym_i8] = ACTIONS(169),
    [anon_sym_u16] = ACTIONS(169),
    [anon_sym_i16] = ACTIONS(169),
    [anon_sym_u32] = ACTIONS(169),
    [anon_sym_i32] = ACTIONS(169),
    [anon_sym_u64] = ACTIONS(169),
    [anon_sym_i64] = ACTIONS(169),
    [anon_sym_u128] = ACTIONS(169),
    [anon_sym_i128] = ACTIONS(169),
    [anon_sym_isize] = ACTIONS(169),
    [anon_sym_usize] = ACTIONS(169),
    [anon_sym_f32] = ACTIONS(169),
    [anon_sym_f64] = ACTIONS(169),
    [anon_sym_bool] = ACTIONS(169),
    [anon_sym_str] = ACTIONS(169),
    [anon_sym_char] = ACTIONS(169),
    [anon_sym_DASH] = ACTIONS(163),
    [anon_sym_AMP] = ACTIONS(172),
    [anon_sym_PIPE] = ACTIONS(175),
    [anon_sym_LT] = ACTIONS(178),
    [anon_sym_DOT_DOT] = ACTIONS(181),
    [anon_sym_COLON_COLON] = ACTIONS(184),
    [anon_sym_POUND] = ACTIONS(187),
    [anon_sym_SQUOTE] = ACTIONS(190),
    [anon_sym_async] = ACTIONS(193),
    [anon_sym_auto] = ACTIONS(196),
    [anon_sym_await] = ACTIONS(199),
    [anon_sym_become] = ACTIONS(202),
    [anon_sym_break] = ACTIONS(205),
    [anon_sym_const] = ACTIONS(208),
    [anon_sym_continue] = ACTIONS(211),
    [anon_sym_default] = ACTIONS(214),
    [anon_sym_enum] = ACTIONS(217),
    [anon_sym_fn] = ACTIONS(220),
    [anon_sym_for] = ACTIONS(223),
    [anon_sym_gen] = ACTIONS(226),
    [anon_sym_if] = ACTIONS(229),
    [anon_sym_impl] = ACTIONS(232),
    [anon_sym_let] = ACTIONS(235),
    [anon_sym_loop] = ACTIONS(238),
    [anon_sym_match] = ACTIONS(241),
    [anon_sym_mod] = ACTIONS(244),
    [anon_sym_pub] = ACTIONS(247),
    [anon_sym_return] = ACTIONS(250),
    [anon_sym_raw] = ACTIONS(199),
    [anon_sym_static] = ACTIONS(253),
    [anon_sym_struct] = ACTIONS(256),
    [anon_sym_trait] = ACTIONS(259),
    [anon_sym_type] = ACTIONS(262),
    [anon_sym_union] = ACTIONS(265),
    [anon_sym_unsafe] = ACTIONS(268),
    [anon_sym_use] = ACTIONS(271),
    [anon_sym_while] = ACTIONS(274),
    [anon_sym_extern] = ACTIONS(277),
    [anon_sym_safe] = ACTIONS(280),
    [anon_sym_yield] = ACTIONS(283),
    [anon_sym_move] = ACTIONS(286),
    [anon_sym_try] = ACTIONS(289),
    [sym_integer_literal] = ACTIONS(292),
    [aux_sym_string_literal_token1] = ACTIONS(295),
    [sym_char_literal] = ACTIONS(292),
    [anon_sym_true] = ACTIONS(298),
    [anon_sym_false] = ACTIONS(298),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(301),
    [sym_super] = ACTIONS(304),
    [sym_crate] = ACTIONS(307),
    [aux_sym_metavariable_token1] = ACTIONS(310),
    [sym__raw_string_literal_start] = ACTIONS(313),
    [sym_float_literal] = ACTIONS(292),
    [sym__outer_block_doc_comment_start] = ACTIONS(316),
    [sym__outer_line_doc_comment_start] = ACTIONS(319),
  },
  [STATE(11)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(11),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(11),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(10),
    [aux_sym_attributes_repeat1] = STATE(596),
    [ts_builtin_sym_end] = ACTIONS(322),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(12)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3438),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(12),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(12),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(13),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(324),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(13)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3274),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(13),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(13),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(326),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(14)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3334),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(14),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(14),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(41),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(328),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(15)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(74),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3444),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(493),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(15),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(15),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(145),
    [anon_sym_SEMI] = ACTIONS(148),
    [anon_sym_macro] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(154),
    [anon_sym_LBRACE] = ACTIONS(157),
    [anon_sym_RBRACE] = ACTIONS(143),
    [anon_sym_macro_rules] = ACTIONS(160),
    [anon_sym_BANG] = ACTIONS(163),
    [anon_sym_LBRACK] = ACTIONS(166),
    [anon_sym_STAR] = ACTIONS(163),
    [anon_sym_u8] = ACTIONS(169),
    [anon_sym_i8] = ACTIONS(169),
    [anon_sym_u16] = ACTIONS(169),
    [anon_sym_i16] = ACTIONS(169),
    [anon_sym_u32] = ACTIONS(169),
    [anon_sym_i32] = ACTIONS(169),
    [anon_sym_u64] = ACTIONS(169),
    [anon_sym_i64] = ACTIONS(169),
    [anon_sym_u128] = ACTIONS(169),
    [anon_sym_i128] = ACTIONS(169),
    [anon_sym_isize] = ACTIONS(169),
    [anon_sym_usize] = ACTIONS(169),
    [anon_sym_f32] = ACTIONS(169),
    [anon_sym_f64] = ACTIONS(169),
    [anon_sym_bool] = ACTIONS(169),
    [anon_sym_str] = ACTIONS(169),
    [anon_sym_char] = ACTIONS(169),
    [anon_sym_DASH] = ACTIONS(163),
    [anon_sym_AMP] = ACTIONS(172),
    [anon_sym_PIPE] = ACTIONS(175),
    [anon_sym_LT] = ACTIONS(178),
    [anon_sym_DOT_DOT] = ACTIONS(181),
    [anon_sym_COLON_COLON] = ACTIONS(184),
    [anon_sym_POUND] = ACTIONS(187),
    [anon_sym_SQUOTE] = ACTIONS(190),
    [anon_sym_async] = ACTIONS(193),
    [anon_sym_auto] = ACTIONS(196),
    [anon_sym_await] = ACTIONS(199),
    [anon_sym_become] = ACTIONS(202),
    [anon_sym_break] = ACTIONS(205),
    [anon_sym_const] = ACTIONS(208),
    [anon_sym_continue] = ACTIONS(211),
    [anon_sym_default] = ACTIONS(214),
    [anon_sym_enum] = ACTIONS(217),
    [anon_sym_fn] = ACTIONS(220),
    [anon_sym_for] = ACTIONS(223),
    [anon_sym_gen] = ACTIONS(226),
    [anon_sym_if] = ACTIONS(229),
    [anon_sym_impl] = ACTIONS(232),
    [anon_sym_let] = ACTIONS(235),
    [anon_sym_loop] = ACTIONS(238),
    [anon_sym_match] = ACTIONS(241),
    [anon_sym_mod] = ACTIONS(244),
    [anon_sym_pub] = ACTIONS(247),
    [anon_sym_return] = ACTIONS(250),
    [anon_sym_raw] = ACTIONS(199),
    [anon_sym_static] = ACTIONS(253),
    [anon_sym_struct] = ACTIONS(256),
    [anon_sym_trait] = ACTIONS(259),
    [anon_sym_type] = ACTIONS(262),
    [anon_sym_union] = ACTIONS(265),
    [anon_sym_unsafe] = ACTIONS(268),
    [anon_sym_use] = ACTIONS(271),
    [anon_sym_while] = ACTIONS(274),
    [anon_sym_extern] = ACTIONS(277),
    [anon_sym_safe] = ACTIONS(280),
    [anon_sym_yield] = ACTIONS(283),
    [anon_sym_move] = ACTIONS(286),
    [anon_sym_try] = ACTIONS(289),
    [sym_integer_literal] = ACTIONS(292),
    [aux_sym_string_literal_token1] = ACTIONS(295),
    [sym_char_literal] = ACTIONS(292),
    [anon_sym_true] = ACTIONS(298),
    [anon_sym_false] = ACTIONS(298),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(301),
    [sym_super] = ACTIONS(304),
    [sym_crate] = ACTIONS(307),
    [aux_sym_metavariable_token1] = ACTIONS(310),
    [sym__raw_string_literal_start] = ACTIONS(313),
    [sym_float_literal] = ACTIONS(292),
    [sym__outer_block_doc_comment_start] = ACTIONS(316),
    [sym__outer_line_doc_comment_start] = ACTIONS(319),
  },
  [STATE(16)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3428),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(16),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(16),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(17),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(330),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(17)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3192),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(17),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(17),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(332),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(18)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3215),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(18),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(18),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(19),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(334),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(19)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3225),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(19),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(19),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(336),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(20)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3239),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(20),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(20),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(21),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(338),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(21)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3242),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(21),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(21),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(340),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(22)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3247),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(22),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(22),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(23),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(342),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(23)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3249),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(23),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(23),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(24)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3253),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(24),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(24),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(25),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(346),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(25)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3254),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(25),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(25),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(348),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(26)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3255),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(26),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(26),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(27),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(350),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(27)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3256),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(27),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(27),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(352),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(28)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3258),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(28),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(28),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(29),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(354),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(29)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3259),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(29),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(29),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(356),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(30)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3260),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(30),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(30),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(31),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(358),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(31)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3261),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(31),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(31),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(360),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(32)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3266),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(32),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(32),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(33),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(362),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(33)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3268),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(33),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(33),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(364),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(34)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3270),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(34),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(34),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(35),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(366),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(35)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3272),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(35),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(35),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(368),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(36)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3276),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(36),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(36),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(37),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(370),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(37)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3277),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(37),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(37),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(372),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(38)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3183),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(38),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(38),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(39),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(374),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(39)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3280),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(39),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(39),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(376),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(40)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3281),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(40),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(40),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(4),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(378),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(41)] = {
    [sym__statement] = STATE(893),
    [sym_empty_statement] = STATE(962),
    [sym_expression_statement] = STATE(994),
    [sym__declaration_statement] = STATE(994),
    [sym_macro_definition_v2] = STATE(962),
    [sym_macro_definition] = STATE(962),
    [sym_attribute_item] = STATE(1374),
    [sym_attributes] = STATE(75),
    [sym_inner_attribute_item] = STATE(962),
    [sym_mod_item] = STATE(962),
    [sym_foreign_mod_item] = STATE(962),
    [sym_struct_item] = STATE(962),
    [sym_union_item] = STATE(962),
    [sym_enum_item] = STATE(962),
    [sym_extern_crate_declaration] = STATE(962),
    [sym_const_item] = STATE(962),
    [sym_static_item] = STATE(962),
    [sym_type_item] = STATE(962),
    [sym_function_item] = STATE(962),
    [sym_function_signature_item] = STATE(962),
    [sym_function_modifiers] = STATE(6282),
    [aux_sym__item_modifiers] = STATE(3805),
    [sym_impl_item] = STATE(962),
    [sym_trait_item] = STATE(962),
    [sym_let_declaration] = STATE(962),
    [sym_use_declaration] = STATE(962),
    [sym_extern_modifier] = STATE(4452),
    [sym_visibility_modifier] = STATE(3501),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3360),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1169),
    [sym_macro_invocation_item] = STATE(962),
    [sym_macro_invocation] = STATE(470),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(486),
    [sym_match_expression] = STATE(486),
    [sym_while_expression] = STATE(486),
    [sym_loop_expression] = STATE(486),
    [sym_for_expression] = STATE(486),
    [sym_const_block] = STATE(486),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(486),
    [sym_async_block] = STATE(486),
    [sym_gen_block] = STATE(486),
    [sym_try_block] = STATE(486),
    [sym_block] = STATE(486),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(41),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(41),
    [sym_metavariable] = STATE(2991),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym_attributes_repeat1] = STATE(596),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(11),
    [anon_sym_macro] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_RBRACE] = ACTIONS(380),
    [anon_sym_macro_rules] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(133),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(43),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(51),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(57),
    [anon_sym_fn] = ACTIONS(59),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(67),
    [anon_sym_let] = ACTIONS(69),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(75),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(81),
    [anon_sym_struct] = ACTIONS(83),
    [anon_sym_trait] = ACTIONS(85),
    [anon_sym_type] = ACTIONS(87),
    [anon_sym_union] = ACTIONS(89),
    [anon_sym_unsafe] = ACTIONS(91),
    [anon_sym_use] = ACTIONS(93),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(97),
    [anon_sym_safe] = ACTIONS(99),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(42)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3026),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(42),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(42),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(384),
    [anon_sym_LPAREN] = ACTIONS(384),
    [anon_sym_RPAREN] = ACTIONS(384),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(384),
    [anon_sym_COMMA] = ACTIONS(384),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(384),
    [anon_sym_RBRACK] = ACTIONS(384),
    [anon_sym_COLON] = ACTIONS(392),
    [anon_sym_PLUS] = ACTIONS(394),
    [anon_sym_STAR] = ACTIONS(394),
    [anon_sym_QMARK] = ACTIONS(384),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(394),
    [anon_sym_SLASH] = ACTIONS(394),
    [anon_sym_PERCENT] = ACTIONS(394),
    [anon_sym_CARET] = ACTIONS(394),
    [anon_sym_AMP] = ACTIONS(394),
    [anon_sym_PIPE] = ACTIONS(394),
    [anon_sym_AMP_AMP] = ACTIONS(384),
    [anon_sym_PIPE_PIPE] = ACTIONS(384),
    [anon_sym_LT_LT] = ACTIONS(394),
    [anon_sym_GT_GT] = ACTIONS(394),
    [anon_sym_PLUS_EQ] = ACTIONS(384),
    [anon_sym_DASH_EQ] = ACTIONS(384),
    [anon_sym_STAR_EQ] = ACTIONS(384),
    [anon_sym_SLASH_EQ] = ACTIONS(384),
    [anon_sym_PERCENT_EQ] = ACTIONS(384),
    [anon_sym_CARET_EQ] = ACTIONS(384),
    [anon_sym_AMP_EQ] = ACTIONS(384),
    [anon_sym_PIPE_EQ] = ACTIONS(384),
    [anon_sym_LT_LT_EQ] = ACTIONS(384),
    [anon_sym_GT_GT_EQ] = ACTIONS(384),
    [anon_sym_EQ] = ACTIONS(394),
    [anon_sym_EQ_EQ] = ACTIONS(384),
    [anon_sym_BANG_EQ] = ACTIONS(384),
    [anon_sym_GT] = ACTIONS(394),
    [anon_sym_LT] = ACTIONS(394),
    [anon_sym_GT_EQ] = ACTIONS(384),
    [anon_sym_LT_EQ] = ACTIONS(384),
    [anon_sym_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(384),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(384),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(394),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(394),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(43)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3045),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(43),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(43),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(424),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(424),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(424),
    [anon_sym_COMMA] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_RBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(426),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(44)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3024),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(42),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(44),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(44),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(428),
    [anon_sym_LPAREN] = ACTIONS(428),
    [anon_sym_RPAREN] = ACTIONS(428),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(428),
    [anon_sym_COMMA] = ACTIONS(428),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(428),
    [anon_sym_RBRACK] = ACTIONS(428),
    [anon_sym_PLUS] = ACTIONS(430),
    [anon_sym_STAR] = ACTIONS(430),
    [anon_sym_QMARK] = ACTIONS(428),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(430),
    [anon_sym_SLASH] = ACTIONS(430),
    [anon_sym_PERCENT] = ACTIONS(430),
    [anon_sym_CARET] = ACTIONS(430),
    [anon_sym_AMP] = ACTIONS(430),
    [anon_sym_PIPE] = ACTIONS(430),
    [anon_sym_AMP_AMP] = ACTIONS(428),
    [anon_sym_PIPE_PIPE] = ACTIONS(428),
    [anon_sym_LT_LT] = ACTIONS(430),
    [anon_sym_GT_GT] = ACTIONS(430),
    [anon_sym_PLUS_EQ] = ACTIONS(428),
    [anon_sym_DASH_EQ] = ACTIONS(428),
    [anon_sym_STAR_EQ] = ACTIONS(428),
    [anon_sym_SLASH_EQ] = ACTIONS(428),
    [anon_sym_PERCENT_EQ] = ACTIONS(428),
    [anon_sym_CARET_EQ] = ACTIONS(428),
    [anon_sym_AMP_EQ] = ACTIONS(428),
    [anon_sym_PIPE_EQ] = ACTIONS(428),
    [anon_sym_LT_LT_EQ] = ACTIONS(428),
    [anon_sym_GT_GT_EQ] = ACTIONS(428),
    [anon_sym_EQ] = ACTIONS(430),
    [anon_sym_EQ_EQ] = ACTIONS(428),
    [anon_sym_BANG_EQ] = ACTIONS(428),
    [anon_sym_GT] = ACTIONS(430),
    [anon_sym_LT] = ACTIONS(430),
    [anon_sym_GT_EQ] = ACTIONS(428),
    [anon_sym_LT_EQ] = ACTIONS(428),
    [anon_sym_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(428),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(428),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(432),
    [anon_sym_as] = ACTIONS(430),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(430),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(45)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3036),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(45),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(45),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(434),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(434),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(434),
    [anon_sym_COMMA] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_RBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(436),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(46)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3037),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(46),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(46),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(438),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(438),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(438),
    [anon_sym_COMMA] = ACTIONS(438),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(438),
    [anon_sym_PLUS] = ACTIONS(440),
    [anon_sym_STAR] = ACTIONS(390),
    [anon_sym_QMARK] = ACTIONS(438),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_SLASH] = ACTIONS(440),
    [anon_sym_PERCENT] = ACTIONS(440),
    [anon_sym_CARET] = ACTIONS(440),
    [anon_sym_AMP] = ACTIONS(442),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(438),
    [anon_sym_PIPE_PIPE] = ACTIONS(438),
    [anon_sym_LT_LT] = ACTIONS(440),
    [anon_sym_GT_GT] = ACTIONS(440),
    [anon_sym_PLUS_EQ] = ACTIONS(438),
    [anon_sym_DASH_EQ] = ACTIONS(438),
    [anon_sym_STAR_EQ] = ACTIONS(438),
    [anon_sym_SLASH_EQ] = ACTIONS(438),
    [anon_sym_PERCENT_EQ] = ACTIONS(438),
    [anon_sym_CARET_EQ] = ACTIONS(438),
    [anon_sym_AMP_EQ] = ACTIONS(438),
    [anon_sym_PIPE_EQ] = ACTIONS(438),
    [anon_sym_LT_LT_EQ] = ACTIONS(438),
    [anon_sym_GT_GT_EQ] = ACTIONS(438),
    [anon_sym_EQ] = ACTIONS(440),
    [anon_sym_EQ_EQ] = ACTIONS(438),
    [anon_sym_BANG_EQ] = ACTIONS(438),
    [anon_sym_GT] = ACTIONS(440),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(438),
    [anon_sym_LT_EQ] = ACTIONS(438),
    [anon_sym_DOT] = ACTIONS(440),
    [anon_sym_DOT_DOT] = ACTIONS(448),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(438),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(438),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(440),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(440),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(47)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3027),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(47),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(47),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(450),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(450),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(450),
    [anon_sym_COMMA] = ACTIONS(450),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(450),
    [anon_sym_PLUS] = ACTIONS(452),
    [anon_sym_STAR] = ACTIONS(390),
    [anon_sym_QMARK] = ACTIONS(450),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_SLASH] = ACTIONS(452),
    [anon_sym_PERCENT] = ACTIONS(452),
    [anon_sym_CARET] = ACTIONS(452),
    [anon_sym_AMP] = ACTIONS(442),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(450),
    [anon_sym_PIPE_PIPE] = ACTIONS(450),
    [anon_sym_LT_LT] = ACTIONS(452),
    [anon_sym_GT_GT] = ACTIONS(452),
    [anon_sym_PLUS_EQ] = ACTIONS(450),
    [anon_sym_DASH_EQ] = ACTIONS(450),
    [anon_sym_STAR_EQ] = ACTIONS(450),
    [anon_sym_SLASH_EQ] = ACTIONS(450),
    [anon_sym_PERCENT_EQ] = ACTIONS(450),
    [anon_sym_CARET_EQ] = ACTIONS(450),
    [anon_sym_AMP_EQ] = ACTIONS(450),
    [anon_sym_PIPE_EQ] = ACTIONS(450),
    [anon_sym_LT_LT_EQ] = ACTIONS(450),
    [anon_sym_GT_GT_EQ] = ACTIONS(450),
    [anon_sym_EQ] = ACTIONS(452),
    [anon_sym_EQ_EQ] = ACTIONS(450),
    [anon_sym_BANG_EQ] = ACTIONS(450),
    [anon_sym_GT] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(450),
    [anon_sym_LT_EQ] = ACTIONS(450),
    [anon_sym_DOT] = ACTIONS(452),
    [anon_sym_DOT_DOT] = ACTIONS(448),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(450),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(450),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(452),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(452),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(48)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3045),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(48),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(48),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(424),
    [anon_sym_LPAREN] = ACTIONS(424),
    [anon_sym_RPAREN] = ACTIONS(424),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(424),
    [anon_sym_COMMA] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_RBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(426),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(49)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3036),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(49),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(49),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_SEMI] = ACTIONS(434),
    [anon_sym_LPAREN] = ACTIONS(434),
    [anon_sym_RPAREN] = ACTIONS(434),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(434),
    [anon_sym_COMMA] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(390),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_RBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_else] = ACTIONS(436),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(50)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3336),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(50),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(50),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(384),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(384),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(384),
    [anon_sym_COLON] = ACTIONS(462),
    [anon_sym_PLUS] = ACTIONS(394),
    [anon_sym_STAR] = ACTIONS(394),
    [anon_sym_QMARK] = ACTIONS(384),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(394),
    [anon_sym_SLASH] = ACTIONS(394),
    [anon_sym_PERCENT] = ACTIONS(394),
    [anon_sym_CARET] = ACTIONS(394),
    [anon_sym_AMP] = ACTIONS(394),
    [anon_sym_PIPE] = ACTIONS(394),
    [anon_sym_AMP_AMP] = ACTIONS(384),
    [anon_sym_PIPE_PIPE] = ACTIONS(384),
    [anon_sym_LT_LT] = ACTIONS(394),
    [anon_sym_GT_GT] = ACTIONS(394),
    [anon_sym_PLUS_EQ] = ACTIONS(384),
    [anon_sym_DASH_EQ] = ACTIONS(384),
    [anon_sym_STAR_EQ] = ACTIONS(384),
    [anon_sym_SLASH_EQ] = ACTIONS(384),
    [anon_sym_PERCENT_EQ] = ACTIONS(384),
    [anon_sym_CARET_EQ] = ACTIONS(384),
    [anon_sym_AMP_EQ] = ACTIONS(384),
    [anon_sym_PIPE_EQ] = ACTIONS(384),
    [anon_sym_LT_LT_EQ] = ACTIONS(384),
    [anon_sym_GT_GT_EQ] = ACTIONS(384),
    [anon_sym_EQ] = ACTIONS(394),
    [anon_sym_EQ_EQ] = ACTIONS(384),
    [anon_sym_BANG_EQ] = ACTIONS(384),
    [anon_sym_GT] = ACTIONS(394),
    [anon_sym_LT] = ACTIONS(394),
    [anon_sym_GT_EQ] = ACTIONS(384),
    [anon_sym_LT_EQ] = ACTIONS(384),
    [anon_sym_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(384),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(384),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(394),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(51)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(250),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3372),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(51),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(51),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(52)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3372),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(52),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(52),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(434),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(53)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(250),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3435),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(53),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(53),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(54)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3435),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(54),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(54),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(424),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(55)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3174),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(55),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(55),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(384),
    [anon_sym_LBRACE] = ACTIONS(384),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(384),
    [anon_sym_COLON] = ACTIONS(392),
    [anon_sym_PLUS] = ACTIONS(394),
    [anon_sym_STAR] = ACTIONS(394),
    [anon_sym_QMARK] = ACTIONS(384),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(394),
    [anon_sym_SLASH] = ACTIONS(394),
    [anon_sym_PERCENT] = ACTIONS(394),
    [anon_sym_CARET] = ACTIONS(394),
    [anon_sym_AMP] = ACTIONS(394),
    [anon_sym_PIPE] = ACTIONS(394),
    [anon_sym_AMP_AMP] = ACTIONS(384),
    [anon_sym_PIPE_PIPE] = ACTIONS(384),
    [anon_sym_LT_LT] = ACTIONS(394),
    [anon_sym_GT_GT] = ACTIONS(394),
    [anon_sym_PLUS_EQ] = ACTIONS(384),
    [anon_sym_DASH_EQ] = ACTIONS(384),
    [anon_sym_STAR_EQ] = ACTIONS(384),
    [anon_sym_SLASH_EQ] = ACTIONS(384),
    [anon_sym_PERCENT_EQ] = ACTIONS(384),
    [anon_sym_CARET_EQ] = ACTIONS(384),
    [anon_sym_AMP_EQ] = ACTIONS(384),
    [anon_sym_PIPE_EQ] = ACTIONS(384),
    [anon_sym_LT_LT_EQ] = ACTIONS(384),
    [anon_sym_GT_GT_EQ] = ACTIONS(384),
    [anon_sym_EQ] = ACTIONS(394),
    [anon_sym_EQ_EQ] = ACTIONS(384),
    [anon_sym_BANG_EQ] = ACTIONS(384),
    [anon_sym_GT] = ACTIONS(394),
    [anon_sym_LT] = ACTIONS(394),
    [anon_sym_GT_EQ] = ACTIONS(384),
    [anon_sym_LT_EQ] = ACTIONS(384),
    [anon_sym_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(384),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(384),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(394),
    [anon_sym_as] = ACTIONS(394),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(56)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3383),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(56),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(56),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(384),
    [anon_sym_LBRACE] = ACTIONS(384),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(384),
    [anon_sym_COLON] = ACTIONS(392),
    [anon_sym_PLUS] = ACTIONS(394),
    [anon_sym_STAR] = ACTIONS(394),
    [anon_sym_QMARK] = ACTIONS(384),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(394),
    [anon_sym_SLASH] = ACTIONS(394),
    [anon_sym_PERCENT] = ACTIONS(394),
    [anon_sym_CARET] = ACTIONS(394),
    [anon_sym_AMP] = ACTIONS(394),
    [anon_sym_PIPE] = ACTIONS(394),
    [anon_sym_AMP_AMP] = ACTIONS(384),
    [anon_sym_PIPE_PIPE] = ACTIONS(384),
    [anon_sym_LT_LT] = ACTIONS(394),
    [anon_sym_GT_GT] = ACTIONS(394),
    [anon_sym_PLUS_EQ] = ACTIONS(384),
    [anon_sym_DASH_EQ] = ACTIONS(384),
    [anon_sym_STAR_EQ] = ACTIONS(384),
    [anon_sym_SLASH_EQ] = ACTIONS(384),
    [anon_sym_PERCENT_EQ] = ACTIONS(384),
    [anon_sym_CARET_EQ] = ACTIONS(384),
    [anon_sym_AMP_EQ] = ACTIONS(384),
    [anon_sym_PIPE_EQ] = ACTIONS(384),
    [anon_sym_LT_LT_EQ] = ACTIONS(384),
    [anon_sym_GT_GT_EQ] = ACTIONS(384),
    [anon_sym_EQ] = ACTIONS(394),
    [anon_sym_EQ_EQ] = ACTIONS(384),
    [anon_sym_BANG_EQ] = ACTIONS(384),
    [anon_sym_GT] = ACTIONS(394),
    [anon_sym_LT] = ACTIONS(394),
    [anon_sym_GT_EQ] = ACTIONS(384),
    [anon_sym_LT_EQ] = ACTIONS(384),
    [anon_sym_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT] = ACTIONS(394),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(384),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(384),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(394),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(57)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3217),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(50),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(57),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(57),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(428),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(428),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(428),
    [anon_sym_PLUS] = ACTIONS(430),
    [anon_sym_STAR] = ACTIONS(430),
    [anon_sym_QMARK] = ACTIONS(428),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(430),
    [anon_sym_SLASH] = ACTIONS(430),
    [anon_sym_PERCENT] = ACTIONS(430),
    [anon_sym_CARET] = ACTIONS(430),
    [anon_sym_AMP] = ACTIONS(430),
    [anon_sym_PIPE] = ACTIONS(430),
    [anon_sym_AMP_AMP] = ACTIONS(428),
    [anon_sym_PIPE_PIPE] = ACTIONS(428),
    [anon_sym_LT_LT] = ACTIONS(430),
    [anon_sym_GT_GT] = ACTIONS(430),
    [anon_sym_PLUS_EQ] = ACTIONS(428),
    [anon_sym_DASH_EQ] = ACTIONS(428),
    [anon_sym_STAR_EQ] = ACTIONS(428),
    [anon_sym_SLASH_EQ] = ACTIONS(428),
    [anon_sym_PERCENT_EQ] = ACTIONS(428),
    [anon_sym_CARET_EQ] = ACTIONS(428),
    [anon_sym_AMP_EQ] = ACTIONS(428),
    [anon_sym_PIPE_EQ] = ACTIONS(428),
    [anon_sym_LT_LT_EQ] = ACTIONS(428),
    [anon_sym_GT_GT_EQ] = ACTIONS(428),
    [anon_sym_EQ] = ACTIONS(430),
    [anon_sym_EQ_EQ] = ACTIONS(428),
    [anon_sym_BANG_EQ] = ACTIONS(428),
    [anon_sym_GT] = ACTIONS(430),
    [anon_sym_LT] = ACTIONS(430),
    [anon_sym_GT_EQ] = ACTIONS(428),
    [anon_sym_LT_EQ] = ACTIONS(428),
    [anon_sym_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(428),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(428),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(572),
    [anon_sym_as] = ACTIONS(430),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(58)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3243),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(58),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(58),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(450),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_PLUS] = ACTIONS(452),
    [anon_sym_STAR] = ACTIONS(460),
    [anon_sym_QMARK] = ACTIONS(450),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(460),
    [anon_sym_SLASH] = ACTIONS(452),
    [anon_sym_PERCENT] = ACTIONS(452),
    [anon_sym_CARET] = ACTIONS(452),
    [anon_sym_AMP] = ACTIONS(576),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(450),
    [anon_sym_PIPE_PIPE] = ACTIONS(450),
    [anon_sym_LT_LT] = ACTIONS(452),
    [anon_sym_GT_GT] = ACTIONS(452),
    [anon_sym_PLUS_EQ] = ACTIONS(450),
    [anon_sym_DASH_EQ] = ACTIONS(450),
    [anon_sym_STAR_EQ] = ACTIONS(450),
    [anon_sym_SLASH_EQ] = ACTIONS(450),
    [anon_sym_PERCENT_EQ] = ACTIONS(450),
    [anon_sym_CARET_EQ] = ACTIONS(450),
    [anon_sym_AMP_EQ] = ACTIONS(450),
    [anon_sym_PIPE_EQ] = ACTIONS(450),
    [anon_sym_LT_LT_EQ] = ACTIONS(450),
    [anon_sym_GT_GT_EQ] = ACTIONS(450),
    [anon_sym_EQ] = ACTIONS(452),
    [anon_sym_EQ_EQ] = ACTIONS(450),
    [anon_sym_BANG_EQ] = ACTIONS(450),
    [anon_sym_GT] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(450),
    [anon_sym_LT_EQ] = ACTIONS(450),
    [anon_sym_DOT] = ACTIONS(452),
    [anon_sym_DOT_DOT] = ACTIONS(578),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(450),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(450),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(452),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(59)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3246),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(59),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(59),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_EQ_GT] = ACTIONS(438),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(460),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_PLUS] = ACTIONS(440),
    [anon_sym_STAR] = ACTIONS(460),
    [anon_sym_QMARK] = ACTIONS(438),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(460),
    [anon_sym_SLASH] = ACTIONS(440),
    [anon_sym_PERCENT] = ACTIONS(440),
    [anon_sym_CARET] = ACTIONS(440),
    [anon_sym_AMP] = ACTIONS(576),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(438),
    [anon_sym_PIPE_PIPE] = ACTIONS(438),
    [anon_sym_LT_LT] = ACTIONS(440),
    [anon_sym_GT_GT] = ACTIONS(440),
    [anon_sym_PLUS_EQ] = ACTIONS(438),
    [anon_sym_DASH_EQ] = ACTIONS(438),
    [anon_sym_STAR_EQ] = ACTIONS(438),
    [anon_sym_SLASH_EQ] = ACTIONS(438),
    [anon_sym_PERCENT_EQ] = ACTIONS(438),
    [anon_sym_CARET_EQ] = ACTIONS(438),
    [anon_sym_AMP_EQ] = ACTIONS(438),
    [anon_sym_PIPE_EQ] = ACTIONS(438),
    [anon_sym_LT_LT_EQ] = ACTIONS(438),
    [anon_sym_GT_GT_EQ] = ACTIONS(438),
    [anon_sym_EQ] = ACTIONS(440),
    [anon_sym_EQ_EQ] = ACTIONS(438),
    [anon_sym_BANG_EQ] = ACTIONS(438),
    [anon_sym_GT] = ACTIONS(440),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(438),
    [anon_sym_LT_EQ] = ACTIONS(438),
    [anon_sym_DOT] = ACTIONS(440),
    [anon_sym_DOT_DOT] = ACTIONS(578),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(438),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(438),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(440),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(60)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3219),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(60),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(60),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(424),
    [anon_sym_LBRACE] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(61)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3224),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(61),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(61),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(440),
    [anon_sym_STAR] = ACTIONS(552),
    [anon_sym_QMARK] = ACTIONS(438),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(552),
    [anon_sym_SLASH] = ACTIONS(440),
    [anon_sym_PERCENT] = ACTIONS(440),
    [anon_sym_CARET] = ACTIONS(440),
    [anon_sym_AMP] = ACTIONS(580),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(438),
    [anon_sym_PIPE_PIPE] = ACTIONS(438),
    [anon_sym_LT_LT] = ACTIONS(440),
    [anon_sym_GT_GT] = ACTIONS(440),
    [anon_sym_PLUS_EQ] = ACTIONS(438),
    [anon_sym_DASH_EQ] = ACTIONS(438),
    [anon_sym_STAR_EQ] = ACTIONS(438),
    [anon_sym_SLASH_EQ] = ACTIONS(438),
    [anon_sym_PERCENT_EQ] = ACTIONS(438),
    [anon_sym_CARET_EQ] = ACTIONS(438),
    [anon_sym_AMP_EQ] = ACTIONS(438),
    [anon_sym_PIPE_EQ] = ACTIONS(438),
    [anon_sym_LT_LT_EQ] = ACTIONS(438),
    [anon_sym_GT_GT_EQ] = ACTIONS(438),
    [anon_sym_EQ] = ACTIONS(440),
    [anon_sym_EQ_EQ] = ACTIONS(438),
    [anon_sym_BANG_EQ] = ACTIONS(438),
    [anon_sym_GT] = ACTIONS(440),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(438),
    [anon_sym_LT_EQ] = ACTIONS(438),
    [anon_sym_DOT] = ACTIONS(440),
    [anon_sym_DOT_DOT] = ACTIONS(582),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(438),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(438),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(440),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(62)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3128),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(62),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(62),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(426),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(63)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3221),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(56),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(63),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(63),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(428),
    [anon_sym_LBRACE] = ACTIONS(428),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(428),
    [anon_sym_PLUS] = ACTIONS(430),
    [anon_sym_STAR] = ACTIONS(430),
    [anon_sym_QMARK] = ACTIONS(428),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(430),
    [anon_sym_SLASH] = ACTIONS(430),
    [anon_sym_PERCENT] = ACTIONS(430),
    [anon_sym_CARET] = ACTIONS(430),
    [anon_sym_AMP] = ACTIONS(430),
    [anon_sym_PIPE] = ACTIONS(430),
    [anon_sym_AMP_AMP] = ACTIONS(428),
    [anon_sym_PIPE_PIPE] = ACTIONS(428),
    [anon_sym_LT_LT] = ACTIONS(430),
    [anon_sym_GT_GT] = ACTIONS(430),
    [anon_sym_PLUS_EQ] = ACTIONS(428),
    [anon_sym_DASH_EQ] = ACTIONS(428),
    [anon_sym_STAR_EQ] = ACTIONS(428),
    [anon_sym_SLASH_EQ] = ACTIONS(428),
    [anon_sym_PERCENT_EQ] = ACTIONS(428),
    [anon_sym_CARET_EQ] = ACTIONS(428),
    [anon_sym_AMP_EQ] = ACTIONS(428),
    [anon_sym_PIPE_EQ] = ACTIONS(428),
    [anon_sym_LT_LT_EQ] = ACTIONS(428),
    [anon_sym_GT_GT_EQ] = ACTIONS(428),
    [anon_sym_EQ] = ACTIONS(430),
    [anon_sym_EQ_EQ] = ACTIONS(428),
    [anon_sym_BANG_EQ] = ACTIONS(428),
    [anon_sym_GT] = ACTIONS(430),
    [anon_sym_LT] = ACTIONS(430),
    [anon_sym_GT_EQ] = ACTIONS(428),
    [anon_sym_LT_EQ] = ACTIONS(428),
    [anon_sym_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(428),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(428),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(432),
    [anon_sym_as] = ACTIONS(430),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(64)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3223),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(64),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(64),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(452),
    [anon_sym_STAR] = ACTIONS(552),
    [anon_sym_QMARK] = ACTIONS(450),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(552),
    [anon_sym_SLASH] = ACTIONS(452),
    [anon_sym_PERCENT] = ACTIONS(452),
    [anon_sym_CARET] = ACTIONS(452),
    [anon_sym_AMP] = ACTIONS(580),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(450),
    [anon_sym_PIPE_PIPE] = ACTIONS(450),
    [anon_sym_LT_LT] = ACTIONS(452),
    [anon_sym_GT_GT] = ACTIONS(452),
    [anon_sym_PLUS_EQ] = ACTIONS(450),
    [anon_sym_DASH_EQ] = ACTIONS(450),
    [anon_sym_STAR_EQ] = ACTIONS(450),
    [anon_sym_SLASH_EQ] = ACTIONS(450),
    [anon_sym_PERCENT_EQ] = ACTIONS(450),
    [anon_sym_CARET_EQ] = ACTIONS(450),
    [anon_sym_AMP_EQ] = ACTIONS(450),
    [anon_sym_PIPE_EQ] = ACTIONS(450),
    [anon_sym_LT_LT_EQ] = ACTIONS(450),
    [anon_sym_GT_GT_EQ] = ACTIONS(450),
    [anon_sym_EQ] = ACTIONS(452),
    [anon_sym_EQ_EQ] = ACTIONS(450),
    [anon_sym_BANG_EQ] = ACTIONS(450),
    [anon_sym_GT] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(450),
    [anon_sym_LT_EQ] = ACTIONS(450),
    [anon_sym_DOT] = ACTIONS(452),
    [anon_sym_DOT_DOT] = ACTIONS(582),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(450),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(450),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(452),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(65)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3181),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(65),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(65),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(436),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(66)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3400),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(66),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(66),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(434),
    [anon_sym_LBRACE] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(67)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3128),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(67),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(67),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(424),
    [anon_sym_LBRACE] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(426),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(68)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(220),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3219),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(68),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(68),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(424),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(424),
    [anon_sym_PLUS] = ACTIONS(426),
    [anon_sym_STAR] = ACTIONS(426),
    [anon_sym_QMARK] = ACTIONS(424),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(426),
    [anon_sym_SLASH] = ACTIONS(426),
    [anon_sym_PERCENT] = ACTIONS(426),
    [anon_sym_CARET] = ACTIONS(426),
    [anon_sym_AMP] = ACTIONS(426),
    [anon_sym_PIPE] = ACTIONS(426),
    [anon_sym_AMP_AMP] = ACTIONS(424),
    [anon_sym_PIPE_PIPE] = ACTIONS(424),
    [anon_sym_LT_LT] = ACTIONS(426),
    [anon_sym_GT_GT] = ACTIONS(426),
    [anon_sym_PLUS_EQ] = ACTIONS(424),
    [anon_sym_DASH_EQ] = ACTIONS(424),
    [anon_sym_STAR_EQ] = ACTIONS(424),
    [anon_sym_SLASH_EQ] = ACTIONS(424),
    [anon_sym_PERCENT_EQ] = ACTIONS(424),
    [anon_sym_CARET_EQ] = ACTIONS(424),
    [anon_sym_AMP_EQ] = ACTIONS(424),
    [anon_sym_PIPE_EQ] = ACTIONS(424),
    [anon_sym_LT_LT_EQ] = ACTIONS(424),
    [anon_sym_GT_GT_EQ] = ACTIONS(424),
    [anon_sym_EQ] = ACTIONS(426),
    [anon_sym_EQ_EQ] = ACTIONS(424),
    [anon_sym_BANG_EQ] = ACTIONS(424),
    [anon_sym_GT] = ACTIONS(426),
    [anon_sym_LT] = ACTIONS(426),
    [anon_sym_GT_EQ] = ACTIONS(424),
    [anon_sym_LT_EQ] = ACTIONS(424),
    [anon_sym_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT] = ACTIONS(426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(424),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(426),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(69)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(220),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3400),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(69),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(69),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(552),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(70)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3181),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(70),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(70),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(434),
    [anon_sym_LBRACE] = ACTIONS(434),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(434),
    [anon_sym_PLUS] = ACTIONS(436),
    [anon_sym_STAR] = ACTIONS(436),
    [anon_sym_QMARK] = ACTIONS(434),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(436),
    [anon_sym_SLASH] = ACTIONS(436),
    [anon_sym_PERCENT] = ACTIONS(436),
    [anon_sym_CARET] = ACTIONS(436),
    [anon_sym_AMP] = ACTIONS(436),
    [anon_sym_PIPE] = ACTIONS(436),
    [anon_sym_AMP_AMP] = ACTIONS(434),
    [anon_sym_PIPE_PIPE] = ACTIONS(434),
    [anon_sym_LT_LT] = ACTIONS(436),
    [anon_sym_GT_GT] = ACTIONS(436),
    [anon_sym_PLUS_EQ] = ACTIONS(434),
    [anon_sym_DASH_EQ] = ACTIONS(434),
    [anon_sym_STAR_EQ] = ACTIONS(434),
    [anon_sym_SLASH_EQ] = ACTIONS(434),
    [anon_sym_PERCENT_EQ] = ACTIONS(434),
    [anon_sym_CARET_EQ] = ACTIONS(434),
    [anon_sym_AMP_EQ] = ACTIONS(434),
    [anon_sym_PIPE_EQ] = ACTIONS(434),
    [anon_sym_LT_LT_EQ] = ACTIONS(434),
    [anon_sym_GT_GT_EQ] = ACTIONS(434),
    [anon_sym_EQ] = ACTIONS(436),
    [anon_sym_EQ_EQ] = ACTIONS(434),
    [anon_sym_BANG_EQ] = ACTIONS(434),
    [anon_sym_GT] = ACTIONS(436),
    [anon_sym_LT] = ACTIONS(436),
    [anon_sym_GT_EQ] = ACTIONS(434),
    [anon_sym_LT_EQ] = ACTIONS(434),
    [anon_sym_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT] = ACTIONS(436),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(434),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(436),
    [anon_sym_as] = ACTIONS(436),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(71)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3133),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(55),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(71),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(71),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(428),
    [anon_sym_LBRACE] = ACTIONS(428),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(428),
    [anon_sym_PLUS] = ACTIONS(430),
    [anon_sym_STAR] = ACTIONS(430),
    [anon_sym_QMARK] = ACTIONS(428),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(430),
    [anon_sym_SLASH] = ACTIONS(430),
    [anon_sym_PERCENT] = ACTIONS(430),
    [anon_sym_CARET] = ACTIONS(430),
    [anon_sym_AMP] = ACTIONS(430),
    [anon_sym_PIPE] = ACTIONS(430),
    [anon_sym_AMP_AMP] = ACTIONS(428),
    [anon_sym_PIPE_PIPE] = ACTIONS(428),
    [anon_sym_LT_LT] = ACTIONS(430),
    [anon_sym_GT_GT] = ACTIONS(430),
    [anon_sym_PLUS_EQ] = ACTIONS(428),
    [anon_sym_DASH_EQ] = ACTIONS(428),
    [anon_sym_STAR_EQ] = ACTIONS(428),
    [anon_sym_SLASH_EQ] = ACTIONS(428),
    [anon_sym_PERCENT_EQ] = ACTIONS(428),
    [anon_sym_CARET_EQ] = ACTIONS(428),
    [anon_sym_AMP_EQ] = ACTIONS(428),
    [anon_sym_PIPE_EQ] = ACTIONS(428),
    [anon_sym_LT_LT_EQ] = ACTIONS(428),
    [anon_sym_GT_GT_EQ] = ACTIONS(428),
    [anon_sym_EQ] = ACTIONS(430),
    [anon_sym_EQ_EQ] = ACTIONS(428),
    [anon_sym_BANG_EQ] = ACTIONS(428),
    [anon_sym_GT] = ACTIONS(430),
    [anon_sym_LT] = ACTIONS(430),
    [anon_sym_GT_EQ] = ACTIONS(428),
    [anon_sym_LT_EQ] = ACTIONS(428),
    [anon_sym_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT] = ACTIONS(430),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(428),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(428),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(430),
    [anon_sym_as] = ACTIONS(430),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(72)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3148),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(72),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(72),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(440),
    [anon_sym_STAR] = ACTIONS(522),
    [anon_sym_QMARK] = ACTIONS(438),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(522),
    [anon_sym_SLASH] = ACTIONS(440),
    [anon_sym_PERCENT] = ACTIONS(440),
    [anon_sym_CARET] = ACTIONS(440),
    [anon_sym_AMP] = ACTIONS(584),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(438),
    [anon_sym_PIPE_PIPE] = ACTIONS(438),
    [anon_sym_LT_LT] = ACTIONS(440),
    [anon_sym_GT_GT] = ACTIONS(440),
    [anon_sym_PLUS_EQ] = ACTIONS(438),
    [anon_sym_DASH_EQ] = ACTIONS(438),
    [anon_sym_STAR_EQ] = ACTIONS(438),
    [anon_sym_SLASH_EQ] = ACTIONS(438),
    [anon_sym_PERCENT_EQ] = ACTIONS(438),
    [anon_sym_CARET_EQ] = ACTIONS(438),
    [anon_sym_AMP_EQ] = ACTIONS(438),
    [anon_sym_PIPE_EQ] = ACTIONS(438),
    [anon_sym_LT_LT_EQ] = ACTIONS(438),
    [anon_sym_GT_GT_EQ] = ACTIONS(438),
    [anon_sym_EQ] = ACTIONS(440),
    [anon_sym_EQ_EQ] = ACTIONS(438),
    [anon_sym_BANG_EQ] = ACTIONS(438),
    [anon_sym_GT] = ACTIONS(440),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(438),
    [anon_sym_LT_EQ] = ACTIONS(438),
    [anon_sym_DOT] = ACTIONS(440),
    [anon_sym_DOT_DOT] = ACTIONS(586),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(438),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(438),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(440),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(73)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3138),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(73),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(73),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(522),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(452),
    [anon_sym_STAR] = ACTIONS(522),
    [anon_sym_QMARK] = ACTIONS(450),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(522),
    [anon_sym_SLASH] = ACTIONS(452),
    [anon_sym_PERCENT] = ACTIONS(452),
    [anon_sym_CARET] = ACTIONS(452),
    [anon_sym_AMP] = ACTIONS(584),
    [anon_sym_PIPE] = ACTIONS(444),
    [anon_sym_AMP_AMP] = ACTIONS(450),
    [anon_sym_PIPE_PIPE] = ACTIONS(450),
    [anon_sym_LT_LT] = ACTIONS(452),
    [anon_sym_GT_GT] = ACTIONS(452),
    [anon_sym_PLUS_EQ] = ACTIONS(450),
    [anon_sym_DASH_EQ] = ACTIONS(450),
    [anon_sym_STAR_EQ] = ACTIONS(450),
    [anon_sym_SLASH_EQ] = ACTIONS(450),
    [anon_sym_PERCENT_EQ] = ACTIONS(450),
    [anon_sym_CARET_EQ] = ACTIONS(450),
    [anon_sym_AMP_EQ] = ACTIONS(450),
    [anon_sym_PIPE_EQ] = ACTIONS(450),
    [anon_sym_LT_LT_EQ] = ACTIONS(450),
    [anon_sym_GT_GT_EQ] = ACTIONS(450),
    [anon_sym_EQ] = ACTIONS(452),
    [anon_sym_EQ_EQ] = ACTIONS(450),
    [anon_sym_BANG_EQ] = ACTIONS(450),
    [anon_sym_GT] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(446),
    [anon_sym_GT_EQ] = ACTIONS(450),
    [anon_sym_LT_EQ] = ACTIONS(450),
    [anon_sym_DOT] = ACTIONS(452),
    [anon_sym_DOT_DOT] = ACTIONS(586),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(450),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(450),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_as] = ACTIONS(452),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(74)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_function_modifiers] = STATE(6468),
    [aux_sym__item_modifiers] = STATE(3782),
    [sym_extern_modifier] = STATE(4856),
    [sym_visibility_modifier] = STATE(3498),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3492),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(492),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(451),
    [sym_match_expression] = STATE(451),
    [sym_while_expression] = STATE(451),
    [sym_loop_expression] = STATE(451),
    [sym_for_expression] = STATE(451),
    [sym_const_block] = STATE(451),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(451),
    [sym_async_block] = STATE(451),
    [sym_gen_block] = STATE(451),
    [sym_try_block] = STATE(451),
    [sym_block] = STATE(451),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(74),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(74),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(588),
    [anon_sym_macro] = ACTIONS(590),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(592),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(594),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(596),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(598),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(600),
    [anon_sym_fn] = ACTIONS(602),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(604),
    [anon_sym_let] = ACTIONS(606),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(608),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(610),
    [anon_sym_struct] = ACTIONS(612),
    [anon_sym_trait] = ACTIONS(614),
    [anon_sym_type] = ACTIONS(616),
    [anon_sym_union] = ACTIONS(618),
    [anon_sym_unsafe] = ACTIONS(620),
    [anon_sym_use] = ACTIONS(622),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(624),
    [anon_sym_safe] = ACTIONS(626),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(75)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_function_modifiers] = STATE(6468),
    [aux_sym__item_modifiers] = STATE(3782),
    [sym_extern_modifier] = STATE(4856),
    [sym_visibility_modifier] = STATE(3498),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3492),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(450),
    [sym_scoped_identifier] = STATE(3072),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(451),
    [sym_match_expression] = STATE(451),
    [sym_while_expression] = STATE(451),
    [sym_loop_expression] = STATE(451),
    [sym_for_expression] = STATE(451),
    [sym_const_block] = STATE(451),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6099),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(451),
    [sym_async_block] = STATE(451),
    [sym_gen_block] = STATE(451),
    [sym_try_block] = STATE(451),
    [sym_block] = STATE(451),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(75),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(75),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_SEMI] = ACTIONS(588),
    [anon_sym_macro] = ACTIONS(590),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [anon_sym_macro_rules] = ACTIONS(592),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(594),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(41),
    [anon_sym_auto] = ACTIONS(596),
    [anon_sym_await] = ACTIONS(45),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(598),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(55),
    [anon_sym_enum] = ACTIONS(600),
    [anon_sym_fn] = ACTIONS(602),
    [anon_sym_for] = ACTIONS(61),
    [anon_sym_gen] = ACTIONS(63),
    [anon_sym_if] = ACTIONS(65),
    [anon_sym_impl] = ACTIONS(604),
    [anon_sym_let] = ACTIONS(606),
    [anon_sym_loop] = ACTIONS(71),
    [anon_sym_match] = ACTIONS(73),
    [anon_sym_mod] = ACTIONS(608),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(45),
    [anon_sym_static] = ACTIONS(610),
    [anon_sym_struct] = ACTIONS(612),
    [anon_sym_trait] = ACTIONS(614),
    [anon_sym_type] = ACTIONS(616),
    [anon_sym_union] = ACTIONS(618),
    [anon_sym_unsafe] = ACTIONS(620),
    [anon_sym_use] = ACTIONS(622),
    [anon_sym_while] = ACTIONS(95),
    [anon_sym_extern] = ACTIONS(624),
    [anon_sym_safe] = ACTIONS(626),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(105),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(119),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(76)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4627),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(76),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(76),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(77)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4858),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(77),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(77),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(78)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4492),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(78),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(78),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(79)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(250),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3480),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_let_condition] = STATE(5692),
    [sym__let_chain] = STATE(5693),
    [sym__condition] = STATE(6010),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(79),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(79),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(640),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_let] = ACTIONS(642),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(80)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4585),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(80),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(80),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(81)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4695),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(81),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(81),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(82)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4548),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(82),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(82),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(83)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4616),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(83),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(83),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(84)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4676),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(84),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(84),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(85)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4555),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(85),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(85),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(86)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4558),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(86),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(86),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(87)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4609),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(87),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(87),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(88)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3300),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(5024),
    [sym__let_chain] = STATE(5045),
    [sym__condition] = STATE(4611),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(88),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(88),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(89)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3387),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(89),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(89),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_field_initializer_list_repeat1] = STATE(5198),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(644),
    [anon_sym_COMMA] = ACTIONS(646),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(90)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3387),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(90),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(90),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_field_initializer_list_repeat1] = STATE(5123),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(648),
    [anon_sym_COMMA] = ACTIONS(650),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(91)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3401),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(91),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(91),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(109),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(652),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(92)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3265),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3288),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(92),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(92),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(460),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(654),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_DASH_GT] = ACTIONS(658),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(93)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3382),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2899),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(93),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(93),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(552),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(664),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(668),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(94)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3038),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2941),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(94),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(94),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(670),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(674),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(95)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3158),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(4456),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(95),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(95),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(96)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3291),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_let_condition] = STATE(4456),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(96),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(96),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_let] = ACTIONS(634),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(97)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3141),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2877),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(97),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(97),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(522),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(678),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(680),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(98)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(278),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3093),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(98),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(98),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_COMMA] = ACTIONS(682),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(684),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(99)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(102),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3091),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(99),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(99),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_COMMA] = ACTIONS(686),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(688),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(100)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(98),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3098),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(100),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(100),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_COMMA] = ACTIONS(690),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(692),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(101)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3025),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2899),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(101),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(101),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(664),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(668),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(102)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(290),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3119),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(102),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(102),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_COMMA] = ACTIONS(694),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(696),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(103)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3139),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(103),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(103),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(698),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_COMMA] = ACTIONS(700),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(104)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3427),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2941),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(104),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(104),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(552),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(670),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(674),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(105)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3153),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2899),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(105),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(105),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(522),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(664),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(668),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(106)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3287),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2877),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(106),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(106),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(552),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(678),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(680),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(107)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3384),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(107),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(107),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(115),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(702),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(108)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3434),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(108),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(108),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(111),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(109)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3434),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(109),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(109),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(116),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(704),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(110)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3384),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(110),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(110),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(116),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(702),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(111)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3196),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(111),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(111),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(116),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(706),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(112)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3154),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2941),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(112),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(112),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(522),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(670),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(674),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(113)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3172),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(113),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(113),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(708),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_COMMA] = ACTIONS(710),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(114)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3385),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(114),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(114),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(110),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(712),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(115)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3290),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(115),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(115),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(116),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(714),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(116)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3476),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(116),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(116),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [aux_sym_tuple_expression_repeat1] = STATE(116),
    [sym_identifier] = ACTIONS(716),
    [anon_sym_LPAREN] = ACTIONS(719),
    [anon_sym_RPAREN] = ACTIONS(722),
    [anon_sym_LBRACE] = ACTIONS(724),
    [anon_sym_macro_rules] = ACTIONS(727),
    [anon_sym_BANG] = ACTIONS(730),
    [anon_sym_LBRACK] = ACTIONS(733),
    [anon_sym_STAR] = ACTIONS(730),
    [anon_sym_u8] = ACTIONS(736),
    [anon_sym_i8] = ACTIONS(736),
    [anon_sym_u16] = ACTIONS(736),
    [anon_sym_i16] = ACTIONS(736),
    [anon_sym_u32] = ACTIONS(736),
    [anon_sym_i32] = ACTIONS(736),
    [anon_sym_u64] = ACTIONS(736),
    [anon_sym_i64] = ACTIONS(736),
    [anon_sym_u128] = ACTIONS(736),
    [anon_sym_i128] = ACTIONS(736),
    [anon_sym_isize] = ACTIONS(736),
    [anon_sym_usize] = ACTIONS(736),
    [anon_sym_f32] = ACTIONS(736),
    [anon_sym_f64] = ACTIONS(736),
    [anon_sym_bool] = ACTIONS(736),
    [anon_sym_str] = ACTIONS(736),
    [anon_sym_char] = ACTIONS(736),
    [anon_sym_DASH] = ACTIONS(730),
    [anon_sym_AMP] = ACTIONS(739),
    [anon_sym_PIPE] = ACTIONS(742),
    [anon_sym_LT] = ACTIONS(745),
    [anon_sym_DOT_DOT] = ACTIONS(748),
    [anon_sym_COLON_COLON] = ACTIONS(751),
    [anon_sym_POUND] = ACTIONS(754),
    [anon_sym_SQUOTE] = ACTIONS(757),
    [anon_sym_async] = ACTIONS(760),
    [anon_sym_auto] = ACTIONS(727),
    [anon_sym_await] = ACTIONS(727),
    [anon_sym_become] = ACTIONS(763),
    [anon_sym_break] = ACTIONS(766),
    [anon_sym_const] = ACTIONS(769),
    [anon_sym_continue] = ACTIONS(772),
    [anon_sym_default] = ACTIONS(727),
    [anon_sym_for] = ACTIONS(775),
    [anon_sym_gen] = ACTIONS(778),
    [anon_sym_if] = ACTIONS(781),
    [anon_sym_loop] = ACTIONS(784),
    [anon_sym_match] = ACTIONS(787),
    [anon_sym_return] = ACTIONS(790),
    [anon_sym_raw] = ACTIONS(727),
    [anon_sym_static] = ACTIONS(793),
    [anon_sym_union] = ACTIONS(727),
    [anon_sym_unsafe] = ACTIONS(796),
    [anon_sym_while] = ACTIONS(799),
    [anon_sym_safe] = ACTIONS(727),
    [anon_sym_yield] = ACTIONS(802),
    [anon_sym_move] = ACTIONS(805),
    [anon_sym_try] = ACTIONS(808),
    [sym_integer_literal] = ACTIONS(811),
    [aux_sym_string_literal_token1] = ACTIONS(814),
    [sym_char_literal] = ACTIONS(811),
    [anon_sym_true] = ACTIONS(817),
    [anon_sym_false] = ACTIONS(817),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(820),
    [sym_super] = ACTIONS(823),
    [sym_crate] = ACTIONS(823),
    [aux_sym_metavariable_token1] = ACTIONS(826),
    [sym__raw_string_literal_start] = ACTIONS(829),
    [sym_float_literal] = ACTIONS(811),
    [sym__outer_block_doc_comment_start] = ACTIONS(832),
    [sym__outer_line_doc_comment_start] = ACTIONS(835),
  },
  [STATE(117)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3387),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(117),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(117),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_RBRACE] = ACTIONS(838),
    [anon_sym_COMMA] = ACTIONS(838),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(118)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3044),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2917),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(118),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(118),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(840),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(842),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(119)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3314),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2917),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(119),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(119),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(840),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(842),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(120)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3335),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2899),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(120),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(120),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(664),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(668),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(121)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3180),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2917),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(121),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(121),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(522),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(840),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(842),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(122)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3362),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_let_condition] = STATE(4456),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(122),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(122),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_let] = ACTIONS(642),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(123)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3389),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2941),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(123),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(123),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(670),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(674),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(124)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3433),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2877),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(124),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(124),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(678),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(680),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(125)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(250),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3458),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_let_condition] = STATE(4456),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(125),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(125),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(640),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_let] = ACTIONS(642),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(126)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3233),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3299),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(126),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(126),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(460),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(844),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_DASH_GT] = ACTIONS(846),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(127)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3241),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3328),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(127),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(127),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(460),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(848),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_DASH_GT] = ACTIONS(850),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(128)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3368),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2917),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(128),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(128),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(552),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(840),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_DASH_GT] = ACTIONS(842),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(129)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3257),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3228),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(129),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(129),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(460),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(852),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_DASH_GT] = ACTIONS(854),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(130)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3018),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2877),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(130),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(130),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(390),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(678),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_DASH_GT] = ACTIONS(680),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(131)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(131),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(131),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(856),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(132)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(132),
    [sym_block_comment] = STATE(132),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(858),
    [anon_sym_SEMI] = ACTIONS(861),
    [anon_sym_LPAREN] = ACTIONS(864),
    [anon_sym_RPAREN] = ACTIONS(867),
    [anon_sym_LBRACE] = ACTIONS(869),
    [anon_sym_RBRACE] = ACTIONS(867),
    [anon_sym_COMMA] = ACTIONS(861),
    [anon_sym_EQ_GT] = ACTIONS(861),
    [anon_sym_BANG] = ACTIONS(872),
    [anon_sym_LBRACK] = ACTIONS(875),
    [anon_sym_RBRACK] = ACTIONS(867),
    [anon_sym_COLON] = ACTIONS(872),
    [anon_sym_DOLLAR] = ACTIONS(878),
    [anon_sym_PLUS] = ACTIONS(872),
    [anon_sym_STAR] = ACTIONS(872),
    [anon_sym_QMARK] = ACTIONS(861),
    [anon_sym_u8] = ACTIONS(881),
    [anon_sym_i8] = ACTIONS(881),
    [anon_sym_u16] = ACTIONS(881),
    [anon_sym_i16] = ACTIONS(881),
    [anon_sym_u32] = ACTIONS(881),
    [anon_sym_i32] = ACTIONS(881),
    [anon_sym_u64] = ACTIONS(881),
    [anon_sym_i64] = ACTIONS(881),
    [anon_sym_u128] = ACTIONS(881),
    [anon_sym_i128] = ACTIONS(881),
    [anon_sym_isize] = ACTIONS(881),
    [anon_sym_usize] = ACTIONS(881),
    [anon_sym_f32] = ACTIONS(881),
    [anon_sym_f64] = ACTIONS(881),
    [anon_sym_bool] = ACTIONS(881),
    [anon_sym_str] = ACTIONS(881),
    [anon_sym_char] = ACTIONS(881),
    [anon_sym_DASH] = ACTIONS(872),
    [anon_sym_SLASH] = ACTIONS(872),
    [anon_sym_PERCENT] = ACTIONS(872),
    [anon_sym_CARET] = ACTIONS(872),
    [anon_sym_AMP] = ACTIONS(872),
    [anon_sym_PIPE] = ACTIONS(872),
    [anon_sym_AMP_AMP] = ACTIONS(861),
    [anon_sym_PIPE_PIPE] = ACTIONS(861),
    [anon_sym_LT_LT] = ACTIONS(872),
    [anon_sym_GT_GT] = ACTIONS(872),
    [anon_sym_PLUS_EQ] = ACTIONS(861),
    [anon_sym_DASH_EQ] = ACTIONS(861),
    [anon_sym_STAR_EQ] = ACTIONS(861),
    [anon_sym_SLASH_EQ] = ACTIONS(861),
    [anon_sym_PERCENT_EQ] = ACTIONS(861),
    [anon_sym_CARET_EQ] = ACTIONS(861),
    [anon_sym_AMP_EQ] = ACTIONS(861),
    [anon_sym_PIPE_EQ] = ACTIONS(861),
    [anon_sym_LT_LT_EQ] = ACTIONS(861),
    [anon_sym_GT_GT_EQ] = ACTIONS(861),
    [anon_sym_EQ] = ACTIONS(872),
    [anon_sym_EQ_EQ] = ACTIONS(861),
    [anon_sym_BANG_EQ] = ACTIONS(861),
    [anon_sym_GT] = ACTIONS(872),
    [anon_sym_LT] = ACTIONS(872),
    [anon_sym_GT_EQ] = ACTIONS(861),
    [anon_sym_LT_EQ] = ACTIONS(861),
    [anon_sym_AT] = ACTIONS(861),
    [anon_sym__] = ACTIONS(872),
    [anon_sym_DOT] = ACTIONS(872),
    [anon_sym_DOT_DOT] = ACTIONS(872),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(861),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(861),
    [anon_sym_COLON_COLON] = ACTIONS(861),
    [anon_sym_DASH_GT] = ACTIONS(861),
    [anon_sym_POUND] = ACTIONS(861),
    [anon_sym_SQUOTE] = ACTIONS(858),
    [anon_sym_as] = ACTIONS(858),
    [anon_sym_async] = ACTIONS(858),
    [anon_sym_auto] = ACTIONS(858),
    [anon_sym_await] = ACTIONS(858),
    [anon_sym_become] = ACTIONS(858),
    [anon_sym_break] = ACTIONS(858),
    [anon_sym_const] = ACTIONS(858),
    [anon_sym_continue] = ACTIONS(858),
    [anon_sym_default] = ACTIONS(858),
    [anon_sym_enum] = ACTIONS(858),
    [anon_sym_fn] = ACTIONS(858),
    [anon_sym_for] = ACTIONS(858),
    [anon_sym_gen] = ACTIONS(858),
    [anon_sym_if] = ACTIONS(858),
    [anon_sym_impl] = ACTIONS(858),
    [anon_sym_let] = ACTIONS(858),
    [anon_sym_loop] = ACTIONS(858),
    [anon_sym_match] = ACTIONS(858),
    [anon_sym_mod] = ACTIONS(858),
    [anon_sym_pub] = ACTIONS(858),
    [anon_sym_return] = ACTIONS(858),
    [anon_sym_raw] = ACTIONS(858),
    [anon_sym_static] = ACTIONS(858),
    [anon_sym_struct] = ACTIONS(858),
    [anon_sym_trait] = ACTIONS(858),
    [anon_sym_type] = ACTIONS(858),
    [anon_sym_union] = ACTIONS(858),
    [anon_sym_unsafe] = ACTIONS(858),
    [anon_sym_use] = ACTIONS(858),
    [anon_sym_where] = ACTIONS(858),
    [anon_sym_while] = ACTIONS(858),
    [sym_mutable_specifier] = ACTIONS(858),
    [sym_integer_literal] = ACTIONS(884),
    [aux_sym_string_literal_token1] = ACTIONS(887),
    [sym_char_literal] = ACTIONS(884),
    [anon_sym_true] = ACTIONS(890),
    [anon_sym_false] = ACTIONS(890),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(858),
    [sym_super] = ACTIONS(858),
    [sym_crate] = ACTIONS(858),
    [aux_sym_metavariable_token1] = ACTIONS(893),
    [sym__raw_string_literal_start] = ACTIONS(896),
    [sym_float_literal] = ACTIONS(884),
  },
  [STATE(133)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(133),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(133),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(899),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(134)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3269),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(134),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(134),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(135)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(135),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(135),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(136)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5410),
    [sym_variadic_parameter] = STATE(5410),
    [sym_parameter] = STATE(5410),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4850),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4279),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(136),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(136),
    [sym_metavariable] = STATE(4014),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_RPAREN] = ACTIONS(909),
    [anon_sym_COMMA] = ACTIONS(911),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(927),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(931),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(977),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(137)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(286),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3324),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1987),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(568),
    [sym_match_expression] = STATE(568),
    [sym_while_expression] = STATE(568),
    [sym_loop_expression] = STATE(568),
    [sym_for_expression] = STATE(568),
    [sym_const_block] = STATE(568),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6412),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(568),
    [sym_async_block] = STATE(568),
    [sym_gen_block] = STATE(568),
    [sym_try_block] = STATE(568),
    [sym_block] = STATE(568),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(137),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(137),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(989),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(991),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(993),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_gen] = ACTIONS(997),
    [anon_sym_if] = ACTIONS(999),
    [anon_sym_loop] = ACTIONS(1001),
    [anon_sym_match] = ACTIONS(1003),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(1005),
    [anon_sym_while] = ACTIONS(1007),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(1009),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(138)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(138),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(138),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(1011),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(139)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(4974),
    [sym_variadic_parameter] = STATE(4974),
    [sym_parameter] = STATE(4974),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4536),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(139),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(139),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1017),
    [anon_sym_COMMA] = ACTIONS(1019),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1027),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(140)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3269),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(140),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(140),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1037),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(141)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(141),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(141),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(1039),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(142)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(286),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3361),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1984),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(569),
    [sym_match_expression] = STATE(569),
    [sym_while_expression] = STATE(569),
    [sym_loop_expression] = STATE(569),
    [sym_for_expression] = STATE(569),
    [sym_const_block] = STATE(569),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6412),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(569),
    [sym_async_block] = STATE(569),
    [sym_gen_block] = STATE(569),
    [sym_try_block] = STATE(569),
    [sym_block] = STATE(569),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(142),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(142),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(989),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(991),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(993),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_gen] = ACTIONS(997),
    [anon_sym_if] = ACTIONS(999),
    [anon_sym_loop] = ACTIONS(1001),
    [anon_sym_match] = ACTIONS(1003),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(1005),
    [anon_sym_while] = ACTIONS(1007),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(1009),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(143)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(286),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3472),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1987),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(568),
    [sym_match_expression] = STATE(568),
    [sym_while_expression] = STATE(568),
    [sym_loop_expression] = STATE(568),
    [sym_for_expression] = STATE(568),
    [sym_const_block] = STATE(568),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6412),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(568),
    [sym_async_block] = STATE(568),
    [sym_gen_block] = STATE(568),
    [sym_try_block] = STATE(568),
    [sym_block] = STATE(568),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(143),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(143),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(989),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(991),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(993),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_gen] = ACTIONS(997),
    [anon_sym_if] = ACTIONS(999),
    [anon_sym_loop] = ACTIONS(1001),
    [anon_sym_match] = ACTIONS(1003),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(1005),
    [anon_sym_while] = ACTIONS(1007),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(1009),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(144)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(286),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3467),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_block_expression_with_attribute] = STATE(1984),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(569),
    [sym_match_expression] = STATE(569),
    [sym_while_expression] = STATE(569),
    [sym_loop_expression] = STATE(569),
    [sym_for_expression] = STATE(569),
    [sym_const_block] = STATE(569),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6412),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(569),
    [sym_async_block] = STATE(569),
    [sym_gen_block] = STATE(569),
    [sym_try_block] = STATE(569),
    [sym_block] = STATE(569),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(144),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(144),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(989),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(991),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(993),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_gen] = ACTIONS(997),
    [anon_sym_if] = ACTIONS(999),
    [anon_sym_loop] = ACTIONS(1001),
    [anon_sym_match] = ACTIONS(1003),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(1005),
    [anon_sym_while] = ACTIONS(1007),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(1009),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(145)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3370),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(145),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(145),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(1041),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [sym_mutable_specifier] = ACTIONS(1043),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(146)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(285),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3375),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(146),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(146),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1045),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(147)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2896),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(147),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(147),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(1047),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [sym_mutable_specifier] = ACTIONS(1049),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(148)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(148),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(148),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(1051),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(149)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5410),
    [sym_variadic_parameter] = STATE(5410),
    [sym_parameter] = STATE(5410),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4850),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4279),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(149),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(149),
    [sym_metavariable] = STATE(4014),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_RPAREN] = ACTIONS(1053),
    [anon_sym_COMMA] = ACTIONS(1055),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(927),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(931),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(977),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(150)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3269),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(150),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(150),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1057),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(151)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5410),
    [sym_variadic_parameter] = STATE(5410),
    [sym_parameter] = STATE(5410),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4850),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(151),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(151),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1059),
    [anon_sym_COMMA] = ACTIONS(1061),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1063),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(152)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(152),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(152),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(1065),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(153)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(153),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(153),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_RBRACK] = ACTIONS(1067),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(154)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2896),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(154),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(154),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(1069),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [sym_mutable_specifier] = ACTIONS(1071),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(155)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(221),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3332),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(155),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(155),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1073),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(156)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(4916),
    [sym_variadic_parameter] = STATE(4916),
    [sym_parameter] = STATE(4916),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4580),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(156),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(156),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1075),
    [anon_sym_COMMA] = ACTIONS(1077),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1079),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(157)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2896),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(157),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(157),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(1081),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [sym_mutable_specifier] = ACTIONS(1083),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(158)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3269),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(158),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(158),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_RPAREN] = ACTIONS(1085),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(159)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5410),
    [sym_variadic_parameter] = STATE(5410),
    [sym_parameter] = STATE(5410),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4850),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4279),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(159),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(159),
    [sym_metavariable] = STATE(4014),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_RPAREN] = ACTIONS(1087),
    [anon_sym_COMMA] = ACTIONS(1089),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(927),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(931),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(977),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(160)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3311),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(160),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(160),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(161)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3110),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(161),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(161),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(162)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3199),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(162),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(162),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(163)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3204),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(163),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(163),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(164)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2916),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(164),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(164),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(165)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3475),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(165),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(165),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(166)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3284),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(166),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(166),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(167)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3473),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(167),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(167),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(168)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3454),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(168),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(168),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(169)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3216),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(169),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(169),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(170)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3462),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(170),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(170),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(171)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3452),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(171),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(171),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(172)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3442),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(172),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(172),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(173)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3448),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(173),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(173),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(174)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3459),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(174),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(174),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(175)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3357),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(175),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(175),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(176)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(220),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3333),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(176),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(176),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(1091),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(177)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3496),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(177),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(177),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(178)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3198),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(178),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(178),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(179)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3088),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(179),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(179),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(180)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3460),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(180),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(180),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(181)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3022),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(181),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(181),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(182)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3492),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(182),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(182),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(183)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2985),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(183),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(183),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(184)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3416),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(184),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(184),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(185)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2869),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(185),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(185),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(186)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3147),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(186),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(186),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(187)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2869),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(187),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(187),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(188)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3152),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(188),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(188),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(189)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3126),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(189),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(189),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(190)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3155),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(190),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(190),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(191)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3394),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(191),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(191),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(192)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3160),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(192),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(192),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(193)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3164),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(193),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(193),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(194)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3166),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(194),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(194),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(195)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3170),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(195),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(195),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(196)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3173),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(196),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(196),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(197)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3041),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(197),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(197),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(198)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3036),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(198),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(198),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(199)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2916),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(199),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(199),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(200)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3019),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(200),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(200),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(201)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3020),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(201),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(201),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(202)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(250),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3449),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(202),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(202),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(640),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(203)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3493),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(203),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(203),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(204)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2985),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(204),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(204),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(205)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3390),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(205),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(205),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(206)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2869),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(206),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(206),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(207)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3391),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(207),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(207),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(208)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3392),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(208),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(208),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(209)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3393),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(209),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(209),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(210)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3158),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(210),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(210),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(211)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3395),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(211),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(211),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(212)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3396),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(212),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(212),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(213)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3397),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(213),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(213),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(214)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3398),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(214),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(214),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(215)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3399),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(215),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(215),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(216)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3181),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(216),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(216),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(217)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2916),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(217),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(217),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(218)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3029),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(218),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(218),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(219)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3039),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(219),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(219),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(220)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3489),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(220),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(220),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(1091),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(221)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3481),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(221),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(221),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(222)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3468),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(222),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(222),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(223)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3351),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(223),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(223),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(224)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3409),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(224),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(224),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(225)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3352),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(225),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(225),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(226)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3353),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(226),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(226),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(227)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3354),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(227),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(227),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(228)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3362),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(228),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(228),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(229)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3363),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(229),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(229),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(230)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3364),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(230),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(230),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(231)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3365),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(231),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(231),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(232)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3367),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(232),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(232),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(233)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3369),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(233),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(233),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(234)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3040),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(234),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(234),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(235)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(220),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3400),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(235),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(235),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(1091),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(236)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3307),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(236),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(236),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(237)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3234),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(237),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(237),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(238)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3483),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(238),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(238),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(239)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3490),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(239),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(239),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(240)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3451),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(240),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(240),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(241)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3453),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(241),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(241),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(242)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3181),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(242),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(242),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(243)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3386),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(243),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(243),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(244)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3042),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(244),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(244),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(245)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3285),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(245),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(245),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(246)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3494),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(246),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(246),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(247)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3021),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(247),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(247),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(248)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3112),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(248),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(248),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(249)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3404),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(249),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(249),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(250)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3491),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(3252),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(250),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(250),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(640),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(251)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3405),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(251),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(251),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(252)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(250),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3372),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(252),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(252),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(640),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(253)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3197),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(253),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(253),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(254)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3495),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(3252),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(254),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(254),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(255)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3023),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(255),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(255),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(256)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(254),
    [sym_bracketed_type] = STATE(6264),
    [sym_generic_function] = STATE(3316),
    [sym_generic_type_with_turbofish] = STATE(5193),
    [sym__expression_except_range] = STATE(3178),
    [sym__expression] = STATE(3372),
    [sym_expression_with_attribute] = STATE(3318),
    [sym__expression_without_attribute] = STATE(3318),
    [sym_macro_invocation] = STATE(3344),
    [sym_scoped_identifier] = STATE(3080),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5631),
    [sym_range_expression] = STATE(3323),
    [sym_unary_expression] = STATE(3316),
    [sym_try_expression] = STATE(3316),
    [sym_reference_expression] = STATE(3316),
    [sym_binary_expression] = STATE(3316),
    [sym_assignment_expression] = STATE(3316),
    [sym_compound_assignment_expr] = STATE(3316),
    [sym_type_cast_expression] = STATE(3316),
    [sym_return_expression] = STATE(3316),
    [sym_become_expression] = STATE(3316),
    [sym_yield_expression] = STATE(3316),
    [sym_call_expression] = STATE(3316),
    [sym_array_expression] = STATE(3316),
    [sym_parenthesized_expression] = STATE(3316),
    [sym_tuple_expression] = STATE(3316),
    [sym_unit_expression] = STATE(3316),
    [sym_struct_expression] = STATE(3316),
    [sym_if_expression] = STATE(3316),
    [sym_match_expression] = STATE(3316),
    [sym_while_expression] = STATE(3316),
    [sym_loop_expression] = STATE(3316),
    [sym_for_expression] = STATE(3316),
    [sym_const_block] = STATE(3316),
    [sym_closure_expression] = STATE(3316),
    [sym_closure_parameters] = STATE(126),
    [sym_label] = STATE(6425),
    [sym_break_expression] = STATE(3316),
    [sym_continue_expression] = STATE(3316),
    [sym_index_expression] = STATE(3316),
    [sym_await_expression] = STATE(3316),
    [sym_field_expression] = STATE(3145),
    [sym_unsafe_block] = STATE(3316),
    [sym_async_block] = STATE(3316),
    [sym_gen_block] = STATE(3316),
    [sym_try_block] = STATE(3316),
    [sym_block] = STATE(3316),
    [sym__literal] = STATE(3316),
    [sym_string_literal] = STATE(3203),
    [sym_raw_string_literal] = STATE(3203),
    [sym_boolean_literal] = STATE(3203),
    [sym_line_comment] = STATE(256),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(256),
    [sym_metavariable] = STATE(3159),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(516),
    [anon_sym_LBRACE] = ACTIONS(456),
    [anon_sym_macro_rules] = ACTIONS(458),
    [anon_sym_BANG] = ACTIONS(636),
    [anon_sym_LBRACK] = ACTIONS(574),
    [anon_sym_STAR] = ACTIONS(636),
    [anon_sym_u8] = ACTIONS(464),
    [anon_sym_i8] = ACTIONS(464),
    [anon_sym_u16] = ACTIONS(464),
    [anon_sym_i16] = ACTIONS(464),
    [anon_sym_u32] = ACTIONS(464),
    [anon_sym_i32] = ACTIONS(464),
    [anon_sym_u64] = ACTIONS(464),
    [anon_sym_i64] = ACTIONS(464),
    [anon_sym_u128] = ACTIONS(464),
    [anon_sym_i128] = ACTIONS(464),
    [anon_sym_isize] = ACTIONS(464),
    [anon_sym_usize] = ACTIONS(464),
    [anon_sym_f32] = ACTIONS(464),
    [anon_sym_f64] = ACTIONS(464),
    [anon_sym_bool] = ACTIONS(464),
    [anon_sym_str] = ACTIONS(464),
    [anon_sym_char] = ACTIONS(464),
    [anon_sym_DASH] = ACTIONS(636),
    [anon_sym_AMP] = ACTIONS(638),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(656),
    [anon_sym_COLON_COLON] = ACTIONS(466),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(468),
    [anon_sym_auto] = ACTIONS(458),
    [anon_sym_await] = ACTIONS(458),
    [anon_sym_become] = ACTIONS(470),
    [anon_sym_break] = ACTIONS(472),
    [anon_sym_const] = ACTIONS(474),
    [anon_sym_continue] = ACTIONS(476),
    [anon_sym_default] = ACTIONS(458),
    [anon_sym_for] = ACTIONS(478),
    [anon_sym_gen] = ACTIONS(480),
    [anon_sym_if] = ACTIONS(482),
    [anon_sym_loop] = ACTIONS(484),
    [anon_sym_match] = ACTIONS(486),
    [anon_sym_return] = ACTIONS(488),
    [anon_sym_raw] = ACTIONS(458),
    [anon_sym_static] = ACTIONS(490),
    [anon_sym_union] = ACTIONS(458),
    [anon_sym_unsafe] = ACTIONS(492),
    [anon_sym_while] = ACTIONS(494),
    [anon_sym_safe] = ACTIONS(458),
    [anon_sym_yield] = ACTIONS(496),
    [anon_sym_move] = ACTIONS(498),
    [anon_sym_try] = ACTIONS(500),
    [sym_integer_literal] = ACTIONS(502),
    [aux_sym_string_literal_token1] = ACTIONS(504),
    [sym_char_literal] = ACTIONS(502),
    [anon_sym_true] = ACTIONS(506),
    [anon_sym_false] = ACTIONS(506),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(508),
    [sym_super] = ACTIONS(510),
    [sym_crate] = ACTIONS(510),
    [aux_sym_metavariable_token1] = ACTIONS(512),
    [sym__raw_string_literal_start] = ACTIONS(514),
    [sym_float_literal] = ACTIONS(502),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(257)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2888),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(257),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(257),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(258)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2888),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(258),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(258),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(259)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(239),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3131),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(259),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(259),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(676),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(260)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3036),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(260),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(260),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(261)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2985),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(261),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(261),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(262)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(177),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3033),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(262),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(262),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(672),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(263)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(237),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3211),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(263),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(263),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(264)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5758),
    [sym_variadic_parameter] = STATE(5758),
    [sym_parameter] = STATE(5758),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4993),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(264),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(264),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1093),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1095),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(265)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5758),
    [sym_variadic_parameter] = STATE(5758),
    [sym_parameter] = STATE(5758),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4993),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(265),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(265),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1097),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1095),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(266)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3484),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(266),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(266),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(267)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3419),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(267),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(267),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(268)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5758),
    [sym_variadic_parameter] = STATE(5758),
    [sym_parameter] = STATE(5758),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4993),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(268),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(268),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1099),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1095),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(269)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3320),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(269),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(269),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(270)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(2888),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(270),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(270),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(271)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3220),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(271),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(271),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(272)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3446),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(272),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(272),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(273)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3312),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(273),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(273),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(274)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5758),
    [sym_variadic_parameter] = STATE(5758),
    [sym_parameter] = STATE(5758),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4993),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(274),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(274),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1101),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1095),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(275)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3313),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(275),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(275),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(276)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5758),
    [sym_variadic_parameter] = STATE(5758),
    [sym_parameter] = STATE(5758),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4993),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(276),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(276),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1103),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1095),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(277)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3315),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(277),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(277),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(278)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3137),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(278),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(278),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(279)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3116),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(279),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(279),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(280)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3486),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(280),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(280),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(281)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5758),
    [sym_variadic_parameter] = STATE(5758),
    [sym_parameter] = STATE(5758),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4993),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(281),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(281),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_RPAREN] = ACTIONS(1105),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1095),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(282)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3325),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(282),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(282),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(283)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3329),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(283),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(283),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(284)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3358),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(284),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(284),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(285)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3445),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(285),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(285),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(286)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3492),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(585),
    [sym_match_expression] = STATE(585),
    [sym_while_expression] = STATE(585),
    [sym_loop_expression] = STATE(585),
    [sym_for_expression] = STATE(585),
    [sym_const_block] = STATE(585),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6412),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(585),
    [sym_async_block] = STATE(585),
    [sym_gen_block] = STATE(585),
    [sym_try_block] = STATE(585),
    [sym_block] = STATE(585),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(286),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(286),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(989),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(991),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(993),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(995),
    [anon_sym_gen] = ACTIONS(997),
    [anon_sym_if] = ACTIONS(999),
    [anon_sym_loop] = ACTIONS(1001),
    [anon_sym_match] = ACTIONS(1003),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(1005),
    [anon_sym_while] = ACTIONS(1007),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(1009),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(287)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(220),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3350),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(287),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(287),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(1091),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(288)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3457),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(288),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(288),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(289)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3463),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(289),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(289),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(290)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3143),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2884),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(290),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(290),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(291)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3478),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(291),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(291),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(292)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3094),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(292),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(292),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(293)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3425),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(293),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(293),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(294)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3474),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(294),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(294),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(295)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3488),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(295),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(295),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(296)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3441),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(296),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(296),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(297)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3189),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(297),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(297),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(298)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3190),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(298),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(298),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(299)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3443),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(299),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(299),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(300)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3201),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(300),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(300),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(301)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3447),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(301),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(301),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(302)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3202),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(302),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(302),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(303)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3085),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(303),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(303),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(304)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3206),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(304),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(304),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(305)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3455),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(305),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(305),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(306)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3456),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(306),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(306),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(307)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3207),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(307),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(307),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(308)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3461),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(308),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(308),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(309)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3464),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(309),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(309),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(310)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3465),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(310),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(310),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(311)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3208),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(311),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(311),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(312)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3466),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(312),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(312),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(313)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3470),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(313),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(313),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(314)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3471),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(314),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(314),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(315)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3482),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(315),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(315),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(316)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3485),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(316),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(316),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(317)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3439),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(317),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(317),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(318)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3487),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(318),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(318),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(319)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3229),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(319),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(319),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(320)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(220),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3222),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(320),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(320),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(1091),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(321)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3469),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(321),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(321),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(322)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3092),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(322),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(322),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(323)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3479),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(323),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(323),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(324)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3477),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(324),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(324),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(325)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(203),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3101),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(121),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(325),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(325),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(628),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(628),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(628),
    [anon_sym_AMP] = ACTIONS(630),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(632),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(528),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(530),
    [anon_sym_break] = ACTIONS(532),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(534),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(536),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(538),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(540),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(542),
    [anon_sym_move] = ACTIONS(544),
    [anon_sym_try] = ACTIONS(546),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(326)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(182),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5161),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3269),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(2946),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(118),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(326),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(326),
    [sym_metavariable] = STATE(2991),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(382),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(388),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(21),
    [anon_sym_u8] = ACTIONS(25),
    [anon_sym_i8] = ACTIONS(25),
    [anon_sym_u16] = ACTIONS(25),
    [anon_sym_i16] = ACTIONS(25),
    [anon_sym_u32] = ACTIONS(25),
    [anon_sym_i32] = ACTIONS(25),
    [anon_sym_u64] = ACTIONS(25),
    [anon_sym_i64] = ACTIONS(25),
    [anon_sym_u128] = ACTIONS(25),
    [anon_sym_i128] = ACTIONS(25),
    [anon_sym_isize] = ACTIONS(25),
    [anon_sym_usize] = ACTIONS(25),
    [anon_sym_f32] = ACTIONS(25),
    [anon_sym_f64] = ACTIONS(25),
    [anon_sym_bool] = ACTIONS(25),
    [anon_sym_str] = ACTIONS(25),
    [anon_sym_char] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(21),
    [anon_sym_AMP] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(33),
    [anon_sym_COLON_COLON] = ACTIONS(35),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(398),
    [anon_sym_auto] = ACTIONS(388),
    [anon_sym_await] = ACTIONS(388),
    [anon_sym_become] = ACTIONS(47),
    [anon_sym_break] = ACTIONS(49),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(388),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(404),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(79),
    [anon_sym_raw] = ACTIONS(388),
    [anon_sym_static] = ACTIONS(412),
    [anon_sym_union] = ACTIONS(388),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(388),
    [anon_sym_yield] = ACTIONS(101),
    [anon_sym_move] = ACTIONS(103),
    [anon_sym_try] = ACTIONS(418),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(115),
    [sym_super] = ACTIONS(117),
    [sym_crate] = ACTIONS(117),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(327)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(220),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3248),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(327),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(327),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(1091),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(328)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_attributes] = STATE(246),
    [sym_bracketed_type] = STATE(6279),
    [sym_generic_function] = STATE(2921),
    [sym_generic_type_with_turbofish] = STATE(5067),
    [sym__expression_except_range] = STATE(2816),
    [sym__expression] = STATE(3400),
    [sym_expression_with_attribute] = STATE(2942),
    [sym__expression_without_attribute] = STATE(2942),
    [sym_macro_invocation] = STATE(2912),
    [sym_scoped_identifier] = STATE(3073),
    [sym_scoped_type_identifier_in_expression_position] = STATE(5558),
    [sym_range_expression] = STATE(2881),
    [sym_unary_expression] = STATE(2921),
    [sym_try_expression] = STATE(2921),
    [sym_reference_expression] = STATE(2921),
    [sym_binary_expression] = STATE(2921),
    [sym_assignment_expression] = STATE(2921),
    [sym_compound_assignment_expr] = STATE(2921),
    [sym_type_cast_expression] = STATE(2921),
    [sym_return_expression] = STATE(2921),
    [sym_become_expression] = STATE(2921),
    [sym_yield_expression] = STATE(2921),
    [sym_call_expression] = STATE(2921),
    [sym_array_expression] = STATE(2921),
    [sym_parenthesized_expression] = STATE(2921),
    [sym_tuple_expression] = STATE(2921),
    [sym_unit_expression] = STATE(2921),
    [sym_struct_expression] = STATE(2921),
    [sym_if_expression] = STATE(2921),
    [sym_match_expression] = STATE(2921),
    [sym_while_expression] = STATE(2921),
    [sym_loop_expression] = STATE(2921),
    [sym_for_expression] = STATE(2921),
    [sym_const_block] = STATE(2921),
    [sym_closure_expression] = STATE(2921),
    [sym_closure_parameters] = STATE(128),
    [sym_label] = STATE(6300),
    [sym_break_expression] = STATE(2921),
    [sym_continue_expression] = STATE(2921),
    [sym_index_expression] = STATE(2921),
    [sym_await_expression] = STATE(2921),
    [sym_field_expression] = STATE(2820),
    [sym_unsafe_block] = STATE(2921),
    [sym_async_block] = STATE(2921),
    [sym_gen_block] = STATE(2921),
    [sym_try_block] = STATE(2921),
    [sym_block] = STATE(2921),
    [sym__literal] = STATE(2921),
    [sym_string_literal] = STATE(2915),
    [sym_raw_string_literal] = STATE(2915),
    [sym_boolean_literal] = STATE(2915),
    [sym_line_comment] = STATE(328),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(328),
    [sym_metavariable] = STATE(3090),
    [aux_sym_attributes_repeat1] = STATE(1927),
    [sym_identifier] = ACTIONS(518),
    [anon_sym_LPAREN] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(386),
    [anon_sym_macro_rules] = ACTIONS(520),
    [anon_sym_BANG] = ACTIONS(660),
    [anon_sym_LBRACK] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_u8] = ACTIONS(524),
    [anon_sym_i8] = ACTIONS(524),
    [anon_sym_u16] = ACTIONS(524),
    [anon_sym_i16] = ACTIONS(524),
    [anon_sym_u32] = ACTIONS(524),
    [anon_sym_i32] = ACTIONS(524),
    [anon_sym_u64] = ACTIONS(524),
    [anon_sym_i64] = ACTIONS(524),
    [anon_sym_u128] = ACTIONS(524),
    [anon_sym_i128] = ACTIONS(524),
    [anon_sym_isize] = ACTIONS(524),
    [anon_sym_usize] = ACTIONS(524),
    [anon_sym_f32] = ACTIONS(524),
    [anon_sym_f64] = ACTIONS(524),
    [anon_sym_bool] = ACTIONS(524),
    [anon_sym_str] = ACTIONS(524),
    [anon_sym_char] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(662),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT] = ACTIONS(666),
    [anon_sym_COLON_COLON] = ACTIONS(526),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(39),
    [anon_sym_async] = ACTIONS(554),
    [anon_sym_auto] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(520),
    [anon_sym_become] = ACTIONS(556),
    [anon_sym_break] = ACTIONS(558),
    [anon_sym_const] = ACTIONS(400),
    [anon_sym_continue] = ACTIONS(53),
    [anon_sym_default] = ACTIONS(520),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_gen] = ACTIONS(560),
    [anon_sym_if] = ACTIONS(406),
    [anon_sym_loop] = ACTIONS(408),
    [anon_sym_match] = ACTIONS(410),
    [anon_sym_return] = ACTIONS(562),
    [anon_sym_raw] = ACTIONS(520),
    [anon_sym_static] = ACTIONS(564),
    [anon_sym_union] = ACTIONS(520),
    [anon_sym_unsafe] = ACTIONS(414),
    [anon_sym_while] = ACTIONS(416),
    [anon_sym_safe] = ACTIONS(520),
    [anon_sym_yield] = ACTIONS(566),
    [anon_sym_move] = ACTIONS(568),
    [anon_sym_try] = ACTIONS(570),
    [sym_integer_literal] = ACTIONS(107),
    [aux_sym_string_literal_token1] = ACTIONS(109),
    [sym_char_literal] = ACTIONS(107),
    [anon_sym_true] = ACTIONS(111),
    [anon_sym_false] = ACTIONS(111),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(548),
    [sym_super] = ACTIONS(550),
    [sym_crate] = ACTIONS(550),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(123),
    [sym_float_literal] = ACTIONS(107),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(329)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1963),
    [sym_attributes] = STATE(1075),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_self_parameter] = STATE(5758),
    [sym_variadic_parameter] = STATE(5758),
    [sym_parameter] = STATE(5758),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4993),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(5415),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5583),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(329),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(329),
    [sym_metavariable] = STATE(4067),
    [aux_sym_attributes_repeat1] = STATE(1948),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1025),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1095),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(935),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(967),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1033),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(330)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(330),
    [sym_block_comment] = STATE(330),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_RBRACE] = ACTIONS(1115),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(331)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(331),
    [sym_block_comment] = STATE(331),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_RBRACK] = ACTIONS(1115),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(332)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(332),
    [sym_block_comment] = STATE(332),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1135),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(333)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(333),
    [sym_block_comment] = STATE(333),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1137),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(334)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(334),
    [sym_block_comment] = STATE(334),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1139),
    [anon_sym_SEMI] = ACTIONS(1142),
    [anon_sym_LPAREN] = ACTIONS(1145),
    [anon_sym_RPAREN] = ACTIONS(1148),
    [anon_sym_LBRACE] = ACTIONS(1150),
    [anon_sym_RBRACE] = ACTIONS(1148),
    [anon_sym_COMMA] = ACTIONS(1142),
    [anon_sym_EQ_GT] = ACTIONS(1142),
    [anon_sym_BANG] = ACTIONS(1153),
    [anon_sym_LBRACK] = ACTIONS(1156),
    [anon_sym_RBRACK] = ACTIONS(1148),
    [anon_sym_COLON] = ACTIONS(1153),
    [anon_sym_DOLLAR] = ACTIONS(1159),
    [anon_sym_PLUS] = ACTIONS(1153),
    [anon_sym_STAR] = ACTIONS(1153),
    [anon_sym_QMARK] = ACTIONS(1142),
    [anon_sym_u8] = ACTIONS(1162),
    [anon_sym_i8] = ACTIONS(1162),
    [anon_sym_u16] = ACTIONS(1162),
    [anon_sym_i16] = ACTIONS(1162),
    [anon_sym_u32] = ACTIONS(1162),
    [anon_sym_i32] = ACTIONS(1162),
    [anon_sym_u64] = ACTIONS(1162),
    [anon_sym_i64] = ACTIONS(1162),
    [anon_sym_u128] = ACTIONS(1162),
    [anon_sym_i128] = ACTIONS(1162),
    [anon_sym_isize] = ACTIONS(1162),
    [anon_sym_usize] = ACTIONS(1162),
    [anon_sym_f32] = ACTIONS(1162),
    [anon_sym_f64] = ACTIONS(1162),
    [anon_sym_bool] = ACTIONS(1162),
    [anon_sym_str] = ACTIONS(1162),
    [anon_sym_char] = ACTIONS(1162),
    [anon_sym_DASH] = ACTIONS(1153),
    [anon_sym_SLASH] = ACTIONS(1153),
    [anon_sym_PERCENT] = ACTIONS(1153),
    [anon_sym_CARET] = ACTIONS(1153),
    [anon_sym_AMP] = ACTIONS(1153),
    [anon_sym_PIPE] = ACTIONS(1153),
    [anon_sym_AMP_AMP] = ACTIONS(1142),
    [anon_sym_PIPE_PIPE] = ACTIONS(1142),
    [anon_sym_LT_LT] = ACTIONS(1153),
    [anon_sym_GT_GT] = ACTIONS(1153),
    [anon_sym_PLUS_EQ] = ACTIONS(1142),
    [anon_sym_DASH_EQ] = ACTIONS(1142),
    [anon_sym_STAR_EQ] = ACTIONS(1142),
    [anon_sym_SLASH_EQ] = ACTIONS(1142),
    [anon_sym_PERCENT_EQ] = ACTIONS(1142),
    [anon_sym_CARET_EQ] = ACTIONS(1142),
    [anon_sym_AMP_EQ] = ACTIONS(1142),
    [anon_sym_PIPE_EQ] = ACTIONS(1142),
    [anon_sym_LT_LT_EQ] = ACTIONS(1142),
    [anon_sym_GT_GT_EQ] = ACTIONS(1142),
    [anon_sym_EQ] = ACTIONS(1153),
    [anon_sym_EQ_EQ] = ACTIONS(1142),
    [anon_sym_BANG_EQ] = ACTIONS(1142),
    [anon_sym_GT] = ACTIONS(1153),
    [anon_sym_LT] = ACTIONS(1153),
    [anon_sym_GT_EQ] = ACTIONS(1142),
    [anon_sym_LT_EQ] = ACTIONS(1142),
    [anon_sym_AT] = ACTIONS(1142),
    [anon_sym__] = ACTIONS(1153),
    [anon_sym_DOT] = ACTIONS(1153),
    [anon_sym_DOT_DOT] = ACTIONS(1153),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1142),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1142),
    [anon_sym_COLON_COLON] = ACTIONS(1142),
    [anon_sym_DASH_GT] = ACTIONS(1142),
    [anon_sym_POUND] = ACTIONS(1142),
    [anon_sym_SQUOTE] = ACTIONS(1139),
    [anon_sym_as] = ACTIONS(1139),
    [anon_sym_async] = ACTIONS(1139),
    [anon_sym_auto] = ACTIONS(1139),
    [anon_sym_await] = ACTIONS(1139),
    [anon_sym_become] = ACTIONS(1139),
    [anon_sym_break] = ACTIONS(1139),
    [anon_sym_const] = ACTIONS(1139),
    [anon_sym_continue] = ACTIONS(1139),
    [anon_sym_default] = ACTIONS(1139),
    [anon_sym_enum] = ACTIONS(1139),
    [anon_sym_fn] = ACTIONS(1139),
    [anon_sym_for] = ACTIONS(1139),
    [anon_sym_gen] = ACTIONS(1139),
    [anon_sym_if] = ACTIONS(1139),
    [anon_sym_impl] = ACTIONS(1139),
    [anon_sym_let] = ACTIONS(1139),
    [anon_sym_loop] = ACTIONS(1139),
    [anon_sym_match] = ACTIONS(1139),
    [anon_sym_mod] = ACTIONS(1139),
    [anon_sym_pub] = ACTIONS(1139),
    [anon_sym_return] = ACTIONS(1139),
    [anon_sym_raw] = ACTIONS(1139),
    [anon_sym_static] = ACTIONS(1139),
    [anon_sym_struct] = ACTIONS(1139),
    [anon_sym_trait] = ACTIONS(1139),
    [anon_sym_type] = ACTIONS(1139),
    [anon_sym_union] = ACTIONS(1139),
    [anon_sym_unsafe] = ACTIONS(1139),
    [anon_sym_use] = ACTIONS(1139),
    [anon_sym_where] = ACTIONS(1139),
    [anon_sym_while] = ACTIONS(1139),
    [sym_mutable_specifier] = ACTIONS(1139),
    [sym_integer_literal] = ACTIONS(1165),
    [aux_sym_string_literal_token1] = ACTIONS(1168),
    [sym_char_literal] = ACTIONS(1165),
    [anon_sym_true] = ACTIONS(1171),
    [anon_sym_false] = ACTIONS(1171),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1139),
    [sym_super] = ACTIONS(1139),
    [sym_crate] = ACTIONS(1139),
    [aux_sym_metavariable_token1] = ACTIONS(1174),
    [sym__raw_string_literal_start] = ACTIONS(1177),
    [sym_float_literal] = ACTIONS(1165),
  },
  [STATE(335)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(335),
    [sym_block_comment] = STATE(335),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_RBRACE] = ACTIONS(1135),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(336)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(336),
    [sym_block_comment] = STATE(336),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(335),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_RBRACE] = ACTIONS(1180),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(337)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(337),
    [sym_block_comment] = STATE(337),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_RBRACK] = ACTIONS(1135),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(338)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(338),
    [sym_block_comment] = STATE(338),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(333),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1182),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(339)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(339),
    [sym_block_comment] = STATE(339),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1184),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(340)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(340),
    [sym_block_comment] = STATE(340),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(346),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1186),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(341)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(341),
    [sym_block_comment] = STATE(341),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(330),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_RBRACE] = ACTIONS(1186),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(342)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(342),
    [sym_block_comment] = STATE(342),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(331),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_RBRACK] = ACTIONS(1186),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(343)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(343),
    [sym_block_comment] = STATE(343),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(337),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_RBRACK] = ACTIONS(1180),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(344)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(344),
    [sym_block_comment] = STATE(344),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(339),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1188),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(345)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(345),
    [sym_block_comment] = STATE(345),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(332),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1180),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(346)] = {
    [sym__token_pattern] = STATE(433),
    [sym_token_tree_pattern] = STATE(445),
    [sym_token_binding_pattern] = STATE(445),
    [sym_token_repetition_pattern] = STATE(445),
    [sym_primitive_type] = STATE(445),
    [sym__literal] = STATE(445),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(346),
    [sym_block_comment] = STATE(346),
    [sym_metavariable] = STATE(416),
    [aux_sym_macro_arguments_declaration_repeat1] = STATE(132),
    [aux_sym__non_special_token_repeat1] = STATE(413),
    [sym_identifier] = ACTIONS(1107),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [anon_sym_RPAREN] = ACTIONS(1115),
    [anon_sym_LBRACE] = ACTIONS(1113),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1119),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1107),
    [anon_sym_as] = ACTIONS(1107),
    [anon_sym_async] = ACTIONS(1107),
    [anon_sym_auto] = ACTIONS(1107),
    [anon_sym_await] = ACTIONS(1107),
    [anon_sym_become] = ACTIONS(1107),
    [anon_sym_break] = ACTIONS(1107),
    [anon_sym_const] = ACTIONS(1107),
    [anon_sym_continue] = ACTIONS(1107),
    [anon_sym_default] = ACTIONS(1107),
    [anon_sym_enum] = ACTIONS(1107),
    [anon_sym_fn] = ACTIONS(1107),
    [anon_sym_for] = ACTIONS(1107),
    [anon_sym_gen] = ACTIONS(1107),
    [anon_sym_if] = ACTIONS(1107),
    [anon_sym_impl] = ACTIONS(1107),
    [anon_sym_let] = ACTIONS(1107),
    [anon_sym_loop] = ACTIONS(1107),
    [anon_sym_match] = ACTIONS(1107),
    [anon_sym_mod] = ACTIONS(1107),
    [anon_sym_pub] = ACTIONS(1107),
    [anon_sym_return] = ACTIONS(1107),
    [anon_sym_raw] = ACTIONS(1107),
    [anon_sym_static] = ACTIONS(1107),
    [anon_sym_struct] = ACTIONS(1107),
    [anon_sym_trait] = ACTIONS(1107),
    [anon_sym_type] = ACTIONS(1107),
    [anon_sym_union] = ACTIONS(1107),
    [anon_sym_unsafe] = ACTIONS(1107),
    [anon_sym_use] = ACTIONS(1107),
    [anon_sym_where] = ACTIONS(1107),
    [anon_sym_while] = ACTIONS(1107),
    [sym_mutable_specifier] = ACTIONS(1107),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1107),
    [sym_super] = ACTIONS(1107),
    [sym_crate] = ACTIONS(1107),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(347)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(347),
    [sym_block_comment] = STATE(347),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1190),
    [anon_sym_SEMI] = ACTIONS(1193),
    [anon_sym_LPAREN] = ACTIONS(1196),
    [anon_sym_RPAREN] = ACTIONS(1199),
    [anon_sym_LBRACE] = ACTIONS(1201),
    [anon_sym_RBRACE] = ACTIONS(1199),
    [anon_sym_COMMA] = ACTIONS(1193),
    [anon_sym_EQ_GT] = ACTIONS(1193),
    [anon_sym_BANG] = ACTIONS(1204),
    [anon_sym_LBRACK] = ACTIONS(1207),
    [anon_sym_RBRACK] = ACTIONS(1199),
    [anon_sym_COLON] = ACTIONS(1204),
    [anon_sym_DOLLAR] = ACTIONS(1210),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_STAR] = ACTIONS(1204),
    [anon_sym_QMARK] = ACTIONS(1193),
    [anon_sym_u8] = ACTIONS(1213),
    [anon_sym_i8] = ACTIONS(1213),
    [anon_sym_u16] = ACTIONS(1213),
    [anon_sym_i16] = ACTIONS(1213),
    [anon_sym_u32] = ACTIONS(1213),
    [anon_sym_i32] = ACTIONS(1213),
    [anon_sym_u64] = ACTIONS(1213),
    [anon_sym_i64] = ACTIONS(1213),
    [anon_sym_u128] = ACTIONS(1213),
    [anon_sym_i128] = ACTIONS(1213),
    [anon_sym_isize] = ACTIONS(1213),
    [anon_sym_usize] = ACTIONS(1213),
    [anon_sym_f32] = ACTIONS(1213),
    [anon_sym_f64] = ACTIONS(1213),
    [anon_sym_bool] = ACTIONS(1213),
    [anon_sym_str] = ACTIONS(1213),
    [anon_sym_char] = ACTIONS(1213),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_SLASH] = ACTIONS(1204),
    [anon_sym_PERCENT] = ACTIONS(1204),
    [anon_sym_CARET] = ACTIONS(1204),
    [anon_sym_AMP] = ACTIONS(1204),
    [anon_sym_PIPE] = ACTIONS(1204),
    [anon_sym_AMP_AMP] = ACTIONS(1193),
    [anon_sym_PIPE_PIPE] = ACTIONS(1193),
    [anon_sym_LT_LT] = ACTIONS(1204),
    [anon_sym_GT_GT] = ACTIONS(1204),
    [anon_sym_PLUS_EQ] = ACTIONS(1193),
    [anon_sym_DASH_EQ] = ACTIONS(1193),
    [anon_sym_STAR_EQ] = ACTIONS(1193),
    [anon_sym_SLASH_EQ] = ACTIONS(1193),
    [anon_sym_PERCENT_EQ] = ACTIONS(1193),
    [anon_sym_CARET_EQ] = ACTIONS(1193),
    [anon_sym_AMP_EQ] = ACTIONS(1193),
    [anon_sym_PIPE_EQ] = ACTIONS(1193),
    [anon_sym_LT_LT_EQ] = ACTIONS(1193),
    [anon_sym_GT_GT_EQ] = ACTIONS(1193),
    [anon_sym_EQ] = ACTIONS(1204),
    [anon_sym_EQ_EQ] = ACTIONS(1193),
    [anon_sym_BANG_EQ] = ACTIONS(1193),
    [anon_sym_GT] = ACTIONS(1204),
    [anon_sym_LT] = ACTIONS(1204),
    [anon_sym_GT_EQ] = ACTIONS(1193),
    [anon_sym_LT_EQ] = ACTIONS(1193),
    [anon_sym_AT] = ACTIONS(1193),
    [anon_sym__] = ACTIONS(1204),
    [anon_sym_DOT] = ACTIONS(1204),
    [anon_sym_DOT_DOT] = ACTIONS(1204),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1193),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1193),
    [anon_sym_COLON_COLON] = ACTIONS(1193),
    [anon_sym_DASH_GT] = ACTIONS(1193),
    [anon_sym_POUND] = ACTIONS(1193),
    [anon_sym_SQUOTE] = ACTIONS(1190),
    [anon_sym_as] = ACTIONS(1190),
    [anon_sym_async] = ACTIONS(1190),
    [anon_sym_auto] = ACTIONS(1190),
    [anon_sym_await] = ACTIONS(1190),
    [anon_sym_become] = ACTIONS(1190),
    [anon_sym_break] = ACTIONS(1190),
    [anon_sym_const] = ACTIONS(1190),
    [anon_sym_continue] = ACTIONS(1190),
    [anon_sym_default] = ACTIONS(1190),
    [anon_sym_enum] = ACTIONS(1190),
    [anon_sym_fn] = ACTIONS(1190),
    [anon_sym_for] = ACTIONS(1190),
    [anon_sym_gen] = ACTIONS(1190),
    [anon_sym_if] = ACTIONS(1190),
    [anon_sym_impl] = ACTIONS(1190),
    [anon_sym_let] = ACTIONS(1190),
    [anon_sym_loop] = ACTIONS(1190),
    [anon_sym_match] = ACTIONS(1190),
    [anon_sym_mod] = ACTIONS(1190),
    [anon_sym_pub] = ACTIONS(1190),
    [anon_sym_return] = ACTIONS(1190),
    [anon_sym_raw] = ACTIONS(1190),
    [anon_sym_static] = ACTIONS(1190),
    [anon_sym_struct] = ACTIONS(1190),
    [anon_sym_trait] = ACTIONS(1190),
    [anon_sym_type] = ACTIONS(1190),
    [anon_sym_union] = ACTIONS(1190),
    [anon_sym_unsafe] = ACTIONS(1190),
    [anon_sym_use] = ACTIONS(1190),
    [anon_sym_where] = ACTIONS(1190),
    [anon_sym_while] = ACTIONS(1190),
    [sym_mutable_specifier] = ACTIONS(1190),
    [sym_integer_literal] = ACTIONS(1216),
    [aux_sym_string_literal_token1] = ACTIONS(1219),
    [sym_char_literal] = ACTIONS(1216),
    [anon_sym_true] = ACTIONS(1222),
    [anon_sym_false] = ACTIONS(1222),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1190),
    [sym_super] = ACTIONS(1190),
    [sym_crate] = ACTIONS(1190),
    [sym__raw_string_literal_start] = ACTIONS(1225),
    [sym_float_literal] = ACTIONS(1216),
  },
  [STATE(348)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(348),
    [sym_block_comment] = STATE(348),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(352),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_RPAREN] = ACTIONS(1232),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(349)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(349),
    [sym_block_comment] = STATE(349),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_RBRACK] = ACTIONS(1240),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(350)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(350),
    [sym_block_comment] = STATE(350),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_RPAREN] = ACTIONS(1242),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(351)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(351),
    [sym_block_comment] = STATE(351),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(358),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1244),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(352)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(352),
    [sym_block_comment] = STATE(352),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_RPAREN] = ACTIONS(1246),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(353)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(353),
    [sym_block_comment] = STATE(353),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1246),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(354)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(354),
    [sym_block_comment] = STATE(354),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_RBRACK] = ACTIONS(1246),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(355)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(355),
    [sym_block_comment] = STATE(355),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(350),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_RPAREN] = ACTIONS(1248),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(356)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(356),
    [sym_block_comment] = STATE(356),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(353),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1232),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(357)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(357),
    [sym_block_comment] = STATE(357),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(354),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_RBRACK] = ACTIONS(1232),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(358)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(358),
    [sym_block_comment] = STATE(358),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1250),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(359)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(359),
    [sym_block_comment] = STATE(359),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(363),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1252),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(360)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(360),
    [sym_block_comment] = STATE(360),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(364),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_RPAREN] = ACTIONS(1254),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(361)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(361),
    [sym_block_comment] = STATE(361),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(365),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1254),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(362)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(362),
    [sym_block_comment] = STATE(362),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(349),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_RBRACK] = ACTIONS(1254),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(363)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(363),
    [sym_block_comment] = STATE(363),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1256),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(364)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(364),
    [sym_block_comment] = STATE(364),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_RPAREN] = ACTIONS(1240),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(365)] = {
    [sym_token_tree] = STATE(440),
    [sym_token_repetition] = STATE(440),
    [sym_primitive_type] = STATE(440),
    [sym__literal] = STATE(440),
    [sym_string_literal] = STATE(436),
    [sym_raw_string_literal] = STATE(436),
    [sym_boolean_literal] = STATE(436),
    [sym_line_comment] = STATE(365),
    [sym_block_comment] = STATE(365),
    [sym_metavariable] = STATE(440),
    [aux_sym_macro_body_v2_repeat1] = STATE(334),
    [aux_sym__non_special_token_repeat1] = STATE(411),
    [sym_identifier] = ACTIONS(1228),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1230),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1240),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1236),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1238),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1123),
    [anon_sym_i8] = ACTIONS(1123),
    [anon_sym_u16] = ACTIONS(1123),
    [anon_sym_i16] = ACTIONS(1123),
    [anon_sym_u32] = ACTIONS(1123),
    [anon_sym_i32] = ACTIONS(1123),
    [anon_sym_u64] = ACTIONS(1123),
    [anon_sym_i64] = ACTIONS(1123),
    [anon_sym_u128] = ACTIONS(1123),
    [anon_sym_i128] = ACTIONS(1123),
    [anon_sym_isize] = ACTIONS(1123),
    [anon_sym_usize] = ACTIONS(1123),
    [anon_sym_f32] = ACTIONS(1123),
    [anon_sym_f64] = ACTIONS(1123),
    [anon_sym_bool] = ACTIONS(1123),
    [anon_sym_str] = ACTIONS(1123),
    [anon_sym_char] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [anon_sym_as] = ACTIONS(1228),
    [anon_sym_async] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_await] = ACTIONS(1228),
    [anon_sym_become] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_fn] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_gen] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_impl] = ACTIONS(1228),
    [anon_sym_let] = ACTIONS(1228),
    [anon_sym_loop] = ACTIONS(1228),
    [anon_sym_match] = ACTIONS(1228),
    [anon_sym_mod] = ACTIONS(1228),
    [anon_sym_pub] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_raw] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_trait] = ACTIONS(1228),
    [anon_sym_type] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_unsafe] = ACTIONS(1228),
    [anon_sym_use] = ACTIONS(1228),
    [anon_sym_where] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [sym_mutable_specifier] = ACTIONS(1228),
    [sym_integer_literal] = ACTIONS(1125),
    [aux_sym_string_literal_token1] = ACTIONS(1127),
    [sym_char_literal] = ACTIONS(1125),
    [anon_sym_true] = ACTIONS(1129),
    [anon_sym_false] = ACTIONS(1129),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1228),
    [sym_super] = ACTIONS(1228),
    [sym_crate] = ACTIONS(1228),
    [aux_sym_metavariable_token1] = ACTIONS(1131),
    [sym__raw_string_literal_start] = ACTIONS(1133),
    [sym_float_literal] = ACTIONS(1125),
  },
  [STATE(366)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(366),
    [sym_block_comment] = STATE(366),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(379),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1266),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(367)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(367),
    [sym_block_comment] = STATE(367),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(371),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1284),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(368)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(368),
    [sym_block_comment] = STATE(368),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(372),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1284),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(369)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(369),
    [sym_block_comment] = STATE(369),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(381),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1286),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(370)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(370),
    [sym_block_comment] = STATE(370),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1288),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(371)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(371),
    [sym_block_comment] = STATE(371),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1288),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(372)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(372),
    [sym_block_comment] = STATE(372),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1288),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(373)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(373),
    [sym_block_comment] = STATE(373),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(385),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1286),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(374)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(374),
    [sym_block_comment] = STATE(374),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1290),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(375)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(375),
    [sym_block_comment] = STATE(375),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(378),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1266),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(376)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(376),
    [sym_block_comment] = STATE(376),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1292),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(377)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(377),
    [sym_block_comment] = STATE(377),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(380),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1266),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(378)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(378),
    [sym_block_comment] = STATE(378),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1294),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(379)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(379),
    [sym_block_comment] = STATE(379),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1294),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(380)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(380),
    [sym_block_comment] = STATE(380),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1294),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(381)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(381),
    [sym_block_comment] = STATE(381),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1290),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(382)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(382),
    [sym_block_comment] = STATE(382),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(388),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1296),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(383)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(383),
    [sym_block_comment] = STATE(383),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(389),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1296),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(384)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(384),
    [sym_block_comment] = STATE(384),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(390),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1296),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(385)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(385),
    [sym_block_comment] = STATE(385),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1290),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(386)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(386),
    [sym_block_comment] = STATE(386),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1292),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(387)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(387),
    [sym_block_comment] = STATE(387),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(395),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1298),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(388)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(388),
    [sym_block_comment] = STATE(388),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1300),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(389)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(389),
    [sym_block_comment] = STATE(389),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1300),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(390)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(390),
    [sym_block_comment] = STATE(390),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1300),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(391)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(391),
    [sym_block_comment] = STATE(391),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1302),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(392)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(392),
    [sym_block_comment] = STATE(392),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1302),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(393)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(393),
    [sym_block_comment] = STATE(393),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(396),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1298),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(394)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(394),
    [sym_block_comment] = STATE(394),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(398),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1298),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(395)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(395),
    [sym_block_comment] = STATE(395),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1304),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(396)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(396),
    [sym_block_comment] = STATE(396),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1304),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(397)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(397),
    [sym_block_comment] = STATE(397),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1302),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(398)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(398),
    [sym_block_comment] = STATE(398),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1304),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(399)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(399),
    [sym_block_comment] = STATE(399),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(374),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1286),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(400)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(400),
    [sym_block_comment] = STATE(400),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(370),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1284),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(401)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(401),
    [sym_block_comment] = STATE(401),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(391),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1306),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(402)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(402),
    [sym_block_comment] = STATE(402),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(392),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1306),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(403)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(403),
    [sym_block_comment] = STATE(403),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(376),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_RPAREN] = ACTIONS(1308),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(404)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(404),
    [sym_block_comment] = STATE(404),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(386),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_RBRACE] = ACTIONS(1308),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(405)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(405),
    [sym_block_comment] = STATE(405),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(407),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1308),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(406)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(406),
    [sym_block_comment] = STATE(406),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(397),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1306),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(407)] = {
    [sym_primitive_type] = STATE(453),
    [sym_delim_token_tree] = STATE(454),
    [sym__delim_tokens] = STATE(455),
    [sym__non_delim_token] = STATE(454),
    [sym__literal] = STATE(453),
    [sym_string_literal] = STATE(460),
    [sym_raw_string_literal] = STATE(460),
    [sym_boolean_literal] = STATE(460),
    [sym_line_comment] = STATE(407),
    [sym_block_comment] = STATE(407),
    [aux_sym__non_special_token_repeat1] = STATE(417),
    [aux_sym_delim_token_tree_repeat1] = STATE(347),
    [sym_identifier] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1262),
    [anon_sym_LBRACE] = ACTIONS(1264),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1270),
    [anon_sym_RBRACK] = ACTIONS(1292),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1274),
    [anon_sym_i8] = ACTIONS(1274),
    [anon_sym_u16] = ACTIONS(1274),
    [anon_sym_i16] = ACTIONS(1274),
    [anon_sym_u32] = ACTIONS(1274),
    [anon_sym_i32] = ACTIONS(1274),
    [anon_sym_u64] = ACTIONS(1274),
    [anon_sym_i64] = ACTIONS(1274),
    [anon_sym_u128] = ACTIONS(1274),
    [anon_sym_i128] = ACTIONS(1274),
    [anon_sym_isize] = ACTIONS(1274),
    [anon_sym_usize] = ACTIONS(1274),
    [anon_sym_f32] = ACTIONS(1274),
    [anon_sym_f64] = ACTIONS(1274),
    [anon_sym_bool] = ACTIONS(1274),
    [anon_sym_str] = ACTIONS(1274),
    [anon_sym_char] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_as] = ACTIONS(1258),
    [anon_sym_async] = ACTIONS(1258),
    [anon_sym_auto] = ACTIONS(1258),
    [anon_sym_await] = ACTIONS(1258),
    [anon_sym_become] = ACTIONS(1258),
    [anon_sym_break] = ACTIONS(1258),
    [anon_sym_const] = ACTIONS(1258),
    [anon_sym_continue] = ACTIONS(1258),
    [anon_sym_default] = ACTIONS(1258),
    [anon_sym_enum] = ACTIONS(1258),
    [anon_sym_fn] = ACTIONS(1258),
    [anon_sym_for] = ACTIONS(1258),
    [anon_sym_gen] = ACTIONS(1258),
    [anon_sym_if] = ACTIONS(1258),
    [anon_sym_impl] = ACTIONS(1258),
    [anon_sym_let] = ACTIONS(1258),
    [anon_sym_loop] = ACTIONS(1258),
    [anon_sym_match] = ACTIONS(1258),
    [anon_sym_mod] = ACTIONS(1258),
    [anon_sym_pub] = ACTIONS(1258),
    [anon_sym_return] = ACTIONS(1258),
    [anon_sym_raw] = ACTIONS(1258),
    [anon_sym_static] = ACTIONS(1258),
    [anon_sym_struct] = ACTIONS(1258),
    [anon_sym_trait] = ACTIONS(1258),
    [anon_sym_type] = ACTIONS(1258),
    [anon_sym_union] = ACTIONS(1258),
    [anon_sym_unsafe] = ACTIONS(1258),
    [anon_sym_use] = ACTIONS(1258),
    [anon_sym_where] = ACTIONS(1258),
    [anon_sym_while] = ACTIONS(1258),
    [sym_mutable_specifier] = ACTIONS(1258),
    [sym_integer_literal] = ACTIONS(1276),
    [aux_sym_string_literal_token1] = ACTIONS(1278),
    [sym_char_literal] = ACTIONS(1276),
    [anon_sym_true] = ACTIONS(1280),
    [anon_sym_false] = ACTIONS(1280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1258),
    [sym_super] = ACTIONS(1258),
    [sym_crate] = ACTIONS(1258),
    [sym__raw_string_literal_start] = ACTIONS(1282),
    [sym_float_literal] = ACTIONS(1276),
  },
  [STATE(408)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4538),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5406),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(4582),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(408),
    [sym_block_comment] = STATE(408),
    [sym_metavariable] = STATE(4014),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_RPAREN] = ACTIONS(1310),
    [anon_sym_COMMA] = ACTIONS(1312),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(1320),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(979),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(409)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4538),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5406),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(4582),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(409),
    [sym_block_comment] = STATE(409),
    [sym_metavariable] = STATE(4014),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_RPAREN] = ACTIONS(1328),
    [anon_sym_COMMA] = ACTIONS(1312),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(1320),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(979),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(410)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4538),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5406),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(4582),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(410),
    [sym_block_comment] = STATE(410),
    [sym_metavariable] = STATE(4014),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_RPAREN] = ACTIONS(1330),
    [anon_sym_COMMA] = ACTIONS(1312),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(1320),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(979),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(411)] = {
    [sym_line_comment] = STATE(411),
    [sym_block_comment] = STATE(411),
    [aux_sym__non_special_token_repeat1] = STATE(412),
    [sym_identifier] = ACTIONS(1332),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1334),
    [anon_sym_RPAREN] = ACTIONS(1334),
    [anon_sym_LBRACE] = ACTIONS(1334),
    [anon_sym_RBRACE] = ACTIONS(1334),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1334),
    [anon_sym_RBRACK] = ACTIONS(1334),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1332),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1332),
    [anon_sym_i8] = ACTIONS(1332),
    [anon_sym_u16] = ACTIONS(1332),
    [anon_sym_i16] = ACTIONS(1332),
    [anon_sym_u32] = ACTIONS(1332),
    [anon_sym_i32] = ACTIONS(1332),
    [anon_sym_u64] = ACTIONS(1332),
    [anon_sym_i64] = ACTIONS(1332),
    [anon_sym_u128] = ACTIONS(1332),
    [anon_sym_i128] = ACTIONS(1332),
    [anon_sym_isize] = ACTIONS(1332),
    [anon_sym_usize] = ACTIONS(1332),
    [anon_sym_f32] = ACTIONS(1332),
    [anon_sym_f64] = ACTIONS(1332),
    [anon_sym_bool] = ACTIONS(1332),
    [anon_sym_str] = ACTIONS(1332),
    [anon_sym_char] = ACTIONS(1332),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1332),
    [anon_sym_as] = ACTIONS(1332),
    [anon_sym_async] = ACTIONS(1332),
    [anon_sym_auto] = ACTIONS(1332),
    [anon_sym_await] = ACTIONS(1332),
    [anon_sym_become] = ACTIONS(1332),
    [anon_sym_break] = ACTIONS(1332),
    [anon_sym_const] = ACTIONS(1332),
    [anon_sym_continue] = ACTIONS(1332),
    [anon_sym_default] = ACTIONS(1332),
    [anon_sym_enum] = ACTIONS(1332),
    [anon_sym_fn] = ACTIONS(1332),
    [anon_sym_for] = ACTIONS(1332),
    [anon_sym_gen] = ACTIONS(1332),
    [anon_sym_if] = ACTIONS(1332),
    [anon_sym_impl] = ACTIONS(1332),
    [anon_sym_let] = ACTIONS(1332),
    [anon_sym_loop] = ACTIONS(1332),
    [anon_sym_match] = ACTIONS(1332),
    [anon_sym_mod] = ACTIONS(1332),
    [anon_sym_pub] = ACTIONS(1332),
    [anon_sym_return] = ACTIONS(1332),
    [anon_sym_raw] = ACTIONS(1332),
    [anon_sym_static] = ACTIONS(1332),
    [anon_sym_struct] = ACTIONS(1332),
    [anon_sym_trait] = ACTIONS(1332),
    [anon_sym_type] = ACTIONS(1332),
    [anon_sym_union] = ACTIONS(1332),
    [anon_sym_unsafe] = ACTIONS(1332),
    [anon_sym_use] = ACTIONS(1332),
    [anon_sym_where] = ACTIONS(1332),
    [anon_sym_while] = ACTIONS(1332),
    [sym_mutable_specifier] = ACTIONS(1332),
    [sym_integer_literal] = ACTIONS(1334),
    [aux_sym_string_literal_token1] = ACTIONS(1334),
    [sym_char_literal] = ACTIONS(1334),
    [anon_sym_true] = ACTIONS(1332),
    [anon_sym_false] = ACTIONS(1332),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1332),
    [sym_super] = ACTIONS(1332),
    [sym_crate] = ACTIONS(1332),
    [aux_sym_metavariable_token1] = ACTIONS(1334),
    [sym__raw_string_literal_start] = ACTIONS(1334),
    [sym_float_literal] = ACTIONS(1334),
  },
  [STATE(412)] = {
    [sym_line_comment] = STATE(412),
    [sym_block_comment] = STATE(412),
    [aux_sym__non_special_token_repeat1] = STATE(412),
    [sym_identifier] = ACTIONS(1336),
    [anon_sym_SEMI] = ACTIONS(1338),
    [anon_sym_LPAREN] = ACTIONS(1341),
    [anon_sym_RPAREN] = ACTIONS(1341),
    [anon_sym_LBRACE] = ACTIONS(1341),
    [anon_sym_RBRACE] = ACTIONS(1341),
    [anon_sym_COMMA] = ACTIONS(1338),
    [anon_sym_EQ_GT] = ACTIONS(1338),
    [anon_sym_BANG] = ACTIONS(1343),
    [anon_sym_LBRACK] = ACTIONS(1341),
    [anon_sym_RBRACK] = ACTIONS(1341),
    [anon_sym_COLON] = ACTIONS(1343),
    [anon_sym_DOLLAR] = ACTIONS(1336),
    [anon_sym_PLUS] = ACTIONS(1343),
    [anon_sym_STAR] = ACTIONS(1343),
    [anon_sym_QMARK] = ACTIONS(1338),
    [anon_sym_u8] = ACTIONS(1336),
    [anon_sym_i8] = ACTIONS(1336),
    [anon_sym_u16] = ACTIONS(1336),
    [anon_sym_i16] = ACTIONS(1336),
    [anon_sym_u32] = ACTIONS(1336),
    [anon_sym_i32] = ACTIONS(1336),
    [anon_sym_u64] = ACTIONS(1336),
    [anon_sym_i64] = ACTIONS(1336),
    [anon_sym_u128] = ACTIONS(1336),
    [anon_sym_i128] = ACTIONS(1336),
    [anon_sym_isize] = ACTIONS(1336),
    [anon_sym_usize] = ACTIONS(1336),
    [anon_sym_f32] = ACTIONS(1336),
    [anon_sym_f64] = ACTIONS(1336),
    [anon_sym_bool] = ACTIONS(1336),
    [anon_sym_str] = ACTIONS(1336),
    [anon_sym_char] = ACTIONS(1336),
    [anon_sym_DASH] = ACTIONS(1343),
    [anon_sym_SLASH] = ACTIONS(1343),
    [anon_sym_PERCENT] = ACTIONS(1343),
    [anon_sym_CARET] = ACTIONS(1343),
    [anon_sym_AMP] = ACTIONS(1343),
    [anon_sym_PIPE] = ACTIONS(1343),
    [anon_sym_AMP_AMP] = ACTIONS(1338),
    [anon_sym_PIPE_PIPE] = ACTIONS(1338),
    [anon_sym_LT_LT] = ACTIONS(1343),
    [anon_sym_GT_GT] = ACTIONS(1343),
    [anon_sym_PLUS_EQ] = ACTIONS(1338),
    [anon_sym_DASH_EQ] = ACTIONS(1338),
    [anon_sym_STAR_EQ] = ACTIONS(1338),
    [anon_sym_SLASH_EQ] = ACTIONS(1338),
    [anon_sym_PERCENT_EQ] = ACTIONS(1338),
    [anon_sym_CARET_EQ] = ACTIONS(1338),
    [anon_sym_AMP_EQ] = ACTIONS(1338),
    [anon_sym_PIPE_EQ] = ACTIONS(1338),
    [anon_sym_LT_LT_EQ] = ACTIONS(1338),
    [anon_sym_GT_GT_EQ] = ACTIONS(1338),
    [anon_sym_EQ] = ACTIONS(1343),
    [anon_sym_EQ_EQ] = ACTIONS(1338),
    [anon_sym_BANG_EQ] = ACTIONS(1338),
    [anon_sym_GT] = ACTIONS(1343),
    [anon_sym_LT] = ACTIONS(1343),
    [anon_sym_GT_EQ] = ACTIONS(1338),
    [anon_sym_LT_EQ] = ACTIONS(1338),
    [anon_sym_AT] = ACTIONS(1338),
    [anon_sym__] = ACTIONS(1343),
    [anon_sym_DOT] = ACTIONS(1343),
    [anon_sym_DOT_DOT] = ACTIONS(1343),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1338),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1338),
    [anon_sym_COLON_COLON] = ACTIONS(1338),
    [anon_sym_DASH_GT] = ACTIONS(1338),
    [anon_sym_POUND] = ACTIONS(1338),
    [anon_sym_SQUOTE] = ACTIONS(1336),
    [anon_sym_as] = ACTIONS(1336),
    [anon_sym_async] = ACTIONS(1336),
    [anon_sym_auto] = ACTIONS(1336),
    [anon_sym_await] = ACTIONS(1336),
    [anon_sym_become] = ACTIONS(1336),
    [anon_sym_break] = ACTIONS(1336),
    [anon_sym_const] = ACTIONS(1336),
    [anon_sym_continue] = ACTIONS(1336),
    [anon_sym_default] = ACTIONS(1336),
    [anon_sym_enum] = ACTIONS(1336),
    [anon_sym_fn] = ACTIONS(1336),
    [anon_sym_for] = ACTIONS(1336),
    [anon_sym_gen] = ACTIONS(1336),
    [anon_sym_if] = ACTIONS(1336),
    [anon_sym_impl] = ACTIONS(1336),
    [anon_sym_let] = ACTIONS(1336),
    [anon_sym_loop] = ACTIONS(1336),
    [anon_sym_match] = ACTIONS(1336),
    [anon_sym_mod] = ACTIONS(1336),
    [anon_sym_pub] = ACTIONS(1336),
    [anon_sym_return] = ACTIONS(1336),
    [anon_sym_raw] = ACTIONS(1336),
    [anon_sym_static] = ACTIONS(1336),
    [anon_sym_struct] = ACTIONS(1336),
    [anon_sym_trait] = ACTIONS(1336),
    [anon_sym_type] = ACTIONS(1336),
    [anon_sym_union] = ACTIONS(1336),
    [anon_sym_unsafe] = ACTIONS(1336),
    [anon_sym_use] = ACTIONS(1336),
    [anon_sym_where] = ACTIONS(1336),
    [anon_sym_while] = ACTIONS(1336),
    [sym_mutable_specifier] = ACTIONS(1336),
    [sym_integer_literal] = ACTIONS(1341),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1341),
    [anon_sym_true] = ACTIONS(1336),
    [anon_sym_false] = ACTIONS(1336),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1336),
    [sym_super] = ACTIONS(1336),
    [sym_crate] = ACTIONS(1336),
    [aux_sym_metavariable_token1] = ACTIONS(1341),
    [sym__raw_string_literal_start] = ACTIONS(1341),
    [sym_float_literal] = ACTIONS(1341),
  },
  [STATE(413)] = {
    [sym_line_comment] = STATE(413),
    [sym_block_comment] = STATE(413),
    [aux_sym__non_special_token_repeat1] = STATE(412),
    [sym_identifier] = ACTIONS(1346),
    [anon_sym_SEMI] = ACTIONS(1109),
    [anon_sym_LPAREN] = ACTIONS(1348),
    [anon_sym_RPAREN] = ACTIONS(1348),
    [anon_sym_LBRACE] = ACTIONS(1348),
    [anon_sym_RBRACE] = ACTIONS(1348),
    [anon_sym_COMMA] = ACTIONS(1109),
    [anon_sym_EQ_GT] = ACTIONS(1109),
    [anon_sym_BANG] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1348),
    [anon_sym_RBRACK] = ACTIONS(1348),
    [anon_sym_COLON] = ACTIONS(1117),
    [anon_sym_DOLLAR] = ACTIONS(1346),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_STAR] = ACTIONS(1117),
    [anon_sym_QMARK] = ACTIONS(1109),
    [anon_sym_u8] = ACTIONS(1346),
    [anon_sym_i8] = ACTIONS(1346),
    [anon_sym_u16] = ACTIONS(1346),
    [anon_sym_i16] = ACTIONS(1346),
    [anon_sym_u32] = ACTIONS(1346),
    [anon_sym_i32] = ACTIONS(1346),
    [anon_sym_u64] = ACTIONS(1346),
    [anon_sym_i64] = ACTIONS(1346),
    [anon_sym_u128] = ACTIONS(1346),
    [anon_sym_i128] = ACTIONS(1346),
    [anon_sym_isize] = ACTIONS(1346),
    [anon_sym_usize] = ACTIONS(1346),
    [anon_sym_f32] = ACTIONS(1346),
    [anon_sym_f64] = ACTIONS(1346),
    [anon_sym_bool] = ACTIONS(1346),
    [anon_sym_str] = ACTIONS(1346),
    [anon_sym_char] = ACTIONS(1346),
    [anon_sym_DASH] = ACTIONS(1117),
    [anon_sym_SLASH] = ACTIONS(1117),
    [anon_sym_PERCENT] = ACTIONS(1117),
    [anon_sym_CARET] = ACTIONS(1117),
    [anon_sym_AMP] = ACTIONS(1117),
    [anon_sym_PIPE] = ACTIONS(1117),
    [anon_sym_AMP_AMP] = ACTIONS(1109),
    [anon_sym_PIPE_PIPE] = ACTIONS(1109),
    [anon_sym_LT_LT] = ACTIONS(1117),
    [anon_sym_GT_GT] = ACTIONS(1117),
    [anon_sym_PLUS_EQ] = ACTIONS(1109),
    [anon_sym_DASH_EQ] = ACTIONS(1109),
    [anon_sym_STAR_EQ] = ACTIONS(1109),
    [anon_sym_SLASH_EQ] = ACTIONS(1109),
    [anon_sym_PERCENT_EQ] = ACTIONS(1109),
    [anon_sym_CARET_EQ] = ACTIONS(1109),
    [anon_sym_AMP_EQ] = ACTIONS(1109),
    [anon_sym_PIPE_EQ] = ACTIONS(1109),
    [anon_sym_LT_LT_EQ] = ACTIONS(1109),
    [anon_sym_GT_GT_EQ] = ACTIONS(1109),
    [anon_sym_EQ] = ACTIONS(1117),
    [anon_sym_EQ_EQ] = ACTIONS(1109),
    [anon_sym_BANG_EQ] = ACTIONS(1109),
    [anon_sym_GT] = ACTIONS(1117),
    [anon_sym_LT] = ACTIONS(1117),
    [anon_sym_GT_EQ] = ACTIONS(1109),
    [anon_sym_LT_EQ] = ACTIONS(1109),
    [anon_sym_AT] = ACTIONS(1109),
    [anon_sym__] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT] = ACTIONS(1117),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1109),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1109),
    [anon_sym_COLON_COLON] = ACTIONS(1109),
    [anon_sym_DASH_GT] = ACTIONS(1109),
    [anon_sym_POUND] = ACTIONS(1109),
    [anon_sym_SQUOTE] = ACTIONS(1346),
    [anon_sym_as] = ACTIONS(1346),
    [anon_sym_async] = ACTIONS(1346),
    [anon_sym_auto] = ACTIONS(1346),
    [anon_sym_await] = ACTIONS(1346),
    [anon_sym_become] = ACTIONS(1346),
    [anon_sym_break] = ACTIONS(1346),
    [anon_sym_const] = ACTIONS(1346),
    [anon_sym_continue] = ACTIONS(1346),
    [anon_sym_default] = ACTIONS(1346),
    [anon_sym_enum] = ACTIONS(1346),
    [anon_sym_fn] = ACTIONS(1346),
    [anon_sym_for] = ACTIONS(1346),
    [anon_sym_gen] = ACTIONS(1346),
    [anon_sym_if] = ACTIONS(1346),
    [anon_sym_impl] = ACTIONS(1346),
    [anon_sym_let] = ACTIONS(1346),
    [anon_sym_loop] = ACTIONS(1346),
    [anon_sym_match] = ACTIONS(1346),
    [anon_sym_mod] = ACTIONS(1346),
    [anon_sym_pub] = ACTIONS(1346),
    [anon_sym_return] = ACTIONS(1346),
    [anon_sym_raw] = ACTIONS(1346),
    [anon_sym_static] = ACTIONS(1346),
    [anon_sym_struct] = ACTIONS(1346),
    [anon_sym_trait] = ACTIONS(1346),
    [anon_sym_type] = ACTIONS(1346),
    [anon_sym_union] = ACTIONS(1346),
    [anon_sym_unsafe] = ACTIONS(1346),
    [anon_sym_use] = ACTIONS(1346),
    [anon_sym_where] = ACTIONS(1346),
    [anon_sym_while] = ACTIONS(1346),
    [sym_mutable_specifier] = ACTIONS(1346),
    [sym_integer_literal] = ACTIONS(1348),
    [aux_sym_string_literal_token1] = ACTIONS(1348),
    [sym_char_literal] = ACTIONS(1348),
    [anon_sym_true] = ACTIONS(1346),
    [anon_sym_false] = ACTIONS(1346),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1346),
    [sym_super] = ACTIONS(1346),
    [sym_crate] = ACTIONS(1346),
    [aux_sym_metavariable_token1] = ACTIONS(1348),
    [sym__raw_string_literal_start] = ACTIONS(1348),
    [sym_float_literal] = ACTIONS(1348),
  },
  [STATE(414)] = {
    [sym_else_clause] = STATE(456),
    [sym_line_comment] = STATE(414),
    [sym_block_comment] = STATE(414),
    [ts_builtin_sym_end] = ACTIONS(1350),
    [sym_identifier] = ACTIONS(1352),
    [anon_sym_SEMI] = ACTIONS(1350),
    [anon_sym_macro] = ACTIONS(1352),
    [anon_sym_LPAREN] = ACTIONS(1350),
    [anon_sym_LBRACE] = ACTIONS(1350),
    [anon_sym_RBRACE] = ACTIONS(1350),
    [anon_sym_macro_rules] = ACTIONS(1352),
    [anon_sym_BANG] = ACTIONS(1352),
    [anon_sym_LBRACK] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1352),
    [anon_sym_STAR] = ACTIONS(1352),
    [anon_sym_QMARK] = ACTIONS(1350),
    [anon_sym_u8] = ACTIONS(1352),
    [anon_sym_i8] = ACTIONS(1352),
    [anon_sym_u16] = ACTIONS(1352),
    [anon_sym_i16] = ACTIONS(1352),
    [anon_sym_u32] = ACTIONS(1352),
    [anon_sym_i32] = ACTIONS(1352),
    [anon_sym_u64] = ACTIONS(1352),
    [anon_sym_i64] = ACTIONS(1352),
    [anon_sym_u128] = ACTIONS(1352),
    [anon_sym_i128] = ACTIONS(1352),
    [anon_sym_isize] = ACTIONS(1352),
    [anon_sym_usize] = ACTIONS(1352),
    [anon_sym_f32] = ACTIONS(1352),
    [anon_sym_f64] = ACTIONS(1352),
    [anon_sym_bool] = ACTIONS(1352),
    [anon_sym_str] = ACTIONS(1352),
    [anon_sym_char] = ACTIONS(1352),
    [anon_sym_DASH] = ACTIONS(1352),
    [anon_sym_SLASH] = ACTIONS(1352),
    [anon_sym_PERCENT] = ACTIONS(1352),
    [anon_sym_CARET] = ACTIONS(1352),
    [anon_sym_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1350),
    [anon_sym_PIPE_PIPE] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1352),
    [anon_sym_GT_GT] = ACTIONS(1352),
    [anon_sym_PLUS_EQ] = ACTIONS(1350),
    [anon_sym_DASH_EQ] = ACTIONS(1350),
    [anon_sym_STAR_EQ] = ACTIONS(1350),
    [anon_sym_SLASH_EQ] = ACTIONS(1350),
    [anon_sym_PERCENT_EQ] = ACTIONS(1350),
    [anon_sym_CARET_EQ] = ACTIONS(1350),
    [anon_sym_AMP_EQ] = ACTIONS(1350),
    [anon_sym_PIPE_EQ] = ACTIONS(1350),
    [anon_sym_LT_LT_EQ] = ACTIONS(1350),
    [anon_sym_GT_GT_EQ] = ACTIONS(1350),
    [anon_sym_EQ] = ACTIONS(1352),
    [anon_sym_EQ_EQ] = ACTIONS(1350),
    [anon_sym_BANG_EQ] = ACTIONS(1350),
    [anon_sym_GT] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1352),
    [anon_sym_GT_EQ] = ACTIONS(1350),
    [anon_sym_LT_EQ] = ACTIONS(1350),
    [anon_sym_DOT] = ACTIONS(1352),
    [anon_sym_DOT_DOT] = ACTIONS(1352),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1350),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1350),
    [anon_sym_COLON_COLON] = ACTIONS(1350),
    [anon_sym_POUND] = ACTIONS(1350),
    [anon_sym_SQUOTE] = ACTIONS(1352),
    [anon_sym_as] = ACTIONS(1352),
    [anon_sym_async] = ACTIONS(1352),
    [anon_sym_auto] = ACTIONS(1352),
    [anon_sym_await] = ACTIONS(1352),
    [anon_sym_become] = ACTIONS(1352),
    [anon_sym_break] = ACTIONS(1352),
    [anon_sym_const] = ACTIONS(1352),
    [anon_sym_continue] = ACTIONS(1352),
    [anon_sym_default] = ACTIONS(1352),
    [anon_sym_enum] = ACTIONS(1352),
    [anon_sym_fn] = ACTIONS(1352),
    [anon_sym_for] = ACTIONS(1352),
    [anon_sym_gen] = ACTIONS(1352),
    [anon_sym_if] = ACTIONS(1352),
    [anon_sym_impl] = ACTIONS(1352),
    [anon_sym_let] = ACTIONS(1352),
    [anon_sym_loop] = ACTIONS(1352),
    [anon_sym_match] = ACTIONS(1352),
    [anon_sym_mod] = ACTIONS(1352),
    [anon_sym_pub] = ACTIONS(1352),
    [anon_sym_return] = ACTIONS(1352),
    [anon_sym_raw] = ACTIONS(1352),
    [anon_sym_static] = ACTIONS(1352),
    [anon_sym_struct] = ACTIONS(1352),
    [anon_sym_trait] = ACTIONS(1352),
    [anon_sym_type] = ACTIONS(1352),
    [anon_sym_union] = ACTIONS(1352),
    [anon_sym_unsafe] = ACTIONS(1352),
    [anon_sym_use] = ACTIONS(1352),
    [anon_sym_while] = ACTIONS(1352),
    [anon_sym_extern] = ACTIONS(1352),
    [anon_sym_safe] = ACTIONS(1352),
    [anon_sym_else] = ACTIONS(1354),
    [anon_sym_yield] = ACTIONS(1352),
    [anon_sym_move] = ACTIONS(1352),
    [anon_sym_try] = ACTIONS(1352),
    [sym_integer_literal] = ACTIONS(1350),
    [aux_sym_string_literal_token1] = ACTIONS(1350),
    [sym_char_literal] = ACTIONS(1350),
    [anon_sym_true] = ACTIONS(1352),
    [anon_sym_false] = ACTIONS(1352),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1352),
    [sym_super] = ACTIONS(1352),
    [sym_crate] = ACTIONS(1352),
    [aux_sym_metavariable_token1] = ACTIONS(1350),
    [sym__raw_string_literal_start] = ACTIONS(1350),
    [sym_float_literal] = ACTIONS(1350),
    [sym__outer_block_doc_comment_start] = ACTIONS(1350),
    [sym__outer_line_doc_comment_start] = ACTIONS(1350),
  },
  [STATE(415)] = {
    [sym_line_comment] = STATE(415),
    [sym_block_comment] = STATE(415),
    [sym_identifier] = ACTIONS(1356),
    [anon_sym_SEMI] = ACTIONS(1358),
    [anon_sym_LPAREN] = ACTIONS(1358),
    [anon_sym_RPAREN] = ACTIONS(1358),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_RBRACE] = ACTIONS(1358),
    [anon_sym_COMMA] = ACTIONS(1358),
    [anon_sym_EQ_GT] = ACTIONS(1358),
    [anon_sym_BANG] = ACTIONS(1356),
    [anon_sym_LBRACK] = ACTIONS(1358),
    [anon_sym_RBRACK] = ACTIONS(1358),
    [anon_sym_COLON] = ACTIONS(1356),
    [anon_sym_DOLLAR] = ACTIONS(1356),
    [anon_sym_PLUS] = ACTIONS(1356),
    [anon_sym_STAR] = ACTIONS(1356),
    [anon_sym_QMARK] = ACTIONS(1358),
    [anon_sym_u8] = ACTIONS(1356),
    [anon_sym_i8] = ACTIONS(1356),
    [anon_sym_u16] = ACTIONS(1356),
    [anon_sym_i16] = ACTIONS(1356),
    [anon_sym_u32] = ACTIONS(1356),
    [anon_sym_i32] = ACTIONS(1356),
    [anon_sym_u64] = ACTIONS(1356),
    [anon_sym_i64] = ACTIONS(1356),
    [anon_sym_u128] = ACTIONS(1356),
    [anon_sym_i128] = ACTIONS(1356),
    [anon_sym_isize] = ACTIONS(1356),
    [anon_sym_usize] = ACTIONS(1356),
    [anon_sym_f32] = ACTIONS(1356),
    [anon_sym_f64] = ACTIONS(1356),
    [anon_sym_bool] = ACTIONS(1356),
    [anon_sym_str] = ACTIONS(1356),
    [anon_sym_char] = ACTIONS(1356),
    [anon_sym_DASH] = ACTIONS(1356),
    [anon_sym_SLASH] = ACTIONS(1356),
    [anon_sym_PERCENT] = ACTIONS(1356),
    [anon_sym_CARET] = ACTIONS(1356),
    [anon_sym_AMP] = ACTIONS(1356),
    [anon_sym_PIPE] = ACTIONS(1356),
    [anon_sym_AMP_AMP] = ACTIONS(1358),
    [anon_sym_PIPE_PIPE] = ACTIONS(1358),
    [anon_sym_LT_LT] = ACTIONS(1356),
    [anon_sym_GT_GT] = ACTIONS(1356),
    [anon_sym_PLUS_EQ] = ACTIONS(1358),
    [anon_sym_DASH_EQ] = ACTIONS(1358),
    [anon_sym_STAR_EQ] = ACTIONS(1358),
    [anon_sym_SLASH_EQ] = ACTIONS(1358),
    [anon_sym_PERCENT_EQ] = ACTIONS(1358),
    [anon_sym_CARET_EQ] = ACTIONS(1358),
    [anon_sym_AMP_EQ] = ACTIONS(1358),
    [anon_sym_PIPE_EQ] = ACTIONS(1358),
    [anon_sym_LT_LT_EQ] = ACTIONS(1358),
    [anon_sym_GT_GT_EQ] = ACTIONS(1358),
    [anon_sym_EQ] = ACTIONS(1356),
    [anon_sym_EQ_EQ] = ACTIONS(1358),
    [anon_sym_BANG_EQ] = ACTIONS(1358),
    [anon_sym_GT] = ACTIONS(1356),
    [anon_sym_LT] = ACTIONS(1356),
    [anon_sym_GT_EQ] = ACTIONS(1358),
    [anon_sym_LT_EQ] = ACTIONS(1358),
    [anon_sym_AT] = ACTIONS(1358),
    [anon_sym__] = ACTIONS(1356),
    [anon_sym_DOT] = ACTIONS(1356),
    [anon_sym_DOT_DOT] = ACTIONS(1356),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1358),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1358),
    [anon_sym_COLON_COLON] = ACTIONS(1358),
    [anon_sym_DASH_GT] = ACTIONS(1358),
    [anon_sym_POUND] = ACTIONS(1358),
    [anon_sym_SQUOTE] = ACTIONS(1356),
    [anon_sym_as] = ACTIONS(1356),
    [anon_sym_async] = ACTIONS(1356),
    [anon_sym_auto] = ACTIONS(1356),
    [anon_sym_await] = ACTIONS(1356),
    [anon_sym_become] = ACTIONS(1356),
    [anon_sym_break] = ACTIONS(1356),
    [anon_sym_const] = ACTIONS(1356),
    [anon_sym_continue] = ACTIONS(1356),
    [anon_sym_default] = ACTIONS(1356),
    [anon_sym_enum] = ACTIONS(1356),
    [anon_sym_fn] = ACTIONS(1356),
    [anon_sym_for] = ACTIONS(1356),
    [anon_sym_gen] = ACTIONS(1356),
    [anon_sym_if] = ACTIONS(1356),
    [anon_sym_impl] = ACTIONS(1356),
    [anon_sym_let] = ACTIONS(1356),
    [anon_sym_loop] = ACTIONS(1356),
    [anon_sym_match] = ACTIONS(1356),
    [anon_sym_mod] = ACTIONS(1356),
    [anon_sym_pub] = ACTIONS(1356),
    [anon_sym_return] = ACTIONS(1356),
    [anon_sym_raw] = ACTIONS(1356),
    [anon_sym_static] = ACTIONS(1356),
    [anon_sym_struct] = ACTIONS(1356),
    [anon_sym_trait] = ACTIONS(1356),
    [anon_sym_type] = ACTIONS(1356),
    [anon_sym_union] = ACTIONS(1356),
    [anon_sym_unsafe] = ACTIONS(1356),
    [anon_sym_use] = ACTIONS(1356),
    [anon_sym_where] = ACTIONS(1356),
    [anon_sym_while] = ACTIONS(1356),
    [sym_mutable_specifier] = ACTIONS(1356),
    [sym_integer_literal] = ACTIONS(1358),
    [aux_sym_string_literal_token1] = ACTIONS(1358),
    [sym_char_literal] = ACTIONS(1358),
    [anon_sym_true] = ACTIONS(1356),
    [anon_sym_false] = ACTIONS(1356),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1356),
    [sym_super] = ACTIONS(1356),
    [sym_crate] = ACTIONS(1356),
    [aux_sym_metavariable_token1] = ACTIONS(1358),
    [sym__raw_string_literal_start] = ACTIONS(1358),
    [sym_float_literal] = ACTIONS(1358),
  },
  [STATE(416)] = {
    [sym_line_comment] = STATE(416),
    [sym_block_comment] = STATE(416),
    [sym_identifier] = ACTIONS(1346),
    [anon_sym_SEMI] = ACTIONS(1348),
    [anon_sym_LPAREN] = ACTIONS(1348),
    [anon_sym_RPAREN] = ACTIONS(1348),
    [anon_sym_LBRACE] = ACTIONS(1348),
    [anon_sym_RBRACE] = ACTIONS(1348),
    [anon_sym_COMMA] = ACTIONS(1348),
    [anon_sym_EQ_GT] = ACTIONS(1348),
    [anon_sym_BANG] = ACTIONS(1346),
    [anon_sym_LBRACK] = ACTIONS(1348),
    [anon_sym_RBRACK] = ACTIONS(1348),
    [anon_sym_COLON] = ACTIONS(1360),
    [anon_sym_DOLLAR] = ACTIONS(1346),
    [anon_sym_PLUS] = ACTIONS(1346),
    [anon_sym_STAR] = ACTIONS(1346),
    [anon_sym_QMARK] = ACTIONS(1348),
    [anon_sym_u8] = ACTIONS(1346),
    [anon_sym_i8] = ACTIONS(1346),
    [anon_sym_u16] = ACTIONS(1346),
    [anon_sym_i16] = ACTIONS(1346),
    [anon_sym_u32] = ACTIONS(1346),
    [anon_sym_i32] = ACTIONS(1346),
    [anon_sym_u64] = ACTIONS(1346),
    [anon_sym_i64] = ACTIONS(1346),
    [anon_sym_u128] = ACTIONS(1346),
    [anon_sym_i128] = ACTIONS(1346),
    [anon_sym_isize] = ACTIONS(1346),
    [anon_sym_usize] = ACTIONS(1346),
    [anon_sym_f32] = ACTIONS(1346),
    [anon_sym_f64] = ACTIONS(1346),
    [anon_sym_bool] = ACTIONS(1346),
    [anon_sym_str] = ACTIONS(1346),
    [anon_sym_char] = ACTIONS(1346),
    [anon_sym_DASH] = ACTIONS(1346),
    [anon_sym_SLASH] = ACTIONS(1346),
    [anon_sym_PERCENT] = ACTIONS(1346),
    [anon_sym_CARET] = ACTIONS(1346),
    [anon_sym_AMP] = ACTIONS(1346),
    [anon_sym_PIPE] = ACTIONS(1346),
    [anon_sym_AMP_AMP] = ACTIONS(1348),
    [anon_sym_PIPE_PIPE] = ACTIONS(1348),
    [anon_sym_LT_LT] = ACTIONS(1346),
    [anon_sym_GT_GT] = ACTIONS(1346),
    [anon_sym_PLUS_EQ] = ACTIONS(1348),
    [anon_sym_DASH_EQ] = ACTIONS(1348),
    [anon_sym_STAR_EQ] = ACTIONS(1348),
    [anon_sym_SLASH_EQ] = ACTIONS(1348),
    [anon_sym_PERCENT_EQ] = ACTIONS(1348),
    [anon_sym_CARET_EQ] = ACTIONS(1348),
    [anon_sym_AMP_EQ] = ACTIONS(1348),
    [anon_sym_PIPE_EQ] = ACTIONS(1348),
    [anon_sym_LT_LT_EQ] = ACTIONS(1348),
    [anon_sym_GT_GT_EQ] = ACTIONS(1348),
    [anon_sym_EQ] = ACTIONS(1346),
    [anon_sym_EQ_EQ] = ACTIONS(1348),
    [anon_sym_BANG_EQ] = ACTIONS(1348),
    [anon_sym_GT] = ACTIONS(1346),
    [anon_sym_LT] = ACTIONS(1346),
    [anon_sym_GT_EQ] = ACTIONS(1348),
    [anon_sym_LT_EQ] = ACTIONS(1348),
    [anon_sym_AT] = ACTIONS(1348),
    [anon_sym__] = ACTIONS(1346),
    [anon_sym_DOT] = ACTIONS(1346),
    [anon_sym_DOT_DOT] = ACTIONS(1346),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1348),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1348),
    [anon_sym_COLON_COLON] = ACTIONS(1348),
    [anon_sym_DASH_GT] = ACTIONS(1348),
    [anon_sym_POUND] = ACTIONS(1348),
    [anon_sym_SQUOTE] = ACTIONS(1346),
    [anon_sym_as] = ACTIONS(1346),
    [anon_sym_async] = ACTIONS(1346),
    [anon_sym_auto] = ACTIONS(1346),
    [anon_sym_await] = ACTIONS(1346),
    [anon_sym_become] = ACTIONS(1346),
    [anon_sym_break] = ACTIONS(1346),
    [anon_sym_const] = ACTIONS(1346),
    [anon_sym_continue] = ACTIONS(1346),
    [anon_sym_default] = ACTIONS(1346),
    [anon_sym_enum] = ACTIONS(1346),
    [anon_sym_fn] = ACTIONS(1346),
    [anon_sym_for] = ACTIONS(1346),
    [anon_sym_gen] = ACTIONS(1346),
    [anon_sym_if] = ACTIONS(1346),
    [anon_sym_impl] = ACTIONS(1346),
    [anon_sym_let] = ACTIONS(1346),
    [anon_sym_loop] = ACTIONS(1346),
    [anon_sym_match] = ACTIONS(1346),
    [anon_sym_mod] = ACTIONS(1346),
    [anon_sym_pub] = ACTIONS(1346),
    [anon_sym_return] = ACTIONS(1346),
    [anon_sym_raw] = ACTIONS(1346),
    [anon_sym_static] = ACTIONS(1346),
    [anon_sym_struct] = ACTIONS(1346),
    [anon_sym_trait] = ACTIONS(1346),
    [anon_sym_type] = ACTIONS(1346),
    [anon_sym_union] = ACTIONS(1346),
    [anon_sym_unsafe] = ACTIONS(1346),
    [anon_sym_use] = ACTIONS(1346),
    [anon_sym_where] = ACTIONS(1346),
    [anon_sym_while] = ACTIONS(1346),
    [sym_mutable_specifier] = ACTIONS(1346),
    [sym_integer_literal] = ACTIONS(1348),
    [aux_sym_string_literal_token1] = ACTIONS(1348),
    [sym_char_literal] = ACTIONS(1348),
    [anon_sym_true] = ACTIONS(1346),
    [anon_sym_false] = ACTIONS(1346),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1346),
    [sym_super] = ACTIONS(1346),
    [sym_crate] = ACTIONS(1346),
    [aux_sym_metavariable_token1] = ACTIONS(1348),
    [sym__raw_string_literal_start] = ACTIONS(1348),
    [sym_float_literal] = ACTIONS(1348),
  },
  [STATE(417)] = {
    [sym_line_comment] = STATE(417),
    [sym_block_comment] = STATE(417),
    [aux_sym__non_special_token_repeat1] = STATE(435),
    [sym_identifier] = ACTIONS(1362),
    [anon_sym_SEMI] = ACTIONS(1260),
    [anon_sym_LPAREN] = ACTIONS(1364),
    [anon_sym_RPAREN] = ACTIONS(1364),
    [anon_sym_LBRACE] = ACTIONS(1364),
    [anon_sym_RBRACE] = ACTIONS(1364),
    [anon_sym_COMMA] = ACTIONS(1260),
    [anon_sym_EQ_GT] = ACTIONS(1260),
    [anon_sym_BANG] = ACTIONS(1268),
    [anon_sym_LBRACK] = ACTIONS(1364),
    [anon_sym_RBRACK] = ACTIONS(1364),
    [anon_sym_COLON] = ACTIONS(1268),
    [anon_sym_DOLLAR] = ACTIONS(1364),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1268),
    [anon_sym_QMARK] = ACTIONS(1260),
    [anon_sym_u8] = ACTIONS(1362),
    [anon_sym_i8] = ACTIONS(1362),
    [anon_sym_u16] = ACTIONS(1362),
    [anon_sym_i16] = ACTIONS(1362),
    [anon_sym_u32] = ACTIONS(1362),
    [anon_sym_i32] = ACTIONS(1362),
    [anon_sym_u64] = ACTIONS(1362),
    [anon_sym_i64] = ACTIONS(1362),
    [anon_sym_u128] = ACTIONS(1362),
    [anon_sym_i128] = ACTIONS(1362),
    [anon_sym_isize] = ACTIONS(1362),
    [anon_sym_usize] = ACTIONS(1362),
    [anon_sym_f32] = ACTIONS(1362),
    [anon_sym_f64] = ACTIONS(1362),
    [anon_sym_bool] = ACTIONS(1362),
    [anon_sym_str] = ACTIONS(1362),
    [anon_sym_char] = ACTIONS(1362),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_SLASH] = ACTIONS(1268),
    [anon_sym_PERCENT] = ACTIONS(1268),
    [anon_sym_CARET] = ACTIONS(1268),
    [anon_sym_AMP] = ACTIONS(1268),
    [anon_sym_PIPE] = ACTIONS(1268),
    [anon_sym_AMP_AMP] = ACTIONS(1260),
    [anon_sym_PIPE_PIPE] = ACTIONS(1260),
    [anon_sym_LT_LT] = ACTIONS(1268),
    [anon_sym_GT_GT] = ACTIONS(1268),
    [anon_sym_PLUS_EQ] = ACTIONS(1260),
    [anon_sym_DASH_EQ] = ACTIONS(1260),
    [anon_sym_STAR_EQ] = ACTIONS(1260),
    [anon_sym_SLASH_EQ] = ACTIONS(1260),
    [anon_sym_PERCENT_EQ] = ACTIONS(1260),
    [anon_sym_CARET_EQ] = ACTIONS(1260),
    [anon_sym_AMP_EQ] = ACTIONS(1260),
    [anon_sym_PIPE_EQ] = ACTIONS(1260),
    [anon_sym_LT_LT_EQ] = ACTIONS(1260),
    [anon_sym_GT_GT_EQ] = ACTIONS(1260),
    [anon_sym_EQ] = ACTIONS(1268),
    [anon_sym_EQ_EQ] = ACTIONS(1260),
    [anon_sym_BANG_EQ] = ACTIONS(1260),
    [anon_sym_GT] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1268),
    [anon_sym_GT_EQ] = ACTIONS(1260),
    [anon_sym_LT_EQ] = ACTIONS(1260),
    [anon_sym_AT] = ACTIONS(1260),
    [anon_sym__] = ACTIONS(1268),
    [anon_sym_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT] = ACTIONS(1268),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1260),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1260),
    [anon_sym_COLON_COLON] = ACTIONS(1260),
    [anon_sym_DASH_GT] = ACTIONS(1260),
    [anon_sym_POUND] = ACTIONS(1260),
    [anon_sym_SQUOTE] = ACTIONS(1362),
    [anon_sym_as] = ACTIONS(1362),
    [anon_sym_async] = ACTIONS(1362),
    [anon_sym_auto] = ACTIONS(1362),
    [anon_sym_await] = ACTIONS(1362),
    [anon_sym_become] = ACTIONS(1362),
    [anon_sym_break] = ACTIONS(1362),
    [anon_sym_const] = ACTIONS(1362),
    [anon_sym_continue] = ACTIONS(1362),
    [anon_sym_default] = ACTIONS(1362),
    [anon_sym_enum] = ACTIONS(1362),
    [anon_sym_fn] = ACTIONS(1362),
    [anon_sym_for] = ACTIONS(1362),
    [anon_sym_gen] = ACTIONS(1362),
    [anon_sym_if] = ACTIONS(1362),
    [anon_sym_impl] = ACTIONS(1362),
    [anon_sym_let] = ACTIONS(1362),
    [anon_sym_loop] = ACTIONS(1362),
    [anon_sym_match] = ACTIONS(1362),
    [anon_sym_mod] = ACTIONS(1362),
    [anon_sym_pub] = ACTIONS(1362),
    [anon_sym_return] = ACTIONS(1362),
    [anon_sym_raw] = ACTIONS(1362),
    [anon_sym_static] = ACTIONS(1362),
    [anon_sym_struct] = ACTIONS(1362),
    [anon_sym_trait] = ACTIONS(1362),
    [anon_sym_type] = ACTIONS(1362),
    [anon_sym_union] = ACTIONS(1362),
    [anon_sym_unsafe] = ACTIONS(1362),
    [anon_sym_use] = ACTIONS(1362),
    [anon_sym_where] = ACTIONS(1362),
    [anon_sym_while] = ACTIONS(1362),
    [sym_mutable_specifier] = ACTIONS(1362),
    [sym_integer_literal] = ACTIONS(1364),
    [aux_sym_string_literal_token1] = ACTIONS(1364),
    [sym_char_literal] = ACTIONS(1364),
    [anon_sym_true] = ACTIONS(1362),
    [anon_sym_false] = ACTIONS(1362),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1362),
    [sym_super] = ACTIONS(1362),
    [sym_crate] = ACTIONS(1362),
    [sym__raw_string_literal_start] = ACTIONS(1364),
    [sym_float_literal] = ACTIONS(1364),
  },
  [STATE(418)] = {
    [sym_line_comment] = STATE(418),
    [sym_block_comment] = STATE(418),
    [sym_identifier] = ACTIONS(1366),
    [anon_sym_SEMI] = ACTIONS(1368),
    [anon_sym_LPAREN] = ACTIONS(1368),
    [anon_sym_RPAREN] = ACTIONS(1368),
    [anon_sym_LBRACE] = ACTIONS(1368),
    [anon_sym_RBRACE] = ACTIONS(1368),
    [anon_sym_COMMA] = ACTIONS(1368),
    [anon_sym_EQ_GT] = ACTIONS(1368),
    [anon_sym_BANG] = ACTIONS(1366),
    [anon_sym_LBRACK] = ACTIONS(1368),
    [anon_sym_RBRACK] = ACTIONS(1368),
    [anon_sym_COLON] = ACTIONS(1366),
    [anon_sym_DOLLAR] = ACTIONS(1366),
    [anon_sym_PLUS] = ACTIONS(1366),
    [anon_sym_STAR] = ACTIONS(1366),
    [anon_sym_QMARK] = ACTIONS(1368),
    [anon_sym_u8] = ACTIONS(1366),
    [anon_sym_i8] = ACTIONS(1366),
    [anon_sym_u16] = ACTIONS(1366),
    [anon_sym_i16] = ACTIONS(1366),
    [anon_sym_u32] = ACTIONS(1366),
    [anon_sym_i32] = ACTIONS(1366),
    [anon_sym_u64] = ACTIONS(1366),
    [anon_sym_i64] = ACTIONS(1366),
    [anon_sym_u128] = ACTIONS(1366),
    [anon_sym_i128] = ACTIONS(1366),
    [anon_sym_isize] = ACTIONS(1366),
    [anon_sym_usize] = ACTIONS(1366),
    [anon_sym_f32] = ACTIONS(1366),
    [anon_sym_f64] = ACTIONS(1366),
    [anon_sym_bool] = ACTIONS(1366),
    [anon_sym_str] = ACTIONS(1366),
    [anon_sym_char] = ACTIONS(1366),
    [anon_sym_DASH] = ACTIONS(1366),
    [anon_sym_SLASH] = ACTIONS(1366),
    [anon_sym_PERCENT] = ACTIONS(1366),
    [anon_sym_CARET] = ACTIONS(1366),
    [anon_sym_AMP] = ACTIONS(1366),
    [anon_sym_PIPE] = ACTIONS(1366),
    [anon_sym_AMP_AMP] = ACTIONS(1368),
    [anon_sym_PIPE_PIPE] = ACTIONS(1368),
    [anon_sym_LT_LT] = ACTIONS(1366),
    [anon_sym_GT_GT] = ACTIONS(1366),
    [anon_sym_PLUS_EQ] = ACTIONS(1368),
    [anon_sym_DASH_EQ] = ACTIONS(1368),
    [anon_sym_STAR_EQ] = ACTIONS(1368),
    [anon_sym_SLASH_EQ] = ACTIONS(1368),
    [anon_sym_PERCENT_EQ] = ACTIONS(1368),
    [anon_sym_CARET_EQ] = ACTIONS(1368),
    [anon_sym_AMP_EQ] = ACTIONS(1368),
    [anon_sym_PIPE_EQ] = ACTIONS(1368),
    [anon_sym_LT_LT_EQ] = ACTIONS(1368),
    [anon_sym_GT_GT_EQ] = ACTIONS(1368),
    [anon_sym_EQ] = ACTIONS(1366),
    [anon_sym_EQ_EQ] = ACTIONS(1368),
    [anon_sym_BANG_EQ] = ACTIONS(1368),
    [anon_sym_GT] = ACTIONS(1366),
    [anon_sym_LT] = ACTIONS(1366),
    [anon_sym_GT_EQ] = ACTIONS(1368),
    [anon_sym_LT_EQ] = ACTIONS(1368),
    [anon_sym_AT] = ACTIONS(1368),
    [anon_sym__] = ACTIONS(1366),
    [anon_sym_DOT] = ACTIONS(1366),
    [anon_sym_DOT_DOT] = ACTIONS(1366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1368),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1368),
    [anon_sym_COLON_COLON] = ACTIONS(1368),
    [anon_sym_DASH_GT] = ACTIONS(1368),
    [anon_sym_POUND] = ACTIONS(1368),
    [anon_sym_SQUOTE] = ACTIONS(1366),
    [anon_sym_as] = ACTIONS(1366),
    [anon_sym_async] = ACTIONS(1366),
    [anon_sym_auto] = ACTIONS(1366),
    [anon_sym_await] = ACTIONS(1366),
    [anon_sym_become] = ACTIONS(1366),
    [anon_sym_break] = ACTIONS(1366),
    [anon_sym_const] = ACTIONS(1366),
    [anon_sym_continue] = ACTIONS(1366),
    [anon_sym_default] = ACTIONS(1366),
    [anon_sym_enum] = ACTIONS(1366),
    [anon_sym_fn] = ACTIONS(1366),
    [anon_sym_for] = ACTIONS(1366),
    [anon_sym_gen] = ACTIONS(1366),
    [anon_sym_if] = ACTIONS(1366),
    [anon_sym_impl] = ACTIONS(1366),
    [anon_sym_let] = ACTIONS(1366),
    [anon_sym_loop] = ACTIONS(1366),
    [anon_sym_match] = ACTIONS(1366),
    [anon_sym_mod] = ACTIONS(1366),
    [anon_sym_pub] = ACTIONS(1366),
    [anon_sym_return] = ACTIONS(1366),
    [anon_sym_raw] = ACTIONS(1366),
    [anon_sym_static] = ACTIONS(1366),
    [anon_sym_struct] = ACTIONS(1366),
    [anon_sym_trait] = ACTIONS(1366),
    [anon_sym_type] = ACTIONS(1366),
    [anon_sym_union] = ACTIONS(1366),
    [anon_sym_unsafe] = ACTIONS(1366),
    [anon_sym_use] = ACTIONS(1366),
    [anon_sym_where] = ACTIONS(1366),
    [anon_sym_while] = ACTIONS(1366),
    [sym_mutable_specifier] = ACTIONS(1366),
    [sym_integer_literal] = ACTIONS(1368),
    [aux_sym_string_literal_token1] = ACTIONS(1368),
    [sym_char_literal] = ACTIONS(1368),
    [anon_sym_true] = ACTIONS(1366),
    [anon_sym_false] = ACTIONS(1366),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1366),
    [sym_super] = ACTIONS(1366),
    [sym_crate] = ACTIONS(1366),
    [aux_sym_metavariable_token1] = ACTIONS(1368),
    [sym__raw_string_literal_start] = ACTIONS(1368),
    [sym_float_literal] = ACTIONS(1368),
  },
  [STATE(419)] = {
    [sym_line_comment] = STATE(419),
    [sym_block_comment] = STATE(419),
    [sym_identifier] = ACTIONS(1370),
    [anon_sym_SEMI] = ACTIONS(1372),
    [anon_sym_LPAREN] = ACTIONS(1372),
    [anon_sym_RPAREN] = ACTIONS(1372),
    [anon_sym_LBRACE] = ACTIONS(1372),
    [anon_sym_RBRACE] = ACTIONS(1372),
    [anon_sym_COMMA] = ACTIONS(1372),
    [anon_sym_EQ_GT] = ACTIONS(1372),
    [anon_sym_BANG] = ACTIONS(1370),
    [anon_sym_LBRACK] = ACTIONS(1372),
    [anon_sym_RBRACK] = ACTIONS(1372),
    [anon_sym_COLON] = ACTIONS(1370),
    [anon_sym_DOLLAR] = ACTIONS(1370),
    [anon_sym_PLUS] = ACTIONS(1370),
    [anon_sym_STAR] = ACTIONS(1370),
    [anon_sym_QMARK] = ACTIONS(1372),
    [anon_sym_u8] = ACTIONS(1370),
    [anon_sym_i8] = ACTIONS(1370),
    [anon_sym_u16] = ACTIONS(1370),
    [anon_sym_i16] = ACTIONS(1370),
    [anon_sym_u32] = ACTIONS(1370),
    [anon_sym_i32] = ACTIONS(1370),
    [anon_sym_u64] = ACTIONS(1370),
    [anon_sym_i64] = ACTIONS(1370),
    [anon_sym_u128] = ACTIONS(1370),
    [anon_sym_i128] = ACTIONS(1370),
    [anon_sym_isize] = ACTIONS(1370),
    [anon_sym_usize] = ACTIONS(1370),
    [anon_sym_f32] = ACTIONS(1370),
    [anon_sym_f64] = ACTIONS(1370),
    [anon_sym_bool] = ACTIONS(1370),
    [anon_sym_str] = ACTIONS(1370),
    [anon_sym_char] = ACTIONS(1370),
    [anon_sym_DASH] = ACTIONS(1370),
    [anon_sym_SLASH] = ACTIONS(1370),
    [anon_sym_PERCENT] = ACTIONS(1370),
    [anon_sym_CARET] = ACTIONS(1370),
    [anon_sym_AMP] = ACTIONS(1370),
    [anon_sym_PIPE] = ACTIONS(1370),
    [anon_sym_AMP_AMP] = ACTIONS(1372),
    [anon_sym_PIPE_PIPE] = ACTIONS(1372),
    [anon_sym_LT_LT] = ACTIONS(1370),
    [anon_sym_GT_GT] = ACTIONS(1370),
    [anon_sym_PLUS_EQ] = ACTIONS(1372),
    [anon_sym_DASH_EQ] = ACTIONS(1372),
    [anon_sym_STAR_EQ] = ACTIONS(1372),
    [anon_sym_SLASH_EQ] = ACTIONS(1372),
    [anon_sym_PERCENT_EQ] = ACTIONS(1372),
    [anon_sym_CARET_EQ] = ACTIONS(1372),
    [anon_sym_AMP_EQ] = ACTIONS(1372),
    [anon_sym_PIPE_EQ] = ACTIONS(1372),
    [anon_sym_LT_LT_EQ] = ACTIONS(1372),
    [anon_sym_GT_GT_EQ] = ACTIONS(1372),
    [anon_sym_EQ] = ACTIONS(1370),
    [anon_sym_EQ_EQ] = ACTIONS(1372),
    [anon_sym_BANG_EQ] = ACTIONS(1372),
    [anon_sym_GT] = ACTIONS(1370),
    [anon_sym_LT] = ACTIONS(1370),
    [anon_sym_GT_EQ] = ACTIONS(1372),
    [anon_sym_LT_EQ] = ACTIONS(1372),
    [anon_sym_AT] = ACTIONS(1372),
    [anon_sym__] = ACTIONS(1370),
    [anon_sym_DOT] = ACTIONS(1370),
    [anon_sym_DOT_DOT] = ACTIONS(1370),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1372),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1372),
    [anon_sym_COLON_COLON] = ACTIONS(1372),
    [anon_sym_DASH_GT] = ACTIONS(1372),
    [anon_sym_POUND] = ACTIONS(1372),
    [anon_sym_SQUOTE] = ACTIONS(1370),
    [anon_sym_as] = ACTIONS(1370),
    [anon_sym_async] = ACTIONS(1370),
    [anon_sym_auto] = ACTIONS(1370),
    [anon_sym_await] = ACTIONS(1370),
    [anon_sym_become] = ACTIONS(1370),
    [anon_sym_break] = ACTIONS(1370),
    [anon_sym_const] = ACTIONS(1370),
    [anon_sym_continue] = ACTIONS(1370),
    [anon_sym_default] = ACTIONS(1370),
    [anon_sym_enum] = ACTIONS(1370),
    [anon_sym_fn] = ACTIONS(1370),
    [anon_sym_for] = ACTIONS(1370),
    [anon_sym_gen] = ACTIONS(1370),
    [anon_sym_if] = ACTIONS(1370),
    [anon_sym_impl] = ACTIONS(1370),
    [anon_sym_let] = ACTIONS(1370),
    [anon_sym_loop] = ACTIONS(1370),
    [anon_sym_match] = ACTIONS(1370),
    [anon_sym_mod] = ACTIONS(1370),
    [anon_sym_pub] = ACTIONS(1370),
    [anon_sym_return] = ACTIONS(1370),
    [anon_sym_raw] = ACTIONS(1370),
    [anon_sym_static] = ACTIONS(1370),
    [anon_sym_struct] = ACTIONS(1370),
    [anon_sym_trait] = ACTIONS(1370),
    [anon_sym_type] = ACTIONS(1370),
    [anon_sym_union] = ACTIONS(1370),
    [anon_sym_unsafe] = ACTIONS(1370),
    [anon_sym_use] = ACTIONS(1370),
    [anon_sym_where] = ACTIONS(1370),
    [anon_sym_while] = ACTIONS(1370),
    [sym_mutable_specifier] = ACTIONS(1370),
    [sym_integer_literal] = ACTIONS(1372),
    [aux_sym_string_literal_token1] = ACTIONS(1372),
    [sym_char_literal] = ACTIONS(1372),
    [anon_sym_true] = ACTIONS(1370),
    [anon_sym_false] = ACTIONS(1370),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1370),
    [sym_super] = ACTIONS(1370),
    [sym_crate] = ACTIONS(1370),
    [aux_sym_metavariable_token1] = ACTIONS(1372),
    [sym__raw_string_literal_start] = ACTIONS(1372),
    [sym_float_literal] = ACTIONS(1372),
  },
  [STATE(420)] = {
    [sym_line_comment] = STATE(420),
    [sym_block_comment] = STATE(420),
    [ts_builtin_sym_end] = ACTIONS(1374),
    [sym_identifier] = ACTIONS(1376),
    [anon_sym_SEMI] = ACTIONS(1374),
    [anon_sym_macro] = ACTIONS(1376),
    [anon_sym_LPAREN] = ACTIONS(1374),
    [anon_sym_LBRACE] = ACTIONS(1374),
    [anon_sym_RBRACE] = ACTIONS(1374),
    [anon_sym_macro_rules] = ACTIONS(1376),
    [anon_sym_BANG] = ACTIONS(1376),
    [anon_sym_LBRACK] = ACTIONS(1374),
    [anon_sym_PLUS] = ACTIONS(1376),
    [anon_sym_STAR] = ACTIONS(1376),
    [anon_sym_QMARK] = ACTIONS(1374),
    [anon_sym_u8] = ACTIONS(1376),
    [anon_sym_i8] = ACTIONS(1376),
    [anon_sym_u16] = ACTIONS(1376),
    [anon_sym_i16] = ACTIONS(1376),
    [anon_sym_u32] = ACTIONS(1376),
    [anon_sym_i32] = ACTIONS(1376),
    [anon_sym_u64] = ACTIONS(1376),
    [anon_sym_i64] = ACTIONS(1376),
    [anon_sym_u128] = ACTIONS(1376),
    [anon_sym_i128] = ACTIONS(1376),
    [anon_sym_isize] = ACTIONS(1376),
    [anon_sym_usize] = ACTIONS(1376),
    [anon_sym_f32] = ACTIONS(1376),
    [anon_sym_f64] = ACTIONS(1376),
    [anon_sym_bool] = ACTIONS(1376),
    [anon_sym_str] = ACTIONS(1376),
    [anon_sym_char] = ACTIONS(1376),
    [anon_sym_DASH] = ACTIONS(1376),
    [anon_sym_SLASH] = ACTIONS(1376),
    [anon_sym_PERCENT] = ACTIONS(1376),
    [anon_sym_CARET] = ACTIONS(1376),
    [anon_sym_AMP] = ACTIONS(1376),
    [anon_sym_PIPE] = ACTIONS(1376),
    [anon_sym_AMP_AMP] = ACTIONS(1374),
    [anon_sym_PIPE_PIPE] = ACTIONS(1374),
    [anon_sym_LT_LT] = ACTIONS(1376),
    [anon_sym_GT_GT] = ACTIONS(1376),
    [anon_sym_PLUS_EQ] = ACTIONS(1374),
    [anon_sym_DASH_EQ] = ACTIONS(1374),
    [anon_sym_STAR_EQ] = ACTIONS(1374),
    [anon_sym_SLASH_EQ] = ACTIONS(1374),
    [anon_sym_PERCENT_EQ] = ACTIONS(1374),
    [anon_sym_CARET_EQ] = ACTIONS(1374),
    [anon_sym_AMP_EQ] = ACTIONS(1374),
    [anon_sym_PIPE_EQ] = ACTIONS(1374),
    [anon_sym_LT_LT_EQ] = ACTIONS(1374),
    [anon_sym_GT_GT_EQ] = ACTIONS(1374),
    [anon_sym_EQ] = ACTIONS(1376),
    [anon_sym_EQ_EQ] = ACTIONS(1374),
    [anon_sym_BANG_EQ] = ACTIONS(1374),
    [anon_sym_GT] = ACTIONS(1376),
    [anon_sym_LT] = ACTIONS(1376),
    [anon_sym_GT_EQ] = ACTIONS(1374),
    [anon_sym_LT_EQ] = ACTIONS(1374),
    [anon_sym_DOT] = ACTIONS(1376),
    [anon_sym_DOT_DOT] = ACTIONS(1376),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1374),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1374),
    [anon_sym_COLON_COLON] = ACTIONS(1374),
    [anon_sym_POUND] = ACTIONS(1374),
    [anon_sym_SQUOTE] = ACTIONS(1376),
    [anon_sym_as] = ACTIONS(1376),
    [anon_sym_async] = ACTIONS(1376),
    [anon_sym_auto] = ACTIONS(1376),
    [anon_sym_await] = ACTIONS(1376),
    [anon_sym_become] = ACTIONS(1376),
    [anon_sym_break] = ACTIONS(1376),
    [anon_sym_const] = ACTIONS(1376),
    [anon_sym_continue] = ACTIONS(1376),
    [anon_sym_default] = ACTIONS(1376),
    [anon_sym_enum] = ACTIONS(1376),
    [anon_sym_fn] = ACTIONS(1376),
    [anon_sym_for] = ACTIONS(1376),
    [anon_sym_gen] = ACTIONS(1376),
    [anon_sym_if] = ACTIONS(1376),
    [anon_sym_impl] = ACTIONS(1376),
    [anon_sym_let] = ACTIONS(1376),
    [anon_sym_loop] = ACTIONS(1376),
    [anon_sym_match] = ACTIONS(1376),
    [anon_sym_mod] = ACTIONS(1376),
    [anon_sym_pub] = ACTIONS(1376),
    [anon_sym_return] = ACTIONS(1376),
    [anon_sym_raw] = ACTIONS(1376),
    [anon_sym_static] = ACTIONS(1376),
    [anon_sym_struct] = ACTIONS(1376),
    [anon_sym_trait] = ACTIONS(1376),
    [anon_sym_type] = ACTIONS(1376),
    [anon_sym_union] = ACTIONS(1376),
    [anon_sym_unsafe] = ACTIONS(1376),
    [anon_sym_use] = ACTIONS(1376),
    [anon_sym_while] = ACTIONS(1376),
    [anon_sym_extern] = ACTIONS(1376),
    [anon_sym_safe] = ACTIONS(1376),
    [anon_sym_else] = ACTIONS(1376),
    [anon_sym_yield] = ACTIONS(1376),
    [anon_sym_move] = ACTIONS(1376),
    [anon_sym_try] = ACTIONS(1376),
    [sym_integer_literal] = ACTIONS(1374),
    [aux_sym_string_literal_token1] = ACTIONS(1374),
    [sym_char_literal] = ACTIONS(1374),
    [anon_sym_true] = ACTIONS(1376),
    [anon_sym_false] = ACTIONS(1376),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1376),
    [sym_super] = ACTIONS(1376),
    [sym_crate] = ACTIONS(1376),
    [aux_sym_metavariable_token1] = ACTIONS(1374),
    [sym__raw_string_literal_start] = ACTIONS(1374),
    [sym_float_literal] = ACTIONS(1374),
    [sym__outer_block_doc_comment_start] = ACTIONS(1374),
    [sym__outer_line_doc_comment_start] = ACTIONS(1374),
  },
  [STATE(421)] = {
    [sym_line_comment] = STATE(421),
    [sym_block_comment] = STATE(421),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_SEMI] = ACTIONS(1380),
    [anon_sym_LPAREN] = ACTIONS(1380),
    [anon_sym_RPAREN] = ACTIONS(1380),
    [anon_sym_LBRACE] = ACTIONS(1380),
    [anon_sym_RBRACE] = ACTIONS(1380),
    [anon_sym_COMMA] = ACTIONS(1380),
    [anon_sym_EQ_GT] = ACTIONS(1380),
    [anon_sym_BANG] = ACTIONS(1378),
    [anon_sym_LBRACK] = ACTIONS(1380),
    [anon_sym_RBRACK] = ACTIONS(1380),
    [anon_sym_COLON] = ACTIONS(1378),
    [anon_sym_DOLLAR] = ACTIONS(1378),
    [anon_sym_PLUS] = ACTIONS(1378),
    [anon_sym_STAR] = ACTIONS(1378),
    [anon_sym_QMARK] = ACTIONS(1380),
    [anon_sym_u8] = ACTIONS(1378),
    [anon_sym_i8] = ACTIONS(1378),
    [anon_sym_u16] = ACTIONS(1378),
    [anon_sym_i16] = ACTIONS(1378),
    [anon_sym_u32] = ACTIONS(1378),
    [anon_sym_i32] = ACTIONS(1378),
    [anon_sym_u64] = ACTIONS(1378),
    [anon_sym_i64] = ACTIONS(1378),
    [anon_sym_u128] = ACTIONS(1378),
    [anon_sym_i128] = ACTIONS(1378),
    [anon_sym_isize] = ACTIONS(1378),
    [anon_sym_usize] = ACTIONS(1378),
    [anon_sym_f32] = ACTIONS(1378),
    [anon_sym_f64] = ACTIONS(1378),
    [anon_sym_bool] = ACTIONS(1378),
    [anon_sym_str] = ACTIONS(1378),
    [anon_sym_char] = ACTIONS(1378),
    [anon_sym_DASH] = ACTIONS(1378),
    [anon_sym_SLASH] = ACTIONS(1378),
    [anon_sym_PERCENT] = ACTIONS(1378),
    [anon_sym_CARET] = ACTIONS(1378),
    [anon_sym_AMP] = ACTIONS(1378),
    [anon_sym_PIPE] = ACTIONS(1378),
    [anon_sym_AMP_AMP] = ACTIONS(1380),
    [anon_sym_PIPE_PIPE] = ACTIONS(1380),
    [anon_sym_LT_LT] = ACTIONS(1378),
    [anon_sym_GT_GT] = ACTIONS(1378),
    [anon_sym_PLUS_EQ] = ACTIONS(1380),
    [anon_sym_DASH_EQ] = ACTIONS(1380),
    [anon_sym_STAR_EQ] = ACTIONS(1380),
    [anon_sym_SLASH_EQ] = ACTIONS(1380),
    [anon_sym_PERCENT_EQ] = ACTIONS(1380),
    [anon_sym_CARET_EQ] = ACTIONS(1380),
    [anon_sym_AMP_EQ] = ACTIONS(1380),
    [anon_sym_PIPE_EQ] = ACTIONS(1380),
    [anon_sym_LT_LT_EQ] = ACTIONS(1380),
    [anon_sym_GT_GT_EQ] = ACTIONS(1380),
    [anon_sym_EQ] = ACTIONS(1378),
    [anon_sym_EQ_EQ] = ACTIONS(1380),
    [anon_sym_BANG_EQ] = ACTIONS(1380),
    [anon_sym_GT] = ACTIONS(1378),
    [anon_sym_LT] = ACTIONS(1378),
    [anon_sym_GT_EQ] = ACTIONS(1380),
    [anon_sym_LT_EQ] = ACTIONS(1380),
    [anon_sym_AT] = ACTIONS(1380),
    [anon_sym__] = ACTIONS(1378),
    [anon_sym_DOT] = ACTIONS(1378),
    [anon_sym_DOT_DOT] = ACTIONS(1378),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1380),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1380),
    [anon_sym_COLON_COLON] = ACTIONS(1380),
    [anon_sym_DASH_GT] = ACTIONS(1380),
    [anon_sym_POUND] = ACTIONS(1380),
    [anon_sym_SQUOTE] = ACTIONS(1378),
    [anon_sym_as] = ACTIONS(1378),
    [anon_sym_async] = ACTIONS(1378),
    [anon_sym_auto] = ACTIONS(1378),
    [anon_sym_await] = ACTIONS(1378),
    [anon_sym_become] = ACTIONS(1378),
    [anon_sym_break] = ACTIONS(1378),
    [anon_sym_const] = ACTIONS(1378),
    [anon_sym_continue] = ACTIONS(1378),
    [anon_sym_default] = ACTIONS(1378),
    [anon_sym_enum] = ACTIONS(1378),
    [anon_sym_fn] = ACTIONS(1378),
    [anon_sym_for] = ACTIONS(1378),
    [anon_sym_gen] = ACTIONS(1378),
    [anon_sym_if] = ACTIONS(1378),
    [anon_sym_impl] = ACTIONS(1378),
    [anon_sym_let] = ACTIONS(1378),
    [anon_sym_loop] = ACTIONS(1378),
    [anon_sym_match] = ACTIONS(1378),
    [anon_sym_mod] = ACTIONS(1378),
    [anon_sym_pub] = ACTIONS(1378),
    [anon_sym_return] = ACTIONS(1378),
    [anon_sym_raw] = ACTIONS(1378),
    [anon_sym_static] = ACTIONS(1378),
    [anon_sym_struct] = ACTIONS(1378),
    [anon_sym_trait] = ACTIONS(1378),
    [anon_sym_type] = ACTIONS(1378),
    [anon_sym_union] = ACTIONS(1378),
    [anon_sym_unsafe] = ACTIONS(1378),
    [anon_sym_use] = ACTIONS(1378),
    [anon_sym_where] = ACTIONS(1378),
    [anon_sym_while] = ACTIONS(1378),
    [sym_mutable_specifier] = ACTIONS(1378),
    [sym_integer_literal] = ACTIONS(1380),
    [aux_sym_string_literal_token1] = ACTIONS(1380),
    [sym_char_literal] = ACTIONS(1380),
    [anon_sym_true] = ACTIONS(1378),
    [anon_sym_false] = ACTIONS(1378),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1378),
    [sym_super] = ACTIONS(1378),
    [sym_crate] = ACTIONS(1378),
    [aux_sym_metavariable_token1] = ACTIONS(1380),
    [sym__raw_string_literal_start] = ACTIONS(1380),
    [sym_float_literal] = ACTIONS(1380),
  },
  [STATE(422)] = {
    [sym_line_comment] = STATE(422),
    [sym_block_comment] = STATE(422),
    [sym_identifier] = ACTIONS(1382),
    [anon_sym_SEMI] = ACTIONS(1384),
    [anon_sym_LPAREN] = ACTIONS(1384),
    [anon_sym_RPAREN] = ACTIONS(1384),
    [anon_sym_LBRACE] = ACTIONS(1384),
    [anon_sym_RBRACE] = ACTIONS(1384),
    [anon_sym_COMMA] = ACTIONS(1384),
    [anon_sym_EQ_GT] = ACTIONS(1384),
    [anon_sym_BANG] = ACTIONS(1382),
    [anon_sym_LBRACK] = ACTIONS(1384),
    [anon_sym_RBRACK] = ACTIONS(1384),
    [anon_sym_COLON] = ACTIONS(1382),
    [anon_sym_DOLLAR] = ACTIONS(1382),
    [anon_sym_PLUS] = ACTIONS(1382),
    [anon_sym_STAR] = ACTIONS(1382),
    [anon_sym_QMARK] = ACTIONS(1384),
    [anon_sym_u8] = ACTIONS(1382),
    [anon_sym_i8] = ACTIONS(1382),
    [anon_sym_u16] = ACTIONS(1382),
    [anon_sym_i16] = ACTIONS(1382),
    [anon_sym_u32] = ACTIONS(1382),
    [anon_sym_i32] = ACTIONS(1382),
    [anon_sym_u64] = ACTIONS(1382),
    [anon_sym_i64] = ACTIONS(1382),
    [anon_sym_u128] = ACTIONS(1382),
    [anon_sym_i128] = ACTIONS(1382),
    [anon_sym_isize] = ACTIONS(1382),
    [anon_sym_usize] = ACTIONS(1382),
    [anon_sym_f32] = ACTIONS(1382),
    [anon_sym_f64] = ACTIONS(1382),
    [anon_sym_bool] = ACTIONS(1382),
    [anon_sym_str] = ACTIONS(1382),
    [anon_sym_char] = ACTIONS(1382),
    [anon_sym_DASH] = ACTIONS(1382),
    [anon_sym_SLASH] = ACTIONS(1382),
    [anon_sym_PERCENT] = ACTIONS(1382),
    [anon_sym_CARET] = ACTIONS(1382),
    [anon_sym_AMP] = ACTIONS(1382),
    [anon_sym_PIPE] = ACTIONS(1382),
    [anon_sym_AMP_AMP] = ACTIONS(1384),
    [anon_sym_PIPE_PIPE] = ACTIONS(1384),
    [anon_sym_LT_LT] = ACTIONS(1382),
    [anon_sym_GT_GT] = ACTIONS(1382),
    [anon_sym_PLUS_EQ] = ACTIONS(1384),
    [anon_sym_DASH_EQ] = ACTIONS(1384),
    [anon_sym_STAR_EQ] = ACTIONS(1384),
    [anon_sym_SLASH_EQ] = ACTIONS(1384),
    [anon_sym_PERCENT_EQ] = ACTIONS(1384),
    [anon_sym_CARET_EQ] = ACTIONS(1384),
    [anon_sym_AMP_EQ] = ACTIONS(1384),
    [anon_sym_PIPE_EQ] = ACTIONS(1384),
    [anon_sym_LT_LT_EQ] = ACTIONS(1384),
    [anon_sym_GT_GT_EQ] = ACTIONS(1384),
    [anon_sym_EQ] = ACTIONS(1382),
    [anon_sym_EQ_EQ] = ACTIONS(1384),
    [anon_sym_BANG_EQ] = ACTIONS(1384),
    [anon_sym_GT] = ACTIONS(1382),
    [anon_sym_LT] = ACTIONS(1382),
    [anon_sym_GT_EQ] = ACTIONS(1384),
    [anon_sym_LT_EQ] = ACTIONS(1384),
    [anon_sym_AT] = ACTIONS(1384),
    [anon_sym__] = ACTIONS(1382),
    [anon_sym_DOT] = ACTIONS(1382),
    [anon_sym_DOT_DOT] = ACTIONS(1382),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1384),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1384),
    [anon_sym_COLON_COLON] = ACTIONS(1384),
    [anon_sym_DASH_GT] = ACTIONS(1384),
    [anon_sym_POUND] = ACTIONS(1384),
    [anon_sym_SQUOTE] = ACTIONS(1382),
    [anon_sym_as] = ACTIONS(1382),
    [anon_sym_async] = ACTIONS(1382),
    [anon_sym_auto] = ACTIONS(1382),
    [anon_sym_await] = ACTIONS(1382),
    [anon_sym_become] = ACTIONS(1382),
    [anon_sym_break] = ACTIONS(1382),
    [anon_sym_const] = ACTIONS(1382),
    [anon_sym_continue] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1382),
    [anon_sym_enum] = ACTIONS(1382),
    [anon_sym_fn] = ACTIONS(1382),
    [anon_sym_for] = ACTIONS(1382),
    [anon_sym_gen] = ACTIONS(1382),
    [anon_sym_if] = ACTIONS(1382),
    [anon_sym_impl] = ACTIONS(1382),
    [anon_sym_let] = ACTIONS(1382),
    [anon_sym_loop] = ACTIONS(1382),
    [anon_sym_match] = ACTIONS(1382),
    [anon_sym_mod] = ACTIONS(1382),
    [anon_sym_pub] = ACTIONS(1382),
    [anon_sym_return] = ACTIONS(1382),
    [anon_sym_raw] = ACTIONS(1382),
    [anon_sym_static] = ACTIONS(1382),
    [anon_sym_struct] = ACTIONS(1382),
    [anon_sym_trait] = ACTIONS(1382),
    [anon_sym_type] = ACTIONS(1382),
    [anon_sym_union] = ACTIONS(1382),
    [anon_sym_unsafe] = ACTIONS(1382),
    [anon_sym_use] = ACTIONS(1382),
    [anon_sym_where] = ACTIONS(1382),
    [anon_sym_while] = ACTIONS(1382),
    [sym_mutable_specifier] = ACTIONS(1382),
    [sym_integer_literal] = ACTIONS(1384),
    [aux_sym_string_literal_token1] = ACTIONS(1384),
    [sym_char_literal] = ACTIONS(1384),
    [anon_sym_true] = ACTIONS(1382),
    [anon_sym_false] = ACTIONS(1382),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1382),
    [sym_super] = ACTIONS(1382),
    [sym_crate] = ACTIONS(1382),
    [aux_sym_metavariable_token1] = ACTIONS(1384),
    [sym__raw_string_literal_start] = ACTIONS(1384),
    [sym_float_literal] = ACTIONS(1384),
  },
  [STATE(423)] = {
    [sym_line_comment] = STATE(423),
    [sym_block_comment] = STATE(423),
    [sym_identifier] = ACTIONS(1386),
    [anon_sym_SEMI] = ACTIONS(1388),
    [anon_sym_LPAREN] = ACTIONS(1388),
    [anon_sym_RPAREN] = ACTIONS(1388),
    [anon_sym_LBRACE] = ACTIONS(1388),
    [anon_sym_RBRACE] = ACTIONS(1388),
    [anon_sym_COMMA] = ACTIONS(1388),
    [anon_sym_EQ_GT] = ACTIONS(1388),
    [anon_sym_BANG] = ACTIONS(1386),
    [anon_sym_LBRACK] = ACTIONS(1388),
    [anon_sym_RBRACK] = ACTIONS(1388),
    [anon_sym_COLON] = ACTIONS(1386),
    [anon_sym_DOLLAR] = ACTIONS(1386),
    [anon_sym_PLUS] = ACTIONS(1386),
    [anon_sym_STAR] = ACTIONS(1386),
    [anon_sym_QMARK] = ACTIONS(1388),
    [anon_sym_u8] = ACTIONS(1386),
    [anon_sym_i8] = ACTIONS(1386),
    [anon_sym_u16] = ACTIONS(1386),
    [anon_sym_i16] = ACTIONS(1386),
    [anon_sym_u32] = ACTIONS(1386),
    [anon_sym_i32] = ACTIONS(1386),
    [anon_sym_u64] = ACTIONS(1386),
    [anon_sym_i64] = ACTIONS(1386),
    [anon_sym_u128] = ACTIONS(1386),
    [anon_sym_i128] = ACTIONS(1386),
    [anon_sym_isize] = ACTIONS(1386),
    [anon_sym_usize] = ACTIONS(1386),
    [anon_sym_f32] = ACTIONS(1386),
    [anon_sym_f64] = ACTIONS(1386),
    [anon_sym_bool] = ACTIONS(1386),
    [anon_sym_str] = ACTIONS(1386),
    [anon_sym_char] = ACTIONS(1386),
    [anon_sym_DASH] = ACTIONS(1386),
    [anon_sym_SLASH] = ACTIONS(1386),
    [anon_sym_PERCENT] = ACTIONS(1386),
    [anon_sym_CARET] = ACTIONS(1386),
    [anon_sym_AMP] = ACTIONS(1386),
    [anon_sym_PIPE] = ACTIONS(1386),
    [anon_sym_AMP_AMP] = ACTIONS(1388),
    [anon_sym_PIPE_PIPE] = ACTIONS(1388),
    [anon_sym_LT_LT] = ACTIONS(1386),
    [anon_sym_GT_GT] = ACTIONS(1386),
    [anon_sym_PLUS_EQ] = ACTIONS(1388),
    [anon_sym_DASH_EQ] = ACTIONS(1388),
    [anon_sym_STAR_EQ] = ACTIONS(1388),
    [anon_sym_SLASH_EQ] = ACTIONS(1388),
    [anon_sym_PERCENT_EQ] = ACTIONS(1388),
    [anon_sym_CARET_EQ] = ACTIONS(1388),
    [anon_sym_AMP_EQ] = ACTIONS(1388),
    [anon_sym_PIPE_EQ] = ACTIONS(1388),
    [anon_sym_LT_LT_EQ] = ACTIONS(1388),
    [anon_sym_GT_GT_EQ] = ACTIONS(1388),
    [anon_sym_EQ] = ACTIONS(1386),
    [anon_sym_EQ_EQ] = ACTIONS(1388),
    [anon_sym_BANG_EQ] = ACTIONS(1388),
    [anon_sym_GT] = ACTIONS(1386),
    [anon_sym_LT] = ACTIONS(1386),
    [anon_sym_GT_EQ] = ACTIONS(1388),
    [anon_sym_LT_EQ] = ACTIONS(1388),
    [anon_sym_AT] = ACTIONS(1388),
    [anon_sym__] = ACTIONS(1386),
    [anon_sym_DOT] = ACTIONS(1386),
    [anon_sym_DOT_DOT] = ACTIONS(1386),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1388),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1388),
    [anon_sym_COLON_COLON] = ACTIONS(1388),
    [anon_sym_DASH_GT] = ACTIONS(1388),
    [anon_sym_POUND] = ACTIONS(1388),
    [anon_sym_SQUOTE] = ACTIONS(1386),
    [anon_sym_as] = ACTIONS(1386),
    [anon_sym_async] = ACTIONS(1386),
    [anon_sym_auto] = ACTIONS(1386),
    [anon_sym_await] = ACTIONS(1386),
    [anon_sym_become] = ACTIONS(1386),
    [anon_sym_break] = ACTIONS(1386),
    [anon_sym_const] = ACTIONS(1386),
    [anon_sym_continue] = ACTIONS(1386),
    [anon_sym_default] = ACTIONS(1386),
    [anon_sym_enum] = ACTIONS(1386),
    [anon_sym_fn] = ACTIONS(1386),
    [anon_sym_for] = ACTIONS(1386),
    [anon_sym_gen] = ACTIONS(1386),
    [anon_sym_if] = ACTIONS(1386),
    [anon_sym_impl] = ACTIONS(1386),
    [anon_sym_let] = ACTIONS(1386),
    [anon_sym_loop] = ACTIONS(1386),
    [anon_sym_match] = ACTIONS(1386),
    [anon_sym_mod] = ACTIONS(1386),
    [anon_sym_pub] = ACTIONS(1386),
    [anon_sym_return] = ACTIONS(1386),
    [anon_sym_raw] = ACTIONS(1386),
    [anon_sym_static] = ACTIONS(1386),
    [anon_sym_struct] = ACTIONS(1386),
    [anon_sym_trait] = ACTIONS(1386),
    [anon_sym_type] = ACTIONS(1386),
    [anon_sym_union] = ACTIONS(1386),
    [anon_sym_unsafe] = ACTIONS(1386),
    [anon_sym_use] = ACTIONS(1386),
    [anon_sym_where] = ACTIONS(1386),
    [anon_sym_while] = ACTIONS(1386),
    [sym_mutable_specifier] = ACTIONS(1386),
    [sym_integer_literal] = ACTIONS(1388),
    [aux_sym_string_literal_token1] = ACTIONS(1388),
    [sym_char_literal] = ACTIONS(1388),
    [anon_sym_true] = ACTIONS(1386),
    [anon_sym_false] = ACTIONS(1386),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1386),
    [sym_super] = ACTIONS(1386),
    [sym_crate] = ACTIONS(1386),
    [aux_sym_metavariable_token1] = ACTIONS(1388),
    [sym__raw_string_literal_start] = ACTIONS(1388),
    [sym_float_literal] = ACTIONS(1388),
  },
  [STATE(424)] = {
    [sym_line_comment] = STATE(424),
    [sym_block_comment] = STATE(424),
    [sym_identifier] = ACTIONS(1390),
    [anon_sym_SEMI] = ACTIONS(1392),
    [anon_sym_LPAREN] = ACTIONS(1392),
    [anon_sym_RPAREN] = ACTIONS(1392),
    [anon_sym_LBRACE] = ACTIONS(1392),
    [anon_sym_RBRACE] = ACTIONS(1392),
    [anon_sym_COMMA] = ACTIONS(1392),
    [anon_sym_EQ_GT] = ACTIONS(1392),
    [anon_sym_BANG] = ACTIONS(1390),
    [anon_sym_LBRACK] = ACTIONS(1392),
    [anon_sym_RBRACK] = ACTIONS(1392),
    [anon_sym_COLON] = ACTIONS(1390),
    [anon_sym_DOLLAR] = ACTIONS(1390),
    [anon_sym_PLUS] = ACTIONS(1390),
    [anon_sym_STAR] = ACTIONS(1390),
    [anon_sym_QMARK] = ACTIONS(1392),
    [anon_sym_u8] = ACTIONS(1390),
    [anon_sym_i8] = ACTIONS(1390),
    [anon_sym_u16] = ACTIONS(1390),
    [anon_sym_i16] = ACTIONS(1390),
    [anon_sym_u32] = ACTIONS(1390),
    [anon_sym_i32] = ACTIONS(1390),
    [anon_sym_u64] = ACTIONS(1390),
    [anon_sym_i64] = ACTIONS(1390),
    [anon_sym_u128] = ACTIONS(1390),
    [anon_sym_i128] = ACTIONS(1390),
    [anon_sym_isize] = ACTIONS(1390),
    [anon_sym_usize] = ACTIONS(1390),
    [anon_sym_f32] = ACTIONS(1390),
    [anon_sym_f64] = ACTIONS(1390),
    [anon_sym_bool] = ACTIONS(1390),
    [anon_sym_str] = ACTIONS(1390),
    [anon_sym_char] = ACTIONS(1390),
    [anon_sym_DASH] = ACTIONS(1390),
    [anon_sym_SLASH] = ACTIONS(1390),
    [anon_sym_PERCENT] = ACTIONS(1390),
    [anon_sym_CARET] = ACTIONS(1390),
    [anon_sym_AMP] = ACTIONS(1390),
    [anon_sym_PIPE] = ACTIONS(1390),
    [anon_sym_AMP_AMP] = ACTIONS(1392),
    [anon_sym_PIPE_PIPE] = ACTIONS(1392),
    [anon_sym_LT_LT] = ACTIONS(1390),
    [anon_sym_GT_GT] = ACTIONS(1390),
    [anon_sym_PLUS_EQ] = ACTIONS(1392),
    [anon_sym_DASH_EQ] = ACTIONS(1392),
    [anon_sym_STAR_EQ] = ACTIONS(1392),
    [anon_sym_SLASH_EQ] = ACTIONS(1392),
    [anon_sym_PERCENT_EQ] = ACTIONS(1392),
    [anon_sym_CARET_EQ] = ACTIONS(1392),
    [anon_sym_AMP_EQ] = ACTIONS(1392),
    [anon_sym_PIPE_EQ] = ACTIONS(1392),
    [anon_sym_LT_LT_EQ] = ACTIONS(1392),
    [anon_sym_GT_GT_EQ] = ACTIONS(1392),
    [anon_sym_EQ] = ACTIONS(1390),
    [anon_sym_EQ_EQ] = ACTIONS(1392),
    [anon_sym_BANG_EQ] = ACTIONS(1392),
    [anon_sym_GT] = ACTIONS(1390),
    [anon_sym_LT] = ACTIONS(1390),
    [anon_sym_GT_EQ] = ACTIONS(1392),
    [anon_sym_LT_EQ] = ACTIONS(1392),
    [anon_sym_AT] = ACTIONS(1392),
    [anon_sym__] = ACTIONS(1390),
    [anon_sym_DOT] = ACTIONS(1390),
    [anon_sym_DOT_DOT] = ACTIONS(1390),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1392),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1392),
    [anon_sym_COLON_COLON] = ACTIONS(1392),
    [anon_sym_DASH_GT] = ACTIONS(1392),
    [anon_sym_POUND] = ACTIONS(1392),
    [anon_sym_SQUOTE] = ACTIONS(1390),
    [anon_sym_as] = ACTIONS(1390),
    [anon_sym_async] = ACTIONS(1390),
    [anon_sym_auto] = ACTIONS(1390),
    [anon_sym_await] = ACTIONS(1390),
    [anon_sym_become] = ACTIONS(1390),
    [anon_sym_break] = ACTIONS(1390),
    [anon_sym_const] = ACTIONS(1390),
    [anon_sym_continue] = ACTIONS(1390),
    [anon_sym_default] = ACTIONS(1390),
    [anon_sym_enum] = ACTIONS(1390),
    [anon_sym_fn] = ACTIONS(1390),
    [anon_sym_for] = ACTIONS(1390),
    [anon_sym_gen] = ACTIONS(1390),
    [anon_sym_if] = ACTIONS(1390),
    [anon_sym_impl] = ACTIONS(1390),
    [anon_sym_let] = ACTIONS(1390),
    [anon_sym_loop] = ACTIONS(1390),
    [anon_sym_match] = ACTIONS(1390),
    [anon_sym_mod] = ACTIONS(1390),
    [anon_sym_pub] = ACTIONS(1390),
    [anon_sym_return] = ACTIONS(1390),
    [anon_sym_raw] = ACTIONS(1390),
    [anon_sym_static] = ACTIONS(1390),
    [anon_sym_struct] = ACTIONS(1390),
    [anon_sym_trait] = ACTIONS(1390),
    [anon_sym_type] = ACTIONS(1390),
    [anon_sym_union] = ACTIONS(1390),
    [anon_sym_unsafe] = ACTIONS(1390),
    [anon_sym_use] = ACTIONS(1390),
    [anon_sym_where] = ACTIONS(1390),
    [anon_sym_while] = ACTIONS(1390),
    [sym_mutable_specifier] = ACTIONS(1390),
    [sym_integer_literal] = ACTIONS(1392),
    [aux_sym_string_literal_token1] = ACTIONS(1392),
    [sym_char_literal] = ACTIONS(1392),
    [anon_sym_true] = ACTIONS(1390),
    [anon_sym_false] = ACTIONS(1390),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1390),
    [sym_super] = ACTIONS(1390),
    [sym_crate] = ACTIONS(1390),
    [aux_sym_metavariable_token1] = ACTIONS(1392),
    [sym__raw_string_literal_start] = ACTIONS(1392),
    [sym_float_literal] = ACTIONS(1392),
  },
  [STATE(425)] = {
    [sym_line_comment] = STATE(425),
    [sym_block_comment] = STATE(425),
    [sym_identifier] = ACTIONS(1394),
    [anon_sym_SEMI] = ACTIONS(1396),
    [anon_sym_LPAREN] = ACTIONS(1396),
    [anon_sym_RPAREN] = ACTIONS(1396),
    [anon_sym_LBRACE] = ACTIONS(1396),
    [anon_sym_RBRACE] = ACTIONS(1396),
    [anon_sym_COMMA] = ACTIONS(1396),
    [anon_sym_EQ_GT] = ACTIONS(1396),
    [anon_sym_BANG] = ACTIONS(1394),
    [anon_sym_LBRACK] = ACTIONS(1396),
    [anon_sym_RBRACK] = ACTIONS(1396),
    [anon_sym_COLON] = ACTIONS(1394),
    [anon_sym_DOLLAR] = ACTIONS(1394),
    [anon_sym_PLUS] = ACTIONS(1394),
    [anon_sym_STAR] = ACTIONS(1394),
    [anon_sym_QMARK] = ACTIONS(1396),
    [anon_sym_u8] = ACTIONS(1394),
    [anon_sym_i8] = ACTIONS(1394),
    [anon_sym_u16] = ACTIONS(1394),
    [anon_sym_i16] = ACTIONS(1394),
    [anon_sym_u32] = ACTIONS(1394),
    [anon_sym_i32] = ACTIONS(1394),
    [anon_sym_u64] = ACTIONS(1394),
    [anon_sym_i64] = ACTIONS(1394),
    [anon_sym_u128] = ACTIONS(1394),
    [anon_sym_i128] = ACTIONS(1394),
    [anon_sym_isize] = ACTIONS(1394),
    [anon_sym_usize] = ACTIONS(1394),
    [anon_sym_f32] = ACTIONS(1394),
    [anon_sym_f64] = ACTIONS(1394),
    [anon_sym_bool] = ACTIONS(1394),
    [anon_sym_str] = ACTIONS(1394),
    [anon_sym_char] = ACTIONS(1394),
    [anon_sym_DASH] = ACTIONS(1394),
    [anon_sym_SLASH] = ACTIONS(1394),
    [anon_sym_PERCENT] = ACTIONS(1394),
    [anon_sym_CARET] = ACTIONS(1394),
    [anon_sym_AMP] = ACTIONS(1394),
    [anon_sym_PIPE] = ACTIONS(1394),
    [anon_sym_AMP_AMP] = ACTIONS(1396),
    [anon_sym_PIPE_PIPE] = ACTIONS(1396),
    [anon_sym_LT_LT] = ACTIONS(1394),
    [anon_sym_GT_GT] = ACTIONS(1394),
    [anon_sym_PLUS_EQ] = ACTIONS(1396),
    [anon_sym_DASH_EQ] = ACTIONS(1396),
    [anon_sym_STAR_EQ] = ACTIONS(1396),
    [anon_sym_SLASH_EQ] = ACTIONS(1396),
    [anon_sym_PERCENT_EQ] = ACTIONS(1396),
    [anon_sym_CARET_EQ] = ACTIONS(1396),
    [anon_sym_AMP_EQ] = ACTIONS(1396),
    [anon_sym_PIPE_EQ] = ACTIONS(1396),
    [anon_sym_LT_LT_EQ] = ACTIONS(1396),
    [anon_sym_GT_GT_EQ] = ACTIONS(1396),
    [anon_sym_EQ] = ACTIONS(1394),
    [anon_sym_EQ_EQ] = ACTIONS(1396),
    [anon_sym_BANG_EQ] = ACTIONS(1396),
    [anon_sym_GT] = ACTIONS(1394),
    [anon_sym_LT] = ACTIONS(1394),
    [anon_sym_GT_EQ] = ACTIONS(1396),
    [anon_sym_LT_EQ] = ACTIONS(1396),
    [anon_sym_AT] = ACTIONS(1396),
    [anon_sym__] = ACTIONS(1394),
    [anon_sym_DOT] = ACTIONS(1394),
    [anon_sym_DOT_DOT] = ACTIONS(1394),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1396),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1396),
    [anon_sym_COLON_COLON] = ACTIONS(1396),
    [anon_sym_DASH_GT] = ACTIONS(1396),
    [anon_sym_POUND] = ACTIONS(1396),
    [anon_sym_SQUOTE] = ACTIONS(1394),
    [anon_sym_as] = ACTIONS(1394),
    [anon_sym_async] = ACTIONS(1394),
    [anon_sym_auto] = ACTIONS(1394),
    [anon_sym_await] = ACTIONS(1394),
    [anon_sym_become] = ACTIONS(1394),
    [anon_sym_break] = ACTIONS(1394),
    [anon_sym_const] = ACTIONS(1394),
    [anon_sym_continue] = ACTIONS(1394),
    [anon_sym_default] = ACTIONS(1394),
    [anon_sym_enum] = ACTIONS(1394),
    [anon_sym_fn] = ACTIONS(1394),
    [anon_sym_for] = ACTIONS(1394),
    [anon_sym_gen] = ACTIONS(1394),
    [anon_sym_if] = ACTIONS(1394),
    [anon_sym_impl] = ACTIONS(1394),
    [anon_sym_let] = ACTIONS(1394),
    [anon_sym_loop] = ACTIONS(1394),
    [anon_sym_match] = ACTIONS(1394),
    [anon_sym_mod] = ACTIONS(1394),
    [anon_sym_pub] = ACTIONS(1394),
    [anon_sym_return] = ACTIONS(1394),
    [anon_sym_raw] = ACTIONS(1394),
    [anon_sym_static] = ACTIONS(1394),
    [anon_sym_struct] = ACTIONS(1394),
    [anon_sym_trait] = ACTIONS(1394),
    [anon_sym_type] = ACTIONS(1394),
    [anon_sym_union] = ACTIONS(1394),
    [anon_sym_unsafe] = ACTIONS(1394),
    [anon_sym_use] = ACTIONS(1394),
    [anon_sym_where] = ACTIONS(1394),
    [anon_sym_while] = ACTIONS(1394),
    [sym_mutable_specifier] = ACTIONS(1394),
    [sym_integer_literal] = ACTIONS(1396),
    [aux_sym_string_literal_token1] = ACTIONS(1396),
    [sym_char_literal] = ACTIONS(1396),
    [anon_sym_true] = ACTIONS(1394),
    [anon_sym_false] = ACTIONS(1394),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1394),
    [sym_super] = ACTIONS(1394),
    [sym_crate] = ACTIONS(1394),
    [aux_sym_metavariable_token1] = ACTIONS(1396),
    [sym__raw_string_literal_start] = ACTIONS(1396),
    [sym_float_literal] = ACTIONS(1396),
  },
  [STATE(426)] = {
    [sym_line_comment] = STATE(426),
    [sym_block_comment] = STATE(426),
    [ts_builtin_sym_end] = ACTIONS(1398),
    [sym_identifier] = ACTIONS(1400),
    [anon_sym_SEMI] = ACTIONS(1398),
    [anon_sym_macro] = ACTIONS(1400),
    [anon_sym_LPAREN] = ACTIONS(1398),
    [anon_sym_LBRACE] = ACTIONS(1398),
    [anon_sym_RBRACE] = ACTIONS(1398),
    [anon_sym_macro_rules] = ACTIONS(1400),
    [anon_sym_BANG] = ACTIONS(1400),
    [anon_sym_LBRACK] = ACTIONS(1398),
    [anon_sym_PLUS] = ACTIONS(1400),
    [anon_sym_STAR] = ACTIONS(1400),
    [anon_sym_QMARK] = ACTIONS(1398),
    [anon_sym_u8] = ACTIONS(1400),
    [anon_sym_i8] = ACTIONS(1400),
    [anon_sym_u16] = ACTIONS(1400),
    [anon_sym_i16] = ACTIONS(1400),
    [anon_sym_u32] = ACTIONS(1400),
    [anon_sym_i32] = ACTIONS(1400),
    [anon_sym_u64] = ACTIONS(1400),
    [anon_sym_i64] = ACTIONS(1400),
    [anon_sym_u128] = ACTIONS(1400),
    [anon_sym_i128] = ACTIONS(1400),
    [anon_sym_isize] = ACTIONS(1400),
    [anon_sym_usize] = ACTIONS(1400),
    [anon_sym_f32] = ACTIONS(1400),
    [anon_sym_f64] = ACTIONS(1400),
    [anon_sym_bool] = ACTIONS(1400),
    [anon_sym_str] = ACTIONS(1400),
    [anon_sym_char] = ACTIONS(1400),
    [anon_sym_DASH] = ACTIONS(1400),
    [anon_sym_SLASH] = ACTIONS(1400),
    [anon_sym_PERCENT] = ACTIONS(1400),
    [anon_sym_CARET] = ACTIONS(1400),
    [anon_sym_AMP] = ACTIONS(1400),
    [anon_sym_PIPE] = ACTIONS(1400),
    [anon_sym_AMP_AMP] = ACTIONS(1398),
    [anon_sym_PIPE_PIPE] = ACTIONS(1398),
    [anon_sym_LT_LT] = ACTIONS(1400),
    [anon_sym_GT_GT] = ACTIONS(1400),
    [anon_sym_PLUS_EQ] = ACTIONS(1398),
    [anon_sym_DASH_EQ] = ACTIONS(1398),
    [anon_sym_STAR_EQ] = ACTIONS(1398),
    [anon_sym_SLASH_EQ] = ACTIONS(1398),
    [anon_sym_PERCENT_EQ] = ACTIONS(1398),
    [anon_sym_CARET_EQ] = ACTIONS(1398),
    [anon_sym_AMP_EQ] = ACTIONS(1398),
    [anon_sym_PIPE_EQ] = ACTIONS(1398),
    [anon_sym_LT_LT_EQ] = ACTIONS(1398),
    [anon_sym_GT_GT_EQ] = ACTIONS(1398),
    [anon_sym_EQ] = ACTIONS(1400),
    [anon_sym_EQ_EQ] = ACTIONS(1398),
    [anon_sym_BANG_EQ] = ACTIONS(1398),
    [anon_sym_GT] = ACTIONS(1400),
    [anon_sym_LT] = ACTIONS(1400),
    [anon_sym_GT_EQ] = ACTIONS(1398),
    [anon_sym_LT_EQ] = ACTIONS(1398),
    [anon_sym_DOT] = ACTIONS(1400),
    [anon_sym_DOT_DOT] = ACTIONS(1400),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1398),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1398),
    [anon_sym_COLON_COLON] = ACTIONS(1398),
    [anon_sym_POUND] = ACTIONS(1398),
    [anon_sym_SQUOTE] = ACTIONS(1400),
    [anon_sym_as] = ACTIONS(1400),
    [anon_sym_async] = ACTIONS(1400),
    [anon_sym_auto] = ACTIONS(1400),
    [anon_sym_await] = ACTIONS(1400),
    [anon_sym_become] = ACTIONS(1400),
    [anon_sym_break] = ACTIONS(1400),
    [anon_sym_const] = ACTIONS(1400),
    [anon_sym_continue] = ACTIONS(1400),
    [anon_sym_default] = ACTIONS(1400),
    [anon_sym_enum] = ACTIONS(1400),
    [anon_sym_fn] = ACTIONS(1400),
    [anon_sym_for] = ACTIONS(1400),
    [anon_sym_gen] = ACTIONS(1400),
    [anon_sym_if] = ACTIONS(1400),
    [anon_sym_impl] = ACTIONS(1400),
    [anon_sym_let] = ACTIONS(1400),
    [anon_sym_loop] = ACTIONS(1400),
    [anon_sym_match] = ACTIONS(1400),
    [anon_sym_mod] = ACTIONS(1400),
    [anon_sym_pub] = ACTIONS(1400),
    [anon_sym_return] = ACTIONS(1400),
    [anon_sym_raw] = ACTIONS(1400),
    [anon_sym_static] = ACTIONS(1400),
    [anon_sym_struct] = ACTIONS(1400),
    [anon_sym_trait] = ACTIONS(1400),
    [anon_sym_type] = ACTIONS(1400),
    [anon_sym_union] = ACTIONS(1400),
    [anon_sym_unsafe] = ACTIONS(1400),
    [anon_sym_use] = ACTIONS(1400),
    [anon_sym_while] = ACTIONS(1400),
    [anon_sym_extern] = ACTIONS(1400),
    [anon_sym_safe] = ACTIONS(1400),
    [anon_sym_else] = ACTIONS(1400),
    [anon_sym_yield] = ACTIONS(1400),
    [anon_sym_move] = ACTIONS(1400),
    [anon_sym_try] = ACTIONS(1400),
    [sym_integer_literal] = ACTIONS(1398),
    [aux_sym_string_literal_token1] = ACTIONS(1398),
    [sym_char_literal] = ACTIONS(1398),
    [anon_sym_true] = ACTIONS(1400),
    [anon_sym_false] = ACTIONS(1400),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1400),
    [sym_super] = ACTIONS(1400),
    [sym_crate] = ACTIONS(1400),
    [aux_sym_metavariable_token1] = ACTIONS(1398),
    [sym__raw_string_literal_start] = ACTIONS(1398),
    [sym_float_literal] = ACTIONS(1398),
    [sym__outer_block_doc_comment_start] = ACTIONS(1398),
    [sym__outer_line_doc_comment_start] = ACTIONS(1398),
  },
  [STATE(427)] = {
    [sym_line_comment] = STATE(427),
    [sym_block_comment] = STATE(427),
    [sym_identifier] = ACTIONS(1402),
    [anon_sym_SEMI] = ACTIONS(1404),
    [anon_sym_LPAREN] = ACTIONS(1404),
    [anon_sym_RPAREN] = ACTIONS(1404),
    [anon_sym_LBRACE] = ACTIONS(1404),
    [anon_sym_RBRACE] = ACTIONS(1404),
    [anon_sym_COMMA] = ACTIONS(1404),
    [anon_sym_EQ_GT] = ACTIONS(1404),
    [anon_sym_BANG] = ACTIONS(1402),
    [anon_sym_LBRACK] = ACTIONS(1404),
    [anon_sym_RBRACK] = ACTIONS(1404),
    [anon_sym_COLON] = ACTIONS(1402),
    [anon_sym_DOLLAR] = ACTIONS(1402),
    [anon_sym_PLUS] = ACTIONS(1402),
    [anon_sym_STAR] = ACTIONS(1402),
    [anon_sym_QMARK] = ACTIONS(1404),
    [anon_sym_u8] = ACTIONS(1402),
    [anon_sym_i8] = ACTIONS(1402),
    [anon_sym_u16] = ACTIONS(1402),
    [anon_sym_i16] = ACTIONS(1402),
    [anon_sym_u32] = ACTIONS(1402),
    [anon_sym_i32] = ACTIONS(1402),
    [anon_sym_u64] = ACTIONS(1402),
    [anon_sym_i64] = ACTIONS(1402),
    [anon_sym_u128] = ACTIONS(1402),
    [anon_sym_i128] = ACTIONS(1402),
    [anon_sym_isize] = ACTIONS(1402),
    [anon_sym_usize] = ACTIONS(1402),
    [anon_sym_f32] = ACTIONS(1402),
    [anon_sym_f64] = ACTIONS(1402),
    [anon_sym_bool] = ACTIONS(1402),
    [anon_sym_str] = ACTIONS(1402),
    [anon_sym_char] = ACTIONS(1402),
    [anon_sym_DASH] = ACTIONS(1402),
    [anon_sym_SLASH] = ACTIONS(1402),
    [anon_sym_PERCENT] = ACTIONS(1402),
    [anon_sym_CARET] = ACTIONS(1402),
    [anon_sym_AMP] = ACTIONS(1402),
    [anon_sym_PIPE] = ACTIONS(1402),
    [anon_sym_AMP_AMP] = ACTIONS(1404),
    [anon_sym_PIPE_PIPE] = ACTIONS(1404),
    [anon_sym_LT_LT] = ACTIONS(1402),
    [anon_sym_GT_GT] = ACTIONS(1402),
    [anon_sym_PLUS_EQ] = ACTIONS(1404),
    [anon_sym_DASH_EQ] = ACTIONS(1404),
    [anon_sym_STAR_EQ] = ACTIONS(1404),
    [anon_sym_SLASH_EQ] = ACTIONS(1404),
    [anon_sym_PERCENT_EQ] = ACTIONS(1404),
    [anon_sym_CARET_EQ] = ACTIONS(1404),
    [anon_sym_AMP_EQ] = ACTIONS(1404),
    [anon_sym_PIPE_EQ] = ACTIONS(1404),
    [anon_sym_LT_LT_EQ] = ACTIONS(1404),
    [anon_sym_GT_GT_EQ] = ACTIONS(1404),
    [anon_sym_EQ] = ACTIONS(1402),
    [anon_sym_EQ_EQ] = ACTIONS(1404),
    [anon_sym_BANG_EQ] = ACTIONS(1404),
    [anon_sym_GT] = ACTIONS(1402),
    [anon_sym_LT] = ACTIONS(1402),
    [anon_sym_GT_EQ] = ACTIONS(1404),
    [anon_sym_LT_EQ] = ACTIONS(1404),
    [anon_sym_AT] = ACTIONS(1404),
    [anon_sym__] = ACTIONS(1402),
    [anon_sym_DOT] = ACTIONS(1402),
    [anon_sym_DOT_DOT] = ACTIONS(1402),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1404),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1404),
    [anon_sym_COLON_COLON] = ACTIONS(1404),
    [anon_sym_DASH_GT] = ACTIONS(1404),
    [anon_sym_POUND] = ACTIONS(1404),
    [anon_sym_SQUOTE] = ACTIONS(1402),
    [anon_sym_as] = ACTIONS(1402),
    [anon_sym_async] = ACTIONS(1402),
    [anon_sym_auto] = ACTIONS(1402),
    [anon_sym_await] = ACTIONS(1402),
    [anon_sym_become] = ACTIONS(1402),
    [anon_sym_break] = ACTIONS(1402),
    [anon_sym_const] = ACTIONS(1402),
    [anon_sym_continue] = ACTIONS(1402),
    [anon_sym_default] = ACTIONS(1402),
    [anon_sym_enum] = ACTIONS(1402),
    [anon_sym_fn] = ACTIONS(1402),
    [anon_sym_for] = ACTIONS(1402),
    [anon_sym_gen] = ACTIONS(1402),
    [anon_sym_if] = ACTIONS(1402),
    [anon_sym_impl] = ACTIONS(1402),
    [anon_sym_let] = ACTIONS(1402),
    [anon_sym_loop] = ACTIONS(1402),
    [anon_sym_match] = ACTIONS(1402),
    [anon_sym_mod] = ACTIONS(1402),
    [anon_sym_pub] = ACTIONS(1402),
    [anon_sym_return] = ACTIONS(1402),
    [anon_sym_raw] = ACTIONS(1402),
    [anon_sym_static] = ACTIONS(1402),
    [anon_sym_struct] = ACTIONS(1402),
    [anon_sym_trait] = ACTIONS(1402),
    [anon_sym_type] = ACTIONS(1402),
    [anon_sym_union] = ACTIONS(1402),
    [anon_sym_unsafe] = ACTIONS(1402),
    [anon_sym_use] = ACTIONS(1402),
    [anon_sym_where] = ACTIONS(1402),
    [anon_sym_while] = ACTIONS(1402),
    [sym_mutable_specifier] = ACTIONS(1402),
    [sym_integer_literal] = ACTIONS(1404),
    [aux_sym_string_literal_token1] = ACTIONS(1404),
    [sym_char_literal] = ACTIONS(1404),
    [anon_sym_true] = ACTIONS(1402),
    [anon_sym_false] = ACTIONS(1402),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1402),
    [sym_super] = ACTIONS(1402),
    [sym_crate] = ACTIONS(1402),
    [aux_sym_metavariable_token1] = ACTIONS(1404),
    [sym__raw_string_literal_start] = ACTIONS(1404),
    [sym_float_literal] = ACTIONS(1404),
  },
  [STATE(428)] = {
    [sym_line_comment] = STATE(428),
    [sym_block_comment] = STATE(428),
    [sym_identifier] = ACTIONS(1406),
    [anon_sym_SEMI] = ACTIONS(1408),
    [anon_sym_LPAREN] = ACTIONS(1408),
    [anon_sym_RPAREN] = ACTIONS(1408),
    [anon_sym_LBRACE] = ACTIONS(1408),
    [anon_sym_RBRACE] = ACTIONS(1408),
    [anon_sym_COMMA] = ACTIONS(1408),
    [anon_sym_EQ_GT] = ACTIONS(1408),
    [anon_sym_BANG] = ACTIONS(1406),
    [anon_sym_LBRACK] = ACTIONS(1408),
    [anon_sym_RBRACK] = ACTIONS(1408),
    [anon_sym_COLON] = ACTIONS(1406),
    [anon_sym_DOLLAR] = ACTIONS(1406),
    [anon_sym_PLUS] = ACTIONS(1406),
    [anon_sym_STAR] = ACTIONS(1406),
    [anon_sym_QMARK] = ACTIONS(1408),
    [anon_sym_u8] = ACTIONS(1406),
    [anon_sym_i8] = ACTIONS(1406),
    [anon_sym_u16] = ACTIONS(1406),
    [anon_sym_i16] = ACTIONS(1406),
    [anon_sym_u32] = ACTIONS(1406),
    [anon_sym_i32] = ACTIONS(1406),
    [anon_sym_u64] = ACTIONS(1406),
    [anon_sym_i64] = ACTIONS(1406),
    [anon_sym_u128] = ACTIONS(1406),
    [anon_sym_i128] = ACTIONS(1406),
    [anon_sym_isize] = ACTIONS(1406),
    [anon_sym_usize] = ACTIONS(1406),
    [anon_sym_f32] = ACTIONS(1406),
    [anon_sym_f64] = ACTIONS(1406),
    [anon_sym_bool] = ACTIONS(1406),
    [anon_sym_str] = ACTIONS(1406),
    [anon_sym_char] = ACTIONS(1406),
    [anon_sym_DASH] = ACTIONS(1406),
    [anon_sym_SLASH] = ACTIONS(1406),
    [anon_sym_PERCENT] = ACTIONS(1406),
    [anon_sym_CARET] = ACTIONS(1406),
    [anon_sym_AMP] = ACTIONS(1406),
    [anon_sym_PIPE] = ACTIONS(1406),
    [anon_sym_AMP_AMP] = ACTIONS(1408),
    [anon_sym_PIPE_PIPE] = ACTIONS(1408),
    [anon_sym_LT_LT] = ACTIONS(1406),
    [anon_sym_GT_GT] = ACTIONS(1406),
    [anon_sym_PLUS_EQ] = ACTIONS(1408),
    [anon_sym_DASH_EQ] = ACTIONS(1408),
    [anon_sym_STAR_EQ] = ACTIONS(1408),
    [anon_sym_SLASH_EQ] = ACTIONS(1408),
    [anon_sym_PERCENT_EQ] = ACTIONS(1408),
    [anon_sym_CARET_EQ] = ACTIONS(1408),
    [anon_sym_AMP_EQ] = ACTIONS(1408),
    [anon_sym_PIPE_EQ] = ACTIONS(1408),
    [anon_sym_LT_LT_EQ] = ACTIONS(1408),
    [anon_sym_GT_GT_EQ] = ACTIONS(1408),
    [anon_sym_EQ] = ACTIONS(1406),
    [anon_sym_EQ_EQ] = ACTIONS(1408),
    [anon_sym_BANG_EQ] = ACTIONS(1408),
    [anon_sym_GT] = ACTIONS(1406),
    [anon_sym_LT] = ACTIONS(1406),
    [anon_sym_GT_EQ] = ACTIONS(1408),
    [anon_sym_LT_EQ] = ACTIONS(1408),
    [anon_sym_AT] = ACTIONS(1408),
    [anon_sym__] = ACTIONS(1406),
    [anon_sym_DOT] = ACTIONS(1406),
    [anon_sym_DOT_DOT] = ACTIONS(1406),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1408),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1408),
    [anon_sym_COLON_COLON] = ACTIONS(1408),
    [anon_sym_DASH_GT] = ACTIONS(1408),
    [anon_sym_POUND] = ACTIONS(1408),
    [anon_sym_SQUOTE] = ACTIONS(1406),
    [anon_sym_as] = ACTIONS(1406),
    [anon_sym_async] = ACTIONS(1406),
    [anon_sym_auto] = ACTIONS(1406),
    [anon_sym_await] = ACTIONS(1406),
    [anon_sym_become] = ACTIONS(1406),
    [anon_sym_break] = ACTIONS(1406),
    [anon_sym_const] = ACTIONS(1406),
    [anon_sym_continue] = ACTIONS(1406),
    [anon_sym_default] = ACTIONS(1406),
    [anon_sym_enum] = ACTIONS(1406),
    [anon_sym_fn] = ACTIONS(1406),
    [anon_sym_for] = ACTIONS(1406),
    [anon_sym_gen] = ACTIONS(1406),
    [anon_sym_if] = ACTIONS(1406),
    [anon_sym_impl] = ACTIONS(1406),
    [anon_sym_let] = ACTIONS(1406),
    [anon_sym_loop] = ACTIONS(1406),
    [anon_sym_match] = ACTIONS(1406),
    [anon_sym_mod] = ACTIONS(1406),
    [anon_sym_pub] = ACTIONS(1406),
    [anon_sym_return] = ACTIONS(1406),
    [anon_sym_raw] = ACTIONS(1406),
    [anon_sym_static] = ACTIONS(1406),
    [anon_sym_struct] = ACTIONS(1406),
    [anon_sym_trait] = ACTIONS(1406),
    [anon_sym_type] = ACTIONS(1406),
    [anon_sym_union] = ACTIONS(1406),
    [anon_sym_unsafe] = ACTIONS(1406),
    [anon_sym_use] = ACTIONS(1406),
    [anon_sym_where] = ACTIONS(1406),
    [anon_sym_while] = ACTIONS(1406),
    [sym_mutable_specifier] = ACTIONS(1406),
    [sym_integer_literal] = ACTIONS(1408),
    [aux_sym_string_literal_token1] = ACTIONS(1408),
    [sym_char_literal] = ACTIONS(1408),
    [anon_sym_true] = ACTIONS(1406),
    [anon_sym_false] = ACTIONS(1406),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1406),
    [sym_super] = ACTIONS(1406),
    [sym_crate] = ACTIONS(1406),
    [aux_sym_metavariable_token1] = ACTIONS(1408),
    [sym__raw_string_literal_start] = ACTIONS(1408),
    [sym_float_literal] = ACTIONS(1408),
  },
  [STATE(429)] = {
    [sym_line_comment] = STATE(429),
    [sym_block_comment] = STATE(429),
    [ts_builtin_sym_end] = ACTIONS(1410),
    [sym_identifier] = ACTIONS(1412),
    [anon_sym_SEMI] = ACTIONS(1410),
    [anon_sym_macro] = ACTIONS(1412),
    [anon_sym_LPAREN] = ACTIONS(1410),
    [anon_sym_LBRACE] = ACTIONS(1410),
    [anon_sym_RBRACE] = ACTIONS(1410),
    [anon_sym_macro_rules] = ACTIONS(1412),
    [anon_sym_BANG] = ACTIONS(1412),
    [anon_sym_LBRACK] = ACTIONS(1410),
    [anon_sym_PLUS] = ACTIONS(1412),
    [anon_sym_STAR] = ACTIONS(1412),
    [anon_sym_QMARK] = ACTIONS(1410),
    [anon_sym_u8] = ACTIONS(1412),
    [anon_sym_i8] = ACTIONS(1412),
    [anon_sym_u16] = ACTIONS(1412),
    [anon_sym_i16] = ACTIONS(1412),
    [anon_sym_u32] = ACTIONS(1412),
    [anon_sym_i32] = ACTIONS(1412),
    [anon_sym_u64] = ACTIONS(1412),
    [anon_sym_i64] = ACTIONS(1412),
    [anon_sym_u128] = ACTIONS(1412),
    [anon_sym_i128] = ACTIONS(1412),
    [anon_sym_isize] = ACTIONS(1412),
    [anon_sym_usize] = ACTIONS(1412),
    [anon_sym_f32] = ACTIONS(1412),
    [anon_sym_f64] = ACTIONS(1412),
    [anon_sym_bool] = ACTIONS(1412),
    [anon_sym_str] = ACTIONS(1412),
    [anon_sym_char] = ACTIONS(1412),
    [anon_sym_DASH] = ACTIONS(1412),
    [anon_sym_SLASH] = ACTIONS(1412),
    [anon_sym_PERCENT] = ACTIONS(1412),
    [anon_sym_CARET] = ACTIONS(1412),
    [anon_sym_AMP] = ACTIONS(1412),
    [anon_sym_PIPE] = ACTIONS(1412),
    [anon_sym_AMP_AMP] = ACTIONS(1410),
    [anon_sym_PIPE_PIPE] = ACTIONS(1410),
    [anon_sym_LT_LT] = ACTIONS(1412),
    [anon_sym_GT_GT] = ACTIONS(1412),
    [anon_sym_PLUS_EQ] = ACTIONS(1410),
    [anon_sym_DASH_EQ] = ACTIONS(1410),
    [anon_sym_STAR_EQ] = ACTIONS(1410),
    [anon_sym_SLASH_EQ] = ACTIONS(1410),
    [anon_sym_PERCENT_EQ] = ACTIONS(1410),
    [anon_sym_CARET_EQ] = ACTIONS(1410),
    [anon_sym_AMP_EQ] = ACTIONS(1410),
    [anon_sym_PIPE_EQ] = ACTIONS(1410),
    [anon_sym_LT_LT_EQ] = ACTIONS(1410),
    [anon_sym_GT_GT_EQ] = ACTIONS(1410),
    [anon_sym_EQ] = ACTIONS(1412),
    [anon_sym_EQ_EQ] = ACTIONS(1410),
    [anon_sym_BANG_EQ] = ACTIONS(1410),
    [anon_sym_GT] = ACTIONS(1412),
    [anon_sym_LT] = ACTIONS(1412),
    [anon_sym_GT_EQ] = ACTIONS(1410),
    [anon_sym_LT_EQ] = ACTIONS(1410),
    [anon_sym_DOT] = ACTIONS(1412),
    [anon_sym_DOT_DOT] = ACTIONS(1412),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1410),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1410),
    [anon_sym_COLON_COLON] = ACTIONS(1410),
    [anon_sym_POUND] = ACTIONS(1410),
    [anon_sym_SQUOTE] = ACTIONS(1412),
    [anon_sym_as] = ACTIONS(1412),
    [anon_sym_async] = ACTIONS(1412),
    [anon_sym_auto] = ACTIONS(1412),
    [anon_sym_await] = ACTIONS(1412),
    [anon_sym_become] = ACTIONS(1412),
    [anon_sym_break] = ACTIONS(1412),
    [anon_sym_const] = ACTIONS(1412),
    [anon_sym_continue] = ACTIONS(1412),
    [anon_sym_default] = ACTIONS(1412),
    [anon_sym_enum] = ACTIONS(1412),
    [anon_sym_fn] = ACTIONS(1412),
    [anon_sym_for] = ACTIONS(1412),
    [anon_sym_gen] = ACTIONS(1412),
    [anon_sym_if] = ACTIONS(1412),
    [anon_sym_impl] = ACTIONS(1412),
    [anon_sym_let] = ACTIONS(1412),
    [anon_sym_loop] = ACTIONS(1412),
    [anon_sym_match] = ACTIONS(1412),
    [anon_sym_mod] = ACTIONS(1412),
    [anon_sym_pub] = ACTIONS(1412),
    [anon_sym_return] = ACTIONS(1412),
    [anon_sym_raw] = ACTIONS(1412),
    [anon_sym_static] = ACTIONS(1412),
    [anon_sym_struct] = ACTIONS(1412),
    [anon_sym_trait] = ACTIONS(1412),
    [anon_sym_type] = ACTIONS(1412),
    [anon_sym_union] = ACTIONS(1412),
    [anon_sym_unsafe] = ACTIONS(1412),
    [anon_sym_use] = ACTIONS(1412),
    [anon_sym_while] = ACTIONS(1412),
    [anon_sym_extern] = ACTIONS(1412),
    [anon_sym_safe] = ACTIONS(1412),
    [anon_sym_else] = ACTIONS(1412),
    [anon_sym_yield] = ACTIONS(1412),
    [anon_sym_move] = ACTIONS(1412),
    [anon_sym_try] = ACTIONS(1412),
    [sym_integer_literal] = ACTIONS(1410),
    [aux_sym_string_literal_token1] = ACTIONS(1410),
    [sym_char_literal] = ACTIONS(1410),
    [anon_sym_true] = ACTIONS(1412),
    [anon_sym_false] = ACTIONS(1412),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1412),
    [sym_super] = ACTIONS(1412),
    [sym_crate] = ACTIONS(1412),
    [aux_sym_metavariable_token1] = ACTIONS(1410),
    [sym__raw_string_literal_start] = ACTIONS(1410),
    [sym_float_literal] = ACTIONS(1410),
    [sym__outer_block_doc_comment_start] = ACTIONS(1410),
    [sym__outer_line_doc_comment_start] = ACTIONS(1410),
  },
  [STATE(430)] = {
    [sym_line_comment] = STATE(430),
    [sym_block_comment] = STATE(430),
    [sym_identifier] = ACTIONS(1414),
    [anon_sym_SEMI] = ACTIONS(1416),
    [anon_sym_LPAREN] = ACTIONS(1416),
    [anon_sym_RPAREN] = ACTIONS(1416),
    [anon_sym_LBRACE] = ACTIONS(1416),
    [anon_sym_RBRACE] = ACTIONS(1416),
    [anon_sym_COMMA] = ACTIONS(1416),
    [anon_sym_EQ_GT] = ACTIONS(1416),
    [anon_sym_BANG] = ACTIONS(1414),
    [anon_sym_LBRACK] = ACTIONS(1416),
    [anon_sym_RBRACK] = ACTIONS(1416),
    [anon_sym_COLON] = ACTIONS(1414),
    [anon_sym_DOLLAR] = ACTIONS(1414),
    [anon_sym_PLUS] = ACTIONS(1414),
    [anon_sym_STAR] = ACTIONS(1414),
    [anon_sym_QMARK] = ACTIONS(1416),
    [anon_sym_u8] = ACTIONS(1414),
    [anon_sym_i8] = ACTIONS(1414),
    [anon_sym_u16] = ACTIONS(1414),
    [anon_sym_i16] = ACTIONS(1414),
    [anon_sym_u32] = ACTIONS(1414),
    [anon_sym_i32] = ACTIONS(1414),
    [anon_sym_u64] = ACTIONS(1414),
    [anon_sym_i64] = ACTIONS(1414),
    [anon_sym_u128] = ACTIONS(1414),
    [anon_sym_i128] = ACTIONS(1414),
    [anon_sym_isize] = ACTIONS(1414),
    [anon_sym_usize] = ACTIONS(1414),
    [anon_sym_f32] = ACTIONS(1414),
    [anon_sym_f64] = ACTIONS(1414),
    [anon_sym_bool] = ACTIONS(1414),
    [anon_sym_str] = ACTIONS(1414),
    [anon_sym_char] = ACTIONS(1414),
    [anon_sym_DASH] = ACTIONS(1414),
    [anon_sym_SLASH] = ACTIONS(1414),
    [anon_sym_PERCENT] = ACTIONS(1414),
    [anon_sym_CARET] = ACTIONS(1414),
    [anon_sym_AMP] = ACTIONS(1414),
    [anon_sym_PIPE] = ACTIONS(1414),
    [anon_sym_AMP_AMP] = ACTIONS(1416),
    [anon_sym_PIPE_PIPE] = ACTIONS(1416),
    [anon_sym_LT_LT] = ACTIONS(1414),
    [anon_sym_GT_GT] = ACTIONS(1414),
    [anon_sym_PLUS_EQ] = ACTIONS(1416),
    [anon_sym_DASH_EQ] = ACTIONS(1416),
    [anon_sym_STAR_EQ] = ACTIONS(1416),
    [anon_sym_SLASH_EQ] = ACTIONS(1416),
    [anon_sym_PERCENT_EQ] = ACTIONS(1416),
    [anon_sym_CARET_EQ] = ACTIONS(1416),
    [anon_sym_AMP_EQ] = ACTIONS(1416),
    [anon_sym_PIPE_EQ] = ACTIONS(1416),
    [anon_sym_LT_LT_EQ] = ACTIONS(1416),
    [anon_sym_GT_GT_EQ] = ACTIONS(1416),
    [anon_sym_EQ] = ACTIONS(1414),
    [anon_sym_EQ_EQ] = ACTIONS(1416),
    [anon_sym_BANG_EQ] = ACTIONS(1416),
    [anon_sym_GT] = ACTIONS(1414),
    [anon_sym_LT] = ACTIONS(1414),
    [anon_sym_GT_EQ] = ACTIONS(1416),
    [anon_sym_LT_EQ] = ACTIONS(1416),
    [anon_sym_AT] = ACTIONS(1416),
    [anon_sym__] = ACTIONS(1414),
    [anon_sym_DOT] = ACTIONS(1414),
    [anon_sym_DOT_DOT] = ACTIONS(1414),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1416),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1416),
    [anon_sym_COLON_COLON] = ACTIONS(1416),
    [anon_sym_DASH_GT] = ACTIONS(1416),
    [anon_sym_POUND] = ACTIONS(1416),
    [anon_sym_SQUOTE] = ACTIONS(1414),
    [anon_sym_as] = ACTIONS(1414),
    [anon_sym_async] = ACTIONS(1414),
    [anon_sym_auto] = ACTIONS(1414),
    [anon_sym_await] = ACTIONS(1414),
    [anon_sym_become] = ACTIONS(1414),
    [anon_sym_break] = ACTIONS(1414),
    [anon_sym_const] = ACTIONS(1414),
    [anon_sym_continue] = ACTIONS(1414),
    [anon_sym_default] = ACTIONS(1414),
    [anon_sym_enum] = ACTIONS(1414),
    [anon_sym_fn] = ACTIONS(1414),
    [anon_sym_for] = ACTIONS(1414),
    [anon_sym_gen] = ACTIONS(1414),
    [anon_sym_if] = ACTIONS(1414),
    [anon_sym_impl] = ACTIONS(1414),
    [anon_sym_let] = ACTIONS(1414),
    [anon_sym_loop] = ACTIONS(1414),
    [anon_sym_match] = ACTIONS(1414),
    [anon_sym_mod] = ACTIONS(1414),
    [anon_sym_pub] = ACTIONS(1414),
    [anon_sym_return] = ACTIONS(1414),
    [anon_sym_raw] = ACTIONS(1414),
    [anon_sym_static] = ACTIONS(1414),
    [anon_sym_struct] = ACTIONS(1414),
    [anon_sym_trait] = ACTIONS(1414),
    [anon_sym_type] = ACTIONS(1414),
    [anon_sym_union] = ACTIONS(1414),
    [anon_sym_unsafe] = ACTIONS(1414),
    [anon_sym_use] = ACTIONS(1414),
    [anon_sym_where] = ACTIONS(1414),
    [anon_sym_while] = ACTIONS(1414),
    [sym_mutable_specifier] = ACTIONS(1414),
    [sym_integer_literal] = ACTIONS(1416),
    [aux_sym_string_literal_token1] = ACTIONS(1416),
    [sym_char_literal] = ACTIONS(1416),
    [anon_sym_true] = ACTIONS(1414),
    [anon_sym_false] = ACTIONS(1414),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1414),
    [sym_super] = ACTIONS(1414),
    [sym_crate] = ACTIONS(1414),
    [aux_sym_metavariable_token1] = ACTIONS(1416),
    [sym__raw_string_literal_start] = ACTIONS(1416),
    [sym_float_literal] = ACTIONS(1416),
  },
  [STATE(431)] = {
    [sym_line_comment] = STATE(431),
    [sym_block_comment] = STATE(431),
    [sym_identifier] = ACTIONS(1418),
    [anon_sym_SEMI] = ACTIONS(1420),
    [anon_sym_LPAREN] = ACTIONS(1420),
    [anon_sym_RPAREN] = ACTIONS(1420),
    [anon_sym_LBRACE] = ACTIONS(1420),
    [anon_sym_RBRACE] = ACTIONS(1420),
    [anon_sym_COMMA] = ACTIONS(1420),
    [anon_sym_EQ_GT] = ACTIONS(1420),
    [anon_sym_BANG] = ACTIONS(1418),
    [anon_sym_LBRACK] = ACTIONS(1420),
    [anon_sym_RBRACK] = ACTIONS(1420),
    [anon_sym_COLON] = ACTIONS(1418),
    [anon_sym_DOLLAR] = ACTIONS(1418),
    [anon_sym_PLUS] = ACTIONS(1418),
    [anon_sym_STAR] = ACTIONS(1418),
    [anon_sym_QMARK] = ACTIONS(1420),
    [anon_sym_u8] = ACTIONS(1418),
    [anon_sym_i8] = ACTIONS(1418),
    [anon_sym_u16] = ACTIONS(1418),
    [anon_sym_i16] = ACTIONS(1418),
    [anon_sym_u32] = ACTIONS(1418),
    [anon_sym_i32] = ACTIONS(1418),
    [anon_sym_u64] = ACTIONS(1418),
    [anon_sym_i64] = ACTIONS(1418),
    [anon_sym_u128] = ACTIONS(1418),
    [anon_sym_i128] = ACTIONS(1418),
    [anon_sym_isize] = ACTIONS(1418),
    [anon_sym_usize] = ACTIONS(1418),
    [anon_sym_f32] = ACTIONS(1418),
    [anon_sym_f64] = ACTIONS(1418),
    [anon_sym_bool] = ACTIONS(1418),
    [anon_sym_str] = ACTIONS(1418),
    [anon_sym_char] = ACTIONS(1418),
    [anon_sym_DASH] = ACTIONS(1418),
    [anon_sym_SLASH] = ACTIONS(1418),
    [anon_sym_PERCENT] = ACTIONS(1418),
    [anon_sym_CARET] = ACTIONS(1418),
    [anon_sym_AMP] = ACTIONS(1418),
    [anon_sym_PIPE] = ACTIONS(1418),
    [anon_sym_AMP_AMP] = ACTIONS(1420),
    [anon_sym_PIPE_PIPE] = ACTIONS(1420),
    [anon_sym_LT_LT] = ACTIONS(1418),
    [anon_sym_GT_GT] = ACTIONS(1418),
    [anon_sym_PLUS_EQ] = ACTIONS(1420),
    [anon_sym_DASH_EQ] = ACTIONS(1420),
    [anon_sym_STAR_EQ] = ACTIONS(1420),
    [anon_sym_SLASH_EQ] = ACTIONS(1420),
    [anon_sym_PERCENT_EQ] = ACTIONS(1420),
    [anon_sym_CARET_EQ] = ACTIONS(1420),
    [anon_sym_AMP_EQ] = ACTIONS(1420),
    [anon_sym_PIPE_EQ] = ACTIONS(1420),
    [anon_sym_LT_LT_EQ] = ACTIONS(1420),
    [anon_sym_GT_GT_EQ] = ACTIONS(1420),
    [anon_sym_EQ] = ACTIONS(1418),
    [anon_sym_EQ_EQ] = ACTIONS(1420),
    [anon_sym_BANG_EQ] = ACTIONS(1420),
    [anon_sym_GT] = ACTIONS(1418),
    [anon_sym_LT] = ACTIONS(1418),
    [anon_sym_GT_EQ] = ACTIONS(1420),
    [anon_sym_LT_EQ] = ACTIONS(1420),
    [anon_sym_AT] = ACTIONS(1420),
    [anon_sym__] = ACTIONS(1418),
    [anon_sym_DOT] = ACTIONS(1418),
    [anon_sym_DOT_DOT] = ACTIONS(1418),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1420),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1420),
    [anon_sym_COLON_COLON] = ACTIONS(1420),
    [anon_sym_DASH_GT] = ACTIONS(1420),
    [anon_sym_POUND] = ACTIONS(1420),
    [anon_sym_SQUOTE] = ACTIONS(1418),
    [anon_sym_as] = ACTIONS(1418),
    [anon_sym_async] = ACTIONS(1418),
    [anon_sym_auto] = ACTIONS(1418),
    [anon_sym_await] = ACTIONS(1418),
    [anon_sym_become] = ACTIONS(1418),
    [anon_sym_break] = ACTIONS(1418),
    [anon_sym_const] = ACTIONS(1418),
    [anon_sym_continue] = ACTIONS(1418),
    [anon_sym_default] = ACTIONS(1418),
    [anon_sym_enum] = ACTIONS(1418),
    [anon_sym_fn] = ACTIONS(1418),
    [anon_sym_for] = ACTIONS(1418),
    [anon_sym_gen] = ACTIONS(1418),
    [anon_sym_if] = ACTIONS(1418),
    [anon_sym_impl] = ACTIONS(1418),
    [anon_sym_let] = ACTIONS(1418),
    [anon_sym_loop] = ACTIONS(1418),
    [anon_sym_match] = ACTIONS(1418),
    [anon_sym_mod] = ACTIONS(1418),
    [anon_sym_pub] = ACTIONS(1418),
    [anon_sym_return] = ACTIONS(1418),
    [anon_sym_raw] = ACTIONS(1418),
    [anon_sym_static] = ACTIONS(1418),
    [anon_sym_struct] = ACTIONS(1418),
    [anon_sym_trait] = ACTIONS(1418),
    [anon_sym_type] = ACTIONS(1418),
    [anon_sym_union] = ACTIONS(1418),
    [anon_sym_unsafe] = ACTIONS(1418),
    [anon_sym_use] = ACTIONS(1418),
    [anon_sym_where] = ACTIONS(1418),
    [anon_sym_while] = ACTIONS(1418),
    [sym_mutable_specifier] = ACTIONS(1418),
    [sym_integer_literal] = ACTIONS(1420),
    [aux_sym_string_literal_token1] = ACTIONS(1420),
    [sym_char_literal] = ACTIONS(1420),
    [anon_sym_true] = ACTIONS(1418),
    [anon_sym_false] = ACTIONS(1418),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1418),
    [sym_super] = ACTIONS(1418),
    [sym_crate] = ACTIONS(1418),
    [aux_sym_metavariable_token1] = ACTIONS(1420),
    [sym__raw_string_literal_start] = ACTIONS(1420),
    [sym_float_literal] = ACTIONS(1420),
  },
  [STATE(432)] = {
    [sym_line_comment] = STATE(432),
    [sym_block_comment] = STATE(432),
    [sym_identifier] = ACTIONS(1422),
    [anon_sym_SEMI] = ACTIONS(1424),
    [anon_sym_LPAREN] = ACTIONS(1424),
    [anon_sym_RPAREN] = ACTIONS(1424),
    [anon_sym_LBRACE] = ACTIONS(1424),
    [anon_sym_RBRACE] = ACTIONS(1424),
    [anon_sym_COMMA] = ACTIONS(1424),
    [anon_sym_EQ_GT] = ACTIONS(1424),
    [anon_sym_BANG] = ACTIONS(1422),
    [anon_sym_LBRACK] = ACTIONS(1424),
    [anon_sym_RBRACK] = ACTIONS(1424),
    [anon_sym_COLON] = ACTIONS(1422),
    [anon_sym_DOLLAR] = ACTIONS(1422),
    [anon_sym_PLUS] = ACTIONS(1422),
    [anon_sym_STAR] = ACTIONS(1422),
    [anon_sym_QMARK] = ACTIONS(1424),
    [anon_sym_u8] = ACTIONS(1422),
    [anon_sym_i8] = ACTIONS(1422),
    [anon_sym_u16] = ACTIONS(1422),
    [anon_sym_i16] = ACTIONS(1422),
    [anon_sym_u32] = ACTIONS(1422),
    [anon_sym_i32] = ACTIONS(1422),
    [anon_sym_u64] = ACTIONS(1422),
    [anon_sym_i64] = ACTIONS(1422),
    [anon_sym_u128] = ACTIONS(1422),
    [anon_sym_i128] = ACTIONS(1422),
    [anon_sym_isize] = ACTIONS(1422),
    [anon_sym_usize] = ACTIONS(1422),
    [anon_sym_f32] = ACTIONS(1422),
    [anon_sym_f64] = ACTIONS(1422),
    [anon_sym_bool] = ACTIONS(1422),
    [anon_sym_str] = ACTIONS(1422),
    [anon_sym_char] = ACTIONS(1422),
    [anon_sym_DASH] = ACTIONS(1422),
    [anon_sym_SLASH] = ACTIONS(1422),
    [anon_sym_PERCENT] = ACTIONS(1422),
    [anon_sym_CARET] = ACTIONS(1422),
    [anon_sym_AMP] = ACTIONS(1422),
    [anon_sym_PIPE] = ACTIONS(1422),
    [anon_sym_AMP_AMP] = ACTIONS(1424),
    [anon_sym_PIPE_PIPE] = ACTIONS(1424),
    [anon_sym_LT_LT] = ACTIONS(1422),
    [anon_sym_GT_GT] = ACTIONS(1422),
    [anon_sym_PLUS_EQ] = ACTIONS(1424),
    [anon_sym_DASH_EQ] = ACTIONS(1424),
    [anon_sym_STAR_EQ] = ACTIONS(1424),
    [anon_sym_SLASH_EQ] = ACTIONS(1424),
    [anon_sym_PERCENT_EQ] = ACTIONS(1424),
    [anon_sym_CARET_EQ] = ACTIONS(1424),
    [anon_sym_AMP_EQ] = ACTIONS(1424),
    [anon_sym_PIPE_EQ] = ACTIONS(1424),
    [anon_sym_LT_LT_EQ] = ACTIONS(1424),
    [anon_sym_GT_GT_EQ] = ACTIONS(1424),
    [anon_sym_EQ] = ACTIONS(1422),
    [anon_sym_EQ_EQ] = ACTIONS(1424),
    [anon_sym_BANG_EQ] = ACTIONS(1424),
    [anon_sym_GT] = ACTIONS(1422),
    [anon_sym_LT] = ACTIONS(1422),
    [anon_sym_GT_EQ] = ACTIONS(1424),
    [anon_sym_LT_EQ] = ACTIONS(1424),
    [anon_sym_AT] = ACTIONS(1424),
    [anon_sym__] = ACTIONS(1422),
    [anon_sym_DOT] = ACTIONS(1422),
    [anon_sym_DOT_DOT] = ACTIONS(1422),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1424),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1424),
    [anon_sym_COLON_COLON] = ACTIONS(1424),
    [anon_sym_DASH_GT] = ACTIONS(1424),
    [anon_sym_POUND] = ACTIONS(1424),
    [anon_sym_SQUOTE] = ACTIONS(1422),
    [anon_sym_as] = ACTIONS(1422),
    [anon_sym_async] = ACTIONS(1422),
    [anon_sym_auto] = ACTIONS(1422),
    [anon_sym_await] = ACTIONS(1422),
    [anon_sym_become] = ACTIONS(1422),
    [anon_sym_break] = ACTIONS(1422),
    [anon_sym_const] = ACTIONS(1422),
    [anon_sym_continue] = ACTIONS(1422),
    [anon_sym_default] = ACTIONS(1422),
    [anon_sym_enum] = ACTIONS(1422),
    [anon_sym_fn] = ACTIONS(1422),
    [anon_sym_for] = ACTIONS(1422),
    [anon_sym_gen] = ACTIONS(1422),
    [anon_sym_if] = ACTIONS(1422),
    [anon_sym_impl] = ACTIONS(1422),
    [anon_sym_let] = ACTIONS(1422),
    [anon_sym_loop] = ACTIONS(1422),
    [anon_sym_match] = ACTIONS(1422),
    [anon_sym_mod] = ACTIONS(1422),
    [anon_sym_pub] = ACTIONS(1422),
    [anon_sym_return] = ACTIONS(1422),
    [anon_sym_raw] = ACTIONS(1422),
    [anon_sym_static] = ACTIONS(1422),
    [anon_sym_struct] = ACTIONS(1422),
    [anon_sym_trait] = ACTIONS(1422),
    [anon_sym_type] = ACTIONS(1422),
    [anon_sym_union] = ACTIONS(1422),
    [anon_sym_unsafe] = ACTIONS(1422),
    [anon_sym_use] = ACTIONS(1422),
    [anon_sym_where] = ACTIONS(1422),
    [anon_sym_while] = ACTIONS(1422),
    [sym_mutable_specifier] = ACTIONS(1422),
    [sym_integer_literal] = ACTIONS(1424),
    [aux_sym_string_literal_token1] = ACTIONS(1424),
    [sym_char_literal] = ACTIONS(1424),
    [anon_sym_true] = ACTIONS(1422),
    [anon_sym_false] = ACTIONS(1422),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1422),
    [sym_super] = ACTIONS(1422),
    [sym_crate] = ACTIONS(1422),
    [aux_sym_metavariable_token1] = ACTIONS(1424),
    [sym__raw_string_literal_start] = ACTIONS(1424),
    [sym_float_literal] = ACTIONS(1424),
  },
  [STATE(433)] = {
    [sym_line_comment] = STATE(433),
    [sym_block_comment] = STATE(433),
    [sym_identifier] = ACTIONS(1426),
    [anon_sym_SEMI] = ACTIONS(1428),
    [anon_sym_LPAREN] = ACTIONS(1428),
    [anon_sym_RPAREN] = ACTIONS(1428),
    [anon_sym_LBRACE] = ACTIONS(1428),
    [anon_sym_RBRACE] = ACTIONS(1428),
    [anon_sym_COMMA] = ACTIONS(1428),
    [anon_sym_EQ_GT] = ACTIONS(1428),
    [anon_sym_BANG] = ACTIONS(1426),
    [anon_sym_LBRACK] = ACTIONS(1428),
    [anon_sym_RBRACK] = ACTIONS(1428),
    [anon_sym_COLON] = ACTIONS(1426),
    [anon_sym_DOLLAR] = ACTIONS(1426),
    [anon_sym_PLUS] = ACTIONS(1426),
    [anon_sym_STAR] = ACTIONS(1426),
    [anon_sym_QMARK] = ACTIONS(1428),
    [anon_sym_u8] = ACTIONS(1426),
    [anon_sym_i8] = ACTIONS(1426),
    [anon_sym_u16] = ACTIONS(1426),
    [anon_sym_i16] = ACTIONS(1426),
    [anon_sym_u32] = ACTIONS(1426),
    [anon_sym_i32] = ACTIONS(1426),
    [anon_sym_u64] = ACTIONS(1426),
    [anon_sym_i64] = ACTIONS(1426),
    [anon_sym_u128] = ACTIONS(1426),
    [anon_sym_i128] = ACTIONS(1426),
    [anon_sym_isize] = ACTIONS(1426),
    [anon_sym_usize] = ACTIONS(1426),
    [anon_sym_f32] = ACTIONS(1426),
    [anon_sym_f64] = ACTIONS(1426),
    [anon_sym_bool] = ACTIONS(1426),
    [anon_sym_str] = ACTIONS(1426),
    [anon_sym_char] = ACTIONS(1426),
    [anon_sym_DASH] = ACTIONS(1426),
    [anon_sym_SLASH] = ACTIONS(1426),
    [anon_sym_PERCENT] = ACTIONS(1426),
    [anon_sym_CARET] = ACTIONS(1426),
    [anon_sym_AMP] = ACTIONS(1426),
    [anon_sym_PIPE] = ACTIONS(1426),
    [anon_sym_AMP_AMP] = ACTIONS(1428),
    [anon_sym_PIPE_PIPE] = ACTIONS(1428),
    [anon_sym_LT_LT] = ACTIONS(1426),
    [anon_sym_GT_GT] = ACTIONS(1426),
    [anon_sym_PLUS_EQ] = ACTIONS(1428),
    [anon_sym_DASH_EQ] = ACTIONS(1428),
    [anon_sym_STAR_EQ] = ACTIONS(1428),
    [anon_sym_SLASH_EQ] = ACTIONS(1428),
    [anon_sym_PERCENT_EQ] = ACTIONS(1428),
    [anon_sym_CARET_EQ] = ACTIONS(1428),
    [anon_sym_AMP_EQ] = ACTIONS(1428),
    [anon_sym_PIPE_EQ] = ACTIONS(1428),
    [anon_sym_LT_LT_EQ] = ACTIONS(1428),
    [anon_sym_GT_GT_EQ] = ACTIONS(1428),
    [anon_sym_EQ] = ACTIONS(1426),
    [anon_sym_EQ_EQ] = ACTIONS(1428),
    [anon_sym_BANG_EQ] = ACTIONS(1428),
    [anon_sym_GT] = ACTIONS(1426),
    [anon_sym_LT] = ACTIONS(1426),
    [anon_sym_GT_EQ] = ACTIONS(1428),
    [anon_sym_LT_EQ] = ACTIONS(1428),
    [anon_sym_AT] = ACTIONS(1428),
    [anon_sym__] = ACTIONS(1426),
    [anon_sym_DOT] = ACTIONS(1426),
    [anon_sym_DOT_DOT] = ACTIONS(1426),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1428),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1428),
    [anon_sym_COLON_COLON] = ACTIONS(1428),
    [anon_sym_DASH_GT] = ACTIONS(1428),
    [anon_sym_POUND] = ACTIONS(1428),
    [anon_sym_SQUOTE] = ACTIONS(1426),
    [anon_sym_as] = ACTIONS(1426),
    [anon_sym_async] = ACTIONS(1426),
    [anon_sym_auto] = ACTIONS(1426),
    [anon_sym_await] = ACTIONS(1426),
    [anon_sym_become] = ACTIONS(1426),
    [anon_sym_break] = ACTIONS(1426),
    [anon_sym_const] = ACTIONS(1426),
    [anon_sym_continue] = ACTIONS(1426),
    [anon_sym_default] = ACTIONS(1426),
    [anon_sym_enum] = ACTIONS(1426),
    [anon_sym_fn] = ACTIONS(1426),
    [anon_sym_for] = ACTIONS(1426),
    [anon_sym_gen] = ACTIONS(1426),
    [anon_sym_if] = ACTIONS(1426),
    [anon_sym_impl] = ACTIONS(1426),
    [anon_sym_let] = ACTIONS(1426),
    [anon_sym_loop] = ACTIONS(1426),
    [anon_sym_match] = ACTIONS(1426),
    [anon_sym_mod] = ACTIONS(1426),
    [anon_sym_pub] = ACTIONS(1426),
    [anon_sym_return] = ACTIONS(1426),
    [anon_sym_raw] = ACTIONS(1426),
    [anon_sym_static] = ACTIONS(1426),
    [anon_sym_struct] = ACTIONS(1426),
    [anon_sym_trait] = ACTIONS(1426),
    [anon_sym_type] = ACTIONS(1426),
    [anon_sym_union] = ACTIONS(1426),
    [anon_sym_unsafe] = ACTIONS(1426),
    [anon_sym_use] = ACTIONS(1426),
    [anon_sym_where] = ACTIONS(1426),
    [anon_sym_while] = ACTIONS(1426),
    [sym_mutable_specifier] = ACTIONS(1426),
    [sym_integer_literal] = ACTIONS(1428),
    [aux_sym_string_literal_token1] = ACTIONS(1428),
    [sym_char_literal] = ACTIONS(1428),
    [anon_sym_true] = ACTIONS(1426),
    [anon_sym_false] = ACTIONS(1426),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1426),
    [sym_super] = ACTIONS(1426),
    [sym_crate] = ACTIONS(1426),
    [aux_sym_metavariable_token1] = ACTIONS(1428),
    [sym__raw_string_literal_start] = ACTIONS(1428),
    [sym_float_literal] = ACTIONS(1428),
  },
  [STATE(434)] = {
    [sym_line_comment] = STATE(434),
    [sym_block_comment] = STATE(434),
    [sym_identifier] = ACTIONS(1430),
    [anon_sym_SEMI] = ACTIONS(1432),
    [anon_sym_LPAREN] = ACTIONS(1432),
    [anon_sym_RPAREN] = ACTIONS(1432),
    [anon_sym_LBRACE] = ACTIONS(1432),
    [anon_sym_RBRACE] = ACTIONS(1432),
    [anon_sym_COMMA] = ACTIONS(1432),
    [anon_sym_EQ_GT] = ACTIONS(1432),
    [anon_sym_BANG] = ACTIONS(1430),
    [anon_sym_LBRACK] = ACTIONS(1432),
    [anon_sym_RBRACK] = ACTIONS(1432),
    [anon_sym_COLON] = ACTIONS(1430),
    [anon_sym_DOLLAR] = ACTIONS(1430),
    [anon_sym_PLUS] = ACTIONS(1430),
    [anon_sym_STAR] = ACTIONS(1430),
    [anon_sym_QMARK] = ACTIONS(1432),
    [anon_sym_u8] = ACTIONS(1430),
    [anon_sym_i8] = ACTIONS(1430),
    [anon_sym_u16] = ACTIONS(1430),
    [anon_sym_i16] = ACTIONS(1430),
    [anon_sym_u32] = ACTIONS(1430),
    [anon_sym_i32] = ACTIONS(1430),
    [anon_sym_u64] = ACTIONS(1430),
    [anon_sym_i64] = ACTIONS(1430),
    [anon_sym_u128] = ACTIONS(1430),
    [anon_sym_i128] = ACTIONS(1430),
    [anon_sym_isize] = ACTIONS(1430),
    [anon_sym_usize] = ACTIONS(1430),
    [anon_sym_f32] = ACTIONS(1430),
    [anon_sym_f64] = ACTIONS(1430),
    [anon_sym_bool] = ACTIONS(1430),
    [anon_sym_str] = ACTIONS(1430),
    [anon_sym_char] = ACTIONS(1430),
    [anon_sym_DASH] = ACTIONS(1430),
    [anon_sym_SLASH] = ACTIONS(1430),
    [anon_sym_PERCENT] = ACTIONS(1430),
    [anon_sym_CARET] = ACTIONS(1430),
    [anon_sym_AMP] = ACTIONS(1430),
    [anon_sym_PIPE] = ACTIONS(1430),
    [anon_sym_AMP_AMP] = ACTIONS(1432),
    [anon_sym_PIPE_PIPE] = ACTIONS(1432),
    [anon_sym_LT_LT] = ACTIONS(1430),
    [anon_sym_GT_GT] = ACTIONS(1430),
    [anon_sym_PLUS_EQ] = ACTIONS(1432),
    [anon_sym_DASH_EQ] = ACTIONS(1432),
    [anon_sym_STAR_EQ] = ACTIONS(1432),
    [anon_sym_SLASH_EQ] = ACTIONS(1432),
    [anon_sym_PERCENT_EQ] = ACTIONS(1432),
    [anon_sym_CARET_EQ] = ACTIONS(1432),
    [anon_sym_AMP_EQ] = ACTIONS(1432),
    [anon_sym_PIPE_EQ] = ACTIONS(1432),
    [anon_sym_LT_LT_EQ] = ACTIONS(1432),
    [anon_sym_GT_GT_EQ] = ACTIONS(1432),
    [anon_sym_EQ] = ACTIONS(1430),
    [anon_sym_EQ_EQ] = ACTIONS(1432),
    [anon_sym_BANG_EQ] = ACTIONS(1432),
    [anon_sym_GT] = ACTIONS(1430),
    [anon_sym_LT] = ACTIONS(1430),
    [anon_sym_GT_EQ] = ACTIONS(1432),
    [anon_sym_LT_EQ] = ACTIONS(1432),
    [anon_sym_AT] = ACTIONS(1432),
    [anon_sym__] = ACTIONS(1430),
    [anon_sym_DOT] = ACTIONS(1430),
    [anon_sym_DOT_DOT] = ACTIONS(1430),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1432),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1432),
    [anon_sym_COLON_COLON] = ACTIONS(1432),
    [anon_sym_DASH_GT] = ACTIONS(1432),
    [anon_sym_POUND] = ACTIONS(1432),
    [anon_sym_SQUOTE] = ACTIONS(1430),
    [anon_sym_as] = ACTIONS(1430),
    [anon_sym_async] = ACTIONS(1430),
    [anon_sym_auto] = ACTIONS(1430),
    [anon_sym_await] = ACTIONS(1430),
    [anon_sym_become] = ACTIONS(1430),
    [anon_sym_break] = ACTIONS(1430),
    [anon_sym_const] = ACTIONS(1430),
    [anon_sym_continue] = ACTIONS(1430),
    [anon_sym_default] = ACTIONS(1430),
    [anon_sym_enum] = ACTIONS(1430),
    [anon_sym_fn] = ACTIONS(1430),
    [anon_sym_for] = ACTIONS(1430),
    [anon_sym_gen] = ACTIONS(1430),
    [anon_sym_if] = ACTIONS(1430),
    [anon_sym_impl] = ACTIONS(1430),
    [anon_sym_let] = ACTIONS(1430),
    [anon_sym_loop] = ACTIONS(1430),
    [anon_sym_match] = ACTIONS(1430),
    [anon_sym_mod] = ACTIONS(1430),
    [anon_sym_pub] = ACTIONS(1430),
    [anon_sym_return] = ACTIONS(1430),
    [anon_sym_raw] = ACTIONS(1430),
    [anon_sym_static] = ACTIONS(1430),
    [anon_sym_struct] = ACTIONS(1430),
    [anon_sym_trait] = ACTIONS(1430),
    [anon_sym_type] = ACTIONS(1430),
    [anon_sym_union] = ACTIONS(1430),
    [anon_sym_unsafe] = ACTIONS(1430),
    [anon_sym_use] = ACTIONS(1430),
    [anon_sym_where] = ACTIONS(1430),
    [anon_sym_while] = ACTIONS(1430),
    [sym_mutable_specifier] = ACTIONS(1430),
    [sym_integer_literal] = ACTIONS(1432),
    [aux_sym_string_literal_token1] = ACTIONS(1432),
    [sym_char_literal] = ACTIONS(1432),
    [anon_sym_true] = ACTIONS(1430),
    [anon_sym_false] = ACTIONS(1430),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1430),
    [sym_super] = ACTIONS(1430),
    [sym_crate] = ACTIONS(1430),
    [aux_sym_metavariable_token1] = ACTIONS(1432),
    [sym__raw_string_literal_start] = ACTIONS(1432),
    [sym_float_literal] = ACTIONS(1432),
  },
  [STATE(435)] = {
    [sym_line_comment] = STATE(435),
    [sym_block_comment] = STATE(435),
    [aux_sym__non_special_token_repeat1] = STATE(435),
    [sym_identifier] = ACTIONS(1336),
    [anon_sym_SEMI] = ACTIONS(1434),
    [anon_sym_LPAREN] = ACTIONS(1341),
    [anon_sym_RPAREN] = ACTIONS(1341),
    [anon_sym_LBRACE] = ACTIONS(1341),
    [anon_sym_RBRACE] = ACTIONS(1341),
    [anon_sym_COMMA] = ACTIONS(1434),
    [anon_sym_EQ_GT] = ACTIONS(1434),
    [anon_sym_BANG] = ACTIONS(1437),
    [anon_sym_LBRACK] = ACTIONS(1341),
    [anon_sym_RBRACK] = ACTIONS(1341),
    [anon_sym_COLON] = ACTIONS(1437),
    [anon_sym_DOLLAR] = ACTIONS(1341),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1437),
    [anon_sym_QMARK] = ACTIONS(1434),
    [anon_sym_u8] = ACTIONS(1336),
    [anon_sym_i8] = ACTIONS(1336),
    [anon_sym_u16] = ACTIONS(1336),
    [anon_sym_i16] = ACTIONS(1336),
    [anon_sym_u32] = ACTIONS(1336),
    [anon_sym_i32] = ACTIONS(1336),
    [anon_sym_u64] = ACTIONS(1336),
    [anon_sym_i64] = ACTIONS(1336),
    [anon_sym_u128] = ACTIONS(1336),
    [anon_sym_i128] = ACTIONS(1336),
    [anon_sym_isize] = ACTIONS(1336),
    [anon_sym_usize] = ACTIONS(1336),
    [anon_sym_f32] = ACTIONS(1336),
    [anon_sym_f64] = ACTIONS(1336),
    [anon_sym_bool] = ACTIONS(1336),
    [anon_sym_str] = ACTIONS(1336),
    [anon_sym_char] = ACTIONS(1336),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_SLASH] = ACTIONS(1437),
    [anon_sym_PERCENT] = ACTIONS(1437),
    [anon_sym_CARET] = ACTIONS(1437),
    [anon_sym_AMP] = ACTIONS(1437),
    [anon_sym_PIPE] = ACTIONS(1437),
    [anon_sym_AMP_AMP] = ACTIONS(1434),
    [anon_sym_PIPE_PIPE] = ACTIONS(1434),
    [anon_sym_LT_LT] = ACTIONS(1437),
    [anon_sym_GT_GT] = ACTIONS(1437),
    [anon_sym_PLUS_EQ] = ACTIONS(1434),
    [anon_sym_DASH_EQ] = ACTIONS(1434),
    [anon_sym_STAR_EQ] = ACTIONS(1434),
    [anon_sym_SLASH_EQ] = ACTIONS(1434),
    [anon_sym_PERCENT_EQ] = ACTIONS(1434),
    [anon_sym_CARET_EQ] = ACTIONS(1434),
    [anon_sym_AMP_EQ] = ACTIONS(1434),
    [anon_sym_PIPE_EQ] = ACTIONS(1434),
    [anon_sym_LT_LT_EQ] = ACTIONS(1434),
    [anon_sym_GT_GT_EQ] = ACTIONS(1434),
    [anon_sym_EQ] = ACTIONS(1437),
    [anon_sym_EQ_EQ] = ACTIONS(1434),
    [anon_sym_BANG_EQ] = ACTIONS(1434),
    [anon_sym_GT] = ACTIONS(1437),
    [anon_sym_LT] = ACTIONS(1437),
    [anon_sym_GT_EQ] = ACTIONS(1434),
    [anon_sym_LT_EQ] = ACTIONS(1434),
    [anon_sym_AT] = ACTIONS(1434),
    [anon_sym__] = ACTIONS(1437),
    [anon_sym_DOT] = ACTIONS(1437),
    [anon_sym_DOT_DOT] = ACTIONS(1437),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1434),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1434),
    [anon_sym_COLON_COLON] = ACTIONS(1434),
    [anon_sym_DASH_GT] = ACTIONS(1434),
    [anon_sym_POUND] = ACTIONS(1434),
    [anon_sym_SQUOTE] = ACTIONS(1336),
    [anon_sym_as] = ACTIONS(1336),
    [anon_sym_async] = ACTIONS(1336),
    [anon_sym_auto] = ACTIONS(1336),
    [anon_sym_await] = ACTIONS(1336),
    [anon_sym_become] = ACTIONS(1336),
    [anon_sym_break] = ACTIONS(1336),
    [anon_sym_const] = ACTIONS(1336),
    [anon_sym_continue] = ACTIONS(1336),
    [anon_sym_default] = ACTIONS(1336),
    [anon_sym_enum] = ACTIONS(1336),
    [anon_sym_fn] = ACTIONS(1336),
    [anon_sym_for] = ACTIONS(1336),
    [anon_sym_gen] = ACTIONS(1336),
    [anon_sym_if] = ACTIONS(1336),
    [anon_sym_impl] = ACTIONS(1336),
    [anon_sym_let] = ACTIONS(1336),
    [anon_sym_loop] = ACTIONS(1336),
    [anon_sym_match] = ACTIONS(1336),
    [anon_sym_mod] = ACTIONS(1336),
    [anon_sym_pub] = ACTIONS(1336),
    [anon_sym_return] = ACTIONS(1336),
    [anon_sym_raw] = ACTIONS(1336),
    [anon_sym_static] = ACTIONS(1336),
    [anon_sym_struct] = ACTIONS(1336),
    [anon_sym_trait] = ACTIONS(1336),
    [anon_sym_type] = ACTIONS(1336),
    [anon_sym_union] = ACTIONS(1336),
    [anon_sym_unsafe] = ACTIONS(1336),
    [anon_sym_use] = ACTIONS(1336),
    [anon_sym_where] = ACTIONS(1336),
    [anon_sym_while] = ACTIONS(1336),
    [sym_mutable_specifier] = ACTIONS(1336),
    [sym_integer_literal] = ACTIONS(1341),
    [aux_sym_string_literal_token1] = ACTIONS(1341),
    [sym_char_literal] = ACTIONS(1341),
    [anon_sym_true] = ACTIONS(1336),
    [anon_sym_false] = ACTIONS(1336),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1336),
    [sym_super] = ACTIONS(1336),
    [sym_crate] = ACTIONS(1336),
    [sym__raw_string_literal_start] = ACTIONS(1341),
    [sym_float_literal] = ACTIONS(1341),
  },
  [STATE(436)] = {
    [sym_line_comment] = STATE(436),
    [sym_block_comment] = STATE(436),
    [sym_identifier] = ACTIONS(1440),
    [anon_sym_SEMI] = ACTIONS(1442),
    [anon_sym_LPAREN] = ACTIONS(1442),
    [anon_sym_RPAREN] = ACTIONS(1442),
    [anon_sym_LBRACE] = ACTIONS(1442),
    [anon_sym_RBRACE] = ACTIONS(1442),
    [anon_sym_COMMA] = ACTIONS(1442),
    [anon_sym_EQ_GT] = ACTIONS(1442),
    [anon_sym_BANG] = ACTIONS(1440),
    [anon_sym_LBRACK] = ACTIONS(1442),
    [anon_sym_RBRACK] = ACTIONS(1442),
    [anon_sym_COLON] = ACTIONS(1440),
    [anon_sym_DOLLAR] = ACTIONS(1440),
    [anon_sym_PLUS] = ACTIONS(1440),
    [anon_sym_STAR] = ACTIONS(1440),
    [anon_sym_QMARK] = ACTIONS(1442),
    [anon_sym_u8] = ACTIONS(1440),
    [anon_sym_i8] = ACTIONS(1440),
    [anon_sym_u16] = ACTIONS(1440),
    [anon_sym_i16] = ACTIONS(1440),
    [anon_sym_u32] = ACTIONS(1440),
    [anon_sym_i32] = ACTIONS(1440),
    [anon_sym_u64] = ACTIONS(1440),
    [anon_sym_i64] = ACTIONS(1440),
    [anon_sym_u128] = ACTIONS(1440),
    [anon_sym_i128] = ACTIONS(1440),
    [anon_sym_isize] = ACTIONS(1440),
    [anon_sym_usize] = ACTIONS(1440),
    [anon_sym_f32] = ACTIONS(1440),
    [anon_sym_f64] = ACTIONS(1440),
    [anon_sym_bool] = ACTIONS(1440),
    [anon_sym_str] = ACTIONS(1440),
    [anon_sym_char] = ACTIONS(1440),
    [anon_sym_DASH] = ACTIONS(1440),
    [anon_sym_SLASH] = ACTIONS(1440),
    [anon_sym_PERCENT] = ACTIONS(1440),
    [anon_sym_CARET] = ACTIONS(1440),
    [anon_sym_AMP] = ACTIONS(1440),
    [anon_sym_PIPE] = ACTIONS(1440),
    [anon_sym_AMP_AMP] = ACTIONS(1442),
    [anon_sym_PIPE_PIPE] = ACTIONS(1442),
    [anon_sym_LT_LT] = ACTIONS(1440),
    [anon_sym_GT_GT] = ACTIONS(1440),
    [anon_sym_PLUS_EQ] = ACTIONS(1442),
    [anon_sym_DASH_EQ] = ACTIONS(1442),
    [anon_sym_STAR_EQ] = ACTIONS(1442),
    [anon_sym_SLASH_EQ] = ACTIONS(1442),
    [anon_sym_PERCENT_EQ] = ACTIONS(1442),
    [anon_sym_CARET_EQ] = ACTIONS(1442),
    [anon_sym_AMP_EQ] = ACTIONS(1442),
    [anon_sym_PIPE_EQ] = ACTIONS(1442),
    [anon_sym_LT_LT_EQ] = ACTIONS(1442),
    [anon_sym_GT_GT_EQ] = ACTIONS(1442),
    [anon_sym_EQ] = ACTIONS(1440),
    [anon_sym_EQ_EQ] = ACTIONS(1442),
    [anon_sym_BANG_EQ] = ACTIONS(1442),
    [anon_sym_GT] = ACTIONS(1440),
    [anon_sym_LT] = ACTIONS(1440),
    [anon_sym_GT_EQ] = ACTIONS(1442),
    [anon_sym_LT_EQ] = ACTIONS(1442),
    [anon_sym_AT] = ACTIONS(1442),
    [anon_sym__] = ACTIONS(1440),
    [anon_sym_DOT] = ACTIONS(1440),
    [anon_sym_DOT_DOT] = ACTIONS(1440),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1442),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1442),
    [anon_sym_COLON_COLON] = ACTIONS(1442),
    [anon_sym_DASH_GT] = ACTIONS(1442),
    [anon_sym_POUND] = ACTIONS(1442),
    [anon_sym_SQUOTE] = ACTIONS(1440),
    [anon_sym_as] = ACTIONS(1440),
    [anon_sym_async] = ACTIONS(1440),
    [anon_sym_auto] = ACTIONS(1440),
    [anon_sym_await] = ACTIONS(1440),
    [anon_sym_become] = ACTIONS(1440),
    [anon_sym_break] = ACTIONS(1440),
    [anon_sym_const] = ACTIONS(1440),
    [anon_sym_continue] = ACTIONS(1440),
    [anon_sym_default] = ACTIONS(1440),
    [anon_sym_enum] = ACTIONS(1440),
    [anon_sym_fn] = ACTIONS(1440),
    [anon_sym_for] = ACTIONS(1440),
    [anon_sym_gen] = ACTIONS(1440),
    [anon_sym_if] = ACTIONS(1440),
    [anon_sym_impl] = ACTIONS(1440),
    [anon_sym_let] = ACTIONS(1440),
    [anon_sym_loop] = ACTIONS(1440),
    [anon_sym_match] = ACTIONS(1440),
    [anon_sym_mod] = ACTIONS(1440),
    [anon_sym_pub] = ACTIONS(1440),
    [anon_sym_return] = ACTIONS(1440),
    [anon_sym_raw] = ACTIONS(1440),
    [anon_sym_static] = ACTIONS(1440),
    [anon_sym_struct] = ACTIONS(1440),
    [anon_sym_trait] = ACTIONS(1440),
    [anon_sym_type] = ACTIONS(1440),
    [anon_sym_union] = ACTIONS(1440),
    [anon_sym_unsafe] = ACTIONS(1440),
    [anon_sym_use] = ACTIONS(1440),
    [anon_sym_where] = ACTIONS(1440),
    [anon_sym_while] = ACTIONS(1440),
    [sym_mutable_specifier] = ACTIONS(1440),
    [sym_integer_literal] = ACTIONS(1442),
    [aux_sym_string_literal_token1] = ACTIONS(1442),
    [sym_char_literal] = ACTIONS(1442),
    [anon_sym_true] = ACTIONS(1440),
    [anon_sym_false] = ACTIONS(1440),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1440),
    [sym_super] = ACTIONS(1440),
    [sym_crate] = ACTIONS(1440),
    [aux_sym_metavariable_token1] = ACTIONS(1442),
    [sym__raw_string_literal_start] = ACTIONS(1442),
    [sym_float_literal] = ACTIONS(1442),
  },
  [STATE(437)] = {
    [sym_line_comment] = STATE(437),
    [sym_block_comment] = STATE(437),
    [sym_identifier] = ACTIONS(1444),
    [anon_sym_SEMI] = ACTIONS(1446),
    [anon_sym_LPAREN] = ACTIONS(1446),
    [anon_sym_RPAREN] = ACTIONS(1446),
    [anon_sym_LBRACE] = ACTIONS(1446),
    [anon_sym_RBRACE] = ACTIONS(1446),
    [anon_sym_COMMA] = ACTIONS(1446),
    [anon_sym_EQ_GT] = ACTIONS(1446),
    [anon_sym_BANG] = ACTIONS(1444),
    [anon_sym_LBRACK] = ACTIONS(1446),
    [anon_sym_RBRACK] = ACTIONS(1446),
    [anon_sym_COLON] = ACTIONS(1444),
    [anon_sym_DOLLAR] = ACTIONS(1444),
    [anon_sym_PLUS] = ACTIONS(1444),
    [anon_sym_STAR] = ACTIONS(1444),
    [anon_sym_QMARK] = ACTIONS(1446),
    [anon_sym_u8] = ACTIONS(1444),
    [anon_sym_i8] = ACTIONS(1444),
    [anon_sym_u16] = ACTIONS(1444),
    [anon_sym_i16] = ACTIONS(1444),
    [anon_sym_u32] = ACTIONS(1444),
    [anon_sym_i32] = ACTIONS(1444),
    [anon_sym_u64] = ACTIONS(1444),
    [anon_sym_i64] = ACTIONS(1444),
    [anon_sym_u128] = ACTIONS(1444),
    [anon_sym_i128] = ACTIONS(1444),
    [anon_sym_isize] = ACTIONS(1444),
    [anon_sym_usize] = ACTIONS(1444),
    [anon_sym_f32] = ACTIONS(1444),
    [anon_sym_f64] = ACTIONS(1444),
    [anon_sym_bool] = ACTIONS(1444),
    [anon_sym_str] = ACTIONS(1444),
    [anon_sym_char] = ACTIONS(1444),
    [anon_sym_DASH] = ACTIONS(1444),
    [anon_sym_SLASH] = ACTIONS(1444),
    [anon_sym_PERCENT] = ACTIONS(1444),
    [anon_sym_CARET] = ACTIONS(1444),
    [anon_sym_AMP] = ACTIONS(1444),
    [anon_sym_PIPE] = ACTIONS(1444),
    [anon_sym_AMP_AMP] = ACTIONS(1446),
    [anon_sym_PIPE_PIPE] = ACTIONS(1446),
    [anon_sym_LT_LT] = ACTIONS(1444),
    [anon_sym_GT_GT] = ACTIONS(1444),
    [anon_sym_PLUS_EQ] = ACTIONS(1446),
    [anon_sym_DASH_EQ] = ACTIONS(1446),
    [anon_sym_STAR_EQ] = ACTIONS(1446),
    [anon_sym_SLASH_EQ] = ACTIONS(1446),
    [anon_sym_PERCENT_EQ] = ACTIONS(1446),
    [anon_sym_CARET_EQ] = ACTIONS(1446),
    [anon_sym_AMP_EQ] = ACTIONS(1446),
    [anon_sym_PIPE_EQ] = ACTIONS(1446),
    [anon_sym_LT_LT_EQ] = ACTIONS(1446),
    [anon_sym_GT_GT_EQ] = ACTIONS(1446),
    [anon_sym_EQ] = ACTIONS(1444),
    [anon_sym_EQ_EQ] = ACTIONS(1446),
    [anon_sym_BANG_EQ] = ACTIONS(1446),
    [anon_sym_GT] = ACTIONS(1444),
    [anon_sym_LT] = ACTIONS(1444),
    [anon_sym_GT_EQ] = ACTIONS(1446),
    [anon_sym_LT_EQ] = ACTIONS(1446),
    [anon_sym_AT] = ACTIONS(1446),
    [anon_sym__] = ACTIONS(1444),
    [anon_sym_DOT] = ACTIONS(1444),
    [anon_sym_DOT_DOT] = ACTIONS(1444),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1446),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1446),
    [anon_sym_COLON_COLON] = ACTIONS(1446),
    [anon_sym_DASH_GT] = ACTIONS(1446),
    [anon_sym_POUND] = ACTIONS(1446),
    [anon_sym_SQUOTE] = ACTIONS(1444),
    [anon_sym_as] = ACTIONS(1444),
    [anon_sym_async] = ACTIONS(1444),
    [anon_sym_auto] = ACTIONS(1444),
    [anon_sym_await] = ACTIONS(1444),
    [anon_sym_become] = ACTIONS(1444),
    [anon_sym_break] = ACTIONS(1444),
    [anon_sym_const] = ACTIONS(1444),
    [anon_sym_continue] = ACTIONS(1444),
    [anon_sym_default] = ACTIONS(1444),
    [anon_sym_enum] = ACTIONS(1444),
    [anon_sym_fn] = ACTIONS(1444),
    [anon_sym_for] = ACTIONS(1444),
    [anon_sym_gen] = ACTIONS(1444),
    [anon_sym_if] = ACTIONS(1444),
    [anon_sym_impl] = ACTIONS(1444),
    [anon_sym_let] = ACTIONS(1444),
    [anon_sym_loop] = ACTIONS(1444),
    [anon_sym_match] = ACTIONS(1444),
    [anon_sym_mod] = ACTIONS(1444),
    [anon_sym_pub] = ACTIONS(1444),
    [anon_sym_return] = ACTIONS(1444),
    [anon_sym_raw] = ACTIONS(1444),
    [anon_sym_static] = ACTIONS(1444),
    [anon_sym_struct] = ACTIONS(1444),
    [anon_sym_trait] = ACTIONS(1444),
    [anon_sym_type] = ACTIONS(1444),
    [anon_sym_union] = ACTIONS(1444),
    [anon_sym_unsafe] = ACTIONS(1444),
    [anon_sym_use] = ACTIONS(1444),
    [anon_sym_where] = ACTIONS(1444),
    [anon_sym_while] = ACTIONS(1444),
    [sym_mutable_specifier] = ACTIONS(1444),
    [sym_integer_literal] = ACTIONS(1446),
    [aux_sym_string_literal_token1] = ACTIONS(1446),
    [sym_char_literal] = ACTIONS(1446),
    [anon_sym_true] = ACTIONS(1444),
    [anon_sym_false] = ACTIONS(1444),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1444),
    [sym_super] = ACTIONS(1444),
    [sym_crate] = ACTIONS(1444),
    [aux_sym_metavariable_token1] = ACTIONS(1446),
    [sym__raw_string_literal_start] = ACTIONS(1446),
    [sym_float_literal] = ACTIONS(1446),
  },
  [STATE(438)] = {
    [sym_line_comment] = STATE(438),
    [sym_block_comment] = STATE(438),
    [sym_identifier] = ACTIONS(1332),
    [anon_sym_SEMI] = ACTIONS(1334),
    [anon_sym_LPAREN] = ACTIONS(1448),
    [anon_sym_RPAREN] = ACTIONS(1334),
    [anon_sym_LBRACE] = ACTIONS(1334),
    [anon_sym_RBRACE] = ACTIONS(1334),
    [anon_sym_COMMA] = ACTIONS(1334),
    [anon_sym_EQ_GT] = ACTIONS(1334),
    [anon_sym_BANG] = ACTIONS(1332),
    [anon_sym_LBRACK] = ACTIONS(1334),
    [anon_sym_RBRACK] = ACTIONS(1334),
    [anon_sym_COLON] = ACTIONS(1332),
    [anon_sym_DOLLAR] = ACTIONS(1332),
    [anon_sym_PLUS] = ACTIONS(1332),
    [anon_sym_STAR] = ACTIONS(1332),
    [anon_sym_QMARK] = ACTIONS(1334),
    [anon_sym_u8] = ACTIONS(1332),
    [anon_sym_i8] = ACTIONS(1332),
    [anon_sym_u16] = ACTIONS(1332),
    [anon_sym_i16] = ACTIONS(1332),
    [anon_sym_u32] = ACTIONS(1332),
    [anon_sym_i32] = ACTIONS(1332),
    [anon_sym_u64] = ACTIONS(1332),
    [anon_sym_i64] = ACTIONS(1332),
    [anon_sym_u128] = ACTIONS(1332),
    [anon_sym_i128] = ACTIONS(1332),
    [anon_sym_isize] = ACTIONS(1332),
    [anon_sym_usize] = ACTIONS(1332),
    [anon_sym_f32] = ACTIONS(1332),
    [anon_sym_f64] = ACTIONS(1332),
    [anon_sym_bool] = ACTIONS(1332),
    [anon_sym_str] = ACTIONS(1332),
    [anon_sym_char] = ACTIONS(1332),
    [anon_sym_DASH] = ACTIONS(1332),
    [anon_sym_SLASH] = ACTIONS(1332),
    [anon_sym_PERCENT] = ACTIONS(1332),
    [anon_sym_CARET] = ACTIONS(1332),
    [anon_sym_AMP] = ACTIONS(1332),
    [anon_sym_PIPE] = ACTIONS(1332),
    [anon_sym_AMP_AMP] = ACTIONS(1334),
    [anon_sym_PIPE_PIPE] = ACTIONS(1334),
    [anon_sym_LT_LT] = ACTIONS(1332),
    [anon_sym_GT_GT] = ACTIONS(1332),
    [anon_sym_PLUS_EQ] = ACTIONS(1334),
    [anon_sym_DASH_EQ] = ACTIONS(1334),
    [anon_sym_STAR_EQ] = ACTIONS(1334),
    [anon_sym_SLASH_EQ] = ACTIONS(1334),
    [anon_sym_PERCENT_EQ] = ACTIONS(1334),
    [anon_sym_CARET_EQ] = ACTIONS(1334),
    [anon_sym_AMP_EQ] = ACTIONS(1334),
    [anon_sym_PIPE_EQ] = ACTIONS(1334),
    [anon_sym_LT_LT_EQ] = ACTIONS(1334),
    [anon_sym_GT_GT_EQ] = ACTIONS(1334),
    [anon_sym_EQ] = ACTIONS(1332),
    [anon_sym_EQ_EQ] = ACTIONS(1334),
    [anon_sym_BANG_EQ] = ACTIONS(1334),
    [anon_sym_GT] = ACTIONS(1332),
    [anon_sym_LT] = ACTIONS(1332),
    [anon_sym_GT_EQ] = ACTIONS(1334),
    [anon_sym_LT_EQ] = ACTIONS(1334),
    [anon_sym_AT] = ACTIONS(1334),
    [anon_sym__] = ACTIONS(1332),
    [anon_sym_DOT] = ACTIONS(1332),
    [anon_sym_DOT_DOT] = ACTIONS(1332),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1334),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1334),
    [anon_sym_COLON_COLON] = ACTIONS(1334),
    [anon_sym_DASH_GT] = ACTIONS(1334),
    [anon_sym_POUND] = ACTIONS(1334),
    [anon_sym_SQUOTE] = ACTIONS(1332),
    [anon_sym_as] = ACTIONS(1332),
    [anon_sym_async] = ACTIONS(1332),
    [anon_sym_auto] = ACTIONS(1332),
    [anon_sym_await] = ACTIONS(1332),
    [anon_sym_become] = ACTIONS(1332),
    [anon_sym_break] = ACTIONS(1332),
    [anon_sym_const] = ACTIONS(1332),
    [anon_sym_continue] = ACTIONS(1332),
    [anon_sym_default] = ACTIONS(1332),
    [anon_sym_enum] = ACTIONS(1332),
    [anon_sym_fn] = ACTIONS(1332),
    [anon_sym_for] = ACTIONS(1332),
    [anon_sym_gen] = ACTIONS(1332),
    [anon_sym_if] = ACTIONS(1332),
    [anon_sym_impl] = ACTIONS(1332),
    [anon_sym_let] = ACTIONS(1332),
    [anon_sym_loop] = ACTIONS(1332),
    [anon_sym_match] = ACTIONS(1332),
    [anon_sym_mod] = ACTIONS(1332),
    [anon_sym_pub] = ACTIONS(1332),
    [anon_sym_return] = ACTIONS(1332),
    [anon_sym_raw] = ACTIONS(1332),
    [anon_sym_static] = ACTIONS(1332),
    [anon_sym_struct] = ACTIONS(1332),
    [anon_sym_trait] = ACTIONS(1332),
    [anon_sym_type] = ACTIONS(1332),
    [anon_sym_union] = ACTIONS(1332),
    [anon_sym_unsafe] = ACTIONS(1332),
    [anon_sym_use] = ACTIONS(1332),
    [anon_sym_where] = ACTIONS(1332),
    [anon_sym_while] = ACTIONS(1332),
    [sym_mutable_specifier] = ACTIONS(1332),
    [sym_integer_literal] = ACTIONS(1334),
    [aux_sym_string_literal_token1] = ACTIONS(1334),
    [sym_char_literal] = ACTIONS(1334),
    [anon_sym_true] = ACTIONS(1332),
    [anon_sym_false] = ACTIONS(1332),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1332),
    [sym_super] = ACTIONS(1332),
    [sym_crate] = ACTIONS(1332),
    [aux_sym_metavariable_token1] = ACTIONS(1334),
    [sym__raw_string_literal_start] = ACTIONS(1334),
    [sym_float_literal] = ACTIONS(1334),
  },
  [STATE(439)] = {
    [sym_line_comment] = STATE(439),
    [sym_block_comment] = STATE(439),
    [sym_identifier] = ACTIONS(1450),
    [anon_sym_SEMI] = ACTIONS(1452),
    [anon_sym_LPAREN] = ACTIONS(1452),
    [anon_sym_RPAREN] = ACTIONS(1452),
    [anon_sym_LBRACE] = ACTIONS(1452),
    [anon_sym_RBRACE] = ACTIONS(1452),
    [anon_sym_COMMA] = ACTIONS(1452),
    [anon_sym_EQ_GT] = ACTIONS(1452),
    [anon_sym_BANG] = ACTIONS(1450),
    [anon_sym_LBRACK] = ACTIONS(1452),
    [anon_sym_RBRACK] = ACTIONS(1452),
    [anon_sym_COLON] = ACTIONS(1450),
    [anon_sym_DOLLAR] = ACTIONS(1450),
    [anon_sym_PLUS] = ACTIONS(1450),
    [anon_sym_STAR] = ACTIONS(1450),
    [anon_sym_QMARK] = ACTIONS(1452),
    [anon_sym_u8] = ACTIONS(1450),
    [anon_sym_i8] = ACTIONS(1450),
    [anon_sym_u16] = ACTIONS(1450),
    [anon_sym_i16] = ACTIONS(1450),
    [anon_sym_u32] = ACTIONS(1450),
    [anon_sym_i32] = ACTIONS(1450),
    [anon_sym_u64] = ACTIONS(1450),
    [anon_sym_i64] = ACTIONS(1450),
    [anon_sym_u128] = ACTIONS(1450),
    [anon_sym_i128] = ACTIONS(1450),
    [anon_sym_isize] = ACTIONS(1450),
    [anon_sym_usize] = ACTIONS(1450),
    [anon_sym_f32] = ACTIONS(1450),
    [anon_sym_f64] = ACTIONS(1450),
    [anon_sym_bool] = ACTIONS(1450),
    [anon_sym_str] = ACTIONS(1450),
    [anon_sym_char] = ACTIONS(1450),
    [anon_sym_DASH] = ACTIONS(1450),
    [anon_sym_SLASH] = ACTIONS(1450),
    [anon_sym_PERCENT] = ACTIONS(1450),
    [anon_sym_CARET] = ACTIONS(1450),
    [anon_sym_AMP] = ACTIONS(1450),
    [anon_sym_PIPE] = ACTIONS(1450),
    [anon_sym_AMP_AMP] = ACTIONS(1452),
    [anon_sym_PIPE_PIPE] = ACTIONS(1452),
    [anon_sym_LT_LT] = ACTIONS(1450),
    [anon_sym_GT_GT] = ACTIONS(1450),
    [anon_sym_PLUS_EQ] = ACTIONS(1452),
    [anon_sym_DASH_EQ] = ACTIONS(1452),
    [anon_sym_STAR_EQ] = ACTIONS(1452),
    [anon_sym_SLASH_EQ] = ACTIONS(1452),
    [anon_sym_PERCENT_EQ] = ACTIONS(1452),
    [anon_sym_CARET_EQ] = ACTIONS(1452),
    [anon_sym_AMP_EQ] = ACTIONS(1452),
    [anon_sym_PIPE_EQ] = ACTIONS(1452),
    [anon_sym_LT_LT_EQ] = ACTIONS(1452),
    [anon_sym_GT_GT_EQ] = ACTIONS(1452),
    [anon_sym_EQ] = ACTIONS(1450),
    [anon_sym_EQ_EQ] = ACTIONS(1452),
    [anon_sym_BANG_EQ] = ACTIONS(1452),
    [anon_sym_GT] = ACTIONS(1450),
    [anon_sym_LT] = ACTIONS(1450),
    [anon_sym_GT_EQ] = ACTIONS(1452),
    [anon_sym_LT_EQ] = ACTIONS(1452),
    [anon_sym_AT] = ACTIONS(1452),
    [anon_sym__] = ACTIONS(1450),
    [anon_sym_DOT] = ACTIONS(1450),
    [anon_sym_DOT_DOT] = ACTIONS(1450),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1452),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1452),
    [anon_sym_COLON_COLON] = ACTIONS(1452),
    [anon_sym_DASH_GT] = ACTIONS(1452),
    [anon_sym_POUND] = ACTIONS(1452),
    [anon_sym_SQUOTE] = ACTIONS(1450),
    [anon_sym_as] = ACTIONS(1450),
    [anon_sym_async] = ACTIONS(1450),
    [anon_sym_auto] = ACTIONS(1450),
    [anon_sym_await] = ACTIONS(1450),
    [anon_sym_become] = ACTIONS(1450),
    [anon_sym_break] = ACTIONS(1450),
    [anon_sym_const] = ACTIONS(1450),
    [anon_sym_continue] = ACTIONS(1450),
    [anon_sym_default] = ACTIONS(1450),
    [anon_sym_enum] = ACTIONS(1450),
    [anon_sym_fn] = ACTIONS(1450),
    [anon_sym_for] = ACTIONS(1450),
    [anon_sym_gen] = ACTIONS(1450),
    [anon_sym_if] = ACTIONS(1450),
    [anon_sym_impl] = ACTIONS(1450),
    [anon_sym_let] = ACTIONS(1450),
    [anon_sym_loop] = ACTIONS(1450),
    [anon_sym_match] = ACTIONS(1450),
    [anon_sym_mod] = ACTIONS(1450),
    [anon_sym_pub] = ACTIONS(1450),
    [anon_sym_return] = ACTIONS(1450),
    [anon_sym_raw] = ACTIONS(1450),
    [anon_sym_static] = ACTIONS(1450),
    [anon_sym_struct] = ACTIONS(1450),
    [anon_sym_trait] = ACTIONS(1450),
    [anon_sym_type] = ACTIONS(1450),
    [anon_sym_union] = ACTIONS(1450),
    [anon_sym_unsafe] = ACTIONS(1450),
    [anon_sym_use] = ACTIONS(1450),
    [anon_sym_where] = ACTIONS(1450),
    [anon_sym_while] = ACTIONS(1450),
    [sym_mutable_specifier] = ACTIONS(1450),
    [sym_integer_literal] = ACTIONS(1452),
    [aux_sym_string_literal_token1] = ACTIONS(1452),
    [sym_char_literal] = ACTIONS(1452),
    [anon_sym_true] = ACTIONS(1450),
    [anon_sym_false] = ACTIONS(1450),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1450),
    [sym_super] = ACTIONS(1450),
    [sym_crate] = ACTIONS(1450),
    [aux_sym_metavariable_token1] = ACTIONS(1452),
    [sym__raw_string_literal_start] = ACTIONS(1452),
    [sym_float_literal] = ACTIONS(1452),
  },
  [STATE(440)] = {
    [sym_line_comment] = STATE(440),
    [sym_block_comment] = STATE(440),
    [sym_identifier] = ACTIONS(1332),
    [anon_sym_SEMI] = ACTIONS(1334),
    [anon_sym_LPAREN] = ACTIONS(1334),
    [anon_sym_RPAREN] = ACTIONS(1334),
    [anon_sym_LBRACE] = ACTIONS(1334),
    [anon_sym_RBRACE] = ACTIONS(1334),
    [anon_sym_COMMA] = ACTIONS(1334),
    [anon_sym_EQ_GT] = ACTIONS(1334),
    [anon_sym_BANG] = ACTIONS(1332),
    [anon_sym_LBRACK] = ACTIONS(1334),
    [anon_sym_RBRACK] = ACTIONS(1334),
    [anon_sym_COLON] = ACTIONS(1332),
    [anon_sym_DOLLAR] = ACTIONS(1332),
    [anon_sym_PLUS] = ACTIONS(1332),
    [anon_sym_STAR] = ACTIONS(1332),
    [anon_sym_QMARK] = ACTIONS(1334),
    [anon_sym_u8] = ACTIONS(1332),
    [anon_sym_i8] = ACTIONS(1332),
    [anon_sym_u16] = ACTIONS(1332),
    [anon_sym_i16] = ACTIONS(1332),
    [anon_sym_u32] = ACTIONS(1332),
    [anon_sym_i32] = ACTIONS(1332),
    [anon_sym_u64] = ACTIONS(1332),
    [anon_sym_i64] = ACTIONS(1332),
    [anon_sym_u128] = ACTIONS(1332),
    [anon_sym_i128] = ACTIONS(1332),
    [anon_sym_isize] = ACTIONS(1332),
    [anon_sym_usize] = ACTIONS(1332),
    [anon_sym_f32] = ACTIONS(1332),
    [anon_sym_f64] = ACTIONS(1332),
    [anon_sym_bool] = ACTIONS(1332),
    [anon_sym_str] = ACTIONS(1332),
    [anon_sym_char] = ACTIONS(1332),
    [anon_sym_DASH] = ACTIONS(1332),
    [anon_sym_SLASH] = ACTIONS(1332),
    [anon_sym_PERCENT] = ACTIONS(1332),
    [anon_sym_CARET] = ACTIONS(1332),
    [anon_sym_AMP] = ACTIONS(1332),
    [anon_sym_PIPE] = ACTIONS(1332),
    [anon_sym_AMP_AMP] = ACTIONS(1334),
    [anon_sym_PIPE_PIPE] = ACTIONS(1334),
    [anon_sym_LT_LT] = ACTIONS(1332),
    [anon_sym_GT_GT] = ACTIONS(1332),
    [anon_sym_PLUS_EQ] = ACTIONS(1334),
    [anon_sym_DASH_EQ] = ACTIONS(1334),
    [anon_sym_STAR_EQ] = ACTIONS(1334),
    [anon_sym_SLASH_EQ] = ACTIONS(1334),
    [anon_sym_PERCENT_EQ] = ACTIONS(1334),
    [anon_sym_CARET_EQ] = ACTIONS(1334),
    [anon_sym_AMP_EQ] = ACTIONS(1334),
    [anon_sym_PIPE_EQ] = ACTIONS(1334),
    [anon_sym_LT_LT_EQ] = ACTIONS(1334),
    [anon_sym_GT_GT_EQ] = ACTIONS(1334),
    [anon_sym_EQ] = ACTIONS(1332),
    [anon_sym_EQ_EQ] = ACTIONS(1334),
    [anon_sym_BANG_EQ] = ACTIONS(1334),
    [anon_sym_GT] = ACTIONS(1332),
    [anon_sym_LT] = ACTIONS(1332),
    [anon_sym_GT_EQ] = ACTIONS(1334),
    [anon_sym_LT_EQ] = ACTIONS(1334),
    [anon_sym_AT] = ACTIONS(1334),
    [anon_sym__] = ACTIONS(1332),
    [anon_sym_DOT] = ACTIONS(1332),
    [anon_sym_DOT_DOT] = ACTIONS(1332),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1334),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1334),
    [anon_sym_COLON_COLON] = ACTIONS(1334),
    [anon_sym_DASH_GT] = ACTIONS(1334),
    [anon_sym_POUND] = ACTIONS(1334),
    [anon_sym_SQUOTE] = ACTIONS(1332),
    [anon_sym_as] = ACTIONS(1332),
    [anon_sym_async] = ACTIONS(1332),
    [anon_sym_auto] = ACTIONS(1332),
    [anon_sym_await] = ACTIONS(1332),
    [anon_sym_become] = ACTIONS(1332),
    [anon_sym_break] = ACTIONS(1332),
    [anon_sym_const] = ACTIONS(1332),
    [anon_sym_continue] = ACTIONS(1332),
    [anon_sym_default] = ACTIONS(1332),
    [anon_sym_enum] = ACTIONS(1332),
    [anon_sym_fn] = ACTIONS(1332),
    [anon_sym_for] = ACTIONS(1332),
    [anon_sym_gen] = ACTIONS(1332),
    [anon_sym_if] = ACTIONS(1332),
    [anon_sym_impl] = ACTIONS(1332),
    [anon_sym_let] = ACTIONS(1332),
    [anon_sym_loop] = ACTIONS(1332),
    [anon_sym_match] = ACTIONS(1332),
    [anon_sym_mod] = ACTIONS(1332),
    [anon_sym_pub] = ACTIONS(1332),
    [anon_sym_return] = ACTIONS(1332),
    [anon_sym_raw] = ACTIONS(1332),
    [anon_sym_static] = ACTIONS(1332),
    [anon_sym_struct] = ACTIONS(1332),
    [anon_sym_trait] = ACTIONS(1332),
    [anon_sym_type] = ACTIONS(1332),
    [anon_sym_union] = ACTIONS(1332),
    [anon_sym_unsafe] = ACTIONS(1332),
    [anon_sym_use] = ACTIONS(1332),
    [anon_sym_where] = ACTIONS(1332),
    [anon_sym_while] = ACTIONS(1332),
    [sym_mutable_specifier] = ACTIONS(1332),
    [sym_integer_literal] = ACTIONS(1334),
    [aux_sym_string_literal_token1] = ACTIONS(1334),
    [sym_char_literal] = ACTIONS(1334),
    [anon_sym_true] = ACTIONS(1332),
    [anon_sym_false] = ACTIONS(1332),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1332),
    [sym_super] = ACTIONS(1332),
    [sym_crate] = ACTIONS(1332),
    [aux_sym_metavariable_token1] = ACTIONS(1334),
    [sym__raw_string_literal_start] = ACTIONS(1334),
    [sym_float_literal] = ACTIONS(1334),
  },
  [STATE(441)] = {
    [sym_line_comment] = STATE(441),
    [sym_block_comment] = STATE(441),
    [sym_identifier] = ACTIONS(1454),
    [anon_sym_SEMI] = ACTIONS(1456),
    [anon_sym_LPAREN] = ACTIONS(1456),
    [anon_sym_RPAREN] = ACTIONS(1456),
    [anon_sym_LBRACE] = ACTIONS(1456),
    [anon_sym_RBRACE] = ACTIONS(1456),
    [anon_sym_COMMA] = ACTIONS(1456),
    [anon_sym_EQ_GT] = ACTIONS(1456),
    [anon_sym_BANG] = ACTIONS(1454),
    [anon_sym_LBRACK] = ACTIONS(1456),
    [anon_sym_RBRACK] = ACTIONS(1456),
    [anon_sym_COLON] = ACTIONS(1454),
    [anon_sym_DOLLAR] = ACTIONS(1454),
    [anon_sym_PLUS] = ACTIONS(1454),
    [anon_sym_STAR] = ACTIONS(1454),
    [anon_sym_QMARK] = ACTIONS(1456),
    [anon_sym_u8] = ACTIONS(1454),
    [anon_sym_i8] = ACTIONS(1454),
    [anon_sym_u16] = ACTIONS(1454),
    [anon_sym_i16] = ACTIONS(1454),
    [anon_sym_u32] = ACTIONS(1454),
    [anon_sym_i32] = ACTIONS(1454),
    [anon_sym_u64] = ACTIONS(1454),
    [anon_sym_i64] = ACTIONS(1454),
    [anon_sym_u128] = ACTIONS(1454),
    [anon_sym_i128] = ACTIONS(1454),
    [anon_sym_isize] = ACTIONS(1454),
    [anon_sym_usize] = ACTIONS(1454),
    [anon_sym_f32] = ACTIONS(1454),
    [anon_sym_f64] = ACTIONS(1454),
    [anon_sym_bool] = ACTIONS(1454),
    [anon_sym_str] = ACTIONS(1454),
    [anon_sym_char] = ACTIONS(1454),
    [anon_sym_DASH] = ACTIONS(1454),
    [anon_sym_SLASH] = ACTIONS(1454),
    [anon_sym_PERCENT] = ACTIONS(1454),
    [anon_sym_CARET] = ACTIONS(1454),
    [anon_sym_AMP] = ACTIONS(1454),
    [anon_sym_PIPE] = ACTIONS(1454),
    [anon_sym_AMP_AMP] = ACTIONS(1456),
    [anon_sym_PIPE_PIPE] = ACTIONS(1456),
    [anon_sym_LT_LT] = ACTIONS(1454),
    [anon_sym_GT_GT] = ACTIONS(1454),
    [anon_sym_PLUS_EQ] = ACTIONS(1456),
    [anon_sym_DASH_EQ] = ACTIONS(1456),
    [anon_sym_STAR_EQ] = ACTIONS(1456),
    [anon_sym_SLASH_EQ] = ACTIONS(1456),
    [anon_sym_PERCENT_EQ] = ACTIONS(1456),
    [anon_sym_CARET_EQ] = ACTIONS(1456),
    [anon_sym_AMP_EQ] = ACTIONS(1456),
    [anon_sym_PIPE_EQ] = ACTIONS(1456),
    [anon_sym_LT_LT_EQ] = ACTIONS(1456),
    [anon_sym_GT_GT_EQ] = ACTIONS(1456),
    [anon_sym_EQ] = ACTIONS(1454),
    [anon_sym_EQ_EQ] = ACTIONS(1456),
    [anon_sym_BANG_EQ] = ACTIONS(1456),
    [anon_sym_GT] = ACTIONS(1454),
    [anon_sym_LT] = ACTIONS(1454),
    [anon_sym_GT_EQ] = ACTIONS(1456),
    [anon_sym_LT_EQ] = ACTIONS(1456),
    [anon_sym_AT] = ACTIONS(1456),
    [anon_sym__] = ACTIONS(1454),
    [anon_sym_DOT] = ACTIONS(1454),
    [anon_sym_DOT_DOT] = ACTIONS(1454),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1456),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1456),
    [anon_sym_COLON_COLON] = ACTIONS(1456),
    [anon_sym_DASH_GT] = ACTIONS(1456),
    [anon_sym_POUND] = ACTIONS(1456),
    [anon_sym_SQUOTE] = ACTIONS(1454),
    [anon_sym_as] = ACTIONS(1454),
    [anon_sym_async] = ACTIONS(1454),
    [anon_sym_auto] = ACTIONS(1454),
    [anon_sym_await] = ACTIONS(1454),
    [anon_sym_become] = ACTIONS(1454),
    [anon_sym_break] = ACTIONS(1454),
    [anon_sym_const] = ACTIONS(1454),
    [anon_sym_continue] = ACTIONS(1454),
    [anon_sym_default] = ACTIONS(1454),
    [anon_sym_enum] = ACTIONS(1454),
    [anon_sym_fn] = ACTIONS(1454),
    [anon_sym_for] = ACTIONS(1454),
    [anon_sym_gen] = ACTIONS(1454),
    [anon_sym_if] = ACTIONS(1454),
    [anon_sym_impl] = ACTIONS(1454),
    [anon_sym_let] = ACTIONS(1454),
    [anon_sym_loop] = ACTIONS(1454),
    [anon_sym_match] = ACTIONS(1454),
    [anon_sym_mod] = ACTIONS(1454),
    [anon_sym_pub] = ACTIONS(1454),
    [anon_sym_return] = ACTIONS(1454),
    [anon_sym_raw] = ACTIONS(1454),
    [anon_sym_static] = ACTIONS(1454),
    [anon_sym_struct] = ACTIONS(1454),
    [anon_sym_trait] = ACTIONS(1454),
    [anon_sym_type] = ACTIONS(1454),
    [anon_sym_union] = ACTIONS(1454),
    [anon_sym_unsafe] = ACTIONS(1454),
    [anon_sym_use] = ACTIONS(1454),
    [anon_sym_where] = ACTIONS(1454),
    [anon_sym_while] = ACTIONS(1454),
    [sym_mutable_specifier] = ACTIONS(1454),
    [sym_integer_literal] = ACTIONS(1456),
    [aux_sym_string_literal_token1] = ACTIONS(1456),
    [sym_char_literal] = ACTIONS(1456),
    [anon_sym_true] = ACTIONS(1454),
    [anon_sym_false] = ACTIONS(1454),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1454),
    [sym_super] = ACTIONS(1454),
    [sym_crate] = ACTIONS(1454),
    [aux_sym_metavariable_token1] = ACTIONS(1456),
    [sym__raw_string_literal_start] = ACTIONS(1456),
    [sym_float_literal] = ACTIONS(1456),
  },
  [STATE(442)] = {
    [sym_line_comment] = STATE(442),
    [sym_block_comment] = STATE(442),
    [ts_builtin_sym_end] = ACTIONS(1458),
    [sym_identifier] = ACTIONS(1460),
    [anon_sym_SEMI] = ACTIONS(1458),
    [anon_sym_macro] = ACTIONS(1460),
    [anon_sym_LPAREN] = ACTIONS(1458),
    [anon_sym_LBRACE] = ACTIONS(1458),
    [anon_sym_RBRACE] = ACTIONS(1458),
    [anon_sym_macro_rules] = ACTIONS(1460),
    [anon_sym_BANG] = ACTIONS(1460),
    [anon_sym_LBRACK] = ACTIONS(1458),
    [anon_sym_PLUS] = ACTIONS(1460),
    [anon_sym_STAR] = ACTIONS(1460),
    [anon_sym_QMARK] = ACTIONS(1458),
    [anon_sym_u8] = ACTIONS(1460),
    [anon_sym_i8] = ACTIONS(1460),
    [anon_sym_u16] = ACTIONS(1460),
    [anon_sym_i16] = ACTIONS(1460),
    [anon_sym_u32] = ACTIONS(1460),
    [anon_sym_i32] = ACTIONS(1460),
    [anon_sym_u64] = ACTIONS(1460),
    [anon_sym_i64] = ACTIONS(1460),
    [anon_sym_u128] = ACTIONS(1460),
    [anon_sym_i128] = ACTIONS(1460),
    [anon_sym_isize] = ACTIONS(1460),
    [anon_sym_usize] = ACTIONS(1460),
    [anon_sym_f32] = ACTIONS(1460),
    [anon_sym_f64] = ACTIONS(1460),
    [anon_sym_bool] = ACTIONS(1460),
    [anon_sym_str] = ACTIONS(1460),
    [anon_sym_char] = ACTIONS(1460),
    [anon_sym_DASH] = ACTIONS(1460),
    [anon_sym_SLASH] = ACTIONS(1460),
    [anon_sym_PERCENT] = ACTIONS(1460),
    [anon_sym_CARET] = ACTIONS(1460),
    [anon_sym_AMP] = ACTIONS(1460),
    [anon_sym_PIPE] = ACTIONS(1460),
    [anon_sym_AMP_AMP] = ACTIONS(1458),
    [anon_sym_PIPE_PIPE] = ACTIONS(1458),
    [anon_sym_LT_LT] = ACTIONS(1460),
    [anon_sym_GT_GT] = ACTIONS(1460),
    [anon_sym_PLUS_EQ] = ACTIONS(1458),
    [anon_sym_DASH_EQ] = ACTIONS(1458),
    [anon_sym_STAR_EQ] = ACTIONS(1458),
    [anon_sym_SLASH_EQ] = ACTIONS(1458),
    [anon_sym_PERCENT_EQ] = ACTIONS(1458),
    [anon_sym_CARET_EQ] = ACTIONS(1458),
    [anon_sym_AMP_EQ] = ACTIONS(1458),
    [anon_sym_PIPE_EQ] = ACTIONS(1458),
    [anon_sym_LT_LT_EQ] = ACTIONS(1458),
    [anon_sym_GT_GT_EQ] = ACTIONS(1458),
    [anon_sym_EQ] = ACTIONS(1460),
    [anon_sym_EQ_EQ] = ACTIONS(1458),
    [anon_sym_BANG_EQ] = ACTIONS(1458),
    [anon_sym_GT] = ACTIONS(1460),
    [anon_sym_LT] = ACTIONS(1460),
    [anon_sym_GT_EQ] = ACTIONS(1458),
    [anon_sym_LT_EQ] = ACTIONS(1458),
    [anon_sym_DOT] = ACTIONS(1460),
    [anon_sym_DOT_DOT] = ACTIONS(1460),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1458),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1458),
    [anon_sym_COLON_COLON] = ACTIONS(1458),
    [anon_sym_POUND] = ACTIONS(1458),
    [anon_sym_SQUOTE] = ACTIONS(1460),
    [anon_sym_as] = ACTIONS(1460),
    [anon_sym_async] = ACTIONS(1460),
    [anon_sym_auto] = ACTIONS(1460),
    [anon_sym_await] = ACTIONS(1460),
    [anon_sym_become] = ACTIONS(1460),
    [anon_sym_break] = ACTIONS(1460),
    [anon_sym_const] = ACTIONS(1460),
    [anon_sym_continue] = ACTIONS(1460),
    [anon_sym_default] = ACTIONS(1460),
    [anon_sym_enum] = ACTIONS(1460),
    [anon_sym_fn] = ACTIONS(1460),
    [anon_sym_for] = ACTIONS(1460),
    [anon_sym_gen] = ACTIONS(1460),
    [anon_sym_if] = ACTIONS(1460),
    [anon_sym_impl] = ACTIONS(1460),
    [anon_sym_let] = ACTIONS(1460),
    [anon_sym_loop] = ACTIONS(1460),
    [anon_sym_match] = ACTIONS(1460),
    [anon_sym_mod] = ACTIONS(1460),
    [anon_sym_pub] = ACTIONS(1460),
    [anon_sym_return] = ACTIONS(1460),
    [anon_sym_raw] = ACTIONS(1460),
    [anon_sym_static] = ACTIONS(1460),
    [anon_sym_struct] = ACTIONS(1460),
    [anon_sym_trait] = ACTIONS(1460),
    [anon_sym_type] = ACTIONS(1460),
    [anon_sym_union] = ACTIONS(1460),
    [anon_sym_unsafe] = ACTIONS(1460),
    [anon_sym_use] = ACTIONS(1460),
    [anon_sym_while] = ACTIONS(1460),
    [anon_sym_extern] = ACTIONS(1460),
    [anon_sym_safe] = ACTIONS(1460),
    [anon_sym_else] = ACTIONS(1460),
    [anon_sym_yield] = ACTIONS(1460),
    [anon_sym_move] = ACTIONS(1460),
    [anon_sym_try] = ACTIONS(1460),
    [sym_integer_literal] = ACTIONS(1458),
    [aux_sym_string_literal_token1] = ACTIONS(1458),
    [sym_char_literal] = ACTIONS(1458),
    [anon_sym_true] = ACTIONS(1460),
    [anon_sym_false] = ACTIONS(1460),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1460),
    [sym_super] = ACTIONS(1460),
    [sym_crate] = ACTIONS(1460),
    [aux_sym_metavariable_token1] = ACTIONS(1458),
    [sym__raw_string_literal_start] = ACTIONS(1458),
    [sym_float_literal] = ACTIONS(1458),
    [sym__outer_block_doc_comment_start] = ACTIONS(1458),
    [sym__outer_line_doc_comment_start] = ACTIONS(1458),
  },
  [STATE(443)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4986),
    [sym_bracketed_type] = STATE(6294),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4212),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4343),
    [sym_scoped_identifier] = STATE(3864),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4784),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(443),
    [sym_block_comment] = STATE(443),
    [sym_metavariable] = STATE(4046),
    [sym_identifier] = ACTIONS(1462),
    [anon_sym_LPAREN] = ACTIONS(1464),
    [anon_sym_COMMA] = ACTIONS(1466),
    [anon_sym_macro_rules] = ACTIONS(1468),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_RBRACK] = ACTIONS(1470),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1472),
    [anon_sym_i8] = ACTIONS(1472),
    [anon_sym_u16] = ACTIONS(1472),
    [anon_sym_i16] = ACTIONS(1472),
    [anon_sym_u32] = ACTIONS(1472),
    [anon_sym_i32] = ACTIONS(1472),
    [anon_sym_u64] = ACTIONS(1472),
    [anon_sym_i64] = ACTIONS(1472),
    [anon_sym_u128] = ACTIONS(1472),
    [anon_sym_i128] = ACTIONS(1472),
    [anon_sym_isize] = ACTIONS(1472),
    [anon_sym_usize] = ACTIONS(1472),
    [anon_sym_f32] = ACTIONS(1472),
    [anon_sym_f64] = ACTIONS(1472),
    [anon_sym_bool] = ACTIONS(1472),
    [anon_sym_str] = ACTIONS(1472),
    [anon_sym_char] = ACTIONS(1472),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1474),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1476),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1468),
    [anon_sym_await] = ACTIONS(1468),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1478),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1478),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1468),
    [anon_sym_union] = ACTIONS(1468),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1478),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1468),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1480),
    [sym_super] = ACTIONS(1480),
    [sym_crate] = ACTIONS(1480),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(444)] = {
    [sym_line_comment] = STATE(444),
    [sym_block_comment] = STATE(444),
    [ts_builtin_sym_end] = ACTIONS(1482),
    [sym_identifier] = ACTIONS(1484),
    [anon_sym_SEMI] = ACTIONS(1482),
    [anon_sym_macro] = ACTIONS(1484),
    [anon_sym_LPAREN] = ACTIONS(1482),
    [anon_sym_LBRACE] = ACTIONS(1482),
    [anon_sym_RBRACE] = ACTIONS(1482),
    [anon_sym_macro_rules] = ACTIONS(1484),
    [anon_sym_BANG] = ACTIONS(1484),
    [anon_sym_LBRACK] = ACTIONS(1482),
    [anon_sym_PLUS] = ACTIONS(1484),
    [anon_sym_STAR] = ACTIONS(1484),
    [anon_sym_QMARK] = ACTIONS(1482),
    [anon_sym_u8] = ACTIONS(1484),
    [anon_sym_i8] = ACTIONS(1484),
    [anon_sym_u16] = ACTIONS(1484),
    [anon_sym_i16] = ACTIONS(1484),
    [anon_sym_u32] = ACTIONS(1484),
    [anon_sym_i32] = ACTIONS(1484),
    [anon_sym_u64] = ACTIONS(1484),
    [anon_sym_i64] = ACTIONS(1484),
    [anon_sym_u128] = ACTIONS(1484),
    [anon_sym_i128] = ACTIONS(1484),
    [anon_sym_isize] = ACTIONS(1484),
    [anon_sym_usize] = ACTIONS(1484),
    [anon_sym_f32] = ACTIONS(1484),
    [anon_sym_f64] = ACTIONS(1484),
    [anon_sym_bool] = ACTIONS(1484),
    [anon_sym_str] = ACTIONS(1484),
    [anon_sym_char] = ACTIONS(1484),
    [anon_sym_DASH] = ACTIONS(1484),
    [anon_sym_SLASH] = ACTIONS(1484),
    [anon_sym_PERCENT] = ACTIONS(1484),
    [anon_sym_CARET] = ACTIONS(1484),
    [anon_sym_AMP] = ACTIONS(1484),
    [anon_sym_PIPE] = ACTIONS(1484),
    [anon_sym_AMP_AMP] = ACTIONS(1482),
    [anon_sym_PIPE_PIPE] = ACTIONS(1482),
    [anon_sym_LT_LT] = ACTIONS(1484),
    [anon_sym_GT_GT] = ACTIONS(1484),
    [anon_sym_PLUS_EQ] = ACTIONS(1482),
    [anon_sym_DASH_EQ] = ACTIONS(1482),
    [anon_sym_STAR_EQ] = ACTIONS(1482),
    [anon_sym_SLASH_EQ] = ACTIONS(1482),
    [anon_sym_PERCENT_EQ] = ACTIONS(1482),
    [anon_sym_CARET_EQ] = ACTIONS(1482),
    [anon_sym_AMP_EQ] = ACTIONS(1482),
    [anon_sym_PIPE_EQ] = ACTIONS(1482),
    [anon_sym_LT_LT_EQ] = ACTIONS(1482),
    [anon_sym_GT_GT_EQ] = ACTIONS(1482),
    [anon_sym_EQ] = ACTIONS(1484),
    [anon_sym_EQ_EQ] = ACTIONS(1482),
    [anon_sym_BANG_EQ] = ACTIONS(1482),
    [anon_sym_GT] = ACTIONS(1484),
    [anon_sym_LT] = ACTIONS(1484),
    [anon_sym_GT_EQ] = ACTIONS(1482),
    [anon_sym_LT_EQ] = ACTIONS(1482),
    [anon_sym_DOT] = ACTIONS(1484),
    [anon_sym_DOT_DOT] = ACTIONS(1484),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1482),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1482),
    [anon_sym_COLON_COLON] = ACTIONS(1482),
    [anon_sym_POUND] = ACTIONS(1482),
    [anon_sym_SQUOTE] = ACTIONS(1484),
    [anon_sym_as] = ACTIONS(1484),
    [anon_sym_async] = ACTIONS(1484),
    [anon_sym_auto] = ACTIONS(1484),
    [anon_sym_await] = ACTIONS(1484),
    [anon_sym_become] = ACTIONS(1484),
    [anon_sym_break] = ACTIONS(1484),
    [anon_sym_const] = ACTIONS(1484),
    [anon_sym_continue] = ACTIONS(1484),
    [anon_sym_default] = ACTIONS(1484),
    [anon_sym_enum] = ACTIONS(1484),
    [anon_sym_fn] = ACTIONS(1484),
    [anon_sym_for] = ACTIONS(1484),
    [anon_sym_gen] = ACTIONS(1484),
    [anon_sym_if] = ACTIONS(1484),
    [anon_sym_impl] = ACTIONS(1484),
    [anon_sym_let] = ACTIONS(1484),
    [anon_sym_loop] = ACTIONS(1484),
    [anon_sym_match] = ACTIONS(1484),
    [anon_sym_mod] = ACTIONS(1484),
    [anon_sym_pub] = ACTIONS(1484),
    [anon_sym_return] = ACTIONS(1484),
    [anon_sym_raw] = ACTIONS(1484),
    [anon_sym_static] = ACTIONS(1484),
    [anon_sym_struct] = ACTIONS(1484),
    [anon_sym_trait] = ACTIONS(1484),
    [anon_sym_type] = ACTIONS(1484),
    [anon_sym_union] = ACTIONS(1484),
    [anon_sym_unsafe] = ACTIONS(1484),
    [anon_sym_use] = ACTIONS(1484),
    [anon_sym_while] = ACTIONS(1484),
    [anon_sym_extern] = ACTIONS(1484),
    [anon_sym_safe] = ACTIONS(1484),
    [anon_sym_else] = ACTIONS(1484),
    [anon_sym_yield] = ACTIONS(1484),
    [anon_sym_move] = ACTIONS(1484),
    [anon_sym_try] = ACTIONS(1484),
    [sym_integer_literal] = ACTIONS(1482),
    [aux_sym_string_literal_token1] = ACTIONS(1482),
    [sym_char_literal] = ACTIONS(1482),
    [anon_sym_true] = ACTIONS(1484),
    [anon_sym_false] = ACTIONS(1484),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1484),
    [sym_super] = ACTIONS(1484),
    [sym_crate] = ACTIONS(1484),
    [aux_sym_metavariable_token1] = ACTIONS(1482),
    [sym__raw_string_literal_start] = ACTIONS(1482),
    [sym_float_literal] = ACTIONS(1482),
    [sym__outer_block_doc_comment_start] = ACTIONS(1482),
    [sym__outer_line_doc_comment_start] = ACTIONS(1482),
  },
  [STATE(445)] = {
    [sym_line_comment] = STATE(445),
    [sym_block_comment] = STATE(445),
    [sym_identifier] = ACTIONS(1346),
    [anon_sym_SEMI] = ACTIONS(1348),
    [anon_sym_LPAREN] = ACTIONS(1348),
    [anon_sym_RPAREN] = ACTIONS(1348),
    [anon_sym_LBRACE] = ACTIONS(1348),
    [anon_sym_RBRACE] = ACTIONS(1348),
    [anon_sym_COMMA] = ACTIONS(1348),
    [anon_sym_EQ_GT] = ACTIONS(1348),
    [anon_sym_BANG] = ACTIONS(1346),
    [anon_sym_LBRACK] = ACTIONS(1348),
    [anon_sym_RBRACK] = ACTIONS(1348),
    [anon_sym_COLON] = ACTIONS(1346),
    [anon_sym_DOLLAR] = ACTIONS(1346),
    [anon_sym_PLUS] = ACTIONS(1346),
    [anon_sym_STAR] = ACTIONS(1346),
    [anon_sym_QMARK] = ACTIONS(1348),
    [anon_sym_u8] = ACTIONS(1346),
    [anon_sym_i8] = ACTIONS(1346),
    [anon_sym_u16] = ACTIONS(1346),
    [anon_sym_i16] = ACTIONS(1346),
    [anon_sym_u32] = ACTIONS(1346),
    [anon_sym_i32] = ACTIONS(1346),
    [anon_sym_u64] = ACTIONS(1346),
    [anon_sym_i64] = ACTIONS(1346),
    [anon_sym_u128] = ACTIONS(1346),
    [anon_sym_i128] = ACTIONS(1346),
    [anon_sym_isize] = ACTIONS(1346),
    [anon_sym_usize] = ACTIONS(1346),
    [anon_sym_f32] = ACTIONS(1346),
    [anon_sym_f64] = ACTIONS(1346),
    [anon_sym_bool] = ACTIONS(1346),
    [anon_sym_str] = ACTIONS(1346),
    [anon_sym_char] = ACTIONS(1346),
    [anon_sym_DASH] = ACTIONS(1346),
    [anon_sym_SLASH] = ACTIONS(1346),
    [anon_sym_PERCENT] = ACTIONS(1346),
    [anon_sym_CARET] = ACTIONS(1346),
    [anon_sym_AMP] = ACTIONS(1346),
    [anon_sym_PIPE] = ACTIONS(1346),
    [anon_sym_AMP_AMP] = ACTIONS(1348),
    [anon_sym_PIPE_PIPE] = ACTIONS(1348),
    [anon_sym_LT_LT] = ACTIONS(1346),
    [anon_sym_GT_GT] = ACTIONS(1346),
    [anon_sym_PLUS_EQ] = ACTIONS(1348),
    [anon_sym_DASH_EQ] = ACTIONS(1348),
    [anon_sym_STAR_EQ] = ACTIONS(1348),
    [anon_sym_SLASH_EQ] = ACTIONS(1348),
    [anon_sym_PERCENT_EQ] = ACTIONS(1348),
    [anon_sym_CARET_EQ] = ACTIONS(1348),
    [anon_sym_AMP_EQ] = ACTIONS(1348),
    [anon_sym_PIPE_EQ] = ACTIONS(1348),
    [anon_sym_LT_LT_EQ] = ACTIONS(1348),
    [anon_sym_GT_GT_EQ] = ACTIONS(1348),
    [anon_sym_EQ] = ACTIONS(1346),
    [anon_sym_EQ_EQ] = ACTIONS(1348),
    [anon_sym_BANG_EQ] = ACTIONS(1348),
    [anon_sym_GT] = ACTIONS(1346),
    [anon_sym_LT] = ACTIONS(1346),
    [anon_sym_GT_EQ] = ACTIONS(1348),
    [anon_sym_LT_EQ] = ACTIONS(1348),
    [anon_sym_AT] = ACTIONS(1348),
    [anon_sym__] = ACTIONS(1346),
    [anon_sym_DOT] = ACTIONS(1346),
    [anon_sym_DOT_DOT] = ACTIONS(1346),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1348),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1348),
    [anon_sym_COLON_COLON] = ACTIONS(1348),
    [anon_sym_DASH_GT] = ACTIONS(1348),
    [anon_sym_POUND] = ACTIONS(1348),
    [anon_sym_SQUOTE] = ACTIONS(1346),
    [anon_sym_as] = ACTIONS(1346),
    [anon_sym_async] = ACTIONS(1346),
    [anon_sym_auto] = ACTIONS(1346),
    [anon_sym_await] = ACTIONS(1346),
    [anon_sym_become] = ACTIONS(1346),
    [anon_sym_break] = ACTIONS(1346),
    [anon_sym_const] = ACTIONS(1346),
    [anon_sym_continue] = ACTIONS(1346),
    [anon_sym_default] = ACTIONS(1346),
    [anon_sym_enum] = ACTIONS(1346),
    [anon_sym_fn] = ACTIONS(1346),
    [anon_sym_for] = ACTIONS(1346),
    [anon_sym_gen] = ACTIONS(1346),
    [anon_sym_if] = ACTIONS(1346),
    [anon_sym_impl] = ACTIONS(1346),
    [anon_sym_let] = ACTIONS(1346),
    [anon_sym_loop] = ACTIONS(1346),
    [anon_sym_match] = ACTIONS(1346),
    [anon_sym_mod] = ACTIONS(1346),
    [anon_sym_pub] = ACTIONS(1346),
    [anon_sym_return] = ACTIONS(1346),
    [anon_sym_raw] = ACTIONS(1346),
    [anon_sym_static] = ACTIONS(1346),
    [anon_sym_struct] = ACTIONS(1346),
    [anon_sym_trait] = ACTIONS(1346),
    [anon_sym_type] = ACTIONS(1346),
    [anon_sym_union] = ACTIONS(1346),
    [anon_sym_unsafe] = ACTIONS(1346),
    [anon_sym_use] = ACTIONS(1346),
    [anon_sym_where] = ACTIONS(1346),
    [anon_sym_while] = ACTIONS(1346),
    [sym_mutable_specifier] = ACTIONS(1346),
    [sym_integer_literal] = ACTIONS(1348),
    [aux_sym_string_literal_token1] = ACTIONS(1348),
    [sym_char_literal] = ACTIONS(1348),
    [anon_sym_true] = ACTIONS(1346),
    [anon_sym_false] = ACTIONS(1346),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1346),
    [sym_super] = ACTIONS(1346),
    [sym_crate] = ACTIONS(1346),
    [aux_sym_metavariable_token1] = ACTIONS(1348),
    [sym__raw_string_literal_start] = ACTIONS(1348),
    [sym_float_literal] = ACTIONS(1348),
  },
  [STATE(446)] = {
    [sym_line_comment] = STATE(446),
    [sym_block_comment] = STATE(446),
    [sym_identifier] = ACTIONS(1486),
    [anon_sym_SEMI] = ACTIONS(1488),
    [anon_sym_LPAREN] = ACTIONS(1488),
    [anon_sym_RPAREN] = ACTIONS(1488),
    [anon_sym_LBRACE] = ACTIONS(1488),
    [anon_sym_RBRACE] = ACTIONS(1488),
    [anon_sym_COMMA] = ACTIONS(1488),
    [anon_sym_EQ_GT] = ACTIONS(1488),
    [anon_sym_BANG] = ACTIONS(1486),
    [anon_sym_LBRACK] = ACTIONS(1488),
    [anon_sym_RBRACK] = ACTIONS(1488),
    [anon_sym_COLON] = ACTIONS(1486),
    [anon_sym_DOLLAR] = ACTIONS(1486),
    [anon_sym_PLUS] = ACTIONS(1486),
    [anon_sym_STAR] = ACTIONS(1486),
    [anon_sym_QMARK] = ACTIONS(1488),
    [anon_sym_u8] = ACTIONS(1486),
    [anon_sym_i8] = ACTIONS(1486),
    [anon_sym_u16] = ACTIONS(1486),
    [anon_sym_i16] = ACTIONS(1486),
    [anon_sym_u32] = ACTIONS(1486),
    [anon_sym_i32] = ACTIONS(1486),
    [anon_sym_u64] = ACTIONS(1486),
    [anon_sym_i64] = ACTIONS(1486),
    [anon_sym_u128] = ACTIONS(1486),
    [anon_sym_i128] = ACTIONS(1486),
    [anon_sym_isize] = ACTIONS(1486),
    [anon_sym_usize] = ACTIONS(1486),
    [anon_sym_f32] = ACTIONS(1486),
    [anon_sym_f64] = ACTIONS(1486),
    [anon_sym_bool] = ACTIONS(1486),
    [anon_sym_str] = ACTIONS(1486),
    [anon_sym_char] = ACTIONS(1486),
    [anon_sym_DASH] = ACTIONS(1486),
    [anon_sym_SLASH] = ACTIONS(1486),
    [anon_sym_PERCENT] = ACTIONS(1486),
    [anon_sym_CARET] = ACTIONS(1486),
    [anon_sym_AMP] = ACTIONS(1486),
    [anon_sym_PIPE] = ACTIONS(1486),
    [anon_sym_AMP_AMP] = ACTIONS(1488),
    [anon_sym_PIPE_PIPE] = ACTIONS(1488),
    [anon_sym_LT_LT] = ACTIONS(1486),
    [anon_sym_GT_GT] = ACTIONS(1486),
    [anon_sym_PLUS_EQ] = ACTIONS(1488),
    [anon_sym_DASH_EQ] = ACTIONS(1488),
    [anon_sym_STAR_EQ] = ACTIONS(1488),
    [anon_sym_SLASH_EQ] = ACTIONS(1488),
    [anon_sym_PERCENT_EQ] = ACTIONS(1488),
    [anon_sym_CARET_EQ] = ACTIONS(1488),
    [anon_sym_AMP_EQ] = ACTIONS(1488),
    [anon_sym_PIPE_EQ] = ACTIONS(1488),
    [anon_sym_LT_LT_EQ] = ACTIONS(1488),
    [anon_sym_GT_GT_EQ] = ACTIONS(1488),
    [anon_sym_EQ] = ACTIONS(1486),
    [anon_sym_EQ_EQ] = ACTIONS(1488),
    [anon_sym_BANG_EQ] = ACTIONS(1488),
    [anon_sym_GT] = ACTIONS(1486),
    [anon_sym_LT] = ACTIONS(1486),
    [anon_sym_GT_EQ] = ACTIONS(1488),
    [anon_sym_LT_EQ] = ACTIONS(1488),
    [anon_sym_AT] = ACTIONS(1488),
    [anon_sym__] = ACTIONS(1486),
    [anon_sym_DOT] = ACTIONS(1486),
    [anon_sym_DOT_DOT] = ACTIONS(1486),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1488),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1488),
    [anon_sym_COLON_COLON] = ACTIONS(1488),
    [anon_sym_DASH_GT] = ACTIONS(1488),
    [anon_sym_POUND] = ACTIONS(1488),
    [anon_sym_SQUOTE] = ACTIONS(1486),
    [anon_sym_as] = ACTIONS(1486),
    [anon_sym_async] = ACTIONS(1486),
    [anon_sym_auto] = ACTIONS(1486),
    [anon_sym_await] = ACTIONS(1486),
    [anon_sym_become] = ACTIONS(1486),
    [anon_sym_break] = ACTIONS(1486),
    [anon_sym_const] = ACTIONS(1486),
    [anon_sym_continue] = ACTIONS(1486),
    [anon_sym_default] = ACTIONS(1486),
    [anon_sym_enum] = ACTIONS(1486),
    [anon_sym_fn] = ACTIONS(1486),
    [anon_sym_for] = ACTIONS(1486),
    [anon_sym_gen] = ACTIONS(1486),
    [anon_sym_if] = ACTIONS(1486),
    [anon_sym_impl] = ACTIONS(1486),
    [anon_sym_let] = ACTIONS(1486),
    [anon_sym_loop] = ACTIONS(1486),
    [anon_sym_match] = ACTIONS(1486),
    [anon_sym_mod] = ACTIONS(1486),
    [anon_sym_pub] = ACTIONS(1486),
    [anon_sym_return] = ACTIONS(1486),
    [anon_sym_raw] = ACTIONS(1486),
    [anon_sym_static] = ACTIONS(1486),
    [anon_sym_struct] = ACTIONS(1486),
    [anon_sym_trait] = ACTIONS(1486),
    [anon_sym_type] = ACTIONS(1486),
    [anon_sym_union] = ACTIONS(1486),
    [anon_sym_unsafe] = ACTIONS(1486),
    [anon_sym_use] = ACTIONS(1486),
    [anon_sym_where] = ACTIONS(1486),
    [anon_sym_while] = ACTIONS(1486),
    [sym_mutable_specifier] = ACTIONS(1486),
    [sym_integer_literal] = ACTIONS(1488),
    [aux_sym_string_literal_token1] = ACTIONS(1488),
    [sym_char_literal] = ACTIONS(1488),
    [anon_sym_true] = ACTIONS(1486),
    [anon_sym_false] = ACTIONS(1486),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1486),
    [sym_super] = ACTIONS(1486),
    [sym_crate] = ACTIONS(1486),
    [aux_sym_metavariable_token1] = ACTIONS(1488),
    [sym__raw_string_literal_start] = ACTIONS(1488),
    [sym_float_literal] = ACTIONS(1488),
  },
  [STATE(447)] = {
    [sym_line_comment] = STATE(447),
    [sym_block_comment] = STATE(447),
    [sym_identifier] = ACTIONS(1490),
    [anon_sym_SEMI] = ACTIONS(1492),
    [anon_sym_LPAREN] = ACTIONS(1492),
    [anon_sym_RPAREN] = ACTIONS(1492),
    [anon_sym_LBRACE] = ACTIONS(1492),
    [anon_sym_RBRACE] = ACTIONS(1492),
    [anon_sym_COMMA] = ACTIONS(1492),
    [anon_sym_EQ_GT] = ACTIONS(1492),
    [anon_sym_BANG] = ACTIONS(1490),
    [anon_sym_LBRACK] = ACTIONS(1492),
    [anon_sym_RBRACK] = ACTIONS(1492),
    [anon_sym_COLON] = ACTIONS(1490),
    [anon_sym_DOLLAR] = ACTIONS(1490),
    [anon_sym_PLUS] = ACTIONS(1490),
    [anon_sym_STAR] = ACTIONS(1490),
    [anon_sym_QMARK] = ACTIONS(1492),
    [anon_sym_u8] = ACTIONS(1490),
    [anon_sym_i8] = ACTIONS(1490),
    [anon_sym_u16] = ACTIONS(1490),
    [anon_sym_i16] = ACTIONS(1490),
    [anon_sym_u32] = ACTIONS(1490),
    [anon_sym_i32] = ACTIONS(1490),
    [anon_sym_u64] = ACTIONS(1490),
    [anon_sym_i64] = ACTIONS(1490),
    [anon_sym_u128] = ACTIONS(1490),
    [anon_sym_i128] = ACTIONS(1490),
    [anon_sym_isize] = ACTIONS(1490),
    [anon_sym_usize] = ACTIONS(1490),
    [anon_sym_f32] = ACTIONS(1490),
    [anon_sym_f64] = ACTIONS(1490),
    [anon_sym_bool] = ACTIONS(1490),
    [anon_sym_str] = ACTIONS(1490),
    [anon_sym_char] = ACTIONS(1490),
    [anon_sym_DASH] = ACTIONS(1490),
    [anon_sym_SLASH] = ACTIONS(1490),
    [anon_sym_PERCENT] = ACTIONS(1490),
    [anon_sym_CARET] = ACTIONS(1490),
    [anon_sym_AMP] = ACTIONS(1490),
    [anon_sym_PIPE] = ACTIONS(1490),
    [anon_sym_AMP_AMP] = ACTIONS(1492),
    [anon_sym_PIPE_PIPE] = ACTIONS(1492),
    [anon_sym_LT_LT] = ACTIONS(1490),
    [anon_sym_GT_GT] = ACTIONS(1490),
    [anon_sym_PLUS_EQ] = ACTIONS(1492),
    [anon_sym_DASH_EQ] = ACTIONS(1492),
    [anon_sym_STAR_EQ] = ACTIONS(1492),
    [anon_sym_SLASH_EQ] = ACTIONS(1492),
    [anon_sym_PERCENT_EQ] = ACTIONS(1492),
    [anon_sym_CARET_EQ] = ACTIONS(1492),
    [anon_sym_AMP_EQ] = ACTIONS(1492),
    [anon_sym_PIPE_EQ] = ACTIONS(1492),
    [anon_sym_LT_LT_EQ] = ACTIONS(1492),
    [anon_sym_GT_GT_EQ] = ACTIONS(1492),
    [anon_sym_EQ] = ACTIONS(1490),
    [anon_sym_EQ_EQ] = ACTIONS(1492),
    [anon_sym_BANG_EQ] = ACTIONS(1492),
    [anon_sym_GT] = ACTIONS(1490),
    [anon_sym_LT] = ACTIONS(1490),
    [anon_sym_GT_EQ] = ACTIONS(1492),
    [anon_sym_LT_EQ] = ACTIONS(1492),
    [anon_sym_AT] = ACTIONS(1492),
    [anon_sym__] = ACTIONS(1490),
    [anon_sym_DOT] = ACTIONS(1490),
    [anon_sym_DOT_DOT] = ACTIONS(1490),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1492),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1492),
    [anon_sym_COLON_COLON] = ACTIONS(1492),
    [anon_sym_DASH_GT] = ACTIONS(1492),
    [anon_sym_POUND] = ACTIONS(1492),
    [anon_sym_SQUOTE] = ACTIONS(1490),
    [anon_sym_as] = ACTIONS(1490),
    [anon_sym_async] = ACTIONS(1490),
    [anon_sym_auto] = ACTIONS(1490),
    [anon_sym_await] = ACTIONS(1490),
    [anon_sym_become] = ACTIONS(1490),
    [anon_sym_break] = ACTIONS(1490),
    [anon_sym_const] = ACTIONS(1490),
    [anon_sym_continue] = ACTIONS(1490),
    [anon_sym_default] = ACTIONS(1490),
    [anon_sym_enum] = ACTIONS(1490),
    [anon_sym_fn] = ACTIONS(1490),
    [anon_sym_for] = ACTIONS(1490),
    [anon_sym_gen] = ACTIONS(1490),
    [anon_sym_if] = ACTIONS(1490),
    [anon_sym_impl] = ACTIONS(1490),
    [anon_sym_let] = ACTIONS(1490),
    [anon_sym_loop] = ACTIONS(1490),
    [anon_sym_match] = ACTIONS(1490),
    [anon_sym_mod] = ACTIONS(1490),
    [anon_sym_pub] = ACTIONS(1490),
    [anon_sym_return] = ACTIONS(1490),
    [anon_sym_raw] = ACTIONS(1490),
    [anon_sym_static] = ACTIONS(1490),
    [anon_sym_struct] = ACTIONS(1490),
    [anon_sym_trait] = ACTIONS(1490),
    [anon_sym_type] = ACTIONS(1490),
    [anon_sym_union] = ACTIONS(1490),
    [anon_sym_unsafe] = ACTIONS(1490),
    [anon_sym_use] = ACTIONS(1490),
    [anon_sym_where] = ACTIONS(1490),
    [anon_sym_while] = ACTIONS(1490),
    [sym_mutable_specifier] = ACTIONS(1490),
    [sym_integer_literal] = ACTIONS(1492),
    [aux_sym_string_literal_token1] = ACTIONS(1492),
    [sym_char_literal] = ACTIONS(1492),
    [anon_sym_true] = ACTIONS(1490),
    [anon_sym_false] = ACTIONS(1490),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1490),
    [sym_super] = ACTIONS(1490),
    [sym_crate] = ACTIONS(1490),
    [aux_sym_metavariable_token1] = ACTIONS(1492),
    [sym__raw_string_literal_start] = ACTIONS(1492),
    [sym_float_literal] = ACTIONS(1492),
  },
  [STATE(448)] = {
    [sym_line_comment] = STATE(448),
    [sym_block_comment] = STATE(448),
    [ts_builtin_sym_end] = ACTIONS(1494),
    [sym_identifier] = ACTIONS(1496),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_macro] = ACTIONS(1496),
    [anon_sym_LPAREN] = ACTIONS(1494),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_RBRACE] = ACTIONS(1494),
    [anon_sym_macro_rules] = ACTIONS(1496),
    [anon_sym_BANG] = ACTIONS(1496),
    [anon_sym_LBRACK] = ACTIONS(1494),
    [anon_sym_PLUS] = ACTIONS(1496),
    [anon_sym_STAR] = ACTIONS(1496),
    [anon_sym_QMARK] = ACTIONS(1494),
    [anon_sym_u8] = ACTIONS(1496),
    [anon_sym_i8] = ACTIONS(1496),
    [anon_sym_u16] = ACTIONS(1496),
    [anon_sym_i16] = ACTIONS(1496),
    [anon_sym_u32] = ACTIONS(1496),
    [anon_sym_i32] = ACTIONS(1496),
    [anon_sym_u64] = ACTIONS(1496),
    [anon_sym_i64] = ACTIONS(1496),
    [anon_sym_u128] = ACTIONS(1496),
    [anon_sym_i128] = ACTIONS(1496),
    [anon_sym_isize] = ACTIONS(1496),
    [anon_sym_usize] = ACTIONS(1496),
    [anon_sym_f32] = ACTIONS(1496),
    [anon_sym_f64] = ACTIONS(1496),
    [anon_sym_bool] = ACTIONS(1496),
    [anon_sym_str] = ACTIONS(1496),
    [anon_sym_char] = ACTIONS(1496),
    [anon_sym_DASH] = ACTIONS(1496),
    [anon_sym_SLASH] = ACTIONS(1496),
    [anon_sym_PERCENT] = ACTIONS(1496),
    [anon_sym_CARET] = ACTIONS(1496),
    [anon_sym_AMP] = ACTIONS(1496),
    [anon_sym_PIPE] = ACTIONS(1496),
    [anon_sym_AMP_AMP] = ACTIONS(1494),
    [anon_sym_PIPE_PIPE] = ACTIONS(1494),
    [anon_sym_LT_LT] = ACTIONS(1496),
    [anon_sym_GT_GT] = ACTIONS(1496),
    [anon_sym_PLUS_EQ] = ACTIONS(1494),
    [anon_sym_DASH_EQ] = ACTIONS(1494),
    [anon_sym_STAR_EQ] = ACTIONS(1494),
    [anon_sym_SLASH_EQ] = ACTIONS(1494),
    [anon_sym_PERCENT_EQ] = ACTIONS(1494),
    [anon_sym_CARET_EQ] = ACTIONS(1494),
    [anon_sym_AMP_EQ] = ACTIONS(1494),
    [anon_sym_PIPE_EQ] = ACTIONS(1494),
    [anon_sym_LT_LT_EQ] = ACTIONS(1494),
    [anon_sym_GT_GT_EQ] = ACTIONS(1494),
    [anon_sym_EQ] = ACTIONS(1496),
    [anon_sym_EQ_EQ] = ACTIONS(1494),
    [anon_sym_BANG_EQ] = ACTIONS(1494),
    [anon_sym_GT] = ACTIONS(1496),
    [anon_sym_LT] = ACTIONS(1496),
    [anon_sym_GT_EQ] = ACTIONS(1494),
    [anon_sym_LT_EQ] = ACTIONS(1494),
    [anon_sym_DOT] = ACTIONS(1496),
    [anon_sym_DOT_DOT] = ACTIONS(1496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1494),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1494),
    [anon_sym_COLON_COLON] = ACTIONS(1494),
    [anon_sym_POUND] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1496),
    [anon_sym_as] = ACTIONS(1496),
    [anon_sym_async] = ACTIONS(1496),
    [anon_sym_auto] = ACTIONS(1496),
    [anon_sym_await] = ACTIONS(1496),
    [anon_sym_become] = ACTIONS(1496),
    [anon_sym_break] = ACTIONS(1496),
    [anon_sym_const] = ACTIONS(1496),
    [anon_sym_continue] = ACTIONS(1496),
    [anon_sym_default] = ACTIONS(1496),
    [anon_sym_enum] = ACTIONS(1496),
    [anon_sym_fn] = ACTIONS(1496),
    [anon_sym_for] = ACTIONS(1496),
    [anon_sym_gen] = ACTIONS(1496),
    [anon_sym_if] = ACTIONS(1496),
    [anon_sym_impl] = ACTIONS(1496),
    [anon_sym_let] = ACTIONS(1496),
    [anon_sym_loop] = ACTIONS(1496),
    [anon_sym_match] = ACTIONS(1496),
    [anon_sym_mod] = ACTIONS(1496),
    [anon_sym_pub] = ACTIONS(1496),
    [anon_sym_return] = ACTIONS(1496),
    [anon_sym_raw] = ACTIONS(1496),
    [anon_sym_static] = ACTIONS(1496),
    [anon_sym_struct] = ACTIONS(1496),
    [anon_sym_trait] = ACTIONS(1496),
    [anon_sym_type] = ACTIONS(1496),
    [anon_sym_union] = ACTIONS(1496),
    [anon_sym_unsafe] = ACTIONS(1496),
    [anon_sym_use] = ACTIONS(1496),
    [anon_sym_while] = ACTIONS(1496),
    [anon_sym_extern] = ACTIONS(1496),
    [anon_sym_safe] = ACTIONS(1496),
    [anon_sym_yield] = ACTIONS(1496),
    [anon_sym_move] = ACTIONS(1496),
    [anon_sym_try] = ACTIONS(1496),
    [sym_integer_literal] = ACTIONS(1494),
    [aux_sym_string_literal_token1] = ACTIONS(1494),
    [sym_char_literal] = ACTIONS(1494),
    [anon_sym_true] = ACTIONS(1496),
    [anon_sym_false] = ACTIONS(1496),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1496),
    [sym_super] = ACTIONS(1496),
    [sym_crate] = ACTIONS(1496),
    [aux_sym_metavariable_token1] = ACTIONS(1494),
    [sym__raw_string_literal_start] = ACTIONS(1494),
    [sym_float_literal] = ACTIONS(1494),
    [sym__outer_block_doc_comment_start] = ACTIONS(1494),
    [sym__outer_line_doc_comment_start] = ACTIONS(1494),
  },
  [STATE(449)] = {
    [sym_line_comment] = STATE(449),
    [sym_block_comment] = STATE(449),
    [ts_builtin_sym_end] = ACTIONS(1498),
    [sym_identifier] = ACTIONS(1500),
    [anon_sym_SEMI] = ACTIONS(1498),
    [anon_sym_macro] = ACTIONS(1500),
    [anon_sym_LPAREN] = ACTIONS(1498),
    [anon_sym_LBRACE] = ACTIONS(1498),
    [anon_sym_RBRACE] = ACTIONS(1498),
    [anon_sym_macro_rules] = ACTIONS(1500),
    [anon_sym_BANG] = ACTIONS(1500),
    [anon_sym_LBRACK] = ACTIONS(1498),
    [anon_sym_PLUS] = ACTIONS(1500),
    [anon_sym_STAR] = ACTIONS(1500),
    [anon_sym_QMARK] = ACTIONS(1498),
    [anon_sym_u8] = ACTIONS(1500),
    [anon_sym_i8] = ACTIONS(1500),
    [anon_sym_u16] = ACTIONS(1500),
    [anon_sym_i16] = ACTIONS(1500),
    [anon_sym_u32] = ACTIONS(1500),
    [anon_sym_i32] = ACTIONS(1500),
    [anon_sym_u64] = ACTIONS(1500),
    [anon_sym_i64] = ACTIONS(1500),
    [anon_sym_u128] = ACTIONS(1500),
    [anon_sym_i128] = ACTIONS(1500),
    [anon_sym_isize] = ACTIONS(1500),
    [anon_sym_usize] = ACTIONS(1500),
    [anon_sym_f32] = ACTIONS(1500),
    [anon_sym_f64] = ACTIONS(1500),
    [anon_sym_bool] = ACTIONS(1500),
    [anon_sym_str] = ACTIONS(1500),
    [anon_sym_char] = ACTIONS(1500),
    [anon_sym_DASH] = ACTIONS(1500),
    [anon_sym_SLASH] = ACTIONS(1500),
    [anon_sym_PERCENT] = ACTIONS(1500),
    [anon_sym_CARET] = ACTIONS(1500),
    [anon_sym_AMP] = ACTIONS(1500),
    [anon_sym_PIPE] = ACTIONS(1500),
    [anon_sym_AMP_AMP] = ACTIONS(1498),
    [anon_sym_PIPE_PIPE] = ACTIONS(1498),
    [anon_sym_LT_LT] = ACTIONS(1500),
    [anon_sym_GT_GT] = ACTIONS(1500),
    [anon_sym_PLUS_EQ] = ACTIONS(1498),
    [anon_sym_DASH_EQ] = ACTIONS(1498),
    [anon_sym_STAR_EQ] = ACTIONS(1498),
    [anon_sym_SLASH_EQ] = ACTIONS(1498),
    [anon_sym_PERCENT_EQ] = ACTIONS(1498),
    [anon_sym_CARET_EQ] = ACTIONS(1498),
    [anon_sym_AMP_EQ] = ACTIONS(1498),
    [anon_sym_PIPE_EQ] = ACTIONS(1498),
    [anon_sym_LT_LT_EQ] = ACTIONS(1498),
    [anon_sym_GT_GT_EQ] = ACTIONS(1498),
    [anon_sym_EQ] = ACTIONS(1500),
    [anon_sym_EQ_EQ] = ACTIONS(1498),
    [anon_sym_BANG_EQ] = ACTIONS(1498),
    [anon_sym_GT] = ACTIONS(1500),
    [anon_sym_LT] = ACTIONS(1500),
    [anon_sym_GT_EQ] = ACTIONS(1498),
    [anon_sym_LT_EQ] = ACTIONS(1498),
    [anon_sym_DOT] = ACTIONS(1500),
    [anon_sym_DOT_DOT] = ACTIONS(1500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1498),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1498),
    [anon_sym_COLON_COLON] = ACTIONS(1498),
    [anon_sym_POUND] = ACTIONS(1498),
    [anon_sym_SQUOTE] = ACTIONS(1500),
    [anon_sym_as] = ACTIONS(1500),
    [anon_sym_async] = ACTIONS(1500),
    [anon_sym_auto] = ACTIONS(1500),
    [anon_sym_await] = ACTIONS(1500),
    [anon_sym_become] = ACTIONS(1500),
    [anon_sym_break] = ACTIONS(1500),
    [anon_sym_const] = ACTIONS(1500),
    [anon_sym_continue] = ACTIONS(1500),
    [anon_sym_default] = ACTIONS(1500),
    [anon_sym_enum] = ACTIONS(1500),
    [anon_sym_fn] = ACTIONS(1500),
    [anon_sym_for] = ACTIONS(1500),
    [anon_sym_gen] = ACTIONS(1500),
    [anon_sym_if] = ACTIONS(1500),
    [anon_sym_impl] = ACTIONS(1500),
    [anon_sym_let] = ACTIONS(1500),
    [anon_sym_loop] = ACTIONS(1500),
    [anon_sym_match] = ACTIONS(1500),
    [anon_sym_mod] = ACTIONS(1500),
    [anon_sym_pub] = ACTIONS(1500),
    [anon_sym_return] = ACTIONS(1500),
    [anon_sym_raw] = ACTIONS(1500),
    [anon_sym_static] = ACTIONS(1500),
    [anon_sym_struct] = ACTIONS(1500),
    [anon_sym_trait] = ACTIONS(1500),
    [anon_sym_type] = ACTIONS(1500),
    [anon_sym_union] = ACTIONS(1500),
    [anon_sym_unsafe] = ACTIONS(1500),
    [anon_sym_use] = ACTIONS(1500),
    [anon_sym_while] = ACTIONS(1500),
    [anon_sym_extern] = ACTIONS(1500),
    [anon_sym_safe] = ACTIONS(1500),
    [anon_sym_yield] = ACTIONS(1500),
    [anon_sym_move] = ACTIONS(1500),
    [anon_sym_try] = ACTIONS(1500),
    [sym_integer_literal] = ACTIONS(1498),
    [aux_sym_string_literal_token1] = ACTIONS(1498),
    [sym_char_literal] = ACTIONS(1498),
    [anon_sym_true] = ACTIONS(1500),
    [anon_sym_false] = ACTIONS(1500),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1500),
    [sym_super] = ACTIONS(1500),
    [sym_crate] = ACTIONS(1500),
    [aux_sym_metavariable_token1] = ACTIONS(1498),
    [sym__raw_string_literal_start] = ACTIONS(1498),
    [sym_float_literal] = ACTIONS(1498),
    [sym__outer_block_doc_comment_start] = ACTIONS(1498),
    [sym__outer_line_doc_comment_start] = ACTIONS(1498),
  },
  [STATE(450)] = {
    [sym_line_comment] = STATE(450),
    [sym_block_comment] = STATE(450),
    [ts_builtin_sym_end] = ACTIONS(1502),
    [sym_identifier] = ACTIONS(1504),
    [anon_sym_SEMI] = ACTIONS(1506),
    [anon_sym_macro] = ACTIONS(1504),
    [anon_sym_LPAREN] = ACTIONS(1506),
    [anon_sym_LBRACE] = ACTIONS(1502),
    [anon_sym_RBRACE] = ACTIONS(1506),
    [anon_sym_macro_rules] = ACTIONS(1504),
    [anon_sym_BANG] = ACTIONS(1504),
    [anon_sym_LBRACK] = ACTIONS(1506),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1508),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(1504),
    [anon_sym_i8] = ACTIONS(1504),
    [anon_sym_u16] = ACTIONS(1504),
    [anon_sym_i16] = ACTIONS(1504),
    [anon_sym_u32] = ACTIONS(1504),
    [anon_sym_i32] = ACTIONS(1504),
    [anon_sym_u64] = ACTIONS(1504),
    [anon_sym_i64] = ACTIONS(1504),
    [anon_sym_u128] = ACTIONS(1504),
    [anon_sym_i128] = ACTIONS(1504),
    [anon_sym_isize] = ACTIONS(1504),
    [anon_sym_usize] = ACTIONS(1504),
    [anon_sym_f32] = ACTIONS(1504),
    [anon_sym_f64] = ACTIONS(1504),
    [anon_sym_bool] = ACTIONS(1504),
    [anon_sym_str] = ACTIONS(1504),
    [anon_sym_char] = ACTIONS(1504),
    [anon_sym_DASH] = ACTIONS(1508),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(1508),
    [anon_sym_PIPE] = ACTIONS(1508),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(1508),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1506),
    [anon_sym_COLON_COLON] = ACTIONS(1502),
    [anon_sym_POUND] = ACTIONS(1502),
    [anon_sym_SQUOTE] = ACTIONS(1504),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_async] = ACTIONS(1504),
    [anon_sym_auto] = ACTIONS(1504),
    [anon_sym_await] = ACTIONS(1504),
    [anon_sym_become] = ACTIONS(1504),
    [anon_sym_break] = ACTIONS(1504),
    [anon_sym_const] = ACTIONS(1504),
    [anon_sym_continue] = ACTIONS(1504),
    [anon_sym_default] = ACTIONS(1504),
    [anon_sym_enum] = ACTIONS(1504),
    [anon_sym_fn] = ACTIONS(1504),
    [anon_sym_for] = ACTIONS(1504),
    [anon_sym_gen] = ACTIONS(1504),
    [anon_sym_if] = ACTIONS(1504),
    [anon_sym_impl] = ACTIONS(1504),
    [anon_sym_let] = ACTIONS(1504),
    [anon_sym_loop] = ACTIONS(1504),
    [anon_sym_match] = ACTIONS(1504),
    [anon_sym_mod] = ACTIONS(1504),
    [anon_sym_pub] = ACTIONS(1504),
    [anon_sym_return] = ACTIONS(1504),
    [anon_sym_raw] = ACTIONS(1504),
    [anon_sym_static] = ACTIONS(1504),
    [anon_sym_struct] = ACTIONS(1504),
    [anon_sym_trait] = ACTIONS(1504),
    [anon_sym_type] = ACTIONS(1504),
    [anon_sym_union] = ACTIONS(1504),
    [anon_sym_unsafe] = ACTIONS(1504),
    [anon_sym_use] = ACTIONS(1504),
    [anon_sym_while] = ACTIONS(1504),
    [anon_sym_extern] = ACTIONS(1504),
    [anon_sym_safe] = ACTIONS(1504),
    [anon_sym_yield] = ACTIONS(1504),
    [anon_sym_move] = ACTIONS(1504),
    [anon_sym_try] = ACTIONS(1504),
    [sym_integer_literal] = ACTIONS(1502),
    [aux_sym_string_literal_token1] = ACTIONS(1502),
    [sym_char_literal] = ACTIONS(1502),
    [anon_sym_true] = ACTIONS(1504),
    [anon_sym_false] = ACTIONS(1504),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1504),
    [sym_super] = ACTIONS(1504),
    [sym_crate] = ACTIONS(1504),
    [aux_sym_metavariable_token1] = ACTIONS(1502),
    [sym__raw_string_literal_start] = ACTIONS(1502),
    [sym_float_literal] = ACTIONS(1502),
    [sym__outer_block_doc_comment_start] = ACTIONS(1502),
    [sym__outer_line_doc_comment_start] = ACTIONS(1502),
  },
  [STATE(451)] = {
    [sym_line_comment] = STATE(451),
    [sym_block_comment] = STATE(451),
    [ts_builtin_sym_end] = ACTIONS(1510),
    [sym_identifier] = ACTIONS(1512),
    [anon_sym_SEMI] = ACTIONS(1510),
    [anon_sym_macro] = ACTIONS(1512),
    [anon_sym_LPAREN] = ACTIONS(1510),
    [anon_sym_LBRACE] = ACTIONS(1510),
    [anon_sym_RBRACE] = ACTIONS(1510),
    [anon_sym_macro_rules] = ACTIONS(1512),
    [anon_sym_BANG] = ACTIONS(1512),
    [anon_sym_LBRACK] = ACTIONS(1510),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1512),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(1512),
    [anon_sym_i8] = ACTIONS(1512),
    [anon_sym_u16] = ACTIONS(1512),
    [anon_sym_i16] = ACTIONS(1512),
    [anon_sym_u32] = ACTIONS(1512),
    [anon_sym_i32] = ACTIONS(1512),
    [anon_sym_u64] = ACTIONS(1512),
    [anon_sym_i64] = ACTIONS(1512),
    [anon_sym_u128] = ACTIONS(1512),
    [anon_sym_i128] = ACTIONS(1512),
    [anon_sym_isize] = ACTIONS(1512),
    [anon_sym_usize] = ACTIONS(1512),
    [anon_sym_f32] = ACTIONS(1512),
    [anon_sym_f64] = ACTIONS(1512),
    [anon_sym_bool] = ACTIONS(1512),
    [anon_sym_str] = ACTIONS(1512),
    [anon_sym_char] = ACTIONS(1512),
    [anon_sym_DASH] = ACTIONS(1512),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(1512),
    [anon_sym_PIPE] = ACTIONS(1512),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(1512),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(1512),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1506),
    [anon_sym_COLON_COLON] = ACTIONS(1510),
    [anon_sym_POUND] = ACTIONS(1510),
    [anon_sym_SQUOTE] = ACTIONS(1512),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_async] = ACTIONS(1512),
    [anon_sym_auto] = ACTIONS(1512),
    [anon_sym_await] = ACTIONS(1512),
    [anon_sym_become] = ACTIONS(1512),
    [anon_sym_break] = ACTIONS(1512),
    [anon_sym_const] = ACTIONS(1512),
    [anon_sym_continue] = ACTIONS(1512),
    [anon_sym_default] = ACTIONS(1512),
    [anon_sym_enum] = ACTIONS(1512),
    [anon_sym_fn] = ACTIONS(1512),
    [anon_sym_for] = ACTIONS(1512),
    [anon_sym_gen] = ACTIONS(1512),
    [anon_sym_if] = ACTIONS(1512),
    [anon_sym_impl] = ACTIONS(1512),
    [anon_sym_let] = ACTIONS(1512),
    [anon_sym_loop] = ACTIONS(1512),
    [anon_sym_match] = ACTIONS(1512),
    [anon_sym_mod] = ACTIONS(1512),
    [anon_sym_pub] = ACTIONS(1512),
    [anon_sym_return] = ACTIONS(1512),
    [anon_sym_raw] = ACTIONS(1512),
    [anon_sym_static] = ACTIONS(1512),
    [anon_sym_struct] = ACTIONS(1512),
    [anon_sym_trait] = ACTIONS(1512),
    [anon_sym_type] = ACTIONS(1512),
    [anon_sym_union] = ACTIONS(1512),
    [anon_sym_unsafe] = ACTIONS(1512),
    [anon_sym_use] = ACTIONS(1512),
    [anon_sym_while] = ACTIONS(1512),
    [anon_sym_extern] = ACTIONS(1512),
    [anon_sym_safe] = ACTIONS(1512),
    [anon_sym_yield] = ACTIONS(1512),
    [anon_sym_move] = ACTIONS(1512),
    [anon_sym_try] = ACTIONS(1512),
    [sym_integer_literal] = ACTIONS(1510),
    [aux_sym_string_literal_token1] = ACTIONS(1510),
    [sym_char_literal] = ACTIONS(1510),
    [anon_sym_true] = ACTIONS(1512),
    [anon_sym_false] = ACTIONS(1512),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1512),
    [sym_super] = ACTIONS(1512),
    [sym_crate] = ACTIONS(1512),
    [aux_sym_metavariable_token1] = ACTIONS(1510),
    [sym__raw_string_literal_start] = ACTIONS(1510),
    [sym_float_literal] = ACTIONS(1510),
    [sym__outer_block_doc_comment_start] = ACTIONS(1510),
    [sym__outer_line_doc_comment_start] = ACTIONS(1510),
  },
  [STATE(452)] = {
    [sym_line_comment] = STATE(452),
    [sym_block_comment] = STATE(452),
    [ts_builtin_sym_end] = ACTIONS(1514),
    [sym_identifier] = ACTIONS(1516),
    [anon_sym_SEMI] = ACTIONS(1514),
    [anon_sym_macro] = ACTIONS(1516),
    [anon_sym_LPAREN] = ACTIONS(1514),
    [anon_sym_LBRACE] = ACTIONS(1514),
    [anon_sym_RBRACE] = ACTIONS(1514),
    [anon_sym_macro_rules] = ACTIONS(1516),
    [anon_sym_BANG] = ACTIONS(1516),
    [anon_sym_LBRACK] = ACTIONS(1514),
    [anon_sym_PLUS] = ACTIONS(1516),
    [anon_sym_STAR] = ACTIONS(1516),
    [anon_sym_QMARK] = ACTIONS(1514),
    [anon_sym_u8] = ACTIONS(1516),
    [anon_sym_i8] = ACTIONS(1516),
    [anon_sym_u16] = ACTIONS(1516),
    [anon_sym_i16] = ACTIONS(1516),
    [anon_sym_u32] = ACTIONS(1516),
    [anon_sym_i32] = ACTIONS(1516),
    [anon_sym_u64] = ACTIONS(1516),
    [anon_sym_i64] = ACTIONS(1516),
    [anon_sym_u128] = ACTIONS(1516),
    [anon_sym_i128] = ACTIONS(1516),
    [anon_sym_isize] = ACTIONS(1516),
    [anon_sym_usize] = ACTIONS(1516),
    [anon_sym_f32] = ACTIONS(1516),
    [anon_sym_f64] = ACTIONS(1516),
    [anon_sym_bool] = ACTIONS(1516),
    [anon_sym_str] = ACTIONS(1516),
    [anon_sym_char] = ACTIONS(1516),
    [anon_sym_DASH] = ACTIONS(1516),
    [anon_sym_SLASH] = ACTIONS(1516),
    [anon_sym_PERCENT] = ACTIONS(1516),
    [anon_sym_CARET] = ACTIONS(1516),
    [anon_sym_AMP] = ACTIONS(1516),
    [anon_sym_PIPE] = ACTIONS(1516),
    [anon_sym_AMP_AMP] = ACTIONS(1514),
    [anon_sym_PIPE_PIPE] = ACTIONS(1514),
    [anon_sym_LT_LT] = ACTIONS(1516),
    [anon_sym_GT_GT] = ACTIONS(1516),
    [anon_sym_PLUS_EQ] = ACTIONS(1514),
    [anon_sym_DASH_EQ] = ACTIONS(1514),
    [anon_sym_STAR_EQ] = ACTIONS(1514),
    [anon_sym_SLASH_EQ] = ACTIONS(1514),
    [anon_sym_PERCENT_EQ] = ACTIONS(1514),
    [anon_sym_CARET_EQ] = ACTIONS(1514),
    [anon_sym_AMP_EQ] = ACTIONS(1514),
    [anon_sym_PIPE_EQ] = ACTIONS(1514),
    [anon_sym_LT_LT_EQ] = ACTIONS(1514),
    [anon_sym_GT_GT_EQ] = ACTIONS(1514),
    [anon_sym_EQ] = ACTIONS(1516),
    [anon_sym_EQ_EQ] = ACTIONS(1514),
    [anon_sym_BANG_EQ] = ACTIONS(1514),
    [anon_sym_GT] = ACTIONS(1516),
    [anon_sym_LT] = ACTIONS(1516),
    [anon_sym_GT_EQ] = ACTIONS(1514),
    [anon_sym_LT_EQ] = ACTIONS(1514),
    [anon_sym_DOT] = ACTIONS(1516),
    [anon_sym_DOT_DOT] = ACTIONS(1516),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1514),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1514),
    [anon_sym_COLON_COLON] = ACTIONS(1514),
    [anon_sym_POUND] = ACTIONS(1514),
    [anon_sym_SQUOTE] = ACTIONS(1516),
    [anon_sym_as] = ACTIONS(1516),
    [anon_sym_async] = ACTIONS(1516),
    [anon_sym_auto] = ACTIONS(1516),
    [anon_sym_await] = ACTIONS(1516),
    [anon_sym_become] = ACTIONS(1516),
    [anon_sym_break] = ACTIONS(1516),
    [anon_sym_const] = ACTIONS(1516),
    [anon_sym_continue] = ACTIONS(1516),
    [anon_sym_default] = ACTIONS(1516),
    [anon_sym_enum] = ACTIONS(1516),
    [anon_sym_fn] = ACTIONS(1516),
    [anon_sym_for] = ACTIONS(1516),
    [anon_sym_gen] = ACTIONS(1516),
    [anon_sym_if] = ACTIONS(1516),
    [anon_sym_impl] = ACTIONS(1516),
    [anon_sym_let] = ACTIONS(1516),
    [anon_sym_loop] = ACTIONS(1516),
    [anon_sym_match] = ACTIONS(1516),
    [anon_sym_mod] = ACTIONS(1516),
    [anon_sym_pub] = ACTIONS(1516),
    [anon_sym_return] = ACTIONS(1516),
    [anon_sym_raw] = ACTIONS(1516),
    [anon_sym_static] = ACTIONS(1516),
    [anon_sym_struct] = ACTIONS(1516),
    [anon_sym_trait] = ACTIONS(1516),
    [anon_sym_type] = ACTIONS(1516),
    [anon_sym_union] = ACTIONS(1516),
    [anon_sym_unsafe] = ACTIONS(1516),
    [anon_sym_use] = ACTIONS(1516),
    [anon_sym_while] = ACTIONS(1516),
    [anon_sym_extern] = ACTIONS(1516),
    [anon_sym_safe] = ACTIONS(1516),
    [anon_sym_yield] = ACTIONS(1516),
    [anon_sym_move] = ACTIONS(1516),
    [anon_sym_try] = ACTIONS(1516),
    [sym_integer_literal] = ACTIONS(1514),
    [aux_sym_string_literal_token1] = ACTIONS(1514),
    [sym_char_literal] = ACTIONS(1514),
    [anon_sym_true] = ACTIONS(1516),
    [anon_sym_false] = ACTIONS(1516),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1516),
    [sym_super] = ACTIONS(1516),
    [sym_crate] = ACTIONS(1516),
    [aux_sym_metavariable_token1] = ACTIONS(1514),
    [sym__raw_string_literal_start] = ACTIONS(1514),
    [sym_float_literal] = ACTIONS(1514),
    [sym__outer_block_doc_comment_start] = ACTIONS(1514),
    [sym__outer_line_doc_comment_start] = ACTIONS(1514),
  },
  [STATE(453)] = {
    [sym_line_comment] = STATE(453),
    [sym_block_comment] = STATE(453),
    [sym_identifier] = ACTIONS(1362),
    [anon_sym_SEMI] = ACTIONS(1364),
    [anon_sym_LPAREN] = ACTIONS(1364),
    [anon_sym_RPAREN] = ACTIONS(1364),
    [anon_sym_LBRACE] = ACTIONS(1364),
    [anon_sym_RBRACE] = ACTIONS(1364),
    [anon_sym_COMMA] = ACTIONS(1364),
    [anon_sym_EQ_GT] = ACTIONS(1364),
    [anon_sym_BANG] = ACTIONS(1362),
    [anon_sym_LBRACK] = ACTIONS(1364),
    [anon_sym_RBRACK] = ACTIONS(1364),
    [anon_sym_COLON] = ACTIONS(1362),
    [anon_sym_DOLLAR] = ACTIONS(1364),
    [anon_sym_PLUS] = ACTIONS(1362),
    [anon_sym_STAR] = ACTIONS(1362),
    [anon_sym_QMARK] = ACTIONS(1364),
    [anon_sym_u8] = ACTIONS(1362),
    [anon_sym_i8] = ACTIONS(1362),
    [anon_sym_u16] = ACTIONS(1362),
    [anon_sym_i16] = ACTIONS(1362),
    [anon_sym_u32] = ACTIONS(1362),
    [anon_sym_i32] = ACTIONS(1362),
    [anon_sym_u64] = ACTIONS(1362),
    [anon_sym_i64] = ACTIONS(1362),
    [anon_sym_u128] = ACTIONS(1362),
    [anon_sym_i128] = ACTIONS(1362),
    [anon_sym_isize] = ACTIONS(1362),
    [anon_sym_usize] = ACTIONS(1362),
    [anon_sym_f32] = ACTIONS(1362),
    [anon_sym_f64] = ACTIONS(1362),
    [anon_sym_bool] = ACTIONS(1362),
    [anon_sym_str] = ACTIONS(1362),
    [anon_sym_char] = ACTIONS(1362),
    [anon_sym_DASH] = ACTIONS(1362),
    [anon_sym_SLASH] = ACTIONS(1362),
    [anon_sym_PERCENT] = ACTIONS(1362),
    [anon_sym_CARET] = ACTIONS(1362),
    [anon_sym_AMP] = ACTIONS(1362),
    [anon_sym_PIPE] = ACTIONS(1362),
    [anon_sym_AMP_AMP] = ACTIONS(1364),
    [anon_sym_PIPE_PIPE] = ACTIONS(1364),
    [anon_sym_LT_LT] = ACTIONS(1362),
    [anon_sym_GT_GT] = ACTIONS(1362),
    [anon_sym_PLUS_EQ] = ACTIONS(1364),
    [anon_sym_DASH_EQ] = ACTIONS(1364),
    [anon_sym_STAR_EQ] = ACTIONS(1364),
    [anon_sym_SLASH_EQ] = ACTIONS(1364),
    [anon_sym_PERCENT_EQ] = ACTIONS(1364),
    [anon_sym_CARET_EQ] = ACTIONS(1364),
    [anon_sym_AMP_EQ] = ACTIONS(1364),
    [anon_sym_PIPE_EQ] = ACTIONS(1364),
    [anon_sym_LT_LT_EQ] = ACTIONS(1364),
    [anon_sym_GT_GT_EQ] = ACTIONS(1364),
    [anon_sym_EQ] = ACTIONS(1362),
    [anon_sym_EQ_EQ] = ACTIONS(1364),
    [anon_sym_BANG_EQ] = ACTIONS(1364),
    [anon_sym_GT] = ACTIONS(1362),
    [anon_sym_LT] = ACTIONS(1362),
    [anon_sym_GT_EQ] = ACTIONS(1364),
    [anon_sym_LT_EQ] = ACTIONS(1364),
    [anon_sym_AT] = ACTIONS(1364),
    [anon_sym__] = ACTIONS(1362),
    [anon_sym_DOT] = ACTIONS(1362),
    [anon_sym_DOT_DOT] = ACTIONS(1362),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1364),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1364),
    [anon_sym_COLON_COLON] = ACTIONS(1364),
    [anon_sym_DASH_GT] = ACTIONS(1364),
    [anon_sym_POUND] = ACTIONS(1364),
    [anon_sym_SQUOTE] = ACTIONS(1362),
    [anon_sym_as] = ACTIONS(1362),
    [anon_sym_async] = ACTIONS(1362),
    [anon_sym_auto] = ACTIONS(1362),
    [anon_sym_await] = ACTIONS(1362),
    [anon_sym_become] = ACTIONS(1362),
    [anon_sym_break] = ACTIONS(1362),
    [anon_sym_const] = ACTIONS(1362),
    [anon_sym_continue] = ACTIONS(1362),
    [anon_sym_default] = ACTIONS(1362),
    [anon_sym_enum] = ACTIONS(1362),
    [anon_sym_fn] = ACTIONS(1362),
    [anon_sym_for] = ACTIONS(1362),
    [anon_sym_gen] = ACTIONS(1362),
    [anon_sym_if] = ACTIONS(1362),
    [anon_sym_impl] = ACTIONS(1362),
    [anon_sym_let] = ACTIONS(1362),
    [anon_sym_loop] = ACTIONS(1362),
    [anon_sym_match] = ACTIONS(1362),
    [anon_sym_mod] = ACTIONS(1362),
    [anon_sym_pub] = ACTIONS(1362),
    [anon_sym_return] = ACTIONS(1362),
    [anon_sym_raw] = ACTIONS(1362),
    [anon_sym_static] = ACTIONS(1362),
    [anon_sym_struct] = ACTIONS(1362),
    [anon_sym_trait] = ACTIONS(1362),
    [anon_sym_type] = ACTIONS(1362),
    [anon_sym_union] = ACTIONS(1362),
    [anon_sym_unsafe] = ACTIONS(1362),
    [anon_sym_use] = ACTIONS(1362),
    [anon_sym_where] = ACTIONS(1362),
    [anon_sym_while] = ACTIONS(1362),
    [sym_mutable_specifier] = ACTIONS(1362),
    [sym_integer_literal] = ACTIONS(1364),
    [aux_sym_string_literal_token1] = ACTIONS(1364),
    [sym_char_literal] = ACTIONS(1364),
    [anon_sym_true] = ACTIONS(1362),
    [anon_sym_false] = ACTIONS(1362),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1362),
    [sym_super] = ACTIONS(1362),
    [sym_crate] = ACTIONS(1362),
    [sym__raw_string_literal_start] = ACTIONS(1364),
    [sym_float_literal] = ACTIONS(1364),
  },
  [STATE(454)] = {
    [sym_line_comment] = STATE(454),
    [sym_block_comment] = STATE(454),
    [sym_identifier] = ACTIONS(1518),
    [anon_sym_SEMI] = ACTIONS(1520),
    [anon_sym_LPAREN] = ACTIONS(1520),
    [anon_sym_RPAREN] = ACTIONS(1520),
    [anon_sym_LBRACE] = ACTIONS(1520),
    [anon_sym_RBRACE] = ACTIONS(1520),
    [anon_sym_COMMA] = ACTIONS(1520),
    [anon_sym_EQ_GT] = ACTIONS(1520),
    [anon_sym_BANG] = ACTIONS(1518),
    [anon_sym_LBRACK] = ACTIONS(1520),
    [anon_sym_RBRACK] = ACTIONS(1520),
    [anon_sym_COLON] = ACTIONS(1518),
    [anon_sym_DOLLAR] = ACTIONS(1520),
    [anon_sym_PLUS] = ACTIONS(1518),
    [anon_sym_STAR] = ACTIONS(1518),
    [anon_sym_QMARK] = ACTIONS(1520),
    [anon_sym_u8] = ACTIONS(1518),
    [anon_sym_i8] = ACTIONS(1518),
    [anon_sym_u16] = ACTIONS(1518),
    [anon_sym_i16] = ACTIONS(1518),
    [anon_sym_u32] = ACTIONS(1518),
    [anon_sym_i32] = ACTIONS(1518),
    [anon_sym_u64] = ACTIONS(1518),
    [anon_sym_i64] = ACTIONS(1518),
    [anon_sym_u128] = ACTIONS(1518),
    [anon_sym_i128] = ACTIONS(1518),
    [anon_sym_isize] = ACTIONS(1518),
    [anon_sym_usize] = ACTIONS(1518),
    [anon_sym_f32] = ACTIONS(1518),
    [anon_sym_f64] = ACTIONS(1518),
    [anon_sym_bool] = ACTIONS(1518),
    [anon_sym_str] = ACTIONS(1518),
    [anon_sym_char] = ACTIONS(1518),
    [anon_sym_DASH] = ACTIONS(1518),
    [anon_sym_SLASH] = ACTIONS(1518),
    [anon_sym_PERCENT] = ACTIONS(1518),
    [anon_sym_CARET] = ACTIONS(1518),
    [anon_sym_AMP] = ACTIONS(1518),
    [anon_sym_PIPE] = ACTIONS(1518),
    [anon_sym_AMP_AMP] = ACTIONS(1520),
    [anon_sym_PIPE_PIPE] = ACTIONS(1520),
    [anon_sym_LT_LT] = ACTIONS(1518),
    [anon_sym_GT_GT] = ACTIONS(1518),
    [anon_sym_PLUS_EQ] = ACTIONS(1520),
    [anon_sym_DASH_EQ] = ACTIONS(1520),
    [anon_sym_STAR_EQ] = ACTIONS(1520),
    [anon_sym_SLASH_EQ] = ACTIONS(1520),
    [anon_sym_PERCENT_EQ] = ACTIONS(1520),
    [anon_sym_CARET_EQ] = ACTIONS(1520),
    [anon_sym_AMP_EQ] = ACTIONS(1520),
    [anon_sym_PIPE_EQ] = ACTIONS(1520),
    [anon_sym_LT_LT_EQ] = ACTIONS(1520),
    [anon_sym_GT_GT_EQ] = ACTIONS(1520),
    [anon_sym_EQ] = ACTIONS(1518),
    [anon_sym_EQ_EQ] = ACTIONS(1520),
    [anon_sym_BANG_EQ] = ACTIONS(1520),
    [anon_sym_GT] = ACTIONS(1518),
    [anon_sym_LT] = ACTIONS(1518),
    [anon_sym_GT_EQ] = ACTIONS(1520),
    [anon_sym_LT_EQ] = ACTIONS(1520),
    [anon_sym_AT] = ACTIONS(1520),
    [anon_sym__] = ACTIONS(1518),
    [anon_sym_DOT] = ACTIONS(1518),
    [anon_sym_DOT_DOT] = ACTIONS(1518),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1520),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1520),
    [anon_sym_COLON_COLON] = ACTIONS(1520),
    [anon_sym_DASH_GT] = ACTIONS(1520),
    [anon_sym_POUND] = ACTIONS(1520),
    [anon_sym_SQUOTE] = ACTIONS(1518),
    [anon_sym_as] = ACTIONS(1518),
    [anon_sym_async] = ACTIONS(1518),
    [anon_sym_auto] = ACTIONS(1518),
    [anon_sym_await] = ACTIONS(1518),
    [anon_sym_become] = ACTIONS(1518),
    [anon_sym_break] = ACTIONS(1518),
    [anon_sym_const] = ACTIONS(1518),
    [anon_sym_continue] = ACTIONS(1518),
    [anon_sym_default] = ACTIONS(1518),
    [anon_sym_enum] = ACTIONS(1518),
    [anon_sym_fn] = ACTIONS(1518),
    [anon_sym_for] = ACTIONS(1518),
    [anon_sym_gen] = ACTIONS(1518),
    [anon_sym_if] = ACTIONS(1518),
    [anon_sym_impl] = ACTIONS(1518),
    [anon_sym_let] = ACTIONS(1518),
    [anon_sym_loop] = ACTIONS(1518),
    [anon_sym_match] = ACTIONS(1518),
    [anon_sym_mod] = ACTIONS(1518),
    [anon_sym_pub] = ACTIONS(1518),
    [anon_sym_return] = ACTIONS(1518),
    [anon_sym_raw] = ACTIONS(1518),
    [anon_sym_static] = ACTIONS(1518),
    [anon_sym_struct] = ACTIONS(1518),
    [anon_sym_trait] = ACTIONS(1518),
    [anon_sym_type] = ACTIONS(1518),
    [anon_sym_union] = ACTIONS(1518),
    [anon_sym_unsafe] = ACTIONS(1518),
    [anon_sym_use] = ACTIONS(1518),
    [anon_sym_where] = ACTIONS(1518),
    [anon_sym_while] = ACTIONS(1518),
    [sym_mutable_specifier] = ACTIONS(1518),
    [sym_integer_literal] = ACTIONS(1520),
    [aux_sym_string_literal_token1] = ACTIONS(1520),
    [sym_char_literal] = ACTIONS(1520),
    [anon_sym_true] = ACTIONS(1518),
    [anon_sym_false] = ACTIONS(1518),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1518),
    [sym_super] = ACTIONS(1518),
    [sym_crate] = ACTIONS(1518),
    [sym__raw_string_literal_start] = ACTIONS(1520),
    [sym_float_literal] = ACTIONS(1520),
  },
  [STATE(455)] = {
    [sym_line_comment] = STATE(455),
    [sym_block_comment] = STATE(455),
    [sym_identifier] = ACTIONS(1522),
    [anon_sym_SEMI] = ACTIONS(1524),
    [anon_sym_LPAREN] = ACTIONS(1524),
    [anon_sym_RPAREN] = ACTIONS(1524),
    [anon_sym_LBRACE] = ACTIONS(1524),
    [anon_sym_RBRACE] = ACTIONS(1524),
    [anon_sym_COMMA] = ACTIONS(1524),
    [anon_sym_EQ_GT] = ACTIONS(1524),
    [anon_sym_BANG] = ACTIONS(1522),
    [anon_sym_LBRACK] = ACTIONS(1524),
    [anon_sym_RBRACK] = ACTIONS(1524),
    [anon_sym_COLON] = ACTIONS(1522),
    [anon_sym_DOLLAR] = ACTIONS(1524),
    [anon_sym_PLUS] = ACTIONS(1522),
    [anon_sym_STAR] = ACTIONS(1522),
    [anon_sym_QMARK] = ACTIONS(1524),
    [anon_sym_u8] = ACTIONS(1522),
    [anon_sym_i8] = ACTIONS(1522),
    [anon_sym_u16] = ACTIONS(1522),
    [anon_sym_i16] = ACTIONS(1522),
    [anon_sym_u32] = ACTIONS(1522),
    [anon_sym_i32] = ACTIONS(1522),
    [anon_sym_u64] = ACTIONS(1522),
    [anon_sym_i64] = ACTIONS(1522),
    [anon_sym_u128] = ACTIONS(1522),
    [anon_sym_i128] = ACTIONS(1522),
    [anon_sym_isize] = ACTIONS(1522),
    [anon_sym_usize] = ACTIONS(1522),
    [anon_sym_f32] = ACTIONS(1522),
    [anon_sym_f64] = ACTIONS(1522),
    [anon_sym_bool] = ACTIONS(1522),
    [anon_sym_str] = ACTIONS(1522),
    [anon_sym_char] = ACTIONS(1522),
    [anon_sym_DASH] = ACTIONS(1522),
    [anon_sym_SLASH] = ACTIONS(1522),
    [anon_sym_PERCENT] = ACTIONS(1522),
    [anon_sym_CARET] = ACTIONS(1522),
    [anon_sym_AMP] = ACTIONS(1522),
    [anon_sym_PIPE] = ACTIONS(1522),
    [anon_sym_AMP_AMP] = ACTIONS(1524),
    [anon_sym_PIPE_PIPE] = ACTIONS(1524),
    [anon_sym_LT_LT] = ACTIONS(1522),
    [anon_sym_GT_GT] = ACTIONS(1522),
    [anon_sym_PLUS_EQ] = ACTIONS(1524),
    [anon_sym_DASH_EQ] = ACTIONS(1524),
    [anon_sym_STAR_EQ] = ACTIONS(1524),
    [anon_sym_SLASH_EQ] = ACTIONS(1524),
    [anon_sym_PERCENT_EQ] = ACTIONS(1524),
    [anon_sym_CARET_EQ] = ACTIONS(1524),
    [anon_sym_AMP_EQ] = ACTIONS(1524),
    [anon_sym_PIPE_EQ] = ACTIONS(1524),
    [anon_sym_LT_LT_EQ] = ACTIONS(1524),
    [anon_sym_GT_GT_EQ] = ACTIONS(1524),
    [anon_sym_EQ] = ACTIONS(1522),
    [anon_sym_EQ_EQ] = ACTIONS(1524),
    [anon_sym_BANG_EQ] = ACTIONS(1524),
    [anon_sym_GT] = ACTIONS(1522),
    [anon_sym_LT] = ACTIONS(1522),
    [anon_sym_GT_EQ] = ACTIONS(1524),
    [anon_sym_LT_EQ] = ACTIONS(1524),
    [anon_sym_AT] = ACTIONS(1524),
    [anon_sym__] = ACTIONS(1522),
    [anon_sym_DOT] = ACTIONS(1522),
    [anon_sym_DOT_DOT] = ACTIONS(1522),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1524),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1524),
    [anon_sym_COLON_COLON] = ACTIONS(1524),
    [anon_sym_DASH_GT] = ACTIONS(1524),
    [anon_sym_POUND] = ACTIONS(1524),
    [anon_sym_SQUOTE] = ACTIONS(1522),
    [anon_sym_as] = ACTIONS(1522),
    [anon_sym_async] = ACTIONS(1522),
    [anon_sym_auto] = ACTIONS(1522),
    [anon_sym_await] = ACTIONS(1522),
    [anon_sym_become] = ACTIONS(1522),
    [anon_sym_break] = ACTIONS(1522),
    [anon_sym_const] = ACTIONS(1522),
    [anon_sym_continue] = ACTIONS(1522),
    [anon_sym_default] = ACTIONS(1522),
    [anon_sym_enum] = ACTIONS(1522),
    [anon_sym_fn] = ACTIONS(1522),
    [anon_sym_for] = ACTIONS(1522),
    [anon_sym_gen] = ACTIONS(1522),
    [anon_sym_if] = ACTIONS(1522),
    [anon_sym_impl] = ACTIONS(1522),
    [anon_sym_let] = ACTIONS(1522),
    [anon_sym_loop] = ACTIONS(1522),
    [anon_sym_match] = ACTIONS(1522),
    [anon_sym_mod] = ACTIONS(1522),
    [anon_sym_pub] = ACTIONS(1522),
    [anon_sym_return] = ACTIONS(1522),
    [anon_sym_raw] = ACTIONS(1522),
    [anon_sym_static] = ACTIONS(1522),
    [anon_sym_struct] = ACTIONS(1522),
    [anon_sym_trait] = ACTIONS(1522),
    [anon_sym_type] = ACTIONS(1522),
    [anon_sym_union] = ACTIONS(1522),
    [anon_sym_unsafe] = ACTIONS(1522),
    [anon_sym_use] = ACTIONS(1522),
    [anon_sym_where] = ACTIONS(1522),
    [anon_sym_while] = ACTIONS(1522),
    [sym_mutable_specifier] = ACTIONS(1522),
    [sym_integer_literal] = ACTIONS(1524),
    [aux_sym_string_literal_token1] = ACTIONS(1524),
    [sym_char_literal] = ACTIONS(1524),
    [anon_sym_true] = ACTIONS(1522),
    [anon_sym_false] = ACTIONS(1522),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1522),
    [sym_super] = ACTIONS(1522),
    [sym_crate] = ACTIONS(1522),
    [sym__raw_string_literal_start] = ACTIONS(1524),
    [sym_float_literal] = ACTIONS(1524),
  },
  [STATE(456)] = {
    [sym_line_comment] = STATE(456),
    [sym_block_comment] = STATE(456),
    [ts_builtin_sym_end] = ACTIONS(1526),
    [sym_identifier] = ACTIONS(1528),
    [anon_sym_SEMI] = ACTIONS(1526),
    [anon_sym_macro] = ACTIONS(1528),
    [anon_sym_LPAREN] = ACTIONS(1526),
    [anon_sym_LBRACE] = ACTIONS(1526),
    [anon_sym_RBRACE] = ACTIONS(1526),
    [anon_sym_macro_rules] = ACTIONS(1528),
    [anon_sym_BANG] = ACTIONS(1528),
    [anon_sym_LBRACK] = ACTIONS(1526),
    [anon_sym_PLUS] = ACTIONS(1528),
    [anon_sym_STAR] = ACTIONS(1528),
    [anon_sym_QMARK] = ACTIONS(1526),
    [anon_sym_u8] = ACTIONS(1528),
    [anon_sym_i8] = ACTIONS(1528),
    [anon_sym_u16] = ACTIONS(1528),
    [anon_sym_i16] = ACTIONS(1528),
    [anon_sym_u32] = ACTIONS(1528),
    [anon_sym_i32] = ACTIONS(1528),
    [anon_sym_u64] = ACTIONS(1528),
    [anon_sym_i64] = ACTIONS(1528),
    [anon_sym_u128] = ACTIONS(1528),
    [anon_sym_i128] = ACTIONS(1528),
    [anon_sym_isize] = ACTIONS(1528),
    [anon_sym_usize] = ACTIONS(1528),
    [anon_sym_f32] = ACTIONS(1528),
    [anon_sym_f64] = ACTIONS(1528),
    [anon_sym_bool] = ACTIONS(1528),
    [anon_sym_str] = ACTIONS(1528),
    [anon_sym_char] = ACTIONS(1528),
    [anon_sym_DASH] = ACTIONS(1528),
    [anon_sym_SLASH] = ACTIONS(1528),
    [anon_sym_PERCENT] = ACTIONS(1528),
    [anon_sym_CARET] = ACTIONS(1528),
    [anon_sym_AMP] = ACTIONS(1528),
    [anon_sym_PIPE] = ACTIONS(1528),
    [anon_sym_AMP_AMP] = ACTIONS(1526),
    [anon_sym_PIPE_PIPE] = ACTIONS(1526),
    [anon_sym_LT_LT] = ACTIONS(1528),
    [anon_sym_GT_GT] = ACTIONS(1528),
    [anon_sym_PLUS_EQ] = ACTIONS(1526),
    [anon_sym_DASH_EQ] = ACTIONS(1526),
    [anon_sym_STAR_EQ] = ACTIONS(1526),
    [anon_sym_SLASH_EQ] = ACTIONS(1526),
    [anon_sym_PERCENT_EQ] = ACTIONS(1526),
    [anon_sym_CARET_EQ] = ACTIONS(1526),
    [anon_sym_AMP_EQ] = ACTIONS(1526),
    [anon_sym_PIPE_EQ] = ACTIONS(1526),
    [anon_sym_LT_LT_EQ] = ACTIONS(1526),
    [anon_sym_GT_GT_EQ] = ACTIONS(1526),
    [anon_sym_EQ] = ACTIONS(1528),
    [anon_sym_EQ_EQ] = ACTIONS(1526),
    [anon_sym_BANG_EQ] = ACTIONS(1526),
    [anon_sym_GT] = ACTIONS(1528),
    [anon_sym_LT] = ACTIONS(1528),
    [anon_sym_GT_EQ] = ACTIONS(1526),
    [anon_sym_LT_EQ] = ACTIONS(1526),
    [anon_sym_DOT] = ACTIONS(1528),
    [anon_sym_DOT_DOT] = ACTIONS(1528),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1526),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1526),
    [anon_sym_COLON_COLON] = ACTIONS(1526),
    [anon_sym_POUND] = ACTIONS(1526),
    [anon_sym_SQUOTE] = ACTIONS(1528),
    [anon_sym_as] = ACTIONS(1528),
    [anon_sym_async] = ACTIONS(1528),
    [anon_sym_auto] = ACTIONS(1528),
    [anon_sym_await] = ACTIONS(1528),
    [anon_sym_become] = ACTIONS(1528),
    [anon_sym_break] = ACTIONS(1528),
    [anon_sym_const] = ACTIONS(1528),
    [anon_sym_continue] = ACTIONS(1528),
    [anon_sym_default] = ACTIONS(1528),
    [anon_sym_enum] = ACTIONS(1528),
    [anon_sym_fn] = ACTIONS(1528),
    [anon_sym_for] = ACTIONS(1528),
    [anon_sym_gen] = ACTIONS(1528),
    [anon_sym_if] = ACTIONS(1528),
    [anon_sym_impl] = ACTIONS(1528),
    [anon_sym_let] = ACTIONS(1528),
    [anon_sym_loop] = ACTIONS(1528),
    [anon_sym_match] = ACTIONS(1528),
    [anon_sym_mod] = ACTIONS(1528),
    [anon_sym_pub] = ACTIONS(1528),
    [anon_sym_return] = ACTIONS(1528),
    [anon_sym_raw] = ACTIONS(1528),
    [anon_sym_static] = ACTIONS(1528),
    [anon_sym_struct] = ACTIONS(1528),
    [anon_sym_trait] = ACTIONS(1528),
    [anon_sym_type] = ACTIONS(1528),
    [anon_sym_union] = ACTIONS(1528),
    [anon_sym_unsafe] = ACTIONS(1528),
    [anon_sym_use] = ACTIONS(1528),
    [anon_sym_while] = ACTIONS(1528),
    [anon_sym_extern] = ACTIONS(1528),
    [anon_sym_safe] = ACTIONS(1528),
    [anon_sym_yield] = ACTIONS(1528),
    [anon_sym_move] = ACTIONS(1528),
    [anon_sym_try] = ACTIONS(1528),
    [sym_integer_literal] = ACTIONS(1526),
    [aux_sym_string_literal_token1] = ACTIONS(1526),
    [sym_char_literal] = ACTIONS(1526),
    [anon_sym_true] = ACTIONS(1528),
    [anon_sym_false] = ACTIONS(1528),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1528),
    [sym_super] = ACTIONS(1528),
    [sym_crate] = ACTIONS(1528),
    [aux_sym_metavariable_token1] = ACTIONS(1526),
    [sym__raw_string_literal_start] = ACTIONS(1526),
    [sym_float_literal] = ACTIONS(1526),
    [sym__outer_block_doc_comment_start] = ACTIONS(1526),
    [sym__outer_line_doc_comment_start] = ACTIONS(1526),
  },
  [STATE(457)] = {
    [sym_line_comment] = STATE(457),
    [sym_block_comment] = STATE(457),
    [sym_identifier] = ACTIONS(1406),
    [anon_sym_SEMI] = ACTIONS(1408),
    [anon_sym_LPAREN] = ACTIONS(1408),
    [anon_sym_RPAREN] = ACTIONS(1408),
    [anon_sym_LBRACE] = ACTIONS(1408),
    [anon_sym_RBRACE] = ACTIONS(1408),
    [anon_sym_COMMA] = ACTIONS(1408),
    [anon_sym_EQ_GT] = ACTIONS(1408),
    [anon_sym_BANG] = ACTIONS(1406),
    [anon_sym_LBRACK] = ACTIONS(1408),
    [anon_sym_RBRACK] = ACTIONS(1408),
    [anon_sym_COLON] = ACTIONS(1406),
    [anon_sym_DOLLAR] = ACTIONS(1408),
    [anon_sym_PLUS] = ACTIONS(1406),
    [anon_sym_STAR] = ACTIONS(1406),
    [anon_sym_QMARK] = ACTIONS(1408),
    [anon_sym_u8] = ACTIONS(1406),
    [anon_sym_i8] = ACTIONS(1406),
    [anon_sym_u16] = ACTIONS(1406),
    [anon_sym_i16] = ACTIONS(1406),
    [anon_sym_u32] = ACTIONS(1406),
    [anon_sym_i32] = ACTIONS(1406),
    [anon_sym_u64] = ACTIONS(1406),
    [anon_sym_i64] = ACTIONS(1406),
    [anon_sym_u128] = ACTIONS(1406),
    [anon_sym_i128] = ACTIONS(1406),
    [anon_sym_isize] = ACTIONS(1406),
    [anon_sym_usize] = ACTIONS(1406),
    [anon_sym_f32] = ACTIONS(1406),
    [anon_sym_f64] = ACTIONS(1406),
    [anon_sym_bool] = ACTIONS(1406),
    [anon_sym_str] = ACTIONS(1406),
    [anon_sym_char] = ACTIONS(1406),
    [anon_sym_DASH] = ACTIONS(1406),
    [anon_sym_SLASH] = ACTIONS(1406),
    [anon_sym_PERCENT] = ACTIONS(1406),
    [anon_sym_CARET] = ACTIONS(1406),
    [anon_sym_AMP] = ACTIONS(1406),
    [anon_sym_PIPE] = ACTIONS(1406),
    [anon_sym_AMP_AMP] = ACTIONS(1408),
    [anon_sym_PIPE_PIPE] = ACTIONS(1408),
    [anon_sym_LT_LT] = ACTIONS(1406),
    [anon_sym_GT_GT] = ACTIONS(1406),
    [anon_sym_PLUS_EQ] = ACTIONS(1408),
    [anon_sym_DASH_EQ] = ACTIONS(1408),
    [anon_sym_STAR_EQ] = ACTIONS(1408),
    [anon_sym_SLASH_EQ] = ACTIONS(1408),
    [anon_sym_PERCENT_EQ] = ACTIONS(1408),
    [anon_sym_CARET_EQ] = ACTIONS(1408),
    [anon_sym_AMP_EQ] = ACTIONS(1408),
    [anon_sym_PIPE_EQ] = ACTIONS(1408),
    [anon_sym_LT_LT_EQ] = ACTIONS(1408),
    [anon_sym_GT_GT_EQ] = ACTIONS(1408),
    [anon_sym_EQ] = ACTIONS(1406),
    [anon_sym_EQ_EQ] = ACTIONS(1408),
    [anon_sym_BANG_EQ] = ACTIONS(1408),
    [anon_sym_GT] = ACTIONS(1406),
    [anon_sym_LT] = ACTIONS(1406),
    [anon_sym_GT_EQ] = ACTIONS(1408),
    [anon_sym_LT_EQ] = ACTIONS(1408),
    [anon_sym_AT] = ACTIONS(1408),
    [anon_sym__] = ACTIONS(1406),
    [anon_sym_DOT] = ACTIONS(1406),
    [anon_sym_DOT_DOT] = ACTIONS(1406),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1408),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1408),
    [anon_sym_COLON_COLON] = ACTIONS(1408),
    [anon_sym_DASH_GT] = ACTIONS(1408),
    [anon_sym_POUND] = ACTIONS(1408),
    [anon_sym_SQUOTE] = ACTIONS(1406),
    [anon_sym_as] = ACTIONS(1406),
    [anon_sym_async] = ACTIONS(1406),
    [anon_sym_auto] = ACTIONS(1406),
    [anon_sym_await] = ACTIONS(1406),
    [anon_sym_become] = ACTIONS(1406),
    [anon_sym_break] = ACTIONS(1406),
    [anon_sym_const] = ACTIONS(1406),
    [anon_sym_continue] = ACTIONS(1406),
    [anon_sym_default] = ACTIONS(1406),
    [anon_sym_enum] = ACTIONS(1406),
    [anon_sym_fn] = ACTIONS(1406),
    [anon_sym_for] = ACTIONS(1406),
    [anon_sym_gen] = ACTIONS(1406),
    [anon_sym_if] = ACTIONS(1406),
    [anon_sym_impl] = ACTIONS(1406),
    [anon_sym_let] = ACTIONS(1406),
    [anon_sym_loop] = ACTIONS(1406),
    [anon_sym_match] = ACTIONS(1406),
    [anon_sym_mod] = ACTIONS(1406),
    [anon_sym_pub] = ACTIONS(1406),
    [anon_sym_return] = ACTIONS(1406),
    [anon_sym_raw] = ACTIONS(1406),
    [anon_sym_static] = ACTIONS(1406),
    [anon_sym_struct] = ACTIONS(1406),
    [anon_sym_trait] = ACTIONS(1406),
    [anon_sym_type] = ACTIONS(1406),
    [anon_sym_union] = ACTIONS(1406),
    [anon_sym_unsafe] = ACTIONS(1406),
    [anon_sym_use] = ACTIONS(1406),
    [anon_sym_where] = ACTIONS(1406),
    [anon_sym_while] = ACTIONS(1406),
    [sym_mutable_specifier] = ACTIONS(1406),
    [sym_integer_literal] = ACTIONS(1408),
    [aux_sym_string_literal_token1] = ACTIONS(1408),
    [sym_char_literal] = ACTIONS(1408),
    [anon_sym_true] = ACTIONS(1406),
    [anon_sym_false] = ACTIONS(1406),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1406),
    [sym_super] = ACTIONS(1406),
    [sym_crate] = ACTIONS(1406),
    [sym__raw_string_literal_start] = ACTIONS(1408),
    [sym_float_literal] = ACTIONS(1408),
  },
  [STATE(458)] = {
    [sym_line_comment] = STATE(458),
    [sym_block_comment] = STATE(458),
    [sym_identifier] = ACTIONS(1450),
    [anon_sym_SEMI] = ACTIONS(1452),
    [anon_sym_LPAREN] = ACTIONS(1452),
    [anon_sym_RPAREN] = ACTIONS(1452),
    [anon_sym_LBRACE] = ACTIONS(1452),
    [anon_sym_RBRACE] = ACTIONS(1452),
    [anon_sym_COMMA] = ACTIONS(1452),
    [anon_sym_EQ_GT] = ACTIONS(1452),
    [anon_sym_BANG] = ACTIONS(1450),
    [anon_sym_LBRACK] = ACTIONS(1452),
    [anon_sym_RBRACK] = ACTIONS(1452),
    [anon_sym_COLON] = ACTIONS(1450),
    [anon_sym_DOLLAR] = ACTIONS(1452),
    [anon_sym_PLUS] = ACTIONS(1450),
    [anon_sym_STAR] = ACTIONS(1450),
    [anon_sym_QMARK] = ACTIONS(1452),
    [anon_sym_u8] = ACTIONS(1450),
    [anon_sym_i8] = ACTIONS(1450),
    [anon_sym_u16] = ACTIONS(1450),
    [anon_sym_i16] = ACTIONS(1450),
    [anon_sym_u32] = ACTIONS(1450),
    [anon_sym_i32] = ACTIONS(1450),
    [anon_sym_u64] = ACTIONS(1450),
    [anon_sym_i64] = ACTIONS(1450),
    [anon_sym_u128] = ACTIONS(1450),
    [anon_sym_i128] = ACTIONS(1450),
    [anon_sym_isize] = ACTIONS(1450),
    [anon_sym_usize] = ACTIONS(1450),
    [anon_sym_f32] = ACTIONS(1450),
    [anon_sym_f64] = ACTIONS(1450),
    [anon_sym_bool] = ACTIONS(1450),
    [anon_sym_str] = ACTIONS(1450),
    [anon_sym_char] = ACTIONS(1450),
    [anon_sym_DASH] = ACTIONS(1450),
    [anon_sym_SLASH] = ACTIONS(1450),
    [anon_sym_PERCENT] = ACTIONS(1450),
    [anon_sym_CARET] = ACTIONS(1450),
    [anon_sym_AMP] = ACTIONS(1450),
    [anon_sym_PIPE] = ACTIONS(1450),
    [anon_sym_AMP_AMP] = ACTIONS(1452),
    [anon_sym_PIPE_PIPE] = ACTIONS(1452),
    [anon_sym_LT_LT] = ACTIONS(1450),
    [anon_sym_GT_GT] = ACTIONS(1450),
    [anon_sym_PLUS_EQ] = ACTIONS(1452),
    [anon_sym_DASH_EQ] = ACTIONS(1452),
    [anon_sym_STAR_EQ] = ACTIONS(1452),
    [anon_sym_SLASH_EQ] = ACTIONS(1452),
    [anon_sym_PERCENT_EQ] = ACTIONS(1452),
    [anon_sym_CARET_EQ] = ACTIONS(1452),
    [anon_sym_AMP_EQ] = ACTIONS(1452),
    [anon_sym_PIPE_EQ] = ACTIONS(1452),
    [anon_sym_LT_LT_EQ] = ACTIONS(1452),
    [anon_sym_GT_GT_EQ] = ACTIONS(1452),
    [anon_sym_EQ] = ACTIONS(1450),
    [anon_sym_EQ_EQ] = ACTIONS(1452),
    [anon_sym_BANG_EQ] = ACTIONS(1452),
    [anon_sym_GT] = ACTIONS(1450),
    [anon_sym_LT] = ACTIONS(1450),
    [anon_sym_GT_EQ] = ACTIONS(1452),
    [anon_sym_LT_EQ] = ACTIONS(1452),
    [anon_sym_AT] = ACTIONS(1452),
    [anon_sym__] = ACTIONS(1450),
    [anon_sym_DOT] = ACTIONS(1450),
    [anon_sym_DOT_DOT] = ACTIONS(1450),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1452),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1452),
    [anon_sym_COLON_COLON] = ACTIONS(1452),
    [anon_sym_DASH_GT] = ACTIONS(1452),
    [anon_sym_POUND] = ACTIONS(1452),
    [anon_sym_SQUOTE] = ACTIONS(1450),
    [anon_sym_as] = ACTIONS(1450),
    [anon_sym_async] = ACTIONS(1450),
    [anon_sym_auto] = ACTIONS(1450),
    [anon_sym_await] = ACTIONS(1450),
    [anon_sym_become] = ACTIONS(1450),
    [anon_sym_break] = ACTIONS(1450),
    [anon_sym_const] = ACTIONS(1450),
    [anon_sym_continue] = ACTIONS(1450),
    [anon_sym_default] = ACTIONS(1450),
    [anon_sym_enum] = ACTIONS(1450),
    [anon_sym_fn] = ACTIONS(1450),
    [anon_sym_for] = ACTIONS(1450),
    [anon_sym_gen] = ACTIONS(1450),
    [anon_sym_if] = ACTIONS(1450),
    [anon_sym_impl] = ACTIONS(1450),
    [anon_sym_let] = ACTIONS(1450),
    [anon_sym_loop] = ACTIONS(1450),
    [anon_sym_match] = ACTIONS(1450),
    [anon_sym_mod] = ACTIONS(1450),
    [anon_sym_pub] = ACTIONS(1450),
    [anon_sym_return] = ACTIONS(1450),
    [anon_sym_raw] = ACTIONS(1450),
    [anon_sym_static] = ACTIONS(1450),
    [anon_sym_struct] = ACTIONS(1450),
    [anon_sym_trait] = ACTIONS(1450),
    [anon_sym_type] = ACTIONS(1450),
    [anon_sym_union] = ACTIONS(1450),
    [anon_sym_unsafe] = ACTIONS(1450),
    [anon_sym_use] = ACTIONS(1450),
    [anon_sym_where] = ACTIONS(1450),
    [anon_sym_while] = ACTIONS(1450),
    [sym_mutable_specifier] = ACTIONS(1450),
    [sym_integer_literal] = ACTIONS(1452),
    [aux_sym_string_literal_token1] = ACTIONS(1452),
    [sym_char_literal] = ACTIONS(1452),
    [anon_sym_true] = ACTIONS(1450),
    [anon_sym_false] = ACTIONS(1450),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1450),
    [sym_super] = ACTIONS(1450),
    [sym_crate] = ACTIONS(1450),
    [sym__raw_string_literal_start] = ACTIONS(1452),
    [sym_float_literal] = ACTIONS(1452),
  },
  [STATE(459)] = {
    [sym_line_comment] = STATE(459),
    [sym_block_comment] = STATE(459),
    [ts_builtin_sym_end] = ACTIONS(1530),
    [sym_identifier] = ACTIONS(1532),
    [anon_sym_SEMI] = ACTIONS(1530),
    [anon_sym_macro] = ACTIONS(1532),
    [anon_sym_LPAREN] = ACTIONS(1530),
    [anon_sym_LBRACE] = ACTIONS(1530),
    [anon_sym_RBRACE] = ACTIONS(1530),
    [anon_sym_macro_rules] = ACTIONS(1532),
    [anon_sym_BANG] = ACTIONS(1532),
    [anon_sym_LBRACK] = ACTIONS(1530),
    [anon_sym_PLUS] = ACTIONS(1532),
    [anon_sym_STAR] = ACTIONS(1532),
    [anon_sym_QMARK] = ACTIONS(1530),
    [anon_sym_u8] = ACTIONS(1532),
    [anon_sym_i8] = ACTIONS(1532),
    [anon_sym_u16] = ACTIONS(1532),
    [anon_sym_i16] = ACTIONS(1532),
    [anon_sym_u32] = ACTIONS(1532),
    [anon_sym_i32] = ACTIONS(1532),
    [anon_sym_u64] = ACTIONS(1532),
    [anon_sym_i64] = ACTIONS(1532),
    [anon_sym_u128] = ACTIONS(1532),
    [anon_sym_i128] = ACTIONS(1532),
    [anon_sym_isize] = ACTIONS(1532),
    [anon_sym_usize] = ACTIONS(1532),
    [anon_sym_f32] = ACTIONS(1532),
    [anon_sym_f64] = ACTIONS(1532),
    [anon_sym_bool] = ACTIONS(1532),
    [anon_sym_str] = ACTIONS(1532),
    [anon_sym_char] = ACTIONS(1532),
    [anon_sym_DASH] = ACTIONS(1532),
    [anon_sym_SLASH] = ACTIONS(1532),
    [anon_sym_PERCENT] = ACTIONS(1532),
    [anon_sym_CARET] = ACTIONS(1532),
    [anon_sym_AMP] = ACTIONS(1532),
    [anon_sym_PIPE] = ACTIONS(1532),
    [anon_sym_AMP_AMP] = ACTIONS(1530),
    [anon_sym_PIPE_PIPE] = ACTIONS(1530),
    [anon_sym_LT_LT] = ACTIONS(1532),
    [anon_sym_GT_GT] = ACTIONS(1532),
    [anon_sym_PLUS_EQ] = ACTIONS(1530),
    [anon_sym_DASH_EQ] = ACTIONS(1530),
    [anon_sym_STAR_EQ] = ACTIONS(1530),
    [anon_sym_SLASH_EQ] = ACTIONS(1530),
    [anon_sym_PERCENT_EQ] = ACTIONS(1530),
    [anon_sym_CARET_EQ] = ACTIONS(1530),
    [anon_sym_AMP_EQ] = ACTIONS(1530),
    [anon_sym_PIPE_EQ] = ACTIONS(1530),
    [anon_sym_LT_LT_EQ] = ACTIONS(1530),
    [anon_sym_GT_GT_EQ] = ACTIONS(1530),
    [anon_sym_EQ] = ACTIONS(1532),
    [anon_sym_EQ_EQ] = ACTIONS(1530),
    [anon_sym_BANG_EQ] = ACTIONS(1530),
    [anon_sym_GT] = ACTIONS(1532),
    [anon_sym_LT] = ACTIONS(1532),
    [anon_sym_GT_EQ] = ACTIONS(1530),
    [anon_sym_LT_EQ] = ACTIONS(1530),
    [anon_sym_DOT] = ACTIONS(1532),
    [anon_sym_DOT_DOT] = ACTIONS(1532),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1530),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1530),
    [anon_sym_COLON_COLON] = ACTIONS(1530),
    [anon_sym_POUND] = ACTIONS(1530),
    [anon_sym_SQUOTE] = ACTIONS(1532),
    [anon_sym_as] = ACTIONS(1532),
    [anon_sym_async] = ACTIONS(1532),
    [anon_sym_auto] = ACTIONS(1532),
    [anon_sym_await] = ACTIONS(1532),
    [anon_sym_become] = ACTIONS(1532),
    [anon_sym_break] = ACTIONS(1532),
    [anon_sym_const] = ACTIONS(1532),
    [anon_sym_continue] = ACTIONS(1532),
    [anon_sym_default] = ACTIONS(1532),
    [anon_sym_enum] = ACTIONS(1532),
    [anon_sym_fn] = ACTIONS(1532),
    [anon_sym_for] = ACTIONS(1532),
    [anon_sym_gen] = ACTIONS(1532),
    [anon_sym_if] = ACTIONS(1532),
    [anon_sym_impl] = ACTIONS(1532),
    [anon_sym_let] = ACTIONS(1532),
    [anon_sym_loop] = ACTIONS(1532),
    [anon_sym_match] = ACTIONS(1532),
    [anon_sym_mod] = ACTIONS(1532),
    [anon_sym_pub] = ACTIONS(1532),
    [anon_sym_return] = ACTIONS(1532),
    [anon_sym_raw] = ACTIONS(1532),
    [anon_sym_static] = ACTIONS(1532),
    [anon_sym_struct] = ACTIONS(1532),
    [anon_sym_trait] = ACTIONS(1532),
    [anon_sym_type] = ACTIONS(1532),
    [anon_sym_union] = ACTIONS(1532),
    [anon_sym_unsafe] = ACTIONS(1532),
    [anon_sym_use] = ACTIONS(1532),
    [anon_sym_while] = ACTIONS(1532),
    [anon_sym_extern] = ACTIONS(1532),
    [anon_sym_safe] = ACTIONS(1532),
    [anon_sym_yield] = ACTIONS(1532),
    [anon_sym_move] = ACTIONS(1532),
    [anon_sym_try] = ACTIONS(1532),
    [sym_integer_literal] = ACTIONS(1530),
    [aux_sym_string_literal_token1] = ACTIONS(1530),
    [sym_char_literal] = ACTIONS(1530),
    [anon_sym_true] = ACTIONS(1532),
    [anon_sym_false] = ACTIONS(1532),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1532),
    [sym_super] = ACTIONS(1532),
    [sym_crate] = ACTIONS(1532),
    [aux_sym_metavariable_token1] = ACTIONS(1530),
    [sym__raw_string_literal_start] = ACTIONS(1530),
    [sym_float_literal] = ACTIONS(1530),
    [sym__outer_block_doc_comment_start] = ACTIONS(1530),
    [sym__outer_line_doc_comment_start] = ACTIONS(1530),
  },
  [STATE(460)] = {
    [sym_line_comment] = STATE(460),
    [sym_block_comment] = STATE(460),
    [sym_identifier] = ACTIONS(1440),
    [anon_sym_SEMI] = ACTIONS(1442),
    [anon_sym_LPAREN] = ACTIONS(1442),
    [anon_sym_RPAREN] = ACTIONS(1442),
    [anon_sym_LBRACE] = ACTIONS(1442),
    [anon_sym_RBRACE] = ACTIONS(1442),
    [anon_sym_COMMA] = ACTIONS(1442),
    [anon_sym_EQ_GT] = ACTIONS(1442),
    [anon_sym_BANG] = ACTIONS(1440),
    [anon_sym_LBRACK] = ACTIONS(1442),
    [anon_sym_RBRACK] = ACTIONS(1442),
    [anon_sym_COLON] = ACTIONS(1440),
    [anon_sym_DOLLAR] = ACTIONS(1442),
    [anon_sym_PLUS] = ACTIONS(1440),
    [anon_sym_STAR] = ACTIONS(1440),
    [anon_sym_QMARK] = ACTIONS(1442),
    [anon_sym_u8] = ACTIONS(1440),
    [anon_sym_i8] = ACTIONS(1440),
    [anon_sym_u16] = ACTIONS(1440),
    [anon_sym_i16] = ACTIONS(1440),
    [anon_sym_u32] = ACTIONS(1440),
    [anon_sym_i32] = ACTIONS(1440),
    [anon_sym_u64] = ACTIONS(1440),
    [anon_sym_i64] = ACTIONS(1440),
    [anon_sym_u128] = ACTIONS(1440),
    [anon_sym_i128] = ACTIONS(1440),
    [anon_sym_isize] = ACTIONS(1440),
    [anon_sym_usize] = ACTIONS(1440),
    [anon_sym_f32] = ACTIONS(1440),
    [anon_sym_f64] = ACTIONS(1440),
    [anon_sym_bool] = ACTIONS(1440),
    [anon_sym_str] = ACTIONS(1440),
    [anon_sym_char] = ACTIONS(1440),
    [anon_sym_DASH] = ACTIONS(1440),
    [anon_sym_SLASH] = ACTIONS(1440),
    [anon_sym_PERCENT] = ACTIONS(1440),
    [anon_sym_CARET] = ACTIONS(1440),
    [anon_sym_AMP] = ACTIONS(1440),
    [anon_sym_PIPE] = ACTIONS(1440),
    [anon_sym_AMP_AMP] = ACTIONS(1442),
    [anon_sym_PIPE_PIPE] = ACTIONS(1442),
    [anon_sym_LT_LT] = ACTIONS(1440),
    [anon_sym_GT_GT] = ACTIONS(1440),
    [anon_sym_PLUS_EQ] = ACTIONS(1442),
    [anon_sym_DASH_EQ] = ACTIONS(1442),
    [anon_sym_STAR_EQ] = ACTIONS(1442),
    [anon_sym_SLASH_EQ] = ACTIONS(1442),
    [anon_sym_PERCENT_EQ] = ACTIONS(1442),
    [anon_sym_CARET_EQ] = ACTIONS(1442),
    [anon_sym_AMP_EQ] = ACTIONS(1442),
    [anon_sym_PIPE_EQ] = ACTIONS(1442),
    [anon_sym_LT_LT_EQ] = ACTIONS(1442),
    [anon_sym_GT_GT_EQ] = ACTIONS(1442),
    [anon_sym_EQ] = ACTIONS(1440),
    [anon_sym_EQ_EQ] = ACTIONS(1442),
    [anon_sym_BANG_EQ] = ACTIONS(1442),
    [anon_sym_GT] = ACTIONS(1440),
    [anon_sym_LT] = ACTIONS(1440),
    [anon_sym_GT_EQ] = ACTIONS(1442),
    [anon_sym_LT_EQ] = ACTIONS(1442),
    [anon_sym_AT] = ACTIONS(1442),
    [anon_sym__] = ACTIONS(1440),
    [anon_sym_DOT] = ACTIONS(1440),
    [anon_sym_DOT_DOT] = ACTIONS(1440),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1442),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1442),
    [anon_sym_COLON_COLON] = ACTIONS(1442),
    [anon_sym_DASH_GT] = ACTIONS(1442),
    [anon_sym_POUND] = ACTIONS(1442),
    [anon_sym_SQUOTE] = ACTIONS(1440),
    [anon_sym_as] = ACTIONS(1440),
    [anon_sym_async] = ACTIONS(1440),
    [anon_sym_auto] = ACTIONS(1440),
    [anon_sym_await] = ACTIONS(1440),
    [anon_sym_become] = ACTIONS(1440),
    [anon_sym_break] = ACTIONS(1440),
    [anon_sym_const] = ACTIONS(1440),
    [anon_sym_continue] = ACTIONS(1440),
    [anon_sym_default] = ACTIONS(1440),
    [anon_sym_enum] = ACTIONS(1440),
    [anon_sym_fn] = ACTIONS(1440),
    [anon_sym_for] = ACTIONS(1440),
    [anon_sym_gen] = ACTIONS(1440),
    [anon_sym_if] = ACTIONS(1440),
    [anon_sym_impl] = ACTIONS(1440),
    [anon_sym_let] = ACTIONS(1440),
    [anon_sym_loop] = ACTIONS(1440),
    [anon_sym_match] = ACTIONS(1440),
    [anon_sym_mod] = ACTIONS(1440),
    [anon_sym_pub] = ACTIONS(1440),
    [anon_sym_return] = ACTIONS(1440),
    [anon_sym_raw] = ACTIONS(1440),
    [anon_sym_static] = ACTIONS(1440),
    [anon_sym_struct] = ACTIONS(1440),
    [anon_sym_trait] = ACTIONS(1440),
    [anon_sym_type] = ACTIONS(1440),
    [anon_sym_union] = ACTIONS(1440),
    [anon_sym_unsafe] = ACTIONS(1440),
    [anon_sym_use] = ACTIONS(1440),
    [anon_sym_where] = ACTIONS(1440),
    [anon_sym_while] = ACTIONS(1440),
    [sym_mutable_specifier] = ACTIONS(1440),
    [sym_integer_literal] = ACTIONS(1442),
    [aux_sym_string_literal_token1] = ACTIONS(1442),
    [sym_char_literal] = ACTIONS(1442),
    [anon_sym_true] = ACTIONS(1440),
    [anon_sym_false] = ACTIONS(1440),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1440),
    [sym_super] = ACTIONS(1440),
    [sym_crate] = ACTIONS(1440),
    [sym__raw_string_literal_start] = ACTIONS(1442),
    [sym_float_literal] = ACTIONS(1442),
  },
  [STATE(461)] = {
    [sym_line_comment] = STATE(461),
    [sym_block_comment] = STATE(461),
    [sym_identifier] = ACTIONS(1496),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_LPAREN] = ACTIONS(1494),
    [anon_sym_RPAREN] = ACTIONS(1494),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_RBRACE] = ACTIONS(1494),
    [anon_sym_COMMA] = ACTIONS(1494),
    [anon_sym_EQ_GT] = ACTIONS(1494),
    [anon_sym_BANG] = ACTIONS(1496),
    [anon_sym_LBRACK] = ACTIONS(1494),
    [anon_sym_RBRACK] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1496),
    [anon_sym_DOLLAR] = ACTIONS(1494),
    [anon_sym_PLUS] = ACTIONS(1496),
    [anon_sym_STAR] = ACTIONS(1496),
    [anon_sym_QMARK] = ACTIONS(1494),
    [anon_sym_u8] = ACTIONS(1496),
    [anon_sym_i8] = ACTIONS(1496),
    [anon_sym_u16] = ACTIONS(1496),
    [anon_sym_i16] = ACTIONS(1496),
    [anon_sym_u32] = ACTIONS(1496),
    [anon_sym_i32] = ACTIONS(1496),
    [anon_sym_u64] = ACTIONS(1496),
    [anon_sym_i64] = ACTIONS(1496),
    [anon_sym_u128] = ACTIONS(1496),
    [anon_sym_i128] = ACTIONS(1496),
    [anon_sym_isize] = ACTIONS(1496),
    [anon_sym_usize] = ACTIONS(1496),
    [anon_sym_f32] = ACTIONS(1496),
    [anon_sym_f64] = ACTIONS(1496),
    [anon_sym_bool] = ACTIONS(1496),
    [anon_sym_str] = ACTIONS(1496),
    [anon_sym_char] = ACTIONS(1496),
    [anon_sym_DASH] = ACTIONS(1496),
    [anon_sym_SLASH] = ACTIONS(1496),
    [anon_sym_PERCENT] = ACTIONS(1496),
    [anon_sym_CARET] = ACTIONS(1496),
    [anon_sym_AMP] = ACTIONS(1496),
    [anon_sym_PIPE] = ACTIONS(1496),
    [anon_sym_AMP_AMP] = ACTIONS(1494),
    [anon_sym_PIPE_PIPE] = ACTIONS(1494),
    [anon_sym_LT_LT] = ACTIONS(1496),
    [anon_sym_GT_GT] = ACTIONS(1496),
    [anon_sym_PLUS_EQ] = ACTIONS(1494),
    [anon_sym_DASH_EQ] = ACTIONS(1494),
    [anon_sym_STAR_EQ] = ACTIONS(1494),
    [anon_sym_SLASH_EQ] = ACTIONS(1494),
    [anon_sym_PERCENT_EQ] = ACTIONS(1494),
    [anon_sym_CARET_EQ] = ACTIONS(1494),
    [anon_sym_AMP_EQ] = ACTIONS(1494),
    [anon_sym_PIPE_EQ] = ACTIONS(1494),
    [anon_sym_LT_LT_EQ] = ACTIONS(1494),
    [anon_sym_GT_GT_EQ] = ACTIONS(1494),
    [anon_sym_EQ] = ACTIONS(1496),
    [anon_sym_EQ_EQ] = ACTIONS(1494),
    [anon_sym_BANG_EQ] = ACTIONS(1494),
    [anon_sym_GT] = ACTIONS(1496),
    [anon_sym_LT] = ACTIONS(1496),
    [anon_sym_GT_EQ] = ACTIONS(1494),
    [anon_sym_LT_EQ] = ACTIONS(1494),
    [anon_sym_AT] = ACTIONS(1494),
    [anon_sym__] = ACTIONS(1496),
    [anon_sym_DOT] = ACTIONS(1496),
    [anon_sym_DOT_DOT] = ACTIONS(1496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1494),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1494),
    [anon_sym_COLON_COLON] = ACTIONS(1494),
    [anon_sym_DASH_GT] = ACTIONS(1494),
    [anon_sym_POUND] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1496),
    [anon_sym_as] = ACTIONS(1496),
    [anon_sym_async] = ACTIONS(1496),
    [anon_sym_auto] = ACTIONS(1496),
    [anon_sym_await] = ACTIONS(1496),
    [anon_sym_become] = ACTIONS(1496),
    [anon_sym_break] = ACTIONS(1496),
    [anon_sym_const] = ACTIONS(1496),
    [anon_sym_continue] = ACTIONS(1496),
    [anon_sym_default] = ACTIONS(1496),
    [anon_sym_enum] = ACTIONS(1496),
    [anon_sym_fn] = ACTIONS(1496),
    [anon_sym_for] = ACTIONS(1496),
    [anon_sym_gen] = ACTIONS(1496),
    [anon_sym_if] = ACTIONS(1496),
    [anon_sym_impl] = ACTIONS(1496),
    [anon_sym_let] = ACTIONS(1496),
    [anon_sym_loop] = ACTIONS(1496),
    [anon_sym_match] = ACTIONS(1496),
    [anon_sym_mod] = ACTIONS(1496),
    [anon_sym_pub] = ACTIONS(1496),
    [anon_sym_return] = ACTIONS(1496),
    [anon_sym_raw] = ACTIONS(1496),
    [anon_sym_static] = ACTIONS(1496),
    [anon_sym_struct] = ACTIONS(1496),
    [anon_sym_trait] = ACTIONS(1496),
    [anon_sym_type] = ACTIONS(1496),
    [anon_sym_union] = ACTIONS(1496),
    [anon_sym_unsafe] = ACTIONS(1496),
    [anon_sym_use] = ACTIONS(1496),
    [anon_sym_where] = ACTIONS(1496),
    [anon_sym_while] = ACTIONS(1496),
    [sym_mutable_specifier] = ACTIONS(1496),
    [sym_integer_literal] = ACTIONS(1494),
    [aux_sym_string_literal_token1] = ACTIONS(1494),
    [sym_char_literal] = ACTIONS(1494),
    [anon_sym_true] = ACTIONS(1496),
    [anon_sym_false] = ACTIONS(1496),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1496),
    [sym_super] = ACTIONS(1496),
    [sym_crate] = ACTIONS(1496),
    [sym__raw_string_literal_start] = ACTIONS(1494),
    [sym_float_literal] = ACTIONS(1494),
  },
  [STATE(462)] = {
    [sym_line_comment] = STATE(462),
    [sym_block_comment] = STATE(462),
    [sym_identifier] = ACTIONS(1534),
    [anon_sym_SEMI] = ACTIONS(1536),
    [anon_sym_LPAREN] = ACTIONS(1536),
    [anon_sym_RPAREN] = ACTIONS(1536),
    [anon_sym_LBRACE] = ACTIONS(1536),
    [anon_sym_RBRACE] = ACTIONS(1536),
    [anon_sym_COMMA] = ACTIONS(1536),
    [anon_sym_EQ_GT] = ACTIONS(1536),
    [anon_sym_BANG] = ACTIONS(1534),
    [anon_sym_LBRACK] = ACTIONS(1536),
    [anon_sym_RBRACK] = ACTIONS(1536),
    [anon_sym_COLON] = ACTIONS(1534),
    [anon_sym_DOLLAR] = ACTIONS(1536),
    [anon_sym_PLUS] = ACTIONS(1534),
    [anon_sym_STAR] = ACTIONS(1534),
    [anon_sym_QMARK] = ACTIONS(1536),
    [anon_sym_u8] = ACTIONS(1534),
    [anon_sym_i8] = ACTIONS(1534),
    [anon_sym_u16] = ACTIONS(1534),
    [anon_sym_i16] = ACTIONS(1534),
    [anon_sym_u32] = ACTIONS(1534),
    [anon_sym_i32] = ACTIONS(1534),
    [anon_sym_u64] = ACTIONS(1534),
    [anon_sym_i64] = ACTIONS(1534),
    [anon_sym_u128] = ACTIONS(1534),
    [anon_sym_i128] = ACTIONS(1534),
    [anon_sym_isize] = ACTIONS(1534),
    [anon_sym_usize] = ACTIONS(1534),
    [anon_sym_f32] = ACTIONS(1534),
    [anon_sym_f64] = ACTIONS(1534),
    [anon_sym_bool] = ACTIONS(1534),
    [anon_sym_str] = ACTIONS(1534),
    [anon_sym_char] = ACTIONS(1534),
    [anon_sym_DASH] = ACTIONS(1534),
    [anon_sym_SLASH] = ACTIONS(1534),
    [anon_sym_PERCENT] = ACTIONS(1534),
    [anon_sym_CARET] = ACTIONS(1534),
    [anon_sym_AMP] = ACTIONS(1534),
    [anon_sym_PIPE] = ACTIONS(1534),
    [anon_sym_AMP_AMP] = ACTIONS(1536),
    [anon_sym_PIPE_PIPE] = ACTIONS(1536),
    [anon_sym_LT_LT] = ACTIONS(1534),
    [anon_sym_GT_GT] = ACTIONS(1534),
    [anon_sym_PLUS_EQ] = ACTIONS(1536),
    [anon_sym_DASH_EQ] = ACTIONS(1536),
    [anon_sym_STAR_EQ] = ACTIONS(1536),
    [anon_sym_SLASH_EQ] = ACTIONS(1536),
    [anon_sym_PERCENT_EQ] = ACTIONS(1536),
    [anon_sym_CARET_EQ] = ACTIONS(1536),
    [anon_sym_AMP_EQ] = ACTIONS(1536),
    [anon_sym_PIPE_EQ] = ACTIONS(1536),
    [anon_sym_LT_LT_EQ] = ACTIONS(1536),
    [anon_sym_GT_GT_EQ] = ACTIONS(1536),
    [anon_sym_EQ] = ACTIONS(1534),
    [anon_sym_EQ_EQ] = ACTIONS(1536),
    [anon_sym_BANG_EQ] = ACTIONS(1536),
    [anon_sym_GT] = ACTIONS(1534),
    [anon_sym_LT] = ACTIONS(1534),
    [anon_sym_GT_EQ] = ACTIONS(1536),
    [anon_sym_LT_EQ] = ACTIONS(1536),
    [anon_sym_AT] = ACTIONS(1536),
    [anon_sym__] = ACTIONS(1534),
    [anon_sym_DOT] = ACTIONS(1534),
    [anon_sym_DOT_DOT] = ACTIONS(1534),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1536),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1536),
    [anon_sym_COLON_COLON] = ACTIONS(1536),
    [anon_sym_DASH_GT] = ACTIONS(1536),
    [anon_sym_POUND] = ACTIONS(1536),
    [anon_sym_SQUOTE] = ACTIONS(1534),
    [anon_sym_as] = ACTIONS(1534),
    [anon_sym_async] = ACTIONS(1534),
    [anon_sym_auto] = ACTIONS(1534),
    [anon_sym_await] = ACTIONS(1534),
    [anon_sym_become] = ACTIONS(1534),
    [anon_sym_break] = ACTIONS(1534),
    [anon_sym_const] = ACTIONS(1534),
    [anon_sym_continue] = ACTIONS(1534),
    [anon_sym_default] = ACTIONS(1534),
    [anon_sym_enum] = ACTIONS(1534),
    [anon_sym_fn] = ACTIONS(1534),
    [anon_sym_for] = ACTIONS(1534),
    [anon_sym_gen] = ACTIONS(1534),
    [anon_sym_if] = ACTIONS(1534),
    [anon_sym_impl] = ACTIONS(1534),
    [anon_sym_let] = ACTIONS(1534),
    [anon_sym_loop] = ACTIONS(1534),
    [anon_sym_match] = ACTIONS(1534),
    [anon_sym_mod] = ACTIONS(1534),
    [anon_sym_pub] = ACTIONS(1534),
    [anon_sym_return] = ACTIONS(1534),
    [anon_sym_raw] = ACTIONS(1534),
    [anon_sym_static] = ACTIONS(1534),
    [anon_sym_struct] = ACTIONS(1534),
    [anon_sym_trait] = ACTIONS(1534),
    [anon_sym_type] = ACTIONS(1534),
    [anon_sym_union] = ACTIONS(1534),
    [anon_sym_unsafe] = ACTIONS(1534),
    [anon_sym_use] = ACTIONS(1534),
    [anon_sym_where] = ACTIONS(1534),
    [anon_sym_while] = ACTIONS(1534),
    [sym_mutable_specifier] = ACTIONS(1534),
    [sym_integer_literal] = ACTIONS(1536),
    [aux_sym_string_literal_token1] = ACTIONS(1536),
    [sym_char_literal] = ACTIONS(1536),
    [anon_sym_true] = ACTIONS(1534),
    [anon_sym_false] = ACTIONS(1534),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1534),
    [sym_super] = ACTIONS(1534),
    [sym_crate] = ACTIONS(1534),
    [sym__raw_string_literal_start] = ACTIONS(1536),
    [sym_float_literal] = ACTIONS(1536),
  },
  [STATE(463)] = {
    [sym_line_comment] = STATE(463),
    [sym_block_comment] = STATE(463),
    [sym_identifier] = ACTIONS(1382),
    [anon_sym_SEMI] = ACTIONS(1384),
    [anon_sym_LPAREN] = ACTIONS(1384),
    [anon_sym_RPAREN] = ACTIONS(1384),
    [anon_sym_LBRACE] = ACTIONS(1384),
    [anon_sym_RBRACE] = ACTIONS(1384),
    [anon_sym_COMMA] = ACTIONS(1384),
    [anon_sym_EQ_GT] = ACTIONS(1384),
    [anon_sym_BANG] = ACTIONS(1382),
    [anon_sym_LBRACK] = ACTIONS(1384),
    [anon_sym_RBRACK] = ACTIONS(1384),
    [anon_sym_COLON] = ACTIONS(1382),
    [anon_sym_DOLLAR] = ACTIONS(1384),
    [anon_sym_PLUS] = ACTIONS(1382),
    [anon_sym_STAR] = ACTIONS(1382),
    [anon_sym_QMARK] = ACTIONS(1384),
    [anon_sym_u8] = ACTIONS(1382),
    [anon_sym_i8] = ACTIONS(1382),
    [anon_sym_u16] = ACTIONS(1382),
    [anon_sym_i16] = ACTIONS(1382),
    [anon_sym_u32] = ACTIONS(1382),
    [anon_sym_i32] = ACTIONS(1382),
    [anon_sym_u64] = ACTIONS(1382),
    [anon_sym_i64] = ACTIONS(1382),
    [anon_sym_u128] = ACTIONS(1382),
    [anon_sym_i128] = ACTIONS(1382),
    [anon_sym_isize] = ACTIONS(1382),
    [anon_sym_usize] = ACTIONS(1382),
    [anon_sym_f32] = ACTIONS(1382),
    [anon_sym_f64] = ACTIONS(1382),
    [anon_sym_bool] = ACTIONS(1382),
    [anon_sym_str] = ACTIONS(1382),
    [anon_sym_char] = ACTIONS(1382),
    [anon_sym_DASH] = ACTIONS(1382),
    [anon_sym_SLASH] = ACTIONS(1382),
    [anon_sym_PERCENT] = ACTIONS(1382),
    [anon_sym_CARET] = ACTIONS(1382),
    [anon_sym_AMP] = ACTIONS(1382),
    [anon_sym_PIPE] = ACTIONS(1382),
    [anon_sym_AMP_AMP] = ACTIONS(1384),
    [anon_sym_PIPE_PIPE] = ACTIONS(1384),
    [anon_sym_LT_LT] = ACTIONS(1382),
    [anon_sym_GT_GT] = ACTIONS(1382),
    [anon_sym_PLUS_EQ] = ACTIONS(1384),
    [anon_sym_DASH_EQ] = ACTIONS(1384),
    [anon_sym_STAR_EQ] = ACTIONS(1384),
    [anon_sym_SLASH_EQ] = ACTIONS(1384),
    [anon_sym_PERCENT_EQ] = ACTIONS(1384),
    [anon_sym_CARET_EQ] = ACTIONS(1384),
    [anon_sym_AMP_EQ] = ACTIONS(1384),
    [anon_sym_PIPE_EQ] = ACTIONS(1384),
    [anon_sym_LT_LT_EQ] = ACTIONS(1384),
    [anon_sym_GT_GT_EQ] = ACTIONS(1384),
    [anon_sym_EQ] = ACTIONS(1382),
    [anon_sym_EQ_EQ] = ACTIONS(1384),
    [anon_sym_BANG_EQ] = ACTIONS(1384),
    [anon_sym_GT] = ACTIONS(1382),
    [anon_sym_LT] = ACTIONS(1382),
    [anon_sym_GT_EQ] = ACTIONS(1384),
    [anon_sym_LT_EQ] = ACTIONS(1384),
    [anon_sym_AT] = ACTIONS(1384),
    [anon_sym__] = ACTIONS(1382),
    [anon_sym_DOT] = ACTIONS(1382),
    [anon_sym_DOT_DOT] = ACTIONS(1382),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1384),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1384),
    [anon_sym_COLON_COLON] = ACTIONS(1384),
    [anon_sym_DASH_GT] = ACTIONS(1384),
    [anon_sym_POUND] = ACTIONS(1384),
    [anon_sym_SQUOTE] = ACTIONS(1382),
    [anon_sym_as] = ACTIONS(1382),
    [anon_sym_async] = ACTIONS(1382),
    [anon_sym_auto] = ACTIONS(1382),
    [anon_sym_await] = ACTIONS(1382),
    [anon_sym_become] = ACTIONS(1382),
    [anon_sym_break] = ACTIONS(1382),
    [anon_sym_const] = ACTIONS(1382),
    [anon_sym_continue] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1382),
    [anon_sym_enum] = ACTIONS(1382),
    [anon_sym_fn] = ACTIONS(1382),
    [anon_sym_for] = ACTIONS(1382),
    [anon_sym_gen] = ACTIONS(1382),
    [anon_sym_if] = ACTIONS(1382),
    [anon_sym_impl] = ACTIONS(1382),
    [anon_sym_let] = ACTIONS(1382),
    [anon_sym_loop] = ACTIONS(1382),
    [anon_sym_match] = ACTIONS(1382),
    [anon_sym_mod] = ACTIONS(1382),
    [anon_sym_pub] = ACTIONS(1382),
    [anon_sym_return] = ACTIONS(1382),
    [anon_sym_raw] = ACTIONS(1382),
    [anon_sym_static] = ACTIONS(1382),
    [anon_sym_struct] = ACTIONS(1382),
    [anon_sym_trait] = ACTIONS(1382),
    [anon_sym_type] = ACTIONS(1382),
    [anon_sym_union] = ACTIONS(1382),
    [anon_sym_unsafe] = ACTIONS(1382),
    [anon_sym_use] = ACTIONS(1382),
    [anon_sym_where] = ACTIONS(1382),
    [anon_sym_while] = ACTIONS(1382),
    [sym_mutable_specifier] = ACTIONS(1382),
    [sym_integer_literal] = ACTIONS(1384),
    [aux_sym_string_literal_token1] = ACTIONS(1384),
    [sym_char_literal] = ACTIONS(1384),
    [anon_sym_true] = ACTIONS(1382),
    [anon_sym_false] = ACTIONS(1382),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1382),
    [sym_super] = ACTIONS(1382),
    [sym_crate] = ACTIONS(1382),
    [sym__raw_string_literal_start] = ACTIONS(1384),
    [sym_float_literal] = ACTIONS(1384),
  },
  [STATE(464)] = {
    [sym_line_comment] = STATE(464),
    [sym_block_comment] = STATE(464),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_SEMI] = ACTIONS(1380),
    [anon_sym_LPAREN] = ACTIONS(1380),
    [anon_sym_RPAREN] = ACTIONS(1380),
    [anon_sym_LBRACE] = ACTIONS(1380),
    [anon_sym_RBRACE] = ACTIONS(1380),
    [anon_sym_COMMA] = ACTIONS(1380),
    [anon_sym_EQ_GT] = ACTIONS(1380),
    [anon_sym_BANG] = ACTIONS(1378),
    [anon_sym_LBRACK] = ACTIONS(1380),
    [anon_sym_RBRACK] = ACTIONS(1380),
    [anon_sym_COLON] = ACTIONS(1378),
    [anon_sym_DOLLAR] = ACTIONS(1380),
    [anon_sym_PLUS] = ACTIONS(1378),
    [anon_sym_STAR] = ACTIONS(1378),
    [anon_sym_QMARK] = ACTIONS(1380),
    [anon_sym_u8] = ACTIONS(1378),
    [anon_sym_i8] = ACTIONS(1378),
    [anon_sym_u16] = ACTIONS(1378),
    [anon_sym_i16] = ACTIONS(1378),
    [anon_sym_u32] = ACTIONS(1378),
    [anon_sym_i32] = ACTIONS(1378),
    [anon_sym_u64] = ACTIONS(1378),
    [anon_sym_i64] = ACTIONS(1378),
    [anon_sym_u128] = ACTIONS(1378),
    [anon_sym_i128] = ACTIONS(1378),
    [anon_sym_isize] = ACTIONS(1378),
    [anon_sym_usize] = ACTIONS(1378),
    [anon_sym_f32] = ACTIONS(1378),
    [anon_sym_f64] = ACTIONS(1378),
    [anon_sym_bool] = ACTIONS(1378),
    [anon_sym_str] = ACTIONS(1378),
    [anon_sym_char] = ACTIONS(1378),
    [anon_sym_DASH] = ACTIONS(1378),
    [anon_sym_SLASH] = ACTIONS(1378),
    [anon_sym_PERCENT] = ACTIONS(1378),
    [anon_sym_CARET] = ACTIONS(1378),
    [anon_sym_AMP] = ACTIONS(1378),
    [anon_sym_PIPE] = ACTIONS(1378),
    [anon_sym_AMP_AMP] = ACTIONS(1380),
    [anon_sym_PIPE_PIPE] = ACTIONS(1380),
    [anon_sym_LT_LT] = ACTIONS(1378),
    [anon_sym_GT_GT] = ACTIONS(1378),
    [anon_sym_PLUS_EQ] = ACTIONS(1380),
    [anon_sym_DASH_EQ] = ACTIONS(1380),
    [anon_sym_STAR_EQ] = ACTIONS(1380),
    [anon_sym_SLASH_EQ] = ACTIONS(1380),
    [anon_sym_PERCENT_EQ] = ACTIONS(1380),
    [anon_sym_CARET_EQ] = ACTIONS(1380),
    [anon_sym_AMP_EQ] = ACTIONS(1380),
    [anon_sym_PIPE_EQ] = ACTIONS(1380),
    [anon_sym_LT_LT_EQ] = ACTIONS(1380),
    [anon_sym_GT_GT_EQ] = ACTIONS(1380),
    [anon_sym_EQ] = ACTIONS(1378),
    [anon_sym_EQ_EQ] = ACTIONS(1380),
    [anon_sym_BANG_EQ] = ACTIONS(1380),
    [anon_sym_GT] = ACTIONS(1378),
    [anon_sym_LT] = ACTIONS(1378),
    [anon_sym_GT_EQ] = ACTIONS(1380),
    [anon_sym_LT_EQ] = ACTIONS(1380),
    [anon_sym_AT] = ACTIONS(1380),
    [anon_sym__] = ACTIONS(1378),
    [anon_sym_DOT] = ACTIONS(1378),
    [anon_sym_DOT_DOT] = ACTIONS(1378),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1380),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1380),
    [anon_sym_COLON_COLON] = ACTIONS(1380),
    [anon_sym_DASH_GT] = ACTIONS(1380),
    [anon_sym_POUND] = ACTIONS(1380),
    [anon_sym_SQUOTE] = ACTIONS(1378),
    [anon_sym_as] = ACTIONS(1378),
    [anon_sym_async] = ACTIONS(1378),
    [anon_sym_auto] = ACTIONS(1378),
    [anon_sym_await] = ACTIONS(1378),
    [anon_sym_become] = ACTIONS(1378),
    [anon_sym_break] = ACTIONS(1378),
    [anon_sym_const] = ACTIONS(1378),
    [anon_sym_continue] = ACTIONS(1378),
    [anon_sym_default] = ACTIONS(1378),
    [anon_sym_enum] = ACTIONS(1378),
    [anon_sym_fn] = ACTIONS(1378),
    [anon_sym_for] = ACTIONS(1378),
    [anon_sym_gen] = ACTIONS(1378),
    [anon_sym_if] = ACTIONS(1378),
    [anon_sym_impl] = ACTIONS(1378),
    [anon_sym_let] = ACTIONS(1378),
    [anon_sym_loop] = ACTIONS(1378),
    [anon_sym_match] = ACTIONS(1378),
    [anon_sym_mod] = ACTIONS(1378),
    [anon_sym_pub] = ACTIONS(1378),
    [anon_sym_return] = ACTIONS(1378),
    [anon_sym_raw] = ACTIONS(1378),
    [anon_sym_static] = ACTIONS(1378),
    [anon_sym_struct] = ACTIONS(1378),
    [anon_sym_trait] = ACTIONS(1378),
    [anon_sym_type] = ACTIONS(1378),
    [anon_sym_union] = ACTIONS(1378),
    [anon_sym_unsafe] = ACTIONS(1378),
    [anon_sym_use] = ACTIONS(1378),
    [anon_sym_where] = ACTIONS(1378),
    [anon_sym_while] = ACTIONS(1378),
    [sym_mutable_specifier] = ACTIONS(1378),
    [sym_integer_literal] = ACTIONS(1380),
    [aux_sym_string_literal_token1] = ACTIONS(1380),
    [sym_char_literal] = ACTIONS(1380),
    [anon_sym_true] = ACTIONS(1378),
    [anon_sym_false] = ACTIONS(1378),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1378),
    [sym_super] = ACTIONS(1378),
    [sym_crate] = ACTIONS(1378),
    [sym__raw_string_literal_start] = ACTIONS(1380),
    [sym_float_literal] = ACTIONS(1380),
  },
  [STATE(465)] = {
    [sym_line_comment] = STATE(465),
    [sym_block_comment] = STATE(465),
    [sym_identifier] = ACTIONS(1454),
    [anon_sym_SEMI] = ACTIONS(1456),
    [anon_sym_LPAREN] = ACTIONS(1456),
    [anon_sym_RPAREN] = ACTIONS(1456),
    [anon_sym_LBRACE] = ACTIONS(1456),
    [anon_sym_RBRACE] = ACTIONS(1456),
    [anon_sym_COMMA] = ACTIONS(1456),
    [anon_sym_EQ_GT] = ACTIONS(1456),
    [anon_sym_BANG] = ACTIONS(1454),
    [anon_sym_LBRACK] = ACTIONS(1456),
    [anon_sym_RBRACK] = ACTIONS(1456),
    [anon_sym_COLON] = ACTIONS(1454),
    [anon_sym_DOLLAR] = ACTIONS(1456),
    [anon_sym_PLUS] = ACTIONS(1454),
    [anon_sym_STAR] = ACTIONS(1454),
    [anon_sym_QMARK] = ACTIONS(1456),
    [anon_sym_u8] = ACTIONS(1454),
    [anon_sym_i8] = ACTIONS(1454),
    [anon_sym_u16] = ACTIONS(1454),
    [anon_sym_i16] = ACTIONS(1454),
    [anon_sym_u32] = ACTIONS(1454),
    [anon_sym_i32] = ACTIONS(1454),
    [anon_sym_u64] = ACTIONS(1454),
    [anon_sym_i64] = ACTIONS(1454),
    [anon_sym_u128] = ACTIONS(1454),
    [anon_sym_i128] = ACTIONS(1454),
    [anon_sym_isize] = ACTIONS(1454),
    [anon_sym_usize] = ACTIONS(1454),
    [anon_sym_f32] = ACTIONS(1454),
    [anon_sym_f64] = ACTIONS(1454),
    [anon_sym_bool] = ACTIONS(1454),
    [anon_sym_str] = ACTIONS(1454),
    [anon_sym_char] = ACTIONS(1454),
    [anon_sym_DASH] = ACTIONS(1454),
    [anon_sym_SLASH] = ACTIONS(1454),
    [anon_sym_PERCENT] = ACTIONS(1454),
    [anon_sym_CARET] = ACTIONS(1454),
    [anon_sym_AMP] = ACTIONS(1454),
    [anon_sym_PIPE] = ACTIONS(1454),
    [anon_sym_AMP_AMP] = ACTIONS(1456),
    [anon_sym_PIPE_PIPE] = ACTIONS(1456),
    [anon_sym_LT_LT] = ACTIONS(1454),
    [anon_sym_GT_GT] = ACTIONS(1454),
    [anon_sym_PLUS_EQ] = ACTIONS(1456),
    [anon_sym_DASH_EQ] = ACTIONS(1456),
    [anon_sym_STAR_EQ] = ACTIONS(1456),
    [anon_sym_SLASH_EQ] = ACTIONS(1456),
    [anon_sym_PERCENT_EQ] = ACTIONS(1456),
    [anon_sym_CARET_EQ] = ACTIONS(1456),
    [anon_sym_AMP_EQ] = ACTIONS(1456),
    [anon_sym_PIPE_EQ] = ACTIONS(1456),
    [anon_sym_LT_LT_EQ] = ACTIONS(1456),
    [anon_sym_GT_GT_EQ] = ACTIONS(1456),
    [anon_sym_EQ] = ACTIONS(1454),
    [anon_sym_EQ_EQ] = ACTIONS(1456),
    [anon_sym_BANG_EQ] = ACTIONS(1456),
    [anon_sym_GT] = ACTIONS(1454),
    [anon_sym_LT] = ACTIONS(1454),
    [anon_sym_GT_EQ] = ACTIONS(1456),
    [anon_sym_LT_EQ] = ACTIONS(1456),
    [anon_sym_AT] = ACTIONS(1456),
    [anon_sym__] = ACTIONS(1454),
    [anon_sym_DOT] = ACTIONS(1454),
    [anon_sym_DOT_DOT] = ACTIONS(1454),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1456),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1456),
    [anon_sym_COLON_COLON] = ACTIONS(1456),
    [anon_sym_DASH_GT] = ACTIONS(1456),
    [anon_sym_POUND] = ACTIONS(1456),
    [anon_sym_SQUOTE] = ACTIONS(1454),
    [anon_sym_as] = ACTIONS(1454),
    [anon_sym_async] = ACTIONS(1454),
    [anon_sym_auto] = ACTIONS(1454),
    [anon_sym_await] = ACTIONS(1454),
    [anon_sym_become] = ACTIONS(1454),
    [anon_sym_break] = ACTIONS(1454),
    [anon_sym_const] = ACTIONS(1454),
    [anon_sym_continue] = ACTIONS(1454),
    [anon_sym_default] = ACTIONS(1454),
    [anon_sym_enum] = ACTIONS(1454),
    [anon_sym_fn] = ACTIONS(1454),
    [anon_sym_for] = ACTIONS(1454),
    [anon_sym_gen] = ACTIONS(1454),
    [anon_sym_if] = ACTIONS(1454),
    [anon_sym_impl] = ACTIONS(1454),
    [anon_sym_let] = ACTIONS(1454),
    [anon_sym_loop] = ACTIONS(1454),
    [anon_sym_match] = ACTIONS(1454),
    [anon_sym_mod] = ACTIONS(1454),
    [anon_sym_pub] = ACTIONS(1454),
    [anon_sym_return] = ACTIONS(1454),
    [anon_sym_raw] = ACTIONS(1454),
    [anon_sym_static] = ACTIONS(1454),
    [anon_sym_struct] = ACTIONS(1454),
    [anon_sym_trait] = ACTIONS(1454),
    [anon_sym_type] = ACTIONS(1454),
    [anon_sym_union] = ACTIONS(1454),
    [anon_sym_unsafe] = ACTIONS(1454),
    [anon_sym_use] = ACTIONS(1454),
    [anon_sym_where] = ACTIONS(1454),
    [anon_sym_while] = ACTIONS(1454),
    [sym_mutable_specifier] = ACTIONS(1454),
    [sym_integer_literal] = ACTIONS(1456),
    [aux_sym_string_literal_token1] = ACTIONS(1456),
    [sym_char_literal] = ACTIONS(1456),
    [anon_sym_true] = ACTIONS(1454),
    [anon_sym_false] = ACTIONS(1454),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1454),
    [sym_super] = ACTIONS(1454),
    [sym_crate] = ACTIONS(1454),
    [sym__raw_string_literal_start] = ACTIONS(1456),
    [sym_float_literal] = ACTIONS(1456),
  },
  [STATE(466)] = {
    [sym_line_comment] = STATE(466),
    [sym_block_comment] = STATE(466),
    [sym_identifier] = ACTIONS(1394),
    [anon_sym_SEMI] = ACTIONS(1396),
    [anon_sym_LPAREN] = ACTIONS(1396),
    [anon_sym_RPAREN] = ACTIONS(1396),
    [anon_sym_LBRACE] = ACTIONS(1396),
    [anon_sym_RBRACE] = ACTIONS(1396),
    [anon_sym_COMMA] = ACTIONS(1396),
    [anon_sym_EQ_GT] = ACTIONS(1396),
    [anon_sym_BANG] = ACTIONS(1394),
    [anon_sym_LBRACK] = ACTIONS(1396),
    [anon_sym_RBRACK] = ACTIONS(1396),
    [anon_sym_COLON] = ACTIONS(1394),
    [anon_sym_DOLLAR] = ACTIONS(1396),
    [anon_sym_PLUS] = ACTIONS(1394),
    [anon_sym_STAR] = ACTIONS(1394),
    [anon_sym_QMARK] = ACTIONS(1396),
    [anon_sym_u8] = ACTIONS(1394),
    [anon_sym_i8] = ACTIONS(1394),
    [anon_sym_u16] = ACTIONS(1394),
    [anon_sym_i16] = ACTIONS(1394),
    [anon_sym_u32] = ACTIONS(1394),
    [anon_sym_i32] = ACTIONS(1394),
    [anon_sym_u64] = ACTIONS(1394),
    [anon_sym_i64] = ACTIONS(1394),
    [anon_sym_u128] = ACTIONS(1394),
    [anon_sym_i128] = ACTIONS(1394),
    [anon_sym_isize] = ACTIONS(1394),
    [anon_sym_usize] = ACTIONS(1394),
    [anon_sym_f32] = ACTIONS(1394),
    [anon_sym_f64] = ACTIONS(1394),
    [anon_sym_bool] = ACTIONS(1394),
    [anon_sym_str] = ACTIONS(1394),
    [anon_sym_char] = ACTIONS(1394),
    [anon_sym_DASH] = ACTIONS(1394),
    [anon_sym_SLASH] = ACTIONS(1394),
    [anon_sym_PERCENT] = ACTIONS(1394),
    [anon_sym_CARET] = ACTIONS(1394),
    [anon_sym_AMP] = ACTIONS(1394),
    [anon_sym_PIPE] = ACTIONS(1394),
    [anon_sym_AMP_AMP] = ACTIONS(1396),
    [anon_sym_PIPE_PIPE] = ACTIONS(1396),
    [anon_sym_LT_LT] = ACTIONS(1394),
    [anon_sym_GT_GT] = ACTIONS(1394),
    [anon_sym_PLUS_EQ] = ACTIONS(1396),
    [anon_sym_DASH_EQ] = ACTIONS(1396),
    [anon_sym_STAR_EQ] = ACTIONS(1396),
    [anon_sym_SLASH_EQ] = ACTIONS(1396),
    [anon_sym_PERCENT_EQ] = ACTIONS(1396),
    [anon_sym_CARET_EQ] = ACTIONS(1396),
    [anon_sym_AMP_EQ] = ACTIONS(1396),
    [anon_sym_PIPE_EQ] = ACTIONS(1396),
    [anon_sym_LT_LT_EQ] = ACTIONS(1396),
    [anon_sym_GT_GT_EQ] = ACTIONS(1396),
    [anon_sym_EQ] = ACTIONS(1394),
    [anon_sym_EQ_EQ] = ACTIONS(1396),
    [anon_sym_BANG_EQ] = ACTIONS(1396),
    [anon_sym_GT] = ACTIONS(1394),
    [anon_sym_LT] = ACTIONS(1394),
    [anon_sym_GT_EQ] = ACTIONS(1396),
    [anon_sym_LT_EQ] = ACTIONS(1396),
    [anon_sym_AT] = ACTIONS(1396),
    [anon_sym__] = ACTIONS(1394),
    [anon_sym_DOT] = ACTIONS(1394),
    [anon_sym_DOT_DOT] = ACTIONS(1394),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1396),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1396),
    [anon_sym_COLON_COLON] = ACTIONS(1396),
    [anon_sym_DASH_GT] = ACTIONS(1396),
    [anon_sym_POUND] = ACTIONS(1396),
    [anon_sym_SQUOTE] = ACTIONS(1394),
    [anon_sym_as] = ACTIONS(1394),
    [anon_sym_async] = ACTIONS(1394),
    [anon_sym_auto] = ACTIONS(1394),
    [anon_sym_await] = ACTIONS(1394),
    [anon_sym_become] = ACTIONS(1394),
    [anon_sym_break] = ACTIONS(1394),
    [anon_sym_const] = ACTIONS(1394),
    [anon_sym_continue] = ACTIONS(1394),
    [anon_sym_default] = ACTIONS(1394),
    [anon_sym_enum] = ACTIONS(1394),
    [anon_sym_fn] = ACTIONS(1394),
    [anon_sym_for] = ACTIONS(1394),
    [anon_sym_gen] = ACTIONS(1394),
    [anon_sym_if] = ACTIONS(1394),
    [anon_sym_impl] = ACTIONS(1394),
    [anon_sym_let] = ACTIONS(1394),
    [anon_sym_loop] = ACTIONS(1394),
    [anon_sym_match] = ACTIONS(1394),
    [anon_sym_mod] = ACTIONS(1394),
    [anon_sym_pub] = ACTIONS(1394),
    [anon_sym_return] = ACTIONS(1394),
    [anon_sym_raw] = ACTIONS(1394),
    [anon_sym_static] = ACTIONS(1394),
    [anon_sym_struct] = ACTIONS(1394),
    [anon_sym_trait] = ACTIONS(1394),
    [anon_sym_type] = ACTIONS(1394),
    [anon_sym_union] = ACTIONS(1394),
    [anon_sym_unsafe] = ACTIONS(1394),
    [anon_sym_use] = ACTIONS(1394),
    [anon_sym_where] = ACTIONS(1394),
    [anon_sym_while] = ACTIONS(1394),
    [sym_mutable_specifier] = ACTIONS(1394),
    [sym_integer_literal] = ACTIONS(1396),
    [aux_sym_string_literal_token1] = ACTIONS(1396),
    [sym_char_literal] = ACTIONS(1396),
    [anon_sym_true] = ACTIONS(1394),
    [anon_sym_false] = ACTIONS(1394),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1394),
    [sym_super] = ACTIONS(1394),
    [sym_crate] = ACTIONS(1394),
    [sym__raw_string_literal_start] = ACTIONS(1396),
    [sym_float_literal] = ACTIONS(1396),
  },
  [STATE(467)] = {
    [sym_line_comment] = STATE(467),
    [sym_block_comment] = STATE(467),
    [ts_builtin_sym_end] = ACTIONS(1538),
    [sym_identifier] = ACTIONS(1540),
    [anon_sym_SEMI] = ACTIONS(1538),
    [anon_sym_macro] = ACTIONS(1540),
    [anon_sym_LPAREN] = ACTIONS(1538),
    [anon_sym_LBRACE] = ACTIONS(1538),
    [anon_sym_RBRACE] = ACTIONS(1538),
    [anon_sym_macro_rules] = ACTIONS(1540),
    [anon_sym_BANG] = ACTIONS(1540),
    [anon_sym_LBRACK] = ACTIONS(1538),
    [anon_sym_PLUS] = ACTIONS(1540),
    [anon_sym_STAR] = ACTIONS(1540),
    [anon_sym_QMARK] = ACTIONS(1538),
    [anon_sym_u8] = ACTIONS(1540),
    [anon_sym_i8] = ACTIONS(1540),
    [anon_sym_u16] = ACTIONS(1540),
    [anon_sym_i16] = ACTIONS(1540),
    [anon_sym_u32] = ACTIONS(1540),
    [anon_sym_i32] = ACTIONS(1540),
    [anon_sym_u64] = ACTIONS(1540),
    [anon_sym_i64] = ACTIONS(1540),
    [anon_sym_u128] = ACTIONS(1540),
    [anon_sym_i128] = ACTIONS(1540),
    [anon_sym_isize] = ACTIONS(1540),
    [anon_sym_usize] = ACTIONS(1540),
    [anon_sym_f32] = ACTIONS(1540),
    [anon_sym_f64] = ACTIONS(1540),
    [anon_sym_bool] = ACTIONS(1540),
    [anon_sym_str] = ACTIONS(1540),
    [anon_sym_char] = ACTIONS(1540),
    [anon_sym_DASH] = ACTIONS(1540),
    [anon_sym_SLASH] = ACTIONS(1540),
    [anon_sym_PERCENT] = ACTIONS(1540),
    [anon_sym_CARET] = ACTIONS(1540),
    [anon_sym_AMP] = ACTIONS(1540),
    [anon_sym_PIPE] = ACTIONS(1540),
    [anon_sym_AMP_AMP] = ACTIONS(1538),
    [anon_sym_PIPE_PIPE] = ACTIONS(1538),
    [anon_sym_LT_LT] = ACTIONS(1540),
    [anon_sym_GT_GT] = ACTIONS(1540),
    [anon_sym_PLUS_EQ] = ACTIONS(1538),
    [anon_sym_DASH_EQ] = ACTIONS(1538),
    [anon_sym_STAR_EQ] = ACTIONS(1538),
    [anon_sym_SLASH_EQ] = ACTIONS(1538),
    [anon_sym_PERCENT_EQ] = ACTIONS(1538),
    [anon_sym_CARET_EQ] = ACTIONS(1538),
    [anon_sym_AMP_EQ] = ACTIONS(1538),
    [anon_sym_PIPE_EQ] = ACTIONS(1538),
    [anon_sym_LT_LT_EQ] = ACTIONS(1538),
    [anon_sym_GT_GT_EQ] = ACTIONS(1538),
    [anon_sym_EQ] = ACTIONS(1540),
    [anon_sym_EQ_EQ] = ACTIONS(1538),
    [anon_sym_BANG_EQ] = ACTIONS(1538),
    [anon_sym_GT] = ACTIONS(1540),
    [anon_sym_LT] = ACTIONS(1540),
    [anon_sym_GT_EQ] = ACTIONS(1538),
    [anon_sym_LT_EQ] = ACTIONS(1538),
    [anon_sym_DOT] = ACTIONS(1540),
    [anon_sym_DOT_DOT] = ACTIONS(1540),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1538),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1538),
    [anon_sym_COLON_COLON] = ACTIONS(1538),
    [anon_sym_POUND] = ACTIONS(1538),
    [anon_sym_SQUOTE] = ACTIONS(1540),
    [anon_sym_as] = ACTIONS(1540),
    [anon_sym_async] = ACTIONS(1540),
    [anon_sym_auto] = ACTIONS(1540),
    [anon_sym_await] = ACTIONS(1540),
    [anon_sym_become] = ACTIONS(1540),
    [anon_sym_break] = ACTIONS(1540),
    [anon_sym_const] = ACTIONS(1540),
    [anon_sym_continue] = ACTIONS(1540),
    [anon_sym_default] = ACTIONS(1540),
    [anon_sym_enum] = ACTIONS(1540),
    [anon_sym_fn] = ACTIONS(1540),
    [anon_sym_for] = ACTIONS(1540),
    [anon_sym_gen] = ACTIONS(1540),
    [anon_sym_if] = ACTIONS(1540),
    [anon_sym_impl] = ACTIONS(1540),
    [anon_sym_let] = ACTIONS(1540),
    [anon_sym_loop] = ACTIONS(1540),
    [anon_sym_match] = ACTIONS(1540),
    [anon_sym_mod] = ACTIONS(1540),
    [anon_sym_pub] = ACTIONS(1540),
    [anon_sym_return] = ACTIONS(1540),
    [anon_sym_raw] = ACTIONS(1540),
    [anon_sym_static] = ACTIONS(1540),
    [anon_sym_struct] = ACTIONS(1540),
    [anon_sym_trait] = ACTIONS(1540),
    [anon_sym_type] = ACTIONS(1540),
    [anon_sym_union] = ACTIONS(1540),
    [anon_sym_unsafe] = ACTIONS(1540),
    [anon_sym_use] = ACTIONS(1540),
    [anon_sym_while] = ACTIONS(1540),
    [anon_sym_extern] = ACTIONS(1540),
    [anon_sym_safe] = ACTIONS(1540),
    [anon_sym_yield] = ACTIONS(1540),
    [anon_sym_move] = ACTIONS(1540),
    [anon_sym_try] = ACTIONS(1540),
    [sym_integer_literal] = ACTIONS(1538),
    [aux_sym_string_literal_token1] = ACTIONS(1538),
    [sym_char_literal] = ACTIONS(1538),
    [anon_sym_true] = ACTIONS(1540),
    [anon_sym_false] = ACTIONS(1540),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1540),
    [sym_super] = ACTIONS(1540),
    [sym_crate] = ACTIONS(1540),
    [aux_sym_metavariable_token1] = ACTIONS(1538),
    [sym__raw_string_literal_start] = ACTIONS(1538),
    [sym_float_literal] = ACTIONS(1538),
    [sym__outer_block_doc_comment_start] = ACTIONS(1538),
    [sym__outer_line_doc_comment_start] = ACTIONS(1538),
  },
  [STATE(468)] = {
    [sym_line_comment] = STATE(468),
    [sym_block_comment] = STATE(468),
    [ts_builtin_sym_end] = ACTIONS(1542),
    [sym_identifier] = ACTIONS(1544),
    [anon_sym_SEMI] = ACTIONS(1542),
    [anon_sym_macro] = ACTIONS(1544),
    [anon_sym_LPAREN] = ACTIONS(1542),
    [anon_sym_LBRACE] = ACTIONS(1542),
    [anon_sym_RBRACE] = ACTIONS(1542),
    [anon_sym_macro_rules] = ACTIONS(1544),
    [anon_sym_BANG] = ACTIONS(1544),
    [anon_sym_LBRACK] = ACTIONS(1542),
    [anon_sym_PLUS] = ACTIONS(1544),
    [anon_sym_STAR] = ACTIONS(1544),
    [anon_sym_QMARK] = ACTIONS(1542),
    [anon_sym_u8] = ACTIONS(1544),
    [anon_sym_i8] = ACTIONS(1544),
    [anon_sym_u16] = ACTIONS(1544),
    [anon_sym_i16] = ACTIONS(1544),
    [anon_sym_u32] = ACTIONS(1544),
    [anon_sym_i32] = ACTIONS(1544),
    [anon_sym_u64] = ACTIONS(1544),
    [anon_sym_i64] = ACTIONS(1544),
    [anon_sym_u128] = ACTIONS(1544),
    [anon_sym_i128] = ACTIONS(1544),
    [anon_sym_isize] = ACTIONS(1544),
    [anon_sym_usize] = ACTIONS(1544),
    [anon_sym_f32] = ACTIONS(1544),
    [anon_sym_f64] = ACTIONS(1544),
    [anon_sym_bool] = ACTIONS(1544),
    [anon_sym_str] = ACTIONS(1544),
    [anon_sym_char] = ACTIONS(1544),
    [anon_sym_DASH] = ACTIONS(1544),
    [anon_sym_SLASH] = ACTIONS(1544),
    [anon_sym_PERCENT] = ACTIONS(1544),
    [anon_sym_CARET] = ACTIONS(1544),
    [anon_sym_AMP] = ACTIONS(1544),
    [anon_sym_PIPE] = ACTIONS(1544),
    [anon_sym_AMP_AMP] = ACTIONS(1542),
    [anon_sym_PIPE_PIPE] = ACTIONS(1542),
    [anon_sym_LT_LT] = ACTIONS(1544),
    [anon_sym_GT_GT] = ACTIONS(1544),
    [anon_sym_PLUS_EQ] = ACTIONS(1542),
    [anon_sym_DASH_EQ] = ACTIONS(1542),
    [anon_sym_STAR_EQ] = ACTIONS(1542),
    [anon_sym_SLASH_EQ] = ACTIONS(1542),
    [anon_sym_PERCENT_EQ] = ACTIONS(1542),
    [anon_sym_CARET_EQ] = ACTIONS(1542),
    [anon_sym_AMP_EQ] = ACTIONS(1542),
    [anon_sym_PIPE_EQ] = ACTIONS(1542),
    [anon_sym_LT_LT_EQ] = ACTIONS(1542),
    [anon_sym_GT_GT_EQ] = ACTIONS(1542),
    [anon_sym_EQ] = ACTIONS(1544),
    [anon_sym_EQ_EQ] = ACTIONS(1542),
    [anon_sym_BANG_EQ] = ACTIONS(1542),
    [anon_sym_GT] = ACTIONS(1544),
    [anon_sym_LT] = ACTIONS(1544),
    [anon_sym_GT_EQ] = ACTIONS(1542),
    [anon_sym_LT_EQ] = ACTIONS(1542),
    [anon_sym_DOT] = ACTIONS(1544),
    [anon_sym_DOT_DOT] = ACTIONS(1544),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1542),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1542),
    [anon_sym_COLON_COLON] = ACTIONS(1542),
    [anon_sym_POUND] = ACTIONS(1542),
    [anon_sym_SQUOTE] = ACTIONS(1544),
    [anon_sym_as] = ACTIONS(1544),
    [anon_sym_async] = ACTIONS(1544),
    [anon_sym_auto] = ACTIONS(1544),
    [anon_sym_await] = ACTIONS(1544),
    [anon_sym_become] = ACTIONS(1544),
    [anon_sym_break] = ACTIONS(1544),
    [anon_sym_const] = ACTIONS(1544),
    [anon_sym_continue] = ACTIONS(1544),
    [anon_sym_default] = ACTIONS(1544),
    [anon_sym_enum] = ACTIONS(1544),
    [anon_sym_fn] = ACTIONS(1544),
    [anon_sym_for] = ACTIONS(1544),
    [anon_sym_gen] = ACTIONS(1544),
    [anon_sym_if] = ACTIONS(1544),
    [anon_sym_impl] = ACTIONS(1544),
    [anon_sym_let] = ACTIONS(1544),
    [anon_sym_loop] = ACTIONS(1544),
    [anon_sym_match] = ACTIONS(1544),
    [anon_sym_mod] = ACTIONS(1544),
    [anon_sym_pub] = ACTIONS(1544),
    [anon_sym_return] = ACTIONS(1544),
    [anon_sym_raw] = ACTIONS(1544),
    [anon_sym_static] = ACTIONS(1544),
    [anon_sym_struct] = ACTIONS(1544),
    [anon_sym_trait] = ACTIONS(1544),
    [anon_sym_type] = ACTIONS(1544),
    [anon_sym_union] = ACTIONS(1544),
    [anon_sym_unsafe] = ACTIONS(1544),
    [anon_sym_use] = ACTIONS(1544),
    [anon_sym_while] = ACTIONS(1544),
    [anon_sym_extern] = ACTIONS(1544),
    [anon_sym_safe] = ACTIONS(1544),
    [anon_sym_yield] = ACTIONS(1544),
    [anon_sym_move] = ACTIONS(1544),
    [anon_sym_try] = ACTIONS(1544),
    [sym_integer_literal] = ACTIONS(1542),
    [aux_sym_string_literal_token1] = ACTIONS(1542),
    [sym_char_literal] = ACTIONS(1542),
    [anon_sym_true] = ACTIONS(1544),
    [anon_sym_false] = ACTIONS(1544),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1544),
    [sym_super] = ACTIONS(1544),
    [sym_crate] = ACTIONS(1544),
    [aux_sym_metavariable_token1] = ACTIONS(1542),
    [sym__raw_string_literal_start] = ACTIONS(1542),
    [sym_float_literal] = ACTIONS(1542),
    [sym__outer_block_doc_comment_start] = ACTIONS(1542),
    [sym__outer_line_doc_comment_start] = ACTIONS(1542),
  },
  [STATE(469)] = {
    [sym_line_comment] = STATE(469),
    [sym_block_comment] = STATE(469),
    [ts_builtin_sym_end] = ACTIONS(1546),
    [sym_identifier] = ACTIONS(1548),
    [anon_sym_SEMI] = ACTIONS(1546),
    [anon_sym_macro] = ACTIONS(1548),
    [anon_sym_LPAREN] = ACTIONS(1546),
    [anon_sym_LBRACE] = ACTIONS(1546),
    [anon_sym_RBRACE] = ACTIONS(1546),
    [anon_sym_macro_rules] = ACTIONS(1548),
    [anon_sym_BANG] = ACTIONS(1548),
    [anon_sym_LBRACK] = ACTIONS(1546),
    [anon_sym_PLUS] = ACTIONS(1548),
    [anon_sym_STAR] = ACTIONS(1548),
    [anon_sym_QMARK] = ACTIONS(1546),
    [anon_sym_u8] = ACTIONS(1548),
    [anon_sym_i8] = ACTIONS(1548),
    [anon_sym_u16] = ACTIONS(1548),
    [anon_sym_i16] = ACTIONS(1548),
    [anon_sym_u32] = ACTIONS(1548),
    [anon_sym_i32] = ACTIONS(1548),
    [anon_sym_u64] = ACTIONS(1548),
    [anon_sym_i64] = ACTIONS(1548),
    [anon_sym_u128] = ACTIONS(1548),
    [anon_sym_i128] = ACTIONS(1548),
    [anon_sym_isize] = ACTIONS(1548),
    [anon_sym_usize] = ACTIONS(1548),
    [anon_sym_f32] = ACTIONS(1548),
    [anon_sym_f64] = ACTIONS(1548),
    [anon_sym_bool] = ACTIONS(1548),
    [anon_sym_str] = ACTIONS(1548),
    [anon_sym_char] = ACTIONS(1548),
    [anon_sym_DASH] = ACTIONS(1548),
    [anon_sym_SLASH] = ACTIONS(1548),
    [anon_sym_PERCENT] = ACTIONS(1548),
    [anon_sym_CARET] = ACTIONS(1548),
    [anon_sym_AMP] = ACTIONS(1548),
    [anon_sym_PIPE] = ACTIONS(1548),
    [anon_sym_AMP_AMP] = ACTIONS(1546),
    [anon_sym_PIPE_PIPE] = ACTIONS(1546),
    [anon_sym_LT_LT] = ACTIONS(1548),
    [anon_sym_GT_GT] = ACTIONS(1548),
    [anon_sym_PLUS_EQ] = ACTIONS(1546),
    [anon_sym_DASH_EQ] = ACTIONS(1546),
    [anon_sym_STAR_EQ] = ACTIONS(1546),
    [anon_sym_SLASH_EQ] = ACTIONS(1546),
    [anon_sym_PERCENT_EQ] = ACTIONS(1546),
    [anon_sym_CARET_EQ] = ACTIONS(1546),
    [anon_sym_AMP_EQ] = ACTIONS(1546),
    [anon_sym_PIPE_EQ] = ACTIONS(1546),
    [anon_sym_LT_LT_EQ] = ACTIONS(1546),
    [anon_sym_GT_GT_EQ] = ACTIONS(1546),
    [anon_sym_EQ] = ACTIONS(1548),
    [anon_sym_EQ_EQ] = ACTIONS(1546),
    [anon_sym_BANG_EQ] = ACTIONS(1546),
    [anon_sym_GT] = ACTIONS(1548),
    [anon_sym_LT] = ACTIONS(1548),
    [anon_sym_GT_EQ] = ACTIONS(1546),
    [anon_sym_LT_EQ] = ACTIONS(1546),
    [anon_sym_DOT] = ACTIONS(1548),
    [anon_sym_DOT_DOT] = ACTIONS(1548),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1546),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1546),
    [anon_sym_COLON_COLON] = ACTIONS(1546),
    [anon_sym_POUND] = ACTIONS(1546),
    [anon_sym_SQUOTE] = ACTIONS(1548),
    [anon_sym_as] = ACTIONS(1548),
    [anon_sym_async] = ACTIONS(1548),
    [anon_sym_auto] = ACTIONS(1548),
    [anon_sym_await] = ACTIONS(1548),
    [anon_sym_become] = ACTIONS(1548),
    [anon_sym_break] = ACTIONS(1548),
    [anon_sym_const] = ACTIONS(1548),
    [anon_sym_continue] = ACTIONS(1548),
    [anon_sym_default] = ACTIONS(1548),
    [anon_sym_enum] = ACTIONS(1548),
    [anon_sym_fn] = ACTIONS(1548),
    [anon_sym_for] = ACTIONS(1548),
    [anon_sym_gen] = ACTIONS(1548),
    [anon_sym_if] = ACTIONS(1548),
    [anon_sym_impl] = ACTIONS(1548),
    [anon_sym_let] = ACTIONS(1548),
    [anon_sym_loop] = ACTIONS(1548),
    [anon_sym_match] = ACTIONS(1548),
    [anon_sym_mod] = ACTIONS(1548),
    [anon_sym_pub] = ACTIONS(1548),
    [anon_sym_return] = ACTIONS(1548),
    [anon_sym_raw] = ACTIONS(1548),
    [anon_sym_static] = ACTIONS(1548),
    [anon_sym_struct] = ACTIONS(1548),
    [anon_sym_trait] = ACTIONS(1548),
    [anon_sym_type] = ACTIONS(1548),
    [anon_sym_union] = ACTIONS(1548),
    [anon_sym_unsafe] = ACTIONS(1548),
    [anon_sym_use] = ACTIONS(1548),
    [anon_sym_while] = ACTIONS(1548),
    [anon_sym_extern] = ACTIONS(1548),
    [anon_sym_safe] = ACTIONS(1548),
    [anon_sym_yield] = ACTIONS(1548),
    [anon_sym_move] = ACTIONS(1548),
    [anon_sym_try] = ACTIONS(1548),
    [sym_integer_literal] = ACTIONS(1546),
    [aux_sym_string_literal_token1] = ACTIONS(1546),
    [sym_char_literal] = ACTIONS(1546),
    [anon_sym_true] = ACTIONS(1548),
    [anon_sym_false] = ACTIONS(1548),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1548),
    [sym_super] = ACTIONS(1548),
    [sym_crate] = ACTIONS(1548),
    [aux_sym_metavariable_token1] = ACTIONS(1546),
    [sym__raw_string_literal_start] = ACTIONS(1546),
    [sym_float_literal] = ACTIONS(1546),
    [sym__outer_block_doc_comment_start] = ACTIONS(1546),
    [sym__outer_line_doc_comment_start] = ACTIONS(1546),
  },
  [STATE(470)] = {
    [sym_line_comment] = STATE(470),
    [sym_block_comment] = STATE(470),
    [ts_builtin_sym_end] = ACTIONS(1550),
    [sym_identifier] = ACTIONS(1552),
    [anon_sym_SEMI] = ACTIONS(1506),
    [anon_sym_macro] = ACTIONS(1552),
    [anon_sym_LPAREN] = ACTIONS(1506),
    [anon_sym_LBRACE] = ACTIONS(1550),
    [anon_sym_RBRACE] = ACTIONS(1506),
    [anon_sym_macro_rules] = ACTIONS(1552),
    [anon_sym_BANG] = ACTIONS(1552),
    [anon_sym_LBRACK] = ACTIONS(1506),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1508),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(1552),
    [anon_sym_i8] = ACTIONS(1552),
    [anon_sym_u16] = ACTIONS(1552),
    [anon_sym_i16] = ACTIONS(1552),
    [anon_sym_u32] = ACTIONS(1552),
    [anon_sym_i32] = ACTIONS(1552),
    [anon_sym_u64] = ACTIONS(1552),
    [anon_sym_i64] = ACTIONS(1552),
    [anon_sym_u128] = ACTIONS(1552),
    [anon_sym_i128] = ACTIONS(1552),
    [anon_sym_isize] = ACTIONS(1552),
    [anon_sym_usize] = ACTIONS(1552),
    [anon_sym_f32] = ACTIONS(1552),
    [anon_sym_f64] = ACTIONS(1552),
    [anon_sym_bool] = ACTIONS(1552),
    [anon_sym_str] = ACTIONS(1552),
    [anon_sym_char] = ACTIONS(1552),
    [anon_sym_DASH] = ACTIONS(1508),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(1508),
    [anon_sym_PIPE] = ACTIONS(1508),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(1508),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1506),
    [anon_sym_COLON_COLON] = ACTIONS(1550),
    [anon_sym_POUND] = ACTIONS(1550),
    [anon_sym_SQUOTE] = ACTIONS(1552),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_async] = ACTIONS(1552),
    [anon_sym_auto] = ACTIONS(1552),
    [anon_sym_await] = ACTIONS(1552),
    [anon_sym_become] = ACTIONS(1552),
    [anon_sym_break] = ACTIONS(1552),
    [anon_sym_const] = ACTIONS(1552),
    [anon_sym_continue] = ACTIONS(1552),
    [anon_sym_default] = ACTIONS(1552),
    [anon_sym_enum] = ACTIONS(1552),
    [anon_sym_fn] = ACTIONS(1552),
    [anon_sym_for] = ACTIONS(1552),
    [anon_sym_gen] = ACTIONS(1552),
    [anon_sym_if] = ACTIONS(1552),
    [anon_sym_impl] = ACTIONS(1552),
    [anon_sym_let] = ACTIONS(1552),
    [anon_sym_loop] = ACTIONS(1552),
    [anon_sym_match] = ACTIONS(1552),
    [anon_sym_mod] = ACTIONS(1552),
    [anon_sym_pub] = ACTIONS(1552),
    [anon_sym_return] = ACTIONS(1552),
    [anon_sym_raw] = ACTIONS(1552),
    [anon_sym_static] = ACTIONS(1552),
    [anon_sym_struct] = ACTIONS(1552),
    [anon_sym_trait] = ACTIONS(1552),
    [anon_sym_type] = ACTIONS(1552),
    [anon_sym_union] = ACTIONS(1552),
    [anon_sym_unsafe] = ACTIONS(1552),
    [anon_sym_use] = ACTIONS(1552),
    [anon_sym_while] = ACTIONS(1552),
    [anon_sym_extern] = ACTIONS(1552),
    [anon_sym_safe] = ACTIONS(1552),
    [anon_sym_yield] = ACTIONS(1552),
    [anon_sym_move] = ACTIONS(1552),
    [anon_sym_try] = ACTIONS(1552),
    [sym_integer_literal] = ACTIONS(1550),
    [aux_sym_string_literal_token1] = ACTIONS(1550),
    [sym_char_literal] = ACTIONS(1550),
    [anon_sym_true] = ACTIONS(1552),
    [anon_sym_false] = ACTIONS(1552),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1552),
    [sym_super] = ACTIONS(1552),
    [sym_crate] = ACTIONS(1552),
    [aux_sym_metavariable_token1] = ACTIONS(1550),
    [sym__raw_string_literal_start] = ACTIONS(1550),
    [sym_float_literal] = ACTIONS(1550),
    [sym__outer_block_doc_comment_start] = ACTIONS(1550),
    [sym__outer_line_doc_comment_start] = ACTIONS(1550),
  },
  [STATE(471)] = {
    [sym_line_comment] = STATE(471),
    [sym_block_comment] = STATE(471),
    [ts_builtin_sym_end] = ACTIONS(1554),
    [sym_identifier] = ACTIONS(1556),
    [anon_sym_SEMI] = ACTIONS(1554),
    [anon_sym_macro] = ACTIONS(1556),
    [anon_sym_LPAREN] = ACTIONS(1554),
    [anon_sym_LBRACE] = ACTIONS(1554),
    [anon_sym_RBRACE] = ACTIONS(1554),
    [anon_sym_macro_rules] = ACTIONS(1556),
    [anon_sym_BANG] = ACTIONS(1556),
    [anon_sym_LBRACK] = ACTIONS(1554),
    [anon_sym_PLUS] = ACTIONS(1556),
    [anon_sym_STAR] = ACTIONS(1556),
    [anon_sym_QMARK] = ACTIONS(1554),
    [anon_sym_u8] = ACTIONS(1556),
    [anon_sym_i8] = ACTIONS(1556),
    [anon_sym_u16] = ACTIONS(1556),
    [anon_sym_i16] = ACTIONS(1556),
    [anon_sym_u32] = ACTIONS(1556),
    [anon_sym_i32] = ACTIONS(1556),
    [anon_sym_u64] = ACTIONS(1556),
    [anon_sym_i64] = ACTIONS(1556),
    [anon_sym_u128] = ACTIONS(1556),
    [anon_sym_i128] = ACTIONS(1556),
    [anon_sym_isize] = ACTIONS(1556),
    [anon_sym_usize] = ACTIONS(1556),
    [anon_sym_f32] = ACTIONS(1556),
    [anon_sym_f64] = ACTIONS(1556),
    [anon_sym_bool] = ACTIONS(1556),
    [anon_sym_str] = ACTIONS(1556),
    [anon_sym_char] = ACTIONS(1556),
    [anon_sym_DASH] = ACTIONS(1556),
    [anon_sym_SLASH] = ACTIONS(1556),
    [anon_sym_PERCENT] = ACTIONS(1556),
    [anon_sym_CARET] = ACTIONS(1556),
    [anon_sym_AMP] = ACTIONS(1556),
    [anon_sym_PIPE] = ACTIONS(1556),
    [anon_sym_AMP_AMP] = ACTIONS(1554),
    [anon_sym_PIPE_PIPE] = ACTIONS(1554),
    [anon_sym_LT_LT] = ACTIONS(1556),
    [anon_sym_GT_GT] = ACTIONS(1556),
    [anon_sym_PLUS_EQ] = ACTIONS(1554),
    [anon_sym_DASH_EQ] = ACTIONS(1554),
    [anon_sym_STAR_EQ] = ACTIONS(1554),
    [anon_sym_SLASH_EQ] = ACTIONS(1554),
    [anon_sym_PERCENT_EQ] = ACTIONS(1554),
    [anon_sym_CARET_EQ] = ACTIONS(1554),
    [anon_sym_AMP_EQ] = ACTIONS(1554),
    [anon_sym_PIPE_EQ] = ACTIONS(1554),
    [anon_sym_LT_LT_EQ] = ACTIONS(1554),
    [anon_sym_GT_GT_EQ] = ACTIONS(1554),
    [anon_sym_EQ] = ACTIONS(1556),
    [anon_sym_EQ_EQ] = ACTIONS(1554),
    [anon_sym_BANG_EQ] = ACTIONS(1554),
    [anon_sym_GT] = ACTIONS(1556),
    [anon_sym_LT] = ACTIONS(1556),
    [anon_sym_GT_EQ] = ACTIONS(1554),
    [anon_sym_LT_EQ] = ACTIONS(1554),
    [anon_sym_DOT] = ACTIONS(1556),
    [anon_sym_DOT_DOT] = ACTIONS(1556),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1554),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1554),
    [anon_sym_COLON_COLON] = ACTIONS(1554),
    [anon_sym_POUND] = ACTIONS(1554),
    [anon_sym_SQUOTE] = ACTIONS(1556),
    [anon_sym_as] = ACTIONS(1556),
    [anon_sym_async] = ACTIONS(1556),
    [anon_sym_auto] = ACTIONS(1556),
    [anon_sym_await] = ACTIONS(1556),
    [anon_sym_become] = ACTIONS(1556),
    [anon_sym_break] = ACTIONS(1556),
    [anon_sym_const] = ACTIONS(1556),
    [anon_sym_continue] = ACTIONS(1556),
    [anon_sym_default] = ACTIONS(1556),
    [anon_sym_enum] = ACTIONS(1556),
    [anon_sym_fn] = ACTIONS(1556),
    [anon_sym_for] = ACTIONS(1556),
    [anon_sym_gen] = ACTIONS(1556),
    [anon_sym_if] = ACTIONS(1556),
    [anon_sym_impl] = ACTIONS(1556),
    [anon_sym_let] = ACTIONS(1556),
    [anon_sym_loop] = ACTIONS(1556),
    [anon_sym_match] = ACTIONS(1556),
    [anon_sym_mod] = ACTIONS(1556),
    [anon_sym_pub] = ACTIONS(1556),
    [anon_sym_return] = ACTIONS(1556),
    [anon_sym_raw] = ACTIONS(1556),
    [anon_sym_static] = ACTIONS(1556),
    [anon_sym_struct] = ACTIONS(1556),
    [anon_sym_trait] = ACTIONS(1556),
    [anon_sym_type] = ACTIONS(1556),
    [anon_sym_union] = ACTIONS(1556),
    [anon_sym_unsafe] = ACTIONS(1556),
    [anon_sym_use] = ACTIONS(1556),
    [anon_sym_while] = ACTIONS(1556),
    [anon_sym_extern] = ACTIONS(1556),
    [anon_sym_safe] = ACTIONS(1556),
    [anon_sym_yield] = ACTIONS(1556),
    [anon_sym_move] = ACTIONS(1556),
    [anon_sym_try] = ACTIONS(1556),
    [sym_integer_literal] = ACTIONS(1554),
    [aux_sym_string_literal_token1] = ACTIONS(1554),
    [sym_char_literal] = ACTIONS(1554),
    [anon_sym_true] = ACTIONS(1556),
    [anon_sym_false] = ACTIONS(1556),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1556),
    [sym_super] = ACTIONS(1556),
    [sym_crate] = ACTIONS(1556),
    [aux_sym_metavariable_token1] = ACTIONS(1554),
    [sym__raw_string_literal_start] = ACTIONS(1554),
    [sym_float_literal] = ACTIONS(1554),
    [sym__outer_block_doc_comment_start] = ACTIONS(1554),
    [sym__outer_line_doc_comment_start] = ACTIONS(1554),
  },
  [STATE(472)] = {
    [sym_line_comment] = STATE(472),
    [sym_block_comment] = STATE(472),
    [ts_builtin_sym_end] = ACTIONS(1558),
    [sym_identifier] = ACTIONS(1560),
    [anon_sym_SEMI] = ACTIONS(1558),
    [anon_sym_macro] = ACTIONS(1560),
    [anon_sym_LPAREN] = ACTIONS(1558),
    [anon_sym_LBRACE] = ACTIONS(1558),
    [anon_sym_RBRACE] = ACTIONS(1558),
    [anon_sym_macro_rules] = ACTIONS(1560),
    [anon_sym_BANG] = ACTIONS(1560),
    [anon_sym_LBRACK] = ACTIONS(1558),
    [anon_sym_PLUS] = ACTIONS(1560),
    [anon_sym_STAR] = ACTIONS(1560),
    [anon_sym_QMARK] = ACTIONS(1558),
    [anon_sym_u8] = ACTIONS(1560),
    [anon_sym_i8] = ACTIONS(1560),
    [anon_sym_u16] = ACTIONS(1560),
    [anon_sym_i16] = ACTIONS(1560),
    [anon_sym_u32] = ACTIONS(1560),
    [anon_sym_i32] = ACTIONS(1560),
    [anon_sym_u64] = ACTIONS(1560),
    [anon_sym_i64] = ACTIONS(1560),
    [anon_sym_u128] = ACTIONS(1560),
    [anon_sym_i128] = ACTIONS(1560),
    [anon_sym_isize] = ACTIONS(1560),
    [anon_sym_usize] = ACTIONS(1560),
    [anon_sym_f32] = ACTIONS(1560),
    [anon_sym_f64] = ACTIONS(1560),
    [anon_sym_bool] = ACTIONS(1560),
    [anon_sym_str] = ACTIONS(1560),
    [anon_sym_char] = ACTIONS(1560),
    [anon_sym_DASH] = ACTIONS(1560),
    [anon_sym_SLASH] = ACTIONS(1560),
    [anon_sym_PERCENT] = ACTIONS(1560),
    [anon_sym_CARET] = ACTIONS(1560),
    [anon_sym_AMP] = ACTIONS(1560),
    [anon_sym_PIPE] = ACTIONS(1560),
    [anon_sym_AMP_AMP] = ACTIONS(1558),
    [anon_sym_PIPE_PIPE] = ACTIONS(1558),
    [anon_sym_LT_LT] = ACTIONS(1560),
    [anon_sym_GT_GT] = ACTIONS(1560),
    [anon_sym_PLUS_EQ] = ACTIONS(1558),
    [anon_sym_DASH_EQ] = ACTIONS(1558),
    [anon_sym_STAR_EQ] = ACTIONS(1558),
    [anon_sym_SLASH_EQ] = ACTIONS(1558),
    [anon_sym_PERCENT_EQ] = ACTIONS(1558),
    [anon_sym_CARET_EQ] = ACTIONS(1558),
    [anon_sym_AMP_EQ] = ACTIONS(1558),
    [anon_sym_PIPE_EQ] = ACTIONS(1558),
    [anon_sym_LT_LT_EQ] = ACTIONS(1558),
    [anon_sym_GT_GT_EQ] = ACTIONS(1558),
    [anon_sym_EQ] = ACTIONS(1560),
    [anon_sym_EQ_EQ] = ACTIONS(1558),
    [anon_sym_BANG_EQ] = ACTIONS(1558),
    [anon_sym_GT] = ACTIONS(1560),
    [anon_sym_LT] = ACTIONS(1560),
    [anon_sym_GT_EQ] = ACTIONS(1558),
    [anon_sym_LT_EQ] = ACTIONS(1558),
    [anon_sym_DOT] = ACTIONS(1560),
    [anon_sym_DOT_DOT] = ACTIONS(1560),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1558),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1558),
    [anon_sym_COLON_COLON] = ACTIONS(1558),
    [anon_sym_POUND] = ACTIONS(1558),
    [anon_sym_SQUOTE] = ACTIONS(1560),
    [anon_sym_as] = ACTIONS(1560),
    [anon_sym_async] = ACTIONS(1560),
    [anon_sym_auto] = ACTIONS(1560),
    [anon_sym_await] = ACTIONS(1560),
    [anon_sym_become] = ACTIONS(1560),
    [anon_sym_break] = ACTIONS(1560),
    [anon_sym_const] = ACTIONS(1560),
    [anon_sym_continue] = ACTIONS(1560),
    [anon_sym_default] = ACTIONS(1560),
    [anon_sym_enum] = ACTIONS(1560),
    [anon_sym_fn] = ACTIONS(1560),
    [anon_sym_for] = ACTIONS(1560),
    [anon_sym_gen] = ACTIONS(1560),
    [anon_sym_if] = ACTIONS(1560),
    [anon_sym_impl] = ACTIONS(1560),
    [anon_sym_let] = ACTIONS(1560),
    [anon_sym_loop] = ACTIONS(1560),
    [anon_sym_match] = ACTIONS(1560),
    [anon_sym_mod] = ACTIONS(1560),
    [anon_sym_pub] = ACTIONS(1560),
    [anon_sym_return] = ACTIONS(1560),
    [anon_sym_raw] = ACTIONS(1560),
    [anon_sym_static] = ACTIONS(1560),
    [anon_sym_struct] = ACTIONS(1560),
    [anon_sym_trait] = ACTIONS(1560),
    [anon_sym_type] = ACTIONS(1560),
    [anon_sym_union] = ACTIONS(1560),
    [anon_sym_unsafe] = ACTIONS(1560),
    [anon_sym_use] = ACTIONS(1560),
    [anon_sym_while] = ACTIONS(1560),
    [anon_sym_extern] = ACTIONS(1560),
    [anon_sym_safe] = ACTIONS(1560),
    [anon_sym_yield] = ACTIONS(1560),
    [anon_sym_move] = ACTIONS(1560),
    [anon_sym_try] = ACTIONS(1560),
    [sym_integer_literal] = ACTIONS(1558),
    [aux_sym_string_literal_token1] = ACTIONS(1558),
    [sym_char_literal] = ACTIONS(1558),
    [anon_sym_true] = ACTIONS(1560),
    [anon_sym_false] = ACTIONS(1560),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1560),
    [sym_super] = ACTIONS(1560),
    [sym_crate] = ACTIONS(1560),
    [aux_sym_metavariable_token1] = ACTIONS(1558),
    [sym__raw_string_literal_start] = ACTIONS(1558),
    [sym_float_literal] = ACTIONS(1558),
    [sym__outer_block_doc_comment_start] = ACTIONS(1558),
    [sym__outer_line_doc_comment_start] = ACTIONS(1558),
  },
  [STATE(473)] = {
    [sym_line_comment] = STATE(473),
    [sym_block_comment] = STATE(473),
    [ts_builtin_sym_end] = ACTIONS(1536),
    [sym_identifier] = ACTIONS(1534),
    [anon_sym_SEMI] = ACTIONS(1536),
    [anon_sym_macro] = ACTIONS(1534),
    [anon_sym_LPAREN] = ACTIONS(1536),
    [anon_sym_LBRACE] = ACTIONS(1536),
    [anon_sym_RBRACE] = ACTIONS(1536),
    [anon_sym_macro_rules] = ACTIONS(1534),
    [anon_sym_BANG] = ACTIONS(1534),
    [anon_sym_LBRACK] = ACTIONS(1536),
    [anon_sym_PLUS] = ACTIONS(1534),
    [anon_sym_STAR] = ACTIONS(1534),
    [anon_sym_QMARK] = ACTIONS(1536),
    [anon_sym_u8] = ACTIONS(1534),
    [anon_sym_i8] = ACTIONS(1534),
    [anon_sym_u16] = ACTIONS(1534),
    [anon_sym_i16] = ACTIONS(1534),
    [anon_sym_u32] = ACTIONS(1534),
    [anon_sym_i32] = ACTIONS(1534),
    [anon_sym_u64] = ACTIONS(1534),
    [anon_sym_i64] = ACTIONS(1534),
    [anon_sym_u128] = ACTIONS(1534),
    [anon_sym_i128] = ACTIONS(1534),
    [anon_sym_isize] = ACTIONS(1534),
    [anon_sym_usize] = ACTIONS(1534),
    [anon_sym_f32] = ACTIONS(1534),
    [anon_sym_f64] = ACTIONS(1534),
    [anon_sym_bool] = ACTIONS(1534),
    [anon_sym_str] = ACTIONS(1534),
    [anon_sym_char] = ACTIONS(1534),
    [anon_sym_DASH] = ACTIONS(1534),
    [anon_sym_SLASH] = ACTIONS(1534),
    [anon_sym_PERCENT] = ACTIONS(1534),
    [anon_sym_CARET] = ACTIONS(1534),
    [anon_sym_AMP] = ACTIONS(1534),
    [anon_sym_PIPE] = ACTIONS(1534),
    [anon_sym_AMP_AMP] = ACTIONS(1536),
    [anon_sym_PIPE_PIPE] = ACTIONS(1536),
    [anon_sym_LT_LT] = ACTIONS(1534),
    [anon_sym_GT_GT] = ACTIONS(1534),
    [anon_sym_PLUS_EQ] = ACTIONS(1536),
    [anon_sym_DASH_EQ] = ACTIONS(1536),
    [anon_sym_STAR_EQ] = ACTIONS(1536),
    [anon_sym_SLASH_EQ] = ACTIONS(1536),
    [anon_sym_PERCENT_EQ] = ACTIONS(1536),
    [anon_sym_CARET_EQ] = ACTIONS(1536),
    [anon_sym_AMP_EQ] = ACTIONS(1536),
    [anon_sym_PIPE_EQ] = ACTIONS(1536),
    [anon_sym_LT_LT_EQ] = ACTIONS(1536),
    [anon_sym_GT_GT_EQ] = ACTIONS(1536),
    [anon_sym_EQ] = ACTIONS(1534),
    [anon_sym_EQ_EQ] = ACTIONS(1536),
    [anon_sym_BANG_EQ] = ACTIONS(1536),
    [anon_sym_GT] = ACTIONS(1534),
    [anon_sym_LT] = ACTIONS(1534),
    [anon_sym_GT_EQ] = ACTIONS(1536),
    [anon_sym_LT_EQ] = ACTIONS(1536),
    [anon_sym_DOT] = ACTIONS(1534),
    [anon_sym_DOT_DOT] = ACTIONS(1534),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1536),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1536),
    [anon_sym_COLON_COLON] = ACTIONS(1536),
    [anon_sym_POUND] = ACTIONS(1536),
    [anon_sym_SQUOTE] = ACTIONS(1534),
    [anon_sym_as] = ACTIONS(1534),
    [anon_sym_async] = ACTIONS(1534),
    [anon_sym_auto] = ACTIONS(1534),
    [anon_sym_await] = ACTIONS(1534),
    [anon_sym_become] = ACTIONS(1534),
    [anon_sym_break] = ACTIONS(1534),
    [anon_sym_const] = ACTIONS(1534),
    [anon_sym_continue] = ACTIONS(1534),
    [anon_sym_default] = ACTIONS(1534),
    [anon_sym_enum] = ACTIONS(1534),
    [anon_sym_fn] = ACTIONS(1534),
    [anon_sym_for] = ACTIONS(1534),
    [anon_sym_gen] = ACTIONS(1534),
    [anon_sym_if] = ACTIONS(1534),
    [anon_sym_impl] = ACTIONS(1534),
    [anon_sym_let] = ACTIONS(1534),
    [anon_sym_loop] = ACTIONS(1534),
    [anon_sym_match] = ACTIONS(1534),
    [anon_sym_mod] = ACTIONS(1534),
    [anon_sym_pub] = ACTIONS(1534),
    [anon_sym_return] = ACTIONS(1534),
    [anon_sym_raw] = ACTIONS(1534),
    [anon_sym_static] = ACTIONS(1534),
    [anon_sym_struct] = ACTIONS(1534),
    [anon_sym_trait] = ACTIONS(1534),
    [anon_sym_type] = ACTIONS(1534),
    [anon_sym_union] = ACTIONS(1534),
    [anon_sym_unsafe] = ACTIONS(1534),
    [anon_sym_use] = ACTIONS(1534),
    [anon_sym_while] = ACTIONS(1534),
    [anon_sym_extern] = ACTIONS(1534),
    [anon_sym_safe] = ACTIONS(1534),
    [anon_sym_yield] = ACTIONS(1534),
    [anon_sym_move] = ACTIONS(1534),
    [anon_sym_try] = ACTIONS(1534),
    [sym_integer_literal] = ACTIONS(1536),
    [aux_sym_string_literal_token1] = ACTIONS(1536),
    [sym_char_literal] = ACTIONS(1536),
    [anon_sym_true] = ACTIONS(1534),
    [anon_sym_false] = ACTIONS(1534),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1534),
    [sym_super] = ACTIONS(1534),
    [sym_crate] = ACTIONS(1534),
    [aux_sym_metavariable_token1] = ACTIONS(1536),
    [sym__raw_string_literal_start] = ACTIONS(1536),
    [sym_float_literal] = ACTIONS(1536),
    [sym__outer_block_doc_comment_start] = ACTIONS(1536),
    [sym__outer_line_doc_comment_start] = ACTIONS(1536),
  },
  [STATE(474)] = {
    [sym_line_comment] = STATE(474),
    [sym_block_comment] = STATE(474),
    [ts_builtin_sym_end] = ACTIONS(1562),
    [sym_identifier] = ACTIONS(1564),
    [anon_sym_SEMI] = ACTIONS(1562),
    [anon_sym_macro] = ACTIONS(1564),
    [anon_sym_LPAREN] = ACTIONS(1562),
    [anon_sym_LBRACE] = ACTIONS(1562),
    [anon_sym_RBRACE] = ACTIONS(1562),
    [anon_sym_macro_rules] = ACTIONS(1564),
    [anon_sym_BANG] = ACTIONS(1564),
    [anon_sym_LBRACK] = ACTIONS(1562),
    [anon_sym_PLUS] = ACTIONS(1564),
    [anon_sym_STAR] = ACTIONS(1564),
    [anon_sym_QMARK] = ACTIONS(1562),
    [anon_sym_u8] = ACTIONS(1564),
    [anon_sym_i8] = ACTIONS(1564),
    [anon_sym_u16] = ACTIONS(1564),
    [anon_sym_i16] = ACTIONS(1564),
    [anon_sym_u32] = ACTIONS(1564),
    [anon_sym_i32] = ACTIONS(1564),
    [anon_sym_u64] = ACTIONS(1564),
    [anon_sym_i64] = ACTIONS(1564),
    [anon_sym_u128] = ACTIONS(1564),
    [anon_sym_i128] = ACTIONS(1564),
    [anon_sym_isize] = ACTIONS(1564),
    [anon_sym_usize] = ACTIONS(1564),
    [anon_sym_f32] = ACTIONS(1564),
    [anon_sym_f64] = ACTIONS(1564),
    [anon_sym_bool] = ACTIONS(1564),
    [anon_sym_str] = ACTIONS(1564),
    [anon_sym_char] = ACTIONS(1564),
    [anon_sym_DASH] = ACTIONS(1564),
    [anon_sym_SLASH] = ACTIONS(1564),
    [anon_sym_PERCENT] = ACTIONS(1564),
    [anon_sym_CARET] = ACTIONS(1564),
    [anon_sym_AMP] = ACTIONS(1564),
    [anon_sym_PIPE] = ACTIONS(1564),
    [anon_sym_AMP_AMP] = ACTIONS(1562),
    [anon_sym_PIPE_PIPE] = ACTIONS(1562),
    [anon_sym_LT_LT] = ACTIONS(1564),
    [anon_sym_GT_GT] = ACTIONS(1564),
    [anon_sym_PLUS_EQ] = ACTIONS(1562),
    [anon_sym_DASH_EQ] = ACTIONS(1562),
    [anon_sym_STAR_EQ] = ACTIONS(1562),
    [anon_sym_SLASH_EQ] = ACTIONS(1562),
    [anon_sym_PERCENT_EQ] = ACTIONS(1562),
    [anon_sym_CARET_EQ] = ACTIONS(1562),
    [anon_sym_AMP_EQ] = ACTIONS(1562),
    [anon_sym_PIPE_EQ] = ACTIONS(1562),
    [anon_sym_LT_LT_EQ] = ACTIONS(1562),
    [anon_sym_GT_GT_EQ] = ACTIONS(1562),
    [anon_sym_EQ] = ACTIONS(1564),
    [anon_sym_EQ_EQ] = ACTIONS(1562),
    [anon_sym_BANG_EQ] = ACTIONS(1562),
    [anon_sym_GT] = ACTIONS(1564),
    [anon_sym_LT] = ACTIONS(1564),
    [anon_sym_GT_EQ] = ACTIONS(1562),
    [anon_sym_LT_EQ] = ACTIONS(1562),
    [anon_sym_DOT] = ACTIONS(1564),
    [anon_sym_DOT_DOT] = ACTIONS(1564),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1562),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1562),
    [anon_sym_COLON_COLON] = ACTIONS(1562),
    [anon_sym_POUND] = ACTIONS(1562),
    [anon_sym_SQUOTE] = ACTIONS(1564),
    [anon_sym_as] = ACTIONS(1564),
    [anon_sym_async] = ACTIONS(1564),
    [anon_sym_auto] = ACTIONS(1564),
    [anon_sym_await] = ACTIONS(1564),
    [anon_sym_become] = ACTIONS(1564),
    [anon_sym_break] = ACTIONS(1564),
    [anon_sym_const] = ACTIONS(1564),
    [anon_sym_continue] = ACTIONS(1564),
    [anon_sym_default] = ACTIONS(1564),
    [anon_sym_enum] = ACTIONS(1564),
    [anon_sym_fn] = ACTIONS(1564),
    [anon_sym_for] = ACTIONS(1564),
    [anon_sym_gen] = ACTIONS(1564),
    [anon_sym_if] = ACTIONS(1564),
    [anon_sym_impl] = ACTIONS(1564),
    [anon_sym_let] = ACTIONS(1564),
    [anon_sym_loop] = ACTIONS(1564),
    [anon_sym_match] = ACTIONS(1564),
    [anon_sym_mod] = ACTIONS(1564),
    [anon_sym_pub] = ACTIONS(1564),
    [anon_sym_return] = ACTIONS(1564),
    [anon_sym_raw] = ACTIONS(1564),
    [anon_sym_static] = ACTIONS(1564),
    [anon_sym_struct] = ACTIONS(1564),
    [anon_sym_trait] = ACTIONS(1564),
    [anon_sym_type] = ACTIONS(1564),
    [anon_sym_union] = ACTIONS(1564),
    [anon_sym_unsafe] = ACTIONS(1564),
    [anon_sym_use] = ACTIONS(1564),
    [anon_sym_while] = ACTIONS(1564),
    [anon_sym_extern] = ACTIONS(1564),
    [anon_sym_safe] = ACTIONS(1564),
    [anon_sym_yield] = ACTIONS(1564),
    [anon_sym_move] = ACTIONS(1564),
    [anon_sym_try] = ACTIONS(1564),
    [sym_integer_literal] = ACTIONS(1562),
    [aux_sym_string_literal_token1] = ACTIONS(1562),
    [sym_char_literal] = ACTIONS(1562),
    [anon_sym_true] = ACTIONS(1564),
    [anon_sym_false] = ACTIONS(1564),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1564),
    [sym_super] = ACTIONS(1564),
    [sym_crate] = ACTIONS(1564),
    [aux_sym_metavariable_token1] = ACTIONS(1562),
    [sym__raw_string_literal_start] = ACTIONS(1562),
    [sym_float_literal] = ACTIONS(1562),
    [sym__outer_block_doc_comment_start] = ACTIONS(1562),
    [sym__outer_line_doc_comment_start] = ACTIONS(1562),
  },
  [STATE(475)] = {
    [sym_line_comment] = STATE(475),
    [sym_block_comment] = STATE(475),
    [ts_builtin_sym_end] = ACTIONS(1566),
    [sym_identifier] = ACTIONS(1568),
    [anon_sym_SEMI] = ACTIONS(1566),
    [anon_sym_macro] = ACTIONS(1568),
    [anon_sym_LPAREN] = ACTIONS(1566),
    [anon_sym_LBRACE] = ACTIONS(1566),
    [anon_sym_RBRACE] = ACTIONS(1566),
    [anon_sym_macro_rules] = ACTIONS(1568),
    [anon_sym_BANG] = ACTIONS(1568),
    [anon_sym_LBRACK] = ACTIONS(1566),
    [anon_sym_PLUS] = ACTIONS(1568),
    [anon_sym_STAR] = ACTIONS(1568),
    [anon_sym_QMARK] = ACTIONS(1566),
    [anon_sym_u8] = ACTIONS(1568),
    [anon_sym_i8] = ACTIONS(1568),
    [anon_sym_u16] = ACTIONS(1568),
    [anon_sym_i16] = ACTIONS(1568),
    [anon_sym_u32] = ACTIONS(1568),
    [anon_sym_i32] = ACTIONS(1568),
    [anon_sym_u64] = ACTIONS(1568),
    [anon_sym_i64] = ACTIONS(1568),
    [anon_sym_u128] = ACTIONS(1568),
    [anon_sym_i128] = ACTIONS(1568),
    [anon_sym_isize] = ACTIONS(1568),
    [anon_sym_usize] = ACTIONS(1568),
    [anon_sym_f32] = ACTIONS(1568),
    [anon_sym_f64] = ACTIONS(1568),
    [anon_sym_bool] = ACTIONS(1568),
    [anon_sym_str] = ACTIONS(1568),
    [anon_sym_char] = ACTIONS(1568),
    [anon_sym_DASH] = ACTIONS(1568),
    [anon_sym_SLASH] = ACTIONS(1568),
    [anon_sym_PERCENT] = ACTIONS(1568),
    [anon_sym_CARET] = ACTIONS(1568),
    [anon_sym_AMP] = ACTIONS(1568),
    [anon_sym_PIPE] = ACTIONS(1568),
    [anon_sym_AMP_AMP] = ACTIONS(1566),
    [anon_sym_PIPE_PIPE] = ACTIONS(1566),
    [anon_sym_LT_LT] = ACTIONS(1568),
    [anon_sym_GT_GT] = ACTIONS(1568),
    [anon_sym_PLUS_EQ] = ACTIONS(1566),
    [anon_sym_DASH_EQ] = ACTIONS(1566),
    [anon_sym_STAR_EQ] = ACTIONS(1566),
    [anon_sym_SLASH_EQ] = ACTIONS(1566),
    [anon_sym_PERCENT_EQ] = ACTIONS(1566),
    [anon_sym_CARET_EQ] = ACTIONS(1566),
    [anon_sym_AMP_EQ] = ACTIONS(1566),
    [anon_sym_PIPE_EQ] = ACTIONS(1566),
    [anon_sym_LT_LT_EQ] = ACTIONS(1566),
    [anon_sym_GT_GT_EQ] = ACTIONS(1566),
    [anon_sym_EQ] = ACTIONS(1568),
    [anon_sym_EQ_EQ] = ACTIONS(1566),
    [anon_sym_BANG_EQ] = ACTIONS(1566),
    [anon_sym_GT] = ACTIONS(1568),
    [anon_sym_LT] = ACTIONS(1568),
    [anon_sym_GT_EQ] = ACTIONS(1566),
    [anon_sym_LT_EQ] = ACTIONS(1566),
    [anon_sym_DOT] = ACTIONS(1568),
    [anon_sym_DOT_DOT] = ACTIONS(1568),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1566),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1566),
    [anon_sym_COLON_COLON] = ACTIONS(1566),
    [anon_sym_POUND] = ACTIONS(1566),
    [anon_sym_SQUOTE] = ACTIONS(1568),
    [anon_sym_as] = ACTIONS(1568),
    [anon_sym_async] = ACTIONS(1568),
    [anon_sym_auto] = ACTIONS(1568),
    [anon_sym_await] = ACTIONS(1568),
    [anon_sym_become] = ACTIONS(1568),
    [anon_sym_break] = ACTIONS(1568),
    [anon_sym_const] = ACTIONS(1568),
    [anon_sym_continue] = ACTIONS(1568),
    [anon_sym_default] = ACTIONS(1568),
    [anon_sym_enum] = ACTIONS(1568),
    [anon_sym_fn] = ACTIONS(1568),
    [anon_sym_for] = ACTIONS(1568),
    [anon_sym_gen] = ACTIONS(1568),
    [anon_sym_if] = ACTIONS(1568),
    [anon_sym_impl] = ACTIONS(1568),
    [anon_sym_let] = ACTIONS(1568),
    [anon_sym_loop] = ACTIONS(1568),
    [anon_sym_match] = ACTIONS(1568),
    [anon_sym_mod] = ACTIONS(1568),
    [anon_sym_pub] = ACTIONS(1568),
    [anon_sym_return] = ACTIONS(1568),
    [anon_sym_raw] = ACTIONS(1568),
    [anon_sym_static] = ACTIONS(1568),
    [anon_sym_struct] = ACTIONS(1568),
    [anon_sym_trait] = ACTIONS(1568),
    [anon_sym_type] = ACTIONS(1568),
    [anon_sym_union] = ACTIONS(1568),
    [anon_sym_unsafe] = ACTIONS(1568),
    [anon_sym_use] = ACTIONS(1568),
    [anon_sym_while] = ACTIONS(1568),
    [anon_sym_extern] = ACTIONS(1568),
    [anon_sym_safe] = ACTIONS(1568),
    [anon_sym_yield] = ACTIONS(1568),
    [anon_sym_move] = ACTIONS(1568),
    [anon_sym_try] = ACTIONS(1568),
    [sym_integer_literal] = ACTIONS(1566),
    [aux_sym_string_literal_token1] = ACTIONS(1566),
    [sym_char_literal] = ACTIONS(1566),
    [anon_sym_true] = ACTIONS(1568),
    [anon_sym_false] = ACTIONS(1568),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1568),
    [sym_super] = ACTIONS(1568),
    [sym_crate] = ACTIONS(1568),
    [aux_sym_metavariable_token1] = ACTIONS(1566),
    [sym__raw_string_literal_start] = ACTIONS(1566),
    [sym_float_literal] = ACTIONS(1566),
    [sym__outer_block_doc_comment_start] = ACTIONS(1566),
    [sym__outer_line_doc_comment_start] = ACTIONS(1566),
  },
  [STATE(476)] = {
    [sym_line_comment] = STATE(476),
    [sym_block_comment] = STATE(476),
    [ts_builtin_sym_end] = ACTIONS(1570),
    [sym_identifier] = ACTIONS(1572),
    [anon_sym_SEMI] = ACTIONS(1570),
    [anon_sym_macro] = ACTIONS(1572),
    [anon_sym_LPAREN] = ACTIONS(1570),
    [anon_sym_LBRACE] = ACTIONS(1570),
    [anon_sym_RBRACE] = ACTIONS(1570),
    [anon_sym_macro_rules] = ACTIONS(1572),
    [anon_sym_BANG] = ACTIONS(1572),
    [anon_sym_LBRACK] = ACTIONS(1570),
    [anon_sym_PLUS] = ACTIONS(1572),
    [anon_sym_STAR] = ACTIONS(1572),
    [anon_sym_QMARK] = ACTIONS(1570),
    [anon_sym_u8] = ACTIONS(1572),
    [anon_sym_i8] = ACTIONS(1572),
    [anon_sym_u16] = ACTIONS(1572),
    [anon_sym_i16] = ACTIONS(1572),
    [anon_sym_u32] = ACTIONS(1572),
    [anon_sym_i32] = ACTIONS(1572),
    [anon_sym_u64] = ACTIONS(1572),
    [anon_sym_i64] = ACTIONS(1572),
    [anon_sym_u128] = ACTIONS(1572),
    [anon_sym_i128] = ACTIONS(1572),
    [anon_sym_isize] = ACTIONS(1572),
    [anon_sym_usize] = ACTIONS(1572),
    [anon_sym_f32] = ACTIONS(1572),
    [anon_sym_f64] = ACTIONS(1572),
    [anon_sym_bool] = ACTIONS(1572),
    [anon_sym_str] = ACTIONS(1572),
    [anon_sym_char] = ACTIONS(1572),
    [anon_sym_DASH] = ACTIONS(1572),
    [anon_sym_SLASH] = ACTIONS(1572),
    [anon_sym_PERCENT] = ACTIONS(1572),
    [anon_sym_CARET] = ACTIONS(1572),
    [anon_sym_AMP] = ACTIONS(1572),
    [anon_sym_PIPE] = ACTIONS(1572),
    [anon_sym_AMP_AMP] = ACTIONS(1570),
    [anon_sym_PIPE_PIPE] = ACTIONS(1570),
    [anon_sym_LT_LT] = ACTIONS(1572),
    [anon_sym_GT_GT] = ACTIONS(1572),
    [anon_sym_PLUS_EQ] = ACTIONS(1570),
    [anon_sym_DASH_EQ] = ACTIONS(1570),
    [anon_sym_STAR_EQ] = ACTIONS(1570),
    [anon_sym_SLASH_EQ] = ACTIONS(1570),
    [anon_sym_PERCENT_EQ] = ACTIONS(1570),
    [anon_sym_CARET_EQ] = ACTIONS(1570),
    [anon_sym_AMP_EQ] = ACTIONS(1570),
    [anon_sym_PIPE_EQ] = ACTIONS(1570),
    [anon_sym_LT_LT_EQ] = ACTIONS(1570),
    [anon_sym_GT_GT_EQ] = ACTIONS(1570),
    [anon_sym_EQ] = ACTIONS(1572),
    [anon_sym_EQ_EQ] = ACTIONS(1570),
    [anon_sym_BANG_EQ] = ACTIONS(1570),
    [anon_sym_GT] = ACTIONS(1572),
    [anon_sym_LT] = ACTIONS(1572),
    [anon_sym_GT_EQ] = ACTIONS(1570),
    [anon_sym_LT_EQ] = ACTIONS(1570),
    [anon_sym_DOT] = ACTIONS(1572),
    [anon_sym_DOT_DOT] = ACTIONS(1572),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1570),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1570),
    [anon_sym_COLON_COLON] = ACTIONS(1570),
    [anon_sym_POUND] = ACTIONS(1570),
    [anon_sym_SQUOTE] = ACTIONS(1572),
    [anon_sym_as] = ACTIONS(1572),
    [anon_sym_async] = ACTIONS(1572),
    [anon_sym_auto] = ACTIONS(1572),
    [anon_sym_await] = ACTIONS(1572),
    [anon_sym_become] = ACTIONS(1572),
    [anon_sym_break] = ACTIONS(1572),
    [anon_sym_const] = ACTIONS(1572),
    [anon_sym_continue] = ACTIONS(1572),
    [anon_sym_default] = ACTIONS(1572),
    [anon_sym_enum] = ACTIONS(1572),
    [anon_sym_fn] = ACTIONS(1572),
    [anon_sym_for] = ACTIONS(1572),
    [anon_sym_gen] = ACTIONS(1572),
    [anon_sym_if] = ACTIONS(1572),
    [anon_sym_impl] = ACTIONS(1572),
    [anon_sym_let] = ACTIONS(1572),
    [anon_sym_loop] = ACTIONS(1572),
    [anon_sym_match] = ACTIONS(1572),
    [anon_sym_mod] = ACTIONS(1572),
    [anon_sym_pub] = ACTIONS(1572),
    [anon_sym_return] = ACTIONS(1572),
    [anon_sym_raw] = ACTIONS(1572),
    [anon_sym_static] = ACTIONS(1572),
    [anon_sym_struct] = ACTIONS(1572),
    [anon_sym_trait] = ACTIONS(1572),
    [anon_sym_type] = ACTIONS(1572),
    [anon_sym_union] = ACTIONS(1572),
    [anon_sym_unsafe] = ACTIONS(1572),
    [anon_sym_use] = ACTIONS(1572),
    [anon_sym_while] = ACTIONS(1572),
    [anon_sym_extern] = ACTIONS(1572),
    [anon_sym_safe] = ACTIONS(1572),
    [anon_sym_yield] = ACTIONS(1572),
    [anon_sym_move] = ACTIONS(1572),
    [anon_sym_try] = ACTIONS(1572),
    [sym_integer_literal] = ACTIONS(1570),
    [aux_sym_string_literal_token1] = ACTIONS(1570),
    [sym_char_literal] = ACTIONS(1570),
    [anon_sym_true] = ACTIONS(1572),
    [anon_sym_false] = ACTIONS(1572),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1572),
    [sym_super] = ACTIONS(1572),
    [sym_crate] = ACTIONS(1572),
    [aux_sym_metavariable_token1] = ACTIONS(1570),
    [sym__raw_string_literal_start] = ACTIONS(1570),
    [sym_float_literal] = ACTIONS(1570),
    [sym__outer_block_doc_comment_start] = ACTIONS(1570),
    [sym__outer_line_doc_comment_start] = ACTIONS(1570),
  },
  [STATE(477)] = {
    [sym_line_comment] = STATE(477),
    [sym_block_comment] = STATE(477),
    [ts_builtin_sym_end] = ACTIONS(1574),
    [sym_identifier] = ACTIONS(1576),
    [anon_sym_SEMI] = ACTIONS(1574),
    [anon_sym_macro] = ACTIONS(1576),
    [anon_sym_LPAREN] = ACTIONS(1574),
    [anon_sym_LBRACE] = ACTIONS(1574),
    [anon_sym_RBRACE] = ACTIONS(1574),
    [anon_sym_macro_rules] = ACTIONS(1576),
    [anon_sym_BANG] = ACTIONS(1576),
    [anon_sym_LBRACK] = ACTIONS(1574),
    [anon_sym_PLUS] = ACTIONS(1576),
    [anon_sym_STAR] = ACTIONS(1576),
    [anon_sym_QMARK] = ACTIONS(1574),
    [anon_sym_u8] = ACTIONS(1576),
    [anon_sym_i8] = ACTIONS(1576),
    [anon_sym_u16] = ACTIONS(1576),
    [anon_sym_i16] = ACTIONS(1576),
    [anon_sym_u32] = ACTIONS(1576),
    [anon_sym_i32] = ACTIONS(1576),
    [anon_sym_u64] = ACTIONS(1576),
    [anon_sym_i64] = ACTIONS(1576),
    [anon_sym_u128] = ACTIONS(1576),
    [anon_sym_i128] = ACTIONS(1576),
    [anon_sym_isize] = ACTIONS(1576),
    [anon_sym_usize] = ACTIONS(1576),
    [anon_sym_f32] = ACTIONS(1576),
    [anon_sym_f64] = ACTIONS(1576),
    [anon_sym_bool] = ACTIONS(1576),
    [anon_sym_str] = ACTIONS(1576),
    [anon_sym_char] = ACTIONS(1576),
    [anon_sym_DASH] = ACTIONS(1576),
    [anon_sym_SLASH] = ACTIONS(1576),
    [anon_sym_PERCENT] = ACTIONS(1576),
    [anon_sym_CARET] = ACTIONS(1576),
    [anon_sym_AMP] = ACTIONS(1576),
    [anon_sym_PIPE] = ACTIONS(1576),
    [anon_sym_AMP_AMP] = ACTIONS(1574),
    [anon_sym_PIPE_PIPE] = ACTIONS(1574),
    [anon_sym_LT_LT] = ACTIONS(1576),
    [anon_sym_GT_GT] = ACTIONS(1576),
    [anon_sym_PLUS_EQ] = ACTIONS(1574),
    [anon_sym_DASH_EQ] = ACTIONS(1574),
    [anon_sym_STAR_EQ] = ACTIONS(1574),
    [anon_sym_SLASH_EQ] = ACTIONS(1574),
    [anon_sym_PERCENT_EQ] = ACTIONS(1574),
    [anon_sym_CARET_EQ] = ACTIONS(1574),
    [anon_sym_AMP_EQ] = ACTIONS(1574),
    [anon_sym_PIPE_EQ] = ACTIONS(1574),
    [anon_sym_LT_LT_EQ] = ACTIONS(1574),
    [anon_sym_GT_GT_EQ] = ACTIONS(1574),
    [anon_sym_EQ] = ACTIONS(1576),
    [anon_sym_EQ_EQ] = ACTIONS(1574),
    [anon_sym_BANG_EQ] = ACTIONS(1574),
    [anon_sym_GT] = ACTIONS(1576),
    [anon_sym_LT] = ACTIONS(1576),
    [anon_sym_GT_EQ] = ACTIONS(1574),
    [anon_sym_LT_EQ] = ACTIONS(1574),
    [anon_sym_DOT] = ACTIONS(1576),
    [anon_sym_DOT_DOT] = ACTIONS(1576),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1574),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1574),
    [anon_sym_COLON_COLON] = ACTIONS(1574),
    [anon_sym_POUND] = ACTIONS(1574),
    [anon_sym_SQUOTE] = ACTIONS(1576),
    [anon_sym_as] = ACTIONS(1576),
    [anon_sym_async] = ACTIONS(1576),
    [anon_sym_auto] = ACTIONS(1576),
    [anon_sym_await] = ACTIONS(1576),
    [anon_sym_become] = ACTIONS(1576),
    [anon_sym_break] = ACTIONS(1576),
    [anon_sym_const] = ACTIONS(1576),
    [anon_sym_continue] = ACTIONS(1576),
    [anon_sym_default] = ACTIONS(1576),
    [anon_sym_enum] = ACTIONS(1576),
    [anon_sym_fn] = ACTIONS(1576),
    [anon_sym_for] = ACTIONS(1576),
    [anon_sym_gen] = ACTIONS(1576),
    [anon_sym_if] = ACTIONS(1576),
    [anon_sym_impl] = ACTIONS(1576),
    [anon_sym_let] = ACTIONS(1576),
    [anon_sym_loop] = ACTIONS(1576),
    [anon_sym_match] = ACTIONS(1576),
    [anon_sym_mod] = ACTIONS(1576),
    [anon_sym_pub] = ACTIONS(1576),
    [anon_sym_return] = ACTIONS(1576),
    [anon_sym_raw] = ACTIONS(1576),
    [anon_sym_static] = ACTIONS(1576),
    [anon_sym_struct] = ACTIONS(1576),
    [anon_sym_trait] = ACTIONS(1576),
    [anon_sym_type] = ACTIONS(1576),
    [anon_sym_union] = ACTIONS(1576),
    [anon_sym_unsafe] = ACTIONS(1576),
    [anon_sym_use] = ACTIONS(1576),
    [anon_sym_while] = ACTIONS(1576),
    [anon_sym_extern] = ACTIONS(1576),
    [anon_sym_safe] = ACTIONS(1576),
    [anon_sym_yield] = ACTIONS(1576),
    [anon_sym_move] = ACTIONS(1576),
    [anon_sym_try] = ACTIONS(1576),
    [sym_integer_literal] = ACTIONS(1574),
    [aux_sym_string_literal_token1] = ACTIONS(1574),
    [sym_char_literal] = ACTIONS(1574),
    [anon_sym_true] = ACTIONS(1576),
    [anon_sym_false] = ACTIONS(1576),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1576),
    [sym_super] = ACTIONS(1576),
    [sym_crate] = ACTIONS(1576),
    [aux_sym_metavariable_token1] = ACTIONS(1574),
    [sym__raw_string_literal_start] = ACTIONS(1574),
    [sym_float_literal] = ACTIONS(1574),
    [sym__outer_block_doc_comment_start] = ACTIONS(1574),
    [sym__outer_line_doc_comment_start] = ACTIONS(1574),
  },
  [STATE(478)] = {
    [sym_line_comment] = STATE(478),
    [sym_block_comment] = STATE(478),
    [ts_builtin_sym_end] = ACTIONS(1578),
    [sym_identifier] = ACTIONS(1580),
    [anon_sym_SEMI] = ACTIONS(1578),
    [anon_sym_macro] = ACTIONS(1580),
    [anon_sym_LPAREN] = ACTIONS(1578),
    [anon_sym_LBRACE] = ACTIONS(1578),
    [anon_sym_RBRACE] = ACTIONS(1578),
    [anon_sym_macro_rules] = ACTIONS(1580),
    [anon_sym_BANG] = ACTIONS(1580),
    [anon_sym_LBRACK] = ACTIONS(1578),
    [anon_sym_PLUS] = ACTIONS(1580),
    [anon_sym_STAR] = ACTIONS(1580),
    [anon_sym_QMARK] = ACTIONS(1578),
    [anon_sym_u8] = ACTIONS(1580),
    [anon_sym_i8] = ACTIONS(1580),
    [anon_sym_u16] = ACTIONS(1580),
    [anon_sym_i16] = ACTIONS(1580),
    [anon_sym_u32] = ACTIONS(1580),
    [anon_sym_i32] = ACTIONS(1580),
    [anon_sym_u64] = ACTIONS(1580),
    [anon_sym_i64] = ACTIONS(1580),
    [anon_sym_u128] = ACTIONS(1580),
    [anon_sym_i128] = ACTIONS(1580),
    [anon_sym_isize] = ACTIONS(1580),
    [anon_sym_usize] = ACTIONS(1580),
    [anon_sym_f32] = ACTIONS(1580),
    [anon_sym_f64] = ACTIONS(1580),
    [anon_sym_bool] = ACTIONS(1580),
    [anon_sym_str] = ACTIONS(1580),
    [anon_sym_char] = ACTIONS(1580),
    [anon_sym_DASH] = ACTIONS(1580),
    [anon_sym_SLASH] = ACTIONS(1580),
    [anon_sym_PERCENT] = ACTIONS(1580),
    [anon_sym_CARET] = ACTIONS(1580),
    [anon_sym_AMP] = ACTIONS(1580),
    [anon_sym_PIPE] = ACTIONS(1580),
    [anon_sym_AMP_AMP] = ACTIONS(1578),
    [anon_sym_PIPE_PIPE] = ACTIONS(1578),
    [anon_sym_LT_LT] = ACTIONS(1580),
    [anon_sym_GT_GT] = ACTIONS(1580),
    [anon_sym_PLUS_EQ] = ACTIONS(1578),
    [anon_sym_DASH_EQ] = ACTIONS(1578),
    [anon_sym_STAR_EQ] = ACTIONS(1578),
    [anon_sym_SLASH_EQ] = ACTIONS(1578),
    [anon_sym_PERCENT_EQ] = ACTIONS(1578),
    [anon_sym_CARET_EQ] = ACTIONS(1578),
    [anon_sym_AMP_EQ] = ACTIONS(1578),
    [anon_sym_PIPE_EQ] = ACTIONS(1578),
    [anon_sym_LT_LT_EQ] = ACTIONS(1578),
    [anon_sym_GT_GT_EQ] = ACTIONS(1578),
    [anon_sym_EQ] = ACTIONS(1580),
    [anon_sym_EQ_EQ] = ACTIONS(1578),
    [anon_sym_BANG_EQ] = ACTIONS(1578),
    [anon_sym_GT] = ACTIONS(1580),
    [anon_sym_LT] = ACTIONS(1580),
    [anon_sym_GT_EQ] = ACTIONS(1578),
    [anon_sym_LT_EQ] = ACTIONS(1578),
    [anon_sym_DOT] = ACTIONS(1580),
    [anon_sym_DOT_DOT] = ACTIONS(1580),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1578),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1578),
    [anon_sym_COLON_COLON] = ACTIONS(1578),
    [anon_sym_POUND] = ACTIONS(1578),
    [anon_sym_SQUOTE] = ACTIONS(1580),
    [anon_sym_as] = ACTIONS(1580),
    [anon_sym_async] = ACTIONS(1580),
    [anon_sym_auto] = ACTIONS(1580),
    [anon_sym_await] = ACTIONS(1580),
    [anon_sym_become] = ACTIONS(1580),
    [anon_sym_break] = ACTIONS(1580),
    [anon_sym_const] = ACTIONS(1580),
    [anon_sym_continue] = ACTIONS(1580),
    [anon_sym_default] = ACTIONS(1580),
    [anon_sym_enum] = ACTIONS(1580),
    [anon_sym_fn] = ACTIONS(1580),
    [anon_sym_for] = ACTIONS(1580),
    [anon_sym_gen] = ACTIONS(1580),
    [anon_sym_if] = ACTIONS(1580),
    [anon_sym_impl] = ACTIONS(1580),
    [anon_sym_let] = ACTIONS(1580),
    [anon_sym_loop] = ACTIONS(1580),
    [anon_sym_match] = ACTIONS(1580),
    [anon_sym_mod] = ACTIONS(1580),
    [anon_sym_pub] = ACTIONS(1580),
    [anon_sym_return] = ACTIONS(1580),
    [anon_sym_raw] = ACTIONS(1580),
    [anon_sym_static] = ACTIONS(1580),
    [anon_sym_struct] = ACTIONS(1580),
    [anon_sym_trait] = ACTIONS(1580),
    [anon_sym_type] = ACTIONS(1580),
    [anon_sym_union] = ACTIONS(1580),
    [anon_sym_unsafe] = ACTIONS(1580),
    [anon_sym_use] = ACTIONS(1580),
    [anon_sym_while] = ACTIONS(1580),
    [anon_sym_extern] = ACTIONS(1580),
    [anon_sym_safe] = ACTIONS(1580),
    [anon_sym_yield] = ACTIONS(1580),
    [anon_sym_move] = ACTIONS(1580),
    [anon_sym_try] = ACTIONS(1580),
    [sym_integer_literal] = ACTIONS(1578),
    [aux_sym_string_literal_token1] = ACTIONS(1578),
    [sym_char_literal] = ACTIONS(1578),
    [anon_sym_true] = ACTIONS(1580),
    [anon_sym_false] = ACTIONS(1580),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1580),
    [sym_super] = ACTIONS(1580),
    [sym_crate] = ACTIONS(1580),
    [aux_sym_metavariable_token1] = ACTIONS(1578),
    [sym__raw_string_literal_start] = ACTIONS(1578),
    [sym_float_literal] = ACTIONS(1578),
    [sym__outer_block_doc_comment_start] = ACTIONS(1578),
    [sym__outer_line_doc_comment_start] = ACTIONS(1578),
  },
  [STATE(479)] = {
    [sym_line_comment] = STATE(479),
    [sym_block_comment] = STATE(479),
    [ts_builtin_sym_end] = ACTIONS(1582),
    [sym_identifier] = ACTIONS(1584),
    [anon_sym_SEMI] = ACTIONS(1582),
    [anon_sym_macro] = ACTIONS(1584),
    [anon_sym_LPAREN] = ACTIONS(1582),
    [anon_sym_LBRACE] = ACTIONS(1582),
    [anon_sym_RBRACE] = ACTIONS(1582),
    [anon_sym_macro_rules] = ACTIONS(1584),
    [anon_sym_BANG] = ACTIONS(1584),
    [anon_sym_LBRACK] = ACTIONS(1582),
    [anon_sym_PLUS] = ACTIONS(1584),
    [anon_sym_STAR] = ACTIONS(1584),
    [anon_sym_QMARK] = ACTIONS(1582),
    [anon_sym_u8] = ACTIONS(1584),
    [anon_sym_i8] = ACTIONS(1584),
    [anon_sym_u16] = ACTIONS(1584),
    [anon_sym_i16] = ACTIONS(1584),
    [anon_sym_u32] = ACTIONS(1584),
    [anon_sym_i32] = ACTIONS(1584),
    [anon_sym_u64] = ACTIONS(1584),
    [anon_sym_i64] = ACTIONS(1584),
    [anon_sym_u128] = ACTIONS(1584),
    [anon_sym_i128] = ACTIONS(1584),
    [anon_sym_isize] = ACTIONS(1584),
    [anon_sym_usize] = ACTIONS(1584),
    [anon_sym_f32] = ACTIONS(1584),
    [anon_sym_f64] = ACTIONS(1584),
    [anon_sym_bool] = ACTIONS(1584),
    [anon_sym_str] = ACTIONS(1584),
    [anon_sym_char] = ACTIONS(1584),
    [anon_sym_DASH] = ACTIONS(1584),
    [anon_sym_SLASH] = ACTIONS(1584),
    [anon_sym_PERCENT] = ACTIONS(1584),
    [anon_sym_CARET] = ACTIONS(1584),
    [anon_sym_AMP] = ACTIONS(1584),
    [anon_sym_PIPE] = ACTIONS(1584),
    [anon_sym_AMP_AMP] = ACTIONS(1582),
    [anon_sym_PIPE_PIPE] = ACTIONS(1582),
    [anon_sym_LT_LT] = ACTIONS(1584),
    [anon_sym_GT_GT] = ACTIONS(1584),
    [anon_sym_PLUS_EQ] = ACTIONS(1582),
    [anon_sym_DASH_EQ] = ACTIONS(1582),
    [anon_sym_STAR_EQ] = ACTIONS(1582),
    [anon_sym_SLASH_EQ] = ACTIONS(1582),
    [anon_sym_PERCENT_EQ] = ACTIONS(1582),
    [anon_sym_CARET_EQ] = ACTIONS(1582),
    [anon_sym_AMP_EQ] = ACTIONS(1582),
    [anon_sym_PIPE_EQ] = ACTIONS(1582),
    [anon_sym_LT_LT_EQ] = ACTIONS(1582),
    [anon_sym_GT_GT_EQ] = ACTIONS(1582),
    [anon_sym_EQ] = ACTIONS(1584),
    [anon_sym_EQ_EQ] = ACTIONS(1582),
    [anon_sym_BANG_EQ] = ACTIONS(1582),
    [anon_sym_GT] = ACTIONS(1584),
    [anon_sym_LT] = ACTIONS(1584),
    [anon_sym_GT_EQ] = ACTIONS(1582),
    [anon_sym_LT_EQ] = ACTIONS(1582),
    [anon_sym_DOT] = ACTIONS(1584),
    [anon_sym_DOT_DOT] = ACTIONS(1584),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1582),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1582),
    [anon_sym_COLON_COLON] = ACTIONS(1582),
    [anon_sym_POUND] = ACTIONS(1582),
    [anon_sym_SQUOTE] = ACTIONS(1584),
    [anon_sym_as] = ACTIONS(1584),
    [anon_sym_async] = ACTIONS(1584),
    [anon_sym_auto] = ACTIONS(1584),
    [anon_sym_await] = ACTIONS(1584),
    [anon_sym_become] = ACTIONS(1584),
    [anon_sym_break] = ACTIONS(1584),
    [anon_sym_const] = ACTIONS(1584),
    [anon_sym_continue] = ACTIONS(1584),
    [anon_sym_default] = ACTIONS(1584),
    [anon_sym_enum] = ACTIONS(1584),
    [anon_sym_fn] = ACTIONS(1584),
    [anon_sym_for] = ACTIONS(1584),
    [anon_sym_gen] = ACTIONS(1584),
    [anon_sym_if] = ACTIONS(1584),
    [anon_sym_impl] = ACTIONS(1584),
    [anon_sym_let] = ACTIONS(1584),
    [anon_sym_loop] = ACTIONS(1584),
    [anon_sym_match] = ACTIONS(1584),
    [anon_sym_mod] = ACTIONS(1584),
    [anon_sym_pub] = ACTIONS(1584),
    [anon_sym_return] = ACTIONS(1584),
    [anon_sym_raw] = ACTIONS(1584),
    [anon_sym_static] = ACTIONS(1584),
    [anon_sym_struct] = ACTIONS(1584),
    [anon_sym_trait] = ACTIONS(1584),
    [anon_sym_type] = ACTIONS(1584),
    [anon_sym_union] = ACTIONS(1584),
    [anon_sym_unsafe] = ACTIONS(1584),
    [anon_sym_use] = ACTIONS(1584),
    [anon_sym_while] = ACTIONS(1584),
    [anon_sym_extern] = ACTIONS(1584),
    [anon_sym_safe] = ACTIONS(1584),
    [anon_sym_yield] = ACTIONS(1584),
    [anon_sym_move] = ACTIONS(1584),
    [anon_sym_try] = ACTIONS(1584),
    [sym_integer_literal] = ACTIONS(1582),
    [aux_sym_string_literal_token1] = ACTIONS(1582),
    [sym_char_literal] = ACTIONS(1582),
    [anon_sym_true] = ACTIONS(1584),
    [anon_sym_false] = ACTIONS(1584),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1584),
    [sym_super] = ACTIONS(1584),
    [sym_crate] = ACTIONS(1584),
    [aux_sym_metavariable_token1] = ACTIONS(1582),
    [sym__raw_string_literal_start] = ACTIONS(1582),
    [sym_float_literal] = ACTIONS(1582),
    [sym__outer_block_doc_comment_start] = ACTIONS(1582),
    [sym__outer_line_doc_comment_start] = ACTIONS(1582),
  },
  [STATE(480)] = {
    [sym_line_comment] = STATE(480),
    [sym_block_comment] = STATE(480),
    [ts_builtin_sym_end] = ACTIONS(1586),
    [sym_identifier] = ACTIONS(1588),
    [anon_sym_SEMI] = ACTIONS(1586),
    [anon_sym_macro] = ACTIONS(1588),
    [anon_sym_LPAREN] = ACTIONS(1586),
    [anon_sym_LBRACE] = ACTIONS(1586),
    [anon_sym_RBRACE] = ACTIONS(1586),
    [anon_sym_macro_rules] = ACTIONS(1588),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_LBRACK] = ACTIONS(1586),
    [anon_sym_PLUS] = ACTIONS(1588),
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_QMARK] = ACTIONS(1586),
    [anon_sym_u8] = ACTIONS(1588),
    [anon_sym_i8] = ACTIONS(1588),
    [anon_sym_u16] = ACTIONS(1588),
    [anon_sym_i16] = ACTIONS(1588),
    [anon_sym_u32] = ACTIONS(1588),
    [anon_sym_i32] = ACTIONS(1588),
    [anon_sym_u64] = ACTIONS(1588),
    [anon_sym_i64] = ACTIONS(1588),
    [anon_sym_u128] = ACTIONS(1588),
    [anon_sym_i128] = ACTIONS(1588),
    [anon_sym_isize] = ACTIONS(1588),
    [anon_sym_usize] = ACTIONS(1588),
    [anon_sym_f32] = ACTIONS(1588),
    [anon_sym_f64] = ACTIONS(1588),
    [anon_sym_bool] = ACTIONS(1588),
    [anon_sym_str] = ACTIONS(1588),
    [anon_sym_char] = ACTIONS(1588),
    [anon_sym_DASH] = ACTIONS(1588),
    [anon_sym_SLASH] = ACTIONS(1588),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1586),
    [anon_sym_PIPE_PIPE] = ACTIONS(1586),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1588),
    [anon_sym_PLUS_EQ] = ACTIONS(1586),
    [anon_sym_DASH_EQ] = ACTIONS(1586),
    [anon_sym_STAR_EQ] = ACTIONS(1586),
    [anon_sym_SLASH_EQ] = ACTIONS(1586),
    [anon_sym_PERCENT_EQ] = ACTIONS(1586),
    [anon_sym_CARET_EQ] = ACTIONS(1586),
    [anon_sym_AMP_EQ] = ACTIONS(1586),
    [anon_sym_PIPE_EQ] = ACTIONS(1586),
    [anon_sym_LT_LT_EQ] = ACTIONS(1586),
    [anon_sym_GT_GT_EQ] = ACTIONS(1586),
    [anon_sym_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1586),
    [anon_sym_BANG_EQ] = ACTIONS(1586),
    [anon_sym_GT] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1586),
    [anon_sym_LT_EQ] = ACTIONS(1586),
    [anon_sym_DOT] = ACTIONS(1588),
    [anon_sym_DOT_DOT] = ACTIONS(1588),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1586),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1586),
    [anon_sym_COLON_COLON] = ACTIONS(1586),
    [anon_sym_POUND] = ACTIONS(1586),
    [anon_sym_SQUOTE] = ACTIONS(1588),
    [anon_sym_as] = ACTIONS(1588),
    [anon_sym_async] = ACTIONS(1588),
    [anon_sym_auto] = ACTIONS(1588),
    [anon_sym_await] = ACTIONS(1588),
    [anon_sym_become] = ACTIONS(1588),
    [anon_sym_break] = ACTIONS(1588),
    [anon_sym_const] = ACTIONS(1588),
    [anon_sym_continue] = ACTIONS(1588),
    [anon_sym_default] = ACTIONS(1588),
    [anon_sym_enum] = ACTIONS(1588),
    [anon_sym_fn] = ACTIONS(1588),
    [anon_sym_for] = ACTIONS(1588),
    [anon_sym_gen] = ACTIONS(1588),
    [anon_sym_if] = ACTIONS(1588),
    [anon_sym_impl] = ACTIONS(1588),
    [anon_sym_let] = ACTIONS(1588),
    [anon_sym_loop] = ACTIONS(1588),
    [anon_sym_match] = ACTIONS(1588),
    [anon_sym_mod] = ACTIONS(1588),
    [anon_sym_pub] = ACTIONS(1588),
    [anon_sym_return] = ACTIONS(1588),
    [anon_sym_raw] = ACTIONS(1588),
    [anon_sym_static] = ACTIONS(1588),
    [anon_sym_struct] = ACTIONS(1588),
    [anon_sym_trait] = ACTIONS(1588),
    [anon_sym_type] = ACTIONS(1588),
    [anon_sym_union] = ACTIONS(1588),
    [anon_sym_unsafe] = ACTIONS(1588),
    [anon_sym_use] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1588),
    [anon_sym_extern] = ACTIONS(1588),
    [anon_sym_safe] = ACTIONS(1588),
    [anon_sym_yield] = ACTIONS(1588),
    [anon_sym_move] = ACTIONS(1588),
    [anon_sym_try] = ACTIONS(1588),
    [sym_integer_literal] = ACTIONS(1586),
    [aux_sym_string_literal_token1] = ACTIONS(1586),
    [sym_char_literal] = ACTIONS(1586),
    [anon_sym_true] = ACTIONS(1588),
    [anon_sym_false] = ACTIONS(1588),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1588),
    [sym_super] = ACTIONS(1588),
    [sym_crate] = ACTIONS(1588),
    [aux_sym_metavariable_token1] = ACTIONS(1586),
    [sym__raw_string_literal_start] = ACTIONS(1586),
    [sym_float_literal] = ACTIONS(1586),
    [sym__outer_block_doc_comment_start] = ACTIONS(1586),
    [sym__outer_line_doc_comment_start] = ACTIONS(1586),
  },
  [STATE(481)] = {
    [sym_line_comment] = STATE(481),
    [sym_block_comment] = STATE(481),
    [ts_builtin_sym_end] = ACTIONS(1590),
    [sym_identifier] = ACTIONS(1592),
    [anon_sym_SEMI] = ACTIONS(1590),
    [anon_sym_macro] = ACTIONS(1592),
    [anon_sym_LPAREN] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1590),
    [anon_sym_RBRACE] = ACTIONS(1590),
    [anon_sym_macro_rules] = ACTIONS(1592),
    [anon_sym_BANG] = ACTIONS(1592),
    [anon_sym_LBRACK] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1592),
    [anon_sym_STAR] = ACTIONS(1592),
    [anon_sym_QMARK] = ACTIONS(1590),
    [anon_sym_u8] = ACTIONS(1592),
    [anon_sym_i8] = ACTIONS(1592),
    [anon_sym_u16] = ACTIONS(1592),
    [anon_sym_i16] = ACTIONS(1592),
    [anon_sym_u32] = ACTIONS(1592),
    [anon_sym_i32] = ACTIONS(1592),
    [anon_sym_u64] = ACTIONS(1592),
    [anon_sym_i64] = ACTIONS(1592),
    [anon_sym_u128] = ACTIONS(1592),
    [anon_sym_i128] = ACTIONS(1592),
    [anon_sym_isize] = ACTIONS(1592),
    [anon_sym_usize] = ACTIONS(1592),
    [anon_sym_f32] = ACTIONS(1592),
    [anon_sym_f64] = ACTIONS(1592),
    [anon_sym_bool] = ACTIONS(1592),
    [anon_sym_str] = ACTIONS(1592),
    [anon_sym_char] = ACTIONS(1592),
    [anon_sym_DASH] = ACTIONS(1592),
    [anon_sym_SLASH] = ACTIONS(1592),
    [anon_sym_PERCENT] = ACTIONS(1592),
    [anon_sym_CARET] = ACTIONS(1592),
    [anon_sym_AMP] = ACTIONS(1592),
    [anon_sym_PIPE] = ACTIONS(1592),
    [anon_sym_AMP_AMP] = ACTIONS(1590),
    [anon_sym_PIPE_PIPE] = ACTIONS(1590),
    [anon_sym_LT_LT] = ACTIONS(1592),
    [anon_sym_GT_GT] = ACTIONS(1592),
    [anon_sym_PLUS_EQ] = ACTIONS(1590),
    [anon_sym_DASH_EQ] = ACTIONS(1590),
    [anon_sym_STAR_EQ] = ACTIONS(1590),
    [anon_sym_SLASH_EQ] = ACTIONS(1590),
    [anon_sym_PERCENT_EQ] = ACTIONS(1590),
    [anon_sym_CARET_EQ] = ACTIONS(1590),
    [anon_sym_AMP_EQ] = ACTIONS(1590),
    [anon_sym_PIPE_EQ] = ACTIONS(1590),
    [anon_sym_LT_LT_EQ] = ACTIONS(1590),
    [anon_sym_GT_GT_EQ] = ACTIONS(1590),
    [anon_sym_EQ] = ACTIONS(1592),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1592),
    [anon_sym_LT] = ACTIONS(1592),
    [anon_sym_GT_EQ] = ACTIONS(1590),
    [anon_sym_LT_EQ] = ACTIONS(1590),
    [anon_sym_DOT] = ACTIONS(1592),
    [anon_sym_DOT_DOT] = ACTIONS(1592),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1590),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1590),
    [anon_sym_COLON_COLON] = ACTIONS(1590),
    [anon_sym_POUND] = ACTIONS(1590),
    [anon_sym_SQUOTE] = ACTIONS(1592),
    [anon_sym_as] = ACTIONS(1592),
    [anon_sym_async] = ACTIONS(1592),
    [anon_sym_auto] = ACTIONS(1592),
    [anon_sym_await] = ACTIONS(1592),
    [anon_sym_become] = ACTIONS(1592),
    [anon_sym_break] = ACTIONS(1592),
    [anon_sym_const] = ACTIONS(1592),
    [anon_sym_continue] = ACTIONS(1592),
    [anon_sym_default] = ACTIONS(1592),
    [anon_sym_enum] = ACTIONS(1592),
    [anon_sym_fn] = ACTIONS(1592),
    [anon_sym_for] = ACTIONS(1592),
    [anon_sym_gen] = ACTIONS(1592),
    [anon_sym_if] = ACTIONS(1592),
    [anon_sym_impl] = ACTIONS(1592),
    [anon_sym_let] = ACTIONS(1592),
    [anon_sym_loop] = ACTIONS(1592),
    [anon_sym_match] = ACTIONS(1592),
    [anon_sym_mod] = ACTIONS(1592),
    [anon_sym_pub] = ACTIONS(1592),
    [anon_sym_return] = ACTIONS(1592),
    [anon_sym_raw] = ACTIONS(1592),
    [anon_sym_static] = ACTIONS(1592),
    [anon_sym_struct] = ACTIONS(1592),
    [anon_sym_trait] = ACTIONS(1592),
    [anon_sym_type] = ACTIONS(1592),
    [anon_sym_union] = ACTIONS(1592),
    [anon_sym_unsafe] = ACTIONS(1592),
    [anon_sym_use] = ACTIONS(1592),
    [anon_sym_while] = ACTIONS(1592),
    [anon_sym_extern] = ACTIONS(1592),
    [anon_sym_safe] = ACTIONS(1592),
    [anon_sym_yield] = ACTIONS(1592),
    [anon_sym_move] = ACTIONS(1592),
    [anon_sym_try] = ACTIONS(1592),
    [sym_integer_literal] = ACTIONS(1590),
    [aux_sym_string_literal_token1] = ACTIONS(1590),
    [sym_char_literal] = ACTIONS(1590),
    [anon_sym_true] = ACTIONS(1592),
    [anon_sym_false] = ACTIONS(1592),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1592),
    [sym_super] = ACTIONS(1592),
    [sym_crate] = ACTIONS(1592),
    [aux_sym_metavariable_token1] = ACTIONS(1590),
    [sym__raw_string_literal_start] = ACTIONS(1590),
    [sym_float_literal] = ACTIONS(1590),
    [sym__outer_block_doc_comment_start] = ACTIONS(1590),
    [sym__outer_line_doc_comment_start] = ACTIONS(1590),
  },
  [STATE(482)] = {
    [sym_line_comment] = STATE(482),
    [sym_block_comment] = STATE(482),
    [ts_builtin_sym_end] = ACTIONS(1594),
    [sym_identifier] = ACTIONS(1596),
    [anon_sym_SEMI] = ACTIONS(1594),
    [anon_sym_macro] = ACTIONS(1596),
    [anon_sym_LPAREN] = ACTIONS(1594),
    [anon_sym_LBRACE] = ACTIONS(1594),
    [anon_sym_RBRACE] = ACTIONS(1594),
    [anon_sym_macro_rules] = ACTIONS(1596),
    [anon_sym_BANG] = ACTIONS(1596),
    [anon_sym_LBRACK] = ACTIONS(1594),
    [anon_sym_PLUS] = ACTIONS(1596),
    [anon_sym_STAR] = ACTIONS(1596),
    [anon_sym_QMARK] = ACTIONS(1594),
    [anon_sym_u8] = ACTIONS(1596),
    [anon_sym_i8] = ACTIONS(1596),
    [anon_sym_u16] = ACTIONS(1596),
    [anon_sym_i16] = ACTIONS(1596),
    [anon_sym_u32] = ACTIONS(1596),
    [anon_sym_i32] = ACTIONS(1596),
    [anon_sym_u64] = ACTIONS(1596),
    [anon_sym_i64] = ACTIONS(1596),
    [anon_sym_u128] = ACTIONS(1596),
    [anon_sym_i128] = ACTIONS(1596),
    [anon_sym_isize] = ACTIONS(1596),
    [anon_sym_usize] = ACTIONS(1596),
    [anon_sym_f32] = ACTIONS(1596),
    [anon_sym_f64] = ACTIONS(1596),
    [anon_sym_bool] = ACTIONS(1596),
    [anon_sym_str] = ACTIONS(1596),
    [anon_sym_char] = ACTIONS(1596),
    [anon_sym_DASH] = ACTIONS(1596),
    [anon_sym_SLASH] = ACTIONS(1596),
    [anon_sym_PERCENT] = ACTIONS(1596),
    [anon_sym_CARET] = ACTIONS(1596),
    [anon_sym_AMP] = ACTIONS(1596),
    [anon_sym_PIPE] = ACTIONS(1596),
    [anon_sym_AMP_AMP] = ACTIONS(1594),
    [anon_sym_PIPE_PIPE] = ACTIONS(1594),
    [anon_sym_LT_LT] = ACTIONS(1596),
    [anon_sym_GT_GT] = ACTIONS(1596),
    [anon_sym_PLUS_EQ] = ACTIONS(1594),
    [anon_sym_DASH_EQ] = ACTIONS(1594),
    [anon_sym_STAR_EQ] = ACTIONS(1594),
    [anon_sym_SLASH_EQ] = ACTIONS(1594),
    [anon_sym_PERCENT_EQ] = ACTIONS(1594),
    [anon_sym_CARET_EQ] = ACTIONS(1594),
    [anon_sym_AMP_EQ] = ACTIONS(1594),
    [anon_sym_PIPE_EQ] = ACTIONS(1594),
    [anon_sym_LT_LT_EQ] = ACTIONS(1594),
    [anon_sym_GT_GT_EQ] = ACTIONS(1594),
    [anon_sym_EQ] = ACTIONS(1596),
    [anon_sym_EQ_EQ] = ACTIONS(1594),
    [anon_sym_BANG_EQ] = ACTIONS(1594),
    [anon_sym_GT] = ACTIONS(1596),
    [anon_sym_LT] = ACTIONS(1596),
    [anon_sym_GT_EQ] = ACTIONS(1594),
    [anon_sym_LT_EQ] = ACTIONS(1594),
    [anon_sym_DOT] = ACTIONS(1596),
    [anon_sym_DOT_DOT] = ACTIONS(1596),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1594),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1594),
    [anon_sym_COLON_COLON] = ACTIONS(1594),
    [anon_sym_POUND] = ACTIONS(1594),
    [anon_sym_SQUOTE] = ACTIONS(1596),
    [anon_sym_as] = ACTIONS(1596),
    [anon_sym_async] = ACTIONS(1596),
    [anon_sym_auto] = ACTIONS(1596),
    [anon_sym_await] = ACTIONS(1596),
    [anon_sym_become] = ACTIONS(1596),
    [anon_sym_break] = ACTIONS(1596),
    [anon_sym_const] = ACTIONS(1596),
    [anon_sym_continue] = ACTIONS(1596),
    [anon_sym_default] = ACTIONS(1596),
    [anon_sym_enum] = ACTIONS(1596),
    [anon_sym_fn] = ACTIONS(1596),
    [anon_sym_for] = ACTIONS(1596),
    [anon_sym_gen] = ACTIONS(1596),
    [anon_sym_if] = ACTIONS(1596),
    [anon_sym_impl] = ACTIONS(1596),
    [anon_sym_let] = ACTIONS(1596),
    [anon_sym_loop] = ACTIONS(1596),
    [anon_sym_match] = ACTIONS(1596),
    [anon_sym_mod] = ACTIONS(1596),
    [anon_sym_pub] = ACTIONS(1596),
    [anon_sym_return] = ACTIONS(1596),
    [anon_sym_raw] = ACTIONS(1596),
    [anon_sym_static] = ACTIONS(1596),
    [anon_sym_struct] = ACTIONS(1596),
    [anon_sym_trait] = ACTIONS(1596),
    [anon_sym_type] = ACTIONS(1596),
    [anon_sym_union] = ACTIONS(1596),
    [anon_sym_unsafe] = ACTIONS(1596),
    [anon_sym_use] = ACTIONS(1596),
    [anon_sym_while] = ACTIONS(1596),
    [anon_sym_extern] = ACTIONS(1596),
    [anon_sym_safe] = ACTIONS(1596),
    [anon_sym_yield] = ACTIONS(1596),
    [anon_sym_move] = ACTIONS(1596),
    [anon_sym_try] = ACTIONS(1596),
    [sym_integer_literal] = ACTIONS(1594),
    [aux_sym_string_literal_token1] = ACTIONS(1594),
    [sym_char_literal] = ACTIONS(1594),
    [anon_sym_true] = ACTIONS(1596),
    [anon_sym_false] = ACTIONS(1596),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1596),
    [sym_super] = ACTIONS(1596),
    [sym_crate] = ACTIONS(1596),
    [aux_sym_metavariable_token1] = ACTIONS(1594),
    [sym__raw_string_literal_start] = ACTIONS(1594),
    [sym_float_literal] = ACTIONS(1594),
    [sym__outer_block_doc_comment_start] = ACTIONS(1594),
    [sym__outer_line_doc_comment_start] = ACTIONS(1594),
  },
  [STATE(483)] = {
    [sym_line_comment] = STATE(483),
    [sym_block_comment] = STATE(483),
    [ts_builtin_sym_end] = ACTIONS(1598),
    [sym_identifier] = ACTIONS(1600),
    [anon_sym_SEMI] = ACTIONS(1598),
    [anon_sym_macro] = ACTIONS(1600),
    [anon_sym_LPAREN] = ACTIONS(1598),
    [anon_sym_LBRACE] = ACTIONS(1598),
    [anon_sym_RBRACE] = ACTIONS(1598),
    [anon_sym_macro_rules] = ACTIONS(1600),
    [anon_sym_BANG] = ACTIONS(1600),
    [anon_sym_LBRACK] = ACTIONS(1598),
    [anon_sym_PLUS] = ACTIONS(1600),
    [anon_sym_STAR] = ACTIONS(1600),
    [anon_sym_QMARK] = ACTIONS(1598),
    [anon_sym_u8] = ACTIONS(1600),
    [anon_sym_i8] = ACTIONS(1600),
    [anon_sym_u16] = ACTIONS(1600),
    [anon_sym_i16] = ACTIONS(1600),
    [anon_sym_u32] = ACTIONS(1600),
    [anon_sym_i32] = ACTIONS(1600),
    [anon_sym_u64] = ACTIONS(1600),
    [anon_sym_i64] = ACTIONS(1600),
    [anon_sym_u128] = ACTIONS(1600),
    [anon_sym_i128] = ACTIONS(1600),
    [anon_sym_isize] = ACTIONS(1600),
    [anon_sym_usize] = ACTIONS(1600),
    [anon_sym_f32] = ACTIONS(1600),
    [anon_sym_f64] = ACTIONS(1600),
    [anon_sym_bool] = ACTIONS(1600),
    [anon_sym_str] = ACTIONS(1600),
    [anon_sym_char] = ACTIONS(1600),
    [anon_sym_DASH] = ACTIONS(1600),
    [anon_sym_SLASH] = ACTIONS(1600),
    [anon_sym_PERCENT] = ACTIONS(1600),
    [anon_sym_CARET] = ACTIONS(1600),
    [anon_sym_AMP] = ACTIONS(1600),
    [anon_sym_PIPE] = ACTIONS(1600),
    [anon_sym_AMP_AMP] = ACTIONS(1598),
    [anon_sym_PIPE_PIPE] = ACTIONS(1598),
    [anon_sym_LT_LT] = ACTIONS(1600),
    [anon_sym_GT_GT] = ACTIONS(1600),
    [anon_sym_PLUS_EQ] = ACTIONS(1598),
    [anon_sym_DASH_EQ] = ACTIONS(1598),
    [anon_sym_STAR_EQ] = ACTIONS(1598),
    [anon_sym_SLASH_EQ] = ACTIONS(1598),
    [anon_sym_PERCENT_EQ] = ACTIONS(1598),
    [anon_sym_CARET_EQ] = ACTIONS(1598),
    [anon_sym_AMP_EQ] = ACTIONS(1598),
    [anon_sym_PIPE_EQ] = ACTIONS(1598),
    [anon_sym_LT_LT_EQ] = ACTIONS(1598),
    [anon_sym_GT_GT_EQ] = ACTIONS(1598),
    [anon_sym_EQ] = ACTIONS(1600),
    [anon_sym_EQ_EQ] = ACTIONS(1598),
    [anon_sym_BANG_EQ] = ACTIONS(1598),
    [anon_sym_GT] = ACTIONS(1600),
    [anon_sym_LT] = ACTIONS(1600),
    [anon_sym_GT_EQ] = ACTIONS(1598),
    [anon_sym_LT_EQ] = ACTIONS(1598),
    [anon_sym_DOT] = ACTIONS(1600),
    [anon_sym_DOT_DOT] = ACTIONS(1600),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1598),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1598),
    [anon_sym_COLON_COLON] = ACTIONS(1598),
    [anon_sym_POUND] = ACTIONS(1598),
    [anon_sym_SQUOTE] = ACTIONS(1600),
    [anon_sym_as] = ACTIONS(1600),
    [anon_sym_async] = ACTIONS(1600),
    [anon_sym_auto] = ACTIONS(1600),
    [anon_sym_await] = ACTIONS(1600),
    [anon_sym_become] = ACTIONS(1600),
    [anon_sym_break] = ACTIONS(1600),
    [anon_sym_const] = ACTIONS(1600),
    [anon_sym_continue] = ACTIONS(1600),
    [anon_sym_default] = ACTIONS(1600),
    [anon_sym_enum] = ACTIONS(1600),
    [anon_sym_fn] = ACTIONS(1600),
    [anon_sym_for] = ACTIONS(1600),
    [anon_sym_gen] = ACTIONS(1600),
    [anon_sym_if] = ACTIONS(1600),
    [anon_sym_impl] = ACTIONS(1600),
    [anon_sym_let] = ACTIONS(1600),
    [anon_sym_loop] = ACTIONS(1600),
    [anon_sym_match] = ACTIONS(1600),
    [anon_sym_mod] = ACTIONS(1600),
    [anon_sym_pub] = ACTIONS(1600),
    [anon_sym_return] = ACTIONS(1600),
    [anon_sym_raw] = ACTIONS(1600),
    [anon_sym_static] = ACTIONS(1600),
    [anon_sym_struct] = ACTIONS(1600),
    [anon_sym_trait] = ACTIONS(1600),
    [anon_sym_type] = ACTIONS(1600),
    [anon_sym_union] = ACTIONS(1600),
    [anon_sym_unsafe] = ACTIONS(1600),
    [anon_sym_use] = ACTIONS(1600),
    [anon_sym_while] = ACTIONS(1600),
    [anon_sym_extern] = ACTIONS(1600),
    [anon_sym_safe] = ACTIONS(1600),
    [anon_sym_yield] = ACTIONS(1600),
    [anon_sym_move] = ACTIONS(1600),
    [anon_sym_try] = ACTIONS(1600),
    [sym_integer_literal] = ACTIONS(1598),
    [aux_sym_string_literal_token1] = ACTIONS(1598),
    [sym_char_literal] = ACTIONS(1598),
    [anon_sym_true] = ACTIONS(1600),
    [anon_sym_false] = ACTIONS(1600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1600),
    [sym_super] = ACTIONS(1600),
    [sym_crate] = ACTIONS(1600),
    [aux_sym_metavariable_token1] = ACTIONS(1598),
    [sym__raw_string_literal_start] = ACTIONS(1598),
    [sym_float_literal] = ACTIONS(1598),
    [sym__outer_block_doc_comment_start] = ACTIONS(1598),
    [sym__outer_line_doc_comment_start] = ACTIONS(1598),
  },
  [STATE(484)] = {
    [sym_line_comment] = STATE(484),
    [sym_block_comment] = STATE(484),
    [ts_builtin_sym_end] = ACTIONS(1602),
    [sym_identifier] = ACTIONS(1604),
    [anon_sym_SEMI] = ACTIONS(1602),
    [anon_sym_macro] = ACTIONS(1604),
    [anon_sym_LPAREN] = ACTIONS(1602),
    [anon_sym_LBRACE] = ACTIONS(1602),
    [anon_sym_RBRACE] = ACTIONS(1602),
    [anon_sym_macro_rules] = ACTIONS(1604),
    [anon_sym_BANG] = ACTIONS(1604),
    [anon_sym_LBRACK] = ACTIONS(1602),
    [anon_sym_PLUS] = ACTIONS(1604),
    [anon_sym_STAR] = ACTIONS(1604),
    [anon_sym_QMARK] = ACTIONS(1602),
    [anon_sym_u8] = ACTIONS(1604),
    [anon_sym_i8] = ACTIONS(1604),
    [anon_sym_u16] = ACTIONS(1604),
    [anon_sym_i16] = ACTIONS(1604),
    [anon_sym_u32] = ACTIONS(1604),
    [anon_sym_i32] = ACTIONS(1604),
    [anon_sym_u64] = ACTIONS(1604),
    [anon_sym_i64] = ACTIONS(1604),
    [anon_sym_u128] = ACTIONS(1604),
    [anon_sym_i128] = ACTIONS(1604),
    [anon_sym_isize] = ACTIONS(1604),
    [anon_sym_usize] = ACTIONS(1604),
    [anon_sym_f32] = ACTIONS(1604),
    [anon_sym_f64] = ACTIONS(1604),
    [anon_sym_bool] = ACTIONS(1604),
    [anon_sym_str] = ACTIONS(1604),
    [anon_sym_char] = ACTIONS(1604),
    [anon_sym_DASH] = ACTIONS(1604),
    [anon_sym_SLASH] = ACTIONS(1604),
    [anon_sym_PERCENT] = ACTIONS(1604),
    [anon_sym_CARET] = ACTIONS(1604),
    [anon_sym_AMP] = ACTIONS(1604),
    [anon_sym_PIPE] = ACTIONS(1604),
    [anon_sym_AMP_AMP] = ACTIONS(1602),
    [anon_sym_PIPE_PIPE] = ACTIONS(1602),
    [anon_sym_LT_LT] = ACTIONS(1604),
    [anon_sym_GT_GT] = ACTIONS(1604),
    [anon_sym_PLUS_EQ] = ACTIONS(1602),
    [anon_sym_DASH_EQ] = ACTIONS(1602),
    [anon_sym_STAR_EQ] = ACTIONS(1602),
    [anon_sym_SLASH_EQ] = ACTIONS(1602),
    [anon_sym_PERCENT_EQ] = ACTIONS(1602),
    [anon_sym_CARET_EQ] = ACTIONS(1602),
    [anon_sym_AMP_EQ] = ACTIONS(1602),
    [anon_sym_PIPE_EQ] = ACTIONS(1602),
    [anon_sym_LT_LT_EQ] = ACTIONS(1602),
    [anon_sym_GT_GT_EQ] = ACTIONS(1602),
    [anon_sym_EQ] = ACTIONS(1604),
    [anon_sym_EQ_EQ] = ACTIONS(1602),
    [anon_sym_BANG_EQ] = ACTIONS(1602),
    [anon_sym_GT] = ACTIONS(1604),
    [anon_sym_LT] = ACTIONS(1604),
    [anon_sym_GT_EQ] = ACTIONS(1602),
    [anon_sym_LT_EQ] = ACTIONS(1602),
    [anon_sym_DOT] = ACTIONS(1604),
    [anon_sym_DOT_DOT] = ACTIONS(1604),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1602),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1602),
    [anon_sym_COLON_COLON] = ACTIONS(1602),
    [anon_sym_POUND] = ACTIONS(1602),
    [anon_sym_SQUOTE] = ACTIONS(1604),
    [anon_sym_as] = ACTIONS(1604),
    [anon_sym_async] = ACTIONS(1604),
    [anon_sym_auto] = ACTIONS(1604),
    [anon_sym_await] = ACTIONS(1604),
    [anon_sym_become] = ACTIONS(1604),
    [anon_sym_break] = ACTIONS(1604),
    [anon_sym_const] = ACTIONS(1604),
    [anon_sym_continue] = ACTIONS(1604),
    [anon_sym_default] = ACTIONS(1604),
    [anon_sym_enum] = ACTIONS(1604),
    [anon_sym_fn] = ACTIONS(1604),
    [anon_sym_for] = ACTIONS(1604),
    [anon_sym_gen] = ACTIONS(1604),
    [anon_sym_if] = ACTIONS(1604),
    [anon_sym_impl] = ACTIONS(1604),
    [anon_sym_let] = ACTIONS(1604),
    [anon_sym_loop] = ACTIONS(1604),
    [anon_sym_match] = ACTIONS(1604),
    [anon_sym_mod] = ACTIONS(1604),
    [anon_sym_pub] = ACTIONS(1604),
    [anon_sym_return] = ACTIONS(1604),
    [anon_sym_raw] = ACTIONS(1604),
    [anon_sym_static] = ACTIONS(1604),
    [anon_sym_struct] = ACTIONS(1604),
    [anon_sym_trait] = ACTIONS(1604),
    [anon_sym_type] = ACTIONS(1604),
    [anon_sym_union] = ACTIONS(1604),
    [anon_sym_unsafe] = ACTIONS(1604),
    [anon_sym_use] = ACTIONS(1604),
    [anon_sym_while] = ACTIONS(1604),
    [anon_sym_extern] = ACTIONS(1604),
    [anon_sym_safe] = ACTIONS(1604),
    [anon_sym_yield] = ACTIONS(1604),
    [anon_sym_move] = ACTIONS(1604),
    [anon_sym_try] = ACTIONS(1604),
    [sym_integer_literal] = ACTIONS(1602),
    [aux_sym_string_literal_token1] = ACTIONS(1602),
    [sym_char_literal] = ACTIONS(1602),
    [anon_sym_true] = ACTIONS(1604),
    [anon_sym_false] = ACTIONS(1604),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1604),
    [sym_super] = ACTIONS(1604),
    [sym_crate] = ACTIONS(1604),
    [aux_sym_metavariable_token1] = ACTIONS(1602),
    [sym__raw_string_literal_start] = ACTIONS(1602),
    [sym_float_literal] = ACTIONS(1602),
    [sym__outer_block_doc_comment_start] = ACTIONS(1602),
    [sym__outer_line_doc_comment_start] = ACTIONS(1602),
  },
  [STATE(485)] = {
    [sym_line_comment] = STATE(485),
    [sym_block_comment] = STATE(485),
    [ts_builtin_sym_end] = ACTIONS(1606),
    [sym_identifier] = ACTIONS(1608),
    [anon_sym_SEMI] = ACTIONS(1606),
    [anon_sym_macro] = ACTIONS(1608),
    [anon_sym_LPAREN] = ACTIONS(1606),
    [anon_sym_LBRACE] = ACTIONS(1606),
    [anon_sym_RBRACE] = ACTIONS(1606),
    [anon_sym_macro_rules] = ACTIONS(1608),
    [anon_sym_BANG] = ACTIONS(1608),
    [anon_sym_LBRACK] = ACTIONS(1606),
    [anon_sym_PLUS] = ACTIONS(1608),
    [anon_sym_STAR] = ACTIONS(1608),
    [anon_sym_QMARK] = ACTIONS(1606),
    [anon_sym_u8] = ACTIONS(1608),
    [anon_sym_i8] = ACTIONS(1608),
    [anon_sym_u16] = ACTIONS(1608),
    [anon_sym_i16] = ACTIONS(1608),
    [anon_sym_u32] = ACTIONS(1608),
    [anon_sym_i32] = ACTIONS(1608),
    [anon_sym_u64] = ACTIONS(1608),
    [anon_sym_i64] = ACTIONS(1608),
    [anon_sym_u128] = ACTIONS(1608),
    [anon_sym_i128] = ACTIONS(1608),
    [anon_sym_isize] = ACTIONS(1608),
    [anon_sym_usize] = ACTIONS(1608),
    [anon_sym_f32] = ACTIONS(1608),
    [anon_sym_f64] = ACTIONS(1608),
    [anon_sym_bool] = ACTIONS(1608),
    [anon_sym_str] = ACTIONS(1608),
    [anon_sym_char] = ACTIONS(1608),
    [anon_sym_DASH] = ACTIONS(1608),
    [anon_sym_SLASH] = ACTIONS(1608),
    [anon_sym_PERCENT] = ACTIONS(1608),
    [anon_sym_CARET] = ACTIONS(1608),
    [anon_sym_AMP] = ACTIONS(1608),
    [anon_sym_PIPE] = ACTIONS(1608),
    [anon_sym_AMP_AMP] = ACTIONS(1606),
    [anon_sym_PIPE_PIPE] = ACTIONS(1606),
    [anon_sym_LT_LT] = ACTIONS(1608),
    [anon_sym_GT_GT] = ACTIONS(1608),
    [anon_sym_PLUS_EQ] = ACTIONS(1606),
    [anon_sym_DASH_EQ] = ACTIONS(1606),
    [anon_sym_STAR_EQ] = ACTIONS(1606),
    [anon_sym_SLASH_EQ] = ACTIONS(1606),
    [anon_sym_PERCENT_EQ] = ACTIONS(1606),
    [anon_sym_CARET_EQ] = ACTIONS(1606),
    [anon_sym_AMP_EQ] = ACTIONS(1606),
    [anon_sym_PIPE_EQ] = ACTIONS(1606),
    [anon_sym_LT_LT_EQ] = ACTIONS(1606),
    [anon_sym_GT_GT_EQ] = ACTIONS(1606),
    [anon_sym_EQ] = ACTIONS(1608),
    [anon_sym_EQ_EQ] = ACTIONS(1606),
    [anon_sym_BANG_EQ] = ACTIONS(1606),
    [anon_sym_GT] = ACTIONS(1608),
    [anon_sym_LT] = ACTIONS(1608),
    [anon_sym_GT_EQ] = ACTIONS(1606),
    [anon_sym_LT_EQ] = ACTIONS(1606),
    [anon_sym_DOT] = ACTIONS(1608),
    [anon_sym_DOT_DOT] = ACTIONS(1608),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1606),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1606),
    [anon_sym_COLON_COLON] = ACTIONS(1606),
    [anon_sym_POUND] = ACTIONS(1606),
    [anon_sym_SQUOTE] = ACTIONS(1608),
    [anon_sym_as] = ACTIONS(1608),
    [anon_sym_async] = ACTIONS(1608),
    [anon_sym_auto] = ACTIONS(1608),
    [anon_sym_await] = ACTIONS(1608),
    [anon_sym_become] = ACTIONS(1608),
    [anon_sym_break] = ACTIONS(1608),
    [anon_sym_const] = ACTIONS(1608),
    [anon_sym_continue] = ACTIONS(1608),
    [anon_sym_default] = ACTIONS(1608),
    [anon_sym_enum] = ACTIONS(1608),
    [anon_sym_fn] = ACTIONS(1608),
    [anon_sym_for] = ACTIONS(1608),
    [anon_sym_gen] = ACTIONS(1608),
    [anon_sym_if] = ACTIONS(1608),
    [anon_sym_impl] = ACTIONS(1608),
    [anon_sym_let] = ACTIONS(1608),
    [anon_sym_loop] = ACTIONS(1608),
    [anon_sym_match] = ACTIONS(1608),
    [anon_sym_mod] = ACTIONS(1608),
    [anon_sym_pub] = ACTIONS(1608),
    [anon_sym_return] = ACTIONS(1608),
    [anon_sym_raw] = ACTIONS(1608),
    [anon_sym_static] = ACTIONS(1608),
    [anon_sym_struct] = ACTIONS(1608),
    [anon_sym_trait] = ACTIONS(1608),
    [anon_sym_type] = ACTIONS(1608),
    [anon_sym_union] = ACTIONS(1608),
    [anon_sym_unsafe] = ACTIONS(1608),
    [anon_sym_use] = ACTIONS(1608),
    [anon_sym_while] = ACTIONS(1608),
    [anon_sym_extern] = ACTIONS(1608),
    [anon_sym_safe] = ACTIONS(1608),
    [anon_sym_yield] = ACTIONS(1608),
    [anon_sym_move] = ACTIONS(1608),
    [anon_sym_try] = ACTIONS(1608),
    [sym_integer_literal] = ACTIONS(1606),
    [aux_sym_string_literal_token1] = ACTIONS(1606),
    [sym_char_literal] = ACTIONS(1606),
    [anon_sym_true] = ACTIONS(1608),
    [anon_sym_false] = ACTIONS(1608),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1608),
    [sym_super] = ACTIONS(1608),
    [sym_crate] = ACTIONS(1608),
    [aux_sym_metavariable_token1] = ACTIONS(1606),
    [sym__raw_string_literal_start] = ACTIONS(1606),
    [sym_float_literal] = ACTIONS(1606),
    [sym__outer_block_doc_comment_start] = ACTIONS(1606),
    [sym__outer_line_doc_comment_start] = ACTIONS(1606),
  },
  [STATE(486)] = {
    [sym_line_comment] = STATE(486),
    [sym_block_comment] = STATE(486),
    [ts_builtin_sym_end] = ACTIONS(1610),
    [sym_identifier] = ACTIONS(1612),
    [anon_sym_SEMI] = ACTIONS(1610),
    [anon_sym_macro] = ACTIONS(1612),
    [anon_sym_LPAREN] = ACTIONS(1610),
    [anon_sym_LBRACE] = ACTIONS(1610),
    [anon_sym_RBRACE] = ACTIONS(1610),
    [anon_sym_macro_rules] = ACTIONS(1612),
    [anon_sym_BANG] = ACTIONS(1612),
    [anon_sym_LBRACK] = ACTIONS(1610),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1612),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(1612),
    [anon_sym_i8] = ACTIONS(1612),
    [anon_sym_u16] = ACTIONS(1612),
    [anon_sym_i16] = ACTIONS(1612),
    [anon_sym_u32] = ACTIONS(1612),
    [anon_sym_i32] = ACTIONS(1612),
    [anon_sym_u64] = ACTIONS(1612),
    [anon_sym_i64] = ACTIONS(1612),
    [anon_sym_u128] = ACTIONS(1612),
    [anon_sym_i128] = ACTIONS(1612),
    [anon_sym_isize] = ACTIONS(1612),
    [anon_sym_usize] = ACTIONS(1612),
    [anon_sym_f32] = ACTIONS(1612),
    [anon_sym_f64] = ACTIONS(1612),
    [anon_sym_bool] = ACTIONS(1612),
    [anon_sym_str] = ACTIONS(1612),
    [anon_sym_char] = ACTIONS(1612),
    [anon_sym_DASH] = ACTIONS(1612),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(1612),
    [anon_sym_PIPE] = ACTIONS(1612),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(1612),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(1612),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1506),
    [anon_sym_COLON_COLON] = ACTIONS(1610),
    [anon_sym_POUND] = ACTIONS(1610),
    [anon_sym_SQUOTE] = ACTIONS(1612),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_async] = ACTIONS(1612),
    [anon_sym_auto] = ACTIONS(1612),
    [anon_sym_await] = ACTIONS(1612),
    [anon_sym_become] = ACTIONS(1612),
    [anon_sym_break] = ACTIONS(1612),
    [anon_sym_const] = ACTIONS(1612),
    [anon_sym_continue] = ACTIONS(1612),
    [anon_sym_default] = ACTIONS(1612),
    [anon_sym_enum] = ACTIONS(1612),
    [anon_sym_fn] = ACTIONS(1612),
    [anon_sym_for] = ACTIONS(1612),
    [anon_sym_gen] = ACTIONS(1612),
    [anon_sym_if] = ACTIONS(1612),
    [anon_sym_impl] = ACTIONS(1612),
    [anon_sym_let] = ACTIONS(1612),
    [anon_sym_loop] = ACTIONS(1612),
    [anon_sym_match] = ACTIONS(1612),
    [anon_sym_mod] = ACTIONS(1612),
    [anon_sym_pub] = ACTIONS(1612),
    [anon_sym_return] = ACTIONS(1612),
    [anon_sym_raw] = ACTIONS(1612),
    [anon_sym_static] = ACTIONS(1612),
    [anon_sym_struct] = ACTIONS(1612),
    [anon_sym_trait] = ACTIONS(1612),
    [anon_sym_type] = ACTIONS(1612),
    [anon_sym_union] = ACTIONS(1612),
    [anon_sym_unsafe] = ACTIONS(1612),
    [anon_sym_use] = ACTIONS(1612),
    [anon_sym_while] = ACTIONS(1612),
    [anon_sym_extern] = ACTIONS(1612),
    [anon_sym_safe] = ACTIONS(1612),
    [anon_sym_yield] = ACTIONS(1612),
    [anon_sym_move] = ACTIONS(1612),
    [anon_sym_try] = ACTIONS(1612),
    [sym_integer_literal] = ACTIONS(1610),
    [aux_sym_string_literal_token1] = ACTIONS(1610),
    [sym_char_literal] = ACTIONS(1610),
    [anon_sym_true] = ACTIONS(1612),
    [anon_sym_false] = ACTIONS(1612),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1612),
    [sym_super] = ACTIONS(1612),
    [sym_crate] = ACTIONS(1612),
    [aux_sym_metavariable_token1] = ACTIONS(1610),
    [sym__raw_string_literal_start] = ACTIONS(1610),
    [sym_float_literal] = ACTIONS(1610),
    [sym__outer_block_doc_comment_start] = ACTIONS(1610),
    [sym__outer_line_doc_comment_start] = ACTIONS(1610),
  },
  [STATE(487)] = {
    [sym_line_comment] = STATE(487),
    [sym_block_comment] = STATE(487),
    [ts_builtin_sym_end] = ACTIONS(1614),
    [sym_identifier] = ACTIONS(1616),
    [anon_sym_SEMI] = ACTIONS(1614),
    [anon_sym_macro] = ACTIONS(1616),
    [anon_sym_LPAREN] = ACTIONS(1614),
    [anon_sym_LBRACE] = ACTIONS(1614),
    [anon_sym_RBRACE] = ACTIONS(1614),
    [anon_sym_macro_rules] = ACTIONS(1616),
    [anon_sym_BANG] = ACTIONS(1616),
    [anon_sym_LBRACK] = ACTIONS(1614),
    [anon_sym_PLUS] = ACTIONS(1616),
    [anon_sym_STAR] = ACTIONS(1616),
    [anon_sym_QMARK] = ACTIONS(1614),
    [anon_sym_u8] = ACTIONS(1616),
    [anon_sym_i8] = ACTIONS(1616),
    [anon_sym_u16] = ACTIONS(1616),
    [anon_sym_i16] = ACTIONS(1616),
    [anon_sym_u32] = ACTIONS(1616),
    [anon_sym_i32] = ACTIONS(1616),
    [anon_sym_u64] = ACTIONS(1616),
    [anon_sym_i64] = ACTIONS(1616),
    [anon_sym_u128] = ACTIONS(1616),
    [anon_sym_i128] = ACTIONS(1616),
    [anon_sym_isize] = ACTIONS(1616),
    [anon_sym_usize] = ACTIONS(1616),
    [anon_sym_f32] = ACTIONS(1616),
    [anon_sym_f64] = ACTIONS(1616),
    [anon_sym_bool] = ACTIONS(1616),
    [anon_sym_str] = ACTIONS(1616),
    [anon_sym_char] = ACTIONS(1616),
    [anon_sym_DASH] = ACTIONS(1616),
    [anon_sym_SLASH] = ACTIONS(1616),
    [anon_sym_PERCENT] = ACTIONS(1616),
    [anon_sym_CARET] = ACTIONS(1616),
    [anon_sym_AMP] = ACTIONS(1616),
    [anon_sym_PIPE] = ACTIONS(1616),
    [anon_sym_AMP_AMP] = ACTIONS(1614),
    [anon_sym_PIPE_PIPE] = ACTIONS(1614),
    [anon_sym_LT_LT] = ACTIONS(1616),
    [anon_sym_GT_GT] = ACTIONS(1616),
    [anon_sym_PLUS_EQ] = ACTIONS(1614),
    [anon_sym_DASH_EQ] = ACTIONS(1614),
    [anon_sym_STAR_EQ] = ACTIONS(1614),
    [anon_sym_SLASH_EQ] = ACTIONS(1614),
    [anon_sym_PERCENT_EQ] = ACTIONS(1614),
    [anon_sym_CARET_EQ] = ACTIONS(1614),
    [anon_sym_AMP_EQ] = ACTIONS(1614),
    [anon_sym_PIPE_EQ] = ACTIONS(1614),
    [anon_sym_LT_LT_EQ] = ACTIONS(1614),
    [anon_sym_GT_GT_EQ] = ACTIONS(1614),
    [anon_sym_EQ] = ACTIONS(1616),
    [anon_sym_EQ_EQ] = ACTIONS(1614),
    [anon_sym_BANG_EQ] = ACTIONS(1614),
    [anon_sym_GT] = ACTIONS(1616),
    [anon_sym_LT] = ACTIONS(1616),
    [anon_sym_GT_EQ] = ACTIONS(1614),
    [anon_sym_LT_EQ] = ACTIONS(1614),
    [anon_sym_DOT] = ACTIONS(1616),
    [anon_sym_DOT_DOT] = ACTIONS(1616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1614),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1614),
    [anon_sym_COLON_COLON] = ACTIONS(1614),
    [anon_sym_POUND] = ACTIONS(1614),
    [anon_sym_SQUOTE] = ACTIONS(1616),
    [anon_sym_as] = ACTIONS(1616),
    [anon_sym_async] = ACTIONS(1616),
    [anon_sym_auto] = ACTIONS(1616),
    [anon_sym_await] = ACTIONS(1616),
    [anon_sym_become] = ACTIONS(1616),
    [anon_sym_break] = ACTIONS(1616),
    [anon_sym_const] = ACTIONS(1616),
    [anon_sym_continue] = ACTIONS(1616),
    [anon_sym_default] = ACTIONS(1616),
    [anon_sym_enum] = ACTIONS(1616),
    [anon_sym_fn] = ACTIONS(1616),
    [anon_sym_for] = ACTIONS(1616),
    [anon_sym_gen] = ACTIONS(1616),
    [anon_sym_if] = ACTIONS(1616),
    [anon_sym_impl] = ACTIONS(1616),
    [anon_sym_let] = ACTIONS(1616),
    [anon_sym_loop] = ACTIONS(1616),
    [anon_sym_match] = ACTIONS(1616),
    [anon_sym_mod] = ACTIONS(1616),
    [anon_sym_pub] = ACTIONS(1616),
    [anon_sym_return] = ACTIONS(1616),
    [anon_sym_raw] = ACTIONS(1616),
    [anon_sym_static] = ACTIONS(1616),
    [anon_sym_struct] = ACTIONS(1616),
    [anon_sym_trait] = ACTIONS(1616),
    [anon_sym_type] = ACTIONS(1616),
    [anon_sym_union] = ACTIONS(1616),
    [anon_sym_unsafe] = ACTIONS(1616),
    [anon_sym_use] = ACTIONS(1616),
    [anon_sym_while] = ACTIONS(1616),
    [anon_sym_extern] = ACTIONS(1616),
    [anon_sym_safe] = ACTIONS(1616),
    [anon_sym_yield] = ACTIONS(1616),
    [anon_sym_move] = ACTIONS(1616),
    [anon_sym_try] = ACTIONS(1616),
    [sym_integer_literal] = ACTIONS(1614),
    [aux_sym_string_literal_token1] = ACTIONS(1614),
    [sym_char_literal] = ACTIONS(1614),
    [anon_sym_true] = ACTIONS(1616),
    [anon_sym_false] = ACTIONS(1616),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1616),
    [sym_super] = ACTIONS(1616),
    [sym_crate] = ACTIONS(1616),
    [aux_sym_metavariable_token1] = ACTIONS(1614),
    [sym__raw_string_literal_start] = ACTIONS(1614),
    [sym_float_literal] = ACTIONS(1614),
    [sym__outer_block_doc_comment_start] = ACTIONS(1614),
    [sym__outer_line_doc_comment_start] = ACTIONS(1614),
  },
  [STATE(488)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3601),
    [sym_bracketed_type] = STATE(6294),
    [sym_lifetime] = STATE(1437),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4212),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4343),
    [sym_scoped_identifier] = STATE(3864),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3764),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(488),
    [sym_block_comment] = STATE(488),
    [sym_metavariable] = STATE(4046),
    [sym_identifier] = ACTIONS(1462),
    [anon_sym_LPAREN] = ACTIONS(1464),
    [anon_sym_macro_rules] = ACTIONS(1468),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1472),
    [anon_sym_i8] = ACTIONS(1472),
    [anon_sym_u16] = ACTIONS(1472),
    [anon_sym_i16] = ACTIONS(1472),
    [anon_sym_u32] = ACTIONS(1472),
    [anon_sym_i32] = ACTIONS(1472),
    [anon_sym_u64] = ACTIONS(1472),
    [anon_sym_i64] = ACTIONS(1472),
    [anon_sym_u128] = ACTIONS(1472),
    [anon_sym_i128] = ACTIONS(1472),
    [anon_sym_isize] = ACTIONS(1472),
    [anon_sym_usize] = ACTIONS(1472),
    [anon_sym_f32] = ACTIONS(1472),
    [anon_sym_f64] = ACTIONS(1472),
    [anon_sym_bool] = ACTIONS(1472),
    [anon_sym_str] = ACTIONS(1472),
    [anon_sym_char] = ACTIONS(1472),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1474),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1476),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1468),
    [anon_sym_await] = ACTIONS(1468),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1478),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1478),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1468),
    [anon_sym_union] = ACTIONS(1468),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1478),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1618),
    [anon_sym_try] = ACTIONS(1468),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1480),
    [sym_super] = ACTIONS(1480),
    [sym_crate] = ACTIONS(1480),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(489)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3757),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(489),
    [sym_block_comment] = STATE(489),
    [sym_metavariable] = STATE(4014),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(979),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(490)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3601),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(1437),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3764),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(490),
    [sym_block_comment] = STATE(490),
    [sym_metavariable] = STATE(4014),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1620),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(979),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(491)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3757),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(491),
    [sym_block_comment] = STATE(491),
    [sym_metavariable] = STATE(4067),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1622),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1624),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(492)] = {
    [sym_line_comment] = STATE(492),
    [sym_block_comment] = STATE(492),
    [sym_identifier] = ACTIONS(1504),
    [anon_sym_SEMI] = ACTIONS(1506),
    [anon_sym_macro] = ACTIONS(1504),
    [anon_sym_LPAREN] = ACTIONS(1506),
    [anon_sym_LBRACE] = ACTIONS(1502),
    [anon_sym_RBRACE] = ACTIONS(1502),
    [anon_sym_macro_rules] = ACTIONS(1504),
    [anon_sym_BANG] = ACTIONS(1504),
    [anon_sym_LBRACK] = ACTIONS(1506),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1508),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(1504),
    [anon_sym_i8] = ACTIONS(1504),
    [anon_sym_u16] = ACTIONS(1504),
    [anon_sym_i16] = ACTIONS(1504),
    [anon_sym_u32] = ACTIONS(1504),
    [anon_sym_i32] = ACTIONS(1504),
    [anon_sym_u64] = ACTIONS(1504),
    [anon_sym_i64] = ACTIONS(1504),
    [anon_sym_u128] = ACTIONS(1504),
    [anon_sym_i128] = ACTIONS(1504),
    [anon_sym_isize] = ACTIONS(1504),
    [anon_sym_usize] = ACTIONS(1504),
    [anon_sym_f32] = ACTIONS(1504),
    [anon_sym_f64] = ACTIONS(1504),
    [anon_sym_bool] = ACTIONS(1504),
    [anon_sym_str] = ACTIONS(1504),
    [anon_sym_char] = ACTIONS(1504),
    [anon_sym_DASH] = ACTIONS(1508),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(1508),
    [anon_sym_PIPE] = ACTIONS(1508),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(1508),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1506),
    [anon_sym_COLON_COLON] = ACTIONS(1502),
    [anon_sym_POUND] = ACTIONS(1502),
    [anon_sym_SQUOTE] = ACTIONS(1504),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_async] = ACTIONS(1504),
    [anon_sym_auto] = ACTIONS(1504),
    [anon_sym_await] = ACTIONS(1504),
    [anon_sym_become] = ACTIONS(1504),
    [anon_sym_break] = ACTIONS(1504),
    [anon_sym_const] = ACTIONS(1504),
    [anon_sym_continue] = ACTIONS(1504),
    [anon_sym_default] = ACTIONS(1504),
    [anon_sym_enum] = ACTIONS(1504),
    [anon_sym_fn] = ACTIONS(1504),
    [anon_sym_for] = ACTIONS(1504),
    [anon_sym_gen] = ACTIONS(1504),
    [anon_sym_if] = ACTIONS(1504),
    [anon_sym_impl] = ACTIONS(1504),
    [anon_sym_let] = ACTIONS(1504),
    [anon_sym_loop] = ACTIONS(1504),
    [anon_sym_match] = ACTIONS(1504),
    [anon_sym_mod] = ACTIONS(1504),
    [anon_sym_pub] = ACTIONS(1504),
    [anon_sym_return] = ACTIONS(1504),
    [anon_sym_raw] = ACTIONS(1504),
    [anon_sym_static] = ACTIONS(1504),
    [anon_sym_struct] = ACTIONS(1504),
    [anon_sym_trait] = ACTIONS(1504),
    [anon_sym_type] = ACTIONS(1504),
    [anon_sym_union] = ACTIONS(1504),
    [anon_sym_unsafe] = ACTIONS(1504),
    [anon_sym_use] = ACTIONS(1504),
    [anon_sym_while] = ACTIONS(1504),
    [anon_sym_extern] = ACTIONS(1504),
    [anon_sym_safe] = ACTIONS(1504),
    [anon_sym_yield] = ACTIONS(1504),
    [anon_sym_move] = ACTIONS(1504),
    [anon_sym_try] = ACTIONS(1504),
    [sym_integer_literal] = ACTIONS(1502),
    [aux_sym_string_literal_token1] = ACTIONS(1502),
    [sym_char_literal] = ACTIONS(1502),
    [anon_sym_true] = ACTIONS(1504),
    [anon_sym_false] = ACTIONS(1504),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1504),
    [sym_super] = ACTIONS(1504),
    [sym_crate] = ACTIONS(1504),
    [aux_sym_metavariable_token1] = ACTIONS(1502),
    [sym__raw_string_literal_start] = ACTIONS(1502),
    [sym_float_literal] = ACTIONS(1502),
    [sym__outer_block_doc_comment_start] = ACTIONS(1502),
    [sym__outer_line_doc_comment_start] = ACTIONS(1502),
  },
  [STATE(493)] = {
    [sym_line_comment] = STATE(493),
    [sym_block_comment] = STATE(493),
    [sym_identifier] = ACTIONS(1552),
    [anon_sym_SEMI] = ACTIONS(1506),
    [anon_sym_macro] = ACTIONS(1552),
    [anon_sym_LPAREN] = ACTIONS(1506),
    [anon_sym_LBRACE] = ACTIONS(1550),
    [anon_sym_RBRACE] = ACTIONS(1550),
    [anon_sym_macro_rules] = ACTIONS(1552),
    [anon_sym_BANG] = ACTIONS(1552),
    [anon_sym_LBRACK] = ACTIONS(1506),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1508),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(1552),
    [anon_sym_i8] = ACTIONS(1552),
    [anon_sym_u16] = ACTIONS(1552),
    [anon_sym_i16] = ACTIONS(1552),
    [anon_sym_u32] = ACTIONS(1552),
    [anon_sym_i32] = ACTIONS(1552),
    [anon_sym_u64] = ACTIONS(1552),
    [anon_sym_i64] = ACTIONS(1552),
    [anon_sym_u128] = ACTIONS(1552),
    [anon_sym_i128] = ACTIONS(1552),
    [anon_sym_isize] = ACTIONS(1552),
    [anon_sym_usize] = ACTIONS(1552),
    [anon_sym_f32] = ACTIONS(1552),
    [anon_sym_f64] = ACTIONS(1552),
    [anon_sym_bool] = ACTIONS(1552),
    [anon_sym_str] = ACTIONS(1552),
    [anon_sym_char] = ACTIONS(1552),
    [anon_sym_DASH] = ACTIONS(1508),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(1508),
    [anon_sym_PIPE] = ACTIONS(1508),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(1508),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1506),
    [anon_sym_COLON_COLON] = ACTIONS(1550),
    [anon_sym_POUND] = ACTIONS(1550),
    [anon_sym_SQUOTE] = ACTIONS(1552),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_async] = ACTIONS(1552),
    [anon_sym_auto] = ACTIONS(1552),
    [anon_sym_await] = ACTIONS(1552),
    [anon_sym_become] = ACTIONS(1552),
    [anon_sym_break] = ACTIONS(1552),
    [anon_sym_const] = ACTIONS(1552),
    [anon_sym_continue] = ACTIONS(1552),
    [anon_sym_default] = ACTIONS(1552),
    [anon_sym_enum] = ACTIONS(1552),
    [anon_sym_fn] = ACTIONS(1552),
    [anon_sym_for] = ACTIONS(1552),
    [anon_sym_gen] = ACTIONS(1552),
    [anon_sym_if] = ACTIONS(1552),
    [anon_sym_impl] = ACTIONS(1552),
    [anon_sym_let] = ACTIONS(1552),
    [anon_sym_loop] = ACTIONS(1552),
    [anon_sym_match] = ACTIONS(1552),
    [anon_sym_mod] = ACTIONS(1552),
    [anon_sym_pub] = ACTIONS(1552),
    [anon_sym_return] = ACTIONS(1552),
    [anon_sym_raw] = ACTIONS(1552),
    [anon_sym_static] = ACTIONS(1552),
    [anon_sym_struct] = ACTIONS(1552),
    [anon_sym_trait] = ACTIONS(1552),
    [anon_sym_type] = ACTIONS(1552),
    [anon_sym_union] = ACTIONS(1552),
    [anon_sym_unsafe] = ACTIONS(1552),
    [anon_sym_use] = ACTIONS(1552),
    [anon_sym_while] = ACTIONS(1552),
    [anon_sym_extern] = ACTIONS(1552),
    [anon_sym_safe] = ACTIONS(1552),
    [anon_sym_yield] = ACTIONS(1552),
    [anon_sym_move] = ACTIONS(1552),
    [anon_sym_try] = ACTIONS(1552),
    [sym_integer_literal] = ACTIONS(1550),
    [aux_sym_string_literal_token1] = ACTIONS(1550),
    [sym_char_literal] = ACTIONS(1550),
    [anon_sym_true] = ACTIONS(1552),
    [anon_sym_false] = ACTIONS(1552),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1552),
    [sym_super] = ACTIONS(1552),
    [sym_crate] = ACTIONS(1552),
    [aux_sym_metavariable_token1] = ACTIONS(1550),
    [sym__raw_string_literal_start] = ACTIONS(1550),
    [sym_float_literal] = ACTIONS(1550),
    [sym__outer_block_doc_comment_start] = ACTIONS(1550),
    [sym__outer_line_doc_comment_start] = ACTIONS(1550),
  },
  [STATE(494)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6294),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4212),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4343),
    [sym_scoped_identifier] = STATE(3864),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3757),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(494),
    [sym_block_comment] = STATE(494),
    [sym_metavariable] = STATE(4046),
    [sym_identifier] = ACTIONS(1462),
    [anon_sym_LPAREN] = ACTIONS(1464),
    [anon_sym_macro_rules] = ACTIONS(1468),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1472),
    [anon_sym_i8] = ACTIONS(1472),
    [anon_sym_u16] = ACTIONS(1472),
    [anon_sym_i16] = ACTIONS(1472),
    [anon_sym_u32] = ACTIONS(1472),
    [anon_sym_i32] = ACTIONS(1472),
    [anon_sym_u64] = ACTIONS(1472),
    [anon_sym_i64] = ACTIONS(1472),
    [anon_sym_u128] = ACTIONS(1472),
    [anon_sym_i128] = ACTIONS(1472),
    [anon_sym_isize] = ACTIONS(1472),
    [anon_sym_usize] = ACTIONS(1472),
    [anon_sym_f32] = ACTIONS(1472),
    [anon_sym_f64] = ACTIONS(1472),
    [anon_sym_bool] = ACTIONS(1472),
    [anon_sym_str] = ACTIONS(1472),
    [anon_sym_char] = ACTIONS(1472),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1474),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1476),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1468),
    [anon_sym_await] = ACTIONS(1468),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1478),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1478),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1468),
    [anon_sym_union] = ACTIONS(1468),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1478),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1468),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1480),
    [sym_super] = ACTIONS(1480),
    [sym_crate] = ACTIONS(1480),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(495)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3601),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(1441),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3764),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(495),
    [sym_block_comment] = STATE(495),
    [sym_metavariable] = STATE(4014),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1626),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1628),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(496)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6293),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4152),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4371),
    [sym_scoped_identifier] = STATE(3801),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3757),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(496),
    [sym_block_comment] = STATE(496),
    [sym_metavariable] = STATE(4014),
    [sym_identifier] = ACTIONS(905),
    [anon_sym_LPAREN] = ACTIONS(907),
    [anon_sym_macro_rules] = ACTIONS(913),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(923),
    [anon_sym_i8] = ACTIONS(923),
    [anon_sym_u16] = ACTIONS(923),
    [anon_sym_i16] = ACTIONS(923),
    [anon_sym_u32] = ACTIONS(923),
    [anon_sym_i32] = ACTIONS(923),
    [anon_sym_u64] = ACTIONS(923),
    [anon_sym_i64] = ACTIONS(923),
    [anon_sym_u128] = ACTIONS(923),
    [anon_sym_i128] = ACTIONS(923),
    [anon_sym_isize] = ACTIONS(923),
    [anon_sym_usize] = ACTIONS(923),
    [anon_sym_f32] = ACTIONS(923),
    [anon_sym_f64] = ACTIONS(923),
    [anon_sym_bool] = ACTIONS(923),
    [anon_sym_str] = ACTIONS(923),
    [anon_sym_char] = ACTIONS(923),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(939),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(913),
    [anon_sym_await] = ACTIONS(913),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(949),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(949),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(913),
    [anon_sym_union] = ACTIONS(913),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(949),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(913),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1630),
    [sym_super] = ACTIONS(979),
    [sym_crate] = ACTIONS(979),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(497)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3601),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(1437),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3764),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(497),
    [sym_block_comment] = STATE(497),
    [sym_metavariable] = STATE(4067),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1622),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1632),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1035),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(498)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3601),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(1441),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3764),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(498),
    [sym_block_comment] = STATE(498),
    [sym_metavariable] = STATE(4067),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1622),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1634),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1636),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(499)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6286),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(4200),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(4437),
    [sym_scoped_identifier] = STATE(3942),
    [sym_scoped_type_identifier] = STATE(3776),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3757),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(499),
    [sym_block_comment] = STATE(499),
    [sym_metavariable] = STATE(4067),
    [sym_identifier] = ACTIONS(1013),
    [anon_sym_LPAREN] = ACTIONS(1015),
    [anon_sym_macro_rules] = ACTIONS(1021),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(917),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1023),
    [anon_sym_i8] = ACTIONS(1023),
    [anon_sym_u16] = ACTIONS(1023),
    [anon_sym_i16] = ACTIONS(1023),
    [anon_sym_u32] = ACTIONS(1023),
    [anon_sym_i32] = ACTIONS(1023),
    [anon_sym_u64] = ACTIONS(1023),
    [anon_sym_i64] = ACTIONS(1023),
    [anon_sym_u128] = ACTIONS(1023),
    [anon_sym_i128] = ACTIONS(1023),
    [anon_sym_isize] = ACTIONS(1023),
    [anon_sym_usize] = ACTIONS(1023),
    [anon_sym_f32] = ACTIONS(1023),
    [anon_sym_f64] = ACTIONS(1023),
    [anon_sym_bool] = ACTIONS(1023),
    [anon_sym_str] = ACTIONS(1023),
    [anon_sym_char] = ACTIONS(1023),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1622),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1029),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1021),
    [anon_sym_await] = ACTIONS(1021),
    [anon_sym_const] = ACTIONS(947),
    [anon_sym_default] = ACTIONS(1031),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1031),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1021),
    [anon_sym_union] = ACTIONS(1021),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1031),
    [anon_sym_ref] = ACTIONS(963),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1021),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1035),
    [sym_super] = ACTIONS(1035),
    [sym_crate] = ACTIONS(1035),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(500)] = {
    [sym_line_comment] = STATE(500),
    [sym_block_comment] = STATE(500),
    [sym_identifier] = ACTIONS(1638),
    [anon_sym_SEMI] = ACTIONS(1640),
    [anon_sym_LPAREN] = ACTIONS(1640),
    [anon_sym_RPAREN] = ACTIONS(1640),
    [anon_sym_LBRACE] = ACTIONS(1640),
    [anon_sym_RBRACE] = ACTIONS(1640),
    [anon_sym_COMMA] = ACTIONS(1640),
    [anon_sym_macro_rules] = ACTIONS(1638),
    [anon_sym_BANG] = ACTIONS(1638),
    [anon_sym_LBRACK] = ACTIONS(1640),
    [anon_sym_RBRACK] = ACTIONS(1640),
    [anon_sym_COLON] = ACTIONS(1638),
    [anon_sym_PLUS] = ACTIONS(1638),
    [anon_sym_STAR] = ACTIONS(1638),
    [anon_sym_QMARK] = ACTIONS(1640),
    [anon_sym_u8] = ACTIONS(1638),
    [anon_sym_i8] = ACTIONS(1638),
    [anon_sym_u16] = ACTIONS(1638),
    [anon_sym_i16] = ACTIONS(1638),
    [anon_sym_u32] = ACTIONS(1638),
    [anon_sym_i32] = ACTIONS(1638),
    [anon_sym_u64] = ACTIONS(1638),
    [anon_sym_i64] = ACTIONS(1638),
    [anon_sym_u128] = ACTIONS(1638),
    [anon_sym_i128] = ACTIONS(1638),
    [anon_sym_isize] = ACTIONS(1638),
    [anon_sym_usize] = ACTIONS(1638),
    [anon_sym_f32] = ACTIONS(1638),
    [anon_sym_f64] = ACTIONS(1638),
    [anon_sym_bool] = ACTIONS(1638),
    [anon_sym_str] = ACTIONS(1638),
    [anon_sym_char] = ACTIONS(1638),
    [anon_sym_DASH] = ACTIONS(1638),
    [anon_sym_SLASH] = ACTIONS(1638),
    [anon_sym_PERCENT] = ACTIONS(1638),
    [anon_sym_CARET] = ACTIONS(1638),
    [anon_sym_AMP] = ACTIONS(1638),
    [anon_sym_PIPE] = ACTIONS(1638),
    [anon_sym_AMP_AMP] = ACTIONS(1640),
    [anon_sym_PIPE_PIPE] = ACTIONS(1640),
    [anon_sym_LT_LT] = ACTIONS(1638),
    [anon_sym_GT_GT] = ACTIONS(1638),
    [anon_sym_PLUS_EQ] = ACTIONS(1640),
    [anon_sym_DASH_EQ] = ACTIONS(1640),
    [anon_sym_STAR_EQ] = ACTIONS(1640),
    [anon_sym_SLASH_EQ] = ACTIONS(1640),
    [anon_sym_PERCENT_EQ] = ACTIONS(1640),
    [anon_sym_CARET_EQ] = ACTIONS(1640),
    [anon_sym_AMP_EQ] = ACTIONS(1640),
    [anon_sym_PIPE_EQ] = ACTIONS(1640),
    [anon_sym_LT_LT_EQ] = ACTIONS(1640),
    [anon_sym_GT_GT_EQ] = ACTIONS(1640),
    [anon_sym_EQ] = ACTIONS(1638),
    [anon_sym_EQ_EQ] = ACTIONS(1640),
    [anon_sym_BANG_EQ] = ACTIONS(1640),
    [anon_sym_GT] = ACTIONS(1638),
    [anon_sym_LT] = ACTIONS(1638),
    [anon_sym_GT_EQ] = ACTIONS(1640),
    [anon_sym_LT_EQ] = ACTIONS(1640),
    [anon_sym_DOT] = ACTIONS(1638),
    [anon_sym_DOT_DOT] = ACTIONS(1638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1640),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1640),
    [anon_sym_COLON_COLON] = ACTIONS(1640),
    [anon_sym_POUND] = ACTIONS(1640),
    [anon_sym_SQUOTE] = ACTIONS(1638),
    [anon_sym_as] = ACTIONS(1638),
    [anon_sym_async] = ACTIONS(1638),
    [anon_sym_auto] = ACTIONS(1638),
    [anon_sym_await] = ACTIONS(1638),
    [anon_sym_become] = ACTIONS(1638),
    [anon_sym_break] = ACTIONS(1638),
    [anon_sym_const] = ACTIONS(1638),
    [anon_sym_continue] = ACTIONS(1638),
    [anon_sym_default] = ACTIONS(1638),
    [anon_sym_for] = ACTIONS(1638),
    [anon_sym_gen] = ACTIONS(1638),
    [anon_sym_if] = ACTIONS(1638),
    [anon_sym_loop] = ACTIONS(1638),
    [anon_sym_match] = ACTIONS(1638),
    [anon_sym_return] = ACTIONS(1638),
    [anon_sym_raw] = ACTIONS(1638),
    [anon_sym_static] = ACTIONS(1638),
    [anon_sym_union] = ACTIONS(1638),
    [anon_sym_unsafe] = ACTIONS(1638),
    [anon_sym_while] = ACTIONS(1638),
    [anon_sym_safe] = ACTIONS(1638),
    [anon_sym_else] = ACTIONS(1638),
    [anon_sym_yield] = ACTIONS(1638),
    [anon_sym_move] = ACTIONS(1638),
    [anon_sym_try] = ACTIONS(1638),
    [sym_integer_literal] = ACTIONS(1640),
    [aux_sym_string_literal_token1] = ACTIONS(1640),
    [sym_char_literal] = ACTIONS(1640),
    [anon_sym_true] = ACTIONS(1638),
    [anon_sym_false] = ACTIONS(1638),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1638),
    [sym_super] = ACTIONS(1638),
    [sym_crate] = ACTIONS(1638),
    [aux_sym_metavariable_token1] = ACTIONS(1640),
    [sym__raw_string_literal_start] = ACTIONS(1640),
    [sym_float_literal] = ACTIONS(1640),
    [sym__outer_block_doc_comment_start] = ACTIONS(1640),
    [sym__outer_line_doc_comment_start] = ACTIONS(1640),
  },
  [STATE(501)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3767),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(501),
    [sym_block_comment] = STATE(501),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1642),
    [anon_sym_LPAREN] = ACTIONS(1644),
    [anon_sym_LBRACE] = ACTIONS(1644),
    [anon_sym_macro_rules] = ACTIONS(1642),
    [anon_sym_BANG] = ACTIONS(1644),
    [anon_sym_LBRACK] = ACTIONS(1644),
    [anon_sym_STAR] = ACTIONS(1644),
    [anon_sym_u8] = ACTIONS(1642),
    [anon_sym_i8] = ACTIONS(1642),
    [anon_sym_u16] = ACTIONS(1642),
    [anon_sym_i16] = ACTIONS(1642),
    [anon_sym_u32] = ACTIONS(1642),
    [anon_sym_i32] = ACTIONS(1642),
    [anon_sym_u64] = ACTIONS(1642),
    [anon_sym_i64] = ACTIONS(1642),
    [anon_sym_u128] = ACTIONS(1642),
    [anon_sym_i128] = ACTIONS(1642),
    [anon_sym_isize] = ACTIONS(1642),
    [anon_sym_usize] = ACTIONS(1642),
    [anon_sym_f32] = ACTIONS(1642),
    [anon_sym_f64] = ACTIONS(1642),
    [anon_sym_bool] = ACTIONS(1642),
    [anon_sym_str] = ACTIONS(1642),
    [anon_sym_char] = ACTIONS(1642),
    [anon_sym_DASH] = ACTIONS(1642),
    [anon_sym_AMP] = ACTIONS(1644),
    [anon_sym_PIPE] = ACTIONS(1644),
    [anon_sym_LT] = ACTIONS(1644),
    [anon_sym__] = ACTIONS(1642),
    [anon_sym_DOT_DOT] = ACTIONS(1642),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1644),
    [anon_sym_DASH_GT] = ACTIONS(1644),
    [anon_sym_POUND] = ACTIONS(1644),
    [anon_sym_SQUOTE] = ACTIONS(1642),
    [anon_sym_async] = ACTIONS(1642),
    [anon_sym_auto] = ACTIONS(1642),
    [anon_sym_await] = ACTIONS(1642),
    [anon_sym_become] = ACTIONS(1642),
    [anon_sym_break] = ACTIONS(1642),
    [anon_sym_const] = ACTIONS(1642),
    [anon_sym_continue] = ACTIONS(1642),
    [anon_sym_default] = ACTIONS(1642),
    [anon_sym_for] = ACTIONS(1642),
    [anon_sym_gen] = ACTIONS(1642),
    [anon_sym_if] = ACTIONS(1642),
    [anon_sym_loop] = ACTIONS(1642),
    [anon_sym_match] = ACTIONS(1642),
    [anon_sym_return] = ACTIONS(1642),
    [anon_sym_raw] = ACTIONS(1642),
    [anon_sym_static] = ACTIONS(1642),
    [anon_sym_union] = ACTIONS(1642),
    [anon_sym_unsafe] = ACTIONS(1642),
    [anon_sym_while] = ACTIONS(1642),
    [anon_sym_safe] = ACTIONS(1642),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_yield] = ACTIONS(1642),
    [anon_sym_move] = ACTIONS(1642),
    [anon_sym_try] = ACTIONS(1642),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(1644),
    [aux_sym_string_literal_token1] = ACTIONS(1644),
    [sym_char_literal] = ACTIONS(1644),
    [anon_sym_true] = ACTIONS(1642),
    [anon_sym_false] = ACTIONS(1642),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1642),
    [sym_super] = ACTIONS(1642),
    [sym_crate] = ACTIONS(1642),
    [aux_sym_metavariable_token1] = ACTIONS(1644),
    [sym__raw_string_literal_start] = ACTIONS(1644),
    [sym_float_literal] = ACTIONS(1644),
    [sym__outer_block_doc_comment_start] = ACTIONS(1644),
    [sym__outer_line_doc_comment_start] = ACTIONS(1644),
  },
  [STATE(502)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3767),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(502),
    [sym_block_comment] = STATE(502),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1646),
    [anon_sym_LPAREN] = ACTIONS(1648),
    [anon_sym_LBRACE] = ACTIONS(1648),
    [anon_sym_macro_rules] = ACTIONS(1646),
    [anon_sym_BANG] = ACTIONS(1648),
    [anon_sym_LBRACK] = ACTIONS(1648),
    [anon_sym_STAR] = ACTIONS(1648),
    [anon_sym_u8] = ACTIONS(1646),
    [anon_sym_i8] = ACTIONS(1646),
    [anon_sym_u16] = ACTIONS(1646),
    [anon_sym_i16] = ACTIONS(1646),
    [anon_sym_u32] = ACTIONS(1646),
    [anon_sym_i32] = ACTIONS(1646),
    [anon_sym_u64] = ACTIONS(1646),
    [anon_sym_i64] = ACTIONS(1646),
    [anon_sym_u128] = ACTIONS(1646),
    [anon_sym_i128] = ACTIONS(1646),
    [anon_sym_isize] = ACTIONS(1646),
    [anon_sym_usize] = ACTIONS(1646),
    [anon_sym_f32] = ACTIONS(1646),
    [anon_sym_f64] = ACTIONS(1646),
    [anon_sym_bool] = ACTIONS(1646),
    [anon_sym_str] = ACTIONS(1646),
    [anon_sym_char] = ACTIONS(1646),
    [anon_sym_DASH] = ACTIONS(1646),
    [anon_sym_AMP] = ACTIONS(1648),
    [anon_sym_PIPE] = ACTIONS(1648),
    [anon_sym_LT] = ACTIONS(1648),
    [anon_sym__] = ACTIONS(1646),
    [anon_sym_DOT_DOT] = ACTIONS(1646),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1648),
    [anon_sym_DASH_GT] = ACTIONS(1648),
    [anon_sym_POUND] = ACTIONS(1648),
    [anon_sym_SQUOTE] = ACTIONS(1646),
    [anon_sym_async] = ACTIONS(1646),
    [anon_sym_auto] = ACTIONS(1646),
    [anon_sym_await] = ACTIONS(1646),
    [anon_sym_become] = ACTIONS(1646),
    [anon_sym_break] = ACTIONS(1646),
    [anon_sym_const] = ACTIONS(1646),
    [anon_sym_continue] = ACTIONS(1646),
    [anon_sym_default] = ACTIONS(1646),
    [anon_sym_for] = ACTIONS(1646),
    [anon_sym_gen] = ACTIONS(1646),
    [anon_sym_if] = ACTIONS(1646),
    [anon_sym_loop] = ACTIONS(1646),
    [anon_sym_match] = ACTIONS(1646),
    [anon_sym_return] = ACTIONS(1646),
    [anon_sym_raw] = ACTIONS(1646),
    [anon_sym_static] = ACTIONS(1646),
    [anon_sym_union] = ACTIONS(1646),
    [anon_sym_unsafe] = ACTIONS(1646),
    [anon_sym_while] = ACTIONS(1646),
    [anon_sym_safe] = ACTIONS(1646),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_yield] = ACTIONS(1646),
    [anon_sym_move] = ACTIONS(1646),
    [anon_sym_try] = ACTIONS(1646),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(1648),
    [aux_sym_string_literal_token1] = ACTIONS(1648),
    [sym_char_literal] = ACTIONS(1648),
    [anon_sym_true] = ACTIONS(1646),
    [anon_sym_false] = ACTIONS(1646),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1646),
    [sym_super] = ACTIONS(1646),
    [sym_crate] = ACTIONS(1646),
    [aux_sym_metavariable_token1] = ACTIONS(1648),
    [sym__raw_string_literal_start] = ACTIONS(1648),
    [sym_float_literal] = ACTIONS(1648),
    [sym__outer_block_doc_comment_start] = ACTIONS(1648),
    [sym__outer_line_doc_comment_start] = ACTIONS(1648),
  },
  [STATE(503)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3767),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(503),
    [sym_block_comment] = STATE(503),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1650),
    [anon_sym_LPAREN] = ACTIONS(1652),
    [anon_sym_LBRACE] = ACTIONS(1652),
    [anon_sym_macro_rules] = ACTIONS(1650),
    [anon_sym_BANG] = ACTIONS(1652),
    [anon_sym_LBRACK] = ACTIONS(1652),
    [anon_sym_STAR] = ACTIONS(1652),
    [anon_sym_u8] = ACTIONS(1650),
    [anon_sym_i8] = ACTIONS(1650),
    [anon_sym_u16] = ACTIONS(1650),
    [anon_sym_i16] = ACTIONS(1650),
    [anon_sym_u32] = ACTIONS(1650),
    [anon_sym_i32] = ACTIONS(1650),
    [anon_sym_u64] = ACTIONS(1650),
    [anon_sym_i64] = ACTIONS(1650),
    [anon_sym_u128] = ACTIONS(1650),
    [anon_sym_i128] = ACTIONS(1650),
    [anon_sym_isize] = ACTIONS(1650),
    [anon_sym_usize] = ACTIONS(1650),
    [anon_sym_f32] = ACTIONS(1650),
    [anon_sym_f64] = ACTIONS(1650),
    [anon_sym_bool] = ACTIONS(1650),
    [anon_sym_str] = ACTIONS(1650),
    [anon_sym_char] = ACTIONS(1650),
    [anon_sym_DASH] = ACTIONS(1650),
    [anon_sym_AMP] = ACTIONS(1652),
    [anon_sym_PIPE] = ACTIONS(1652),
    [anon_sym_LT] = ACTIONS(1652),
    [anon_sym__] = ACTIONS(1650),
    [anon_sym_DOT_DOT] = ACTIONS(1650),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1652),
    [anon_sym_DASH_GT] = ACTIONS(1652),
    [anon_sym_POUND] = ACTIONS(1652),
    [anon_sym_SQUOTE] = ACTIONS(1650),
    [anon_sym_async] = ACTIONS(1650),
    [anon_sym_auto] = ACTIONS(1650),
    [anon_sym_await] = ACTIONS(1650),
    [anon_sym_become] = ACTIONS(1650),
    [anon_sym_break] = ACTIONS(1650),
    [anon_sym_const] = ACTIONS(1650),
    [anon_sym_continue] = ACTIONS(1650),
    [anon_sym_default] = ACTIONS(1650),
    [anon_sym_for] = ACTIONS(1650),
    [anon_sym_gen] = ACTIONS(1650),
    [anon_sym_if] = ACTIONS(1650),
    [anon_sym_loop] = ACTIONS(1650),
    [anon_sym_match] = ACTIONS(1650),
    [anon_sym_return] = ACTIONS(1650),
    [anon_sym_raw] = ACTIONS(1650),
    [anon_sym_static] = ACTIONS(1650),
    [anon_sym_union] = ACTIONS(1650),
    [anon_sym_unsafe] = ACTIONS(1650),
    [anon_sym_while] = ACTIONS(1650),
    [anon_sym_safe] = ACTIONS(1650),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_yield] = ACTIONS(1650),
    [anon_sym_move] = ACTIONS(1650),
    [anon_sym_try] = ACTIONS(1650),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(1652),
    [aux_sym_string_literal_token1] = ACTIONS(1652),
    [sym_char_literal] = ACTIONS(1652),
    [anon_sym_true] = ACTIONS(1650),
    [anon_sym_false] = ACTIONS(1650),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1650),
    [sym_super] = ACTIONS(1650),
    [sym_crate] = ACTIONS(1650),
    [aux_sym_metavariable_token1] = ACTIONS(1652),
    [sym__raw_string_literal_start] = ACTIONS(1652),
    [sym_float_literal] = ACTIONS(1652),
    [sym__outer_block_doc_comment_start] = ACTIONS(1652),
    [sym__outer_line_doc_comment_start] = ACTIONS(1652),
  },
  [STATE(504)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3769),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(504),
    [sym_block_comment] = STATE(504),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1654),
    [anon_sym_LPAREN] = ACTIONS(1656),
    [anon_sym_LBRACE] = ACTIONS(1656),
    [anon_sym_macro_rules] = ACTIONS(1654),
    [anon_sym_BANG] = ACTIONS(1656),
    [anon_sym_LBRACK] = ACTIONS(1656),
    [anon_sym_STAR] = ACTIONS(1656),
    [anon_sym_u8] = ACTIONS(1654),
    [anon_sym_i8] = ACTIONS(1654),
    [anon_sym_u16] = ACTIONS(1654),
    [anon_sym_i16] = ACTIONS(1654),
    [anon_sym_u32] = ACTIONS(1654),
    [anon_sym_i32] = ACTIONS(1654),
    [anon_sym_u64] = ACTIONS(1654),
    [anon_sym_i64] = ACTIONS(1654),
    [anon_sym_u128] = ACTIONS(1654),
    [anon_sym_i128] = ACTIONS(1654),
    [anon_sym_isize] = ACTIONS(1654),
    [anon_sym_usize] = ACTIONS(1654),
    [anon_sym_f32] = ACTIONS(1654),
    [anon_sym_f64] = ACTIONS(1654),
    [anon_sym_bool] = ACTIONS(1654),
    [anon_sym_str] = ACTIONS(1654),
    [anon_sym_char] = ACTIONS(1654),
    [anon_sym_DASH] = ACTIONS(1654),
    [anon_sym_AMP] = ACTIONS(1656),
    [anon_sym_PIPE] = ACTIONS(1656),
    [anon_sym_LT] = ACTIONS(1656),
    [anon_sym__] = ACTIONS(1654),
    [anon_sym_DOT_DOT] = ACTIONS(1654),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1656),
    [anon_sym_DASH_GT] = ACTIONS(1656),
    [anon_sym_POUND] = ACTIONS(1656),
    [anon_sym_SQUOTE] = ACTIONS(1654),
    [anon_sym_async] = ACTIONS(1654),
    [anon_sym_auto] = ACTIONS(1654),
    [anon_sym_await] = ACTIONS(1654),
    [anon_sym_become] = ACTIONS(1654),
    [anon_sym_break] = ACTIONS(1654),
    [anon_sym_const] = ACTIONS(1654),
    [anon_sym_continue] = ACTIONS(1654),
    [anon_sym_default] = ACTIONS(1654),
    [anon_sym_for] = ACTIONS(1654),
    [anon_sym_gen] = ACTIONS(1654),
    [anon_sym_if] = ACTIONS(1654),
    [anon_sym_loop] = ACTIONS(1654),
    [anon_sym_match] = ACTIONS(1654),
    [anon_sym_return] = ACTIONS(1654),
    [anon_sym_raw] = ACTIONS(1654),
    [anon_sym_static] = ACTIONS(1654),
    [anon_sym_union] = ACTIONS(1654),
    [anon_sym_unsafe] = ACTIONS(1654),
    [anon_sym_while] = ACTIONS(1654),
    [anon_sym_safe] = ACTIONS(1654),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_yield] = ACTIONS(1654),
    [anon_sym_move] = ACTIONS(1654),
    [anon_sym_try] = ACTIONS(1654),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(1656),
    [aux_sym_string_literal_token1] = ACTIONS(1656),
    [sym_char_literal] = ACTIONS(1656),
    [anon_sym_true] = ACTIONS(1654),
    [anon_sym_false] = ACTIONS(1654),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1654),
    [sym_super] = ACTIONS(1654),
    [sym_crate] = ACTIONS(1654),
    [aux_sym_metavariable_token1] = ACTIONS(1656),
    [sym__raw_string_literal_start] = ACTIONS(1656),
    [sym_float_literal] = ACTIONS(1656),
    [sym__outer_block_doc_comment_start] = ACTIONS(1656),
    [sym__outer_line_doc_comment_start] = ACTIONS(1656),
  },
  [STATE(505)] = {
    [sym_line_comment] = STATE(505),
    [sym_block_comment] = STATE(505),
    [sym_identifier] = ACTIONS(1638),
    [anon_sym_LPAREN] = ACTIONS(1640),
    [anon_sym_LBRACE] = ACTIONS(1640),
    [anon_sym_EQ_GT] = ACTIONS(1640),
    [anon_sym_macro_rules] = ACTIONS(1638),
    [anon_sym_BANG] = ACTIONS(1638),
    [anon_sym_LBRACK] = ACTIONS(1640),
    [anon_sym_COLON] = ACTIONS(1638),
    [anon_sym_PLUS] = ACTIONS(1638),
    [anon_sym_STAR] = ACTIONS(1638),
    [anon_sym_QMARK] = ACTIONS(1640),
    [anon_sym_u8] = ACTIONS(1638),
    [anon_sym_i8] = ACTIONS(1638),
    [anon_sym_u16] = ACTIONS(1638),
    [anon_sym_i16] = ACTIONS(1638),
    [anon_sym_u32] = ACTIONS(1638),
    [anon_sym_i32] = ACTIONS(1638),
    [anon_sym_u64] = ACTIONS(1638),
    [anon_sym_i64] = ACTIONS(1638),
    [anon_sym_u128] = ACTIONS(1638),
    [anon_sym_i128] = ACTIONS(1638),
    [anon_sym_isize] = ACTIONS(1638),
    [anon_sym_usize] = ACTIONS(1638),
    [anon_sym_f32] = ACTIONS(1638),
    [anon_sym_f64] = ACTIONS(1638),
    [anon_sym_bool] = ACTIONS(1638),
    [anon_sym_str] = ACTIONS(1638),
    [anon_sym_char] = ACTIONS(1638),
    [anon_sym_DASH] = ACTIONS(1638),
    [anon_sym_SLASH] = ACTIONS(1638),
    [anon_sym_PERCENT] = ACTIONS(1638),
    [anon_sym_CARET] = ACTIONS(1638),
    [anon_sym_AMP] = ACTIONS(1638),
    [anon_sym_PIPE] = ACTIONS(1638),
    [anon_sym_AMP_AMP] = ACTIONS(1640),
    [anon_sym_PIPE_PIPE] = ACTIONS(1640),
    [anon_sym_LT_LT] = ACTIONS(1638),
    [anon_sym_GT_GT] = ACTIONS(1638),
    [anon_sym_PLUS_EQ] = ACTIONS(1640),
    [anon_sym_DASH_EQ] = ACTIONS(1640),
    [anon_sym_STAR_EQ] = ACTIONS(1640),
    [anon_sym_SLASH_EQ] = ACTIONS(1640),
    [anon_sym_PERCENT_EQ] = ACTIONS(1640),
    [anon_sym_CARET_EQ] = ACTIONS(1640),
    [anon_sym_AMP_EQ] = ACTIONS(1640),
    [anon_sym_PIPE_EQ] = ACTIONS(1640),
    [anon_sym_LT_LT_EQ] = ACTIONS(1640),
    [anon_sym_GT_GT_EQ] = ACTIONS(1640),
    [anon_sym_EQ] = ACTIONS(1638),
    [anon_sym_EQ_EQ] = ACTIONS(1640),
    [anon_sym_BANG_EQ] = ACTIONS(1640),
    [anon_sym_GT] = ACTIONS(1638),
    [anon_sym_LT] = ACTIONS(1638),
    [anon_sym_GT_EQ] = ACTIONS(1640),
    [anon_sym_LT_EQ] = ACTIONS(1640),
    [anon_sym_DOT] = ACTIONS(1638),
    [anon_sym_DOT_DOT] = ACTIONS(1638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1640),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1640),
    [anon_sym_COLON_COLON] = ACTIONS(1640),
    [anon_sym_POUND] = ACTIONS(1640),
    [anon_sym_SQUOTE] = ACTIONS(1638),
    [anon_sym_as] = ACTIONS(1638),
    [anon_sym_async] = ACTIONS(1638),
    [anon_sym_auto] = ACTIONS(1638),
    [anon_sym_await] = ACTIONS(1638),
    [anon_sym_become] = ACTIONS(1638),
    [anon_sym_break] = ACTIONS(1638),
    [anon_sym_const] = ACTIONS(1638),
    [anon_sym_continue] = ACTIONS(1638),
    [anon_sym_default] = ACTIONS(1638),
    [anon_sym_for] = ACTIONS(1638),
    [anon_sym_gen] = ACTIONS(1638),
    [anon_sym_if] = ACTIONS(1638),
    [anon_sym_loop] = ACTIONS(1638),
    [anon_sym_match] = ACTIONS(1638),
    [anon_sym_return] = ACTIONS(1638),
    [anon_sym_raw] = ACTIONS(1638),
    [anon_sym_static] = ACTIONS(1638),
    [anon_sym_union] = ACTIONS(1638),
    [anon_sym_unsafe] = ACTIONS(1638),
    [anon_sym_while] = ACTIONS(1638),
    [anon_sym_safe] = ACTIONS(1638),
    [anon_sym_yield] = ACTIONS(1638),
    [anon_sym_move] = ACTIONS(1638),
    [anon_sym_try] = ACTIONS(1638),
    [sym_integer_literal] = ACTIONS(1640),
    [aux_sym_string_literal_token1] = ACTIONS(1640),
    [sym_char_literal] = ACTIONS(1640),
    [anon_sym_true] = ACTIONS(1638),
    [anon_sym_false] = ACTIONS(1638),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1638),
    [sym_super] = ACTIONS(1638),
    [sym_crate] = ACTIONS(1638),
    [aux_sym_metavariable_token1] = ACTIONS(1640),
    [sym__raw_string_literal_start] = ACTIONS(1640),
    [sym_float_literal] = ACTIONS(1640),
    [sym__outer_block_doc_comment_start] = ACTIONS(1640),
    [sym__outer_line_doc_comment_start] = ACTIONS(1640),
  },
  [STATE(506)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(5969),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(506),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(506),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(507),
    [aux_sym_match_block_repeat2] = STATE(535),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1662),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(507)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6469),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(507),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(507),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(1945),
    [aux_sym_match_block_repeat2] = STATE(531),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1708),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(508)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4077),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4077),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4285),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4285),
    [sym__literal] = STATE(4285),
    [sym_negative_literal] = STATE(4285),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(508),
    [sym_block_comment] = STATE(508),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_COMMA] = ACTIONS(1716),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1730),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(509)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6070),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(509),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(509),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(1945),
    [aux_sym_match_block_repeat2] = STATE(532),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1752),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(510)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4113),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4113),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4373),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4373),
    [sym__literal] = STATE(4373),
    [sym_negative_literal] = STATE(4373),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(510),
    [sym_block_comment] = STATE(510),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_COMMA] = ACTIONS(1754),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1756),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(511)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6389),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(511),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(511),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(1945),
    [aux_sym_match_block_repeat2] = STATE(538),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1758),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(512)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6026),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(512),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(512),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(1945),
    [aux_sym_match_block_repeat2] = STATE(536),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1760),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(513)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(5961),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(513),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(513),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(512),
    [aux_sym_match_block_repeat2] = STATE(534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1762),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(514)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4098),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4098),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4362),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4362),
    [sym__literal] = STATE(4362),
    [sym_negative_literal] = STATE(4362),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(514),
    [sym_block_comment] = STATE(514),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_COMMA] = ACTIONS(1764),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1766),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(515)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6362),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(515),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(515),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(509),
    [aux_sym_match_block_repeat2] = STATE(533),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1768),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(516)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1368),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6355),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(516),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(516),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat1] = STATE(511),
    [aux_sym_match_block_repeat2] = STATE(537),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_RBRACE] = ACTIONS(1770),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1684),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(517)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4207),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4207),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4384),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4384),
    [sym__literal] = STATE(4384),
    [sym_negative_literal] = STATE(4384),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(517),
    [sym_block_comment] = STATE(517),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_COMMA] = ACTIONS(1772),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1774),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(518)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(518),
    [sym_block_comment] = STATE(518),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1776),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(519)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(519),
    [sym_block_comment] = STATE(519),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1778),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(520)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(520),
    [sym_block_comment] = STATE(520),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1780),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(521)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(521),
    [sym_block_comment] = STATE(521),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1782),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(522)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(522),
    [sym_block_comment] = STATE(522),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1784),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(523)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(523),
    [sym_block_comment] = STATE(523),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1786),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(524)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(524),
    [sym_block_comment] = STATE(524),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1788),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(525)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(525),
    [sym_block_comment] = STATE(525),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1790),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(526)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(526),
    [sym_block_comment] = STATE(526),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1792),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(527)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(527),
    [sym_block_comment] = STATE(527),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1794),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(528)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(528),
    [sym_block_comment] = STATE(528),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1796),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(529)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(529),
    [sym_block_comment] = STATE(529),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1798),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(530)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4271),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(4271),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_type_binding] = STATE(4556),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_label] = STATE(6417),
    [sym_block] = STATE(4556),
    [sym__literal] = STATE(4556),
    [sym_negative_literal] = STATE(4556),
    [sym_string_literal] = STATE(5065),
    [sym_raw_string_literal] = STATE(5065),
    [sym_boolean_literal] = STATE(5065),
    [sym_line_comment] = STATE(530),
    [sym_block_comment] = STATE(530),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(1710),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(1714),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1734),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [sym_integer_literal] = ACTIONS(1748),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(1748),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(1748),
  },
  [STATE(531)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6494),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(531),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(531),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(532)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6097),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(532),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(532),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(533)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6070),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(533),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(533),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(534)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6026),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(534),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(534),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(535)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6469),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(535),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(535),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(536)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6084),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(536),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(536),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(537)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6389),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(537),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(537),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(538)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1384),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_last_match_arm] = STATE(6402),
    [sym_match_pattern] = STATE(6365),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(538),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(538),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(539)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1379),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_arm] = STATE(1980),
    [sym_match_pattern] = STATE(6244),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(539),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(539),
    [sym_metavariable] = STATE(4534),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [aux_sym_match_block_repeat2] = STATE(539),
    [sym_identifier] = ACTIONS(1802),
    [anon_sym_LPAREN] = ACTIONS(1805),
    [anon_sym_macro_rules] = ACTIONS(1808),
    [anon_sym_LBRACK] = ACTIONS(1811),
    [anon_sym_u8] = ACTIONS(1814),
    [anon_sym_i8] = ACTIONS(1814),
    [anon_sym_u16] = ACTIONS(1814),
    [anon_sym_i16] = ACTIONS(1814),
    [anon_sym_u32] = ACTIONS(1814),
    [anon_sym_i32] = ACTIONS(1814),
    [anon_sym_u64] = ACTIONS(1814),
    [anon_sym_i64] = ACTIONS(1814),
    [anon_sym_u128] = ACTIONS(1814),
    [anon_sym_i128] = ACTIONS(1814),
    [anon_sym_isize] = ACTIONS(1814),
    [anon_sym_usize] = ACTIONS(1814),
    [anon_sym_f32] = ACTIONS(1814),
    [anon_sym_f64] = ACTIONS(1814),
    [anon_sym_bool] = ACTIONS(1814),
    [anon_sym_str] = ACTIONS(1814),
    [anon_sym_char] = ACTIONS(1814),
    [anon_sym_DASH] = ACTIONS(1817),
    [anon_sym_AMP] = ACTIONS(1820),
    [anon_sym_PIPE] = ACTIONS(1823),
    [anon_sym_LT] = ACTIONS(1826),
    [anon_sym__] = ACTIONS(1829),
    [anon_sym_DOT_DOT] = ACTIONS(1832),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1835),
    [anon_sym_COLON_COLON] = ACTIONS(1838),
    [anon_sym_POUND] = ACTIONS(1841),
    [anon_sym_auto] = ACTIONS(1808),
    [anon_sym_await] = ACTIONS(1808),
    [anon_sym_const] = ACTIONS(1844),
    [anon_sym_default] = ACTIONS(1808),
    [anon_sym_gen] = ACTIONS(1808),
    [anon_sym_raw] = ACTIONS(1808),
    [anon_sym_union] = ACTIONS(1808),
    [anon_sym_safe] = ACTIONS(1808),
    [anon_sym_ref] = ACTIONS(1847),
    [sym_mutable_specifier] = ACTIONS(1850),
    [anon_sym_try] = ACTIONS(1808),
    [anon_sym_box] = ACTIONS(1853),
    [sym_integer_literal] = ACTIONS(1856),
    [aux_sym_string_literal_token1] = ACTIONS(1859),
    [sym_char_literal] = ACTIONS(1856),
    [anon_sym_true] = ACTIONS(1862),
    [anon_sym_false] = ACTIONS(1862),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1865),
    [sym_super] = ACTIONS(1865),
    [sym_crate] = ACTIONS(1865),
    [aux_sym_metavariable_token1] = ACTIONS(1868),
    [sym__raw_string_literal_start] = ACTIONS(1871),
    [sym_float_literal] = ACTIONS(1856),
    [sym__outer_block_doc_comment_start] = ACTIONS(1874),
    [sym__outer_line_doc_comment_start] = ACTIONS(1877),
  },
  [STATE(540)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1967),
    [sym_attributes] = STATE(3848),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_const_parameter] = STATE(5209),
    [sym_type_parameter] = STATE(5209),
    [sym_lifetime_parameter] = STATE(5209),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5127),
    [sym_bracketed_type] = STATE(6234),
    [sym_qualified_type] = STATE(6139),
    [sym_lifetime] = STATE(4408),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(540),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(540),
    [sym_metavariable] = STATE(4176),
    [aux_sym_attributes_repeat1] = STATE(1950),
    [sym_identifier] = ACTIONS(1880),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_COMMA] = ACTIONS(1882),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_GT] = ACTIONS(1884),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(1886),
    [anon_sym_SQUOTE] = ACTIONS(1888),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(1890),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(1892),
    [sym__outer_line_doc_comment_start] = ACTIONS(1894),
  },
  [STATE(541)] = {
    [sym_empty_statement] = STATE(2524),
    [sym__declaration_statement] = STATE(2422),
    [sym_macro_definition_v2] = STATE(2524),
    [sym_macro_definition] = STATE(2524),
    [sym_attribute_item] = STATE(1967),
    [sym_attributes] = STATE(1952),
    [sym_inner_attribute_item] = STATE(2524),
    [sym_mod_item] = STATE(2524),
    [sym_foreign_mod_item] = STATE(2524),
    [sym_struct_item] = STATE(2524),
    [sym_union_item] = STATE(2524),
    [sym_enum_item] = STATE(2524),
    [sym_extern_crate_declaration] = STATE(2524),
    [sym_const_item] = STATE(2524),
    [sym_static_item] = STATE(2524),
    [sym_type_item] = STATE(2524),
    [sym_function_item] = STATE(2524),
    [sym_function_signature_item] = STATE(2524),
    [sym_function_modifiers] = STATE(6437),
    [aux_sym__item_modifiers] = STATE(3791),
    [sym_impl_item] = STATE(2524),
    [sym_trait_item] = STATE(2524),
    [sym_let_declaration] = STATE(2524),
    [sym_use_declaration] = STATE(2524),
    [sym_extern_modifier] = STATE(4451),
    [sym_visibility_modifier] = STATE(3500),
    [sym_bracketed_type] = STATE(6226),
    [sym_generic_type_with_turbofish] = STATE(6299),
    [sym_macro_invocation_item] = STATE(2524),
    [sym_macro_invocation] = STATE(2426),
    [sym_scoped_identifier] = STATE(5653),
    [sym_line_comment] = STATE(541),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(541),
    [sym_metavariable] = STATE(6226),
    [aux_sym_attributes_repeat1] = STATE(1950),
    [aux_sym_declaration_list_repeat1] = STATE(546),
    [sym_identifier] = ACTIONS(1896),
    [anon_sym_SEMI] = ACTIONS(1898),
    [anon_sym_macro] = ACTIONS(1900),
    [anon_sym_RBRACE] = ACTIONS(1902),
    [anon_sym_macro_rules] = ACTIONS(1904),
    [anon_sym_u8] = ACTIONS(1906),
    [anon_sym_i8] = ACTIONS(1906),
    [anon_sym_u16] = ACTIONS(1906),
    [anon_sym_i16] = ACTIONS(1906),
    [anon_sym_u32] = ACTIONS(1906),
    [anon_sym_i32] = ACTIONS(1906),
    [anon_sym_u64] = ACTIONS(1906),
    [anon_sym_i64] = ACTIONS(1906),
    [anon_sym_u128] = ACTIONS(1906),
    [anon_sym_i128] = ACTIONS(1906),
    [anon_sym_isize] = ACTIONS(1906),
    [anon_sym_usize] = ACTIONS(1906),
    [anon_sym_f32] = ACTIONS(1906),
    [anon_sym_f64] = ACTIONS(1906),
    [anon_sym_bool] = ACTIONS(1906),
    [anon_sym_str] = ACTIONS(1906),
    [anon_sym_char] = ACTIONS(1906),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1908),
    [anon_sym_POUND] = ACTIONS(1910),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1912),
    [anon_sym_await] = ACTIONS(1914),
    [anon_sym_const] = ACTIONS(1916),
    [anon_sym_default] = ACTIONS(1918),
    [anon_sym_enum] = ACTIONS(1920),
    [anon_sym_fn] = ACTIONS(1922),
    [anon_sym_gen] = ACTIONS(1918),
    [anon_sym_impl] = ACTIONS(1924),
    [anon_sym_let] = ACTIONS(1926),
    [anon_sym_mod] = ACTIONS(1928),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_raw] = ACTIONS(1914),
    [anon_sym_static] = ACTIONS(1930),
    [anon_sym_struct] = ACTIONS(1932),
    [anon_sym_trait] = ACTIONS(1934),
    [anon_sym_type] = ACTIONS(1936),
    [anon_sym_union] = ACTIONS(1938),
    [anon_sym_unsafe] = ACTIONS(1940),
    [anon_sym_use] = ACTIONS(1942),
    [anon_sym_extern] = ACTIONS(1944),
    [anon_sym_safe] = ACTIONS(1946),
    [anon_sym_try] = ACTIONS(1914),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1948),
    [sym_super] = ACTIONS(1948),
    [sym_crate] = ACTIONS(1950),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__outer_block_doc_comment_start] = ACTIONS(1892),
    [sym__outer_line_doc_comment_start] = ACTIONS(1894),
  },
  [STATE(542)] = {
    [sym_empty_statement] = STATE(2524),
    [sym__declaration_statement] = STATE(2422),
    [sym_macro_definition_v2] = STATE(2524),
    [sym_macro_definition] = STATE(2524),
    [sym_attribute_item] = STATE(1967),
    [sym_attributes] = STATE(1952),
    [sym_inner_attribute_item] = STATE(2524),
    [sym_mod_item] = STATE(2524),
    [sym_foreign_mod_item] = STATE(2524),
    [sym_struct_item] = STATE(2524),
    [sym_union_item] = STATE(2524),
    [sym_enum_item] = STATE(2524),
    [sym_extern_crate_declaration] = STATE(2524),
    [sym_const_item] = STATE(2524),
    [sym_static_item] = STATE(2524),
    [sym_type_item] = STATE(2524),
    [sym_function_item] = STATE(2524),
    [sym_function_signature_item] = STATE(2524),
    [sym_function_modifiers] = STATE(6437),
    [aux_sym__item_modifiers] = STATE(3791),
    [sym_impl_item] = STATE(2524),
    [sym_trait_item] = STATE(2524),
    [sym_let_declaration] = STATE(2524),
    [sym_use_declaration] = STATE(2524),
    [sym_extern_modifier] = STATE(4451),
    [sym_visibility_modifier] = STATE(3500),
    [sym_bracketed_type] = STATE(6226),
    [sym_generic_type_with_turbofish] = STATE(6299),
    [sym_macro_invocation_item] = STATE(2524),
    [sym_macro_invocation] = STATE(2426),
    [sym_scoped_identifier] = STATE(5653),
    [sym_line_comment] = STATE(542),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(542),
    [sym_metavariable] = STATE(6226),
    [aux_sym_attributes_repeat1] = STATE(1950),
    [aux_sym_declaration_list_repeat1] = STATE(545),
    [sym_identifier] = ACTIONS(1896),
    [anon_sym_SEMI] = ACTIONS(1898),
    [anon_sym_macro] = ACTIONS(1900),
    [anon_sym_RBRACE] = ACTIONS(1952),
    [anon_sym_macro_rules] = ACTIONS(1904),
    [anon_sym_u8] = ACTIONS(1906),
    [anon_sym_i8] = ACTIONS(1906),
    [anon_sym_u16] = ACTIONS(1906),
    [anon_sym_i16] = ACTIONS(1906),
    [anon_sym_u32] = ACTIONS(1906),
    [anon_sym_i32] = ACTIONS(1906),
    [anon_sym_u64] = ACTIONS(1906),
    [anon_sym_i64] = ACTIONS(1906),
    [anon_sym_u128] = ACTIONS(1906),
    [anon_sym_i128] = ACTIONS(1906),
    [anon_sym_isize] = ACTIONS(1906),
    [anon_sym_usize] = ACTIONS(1906),
    [anon_sym_f32] = ACTIONS(1906),
    [anon_sym_f64] = ACTIONS(1906),
    [anon_sym_bool] = ACTIONS(1906),
    [anon_sym_str] = ACTIONS(1906),
    [anon_sym_char] = ACTIONS(1906),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1908),
    [anon_sym_POUND] = ACTIONS(1910),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1912),
    [anon_sym_await] = ACTIONS(1914),
    [anon_sym_const] = ACTIONS(1916),
    [anon_sym_default] = ACTIONS(1918),
    [anon_sym_enum] = ACTIONS(1920),
    [anon_sym_fn] = ACTIONS(1922),
    [anon_sym_gen] = ACTIONS(1918),
    [anon_sym_impl] = ACTIONS(1924),
    [anon_sym_let] = ACTIONS(1926),
    [anon_sym_mod] = ACTIONS(1928),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_raw] = ACTIONS(1914),
    [anon_sym_static] = ACTIONS(1930),
    [anon_sym_struct] = ACTIONS(1932),
    [anon_sym_trait] = ACTIONS(1934),
    [anon_sym_type] = ACTIONS(1936),
    [anon_sym_union] = ACTIONS(1938),
    [anon_sym_unsafe] = ACTIONS(1940),
    [anon_sym_use] = ACTIONS(1942),
    [anon_sym_extern] = ACTIONS(1944),
    [anon_sym_safe] = ACTIONS(1946),
    [anon_sym_try] = ACTIONS(1914),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1948),
    [sym_super] = ACTIONS(1948),
    [sym_crate] = ACTIONS(1950),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__outer_block_doc_comment_start] = ACTIONS(1892),
    [sym__outer_line_doc_comment_start] = ACTIONS(1894),
  },
  [STATE(543)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1436),
    [sym_parameter] = STATE(5265),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4367),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(543),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(543),
    [sym_metavariable] = STATE(4747),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_COMMA] = ACTIONS(1958),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1968),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1974),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1976),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(544)] = {
    [sym_empty_statement] = STATE(2524),
    [sym__declaration_statement] = STATE(2422),
    [sym_macro_definition_v2] = STATE(2524),
    [sym_macro_definition] = STATE(2524),
    [sym_attribute_item] = STATE(1967),
    [sym_attributes] = STATE(1952),
    [sym_inner_attribute_item] = STATE(2524),
    [sym_mod_item] = STATE(2524),
    [sym_foreign_mod_item] = STATE(2524),
    [sym_struct_item] = STATE(2524),
    [sym_union_item] = STATE(2524),
    [sym_enum_item] = STATE(2524),
    [sym_extern_crate_declaration] = STATE(2524),
    [sym_const_item] = STATE(2524),
    [sym_static_item] = STATE(2524),
    [sym_type_item] = STATE(2524),
    [sym_function_item] = STATE(2524),
    [sym_function_signature_item] = STATE(2524),
    [sym_function_modifiers] = STATE(6437),
    [aux_sym__item_modifiers] = STATE(3791),
    [sym_impl_item] = STATE(2524),
    [sym_trait_item] = STATE(2524),
    [sym_let_declaration] = STATE(2524),
    [sym_use_declaration] = STATE(2524),
    [sym_extern_modifier] = STATE(4451),
    [sym_visibility_modifier] = STATE(3500),
    [sym_bracketed_type] = STATE(6226),
    [sym_generic_type_with_turbofish] = STATE(6299),
    [sym_macro_invocation_item] = STATE(2524),
    [sym_macro_invocation] = STATE(2426),
    [sym_scoped_identifier] = STATE(5653),
    [sym_line_comment] = STATE(544),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(544),
    [sym_metavariable] = STATE(6226),
    [aux_sym_attributes_repeat1] = STATE(1950),
    [aux_sym_declaration_list_repeat1] = STATE(544),
    [sym_identifier] = ACTIONS(1980),
    [anon_sym_SEMI] = ACTIONS(1983),
    [anon_sym_macro] = ACTIONS(1986),
    [anon_sym_RBRACE] = ACTIONS(1989),
    [anon_sym_macro_rules] = ACTIONS(1991),
    [anon_sym_u8] = ACTIONS(1994),
    [anon_sym_i8] = ACTIONS(1994),
    [anon_sym_u16] = ACTIONS(1994),
    [anon_sym_i16] = ACTIONS(1994),
    [anon_sym_u32] = ACTIONS(1994),
    [anon_sym_i32] = ACTIONS(1994),
    [anon_sym_u64] = ACTIONS(1994),
    [anon_sym_i64] = ACTIONS(1994),
    [anon_sym_u128] = ACTIONS(1994),
    [anon_sym_i128] = ACTIONS(1994),
    [anon_sym_isize] = ACTIONS(1994),
    [anon_sym_usize] = ACTIONS(1994),
    [anon_sym_f32] = ACTIONS(1994),
    [anon_sym_f64] = ACTIONS(1994),
    [anon_sym_bool] = ACTIONS(1994),
    [anon_sym_str] = ACTIONS(1994),
    [anon_sym_char] = ACTIONS(1994),
    [anon_sym_LT] = ACTIONS(1997),
    [anon_sym_COLON_COLON] = ACTIONS(2000),
    [anon_sym_POUND] = ACTIONS(2003),
    [anon_sym_async] = ACTIONS(2006),
    [anon_sym_auto] = ACTIONS(2009),
    [anon_sym_await] = ACTIONS(2012),
    [anon_sym_const] = ACTIONS(2015),
    [anon_sym_default] = ACTIONS(2018),
    [anon_sym_enum] = ACTIONS(2021),
    [anon_sym_fn] = ACTIONS(2024),
    [anon_sym_gen] = ACTIONS(2018),
    [anon_sym_impl] = ACTIONS(2027),
    [anon_sym_let] = ACTIONS(2030),
    [anon_sym_mod] = ACTIONS(2033),
    [anon_sym_pub] = ACTIONS(2036),
    [anon_sym_raw] = ACTIONS(2012),
    [anon_sym_static] = ACTIONS(2039),
    [anon_sym_struct] = ACTIONS(2042),
    [anon_sym_trait] = ACTIONS(2045),
    [anon_sym_type] = ACTIONS(2048),
    [anon_sym_union] = ACTIONS(2051),
    [anon_sym_unsafe] = ACTIONS(2054),
    [anon_sym_use] = ACTIONS(2057),
    [anon_sym_extern] = ACTIONS(2060),
    [anon_sym_safe] = ACTIONS(2063),
    [anon_sym_try] = ACTIONS(2012),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_crate] = ACTIONS(2069),
    [aux_sym_metavariable_token1] = ACTIONS(2072),
    [sym__outer_block_doc_comment_start] = ACTIONS(2075),
    [sym__outer_line_doc_comment_start] = ACTIONS(2078),
  },
  [STATE(545)] = {
    [sym_empty_statement] = STATE(2524),
    [sym__declaration_statement] = STATE(2422),
    [sym_macro_definition_v2] = STATE(2524),
    [sym_macro_definition] = STATE(2524),
    [sym_attribute_item] = STATE(1967),
    [sym_attributes] = STATE(1952),
    [sym_inner_attribute_item] = STATE(2524),
    [sym_mod_item] = STATE(2524),
    [sym_foreign_mod_item] = STATE(2524),
    [sym_struct_item] = STATE(2524),
    [sym_union_item] = STATE(2524),
    [sym_enum_item] = STATE(2524),
    [sym_extern_crate_declaration] = STATE(2524),
    [sym_const_item] = STATE(2524),
    [sym_static_item] = STATE(2524),
    [sym_type_item] = STATE(2524),
    [sym_function_item] = STATE(2524),
    [sym_function_signature_item] = STATE(2524),
    [sym_function_modifiers] = STATE(6437),
    [aux_sym__item_modifiers] = STATE(3791),
    [sym_impl_item] = STATE(2524),
    [sym_trait_item] = STATE(2524),
    [sym_let_declaration] = STATE(2524),
    [sym_use_declaration] = STATE(2524),
    [sym_extern_modifier] = STATE(4451),
    [sym_visibility_modifier] = STATE(3500),
    [sym_bracketed_type] = STATE(6226),
    [sym_generic_type_with_turbofish] = STATE(6299),
    [sym_macro_invocation_item] = STATE(2524),
    [sym_macro_invocation] = STATE(2426),
    [sym_scoped_identifier] = STATE(5653),
    [sym_line_comment] = STATE(545),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(545),
    [sym_metavariable] = STATE(6226),
    [aux_sym_attributes_repeat1] = STATE(1950),
    [aux_sym_declaration_list_repeat1] = STATE(544),
    [sym_identifier] = ACTIONS(1896),
    [anon_sym_SEMI] = ACTIONS(1898),
    [anon_sym_macro] = ACTIONS(1900),
    [anon_sym_RBRACE] = ACTIONS(2081),
    [anon_sym_macro_rules] = ACTIONS(1904),
    [anon_sym_u8] = ACTIONS(1906),
    [anon_sym_i8] = ACTIONS(1906),
    [anon_sym_u16] = ACTIONS(1906),
    [anon_sym_i16] = ACTIONS(1906),
    [anon_sym_u32] = ACTIONS(1906),
    [anon_sym_i32] = ACTIONS(1906),
    [anon_sym_u64] = ACTIONS(1906),
    [anon_sym_i64] = ACTIONS(1906),
    [anon_sym_u128] = ACTIONS(1906),
    [anon_sym_i128] = ACTIONS(1906),
    [anon_sym_isize] = ACTIONS(1906),
    [anon_sym_usize] = ACTIONS(1906),
    [anon_sym_f32] = ACTIONS(1906),
    [anon_sym_f64] = ACTIONS(1906),
    [anon_sym_bool] = ACTIONS(1906),
    [anon_sym_str] = ACTIONS(1906),
    [anon_sym_char] = ACTIONS(1906),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1908),
    [anon_sym_POUND] = ACTIONS(1910),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1912),
    [anon_sym_await] = ACTIONS(1914),
    [anon_sym_const] = ACTIONS(1916),
    [anon_sym_default] = ACTIONS(1918),
    [anon_sym_enum] = ACTIONS(1920),
    [anon_sym_fn] = ACTIONS(1922),
    [anon_sym_gen] = ACTIONS(1918),
    [anon_sym_impl] = ACTIONS(1924),
    [anon_sym_let] = ACTIONS(1926),
    [anon_sym_mod] = ACTIONS(1928),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_raw] = ACTIONS(1914),
    [anon_sym_static] = ACTIONS(1930),
    [anon_sym_struct] = ACTIONS(1932),
    [anon_sym_trait] = ACTIONS(1934),
    [anon_sym_type] = ACTIONS(1936),
    [anon_sym_union] = ACTIONS(1938),
    [anon_sym_unsafe] = ACTIONS(1940),
    [anon_sym_use] = ACTIONS(1942),
    [anon_sym_extern] = ACTIONS(1944),
    [anon_sym_safe] = ACTIONS(1946),
    [anon_sym_try] = ACTIONS(1914),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1948),
    [sym_super] = ACTIONS(1948),
    [sym_crate] = ACTIONS(1950),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__outer_block_doc_comment_start] = ACTIONS(1892),
    [sym__outer_line_doc_comment_start] = ACTIONS(1894),
  },
  [STATE(546)] = {
    [sym_empty_statement] = STATE(2524),
    [sym__declaration_statement] = STATE(2422),
    [sym_macro_definition_v2] = STATE(2524),
    [sym_macro_definition] = STATE(2524),
    [sym_attribute_item] = STATE(1967),
    [sym_attributes] = STATE(1952),
    [sym_inner_attribute_item] = STATE(2524),
    [sym_mod_item] = STATE(2524),
    [sym_foreign_mod_item] = STATE(2524),
    [sym_struct_item] = STATE(2524),
    [sym_union_item] = STATE(2524),
    [sym_enum_item] = STATE(2524),
    [sym_extern_crate_declaration] = STATE(2524),
    [sym_const_item] = STATE(2524),
    [sym_static_item] = STATE(2524),
    [sym_type_item] = STATE(2524),
    [sym_function_item] = STATE(2524),
    [sym_function_signature_item] = STATE(2524),
    [sym_function_modifiers] = STATE(6437),
    [aux_sym__item_modifiers] = STATE(3791),
    [sym_impl_item] = STATE(2524),
    [sym_trait_item] = STATE(2524),
    [sym_let_declaration] = STATE(2524),
    [sym_use_declaration] = STATE(2524),
    [sym_extern_modifier] = STATE(4451),
    [sym_visibility_modifier] = STATE(3500),
    [sym_bracketed_type] = STATE(6226),
    [sym_generic_type_with_turbofish] = STATE(6299),
    [sym_macro_invocation_item] = STATE(2524),
    [sym_macro_invocation] = STATE(2426),
    [sym_scoped_identifier] = STATE(5653),
    [sym_line_comment] = STATE(546),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(546),
    [sym_metavariable] = STATE(6226),
    [aux_sym_attributes_repeat1] = STATE(1950),
    [aux_sym_declaration_list_repeat1] = STATE(544),
    [sym_identifier] = ACTIONS(1896),
    [anon_sym_SEMI] = ACTIONS(1898),
    [anon_sym_macro] = ACTIONS(1900),
    [anon_sym_RBRACE] = ACTIONS(2083),
    [anon_sym_macro_rules] = ACTIONS(1904),
    [anon_sym_u8] = ACTIONS(1906),
    [anon_sym_i8] = ACTIONS(1906),
    [anon_sym_u16] = ACTIONS(1906),
    [anon_sym_i16] = ACTIONS(1906),
    [anon_sym_u32] = ACTIONS(1906),
    [anon_sym_i32] = ACTIONS(1906),
    [anon_sym_u64] = ACTIONS(1906),
    [anon_sym_i64] = ACTIONS(1906),
    [anon_sym_u128] = ACTIONS(1906),
    [anon_sym_i128] = ACTIONS(1906),
    [anon_sym_isize] = ACTIONS(1906),
    [anon_sym_usize] = ACTIONS(1906),
    [anon_sym_f32] = ACTIONS(1906),
    [anon_sym_f64] = ACTIONS(1906),
    [anon_sym_bool] = ACTIONS(1906),
    [anon_sym_str] = ACTIONS(1906),
    [anon_sym_char] = ACTIONS(1906),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1908),
    [anon_sym_POUND] = ACTIONS(1910),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1912),
    [anon_sym_await] = ACTIONS(1914),
    [anon_sym_const] = ACTIONS(1916),
    [anon_sym_default] = ACTIONS(1918),
    [anon_sym_enum] = ACTIONS(1920),
    [anon_sym_fn] = ACTIONS(1922),
    [anon_sym_gen] = ACTIONS(1918),
    [anon_sym_impl] = ACTIONS(1924),
    [anon_sym_let] = ACTIONS(1926),
    [anon_sym_mod] = ACTIONS(1928),
    [anon_sym_pub] = ACTIONS(77),
    [anon_sym_raw] = ACTIONS(1914),
    [anon_sym_static] = ACTIONS(1930),
    [anon_sym_struct] = ACTIONS(1932),
    [anon_sym_trait] = ACTIONS(1934),
    [anon_sym_type] = ACTIONS(1936),
    [anon_sym_union] = ACTIONS(1938),
    [anon_sym_unsafe] = ACTIONS(1940),
    [anon_sym_use] = ACTIONS(1942),
    [anon_sym_extern] = ACTIONS(1944),
    [anon_sym_safe] = ACTIONS(1946),
    [anon_sym_try] = ACTIONS(1914),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1948),
    [sym_super] = ACTIONS(1948),
    [sym_crate] = ACTIONS(1950),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__outer_block_doc_comment_start] = ACTIONS(1892),
    [sym__outer_line_doc_comment_start] = ACTIONS(1894),
  },
  [STATE(547)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1436),
    [sym_parameter] = STATE(5265),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4857),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(547),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(547),
    [sym_metavariable] = STATE(4747),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_COMMA] = ACTIONS(1958),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1968),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1974),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1976),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(548)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1436),
    [sym_parameter] = STATE(5492),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5299),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(548),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(548),
    [sym_metavariable] = STATE(4747),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(2085),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1974),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1976),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(549)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1436),
    [sym_parameter] = STATE(5492),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5299),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(549),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(549),
    [sym_metavariable] = STATE(4747),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1974),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1976),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(550)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(1981),
    [sym_attributes] = STATE(1436),
    [sym_parameter] = STATE(5492),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5299),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(550),
    [sym_line_outer_doc_comment] = STATE(1981),
    [sym_block_outer_doc_comment] = STATE(1981),
    [sym_block_comment] = STATE(550),
    [sym_metavariable] = STATE(4747),
    [aux_sym_attributes_repeat1] = STATE(1955),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(2087),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_POUND] = ACTIONS(1800),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1974),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1976),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
    [sym__outer_block_doc_comment_start] = ACTIONS(1704),
    [sym__outer_line_doc_comment_start] = ACTIONS(1706),
  },
  [STATE(551)] = {
    [sym_else_clause] = STATE(578),
    [sym_line_comment] = STATE(551),
    [sym_block_comment] = STATE(551),
    [sym_identifier] = ACTIONS(1352),
    [anon_sym_LPAREN] = ACTIONS(1350),
    [anon_sym_RBRACE] = ACTIONS(1350),
    [anon_sym_COMMA] = ACTIONS(1350),
    [anon_sym_macro_rules] = ACTIONS(1352),
    [anon_sym_LBRACK] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1352),
    [anon_sym_STAR] = ACTIONS(1352),
    [anon_sym_QMARK] = ACTIONS(1350),
    [anon_sym_u8] = ACTIONS(1352),
    [anon_sym_i8] = ACTIONS(1352),
    [anon_sym_u16] = ACTIONS(1352),
    [anon_sym_i16] = ACTIONS(1352),
    [anon_sym_u32] = ACTIONS(1352),
    [anon_sym_i32] = ACTIONS(1352),
    [anon_sym_u64] = ACTIONS(1352),
    [anon_sym_i64] = ACTIONS(1352),
    [anon_sym_u128] = ACTIONS(1352),
    [anon_sym_i128] = ACTIONS(1352),
    [anon_sym_isize] = ACTIONS(1352),
    [anon_sym_usize] = ACTIONS(1352),
    [anon_sym_f32] = ACTIONS(1352),
    [anon_sym_f64] = ACTIONS(1352),
    [anon_sym_bool] = ACTIONS(1352),
    [anon_sym_str] = ACTIONS(1352),
    [anon_sym_char] = ACTIONS(1352),
    [anon_sym_DASH] = ACTIONS(1352),
    [anon_sym_SLASH] = ACTIONS(1352),
    [anon_sym_PERCENT] = ACTIONS(1352),
    [anon_sym_CARET] = ACTIONS(1352),
    [anon_sym_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1350),
    [anon_sym_PIPE_PIPE] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1352),
    [anon_sym_GT_GT] = ACTIONS(1352),
    [anon_sym_PLUS_EQ] = ACTIONS(1350),
    [anon_sym_DASH_EQ] = ACTIONS(1350),
    [anon_sym_STAR_EQ] = ACTIONS(1350),
    [anon_sym_SLASH_EQ] = ACTIONS(1350),
    [anon_sym_PERCENT_EQ] = ACTIONS(1350),
    [anon_sym_CARET_EQ] = ACTIONS(1350),
    [anon_sym_AMP_EQ] = ACTIONS(1350),
    [anon_sym_PIPE_EQ] = ACTIONS(1350),
    [anon_sym_LT_LT_EQ] = ACTIONS(1350),
    [anon_sym_GT_GT_EQ] = ACTIONS(1350),
    [anon_sym_EQ] = ACTIONS(1352),
    [anon_sym_EQ_EQ] = ACTIONS(1350),
    [anon_sym_BANG_EQ] = ACTIONS(1350),
    [anon_sym_GT] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1352),
    [anon_sym_GT_EQ] = ACTIONS(1350),
    [anon_sym_LT_EQ] = ACTIONS(1350),
    [anon_sym__] = ACTIONS(1352),
    [anon_sym_DOT] = ACTIONS(1352),
    [anon_sym_DOT_DOT] = ACTIONS(1352),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1350),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1350),
    [anon_sym_COLON_COLON] = ACTIONS(1350),
    [anon_sym_POUND] = ACTIONS(1350),
    [anon_sym_as] = ACTIONS(1352),
    [anon_sym_auto] = ACTIONS(1352),
    [anon_sym_await] = ACTIONS(1352),
    [anon_sym_const] = ACTIONS(1352),
    [anon_sym_default] = ACTIONS(1352),
    [anon_sym_gen] = ACTIONS(1352),
    [anon_sym_raw] = ACTIONS(1352),
    [anon_sym_union] = ACTIONS(1352),
    [anon_sym_safe] = ACTIONS(1352),
    [anon_sym_ref] = ACTIONS(1352),
    [anon_sym_else] = ACTIONS(2089),
    [sym_mutable_specifier] = ACTIONS(1352),
    [anon_sym_try] = ACTIONS(1352),
    [anon_sym_box] = ACTIONS(1352),
    [sym_integer_literal] = ACTIONS(1350),
    [aux_sym_string_literal_token1] = ACTIONS(1350),
    [sym_char_literal] = ACTIONS(1350),
    [anon_sym_true] = ACTIONS(1352),
    [anon_sym_false] = ACTIONS(1352),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1352),
    [sym_super] = ACTIONS(1352),
    [sym_crate] = ACTIONS(1352),
    [aux_sym_metavariable_token1] = ACTIONS(1350),
    [sym__raw_string_literal_start] = ACTIONS(1350),
    [sym_float_literal] = ACTIONS(1350),
    [sym__outer_block_doc_comment_start] = ACTIONS(1350),
    [sym__outer_line_doc_comment_start] = ACTIONS(1350),
  },
  [STATE(552)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1444),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1479),
    [sym__type] = STATE(4656),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(552),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(552),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2093),
    [anon_sym_COMMA] = ACTIONS(2095),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(553)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(553),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(553),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2111),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(554)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5406),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(4582),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(554),
    [sym_block_comment] = STATE(554),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(2113),
    [anon_sym_COMMA] = ACTIONS(1312),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_async] = ACTIONS(2115),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(555)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(555),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(555),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2117),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(556)] = {
    [sym_line_comment] = STATE(556),
    [sym_block_comment] = STATE(556),
    [sym_identifier] = ACTIONS(1460),
    [anon_sym_LPAREN] = ACTIONS(1458),
    [anon_sym_RBRACE] = ACTIONS(1458),
    [anon_sym_COMMA] = ACTIONS(1458),
    [anon_sym_macro_rules] = ACTIONS(1460),
    [anon_sym_LBRACK] = ACTIONS(1458),
    [anon_sym_PLUS] = ACTIONS(1460),
    [anon_sym_STAR] = ACTIONS(1460),
    [anon_sym_QMARK] = ACTIONS(1458),
    [anon_sym_u8] = ACTIONS(1460),
    [anon_sym_i8] = ACTIONS(1460),
    [anon_sym_u16] = ACTIONS(1460),
    [anon_sym_i16] = ACTIONS(1460),
    [anon_sym_u32] = ACTIONS(1460),
    [anon_sym_i32] = ACTIONS(1460),
    [anon_sym_u64] = ACTIONS(1460),
    [anon_sym_i64] = ACTIONS(1460),
    [anon_sym_u128] = ACTIONS(1460),
    [anon_sym_i128] = ACTIONS(1460),
    [anon_sym_isize] = ACTIONS(1460),
    [anon_sym_usize] = ACTIONS(1460),
    [anon_sym_f32] = ACTIONS(1460),
    [anon_sym_f64] = ACTIONS(1460),
    [anon_sym_bool] = ACTIONS(1460),
    [anon_sym_str] = ACTIONS(1460),
    [anon_sym_char] = ACTIONS(1460),
    [anon_sym_DASH] = ACTIONS(1460),
    [anon_sym_SLASH] = ACTIONS(1460),
    [anon_sym_PERCENT] = ACTIONS(1460),
    [anon_sym_CARET] = ACTIONS(1460),
    [anon_sym_AMP] = ACTIONS(1460),
    [anon_sym_PIPE] = ACTIONS(1460),
    [anon_sym_AMP_AMP] = ACTIONS(1458),
    [anon_sym_PIPE_PIPE] = ACTIONS(1458),
    [anon_sym_LT_LT] = ACTIONS(1460),
    [anon_sym_GT_GT] = ACTIONS(1460),
    [anon_sym_PLUS_EQ] = ACTIONS(1458),
    [anon_sym_DASH_EQ] = ACTIONS(1458),
    [anon_sym_STAR_EQ] = ACTIONS(1458),
    [anon_sym_SLASH_EQ] = ACTIONS(1458),
    [anon_sym_PERCENT_EQ] = ACTIONS(1458),
    [anon_sym_CARET_EQ] = ACTIONS(1458),
    [anon_sym_AMP_EQ] = ACTIONS(1458),
    [anon_sym_PIPE_EQ] = ACTIONS(1458),
    [anon_sym_LT_LT_EQ] = ACTIONS(1458),
    [anon_sym_GT_GT_EQ] = ACTIONS(1458),
    [anon_sym_EQ] = ACTIONS(1460),
    [anon_sym_EQ_EQ] = ACTIONS(1458),
    [anon_sym_BANG_EQ] = ACTIONS(1458),
    [anon_sym_GT] = ACTIONS(1460),
    [anon_sym_LT] = ACTIONS(1460),
    [anon_sym_GT_EQ] = ACTIONS(1458),
    [anon_sym_LT_EQ] = ACTIONS(1458),
    [anon_sym__] = ACTIONS(1460),
    [anon_sym_DOT] = ACTIONS(1460),
    [anon_sym_DOT_DOT] = ACTIONS(1460),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1458),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1458),
    [anon_sym_COLON_COLON] = ACTIONS(1458),
    [anon_sym_POUND] = ACTIONS(1458),
    [anon_sym_as] = ACTIONS(1460),
    [anon_sym_auto] = ACTIONS(1460),
    [anon_sym_await] = ACTIONS(1460),
    [anon_sym_const] = ACTIONS(1460),
    [anon_sym_default] = ACTIONS(1460),
    [anon_sym_gen] = ACTIONS(1460),
    [anon_sym_raw] = ACTIONS(1460),
    [anon_sym_union] = ACTIONS(1460),
    [anon_sym_safe] = ACTIONS(1460),
    [anon_sym_ref] = ACTIONS(1460),
    [anon_sym_else] = ACTIONS(1460),
    [sym_mutable_specifier] = ACTIONS(1460),
    [anon_sym_try] = ACTIONS(1460),
    [anon_sym_box] = ACTIONS(1460),
    [sym_integer_literal] = ACTIONS(1458),
    [aux_sym_string_literal_token1] = ACTIONS(1458),
    [sym_char_literal] = ACTIONS(1458),
    [anon_sym_true] = ACTIONS(1460),
    [anon_sym_false] = ACTIONS(1460),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1460),
    [sym_super] = ACTIONS(1460),
    [sym_crate] = ACTIONS(1460),
    [aux_sym_metavariable_token1] = ACTIONS(1458),
    [sym__raw_string_literal_start] = ACTIONS(1458),
    [sym_float_literal] = ACTIONS(1458),
    [sym__outer_block_doc_comment_start] = ACTIONS(1458),
    [sym__outer_line_doc_comment_start] = ACTIONS(1458),
  },
  [STATE(557)] = {
    [sym_line_comment] = STATE(557),
    [sym_block_comment] = STATE(557),
    [sym_identifier] = ACTIONS(1400),
    [anon_sym_LPAREN] = ACTIONS(1398),
    [anon_sym_RBRACE] = ACTIONS(1398),
    [anon_sym_COMMA] = ACTIONS(1398),
    [anon_sym_macro_rules] = ACTIONS(1400),
    [anon_sym_LBRACK] = ACTIONS(1398),
    [anon_sym_PLUS] = ACTIONS(1400),
    [anon_sym_STAR] = ACTIONS(1400),
    [anon_sym_QMARK] = ACTIONS(1398),
    [anon_sym_u8] = ACTIONS(1400),
    [anon_sym_i8] = ACTIONS(1400),
    [anon_sym_u16] = ACTIONS(1400),
    [anon_sym_i16] = ACTIONS(1400),
    [anon_sym_u32] = ACTIONS(1400),
    [anon_sym_i32] = ACTIONS(1400),
    [anon_sym_u64] = ACTIONS(1400),
    [anon_sym_i64] = ACTIONS(1400),
    [anon_sym_u128] = ACTIONS(1400),
    [anon_sym_i128] = ACTIONS(1400),
    [anon_sym_isize] = ACTIONS(1400),
    [anon_sym_usize] = ACTIONS(1400),
    [anon_sym_f32] = ACTIONS(1400),
    [anon_sym_f64] = ACTIONS(1400),
    [anon_sym_bool] = ACTIONS(1400),
    [anon_sym_str] = ACTIONS(1400),
    [anon_sym_char] = ACTIONS(1400),
    [anon_sym_DASH] = ACTIONS(1400),
    [anon_sym_SLASH] = ACTIONS(1400),
    [anon_sym_PERCENT] = ACTIONS(1400),
    [anon_sym_CARET] = ACTIONS(1400),
    [anon_sym_AMP] = ACTIONS(1400),
    [anon_sym_PIPE] = ACTIONS(1400),
    [anon_sym_AMP_AMP] = ACTIONS(1398),
    [anon_sym_PIPE_PIPE] = ACTIONS(1398),
    [anon_sym_LT_LT] = ACTIONS(1400),
    [anon_sym_GT_GT] = ACTIONS(1400),
    [anon_sym_PLUS_EQ] = ACTIONS(1398),
    [anon_sym_DASH_EQ] = ACTIONS(1398),
    [anon_sym_STAR_EQ] = ACTIONS(1398),
    [anon_sym_SLASH_EQ] = ACTIONS(1398),
    [anon_sym_PERCENT_EQ] = ACTIONS(1398),
    [anon_sym_CARET_EQ] = ACTIONS(1398),
    [anon_sym_AMP_EQ] = ACTIONS(1398),
    [anon_sym_PIPE_EQ] = ACTIONS(1398),
    [anon_sym_LT_LT_EQ] = ACTIONS(1398),
    [anon_sym_GT_GT_EQ] = ACTIONS(1398),
    [anon_sym_EQ] = ACTIONS(1400),
    [anon_sym_EQ_EQ] = ACTIONS(1398),
    [anon_sym_BANG_EQ] = ACTIONS(1398),
    [anon_sym_GT] = ACTIONS(1400),
    [anon_sym_LT] = ACTIONS(1400),
    [anon_sym_GT_EQ] = ACTIONS(1398),
    [anon_sym_LT_EQ] = ACTIONS(1398),
    [anon_sym__] = ACTIONS(1400),
    [anon_sym_DOT] = ACTIONS(1400),
    [anon_sym_DOT_DOT] = ACTIONS(1400),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1398),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1398),
    [anon_sym_COLON_COLON] = ACTIONS(1398),
    [anon_sym_POUND] = ACTIONS(1398),
    [anon_sym_as] = ACTIONS(1400),
    [anon_sym_auto] = ACTIONS(1400),
    [anon_sym_await] = ACTIONS(1400),
    [anon_sym_const] = ACTIONS(1400),
    [anon_sym_default] = ACTIONS(1400),
    [anon_sym_gen] = ACTIONS(1400),
    [anon_sym_raw] = ACTIONS(1400),
    [anon_sym_union] = ACTIONS(1400),
    [anon_sym_safe] = ACTIONS(1400),
    [anon_sym_ref] = ACTIONS(1400),
    [anon_sym_else] = ACTIONS(1400),
    [sym_mutable_specifier] = ACTIONS(1400),
    [anon_sym_try] = ACTIONS(1400),
    [anon_sym_box] = ACTIONS(1400),
    [sym_integer_literal] = ACTIONS(1398),
    [aux_sym_string_literal_token1] = ACTIONS(1398),
    [sym_char_literal] = ACTIONS(1398),
    [anon_sym_true] = ACTIONS(1400),
    [anon_sym_false] = ACTIONS(1400),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1400),
    [sym_super] = ACTIONS(1400),
    [sym_crate] = ACTIONS(1400),
    [aux_sym_metavariable_token1] = ACTIONS(1398),
    [sym__raw_string_literal_start] = ACTIONS(1398),
    [sym_float_literal] = ACTIONS(1398),
    [sym__outer_block_doc_comment_start] = ACTIONS(1398),
    [sym__outer_line_doc_comment_start] = ACTIONS(1398),
  },
  [STATE(558)] = {
    [sym_line_comment] = STATE(558),
    [sym_block_comment] = STATE(558),
    [sym_identifier] = ACTIONS(1484),
    [anon_sym_LPAREN] = ACTIONS(1482),
    [anon_sym_RBRACE] = ACTIONS(1482),
    [anon_sym_COMMA] = ACTIONS(1482),
    [anon_sym_macro_rules] = ACTIONS(1484),
    [anon_sym_LBRACK] = ACTIONS(1482),
    [anon_sym_PLUS] = ACTIONS(1484),
    [anon_sym_STAR] = ACTIONS(1484),
    [anon_sym_QMARK] = ACTIONS(1482),
    [anon_sym_u8] = ACTIONS(1484),
    [anon_sym_i8] = ACTIONS(1484),
    [anon_sym_u16] = ACTIONS(1484),
    [anon_sym_i16] = ACTIONS(1484),
    [anon_sym_u32] = ACTIONS(1484),
    [anon_sym_i32] = ACTIONS(1484),
    [anon_sym_u64] = ACTIONS(1484),
    [anon_sym_i64] = ACTIONS(1484),
    [anon_sym_u128] = ACTIONS(1484),
    [anon_sym_i128] = ACTIONS(1484),
    [anon_sym_isize] = ACTIONS(1484),
    [anon_sym_usize] = ACTIONS(1484),
    [anon_sym_f32] = ACTIONS(1484),
    [anon_sym_f64] = ACTIONS(1484),
    [anon_sym_bool] = ACTIONS(1484),
    [anon_sym_str] = ACTIONS(1484),
    [anon_sym_char] = ACTIONS(1484),
    [anon_sym_DASH] = ACTIONS(1484),
    [anon_sym_SLASH] = ACTIONS(1484),
    [anon_sym_PERCENT] = ACTIONS(1484),
    [anon_sym_CARET] = ACTIONS(1484),
    [anon_sym_AMP] = ACTIONS(1484),
    [anon_sym_PIPE] = ACTIONS(1484),
    [anon_sym_AMP_AMP] = ACTIONS(1482),
    [anon_sym_PIPE_PIPE] = ACTIONS(1482),
    [anon_sym_LT_LT] = ACTIONS(1484),
    [anon_sym_GT_GT] = ACTIONS(1484),
    [anon_sym_PLUS_EQ] = ACTIONS(1482),
    [anon_sym_DASH_EQ] = ACTIONS(1482),
    [anon_sym_STAR_EQ] = ACTIONS(1482),
    [anon_sym_SLASH_EQ] = ACTIONS(1482),
    [anon_sym_PERCENT_EQ] = ACTIONS(1482),
    [anon_sym_CARET_EQ] = ACTIONS(1482),
    [anon_sym_AMP_EQ] = ACTIONS(1482),
    [anon_sym_PIPE_EQ] = ACTIONS(1482),
    [anon_sym_LT_LT_EQ] = ACTIONS(1482),
    [anon_sym_GT_GT_EQ] = ACTIONS(1482),
    [anon_sym_EQ] = ACTIONS(1484),
    [anon_sym_EQ_EQ] = ACTIONS(1482),
    [anon_sym_BANG_EQ] = ACTIONS(1482),
    [anon_sym_GT] = ACTIONS(1484),
    [anon_sym_LT] = ACTIONS(1484),
    [anon_sym_GT_EQ] = ACTIONS(1482),
    [anon_sym_LT_EQ] = ACTIONS(1482),
    [anon_sym__] = ACTIONS(1484),
    [anon_sym_DOT] = ACTIONS(1484),
    [anon_sym_DOT_DOT] = ACTIONS(1484),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1482),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1482),
    [anon_sym_COLON_COLON] = ACTIONS(1482),
    [anon_sym_POUND] = ACTIONS(1482),
    [anon_sym_as] = ACTIONS(1484),
    [anon_sym_auto] = ACTIONS(1484),
    [anon_sym_await] = ACTIONS(1484),
    [anon_sym_const] = ACTIONS(1484),
    [anon_sym_default] = ACTIONS(1484),
    [anon_sym_gen] = ACTIONS(1484),
    [anon_sym_raw] = ACTIONS(1484),
    [anon_sym_union] = ACTIONS(1484),
    [anon_sym_safe] = ACTIONS(1484),
    [anon_sym_ref] = ACTIONS(1484),
    [anon_sym_else] = ACTIONS(1484),
    [sym_mutable_specifier] = ACTIONS(1484),
    [anon_sym_try] = ACTIONS(1484),
    [anon_sym_box] = ACTIONS(1484),
    [sym_integer_literal] = ACTIONS(1482),
    [aux_sym_string_literal_token1] = ACTIONS(1482),
    [sym_char_literal] = ACTIONS(1482),
    [anon_sym_true] = ACTIONS(1484),
    [anon_sym_false] = ACTIONS(1484),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1484),
    [sym_super] = ACTIONS(1484),
    [sym_crate] = ACTIONS(1484),
    [aux_sym_metavariable_token1] = ACTIONS(1482),
    [sym__raw_string_literal_start] = ACTIONS(1482),
    [sym_float_literal] = ACTIONS(1482),
    [sym__outer_block_doc_comment_start] = ACTIONS(1482),
    [sym__outer_line_doc_comment_start] = ACTIONS(1482),
  },
  [STATE(559)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(559),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(559),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2119),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(560)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(560),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(560),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2121),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(561)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(561),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(561),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2123),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(562)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(4941),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(4777),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(562),
    [sym_block_comment] = STATE(562),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(2125),
    [anon_sym_COMMA] = ACTIONS(2127),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_async] = ACTIONS(2115),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(563)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(563),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(563),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2129),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(564)] = {
    [sym_line_comment] = STATE(564),
    [sym_block_comment] = STATE(564),
    [sym_identifier] = ACTIONS(1376),
    [anon_sym_LPAREN] = ACTIONS(1374),
    [anon_sym_RBRACE] = ACTIONS(1374),
    [anon_sym_COMMA] = ACTIONS(1374),
    [anon_sym_macro_rules] = ACTIONS(1376),
    [anon_sym_LBRACK] = ACTIONS(1374),
    [anon_sym_PLUS] = ACTIONS(1376),
    [anon_sym_STAR] = ACTIONS(1376),
    [anon_sym_QMARK] = ACTIONS(1374),
    [anon_sym_u8] = ACTIONS(1376),
    [anon_sym_i8] = ACTIONS(1376),
    [anon_sym_u16] = ACTIONS(1376),
    [anon_sym_i16] = ACTIONS(1376),
    [anon_sym_u32] = ACTIONS(1376),
    [anon_sym_i32] = ACTIONS(1376),
    [anon_sym_u64] = ACTIONS(1376),
    [anon_sym_i64] = ACTIONS(1376),
    [anon_sym_u128] = ACTIONS(1376),
    [anon_sym_i128] = ACTIONS(1376),
    [anon_sym_isize] = ACTIONS(1376),
    [anon_sym_usize] = ACTIONS(1376),
    [anon_sym_f32] = ACTIONS(1376),
    [anon_sym_f64] = ACTIONS(1376),
    [anon_sym_bool] = ACTIONS(1376),
    [anon_sym_str] = ACTIONS(1376),
    [anon_sym_char] = ACTIONS(1376),
    [anon_sym_DASH] = ACTIONS(1376),
    [anon_sym_SLASH] = ACTIONS(1376),
    [anon_sym_PERCENT] = ACTIONS(1376),
    [anon_sym_CARET] = ACTIONS(1376),
    [anon_sym_AMP] = ACTIONS(1376),
    [anon_sym_PIPE] = ACTIONS(1376),
    [anon_sym_AMP_AMP] = ACTIONS(1374),
    [anon_sym_PIPE_PIPE] = ACTIONS(1374),
    [anon_sym_LT_LT] = ACTIONS(1376),
    [anon_sym_GT_GT] = ACTIONS(1376),
    [anon_sym_PLUS_EQ] = ACTIONS(1374),
    [anon_sym_DASH_EQ] = ACTIONS(1374),
    [anon_sym_STAR_EQ] = ACTIONS(1374),
    [anon_sym_SLASH_EQ] = ACTIONS(1374),
    [anon_sym_PERCENT_EQ] = ACTIONS(1374),
    [anon_sym_CARET_EQ] = ACTIONS(1374),
    [anon_sym_AMP_EQ] = ACTIONS(1374),
    [anon_sym_PIPE_EQ] = ACTIONS(1374),
    [anon_sym_LT_LT_EQ] = ACTIONS(1374),
    [anon_sym_GT_GT_EQ] = ACTIONS(1374),
    [anon_sym_EQ] = ACTIONS(1376),
    [anon_sym_EQ_EQ] = ACTIONS(1374),
    [anon_sym_BANG_EQ] = ACTIONS(1374),
    [anon_sym_GT] = ACTIONS(1376),
    [anon_sym_LT] = ACTIONS(1376),
    [anon_sym_GT_EQ] = ACTIONS(1374),
    [anon_sym_LT_EQ] = ACTIONS(1374),
    [anon_sym__] = ACTIONS(1376),
    [anon_sym_DOT] = ACTIONS(1376),
    [anon_sym_DOT_DOT] = ACTIONS(1376),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1374),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1374),
    [anon_sym_COLON_COLON] = ACTIONS(1374),
    [anon_sym_POUND] = ACTIONS(1374),
    [anon_sym_as] = ACTIONS(1376),
    [anon_sym_auto] = ACTIONS(1376),
    [anon_sym_await] = ACTIONS(1376),
    [anon_sym_const] = ACTIONS(1376),
    [anon_sym_default] = ACTIONS(1376),
    [anon_sym_gen] = ACTIONS(1376),
    [anon_sym_raw] = ACTIONS(1376),
    [anon_sym_union] = ACTIONS(1376),
    [anon_sym_safe] = ACTIONS(1376),
    [anon_sym_ref] = ACTIONS(1376),
    [anon_sym_else] = ACTIONS(1376),
    [sym_mutable_specifier] = ACTIONS(1376),
    [anon_sym_try] = ACTIONS(1376),
    [anon_sym_box] = ACTIONS(1376),
    [sym_integer_literal] = ACTIONS(1374),
    [aux_sym_string_literal_token1] = ACTIONS(1374),
    [sym_char_literal] = ACTIONS(1374),
    [anon_sym_true] = ACTIONS(1376),
    [anon_sym_false] = ACTIONS(1376),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1376),
    [sym_super] = ACTIONS(1376),
    [sym_crate] = ACTIONS(1376),
    [aux_sym_metavariable_token1] = ACTIONS(1374),
    [sym__raw_string_literal_start] = ACTIONS(1374),
    [sym_float_literal] = ACTIONS(1374),
    [sym__outer_block_doc_comment_start] = ACTIONS(1374),
    [sym__outer_line_doc_comment_start] = ACTIONS(1374),
  },
  [STATE(565)] = {
    [sym_line_comment] = STATE(565),
    [sym_block_comment] = STATE(565),
    [sym_identifier] = ACTIONS(1412),
    [anon_sym_LPAREN] = ACTIONS(1410),
    [anon_sym_RBRACE] = ACTIONS(1410),
    [anon_sym_COMMA] = ACTIONS(1410),
    [anon_sym_macro_rules] = ACTIONS(1412),
    [anon_sym_LBRACK] = ACTIONS(1410),
    [anon_sym_PLUS] = ACTIONS(1412),
    [anon_sym_STAR] = ACTIONS(1412),
    [anon_sym_QMARK] = ACTIONS(1410),
    [anon_sym_u8] = ACTIONS(1412),
    [anon_sym_i8] = ACTIONS(1412),
    [anon_sym_u16] = ACTIONS(1412),
    [anon_sym_i16] = ACTIONS(1412),
    [anon_sym_u32] = ACTIONS(1412),
    [anon_sym_i32] = ACTIONS(1412),
    [anon_sym_u64] = ACTIONS(1412),
    [anon_sym_i64] = ACTIONS(1412),
    [anon_sym_u128] = ACTIONS(1412),
    [anon_sym_i128] = ACTIONS(1412),
    [anon_sym_isize] = ACTIONS(1412),
    [anon_sym_usize] = ACTIONS(1412),
    [anon_sym_f32] = ACTIONS(1412),
    [anon_sym_f64] = ACTIONS(1412),
    [anon_sym_bool] = ACTIONS(1412),
    [anon_sym_str] = ACTIONS(1412),
    [anon_sym_char] = ACTIONS(1412),
    [anon_sym_DASH] = ACTIONS(1412),
    [anon_sym_SLASH] = ACTIONS(1412),
    [anon_sym_PERCENT] = ACTIONS(1412),
    [anon_sym_CARET] = ACTIONS(1412),
    [anon_sym_AMP] = ACTIONS(1412),
    [anon_sym_PIPE] = ACTIONS(1412),
    [anon_sym_AMP_AMP] = ACTIONS(1410),
    [anon_sym_PIPE_PIPE] = ACTIONS(1410),
    [anon_sym_LT_LT] = ACTIONS(1412),
    [anon_sym_GT_GT] = ACTIONS(1412),
    [anon_sym_PLUS_EQ] = ACTIONS(1410),
    [anon_sym_DASH_EQ] = ACTIONS(1410),
    [anon_sym_STAR_EQ] = ACTIONS(1410),
    [anon_sym_SLASH_EQ] = ACTIONS(1410),
    [anon_sym_PERCENT_EQ] = ACTIONS(1410),
    [anon_sym_CARET_EQ] = ACTIONS(1410),
    [anon_sym_AMP_EQ] = ACTIONS(1410),
    [anon_sym_PIPE_EQ] = ACTIONS(1410),
    [anon_sym_LT_LT_EQ] = ACTIONS(1410),
    [anon_sym_GT_GT_EQ] = ACTIONS(1410),
    [anon_sym_EQ] = ACTIONS(1412),
    [anon_sym_EQ_EQ] = ACTIONS(1410),
    [anon_sym_BANG_EQ] = ACTIONS(1410),
    [anon_sym_GT] = ACTIONS(1412),
    [anon_sym_LT] = ACTIONS(1412),
    [anon_sym_GT_EQ] = ACTIONS(1410),
    [anon_sym_LT_EQ] = ACTIONS(1410),
    [anon_sym__] = ACTIONS(1412),
    [anon_sym_DOT] = ACTIONS(1412),
    [anon_sym_DOT_DOT] = ACTIONS(1412),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1410),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1410),
    [anon_sym_COLON_COLON] = ACTIONS(1410),
    [anon_sym_POUND] = ACTIONS(1410),
    [anon_sym_as] = ACTIONS(1412),
    [anon_sym_auto] = ACTIONS(1412),
    [anon_sym_await] = ACTIONS(1412),
    [anon_sym_const] = ACTIONS(1412),
    [anon_sym_default] = ACTIONS(1412),
    [anon_sym_gen] = ACTIONS(1412),
    [anon_sym_raw] = ACTIONS(1412),
    [anon_sym_union] = ACTIONS(1412),
    [anon_sym_safe] = ACTIONS(1412),
    [anon_sym_ref] = ACTIONS(1412),
    [anon_sym_else] = ACTIONS(1412),
    [sym_mutable_specifier] = ACTIONS(1412),
    [anon_sym_try] = ACTIONS(1412),
    [anon_sym_box] = ACTIONS(1412),
    [sym_integer_literal] = ACTIONS(1410),
    [aux_sym_string_literal_token1] = ACTIONS(1410),
    [sym_char_literal] = ACTIONS(1410),
    [anon_sym_true] = ACTIONS(1412),
    [anon_sym_false] = ACTIONS(1412),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1412),
    [sym_super] = ACTIONS(1412),
    [sym_crate] = ACTIONS(1412),
    [aux_sym_metavariable_token1] = ACTIONS(1410),
    [sym__raw_string_literal_start] = ACTIONS(1410),
    [sym_float_literal] = ACTIONS(1410),
    [sym__outer_block_doc_comment_start] = ACTIONS(1410),
    [sym__outer_line_doc_comment_start] = ACTIONS(1410),
  },
  [STATE(566)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(566),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(566),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2131),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(567)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(567),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(567),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(2133),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(568)] = {
    [sym_line_comment] = STATE(568),
    [sym_block_comment] = STATE(568),
    [sym_identifier] = ACTIONS(2135),
    [anon_sym_LPAREN] = ACTIONS(2137),
    [anon_sym_RBRACE] = ACTIONS(1506),
    [anon_sym_COMMA] = ACTIONS(1506),
    [anon_sym_macro_rules] = ACTIONS(2135),
    [anon_sym_LBRACK] = ACTIONS(2137),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1508),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(2135),
    [anon_sym_i8] = ACTIONS(2135),
    [anon_sym_u16] = ACTIONS(2135),
    [anon_sym_i16] = ACTIONS(2135),
    [anon_sym_u32] = ACTIONS(2135),
    [anon_sym_i32] = ACTIONS(2135),
    [anon_sym_u64] = ACTIONS(2135),
    [anon_sym_i64] = ACTIONS(2135),
    [anon_sym_u128] = ACTIONS(2135),
    [anon_sym_i128] = ACTIONS(2135),
    [anon_sym_isize] = ACTIONS(2135),
    [anon_sym_usize] = ACTIONS(2135),
    [anon_sym_f32] = ACTIONS(2135),
    [anon_sym_f64] = ACTIONS(2135),
    [anon_sym_bool] = ACTIONS(2135),
    [anon_sym_str] = ACTIONS(2135),
    [anon_sym_char] = ACTIONS(2135),
    [anon_sym_DASH] = ACTIONS(2135),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(2135),
    [anon_sym_PIPE] = ACTIONS(2135),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(2135),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym__] = ACTIONS(2135),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(2135),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(2137),
    [anon_sym_COLON_COLON] = ACTIONS(2137),
    [anon_sym_POUND] = ACTIONS(2137),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_auto] = ACTIONS(2135),
    [anon_sym_await] = ACTIONS(2135),
    [anon_sym_const] = ACTIONS(2135),
    [anon_sym_default] = ACTIONS(2135),
    [anon_sym_gen] = ACTIONS(2135),
    [anon_sym_raw] = ACTIONS(2135),
    [anon_sym_union] = ACTIONS(2135),
    [anon_sym_safe] = ACTIONS(2135),
    [anon_sym_ref] = ACTIONS(2135),
    [sym_mutable_specifier] = ACTIONS(2135),
    [anon_sym_try] = ACTIONS(2135),
    [anon_sym_box] = ACTIONS(2135),
    [sym_integer_literal] = ACTIONS(2137),
    [aux_sym_string_literal_token1] = ACTIONS(2137),
    [sym_char_literal] = ACTIONS(2137),
    [anon_sym_true] = ACTIONS(2135),
    [anon_sym_false] = ACTIONS(2135),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2135),
    [sym_super] = ACTIONS(2135),
    [sym_crate] = ACTIONS(2135),
    [aux_sym_metavariable_token1] = ACTIONS(2137),
    [sym__raw_string_literal_start] = ACTIONS(2137),
    [sym_float_literal] = ACTIONS(2137),
    [sym__outer_block_doc_comment_start] = ACTIONS(2137),
    [sym__outer_line_doc_comment_start] = ACTIONS(2137),
  },
  [STATE(569)] = {
    [sym_line_comment] = STATE(569),
    [sym_block_comment] = STATE(569),
    [sym_identifier] = ACTIONS(2139),
    [anon_sym_LPAREN] = ACTIONS(2141),
    [anon_sym_RBRACE] = ACTIONS(1506),
    [anon_sym_COMMA] = ACTIONS(1506),
    [anon_sym_macro_rules] = ACTIONS(2139),
    [anon_sym_LBRACK] = ACTIONS(2141),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1508),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(2139),
    [anon_sym_i8] = ACTIONS(2139),
    [anon_sym_u16] = ACTIONS(2139),
    [anon_sym_i16] = ACTIONS(2139),
    [anon_sym_u32] = ACTIONS(2139),
    [anon_sym_i32] = ACTIONS(2139),
    [anon_sym_u64] = ACTIONS(2139),
    [anon_sym_i64] = ACTIONS(2139),
    [anon_sym_u128] = ACTIONS(2139),
    [anon_sym_i128] = ACTIONS(2139),
    [anon_sym_isize] = ACTIONS(2139),
    [anon_sym_usize] = ACTIONS(2139),
    [anon_sym_f32] = ACTIONS(2139),
    [anon_sym_f64] = ACTIONS(2139),
    [anon_sym_bool] = ACTIONS(2139),
    [anon_sym_str] = ACTIONS(2139),
    [anon_sym_char] = ACTIONS(2139),
    [anon_sym_DASH] = ACTIONS(2139),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(2139),
    [anon_sym_PIPE] = ACTIONS(2139),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(2139),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym__] = ACTIONS(2139),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(2139),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(2141),
    [anon_sym_COLON_COLON] = ACTIONS(2141),
    [anon_sym_POUND] = ACTIONS(2141),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_auto] = ACTIONS(2139),
    [anon_sym_await] = ACTIONS(2139),
    [anon_sym_const] = ACTIONS(2139),
    [anon_sym_default] = ACTIONS(2139),
    [anon_sym_gen] = ACTIONS(2139),
    [anon_sym_raw] = ACTIONS(2139),
    [anon_sym_union] = ACTIONS(2139),
    [anon_sym_safe] = ACTIONS(2139),
    [anon_sym_ref] = ACTIONS(2139),
    [sym_mutable_specifier] = ACTIONS(2139),
    [anon_sym_try] = ACTIONS(2139),
    [anon_sym_box] = ACTIONS(2139),
    [sym_integer_literal] = ACTIONS(2141),
    [aux_sym_string_literal_token1] = ACTIONS(2141),
    [sym_char_literal] = ACTIONS(2141),
    [anon_sym_true] = ACTIONS(2139),
    [anon_sym_false] = ACTIONS(2139),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2139),
    [sym_super] = ACTIONS(2139),
    [sym_crate] = ACTIONS(2139),
    [aux_sym_metavariable_token1] = ACTIONS(2141),
    [sym__raw_string_literal_start] = ACTIONS(2141),
    [sym_float_literal] = ACTIONS(2141),
    [sym__outer_block_doc_comment_start] = ACTIONS(2141),
    [sym__outer_line_doc_comment_start] = ACTIONS(2141),
  },
  [STATE(570)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5565),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(5247),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(570),
    [sym_block_comment] = STATE(570),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(2143),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_async] = ACTIONS(2115),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(571)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_attribute_item] = STATE(2751),
    [sym_attributes] = STATE(1440),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1492),
    [sym__type] = STATE(4885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(571),
    [sym_line_outer_doc_comment] = STATE(2751),
    [sym_block_outer_doc_comment] = STATE(2751),
    [sym_block_comment] = STATE(571),
    [sym_metavariable] = STATE(3583),
    [aux_sym_attributes_repeat1] = STATE(1965),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_POUND] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
    [sym__outer_block_doc_comment_start] = ACTIONS(2107),
    [sym__outer_line_doc_comment_start] = ACTIONS(2109),
  },
  [STATE(572)] = {
    [sym_line_comment] = STATE(572),
    [sym_block_comment] = STATE(572),
    [sym_identifier] = ACTIONS(1592),
    [anon_sym_LPAREN] = ACTIONS(1590),
    [anon_sym_RBRACE] = ACTIONS(1590),
    [anon_sym_COMMA] = ACTIONS(1590),
    [anon_sym_macro_rules] = ACTIONS(1592),
    [anon_sym_LBRACK] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1592),
    [anon_sym_STAR] = ACTIONS(1592),
    [anon_sym_QMARK] = ACTIONS(1590),
    [anon_sym_u8] = ACTIONS(1592),
    [anon_sym_i8] = ACTIONS(1592),
    [anon_sym_u16] = ACTIONS(1592),
    [anon_sym_i16] = ACTIONS(1592),
    [anon_sym_u32] = ACTIONS(1592),
    [anon_sym_i32] = ACTIONS(1592),
    [anon_sym_u64] = ACTIONS(1592),
    [anon_sym_i64] = ACTIONS(1592),
    [anon_sym_u128] = ACTIONS(1592),
    [anon_sym_i128] = ACTIONS(1592),
    [anon_sym_isize] = ACTIONS(1592),
    [anon_sym_usize] = ACTIONS(1592),
    [anon_sym_f32] = ACTIONS(1592),
    [anon_sym_f64] = ACTIONS(1592),
    [anon_sym_bool] = ACTIONS(1592),
    [anon_sym_str] = ACTIONS(1592),
    [anon_sym_char] = ACTIONS(1592),
    [anon_sym_DASH] = ACTIONS(1592),
    [anon_sym_SLASH] = ACTIONS(1592),
    [anon_sym_PERCENT] = ACTIONS(1592),
    [anon_sym_CARET] = ACTIONS(1592),
    [anon_sym_AMP] = ACTIONS(1592),
    [anon_sym_PIPE] = ACTIONS(1592),
    [anon_sym_AMP_AMP] = ACTIONS(1590),
    [anon_sym_PIPE_PIPE] = ACTIONS(1590),
    [anon_sym_LT_LT] = ACTIONS(1592),
    [anon_sym_GT_GT] = ACTIONS(1592),
    [anon_sym_PLUS_EQ] = ACTIONS(1590),
    [anon_sym_DASH_EQ] = ACTIONS(1590),
    [anon_sym_STAR_EQ] = ACTIONS(1590),
    [anon_sym_SLASH_EQ] = ACTIONS(1590),
    [anon_sym_PERCENT_EQ] = ACTIONS(1590),
    [anon_sym_CARET_EQ] = ACTIONS(1590),
    [anon_sym_AMP_EQ] = ACTIONS(1590),
    [anon_sym_PIPE_EQ] = ACTIONS(1590),
    [anon_sym_LT_LT_EQ] = ACTIONS(1590),
    [anon_sym_GT_GT_EQ] = ACTIONS(1590),
    [anon_sym_EQ] = ACTIONS(1592),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1592),
    [anon_sym_LT] = ACTIONS(1592),
    [anon_sym_GT_EQ] = ACTIONS(1590),
    [anon_sym_LT_EQ] = ACTIONS(1590),
    [anon_sym__] = ACTIONS(1592),
    [anon_sym_DOT] = ACTIONS(1592),
    [anon_sym_DOT_DOT] = ACTIONS(1592),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1590),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1590),
    [anon_sym_COLON_COLON] = ACTIONS(1590),
    [anon_sym_POUND] = ACTIONS(1590),
    [anon_sym_as] = ACTIONS(1592),
    [anon_sym_auto] = ACTIONS(1592),
    [anon_sym_await] = ACTIONS(1592),
    [anon_sym_const] = ACTIONS(1592),
    [anon_sym_default] = ACTIONS(1592),
    [anon_sym_gen] = ACTIONS(1592),
    [anon_sym_raw] = ACTIONS(1592),
    [anon_sym_union] = ACTIONS(1592),
    [anon_sym_safe] = ACTIONS(1592),
    [anon_sym_ref] = ACTIONS(1592),
    [sym_mutable_specifier] = ACTIONS(1592),
    [anon_sym_try] = ACTIONS(1592),
    [anon_sym_box] = ACTIONS(1592),
    [sym_integer_literal] = ACTIONS(1590),
    [aux_sym_string_literal_token1] = ACTIONS(1590),
    [sym_char_literal] = ACTIONS(1590),
    [anon_sym_true] = ACTIONS(1592),
    [anon_sym_false] = ACTIONS(1592),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1592),
    [sym_super] = ACTIONS(1592),
    [sym_crate] = ACTIONS(1592),
    [aux_sym_metavariable_token1] = ACTIONS(1590),
    [sym__raw_string_literal_start] = ACTIONS(1590),
    [sym_float_literal] = ACTIONS(1590),
    [sym__outer_block_doc_comment_start] = ACTIONS(1590),
    [sym__outer_line_doc_comment_start] = ACTIONS(1590),
  },
  [STATE(573)] = {
    [sym_line_comment] = STATE(573),
    [sym_block_comment] = STATE(573),
    [sym_identifier] = ACTIONS(1600),
    [anon_sym_LPAREN] = ACTIONS(1598),
    [anon_sym_RBRACE] = ACTIONS(1598),
    [anon_sym_COMMA] = ACTIONS(1598),
    [anon_sym_macro_rules] = ACTIONS(1600),
    [anon_sym_LBRACK] = ACTIONS(1598),
    [anon_sym_PLUS] = ACTIONS(1600),
    [anon_sym_STAR] = ACTIONS(1600),
    [anon_sym_QMARK] = ACTIONS(1598),
    [anon_sym_u8] = ACTIONS(1600),
    [anon_sym_i8] = ACTIONS(1600),
    [anon_sym_u16] = ACTIONS(1600),
    [anon_sym_i16] = ACTIONS(1600),
    [anon_sym_u32] = ACTIONS(1600),
    [anon_sym_i32] = ACTIONS(1600),
    [anon_sym_u64] = ACTIONS(1600),
    [anon_sym_i64] = ACTIONS(1600),
    [anon_sym_u128] = ACTIONS(1600),
    [anon_sym_i128] = ACTIONS(1600),
    [anon_sym_isize] = ACTIONS(1600),
    [anon_sym_usize] = ACTIONS(1600),
    [anon_sym_f32] = ACTIONS(1600),
    [anon_sym_f64] = ACTIONS(1600),
    [anon_sym_bool] = ACTIONS(1600),
    [anon_sym_str] = ACTIONS(1600),
    [anon_sym_char] = ACTIONS(1600),
    [anon_sym_DASH] = ACTIONS(1600),
    [anon_sym_SLASH] = ACTIONS(1600),
    [anon_sym_PERCENT] = ACTIONS(1600),
    [anon_sym_CARET] = ACTIONS(1600),
    [anon_sym_AMP] = ACTIONS(1600),
    [anon_sym_PIPE] = ACTIONS(1600),
    [anon_sym_AMP_AMP] = ACTIONS(1598),
    [anon_sym_PIPE_PIPE] = ACTIONS(1598),
    [anon_sym_LT_LT] = ACTIONS(1600),
    [anon_sym_GT_GT] = ACTIONS(1600),
    [anon_sym_PLUS_EQ] = ACTIONS(1598),
    [anon_sym_DASH_EQ] = ACTIONS(1598),
    [anon_sym_STAR_EQ] = ACTIONS(1598),
    [anon_sym_SLASH_EQ] = ACTIONS(1598),
    [anon_sym_PERCENT_EQ] = ACTIONS(1598),
    [anon_sym_CARET_EQ] = ACTIONS(1598),
    [anon_sym_AMP_EQ] = ACTIONS(1598),
    [anon_sym_PIPE_EQ] = ACTIONS(1598),
    [anon_sym_LT_LT_EQ] = ACTIONS(1598),
    [anon_sym_GT_GT_EQ] = ACTIONS(1598),
    [anon_sym_EQ] = ACTIONS(1600),
    [anon_sym_EQ_EQ] = ACTIONS(1598),
    [anon_sym_BANG_EQ] = ACTIONS(1598),
    [anon_sym_GT] = ACTIONS(1600),
    [anon_sym_LT] = ACTIONS(1600),
    [anon_sym_GT_EQ] = ACTIONS(1598),
    [anon_sym_LT_EQ] = ACTIONS(1598),
    [anon_sym__] = ACTIONS(1600),
    [anon_sym_DOT] = ACTIONS(1600),
    [anon_sym_DOT_DOT] = ACTIONS(1600),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1598),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1598),
    [anon_sym_COLON_COLON] = ACTIONS(1598),
    [anon_sym_POUND] = ACTIONS(1598),
    [anon_sym_as] = ACTIONS(1600),
    [anon_sym_auto] = ACTIONS(1600),
    [anon_sym_await] = ACTIONS(1600),
    [anon_sym_const] = ACTIONS(1600),
    [anon_sym_default] = ACTIONS(1600),
    [anon_sym_gen] = ACTIONS(1600),
    [anon_sym_raw] = ACTIONS(1600),
    [anon_sym_union] = ACTIONS(1600),
    [anon_sym_safe] = ACTIONS(1600),
    [anon_sym_ref] = ACTIONS(1600),
    [sym_mutable_specifier] = ACTIONS(1600),
    [anon_sym_try] = ACTIONS(1600),
    [anon_sym_box] = ACTIONS(1600),
    [sym_integer_literal] = ACTIONS(1598),
    [aux_sym_string_literal_token1] = ACTIONS(1598),
    [sym_char_literal] = ACTIONS(1598),
    [anon_sym_true] = ACTIONS(1600),
    [anon_sym_false] = ACTIONS(1600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1600),
    [sym_super] = ACTIONS(1600),
    [sym_crate] = ACTIONS(1600),
    [aux_sym_metavariable_token1] = ACTIONS(1598),
    [sym__raw_string_literal_start] = ACTIONS(1598),
    [sym_float_literal] = ACTIONS(1598),
    [sym__outer_block_doc_comment_start] = ACTIONS(1598),
    [sym__outer_line_doc_comment_start] = ACTIONS(1598),
  },
  [STATE(574)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5565),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(5247),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(574),
    [sym_block_comment] = STATE(574),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(2145),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_async] = ACTIONS(2115),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(575)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5565),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(5247),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(575),
    [sym_block_comment] = STATE(575),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(2147),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_async] = ACTIONS(2115),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(576)] = {
    [sym_line_comment] = STATE(576),
    [sym_block_comment] = STATE(576),
    [sym_identifier] = ACTIONS(1588),
    [anon_sym_LPAREN] = ACTIONS(1586),
    [anon_sym_RBRACE] = ACTIONS(1586),
    [anon_sym_COMMA] = ACTIONS(1586),
    [anon_sym_macro_rules] = ACTIONS(1588),
    [anon_sym_LBRACK] = ACTIONS(1586),
    [anon_sym_PLUS] = ACTIONS(1588),
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_QMARK] = ACTIONS(1586),
    [anon_sym_u8] = ACTIONS(1588),
    [anon_sym_i8] = ACTIONS(1588),
    [anon_sym_u16] = ACTIONS(1588),
    [anon_sym_i16] = ACTIONS(1588),
    [anon_sym_u32] = ACTIONS(1588),
    [anon_sym_i32] = ACTIONS(1588),
    [anon_sym_u64] = ACTIONS(1588),
    [anon_sym_i64] = ACTIONS(1588),
    [anon_sym_u128] = ACTIONS(1588),
    [anon_sym_i128] = ACTIONS(1588),
    [anon_sym_isize] = ACTIONS(1588),
    [anon_sym_usize] = ACTIONS(1588),
    [anon_sym_f32] = ACTIONS(1588),
    [anon_sym_f64] = ACTIONS(1588),
    [anon_sym_bool] = ACTIONS(1588),
    [anon_sym_str] = ACTIONS(1588),
    [anon_sym_char] = ACTIONS(1588),
    [anon_sym_DASH] = ACTIONS(1588),
    [anon_sym_SLASH] = ACTIONS(1588),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1586),
    [anon_sym_PIPE_PIPE] = ACTIONS(1586),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1588),
    [anon_sym_PLUS_EQ] = ACTIONS(1586),
    [anon_sym_DASH_EQ] = ACTIONS(1586),
    [anon_sym_STAR_EQ] = ACTIONS(1586),
    [anon_sym_SLASH_EQ] = ACTIONS(1586),
    [anon_sym_PERCENT_EQ] = ACTIONS(1586),
    [anon_sym_CARET_EQ] = ACTIONS(1586),
    [anon_sym_AMP_EQ] = ACTIONS(1586),
    [anon_sym_PIPE_EQ] = ACTIONS(1586),
    [anon_sym_LT_LT_EQ] = ACTIONS(1586),
    [anon_sym_GT_GT_EQ] = ACTIONS(1586),
    [anon_sym_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1586),
    [anon_sym_BANG_EQ] = ACTIONS(1586),
    [anon_sym_GT] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1586),
    [anon_sym_LT_EQ] = ACTIONS(1586),
    [anon_sym__] = ACTIONS(1588),
    [anon_sym_DOT] = ACTIONS(1588),
    [anon_sym_DOT_DOT] = ACTIONS(1588),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1586),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1586),
    [anon_sym_COLON_COLON] = ACTIONS(1586),
    [anon_sym_POUND] = ACTIONS(1586),
    [anon_sym_as] = ACTIONS(1588),
    [anon_sym_auto] = ACTIONS(1588),
    [anon_sym_await] = ACTIONS(1588),
    [anon_sym_const] = ACTIONS(1588),
    [anon_sym_default] = ACTIONS(1588),
    [anon_sym_gen] = ACTIONS(1588),
    [anon_sym_raw] = ACTIONS(1588),
    [anon_sym_union] = ACTIONS(1588),
    [anon_sym_safe] = ACTIONS(1588),
    [anon_sym_ref] = ACTIONS(1588),
    [sym_mutable_specifier] = ACTIONS(1588),
    [anon_sym_try] = ACTIONS(1588),
    [anon_sym_box] = ACTIONS(1588),
    [sym_integer_literal] = ACTIONS(1586),
    [aux_sym_string_literal_token1] = ACTIONS(1586),
    [sym_char_literal] = ACTIONS(1586),
    [anon_sym_true] = ACTIONS(1588),
    [anon_sym_false] = ACTIONS(1588),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1588),
    [sym_super] = ACTIONS(1588),
    [sym_crate] = ACTIONS(1588),
    [aux_sym_metavariable_token1] = ACTIONS(1586),
    [sym__raw_string_literal_start] = ACTIONS(1586),
    [sym_float_literal] = ACTIONS(1586),
    [sym__outer_block_doc_comment_start] = ACTIONS(1586),
    [sym__outer_line_doc_comment_start] = ACTIONS(1586),
  },
  [STATE(577)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5565),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(5247),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(577),
    [sym_block_comment] = STATE(577),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(2149),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_async] = ACTIONS(2115),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(578)] = {
    [sym_line_comment] = STATE(578),
    [sym_block_comment] = STATE(578),
    [sym_identifier] = ACTIONS(1528),
    [anon_sym_LPAREN] = ACTIONS(1526),
    [anon_sym_RBRACE] = ACTIONS(1526),
    [anon_sym_COMMA] = ACTIONS(1526),
    [anon_sym_macro_rules] = ACTIONS(1528),
    [anon_sym_LBRACK] = ACTIONS(1526),
    [anon_sym_PLUS] = ACTIONS(1528),
    [anon_sym_STAR] = ACTIONS(1528),
    [anon_sym_QMARK] = ACTIONS(1526),
    [anon_sym_u8] = ACTIONS(1528),
    [anon_sym_i8] = ACTIONS(1528),
    [anon_sym_u16] = ACTIONS(1528),
    [anon_sym_i16] = ACTIONS(1528),
    [anon_sym_u32] = ACTIONS(1528),
    [anon_sym_i32] = ACTIONS(1528),
    [anon_sym_u64] = ACTIONS(1528),
    [anon_sym_i64] = ACTIONS(1528),
    [anon_sym_u128] = ACTIONS(1528),
    [anon_sym_i128] = ACTIONS(1528),
    [anon_sym_isize] = ACTIONS(1528),
    [anon_sym_usize] = ACTIONS(1528),
    [anon_sym_f32] = ACTIONS(1528),
    [anon_sym_f64] = ACTIONS(1528),
    [anon_sym_bool] = ACTIONS(1528),
    [anon_sym_str] = ACTIONS(1528),
    [anon_sym_char] = ACTIONS(1528),
    [anon_sym_DASH] = ACTIONS(1528),
    [anon_sym_SLASH] = ACTIONS(1528),
    [anon_sym_PERCENT] = ACTIONS(1528),
    [anon_sym_CARET] = ACTIONS(1528),
    [anon_sym_AMP] = ACTIONS(1528),
    [anon_sym_PIPE] = ACTIONS(1528),
    [anon_sym_AMP_AMP] = ACTIONS(1526),
    [anon_sym_PIPE_PIPE] = ACTIONS(1526),
    [anon_sym_LT_LT] = ACTIONS(1528),
    [anon_sym_GT_GT] = ACTIONS(1528),
    [anon_sym_PLUS_EQ] = ACTIONS(1526),
    [anon_sym_DASH_EQ] = ACTIONS(1526),
    [anon_sym_STAR_EQ] = ACTIONS(1526),
    [anon_sym_SLASH_EQ] = ACTIONS(1526),
    [anon_sym_PERCENT_EQ] = ACTIONS(1526),
    [anon_sym_CARET_EQ] = ACTIONS(1526),
    [anon_sym_AMP_EQ] = ACTIONS(1526),
    [anon_sym_PIPE_EQ] = ACTIONS(1526),
    [anon_sym_LT_LT_EQ] = ACTIONS(1526),
    [anon_sym_GT_GT_EQ] = ACTIONS(1526),
    [anon_sym_EQ] = ACTIONS(1528),
    [anon_sym_EQ_EQ] = ACTIONS(1526),
    [anon_sym_BANG_EQ] = ACTIONS(1526),
    [anon_sym_GT] = ACTIONS(1528),
    [anon_sym_LT] = ACTIONS(1528),
    [anon_sym_GT_EQ] = ACTIONS(1526),
    [anon_sym_LT_EQ] = ACTIONS(1526),
    [anon_sym__] = ACTIONS(1528),
    [anon_sym_DOT] = ACTIONS(1528),
    [anon_sym_DOT_DOT] = ACTIONS(1528),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1526),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1526),
    [anon_sym_COLON_COLON] = ACTIONS(1526),
    [anon_sym_POUND] = ACTIONS(1526),
    [anon_sym_as] = ACTIONS(1528),
    [anon_sym_auto] = ACTIONS(1528),
    [anon_sym_await] = ACTIONS(1528),
    [anon_sym_const] = ACTIONS(1528),
    [anon_sym_default] = ACTIONS(1528),
    [anon_sym_gen] = ACTIONS(1528),
    [anon_sym_raw] = ACTIONS(1528),
    [anon_sym_union] = ACTIONS(1528),
    [anon_sym_safe] = ACTIONS(1528),
    [anon_sym_ref] = ACTIONS(1528),
    [sym_mutable_specifier] = ACTIONS(1528),
    [anon_sym_try] = ACTIONS(1528),
    [anon_sym_box] = ACTIONS(1528),
    [sym_integer_literal] = ACTIONS(1526),
    [aux_sym_string_literal_token1] = ACTIONS(1526),
    [sym_char_literal] = ACTIONS(1526),
    [anon_sym_true] = ACTIONS(1528),
    [anon_sym_false] = ACTIONS(1528),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1528),
    [sym_super] = ACTIONS(1528),
    [sym_crate] = ACTIONS(1528),
    [aux_sym_metavariable_token1] = ACTIONS(1526),
    [sym__raw_string_literal_start] = ACTIONS(1526),
    [sym_float_literal] = ACTIONS(1526),
    [sym__outer_block_doc_comment_start] = ACTIONS(1526),
    [sym__outer_line_doc_comment_start] = ACTIONS(1526),
  },
  [STATE(579)] = {
    [sym_line_comment] = STATE(579),
    [sym_block_comment] = STATE(579),
    [sym_identifier] = ACTIONS(1532),
    [anon_sym_LPAREN] = ACTIONS(1530),
    [anon_sym_RBRACE] = ACTIONS(1530),
    [anon_sym_COMMA] = ACTIONS(1530),
    [anon_sym_macro_rules] = ACTIONS(1532),
    [anon_sym_LBRACK] = ACTIONS(1530),
    [anon_sym_PLUS] = ACTIONS(1532),
    [anon_sym_STAR] = ACTIONS(1532),
    [anon_sym_QMARK] = ACTIONS(1530),
    [anon_sym_u8] = ACTIONS(1532),
    [anon_sym_i8] = ACTIONS(1532),
    [anon_sym_u16] = ACTIONS(1532),
    [anon_sym_i16] = ACTIONS(1532),
    [anon_sym_u32] = ACTIONS(1532),
    [anon_sym_i32] = ACTIONS(1532),
    [anon_sym_u64] = ACTIONS(1532),
    [anon_sym_i64] = ACTIONS(1532),
    [anon_sym_u128] = ACTIONS(1532),
    [anon_sym_i128] = ACTIONS(1532),
    [anon_sym_isize] = ACTIONS(1532),
    [anon_sym_usize] = ACTIONS(1532),
    [anon_sym_f32] = ACTIONS(1532),
    [anon_sym_f64] = ACTIONS(1532),
    [anon_sym_bool] = ACTIONS(1532),
    [anon_sym_str] = ACTIONS(1532),
    [anon_sym_char] = ACTIONS(1532),
    [anon_sym_DASH] = ACTIONS(1532),
    [anon_sym_SLASH] = ACTIONS(1532),
    [anon_sym_PERCENT] = ACTIONS(1532),
    [anon_sym_CARET] = ACTIONS(1532),
    [anon_sym_AMP] = ACTIONS(1532),
    [anon_sym_PIPE] = ACTIONS(1532),
    [anon_sym_AMP_AMP] = ACTIONS(1530),
    [anon_sym_PIPE_PIPE] = ACTIONS(1530),
    [anon_sym_LT_LT] = ACTIONS(1532),
    [anon_sym_GT_GT] = ACTIONS(1532),
    [anon_sym_PLUS_EQ] = ACTIONS(1530),
    [anon_sym_DASH_EQ] = ACTIONS(1530),
    [anon_sym_STAR_EQ] = ACTIONS(1530),
    [anon_sym_SLASH_EQ] = ACTIONS(1530),
    [anon_sym_PERCENT_EQ] = ACTIONS(1530),
    [anon_sym_CARET_EQ] = ACTIONS(1530),
    [anon_sym_AMP_EQ] = ACTIONS(1530),
    [anon_sym_PIPE_EQ] = ACTIONS(1530),
    [anon_sym_LT_LT_EQ] = ACTIONS(1530),
    [anon_sym_GT_GT_EQ] = ACTIONS(1530),
    [anon_sym_EQ] = ACTIONS(1532),
    [anon_sym_EQ_EQ] = ACTIONS(1530),
    [anon_sym_BANG_EQ] = ACTIONS(1530),
    [anon_sym_GT] = ACTIONS(1532),
    [anon_sym_LT] = ACTIONS(1532),
    [anon_sym_GT_EQ] = ACTIONS(1530),
    [anon_sym_LT_EQ] = ACTIONS(1530),
    [anon_sym__] = ACTIONS(1532),
    [anon_sym_DOT] = ACTIONS(1532),
    [anon_sym_DOT_DOT] = ACTIONS(1532),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1530),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1530),
    [anon_sym_COLON_COLON] = ACTIONS(1530),
    [anon_sym_POUND] = ACTIONS(1530),
    [anon_sym_as] = ACTIONS(1532),
    [anon_sym_auto] = ACTIONS(1532),
    [anon_sym_await] = ACTIONS(1532),
    [anon_sym_const] = ACTIONS(1532),
    [anon_sym_default] = ACTIONS(1532),
    [anon_sym_gen] = ACTIONS(1532),
    [anon_sym_raw] = ACTIONS(1532),
    [anon_sym_union] = ACTIONS(1532),
    [anon_sym_safe] = ACTIONS(1532),
    [anon_sym_ref] = ACTIONS(1532),
    [sym_mutable_specifier] = ACTIONS(1532),
    [anon_sym_try] = ACTIONS(1532),
    [anon_sym_box] = ACTIONS(1532),
    [sym_integer_literal] = ACTIONS(1530),
    [aux_sym_string_literal_token1] = ACTIONS(1530),
    [sym_char_literal] = ACTIONS(1530),
    [anon_sym_true] = ACTIONS(1532),
    [anon_sym_false] = ACTIONS(1532),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1532),
    [sym_super] = ACTIONS(1532),
    [sym_crate] = ACTIONS(1532),
    [aux_sym_metavariable_token1] = ACTIONS(1530),
    [sym__raw_string_literal_start] = ACTIONS(1530),
    [sym_float_literal] = ACTIONS(1530),
    [sym__outer_block_doc_comment_start] = ACTIONS(1530),
    [sym__outer_line_doc_comment_start] = ACTIONS(1530),
  },
  [STATE(580)] = {
    [sym_line_comment] = STATE(580),
    [sym_block_comment] = STATE(580),
    [sym_identifier] = ACTIONS(1516),
    [anon_sym_LPAREN] = ACTIONS(1514),
    [anon_sym_RBRACE] = ACTIONS(1514),
    [anon_sym_COMMA] = ACTIONS(1514),
    [anon_sym_macro_rules] = ACTIONS(1516),
    [anon_sym_LBRACK] = ACTIONS(1514),
    [anon_sym_PLUS] = ACTIONS(1516),
    [anon_sym_STAR] = ACTIONS(1516),
    [anon_sym_QMARK] = ACTIONS(1514),
    [anon_sym_u8] = ACTIONS(1516),
    [anon_sym_i8] = ACTIONS(1516),
    [anon_sym_u16] = ACTIONS(1516),
    [anon_sym_i16] = ACTIONS(1516),
    [anon_sym_u32] = ACTIONS(1516),
    [anon_sym_i32] = ACTIONS(1516),
    [anon_sym_u64] = ACTIONS(1516),
    [anon_sym_i64] = ACTIONS(1516),
    [anon_sym_u128] = ACTIONS(1516),
    [anon_sym_i128] = ACTIONS(1516),
    [anon_sym_isize] = ACTIONS(1516),
    [anon_sym_usize] = ACTIONS(1516),
    [anon_sym_f32] = ACTIONS(1516),
    [anon_sym_f64] = ACTIONS(1516),
    [anon_sym_bool] = ACTIONS(1516),
    [anon_sym_str] = ACTIONS(1516),
    [anon_sym_char] = ACTIONS(1516),
    [anon_sym_DASH] = ACTIONS(1516),
    [anon_sym_SLASH] = ACTIONS(1516),
    [anon_sym_PERCENT] = ACTIONS(1516),
    [anon_sym_CARET] = ACTIONS(1516),
    [anon_sym_AMP] = ACTIONS(1516),
    [anon_sym_PIPE] = ACTIONS(1516),
    [anon_sym_AMP_AMP] = ACTIONS(1514),
    [anon_sym_PIPE_PIPE] = ACTIONS(1514),
    [anon_sym_LT_LT] = ACTIONS(1516),
    [anon_sym_GT_GT] = ACTIONS(1516),
    [anon_sym_PLUS_EQ] = ACTIONS(1514),
    [anon_sym_DASH_EQ] = ACTIONS(1514),
    [anon_sym_STAR_EQ] = ACTIONS(1514),
    [anon_sym_SLASH_EQ] = ACTIONS(1514),
    [anon_sym_PERCENT_EQ] = ACTIONS(1514),
    [anon_sym_CARET_EQ] = ACTIONS(1514),
    [anon_sym_AMP_EQ] = ACTIONS(1514),
    [anon_sym_PIPE_EQ] = ACTIONS(1514),
    [anon_sym_LT_LT_EQ] = ACTIONS(1514),
    [anon_sym_GT_GT_EQ] = ACTIONS(1514),
    [anon_sym_EQ] = ACTIONS(1516),
    [anon_sym_EQ_EQ] = ACTIONS(1514),
    [anon_sym_BANG_EQ] = ACTIONS(1514),
    [anon_sym_GT] = ACTIONS(1516),
    [anon_sym_LT] = ACTIONS(1516),
    [anon_sym_GT_EQ] = ACTIONS(1514),
    [anon_sym_LT_EQ] = ACTIONS(1514),
    [anon_sym__] = ACTIONS(1516),
    [anon_sym_DOT] = ACTIONS(1516),
    [anon_sym_DOT_DOT] = ACTIONS(1516),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1514),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1514),
    [anon_sym_COLON_COLON] = ACTIONS(1514),
    [anon_sym_POUND] = ACTIONS(1514),
    [anon_sym_as] = ACTIONS(1516),
    [anon_sym_auto] = ACTIONS(1516),
    [anon_sym_await] = ACTIONS(1516),
    [anon_sym_const] = ACTIONS(1516),
    [anon_sym_default] = ACTIONS(1516),
    [anon_sym_gen] = ACTIONS(1516),
    [anon_sym_raw] = ACTIONS(1516),
    [anon_sym_union] = ACTIONS(1516),
    [anon_sym_safe] = ACTIONS(1516),
    [anon_sym_ref] = ACTIONS(1516),
    [sym_mutable_specifier] = ACTIONS(1516),
    [anon_sym_try] = ACTIONS(1516),
    [anon_sym_box] = ACTIONS(1516),
    [sym_integer_literal] = ACTIONS(1514),
    [aux_sym_string_literal_token1] = ACTIONS(1514),
    [sym_char_literal] = ACTIONS(1514),
    [anon_sym_true] = ACTIONS(1516),
    [anon_sym_false] = ACTIONS(1516),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1516),
    [sym_super] = ACTIONS(1516),
    [sym_crate] = ACTIONS(1516),
    [aux_sym_metavariable_token1] = ACTIONS(1514),
    [sym__raw_string_literal_start] = ACTIONS(1514),
    [sym_float_literal] = ACTIONS(1514),
    [sym__outer_block_doc_comment_start] = ACTIONS(1514),
    [sym__outer_line_doc_comment_start] = ACTIONS(1514),
  },
  [STATE(581)] = {
    [sym_line_comment] = STATE(581),
    [sym_block_comment] = STATE(581),
    [sym_identifier] = ACTIONS(1604),
    [anon_sym_LPAREN] = ACTIONS(1602),
    [anon_sym_RBRACE] = ACTIONS(1602),
    [anon_sym_COMMA] = ACTIONS(1602),
    [anon_sym_macro_rules] = ACTIONS(1604),
    [anon_sym_LBRACK] = ACTIONS(1602),
    [anon_sym_PLUS] = ACTIONS(1604),
    [anon_sym_STAR] = ACTIONS(1604),
    [anon_sym_QMARK] = ACTIONS(1602),
    [anon_sym_u8] = ACTIONS(1604),
    [anon_sym_i8] = ACTIONS(1604),
    [anon_sym_u16] = ACTIONS(1604),
    [anon_sym_i16] = ACTIONS(1604),
    [anon_sym_u32] = ACTIONS(1604),
    [anon_sym_i32] = ACTIONS(1604),
    [anon_sym_u64] = ACTIONS(1604),
    [anon_sym_i64] = ACTIONS(1604),
    [anon_sym_u128] = ACTIONS(1604),
    [anon_sym_i128] = ACTIONS(1604),
    [anon_sym_isize] = ACTIONS(1604),
    [anon_sym_usize] = ACTIONS(1604),
    [anon_sym_f32] = ACTIONS(1604),
    [anon_sym_f64] = ACTIONS(1604),
    [anon_sym_bool] = ACTIONS(1604),
    [anon_sym_str] = ACTIONS(1604),
    [anon_sym_char] = ACTIONS(1604),
    [anon_sym_DASH] = ACTIONS(1604),
    [anon_sym_SLASH] = ACTIONS(1604),
    [anon_sym_PERCENT] = ACTIONS(1604),
    [anon_sym_CARET] = ACTIONS(1604),
    [anon_sym_AMP] = ACTIONS(1604),
    [anon_sym_PIPE] = ACTIONS(1604),
    [anon_sym_AMP_AMP] = ACTIONS(1602),
    [anon_sym_PIPE_PIPE] = ACTIONS(1602),
    [anon_sym_LT_LT] = ACTIONS(1604),
    [anon_sym_GT_GT] = ACTIONS(1604),
    [anon_sym_PLUS_EQ] = ACTIONS(1602),
    [anon_sym_DASH_EQ] = ACTIONS(1602),
    [anon_sym_STAR_EQ] = ACTIONS(1602),
    [anon_sym_SLASH_EQ] = ACTIONS(1602),
    [anon_sym_PERCENT_EQ] = ACTIONS(1602),
    [anon_sym_CARET_EQ] = ACTIONS(1602),
    [anon_sym_AMP_EQ] = ACTIONS(1602),
    [anon_sym_PIPE_EQ] = ACTIONS(1602),
    [anon_sym_LT_LT_EQ] = ACTIONS(1602),
    [anon_sym_GT_GT_EQ] = ACTIONS(1602),
    [anon_sym_EQ] = ACTIONS(1604),
    [anon_sym_EQ_EQ] = ACTIONS(1602),
    [anon_sym_BANG_EQ] = ACTIONS(1602),
    [anon_sym_GT] = ACTIONS(1604),
    [anon_sym_LT] = ACTIONS(1604),
    [anon_sym_GT_EQ] = ACTIONS(1602),
    [anon_sym_LT_EQ] = ACTIONS(1602),
    [anon_sym__] = ACTIONS(1604),
    [anon_sym_DOT] = ACTIONS(1604),
    [anon_sym_DOT_DOT] = ACTIONS(1604),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1602),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1602),
    [anon_sym_COLON_COLON] = ACTIONS(1602),
    [anon_sym_POUND] = ACTIONS(1602),
    [anon_sym_as] = ACTIONS(1604),
    [anon_sym_auto] = ACTIONS(1604),
    [anon_sym_await] = ACTIONS(1604),
    [anon_sym_const] = ACTIONS(1604),
    [anon_sym_default] = ACTIONS(1604),
    [anon_sym_gen] = ACTIONS(1604),
    [anon_sym_raw] = ACTIONS(1604),
    [anon_sym_union] = ACTIONS(1604),
    [anon_sym_safe] = ACTIONS(1604),
    [anon_sym_ref] = ACTIONS(1604),
    [sym_mutable_specifier] = ACTIONS(1604),
    [anon_sym_try] = ACTIONS(1604),
    [anon_sym_box] = ACTIONS(1604),
    [sym_integer_literal] = ACTIONS(1602),
    [aux_sym_string_literal_token1] = ACTIONS(1602),
    [sym_char_literal] = ACTIONS(1602),
    [anon_sym_true] = ACTIONS(1604),
    [anon_sym_false] = ACTIONS(1604),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1604),
    [sym_super] = ACTIONS(1604),
    [sym_crate] = ACTIONS(1604),
    [aux_sym_metavariable_token1] = ACTIONS(1602),
    [sym__raw_string_literal_start] = ACTIONS(1602),
    [sym_float_literal] = ACTIONS(1602),
    [sym__outer_block_doc_comment_start] = ACTIONS(1602),
    [sym__outer_line_doc_comment_start] = ACTIONS(1602),
  },
  [STATE(582)] = {
    [sym_line_comment] = STATE(582),
    [sym_block_comment] = STATE(582),
    [sym_identifier] = ACTIONS(1544),
    [anon_sym_LPAREN] = ACTIONS(1542),
    [anon_sym_RBRACE] = ACTIONS(1542),
    [anon_sym_COMMA] = ACTIONS(1542),
    [anon_sym_macro_rules] = ACTIONS(1544),
    [anon_sym_LBRACK] = ACTIONS(1542),
    [anon_sym_PLUS] = ACTIONS(1544),
    [anon_sym_STAR] = ACTIONS(1544),
    [anon_sym_QMARK] = ACTIONS(1542),
    [anon_sym_u8] = ACTIONS(1544),
    [anon_sym_i8] = ACTIONS(1544),
    [anon_sym_u16] = ACTIONS(1544),
    [anon_sym_i16] = ACTIONS(1544),
    [anon_sym_u32] = ACTIONS(1544),
    [anon_sym_i32] = ACTIONS(1544),
    [anon_sym_u64] = ACTIONS(1544),
    [anon_sym_i64] = ACTIONS(1544),
    [anon_sym_u128] = ACTIONS(1544),
    [anon_sym_i128] = ACTIONS(1544),
    [anon_sym_isize] = ACTIONS(1544),
    [anon_sym_usize] = ACTIONS(1544),
    [anon_sym_f32] = ACTIONS(1544),
    [anon_sym_f64] = ACTIONS(1544),
    [anon_sym_bool] = ACTIONS(1544),
    [anon_sym_str] = ACTIONS(1544),
    [anon_sym_char] = ACTIONS(1544),
    [anon_sym_DASH] = ACTIONS(1544),
    [anon_sym_SLASH] = ACTIONS(1544),
    [anon_sym_PERCENT] = ACTIONS(1544),
    [anon_sym_CARET] = ACTIONS(1544),
    [anon_sym_AMP] = ACTIONS(1544),
    [anon_sym_PIPE] = ACTIONS(1544),
    [anon_sym_AMP_AMP] = ACTIONS(1542),
    [anon_sym_PIPE_PIPE] = ACTIONS(1542),
    [anon_sym_LT_LT] = ACTIONS(1544),
    [anon_sym_GT_GT] = ACTIONS(1544),
    [anon_sym_PLUS_EQ] = ACTIONS(1542),
    [anon_sym_DASH_EQ] = ACTIONS(1542),
    [anon_sym_STAR_EQ] = ACTIONS(1542),
    [anon_sym_SLASH_EQ] = ACTIONS(1542),
    [anon_sym_PERCENT_EQ] = ACTIONS(1542),
    [anon_sym_CARET_EQ] = ACTIONS(1542),
    [anon_sym_AMP_EQ] = ACTIONS(1542),
    [anon_sym_PIPE_EQ] = ACTIONS(1542),
    [anon_sym_LT_LT_EQ] = ACTIONS(1542),
    [anon_sym_GT_GT_EQ] = ACTIONS(1542),
    [anon_sym_EQ] = ACTIONS(1544),
    [anon_sym_EQ_EQ] = ACTIONS(1542),
    [anon_sym_BANG_EQ] = ACTIONS(1542),
    [anon_sym_GT] = ACTIONS(1544),
    [anon_sym_LT] = ACTIONS(1544),
    [anon_sym_GT_EQ] = ACTIONS(1542),
    [anon_sym_LT_EQ] = ACTIONS(1542),
    [anon_sym__] = ACTIONS(1544),
    [anon_sym_DOT] = ACTIONS(1544),
    [anon_sym_DOT_DOT] = ACTIONS(1544),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1542),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1542),
    [anon_sym_COLON_COLON] = ACTIONS(1542),
    [anon_sym_POUND] = ACTIONS(1542),
    [anon_sym_as] = ACTIONS(1544),
    [anon_sym_auto] = ACTIONS(1544),
    [anon_sym_await] = ACTIONS(1544),
    [anon_sym_const] = ACTIONS(1544),
    [anon_sym_default] = ACTIONS(1544),
    [anon_sym_gen] = ACTIONS(1544),
    [anon_sym_raw] = ACTIONS(1544),
    [anon_sym_union] = ACTIONS(1544),
    [anon_sym_safe] = ACTIONS(1544),
    [anon_sym_ref] = ACTIONS(1544),
    [sym_mutable_specifier] = ACTIONS(1544),
    [anon_sym_try] = ACTIONS(1544),
    [anon_sym_box] = ACTIONS(1544),
    [sym_integer_literal] = ACTIONS(1542),
    [aux_sym_string_literal_token1] = ACTIONS(1542),
    [sym_char_literal] = ACTIONS(1542),
    [anon_sym_true] = ACTIONS(1544),
    [anon_sym_false] = ACTIONS(1544),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1544),
    [sym_super] = ACTIONS(1544),
    [sym_crate] = ACTIONS(1544),
    [aux_sym_metavariable_token1] = ACTIONS(1542),
    [sym__raw_string_literal_start] = ACTIONS(1542),
    [sym_float_literal] = ACTIONS(1542),
    [sym__outer_block_doc_comment_start] = ACTIONS(1542),
    [sym__outer_line_doc_comment_start] = ACTIONS(1542),
  },
  [STATE(583)] = {
    [sym_line_comment] = STATE(583),
    [sym_block_comment] = STATE(583),
    [sym_identifier] = ACTIONS(1548),
    [anon_sym_LPAREN] = ACTIONS(1546),
    [anon_sym_RBRACE] = ACTIONS(1546),
    [anon_sym_COMMA] = ACTIONS(1546),
    [anon_sym_macro_rules] = ACTIONS(1548),
    [anon_sym_LBRACK] = ACTIONS(1546),
    [anon_sym_PLUS] = ACTIONS(1548),
    [anon_sym_STAR] = ACTIONS(1548),
    [anon_sym_QMARK] = ACTIONS(1546),
    [anon_sym_u8] = ACTIONS(1548),
    [anon_sym_i8] = ACTIONS(1548),
    [anon_sym_u16] = ACTIONS(1548),
    [anon_sym_i16] = ACTIONS(1548),
    [anon_sym_u32] = ACTIONS(1548),
    [anon_sym_i32] = ACTIONS(1548),
    [anon_sym_u64] = ACTIONS(1548),
    [anon_sym_i64] = ACTIONS(1548),
    [anon_sym_u128] = ACTIONS(1548),
    [anon_sym_i128] = ACTIONS(1548),
    [anon_sym_isize] = ACTIONS(1548),
    [anon_sym_usize] = ACTIONS(1548),
    [anon_sym_f32] = ACTIONS(1548),
    [anon_sym_f64] = ACTIONS(1548),
    [anon_sym_bool] = ACTIONS(1548),
    [anon_sym_str] = ACTIONS(1548),
    [anon_sym_char] = ACTIONS(1548),
    [anon_sym_DASH] = ACTIONS(1548),
    [anon_sym_SLASH] = ACTIONS(1548),
    [anon_sym_PERCENT] = ACTIONS(1548),
    [anon_sym_CARET] = ACTIONS(1548),
    [anon_sym_AMP] = ACTIONS(1548),
    [anon_sym_PIPE] = ACTIONS(1548),
    [anon_sym_AMP_AMP] = ACTIONS(1546),
    [anon_sym_PIPE_PIPE] = ACTIONS(1546),
    [anon_sym_LT_LT] = ACTIONS(1548),
    [anon_sym_GT_GT] = ACTIONS(1548),
    [anon_sym_PLUS_EQ] = ACTIONS(1546),
    [anon_sym_DASH_EQ] = ACTIONS(1546),
    [anon_sym_STAR_EQ] = ACTIONS(1546),
    [anon_sym_SLASH_EQ] = ACTIONS(1546),
    [anon_sym_PERCENT_EQ] = ACTIONS(1546),
    [anon_sym_CARET_EQ] = ACTIONS(1546),
    [anon_sym_AMP_EQ] = ACTIONS(1546),
    [anon_sym_PIPE_EQ] = ACTIONS(1546),
    [anon_sym_LT_LT_EQ] = ACTIONS(1546),
    [anon_sym_GT_GT_EQ] = ACTIONS(1546),
    [anon_sym_EQ] = ACTIONS(1548),
    [anon_sym_EQ_EQ] = ACTIONS(1546),
    [anon_sym_BANG_EQ] = ACTIONS(1546),
    [anon_sym_GT] = ACTIONS(1548),
    [anon_sym_LT] = ACTIONS(1548),
    [anon_sym_GT_EQ] = ACTIONS(1546),
    [anon_sym_LT_EQ] = ACTIONS(1546),
    [anon_sym__] = ACTIONS(1548),
    [anon_sym_DOT] = ACTIONS(1548),
    [anon_sym_DOT_DOT] = ACTIONS(1548),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1546),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1546),
    [anon_sym_COLON_COLON] = ACTIONS(1546),
    [anon_sym_POUND] = ACTIONS(1546),
    [anon_sym_as] = ACTIONS(1548),
    [anon_sym_auto] = ACTIONS(1548),
    [anon_sym_await] = ACTIONS(1548),
    [anon_sym_const] = ACTIONS(1548),
    [anon_sym_default] = ACTIONS(1548),
    [anon_sym_gen] = ACTIONS(1548),
    [anon_sym_raw] = ACTIONS(1548),
    [anon_sym_union] = ACTIONS(1548),
    [anon_sym_safe] = ACTIONS(1548),
    [anon_sym_ref] = ACTIONS(1548),
    [sym_mutable_specifier] = ACTIONS(1548),
    [anon_sym_try] = ACTIONS(1548),
    [anon_sym_box] = ACTIONS(1548),
    [sym_integer_literal] = ACTIONS(1546),
    [aux_sym_string_literal_token1] = ACTIONS(1546),
    [sym_char_literal] = ACTIONS(1546),
    [anon_sym_true] = ACTIONS(1548),
    [anon_sym_false] = ACTIONS(1548),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1548),
    [sym_super] = ACTIONS(1548),
    [sym_crate] = ACTIONS(1548),
    [aux_sym_metavariable_token1] = ACTIONS(1546),
    [sym__raw_string_literal_start] = ACTIONS(1546),
    [sym_float_literal] = ACTIONS(1546),
    [sym__outer_block_doc_comment_start] = ACTIONS(1546),
    [sym__outer_line_doc_comment_start] = ACTIONS(1546),
  },
  [STATE(584)] = {
    [sym_line_comment] = STATE(584),
    [sym_block_comment] = STATE(584),
    [sym_identifier] = ACTIONS(1572),
    [anon_sym_LPAREN] = ACTIONS(1570),
    [anon_sym_RBRACE] = ACTIONS(1570),
    [anon_sym_COMMA] = ACTIONS(1570),
    [anon_sym_macro_rules] = ACTIONS(1572),
    [anon_sym_LBRACK] = ACTIONS(1570),
    [anon_sym_PLUS] = ACTIONS(1572),
    [anon_sym_STAR] = ACTIONS(1572),
    [anon_sym_QMARK] = ACTIONS(1570),
    [anon_sym_u8] = ACTIONS(1572),
    [anon_sym_i8] = ACTIONS(1572),
    [anon_sym_u16] = ACTIONS(1572),
    [anon_sym_i16] = ACTIONS(1572),
    [anon_sym_u32] = ACTIONS(1572),
    [anon_sym_i32] = ACTIONS(1572),
    [anon_sym_u64] = ACTIONS(1572),
    [anon_sym_i64] = ACTIONS(1572),
    [anon_sym_u128] = ACTIONS(1572),
    [anon_sym_i128] = ACTIONS(1572),
    [anon_sym_isize] = ACTIONS(1572),
    [anon_sym_usize] = ACTIONS(1572),
    [anon_sym_f32] = ACTIONS(1572),
    [anon_sym_f64] = ACTIONS(1572),
    [anon_sym_bool] = ACTIONS(1572),
    [anon_sym_str] = ACTIONS(1572),
    [anon_sym_char] = ACTIONS(1572),
    [anon_sym_DASH] = ACTIONS(1572),
    [anon_sym_SLASH] = ACTIONS(1572),
    [anon_sym_PERCENT] = ACTIONS(1572),
    [anon_sym_CARET] = ACTIONS(1572),
    [anon_sym_AMP] = ACTIONS(1572),
    [anon_sym_PIPE] = ACTIONS(1572),
    [anon_sym_AMP_AMP] = ACTIONS(1570),
    [anon_sym_PIPE_PIPE] = ACTIONS(1570),
    [anon_sym_LT_LT] = ACTIONS(1572),
    [anon_sym_GT_GT] = ACTIONS(1572),
    [anon_sym_PLUS_EQ] = ACTIONS(1570),
    [anon_sym_DASH_EQ] = ACTIONS(1570),
    [anon_sym_STAR_EQ] = ACTIONS(1570),
    [anon_sym_SLASH_EQ] = ACTIONS(1570),
    [anon_sym_PERCENT_EQ] = ACTIONS(1570),
    [anon_sym_CARET_EQ] = ACTIONS(1570),
    [anon_sym_AMP_EQ] = ACTIONS(1570),
    [anon_sym_PIPE_EQ] = ACTIONS(1570),
    [anon_sym_LT_LT_EQ] = ACTIONS(1570),
    [anon_sym_GT_GT_EQ] = ACTIONS(1570),
    [anon_sym_EQ] = ACTIONS(1572),
    [anon_sym_EQ_EQ] = ACTIONS(1570),
    [anon_sym_BANG_EQ] = ACTIONS(1570),
    [anon_sym_GT] = ACTIONS(1572),
    [anon_sym_LT] = ACTIONS(1572),
    [anon_sym_GT_EQ] = ACTIONS(1570),
    [anon_sym_LT_EQ] = ACTIONS(1570),
    [anon_sym__] = ACTIONS(1572),
    [anon_sym_DOT] = ACTIONS(1572),
    [anon_sym_DOT_DOT] = ACTIONS(1572),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1570),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1570),
    [anon_sym_COLON_COLON] = ACTIONS(1570),
    [anon_sym_POUND] = ACTIONS(1570),
    [anon_sym_as] = ACTIONS(1572),
    [anon_sym_auto] = ACTIONS(1572),
    [anon_sym_await] = ACTIONS(1572),
    [anon_sym_const] = ACTIONS(1572),
    [anon_sym_default] = ACTIONS(1572),
    [anon_sym_gen] = ACTIONS(1572),
    [anon_sym_raw] = ACTIONS(1572),
    [anon_sym_union] = ACTIONS(1572),
    [anon_sym_safe] = ACTIONS(1572),
    [anon_sym_ref] = ACTIONS(1572),
    [sym_mutable_specifier] = ACTIONS(1572),
    [anon_sym_try] = ACTIONS(1572),
    [anon_sym_box] = ACTIONS(1572),
    [sym_integer_literal] = ACTIONS(1570),
    [aux_sym_string_literal_token1] = ACTIONS(1570),
    [sym_char_literal] = ACTIONS(1570),
    [anon_sym_true] = ACTIONS(1572),
    [anon_sym_false] = ACTIONS(1572),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1572),
    [sym_super] = ACTIONS(1572),
    [sym_crate] = ACTIONS(1572),
    [aux_sym_metavariable_token1] = ACTIONS(1570),
    [sym__raw_string_literal_start] = ACTIONS(1570),
    [sym_float_literal] = ACTIONS(1570),
    [sym__outer_block_doc_comment_start] = ACTIONS(1570),
    [sym__outer_line_doc_comment_start] = ACTIONS(1570),
  },
  [STATE(585)] = {
    [sym_line_comment] = STATE(585),
    [sym_block_comment] = STATE(585),
    [sym_identifier] = ACTIONS(1512),
    [anon_sym_LPAREN] = ACTIONS(1510),
    [anon_sym_RBRACE] = ACTIONS(1506),
    [anon_sym_COMMA] = ACTIONS(1506),
    [anon_sym_macro_rules] = ACTIONS(1512),
    [anon_sym_LBRACK] = ACTIONS(1510),
    [anon_sym_PLUS] = ACTIONS(1508),
    [anon_sym_STAR] = ACTIONS(1508),
    [anon_sym_QMARK] = ACTIONS(1506),
    [anon_sym_u8] = ACTIONS(1512),
    [anon_sym_i8] = ACTIONS(1512),
    [anon_sym_u16] = ACTIONS(1512),
    [anon_sym_i16] = ACTIONS(1512),
    [anon_sym_u32] = ACTIONS(1512),
    [anon_sym_i32] = ACTIONS(1512),
    [anon_sym_u64] = ACTIONS(1512),
    [anon_sym_i64] = ACTIONS(1512),
    [anon_sym_u128] = ACTIONS(1512),
    [anon_sym_i128] = ACTIONS(1512),
    [anon_sym_isize] = ACTIONS(1512),
    [anon_sym_usize] = ACTIONS(1512),
    [anon_sym_f32] = ACTIONS(1512),
    [anon_sym_f64] = ACTIONS(1512),
    [anon_sym_bool] = ACTIONS(1512),
    [anon_sym_str] = ACTIONS(1512),
    [anon_sym_char] = ACTIONS(1512),
    [anon_sym_DASH] = ACTIONS(1512),
    [anon_sym_SLASH] = ACTIONS(1508),
    [anon_sym_PERCENT] = ACTIONS(1508),
    [anon_sym_CARET] = ACTIONS(1508),
    [anon_sym_AMP] = ACTIONS(1512),
    [anon_sym_PIPE] = ACTIONS(1512),
    [anon_sym_AMP_AMP] = ACTIONS(1506),
    [anon_sym_PIPE_PIPE] = ACTIONS(1506),
    [anon_sym_LT_LT] = ACTIONS(1508),
    [anon_sym_GT_GT] = ACTIONS(1508),
    [anon_sym_PLUS_EQ] = ACTIONS(1506),
    [anon_sym_DASH_EQ] = ACTIONS(1506),
    [anon_sym_STAR_EQ] = ACTIONS(1506),
    [anon_sym_SLASH_EQ] = ACTIONS(1506),
    [anon_sym_PERCENT_EQ] = ACTIONS(1506),
    [anon_sym_CARET_EQ] = ACTIONS(1506),
    [anon_sym_AMP_EQ] = ACTIONS(1506),
    [anon_sym_PIPE_EQ] = ACTIONS(1506),
    [anon_sym_LT_LT_EQ] = ACTIONS(1506),
    [anon_sym_GT_GT_EQ] = ACTIONS(1506),
    [anon_sym_EQ] = ACTIONS(1508),
    [anon_sym_EQ_EQ] = ACTIONS(1506),
    [anon_sym_BANG_EQ] = ACTIONS(1506),
    [anon_sym_GT] = ACTIONS(1508),
    [anon_sym_LT] = ACTIONS(1512),
    [anon_sym_GT_EQ] = ACTIONS(1506),
    [anon_sym_LT_EQ] = ACTIONS(1506),
    [anon_sym__] = ACTIONS(1512),
    [anon_sym_DOT] = ACTIONS(1508),
    [anon_sym_DOT_DOT] = ACTIONS(1512),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1506),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1510),
    [anon_sym_COLON_COLON] = ACTIONS(1510),
    [anon_sym_POUND] = ACTIONS(1510),
    [anon_sym_as] = ACTIONS(1508),
    [anon_sym_auto] = ACTIONS(1512),
    [anon_sym_await] = ACTIONS(1512),
    [anon_sym_const] = ACTIONS(1512),
    [anon_sym_default] = ACTIONS(1512),
    [anon_sym_gen] = ACTIONS(1512),
    [anon_sym_raw] = ACTIONS(1512),
    [anon_sym_union] = ACTIONS(1512),
    [anon_sym_safe] = ACTIONS(1512),
    [anon_sym_ref] = ACTIONS(1512),
    [sym_mutable_specifier] = ACTIONS(1512),
    [anon_sym_try] = ACTIONS(1512),
    [anon_sym_box] = ACTIONS(1512),
    [sym_integer_literal] = ACTIONS(1510),
    [aux_sym_string_literal_token1] = ACTIONS(1510),
    [sym_char_literal] = ACTIONS(1510),
    [anon_sym_true] = ACTIONS(1512),
    [anon_sym_false] = ACTIONS(1512),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1512),
    [sym_super] = ACTIONS(1512),
    [sym_crate] = ACTIONS(1512),
    [aux_sym_metavariable_token1] = ACTIONS(1510),
    [sym__raw_string_literal_start] = ACTIONS(1510),
    [sym_float_literal] = ACTIONS(1510),
    [sym__outer_block_doc_comment_start] = ACTIONS(1510),
    [sym__outer_line_doc_comment_start] = ACTIONS(1510),
  },
  [STATE(586)] = {
    [sym_line_comment] = STATE(586),
    [sym_block_comment] = STATE(586),
    [sym_identifier] = ACTIONS(1564),
    [anon_sym_LPAREN] = ACTIONS(1562),
    [anon_sym_RBRACE] = ACTIONS(1562),
    [anon_sym_COMMA] = ACTIONS(1562),
    [anon_sym_macro_rules] = ACTIONS(1564),
    [anon_sym_LBRACK] = ACTIONS(1562),
    [anon_sym_PLUS] = ACTIONS(1564),
    [anon_sym_STAR] = ACTIONS(1564),
    [anon_sym_QMARK] = ACTIONS(1562),
    [anon_sym_u8] = ACTIONS(1564),
    [anon_sym_i8] = ACTIONS(1564),
    [anon_sym_u16] = ACTIONS(1564),
    [anon_sym_i16] = ACTIONS(1564),
    [anon_sym_u32] = ACTIONS(1564),
    [anon_sym_i32] = ACTIONS(1564),
    [anon_sym_u64] = ACTIONS(1564),
    [anon_sym_i64] = ACTIONS(1564),
    [anon_sym_u128] = ACTIONS(1564),
    [anon_sym_i128] = ACTIONS(1564),
    [anon_sym_isize] = ACTIONS(1564),
    [anon_sym_usize] = ACTIONS(1564),
    [anon_sym_f32] = ACTIONS(1564),
    [anon_sym_f64] = ACTIONS(1564),
    [anon_sym_bool] = ACTIONS(1564),
    [anon_sym_str] = ACTIONS(1564),
    [anon_sym_char] = ACTIONS(1564),
    [anon_sym_DASH] = ACTIONS(1564),
    [anon_sym_SLASH] = ACTIONS(1564),
    [anon_sym_PERCENT] = ACTIONS(1564),
    [anon_sym_CARET] = ACTIONS(1564),
    [anon_sym_AMP] = ACTIONS(1564),
    [anon_sym_PIPE] = ACTIONS(1564),
    [anon_sym_AMP_AMP] = ACTIONS(1562),
    [anon_sym_PIPE_PIPE] = ACTIONS(1562),
    [anon_sym_LT_LT] = ACTIONS(1564),
    [anon_sym_GT_GT] = ACTIONS(1564),
    [anon_sym_PLUS_EQ] = ACTIONS(1562),
    [anon_sym_DASH_EQ] = ACTIONS(1562),
    [anon_sym_STAR_EQ] = ACTIONS(1562),
    [anon_sym_SLASH_EQ] = ACTIONS(1562),
    [anon_sym_PERCENT_EQ] = ACTIONS(1562),
    [anon_sym_CARET_EQ] = ACTIONS(1562),
    [anon_sym_AMP_EQ] = ACTIONS(1562),
    [anon_sym_PIPE_EQ] = ACTIONS(1562),
    [anon_sym_LT_LT_EQ] = ACTIONS(1562),
    [anon_sym_GT_GT_EQ] = ACTIONS(1562),
    [anon_sym_EQ] = ACTIONS(1564),
    [anon_sym_EQ_EQ] = ACTIONS(1562),
    [anon_sym_BANG_EQ] = ACTIONS(1562),
    [anon_sym_GT] = ACTIONS(1564),
    [anon_sym_LT] = ACTIONS(1564),
    [anon_sym_GT_EQ] = ACTIONS(1562),
    [anon_sym_LT_EQ] = ACTIONS(1562),
    [anon_sym__] = ACTIONS(1564),
    [anon_sym_DOT] = ACTIONS(1564),
    [anon_sym_DOT_DOT] = ACTIONS(1564),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1562),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1562),
    [anon_sym_COLON_COLON] = ACTIONS(1562),
    [anon_sym_POUND] = ACTIONS(1562),
    [anon_sym_as] = ACTIONS(1564),
    [anon_sym_auto] = ACTIONS(1564),
    [anon_sym_await] = ACTIONS(1564),
    [anon_sym_const] = ACTIONS(1564),
    [anon_sym_default] = ACTIONS(1564),
    [anon_sym_gen] = ACTIONS(1564),
    [anon_sym_raw] = ACTIONS(1564),
    [anon_sym_union] = ACTIONS(1564),
    [anon_sym_safe] = ACTIONS(1564),
    [anon_sym_ref] = ACTIONS(1564),
    [sym_mutable_specifier] = ACTIONS(1564),
    [anon_sym_try] = ACTIONS(1564),
    [anon_sym_box] = ACTIONS(1564),
    [sym_integer_literal] = ACTIONS(1562),
    [aux_sym_string_literal_token1] = ACTIONS(1562),
    [sym_char_literal] = ACTIONS(1562),
    [anon_sym_true] = ACTIONS(1564),
    [anon_sym_false] = ACTIONS(1564),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1564),
    [sym_super] = ACTIONS(1564),
    [sym_crate] = ACTIONS(1564),
    [aux_sym_metavariable_token1] = ACTIONS(1562),
    [sym__raw_string_literal_start] = ACTIONS(1562),
    [sym_float_literal] = ACTIONS(1562),
    [sym__outer_block_doc_comment_start] = ACTIONS(1562),
    [sym__outer_line_doc_comment_start] = ACTIONS(1562),
  },
  [STATE(587)] = {
    [sym_line_comment] = STATE(587),
    [sym_block_comment] = STATE(587),
    [sym_identifier] = ACTIONS(1576),
    [anon_sym_LPAREN] = ACTIONS(1574),
    [anon_sym_RBRACE] = ACTIONS(1574),
    [anon_sym_COMMA] = ACTIONS(1574),
    [anon_sym_macro_rules] = ACTIONS(1576),
    [anon_sym_LBRACK] = ACTIONS(1574),
    [anon_sym_PLUS] = ACTIONS(1576),
    [anon_sym_STAR] = ACTIONS(1576),
    [anon_sym_QMARK] = ACTIONS(1574),
    [anon_sym_u8] = ACTIONS(1576),
    [anon_sym_i8] = ACTIONS(1576),
    [anon_sym_u16] = ACTIONS(1576),
    [anon_sym_i16] = ACTIONS(1576),
    [anon_sym_u32] = ACTIONS(1576),
    [anon_sym_i32] = ACTIONS(1576),
    [anon_sym_u64] = ACTIONS(1576),
    [anon_sym_i64] = ACTIONS(1576),
    [anon_sym_u128] = ACTIONS(1576),
    [anon_sym_i128] = ACTIONS(1576),
    [anon_sym_isize] = ACTIONS(1576),
    [anon_sym_usize] = ACTIONS(1576),
    [anon_sym_f32] = ACTIONS(1576),
    [anon_sym_f64] = ACTIONS(1576),
    [anon_sym_bool] = ACTIONS(1576),
    [anon_sym_str] = ACTIONS(1576),
    [anon_sym_char] = ACTIONS(1576),
    [anon_sym_DASH] = ACTIONS(1576),
    [anon_sym_SLASH] = ACTIONS(1576),
    [anon_sym_PERCENT] = ACTIONS(1576),
    [anon_sym_CARET] = ACTIONS(1576),
    [anon_sym_AMP] = ACTIONS(1576),
    [anon_sym_PIPE] = ACTIONS(1576),
    [anon_sym_AMP_AMP] = ACTIONS(1574),
    [anon_sym_PIPE_PIPE] = ACTIONS(1574),
    [anon_sym_LT_LT] = ACTIONS(1576),
    [anon_sym_GT_GT] = ACTIONS(1576),
    [anon_sym_PLUS_EQ] = ACTIONS(1574),
    [anon_sym_DASH_EQ] = ACTIONS(1574),
    [anon_sym_STAR_EQ] = ACTIONS(1574),
    [anon_sym_SLASH_EQ] = ACTIONS(1574),
    [anon_sym_PERCENT_EQ] = ACTIONS(1574),
    [anon_sym_CARET_EQ] = ACTIONS(1574),
    [anon_sym_AMP_EQ] = ACTIONS(1574),
    [anon_sym_PIPE_EQ] = ACTIONS(1574),
    [anon_sym_LT_LT_EQ] = ACTIONS(1574),
    [anon_sym_GT_GT_EQ] = ACTIONS(1574),
    [anon_sym_EQ] = ACTIONS(1576),
    [anon_sym_EQ_EQ] = ACTIONS(1574),
    [anon_sym_BANG_EQ] = ACTIONS(1574),
    [anon_sym_GT] = ACTIONS(1576),
    [anon_sym_LT] = ACTIONS(1576),
    [anon_sym_GT_EQ] = ACTIONS(1574),
    [anon_sym_LT_EQ] = ACTIONS(1574),
    [anon_sym__] = ACTIONS(1576),
    [anon_sym_DOT] = ACTIONS(1576),
    [anon_sym_DOT_DOT] = ACTIONS(1576),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1574),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1574),
    [anon_sym_COLON_COLON] = ACTIONS(1574),
    [anon_sym_POUND] = ACTIONS(1574),
    [anon_sym_as] = ACTIONS(1576),
    [anon_sym_auto] = ACTIONS(1576),
    [anon_sym_await] = ACTIONS(1576),
    [anon_sym_const] = ACTIONS(1576),
    [anon_sym_default] = ACTIONS(1576),
    [anon_sym_gen] = ACTIONS(1576),
    [anon_sym_raw] = ACTIONS(1576),
    [anon_sym_union] = ACTIONS(1576),
    [anon_sym_safe] = ACTIONS(1576),
    [anon_sym_ref] = ACTIONS(1576),
    [sym_mutable_specifier] = ACTIONS(1576),
    [anon_sym_try] = ACTIONS(1576),
    [anon_sym_box] = ACTIONS(1576),
    [sym_integer_literal] = ACTIONS(1574),
    [aux_sym_string_literal_token1] = ACTIONS(1574),
    [sym_char_literal] = ACTIONS(1574),
    [anon_sym_true] = ACTIONS(1576),
    [anon_sym_false] = ACTIONS(1576),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1576),
    [sym_super] = ACTIONS(1576),
    [sym_crate] = ACTIONS(1576),
    [aux_sym_metavariable_token1] = ACTIONS(1574),
    [sym__raw_string_literal_start] = ACTIONS(1574),
    [sym_float_literal] = ACTIONS(1574),
    [sym__outer_block_doc_comment_start] = ACTIONS(1574),
    [sym__outer_line_doc_comment_start] = ACTIONS(1574),
  },
  [STATE(588)] = {
    [sym_line_comment] = STATE(588),
    [sym_block_comment] = STATE(588),
    [sym_identifier] = ACTIONS(1568),
    [anon_sym_LPAREN] = ACTIONS(1566),
    [anon_sym_RBRACE] = ACTIONS(1566),
    [anon_sym_COMMA] = ACTIONS(1566),
    [anon_sym_macro_rules] = ACTIONS(1568),
    [anon_sym_LBRACK] = ACTIONS(1566),
    [anon_sym_PLUS] = ACTIONS(1568),
    [anon_sym_STAR] = ACTIONS(1568),
    [anon_sym_QMARK] = ACTIONS(1566),
    [anon_sym_u8] = ACTIONS(1568),
    [anon_sym_i8] = ACTIONS(1568),
    [anon_sym_u16] = ACTIONS(1568),
    [anon_sym_i16] = ACTIONS(1568),
    [anon_sym_u32] = ACTIONS(1568),
    [anon_sym_i32] = ACTIONS(1568),
    [anon_sym_u64] = ACTIONS(1568),
    [anon_sym_i64] = ACTIONS(1568),
    [anon_sym_u128] = ACTIONS(1568),
    [anon_sym_i128] = ACTIONS(1568),
    [anon_sym_isize] = ACTIONS(1568),
    [anon_sym_usize] = ACTIONS(1568),
    [anon_sym_f32] = ACTIONS(1568),
    [anon_sym_f64] = ACTIONS(1568),
    [anon_sym_bool] = ACTIONS(1568),
    [anon_sym_str] = ACTIONS(1568),
    [anon_sym_char] = ACTIONS(1568),
    [anon_sym_DASH] = ACTIONS(1568),
    [anon_sym_SLASH] = ACTIONS(1568),
    [anon_sym_PERCENT] = ACTIONS(1568),
    [anon_sym_CARET] = ACTIONS(1568),
    [anon_sym_AMP] = ACTIONS(1568),
    [anon_sym_PIPE] = ACTIONS(1568),
    [anon_sym_AMP_AMP] = ACTIONS(1566),
    [anon_sym_PIPE_PIPE] = ACTIONS(1566),
    [anon_sym_LT_LT] = ACTIONS(1568),
    [anon_sym_GT_GT] = ACTIONS(1568),
    [anon_sym_PLUS_EQ] = ACTIONS(1566),
    [anon_sym_DASH_EQ] = ACTIONS(1566),
    [anon_sym_STAR_EQ] = ACTIONS(1566),
    [anon_sym_SLASH_EQ] = ACTIONS(1566),
    [anon_sym_PERCENT_EQ] = ACTIONS(1566),
    [anon_sym_CARET_EQ] = ACTIONS(1566),
    [anon_sym_AMP_EQ] = ACTIONS(1566),
    [anon_sym_PIPE_EQ] = ACTIONS(1566),
    [anon_sym_LT_LT_EQ] = ACTIONS(1566),
    [anon_sym_GT_GT_EQ] = ACTIONS(1566),
    [anon_sym_EQ] = ACTIONS(1568),
    [anon_sym_EQ_EQ] = ACTIONS(1566),
    [anon_sym_BANG_EQ] = ACTIONS(1566),
    [anon_sym_GT] = ACTIONS(1568),
    [anon_sym_LT] = ACTIONS(1568),
    [anon_sym_GT_EQ] = ACTIONS(1566),
    [anon_sym_LT_EQ] = ACTIONS(1566),
    [anon_sym__] = ACTIONS(1568),
    [anon_sym_DOT] = ACTIONS(1568),
    [anon_sym_DOT_DOT] = ACTIONS(1568),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1566),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1566),
    [anon_sym_COLON_COLON] = ACTIONS(1566),
    [anon_sym_POUND] = ACTIONS(1566),
    [anon_sym_as] = ACTIONS(1568),
    [anon_sym_auto] = ACTIONS(1568),
    [anon_sym_await] = ACTIONS(1568),
    [anon_sym_const] = ACTIONS(1568),
    [anon_sym_default] = ACTIONS(1568),
    [anon_sym_gen] = ACTIONS(1568),
    [anon_sym_raw] = ACTIONS(1568),
    [anon_sym_union] = ACTIONS(1568),
    [anon_sym_safe] = ACTIONS(1568),
    [anon_sym_ref] = ACTIONS(1568),
    [sym_mutable_specifier] = ACTIONS(1568),
    [anon_sym_try] = ACTIONS(1568),
    [anon_sym_box] = ACTIONS(1568),
    [sym_integer_literal] = ACTIONS(1566),
    [aux_sym_string_literal_token1] = ACTIONS(1566),
    [sym_char_literal] = ACTIONS(1566),
    [anon_sym_true] = ACTIONS(1568),
    [anon_sym_false] = ACTIONS(1568),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1568),
    [sym_super] = ACTIONS(1568),
    [sym_crate] = ACTIONS(1568),
    [aux_sym_metavariable_token1] = ACTIONS(1566),
    [sym__raw_string_literal_start] = ACTIONS(1566),
    [sym_float_literal] = ACTIONS(1566),
    [sym__outer_block_doc_comment_start] = ACTIONS(1566),
    [sym__outer_line_doc_comment_start] = ACTIONS(1566),
  },
  [STATE(589)] = {
    [sym_line_comment] = STATE(589),
    [sym_block_comment] = STATE(589),
    [sym_identifier] = ACTIONS(1584),
    [anon_sym_LPAREN] = ACTIONS(1582),
    [anon_sym_RBRACE] = ACTIONS(1582),
    [anon_sym_COMMA] = ACTIONS(1582),
    [anon_sym_macro_rules] = ACTIONS(1584),
    [anon_sym_LBRACK] = ACTIONS(1582),
    [anon_sym_PLUS] = ACTIONS(1584),
    [anon_sym_STAR] = ACTIONS(1584),
    [anon_sym_QMARK] = ACTIONS(1582),
    [anon_sym_u8] = ACTIONS(1584),
    [anon_sym_i8] = ACTIONS(1584),
    [anon_sym_u16] = ACTIONS(1584),
    [anon_sym_i16] = ACTIONS(1584),
    [anon_sym_u32] = ACTIONS(1584),
    [anon_sym_i32] = ACTIONS(1584),
    [anon_sym_u64] = ACTIONS(1584),
    [anon_sym_i64] = ACTIONS(1584),
    [anon_sym_u128] = ACTIONS(1584),
    [anon_sym_i128] = ACTIONS(1584),
    [anon_sym_isize] = ACTIONS(1584),
    [anon_sym_usize] = ACTIONS(1584),
    [anon_sym_f32] = ACTIONS(1584),
    [anon_sym_f64] = ACTIONS(1584),
    [anon_sym_bool] = ACTIONS(1584),
    [anon_sym_str] = ACTIONS(1584),
    [anon_sym_char] = ACTIONS(1584),
    [anon_sym_DASH] = ACTIONS(1584),
    [anon_sym_SLASH] = ACTIONS(1584),
    [anon_sym_PERCENT] = ACTIONS(1584),
    [anon_sym_CARET] = ACTIONS(1584),
    [anon_sym_AMP] = ACTIONS(1584),
    [anon_sym_PIPE] = ACTIONS(1584),
    [anon_sym_AMP_AMP] = ACTIONS(1582),
    [anon_sym_PIPE_PIPE] = ACTIONS(1582),
    [anon_sym_LT_LT] = ACTIONS(1584),
    [anon_sym_GT_GT] = ACTIONS(1584),
    [anon_sym_PLUS_EQ] = ACTIONS(1582),
    [anon_sym_DASH_EQ] = ACTIONS(1582),
    [anon_sym_STAR_EQ] = ACTIONS(1582),
    [anon_sym_SLASH_EQ] = ACTIONS(1582),
    [anon_sym_PERCENT_EQ] = ACTIONS(1582),
    [anon_sym_CARET_EQ] = ACTIONS(1582),
    [anon_sym_AMP_EQ] = ACTIONS(1582),
    [anon_sym_PIPE_EQ] = ACTIONS(1582),
    [anon_sym_LT_LT_EQ] = ACTIONS(1582),
    [anon_sym_GT_GT_EQ] = ACTIONS(1582),
    [anon_sym_EQ] = ACTIONS(1584),
    [anon_sym_EQ_EQ] = ACTIONS(1582),
    [anon_sym_BANG_EQ] = ACTIONS(1582),
    [anon_sym_GT] = ACTIONS(1584),
    [anon_sym_LT] = ACTIONS(1584),
    [anon_sym_GT_EQ] = ACTIONS(1582),
    [anon_sym_LT_EQ] = ACTIONS(1582),
    [anon_sym__] = ACTIONS(1584),
    [anon_sym_DOT] = ACTIONS(1584),
    [anon_sym_DOT_DOT] = ACTIONS(1584),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1582),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1582),
    [anon_sym_COLON_COLON] = ACTIONS(1582),
    [anon_sym_POUND] = ACTIONS(1582),
    [anon_sym_as] = ACTIONS(1584),
    [anon_sym_auto] = ACTIONS(1584),
    [anon_sym_await] = ACTIONS(1584),
    [anon_sym_const] = ACTIONS(1584),
    [anon_sym_default] = ACTIONS(1584),
    [anon_sym_gen] = ACTIONS(1584),
    [anon_sym_raw] = ACTIONS(1584),
    [anon_sym_union] = ACTIONS(1584),
    [anon_sym_safe] = ACTIONS(1584),
    [anon_sym_ref] = ACTIONS(1584),
    [sym_mutable_specifier] = ACTIONS(1584),
    [anon_sym_try] = ACTIONS(1584),
    [anon_sym_box] = ACTIONS(1584),
    [sym_integer_literal] = ACTIONS(1582),
    [aux_sym_string_literal_token1] = ACTIONS(1582),
    [sym_char_literal] = ACTIONS(1582),
    [anon_sym_true] = ACTIONS(1584),
    [anon_sym_false] = ACTIONS(1584),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1584),
    [sym_super] = ACTIONS(1584),
    [sym_crate] = ACTIONS(1584),
    [aux_sym_metavariable_token1] = ACTIONS(1582),
    [sym__raw_string_literal_start] = ACTIONS(1582),
    [sym_float_literal] = ACTIONS(1582),
    [sym__outer_block_doc_comment_start] = ACTIONS(1582),
    [sym__outer_line_doc_comment_start] = ACTIONS(1582),
  },
  [STATE(590)] = {
    [sym_line_comment] = STATE(590),
    [sym_block_comment] = STATE(590),
    [sym_identifier] = ACTIONS(1560),
    [anon_sym_LPAREN] = ACTIONS(1558),
    [anon_sym_RBRACE] = ACTIONS(1558),
    [anon_sym_COMMA] = ACTIONS(1558),
    [anon_sym_macro_rules] = ACTIONS(1560),
    [anon_sym_LBRACK] = ACTIONS(1558),
    [anon_sym_PLUS] = ACTIONS(1560),
    [anon_sym_STAR] = ACTIONS(1560),
    [anon_sym_QMARK] = ACTIONS(1558),
    [anon_sym_u8] = ACTIONS(1560),
    [anon_sym_i8] = ACTIONS(1560),
    [anon_sym_u16] = ACTIONS(1560),
    [anon_sym_i16] = ACTIONS(1560),
    [anon_sym_u32] = ACTIONS(1560),
    [anon_sym_i32] = ACTIONS(1560),
    [anon_sym_u64] = ACTIONS(1560),
    [anon_sym_i64] = ACTIONS(1560),
    [anon_sym_u128] = ACTIONS(1560),
    [anon_sym_i128] = ACTIONS(1560),
    [anon_sym_isize] = ACTIONS(1560),
    [anon_sym_usize] = ACTIONS(1560),
    [anon_sym_f32] = ACTIONS(1560),
    [anon_sym_f64] = ACTIONS(1560),
    [anon_sym_bool] = ACTIONS(1560),
    [anon_sym_str] = ACTIONS(1560),
    [anon_sym_char] = ACTIONS(1560),
    [anon_sym_DASH] = ACTIONS(1560),
    [anon_sym_SLASH] = ACTIONS(1560),
    [anon_sym_PERCENT] = ACTIONS(1560),
    [anon_sym_CARET] = ACTIONS(1560),
    [anon_sym_AMP] = ACTIONS(1560),
    [anon_sym_PIPE] = ACTIONS(1560),
    [anon_sym_AMP_AMP] = ACTIONS(1558),
    [anon_sym_PIPE_PIPE] = ACTIONS(1558),
    [anon_sym_LT_LT] = ACTIONS(1560),
    [anon_sym_GT_GT] = ACTIONS(1560),
    [anon_sym_PLUS_EQ] = ACTIONS(1558),
    [anon_sym_DASH_EQ] = ACTIONS(1558),
    [anon_sym_STAR_EQ] = ACTIONS(1558),
    [anon_sym_SLASH_EQ] = ACTIONS(1558),
    [anon_sym_PERCENT_EQ] = ACTIONS(1558),
    [anon_sym_CARET_EQ] = ACTIONS(1558),
    [anon_sym_AMP_EQ] = ACTIONS(1558),
    [anon_sym_PIPE_EQ] = ACTIONS(1558),
    [anon_sym_LT_LT_EQ] = ACTIONS(1558),
    [anon_sym_GT_GT_EQ] = ACTIONS(1558),
    [anon_sym_EQ] = ACTIONS(1560),
    [anon_sym_EQ_EQ] = ACTIONS(1558),
    [anon_sym_BANG_EQ] = ACTIONS(1558),
    [anon_sym_GT] = ACTIONS(1560),
    [anon_sym_LT] = ACTIONS(1560),
    [anon_sym_GT_EQ] = ACTIONS(1558),
    [anon_sym_LT_EQ] = ACTIONS(1558),
    [anon_sym__] = ACTIONS(1560),
    [anon_sym_DOT] = ACTIONS(1560),
    [anon_sym_DOT_DOT] = ACTIONS(1560),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1558),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1558),
    [anon_sym_COLON_COLON] = ACTIONS(1558),
    [anon_sym_POUND] = ACTIONS(1558),
    [anon_sym_as] = ACTIONS(1560),
    [anon_sym_auto] = ACTIONS(1560),
    [anon_sym_await] = ACTIONS(1560),
    [anon_sym_const] = ACTIONS(1560),
    [anon_sym_default] = ACTIONS(1560),
    [anon_sym_gen] = ACTIONS(1560),
    [anon_sym_raw] = ACTIONS(1560),
    [anon_sym_union] = ACTIONS(1560),
    [anon_sym_safe] = ACTIONS(1560),
    [anon_sym_ref] = ACTIONS(1560),
    [sym_mutable_specifier] = ACTIONS(1560),
    [anon_sym_try] = ACTIONS(1560),
    [anon_sym_box] = ACTIONS(1560),
    [sym_integer_literal] = ACTIONS(1558),
    [aux_sym_string_literal_token1] = ACTIONS(1558),
    [sym_char_literal] = ACTIONS(1558),
    [anon_sym_true] = ACTIONS(1560),
    [anon_sym_false] = ACTIONS(1560),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1560),
    [sym_super] = ACTIONS(1560),
    [sym_crate] = ACTIONS(1560),
    [aux_sym_metavariable_token1] = ACTIONS(1558),
    [sym__raw_string_literal_start] = ACTIONS(1558),
    [sym_float_literal] = ACTIONS(1558),
    [sym__outer_block_doc_comment_start] = ACTIONS(1558),
    [sym__outer_line_doc_comment_start] = ACTIONS(1558),
  },
  [STATE(591)] = {
    [sym_line_comment] = STATE(591),
    [sym_block_comment] = STATE(591),
    [sym_identifier] = ACTIONS(1500),
    [anon_sym_LPAREN] = ACTIONS(1498),
    [anon_sym_RBRACE] = ACTIONS(1498),
    [anon_sym_COMMA] = ACTIONS(1498),
    [anon_sym_macro_rules] = ACTIONS(1500),
    [anon_sym_LBRACK] = ACTIONS(1498),
    [anon_sym_PLUS] = ACTIONS(1500),
    [anon_sym_STAR] = ACTIONS(1500),
    [anon_sym_QMARK] = ACTIONS(1498),
    [anon_sym_u8] = ACTIONS(1500),
    [anon_sym_i8] = ACTIONS(1500),
    [anon_sym_u16] = ACTIONS(1500),
    [anon_sym_i16] = ACTIONS(1500),
    [anon_sym_u32] = ACTIONS(1500),
    [anon_sym_i32] = ACTIONS(1500),
    [anon_sym_u64] = ACTIONS(1500),
    [anon_sym_i64] = ACTIONS(1500),
    [anon_sym_u128] = ACTIONS(1500),
    [anon_sym_i128] = ACTIONS(1500),
    [anon_sym_isize] = ACTIONS(1500),
    [anon_sym_usize] = ACTIONS(1500),
    [anon_sym_f32] = ACTIONS(1500),
    [anon_sym_f64] = ACTIONS(1500),
    [anon_sym_bool] = ACTIONS(1500),
    [anon_sym_str] = ACTIONS(1500),
    [anon_sym_char] = ACTIONS(1500),
    [anon_sym_DASH] = ACTIONS(1500),
    [anon_sym_SLASH] = ACTIONS(1500),
    [anon_sym_PERCENT] = ACTIONS(1500),
    [anon_sym_CARET] = ACTIONS(1500),
    [anon_sym_AMP] = ACTIONS(1500),
    [anon_sym_PIPE] = ACTIONS(1500),
    [anon_sym_AMP_AMP] = ACTIONS(1498),
    [anon_sym_PIPE_PIPE] = ACTIONS(1498),
    [anon_sym_LT_LT] = ACTIONS(1500),
    [anon_sym_GT_GT] = ACTIONS(1500),
    [anon_sym_PLUS_EQ] = ACTIONS(1498),
    [anon_sym_DASH_EQ] = ACTIONS(1498),
    [anon_sym_STAR_EQ] = ACTIONS(1498),
    [anon_sym_SLASH_EQ] = ACTIONS(1498),
    [anon_sym_PERCENT_EQ] = ACTIONS(1498),
    [anon_sym_CARET_EQ] = ACTIONS(1498),
    [anon_sym_AMP_EQ] = ACTIONS(1498),
    [anon_sym_PIPE_EQ] = ACTIONS(1498),
    [anon_sym_LT_LT_EQ] = ACTIONS(1498),
    [anon_sym_GT_GT_EQ] = ACTIONS(1498),
    [anon_sym_EQ] = ACTIONS(1500),
    [anon_sym_EQ_EQ] = ACTIONS(1498),
    [anon_sym_BANG_EQ] = ACTIONS(1498),
    [anon_sym_GT] = ACTIONS(1500),
    [anon_sym_LT] = ACTIONS(1500),
    [anon_sym_GT_EQ] = ACTIONS(1498),
    [anon_sym_LT_EQ] = ACTIONS(1498),
    [anon_sym__] = ACTIONS(1500),
    [anon_sym_DOT] = ACTIONS(1500),
    [anon_sym_DOT_DOT] = ACTIONS(1500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1498),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1498),
    [anon_sym_COLON_COLON] = ACTIONS(1498),
    [anon_sym_POUND] = ACTIONS(1498),
    [anon_sym_as] = ACTIONS(1500),
    [anon_sym_auto] = ACTIONS(1500),
    [anon_sym_await] = ACTIONS(1500),
    [anon_sym_const] = ACTIONS(1500),
    [anon_sym_default] = ACTIONS(1500),
    [anon_sym_gen] = ACTIONS(1500),
    [anon_sym_raw] = ACTIONS(1500),
    [anon_sym_union] = ACTIONS(1500),
    [anon_sym_safe] = ACTIONS(1500),
    [anon_sym_ref] = ACTIONS(1500),
    [sym_mutable_specifier] = ACTIONS(1500),
    [anon_sym_try] = ACTIONS(1500),
    [anon_sym_box] = ACTIONS(1500),
    [sym_integer_literal] = ACTIONS(1498),
    [aux_sym_string_literal_token1] = ACTIONS(1498),
    [sym_char_literal] = ACTIONS(1498),
    [anon_sym_true] = ACTIONS(1500),
    [anon_sym_false] = ACTIONS(1500),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1500),
    [sym_super] = ACTIONS(1500),
    [sym_crate] = ACTIONS(1500),
    [aux_sym_metavariable_token1] = ACTIONS(1498),
    [sym__raw_string_literal_start] = ACTIONS(1498),
    [sym_float_literal] = ACTIONS(1498),
    [sym__outer_block_doc_comment_start] = ACTIONS(1498),
    [sym__outer_line_doc_comment_start] = ACTIONS(1498),
  },
  [STATE(592)] = {
    [sym_line_comment] = STATE(592),
    [sym_block_comment] = STATE(592),
    [sym_identifier] = ACTIONS(1596),
    [anon_sym_LPAREN] = ACTIONS(1594),
    [anon_sym_RBRACE] = ACTIONS(1594),
    [anon_sym_COMMA] = ACTIONS(1594),
    [anon_sym_macro_rules] = ACTIONS(1596),
    [anon_sym_LBRACK] = ACTIONS(1594),
    [anon_sym_PLUS] = ACTIONS(1596),
    [anon_sym_STAR] = ACTIONS(1596),
    [anon_sym_QMARK] = ACTIONS(1594),
    [anon_sym_u8] = ACTIONS(1596),
    [anon_sym_i8] = ACTIONS(1596),
    [anon_sym_u16] = ACTIONS(1596),
    [anon_sym_i16] = ACTIONS(1596),
    [anon_sym_u32] = ACTIONS(1596),
    [anon_sym_i32] = ACTIONS(1596),
    [anon_sym_u64] = ACTIONS(1596),
    [anon_sym_i64] = ACTIONS(1596),
    [anon_sym_u128] = ACTIONS(1596),
    [anon_sym_i128] = ACTIONS(1596),
    [anon_sym_isize] = ACTIONS(1596),
    [anon_sym_usize] = ACTIONS(1596),
    [anon_sym_f32] = ACTIONS(1596),
    [anon_sym_f64] = ACTIONS(1596),
    [anon_sym_bool] = ACTIONS(1596),
    [anon_sym_str] = ACTIONS(1596),
    [anon_sym_char] = ACTIONS(1596),
    [anon_sym_DASH] = ACTIONS(1596),
    [anon_sym_SLASH] = ACTIONS(1596),
    [anon_sym_PERCENT] = ACTIONS(1596),
    [anon_sym_CARET] = ACTIONS(1596),
    [anon_sym_AMP] = ACTIONS(1596),
    [anon_sym_PIPE] = ACTIONS(1596),
    [anon_sym_AMP_AMP] = ACTIONS(1594),
    [anon_sym_PIPE_PIPE] = ACTIONS(1594),
    [anon_sym_LT_LT] = ACTIONS(1596),
    [anon_sym_GT_GT] = ACTIONS(1596),
    [anon_sym_PLUS_EQ] = ACTIONS(1594),
    [anon_sym_DASH_EQ] = ACTIONS(1594),
    [anon_sym_STAR_EQ] = ACTIONS(1594),
    [anon_sym_SLASH_EQ] = ACTIONS(1594),
    [anon_sym_PERCENT_EQ] = ACTIONS(1594),
    [anon_sym_CARET_EQ] = ACTIONS(1594),
    [anon_sym_AMP_EQ] = ACTIONS(1594),
    [anon_sym_PIPE_EQ] = ACTIONS(1594),
    [anon_sym_LT_LT_EQ] = ACTIONS(1594),
    [anon_sym_GT_GT_EQ] = ACTIONS(1594),
    [anon_sym_EQ] = ACTIONS(1596),
    [anon_sym_EQ_EQ] = ACTIONS(1594),
    [anon_sym_BANG_EQ] = ACTIONS(1594),
    [anon_sym_GT] = ACTIONS(1596),
    [anon_sym_LT] = ACTIONS(1596),
    [anon_sym_GT_EQ] = ACTIONS(1594),
    [anon_sym_LT_EQ] = ACTIONS(1594),
    [anon_sym__] = ACTIONS(1596),
    [anon_sym_DOT] = ACTIONS(1596),
    [anon_sym_DOT_DOT] = ACTIONS(1596),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1594),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1594),
    [anon_sym_COLON_COLON] = ACTIONS(1594),
    [anon_sym_POUND] = ACTIONS(1594),
    [anon_sym_as] = ACTIONS(1596),
    [anon_sym_auto] = ACTIONS(1596),
    [anon_sym_await] = ACTIONS(1596),
    [anon_sym_const] = ACTIONS(1596),
    [anon_sym_default] = ACTIONS(1596),
    [anon_sym_gen] = ACTIONS(1596),
    [anon_sym_raw] = ACTIONS(1596),
    [anon_sym_union] = ACTIONS(1596),
    [anon_sym_safe] = ACTIONS(1596),
    [anon_sym_ref] = ACTIONS(1596),
    [sym_mutable_specifier] = ACTIONS(1596),
    [anon_sym_try] = ACTIONS(1596),
    [anon_sym_box] = ACTIONS(1596),
    [sym_integer_literal] = ACTIONS(1594),
    [aux_sym_string_literal_token1] = ACTIONS(1594),
    [sym_char_literal] = ACTIONS(1594),
    [anon_sym_true] = ACTIONS(1596),
    [anon_sym_false] = ACTIONS(1596),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1596),
    [sym_super] = ACTIONS(1596),
    [sym_crate] = ACTIONS(1596),
    [aux_sym_metavariable_token1] = ACTIONS(1594),
    [sym__raw_string_literal_start] = ACTIONS(1594),
    [sym_float_literal] = ACTIONS(1594),
    [sym__outer_block_doc_comment_start] = ACTIONS(1594),
    [sym__outer_line_doc_comment_start] = ACTIONS(1594),
  },
  [STATE(593)] = {
    [sym_line_comment] = STATE(593),
    [sym_block_comment] = STATE(593),
    [sym_identifier] = ACTIONS(1580),
    [anon_sym_LPAREN] = ACTIONS(1578),
    [anon_sym_RBRACE] = ACTIONS(1578),
    [anon_sym_COMMA] = ACTIONS(1578),
    [anon_sym_macro_rules] = ACTIONS(1580),
    [anon_sym_LBRACK] = ACTIONS(1578),
    [anon_sym_PLUS] = ACTIONS(1580),
    [anon_sym_STAR] = ACTIONS(1580),
    [anon_sym_QMARK] = ACTIONS(1578),
    [anon_sym_u8] = ACTIONS(1580),
    [anon_sym_i8] = ACTIONS(1580),
    [anon_sym_u16] = ACTIONS(1580),
    [anon_sym_i16] = ACTIONS(1580),
    [anon_sym_u32] = ACTIONS(1580),
    [anon_sym_i32] = ACTIONS(1580),
    [anon_sym_u64] = ACTIONS(1580),
    [anon_sym_i64] = ACTIONS(1580),
    [anon_sym_u128] = ACTIONS(1580),
    [anon_sym_i128] = ACTIONS(1580),
    [anon_sym_isize] = ACTIONS(1580),
    [anon_sym_usize] = ACTIONS(1580),
    [anon_sym_f32] = ACTIONS(1580),
    [anon_sym_f64] = ACTIONS(1580),
    [anon_sym_bool] = ACTIONS(1580),
    [anon_sym_str] = ACTIONS(1580),
    [anon_sym_char] = ACTIONS(1580),
    [anon_sym_DASH] = ACTIONS(1580),
    [anon_sym_SLASH] = ACTIONS(1580),
    [anon_sym_PERCENT] = ACTIONS(1580),
    [anon_sym_CARET] = ACTIONS(1580),
    [anon_sym_AMP] = ACTIONS(1580),
    [anon_sym_PIPE] = ACTIONS(1580),
    [anon_sym_AMP_AMP] = ACTIONS(1578),
    [anon_sym_PIPE_PIPE] = ACTIONS(1578),
    [anon_sym_LT_LT] = ACTIONS(1580),
    [anon_sym_GT_GT] = ACTIONS(1580),
    [anon_sym_PLUS_EQ] = ACTIONS(1578),
    [anon_sym_DASH_EQ] = ACTIONS(1578),
    [anon_sym_STAR_EQ] = ACTIONS(1578),
    [anon_sym_SLASH_EQ] = ACTIONS(1578),
    [anon_sym_PERCENT_EQ] = ACTIONS(1578),
    [anon_sym_CARET_EQ] = ACTIONS(1578),
    [anon_sym_AMP_EQ] = ACTIONS(1578),
    [anon_sym_PIPE_EQ] = ACTIONS(1578),
    [anon_sym_LT_LT_EQ] = ACTIONS(1578),
    [anon_sym_GT_GT_EQ] = ACTIONS(1578),
    [anon_sym_EQ] = ACTIONS(1580),
    [anon_sym_EQ_EQ] = ACTIONS(1578),
    [anon_sym_BANG_EQ] = ACTIONS(1578),
    [anon_sym_GT] = ACTIONS(1580),
    [anon_sym_LT] = ACTIONS(1580),
    [anon_sym_GT_EQ] = ACTIONS(1578),
    [anon_sym_LT_EQ] = ACTIONS(1578),
    [anon_sym__] = ACTIONS(1580),
    [anon_sym_DOT] = ACTIONS(1580),
    [anon_sym_DOT_DOT] = ACTIONS(1580),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1578),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1578),
    [anon_sym_COLON_COLON] = ACTIONS(1578),
    [anon_sym_POUND] = ACTIONS(1578),
    [anon_sym_as] = ACTIONS(1580),
    [anon_sym_auto] = ACTIONS(1580),
    [anon_sym_await] = ACTIONS(1580),
    [anon_sym_const] = ACTIONS(1580),
    [anon_sym_default] = ACTIONS(1580),
    [anon_sym_gen] = ACTIONS(1580),
    [anon_sym_raw] = ACTIONS(1580),
    [anon_sym_union] = ACTIONS(1580),
    [anon_sym_safe] = ACTIONS(1580),
    [anon_sym_ref] = ACTIONS(1580),
    [sym_mutable_specifier] = ACTIONS(1580),
    [anon_sym_try] = ACTIONS(1580),
    [anon_sym_box] = ACTIONS(1580),
    [sym_integer_literal] = ACTIONS(1578),
    [aux_sym_string_literal_token1] = ACTIONS(1578),
    [sym_char_literal] = ACTIONS(1578),
    [anon_sym_true] = ACTIONS(1580),
    [anon_sym_false] = ACTIONS(1580),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1580),
    [sym_super] = ACTIONS(1580),
    [sym_crate] = ACTIONS(1580),
    [aux_sym_metavariable_token1] = ACTIONS(1578),
    [sym__raw_string_literal_start] = ACTIONS(1578),
    [sym_float_literal] = ACTIONS(1578),
    [sym__outer_block_doc_comment_start] = ACTIONS(1578),
    [sym__outer_line_doc_comment_start] = ACTIONS(1578),
  },
  [STATE(594)] = {
    [sym_line_comment] = STATE(594),
    [sym_block_comment] = STATE(594),
    [sym_identifier] = ACTIONS(1616),
    [anon_sym_LPAREN] = ACTIONS(1614),
    [anon_sym_RBRACE] = ACTIONS(1614),
    [anon_sym_COMMA] = ACTIONS(1614),
    [anon_sym_macro_rules] = ACTIONS(1616),
    [anon_sym_LBRACK] = ACTIONS(1614),
    [anon_sym_PLUS] = ACTIONS(1616),
    [anon_sym_STAR] = ACTIONS(1616),
    [anon_sym_QMARK] = ACTIONS(1614),
    [anon_sym_u8] = ACTIONS(1616),
    [anon_sym_i8] = ACTIONS(1616),
    [anon_sym_u16] = ACTIONS(1616),
    [anon_sym_i16] = ACTIONS(1616),
    [anon_sym_u32] = ACTIONS(1616),
    [anon_sym_i32] = ACTIONS(1616),
    [anon_sym_u64] = ACTIONS(1616),
    [anon_sym_i64] = ACTIONS(1616),
    [anon_sym_u128] = ACTIONS(1616),
    [anon_sym_i128] = ACTIONS(1616),
    [anon_sym_isize] = ACTIONS(1616),
    [anon_sym_usize] = ACTIONS(1616),
    [anon_sym_f32] = ACTIONS(1616),
    [anon_sym_f64] = ACTIONS(1616),
    [anon_sym_bool] = ACTIONS(1616),
    [anon_sym_str] = ACTIONS(1616),
    [anon_sym_char] = ACTIONS(1616),
    [anon_sym_DASH] = ACTIONS(1616),
    [anon_sym_SLASH] = ACTIONS(1616),
    [anon_sym_PERCENT] = ACTIONS(1616),
    [anon_sym_CARET] = ACTIONS(1616),
    [anon_sym_AMP] = ACTIONS(1616),
    [anon_sym_PIPE] = ACTIONS(1616),
    [anon_sym_AMP_AMP] = ACTIONS(1614),
    [anon_sym_PIPE_PIPE] = ACTIONS(1614),
    [anon_sym_LT_LT] = ACTIONS(1616),
    [anon_sym_GT_GT] = ACTIONS(1616),
    [anon_sym_PLUS_EQ] = ACTIONS(1614),
    [anon_sym_DASH_EQ] = ACTIONS(1614),
    [anon_sym_STAR_EQ] = ACTIONS(1614),
    [anon_sym_SLASH_EQ] = ACTIONS(1614),
    [anon_sym_PERCENT_EQ] = ACTIONS(1614),
    [anon_sym_CARET_EQ] = ACTIONS(1614),
    [anon_sym_AMP_EQ] = ACTIONS(1614),
    [anon_sym_PIPE_EQ] = ACTIONS(1614),
    [anon_sym_LT_LT_EQ] = ACTIONS(1614),
    [anon_sym_GT_GT_EQ] = ACTIONS(1614),
    [anon_sym_EQ] = ACTIONS(1616),
    [anon_sym_EQ_EQ] = ACTIONS(1614),
    [anon_sym_BANG_EQ] = ACTIONS(1614),
    [anon_sym_GT] = ACTIONS(1616),
    [anon_sym_LT] = ACTIONS(1616),
    [anon_sym_GT_EQ] = ACTIONS(1614),
    [anon_sym_LT_EQ] = ACTIONS(1614),
    [anon_sym__] = ACTIONS(1616),
    [anon_sym_DOT] = ACTIONS(1616),
    [anon_sym_DOT_DOT] = ACTIONS(1616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1614),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1614),
    [anon_sym_COLON_COLON] = ACTIONS(1614),
    [anon_sym_POUND] = ACTIONS(1614),
    [anon_sym_as] = ACTIONS(1616),
    [anon_sym_auto] = ACTIONS(1616),
    [anon_sym_await] = ACTIONS(1616),
    [anon_sym_const] = ACTIONS(1616),
    [anon_sym_default] = ACTIONS(1616),
    [anon_sym_gen] = ACTIONS(1616),
    [anon_sym_raw] = ACTIONS(1616),
    [anon_sym_union] = ACTIONS(1616),
    [anon_sym_safe] = ACTIONS(1616),
    [anon_sym_ref] = ACTIONS(1616),
    [sym_mutable_specifier] = ACTIONS(1616),
    [anon_sym_try] = ACTIONS(1616),
    [anon_sym_box] = ACTIONS(1616),
    [sym_integer_literal] = ACTIONS(1614),
    [aux_sym_string_literal_token1] = ACTIONS(1614),
    [sym_char_literal] = ACTIONS(1614),
    [anon_sym_true] = ACTIONS(1616),
    [anon_sym_false] = ACTIONS(1616),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1616),
    [sym_super] = ACTIONS(1616),
    [sym_crate] = ACTIONS(1616),
    [aux_sym_metavariable_token1] = ACTIONS(1614),
    [sym__raw_string_literal_start] = ACTIONS(1614),
    [sym_float_literal] = ACTIONS(1614),
    [sym__outer_block_doc_comment_start] = ACTIONS(1614),
    [sym__outer_line_doc_comment_start] = ACTIONS(1614),
  },
  [STATE(595)] = {
    [sym_line_comment] = STATE(595),
    [sym_block_comment] = STATE(595),
    [sym_identifier] = ACTIONS(1556),
    [anon_sym_LPAREN] = ACTIONS(1554),
    [anon_sym_RBRACE] = ACTIONS(1554),
    [anon_sym_COMMA] = ACTIONS(1554),
    [anon_sym_macro_rules] = ACTIONS(1556),
    [anon_sym_LBRACK] = ACTIONS(1554),
    [anon_sym_PLUS] = ACTIONS(1556),
    [anon_sym_STAR] = ACTIONS(1556),
    [anon_sym_QMARK] = ACTIONS(1554),
    [anon_sym_u8] = ACTIONS(1556),
    [anon_sym_i8] = ACTIONS(1556),
    [anon_sym_u16] = ACTIONS(1556),
    [anon_sym_i16] = ACTIONS(1556),
    [anon_sym_u32] = ACTIONS(1556),
    [anon_sym_i32] = ACTIONS(1556),
    [anon_sym_u64] = ACTIONS(1556),
    [anon_sym_i64] = ACTIONS(1556),
    [anon_sym_u128] = ACTIONS(1556),
    [anon_sym_i128] = ACTIONS(1556),
    [anon_sym_isize] = ACTIONS(1556),
    [anon_sym_usize] = ACTIONS(1556),
    [anon_sym_f32] = ACTIONS(1556),
    [anon_sym_f64] = ACTIONS(1556),
    [anon_sym_bool] = ACTIONS(1556),
    [anon_sym_str] = ACTIONS(1556),
    [anon_sym_char] = ACTIONS(1556),
    [anon_sym_DASH] = ACTIONS(1556),
    [anon_sym_SLASH] = ACTIONS(1556),
    [anon_sym_PERCENT] = ACTIONS(1556),
    [anon_sym_CARET] = ACTIONS(1556),
    [anon_sym_AMP] = ACTIONS(1556),
    [anon_sym_PIPE] = ACTIONS(1556),
    [anon_sym_AMP_AMP] = ACTIONS(1554),
    [anon_sym_PIPE_PIPE] = ACTIONS(1554),
    [anon_sym_LT_LT] = ACTIONS(1556),
    [anon_sym_GT_GT] = ACTIONS(1556),
    [anon_sym_PLUS_EQ] = ACTIONS(1554),
    [anon_sym_DASH_EQ] = ACTIONS(1554),
    [anon_sym_STAR_EQ] = ACTIONS(1554),
    [anon_sym_SLASH_EQ] = ACTIONS(1554),
    [anon_sym_PERCENT_EQ] = ACTIONS(1554),
    [anon_sym_CARET_EQ] = ACTIONS(1554),
    [anon_sym_AMP_EQ] = ACTIONS(1554),
    [anon_sym_PIPE_EQ] = ACTIONS(1554),
    [anon_sym_LT_LT_EQ] = ACTIONS(1554),
    [anon_sym_GT_GT_EQ] = ACTIONS(1554),
    [anon_sym_EQ] = ACTIONS(1556),
    [anon_sym_EQ_EQ] = ACTIONS(1554),
    [anon_sym_BANG_EQ] = ACTIONS(1554),
    [anon_sym_GT] = ACTIONS(1556),
    [anon_sym_LT] = ACTIONS(1556),
    [anon_sym_GT_EQ] = ACTIONS(1554),
    [anon_sym_LT_EQ] = ACTIONS(1554),
    [anon_sym__] = ACTIONS(1556),
    [anon_sym_DOT] = ACTIONS(1556),
    [anon_sym_DOT_DOT] = ACTIONS(1556),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1554),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1554),
    [anon_sym_COLON_COLON] = ACTIONS(1554),
    [anon_sym_POUND] = ACTIONS(1554),
    [anon_sym_as] = ACTIONS(1556),
    [anon_sym_auto] = ACTIONS(1556),
    [anon_sym_await] = ACTIONS(1556),
    [anon_sym_const] = ACTIONS(1556),
    [anon_sym_default] = ACTIONS(1556),
    [anon_sym_gen] = ACTIONS(1556),
    [anon_sym_raw] = ACTIONS(1556),
    [anon_sym_union] = ACTIONS(1556),
    [anon_sym_safe] = ACTIONS(1556),
    [anon_sym_ref] = ACTIONS(1556),
    [sym_mutable_specifier] = ACTIONS(1556),
    [anon_sym_try] = ACTIONS(1556),
    [anon_sym_box] = ACTIONS(1556),
    [sym_integer_literal] = ACTIONS(1554),
    [aux_sym_string_literal_token1] = ACTIONS(1554),
    [sym_char_literal] = ACTIONS(1554),
    [anon_sym_true] = ACTIONS(1556),
    [anon_sym_false] = ACTIONS(1556),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1556),
    [sym_super] = ACTIONS(1556),
    [sym_crate] = ACTIONS(1556),
    [aux_sym_metavariable_token1] = ACTIONS(1554),
    [sym__raw_string_literal_start] = ACTIONS(1554),
    [sym_float_literal] = ACTIONS(1554),
    [sym__outer_block_doc_comment_start] = ACTIONS(1554),
    [sym__outer_line_doc_comment_start] = ACTIONS(1554),
  },
  [STATE(596)] = {
    [sym_attribute_item] = STATE(1374),
    [sym_line_comment] = STATE(596),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(596),
    [aux_sym_attributes_repeat1] = STATE(597),
    [sym_identifier] = ACTIONS(2151),
    [anon_sym_SEMI] = ACTIONS(2153),
    [anon_sym_macro] = ACTIONS(2151),
    [anon_sym_LPAREN] = ACTIONS(2153),
    [anon_sym_LBRACE] = ACTIONS(2153),
    [anon_sym_macro_rules] = ACTIONS(2151),
    [anon_sym_BANG] = ACTIONS(2153),
    [anon_sym_LBRACK] = ACTIONS(2153),
    [anon_sym_STAR] = ACTIONS(2153),
    [anon_sym_u8] = ACTIONS(2151),
    [anon_sym_i8] = ACTIONS(2151),
    [anon_sym_u16] = ACTIONS(2151),
    [anon_sym_i16] = ACTIONS(2151),
    [anon_sym_u32] = ACTIONS(2151),
    [anon_sym_i32] = ACTIONS(2151),
    [anon_sym_u64] = ACTIONS(2151),
    [anon_sym_i64] = ACTIONS(2151),
    [anon_sym_u128] = ACTIONS(2151),
    [anon_sym_i128] = ACTIONS(2151),
    [anon_sym_isize] = ACTIONS(2151),
    [anon_sym_usize] = ACTIONS(2151),
    [anon_sym_f32] = ACTIONS(2151),
    [anon_sym_f64] = ACTIONS(2151),
    [anon_sym_bool] = ACTIONS(2151),
    [anon_sym_str] = ACTIONS(2151),
    [anon_sym_char] = ACTIONS(2151),
    [anon_sym_DASH] = ACTIONS(2153),
    [anon_sym_AMP] = ACTIONS(2153),
    [anon_sym_PIPE] = ACTIONS(2153),
    [anon_sym_LT] = ACTIONS(2153),
    [anon_sym_DOT_DOT] = ACTIONS(2153),
    [anon_sym_COLON_COLON] = ACTIONS(2153),
    [anon_sym_POUND] = ACTIONS(2155),
    [anon_sym_SQUOTE] = ACTIONS(2151),
    [anon_sym_async] = ACTIONS(2151),
    [anon_sym_auto] = ACTIONS(2151),
    [anon_sym_await] = ACTIONS(2151),
    [anon_sym_become] = ACTIONS(2151),
    [anon_sym_break] = ACTIONS(2151),
    [anon_sym_const] = ACTIONS(2151),
    [anon_sym_continue] = ACTIONS(2151),
    [anon_sym_default] = ACTIONS(2151),
    [anon_sym_enum] = ACTIONS(2151),
    [anon_sym_fn] = ACTIONS(2151),
    [anon_sym_for] = ACTIONS(2151),
    [anon_sym_gen] = ACTIONS(2151),
    [anon_sym_if] = ACTIONS(2151),
    [anon_sym_impl] = ACTIONS(2151),
    [anon_sym_let] = ACTIONS(2151),
    [anon_sym_loop] = ACTIONS(2151),
    [anon_sym_match] = ACTIONS(2151),
    [anon_sym_mod] = ACTIONS(2151),
    [anon_sym_pub] = ACTIONS(2151),
    [anon_sym_return] = ACTIONS(2151),
    [anon_sym_raw] = ACTIONS(2151),
    [anon_sym_static] = ACTIONS(2151),
    [anon_sym_struct] = ACTIONS(2151),
    [anon_sym_trait] = ACTIONS(2151),
    [anon_sym_type] = ACTIONS(2151),
    [anon_sym_union] = ACTIONS(2151),
    [anon_sym_unsafe] = ACTIONS(2151),
    [anon_sym_use] = ACTIONS(2151),
    [anon_sym_while] = ACTIONS(2151),
    [anon_sym_extern] = ACTIONS(2151),
    [anon_sym_safe] = ACTIONS(2151),
    [anon_sym_yield] = ACTIONS(2151),
    [anon_sym_move] = ACTIONS(2151),
    [anon_sym_try] = ACTIONS(2151),
    [sym_integer_literal] = ACTIONS(2153),
    [aux_sym_string_literal_token1] = ACTIONS(2153),
    [sym_char_literal] = ACTIONS(2153),
    [anon_sym_true] = ACTIONS(2151),
    [anon_sym_false] = ACTIONS(2151),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2151),
    [sym_super] = ACTIONS(2151),
    [sym_crate] = ACTIONS(2151),
    [aux_sym_metavariable_token1] = ACTIONS(2153),
    [sym__raw_string_literal_start] = ACTIONS(2153),
    [sym_float_literal] = ACTIONS(2153),
    [sym__outer_block_doc_comment_start] = ACTIONS(125),
    [sym__outer_line_doc_comment_start] = ACTIONS(127),
  },
  [STATE(597)] = {
    [sym_attribute_item] = STATE(1374),
    [sym_line_comment] = STATE(597),
    [sym_line_outer_doc_comment] = STATE(1374),
    [sym_block_outer_doc_comment] = STATE(1374),
    [sym_block_comment] = STATE(597),
    [aux_sym_attributes_repeat1] = STATE(597),
    [sym_identifier] = ACTIONS(2157),
    [anon_sym_SEMI] = ACTIONS(2159),
    [anon_sym_macro] = ACTIONS(2157),
    [anon_sym_LPAREN] = ACTIONS(2159),
    [anon_sym_LBRACE] = ACTIONS(2159),
    [anon_sym_macro_rules] = ACTIONS(2157),
    [anon_sym_BANG] = ACTIONS(2159),
    [anon_sym_LBRACK] = ACTIONS(2159),
    [anon_sym_STAR] = ACTIONS(2159),
    [anon_sym_u8] = ACTIONS(2157),
    [anon_sym_i8] = ACTIONS(2157),
    [anon_sym_u16] = ACTIONS(2157),
    [anon_sym_i16] = ACTIONS(2157),
    [anon_sym_u32] = ACTIONS(2157),
    [anon_sym_i32] = ACTIONS(2157),
    [anon_sym_u64] = ACTIONS(2157),
    [anon_sym_i64] = ACTIONS(2157),
    [anon_sym_u128] = ACTIONS(2157),
    [anon_sym_i128] = ACTIONS(2157),
    [anon_sym_isize] = ACTIONS(2157),
    [anon_sym_usize] = ACTIONS(2157),
    [anon_sym_f32] = ACTIONS(2157),
    [anon_sym_f64] = ACTIONS(2157),
    [anon_sym_bool] = ACTIONS(2157),
    [anon_sym_str] = ACTIONS(2157),
    [anon_sym_char] = ACTIONS(2157),
    [anon_sym_DASH] = ACTIONS(2159),
    [anon_sym_AMP] = ACTIONS(2159),
    [anon_sym_PIPE] = ACTIONS(2159),
    [anon_sym_LT] = ACTIONS(2159),
    [anon_sym_DOT_DOT] = ACTIONS(2159),
    [anon_sym_COLON_COLON] = ACTIONS(2159),
    [anon_sym_POUND] = ACTIONS(2161),
    [anon_sym_SQUOTE] = ACTIONS(2157),
    [anon_sym_async] = ACTIONS(2157),
    [anon_sym_auto] = ACTIONS(2157),
    [anon_sym_await] = ACTIONS(2157),
    [anon_sym_become] = ACTIONS(2157),
    [anon_sym_break] = ACTIONS(2157),
    [anon_sym_const] = ACTIONS(2157),
    [anon_sym_continue] = ACTIONS(2157),
    [anon_sym_default] = ACTIONS(2157),
    [anon_sym_enum] = ACTIONS(2157),
    [anon_sym_fn] = ACTIONS(2157),
    [anon_sym_for] = ACTIONS(2157),
    [anon_sym_gen] = ACTIONS(2157),
    [anon_sym_if] = ACTIONS(2157),
    [anon_sym_impl] = ACTIONS(2157),
    [anon_sym_let] = ACTIONS(2157),
    [anon_sym_loop] = ACTIONS(2157),
    [anon_sym_match] = ACTIONS(2157),
    [anon_sym_mod] = ACTIONS(2157),
    [anon_sym_pub] = ACTIONS(2157),
    [anon_sym_return] = ACTIONS(2157),
    [anon_sym_raw] = ACTIONS(2157),
    [anon_sym_static] = ACTIONS(2157),
    [anon_sym_struct] = ACTIONS(2157),
    [anon_sym_trait] = ACTIONS(2157),
    [anon_sym_type] = ACTIONS(2157),
    [anon_sym_union] = ACTIONS(2157),
    [anon_sym_unsafe] = ACTIONS(2157),
    [anon_sym_use] = ACTIONS(2157),
    [anon_sym_while] = ACTIONS(2157),
    [anon_sym_extern] = ACTIONS(2157),
    [anon_sym_safe] = ACTIONS(2157),
    [anon_sym_yield] = ACTIONS(2157),
    [anon_sym_move] = ACTIONS(2157),
    [anon_sym_try] = ACTIONS(2157),
    [sym_integer_literal] = ACTIONS(2159),
    [aux_sym_string_literal_token1] = ACTIONS(2159),
    [sym_char_literal] = ACTIONS(2159),
    [anon_sym_true] = ACTIONS(2157),
    [anon_sym_false] = ACTIONS(2157),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2157),
    [sym_super] = ACTIONS(2157),
    [sym_crate] = ACTIONS(2157),
    [aux_sym_metavariable_token1] = ACTIONS(2159),
    [sym__raw_string_literal_start] = ACTIONS(2159),
    [sym_float_literal] = ACTIONS(2159),
    [sym__outer_block_doc_comment_start] = ACTIONS(2164),
    [sym__outer_line_doc_comment_start] = ACTIONS(2167),
  },
  [STATE(598)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym_closure_expression] = STATE(5565),
    [sym_closure_parameters] = STATE(119),
    [sym__pattern] = STATE(5247),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(598),
    [sym_block_comment] = STATE(598),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(1316),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_async] = ACTIONS(2115),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_static] = ACTIONS(1322),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_move] = ACTIONS(1326),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(599)] = {
    [sym_line_comment] = STATE(599),
    [sym_block_comment] = STATE(599),
    [ts_builtin_sym_end] = ACTIONS(2170),
    [sym_identifier] = ACTIONS(2172),
    [anon_sym_SEMI] = ACTIONS(2170),
    [anon_sym_macro] = ACTIONS(2172),
    [anon_sym_LPAREN] = ACTIONS(2170),
    [anon_sym_LBRACE] = ACTIONS(2170),
    [anon_sym_RBRACE] = ACTIONS(2170),
    [anon_sym_macro_rules] = ACTIONS(2172),
    [anon_sym_BANG] = ACTIONS(2170),
    [anon_sym_LBRACK] = ACTIONS(2170),
    [anon_sym_STAR] = ACTIONS(2170),
    [anon_sym_u8] = ACTIONS(2172),
    [anon_sym_i8] = ACTIONS(2172),
    [anon_sym_u16] = ACTIONS(2172),
    [anon_sym_i16] = ACTIONS(2172),
    [anon_sym_u32] = ACTIONS(2172),
    [anon_sym_i32] = ACTIONS(2172),
    [anon_sym_u64] = ACTIONS(2172),
    [anon_sym_i64] = ACTIONS(2172),
    [anon_sym_u128] = ACTIONS(2172),
    [anon_sym_i128] = ACTIONS(2172),
    [anon_sym_isize] = ACTIONS(2172),
    [anon_sym_usize] = ACTIONS(2172),
    [anon_sym_f32] = ACTIONS(2172),
    [anon_sym_f64] = ACTIONS(2172),
    [anon_sym_bool] = ACTIONS(2172),
    [anon_sym_str] = ACTIONS(2172),
    [anon_sym_char] = ACTIONS(2172),
    [anon_sym_DASH] = ACTIONS(2170),
    [anon_sym_AMP] = ACTIONS(2170),
    [anon_sym_PIPE] = ACTIONS(2170),
    [anon_sym_LT] = ACTIONS(2170),
    [anon_sym_DOT_DOT] = ACTIONS(2170),
    [anon_sym_COLON_COLON] = ACTIONS(2170),
    [anon_sym_POUND] = ACTIONS(2170),
    [anon_sym_SQUOTE] = ACTIONS(2172),
    [anon_sym_async] = ACTIONS(2172),
    [anon_sym_auto] = ACTIONS(2172),
    [anon_sym_await] = ACTIONS(2172),
    [anon_sym_become] = ACTIONS(2172),
    [anon_sym_break] = ACTIONS(2172),
    [anon_sym_const] = ACTIONS(2172),
    [anon_sym_continue] = ACTIONS(2172),
    [anon_sym_default] = ACTIONS(2172),
    [anon_sym_enum] = ACTIONS(2172),
    [anon_sym_fn] = ACTIONS(2172),
    [anon_sym_for] = ACTIONS(2172),
    [anon_sym_gen] = ACTIONS(2172),
    [anon_sym_if] = ACTIONS(2172),
    [anon_sym_impl] = ACTIONS(2172),
    [anon_sym_let] = ACTIONS(2172),
    [anon_sym_loop] = ACTIONS(2172),
    [anon_sym_match] = ACTIONS(2172),
    [anon_sym_mod] = ACTIONS(2172),
    [anon_sym_pub] = ACTIONS(2172),
    [anon_sym_return] = ACTIONS(2172),
    [anon_sym_raw] = ACTIONS(2172),
    [anon_sym_static] = ACTIONS(2172),
    [anon_sym_struct] = ACTIONS(2172),
    [anon_sym_trait] = ACTIONS(2172),
    [anon_sym_type] = ACTIONS(2172),
    [anon_sym_union] = ACTIONS(2172),
    [anon_sym_unsafe] = ACTIONS(2172),
    [anon_sym_use] = ACTIONS(2172),
    [anon_sym_while] = ACTIONS(2172),
    [anon_sym_extern] = ACTIONS(2172),
    [anon_sym_safe] = ACTIONS(2172),
    [anon_sym_yield] = ACTIONS(2172),
    [anon_sym_move] = ACTIONS(2172),
    [anon_sym_try] = ACTIONS(2172),
    [sym_integer_literal] = ACTIONS(2170),
    [aux_sym_string_literal_token1] = ACTIONS(2170),
    [sym_char_literal] = ACTIONS(2170),
    [anon_sym_true] = ACTIONS(2172),
    [anon_sym_false] = ACTIONS(2172),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2172),
    [sym_super] = ACTIONS(2172),
    [sym_crate] = ACTIONS(2172),
    [aux_sym_metavariable_token1] = ACTIONS(2170),
    [sym__raw_string_literal_start] = ACTIONS(2170),
    [sym_float_literal] = ACTIONS(2170),
    [sym__outer_block_doc_comment_start] = ACTIONS(2170),
    [sym__outer_line_doc_comment_start] = ACTIONS(2170),
  },
  [STATE(600)] = {
    [sym_line_comment] = STATE(600),
    [sym_block_comment] = STATE(600),
    [ts_builtin_sym_end] = ACTIONS(2174),
    [sym_identifier] = ACTIONS(2176),
    [anon_sym_SEMI] = ACTIONS(2174),
    [anon_sym_macro] = ACTIONS(2176),
    [anon_sym_LPAREN] = ACTIONS(2174),
    [anon_sym_LBRACE] = ACTIONS(2174),
    [anon_sym_RBRACE] = ACTIONS(2174),
    [anon_sym_macro_rules] = ACTIONS(2176),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_LBRACK] = ACTIONS(2174),
    [anon_sym_STAR] = ACTIONS(2174),
    [anon_sym_u8] = ACTIONS(2176),
    [anon_sym_i8] = ACTIONS(2176),
    [anon_sym_u16] = ACTIONS(2176),
    [anon_sym_i16] = ACTIONS(2176),
    [anon_sym_u32] = ACTIONS(2176),
    [anon_sym_i32] = ACTIONS(2176),
    [anon_sym_u64] = ACTIONS(2176),
    [anon_sym_i64] = ACTIONS(2176),
    [anon_sym_u128] = ACTIONS(2176),
    [anon_sym_i128] = ACTIONS(2176),
    [anon_sym_isize] = ACTIONS(2176),
    [anon_sym_usize] = ACTIONS(2176),
    [anon_sym_f32] = ACTIONS(2176),
    [anon_sym_f64] = ACTIONS(2176),
    [anon_sym_bool] = ACTIONS(2176),
    [anon_sym_str] = ACTIONS(2176),
    [anon_sym_char] = ACTIONS(2176),
    [anon_sym_DASH] = ACTIONS(2174),
    [anon_sym_AMP] = ACTIONS(2174),
    [anon_sym_PIPE] = ACTIONS(2174),
    [anon_sym_LT] = ACTIONS(2174),
    [anon_sym_DOT_DOT] = ACTIONS(2174),
    [anon_sym_COLON_COLON] = ACTIONS(2174),
    [anon_sym_POUND] = ACTIONS(2174),
    [anon_sym_SQUOTE] = ACTIONS(2176),
    [anon_sym_async] = ACTIONS(2176),
    [anon_sym_auto] = ACTIONS(2176),
    [anon_sym_await] = ACTIONS(2176),
    [anon_sym_become] = ACTIONS(2176),
    [anon_sym_break] = ACTIONS(2176),
    [anon_sym_const] = ACTIONS(2176),
    [anon_sym_continue] = ACTIONS(2176),
    [anon_sym_default] = ACTIONS(2176),
    [anon_sym_enum] = ACTIONS(2176),
    [anon_sym_fn] = ACTIONS(2176),
    [anon_sym_for] = ACTIONS(2176),
    [anon_sym_gen] = ACTIONS(2176),
    [anon_sym_if] = ACTIONS(2176),
    [anon_sym_impl] = ACTIONS(2176),
    [anon_sym_let] = ACTIONS(2176),
    [anon_sym_loop] = ACTIONS(2176),
    [anon_sym_match] = ACTIONS(2176),
    [anon_sym_mod] = ACTIONS(2176),
    [anon_sym_pub] = ACTIONS(2176),
    [anon_sym_return] = ACTIONS(2176),
    [anon_sym_raw] = ACTIONS(2176),
    [anon_sym_static] = ACTIONS(2176),
    [anon_sym_struct] = ACTIONS(2176),
    [anon_sym_trait] = ACTIONS(2176),
    [anon_sym_type] = ACTIONS(2176),
    [anon_sym_union] = ACTIONS(2176),
    [anon_sym_unsafe] = ACTIONS(2176),
    [anon_sym_use] = ACTIONS(2176),
    [anon_sym_while] = ACTIONS(2176),
    [anon_sym_extern] = ACTIONS(2176),
    [anon_sym_safe] = ACTIONS(2176),
    [anon_sym_yield] = ACTIONS(2176),
    [anon_sym_move] = ACTIONS(2176),
    [anon_sym_try] = ACTIONS(2176),
    [sym_integer_literal] = ACTIONS(2174),
    [aux_sym_string_literal_token1] = ACTIONS(2174),
    [sym_char_literal] = ACTIONS(2174),
    [anon_sym_true] = ACTIONS(2176),
    [anon_sym_false] = ACTIONS(2176),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2176),
    [sym_super] = ACTIONS(2176),
    [sym_crate] = ACTIONS(2176),
    [aux_sym_metavariable_token1] = ACTIONS(2174),
    [sym__raw_string_literal_start] = ACTIONS(2174),
    [sym_float_literal] = ACTIONS(2174),
    [sym__outer_block_doc_comment_start] = ACTIONS(2174),
    [sym__outer_line_doc_comment_start] = ACTIONS(2174),
  },
  [STATE(601)] = {
    [sym_line_comment] = STATE(601),
    [sym_block_comment] = STATE(601),
    [ts_builtin_sym_end] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_macro] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_macro_rules] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2178),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_STAR] = ACTIONS(2178),
    [anon_sym_u8] = ACTIONS(2180),
    [anon_sym_i8] = ACTIONS(2180),
    [anon_sym_u16] = ACTIONS(2180),
    [anon_sym_i16] = ACTIONS(2180),
    [anon_sym_u32] = ACTIONS(2180),
    [anon_sym_i32] = ACTIONS(2180),
    [anon_sym_u64] = ACTIONS(2180),
    [anon_sym_i64] = ACTIONS(2180),
    [anon_sym_u128] = ACTIONS(2180),
    [anon_sym_i128] = ACTIONS(2180),
    [anon_sym_isize] = ACTIONS(2180),
    [anon_sym_usize] = ACTIONS(2180),
    [anon_sym_f32] = ACTIONS(2180),
    [anon_sym_f64] = ACTIONS(2180),
    [anon_sym_bool] = ACTIONS(2180),
    [anon_sym_str] = ACTIONS(2180),
    [anon_sym_char] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2178),
    [anon_sym_AMP] = ACTIONS(2178),
    [anon_sym_PIPE] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2178),
    [anon_sym_DOT_DOT] = ACTIONS(2178),
    [anon_sym_COLON_COLON] = ACTIONS(2178),
    [anon_sym_POUND] = ACTIONS(2178),
    [anon_sym_SQUOTE] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_auto] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_become] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_default] = ACTIONS(2180),
    [anon_sym_enum] = ACTIONS(2180),
    [anon_sym_fn] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_gen] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_impl] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_loop] = ACTIONS(2180),
    [anon_sym_match] = ACTIONS(2180),
    [anon_sym_mod] = ACTIONS(2180),
    [anon_sym_pub] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_raw] = ACTIONS(2180),
    [anon_sym_static] = ACTIONS(2180),
    [anon_sym_struct] = ACTIONS(2180),
    [anon_sym_trait] = ACTIONS(2180),
    [anon_sym_type] = ACTIONS(2180),
    [anon_sym_union] = ACTIONS(2180),
    [anon_sym_unsafe] = ACTIONS(2180),
    [anon_sym_use] = ACTIONS(2180),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_extern] = ACTIONS(2180),
    [anon_sym_safe] = ACTIONS(2180),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_move] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [sym_integer_literal] = ACTIONS(2178),
    [aux_sym_string_literal_token1] = ACTIONS(2178),
    [sym_char_literal] = ACTIONS(2178),
    [anon_sym_true] = ACTIONS(2180),
    [anon_sym_false] = ACTIONS(2180),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_crate] = ACTIONS(2180),
    [aux_sym_metavariable_token1] = ACTIONS(2178),
    [sym__raw_string_literal_start] = ACTIONS(2178),
    [sym_float_literal] = ACTIONS(2178),
    [sym__outer_block_doc_comment_start] = ACTIONS(2178),
    [sym__outer_line_doc_comment_start] = ACTIONS(2178),
  },
  [STATE(602)] = {
    [sym_line_comment] = STATE(602),
    [sym_block_comment] = STATE(602),
    [ts_builtin_sym_end] = ACTIONS(2182),
    [sym_identifier] = ACTIONS(2184),
    [anon_sym_SEMI] = ACTIONS(2182),
    [anon_sym_macro] = ACTIONS(2184),
    [anon_sym_LPAREN] = ACTIONS(2182),
    [anon_sym_LBRACE] = ACTIONS(2182),
    [anon_sym_RBRACE] = ACTIONS(2182),
    [anon_sym_macro_rules] = ACTIONS(2184),
    [anon_sym_BANG] = ACTIONS(2182),
    [anon_sym_LBRACK] = ACTIONS(2182),
    [anon_sym_STAR] = ACTIONS(2182),
    [anon_sym_u8] = ACTIONS(2184),
    [anon_sym_i8] = ACTIONS(2184),
    [anon_sym_u16] = ACTIONS(2184),
    [anon_sym_i16] = ACTIONS(2184),
    [anon_sym_u32] = ACTIONS(2184),
    [anon_sym_i32] = ACTIONS(2184),
    [anon_sym_u64] = ACTIONS(2184),
    [anon_sym_i64] = ACTIONS(2184),
    [anon_sym_u128] = ACTIONS(2184),
    [anon_sym_i128] = ACTIONS(2184),
    [anon_sym_isize] = ACTIONS(2184),
    [anon_sym_usize] = ACTIONS(2184),
    [anon_sym_f32] = ACTIONS(2184),
    [anon_sym_f64] = ACTIONS(2184),
    [anon_sym_bool] = ACTIONS(2184),
    [anon_sym_str] = ACTIONS(2184),
    [anon_sym_char] = ACTIONS(2184),
    [anon_sym_DASH] = ACTIONS(2182),
    [anon_sym_AMP] = ACTIONS(2182),
    [anon_sym_PIPE] = ACTIONS(2182),
    [anon_sym_LT] = ACTIONS(2182),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_COLON_COLON] = ACTIONS(2182),
    [anon_sym_POUND] = ACTIONS(2182),
    [anon_sym_SQUOTE] = ACTIONS(2184),
    [anon_sym_async] = ACTIONS(2184),
    [anon_sym_auto] = ACTIONS(2184),
    [anon_sym_await] = ACTIONS(2184),
    [anon_sym_become] = ACTIONS(2184),
    [anon_sym_break] = ACTIONS(2184),
    [anon_sym_const] = ACTIONS(2184),
    [anon_sym_continue] = ACTIONS(2184),
    [anon_sym_default] = ACTIONS(2184),
    [anon_sym_enum] = ACTIONS(2184),
    [anon_sym_fn] = ACTIONS(2184),
    [anon_sym_for] = ACTIONS(2184),
    [anon_sym_gen] = ACTIONS(2184),
    [anon_sym_if] = ACTIONS(2184),
    [anon_sym_impl] = ACTIONS(2184),
    [anon_sym_let] = ACTIONS(2184),
    [anon_sym_loop] = ACTIONS(2184),
    [anon_sym_match] = ACTIONS(2184),
    [anon_sym_mod] = ACTIONS(2184),
    [anon_sym_pub] = ACTIONS(2184),
    [anon_sym_return] = ACTIONS(2184),
    [anon_sym_raw] = ACTIONS(2184),
    [anon_sym_static] = ACTIONS(2184),
    [anon_sym_struct] = ACTIONS(2184),
    [anon_sym_trait] = ACTIONS(2184),
    [anon_sym_type] = ACTIONS(2184),
    [anon_sym_union] = ACTIONS(2184),
    [anon_sym_unsafe] = ACTIONS(2184),
    [anon_sym_use] = ACTIONS(2184),
    [anon_sym_while] = ACTIONS(2184),
    [anon_sym_extern] = ACTIONS(2184),
    [anon_sym_safe] = ACTIONS(2184),
    [anon_sym_yield] = ACTIONS(2184),
    [anon_sym_move] = ACTIONS(2184),
    [anon_sym_try] = ACTIONS(2184),
    [sym_integer_literal] = ACTIONS(2182),
    [aux_sym_string_literal_token1] = ACTIONS(2182),
    [sym_char_literal] = ACTIONS(2182),
    [anon_sym_true] = ACTIONS(2184),
    [anon_sym_false] = ACTIONS(2184),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2184),
    [sym_super] = ACTIONS(2184),
    [sym_crate] = ACTIONS(2184),
    [aux_sym_metavariable_token1] = ACTIONS(2182),
    [sym__raw_string_literal_start] = ACTIONS(2182),
    [sym_float_literal] = ACTIONS(2182),
    [sym__outer_block_doc_comment_start] = ACTIONS(2182),
    [sym__outer_line_doc_comment_start] = ACTIONS(2182),
  },
  [STATE(603)] = {
    [sym_line_comment] = STATE(603),
    [sym_block_comment] = STATE(603),
    [ts_builtin_sym_end] = ACTIONS(2186),
    [sym_identifier] = ACTIONS(2188),
    [anon_sym_SEMI] = ACTIONS(2186),
    [anon_sym_macro] = ACTIONS(2188),
    [anon_sym_LPAREN] = ACTIONS(2186),
    [anon_sym_LBRACE] = ACTIONS(2186),
    [anon_sym_RBRACE] = ACTIONS(2186),
    [anon_sym_macro_rules] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2186),
    [anon_sym_LBRACK] = ACTIONS(2186),
    [anon_sym_STAR] = ACTIONS(2186),
    [anon_sym_u8] = ACTIONS(2188),
    [anon_sym_i8] = ACTIONS(2188),
    [anon_sym_u16] = ACTIONS(2188),
    [anon_sym_i16] = ACTIONS(2188),
    [anon_sym_u32] = ACTIONS(2188),
    [anon_sym_i32] = ACTIONS(2188),
    [anon_sym_u64] = ACTIONS(2188),
    [anon_sym_i64] = ACTIONS(2188),
    [anon_sym_u128] = ACTIONS(2188),
    [anon_sym_i128] = ACTIONS(2188),
    [anon_sym_isize] = ACTIONS(2188),
    [anon_sym_usize] = ACTIONS(2188),
    [anon_sym_f32] = ACTIONS(2188),
    [anon_sym_f64] = ACTIONS(2188),
    [anon_sym_bool] = ACTIONS(2188),
    [anon_sym_str] = ACTIONS(2188),
    [anon_sym_char] = ACTIONS(2188),
    [anon_sym_DASH] = ACTIONS(2186),
    [anon_sym_AMP] = ACTIONS(2186),
    [anon_sym_PIPE] = ACTIONS(2186),
    [anon_sym_LT] = ACTIONS(2186),
    [anon_sym_DOT_DOT] = ACTIONS(2186),
    [anon_sym_COLON_COLON] = ACTIONS(2186),
    [anon_sym_POUND] = ACTIONS(2186),
    [anon_sym_SQUOTE] = ACTIONS(2188),
    [anon_sym_async] = ACTIONS(2188),
    [anon_sym_auto] = ACTIONS(2188),
    [anon_sym_await] = ACTIONS(2188),
    [anon_sym_become] = ACTIONS(2188),
    [anon_sym_break] = ACTIONS(2188),
    [anon_sym_const] = ACTIONS(2188),
    [anon_sym_continue] = ACTIONS(2188),
    [anon_sym_default] = ACTIONS(2188),
    [anon_sym_enum] = ACTIONS(2188),
    [anon_sym_fn] = ACTIONS(2188),
    [anon_sym_for] = ACTIONS(2188),
    [anon_sym_gen] = ACTIONS(2188),
    [anon_sym_if] = ACTIONS(2188),
    [anon_sym_impl] = ACTIONS(2188),
    [anon_sym_let] = ACTIONS(2188),
    [anon_sym_loop] = ACTIONS(2188),
    [anon_sym_match] = ACTIONS(2188),
    [anon_sym_mod] = ACTIONS(2188),
    [anon_sym_pub] = ACTIONS(2188),
    [anon_sym_return] = ACTIONS(2188),
    [anon_sym_raw] = ACTIONS(2188),
    [anon_sym_static] = ACTIONS(2188),
    [anon_sym_struct] = ACTIONS(2188),
    [anon_sym_trait] = ACTIONS(2188),
    [anon_sym_type] = ACTIONS(2188),
    [anon_sym_union] = ACTIONS(2188),
    [anon_sym_unsafe] = ACTIONS(2188),
    [anon_sym_use] = ACTIONS(2188),
    [anon_sym_while] = ACTIONS(2188),
    [anon_sym_extern] = ACTIONS(2188),
    [anon_sym_safe] = ACTIONS(2188),
    [anon_sym_yield] = ACTIONS(2188),
    [anon_sym_move] = ACTIONS(2188),
    [anon_sym_try] = ACTIONS(2188),
    [sym_integer_literal] = ACTIONS(2186),
    [aux_sym_string_literal_token1] = ACTIONS(2186),
    [sym_char_literal] = ACTIONS(2186),
    [anon_sym_true] = ACTIONS(2188),
    [anon_sym_false] = ACTIONS(2188),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2188),
    [sym_super] = ACTIONS(2188),
    [sym_crate] = ACTIONS(2188),
    [aux_sym_metavariable_token1] = ACTIONS(2186),
    [sym__raw_string_literal_start] = ACTIONS(2186),
    [sym_float_literal] = ACTIONS(2186),
    [sym__outer_block_doc_comment_start] = ACTIONS(2186),
    [sym__outer_line_doc_comment_start] = ACTIONS(2186),
  },
  [STATE(604)] = {
    [sym_line_comment] = STATE(604),
    [sym_block_comment] = STATE(604),
    [ts_builtin_sym_end] = ACTIONS(2190),
    [sym_identifier] = ACTIONS(2192),
    [anon_sym_SEMI] = ACTIONS(2190),
    [anon_sym_macro] = ACTIONS(2192),
    [anon_sym_LPAREN] = ACTIONS(2190),
    [anon_sym_LBRACE] = ACTIONS(2190),
    [anon_sym_RBRACE] = ACTIONS(2190),
    [anon_sym_macro_rules] = ACTIONS(2192),
    [anon_sym_BANG] = ACTIONS(2190),
    [anon_sym_LBRACK] = ACTIONS(2190),
    [anon_sym_STAR] = ACTIONS(2190),
    [anon_sym_u8] = ACTIONS(2192),
    [anon_sym_i8] = ACTIONS(2192),
    [anon_sym_u16] = ACTIONS(2192),
    [anon_sym_i16] = ACTIONS(2192),
    [anon_sym_u32] = ACTIONS(2192),
    [anon_sym_i32] = ACTIONS(2192),
    [anon_sym_u64] = ACTIONS(2192),
    [anon_sym_i64] = ACTIONS(2192),
    [anon_sym_u128] = ACTIONS(2192),
    [anon_sym_i128] = ACTIONS(2192),
    [anon_sym_isize] = ACTIONS(2192),
    [anon_sym_usize] = ACTIONS(2192),
    [anon_sym_f32] = ACTIONS(2192),
    [anon_sym_f64] = ACTIONS(2192),
    [anon_sym_bool] = ACTIONS(2192),
    [anon_sym_str] = ACTIONS(2192),
    [anon_sym_char] = ACTIONS(2192),
    [anon_sym_DASH] = ACTIONS(2190),
    [anon_sym_AMP] = ACTIONS(2190),
    [anon_sym_PIPE] = ACTIONS(2190),
    [anon_sym_LT] = ACTIONS(2190),
    [anon_sym_DOT_DOT] = ACTIONS(2190),
    [anon_sym_COLON_COLON] = ACTIONS(2190),
    [anon_sym_POUND] = ACTIONS(2190),
    [anon_sym_SQUOTE] = ACTIONS(2192),
    [anon_sym_async] = ACTIONS(2192),
    [anon_sym_auto] = ACTIONS(2192),
    [anon_sym_await] = ACTIONS(2192),
    [anon_sym_become] = ACTIONS(2192),
    [anon_sym_break] = ACTIONS(2192),
    [anon_sym_const] = ACTIONS(2192),
    [anon_sym_continue] = ACTIONS(2192),
    [anon_sym_default] = ACTIONS(2192),
    [anon_sym_enum] = ACTIONS(2192),
    [anon_sym_fn] = ACTIONS(2192),
    [anon_sym_for] = ACTIONS(2192),
    [anon_sym_gen] = ACTIONS(2192),
    [anon_sym_if] = ACTIONS(2192),
    [anon_sym_impl] = ACTIONS(2192),
    [anon_sym_let] = ACTIONS(2192),
    [anon_sym_loop] = ACTIONS(2192),
    [anon_sym_match] = ACTIONS(2192),
    [anon_sym_mod] = ACTIONS(2192),
    [anon_sym_pub] = ACTIONS(2192),
    [anon_sym_return] = ACTIONS(2192),
    [anon_sym_raw] = ACTIONS(2192),
    [anon_sym_static] = ACTIONS(2192),
    [anon_sym_struct] = ACTIONS(2192),
    [anon_sym_trait] = ACTIONS(2192),
    [anon_sym_type] = ACTIONS(2192),
    [anon_sym_union] = ACTIONS(2192),
    [anon_sym_unsafe] = ACTIONS(2192),
    [anon_sym_use] = ACTIONS(2192),
    [anon_sym_while] = ACTIONS(2192),
    [anon_sym_extern] = ACTIONS(2192),
    [anon_sym_safe] = ACTIONS(2192),
    [anon_sym_yield] = ACTIONS(2192),
    [anon_sym_move] = ACTIONS(2192),
    [anon_sym_try] = ACTIONS(2192),
    [sym_integer_literal] = ACTIONS(2190),
    [aux_sym_string_literal_token1] = ACTIONS(2190),
    [sym_char_literal] = ACTIONS(2190),
    [anon_sym_true] = ACTIONS(2192),
    [anon_sym_false] = ACTIONS(2192),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2192),
    [sym_super] = ACTIONS(2192),
    [sym_crate] = ACTIONS(2192),
    [aux_sym_metavariable_token1] = ACTIONS(2190),
    [sym__raw_string_literal_start] = ACTIONS(2190),
    [sym_float_literal] = ACTIONS(2190),
    [sym__outer_block_doc_comment_start] = ACTIONS(2190),
    [sym__outer_line_doc_comment_start] = ACTIONS(2190),
  },
  [STATE(605)] = {
    [sym_line_comment] = STATE(605),
    [sym_block_comment] = STATE(605),
    [ts_builtin_sym_end] = ACTIONS(2194),
    [sym_identifier] = ACTIONS(2196),
    [anon_sym_SEMI] = ACTIONS(2194),
    [anon_sym_macro] = ACTIONS(2196),
    [anon_sym_LPAREN] = ACTIONS(2194),
    [anon_sym_LBRACE] = ACTIONS(2194),
    [anon_sym_RBRACE] = ACTIONS(2194),
    [anon_sym_macro_rules] = ACTIONS(2196),
    [anon_sym_BANG] = ACTIONS(2194),
    [anon_sym_LBRACK] = ACTIONS(2194),
    [anon_sym_STAR] = ACTIONS(2194),
    [anon_sym_u8] = ACTIONS(2196),
    [anon_sym_i8] = ACTIONS(2196),
    [anon_sym_u16] = ACTIONS(2196),
    [anon_sym_i16] = ACTIONS(2196),
    [anon_sym_u32] = ACTIONS(2196),
    [anon_sym_i32] = ACTIONS(2196),
    [anon_sym_u64] = ACTIONS(2196),
    [anon_sym_i64] = ACTIONS(2196),
    [anon_sym_u128] = ACTIONS(2196),
    [anon_sym_i128] = ACTIONS(2196),
    [anon_sym_isize] = ACTIONS(2196),
    [anon_sym_usize] = ACTIONS(2196),
    [anon_sym_f32] = ACTIONS(2196),
    [anon_sym_f64] = ACTIONS(2196),
    [anon_sym_bool] = ACTIONS(2196),
    [anon_sym_str] = ACTIONS(2196),
    [anon_sym_char] = ACTIONS(2196),
    [anon_sym_DASH] = ACTIONS(2194),
    [anon_sym_AMP] = ACTIONS(2194),
    [anon_sym_PIPE] = ACTIONS(2194),
    [anon_sym_LT] = ACTIONS(2194),
    [anon_sym_DOT_DOT] = ACTIONS(2194),
    [anon_sym_COLON_COLON] = ACTIONS(2194),
    [anon_sym_POUND] = ACTIONS(2194),
    [anon_sym_SQUOTE] = ACTIONS(2196),
    [anon_sym_async] = ACTIONS(2196),
    [anon_sym_auto] = ACTIONS(2196),
    [anon_sym_await] = ACTIONS(2196),
    [anon_sym_become] = ACTIONS(2196),
    [anon_sym_break] = ACTIONS(2196),
    [anon_sym_const] = ACTIONS(2196),
    [anon_sym_continue] = ACTIONS(2196),
    [anon_sym_default] = ACTIONS(2196),
    [anon_sym_enum] = ACTIONS(2196),
    [anon_sym_fn] = ACTIONS(2196),
    [anon_sym_for] = ACTIONS(2196),
    [anon_sym_gen] = ACTIONS(2196),
    [anon_sym_if] = ACTIONS(2196),
    [anon_sym_impl] = ACTIONS(2196),
    [anon_sym_let] = ACTIONS(2196),
    [anon_sym_loop] = ACTIONS(2196),
    [anon_sym_match] = ACTIONS(2196),
    [anon_sym_mod] = ACTIONS(2196),
    [anon_sym_pub] = ACTIONS(2196),
    [anon_sym_return] = ACTIONS(2196),
    [anon_sym_raw] = ACTIONS(2196),
    [anon_sym_static] = ACTIONS(2196),
    [anon_sym_struct] = ACTIONS(2196),
    [anon_sym_trait] = ACTIONS(2196),
    [anon_sym_type] = ACTIONS(2196),
    [anon_sym_union] = ACTIONS(2196),
    [anon_sym_unsafe] = ACTIONS(2196),
    [anon_sym_use] = ACTIONS(2196),
    [anon_sym_while] = ACTIONS(2196),
    [anon_sym_extern] = ACTIONS(2196),
    [anon_sym_safe] = ACTIONS(2196),
    [anon_sym_yield] = ACTIONS(2196),
    [anon_sym_move] = ACTIONS(2196),
    [anon_sym_try] = ACTIONS(2196),
    [sym_integer_literal] = ACTIONS(2194),
    [aux_sym_string_literal_token1] = ACTIONS(2194),
    [sym_char_literal] = ACTIONS(2194),
    [anon_sym_true] = ACTIONS(2196),
    [anon_sym_false] = ACTIONS(2196),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2196),
    [sym_super] = ACTIONS(2196),
    [sym_crate] = ACTIONS(2196),
    [aux_sym_metavariable_token1] = ACTIONS(2194),
    [sym__raw_string_literal_start] = ACTIONS(2194),
    [sym_float_literal] = ACTIONS(2194),
    [sym__outer_block_doc_comment_start] = ACTIONS(2194),
    [sym__outer_line_doc_comment_start] = ACTIONS(2194),
  },
  [STATE(606)] = {
    [sym_line_comment] = STATE(606),
    [sym_block_comment] = STATE(606),
    [ts_builtin_sym_end] = ACTIONS(2198),
    [sym_identifier] = ACTIONS(2200),
    [anon_sym_SEMI] = ACTIONS(2198),
    [anon_sym_macro] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(2198),
    [anon_sym_LBRACE] = ACTIONS(2198),
    [anon_sym_RBRACE] = ACTIONS(2198),
    [anon_sym_macro_rules] = ACTIONS(2200),
    [anon_sym_BANG] = ACTIONS(2198),
    [anon_sym_LBRACK] = ACTIONS(2198),
    [anon_sym_STAR] = ACTIONS(2198),
    [anon_sym_u8] = ACTIONS(2200),
    [anon_sym_i8] = ACTIONS(2200),
    [anon_sym_u16] = ACTIONS(2200),
    [anon_sym_i16] = ACTIONS(2200),
    [anon_sym_u32] = ACTIONS(2200),
    [anon_sym_i32] = ACTIONS(2200),
    [anon_sym_u64] = ACTIONS(2200),
    [anon_sym_i64] = ACTIONS(2200),
    [anon_sym_u128] = ACTIONS(2200),
    [anon_sym_i128] = ACTIONS(2200),
    [anon_sym_isize] = ACTIONS(2200),
    [anon_sym_usize] = ACTIONS(2200),
    [anon_sym_f32] = ACTIONS(2200),
    [anon_sym_f64] = ACTIONS(2200),
    [anon_sym_bool] = ACTIONS(2200),
    [anon_sym_str] = ACTIONS(2200),
    [anon_sym_char] = ACTIONS(2200),
    [anon_sym_DASH] = ACTIONS(2198),
    [anon_sym_AMP] = ACTIONS(2198),
    [anon_sym_PIPE] = ACTIONS(2198),
    [anon_sym_LT] = ACTIONS(2198),
    [anon_sym_DOT_DOT] = ACTIONS(2198),
    [anon_sym_COLON_COLON] = ACTIONS(2198),
    [anon_sym_POUND] = ACTIONS(2198),
    [anon_sym_SQUOTE] = ACTIONS(2200),
    [anon_sym_async] = ACTIONS(2200),
    [anon_sym_auto] = ACTIONS(2200),
    [anon_sym_await] = ACTIONS(2200),
    [anon_sym_become] = ACTIONS(2200),
    [anon_sym_break] = ACTIONS(2200),
    [anon_sym_const] = ACTIONS(2200),
    [anon_sym_continue] = ACTIONS(2200),
    [anon_sym_default] = ACTIONS(2200),
    [anon_sym_enum] = ACTIONS(2200),
    [anon_sym_fn] = ACTIONS(2200),
    [anon_sym_for] = ACTIONS(2200),
    [anon_sym_gen] = ACTIONS(2200),
    [anon_sym_if] = ACTIONS(2200),
    [anon_sym_impl] = ACTIONS(2200),
    [anon_sym_let] = ACTIONS(2200),
    [anon_sym_loop] = ACTIONS(2200),
    [anon_sym_match] = ACTIONS(2200),
    [anon_sym_mod] = ACTIONS(2200),
    [anon_sym_pub] = ACTIONS(2200),
    [anon_sym_return] = ACTIONS(2200),
    [anon_sym_raw] = ACTIONS(2200),
    [anon_sym_static] = ACTIONS(2200),
    [anon_sym_struct] = ACTIONS(2200),
    [anon_sym_trait] = ACTIONS(2200),
    [anon_sym_type] = ACTIONS(2200),
    [anon_sym_union] = ACTIONS(2200),
    [anon_sym_unsafe] = ACTIONS(2200),
    [anon_sym_use] = ACTIONS(2200),
    [anon_sym_while] = ACTIONS(2200),
    [anon_sym_extern] = ACTIONS(2200),
    [anon_sym_safe] = ACTIONS(2200),
    [anon_sym_yield] = ACTIONS(2200),
    [anon_sym_move] = ACTIONS(2200),
    [anon_sym_try] = ACTIONS(2200),
    [sym_integer_literal] = ACTIONS(2198),
    [aux_sym_string_literal_token1] = ACTIONS(2198),
    [sym_char_literal] = ACTIONS(2198),
    [anon_sym_true] = ACTIONS(2200),
    [anon_sym_false] = ACTIONS(2200),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2200),
    [sym_super] = ACTIONS(2200),
    [sym_crate] = ACTIONS(2200),
    [aux_sym_metavariable_token1] = ACTIONS(2198),
    [sym__raw_string_literal_start] = ACTIONS(2198),
    [sym_float_literal] = ACTIONS(2198),
    [sym__outer_block_doc_comment_start] = ACTIONS(2198),
    [sym__outer_line_doc_comment_start] = ACTIONS(2198),
  },
  [STATE(607)] = {
    [sym_line_comment] = STATE(607),
    [sym_block_comment] = STATE(607),
    [ts_builtin_sym_end] = ACTIONS(2202),
    [sym_identifier] = ACTIONS(2204),
    [anon_sym_SEMI] = ACTIONS(2202),
    [anon_sym_macro] = ACTIONS(2204),
    [anon_sym_LPAREN] = ACTIONS(2202),
    [anon_sym_LBRACE] = ACTIONS(2202),
    [anon_sym_RBRACE] = ACTIONS(2202),
    [anon_sym_macro_rules] = ACTIONS(2204),
    [anon_sym_BANG] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(2202),
    [anon_sym_STAR] = ACTIONS(2202),
    [anon_sym_u8] = ACTIONS(2204),
    [anon_sym_i8] = ACTIONS(2204),
    [anon_sym_u16] = ACTIONS(2204),
    [anon_sym_i16] = ACTIONS(2204),
    [anon_sym_u32] = ACTIONS(2204),
    [anon_sym_i32] = ACTIONS(2204),
    [anon_sym_u64] = ACTIONS(2204),
    [anon_sym_i64] = ACTIONS(2204),
    [anon_sym_u128] = ACTIONS(2204),
    [anon_sym_i128] = ACTIONS(2204),
    [anon_sym_isize] = ACTIONS(2204),
    [anon_sym_usize] = ACTIONS(2204),
    [anon_sym_f32] = ACTIONS(2204),
    [anon_sym_f64] = ACTIONS(2204),
    [anon_sym_bool] = ACTIONS(2204),
    [anon_sym_str] = ACTIONS(2204),
    [anon_sym_char] = ACTIONS(2204),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_DOT_DOT] = ACTIONS(2202),
    [anon_sym_COLON_COLON] = ACTIONS(2202),
    [anon_sym_POUND] = ACTIONS(2202),
    [anon_sym_SQUOTE] = ACTIONS(2204),
    [anon_sym_async] = ACTIONS(2204),
    [anon_sym_auto] = ACTIONS(2204),
    [anon_sym_await] = ACTIONS(2204),
    [anon_sym_become] = ACTIONS(2204),
    [anon_sym_break] = ACTIONS(2204),
    [anon_sym_const] = ACTIONS(2204),
    [anon_sym_continue] = ACTIONS(2204),
    [anon_sym_default] = ACTIONS(2204),
    [anon_sym_enum] = ACTIONS(2204),
    [anon_sym_fn] = ACTIONS(2204),
    [anon_sym_for] = ACTIONS(2204),
    [anon_sym_gen] = ACTIONS(2204),
    [anon_sym_if] = ACTIONS(2204),
    [anon_sym_impl] = ACTIONS(2204),
    [anon_sym_let] = ACTIONS(2204),
    [anon_sym_loop] = ACTIONS(2204),
    [anon_sym_match] = ACTIONS(2204),
    [anon_sym_mod] = ACTIONS(2204),
    [anon_sym_pub] = ACTIONS(2204),
    [anon_sym_return] = ACTIONS(2204),
    [anon_sym_raw] = ACTIONS(2204),
    [anon_sym_static] = ACTIONS(2204),
    [anon_sym_struct] = ACTIONS(2204),
    [anon_sym_trait] = ACTIONS(2204),
    [anon_sym_type] = ACTIONS(2204),
    [anon_sym_union] = ACTIONS(2204),
    [anon_sym_unsafe] = ACTIONS(2204),
    [anon_sym_use] = ACTIONS(2204),
    [anon_sym_while] = ACTIONS(2204),
    [anon_sym_extern] = ACTIONS(2204),
    [anon_sym_safe] = ACTIONS(2204),
    [anon_sym_yield] = ACTIONS(2204),
    [anon_sym_move] = ACTIONS(2204),
    [anon_sym_try] = ACTIONS(2204),
    [sym_integer_literal] = ACTIONS(2202),
    [aux_sym_string_literal_token1] = ACTIONS(2202),
    [sym_char_literal] = ACTIONS(2202),
    [anon_sym_true] = ACTIONS(2204),
    [anon_sym_false] = ACTIONS(2204),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2204),
    [sym_super] = ACTIONS(2204),
    [sym_crate] = ACTIONS(2204),
    [aux_sym_metavariable_token1] = ACTIONS(2202),
    [sym__raw_string_literal_start] = ACTIONS(2202),
    [sym_float_literal] = ACTIONS(2202),
    [sym__outer_block_doc_comment_start] = ACTIONS(2202),
    [sym__outer_line_doc_comment_start] = ACTIONS(2202),
  },
  [STATE(608)] = {
    [sym_line_comment] = STATE(608),
    [sym_block_comment] = STATE(608),
    [ts_builtin_sym_end] = ACTIONS(2206),
    [sym_identifier] = ACTIONS(2208),
    [anon_sym_SEMI] = ACTIONS(2206),
    [anon_sym_macro] = ACTIONS(2208),
    [anon_sym_LPAREN] = ACTIONS(2206),
    [anon_sym_LBRACE] = ACTIONS(2206),
    [anon_sym_RBRACE] = ACTIONS(2206),
    [anon_sym_macro_rules] = ACTIONS(2208),
    [anon_sym_BANG] = ACTIONS(2206),
    [anon_sym_LBRACK] = ACTIONS(2206),
    [anon_sym_STAR] = ACTIONS(2206),
    [anon_sym_u8] = ACTIONS(2208),
    [anon_sym_i8] = ACTIONS(2208),
    [anon_sym_u16] = ACTIONS(2208),
    [anon_sym_i16] = ACTIONS(2208),
    [anon_sym_u32] = ACTIONS(2208),
    [anon_sym_i32] = ACTIONS(2208),
    [anon_sym_u64] = ACTIONS(2208),
    [anon_sym_i64] = ACTIONS(2208),
    [anon_sym_u128] = ACTIONS(2208),
    [anon_sym_i128] = ACTIONS(2208),
    [anon_sym_isize] = ACTIONS(2208),
    [anon_sym_usize] = ACTIONS(2208),
    [anon_sym_f32] = ACTIONS(2208),
    [anon_sym_f64] = ACTIONS(2208),
    [anon_sym_bool] = ACTIONS(2208),
    [anon_sym_str] = ACTIONS(2208),
    [anon_sym_char] = ACTIONS(2208),
    [anon_sym_DASH] = ACTIONS(2206),
    [anon_sym_AMP] = ACTIONS(2206),
    [anon_sym_PIPE] = ACTIONS(2206),
    [anon_sym_LT] = ACTIONS(2206),
    [anon_sym_DOT_DOT] = ACTIONS(2206),
    [anon_sym_COLON_COLON] = ACTIONS(2206),
    [anon_sym_POUND] = ACTIONS(2206),
    [anon_sym_SQUOTE] = ACTIONS(2208),
    [anon_sym_async] = ACTIONS(2208),
    [anon_sym_auto] = ACTIONS(2208),
    [anon_sym_await] = ACTIONS(2208),
    [anon_sym_become] = ACTIONS(2208),
    [anon_sym_break] = ACTIONS(2208),
    [anon_sym_const] = ACTIONS(2208),
    [anon_sym_continue] = ACTIONS(2208),
    [anon_sym_default] = ACTIONS(2208),
    [anon_sym_enum] = ACTIONS(2208),
    [anon_sym_fn] = ACTIONS(2208),
    [anon_sym_for] = ACTIONS(2208),
    [anon_sym_gen] = ACTIONS(2208),
    [anon_sym_if] = ACTIONS(2208),
    [anon_sym_impl] = ACTIONS(2208),
    [anon_sym_let] = ACTIONS(2208),
    [anon_sym_loop] = ACTIONS(2208),
    [anon_sym_match] = ACTIONS(2208),
    [anon_sym_mod] = ACTIONS(2208),
    [anon_sym_pub] = ACTIONS(2208),
    [anon_sym_return] = ACTIONS(2208),
    [anon_sym_raw] = ACTIONS(2208),
    [anon_sym_static] = ACTIONS(2208),
    [anon_sym_struct] = ACTIONS(2208),
    [anon_sym_trait] = ACTIONS(2208),
    [anon_sym_type] = ACTIONS(2208),
    [anon_sym_union] = ACTIONS(2208),
    [anon_sym_unsafe] = ACTIONS(2208),
    [anon_sym_use] = ACTIONS(2208),
    [anon_sym_while] = ACTIONS(2208),
    [anon_sym_extern] = ACTIONS(2208),
    [anon_sym_safe] = ACTIONS(2208),
    [anon_sym_yield] = ACTIONS(2208),
    [anon_sym_move] = ACTIONS(2208),
    [anon_sym_try] = ACTIONS(2208),
    [sym_integer_literal] = ACTIONS(2206),
    [aux_sym_string_literal_token1] = ACTIONS(2206),
    [sym_char_literal] = ACTIONS(2206),
    [anon_sym_true] = ACTIONS(2208),
    [anon_sym_false] = ACTIONS(2208),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2208),
    [sym_super] = ACTIONS(2208),
    [sym_crate] = ACTIONS(2208),
    [aux_sym_metavariable_token1] = ACTIONS(2206),
    [sym__raw_string_literal_start] = ACTIONS(2206),
    [sym_float_literal] = ACTIONS(2206),
    [sym__outer_block_doc_comment_start] = ACTIONS(2206),
    [sym__outer_line_doc_comment_start] = ACTIONS(2206),
  },
  [STATE(609)] = {
    [sym_line_comment] = STATE(609),
    [sym_block_comment] = STATE(609),
    [ts_builtin_sym_end] = ACTIONS(2210),
    [sym_identifier] = ACTIONS(2212),
    [anon_sym_SEMI] = ACTIONS(2210),
    [anon_sym_macro] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(2210),
    [anon_sym_LBRACE] = ACTIONS(2210),
    [anon_sym_RBRACE] = ACTIONS(2210),
    [anon_sym_macro_rules] = ACTIONS(2212),
    [anon_sym_BANG] = ACTIONS(2210),
    [anon_sym_LBRACK] = ACTIONS(2210),
    [anon_sym_STAR] = ACTIONS(2210),
    [anon_sym_u8] = ACTIONS(2212),
    [anon_sym_i8] = ACTIONS(2212),
    [anon_sym_u16] = ACTIONS(2212),
    [anon_sym_i16] = ACTIONS(2212),
    [anon_sym_u32] = ACTIONS(2212),
    [anon_sym_i32] = ACTIONS(2212),
    [anon_sym_u64] = ACTIONS(2212),
    [anon_sym_i64] = ACTIONS(2212),
    [anon_sym_u128] = ACTIONS(2212),
    [anon_sym_i128] = ACTIONS(2212),
    [anon_sym_isize] = ACTIONS(2212),
    [anon_sym_usize] = ACTIONS(2212),
    [anon_sym_f32] = ACTIONS(2212),
    [anon_sym_f64] = ACTIONS(2212),
    [anon_sym_bool] = ACTIONS(2212),
    [anon_sym_str] = ACTIONS(2212),
    [anon_sym_char] = ACTIONS(2212),
    [anon_sym_DASH] = ACTIONS(2210),
    [anon_sym_AMP] = ACTIONS(2210),
    [anon_sym_PIPE] = ACTIONS(2210),
    [anon_sym_LT] = ACTIONS(2210),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_COLON_COLON] = ACTIONS(2210),
    [anon_sym_POUND] = ACTIONS(2210),
    [anon_sym_SQUOTE] = ACTIONS(2212),
    [anon_sym_async] = ACTIONS(2212),
    [anon_sym_auto] = ACTIONS(2212),
    [anon_sym_await] = ACTIONS(2212),
    [anon_sym_become] = ACTIONS(2212),
    [anon_sym_break] = ACTIONS(2212),
    [anon_sym_const] = ACTIONS(2212),
    [anon_sym_continue] = ACTIONS(2212),
    [anon_sym_default] = ACTIONS(2212),
    [anon_sym_enum] = ACTIONS(2212),
    [anon_sym_fn] = ACTIONS(2212),
    [anon_sym_for] = ACTIONS(2212),
    [anon_sym_gen] = ACTIONS(2212),
    [anon_sym_if] = ACTIONS(2212),
    [anon_sym_impl] = ACTIONS(2212),
    [anon_sym_let] = ACTIONS(2212),
    [anon_sym_loop] = ACTIONS(2212),
    [anon_sym_match] = ACTIONS(2212),
    [anon_sym_mod] = ACTIONS(2212),
    [anon_sym_pub] = ACTIONS(2212),
    [anon_sym_return] = ACTIONS(2212),
    [anon_sym_raw] = ACTIONS(2212),
    [anon_sym_static] = ACTIONS(2212),
    [anon_sym_struct] = ACTIONS(2212),
    [anon_sym_trait] = ACTIONS(2212),
    [anon_sym_type] = ACTIONS(2212),
    [anon_sym_union] = ACTIONS(2212),
    [anon_sym_unsafe] = ACTIONS(2212),
    [anon_sym_use] = ACTIONS(2212),
    [anon_sym_while] = ACTIONS(2212),
    [anon_sym_extern] = ACTIONS(2212),
    [anon_sym_safe] = ACTIONS(2212),
    [anon_sym_yield] = ACTIONS(2212),
    [anon_sym_move] = ACTIONS(2212),
    [anon_sym_try] = ACTIONS(2212),
    [sym_integer_literal] = ACTIONS(2210),
    [aux_sym_string_literal_token1] = ACTIONS(2210),
    [sym_char_literal] = ACTIONS(2210),
    [anon_sym_true] = ACTIONS(2212),
    [anon_sym_false] = ACTIONS(2212),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2212),
    [sym_super] = ACTIONS(2212),
    [sym_crate] = ACTIONS(2212),
    [aux_sym_metavariable_token1] = ACTIONS(2210),
    [sym__raw_string_literal_start] = ACTIONS(2210),
    [sym_float_literal] = ACTIONS(2210),
    [sym__outer_block_doc_comment_start] = ACTIONS(2210),
    [sym__outer_line_doc_comment_start] = ACTIONS(2210),
  },
  [STATE(610)] = {
    [sym_line_comment] = STATE(610),
    [sym_block_comment] = STATE(610),
    [ts_builtin_sym_end] = ACTIONS(2214),
    [sym_identifier] = ACTIONS(2216),
    [anon_sym_SEMI] = ACTIONS(2214),
    [anon_sym_macro] = ACTIONS(2216),
    [anon_sym_LPAREN] = ACTIONS(2214),
    [anon_sym_LBRACE] = ACTIONS(2214),
    [anon_sym_RBRACE] = ACTIONS(2214),
    [anon_sym_macro_rules] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2214),
    [anon_sym_LBRACK] = ACTIONS(2214),
    [anon_sym_STAR] = ACTIONS(2214),
    [anon_sym_u8] = ACTIONS(2216),
    [anon_sym_i8] = ACTIONS(2216),
    [anon_sym_u16] = ACTIONS(2216),
    [anon_sym_i16] = ACTIONS(2216),
    [anon_sym_u32] = ACTIONS(2216),
    [anon_sym_i32] = ACTIONS(2216),
    [anon_sym_u64] = ACTIONS(2216),
    [anon_sym_i64] = ACTIONS(2216),
    [anon_sym_u128] = ACTIONS(2216),
    [anon_sym_i128] = ACTIONS(2216),
    [anon_sym_isize] = ACTIONS(2216),
    [anon_sym_usize] = ACTIONS(2216),
    [anon_sym_f32] = ACTIONS(2216),
    [anon_sym_f64] = ACTIONS(2216),
    [anon_sym_bool] = ACTIONS(2216),
    [anon_sym_str] = ACTIONS(2216),
    [anon_sym_char] = ACTIONS(2216),
    [anon_sym_DASH] = ACTIONS(2214),
    [anon_sym_AMP] = ACTIONS(2214),
    [anon_sym_PIPE] = ACTIONS(2214),
    [anon_sym_LT] = ACTIONS(2214),
    [anon_sym_DOT_DOT] = ACTIONS(2214),
    [anon_sym_COLON_COLON] = ACTIONS(2214),
    [anon_sym_POUND] = ACTIONS(2214),
    [anon_sym_SQUOTE] = ACTIONS(2216),
    [anon_sym_async] = ACTIONS(2216),
    [anon_sym_auto] = ACTIONS(2216),
    [anon_sym_await] = ACTIONS(2216),
    [anon_sym_become] = ACTIONS(2216),
    [anon_sym_break] = ACTIONS(2216),
    [anon_sym_const] = ACTIONS(2216),
    [anon_sym_continue] = ACTIONS(2216),
    [anon_sym_default] = ACTIONS(2216),
    [anon_sym_enum] = ACTIONS(2216),
    [anon_sym_fn] = ACTIONS(2216),
    [anon_sym_for] = ACTIONS(2216),
    [anon_sym_gen] = ACTIONS(2216),
    [anon_sym_if] = ACTIONS(2216),
    [anon_sym_impl] = ACTIONS(2216),
    [anon_sym_let] = ACTIONS(2216),
    [anon_sym_loop] = ACTIONS(2216),
    [anon_sym_match] = ACTIONS(2216),
    [anon_sym_mod] = ACTIONS(2216),
    [anon_sym_pub] = ACTIONS(2216),
    [anon_sym_return] = ACTIONS(2216),
    [anon_sym_raw] = ACTIONS(2216),
    [anon_sym_static] = ACTIONS(2216),
    [anon_sym_struct] = ACTIONS(2216),
    [anon_sym_trait] = ACTIONS(2216),
    [anon_sym_type] = ACTIONS(2216),
    [anon_sym_union] = ACTIONS(2216),
    [anon_sym_unsafe] = ACTIONS(2216),
    [anon_sym_use] = ACTIONS(2216),
    [anon_sym_while] = ACTIONS(2216),
    [anon_sym_extern] = ACTIONS(2216),
    [anon_sym_safe] = ACTIONS(2216),
    [anon_sym_yield] = ACTIONS(2216),
    [anon_sym_move] = ACTIONS(2216),
    [anon_sym_try] = ACTIONS(2216),
    [sym_integer_literal] = ACTIONS(2214),
    [aux_sym_string_literal_token1] = ACTIONS(2214),
    [sym_char_literal] = ACTIONS(2214),
    [anon_sym_true] = ACTIONS(2216),
    [anon_sym_false] = ACTIONS(2216),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2216),
    [sym_super] = ACTIONS(2216),
    [sym_crate] = ACTIONS(2216),
    [aux_sym_metavariable_token1] = ACTIONS(2214),
    [sym__raw_string_literal_start] = ACTIONS(2214),
    [sym_float_literal] = ACTIONS(2214),
    [sym__outer_block_doc_comment_start] = ACTIONS(2214),
    [sym__outer_line_doc_comment_start] = ACTIONS(2214),
  },
  [STATE(611)] = {
    [sym_line_comment] = STATE(611),
    [sym_block_comment] = STATE(611),
    [ts_builtin_sym_end] = ACTIONS(2218),
    [sym_identifier] = ACTIONS(2220),
    [anon_sym_SEMI] = ACTIONS(2218),
    [anon_sym_macro] = ACTIONS(2220),
    [anon_sym_LPAREN] = ACTIONS(2218),
    [anon_sym_LBRACE] = ACTIONS(2218),
    [anon_sym_RBRACE] = ACTIONS(2218),
    [anon_sym_macro_rules] = ACTIONS(2220),
    [anon_sym_BANG] = ACTIONS(2218),
    [anon_sym_LBRACK] = ACTIONS(2218),
    [anon_sym_STAR] = ACTIONS(2218),
    [anon_sym_u8] = ACTIONS(2220),
    [anon_sym_i8] = ACTIONS(2220),
    [anon_sym_u16] = ACTIONS(2220),
    [anon_sym_i16] = ACTIONS(2220),
    [anon_sym_u32] = ACTIONS(2220),
    [anon_sym_i32] = ACTIONS(2220),
    [anon_sym_u64] = ACTIONS(2220),
    [anon_sym_i64] = ACTIONS(2220),
    [anon_sym_u128] = ACTIONS(2220),
    [anon_sym_i128] = ACTIONS(2220),
    [anon_sym_isize] = ACTIONS(2220),
    [anon_sym_usize] = ACTIONS(2220),
    [anon_sym_f32] = ACTIONS(2220),
    [anon_sym_f64] = ACTIONS(2220),
    [anon_sym_bool] = ACTIONS(2220),
    [anon_sym_str] = ACTIONS(2220),
    [anon_sym_char] = ACTIONS(2220),
    [anon_sym_DASH] = ACTIONS(2218),
    [anon_sym_AMP] = ACTIONS(2218),
    [anon_sym_PIPE] = ACTIONS(2218),
    [anon_sym_LT] = ACTIONS(2218),
    [anon_sym_DOT_DOT] = ACTIONS(2218),
    [anon_sym_COLON_COLON] = ACTIONS(2218),
    [anon_sym_POUND] = ACTIONS(2218),
    [anon_sym_SQUOTE] = ACTIONS(2220),
    [anon_sym_async] = ACTIONS(2220),
    [anon_sym_auto] = ACTIONS(2220),
    [anon_sym_await] = ACTIONS(2220),
    [anon_sym_become] = ACTIONS(2220),
    [anon_sym_break] = ACTIONS(2220),
    [anon_sym_const] = ACTIONS(2220),
    [anon_sym_continue] = ACTIONS(2220),
    [anon_sym_default] = ACTIONS(2220),
    [anon_sym_enum] = ACTIONS(2220),
    [anon_sym_fn] = ACTIONS(2220),
    [anon_sym_for] = ACTIONS(2220),
    [anon_sym_gen] = ACTIONS(2220),
    [anon_sym_if] = ACTIONS(2220),
    [anon_sym_impl] = ACTIONS(2220),
    [anon_sym_let] = ACTIONS(2220),
    [anon_sym_loop] = ACTIONS(2220),
    [anon_sym_match] = ACTIONS(2220),
    [anon_sym_mod] = ACTIONS(2220),
    [anon_sym_pub] = ACTIONS(2220),
    [anon_sym_return] = ACTIONS(2220),
    [anon_sym_raw] = ACTIONS(2220),
    [anon_sym_static] = ACTIONS(2220),
    [anon_sym_struct] = ACTIONS(2220),
    [anon_sym_trait] = ACTIONS(2220),
    [anon_sym_type] = ACTIONS(2220),
    [anon_sym_union] = ACTIONS(2220),
    [anon_sym_unsafe] = ACTIONS(2220),
    [anon_sym_use] = ACTIONS(2220),
    [anon_sym_while] = ACTIONS(2220),
    [anon_sym_extern] = ACTIONS(2220),
    [anon_sym_safe] = ACTIONS(2220),
    [anon_sym_yield] = ACTIONS(2220),
    [anon_sym_move] = ACTIONS(2220),
    [anon_sym_try] = ACTIONS(2220),
    [sym_integer_literal] = ACTIONS(2218),
    [aux_sym_string_literal_token1] = ACTIONS(2218),
    [sym_char_literal] = ACTIONS(2218),
    [anon_sym_true] = ACTIONS(2220),
    [anon_sym_false] = ACTIONS(2220),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2220),
    [sym_super] = ACTIONS(2220),
    [sym_crate] = ACTIONS(2220),
    [aux_sym_metavariable_token1] = ACTIONS(2218),
    [sym__raw_string_literal_start] = ACTIONS(2218),
    [sym_float_literal] = ACTIONS(2218),
    [sym__outer_block_doc_comment_start] = ACTIONS(2218),
    [sym__outer_line_doc_comment_start] = ACTIONS(2218),
  },
  [STATE(612)] = {
    [sym_line_comment] = STATE(612),
    [sym_block_comment] = STATE(612),
    [ts_builtin_sym_end] = ACTIONS(2222),
    [sym_identifier] = ACTIONS(2224),
    [anon_sym_SEMI] = ACTIONS(2222),
    [anon_sym_macro] = ACTIONS(2224),
    [anon_sym_LPAREN] = ACTIONS(2222),
    [anon_sym_LBRACE] = ACTIONS(2222),
    [anon_sym_RBRACE] = ACTIONS(2222),
    [anon_sym_macro_rules] = ACTIONS(2224),
    [anon_sym_BANG] = ACTIONS(2222),
    [anon_sym_LBRACK] = ACTIONS(2222),
    [anon_sym_STAR] = ACTIONS(2222),
    [anon_sym_u8] = ACTIONS(2224),
    [anon_sym_i8] = ACTIONS(2224),
    [anon_sym_u16] = ACTIONS(2224),
    [anon_sym_i16] = ACTIONS(2224),
    [anon_sym_u32] = ACTIONS(2224),
    [anon_sym_i32] = ACTIONS(2224),
    [anon_sym_u64] = ACTIONS(2224),
    [anon_sym_i64] = ACTIONS(2224),
    [anon_sym_u128] = ACTIONS(2224),
    [anon_sym_i128] = ACTIONS(2224),
    [anon_sym_isize] = ACTIONS(2224),
    [anon_sym_usize] = ACTIONS(2224),
    [anon_sym_f32] = ACTIONS(2224),
    [anon_sym_f64] = ACTIONS(2224),
    [anon_sym_bool] = ACTIONS(2224),
    [anon_sym_str] = ACTIONS(2224),
    [anon_sym_char] = ACTIONS(2224),
    [anon_sym_DASH] = ACTIONS(2222),
    [anon_sym_AMP] = ACTIONS(2222),
    [anon_sym_PIPE] = ACTIONS(2222),
    [anon_sym_LT] = ACTIONS(2222),
    [anon_sym_DOT_DOT] = ACTIONS(2222),
    [anon_sym_COLON_COLON] = ACTIONS(2222),
    [anon_sym_POUND] = ACTIONS(2222),
    [anon_sym_SQUOTE] = ACTIONS(2224),
    [anon_sym_async] = ACTIONS(2224),
    [anon_sym_auto] = ACTIONS(2224),
    [anon_sym_await] = ACTIONS(2224),
    [anon_sym_become] = ACTIONS(2224),
    [anon_sym_break] = ACTIONS(2224),
    [anon_sym_const] = ACTIONS(2224),
    [anon_sym_continue] = ACTIONS(2224),
    [anon_sym_default] = ACTIONS(2224),
    [anon_sym_enum] = ACTIONS(2224),
    [anon_sym_fn] = ACTIONS(2224),
    [anon_sym_for] = ACTIONS(2224),
    [anon_sym_gen] = ACTIONS(2224),
    [anon_sym_if] = ACTIONS(2224),
    [anon_sym_impl] = ACTIONS(2224),
    [anon_sym_let] = ACTIONS(2224),
    [anon_sym_loop] = ACTIONS(2224),
    [anon_sym_match] = ACTIONS(2224),
    [anon_sym_mod] = ACTIONS(2224),
    [anon_sym_pub] = ACTIONS(2224),
    [anon_sym_return] = ACTIONS(2224),
    [anon_sym_raw] = ACTIONS(2224),
    [anon_sym_static] = ACTIONS(2224),
    [anon_sym_struct] = ACTIONS(2224),
    [anon_sym_trait] = ACTIONS(2224),
    [anon_sym_type] = ACTIONS(2224),
    [anon_sym_union] = ACTIONS(2224),
    [anon_sym_unsafe] = ACTIONS(2224),
    [anon_sym_use] = ACTIONS(2224),
    [anon_sym_while] = ACTIONS(2224),
    [anon_sym_extern] = ACTIONS(2224),
    [anon_sym_safe] = ACTIONS(2224),
    [anon_sym_yield] = ACTIONS(2224),
    [anon_sym_move] = ACTIONS(2224),
    [anon_sym_try] = ACTIONS(2224),
    [sym_integer_literal] = ACTIONS(2222),
    [aux_sym_string_literal_token1] = ACTIONS(2222),
    [sym_char_literal] = ACTIONS(2222),
    [anon_sym_true] = ACTIONS(2224),
    [anon_sym_false] = ACTIONS(2224),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2224),
    [sym_super] = ACTIONS(2224),
    [sym_crate] = ACTIONS(2224),
    [aux_sym_metavariable_token1] = ACTIONS(2222),
    [sym__raw_string_literal_start] = ACTIONS(2222),
    [sym_float_literal] = ACTIONS(2222),
    [sym__outer_block_doc_comment_start] = ACTIONS(2222),
    [sym__outer_line_doc_comment_start] = ACTIONS(2222),
  },
  [STATE(613)] = {
    [sym_line_comment] = STATE(613),
    [sym_block_comment] = STATE(613),
    [ts_builtin_sym_end] = ACTIONS(2226),
    [sym_identifier] = ACTIONS(2228),
    [anon_sym_SEMI] = ACTIONS(2226),
    [anon_sym_macro] = ACTIONS(2228),
    [anon_sym_LPAREN] = ACTIONS(2226),
    [anon_sym_LBRACE] = ACTIONS(2226),
    [anon_sym_RBRACE] = ACTIONS(2226),
    [anon_sym_macro_rules] = ACTIONS(2228),
    [anon_sym_BANG] = ACTIONS(2226),
    [anon_sym_LBRACK] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(2226),
    [anon_sym_u8] = ACTIONS(2228),
    [anon_sym_i8] = ACTIONS(2228),
    [anon_sym_u16] = ACTIONS(2228),
    [anon_sym_i16] = ACTIONS(2228),
    [anon_sym_u32] = ACTIONS(2228),
    [anon_sym_i32] = ACTIONS(2228),
    [anon_sym_u64] = ACTIONS(2228),
    [anon_sym_i64] = ACTIONS(2228),
    [anon_sym_u128] = ACTIONS(2228),
    [anon_sym_i128] = ACTIONS(2228),
    [anon_sym_isize] = ACTIONS(2228),
    [anon_sym_usize] = ACTIONS(2228),
    [anon_sym_f32] = ACTIONS(2228),
    [anon_sym_f64] = ACTIONS(2228),
    [anon_sym_bool] = ACTIONS(2228),
    [anon_sym_str] = ACTIONS(2228),
    [anon_sym_char] = ACTIONS(2228),
    [anon_sym_DASH] = ACTIONS(2226),
    [anon_sym_AMP] = ACTIONS(2226),
    [anon_sym_PIPE] = ACTIONS(2226),
    [anon_sym_LT] = ACTIONS(2226),
    [anon_sym_DOT_DOT] = ACTIONS(2226),
    [anon_sym_COLON_COLON] = ACTIONS(2226),
    [anon_sym_POUND] = ACTIONS(2226),
    [anon_sym_SQUOTE] = ACTIONS(2228),
    [anon_sym_async] = ACTIONS(2228),
    [anon_sym_auto] = ACTIONS(2228),
    [anon_sym_await] = ACTIONS(2228),
    [anon_sym_become] = ACTIONS(2228),
    [anon_sym_break] = ACTIONS(2228),
    [anon_sym_const] = ACTIONS(2228),
    [anon_sym_continue] = ACTIONS(2228),
    [anon_sym_default] = ACTIONS(2228),
    [anon_sym_enum] = ACTIONS(2228),
    [anon_sym_fn] = ACTIONS(2228),
    [anon_sym_for] = ACTIONS(2228),
    [anon_sym_gen] = ACTIONS(2228),
    [anon_sym_if] = ACTIONS(2228),
    [anon_sym_impl] = ACTIONS(2228),
    [anon_sym_let] = ACTIONS(2228),
    [anon_sym_loop] = ACTIONS(2228),
    [anon_sym_match] = ACTIONS(2228),
    [anon_sym_mod] = ACTIONS(2228),
    [anon_sym_pub] = ACTIONS(2228),
    [anon_sym_return] = ACTIONS(2228),
    [anon_sym_raw] = ACTIONS(2228),
    [anon_sym_static] = ACTIONS(2228),
    [anon_sym_struct] = ACTIONS(2228),
    [anon_sym_trait] = ACTIONS(2228),
    [anon_sym_type] = ACTIONS(2228),
    [anon_sym_union] = ACTIONS(2228),
    [anon_sym_unsafe] = ACTIONS(2228),
    [anon_sym_use] = ACTIONS(2228),
    [anon_sym_while] = ACTIONS(2228),
    [anon_sym_extern] = ACTIONS(2228),
    [anon_sym_safe] = ACTIONS(2228),
    [anon_sym_yield] = ACTIONS(2228),
    [anon_sym_move] = ACTIONS(2228),
    [anon_sym_try] = ACTIONS(2228),
    [sym_integer_literal] = ACTIONS(2226),
    [aux_sym_string_literal_token1] = ACTIONS(2226),
    [sym_char_literal] = ACTIONS(2226),
    [anon_sym_true] = ACTIONS(2228),
    [anon_sym_false] = ACTIONS(2228),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2228),
    [sym_super] = ACTIONS(2228),
    [sym_crate] = ACTIONS(2228),
    [aux_sym_metavariable_token1] = ACTIONS(2226),
    [sym__raw_string_literal_start] = ACTIONS(2226),
    [sym_float_literal] = ACTIONS(2226),
    [sym__outer_block_doc_comment_start] = ACTIONS(2226),
    [sym__outer_line_doc_comment_start] = ACTIONS(2226),
  },
  [STATE(614)] = {
    [sym_line_comment] = STATE(614),
    [sym_block_comment] = STATE(614),
    [ts_builtin_sym_end] = ACTIONS(2230),
    [sym_identifier] = ACTIONS(2232),
    [anon_sym_SEMI] = ACTIONS(2230),
    [anon_sym_macro] = ACTIONS(2232),
    [anon_sym_LPAREN] = ACTIONS(2230),
    [anon_sym_LBRACE] = ACTIONS(2230),
    [anon_sym_RBRACE] = ACTIONS(2230),
    [anon_sym_macro_rules] = ACTIONS(2232),
    [anon_sym_BANG] = ACTIONS(2230),
    [anon_sym_LBRACK] = ACTIONS(2230),
    [anon_sym_STAR] = ACTIONS(2230),
    [anon_sym_u8] = ACTIONS(2232),
    [anon_sym_i8] = ACTIONS(2232),
    [anon_sym_u16] = ACTIONS(2232),
    [anon_sym_i16] = ACTIONS(2232),
    [anon_sym_u32] = ACTIONS(2232),
    [anon_sym_i32] = ACTIONS(2232),
    [anon_sym_u64] = ACTIONS(2232),
    [anon_sym_i64] = ACTIONS(2232),
    [anon_sym_u128] = ACTIONS(2232),
    [anon_sym_i128] = ACTIONS(2232),
    [anon_sym_isize] = ACTIONS(2232),
    [anon_sym_usize] = ACTIONS(2232),
    [anon_sym_f32] = ACTIONS(2232),
    [anon_sym_f64] = ACTIONS(2232),
    [anon_sym_bool] = ACTIONS(2232),
    [anon_sym_str] = ACTIONS(2232),
    [anon_sym_char] = ACTIONS(2232),
    [anon_sym_DASH] = ACTIONS(2230),
    [anon_sym_AMP] = ACTIONS(2230),
    [anon_sym_PIPE] = ACTIONS(2230),
    [anon_sym_LT] = ACTIONS(2230),
    [anon_sym_DOT_DOT] = ACTIONS(2230),
    [anon_sym_COLON_COLON] = ACTIONS(2230),
    [anon_sym_POUND] = ACTIONS(2230),
    [anon_sym_SQUOTE] = ACTIONS(2232),
    [anon_sym_async] = ACTIONS(2232),
    [anon_sym_auto] = ACTIONS(2232),
    [anon_sym_await] = ACTIONS(2232),
    [anon_sym_become] = ACTIONS(2232),
    [anon_sym_break] = ACTIONS(2232),
    [anon_sym_const] = ACTIONS(2232),
    [anon_sym_continue] = ACTIONS(2232),
    [anon_sym_default] = ACTIONS(2232),
    [anon_sym_enum] = ACTIONS(2232),
    [anon_sym_fn] = ACTIONS(2232),
    [anon_sym_for] = ACTIONS(2232),
    [anon_sym_gen] = ACTIONS(2232),
    [anon_sym_if] = ACTIONS(2232),
    [anon_sym_impl] = ACTIONS(2232),
    [anon_sym_let] = ACTIONS(2232),
    [anon_sym_loop] = ACTIONS(2232),
    [anon_sym_match] = ACTIONS(2232),
    [anon_sym_mod] = ACTIONS(2232),
    [anon_sym_pub] = ACTIONS(2232),
    [anon_sym_return] = ACTIONS(2232),
    [anon_sym_raw] = ACTIONS(2232),
    [anon_sym_static] = ACTIONS(2232),
    [anon_sym_struct] = ACTIONS(2232),
    [anon_sym_trait] = ACTIONS(2232),
    [anon_sym_type] = ACTIONS(2232),
    [anon_sym_union] = ACTIONS(2232),
    [anon_sym_unsafe] = ACTIONS(2232),
    [anon_sym_use] = ACTIONS(2232),
    [anon_sym_while] = ACTIONS(2232),
    [anon_sym_extern] = ACTIONS(2232),
    [anon_sym_safe] = ACTIONS(2232),
    [anon_sym_yield] = ACTIONS(2232),
    [anon_sym_move] = ACTIONS(2232),
    [anon_sym_try] = ACTIONS(2232),
    [sym_integer_literal] = ACTIONS(2230),
    [aux_sym_string_literal_token1] = ACTIONS(2230),
    [sym_char_literal] = ACTIONS(2230),
    [anon_sym_true] = ACTIONS(2232),
    [anon_sym_false] = ACTIONS(2232),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2232),
    [sym_super] = ACTIONS(2232),
    [sym_crate] = ACTIONS(2232),
    [aux_sym_metavariable_token1] = ACTIONS(2230),
    [sym__raw_string_literal_start] = ACTIONS(2230),
    [sym_float_literal] = ACTIONS(2230),
    [sym__outer_block_doc_comment_start] = ACTIONS(2230),
    [sym__outer_line_doc_comment_start] = ACTIONS(2230),
  },
  [STATE(615)] = {
    [sym_line_comment] = STATE(615),
    [sym_block_comment] = STATE(615),
    [ts_builtin_sym_end] = ACTIONS(2234),
    [sym_identifier] = ACTIONS(2236),
    [anon_sym_SEMI] = ACTIONS(2234),
    [anon_sym_macro] = ACTIONS(2236),
    [anon_sym_LPAREN] = ACTIONS(2234),
    [anon_sym_LBRACE] = ACTIONS(2234),
    [anon_sym_RBRACE] = ACTIONS(2234),
    [anon_sym_macro_rules] = ACTIONS(2236),
    [anon_sym_BANG] = ACTIONS(2234),
    [anon_sym_LBRACK] = ACTIONS(2234),
    [anon_sym_STAR] = ACTIONS(2234),
    [anon_sym_u8] = ACTIONS(2236),
    [anon_sym_i8] = ACTIONS(2236),
    [anon_sym_u16] = ACTIONS(2236),
    [anon_sym_i16] = ACTIONS(2236),
    [anon_sym_u32] = ACTIONS(2236),
    [anon_sym_i32] = ACTIONS(2236),
    [anon_sym_u64] = ACTIONS(2236),
    [anon_sym_i64] = ACTIONS(2236),
    [anon_sym_u128] = ACTIONS(2236),
    [anon_sym_i128] = ACTIONS(2236),
    [anon_sym_isize] = ACTIONS(2236),
    [anon_sym_usize] = ACTIONS(2236),
    [anon_sym_f32] = ACTIONS(2236),
    [anon_sym_f64] = ACTIONS(2236),
    [anon_sym_bool] = ACTIONS(2236),
    [anon_sym_str] = ACTIONS(2236),
    [anon_sym_char] = ACTIONS(2236),
    [anon_sym_DASH] = ACTIONS(2234),
    [anon_sym_AMP] = ACTIONS(2234),
    [anon_sym_PIPE] = ACTIONS(2234),
    [anon_sym_LT] = ACTIONS(2234),
    [anon_sym_DOT_DOT] = ACTIONS(2234),
    [anon_sym_COLON_COLON] = ACTIONS(2234),
    [anon_sym_POUND] = ACTIONS(2234),
    [anon_sym_SQUOTE] = ACTIONS(2236),
    [anon_sym_async] = ACTIONS(2236),
    [anon_sym_auto] = ACTIONS(2236),
    [anon_sym_await] = ACTIONS(2236),
    [anon_sym_become] = ACTIONS(2236),
    [anon_sym_break] = ACTIONS(2236),
    [anon_sym_const] = ACTIONS(2236),
    [anon_sym_continue] = ACTIONS(2236),
    [anon_sym_default] = ACTIONS(2236),
    [anon_sym_enum] = ACTIONS(2236),
    [anon_sym_fn] = ACTIONS(2236),
    [anon_sym_for] = ACTIONS(2236),
    [anon_sym_gen] = ACTIONS(2236),
    [anon_sym_if] = ACTIONS(2236),
    [anon_sym_impl] = ACTIONS(2236),
    [anon_sym_let] = ACTIONS(2236),
    [anon_sym_loop] = ACTIONS(2236),
    [anon_sym_match] = ACTIONS(2236),
    [anon_sym_mod] = ACTIONS(2236),
    [anon_sym_pub] = ACTIONS(2236),
    [anon_sym_return] = ACTIONS(2236),
    [anon_sym_raw] = ACTIONS(2236),
    [anon_sym_static] = ACTIONS(2236),
    [anon_sym_struct] = ACTIONS(2236),
    [anon_sym_trait] = ACTIONS(2236),
    [anon_sym_type] = ACTIONS(2236),
    [anon_sym_union] = ACTIONS(2236),
    [anon_sym_unsafe] = ACTIONS(2236),
    [anon_sym_use] = ACTIONS(2236),
    [anon_sym_while] = ACTIONS(2236),
    [anon_sym_extern] = ACTIONS(2236),
    [anon_sym_safe] = ACTIONS(2236),
    [anon_sym_yield] = ACTIONS(2236),
    [anon_sym_move] = ACTIONS(2236),
    [anon_sym_try] = ACTIONS(2236),
    [sym_integer_literal] = ACTIONS(2234),
    [aux_sym_string_literal_token1] = ACTIONS(2234),
    [sym_char_literal] = ACTIONS(2234),
    [anon_sym_true] = ACTIONS(2236),
    [anon_sym_false] = ACTIONS(2236),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2236),
    [sym_super] = ACTIONS(2236),
    [sym_crate] = ACTIONS(2236),
    [aux_sym_metavariable_token1] = ACTIONS(2234),
    [sym__raw_string_literal_start] = ACTIONS(2234),
    [sym_float_literal] = ACTIONS(2234),
    [sym__outer_block_doc_comment_start] = ACTIONS(2234),
    [sym__outer_line_doc_comment_start] = ACTIONS(2234),
  },
  [STATE(616)] = {
    [sym_line_comment] = STATE(616),
    [sym_block_comment] = STATE(616),
    [ts_builtin_sym_end] = ACTIONS(2238),
    [sym_identifier] = ACTIONS(2240),
    [anon_sym_SEMI] = ACTIONS(2238),
    [anon_sym_macro] = ACTIONS(2240),
    [anon_sym_LPAREN] = ACTIONS(2238),
    [anon_sym_LBRACE] = ACTIONS(2238),
    [anon_sym_RBRACE] = ACTIONS(2238),
    [anon_sym_macro_rules] = ACTIONS(2240),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_LBRACK] = ACTIONS(2238),
    [anon_sym_STAR] = ACTIONS(2238),
    [anon_sym_u8] = ACTIONS(2240),
    [anon_sym_i8] = ACTIONS(2240),
    [anon_sym_u16] = ACTIONS(2240),
    [anon_sym_i16] = ACTIONS(2240),
    [anon_sym_u32] = ACTIONS(2240),
    [anon_sym_i32] = ACTIONS(2240),
    [anon_sym_u64] = ACTIONS(2240),
    [anon_sym_i64] = ACTIONS(2240),
    [anon_sym_u128] = ACTIONS(2240),
    [anon_sym_i128] = ACTIONS(2240),
    [anon_sym_isize] = ACTIONS(2240),
    [anon_sym_usize] = ACTIONS(2240),
    [anon_sym_f32] = ACTIONS(2240),
    [anon_sym_f64] = ACTIONS(2240),
    [anon_sym_bool] = ACTIONS(2240),
    [anon_sym_str] = ACTIONS(2240),
    [anon_sym_char] = ACTIONS(2240),
    [anon_sym_DASH] = ACTIONS(2238),
    [anon_sym_AMP] = ACTIONS(2238),
    [anon_sym_PIPE] = ACTIONS(2238),
    [anon_sym_LT] = ACTIONS(2238),
    [anon_sym_DOT_DOT] = ACTIONS(2238),
    [anon_sym_COLON_COLON] = ACTIONS(2238),
    [anon_sym_POUND] = ACTIONS(2238),
    [anon_sym_SQUOTE] = ACTIONS(2240),
    [anon_sym_async] = ACTIONS(2240),
    [anon_sym_auto] = ACTIONS(2240),
    [anon_sym_await] = ACTIONS(2240),
    [anon_sym_become] = ACTIONS(2240),
    [anon_sym_break] = ACTIONS(2240),
    [anon_sym_const] = ACTIONS(2240),
    [anon_sym_continue] = ACTIONS(2240),
    [anon_sym_default] = ACTIONS(2240),
    [anon_sym_enum] = ACTIONS(2240),
    [anon_sym_fn] = ACTIONS(2240),
    [anon_sym_for] = ACTIONS(2240),
    [anon_sym_gen] = ACTIONS(2240),
    [anon_sym_if] = ACTIONS(2240),
    [anon_sym_impl] = ACTIONS(2240),
    [anon_sym_let] = ACTIONS(2240),
    [anon_sym_loop] = ACTIONS(2240),
    [anon_sym_match] = ACTIONS(2240),
    [anon_sym_mod] = ACTIONS(2240),
    [anon_sym_pub] = ACTIONS(2240),
    [anon_sym_return] = ACTIONS(2240),
    [anon_sym_raw] = ACTIONS(2240),
    [anon_sym_static] = ACTIONS(2240),
    [anon_sym_struct] = ACTIONS(2240),
    [anon_sym_trait] = ACTIONS(2240),
    [anon_sym_type] = ACTIONS(2240),
    [anon_sym_union] = ACTIONS(2240),
    [anon_sym_unsafe] = ACTIONS(2240),
    [anon_sym_use] = ACTIONS(2240),
    [anon_sym_while] = ACTIONS(2240),
    [anon_sym_extern] = ACTIONS(2240),
    [anon_sym_safe] = ACTIONS(2240),
    [anon_sym_yield] = ACTIONS(2240),
    [anon_sym_move] = ACTIONS(2240),
    [anon_sym_try] = ACTIONS(2240),
    [sym_integer_literal] = ACTIONS(2238),
    [aux_sym_string_literal_token1] = ACTIONS(2238),
    [sym_char_literal] = ACTIONS(2238),
    [anon_sym_true] = ACTIONS(2240),
    [anon_sym_false] = ACTIONS(2240),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2240),
    [sym_super] = ACTIONS(2240),
    [sym_crate] = ACTIONS(2240),
    [aux_sym_metavariable_token1] = ACTIONS(2238),
    [sym__raw_string_literal_start] = ACTIONS(2238),
    [sym_float_literal] = ACTIONS(2238),
    [sym__outer_block_doc_comment_start] = ACTIONS(2238),
    [sym__outer_line_doc_comment_start] = ACTIONS(2238),
  },
  [STATE(617)] = {
    [sym_line_comment] = STATE(617),
    [sym_block_comment] = STATE(617),
    [ts_builtin_sym_end] = ACTIONS(2242),
    [sym_identifier] = ACTIONS(2244),
    [anon_sym_SEMI] = ACTIONS(2242),
    [anon_sym_macro] = ACTIONS(2244),
    [anon_sym_LPAREN] = ACTIONS(2242),
    [anon_sym_LBRACE] = ACTIONS(2242),
    [anon_sym_RBRACE] = ACTIONS(2242),
    [anon_sym_macro_rules] = ACTIONS(2244),
    [anon_sym_BANG] = ACTIONS(2242),
    [anon_sym_LBRACK] = ACTIONS(2242),
    [anon_sym_STAR] = ACTIONS(2242),
    [anon_sym_u8] = ACTIONS(2244),
    [anon_sym_i8] = ACTIONS(2244),
    [anon_sym_u16] = ACTIONS(2244),
    [anon_sym_i16] = ACTIONS(2244),
    [anon_sym_u32] = ACTIONS(2244),
    [anon_sym_i32] = ACTIONS(2244),
    [anon_sym_u64] = ACTIONS(2244),
    [anon_sym_i64] = ACTIONS(2244),
    [anon_sym_u128] = ACTIONS(2244),
    [anon_sym_i128] = ACTIONS(2244),
    [anon_sym_isize] = ACTIONS(2244),
    [anon_sym_usize] = ACTIONS(2244),
    [anon_sym_f32] = ACTIONS(2244),
    [anon_sym_f64] = ACTIONS(2244),
    [anon_sym_bool] = ACTIONS(2244),
    [anon_sym_str] = ACTIONS(2244),
    [anon_sym_char] = ACTIONS(2244),
    [anon_sym_DASH] = ACTIONS(2242),
    [anon_sym_AMP] = ACTIONS(2242),
    [anon_sym_PIPE] = ACTIONS(2242),
    [anon_sym_LT] = ACTIONS(2242),
    [anon_sym_DOT_DOT] = ACTIONS(2242),
    [anon_sym_COLON_COLON] = ACTIONS(2242),
    [anon_sym_POUND] = ACTIONS(2242),
    [anon_sym_SQUOTE] = ACTIONS(2244),
    [anon_sym_async] = ACTIONS(2244),
    [anon_sym_auto] = ACTIONS(2244),
    [anon_sym_await] = ACTIONS(2244),
    [anon_sym_become] = ACTIONS(2244),
    [anon_sym_break] = ACTIONS(2244),
    [anon_sym_const] = ACTIONS(2244),
    [anon_sym_continue] = ACTIONS(2244),
    [anon_sym_default] = ACTIONS(2244),
    [anon_sym_enum] = ACTIONS(2244),
    [anon_sym_fn] = ACTIONS(2244),
    [anon_sym_for] = ACTIONS(2244),
    [anon_sym_gen] = ACTIONS(2244),
    [anon_sym_if] = ACTIONS(2244),
    [anon_sym_impl] = ACTIONS(2244),
    [anon_sym_let] = ACTIONS(2244),
    [anon_sym_loop] = ACTIONS(2244),
    [anon_sym_match] = ACTIONS(2244),
    [anon_sym_mod] = ACTIONS(2244),
    [anon_sym_pub] = ACTIONS(2244),
    [anon_sym_return] = ACTIONS(2244),
    [anon_sym_raw] = ACTIONS(2244),
    [anon_sym_static] = ACTIONS(2244),
    [anon_sym_struct] = ACTIONS(2244),
    [anon_sym_trait] = ACTIONS(2244),
    [anon_sym_type] = ACTIONS(2244),
    [anon_sym_union] = ACTIONS(2244),
    [anon_sym_unsafe] = ACTIONS(2244),
    [anon_sym_use] = ACTIONS(2244),
    [anon_sym_while] = ACTIONS(2244),
    [anon_sym_extern] = ACTIONS(2244),
    [anon_sym_safe] = ACTIONS(2244),
    [anon_sym_yield] = ACTIONS(2244),
    [anon_sym_move] = ACTIONS(2244),
    [anon_sym_try] = ACTIONS(2244),
    [sym_integer_literal] = ACTIONS(2242),
    [aux_sym_string_literal_token1] = ACTIONS(2242),
    [sym_char_literal] = ACTIONS(2242),
    [anon_sym_true] = ACTIONS(2244),
    [anon_sym_false] = ACTIONS(2244),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2244),
    [sym_super] = ACTIONS(2244),
    [sym_crate] = ACTIONS(2244),
    [aux_sym_metavariable_token1] = ACTIONS(2242),
    [sym__raw_string_literal_start] = ACTIONS(2242),
    [sym_float_literal] = ACTIONS(2242),
    [sym__outer_block_doc_comment_start] = ACTIONS(2242),
    [sym__outer_line_doc_comment_start] = ACTIONS(2242),
  },
  [STATE(618)] = {
    [sym_line_comment] = STATE(618),
    [sym_block_comment] = STATE(618),
    [ts_builtin_sym_end] = ACTIONS(2246),
    [sym_identifier] = ACTIONS(2248),
    [anon_sym_SEMI] = ACTIONS(2246),
    [anon_sym_macro] = ACTIONS(2248),
    [anon_sym_LPAREN] = ACTIONS(2246),
    [anon_sym_LBRACE] = ACTIONS(2246),
    [anon_sym_RBRACE] = ACTIONS(2246),
    [anon_sym_macro_rules] = ACTIONS(2248),
    [anon_sym_BANG] = ACTIONS(2246),
    [anon_sym_LBRACK] = ACTIONS(2246),
    [anon_sym_STAR] = ACTIONS(2246),
    [anon_sym_u8] = ACTIONS(2248),
    [anon_sym_i8] = ACTIONS(2248),
    [anon_sym_u16] = ACTIONS(2248),
    [anon_sym_i16] = ACTIONS(2248),
    [anon_sym_u32] = ACTIONS(2248),
    [anon_sym_i32] = ACTIONS(2248),
    [anon_sym_u64] = ACTIONS(2248),
    [anon_sym_i64] = ACTIONS(2248),
    [anon_sym_u128] = ACTIONS(2248),
    [anon_sym_i128] = ACTIONS(2248),
    [anon_sym_isize] = ACTIONS(2248),
    [anon_sym_usize] = ACTIONS(2248),
    [anon_sym_f32] = ACTIONS(2248),
    [anon_sym_f64] = ACTIONS(2248),
    [anon_sym_bool] = ACTIONS(2248),
    [anon_sym_str] = ACTIONS(2248),
    [anon_sym_char] = ACTIONS(2248),
    [anon_sym_DASH] = ACTIONS(2246),
    [anon_sym_AMP] = ACTIONS(2246),
    [anon_sym_PIPE] = ACTIONS(2246),
    [anon_sym_LT] = ACTIONS(2246),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_COLON_COLON] = ACTIONS(2246),
    [anon_sym_POUND] = ACTIONS(2246),
    [anon_sym_SQUOTE] = ACTIONS(2248),
    [anon_sym_async] = ACTIONS(2248),
    [anon_sym_auto] = ACTIONS(2248),
    [anon_sym_await] = ACTIONS(2248),
    [anon_sym_become] = ACTIONS(2248),
    [anon_sym_break] = ACTIONS(2248),
    [anon_sym_const] = ACTIONS(2248),
    [anon_sym_continue] = ACTIONS(2248),
    [anon_sym_default] = ACTIONS(2248),
    [anon_sym_enum] = ACTIONS(2248),
    [anon_sym_fn] = ACTIONS(2248),
    [anon_sym_for] = ACTIONS(2248),
    [anon_sym_gen] = ACTIONS(2248),
    [anon_sym_if] = ACTIONS(2248),
    [anon_sym_impl] = ACTIONS(2248),
    [anon_sym_let] = ACTIONS(2248),
    [anon_sym_loop] = ACTIONS(2248),
    [anon_sym_match] = ACTIONS(2248),
    [anon_sym_mod] = ACTIONS(2248),
    [anon_sym_pub] = ACTIONS(2248),
    [anon_sym_return] = ACTIONS(2248),
    [anon_sym_raw] = ACTIONS(2248),
    [anon_sym_static] = ACTIONS(2248),
    [anon_sym_struct] = ACTIONS(2248),
    [anon_sym_trait] = ACTIONS(2248),
    [anon_sym_type] = ACTIONS(2248),
    [anon_sym_union] = ACTIONS(2248),
    [anon_sym_unsafe] = ACTIONS(2248),
    [anon_sym_use] = ACTIONS(2248),
    [anon_sym_while] = ACTIONS(2248),
    [anon_sym_extern] = ACTIONS(2248),
    [anon_sym_safe] = ACTIONS(2248),
    [anon_sym_yield] = ACTIONS(2248),
    [anon_sym_move] = ACTIONS(2248),
    [anon_sym_try] = ACTIONS(2248),
    [sym_integer_literal] = ACTIONS(2246),
    [aux_sym_string_literal_token1] = ACTIONS(2246),
    [sym_char_literal] = ACTIONS(2246),
    [anon_sym_true] = ACTIONS(2248),
    [anon_sym_false] = ACTIONS(2248),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2248),
    [sym_super] = ACTIONS(2248),
    [sym_crate] = ACTIONS(2248),
    [aux_sym_metavariable_token1] = ACTIONS(2246),
    [sym__raw_string_literal_start] = ACTIONS(2246),
    [sym_float_literal] = ACTIONS(2246),
    [sym__outer_block_doc_comment_start] = ACTIONS(2246),
    [sym__outer_line_doc_comment_start] = ACTIONS(2246),
  },
  [STATE(619)] = {
    [sym_line_comment] = STATE(619),
    [sym_block_comment] = STATE(619),
    [ts_builtin_sym_end] = ACTIONS(2250),
    [sym_identifier] = ACTIONS(2252),
    [anon_sym_SEMI] = ACTIONS(2250),
    [anon_sym_macro] = ACTIONS(2252),
    [anon_sym_LPAREN] = ACTIONS(2250),
    [anon_sym_LBRACE] = ACTIONS(2250),
    [anon_sym_RBRACE] = ACTIONS(2250),
    [anon_sym_macro_rules] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2250),
    [anon_sym_LBRACK] = ACTIONS(2250),
    [anon_sym_STAR] = ACTIONS(2250),
    [anon_sym_u8] = ACTIONS(2252),
    [anon_sym_i8] = ACTIONS(2252),
    [anon_sym_u16] = ACTIONS(2252),
    [anon_sym_i16] = ACTIONS(2252),
    [anon_sym_u32] = ACTIONS(2252),
    [anon_sym_i32] = ACTIONS(2252),
    [anon_sym_u64] = ACTIONS(2252),
    [anon_sym_i64] = ACTIONS(2252),
    [anon_sym_u128] = ACTIONS(2252),
    [anon_sym_i128] = ACTIONS(2252),
    [anon_sym_isize] = ACTIONS(2252),
    [anon_sym_usize] = ACTIONS(2252),
    [anon_sym_f32] = ACTIONS(2252),
    [anon_sym_f64] = ACTIONS(2252),
    [anon_sym_bool] = ACTIONS(2252),
    [anon_sym_str] = ACTIONS(2252),
    [anon_sym_char] = ACTIONS(2252),
    [anon_sym_DASH] = ACTIONS(2250),
    [anon_sym_AMP] = ACTIONS(2250),
    [anon_sym_PIPE] = ACTIONS(2250),
    [anon_sym_LT] = ACTIONS(2250),
    [anon_sym_DOT_DOT] = ACTIONS(2250),
    [anon_sym_COLON_COLON] = ACTIONS(2250),
    [anon_sym_POUND] = ACTIONS(2250),
    [anon_sym_SQUOTE] = ACTIONS(2252),
    [anon_sym_async] = ACTIONS(2252),
    [anon_sym_auto] = ACTIONS(2252),
    [anon_sym_await] = ACTIONS(2252),
    [anon_sym_become] = ACTIONS(2252),
    [anon_sym_break] = ACTIONS(2252),
    [anon_sym_const] = ACTIONS(2252),
    [anon_sym_continue] = ACTIONS(2252),
    [anon_sym_default] = ACTIONS(2252),
    [anon_sym_enum] = ACTIONS(2252),
    [anon_sym_fn] = ACTIONS(2252),
    [anon_sym_for] = ACTIONS(2252),
    [anon_sym_gen] = ACTIONS(2252),
    [anon_sym_if] = ACTIONS(2252),
    [anon_sym_impl] = ACTIONS(2252),
    [anon_sym_let] = ACTIONS(2252),
    [anon_sym_loop] = ACTIONS(2252),
    [anon_sym_match] = ACTIONS(2252),
    [anon_sym_mod] = ACTIONS(2252),
    [anon_sym_pub] = ACTIONS(2252),
    [anon_sym_return] = ACTIONS(2252),
    [anon_sym_raw] = ACTIONS(2252),
    [anon_sym_static] = ACTIONS(2252),
    [anon_sym_struct] = ACTIONS(2252),
    [anon_sym_trait] = ACTIONS(2252),
    [anon_sym_type] = ACTIONS(2252),
    [anon_sym_union] = ACTIONS(2252),
    [anon_sym_unsafe] = ACTIONS(2252),
    [anon_sym_use] = ACTIONS(2252),
    [anon_sym_while] = ACTIONS(2252),
    [anon_sym_extern] = ACTIONS(2252),
    [anon_sym_safe] = ACTIONS(2252),
    [anon_sym_yield] = ACTIONS(2252),
    [anon_sym_move] = ACTIONS(2252),
    [anon_sym_try] = ACTIONS(2252),
    [sym_integer_literal] = ACTIONS(2250),
    [aux_sym_string_literal_token1] = ACTIONS(2250),
    [sym_char_literal] = ACTIONS(2250),
    [anon_sym_true] = ACTIONS(2252),
    [anon_sym_false] = ACTIONS(2252),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2252),
    [sym_super] = ACTIONS(2252),
    [sym_crate] = ACTIONS(2252),
    [aux_sym_metavariable_token1] = ACTIONS(2250),
    [sym__raw_string_literal_start] = ACTIONS(2250),
    [sym_float_literal] = ACTIONS(2250),
    [sym__outer_block_doc_comment_start] = ACTIONS(2250),
    [sym__outer_line_doc_comment_start] = ACTIONS(2250),
  },
  [STATE(620)] = {
    [sym_line_comment] = STATE(620),
    [sym_block_comment] = STATE(620),
    [ts_builtin_sym_end] = ACTIONS(2254),
    [sym_identifier] = ACTIONS(2256),
    [anon_sym_SEMI] = ACTIONS(2254),
    [anon_sym_macro] = ACTIONS(2256),
    [anon_sym_LPAREN] = ACTIONS(2254),
    [anon_sym_LBRACE] = ACTIONS(2254),
    [anon_sym_RBRACE] = ACTIONS(2254),
    [anon_sym_macro_rules] = ACTIONS(2256),
    [anon_sym_BANG] = ACTIONS(2254),
    [anon_sym_LBRACK] = ACTIONS(2254),
    [anon_sym_STAR] = ACTIONS(2254),
    [anon_sym_u8] = ACTIONS(2256),
    [anon_sym_i8] = ACTIONS(2256),
    [anon_sym_u16] = ACTIONS(2256),
    [anon_sym_i16] = ACTIONS(2256),
    [anon_sym_u32] = ACTIONS(2256),
    [anon_sym_i32] = ACTIONS(2256),
    [anon_sym_u64] = ACTIONS(2256),
    [anon_sym_i64] = ACTIONS(2256),
    [anon_sym_u128] = ACTIONS(2256),
    [anon_sym_i128] = ACTIONS(2256),
    [anon_sym_isize] = ACTIONS(2256),
    [anon_sym_usize] = ACTIONS(2256),
    [anon_sym_f32] = ACTIONS(2256),
    [anon_sym_f64] = ACTIONS(2256),
    [anon_sym_bool] = ACTIONS(2256),
    [anon_sym_str] = ACTIONS(2256),
    [anon_sym_char] = ACTIONS(2256),
    [anon_sym_DASH] = ACTIONS(2254),
    [anon_sym_AMP] = ACTIONS(2254),
    [anon_sym_PIPE] = ACTIONS(2254),
    [anon_sym_LT] = ACTIONS(2254),
    [anon_sym_DOT_DOT] = ACTIONS(2254),
    [anon_sym_COLON_COLON] = ACTIONS(2254),
    [anon_sym_POUND] = ACTIONS(2254),
    [anon_sym_SQUOTE] = ACTIONS(2256),
    [anon_sym_async] = ACTIONS(2256),
    [anon_sym_auto] = ACTIONS(2256),
    [anon_sym_await] = ACTIONS(2256),
    [anon_sym_become] = ACTIONS(2256),
    [anon_sym_break] = ACTIONS(2256),
    [anon_sym_const] = ACTIONS(2256),
    [anon_sym_continue] = ACTIONS(2256),
    [anon_sym_default] = ACTIONS(2256),
    [anon_sym_enum] = ACTIONS(2256),
    [anon_sym_fn] = ACTIONS(2256),
    [anon_sym_for] = ACTIONS(2256),
    [anon_sym_gen] = ACTIONS(2256),
    [anon_sym_if] = ACTIONS(2256),
    [anon_sym_impl] = ACTIONS(2256),
    [anon_sym_let] = ACTIONS(2256),
    [anon_sym_loop] = ACTIONS(2256),
    [anon_sym_match] = ACTIONS(2256),
    [anon_sym_mod] = ACTIONS(2256),
    [anon_sym_pub] = ACTIONS(2256),
    [anon_sym_return] = ACTIONS(2256),
    [anon_sym_raw] = ACTIONS(2256),
    [anon_sym_static] = ACTIONS(2256),
    [anon_sym_struct] = ACTIONS(2256),
    [anon_sym_trait] = ACTIONS(2256),
    [anon_sym_type] = ACTIONS(2256),
    [anon_sym_union] = ACTIONS(2256),
    [anon_sym_unsafe] = ACTIONS(2256),
    [anon_sym_use] = ACTIONS(2256),
    [anon_sym_while] = ACTIONS(2256),
    [anon_sym_extern] = ACTIONS(2256),
    [anon_sym_safe] = ACTIONS(2256),
    [anon_sym_yield] = ACTIONS(2256),
    [anon_sym_move] = ACTIONS(2256),
    [anon_sym_try] = ACTIONS(2256),
    [sym_integer_literal] = ACTIONS(2254),
    [aux_sym_string_literal_token1] = ACTIONS(2254),
    [sym_char_literal] = ACTIONS(2254),
    [anon_sym_true] = ACTIONS(2256),
    [anon_sym_false] = ACTIONS(2256),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2256),
    [sym_super] = ACTIONS(2256),
    [sym_crate] = ACTIONS(2256),
    [aux_sym_metavariable_token1] = ACTIONS(2254),
    [sym__raw_string_literal_start] = ACTIONS(2254),
    [sym_float_literal] = ACTIONS(2254),
    [sym__outer_block_doc_comment_start] = ACTIONS(2254),
    [sym__outer_line_doc_comment_start] = ACTIONS(2254),
  },
  [STATE(621)] = {
    [sym_line_comment] = STATE(621),
    [sym_block_comment] = STATE(621),
    [ts_builtin_sym_end] = ACTIONS(2258),
    [sym_identifier] = ACTIONS(2260),
    [anon_sym_SEMI] = ACTIONS(2258),
    [anon_sym_macro] = ACTIONS(2260),
    [anon_sym_LPAREN] = ACTIONS(2258),
    [anon_sym_LBRACE] = ACTIONS(2258),
    [anon_sym_RBRACE] = ACTIONS(2258),
    [anon_sym_macro_rules] = ACTIONS(2260),
    [anon_sym_BANG] = ACTIONS(2258),
    [anon_sym_LBRACK] = ACTIONS(2258),
    [anon_sym_STAR] = ACTIONS(2258),
    [anon_sym_u8] = ACTIONS(2260),
    [anon_sym_i8] = ACTIONS(2260),
    [anon_sym_u16] = ACTIONS(2260),
    [anon_sym_i16] = ACTIONS(2260),
    [anon_sym_u32] = ACTIONS(2260),
    [anon_sym_i32] = ACTIONS(2260),
    [anon_sym_u64] = ACTIONS(2260),
    [anon_sym_i64] = ACTIONS(2260),
    [anon_sym_u128] = ACTIONS(2260),
    [anon_sym_i128] = ACTIONS(2260),
    [anon_sym_isize] = ACTIONS(2260),
    [anon_sym_usize] = ACTIONS(2260),
    [anon_sym_f32] = ACTIONS(2260),
    [anon_sym_f64] = ACTIONS(2260),
    [anon_sym_bool] = ACTIONS(2260),
    [anon_sym_str] = ACTIONS(2260),
    [anon_sym_char] = ACTIONS(2260),
    [anon_sym_DASH] = ACTIONS(2258),
    [anon_sym_AMP] = ACTIONS(2258),
    [anon_sym_PIPE] = ACTIONS(2258),
    [anon_sym_LT] = ACTIONS(2258),
    [anon_sym_DOT_DOT] = ACTIONS(2258),
    [anon_sym_COLON_COLON] = ACTIONS(2258),
    [anon_sym_POUND] = ACTIONS(2258),
    [anon_sym_SQUOTE] = ACTIONS(2260),
    [anon_sym_async] = ACTIONS(2260),
    [anon_sym_auto] = ACTIONS(2260),
    [anon_sym_await] = ACTIONS(2260),
    [anon_sym_become] = ACTIONS(2260),
    [anon_sym_break] = ACTIONS(2260),
    [anon_sym_const] = ACTIONS(2260),
    [anon_sym_continue] = ACTIONS(2260),
    [anon_sym_default] = ACTIONS(2260),
    [anon_sym_enum] = ACTIONS(2260),
    [anon_sym_fn] = ACTIONS(2260),
    [anon_sym_for] = ACTIONS(2260),
    [anon_sym_gen] = ACTIONS(2260),
    [anon_sym_if] = ACTIONS(2260),
    [anon_sym_impl] = ACTIONS(2260),
    [anon_sym_let] = ACTIONS(2260),
    [anon_sym_loop] = ACTIONS(2260),
    [anon_sym_match] = ACTIONS(2260),
    [anon_sym_mod] = ACTIONS(2260),
    [anon_sym_pub] = ACTIONS(2260),
    [anon_sym_return] = ACTIONS(2260),
    [anon_sym_raw] = ACTIONS(2260),
    [anon_sym_static] = ACTIONS(2260),
    [anon_sym_struct] = ACTIONS(2260),
    [anon_sym_trait] = ACTIONS(2260),
    [anon_sym_type] = ACTIONS(2260),
    [anon_sym_union] = ACTIONS(2260),
    [anon_sym_unsafe] = ACTIONS(2260),
    [anon_sym_use] = ACTIONS(2260),
    [anon_sym_while] = ACTIONS(2260),
    [anon_sym_extern] = ACTIONS(2260),
    [anon_sym_safe] = ACTIONS(2260),
    [anon_sym_yield] = ACTIONS(2260),
    [anon_sym_move] = ACTIONS(2260),
    [anon_sym_try] = ACTIONS(2260),
    [sym_integer_literal] = ACTIONS(2258),
    [aux_sym_string_literal_token1] = ACTIONS(2258),
    [sym_char_literal] = ACTIONS(2258),
    [anon_sym_true] = ACTIONS(2260),
    [anon_sym_false] = ACTIONS(2260),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2260),
    [sym_super] = ACTIONS(2260),
    [sym_crate] = ACTIONS(2260),
    [aux_sym_metavariable_token1] = ACTIONS(2258),
    [sym__raw_string_literal_start] = ACTIONS(2258),
    [sym_float_literal] = ACTIONS(2258),
    [sym__outer_block_doc_comment_start] = ACTIONS(2258),
    [sym__outer_line_doc_comment_start] = ACTIONS(2258),
  },
  [STATE(622)] = {
    [sym_line_comment] = STATE(622),
    [sym_block_comment] = STATE(622),
    [ts_builtin_sym_end] = ACTIONS(2262),
    [sym_identifier] = ACTIONS(2264),
    [anon_sym_SEMI] = ACTIONS(2262),
    [anon_sym_macro] = ACTIONS(2264),
    [anon_sym_LPAREN] = ACTIONS(2262),
    [anon_sym_LBRACE] = ACTIONS(2262),
    [anon_sym_RBRACE] = ACTIONS(2262),
    [anon_sym_macro_rules] = ACTIONS(2264),
    [anon_sym_BANG] = ACTIONS(2262),
    [anon_sym_LBRACK] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(2262),
    [anon_sym_u8] = ACTIONS(2264),
    [anon_sym_i8] = ACTIONS(2264),
    [anon_sym_u16] = ACTIONS(2264),
    [anon_sym_i16] = ACTIONS(2264),
    [anon_sym_u32] = ACTIONS(2264),
    [anon_sym_i32] = ACTIONS(2264),
    [anon_sym_u64] = ACTIONS(2264),
    [anon_sym_i64] = ACTIONS(2264),
    [anon_sym_u128] = ACTIONS(2264),
    [anon_sym_i128] = ACTIONS(2264),
    [anon_sym_isize] = ACTIONS(2264),
    [anon_sym_usize] = ACTIONS(2264),
    [anon_sym_f32] = ACTIONS(2264),
    [anon_sym_f64] = ACTIONS(2264),
    [anon_sym_bool] = ACTIONS(2264),
    [anon_sym_str] = ACTIONS(2264),
    [anon_sym_char] = ACTIONS(2264),
    [anon_sym_DASH] = ACTIONS(2262),
    [anon_sym_AMP] = ACTIONS(2262),
    [anon_sym_PIPE] = ACTIONS(2262),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_DOT_DOT] = ACTIONS(2262),
    [anon_sym_COLON_COLON] = ACTIONS(2262),
    [anon_sym_POUND] = ACTIONS(2262),
    [anon_sym_SQUOTE] = ACTIONS(2264),
    [anon_sym_async] = ACTIONS(2264),
    [anon_sym_auto] = ACTIONS(2264),
    [anon_sym_await] = ACTIONS(2264),
    [anon_sym_become] = ACTIONS(2264),
    [anon_sym_break] = ACTIONS(2264),
    [anon_sym_const] = ACTIONS(2264),
    [anon_sym_continue] = ACTIONS(2264),
    [anon_sym_default] = ACTIONS(2264),
    [anon_sym_enum] = ACTIONS(2264),
    [anon_sym_fn] = ACTIONS(2264),
    [anon_sym_for] = ACTIONS(2264),
    [anon_sym_gen] = ACTIONS(2264),
    [anon_sym_if] = ACTIONS(2264),
    [anon_sym_impl] = ACTIONS(2264),
    [anon_sym_let] = ACTIONS(2264),
    [anon_sym_loop] = ACTIONS(2264),
    [anon_sym_match] = ACTIONS(2264),
    [anon_sym_mod] = ACTIONS(2264),
    [anon_sym_pub] = ACTIONS(2264),
    [anon_sym_return] = ACTIONS(2264),
    [anon_sym_raw] = ACTIONS(2264),
    [anon_sym_static] = ACTIONS(2264),
    [anon_sym_struct] = ACTIONS(2264),
    [anon_sym_trait] = ACTIONS(2264),
    [anon_sym_type] = ACTIONS(2264),
    [anon_sym_union] = ACTIONS(2264),
    [anon_sym_unsafe] = ACTIONS(2264),
    [anon_sym_use] = ACTIONS(2264),
    [anon_sym_while] = ACTIONS(2264),
    [anon_sym_extern] = ACTIONS(2264),
    [anon_sym_safe] = ACTIONS(2264),
    [anon_sym_yield] = ACTIONS(2264),
    [anon_sym_move] = ACTIONS(2264),
    [anon_sym_try] = ACTIONS(2264),
    [sym_integer_literal] = ACTIONS(2262),
    [aux_sym_string_literal_token1] = ACTIONS(2262),
    [sym_char_literal] = ACTIONS(2262),
    [anon_sym_true] = ACTIONS(2264),
    [anon_sym_false] = ACTIONS(2264),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2264),
    [sym_super] = ACTIONS(2264),
    [sym_crate] = ACTIONS(2264),
    [aux_sym_metavariable_token1] = ACTIONS(2262),
    [sym__raw_string_literal_start] = ACTIONS(2262),
    [sym_float_literal] = ACTIONS(2262),
    [sym__outer_block_doc_comment_start] = ACTIONS(2262),
    [sym__outer_line_doc_comment_start] = ACTIONS(2262),
  },
  [STATE(623)] = {
    [sym_line_comment] = STATE(623),
    [sym_block_comment] = STATE(623),
    [ts_builtin_sym_end] = ACTIONS(2266),
    [sym_identifier] = ACTIONS(2268),
    [anon_sym_SEMI] = ACTIONS(2266),
    [anon_sym_macro] = ACTIONS(2268),
    [anon_sym_LPAREN] = ACTIONS(2266),
    [anon_sym_LBRACE] = ACTIONS(2266),
    [anon_sym_RBRACE] = ACTIONS(2266),
    [anon_sym_macro_rules] = ACTIONS(2268),
    [anon_sym_BANG] = ACTIONS(2266),
    [anon_sym_LBRACK] = ACTIONS(2266),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_u8] = ACTIONS(2268),
    [anon_sym_i8] = ACTIONS(2268),
    [anon_sym_u16] = ACTIONS(2268),
    [anon_sym_i16] = ACTIONS(2268),
    [anon_sym_u32] = ACTIONS(2268),
    [anon_sym_i32] = ACTIONS(2268),
    [anon_sym_u64] = ACTIONS(2268),
    [anon_sym_i64] = ACTIONS(2268),
    [anon_sym_u128] = ACTIONS(2268),
    [anon_sym_i128] = ACTIONS(2268),
    [anon_sym_isize] = ACTIONS(2268),
    [anon_sym_usize] = ACTIONS(2268),
    [anon_sym_f32] = ACTIONS(2268),
    [anon_sym_f64] = ACTIONS(2268),
    [anon_sym_bool] = ACTIONS(2268),
    [anon_sym_str] = ACTIONS(2268),
    [anon_sym_char] = ACTIONS(2268),
    [anon_sym_DASH] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2266),
    [anon_sym_PIPE] = ACTIONS(2266),
    [anon_sym_LT] = ACTIONS(2266),
    [anon_sym_DOT_DOT] = ACTIONS(2266),
    [anon_sym_COLON_COLON] = ACTIONS(2266),
    [anon_sym_POUND] = ACTIONS(2266),
    [anon_sym_SQUOTE] = ACTIONS(2268),
    [anon_sym_async] = ACTIONS(2268),
    [anon_sym_auto] = ACTIONS(2268),
    [anon_sym_await] = ACTIONS(2268),
    [anon_sym_become] = ACTIONS(2268),
    [anon_sym_break] = ACTIONS(2268),
    [anon_sym_const] = ACTIONS(2268),
    [anon_sym_continue] = ACTIONS(2268),
    [anon_sym_default] = ACTIONS(2268),
    [anon_sym_enum] = ACTIONS(2268),
    [anon_sym_fn] = ACTIONS(2268),
    [anon_sym_for] = ACTIONS(2268),
    [anon_sym_gen] = ACTIONS(2268),
    [anon_sym_if] = ACTIONS(2268),
    [anon_sym_impl] = ACTIONS(2268),
    [anon_sym_let] = ACTIONS(2268),
    [anon_sym_loop] = ACTIONS(2268),
    [anon_sym_match] = ACTIONS(2268),
    [anon_sym_mod] = ACTIONS(2268),
    [anon_sym_pub] = ACTIONS(2268),
    [anon_sym_return] = ACTIONS(2268),
    [anon_sym_raw] = ACTIONS(2268),
    [anon_sym_static] = ACTIONS(2268),
    [anon_sym_struct] = ACTIONS(2268),
    [anon_sym_trait] = ACTIONS(2268),
    [anon_sym_type] = ACTIONS(2268),
    [anon_sym_union] = ACTIONS(2268),
    [anon_sym_unsafe] = ACTIONS(2268),
    [anon_sym_use] = ACTIONS(2268),
    [anon_sym_while] = ACTIONS(2268),
    [anon_sym_extern] = ACTIONS(2268),
    [anon_sym_safe] = ACTIONS(2268),
    [anon_sym_yield] = ACTIONS(2268),
    [anon_sym_move] = ACTIONS(2268),
    [anon_sym_try] = ACTIONS(2268),
    [sym_integer_literal] = ACTIONS(2266),
    [aux_sym_string_literal_token1] = ACTIONS(2266),
    [sym_char_literal] = ACTIONS(2266),
    [anon_sym_true] = ACTIONS(2268),
    [anon_sym_false] = ACTIONS(2268),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2268),
    [sym_super] = ACTIONS(2268),
    [sym_crate] = ACTIONS(2268),
    [aux_sym_metavariable_token1] = ACTIONS(2266),
    [sym__raw_string_literal_start] = ACTIONS(2266),
    [sym_float_literal] = ACTIONS(2266),
    [sym__outer_block_doc_comment_start] = ACTIONS(2266),
    [sym__outer_line_doc_comment_start] = ACTIONS(2266),
  },
  [STATE(624)] = {
    [sym_line_comment] = STATE(624),
    [sym_block_comment] = STATE(624),
    [ts_builtin_sym_end] = ACTIONS(2270),
    [sym_identifier] = ACTIONS(2272),
    [anon_sym_SEMI] = ACTIONS(2270),
    [anon_sym_macro] = ACTIONS(2272),
    [anon_sym_LPAREN] = ACTIONS(2270),
    [anon_sym_LBRACE] = ACTIONS(2270),
    [anon_sym_RBRACE] = ACTIONS(2270),
    [anon_sym_macro_rules] = ACTIONS(2272),
    [anon_sym_BANG] = ACTIONS(2270),
    [anon_sym_LBRACK] = ACTIONS(2270),
    [anon_sym_STAR] = ACTIONS(2270),
    [anon_sym_u8] = ACTIONS(2272),
    [anon_sym_i8] = ACTIONS(2272),
    [anon_sym_u16] = ACTIONS(2272),
    [anon_sym_i16] = ACTIONS(2272),
    [anon_sym_u32] = ACTIONS(2272),
    [anon_sym_i32] = ACTIONS(2272),
    [anon_sym_u64] = ACTIONS(2272),
    [anon_sym_i64] = ACTIONS(2272),
    [anon_sym_u128] = ACTIONS(2272),
    [anon_sym_i128] = ACTIONS(2272),
    [anon_sym_isize] = ACTIONS(2272),
    [anon_sym_usize] = ACTIONS(2272),
    [anon_sym_f32] = ACTIONS(2272),
    [anon_sym_f64] = ACTIONS(2272),
    [anon_sym_bool] = ACTIONS(2272),
    [anon_sym_str] = ACTIONS(2272),
    [anon_sym_char] = ACTIONS(2272),
    [anon_sym_DASH] = ACTIONS(2270),
    [anon_sym_AMP] = ACTIONS(2270),
    [anon_sym_PIPE] = ACTIONS(2270),
    [anon_sym_LT] = ACTIONS(2270),
    [anon_sym_DOT_DOT] = ACTIONS(2270),
    [anon_sym_COLON_COLON] = ACTIONS(2270),
    [anon_sym_POUND] = ACTIONS(2270),
    [anon_sym_SQUOTE] = ACTIONS(2272),
    [anon_sym_async] = ACTIONS(2272),
    [anon_sym_auto] = ACTIONS(2272),
    [anon_sym_await] = ACTIONS(2272),
    [anon_sym_become] = ACTIONS(2272),
    [anon_sym_break] = ACTIONS(2272),
    [anon_sym_const] = ACTIONS(2272),
    [anon_sym_continue] = ACTIONS(2272),
    [anon_sym_default] = ACTIONS(2272),
    [anon_sym_enum] = ACTIONS(2272),
    [anon_sym_fn] = ACTIONS(2272),
    [anon_sym_for] = ACTIONS(2272),
    [anon_sym_gen] = ACTIONS(2272),
    [anon_sym_if] = ACTIONS(2272),
    [anon_sym_impl] = ACTIONS(2272),
    [anon_sym_let] = ACTIONS(2272),
    [anon_sym_loop] = ACTIONS(2272),
    [anon_sym_match] = ACTIONS(2272),
    [anon_sym_mod] = ACTIONS(2272),
    [anon_sym_pub] = ACTIONS(2272),
    [anon_sym_return] = ACTIONS(2272),
    [anon_sym_raw] = ACTIONS(2272),
    [anon_sym_static] = ACTIONS(2272),
    [anon_sym_struct] = ACTIONS(2272),
    [anon_sym_trait] = ACTIONS(2272),
    [anon_sym_type] = ACTIONS(2272),
    [anon_sym_union] = ACTIONS(2272),
    [anon_sym_unsafe] = ACTIONS(2272),
    [anon_sym_use] = ACTIONS(2272),
    [anon_sym_while] = ACTIONS(2272),
    [anon_sym_extern] = ACTIONS(2272),
    [anon_sym_safe] = ACTIONS(2272),
    [anon_sym_yield] = ACTIONS(2272),
    [anon_sym_move] = ACTIONS(2272),
    [anon_sym_try] = ACTIONS(2272),
    [sym_integer_literal] = ACTIONS(2270),
    [aux_sym_string_literal_token1] = ACTIONS(2270),
    [sym_char_literal] = ACTIONS(2270),
    [anon_sym_true] = ACTIONS(2272),
    [anon_sym_false] = ACTIONS(2272),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2272),
    [sym_super] = ACTIONS(2272),
    [sym_crate] = ACTIONS(2272),
    [aux_sym_metavariable_token1] = ACTIONS(2270),
    [sym__raw_string_literal_start] = ACTIONS(2270),
    [sym_float_literal] = ACTIONS(2270),
    [sym__outer_block_doc_comment_start] = ACTIONS(2270),
    [sym__outer_line_doc_comment_start] = ACTIONS(2270),
  },
  [STATE(625)] = {
    [sym_line_comment] = STATE(625),
    [sym_block_comment] = STATE(625),
    [ts_builtin_sym_end] = ACTIONS(2274),
    [sym_identifier] = ACTIONS(2276),
    [anon_sym_SEMI] = ACTIONS(2274),
    [anon_sym_macro] = ACTIONS(2276),
    [anon_sym_LPAREN] = ACTIONS(2274),
    [anon_sym_LBRACE] = ACTIONS(2274),
    [anon_sym_RBRACE] = ACTIONS(2274),
    [anon_sym_macro_rules] = ACTIONS(2276),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_LBRACK] = ACTIONS(2274),
    [anon_sym_STAR] = ACTIONS(2274),
    [anon_sym_u8] = ACTIONS(2276),
    [anon_sym_i8] = ACTIONS(2276),
    [anon_sym_u16] = ACTIONS(2276),
    [anon_sym_i16] = ACTIONS(2276),
    [anon_sym_u32] = ACTIONS(2276),
    [anon_sym_i32] = ACTIONS(2276),
    [anon_sym_u64] = ACTIONS(2276),
    [anon_sym_i64] = ACTIONS(2276),
    [anon_sym_u128] = ACTIONS(2276),
    [anon_sym_i128] = ACTIONS(2276),
    [anon_sym_isize] = ACTIONS(2276),
    [anon_sym_usize] = ACTIONS(2276),
    [anon_sym_f32] = ACTIONS(2276),
    [anon_sym_f64] = ACTIONS(2276),
    [anon_sym_bool] = ACTIONS(2276),
    [anon_sym_str] = ACTIONS(2276),
    [anon_sym_char] = ACTIONS(2276),
    [anon_sym_DASH] = ACTIONS(2274),
    [anon_sym_AMP] = ACTIONS(2274),
    [anon_sym_PIPE] = ACTIONS(2274),
    [anon_sym_LT] = ACTIONS(2274),
    [anon_sym_DOT_DOT] = ACTIONS(2274),
    [anon_sym_COLON_COLON] = ACTIONS(2274),
    [anon_sym_POUND] = ACTIONS(2274),
    [anon_sym_SQUOTE] = ACTIONS(2276),
    [anon_sym_async] = ACTIONS(2276),
    [anon_sym_auto] = ACTIONS(2276),
    [anon_sym_await] = ACTIONS(2276),
    [anon_sym_become] = ACTIONS(2276),
    [anon_sym_break] = ACTIONS(2276),
    [anon_sym_const] = ACTIONS(2276),
    [anon_sym_continue] = ACTIONS(2276),
    [anon_sym_default] = ACTIONS(2276),
    [anon_sym_enum] = ACTIONS(2276),
    [anon_sym_fn] = ACTIONS(2276),
    [anon_sym_for] = ACTIONS(2276),
    [anon_sym_gen] = ACTIONS(2276),
    [anon_sym_if] = ACTIONS(2276),
    [anon_sym_impl] = ACTIONS(2276),
    [anon_sym_let] = ACTIONS(2276),
    [anon_sym_loop] = ACTIONS(2276),
    [anon_sym_match] = ACTIONS(2276),
    [anon_sym_mod] = ACTIONS(2276),
    [anon_sym_pub] = ACTIONS(2276),
    [anon_sym_return] = ACTIONS(2276),
    [anon_sym_raw] = ACTIONS(2276),
    [anon_sym_static] = ACTIONS(2276),
    [anon_sym_struct] = ACTIONS(2276),
    [anon_sym_trait] = ACTIONS(2276),
    [anon_sym_type] = ACTIONS(2276),
    [anon_sym_union] = ACTIONS(2276),
    [anon_sym_unsafe] = ACTIONS(2276),
    [anon_sym_use] = ACTIONS(2276),
    [anon_sym_while] = ACTIONS(2276),
    [anon_sym_extern] = ACTIONS(2276),
    [anon_sym_safe] = ACTIONS(2276),
    [anon_sym_yield] = ACTIONS(2276),
    [anon_sym_move] = ACTIONS(2276),
    [anon_sym_try] = ACTIONS(2276),
    [sym_integer_literal] = ACTIONS(2274),
    [aux_sym_string_literal_token1] = ACTIONS(2274),
    [sym_char_literal] = ACTIONS(2274),
    [anon_sym_true] = ACTIONS(2276),
    [anon_sym_false] = ACTIONS(2276),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2276),
    [sym_super] = ACTIONS(2276),
    [sym_crate] = ACTIONS(2276),
    [aux_sym_metavariable_token1] = ACTIONS(2274),
    [sym__raw_string_literal_start] = ACTIONS(2274),
    [sym_float_literal] = ACTIONS(2274),
    [sym__outer_block_doc_comment_start] = ACTIONS(2274),
    [sym__outer_line_doc_comment_start] = ACTIONS(2274),
  },
  [STATE(626)] = {
    [sym_line_comment] = STATE(626),
    [sym_block_comment] = STATE(626),
    [ts_builtin_sym_end] = ACTIONS(2278),
    [sym_identifier] = ACTIONS(2280),
    [anon_sym_SEMI] = ACTIONS(2278),
    [anon_sym_macro] = ACTIONS(2280),
    [anon_sym_LPAREN] = ACTIONS(2278),
    [anon_sym_LBRACE] = ACTIONS(2278),
    [anon_sym_RBRACE] = ACTIONS(2278),
    [anon_sym_macro_rules] = ACTIONS(2280),
    [anon_sym_BANG] = ACTIONS(2278),
    [anon_sym_LBRACK] = ACTIONS(2278),
    [anon_sym_STAR] = ACTIONS(2278),
    [anon_sym_u8] = ACTIONS(2280),
    [anon_sym_i8] = ACTIONS(2280),
    [anon_sym_u16] = ACTIONS(2280),
    [anon_sym_i16] = ACTIONS(2280),
    [anon_sym_u32] = ACTIONS(2280),
    [anon_sym_i32] = ACTIONS(2280),
    [anon_sym_u64] = ACTIONS(2280),
    [anon_sym_i64] = ACTIONS(2280),
    [anon_sym_u128] = ACTIONS(2280),
    [anon_sym_i128] = ACTIONS(2280),
    [anon_sym_isize] = ACTIONS(2280),
    [anon_sym_usize] = ACTIONS(2280),
    [anon_sym_f32] = ACTIONS(2280),
    [anon_sym_f64] = ACTIONS(2280),
    [anon_sym_bool] = ACTIONS(2280),
    [anon_sym_str] = ACTIONS(2280),
    [anon_sym_char] = ACTIONS(2280),
    [anon_sym_DASH] = ACTIONS(2278),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_PIPE] = ACTIONS(2278),
    [anon_sym_LT] = ACTIONS(2278),
    [anon_sym_DOT_DOT] = ACTIONS(2278),
    [anon_sym_COLON_COLON] = ACTIONS(2278),
    [anon_sym_POUND] = ACTIONS(2278),
    [anon_sym_SQUOTE] = ACTIONS(2280),
    [anon_sym_async] = ACTIONS(2280),
    [anon_sym_auto] = ACTIONS(2280),
    [anon_sym_await] = ACTIONS(2280),
    [anon_sym_become] = ACTIONS(2280),
    [anon_sym_break] = ACTIONS(2280),
    [anon_sym_const] = ACTIONS(2280),
    [anon_sym_continue] = ACTIONS(2280),
    [anon_sym_default] = ACTIONS(2280),
    [anon_sym_enum] = ACTIONS(2280),
    [anon_sym_fn] = ACTIONS(2280),
    [anon_sym_for] = ACTIONS(2280),
    [anon_sym_gen] = ACTIONS(2280),
    [anon_sym_if] = ACTIONS(2280),
    [anon_sym_impl] = ACTIONS(2280),
    [anon_sym_let] = ACTIONS(2280),
    [anon_sym_loop] = ACTIONS(2280),
    [anon_sym_match] = ACTIONS(2280),
    [anon_sym_mod] = ACTIONS(2280),
    [anon_sym_pub] = ACTIONS(2280),
    [anon_sym_return] = ACTIONS(2280),
    [anon_sym_raw] = ACTIONS(2280),
    [anon_sym_static] = ACTIONS(2280),
    [anon_sym_struct] = ACTIONS(2280),
    [anon_sym_trait] = ACTIONS(2280),
    [anon_sym_type] = ACTIONS(2280),
    [anon_sym_union] = ACTIONS(2280),
    [anon_sym_unsafe] = ACTIONS(2280),
    [anon_sym_use] = ACTIONS(2280),
    [anon_sym_while] = ACTIONS(2280),
    [anon_sym_extern] = ACTIONS(2280),
    [anon_sym_safe] = ACTIONS(2280),
    [anon_sym_yield] = ACTIONS(2280),
    [anon_sym_move] = ACTIONS(2280),
    [anon_sym_try] = ACTIONS(2280),
    [sym_integer_literal] = ACTIONS(2278),
    [aux_sym_string_literal_token1] = ACTIONS(2278),
    [sym_char_literal] = ACTIONS(2278),
    [anon_sym_true] = ACTIONS(2280),
    [anon_sym_false] = ACTIONS(2280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2280),
    [sym_super] = ACTIONS(2280),
    [sym_crate] = ACTIONS(2280),
    [aux_sym_metavariable_token1] = ACTIONS(2278),
    [sym__raw_string_literal_start] = ACTIONS(2278),
    [sym_float_literal] = ACTIONS(2278),
    [sym__outer_block_doc_comment_start] = ACTIONS(2278),
    [sym__outer_line_doc_comment_start] = ACTIONS(2278),
  },
  [STATE(627)] = {
    [sym_line_comment] = STATE(627),
    [sym_block_comment] = STATE(627),
    [ts_builtin_sym_end] = ACTIONS(2282),
    [sym_identifier] = ACTIONS(2284),
    [anon_sym_SEMI] = ACTIONS(2282),
    [anon_sym_macro] = ACTIONS(2284),
    [anon_sym_LPAREN] = ACTIONS(2282),
    [anon_sym_LBRACE] = ACTIONS(2282),
    [anon_sym_RBRACE] = ACTIONS(2282),
    [anon_sym_macro_rules] = ACTIONS(2284),
    [anon_sym_BANG] = ACTIONS(2282),
    [anon_sym_LBRACK] = ACTIONS(2282),
    [anon_sym_STAR] = ACTIONS(2282),
    [anon_sym_u8] = ACTIONS(2284),
    [anon_sym_i8] = ACTIONS(2284),
    [anon_sym_u16] = ACTIONS(2284),
    [anon_sym_i16] = ACTIONS(2284),
    [anon_sym_u32] = ACTIONS(2284),
    [anon_sym_i32] = ACTIONS(2284),
    [anon_sym_u64] = ACTIONS(2284),
    [anon_sym_i64] = ACTIONS(2284),
    [anon_sym_u128] = ACTIONS(2284),
    [anon_sym_i128] = ACTIONS(2284),
    [anon_sym_isize] = ACTIONS(2284),
    [anon_sym_usize] = ACTIONS(2284),
    [anon_sym_f32] = ACTIONS(2284),
    [anon_sym_f64] = ACTIONS(2284),
    [anon_sym_bool] = ACTIONS(2284),
    [anon_sym_str] = ACTIONS(2284),
    [anon_sym_char] = ACTIONS(2284),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_AMP] = ACTIONS(2282),
    [anon_sym_PIPE] = ACTIONS(2282),
    [anon_sym_LT] = ACTIONS(2282),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_COLON_COLON] = ACTIONS(2282),
    [anon_sym_POUND] = ACTIONS(2282),
    [anon_sym_SQUOTE] = ACTIONS(2284),
    [anon_sym_async] = ACTIONS(2284),
    [anon_sym_auto] = ACTIONS(2284),
    [anon_sym_await] = ACTIONS(2284),
    [anon_sym_become] = ACTIONS(2284),
    [anon_sym_break] = ACTIONS(2284),
    [anon_sym_const] = ACTIONS(2284),
    [anon_sym_continue] = ACTIONS(2284),
    [anon_sym_default] = ACTIONS(2284),
    [anon_sym_enum] = ACTIONS(2284),
    [anon_sym_fn] = ACTIONS(2284),
    [anon_sym_for] = ACTIONS(2284),
    [anon_sym_gen] = ACTIONS(2284),
    [anon_sym_if] = ACTIONS(2284),
    [anon_sym_impl] = ACTIONS(2284),
    [anon_sym_let] = ACTIONS(2284),
    [anon_sym_loop] = ACTIONS(2284),
    [anon_sym_match] = ACTIONS(2284),
    [anon_sym_mod] = ACTIONS(2284),
    [anon_sym_pub] = ACTIONS(2284),
    [anon_sym_return] = ACTIONS(2284),
    [anon_sym_raw] = ACTIONS(2284),
    [anon_sym_static] = ACTIONS(2284),
    [anon_sym_struct] = ACTIONS(2284),
    [anon_sym_trait] = ACTIONS(2284),
    [anon_sym_type] = ACTIONS(2284),
    [anon_sym_union] = ACTIONS(2284),
    [anon_sym_unsafe] = ACTIONS(2284),
    [anon_sym_use] = ACTIONS(2284),
    [anon_sym_while] = ACTIONS(2284),
    [anon_sym_extern] = ACTIONS(2284),
    [anon_sym_safe] = ACTIONS(2284),
    [anon_sym_yield] = ACTIONS(2284),
    [anon_sym_move] = ACTIONS(2284),
    [anon_sym_try] = ACTIONS(2284),
    [sym_integer_literal] = ACTIONS(2282),
    [aux_sym_string_literal_token1] = ACTIONS(2282),
    [sym_char_literal] = ACTIONS(2282),
    [anon_sym_true] = ACTIONS(2284),
    [anon_sym_false] = ACTIONS(2284),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2284),
    [sym_super] = ACTIONS(2284),
    [sym_crate] = ACTIONS(2284),
    [aux_sym_metavariable_token1] = ACTIONS(2282),
    [sym__raw_string_literal_start] = ACTIONS(2282),
    [sym_float_literal] = ACTIONS(2282),
    [sym__outer_block_doc_comment_start] = ACTIONS(2282),
    [sym__outer_line_doc_comment_start] = ACTIONS(2282),
  },
  [STATE(628)] = {
    [sym_line_comment] = STATE(628),
    [sym_block_comment] = STATE(628),
    [ts_builtin_sym_end] = ACTIONS(2286),
    [sym_identifier] = ACTIONS(2288),
    [anon_sym_SEMI] = ACTIONS(2286),
    [anon_sym_macro] = ACTIONS(2288),
    [anon_sym_LPAREN] = ACTIONS(2286),
    [anon_sym_LBRACE] = ACTIONS(2286),
    [anon_sym_RBRACE] = ACTIONS(2286),
    [anon_sym_macro_rules] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2286),
    [anon_sym_LBRACK] = ACTIONS(2286),
    [anon_sym_STAR] = ACTIONS(2286),
    [anon_sym_u8] = ACTIONS(2288),
    [anon_sym_i8] = ACTIONS(2288),
    [anon_sym_u16] = ACTIONS(2288),
    [anon_sym_i16] = ACTIONS(2288),
    [anon_sym_u32] = ACTIONS(2288),
    [anon_sym_i32] = ACTIONS(2288),
    [anon_sym_u64] = ACTIONS(2288),
    [anon_sym_i64] = ACTIONS(2288),
    [anon_sym_u128] = ACTIONS(2288),
    [anon_sym_i128] = ACTIONS(2288),
    [anon_sym_isize] = ACTIONS(2288),
    [anon_sym_usize] = ACTIONS(2288),
    [anon_sym_f32] = ACTIONS(2288),
    [anon_sym_f64] = ACTIONS(2288),
    [anon_sym_bool] = ACTIONS(2288),
    [anon_sym_str] = ACTIONS(2288),
    [anon_sym_char] = ACTIONS(2288),
    [anon_sym_DASH] = ACTIONS(2286),
    [anon_sym_AMP] = ACTIONS(2286),
    [anon_sym_PIPE] = ACTIONS(2286),
    [anon_sym_LT] = ACTIONS(2286),
    [anon_sym_DOT_DOT] = ACTIONS(2286),
    [anon_sym_COLON_COLON] = ACTIONS(2286),
    [anon_sym_POUND] = ACTIONS(2286),
    [anon_sym_SQUOTE] = ACTIONS(2288),
    [anon_sym_async] = ACTIONS(2288),
    [anon_sym_auto] = ACTIONS(2288),
    [anon_sym_await] = ACTIONS(2288),
    [anon_sym_become] = ACTIONS(2288),
    [anon_sym_break] = ACTIONS(2288),
    [anon_sym_const] = ACTIONS(2288),
    [anon_sym_continue] = ACTIONS(2288),
    [anon_sym_default] = ACTIONS(2288),
    [anon_sym_enum] = ACTIONS(2288),
    [anon_sym_fn] = ACTIONS(2288),
    [anon_sym_for] = ACTIONS(2288),
    [anon_sym_gen] = ACTIONS(2288),
    [anon_sym_if] = ACTIONS(2288),
    [anon_sym_impl] = ACTIONS(2288),
    [anon_sym_let] = ACTIONS(2288),
    [anon_sym_loop] = ACTIONS(2288),
    [anon_sym_match] = ACTIONS(2288),
    [anon_sym_mod] = ACTIONS(2288),
    [anon_sym_pub] = ACTIONS(2288),
    [anon_sym_return] = ACTIONS(2288),
    [anon_sym_raw] = ACTIONS(2288),
    [anon_sym_static] = ACTIONS(2288),
    [anon_sym_struct] = ACTIONS(2288),
    [anon_sym_trait] = ACTIONS(2288),
    [anon_sym_type] = ACTIONS(2288),
    [anon_sym_union] = ACTIONS(2288),
    [anon_sym_unsafe] = ACTIONS(2288),
    [anon_sym_use] = ACTIONS(2288),
    [anon_sym_while] = ACTIONS(2288),
    [anon_sym_extern] = ACTIONS(2288),
    [anon_sym_safe] = ACTIONS(2288),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_move] = ACTIONS(2288),
    [anon_sym_try] = ACTIONS(2288),
    [sym_integer_literal] = ACTIONS(2286),
    [aux_sym_string_literal_token1] = ACTIONS(2286),
    [sym_char_literal] = ACTIONS(2286),
    [anon_sym_true] = ACTIONS(2288),
    [anon_sym_false] = ACTIONS(2288),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2288),
    [sym_super] = ACTIONS(2288),
    [sym_crate] = ACTIONS(2288),
    [aux_sym_metavariable_token1] = ACTIONS(2286),
    [sym__raw_string_literal_start] = ACTIONS(2286),
    [sym_float_literal] = ACTIONS(2286),
    [sym__outer_block_doc_comment_start] = ACTIONS(2286),
    [sym__outer_line_doc_comment_start] = ACTIONS(2286),
  },
  [STATE(629)] = {
    [sym_line_comment] = STATE(629),
    [sym_block_comment] = STATE(629),
    [ts_builtin_sym_end] = ACTIONS(2290),
    [sym_identifier] = ACTIONS(2292),
    [anon_sym_SEMI] = ACTIONS(2290),
    [anon_sym_macro] = ACTIONS(2292),
    [anon_sym_LPAREN] = ACTIONS(2290),
    [anon_sym_LBRACE] = ACTIONS(2290),
    [anon_sym_RBRACE] = ACTIONS(2290),
    [anon_sym_macro_rules] = ACTIONS(2292),
    [anon_sym_BANG] = ACTIONS(2290),
    [anon_sym_LBRACK] = ACTIONS(2290),
    [anon_sym_STAR] = ACTIONS(2290),
    [anon_sym_u8] = ACTIONS(2292),
    [anon_sym_i8] = ACTIONS(2292),
    [anon_sym_u16] = ACTIONS(2292),
    [anon_sym_i16] = ACTIONS(2292),
    [anon_sym_u32] = ACTIONS(2292),
    [anon_sym_i32] = ACTIONS(2292),
    [anon_sym_u64] = ACTIONS(2292),
    [anon_sym_i64] = ACTIONS(2292),
    [anon_sym_u128] = ACTIONS(2292),
    [anon_sym_i128] = ACTIONS(2292),
    [anon_sym_isize] = ACTIONS(2292),
    [anon_sym_usize] = ACTIONS(2292),
    [anon_sym_f32] = ACTIONS(2292),
    [anon_sym_f64] = ACTIONS(2292),
    [anon_sym_bool] = ACTIONS(2292),
    [anon_sym_str] = ACTIONS(2292),
    [anon_sym_char] = ACTIONS(2292),
    [anon_sym_DASH] = ACTIONS(2290),
    [anon_sym_AMP] = ACTIONS(2290),
    [anon_sym_PIPE] = ACTIONS(2290),
    [anon_sym_LT] = ACTIONS(2290),
    [anon_sym_DOT_DOT] = ACTIONS(2290),
    [anon_sym_COLON_COLON] = ACTIONS(2290),
    [anon_sym_POUND] = ACTIONS(2290),
    [anon_sym_SQUOTE] = ACTIONS(2292),
    [anon_sym_async] = ACTIONS(2292),
    [anon_sym_auto] = ACTIONS(2292),
    [anon_sym_await] = ACTIONS(2292),
    [anon_sym_become] = ACTIONS(2292),
    [anon_sym_break] = ACTIONS(2292),
    [anon_sym_const] = ACTIONS(2292),
    [anon_sym_continue] = ACTIONS(2292),
    [anon_sym_default] = ACTIONS(2292),
    [anon_sym_enum] = ACTIONS(2292),
    [anon_sym_fn] = ACTIONS(2292),
    [anon_sym_for] = ACTIONS(2292),
    [anon_sym_gen] = ACTIONS(2292),
    [anon_sym_if] = ACTIONS(2292),
    [anon_sym_impl] = ACTIONS(2292),
    [anon_sym_let] = ACTIONS(2292),
    [anon_sym_loop] = ACTIONS(2292),
    [anon_sym_match] = ACTIONS(2292),
    [anon_sym_mod] = ACTIONS(2292),
    [anon_sym_pub] = ACTIONS(2292),
    [anon_sym_return] = ACTIONS(2292),
    [anon_sym_raw] = ACTIONS(2292),
    [anon_sym_static] = ACTIONS(2292),
    [anon_sym_struct] = ACTIONS(2292),
    [anon_sym_trait] = ACTIONS(2292),
    [anon_sym_type] = ACTIONS(2292),
    [anon_sym_union] = ACTIONS(2292),
    [anon_sym_unsafe] = ACTIONS(2292),
    [anon_sym_use] = ACTIONS(2292),
    [anon_sym_while] = ACTIONS(2292),
    [anon_sym_extern] = ACTIONS(2292),
    [anon_sym_safe] = ACTIONS(2292),
    [anon_sym_yield] = ACTIONS(2292),
    [anon_sym_move] = ACTIONS(2292),
    [anon_sym_try] = ACTIONS(2292),
    [sym_integer_literal] = ACTIONS(2290),
    [aux_sym_string_literal_token1] = ACTIONS(2290),
    [sym_char_literal] = ACTIONS(2290),
    [anon_sym_true] = ACTIONS(2292),
    [anon_sym_false] = ACTIONS(2292),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2292),
    [sym_super] = ACTIONS(2292),
    [sym_crate] = ACTIONS(2292),
    [aux_sym_metavariable_token1] = ACTIONS(2290),
    [sym__raw_string_literal_start] = ACTIONS(2290),
    [sym_float_literal] = ACTIONS(2290),
    [sym__outer_block_doc_comment_start] = ACTIONS(2290),
    [sym__outer_line_doc_comment_start] = ACTIONS(2290),
  },
  [STATE(630)] = {
    [sym_line_comment] = STATE(630),
    [sym_block_comment] = STATE(630),
    [ts_builtin_sym_end] = ACTIONS(2294),
    [sym_identifier] = ACTIONS(2296),
    [anon_sym_SEMI] = ACTIONS(2294),
    [anon_sym_macro] = ACTIONS(2296),
    [anon_sym_LPAREN] = ACTIONS(2294),
    [anon_sym_LBRACE] = ACTIONS(2294),
    [anon_sym_RBRACE] = ACTIONS(2294),
    [anon_sym_macro_rules] = ACTIONS(2296),
    [anon_sym_BANG] = ACTIONS(2294),
    [anon_sym_LBRACK] = ACTIONS(2294),
    [anon_sym_STAR] = ACTIONS(2294),
    [anon_sym_u8] = ACTIONS(2296),
    [anon_sym_i8] = ACTIONS(2296),
    [anon_sym_u16] = ACTIONS(2296),
    [anon_sym_i16] = ACTIONS(2296),
    [anon_sym_u32] = ACTIONS(2296),
    [anon_sym_i32] = ACTIONS(2296),
    [anon_sym_u64] = ACTIONS(2296),
    [anon_sym_i64] = ACTIONS(2296),
    [anon_sym_u128] = ACTIONS(2296),
    [anon_sym_i128] = ACTIONS(2296),
    [anon_sym_isize] = ACTIONS(2296),
    [anon_sym_usize] = ACTIONS(2296),
    [anon_sym_f32] = ACTIONS(2296),
    [anon_sym_f64] = ACTIONS(2296),
    [anon_sym_bool] = ACTIONS(2296),
    [anon_sym_str] = ACTIONS(2296),
    [anon_sym_char] = ACTIONS(2296),
    [anon_sym_DASH] = ACTIONS(2294),
    [anon_sym_AMP] = ACTIONS(2294),
    [anon_sym_PIPE] = ACTIONS(2294),
    [anon_sym_LT] = ACTIONS(2294),
    [anon_sym_DOT_DOT] = ACTIONS(2294),
    [anon_sym_COLON_COLON] = ACTIONS(2294),
    [anon_sym_POUND] = ACTIONS(2294),
    [anon_sym_SQUOTE] = ACTIONS(2296),
    [anon_sym_async] = ACTIONS(2296),
    [anon_sym_auto] = ACTIONS(2296),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_become] = ACTIONS(2296),
    [anon_sym_break] = ACTIONS(2296),
    [anon_sym_const] = ACTIONS(2296),
    [anon_sym_continue] = ACTIONS(2296),
    [anon_sym_default] = ACTIONS(2296),
    [anon_sym_enum] = ACTIONS(2296),
    [anon_sym_fn] = ACTIONS(2296),
    [anon_sym_for] = ACTIONS(2296),
    [anon_sym_gen] = ACTIONS(2296),
    [anon_sym_if] = ACTIONS(2296),
    [anon_sym_impl] = ACTIONS(2296),
    [anon_sym_let] = ACTIONS(2296),
    [anon_sym_loop] = ACTIONS(2296),
    [anon_sym_match] = ACTIONS(2296),
    [anon_sym_mod] = ACTIONS(2296),
    [anon_sym_pub] = ACTIONS(2296),
    [anon_sym_return] = ACTIONS(2296),
    [anon_sym_raw] = ACTIONS(2296),
    [anon_sym_static] = ACTIONS(2296),
    [anon_sym_struct] = ACTIONS(2296),
    [anon_sym_trait] = ACTIONS(2296),
    [anon_sym_type] = ACTIONS(2296),
    [anon_sym_union] = ACTIONS(2296),
    [anon_sym_unsafe] = ACTIONS(2296),
    [anon_sym_use] = ACTIONS(2296),
    [anon_sym_while] = ACTIONS(2296),
    [anon_sym_extern] = ACTIONS(2296),
    [anon_sym_safe] = ACTIONS(2296),
    [anon_sym_yield] = ACTIONS(2296),
    [anon_sym_move] = ACTIONS(2296),
    [anon_sym_try] = ACTIONS(2296),
    [sym_integer_literal] = ACTIONS(2294),
    [aux_sym_string_literal_token1] = ACTIONS(2294),
    [sym_char_literal] = ACTIONS(2294),
    [anon_sym_true] = ACTIONS(2296),
    [anon_sym_false] = ACTIONS(2296),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2296),
    [sym_super] = ACTIONS(2296),
    [sym_crate] = ACTIONS(2296),
    [aux_sym_metavariable_token1] = ACTIONS(2294),
    [sym__raw_string_literal_start] = ACTIONS(2294),
    [sym_float_literal] = ACTIONS(2294),
    [sym__outer_block_doc_comment_start] = ACTIONS(2294),
    [sym__outer_line_doc_comment_start] = ACTIONS(2294),
  },
  [STATE(631)] = {
    [sym_line_comment] = STATE(631),
    [sym_block_comment] = STATE(631),
    [ts_builtin_sym_end] = ACTIONS(2298),
    [sym_identifier] = ACTIONS(2300),
    [anon_sym_SEMI] = ACTIONS(2298),
    [anon_sym_macro] = ACTIONS(2300),
    [anon_sym_LPAREN] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(2298),
    [anon_sym_RBRACE] = ACTIONS(2298),
    [anon_sym_macro_rules] = ACTIONS(2300),
    [anon_sym_BANG] = ACTIONS(2298),
    [anon_sym_LBRACK] = ACTIONS(2298),
    [anon_sym_STAR] = ACTIONS(2298),
    [anon_sym_u8] = ACTIONS(2300),
    [anon_sym_i8] = ACTIONS(2300),
    [anon_sym_u16] = ACTIONS(2300),
    [anon_sym_i16] = ACTIONS(2300),
    [anon_sym_u32] = ACTIONS(2300),
    [anon_sym_i32] = ACTIONS(2300),
    [anon_sym_u64] = ACTIONS(2300),
    [anon_sym_i64] = ACTIONS(2300),
    [anon_sym_u128] = ACTIONS(2300),
    [anon_sym_i128] = ACTIONS(2300),
    [anon_sym_isize] = ACTIONS(2300),
    [anon_sym_usize] = ACTIONS(2300),
    [anon_sym_f32] = ACTIONS(2300),
    [anon_sym_f64] = ACTIONS(2300),
    [anon_sym_bool] = ACTIONS(2300),
    [anon_sym_str] = ACTIONS(2300),
    [anon_sym_char] = ACTIONS(2300),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_AMP] = ACTIONS(2298),
    [anon_sym_PIPE] = ACTIONS(2298),
    [anon_sym_LT] = ACTIONS(2298),
    [anon_sym_DOT_DOT] = ACTIONS(2298),
    [anon_sym_COLON_COLON] = ACTIONS(2298),
    [anon_sym_POUND] = ACTIONS(2298),
    [anon_sym_SQUOTE] = ACTIONS(2300),
    [anon_sym_async] = ACTIONS(2300),
    [anon_sym_auto] = ACTIONS(2300),
    [anon_sym_await] = ACTIONS(2300),
    [anon_sym_become] = ACTIONS(2300),
    [anon_sym_break] = ACTIONS(2300),
    [anon_sym_const] = ACTIONS(2300),
    [anon_sym_continue] = ACTIONS(2300),
    [anon_sym_default] = ACTIONS(2300),
    [anon_sym_enum] = ACTIONS(2300),
    [anon_sym_fn] = ACTIONS(2300),
    [anon_sym_for] = ACTIONS(2300),
    [anon_sym_gen] = ACTIONS(2300),
    [anon_sym_if] = ACTIONS(2300),
    [anon_sym_impl] = ACTIONS(2300),
    [anon_sym_let] = ACTIONS(2300),
    [anon_sym_loop] = ACTIONS(2300),
    [anon_sym_match] = ACTIONS(2300),
    [anon_sym_mod] = ACTIONS(2300),
    [anon_sym_pub] = ACTIONS(2300),
    [anon_sym_return] = ACTIONS(2300),
    [anon_sym_raw] = ACTIONS(2300),
    [anon_sym_static] = ACTIONS(2300),
    [anon_sym_struct] = ACTIONS(2300),
    [anon_sym_trait] = ACTIONS(2300),
    [anon_sym_type] = ACTIONS(2300),
    [anon_sym_union] = ACTIONS(2300),
    [anon_sym_unsafe] = ACTIONS(2300),
    [anon_sym_use] = ACTIONS(2300),
    [anon_sym_while] = ACTIONS(2300),
    [anon_sym_extern] = ACTIONS(2300),
    [anon_sym_safe] = ACTIONS(2300),
    [anon_sym_yield] = ACTIONS(2300),
    [anon_sym_move] = ACTIONS(2300),
    [anon_sym_try] = ACTIONS(2300),
    [sym_integer_literal] = ACTIONS(2298),
    [aux_sym_string_literal_token1] = ACTIONS(2298),
    [sym_char_literal] = ACTIONS(2298),
    [anon_sym_true] = ACTIONS(2300),
    [anon_sym_false] = ACTIONS(2300),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2300),
    [sym_super] = ACTIONS(2300),
    [sym_crate] = ACTIONS(2300),
    [aux_sym_metavariable_token1] = ACTIONS(2298),
    [sym__raw_string_literal_start] = ACTIONS(2298),
    [sym_float_literal] = ACTIONS(2298),
    [sym__outer_block_doc_comment_start] = ACTIONS(2298),
    [sym__outer_line_doc_comment_start] = ACTIONS(2298),
  },
  [STATE(632)] = {
    [sym_line_comment] = STATE(632),
    [sym_block_comment] = STATE(632),
    [ts_builtin_sym_end] = ACTIONS(2302),
    [sym_identifier] = ACTIONS(2304),
    [anon_sym_SEMI] = ACTIONS(2302),
    [anon_sym_macro] = ACTIONS(2304),
    [anon_sym_LPAREN] = ACTIONS(2302),
    [anon_sym_LBRACE] = ACTIONS(2302),
    [anon_sym_RBRACE] = ACTIONS(2302),
    [anon_sym_macro_rules] = ACTIONS(2304),
    [anon_sym_BANG] = ACTIONS(2302),
    [anon_sym_LBRACK] = ACTIONS(2302),
    [anon_sym_STAR] = ACTIONS(2302),
    [anon_sym_u8] = ACTIONS(2304),
    [anon_sym_i8] = ACTIONS(2304),
    [anon_sym_u16] = ACTIONS(2304),
    [anon_sym_i16] = ACTIONS(2304),
    [anon_sym_u32] = ACTIONS(2304),
    [anon_sym_i32] = ACTIONS(2304),
    [anon_sym_u64] = ACTIONS(2304),
    [anon_sym_i64] = ACTIONS(2304),
    [anon_sym_u128] = ACTIONS(2304),
    [anon_sym_i128] = ACTIONS(2304),
    [anon_sym_isize] = ACTIONS(2304),
    [anon_sym_usize] = ACTIONS(2304),
    [anon_sym_f32] = ACTIONS(2304),
    [anon_sym_f64] = ACTIONS(2304),
    [anon_sym_bool] = ACTIONS(2304),
    [anon_sym_str] = ACTIONS(2304),
    [anon_sym_char] = ACTIONS(2304),
    [anon_sym_DASH] = ACTIONS(2302),
    [anon_sym_AMP] = ACTIONS(2302),
    [anon_sym_PIPE] = ACTIONS(2302),
    [anon_sym_LT] = ACTIONS(2302),
    [anon_sym_DOT_DOT] = ACTIONS(2302),
    [anon_sym_COLON_COLON] = ACTIONS(2302),
    [anon_sym_POUND] = ACTIONS(2302),
    [anon_sym_SQUOTE] = ACTIONS(2304),
    [anon_sym_async] = ACTIONS(2304),
    [anon_sym_auto] = ACTIONS(2304),
    [anon_sym_await] = ACTIONS(2304),
    [anon_sym_become] = ACTIONS(2304),
    [anon_sym_break] = ACTIONS(2304),
    [anon_sym_const] = ACTIONS(2304),
    [anon_sym_continue] = ACTIONS(2304),
    [anon_sym_default] = ACTIONS(2304),
    [anon_sym_enum] = ACTIONS(2304),
    [anon_sym_fn] = ACTIONS(2304),
    [anon_sym_for] = ACTIONS(2304),
    [anon_sym_gen] = ACTIONS(2304),
    [anon_sym_if] = ACTIONS(2304),
    [anon_sym_impl] = ACTIONS(2304),
    [anon_sym_let] = ACTIONS(2304),
    [anon_sym_loop] = ACTIONS(2304),
    [anon_sym_match] = ACTIONS(2304),
    [anon_sym_mod] = ACTIONS(2304),
    [anon_sym_pub] = ACTIONS(2304),
    [anon_sym_return] = ACTIONS(2304),
    [anon_sym_raw] = ACTIONS(2304),
    [anon_sym_static] = ACTIONS(2304),
    [anon_sym_struct] = ACTIONS(2304),
    [anon_sym_trait] = ACTIONS(2304),
    [anon_sym_type] = ACTIONS(2304),
    [anon_sym_union] = ACTIONS(2304),
    [anon_sym_unsafe] = ACTIONS(2304),
    [anon_sym_use] = ACTIONS(2304),
    [anon_sym_while] = ACTIONS(2304),
    [anon_sym_extern] = ACTIONS(2304),
    [anon_sym_safe] = ACTIONS(2304),
    [anon_sym_yield] = ACTIONS(2304),
    [anon_sym_move] = ACTIONS(2304),
    [anon_sym_try] = ACTIONS(2304),
    [sym_integer_literal] = ACTIONS(2302),
    [aux_sym_string_literal_token1] = ACTIONS(2302),
    [sym_char_literal] = ACTIONS(2302),
    [anon_sym_true] = ACTIONS(2304),
    [anon_sym_false] = ACTIONS(2304),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2304),
    [sym_super] = ACTIONS(2304),
    [sym_crate] = ACTIONS(2304),
    [aux_sym_metavariable_token1] = ACTIONS(2302),
    [sym__raw_string_literal_start] = ACTIONS(2302),
    [sym_float_literal] = ACTIONS(2302),
    [sym__outer_block_doc_comment_start] = ACTIONS(2302),
    [sym__outer_line_doc_comment_start] = ACTIONS(2302),
  },
  [STATE(633)] = {
    [sym_line_comment] = STATE(633),
    [sym_block_comment] = STATE(633),
    [ts_builtin_sym_end] = ACTIONS(2306),
    [sym_identifier] = ACTIONS(2308),
    [anon_sym_SEMI] = ACTIONS(2306),
    [anon_sym_macro] = ACTIONS(2308),
    [anon_sym_LPAREN] = ACTIONS(2306),
    [anon_sym_LBRACE] = ACTIONS(2306),
    [anon_sym_RBRACE] = ACTIONS(2306),
    [anon_sym_macro_rules] = ACTIONS(2308),
    [anon_sym_BANG] = ACTIONS(2306),
    [anon_sym_LBRACK] = ACTIONS(2306),
    [anon_sym_STAR] = ACTIONS(2306),
    [anon_sym_u8] = ACTIONS(2308),
    [anon_sym_i8] = ACTIONS(2308),
    [anon_sym_u16] = ACTIONS(2308),
    [anon_sym_i16] = ACTIONS(2308),
    [anon_sym_u32] = ACTIONS(2308),
    [anon_sym_i32] = ACTIONS(2308),
    [anon_sym_u64] = ACTIONS(2308),
    [anon_sym_i64] = ACTIONS(2308),
    [anon_sym_u128] = ACTIONS(2308),
    [anon_sym_i128] = ACTIONS(2308),
    [anon_sym_isize] = ACTIONS(2308),
    [anon_sym_usize] = ACTIONS(2308),
    [anon_sym_f32] = ACTIONS(2308),
    [anon_sym_f64] = ACTIONS(2308),
    [anon_sym_bool] = ACTIONS(2308),
    [anon_sym_str] = ACTIONS(2308),
    [anon_sym_char] = ACTIONS(2308),
    [anon_sym_DASH] = ACTIONS(2306),
    [anon_sym_AMP] = ACTIONS(2306),
    [anon_sym_PIPE] = ACTIONS(2306),
    [anon_sym_LT] = ACTIONS(2306),
    [anon_sym_DOT_DOT] = ACTIONS(2306),
    [anon_sym_COLON_COLON] = ACTIONS(2306),
    [anon_sym_POUND] = ACTIONS(2306),
    [anon_sym_SQUOTE] = ACTIONS(2308),
    [anon_sym_async] = ACTIONS(2308),
    [anon_sym_auto] = ACTIONS(2308),
    [anon_sym_await] = ACTIONS(2308),
    [anon_sym_become] = ACTIONS(2308),
    [anon_sym_break] = ACTIONS(2308),
    [anon_sym_const] = ACTIONS(2308),
    [anon_sym_continue] = ACTIONS(2308),
    [anon_sym_default] = ACTIONS(2308),
    [anon_sym_enum] = ACTIONS(2308),
    [anon_sym_fn] = ACTIONS(2308),
    [anon_sym_for] = ACTIONS(2308),
    [anon_sym_gen] = ACTIONS(2308),
    [anon_sym_if] = ACTIONS(2308),
    [anon_sym_impl] = ACTIONS(2308),
    [anon_sym_let] = ACTIONS(2308),
    [anon_sym_loop] = ACTIONS(2308),
    [anon_sym_match] = ACTIONS(2308),
    [anon_sym_mod] = ACTIONS(2308),
    [anon_sym_pub] = ACTIONS(2308),
    [anon_sym_return] = ACTIONS(2308),
    [anon_sym_raw] = ACTIONS(2308),
    [anon_sym_static] = ACTIONS(2308),
    [anon_sym_struct] = ACTIONS(2308),
    [anon_sym_trait] = ACTIONS(2308),
    [anon_sym_type] = ACTIONS(2308),
    [anon_sym_union] = ACTIONS(2308),
    [anon_sym_unsafe] = ACTIONS(2308),
    [anon_sym_use] = ACTIONS(2308),
    [anon_sym_while] = ACTIONS(2308),
    [anon_sym_extern] = ACTIONS(2308),
    [anon_sym_safe] = ACTIONS(2308),
    [anon_sym_yield] = ACTIONS(2308),
    [anon_sym_move] = ACTIONS(2308),
    [anon_sym_try] = ACTIONS(2308),
    [sym_integer_literal] = ACTIONS(2306),
    [aux_sym_string_literal_token1] = ACTIONS(2306),
    [sym_char_literal] = ACTIONS(2306),
    [anon_sym_true] = ACTIONS(2308),
    [anon_sym_false] = ACTIONS(2308),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2308),
    [sym_super] = ACTIONS(2308),
    [sym_crate] = ACTIONS(2308),
    [aux_sym_metavariable_token1] = ACTIONS(2306),
    [sym__raw_string_literal_start] = ACTIONS(2306),
    [sym_float_literal] = ACTIONS(2306),
    [sym__outer_block_doc_comment_start] = ACTIONS(2306),
    [sym__outer_line_doc_comment_start] = ACTIONS(2306),
  },
  [STATE(634)] = {
    [sym_line_comment] = STATE(634),
    [sym_block_comment] = STATE(634),
    [ts_builtin_sym_end] = ACTIONS(2310),
    [sym_identifier] = ACTIONS(2312),
    [anon_sym_SEMI] = ACTIONS(2310),
    [anon_sym_macro] = ACTIONS(2312),
    [anon_sym_LPAREN] = ACTIONS(2310),
    [anon_sym_LBRACE] = ACTIONS(2310),
    [anon_sym_RBRACE] = ACTIONS(2310),
    [anon_sym_macro_rules] = ACTIONS(2312),
    [anon_sym_BANG] = ACTIONS(2310),
    [anon_sym_LBRACK] = ACTIONS(2310),
    [anon_sym_STAR] = ACTIONS(2310),
    [anon_sym_u8] = ACTIONS(2312),
    [anon_sym_i8] = ACTIONS(2312),
    [anon_sym_u16] = ACTIONS(2312),
    [anon_sym_i16] = ACTIONS(2312),
    [anon_sym_u32] = ACTIONS(2312),
    [anon_sym_i32] = ACTIONS(2312),
    [anon_sym_u64] = ACTIONS(2312),
    [anon_sym_i64] = ACTIONS(2312),
    [anon_sym_u128] = ACTIONS(2312),
    [anon_sym_i128] = ACTIONS(2312),
    [anon_sym_isize] = ACTIONS(2312),
    [anon_sym_usize] = ACTIONS(2312),
    [anon_sym_f32] = ACTIONS(2312),
    [anon_sym_f64] = ACTIONS(2312),
    [anon_sym_bool] = ACTIONS(2312),
    [anon_sym_str] = ACTIONS(2312),
    [anon_sym_char] = ACTIONS(2312),
    [anon_sym_DASH] = ACTIONS(2310),
    [anon_sym_AMP] = ACTIONS(2310),
    [anon_sym_PIPE] = ACTIONS(2310),
    [anon_sym_LT] = ACTIONS(2310),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_COLON_COLON] = ACTIONS(2310),
    [anon_sym_POUND] = ACTIONS(2310),
    [anon_sym_SQUOTE] = ACTIONS(2312),
    [anon_sym_async] = ACTIONS(2312),
    [anon_sym_auto] = ACTIONS(2312),
    [anon_sym_await] = ACTIONS(2312),
    [anon_sym_become] = ACTIONS(2312),
    [anon_sym_break] = ACTIONS(2312),
    [anon_sym_const] = ACTIONS(2312),
    [anon_sym_continue] = ACTIONS(2312),
    [anon_sym_default] = ACTIONS(2312),
    [anon_sym_enum] = ACTIONS(2312),
    [anon_sym_fn] = ACTIONS(2312),
    [anon_sym_for] = ACTIONS(2312),
    [anon_sym_gen] = ACTIONS(2312),
    [anon_sym_if] = ACTIONS(2312),
    [anon_sym_impl] = ACTIONS(2312),
    [anon_sym_let] = ACTIONS(2312),
    [anon_sym_loop] = ACTIONS(2312),
    [anon_sym_match] = ACTIONS(2312),
    [anon_sym_mod] = ACTIONS(2312),
    [anon_sym_pub] = ACTIONS(2312),
    [anon_sym_return] = ACTIONS(2312),
    [anon_sym_raw] = ACTIONS(2312),
    [anon_sym_static] = ACTIONS(2312),
    [anon_sym_struct] = ACTIONS(2312),
    [anon_sym_trait] = ACTIONS(2312),
    [anon_sym_type] = ACTIONS(2312),
    [anon_sym_union] = ACTIONS(2312),
    [anon_sym_unsafe] = ACTIONS(2312),
    [anon_sym_use] = ACTIONS(2312),
    [anon_sym_while] = ACTIONS(2312),
    [anon_sym_extern] = ACTIONS(2312),
    [anon_sym_safe] = ACTIONS(2312),
    [anon_sym_yield] = ACTIONS(2312),
    [anon_sym_move] = ACTIONS(2312),
    [anon_sym_try] = ACTIONS(2312),
    [sym_integer_literal] = ACTIONS(2310),
    [aux_sym_string_literal_token1] = ACTIONS(2310),
    [sym_char_literal] = ACTIONS(2310),
    [anon_sym_true] = ACTIONS(2312),
    [anon_sym_false] = ACTIONS(2312),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2312),
    [sym_super] = ACTIONS(2312),
    [sym_crate] = ACTIONS(2312),
    [aux_sym_metavariable_token1] = ACTIONS(2310),
    [sym__raw_string_literal_start] = ACTIONS(2310),
    [sym_float_literal] = ACTIONS(2310),
    [sym__outer_block_doc_comment_start] = ACTIONS(2310),
    [sym__outer_line_doc_comment_start] = ACTIONS(2310),
  },
  [STATE(635)] = {
    [sym_line_comment] = STATE(635),
    [sym_block_comment] = STATE(635),
    [ts_builtin_sym_end] = ACTIONS(2314),
    [sym_identifier] = ACTIONS(2316),
    [anon_sym_SEMI] = ACTIONS(2314),
    [anon_sym_macro] = ACTIONS(2316),
    [anon_sym_LPAREN] = ACTIONS(2314),
    [anon_sym_LBRACE] = ACTIONS(2314),
    [anon_sym_RBRACE] = ACTIONS(2314),
    [anon_sym_macro_rules] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2314),
    [anon_sym_LBRACK] = ACTIONS(2314),
    [anon_sym_STAR] = ACTIONS(2314),
    [anon_sym_u8] = ACTIONS(2316),
    [anon_sym_i8] = ACTIONS(2316),
    [anon_sym_u16] = ACTIONS(2316),
    [anon_sym_i16] = ACTIONS(2316),
    [anon_sym_u32] = ACTIONS(2316),
    [anon_sym_i32] = ACTIONS(2316),
    [anon_sym_u64] = ACTIONS(2316),
    [anon_sym_i64] = ACTIONS(2316),
    [anon_sym_u128] = ACTIONS(2316),
    [anon_sym_i128] = ACTIONS(2316),
    [anon_sym_isize] = ACTIONS(2316),
    [anon_sym_usize] = ACTIONS(2316),
    [anon_sym_f32] = ACTIONS(2316),
    [anon_sym_f64] = ACTIONS(2316),
    [anon_sym_bool] = ACTIONS(2316),
    [anon_sym_str] = ACTIONS(2316),
    [anon_sym_char] = ACTIONS(2316),
    [anon_sym_DASH] = ACTIONS(2314),
    [anon_sym_AMP] = ACTIONS(2314),
    [anon_sym_PIPE] = ACTIONS(2314),
    [anon_sym_LT] = ACTIONS(2314),
    [anon_sym_DOT_DOT] = ACTIONS(2314),
    [anon_sym_COLON_COLON] = ACTIONS(2314),
    [anon_sym_POUND] = ACTIONS(2314),
    [anon_sym_SQUOTE] = ACTIONS(2316),
    [anon_sym_async] = ACTIONS(2316),
    [anon_sym_auto] = ACTIONS(2316),
    [anon_sym_await] = ACTIONS(2316),
    [anon_sym_become] = ACTIONS(2316),
    [anon_sym_break] = ACTIONS(2316),
    [anon_sym_const] = ACTIONS(2316),
    [anon_sym_continue] = ACTIONS(2316),
    [anon_sym_default] = ACTIONS(2316),
    [anon_sym_enum] = ACTIONS(2316),
    [anon_sym_fn] = ACTIONS(2316),
    [anon_sym_for] = ACTIONS(2316),
    [anon_sym_gen] = ACTIONS(2316),
    [anon_sym_if] = ACTIONS(2316),
    [anon_sym_impl] = ACTIONS(2316),
    [anon_sym_let] = ACTIONS(2316),
    [anon_sym_loop] = ACTIONS(2316),
    [anon_sym_match] = ACTIONS(2316),
    [anon_sym_mod] = ACTIONS(2316),
    [anon_sym_pub] = ACTIONS(2316),
    [anon_sym_return] = ACTIONS(2316),
    [anon_sym_raw] = ACTIONS(2316),
    [anon_sym_static] = ACTIONS(2316),
    [anon_sym_struct] = ACTIONS(2316),
    [anon_sym_trait] = ACTIONS(2316),
    [anon_sym_type] = ACTIONS(2316),
    [anon_sym_union] = ACTIONS(2316),
    [anon_sym_unsafe] = ACTIONS(2316),
    [anon_sym_use] = ACTIONS(2316),
    [anon_sym_while] = ACTIONS(2316),
    [anon_sym_extern] = ACTIONS(2316),
    [anon_sym_safe] = ACTIONS(2316),
    [anon_sym_yield] = ACTIONS(2316),
    [anon_sym_move] = ACTIONS(2316),
    [anon_sym_try] = ACTIONS(2316),
    [sym_integer_literal] = ACTIONS(2314),
    [aux_sym_string_literal_token1] = ACTIONS(2314),
    [sym_char_literal] = ACTIONS(2314),
    [anon_sym_true] = ACTIONS(2316),
    [anon_sym_false] = ACTIONS(2316),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2316),
    [sym_super] = ACTIONS(2316),
    [sym_crate] = ACTIONS(2316),
    [aux_sym_metavariable_token1] = ACTIONS(2314),
    [sym__raw_string_literal_start] = ACTIONS(2314),
    [sym_float_literal] = ACTIONS(2314),
    [sym__outer_block_doc_comment_start] = ACTIONS(2314),
    [sym__outer_line_doc_comment_start] = ACTIONS(2314),
  },
  [STATE(636)] = {
    [sym_line_comment] = STATE(636),
    [sym_block_comment] = STATE(636),
    [ts_builtin_sym_end] = ACTIONS(2318),
    [sym_identifier] = ACTIONS(2320),
    [anon_sym_SEMI] = ACTIONS(2318),
    [anon_sym_macro] = ACTIONS(2320),
    [anon_sym_LPAREN] = ACTIONS(2318),
    [anon_sym_LBRACE] = ACTIONS(2318),
    [anon_sym_RBRACE] = ACTIONS(2318),
    [anon_sym_macro_rules] = ACTIONS(2320),
    [anon_sym_BANG] = ACTIONS(2318),
    [anon_sym_LBRACK] = ACTIONS(2318),
    [anon_sym_STAR] = ACTIONS(2318),
    [anon_sym_u8] = ACTIONS(2320),
    [anon_sym_i8] = ACTIONS(2320),
    [anon_sym_u16] = ACTIONS(2320),
    [anon_sym_i16] = ACTIONS(2320),
    [anon_sym_u32] = ACTIONS(2320),
    [anon_sym_i32] = ACTIONS(2320),
    [anon_sym_u64] = ACTIONS(2320),
    [anon_sym_i64] = ACTIONS(2320),
    [anon_sym_u128] = ACTIONS(2320),
    [anon_sym_i128] = ACTIONS(2320),
    [anon_sym_isize] = ACTIONS(2320),
    [anon_sym_usize] = ACTIONS(2320),
    [anon_sym_f32] = ACTIONS(2320),
    [anon_sym_f64] = ACTIONS(2320),
    [anon_sym_bool] = ACTIONS(2320),
    [anon_sym_str] = ACTIONS(2320),
    [anon_sym_char] = ACTIONS(2320),
    [anon_sym_DASH] = ACTIONS(2318),
    [anon_sym_AMP] = ACTIONS(2318),
    [anon_sym_PIPE] = ACTIONS(2318),
    [anon_sym_LT] = ACTIONS(2318),
    [anon_sym_DOT_DOT] = ACTIONS(2318),
    [anon_sym_COLON_COLON] = ACTIONS(2318),
    [anon_sym_POUND] = ACTIONS(2318),
    [anon_sym_SQUOTE] = ACTIONS(2320),
    [anon_sym_async] = ACTIONS(2320),
    [anon_sym_auto] = ACTIONS(2320),
    [anon_sym_await] = ACTIONS(2320),
    [anon_sym_become] = ACTIONS(2320),
    [anon_sym_break] = ACTIONS(2320),
    [anon_sym_const] = ACTIONS(2320),
    [anon_sym_continue] = ACTIONS(2320),
    [anon_sym_default] = ACTIONS(2320),
    [anon_sym_enum] = ACTIONS(2320),
    [anon_sym_fn] = ACTIONS(2320),
    [anon_sym_for] = ACTIONS(2320),
    [anon_sym_gen] = ACTIONS(2320),
    [anon_sym_if] = ACTIONS(2320),
    [anon_sym_impl] = ACTIONS(2320),
    [anon_sym_let] = ACTIONS(2320),
    [anon_sym_loop] = ACTIONS(2320),
    [anon_sym_match] = ACTIONS(2320),
    [anon_sym_mod] = ACTIONS(2320),
    [anon_sym_pub] = ACTIONS(2320),
    [anon_sym_return] = ACTIONS(2320),
    [anon_sym_raw] = ACTIONS(2320),
    [anon_sym_static] = ACTIONS(2320),
    [anon_sym_struct] = ACTIONS(2320),
    [anon_sym_trait] = ACTIONS(2320),
    [anon_sym_type] = ACTIONS(2320),
    [anon_sym_union] = ACTIONS(2320),
    [anon_sym_unsafe] = ACTIONS(2320),
    [anon_sym_use] = ACTIONS(2320),
    [anon_sym_while] = ACTIONS(2320),
    [anon_sym_extern] = ACTIONS(2320),
    [anon_sym_safe] = ACTIONS(2320),
    [anon_sym_yield] = ACTIONS(2320),
    [anon_sym_move] = ACTIONS(2320),
    [anon_sym_try] = ACTIONS(2320),
    [sym_integer_literal] = ACTIONS(2318),
    [aux_sym_string_literal_token1] = ACTIONS(2318),
    [sym_char_literal] = ACTIONS(2318),
    [anon_sym_true] = ACTIONS(2320),
    [anon_sym_false] = ACTIONS(2320),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2320),
    [sym_super] = ACTIONS(2320),
    [sym_crate] = ACTIONS(2320),
    [aux_sym_metavariable_token1] = ACTIONS(2318),
    [sym__raw_string_literal_start] = ACTIONS(2318),
    [sym_float_literal] = ACTIONS(2318),
    [sym__outer_block_doc_comment_start] = ACTIONS(2318),
    [sym__outer_line_doc_comment_start] = ACTIONS(2318),
  },
  [STATE(637)] = {
    [sym_line_comment] = STATE(637),
    [sym_block_comment] = STATE(637),
    [ts_builtin_sym_end] = ACTIONS(2322),
    [sym_identifier] = ACTIONS(2324),
    [anon_sym_SEMI] = ACTIONS(2322),
    [anon_sym_macro] = ACTIONS(2324),
    [anon_sym_LPAREN] = ACTIONS(2322),
    [anon_sym_LBRACE] = ACTIONS(2322),
    [anon_sym_RBRACE] = ACTIONS(2322),
    [anon_sym_macro_rules] = ACTIONS(2324),
    [anon_sym_BANG] = ACTIONS(2322),
    [anon_sym_LBRACK] = ACTIONS(2322),
    [anon_sym_STAR] = ACTIONS(2322),
    [anon_sym_u8] = ACTIONS(2324),
    [anon_sym_i8] = ACTIONS(2324),
    [anon_sym_u16] = ACTIONS(2324),
    [anon_sym_i16] = ACTIONS(2324),
    [anon_sym_u32] = ACTIONS(2324),
    [anon_sym_i32] = ACTIONS(2324),
    [anon_sym_u64] = ACTIONS(2324),
    [anon_sym_i64] = ACTIONS(2324),
    [anon_sym_u128] = ACTIONS(2324),
    [anon_sym_i128] = ACTIONS(2324),
    [anon_sym_isize] = ACTIONS(2324),
    [anon_sym_usize] = ACTIONS(2324),
    [anon_sym_f32] = ACTIONS(2324),
    [anon_sym_f64] = ACTIONS(2324),
    [anon_sym_bool] = ACTIONS(2324),
    [anon_sym_str] = ACTIONS(2324),
    [anon_sym_char] = ACTIONS(2324),
    [anon_sym_DASH] = ACTIONS(2322),
    [anon_sym_AMP] = ACTIONS(2322),
    [anon_sym_PIPE] = ACTIONS(2322),
    [anon_sym_LT] = ACTIONS(2322),
    [anon_sym_DOT_DOT] = ACTIONS(2322),
    [anon_sym_COLON_COLON] = ACTIONS(2322),
    [anon_sym_POUND] = ACTIONS(2322),
    [anon_sym_SQUOTE] = ACTIONS(2324),
    [anon_sym_async] = ACTIONS(2324),
    [anon_sym_auto] = ACTIONS(2324),
    [anon_sym_await] = ACTIONS(2324),
    [anon_sym_become] = ACTIONS(2324),
    [anon_sym_break] = ACTIONS(2324),
    [anon_sym_const] = ACTIONS(2324),
    [anon_sym_continue] = ACTIONS(2324),
    [anon_sym_default] = ACTIONS(2324),
    [anon_sym_enum] = ACTIONS(2324),
    [anon_sym_fn] = ACTIONS(2324),
    [anon_sym_for] = ACTIONS(2324),
    [anon_sym_gen] = ACTIONS(2324),
    [anon_sym_if] = ACTIONS(2324),
    [anon_sym_impl] = ACTIONS(2324),
    [anon_sym_let] = ACTIONS(2324),
    [anon_sym_loop] = ACTIONS(2324),
    [anon_sym_match] = ACTIONS(2324),
    [anon_sym_mod] = ACTIONS(2324),
    [anon_sym_pub] = ACTIONS(2324),
    [anon_sym_return] = ACTIONS(2324),
    [anon_sym_raw] = ACTIONS(2324),
    [anon_sym_static] = ACTIONS(2324),
    [anon_sym_struct] = ACTIONS(2324),
    [anon_sym_trait] = ACTIONS(2324),
    [anon_sym_type] = ACTIONS(2324),
    [anon_sym_union] = ACTIONS(2324),
    [anon_sym_unsafe] = ACTIONS(2324),
    [anon_sym_use] = ACTIONS(2324),
    [anon_sym_while] = ACTIONS(2324),
    [anon_sym_extern] = ACTIONS(2324),
    [anon_sym_safe] = ACTIONS(2324),
    [anon_sym_yield] = ACTIONS(2324),
    [anon_sym_move] = ACTIONS(2324),
    [anon_sym_try] = ACTIONS(2324),
    [sym_integer_literal] = ACTIONS(2322),
    [aux_sym_string_literal_token1] = ACTIONS(2322),
    [sym_char_literal] = ACTIONS(2322),
    [anon_sym_true] = ACTIONS(2324),
    [anon_sym_false] = ACTIONS(2324),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2324),
    [sym_super] = ACTIONS(2324),
    [sym_crate] = ACTIONS(2324),
    [aux_sym_metavariable_token1] = ACTIONS(2322),
    [sym__raw_string_literal_start] = ACTIONS(2322),
    [sym_float_literal] = ACTIONS(2322),
    [sym__outer_block_doc_comment_start] = ACTIONS(2322),
    [sym__outer_line_doc_comment_start] = ACTIONS(2322),
  },
  [STATE(638)] = {
    [sym_line_comment] = STATE(638),
    [sym_block_comment] = STATE(638),
    [ts_builtin_sym_end] = ACTIONS(2326),
    [sym_identifier] = ACTIONS(2328),
    [anon_sym_SEMI] = ACTIONS(2326),
    [anon_sym_macro] = ACTIONS(2328),
    [anon_sym_LPAREN] = ACTIONS(2326),
    [anon_sym_LBRACE] = ACTIONS(2326),
    [anon_sym_RBRACE] = ACTIONS(2326),
    [anon_sym_macro_rules] = ACTIONS(2328),
    [anon_sym_BANG] = ACTIONS(2326),
    [anon_sym_LBRACK] = ACTIONS(2326),
    [anon_sym_STAR] = ACTIONS(2326),
    [anon_sym_u8] = ACTIONS(2328),
    [anon_sym_i8] = ACTIONS(2328),
    [anon_sym_u16] = ACTIONS(2328),
    [anon_sym_i16] = ACTIONS(2328),
    [anon_sym_u32] = ACTIONS(2328),
    [anon_sym_i32] = ACTIONS(2328),
    [anon_sym_u64] = ACTIONS(2328),
    [anon_sym_i64] = ACTIONS(2328),
    [anon_sym_u128] = ACTIONS(2328),
    [anon_sym_i128] = ACTIONS(2328),
    [anon_sym_isize] = ACTIONS(2328),
    [anon_sym_usize] = ACTIONS(2328),
    [anon_sym_f32] = ACTIONS(2328),
    [anon_sym_f64] = ACTIONS(2328),
    [anon_sym_bool] = ACTIONS(2328),
    [anon_sym_str] = ACTIONS(2328),
    [anon_sym_char] = ACTIONS(2328),
    [anon_sym_DASH] = ACTIONS(2326),
    [anon_sym_AMP] = ACTIONS(2326),
    [anon_sym_PIPE] = ACTIONS(2326),
    [anon_sym_LT] = ACTIONS(2326),
    [anon_sym_DOT_DOT] = ACTIONS(2326),
    [anon_sym_COLON_COLON] = ACTIONS(2326),
    [anon_sym_POUND] = ACTIONS(2326),
    [anon_sym_SQUOTE] = ACTIONS(2328),
    [anon_sym_async] = ACTIONS(2328),
    [anon_sym_auto] = ACTIONS(2328),
    [anon_sym_await] = ACTIONS(2328),
    [anon_sym_become] = ACTIONS(2328),
    [anon_sym_break] = ACTIONS(2328),
    [anon_sym_const] = ACTIONS(2328),
    [anon_sym_continue] = ACTIONS(2328),
    [anon_sym_default] = ACTIONS(2328),
    [anon_sym_enum] = ACTIONS(2328),
    [anon_sym_fn] = ACTIONS(2328),
    [anon_sym_for] = ACTIONS(2328),
    [anon_sym_gen] = ACTIONS(2328),
    [anon_sym_if] = ACTIONS(2328),
    [anon_sym_impl] = ACTIONS(2328),
    [anon_sym_let] = ACTIONS(2328),
    [anon_sym_loop] = ACTIONS(2328),
    [anon_sym_match] = ACTIONS(2328),
    [anon_sym_mod] = ACTIONS(2328),
    [anon_sym_pub] = ACTIONS(2328),
    [anon_sym_return] = ACTIONS(2328),
    [anon_sym_raw] = ACTIONS(2328),
    [anon_sym_static] = ACTIONS(2328),
    [anon_sym_struct] = ACTIONS(2328),
    [anon_sym_trait] = ACTIONS(2328),
    [anon_sym_type] = ACTIONS(2328),
    [anon_sym_union] = ACTIONS(2328),
    [anon_sym_unsafe] = ACTIONS(2328),
    [anon_sym_use] = ACTIONS(2328),
    [anon_sym_while] = ACTIONS(2328),
    [anon_sym_extern] = ACTIONS(2328),
    [anon_sym_safe] = ACTIONS(2328),
    [anon_sym_yield] = ACTIONS(2328),
    [anon_sym_move] = ACTIONS(2328),
    [anon_sym_try] = ACTIONS(2328),
    [sym_integer_literal] = ACTIONS(2326),
    [aux_sym_string_literal_token1] = ACTIONS(2326),
    [sym_char_literal] = ACTIONS(2326),
    [anon_sym_true] = ACTIONS(2328),
    [anon_sym_false] = ACTIONS(2328),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2328),
    [sym_super] = ACTIONS(2328),
    [sym_crate] = ACTIONS(2328),
    [aux_sym_metavariable_token1] = ACTIONS(2326),
    [sym__raw_string_literal_start] = ACTIONS(2326),
    [sym_float_literal] = ACTIONS(2326),
    [sym__outer_block_doc_comment_start] = ACTIONS(2326),
    [sym__outer_line_doc_comment_start] = ACTIONS(2326),
  },
  [STATE(639)] = {
    [sym_line_comment] = STATE(639),
    [sym_block_comment] = STATE(639),
    [ts_builtin_sym_end] = ACTIONS(2330),
    [sym_identifier] = ACTIONS(2332),
    [anon_sym_SEMI] = ACTIONS(2330),
    [anon_sym_macro] = ACTIONS(2332),
    [anon_sym_LPAREN] = ACTIONS(2330),
    [anon_sym_LBRACE] = ACTIONS(2330),
    [anon_sym_RBRACE] = ACTIONS(2330),
    [anon_sym_macro_rules] = ACTIONS(2332),
    [anon_sym_BANG] = ACTIONS(2330),
    [anon_sym_LBRACK] = ACTIONS(2330),
    [anon_sym_STAR] = ACTIONS(2330),
    [anon_sym_u8] = ACTIONS(2332),
    [anon_sym_i8] = ACTIONS(2332),
    [anon_sym_u16] = ACTIONS(2332),
    [anon_sym_i16] = ACTIONS(2332),
    [anon_sym_u32] = ACTIONS(2332),
    [anon_sym_i32] = ACTIONS(2332),
    [anon_sym_u64] = ACTIONS(2332),
    [anon_sym_i64] = ACTIONS(2332),
    [anon_sym_u128] = ACTIONS(2332),
    [anon_sym_i128] = ACTIONS(2332),
    [anon_sym_isize] = ACTIONS(2332),
    [anon_sym_usize] = ACTIONS(2332),
    [anon_sym_f32] = ACTIONS(2332),
    [anon_sym_f64] = ACTIONS(2332),
    [anon_sym_bool] = ACTIONS(2332),
    [anon_sym_str] = ACTIONS(2332),
    [anon_sym_char] = ACTIONS(2332),
    [anon_sym_DASH] = ACTIONS(2330),
    [anon_sym_AMP] = ACTIONS(2330),
    [anon_sym_PIPE] = ACTIONS(2330),
    [anon_sym_LT] = ACTIONS(2330),
    [anon_sym_DOT_DOT] = ACTIONS(2330),
    [anon_sym_COLON_COLON] = ACTIONS(2330),
    [anon_sym_POUND] = ACTIONS(2330),
    [anon_sym_SQUOTE] = ACTIONS(2332),
    [anon_sym_async] = ACTIONS(2332),
    [anon_sym_auto] = ACTIONS(2332),
    [anon_sym_await] = ACTIONS(2332),
    [anon_sym_become] = ACTIONS(2332),
    [anon_sym_break] = ACTIONS(2332),
    [anon_sym_const] = ACTIONS(2332),
    [anon_sym_continue] = ACTIONS(2332),
    [anon_sym_default] = ACTIONS(2332),
    [anon_sym_enum] = ACTIONS(2332),
    [anon_sym_fn] = ACTIONS(2332),
    [anon_sym_for] = ACTIONS(2332),
    [anon_sym_gen] = ACTIONS(2332),
    [anon_sym_if] = ACTIONS(2332),
    [anon_sym_impl] = ACTIONS(2332),
    [anon_sym_let] = ACTIONS(2332),
    [anon_sym_loop] = ACTIONS(2332),
    [anon_sym_match] = ACTIONS(2332),
    [anon_sym_mod] = ACTIONS(2332),
    [anon_sym_pub] = ACTIONS(2332),
    [anon_sym_return] = ACTIONS(2332),
    [anon_sym_raw] = ACTIONS(2332),
    [anon_sym_static] = ACTIONS(2332),
    [anon_sym_struct] = ACTIONS(2332),
    [anon_sym_trait] = ACTIONS(2332),
    [anon_sym_type] = ACTIONS(2332),
    [anon_sym_union] = ACTIONS(2332),
    [anon_sym_unsafe] = ACTIONS(2332),
    [anon_sym_use] = ACTIONS(2332),
    [anon_sym_while] = ACTIONS(2332),
    [anon_sym_extern] = ACTIONS(2332),
    [anon_sym_safe] = ACTIONS(2332),
    [anon_sym_yield] = ACTIONS(2332),
    [anon_sym_move] = ACTIONS(2332),
    [anon_sym_try] = ACTIONS(2332),
    [sym_integer_literal] = ACTIONS(2330),
    [aux_sym_string_literal_token1] = ACTIONS(2330),
    [sym_char_literal] = ACTIONS(2330),
    [anon_sym_true] = ACTIONS(2332),
    [anon_sym_false] = ACTIONS(2332),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2332),
    [sym_super] = ACTIONS(2332),
    [sym_crate] = ACTIONS(2332),
    [aux_sym_metavariable_token1] = ACTIONS(2330),
    [sym__raw_string_literal_start] = ACTIONS(2330),
    [sym_float_literal] = ACTIONS(2330),
    [sym__outer_block_doc_comment_start] = ACTIONS(2330),
    [sym__outer_line_doc_comment_start] = ACTIONS(2330),
  },
  [STATE(640)] = {
    [sym_line_comment] = STATE(640),
    [sym_block_comment] = STATE(640),
    [ts_builtin_sym_end] = ACTIONS(2334),
    [sym_identifier] = ACTIONS(2336),
    [anon_sym_SEMI] = ACTIONS(2334),
    [anon_sym_macro] = ACTIONS(2336),
    [anon_sym_LPAREN] = ACTIONS(2334),
    [anon_sym_LBRACE] = ACTIONS(2334),
    [anon_sym_RBRACE] = ACTIONS(2334),
    [anon_sym_macro_rules] = ACTIONS(2336),
    [anon_sym_BANG] = ACTIONS(2334),
    [anon_sym_LBRACK] = ACTIONS(2334),
    [anon_sym_STAR] = ACTIONS(2334),
    [anon_sym_u8] = ACTIONS(2336),
    [anon_sym_i8] = ACTIONS(2336),
    [anon_sym_u16] = ACTIONS(2336),
    [anon_sym_i16] = ACTIONS(2336),
    [anon_sym_u32] = ACTIONS(2336),
    [anon_sym_i32] = ACTIONS(2336),
    [anon_sym_u64] = ACTIONS(2336),
    [anon_sym_i64] = ACTIONS(2336),
    [anon_sym_u128] = ACTIONS(2336),
    [anon_sym_i128] = ACTIONS(2336),
    [anon_sym_isize] = ACTIONS(2336),
    [anon_sym_usize] = ACTIONS(2336),
    [anon_sym_f32] = ACTIONS(2336),
    [anon_sym_f64] = ACTIONS(2336),
    [anon_sym_bool] = ACTIONS(2336),
    [anon_sym_str] = ACTIONS(2336),
    [anon_sym_char] = ACTIONS(2336),
    [anon_sym_DASH] = ACTIONS(2334),
    [anon_sym_AMP] = ACTIONS(2334),
    [anon_sym_PIPE] = ACTIONS(2334),
    [anon_sym_LT] = ACTIONS(2334),
    [anon_sym_DOT_DOT] = ACTIONS(2334),
    [anon_sym_COLON_COLON] = ACTIONS(2334),
    [anon_sym_POUND] = ACTIONS(2334),
    [anon_sym_SQUOTE] = ACTIONS(2336),
    [anon_sym_async] = ACTIONS(2336),
    [anon_sym_auto] = ACTIONS(2336),
    [anon_sym_await] = ACTIONS(2336),
    [anon_sym_become] = ACTIONS(2336),
    [anon_sym_break] = ACTIONS(2336),
    [anon_sym_const] = ACTIONS(2336),
    [anon_sym_continue] = ACTIONS(2336),
    [anon_sym_default] = ACTIONS(2336),
    [anon_sym_enum] = ACTIONS(2336),
    [anon_sym_fn] = ACTIONS(2336),
    [anon_sym_for] = ACTIONS(2336),
    [anon_sym_gen] = ACTIONS(2336),
    [anon_sym_if] = ACTIONS(2336),
    [anon_sym_impl] = ACTIONS(2336),
    [anon_sym_let] = ACTIONS(2336),
    [anon_sym_loop] = ACTIONS(2336),
    [anon_sym_match] = ACTIONS(2336),
    [anon_sym_mod] = ACTIONS(2336),
    [anon_sym_pub] = ACTIONS(2336),
    [anon_sym_return] = ACTIONS(2336),
    [anon_sym_raw] = ACTIONS(2336),
    [anon_sym_static] = ACTIONS(2336),
    [anon_sym_struct] = ACTIONS(2336),
    [anon_sym_trait] = ACTIONS(2336),
    [anon_sym_type] = ACTIONS(2336),
    [anon_sym_union] = ACTIONS(2336),
    [anon_sym_unsafe] = ACTIONS(2336),
    [anon_sym_use] = ACTIONS(2336),
    [anon_sym_while] = ACTIONS(2336),
    [anon_sym_extern] = ACTIONS(2336),
    [anon_sym_safe] = ACTIONS(2336),
    [anon_sym_yield] = ACTIONS(2336),
    [anon_sym_move] = ACTIONS(2336),
    [anon_sym_try] = ACTIONS(2336),
    [sym_integer_literal] = ACTIONS(2334),
    [aux_sym_string_literal_token1] = ACTIONS(2334),
    [sym_char_literal] = ACTIONS(2334),
    [anon_sym_true] = ACTIONS(2336),
    [anon_sym_false] = ACTIONS(2336),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2336),
    [sym_super] = ACTIONS(2336),
    [sym_crate] = ACTIONS(2336),
    [aux_sym_metavariable_token1] = ACTIONS(2334),
    [sym__raw_string_literal_start] = ACTIONS(2334),
    [sym_float_literal] = ACTIONS(2334),
    [sym__outer_block_doc_comment_start] = ACTIONS(2334),
    [sym__outer_line_doc_comment_start] = ACTIONS(2334),
  },
  [STATE(641)] = {
    [sym_line_comment] = STATE(641),
    [sym_block_comment] = STATE(641),
    [ts_builtin_sym_end] = ACTIONS(2338),
    [sym_identifier] = ACTIONS(2340),
    [anon_sym_SEMI] = ACTIONS(2338),
    [anon_sym_macro] = ACTIONS(2340),
    [anon_sym_LPAREN] = ACTIONS(2338),
    [anon_sym_LBRACE] = ACTIONS(2338),
    [anon_sym_RBRACE] = ACTIONS(2338),
    [anon_sym_macro_rules] = ACTIONS(2340),
    [anon_sym_BANG] = ACTIONS(2338),
    [anon_sym_LBRACK] = ACTIONS(2338),
    [anon_sym_STAR] = ACTIONS(2338),
    [anon_sym_u8] = ACTIONS(2340),
    [anon_sym_i8] = ACTIONS(2340),
    [anon_sym_u16] = ACTIONS(2340),
    [anon_sym_i16] = ACTIONS(2340),
    [anon_sym_u32] = ACTIONS(2340),
    [anon_sym_i32] = ACTIONS(2340),
    [anon_sym_u64] = ACTIONS(2340),
    [anon_sym_i64] = ACTIONS(2340),
    [anon_sym_u128] = ACTIONS(2340),
    [anon_sym_i128] = ACTIONS(2340),
    [anon_sym_isize] = ACTIONS(2340),
    [anon_sym_usize] = ACTIONS(2340),
    [anon_sym_f32] = ACTIONS(2340),
    [anon_sym_f64] = ACTIONS(2340),
    [anon_sym_bool] = ACTIONS(2340),
    [anon_sym_str] = ACTIONS(2340),
    [anon_sym_char] = ACTIONS(2340),
    [anon_sym_DASH] = ACTIONS(2338),
    [anon_sym_AMP] = ACTIONS(2338),
    [anon_sym_PIPE] = ACTIONS(2338),
    [anon_sym_LT] = ACTIONS(2338),
    [anon_sym_DOT_DOT] = ACTIONS(2338),
    [anon_sym_COLON_COLON] = ACTIONS(2338),
    [anon_sym_POUND] = ACTIONS(2338),
    [anon_sym_SQUOTE] = ACTIONS(2340),
    [anon_sym_async] = ACTIONS(2340),
    [anon_sym_auto] = ACTIONS(2340),
    [anon_sym_await] = ACTIONS(2340),
    [anon_sym_become] = ACTIONS(2340),
    [anon_sym_break] = ACTIONS(2340),
    [anon_sym_const] = ACTIONS(2340),
    [anon_sym_continue] = ACTIONS(2340),
    [anon_sym_default] = ACTIONS(2340),
    [anon_sym_enum] = ACTIONS(2340),
    [anon_sym_fn] = ACTIONS(2340),
    [anon_sym_for] = ACTIONS(2340),
    [anon_sym_gen] = ACTIONS(2340),
    [anon_sym_if] = ACTIONS(2340),
    [anon_sym_impl] = ACTIONS(2340),
    [anon_sym_let] = ACTIONS(2340),
    [anon_sym_loop] = ACTIONS(2340),
    [anon_sym_match] = ACTIONS(2340),
    [anon_sym_mod] = ACTIONS(2340),
    [anon_sym_pub] = ACTIONS(2340),
    [anon_sym_return] = ACTIONS(2340),
    [anon_sym_raw] = ACTIONS(2340),
    [anon_sym_static] = ACTIONS(2340),
    [anon_sym_struct] = ACTIONS(2340),
    [anon_sym_trait] = ACTIONS(2340),
    [anon_sym_type] = ACTIONS(2340),
    [anon_sym_union] = ACTIONS(2340),
    [anon_sym_unsafe] = ACTIONS(2340),
    [anon_sym_use] = ACTIONS(2340),
    [anon_sym_while] = ACTIONS(2340),
    [anon_sym_extern] = ACTIONS(2340),
    [anon_sym_safe] = ACTIONS(2340),
    [anon_sym_yield] = ACTIONS(2340),
    [anon_sym_move] = ACTIONS(2340),
    [anon_sym_try] = ACTIONS(2340),
    [sym_integer_literal] = ACTIONS(2338),
    [aux_sym_string_literal_token1] = ACTIONS(2338),
    [sym_char_literal] = ACTIONS(2338),
    [anon_sym_true] = ACTIONS(2340),
    [anon_sym_false] = ACTIONS(2340),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2340),
    [sym_super] = ACTIONS(2340),
    [sym_crate] = ACTIONS(2340),
    [aux_sym_metavariable_token1] = ACTIONS(2338),
    [sym__raw_string_literal_start] = ACTIONS(2338),
    [sym_float_literal] = ACTIONS(2338),
    [sym__outer_block_doc_comment_start] = ACTIONS(2338),
    [sym__outer_line_doc_comment_start] = ACTIONS(2338),
  },
  [STATE(642)] = {
    [sym_line_comment] = STATE(642),
    [sym_block_comment] = STATE(642),
    [ts_builtin_sym_end] = ACTIONS(2342),
    [sym_identifier] = ACTIONS(2344),
    [anon_sym_SEMI] = ACTIONS(2342),
    [anon_sym_macro] = ACTIONS(2344),
    [anon_sym_LPAREN] = ACTIONS(2342),
    [anon_sym_LBRACE] = ACTIONS(2342),
    [anon_sym_RBRACE] = ACTIONS(2342),
    [anon_sym_macro_rules] = ACTIONS(2344),
    [anon_sym_BANG] = ACTIONS(2342),
    [anon_sym_LBRACK] = ACTIONS(2342),
    [anon_sym_STAR] = ACTIONS(2342),
    [anon_sym_u8] = ACTIONS(2344),
    [anon_sym_i8] = ACTIONS(2344),
    [anon_sym_u16] = ACTIONS(2344),
    [anon_sym_i16] = ACTIONS(2344),
    [anon_sym_u32] = ACTIONS(2344),
    [anon_sym_i32] = ACTIONS(2344),
    [anon_sym_u64] = ACTIONS(2344),
    [anon_sym_i64] = ACTIONS(2344),
    [anon_sym_u128] = ACTIONS(2344),
    [anon_sym_i128] = ACTIONS(2344),
    [anon_sym_isize] = ACTIONS(2344),
    [anon_sym_usize] = ACTIONS(2344),
    [anon_sym_f32] = ACTIONS(2344),
    [anon_sym_f64] = ACTIONS(2344),
    [anon_sym_bool] = ACTIONS(2344),
    [anon_sym_str] = ACTIONS(2344),
    [anon_sym_char] = ACTIONS(2344),
    [anon_sym_DASH] = ACTIONS(2342),
    [anon_sym_AMP] = ACTIONS(2342),
    [anon_sym_PIPE] = ACTIONS(2342),
    [anon_sym_LT] = ACTIONS(2342),
    [anon_sym_DOT_DOT] = ACTIONS(2342),
    [anon_sym_COLON_COLON] = ACTIONS(2342),
    [anon_sym_POUND] = ACTIONS(2342),
    [anon_sym_SQUOTE] = ACTIONS(2344),
    [anon_sym_async] = ACTIONS(2344),
    [anon_sym_auto] = ACTIONS(2344),
    [anon_sym_await] = ACTIONS(2344),
    [anon_sym_become] = ACTIONS(2344),
    [anon_sym_break] = ACTIONS(2344),
    [anon_sym_const] = ACTIONS(2344),
    [anon_sym_continue] = ACTIONS(2344),
    [anon_sym_default] = ACTIONS(2344),
    [anon_sym_enum] = ACTIONS(2344),
    [anon_sym_fn] = ACTIONS(2344),
    [anon_sym_for] = ACTIONS(2344),
    [anon_sym_gen] = ACTIONS(2344),
    [anon_sym_if] = ACTIONS(2344),
    [anon_sym_impl] = ACTIONS(2344),
    [anon_sym_let] = ACTIONS(2344),
    [anon_sym_loop] = ACTIONS(2344),
    [anon_sym_match] = ACTIONS(2344),
    [anon_sym_mod] = ACTIONS(2344),
    [anon_sym_pub] = ACTIONS(2344),
    [anon_sym_return] = ACTIONS(2344),
    [anon_sym_raw] = ACTIONS(2344),
    [anon_sym_static] = ACTIONS(2344),
    [anon_sym_struct] = ACTIONS(2344),
    [anon_sym_trait] = ACTIONS(2344),
    [anon_sym_type] = ACTIONS(2344),
    [anon_sym_union] = ACTIONS(2344),
    [anon_sym_unsafe] = ACTIONS(2344),
    [anon_sym_use] = ACTIONS(2344),
    [anon_sym_while] = ACTIONS(2344),
    [anon_sym_extern] = ACTIONS(2344),
    [anon_sym_safe] = ACTIONS(2344),
    [anon_sym_yield] = ACTIONS(2344),
    [anon_sym_move] = ACTIONS(2344),
    [anon_sym_try] = ACTIONS(2344),
    [sym_integer_literal] = ACTIONS(2342),
    [aux_sym_string_literal_token1] = ACTIONS(2342),
    [sym_char_literal] = ACTIONS(2342),
    [anon_sym_true] = ACTIONS(2344),
    [anon_sym_false] = ACTIONS(2344),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2344),
    [sym_super] = ACTIONS(2344),
    [sym_crate] = ACTIONS(2344),
    [aux_sym_metavariable_token1] = ACTIONS(2342),
    [sym__raw_string_literal_start] = ACTIONS(2342),
    [sym_float_literal] = ACTIONS(2342),
    [sym__outer_block_doc_comment_start] = ACTIONS(2342),
    [sym__outer_line_doc_comment_start] = ACTIONS(2342),
  },
  [STATE(643)] = {
    [sym_line_comment] = STATE(643),
    [sym_block_comment] = STATE(643),
    [ts_builtin_sym_end] = ACTIONS(2346),
    [sym_identifier] = ACTIONS(2348),
    [anon_sym_SEMI] = ACTIONS(2346),
    [anon_sym_macro] = ACTIONS(2348),
    [anon_sym_LPAREN] = ACTIONS(2346),
    [anon_sym_LBRACE] = ACTIONS(2346),
    [anon_sym_RBRACE] = ACTIONS(2346),
    [anon_sym_macro_rules] = ACTIONS(2348),
    [anon_sym_BANG] = ACTIONS(2346),
    [anon_sym_LBRACK] = ACTIONS(2346),
    [anon_sym_STAR] = ACTIONS(2346),
    [anon_sym_u8] = ACTIONS(2348),
    [anon_sym_i8] = ACTIONS(2348),
    [anon_sym_u16] = ACTIONS(2348),
    [anon_sym_i16] = ACTIONS(2348),
    [anon_sym_u32] = ACTIONS(2348),
    [anon_sym_i32] = ACTIONS(2348),
    [anon_sym_u64] = ACTIONS(2348),
    [anon_sym_i64] = ACTIONS(2348),
    [anon_sym_u128] = ACTIONS(2348),
    [anon_sym_i128] = ACTIONS(2348),
    [anon_sym_isize] = ACTIONS(2348),
    [anon_sym_usize] = ACTIONS(2348),
    [anon_sym_f32] = ACTIONS(2348),
    [anon_sym_f64] = ACTIONS(2348),
    [anon_sym_bool] = ACTIONS(2348),
    [anon_sym_str] = ACTIONS(2348),
    [anon_sym_char] = ACTIONS(2348),
    [anon_sym_DASH] = ACTIONS(2346),
    [anon_sym_AMP] = ACTIONS(2346),
    [anon_sym_PIPE] = ACTIONS(2346),
    [anon_sym_LT] = ACTIONS(2346),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_COLON_COLON] = ACTIONS(2346),
    [anon_sym_POUND] = ACTIONS(2346),
    [anon_sym_SQUOTE] = ACTIONS(2348),
    [anon_sym_async] = ACTIONS(2348),
    [anon_sym_auto] = ACTIONS(2348),
    [anon_sym_await] = ACTIONS(2348),
    [anon_sym_become] = ACTIONS(2348),
    [anon_sym_break] = ACTIONS(2348),
    [anon_sym_const] = ACTIONS(2348),
    [anon_sym_continue] = ACTIONS(2348),
    [anon_sym_default] = ACTIONS(2348),
    [anon_sym_enum] = ACTIONS(2348),
    [anon_sym_fn] = ACTIONS(2348),
    [anon_sym_for] = ACTIONS(2348),
    [anon_sym_gen] = ACTIONS(2348),
    [anon_sym_if] = ACTIONS(2348),
    [anon_sym_impl] = ACTIONS(2348),
    [anon_sym_let] = ACTIONS(2348),
    [anon_sym_loop] = ACTIONS(2348),
    [anon_sym_match] = ACTIONS(2348),
    [anon_sym_mod] = ACTIONS(2348),
    [anon_sym_pub] = ACTIONS(2348),
    [anon_sym_return] = ACTIONS(2348),
    [anon_sym_raw] = ACTIONS(2348),
    [anon_sym_static] = ACTIONS(2348),
    [anon_sym_struct] = ACTIONS(2348),
    [anon_sym_trait] = ACTIONS(2348),
    [anon_sym_type] = ACTIONS(2348),
    [anon_sym_union] = ACTIONS(2348),
    [anon_sym_unsafe] = ACTIONS(2348),
    [anon_sym_use] = ACTIONS(2348),
    [anon_sym_while] = ACTIONS(2348),
    [anon_sym_extern] = ACTIONS(2348),
    [anon_sym_safe] = ACTIONS(2348),
    [anon_sym_yield] = ACTIONS(2348),
    [anon_sym_move] = ACTIONS(2348),
    [anon_sym_try] = ACTIONS(2348),
    [sym_integer_literal] = ACTIONS(2346),
    [aux_sym_string_literal_token1] = ACTIONS(2346),
    [sym_char_literal] = ACTIONS(2346),
    [anon_sym_true] = ACTIONS(2348),
    [anon_sym_false] = ACTIONS(2348),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2348),
    [sym_super] = ACTIONS(2348),
    [sym_crate] = ACTIONS(2348),
    [aux_sym_metavariable_token1] = ACTIONS(2346),
    [sym__raw_string_literal_start] = ACTIONS(2346),
    [sym_float_literal] = ACTIONS(2346),
    [sym__outer_block_doc_comment_start] = ACTIONS(2346),
    [sym__outer_line_doc_comment_start] = ACTIONS(2346),
  },
  [STATE(644)] = {
    [sym_line_comment] = STATE(644),
    [sym_block_comment] = STATE(644),
    [ts_builtin_sym_end] = ACTIONS(2350),
    [sym_identifier] = ACTIONS(2352),
    [anon_sym_SEMI] = ACTIONS(2350),
    [anon_sym_macro] = ACTIONS(2352),
    [anon_sym_LPAREN] = ACTIONS(2350),
    [anon_sym_LBRACE] = ACTIONS(2350),
    [anon_sym_RBRACE] = ACTIONS(2350),
    [anon_sym_macro_rules] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2350),
    [anon_sym_LBRACK] = ACTIONS(2350),
    [anon_sym_STAR] = ACTIONS(2350),
    [anon_sym_u8] = ACTIONS(2352),
    [anon_sym_i8] = ACTIONS(2352),
    [anon_sym_u16] = ACTIONS(2352),
    [anon_sym_i16] = ACTIONS(2352),
    [anon_sym_u32] = ACTIONS(2352),
    [anon_sym_i32] = ACTIONS(2352),
    [anon_sym_u64] = ACTIONS(2352),
    [anon_sym_i64] = ACTIONS(2352),
    [anon_sym_u128] = ACTIONS(2352),
    [anon_sym_i128] = ACTIONS(2352),
    [anon_sym_isize] = ACTIONS(2352),
    [anon_sym_usize] = ACTIONS(2352),
    [anon_sym_f32] = ACTIONS(2352),
    [anon_sym_f64] = ACTIONS(2352),
    [anon_sym_bool] = ACTIONS(2352),
    [anon_sym_str] = ACTIONS(2352),
    [anon_sym_char] = ACTIONS(2352),
    [anon_sym_DASH] = ACTIONS(2350),
    [anon_sym_AMP] = ACTIONS(2350),
    [anon_sym_PIPE] = ACTIONS(2350),
    [anon_sym_LT] = ACTIONS(2350),
    [anon_sym_DOT_DOT] = ACTIONS(2350),
    [anon_sym_COLON_COLON] = ACTIONS(2350),
    [anon_sym_POUND] = ACTIONS(2350),
    [anon_sym_SQUOTE] = ACTIONS(2352),
    [anon_sym_async] = ACTIONS(2352),
    [anon_sym_auto] = ACTIONS(2352),
    [anon_sym_await] = ACTIONS(2352),
    [anon_sym_become] = ACTIONS(2352),
    [anon_sym_break] = ACTIONS(2352),
    [anon_sym_const] = ACTIONS(2352),
    [anon_sym_continue] = ACTIONS(2352),
    [anon_sym_default] = ACTIONS(2352),
    [anon_sym_enum] = ACTIONS(2352),
    [anon_sym_fn] = ACTIONS(2352),
    [anon_sym_for] = ACTIONS(2352),
    [anon_sym_gen] = ACTIONS(2352),
    [anon_sym_if] = ACTIONS(2352),
    [anon_sym_impl] = ACTIONS(2352),
    [anon_sym_let] = ACTIONS(2352),
    [anon_sym_loop] = ACTIONS(2352),
    [anon_sym_match] = ACTIONS(2352),
    [anon_sym_mod] = ACTIONS(2352),
    [anon_sym_pub] = ACTIONS(2352),
    [anon_sym_return] = ACTIONS(2352),
    [anon_sym_raw] = ACTIONS(2352),
    [anon_sym_static] = ACTIONS(2352),
    [anon_sym_struct] = ACTIONS(2352),
    [anon_sym_trait] = ACTIONS(2352),
    [anon_sym_type] = ACTIONS(2352),
    [anon_sym_union] = ACTIONS(2352),
    [anon_sym_unsafe] = ACTIONS(2352),
    [anon_sym_use] = ACTIONS(2352),
    [anon_sym_while] = ACTIONS(2352),
    [anon_sym_extern] = ACTIONS(2352),
    [anon_sym_safe] = ACTIONS(2352),
    [anon_sym_yield] = ACTIONS(2352),
    [anon_sym_move] = ACTIONS(2352),
    [anon_sym_try] = ACTIONS(2352),
    [sym_integer_literal] = ACTIONS(2350),
    [aux_sym_string_literal_token1] = ACTIONS(2350),
    [sym_char_literal] = ACTIONS(2350),
    [anon_sym_true] = ACTIONS(2352),
    [anon_sym_false] = ACTIONS(2352),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2352),
    [sym_super] = ACTIONS(2352),
    [sym_crate] = ACTIONS(2352),
    [aux_sym_metavariable_token1] = ACTIONS(2350),
    [sym__raw_string_literal_start] = ACTIONS(2350),
    [sym_float_literal] = ACTIONS(2350),
    [sym__outer_block_doc_comment_start] = ACTIONS(2350),
    [sym__outer_line_doc_comment_start] = ACTIONS(2350),
  },
  [STATE(645)] = {
    [sym_line_comment] = STATE(645),
    [sym_block_comment] = STATE(645),
    [ts_builtin_sym_end] = ACTIONS(2354),
    [sym_identifier] = ACTIONS(2356),
    [anon_sym_SEMI] = ACTIONS(2354),
    [anon_sym_macro] = ACTIONS(2356),
    [anon_sym_LPAREN] = ACTIONS(2354),
    [anon_sym_LBRACE] = ACTIONS(2354),
    [anon_sym_RBRACE] = ACTIONS(2354),
    [anon_sym_macro_rules] = ACTIONS(2356),
    [anon_sym_BANG] = ACTIONS(2354),
    [anon_sym_LBRACK] = ACTIONS(2354),
    [anon_sym_STAR] = ACTIONS(2354),
    [anon_sym_u8] = ACTIONS(2356),
    [anon_sym_i8] = ACTIONS(2356),
    [anon_sym_u16] = ACTIONS(2356),
    [anon_sym_i16] = ACTIONS(2356),
    [anon_sym_u32] = ACTIONS(2356),
    [anon_sym_i32] = ACTIONS(2356),
    [anon_sym_u64] = ACTIONS(2356),
    [anon_sym_i64] = ACTIONS(2356),
    [anon_sym_u128] = ACTIONS(2356),
    [anon_sym_i128] = ACTIONS(2356),
    [anon_sym_isize] = ACTIONS(2356),
    [anon_sym_usize] = ACTIONS(2356),
    [anon_sym_f32] = ACTIONS(2356),
    [anon_sym_f64] = ACTIONS(2356),
    [anon_sym_bool] = ACTIONS(2356),
    [anon_sym_str] = ACTIONS(2356),
    [anon_sym_char] = ACTIONS(2356),
    [anon_sym_DASH] = ACTIONS(2354),
    [anon_sym_AMP] = ACTIONS(2354),
    [anon_sym_PIPE] = ACTIONS(2354),
    [anon_sym_LT] = ACTIONS(2354),
    [anon_sym_DOT_DOT] = ACTIONS(2354),
    [anon_sym_COLON_COLON] = ACTIONS(2354),
    [anon_sym_POUND] = ACTIONS(2354),
    [anon_sym_SQUOTE] = ACTIONS(2356),
    [anon_sym_async] = ACTIONS(2356),
    [anon_sym_auto] = ACTIONS(2356),
    [anon_sym_await] = ACTIONS(2356),
    [anon_sym_become] = ACTIONS(2356),
    [anon_sym_break] = ACTIONS(2356),
    [anon_sym_const] = ACTIONS(2356),
    [anon_sym_continue] = ACTIONS(2356),
    [anon_sym_default] = ACTIONS(2356),
    [anon_sym_enum] = ACTIONS(2356),
    [anon_sym_fn] = ACTIONS(2356),
    [anon_sym_for] = ACTIONS(2356),
    [anon_sym_gen] = ACTIONS(2356),
    [anon_sym_if] = ACTIONS(2356),
    [anon_sym_impl] = ACTIONS(2356),
    [anon_sym_let] = ACTIONS(2356),
    [anon_sym_loop] = ACTIONS(2356),
    [anon_sym_match] = ACTIONS(2356),
    [anon_sym_mod] = ACTIONS(2356),
    [anon_sym_pub] = ACTIONS(2356),
    [anon_sym_return] = ACTIONS(2356),
    [anon_sym_raw] = ACTIONS(2356),
    [anon_sym_static] = ACTIONS(2356),
    [anon_sym_struct] = ACTIONS(2356),
    [anon_sym_trait] = ACTIONS(2356),
    [anon_sym_type] = ACTIONS(2356),
    [anon_sym_union] = ACTIONS(2356),
    [anon_sym_unsafe] = ACTIONS(2356),
    [anon_sym_use] = ACTIONS(2356),
    [anon_sym_while] = ACTIONS(2356),
    [anon_sym_extern] = ACTIONS(2356),
    [anon_sym_safe] = ACTIONS(2356),
    [anon_sym_yield] = ACTIONS(2356),
    [anon_sym_move] = ACTIONS(2356),
    [anon_sym_try] = ACTIONS(2356),
    [sym_integer_literal] = ACTIONS(2354),
    [aux_sym_string_literal_token1] = ACTIONS(2354),
    [sym_char_literal] = ACTIONS(2354),
    [anon_sym_true] = ACTIONS(2356),
    [anon_sym_false] = ACTIONS(2356),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2356),
    [sym_super] = ACTIONS(2356),
    [sym_crate] = ACTIONS(2356),
    [aux_sym_metavariable_token1] = ACTIONS(2354),
    [sym__raw_string_literal_start] = ACTIONS(2354),
    [sym_float_literal] = ACTIONS(2354),
    [sym__outer_block_doc_comment_start] = ACTIONS(2354),
    [sym__outer_line_doc_comment_start] = ACTIONS(2354),
  },
  [STATE(646)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3896),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(646),
    [sym_block_comment] = STATE(646),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(2358),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(2358),
    [anon_sym_COMMA] = ACTIONS(2358),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(2358),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_EQ] = ACTIONS(2358),
    [anon_sym_GT] = ACTIONS(2358),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(2360),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(647)] = {
    [sym_line_comment] = STATE(647),
    [sym_block_comment] = STATE(647),
    [ts_builtin_sym_end] = ACTIONS(2362),
    [sym_identifier] = ACTIONS(2364),
    [anon_sym_SEMI] = ACTIONS(2362),
    [anon_sym_macro] = ACTIONS(2364),
    [anon_sym_LPAREN] = ACTIONS(2362),
    [anon_sym_LBRACE] = ACTIONS(2362),
    [anon_sym_RBRACE] = ACTIONS(2362),
    [anon_sym_macro_rules] = ACTIONS(2364),
    [anon_sym_BANG] = ACTIONS(2362),
    [anon_sym_LBRACK] = ACTIONS(2362),
    [anon_sym_STAR] = ACTIONS(2362),
    [anon_sym_u8] = ACTIONS(2364),
    [anon_sym_i8] = ACTIONS(2364),
    [anon_sym_u16] = ACTIONS(2364),
    [anon_sym_i16] = ACTIONS(2364),
    [anon_sym_u32] = ACTIONS(2364),
    [anon_sym_i32] = ACTIONS(2364),
    [anon_sym_u64] = ACTIONS(2364),
    [anon_sym_i64] = ACTIONS(2364),
    [anon_sym_u128] = ACTIONS(2364),
    [anon_sym_i128] = ACTIONS(2364),
    [anon_sym_isize] = ACTIONS(2364),
    [anon_sym_usize] = ACTIONS(2364),
    [anon_sym_f32] = ACTIONS(2364),
    [anon_sym_f64] = ACTIONS(2364),
    [anon_sym_bool] = ACTIONS(2364),
    [anon_sym_str] = ACTIONS(2364),
    [anon_sym_char] = ACTIONS(2364),
    [anon_sym_DASH] = ACTIONS(2362),
    [anon_sym_AMP] = ACTIONS(2362),
    [anon_sym_PIPE] = ACTIONS(2362),
    [anon_sym_LT] = ACTIONS(2362),
    [anon_sym_DOT_DOT] = ACTIONS(2362),
    [anon_sym_COLON_COLON] = ACTIONS(2362),
    [anon_sym_POUND] = ACTIONS(2362),
    [anon_sym_SQUOTE] = ACTIONS(2364),
    [anon_sym_async] = ACTIONS(2364),
    [anon_sym_auto] = ACTIONS(2364),
    [anon_sym_await] = ACTIONS(2364),
    [anon_sym_become] = ACTIONS(2364),
    [anon_sym_break] = ACTIONS(2364),
    [anon_sym_const] = ACTIONS(2364),
    [anon_sym_continue] = ACTIONS(2364),
    [anon_sym_default] = ACTIONS(2364),
    [anon_sym_enum] = ACTIONS(2364),
    [anon_sym_fn] = ACTIONS(2364),
    [anon_sym_for] = ACTIONS(2364),
    [anon_sym_gen] = ACTIONS(2364),
    [anon_sym_if] = ACTIONS(2364),
    [anon_sym_impl] = ACTIONS(2364),
    [anon_sym_let] = ACTIONS(2364),
    [anon_sym_loop] = ACTIONS(2364),
    [anon_sym_match] = ACTIONS(2364),
    [anon_sym_mod] = ACTIONS(2364),
    [anon_sym_pub] = ACTIONS(2364),
    [anon_sym_return] = ACTIONS(2364),
    [anon_sym_raw] = ACTIONS(2364),
    [anon_sym_static] = ACTIONS(2364),
    [anon_sym_struct] = ACTIONS(2364),
    [anon_sym_trait] = ACTIONS(2364),
    [anon_sym_type] = ACTIONS(2364),
    [anon_sym_union] = ACTIONS(2364),
    [anon_sym_unsafe] = ACTIONS(2364),
    [anon_sym_use] = ACTIONS(2364),
    [anon_sym_while] = ACTIONS(2364),
    [anon_sym_extern] = ACTIONS(2364),
    [anon_sym_safe] = ACTIONS(2364),
    [anon_sym_yield] = ACTIONS(2364),
    [anon_sym_move] = ACTIONS(2364),
    [anon_sym_try] = ACTIONS(2364),
    [sym_integer_literal] = ACTIONS(2362),
    [aux_sym_string_literal_token1] = ACTIONS(2362),
    [sym_char_literal] = ACTIONS(2362),
    [anon_sym_true] = ACTIONS(2364),
    [anon_sym_false] = ACTIONS(2364),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2364),
    [sym_super] = ACTIONS(2364),
    [sym_crate] = ACTIONS(2364),
    [aux_sym_metavariable_token1] = ACTIONS(2362),
    [sym__raw_string_literal_start] = ACTIONS(2362),
    [sym_float_literal] = ACTIONS(2362),
    [sym__outer_block_doc_comment_start] = ACTIONS(2362),
    [sym__outer_line_doc_comment_start] = ACTIONS(2362),
  },
  [STATE(648)] = {
    [sym_line_comment] = STATE(648),
    [sym_block_comment] = STATE(648),
    [ts_builtin_sym_end] = ACTIONS(2366),
    [sym_identifier] = ACTIONS(2368),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_macro] = ACTIONS(2368),
    [anon_sym_LPAREN] = ACTIONS(2366),
    [anon_sym_LBRACE] = ACTIONS(2366),
    [anon_sym_RBRACE] = ACTIONS(2366),
    [anon_sym_macro_rules] = ACTIONS(2368),
    [anon_sym_BANG] = ACTIONS(2366),
    [anon_sym_LBRACK] = ACTIONS(2366),
    [anon_sym_STAR] = ACTIONS(2366),
    [anon_sym_u8] = ACTIONS(2368),
    [anon_sym_i8] = ACTIONS(2368),
    [anon_sym_u16] = ACTIONS(2368),
    [anon_sym_i16] = ACTIONS(2368),
    [anon_sym_u32] = ACTIONS(2368),
    [anon_sym_i32] = ACTIONS(2368),
    [anon_sym_u64] = ACTIONS(2368),
    [anon_sym_i64] = ACTIONS(2368),
    [anon_sym_u128] = ACTIONS(2368),
    [anon_sym_i128] = ACTIONS(2368),
    [anon_sym_isize] = ACTIONS(2368),
    [anon_sym_usize] = ACTIONS(2368),
    [anon_sym_f32] = ACTIONS(2368),
    [anon_sym_f64] = ACTIONS(2368),
    [anon_sym_bool] = ACTIONS(2368),
    [anon_sym_str] = ACTIONS(2368),
    [anon_sym_char] = ACTIONS(2368),
    [anon_sym_DASH] = ACTIONS(2366),
    [anon_sym_AMP] = ACTIONS(2366),
    [anon_sym_PIPE] = ACTIONS(2366),
    [anon_sym_LT] = ACTIONS(2366),
    [anon_sym_DOT_DOT] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(2366),
    [anon_sym_POUND] = ACTIONS(2366),
    [anon_sym_SQUOTE] = ACTIONS(2368),
    [anon_sym_async] = ACTIONS(2368),
    [anon_sym_auto] = ACTIONS(2368),
    [anon_sym_await] = ACTIONS(2368),
    [anon_sym_become] = ACTIONS(2368),
    [anon_sym_break] = ACTIONS(2368),
    [anon_sym_const] = ACTIONS(2368),
    [anon_sym_continue] = ACTIONS(2368),
    [anon_sym_default] = ACTIONS(2368),
    [anon_sym_enum] = ACTIONS(2368),
    [anon_sym_fn] = ACTIONS(2368),
    [anon_sym_for] = ACTIONS(2368),
    [anon_sym_gen] = ACTIONS(2368),
    [anon_sym_if] = ACTIONS(2368),
    [anon_sym_impl] = ACTIONS(2368),
    [anon_sym_let] = ACTIONS(2368),
    [anon_sym_loop] = ACTIONS(2368),
    [anon_sym_match] = ACTIONS(2368),
    [anon_sym_mod] = ACTIONS(2368),
    [anon_sym_pub] = ACTIONS(2368),
    [anon_sym_return] = ACTIONS(2368),
    [anon_sym_raw] = ACTIONS(2368),
    [anon_sym_static] = ACTIONS(2368),
    [anon_sym_struct] = ACTIONS(2368),
    [anon_sym_trait] = ACTIONS(2368),
    [anon_sym_type] = ACTIONS(2368),
    [anon_sym_union] = ACTIONS(2368),
    [anon_sym_unsafe] = ACTIONS(2368),
    [anon_sym_use] = ACTIONS(2368),
    [anon_sym_while] = ACTIONS(2368),
    [anon_sym_extern] = ACTIONS(2368),
    [anon_sym_safe] = ACTIONS(2368),
    [anon_sym_yield] = ACTIONS(2368),
    [anon_sym_move] = ACTIONS(2368),
    [anon_sym_try] = ACTIONS(2368),
    [sym_integer_literal] = ACTIONS(2366),
    [aux_sym_string_literal_token1] = ACTIONS(2366),
    [sym_char_literal] = ACTIONS(2366),
    [anon_sym_true] = ACTIONS(2368),
    [anon_sym_false] = ACTIONS(2368),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2368),
    [sym_super] = ACTIONS(2368),
    [sym_crate] = ACTIONS(2368),
    [aux_sym_metavariable_token1] = ACTIONS(2366),
    [sym__raw_string_literal_start] = ACTIONS(2366),
    [sym_float_literal] = ACTIONS(2366),
    [sym__outer_block_doc_comment_start] = ACTIONS(2366),
    [sym__outer_line_doc_comment_start] = ACTIONS(2366),
  },
  [STATE(649)] = {
    [sym_line_comment] = STATE(649),
    [sym_block_comment] = STATE(649),
    [ts_builtin_sym_end] = ACTIONS(2370),
    [sym_identifier] = ACTIONS(2372),
    [anon_sym_SEMI] = ACTIONS(2370),
    [anon_sym_macro] = ACTIONS(2372),
    [anon_sym_LPAREN] = ACTIONS(2370),
    [anon_sym_LBRACE] = ACTIONS(2370),
    [anon_sym_RBRACE] = ACTIONS(2370),
    [anon_sym_macro_rules] = ACTIONS(2372),
    [anon_sym_BANG] = ACTIONS(2370),
    [anon_sym_LBRACK] = ACTIONS(2370),
    [anon_sym_STAR] = ACTIONS(2370),
    [anon_sym_u8] = ACTIONS(2372),
    [anon_sym_i8] = ACTIONS(2372),
    [anon_sym_u16] = ACTIONS(2372),
    [anon_sym_i16] = ACTIONS(2372),
    [anon_sym_u32] = ACTIONS(2372),
    [anon_sym_i32] = ACTIONS(2372),
    [anon_sym_u64] = ACTIONS(2372),
    [anon_sym_i64] = ACTIONS(2372),
    [anon_sym_u128] = ACTIONS(2372),
    [anon_sym_i128] = ACTIONS(2372),
    [anon_sym_isize] = ACTIONS(2372),
    [anon_sym_usize] = ACTIONS(2372),
    [anon_sym_f32] = ACTIONS(2372),
    [anon_sym_f64] = ACTIONS(2372),
    [anon_sym_bool] = ACTIONS(2372),
    [anon_sym_str] = ACTIONS(2372),
    [anon_sym_char] = ACTIONS(2372),
    [anon_sym_DASH] = ACTIONS(2370),
    [anon_sym_AMP] = ACTIONS(2370),
    [anon_sym_PIPE] = ACTIONS(2370),
    [anon_sym_LT] = ACTIONS(2370),
    [anon_sym_DOT_DOT] = ACTIONS(2370),
    [anon_sym_COLON_COLON] = ACTIONS(2370),
    [anon_sym_POUND] = ACTIONS(2370),
    [anon_sym_SQUOTE] = ACTIONS(2372),
    [anon_sym_async] = ACTIONS(2372),
    [anon_sym_auto] = ACTIONS(2372),
    [anon_sym_await] = ACTIONS(2372),
    [anon_sym_become] = ACTIONS(2372),
    [anon_sym_break] = ACTIONS(2372),
    [anon_sym_const] = ACTIONS(2372),
    [anon_sym_continue] = ACTIONS(2372),
    [anon_sym_default] = ACTIONS(2372),
    [anon_sym_enum] = ACTIONS(2372),
    [anon_sym_fn] = ACTIONS(2372),
    [anon_sym_for] = ACTIONS(2372),
    [anon_sym_gen] = ACTIONS(2372),
    [anon_sym_if] = ACTIONS(2372),
    [anon_sym_impl] = ACTIONS(2372),
    [anon_sym_let] = ACTIONS(2372),
    [anon_sym_loop] = ACTIONS(2372),
    [anon_sym_match] = ACTIONS(2372),
    [anon_sym_mod] = ACTIONS(2372),
    [anon_sym_pub] = ACTIONS(2372),
    [anon_sym_return] = ACTIONS(2372),
    [anon_sym_raw] = ACTIONS(2372),
    [anon_sym_static] = ACTIONS(2372),
    [anon_sym_struct] = ACTIONS(2372),
    [anon_sym_trait] = ACTIONS(2372),
    [anon_sym_type] = ACTIONS(2372),
    [anon_sym_union] = ACTIONS(2372),
    [anon_sym_unsafe] = ACTIONS(2372),
    [anon_sym_use] = ACTIONS(2372),
    [anon_sym_while] = ACTIONS(2372),
    [anon_sym_extern] = ACTIONS(2372),
    [anon_sym_safe] = ACTIONS(2372),
    [anon_sym_yield] = ACTIONS(2372),
    [anon_sym_move] = ACTIONS(2372),
    [anon_sym_try] = ACTIONS(2372),
    [sym_integer_literal] = ACTIONS(2370),
    [aux_sym_string_literal_token1] = ACTIONS(2370),
    [sym_char_literal] = ACTIONS(2370),
    [anon_sym_true] = ACTIONS(2372),
    [anon_sym_false] = ACTIONS(2372),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2372),
    [sym_super] = ACTIONS(2372),
    [sym_crate] = ACTIONS(2372),
    [aux_sym_metavariable_token1] = ACTIONS(2370),
    [sym__raw_string_literal_start] = ACTIONS(2370),
    [sym_float_literal] = ACTIONS(2370),
    [sym__outer_block_doc_comment_start] = ACTIONS(2370),
    [sym__outer_line_doc_comment_start] = ACTIONS(2370),
  },
  [STATE(650)] = {
    [sym_line_comment] = STATE(650),
    [sym_block_comment] = STATE(650),
    [ts_builtin_sym_end] = ACTIONS(2374),
    [sym_identifier] = ACTIONS(2376),
    [anon_sym_SEMI] = ACTIONS(2374),
    [anon_sym_macro] = ACTIONS(2376),
    [anon_sym_LPAREN] = ACTIONS(2374),
    [anon_sym_LBRACE] = ACTIONS(2374),
    [anon_sym_RBRACE] = ACTIONS(2374),
    [anon_sym_macro_rules] = ACTIONS(2376),
    [anon_sym_BANG] = ACTIONS(2374),
    [anon_sym_LBRACK] = ACTIONS(2374),
    [anon_sym_STAR] = ACTIONS(2374),
    [anon_sym_u8] = ACTIONS(2376),
    [anon_sym_i8] = ACTIONS(2376),
    [anon_sym_u16] = ACTIONS(2376),
    [anon_sym_i16] = ACTIONS(2376),
    [anon_sym_u32] = ACTIONS(2376),
    [anon_sym_i32] = ACTIONS(2376),
    [anon_sym_u64] = ACTIONS(2376),
    [anon_sym_i64] = ACTIONS(2376),
    [anon_sym_u128] = ACTIONS(2376),
    [anon_sym_i128] = ACTIONS(2376),
    [anon_sym_isize] = ACTIONS(2376),
    [anon_sym_usize] = ACTIONS(2376),
    [anon_sym_f32] = ACTIONS(2376),
    [anon_sym_f64] = ACTIONS(2376),
    [anon_sym_bool] = ACTIONS(2376),
    [anon_sym_str] = ACTIONS(2376),
    [anon_sym_char] = ACTIONS(2376),
    [anon_sym_DASH] = ACTIONS(2374),
    [anon_sym_AMP] = ACTIONS(2374),
    [anon_sym_PIPE] = ACTIONS(2374),
    [anon_sym_LT] = ACTIONS(2374),
    [anon_sym_DOT_DOT] = ACTIONS(2374),
    [anon_sym_COLON_COLON] = ACTIONS(2374),
    [anon_sym_POUND] = ACTIONS(2374),
    [anon_sym_SQUOTE] = ACTIONS(2376),
    [anon_sym_async] = ACTIONS(2376),
    [anon_sym_auto] = ACTIONS(2376),
    [anon_sym_await] = ACTIONS(2376),
    [anon_sym_become] = ACTIONS(2376),
    [anon_sym_break] = ACTIONS(2376),
    [anon_sym_const] = ACTIONS(2376),
    [anon_sym_continue] = ACTIONS(2376),
    [anon_sym_default] = ACTIONS(2376),
    [anon_sym_enum] = ACTIONS(2376),
    [anon_sym_fn] = ACTIONS(2376),
    [anon_sym_for] = ACTIONS(2376),
    [anon_sym_gen] = ACTIONS(2376),
    [anon_sym_if] = ACTIONS(2376),
    [anon_sym_impl] = ACTIONS(2376),
    [anon_sym_let] = ACTIONS(2376),
    [anon_sym_loop] = ACTIONS(2376),
    [anon_sym_match] = ACTIONS(2376),
    [anon_sym_mod] = ACTIONS(2376),
    [anon_sym_pub] = ACTIONS(2376),
    [anon_sym_return] = ACTIONS(2376),
    [anon_sym_raw] = ACTIONS(2376),
    [anon_sym_static] = ACTIONS(2376),
    [anon_sym_struct] = ACTIONS(2376),
    [anon_sym_trait] = ACTIONS(2376),
    [anon_sym_type] = ACTIONS(2376),
    [anon_sym_union] = ACTIONS(2376),
    [anon_sym_unsafe] = ACTIONS(2376),
    [anon_sym_use] = ACTIONS(2376),
    [anon_sym_while] = ACTIONS(2376),
    [anon_sym_extern] = ACTIONS(2376),
    [anon_sym_safe] = ACTIONS(2376),
    [anon_sym_yield] = ACTIONS(2376),
    [anon_sym_move] = ACTIONS(2376),
    [anon_sym_try] = ACTIONS(2376),
    [sym_integer_literal] = ACTIONS(2374),
    [aux_sym_string_literal_token1] = ACTIONS(2374),
    [sym_char_literal] = ACTIONS(2374),
    [anon_sym_true] = ACTIONS(2376),
    [anon_sym_false] = ACTIONS(2376),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2376),
    [sym_super] = ACTIONS(2376),
    [sym_crate] = ACTIONS(2376),
    [aux_sym_metavariable_token1] = ACTIONS(2374),
    [sym__raw_string_literal_start] = ACTIONS(2374),
    [sym_float_literal] = ACTIONS(2374),
    [sym__outer_block_doc_comment_start] = ACTIONS(2374),
    [sym__outer_line_doc_comment_start] = ACTIONS(2374),
  },
  [STATE(651)] = {
    [sym_line_comment] = STATE(651),
    [sym_block_comment] = STATE(651),
    [ts_builtin_sym_end] = ACTIONS(2378),
    [sym_identifier] = ACTIONS(2380),
    [anon_sym_SEMI] = ACTIONS(2378),
    [anon_sym_macro] = ACTIONS(2380),
    [anon_sym_LPAREN] = ACTIONS(2378),
    [anon_sym_LBRACE] = ACTIONS(2378),
    [anon_sym_RBRACE] = ACTIONS(2378),
    [anon_sym_macro_rules] = ACTIONS(2380),
    [anon_sym_BANG] = ACTIONS(2378),
    [anon_sym_LBRACK] = ACTIONS(2378),
    [anon_sym_STAR] = ACTIONS(2378),
    [anon_sym_u8] = ACTIONS(2380),
    [anon_sym_i8] = ACTIONS(2380),
    [anon_sym_u16] = ACTIONS(2380),
    [anon_sym_i16] = ACTIONS(2380),
    [anon_sym_u32] = ACTIONS(2380),
    [anon_sym_i32] = ACTIONS(2380),
    [anon_sym_u64] = ACTIONS(2380),
    [anon_sym_i64] = ACTIONS(2380),
    [anon_sym_u128] = ACTIONS(2380),
    [anon_sym_i128] = ACTIONS(2380),
    [anon_sym_isize] = ACTIONS(2380),
    [anon_sym_usize] = ACTIONS(2380),
    [anon_sym_f32] = ACTIONS(2380),
    [anon_sym_f64] = ACTIONS(2380),
    [anon_sym_bool] = ACTIONS(2380),
    [anon_sym_str] = ACTIONS(2380),
    [anon_sym_char] = ACTIONS(2380),
    [anon_sym_DASH] = ACTIONS(2378),
    [anon_sym_AMP] = ACTIONS(2378),
    [anon_sym_PIPE] = ACTIONS(2378),
    [anon_sym_LT] = ACTIONS(2378),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_COLON_COLON] = ACTIONS(2378),
    [anon_sym_POUND] = ACTIONS(2378),
    [anon_sym_SQUOTE] = ACTIONS(2380),
    [anon_sym_async] = ACTIONS(2380),
    [anon_sym_auto] = ACTIONS(2380),
    [anon_sym_await] = ACTIONS(2380),
    [anon_sym_become] = ACTIONS(2380),
    [anon_sym_break] = ACTIONS(2380),
    [anon_sym_const] = ACTIONS(2380),
    [anon_sym_continue] = ACTIONS(2380),
    [anon_sym_default] = ACTIONS(2380),
    [anon_sym_enum] = ACTIONS(2380),
    [anon_sym_fn] = ACTIONS(2380),
    [anon_sym_for] = ACTIONS(2380),
    [anon_sym_gen] = ACTIONS(2380),
    [anon_sym_if] = ACTIONS(2380),
    [anon_sym_impl] = ACTIONS(2380),
    [anon_sym_let] = ACTIONS(2380),
    [anon_sym_loop] = ACTIONS(2380),
    [anon_sym_match] = ACTIONS(2380),
    [anon_sym_mod] = ACTIONS(2380),
    [anon_sym_pub] = ACTIONS(2380),
    [anon_sym_return] = ACTIONS(2380),
    [anon_sym_raw] = ACTIONS(2380),
    [anon_sym_static] = ACTIONS(2380),
    [anon_sym_struct] = ACTIONS(2380),
    [anon_sym_trait] = ACTIONS(2380),
    [anon_sym_type] = ACTIONS(2380),
    [anon_sym_union] = ACTIONS(2380),
    [anon_sym_unsafe] = ACTIONS(2380),
    [anon_sym_use] = ACTIONS(2380),
    [anon_sym_while] = ACTIONS(2380),
    [anon_sym_extern] = ACTIONS(2380),
    [anon_sym_safe] = ACTIONS(2380),
    [anon_sym_yield] = ACTIONS(2380),
    [anon_sym_move] = ACTIONS(2380),
    [anon_sym_try] = ACTIONS(2380),
    [sym_integer_literal] = ACTIONS(2378),
    [aux_sym_string_literal_token1] = ACTIONS(2378),
    [sym_char_literal] = ACTIONS(2378),
    [anon_sym_true] = ACTIONS(2380),
    [anon_sym_false] = ACTIONS(2380),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2380),
    [sym_super] = ACTIONS(2380),
    [sym_crate] = ACTIONS(2380),
    [aux_sym_metavariable_token1] = ACTIONS(2378),
    [sym__raw_string_literal_start] = ACTIONS(2378),
    [sym_float_literal] = ACTIONS(2378),
    [sym__outer_block_doc_comment_start] = ACTIONS(2378),
    [sym__outer_line_doc_comment_start] = ACTIONS(2378),
  },
  [STATE(652)] = {
    [sym_line_comment] = STATE(652),
    [sym_block_comment] = STATE(652),
    [ts_builtin_sym_end] = ACTIONS(2382),
    [sym_identifier] = ACTIONS(2384),
    [anon_sym_SEMI] = ACTIONS(2382),
    [anon_sym_macro] = ACTIONS(2384),
    [anon_sym_LPAREN] = ACTIONS(2382),
    [anon_sym_LBRACE] = ACTIONS(2382),
    [anon_sym_RBRACE] = ACTIONS(2382),
    [anon_sym_macro_rules] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2382),
    [anon_sym_LBRACK] = ACTIONS(2382),
    [anon_sym_STAR] = ACTIONS(2382),
    [anon_sym_u8] = ACTIONS(2384),
    [anon_sym_i8] = ACTIONS(2384),
    [anon_sym_u16] = ACTIONS(2384),
    [anon_sym_i16] = ACTIONS(2384),
    [anon_sym_u32] = ACTIONS(2384),
    [anon_sym_i32] = ACTIONS(2384),
    [anon_sym_u64] = ACTIONS(2384),
    [anon_sym_i64] = ACTIONS(2384),
    [anon_sym_u128] = ACTIONS(2384),
    [anon_sym_i128] = ACTIONS(2384),
    [anon_sym_isize] = ACTIONS(2384),
    [anon_sym_usize] = ACTIONS(2384),
    [anon_sym_f32] = ACTIONS(2384),
    [anon_sym_f64] = ACTIONS(2384),
    [anon_sym_bool] = ACTIONS(2384),
    [anon_sym_str] = ACTIONS(2384),
    [anon_sym_char] = ACTIONS(2384),
    [anon_sym_DASH] = ACTIONS(2382),
    [anon_sym_AMP] = ACTIONS(2382),
    [anon_sym_PIPE] = ACTIONS(2382),
    [anon_sym_LT] = ACTIONS(2382),
    [anon_sym_DOT_DOT] = ACTIONS(2382),
    [anon_sym_COLON_COLON] = ACTIONS(2382),
    [anon_sym_POUND] = ACTIONS(2382),
    [anon_sym_SQUOTE] = ACTIONS(2384),
    [anon_sym_async] = ACTIONS(2384),
    [anon_sym_auto] = ACTIONS(2384),
    [anon_sym_await] = ACTIONS(2384),
    [anon_sym_become] = ACTIONS(2384),
    [anon_sym_break] = ACTIONS(2384),
    [anon_sym_const] = ACTIONS(2384),
    [anon_sym_continue] = ACTIONS(2384),
    [anon_sym_default] = ACTIONS(2384),
    [anon_sym_enum] = ACTIONS(2384),
    [anon_sym_fn] = ACTIONS(2384),
    [anon_sym_for] = ACTIONS(2384),
    [anon_sym_gen] = ACTIONS(2384),
    [anon_sym_if] = ACTIONS(2384),
    [anon_sym_impl] = ACTIONS(2384),
    [anon_sym_let] = ACTIONS(2384),
    [anon_sym_loop] = ACTIONS(2384),
    [anon_sym_match] = ACTIONS(2384),
    [anon_sym_mod] = ACTIONS(2384),
    [anon_sym_pub] = ACTIONS(2384),
    [anon_sym_return] = ACTIONS(2384),
    [anon_sym_raw] = ACTIONS(2384),
    [anon_sym_static] = ACTIONS(2384),
    [anon_sym_struct] = ACTIONS(2384),
    [anon_sym_trait] = ACTIONS(2384),
    [anon_sym_type] = ACTIONS(2384),
    [anon_sym_union] = ACTIONS(2384),
    [anon_sym_unsafe] = ACTIONS(2384),
    [anon_sym_use] = ACTIONS(2384),
    [anon_sym_while] = ACTIONS(2384),
    [anon_sym_extern] = ACTIONS(2384),
    [anon_sym_safe] = ACTIONS(2384),
    [anon_sym_yield] = ACTIONS(2384),
    [anon_sym_move] = ACTIONS(2384),
    [anon_sym_try] = ACTIONS(2384),
    [sym_integer_literal] = ACTIONS(2382),
    [aux_sym_string_literal_token1] = ACTIONS(2382),
    [sym_char_literal] = ACTIONS(2382),
    [anon_sym_true] = ACTIONS(2384),
    [anon_sym_false] = ACTIONS(2384),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2384),
    [sym_super] = ACTIONS(2384),
    [sym_crate] = ACTIONS(2384),
    [aux_sym_metavariable_token1] = ACTIONS(2382),
    [sym__raw_string_literal_start] = ACTIONS(2382),
    [sym_float_literal] = ACTIONS(2382),
    [sym__outer_block_doc_comment_start] = ACTIONS(2382),
    [sym__outer_line_doc_comment_start] = ACTIONS(2382),
  },
  [STATE(653)] = {
    [sym_line_comment] = STATE(653),
    [sym_block_comment] = STATE(653),
    [ts_builtin_sym_end] = ACTIONS(2386),
    [sym_identifier] = ACTIONS(2388),
    [anon_sym_SEMI] = ACTIONS(2386),
    [anon_sym_macro] = ACTIONS(2388),
    [anon_sym_LPAREN] = ACTIONS(2386),
    [anon_sym_LBRACE] = ACTIONS(2386),
    [anon_sym_RBRACE] = ACTIONS(2386),
    [anon_sym_macro_rules] = ACTIONS(2388),
    [anon_sym_BANG] = ACTIONS(2386),
    [anon_sym_LBRACK] = ACTIONS(2386),
    [anon_sym_STAR] = ACTIONS(2386),
    [anon_sym_u8] = ACTIONS(2388),
    [anon_sym_i8] = ACTIONS(2388),
    [anon_sym_u16] = ACTIONS(2388),
    [anon_sym_i16] = ACTIONS(2388),
    [anon_sym_u32] = ACTIONS(2388),
    [anon_sym_i32] = ACTIONS(2388),
    [anon_sym_u64] = ACTIONS(2388),
    [anon_sym_i64] = ACTIONS(2388),
    [anon_sym_u128] = ACTIONS(2388),
    [anon_sym_i128] = ACTIONS(2388),
    [anon_sym_isize] = ACTIONS(2388),
    [anon_sym_usize] = ACTIONS(2388),
    [anon_sym_f32] = ACTIONS(2388),
    [anon_sym_f64] = ACTIONS(2388),
    [anon_sym_bool] = ACTIONS(2388),
    [anon_sym_str] = ACTIONS(2388),
    [anon_sym_char] = ACTIONS(2388),
    [anon_sym_DASH] = ACTIONS(2386),
    [anon_sym_AMP] = ACTIONS(2386),
    [anon_sym_PIPE] = ACTIONS(2386),
    [anon_sym_LT] = ACTIONS(2386),
    [anon_sym_DOT_DOT] = ACTIONS(2386),
    [anon_sym_COLON_COLON] = ACTIONS(2386),
    [anon_sym_POUND] = ACTIONS(2386),
    [anon_sym_SQUOTE] = ACTIONS(2388),
    [anon_sym_async] = ACTIONS(2388),
    [anon_sym_auto] = ACTIONS(2388),
    [anon_sym_await] = ACTIONS(2388),
    [anon_sym_become] = ACTIONS(2388),
    [anon_sym_break] = ACTIONS(2388),
    [anon_sym_const] = ACTIONS(2388),
    [anon_sym_continue] = ACTIONS(2388),
    [anon_sym_default] = ACTIONS(2388),
    [anon_sym_enum] = ACTIONS(2388),
    [anon_sym_fn] = ACTIONS(2388),
    [anon_sym_for] = ACTIONS(2388),
    [anon_sym_gen] = ACTIONS(2388),
    [anon_sym_if] = ACTIONS(2388),
    [anon_sym_impl] = ACTIONS(2388),
    [anon_sym_let] = ACTIONS(2388),
    [anon_sym_loop] = ACTIONS(2388),
    [anon_sym_match] = ACTIONS(2388),
    [anon_sym_mod] = ACTIONS(2388),
    [anon_sym_pub] = ACTIONS(2388),
    [anon_sym_return] = ACTIONS(2388),
    [anon_sym_raw] = ACTIONS(2388),
    [anon_sym_static] = ACTIONS(2388),
    [anon_sym_struct] = ACTIONS(2388),
    [anon_sym_trait] = ACTIONS(2388),
    [anon_sym_type] = ACTIONS(2388),
    [anon_sym_union] = ACTIONS(2388),
    [anon_sym_unsafe] = ACTIONS(2388),
    [anon_sym_use] = ACTIONS(2388),
    [anon_sym_while] = ACTIONS(2388),
    [anon_sym_extern] = ACTIONS(2388),
    [anon_sym_safe] = ACTIONS(2388),
    [anon_sym_yield] = ACTIONS(2388),
    [anon_sym_move] = ACTIONS(2388),
    [anon_sym_try] = ACTIONS(2388),
    [sym_integer_literal] = ACTIONS(2386),
    [aux_sym_string_literal_token1] = ACTIONS(2386),
    [sym_char_literal] = ACTIONS(2386),
    [anon_sym_true] = ACTIONS(2388),
    [anon_sym_false] = ACTIONS(2388),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2388),
    [sym_super] = ACTIONS(2388),
    [sym_crate] = ACTIONS(2388),
    [aux_sym_metavariable_token1] = ACTIONS(2386),
    [sym__raw_string_literal_start] = ACTIONS(2386),
    [sym_float_literal] = ACTIONS(2386),
    [sym__outer_block_doc_comment_start] = ACTIONS(2386),
    [sym__outer_line_doc_comment_start] = ACTIONS(2386),
  },
  [STATE(654)] = {
    [sym_line_comment] = STATE(654),
    [sym_block_comment] = STATE(654),
    [ts_builtin_sym_end] = ACTIONS(2390),
    [sym_identifier] = ACTIONS(2392),
    [anon_sym_SEMI] = ACTIONS(2390),
    [anon_sym_macro] = ACTIONS(2392),
    [anon_sym_LPAREN] = ACTIONS(2390),
    [anon_sym_LBRACE] = ACTIONS(2390),
    [anon_sym_RBRACE] = ACTIONS(2390),
    [anon_sym_macro_rules] = ACTIONS(2392),
    [anon_sym_BANG] = ACTIONS(2390),
    [anon_sym_LBRACK] = ACTIONS(2390),
    [anon_sym_STAR] = ACTIONS(2390),
    [anon_sym_u8] = ACTIONS(2392),
    [anon_sym_i8] = ACTIONS(2392),
    [anon_sym_u16] = ACTIONS(2392),
    [anon_sym_i16] = ACTIONS(2392),
    [anon_sym_u32] = ACTIONS(2392),
    [anon_sym_i32] = ACTIONS(2392),
    [anon_sym_u64] = ACTIONS(2392),
    [anon_sym_i64] = ACTIONS(2392),
    [anon_sym_u128] = ACTIONS(2392),
    [anon_sym_i128] = ACTIONS(2392),
    [anon_sym_isize] = ACTIONS(2392),
    [anon_sym_usize] = ACTIONS(2392),
    [anon_sym_f32] = ACTIONS(2392),
    [anon_sym_f64] = ACTIONS(2392),
    [anon_sym_bool] = ACTIONS(2392),
    [anon_sym_str] = ACTIONS(2392),
    [anon_sym_char] = ACTIONS(2392),
    [anon_sym_DASH] = ACTIONS(2390),
    [anon_sym_AMP] = ACTIONS(2390),
    [anon_sym_PIPE] = ACTIONS(2390),
    [anon_sym_LT] = ACTIONS(2390),
    [anon_sym_DOT_DOT] = ACTIONS(2390),
    [anon_sym_COLON_COLON] = ACTIONS(2390),
    [anon_sym_POUND] = ACTIONS(2390),
    [anon_sym_SQUOTE] = ACTIONS(2392),
    [anon_sym_async] = ACTIONS(2392),
    [anon_sym_auto] = ACTIONS(2392),
    [anon_sym_await] = ACTIONS(2392),
    [anon_sym_become] = ACTIONS(2392),
    [anon_sym_break] = ACTIONS(2392),
    [anon_sym_const] = ACTIONS(2392),
    [anon_sym_continue] = ACTIONS(2392),
    [anon_sym_default] = ACTIONS(2392),
    [anon_sym_enum] = ACTIONS(2392),
    [anon_sym_fn] = ACTIONS(2392),
    [anon_sym_for] = ACTIONS(2392),
    [anon_sym_gen] = ACTIONS(2392),
    [anon_sym_if] = ACTIONS(2392),
    [anon_sym_impl] = ACTIONS(2392),
    [anon_sym_let] = ACTIONS(2392),
    [anon_sym_loop] = ACTIONS(2392),
    [anon_sym_match] = ACTIONS(2392),
    [anon_sym_mod] = ACTIONS(2392),
    [anon_sym_pub] = ACTIONS(2392),
    [anon_sym_return] = ACTIONS(2392),
    [anon_sym_raw] = ACTIONS(2392),
    [anon_sym_static] = ACTIONS(2392),
    [anon_sym_struct] = ACTIONS(2392),
    [anon_sym_trait] = ACTIONS(2392),
    [anon_sym_type] = ACTIONS(2392),
    [anon_sym_union] = ACTIONS(2392),
    [anon_sym_unsafe] = ACTIONS(2392),
    [anon_sym_use] = ACTIONS(2392),
    [anon_sym_while] = ACTIONS(2392),
    [anon_sym_extern] = ACTIONS(2392),
    [anon_sym_safe] = ACTIONS(2392),
    [anon_sym_yield] = ACTIONS(2392),
    [anon_sym_move] = ACTIONS(2392),
    [anon_sym_try] = ACTIONS(2392),
    [sym_integer_literal] = ACTIONS(2390),
    [aux_sym_string_literal_token1] = ACTIONS(2390),
    [sym_char_literal] = ACTIONS(2390),
    [anon_sym_true] = ACTIONS(2392),
    [anon_sym_false] = ACTIONS(2392),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2392),
    [sym_super] = ACTIONS(2392),
    [sym_crate] = ACTIONS(2392),
    [aux_sym_metavariable_token1] = ACTIONS(2390),
    [sym__raw_string_literal_start] = ACTIONS(2390),
    [sym_float_literal] = ACTIONS(2390),
    [sym__outer_block_doc_comment_start] = ACTIONS(2390),
    [sym__outer_line_doc_comment_start] = ACTIONS(2390),
  },
  [STATE(655)] = {
    [sym_line_comment] = STATE(655),
    [sym_block_comment] = STATE(655),
    [ts_builtin_sym_end] = ACTIONS(2394),
    [sym_identifier] = ACTIONS(2396),
    [anon_sym_SEMI] = ACTIONS(2394),
    [anon_sym_macro] = ACTIONS(2396),
    [anon_sym_LPAREN] = ACTIONS(2394),
    [anon_sym_LBRACE] = ACTIONS(2394),
    [anon_sym_RBRACE] = ACTIONS(2394),
    [anon_sym_macro_rules] = ACTIONS(2396),
    [anon_sym_BANG] = ACTIONS(2394),
    [anon_sym_LBRACK] = ACTIONS(2394),
    [anon_sym_STAR] = ACTIONS(2394),
    [anon_sym_u8] = ACTIONS(2396),
    [anon_sym_i8] = ACTIONS(2396),
    [anon_sym_u16] = ACTIONS(2396),
    [anon_sym_i16] = ACTIONS(2396),
    [anon_sym_u32] = ACTIONS(2396),
    [anon_sym_i32] = ACTIONS(2396),
    [anon_sym_u64] = ACTIONS(2396),
    [anon_sym_i64] = ACTIONS(2396),
    [anon_sym_u128] = ACTIONS(2396),
    [anon_sym_i128] = ACTIONS(2396),
    [anon_sym_isize] = ACTIONS(2396),
    [anon_sym_usize] = ACTIONS(2396),
    [anon_sym_f32] = ACTIONS(2396),
    [anon_sym_f64] = ACTIONS(2396),
    [anon_sym_bool] = ACTIONS(2396),
    [anon_sym_str] = ACTIONS(2396),
    [anon_sym_char] = ACTIONS(2396),
    [anon_sym_DASH] = ACTIONS(2394),
    [anon_sym_AMP] = ACTIONS(2394),
    [anon_sym_PIPE] = ACTIONS(2394),
    [anon_sym_LT] = ACTIONS(2394),
    [anon_sym_DOT_DOT] = ACTIONS(2394),
    [anon_sym_COLON_COLON] = ACTIONS(2394),
    [anon_sym_POUND] = ACTIONS(2394),
    [anon_sym_SQUOTE] = ACTIONS(2396),
    [anon_sym_async] = ACTIONS(2396),
    [anon_sym_auto] = ACTIONS(2396),
    [anon_sym_await] = ACTIONS(2396),
    [anon_sym_become] = ACTIONS(2396),
    [anon_sym_break] = ACTIONS(2396),
    [anon_sym_const] = ACTIONS(2396),
    [anon_sym_continue] = ACTIONS(2396),
    [anon_sym_default] = ACTIONS(2396),
    [anon_sym_enum] = ACTIONS(2396),
    [anon_sym_fn] = ACTIONS(2396),
    [anon_sym_for] = ACTIONS(2396),
    [anon_sym_gen] = ACTIONS(2396),
    [anon_sym_if] = ACTIONS(2396),
    [anon_sym_impl] = ACTIONS(2396),
    [anon_sym_let] = ACTIONS(2396),
    [anon_sym_loop] = ACTIONS(2396),
    [anon_sym_match] = ACTIONS(2396),
    [anon_sym_mod] = ACTIONS(2396),
    [anon_sym_pub] = ACTIONS(2396),
    [anon_sym_return] = ACTIONS(2396),
    [anon_sym_raw] = ACTIONS(2396),
    [anon_sym_static] = ACTIONS(2396),
    [anon_sym_struct] = ACTIONS(2396),
    [anon_sym_trait] = ACTIONS(2396),
    [anon_sym_type] = ACTIONS(2396),
    [anon_sym_union] = ACTIONS(2396),
    [anon_sym_unsafe] = ACTIONS(2396),
    [anon_sym_use] = ACTIONS(2396),
    [anon_sym_while] = ACTIONS(2396),
    [anon_sym_extern] = ACTIONS(2396),
    [anon_sym_safe] = ACTIONS(2396),
    [anon_sym_yield] = ACTIONS(2396),
    [anon_sym_move] = ACTIONS(2396),
    [anon_sym_try] = ACTIONS(2396),
    [sym_integer_literal] = ACTIONS(2394),
    [aux_sym_string_literal_token1] = ACTIONS(2394),
    [sym_char_literal] = ACTIONS(2394),
    [anon_sym_true] = ACTIONS(2396),
    [anon_sym_false] = ACTIONS(2396),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2396),
    [sym_super] = ACTIONS(2396),
    [sym_crate] = ACTIONS(2396),
    [aux_sym_metavariable_token1] = ACTIONS(2394),
    [sym__raw_string_literal_start] = ACTIONS(2394),
    [sym_float_literal] = ACTIONS(2394),
    [sym__outer_block_doc_comment_start] = ACTIONS(2394),
    [sym__outer_line_doc_comment_start] = ACTIONS(2394),
  },
  [STATE(656)] = {
    [sym_line_comment] = STATE(656),
    [sym_block_comment] = STATE(656),
    [ts_builtin_sym_end] = ACTIONS(2398),
    [sym_identifier] = ACTIONS(2400),
    [anon_sym_SEMI] = ACTIONS(2398),
    [anon_sym_macro] = ACTIONS(2400),
    [anon_sym_LPAREN] = ACTIONS(2398),
    [anon_sym_LBRACE] = ACTIONS(2398),
    [anon_sym_RBRACE] = ACTIONS(2398),
    [anon_sym_macro_rules] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2398),
    [anon_sym_LBRACK] = ACTIONS(2398),
    [anon_sym_STAR] = ACTIONS(2398),
    [anon_sym_u8] = ACTIONS(2400),
    [anon_sym_i8] = ACTIONS(2400),
    [anon_sym_u16] = ACTIONS(2400),
    [anon_sym_i16] = ACTIONS(2400),
    [anon_sym_u32] = ACTIONS(2400),
    [anon_sym_i32] = ACTIONS(2400),
    [anon_sym_u64] = ACTIONS(2400),
    [anon_sym_i64] = ACTIONS(2400),
    [anon_sym_u128] = ACTIONS(2400),
    [anon_sym_i128] = ACTIONS(2400),
    [anon_sym_isize] = ACTIONS(2400),
    [anon_sym_usize] = ACTIONS(2400),
    [anon_sym_f32] = ACTIONS(2400),
    [anon_sym_f64] = ACTIONS(2400),
    [anon_sym_bool] = ACTIONS(2400),
    [anon_sym_str] = ACTIONS(2400),
    [anon_sym_char] = ACTIONS(2400),
    [anon_sym_DASH] = ACTIONS(2398),
    [anon_sym_AMP] = ACTIONS(2398),
    [anon_sym_PIPE] = ACTIONS(2398),
    [anon_sym_LT] = ACTIONS(2398),
    [anon_sym_DOT_DOT] = ACTIONS(2398),
    [anon_sym_COLON_COLON] = ACTIONS(2398),
    [anon_sym_POUND] = ACTIONS(2398),
    [anon_sym_SQUOTE] = ACTIONS(2400),
    [anon_sym_async] = ACTIONS(2400),
    [anon_sym_auto] = ACTIONS(2400),
    [anon_sym_await] = ACTIONS(2400),
    [anon_sym_become] = ACTIONS(2400),
    [anon_sym_break] = ACTIONS(2400),
    [anon_sym_const] = ACTIONS(2400),
    [anon_sym_continue] = ACTIONS(2400),
    [anon_sym_default] = ACTIONS(2400),
    [anon_sym_enum] = ACTIONS(2400),
    [anon_sym_fn] = ACTIONS(2400),
    [anon_sym_for] = ACTIONS(2400),
    [anon_sym_gen] = ACTIONS(2400),
    [anon_sym_if] = ACTIONS(2400),
    [anon_sym_impl] = ACTIONS(2400),
    [anon_sym_let] = ACTIONS(2400),
    [anon_sym_loop] = ACTIONS(2400),
    [anon_sym_match] = ACTIONS(2400),
    [anon_sym_mod] = ACTIONS(2400),
    [anon_sym_pub] = ACTIONS(2400),
    [anon_sym_return] = ACTIONS(2400),
    [anon_sym_raw] = ACTIONS(2400),
    [anon_sym_static] = ACTIONS(2400),
    [anon_sym_struct] = ACTIONS(2400),
    [anon_sym_trait] = ACTIONS(2400),
    [anon_sym_type] = ACTIONS(2400),
    [anon_sym_union] = ACTIONS(2400),
    [anon_sym_unsafe] = ACTIONS(2400),
    [anon_sym_use] = ACTIONS(2400),
    [anon_sym_while] = ACTIONS(2400),
    [anon_sym_extern] = ACTIONS(2400),
    [anon_sym_safe] = ACTIONS(2400),
    [anon_sym_yield] = ACTIONS(2400),
    [anon_sym_move] = ACTIONS(2400),
    [anon_sym_try] = ACTIONS(2400),
    [sym_integer_literal] = ACTIONS(2398),
    [aux_sym_string_literal_token1] = ACTIONS(2398),
    [sym_char_literal] = ACTIONS(2398),
    [anon_sym_true] = ACTIONS(2400),
    [anon_sym_false] = ACTIONS(2400),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2400),
    [sym_super] = ACTIONS(2400),
    [sym_crate] = ACTIONS(2400),
    [aux_sym_metavariable_token1] = ACTIONS(2398),
    [sym__raw_string_literal_start] = ACTIONS(2398),
    [sym_float_literal] = ACTIONS(2398),
    [sym__outer_block_doc_comment_start] = ACTIONS(2398),
    [sym__outer_line_doc_comment_start] = ACTIONS(2398),
  },
  [STATE(657)] = {
    [sym_line_comment] = STATE(657),
    [sym_block_comment] = STATE(657),
    [ts_builtin_sym_end] = ACTIONS(2402),
    [sym_identifier] = ACTIONS(2404),
    [anon_sym_SEMI] = ACTIONS(2402),
    [anon_sym_macro] = ACTIONS(2404),
    [anon_sym_LPAREN] = ACTIONS(2402),
    [anon_sym_LBRACE] = ACTIONS(2402),
    [anon_sym_RBRACE] = ACTIONS(2402),
    [anon_sym_macro_rules] = ACTIONS(2404),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_LBRACK] = ACTIONS(2402),
    [anon_sym_STAR] = ACTIONS(2402),
    [anon_sym_u8] = ACTIONS(2404),
    [anon_sym_i8] = ACTIONS(2404),
    [anon_sym_u16] = ACTIONS(2404),
    [anon_sym_i16] = ACTIONS(2404),
    [anon_sym_u32] = ACTIONS(2404),
    [anon_sym_i32] = ACTIONS(2404),
    [anon_sym_u64] = ACTIONS(2404),
    [anon_sym_i64] = ACTIONS(2404),
    [anon_sym_u128] = ACTIONS(2404),
    [anon_sym_i128] = ACTIONS(2404),
    [anon_sym_isize] = ACTIONS(2404),
    [anon_sym_usize] = ACTIONS(2404),
    [anon_sym_f32] = ACTIONS(2404),
    [anon_sym_f64] = ACTIONS(2404),
    [anon_sym_bool] = ACTIONS(2404),
    [anon_sym_str] = ACTIONS(2404),
    [anon_sym_char] = ACTIONS(2404),
    [anon_sym_DASH] = ACTIONS(2402),
    [anon_sym_AMP] = ACTIONS(2402),
    [anon_sym_PIPE] = ACTIONS(2402),
    [anon_sym_LT] = ACTIONS(2402),
    [anon_sym_DOT_DOT] = ACTIONS(2402),
    [anon_sym_COLON_COLON] = ACTIONS(2402),
    [anon_sym_POUND] = ACTIONS(2402),
    [anon_sym_SQUOTE] = ACTIONS(2404),
    [anon_sym_async] = ACTIONS(2404),
    [anon_sym_auto] = ACTIONS(2404),
    [anon_sym_await] = ACTIONS(2404),
    [anon_sym_become] = ACTIONS(2404),
    [anon_sym_break] = ACTIONS(2404),
    [anon_sym_const] = ACTIONS(2404),
    [anon_sym_continue] = ACTIONS(2404),
    [anon_sym_default] = ACTIONS(2404),
    [anon_sym_enum] = ACTIONS(2404),
    [anon_sym_fn] = ACTIONS(2404),
    [anon_sym_for] = ACTIONS(2404),
    [anon_sym_gen] = ACTIONS(2404),
    [anon_sym_if] = ACTIONS(2404),
    [anon_sym_impl] = ACTIONS(2404),
    [anon_sym_let] = ACTIONS(2404),
    [anon_sym_loop] = ACTIONS(2404),
    [anon_sym_match] = ACTIONS(2404),
    [anon_sym_mod] = ACTIONS(2404),
    [anon_sym_pub] = ACTIONS(2404),
    [anon_sym_return] = ACTIONS(2404),
    [anon_sym_raw] = ACTIONS(2404),
    [anon_sym_static] = ACTIONS(2404),
    [anon_sym_struct] = ACTIONS(2404),
    [anon_sym_trait] = ACTIONS(2404),
    [anon_sym_type] = ACTIONS(2404),
    [anon_sym_union] = ACTIONS(2404),
    [anon_sym_unsafe] = ACTIONS(2404),
    [anon_sym_use] = ACTIONS(2404),
    [anon_sym_while] = ACTIONS(2404),
    [anon_sym_extern] = ACTIONS(2404),
    [anon_sym_safe] = ACTIONS(2404),
    [anon_sym_yield] = ACTIONS(2404),
    [anon_sym_move] = ACTIONS(2404),
    [anon_sym_try] = ACTIONS(2404),
    [sym_integer_literal] = ACTIONS(2402),
    [aux_sym_string_literal_token1] = ACTIONS(2402),
    [sym_char_literal] = ACTIONS(2402),
    [anon_sym_true] = ACTIONS(2404),
    [anon_sym_false] = ACTIONS(2404),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2404),
    [sym_super] = ACTIONS(2404),
    [sym_crate] = ACTIONS(2404),
    [aux_sym_metavariable_token1] = ACTIONS(2402),
    [sym__raw_string_literal_start] = ACTIONS(2402),
    [sym_float_literal] = ACTIONS(2402),
    [sym__outer_block_doc_comment_start] = ACTIONS(2402),
    [sym__outer_line_doc_comment_start] = ACTIONS(2402),
  },
  [STATE(658)] = {
    [sym_line_comment] = STATE(658),
    [sym_block_comment] = STATE(658),
    [ts_builtin_sym_end] = ACTIONS(2406),
    [sym_identifier] = ACTIONS(2408),
    [anon_sym_SEMI] = ACTIONS(2406),
    [anon_sym_macro] = ACTIONS(2408),
    [anon_sym_LPAREN] = ACTIONS(2406),
    [anon_sym_LBRACE] = ACTIONS(2406),
    [anon_sym_RBRACE] = ACTIONS(2406),
    [anon_sym_macro_rules] = ACTIONS(2408),
    [anon_sym_BANG] = ACTIONS(2406),
    [anon_sym_LBRACK] = ACTIONS(2406),
    [anon_sym_STAR] = ACTIONS(2406),
    [anon_sym_u8] = ACTIONS(2408),
    [anon_sym_i8] = ACTIONS(2408),
    [anon_sym_u16] = ACTIONS(2408),
    [anon_sym_i16] = ACTIONS(2408),
    [anon_sym_u32] = ACTIONS(2408),
    [anon_sym_i32] = ACTIONS(2408),
    [anon_sym_u64] = ACTIONS(2408),
    [anon_sym_i64] = ACTIONS(2408),
    [anon_sym_u128] = ACTIONS(2408),
    [anon_sym_i128] = ACTIONS(2408),
    [anon_sym_isize] = ACTIONS(2408),
    [anon_sym_usize] = ACTIONS(2408),
    [anon_sym_f32] = ACTIONS(2408),
    [anon_sym_f64] = ACTIONS(2408),
    [anon_sym_bool] = ACTIONS(2408),
    [anon_sym_str] = ACTIONS(2408),
    [anon_sym_char] = ACTIONS(2408),
    [anon_sym_DASH] = ACTIONS(2406),
    [anon_sym_AMP] = ACTIONS(2406),
    [anon_sym_PIPE] = ACTIONS(2406),
    [anon_sym_LT] = ACTIONS(2406),
    [anon_sym_DOT_DOT] = ACTIONS(2406),
    [anon_sym_COLON_COLON] = ACTIONS(2406),
    [anon_sym_POUND] = ACTIONS(2406),
    [anon_sym_SQUOTE] = ACTIONS(2408),
    [anon_sym_async] = ACTIONS(2408),
    [anon_sym_auto] = ACTIONS(2408),
    [anon_sym_await] = ACTIONS(2408),
    [anon_sym_become] = ACTIONS(2408),
    [anon_sym_break] = ACTIONS(2408),
    [anon_sym_const] = ACTIONS(2408),
    [anon_sym_continue] = ACTIONS(2408),
    [anon_sym_default] = ACTIONS(2408),
    [anon_sym_enum] = ACTIONS(2408),
    [anon_sym_fn] = ACTIONS(2408),
    [anon_sym_for] = ACTIONS(2408),
    [anon_sym_gen] = ACTIONS(2408),
    [anon_sym_if] = ACTIONS(2408),
    [anon_sym_impl] = ACTIONS(2408),
    [anon_sym_let] = ACTIONS(2408),
    [anon_sym_loop] = ACTIONS(2408),
    [anon_sym_match] = ACTIONS(2408),
    [anon_sym_mod] = ACTIONS(2408),
    [anon_sym_pub] = ACTIONS(2408),
    [anon_sym_return] = ACTIONS(2408),
    [anon_sym_raw] = ACTIONS(2408),
    [anon_sym_static] = ACTIONS(2408),
    [anon_sym_struct] = ACTIONS(2408),
    [anon_sym_trait] = ACTIONS(2408),
    [anon_sym_type] = ACTIONS(2408),
    [anon_sym_union] = ACTIONS(2408),
    [anon_sym_unsafe] = ACTIONS(2408),
    [anon_sym_use] = ACTIONS(2408),
    [anon_sym_while] = ACTIONS(2408),
    [anon_sym_extern] = ACTIONS(2408),
    [anon_sym_safe] = ACTIONS(2408),
    [anon_sym_yield] = ACTIONS(2408),
    [anon_sym_move] = ACTIONS(2408),
    [anon_sym_try] = ACTIONS(2408),
    [sym_integer_literal] = ACTIONS(2406),
    [aux_sym_string_literal_token1] = ACTIONS(2406),
    [sym_char_literal] = ACTIONS(2406),
    [anon_sym_true] = ACTIONS(2408),
    [anon_sym_false] = ACTIONS(2408),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2408),
    [sym_super] = ACTIONS(2408),
    [sym_crate] = ACTIONS(2408),
    [aux_sym_metavariable_token1] = ACTIONS(2406),
    [sym__raw_string_literal_start] = ACTIONS(2406),
    [sym_float_literal] = ACTIONS(2406),
    [sym__outer_block_doc_comment_start] = ACTIONS(2406),
    [sym__outer_line_doc_comment_start] = ACTIONS(2406),
  },
  [STATE(659)] = {
    [sym_line_comment] = STATE(659),
    [sym_block_comment] = STATE(659),
    [ts_builtin_sym_end] = ACTIONS(2410),
    [sym_identifier] = ACTIONS(2412),
    [anon_sym_SEMI] = ACTIONS(2410),
    [anon_sym_macro] = ACTIONS(2412),
    [anon_sym_LPAREN] = ACTIONS(2410),
    [anon_sym_LBRACE] = ACTIONS(2410),
    [anon_sym_RBRACE] = ACTIONS(2410),
    [anon_sym_macro_rules] = ACTIONS(2412),
    [anon_sym_BANG] = ACTIONS(2410),
    [anon_sym_LBRACK] = ACTIONS(2410),
    [anon_sym_STAR] = ACTIONS(2410),
    [anon_sym_u8] = ACTIONS(2412),
    [anon_sym_i8] = ACTIONS(2412),
    [anon_sym_u16] = ACTIONS(2412),
    [anon_sym_i16] = ACTIONS(2412),
    [anon_sym_u32] = ACTIONS(2412),
    [anon_sym_i32] = ACTIONS(2412),
    [anon_sym_u64] = ACTIONS(2412),
    [anon_sym_i64] = ACTIONS(2412),
    [anon_sym_u128] = ACTIONS(2412),
    [anon_sym_i128] = ACTIONS(2412),
    [anon_sym_isize] = ACTIONS(2412),
    [anon_sym_usize] = ACTIONS(2412),
    [anon_sym_f32] = ACTIONS(2412),
    [anon_sym_f64] = ACTIONS(2412),
    [anon_sym_bool] = ACTIONS(2412),
    [anon_sym_str] = ACTIONS(2412),
    [anon_sym_char] = ACTIONS(2412),
    [anon_sym_DASH] = ACTIONS(2410),
    [anon_sym_AMP] = ACTIONS(2410),
    [anon_sym_PIPE] = ACTIONS(2410),
    [anon_sym_LT] = ACTIONS(2410),
    [anon_sym_DOT_DOT] = ACTIONS(2410),
    [anon_sym_COLON_COLON] = ACTIONS(2410),
    [anon_sym_POUND] = ACTIONS(2410),
    [anon_sym_SQUOTE] = ACTIONS(2412),
    [anon_sym_async] = ACTIONS(2412),
    [anon_sym_auto] = ACTIONS(2412),
    [anon_sym_await] = ACTIONS(2412),
    [anon_sym_become] = ACTIONS(2412),
    [anon_sym_break] = ACTIONS(2412),
    [anon_sym_const] = ACTIONS(2412),
    [anon_sym_continue] = ACTIONS(2412),
    [anon_sym_default] = ACTIONS(2412),
    [anon_sym_enum] = ACTIONS(2412),
    [anon_sym_fn] = ACTIONS(2412),
    [anon_sym_for] = ACTIONS(2412),
    [anon_sym_gen] = ACTIONS(2412),
    [anon_sym_if] = ACTIONS(2412),
    [anon_sym_impl] = ACTIONS(2412),
    [anon_sym_let] = ACTIONS(2412),
    [anon_sym_loop] = ACTIONS(2412),
    [anon_sym_match] = ACTIONS(2412),
    [anon_sym_mod] = ACTIONS(2412),
    [anon_sym_pub] = ACTIONS(2412),
    [anon_sym_return] = ACTIONS(2412),
    [anon_sym_raw] = ACTIONS(2412),
    [anon_sym_static] = ACTIONS(2412),
    [anon_sym_struct] = ACTIONS(2412),
    [anon_sym_trait] = ACTIONS(2412),
    [anon_sym_type] = ACTIONS(2412),
    [anon_sym_union] = ACTIONS(2412),
    [anon_sym_unsafe] = ACTIONS(2412),
    [anon_sym_use] = ACTIONS(2412),
    [anon_sym_while] = ACTIONS(2412),
    [anon_sym_extern] = ACTIONS(2412),
    [anon_sym_safe] = ACTIONS(2412),
    [anon_sym_yield] = ACTIONS(2412),
    [anon_sym_move] = ACTIONS(2412),
    [anon_sym_try] = ACTIONS(2412),
    [sym_integer_literal] = ACTIONS(2410),
    [aux_sym_string_literal_token1] = ACTIONS(2410),
    [sym_char_literal] = ACTIONS(2410),
    [anon_sym_true] = ACTIONS(2412),
    [anon_sym_false] = ACTIONS(2412),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2412),
    [sym_super] = ACTIONS(2412),
    [sym_crate] = ACTIONS(2412),
    [aux_sym_metavariable_token1] = ACTIONS(2410),
    [sym__raw_string_literal_start] = ACTIONS(2410),
    [sym_float_literal] = ACTIONS(2410),
    [sym__outer_block_doc_comment_start] = ACTIONS(2410),
    [sym__outer_line_doc_comment_start] = ACTIONS(2410),
  },
  [STATE(660)] = {
    [sym_line_comment] = STATE(660),
    [sym_block_comment] = STATE(660),
    [ts_builtin_sym_end] = ACTIONS(2414),
    [sym_identifier] = ACTIONS(2416),
    [anon_sym_SEMI] = ACTIONS(2414),
    [anon_sym_macro] = ACTIONS(2416),
    [anon_sym_LPAREN] = ACTIONS(2414),
    [anon_sym_LBRACE] = ACTIONS(2414),
    [anon_sym_RBRACE] = ACTIONS(2414),
    [anon_sym_macro_rules] = ACTIONS(2416),
    [anon_sym_BANG] = ACTIONS(2414),
    [anon_sym_LBRACK] = ACTIONS(2414),
    [anon_sym_STAR] = ACTIONS(2414),
    [anon_sym_u8] = ACTIONS(2416),
    [anon_sym_i8] = ACTIONS(2416),
    [anon_sym_u16] = ACTIONS(2416),
    [anon_sym_i16] = ACTIONS(2416),
    [anon_sym_u32] = ACTIONS(2416),
    [anon_sym_i32] = ACTIONS(2416),
    [anon_sym_u64] = ACTIONS(2416),
    [anon_sym_i64] = ACTIONS(2416),
    [anon_sym_u128] = ACTIONS(2416),
    [anon_sym_i128] = ACTIONS(2416),
    [anon_sym_isize] = ACTIONS(2416),
    [anon_sym_usize] = ACTIONS(2416),
    [anon_sym_f32] = ACTIONS(2416),
    [anon_sym_f64] = ACTIONS(2416),
    [anon_sym_bool] = ACTIONS(2416),
    [anon_sym_str] = ACTIONS(2416),
    [anon_sym_char] = ACTIONS(2416),
    [anon_sym_DASH] = ACTIONS(2414),
    [anon_sym_AMP] = ACTIONS(2414),
    [anon_sym_PIPE] = ACTIONS(2414),
    [anon_sym_LT] = ACTIONS(2414),
    [anon_sym_DOT_DOT] = ACTIONS(2414),
    [anon_sym_COLON_COLON] = ACTIONS(2414),
    [anon_sym_POUND] = ACTIONS(2414),
    [anon_sym_SQUOTE] = ACTIONS(2416),
    [anon_sym_async] = ACTIONS(2416),
    [anon_sym_auto] = ACTIONS(2416),
    [anon_sym_await] = ACTIONS(2416),
    [anon_sym_become] = ACTIONS(2416),
    [anon_sym_break] = ACTIONS(2416),
    [anon_sym_const] = ACTIONS(2416),
    [anon_sym_continue] = ACTIONS(2416),
    [anon_sym_default] = ACTIONS(2416),
    [anon_sym_enum] = ACTIONS(2416),
    [anon_sym_fn] = ACTIONS(2416),
    [anon_sym_for] = ACTIONS(2416),
    [anon_sym_gen] = ACTIONS(2416),
    [anon_sym_if] = ACTIONS(2416),
    [anon_sym_impl] = ACTIONS(2416),
    [anon_sym_let] = ACTIONS(2416),
    [anon_sym_loop] = ACTIONS(2416),
    [anon_sym_match] = ACTIONS(2416),
    [anon_sym_mod] = ACTIONS(2416),
    [anon_sym_pub] = ACTIONS(2416),
    [anon_sym_return] = ACTIONS(2416),
    [anon_sym_raw] = ACTIONS(2416),
    [anon_sym_static] = ACTIONS(2416),
    [anon_sym_struct] = ACTIONS(2416),
    [anon_sym_trait] = ACTIONS(2416),
    [anon_sym_type] = ACTIONS(2416),
    [anon_sym_union] = ACTIONS(2416),
    [anon_sym_unsafe] = ACTIONS(2416),
    [anon_sym_use] = ACTIONS(2416),
    [anon_sym_while] = ACTIONS(2416),
    [anon_sym_extern] = ACTIONS(2416),
    [anon_sym_safe] = ACTIONS(2416),
    [anon_sym_yield] = ACTIONS(2416),
    [anon_sym_move] = ACTIONS(2416),
    [anon_sym_try] = ACTIONS(2416),
    [sym_integer_literal] = ACTIONS(2414),
    [aux_sym_string_literal_token1] = ACTIONS(2414),
    [sym_char_literal] = ACTIONS(2414),
    [anon_sym_true] = ACTIONS(2416),
    [anon_sym_false] = ACTIONS(2416),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2416),
    [sym_super] = ACTIONS(2416),
    [sym_crate] = ACTIONS(2416),
    [aux_sym_metavariable_token1] = ACTIONS(2414),
    [sym__raw_string_literal_start] = ACTIONS(2414),
    [sym_float_literal] = ACTIONS(2414),
    [sym__outer_block_doc_comment_start] = ACTIONS(2414),
    [sym__outer_line_doc_comment_start] = ACTIONS(2414),
  },
  [STATE(661)] = {
    [sym_line_comment] = STATE(661),
    [sym_block_comment] = STATE(661),
    [ts_builtin_sym_end] = ACTIONS(2418),
    [sym_identifier] = ACTIONS(2420),
    [anon_sym_SEMI] = ACTIONS(2418),
    [anon_sym_macro] = ACTIONS(2420),
    [anon_sym_LPAREN] = ACTIONS(2418),
    [anon_sym_LBRACE] = ACTIONS(2418),
    [anon_sym_RBRACE] = ACTIONS(2418),
    [anon_sym_macro_rules] = ACTIONS(2420),
    [anon_sym_BANG] = ACTIONS(2418),
    [anon_sym_LBRACK] = ACTIONS(2418),
    [anon_sym_STAR] = ACTIONS(2418),
    [anon_sym_u8] = ACTIONS(2420),
    [anon_sym_i8] = ACTIONS(2420),
    [anon_sym_u16] = ACTIONS(2420),
    [anon_sym_i16] = ACTIONS(2420),
    [anon_sym_u32] = ACTIONS(2420),
    [anon_sym_i32] = ACTIONS(2420),
    [anon_sym_u64] = ACTIONS(2420),
    [anon_sym_i64] = ACTIONS(2420),
    [anon_sym_u128] = ACTIONS(2420),
    [anon_sym_i128] = ACTIONS(2420),
    [anon_sym_isize] = ACTIONS(2420),
    [anon_sym_usize] = ACTIONS(2420),
    [anon_sym_f32] = ACTIONS(2420),
    [anon_sym_f64] = ACTIONS(2420),
    [anon_sym_bool] = ACTIONS(2420),
    [anon_sym_str] = ACTIONS(2420),
    [anon_sym_char] = ACTIONS(2420),
    [anon_sym_DASH] = ACTIONS(2418),
    [anon_sym_AMP] = ACTIONS(2418),
    [anon_sym_PIPE] = ACTIONS(2418),
    [anon_sym_LT] = ACTIONS(2418),
    [anon_sym_DOT_DOT] = ACTIONS(2418),
    [anon_sym_COLON_COLON] = ACTIONS(2418),
    [anon_sym_POUND] = ACTIONS(2418),
    [anon_sym_SQUOTE] = ACTIONS(2420),
    [anon_sym_async] = ACTIONS(2420),
    [anon_sym_auto] = ACTIONS(2420),
    [anon_sym_await] = ACTIONS(2420),
    [anon_sym_become] = ACTIONS(2420),
    [anon_sym_break] = ACTIONS(2420),
    [anon_sym_const] = ACTIONS(2420),
    [anon_sym_continue] = ACTIONS(2420),
    [anon_sym_default] = ACTIONS(2420),
    [anon_sym_enum] = ACTIONS(2420),
    [anon_sym_fn] = ACTIONS(2420),
    [anon_sym_for] = ACTIONS(2420),
    [anon_sym_gen] = ACTIONS(2420),
    [anon_sym_if] = ACTIONS(2420),
    [anon_sym_impl] = ACTIONS(2420),
    [anon_sym_let] = ACTIONS(2420),
    [anon_sym_loop] = ACTIONS(2420),
    [anon_sym_match] = ACTIONS(2420),
    [anon_sym_mod] = ACTIONS(2420),
    [anon_sym_pub] = ACTIONS(2420),
    [anon_sym_return] = ACTIONS(2420),
    [anon_sym_raw] = ACTIONS(2420),
    [anon_sym_static] = ACTIONS(2420),
    [anon_sym_struct] = ACTIONS(2420),
    [anon_sym_trait] = ACTIONS(2420),
    [anon_sym_type] = ACTIONS(2420),
    [anon_sym_union] = ACTIONS(2420),
    [anon_sym_unsafe] = ACTIONS(2420),
    [anon_sym_use] = ACTIONS(2420),
    [anon_sym_while] = ACTIONS(2420),
    [anon_sym_extern] = ACTIONS(2420),
    [anon_sym_safe] = ACTIONS(2420),
    [anon_sym_yield] = ACTIONS(2420),
    [anon_sym_move] = ACTIONS(2420),
    [anon_sym_try] = ACTIONS(2420),
    [sym_integer_literal] = ACTIONS(2418),
    [aux_sym_string_literal_token1] = ACTIONS(2418),
    [sym_char_literal] = ACTIONS(2418),
    [anon_sym_true] = ACTIONS(2420),
    [anon_sym_false] = ACTIONS(2420),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2420),
    [sym_super] = ACTIONS(2420),
    [sym_crate] = ACTIONS(2420),
    [aux_sym_metavariable_token1] = ACTIONS(2418),
    [sym__raw_string_literal_start] = ACTIONS(2418),
    [sym_float_literal] = ACTIONS(2418),
    [sym__outer_block_doc_comment_start] = ACTIONS(2418),
    [sym__outer_line_doc_comment_start] = ACTIONS(2418),
  },
  [STATE(662)] = {
    [sym_line_comment] = STATE(662),
    [sym_block_comment] = STATE(662),
    [ts_builtin_sym_end] = ACTIONS(2422),
    [sym_identifier] = ACTIONS(2424),
    [anon_sym_SEMI] = ACTIONS(2422),
    [anon_sym_macro] = ACTIONS(2424),
    [anon_sym_LPAREN] = ACTIONS(2422),
    [anon_sym_LBRACE] = ACTIONS(2422),
    [anon_sym_RBRACE] = ACTIONS(2422),
    [anon_sym_macro_rules] = ACTIONS(2424),
    [anon_sym_BANG] = ACTIONS(2422),
    [anon_sym_LBRACK] = ACTIONS(2422),
    [anon_sym_STAR] = ACTIONS(2422),
    [anon_sym_u8] = ACTIONS(2424),
    [anon_sym_i8] = ACTIONS(2424),
    [anon_sym_u16] = ACTIONS(2424),
    [anon_sym_i16] = ACTIONS(2424),
    [anon_sym_u32] = ACTIONS(2424),
    [anon_sym_i32] = ACTIONS(2424),
    [anon_sym_u64] = ACTIONS(2424),
    [anon_sym_i64] = ACTIONS(2424),
    [anon_sym_u128] = ACTIONS(2424),
    [anon_sym_i128] = ACTIONS(2424),
    [anon_sym_isize] = ACTIONS(2424),
    [anon_sym_usize] = ACTIONS(2424),
    [anon_sym_f32] = ACTIONS(2424),
    [anon_sym_f64] = ACTIONS(2424),
    [anon_sym_bool] = ACTIONS(2424),
    [anon_sym_str] = ACTIONS(2424),
    [anon_sym_char] = ACTIONS(2424),
    [anon_sym_DASH] = ACTIONS(2422),
    [anon_sym_AMP] = ACTIONS(2422),
    [anon_sym_PIPE] = ACTIONS(2422),
    [anon_sym_LT] = ACTIONS(2422),
    [anon_sym_DOT_DOT] = ACTIONS(2422),
    [anon_sym_COLON_COLON] = ACTIONS(2422),
    [anon_sym_POUND] = ACTIONS(2422),
    [anon_sym_SQUOTE] = ACTIONS(2424),
    [anon_sym_async] = ACTIONS(2424),
    [anon_sym_auto] = ACTIONS(2424),
    [anon_sym_await] = ACTIONS(2424),
    [anon_sym_become] = ACTIONS(2424),
    [anon_sym_break] = ACTIONS(2424),
    [anon_sym_const] = ACTIONS(2424),
    [anon_sym_continue] = ACTIONS(2424),
    [anon_sym_default] = ACTIONS(2424),
    [anon_sym_enum] = ACTIONS(2424),
    [anon_sym_fn] = ACTIONS(2424),
    [anon_sym_for] = ACTIONS(2424),
    [anon_sym_gen] = ACTIONS(2424),
    [anon_sym_if] = ACTIONS(2424),
    [anon_sym_impl] = ACTIONS(2424),
    [anon_sym_let] = ACTIONS(2424),
    [anon_sym_loop] = ACTIONS(2424),
    [anon_sym_match] = ACTIONS(2424),
    [anon_sym_mod] = ACTIONS(2424),
    [anon_sym_pub] = ACTIONS(2424),
    [anon_sym_return] = ACTIONS(2424),
    [anon_sym_raw] = ACTIONS(2424),
    [anon_sym_static] = ACTIONS(2424),
    [anon_sym_struct] = ACTIONS(2424),
    [anon_sym_trait] = ACTIONS(2424),
    [anon_sym_type] = ACTIONS(2424),
    [anon_sym_union] = ACTIONS(2424),
    [anon_sym_unsafe] = ACTIONS(2424),
    [anon_sym_use] = ACTIONS(2424),
    [anon_sym_while] = ACTIONS(2424),
    [anon_sym_extern] = ACTIONS(2424),
    [anon_sym_safe] = ACTIONS(2424),
    [anon_sym_yield] = ACTIONS(2424),
    [anon_sym_move] = ACTIONS(2424),
    [anon_sym_try] = ACTIONS(2424),
    [sym_integer_literal] = ACTIONS(2422),
    [aux_sym_string_literal_token1] = ACTIONS(2422),
    [sym_char_literal] = ACTIONS(2422),
    [anon_sym_true] = ACTIONS(2424),
    [anon_sym_false] = ACTIONS(2424),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2424),
    [sym_super] = ACTIONS(2424),
    [sym_crate] = ACTIONS(2424),
    [aux_sym_metavariable_token1] = ACTIONS(2422),
    [sym__raw_string_literal_start] = ACTIONS(2422),
    [sym_float_literal] = ACTIONS(2422),
    [sym__outer_block_doc_comment_start] = ACTIONS(2422),
    [sym__outer_line_doc_comment_start] = ACTIONS(2422),
  },
  [STATE(663)] = {
    [sym_line_comment] = STATE(663),
    [sym_block_comment] = STATE(663),
    [ts_builtin_sym_end] = ACTIONS(2426),
    [sym_identifier] = ACTIONS(2428),
    [anon_sym_SEMI] = ACTIONS(2426),
    [anon_sym_macro] = ACTIONS(2428),
    [anon_sym_LPAREN] = ACTIONS(2426),
    [anon_sym_LBRACE] = ACTIONS(2426),
    [anon_sym_RBRACE] = ACTIONS(2426),
    [anon_sym_macro_rules] = ACTIONS(2428),
    [anon_sym_BANG] = ACTIONS(2426),
    [anon_sym_LBRACK] = ACTIONS(2426),
    [anon_sym_STAR] = ACTIONS(2426),
    [anon_sym_u8] = ACTIONS(2428),
    [anon_sym_i8] = ACTIONS(2428),
    [anon_sym_u16] = ACTIONS(2428),
    [anon_sym_i16] = ACTIONS(2428),
    [anon_sym_u32] = ACTIONS(2428),
    [anon_sym_i32] = ACTIONS(2428),
    [anon_sym_u64] = ACTIONS(2428),
    [anon_sym_i64] = ACTIONS(2428),
    [anon_sym_u128] = ACTIONS(2428),
    [anon_sym_i128] = ACTIONS(2428),
    [anon_sym_isize] = ACTIONS(2428),
    [anon_sym_usize] = ACTIONS(2428),
    [anon_sym_f32] = ACTIONS(2428),
    [anon_sym_f64] = ACTIONS(2428),
    [anon_sym_bool] = ACTIONS(2428),
    [anon_sym_str] = ACTIONS(2428),
    [anon_sym_char] = ACTIONS(2428),
    [anon_sym_DASH] = ACTIONS(2426),
    [anon_sym_AMP] = ACTIONS(2426),
    [anon_sym_PIPE] = ACTIONS(2426),
    [anon_sym_LT] = ACTIONS(2426),
    [anon_sym_DOT_DOT] = ACTIONS(2426),
    [anon_sym_COLON_COLON] = ACTIONS(2426),
    [anon_sym_POUND] = ACTIONS(2426),
    [anon_sym_SQUOTE] = ACTIONS(2428),
    [anon_sym_async] = ACTIONS(2428),
    [anon_sym_auto] = ACTIONS(2428),
    [anon_sym_await] = ACTIONS(2428),
    [anon_sym_become] = ACTIONS(2428),
    [anon_sym_break] = ACTIONS(2428),
    [anon_sym_const] = ACTIONS(2428),
    [anon_sym_continue] = ACTIONS(2428),
    [anon_sym_default] = ACTIONS(2428),
    [anon_sym_enum] = ACTIONS(2428),
    [anon_sym_fn] = ACTIONS(2428),
    [anon_sym_for] = ACTIONS(2428),
    [anon_sym_gen] = ACTIONS(2428),
    [anon_sym_if] = ACTIONS(2428),
    [anon_sym_impl] = ACTIONS(2428),
    [anon_sym_let] = ACTIONS(2428),
    [anon_sym_loop] = ACTIONS(2428),
    [anon_sym_match] = ACTIONS(2428),
    [anon_sym_mod] = ACTIONS(2428),
    [anon_sym_pub] = ACTIONS(2428),
    [anon_sym_return] = ACTIONS(2428),
    [anon_sym_raw] = ACTIONS(2428),
    [anon_sym_static] = ACTIONS(2428),
    [anon_sym_struct] = ACTIONS(2428),
    [anon_sym_trait] = ACTIONS(2428),
    [anon_sym_type] = ACTIONS(2428),
    [anon_sym_union] = ACTIONS(2428),
    [anon_sym_unsafe] = ACTIONS(2428),
    [anon_sym_use] = ACTIONS(2428),
    [anon_sym_while] = ACTIONS(2428),
    [anon_sym_extern] = ACTIONS(2428),
    [anon_sym_safe] = ACTIONS(2428),
    [anon_sym_yield] = ACTIONS(2428),
    [anon_sym_move] = ACTIONS(2428),
    [anon_sym_try] = ACTIONS(2428),
    [sym_integer_literal] = ACTIONS(2426),
    [aux_sym_string_literal_token1] = ACTIONS(2426),
    [sym_char_literal] = ACTIONS(2426),
    [anon_sym_true] = ACTIONS(2428),
    [anon_sym_false] = ACTIONS(2428),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2428),
    [sym_super] = ACTIONS(2428),
    [sym_crate] = ACTIONS(2428),
    [aux_sym_metavariable_token1] = ACTIONS(2426),
    [sym__raw_string_literal_start] = ACTIONS(2426),
    [sym_float_literal] = ACTIONS(2426),
    [sym__outer_block_doc_comment_start] = ACTIONS(2426),
    [sym__outer_line_doc_comment_start] = ACTIONS(2426),
  },
  [STATE(664)] = {
    [sym_line_comment] = STATE(664),
    [sym_block_comment] = STATE(664),
    [ts_builtin_sym_end] = ACTIONS(2430),
    [sym_identifier] = ACTIONS(2432),
    [anon_sym_SEMI] = ACTIONS(2430),
    [anon_sym_macro] = ACTIONS(2432),
    [anon_sym_LPAREN] = ACTIONS(2430),
    [anon_sym_LBRACE] = ACTIONS(2430),
    [anon_sym_RBRACE] = ACTIONS(2430),
    [anon_sym_macro_rules] = ACTIONS(2432),
    [anon_sym_BANG] = ACTIONS(2430),
    [anon_sym_LBRACK] = ACTIONS(2430),
    [anon_sym_STAR] = ACTIONS(2430),
    [anon_sym_u8] = ACTIONS(2432),
    [anon_sym_i8] = ACTIONS(2432),
    [anon_sym_u16] = ACTIONS(2432),
    [anon_sym_i16] = ACTIONS(2432),
    [anon_sym_u32] = ACTIONS(2432),
    [anon_sym_i32] = ACTIONS(2432),
    [anon_sym_u64] = ACTIONS(2432),
    [anon_sym_i64] = ACTIONS(2432),
    [anon_sym_u128] = ACTIONS(2432),
    [anon_sym_i128] = ACTIONS(2432),
    [anon_sym_isize] = ACTIONS(2432),
    [anon_sym_usize] = ACTIONS(2432),
    [anon_sym_f32] = ACTIONS(2432),
    [anon_sym_f64] = ACTIONS(2432),
    [anon_sym_bool] = ACTIONS(2432),
    [anon_sym_str] = ACTIONS(2432),
    [anon_sym_char] = ACTIONS(2432),
    [anon_sym_DASH] = ACTIONS(2430),
    [anon_sym_AMP] = ACTIONS(2430),
    [anon_sym_PIPE] = ACTIONS(2430),
    [anon_sym_LT] = ACTIONS(2430),
    [anon_sym_DOT_DOT] = ACTIONS(2430),
    [anon_sym_COLON_COLON] = ACTIONS(2430),
    [anon_sym_POUND] = ACTIONS(2430),
    [anon_sym_SQUOTE] = ACTIONS(2432),
    [anon_sym_async] = ACTIONS(2432),
    [anon_sym_auto] = ACTIONS(2432),
    [anon_sym_await] = ACTIONS(2432),
    [anon_sym_become] = ACTIONS(2432),
    [anon_sym_break] = ACTIONS(2432),
    [anon_sym_const] = ACTIONS(2432),
    [anon_sym_continue] = ACTIONS(2432),
    [anon_sym_default] = ACTIONS(2432),
    [anon_sym_enum] = ACTIONS(2432),
    [anon_sym_fn] = ACTIONS(2432),
    [anon_sym_for] = ACTIONS(2432),
    [anon_sym_gen] = ACTIONS(2432),
    [anon_sym_if] = ACTIONS(2432),
    [anon_sym_impl] = ACTIONS(2432),
    [anon_sym_let] = ACTIONS(2432),
    [anon_sym_loop] = ACTIONS(2432),
    [anon_sym_match] = ACTIONS(2432),
    [anon_sym_mod] = ACTIONS(2432),
    [anon_sym_pub] = ACTIONS(2432),
    [anon_sym_return] = ACTIONS(2432),
    [anon_sym_raw] = ACTIONS(2432),
    [anon_sym_static] = ACTIONS(2432),
    [anon_sym_struct] = ACTIONS(2432),
    [anon_sym_trait] = ACTIONS(2432),
    [anon_sym_type] = ACTIONS(2432),
    [anon_sym_union] = ACTIONS(2432),
    [anon_sym_unsafe] = ACTIONS(2432),
    [anon_sym_use] = ACTIONS(2432),
    [anon_sym_while] = ACTIONS(2432),
    [anon_sym_extern] = ACTIONS(2432),
    [anon_sym_safe] = ACTIONS(2432),
    [anon_sym_yield] = ACTIONS(2432),
    [anon_sym_move] = ACTIONS(2432),
    [anon_sym_try] = ACTIONS(2432),
    [sym_integer_literal] = ACTIONS(2430),
    [aux_sym_string_literal_token1] = ACTIONS(2430),
    [sym_char_literal] = ACTIONS(2430),
    [anon_sym_true] = ACTIONS(2432),
    [anon_sym_false] = ACTIONS(2432),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2432),
    [sym_super] = ACTIONS(2432),
    [sym_crate] = ACTIONS(2432),
    [aux_sym_metavariable_token1] = ACTIONS(2430),
    [sym__raw_string_literal_start] = ACTIONS(2430),
    [sym_float_literal] = ACTIONS(2430),
    [sym__outer_block_doc_comment_start] = ACTIONS(2430),
    [sym__outer_line_doc_comment_start] = ACTIONS(2430),
  },
  [STATE(665)] = {
    [sym_line_comment] = STATE(665),
    [sym_block_comment] = STATE(665),
    [ts_builtin_sym_end] = ACTIONS(2434),
    [sym_identifier] = ACTIONS(2436),
    [anon_sym_SEMI] = ACTIONS(2434),
    [anon_sym_macro] = ACTIONS(2436),
    [anon_sym_LPAREN] = ACTIONS(2434),
    [anon_sym_LBRACE] = ACTIONS(2434),
    [anon_sym_RBRACE] = ACTIONS(2434),
    [anon_sym_macro_rules] = ACTIONS(2436),
    [anon_sym_BANG] = ACTIONS(2434),
    [anon_sym_LBRACK] = ACTIONS(2434),
    [anon_sym_STAR] = ACTIONS(2434),
    [anon_sym_u8] = ACTIONS(2436),
    [anon_sym_i8] = ACTIONS(2436),
    [anon_sym_u16] = ACTIONS(2436),
    [anon_sym_i16] = ACTIONS(2436),
    [anon_sym_u32] = ACTIONS(2436),
    [anon_sym_i32] = ACTIONS(2436),
    [anon_sym_u64] = ACTIONS(2436),
    [anon_sym_i64] = ACTIONS(2436),
    [anon_sym_u128] = ACTIONS(2436),
    [anon_sym_i128] = ACTIONS(2436),
    [anon_sym_isize] = ACTIONS(2436),
    [anon_sym_usize] = ACTIONS(2436),
    [anon_sym_f32] = ACTIONS(2436),
    [anon_sym_f64] = ACTIONS(2436),
    [anon_sym_bool] = ACTIONS(2436),
    [anon_sym_str] = ACTIONS(2436),
    [anon_sym_char] = ACTIONS(2436),
    [anon_sym_DASH] = ACTIONS(2434),
    [anon_sym_AMP] = ACTIONS(2434),
    [anon_sym_PIPE] = ACTIONS(2434),
    [anon_sym_LT] = ACTIONS(2434),
    [anon_sym_DOT_DOT] = ACTIONS(2434),
    [anon_sym_COLON_COLON] = ACTIONS(2434),
    [anon_sym_POUND] = ACTIONS(2434),
    [anon_sym_SQUOTE] = ACTIONS(2436),
    [anon_sym_async] = ACTIONS(2436),
    [anon_sym_auto] = ACTIONS(2436),
    [anon_sym_await] = ACTIONS(2436),
    [anon_sym_become] = ACTIONS(2436),
    [anon_sym_break] = ACTIONS(2436),
    [anon_sym_const] = ACTIONS(2436),
    [anon_sym_continue] = ACTIONS(2436),
    [anon_sym_default] = ACTIONS(2436),
    [anon_sym_enum] = ACTIONS(2436),
    [anon_sym_fn] = ACTIONS(2436),
    [anon_sym_for] = ACTIONS(2436),
    [anon_sym_gen] = ACTIONS(2436),
    [anon_sym_if] = ACTIONS(2436),
    [anon_sym_impl] = ACTIONS(2436),
    [anon_sym_let] = ACTIONS(2436),
    [anon_sym_loop] = ACTIONS(2436),
    [anon_sym_match] = ACTIONS(2436),
    [anon_sym_mod] = ACTIONS(2436),
    [anon_sym_pub] = ACTIONS(2436),
    [anon_sym_return] = ACTIONS(2436),
    [anon_sym_raw] = ACTIONS(2436),
    [anon_sym_static] = ACTIONS(2436),
    [anon_sym_struct] = ACTIONS(2436),
    [anon_sym_trait] = ACTIONS(2436),
    [anon_sym_type] = ACTIONS(2436),
    [anon_sym_union] = ACTIONS(2436),
    [anon_sym_unsafe] = ACTIONS(2436),
    [anon_sym_use] = ACTIONS(2436),
    [anon_sym_while] = ACTIONS(2436),
    [anon_sym_extern] = ACTIONS(2436),
    [anon_sym_safe] = ACTIONS(2436),
    [anon_sym_yield] = ACTIONS(2436),
    [anon_sym_move] = ACTIONS(2436),
    [anon_sym_try] = ACTIONS(2436),
    [sym_integer_literal] = ACTIONS(2434),
    [aux_sym_string_literal_token1] = ACTIONS(2434),
    [sym_char_literal] = ACTIONS(2434),
    [anon_sym_true] = ACTIONS(2436),
    [anon_sym_false] = ACTIONS(2436),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2436),
    [sym_super] = ACTIONS(2436),
    [sym_crate] = ACTIONS(2436),
    [aux_sym_metavariable_token1] = ACTIONS(2434),
    [sym__raw_string_literal_start] = ACTIONS(2434),
    [sym_float_literal] = ACTIONS(2434),
    [sym__outer_block_doc_comment_start] = ACTIONS(2434),
    [sym__outer_line_doc_comment_start] = ACTIONS(2434),
  },
  [STATE(666)] = {
    [sym_line_comment] = STATE(666),
    [sym_block_comment] = STATE(666),
    [ts_builtin_sym_end] = ACTIONS(2438),
    [sym_identifier] = ACTIONS(2440),
    [anon_sym_SEMI] = ACTIONS(2438),
    [anon_sym_macro] = ACTIONS(2440),
    [anon_sym_LPAREN] = ACTIONS(2438),
    [anon_sym_LBRACE] = ACTIONS(2438),
    [anon_sym_RBRACE] = ACTIONS(2438),
    [anon_sym_macro_rules] = ACTIONS(2440),
    [anon_sym_BANG] = ACTIONS(2438),
    [anon_sym_LBRACK] = ACTIONS(2438),
    [anon_sym_STAR] = ACTIONS(2438),
    [anon_sym_u8] = ACTIONS(2440),
    [anon_sym_i8] = ACTIONS(2440),
    [anon_sym_u16] = ACTIONS(2440),
    [anon_sym_i16] = ACTIONS(2440),
    [anon_sym_u32] = ACTIONS(2440),
    [anon_sym_i32] = ACTIONS(2440),
    [anon_sym_u64] = ACTIONS(2440),
    [anon_sym_i64] = ACTIONS(2440),
    [anon_sym_u128] = ACTIONS(2440),
    [anon_sym_i128] = ACTIONS(2440),
    [anon_sym_isize] = ACTIONS(2440),
    [anon_sym_usize] = ACTIONS(2440),
    [anon_sym_f32] = ACTIONS(2440),
    [anon_sym_f64] = ACTIONS(2440),
    [anon_sym_bool] = ACTIONS(2440),
    [anon_sym_str] = ACTIONS(2440),
    [anon_sym_char] = ACTIONS(2440),
    [anon_sym_DASH] = ACTIONS(2438),
    [anon_sym_AMP] = ACTIONS(2438),
    [anon_sym_PIPE] = ACTIONS(2438),
    [anon_sym_LT] = ACTIONS(2438),
    [anon_sym_DOT_DOT] = ACTIONS(2438),
    [anon_sym_COLON_COLON] = ACTIONS(2438),
    [anon_sym_POUND] = ACTIONS(2438),
    [anon_sym_SQUOTE] = ACTIONS(2440),
    [anon_sym_async] = ACTIONS(2440),
    [anon_sym_auto] = ACTIONS(2440),
    [anon_sym_await] = ACTIONS(2440),
    [anon_sym_become] = ACTIONS(2440),
    [anon_sym_break] = ACTIONS(2440),
    [anon_sym_const] = ACTIONS(2440),
    [anon_sym_continue] = ACTIONS(2440),
    [anon_sym_default] = ACTIONS(2440),
    [anon_sym_enum] = ACTIONS(2440),
    [anon_sym_fn] = ACTIONS(2440),
    [anon_sym_for] = ACTIONS(2440),
    [anon_sym_gen] = ACTIONS(2440),
    [anon_sym_if] = ACTIONS(2440),
    [anon_sym_impl] = ACTIONS(2440),
    [anon_sym_let] = ACTIONS(2440),
    [anon_sym_loop] = ACTIONS(2440),
    [anon_sym_match] = ACTIONS(2440),
    [anon_sym_mod] = ACTIONS(2440),
    [anon_sym_pub] = ACTIONS(2440),
    [anon_sym_return] = ACTIONS(2440),
    [anon_sym_raw] = ACTIONS(2440),
    [anon_sym_static] = ACTIONS(2440),
    [anon_sym_struct] = ACTIONS(2440),
    [anon_sym_trait] = ACTIONS(2440),
    [anon_sym_type] = ACTIONS(2440),
    [anon_sym_union] = ACTIONS(2440),
    [anon_sym_unsafe] = ACTIONS(2440),
    [anon_sym_use] = ACTIONS(2440),
    [anon_sym_while] = ACTIONS(2440),
    [anon_sym_extern] = ACTIONS(2440),
    [anon_sym_safe] = ACTIONS(2440),
    [anon_sym_yield] = ACTIONS(2440),
    [anon_sym_move] = ACTIONS(2440),
    [anon_sym_try] = ACTIONS(2440),
    [sym_integer_literal] = ACTIONS(2438),
    [aux_sym_string_literal_token1] = ACTIONS(2438),
    [sym_char_literal] = ACTIONS(2438),
    [anon_sym_true] = ACTIONS(2440),
    [anon_sym_false] = ACTIONS(2440),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2440),
    [sym_super] = ACTIONS(2440),
    [sym_crate] = ACTIONS(2440),
    [aux_sym_metavariable_token1] = ACTIONS(2438),
    [sym__raw_string_literal_start] = ACTIONS(2438),
    [sym_float_literal] = ACTIONS(2438),
    [sym__outer_block_doc_comment_start] = ACTIONS(2438),
    [sym__outer_line_doc_comment_start] = ACTIONS(2438),
  },
  [STATE(667)] = {
    [sym_line_comment] = STATE(667),
    [sym_block_comment] = STATE(667),
    [ts_builtin_sym_end] = ACTIONS(2442),
    [sym_identifier] = ACTIONS(2444),
    [anon_sym_SEMI] = ACTIONS(2442),
    [anon_sym_macro] = ACTIONS(2444),
    [anon_sym_LPAREN] = ACTIONS(2442),
    [anon_sym_LBRACE] = ACTIONS(2442),
    [anon_sym_RBRACE] = ACTIONS(2442),
    [anon_sym_macro_rules] = ACTIONS(2444),
    [anon_sym_BANG] = ACTIONS(2442),
    [anon_sym_LBRACK] = ACTIONS(2442),
    [anon_sym_STAR] = ACTIONS(2442),
    [anon_sym_u8] = ACTIONS(2444),
    [anon_sym_i8] = ACTIONS(2444),
    [anon_sym_u16] = ACTIONS(2444),
    [anon_sym_i16] = ACTIONS(2444),
    [anon_sym_u32] = ACTIONS(2444),
    [anon_sym_i32] = ACTIONS(2444),
    [anon_sym_u64] = ACTIONS(2444),
    [anon_sym_i64] = ACTIONS(2444),
    [anon_sym_u128] = ACTIONS(2444),
    [anon_sym_i128] = ACTIONS(2444),
    [anon_sym_isize] = ACTIONS(2444),
    [anon_sym_usize] = ACTIONS(2444),
    [anon_sym_f32] = ACTIONS(2444),
    [anon_sym_f64] = ACTIONS(2444),
    [anon_sym_bool] = ACTIONS(2444),
    [anon_sym_str] = ACTIONS(2444),
    [anon_sym_char] = ACTIONS(2444),
    [anon_sym_DASH] = ACTIONS(2442),
    [anon_sym_AMP] = ACTIONS(2442),
    [anon_sym_PIPE] = ACTIONS(2442),
    [anon_sym_LT] = ACTIONS(2442),
    [anon_sym_DOT_DOT] = ACTIONS(2442),
    [anon_sym_COLON_COLON] = ACTIONS(2442),
    [anon_sym_POUND] = ACTIONS(2442),
    [anon_sym_SQUOTE] = ACTIONS(2444),
    [anon_sym_async] = ACTIONS(2444),
    [anon_sym_auto] = ACTIONS(2444),
    [anon_sym_await] = ACTIONS(2444),
    [anon_sym_become] = ACTIONS(2444),
    [anon_sym_break] = ACTIONS(2444),
    [anon_sym_const] = ACTIONS(2444),
    [anon_sym_continue] = ACTIONS(2444),
    [anon_sym_default] = ACTIONS(2444),
    [anon_sym_enum] = ACTIONS(2444),
    [anon_sym_fn] = ACTIONS(2444),
    [anon_sym_for] = ACTIONS(2444),
    [anon_sym_gen] = ACTIONS(2444),
    [anon_sym_if] = ACTIONS(2444),
    [anon_sym_impl] = ACTIONS(2444),
    [anon_sym_let] = ACTIONS(2444),
    [anon_sym_loop] = ACTIONS(2444),
    [anon_sym_match] = ACTIONS(2444),
    [anon_sym_mod] = ACTIONS(2444),
    [anon_sym_pub] = ACTIONS(2444),
    [anon_sym_return] = ACTIONS(2444),
    [anon_sym_raw] = ACTIONS(2444),
    [anon_sym_static] = ACTIONS(2444),
    [anon_sym_struct] = ACTIONS(2444),
    [anon_sym_trait] = ACTIONS(2444),
    [anon_sym_type] = ACTIONS(2444),
    [anon_sym_union] = ACTIONS(2444),
    [anon_sym_unsafe] = ACTIONS(2444),
    [anon_sym_use] = ACTIONS(2444),
    [anon_sym_while] = ACTIONS(2444),
    [anon_sym_extern] = ACTIONS(2444),
    [anon_sym_safe] = ACTIONS(2444),
    [anon_sym_yield] = ACTIONS(2444),
    [anon_sym_move] = ACTIONS(2444),
    [anon_sym_try] = ACTIONS(2444),
    [sym_integer_literal] = ACTIONS(2442),
    [aux_sym_string_literal_token1] = ACTIONS(2442),
    [sym_char_literal] = ACTIONS(2442),
    [anon_sym_true] = ACTIONS(2444),
    [anon_sym_false] = ACTIONS(2444),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2444),
    [sym_super] = ACTIONS(2444),
    [sym_crate] = ACTIONS(2444),
    [aux_sym_metavariable_token1] = ACTIONS(2442),
    [sym__raw_string_literal_start] = ACTIONS(2442),
    [sym_float_literal] = ACTIONS(2442),
    [sym__outer_block_doc_comment_start] = ACTIONS(2442),
    [sym__outer_line_doc_comment_start] = ACTIONS(2442),
  },
  [STATE(668)] = {
    [sym_line_comment] = STATE(668),
    [sym_block_comment] = STATE(668),
    [ts_builtin_sym_end] = ACTIONS(2446),
    [sym_identifier] = ACTIONS(2448),
    [anon_sym_SEMI] = ACTIONS(2446),
    [anon_sym_macro] = ACTIONS(2448),
    [anon_sym_LPAREN] = ACTIONS(2446),
    [anon_sym_LBRACE] = ACTIONS(2446),
    [anon_sym_RBRACE] = ACTIONS(2446),
    [anon_sym_macro_rules] = ACTIONS(2448),
    [anon_sym_BANG] = ACTIONS(2446),
    [anon_sym_LBRACK] = ACTIONS(2446),
    [anon_sym_STAR] = ACTIONS(2446),
    [anon_sym_u8] = ACTIONS(2448),
    [anon_sym_i8] = ACTIONS(2448),
    [anon_sym_u16] = ACTIONS(2448),
    [anon_sym_i16] = ACTIONS(2448),
    [anon_sym_u32] = ACTIONS(2448),
    [anon_sym_i32] = ACTIONS(2448),
    [anon_sym_u64] = ACTIONS(2448),
    [anon_sym_i64] = ACTIONS(2448),
    [anon_sym_u128] = ACTIONS(2448),
    [anon_sym_i128] = ACTIONS(2448),
    [anon_sym_isize] = ACTIONS(2448),
    [anon_sym_usize] = ACTIONS(2448),
    [anon_sym_f32] = ACTIONS(2448),
    [anon_sym_f64] = ACTIONS(2448),
    [anon_sym_bool] = ACTIONS(2448),
    [anon_sym_str] = ACTIONS(2448),
    [anon_sym_char] = ACTIONS(2448),
    [anon_sym_DASH] = ACTIONS(2446),
    [anon_sym_AMP] = ACTIONS(2446),
    [anon_sym_PIPE] = ACTIONS(2446),
    [anon_sym_LT] = ACTIONS(2446),
    [anon_sym_DOT_DOT] = ACTIONS(2446),
    [anon_sym_COLON_COLON] = ACTIONS(2446),
    [anon_sym_POUND] = ACTIONS(2446),
    [anon_sym_SQUOTE] = ACTIONS(2448),
    [anon_sym_async] = ACTIONS(2448),
    [anon_sym_auto] = ACTIONS(2448),
    [anon_sym_await] = ACTIONS(2448),
    [anon_sym_become] = ACTIONS(2448),
    [anon_sym_break] = ACTIONS(2448),
    [anon_sym_const] = ACTIONS(2448),
    [anon_sym_continue] = ACTIONS(2448),
    [anon_sym_default] = ACTIONS(2448),
    [anon_sym_enum] = ACTIONS(2448),
    [anon_sym_fn] = ACTIONS(2448),
    [anon_sym_for] = ACTIONS(2448),
    [anon_sym_gen] = ACTIONS(2448),
    [anon_sym_if] = ACTIONS(2448),
    [anon_sym_impl] = ACTIONS(2448),
    [anon_sym_let] = ACTIONS(2448),
    [anon_sym_loop] = ACTIONS(2448),
    [anon_sym_match] = ACTIONS(2448),
    [anon_sym_mod] = ACTIONS(2448),
    [anon_sym_pub] = ACTIONS(2448),
    [anon_sym_return] = ACTIONS(2448),
    [anon_sym_raw] = ACTIONS(2448),
    [anon_sym_static] = ACTIONS(2448),
    [anon_sym_struct] = ACTIONS(2448),
    [anon_sym_trait] = ACTIONS(2448),
    [anon_sym_type] = ACTIONS(2448),
    [anon_sym_union] = ACTIONS(2448),
    [anon_sym_unsafe] = ACTIONS(2448),
    [anon_sym_use] = ACTIONS(2448),
    [anon_sym_while] = ACTIONS(2448),
    [anon_sym_extern] = ACTIONS(2448),
    [anon_sym_safe] = ACTIONS(2448),
    [anon_sym_yield] = ACTIONS(2448),
    [anon_sym_move] = ACTIONS(2448),
    [anon_sym_try] = ACTIONS(2448),
    [sym_integer_literal] = ACTIONS(2446),
    [aux_sym_string_literal_token1] = ACTIONS(2446),
    [sym_char_literal] = ACTIONS(2446),
    [anon_sym_true] = ACTIONS(2448),
    [anon_sym_false] = ACTIONS(2448),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2448),
    [sym_super] = ACTIONS(2448),
    [sym_crate] = ACTIONS(2448),
    [aux_sym_metavariable_token1] = ACTIONS(2446),
    [sym__raw_string_literal_start] = ACTIONS(2446),
    [sym_float_literal] = ACTIONS(2446),
    [sym__outer_block_doc_comment_start] = ACTIONS(2446),
    [sym__outer_line_doc_comment_start] = ACTIONS(2446),
  },
  [STATE(669)] = {
    [sym_line_comment] = STATE(669),
    [sym_block_comment] = STATE(669),
    [ts_builtin_sym_end] = ACTIONS(2450),
    [sym_identifier] = ACTIONS(2452),
    [anon_sym_SEMI] = ACTIONS(2450),
    [anon_sym_macro] = ACTIONS(2452),
    [anon_sym_LPAREN] = ACTIONS(2450),
    [anon_sym_LBRACE] = ACTIONS(2450),
    [anon_sym_RBRACE] = ACTIONS(2450),
    [anon_sym_macro_rules] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2450),
    [anon_sym_LBRACK] = ACTIONS(2450),
    [anon_sym_STAR] = ACTIONS(2450),
    [anon_sym_u8] = ACTIONS(2452),
    [anon_sym_i8] = ACTIONS(2452),
    [anon_sym_u16] = ACTIONS(2452),
    [anon_sym_i16] = ACTIONS(2452),
    [anon_sym_u32] = ACTIONS(2452),
    [anon_sym_i32] = ACTIONS(2452),
    [anon_sym_u64] = ACTIONS(2452),
    [anon_sym_i64] = ACTIONS(2452),
    [anon_sym_u128] = ACTIONS(2452),
    [anon_sym_i128] = ACTIONS(2452),
    [anon_sym_isize] = ACTIONS(2452),
    [anon_sym_usize] = ACTIONS(2452),
    [anon_sym_f32] = ACTIONS(2452),
    [anon_sym_f64] = ACTIONS(2452),
    [anon_sym_bool] = ACTIONS(2452),
    [anon_sym_str] = ACTIONS(2452),
    [anon_sym_char] = ACTIONS(2452),
    [anon_sym_DASH] = ACTIONS(2450),
    [anon_sym_AMP] = ACTIONS(2450),
    [anon_sym_PIPE] = ACTIONS(2450),
    [anon_sym_LT] = ACTIONS(2450),
    [anon_sym_DOT_DOT] = ACTIONS(2450),
    [anon_sym_COLON_COLON] = ACTIONS(2450),
    [anon_sym_POUND] = ACTIONS(2450),
    [anon_sym_SQUOTE] = ACTIONS(2452),
    [anon_sym_async] = ACTIONS(2452),
    [anon_sym_auto] = ACTIONS(2452),
    [anon_sym_await] = ACTIONS(2452),
    [anon_sym_become] = ACTIONS(2452),
    [anon_sym_break] = ACTIONS(2452),
    [anon_sym_const] = ACTIONS(2452),
    [anon_sym_continue] = ACTIONS(2452),
    [anon_sym_default] = ACTIONS(2452),
    [anon_sym_enum] = ACTIONS(2452),
    [anon_sym_fn] = ACTIONS(2452),
    [anon_sym_for] = ACTIONS(2452),
    [anon_sym_gen] = ACTIONS(2452),
    [anon_sym_if] = ACTIONS(2452),
    [anon_sym_impl] = ACTIONS(2452),
    [anon_sym_let] = ACTIONS(2452),
    [anon_sym_loop] = ACTIONS(2452),
    [anon_sym_match] = ACTIONS(2452),
    [anon_sym_mod] = ACTIONS(2452),
    [anon_sym_pub] = ACTIONS(2452),
    [anon_sym_return] = ACTIONS(2452),
    [anon_sym_raw] = ACTIONS(2452),
    [anon_sym_static] = ACTIONS(2452),
    [anon_sym_struct] = ACTIONS(2452),
    [anon_sym_trait] = ACTIONS(2452),
    [anon_sym_type] = ACTIONS(2452),
    [anon_sym_union] = ACTIONS(2452),
    [anon_sym_unsafe] = ACTIONS(2452),
    [anon_sym_use] = ACTIONS(2452),
    [anon_sym_while] = ACTIONS(2452),
    [anon_sym_extern] = ACTIONS(2452),
    [anon_sym_safe] = ACTIONS(2452),
    [anon_sym_yield] = ACTIONS(2452),
    [anon_sym_move] = ACTIONS(2452),
    [anon_sym_try] = ACTIONS(2452),
    [sym_integer_literal] = ACTIONS(2450),
    [aux_sym_string_literal_token1] = ACTIONS(2450),
    [sym_char_literal] = ACTIONS(2450),
    [anon_sym_true] = ACTIONS(2452),
    [anon_sym_false] = ACTIONS(2452),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2452),
    [sym_super] = ACTIONS(2452),
    [sym_crate] = ACTIONS(2452),
    [aux_sym_metavariable_token1] = ACTIONS(2450),
    [sym__raw_string_literal_start] = ACTIONS(2450),
    [sym_float_literal] = ACTIONS(2450),
    [sym__outer_block_doc_comment_start] = ACTIONS(2450),
    [sym__outer_line_doc_comment_start] = ACTIONS(2450),
  },
  [STATE(670)] = {
    [sym_line_comment] = STATE(670),
    [sym_block_comment] = STATE(670),
    [ts_builtin_sym_end] = ACTIONS(2454),
    [sym_identifier] = ACTIONS(2456),
    [anon_sym_SEMI] = ACTIONS(2454),
    [anon_sym_macro] = ACTIONS(2456),
    [anon_sym_LPAREN] = ACTIONS(2454),
    [anon_sym_LBRACE] = ACTIONS(2454),
    [anon_sym_RBRACE] = ACTIONS(2454),
    [anon_sym_macro_rules] = ACTIONS(2456),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_LBRACK] = ACTIONS(2454),
    [anon_sym_STAR] = ACTIONS(2454),
    [anon_sym_u8] = ACTIONS(2456),
    [anon_sym_i8] = ACTIONS(2456),
    [anon_sym_u16] = ACTIONS(2456),
    [anon_sym_i16] = ACTIONS(2456),
    [anon_sym_u32] = ACTIONS(2456),
    [anon_sym_i32] = ACTIONS(2456),
    [anon_sym_u64] = ACTIONS(2456),
    [anon_sym_i64] = ACTIONS(2456),
    [anon_sym_u128] = ACTIONS(2456),
    [anon_sym_i128] = ACTIONS(2456),
    [anon_sym_isize] = ACTIONS(2456),
    [anon_sym_usize] = ACTIONS(2456),
    [anon_sym_f32] = ACTIONS(2456),
    [anon_sym_f64] = ACTIONS(2456),
    [anon_sym_bool] = ACTIONS(2456),
    [anon_sym_str] = ACTIONS(2456),
    [anon_sym_char] = ACTIONS(2456),
    [anon_sym_DASH] = ACTIONS(2454),
    [anon_sym_AMP] = ACTIONS(2454),
    [anon_sym_PIPE] = ACTIONS(2454),
    [anon_sym_LT] = ACTIONS(2454),
    [anon_sym_DOT_DOT] = ACTIONS(2454),
    [anon_sym_COLON_COLON] = ACTIONS(2454),
    [anon_sym_POUND] = ACTIONS(2454),
    [anon_sym_SQUOTE] = ACTIONS(2456),
    [anon_sym_async] = ACTIONS(2456),
    [anon_sym_auto] = ACTIONS(2456),
    [anon_sym_await] = ACTIONS(2456),
    [anon_sym_become] = ACTIONS(2456),
    [anon_sym_break] = ACTIONS(2456),
    [anon_sym_const] = ACTIONS(2456),
    [anon_sym_continue] = ACTIONS(2456),
    [anon_sym_default] = ACTIONS(2456),
    [anon_sym_enum] = ACTIONS(2456),
    [anon_sym_fn] = ACTIONS(2456),
    [anon_sym_for] = ACTIONS(2456),
    [anon_sym_gen] = ACTIONS(2456),
    [anon_sym_if] = ACTIONS(2456),
    [anon_sym_impl] = ACTIONS(2456),
    [anon_sym_let] = ACTIONS(2456),
    [anon_sym_loop] = ACTIONS(2456),
    [anon_sym_match] = ACTIONS(2456),
    [anon_sym_mod] = ACTIONS(2456),
    [anon_sym_pub] = ACTIONS(2456),
    [anon_sym_return] = ACTIONS(2456),
    [anon_sym_raw] = ACTIONS(2456),
    [anon_sym_static] = ACTIONS(2456),
    [anon_sym_struct] = ACTIONS(2456),
    [anon_sym_trait] = ACTIONS(2456),
    [anon_sym_type] = ACTIONS(2456),
    [anon_sym_union] = ACTIONS(2456),
    [anon_sym_unsafe] = ACTIONS(2456),
    [anon_sym_use] = ACTIONS(2456),
    [anon_sym_while] = ACTIONS(2456),
    [anon_sym_extern] = ACTIONS(2456),
    [anon_sym_safe] = ACTIONS(2456),
    [anon_sym_yield] = ACTIONS(2456),
    [anon_sym_move] = ACTIONS(2456),
    [anon_sym_try] = ACTIONS(2456),
    [sym_integer_literal] = ACTIONS(2454),
    [aux_sym_string_literal_token1] = ACTIONS(2454),
    [sym_char_literal] = ACTIONS(2454),
    [anon_sym_true] = ACTIONS(2456),
    [anon_sym_false] = ACTIONS(2456),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2456),
    [sym_super] = ACTIONS(2456),
    [sym_crate] = ACTIONS(2456),
    [aux_sym_metavariable_token1] = ACTIONS(2454),
    [sym__raw_string_literal_start] = ACTIONS(2454),
    [sym_float_literal] = ACTIONS(2454),
    [sym__outer_block_doc_comment_start] = ACTIONS(2454),
    [sym__outer_line_doc_comment_start] = ACTIONS(2454),
  },
  [STATE(671)] = {
    [sym_line_comment] = STATE(671),
    [sym_block_comment] = STATE(671),
    [ts_builtin_sym_end] = ACTIONS(2458),
    [sym_identifier] = ACTIONS(2460),
    [anon_sym_SEMI] = ACTIONS(2458),
    [anon_sym_macro] = ACTIONS(2460),
    [anon_sym_LPAREN] = ACTIONS(2458),
    [anon_sym_LBRACE] = ACTIONS(2458),
    [anon_sym_RBRACE] = ACTIONS(2458),
    [anon_sym_macro_rules] = ACTIONS(2460),
    [anon_sym_BANG] = ACTIONS(2458),
    [anon_sym_LBRACK] = ACTIONS(2458),
    [anon_sym_STAR] = ACTIONS(2458),
    [anon_sym_u8] = ACTIONS(2460),
    [anon_sym_i8] = ACTIONS(2460),
    [anon_sym_u16] = ACTIONS(2460),
    [anon_sym_i16] = ACTIONS(2460),
    [anon_sym_u32] = ACTIONS(2460),
    [anon_sym_i32] = ACTIONS(2460),
    [anon_sym_u64] = ACTIONS(2460),
    [anon_sym_i64] = ACTIONS(2460),
    [anon_sym_u128] = ACTIONS(2460),
    [anon_sym_i128] = ACTIONS(2460),
    [anon_sym_isize] = ACTIONS(2460),
    [anon_sym_usize] = ACTIONS(2460),
    [anon_sym_f32] = ACTIONS(2460),
    [anon_sym_f64] = ACTIONS(2460),
    [anon_sym_bool] = ACTIONS(2460),
    [anon_sym_str] = ACTIONS(2460),
    [anon_sym_char] = ACTIONS(2460),
    [anon_sym_DASH] = ACTIONS(2458),
    [anon_sym_AMP] = ACTIONS(2458),
    [anon_sym_PIPE] = ACTIONS(2458),
    [anon_sym_LT] = ACTIONS(2458),
    [anon_sym_DOT_DOT] = ACTIONS(2458),
    [anon_sym_COLON_COLON] = ACTIONS(2458),
    [anon_sym_POUND] = ACTIONS(2458),
    [anon_sym_SQUOTE] = ACTIONS(2460),
    [anon_sym_async] = ACTIONS(2460),
    [anon_sym_auto] = ACTIONS(2460),
    [anon_sym_await] = ACTIONS(2460),
    [anon_sym_become] = ACTIONS(2460),
    [anon_sym_break] = ACTIONS(2460),
    [anon_sym_const] = ACTIONS(2460),
    [anon_sym_continue] = ACTIONS(2460),
    [anon_sym_default] = ACTIONS(2460),
    [anon_sym_enum] = ACTIONS(2460),
    [anon_sym_fn] = ACTIONS(2460),
    [anon_sym_for] = ACTIONS(2460),
    [anon_sym_gen] = ACTIONS(2460),
    [anon_sym_if] = ACTIONS(2460),
    [anon_sym_impl] = ACTIONS(2460),
    [anon_sym_let] = ACTIONS(2460),
    [anon_sym_loop] = ACTIONS(2460),
    [anon_sym_match] = ACTIONS(2460),
    [anon_sym_mod] = ACTIONS(2460),
    [anon_sym_pub] = ACTIONS(2460),
    [anon_sym_return] = ACTIONS(2460),
    [anon_sym_raw] = ACTIONS(2460),
    [anon_sym_static] = ACTIONS(2460),
    [anon_sym_struct] = ACTIONS(2460),
    [anon_sym_trait] = ACTIONS(2460),
    [anon_sym_type] = ACTIONS(2460),
    [anon_sym_union] = ACTIONS(2460),
    [anon_sym_unsafe] = ACTIONS(2460),
    [anon_sym_use] = ACTIONS(2460),
    [anon_sym_while] = ACTIONS(2460),
    [anon_sym_extern] = ACTIONS(2460),
    [anon_sym_safe] = ACTIONS(2460),
    [anon_sym_yield] = ACTIONS(2460),
    [anon_sym_move] = ACTIONS(2460),
    [anon_sym_try] = ACTIONS(2460),
    [sym_integer_literal] = ACTIONS(2458),
    [aux_sym_string_literal_token1] = ACTIONS(2458),
    [sym_char_literal] = ACTIONS(2458),
    [anon_sym_true] = ACTIONS(2460),
    [anon_sym_false] = ACTIONS(2460),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2460),
    [sym_super] = ACTIONS(2460),
    [sym_crate] = ACTIONS(2460),
    [aux_sym_metavariable_token1] = ACTIONS(2458),
    [sym__raw_string_literal_start] = ACTIONS(2458),
    [sym_float_literal] = ACTIONS(2458),
    [sym__outer_block_doc_comment_start] = ACTIONS(2458),
    [sym__outer_line_doc_comment_start] = ACTIONS(2458),
  },
  [STATE(672)] = {
    [sym_line_comment] = STATE(672),
    [sym_block_comment] = STATE(672),
    [ts_builtin_sym_end] = ACTIONS(2462),
    [sym_identifier] = ACTIONS(2464),
    [anon_sym_SEMI] = ACTIONS(2462),
    [anon_sym_macro] = ACTIONS(2464),
    [anon_sym_LPAREN] = ACTIONS(2462),
    [anon_sym_LBRACE] = ACTIONS(2462),
    [anon_sym_RBRACE] = ACTIONS(2462),
    [anon_sym_macro_rules] = ACTIONS(2464),
    [anon_sym_BANG] = ACTIONS(2462),
    [anon_sym_LBRACK] = ACTIONS(2462),
    [anon_sym_STAR] = ACTIONS(2462),
    [anon_sym_u8] = ACTIONS(2464),
    [anon_sym_i8] = ACTIONS(2464),
    [anon_sym_u16] = ACTIONS(2464),
    [anon_sym_i16] = ACTIONS(2464),
    [anon_sym_u32] = ACTIONS(2464),
    [anon_sym_i32] = ACTIONS(2464),
    [anon_sym_u64] = ACTIONS(2464),
    [anon_sym_i64] = ACTIONS(2464),
    [anon_sym_u128] = ACTIONS(2464),
    [anon_sym_i128] = ACTIONS(2464),
    [anon_sym_isize] = ACTIONS(2464),
    [anon_sym_usize] = ACTIONS(2464),
    [anon_sym_f32] = ACTIONS(2464),
    [anon_sym_f64] = ACTIONS(2464),
    [anon_sym_bool] = ACTIONS(2464),
    [anon_sym_str] = ACTIONS(2464),
    [anon_sym_char] = ACTIONS(2464),
    [anon_sym_DASH] = ACTIONS(2462),
    [anon_sym_AMP] = ACTIONS(2462),
    [anon_sym_PIPE] = ACTIONS(2462),
    [anon_sym_LT] = ACTIONS(2462),
    [anon_sym_DOT_DOT] = ACTIONS(2462),
    [anon_sym_COLON_COLON] = ACTIONS(2462),
    [anon_sym_POUND] = ACTIONS(2462),
    [anon_sym_SQUOTE] = ACTIONS(2464),
    [anon_sym_async] = ACTIONS(2464),
    [anon_sym_auto] = ACTIONS(2464),
    [anon_sym_await] = ACTIONS(2464),
    [anon_sym_become] = ACTIONS(2464),
    [anon_sym_break] = ACTIONS(2464),
    [anon_sym_const] = ACTIONS(2464),
    [anon_sym_continue] = ACTIONS(2464),
    [anon_sym_default] = ACTIONS(2464),
    [anon_sym_enum] = ACTIONS(2464),
    [anon_sym_fn] = ACTIONS(2464),
    [anon_sym_for] = ACTIONS(2464),
    [anon_sym_gen] = ACTIONS(2464),
    [anon_sym_if] = ACTIONS(2464),
    [anon_sym_impl] = ACTIONS(2464),
    [anon_sym_let] = ACTIONS(2464),
    [anon_sym_loop] = ACTIONS(2464),
    [anon_sym_match] = ACTIONS(2464),
    [anon_sym_mod] = ACTIONS(2464),
    [anon_sym_pub] = ACTIONS(2464),
    [anon_sym_return] = ACTIONS(2464),
    [anon_sym_raw] = ACTIONS(2464),
    [anon_sym_static] = ACTIONS(2464),
    [anon_sym_struct] = ACTIONS(2464),
    [anon_sym_trait] = ACTIONS(2464),
    [anon_sym_type] = ACTIONS(2464),
    [anon_sym_union] = ACTIONS(2464),
    [anon_sym_unsafe] = ACTIONS(2464),
    [anon_sym_use] = ACTIONS(2464),
    [anon_sym_while] = ACTIONS(2464),
    [anon_sym_extern] = ACTIONS(2464),
    [anon_sym_safe] = ACTIONS(2464),
    [anon_sym_yield] = ACTIONS(2464),
    [anon_sym_move] = ACTIONS(2464),
    [anon_sym_try] = ACTIONS(2464),
    [sym_integer_literal] = ACTIONS(2462),
    [aux_sym_string_literal_token1] = ACTIONS(2462),
    [sym_char_literal] = ACTIONS(2462),
    [anon_sym_true] = ACTIONS(2464),
    [anon_sym_false] = ACTIONS(2464),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2464),
    [sym_super] = ACTIONS(2464),
    [sym_crate] = ACTIONS(2464),
    [aux_sym_metavariable_token1] = ACTIONS(2462),
    [sym__raw_string_literal_start] = ACTIONS(2462),
    [sym_float_literal] = ACTIONS(2462),
    [sym__outer_block_doc_comment_start] = ACTIONS(2462),
    [sym__outer_line_doc_comment_start] = ACTIONS(2462),
  },
  [STATE(673)] = {
    [sym_line_comment] = STATE(673),
    [sym_block_comment] = STATE(673),
    [ts_builtin_sym_end] = ACTIONS(2466),
    [sym_identifier] = ACTIONS(2468),
    [anon_sym_SEMI] = ACTIONS(2466),
    [anon_sym_macro] = ACTIONS(2468),
    [anon_sym_LPAREN] = ACTIONS(2466),
    [anon_sym_LBRACE] = ACTIONS(2466),
    [anon_sym_RBRACE] = ACTIONS(2466),
    [anon_sym_macro_rules] = ACTIONS(2468),
    [anon_sym_BANG] = ACTIONS(2466),
    [anon_sym_LBRACK] = ACTIONS(2466),
    [anon_sym_STAR] = ACTIONS(2466),
    [anon_sym_u8] = ACTIONS(2468),
    [anon_sym_i8] = ACTIONS(2468),
    [anon_sym_u16] = ACTIONS(2468),
    [anon_sym_i16] = ACTIONS(2468),
    [anon_sym_u32] = ACTIONS(2468),
    [anon_sym_i32] = ACTIONS(2468),
    [anon_sym_u64] = ACTIONS(2468),
    [anon_sym_i64] = ACTIONS(2468),
    [anon_sym_u128] = ACTIONS(2468),
    [anon_sym_i128] = ACTIONS(2468),
    [anon_sym_isize] = ACTIONS(2468),
    [anon_sym_usize] = ACTIONS(2468),
    [anon_sym_f32] = ACTIONS(2468),
    [anon_sym_f64] = ACTIONS(2468),
    [anon_sym_bool] = ACTIONS(2468),
    [anon_sym_str] = ACTIONS(2468),
    [anon_sym_char] = ACTIONS(2468),
    [anon_sym_DASH] = ACTIONS(2466),
    [anon_sym_AMP] = ACTIONS(2466),
    [anon_sym_PIPE] = ACTIONS(2466),
    [anon_sym_LT] = ACTIONS(2466),
    [anon_sym_DOT_DOT] = ACTIONS(2466),
    [anon_sym_COLON_COLON] = ACTIONS(2466),
    [anon_sym_POUND] = ACTIONS(2466),
    [anon_sym_SQUOTE] = ACTIONS(2468),
    [anon_sym_async] = ACTIONS(2468),
    [anon_sym_auto] = ACTIONS(2468),
    [anon_sym_await] = ACTIONS(2468),
    [anon_sym_become] = ACTIONS(2468),
    [anon_sym_break] = ACTIONS(2468),
    [anon_sym_const] = ACTIONS(2468),
    [anon_sym_continue] = ACTIONS(2468),
    [anon_sym_default] = ACTIONS(2468),
    [anon_sym_enum] = ACTIONS(2468),
    [anon_sym_fn] = ACTIONS(2468),
    [anon_sym_for] = ACTIONS(2468),
    [anon_sym_gen] = ACTIONS(2468),
    [anon_sym_if] = ACTIONS(2468),
    [anon_sym_impl] = ACTIONS(2468),
    [anon_sym_let] = ACTIONS(2468),
    [anon_sym_loop] = ACTIONS(2468),
    [anon_sym_match] = ACTIONS(2468),
    [anon_sym_mod] = ACTIONS(2468),
    [anon_sym_pub] = ACTIONS(2468),
    [anon_sym_return] = ACTIONS(2468),
    [anon_sym_raw] = ACTIONS(2468),
    [anon_sym_static] = ACTIONS(2468),
    [anon_sym_struct] = ACTIONS(2468),
    [anon_sym_trait] = ACTIONS(2468),
    [anon_sym_type] = ACTIONS(2468),
    [anon_sym_union] = ACTIONS(2468),
    [anon_sym_unsafe] = ACTIONS(2468),
    [anon_sym_use] = ACTIONS(2468),
    [anon_sym_while] = ACTIONS(2468),
    [anon_sym_extern] = ACTIONS(2468),
    [anon_sym_safe] = ACTIONS(2468),
    [anon_sym_yield] = ACTIONS(2468),
    [anon_sym_move] = ACTIONS(2468),
    [anon_sym_try] = ACTIONS(2468),
    [sym_integer_literal] = ACTIONS(2466),
    [aux_sym_string_literal_token1] = ACTIONS(2466),
    [sym_char_literal] = ACTIONS(2466),
    [anon_sym_true] = ACTIONS(2468),
    [anon_sym_false] = ACTIONS(2468),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2468),
    [sym_super] = ACTIONS(2468),
    [sym_crate] = ACTIONS(2468),
    [aux_sym_metavariable_token1] = ACTIONS(2466),
    [sym__raw_string_literal_start] = ACTIONS(2466),
    [sym_float_literal] = ACTIONS(2466),
    [sym__outer_block_doc_comment_start] = ACTIONS(2466),
    [sym__outer_line_doc_comment_start] = ACTIONS(2466),
  },
  [STATE(674)] = {
    [sym_line_comment] = STATE(674),
    [sym_block_comment] = STATE(674),
    [ts_builtin_sym_end] = ACTIONS(2470),
    [sym_identifier] = ACTIONS(2472),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_macro] = ACTIONS(2472),
    [anon_sym_LPAREN] = ACTIONS(2470),
    [anon_sym_LBRACE] = ACTIONS(2470),
    [anon_sym_RBRACE] = ACTIONS(2470),
    [anon_sym_macro_rules] = ACTIONS(2472),
    [anon_sym_BANG] = ACTIONS(2470),
    [anon_sym_LBRACK] = ACTIONS(2470),
    [anon_sym_STAR] = ACTIONS(2470),
    [anon_sym_u8] = ACTIONS(2472),
    [anon_sym_i8] = ACTIONS(2472),
    [anon_sym_u16] = ACTIONS(2472),
    [anon_sym_i16] = ACTIONS(2472),
    [anon_sym_u32] = ACTIONS(2472),
    [anon_sym_i32] = ACTIONS(2472),
    [anon_sym_u64] = ACTIONS(2472),
    [anon_sym_i64] = ACTIONS(2472),
    [anon_sym_u128] = ACTIONS(2472),
    [anon_sym_i128] = ACTIONS(2472),
    [anon_sym_isize] = ACTIONS(2472),
    [anon_sym_usize] = ACTIONS(2472),
    [anon_sym_f32] = ACTIONS(2472),
    [anon_sym_f64] = ACTIONS(2472),
    [anon_sym_bool] = ACTIONS(2472),
    [anon_sym_str] = ACTIONS(2472),
    [anon_sym_char] = ACTIONS(2472),
    [anon_sym_DASH] = ACTIONS(2470),
    [anon_sym_AMP] = ACTIONS(2470),
    [anon_sym_PIPE] = ACTIONS(2470),
    [anon_sym_LT] = ACTIONS(2470),
    [anon_sym_DOT_DOT] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(2470),
    [anon_sym_POUND] = ACTIONS(2470),
    [anon_sym_SQUOTE] = ACTIONS(2472),
    [anon_sym_async] = ACTIONS(2472),
    [anon_sym_auto] = ACTIONS(2472),
    [anon_sym_await] = ACTIONS(2472),
    [anon_sym_become] = ACTIONS(2472),
    [anon_sym_break] = ACTIONS(2472),
    [anon_sym_const] = ACTIONS(2472),
    [anon_sym_continue] = ACTIONS(2472),
    [anon_sym_default] = ACTIONS(2472),
    [anon_sym_enum] = ACTIONS(2472),
    [anon_sym_fn] = ACTIONS(2472),
    [anon_sym_for] = ACTIONS(2472),
    [anon_sym_gen] = ACTIONS(2472),
    [anon_sym_if] = ACTIONS(2472),
    [anon_sym_impl] = ACTIONS(2472),
    [anon_sym_let] = ACTIONS(2472),
    [anon_sym_loop] = ACTIONS(2472),
    [anon_sym_match] = ACTIONS(2472),
    [anon_sym_mod] = ACTIONS(2472),
    [anon_sym_pub] = ACTIONS(2472),
    [anon_sym_return] = ACTIONS(2472),
    [anon_sym_raw] = ACTIONS(2472),
    [anon_sym_static] = ACTIONS(2472),
    [anon_sym_struct] = ACTIONS(2472),
    [anon_sym_trait] = ACTIONS(2472),
    [anon_sym_type] = ACTIONS(2472),
    [anon_sym_union] = ACTIONS(2472),
    [anon_sym_unsafe] = ACTIONS(2472),
    [anon_sym_use] = ACTIONS(2472),
    [anon_sym_while] = ACTIONS(2472),
    [anon_sym_extern] = ACTIONS(2472),
    [anon_sym_safe] = ACTIONS(2472),
    [anon_sym_yield] = ACTIONS(2472),
    [anon_sym_move] = ACTIONS(2472),
    [anon_sym_try] = ACTIONS(2472),
    [sym_integer_literal] = ACTIONS(2470),
    [aux_sym_string_literal_token1] = ACTIONS(2470),
    [sym_char_literal] = ACTIONS(2470),
    [anon_sym_true] = ACTIONS(2472),
    [anon_sym_false] = ACTIONS(2472),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2472),
    [sym_super] = ACTIONS(2472),
    [sym_crate] = ACTIONS(2472),
    [aux_sym_metavariable_token1] = ACTIONS(2470),
    [sym__raw_string_literal_start] = ACTIONS(2470),
    [sym_float_literal] = ACTIONS(2470),
    [sym__outer_block_doc_comment_start] = ACTIONS(2470),
    [sym__outer_line_doc_comment_start] = ACTIONS(2470),
  },
  [STATE(675)] = {
    [sym_line_comment] = STATE(675),
    [sym_block_comment] = STATE(675),
    [ts_builtin_sym_end] = ACTIONS(2474),
    [sym_identifier] = ACTIONS(2476),
    [anon_sym_SEMI] = ACTIONS(2474),
    [anon_sym_macro] = ACTIONS(2476),
    [anon_sym_LPAREN] = ACTIONS(2474),
    [anon_sym_LBRACE] = ACTIONS(2474),
    [anon_sym_RBRACE] = ACTIONS(2474),
    [anon_sym_macro_rules] = ACTIONS(2476),
    [anon_sym_BANG] = ACTIONS(2474),
    [anon_sym_LBRACK] = ACTIONS(2474),
    [anon_sym_STAR] = ACTIONS(2474),
    [anon_sym_u8] = ACTIONS(2476),
    [anon_sym_i8] = ACTIONS(2476),
    [anon_sym_u16] = ACTIONS(2476),
    [anon_sym_i16] = ACTIONS(2476),
    [anon_sym_u32] = ACTIONS(2476),
    [anon_sym_i32] = ACTIONS(2476),
    [anon_sym_u64] = ACTIONS(2476),
    [anon_sym_i64] = ACTIONS(2476),
    [anon_sym_u128] = ACTIONS(2476),
    [anon_sym_i128] = ACTIONS(2476),
    [anon_sym_isize] = ACTIONS(2476),
    [anon_sym_usize] = ACTIONS(2476),
    [anon_sym_f32] = ACTIONS(2476),
    [anon_sym_f64] = ACTIONS(2476),
    [anon_sym_bool] = ACTIONS(2476),
    [anon_sym_str] = ACTIONS(2476),
    [anon_sym_char] = ACTIONS(2476),
    [anon_sym_DASH] = ACTIONS(2474),
    [anon_sym_AMP] = ACTIONS(2474),
    [anon_sym_PIPE] = ACTIONS(2474),
    [anon_sym_LT] = ACTIONS(2474),
    [anon_sym_DOT_DOT] = ACTIONS(2474),
    [anon_sym_COLON_COLON] = ACTIONS(2474),
    [anon_sym_POUND] = ACTIONS(2474),
    [anon_sym_SQUOTE] = ACTIONS(2476),
    [anon_sym_async] = ACTIONS(2476),
    [anon_sym_auto] = ACTIONS(2476),
    [anon_sym_await] = ACTIONS(2476),
    [anon_sym_become] = ACTIONS(2476),
    [anon_sym_break] = ACTIONS(2476),
    [anon_sym_const] = ACTIONS(2476),
    [anon_sym_continue] = ACTIONS(2476),
    [anon_sym_default] = ACTIONS(2476),
    [anon_sym_enum] = ACTIONS(2476),
    [anon_sym_fn] = ACTIONS(2476),
    [anon_sym_for] = ACTIONS(2476),
    [anon_sym_gen] = ACTIONS(2476),
    [anon_sym_if] = ACTIONS(2476),
    [anon_sym_impl] = ACTIONS(2476),
    [anon_sym_let] = ACTIONS(2476),
    [anon_sym_loop] = ACTIONS(2476),
    [anon_sym_match] = ACTIONS(2476),
    [anon_sym_mod] = ACTIONS(2476),
    [anon_sym_pub] = ACTIONS(2476),
    [anon_sym_return] = ACTIONS(2476),
    [anon_sym_raw] = ACTIONS(2476),
    [anon_sym_static] = ACTIONS(2476),
    [anon_sym_struct] = ACTIONS(2476),
    [anon_sym_trait] = ACTIONS(2476),
    [anon_sym_type] = ACTIONS(2476),
    [anon_sym_union] = ACTIONS(2476),
    [anon_sym_unsafe] = ACTIONS(2476),
    [anon_sym_use] = ACTIONS(2476),
    [anon_sym_while] = ACTIONS(2476),
    [anon_sym_extern] = ACTIONS(2476),
    [anon_sym_safe] = ACTIONS(2476),
    [anon_sym_yield] = ACTIONS(2476),
    [anon_sym_move] = ACTIONS(2476),
    [anon_sym_try] = ACTIONS(2476),
    [sym_integer_literal] = ACTIONS(2474),
    [aux_sym_string_literal_token1] = ACTIONS(2474),
    [sym_char_literal] = ACTIONS(2474),
    [anon_sym_true] = ACTIONS(2476),
    [anon_sym_false] = ACTIONS(2476),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2476),
    [sym_super] = ACTIONS(2476),
    [sym_crate] = ACTIONS(2476),
    [aux_sym_metavariable_token1] = ACTIONS(2474),
    [sym__raw_string_literal_start] = ACTIONS(2474),
    [sym_float_literal] = ACTIONS(2474),
    [sym__outer_block_doc_comment_start] = ACTIONS(2474),
    [sym__outer_line_doc_comment_start] = ACTIONS(2474),
  },
  [STATE(676)] = {
    [sym_line_comment] = STATE(676),
    [sym_block_comment] = STATE(676),
    [ts_builtin_sym_end] = ACTIONS(2478),
    [sym_identifier] = ACTIONS(2480),
    [anon_sym_SEMI] = ACTIONS(2478),
    [anon_sym_macro] = ACTIONS(2480),
    [anon_sym_LPAREN] = ACTIONS(2478),
    [anon_sym_LBRACE] = ACTIONS(2478),
    [anon_sym_RBRACE] = ACTIONS(2478),
    [anon_sym_macro_rules] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2478),
    [anon_sym_LBRACK] = ACTIONS(2478),
    [anon_sym_STAR] = ACTIONS(2478),
    [anon_sym_u8] = ACTIONS(2480),
    [anon_sym_i8] = ACTIONS(2480),
    [anon_sym_u16] = ACTIONS(2480),
    [anon_sym_i16] = ACTIONS(2480),
    [anon_sym_u32] = ACTIONS(2480),
    [anon_sym_i32] = ACTIONS(2480),
    [anon_sym_u64] = ACTIONS(2480),
    [anon_sym_i64] = ACTIONS(2480),
    [anon_sym_u128] = ACTIONS(2480),
    [anon_sym_i128] = ACTIONS(2480),
    [anon_sym_isize] = ACTIONS(2480),
    [anon_sym_usize] = ACTIONS(2480),
    [anon_sym_f32] = ACTIONS(2480),
    [anon_sym_f64] = ACTIONS(2480),
    [anon_sym_bool] = ACTIONS(2480),
    [anon_sym_str] = ACTIONS(2480),
    [anon_sym_char] = ACTIONS(2480),
    [anon_sym_DASH] = ACTIONS(2478),
    [anon_sym_AMP] = ACTIONS(2478),
    [anon_sym_PIPE] = ACTIONS(2478),
    [anon_sym_LT] = ACTIONS(2478),
    [anon_sym_DOT_DOT] = ACTIONS(2478),
    [anon_sym_COLON_COLON] = ACTIONS(2478),
    [anon_sym_POUND] = ACTIONS(2478),
    [anon_sym_SQUOTE] = ACTIONS(2480),
    [anon_sym_async] = ACTIONS(2480),
    [anon_sym_auto] = ACTIONS(2480),
    [anon_sym_await] = ACTIONS(2480),
    [anon_sym_become] = ACTIONS(2480),
    [anon_sym_break] = ACTIONS(2480),
    [anon_sym_const] = ACTIONS(2480),
    [anon_sym_continue] = ACTIONS(2480),
    [anon_sym_default] = ACTIONS(2480),
    [anon_sym_enum] = ACTIONS(2480),
    [anon_sym_fn] = ACTIONS(2480),
    [anon_sym_for] = ACTIONS(2480),
    [anon_sym_gen] = ACTIONS(2480),
    [anon_sym_if] = ACTIONS(2480),
    [anon_sym_impl] = ACTIONS(2480),
    [anon_sym_let] = ACTIONS(2480),
    [anon_sym_loop] = ACTIONS(2480),
    [anon_sym_match] = ACTIONS(2480),
    [anon_sym_mod] = ACTIONS(2480),
    [anon_sym_pub] = ACTIONS(2480),
    [anon_sym_return] = ACTIONS(2480),
    [anon_sym_raw] = ACTIONS(2480),
    [anon_sym_static] = ACTIONS(2480),
    [anon_sym_struct] = ACTIONS(2480),
    [anon_sym_trait] = ACTIONS(2480),
    [anon_sym_type] = ACTIONS(2480),
    [anon_sym_union] = ACTIONS(2480),
    [anon_sym_unsafe] = ACTIONS(2480),
    [anon_sym_use] = ACTIONS(2480),
    [anon_sym_while] = ACTIONS(2480),
    [anon_sym_extern] = ACTIONS(2480),
    [anon_sym_safe] = ACTIONS(2480),
    [anon_sym_yield] = ACTIONS(2480),
    [anon_sym_move] = ACTIONS(2480),
    [anon_sym_try] = ACTIONS(2480),
    [sym_integer_literal] = ACTIONS(2478),
    [aux_sym_string_literal_token1] = ACTIONS(2478),
    [sym_char_literal] = ACTIONS(2478),
    [anon_sym_true] = ACTIONS(2480),
    [anon_sym_false] = ACTIONS(2480),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2480),
    [sym_super] = ACTIONS(2480),
    [sym_crate] = ACTIONS(2480),
    [aux_sym_metavariable_token1] = ACTIONS(2478),
    [sym__raw_string_literal_start] = ACTIONS(2478),
    [sym_float_literal] = ACTIONS(2478),
    [sym__outer_block_doc_comment_start] = ACTIONS(2478),
    [sym__outer_line_doc_comment_start] = ACTIONS(2478),
  },
  [STATE(677)] = {
    [sym_line_comment] = STATE(677),
    [sym_block_comment] = STATE(677),
    [ts_builtin_sym_end] = ACTIONS(2482),
    [sym_identifier] = ACTIONS(2484),
    [anon_sym_SEMI] = ACTIONS(2482),
    [anon_sym_macro] = ACTIONS(2484),
    [anon_sym_LPAREN] = ACTIONS(2482),
    [anon_sym_LBRACE] = ACTIONS(2482),
    [anon_sym_RBRACE] = ACTIONS(2482),
    [anon_sym_macro_rules] = ACTIONS(2484),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_LBRACK] = ACTIONS(2482),
    [anon_sym_STAR] = ACTIONS(2482),
    [anon_sym_u8] = ACTIONS(2484),
    [anon_sym_i8] = ACTIONS(2484),
    [anon_sym_u16] = ACTIONS(2484),
    [anon_sym_i16] = ACTIONS(2484),
    [anon_sym_u32] = ACTIONS(2484),
    [anon_sym_i32] = ACTIONS(2484),
    [anon_sym_u64] = ACTIONS(2484),
    [anon_sym_i64] = ACTIONS(2484),
    [anon_sym_u128] = ACTIONS(2484),
    [anon_sym_i128] = ACTIONS(2484),
    [anon_sym_isize] = ACTIONS(2484),
    [anon_sym_usize] = ACTIONS(2484),
    [anon_sym_f32] = ACTIONS(2484),
    [anon_sym_f64] = ACTIONS(2484),
    [anon_sym_bool] = ACTIONS(2484),
    [anon_sym_str] = ACTIONS(2484),
    [anon_sym_char] = ACTIONS(2484),
    [anon_sym_DASH] = ACTIONS(2482),
    [anon_sym_AMP] = ACTIONS(2482),
    [anon_sym_PIPE] = ACTIONS(2482),
    [anon_sym_LT] = ACTIONS(2482),
    [anon_sym_DOT_DOT] = ACTIONS(2482),
    [anon_sym_COLON_COLON] = ACTIONS(2482),
    [anon_sym_POUND] = ACTIONS(2482),
    [anon_sym_SQUOTE] = ACTIONS(2484),
    [anon_sym_async] = ACTIONS(2484),
    [anon_sym_auto] = ACTIONS(2484),
    [anon_sym_await] = ACTIONS(2484),
    [anon_sym_become] = ACTIONS(2484),
    [anon_sym_break] = ACTIONS(2484),
    [anon_sym_const] = ACTIONS(2484),
    [anon_sym_continue] = ACTIONS(2484),
    [anon_sym_default] = ACTIONS(2484),
    [anon_sym_enum] = ACTIONS(2484),
    [anon_sym_fn] = ACTIONS(2484),
    [anon_sym_for] = ACTIONS(2484),
    [anon_sym_gen] = ACTIONS(2484),
    [anon_sym_if] = ACTIONS(2484),
    [anon_sym_impl] = ACTIONS(2484),
    [anon_sym_let] = ACTIONS(2484),
    [anon_sym_loop] = ACTIONS(2484),
    [anon_sym_match] = ACTIONS(2484),
    [anon_sym_mod] = ACTIONS(2484),
    [anon_sym_pub] = ACTIONS(2484),
    [anon_sym_return] = ACTIONS(2484),
    [anon_sym_raw] = ACTIONS(2484),
    [anon_sym_static] = ACTIONS(2484),
    [anon_sym_struct] = ACTIONS(2484),
    [anon_sym_trait] = ACTIONS(2484),
    [anon_sym_type] = ACTIONS(2484),
    [anon_sym_union] = ACTIONS(2484),
    [anon_sym_unsafe] = ACTIONS(2484),
    [anon_sym_use] = ACTIONS(2484),
    [anon_sym_while] = ACTIONS(2484),
    [anon_sym_extern] = ACTIONS(2484),
    [anon_sym_safe] = ACTIONS(2484),
    [anon_sym_yield] = ACTIONS(2484),
    [anon_sym_move] = ACTIONS(2484),
    [anon_sym_try] = ACTIONS(2484),
    [sym_integer_literal] = ACTIONS(2482),
    [aux_sym_string_literal_token1] = ACTIONS(2482),
    [sym_char_literal] = ACTIONS(2482),
    [anon_sym_true] = ACTIONS(2484),
    [anon_sym_false] = ACTIONS(2484),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2484),
    [sym_super] = ACTIONS(2484),
    [sym_crate] = ACTIONS(2484),
    [aux_sym_metavariable_token1] = ACTIONS(2482),
    [sym__raw_string_literal_start] = ACTIONS(2482),
    [sym_float_literal] = ACTIONS(2482),
    [sym__outer_block_doc_comment_start] = ACTIONS(2482),
    [sym__outer_line_doc_comment_start] = ACTIONS(2482),
  },
  [STATE(678)] = {
    [sym_line_comment] = STATE(678),
    [sym_block_comment] = STATE(678),
    [ts_builtin_sym_end] = ACTIONS(2486),
    [sym_identifier] = ACTIONS(2488),
    [anon_sym_SEMI] = ACTIONS(2486),
    [anon_sym_macro] = ACTIONS(2488),
    [anon_sym_LPAREN] = ACTIONS(2486),
    [anon_sym_LBRACE] = ACTIONS(2486),
    [anon_sym_RBRACE] = ACTIONS(2486),
    [anon_sym_macro_rules] = ACTIONS(2488),
    [anon_sym_BANG] = ACTIONS(2486),
    [anon_sym_LBRACK] = ACTIONS(2486),
    [anon_sym_STAR] = ACTIONS(2486),
    [anon_sym_u8] = ACTIONS(2488),
    [anon_sym_i8] = ACTIONS(2488),
    [anon_sym_u16] = ACTIONS(2488),
    [anon_sym_i16] = ACTIONS(2488),
    [anon_sym_u32] = ACTIONS(2488),
    [anon_sym_i32] = ACTIONS(2488),
    [anon_sym_u64] = ACTIONS(2488),
    [anon_sym_i64] = ACTIONS(2488),
    [anon_sym_u128] = ACTIONS(2488),
    [anon_sym_i128] = ACTIONS(2488),
    [anon_sym_isize] = ACTIONS(2488),
    [anon_sym_usize] = ACTIONS(2488),
    [anon_sym_f32] = ACTIONS(2488),
    [anon_sym_f64] = ACTIONS(2488),
    [anon_sym_bool] = ACTIONS(2488),
    [anon_sym_str] = ACTIONS(2488),
    [anon_sym_char] = ACTIONS(2488),
    [anon_sym_DASH] = ACTIONS(2486),
    [anon_sym_AMP] = ACTIONS(2486),
    [anon_sym_PIPE] = ACTIONS(2486),
    [anon_sym_LT] = ACTIONS(2486),
    [anon_sym_DOT_DOT] = ACTIONS(2486),
    [anon_sym_COLON_COLON] = ACTIONS(2486),
    [anon_sym_POUND] = ACTIONS(2486),
    [anon_sym_SQUOTE] = ACTIONS(2488),
    [anon_sym_async] = ACTIONS(2488),
    [anon_sym_auto] = ACTIONS(2488),
    [anon_sym_await] = ACTIONS(2488),
    [anon_sym_become] = ACTIONS(2488),
    [anon_sym_break] = ACTIONS(2488),
    [anon_sym_const] = ACTIONS(2488),
    [anon_sym_continue] = ACTIONS(2488),
    [anon_sym_default] = ACTIONS(2488),
    [anon_sym_enum] = ACTIONS(2488),
    [anon_sym_fn] = ACTIONS(2488),
    [anon_sym_for] = ACTIONS(2488),
    [anon_sym_gen] = ACTIONS(2488),
    [anon_sym_if] = ACTIONS(2488),
    [anon_sym_impl] = ACTIONS(2488),
    [anon_sym_let] = ACTIONS(2488),
    [anon_sym_loop] = ACTIONS(2488),
    [anon_sym_match] = ACTIONS(2488),
    [anon_sym_mod] = ACTIONS(2488),
    [anon_sym_pub] = ACTIONS(2488),
    [anon_sym_return] = ACTIONS(2488),
    [anon_sym_raw] = ACTIONS(2488),
    [anon_sym_static] = ACTIONS(2488),
    [anon_sym_struct] = ACTIONS(2488),
    [anon_sym_trait] = ACTIONS(2488),
    [anon_sym_type] = ACTIONS(2488),
    [anon_sym_union] = ACTIONS(2488),
    [anon_sym_unsafe] = ACTIONS(2488),
    [anon_sym_use] = ACTIONS(2488),
    [anon_sym_while] = ACTIONS(2488),
    [anon_sym_extern] = ACTIONS(2488),
    [anon_sym_safe] = ACTIONS(2488),
    [anon_sym_yield] = ACTIONS(2488),
    [anon_sym_move] = ACTIONS(2488),
    [anon_sym_try] = ACTIONS(2488),
    [sym_integer_literal] = ACTIONS(2486),
    [aux_sym_string_literal_token1] = ACTIONS(2486),
    [sym_char_literal] = ACTIONS(2486),
    [anon_sym_true] = ACTIONS(2488),
    [anon_sym_false] = ACTIONS(2488),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2488),
    [sym_super] = ACTIONS(2488),
    [sym_crate] = ACTIONS(2488),
    [aux_sym_metavariable_token1] = ACTIONS(2486),
    [sym__raw_string_literal_start] = ACTIONS(2486),
    [sym_float_literal] = ACTIONS(2486),
    [sym__outer_block_doc_comment_start] = ACTIONS(2486),
    [sym__outer_line_doc_comment_start] = ACTIONS(2486),
  },
  [STATE(679)] = {
    [sym_line_comment] = STATE(679),
    [sym_block_comment] = STATE(679),
    [ts_builtin_sym_end] = ACTIONS(2490),
    [sym_identifier] = ACTIONS(2492),
    [anon_sym_SEMI] = ACTIONS(2490),
    [anon_sym_macro] = ACTIONS(2492),
    [anon_sym_LPAREN] = ACTIONS(2490),
    [anon_sym_LBRACE] = ACTIONS(2490),
    [anon_sym_RBRACE] = ACTIONS(2490),
    [anon_sym_macro_rules] = ACTIONS(2492),
    [anon_sym_BANG] = ACTIONS(2490),
    [anon_sym_LBRACK] = ACTIONS(2490),
    [anon_sym_STAR] = ACTIONS(2490),
    [anon_sym_u8] = ACTIONS(2492),
    [anon_sym_i8] = ACTIONS(2492),
    [anon_sym_u16] = ACTIONS(2492),
    [anon_sym_i16] = ACTIONS(2492),
    [anon_sym_u32] = ACTIONS(2492),
    [anon_sym_i32] = ACTIONS(2492),
    [anon_sym_u64] = ACTIONS(2492),
    [anon_sym_i64] = ACTIONS(2492),
    [anon_sym_u128] = ACTIONS(2492),
    [anon_sym_i128] = ACTIONS(2492),
    [anon_sym_isize] = ACTIONS(2492),
    [anon_sym_usize] = ACTIONS(2492),
    [anon_sym_f32] = ACTIONS(2492),
    [anon_sym_f64] = ACTIONS(2492),
    [anon_sym_bool] = ACTIONS(2492),
    [anon_sym_str] = ACTIONS(2492),
    [anon_sym_char] = ACTIONS(2492),
    [anon_sym_DASH] = ACTIONS(2490),
    [anon_sym_AMP] = ACTIONS(2490),
    [anon_sym_PIPE] = ACTIONS(2490),
    [anon_sym_LT] = ACTIONS(2490),
    [anon_sym_DOT_DOT] = ACTIONS(2490),
    [anon_sym_COLON_COLON] = ACTIONS(2490),
    [anon_sym_POUND] = ACTIONS(2490),
    [anon_sym_SQUOTE] = ACTIONS(2492),
    [anon_sym_async] = ACTIONS(2492),
    [anon_sym_auto] = ACTIONS(2492),
    [anon_sym_await] = ACTIONS(2492),
    [anon_sym_become] = ACTIONS(2492),
    [anon_sym_break] = ACTIONS(2492),
    [anon_sym_const] = ACTIONS(2492),
    [anon_sym_continue] = ACTIONS(2492),
    [anon_sym_default] = ACTIONS(2492),
    [anon_sym_enum] = ACTIONS(2492),
    [anon_sym_fn] = ACTIONS(2492),
    [anon_sym_for] = ACTIONS(2492),
    [anon_sym_gen] = ACTIONS(2492),
    [anon_sym_if] = ACTIONS(2492),
    [anon_sym_impl] = ACTIONS(2492),
    [anon_sym_let] = ACTIONS(2492),
    [anon_sym_loop] = ACTIONS(2492),
    [anon_sym_match] = ACTIONS(2492),
    [anon_sym_mod] = ACTIONS(2492),
    [anon_sym_pub] = ACTIONS(2492),
    [anon_sym_return] = ACTIONS(2492),
    [anon_sym_raw] = ACTIONS(2492),
    [anon_sym_static] = ACTIONS(2492),
    [anon_sym_struct] = ACTIONS(2492),
    [anon_sym_trait] = ACTIONS(2492),
    [anon_sym_type] = ACTIONS(2492),
    [anon_sym_union] = ACTIONS(2492),
    [anon_sym_unsafe] = ACTIONS(2492),
    [anon_sym_use] = ACTIONS(2492),
    [anon_sym_while] = ACTIONS(2492),
    [anon_sym_extern] = ACTIONS(2492),
    [anon_sym_safe] = ACTIONS(2492),
    [anon_sym_yield] = ACTIONS(2492),
    [anon_sym_move] = ACTIONS(2492),
    [anon_sym_try] = ACTIONS(2492),
    [sym_integer_literal] = ACTIONS(2490),
    [aux_sym_string_literal_token1] = ACTIONS(2490),
    [sym_char_literal] = ACTIONS(2490),
    [anon_sym_true] = ACTIONS(2492),
    [anon_sym_false] = ACTIONS(2492),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2492),
    [sym_super] = ACTIONS(2492),
    [sym_crate] = ACTIONS(2492),
    [aux_sym_metavariable_token1] = ACTIONS(2490),
    [sym__raw_string_literal_start] = ACTIONS(2490),
    [sym_float_literal] = ACTIONS(2490),
    [sym__outer_block_doc_comment_start] = ACTIONS(2490),
    [sym__outer_line_doc_comment_start] = ACTIONS(2490),
  },
  [STATE(680)] = {
    [sym_line_comment] = STATE(680),
    [sym_block_comment] = STATE(680),
    [ts_builtin_sym_end] = ACTIONS(2494),
    [sym_identifier] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2494),
    [anon_sym_macro] = ACTIONS(2496),
    [anon_sym_LPAREN] = ACTIONS(2494),
    [anon_sym_LBRACE] = ACTIONS(2494),
    [anon_sym_RBRACE] = ACTIONS(2494),
    [anon_sym_macro_rules] = ACTIONS(2496),
    [anon_sym_BANG] = ACTIONS(2494),
    [anon_sym_LBRACK] = ACTIONS(2494),
    [anon_sym_STAR] = ACTIONS(2494),
    [anon_sym_u8] = ACTIONS(2496),
    [anon_sym_i8] = ACTIONS(2496),
    [anon_sym_u16] = ACTIONS(2496),
    [anon_sym_i16] = ACTIONS(2496),
    [anon_sym_u32] = ACTIONS(2496),
    [anon_sym_i32] = ACTIONS(2496),
    [anon_sym_u64] = ACTIONS(2496),
    [anon_sym_i64] = ACTIONS(2496),
    [anon_sym_u128] = ACTIONS(2496),
    [anon_sym_i128] = ACTIONS(2496),
    [anon_sym_isize] = ACTIONS(2496),
    [anon_sym_usize] = ACTIONS(2496),
    [anon_sym_f32] = ACTIONS(2496),
    [anon_sym_f64] = ACTIONS(2496),
    [anon_sym_bool] = ACTIONS(2496),
    [anon_sym_str] = ACTIONS(2496),
    [anon_sym_char] = ACTIONS(2496),
    [anon_sym_DASH] = ACTIONS(2494),
    [anon_sym_AMP] = ACTIONS(2494),
    [anon_sym_PIPE] = ACTIONS(2494),
    [anon_sym_LT] = ACTIONS(2494),
    [anon_sym_DOT_DOT] = ACTIONS(2494),
    [anon_sym_COLON_COLON] = ACTIONS(2494),
    [anon_sym_POUND] = ACTIONS(2494),
    [anon_sym_SQUOTE] = ACTIONS(2496),
    [anon_sym_async] = ACTIONS(2496),
    [anon_sym_auto] = ACTIONS(2496),
    [anon_sym_await] = ACTIONS(2496),
    [anon_sym_become] = ACTIONS(2496),
    [anon_sym_break] = ACTIONS(2496),
    [anon_sym_const] = ACTIONS(2496),
    [anon_sym_continue] = ACTIONS(2496),
    [anon_sym_default] = ACTIONS(2496),
    [anon_sym_enum] = ACTIONS(2496),
    [anon_sym_fn] = ACTIONS(2496),
    [anon_sym_for] = ACTIONS(2496),
    [anon_sym_gen] = ACTIONS(2496),
    [anon_sym_if] = ACTIONS(2496),
    [anon_sym_impl] = ACTIONS(2496),
    [anon_sym_let] = ACTIONS(2496),
    [anon_sym_loop] = ACTIONS(2496),
    [anon_sym_match] = ACTIONS(2496),
    [anon_sym_mod] = ACTIONS(2496),
    [anon_sym_pub] = ACTIONS(2496),
    [anon_sym_return] = ACTIONS(2496),
    [anon_sym_raw] = ACTIONS(2496),
    [anon_sym_static] = ACTIONS(2496),
    [anon_sym_struct] = ACTIONS(2496),
    [anon_sym_trait] = ACTIONS(2496),
    [anon_sym_type] = ACTIONS(2496),
    [anon_sym_union] = ACTIONS(2496),
    [anon_sym_unsafe] = ACTIONS(2496),
    [anon_sym_use] = ACTIONS(2496),
    [anon_sym_while] = ACTIONS(2496),
    [anon_sym_extern] = ACTIONS(2496),
    [anon_sym_safe] = ACTIONS(2496),
    [anon_sym_yield] = ACTIONS(2496),
    [anon_sym_move] = ACTIONS(2496),
    [anon_sym_try] = ACTIONS(2496),
    [sym_integer_literal] = ACTIONS(2494),
    [aux_sym_string_literal_token1] = ACTIONS(2494),
    [sym_char_literal] = ACTIONS(2494),
    [anon_sym_true] = ACTIONS(2496),
    [anon_sym_false] = ACTIONS(2496),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2496),
    [sym_super] = ACTIONS(2496),
    [sym_crate] = ACTIONS(2496),
    [aux_sym_metavariable_token1] = ACTIONS(2494),
    [sym__raw_string_literal_start] = ACTIONS(2494),
    [sym_float_literal] = ACTIONS(2494),
    [sym__outer_block_doc_comment_start] = ACTIONS(2494),
    [sym__outer_line_doc_comment_start] = ACTIONS(2494),
  },
  [STATE(681)] = {
    [sym_line_comment] = STATE(681),
    [sym_block_comment] = STATE(681),
    [ts_builtin_sym_end] = ACTIONS(2498),
    [sym_identifier] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2498),
    [anon_sym_macro] = ACTIONS(2500),
    [anon_sym_LPAREN] = ACTIONS(2498),
    [anon_sym_LBRACE] = ACTIONS(2498),
    [anon_sym_RBRACE] = ACTIONS(2498),
    [anon_sym_macro_rules] = ACTIONS(2500),
    [anon_sym_BANG] = ACTIONS(2498),
    [anon_sym_LBRACK] = ACTIONS(2498),
    [anon_sym_STAR] = ACTIONS(2498),
    [anon_sym_u8] = ACTIONS(2500),
    [anon_sym_i8] = ACTIONS(2500),
    [anon_sym_u16] = ACTIONS(2500),
    [anon_sym_i16] = ACTIONS(2500),
    [anon_sym_u32] = ACTIONS(2500),
    [anon_sym_i32] = ACTIONS(2500),
    [anon_sym_u64] = ACTIONS(2500),
    [anon_sym_i64] = ACTIONS(2500),
    [anon_sym_u128] = ACTIONS(2500),
    [anon_sym_i128] = ACTIONS(2500),
    [anon_sym_isize] = ACTIONS(2500),
    [anon_sym_usize] = ACTIONS(2500),
    [anon_sym_f32] = ACTIONS(2500),
    [anon_sym_f64] = ACTIONS(2500),
    [anon_sym_bool] = ACTIONS(2500),
    [anon_sym_str] = ACTIONS(2500),
    [anon_sym_char] = ACTIONS(2500),
    [anon_sym_DASH] = ACTIONS(2498),
    [anon_sym_AMP] = ACTIONS(2498),
    [anon_sym_PIPE] = ACTIONS(2498),
    [anon_sym_LT] = ACTIONS(2498),
    [anon_sym_DOT_DOT] = ACTIONS(2498),
    [anon_sym_COLON_COLON] = ACTIONS(2498),
    [anon_sym_POUND] = ACTIONS(2498),
    [anon_sym_SQUOTE] = ACTIONS(2500),
    [anon_sym_async] = ACTIONS(2500),
    [anon_sym_auto] = ACTIONS(2500),
    [anon_sym_await] = ACTIONS(2500),
    [anon_sym_become] = ACTIONS(2500),
    [anon_sym_break] = ACTIONS(2500),
    [anon_sym_const] = ACTIONS(2500),
    [anon_sym_continue] = ACTIONS(2500),
    [anon_sym_default] = ACTIONS(2500),
    [anon_sym_enum] = ACTIONS(2500),
    [anon_sym_fn] = ACTIONS(2500),
    [anon_sym_for] = ACTIONS(2500),
    [anon_sym_gen] = ACTIONS(2500),
    [anon_sym_if] = ACTIONS(2500),
    [anon_sym_impl] = ACTIONS(2500),
    [anon_sym_let] = ACTIONS(2500),
    [anon_sym_loop] = ACTIONS(2500),
    [anon_sym_match] = ACTIONS(2500),
    [anon_sym_mod] = ACTIONS(2500),
    [anon_sym_pub] = ACTIONS(2500),
    [anon_sym_return] = ACTIONS(2500),
    [anon_sym_raw] = ACTIONS(2500),
    [anon_sym_static] = ACTIONS(2500),
    [anon_sym_struct] = ACTIONS(2500),
    [anon_sym_trait] = ACTIONS(2500),
    [anon_sym_type] = ACTIONS(2500),
    [anon_sym_union] = ACTIONS(2500),
    [anon_sym_unsafe] = ACTIONS(2500),
    [anon_sym_use] = ACTIONS(2500),
    [anon_sym_while] = ACTIONS(2500),
    [anon_sym_extern] = ACTIONS(2500),
    [anon_sym_safe] = ACTIONS(2500),
    [anon_sym_yield] = ACTIONS(2500),
    [anon_sym_move] = ACTIONS(2500),
    [anon_sym_try] = ACTIONS(2500),
    [sym_integer_literal] = ACTIONS(2498),
    [aux_sym_string_literal_token1] = ACTIONS(2498),
    [sym_char_literal] = ACTIONS(2498),
    [anon_sym_true] = ACTIONS(2500),
    [anon_sym_false] = ACTIONS(2500),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2500),
    [sym_super] = ACTIONS(2500),
    [sym_crate] = ACTIONS(2500),
    [aux_sym_metavariable_token1] = ACTIONS(2498),
    [sym__raw_string_literal_start] = ACTIONS(2498),
    [sym_float_literal] = ACTIONS(2498),
    [sym__outer_block_doc_comment_start] = ACTIONS(2498),
    [sym__outer_line_doc_comment_start] = ACTIONS(2498),
  },
  [STATE(682)] = {
    [sym_line_comment] = STATE(682),
    [sym_block_comment] = STATE(682),
    [ts_builtin_sym_end] = ACTIONS(2502),
    [sym_identifier] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2502),
    [anon_sym_macro] = ACTIONS(2504),
    [anon_sym_LPAREN] = ACTIONS(2502),
    [anon_sym_LBRACE] = ACTIONS(2502),
    [anon_sym_RBRACE] = ACTIONS(2502),
    [anon_sym_macro_rules] = ACTIONS(2504),
    [anon_sym_BANG] = ACTIONS(2502),
    [anon_sym_LBRACK] = ACTIONS(2502),
    [anon_sym_STAR] = ACTIONS(2502),
    [anon_sym_u8] = ACTIONS(2504),
    [anon_sym_i8] = ACTIONS(2504),
    [anon_sym_u16] = ACTIONS(2504),
    [anon_sym_i16] = ACTIONS(2504),
    [anon_sym_u32] = ACTIONS(2504),
    [anon_sym_i32] = ACTIONS(2504),
    [anon_sym_u64] = ACTIONS(2504),
    [anon_sym_i64] = ACTIONS(2504),
    [anon_sym_u128] = ACTIONS(2504),
    [anon_sym_i128] = ACTIONS(2504),
    [anon_sym_isize] = ACTIONS(2504),
    [anon_sym_usize] = ACTIONS(2504),
    [anon_sym_f32] = ACTIONS(2504),
    [anon_sym_f64] = ACTIONS(2504),
    [anon_sym_bool] = ACTIONS(2504),
    [anon_sym_str] = ACTIONS(2504),
    [anon_sym_char] = ACTIONS(2504),
    [anon_sym_DASH] = ACTIONS(2502),
    [anon_sym_AMP] = ACTIONS(2502),
    [anon_sym_PIPE] = ACTIONS(2502),
    [anon_sym_LT] = ACTIONS(2502),
    [anon_sym_DOT_DOT] = ACTIONS(2502),
    [anon_sym_COLON_COLON] = ACTIONS(2502),
    [anon_sym_POUND] = ACTIONS(2502),
    [anon_sym_SQUOTE] = ACTIONS(2504),
    [anon_sym_async] = ACTIONS(2504),
    [anon_sym_auto] = ACTIONS(2504),
    [anon_sym_await] = ACTIONS(2504),
    [anon_sym_become] = ACTIONS(2504),
    [anon_sym_break] = ACTIONS(2504),
    [anon_sym_const] = ACTIONS(2504),
    [anon_sym_continue] = ACTIONS(2504),
    [anon_sym_default] = ACTIONS(2504),
    [anon_sym_enum] = ACTIONS(2504),
    [anon_sym_fn] = ACTIONS(2504),
    [anon_sym_for] = ACTIONS(2504),
    [anon_sym_gen] = ACTIONS(2504),
    [anon_sym_if] = ACTIONS(2504),
    [anon_sym_impl] = ACTIONS(2504),
    [anon_sym_let] = ACTIONS(2504),
    [anon_sym_loop] = ACTIONS(2504),
    [anon_sym_match] = ACTIONS(2504),
    [anon_sym_mod] = ACTIONS(2504),
    [anon_sym_pub] = ACTIONS(2504),
    [anon_sym_return] = ACTIONS(2504),
    [anon_sym_raw] = ACTIONS(2504),
    [anon_sym_static] = ACTIONS(2504),
    [anon_sym_struct] = ACTIONS(2504),
    [anon_sym_trait] = ACTIONS(2504),
    [anon_sym_type] = ACTIONS(2504),
    [anon_sym_union] = ACTIONS(2504),
    [anon_sym_unsafe] = ACTIONS(2504),
    [anon_sym_use] = ACTIONS(2504),
    [anon_sym_while] = ACTIONS(2504),
    [anon_sym_extern] = ACTIONS(2504),
    [anon_sym_safe] = ACTIONS(2504),
    [anon_sym_yield] = ACTIONS(2504),
    [anon_sym_move] = ACTIONS(2504),
    [anon_sym_try] = ACTIONS(2504),
    [sym_integer_literal] = ACTIONS(2502),
    [aux_sym_string_literal_token1] = ACTIONS(2502),
    [sym_char_literal] = ACTIONS(2502),
    [anon_sym_true] = ACTIONS(2504),
    [anon_sym_false] = ACTIONS(2504),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2504),
    [sym_super] = ACTIONS(2504),
    [sym_crate] = ACTIONS(2504),
    [aux_sym_metavariable_token1] = ACTIONS(2502),
    [sym__raw_string_literal_start] = ACTIONS(2502),
    [sym_float_literal] = ACTIONS(2502),
    [sym__outer_block_doc_comment_start] = ACTIONS(2502),
    [sym__outer_line_doc_comment_start] = ACTIONS(2502),
  },
  [STATE(683)] = {
    [sym_line_comment] = STATE(683),
    [sym_block_comment] = STATE(683),
    [ts_builtin_sym_end] = ACTIONS(2506),
    [sym_identifier] = ACTIONS(2508),
    [anon_sym_SEMI] = ACTIONS(2506),
    [anon_sym_macro] = ACTIONS(2508),
    [anon_sym_LPAREN] = ACTIONS(2506),
    [anon_sym_LBRACE] = ACTIONS(2506),
    [anon_sym_RBRACE] = ACTIONS(2506),
    [anon_sym_macro_rules] = ACTIONS(2508),
    [anon_sym_BANG] = ACTIONS(2506),
    [anon_sym_LBRACK] = ACTIONS(2506),
    [anon_sym_STAR] = ACTIONS(2506),
    [anon_sym_u8] = ACTIONS(2508),
    [anon_sym_i8] = ACTIONS(2508),
    [anon_sym_u16] = ACTIONS(2508),
    [anon_sym_i16] = ACTIONS(2508),
    [anon_sym_u32] = ACTIONS(2508),
    [anon_sym_i32] = ACTIONS(2508),
    [anon_sym_u64] = ACTIONS(2508),
    [anon_sym_i64] = ACTIONS(2508),
    [anon_sym_u128] = ACTIONS(2508),
    [anon_sym_i128] = ACTIONS(2508),
    [anon_sym_isize] = ACTIONS(2508),
    [anon_sym_usize] = ACTIONS(2508),
    [anon_sym_f32] = ACTIONS(2508),
    [anon_sym_f64] = ACTIONS(2508),
    [anon_sym_bool] = ACTIONS(2508),
    [anon_sym_str] = ACTIONS(2508),
    [anon_sym_char] = ACTIONS(2508),
    [anon_sym_DASH] = ACTIONS(2506),
    [anon_sym_AMP] = ACTIONS(2506),
    [anon_sym_PIPE] = ACTIONS(2506),
    [anon_sym_LT] = ACTIONS(2506),
    [anon_sym_DOT_DOT] = ACTIONS(2506),
    [anon_sym_COLON_COLON] = ACTIONS(2506),
    [anon_sym_POUND] = ACTIONS(2506),
    [anon_sym_SQUOTE] = ACTIONS(2508),
    [anon_sym_async] = ACTIONS(2508),
    [anon_sym_auto] = ACTIONS(2508),
    [anon_sym_await] = ACTIONS(2508),
    [anon_sym_become] = ACTIONS(2508),
    [anon_sym_break] = ACTIONS(2508),
    [anon_sym_const] = ACTIONS(2508),
    [anon_sym_continue] = ACTIONS(2508),
    [anon_sym_default] = ACTIONS(2508),
    [anon_sym_enum] = ACTIONS(2508),
    [anon_sym_fn] = ACTIONS(2508),
    [anon_sym_for] = ACTIONS(2508),
    [anon_sym_gen] = ACTIONS(2508),
    [anon_sym_if] = ACTIONS(2508),
    [anon_sym_impl] = ACTIONS(2508),
    [anon_sym_let] = ACTIONS(2508),
    [anon_sym_loop] = ACTIONS(2508),
    [anon_sym_match] = ACTIONS(2508),
    [anon_sym_mod] = ACTIONS(2508),
    [anon_sym_pub] = ACTIONS(2508),
    [anon_sym_return] = ACTIONS(2508),
    [anon_sym_raw] = ACTIONS(2508),
    [anon_sym_static] = ACTIONS(2508),
    [anon_sym_struct] = ACTIONS(2508),
    [anon_sym_trait] = ACTIONS(2508),
    [anon_sym_type] = ACTIONS(2508),
    [anon_sym_union] = ACTIONS(2508),
    [anon_sym_unsafe] = ACTIONS(2508),
    [anon_sym_use] = ACTIONS(2508),
    [anon_sym_while] = ACTIONS(2508),
    [anon_sym_extern] = ACTIONS(2508),
    [anon_sym_safe] = ACTIONS(2508),
    [anon_sym_yield] = ACTIONS(2508),
    [anon_sym_move] = ACTIONS(2508),
    [anon_sym_try] = ACTIONS(2508),
    [sym_integer_literal] = ACTIONS(2506),
    [aux_sym_string_literal_token1] = ACTIONS(2506),
    [sym_char_literal] = ACTIONS(2506),
    [anon_sym_true] = ACTIONS(2508),
    [anon_sym_false] = ACTIONS(2508),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2508),
    [sym_super] = ACTIONS(2508),
    [sym_crate] = ACTIONS(2508),
    [aux_sym_metavariable_token1] = ACTIONS(2506),
    [sym__raw_string_literal_start] = ACTIONS(2506),
    [sym_float_literal] = ACTIONS(2506),
    [sym__outer_block_doc_comment_start] = ACTIONS(2506),
    [sym__outer_line_doc_comment_start] = ACTIONS(2506),
  },
  [STATE(684)] = {
    [sym_line_comment] = STATE(684),
    [sym_block_comment] = STATE(684),
    [ts_builtin_sym_end] = ACTIONS(2510),
    [sym_identifier] = ACTIONS(2512),
    [anon_sym_SEMI] = ACTIONS(2510),
    [anon_sym_macro] = ACTIONS(2512),
    [anon_sym_LPAREN] = ACTIONS(2510),
    [anon_sym_LBRACE] = ACTIONS(2510),
    [anon_sym_RBRACE] = ACTIONS(2510),
    [anon_sym_macro_rules] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2510),
    [anon_sym_LBRACK] = ACTIONS(2510),
    [anon_sym_STAR] = ACTIONS(2510),
    [anon_sym_u8] = ACTIONS(2512),
    [anon_sym_i8] = ACTIONS(2512),
    [anon_sym_u16] = ACTIONS(2512),
    [anon_sym_i16] = ACTIONS(2512),
    [anon_sym_u32] = ACTIONS(2512),
    [anon_sym_i32] = ACTIONS(2512),
    [anon_sym_u64] = ACTIONS(2512),
    [anon_sym_i64] = ACTIONS(2512),
    [anon_sym_u128] = ACTIONS(2512),
    [anon_sym_i128] = ACTIONS(2512),
    [anon_sym_isize] = ACTIONS(2512),
    [anon_sym_usize] = ACTIONS(2512),
    [anon_sym_f32] = ACTIONS(2512),
    [anon_sym_f64] = ACTIONS(2512),
    [anon_sym_bool] = ACTIONS(2512),
    [anon_sym_str] = ACTIONS(2512),
    [anon_sym_char] = ACTIONS(2512),
    [anon_sym_DASH] = ACTIONS(2510),
    [anon_sym_AMP] = ACTIONS(2510),
    [anon_sym_PIPE] = ACTIONS(2510),
    [anon_sym_LT] = ACTIONS(2510),
    [anon_sym_DOT_DOT] = ACTIONS(2510),
    [anon_sym_COLON_COLON] = ACTIONS(2510),
    [anon_sym_POUND] = ACTIONS(2510),
    [anon_sym_SQUOTE] = ACTIONS(2512),
    [anon_sym_async] = ACTIONS(2512),
    [anon_sym_auto] = ACTIONS(2512),
    [anon_sym_await] = ACTIONS(2512),
    [anon_sym_become] = ACTIONS(2512),
    [anon_sym_break] = ACTIONS(2512),
    [anon_sym_const] = ACTIONS(2512),
    [anon_sym_continue] = ACTIONS(2512),
    [anon_sym_default] = ACTIONS(2512),
    [anon_sym_enum] = ACTIONS(2512),
    [anon_sym_fn] = ACTIONS(2512),
    [anon_sym_for] = ACTIONS(2512),
    [anon_sym_gen] = ACTIONS(2512),
    [anon_sym_if] = ACTIONS(2512),
    [anon_sym_impl] = ACTIONS(2512),
    [anon_sym_let] = ACTIONS(2512),
    [anon_sym_loop] = ACTIONS(2512),
    [anon_sym_match] = ACTIONS(2512),
    [anon_sym_mod] = ACTIONS(2512),
    [anon_sym_pub] = ACTIONS(2512),
    [anon_sym_return] = ACTIONS(2512),
    [anon_sym_raw] = ACTIONS(2512),
    [anon_sym_static] = ACTIONS(2512),
    [anon_sym_struct] = ACTIONS(2512),
    [anon_sym_trait] = ACTIONS(2512),
    [anon_sym_type] = ACTIONS(2512),
    [anon_sym_union] = ACTIONS(2512),
    [anon_sym_unsafe] = ACTIONS(2512),
    [anon_sym_use] = ACTIONS(2512),
    [anon_sym_while] = ACTIONS(2512),
    [anon_sym_extern] = ACTIONS(2512),
    [anon_sym_safe] = ACTIONS(2512),
    [anon_sym_yield] = ACTIONS(2512),
    [anon_sym_move] = ACTIONS(2512),
    [anon_sym_try] = ACTIONS(2512),
    [sym_integer_literal] = ACTIONS(2510),
    [aux_sym_string_literal_token1] = ACTIONS(2510),
    [sym_char_literal] = ACTIONS(2510),
    [anon_sym_true] = ACTIONS(2512),
    [anon_sym_false] = ACTIONS(2512),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2512),
    [sym_super] = ACTIONS(2512),
    [sym_crate] = ACTIONS(2512),
    [aux_sym_metavariable_token1] = ACTIONS(2510),
    [sym__raw_string_literal_start] = ACTIONS(2510),
    [sym_float_literal] = ACTIONS(2510),
    [sym__outer_block_doc_comment_start] = ACTIONS(2510),
    [sym__outer_line_doc_comment_start] = ACTIONS(2510),
  },
  [STATE(685)] = {
    [sym_line_comment] = STATE(685),
    [sym_block_comment] = STATE(685),
    [ts_builtin_sym_end] = ACTIONS(2514),
    [sym_identifier] = ACTIONS(2516),
    [anon_sym_SEMI] = ACTIONS(2514),
    [anon_sym_macro] = ACTIONS(2516),
    [anon_sym_LPAREN] = ACTIONS(2514),
    [anon_sym_LBRACE] = ACTIONS(2514),
    [anon_sym_RBRACE] = ACTIONS(2514),
    [anon_sym_macro_rules] = ACTIONS(2516),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_LBRACK] = ACTIONS(2514),
    [anon_sym_STAR] = ACTIONS(2514),
    [anon_sym_u8] = ACTIONS(2516),
    [anon_sym_i8] = ACTIONS(2516),
    [anon_sym_u16] = ACTIONS(2516),
    [anon_sym_i16] = ACTIONS(2516),
    [anon_sym_u32] = ACTIONS(2516),
    [anon_sym_i32] = ACTIONS(2516),
    [anon_sym_u64] = ACTIONS(2516),
    [anon_sym_i64] = ACTIONS(2516),
    [anon_sym_u128] = ACTIONS(2516),
    [anon_sym_i128] = ACTIONS(2516),
    [anon_sym_isize] = ACTIONS(2516),
    [anon_sym_usize] = ACTIONS(2516),
    [anon_sym_f32] = ACTIONS(2516),
    [anon_sym_f64] = ACTIONS(2516),
    [anon_sym_bool] = ACTIONS(2516),
    [anon_sym_str] = ACTIONS(2516),
    [anon_sym_char] = ACTIONS(2516),
    [anon_sym_DASH] = ACTIONS(2514),
    [anon_sym_AMP] = ACTIONS(2514),
    [anon_sym_PIPE] = ACTIONS(2514),
    [anon_sym_LT] = ACTIONS(2514),
    [anon_sym_DOT_DOT] = ACTIONS(2514),
    [anon_sym_COLON_COLON] = ACTIONS(2514),
    [anon_sym_POUND] = ACTIONS(2514),
    [anon_sym_SQUOTE] = ACTIONS(2516),
    [anon_sym_async] = ACTIONS(2516),
    [anon_sym_auto] = ACTIONS(2516),
    [anon_sym_await] = ACTIONS(2516),
    [anon_sym_become] = ACTIONS(2516),
    [anon_sym_break] = ACTIONS(2516),
    [anon_sym_const] = ACTIONS(2516),
    [anon_sym_continue] = ACTIONS(2516),
    [anon_sym_default] = ACTIONS(2516),
    [anon_sym_enum] = ACTIONS(2516),
    [anon_sym_fn] = ACTIONS(2516),
    [anon_sym_for] = ACTIONS(2516),
    [anon_sym_gen] = ACTIONS(2516),
    [anon_sym_if] = ACTIONS(2516),
    [anon_sym_impl] = ACTIONS(2516),
    [anon_sym_let] = ACTIONS(2516),
    [anon_sym_loop] = ACTIONS(2516),
    [anon_sym_match] = ACTIONS(2516),
    [anon_sym_mod] = ACTIONS(2516),
    [anon_sym_pub] = ACTIONS(2516),
    [anon_sym_return] = ACTIONS(2516),
    [anon_sym_raw] = ACTIONS(2516),
    [anon_sym_static] = ACTIONS(2516),
    [anon_sym_struct] = ACTIONS(2516),
    [anon_sym_trait] = ACTIONS(2516),
    [anon_sym_type] = ACTIONS(2516),
    [anon_sym_union] = ACTIONS(2516),
    [anon_sym_unsafe] = ACTIONS(2516),
    [anon_sym_use] = ACTIONS(2516),
    [anon_sym_while] = ACTIONS(2516),
    [anon_sym_extern] = ACTIONS(2516),
    [anon_sym_safe] = ACTIONS(2516),
    [anon_sym_yield] = ACTIONS(2516),
    [anon_sym_move] = ACTIONS(2516),
    [anon_sym_try] = ACTIONS(2516),
    [sym_integer_literal] = ACTIONS(2514),
    [aux_sym_string_literal_token1] = ACTIONS(2514),
    [sym_char_literal] = ACTIONS(2514),
    [anon_sym_true] = ACTIONS(2516),
    [anon_sym_false] = ACTIONS(2516),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2516),
    [sym_super] = ACTIONS(2516),
    [sym_crate] = ACTIONS(2516),
    [aux_sym_metavariable_token1] = ACTIONS(2514),
    [sym__raw_string_literal_start] = ACTIONS(2514),
    [sym_float_literal] = ACTIONS(2514),
    [sym__outer_block_doc_comment_start] = ACTIONS(2514),
    [sym__outer_line_doc_comment_start] = ACTIONS(2514),
  },
  [STATE(686)] = {
    [sym_line_comment] = STATE(686),
    [sym_block_comment] = STATE(686),
    [ts_builtin_sym_end] = ACTIONS(2518),
    [sym_identifier] = ACTIONS(2520),
    [anon_sym_SEMI] = ACTIONS(2518),
    [anon_sym_macro] = ACTIONS(2520),
    [anon_sym_LPAREN] = ACTIONS(2518),
    [anon_sym_LBRACE] = ACTIONS(2518),
    [anon_sym_RBRACE] = ACTIONS(2518),
    [anon_sym_macro_rules] = ACTIONS(2520),
    [anon_sym_BANG] = ACTIONS(2518),
    [anon_sym_LBRACK] = ACTIONS(2518),
    [anon_sym_STAR] = ACTIONS(2518),
    [anon_sym_u8] = ACTIONS(2520),
    [anon_sym_i8] = ACTIONS(2520),
    [anon_sym_u16] = ACTIONS(2520),
    [anon_sym_i16] = ACTIONS(2520),
    [anon_sym_u32] = ACTIONS(2520),
    [anon_sym_i32] = ACTIONS(2520),
    [anon_sym_u64] = ACTIONS(2520),
    [anon_sym_i64] = ACTIONS(2520),
    [anon_sym_u128] = ACTIONS(2520),
    [anon_sym_i128] = ACTIONS(2520),
    [anon_sym_isize] = ACTIONS(2520),
    [anon_sym_usize] = ACTIONS(2520),
    [anon_sym_f32] = ACTIONS(2520),
    [anon_sym_f64] = ACTIONS(2520),
    [anon_sym_bool] = ACTIONS(2520),
    [anon_sym_str] = ACTIONS(2520),
    [anon_sym_char] = ACTIONS(2520),
    [anon_sym_DASH] = ACTIONS(2518),
    [anon_sym_AMP] = ACTIONS(2518),
    [anon_sym_PIPE] = ACTIONS(2518),
    [anon_sym_LT] = ACTIONS(2518),
    [anon_sym_DOT_DOT] = ACTIONS(2518),
    [anon_sym_COLON_COLON] = ACTIONS(2518),
    [anon_sym_POUND] = ACTIONS(2518),
    [anon_sym_SQUOTE] = ACTIONS(2520),
    [anon_sym_async] = ACTIONS(2520),
    [anon_sym_auto] = ACTIONS(2520),
    [anon_sym_await] = ACTIONS(2520),
    [anon_sym_become] = ACTIONS(2520),
    [anon_sym_break] = ACTIONS(2520),
    [anon_sym_const] = ACTIONS(2520),
    [anon_sym_continue] = ACTIONS(2520),
    [anon_sym_default] = ACTIONS(2520),
    [anon_sym_enum] = ACTIONS(2520),
    [anon_sym_fn] = ACTIONS(2520),
    [anon_sym_for] = ACTIONS(2520),
    [anon_sym_gen] = ACTIONS(2520),
    [anon_sym_if] = ACTIONS(2520),
    [anon_sym_impl] = ACTIONS(2520),
    [anon_sym_let] = ACTIONS(2520),
    [anon_sym_loop] = ACTIONS(2520),
    [anon_sym_match] = ACTIONS(2520),
    [anon_sym_mod] = ACTIONS(2520),
    [anon_sym_pub] = ACTIONS(2520),
    [anon_sym_return] = ACTIONS(2520),
    [anon_sym_raw] = ACTIONS(2520),
    [anon_sym_static] = ACTIONS(2520),
    [anon_sym_struct] = ACTIONS(2520),
    [anon_sym_trait] = ACTIONS(2520),
    [anon_sym_type] = ACTIONS(2520),
    [anon_sym_union] = ACTIONS(2520),
    [anon_sym_unsafe] = ACTIONS(2520),
    [anon_sym_use] = ACTIONS(2520),
    [anon_sym_while] = ACTIONS(2520),
    [anon_sym_extern] = ACTIONS(2520),
    [anon_sym_safe] = ACTIONS(2520),
    [anon_sym_yield] = ACTIONS(2520),
    [anon_sym_move] = ACTIONS(2520),
    [anon_sym_try] = ACTIONS(2520),
    [sym_integer_literal] = ACTIONS(2518),
    [aux_sym_string_literal_token1] = ACTIONS(2518),
    [sym_char_literal] = ACTIONS(2518),
    [anon_sym_true] = ACTIONS(2520),
    [anon_sym_false] = ACTIONS(2520),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2520),
    [sym_super] = ACTIONS(2520),
    [sym_crate] = ACTIONS(2520),
    [aux_sym_metavariable_token1] = ACTIONS(2518),
    [sym__raw_string_literal_start] = ACTIONS(2518),
    [sym_float_literal] = ACTIONS(2518),
    [sym__outer_block_doc_comment_start] = ACTIONS(2518),
    [sym__outer_line_doc_comment_start] = ACTIONS(2518),
  },
  [STATE(687)] = {
    [sym_line_comment] = STATE(687),
    [sym_block_comment] = STATE(687),
    [ts_builtin_sym_end] = ACTIONS(2522),
    [sym_identifier] = ACTIONS(2524),
    [anon_sym_SEMI] = ACTIONS(2522),
    [anon_sym_macro] = ACTIONS(2524),
    [anon_sym_LPAREN] = ACTIONS(2522),
    [anon_sym_LBRACE] = ACTIONS(2522),
    [anon_sym_RBRACE] = ACTIONS(2522),
    [anon_sym_macro_rules] = ACTIONS(2524),
    [anon_sym_BANG] = ACTIONS(2522),
    [anon_sym_LBRACK] = ACTIONS(2522),
    [anon_sym_STAR] = ACTIONS(2522),
    [anon_sym_u8] = ACTIONS(2524),
    [anon_sym_i8] = ACTIONS(2524),
    [anon_sym_u16] = ACTIONS(2524),
    [anon_sym_i16] = ACTIONS(2524),
    [anon_sym_u32] = ACTIONS(2524),
    [anon_sym_i32] = ACTIONS(2524),
    [anon_sym_u64] = ACTIONS(2524),
    [anon_sym_i64] = ACTIONS(2524),
    [anon_sym_u128] = ACTIONS(2524),
    [anon_sym_i128] = ACTIONS(2524),
    [anon_sym_isize] = ACTIONS(2524),
    [anon_sym_usize] = ACTIONS(2524),
    [anon_sym_f32] = ACTIONS(2524),
    [anon_sym_f64] = ACTIONS(2524),
    [anon_sym_bool] = ACTIONS(2524),
    [anon_sym_str] = ACTIONS(2524),
    [anon_sym_char] = ACTIONS(2524),
    [anon_sym_DASH] = ACTIONS(2522),
    [anon_sym_AMP] = ACTIONS(2522),
    [anon_sym_PIPE] = ACTIONS(2522),
    [anon_sym_LT] = ACTIONS(2522),
    [anon_sym_DOT_DOT] = ACTIONS(2522),
    [anon_sym_COLON_COLON] = ACTIONS(2522),
    [anon_sym_POUND] = ACTIONS(2522),
    [anon_sym_SQUOTE] = ACTIONS(2524),
    [anon_sym_async] = ACTIONS(2524),
    [anon_sym_auto] = ACTIONS(2524),
    [anon_sym_await] = ACTIONS(2524),
    [anon_sym_become] = ACTIONS(2524),
    [anon_sym_break] = ACTIONS(2524),
    [anon_sym_const] = ACTIONS(2524),
    [anon_sym_continue] = ACTIONS(2524),
    [anon_sym_default] = ACTIONS(2524),
    [anon_sym_enum] = ACTIONS(2524),
    [anon_sym_fn] = ACTIONS(2524),
    [anon_sym_for] = ACTIONS(2524),
    [anon_sym_gen] = ACTIONS(2524),
    [anon_sym_if] = ACTIONS(2524),
    [anon_sym_impl] = ACTIONS(2524),
    [anon_sym_let] = ACTIONS(2524),
    [anon_sym_loop] = ACTIONS(2524),
    [anon_sym_match] = ACTIONS(2524),
    [anon_sym_mod] = ACTIONS(2524),
    [anon_sym_pub] = ACTIONS(2524),
    [anon_sym_return] = ACTIONS(2524),
    [anon_sym_raw] = ACTIONS(2524),
    [anon_sym_static] = ACTIONS(2524),
    [anon_sym_struct] = ACTIONS(2524),
    [anon_sym_trait] = ACTIONS(2524),
    [anon_sym_type] = ACTIONS(2524),
    [anon_sym_union] = ACTIONS(2524),
    [anon_sym_unsafe] = ACTIONS(2524),
    [anon_sym_use] = ACTIONS(2524),
    [anon_sym_while] = ACTIONS(2524),
    [anon_sym_extern] = ACTIONS(2524),
    [anon_sym_safe] = ACTIONS(2524),
    [anon_sym_yield] = ACTIONS(2524),
    [anon_sym_move] = ACTIONS(2524),
    [anon_sym_try] = ACTIONS(2524),
    [sym_integer_literal] = ACTIONS(2522),
    [aux_sym_string_literal_token1] = ACTIONS(2522),
    [sym_char_literal] = ACTIONS(2522),
    [anon_sym_true] = ACTIONS(2524),
    [anon_sym_false] = ACTIONS(2524),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2524),
    [sym_super] = ACTIONS(2524),
    [sym_crate] = ACTIONS(2524),
    [aux_sym_metavariable_token1] = ACTIONS(2522),
    [sym__raw_string_literal_start] = ACTIONS(2522),
    [sym_float_literal] = ACTIONS(2522),
    [sym__outer_block_doc_comment_start] = ACTIONS(2522),
    [sym__outer_line_doc_comment_start] = ACTIONS(2522),
  },
  [STATE(688)] = {
    [sym_line_comment] = STATE(688),
    [sym_block_comment] = STATE(688),
    [ts_builtin_sym_end] = ACTIONS(2526),
    [sym_identifier] = ACTIONS(2528),
    [anon_sym_SEMI] = ACTIONS(2526),
    [anon_sym_macro] = ACTIONS(2528),
    [anon_sym_LPAREN] = ACTIONS(2526),
    [anon_sym_LBRACE] = ACTIONS(2526),
    [anon_sym_RBRACE] = ACTIONS(2526),
    [anon_sym_macro_rules] = ACTIONS(2528),
    [anon_sym_BANG] = ACTIONS(2526),
    [anon_sym_LBRACK] = ACTIONS(2526),
    [anon_sym_STAR] = ACTIONS(2526),
    [anon_sym_u8] = ACTIONS(2528),
    [anon_sym_i8] = ACTIONS(2528),
    [anon_sym_u16] = ACTIONS(2528),
    [anon_sym_i16] = ACTIONS(2528),
    [anon_sym_u32] = ACTIONS(2528),
    [anon_sym_i32] = ACTIONS(2528),
    [anon_sym_u64] = ACTIONS(2528),
    [anon_sym_i64] = ACTIONS(2528),
    [anon_sym_u128] = ACTIONS(2528),
    [anon_sym_i128] = ACTIONS(2528),
    [anon_sym_isize] = ACTIONS(2528),
    [anon_sym_usize] = ACTIONS(2528),
    [anon_sym_f32] = ACTIONS(2528),
    [anon_sym_f64] = ACTIONS(2528),
    [anon_sym_bool] = ACTIONS(2528),
    [anon_sym_str] = ACTIONS(2528),
    [anon_sym_char] = ACTIONS(2528),
    [anon_sym_DASH] = ACTIONS(2526),
    [anon_sym_AMP] = ACTIONS(2526),
    [anon_sym_PIPE] = ACTIONS(2526),
    [anon_sym_LT] = ACTIONS(2526),
    [anon_sym_DOT_DOT] = ACTIONS(2526),
    [anon_sym_COLON_COLON] = ACTIONS(2526),
    [anon_sym_POUND] = ACTIONS(2526),
    [anon_sym_SQUOTE] = ACTIONS(2528),
    [anon_sym_async] = ACTIONS(2528),
    [anon_sym_auto] = ACTIONS(2528),
    [anon_sym_await] = ACTIONS(2528),
    [anon_sym_become] = ACTIONS(2528),
    [anon_sym_break] = ACTIONS(2528),
    [anon_sym_const] = ACTIONS(2528),
    [anon_sym_continue] = ACTIONS(2528),
    [anon_sym_default] = ACTIONS(2528),
    [anon_sym_enum] = ACTIONS(2528),
    [anon_sym_fn] = ACTIONS(2528),
    [anon_sym_for] = ACTIONS(2528),
    [anon_sym_gen] = ACTIONS(2528),
    [anon_sym_if] = ACTIONS(2528),
    [anon_sym_impl] = ACTIONS(2528),
    [anon_sym_let] = ACTIONS(2528),
    [anon_sym_loop] = ACTIONS(2528),
    [anon_sym_match] = ACTIONS(2528),
    [anon_sym_mod] = ACTIONS(2528),
    [anon_sym_pub] = ACTIONS(2528),
    [anon_sym_return] = ACTIONS(2528),
    [anon_sym_raw] = ACTIONS(2528),
    [anon_sym_static] = ACTIONS(2528),
    [anon_sym_struct] = ACTIONS(2528),
    [anon_sym_trait] = ACTIONS(2528),
    [anon_sym_type] = ACTIONS(2528),
    [anon_sym_union] = ACTIONS(2528),
    [anon_sym_unsafe] = ACTIONS(2528),
    [anon_sym_use] = ACTIONS(2528),
    [anon_sym_while] = ACTIONS(2528),
    [anon_sym_extern] = ACTIONS(2528),
    [anon_sym_safe] = ACTIONS(2528),
    [anon_sym_yield] = ACTIONS(2528),
    [anon_sym_move] = ACTIONS(2528),
    [anon_sym_try] = ACTIONS(2528),
    [sym_integer_literal] = ACTIONS(2526),
    [aux_sym_string_literal_token1] = ACTIONS(2526),
    [sym_char_literal] = ACTIONS(2526),
    [anon_sym_true] = ACTIONS(2528),
    [anon_sym_false] = ACTIONS(2528),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2528),
    [sym_super] = ACTIONS(2528),
    [sym_crate] = ACTIONS(2528),
    [aux_sym_metavariable_token1] = ACTIONS(2526),
    [sym__raw_string_literal_start] = ACTIONS(2526),
    [sym_float_literal] = ACTIONS(2526),
    [sym__outer_block_doc_comment_start] = ACTIONS(2526),
    [sym__outer_line_doc_comment_start] = ACTIONS(2526),
  },
  [STATE(689)] = {
    [sym_line_comment] = STATE(689),
    [sym_block_comment] = STATE(689),
    [ts_builtin_sym_end] = ACTIONS(2530),
    [sym_identifier] = ACTIONS(2532),
    [anon_sym_SEMI] = ACTIONS(2530),
    [anon_sym_macro] = ACTIONS(2532),
    [anon_sym_LPAREN] = ACTIONS(2530),
    [anon_sym_LBRACE] = ACTIONS(2530),
    [anon_sym_RBRACE] = ACTIONS(2530),
    [anon_sym_macro_rules] = ACTIONS(2532),
    [anon_sym_BANG] = ACTIONS(2530),
    [anon_sym_LBRACK] = ACTIONS(2530),
    [anon_sym_STAR] = ACTIONS(2530),
    [anon_sym_u8] = ACTIONS(2532),
    [anon_sym_i8] = ACTIONS(2532),
    [anon_sym_u16] = ACTIONS(2532),
    [anon_sym_i16] = ACTIONS(2532),
    [anon_sym_u32] = ACTIONS(2532),
    [anon_sym_i32] = ACTIONS(2532),
    [anon_sym_u64] = ACTIONS(2532),
    [anon_sym_i64] = ACTIONS(2532),
    [anon_sym_u128] = ACTIONS(2532),
    [anon_sym_i128] = ACTIONS(2532),
    [anon_sym_isize] = ACTIONS(2532),
    [anon_sym_usize] = ACTIONS(2532),
    [anon_sym_f32] = ACTIONS(2532),
    [anon_sym_f64] = ACTIONS(2532),
    [anon_sym_bool] = ACTIONS(2532),
    [anon_sym_str] = ACTIONS(2532),
    [anon_sym_char] = ACTIONS(2532),
    [anon_sym_DASH] = ACTIONS(2530),
    [anon_sym_AMP] = ACTIONS(2530),
    [anon_sym_PIPE] = ACTIONS(2530),
    [anon_sym_LT] = ACTIONS(2530),
    [anon_sym_DOT_DOT] = ACTIONS(2530),
    [anon_sym_COLON_COLON] = ACTIONS(2530),
    [anon_sym_POUND] = ACTIONS(2530),
    [anon_sym_SQUOTE] = ACTIONS(2532),
    [anon_sym_async] = ACTIONS(2532),
    [anon_sym_auto] = ACTIONS(2532),
    [anon_sym_await] = ACTIONS(2532),
    [anon_sym_become] = ACTIONS(2532),
    [anon_sym_break] = ACTIONS(2532),
    [anon_sym_const] = ACTIONS(2532),
    [anon_sym_continue] = ACTIONS(2532),
    [anon_sym_default] = ACTIONS(2532),
    [anon_sym_enum] = ACTIONS(2532),
    [anon_sym_fn] = ACTIONS(2532),
    [anon_sym_for] = ACTIONS(2532),
    [anon_sym_gen] = ACTIONS(2532),
    [anon_sym_if] = ACTIONS(2532),
    [anon_sym_impl] = ACTIONS(2532),
    [anon_sym_let] = ACTIONS(2532),
    [anon_sym_loop] = ACTIONS(2532),
    [anon_sym_match] = ACTIONS(2532),
    [anon_sym_mod] = ACTIONS(2532),
    [anon_sym_pub] = ACTIONS(2532),
    [anon_sym_return] = ACTIONS(2532),
    [anon_sym_raw] = ACTIONS(2532),
    [anon_sym_static] = ACTIONS(2532),
    [anon_sym_struct] = ACTIONS(2532),
    [anon_sym_trait] = ACTIONS(2532),
    [anon_sym_type] = ACTIONS(2532),
    [anon_sym_union] = ACTIONS(2532),
    [anon_sym_unsafe] = ACTIONS(2532),
    [anon_sym_use] = ACTIONS(2532),
    [anon_sym_while] = ACTIONS(2532),
    [anon_sym_extern] = ACTIONS(2532),
    [anon_sym_safe] = ACTIONS(2532),
    [anon_sym_yield] = ACTIONS(2532),
    [anon_sym_move] = ACTIONS(2532),
    [anon_sym_try] = ACTIONS(2532),
    [sym_integer_literal] = ACTIONS(2530),
    [aux_sym_string_literal_token1] = ACTIONS(2530),
    [sym_char_literal] = ACTIONS(2530),
    [anon_sym_true] = ACTIONS(2532),
    [anon_sym_false] = ACTIONS(2532),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2532),
    [sym_super] = ACTIONS(2532),
    [sym_crate] = ACTIONS(2532),
    [aux_sym_metavariable_token1] = ACTIONS(2530),
    [sym__raw_string_literal_start] = ACTIONS(2530),
    [sym_float_literal] = ACTIONS(2530),
    [sym__outer_block_doc_comment_start] = ACTIONS(2530),
    [sym__outer_line_doc_comment_start] = ACTIONS(2530),
  },
  [STATE(690)] = {
    [sym_line_comment] = STATE(690),
    [sym_block_comment] = STATE(690),
    [ts_builtin_sym_end] = ACTIONS(2534),
    [sym_identifier] = ACTIONS(2536),
    [anon_sym_SEMI] = ACTIONS(2534),
    [anon_sym_macro] = ACTIONS(2536),
    [anon_sym_LPAREN] = ACTIONS(2534),
    [anon_sym_LBRACE] = ACTIONS(2534),
    [anon_sym_RBRACE] = ACTIONS(2534),
    [anon_sym_macro_rules] = ACTIONS(2536),
    [anon_sym_BANG] = ACTIONS(2534),
    [anon_sym_LBRACK] = ACTIONS(2534),
    [anon_sym_STAR] = ACTIONS(2534),
    [anon_sym_u8] = ACTIONS(2536),
    [anon_sym_i8] = ACTIONS(2536),
    [anon_sym_u16] = ACTIONS(2536),
    [anon_sym_i16] = ACTIONS(2536),
    [anon_sym_u32] = ACTIONS(2536),
    [anon_sym_i32] = ACTIONS(2536),
    [anon_sym_u64] = ACTIONS(2536),
    [anon_sym_i64] = ACTIONS(2536),
    [anon_sym_u128] = ACTIONS(2536),
    [anon_sym_i128] = ACTIONS(2536),
    [anon_sym_isize] = ACTIONS(2536),
    [anon_sym_usize] = ACTIONS(2536),
    [anon_sym_f32] = ACTIONS(2536),
    [anon_sym_f64] = ACTIONS(2536),
    [anon_sym_bool] = ACTIONS(2536),
    [anon_sym_str] = ACTIONS(2536),
    [anon_sym_char] = ACTIONS(2536),
    [anon_sym_DASH] = ACTIONS(2534),
    [anon_sym_AMP] = ACTIONS(2534),
    [anon_sym_PIPE] = ACTIONS(2534),
    [anon_sym_LT] = ACTIONS(2534),
    [anon_sym_DOT_DOT] = ACTIONS(2534),
    [anon_sym_COLON_COLON] = ACTIONS(2534),
    [anon_sym_POUND] = ACTIONS(2534),
    [anon_sym_SQUOTE] = ACTIONS(2536),
    [anon_sym_async] = ACTIONS(2536),
    [anon_sym_auto] = ACTIONS(2536),
    [anon_sym_await] = ACTIONS(2536),
    [anon_sym_become] = ACTIONS(2536),
    [anon_sym_break] = ACTIONS(2536),
    [anon_sym_const] = ACTIONS(2536),
    [anon_sym_continue] = ACTIONS(2536),
    [anon_sym_default] = ACTIONS(2536),
    [anon_sym_enum] = ACTIONS(2536),
    [anon_sym_fn] = ACTIONS(2536),
    [anon_sym_for] = ACTIONS(2536),
    [anon_sym_gen] = ACTIONS(2536),
    [anon_sym_if] = ACTIONS(2536),
    [anon_sym_impl] = ACTIONS(2536),
    [anon_sym_let] = ACTIONS(2536),
    [anon_sym_loop] = ACTIONS(2536),
    [anon_sym_match] = ACTIONS(2536),
    [anon_sym_mod] = ACTIONS(2536),
    [anon_sym_pub] = ACTIONS(2536),
    [anon_sym_return] = ACTIONS(2536),
    [anon_sym_raw] = ACTIONS(2536),
    [anon_sym_static] = ACTIONS(2536),
    [anon_sym_struct] = ACTIONS(2536),
    [anon_sym_trait] = ACTIONS(2536),
    [anon_sym_type] = ACTIONS(2536),
    [anon_sym_union] = ACTIONS(2536),
    [anon_sym_unsafe] = ACTIONS(2536),
    [anon_sym_use] = ACTIONS(2536),
    [anon_sym_while] = ACTIONS(2536),
    [anon_sym_extern] = ACTIONS(2536),
    [anon_sym_safe] = ACTIONS(2536),
    [anon_sym_yield] = ACTIONS(2536),
    [anon_sym_move] = ACTIONS(2536),
    [anon_sym_try] = ACTIONS(2536),
    [sym_integer_literal] = ACTIONS(2534),
    [aux_sym_string_literal_token1] = ACTIONS(2534),
    [sym_char_literal] = ACTIONS(2534),
    [anon_sym_true] = ACTIONS(2536),
    [anon_sym_false] = ACTIONS(2536),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2536),
    [sym_super] = ACTIONS(2536),
    [sym_crate] = ACTIONS(2536),
    [aux_sym_metavariable_token1] = ACTIONS(2534),
    [sym__raw_string_literal_start] = ACTIONS(2534),
    [sym_float_literal] = ACTIONS(2534),
    [sym__outer_block_doc_comment_start] = ACTIONS(2534),
    [sym__outer_line_doc_comment_start] = ACTIONS(2534),
  },
  [STATE(691)] = {
    [sym_line_comment] = STATE(691),
    [sym_block_comment] = STATE(691),
    [ts_builtin_sym_end] = ACTIONS(2538),
    [sym_identifier] = ACTIONS(2540),
    [anon_sym_SEMI] = ACTIONS(2538),
    [anon_sym_macro] = ACTIONS(2540),
    [anon_sym_LPAREN] = ACTIONS(2538),
    [anon_sym_LBRACE] = ACTIONS(2538),
    [anon_sym_RBRACE] = ACTIONS(2538),
    [anon_sym_macro_rules] = ACTIONS(2540),
    [anon_sym_BANG] = ACTIONS(2538),
    [anon_sym_LBRACK] = ACTIONS(2538),
    [anon_sym_STAR] = ACTIONS(2538),
    [anon_sym_u8] = ACTIONS(2540),
    [anon_sym_i8] = ACTIONS(2540),
    [anon_sym_u16] = ACTIONS(2540),
    [anon_sym_i16] = ACTIONS(2540),
    [anon_sym_u32] = ACTIONS(2540),
    [anon_sym_i32] = ACTIONS(2540),
    [anon_sym_u64] = ACTIONS(2540),
    [anon_sym_i64] = ACTIONS(2540),
    [anon_sym_u128] = ACTIONS(2540),
    [anon_sym_i128] = ACTIONS(2540),
    [anon_sym_isize] = ACTIONS(2540),
    [anon_sym_usize] = ACTIONS(2540),
    [anon_sym_f32] = ACTIONS(2540),
    [anon_sym_f64] = ACTIONS(2540),
    [anon_sym_bool] = ACTIONS(2540),
    [anon_sym_str] = ACTIONS(2540),
    [anon_sym_char] = ACTIONS(2540),
    [anon_sym_DASH] = ACTIONS(2538),
    [anon_sym_AMP] = ACTIONS(2538),
    [anon_sym_PIPE] = ACTIONS(2538),
    [anon_sym_LT] = ACTIONS(2538),
    [anon_sym_DOT_DOT] = ACTIONS(2538),
    [anon_sym_COLON_COLON] = ACTIONS(2538),
    [anon_sym_POUND] = ACTIONS(2538),
    [anon_sym_SQUOTE] = ACTIONS(2540),
    [anon_sym_async] = ACTIONS(2540),
    [anon_sym_auto] = ACTIONS(2540),
    [anon_sym_await] = ACTIONS(2540),
    [anon_sym_become] = ACTIONS(2540),
    [anon_sym_break] = ACTIONS(2540),
    [anon_sym_const] = ACTIONS(2540),
    [anon_sym_continue] = ACTIONS(2540),
    [anon_sym_default] = ACTIONS(2540),
    [anon_sym_enum] = ACTIONS(2540),
    [anon_sym_fn] = ACTIONS(2540),
    [anon_sym_for] = ACTIONS(2540),
    [anon_sym_gen] = ACTIONS(2540),
    [anon_sym_if] = ACTIONS(2540),
    [anon_sym_impl] = ACTIONS(2540),
    [anon_sym_let] = ACTIONS(2540),
    [anon_sym_loop] = ACTIONS(2540),
    [anon_sym_match] = ACTIONS(2540),
    [anon_sym_mod] = ACTIONS(2540),
    [anon_sym_pub] = ACTIONS(2540),
    [anon_sym_return] = ACTIONS(2540),
    [anon_sym_raw] = ACTIONS(2540),
    [anon_sym_static] = ACTIONS(2540),
    [anon_sym_struct] = ACTIONS(2540),
    [anon_sym_trait] = ACTIONS(2540),
    [anon_sym_type] = ACTIONS(2540),
    [anon_sym_union] = ACTIONS(2540),
    [anon_sym_unsafe] = ACTIONS(2540),
    [anon_sym_use] = ACTIONS(2540),
    [anon_sym_while] = ACTIONS(2540),
    [anon_sym_extern] = ACTIONS(2540),
    [anon_sym_safe] = ACTIONS(2540),
    [anon_sym_yield] = ACTIONS(2540),
    [anon_sym_move] = ACTIONS(2540),
    [anon_sym_try] = ACTIONS(2540),
    [sym_integer_literal] = ACTIONS(2538),
    [aux_sym_string_literal_token1] = ACTIONS(2538),
    [sym_char_literal] = ACTIONS(2538),
    [anon_sym_true] = ACTIONS(2540),
    [anon_sym_false] = ACTIONS(2540),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2540),
    [sym_super] = ACTIONS(2540),
    [sym_crate] = ACTIONS(2540),
    [aux_sym_metavariable_token1] = ACTIONS(2538),
    [sym__raw_string_literal_start] = ACTIONS(2538),
    [sym_float_literal] = ACTIONS(2538),
    [sym__outer_block_doc_comment_start] = ACTIONS(2538),
    [sym__outer_line_doc_comment_start] = ACTIONS(2538),
  },
  [STATE(692)] = {
    [sym_line_comment] = STATE(692),
    [sym_block_comment] = STATE(692),
    [ts_builtin_sym_end] = ACTIONS(2542),
    [sym_identifier] = ACTIONS(2544),
    [anon_sym_SEMI] = ACTIONS(2542),
    [anon_sym_macro] = ACTIONS(2544),
    [anon_sym_LPAREN] = ACTIONS(2542),
    [anon_sym_LBRACE] = ACTIONS(2542),
    [anon_sym_RBRACE] = ACTIONS(2542),
    [anon_sym_macro_rules] = ACTIONS(2544),
    [anon_sym_BANG] = ACTIONS(2542),
    [anon_sym_LBRACK] = ACTIONS(2542),
    [anon_sym_STAR] = ACTIONS(2542),
    [anon_sym_u8] = ACTIONS(2544),
    [anon_sym_i8] = ACTIONS(2544),
    [anon_sym_u16] = ACTIONS(2544),
    [anon_sym_i16] = ACTIONS(2544),
    [anon_sym_u32] = ACTIONS(2544),
    [anon_sym_i32] = ACTIONS(2544),
    [anon_sym_u64] = ACTIONS(2544),
    [anon_sym_i64] = ACTIONS(2544),
    [anon_sym_u128] = ACTIONS(2544),
    [anon_sym_i128] = ACTIONS(2544),
    [anon_sym_isize] = ACTIONS(2544),
    [anon_sym_usize] = ACTIONS(2544),
    [anon_sym_f32] = ACTIONS(2544),
    [anon_sym_f64] = ACTIONS(2544),
    [anon_sym_bool] = ACTIONS(2544),
    [anon_sym_str] = ACTIONS(2544),
    [anon_sym_char] = ACTIONS(2544),
    [anon_sym_DASH] = ACTIONS(2542),
    [anon_sym_AMP] = ACTIONS(2542),
    [anon_sym_PIPE] = ACTIONS(2542),
    [anon_sym_LT] = ACTIONS(2542),
    [anon_sym_DOT_DOT] = ACTIONS(2542),
    [anon_sym_COLON_COLON] = ACTIONS(2542),
    [anon_sym_POUND] = ACTIONS(2542),
    [anon_sym_SQUOTE] = ACTIONS(2544),
    [anon_sym_async] = ACTIONS(2544),
    [anon_sym_auto] = ACTIONS(2544),
    [anon_sym_await] = ACTIONS(2544),
    [anon_sym_become] = ACTIONS(2544),
    [anon_sym_break] = ACTIONS(2544),
    [anon_sym_const] = ACTIONS(2544),
    [anon_sym_continue] = ACTIONS(2544),
    [anon_sym_default] = ACTIONS(2544),
    [anon_sym_enum] = ACTIONS(2544),
    [anon_sym_fn] = ACTIONS(2544),
    [anon_sym_for] = ACTIONS(2544),
    [anon_sym_gen] = ACTIONS(2544),
    [anon_sym_if] = ACTIONS(2544),
    [anon_sym_impl] = ACTIONS(2544),
    [anon_sym_let] = ACTIONS(2544),
    [anon_sym_loop] = ACTIONS(2544),
    [anon_sym_match] = ACTIONS(2544),
    [anon_sym_mod] = ACTIONS(2544),
    [anon_sym_pub] = ACTIONS(2544),
    [anon_sym_return] = ACTIONS(2544),
    [anon_sym_raw] = ACTIONS(2544),
    [anon_sym_static] = ACTIONS(2544),
    [anon_sym_struct] = ACTIONS(2544),
    [anon_sym_trait] = ACTIONS(2544),
    [anon_sym_type] = ACTIONS(2544),
    [anon_sym_union] = ACTIONS(2544),
    [anon_sym_unsafe] = ACTIONS(2544),
    [anon_sym_use] = ACTIONS(2544),
    [anon_sym_while] = ACTIONS(2544),
    [anon_sym_extern] = ACTIONS(2544),
    [anon_sym_safe] = ACTIONS(2544),
    [anon_sym_yield] = ACTIONS(2544),
    [anon_sym_move] = ACTIONS(2544),
    [anon_sym_try] = ACTIONS(2544),
    [sym_integer_literal] = ACTIONS(2542),
    [aux_sym_string_literal_token1] = ACTIONS(2542),
    [sym_char_literal] = ACTIONS(2542),
    [anon_sym_true] = ACTIONS(2544),
    [anon_sym_false] = ACTIONS(2544),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2544),
    [sym_super] = ACTIONS(2544),
    [sym_crate] = ACTIONS(2544),
    [aux_sym_metavariable_token1] = ACTIONS(2542),
    [sym__raw_string_literal_start] = ACTIONS(2542),
    [sym_float_literal] = ACTIONS(2542),
    [sym__outer_block_doc_comment_start] = ACTIONS(2542),
    [sym__outer_line_doc_comment_start] = ACTIONS(2542),
  },
  [STATE(693)] = {
    [sym_line_comment] = STATE(693),
    [sym_block_comment] = STATE(693),
    [ts_builtin_sym_end] = ACTIONS(2546),
    [sym_identifier] = ACTIONS(2548),
    [anon_sym_SEMI] = ACTIONS(2546),
    [anon_sym_macro] = ACTIONS(2548),
    [anon_sym_LPAREN] = ACTIONS(2546),
    [anon_sym_LBRACE] = ACTIONS(2546),
    [anon_sym_RBRACE] = ACTIONS(2546),
    [anon_sym_macro_rules] = ACTIONS(2548),
    [anon_sym_BANG] = ACTIONS(2546),
    [anon_sym_LBRACK] = ACTIONS(2546),
    [anon_sym_STAR] = ACTIONS(2546),
    [anon_sym_u8] = ACTIONS(2548),
    [anon_sym_i8] = ACTIONS(2548),
    [anon_sym_u16] = ACTIONS(2548),
    [anon_sym_i16] = ACTIONS(2548),
    [anon_sym_u32] = ACTIONS(2548),
    [anon_sym_i32] = ACTIONS(2548),
    [anon_sym_u64] = ACTIONS(2548),
    [anon_sym_i64] = ACTIONS(2548),
    [anon_sym_u128] = ACTIONS(2548),
    [anon_sym_i128] = ACTIONS(2548),
    [anon_sym_isize] = ACTIONS(2548),
    [anon_sym_usize] = ACTIONS(2548),
    [anon_sym_f32] = ACTIONS(2548),
    [anon_sym_f64] = ACTIONS(2548),
    [anon_sym_bool] = ACTIONS(2548),
    [anon_sym_str] = ACTIONS(2548),
    [anon_sym_char] = ACTIONS(2548),
    [anon_sym_DASH] = ACTIONS(2546),
    [anon_sym_AMP] = ACTIONS(2546),
    [anon_sym_PIPE] = ACTIONS(2546),
    [anon_sym_LT] = ACTIONS(2546),
    [anon_sym_DOT_DOT] = ACTIONS(2546),
    [anon_sym_COLON_COLON] = ACTIONS(2546),
    [anon_sym_POUND] = ACTIONS(2546),
    [anon_sym_SQUOTE] = ACTIONS(2548),
    [anon_sym_async] = ACTIONS(2548),
    [anon_sym_auto] = ACTIONS(2548),
    [anon_sym_await] = ACTIONS(2548),
    [anon_sym_become] = ACTIONS(2548),
    [anon_sym_break] = ACTIONS(2548),
    [anon_sym_const] = ACTIONS(2548),
    [anon_sym_continue] = ACTIONS(2548),
    [anon_sym_default] = ACTIONS(2548),
    [anon_sym_enum] = ACTIONS(2548),
    [anon_sym_fn] = ACTIONS(2548),
    [anon_sym_for] = ACTIONS(2548),
    [anon_sym_gen] = ACTIONS(2548),
    [anon_sym_if] = ACTIONS(2548),
    [anon_sym_impl] = ACTIONS(2548),
    [anon_sym_let] = ACTIONS(2548),
    [anon_sym_loop] = ACTIONS(2548),
    [anon_sym_match] = ACTIONS(2548),
    [anon_sym_mod] = ACTIONS(2548),
    [anon_sym_pub] = ACTIONS(2548),
    [anon_sym_return] = ACTIONS(2548),
    [anon_sym_raw] = ACTIONS(2548),
    [anon_sym_static] = ACTIONS(2548),
    [anon_sym_struct] = ACTIONS(2548),
    [anon_sym_trait] = ACTIONS(2548),
    [anon_sym_type] = ACTIONS(2548),
    [anon_sym_union] = ACTIONS(2548),
    [anon_sym_unsafe] = ACTIONS(2548),
    [anon_sym_use] = ACTIONS(2548),
    [anon_sym_while] = ACTIONS(2548),
    [anon_sym_extern] = ACTIONS(2548),
    [anon_sym_safe] = ACTIONS(2548),
    [anon_sym_yield] = ACTIONS(2548),
    [anon_sym_move] = ACTIONS(2548),
    [anon_sym_try] = ACTIONS(2548),
    [sym_integer_literal] = ACTIONS(2546),
    [aux_sym_string_literal_token1] = ACTIONS(2546),
    [sym_char_literal] = ACTIONS(2546),
    [anon_sym_true] = ACTIONS(2548),
    [anon_sym_false] = ACTIONS(2548),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2548),
    [sym_super] = ACTIONS(2548),
    [sym_crate] = ACTIONS(2548),
    [aux_sym_metavariable_token1] = ACTIONS(2546),
    [sym__raw_string_literal_start] = ACTIONS(2546),
    [sym_float_literal] = ACTIONS(2546),
    [sym__outer_block_doc_comment_start] = ACTIONS(2546),
    [sym__outer_line_doc_comment_start] = ACTIONS(2546),
  },
  [STATE(694)] = {
    [sym_line_comment] = STATE(694),
    [sym_block_comment] = STATE(694),
    [ts_builtin_sym_end] = ACTIONS(2550),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_SEMI] = ACTIONS(2550),
    [anon_sym_macro] = ACTIONS(2552),
    [anon_sym_LPAREN] = ACTIONS(2550),
    [anon_sym_LBRACE] = ACTIONS(2550),
    [anon_sym_RBRACE] = ACTIONS(2550),
    [anon_sym_macro_rules] = ACTIONS(2552),
    [anon_sym_BANG] = ACTIONS(2550),
    [anon_sym_LBRACK] = ACTIONS(2550),
    [anon_sym_STAR] = ACTIONS(2550),
    [anon_sym_u8] = ACTIONS(2552),
    [anon_sym_i8] = ACTIONS(2552),
    [anon_sym_u16] = ACTIONS(2552),
    [anon_sym_i16] = ACTIONS(2552),
    [anon_sym_u32] = ACTIONS(2552),
    [anon_sym_i32] = ACTIONS(2552),
    [anon_sym_u64] = ACTIONS(2552),
    [anon_sym_i64] = ACTIONS(2552),
    [anon_sym_u128] = ACTIONS(2552),
    [anon_sym_i128] = ACTIONS(2552),
    [anon_sym_isize] = ACTIONS(2552),
    [anon_sym_usize] = ACTIONS(2552),
    [anon_sym_f32] = ACTIONS(2552),
    [anon_sym_f64] = ACTIONS(2552),
    [anon_sym_bool] = ACTIONS(2552),
    [anon_sym_str] = ACTIONS(2552),
    [anon_sym_char] = ACTIONS(2552),
    [anon_sym_DASH] = ACTIONS(2550),
    [anon_sym_AMP] = ACTIONS(2550),
    [anon_sym_PIPE] = ACTIONS(2550),
    [anon_sym_LT] = ACTIONS(2550),
    [anon_sym_DOT_DOT] = ACTIONS(2550),
    [anon_sym_COLON_COLON] = ACTIONS(2550),
    [anon_sym_POUND] = ACTIONS(2550),
    [anon_sym_SQUOTE] = ACTIONS(2552),
    [anon_sym_async] = ACTIONS(2552),
    [anon_sym_auto] = ACTIONS(2552),
    [anon_sym_await] = ACTIONS(2552),
    [anon_sym_become] = ACTIONS(2552),
    [anon_sym_break] = ACTIONS(2552),
    [anon_sym_const] = ACTIONS(2552),
    [anon_sym_continue] = ACTIONS(2552),
    [anon_sym_default] = ACTIONS(2552),
    [anon_sym_enum] = ACTIONS(2552),
    [anon_sym_fn] = ACTIONS(2552),
    [anon_sym_for] = ACTIONS(2552),
    [anon_sym_gen] = ACTIONS(2552),
    [anon_sym_if] = ACTIONS(2552),
    [anon_sym_impl] = ACTIONS(2552),
    [anon_sym_let] = ACTIONS(2552),
    [anon_sym_loop] = ACTIONS(2552),
    [anon_sym_match] = ACTIONS(2552),
    [anon_sym_mod] = ACTIONS(2552),
    [anon_sym_pub] = ACTIONS(2552),
    [anon_sym_return] = ACTIONS(2552),
    [anon_sym_raw] = ACTIONS(2552),
    [anon_sym_static] = ACTIONS(2552),
    [anon_sym_struct] = ACTIONS(2552),
    [anon_sym_trait] = ACTIONS(2552),
    [anon_sym_type] = ACTIONS(2552),
    [anon_sym_union] = ACTIONS(2552),
    [anon_sym_unsafe] = ACTIONS(2552),
    [anon_sym_use] = ACTIONS(2552),
    [anon_sym_while] = ACTIONS(2552),
    [anon_sym_extern] = ACTIONS(2552),
    [anon_sym_safe] = ACTIONS(2552),
    [anon_sym_yield] = ACTIONS(2552),
    [anon_sym_move] = ACTIONS(2552),
    [anon_sym_try] = ACTIONS(2552),
    [sym_integer_literal] = ACTIONS(2550),
    [aux_sym_string_literal_token1] = ACTIONS(2550),
    [sym_char_literal] = ACTIONS(2550),
    [anon_sym_true] = ACTIONS(2552),
    [anon_sym_false] = ACTIONS(2552),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2552),
    [sym_super] = ACTIONS(2552),
    [sym_crate] = ACTIONS(2552),
    [aux_sym_metavariable_token1] = ACTIONS(2550),
    [sym__raw_string_literal_start] = ACTIONS(2550),
    [sym_float_literal] = ACTIONS(2550),
    [sym__outer_block_doc_comment_start] = ACTIONS(2550),
    [sym__outer_line_doc_comment_start] = ACTIONS(2550),
  },
  [STATE(695)] = {
    [sym_line_comment] = STATE(695),
    [sym_block_comment] = STATE(695),
    [ts_builtin_sym_end] = ACTIONS(2554),
    [sym_identifier] = ACTIONS(2556),
    [anon_sym_SEMI] = ACTIONS(2554),
    [anon_sym_macro] = ACTIONS(2556),
    [anon_sym_LPAREN] = ACTIONS(2554),
    [anon_sym_LBRACE] = ACTIONS(2554),
    [anon_sym_RBRACE] = ACTIONS(2554),
    [anon_sym_macro_rules] = ACTIONS(2556),
    [anon_sym_BANG] = ACTIONS(2554),
    [anon_sym_LBRACK] = ACTIONS(2554),
    [anon_sym_STAR] = ACTIONS(2554),
    [anon_sym_u8] = ACTIONS(2556),
    [anon_sym_i8] = ACTIONS(2556),
    [anon_sym_u16] = ACTIONS(2556),
    [anon_sym_i16] = ACTIONS(2556),
    [anon_sym_u32] = ACTIONS(2556),
    [anon_sym_i32] = ACTIONS(2556),
    [anon_sym_u64] = ACTIONS(2556),
    [anon_sym_i64] = ACTIONS(2556),
    [anon_sym_u128] = ACTIONS(2556),
    [anon_sym_i128] = ACTIONS(2556),
    [anon_sym_isize] = ACTIONS(2556),
    [anon_sym_usize] = ACTIONS(2556),
    [anon_sym_f32] = ACTIONS(2556),
    [anon_sym_f64] = ACTIONS(2556),
    [anon_sym_bool] = ACTIONS(2556),
    [anon_sym_str] = ACTIONS(2556),
    [anon_sym_char] = ACTIONS(2556),
    [anon_sym_DASH] = ACTIONS(2554),
    [anon_sym_AMP] = ACTIONS(2554),
    [anon_sym_PIPE] = ACTIONS(2554),
    [anon_sym_LT] = ACTIONS(2554),
    [anon_sym_DOT_DOT] = ACTIONS(2554),
    [anon_sym_COLON_COLON] = ACTIONS(2554),
    [anon_sym_POUND] = ACTIONS(2554),
    [anon_sym_SQUOTE] = ACTIONS(2556),
    [anon_sym_async] = ACTIONS(2556),
    [anon_sym_auto] = ACTIONS(2556),
    [anon_sym_await] = ACTIONS(2556),
    [anon_sym_become] = ACTIONS(2556),
    [anon_sym_break] = ACTIONS(2556),
    [anon_sym_const] = ACTIONS(2556),
    [anon_sym_continue] = ACTIONS(2556),
    [anon_sym_default] = ACTIONS(2556),
    [anon_sym_enum] = ACTIONS(2556),
    [anon_sym_fn] = ACTIONS(2556),
    [anon_sym_for] = ACTIONS(2556),
    [anon_sym_gen] = ACTIONS(2556),
    [anon_sym_if] = ACTIONS(2556),
    [anon_sym_impl] = ACTIONS(2556),
    [anon_sym_let] = ACTIONS(2556),
    [anon_sym_loop] = ACTIONS(2556),
    [anon_sym_match] = ACTIONS(2556),
    [anon_sym_mod] = ACTIONS(2556),
    [anon_sym_pub] = ACTIONS(2556),
    [anon_sym_return] = ACTIONS(2556),
    [anon_sym_raw] = ACTIONS(2556),
    [anon_sym_static] = ACTIONS(2556),
    [anon_sym_struct] = ACTIONS(2556),
    [anon_sym_trait] = ACTIONS(2556),
    [anon_sym_type] = ACTIONS(2556),
    [anon_sym_union] = ACTIONS(2556),
    [anon_sym_unsafe] = ACTIONS(2556),
    [anon_sym_use] = ACTIONS(2556),
    [anon_sym_while] = ACTIONS(2556),
    [anon_sym_extern] = ACTIONS(2556),
    [anon_sym_safe] = ACTIONS(2556),
    [anon_sym_yield] = ACTIONS(2556),
    [anon_sym_move] = ACTIONS(2556),
    [anon_sym_try] = ACTIONS(2556),
    [sym_integer_literal] = ACTIONS(2554),
    [aux_sym_string_literal_token1] = ACTIONS(2554),
    [sym_char_literal] = ACTIONS(2554),
    [anon_sym_true] = ACTIONS(2556),
    [anon_sym_false] = ACTIONS(2556),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2556),
    [sym_super] = ACTIONS(2556),
    [sym_crate] = ACTIONS(2556),
    [aux_sym_metavariable_token1] = ACTIONS(2554),
    [sym__raw_string_literal_start] = ACTIONS(2554),
    [sym_float_literal] = ACTIONS(2554),
    [sym__outer_block_doc_comment_start] = ACTIONS(2554),
    [sym__outer_line_doc_comment_start] = ACTIONS(2554),
  },
  [STATE(696)] = {
    [sym_line_comment] = STATE(696),
    [sym_block_comment] = STATE(696),
    [ts_builtin_sym_end] = ACTIONS(2558),
    [sym_identifier] = ACTIONS(2560),
    [anon_sym_SEMI] = ACTIONS(2558),
    [anon_sym_macro] = ACTIONS(2560),
    [anon_sym_LPAREN] = ACTIONS(2558),
    [anon_sym_LBRACE] = ACTIONS(2558),
    [anon_sym_RBRACE] = ACTIONS(2558),
    [anon_sym_macro_rules] = ACTIONS(2560),
    [anon_sym_BANG] = ACTIONS(2558),
    [anon_sym_LBRACK] = ACTIONS(2558),
    [anon_sym_STAR] = ACTIONS(2558),
    [anon_sym_u8] = ACTIONS(2560),
    [anon_sym_i8] = ACTIONS(2560),
    [anon_sym_u16] = ACTIONS(2560),
    [anon_sym_i16] = ACTIONS(2560),
    [anon_sym_u32] = ACTIONS(2560),
    [anon_sym_i32] = ACTIONS(2560),
    [anon_sym_u64] = ACTIONS(2560),
    [anon_sym_i64] = ACTIONS(2560),
    [anon_sym_u128] = ACTIONS(2560),
    [anon_sym_i128] = ACTIONS(2560),
    [anon_sym_isize] = ACTIONS(2560),
    [anon_sym_usize] = ACTIONS(2560),
    [anon_sym_f32] = ACTIONS(2560),
    [anon_sym_f64] = ACTIONS(2560),
    [anon_sym_bool] = ACTIONS(2560),
    [anon_sym_str] = ACTIONS(2560),
    [anon_sym_char] = ACTIONS(2560),
    [anon_sym_DASH] = ACTIONS(2558),
    [anon_sym_AMP] = ACTIONS(2558),
    [anon_sym_PIPE] = ACTIONS(2558),
    [anon_sym_LT] = ACTIONS(2558),
    [anon_sym_DOT_DOT] = ACTIONS(2558),
    [anon_sym_COLON_COLON] = ACTIONS(2558),
    [anon_sym_POUND] = ACTIONS(2558),
    [anon_sym_SQUOTE] = ACTIONS(2560),
    [anon_sym_async] = ACTIONS(2560),
    [anon_sym_auto] = ACTIONS(2560),
    [anon_sym_await] = ACTIONS(2560),
    [anon_sym_become] = ACTIONS(2560),
    [anon_sym_break] = ACTIONS(2560),
    [anon_sym_const] = ACTIONS(2560),
    [anon_sym_continue] = ACTIONS(2560),
    [anon_sym_default] = ACTIONS(2560),
    [anon_sym_enum] = ACTIONS(2560),
    [anon_sym_fn] = ACTIONS(2560),
    [anon_sym_for] = ACTIONS(2560),
    [anon_sym_gen] = ACTIONS(2560),
    [anon_sym_if] = ACTIONS(2560),
    [anon_sym_impl] = ACTIONS(2560),
    [anon_sym_let] = ACTIONS(2560),
    [anon_sym_loop] = ACTIONS(2560),
    [anon_sym_match] = ACTIONS(2560),
    [anon_sym_mod] = ACTIONS(2560),
    [anon_sym_pub] = ACTIONS(2560),
    [anon_sym_return] = ACTIONS(2560),
    [anon_sym_raw] = ACTIONS(2560),
    [anon_sym_static] = ACTIONS(2560),
    [anon_sym_struct] = ACTIONS(2560),
    [anon_sym_trait] = ACTIONS(2560),
    [anon_sym_type] = ACTIONS(2560),
    [anon_sym_union] = ACTIONS(2560),
    [anon_sym_unsafe] = ACTIONS(2560),
    [anon_sym_use] = ACTIONS(2560),
    [anon_sym_while] = ACTIONS(2560),
    [anon_sym_extern] = ACTIONS(2560),
    [anon_sym_safe] = ACTIONS(2560),
    [anon_sym_yield] = ACTIONS(2560),
    [anon_sym_move] = ACTIONS(2560),
    [anon_sym_try] = ACTIONS(2560),
    [sym_integer_literal] = ACTIONS(2558),
    [aux_sym_string_literal_token1] = ACTIONS(2558),
    [sym_char_literal] = ACTIONS(2558),
    [anon_sym_true] = ACTIONS(2560),
    [anon_sym_false] = ACTIONS(2560),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2560),
    [sym_super] = ACTIONS(2560),
    [sym_crate] = ACTIONS(2560),
    [aux_sym_metavariable_token1] = ACTIONS(2558),
    [sym__raw_string_literal_start] = ACTIONS(2558),
    [sym_float_literal] = ACTIONS(2558),
    [sym__outer_block_doc_comment_start] = ACTIONS(2558),
    [sym__outer_line_doc_comment_start] = ACTIONS(2558),
  },
  [STATE(697)] = {
    [sym_line_comment] = STATE(697),
    [sym_block_comment] = STATE(697),
    [ts_builtin_sym_end] = ACTIONS(2562),
    [sym_identifier] = ACTIONS(2564),
    [anon_sym_SEMI] = ACTIONS(2562),
    [anon_sym_macro] = ACTIONS(2564),
    [anon_sym_LPAREN] = ACTIONS(2562),
    [anon_sym_LBRACE] = ACTIONS(2562),
    [anon_sym_RBRACE] = ACTIONS(2562),
    [anon_sym_macro_rules] = ACTIONS(2564),
    [anon_sym_BANG] = ACTIONS(2562),
    [anon_sym_LBRACK] = ACTIONS(2562),
    [anon_sym_STAR] = ACTIONS(2562),
    [anon_sym_u8] = ACTIONS(2564),
    [anon_sym_i8] = ACTIONS(2564),
    [anon_sym_u16] = ACTIONS(2564),
    [anon_sym_i16] = ACTIONS(2564),
    [anon_sym_u32] = ACTIONS(2564),
    [anon_sym_i32] = ACTIONS(2564),
    [anon_sym_u64] = ACTIONS(2564),
    [anon_sym_i64] = ACTIONS(2564),
    [anon_sym_u128] = ACTIONS(2564),
    [anon_sym_i128] = ACTIONS(2564),
    [anon_sym_isize] = ACTIONS(2564),
    [anon_sym_usize] = ACTIONS(2564),
    [anon_sym_f32] = ACTIONS(2564),
    [anon_sym_f64] = ACTIONS(2564),
    [anon_sym_bool] = ACTIONS(2564),
    [anon_sym_str] = ACTIONS(2564),
    [anon_sym_char] = ACTIONS(2564),
    [anon_sym_DASH] = ACTIONS(2562),
    [anon_sym_AMP] = ACTIONS(2562),
    [anon_sym_PIPE] = ACTIONS(2562),
    [anon_sym_LT] = ACTIONS(2562),
    [anon_sym_DOT_DOT] = ACTIONS(2562),
    [anon_sym_COLON_COLON] = ACTIONS(2562),
    [anon_sym_POUND] = ACTIONS(2562),
    [anon_sym_SQUOTE] = ACTIONS(2564),
    [anon_sym_async] = ACTIONS(2564),
    [anon_sym_auto] = ACTIONS(2564),
    [anon_sym_await] = ACTIONS(2564),
    [anon_sym_become] = ACTIONS(2564),
    [anon_sym_break] = ACTIONS(2564),
    [anon_sym_const] = ACTIONS(2564),
    [anon_sym_continue] = ACTIONS(2564),
    [anon_sym_default] = ACTIONS(2564),
    [anon_sym_enum] = ACTIONS(2564),
    [anon_sym_fn] = ACTIONS(2564),
    [anon_sym_for] = ACTIONS(2564),
    [anon_sym_gen] = ACTIONS(2564),
    [anon_sym_if] = ACTIONS(2564),
    [anon_sym_impl] = ACTIONS(2564),
    [anon_sym_let] = ACTIONS(2564),
    [anon_sym_loop] = ACTIONS(2564),
    [anon_sym_match] = ACTIONS(2564),
    [anon_sym_mod] = ACTIONS(2564),
    [anon_sym_pub] = ACTIONS(2564),
    [anon_sym_return] = ACTIONS(2564),
    [anon_sym_raw] = ACTIONS(2564),
    [anon_sym_static] = ACTIONS(2564),
    [anon_sym_struct] = ACTIONS(2564),
    [anon_sym_trait] = ACTIONS(2564),
    [anon_sym_type] = ACTIONS(2564),
    [anon_sym_union] = ACTIONS(2564),
    [anon_sym_unsafe] = ACTIONS(2564),
    [anon_sym_use] = ACTIONS(2564),
    [anon_sym_while] = ACTIONS(2564),
    [anon_sym_extern] = ACTIONS(2564),
    [anon_sym_safe] = ACTIONS(2564),
    [anon_sym_yield] = ACTIONS(2564),
    [anon_sym_move] = ACTIONS(2564),
    [anon_sym_try] = ACTIONS(2564),
    [sym_integer_literal] = ACTIONS(2562),
    [aux_sym_string_literal_token1] = ACTIONS(2562),
    [sym_char_literal] = ACTIONS(2562),
    [anon_sym_true] = ACTIONS(2564),
    [anon_sym_false] = ACTIONS(2564),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2564),
    [sym_super] = ACTIONS(2564),
    [sym_crate] = ACTIONS(2564),
    [aux_sym_metavariable_token1] = ACTIONS(2562),
    [sym__raw_string_literal_start] = ACTIONS(2562),
    [sym_float_literal] = ACTIONS(2562),
    [sym__outer_block_doc_comment_start] = ACTIONS(2562),
    [sym__outer_line_doc_comment_start] = ACTIONS(2562),
  },
  [STATE(698)] = {
    [sym_line_comment] = STATE(698),
    [sym_block_comment] = STATE(698),
    [ts_builtin_sym_end] = ACTIONS(2566),
    [sym_identifier] = ACTIONS(2568),
    [anon_sym_SEMI] = ACTIONS(2566),
    [anon_sym_macro] = ACTIONS(2568),
    [anon_sym_LPAREN] = ACTIONS(2566),
    [anon_sym_LBRACE] = ACTIONS(2566),
    [anon_sym_RBRACE] = ACTIONS(2566),
    [anon_sym_macro_rules] = ACTIONS(2568),
    [anon_sym_BANG] = ACTIONS(2566),
    [anon_sym_LBRACK] = ACTIONS(2566),
    [anon_sym_STAR] = ACTIONS(2566),
    [anon_sym_u8] = ACTIONS(2568),
    [anon_sym_i8] = ACTIONS(2568),
    [anon_sym_u16] = ACTIONS(2568),
    [anon_sym_i16] = ACTIONS(2568),
    [anon_sym_u32] = ACTIONS(2568),
    [anon_sym_i32] = ACTIONS(2568),
    [anon_sym_u64] = ACTIONS(2568),
    [anon_sym_i64] = ACTIONS(2568),
    [anon_sym_u128] = ACTIONS(2568),
    [anon_sym_i128] = ACTIONS(2568),
    [anon_sym_isize] = ACTIONS(2568),
    [anon_sym_usize] = ACTIONS(2568),
    [anon_sym_f32] = ACTIONS(2568),
    [anon_sym_f64] = ACTIONS(2568),
    [anon_sym_bool] = ACTIONS(2568),
    [anon_sym_str] = ACTIONS(2568),
    [anon_sym_char] = ACTIONS(2568),
    [anon_sym_DASH] = ACTIONS(2566),
    [anon_sym_AMP] = ACTIONS(2566),
    [anon_sym_PIPE] = ACTIONS(2566),
    [anon_sym_LT] = ACTIONS(2566),
    [anon_sym_DOT_DOT] = ACTIONS(2566),
    [anon_sym_COLON_COLON] = ACTIONS(2566),
    [anon_sym_POUND] = ACTIONS(2566),
    [anon_sym_SQUOTE] = ACTIONS(2568),
    [anon_sym_async] = ACTIONS(2568),
    [anon_sym_auto] = ACTIONS(2568),
    [anon_sym_await] = ACTIONS(2568),
    [anon_sym_become] = ACTIONS(2568),
    [anon_sym_break] = ACTIONS(2568),
    [anon_sym_const] = ACTIONS(2568),
    [anon_sym_continue] = ACTIONS(2568),
    [anon_sym_default] = ACTIONS(2568),
    [anon_sym_enum] = ACTIONS(2568),
    [anon_sym_fn] = ACTIONS(2568),
    [anon_sym_for] = ACTIONS(2568),
    [anon_sym_gen] = ACTIONS(2568),
    [anon_sym_if] = ACTIONS(2568),
    [anon_sym_impl] = ACTIONS(2568),
    [anon_sym_let] = ACTIONS(2568),
    [anon_sym_loop] = ACTIONS(2568),
    [anon_sym_match] = ACTIONS(2568),
    [anon_sym_mod] = ACTIONS(2568),
    [anon_sym_pub] = ACTIONS(2568),
    [anon_sym_return] = ACTIONS(2568),
    [anon_sym_raw] = ACTIONS(2568),
    [anon_sym_static] = ACTIONS(2568),
    [anon_sym_struct] = ACTIONS(2568),
    [anon_sym_trait] = ACTIONS(2568),
    [anon_sym_type] = ACTIONS(2568),
    [anon_sym_union] = ACTIONS(2568),
    [anon_sym_unsafe] = ACTIONS(2568),
    [anon_sym_use] = ACTIONS(2568),
    [anon_sym_while] = ACTIONS(2568),
    [anon_sym_extern] = ACTIONS(2568),
    [anon_sym_safe] = ACTIONS(2568),
    [anon_sym_yield] = ACTIONS(2568),
    [anon_sym_move] = ACTIONS(2568),
    [anon_sym_try] = ACTIONS(2568),
    [sym_integer_literal] = ACTIONS(2566),
    [aux_sym_string_literal_token1] = ACTIONS(2566),
    [sym_char_literal] = ACTIONS(2566),
    [anon_sym_true] = ACTIONS(2568),
    [anon_sym_false] = ACTIONS(2568),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2568),
    [sym_super] = ACTIONS(2568),
    [sym_crate] = ACTIONS(2568),
    [aux_sym_metavariable_token1] = ACTIONS(2566),
    [sym__raw_string_literal_start] = ACTIONS(2566),
    [sym_float_literal] = ACTIONS(2566),
    [sym__outer_block_doc_comment_start] = ACTIONS(2566),
    [sym__outer_line_doc_comment_start] = ACTIONS(2566),
  },
  [STATE(699)] = {
    [sym_line_comment] = STATE(699),
    [sym_block_comment] = STATE(699),
    [ts_builtin_sym_end] = ACTIONS(2570),
    [sym_identifier] = ACTIONS(2572),
    [anon_sym_SEMI] = ACTIONS(2570),
    [anon_sym_macro] = ACTIONS(2572),
    [anon_sym_LPAREN] = ACTIONS(2570),
    [anon_sym_LBRACE] = ACTIONS(2570),
    [anon_sym_RBRACE] = ACTIONS(2570),
    [anon_sym_macro_rules] = ACTIONS(2572),
    [anon_sym_BANG] = ACTIONS(2570),
    [anon_sym_LBRACK] = ACTIONS(2570),
    [anon_sym_STAR] = ACTIONS(2570),
    [anon_sym_u8] = ACTIONS(2572),
    [anon_sym_i8] = ACTIONS(2572),
    [anon_sym_u16] = ACTIONS(2572),
    [anon_sym_i16] = ACTIONS(2572),
    [anon_sym_u32] = ACTIONS(2572),
    [anon_sym_i32] = ACTIONS(2572),
    [anon_sym_u64] = ACTIONS(2572),
    [anon_sym_i64] = ACTIONS(2572),
    [anon_sym_u128] = ACTIONS(2572),
    [anon_sym_i128] = ACTIONS(2572),
    [anon_sym_isize] = ACTIONS(2572),
    [anon_sym_usize] = ACTIONS(2572),
    [anon_sym_f32] = ACTIONS(2572),
    [anon_sym_f64] = ACTIONS(2572),
    [anon_sym_bool] = ACTIONS(2572),
    [anon_sym_str] = ACTIONS(2572),
    [anon_sym_char] = ACTIONS(2572),
    [anon_sym_DASH] = ACTIONS(2570),
    [anon_sym_AMP] = ACTIONS(2570),
    [anon_sym_PIPE] = ACTIONS(2570),
    [anon_sym_LT] = ACTIONS(2570),
    [anon_sym_DOT_DOT] = ACTIONS(2570),
    [anon_sym_COLON_COLON] = ACTIONS(2570),
    [anon_sym_POUND] = ACTIONS(2570),
    [anon_sym_SQUOTE] = ACTIONS(2572),
    [anon_sym_async] = ACTIONS(2572),
    [anon_sym_auto] = ACTIONS(2572),
    [anon_sym_await] = ACTIONS(2572),
    [anon_sym_become] = ACTIONS(2572),
    [anon_sym_break] = ACTIONS(2572),
    [anon_sym_const] = ACTIONS(2572),
    [anon_sym_continue] = ACTIONS(2572),
    [anon_sym_default] = ACTIONS(2572),
    [anon_sym_enum] = ACTIONS(2572),
    [anon_sym_fn] = ACTIONS(2572),
    [anon_sym_for] = ACTIONS(2572),
    [anon_sym_gen] = ACTIONS(2572),
    [anon_sym_if] = ACTIONS(2572),
    [anon_sym_impl] = ACTIONS(2572),
    [anon_sym_let] = ACTIONS(2572),
    [anon_sym_loop] = ACTIONS(2572),
    [anon_sym_match] = ACTIONS(2572),
    [anon_sym_mod] = ACTIONS(2572),
    [anon_sym_pub] = ACTIONS(2572),
    [anon_sym_return] = ACTIONS(2572),
    [anon_sym_raw] = ACTIONS(2572),
    [anon_sym_static] = ACTIONS(2572),
    [anon_sym_struct] = ACTIONS(2572),
    [anon_sym_trait] = ACTIONS(2572),
    [anon_sym_type] = ACTIONS(2572),
    [anon_sym_union] = ACTIONS(2572),
    [anon_sym_unsafe] = ACTIONS(2572),
    [anon_sym_use] = ACTIONS(2572),
    [anon_sym_while] = ACTIONS(2572),
    [anon_sym_extern] = ACTIONS(2572),
    [anon_sym_safe] = ACTIONS(2572),
    [anon_sym_yield] = ACTIONS(2572),
    [anon_sym_move] = ACTIONS(2572),
    [anon_sym_try] = ACTIONS(2572),
    [sym_integer_literal] = ACTIONS(2570),
    [aux_sym_string_literal_token1] = ACTIONS(2570),
    [sym_char_literal] = ACTIONS(2570),
    [anon_sym_true] = ACTIONS(2572),
    [anon_sym_false] = ACTIONS(2572),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2572),
    [sym_super] = ACTIONS(2572),
    [sym_crate] = ACTIONS(2572),
    [aux_sym_metavariable_token1] = ACTIONS(2570),
    [sym__raw_string_literal_start] = ACTIONS(2570),
    [sym_float_literal] = ACTIONS(2570),
    [sym__outer_block_doc_comment_start] = ACTIONS(2570),
    [sym__outer_line_doc_comment_start] = ACTIONS(2570),
  },
  [STATE(700)] = {
    [sym_line_comment] = STATE(700),
    [sym_block_comment] = STATE(700),
    [ts_builtin_sym_end] = ACTIONS(2574),
    [sym_identifier] = ACTIONS(2576),
    [anon_sym_SEMI] = ACTIONS(2574),
    [anon_sym_macro] = ACTIONS(2576),
    [anon_sym_LPAREN] = ACTIONS(2574),
    [anon_sym_LBRACE] = ACTIONS(2574),
    [anon_sym_RBRACE] = ACTIONS(2574),
    [anon_sym_macro_rules] = ACTIONS(2576),
    [anon_sym_BANG] = ACTIONS(2574),
    [anon_sym_LBRACK] = ACTIONS(2574),
    [anon_sym_STAR] = ACTIONS(2574),
    [anon_sym_u8] = ACTIONS(2576),
    [anon_sym_i8] = ACTIONS(2576),
    [anon_sym_u16] = ACTIONS(2576),
    [anon_sym_i16] = ACTIONS(2576),
    [anon_sym_u32] = ACTIONS(2576),
    [anon_sym_i32] = ACTIONS(2576),
    [anon_sym_u64] = ACTIONS(2576),
    [anon_sym_i64] = ACTIONS(2576),
    [anon_sym_u128] = ACTIONS(2576),
    [anon_sym_i128] = ACTIONS(2576),
    [anon_sym_isize] = ACTIONS(2576),
    [anon_sym_usize] = ACTIONS(2576),
    [anon_sym_f32] = ACTIONS(2576),
    [anon_sym_f64] = ACTIONS(2576),
    [anon_sym_bool] = ACTIONS(2576),
    [anon_sym_str] = ACTIONS(2576),
    [anon_sym_char] = ACTIONS(2576),
    [anon_sym_DASH] = ACTIONS(2574),
    [anon_sym_AMP] = ACTIONS(2574),
    [anon_sym_PIPE] = ACTIONS(2574),
    [anon_sym_LT] = ACTIONS(2574),
    [anon_sym_DOT_DOT] = ACTIONS(2574),
    [anon_sym_COLON_COLON] = ACTIONS(2574),
    [anon_sym_POUND] = ACTIONS(2574),
    [anon_sym_SQUOTE] = ACTIONS(2576),
    [anon_sym_async] = ACTIONS(2576),
    [anon_sym_auto] = ACTIONS(2576),
    [anon_sym_await] = ACTIONS(2576),
    [anon_sym_become] = ACTIONS(2576),
    [anon_sym_break] = ACTIONS(2576),
    [anon_sym_const] = ACTIONS(2576),
    [anon_sym_continue] = ACTIONS(2576),
    [anon_sym_default] = ACTIONS(2576),
    [anon_sym_enum] = ACTIONS(2576),
    [anon_sym_fn] = ACTIONS(2576),
    [anon_sym_for] = ACTIONS(2576),
    [anon_sym_gen] = ACTIONS(2576),
    [anon_sym_if] = ACTIONS(2576),
    [anon_sym_impl] = ACTIONS(2576),
    [anon_sym_let] = ACTIONS(2576),
    [anon_sym_loop] = ACTIONS(2576),
    [anon_sym_match] = ACTIONS(2576),
    [anon_sym_mod] = ACTIONS(2576),
    [anon_sym_pub] = ACTIONS(2576),
    [anon_sym_return] = ACTIONS(2576),
    [anon_sym_raw] = ACTIONS(2576),
    [anon_sym_static] = ACTIONS(2576),
    [anon_sym_struct] = ACTIONS(2576),
    [anon_sym_trait] = ACTIONS(2576),
    [anon_sym_type] = ACTIONS(2576),
    [anon_sym_union] = ACTIONS(2576),
    [anon_sym_unsafe] = ACTIONS(2576),
    [anon_sym_use] = ACTIONS(2576),
    [anon_sym_while] = ACTIONS(2576),
    [anon_sym_extern] = ACTIONS(2576),
    [anon_sym_safe] = ACTIONS(2576),
    [anon_sym_yield] = ACTIONS(2576),
    [anon_sym_move] = ACTIONS(2576),
    [anon_sym_try] = ACTIONS(2576),
    [sym_integer_literal] = ACTIONS(2574),
    [aux_sym_string_literal_token1] = ACTIONS(2574),
    [sym_char_literal] = ACTIONS(2574),
    [anon_sym_true] = ACTIONS(2576),
    [anon_sym_false] = ACTIONS(2576),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2576),
    [sym_super] = ACTIONS(2576),
    [sym_crate] = ACTIONS(2576),
    [aux_sym_metavariable_token1] = ACTIONS(2574),
    [sym__raw_string_literal_start] = ACTIONS(2574),
    [sym_float_literal] = ACTIONS(2574),
    [sym__outer_block_doc_comment_start] = ACTIONS(2574),
    [sym__outer_line_doc_comment_start] = ACTIONS(2574),
  },
  [STATE(701)] = {
    [sym_line_comment] = STATE(701),
    [sym_block_comment] = STATE(701),
    [ts_builtin_sym_end] = ACTIONS(2578),
    [sym_identifier] = ACTIONS(2580),
    [anon_sym_SEMI] = ACTIONS(2578),
    [anon_sym_macro] = ACTIONS(2580),
    [anon_sym_LPAREN] = ACTIONS(2578),
    [anon_sym_LBRACE] = ACTIONS(2578),
    [anon_sym_RBRACE] = ACTIONS(2578),
    [anon_sym_macro_rules] = ACTIONS(2580),
    [anon_sym_BANG] = ACTIONS(2578),
    [anon_sym_LBRACK] = ACTIONS(2578),
    [anon_sym_STAR] = ACTIONS(2578),
    [anon_sym_u8] = ACTIONS(2580),
    [anon_sym_i8] = ACTIONS(2580),
    [anon_sym_u16] = ACTIONS(2580),
    [anon_sym_i16] = ACTIONS(2580),
    [anon_sym_u32] = ACTIONS(2580),
    [anon_sym_i32] = ACTIONS(2580),
    [anon_sym_u64] = ACTIONS(2580),
    [anon_sym_i64] = ACTIONS(2580),
    [anon_sym_u128] = ACTIONS(2580),
    [anon_sym_i128] = ACTIONS(2580),
    [anon_sym_isize] = ACTIONS(2580),
    [anon_sym_usize] = ACTIONS(2580),
    [anon_sym_f32] = ACTIONS(2580),
    [anon_sym_f64] = ACTIONS(2580),
    [anon_sym_bool] = ACTIONS(2580),
    [anon_sym_str] = ACTIONS(2580),
    [anon_sym_char] = ACTIONS(2580),
    [anon_sym_DASH] = ACTIONS(2578),
    [anon_sym_AMP] = ACTIONS(2578),
    [anon_sym_PIPE] = ACTIONS(2578),
    [anon_sym_LT] = ACTIONS(2578),
    [anon_sym_DOT_DOT] = ACTIONS(2578),
    [anon_sym_COLON_COLON] = ACTIONS(2578),
    [anon_sym_POUND] = ACTIONS(2578),
    [anon_sym_SQUOTE] = ACTIONS(2580),
    [anon_sym_async] = ACTIONS(2580),
    [anon_sym_auto] = ACTIONS(2580),
    [anon_sym_await] = ACTIONS(2580),
    [anon_sym_become] = ACTIONS(2580),
    [anon_sym_break] = ACTIONS(2580),
    [anon_sym_const] = ACTIONS(2580),
    [anon_sym_continue] = ACTIONS(2580),
    [anon_sym_default] = ACTIONS(2580),
    [anon_sym_enum] = ACTIONS(2580),
    [anon_sym_fn] = ACTIONS(2580),
    [anon_sym_for] = ACTIONS(2580),
    [anon_sym_gen] = ACTIONS(2580),
    [anon_sym_if] = ACTIONS(2580),
    [anon_sym_impl] = ACTIONS(2580),
    [anon_sym_let] = ACTIONS(2580),
    [anon_sym_loop] = ACTIONS(2580),
    [anon_sym_match] = ACTIONS(2580),
    [anon_sym_mod] = ACTIONS(2580),
    [anon_sym_pub] = ACTIONS(2580),
    [anon_sym_return] = ACTIONS(2580),
    [anon_sym_raw] = ACTIONS(2580),
    [anon_sym_static] = ACTIONS(2580),
    [anon_sym_struct] = ACTIONS(2580),
    [anon_sym_trait] = ACTIONS(2580),
    [anon_sym_type] = ACTIONS(2580),
    [anon_sym_union] = ACTIONS(2580),
    [anon_sym_unsafe] = ACTIONS(2580),
    [anon_sym_use] = ACTIONS(2580),
    [anon_sym_while] = ACTIONS(2580),
    [anon_sym_extern] = ACTIONS(2580),
    [anon_sym_safe] = ACTIONS(2580),
    [anon_sym_yield] = ACTIONS(2580),
    [anon_sym_move] = ACTIONS(2580),
    [anon_sym_try] = ACTIONS(2580),
    [sym_integer_literal] = ACTIONS(2578),
    [aux_sym_string_literal_token1] = ACTIONS(2578),
    [sym_char_literal] = ACTIONS(2578),
    [anon_sym_true] = ACTIONS(2580),
    [anon_sym_false] = ACTIONS(2580),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2580),
    [sym_super] = ACTIONS(2580),
    [sym_crate] = ACTIONS(2580),
    [aux_sym_metavariable_token1] = ACTIONS(2578),
    [sym__raw_string_literal_start] = ACTIONS(2578),
    [sym_float_literal] = ACTIONS(2578),
    [sym__outer_block_doc_comment_start] = ACTIONS(2578),
    [sym__outer_line_doc_comment_start] = ACTIONS(2578),
  },
  [STATE(702)] = {
    [sym_line_comment] = STATE(702),
    [sym_block_comment] = STATE(702),
    [ts_builtin_sym_end] = ACTIONS(2582),
    [sym_identifier] = ACTIONS(2584),
    [anon_sym_SEMI] = ACTIONS(2582),
    [anon_sym_macro] = ACTIONS(2584),
    [anon_sym_LPAREN] = ACTIONS(2582),
    [anon_sym_LBRACE] = ACTIONS(2582),
    [anon_sym_RBRACE] = ACTIONS(2582),
    [anon_sym_macro_rules] = ACTIONS(2584),
    [anon_sym_BANG] = ACTIONS(2582),
    [anon_sym_LBRACK] = ACTIONS(2582),
    [anon_sym_STAR] = ACTIONS(2582),
    [anon_sym_u8] = ACTIONS(2584),
    [anon_sym_i8] = ACTIONS(2584),
    [anon_sym_u16] = ACTIONS(2584),
    [anon_sym_i16] = ACTIONS(2584),
    [anon_sym_u32] = ACTIONS(2584),
    [anon_sym_i32] = ACTIONS(2584),
    [anon_sym_u64] = ACTIONS(2584),
    [anon_sym_i64] = ACTIONS(2584),
    [anon_sym_u128] = ACTIONS(2584),
    [anon_sym_i128] = ACTIONS(2584),
    [anon_sym_isize] = ACTIONS(2584),
    [anon_sym_usize] = ACTIONS(2584),
    [anon_sym_f32] = ACTIONS(2584),
    [anon_sym_f64] = ACTIONS(2584),
    [anon_sym_bool] = ACTIONS(2584),
    [anon_sym_str] = ACTIONS(2584),
    [anon_sym_char] = ACTIONS(2584),
    [anon_sym_DASH] = ACTIONS(2582),
    [anon_sym_AMP] = ACTIONS(2582),
    [anon_sym_PIPE] = ACTIONS(2582),
    [anon_sym_LT] = ACTIONS(2582),
    [anon_sym_DOT_DOT] = ACTIONS(2582),
    [anon_sym_COLON_COLON] = ACTIONS(2582),
    [anon_sym_POUND] = ACTIONS(2582),
    [anon_sym_SQUOTE] = ACTIONS(2584),
    [anon_sym_async] = ACTIONS(2584),
    [anon_sym_auto] = ACTIONS(2584),
    [anon_sym_await] = ACTIONS(2584),
    [anon_sym_become] = ACTIONS(2584),
    [anon_sym_break] = ACTIONS(2584),
    [anon_sym_const] = ACTIONS(2584),
    [anon_sym_continue] = ACTIONS(2584),
    [anon_sym_default] = ACTIONS(2584),
    [anon_sym_enum] = ACTIONS(2584),
    [anon_sym_fn] = ACTIONS(2584),
    [anon_sym_for] = ACTIONS(2584),
    [anon_sym_gen] = ACTIONS(2584),
    [anon_sym_if] = ACTIONS(2584),
    [anon_sym_impl] = ACTIONS(2584),
    [anon_sym_let] = ACTIONS(2584),
    [anon_sym_loop] = ACTIONS(2584),
    [anon_sym_match] = ACTIONS(2584),
    [anon_sym_mod] = ACTIONS(2584),
    [anon_sym_pub] = ACTIONS(2584),
    [anon_sym_return] = ACTIONS(2584),
    [anon_sym_raw] = ACTIONS(2584),
    [anon_sym_static] = ACTIONS(2584),
    [anon_sym_struct] = ACTIONS(2584),
    [anon_sym_trait] = ACTIONS(2584),
    [anon_sym_type] = ACTIONS(2584),
    [anon_sym_union] = ACTIONS(2584),
    [anon_sym_unsafe] = ACTIONS(2584),
    [anon_sym_use] = ACTIONS(2584),
    [anon_sym_while] = ACTIONS(2584),
    [anon_sym_extern] = ACTIONS(2584),
    [anon_sym_safe] = ACTIONS(2584),
    [anon_sym_yield] = ACTIONS(2584),
    [anon_sym_move] = ACTIONS(2584),
    [anon_sym_try] = ACTIONS(2584),
    [sym_integer_literal] = ACTIONS(2582),
    [aux_sym_string_literal_token1] = ACTIONS(2582),
    [sym_char_literal] = ACTIONS(2582),
    [anon_sym_true] = ACTIONS(2584),
    [anon_sym_false] = ACTIONS(2584),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2584),
    [sym_super] = ACTIONS(2584),
    [sym_crate] = ACTIONS(2584),
    [aux_sym_metavariable_token1] = ACTIONS(2582),
    [sym__raw_string_literal_start] = ACTIONS(2582),
    [sym_float_literal] = ACTIONS(2582),
    [sym__outer_block_doc_comment_start] = ACTIONS(2582),
    [sym__outer_line_doc_comment_start] = ACTIONS(2582),
  },
  [STATE(703)] = {
    [sym_line_comment] = STATE(703),
    [sym_block_comment] = STATE(703),
    [ts_builtin_sym_end] = ACTIONS(2586),
    [sym_identifier] = ACTIONS(2588),
    [anon_sym_SEMI] = ACTIONS(2586),
    [anon_sym_macro] = ACTIONS(2588),
    [anon_sym_LPAREN] = ACTIONS(2586),
    [anon_sym_LBRACE] = ACTIONS(2586),
    [anon_sym_RBRACE] = ACTIONS(2586),
    [anon_sym_macro_rules] = ACTIONS(2588),
    [anon_sym_BANG] = ACTIONS(2586),
    [anon_sym_LBRACK] = ACTIONS(2586),
    [anon_sym_STAR] = ACTIONS(2586),
    [anon_sym_u8] = ACTIONS(2588),
    [anon_sym_i8] = ACTIONS(2588),
    [anon_sym_u16] = ACTIONS(2588),
    [anon_sym_i16] = ACTIONS(2588),
    [anon_sym_u32] = ACTIONS(2588),
    [anon_sym_i32] = ACTIONS(2588),
    [anon_sym_u64] = ACTIONS(2588),
    [anon_sym_i64] = ACTIONS(2588),
    [anon_sym_u128] = ACTIONS(2588),
    [anon_sym_i128] = ACTIONS(2588),
    [anon_sym_isize] = ACTIONS(2588),
    [anon_sym_usize] = ACTIONS(2588),
    [anon_sym_f32] = ACTIONS(2588),
    [anon_sym_f64] = ACTIONS(2588),
    [anon_sym_bool] = ACTIONS(2588),
    [anon_sym_str] = ACTIONS(2588),
    [anon_sym_char] = ACTIONS(2588),
    [anon_sym_DASH] = ACTIONS(2586),
    [anon_sym_AMP] = ACTIONS(2586),
    [anon_sym_PIPE] = ACTIONS(2586),
    [anon_sym_LT] = ACTIONS(2586),
    [anon_sym_DOT_DOT] = ACTIONS(2586),
    [anon_sym_COLON_COLON] = ACTIONS(2586),
    [anon_sym_POUND] = ACTIONS(2586),
    [anon_sym_SQUOTE] = ACTIONS(2588),
    [anon_sym_async] = ACTIONS(2588),
    [anon_sym_auto] = ACTIONS(2588),
    [anon_sym_await] = ACTIONS(2588),
    [anon_sym_become] = ACTIONS(2588),
    [anon_sym_break] = ACTIONS(2588),
    [anon_sym_const] = ACTIONS(2588),
    [anon_sym_continue] = ACTIONS(2588),
    [anon_sym_default] = ACTIONS(2588),
    [anon_sym_enum] = ACTIONS(2588),
    [anon_sym_fn] = ACTIONS(2588),
    [anon_sym_for] = ACTIONS(2588),
    [anon_sym_gen] = ACTIONS(2588),
    [anon_sym_if] = ACTIONS(2588),
    [anon_sym_impl] = ACTIONS(2588),
    [anon_sym_let] = ACTIONS(2588),
    [anon_sym_loop] = ACTIONS(2588),
    [anon_sym_match] = ACTIONS(2588),
    [anon_sym_mod] = ACTIONS(2588),
    [anon_sym_pub] = ACTIONS(2588),
    [anon_sym_return] = ACTIONS(2588),
    [anon_sym_raw] = ACTIONS(2588),
    [anon_sym_static] = ACTIONS(2588),
    [anon_sym_struct] = ACTIONS(2588),
    [anon_sym_trait] = ACTIONS(2588),
    [anon_sym_type] = ACTIONS(2588),
    [anon_sym_union] = ACTIONS(2588),
    [anon_sym_unsafe] = ACTIONS(2588),
    [anon_sym_use] = ACTIONS(2588),
    [anon_sym_while] = ACTIONS(2588),
    [anon_sym_extern] = ACTIONS(2588),
    [anon_sym_safe] = ACTIONS(2588),
    [anon_sym_yield] = ACTIONS(2588),
    [anon_sym_move] = ACTIONS(2588),
    [anon_sym_try] = ACTIONS(2588),
    [sym_integer_literal] = ACTIONS(2586),
    [aux_sym_string_literal_token1] = ACTIONS(2586),
    [sym_char_literal] = ACTIONS(2586),
    [anon_sym_true] = ACTIONS(2588),
    [anon_sym_false] = ACTIONS(2588),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2588),
    [sym_super] = ACTIONS(2588),
    [sym_crate] = ACTIONS(2588),
    [aux_sym_metavariable_token1] = ACTIONS(2586),
    [sym__raw_string_literal_start] = ACTIONS(2586),
    [sym_float_literal] = ACTIONS(2586),
    [sym__outer_block_doc_comment_start] = ACTIONS(2586),
    [sym__outer_line_doc_comment_start] = ACTIONS(2586),
  },
  [STATE(704)] = {
    [sym_line_comment] = STATE(704),
    [sym_block_comment] = STATE(704),
    [ts_builtin_sym_end] = ACTIONS(2590),
    [sym_identifier] = ACTIONS(2592),
    [anon_sym_SEMI] = ACTIONS(2590),
    [anon_sym_macro] = ACTIONS(2592),
    [anon_sym_LPAREN] = ACTIONS(2590),
    [anon_sym_LBRACE] = ACTIONS(2590),
    [anon_sym_RBRACE] = ACTIONS(2590),
    [anon_sym_macro_rules] = ACTIONS(2592),
    [anon_sym_BANG] = ACTIONS(2590),
    [anon_sym_LBRACK] = ACTIONS(2590),
    [anon_sym_STAR] = ACTIONS(2590),
    [anon_sym_u8] = ACTIONS(2592),
    [anon_sym_i8] = ACTIONS(2592),
    [anon_sym_u16] = ACTIONS(2592),
    [anon_sym_i16] = ACTIONS(2592),
    [anon_sym_u32] = ACTIONS(2592),
    [anon_sym_i32] = ACTIONS(2592),
    [anon_sym_u64] = ACTIONS(2592),
    [anon_sym_i64] = ACTIONS(2592),
    [anon_sym_u128] = ACTIONS(2592),
    [anon_sym_i128] = ACTIONS(2592),
    [anon_sym_isize] = ACTIONS(2592),
    [anon_sym_usize] = ACTIONS(2592),
    [anon_sym_f32] = ACTIONS(2592),
    [anon_sym_f64] = ACTIONS(2592),
    [anon_sym_bool] = ACTIONS(2592),
    [anon_sym_str] = ACTIONS(2592),
    [anon_sym_char] = ACTIONS(2592),
    [anon_sym_DASH] = ACTIONS(2590),
    [anon_sym_AMP] = ACTIONS(2590),
    [anon_sym_PIPE] = ACTIONS(2590),
    [anon_sym_LT] = ACTIONS(2590),
    [anon_sym_DOT_DOT] = ACTIONS(2590),
    [anon_sym_COLON_COLON] = ACTIONS(2590),
    [anon_sym_POUND] = ACTIONS(2590),
    [anon_sym_SQUOTE] = ACTIONS(2592),
    [anon_sym_async] = ACTIONS(2592),
    [anon_sym_auto] = ACTIONS(2592),
    [anon_sym_await] = ACTIONS(2592),
    [anon_sym_become] = ACTIONS(2592),
    [anon_sym_break] = ACTIONS(2592),
    [anon_sym_const] = ACTIONS(2592),
    [anon_sym_continue] = ACTIONS(2592),
    [anon_sym_default] = ACTIONS(2592),
    [anon_sym_enum] = ACTIONS(2592),
    [anon_sym_fn] = ACTIONS(2592),
    [anon_sym_for] = ACTIONS(2592),
    [anon_sym_gen] = ACTIONS(2592),
    [anon_sym_if] = ACTIONS(2592),
    [anon_sym_impl] = ACTIONS(2592),
    [anon_sym_let] = ACTIONS(2592),
    [anon_sym_loop] = ACTIONS(2592),
    [anon_sym_match] = ACTIONS(2592),
    [anon_sym_mod] = ACTIONS(2592),
    [anon_sym_pub] = ACTIONS(2592),
    [anon_sym_return] = ACTIONS(2592),
    [anon_sym_raw] = ACTIONS(2592),
    [anon_sym_static] = ACTIONS(2592),
    [anon_sym_struct] = ACTIONS(2592),
    [anon_sym_trait] = ACTIONS(2592),
    [anon_sym_type] = ACTIONS(2592),
    [anon_sym_union] = ACTIONS(2592),
    [anon_sym_unsafe] = ACTIONS(2592),
    [anon_sym_use] = ACTIONS(2592),
    [anon_sym_while] = ACTIONS(2592),
    [anon_sym_extern] = ACTIONS(2592),
    [anon_sym_safe] = ACTIONS(2592),
    [anon_sym_yield] = ACTIONS(2592),
    [anon_sym_move] = ACTIONS(2592),
    [anon_sym_try] = ACTIONS(2592),
    [sym_integer_literal] = ACTIONS(2590),
    [aux_sym_string_literal_token1] = ACTIONS(2590),
    [sym_char_literal] = ACTIONS(2590),
    [anon_sym_true] = ACTIONS(2592),
    [anon_sym_false] = ACTIONS(2592),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2592),
    [sym_super] = ACTIONS(2592),
    [sym_crate] = ACTIONS(2592),
    [aux_sym_metavariable_token1] = ACTIONS(2590),
    [sym__raw_string_literal_start] = ACTIONS(2590),
    [sym_float_literal] = ACTIONS(2590),
    [sym__outer_block_doc_comment_start] = ACTIONS(2590),
    [sym__outer_line_doc_comment_start] = ACTIONS(2590),
  },
  [STATE(705)] = {
    [sym_line_comment] = STATE(705),
    [sym_block_comment] = STATE(705),
    [ts_builtin_sym_end] = ACTIONS(2594),
    [sym_identifier] = ACTIONS(2596),
    [anon_sym_SEMI] = ACTIONS(2594),
    [anon_sym_macro] = ACTIONS(2596),
    [anon_sym_LPAREN] = ACTIONS(2594),
    [anon_sym_LBRACE] = ACTIONS(2594),
    [anon_sym_RBRACE] = ACTIONS(2594),
    [anon_sym_macro_rules] = ACTIONS(2596),
    [anon_sym_BANG] = ACTIONS(2594),
    [anon_sym_LBRACK] = ACTIONS(2594),
    [anon_sym_STAR] = ACTIONS(2594),
    [anon_sym_u8] = ACTIONS(2596),
    [anon_sym_i8] = ACTIONS(2596),
    [anon_sym_u16] = ACTIONS(2596),
    [anon_sym_i16] = ACTIONS(2596),
    [anon_sym_u32] = ACTIONS(2596),
    [anon_sym_i32] = ACTIONS(2596),
    [anon_sym_u64] = ACTIONS(2596),
    [anon_sym_i64] = ACTIONS(2596),
    [anon_sym_u128] = ACTIONS(2596),
    [anon_sym_i128] = ACTIONS(2596),
    [anon_sym_isize] = ACTIONS(2596),
    [anon_sym_usize] = ACTIONS(2596),
    [anon_sym_f32] = ACTIONS(2596),
    [anon_sym_f64] = ACTIONS(2596),
    [anon_sym_bool] = ACTIONS(2596),
    [anon_sym_str] = ACTIONS(2596),
    [anon_sym_char] = ACTIONS(2596),
    [anon_sym_DASH] = ACTIONS(2594),
    [anon_sym_AMP] = ACTIONS(2594),
    [anon_sym_PIPE] = ACTIONS(2594),
    [anon_sym_LT] = ACTIONS(2594),
    [anon_sym_DOT_DOT] = ACTIONS(2594),
    [anon_sym_COLON_COLON] = ACTIONS(2594),
    [anon_sym_POUND] = ACTIONS(2594),
    [anon_sym_SQUOTE] = ACTIONS(2596),
    [anon_sym_async] = ACTIONS(2596),
    [anon_sym_auto] = ACTIONS(2596),
    [anon_sym_await] = ACTIONS(2596),
    [anon_sym_become] = ACTIONS(2596),
    [anon_sym_break] = ACTIONS(2596),
    [anon_sym_const] = ACTIONS(2596),
    [anon_sym_continue] = ACTIONS(2596),
    [anon_sym_default] = ACTIONS(2596),
    [anon_sym_enum] = ACTIONS(2596),
    [anon_sym_fn] = ACTIONS(2596),
    [anon_sym_for] = ACTIONS(2596),
    [anon_sym_gen] = ACTIONS(2596),
    [anon_sym_if] = ACTIONS(2596),
    [anon_sym_impl] = ACTIONS(2596),
    [anon_sym_let] = ACTIONS(2596),
    [anon_sym_loop] = ACTIONS(2596),
    [anon_sym_match] = ACTIONS(2596),
    [anon_sym_mod] = ACTIONS(2596),
    [anon_sym_pub] = ACTIONS(2596),
    [anon_sym_return] = ACTIONS(2596),
    [anon_sym_raw] = ACTIONS(2596),
    [anon_sym_static] = ACTIONS(2596),
    [anon_sym_struct] = ACTIONS(2596),
    [anon_sym_trait] = ACTIONS(2596),
    [anon_sym_type] = ACTIONS(2596),
    [anon_sym_union] = ACTIONS(2596),
    [anon_sym_unsafe] = ACTIONS(2596),
    [anon_sym_use] = ACTIONS(2596),
    [anon_sym_while] = ACTIONS(2596),
    [anon_sym_extern] = ACTIONS(2596),
    [anon_sym_safe] = ACTIONS(2596),
    [anon_sym_yield] = ACTIONS(2596),
    [anon_sym_move] = ACTIONS(2596),
    [anon_sym_try] = ACTIONS(2596),
    [sym_integer_literal] = ACTIONS(2594),
    [aux_sym_string_literal_token1] = ACTIONS(2594),
    [sym_char_literal] = ACTIONS(2594),
    [anon_sym_true] = ACTIONS(2596),
    [anon_sym_false] = ACTIONS(2596),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2596),
    [sym_super] = ACTIONS(2596),
    [sym_crate] = ACTIONS(2596),
    [aux_sym_metavariable_token1] = ACTIONS(2594),
    [sym__raw_string_literal_start] = ACTIONS(2594),
    [sym_float_literal] = ACTIONS(2594),
    [sym__outer_block_doc_comment_start] = ACTIONS(2594),
    [sym__outer_line_doc_comment_start] = ACTIONS(2594),
  },
  [STATE(706)] = {
    [sym_line_comment] = STATE(706),
    [sym_block_comment] = STATE(706),
    [ts_builtin_sym_end] = ACTIONS(2598),
    [sym_identifier] = ACTIONS(2600),
    [anon_sym_SEMI] = ACTIONS(2598),
    [anon_sym_macro] = ACTIONS(2600),
    [anon_sym_LPAREN] = ACTIONS(2598),
    [anon_sym_LBRACE] = ACTIONS(2598),
    [anon_sym_RBRACE] = ACTIONS(2598),
    [anon_sym_macro_rules] = ACTIONS(2600),
    [anon_sym_BANG] = ACTIONS(2598),
    [anon_sym_LBRACK] = ACTIONS(2598),
    [anon_sym_STAR] = ACTIONS(2598),
    [anon_sym_u8] = ACTIONS(2600),
    [anon_sym_i8] = ACTIONS(2600),
    [anon_sym_u16] = ACTIONS(2600),
    [anon_sym_i16] = ACTIONS(2600),
    [anon_sym_u32] = ACTIONS(2600),
    [anon_sym_i32] = ACTIONS(2600),
    [anon_sym_u64] = ACTIONS(2600),
    [anon_sym_i64] = ACTIONS(2600),
    [anon_sym_u128] = ACTIONS(2600),
    [anon_sym_i128] = ACTIONS(2600),
    [anon_sym_isize] = ACTIONS(2600),
    [anon_sym_usize] = ACTIONS(2600),
    [anon_sym_f32] = ACTIONS(2600),
    [anon_sym_f64] = ACTIONS(2600),
    [anon_sym_bool] = ACTIONS(2600),
    [anon_sym_str] = ACTIONS(2600),
    [anon_sym_char] = ACTIONS(2600),
    [anon_sym_DASH] = ACTIONS(2598),
    [anon_sym_AMP] = ACTIONS(2598),
    [anon_sym_PIPE] = ACTIONS(2598),
    [anon_sym_LT] = ACTIONS(2598),
    [anon_sym_DOT_DOT] = ACTIONS(2598),
    [anon_sym_COLON_COLON] = ACTIONS(2598),
    [anon_sym_POUND] = ACTIONS(2598),
    [anon_sym_SQUOTE] = ACTIONS(2600),
    [anon_sym_async] = ACTIONS(2600),
    [anon_sym_auto] = ACTIONS(2600),
    [anon_sym_await] = ACTIONS(2600),
    [anon_sym_become] = ACTIONS(2600),
    [anon_sym_break] = ACTIONS(2600),
    [anon_sym_const] = ACTIONS(2600),
    [anon_sym_continue] = ACTIONS(2600),
    [anon_sym_default] = ACTIONS(2600),
    [anon_sym_enum] = ACTIONS(2600),
    [anon_sym_fn] = ACTIONS(2600),
    [anon_sym_for] = ACTIONS(2600),
    [anon_sym_gen] = ACTIONS(2600),
    [anon_sym_if] = ACTIONS(2600),
    [anon_sym_impl] = ACTIONS(2600),
    [anon_sym_let] = ACTIONS(2600),
    [anon_sym_loop] = ACTIONS(2600),
    [anon_sym_match] = ACTIONS(2600),
    [anon_sym_mod] = ACTIONS(2600),
    [anon_sym_pub] = ACTIONS(2600),
    [anon_sym_return] = ACTIONS(2600),
    [anon_sym_raw] = ACTIONS(2600),
    [anon_sym_static] = ACTIONS(2600),
    [anon_sym_struct] = ACTIONS(2600),
    [anon_sym_trait] = ACTIONS(2600),
    [anon_sym_type] = ACTIONS(2600),
    [anon_sym_union] = ACTIONS(2600),
    [anon_sym_unsafe] = ACTIONS(2600),
    [anon_sym_use] = ACTIONS(2600),
    [anon_sym_while] = ACTIONS(2600),
    [anon_sym_extern] = ACTIONS(2600),
    [anon_sym_safe] = ACTIONS(2600),
    [anon_sym_yield] = ACTIONS(2600),
    [anon_sym_move] = ACTIONS(2600),
    [anon_sym_try] = ACTIONS(2600),
    [sym_integer_literal] = ACTIONS(2598),
    [aux_sym_string_literal_token1] = ACTIONS(2598),
    [sym_char_literal] = ACTIONS(2598),
    [anon_sym_true] = ACTIONS(2600),
    [anon_sym_false] = ACTIONS(2600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2600),
    [sym_super] = ACTIONS(2600),
    [sym_crate] = ACTIONS(2600),
    [aux_sym_metavariable_token1] = ACTIONS(2598),
    [sym__raw_string_literal_start] = ACTIONS(2598),
    [sym_float_literal] = ACTIONS(2598),
    [sym__outer_block_doc_comment_start] = ACTIONS(2598),
    [sym__outer_line_doc_comment_start] = ACTIONS(2598),
  },
  [STATE(707)] = {
    [sym_line_comment] = STATE(707),
    [sym_block_comment] = STATE(707),
    [ts_builtin_sym_end] = ACTIONS(2602),
    [sym_identifier] = ACTIONS(2604),
    [anon_sym_SEMI] = ACTIONS(2602),
    [anon_sym_macro] = ACTIONS(2604),
    [anon_sym_LPAREN] = ACTIONS(2602),
    [anon_sym_LBRACE] = ACTIONS(2602),
    [anon_sym_RBRACE] = ACTIONS(2602),
    [anon_sym_macro_rules] = ACTIONS(2604),
    [anon_sym_BANG] = ACTIONS(2602),
    [anon_sym_LBRACK] = ACTIONS(2602),
    [anon_sym_STAR] = ACTIONS(2602),
    [anon_sym_u8] = ACTIONS(2604),
    [anon_sym_i8] = ACTIONS(2604),
    [anon_sym_u16] = ACTIONS(2604),
    [anon_sym_i16] = ACTIONS(2604),
    [anon_sym_u32] = ACTIONS(2604),
    [anon_sym_i32] = ACTIONS(2604),
    [anon_sym_u64] = ACTIONS(2604),
    [anon_sym_i64] = ACTIONS(2604),
    [anon_sym_u128] = ACTIONS(2604),
    [anon_sym_i128] = ACTIONS(2604),
    [anon_sym_isize] = ACTIONS(2604),
    [anon_sym_usize] = ACTIONS(2604),
    [anon_sym_f32] = ACTIONS(2604),
    [anon_sym_f64] = ACTIONS(2604),
    [anon_sym_bool] = ACTIONS(2604),
    [anon_sym_str] = ACTIONS(2604),
    [anon_sym_char] = ACTIONS(2604),
    [anon_sym_DASH] = ACTIONS(2602),
    [anon_sym_AMP] = ACTIONS(2602),
    [anon_sym_PIPE] = ACTIONS(2602),
    [anon_sym_LT] = ACTIONS(2602),
    [anon_sym_DOT_DOT] = ACTIONS(2602),
    [anon_sym_COLON_COLON] = ACTIONS(2602),
    [anon_sym_POUND] = ACTIONS(2602),
    [anon_sym_SQUOTE] = ACTIONS(2604),
    [anon_sym_async] = ACTIONS(2604),
    [anon_sym_auto] = ACTIONS(2604),
    [anon_sym_await] = ACTIONS(2604),
    [anon_sym_become] = ACTIONS(2604),
    [anon_sym_break] = ACTIONS(2604),
    [anon_sym_const] = ACTIONS(2604),
    [anon_sym_continue] = ACTIONS(2604),
    [anon_sym_default] = ACTIONS(2604),
    [anon_sym_enum] = ACTIONS(2604),
    [anon_sym_fn] = ACTIONS(2604),
    [anon_sym_for] = ACTIONS(2604),
    [anon_sym_gen] = ACTIONS(2604),
    [anon_sym_if] = ACTIONS(2604),
    [anon_sym_impl] = ACTIONS(2604),
    [anon_sym_let] = ACTIONS(2604),
    [anon_sym_loop] = ACTIONS(2604),
    [anon_sym_match] = ACTIONS(2604),
    [anon_sym_mod] = ACTIONS(2604),
    [anon_sym_pub] = ACTIONS(2604),
    [anon_sym_return] = ACTIONS(2604),
    [anon_sym_raw] = ACTIONS(2604),
    [anon_sym_static] = ACTIONS(2604),
    [anon_sym_struct] = ACTIONS(2604),
    [anon_sym_trait] = ACTIONS(2604),
    [anon_sym_type] = ACTIONS(2604),
    [anon_sym_union] = ACTIONS(2604),
    [anon_sym_unsafe] = ACTIONS(2604),
    [anon_sym_use] = ACTIONS(2604),
    [anon_sym_while] = ACTIONS(2604),
    [anon_sym_extern] = ACTIONS(2604),
    [anon_sym_safe] = ACTIONS(2604),
    [anon_sym_yield] = ACTIONS(2604),
    [anon_sym_move] = ACTIONS(2604),
    [anon_sym_try] = ACTIONS(2604),
    [sym_integer_literal] = ACTIONS(2602),
    [aux_sym_string_literal_token1] = ACTIONS(2602),
    [sym_char_literal] = ACTIONS(2602),
    [anon_sym_true] = ACTIONS(2604),
    [anon_sym_false] = ACTIONS(2604),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2604),
    [sym_super] = ACTIONS(2604),
    [sym_crate] = ACTIONS(2604),
    [aux_sym_metavariable_token1] = ACTIONS(2602),
    [sym__raw_string_literal_start] = ACTIONS(2602),
    [sym_float_literal] = ACTIONS(2602),
    [sym__outer_block_doc_comment_start] = ACTIONS(2602),
    [sym__outer_line_doc_comment_start] = ACTIONS(2602),
  },
  [STATE(708)] = {
    [sym_line_comment] = STATE(708),
    [sym_block_comment] = STATE(708),
    [ts_builtin_sym_end] = ACTIONS(2606),
    [sym_identifier] = ACTIONS(2608),
    [anon_sym_SEMI] = ACTIONS(2606),
    [anon_sym_macro] = ACTIONS(2608),
    [anon_sym_LPAREN] = ACTIONS(2606),
    [anon_sym_LBRACE] = ACTIONS(2606),
    [anon_sym_RBRACE] = ACTIONS(2606),
    [anon_sym_macro_rules] = ACTIONS(2608),
    [anon_sym_BANG] = ACTIONS(2606),
    [anon_sym_LBRACK] = ACTIONS(2606),
    [anon_sym_STAR] = ACTIONS(2606),
    [anon_sym_u8] = ACTIONS(2608),
    [anon_sym_i8] = ACTIONS(2608),
    [anon_sym_u16] = ACTIONS(2608),
    [anon_sym_i16] = ACTIONS(2608),
    [anon_sym_u32] = ACTIONS(2608),
    [anon_sym_i32] = ACTIONS(2608),
    [anon_sym_u64] = ACTIONS(2608),
    [anon_sym_i64] = ACTIONS(2608),
    [anon_sym_u128] = ACTIONS(2608),
    [anon_sym_i128] = ACTIONS(2608),
    [anon_sym_isize] = ACTIONS(2608),
    [anon_sym_usize] = ACTIONS(2608),
    [anon_sym_f32] = ACTIONS(2608),
    [anon_sym_f64] = ACTIONS(2608),
    [anon_sym_bool] = ACTIONS(2608),
    [anon_sym_str] = ACTIONS(2608),
    [anon_sym_char] = ACTIONS(2608),
    [anon_sym_DASH] = ACTIONS(2606),
    [anon_sym_AMP] = ACTIONS(2606),
    [anon_sym_PIPE] = ACTIONS(2606),
    [anon_sym_LT] = ACTIONS(2606),
    [anon_sym_DOT_DOT] = ACTIONS(2606),
    [anon_sym_COLON_COLON] = ACTIONS(2606),
    [anon_sym_POUND] = ACTIONS(2606),
    [anon_sym_SQUOTE] = ACTIONS(2608),
    [anon_sym_async] = ACTIONS(2608),
    [anon_sym_auto] = ACTIONS(2608),
    [anon_sym_await] = ACTIONS(2608),
    [anon_sym_become] = ACTIONS(2608),
    [anon_sym_break] = ACTIONS(2608),
    [anon_sym_const] = ACTIONS(2608),
    [anon_sym_continue] = ACTIONS(2608),
    [anon_sym_default] = ACTIONS(2608),
    [anon_sym_enum] = ACTIONS(2608),
    [anon_sym_fn] = ACTIONS(2608),
    [anon_sym_for] = ACTIONS(2608),
    [anon_sym_gen] = ACTIONS(2608),
    [anon_sym_if] = ACTIONS(2608),
    [anon_sym_impl] = ACTIONS(2608),
    [anon_sym_let] = ACTIONS(2608),
    [anon_sym_loop] = ACTIONS(2608),
    [anon_sym_match] = ACTIONS(2608),
    [anon_sym_mod] = ACTIONS(2608),
    [anon_sym_pub] = ACTIONS(2608),
    [anon_sym_return] = ACTIONS(2608),
    [anon_sym_raw] = ACTIONS(2608),
    [anon_sym_static] = ACTIONS(2608),
    [anon_sym_struct] = ACTIONS(2608),
    [anon_sym_trait] = ACTIONS(2608),
    [anon_sym_type] = ACTIONS(2608),
    [anon_sym_union] = ACTIONS(2608),
    [anon_sym_unsafe] = ACTIONS(2608),
    [anon_sym_use] = ACTIONS(2608),
    [anon_sym_while] = ACTIONS(2608),
    [anon_sym_extern] = ACTIONS(2608),
    [anon_sym_safe] = ACTIONS(2608),
    [anon_sym_yield] = ACTIONS(2608),
    [anon_sym_move] = ACTIONS(2608),
    [anon_sym_try] = ACTIONS(2608),
    [sym_integer_literal] = ACTIONS(2606),
    [aux_sym_string_literal_token1] = ACTIONS(2606),
    [sym_char_literal] = ACTIONS(2606),
    [anon_sym_true] = ACTIONS(2608),
    [anon_sym_false] = ACTIONS(2608),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2608),
    [sym_super] = ACTIONS(2608),
    [sym_crate] = ACTIONS(2608),
    [aux_sym_metavariable_token1] = ACTIONS(2606),
    [sym__raw_string_literal_start] = ACTIONS(2606),
    [sym_float_literal] = ACTIONS(2606),
    [sym__outer_block_doc_comment_start] = ACTIONS(2606),
    [sym__outer_line_doc_comment_start] = ACTIONS(2606),
  },
  [STATE(709)] = {
    [sym_line_comment] = STATE(709),
    [sym_block_comment] = STATE(709),
    [ts_builtin_sym_end] = ACTIONS(2610),
    [sym_identifier] = ACTIONS(2612),
    [anon_sym_SEMI] = ACTIONS(2610),
    [anon_sym_macro] = ACTIONS(2612),
    [anon_sym_LPAREN] = ACTIONS(2610),
    [anon_sym_LBRACE] = ACTIONS(2610),
    [anon_sym_RBRACE] = ACTIONS(2610),
    [anon_sym_macro_rules] = ACTIONS(2612),
    [anon_sym_BANG] = ACTIONS(2610),
    [anon_sym_LBRACK] = ACTIONS(2610),
    [anon_sym_STAR] = ACTIONS(2610),
    [anon_sym_u8] = ACTIONS(2612),
    [anon_sym_i8] = ACTIONS(2612),
    [anon_sym_u16] = ACTIONS(2612),
    [anon_sym_i16] = ACTIONS(2612),
    [anon_sym_u32] = ACTIONS(2612),
    [anon_sym_i32] = ACTIONS(2612),
    [anon_sym_u64] = ACTIONS(2612),
    [anon_sym_i64] = ACTIONS(2612),
    [anon_sym_u128] = ACTIONS(2612),
    [anon_sym_i128] = ACTIONS(2612),
    [anon_sym_isize] = ACTIONS(2612),
    [anon_sym_usize] = ACTIONS(2612),
    [anon_sym_f32] = ACTIONS(2612),
    [anon_sym_f64] = ACTIONS(2612),
    [anon_sym_bool] = ACTIONS(2612),
    [anon_sym_str] = ACTIONS(2612),
    [anon_sym_char] = ACTIONS(2612),
    [anon_sym_DASH] = ACTIONS(2610),
    [anon_sym_AMP] = ACTIONS(2610),
    [anon_sym_PIPE] = ACTIONS(2610),
    [anon_sym_LT] = ACTIONS(2610),
    [anon_sym_DOT_DOT] = ACTIONS(2610),
    [anon_sym_COLON_COLON] = ACTIONS(2610),
    [anon_sym_POUND] = ACTIONS(2610),
    [anon_sym_SQUOTE] = ACTIONS(2612),
    [anon_sym_async] = ACTIONS(2612),
    [anon_sym_auto] = ACTIONS(2612),
    [anon_sym_await] = ACTIONS(2612),
    [anon_sym_become] = ACTIONS(2612),
    [anon_sym_break] = ACTIONS(2612),
    [anon_sym_const] = ACTIONS(2612),
    [anon_sym_continue] = ACTIONS(2612),
    [anon_sym_default] = ACTIONS(2612),
    [anon_sym_enum] = ACTIONS(2612),
    [anon_sym_fn] = ACTIONS(2612),
    [anon_sym_for] = ACTIONS(2612),
    [anon_sym_gen] = ACTIONS(2612),
    [anon_sym_if] = ACTIONS(2612),
    [anon_sym_impl] = ACTIONS(2612),
    [anon_sym_let] = ACTIONS(2612),
    [anon_sym_loop] = ACTIONS(2612),
    [anon_sym_match] = ACTIONS(2612),
    [anon_sym_mod] = ACTIONS(2612),
    [anon_sym_pub] = ACTIONS(2612),
    [anon_sym_return] = ACTIONS(2612),
    [anon_sym_raw] = ACTIONS(2612),
    [anon_sym_static] = ACTIONS(2612),
    [anon_sym_struct] = ACTIONS(2612),
    [anon_sym_trait] = ACTIONS(2612),
    [anon_sym_type] = ACTIONS(2612),
    [anon_sym_union] = ACTIONS(2612),
    [anon_sym_unsafe] = ACTIONS(2612),
    [anon_sym_use] = ACTIONS(2612),
    [anon_sym_while] = ACTIONS(2612),
    [anon_sym_extern] = ACTIONS(2612),
    [anon_sym_safe] = ACTIONS(2612),
    [anon_sym_yield] = ACTIONS(2612),
    [anon_sym_move] = ACTIONS(2612),
    [anon_sym_try] = ACTIONS(2612),
    [sym_integer_literal] = ACTIONS(2610),
    [aux_sym_string_literal_token1] = ACTIONS(2610),
    [sym_char_literal] = ACTIONS(2610),
    [anon_sym_true] = ACTIONS(2612),
    [anon_sym_false] = ACTIONS(2612),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2612),
    [sym_super] = ACTIONS(2612),
    [sym_crate] = ACTIONS(2612),
    [aux_sym_metavariable_token1] = ACTIONS(2610),
    [sym__raw_string_literal_start] = ACTIONS(2610),
    [sym_float_literal] = ACTIONS(2610),
    [sym__outer_block_doc_comment_start] = ACTIONS(2610),
    [sym__outer_line_doc_comment_start] = ACTIONS(2610),
  },
  [STATE(710)] = {
    [sym_line_comment] = STATE(710),
    [sym_block_comment] = STATE(710),
    [ts_builtin_sym_end] = ACTIONS(2614),
    [sym_identifier] = ACTIONS(2616),
    [anon_sym_SEMI] = ACTIONS(2614),
    [anon_sym_macro] = ACTIONS(2616),
    [anon_sym_LPAREN] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2614),
    [anon_sym_RBRACE] = ACTIONS(2614),
    [anon_sym_macro_rules] = ACTIONS(2616),
    [anon_sym_BANG] = ACTIONS(2614),
    [anon_sym_LBRACK] = ACTIONS(2614),
    [anon_sym_STAR] = ACTIONS(2614),
    [anon_sym_u8] = ACTIONS(2616),
    [anon_sym_i8] = ACTIONS(2616),
    [anon_sym_u16] = ACTIONS(2616),
    [anon_sym_i16] = ACTIONS(2616),
    [anon_sym_u32] = ACTIONS(2616),
    [anon_sym_i32] = ACTIONS(2616),
    [anon_sym_u64] = ACTIONS(2616),
    [anon_sym_i64] = ACTIONS(2616),
    [anon_sym_u128] = ACTIONS(2616),
    [anon_sym_i128] = ACTIONS(2616),
    [anon_sym_isize] = ACTIONS(2616),
    [anon_sym_usize] = ACTIONS(2616),
    [anon_sym_f32] = ACTIONS(2616),
    [anon_sym_f64] = ACTIONS(2616),
    [anon_sym_bool] = ACTIONS(2616),
    [anon_sym_str] = ACTIONS(2616),
    [anon_sym_char] = ACTIONS(2616),
    [anon_sym_DASH] = ACTIONS(2614),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_COLON_COLON] = ACTIONS(2614),
    [anon_sym_POUND] = ACTIONS(2614),
    [anon_sym_SQUOTE] = ACTIONS(2616),
    [anon_sym_async] = ACTIONS(2616),
    [anon_sym_auto] = ACTIONS(2616),
    [anon_sym_await] = ACTIONS(2616),
    [anon_sym_become] = ACTIONS(2616),
    [anon_sym_break] = ACTIONS(2616),
    [anon_sym_const] = ACTIONS(2616),
    [anon_sym_continue] = ACTIONS(2616),
    [anon_sym_default] = ACTIONS(2616),
    [anon_sym_enum] = ACTIONS(2616),
    [anon_sym_fn] = ACTIONS(2616),
    [anon_sym_for] = ACTIONS(2616),
    [anon_sym_gen] = ACTIONS(2616),
    [anon_sym_if] = ACTIONS(2616),
    [anon_sym_impl] = ACTIONS(2616),
    [anon_sym_let] = ACTIONS(2616),
    [anon_sym_loop] = ACTIONS(2616),
    [anon_sym_match] = ACTIONS(2616),
    [anon_sym_mod] = ACTIONS(2616),
    [anon_sym_pub] = ACTIONS(2616),
    [anon_sym_return] = ACTIONS(2616),
    [anon_sym_raw] = ACTIONS(2616),
    [anon_sym_static] = ACTIONS(2616),
    [anon_sym_struct] = ACTIONS(2616),
    [anon_sym_trait] = ACTIONS(2616),
    [anon_sym_type] = ACTIONS(2616),
    [anon_sym_union] = ACTIONS(2616),
    [anon_sym_unsafe] = ACTIONS(2616),
    [anon_sym_use] = ACTIONS(2616),
    [anon_sym_while] = ACTIONS(2616),
    [anon_sym_extern] = ACTIONS(2616),
    [anon_sym_safe] = ACTIONS(2616),
    [anon_sym_yield] = ACTIONS(2616),
    [anon_sym_move] = ACTIONS(2616),
    [anon_sym_try] = ACTIONS(2616),
    [sym_integer_literal] = ACTIONS(2614),
    [aux_sym_string_literal_token1] = ACTIONS(2614),
    [sym_char_literal] = ACTIONS(2614),
    [anon_sym_true] = ACTIONS(2616),
    [anon_sym_false] = ACTIONS(2616),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2616),
    [sym_super] = ACTIONS(2616),
    [sym_crate] = ACTIONS(2616),
    [aux_sym_metavariable_token1] = ACTIONS(2614),
    [sym__raw_string_literal_start] = ACTIONS(2614),
    [sym_float_literal] = ACTIONS(2614),
    [sym__outer_block_doc_comment_start] = ACTIONS(2614),
    [sym__outer_line_doc_comment_start] = ACTIONS(2614),
  },
  [STATE(711)] = {
    [sym_line_comment] = STATE(711),
    [sym_block_comment] = STATE(711),
    [ts_builtin_sym_end] = ACTIONS(2618),
    [sym_identifier] = ACTIONS(2620),
    [anon_sym_SEMI] = ACTIONS(2618),
    [anon_sym_macro] = ACTIONS(2620),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2618),
    [anon_sym_RBRACE] = ACTIONS(2618),
    [anon_sym_macro_rules] = ACTIONS(2620),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_LBRACK] = ACTIONS(2618),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_u8] = ACTIONS(2620),
    [anon_sym_i8] = ACTIONS(2620),
    [anon_sym_u16] = ACTIONS(2620),
    [anon_sym_i16] = ACTIONS(2620),
    [anon_sym_u32] = ACTIONS(2620),
    [anon_sym_i32] = ACTIONS(2620),
    [anon_sym_u64] = ACTIONS(2620),
    [anon_sym_i64] = ACTIONS(2620),
    [anon_sym_u128] = ACTIONS(2620),
    [anon_sym_i128] = ACTIONS(2620),
    [anon_sym_isize] = ACTIONS(2620),
    [anon_sym_usize] = ACTIONS(2620),
    [anon_sym_f32] = ACTIONS(2620),
    [anon_sym_f64] = ACTIONS(2620),
    [anon_sym_bool] = ACTIONS(2620),
    [anon_sym_str] = ACTIONS(2620),
    [anon_sym_char] = ACTIONS(2620),
    [anon_sym_DASH] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2618),
    [anon_sym_PIPE] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2618),
    [anon_sym_DOT_DOT] = ACTIONS(2618),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_POUND] = ACTIONS(2618),
    [anon_sym_SQUOTE] = ACTIONS(2620),
    [anon_sym_async] = ACTIONS(2620),
    [anon_sym_auto] = ACTIONS(2620),
    [anon_sym_await] = ACTIONS(2620),
    [anon_sym_become] = ACTIONS(2620),
    [anon_sym_break] = ACTIONS(2620),
    [anon_sym_const] = ACTIONS(2620),
    [anon_sym_continue] = ACTIONS(2620),
    [anon_sym_default] = ACTIONS(2620),
    [anon_sym_enum] = ACTIONS(2620),
    [anon_sym_fn] = ACTIONS(2620),
    [anon_sym_for] = ACTIONS(2620),
    [anon_sym_gen] = ACTIONS(2620),
    [anon_sym_if] = ACTIONS(2620),
    [anon_sym_impl] = ACTIONS(2620),
    [anon_sym_let] = ACTIONS(2620),
    [anon_sym_loop] = ACTIONS(2620),
    [anon_sym_match] = ACTIONS(2620),
    [anon_sym_mod] = ACTIONS(2620),
    [anon_sym_pub] = ACTIONS(2620),
    [anon_sym_return] = ACTIONS(2620),
    [anon_sym_raw] = ACTIONS(2620),
    [anon_sym_static] = ACTIONS(2620),
    [anon_sym_struct] = ACTIONS(2620),
    [anon_sym_trait] = ACTIONS(2620),
    [anon_sym_type] = ACTIONS(2620),
    [anon_sym_union] = ACTIONS(2620),
    [anon_sym_unsafe] = ACTIONS(2620),
    [anon_sym_use] = ACTIONS(2620),
    [anon_sym_while] = ACTIONS(2620),
    [anon_sym_extern] = ACTIONS(2620),
    [anon_sym_safe] = ACTIONS(2620),
    [anon_sym_yield] = ACTIONS(2620),
    [anon_sym_move] = ACTIONS(2620),
    [anon_sym_try] = ACTIONS(2620),
    [sym_integer_literal] = ACTIONS(2618),
    [aux_sym_string_literal_token1] = ACTIONS(2618),
    [sym_char_literal] = ACTIONS(2618),
    [anon_sym_true] = ACTIONS(2620),
    [anon_sym_false] = ACTIONS(2620),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2620),
    [sym_super] = ACTIONS(2620),
    [sym_crate] = ACTIONS(2620),
    [aux_sym_metavariable_token1] = ACTIONS(2618),
    [sym__raw_string_literal_start] = ACTIONS(2618),
    [sym_float_literal] = ACTIONS(2618),
    [sym__outer_block_doc_comment_start] = ACTIONS(2618),
    [sym__outer_line_doc_comment_start] = ACTIONS(2618),
  },
  [STATE(712)] = {
    [sym_line_comment] = STATE(712),
    [sym_block_comment] = STATE(712),
    [ts_builtin_sym_end] = ACTIONS(2622),
    [sym_identifier] = ACTIONS(2624),
    [anon_sym_SEMI] = ACTIONS(2622),
    [anon_sym_macro] = ACTIONS(2624),
    [anon_sym_LPAREN] = ACTIONS(2622),
    [anon_sym_LBRACE] = ACTIONS(2622),
    [anon_sym_RBRACE] = ACTIONS(2622),
    [anon_sym_macro_rules] = ACTIONS(2624),
    [anon_sym_BANG] = ACTIONS(2622),
    [anon_sym_LBRACK] = ACTIONS(2622),
    [anon_sym_STAR] = ACTIONS(2622),
    [anon_sym_u8] = ACTIONS(2624),
    [anon_sym_i8] = ACTIONS(2624),
    [anon_sym_u16] = ACTIONS(2624),
    [anon_sym_i16] = ACTIONS(2624),
    [anon_sym_u32] = ACTIONS(2624),
    [anon_sym_i32] = ACTIONS(2624),
    [anon_sym_u64] = ACTIONS(2624),
    [anon_sym_i64] = ACTIONS(2624),
    [anon_sym_u128] = ACTIONS(2624),
    [anon_sym_i128] = ACTIONS(2624),
    [anon_sym_isize] = ACTIONS(2624),
    [anon_sym_usize] = ACTIONS(2624),
    [anon_sym_f32] = ACTIONS(2624),
    [anon_sym_f64] = ACTIONS(2624),
    [anon_sym_bool] = ACTIONS(2624),
    [anon_sym_str] = ACTIONS(2624),
    [anon_sym_char] = ACTIONS(2624),
    [anon_sym_DASH] = ACTIONS(2622),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2622),
    [anon_sym_POUND] = ACTIONS(2622),
    [anon_sym_SQUOTE] = ACTIONS(2624),
    [anon_sym_async] = ACTIONS(2624),
    [anon_sym_auto] = ACTIONS(2624),
    [anon_sym_await] = ACTIONS(2624),
    [anon_sym_become] = ACTIONS(2624),
    [anon_sym_break] = ACTIONS(2624),
    [anon_sym_const] = ACTIONS(2624),
    [anon_sym_continue] = ACTIONS(2624),
    [anon_sym_default] = ACTIONS(2624),
    [anon_sym_enum] = ACTIONS(2624),
    [anon_sym_fn] = ACTIONS(2624),
    [anon_sym_for] = ACTIONS(2624),
    [anon_sym_gen] = ACTIONS(2624),
    [anon_sym_if] = ACTIONS(2624),
    [anon_sym_impl] = ACTIONS(2624),
    [anon_sym_let] = ACTIONS(2624),
    [anon_sym_loop] = ACTIONS(2624),
    [anon_sym_match] = ACTIONS(2624),
    [anon_sym_mod] = ACTIONS(2624),
    [anon_sym_pub] = ACTIONS(2624),
    [anon_sym_return] = ACTIONS(2624),
    [anon_sym_raw] = ACTIONS(2624),
    [anon_sym_static] = ACTIONS(2624),
    [anon_sym_struct] = ACTIONS(2624),
    [anon_sym_trait] = ACTIONS(2624),
    [anon_sym_type] = ACTIONS(2624),
    [anon_sym_union] = ACTIONS(2624),
    [anon_sym_unsafe] = ACTIONS(2624),
    [anon_sym_use] = ACTIONS(2624),
    [anon_sym_while] = ACTIONS(2624),
    [anon_sym_extern] = ACTIONS(2624),
    [anon_sym_safe] = ACTIONS(2624),
    [anon_sym_yield] = ACTIONS(2624),
    [anon_sym_move] = ACTIONS(2624),
    [anon_sym_try] = ACTIONS(2624),
    [sym_integer_literal] = ACTIONS(2622),
    [aux_sym_string_literal_token1] = ACTIONS(2622),
    [sym_char_literal] = ACTIONS(2622),
    [anon_sym_true] = ACTIONS(2624),
    [anon_sym_false] = ACTIONS(2624),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2624),
    [sym_super] = ACTIONS(2624),
    [sym_crate] = ACTIONS(2624),
    [aux_sym_metavariable_token1] = ACTIONS(2622),
    [sym__raw_string_literal_start] = ACTIONS(2622),
    [sym_float_literal] = ACTIONS(2622),
    [sym__outer_block_doc_comment_start] = ACTIONS(2622),
    [sym__outer_line_doc_comment_start] = ACTIONS(2622),
  },
  [STATE(713)] = {
    [sym_line_comment] = STATE(713),
    [sym_block_comment] = STATE(713),
    [ts_builtin_sym_end] = ACTIONS(2626),
    [sym_identifier] = ACTIONS(2628),
    [anon_sym_SEMI] = ACTIONS(2626),
    [anon_sym_macro] = ACTIONS(2628),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2626),
    [anon_sym_RBRACE] = ACTIONS(2626),
    [anon_sym_macro_rules] = ACTIONS(2628),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_LBRACK] = ACTIONS(2626),
    [anon_sym_STAR] = ACTIONS(2626),
    [anon_sym_u8] = ACTIONS(2628),
    [anon_sym_i8] = ACTIONS(2628),
    [anon_sym_u16] = ACTIONS(2628),
    [anon_sym_i16] = ACTIONS(2628),
    [anon_sym_u32] = ACTIONS(2628),
    [anon_sym_i32] = ACTIONS(2628),
    [anon_sym_u64] = ACTIONS(2628),
    [anon_sym_i64] = ACTIONS(2628),
    [anon_sym_u128] = ACTIONS(2628),
    [anon_sym_i128] = ACTIONS(2628),
    [anon_sym_isize] = ACTIONS(2628),
    [anon_sym_usize] = ACTIONS(2628),
    [anon_sym_f32] = ACTIONS(2628),
    [anon_sym_f64] = ACTIONS(2628),
    [anon_sym_bool] = ACTIONS(2628),
    [anon_sym_str] = ACTIONS(2628),
    [anon_sym_char] = ACTIONS(2628),
    [anon_sym_DASH] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_POUND] = ACTIONS(2626),
    [anon_sym_SQUOTE] = ACTIONS(2628),
    [anon_sym_async] = ACTIONS(2628),
    [anon_sym_auto] = ACTIONS(2628),
    [anon_sym_await] = ACTIONS(2628),
    [anon_sym_become] = ACTIONS(2628),
    [anon_sym_break] = ACTIONS(2628),
    [anon_sym_const] = ACTIONS(2628),
    [anon_sym_continue] = ACTIONS(2628),
    [anon_sym_default] = ACTIONS(2628),
    [anon_sym_enum] = ACTIONS(2628),
    [anon_sym_fn] = ACTIONS(2628),
    [anon_sym_for] = ACTIONS(2628),
    [anon_sym_gen] = ACTIONS(2628),
    [anon_sym_if] = ACTIONS(2628),
    [anon_sym_impl] = ACTIONS(2628),
    [anon_sym_let] = ACTIONS(2628),
    [anon_sym_loop] = ACTIONS(2628),
    [anon_sym_match] = ACTIONS(2628),
    [anon_sym_mod] = ACTIONS(2628),
    [anon_sym_pub] = ACTIONS(2628),
    [anon_sym_return] = ACTIONS(2628),
    [anon_sym_raw] = ACTIONS(2628),
    [anon_sym_static] = ACTIONS(2628),
    [anon_sym_struct] = ACTIONS(2628),
    [anon_sym_trait] = ACTIONS(2628),
    [anon_sym_type] = ACTIONS(2628),
    [anon_sym_union] = ACTIONS(2628),
    [anon_sym_unsafe] = ACTIONS(2628),
    [anon_sym_use] = ACTIONS(2628),
    [anon_sym_while] = ACTIONS(2628),
    [anon_sym_extern] = ACTIONS(2628),
    [anon_sym_safe] = ACTIONS(2628),
    [anon_sym_yield] = ACTIONS(2628),
    [anon_sym_move] = ACTIONS(2628),
    [anon_sym_try] = ACTIONS(2628),
    [sym_integer_literal] = ACTIONS(2626),
    [aux_sym_string_literal_token1] = ACTIONS(2626),
    [sym_char_literal] = ACTIONS(2626),
    [anon_sym_true] = ACTIONS(2628),
    [anon_sym_false] = ACTIONS(2628),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2628),
    [sym_super] = ACTIONS(2628),
    [sym_crate] = ACTIONS(2628),
    [aux_sym_metavariable_token1] = ACTIONS(2626),
    [sym__raw_string_literal_start] = ACTIONS(2626),
    [sym_float_literal] = ACTIONS(2626),
    [sym__outer_block_doc_comment_start] = ACTIONS(2626),
    [sym__outer_line_doc_comment_start] = ACTIONS(2626),
  },
  [STATE(714)] = {
    [sym_line_comment] = STATE(714),
    [sym_block_comment] = STATE(714),
    [ts_builtin_sym_end] = ACTIONS(2630),
    [sym_identifier] = ACTIONS(2632),
    [anon_sym_SEMI] = ACTIONS(2630),
    [anon_sym_macro] = ACTIONS(2632),
    [anon_sym_LPAREN] = ACTIONS(2630),
    [anon_sym_LBRACE] = ACTIONS(2630),
    [anon_sym_RBRACE] = ACTIONS(2630),
    [anon_sym_macro_rules] = ACTIONS(2632),
    [anon_sym_BANG] = ACTIONS(2630),
    [anon_sym_LBRACK] = ACTIONS(2630),
    [anon_sym_STAR] = ACTIONS(2630),
    [anon_sym_u8] = ACTIONS(2632),
    [anon_sym_i8] = ACTIONS(2632),
    [anon_sym_u16] = ACTIONS(2632),
    [anon_sym_i16] = ACTIONS(2632),
    [anon_sym_u32] = ACTIONS(2632),
    [anon_sym_i32] = ACTIONS(2632),
    [anon_sym_u64] = ACTIONS(2632),
    [anon_sym_i64] = ACTIONS(2632),
    [anon_sym_u128] = ACTIONS(2632),
    [anon_sym_i128] = ACTIONS(2632),
    [anon_sym_isize] = ACTIONS(2632),
    [anon_sym_usize] = ACTIONS(2632),
    [anon_sym_f32] = ACTIONS(2632),
    [anon_sym_f64] = ACTIONS(2632),
    [anon_sym_bool] = ACTIONS(2632),
    [anon_sym_str] = ACTIONS(2632),
    [anon_sym_char] = ACTIONS(2632),
    [anon_sym_DASH] = ACTIONS(2630),
    [anon_sym_AMP] = ACTIONS(2630),
    [anon_sym_PIPE] = ACTIONS(2630),
    [anon_sym_LT] = ACTIONS(2630),
    [anon_sym_DOT_DOT] = ACTIONS(2630),
    [anon_sym_COLON_COLON] = ACTIONS(2630),
    [anon_sym_POUND] = ACTIONS(2630),
    [anon_sym_SQUOTE] = ACTIONS(2632),
    [anon_sym_async] = ACTIONS(2632),
    [anon_sym_auto] = ACTIONS(2632),
    [anon_sym_await] = ACTIONS(2632),
    [anon_sym_become] = ACTIONS(2632),
    [anon_sym_break] = ACTIONS(2632),
    [anon_sym_const] = ACTIONS(2632),
    [anon_sym_continue] = ACTIONS(2632),
    [anon_sym_default] = ACTIONS(2632),
    [anon_sym_enum] = ACTIONS(2632),
    [anon_sym_fn] = ACTIONS(2632),
    [anon_sym_for] = ACTIONS(2632),
    [anon_sym_gen] = ACTIONS(2632),
    [anon_sym_if] = ACTIONS(2632),
    [anon_sym_impl] = ACTIONS(2632),
    [anon_sym_let] = ACTIONS(2632),
    [anon_sym_loop] = ACTIONS(2632),
    [anon_sym_match] = ACTIONS(2632),
    [anon_sym_mod] = ACTIONS(2632),
    [anon_sym_pub] = ACTIONS(2632),
    [anon_sym_return] = ACTIONS(2632),
    [anon_sym_raw] = ACTIONS(2632),
    [anon_sym_static] = ACTIONS(2632),
    [anon_sym_struct] = ACTIONS(2632),
    [anon_sym_trait] = ACTIONS(2632),
    [anon_sym_type] = ACTIONS(2632),
    [anon_sym_union] = ACTIONS(2632),
    [anon_sym_unsafe] = ACTIONS(2632),
    [anon_sym_use] = ACTIONS(2632),
    [anon_sym_while] = ACTIONS(2632),
    [anon_sym_extern] = ACTIONS(2632),
    [anon_sym_safe] = ACTIONS(2632),
    [anon_sym_yield] = ACTIONS(2632),
    [anon_sym_move] = ACTIONS(2632),
    [anon_sym_try] = ACTIONS(2632),
    [sym_integer_literal] = ACTIONS(2630),
    [aux_sym_string_literal_token1] = ACTIONS(2630),
    [sym_char_literal] = ACTIONS(2630),
    [anon_sym_true] = ACTIONS(2632),
    [anon_sym_false] = ACTIONS(2632),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2632),
    [sym_super] = ACTIONS(2632),
    [sym_crate] = ACTIONS(2632),
    [aux_sym_metavariable_token1] = ACTIONS(2630),
    [sym__raw_string_literal_start] = ACTIONS(2630),
    [sym_float_literal] = ACTIONS(2630),
    [sym__outer_block_doc_comment_start] = ACTIONS(2630),
    [sym__outer_line_doc_comment_start] = ACTIONS(2630),
  },
  [STATE(715)] = {
    [sym_line_comment] = STATE(715),
    [sym_block_comment] = STATE(715),
    [ts_builtin_sym_end] = ACTIONS(2634),
    [sym_identifier] = ACTIONS(2636),
    [anon_sym_SEMI] = ACTIONS(2634),
    [anon_sym_macro] = ACTIONS(2636),
    [anon_sym_LPAREN] = ACTIONS(2634),
    [anon_sym_LBRACE] = ACTIONS(2634),
    [anon_sym_RBRACE] = ACTIONS(2634),
    [anon_sym_macro_rules] = ACTIONS(2636),
    [anon_sym_BANG] = ACTIONS(2634),
    [anon_sym_LBRACK] = ACTIONS(2634),
    [anon_sym_STAR] = ACTIONS(2634),
    [anon_sym_u8] = ACTIONS(2636),
    [anon_sym_i8] = ACTIONS(2636),
    [anon_sym_u16] = ACTIONS(2636),
    [anon_sym_i16] = ACTIONS(2636),
    [anon_sym_u32] = ACTIONS(2636),
    [anon_sym_i32] = ACTIONS(2636),
    [anon_sym_u64] = ACTIONS(2636),
    [anon_sym_i64] = ACTIONS(2636),
    [anon_sym_u128] = ACTIONS(2636),
    [anon_sym_i128] = ACTIONS(2636),
    [anon_sym_isize] = ACTIONS(2636),
    [anon_sym_usize] = ACTIONS(2636),
    [anon_sym_f32] = ACTIONS(2636),
    [anon_sym_f64] = ACTIONS(2636),
    [anon_sym_bool] = ACTIONS(2636),
    [anon_sym_str] = ACTIONS(2636),
    [anon_sym_char] = ACTIONS(2636),
    [anon_sym_DASH] = ACTIONS(2634),
    [anon_sym_AMP] = ACTIONS(2634),
    [anon_sym_PIPE] = ACTIONS(2634),
    [anon_sym_LT] = ACTIONS(2634),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_COLON_COLON] = ACTIONS(2634),
    [anon_sym_POUND] = ACTIONS(2634),
    [anon_sym_SQUOTE] = ACTIONS(2636),
    [anon_sym_async] = ACTIONS(2636),
    [anon_sym_auto] = ACTIONS(2636),
    [anon_sym_await] = ACTIONS(2636),
    [anon_sym_become] = ACTIONS(2636),
    [anon_sym_break] = ACTIONS(2636),
    [anon_sym_const] = ACTIONS(2636),
    [anon_sym_continue] = ACTIONS(2636),
    [anon_sym_default] = ACTIONS(2636),
    [anon_sym_enum] = ACTIONS(2636),
    [anon_sym_fn] = ACTIONS(2636),
    [anon_sym_for] = ACTIONS(2636),
    [anon_sym_gen] = ACTIONS(2636),
    [anon_sym_if] = ACTIONS(2636),
    [anon_sym_impl] = ACTIONS(2636),
    [anon_sym_let] = ACTIONS(2636),
    [anon_sym_loop] = ACTIONS(2636),
    [anon_sym_match] = ACTIONS(2636),
    [anon_sym_mod] = ACTIONS(2636),
    [anon_sym_pub] = ACTIONS(2636),
    [anon_sym_return] = ACTIONS(2636),
    [anon_sym_raw] = ACTIONS(2636),
    [anon_sym_static] = ACTIONS(2636),
    [anon_sym_struct] = ACTIONS(2636),
    [anon_sym_trait] = ACTIONS(2636),
    [anon_sym_type] = ACTIONS(2636),
    [anon_sym_union] = ACTIONS(2636),
    [anon_sym_unsafe] = ACTIONS(2636),
    [anon_sym_use] = ACTIONS(2636),
    [anon_sym_while] = ACTIONS(2636),
    [anon_sym_extern] = ACTIONS(2636),
    [anon_sym_safe] = ACTIONS(2636),
    [anon_sym_yield] = ACTIONS(2636),
    [anon_sym_move] = ACTIONS(2636),
    [anon_sym_try] = ACTIONS(2636),
    [sym_integer_literal] = ACTIONS(2634),
    [aux_sym_string_literal_token1] = ACTIONS(2634),
    [sym_char_literal] = ACTIONS(2634),
    [anon_sym_true] = ACTIONS(2636),
    [anon_sym_false] = ACTIONS(2636),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2636),
    [sym_super] = ACTIONS(2636),
    [sym_crate] = ACTIONS(2636),
    [aux_sym_metavariable_token1] = ACTIONS(2634),
    [sym__raw_string_literal_start] = ACTIONS(2634),
    [sym_float_literal] = ACTIONS(2634),
    [sym__outer_block_doc_comment_start] = ACTIONS(2634),
    [sym__outer_line_doc_comment_start] = ACTIONS(2634),
  },
  [STATE(716)] = {
    [sym_line_comment] = STATE(716),
    [sym_block_comment] = STATE(716),
    [ts_builtin_sym_end] = ACTIONS(2638),
    [sym_identifier] = ACTIONS(2640),
    [anon_sym_SEMI] = ACTIONS(2638),
    [anon_sym_macro] = ACTIONS(2640),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2638),
    [anon_sym_RBRACE] = ACTIONS(2638),
    [anon_sym_macro_rules] = ACTIONS(2640),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_LBRACK] = ACTIONS(2638),
    [anon_sym_STAR] = ACTIONS(2638),
    [anon_sym_u8] = ACTIONS(2640),
    [anon_sym_i8] = ACTIONS(2640),
    [anon_sym_u16] = ACTIONS(2640),
    [anon_sym_i16] = ACTIONS(2640),
    [anon_sym_u32] = ACTIONS(2640),
    [anon_sym_i32] = ACTIONS(2640),
    [anon_sym_u64] = ACTIONS(2640),
    [anon_sym_i64] = ACTIONS(2640),
    [anon_sym_u128] = ACTIONS(2640),
    [anon_sym_i128] = ACTIONS(2640),
    [anon_sym_isize] = ACTIONS(2640),
    [anon_sym_usize] = ACTIONS(2640),
    [anon_sym_f32] = ACTIONS(2640),
    [anon_sym_f64] = ACTIONS(2640),
    [anon_sym_bool] = ACTIONS(2640),
    [anon_sym_str] = ACTIONS(2640),
    [anon_sym_char] = ACTIONS(2640),
    [anon_sym_DASH] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_POUND] = ACTIONS(2638),
    [anon_sym_SQUOTE] = ACTIONS(2640),
    [anon_sym_async] = ACTIONS(2640),
    [anon_sym_auto] = ACTIONS(2640),
    [anon_sym_await] = ACTIONS(2640),
    [anon_sym_become] = ACTIONS(2640),
    [anon_sym_break] = ACTIONS(2640),
    [anon_sym_const] = ACTIONS(2640),
    [anon_sym_continue] = ACTIONS(2640),
    [anon_sym_default] = ACTIONS(2640),
    [anon_sym_enum] = ACTIONS(2640),
    [anon_sym_fn] = ACTIONS(2640),
    [anon_sym_for] = ACTIONS(2640),
    [anon_sym_gen] = ACTIONS(2640),
    [anon_sym_if] = ACTIONS(2640),
    [anon_sym_impl] = ACTIONS(2640),
    [anon_sym_let] = ACTIONS(2640),
    [anon_sym_loop] = ACTIONS(2640),
    [anon_sym_match] = ACTIONS(2640),
    [anon_sym_mod] = ACTIONS(2640),
    [anon_sym_pub] = ACTIONS(2640),
    [anon_sym_return] = ACTIONS(2640),
    [anon_sym_raw] = ACTIONS(2640),
    [anon_sym_static] = ACTIONS(2640),
    [anon_sym_struct] = ACTIONS(2640),
    [anon_sym_trait] = ACTIONS(2640),
    [anon_sym_type] = ACTIONS(2640),
    [anon_sym_union] = ACTIONS(2640),
    [anon_sym_unsafe] = ACTIONS(2640),
    [anon_sym_use] = ACTIONS(2640),
    [anon_sym_while] = ACTIONS(2640),
    [anon_sym_extern] = ACTIONS(2640),
    [anon_sym_safe] = ACTIONS(2640),
    [anon_sym_yield] = ACTIONS(2640),
    [anon_sym_move] = ACTIONS(2640),
    [anon_sym_try] = ACTIONS(2640),
    [sym_integer_literal] = ACTIONS(2638),
    [aux_sym_string_literal_token1] = ACTIONS(2638),
    [sym_char_literal] = ACTIONS(2638),
    [anon_sym_true] = ACTIONS(2640),
    [anon_sym_false] = ACTIONS(2640),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2640),
    [sym_super] = ACTIONS(2640),
    [sym_crate] = ACTIONS(2640),
    [aux_sym_metavariable_token1] = ACTIONS(2638),
    [sym__raw_string_literal_start] = ACTIONS(2638),
    [sym_float_literal] = ACTIONS(2638),
    [sym__outer_block_doc_comment_start] = ACTIONS(2638),
    [sym__outer_line_doc_comment_start] = ACTIONS(2638),
  },
  [STATE(717)] = {
    [sym_line_comment] = STATE(717),
    [sym_block_comment] = STATE(717),
    [ts_builtin_sym_end] = ACTIONS(2642),
    [sym_identifier] = ACTIONS(2644),
    [anon_sym_SEMI] = ACTIONS(2642),
    [anon_sym_macro] = ACTIONS(2644),
    [anon_sym_LPAREN] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2642),
    [anon_sym_RBRACE] = ACTIONS(2642),
    [anon_sym_macro_rules] = ACTIONS(2644),
    [anon_sym_BANG] = ACTIONS(2642),
    [anon_sym_LBRACK] = ACTIONS(2642),
    [anon_sym_STAR] = ACTIONS(2642),
    [anon_sym_u8] = ACTIONS(2644),
    [anon_sym_i8] = ACTIONS(2644),
    [anon_sym_u16] = ACTIONS(2644),
    [anon_sym_i16] = ACTIONS(2644),
    [anon_sym_u32] = ACTIONS(2644),
    [anon_sym_i32] = ACTIONS(2644),
    [anon_sym_u64] = ACTIONS(2644),
    [anon_sym_i64] = ACTIONS(2644),
    [anon_sym_u128] = ACTIONS(2644),
    [anon_sym_i128] = ACTIONS(2644),
    [anon_sym_isize] = ACTIONS(2644),
    [anon_sym_usize] = ACTIONS(2644),
    [anon_sym_f32] = ACTIONS(2644),
    [anon_sym_f64] = ACTIONS(2644),
    [anon_sym_bool] = ACTIONS(2644),
    [anon_sym_str] = ACTIONS(2644),
    [anon_sym_char] = ACTIONS(2644),
    [anon_sym_DASH] = ACTIONS(2642),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_COLON_COLON] = ACTIONS(2642),
    [anon_sym_POUND] = ACTIONS(2642),
    [anon_sym_SQUOTE] = ACTIONS(2644),
    [anon_sym_async] = ACTIONS(2644),
    [anon_sym_auto] = ACTIONS(2644),
    [anon_sym_await] = ACTIONS(2644),
    [anon_sym_become] = ACTIONS(2644),
    [anon_sym_break] = ACTIONS(2644),
    [anon_sym_const] = ACTIONS(2644),
    [anon_sym_continue] = ACTIONS(2644),
    [anon_sym_default] = ACTIONS(2644),
    [anon_sym_enum] = ACTIONS(2644),
    [anon_sym_fn] = ACTIONS(2644),
    [anon_sym_for] = ACTIONS(2644),
    [anon_sym_gen] = ACTIONS(2644),
    [anon_sym_if] = ACTIONS(2644),
    [anon_sym_impl] = ACTIONS(2644),
    [anon_sym_let] = ACTIONS(2644),
    [anon_sym_loop] = ACTIONS(2644),
    [anon_sym_match] = ACTIONS(2644),
    [anon_sym_mod] = ACTIONS(2644),
    [anon_sym_pub] = ACTIONS(2644),
    [anon_sym_return] = ACTIONS(2644),
    [anon_sym_raw] = ACTIONS(2644),
    [anon_sym_static] = ACTIONS(2644),
    [anon_sym_struct] = ACTIONS(2644),
    [anon_sym_trait] = ACTIONS(2644),
    [anon_sym_type] = ACTIONS(2644),
    [anon_sym_union] = ACTIONS(2644),
    [anon_sym_unsafe] = ACTIONS(2644),
    [anon_sym_use] = ACTIONS(2644),
    [anon_sym_while] = ACTIONS(2644),
    [anon_sym_extern] = ACTIONS(2644),
    [anon_sym_safe] = ACTIONS(2644),
    [anon_sym_yield] = ACTIONS(2644),
    [anon_sym_move] = ACTIONS(2644),
    [anon_sym_try] = ACTIONS(2644),
    [sym_integer_literal] = ACTIONS(2642),
    [aux_sym_string_literal_token1] = ACTIONS(2642),
    [sym_char_literal] = ACTIONS(2642),
    [anon_sym_true] = ACTIONS(2644),
    [anon_sym_false] = ACTIONS(2644),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2644),
    [sym_super] = ACTIONS(2644),
    [sym_crate] = ACTIONS(2644),
    [aux_sym_metavariable_token1] = ACTIONS(2642),
    [sym__raw_string_literal_start] = ACTIONS(2642),
    [sym_float_literal] = ACTIONS(2642),
    [sym__outer_block_doc_comment_start] = ACTIONS(2642),
    [sym__outer_line_doc_comment_start] = ACTIONS(2642),
  },
  [STATE(718)] = {
    [sym_line_comment] = STATE(718),
    [sym_block_comment] = STATE(718),
    [ts_builtin_sym_end] = ACTIONS(2646),
    [sym_identifier] = ACTIONS(2648),
    [anon_sym_SEMI] = ACTIONS(2646),
    [anon_sym_macro] = ACTIONS(2648),
    [anon_sym_LPAREN] = ACTIONS(2646),
    [anon_sym_LBRACE] = ACTIONS(2646),
    [anon_sym_RBRACE] = ACTIONS(2646),
    [anon_sym_macro_rules] = ACTIONS(2648),
    [anon_sym_BANG] = ACTIONS(2646),
    [anon_sym_LBRACK] = ACTIONS(2646),
    [anon_sym_STAR] = ACTIONS(2646),
    [anon_sym_u8] = ACTIONS(2648),
    [anon_sym_i8] = ACTIONS(2648),
    [anon_sym_u16] = ACTIONS(2648),
    [anon_sym_i16] = ACTIONS(2648),
    [anon_sym_u32] = ACTIONS(2648),
    [anon_sym_i32] = ACTIONS(2648),
    [anon_sym_u64] = ACTIONS(2648),
    [anon_sym_i64] = ACTIONS(2648),
    [anon_sym_u128] = ACTIONS(2648),
    [anon_sym_i128] = ACTIONS(2648),
    [anon_sym_isize] = ACTIONS(2648),
    [anon_sym_usize] = ACTIONS(2648),
    [anon_sym_f32] = ACTIONS(2648),
    [anon_sym_f64] = ACTIONS(2648),
    [anon_sym_bool] = ACTIONS(2648),
    [anon_sym_str] = ACTIONS(2648),
    [anon_sym_char] = ACTIONS(2648),
    [anon_sym_DASH] = ACTIONS(2646),
    [anon_sym_AMP] = ACTIONS(2646),
    [anon_sym_PIPE] = ACTIONS(2646),
    [anon_sym_LT] = ACTIONS(2646),
    [anon_sym_DOT_DOT] = ACTIONS(2646),
    [anon_sym_COLON_COLON] = ACTIONS(2646),
    [anon_sym_POUND] = ACTIONS(2646),
    [anon_sym_SQUOTE] = ACTIONS(2648),
    [anon_sym_async] = ACTIONS(2648),
    [anon_sym_auto] = ACTIONS(2648),
    [anon_sym_await] = ACTIONS(2648),
    [anon_sym_become] = ACTIONS(2648),
    [anon_sym_break] = ACTIONS(2648),
    [anon_sym_const] = ACTIONS(2648),
    [anon_sym_continue] = ACTIONS(2648),
    [anon_sym_default] = ACTIONS(2648),
    [anon_sym_enum] = ACTIONS(2648),
    [anon_sym_fn] = ACTIONS(2648),
    [anon_sym_for] = ACTIONS(2648),
    [anon_sym_gen] = ACTIONS(2648),
    [anon_sym_if] = ACTIONS(2648),
    [anon_sym_impl] = ACTIONS(2648),
    [anon_sym_let] = ACTIONS(2648),
    [anon_sym_loop] = ACTIONS(2648),
    [anon_sym_match] = ACTIONS(2648),
    [anon_sym_mod] = ACTIONS(2648),
    [anon_sym_pub] = ACTIONS(2648),
    [anon_sym_return] = ACTIONS(2648),
    [anon_sym_raw] = ACTIONS(2648),
    [anon_sym_static] = ACTIONS(2648),
    [anon_sym_struct] = ACTIONS(2648),
    [anon_sym_trait] = ACTIONS(2648),
    [anon_sym_type] = ACTIONS(2648),
    [anon_sym_union] = ACTIONS(2648),
    [anon_sym_unsafe] = ACTIONS(2648),
    [anon_sym_use] = ACTIONS(2648),
    [anon_sym_while] = ACTIONS(2648),
    [anon_sym_extern] = ACTIONS(2648),
    [anon_sym_safe] = ACTIONS(2648),
    [anon_sym_yield] = ACTIONS(2648),
    [anon_sym_move] = ACTIONS(2648),
    [anon_sym_try] = ACTIONS(2648),
    [sym_integer_literal] = ACTIONS(2646),
    [aux_sym_string_literal_token1] = ACTIONS(2646),
    [sym_char_literal] = ACTIONS(2646),
    [anon_sym_true] = ACTIONS(2648),
    [anon_sym_false] = ACTIONS(2648),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2648),
    [sym_super] = ACTIONS(2648),
    [sym_crate] = ACTIONS(2648),
    [aux_sym_metavariable_token1] = ACTIONS(2646),
    [sym__raw_string_literal_start] = ACTIONS(2646),
    [sym_float_literal] = ACTIONS(2646),
    [sym__outer_block_doc_comment_start] = ACTIONS(2646),
    [sym__outer_line_doc_comment_start] = ACTIONS(2646),
  },
  [STATE(719)] = {
    [sym_line_comment] = STATE(719),
    [sym_block_comment] = STATE(719),
    [ts_builtin_sym_end] = ACTIONS(2650),
    [sym_identifier] = ACTIONS(2652),
    [anon_sym_SEMI] = ACTIONS(2650),
    [anon_sym_macro] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2650),
    [anon_sym_LBRACE] = ACTIONS(2650),
    [anon_sym_RBRACE] = ACTIONS(2650),
    [anon_sym_macro_rules] = ACTIONS(2652),
    [anon_sym_BANG] = ACTIONS(2650),
    [anon_sym_LBRACK] = ACTIONS(2650),
    [anon_sym_STAR] = ACTIONS(2650),
    [anon_sym_u8] = ACTIONS(2652),
    [anon_sym_i8] = ACTIONS(2652),
    [anon_sym_u16] = ACTIONS(2652),
    [anon_sym_i16] = ACTIONS(2652),
    [anon_sym_u32] = ACTIONS(2652),
    [anon_sym_i32] = ACTIONS(2652),
    [anon_sym_u64] = ACTIONS(2652),
    [anon_sym_i64] = ACTIONS(2652),
    [anon_sym_u128] = ACTIONS(2652),
    [anon_sym_i128] = ACTIONS(2652),
    [anon_sym_isize] = ACTIONS(2652),
    [anon_sym_usize] = ACTIONS(2652),
    [anon_sym_f32] = ACTIONS(2652),
    [anon_sym_f64] = ACTIONS(2652),
    [anon_sym_bool] = ACTIONS(2652),
    [anon_sym_str] = ACTIONS(2652),
    [anon_sym_char] = ACTIONS(2652),
    [anon_sym_DASH] = ACTIONS(2650),
    [anon_sym_AMP] = ACTIONS(2650),
    [anon_sym_PIPE] = ACTIONS(2650),
    [anon_sym_LT] = ACTIONS(2650),
    [anon_sym_DOT_DOT] = ACTIONS(2650),
    [anon_sym_COLON_COLON] = ACTIONS(2650),
    [anon_sym_POUND] = ACTIONS(2650),
    [anon_sym_SQUOTE] = ACTIONS(2652),
    [anon_sym_async] = ACTIONS(2652),
    [anon_sym_auto] = ACTIONS(2652),
    [anon_sym_await] = ACTIONS(2652),
    [anon_sym_become] = ACTIONS(2652),
    [anon_sym_break] = ACTIONS(2652),
    [anon_sym_const] = ACTIONS(2652),
    [anon_sym_continue] = ACTIONS(2652),
    [anon_sym_default] = ACTIONS(2652),
    [anon_sym_enum] = ACTIONS(2652),
    [anon_sym_fn] = ACTIONS(2652),
    [anon_sym_for] = ACTIONS(2652),
    [anon_sym_gen] = ACTIONS(2652),
    [anon_sym_if] = ACTIONS(2652),
    [anon_sym_impl] = ACTIONS(2652),
    [anon_sym_let] = ACTIONS(2652),
    [anon_sym_loop] = ACTIONS(2652),
    [anon_sym_match] = ACTIONS(2652),
    [anon_sym_mod] = ACTIONS(2652),
    [anon_sym_pub] = ACTIONS(2652),
    [anon_sym_return] = ACTIONS(2652),
    [anon_sym_raw] = ACTIONS(2652),
    [anon_sym_static] = ACTIONS(2652),
    [anon_sym_struct] = ACTIONS(2652),
    [anon_sym_trait] = ACTIONS(2652),
    [anon_sym_type] = ACTIONS(2652),
    [anon_sym_union] = ACTIONS(2652),
    [anon_sym_unsafe] = ACTIONS(2652),
    [anon_sym_use] = ACTIONS(2652),
    [anon_sym_while] = ACTIONS(2652),
    [anon_sym_extern] = ACTIONS(2652),
    [anon_sym_safe] = ACTIONS(2652),
    [anon_sym_yield] = ACTIONS(2652),
    [anon_sym_move] = ACTIONS(2652),
    [anon_sym_try] = ACTIONS(2652),
    [sym_integer_literal] = ACTIONS(2650),
    [aux_sym_string_literal_token1] = ACTIONS(2650),
    [sym_char_literal] = ACTIONS(2650),
    [anon_sym_true] = ACTIONS(2652),
    [anon_sym_false] = ACTIONS(2652),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2652),
    [sym_super] = ACTIONS(2652),
    [sym_crate] = ACTIONS(2652),
    [aux_sym_metavariable_token1] = ACTIONS(2650),
    [sym__raw_string_literal_start] = ACTIONS(2650),
    [sym_float_literal] = ACTIONS(2650),
    [sym__outer_block_doc_comment_start] = ACTIONS(2650),
    [sym__outer_line_doc_comment_start] = ACTIONS(2650),
  },
  [STATE(720)] = {
    [sym_line_comment] = STATE(720),
    [sym_block_comment] = STATE(720),
    [ts_builtin_sym_end] = ACTIONS(2654),
    [sym_identifier] = ACTIONS(2656),
    [anon_sym_SEMI] = ACTIONS(2654),
    [anon_sym_macro] = ACTIONS(2656),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_LBRACE] = ACTIONS(2654),
    [anon_sym_RBRACE] = ACTIONS(2654),
    [anon_sym_macro_rules] = ACTIONS(2656),
    [anon_sym_BANG] = ACTIONS(2654),
    [anon_sym_LBRACK] = ACTIONS(2654),
    [anon_sym_STAR] = ACTIONS(2654),
    [anon_sym_u8] = ACTIONS(2656),
    [anon_sym_i8] = ACTIONS(2656),
    [anon_sym_u16] = ACTIONS(2656),
    [anon_sym_i16] = ACTIONS(2656),
    [anon_sym_u32] = ACTIONS(2656),
    [anon_sym_i32] = ACTIONS(2656),
    [anon_sym_u64] = ACTIONS(2656),
    [anon_sym_i64] = ACTIONS(2656),
    [anon_sym_u128] = ACTIONS(2656),
    [anon_sym_i128] = ACTIONS(2656),
    [anon_sym_isize] = ACTIONS(2656),
    [anon_sym_usize] = ACTIONS(2656),
    [anon_sym_f32] = ACTIONS(2656),
    [anon_sym_f64] = ACTIONS(2656),
    [anon_sym_bool] = ACTIONS(2656),
    [anon_sym_str] = ACTIONS(2656),
    [anon_sym_char] = ACTIONS(2656),
    [anon_sym_DASH] = ACTIONS(2654),
    [anon_sym_AMP] = ACTIONS(2654),
    [anon_sym_PIPE] = ACTIONS(2654),
    [anon_sym_LT] = ACTIONS(2654),
    [anon_sym_DOT_DOT] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2654),
    [anon_sym_POUND] = ACTIONS(2654),
    [anon_sym_SQUOTE] = ACTIONS(2656),
    [anon_sym_async] = ACTIONS(2656),
    [anon_sym_auto] = ACTIONS(2656),
    [anon_sym_await] = ACTIONS(2656),
    [anon_sym_become] = ACTIONS(2656),
    [anon_sym_break] = ACTIONS(2656),
    [anon_sym_const] = ACTIONS(2656),
    [anon_sym_continue] = ACTIONS(2656),
    [anon_sym_default] = ACTIONS(2656),
    [anon_sym_enum] = ACTIONS(2656),
    [anon_sym_fn] = ACTIONS(2656),
    [anon_sym_for] = ACTIONS(2656),
    [anon_sym_gen] = ACTIONS(2656),
    [anon_sym_if] = ACTIONS(2656),
    [anon_sym_impl] = ACTIONS(2656),
    [anon_sym_let] = ACTIONS(2656),
    [anon_sym_loop] = ACTIONS(2656),
    [anon_sym_match] = ACTIONS(2656),
    [anon_sym_mod] = ACTIONS(2656),
    [anon_sym_pub] = ACTIONS(2656),
    [anon_sym_return] = ACTIONS(2656),
    [anon_sym_raw] = ACTIONS(2656),
    [anon_sym_static] = ACTIONS(2656),
    [anon_sym_struct] = ACTIONS(2656),
    [anon_sym_trait] = ACTIONS(2656),
    [anon_sym_type] = ACTIONS(2656),
    [anon_sym_union] = ACTIONS(2656),
    [anon_sym_unsafe] = ACTIONS(2656),
    [anon_sym_use] = ACTIONS(2656),
    [anon_sym_while] = ACTIONS(2656),
    [anon_sym_extern] = ACTIONS(2656),
    [anon_sym_safe] = ACTIONS(2656),
    [anon_sym_yield] = ACTIONS(2656),
    [anon_sym_move] = ACTIONS(2656),
    [anon_sym_try] = ACTIONS(2656),
    [sym_integer_literal] = ACTIONS(2654),
    [aux_sym_string_literal_token1] = ACTIONS(2654),
    [sym_char_literal] = ACTIONS(2654),
    [anon_sym_true] = ACTIONS(2656),
    [anon_sym_false] = ACTIONS(2656),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2656),
    [sym_super] = ACTIONS(2656),
    [sym_crate] = ACTIONS(2656),
    [aux_sym_metavariable_token1] = ACTIONS(2654),
    [sym__raw_string_literal_start] = ACTIONS(2654),
    [sym_float_literal] = ACTIONS(2654),
    [sym__outer_block_doc_comment_start] = ACTIONS(2654),
    [sym__outer_line_doc_comment_start] = ACTIONS(2654),
  },
  [STATE(721)] = {
    [sym_line_comment] = STATE(721),
    [sym_block_comment] = STATE(721),
    [ts_builtin_sym_end] = ACTIONS(2658),
    [sym_identifier] = ACTIONS(2660),
    [anon_sym_SEMI] = ACTIONS(2658),
    [anon_sym_macro] = ACTIONS(2660),
    [anon_sym_LPAREN] = ACTIONS(2658),
    [anon_sym_LBRACE] = ACTIONS(2658),
    [anon_sym_RBRACE] = ACTIONS(2658),
    [anon_sym_macro_rules] = ACTIONS(2660),
    [anon_sym_BANG] = ACTIONS(2658),
    [anon_sym_LBRACK] = ACTIONS(2658),
    [anon_sym_STAR] = ACTIONS(2658),
    [anon_sym_u8] = ACTIONS(2660),
    [anon_sym_i8] = ACTIONS(2660),
    [anon_sym_u16] = ACTIONS(2660),
    [anon_sym_i16] = ACTIONS(2660),
    [anon_sym_u32] = ACTIONS(2660),
    [anon_sym_i32] = ACTIONS(2660),
    [anon_sym_u64] = ACTIONS(2660),
    [anon_sym_i64] = ACTIONS(2660),
    [anon_sym_u128] = ACTIONS(2660),
    [anon_sym_i128] = ACTIONS(2660),
    [anon_sym_isize] = ACTIONS(2660),
    [anon_sym_usize] = ACTIONS(2660),
    [anon_sym_f32] = ACTIONS(2660),
    [anon_sym_f64] = ACTIONS(2660),
    [anon_sym_bool] = ACTIONS(2660),
    [anon_sym_str] = ACTIONS(2660),
    [anon_sym_char] = ACTIONS(2660),
    [anon_sym_DASH] = ACTIONS(2658),
    [anon_sym_AMP] = ACTIONS(2658),
    [anon_sym_PIPE] = ACTIONS(2658),
    [anon_sym_LT] = ACTIONS(2658),
    [anon_sym_DOT_DOT] = ACTIONS(2658),
    [anon_sym_COLON_COLON] = ACTIONS(2658),
    [anon_sym_POUND] = ACTIONS(2658),
    [anon_sym_SQUOTE] = ACTIONS(2660),
    [anon_sym_async] = ACTIONS(2660),
    [anon_sym_auto] = ACTIONS(2660),
    [anon_sym_await] = ACTIONS(2660),
    [anon_sym_become] = ACTIONS(2660),
    [anon_sym_break] = ACTIONS(2660),
    [anon_sym_const] = ACTIONS(2660),
    [anon_sym_continue] = ACTIONS(2660),
    [anon_sym_default] = ACTIONS(2660),
    [anon_sym_enum] = ACTIONS(2660),
    [anon_sym_fn] = ACTIONS(2660),
    [anon_sym_for] = ACTIONS(2660),
    [anon_sym_gen] = ACTIONS(2660),
    [anon_sym_if] = ACTIONS(2660),
    [anon_sym_impl] = ACTIONS(2660),
    [anon_sym_let] = ACTIONS(2660),
    [anon_sym_loop] = ACTIONS(2660),
    [anon_sym_match] = ACTIONS(2660),
    [anon_sym_mod] = ACTIONS(2660),
    [anon_sym_pub] = ACTIONS(2660),
    [anon_sym_return] = ACTIONS(2660),
    [anon_sym_raw] = ACTIONS(2660),
    [anon_sym_static] = ACTIONS(2660),
    [anon_sym_struct] = ACTIONS(2660),
    [anon_sym_trait] = ACTIONS(2660),
    [anon_sym_type] = ACTIONS(2660),
    [anon_sym_union] = ACTIONS(2660),
    [anon_sym_unsafe] = ACTIONS(2660),
    [anon_sym_use] = ACTIONS(2660),
    [anon_sym_while] = ACTIONS(2660),
    [anon_sym_extern] = ACTIONS(2660),
    [anon_sym_safe] = ACTIONS(2660),
    [anon_sym_yield] = ACTIONS(2660),
    [anon_sym_move] = ACTIONS(2660),
    [anon_sym_try] = ACTIONS(2660),
    [sym_integer_literal] = ACTIONS(2658),
    [aux_sym_string_literal_token1] = ACTIONS(2658),
    [sym_char_literal] = ACTIONS(2658),
    [anon_sym_true] = ACTIONS(2660),
    [anon_sym_false] = ACTIONS(2660),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2660),
    [sym_super] = ACTIONS(2660),
    [sym_crate] = ACTIONS(2660),
    [aux_sym_metavariable_token1] = ACTIONS(2658),
    [sym__raw_string_literal_start] = ACTIONS(2658),
    [sym_float_literal] = ACTIONS(2658),
    [sym__outer_block_doc_comment_start] = ACTIONS(2658),
    [sym__outer_line_doc_comment_start] = ACTIONS(2658),
  },
  [STATE(722)] = {
    [sym_line_comment] = STATE(722),
    [sym_block_comment] = STATE(722),
    [ts_builtin_sym_end] = ACTIONS(2662),
    [sym_identifier] = ACTIONS(2664),
    [anon_sym_SEMI] = ACTIONS(2662),
    [anon_sym_macro] = ACTIONS(2664),
    [anon_sym_LPAREN] = ACTIONS(2662),
    [anon_sym_LBRACE] = ACTIONS(2662),
    [anon_sym_RBRACE] = ACTIONS(2662),
    [anon_sym_macro_rules] = ACTIONS(2664),
    [anon_sym_BANG] = ACTIONS(2662),
    [anon_sym_LBRACK] = ACTIONS(2662),
    [anon_sym_STAR] = ACTIONS(2662),
    [anon_sym_u8] = ACTIONS(2664),
    [anon_sym_i8] = ACTIONS(2664),
    [anon_sym_u16] = ACTIONS(2664),
    [anon_sym_i16] = ACTIONS(2664),
    [anon_sym_u32] = ACTIONS(2664),
    [anon_sym_i32] = ACTIONS(2664),
    [anon_sym_u64] = ACTIONS(2664),
    [anon_sym_i64] = ACTIONS(2664),
    [anon_sym_u128] = ACTIONS(2664),
    [anon_sym_i128] = ACTIONS(2664),
    [anon_sym_isize] = ACTIONS(2664),
    [anon_sym_usize] = ACTIONS(2664),
    [anon_sym_f32] = ACTIONS(2664),
    [anon_sym_f64] = ACTIONS(2664),
    [anon_sym_bool] = ACTIONS(2664),
    [anon_sym_str] = ACTIONS(2664),
    [anon_sym_char] = ACTIONS(2664),
    [anon_sym_DASH] = ACTIONS(2662),
    [anon_sym_AMP] = ACTIONS(2662),
    [anon_sym_PIPE] = ACTIONS(2662),
    [anon_sym_LT] = ACTIONS(2662),
    [anon_sym_DOT_DOT] = ACTIONS(2662),
    [anon_sym_COLON_COLON] = ACTIONS(2662),
    [anon_sym_POUND] = ACTIONS(2662),
    [anon_sym_SQUOTE] = ACTIONS(2664),
    [anon_sym_async] = ACTIONS(2664),
    [anon_sym_auto] = ACTIONS(2664),
    [anon_sym_await] = ACTIONS(2664),
    [anon_sym_become] = ACTIONS(2664),
    [anon_sym_break] = ACTIONS(2664),
    [anon_sym_const] = ACTIONS(2664),
    [anon_sym_continue] = ACTIONS(2664),
    [anon_sym_default] = ACTIONS(2664),
    [anon_sym_enum] = ACTIONS(2664),
    [anon_sym_fn] = ACTIONS(2664),
    [anon_sym_for] = ACTIONS(2664),
    [anon_sym_gen] = ACTIONS(2664),
    [anon_sym_if] = ACTIONS(2664),
    [anon_sym_impl] = ACTIONS(2664),
    [anon_sym_let] = ACTIONS(2664),
    [anon_sym_loop] = ACTIONS(2664),
    [anon_sym_match] = ACTIONS(2664),
    [anon_sym_mod] = ACTIONS(2664),
    [anon_sym_pub] = ACTIONS(2664),
    [anon_sym_return] = ACTIONS(2664),
    [anon_sym_raw] = ACTIONS(2664),
    [anon_sym_static] = ACTIONS(2664),
    [anon_sym_struct] = ACTIONS(2664),
    [anon_sym_trait] = ACTIONS(2664),
    [anon_sym_type] = ACTIONS(2664),
    [anon_sym_union] = ACTIONS(2664),
    [anon_sym_unsafe] = ACTIONS(2664),
    [anon_sym_use] = ACTIONS(2664),
    [anon_sym_while] = ACTIONS(2664),
    [anon_sym_extern] = ACTIONS(2664),
    [anon_sym_safe] = ACTIONS(2664),
    [anon_sym_yield] = ACTIONS(2664),
    [anon_sym_move] = ACTIONS(2664),
    [anon_sym_try] = ACTIONS(2664),
    [sym_integer_literal] = ACTIONS(2662),
    [aux_sym_string_literal_token1] = ACTIONS(2662),
    [sym_char_literal] = ACTIONS(2662),
    [anon_sym_true] = ACTIONS(2664),
    [anon_sym_false] = ACTIONS(2664),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2664),
    [sym_super] = ACTIONS(2664),
    [sym_crate] = ACTIONS(2664),
    [aux_sym_metavariable_token1] = ACTIONS(2662),
    [sym__raw_string_literal_start] = ACTIONS(2662),
    [sym_float_literal] = ACTIONS(2662),
    [sym__outer_block_doc_comment_start] = ACTIONS(2662),
    [sym__outer_line_doc_comment_start] = ACTIONS(2662),
  },
  [STATE(723)] = {
    [sym_line_comment] = STATE(723),
    [sym_block_comment] = STATE(723),
    [ts_builtin_sym_end] = ACTIONS(2666),
    [sym_identifier] = ACTIONS(2668),
    [anon_sym_SEMI] = ACTIONS(2666),
    [anon_sym_macro] = ACTIONS(2668),
    [anon_sym_LPAREN] = ACTIONS(2666),
    [anon_sym_LBRACE] = ACTIONS(2666),
    [anon_sym_RBRACE] = ACTIONS(2666),
    [anon_sym_macro_rules] = ACTIONS(2668),
    [anon_sym_BANG] = ACTIONS(2666),
    [anon_sym_LBRACK] = ACTIONS(2666),
    [anon_sym_STAR] = ACTIONS(2666),
    [anon_sym_u8] = ACTIONS(2668),
    [anon_sym_i8] = ACTIONS(2668),
    [anon_sym_u16] = ACTIONS(2668),
    [anon_sym_i16] = ACTIONS(2668),
    [anon_sym_u32] = ACTIONS(2668),
    [anon_sym_i32] = ACTIONS(2668),
    [anon_sym_u64] = ACTIONS(2668),
    [anon_sym_i64] = ACTIONS(2668),
    [anon_sym_u128] = ACTIONS(2668),
    [anon_sym_i128] = ACTIONS(2668),
    [anon_sym_isize] = ACTIONS(2668),
    [anon_sym_usize] = ACTIONS(2668),
    [anon_sym_f32] = ACTIONS(2668),
    [anon_sym_f64] = ACTIONS(2668),
    [anon_sym_bool] = ACTIONS(2668),
    [anon_sym_str] = ACTIONS(2668),
    [anon_sym_char] = ACTIONS(2668),
    [anon_sym_DASH] = ACTIONS(2666),
    [anon_sym_AMP] = ACTIONS(2666),
    [anon_sym_PIPE] = ACTIONS(2666),
    [anon_sym_LT] = ACTIONS(2666),
    [anon_sym_DOT_DOT] = ACTIONS(2666),
    [anon_sym_COLON_COLON] = ACTIONS(2666),
    [anon_sym_POUND] = ACTIONS(2666),
    [anon_sym_SQUOTE] = ACTIONS(2668),
    [anon_sym_async] = ACTIONS(2668),
    [anon_sym_auto] = ACTIONS(2668),
    [anon_sym_await] = ACTIONS(2668),
    [anon_sym_become] = ACTIONS(2668),
    [anon_sym_break] = ACTIONS(2668),
    [anon_sym_const] = ACTIONS(2668),
    [anon_sym_continue] = ACTIONS(2668),
    [anon_sym_default] = ACTIONS(2668),
    [anon_sym_enum] = ACTIONS(2668),
    [anon_sym_fn] = ACTIONS(2668),
    [anon_sym_for] = ACTIONS(2668),
    [anon_sym_gen] = ACTIONS(2668),
    [anon_sym_if] = ACTIONS(2668),
    [anon_sym_impl] = ACTIONS(2668),
    [anon_sym_let] = ACTIONS(2668),
    [anon_sym_loop] = ACTIONS(2668),
    [anon_sym_match] = ACTIONS(2668),
    [anon_sym_mod] = ACTIONS(2668),
    [anon_sym_pub] = ACTIONS(2668),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_raw] = ACTIONS(2668),
    [anon_sym_static] = ACTIONS(2668),
    [anon_sym_struct] = ACTIONS(2668),
    [anon_sym_trait] = ACTIONS(2668),
    [anon_sym_type] = ACTIONS(2668),
    [anon_sym_union] = ACTIONS(2668),
    [anon_sym_unsafe] = ACTIONS(2668),
    [anon_sym_use] = ACTIONS(2668),
    [anon_sym_while] = ACTIONS(2668),
    [anon_sym_extern] = ACTIONS(2668),
    [anon_sym_safe] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2668),
    [anon_sym_move] = ACTIONS(2668),
    [anon_sym_try] = ACTIONS(2668),
    [sym_integer_literal] = ACTIONS(2666),
    [aux_sym_string_literal_token1] = ACTIONS(2666),
    [sym_char_literal] = ACTIONS(2666),
    [anon_sym_true] = ACTIONS(2668),
    [anon_sym_false] = ACTIONS(2668),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2668),
    [sym_super] = ACTIONS(2668),
    [sym_crate] = ACTIONS(2668),
    [aux_sym_metavariable_token1] = ACTIONS(2666),
    [sym__raw_string_literal_start] = ACTIONS(2666),
    [sym_float_literal] = ACTIONS(2666),
    [sym__outer_block_doc_comment_start] = ACTIONS(2666),
    [sym__outer_line_doc_comment_start] = ACTIONS(2666),
  },
  [STATE(724)] = {
    [sym_line_comment] = STATE(724),
    [sym_block_comment] = STATE(724),
    [ts_builtin_sym_end] = ACTIONS(2670),
    [sym_identifier] = ACTIONS(2672),
    [anon_sym_SEMI] = ACTIONS(2670),
    [anon_sym_macro] = ACTIONS(2672),
    [anon_sym_LPAREN] = ACTIONS(2670),
    [anon_sym_LBRACE] = ACTIONS(2670),
    [anon_sym_RBRACE] = ACTIONS(2670),
    [anon_sym_macro_rules] = ACTIONS(2672),
    [anon_sym_BANG] = ACTIONS(2670),
    [anon_sym_LBRACK] = ACTIONS(2670),
    [anon_sym_STAR] = ACTIONS(2670),
    [anon_sym_u8] = ACTIONS(2672),
    [anon_sym_i8] = ACTIONS(2672),
    [anon_sym_u16] = ACTIONS(2672),
    [anon_sym_i16] = ACTIONS(2672),
    [anon_sym_u32] = ACTIONS(2672),
    [anon_sym_i32] = ACTIONS(2672),
    [anon_sym_u64] = ACTIONS(2672),
    [anon_sym_i64] = ACTIONS(2672),
    [anon_sym_u128] = ACTIONS(2672),
    [anon_sym_i128] = ACTIONS(2672),
    [anon_sym_isize] = ACTIONS(2672),
    [anon_sym_usize] = ACTIONS(2672),
    [anon_sym_f32] = ACTIONS(2672),
    [anon_sym_f64] = ACTIONS(2672),
    [anon_sym_bool] = ACTIONS(2672),
    [anon_sym_str] = ACTIONS(2672),
    [anon_sym_char] = ACTIONS(2672),
    [anon_sym_DASH] = ACTIONS(2670),
    [anon_sym_AMP] = ACTIONS(2670),
    [anon_sym_PIPE] = ACTIONS(2670),
    [anon_sym_LT] = ACTIONS(2670),
    [anon_sym_DOT_DOT] = ACTIONS(2670),
    [anon_sym_COLON_COLON] = ACTIONS(2670),
    [anon_sym_POUND] = ACTIONS(2670),
    [anon_sym_SQUOTE] = ACTIONS(2672),
    [anon_sym_async] = ACTIONS(2672),
    [anon_sym_auto] = ACTIONS(2672),
    [anon_sym_await] = ACTIONS(2672),
    [anon_sym_become] = ACTIONS(2672),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_const] = ACTIONS(2672),
    [anon_sym_continue] = ACTIONS(2672),
    [anon_sym_default] = ACTIONS(2672),
    [anon_sym_enum] = ACTIONS(2672),
    [anon_sym_fn] = ACTIONS(2672),
    [anon_sym_for] = ACTIONS(2672),
    [anon_sym_gen] = ACTIONS(2672),
    [anon_sym_if] = ACTIONS(2672),
    [anon_sym_impl] = ACTIONS(2672),
    [anon_sym_let] = ACTIONS(2672),
    [anon_sym_loop] = ACTIONS(2672),
    [anon_sym_match] = ACTIONS(2672),
    [anon_sym_mod] = ACTIONS(2672),
    [anon_sym_pub] = ACTIONS(2672),
    [anon_sym_return] = ACTIONS(2672),
    [anon_sym_raw] = ACTIONS(2672),
    [anon_sym_static] = ACTIONS(2672),
    [anon_sym_struct] = ACTIONS(2672),
    [anon_sym_trait] = ACTIONS(2672),
    [anon_sym_type] = ACTIONS(2672),
    [anon_sym_union] = ACTIONS(2672),
    [anon_sym_unsafe] = ACTIONS(2672),
    [anon_sym_use] = ACTIONS(2672),
    [anon_sym_while] = ACTIONS(2672),
    [anon_sym_extern] = ACTIONS(2672),
    [anon_sym_safe] = ACTIONS(2672),
    [anon_sym_yield] = ACTIONS(2672),
    [anon_sym_move] = ACTIONS(2672),
    [anon_sym_try] = ACTIONS(2672),
    [sym_integer_literal] = ACTIONS(2670),
    [aux_sym_string_literal_token1] = ACTIONS(2670),
    [sym_char_literal] = ACTIONS(2670),
    [anon_sym_true] = ACTIONS(2672),
    [anon_sym_false] = ACTIONS(2672),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2672),
    [sym_super] = ACTIONS(2672),
    [sym_crate] = ACTIONS(2672),
    [aux_sym_metavariable_token1] = ACTIONS(2670),
    [sym__raw_string_literal_start] = ACTIONS(2670),
    [sym_float_literal] = ACTIONS(2670),
    [sym__outer_block_doc_comment_start] = ACTIONS(2670),
    [sym__outer_line_doc_comment_start] = ACTIONS(2670),
  },
  [STATE(725)] = {
    [sym_line_comment] = STATE(725),
    [sym_block_comment] = STATE(725),
    [ts_builtin_sym_end] = ACTIONS(2674),
    [sym_identifier] = ACTIONS(2676),
    [anon_sym_SEMI] = ACTIONS(2674),
    [anon_sym_macro] = ACTIONS(2676),
    [anon_sym_LPAREN] = ACTIONS(2674),
    [anon_sym_LBRACE] = ACTIONS(2674),
    [anon_sym_RBRACE] = ACTIONS(2674),
    [anon_sym_macro_rules] = ACTIONS(2676),
    [anon_sym_BANG] = ACTIONS(2674),
    [anon_sym_LBRACK] = ACTIONS(2674),
    [anon_sym_STAR] = ACTIONS(2674),
    [anon_sym_u8] = ACTIONS(2676),
    [anon_sym_i8] = ACTIONS(2676),
    [anon_sym_u16] = ACTIONS(2676),
    [anon_sym_i16] = ACTIONS(2676),
    [anon_sym_u32] = ACTIONS(2676),
    [anon_sym_i32] = ACTIONS(2676),
    [anon_sym_u64] = ACTIONS(2676),
    [anon_sym_i64] = ACTIONS(2676),
    [anon_sym_u128] = ACTIONS(2676),
    [anon_sym_i128] = ACTIONS(2676),
    [anon_sym_isize] = ACTIONS(2676),
    [anon_sym_usize] = ACTIONS(2676),
    [anon_sym_f32] = ACTIONS(2676),
    [anon_sym_f64] = ACTIONS(2676),
    [anon_sym_bool] = ACTIONS(2676),
    [anon_sym_str] = ACTIONS(2676),
    [anon_sym_char] = ACTIONS(2676),
    [anon_sym_DASH] = ACTIONS(2674),
    [anon_sym_AMP] = ACTIONS(2674),
    [anon_sym_PIPE] = ACTIONS(2674),
    [anon_sym_LT] = ACTIONS(2674),
    [anon_sym_DOT_DOT] = ACTIONS(2674),
    [anon_sym_COLON_COLON] = ACTIONS(2674),
    [anon_sym_POUND] = ACTIONS(2674),
    [anon_sym_SQUOTE] = ACTIONS(2676),
    [anon_sym_async] = ACTIONS(2676),
    [anon_sym_auto] = ACTIONS(2676),
    [anon_sym_await] = ACTIONS(2676),
    [anon_sym_become] = ACTIONS(2676),
    [anon_sym_break] = ACTIONS(2676),
    [anon_sym_const] = ACTIONS(2676),
    [anon_sym_continue] = ACTIONS(2676),
    [anon_sym_default] = ACTIONS(2676),
    [anon_sym_enum] = ACTIONS(2676),
    [anon_sym_fn] = ACTIONS(2676),
    [anon_sym_for] = ACTIONS(2676),
    [anon_sym_gen] = ACTIONS(2676),
    [anon_sym_if] = ACTIONS(2676),
    [anon_sym_impl] = ACTIONS(2676),
    [anon_sym_let] = ACTIONS(2676),
    [anon_sym_loop] = ACTIONS(2676),
    [anon_sym_match] = ACTIONS(2676),
    [anon_sym_mod] = ACTIONS(2676),
    [anon_sym_pub] = ACTIONS(2676),
    [anon_sym_return] = ACTIONS(2676),
    [anon_sym_raw] = ACTIONS(2676),
    [anon_sym_static] = ACTIONS(2676),
    [anon_sym_struct] = ACTIONS(2676),
    [anon_sym_trait] = ACTIONS(2676),
    [anon_sym_type] = ACTIONS(2676),
    [anon_sym_union] = ACTIONS(2676),
    [anon_sym_unsafe] = ACTIONS(2676),
    [anon_sym_use] = ACTIONS(2676),
    [anon_sym_while] = ACTIONS(2676),
    [anon_sym_extern] = ACTIONS(2676),
    [anon_sym_safe] = ACTIONS(2676),
    [anon_sym_yield] = ACTIONS(2676),
    [anon_sym_move] = ACTIONS(2676),
    [anon_sym_try] = ACTIONS(2676),
    [sym_integer_literal] = ACTIONS(2674),
    [aux_sym_string_literal_token1] = ACTIONS(2674),
    [sym_char_literal] = ACTIONS(2674),
    [anon_sym_true] = ACTIONS(2676),
    [anon_sym_false] = ACTIONS(2676),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2676),
    [sym_super] = ACTIONS(2676),
    [sym_crate] = ACTIONS(2676),
    [aux_sym_metavariable_token1] = ACTIONS(2674),
    [sym__raw_string_literal_start] = ACTIONS(2674),
    [sym_float_literal] = ACTIONS(2674),
    [sym__outer_block_doc_comment_start] = ACTIONS(2674),
    [sym__outer_line_doc_comment_start] = ACTIONS(2674),
  },
  [STATE(726)] = {
    [sym_line_comment] = STATE(726),
    [sym_block_comment] = STATE(726),
    [ts_builtin_sym_end] = ACTIONS(2678),
    [sym_identifier] = ACTIONS(2680),
    [anon_sym_SEMI] = ACTIONS(2678),
    [anon_sym_macro] = ACTIONS(2680),
    [anon_sym_LPAREN] = ACTIONS(2678),
    [anon_sym_LBRACE] = ACTIONS(2678),
    [anon_sym_RBRACE] = ACTIONS(2678),
    [anon_sym_macro_rules] = ACTIONS(2680),
    [anon_sym_BANG] = ACTIONS(2678),
    [anon_sym_LBRACK] = ACTIONS(2678),
    [anon_sym_STAR] = ACTIONS(2678),
    [anon_sym_u8] = ACTIONS(2680),
    [anon_sym_i8] = ACTIONS(2680),
    [anon_sym_u16] = ACTIONS(2680),
    [anon_sym_i16] = ACTIONS(2680),
    [anon_sym_u32] = ACTIONS(2680),
    [anon_sym_i32] = ACTIONS(2680),
    [anon_sym_u64] = ACTIONS(2680),
    [anon_sym_i64] = ACTIONS(2680),
    [anon_sym_u128] = ACTIONS(2680),
    [anon_sym_i128] = ACTIONS(2680),
    [anon_sym_isize] = ACTIONS(2680),
    [anon_sym_usize] = ACTIONS(2680),
    [anon_sym_f32] = ACTIONS(2680),
    [anon_sym_f64] = ACTIONS(2680),
    [anon_sym_bool] = ACTIONS(2680),
    [anon_sym_str] = ACTIONS(2680),
    [anon_sym_char] = ACTIONS(2680),
    [anon_sym_DASH] = ACTIONS(2678),
    [anon_sym_AMP] = ACTIONS(2678),
    [anon_sym_PIPE] = ACTIONS(2678),
    [anon_sym_LT] = ACTIONS(2678),
    [anon_sym_DOT_DOT] = ACTIONS(2678),
    [anon_sym_COLON_COLON] = ACTIONS(2678),
    [anon_sym_POUND] = ACTIONS(2678),
    [anon_sym_SQUOTE] = ACTIONS(2680),
    [anon_sym_async] = ACTIONS(2680),
    [anon_sym_auto] = ACTIONS(2680),
    [anon_sym_await] = ACTIONS(2680),
    [anon_sym_become] = ACTIONS(2680),
    [anon_sym_break] = ACTIONS(2680),
    [anon_sym_const] = ACTIONS(2680),
    [anon_sym_continue] = ACTIONS(2680),
    [anon_sym_default] = ACTIONS(2680),
    [anon_sym_enum] = ACTIONS(2680),
    [anon_sym_fn] = ACTIONS(2680),
    [anon_sym_for] = ACTIONS(2680),
    [anon_sym_gen] = ACTIONS(2680),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_impl] = ACTIONS(2680),
    [anon_sym_let] = ACTIONS(2680),
    [anon_sym_loop] = ACTIONS(2680),
    [anon_sym_match] = ACTIONS(2680),
    [anon_sym_mod] = ACTIONS(2680),
    [anon_sym_pub] = ACTIONS(2680),
    [anon_sym_return] = ACTIONS(2680),
    [anon_sym_raw] = ACTIONS(2680),
    [anon_sym_static] = ACTIONS(2680),
    [anon_sym_struct] = ACTIONS(2680),
    [anon_sym_trait] = ACTIONS(2680),
    [anon_sym_type] = ACTIONS(2680),
    [anon_sym_union] = ACTIONS(2680),
    [anon_sym_unsafe] = ACTIONS(2680),
    [anon_sym_use] = ACTIONS(2680),
    [anon_sym_while] = ACTIONS(2680),
    [anon_sym_extern] = ACTIONS(2680),
    [anon_sym_safe] = ACTIONS(2680),
    [anon_sym_yield] = ACTIONS(2680),
    [anon_sym_move] = ACTIONS(2680),
    [anon_sym_try] = ACTIONS(2680),
    [sym_integer_literal] = ACTIONS(2678),
    [aux_sym_string_literal_token1] = ACTIONS(2678),
    [sym_char_literal] = ACTIONS(2678),
    [anon_sym_true] = ACTIONS(2680),
    [anon_sym_false] = ACTIONS(2680),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2680),
    [sym_super] = ACTIONS(2680),
    [sym_crate] = ACTIONS(2680),
    [aux_sym_metavariable_token1] = ACTIONS(2678),
    [sym__raw_string_literal_start] = ACTIONS(2678),
    [sym_float_literal] = ACTIONS(2678),
    [sym__outer_block_doc_comment_start] = ACTIONS(2678),
    [sym__outer_line_doc_comment_start] = ACTIONS(2678),
  },
  [STATE(727)] = {
    [sym_line_comment] = STATE(727),
    [sym_block_comment] = STATE(727),
    [ts_builtin_sym_end] = ACTIONS(2682),
    [sym_identifier] = ACTIONS(2684),
    [anon_sym_SEMI] = ACTIONS(2682),
    [anon_sym_macro] = ACTIONS(2684),
    [anon_sym_LPAREN] = ACTIONS(2682),
    [anon_sym_LBRACE] = ACTIONS(2682),
    [anon_sym_RBRACE] = ACTIONS(2682),
    [anon_sym_macro_rules] = ACTIONS(2684),
    [anon_sym_BANG] = ACTIONS(2682),
    [anon_sym_LBRACK] = ACTIONS(2682),
    [anon_sym_STAR] = ACTIONS(2682),
    [anon_sym_u8] = ACTIONS(2684),
    [anon_sym_i8] = ACTIONS(2684),
    [anon_sym_u16] = ACTIONS(2684),
    [anon_sym_i16] = ACTIONS(2684),
    [anon_sym_u32] = ACTIONS(2684),
    [anon_sym_i32] = ACTIONS(2684),
    [anon_sym_u64] = ACTIONS(2684),
    [anon_sym_i64] = ACTIONS(2684),
    [anon_sym_u128] = ACTIONS(2684),
    [anon_sym_i128] = ACTIONS(2684),
    [anon_sym_isize] = ACTIONS(2684),
    [anon_sym_usize] = ACTIONS(2684),
    [anon_sym_f32] = ACTIONS(2684),
    [anon_sym_f64] = ACTIONS(2684),
    [anon_sym_bool] = ACTIONS(2684),
    [anon_sym_str] = ACTIONS(2684),
    [anon_sym_char] = ACTIONS(2684),
    [anon_sym_DASH] = ACTIONS(2682),
    [anon_sym_AMP] = ACTIONS(2682),
    [anon_sym_PIPE] = ACTIONS(2682),
    [anon_sym_LT] = ACTIONS(2682),
    [anon_sym_DOT_DOT] = ACTIONS(2682),
    [anon_sym_COLON_COLON] = ACTIONS(2682),
    [anon_sym_POUND] = ACTIONS(2682),
    [anon_sym_SQUOTE] = ACTIONS(2684),
    [anon_sym_async] = ACTIONS(2684),
    [anon_sym_auto] = ACTIONS(2684),
    [anon_sym_await] = ACTIONS(2684),
    [anon_sym_become] = ACTIONS(2684),
    [anon_sym_break] = ACTIONS(2684),
    [anon_sym_const] = ACTIONS(2684),
    [anon_sym_continue] = ACTIONS(2684),
    [anon_sym_default] = ACTIONS(2684),
    [anon_sym_enum] = ACTIONS(2684),
    [anon_sym_fn] = ACTIONS(2684),
    [anon_sym_for] = ACTIONS(2684),
    [anon_sym_gen] = ACTIONS(2684),
    [anon_sym_if] = ACTIONS(2684),
    [anon_sym_impl] = ACTIONS(2684),
    [anon_sym_let] = ACTIONS(2684),
    [anon_sym_loop] = ACTIONS(2684),
    [anon_sym_match] = ACTIONS(2684),
    [anon_sym_mod] = ACTIONS(2684),
    [anon_sym_pub] = ACTIONS(2684),
    [anon_sym_return] = ACTIONS(2684),
    [anon_sym_raw] = ACTIONS(2684),
    [anon_sym_static] = ACTIONS(2684),
    [anon_sym_struct] = ACTIONS(2684),
    [anon_sym_trait] = ACTIONS(2684),
    [anon_sym_type] = ACTIONS(2684),
    [anon_sym_union] = ACTIONS(2684),
    [anon_sym_unsafe] = ACTIONS(2684),
    [anon_sym_use] = ACTIONS(2684),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_extern] = ACTIONS(2684),
    [anon_sym_safe] = ACTIONS(2684),
    [anon_sym_yield] = ACTIONS(2684),
    [anon_sym_move] = ACTIONS(2684),
    [anon_sym_try] = ACTIONS(2684),
    [sym_integer_literal] = ACTIONS(2682),
    [aux_sym_string_literal_token1] = ACTIONS(2682),
    [sym_char_literal] = ACTIONS(2682),
    [anon_sym_true] = ACTIONS(2684),
    [anon_sym_false] = ACTIONS(2684),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2684),
    [sym_super] = ACTIONS(2684),
    [sym_crate] = ACTIONS(2684),
    [aux_sym_metavariable_token1] = ACTIONS(2682),
    [sym__raw_string_literal_start] = ACTIONS(2682),
    [sym_float_literal] = ACTIONS(2682),
    [sym__outer_block_doc_comment_start] = ACTIONS(2682),
    [sym__outer_line_doc_comment_start] = ACTIONS(2682),
  },
  [STATE(728)] = {
    [sym_line_comment] = STATE(728),
    [sym_block_comment] = STATE(728),
    [ts_builtin_sym_end] = ACTIONS(2686),
    [sym_identifier] = ACTIONS(2688),
    [anon_sym_SEMI] = ACTIONS(2686),
    [anon_sym_macro] = ACTIONS(2688),
    [anon_sym_LPAREN] = ACTIONS(2686),
    [anon_sym_LBRACE] = ACTIONS(2686),
    [anon_sym_RBRACE] = ACTIONS(2686),
    [anon_sym_macro_rules] = ACTIONS(2688),
    [anon_sym_BANG] = ACTIONS(2686),
    [anon_sym_LBRACK] = ACTIONS(2686),
    [anon_sym_STAR] = ACTIONS(2686),
    [anon_sym_u8] = ACTIONS(2688),
    [anon_sym_i8] = ACTIONS(2688),
    [anon_sym_u16] = ACTIONS(2688),
    [anon_sym_i16] = ACTIONS(2688),
    [anon_sym_u32] = ACTIONS(2688),
    [anon_sym_i32] = ACTIONS(2688),
    [anon_sym_u64] = ACTIONS(2688),
    [anon_sym_i64] = ACTIONS(2688),
    [anon_sym_u128] = ACTIONS(2688),
    [anon_sym_i128] = ACTIONS(2688),
    [anon_sym_isize] = ACTIONS(2688),
    [anon_sym_usize] = ACTIONS(2688),
    [anon_sym_f32] = ACTIONS(2688),
    [anon_sym_f64] = ACTIONS(2688),
    [anon_sym_bool] = ACTIONS(2688),
    [anon_sym_str] = ACTIONS(2688),
    [anon_sym_char] = ACTIONS(2688),
    [anon_sym_DASH] = ACTIONS(2686),
    [anon_sym_AMP] = ACTIONS(2686),
    [anon_sym_PIPE] = ACTIONS(2686),
    [anon_sym_LT] = ACTIONS(2686),
    [anon_sym_DOT_DOT] = ACTIONS(2686),
    [anon_sym_COLON_COLON] = ACTIONS(2686),
    [anon_sym_POUND] = ACTIONS(2686),
    [anon_sym_SQUOTE] = ACTIONS(2688),
    [anon_sym_async] = ACTIONS(2688),
    [anon_sym_auto] = ACTIONS(2688),
    [anon_sym_await] = ACTIONS(2688),
    [anon_sym_become] = ACTIONS(2688),
    [anon_sym_break] = ACTIONS(2688),
    [anon_sym_const] = ACTIONS(2688),
    [anon_sym_continue] = ACTIONS(2688),
    [anon_sym_default] = ACTIONS(2688),
    [anon_sym_enum] = ACTIONS(2688),
    [anon_sym_fn] = ACTIONS(2688),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_gen] = ACTIONS(2688),
    [anon_sym_if] = ACTIONS(2688),
    [anon_sym_impl] = ACTIONS(2688),
    [anon_sym_let] = ACTIONS(2688),
    [anon_sym_loop] = ACTIONS(2688),
    [anon_sym_match] = ACTIONS(2688),
    [anon_sym_mod] = ACTIONS(2688),
    [anon_sym_pub] = ACTIONS(2688),
    [anon_sym_return] = ACTIONS(2688),
    [anon_sym_raw] = ACTIONS(2688),
    [anon_sym_static] = ACTIONS(2688),
    [anon_sym_struct] = ACTIONS(2688),
    [anon_sym_trait] = ACTIONS(2688),
    [anon_sym_type] = ACTIONS(2688),
    [anon_sym_union] = ACTIONS(2688),
    [anon_sym_unsafe] = ACTIONS(2688),
    [anon_sym_use] = ACTIONS(2688),
    [anon_sym_while] = ACTIONS(2688),
    [anon_sym_extern] = ACTIONS(2688),
    [anon_sym_safe] = ACTIONS(2688),
    [anon_sym_yield] = ACTIONS(2688),
    [anon_sym_move] = ACTIONS(2688),
    [anon_sym_try] = ACTIONS(2688),
    [sym_integer_literal] = ACTIONS(2686),
    [aux_sym_string_literal_token1] = ACTIONS(2686),
    [sym_char_literal] = ACTIONS(2686),
    [anon_sym_true] = ACTIONS(2688),
    [anon_sym_false] = ACTIONS(2688),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2688),
    [sym_super] = ACTIONS(2688),
    [sym_crate] = ACTIONS(2688),
    [aux_sym_metavariable_token1] = ACTIONS(2686),
    [sym__raw_string_literal_start] = ACTIONS(2686),
    [sym_float_literal] = ACTIONS(2686),
    [sym__outer_block_doc_comment_start] = ACTIONS(2686),
    [sym__outer_line_doc_comment_start] = ACTIONS(2686),
  },
  [STATE(729)] = {
    [sym_line_comment] = STATE(729),
    [sym_block_comment] = STATE(729),
    [ts_builtin_sym_end] = ACTIONS(2690),
    [sym_identifier] = ACTIONS(2692),
    [anon_sym_SEMI] = ACTIONS(2690),
    [anon_sym_macro] = ACTIONS(2692),
    [anon_sym_LPAREN] = ACTIONS(2690),
    [anon_sym_LBRACE] = ACTIONS(2690),
    [anon_sym_RBRACE] = ACTIONS(2690),
    [anon_sym_macro_rules] = ACTIONS(2692),
    [anon_sym_BANG] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2690),
    [anon_sym_STAR] = ACTIONS(2690),
    [anon_sym_u8] = ACTIONS(2692),
    [anon_sym_i8] = ACTIONS(2692),
    [anon_sym_u16] = ACTIONS(2692),
    [anon_sym_i16] = ACTIONS(2692),
    [anon_sym_u32] = ACTIONS(2692),
    [anon_sym_i32] = ACTIONS(2692),
    [anon_sym_u64] = ACTIONS(2692),
    [anon_sym_i64] = ACTIONS(2692),
    [anon_sym_u128] = ACTIONS(2692),
    [anon_sym_i128] = ACTIONS(2692),
    [anon_sym_isize] = ACTIONS(2692),
    [anon_sym_usize] = ACTIONS(2692),
    [anon_sym_f32] = ACTIONS(2692),
    [anon_sym_f64] = ACTIONS(2692),
    [anon_sym_bool] = ACTIONS(2692),
    [anon_sym_str] = ACTIONS(2692),
    [anon_sym_char] = ACTIONS(2692),
    [anon_sym_DASH] = ACTIONS(2690),
    [anon_sym_AMP] = ACTIONS(2690),
    [anon_sym_PIPE] = ACTIONS(2690),
    [anon_sym_LT] = ACTIONS(2690),
    [anon_sym_DOT_DOT] = ACTIONS(2690),
    [anon_sym_COLON_COLON] = ACTIONS(2690),
    [anon_sym_POUND] = ACTIONS(2690),
    [anon_sym_SQUOTE] = ACTIONS(2692),
    [anon_sym_async] = ACTIONS(2692),
    [anon_sym_auto] = ACTIONS(2692),
    [anon_sym_await] = ACTIONS(2692),
    [anon_sym_become] = ACTIONS(2692),
    [anon_sym_break] = ACTIONS(2692),
    [anon_sym_const] = ACTIONS(2692),
    [anon_sym_continue] = ACTIONS(2692),
    [anon_sym_default] = ACTIONS(2692),
    [anon_sym_enum] = ACTIONS(2692),
    [anon_sym_fn] = ACTIONS(2692),
    [anon_sym_for] = ACTIONS(2692),
    [anon_sym_gen] = ACTIONS(2692),
    [anon_sym_if] = ACTIONS(2692),
    [anon_sym_impl] = ACTIONS(2692),
    [anon_sym_let] = ACTIONS(2692),
    [anon_sym_loop] = ACTIONS(2692),
    [anon_sym_match] = ACTIONS(2692),
    [anon_sym_mod] = ACTIONS(2692),
    [anon_sym_pub] = ACTIONS(2692),
    [anon_sym_return] = ACTIONS(2692),
    [anon_sym_raw] = ACTIONS(2692),
    [anon_sym_static] = ACTIONS(2692),
    [anon_sym_struct] = ACTIONS(2692),
    [anon_sym_trait] = ACTIONS(2692),
    [anon_sym_type] = ACTIONS(2692),
    [anon_sym_union] = ACTIONS(2692),
    [anon_sym_unsafe] = ACTIONS(2692),
    [anon_sym_use] = ACTIONS(2692),
    [anon_sym_while] = ACTIONS(2692),
    [anon_sym_extern] = ACTIONS(2692),
    [anon_sym_safe] = ACTIONS(2692),
    [anon_sym_yield] = ACTIONS(2692),
    [anon_sym_move] = ACTIONS(2692),
    [anon_sym_try] = ACTIONS(2692),
    [sym_integer_literal] = ACTIONS(2690),
    [aux_sym_string_literal_token1] = ACTIONS(2690),
    [sym_char_literal] = ACTIONS(2690),
    [anon_sym_true] = ACTIONS(2692),
    [anon_sym_false] = ACTIONS(2692),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2692),
    [sym_super] = ACTIONS(2692),
    [sym_crate] = ACTIONS(2692),
    [aux_sym_metavariable_token1] = ACTIONS(2690),
    [sym__raw_string_literal_start] = ACTIONS(2690),
    [sym_float_literal] = ACTIONS(2690),
    [sym__outer_block_doc_comment_start] = ACTIONS(2690),
    [sym__outer_line_doc_comment_start] = ACTIONS(2690),
  },
  [STATE(730)] = {
    [sym_line_comment] = STATE(730),
    [sym_block_comment] = STATE(730),
    [ts_builtin_sym_end] = ACTIONS(2694),
    [sym_identifier] = ACTIONS(2696),
    [anon_sym_SEMI] = ACTIONS(2694),
    [anon_sym_macro] = ACTIONS(2696),
    [anon_sym_LPAREN] = ACTIONS(2694),
    [anon_sym_LBRACE] = ACTIONS(2694),
    [anon_sym_RBRACE] = ACTIONS(2694),
    [anon_sym_macro_rules] = ACTIONS(2696),
    [anon_sym_BANG] = ACTIONS(2694),
    [anon_sym_LBRACK] = ACTIONS(2694),
    [anon_sym_STAR] = ACTIONS(2694),
    [anon_sym_u8] = ACTIONS(2696),
    [anon_sym_i8] = ACTIONS(2696),
    [anon_sym_u16] = ACTIONS(2696),
    [anon_sym_i16] = ACTIONS(2696),
    [anon_sym_u32] = ACTIONS(2696),
    [anon_sym_i32] = ACTIONS(2696),
    [anon_sym_u64] = ACTIONS(2696),
    [anon_sym_i64] = ACTIONS(2696),
    [anon_sym_u128] = ACTIONS(2696),
    [anon_sym_i128] = ACTIONS(2696),
    [anon_sym_isize] = ACTIONS(2696),
    [anon_sym_usize] = ACTIONS(2696),
    [anon_sym_f32] = ACTIONS(2696),
    [anon_sym_f64] = ACTIONS(2696),
    [anon_sym_bool] = ACTIONS(2696),
    [anon_sym_str] = ACTIONS(2696),
    [anon_sym_char] = ACTIONS(2696),
    [anon_sym_DASH] = ACTIONS(2694),
    [anon_sym_AMP] = ACTIONS(2694),
    [anon_sym_PIPE] = ACTIONS(2694),
    [anon_sym_LT] = ACTIONS(2694),
    [anon_sym_DOT_DOT] = ACTIONS(2694),
    [anon_sym_COLON_COLON] = ACTIONS(2694),
    [anon_sym_POUND] = ACTIONS(2694),
    [anon_sym_SQUOTE] = ACTIONS(2696),
    [anon_sym_async] = ACTIONS(2696),
    [anon_sym_auto] = ACTIONS(2696),
    [anon_sym_await] = ACTIONS(2696),
    [anon_sym_become] = ACTIONS(2696),
    [anon_sym_break] = ACTIONS(2696),
    [anon_sym_const] = ACTIONS(2696),
    [anon_sym_continue] = ACTIONS(2696),
    [anon_sym_default] = ACTIONS(2696),
    [anon_sym_enum] = ACTIONS(2696),
    [anon_sym_fn] = ACTIONS(2696),
    [anon_sym_for] = ACTIONS(2696),
    [anon_sym_gen] = ACTIONS(2696),
    [anon_sym_if] = ACTIONS(2696),
    [anon_sym_impl] = ACTIONS(2696),
    [anon_sym_let] = ACTIONS(2696),
    [anon_sym_loop] = ACTIONS(2696),
    [anon_sym_match] = ACTIONS(2696),
    [anon_sym_mod] = ACTIONS(2696),
    [anon_sym_pub] = ACTIONS(2696),
    [anon_sym_return] = ACTIONS(2696),
    [anon_sym_raw] = ACTIONS(2696),
    [anon_sym_static] = ACTIONS(2696),
    [anon_sym_struct] = ACTIONS(2696),
    [anon_sym_trait] = ACTIONS(2696),
    [anon_sym_type] = ACTIONS(2696),
    [anon_sym_union] = ACTIONS(2696),
    [anon_sym_unsafe] = ACTIONS(2696),
    [anon_sym_use] = ACTIONS(2696),
    [anon_sym_while] = ACTIONS(2696),
    [anon_sym_extern] = ACTIONS(2696),
    [anon_sym_safe] = ACTIONS(2696),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_move] = ACTIONS(2696),
    [anon_sym_try] = ACTIONS(2696),
    [sym_integer_literal] = ACTIONS(2694),
    [aux_sym_string_literal_token1] = ACTIONS(2694),
    [sym_char_literal] = ACTIONS(2694),
    [anon_sym_true] = ACTIONS(2696),
    [anon_sym_false] = ACTIONS(2696),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2696),
    [sym_super] = ACTIONS(2696),
    [sym_crate] = ACTIONS(2696),
    [aux_sym_metavariable_token1] = ACTIONS(2694),
    [sym__raw_string_literal_start] = ACTIONS(2694),
    [sym_float_literal] = ACTIONS(2694),
    [sym__outer_block_doc_comment_start] = ACTIONS(2694),
    [sym__outer_line_doc_comment_start] = ACTIONS(2694),
  },
  [STATE(731)] = {
    [sym_line_comment] = STATE(731),
    [sym_block_comment] = STATE(731),
    [ts_builtin_sym_end] = ACTIONS(2698),
    [sym_identifier] = ACTIONS(2700),
    [anon_sym_SEMI] = ACTIONS(2698),
    [anon_sym_macro] = ACTIONS(2700),
    [anon_sym_LPAREN] = ACTIONS(2698),
    [anon_sym_LBRACE] = ACTIONS(2698),
    [anon_sym_RBRACE] = ACTIONS(2698),
    [anon_sym_macro_rules] = ACTIONS(2700),
    [anon_sym_BANG] = ACTIONS(2698),
    [anon_sym_LBRACK] = ACTIONS(2698),
    [anon_sym_STAR] = ACTIONS(2698),
    [anon_sym_u8] = ACTIONS(2700),
    [anon_sym_i8] = ACTIONS(2700),
    [anon_sym_u16] = ACTIONS(2700),
    [anon_sym_i16] = ACTIONS(2700),
    [anon_sym_u32] = ACTIONS(2700),
    [anon_sym_i32] = ACTIONS(2700),
    [anon_sym_u64] = ACTIONS(2700),
    [anon_sym_i64] = ACTIONS(2700),
    [anon_sym_u128] = ACTIONS(2700),
    [anon_sym_i128] = ACTIONS(2700),
    [anon_sym_isize] = ACTIONS(2700),
    [anon_sym_usize] = ACTIONS(2700),
    [anon_sym_f32] = ACTIONS(2700),
    [anon_sym_f64] = ACTIONS(2700),
    [anon_sym_bool] = ACTIONS(2700),
    [anon_sym_str] = ACTIONS(2700),
    [anon_sym_char] = ACTIONS(2700),
    [anon_sym_DASH] = ACTIONS(2698),
    [anon_sym_AMP] = ACTIONS(2698),
    [anon_sym_PIPE] = ACTIONS(2698),
    [anon_sym_LT] = ACTIONS(2698),
    [anon_sym_DOT_DOT] = ACTIONS(2698),
    [anon_sym_COLON_COLON] = ACTIONS(2698),
    [anon_sym_POUND] = ACTIONS(2698),
    [anon_sym_SQUOTE] = ACTIONS(2700),
    [anon_sym_async] = ACTIONS(2700),
    [anon_sym_auto] = ACTIONS(2700),
    [anon_sym_await] = ACTIONS(2700),
    [anon_sym_become] = ACTIONS(2700),
    [anon_sym_break] = ACTIONS(2700),
    [anon_sym_const] = ACTIONS(2700),
    [anon_sym_continue] = ACTIONS(2700),
    [anon_sym_default] = ACTIONS(2700),
    [anon_sym_enum] = ACTIONS(2700),
    [anon_sym_fn] = ACTIONS(2700),
    [anon_sym_for] = ACTIONS(2700),
    [anon_sym_gen] = ACTIONS(2700),
    [anon_sym_if] = ACTIONS(2700),
    [anon_sym_impl] = ACTIONS(2700),
    [anon_sym_let] = ACTIONS(2700),
    [anon_sym_loop] = ACTIONS(2700),
    [anon_sym_match] = ACTIONS(2700),
    [anon_sym_mod] = ACTIONS(2700),
    [anon_sym_pub] = ACTIONS(2700),
    [anon_sym_return] = ACTIONS(2700),
    [anon_sym_raw] = ACTIONS(2700),
    [anon_sym_static] = ACTIONS(2700),
    [anon_sym_struct] = ACTIONS(2700),
    [anon_sym_trait] = ACTIONS(2700),
    [anon_sym_type] = ACTIONS(2700),
    [anon_sym_union] = ACTIONS(2700),
    [anon_sym_unsafe] = ACTIONS(2700),
    [anon_sym_use] = ACTIONS(2700),
    [anon_sym_while] = ACTIONS(2700),
    [anon_sym_extern] = ACTIONS(2700),
    [anon_sym_safe] = ACTIONS(2700),
    [anon_sym_yield] = ACTIONS(2700),
    [anon_sym_move] = ACTIONS(2700),
    [anon_sym_try] = ACTIONS(2700),
    [sym_integer_literal] = ACTIONS(2698),
    [aux_sym_string_literal_token1] = ACTIONS(2698),
    [sym_char_literal] = ACTIONS(2698),
    [anon_sym_true] = ACTIONS(2700),
    [anon_sym_false] = ACTIONS(2700),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2700),
    [sym_super] = ACTIONS(2700),
    [sym_crate] = ACTIONS(2700),
    [aux_sym_metavariable_token1] = ACTIONS(2698),
    [sym__raw_string_literal_start] = ACTIONS(2698),
    [sym_float_literal] = ACTIONS(2698),
    [sym__outer_block_doc_comment_start] = ACTIONS(2698),
    [sym__outer_line_doc_comment_start] = ACTIONS(2698),
  },
  [STATE(732)] = {
    [sym_line_comment] = STATE(732),
    [sym_block_comment] = STATE(732),
    [ts_builtin_sym_end] = ACTIONS(2702),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_SEMI] = ACTIONS(2702),
    [anon_sym_macro] = ACTIONS(2704),
    [anon_sym_LPAREN] = ACTIONS(2702),
    [anon_sym_LBRACE] = ACTIONS(2702),
    [anon_sym_RBRACE] = ACTIONS(2702),
    [anon_sym_macro_rules] = ACTIONS(2704),
    [anon_sym_BANG] = ACTIONS(2702),
    [anon_sym_LBRACK] = ACTIONS(2702),
    [anon_sym_STAR] = ACTIONS(2702),
    [anon_sym_u8] = ACTIONS(2704),
    [anon_sym_i8] = ACTIONS(2704),
    [anon_sym_u16] = ACTIONS(2704),
    [anon_sym_i16] = ACTIONS(2704),
    [anon_sym_u32] = ACTIONS(2704),
    [anon_sym_i32] = ACTIONS(2704),
    [anon_sym_u64] = ACTIONS(2704),
    [anon_sym_i64] = ACTIONS(2704),
    [anon_sym_u128] = ACTIONS(2704),
    [anon_sym_i128] = ACTIONS(2704),
    [anon_sym_isize] = ACTIONS(2704),
    [anon_sym_usize] = ACTIONS(2704),
    [anon_sym_f32] = ACTIONS(2704),
    [anon_sym_f64] = ACTIONS(2704),
    [anon_sym_bool] = ACTIONS(2704),
    [anon_sym_str] = ACTIONS(2704),
    [anon_sym_char] = ACTIONS(2704),
    [anon_sym_DASH] = ACTIONS(2702),
    [anon_sym_AMP] = ACTIONS(2702),
    [anon_sym_PIPE] = ACTIONS(2702),
    [anon_sym_LT] = ACTIONS(2702),
    [anon_sym_DOT_DOT] = ACTIONS(2702),
    [anon_sym_COLON_COLON] = ACTIONS(2702),
    [anon_sym_POUND] = ACTIONS(2702),
    [anon_sym_SQUOTE] = ACTIONS(2704),
    [anon_sym_async] = ACTIONS(2704),
    [anon_sym_auto] = ACTIONS(2704),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_become] = ACTIONS(2704),
    [anon_sym_break] = ACTIONS(2704),
    [anon_sym_const] = ACTIONS(2704),
    [anon_sym_continue] = ACTIONS(2704),
    [anon_sym_default] = ACTIONS(2704),
    [anon_sym_enum] = ACTIONS(2704),
    [anon_sym_fn] = ACTIONS(2704),
    [anon_sym_for] = ACTIONS(2704),
    [anon_sym_gen] = ACTIONS(2704),
    [anon_sym_if] = ACTIONS(2704),
    [anon_sym_impl] = ACTIONS(2704),
    [anon_sym_let] = ACTIONS(2704),
    [anon_sym_loop] = ACTIONS(2704),
    [anon_sym_match] = ACTIONS(2704),
    [anon_sym_mod] = ACTIONS(2704),
    [anon_sym_pub] = ACTIONS(2704),
    [anon_sym_return] = ACTIONS(2704),
    [anon_sym_raw] = ACTIONS(2704),
    [anon_sym_static] = ACTIONS(2704),
    [anon_sym_struct] = ACTIONS(2704),
    [anon_sym_trait] = ACTIONS(2704),
    [anon_sym_type] = ACTIONS(2704),
    [anon_sym_union] = ACTIONS(2704),
    [anon_sym_unsafe] = ACTIONS(2704),
    [anon_sym_use] = ACTIONS(2704),
    [anon_sym_while] = ACTIONS(2704),
    [anon_sym_extern] = ACTIONS(2704),
    [anon_sym_safe] = ACTIONS(2704),
    [anon_sym_yield] = ACTIONS(2704),
    [anon_sym_move] = ACTIONS(2704),
    [anon_sym_try] = ACTIONS(2704),
    [sym_integer_literal] = ACTIONS(2702),
    [aux_sym_string_literal_token1] = ACTIONS(2702),
    [sym_char_literal] = ACTIONS(2702),
    [anon_sym_true] = ACTIONS(2704),
    [anon_sym_false] = ACTIONS(2704),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2704),
    [sym_super] = ACTIONS(2704),
    [sym_crate] = ACTIONS(2704),
    [aux_sym_metavariable_token1] = ACTIONS(2702),
    [sym__raw_string_literal_start] = ACTIONS(2702),
    [sym_float_literal] = ACTIONS(2702),
    [sym__outer_block_doc_comment_start] = ACTIONS(2702),
    [sym__outer_line_doc_comment_start] = ACTIONS(2702),
  },
  [STATE(733)] = {
    [sym_line_comment] = STATE(733),
    [sym_block_comment] = STATE(733),
    [ts_builtin_sym_end] = ACTIONS(2706),
    [sym_identifier] = ACTIONS(2708),
    [anon_sym_SEMI] = ACTIONS(2706),
    [anon_sym_macro] = ACTIONS(2708),
    [anon_sym_LPAREN] = ACTIONS(2706),
    [anon_sym_LBRACE] = ACTIONS(2706),
    [anon_sym_RBRACE] = ACTIONS(2706),
    [anon_sym_macro_rules] = ACTIONS(2708),
    [anon_sym_BANG] = ACTIONS(2706),
    [anon_sym_LBRACK] = ACTIONS(2706),
    [anon_sym_STAR] = ACTIONS(2706),
    [anon_sym_u8] = ACTIONS(2708),
    [anon_sym_i8] = ACTIONS(2708),
    [anon_sym_u16] = ACTIONS(2708),
    [anon_sym_i16] = ACTIONS(2708),
    [anon_sym_u32] = ACTIONS(2708),
    [anon_sym_i32] = ACTIONS(2708),
    [anon_sym_u64] = ACTIONS(2708),
    [anon_sym_i64] = ACTIONS(2708),
    [anon_sym_u128] = ACTIONS(2708),
    [anon_sym_i128] = ACTIONS(2708),
    [anon_sym_isize] = ACTIONS(2708),
    [anon_sym_usize] = ACTIONS(2708),
    [anon_sym_f32] = ACTIONS(2708),
    [anon_sym_f64] = ACTIONS(2708),
    [anon_sym_bool] = ACTIONS(2708),
    [anon_sym_str] = ACTIONS(2708),
    [anon_sym_char] = ACTIONS(2708),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_AMP] = ACTIONS(2706),
    [anon_sym_PIPE] = ACTIONS(2706),
    [anon_sym_LT] = ACTIONS(2706),
    [anon_sym_DOT_DOT] = ACTIONS(2706),
    [anon_sym_COLON_COLON] = ACTIONS(2706),
    [anon_sym_POUND] = ACTIONS(2706),
    [anon_sym_SQUOTE] = ACTIONS(2708),
    [anon_sym_async] = ACTIONS(2708),
    [anon_sym_auto] = ACTIONS(2708),
    [anon_sym_await] = ACTIONS(2708),
    [anon_sym_become] = ACTIONS(2708),
    [anon_sym_break] = ACTIONS(2708),
    [anon_sym_const] = ACTIONS(2708),
    [anon_sym_continue] = ACTIONS(2708),
    [anon_sym_default] = ACTIONS(2708),
    [anon_sym_enum] = ACTIONS(2708),
    [anon_sym_fn] = ACTIONS(2708),
    [anon_sym_for] = ACTIONS(2708),
    [anon_sym_gen] = ACTIONS(2708),
    [anon_sym_if] = ACTIONS(2708),
    [anon_sym_impl] = ACTIONS(2708),
    [anon_sym_let] = ACTIONS(2708),
    [anon_sym_loop] = ACTIONS(2708),
    [anon_sym_match] = ACTIONS(2708),
    [anon_sym_mod] = ACTIONS(2708),
    [anon_sym_pub] = ACTIONS(2708),
    [anon_sym_return] = ACTIONS(2708),
    [anon_sym_raw] = ACTIONS(2708),
    [anon_sym_static] = ACTIONS(2708),
    [anon_sym_struct] = ACTIONS(2708),
    [anon_sym_trait] = ACTIONS(2708),
    [anon_sym_type] = ACTIONS(2708),
    [anon_sym_union] = ACTIONS(2708),
    [anon_sym_unsafe] = ACTIONS(2708),
    [anon_sym_use] = ACTIONS(2708),
    [anon_sym_while] = ACTIONS(2708),
    [anon_sym_extern] = ACTIONS(2708),
    [anon_sym_safe] = ACTIONS(2708),
    [anon_sym_yield] = ACTIONS(2708),
    [anon_sym_move] = ACTIONS(2708),
    [anon_sym_try] = ACTIONS(2708),
    [sym_integer_literal] = ACTIONS(2706),
    [aux_sym_string_literal_token1] = ACTIONS(2706),
    [sym_char_literal] = ACTIONS(2706),
    [anon_sym_true] = ACTIONS(2708),
    [anon_sym_false] = ACTIONS(2708),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2708),
    [sym_super] = ACTIONS(2708),
    [sym_crate] = ACTIONS(2708),
    [aux_sym_metavariable_token1] = ACTIONS(2706),
    [sym__raw_string_literal_start] = ACTIONS(2706),
    [sym_float_literal] = ACTIONS(2706),
    [sym__outer_block_doc_comment_start] = ACTIONS(2706),
    [sym__outer_line_doc_comment_start] = ACTIONS(2706),
  },
  [STATE(734)] = {
    [sym_line_comment] = STATE(734),
    [sym_block_comment] = STATE(734),
    [ts_builtin_sym_end] = ACTIONS(2710),
    [sym_identifier] = ACTIONS(2712),
    [anon_sym_SEMI] = ACTIONS(2710),
    [anon_sym_macro] = ACTIONS(2712),
    [anon_sym_LPAREN] = ACTIONS(2710),
    [anon_sym_LBRACE] = ACTIONS(2710),
    [anon_sym_RBRACE] = ACTIONS(2710),
    [anon_sym_macro_rules] = ACTIONS(2712),
    [anon_sym_BANG] = ACTIONS(2710),
    [anon_sym_LBRACK] = ACTIONS(2710),
    [anon_sym_STAR] = ACTIONS(2710),
    [anon_sym_u8] = ACTIONS(2712),
    [anon_sym_i8] = ACTIONS(2712),
    [anon_sym_u16] = ACTIONS(2712),
    [anon_sym_i16] = ACTIONS(2712),
    [anon_sym_u32] = ACTIONS(2712),
    [anon_sym_i32] = ACTIONS(2712),
    [anon_sym_u64] = ACTIONS(2712),
    [anon_sym_i64] = ACTIONS(2712),
    [anon_sym_u128] = ACTIONS(2712),
    [anon_sym_i128] = ACTIONS(2712),
    [anon_sym_isize] = ACTIONS(2712),
    [anon_sym_usize] = ACTIONS(2712),
    [anon_sym_f32] = ACTIONS(2712),
    [anon_sym_f64] = ACTIONS(2712),
    [anon_sym_bool] = ACTIONS(2712),
    [anon_sym_str] = ACTIONS(2712),
    [anon_sym_char] = ACTIONS(2712),
    [anon_sym_DASH] = ACTIONS(2710),
    [anon_sym_AMP] = ACTIONS(2710),
    [anon_sym_PIPE] = ACTIONS(2710),
    [anon_sym_LT] = ACTIONS(2710),
    [anon_sym_DOT_DOT] = ACTIONS(2710),
    [anon_sym_COLON_COLON] = ACTIONS(2710),
    [anon_sym_POUND] = ACTIONS(2710),
    [anon_sym_SQUOTE] = ACTIONS(2712),
    [anon_sym_async] = ACTIONS(2712),
    [anon_sym_auto] = ACTIONS(2712),
    [anon_sym_await] = ACTIONS(2712),
    [anon_sym_become] = ACTIONS(2712),
    [anon_sym_break] = ACTIONS(2712),
    [anon_sym_const] = ACTIONS(2712),
    [anon_sym_continue] = ACTIONS(2712),
    [anon_sym_default] = ACTIONS(2712),
    [anon_sym_enum] = ACTIONS(2712),
    [anon_sym_fn] = ACTIONS(2712),
    [anon_sym_for] = ACTIONS(2712),
    [anon_sym_gen] = ACTIONS(2712),
    [anon_sym_if] = ACTIONS(2712),
    [anon_sym_impl] = ACTIONS(2712),
    [anon_sym_let] = ACTIONS(2712),
    [anon_sym_loop] = ACTIONS(2712),
    [anon_sym_match] = ACTIONS(2712),
    [anon_sym_mod] = ACTIONS(2712),
    [anon_sym_pub] = ACTIONS(2712),
    [anon_sym_return] = ACTIONS(2712),
    [anon_sym_raw] = ACTIONS(2712),
    [anon_sym_static] = ACTIONS(2712),
    [anon_sym_struct] = ACTIONS(2712),
    [anon_sym_trait] = ACTIONS(2712),
    [anon_sym_type] = ACTIONS(2712),
    [anon_sym_union] = ACTIONS(2712),
    [anon_sym_unsafe] = ACTIONS(2712),
    [anon_sym_use] = ACTIONS(2712),
    [anon_sym_while] = ACTIONS(2712),
    [anon_sym_extern] = ACTIONS(2712),
    [anon_sym_safe] = ACTIONS(2712),
    [anon_sym_yield] = ACTIONS(2712),
    [anon_sym_move] = ACTIONS(2712),
    [anon_sym_try] = ACTIONS(2712),
    [sym_integer_literal] = ACTIONS(2710),
    [aux_sym_string_literal_token1] = ACTIONS(2710),
    [sym_char_literal] = ACTIONS(2710),
    [anon_sym_true] = ACTIONS(2712),
    [anon_sym_false] = ACTIONS(2712),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2712),
    [sym_super] = ACTIONS(2712),
    [sym_crate] = ACTIONS(2712),
    [aux_sym_metavariable_token1] = ACTIONS(2710),
    [sym__raw_string_literal_start] = ACTIONS(2710),
    [sym_float_literal] = ACTIONS(2710),
    [sym__outer_block_doc_comment_start] = ACTIONS(2710),
    [sym__outer_line_doc_comment_start] = ACTIONS(2710),
  },
  [STATE(735)] = {
    [sym_line_comment] = STATE(735),
    [sym_block_comment] = STATE(735),
    [ts_builtin_sym_end] = ACTIONS(2714),
    [sym_identifier] = ACTIONS(2716),
    [anon_sym_SEMI] = ACTIONS(2714),
    [anon_sym_macro] = ACTIONS(2716),
    [anon_sym_LPAREN] = ACTIONS(2714),
    [anon_sym_LBRACE] = ACTIONS(2714),
    [anon_sym_RBRACE] = ACTIONS(2714),
    [anon_sym_macro_rules] = ACTIONS(2716),
    [anon_sym_BANG] = ACTIONS(2714),
    [anon_sym_LBRACK] = ACTIONS(2714),
    [anon_sym_STAR] = ACTIONS(2714),
    [anon_sym_u8] = ACTIONS(2716),
    [anon_sym_i8] = ACTIONS(2716),
    [anon_sym_u16] = ACTIONS(2716),
    [anon_sym_i16] = ACTIONS(2716),
    [anon_sym_u32] = ACTIONS(2716),
    [anon_sym_i32] = ACTIONS(2716),
    [anon_sym_u64] = ACTIONS(2716),
    [anon_sym_i64] = ACTIONS(2716),
    [anon_sym_u128] = ACTIONS(2716),
    [anon_sym_i128] = ACTIONS(2716),
    [anon_sym_isize] = ACTIONS(2716),
    [anon_sym_usize] = ACTIONS(2716),
    [anon_sym_f32] = ACTIONS(2716),
    [anon_sym_f64] = ACTIONS(2716),
    [anon_sym_bool] = ACTIONS(2716),
    [anon_sym_str] = ACTIONS(2716),
    [anon_sym_char] = ACTIONS(2716),
    [anon_sym_DASH] = ACTIONS(2714),
    [anon_sym_AMP] = ACTIONS(2714),
    [anon_sym_PIPE] = ACTIONS(2714),
    [anon_sym_LT] = ACTIONS(2714),
    [anon_sym_DOT_DOT] = ACTIONS(2714),
    [anon_sym_COLON_COLON] = ACTIONS(2714),
    [anon_sym_POUND] = ACTIONS(2714),
    [anon_sym_SQUOTE] = ACTIONS(2716),
    [anon_sym_async] = ACTIONS(2716),
    [anon_sym_auto] = ACTIONS(2716),
    [anon_sym_await] = ACTIONS(2716),
    [anon_sym_become] = ACTIONS(2716),
    [anon_sym_break] = ACTIONS(2716),
    [anon_sym_const] = ACTIONS(2716),
    [anon_sym_continue] = ACTIONS(2716),
    [anon_sym_default] = ACTIONS(2716),
    [anon_sym_enum] = ACTIONS(2716),
    [anon_sym_fn] = ACTIONS(2716),
    [anon_sym_for] = ACTIONS(2716),
    [anon_sym_gen] = ACTIONS(2716),
    [anon_sym_if] = ACTIONS(2716),
    [anon_sym_impl] = ACTIONS(2716),
    [anon_sym_let] = ACTIONS(2716),
    [anon_sym_loop] = ACTIONS(2716),
    [anon_sym_match] = ACTIONS(2716),
    [anon_sym_mod] = ACTIONS(2716),
    [anon_sym_pub] = ACTIONS(2716),
    [anon_sym_return] = ACTIONS(2716),
    [anon_sym_raw] = ACTIONS(2716),
    [anon_sym_static] = ACTIONS(2716),
    [anon_sym_struct] = ACTIONS(2716),
    [anon_sym_trait] = ACTIONS(2716),
    [anon_sym_type] = ACTIONS(2716),
    [anon_sym_union] = ACTIONS(2716),
    [anon_sym_unsafe] = ACTIONS(2716),
    [anon_sym_use] = ACTIONS(2716),
    [anon_sym_while] = ACTIONS(2716),
    [anon_sym_extern] = ACTIONS(2716),
    [anon_sym_safe] = ACTIONS(2716),
    [anon_sym_yield] = ACTIONS(2716),
    [anon_sym_move] = ACTIONS(2716),
    [anon_sym_try] = ACTIONS(2716),
    [sym_integer_literal] = ACTIONS(2714),
    [aux_sym_string_literal_token1] = ACTIONS(2714),
    [sym_char_literal] = ACTIONS(2714),
    [anon_sym_true] = ACTIONS(2716),
    [anon_sym_false] = ACTIONS(2716),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2716),
    [sym_super] = ACTIONS(2716),
    [sym_crate] = ACTIONS(2716),
    [aux_sym_metavariable_token1] = ACTIONS(2714),
    [sym__raw_string_literal_start] = ACTIONS(2714),
    [sym_float_literal] = ACTIONS(2714),
    [sym__outer_block_doc_comment_start] = ACTIONS(2714),
    [sym__outer_line_doc_comment_start] = ACTIONS(2714),
  },
  [STATE(736)] = {
    [sym_line_comment] = STATE(736),
    [sym_block_comment] = STATE(736),
    [ts_builtin_sym_end] = ACTIONS(2718),
    [sym_identifier] = ACTIONS(2720),
    [anon_sym_SEMI] = ACTIONS(2718),
    [anon_sym_macro] = ACTIONS(2720),
    [anon_sym_LPAREN] = ACTIONS(2718),
    [anon_sym_LBRACE] = ACTIONS(2718),
    [anon_sym_RBRACE] = ACTIONS(2718),
    [anon_sym_macro_rules] = ACTIONS(2720),
    [anon_sym_BANG] = ACTIONS(2718),
    [anon_sym_LBRACK] = ACTIONS(2718),
    [anon_sym_STAR] = ACTIONS(2718),
    [anon_sym_u8] = ACTIONS(2720),
    [anon_sym_i8] = ACTIONS(2720),
    [anon_sym_u16] = ACTIONS(2720),
    [anon_sym_i16] = ACTIONS(2720),
    [anon_sym_u32] = ACTIONS(2720),
    [anon_sym_i32] = ACTIONS(2720),
    [anon_sym_u64] = ACTIONS(2720),
    [anon_sym_i64] = ACTIONS(2720),
    [anon_sym_u128] = ACTIONS(2720),
    [anon_sym_i128] = ACTIONS(2720),
    [anon_sym_isize] = ACTIONS(2720),
    [anon_sym_usize] = ACTIONS(2720),
    [anon_sym_f32] = ACTIONS(2720),
    [anon_sym_f64] = ACTIONS(2720),
    [anon_sym_bool] = ACTIONS(2720),
    [anon_sym_str] = ACTIONS(2720),
    [anon_sym_char] = ACTIONS(2720),
    [anon_sym_DASH] = ACTIONS(2718),
    [anon_sym_AMP] = ACTIONS(2718),
    [anon_sym_PIPE] = ACTIONS(2718),
    [anon_sym_LT] = ACTIONS(2718),
    [anon_sym_DOT_DOT] = ACTIONS(2718),
    [anon_sym_COLON_COLON] = ACTIONS(2718),
    [anon_sym_POUND] = ACTIONS(2718),
    [anon_sym_SQUOTE] = ACTIONS(2720),
    [anon_sym_async] = ACTIONS(2720),
    [anon_sym_auto] = ACTIONS(2720),
    [anon_sym_await] = ACTIONS(2720),
    [anon_sym_become] = ACTIONS(2720),
    [anon_sym_break] = ACTIONS(2720),
    [anon_sym_const] = ACTIONS(2720),
    [anon_sym_continue] = ACTIONS(2720),
    [anon_sym_default] = ACTIONS(2720),
    [anon_sym_enum] = ACTIONS(2720),
    [anon_sym_fn] = ACTIONS(2720),
    [anon_sym_for] = ACTIONS(2720),
    [anon_sym_gen] = ACTIONS(2720),
    [anon_sym_if] = ACTIONS(2720),
    [anon_sym_impl] = ACTIONS(2720),
    [anon_sym_let] = ACTIONS(2720),
    [anon_sym_loop] = ACTIONS(2720),
    [anon_sym_match] = ACTIONS(2720),
    [anon_sym_mod] = ACTIONS(2720),
    [anon_sym_pub] = ACTIONS(2720),
    [anon_sym_return] = ACTIONS(2720),
    [anon_sym_raw] = ACTIONS(2720),
    [anon_sym_static] = ACTIONS(2720),
    [anon_sym_struct] = ACTIONS(2720),
    [anon_sym_trait] = ACTIONS(2720),
    [anon_sym_type] = ACTIONS(2720),
    [anon_sym_union] = ACTIONS(2720),
    [anon_sym_unsafe] = ACTIONS(2720),
    [anon_sym_use] = ACTIONS(2720),
    [anon_sym_while] = ACTIONS(2720),
    [anon_sym_extern] = ACTIONS(2720),
    [anon_sym_safe] = ACTIONS(2720),
    [anon_sym_yield] = ACTIONS(2720),
    [anon_sym_move] = ACTIONS(2720),
    [anon_sym_try] = ACTIONS(2720),
    [sym_integer_literal] = ACTIONS(2718),
    [aux_sym_string_literal_token1] = ACTIONS(2718),
    [sym_char_literal] = ACTIONS(2718),
    [anon_sym_true] = ACTIONS(2720),
    [anon_sym_false] = ACTIONS(2720),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2720),
    [sym_super] = ACTIONS(2720),
    [sym_crate] = ACTIONS(2720),
    [aux_sym_metavariable_token1] = ACTIONS(2718),
    [sym__raw_string_literal_start] = ACTIONS(2718),
    [sym_float_literal] = ACTIONS(2718),
    [sym__outer_block_doc_comment_start] = ACTIONS(2718),
    [sym__outer_line_doc_comment_start] = ACTIONS(2718),
  },
  [STATE(737)] = {
    [sym_line_comment] = STATE(737),
    [sym_block_comment] = STATE(737),
    [ts_builtin_sym_end] = ACTIONS(2722),
    [sym_identifier] = ACTIONS(2724),
    [anon_sym_SEMI] = ACTIONS(2722),
    [anon_sym_macro] = ACTIONS(2724),
    [anon_sym_LPAREN] = ACTIONS(2722),
    [anon_sym_LBRACE] = ACTIONS(2722),
    [anon_sym_RBRACE] = ACTIONS(2722),
    [anon_sym_macro_rules] = ACTIONS(2724),
    [anon_sym_BANG] = ACTIONS(2722),
    [anon_sym_LBRACK] = ACTIONS(2722),
    [anon_sym_STAR] = ACTIONS(2722),
    [anon_sym_u8] = ACTIONS(2724),
    [anon_sym_i8] = ACTIONS(2724),
    [anon_sym_u16] = ACTIONS(2724),
    [anon_sym_i16] = ACTIONS(2724),
    [anon_sym_u32] = ACTIONS(2724),
    [anon_sym_i32] = ACTIONS(2724),
    [anon_sym_u64] = ACTIONS(2724),
    [anon_sym_i64] = ACTIONS(2724),
    [anon_sym_u128] = ACTIONS(2724),
    [anon_sym_i128] = ACTIONS(2724),
    [anon_sym_isize] = ACTIONS(2724),
    [anon_sym_usize] = ACTIONS(2724),
    [anon_sym_f32] = ACTIONS(2724),
    [anon_sym_f64] = ACTIONS(2724),
    [anon_sym_bool] = ACTIONS(2724),
    [anon_sym_str] = ACTIONS(2724),
    [anon_sym_char] = ACTIONS(2724),
    [anon_sym_DASH] = ACTIONS(2722),
    [anon_sym_AMP] = ACTIONS(2722),
    [anon_sym_PIPE] = ACTIONS(2722),
    [anon_sym_LT] = ACTIONS(2722),
    [anon_sym_DOT_DOT] = ACTIONS(2722),
    [anon_sym_COLON_COLON] = ACTIONS(2722),
    [anon_sym_POUND] = ACTIONS(2722),
    [anon_sym_SQUOTE] = ACTIONS(2724),
    [anon_sym_async] = ACTIONS(2724),
    [anon_sym_auto] = ACTIONS(2724),
    [anon_sym_await] = ACTIONS(2724),
    [anon_sym_become] = ACTIONS(2724),
    [anon_sym_break] = ACTIONS(2724),
    [anon_sym_const] = ACTIONS(2724),
    [anon_sym_continue] = ACTIONS(2724),
    [anon_sym_default] = ACTIONS(2724),
    [anon_sym_enum] = ACTIONS(2724),
    [anon_sym_fn] = ACTIONS(2724),
    [anon_sym_for] = ACTIONS(2724),
    [anon_sym_gen] = ACTIONS(2724),
    [anon_sym_if] = ACTIONS(2724),
    [anon_sym_impl] = ACTIONS(2724),
    [anon_sym_let] = ACTIONS(2724),
    [anon_sym_loop] = ACTIONS(2724),
    [anon_sym_match] = ACTIONS(2724),
    [anon_sym_mod] = ACTIONS(2724),
    [anon_sym_pub] = ACTIONS(2724),
    [anon_sym_return] = ACTIONS(2724),
    [anon_sym_raw] = ACTIONS(2724),
    [anon_sym_static] = ACTIONS(2724),
    [anon_sym_struct] = ACTIONS(2724),
    [anon_sym_trait] = ACTIONS(2724),
    [anon_sym_type] = ACTIONS(2724),
    [anon_sym_union] = ACTIONS(2724),
    [anon_sym_unsafe] = ACTIONS(2724),
    [anon_sym_use] = ACTIONS(2724),
    [anon_sym_while] = ACTIONS(2724),
    [anon_sym_extern] = ACTIONS(2724),
    [anon_sym_safe] = ACTIONS(2724),
    [anon_sym_yield] = ACTIONS(2724),
    [anon_sym_move] = ACTIONS(2724),
    [anon_sym_try] = ACTIONS(2724),
    [sym_integer_literal] = ACTIONS(2722),
    [aux_sym_string_literal_token1] = ACTIONS(2722),
    [sym_char_literal] = ACTIONS(2722),
    [anon_sym_true] = ACTIONS(2724),
    [anon_sym_false] = ACTIONS(2724),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2724),
    [sym_super] = ACTIONS(2724),
    [sym_crate] = ACTIONS(2724),
    [aux_sym_metavariable_token1] = ACTIONS(2722),
    [sym__raw_string_literal_start] = ACTIONS(2722),
    [sym_float_literal] = ACTIONS(2722),
    [sym__outer_block_doc_comment_start] = ACTIONS(2722),
    [sym__outer_line_doc_comment_start] = ACTIONS(2722),
  },
  [STATE(738)] = {
    [sym_line_comment] = STATE(738),
    [sym_block_comment] = STATE(738),
    [ts_builtin_sym_end] = ACTIONS(2726),
    [sym_identifier] = ACTIONS(2728),
    [anon_sym_SEMI] = ACTIONS(2726),
    [anon_sym_macro] = ACTIONS(2728),
    [anon_sym_LPAREN] = ACTIONS(2726),
    [anon_sym_LBRACE] = ACTIONS(2726),
    [anon_sym_RBRACE] = ACTIONS(2726),
    [anon_sym_macro_rules] = ACTIONS(2728),
    [anon_sym_BANG] = ACTIONS(2726),
    [anon_sym_LBRACK] = ACTIONS(2726),
    [anon_sym_STAR] = ACTIONS(2726),
    [anon_sym_u8] = ACTIONS(2728),
    [anon_sym_i8] = ACTIONS(2728),
    [anon_sym_u16] = ACTIONS(2728),
    [anon_sym_i16] = ACTIONS(2728),
    [anon_sym_u32] = ACTIONS(2728),
    [anon_sym_i32] = ACTIONS(2728),
    [anon_sym_u64] = ACTIONS(2728),
    [anon_sym_i64] = ACTIONS(2728),
    [anon_sym_u128] = ACTIONS(2728),
    [anon_sym_i128] = ACTIONS(2728),
    [anon_sym_isize] = ACTIONS(2728),
    [anon_sym_usize] = ACTIONS(2728),
    [anon_sym_f32] = ACTIONS(2728),
    [anon_sym_f64] = ACTIONS(2728),
    [anon_sym_bool] = ACTIONS(2728),
    [anon_sym_str] = ACTIONS(2728),
    [anon_sym_char] = ACTIONS(2728),
    [anon_sym_DASH] = ACTIONS(2726),
    [anon_sym_AMP] = ACTIONS(2726),
    [anon_sym_PIPE] = ACTIONS(2726),
    [anon_sym_LT] = ACTIONS(2726),
    [anon_sym_DOT_DOT] = ACTIONS(2726),
    [anon_sym_COLON_COLON] = ACTIONS(2726),
    [anon_sym_POUND] = ACTIONS(2726),
    [anon_sym_SQUOTE] = ACTIONS(2728),
    [anon_sym_async] = ACTIONS(2728),
    [anon_sym_auto] = ACTIONS(2728),
    [anon_sym_await] = ACTIONS(2728),
    [anon_sym_become] = ACTIONS(2728),
    [anon_sym_break] = ACTIONS(2728),
    [anon_sym_const] = ACTIONS(2728),
    [anon_sym_continue] = ACTIONS(2728),
    [anon_sym_default] = ACTIONS(2728),
    [anon_sym_enum] = ACTIONS(2728),
    [anon_sym_fn] = ACTIONS(2728),
    [anon_sym_for] = ACTIONS(2728),
    [anon_sym_gen] = ACTIONS(2728),
    [anon_sym_if] = ACTIONS(2728),
    [anon_sym_impl] = ACTIONS(2728),
    [anon_sym_let] = ACTIONS(2728),
    [anon_sym_loop] = ACTIONS(2728),
    [anon_sym_match] = ACTIONS(2728),
    [anon_sym_mod] = ACTIONS(2728),
    [anon_sym_pub] = ACTIONS(2728),
    [anon_sym_return] = ACTIONS(2728),
    [anon_sym_raw] = ACTIONS(2728),
    [anon_sym_static] = ACTIONS(2728),
    [anon_sym_struct] = ACTIONS(2728),
    [anon_sym_trait] = ACTIONS(2728),
    [anon_sym_type] = ACTIONS(2728),
    [anon_sym_union] = ACTIONS(2728),
    [anon_sym_unsafe] = ACTIONS(2728),
    [anon_sym_use] = ACTIONS(2728),
    [anon_sym_while] = ACTIONS(2728),
    [anon_sym_extern] = ACTIONS(2728),
    [anon_sym_safe] = ACTIONS(2728),
    [anon_sym_yield] = ACTIONS(2728),
    [anon_sym_move] = ACTIONS(2728),
    [anon_sym_try] = ACTIONS(2728),
    [sym_integer_literal] = ACTIONS(2726),
    [aux_sym_string_literal_token1] = ACTIONS(2726),
    [sym_char_literal] = ACTIONS(2726),
    [anon_sym_true] = ACTIONS(2728),
    [anon_sym_false] = ACTIONS(2728),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2728),
    [sym_super] = ACTIONS(2728),
    [sym_crate] = ACTIONS(2728),
    [aux_sym_metavariable_token1] = ACTIONS(2726),
    [sym__raw_string_literal_start] = ACTIONS(2726),
    [sym_float_literal] = ACTIONS(2726),
    [sym__outer_block_doc_comment_start] = ACTIONS(2726),
    [sym__outer_line_doc_comment_start] = ACTIONS(2726),
  },
  [STATE(739)] = {
    [sym_line_comment] = STATE(739),
    [sym_block_comment] = STATE(739),
    [ts_builtin_sym_end] = ACTIONS(2730),
    [sym_identifier] = ACTIONS(2732),
    [anon_sym_SEMI] = ACTIONS(2730),
    [anon_sym_macro] = ACTIONS(2732),
    [anon_sym_LPAREN] = ACTIONS(2730),
    [anon_sym_LBRACE] = ACTIONS(2730),
    [anon_sym_RBRACE] = ACTIONS(2730),
    [anon_sym_macro_rules] = ACTIONS(2732),
    [anon_sym_BANG] = ACTIONS(2730),
    [anon_sym_LBRACK] = ACTIONS(2730),
    [anon_sym_STAR] = ACTIONS(2730),
    [anon_sym_u8] = ACTIONS(2732),
    [anon_sym_i8] = ACTIONS(2732),
    [anon_sym_u16] = ACTIONS(2732),
    [anon_sym_i16] = ACTIONS(2732),
    [anon_sym_u32] = ACTIONS(2732),
    [anon_sym_i32] = ACTIONS(2732),
    [anon_sym_u64] = ACTIONS(2732),
    [anon_sym_i64] = ACTIONS(2732),
    [anon_sym_u128] = ACTIONS(2732),
    [anon_sym_i128] = ACTIONS(2732),
    [anon_sym_isize] = ACTIONS(2732),
    [anon_sym_usize] = ACTIONS(2732),
    [anon_sym_f32] = ACTIONS(2732),
    [anon_sym_f64] = ACTIONS(2732),
    [anon_sym_bool] = ACTIONS(2732),
    [anon_sym_str] = ACTIONS(2732),
    [anon_sym_char] = ACTIONS(2732),
    [anon_sym_DASH] = ACTIONS(2730),
    [anon_sym_AMP] = ACTIONS(2730),
    [anon_sym_PIPE] = ACTIONS(2730),
    [anon_sym_LT] = ACTIONS(2730),
    [anon_sym_DOT_DOT] = ACTIONS(2730),
    [anon_sym_COLON_COLON] = ACTIONS(2730),
    [anon_sym_POUND] = ACTIONS(2730),
    [anon_sym_SQUOTE] = ACTIONS(2732),
    [anon_sym_async] = ACTIONS(2732),
    [anon_sym_auto] = ACTIONS(2732),
    [anon_sym_await] = ACTIONS(2732),
    [anon_sym_become] = ACTIONS(2732),
    [anon_sym_break] = ACTIONS(2732),
    [anon_sym_const] = ACTIONS(2732),
    [anon_sym_continue] = ACTIONS(2732),
    [anon_sym_default] = ACTIONS(2732),
    [anon_sym_enum] = ACTIONS(2732),
    [anon_sym_fn] = ACTIONS(2732),
    [anon_sym_for] = ACTIONS(2732),
    [anon_sym_gen] = ACTIONS(2732),
    [anon_sym_if] = ACTIONS(2732),
    [anon_sym_impl] = ACTIONS(2732),
    [anon_sym_let] = ACTIONS(2732),
    [anon_sym_loop] = ACTIONS(2732),
    [anon_sym_match] = ACTIONS(2732),
    [anon_sym_mod] = ACTIONS(2732),
    [anon_sym_pub] = ACTIONS(2732),
    [anon_sym_return] = ACTIONS(2732),
    [anon_sym_raw] = ACTIONS(2732),
    [anon_sym_static] = ACTIONS(2732),
    [anon_sym_struct] = ACTIONS(2732),
    [anon_sym_trait] = ACTIONS(2732),
    [anon_sym_type] = ACTIONS(2732),
    [anon_sym_union] = ACTIONS(2732),
    [anon_sym_unsafe] = ACTIONS(2732),
    [anon_sym_use] = ACTIONS(2732),
    [anon_sym_while] = ACTIONS(2732),
    [anon_sym_extern] = ACTIONS(2732),
    [anon_sym_safe] = ACTIONS(2732),
    [anon_sym_yield] = ACTIONS(2732),
    [anon_sym_move] = ACTIONS(2732),
    [anon_sym_try] = ACTIONS(2732),
    [sym_integer_literal] = ACTIONS(2730),
    [aux_sym_string_literal_token1] = ACTIONS(2730),
    [sym_char_literal] = ACTIONS(2730),
    [anon_sym_true] = ACTIONS(2732),
    [anon_sym_false] = ACTIONS(2732),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2732),
    [sym_super] = ACTIONS(2732),
    [sym_crate] = ACTIONS(2732),
    [aux_sym_metavariable_token1] = ACTIONS(2730),
    [sym__raw_string_literal_start] = ACTIONS(2730),
    [sym_float_literal] = ACTIONS(2730),
    [sym__outer_block_doc_comment_start] = ACTIONS(2730),
    [sym__outer_line_doc_comment_start] = ACTIONS(2730),
  },
  [STATE(740)] = {
    [sym_line_comment] = STATE(740),
    [sym_block_comment] = STATE(740),
    [ts_builtin_sym_end] = ACTIONS(2734),
    [sym_identifier] = ACTIONS(2736),
    [anon_sym_SEMI] = ACTIONS(2734),
    [anon_sym_macro] = ACTIONS(2736),
    [anon_sym_LPAREN] = ACTIONS(2734),
    [anon_sym_LBRACE] = ACTIONS(2734),
    [anon_sym_RBRACE] = ACTIONS(2734),
    [anon_sym_macro_rules] = ACTIONS(2736),
    [anon_sym_BANG] = ACTIONS(2734),
    [anon_sym_LBRACK] = ACTIONS(2734),
    [anon_sym_STAR] = ACTIONS(2734),
    [anon_sym_u8] = ACTIONS(2736),
    [anon_sym_i8] = ACTIONS(2736),
    [anon_sym_u16] = ACTIONS(2736),
    [anon_sym_i16] = ACTIONS(2736),
    [anon_sym_u32] = ACTIONS(2736),
    [anon_sym_i32] = ACTIONS(2736),
    [anon_sym_u64] = ACTIONS(2736),
    [anon_sym_i64] = ACTIONS(2736),
    [anon_sym_u128] = ACTIONS(2736),
    [anon_sym_i128] = ACTIONS(2736),
    [anon_sym_isize] = ACTIONS(2736),
    [anon_sym_usize] = ACTIONS(2736),
    [anon_sym_f32] = ACTIONS(2736),
    [anon_sym_f64] = ACTIONS(2736),
    [anon_sym_bool] = ACTIONS(2736),
    [anon_sym_str] = ACTIONS(2736),
    [anon_sym_char] = ACTIONS(2736),
    [anon_sym_DASH] = ACTIONS(2734),
    [anon_sym_AMP] = ACTIONS(2734),
    [anon_sym_PIPE] = ACTIONS(2734),
    [anon_sym_LT] = ACTIONS(2734),
    [anon_sym_DOT_DOT] = ACTIONS(2734),
    [anon_sym_COLON_COLON] = ACTIONS(2734),
    [anon_sym_POUND] = ACTIONS(2734),
    [anon_sym_SQUOTE] = ACTIONS(2736),
    [anon_sym_async] = ACTIONS(2736),
    [anon_sym_auto] = ACTIONS(2736),
    [anon_sym_await] = ACTIONS(2736),
    [anon_sym_become] = ACTIONS(2736),
    [anon_sym_break] = ACTIONS(2736),
    [anon_sym_const] = ACTIONS(2736),
    [anon_sym_continue] = ACTIONS(2736),
    [anon_sym_default] = ACTIONS(2736),
    [anon_sym_enum] = ACTIONS(2736),
    [anon_sym_fn] = ACTIONS(2736),
    [anon_sym_for] = ACTIONS(2736),
    [anon_sym_gen] = ACTIONS(2736),
    [anon_sym_if] = ACTIONS(2736),
    [anon_sym_impl] = ACTIONS(2736),
    [anon_sym_let] = ACTIONS(2736),
    [anon_sym_loop] = ACTIONS(2736),
    [anon_sym_match] = ACTIONS(2736),
    [anon_sym_mod] = ACTIONS(2736),
    [anon_sym_pub] = ACTIONS(2736),
    [anon_sym_return] = ACTIONS(2736),
    [anon_sym_raw] = ACTIONS(2736),
    [anon_sym_static] = ACTIONS(2736),
    [anon_sym_struct] = ACTIONS(2736),
    [anon_sym_trait] = ACTIONS(2736),
    [anon_sym_type] = ACTIONS(2736),
    [anon_sym_union] = ACTIONS(2736),
    [anon_sym_unsafe] = ACTIONS(2736),
    [anon_sym_use] = ACTIONS(2736),
    [anon_sym_while] = ACTIONS(2736),
    [anon_sym_extern] = ACTIONS(2736),
    [anon_sym_safe] = ACTIONS(2736),
    [anon_sym_yield] = ACTIONS(2736),
    [anon_sym_move] = ACTIONS(2736),
    [anon_sym_try] = ACTIONS(2736),
    [sym_integer_literal] = ACTIONS(2734),
    [aux_sym_string_literal_token1] = ACTIONS(2734),
    [sym_char_literal] = ACTIONS(2734),
    [anon_sym_true] = ACTIONS(2736),
    [anon_sym_false] = ACTIONS(2736),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2736),
    [sym_super] = ACTIONS(2736),
    [sym_crate] = ACTIONS(2736),
    [aux_sym_metavariable_token1] = ACTIONS(2734),
    [sym__raw_string_literal_start] = ACTIONS(2734),
    [sym_float_literal] = ACTIONS(2734),
    [sym__outer_block_doc_comment_start] = ACTIONS(2734),
    [sym__outer_line_doc_comment_start] = ACTIONS(2734),
  },
  [STATE(741)] = {
    [sym_line_comment] = STATE(741),
    [sym_block_comment] = STATE(741),
    [ts_builtin_sym_end] = ACTIONS(2738),
    [sym_identifier] = ACTIONS(2740),
    [anon_sym_SEMI] = ACTIONS(2738),
    [anon_sym_macro] = ACTIONS(2740),
    [anon_sym_LPAREN] = ACTIONS(2738),
    [anon_sym_LBRACE] = ACTIONS(2738),
    [anon_sym_RBRACE] = ACTIONS(2738),
    [anon_sym_macro_rules] = ACTIONS(2740),
    [anon_sym_BANG] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(2738),
    [anon_sym_STAR] = ACTIONS(2738),
    [anon_sym_u8] = ACTIONS(2740),
    [anon_sym_i8] = ACTIONS(2740),
    [anon_sym_u16] = ACTIONS(2740),
    [anon_sym_i16] = ACTIONS(2740),
    [anon_sym_u32] = ACTIONS(2740),
    [anon_sym_i32] = ACTIONS(2740),
    [anon_sym_u64] = ACTIONS(2740),
    [anon_sym_i64] = ACTIONS(2740),
    [anon_sym_u128] = ACTIONS(2740),
    [anon_sym_i128] = ACTIONS(2740),
    [anon_sym_isize] = ACTIONS(2740),
    [anon_sym_usize] = ACTIONS(2740),
    [anon_sym_f32] = ACTIONS(2740),
    [anon_sym_f64] = ACTIONS(2740),
    [anon_sym_bool] = ACTIONS(2740),
    [anon_sym_str] = ACTIONS(2740),
    [anon_sym_char] = ACTIONS(2740),
    [anon_sym_DASH] = ACTIONS(2738),
    [anon_sym_AMP] = ACTIONS(2738),
    [anon_sym_PIPE] = ACTIONS(2738),
    [anon_sym_LT] = ACTIONS(2738),
    [anon_sym_DOT_DOT] = ACTIONS(2738),
    [anon_sym_COLON_COLON] = ACTIONS(2738),
    [anon_sym_POUND] = ACTIONS(2738),
    [anon_sym_SQUOTE] = ACTIONS(2740),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_auto] = ACTIONS(2740),
    [anon_sym_await] = ACTIONS(2740),
    [anon_sym_become] = ACTIONS(2740),
    [anon_sym_break] = ACTIONS(2740),
    [anon_sym_const] = ACTIONS(2740),
    [anon_sym_continue] = ACTIONS(2740),
    [anon_sym_default] = ACTIONS(2740),
    [anon_sym_enum] = ACTIONS(2740),
    [anon_sym_fn] = ACTIONS(2740),
    [anon_sym_for] = ACTIONS(2740),
    [anon_sym_gen] = ACTIONS(2740),
    [anon_sym_if] = ACTIONS(2740),
    [anon_sym_impl] = ACTIONS(2740),
    [anon_sym_let] = ACTIONS(2740),
    [anon_sym_loop] = ACTIONS(2740),
    [anon_sym_match] = ACTIONS(2740),
    [anon_sym_mod] = ACTIONS(2740),
    [anon_sym_pub] = ACTIONS(2740),
    [anon_sym_return] = ACTIONS(2740),
    [anon_sym_raw] = ACTIONS(2740),
    [anon_sym_static] = ACTIONS(2740),
    [anon_sym_struct] = ACTIONS(2740),
    [anon_sym_trait] = ACTIONS(2740),
    [anon_sym_type] = ACTIONS(2740),
    [anon_sym_union] = ACTIONS(2740),
    [anon_sym_unsafe] = ACTIONS(2740),
    [anon_sym_use] = ACTIONS(2740),
    [anon_sym_while] = ACTIONS(2740),
    [anon_sym_extern] = ACTIONS(2740),
    [anon_sym_safe] = ACTIONS(2740),
    [anon_sym_yield] = ACTIONS(2740),
    [anon_sym_move] = ACTIONS(2740),
    [anon_sym_try] = ACTIONS(2740),
    [sym_integer_literal] = ACTIONS(2738),
    [aux_sym_string_literal_token1] = ACTIONS(2738),
    [sym_char_literal] = ACTIONS(2738),
    [anon_sym_true] = ACTIONS(2740),
    [anon_sym_false] = ACTIONS(2740),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2740),
    [sym_super] = ACTIONS(2740),
    [sym_crate] = ACTIONS(2740),
    [aux_sym_metavariable_token1] = ACTIONS(2738),
    [sym__raw_string_literal_start] = ACTIONS(2738),
    [sym_float_literal] = ACTIONS(2738),
    [sym__outer_block_doc_comment_start] = ACTIONS(2738),
    [sym__outer_line_doc_comment_start] = ACTIONS(2738),
  },
  [STATE(742)] = {
    [sym_line_comment] = STATE(742),
    [sym_block_comment] = STATE(742),
    [ts_builtin_sym_end] = ACTIONS(2742),
    [sym_identifier] = ACTIONS(2744),
    [anon_sym_SEMI] = ACTIONS(2742),
    [anon_sym_macro] = ACTIONS(2744),
    [anon_sym_LPAREN] = ACTIONS(2742),
    [anon_sym_LBRACE] = ACTIONS(2742),
    [anon_sym_RBRACE] = ACTIONS(2742),
    [anon_sym_macro_rules] = ACTIONS(2744),
    [anon_sym_BANG] = ACTIONS(2742),
    [anon_sym_LBRACK] = ACTIONS(2742),
    [anon_sym_STAR] = ACTIONS(2742),
    [anon_sym_u8] = ACTIONS(2744),
    [anon_sym_i8] = ACTIONS(2744),
    [anon_sym_u16] = ACTIONS(2744),
    [anon_sym_i16] = ACTIONS(2744),
    [anon_sym_u32] = ACTIONS(2744),
    [anon_sym_i32] = ACTIONS(2744),
    [anon_sym_u64] = ACTIONS(2744),
    [anon_sym_i64] = ACTIONS(2744),
    [anon_sym_u128] = ACTIONS(2744),
    [anon_sym_i128] = ACTIONS(2744),
    [anon_sym_isize] = ACTIONS(2744),
    [anon_sym_usize] = ACTIONS(2744),
    [anon_sym_f32] = ACTIONS(2744),
    [anon_sym_f64] = ACTIONS(2744),
    [anon_sym_bool] = ACTIONS(2744),
    [anon_sym_str] = ACTIONS(2744),
    [anon_sym_char] = ACTIONS(2744),
    [anon_sym_DASH] = ACTIONS(2742),
    [anon_sym_AMP] = ACTIONS(2742),
    [anon_sym_PIPE] = ACTIONS(2742),
    [anon_sym_LT] = ACTIONS(2742),
    [anon_sym_DOT_DOT] = ACTIONS(2742),
    [anon_sym_COLON_COLON] = ACTIONS(2742),
    [anon_sym_POUND] = ACTIONS(2742),
    [anon_sym_SQUOTE] = ACTIONS(2744),
    [anon_sym_async] = ACTIONS(2744),
    [anon_sym_auto] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2744),
    [anon_sym_become] = ACTIONS(2744),
    [anon_sym_break] = ACTIONS(2744),
    [anon_sym_const] = ACTIONS(2744),
    [anon_sym_continue] = ACTIONS(2744),
    [anon_sym_default] = ACTIONS(2744),
    [anon_sym_enum] = ACTIONS(2744),
    [anon_sym_fn] = ACTIONS(2744),
    [anon_sym_for] = ACTIONS(2744),
    [anon_sym_gen] = ACTIONS(2744),
    [anon_sym_if] = ACTIONS(2744),
    [anon_sym_impl] = ACTIONS(2744),
    [anon_sym_let] = ACTIONS(2744),
    [anon_sym_loop] = ACTIONS(2744),
    [anon_sym_match] = ACTIONS(2744),
    [anon_sym_mod] = ACTIONS(2744),
    [anon_sym_pub] = ACTIONS(2744),
    [anon_sym_return] = ACTIONS(2744),
    [anon_sym_raw] = ACTIONS(2744),
    [anon_sym_static] = ACTIONS(2744),
    [anon_sym_struct] = ACTIONS(2744),
    [anon_sym_trait] = ACTIONS(2744),
    [anon_sym_type] = ACTIONS(2744),
    [anon_sym_union] = ACTIONS(2744),
    [anon_sym_unsafe] = ACTIONS(2744),
    [anon_sym_use] = ACTIONS(2744),
    [anon_sym_while] = ACTIONS(2744),
    [anon_sym_extern] = ACTIONS(2744),
    [anon_sym_safe] = ACTIONS(2744),
    [anon_sym_yield] = ACTIONS(2744),
    [anon_sym_move] = ACTIONS(2744),
    [anon_sym_try] = ACTIONS(2744),
    [sym_integer_literal] = ACTIONS(2742),
    [aux_sym_string_literal_token1] = ACTIONS(2742),
    [sym_char_literal] = ACTIONS(2742),
    [anon_sym_true] = ACTIONS(2744),
    [anon_sym_false] = ACTIONS(2744),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2744),
    [sym_super] = ACTIONS(2744),
    [sym_crate] = ACTIONS(2744),
    [aux_sym_metavariable_token1] = ACTIONS(2742),
    [sym__raw_string_literal_start] = ACTIONS(2742),
    [sym_float_literal] = ACTIONS(2742),
    [sym__outer_block_doc_comment_start] = ACTIONS(2742),
    [sym__outer_line_doc_comment_start] = ACTIONS(2742),
  },
  [STATE(743)] = {
    [sym_line_comment] = STATE(743),
    [sym_block_comment] = STATE(743),
    [ts_builtin_sym_end] = ACTIONS(2746),
    [sym_identifier] = ACTIONS(2748),
    [anon_sym_SEMI] = ACTIONS(2746),
    [anon_sym_macro] = ACTIONS(2748),
    [anon_sym_LPAREN] = ACTIONS(2746),
    [anon_sym_LBRACE] = ACTIONS(2746),
    [anon_sym_RBRACE] = ACTIONS(2746),
    [anon_sym_macro_rules] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2746),
    [anon_sym_LBRACK] = ACTIONS(2746),
    [anon_sym_STAR] = ACTIONS(2746),
    [anon_sym_u8] = ACTIONS(2748),
    [anon_sym_i8] = ACTIONS(2748),
    [anon_sym_u16] = ACTIONS(2748),
    [anon_sym_i16] = ACTIONS(2748),
    [anon_sym_u32] = ACTIONS(2748),
    [anon_sym_i32] = ACTIONS(2748),
    [anon_sym_u64] = ACTIONS(2748),
    [anon_sym_i64] = ACTIONS(2748),
    [anon_sym_u128] = ACTIONS(2748),
    [anon_sym_i128] = ACTIONS(2748),
    [anon_sym_isize] = ACTIONS(2748),
    [anon_sym_usize] = ACTIONS(2748),
    [anon_sym_f32] = ACTIONS(2748),
    [anon_sym_f64] = ACTIONS(2748),
    [anon_sym_bool] = ACTIONS(2748),
    [anon_sym_str] = ACTIONS(2748),
    [anon_sym_char] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2746),
    [anon_sym_AMP] = ACTIONS(2746),
    [anon_sym_PIPE] = ACTIONS(2746),
    [anon_sym_LT] = ACTIONS(2746),
    [anon_sym_DOT_DOT] = ACTIONS(2746),
    [anon_sym_COLON_COLON] = ACTIONS(2746),
    [anon_sym_POUND] = ACTIONS(2746),
    [anon_sym_SQUOTE] = ACTIONS(2748),
    [anon_sym_async] = ACTIONS(2748),
    [anon_sym_auto] = ACTIONS(2748),
    [anon_sym_await] = ACTIONS(2748),
    [anon_sym_become] = ACTIONS(2748),
    [anon_sym_break] = ACTIONS(2748),
    [anon_sym_const] = ACTIONS(2748),
    [anon_sym_continue] = ACTIONS(2748),
    [anon_sym_default] = ACTIONS(2748),
    [anon_sym_enum] = ACTIONS(2748),
    [anon_sym_fn] = ACTIONS(2748),
    [anon_sym_for] = ACTIONS(2748),
    [anon_sym_gen] = ACTIONS(2748),
    [anon_sym_if] = ACTIONS(2748),
    [anon_sym_impl] = ACTIONS(2748),
    [anon_sym_let] = ACTIONS(2748),
    [anon_sym_loop] = ACTIONS(2748),
    [anon_sym_match] = ACTIONS(2748),
    [anon_sym_mod] = ACTIONS(2748),
    [anon_sym_pub] = ACTIONS(2748),
    [anon_sym_return] = ACTIONS(2748),
    [anon_sym_raw] = ACTIONS(2748),
    [anon_sym_static] = ACTIONS(2748),
    [anon_sym_struct] = ACTIONS(2748),
    [anon_sym_trait] = ACTIONS(2748),
    [anon_sym_type] = ACTIONS(2748),
    [anon_sym_union] = ACTIONS(2748),
    [anon_sym_unsafe] = ACTIONS(2748),
    [anon_sym_use] = ACTIONS(2748),
    [anon_sym_while] = ACTIONS(2748),
    [anon_sym_extern] = ACTIONS(2748),
    [anon_sym_safe] = ACTIONS(2748),
    [anon_sym_yield] = ACTIONS(2748),
    [anon_sym_move] = ACTIONS(2748),
    [anon_sym_try] = ACTIONS(2748),
    [sym_integer_literal] = ACTIONS(2746),
    [aux_sym_string_literal_token1] = ACTIONS(2746),
    [sym_char_literal] = ACTIONS(2746),
    [anon_sym_true] = ACTIONS(2748),
    [anon_sym_false] = ACTIONS(2748),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2748),
    [sym_super] = ACTIONS(2748),
    [sym_crate] = ACTIONS(2748),
    [aux_sym_metavariable_token1] = ACTIONS(2746),
    [sym__raw_string_literal_start] = ACTIONS(2746),
    [sym_float_literal] = ACTIONS(2746),
    [sym__outer_block_doc_comment_start] = ACTIONS(2746),
    [sym__outer_line_doc_comment_start] = ACTIONS(2746),
  },
  [STATE(744)] = {
    [sym_line_comment] = STATE(744),
    [sym_block_comment] = STATE(744),
    [ts_builtin_sym_end] = ACTIONS(2750),
    [sym_identifier] = ACTIONS(2752),
    [anon_sym_SEMI] = ACTIONS(2750),
    [anon_sym_macro] = ACTIONS(2752),
    [anon_sym_LPAREN] = ACTIONS(2750),
    [anon_sym_LBRACE] = ACTIONS(2750),
    [anon_sym_RBRACE] = ACTIONS(2750),
    [anon_sym_macro_rules] = ACTIONS(2752),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_LBRACK] = ACTIONS(2750),
    [anon_sym_STAR] = ACTIONS(2750),
    [anon_sym_u8] = ACTIONS(2752),
    [anon_sym_i8] = ACTIONS(2752),
    [anon_sym_u16] = ACTIONS(2752),
    [anon_sym_i16] = ACTIONS(2752),
    [anon_sym_u32] = ACTIONS(2752),
    [anon_sym_i32] = ACTIONS(2752),
    [anon_sym_u64] = ACTIONS(2752),
    [anon_sym_i64] = ACTIONS(2752),
    [anon_sym_u128] = ACTIONS(2752),
    [anon_sym_i128] = ACTIONS(2752),
    [anon_sym_isize] = ACTIONS(2752),
    [anon_sym_usize] = ACTIONS(2752),
    [anon_sym_f32] = ACTIONS(2752),
    [anon_sym_f64] = ACTIONS(2752),
    [anon_sym_bool] = ACTIONS(2752),
    [anon_sym_str] = ACTIONS(2752),
    [anon_sym_char] = ACTIONS(2752),
    [anon_sym_DASH] = ACTIONS(2750),
    [anon_sym_AMP] = ACTIONS(2750),
    [anon_sym_PIPE] = ACTIONS(2750),
    [anon_sym_LT] = ACTIONS(2750),
    [anon_sym_DOT_DOT] = ACTIONS(2750),
    [anon_sym_COLON_COLON] = ACTIONS(2750),
    [anon_sym_POUND] = ACTIONS(2750),
    [anon_sym_SQUOTE] = ACTIONS(2752),
    [anon_sym_async] = ACTIONS(2752),
    [anon_sym_auto] = ACTIONS(2752),
    [anon_sym_await] = ACTIONS(2752),
    [anon_sym_become] = ACTIONS(2752),
    [anon_sym_break] = ACTIONS(2752),
    [anon_sym_const] = ACTIONS(2752),
    [anon_sym_continue] = ACTIONS(2752),
    [anon_sym_default] = ACTIONS(2752),
    [anon_sym_enum] = ACTIONS(2752),
    [anon_sym_fn] = ACTIONS(2752),
    [anon_sym_for] = ACTIONS(2752),
    [anon_sym_gen] = ACTIONS(2752),
    [anon_sym_if] = ACTIONS(2752),
    [anon_sym_impl] = ACTIONS(2752),
    [anon_sym_let] = ACTIONS(2752),
    [anon_sym_loop] = ACTIONS(2752),
    [anon_sym_match] = ACTIONS(2752),
    [anon_sym_mod] = ACTIONS(2752),
    [anon_sym_pub] = ACTIONS(2752),
    [anon_sym_return] = ACTIONS(2752),
    [anon_sym_raw] = ACTIONS(2752),
    [anon_sym_static] = ACTIONS(2752),
    [anon_sym_struct] = ACTIONS(2752),
    [anon_sym_trait] = ACTIONS(2752),
    [anon_sym_type] = ACTIONS(2752),
    [anon_sym_union] = ACTIONS(2752),
    [anon_sym_unsafe] = ACTIONS(2752),
    [anon_sym_use] = ACTIONS(2752),
    [anon_sym_while] = ACTIONS(2752),
    [anon_sym_extern] = ACTIONS(2752),
    [anon_sym_safe] = ACTIONS(2752),
    [anon_sym_yield] = ACTIONS(2752),
    [anon_sym_move] = ACTIONS(2752),
    [anon_sym_try] = ACTIONS(2752),
    [sym_integer_literal] = ACTIONS(2750),
    [aux_sym_string_literal_token1] = ACTIONS(2750),
    [sym_char_literal] = ACTIONS(2750),
    [anon_sym_true] = ACTIONS(2752),
    [anon_sym_false] = ACTIONS(2752),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2752),
    [sym_super] = ACTIONS(2752),
    [sym_crate] = ACTIONS(2752),
    [aux_sym_metavariable_token1] = ACTIONS(2750),
    [sym__raw_string_literal_start] = ACTIONS(2750),
    [sym_float_literal] = ACTIONS(2750),
    [sym__outer_block_doc_comment_start] = ACTIONS(2750),
    [sym__outer_line_doc_comment_start] = ACTIONS(2750),
  },
  [STATE(745)] = {
    [sym_line_comment] = STATE(745),
    [sym_block_comment] = STATE(745),
    [ts_builtin_sym_end] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2756),
    [anon_sym_SEMI] = ACTIONS(2754),
    [anon_sym_macro] = ACTIONS(2756),
    [anon_sym_LPAREN] = ACTIONS(2754),
    [anon_sym_LBRACE] = ACTIONS(2754),
    [anon_sym_RBRACE] = ACTIONS(2754),
    [anon_sym_macro_rules] = ACTIONS(2756),
    [anon_sym_BANG] = ACTIONS(2754),
    [anon_sym_LBRACK] = ACTIONS(2754),
    [anon_sym_STAR] = ACTIONS(2754),
    [anon_sym_u8] = ACTIONS(2756),
    [anon_sym_i8] = ACTIONS(2756),
    [anon_sym_u16] = ACTIONS(2756),
    [anon_sym_i16] = ACTIONS(2756),
    [anon_sym_u32] = ACTIONS(2756),
    [anon_sym_i32] = ACTIONS(2756),
    [anon_sym_u64] = ACTIONS(2756),
    [anon_sym_i64] = ACTIONS(2756),
    [anon_sym_u128] = ACTIONS(2756),
    [anon_sym_i128] = ACTIONS(2756),
    [anon_sym_isize] = ACTIONS(2756),
    [anon_sym_usize] = ACTIONS(2756),
    [anon_sym_f32] = ACTIONS(2756),
    [anon_sym_f64] = ACTIONS(2756),
    [anon_sym_bool] = ACTIONS(2756),
    [anon_sym_str] = ACTIONS(2756),
    [anon_sym_char] = ACTIONS(2756),
    [anon_sym_DASH] = ACTIONS(2754),
    [anon_sym_AMP] = ACTIONS(2754),
    [anon_sym_PIPE] = ACTIONS(2754),
    [anon_sym_LT] = ACTIONS(2754),
    [anon_sym_DOT_DOT] = ACTIONS(2754),
    [anon_sym_COLON_COLON] = ACTIONS(2754),
    [anon_sym_POUND] = ACTIONS(2754),
    [anon_sym_SQUOTE] = ACTIONS(2756),
    [anon_sym_async] = ACTIONS(2756),
    [anon_sym_auto] = ACTIONS(2756),
    [anon_sym_await] = ACTIONS(2756),
    [anon_sym_become] = ACTIONS(2756),
    [anon_sym_break] = ACTIONS(2756),
    [anon_sym_const] = ACTIONS(2756),
    [anon_sym_continue] = ACTIONS(2756),
    [anon_sym_default] = ACTIONS(2756),
    [anon_sym_enum] = ACTIONS(2756),
    [anon_sym_fn] = ACTIONS(2756),
    [anon_sym_for] = ACTIONS(2756),
    [anon_sym_gen] = ACTIONS(2756),
    [anon_sym_if] = ACTIONS(2756),
    [anon_sym_impl] = ACTIONS(2756),
    [anon_sym_let] = ACTIONS(2756),
    [anon_sym_loop] = ACTIONS(2756),
    [anon_sym_match] = ACTIONS(2756),
    [anon_sym_mod] = ACTIONS(2756),
    [anon_sym_pub] = ACTIONS(2756),
    [anon_sym_return] = ACTIONS(2756),
    [anon_sym_raw] = ACTIONS(2756),
    [anon_sym_static] = ACTIONS(2756),
    [anon_sym_struct] = ACTIONS(2756),
    [anon_sym_trait] = ACTIONS(2756),
    [anon_sym_type] = ACTIONS(2756),
    [anon_sym_union] = ACTIONS(2756),
    [anon_sym_unsafe] = ACTIONS(2756),
    [anon_sym_use] = ACTIONS(2756),
    [anon_sym_while] = ACTIONS(2756),
    [anon_sym_extern] = ACTIONS(2756),
    [anon_sym_safe] = ACTIONS(2756),
    [anon_sym_yield] = ACTIONS(2756),
    [anon_sym_move] = ACTIONS(2756),
    [anon_sym_try] = ACTIONS(2756),
    [sym_integer_literal] = ACTIONS(2754),
    [aux_sym_string_literal_token1] = ACTIONS(2754),
    [sym_char_literal] = ACTIONS(2754),
    [anon_sym_true] = ACTIONS(2756),
    [anon_sym_false] = ACTIONS(2756),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2756),
    [sym_super] = ACTIONS(2756),
    [sym_crate] = ACTIONS(2756),
    [aux_sym_metavariable_token1] = ACTIONS(2754),
    [sym__raw_string_literal_start] = ACTIONS(2754),
    [sym_float_literal] = ACTIONS(2754),
    [sym__outer_block_doc_comment_start] = ACTIONS(2754),
    [sym__outer_line_doc_comment_start] = ACTIONS(2754),
  },
  [STATE(746)] = {
    [sym_line_comment] = STATE(746),
    [sym_block_comment] = STATE(746),
    [ts_builtin_sym_end] = ACTIONS(2758),
    [sym_identifier] = ACTIONS(2760),
    [anon_sym_SEMI] = ACTIONS(2758),
    [anon_sym_macro] = ACTIONS(2760),
    [anon_sym_LPAREN] = ACTIONS(2758),
    [anon_sym_LBRACE] = ACTIONS(2758),
    [anon_sym_RBRACE] = ACTIONS(2758),
    [anon_sym_macro_rules] = ACTIONS(2760),
    [anon_sym_BANG] = ACTIONS(2758),
    [anon_sym_LBRACK] = ACTIONS(2758),
    [anon_sym_STAR] = ACTIONS(2758),
    [anon_sym_u8] = ACTIONS(2760),
    [anon_sym_i8] = ACTIONS(2760),
    [anon_sym_u16] = ACTIONS(2760),
    [anon_sym_i16] = ACTIONS(2760),
    [anon_sym_u32] = ACTIONS(2760),
    [anon_sym_i32] = ACTIONS(2760),
    [anon_sym_u64] = ACTIONS(2760),
    [anon_sym_i64] = ACTIONS(2760),
    [anon_sym_u128] = ACTIONS(2760),
    [anon_sym_i128] = ACTIONS(2760),
    [anon_sym_isize] = ACTIONS(2760),
    [anon_sym_usize] = ACTIONS(2760),
    [anon_sym_f32] = ACTIONS(2760),
    [anon_sym_f64] = ACTIONS(2760),
    [anon_sym_bool] = ACTIONS(2760),
    [anon_sym_str] = ACTIONS(2760),
    [anon_sym_char] = ACTIONS(2760),
    [anon_sym_DASH] = ACTIONS(2758),
    [anon_sym_AMP] = ACTIONS(2758),
    [anon_sym_PIPE] = ACTIONS(2758),
    [anon_sym_LT] = ACTIONS(2758),
    [anon_sym_DOT_DOT] = ACTIONS(2758),
    [anon_sym_COLON_COLON] = ACTIONS(2758),
    [anon_sym_POUND] = ACTIONS(2758),
    [anon_sym_SQUOTE] = ACTIONS(2760),
    [anon_sym_async] = ACTIONS(2760),
    [anon_sym_auto] = ACTIONS(2760),
    [anon_sym_await] = ACTIONS(2760),
    [anon_sym_become] = ACTIONS(2760),
    [anon_sym_break] = ACTIONS(2760),
    [anon_sym_const] = ACTIONS(2760),
    [anon_sym_continue] = ACTIONS(2760),
    [anon_sym_default] = ACTIONS(2760),
    [anon_sym_enum] = ACTIONS(2760),
    [anon_sym_fn] = ACTIONS(2760),
    [anon_sym_for] = ACTIONS(2760),
    [anon_sym_gen] = ACTIONS(2760),
    [anon_sym_if] = ACTIONS(2760),
    [anon_sym_impl] = ACTIONS(2760),
    [anon_sym_let] = ACTIONS(2760),
    [anon_sym_loop] = ACTIONS(2760),
    [anon_sym_match] = ACTIONS(2760),
    [anon_sym_mod] = ACTIONS(2760),
    [anon_sym_pub] = ACTIONS(2760),
    [anon_sym_return] = ACTIONS(2760),
    [anon_sym_raw] = ACTIONS(2760),
    [anon_sym_static] = ACTIONS(2760),
    [anon_sym_struct] = ACTIONS(2760),
    [anon_sym_trait] = ACTIONS(2760),
    [anon_sym_type] = ACTIONS(2760),
    [anon_sym_union] = ACTIONS(2760),
    [anon_sym_unsafe] = ACTIONS(2760),
    [anon_sym_use] = ACTIONS(2760),
    [anon_sym_while] = ACTIONS(2760),
    [anon_sym_extern] = ACTIONS(2760),
    [anon_sym_safe] = ACTIONS(2760),
    [anon_sym_yield] = ACTIONS(2760),
    [anon_sym_move] = ACTIONS(2760),
    [anon_sym_try] = ACTIONS(2760),
    [sym_integer_literal] = ACTIONS(2758),
    [aux_sym_string_literal_token1] = ACTIONS(2758),
    [sym_char_literal] = ACTIONS(2758),
    [anon_sym_true] = ACTIONS(2760),
    [anon_sym_false] = ACTIONS(2760),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2760),
    [sym_super] = ACTIONS(2760),
    [sym_crate] = ACTIONS(2760),
    [aux_sym_metavariable_token1] = ACTIONS(2758),
    [sym__raw_string_literal_start] = ACTIONS(2758),
    [sym_float_literal] = ACTIONS(2758),
    [sym__outer_block_doc_comment_start] = ACTIONS(2758),
    [sym__outer_line_doc_comment_start] = ACTIONS(2758),
  },
  [STATE(747)] = {
    [sym_line_comment] = STATE(747),
    [sym_block_comment] = STATE(747),
    [ts_builtin_sym_end] = ACTIONS(2762),
    [sym_identifier] = ACTIONS(2764),
    [anon_sym_SEMI] = ACTIONS(2762),
    [anon_sym_macro] = ACTIONS(2764),
    [anon_sym_LPAREN] = ACTIONS(2762),
    [anon_sym_LBRACE] = ACTIONS(2762),
    [anon_sym_RBRACE] = ACTIONS(2762),
    [anon_sym_macro_rules] = ACTIONS(2764),
    [anon_sym_BANG] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(2762),
    [anon_sym_STAR] = ACTIONS(2762),
    [anon_sym_u8] = ACTIONS(2764),
    [anon_sym_i8] = ACTIONS(2764),
    [anon_sym_u16] = ACTIONS(2764),
    [anon_sym_i16] = ACTIONS(2764),
    [anon_sym_u32] = ACTIONS(2764),
    [anon_sym_i32] = ACTIONS(2764),
    [anon_sym_u64] = ACTIONS(2764),
    [anon_sym_i64] = ACTIONS(2764),
    [anon_sym_u128] = ACTIONS(2764),
    [anon_sym_i128] = ACTIONS(2764),
    [anon_sym_isize] = ACTIONS(2764),
    [anon_sym_usize] = ACTIONS(2764),
    [anon_sym_f32] = ACTIONS(2764),
    [anon_sym_f64] = ACTIONS(2764),
    [anon_sym_bool] = ACTIONS(2764),
    [anon_sym_str] = ACTIONS(2764),
    [anon_sym_char] = ACTIONS(2764),
    [anon_sym_DASH] = ACTIONS(2762),
    [anon_sym_AMP] = ACTIONS(2762),
    [anon_sym_PIPE] = ACTIONS(2762),
    [anon_sym_LT] = ACTIONS(2762),
    [anon_sym_DOT_DOT] = ACTIONS(2762),
    [anon_sym_COLON_COLON] = ACTIONS(2762),
    [anon_sym_POUND] = ACTIONS(2762),
    [anon_sym_SQUOTE] = ACTIONS(2764),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_auto] = ACTIONS(2764),
    [anon_sym_await] = ACTIONS(2764),
    [anon_sym_become] = ACTIONS(2764),
    [anon_sym_break] = ACTIONS(2764),
    [anon_sym_const] = ACTIONS(2764),
    [anon_sym_continue] = ACTIONS(2764),
    [anon_sym_default] = ACTIONS(2764),
    [anon_sym_enum] = ACTIONS(2764),
    [anon_sym_fn] = ACTIONS(2764),
    [anon_sym_for] = ACTIONS(2764),
    [anon_sym_gen] = ACTIONS(2764),
    [anon_sym_if] = ACTIONS(2764),
    [anon_sym_impl] = ACTIONS(2764),
    [anon_sym_let] = ACTIONS(2764),
    [anon_sym_loop] = ACTIONS(2764),
    [anon_sym_match] = ACTIONS(2764),
    [anon_sym_mod] = ACTIONS(2764),
    [anon_sym_pub] = ACTIONS(2764),
    [anon_sym_return] = ACTIONS(2764),
    [anon_sym_raw] = ACTIONS(2764),
    [anon_sym_static] = ACTIONS(2764),
    [anon_sym_struct] = ACTIONS(2764),
    [anon_sym_trait] = ACTIONS(2764),
    [anon_sym_type] = ACTIONS(2764),
    [anon_sym_union] = ACTIONS(2764),
    [anon_sym_unsafe] = ACTIONS(2764),
    [anon_sym_use] = ACTIONS(2764),
    [anon_sym_while] = ACTIONS(2764),
    [anon_sym_extern] = ACTIONS(2764),
    [anon_sym_safe] = ACTIONS(2764),
    [anon_sym_yield] = ACTIONS(2764),
    [anon_sym_move] = ACTIONS(2764),
    [anon_sym_try] = ACTIONS(2764),
    [sym_integer_literal] = ACTIONS(2762),
    [aux_sym_string_literal_token1] = ACTIONS(2762),
    [sym_char_literal] = ACTIONS(2762),
    [anon_sym_true] = ACTIONS(2764),
    [anon_sym_false] = ACTIONS(2764),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2764),
    [sym_super] = ACTIONS(2764),
    [sym_crate] = ACTIONS(2764),
    [aux_sym_metavariable_token1] = ACTIONS(2762),
    [sym__raw_string_literal_start] = ACTIONS(2762),
    [sym_float_literal] = ACTIONS(2762),
    [sym__outer_block_doc_comment_start] = ACTIONS(2762),
    [sym__outer_line_doc_comment_start] = ACTIONS(2762),
  },
  [STATE(748)] = {
    [sym_line_comment] = STATE(748),
    [sym_block_comment] = STATE(748),
    [ts_builtin_sym_end] = ACTIONS(2766),
    [sym_identifier] = ACTIONS(2768),
    [anon_sym_SEMI] = ACTIONS(2766),
    [anon_sym_macro] = ACTIONS(2768),
    [anon_sym_LPAREN] = ACTIONS(2766),
    [anon_sym_LBRACE] = ACTIONS(2766),
    [anon_sym_RBRACE] = ACTIONS(2766),
    [anon_sym_macro_rules] = ACTIONS(2768),
    [anon_sym_BANG] = ACTIONS(2766),
    [anon_sym_LBRACK] = ACTIONS(2766),
    [anon_sym_STAR] = ACTIONS(2766),
    [anon_sym_u8] = ACTIONS(2768),
    [anon_sym_i8] = ACTIONS(2768),
    [anon_sym_u16] = ACTIONS(2768),
    [anon_sym_i16] = ACTIONS(2768),
    [anon_sym_u32] = ACTIONS(2768),
    [anon_sym_i32] = ACTIONS(2768),
    [anon_sym_u64] = ACTIONS(2768),
    [anon_sym_i64] = ACTIONS(2768),
    [anon_sym_u128] = ACTIONS(2768),
    [anon_sym_i128] = ACTIONS(2768),
    [anon_sym_isize] = ACTIONS(2768),
    [anon_sym_usize] = ACTIONS(2768),
    [anon_sym_f32] = ACTIONS(2768),
    [anon_sym_f64] = ACTIONS(2768),
    [anon_sym_bool] = ACTIONS(2768),
    [anon_sym_str] = ACTIONS(2768),
    [anon_sym_char] = ACTIONS(2768),
    [anon_sym_DASH] = ACTIONS(2766),
    [anon_sym_AMP] = ACTIONS(2766),
    [anon_sym_PIPE] = ACTIONS(2766),
    [anon_sym_LT] = ACTIONS(2766),
    [anon_sym_DOT_DOT] = ACTIONS(2766),
    [anon_sym_COLON_COLON] = ACTIONS(2766),
    [anon_sym_POUND] = ACTIONS(2766),
    [anon_sym_SQUOTE] = ACTIONS(2768),
    [anon_sym_async] = ACTIONS(2768),
    [anon_sym_auto] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2768),
    [anon_sym_become] = ACTIONS(2768),
    [anon_sym_break] = ACTIONS(2768),
    [anon_sym_const] = ACTIONS(2768),
    [anon_sym_continue] = ACTIONS(2768),
    [anon_sym_default] = ACTIONS(2768),
    [anon_sym_enum] = ACTIONS(2768),
    [anon_sym_fn] = ACTIONS(2768),
    [anon_sym_for] = ACTIONS(2768),
    [anon_sym_gen] = ACTIONS(2768),
    [anon_sym_if] = ACTIONS(2768),
    [anon_sym_impl] = ACTIONS(2768),
    [anon_sym_let] = ACTIONS(2768),
    [anon_sym_loop] = ACTIONS(2768),
    [anon_sym_match] = ACTIONS(2768),
    [anon_sym_mod] = ACTIONS(2768),
    [anon_sym_pub] = ACTIONS(2768),
    [anon_sym_return] = ACTIONS(2768),
    [anon_sym_raw] = ACTIONS(2768),
    [anon_sym_static] = ACTIONS(2768),
    [anon_sym_struct] = ACTIONS(2768),
    [anon_sym_trait] = ACTIONS(2768),
    [anon_sym_type] = ACTIONS(2768),
    [anon_sym_union] = ACTIONS(2768),
    [anon_sym_unsafe] = ACTIONS(2768),
    [anon_sym_use] = ACTIONS(2768),
    [anon_sym_while] = ACTIONS(2768),
    [anon_sym_extern] = ACTIONS(2768),
    [anon_sym_safe] = ACTIONS(2768),
    [anon_sym_yield] = ACTIONS(2768),
    [anon_sym_move] = ACTIONS(2768),
    [anon_sym_try] = ACTIONS(2768),
    [sym_integer_literal] = ACTIONS(2766),
    [aux_sym_string_literal_token1] = ACTIONS(2766),
    [sym_char_literal] = ACTIONS(2766),
    [anon_sym_true] = ACTIONS(2768),
    [anon_sym_false] = ACTIONS(2768),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2768),
    [sym_super] = ACTIONS(2768),
    [sym_crate] = ACTIONS(2768),
    [aux_sym_metavariable_token1] = ACTIONS(2766),
    [sym__raw_string_literal_start] = ACTIONS(2766),
    [sym_float_literal] = ACTIONS(2766),
    [sym__outer_block_doc_comment_start] = ACTIONS(2766),
    [sym__outer_line_doc_comment_start] = ACTIONS(2766),
  },
  [STATE(749)] = {
    [sym_line_comment] = STATE(749),
    [sym_block_comment] = STATE(749),
    [ts_builtin_sym_end] = ACTIONS(2770),
    [sym_identifier] = ACTIONS(2772),
    [anon_sym_SEMI] = ACTIONS(2770),
    [anon_sym_macro] = ACTIONS(2772),
    [anon_sym_LPAREN] = ACTIONS(2770),
    [anon_sym_LBRACE] = ACTIONS(2770),
    [anon_sym_RBRACE] = ACTIONS(2770),
    [anon_sym_macro_rules] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2770),
    [anon_sym_LBRACK] = ACTIONS(2770),
    [anon_sym_STAR] = ACTIONS(2770),
    [anon_sym_u8] = ACTIONS(2772),
    [anon_sym_i8] = ACTIONS(2772),
    [anon_sym_u16] = ACTIONS(2772),
    [anon_sym_i16] = ACTIONS(2772),
    [anon_sym_u32] = ACTIONS(2772),
    [anon_sym_i32] = ACTIONS(2772),
    [anon_sym_u64] = ACTIONS(2772),
    [anon_sym_i64] = ACTIONS(2772),
    [anon_sym_u128] = ACTIONS(2772),
    [anon_sym_i128] = ACTIONS(2772),
    [anon_sym_isize] = ACTIONS(2772),
    [anon_sym_usize] = ACTIONS(2772),
    [anon_sym_f32] = ACTIONS(2772),
    [anon_sym_f64] = ACTIONS(2772),
    [anon_sym_bool] = ACTIONS(2772),
    [anon_sym_str] = ACTIONS(2772),
    [anon_sym_char] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2770),
    [anon_sym_AMP] = ACTIONS(2770),
    [anon_sym_PIPE] = ACTIONS(2770),
    [anon_sym_LT] = ACTIONS(2770),
    [anon_sym_DOT_DOT] = ACTIONS(2770),
    [anon_sym_COLON_COLON] = ACTIONS(2770),
    [anon_sym_POUND] = ACTIONS(2770),
    [anon_sym_SQUOTE] = ACTIONS(2772),
    [anon_sym_async] = ACTIONS(2772),
    [anon_sym_auto] = ACTIONS(2772),
    [anon_sym_await] = ACTIONS(2772),
    [anon_sym_become] = ACTIONS(2772),
    [anon_sym_break] = ACTIONS(2772),
    [anon_sym_const] = ACTIONS(2772),
    [anon_sym_continue] = ACTIONS(2772),
    [anon_sym_default] = ACTIONS(2772),
    [anon_sym_enum] = ACTIONS(2772),
    [anon_sym_fn] = ACTIONS(2772),
    [anon_sym_for] = ACTIONS(2772),
    [anon_sym_gen] = ACTIONS(2772),
    [anon_sym_if] = ACTIONS(2772),
    [anon_sym_impl] = ACTIONS(2772),
    [anon_sym_let] = ACTIONS(2772),
    [anon_sym_loop] = ACTIONS(2772),
    [anon_sym_match] = ACTIONS(2772),
    [anon_sym_mod] = ACTIONS(2772),
    [anon_sym_pub] = ACTIONS(2772),
    [anon_sym_return] = ACTIONS(2772),
    [anon_sym_raw] = ACTIONS(2772),
    [anon_sym_static] = ACTIONS(2772),
    [anon_sym_struct] = ACTIONS(2772),
    [anon_sym_trait] = ACTIONS(2772),
    [anon_sym_type] = ACTIONS(2772),
    [anon_sym_union] = ACTIONS(2772),
    [anon_sym_unsafe] = ACTIONS(2772),
    [anon_sym_use] = ACTIONS(2772),
    [anon_sym_while] = ACTIONS(2772),
    [anon_sym_extern] = ACTIONS(2772),
    [anon_sym_safe] = ACTIONS(2772),
    [anon_sym_yield] = ACTIONS(2772),
    [anon_sym_move] = ACTIONS(2772),
    [anon_sym_try] = ACTIONS(2772),
    [sym_integer_literal] = ACTIONS(2770),
    [aux_sym_string_literal_token1] = ACTIONS(2770),
    [sym_char_literal] = ACTIONS(2770),
    [anon_sym_true] = ACTIONS(2772),
    [anon_sym_false] = ACTIONS(2772),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2772),
    [sym_super] = ACTIONS(2772),
    [sym_crate] = ACTIONS(2772),
    [aux_sym_metavariable_token1] = ACTIONS(2770),
    [sym__raw_string_literal_start] = ACTIONS(2770),
    [sym_float_literal] = ACTIONS(2770),
    [sym__outer_block_doc_comment_start] = ACTIONS(2770),
    [sym__outer_line_doc_comment_start] = ACTIONS(2770),
  },
  [STATE(750)] = {
    [sym_line_comment] = STATE(750),
    [sym_block_comment] = STATE(750),
    [ts_builtin_sym_end] = ACTIONS(2774),
    [sym_identifier] = ACTIONS(2776),
    [anon_sym_SEMI] = ACTIONS(2774),
    [anon_sym_macro] = ACTIONS(2776),
    [anon_sym_LPAREN] = ACTIONS(2774),
    [anon_sym_LBRACE] = ACTIONS(2774),
    [anon_sym_RBRACE] = ACTIONS(2774),
    [anon_sym_macro_rules] = ACTIONS(2776),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_LBRACK] = ACTIONS(2774),
    [anon_sym_STAR] = ACTIONS(2774),
    [anon_sym_u8] = ACTIONS(2776),
    [anon_sym_i8] = ACTIONS(2776),
    [anon_sym_u16] = ACTIONS(2776),
    [anon_sym_i16] = ACTIONS(2776),
    [anon_sym_u32] = ACTIONS(2776),
    [anon_sym_i32] = ACTIONS(2776),
    [anon_sym_u64] = ACTIONS(2776),
    [anon_sym_i64] = ACTIONS(2776),
    [anon_sym_u128] = ACTIONS(2776),
    [anon_sym_i128] = ACTIONS(2776),
    [anon_sym_isize] = ACTIONS(2776),
    [anon_sym_usize] = ACTIONS(2776),
    [anon_sym_f32] = ACTIONS(2776),
    [anon_sym_f64] = ACTIONS(2776),
    [anon_sym_bool] = ACTIONS(2776),
    [anon_sym_str] = ACTIONS(2776),
    [anon_sym_char] = ACTIONS(2776),
    [anon_sym_DASH] = ACTIONS(2774),
    [anon_sym_AMP] = ACTIONS(2774),
    [anon_sym_PIPE] = ACTIONS(2774),
    [anon_sym_LT] = ACTIONS(2774),
    [anon_sym_DOT_DOT] = ACTIONS(2774),
    [anon_sym_COLON_COLON] = ACTIONS(2774),
    [anon_sym_POUND] = ACTIONS(2774),
    [anon_sym_SQUOTE] = ACTIONS(2776),
    [anon_sym_async] = ACTIONS(2776),
    [anon_sym_auto] = ACTIONS(2776),
    [anon_sym_await] = ACTIONS(2776),
    [anon_sym_become] = ACTIONS(2776),
    [anon_sym_break] = ACTIONS(2776),
    [anon_sym_const] = ACTIONS(2776),
    [anon_sym_continue] = ACTIONS(2776),
    [anon_sym_default] = ACTIONS(2776),
    [anon_sym_enum] = ACTIONS(2776),
    [anon_sym_fn] = ACTIONS(2776),
    [anon_sym_for] = ACTIONS(2776),
    [anon_sym_gen] = ACTIONS(2776),
    [anon_sym_if] = ACTIONS(2776),
    [anon_sym_impl] = ACTIONS(2776),
    [anon_sym_let] = ACTIONS(2776),
    [anon_sym_loop] = ACTIONS(2776),
    [anon_sym_match] = ACTIONS(2776),
    [anon_sym_mod] = ACTIONS(2776),
    [anon_sym_pub] = ACTIONS(2776),
    [anon_sym_return] = ACTIONS(2776),
    [anon_sym_raw] = ACTIONS(2776),
    [anon_sym_static] = ACTIONS(2776),
    [anon_sym_struct] = ACTIONS(2776),
    [anon_sym_trait] = ACTIONS(2776),
    [anon_sym_type] = ACTIONS(2776),
    [anon_sym_union] = ACTIONS(2776),
    [anon_sym_unsafe] = ACTIONS(2776),
    [anon_sym_use] = ACTIONS(2776),
    [anon_sym_while] = ACTIONS(2776),
    [anon_sym_extern] = ACTIONS(2776),
    [anon_sym_safe] = ACTIONS(2776),
    [anon_sym_yield] = ACTIONS(2776),
    [anon_sym_move] = ACTIONS(2776),
    [anon_sym_try] = ACTIONS(2776),
    [sym_integer_literal] = ACTIONS(2774),
    [aux_sym_string_literal_token1] = ACTIONS(2774),
    [sym_char_literal] = ACTIONS(2774),
    [anon_sym_true] = ACTIONS(2776),
    [anon_sym_false] = ACTIONS(2776),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2776),
    [sym_super] = ACTIONS(2776),
    [sym_crate] = ACTIONS(2776),
    [aux_sym_metavariable_token1] = ACTIONS(2774),
    [sym__raw_string_literal_start] = ACTIONS(2774),
    [sym_float_literal] = ACTIONS(2774),
    [sym__outer_block_doc_comment_start] = ACTIONS(2774),
    [sym__outer_line_doc_comment_start] = ACTIONS(2774),
  },
  [STATE(751)] = {
    [sym_line_comment] = STATE(751),
    [sym_block_comment] = STATE(751),
    [ts_builtin_sym_end] = ACTIONS(2778),
    [sym_identifier] = ACTIONS(2780),
    [anon_sym_SEMI] = ACTIONS(2778),
    [anon_sym_macro] = ACTIONS(2780),
    [anon_sym_LPAREN] = ACTIONS(2778),
    [anon_sym_LBRACE] = ACTIONS(2778),
    [anon_sym_RBRACE] = ACTIONS(2778),
    [anon_sym_macro_rules] = ACTIONS(2780),
    [anon_sym_BANG] = ACTIONS(2778),
    [anon_sym_LBRACK] = ACTIONS(2778),
    [anon_sym_STAR] = ACTIONS(2778),
    [anon_sym_u8] = ACTIONS(2780),
    [anon_sym_i8] = ACTIONS(2780),
    [anon_sym_u16] = ACTIONS(2780),
    [anon_sym_i16] = ACTIONS(2780),
    [anon_sym_u32] = ACTIONS(2780),
    [anon_sym_i32] = ACTIONS(2780),
    [anon_sym_u64] = ACTIONS(2780),
    [anon_sym_i64] = ACTIONS(2780),
    [anon_sym_u128] = ACTIONS(2780),
    [anon_sym_i128] = ACTIONS(2780),
    [anon_sym_isize] = ACTIONS(2780),
    [anon_sym_usize] = ACTIONS(2780),
    [anon_sym_f32] = ACTIONS(2780),
    [anon_sym_f64] = ACTIONS(2780),
    [anon_sym_bool] = ACTIONS(2780),
    [anon_sym_str] = ACTIONS(2780),
    [anon_sym_char] = ACTIONS(2780),
    [anon_sym_DASH] = ACTIONS(2778),
    [anon_sym_AMP] = ACTIONS(2778),
    [anon_sym_PIPE] = ACTIONS(2778),
    [anon_sym_LT] = ACTIONS(2778),
    [anon_sym_DOT_DOT] = ACTIONS(2778),
    [anon_sym_COLON_COLON] = ACTIONS(2778),
    [anon_sym_POUND] = ACTIONS(2778),
    [anon_sym_SQUOTE] = ACTIONS(2780),
    [anon_sym_async] = ACTIONS(2780),
    [anon_sym_auto] = ACTIONS(2780),
    [anon_sym_await] = ACTIONS(2780),
    [anon_sym_become] = ACTIONS(2780),
    [anon_sym_break] = ACTIONS(2780),
    [anon_sym_const] = ACTIONS(2780),
    [anon_sym_continue] = ACTIONS(2780),
    [anon_sym_default] = ACTIONS(2780),
    [anon_sym_enum] = ACTIONS(2780),
    [anon_sym_fn] = ACTIONS(2780),
    [anon_sym_for] = ACTIONS(2780),
    [anon_sym_gen] = ACTIONS(2780),
    [anon_sym_if] = ACTIONS(2780),
    [anon_sym_impl] = ACTIONS(2780),
    [anon_sym_let] = ACTIONS(2780),
    [anon_sym_loop] = ACTIONS(2780),
    [anon_sym_match] = ACTIONS(2780),
    [anon_sym_mod] = ACTIONS(2780),
    [anon_sym_pub] = ACTIONS(2780),
    [anon_sym_return] = ACTIONS(2780),
    [anon_sym_raw] = ACTIONS(2780),
    [anon_sym_static] = ACTIONS(2780),
    [anon_sym_struct] = ACTIONS(2780),
    [anon_sym_trait] = ACTIONS(2780),
    [anon_sym_type] = ACTIONS(2780),
    [anon_sym_union] = ACTIONS(2780),
    [anon_sym_unsafe] = ACTIONS(2780),
    [anon_sym_use] = ACTIONS(2780),
    [anon_sym_while] = ACTIONS(2780),
    [anon_sym_extern] = ACTIONS(2780),
    [anon_sym_safe] = ACTIONS(2780),
    [anon_sym_yield] = ACTIONS(2780),
    [anon_sym_move] = ACTIONS(2780),
    [anon_sym_try] = ACTIONS(2780),
    [sym_integer_literal] = ACTIONS(2778),
    [aux_sym_string_literal_token1] = ACTIONS(2778),
    [sym_char_literal] = ACTIONS(2778),
    [anon_sym_true] = ACTIONS(2780),
    [anon_sym_false] = ACTIONS(2780),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2780),
    [sym_super] = ACTIONS(2780),
    [sym_crate] = ACTIONS(2780),
    [aux_sym_metavariable_token1] = ACTIONS(2778),
    [sym__raw_string_literal_start] = ACTIONS(2778),
    [sym_float_literal] = ACTIONS(2778),
    [sym__outer_block_doc_comment_start] = ACTIONS(2778),
    [sym__outer_line_doc_comment_start] = ACTIONS(2778),
  },
  [STATE(752)] = {
    [sym_line_comment] = STATE(752),
    [sym_block_comment] = STATE(752),
    [ts_builtin_sym_end] = ACTIONS(2782),
    [sym_identifier] = ACTIONS(2784),
    [anon_sym_SEMI] = ACTIONS(2782),
    [anon_sym_macro] = ACTIONS(2784),
    [anon_sym_LPAREN] = ACTIONS(2782),
    [anon_sym_LBRACE] = ACTIONS(2782),
    [anon_sym_RBRACE] = ACTIONS(2782),
    [anon_sym_macro_rules] = ACTIONS(2784),
    [anon_sym_BANG] = ACTIONS(2782),
    [anon_sym_LBRACK] = ACTIONS(2782),
    [anon_sym_STAR] = ACTIONS(2782),
    [anon_sym_u8] = ACTIONS(2784),
    [anon_sym_i8] = ACTIONS(2784),
    [anon_sym_u16] = ACTIONS(2784),
    [anon_sym_i16] = ACTIONS(2784),
    [anon_sym_u32] = ACTIONS(2784),
    [anon_sym_i32] = ACTIONS(2784),
    [anon_sym_u64] = ACTIONS(2784),
    [anon_sym_i64] = ACTIONS(2784),
    [anon_sym_u128] = ACTIONS(2784),
    [anon_sym_i128] = ACTIONS(2784),
    [anon_sym_isize] = ACTIONS(2784),
    [anon_sym_usize] = ACTIONS(2784),
    [anon_sym_f32] = ACTIONS(2784),
    [anon_sym_f64] = ACTIONS(2784),
    [anon_sym_bool] = ACTIONS(2784),
    [anon_sym_str] = ACTIONS(2784),
    [anon_sym_char] = ACTIONS(2784),
    [anon_sym_DASH] = ACTIONS(2782),
    [anon_sym_AMP] = ACTIONS(2782),
    [anon_sym_PIPE] = ACTIONS(2782),
    [anon_sym_LT] = ACTIONS(2782),
    [anon_sym_DOT_DOT] = ACTIONS(2782),
    [anon_sym_COLON_COLON] = ACTIONS(2782),
    [anon_sym_POUND] = ACTIONS(2782),
    [anon_sym_SQUOTE] = ACTIONS(2784),
    [anon_sym_async] = ACTIONS(2784),
    [anon_sym_auto] = ACTIONS(2784),
    [anon_sym_await] = ACTIONS(2784),
    [anon_sym_become] = ACTIONS(2784),
    [anon_sym_break] = ACTIONS(2784),
    [anon_sym_const] = ACTIONS(2784),
    [anon_sym_continue] = ACTIONS(2784),
    [anon_sym_default] = ACTIONS(2784),
    [anon_sym_enum] = ACTIONS(2784),
    [anon_sym_fn] = ACTIONS(2784),
    [anon_sym_for] = ACTIONS(2784),
    [anon_sym_gen] = ACTIONS(2784),
    [anon_sym_if] = ACTIONS(2784),
    [anon_sym_impl] = ACTIONS(2784),
    [anon_sym_let] = ACTIONS(2784),
    [anon_sym_loop] = ACTIONS(2784),
    [anon_sym_match] = ACTIONS(2784),
    [anon_sym_mod] = ACTIONS(2784),
    [anon_sym_pub] = ACTIONS(2784),
    [anon_sym_return] = ACTIONS(2784),
    [anon_sym_raw] = ACTIONS(2784),
    [anon_sym_static] = ACTIONS(2784),
    [anon_sym_struct] = ACTIONS(2784),
    [anon_sym_trait] = ACTIONS(2784),
    [anon_sym_type] = ACTIONS(2784),
    [anon_sym_union] = ACTIONS(2784),
    [anon_sym_unsafe] = ACTIONS(2784),
    [anon_sym_use] = ACTIONS(2784),
    [anon_sym_while] = ACTIONS(2784),
    [anon_sym_extern] = ACTIONS(2784),
    [anon_sym_safe] = ACTIONS(2784),
    [anon_sym_yield] = ACTIONS(2784),
    [anon_sym_move] = ACTIONS(2784),
    [anon_sym_try] = ACTIONS(2784),
    [sym_integer_literal] = ACTIONS(2782),
    [aux_sym_string_literal_token1] = ACTIONS(2782),
    [sym_char_literal] = ACTIONS(2782),
    [anon_sym_true] = ACTIONS(2784),
    [anon_sym_false] = ACTIONS(2784),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2784),
    [sym_super] = ACTIONS(2784),
    [sym_crate] = ACTIONS(2784),
    [aux_sym_metavariable_token1] = ACTIONS(2782),
    [sym__raw_string_literal_start] = ACTIONS(2782),
    [sym_float_literal] = ACTIONS(2782),
    [sym__outer_block_doc_comment_start] = ACTIONS(2782),
    [sym__outer_line_doc_comment_start] = ACTIONS(2782),
  },
  [STATE(753)] = {
    [sym_line_comment] = STATE(753),
    [sym_block_comment] = STATE(753),
    [ts_builtin_sym_end] = ACTIONS(2786),
    [sym_identifier] = ACTIONS(2788),
    [anon_sym_SEMI] = ACTIONS(2786),
    [anon_sym_macro] = ACTIONS(2788),
    [anon_sym_LPAREN] = ACTIONS(2786),
    [anon_sym_LBRACE] = ACTIONS(2786),
    [anon_sym_RBRACE] = ACTIONS(2786),
    [anon_sym_macro_rules] = ACTIONS(2788),
    [anon_sym_BANG] = ACTIONS(2786),
    [anon_sym_LBRACK] = ACTIONS(2786),
    [anon_sym_STAR] = ACTIONS(2786),
    [anon_sym_u8] = ACTIONS(2788),
    [anon_sym_i8] = ACTIONS(2788),
    [anon_sym_u16] = ACTIONS(2788),
    [anon_sym_i16] = ACTIONS(2788),
    [anon_sym_u32] = ACTIONS(2788),
    [anon_sym_i32] = ACTIONS(2788),
    [anon_sym_u64] = ACTIONS(2788),
    [anon_sym_i64] = ACTIONS(2788),
    [anon_sym_u128] = ACTIONS(2788),
    [anon_sym_i128] = ACTIONS(2788),
    [anon_sym_isize] = ACTIONS(2788),
    [anon_sym_usize] = ACTIONS(2788),
    [anon_sym_f32] = ACTIONS(2788),
    [anon_sym_f64] = ACTIONS(2788),
    [anon_sym_bool] = ACTIONS(2788),
    [anon_sym_str] = ACTIONS(2788),
    [anon_sym_char] = ACTIONS(2788),
    [anon_sym_DASH] = ACTIONS(2786),
    [anon_sym_AMP] = ACTIONS(2786),
    [anon_sym_PIPE] = ACTIONS(2786),
    [anon_sym_LT] = ACTIONS(2786),
    [anon_sym_DOT_DOT] = ACTIONS(2786),
    [anon_sym_COLON_COLON] = ACTIONS(2786),
    [anon_sym_POUND] = ACTIONS(2786),
    [anon_sym_SQUOTE] = ACTIONS(2788),
    [anon_sym_async] = ACTIONS(2788),
    [anon_sym_auto] = ACTIONS(2788),
    [anon_sym_await] = ACTIONS(2788),
    [anon_sym_become] = ACTIONS(2788),
    [anon_sym_break] = ACTIONS(2788),
    [anon_sym_const] = ACTIONS(2788),
    [anon_sym_continue] = ACTIONS(2788),
    [anon_sym_default] = ACTIONS(2788),
    [anon_sym_enum] = ACTIONS(2788),
    [anon_sym_fn] = ACTIONS(2788),
    [anon_sym_for] = ACTIONS(2788),
    [anon_sym_gen] = ACTIONS(2788),
    [anon_sym_if] = ACTIONS(2788),
    [anon_sym_impl] = ACTIONS(2788),
    [anon_sym_let] = ACTIONS(2788),
    [anon_sym_loop] = ACTIONS(2788),
    [anon_sym_match] = ACTIONS(2788),
    [anon_sym_mod] = ACTIONS(2788),
    [anon_sym_pub] = ACTIONS(2788),
    [anon_sym_return] = ACTIONS(2788),
    [anon_sym_raw] = ACTIONS(2788),
    [anon_sym_static] = ACTIONS(2788),
    [anon_sym_struct] = ACTIONS(2788),
    [anon_sym_trait] = ACTIONS(2788),
    [anon_sym_type] = ACTIONS(2788),
    [anon_sym_union] = ACTIONS(2788),
    [anon_sym_unsafe] = ACTIONS(2788),
    [anon_sym_use] = ACTIONS(2788),
    [anon_sym_while] = ACTIONS(2788),
    [anon_sym_extern] = ACTIONS(2788),
    [anon_sym_safe] = ACTIONS(2788),
    [anon_sym_yield] = ACTIONS(2788),
    [anon_sym_move] = ACTIONS(2788),
    [anon_sym_try] = ACTIONS(2788),
    [sym_integer_literal] = ACTIONS(2786),
    [aux_sym_string_literal_token1] = ACTIONS(2786),
    [sym_char_literal] = ACTIONS(2786),
    [anon_sym_true] = ACTIONS(2788),
    [anon_sym_false] = ACTIONS(2788),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2788),
    [sym_super] = ACTIONS(2788),
    [sym_crate] = ACTIONS(2788),
    [aux_sym_metavariable_token1] = ACTIONS(2786),
    [sym__raw_string_literal_start] = ACTIONS(2786),
    [sym_float_literal] = ACTIONS(2786),
    [sym__outer_block_doc_comment_start] = ACTIONS(2786),
    [sym__outer_line_doc_comment_start] = ACTIONS(2786),
  },
  [STATE(754)] = {
    [sym_line_comment] = STATE(754),
    [sym_block_comment] = STATE(754),
    [ts_builtin_sym_end] = ACTIONS(2790),
    [sym_identifier] = ACTIONS(2792),
    [anon_sym_SEMI] = ACTIONS(2790),
    [anon_sym_macro] = ACTIONS(2792),
    [anon_sym_LPAREN] = ACTIONS(2790),
    [anon_sym_LBRACE] = ACTIONS(2790),
    [anon_sym_RBRACE] = ACTIONS(2790),
    [anon_sym_macro_rules] = ACTIONS(2792),
    [anon_sym_BANG] = ACTIONS(2790),
    [anon_sym_LBRACK] = ACTIONS(2790),
    [anon_sym_STAR] = ACTIONS(2790),
    [anon_sym_u8] = ACTIONS(2792),
    [anon_sym_i8] = ACTIONS(2792),
    [anon_sym_u16] = ACTIONS(2792),
    [anon_sym_i16] = ACTIONS(2792),
    [anon_sym_u32] = ACTIONS(2792),
    [anon_sym_i32] = ACTIONS(2792),
    [anon_sym_u64] = ACTIONS(2792),
    [anon_sym_i64] = ACTIONS(2792),
    [anon_sym_u128] = ACTIONS(2792),
    [anon_sym_i128] = ACTIONS(2792),
    [anon_sym_isize] = ACTIONS(2792),
    [anon_sym_usize] = ACTIONS(2792),
    [anon_sym_f32] = ACTIONS(2792),
    [anon_sym_f64] = ACTIONS(2792),
    [anon_sym_bool] = ACTIONS(2792),
    [anon_sym_str] = ACTIONS(2792),
    [anon_sym_char] = ACTIONS(2792),
    [anon_sym_DASH] = ACTIONS(2790),
    [anon_sym_AMP] = ACTIONS(2790),
    [anon_sym_PIPE] = ACTIONS(2790),
    [anon_sym_LT] = ACTIONS(2790),
    [anon_sym_DOT_DOT] = ACTIONS(2790),
    [anon_sym_COLON_COLON] = ACTIONS(2790),
    [anon_sym_POUND] = ACTIONS(2790),
    [anon_sym_SQUOTE] = ACTIONS(2792),
    [anon_sym_async] = ACTIONS(2792),
    [anon_sym_auto] = ACTIONS(2792),
    [anon_sym_await] = ACTIONS(2792),
    [anon_sym_become] = ACTIONS(2792),
    [anon_sym_break] = ACTIONS(2792),
    [anon_sym_const] = ACTIONS(2792),
    [anon_sym_continue] = ACTIONS(2792),
    [anon_sym_default] = ACTIONS(2792),
    [anon_sym_enum] = ACTIONS(2792),
    [anon_sym_fn] = ACTIONS(2792),
    [anon_sym_for] = ACTIONS(2792),
    [anon_sym_gen] = ACTIONS(2792),
    [anon_sym_if] = ACTIONS(2792),
    [anon_sym_impl] = ACTIONS(2792),
    [anon_sym_let] = ACTIONS(2792),
    [anon_sym_loop] = ACTIONS(2792),
    [anon_sym_match] = ACTIONS(2792),
    [anon_sym_mod] = ACTIONS(2792),
    [anon_sym_pub] = ACTIONS(2792),
    [anon_sym_return] = ACTIONS(2792),
    [anon_sym_raw] = ACTIONS(2792),
    [anon_sym_static] = ACTIONS(2792),
    [anon_sym_struct] = ACTIONS(2792),
    [anon_sym_trait] = ACTIONS(2792),
    [anon_sym_type] = ACTIONS(2792),
    [anon_sym_union] = ACTIONS(2792),
    [anon_sym_unsafe] = ACTIONS(2792),
    [anon_sym_use] = ACTIONS(2792),
    [anon_sym_while] = ACTIONS(2792),
    [anon_sym_extern] = ACTIONS(2792),
    [anon_sym_safe] = ACTIONS(2792),
    [anon_sym_yield] = ACTIONS(2792),
    [anon_sym_move] = ACTIONS(2792),
    [anon_sym_try] = ACTIONS(2792),
    [sym_integer_literal] = ACTIONS(2790),
    [aux_sym_string_literal_token1] = ACTIONS(2790),
    [sym_char_literal] = ACTIONS(2790),
    [anon_sym_true] = ACTIONS(2792),
    [anon_sym_false] = ACTIONS(2792),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2792),
    [sym_super] = ACTIONS(2792),
    [sym_crate] = ACTIONS(2792),
    [aux_sym_metavariable_token1] = ACTIONS(2790),
    [sym__raw_string_literal_start] = ACTIONS(2790),
    [sym_float_literal] = ACTIONS(2790),
    [sym__outer_block_doc_comment_start] = ACTIONS(2790),
    [sym__outer_line_doc_comment_start] = ACTIONS(2790),
  },
  [STATE(755)] = {
    [sym_line_comment] = STATE(755),
    [sym_block_comment] = STATE(755),
    [ts_builtin_sym_end] = ACTIONS(2794),
    [sym_identifier] = ACTIONS(2796),
    [anon_sym_SEMI] = ACTIONS(2794),
    [anon_sym_macro] = ACTIONS(2796),
    [anon_sym_LPAREN] = ACTIONS(2794),
    [anon_sym_LBRACE] = ACTIONS(2794),
    [anon_sym_RBRACE] = ACTIONS(2794),
    [anon_sym_macro_rules] = ACTIONS(2796),
    [anon_sym_BANG] = ACTIONS(2794),
    [anon_sym_LBRACK] = ACTIONS(2794),
    [anon_sym_STAR] = ACTIONS(2794),
    [anon_sym_u8] = ACTIONS(2796),
    [anon_sym_i8] = ACTIONS(2796),
    [anon_sym_u16] = ACTIONS(2796),
    [anon_sym_i16] = ACTIONS(2796),
    [anon_sym_u32] = ACTIONS(2796),
    [anon_sym_i32] = ACTIONS(2796),
    [anon_sym_u64] = ACTIONS(2796),
    [anon_sym_i64] = ACTIONS(2796),
    [anon_sym_u128] = ACTIONS(2796),
    [anon_sym_i128] = ACTIONS(2796),
    [anon_sym_isize] = ACTIONS(2796),
    [anon_sym_usize] = ACTIONS(2796),
    [anon_sym_f32] = ACTIONS(2796),
    [anon_sym_f64] = ACTIONS(2796),
    [anon_sym_bool] = ACTIONS(2796),
    [anon_sym_str] = ACTIONS(2796),
    [anon_sym_char] = ACTIONS(2796),
    [anon_sym_DASH] = ACTIONS(2794),
    [anon_sym_AMP] = ACTIONS(2794),
    [anon_sym_PIPE] = ACTIONS(2794),
    [anon_sym_LT] = ACTIONS(2794),
    [anon_sym_DOT_DOT] = ACTIONS(2794),
    [anon_sym_COLON_COLON] = ACTIONS(2794),
    [anon_sym_POUND] = ACTIONS(2794),
    [anon_sym_SQUOTE] = ACTIONS(2796),
    [anon_sym_async] = ACTIONS(2796),
    [anon_sym_auto] = ACTIONS(2796),
    [anon_sym_await] = ACTIONS(2796),
    [anon_sym_become] = ACTIONS(2796),
    [anon_sym_break] = ACTIONS(2796),
    [anon_sym_const] = ACTIONS(2796),
    [anon_sym_continue] = ACTIONS(2796),
    [anon_sym_default] = ACTIONS(2796),
    [anon_sym_enum] = ACTIONS(2796),
    [anon_sym_fn] = ACTIONS(2796),
    [anon_sym_for] = ACTIONS(2796),
    [anon_sym_gen] = ACTIONS(2796),
    [anon_sym_if] = ACTIONS(2796),
    [anon_sym_impl] = ACTIONS(2796),
    [anon_sym_let] = ACTIONS(2796),
    [anon_sym_loop] = ACTIONS(2796),
    [anon_sym_match] = ACTIONS(2796),
    [anon_sym_mod] = ACTIONS(2796),
    [anon_sym_pub] = ACTIONS(2796),
    [anon_sym_return] = ACTIONS(2796),
    [anon_sym_raw] = ACTIONS(2796),
    [anon_sym_static] = ACTIONS(2796),
    [anon_sym_struct] = ACTIONS(2796),
    [anon_sym_trait] = ACTIONS(2796),
    [anon_sym_type] = ACTIONS(2796),
    [anon_sym_union] = ACTIONS(2796),
    [anon_sym_unsafe] = ACTIONS(2796),
    [anon_sym_use] = ACTIONS(2796),
    [anon_sym_while] = ACTIONS(2796),
    [anon_sym_extern] = ACTIONS(2796),
    [anon_sym_safe] = ACTIONS(2796),
    [anon_sym_yield] = ACTIONS(2796),
    [anon_sym_move] = ACTIONS(2796),
    [anon_sym_try] = ACTIONS(2796),
    [sym_integer_literal] = ACTIONS(2794),
    [aux_sym_string_literal_token1] = ACTIONS(2794),
    [sym_char_literal] = ACTIONS(2794),
    [anon_sym_true] = ACTIONS(2796),
    [anon_sym_false] = ACTIONS(2796),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2796),
    [sym_super] = ACTIONS(2796),
    [sym_crate] = ACTIONS(2796),
    [aux_sym_metavariable_token1] = ACTIONS(2794),
    [sym__raw_string_literal_start] = ACTIONS(2794),
    [sym_float_literal] = ACTIONS(2794),
    [sym__outer_block_doc_comment_start] = ACTIONS(2794),
    [sym__outer_line_doc_comment_start] = ACTIONS(2794),
  },
  [STATE(756)] = {
    [sym_line_comment] = STATE(756),
    [sym_block_comment] = STATE(756),
    [ts_builtin_sym_end] = ACTIONS(2798),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_SEMI] = ACTIONS(2798),
    [anon_sym_macro] = ACTIONS(2800),
    [anon_sym_LPAREN] = ACTIONS(2798),
    [anon_sym_LBRACE] = ACTIONS(2798),
    [anon_sym_RBRACE] = ACTIONS(2798),
    [anon_sym_macro_rules] = ACTIONS(2800),
    [anon_sym_BANG] = ACTIONS(2798),
    [anon_sym_LBRACK] = ACTIONS(2798),
    [anon_sym_STAR] = ACTIONS(2798),
    [anon_sym_u8] = ACTIONS(2800),
    [anon_sym_i8] = ACTIONS(2800),
    [anon_sym_u16] = ACTIONS(2800),
    [anon_sym_i16] = ACTIONS(2800),
    [anon_sym_u32] = ACTIONS(2800),
    [anon_sym_i32] = ACTIONS(2800),
    [anon_sym_u64] = ACTIONS(2800),
    [anon_sym_i64] = ACTIONS(2800),
    [anon_sym_u128] = ACTIONS(2800),
    [anon_sym_i128] = ACTIONS(2800),
    [anon_sym_isize] = ACTIONS(2800),
    [anon_sym_usize] = ACTIONS(2800),
    [anon_sym_f32] = ACTIONS(2800),
    [anon_sym_f64] = ACTIONS(2800),
    [anon_sym_bool] = ACTIONS(2800),
    [anon_sym_str] = ACTIONS(2800),
    [anon_sym_char] = ACTIONS(2800),
    [anon_sym_DASH] = ACTIONS(2798),
    [anon_sym_AMP] = ACTIONS(2798),
    [anon_sym_PIPE] = ACTIONS(2798),
    [anon_sym_LT] = ACTIONS(2798),
    [anon_sym_DOT_DOT] = ACTIONS(2798),
    [anon_sym_COLON_COLON] = ACTIONS(2798),
    [anon_sym_POUND] = ACTIONS(2798),
    [anon_sym_SQUOTE] = ACTIONS(2800),
    [anon_sym_async] = ACTIONS(2800),
    [anon_sym_auto] = ACTIONS(2800),
    [anon_sym_await] = ACTIONS(2800),
    [anon_sym_become] = ACTIONS(2800),
    [anon_sym_break] = ACTIONS(2800),
    [anon_sym_const] = ACTIONS(2800),
    [anon_sym_continue] = ACTIONS(2800),
    [anon_sym_default] = ACTIONS(2800),
    [anon_sym_enum] = ACTIONS(2800),
    [anon_sym_fn] = ACTIONS(2800),
    [anon_sym_for] = ACTIONS(2800),
    [anon_sym_gen] = ACTIONS(2800),
    [anon_sym_if] = ACTIONS(2800),
    [anon_sym_impl] = ACTIONS(2800),
    [anon_sym_let] = ACTIONS(2800),
    [anon_sym_loop] = ACTIONS(2800),
    [anon_sym_match] = ACTIONS(2800),
    [anon_sym_mod] = ACTIONS(2800),
    [anon_sym_pub] = ACTIONS(2800),
    [anon_sym_return] = ACTIONS(2800),
    [anon_sym_raw] = ACTIONS(2800),
    [anon_sym_static] = ACTIONS(2800),
    [anon_sym_struct] = ACTIONS(2800),
    [anon_sym_trait] = ACTIONS(2800),
    [anon_sym_type] = ACTIONS(2800),
    [anon_sym_union] = ACTIONS(2800),
    [anon_sym_unsafe] = ACTIONS(2800),
    [anon_sym_use] = ACTIONS(2800),
    [anon_sym_while] = ACTIONS(2800),
    [anon_sym_extern] = ACTIONS(2800),
    [anon_sym_safe] = ACTIONS(2800),
    [anon_sym_yield] = ACTIONS(2800),
    [anon_sym_move] = ACTIONS(2800),
    [anon_sym_try] = ACTIONS(2800),
    [sym_integer_literal] = ACTIONS(2798),
    [aux_sym_string_literal_token1] = ACTIONS(2798),
    [sym_char_literal] = ACTIONS(2798),
    [anon_sym_true] = ACTIONS(2800),
    [anon_sym_false] = ACTIONS(2800),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2800),
    [sym_super] = ACTIONS(2800),
    [sym_crate] = ACTIONS(2800),
    [aux_sym_metavariable_token1] = ACTIONS(2798),
    [sym__raw_string_literal_start] = ACTIONS(2798),
    [sym_float_literal] = ACTIONS(2798),
    [sym__outer_block_doc_comment_start] = ACTIONS(2798),
    [sym__outer_line_doc_comment_start] = ACTIONS(2798),
  },
  [STATE(757)] = {
    [sym_line_comment] = STATE(757),
    [sym_block_comment] = STATE(757),
    [ts_builtin_sym_end] = ACTIONS(2802),
    [sym_identifier] = ACTIONS(2804),
    [anon_sym_SEMI] = ACTIONS(2802),
    [anon_sym_macro] = ACTIONS(2804),
    [anon_sym_LPAREN] = ACTIONS(2802),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_RBRACE] = ACTIONS(2802),
    [anon_sym_macro_rules] = ACTIONS(2804),
    [anon_sym_BANG] = ACTIONS(2802),
    [anon_sym_LBRACK] = ACTIONS(2802),
    [anon_sym_STAR] = ACTIONS(2802),
    [anon_sym_u8] = ACTIONS(2804),
    [anon_sym_i8] = ACTIONS(2804),
    [anon_sym_u16] = ACTIONS(2804),
    [anon_sym_i16] = ACTIONS(2804),
    [anon_sym_u32] = ACTIONS(2804),
    [anon_sym_i32] = ACTIONS(2804),
    [anon_sym_u64] = ACTIONS(2804),
    [anon_sym_i64] = ACTIONS(2804),
    [anon_sym_u128] = ACTIONS(2804),
    [anon_sym_i128] = ACTIONS(2804),
    [anon_sym_isize] = ACTIONS(2804),
    [anon_sym_usize] = ACTIONS(2804),
    [anon_sym_f32] = ACTIONS(2804),
    [anon_sym_f64] = ACTIONS(2804),
    [anon_sym_bool] = ACTIONS(2804),
    [anon_sym_str] = ACTIONS(2804),
    [anon_sym_char] = ACTIONS(2804),
    [anon_sym_DASH] = ACTIONS(2802),
    [anon_sym_AMP] = ACTIONS(2802),
    [anon_sym_PIPE] = ACTIONS(2802),
    [anon_sym_LT] = ACTIONS(2802),
    [anon_sym_DOT_DOT] = ACTIONS(2802),
    [anon_sym_COLON_COLON] = ACTIONS(2802),
    [anon_sym_POUND] = ACTIONS(2802),
    [anon_sym_SQUOTE] = ACTIONS(2804),
    [anon_sym_async] = ACTIONS(2804),
    [anon_sym_auto] = ACTIONS(2804),
    [anon_sym_await] = ACTIONS(2804),
    [anon_sym_become] = ACTIONS(2804),
    [anon_sym_break] = ACTIONS(2804),
    [anon_sym_const] = ACTIONS(2804),
    [anon_sym_continue] = ACTIONS(2804),
    [anon_sym_default] = ACTIONS(2804),
    [anon_sym_enum] = ACTIONS(2804),
    [anon_sym_fn] = ACTIONS(2804),
    [anon_sym_for] = ACTIONS(2804),
    [anon_sym_gen] = ACTIONS(2804),
    [anon_sym_if] = ACTIONS(2804),
    [anon_sym_impl] = ACTIONS(2804),
    [anon_sym_let] = ACTIONS(2804),
    [anon_sym_loop] = ACTIONS(2804),
    [anon_sym_match] = ACTIONS(2804),
    [anon_sym_mod] = ACTIONS(2804),
    [anon_sym_pub] = ACTIONS(2804),
    [anon_sym_return] = ACTIONS(2804),
    [anon_sym_raw] = ACTIONS(2804),
    [anon_sym_static] = ACTIONS(2804),
    [anon_sym_struct] = ACTIONS(2804),
    [anon_sym_trait] = ACTIONS(2804),
    [anon_sym_type] = ACTIONS(2804),
    [anon_sym_union] = ACTIONS(2804),
    [anon_sym_unsafe] = ACTIONS(2804),
    [anon_sym_use] = ACTIONS(2804),
    [anon_sym_while] = ACTIONS(2804),
    [anon_sym_extern] = ACTIONS(2804),
    [anon_sym_safe] = ACTIONS(2804),
    [anon_sym_yield] = ACTIONS(2804),
    [anon_sym_move] = ACTIONS(2804),
    [anon_sym_try] = ACTIONS(2804),
    [sym_integer_literal] = ACTIONS(2802),
    [aux_sym_string_literal_token1] = ACTIONS(2802),
    [sym_char_literal] = ACTIONS(2802),
    [anon_sym_true] = ACTIONS(2804),
    [anon_sym_false] = ACTIONS(2804),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2804),
    [sym_super] = ACTIONS(2804),
    [sym_crate] = ACTIONS(2804),
    [aux_sym_metavariable_token1] = ACTIONS(2802),
    [sym__raw_string_literal_start] = ACTIONS(2802),
    [sym_float_literal] = ACTIONS(2802),
    [sym__outer_block_doc_comment_start] = ACTIONS(2802),
    [sym__outer_line_doc_comment_start] = ACTIONS(2802),
  },
  [STATE(758)] = {
    [sym_line_comment] = STATE(758),
    [sym_block_comment] = STATE(758),
    [ts_builtin_sym_end] = ACTIONS(2806),
    [sym_identifier] = ACTIONS(2808),
    [anon_sym_SEMI] = ACTIONS(2806),
    [anon_sym_macro] = ACTIONS(2808),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_LBRACE] = ACTIONS(2806),
    [anon_sym_RBRACE] = ACTIONS(2806),
    [anon_sym_macro_rules] = ACTIONS(2808),
    [anon_sym_BANG] = ACTIONS(2806),
    [anon_sym_LBRACK] = ACTIONS(2806),
    [anon_sym_STAR] = ACTIONS(2806),
    [anon_sym_u8] = ACTIONS(2808),
    [anon_sym_i8] = ACTIONS(2808),
    [anon_sym_u16] = ACTIONS(2808),
    [anon_sym_i16] = ACTIONS(2808),
    [anon_sym_u32] = ACTIONS(2808),
    [anon_sym_i32] = ACTIONS(2808),
    [anon_sym_u64] = ACTIONS(2808),
    [anon_sym_i64] = ACTIONS(2808),
    [anon_sym_u128] = ACTIONS(2808),
    [anon_sym_i128] = ACTIONS(2808),
    [anon_sym_isize] = ACTIONS(2808),
    [anon_sym_usize] = ACTIONS(2808),
    [anon_sym_f32] = ACTIONS(2808),
    [anon_sym_f64] = ACTIONS(2808),
    [anon_sym_bool] = ACTIONS(2808),
    [anon_sym_str] = ACTIONS(2808),
    [anon_sym_char] = ACTIONS(2808),
    [anon_sym_DASH] = ACTIONS(2806),
    [anon_sym_AMP] = ACTIONS(2806),
    [anon_sym_PIPE] = ACTIONS(2806),
    [anon_sym_LT] = ACTIONS(2806),
    [anon_sym_DOT_DOT] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2806),
    [anon_sym_POUND] = ACTIONS(2806),
    [anon_sym_SQUOTE] = ACTIONS(2808),
    [anon_sym_async] = ACTIONS(2808),
    [anon_sym_auto] = ACTIONS(2808),
    [anon_sym_await] = ACTIONS(2808),
    [anon_sym_become] = ACTIONS(2808),
    [anon_sym_break] = ACTIONS(2808),
    [anon_sym_const] = ACTIONS(2808),
    [anon_sym_continue] = ACTIONS(2808),
    [anon_sym_default] = ACTIONS(2808),
    [anon_sym_enum] = ACTIONS(2808),
    [anon_sym_fn] = ACTIONS(2808),
    [anon_sym_for] = ACTIONS(2808),
    [anon_sym_gen] = ACTIONS(2808),
    [anon_sym_if] = ACTIONS(2808),
    [anon_sym_impl] = ACTIONS(2808),
    [anon_sym_let] = ACTIONS(2808),
    [anon_sym_loop] = ACTIONS(2808),
    [anon_sym_match] = ACTIONS(2808),
    [anon_sym_mod] = ACTIONS(2808),
    [anon_sym_pub] = ACTIONS(2808),
    [anon_sym_return] = ACTIONS(2808),
    [anon_sym_raw] = ACTIONS(2808),
    [anon_sym_static] = ACTIONS(2808),
    [anon_sym_struct] = ACTIONS(2808),
    [anon_sym_trait] = ACTIONS(2808),
    [anon_sym_type] = ACTIONS(2808),
    [anon_sym_union] = ACTIONS(2808),
    [anon_sym_unsafe] = ACTIONS(2808),
    [anon_sym_use] = ACTIONS(2808),
    [anon_sym_while] = ACTIONS(2808),
    [anon_sym_extern] = ACTIONS(2808),
    [anon_sym_safe] = ACTIONS(2808),
    [anon_sym_yield] = ACTIONS(2808),
    [anon_sym_move] = ACTIONS(2808),
    [anon_sym_try] = ACTIONS(2808),
    [sym_integer_literal] = ACTIONS(2806),
    [aux_sym_string_literal_token1] = ACTIONS(2806),
    [sym_char_literal] = ACTIONS(2806),
    [anon_sym_true] = ACTIONS(2808),
    [anon_sym_false] = ACTIONS(2808),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2808),
    [sym_super] = ACTIONS(2808),
    [sym_crate] = ACTIONS(2808),
    [aux_sym_metavariable_token1] = ACTIONS(2806),
    [sym__raw_string_literal_start] = ACTIONS(2806),
    [sym_float_literal] = ACTIONS(2806),
    [sym__outer_block_doc_comment_start] = ACTIONS(2806),
    [sym__outer_line_doc_comment_start] = ACTIONS(2806),
  },
  [STATE(759)] = {
    [sym_line_comment] = STATE(759),
    [sym_block_comment] = STATE(759),
    [ts_builtin_sym_end] = ACTIONS(2810),
    [sym_identifier] = ACTIONS(2812),
    [anon_sym_SEMI] = ACTIONS(2810),
    [anon_sym_macro] = ACTIONS(2812),
    [anon_sym_LPAREN] = ACTIONS(2810),
    [anon_sym_LBRACE] = ACTIONS(2810),
    [anon_sym_RBRACE] = ACTIONS(2810),
    [anon_sym_macro_rules] = ACTIONS(2812),
    [anon_sym_BANG] = ACTIONS(2810),
    [anon_sym_LBRACK] = ACTIONS(2810),
    [anon_sym_STAR] = ACTIONS(2810),
    [anon_sym_u8] = ACTIONS(2812),
    [anon_sym_i8] = ACTIONS(2812),
    [anon_sym_u16] = ACTIONS(2812),
    [anon_sym_i16] = ACTIONS(2812),
    [anon_sym_u32] = ACTIONS(2812),
    [anon_sym_i32] = ACTIONS(2812),
    [anon_sym_u64] = ACTIONS(2812),
    [anon_sym_i64] = ACTIONS(2812),
    [anon_sym_u128] = ACTIONS(2812),
    [anon_sym_i128] = ACTIONS(2812),
    [anon_sym_isize] = ACTIONS(2812),
    [anon_sym_usize] = ACTIONS(2812),
    [anon_sym_f32] = ACTIONS(2812),
    [anon_sym_f64] = ACTIONS(2812),
    [anon_sym_bool] = ACTIONS(2812),
    [anon_sym_str] = ACTIONS(2812),
    [anon_sym_char] = ACTIONS(2812),
    [anon_sym_DASH] = ACTIONS(2810),
    [anon_sym_AMP] = ACTIONS(2810),
    [anon_sym_PIPE] = ACTIONS(2810),
    [anon_sym_LT] = ACTIONS(2810),
    [anon_sym_DOT_DOT] = ACTIONS(2810),
    [anon_sym_COLON_COLON] = ACTIONS(2810),
    [anon_sym_POUND] = ACTIONS(2810),
    [anon_sym_SQUOTE] = ACTIONS(2812),
    [anon_sym_async] = ACTIONS(2812),
    [anon_sym_auto] = ACTIONS(2812),
    [anon_sym_await] = ACTIONS(2812),
    [anon_sym_become] = ACTIONS(2812),
    [anon_sym_break] = ACTIONS(2812),
    [anon_sym_const] = ACTIONS(2812),
    [anon_sym_continue] = ACTIONS(2812),
    [anon_sym_default] = ACTIONS(2812),
    [anon_sym_enum] = ACTIONS(2812),
    [anon_sym_fn] = ACTIONS(2812),
    [anon_sym_for] = ACTIONS(2812),
    [anon_sym_gen] = ACTIONS(2812),
    [anon_sym_if] = ACTIONS(2812),
    [anon_sym_impl] = ACTIONS(2812),
    [anon_sym_let] = ACTIONS(2812),
    [anon_sym_loop] = ACTIONS(2812),
    [anon_sym_match] = ACTIONS(2812),
    [anon_sym_mod] = ACTIONS(2812),
    [anon_sym_pub] = ACTIONS(2812),
    [anon_sym_return] = ACTIONS(2812),
    [anon_sym_raw] = ACTIONS(2812),
    [anon_sym_static] = ACTIONS(2812),
    [anon_sym_struct] = ACTIONS(2812),
    [anon_sym_trait] = ACTIONS(2812),
    [anon_sym_type] = ACTIONS(2812),
    [anon_sym_union] = ACTIONS(2812),
    [anon_sym_unsafe] = ACTIONS(2812),
    [anon_sym_use] = ACTIONS(2812),
    [anon_sym_while] = ACTIONS(2812),
    [anon_sym_extern] = ACTIONS(2812),
    [anon_sym_safe] = ACTIONS(2812),
    [anon_sym_yield] = ACTIONS(2812),
    [anon_sym_move] = ACTIONS(2812),
    [anon_sym_try] = ACTIONS(2812),
    [sym_integer_literal] = ACTIONS(2810),
    [aux_sym_string_literal_token1] = ACTIONS(2810),
    [sym_char_literal] = ACTIONS(2810),
    [anon_sym_true] = ACTIONS(2812),
    [anon_sym_false] = ACTIONS(2812),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2812),
    [sym_super] = ACTIONS(2812),
    [sym_crate] = ACTIONS(2812),
    [aux_sym_metavariable_token1] = ACTIONS(2810),
    [sym__raw_string_literal_start] = ACTIONS(2810),
    [sym_float_literal] = ACTIONS(2810),
    [sym__outer_block_doc_comment_start] = ACTIONS(2810),
    [sym__outer_line_doc_comment_start] = ACTIONS(2810),
  },
  [STATE(760)] = {
    [sym_line_comment] = STATE(760),
    [sym_block_comment] = STATE(760),
    [ts_builtin_sym_end] = ACTIONS(2814),
    [sym_identifier] = ACTIONS(2816),
    [anon_sym_SEMI] = ACTIONS(2814),
    [anon_sym_macro] = ACTIONS(2816),
    [anon_sym_LPAREN] = ACTIONS(2814),
    [anon_sym_LBRACE] = ACTIONS(2814),
    [anon_sym_RBRACE] = ACTIONS(2814),
    [anon_sym_macro_rules] = ACTIONS(2816),
    [anon_sym_BANG] = ACTIONS(2814),
    [anon_sym_LBRACK] = ACTIONS(2814),
    [anon_sym_STAR] = ACTIONS(2814),
    [anon_sym_u8] = ACTIONS(2816),
    [anon_sym_i8] = ACTIONS(2816),
    [anon_sym_u16] = ACTIONS(2816),
    [anon_sym_i16] = ACTIONS(2816),
    [anon_sym_u32] = ACTIONS(2816),
    [anon_sym_i32] = ACTIONS(2816),
    [anon_sym_u64] = ACTIONS(2816),
    [anon_sym_i64] = ACTIONS(2816),
    [anon_sym_u128] = ACTIONS(2816),
    [anon_sym_i128] = ACTIONS(2816),
    [anon_sym_isize] = ACTIONS(2816),
    [anon_sym_usize] = ACTIONS(2816),
    [anon_sym_f32] = ACTIONS(2816),
    [anon_sym_f64] = ACTIONS(2816),
    [anon_sym_bool] = ACTIONS(2816),
    [anon_sym_str] = ACTIONS(2816),
    [anon_sym_char] = ACTIONS(2816),
    [anon_sym_DASH] = ACTIONS(2814),
    [anon_sym_AMP] = ACTIONS(2814),
    [anon_sym_PIPE] = ACTIONS(2814),
    [anon_sym_LT] = ACTIONS(2814),
    [anon_sym_DOT_DOT] = ACTIONS(2814),
    [anon_sym_COLON_COLON] = ACTIONS(2814),
    [anon_sym_POUND] = ACTIONS(2814),
    [anon_sym_SQUOTE] = ACTIONS(2816),
    [anon_sym_async] = ACTIONS(2816),
    [anon_sym_auto] = ACTIONS(2816),
    [anon_sym_await] = ACTIONS(2816),
    [anon_sym_become] = ACTIONS(2816),
    [anon_sym_break] = ACTIONS(2816),
    [anon_sym_const] = ACTIONS(2816),
    [anon_sym_continue] = ACTIONS(2816),
    [anon_sym_default] = ACTIONS(2816),
    [anon_sym_enum] = ACTIONS(2816),
    [anon_sym_fn] = ACTIONS(2816),
    [anon_sym_for] = ACTIONS(2816),
    [anon_sym_gen] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2816),
    [anon_sym_impl] = ACTIONS(2816),
    [anon_sym_let] = ACTIONS(2816),
    [anon_sym_loop] = ACTIONS(2816),
    [anon_sym_match] = ACTIONS(2816),
    [anon_sym_mod] = ACTIONS(2816),
    [anon_sym_pub] = ACTIONS(2816),
    [anon_sym_return] = ACTIONS(2816),
    [anon_sym_raw] = ACTIONS(2816),
    [anon_sym_static] = ACTIONS(2816),
    [anon_sym_struct] = ACTIONS(2816),
    [anon_sym_trait] = ACTIONS(2816),
    [anon_sym_type] = ACTIONS(2816),
    [anon_sym_union] = ACTIONS(2816),
    [anon_sym_unsafe] = ACTIONS(2816),
    [anon_sym_use] = ACTIONS(2816),
    [anon_sym_while] = ACTIONS(2816),
    [anon_sym_extern] = ACTIONS(2816),
    [anon_sym_safe] = ACTIONS(2816),
    [anon_sym_yield] = ACTIONS(2816),
    [anon_sym_move] = ACTIONS(2816),
    [anon_sym_try] = ACTIONS(2816),
    [sym_integer_literal] = ACTIONS(2814),
    [aux_sym_string_literal_token1] = ACTIONS(2814),
    [sym_char_literal] = ACTIONS(2814),
    [anon_sym_true] = ACTIONS(2816),
    [anon_sym_false] = ACTIONS(2816),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2816),
    [sym_super] = ACTIONS(2816),
    [sym_crate] = ACTIONS(2816),
    [aux_sym_metavariable_token1] = ACTIONS(2814),
    [sym__raw_string_literal_start] = ACTIONS(2814),
    [sym_float_literal] = ACTIONS(2814),
    [sym__outer_block_doc_comment_start] = ACTIONS(2814),
    [sym__outer_line_doc_comment_start] = ACTIONS(2814),
  },
  [STATE(761)] = {
    [sym_line_comment] = STATE(761),
    [sym_block_comment] = STATE(761),
    [ts_builtin_sym_end] = ACTIONS(2818),
    [sym_identifier] = ACTIONS(2820),
    [anon_sym_SEMI] = ACTIONS(2818),
    [anon_sym_macro] = ACTIONS(2820),
    [anon_sym_LPAREN] = ACTIONS(2818),
    [anon_sym_LBRACE] = ACTIONS(2818),
    [anon_sym_RBRACE] = ACTIONS(2818),
    [anon_sym_macro_rules] = ACTIONS(2820),
    [anon_sym_BANG] = ACTIONS(2818),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_STAR] = ACTIONS(2818),
    [anon_sym_u8] = ACTIONS(2820),
    [anon_sym_i8] = ACTIONS(2820),
    [anon_sym_u16] = ACTIONS(2820),
    [anon_sym_i16] = ACTIONS(2820),
    [anon_sym_u32] = ACTIONS(2820),
    [anon_sym_i32] = ACTIONS(2820),
    [anon_sym_u64] = ACTIONS(2820),
    [anon_sym_i64] = ACTIONS(2820),
    [anon_sym_u128] = ACTIONS(2820),
    [anon_sym_i128] = ACTIONS(2820),
    [anon_sym_isize] = ACTIONS(2820),
    [anon_sym_usize] = ACTIONS(2820),
    [anon_sym_f32] = ACTIONS(2820),
    [anon_sym_f64] = ACTIONS(2820),
    [anon_sym_bool] = ACTIONS(2820),
    [anon_sym_str] = ACTIONS(2820),
    [anon_sym_char] = ACTIONS(2820),
    [anon_sym_DASH] = ACTIONS(2818),
    [anon_sym_AMP] = ACTIONS(2818),
    [anon_sym_PIPE] = ACTIONS(2818),
    [anon_sym_LT] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2818),
    [anon_sym_COLON_COLON] = ACTIONS(2818),
    [anon_sym_POUND] = ACTIONS(2818),
    [anon_sym_SQUOTE] = ACTIONS(2820),
    [anon_sym_async] = ACTIONS(2820),
    [anon_sym_auto] = ACTIONS(2820),
    [anon_sym_await] = ACTIONS(2820),
    [anon_sym_become] = ACTIONS(2820),
    [anon_sym_break] = ACTIONS(2820),
    [anon_sym_const] = ACTIONS(2820),
    [anon_sym_continue] = ACTIONS(2820),
    [anon_sym_default] = ACTIONS(2820),
    [anon_sym_enum] = ACTIONS(2820),
    [anon_sym_fn] = ACTIONS(2820),
    [anon_sym_for] = ACTIONS(2820),
    [anon_sym_gen] = ACTIONS(2820),
    [anon_sym_if] = ACTIONS(2820),
    [anon_sym_impl] = ACTIONS(2820),
    [anon_sym_let] = ACTIONS(2820),
    [anon_sym_loop] = ACTIONS(2820),
    [anon_sym_match] = ACTIONS(2820),
    [anon_sym_mod] = ACTIONS(2820),
    [anon_sym_pub] = ACTIONS(2820),
    [anon_sym_return] = ACTIONS(2820),
    [anon_sym_raw] = ACTIONS(2820),
    [anon_sym_static] = ACTIONS(2820),
    [anon_sym_struct] = ACTIONS(2820),
    [anon_sym_trait] = ACTIONS(2820),
    [anon_sym_type] = ACTIONS(2820),
    [anon_sym_union] = ACTIONS(2820),
    [anon_sym_unsafe] = ACTIONS(2820),
    [anon_sym_use] = ACTIONS(2820),
    [anon_sym_while] = ACTIONS(2820),
    [anon_sym_extern] = ACTIONS(2820),
    [anon_sym_safe] = ACTIONS(2820),
    [anon_sym_yield] = ACTIONS(2820),
    [anon_sym_move] = ACTIONS(2820),
    [anon_sym_try] = ACTIONS(2820),
    [sym_integer_literal] = ACTIONS(2818),
    [aux_sym_string_literal_token1] = ACTIONS(2818),
    [sym_char_literal] = ACTIONS(2818),
    [anon_sym_true] = ACTIONS(2820),
    [anon_sym_false] = ACTIONS(2820),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2820),
    [sym_super] = ACTIONS(2820),
    [sym_crate] = ACTIONS(2820),
    [aux_sym_metavariable_token1] = ACTIONS(2818),
    [sym__raw_string_literal_start] = ACTIONS(2818),
    [sym_float_literal] = ACTIONS(2818),
    [sym__outer_block_doc_comment_start] = ACTIONS(2818),
    [sym__outer_line_doc_comment_start] = ACTIONS(2818),
  },
  [STATE(762)] = {
    [sym_line_comment] = STATE(762),
    [sym_block_comment] = STATE(762),
    [ts_builtin_sym_end] = ACTIONS(2822),
    [sym_identifier] = ACTIONS(2824),
    [anon_sym_SEMI] = ACTIONS(2822),
    [anon_sym_macro] = ACTIONS(2824),
    [anon_sym_LPAREN] = ACTIONS(2822),
    [anon_sym_LBRACE] = ACTIONS(2822),
    [anon_sym_RBRACE] = ACTIONS(2822),
    [anon_sym_macro_rules] = ACTIONS(2824),
    [anon_sym_BANG] = ACTIONS(2822),
    [anon_sym_LBRACK] = ACTIONS(2822),
    [anon_sym_STAR] = ACTIONS(2822),
    [anon_sym_u8] = ACTIONS(2824),
    [anon_sym_i8] = ACTIONS(2824),
    [anon_sym_u16] = ACTIONS(2824),
    [anon_sym_i16] = ACTIONS(2824),
    [anon_sym_u32] = ACTIONS(2824),
    [anon_sym_i32] = ACTIONS(2824),
    [anon_sym_u64] = ACTIONS(2824),
    [anon_sym_i64] = ACTIONS(2824),
    [anon_sym_u128] = ACTIONS(2824),
    [anon_sym_i128] = ACTIONS(2824),
    [anon_sym_isize] = ACTIONS(2824),
    [anon_sym_usize] = ACTIONS(2824),
    [anon_sym_f32] = ACTIONS(2824),
    [anon_sym_f64] = ACTIONS(2824),
    [anon_sym_bool] = ACTIONS(2824),
    [anon_sym_str] = ACTIONS(2824),
    [anon_sym_char] = ACTIONS(2824),
    [anon_sym_DASH] = ACTIONS(2822),
    [anon_sym_AMP] = ACTIONS(2822),
    [anon_sym_PIPE] = ACTIONS(2822),
    [anon_sym_LT] = ACTIONS(2822),
    [anon_sym_DOT_DOT] = ACTIONS(2822),
    [anon_sym_COLON_COLON] = ACTIONS(2822),
    [anon_sym_POUND] = ACTIONS(2822),
    [anon_sym_SQUOTE] = ACTIONS(2824),
    [anon_sym_async] = ACTIONS(2824),
    [anon_sym_auto] = ACTIONS(2824),
    [anon_sym_await] = ACTIONS(2824),
    [anon_sym_become] = ACTIONS(2824),
    [anon_sym_break] = ACTIONS(2824),
    [anon_sym_const] = ACTIONS(2824),
    [anon_sym_continue] = ACTIONS(2824),
    [anon_sym_default] = ACTIONS(2824),
    [anon_sym_enum] = ACTIONS(2824),
    [anon_sym_fn] = ACTIONS(2824),
    [anon_sym_for] = ACTIONS(2824),
    [anon_sym_gen] = ACTIONS(2824),
    [anon_sym_if] = ACTIONS(2824),
    [anon_sym_impl] = ACTIONS(2824),
    [anon_sym_let] = ACTIONS(2824),
    [anon_sym_loop] = ACTIONS(2824),
    [anon_sym_match] = ACTIONS(2824),
    [anon_sym_mod] = ACTIONS(2824),
    [anon_sym_pub] = ACTIONS(2824),
    [anon_sym_return] = ACTIONS(2824),
    [anon_sym_raw] = ACTIONS(2824),
    [anon_sym_static] = ACTIONS(2824),
    [anon_sym_struct] = ACTIONS(2824),
    [anon_sym_trait] = ACTIONS(2824),
    [anon_sym_type] = ACTIONS(2824),
    [anon_sym_union] = ACTIONS(2824),
    [anon_sym_unsafe] = ACTIONS(2824),
    [anon_sym_use] = ACTIONS(2824),
    [anon_sym_while] = ACTIONS(2824),
    [anon_sym_extern] = ACTIONS(2824),
    [anon_sym_safe] = ACTIONS(2824),
    [anon_sym_yield] = ACTIONS(2824),
    [anon_sym_move] = ACTIONS(2824),
    [anon_sym_try] = ACTIONS(2824),
    [sym_integer_literal] = ACTIONS(2822),
    [aux_sym_string_literal_token1] = ACTIONS(2822),
    [sym_char_literal] = ACTIONS(2822),
    [anon_sym_true] = ACTIONS(2824),
    [anon_sym_false] = ACTIONS(2824),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2824),
    [sym_super] = ACTIONS(2824),
    [sym_crate] = ACTIONS(2824),
    [aux_sym_metavariable_token1] = ACTIONS(2822),
    [sym__raw_string_literal_start] = ACTIONS(2822),
    [sym_float_literal] = ACTIONS(2822),
    [sym__outer_block_doc_comment_start] = ACTIONS(2822),
    [sym__outer_line_doc_comment_start] = ACTIONS(2822),
  },
  [STATE(763)] = {
    [sym_line_comment] = STATE(763),
    [sym_block_comment] = STATE(763),
    [ts_builtin_sym_end] = ACTIONS(2826),
    [sym_identifier] = ACTIONS(2828),
    [anon_sym_SEMI] = ACTIONS(2826),
    [anon_sym_macro] = ACTIONS(2828),
    [anon_sym_LPAREN] = ACTIONS(2826),
    [anon_sym_LBRACE] = ACTIONS(2826),
    [anon_sym_RBRACE] = ACTIONS(2826),
    [anon_sym_macro_rules] = ACTIONS(2828),
    [anon_sym_BANG] = ACTIONS(2826),
    [anon_sym_LBRACK] = ACTIONS(2826),
    [anon_sym_STAR] = ACTIONS(2826),
    [anon_sym_u8] = ACTIONS(2828),
    [anon_sym_i8] = ACTIONS(2828),
    [anon_sym_u16] = ACTIONS(2828),
    [anon_sym_i16] = ACTIONS(2828),
    [anon_sym_u32] = ACTIONS(2828),
    [anon_sym_i32] = ACTIONS(2828),
    [anon_sym_u64] = ACTIONS(2828),
    [anon_sym_i64] = ACTIONS(2828),
    [anon_sym_u128] = ACTIONS(2828),
    [anon_sym_i128] = ACTIONS(2828),
    [anon_sym_isize] = ACTIONS(2828),
    [anon_sym_usize] = ACTIONS(2828),
    [anon_sym_f32] = ACTIONS(2828),
    [anon_sym_f64] = ACTIONS(2828),
    [anon_sym_bool] = ACTIONS(2828),
    [anon_sym_str] = ACTIONS(2828),
    [anon_sym_char] = ACTIONS(2828),
    [anon_sym_DASH] = ACTIONS(2826),
    [anon_sym_AMP] = ACTIONS(2826),
    [anon_sym_PIPE] = ACTIONS(2826),
    [anon_sym_LT] = ACTIONS(2826),
    [anon_sym_DOT_DOT] = ACTIONS(2826),
    [anon_sym_COLON_COLON] = ACTIONS(2826),
    [anon_sym_POUND] = ACTIONS(2826),
    [anon_sym_SQUOTE] = ACTIONS(2828),
    [anon_sym_async] = ACTIONS(2828),
    [anon_sym_auto] = ACTIONS(2828),
    [anon_sym_await] = ACTIONS(2828),
    [anon_sym_become] = ACTIONS(2828),
    [anon_sym_break] = ACTIONS(2828),
    [anon_sym_const] = ACTIONS(2828),
    [anon_sym_continue] = ACTIONS(2828),
    [anon_sym_default] = ACTIONS(2828),
    [anon_sym_enum] = ACTIONS(2828),
    [anon_sym_fn] = ACTIONS(2828),
    [anon_sym_for] = ACTIONS(2828),
    [anon_sym_gen] = ACTIONS(2828),
    [anon_sym_if] = ACTIONS(2828),
    [anon_sym_impl] = ACTIONS(2828),
    [anon_sym_let] = ACTIONS(2828),
    [anon_sym_loop] = ACTIONS(2828),
    [anon_sym_match] = ACTIONS(2828),
    [anon_sym_mod] = ACTIONS(2828),
    [anon_sym_pub] = ACTIONS(2828),
    [anon_sym_return] = ACTIONS(2828),
    [anon_sym_raw] = ACTIONS(2828),
    [anon_sym_static] = ACTIONS(2828),
    [anon_sym_struct] = ACTIONS(2828),
    [anon_sym_trait] = ACTIONS(2828),
    [anon_sym_type] = ACTIONS(2828),
    [anon_sym_union] = ACTIONS(2828),
    [anon_sym_unsafe] = ACTIONS(2828),
    [anon_sym_use] = ACTIONS(2828),
    [anon_sym_while] = ACTIONS(2828),
    [anon_sym_extern] = ACTIONS(2828),
    [anon_sym_safe] = ACTIONS(2828),
    [anon_sym_yield] = ACTIONS(2828),
    [anon_sym_move] = ACTIONS(2828),
    [anon_sym_try] = ACTIONS(2828),
    [sym_integer_literal] = ACTIONS(2826),
    [aux_sym_string_literal_token1] = ACTIONS(2826),
    [sym_char_literal] = ACTIONS(2826),
    [anon_sym_true] = ACTIONS(2828),
    [anon_sym_false] = ACTIONS(2828),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2828),
    [sym_super] = ACTIONS(2828),
    [sym_crate] = ACTIONS(2828),
    [aux_sym_metavariable_token1] = ACTIONS(2826),
    [sym__raw_string_literal_start] = ACTIONS(2826),
    [sym_float_literal] = ACTIONS(2826),
    [sym__outer_block_doc_comment_start] = ACTIONS(2826),
    [sym__outer_line_doc_comment_start] = ACTIONS(2826),
  },
  [STATE(764)] = {
    [sym_line_comment] = STATE(764),
    [sym_block_comment] = STATE(764),
    [ts_builtin_sym_end] = ACTIONS(2830),
    [sym_identifier] = ACTIONS(2832),
    [anon_sym_SEMI] = ACTIONS(2830),
    [anon_sym_macro] = ACTIONS(2832),
    [anon_sym_LPAREN] = ACTIONS(2830),
    [anon_sym_LBRACE] = ACTIONS(2830),
    [anon_sym_RBRACE] = ACTIONS(2830),
    [anon_sym_macro_rules] = ACTIONS(2832),
    [anon_sym_BANG] = ACTIONS(2830),
    [anon_sym_LBRACK] = ACTIONS(2830),
    [anon_sym_STAR] = ACTIONS(2830),
    [anon_sym_u8] = ACTIONS(2832),
    [anon_sym_i8] = ACTIONS(2832),
    [anon_sym_u16] = ACTIONS(2832),
    [anon_sym_i16] = ACTIONS(2832),
    [anon_sym_u32] = ACTIONS(2832),
    [anon_sym_i32] = ACTIONS(2832),
    [anon_sym_u64] = ACTIONS(2832),
    [anon_sym_i64] = ACTIONS(2832),
    [anon_sym_u128] = ACTIONS(2832),
    [anon_sym_i128] = ACTIONS(2832),
    [anon_sym_isize] = ACTIONS(2832),
    [anon_sym_usize] = ACTIONS(2832),
    [anon_sym_f32] = ACTIONS(2832),
    [anon_sym_f64] = ACTIONS(2832),
    [anon_sym_bool] = ACTIONS(2832),
    [anon_sym_str] = ACTIONS(2832),
    [anon_sym_char] = ACTIONS(2832),
    [anon_sym_DASH] = ACTIONS(2830),
    [anon_sym_AMP] = ACTIONS(2830),
    [anon_sym_PIPE] = ACTIONS(2830),
    [anon_sym_LT] = ACTIONS(2830),
    [anon_sym_DOT_DOT] = ACTIONS(2830),
    [anon_sym_COLON_COLON] = ACTIONS(2830),
    [anon_sym_POUND] = ACTIONS(2830),
    [anon_sym_SQUOTE] = ACTIONS(2832),
    [anon_sym_async] = ACTIONS(2832),
    [anon_sym_auto] = ACTIONS(2832),
    [anon_sym_await] = ACTIONS(2832),
    [anon_sym_become] = ACTIONS(2832),
    [anon_sym_break] = ACTIONS(2832),
    [anon_sym_const] = ACTIONS(2832),
    [anon_sym_continue] = ACTIONS(2832),
    [anon_sym_default] = ACTIONS(2832),
    [anon_sym_enum] = ACTIONS(2832),
    [anon_sym_fn] = ACTIONS(2832),
    [anon_sym_for] = ACTIONS(2832),
    [anon_sym_gen] = ACTIONS(2832),
    [anon_sym_if] = ACTIONS(2832),
    [anon_sym_impl] = ACTIONS(2832),
    [anon_sym_let] = ACTIONS(2832),
    [anon_sym_loop] = ACTIONS(2832),
    [anon_sym_match] = ACTIONS(2832),
    [anon_sym_mod] = ACTIONS(2832),
    [anon_sym_pub] = ACTIONS(2832),
    [anon_sym_return] = ACTIONS(2832),
    [anon_sym_raw] = ACTIONS(2832),
    [anon_sym_static] = ACTIONS(2832),
    [anon_sym_struct] = ACTIONS(2832),
    [anon_sym_trait] = ACTIONS(2832),
    [anon_sym_type] = ACTIONS(2832),
    [anon_sym_union] = ACTIONS(2832),
    [anon_sym_unsafe] = ACTIONS(2832),
    [anon_sym_use] = ACTIONS(2832),
    [anon_sym_while] = ACTIONS(2832),
    [anon_sym_extern] = ACTIONS(2832),
    [anon_sym_safe] = ACTIONS(2832),
    [anon_sym_yield] = ACTIONS(2832),
    [anon_sym_move] = ACTIONS(2832),
    [anon_sym_try] = ACTIONS(2832),
    [sym_integer_literal] = ACTIONS(2830),
    [aux_sym_string_literal_token1] = ACTIONS(2830),
    [sym_char_literal] = ACTIONS(2830),
    [anon_sym_true] = ACTIONS(2832),
    [anon_sym_false] = ACTIONS(2832),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2832),
    [sym_super] = ACTIONS(2832),
    [sym_crate] = ACTIONS(2832),
    [aux_sym_metavariable_token1] = ACTIONS(2830),
    [sym__raw_string_literal_start] = ACTIONS(2830),
    [sym_float_literal] = ACTIONS(2830),
    [sym__outer_block_doc_comment_start] = ACTIONS(2830),
    [sym__outer_line_doc_comment_start] = ACTIONS(2830),
  },
  [STATE(765)] = {
    [sym_line_comment] = STATE(765),
    [sym_block_comment] = STATE(765),
    [ts_builtin_sym_end] = ACTIONS(2834),
    [sym_identifier] = ACTIONS(2836),
    [anon_sym_SEMI] = ACTIONS(2834),
    [anon_sym_macro] = ACTIONS(2836),
    [anon_sym_LPAREN] = ACTIONS(2834),
    [anon_sym_LBRACE] = ACTIONS(2834),
    [anon_sym_RBRACE] = ACTIONS(2834),
    [anon_sym_macro_rules] = ACTIONS(2836),
    [anon_sym_BANG] = ACTIONS(2834),
    [anon_sym_LBRACK] = ACTIONS(2834),
    [anon_sym_STAR] = ACTIONS(2834),
    [anon_sym_u8] = ACTIONS(2836),
    [anon_sym_i8] = ACTIONS(2836),
    [anon_sym_u16] = ACTIONS(2836),
    [anon_sym_i16] = ACTIONS(2836),
    [anon_sym_u32] = ACTIONS(2836),
    [anon_sym_i32] = ACTIONS(2836),
    [anon_sym_u64] = ACTIONS(2836),
    [anon_sym_i64] = ACTIONS(2836),
    [anon_sym_u128] = ACTIONS(2836),
    [anon_sym_i128] = ACTIONS(2836),
    [anon_sym_isize] = ACTIONS(2836),
    [anon_sym_usize] = ACTIONS(2836),
    [anon_sym_f32] = ACTIONS(2836),
    [anon_sym_f64] = ACTIONS(2836),
    [anon_sym_bool] = ACTIONS(2836),
    [anon_sym_str] = ACTIONS(2836),
    [anon_sym_char] = ACTIONS(2836),
    [anon_sym_DASH] = ACTIONS(2834),
    [anon_sym_AMP] = ACTIONS(2834),
    [anon_sym_PIPE] = ACTIONS(2834),
    [anon_sym_LT] = ACTIONS(2834),
    [anon_sym_DOT_DOT] = ACTIONS(2834),
    [anon_sym_COLON_COLON] = ACTIONS(2834),
    [anon_sym_POUND] = ACTIONS(2834),
    [anon_sym_SQUOTE] = ACTIONS(2836),
    [anon_sym_async] = ACTIONS(2836),
    [anon_sym_auto] = ACTIONS(2836),
    [anon_sym_await] = ACTIONS(2836),
    [anon_sym_become] = ACTIONS(2836),
    [anon_sym_break] = ACTIONS(2836),
    [anon_sym_const] = ACTIONS(2836),
    [anon_sym_continue] = ACTIONS(2836),
    [anon_sym_default] = ACTIONS(2836),
    [anon_sym_enum] = ACTIONS(2836),
    [anon_sym_fn] = ACTIONS(2836),
    [anon_sym_for] = ACTIONS(2836),
    [anon_sym_gen] = ACTIONS(2836),
    [anon_sym_if] = ACTIONS(2836),
    [anon_sym_impl] = ACTIONS(2836),
    [anon_sym_let] = ACTIONS(2836),
    [anon_sym_loop] = ACTIONS(2836),
    [anon_sym_match] = ACTIONS(2836),
    [anon_sym_mod] = ACTIONS(2836),
    [anon_sym_pub] = ACTIONS(2836),
    [anon_sym_return] = ACTIONS(2836),
    [anon_sym_raw] = ACTIONS(2836),
    [anon_sym_static] = ACTIONS(2836),
    [anon_sym_struct] = ACTIONS(2836),
    [anon_sym_trait] = ACTIONS(2836),
    [anon_sym_type] = ACTIONS(2836),
    [anon_sym_union] = ACTIONS(2836),
    [anon_sym_unsafe] = ACTIONS(2836),
    [anon_sym_use] = ACTIONS(2836),
    [anon_sym_while] = ACTIONS(2836),
    [anon_sym_extern] = ACTIONS(2836),
    [anon_sym_safe] = ACTIONS(2836),
    [anon_sym_yield] = ACTIONS(2836),
    [anon_sym_move] = ACTIONS(2836),
    [anon_sym_try] = ACTIONS(2836),
    [sym_integer_literal] = ACTIONS(2834),
    [aux_sym_string_literal_token1] = ACTIONS(2834),
    [sym_char_literal] = ACTIONS(2834),
    [anon_sym_true] = ACTIONS(2836),
    [anon_sym_false] = ACTIONS(2836),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2836),
    [sym_super] = ACTIONS(2836),
    [sym_crate] = ACTIONS(2836),
    [aux_sym_metavariable_token1] = ACTIONS(2834),
    [sym__raw_string_literal_start] = ACTIONS(2834),
    [sym_float_literal] = ACTIONS(2834),
    [sym__outer_block_doc_comment_start] = ACTIONS(2834),
    [sym__outer_line_doc_comment_start] = ACTIONS(2834),
  },
  [STATE(766)] = {
    [sym_line_comment] = STATE(766),
    [sym_block_comment] = STATE(766),
    [ts_builtin_sym_end] = ACTIONS(2838),
    [sym_identifier] = ACTIONS(2840),
    [anon_sym_SEMI] = ACTIONS(2838),
    [anon_sym_macro] = ACTIONS(2840),
    [anon_sym_LPAREN] = ACTIONS(2838),
    [anon_sym_LBRACE] = ACTIONS(2838),
    [anon_sym_RBRACE] = ACTIONS(2838),
    [anon_sym_macro_rules] = ACTIONS(2840),
    [anon_sym_BANG] = ACTIONS(2838),
    [anon_sym_LBRACK] = ACTIONS(2838),
    [anon_sym_STAR] = ACTIONS(2838),
    [anon_sym_u8] = ACTIONS(2840),
    [anon_sym_i8] = ACTIONS(2840),
    [anon_sym_u16] = ACTIONS(2840),
    [anon_sym_i16] = ACTIONS(2840),
    [anon_sym_u32] = ACTIONS(2840),
    [anon_sym_i32] = ACTIONS(2840),
    [anon_sym_u64] = ACTIONS(2840),
    [anon_sym_i64] = ACTIONS(2840),
    [anon_sym_u128] = ACTIONS(2840),
    [anon_sym_i128] = ACTIONS(2840),
    [anon_sym_isize] = ACTIONS(2840),
    [anon_sym_usize] = ACTIONS(2840),
    [anon_sym_f32] = ACTIONS(2840),
    [anon_sym_f64] = ACTIONS(2840),
    [anon_sym_bool] = ACTIONS(2840),
    [anon_sym_str] = ACTIONS(2840),
    [anon_sym_char] = ACTIONS(2840),
    [anon_sym_DASH] = ACTIONS(2838),
    [anon_sym_AMP] = ACTIONS(2838),
    [anon_sym_PIPE] = ACTIONS(2838),
    [anon_sym_LT] = ACTIONS(2838),
    [anon_sym_DOT_DOT] = ACTIONS(2838),
    [anon_sym_COLON_COLON] = ACTIONS(2838),
    [anon_sym_POUND] = ACTIONS(2838),
    [anon_sym_SQUOTE] = ACTIONS(2840),
    [anon_sym_async] = ACTIONS(2840),
    [anon_sym_auto] = ACTIONS(2840),
    [anon_sym_await] = ACTIONS(2840),
    [anon_sym_become] = ACTIONS(2840),
    [anon_sym_break] = ACTIONS(2840),
    [anon_sym_const] = ACTIONS(2840),
    [anon_sym_continue] = ACTIONS(2840),
    [anon_sym_default] = ACTIONS(2840),
    [anon_sym_enum] = ACTIONS(2840),
    [anon_sym_fn] = ACTIONS(2840),
    [anon_sym_for] = ACTIONS(2840),
    [anon_sym_gen] = ACTIONS(2840),
    [anon_sym_if] = ACTIONS(2840),
    [anon_sym_impl] = ACTIONS(2840),
    [anon_sym_let] = ACTIONS(2840),
    [anon_sym_loop] = ACTIONS(2840),
    [anon_sym_match] = ACTIONS(2840),
    [anon_sym_mod] = ACTIONS(2840),
    [anon_sym_pub] = ACTIONS(2840),
    [anon_sym_return] = ACTIONS(2840),
    [anon_sym_raw] = ACTIONS(2840),
    [anon_sym_static] = ACTIONS(2840),
    [anon_sym_struct] = ACTIONS(2840),
    [anon_sym_trait] = ACTIONS(2840),
    [anon_sym_type] = ACTIONS(2840),
    [anon_sym_union] = ACTIONS(2840),
    [anon_sym_unsafe] = ACTIONS(2840),
    [anon_sym_use] = ACTIONS(2840),
    [anon_sym_while] = ACTIONS(2840),
    [anon_sym_extern] = ACTIONS(2840),
    [anon_sym_safe] = ACTIONS(2840),
    [anon_sym_yield] = ACTIONS(2840),
    [anon_sym_move] = ACTIONS(2840),
    [anon_sym_try] = ACTIONS(2840),
    [sym_integer_literal] = ACTIONS(2838),
    [aux_sym_string_literal_token1] = ACTIONS(2838),
    [sym_char_literal] = ACTIONS(2838),
    [anon_sym_true] = ACTIONS(2840),
    [anon_sym_false] = ACTIONS(2840),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2840),
    [sym_super] = ACTIONS(2840),
    [sym_crate] = ACTIONS(2840),
    [aux_sym_metavariable_token1] = ACTIONS(2838),
    [sym__raw_string_literal_start] = ACTIONS(2838),
    [sym_float_literal] = ACTIONS(2838),
    [sym__outer_block_doc_comment_start] = ACTIONS(2838),
    [sym__outer_line_doc_comment_start] = ACTIONS(2838),
  },
  [STATE(767)] = {
    [sym_line_comment] = STATE(767),
    [sym_block_comment] = STATE(767),
    [ts_builtin_sym_end] = ACTIONS(2842),
    [sym_identifier] = ACTIONS(2844),
    [anon_sym_SEMI] = ACTIONS(2842),
    [anon_sym_macro] = ACTIONS(2844),
    [anon_sym_LPAREN] = ACTIONS(2842),
    [anon_sym_LBRACE] = ACTIONS(2842),
    [anon_sym_RBRACE] = ACTIONS(2842),
    [anon_sym_macro_rules] = ACTIONS(2844),
    [anon_sym_BANG] = ACTIONS(2842),
    [anon_sym_LBRACK] = ACTIONS(2842),
    [anon_sym_STAR] = ACTIONS(2842),
    [anon_sym_u8] = ACTIONS(2844),
    [anon_sym_i8] = ACTIONS(2844),
    [anon_sym_u16] = ACTIONS(2844),
    [anon_sym_i16] = ACTIONS(2844),
    [anon_sym_u32] = ACTIONS(2844),
    [anon_sym_i32] = ACTIONS(2844),
    [anon_sym_u64] = ACTIONS(2844),
    [anon_sym_i64] = ACTIONS(2844),
    [anon_sym_u128] = ACTIONS(2844),
    [anon_sym_i128] = ACTIONS(2844),
    [anon_sym_isize] = ACTIONS(2844),
    [anon_sym_usize] = ACTIONS(2844),
    [anon_sym_f32] = ACTIONS(2844),
    [anon_sym_f64] = ACTIONS(2844),
    [anon_sym_bool] = ACTIONS(2844),
    [anon_sym_str] = ACTIONS(2844),
    [anon_sym_char] = ACTIONS(2844),
    [anon_sym_DASH] = ACTIONS(2842),
    [anon_sym_AMP] = ACTIONS(2842),
    [anon_sym_PIPE] = ACTIONS(2842),
    [anon_sym_LT] = ACTIONS(2842),
    [anon_sym_DOT_DOT] = ACTIONS(2842),
    [anon_sym_COLON_COLON] = ACTIONS(2842),
    [anon_sym_POUND] = ACTIONS(2842),
    [anon_sym_SQUOTE] = ACTIONS(2844),
    [anon_sym_async] = ACTIONS(2844),
    [anon_sym_auto] = ACTIONS(2844),
    [anon_sym_await] = ACTIONS(2844),
    [anon_sym_become] = ACTIONS(2844),
    [anon_sym_break] = ACTIONS(2844),
    [anon_sym_const] = ACTIONS(2844),
    [anon_sym_continue] = ACTIONS(2844),
    [anon_sym_default] = ACTIONS(2844),
    [anon_sym_enum] = ACTIONS(2844),
    [anon_sym_fn] = ACTIONS(2844),
    [anon_sym_for] = ACTIONS(2844),
    [anon_sym_gen] = ACTIONS(2844),
    [anon_sym_if] = ACTIONS(2844),
    [anon_sym_impl] = ACTIONS(2844),
    [anon_sym_let] = ACTIONS(2844),
    [anon_sym_loop] = ACTIONS(2844),
    [anon_sym_match] = ACTIONS(2844),
    [anon_sym_mod] = ACTIONS(2844),
    [anon_sym_pub] = ACTIONS(2844),
    [anon_sym_return] = ACTIONS(2844),
    [anon_sym_raw] = ACTIONS(2844),
    [anon_sym_static] = ACTIONS(2844),
    [anon_sym_struct] = ACTIONS(2844),
    [anon_sym_trait] = ACTIONS(2844),
    [anon_sym_type] = ACTIONS(2844),
    [anon_sym_union] = ACTIONS(2844),
    [anon_sym_unsafe] = ACTIONS(2844),
    [anon_sym_use] = ACTIONS(2844),
    [anon_sym_while] = ACTIONS(2844),
    [anon_sym_extern] = ACTIONS(2844),
    [anon_sym_safe] = ACTIONS(2844),
    [anon_sym_yield] = ACTIONS(2844),
    [anon_sym_move] = ACTIONS(2844),
    [anon_sym_try] = ACTIONS(2844),
    [sym_integer_literal] = ACTIONS(2842),
    [aux_sym_string_literal_token1] = ACTIONS(2842),
    [sym_char_literal] = ACTIONS(2842),
    [anon_sym_true] = ACTIONS(2844),
    [anon_sym_false] = ACTIONS(2844),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2844),
    [sym_super] = ACTIONS(2844),
    [sym_crate] = ACTIONS(2844),
    [aux_sym_metavariable_token1] = ACTIONS(2842),
    [sym__raw_string_literal_start] = ACTIONS(2842),
    [sym_float_literal] = ACTIONS(2842),
    [sym__outer_block_doc_comment_start] = ACTIONS(2842),
    [sym__outer_line_doc_comment_start] = ACTIONS(2842),
  },
  [STATE(768)] = {
    [sym_line_comment] = STATE(768),
    [sym_block_comment] = STATE(768),
    [ts_builtin_sym_end] = ACTIONS(2846),
    [sym_identifier] = ACTIONS(2848),
    [anon_sym_SEMI] = ACTIONS(2846),
    [anon_sym_macro] = ACTIONS(2848),
    [anon_sym_LPAREN] = ACTIONS(2846),
    [anon_sym_LBRACE] = ACTIONS(2846),
    [anon_sym_RBRACE] = ACTIONS(2846),
    [anon_sym_macro_rules] = ACTIONS(2848),
    [anon_sym_BANG] = ACTIONS(2846),
    [anon_sym_LBRACK] = ACTIONS(2846),
    [anon_sym_STAR] = ACTIONS(2846),
    [anon_sym_u8] = ACTIONS(2848),
    [anon_sym_i8] = ACTIONS(2848),
    [anon_sym_u16] = ACTIONS(2848),
    [anon_sym_i16] = ACTIONS(2848),
    [anon_sym_u32] = ACTIONS(2848),
    [anon_sym_i32] = ACTIONS(2848),
    [anon_sym_u64] = ACTIONS(2848),
    [anon_sym_i64] = ACTIONS(2848),
    [anon_sym_u128] = ACTIONS(2848),
    [anon_sym_i128] = ACTIONS(2848),
    [anon_sym_isize] = ACTIONS(2848),
    [anon_sym_usize] = ACTIONS(2848),
    [anon_sym_f32] = ACTIONS(2848),
    [anon_sym_f64] = ACTIONS(2848),
    [anon_sym_bool] = ACTIONS(2848),
    [anon_sym_str] = ACTIONS(2848),
    [anon_sym_char] = ACTIONS(2848),
    [anon_sym_DASH] = ACTIONS(2846),
    [anon_sym_AMP] = ACTIONS(2846),
    [anon_sym_PIPE] = ACTIONS(2846),
    [anon_sym_LT] = ACTIONS(2846),
    [anon_sym_DOT_DOT] = ACTIONS(2846),
    [anon_sym_COLON_COLON] = ACTIONS(2846),
    [anon_sym_POUND] = ACTIONS(2846),
    [anon_sym_SQUOTE] = ACTIONS(2848),
    [anon_sym_async] = ACTIONS(2848),
    [anon_sym_auto] = ACTIONS(2848),
    [anon_sym_await] = ACTIONS(2848),
    [anon_sym_become] = ACTIONS(2848),
    [anon_sym_break] = ACTIONS(2848),
    [anon_sym_const] = ACTIONS(2848),
    [anon_sym_continue] = ACTIONS(2848),
    [anon_sym_default] = ACTIONS(2848),
    [anon_sym_enum] = ACTIONS(2848),
    [anon_sym_fn] = ACTIONS(2848),
    [anon_sym_for] = ACTIONS(2848),
    [anon_sym_gen] = ACTIONS(2848),
    [anon_sym_if] = ACTIONS(2848),
    [anon_sym_impl] = ACTIONS(2848),
    [anon_sym_let] = ACTIONS(2848),
    [anon_sym_loop] = ACTIONS(2848),
    [anon_sym_match] = ACTIONS(2848),
    [anon_sym_mod] = ACTIONS(2848),
    [anon_sym_pub] = ACTIONS(2848),
    [anon_sym_return] = ACTIONS(2848),
    [anon_sym_raw] = ACTIONS(2848),
    [anon_sym_static] = ACTIONS(2848),
    [anon_sym_struct] = ACTIONS(2848),
    [anon_sym_trait] = ACTIONS(2848),
    [anon_sym_type] = ACTIONS(2848),
    [anon_sym_union] = ACTIONS(2848),
    [anon_sym_unsafe] = ACTIONS(2848),
    [anon_sym_use] = ACTIONS(2848),
    [anon_sym_while] = ACTIONS(2848),
    [anon_sym_extern] = ACTIONS(2848),
    [anon_sym_safe] = ACTIONS(2848),
    [anon_sym_yield] = ACTIONS(2848),
    [anon_sym_move] = ACTIONS(2848),
    [anon_sym_try] = ACTIONS(2848),
    [sym_integer_literal] = ACTIONS(2846),
    [aux_sym_string_literal_token1] = ACTIONS(2846),
    [sym_char_literal] = ACTIONS(2846),
    [anon_sym_true] = ACTIONS(2848),
    [anon_sym_false] = ACTIONS(2848),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2848),
    [sym_super] = ACTIONS(2848),
    [sym_crate] = ACTIONS(2848),
    [aux_sym_metavariable_token1] = ACTIONS(2846),
    [sym__raw_string_literal_start] = ACTIONS(2846),
    [sym_float_literal] = ACTIONS(2846),
    [sym__outer_block_doc_comment_start] = ACTIONS(2846),
    [sym__outer_line_doc_comment_start] = ACTIONS(2846),
  },
  [STATE(769)] = {
    [sym_line_comment] = STATE(769),
    [sym_block_comment] = STATE(769),
    [ts_builtin_sym_end] = ACTIONS(2850),
    [sym_identifier] = ACTIONS(2852),
    [anon_sym_SEMI] = ACTIONS(2850),
    [anon_sym_macro] = ACTIONS(2852),
    [anon_sym_LPAREN] = ACTIONS(2850),
    [anon_sym_LBRACE] = ACTIONS(2850),
    [anon_sym_RBRACE] = ACTIONS(2850),
    [anon_sym_macro_rules] = ACTIONS(2852),
    [anon_sym_BANG] = ACTIONS(2850),
    [anon_sym_LBRACK] = ACTIONS(2850),
    [anon_sym_STAR] = ACTIONS(2850),
    [anon_sym_u8] = ACTIONS(2852),
    [anon_sym_i8] = ACTIONS(2852),
    [anon_sym_u16] = ACTIONS(2852),
    [anon_sym_i16] = ACTIONS(2852),
    [anon_sym_u32] = ACTIONS(2852),
    [anon_sym_i32] = ACTIONS(2852),
    [anon_sym_u64] = ACTIONS(2852),
    [anon_sym_i64] = ACTIONS(2852),
    [anon_sym_u128] = ACTIONS(2852),
    [anon_sym_i128] = ACTIONS(2852),
    [anon_sym_isize] = ACTIONS(2852),
    [anon_sym_usize] = ACTIONS(2852),
    [anon_sym_f32] = ACTIONS(2852),
    [anon_sym_f64] = ACTIONS(2852),
    [anon_sym_bool] = ACTIONS(2852),
    [anon_sym_str] = ACTIONS(2852),
    [anon_sym_char] = ACTIONS(2852),
    [anon_sym_DASH] = ACTIONS(2850),
    [anon_sym_AMP] = ACTIONS(2850),
    [anon_sym_PIPE] = ACTIONS(2850),
    [anon_sym_LT] = ACTIONS(2850),
    [anon_sym_DOT_DOT] = ACTIONS(2850),
    [anon_sym_COLON_COLON] = ACTIONS(2850),
    [anon_sym_POUND] = ACTIONS(2850),
    [anon_sym_SQUOTE] = ACTIONS(2852),
    [anon_sym_async] = ACTIONS(2852),
    [anon_sym_auto] = ACTIONS(2852),
    [anon_sym_await] = ACTIONS(2852),
    [anon_sym_become] = ACTIONS(2852),
    [anon_sym_break] = ACTIONS(2852),
    [anon_sym_const] = ACTIONS(2852),
    [anon_sym_continue] = ACTIONS(2852),
    [anon_sym_default] = ACTIONS(2852),
    [anon_sym_enum] = ACTIONS(2852),
    [anon_sym_fn] = ACTIONS(2852),
    [anon_sym_for] = ACTIONS(2852),
    [anon_sym_gen] = ACTIONS(2852),
    [anon_sym_if] = ACTIONS(2852),
    [anon_sym_impl] = ACTIONS(2852),
    [anon_sym_let] = ACTIONS(2852),
    [anon_sym_loop] = ACTIONS(2852),
    [anon_sym_match] = ACTIONS(2852),
    [anon_sym_mod] = ACTIONS(2852),
    [anon_sym_pub] = ACTIONS(2852),
    [anon_sym_return] = ACTIONS(2852),
    [anon_sym_raw] = ACTIONS(2852),
    [anon_sym_static] = ACTIONS(2852),
    [anon_sym_struct] = ACTIONS(2852),
    [anon_sym_trait] = ACTIONS(2852),
    [anon_sym_type] = ACTIONS(2852),
    [anon_sym_union] = ACTIONS(2852),
    [anon_sym_unsafe] = ACTIONS(2852),
    [anon_sym_use] = ACTIONS(2852),
    [anon_sym_while] = ACTIONS(2852),
    [anon_sym_extern] = ACTIONS(2852),
    [anon_sym_safe] = ACTIONS(2852),
    [anon_sym_yield] = ACTIONS(2852),
    [anon_sym_move] = ACTIONS(2852),
    [anon_sym_try] = ACTIONS(2852),
    [sym_integer_literal] = ACTIONS(2850),
    [aux_sym_string_literal_token1] = ACTIONS(2850),
    [sym_char_literal] = ACTIONS(2850),
    [anon_sym_true] = ACTIONS(2852),
    [anon_sym_false] = ACTIONS(2852),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2852),
    [sym_super] = ACTIONS(2852),
    [sym_crate] = ACTIONS(2852),
    [aux_sym_metavariable_token1] = ACTIONS(2850),
    [sym__raw_string_literal_start] = ACTIONS(2850),
    [sym_float_literal] = ACTIONS(2850),
    [sym__outer_block_doc_comment_start] = ACTIONS(2850),
    [sym__outer_line_doc_comment_start] = ACTIONS(2850),
  },
  [STATE(770)] = {
    [sym_line_comment] = STATE(770),
    [sym_block_comment] = STATE(770),
    [ts_builtin_sym_end] = ACTIONS(2854),
    [sym_identifier] = ACTIONS(2856),
    [anon_sym_SEMI] = ACTIONS(2854),
    [anon_sym_macro] = ACTIONS(2856),
    [anon_sym_LPAREN] = ACTIONS(2854),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_RBRACE] = ACTIONS(2854),
    [anon_sym_macro_rules] = ACTIONS(2856),
    [anon_sym_BANG] = ACTIONS(2854),
    [anon_sym_LBRACK] = ACTIONS(2854),
    [anon_sym_STAR] = ACTIONS(2854),
    [anon_sym_u8] = ACTIONS(2856),
    [anon_sym_i8] = ACTIONS(2856),
    [anon_sym_u16] = ACTIONS(2856),
    [anon_sym_i16] = ACTIONS(2856),
    [anon_sym_u32] = ACTIONS(2856),
    [anon_sym_i32] = ACTIONS(2856),
    [anon_sym_u64] = ACTIONS(2856),
    [anon_sym_i64] = ACTIONS(2856),
    [anon_sym_u128] = ACTIONS(2856),
    [anon_sym_i128] = ACTIONS(2856),
    [anon_sym_isize] = ACTIONS(2856),
    [anon_sym_usize] = ACTIONS(2856),
    [anon_sym_f32] = ACTIONS(2856),
    [anon_sym_f64] = ACTIONS(2856),
    [anon_sym_bool] = ACTIONS(2856),
    [anon_sym_str] = ACTIONS(2856),
    [anon_sym_char] = ACTIONS(2856),
    [anon_sym_DASH] = ACTIONS(2854),
    [anon_sym_AMP] = ACTIONS(2854),
    [anon_sym_PIPE] = ACTIONS(2854),
    [anon_sym_LT] = ACTIONS(2854),
    [anon_sym_DOT_DOT] = ACTIONS(2854),
    [anon_sym_COLON_COLON] = ACTIONS(2854),
    [anon_sym_POUND] = ACTIONS(2854),
    [anon_sym_SQUOTE] = ACTIONS(2856),
    [anon_sym_async] = ACTIONS(2856),
    [anon_sym_auto] = ACTIONS(2856),
    [anon_sym_await] = ACTIONS(2856),
    [anon_sym_become] = ACTIONS(2856),
    [anon_sym_break] = ACTIONS(2856),
    [anon_sym_const] = ACTIONS(2856),
    [anon_sym_continue] = ACTIONS(2856),
    [anon_sym_default] = ACTIONS(2856),
    [anon_sym_enum] = ACTIONS(2856),
    [anon_sym_fn] = ACTIONS(2856),
    [anon_sym_for] = ACTIONS(2856),
    [anon_sym_gen] = ACTIONS(2856),
    [anon_sym_if] = ACTIONS(2856),
    [anon_sym_impl] = ACTIONS(2856),
    [anon_sym_let] = ACTIONS(2856),
    [anon_sym_loop] = ACTIONS(2856),
    [anon_sym_match] = ACTIONS(2856),
    [anon_sym_mod] = ACTIONS(2856),
    [anon_sym_pub] = ACTIONS(2856),
    [anon_sym_return] = ACTIONS(2856),
    [anon_sym_raw] = ACTIONS(2856),
    [anon_sym_static] = ACTIONS(2856),
    [anon_sym_struct] = ACTIONS(2856),
    [anon_sym_trait] = ACTIONS(2856),
    [anon_sym_type] = ACTIONS(2856),
    [anon_sym_union] = ACTIONS(2856),
    [anon_sym_unsafe] = ACTIONS(2856),
    [anon_sym_use] = ACTIONS(2856),
    [anon_sym_while] = ACTIONS(2856),
    [anon_sym_extern] = ACTIONS(2856),
    [anon_sym_safe] = ACTIONS(2856),
    [anon_sym_yield] = ACTIONS(2856),
    [anon_sym_move] = ACTIONS(2856),
    [anon_sym_try] = ACTIONS(2856),
    [sym_integer_literal] = ACTIONS(2854),
    [aux_sym_string_literal_token1] = ACTIONS(2854),
    [sym_char_literal] = ACTIONS(2854),
    [anon_sym_true] = ACTIONS(2856),
    [anon_sym_false] = ACTIONS(2856),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2856),
    [sym_super] = ACTIONS(2856),
    [sym_crate] = ACTIONS(2856),
    [aux_sym_metavariable_token1] = ACTIONS(2854),
    [sym__raw_string_literal_start] = ACTIONS(2854),
    [sym_float_literal] = ACTIONS(2854),
    [sym__outer_block_doc_comment_start] = ACTIONS(2854),
    [sym__outer_line_doc_comment_start] = ACTIONS(2854),
  },
  [STATE(771)] = {
    [sym_line_comment] = STATE(771),
    [sym_block_comment] = STATE(771),
    [ts_builtin_sym_end] = ACTIONS(2858),
    [sym_identifier] = ACTIONS(2860),
    [anon_sym_SEMI] = ACTIONS(2858),
    [anon_sym_macro] = ACTIONS(2860),
    [anon_sym_LPAREN] = ACTIONS(2858),
    [anon_sym_LBRACE] = ACTIONS(2858),
    [anon_sym_RBRACE] = ACTIONS(2858),
    [anon_sym_macro_rules] = ACTIONS(2860),
    [anon_sym_BANG] = ACTIONS(2858),
    [anon_sym_LBRACK] = ACTIONS(2858),
    [anon_sym_STAR] = ACTIONS(2858),
    [anon_sym_u8] = ACTIONS(2860),
    [anon_sym_i8] = ACTIONS(2860),
    [anon_sym_u16] = ACTIONS(2860),
    [anon_sym_i16] = ACTIONS(2860),
    [anon_sym_u32] = ACTIONS(2860),
    [anon_sym_i32] = ACTIONS(2860),
    [anon_sym_u64] = ACTIONS(2860),
    [anon_sym_i64] = ACTIONS(2860),
    [anon_sym_u128] = ACTIONS(2860),
    [anon_sym_i128] = ACTIONS(2860),
    [anon_sym_isize] = ACTIONS(2860),
    [anon_sym_usize] = ACTIONS(2860),
    [anon_sym_f32] = ACTIONS(2860),
    [anon_sym_f64] = ACTIONS(2860),
    [anon_sym_bool] = ACTIONS(2860),
    [anon_sym_str] = ACTIONS(2860),
    [anon_sym_char] = ACTIONS(2860),
    [anon_sym_DASH] = ACTIONS(2858),
    [anon_sym_AMP] = ACTIONS(2858),
    [anon_sym_PIPE] = ACTIONS(2858),
    [anon_sym_LT] = ACTIONS(2858),
    [anon_sym_DOT_DOT] = ACTIONS(2858),
    [anon_sym_COLON_COLON] = ACTIONS(2858),
    [anon_sym_POUND] = ACTIONS(2858),
    [anon_sym_SQUOTE] = ACTIONS(2860),
    [anon_sym_async] = ACTIONS(2860),
    [anon_sym_auto] = ACTIONS(2860),
    [anon_sym_await] = ACTIONS(2860),
    [anon_sym_become] = ACTIONS(2860),
    [anon_sym_break] = ACTIONS(2860),
    [anon_sym_const] = ACTIONS(2860),
    [anon_sym_continue] = ACTIONS(2860),
    [anon_sym_default] = ACTIONS(2860),
    [anon_sym_enum] = ACTIONS(2860),
    [anon_sym_fn] = ACTIONS(2860),
    [anon_sym_for] = ACTIONS(2860),
    [anon_sym_gen] = ACTIONS(2860),
    [anon_sym_if] = ACTIONS(2860),
    [anon_sym_impl] = ACTIONS(2860),
    [anon_sym_let] = ACTIONS(2860),
    [anon_sym_loop] = ACTIONS(2860),
    [anon_sym_match] = ACTIONS(2860),
    [anon_sym_mod] = ACTIONS(2860),
    [anon_sym_pub] = ACTIONS(2860),
    [anon_sym_return] = ACTIONS(2860),
    [anon_sym_raw] = ACTIONS(2860),
    [anon_sym_static] = ACTIONS(2860),
    [anon_sym_struct] = ACTIONS(2860),
    [anon_sym_trait] = ACTIONS(2860),
    [anon_sym_type] = ACTIONS(2860),
    [anon_sym_union] = ACTIONS(2860),
    [anon_sym_unsafe] = ACTIONS(2860),
    [anon_sym_use] = ACTIONS(2860),
    [anon_sym_while] = ACTIONS(2860),
    [anon_sym_extern] = ACTIONS(2860),
    [anon_sym_safe] = ACTIONS(2860),
    [anon_sym_yield] = ACTIONS(2860),
    [anon_sym_move] = ACTIONS(2860),
    [anon_sym_try] = ACTIONS(2860),
    [sym_integer_literal] = ACTIONS(2858),
    [aux_sym_string_literal_token1] = ACTIONS(2858),
    [sym_char_literal] = ACTIONS(2858),
    [anon_sym_true] = ACTIONS(2860),
    [anon_sym_false] = ACTIONS(2860),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2860),
    [sym_super] = ACTIONS(2860),
    [sym_crate] = ACTIONS(2860),
    [aux_sym_metavariable_token1] = ACTIONS(2858),
    [sym__raw_string_literal_start] = ACTIONS(2858),
    [sym_float_literal] = ACTIONS(2858),
    [sym__outer_block_doc_comment_start] = ACTIONS(2858),
    [sym__outer_line_doc_comment_start] = ACTIONS(2858),
  },
  [STATE(772)] = {
    [sym_line_comment] = STATE(772),
    [sym_block_comment] = STATE(772),
    [ts_builtin_sym_end] = ACTIONS(2862),
    [sym_identifier] = ACTIONS(2864),
    [anon_sym_SEMI] = ACTIONS(2862),
    [anon_sym_macro] = ACTIONS(2864),
    [anon_sym_LPAREN] = ACTIONS(2862),
    [anon_sym_LBRACE] = ACTIONS(2862),
    [anon_sym_RBRACE] = ACTIONS(2862),
    [anon_sym_macro_rules] = ACTIONS(2864),
    [anon_sym_BANG] = ACTIONS(2862),
    [anon_sym_LBRACK] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2862),
    [anon_sym_u8] = ACTIONS(2864),
    [anon_sym_i8] = ACTIONS(2864),
    [anon_sym_u16] = ACTIONS(2864),
    [anon_sym_i16] = ACTIONS(2864),
    [anon_sym_u32] = ACTIONS(2864),
    [anon_sym_i32] = ACTIONS(2864),
    [anon_sym_u64] = ACTIONS(2864),
    [anon_sym_i64] = ACTIONS(2864),
    [anon_sym_u128] = ACTIONS(2864),
    [anon_sym_i128] = ACTIONS(2864),
    [anon_sym_isize] = ACTIONS(2864),
    [anon_sym_usize] = ACTIONS(2864),
    [anon_sym_f32] = ACTIONS(2864),
    [anon_sym_f64] = ACTIONS(2864),
    [anon_sym_bool] = ACTIONS(2864),
    [anon_sym_str] = ACTIONS(2864),
    [anon_sym_char] = ACTIONS(2864),
    [anon_sym_DASH] = ACTIONS(2862),
    [anon_sym_AMP] = ACTIONS(2862),
    [anon_sym_PIPE] = ACTIONS(2862),
    [anon_sym_LT] = ACTIONS(2862),
    [anon_sym_DOT_DOT] = ACTIONS(2862),
    [anon_sym_COLON_COLON] = ACTIONS(2862),
    [anon_sym_POUND] = ACTIONS(2862),
    [anon_sym_SQUOTE] = ACTIONS(2864),
    [anon_sym_async] = ACTIONS(2864),
    [anon_sym_auto] = ACTIONS(2864),
    [anon_sym_await] = ACTIONS(2864),
    [anon_sym_become] = ACTIONS(2864),
    [anon_sym_break] = ACTIONS(2864),
    [anon_sym_const] = ACTIONS(2864),
    [anon_sym_continue] = ACTIONS(2864),
    [anon_sym_default] = ACTIONS(2864),
    [anon_sym_enum] = ACTIONS(2864),
    [anon_sym_fn] = ACTIONS(2864),
    [anon_sym_for] = ACTIONS(2864),
    [anon_sym_gen] = ACTIONS(2864),
    [anon_sym_if] = ACTIONS(2864),
    [anon_sym_impl] = ACTIONS(2864),
    [anon_sym_let] = ACTIONS(2864),
    [anon_sym_loop] = ACTIONS(2864),
    [anon_sym_match] = ACTIONS(2864),
    [anon_sym_mod] = ACTIONS(2864),
    [anon_sym_pub] = ACTIONS(2864),
    [anon_sym_return] = ACTIONS(2864),
    [anon_sym_raw] = ACTIONS(2864),
    [anon_sym_static] = ACTIONS(2864),
    [anon_sym_struct] = ACTIONS(2864),
    [anon_sym_trait] = ACTIONS(2864),
    [anon_sym_type] = ACTIONS(2864),
    [anon_sym_union] = ACTIONS(2864),
    [anon_sym_unsafe] = ACTIONS(2864),
    [anon_sym_use] = ACTIONS(2864),
    [anon_sym_while] = ACTIONS(2864),
    [anon_sym_extern] = ACTIONS(2864),
    [anon_sym_safe] = ACTIONS(2864),
    [anon_sym_yield] = ACTIONS(2864),
    [anon_sym_move] = ACTIONS(2864),
    [anon_sym_try] = ACTIONS(2864),
    [sym_integer_literal] = ACTIONS(2862),
    [aux_sym_string_literal_token1] = ACTIONS(2862),
    [sym_char_literal] = ACTIONS(2862),
    [anon_sym_true] = ACTIONS(2864),
    [anon_sym_false] = ACTIONS(2864),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2864),
    [sym_super] = ACTIONS(2864),
    [sym_crate] = ACTIONS(2864),
    [aux_sym_metavariable_token1] = ACTIONS(2862),
    [sym__raw_string_literal_start] = ACTIONS(2862),
    [sym_float_literal] = ACTIONS(2862),
    [sym__outer_block_doc_comment_start] = ACTIONS(2862),
    [sym__outer_line_doc_comment_start] = ACTIONS(2862),
  },
  [STATE(773)] = {
    [sym_line_comment] = STATE(773),
    [sym_block_comment] = STATE(773),
    [ts_builtin_sym_end] = ACTIONS(2866),
    [sym_identifier] = ACTIONS(2868),
    [anon_sym_SEMI] = ACTIONS(2866),
    [anon_sym_macro] = ACTIONS(2868),
    [anon_sym_LPAREN] = ACTIONS(2866),
    [anon_sym_LBRACE] = ACTIONS(2866),
    [anon_sym_RBRACE] = ACTIONS(2866),
    [anon_sym_macro_rules] = ACTIONS(2868),
    [anon_sym_BANG] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2866),
    [anon_sym_STAR] = ACTIONS(2866),
    [anon_sym_u8] = ACTIONS(2868),
    [anon_sym_i8] = ACTIONS(2868),
    [anon_sym_u16] = ACTIONS(2868),
    [anon_sym_i16] = ACTIONS(2868),
    [anon_sym_u32] = ACTIONS(2868),
    [anon_sym_i32] = ACTIONS(2868),
    [anon_sym_u64] = ACTIONS(2868),
    [anon_sym_i64] = ACTIONS(2868),
    [anon_sym_u128] = ACTIONS(2868),
    [anon_sym_i128] = ACTIONS(2868),
    [anon_sym_isize] = ACTIONS(2868),
    [anon_sym_usize] = ACTIONS(2868),
    [anon_sym_f32] = ACTIONS(2868),
    [anon_sym_f64] = ACTIONS(2868),
    [anon_sym_bool] = ACTIONS(2868),
    [anon_sym_str] = ACTIONS(2868),
    [anon_sym_char] = ACTIONS(2868),
    [anon_sym_DASH] = ACTIONS(2866),
    [anon_sym_AMP] = ACTIONS(2866),
    [anon_sym_PIPE] = ACTIONS(2866),
    [anon_sym_LT] = ACTIONS(2866),
    [anon_sym_DOT_DOT] = ACTIONS(2866),
    [anon_sym_COLON_COLON] = ACTIONS(2866),
    [anon_sym_POUND] = ACTIONS(2866),
    [anon_sym_SQUOTE] = ACTIONS(2868),
    [anon_sym_async] = ACTIONS(2868),
    [anon_sym_auto] = ACTIONS(2868),
    [anon_sym_await] = ACTIONS(2868),
    [anon_sym_become] = ACTIONS(2868),
    [anon_sym_break] = ACTIONS(2868),
    [anon_sym_const] = ACTIONS(2868),
    [anon_sym_continue] = ACTIONS(2868),
    [anon_sym_default] = ACTIONS(2868),
    [anon_sym_enum] = ACTIONS(2868),
    [anon_sym_fn] = ACTIONS(2868),
    [anon_sym_for] = ACTIONS(2868),
    [anon_sym_gen] = ACTIONS(2868),
    [anon_sym_if] = ACTIONS(2868),
    [anon_sym_impl] = ACTIONS(2868),
    [anon_sym_let] = ACTIONS(2868),
    [anon_sym_loop] = ACTIONS(2868),
    [anon_sym_match] = ACTIONS(2868),
    [anon_sym_mod] = ACTIONS(2868),
    [anon_sym_pub] = ACTIONS(2868),
    [anon_sym_return] = ACTIONS(2868),
    [anon_sym_raw] = ACTIONS(2868),
    [anon_sym_static] = ACTIONS(2868),
    [anon_sym_struct] = ACTIONS(2868),
    [anon_sym_trait] = ACTIONS(2868),
    [anon_sym_type] = ACTIONS(2868),
    [anon_sym_union] = ACTIONS(2868),
    [anon_sym_unsafe] = ACTIONS(2868),
    [anon_sym_use] = ACTIONS(2868),
    [anon_sym_while] = ACTIONS(2868),
    [anon_sym_extern] = ACTIONS(2868),
    [anon_sym_safe] = ACTIONS(2868),
    [anon_sym_yield] = ACTIONS(2868),
    [anon_sym_move] = ACTIONS(2868),
    [anon_sym_try] = ACTIONS(2868),
    [sym_integer_literal] = ACTIONS(2866),
    [aux_sym_string_literal_token1] = ACTIONS(2866),
    [sym_char_literal] = ACTIONS(2866),
    [anon_sym_true] = ACTIONS(2868),
    [anon_sym_false] = ACTIONS(2868),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2868),
    [sym_super] = ACTIONS(2868),
    [sym_crate] = ACTIONS(2868),
    [aux_sym_metavariable_token1] = ACTIONS(2866),
    [sym__raw_string_literal_start] = ACTIONS(2866),
    [sym_float_literal] = ACTIONS(2866),
    [sym__outer_block_doc_comment_start] = ACTIONS(2866),
    [sym__outer_line_doc_comment_start] = ACTIONS(2866),
  },
  [STATE(774)] = {
    [sym_line_comment] = STATE(774),
    [sym_block_comment] = STATE(774),
    [ts_builtin_sym_end] = ACTIONS(2870),
    [sym_identifier] = ACTIONS(2872),
    [anon_sym_SEMI] = ACTIONS(2870),
    [anon_sym_macro] = ACTIONS(2872),
    [anon_sym_LPAREN] = ACTIONS(2870),
    [anon_sym_LBRACE] = ACTIONS(2870),
    [anon_sym_RBRACE] = ACTIONS(2870),
    [anon_sym_macro_rules] = ACTIONS(2872),
    [anon_sym_BANG] = ACTIONS(2870),
    [anon_sym_LBRACK] = ACTIONS(2870),
    [anon_sym_STAR] = ACTIONS(2870),
    [anon_sym_u8] = ACTIONS(2872),
    [anon_sym_i8] = ACTIONS(2872),
    [anon_sym_u16] = ACTIONS(2872),
    [anon_sym_i16] = ACTIONS(2872),
    [anon_sym_u32] = ACTIONS(2872),
    [anon_sym_i32] = ACTIONS(2872),
    [anon_sym_u64] = ACTIONS(2872),
    [anon_sym_i64] = ACTIONS(2872),
    [anon_sym_u128] = ACTIONS(2872),
    [anon_sym_i128] = ACTIONS(2872),
    [anon_sym_isize] = ACTIONS(2872),
    [anon_sym_usize] = ACTIONS(2872),
    [anon_sym_f32] = ACTIONS(2872),
    [anon_sym_f64] = ACTIONS(2872),
    [anon_sym_bool] = ACTIONS(2872),
    [anon_sym_str] = ACTIONS(2872),
    [anon_sym_char] = ACTIONS(2872),
    [anon_sym_DASH] = ACTIONS(2870),
    [anon_sym_AMP] = ACTIONS(2870),
    [anon_sym_PIPE] = ACTIONS(2870),
    [anon_sym_LT] = ACTIONS(2870),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [anon_sym_COLON_COLON] = ACTIONS(2870),
    [anon_sym_POUND] = ACTIONS(2870),
    [anon_sym_SQUOTE] = ACTIONS(2872),
    [anon_sym_async] = ACTIONS(2872),
    [anon_sym_auto] = ACTIONS(2872),
    [anon_sym_await] = ACTIONS(2872),
    [anon_sym_become] = ACTIONS(2872),
    [anon_sym_break] = ACTIONS(2872),
    [anon_sym_const] = ACTIONS(2872),
    [anon_sym_continue] = ACTIONS(2872),
    [anon_sym_default] = ACTIONS(2872),
    [anon_sym_enum] = ACTIONS(2872),
    [anon_sym_fn] = ACTIONS(2872),
    [anon_sym_for] = ACTIONS(2872),
    [anon_sym_gen] = ACTIONS(2872),
    [anon_sym_if] = ACTIONS(2872),
    [anon_sym_impl] = ACTIONS(2872),
    [anon_sym_let] = ACTIONS(2872),
    [anon_sym_loop] = ACTIONS(2872),
    [anon_sym_match] = ACTIONS(2872),
    [anon_sym_mod] = ACTIONS(2872),
    [anon_sym_pub] = ACTIONS(2872),
    [anon_sym_return] = ACTIONS(2872),
    [anon_sym_raw] = ACTIONS(2872),
    [anon_sym_static] = ACTIONS(2872),
    [anon_sym_struct] = ACTIONS(2872),
    [anon_sym_trait] = ACTIONS(2872),
    [anon_sym_type] = ACTIONS(2872),
    [anon_sym_union] = ACTIONS(2872),
    [anon_sym_unsafe] = ACTIONS(2872),
    [anon_sym_use] = ACTIONS(2872),
    [anon_sym_while] = ACTIONS(2872),
    [anon_sym_extern] = ACTIONS(2872),
    [anon_sym_safe] = ACTIONS(2872),
    [anon_sym_yield] = ACTIONS(2872),
    [anon_sym_move] = ACTIONS(2872),
    [anon_sym_try] = ACTIONS(2872),
    [sym_integer_literal] = ACTIONS(2870),
    [aux_sym_string_literal_token1] = ACTIONS(2870),
    [sym_char_literal] = ACTIONS(2870),
    [anon_sym_true] = ACTIONS(2872),
    [anon_sym_false] = ACTIONS(2872),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2872),
    [sym_super] = ACTIONS(2872),
    [sym_crate] = ACTIONS(2872),
    [aux_sym_metavariable_token1] = ACTIONS(2870),
    [sym__raw_string_literal_start] = ACTIONS(2870),
    [sym_float_literal] = ACTIONS(2870),
    [sym__outer_block_doc_comment_start] = ACTIONS(2870),
    [sym__outer_line_doc_comment_start] = ACTIONS(2870),
  },
  [STATE(775)] = {
    [sym_line_comment] = STATE(775),
    [sym_block_comment] = STATE(775),
    [ts_builtin_sym_end] = ACTIONS(2874),
    [sym_identifier] = ACTIONS(2876),
    [anon_sym_SEMI] = ACTIONS(2874),
    [anon_sym_macro] = ACTIONS(2876),
    [anon_sym_LPAREN] = ACTIONS(2874),
    [anon_sym_LBRACE] = ACTIONS(2874),
    [anon_sym_RBRACE] = ACTIONS(2874),
    [anon_sym_macro_rules] = ACTIONS(2876),
    [anon_sym_BANG] = ACTIONS(2874),
    [anon_sym_LBRACK] = ACTIONS(2874),
    [anon_sym_STAR] = ACTIONS(2874),
    [anon_sym_u8] = ACTIONS(2876),
    [anon_sym_i8] = ACTIONS(2876),
    [anon_sym_u16] = ACTIONS(2876),
    [anon_sym_i16] = ACTIONS(2876),
    [anon_sym_u32] = ACTIONS(2876),
    [anon_sym_i32] = ACTIONS(2876),
    [anon_sym_u64] = ACTIONS(2876),
    [anon_sym_i64] = ACTIONS(2876),
    [anon_sym_u128] = ACTIONS(2876),
    [anon_sym_i128] = ACTIONS(2876),
    [anon_sym_isize] = ACTIONS(2876),
    [anon_sym_usize] = ACTIONS(2876),
    [anon_sym_f32] = ACTIONS(2876),
    [anon_sym_f64] = ACTIONS(2876),
    [anon_sym_bool] = ACTIONS(2876),
    [anon_sym_str] = ACTIONS(2876),
    [anon_sym_char] = ACTIONS(2876),
    [anon_sym_DASH] = ACTIONS(2874),
    [anon_sym_AMP] = ACTIONS(2874),
    [anon_sym_PIPE] = ACTIONS(2874),
    [anon_sym_LT] = ACTIONS(2874),
    [anon_sym_DOT_DOT] = ACTIONS(2874),
    [anon_sym_COLON_COLON] = ACTIONS(2874),
    [anon_sym_POUND] = ACTIONS(2874),
    [anon_sym_SQUOTE] = ACTIONS(2876),
    [anon_sym_async] = ACTIONS(2876),
    [anon_sym_auto] = ACTIONS(2876),
    [anon_sym_await] = ACTIONS(2876),
    [anon_sym_become] = ACTIONS(2876),
    [anon_sym_break] = ACTIONS(2876),
    [anon_sym_const] = ACTIONS(2876),
    [anon_sym_continue] = ACTIONS(2876),
    [anon_sym_default] = ACTIONS(2876),
    [anon_sym_enum] = ACTIONS(2876),
    [anon_sym_fn] = ACTIONS(2876),
    [anon_sym_for] = ACTIONS(2876),
    [anon_sym_gen] = ACTIONS(2876),
    [anon_sym_if] = ACTIONS(2876),
    [anon_sym_impl] = ACTIONS(2876),
    [anon_sym_let] = ACTIONS(2876),
    [anon_sym_loop] = ACTIONS(2876),
    [anon_sym_match] = ACTIONS(2876),
    [anon_sym_mod] = ACTIONS(2876),
    [anon_sym_pub] = ACTIONS(2876),
    [anon_sym_return] = ACTIONS(2876),
    [anon_sym_raw] = ACTIONS(2876),
    [anon_sym_static] = ACTIONS(2876),
    [anon_sym_struct] = ACTIONS(2876),
    [anon_sym_trait] = ACTIONS(2876),
    [anon_sym_type] = ACTIONS(2876),
    [anon_sym_union] = ACTIONS(2876),
    [anon_sym_unsafe] = ACTIONS(2876),
    [anon_sym_use] = ACTIONS(2876),
    [anon_sym_while] = ACTIONS(2876),
    [anon_sym_extern] = ACTIONS(2876),
    [anon_sym_safe] = ACTIONS(2876),
    [anon_sym_yield] = ACTIONS(2876),
    [anon_sym_move] = ACTIONS(2876),
    [anon_sym_try] = ACTIONS(2876),
    [sym_integer_literal] = ACTIONS(2874),
    [aux_sym_string_literal_token1] = ACTIONS(2874),
    [sym_char_literal] = ACTIONS(2874),
    [anon_sym_true] = ACTIONS(2876),
    [anon_sym_false] = ACTIONS(2876),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2876),
    [sym_super] = ACTIONS(2876),
    [sym_crate] = ACTIONS(2876),
    [aux_sym_metavariable_token1] = ACTIONS(2874),
    [sym__raw_string_literal_start] = ACTIONS(2874),
    [sym_float_literal] = ACTIONS(2874),
    [sym__outer_block_doc_comment_start] = ACTIONS(2874),
    [sym__outer_line_doc_comment_start] = ACTIONS(2874),
  },
  [STATE(776)] = {
    [sym_line_comment] = STATE(776),
    [sym_block_comment] = STATE(776),
    [ts_builtin_sym_end] = ACTIONS(2878),
    [sym_identifier] = ACTIONS(2880),
    [anon_sym_SEMI] = ACTIONS(2878),
    [anon_sym_macro] = ACTIONS(2880),
    [anon_sym_LPAREN] = ACTIONS(2878),
    [anon_sym_LBRACE] = ACTIONS(2878),
    [anon_sym_RBRACE] = ACTIONS(2878),
    [anon_sym_macro_rules] = ACTIONS(2880),
    [anon_sym_BANG] = ACTIONS(2878),
    [anon_sym_LBRACK] = ACTIONS(2878),
    [anon_sym_STAR] = ACTIONS(2878),
    [anon_sym_u8] = ACTIONS(2880),
    [anon_sym_i8] = ACTIONS(2880),
    [anon_sym_u16] = ACTIONS(2880),
    [anon_sym_i16] = ACTIONS(2880),
    [anon_sym_u32] = ACTIONS(2880),
    [anon_sym_i32] = ACTIONS(2880),
    [anon_sym_u64] = ACTIONS(2880),
    [anon_sym_i64] = ACTIONS(2880),
    [anon_sym_u128] = ACTIONS(2880),
    [anon_sym_i128] = ACTIONS(2880),
    [anon_sym_isize] = ACTIONS(2880),
    [anon_sym_usize] = ACTIONS(2880),
    [anon_sym_f32] = ACTIONS(2880),
    [anon_sym_f64] = ACTIONS(2880),
    [anon_sym_bool] = ACTIONS(2880),
    [anon_sym_str] = ACTIONS(2880),
    [anon_sym_char] = ACTIONS(2880),
    [anon_sym_DASH] = ACTIONS(2878),
    [anon_sym_AMP] = ACTIONS(2878),
    [anon_sym_PIPE] = ACTIONS(2878),
    [anon_sym_LT] = ACTIONS(2878),
    [anon_sym_DOT_DOT] = ACTIONS(2878),
    [anon_sym_COLON_COLON] = ACTIONS(2878),
    [anon_sym_POUND] = ACTIONS(2878),
    [anon_sym_SQUOTE] = ACTIONS(2880),
    [anon_sym_async] = ACTIONS(2880),
    [anon_sym_auto] = ACTIONS(2880),
    [anon_sym_await] = ACTIONS(2880),
    [anon_sym_become] = ACTIONS(2880),
    [anon_sym_break] = ACTIONS(2880),
    [anon_sym_const] = ACTIONS(2880),
    [anon_sym_continue] = ACTIONS(2880),
    [anon_sym_default] = ACTIONS(2880),
    [anon_sym_enum] = ACTIONS(2880),
    [anon_sym_fn] = ACTIONS(2880),
    [anon_sym_for] = ACTIONS(2880),
    [anon_sym_gen] = ACTIONS(2880),
    [anon_sym_if] = ACTIONS(2880),
    [anon_sym_impl] = ACTIONS(2880),
    [anon_sym_let] = ACTIONS(2880),
    [anon_sym_loop] = ACTIONS(2880),
    [anon_sym_match] = ACTIONS(2880),
    [anon_sym_mod] = ACTIONS(2880),
    [anon_sym_pub] = ACTIONS(2880),
    [anon_sym_return] = ACTIONS(2880),
    [anon_sym_raw] = ACTIONS(2880),
    [anon_sym_static] = ACTIONS(2880),
    [anon_sym_struct] = ACTIONS(2880),
    [anon_sym_trait] = ACTIONS(2880),
    [anon_sym_type] = ACTIONS(2880),
    [anon_sym_union] = ACTIONS(2880),
    [anon_sym_unsafe] = ACTIONS(2880),
    [anon_sym_use] = ACTIONS(2880),
    [anon_sym_while] = ACTIONS(2880),
    [anon_sym_extern] = ACTIONS(2880),
    [anon_sym_safe] = ACTIONS(2880),
    [anon_sym_yield] = ACTIONS(2880),
    [anon_sym_move] = ACTIONS(2880),
    [anon_sym_try] = ACTIONS(2880),
    [sym_integer_literal] = ACTIONS(2878),
    [aux_sym_string_literal_token1] = ACTIONS(2878),
    [sym_char_literal] = ACTIONS(2878),
    [anon_sym_true] = ACTIONS(2880),
    [anon_sym_false] = ACTIONS(2880),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2880),
    [sym_super] = ACTIONS(2880),
    [sym_crate] = ACTIONS(2880),
    [aux_sym_metavariable_token1] = ACTIONS(2878),
    [sym__raw_string_literal_start] = ACTIONS(2878),
    [sym_float_literal] = ACTIONS(2878),
    [sym__outer_block_doc_comment_start] = ACTIONS(2878),
    [sym__outer_line_doc_comment_start] = ACTIONS(2878),
  },
  [STATE(777)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3935),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(777),
    [sym_block_comment] = STATE(777),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(2358),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(2358),
    [anon_sym_COMMA] = ACTIONS(2358),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(2358),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_EQ] = ACTIONS(2358),
    [anon_sym_GT] = ACTIONS(2358),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(2360),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(778)] = {
    [sym_line_comment] = STATE(778),
    [sym_block_comment] = STATE(778),
    [ts_builtin_sym_end] = ACTIONS(2882),
    [sym_identifier] = ACTIONS(2884),
    [anon_sym_SEMI] = ACTIONS(2882),
    [anon_sym_macro] = ACTIONS(2884),
    [anon_sym_LPAREN] = ACTIONS(2882),
    [anon_sym_LBRACE] = ACTIONS(2882),
    [anon_sym_RBRACE] = ACTIONS(2882),
    [anon_sym_macro_rules] = ACTIONS(2884),
    [anon_sym_BANG] = ACTIONS(2882),
    [anon_sym_LBRACK] = ACTIONS(2882),
    [anon_sym_STAR] = ACTIONS(2882),
    [anon_sym_u8] = ACTIONS(2884),
    [anon_sym_i8] = ACTIONS(2884),
    [anon_sym_u16] = ACTIONS(2884),
    [anon_sym_i16] = ACTIONS(2884),
    [anon_sym_u32] = ACTIONS(2884),
    [anon_sym_i32] = ACTIONS(2884),
    [anon_sym_u64] = ACTIONS(2884),
    [anon_sym_i64] = ACTIONS(2884),
    [anon_sym_u128] = ACTIONS(2884),
    [anon_sym_i128] = ACTIONS(2884),
    [anon_sym_isize] = ACTIONS(2884),
    [anon_sym_usize] = ACTIONS(2884),
    [anon_sym_f32] = ACTIONS(2884),
    [anon_sym_f64] = ACTIONS(2884),
    [anon_sym_bool] = ACTIONS(2884),
    [anon_sym_str] = ACTIONS(2884),
    [anon_sym_char] = ACTIONS(2884),
    [anon_sym_DASH] = ACTIONS(2882),
    [anon_sym_AMP] = ACTIONS(2882),
    [anon_sym_PIPE] = ACTIONS(2882),
    [anon_sym_LT] = ACTIONS(2882),
    [anon_sym_DOT_DOT] = ACTIONS(2882),
    [anon_sym_COLON_COLON] = ACTIONS(2882),
    [anon_sym_POUND] = ACTIONS(2882),
    [anon_sym_SQUOTE] = ACTIONS(2884),
    [anon_sym_async] = ACTIONS(2884),
    [anon_sym_auto] = ACTIONS(2884),
    [anon_sym_await] = ACTIONS(2884),
    [anon_sym_become] = ACTIONS(2884),
    [anon_sym_break] = ACTIONS(2884),
    [anon_sym_const] = ACTIONS(2884),
    [anon_sym_continue] = ACTIONS(2884),
    [anon_sym_default] = ACTIONS(2884),
    [anon_sym_enum] = ACTIONS(2884),
    [anon_sym_fn] = ACTIONS(2884),
    [anon_sym_for] = ACTIONS(2884),
    [anon_sym_gen] = ACTIONS(2884),
    [anon_sym_if] = ACTIONS(2884),
    [anon_sym_impl] = ACTIONS(2884),
    [anon_sym_let] = ACTIONS(2884),
    [anon_sym_loop] = ACTIONS(2884),
    [anon_sym_match] = ACTIONS(2884),
    [anon_sym_mod] = ACTIONS(2884),
    [anon_sym_pub] = ACTIONS(2884),
    [anon_sym_return] = ACTIONS(2884),
    [anon_sym_raw] = ACTIONS(2884),
    [anon_sym_static] = ACTIONS(2884),
    [anon_sym_struct] = ACTIONS(2884),
    [anon_sym_trait] = ACTIONS(2884),
    [anon_sym_type] = ACTIONS(2884),
    [anon_sym_union] = ACTIONS(2884),
    [anon_sym_unsafe] = ACTIONS(2884),
    [anon_sym_use] = ACTIONS(2884),
    [anon_sym_while] = ACTIONS(2884),
    [anon_sym_extern] = ACTIONS(2884),
    [anon_sym_safe] = ACTIONS(2884),
    [anon_sym_yield] = ACTIONS(2884),
    [anon_sym_move] = ACTIONS(2884),
    [anon_sym_try] = ACTIONS(2884),
    [sym_integer_literal] = ACTIONS(2882),
    [aux_sym_string_literal_token1] = ACTIONS(2882),
    [sym_char_literal] = ACTIONS(2882),
    [anon_sym_true] = ACTIONS(2884),
    [anon_sym_false] = ACTIONS(2884),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2884),
    [sym_super] = ACTIONS(2884),
    [sym_crate] = ACTIONS(2884),
    [aux_sym_metavariable_token1] = ACTIONS(2882),
    [sym__raw_string_literal_start] = ACTIONS(2882),
    [sym_float_literal] = ACTIONS(2882),
    [sym__outer_block_doc_comment_start] = ACTIONS(2882),
    [sym__outer_line_doc_comment_start] = ACTIONS(2882),
  },
  [STATE(779)] = {
    [sym_line_comment] = STATE(779),
    [sym_block_comment] = STATE(779),
    [ts_builtin_sym_end] = ACTIONS(2886),
    [sym_identifier] = ACTIONS(2888),
    [anon_sym_SEMI] = ACTIONS(2886),
    [anon_sym_macro] = ACTIONS(2888),
    [anon_sym_LPAREN] = ACTIONS(2886),
    [anon_sym_LBRACE] = ACTIONS(2886),
    [anon_sym_RBRACE] = ACTIONS(2886),
    [anon_sym_macro_rules] = ACTIONS(2888),
    [anon_sym_BANG] = ACTIONS(2886),
    [anon_sym_LBRACK] = ACTIONS(2886),
    [anon_sym_STAR] = ACTIONS(2886),
    [anon_sym_u8] = ACTIONS(2888),
    [anon_sym_i8] = ACTIONS(2888),
    [anon_sym_u16] = ACTIONS(2888),
    [anon_sym_i16] = ACTIONS(2888),
    [anon_sym_u32] = ACTIONS(2888),
    [anon_sym_i32] = ACTIONS(2888),
    [anon_sym_u64] = ACTIONS(2888),
    [anon_sym_i64] = ACTIONS(2888),
    [anon_sym_u128] = ACTIONS(2888),
    [anon_sym_i128] = ACTIONS(2888),
    [anon_sym_isize] = ACTIONS(2888),
    [anon_sym_usize] = ACTIONS(2888),
    [anon_sym_f32] = ACTIONS(2888),
    [anon_sym_f64] = ACTIONS(2888),
    [anon_sym_bool] = ACTIONS(2888),
    [anon_sym_str] = ACTIONS(2888),
    [anon_sym_char] = ACTIONS(2888),
    [anon_sym_DASH] = ACTIONS(2886),
    [anon_sym_AMP] = ACTIONS(2886),
    [anon_sym_PIPE] = ACTIONS(2886),
    [anon_sym_LT] = ACTIONS(2886),
    [anon_sym_DOT_DOT] = ACTIONS(2886),
    [anon_sym_COLON_COLON] = ACTIONS(2886),
    [anon_sym_POUND] = ACTIONS(2886),
    [anon_sym_SQUOTE] = ACTIONS(2888),
    [anon_sym_async] = ACTIONS(2888),
    [anon_sym_auto] = ACTIONS(2888),
    [anon_sym_await] = ACTIONS(2888),
    [anon_sym_become] = ACTIONS(2888),
    [anon_sym_break] = ACTIONS(2888),
    [anon_sym_const] = ACTIONS(2888),
    [anon_sym_continue] = ACTIONS(2888),
    [anon_sym_default] = ACTIONS(2888),
    [anon_sym_enum] = ACTIONS(2888),
    [anon_sym_fn] = ACTIONS(2888),
    [anon_sym_for] = ACTIONS(2888),
    [anon_sym_gen] = ACTIONS(2888),
    [anon_sym_if] = ACTIONS(2888),
    [anon_sym_impl] = ACTIONS(2888),
    [anon_sym_let] = ACTIONS(2888),
    [anon_sym_loop] = ACTIONS(2888),
    [anon_sym_match] = ACTIONS(2888),
    [anon_sym_mod] = ACTIONS(2888),
    [anon_sym_pub] = ACTIONS(2888),
    [anon_sym_return] = ACTIONS(2888),
    [anon_sym_raw] = ACTIONS(2888),
    [anon_sym_static] = ACTIONS(2888),
    [anon_sym_struct] = ACTIONS(2888),
    [anon_sym_trait] = ACTIONS(2888),
    [anon_sym_type] = ACTIONS(2888),
    [anon_sym_union] = ACTIONS(2888),
    [anon_sym_unsafe] = ACTIONS(2888),
    [anon_sym_use] = ACTIONS(2888),
    [anon_sym_while] = ACTIONS(2888),
    [anon_sym_extern] = ACTIONS(2888),
    [anon_sym_safe] = ACTIONS(2888),
    [anon_sym_yield] = ACTIONS(2888),
    [anon_sym_move] = ACTIONS(2888),
    [anon_sym_try] = ACTIONS(2888),
    [sym_integer_literal] = ACTIONS(2886),
    [aux_sym_string_literal_token1] = ACTIONS(2886),
    [sym_char_literal] = ACTIONS(2886),
    [anon_sym_true] = ACTIONS(2888),
    [anon_sym_false] = ACTIONS(2888),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2888),
    [sym_super] = ACTIONS(2888),
    [sym_crate] = ACTIONS(2888),
    [aux_sym_metavariable_token1] = ACTIONS(2886),
    [sym__raw_string_literal_start] = ACTIONS(2886),
    [sym_float_literal] = ACTIONS(2886),
    [sym__outer_block_doc_comment_start] = ACTIONS(2886),
    [sym__outer_line_doc_comment_start] = ACTIONS(2886),
  },
  [STATE(780)] = {
    [sym_line_comment] = STATE(780),
    [sym_block_comment] = STATE(780),
    [ts_builtin_sym_end] = ACTIONS(2890),
    [sym_identifier] = ACTIONS(2892),
    [anon_sym_SEMI] = ACTIONS(2890),
    [anon_sym_macro] = ACTIONS(2892),
    [anon_sym_LPAREN] = ACTIONS(2890),
    [anon_sym_LBRACE] = ACTIONS(2890),
    [anon_sym_RBRACE] = ACTIONS(2890),
    [anon_sym_macro_rules] = ACTIONS(2892),
    [anon_sym_BANG] = ACTIONS(2890),
    [anon_sym_LBRACK] = ACTIONS(2890),
    [anon_sym_STAR] = ACTIONS(2890),
    [anon_sym_u8] = ACTIONS(2892),
    [anon_sym_i8] = ACTIONS(2892),
    [anon_sym_u16] = ACTIONS(2892),
    [anon_sym_i16] = ACTIONS(2892),
    [anon_sym_u32] = ACTIONS(2892),
    [anon_sym_i32] = ACTIONS(2892),
    [anon_sym_u64] = ACTIONS(2892),
    [anon_sym_i64] = ACTIONS(2892),
    [anon_sym_u128] = ACTIONS(2892),
    [anon_sym_i128] = ACTIONS(2892),
    [anon_sym_isize] = ACTIONS(2892),
    [anon_sym_usize] = ACTIONS(2892),
    [anon_sym_f32] = ACTIONS(2892),
    [anon_sym_f64] = ACTIONS(2892),
    [anon_sym_bool] = ACTIONS(2892),
    [anon_sym_str] = ACTIONS(2892),
    [anon_sym_char] = ACTIONS(2892),
    [anon_sym_DASH] = ACTIONS(2890),
    [anon_sym_AMP] = ACTIONS(2890),
    [anon_sym_PIPE] = ACTIONS(2890),
    [anon_sym_LT] = ACTIONS(2890),
    [anon_sym_DOT_DOT] = ACTIONS(2890),
    [anon_sym_COLON_COLON] = ACTIONS(2890),
    [anon_sym_POUND] = ACTIONS(2890),
    [anon_sym_SQUOTE] = ACTIONS(2892),
    [anon_sym_async] = ACTIONS(2892),
    [anon_sym_auto] = ACTIONS(2892),
    [anon_sym_await] = ACTIONS(2892),
    [anon_sym_become] = ACTIONS(2892),
    [anon_sym_break] = ACTIONS(2892),
    [anon_sym_const] = ACTIONS(2892),
    [anon_sym_continue] = ACTIONS(2892),
    [anon_sym_default] = ACTIONS(2892),
    [anon_sym_enum] = ACTIONS(2892),
    [anon_sym_fn] = ACTIONS(2892),
    [anon_sym_for] = ACTIONS(2892),
    [anon_sym_gen] = ACTIONS(2892),
    [anon_sym_if] = ACTIONS(2892),
    [anon_sym_impl] = ACTIONS(2892),
    [anon_sym_let] = ACTIONS(2892),
    [anon_sym_loop] = ACTIONS(2892),
    [anon_sym_match] = ACTIONS(2892),
    [anon_sym_mod] = ACTIONS(2892),
    [anon_sym_pub] = ACTIONS(2892),
    [anon_sym_return] = ACTIONS(2892),
    [anon_sym_raw] = ACTIONS(2892),
    [anon_sym_static] = ACTIONS(2892),
    [anon_sym_struct] = ACTIONS(2892),
    [anon_sym_trait] = ACTIONS(2892),
    [anon_sym_type] = ACTIONS(2892),
    [anon_sym_union] = ACTIONS(2892),
    [anon_sym_unsafe] = ACTIONS(2892),
    [anon_sym_use] = ACTIONS(2892),
    [anon_sym_while] = ACTIONS(2892),
    [anon_sym_extern] = ACTIONS(2892),
    [anon_sym_safe] = ACTIONS(2892),
    [anon_sym_yield] = ACTIONS(2892),
    [anon_sym_move] = ACTIONS(2892),
    [anon_sym_try] = ACTIONS(2892),
    [sym_integer_literal] = ACTIONS(2890),
    [aux_sym_string_literal_token1] = ACTIONS(2890),
    [sym_char_literal] = ACTIONS(2890),
    [anon_sym_true] = ACTIONS(2892),
    [anon_sym_false] = ACTIONS(2892),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2892),
    [sym_super] = ACTIONS(2892),
    [sym_crate] = ACTIONS(2892),
    [aux_sym_metavariable_token1] = ACTIONS(2890),
    [sym__raw_string_literal_start] = ACTIONS(2890),
    [sym_float_literal] = ACTIONS(2890),
    [sym__outer_block_doc_comment_start] = ACTIONS(2890),
    [sym__outer_line_doc_comment_start] = ACTIONS(2890),
  },
  [STATE(781)] = {
    [sym_line_comment] = STATE(781),
    [sym_block_comment] = STATE(781),
    [ts_builtin_sym_end] = ACTIONS(2894),
    [sym_identifier] = ACTIONS(2896),
    [anon_sym_SEMI] = ACTIONS(2894),
    [anon_sym_macro] = ACTIONS(2896),
    [anon_sym_LPAREN] = ACTIONS(2894),
    [anon_sym_LBRACE] = ACTIONS(2894),
    [anon_sym_RBRACE] = ACTIONS(2894),
    [anon_sym_macro_rules] = ACTIONS(2896),
    [anon_sym_BANG] = ACTIONS(2894),
    [anon_sym_LBRACK] = ACTIONS(2894),
    [anon_sym_STAR] = ACTIONS(2894),
    [anon_sym_u8] = ACTIONS(2896),
    [anon_sym_i8] = ACTIONS(2896),
    [anon_sym_u16] = ACTIONS(2896),
    [anon_sym_i16] = ACTIONS(2896),
    [anon_sym_u32] = ACTIONS(2896),
    [anon_sym_i32] = ACTIONS(2896),
    [anon_sym_u64] = ACTIONS(2896),
    [anon_sym_i64] = ACTIONS(2896),
    [anon_sym_u128] = ACTIONS(2896),
    [anon_sym_i128] = ACTIONS(2896),
    [anon_sym_isize] = ACTIONS(2896),
    [anon_sym_usize] = ACTIONS(2896),
    [anon_sym_f32] = ACTIONS(2896),
    [anon_sym_f64] = ACTIONS(2896),
    [anon_sym_bool] = ACTIONS(2896),
    [anon_sym_str] = ACTIONS(2896),
    [anon_sym_char] = ACTIONS(2896),
    [anon_sym_DASH] = ACTIONS(2894),
    [anon_sym_AMP] = ACTIONS(2894),
    [anon_sym_PIPE] = ACTIONS(2894),
    [anon_sym_LT] = ACTIONS(2894),
    [anon_sym_DOT_DOT] = ACTIONS(2894),
    [anon_sym_COLON_COLON] = ACTIONS(2894),
    [anon_sym_POUND] = ACTIONS(2894),
    [anon_sym_SQUOTE] = ACTIONS(2896),
    [anon_sym_async] = ACTIONS(2896),
    [anon_sym_auto] = ACTIONS(2896),
    [anon_sym_await] = ACTIONS(2896),
    [anon_sym_become] = ACTIONS(2896),
    [anon_sym_break] = ACTIONS(2896),
    [anon_sym_const] = ACTIONS(2896),
    [anon_sym_continue] = ACTIONS(2896),
    [anon_sym_default] = ACTIONS(2896),
    [anon_sym_enum] = ACTIONS(2896),
    [anon_sym_fn] = ACTIONS(2896),
    [anon_sym_for] = ACTIONS(2896),
    [anon_sym_gen] = ACTIONS(2896),
    [anon_sym_if] = ACTIONS(2896),
    [anon_sym_impl] = ACTIONS(2896),
    [anon_sym_let] = ACTIONS(2896),
    [anon_sym_loop] = ACTIONS(2896),
    [anon_sym_match] = ACTIONS(2896),
    [anon_sym_mod] = ACTIONS(2896),
    [anon_sym_pub] = ACTIONS(2896),
    [anon_sym_return] = ACTIONS(2896),
    [anon_sym_raw] = ACTIONS(2896),
    [anon_sym_static] = ACTIONS(2896),
    [anon_sym_struct] = ACTIONS(2896),
    [anon_sym_trait] = ACTIONS(2896),
    [anon_sym_type] = ACTIONS(2896),
    [anon_sym_union] = ACTIONS(2896),
    [anon_sym_unsafe] = ACTIONS(2896),
    [anon_sym_use] = ACTIONS(2896),
    [anon_sym_while] = ACTIONS(2896),
    [anon_sym_extern] = ACTIONS(2896),
    [anon_sym_safe] = ACTIONS(2896),
    [anon_sym_yield] = ACTIONS(2896),
    [anon_sym_move] = ACTIONS(2896),
    [anon_sym_try] = ACTIONS(2896),
    [sym_integer_literal] = ACTIONS(2894),
    [aux_sym_string_literal_token1] = ACTIONS(2894),
    [sym_char_literal] = ACTIONS(2894),
    [anon_sym_true] = ACTIONS(2896),
    [anon_sym_false] = ACTIONS(2896),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2896),
    [sym_super] = ACTIONS(2896),
    [sym_crate] = ACTIONS(2896),
    [aux_sym_metavariable_token1] = ACTIONS(2894),
    [sym__raw_string_literal_start] = ACTIONS(2894),
    [sym_float_literal] = ACTIONS(2894),
    [sym__outer_block_doc_comment_start] = ACTIONS(2894),
    [sym__outer_line_doc_comment_start] = ACTIONS(2894),
  },
  [STATE(782)] = {
    [sym_line_comment] = STATE(782),
    [sym_block_comment] = STATE(782),
    [ts_builtin_sym_end] = ACTIONS(2898),
    [sym_identifier] = ACTIONS(2900),
    [anon_sym_SEMI] = ACTIONS(2898),
    [anon_sym_macro] = ACTIONS(2900),
    [anon_sym_LPAREN] = ACTIONS(2898),
    [anon_sym_LBRACE] = ACTIONS(2898),
    [anon_sym_RBRACE] = ACTIONS(2898),
    [anon_sym_macro_rules] = ACTIONS(2900),
    [anon_sym_BANG] = ACTIONS(2898),
    [anon_sym_LBRACK] = ACTIONS(2898),
    [anon_sym_STAR] = ACTIONS(2898),
    [anon_sym_u8] = ACTIONS(2900),
    [anon_sym_i8] = ACTIONS(2900),
    [anon_sym_u16] = ACTIONS(2900),
    [anon_sym_i16] = ACTIONS(2900),
    [anon_sym_u32] = ACTIONS(2900),
    [anon_sym_i32] = ACTIONS(2900),
    [anon_sym_u64] = ACTIONS(2900),
    [anon_sym_i64] = ACTIONS(2900),
    [anon_sym_u128] = ACTIONS(2900),
    [anon_sym_i128] = ACTIONS(2900),
    [anon_sym_isize] = ACTIONS(2900),
    [anon_sym_usize] = ACTIONS(2900),
    [anon_sym_f32] = ACTIONS(2900),
    [anon_sym_f64] = ACTIONS(2900),
    [anon_sym_bool] = ACTIONS(2900),
    [anon_sym_str] = ACTIONS(2900),
    [anon_sym_char] = ACTIONS(2900),
    [anon_sym_DASH] = ACTIONS(2898),
    [anon_sym_AMP] = ACTIONS(2898),
    [anon_sym_PIPE] = ACTIONS(2898),
    [anon_sym_LT] = ACTIONS(2898),
    [anon_sym_DOT_DOT] = ACTIONS(2898),
    [anon_sym_COLON_COLON] = ACTIONS(2898),
    [anon_sym_POUND] = ACTIONS(2898),
    [anon_sym_SQUOTE] = ACTIONS(2900),
    [anon_sym_async] = ACTIONS(2900),
    [anon_sym_auto] = ACTIONS(2900),
    [anon_sym_await] = ACTIONS(2900),
    [anon_sym_become] = ACTIONS(2900),
    [anon_sym_break] = ACTIONS(2900),
    [anon_sym_const] = ACTIONS(2900),
    [anon_sym_continue] = ACTIONS(2900),
    [anon_sym_default] = ACTIONS(2900),
    [anon_sym_enum] = ACTIONS(2900),
    [anon_sym_fn] = ACTIONS(2900),
    [anon_sym_for] = ACTIONS(2900),
    [anon_sym_gen] = ACTIONS(2900),
    [anon_sym_if] = ACTIONS(2900),
    [anon_sym_impl] = ACTIONS(2900),
    [anon_sym_let] = ACTIONS(2900),
    [anon_sym_loop] = ACTIONS(2900),
    [anon_sym_match] = ACTIONS(2900),
    [anon_sym_mod] = ACTIONS(2900),
    [anon_sym_pub] = ACTIONS(2900),
    [anon_sym_return] = ACTIONS(2900),
    [anon_sym_raw] = ACTIONS(2900),
    [anon_sym_static] = ACTIONS(2900),
    [anon_sym_struct] = ACTIONS(2900),
    [anon_sym_trait] = ACTIONS(2900),
    [anon_sym_type] = ACTIONS(2900),
    [anon_sym_union] = ACTIONS(2900),
    [anon_sym_unsafe] = ACTIONS(2900),
    [anon_sym_use] = ACTIONS(2900),
    [anon_sym_while] = ACTIONS(2900),
    [anon_sym_extern] = ACTIONS(2900),
    [anon_sym_safe] = ACTIONS(2900),
    [anon_sym_yield] = ACTIONS(2900),
    [anon_sym_move] = ACTIONS(2900),
    [anon_sym_try] = ACTIONS(2900),
    [sym_integer_literal] = ACTIONS(2898),
    [aux_sym_string_literal_token1] = ACTIONS(2898),
    [sym_char_literal] = ACTIONS(2898),
    [anon_sym_true] = ACTIONS(2900),
    [anon_sym_false] = ACTIONS(2900),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2900),
    [sym_super] = ACTIONS(2900),
    [sym_crate] = ACTIONS(2900),
    [aux_sym_metavariable_token1] = ACTIONS(2898),
    [sym__raw_string_literal_start] = ACTIONS(2898),
    [sym_float_literal] = ACTIONS(2898),
    [sym__outer_block_doc_comment_start] = ACTIONS(2898),
    [sym__outer_line_doc_comment_start] = ACTIONS(2898),
  },
  [STATE(783)] = {
    [sym_line_comment] = STATE(783),
    [sym_block_comment] = STATE(783),
    [ts_builtin_sym_end] = ACTIONS(2902),
    [sym_identifier] = ACTIONS(2904),
    [anon_sym_SEMI] = ACTIONS(2902),
    [anon_sym_macro] = ACTIONS(2904),
    [anon_sym_LPAREN] = ACTIONS(2902),
    [anon_sym_LBRACE] = ACTIONS(2902),
    [anon_sym_RBRACE] = ACTIONS(2902),
    [anon_sym_macro_rules] = ACTIONS(2904),
    [anon_sym_BANG] = ACTIONS(2902),
    [anon_sym_LBRACK] = ACTIONS(2902),
    [anon_sym_STAR] = ACTIONS(2902),
    [anon_sym_u8] = ACTIONS(2904),
    [anon_sym_i8] = ACTIONS(2904),
    [anon_sym_u16] = ACTIONS(2904),
    [anon_sym_i16] = ACTIONS(2904),
    [anon_sym_u32] = ACTIONS(2904),
    [anon_sym_i32] = ACTIONS(2904),
    [anon_sym_u64] = ACTIONS(2904),
    [anon_sym_i64] = ACTIONS(2904),
    [anon_sym_u128] = ACTIONS(2904),
    [anon_sym_i128] = ACTIONS(2904),
    [anon_sym_isize] = ACTIONS(2904),
    [anon_sym_usize] = ACTIONS(2904),
    [anon_sym_f32] = ACTIONS(2904),
    [anon_sym_f64] = ACTIONS(2904),
    [anon_sym_bool] = ACTIONS(2904),
    [anon_sym_str] = ACTIONS(2904),
    [anon_sym_char] = ACTIONS(2904),
    [anon_sym_DASH] = ACTIONS(2902),
    [anon_sym_AMP] = ACTIONS(2902),
    [anon_sym_PIPE] = ACTIONS(2902),
    [anon_sym_LT] = ACTIONS(2902),
    [anon_sym_DOT_DOT] = ACTIONS(2902),
    [anon_sym_COLON_COLON] = ACTIONS(2902),
    [anon_sym_POUND] = ACTIONS(2902),
    [anon_sym_SQUOTE] = ACTIONS(2904),
    [anon_sym_async] = ACTIONS(2904),
    [anon_sym_auto] = ACTIONS(2904),
    [anon_sym_await] = ACTIONS(2904),
    [anon_sym_become] = ACTIONS(2904),
    [anon_sym_break] = ACTIONS(2904),
    [anon_sym_const] = ACTIONS(2904),
    [anon_sym_continue] = ACTIONS(2904),
    [anon_sym_default] = ACTIONS(2904),
    [anon_sym_enum] = ACTIONS(2904),
    [anon_sym_fn] = ACTIONS(2904),
    [anon_sym_for] = ACTIONS(2904),
    [anon_sym_gen] = ACTIONS(2904),
    [anon_sym_if] = ACTIONS(2904),
    [anon_sym_impl] = ACTIONS(2904),
    [anon_sym_let] = ACTIONS(2904),
    [anon_sym_loop] = ACTIONS(2904),
    [anon_sym_match] = ACTIONS(2904),
    [anon_sym_mod] = ACTIONS(2904),
    [anon_sym_pub] = ACTIONS(2904),
    [anon_sym_return] = ACTIONS(2904),
    [anon_sym_raw] = ACTIONS(2904),
    [anon_sym_static] = ACTIONS(2904),
    [anon_sym_struct] = ACTIONS(2904),
    [anon_sym_trait] = ACTIONS(2904),
    [anon_sym_type] = ACTIONS(2904),
    [anon_sym_union] = ACTIONS(2904),
    [anon_sym_unsafe] = ACTIONS(2904),
    [anon_sym_use] = ACTIONS(2904),
    [anon_sym_while] = ACTIONS(2904),
    [anon_sym_extern] = ACTIONS(2904),
    [anon_sym_safe] = ACTIONS(2904),
    [anon_sym_yield] = ACTIONS(2904),
    [anon_sym_move] = ACTIONS(2904),
    [anon_sym_try] = ACTIONS(2904),
    [sym_integer_literal] = ACTIONS(2902),
    [aux_sym_string_literal_token1] = ACTIONS(2902),
    [sym_char_literal] = ACTIONS(2902),
    [anon_sym_true] = ACTIONS(2904),
    [anon_sym_false] = ACTIONS(2904),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2904),
    [sym_super] = ACTIONS(2904),
    [sym_crate] = ACTIONS(2904),
    [aux_sym_metavariable_token1] = ACTIONS(2902),
    [sym__raw_string_literal_start] = ACTIONS(2902),
    [sym_float_literal] = ACTIONS(2902),
    [sym__outer_block_doc_comment_start] = ACTIONS(2902),
    [sym__outer_line_doc_comment_start] = ACTIONS(2902),
  },
  [STATE(784)] = {
    [sym_line_comment] = STATE(784),
    [sym_block_comment] = STATE(784),
    [ts_builtin_sym_end] = ACTIONS(2906),
    [sym_identifier] = ACTIONS(2908),
    [anon_sym_SEMI] = ACTIONS(2906),
    [anon_sym_macro] = ACTIONS(2908),
    [anon_sym_LPAREN] = ACTIONS(2906),
    [anon_sym_LBRACE] = ACTIONS(2906),
    [anon_sym_RBRACE] = ACTIONS(2906),
    [anon_sym_macro_rules] = ACTIONS(2908),
    [anon_sym_BANG] = ACTIONS(2906),
    [anon_sym_LBRACK] = ACTIONS(2906),
    [anon_sym_STAR] = ACTIONS(2906),
    [anon_sym_u8] = ACTIONS(2908),
    [anon_sym_i8] = ACTIONS(2908),
    [anon_sym_u16] = ACTIONS(2908),
    [anon_sym_i16] = ACTIONS(2908),
    [anon_sym_u32] = ACTIONS(2908),
    [anon_sym_i32] = ACTIONS(2908),
    [anon_sym_u64] = ACTIONS(2908),
    [anon_sym_i64] = ACTIONS(2908),
    [anon_sym_u128] = ACTIONS(2908),
    [anon_sym_i128] = ACTIONS(2908),
    [anon_sym_isize] = ACTIONS(2908),
    [anon_sym_usize] = ACTIONS(2908),
    [anon_sym_f32] = ACTIONS(2908),
    [anon_sym_f64] = ACTIONS(2908),
    [anon_sym_bool] = ACTIONS(2908),
    [anon_sym_str] = ACTIONS(2908),
    [anon_sym_char] = ACTIONS(2908),
    [anon_sym_DASH] = ACTIONS(2906),
    [anon_sym_AMP] = ACTIONS(2906),
    [anon_sym_PIPE] = ACTIONS(2906),
    [anon_sym_LT] = ACTIONS(2906),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [anon_sym_COLON_COLON] = ACTIONS(2906),
    [anon_sym_POUND] = ACTIONS(2906),
    [anon_sym_SQUOTE] = ACTIONS(2908),
    [anon_sym_async] = ACTIONS(2908),
    [anon_sym_auto] = ACTIONS(2908),
    [anon_sym_await] = ACTIONS(2908),
    [anon_sym_become] = ACTIONS(2908),
    [anon_sym_break] = ACTIONS(2908),
    [anon_sym_const] = ACTIONS(2908),
    [anon_sym_continue] = ACTIONS(2908),
    [anon_sym_default] = ACTIONS(2908),
    [anon_sym_enum] = ACTIONS(2908),
    [anon_sym_fn] = ACTIONS(2908),
    [anon_sym_for] = ACTIONS(2908),
    [anon_sym_gen] = ACTIONS(2908),
    [anon_sym_if] = ACTIONS(2908),
    [anon_sym_impl] = ACTIONS(2908),
    [anon_sym_let] = ACTIONS(2908),
    [anon_sym_loop] = ACTIONS(2908),
    [anon_sym_match] = ACTIONS(2908),
    [anon_sym_mod] = ACTIONS(2908),
    [anon_sym_pub] = ACTIONS(2908),
    [anon_sym_return] = ACTIONS(2908),
    [anon_sym_raw] = ACTIONS(2908),
    [anon_sym_static] = ACTIONS(2908),
    [anon_sym_struct] = ACTIONS(2908),
    [anon_sym_trait] = ACTIONS(2908),
    [anon_sym_type] = ACTIONS(2908),
    [anon_sym_union] = ACTIONS(2908),
    [anon_sym_unsafe] = ACTIONS(2908),
    [anon_sym_use] = ACTIONS(2908),
    [anon_sym_while] = ACTIONS(2908),
    [anon_sym_extern] = ACTIONS(2908),
    [anon_sym_safe] = ACTIONS(2908),
    [anon_sym_yield] = ACTIONS(2908),
    [anon_sym_move] = ACTIONS(2908),
    [anon_sym_try] = ACTIONS(2908),
    [sym_integer_literal] = ACTIONS(2906),
    [aux_sym_string_literal_token1] = ACTIONS(2906),
    [sym_char_literal] = ACTIONS(2906),
    [anon_sym_true] = ACTIONS(2908),
    [anon_sym_false] = ACTIONS(2908),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2908),
    [sym_super] = ACTIONS(2908),
    [sym_crate] = ACTIONS(2908),
    [aux_sym_metavariable_token1] = ACTIONS(2906),
    [sym__raw_string_literal_start] = ACTIONS(2906),
    [sym_float_literal] = ACTIONS(2906),
    [sym__outer_block_doc_comment_start] = ACTIONS(2906),
    [sym__outer_line_doc_comment_start] = ACTIONS(2906),
  },
  [STATE(785)] = {
    [sym_line_comment] = STATE(785),
    [sym_block_comment] = STATE(785),
    [ts_builtin_sym_end] = ACTIONS(2910),
    [sym_identifier] = ACTIONS(2912),
    [anon_sym_SEMI] = ACTIONS(2910),
    [anon_sym_macro] = ACTIONS(2912),
    [anon_sym_LPAREN] = ACTIONS(2910),
    [anon_sym_LBRACE] = ACTIONS(2910),
    [anon_sym_RBRACE] = ACTIONS(2910),
    [anon_sym_macro_rules] = ACTIONS(2912),
    [anon_sym_BANG] = ACTIONS(2910),
    [anon_sym_LBRACK] = ACTIONS(2910),
    [anon_sym_STAR] = ACTIONS(2910),
    [anon_sym_u8] = ACTIONS(2912),
    [anon_sym_i8] = ACTIONS(2912),
    [anon_sym_u16] = ACTIONS(2912),
    [anon_sym_i16] = ACTIONS(2912),
    [anon_sym_u32] = ACTIONS(2912),
    [anon_sym_i32] = ACTIONS(2912),
    [anon_sym_u64] = ACTIONS(2912),
    [anon_sym_i64] = ACTIONS(2912),
    [anon_sym_u128] = ACTIONS(2912),
    [anon_sym_i128] = ACTIONS(2912),
    [anon_sym_isize] = ACTIONS(2912),
    [anon_sym_usize] = ACTIONS(2912),
    [anon_sym_f32] = ACTIONS(2912),
    [anon_sym_f64] = ACTIONS(2912),
    [anon_sym_bool] = ACTIONS(2912),
    [anon_sym_str] = ACTIONS(2912),
    [anon_sym_char] = ACTIONS(2912),
    [anon_sym_DASH] = ACTIONS(2910),
    [anon_sym_AMP] = ACTIONS(2910),
    [anon_sym_PIPE] = ACTIONS(2910),
    [anon_sym_LT] = ACTIONS(2910),
    [anon_sym_DOT_DOT] = ACTIONS(2910),
    [anon_sym_COLON_COLON] = ACTIONS(2910),
    [anon_sym_POUND] = ACTIONS(2910),
    [anon_sym_SQUOTE] = ACTIONS(2912),
    [anon_sym_async] = ACTIONS(2912),
    [anon_sym_auto] = ACTIONS(2912),
    [anon_sym_await] = ACTIONS(2912),
    [anon_sym_become] = ACTIONS(2912),
    [anon_sym_break] = ACTIONS(2912),
    [anon_sym_const] = ACTIONS(2912),
    [anon_sym_continue] = ACTIONS(2912),
    [anon_sym_default] = ACTIONS(2912),
    [anon_sym_enum] = ACTIONS(2912),
    [anon_sym_fn] = ACTIONS(2912),
    [anon_sym_for] = ACTIONS(2912),
    [anon_sym_gen] = ACTIONS(2912),
    [anon_sym_if] = ACTIONS(2912),
    [anon_sym_impl] = ACTIONS(2912),
    [anon_sym_let] = ACTIONS(2912),
    [anon_sym_loop] = ACTIONS(2912),
    [anon_sym_match] = ACTIONS(2912),
    [anon_sym_mod] = ACTIONS(2912),
    [anon_sym_pub] = ACTIONS(2912),
    [anon_sym_return] = ACTIONS(2912),
    [anon_sym_raw] = ACTIONS(2912),
    [anon_sym_static] = ACTIONS(2912),
    [anon_sym_struct] = ACTIONS(2912),
    [anon_sym_trait] = ACTIONS(2912),
    [anon_sym_type] = ACTIONS(2912),
    [anon_sym_union] = ACTIONS(2912),
    [anon_sym_unsafe] = ACTIONS(2912),
    [anon_sym_use] = ACTIONS(2912),
    [anon_sym_while] = ACTIONS(2912),
    [anon_sym_extern] = ACTIONS(2912),
    [anon_sym_safe] = ACTIONS(2912),
    [anon_sym_yield] = ACTIONS(2912),
    [anon_sym_move] = ACTIONS(2912),
    [anon_sym_try] = ACTIONS(2912),
    [sym_integer_literal] = ACTIONS(2910),
    [aux_sym_string_literal_token1] = ACTIONS(2910),
    [sym_char_literal] = ACTIONS(2910),
    [anon_sym_true] = ACTIONS(2912),
    [anon_sym_false] = ACTIONS(2912),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2912),
    [sym_super] = ACTIONS(2912),
    [sym_crate] = ACTIONS(2912),
    [aux_sym_metavariable_token1] = ACTIONS(2910),
    [sym__raw_string_literal_start] = ACTIONS(2910),
    [sym_float_literal] = ACTIONS(2910),
    [sym__outer_block_doc_comment_start] = ACTIONS(2910),
    [sym__outer_line_doc_comment_start] = ACTIONS(2910),
  },
  [STATE(786)] = {
    [sym_line_comment] = STATE(786),
    [sym_block_comment] = STATE(786),
    [ts_builtin_sym_end] = ACTIONS(2914),
    [sym_identifier] = ACTIONS(2916),
    [anon_sym_SEMI] = ACTIONS(2914),
    [anon_sym_macro] = ACTIONS(2916),
    [anon_sym_LPAREN] = ACTIONS(2914),
    [anon_sym_LBRACE] = ACTIONS(2914),
    [anon_sym_RBRACE] = ACTIONS(2914),
    [anon_sym_macro_rules] = ACTIONS(2916),
    [anon_sym_BANG] = ACTIONS(2914),
    [anon_sym_LBRACK] = ACTIONS(2914),
    [anon_sym_STAR] = ACTIONS(2914),
    [anon_sym_u8] = ACTIONS(2916),
    [anon_sym_i8] = ACTIONS(2916),
    [anon_sym_u16] = ACTIONS(2916),
    [anon_sym_i16] = ACTIONS(2916),
    [anon_sym_u32] = ACTIONS(2916),
    [anon_sym_i32] = ACTIONS(2916),
    [anon_sym_u64] = ACTIONS(2916),
    [anon_sym_i64] = ACTIONS(2916),
    [anon_sym_u128] = ACTIONS(2916),
    [anon_sym_i128] = ACTIONS(2916),
    [anon_sym_isize] = ACTIONS(2916),
    [anon_sym_usize] = ACTIONS(2916),
    [anon_sym_f32] = ACTIONS(2916),
    [anon_sym_f64] = ACTIONS(2916),
    [anon_sym_bool] = ACTIONS(2916),
    [anon_sym_str] = ACTIONS(2916),
    [anon_sym_char] = ACTIONS(2916),
    [anon_sym_DASH] = ACTIONS(2914),
    [anon_sym_AMP] = ACTIONS(2914),
    [anon_sym_PIPE] = ACTIONS(2914),
    [anon_sym_LT] = ACTIONS(2914),
    [anon_sym_DOT_DOT] = ACTIONS(2914),
    [anon_sym_COLON_COLON] = ACTIONS(2914),
    [anon_sym_POUND] = ACTIONS(2914),
    [anon_sym_SQUOTE] = ACTIONS(2916),
    [anon_sym_async] = ACTIONS(2916),
    [anon_sym_auto] = ACTIONS(2916),
    [anon_sym_await] = ACTIONS(2916),
    [anon_sym_become] = ACTIONS(2916),
    [anon_sym_break] = ACTIONS(2916),
    [anon_sym_const] = ACTIONS(2916),
    [anon_sym_continue] = ACTIONS(2916),
    [anon_sym_default] = ACTIONS(2916),
    [anon_sym_enum] = ACTIONS(2916),
    [anon_sym_fn] = ACTIONS(2916),
    [anon_sym_for] = ACTIONS(2916),
    [anon_sym_gen] = ACTIONS(2916),
    [anon_sym_if] = ACTIONS(2916),
    [anon_sym_impl] = ACTIONS(2916),
    [anon_sym_let] = ACTIONS(2916),
    [anon_sym_loop] = ACTIONS(2916),
    [anon_sym_match] = ACTIONS(2916),
    [anon_sym_mod] = ACTIONS(2916),
    [anon_sym_pub] = ACTIONS(2916),
    [anon_sym_return] = ACTIONS(2916),
    [anon_sym_raw] = ACTIONS(2916),
    [anon_sym_static] = ACTIONS(2916),
    [anon_sym_struct] = ACTIONS(2916),
    [anon_sym_trait] = ACTIONS(2916),
    [anon_sym_type] = ACTIONS(2916),
    [anon_sym_union] = ACTIONS(2916),
    [anon_sym_unsafe] = ACTIONS(2916),
    [anon_sym_use] = ACTIONS(2916),
    [anon_sym_while] = ACTIONS(2916),
    [anon_sym_extern] = ACTIONS(2916),
    [anon_sym_safe] = ACTIONS(2916),
    [anon_sym_yield] = ACTIONS(2916),
    [anon_sym_move] = ACTIONS(2916),
    [anon_sym_try] = ACTIONS(2916),
    [sym_integer_literal] = ACTIONS(2914),
    [aux_sym_string_literal_token1] = ACTIONS(2914),
    [sym_char_literal] = ACTIONS(2914),
    [anon_sym_true] = ACTIONS(2916),
    [anon_sym_false] = ACTIONS(2916),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2916),
    [sym_super] = ACTIONS(2916),
    [sym_crate] = ACTIONS(2916),
    [aux_sym_metavariable_token1] = ACTIONS(2914),
    [sym__raw_string_literal_start] = ACTIONS(2914),
    [sym_float_literal] = ACTIONS(2914),
    [sym__outer_block_doc_comment_start] = ACTIONS(2914),
    [sym__outer_line_doc_comment_start] = ACTIONS(2914),
  },
  [STATE(787)] = {
    [sym_line_comment] = STATE(787),
    [sym_block_comment] = STATE(787),
    [ts_builtin_sym_end] = ACTIONS(2918),
    [sym_identifier] = ACTIONS(2920),
    [anon_sym_SEMI] = ACTIONS(2918),
    [anon_sym_macro] = ACTIONS(2920),
    [anon_sym_LPAREN] = ACTIONS(2918),
    [anon_sym_LBRACE] = ACTIONS(2918),
    [anon_sym_RBRACE] = ACTIONS(2918),
    [anon_sym_macro_rules] = ACTIONS(2920),
    [anon_sym_BANG] = ACTIONS(2918),
    [anon_sym_LBRACK] = ACTIONS(2918),
    [anon_sym_STAR] = ACTIONS(2918),
    [anon_sym_u8] = ACTIONS(2920),
    [anon_sym_i8] = ACTIONS(2920),
    [anon_sym_u16] = ACTIONS(2920),
    [anon_sym_i16] = ACTIONS(2920),
    [anon_sym_u32] = ACTIONS(2920),
    [anon_sym_i32] = ACTIONS(2920),
    [anon_sym_u64] = ACTIONS(2920),
    [anon_sym_i64] = ACTIONS(2920),
    [anon_sym_u128] = ACTIONS(2920),
    [anon_sym_i128] = ACTIONS(2920),
    [anon_sym_isize] = ACTIONS(2920),
    [anon_sym_usize] = ACTIONS(2920),
    [anon_sym_f32] = ACTIONS(2920),
    [anon_sym_f64] = ACTIONS(2920),
    [anon_sym_bool] = ACTIONS(2920),
    [anon_sym_str] = ACTIONS(2920),
    [anon_sym_char] = ACTIONS(2920),
    [anon_sym_DASH] = ACTIONS(2918),
    [anon_sym_AMP] = ACTIONS(2918),
    [anon_sym_PIPE] = ACTIONS(2918),
    [anon_sym_LT] = ACTIONS(2918),
    [anon_sym_DOT_DOT] = ACTIONS(2918),
    [anon_sym_COLON_COLON] = ACTIONS(2918),
    [anon_sym_POUND] = ACTIONS(2918),
    [anon_sym_SQUOTE] = ACTIONS(2920),
    [anon_sym_async] = ACTIONS(2920),
    [anon_sym_auto] = ACTIONS(2920),
    [anon_sym_await] = ACTIONS(2920),
    [anon_sym_become] = ACTIONS(2920),
    [anon_sym_break] = ACTIONS(2920),
    [anon_sym_const] = ACTIONS(2920),
    [anon_sym_continue] = ACTIONS(2920),
    [anon_sym_default] = ACTIONS(2920),
    [anon_sym_enum] = ACTIONS(2920),
    [anon_sym_fn] = ACTIONS(2920),
    [anon_sym_for] = ACTIONS(2920),
    [anon_sym_gen] = ACTIONS(2920),
    [anon_sym_if] = ACTIONS(2920),
    [anon_sym_impl] = ACTIONS(2920),
    [anon_sym_let] = ACTIONS(2920),
    [anon_sym_loop] = ACTIONS(2920),
    [anon_sym_match] = ACTIONS(2920),
    [anon_sym_mod] = ACTIONS(2920),
    [anon_sym_pub] = ACTIONS(2920),
    [anon_sym_return] = ACTIONS(2920),
    [anon_sym_raw] = ACTIONS(2920),
    [anon_sym_static] = ACTIONS(2920),
    [anon_sym_struct] = ACTIONS(2920),
    [anon_sym_trait] = ACTIONS(2920),
    [anon_sym_type] = ACTIONS(2920),
    [anon_sym_union] = ACTIONS(2920),
    [anon_sym_unsafe] = ACTIONS(2920),
    [anon_sym_use] = ACTIONS(2920),
    [anon_sym_while] = ACTIONS(2920),
    [anon_sym_extern] = ACTIONS(2920),
    [anon_sym_safe] = ACTIONS(2920),
    [anon_sym_yield] = ACTIONS(2920),
    [anon_sym_move] = ACTIONS(2920),
    [anon_sym_try] = ACTIONS(2920),
    [sym_integer_literal] = ACTIONS(2918),
    [aux_sym_string_literal_token1] = ACTIONS(2918),
    [sym_char_literal] = ACTIONS(2918),
    [anon_sym_true] = ACTIONS(2920),
    [anon_sym_false] = ACTIONS(2920),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2920),
    [sym_super] = ACTIONS(2920),
    [sym_crate] = ACTIONS(2920),
    [aux_sym_metavariable_token1] = ACTIONS(2918),
    [sym__raw_string_literal_start] = ACTIONS(2918),
    [sym_float_literal] = ACTIONS(2918),
    [sym__outer_block_doc_comment_start] = ACTIONS(2918),
    [sym__outer_line_doc_comment_start] = ACTIONS(2918),
  },
  [STATE(788)] = {
    [sym_line_comment] = STATE(788),
    [sym_block_comment] = STATE(788),
    [ts_builtin_sym_end] = ACTIONS(2922),
    [sym_identifier] = ACTIONS(2924),
    [anon_sym_SEMI] = ACTIONS(2922),
    [anon_sym_macro] = ACTIONS(2924),
    [anon_sym_LPAREN] = ACTIONS(2922),
    [anon_sym_LBRACE] = ACTIONS(2922),
    [anon_sym_RBRACE] = ACTIONS(2922),
    [anon_sym_macro_rules] = ACTIONS(2924),
    [anon_sym_BANG] = ACTIONS(2922),
    [anon_sym_LBRACK] = ACTIONS(2922),
    [anon_sym_STAR] = ACTIONS(2922),
    [anon_sym_u8] = ACTIONS(2924),
    [anon_sym_i8] = ACTIONS(2924),
    [anon_sym_u16] = ACTIONS(2924),
    [anon_sym_i16] = ACTIONS(2924),
    [anon_sym_u32] = ACTIONS(2924),
    [anon_sym_i32] = ACTIONS(2924),
    [anon_sym_u64] = ACTIONS(2924),
    [anon_sym_i64] = ACTIONS(2924),
    [anon_sym_u128] = ACTIONS(2924),
    [anon_sym_i128] = ACTIONS(2924),
    [anon_sym_isize] = ACTIONS(2924),
    [anon_sym_usize] = ACTIONS(2924),
    [anon_sym_f32] = ACTIONS(2924),
    [anon_sym_f64] = ACTIONS(2924),
    [anon_sym_bool] = ACTIONS(2924),
    [anon_sym_str] = ACTIONS(2924),
    [anon_sym_char] = ACTIONS(2924),
    [anon_sym_DASH] = ACTIONS(2922),
    [anon_sym_AMP] = ACTIONS(2922),
    [anon_sym_PIPE] = ACTIONS(2922),
    [anon_sym_LT] = ACTIONS(2922),
    [anon_sym_DOT_DOT] = ACTIONS(2922),
    [anon_sym_COLON_COLON] = ACTIONS(2922),
    [anon_sym_POUND] = ACTIONS(2922),
    [anon_sym_SQUOTE] = ACTIONS(2924),
    [anon_sym_async] = ACTIONS(2924),
    [anon_sym_auto] = ACTIONS(2924),
    [anon_sym_await] = ACTIONS(2924),
    [anon_sym_become] = ACTIONS(2924),
    [anon_sym_break] = ACTIONS(2924),
    [anon_sym_const] = ACTIONS(2924),
    [anon_sym_continue] = ACTIONS(2924),
    [anon_sym_default] = ACTIONS(2924),
    [anon_sym_enum] = ACTIONS(2924),
    [anon_sym_fn] = ACTIONS(2924),
    [anon_sym_for] = ACTIONS(2924),
    [anon_sym_gen] = ACTIONS(2924),
    [anon_sym_if] = ACTIONS(2924),
    [anon_sym_impl] = ACTIONS(2924),
    [anon_sym_let] = ACTIONS(2924),
    [anon_sym_loop] = ACTIONS(2924),
    [anon_sym_match] = ACTIONS(2924),
    [anon_sym_mod] = ACTIONS(2924),
    [anon_sym_pub] = ACTIONS(2924),
    [anon_sym_return] = ACTIONS(2924),
    [anon_sym_raw] = ACTIONS(2924),
    [anon_sym_static] = ACTIONS(2924),
    [anon_sym_struct] = ACTIONS(2924),
    [anon_sym_trait] = ACTIONS(2924),
    [anon_sym_type] = ACTIONS(2924),
    [anon_sym_union] = ACTIONS(2924),
    [anon_sym_unsafe] = ACTIONS(2924),
    [anon_sym_use] = ACTIONS(2924),
    [anon_sym_while] = ACTIONS(2924),
    [anon_sym_extern] = ACTIONS(2924),
    [anon_sym_safe] = ACTIONS(2924),
    [anon_sym_yield] = ACTIONS(2924),
    [anon_sym_move] = ACTIONS(2924),
    [anon_sym_try] = ACTIONS(2924),
    [sym_integer_literal] = ACTIONS(2922),
    [aux_sym_string_literal_token1] = ACTIONS(2922),
    [sym_char_literal] = ACTIONS(2922),
    [anon_sym_true] = ACTIONS(2924),
    [anon_sym_false] = ACTIONS(2924),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2924),
    [sym_super] = ACTIONS(2924),
    [sym_crate] = ACTIONS(2924),
    [aux_sym_metavariable_token1] = ACTIONS(2922),
    [sym__raw_string_literal_start] = ACTIONS(2922),
    [sym_float_literal] = ACTIONS(2922),
    [sym__outer_block_doc_comment_start] = ACTIONS(2922),
    [sym__outer_line_doc_comment_start] = ACTIONS(2922),
  },
  [STATE(789)] = {
    [sym_line_comment] = STATE(789),
    [sym_block_comment] = STATE(789),
    [ts_builtin_sym_end] = ACTIONS(2926),
    [sym_identifier] = ACTIONS(2928),
    [anon_sym_SEMI] = ACTIONS(2926),
    [anon_sym_macro] = ACTIONS(2928),
    [anon_sym_LPAREN] = ACTIONS(2926),
    [anon_sym_LBRACE] = ACTIONS(2926),
    [anon_sym_RBRACE] = ACTIONS(2926),
    [anon_sym_macro_rules] = ACTIONS(2928),
    [anon_sym_BANG] = ACTIONS(2926),
    [anon_sym_LBRACK] = ACTIONS(2926),
    [anon_sym_STAR] = ACTIONS(2926),
    [anon_sym_u8] = ACTIONS(2928),
    [anon_sym_i8] = ACTIONS(2928),
    [anon_sym_u16] = ACTIONS(2928),
    [anon_sym_i16] = ACTIONS(2928),
    [anon_sym_u32] = ACTIONS(2928),
    [anon_sym_i32] = ACTIONS(2928),
    [anon_sym_u64] = ACTIONS(2928),
    [anon_sym_i64] = ACTIONS(2928),
    [anon_sym_u128] = ACTIONS(2928),
    [anon_sym_i128] = ACTIONS(2928),
    [anon_sym_isize] = ACTIONS(2928),
    [anon_sym_usize] = ACTIONS(2928),
    [anon_sym_f32] = ACTIONS(2928),
    [anon_sym_f64] = ACTIONS(2928),
    [anon_sym_bool] = ACTIONS(2928),
    [anon_sym_str] = ACTIONS(2928),
    [anon_sym_char] = ACTIONS(2928),
    [anon_sym_DASH] = ACTIONS(2926),
    [anon_sym_AMP] = ACTIONS(2926),
    [anon_sym_PIPE] = ACTIONS(2926),
    [anon_sym_LT] = ACTIONS(2926),
    [anon_sym_DOT_DOT] = ACTIONS(2926),
    [anon_sym_COLON_COLON] = ACTIONS(2926),
    [anon_sym_POUND] = ACTIONS(2926),
    [anon_sym_SQUOTE] = ACTIONS(2928),
    [anon_sym_async] = ACTIONS(2928),
    [anon_sym_auto] = ACTIONS(2928),
    [anon_sym_await] = ACTIONS(2928),
    [anon_sym_become] = ACTIONS(2928),
    [anon_sym_break] = ACTIONS(2928),
    [anon_sym_const] = ACTIONS(2928),
    [anon_sym_continue] = ACTIONS(2928),
    [anon_sym_default] = ACTIONS(2928),
    [anon_sym_enum] = ACTIONS(2928),
    [anon_sym_fn] = ACTIONS(2928),
    [anon_sym_for] = ACTIONS(2928),
    [anon_sym_gen] = ACTIONS(2928),
    [anon_sym_if] = ACTIONS(2928),
    [anon_sym_impl] = ACTIONS(2928),
    [anon_sym_let] = ACTIONS(2928),
    [anon_sym_loop] = ACTIONS(2928),
    [anon_sym_match] = ACTIONS(2928),
    [anon_sym_mod] = ACTIONS(2928),
    [anon_sym_pub] = ACTIONS(2928),
    [anon_sym_return] = ACTIONS(2928),
    [anon_sym_raw] = ACTIONS(2928),
    [anon_sym_static] = ACTIONS(2928),
    [anon_sym_struct] = ACTIONS(2928),
    [anon_sym_trait] = ACTIONS(2928),
    [anon_sym_type] = ACTIONS(2928),
    [anon_sym_union] = ACTIONS(2928),
    [anon_sym_unsafe] = ACTIONS(2928),
    [anon_sym_use] = ACTIONS(2928),
    [anon_sym_while] = ACTIONS(2928),
    [anon_sym_extern] = ACTIONS(2928),
    [anon_sym_safe] = ACTIONS(2928),
    [anon_sym_yield] = ACTIONS(2928),
    [anon_sym_move] = ACTIONS(2928),
    [anon_sym_try] = ACTIONS(2928),
    [sym_integer_literal] = ACTIONS(2926),
    [aux_sym_string_literal_token1] = ACTIONS(2926),
    [sym_char_literal] = ACTIONS(2926),
    [anon_sym_true] = ACTIONS(2928),
    [anon_sym_false] = ACTIONS(2928),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2928),
    [sym_super] = ACTIONS(2928),
    [sym_crate] = ACTIONS(2928),
    [aux_sym_metavariable_token1] = ACTIONS(2926),
    [sym__raw_string_literal_start] = ACTIONS(2926),
    [sym_float_literal] = ACTIONS(2926),
    [sym__outer_block_doc_comment_start] = ACTIONS(2926),
    [sym__outer_line_doc_comment_start] = ACTIONS(2926),
  },
  [STATE(790)] = {
    [sym_line_comment] = STATE(790),
    [sym_block_comment] = STATE(790),
    [ts_builtin_sym_end] = ACTIONS(2930),
    [sym_identifier] = ACTIONS(2932),
    [anon_sym_SEMI] = ACTIONS(2930),
    [anon_sym_macro] = ACTIONS(2932),
    [anon_sym_LPAREN] = ACTIONS(2930),
    [anon_sym_LBRACE] = ACTIONS(2930),
    [anon_sym_RBRACE] = ACTIONS(2930),
    [anon_sym_macro_rules] = ACTIONS(2932),
    [anon_sym_BANG] = ACTIONS(2930),
    [anon_sym_LBRACK] = ACTIONS(2930),
    [anon_sym_STAR] = ACTIONS(2930),
    [anon_sym_u8] = ACTIONS(2932),
    [anon_sym_i8] = ACTIONS(2932),
    [anon_sym_u16] = ACTIONS(2932),
    [anon_sym_i16] = ACTIONS(2932),
    [anon_sym_u32] = ACTIONS(2932),
    [anon_sym_i32] = ACTIONS(2932),
    [anon_sym_u64] = ACTIONS(2932),
    [anon_sym_i64] = ACTIONS(2932),
    [anon_sym_u128] = ACTIONS(2932),
    [anon_sym_i128] = ACTIONS(2932),
    [anon_sym_isize] = ACTIONS(2932),
    [anon_sym_usize] = ACTIONS(2932),
    [anon_sym_f32] = ACTIONS(2932),
    [anon_sym_f64] = ACTIONS(2932),
    [anon_sym_bool] = ACTIONS(2932),
    [anon_sym_str] = ACTIONS(2932),
    [anon_sym_char] = ACTIONS(2932),
    [anon_sym_DASH] = ACTIONS(2930),
    [anon_sym_AMP] = ACTIONS(2930),
    [anon_sym_PIPE] = ACTIONS(2930),
    [anon_sym_LT] = ACTIONS(2930),
    [anon_sym_DOT_DOT] = ACTIONS(2930),
    [anon_sym_COLON_COLON] = ACTIONS(2930),
    [anon_sym_POUND] = ACTIONS(2930),
    [anon_sym_SQUOTE] = ACTIONS(2932),
    [anon_sym_async] = ACTIONS(2932),
    [anon_sym_auto] = ACTIONS(2932),
    [anon_sym_await] = ACTIONS(2932),
    [anon_sym_become] = ACTIONS(2932),
    [anon_sym_break] = ACTIONS(2932),
    [anon_sym_const] = ACTIONS(2932),
    [anon_sym_continue] = ACTIONS(2932),
    [anon_sym_default] = ACTIONS(2932),
    [anon_sym_enum] = ACTIONS(2932),
    [anon_sym_fn] = ACTIONS(2932),
    [anon_sym_for] = ACTIONS(2932),
    [anon_sym_gen] = ACTIONS(2932),
    [anon_sym_if] = ACTIONS(2932),
    [anon_sym_impl] = ACTIONS(2932),
    [anon_sym_let] = ACTIONS(2932),
    [anon_sym_loop] = ACTIONS(2932),
    [anon_sym_match] = ACTIONS(2932),
    [anon_sym_mod] = ACTIONS(2932),
    [anon_sym_pub] = ACTIONS(2932),
    [anon_sym_return] = ACTIONS(2932),
    [anon_sym_raw] = ACTIONS(2932),
    [anon_sym_static] = ACTIONS(2932),
    [anon_sym_struct] = ACTIONS(2932),
    [anon_sym_trait] = ACTIONS(2932),
    [anon_sym_type] = ACTIONS(2932),
    [anon_sym_union] = ACTIONS(2932),
    [anon_sym_unsafe] = ACTIONS(2932),
    [anon_sym_use] = ACTIONS(2932),
    [anon_sym_while] = ACTIONS(2932),
    [anon_sym_extern] = ACTIONS(2932),
    [anon_sym_safe] = ACTIONS(2932),
    [anon_sym_yield] = ACTIONS(2932),
    [anon_sym_move] = ACTIONS(2932),
    [anon_sym_try] = ACTIONS(2932),
    [sym_integer_literal] = ACTIONS(2930),
    [aux_sym_string_literal_token1] = ACTIONS(2930),
    [sym_char_literal] = ACTIONS(2930),
    [anon_sym_true] = ACTIONS(2932),
    [anon_sym_false] = ACTIONS(2932),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2932),
    [sym_super] = ACTIONS(2932),
    [sym_crate] = ACTIONS(2932),
    [aux_sym_metavariable_token1] = ACTIONS(2930),
    [sym__raw_string_literal_start] = ACTIONS(2930),
    [sym_float_literal] = ACTIONS(2930),
    [sym__outer_block_doc_comment_start] = ACTIONS(2930),
    [sym__outer_line_doc_comment_start] = ACTIONS(2930),
  },
  [STATE(791)] = {
    [sym_line_comment] = STATE(791),
    [sym_block_comment] = STATE(791),
    [ts_builtin_sym_end] = ACTIONS(2934),
    [sym_identifier] = ACTIONS(2936),
    [anon_sym_SEMI] = ACTIONS(2934),
    [anon_sym_macro] = ACTIONS(2936),
    [anon_sym_LPAREN] = ACTIONS(2934),
    [anon_sym_LBRACE] = ACTIONS(2934),
    [anon_sym_RBRACE] = ACTIONS(2934),
    [anon_sym_macro_rules] = ACTIONS(2936),
    [anon_sym_BANG] = ACTIONS(2934),
    [anon_sym_LBRACK] = ACTIONS(2934),
    [anon_sym_STAR] = ACTIONS(2934),
    [anon_sym_u8] = ACTIONS(2936),
    [anon_sym_i8] = ACTIONS(2936),
    [anon_sym_u16] = ACTIONS(2936),
    [anon_sym_i16] = ACTIONS(2936),
    [anon_sym_u32] = ACTIONS(2936),
    [anon_sym_i32] = ACTIONS(2936),
    [anon_sym_u64] = ACTIONS(2936),
    [anon_sym_i64] = ACTIONS(2936),
    [anon_sym_u128] = ACTIONS(2936),
    [anon_sym_i128] = ACTIONS(2936),
    [anon_sym_isize] = ACTIONS(2936),
    [anon_sym_usize] = ACTIONS(2936),
    [anon_sym_f32] = ACTIONS(2936),
    [anon_sym_f64] = ACTIONS(2936),
    [anon_sym_bool] = ACTIONS(2936),
    [anon_sym_str] = ACTIONS(2936),
    [anon_sym_char] = ACTIONS(2936),
    [anon_sym_DASH] = ACTIONS(2934),
    [anon_sym_AMP] = ACTIONS(2934),
    [anon_sym_PIPE] = ACTIONS(2934),
    [anon_sym_LT] = ACTIONS(2934),
    [anon_sym_DOT_DOT] = ACTIONS(2934),
    [anon_sym_COLON_COLON] = ACTIONS(2934),
    [anon_sym_POUND] = ACTIONS(2934),
    [anon_sym_SQUOTE] = ACTIONS(2936),
    [anon_sym_async] = ACTIONS(2936),
    [anon_sym_auto] = ACTIONS(2936),
    [anon_sym_await] = ACTIONS(2936),
    [anon_sym_become] = ACTIONS(2936),
    [anon_sym_break] = ACTIONS(2936),
    [anon_sym_const] = ACTIONS(2936),
    [anon_sym_continue] = ACTIONS(2936),
    [anon_sym_default] = ACTIONS(2936),
    [anon_sym_enum] = ACTIONS(2936),
    [anon_sym_fn] = ACTIONS(2936),
    [anon_sym_for] = ACTIONS(2936),
    [anon_sym_gen] = ACTIONS(2936),
    [anon_sym_if] = ACTIONS(2936),
    [anon_sym_impl] = ACTIONS(2936),
    [anon_sym_let] = ACTIONS(2936),
    [anon_sym_loop] = ACTIONS(2936),
    [anon_sym_match] = ACTIONS(2936),
    [anon_sym_mod] = ACTIONS(2936),
    [anon_sym_pub] = ACTIONS(2936),
    [anon_sym_return] = ACTIONS(2936),
    [anon_sym_raw] = ACTIONS(2936),
    [anon_sym_static] = ACTIONS(2936),
    [anon_sym_struct] = ACTIONS(2936),
    [anon_sym_trait] = ACTIONS(2936),
    [anon_sym_type] = ACTIONS(2936),
    [anon_sym_union] = ACTIONS(2936),
    [anon_sym_unsafe] = ACTIONS(2936),
    [anon_sym_use] = ACTIONS(2936),
    [anon_sym_while] = ACTIONS(2936),
    [anon_sym_extern] = ACTIONS(2936),
    [anon_sym_safe] = ACTIONS(2936),
    [anon_sym_yield] = ACTIONS(2936),
    [anon_sym_move] = ACTIONS(2936),
    [anon_sym_try] = ACTIONS(2936),
    [sym_integer_literal] = ACTIONS(2934),
    [aux_sym_string_literal_token1] = ACTIONS(2934),
    [sym_char_literal] = ACTIONS(2934),
    [anon_sym_true] = ACTIONS(2936),
    [anon_sym_false] = ACTIONS(2936),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2936),
    [sym_super] = ACTIONS(2936),
    [sym_crate] = ACTIONS(2936),
    [aux_sym_metavariable_token1] = ACTIONS(2934),
    [sym__raw_string_literal_start] = ACTIONS(2934),
    [sym_float_literal] = ACTIONS(2934),
    [sym__outer_block_doc_comment_start] = ACTIONS(2934),
    [sym__outer_line_doc_comment_start] = ACTIONS(2934),
  },
  [STATE(792)] = {
    [sym_line_comment] = STATE(792),
    [sym_block_comment] = STATE(792),
    [ts_builtin_sym_end] = ACTIONS(2938),
    [sym_identifier] = ACTIONS(2940),
    [anon_sym_SEMI] = ACTIONS(2938),
    [anon_sym_macro] = ACTIONS(2940),
    [anon_sym_LPAREN] = ACTIONS(2938),
    [anon_sym_LBRACE] = ACTIONS(2938),
    [anon_sym_RBRACE] = ACTIONS(2938),
    [anon_sym_macro_rules] = ACTIONS(2940),
    [anon_sym_BANG] = ACTIONS(2938),
    [anon_sym_LBRACK] = ACTIONS(2938),
    [anon_sym_STAR] = ACTIONS(2938),
    [anon_sym_u8] = ACTIONS(2940),
    [anon_sym_i8] = ACTIONS(2940),
    [anon_sym_u16] = ACTIONS(2940),
    [anon_sym_i16] = ACTIONS(2940),
    [anon_sym_u32] = ACTIONS(2940),
    [anon_sym_i32] = ACTIONS(2940),
    [anon_sym_u64] = ACTIONS(2940),
    [anon_sym_i64] = ACTIONS(2940),
    [anon_sym_u128] = ACTIONS(2940),
    [anon_sym_i128] = ACTIONS(2940),
    [anon_sym_isize] = ACTIONS(2940),
    [anon_sym_usize] = ACTIONS(2940),
    [anon_sym_f32] = ACTIONS(2940),
    [anon_sym_f64] = ACTIONS(2940),
    [anon_sym_bool] = ACTIONS(2940),
    [anon_sym_str] = ACTIONS(2940),
    [anon_sym_char] = ACTIONS(2940),
    [anon_sym_DASH] = ACTIONS(2938),
    [anon_sym_AMP] = ACTIONS(2938),
    [anon_sym_PIPE] = ACTIONS(2938),
    [anon_sym_LT] = ACTIONS(2938),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [anon_sym_COLON_COLON] = ACTIONS(2938),
    [anon_sym_POUND] = ACTIONS(2938),
    [anon_sym_SQUOTE] = ACTIONS(2940),
    [anon_sym_async] = ACTIONS(2940),
    [anon_sym_auto] = ACTIONS(2940),
    [anon_sym_await] = ACTIONS(2940),
    [anon_sym_become] = ACTIONS(2940),
    [anon_sym_break] = ACTIONS(2940),
    [anon_sym_const] = ACTIONS(2940),
    [anon_sym_continue] = ACTIONS(2940),
    [anon_sym_default] = ACTIONS(2940),
    [anon_sym_enum] = ACTIONS(2940),
    [anon_sym_fn] = ACTIONS(2940),
    [anon_sym_for] = ACTIONS(2940),
    [anon_sym_gen] = ACTIONS(2940),
    [anon_sym_if] = ACTIONS(2940),
    [anon_sym_impl] = ACTIONS(2940),
    [anon_sym_let] = ACTIONS(2940),
    [anon_sym_loop] = ACTIONS(2940),
    [anon_sym_match] = ACTIONS(2940),
    [anon_sym_mod] = ACTIONS(2940),
    [anon_sym_pub] = ACTIONS(2940),
    [anon_sym_return] = ACTIONS(2940),
    [anon_sym_raw] = ACTIONS(2940),
    [anon_sym_static] = ACTIONS(2940),
    [anon_sym_struct] = ACTIONS(2940),
    [anon_sym_trait] = ACTIONS(2940),
    [anon_sym_type] = ACTIONS(2940),
    [anon_sym_union] = ACTIONS(2940),
    [anon_sym_unsafe] = ACTIONS(2940),
    [anon_sym_use] = ACTIONS(2940),
    [anon_sym_while] = ACTIONS(2940),
    [anon_sym_extern] = ACTIONS(2940),
    [anon_sym_safe] = ACTIONS(2940),
    [anon_sym_yield] = ACTIONS(2940),
    [anon_sym_move] = ACTIONS(2940),
    [anon_sym_try] = ACTIONS(2940),
    [sym_integer_literal] = ACTIONS(2938),
    [aux_sym_string_literal_token1] = ACTIONS(2938),
    [sym_char_literal] = ACTIONS(2938),
    [anon_sym_true] = ACTIONS(2940),
    [anon_sym_false] = ACTIONS(2940),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2940),
    [sym_super] = ACTIONS(2940),
    [sym_crate] = ACTIONS(2940),
    [aux_sym_metavariable_token1] = ACTIONS(2938),
    [sym__raw_string_literal_start] = ACTIONS(2938),
    [sym_float_literal] = ACTIONS(2938),
    [sym__outer_block_doc_comment_start] = ACTIONS(2938),
    [sym__outer_line_doc_comment_start] = ACTIONS(2938),
  },
  [STATE(793)] = {
    [sym_line_comment] = STATE(793),
    [sym_block_comment] = STATE(793),
    [ts_builtin_sym_end] = ACTIONS(2942),
    [sym_identifier] = ACTIONS(2944),
    [anon_sym_SEMI] = ACTIONS(2942),
    [anon_sym_macro] = ACTIONS(2944),
    [anon_sym_LPAREN] = ACTIONS(2942),
    [anon_sym_LBRACE] = ACTIONS(2942),
    [anon_sym_RBRACE] = ACTIONS(2942),
    [anon_sym_macro_rules] = ACTIONS(2944),
    [anon_sym_BANG] = ACTIONS(2942),
    [anon_sym_LBRACK] = ACTIONS(2942),
    [anon_sym_STAR] = ACTIONS(2942),
    [anon_sym_u8] = ACTIONS(2944),
    [anon_sym_i8] = ACTIONS(2944),
    [anon_sym_u16] = ACTIONS(2944),
    [anon_sym_i16] = ACTIONS(2944),
    [anon_sym_u32] = ACTIONS(2944),
    [anon_sym_i32] = ACTIONS(2944),
    [anon_sym_u64] = ACTIONS(2944),
    [anon_sym_i64] = ACTIONS(2944),
    [anon_sym_u128] = ACTIONS(2944),
    [anon_sym_i128] = ACTIONS(2944),
    [anon_sym_isize] = ACTIONS(2944),
    [anon_sym_usize] = ACTIONS(2944),
    [anon_sym_f32] = ACTIONS(2944),
    [anon_sym_f64] = ACTIONS(2944),
    [anon_sym_bool] = ACTIONS(2944),
    [anon_sym_str] = ACTIONS(2944),
    [anon_sym_char] = ACTIONS(2944),
    [anon_sym_DASH] = ACTIONS(2942),
    [anon_sym_AMP] = ACTIONS(2942),
    [anon_sym_PIPE] = ACTIONS(2942),
    [anon_sym_LT] = ACTIONS(2942),
    [anon_sym_DOT_DOT] = ACTIONS(2942),
    [anon_sym_COLON_COLON] = ACTIONS(2942),
    [anon_sym_POUND] = ACTIONS(2942),
    [anon_sym_SQUOTE] = ACTIONS(2944),
    [anon_sym_async] = ACTIONS(2944),
    [anon_sym_auto] = ACTIONS(2944),
    [anon_sym_await] = ACTIONS(2944),
    [anon_sym_become] = ACTIONS(2944),
    [anon_sym_break] = ACTIONS(2944),
    [anon_sym_const] = ACTIONS(2944),
    [anon_sym_continue] = ACTIONS(2944),
    [anon_sym_default] = ACTIONS(2944),
    [anon_sym_enum] = ACTIONS(2944),
    [anon_sym_fn] = ACTIONS(2944),
    [anon_sym_for] = ACTIONS(2944),
    [anon_sym_gen] = ACTIONS(2944),
    [anon_sym_if] = ACTIONS(2944),
    [anon_sym_impl] = ACTIONS(2944),
    [anon_sym_let] = ACTIONS(2944),
    [anon_sym_loop] = ACTIONS(2944),
    [anon_sym_match] = ACTIONS(2944),
    [anon_sym_mod] = ACTIONS(2944),
    [anon_sym_pub] = ACTIONS(2944),
    [anon_sym_return] = ACTIONS(2944),
    [anon_sym_raw] = ACTIONS(2944),
    [anon_sym_static] = ACTIONS(2944),
    [anon_sym_struct] = ACTIONS(2944),
    [anon_sym_trait] = ACTIONS(2944),
    [anon_sym_type] = ACTIONS(2944),
    [anon_sym_union] = ACTIONS(2944),
    [anon_sym_unsafe] = ACTIONS(2944),
    [anon_sym_use] = ACTIONS(2944),
    [anon_sym_while] = ACTIONS(2944),
    [anon_sym_extern] = ACTIONS(2944),
    [anon_sym_safe] = ACTIONS(2944),
    [anon_sym_yield] = ACTIONS(2944),
    [anon_sym_move] = ACTIONS(2944),
    [anon_sym_try] = ACTIONS(2944),
    [sym_integer_literal] = ACTIONS(2942),
    [aux_sym_string_literal_token1] = ACTIONS(2942),
    [sym_char_literal] = ACTIONS(2942),
    [anon_sym_true] = ACTIONS(2944),
    [anon_sym_false] = ACTIONS(2944),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2944),
    [sym_super] = ACTIONS(2944),
    [sym_crate] = ACTIONS(2944),
    [aux_sym_metavariable_token1] = ACTIONS(2942),
    [sym__raw_string_literal_start] = ACTIONS(2942),
    [sym_float_literal] = ACTIONS(2942),
    [sym__outer_block_doc_comment_start] = ACTIONS(2942),
    [sym__outer_line_doc_comment_start] = ACTIONS(2942),
  },
  [STATE(794)] = {
    [sym_line_comment] = STATE(794),
    [sym_block_comment] = STATE(794),
    [ts_builtin_sym_end] = ACTIONS(2946),
    [sym_identifier] = ACTIONS(2948),
    [anon_sym_SEMI] = ACTIONS(2946),
    [anon_sym_macro] = ACTIONS(2948),
    [anon_sym_LPAREN] = ACTIONS(2946),
    [anon_sym_LBRACE] = ACTIONS(2946),
    [anon_sym_RBRACE] = ACTIONS(2946),
    [anon_sym_macro_rules] = ACTIONS(2948),
    [anon_sym_BANG] = ACTIONS(2946),
    [anon_sym_LBRACK] = ACTIONS(2946),
    [anon_sym_STAR] = ACTIONS(2946),
    [anon_sym_u8] = ACTIONS(2948),
    [anon_sym_i8] = ACTIONS(2948),
    [anon_sym_u16] = ACTIONS(2948),
    [anon_sym_i16] = ACTIONS(2948),
    [anon_sym_u32] = ACTIONS(2948),
    [anon_sym_i32] = ACTIONS(2948),
    [anon_sym_u64] = ACTIONS(2948),
    [anon_sym_i64] = ACTIONS(2948),
    [anon_sym_u128] = ACTIONS(2948),
    [anon_sym_i128] = ACTIONS(2948),
    [anon_sym_isize] = ACTIONS(2948),
    [anon_sym_usize] = ACTIONS(2948),
    [anon_sym_f32] = ACTIONS(2948),
    [anon_sym_f64] = ACTIONS(2948),
    [anon_sym_bool] = ACTIONS(2948),
    [anon_sym_str] = ACTIONS(2948),
    [anon_sym_char] = ACTIONS(2948),
    [anon_sym_DASH] = ACTIONS(2946),
    [anon_sym_AMP] = ACTIONS(2946),
    [anon_sym_PIPE] = ACTIONS(2946),
    [anon_sym_LT] = ACTIONS(2946),
    [anon_sym_DOT_DOT] = ACTIONS(2946),
    [anon_sym_COLON_COLON] = ACTIONS(2946),
    [anon_sym_POUND] = ACTIONS(2946),
    [anon_sym_SQUOTE] = ACTIONS(2948),
    [anon_sym_async] = ACTIONS(2948),
    [anon_sym_auto] = ACTIONS(2948),
    [anon_sym_await] = ACTIONS(2948),
    [anon_sym_become] = ACTIONS(2948),
    [anon_sym_break] = ACTIONS(2948),
    [anon_sym_const] = ACTIONS(2948),
    [anon_sym_continue] = ACTIONS(2948),
    [anon_sym_default] = ACTIONS(2948),
    [anon_sym_enum] = ACTIONS(2948),
    [anon_sym_fn] = ACTIONS(2948),
    [anon_sym_for] = ACTIONS(2948),
    [anon_sym_gen] = ACTIONS(2948),
    [anon_sym_if] = ACTIONS(2948),
    [anon_sym_impl] = ACTIONS(2948),
    [anon_sym_let] = ACTIONS(2948),
    [anon_sym_loop] = ACTIONS(2948),
    [anon_sym_match] = ACTIONS(2948),
    [anon_sym_mod] = ACTIONS(2948),
    [anon_sym_pub] = ACTIONS(2948),
    [anon_sym_return] = ACTIONS(2948),
    [anon_sym_raw] = ACTIONS(2948),
    [anon_sym_static] = ACTIONS(2948),
    [anon_sym_struct] = ACTIONS(2948),
    [anon_sym_trait] = ACTIONS(2948),
    [anon_sym_type] = ACTIONS(2948),
    [anon_sym_union] = ACTIONS(2948),
    [anon_sym_unsafe] = ACTIONS(2948),
    [anon_sym_use] = ACTIONS(2948),
    [anon_sym_while] = ACTIONS(2948),
    [anon_sym_extern] = ACTIONS(2948),
    [anon_sym_safe] = ACTIONS(2948),
    [anon_sym_yield] = ACTIONS(2948),
    [anon_sym_move] = ACTIONS(2948),
    [anon_sym_try] = ACTIONS(2948),
    [sym_integer_literal] = ACTIONS(2946),
    [aux_sym_string_literal_token1] = ACTIONS(2946),
    [sym_char_literal] = ACTIONS(2946),
    [anon_sym_true] = ACTIONS(2948),
    [anon_sym_false] = ACTIONS(2948),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2948),
    [sym_super] = ACTIONS(2948),
    [sym_crate] = ACTIONS(2948),
    [aux_sym_metavariable_token1] = ACTIONS(2946),
    [sym__raw_string_literal_start] = ACTIONS(2946),
    [sym_float_literal] = ACTIONS(2946),
    [sym__outer_block_doc_comment_start] = ACTIONS(2946),
    [sym__outer_line_doc_comment_start] = ACTIONS(2946),
  },
  [STATE(795)] = {
    [sym_line_comment] = STATE(795),
    [sym_block_comment] = STATE(795),
    [ts_builtin_sym_end] = ACTIONS(2950),
    [sym_identifier] = ACTIONS(2952),
    [anon_sym_SEMI] = ACTIONS(2950),
    [anon_sym_macro] = ACTIONS(2952),
    [anon_sym_LPAREN] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2950),
    [anon_sym_RBRACE] = ACTIONS(2950),
    [anon_sym_macro_rules] = ACTIONS(2952),
    [anon_sym_BANG] = ACTIONS(2950),
    [anon_sym_LBRACK] = ACTIONS(2950),
    [anon_sym_STAR] = ACTIONS(2950),
    [anon_sym_u8] = ACTIONS(2952),
    [anon_sym_i8] = ACTIONS(2952),
    [anon_sym_u16] = ACTIONS(2952),
    [anon_sym_i16] = ACTIONS(2952),
    [anon_sym_u32] = ACTIONS(2952),
    [anon_sym_i32] = ACTIONS(2952),
    [anon_sym_u64] = ACTIONS(2952),
    [anon_sym_i64] = ACTIONS(2952),
    [anon_sym_u128] = ACTIONS(2952),
    [anon_sym_i128] = ACTIONS(2952),
    [anon_sym_isize] = ACTIONS(2952),
    [anon_sym_usize] = ACTIONS(2952),
    [anon_sym_f32] = ACTIONS(2952),
    [anon_sym_f64] = ACTIONS(2952),
    [anon_sym_bool] = ACTIONS(2952),
    [anon_sym_str] = ACTIONS(2952),
    [anon_sym_char] = ACTIONS(2952),
    [anon_sym_DASH] = ACTIONS(2950),
    [anon_sym_AMP] = ACTIONS(2950),
    [anon_sym_PIPE] = ACTIONS(2950),
    [anon_sym_LT] = ACTIONS(2950),
    [anon_sym_DOT_DOT] = ACTIONS(2950),
    [anon_sym_COLON_COLON] = ACTIONS(2950),
    [anon_sym_POUND] = ACTIONS(2950),
    [anon_sym_SQUOTE] = ACTIONS(2952),
    [anon_sym_async] = ACTIONS(2952),
    [anon_sym_auto] = ACTIONS(2952),
    [anon_sym_await] = ACTIONS(2952),
    [anon_sym_become] = ACTIONS(2952),
    [anon_sym_break] = ACTIONS(2952),
    [anon_sym_const] = ACTIONS(2952),
    [anon_sym_continue] = ACTIONS(2952),
    [anon_sym_default] = ACTIONS(2952),
    [anon_sym_enum] = ACTIONS(2952),
    [anon_sym_fn] = ACTIONS(2952),
    [anon_sym_for] = ACTIONS(2952),
    [anon_sym_gen] = ACTIONS(2952),
    [anon_sym_if] = ACTIONS(2952),
    [anon_sym_impl] = ACTIONS(2952),
    [anon_sym_let] = ACTIONS(2952),
    [anon_sym_loop] = ACTIONS(2952),
    [anon_sym_match] = ACTIONS(2952),
    [anon_sym_mod] = ACTIONS(2952),
    [anon_sym_pub] = ACTIONS(2952),
    [anon_sym_return] = ACTIONS(2952),
    [anon_sym_raw] = ACTIONS(2952),
    [anon_sym_static] = ACTIONS(2952),
    [anon_sym_struct] = ACTIONS(2952),
    [anon_sym_trait] = ACTIONS(2952),
    [anon_sym_type] = ACTIONS(2952),
    [anon_sym_union] = ACTIONS(2952),
    [anon_sym_unsafe] = ACTIONS(2952),
    [anon_sym_use] = ACTIONS(2952),
    [anon_sym_while] = ACTIONS(2952),
    [anon_sym_extern] = ACTIONS(2952),
    [anon_sym_safe] = ACTIONS(2952),
    [anon_sym_yield] = ACTIONS(2952),
    [anon_sym_move] = ACTIONS(2952),
    [anon_sym_try] = ACTIONS(2952),
    [sym_integer_literal] = ACTIONS(2950),
    [aux_sym_string_literal_token1] = ACTIONS(2950),
    [sym_char_literal] = ACTIONS(2950),
    [anon_sym_true] = ACTIONS(2952),
    [anon_sym_false] = ACTIONS(2952),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2952),
    [sym_super] = ACTIONS(2952),
    [sym_crate] = ACTIONS(2952),
    [aux_sym_metavariable_token1] = ACTIONS(2950),
    [sym__raw_string_literal_start] = ACTIONS(2950),
    [sym_float_literal] = ACTIONS(2950),
    [sym__outer_block_doc_comment_start] = ACTIONS(2950),
    [sym__outer_line_doc_comment_start] = ACTIONS(2950),
  },
  [STATE(796)] = {
    [sym_line_comment] = STATE(796),
    [sym_block_comment] = STATE(796),
    [ts_builtin_sym_end] = ACTIONS(2954),
    [sym_identifier] = ACTIONS(2956),
    [anon_sym_SEMI] = ACTIONS(2954),
    [anon_sym_macro] = ACTIONS(2956),
    [anon_sym_LPAREN] = ACTIONS(2954),
    [anon_sym_LBRACE] = ACTIONS(2954),
    [anon_sym_RBRACE] = ACTIONS(2954),
    [anon_sym_macro_rules] = ACTIONS(2956),
    [anon_sym_BANG] = ACTIONS(2954),
    [anon_sym_LBRACK] = ACTIONS(2954),
    [anon_sym_STAR] = ACTIONS(2954),
    [anon_sym_u8] = ACTIONS(2956),
    [anon_sym_i8] = ACTIONS(2956),
    [anon_sym_u16] = ACTIONS(2956),
    [anon_sym_i16] = ACTIONS(2956),
    [anon_sym_u32] = ACTIONS(2956),
    [anon_sym_i32] = ACTIONS(2956),
    [anon_sym_u64] = ACTIONS(2956),
    [anon_sym_i64] = ACTIONS(2956),
    [anon_sym_u128] = ACTIONS(2956),
    [anon_sym_i128] = ACTIONS(2956),
    [anon_sym_isize] = ACTIONS(2956),
    [anon_sym_usize] = ACTIONS(2956),
    [anon_sym_f32] = ACTIONS(2956),
    [anon_sym_f64] = ACTIONS(2956),
    [anon_sym_bool] = ACTIONS(2956),
    [anon_sym_str] = ACTIONS(2956),
    [anon_sym_char] = ACTIONS(2956),
    [anon_sym_DASH] = ACTIONS(2954),
    [anon_sym_AMP] = ACTIONS(2954),
    [anon_sym_PIPE] = ACTIONS(2954),
    [anon_sym_LT] = ACTIONS(2954),
    [anon_sym_DOT_DOT] = ACTIONS(2954),
    [anon_sym_COLON_COLON] = ACTIONS(2954),
    [anon_sym_POUND] = ACTIONS(2954),
    [anon_sym_SQUOTE] = ACTIONS(2956),
    [anon_sym_async] = ACTIONS(2956),
    [anon_sym_auto] = ACTIONS(2956),
    [anon_sym_await] = ACTIONS(2956),
    [anon_sym_become] = ACTIONS(2956),
    [anon_sym_break] = ACTIONS(2956),
    [anon_sym_const] = ACTIONS(2956),
    [anon_sym_continue] = ACTIONS(2956),
    [anon_sym_default] = ACTIONS(2956),
    [anon_sym_enum] = ACTIONS(2956),
    [anon_sym_fn] = ACTIONS(2956),
    [anon_sym_for] = ACTIONS(2956),
    [anon_sym_gen] = ACTIONS(2956),
    [anon_sym_if] = ACTIONS(2956),
    [anon_sym_impl] = ACTIONS(2956),
    [anon_sym_let] = ACTIONS(2956),
    [anon_sym_loop] = ACTIONS(2956),
    [anon_sym_match] = ACTIONS(2956),
    [anon_sym_mod] = ACTIONS(2956),
    [anon_sym_pub] = ACTIONS(2956),
    [anon_sym_return] = ACTIONS(2956),
    [anon_sym_raw] = ACTIONS(2956),
    [anon_sym_static] = ACTIONS(2956),
    [anon_sym_struct] = ACTIONS(2956),
    [anon_sym_trait] = ACTIONS(2956),
    [anon_sym_type] = ACTIONS(2956),
    [anon_sym_union] = ACTIONS(2956),
    [anon_sym_unsafe] = ACTIONS(2956),
    [anon_sym_use] = ACTIONS(2956),
    [anon_sym_while] = ACTIONS(2956),
    [anon_sym_extern] = ACTIONS(2956),
    [anon_sym_safe] = ACTIONS(2956),
    [anon_sym_yield] = ACTIONS(2956),
    [anon_sym_move] = ACTIONS(2956),
    [anon_sym_try] = ACTIONS(2956),
    [sym_integer_literal] = ACTIONS(2954),
    [aux_sym_string_literal_token1] = ACTIONS(2954),
    [sym_char_literal] = ACTIONS(2954),
    [anon_sym_true] = ACTIONS(2956),
    [anon_sym_false] = ACTIONS(2956),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2956),
    [sym_super] = ACTIONS(2956),
    [sym_crate] = ACTIONS(2956),
    [aux_sym_metavariable_token1] = ACTIONS(2954),
    [sym__raw_string_literal_start] = ACTIONS(2954),
    [sym_float_literal] = ACTIONS(2954),
    [sym__outer_block_doc_comment_start] = ACTIONS(2954),
    [sym__outer_line_doc_comment_start] = ACTIONS(2954),
  },
  [STATE(797)] = {
    [sym_line_comment] = STATE(797),
    [sym_block_comment] = STATE(797),
    [ts_builtin_sym_end] = ACTIONS(2958),
    [sym_identifier] = ACTIONS(2960),
    [anon_sym_SEMI] = ACTIONS(2958),
    [anon_sym_macro] = ACTIONS(2960),
    [anon_sym_LPAREN] = ACTIONS(2958),
    [anon_sym_LBRACE] = ACTIONS(2958),
    [anon_sym_RBRACE] = ACTIONS(2958),
    [anon_sym_macro_rules] = ACTIONS(2960),
    [anon_sym_BANG] = ACTIONS(2958),
    [anon_sym_LBRACK] = ACTIONS(2958),
    [anon_sym_STAR] = ACTIONS(2958),
    [anon_sym_u8] = ACTIONS(2960),
    [anon_sym_i8] = ACTIONS(2960),
    [anon_sym_u16] = ACTIONS(2960),
    [anon_sym_i16] = ACTIONS(2960),
    [anon_sym_u32] = ACTIONS(2960),
    [anon_sym_i32] = ACTIONS(2960),
    [anon_sym_u64] = ACTIONS(2960),
    [anon_sym_i64] = ACTIONS(2960),
    [anon_sym_u128] = ACTIONS(2960),
    [anon_sym_i128] = ACTIONS(2960),
    [anon_sym_isize] = ACTIONS(2960),
    [anon_sym_usize] = ACTIONS(2960),
    [anon_sym_f32] = ACTIONS(2960),
    [anon_sym_f64] = ACTIONS(2960),
    [anon_sym_bool] = ACTIONS(2960),
    [anon_sym_str] = ACTIONS(2960),
    [anon_sym_char] = ACTIONS(2960),
    [anon_sym_DASH] = ACTIONS(2958),
    [anon_sym_AMP] = ACTIONS(2958),
    [anon_sym_PIPE] = ACTIONS(2958),
    [anon_sym_LT] = ACTIONS(2958),
    [anon_sym_DOT_DOT] = ACTIONS(2958),
    [anon_sym_COLON_COLON] = ACTIONS(2958),
    [anon_sym_POUND] = ACTIONS(2958),
    [anon_sym_SQUOTE] = ACTIONS(2960),
    [anon_sym_async] = ACTIONS(2960),
    [anon_sym_auto] = ACTIONS(2960),
    [anon_sym_await] = ACTIONS(2960),
    [anon_sym_become] = ACTIONS(2960),
    [anon_sym_break] = ACTIONS(2960),
    [anon_sym_const] = ACTIONS(2960),
    [anon_sym_continue] = ACTIONS(2960),
    [anon_sym_default] = ACTIONS(2960),
    [anon_sym_enum] = ACTIONS(2960),
    [anon_sym_fn] = ACTIONS(2960),
    [anon_sym_for] = ACTIONS(2960),
    [anon_sym_gen] = ACTIONS(2960),
    [anon_sym_if] = ACTIONS(2960),
    [anon_sym_impl] = ACTIONS(2960),
    [anon_sym_let] = ACTIONS(2960),
    [anon_sym_loop] = ACTIONS(2960),
    [anon_sym_match] = ACTIONS(2960),
    [anon_sym_mod] = ACTIONS(2960),
    [anon_sym_pub] = ACTIONS(2960),
    [anon_sym_return] = ACTIONS(2960),
    [anon_sym_raw] = ACTIONS(2960),
    [anon_sym_static] = ACTIONS(2960),
    [anon_sym_struct] = ACTIONS(2960),
    [anon_sym_trait] = ACTIONS(2960),
    [anon_sym_type] = ACTIONS(2960),
    [anon_sym_union] = ACTIONS(2960),
    [anon_sym_unsafe] = ACTIONS(2960),
    [anon_sym_use] = ACTIONS(2960),
    [anon_sym_while] = ACTIONS(2960),
    [anon_sym_extern] = ACTIONS(2960),
    [anon_sym_safe] = ACTIONS(2960),
    [anon_sym_yield] = ACTIONS(2960),
    [anon_sym_move] = ACTIONS(2960),
    [anon_sym_try] = ACTIONS(2960),
    [sym_integer_literal] = ACTIONS(2958),
    [aux_sym_string_literal_token1] = ACTIONS(2958),
    [sym_char_literal] = ACTIONS(2958),
    [anon_sym_true] = ACTIONS(2960),
    [anon_sym_false] = ACTIONS(2960),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2960),
    [sym_super] = ACTIONS(2960),
    [sym_crate] = ACTIONS(2960),
    [aux_sym_metavariable_token1] = ACTIONS(2958),
    [sym__raw_string_literal_start] = ACTIONS(2958),
    [sym_float_literal] = ACTIONS(2958),
    [sym__outer_block_doc_comment_start] = ACTIONS(2958),
    [sym__outer_line_doc_comment_start] = ACTIONS(2958),
  },
  [STATE(798)] = {
    [sym_line_comment] = STATE(798),
    [sym_block_comment] = STATE(798),
    [ts_builtin_sym_end] = ACTIONS(2962),
    [sym_identifier] = ACTIONS(2964),
    [anon_sym_SEMI] = ACTIONS(2962),
    [anon_sym_macro] = ACTIONS(2964),
    [anon_sym_LPAREN] = ACTIONS(2962),
    [anon_sym_LBRACE] = ACTIONS(2962),
    [anon_sym_RBRACE] = ACTIONS(2962),
    [anon_sym_macro_rules] = ACTIONS(2964),
    [anon_sym_BANG] = ACTIONS(2962),
    [anon_sym_LBRACK] = ACTIONS(2962),
    [anon_sym_STAR] = ACTIONS(2962),
    [anon_sym_u8] = ACTIONS(2964),
    [anon_sym_i8] = ACTIONS(2964),
    [anon_sym_u16] = ACTIONS(2964),
    [anon_sym_i16] = ACTIONS(2964),
    [anon_sym_u32] = ACTIONS(2964),
    [anon_sym_i32] = ACTIONS(2964),
    [anon_sym_u64] = ACTIONS(2964),
    [anon_sym_i64] = ACTIONS(2964),
    [anon_sym_u128] = ACTIONS(2964),
    [anon_sym_i128] = ACTIONS(2964),
    [anon_sym_isize] = ACTIONS(2964),
    [anon_sym_usize] = ACTIONS(2964),
    [anon_sym_f32] = ACTIONS(2964),
    [anon_sym_f64] = ACTIONS(2964),
    [anon_sym_bool] = ACTIONS(2964),
    [anon_sym_str] = ACTIONS(2964),
    [anon_sym_char] = ACTIONS(2964),
    [anon_sym_DASH] = ACTIONS(2962),
    [anon_sym_AMP] = ACTIONS(2962),
    [anon_sym_PIPE] = ACTIONS(2962),
    [anon_sym_LT] = ACTIONS(2962),
    [anon_sym_DOT_DOT] = ACTIONS(2962),
    [anon_sym_COLON_COLON] = ACTIONS(2962),
    [anon_sym_POUND] = ACTIONS(2962),
    [anon_sym_SQUOTE] = ACTIONS(2964),
    [anon_sym_async] = ACTIONS(2964),
    [anon_sym_auto] = ACTIONS(2964),
    [anon_sym_await] = ACTIONS(2964),
    [anon_sym_become] = ACTIONS(2964),
    [anon_sym_break] = ACTIONS(2964),
    [anon_sym_const] = ACTIONS(2964),
    [anon_sym_continue] = ACTIONS(2964),
    [anon_sym_default] = ACTIONS(2964),
    [anon_sym_enum] = ACTIONS(2964),
    [anon_sym_fn] = ACTIONS(2964),
    [anon_sym_for] = ACTIONS(2964),
    [anon_sym_gen] = ACTIONS(2964),
    [anon_sym_if] = ACTIONS(2964),
    [anon_sym_impl] = ACTIONS(2964),
    [anon_sym_let] = ACTIONS(2964),
    [anon_sym_loop] = ACTIONS(2964),
    [anon_sym_match] = ACTIONS(2964),
    [anon_sym_mod] = ACTIONS(2964),
    [anon_sym_pub] = ACTIONS(2964),
    [anon_sym_return] = ACTIONS(2964),
    [anon_sym_raw] = ACTIONS(2964),
    [anon_sym_static] = ACTIONS(2964),
    [anon_sym_struct] = ACTIONS(2964),
    [anon_sym_trait] = ACTIONS(2964),
    [anon_sym_type] = ACTIONS(2964),
    [anon_sym_union] = ACTIONS(2964),
    [anon_sym_unsafe] = ACTIONS(2964),
    [anon_sym_use] = ACTIONS(2964),
    [anon_sym_while] = ACTIONS(2964),
    [anon_sym_extern] = ACTIONS(2964),
    [anon_sym_safe] = ACTIONS(2964),
    [anon_sym_yield] = ACTIONS(2964),
    [anon_sym_move] = ACTIONS(2964),
    [anon_sym_try] = ACTIONS(2964),
    [sym_integer_literal] = ACTIONS(2962),
    [aux_sym_string_literal_token1] = ACTIONS(2962),
    [sym_char_literal] = ACTIONS(2962),
    [anon_sym_true] = ACTIONS(2964),
    [anon_sym_false] = ACTIONS(2964),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2964),
    [sym_super] = ACTIONS(2964),
    [sym_crate] = ACTIONS(2964),
    [aux_sym_metavariable_token1] = ACTIONS(2962),
    [sym__raw_string_literal_start] = ACTIONS(2962),
    [sym_float_literal] = ACTIONS(2962),
    [sym__outer_block_doc_comment_start] = ACTIONS(2962),
    [sym__outer_line_doc_comment_start] = ACTIONS(2962),
  },
  [STATE(799)] = {
    [sym_line_comment] = STATE(799),
    [sym_block_comment] = STATE(799),
    [ts_builtin_sym_end] = ACTIONS(2966),
    [sym_identifier] = ACTIONS(2968),
    [anon_sym_SEMI] = ACTIONS(2966),
    [anon_sym_macro] = ACTIONS(2968),
    [anon_sym_LPAREN] = ACTIONS(2966),
    [anon_sym_LBRACE] = ACTIONS(2966),
    [anon_sym_RBRACE] = ACTIONS(2966),
    [anon_sym_macro_rules] = ACTIONS(2968),
    [anon_sym_BANG] = ACTIONS(2966),
    [anon_sym_LBRACK] = ACTIONS(2966),
    [anon_sym_STAR] = ACTIONS(2966),
    [anon_sym_u8] = ACTIONS(2968),
    [anon_sym_i8] = ACTIONS(2968),
    [anon_sym_u16] = ACTIONS(2968),
    [anon_sym_i16] = ACTIONS(2968),
    [anon_sym_u32] = ACTIONS(2968),
    [anon_sym_i32] = ACTIONS(2968),
    [anon_sym_u64] = ACTIONS(2968),
    [anon_sym_i64] = ACTIONS(2968),
    [anon_sym_u128] = ACTIONS(2968),
    [anon_sym_i128] = ACTIONS(2968),
    [anon_sym_isize] = ACTIONS(2968),
    [anon_sym_usize] = ACTIONS(2968),
    [anon_sym_f32] = ACTIONS(2968),
    [anon_sym_f64] = ACTIONS(2968),
    [anon_sym_bool] = ACTIONS(2968),
    [anon_sym_str] = ACTIONS(2968),
    [anon_sym_char] = ACTIONS(2968),
    [anon_sym_DASH] = ACTIONS(2966),
    [anon_sym_AMP] = ACTIONS(2966),
    [anon_sym_PIPE] = ACTIONS(2966),
    [anon_sym_LT] = ACTIONS(2966),
    [anon_sym_DOT_DOT] = ACTIONS(2966),
    [anon_sym_COLON_COLON] = ACTIONS(2966),
    [anon_sym_POUND] = ACTIONS(2966),
    [anon_sym_SQUOTE] = ACTIONS(2968),
    [anon_sym_async] = ACTIONS(2968),
    [anon_sym_auto] = ACTIONS(2968),
    [anon_sym_await] = ACTIONS(2968),
    [anon_sym_become] = ACTIONS(2968),
    [anon_sym_break] = ACTIONS(2968),
    [anon_sym_const] = ACTIONS(2968),
    [anon_sym_continue] = ACTIONS(2968),
    [anon_sym_default] = ACTIONS(2968),
    [anon_sym_enum] = ACTIONS(2968),
    [anon_sym_fn] = ACTIONS(2968),
    [anon_sym_for] = ACTIONS(2968),
    [anon_sym_gen] = ACTIONS(2968),
    [anon_sym_if] = ACTIONS(2968),
    [anon_sym_impl] = ACTIONS(2968),
    [anon_sym_let] = ACTIONS(2968),
    [anon_sym_loop] = ACTIONS(2968),
    [anon_sym_match] = ACTIONS(2968),
    [anon_sym_mod] = ACTIONS(2968),
    [anon_sym_pub] = ACTIONS(2968),
    [anon_sym_return] = ACTIONS(2968),
    [anon_sym_raw] = ACTIONS(2968),
    [anon_sym_static] = ACTIONS(2968),
    [anon_sym_struct] = ACTIONS(2968),
    [anon_sym_trait] = ACTIONS(2968),
    [anon_sym_type] = ACTIONS(2968),
    [anon_sym_union] = ACTIONS(2968),
    [anon_sym_unsafe] = ACTIONS(2968),
    [anon_sym_use] = ACTIONS(2968),
    [anon_sym_while] = ACTIONS(2968),
    [anon_sym_extern] = ACTIONS(2968),
    [anon_sym_safe] = ACTIONS(2968),
    [anon_sym_yield] = ACTIONS(2968),
    [anon_sym_move] = ACTIONS(2968),
    [anon_sym_try] = ACTIONS(2968),
    [sym_integer_literal] = ACTIONS(2966),
    [aux_sym_string_literal_token1] = ACTIONS(2966),
    [sym_char_literal] = ACTIONS(2966),
    [anon_sym_true] = ACTIONS(2968),
    [anon_sym_false] = ACTIONS(2968),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2968),
    [sym_super] = ACTIONS(2968),
    [sym_crate] = ACTIONS(2968),
    [aux_sym_metavariable_token1] = ACTIONS(2966),
    [sym__raw_string_literal_start] = ACTIONS(2966),
    [sym_float_literal] = ACTIONS(2966),
    [sym__outer_block_doc_comment_start] = ACTIONS(2966),
    [sym__outer_line_doc_comment_start] = ACTIONS(2966),
  },
  [STATE(800)] = {
    [sym_line_comment] = STATE(800),
    [sym_block_comment] = STATE(800),
    [ts_builtin_sym_end] = ACTIONS(2970),
    [sym_identifier] = ACTIONS(2972),
    [anon_sym_SEMI] = ACTIONS(2970),
    [anon_sym_macro] = ACTIONS(2972),
    [anon_sym_LPAREN] = ACTIONS(2970),
    [anon_sym_LBRACE] = ACTIONS(2970),
    [anon_sym_RBRACE] = ACTIONS(2970),
    [anon_sym_macro_rules] = ACTIONS(2972),
    [anon_sym_BANG] = ACTIONS(2970),
    [anon_sym_LBRACK] = ACTIONS(2970),
    [anon_sym_STAR] = ACTIONS(2970),
    [anon_sym_u8] = ACTIONS(2972),
    [anon_sym_i8] = ACTIONS(2972),
    [anon_sym_u16] = ACTIONS(2972),
    [anon_sym_i16] = ACTIONS(2972),
    [anon_sym_u32] = ACTIONS(2972),
    [anon_sym_i32] = ACTIONS(2972),
    [anon_sym_u64] = ACTIONS(2972),
    [anon_sym_i64] = ACTIONS(2972),
    [anon_sym_u128] = ACTIONS(2972),
    [anon_sym_i128] = ACTIONS(2972),
    [anon_sym_isize] = ACTIONS(2972),
    [anon_sym_usize] = ACTIONS(2972),
    [anon_sym_f32] = ACTIONS(2972),
    [anon_sym_f64] = ACTIONS(2972),
    [anon_sym_bool] = ACTIONS(2972),
    [anon_sym_str] = ACTIONS(2972),
    [anon_sym_char] = ACTIONS(2972),
    [anon_sym_DASH] = ACTIONS(2970),
    [anon_sym_AMP] = ACTIONS(2970),
    [anon_sym_PIPE] = ACTIONS(2970),
    [anon_sym_LT] = ACTIONS(2970),
    [anon_sym_DOT_DOT] = ACTIONS(2970),
    [anon_sym_COLON_COLON] = ACTIONS(2970),
    [anon_sym_POUND] = ACTIONS(2970),
    [anon_sym_SQUOTE] = ACTIONS(2972),
    [anon_sym_async] = ACTIONS(2972),
    [anon_sym_auto] = ACTIONS(2972),
    [anon_sym_await] = ACTIONS(2972),
    [anon_sym_become] = ACTIONS(2972),
    [anon_sym_break] = ACTIONS(2972),
    [anon_sym_const] = ACTIONS(2972),
    [anon_sym_continue] = ACTIONS(2972),
    [anon_sym_default] = ACTIONS(2972),
    [anon_sym_enum] = ACTIONS(2972),
    [anon_sym_fn] = ACTIONS(2972),
    [anon_sym_for] = ACTIONS(2972),
    [anon_sym_gen] = ACTIONS(2972),
    [anon_sym_if] = ACTIONS(2972),
    [anon_sym_impl] = ACTIONS(2972),
    [anon_sym_let] = ACTIONS(2972),
    [anon_sym_loop] = ACTIONS(2972),
    [anon_sym_match] = ACTIONS(2972),
    [anon_sym_mod] = ACTIONS(2972),
    [anon_sym_pub] = ACTIONS(2972),
    [anon_sym_return] = ACTIONS(2972),
    [anon_sym_raw] = ACTIONS(2972),
    [anon_sym_static] = ACTIONS(2972),
    [anon_sym_struct] = ACTIONS(2972),
    [anon_sym_trait] = ACTIONS(2972),
    [anon_sym_type] = ACTIONS(2972),
    [anon_sym_union] = ACTIONS(2972),
    [anon_sym_unsafe] = ACTIONS(2972),
    [anon_sym_use] = ACTIONS(2972),
    [anon_sym_while] = ACTIONS(2972),
    [anon_sym_extern] = ACTIONS(2972),
    [anon_sym_safe] = ACTIONS(2972),
    [anon_sym_yield] = ACTIONS(2972),
    [anon_sym_move] = ACTIONS(2972),
    [anon_sym_try] = ACTIONS(2972),
    [sym_integer_literal] = ACTIONS(2970),
    [aux_sym_string_literal_token1] = ACTIONS(2970),
    [sym_char_literal] = ACTIONS(2970),
    [anon_sym_true] = ACTIONS(2972),
    [anon_sym_false] = ACTIONS(2972),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2972),
    [sym_super] = ACTIONS(2972),
    [sym_crate] = ACTIONS(2972),
    [aux_sym_metavariable_token1] = ACTIONS(2970),
    [sym__raw_string_literal_start] = ACTIONS(2970),
    [sym_float_literal] = ACTIONS(2970),
    [sym__outer_block_doc_comment_start] = ACTIONS(2970),
    [sym__outer_line_doc_comment_start] = ACTIONS(2970),
  },
  [STATE(801)] = {
    [sym_line_comment] = STATE(801),
    [sym_block_comment] = STATE(801),
    [ts_builtin_sym_end] = ACTIONS(2974),
    [sym_identifier] = ACTIONS(2976),
    [anon_sym_SEMI] = ACTIONS(2974),
    [anon_sym_macro] = ACTIONS(2976),
    [anon_sym_LPAREN] = ACTIONS(2974),
    [anon_sym_LBRACE] = ACTIONS(2974),
    [anon_sym_RBRACE] = ACTIONS(2974),
    [anon_sym_macro_rules] = ACTIONS(2976),
    [anon_sym_BANG] = ACTIONS(2974),
    [anon_sym_LBRACK] = ACTIONS(2974),
    [anon_sym_STAR] = ACTIONS(2974),
    [anon_sym_u8] = ACTIONS(2976),
    [anon_sym_i8] = ACTIONS(2976),
    [anon_sym_u16] = ACTIONS(2976),
    [anon_sym_i16] = ACTIONS(2976),
    [anon_sym_u32] = ACTIONS(2976),
    [anon_sym_i32] = ACTIONS(2976),
    [anon_sym_u64] = ACTIONS(2976),
    [anon_sym_i64] = ACTIONS(2976),
    [anon_sym_u128] = ACTIONS(2976),
    [anon_sym_i128] = ACTIONS(2976),
    [anon_sym_isize] = ACTIONS(2976),
    [anon_sym_usize] = ACTIONS(2976),
    [anon_sym_f32] = ACTIONS(2976),
    [anon_sym_f64] = ACTIONS(2976),
    [anon_sym_bool] = ACTIONS(2976),
    [anon_sym_str] = ACTIONS(2976),
    [anon_sym_char] = ACTIONS(2976),
    [anon_sym_DASH] = ACTIONS(2974),
    [anon_sym_AMP] = ACTIONS(2974),
    [anon_sym_PIPE] = ACTIONS(2974),
    [anon_sym_LT] = ACTIONS(2974),
    [anon_sym_DOT_DOT] = ACTIONS(2974),
    [anon_sym_COLON_COLON] = ACTIONS(2974),
    [anon_sym_POUND] = ACTIONS(2974),
    [anon_sym_SQUOTE] = ACTIONS(2976),
    [anon_sym_async] = ACTIONS(2976),
    [anon_sym_auto] = ACTIONS(2976),
    [anon_sym_await] = ACTIONS(2976),
    [anon_sym_become] = ACTIONS(2976),
    [anon_sym_break] = ACTIONS(2976),
    [anon_sym_const] = ACTIONS(2976),
    [anon_sym_continue] = ACTIONS(2976),
    [anon_sym_default] = ACTIONS(2976),
    [anon_sym_enum] = ACTIONS(2976),
    [anon_sym_fn] = ACTIONS(2976),
    [anon_sym_for] = ACTIONS(2976),
    [anon_sym_gen] = ACTIONS(2976),
    [anon_sym_if] = ACTIONS(2976),
    [anon_sym_impl] = ACTIONS(2976),
    [anon_sym_let] = ACTIONS(2976),
    [anon_sym_loop] = ACTIONS(2976),
    [anon_sym_match] = ACTIONS(2976),
    [anon_sym_mod] = ACTIONS(2976),
    [anon_sym_pub] = ACTIONS(2976),
    [anon_sym_return] = ACTIONS(2976),
    [anon_sym_raw] = ACTIONS(2976),
    [anon_sym_static] = ACTIONS(2976),
    [anon_sym_struct] = ACTIONS(2976),
    [anon_sym_trait] = ACTIONS(2976),
    [anon_sym_type] = ACTIONS(2976),
    [anon_sym_union] = ACTIONS(2976),
    [anon_sym_unsafe] = ACTIONS(2976),
    [anon_sym_use] = ACTIONS(2976),
    [anon_sym_while] = ACTIONS(2976),
    [anon_sym_extern] = ACTIONS(2976),
    [anon_sym_safe] = ACTIONS(2976),
    [anon_sym_yield] = ACTIONS(2976),
    [anon_sym_move] = ACTIONS(2976),
    [anon_sym_try] = ACTIONS(2976),
    [sym_integer_literal] = ACTIONS(2974),
    [aux_sym_string_literal_token1] = ACTIONS(2974),
    [sym_char_literal] = ACTIONS(2974),
    [anon_sym_true] = ACTIONS(2976),
    [anon_sym_false] = ACTIONS(2976),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2976),
    [sym_super] = ACTIONS(2976),
    [sym_crate] = ACTIONS(2976),
    [aux_sym_metavariable_token1] = ACTIONS(2974),
    [sym__raw_string_literal_start] = ACTIONS(2974),
    [sym_float_literal] = ACTIONS(2974),
    [sym__outer_block_doc_comment_start] = ACTIONS(2974),
    [sym__outer_line_doc_comment_start] = ACTIONS(2974),
  },
  [STATE(802)] = {
    [sym_line_comment] = STATE(802),
    [sym_block_comment] = STATE(802),
    [ts_builtin_sym_end] = ACTIONS(2978),
    [sym_identifier] = ACTIONS(2980),
    [anon_sym_SEMI] = ACTIONS(2978),
    [anon_sym_macro] = ACTIONS(2980),
    [anon_sym_LPAREN] = ACTIONS(2978),
    [anon_sym_LBRACE] = ACTIONS(2978),
    [anon_sym_RBRACE] = ACTIONS(2978),
    [anon_sym_macro_rules] = ACTIONS(2980),
    [anon_sym_BANG] = ACTIONS(2978),
    [anon_sym_LBRACK] = ACTIONS(2978),
    [anon_sym_STAR] = ACTIONS(2978),
    [anon_sym_u8] = ACTIONS(2980),
    [anon_sym_i8] = ACTIONS(2980),
    [anon_sym_u16] = ACTIONS(2980),
    [anon_sym_i16] = ACTIONS(2980),
    [anon_sym_u32] = ACTIONS(2980),
    [anon_sym_i32] = ACTIONS(2980),
    [anon_sym_u64] = ACTIONS(2980),
    [anon_sym_i64] = ACTIONS(2980),
    [anon_sym_u128] = ACTIONS(2980),
    [anon_sym_i128] = ACTIONS(2980),
    [anon_sym_isize] = ACTIONS(2980),
    [anon_sym_usize] = ACTIONS(2980),
    [anon_sym_f32] = ACTIONS(2980),
    [anon_sym_f64] = ACTIONS(2980),
    [anon_sym_bool] = ACTIONS(2980),
    [anon_sym_str] = ACTIONS(2980),
    [anon_sym_char] = ACTIONS(2980),
    [anon_sym_DASH] = ACTIONS(2978),
    [anon_sym_AMP] = ACTIONS(2978),
    [anon_sym_PIPE] = ACTIONS(2978),
    [anon_sym_LT] = ACTIONS(2978),
    [anon_sym_DOT_DOT] = ACTIONS(2978),
    [anon_sym_COLON_COLON] = ACTIONS(2978),
    [anon_sym_POUND] = ACTIONS(2978),
    [anon_sym_SQUOTE] = ACTIONS(2980),
    [anon_sym_async] = ACTIONS(2980),
    [anon_sym_auto] = ACTIONS(2980),
    [anon_sym_await] = ACTIONS(2980),
    [anon_sym_become] = ACTIONS(2980),
    [anon_sym_break] = ACTIONS(2980),
    [anon_sym_const] = ACTIONS(2980),
    [anon_sym_continue] = ACTIONS(2980),
    [anon_sym_default] = ACTIONS(2980),
    [anon_sym_enum] = ACTIONS(2980),
    [anon_sym_fn] = ACTIONS(2980),
    [anon_sym_for] = ACTIONS(2980),
    [anon_sym_gen] = ACTIONS(2980),
    [anon_sym_if] = ACTIONS(2980),
    [anon_sym_impl] = ACTIONS(2980),
    [anon_sym_let] = ACTIONS(2980),
    [anon_sym_loop] = ACTIONS(2980),
    [anon_sym_match] = ACTIONS(2980),
    [anon_sym_mod] = ACTIONS(2980),
    [anon_sym_pub] = ACTIONS(2980),
    [anon_sym_return] = ACTIONS(2980),
    [anon_sym_raw] = ACTIONS(2980),
    [anon_sym_static] = ACTIONS(2980),
    [anon_sym_struct] = ACTIONS(2980),
    [anon_sym_trait] = ACTIONS(2980),
    [anon_sym_type] = ACTIONS(2980),
    [anon_sym_union] = ACTIONS(2980),
    [anon_sym_unsafe] = ACTIONS(2980),
    [anon_sym_use] = ACTIONS(2980),
    [anon_sym_while] = ACTIONS(2980),
    [anon_sym_extern] = ACTIONS(2980),
    [anon_sym_safe] = ACTIONS(2980),
    [anon_sym_yield] = ACTIONS(2980),
    [anon_sym_move] = ACTIONS(2980),
    [anon_sym_try] = ACTIONS(2980),
    [sym_integer_literal] = ACTIONS(2978),
    [aux_sym_string_literal_token1] = ACTIONS(2978),
    [sym_char_literal] = ACTIONS(2978),
    [anon_sym_true] = ACTIONS(2980),
    [anon_sym_false] = ACTIONS(2980),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2980),
    [sym_super] = ACTIONS(2980),
    [sym_crate] = ACTIONS(2980),
    [aux_sym_metavariable_token1] = ACTIONS(2978),
    [sym__raw_string_literal_start] = ACTIONS(2978),
    [sym_float_literal] = ACTIONS(2978),
    [sym__outer_block_doc_comment_start] = ACTIONS(2978),
    [sym__outer_line_doc_comment_start] = ACTIONS(2978),
  },
  [STATE(803)] = {
    [sym_line_comment] = STATE(803),
    [sym_block_comment] = STATE(803),
    [ts_builtin_sym_end] = ACTIONS(2982),
    [sym_identifier] = ACTIONS(2984),
    [anon_sym_SEMI] = ACTIONS(2982),
    [anon_sym_macro] = ACTIONS(2984),
    [anon_sym_LPAREN] = ACTIONS(2982),
    [anon_sym_LBRACE] = ACTIONS(2982),
    [anon_sym_RBRACE] = ACTIONS(2982),
    [anon_sym_macro_rules] = ACTIONS(2984),
    [anon_sym_BANG] = ACTIONS(2982),
    [anon_sym_LBRACK] = ACTIONS(2982),
    [anon_sym_STAR] = ACTIONS(2982),
    [anon_sym_u8] = ACTIONS(2984),
    [anon_sym_i8] = ACTIONS(2984),
    [anon_sym_u16] = ACTIONS(2984),
    [anon_sym_i16] = ACTIONS(2984),
    [anon_sym_u32] = ACTIONS(2984),
    [anon_sym_i32] = ACTIONS(2984),
    [anon_sym_u64] = ACTIONS(2984),
    [anon_sym_i64] = ACTIONS(2984),
    [anon_sym_u128] = ACTIONS(2984),
    [anon_sym_i128] = ACTIONS(2984),
    [anon_sym_isize] = ACTIONS(2984),
    [anon_sym_usize] = ACTIONS(2984),
    [anon_sym_f32] = ACTIONS(2984),
    [anon_sym_f64] = ACTIONS(2984),
    [anon_sym_bool] = ACTIONS(2984),
    [anon_sym_str] = ACTIONS(2984),
    [anon_sym_char] = ACTIONS(2984),
    [anon_sym_DASH] = ACTIONS(2982),
    [anon_sym_AMP] = ACTIONS(2982),
    [anon_sym_PIPE] = ACTIONS(2982),
    [anon_sym_LT] = ACTIONS(2982),
    [anon_sym_DOT_DOT] = ACTIONS(2982),
    [anon_sym_COLON_COLON] = ACTIONS(2982),
    [anon_sym_POUND] = ACTIONS(2982),
    [anon_sym_SQUOTE] = ACTIONS(2984),
    [anon_sym_async] = ACTIONS(2984),
    [anon_sym_auto] = ACTIONS(2984),
    [anon_sym_await] = ACTIONS(2984),
    [anon_sym_become] = ACTIONS(2984),
    [anon_sym_break] = ACTIONS(2984),
    [anon_sym_const] = ACTIONS(2984),
    [anon_sym_continue] = ACTIONS(2984),
    [anon_sym_default] = ACTIONS(2984),
    [anon_sym_enum] = ACTIONS(2984),
    [anon_sym_fn] = ACTIONS(2984),
    [anon_sym_for] = ACTIONS(2984),
    [anon_sym_gen] = ACTIONS(2984),
    [anon_sym_if] = ACTIONS(2984),
    [anon_sym_impl] = ACTIONS(2984),
    [anon_sym_let] = ACTIONS(2984),
    [anon_sym_loop] = ACTIONS(2984),
    [anon_sym_match] = ACTIONS(2984),
    [anon_sym_mod] = ACTIONS(2984),
    [anon_sym_pub] = ACTIONS(2984),
    [anon_sym_return] = ACTIONS(2984),
    [anon_sym_raw] = ACTIONS(2984),
    [anon_sym_static] = ACTIONS(2984),
    [anon_sym_struct] = ACTIONS(2984),
    [anon_sym_trait] = ACTIONS(2984),
    [anon_sym_type] = ACTIONS(2984),
    [anon_sym_union] = ACTIONS(2984),
    [anon_sym_unsafe] = ACTIONS(2984),
    [anon_sym_use] = ACTIONS(2984),
    [anon_sym_while] = ACTIONS(2984),
    [anon_sym_extern] = ACTIONS(2984),
    [anon_sym_safe] = ACTIONS(2984),
    [anon_sym_yield] = ACTIONS(2984),
    [anon_sym_move] = ACTIONS(2984),
    [anon_sym_try] = ACTIONS(2984),
    [sym_integer_literal] = ACTIONS(2982),
    [aux_sym_string_literal_token1] = ACTIONS(2982),
    [sym_char_literal] = ACTIONS(2982),
    [anon_sym_true] = ACTIONS(2984),
    [anon_sym_false] = ACTIONS(2984),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2984),
    [sym_super] = ACTIONS(2984),
    [sym_crate] = ACTIONS(2984),
    [aux_sym_metavariable_token1] = ACTIONS(2982),
    [sym__raw_string_literal_start] = ACTIONS(2982),
    [sym_float_literal] = ACTIONS(2982),
    [sym__outer_block_doc_comment_start] = ACTIONS(2982),
    [sym__outer_line_doc_comment_start] = ACTIONS(2982),
  },
  [STATE(804)] = {
    [sym_line_comment] = STATE(804),
    [sym_block_comment] = STATE(804),
    [ts_builtin_sym_end] = ACTIONS(2986),
    [sym_identifier] = ACTIONS(2988),
    [anon_sym_SEMI] = ACTIONS(2986),
    [anon_sym_macro] = ACTIONS(2988),
    [anon_sym_LPAREN] = ACTIONS(2986),
    [anon_sym_LBRACE] = ACTIONS(2986),
    [anon_sym_RBRACE] = ACTIONS(2986),
    [anon_sym_macro_rules] = ACTIONS(2988),
    [anon_sym_BANG] = ACTIONS(2986),
    [anon_sym_LBRACK] = ACTIONS(2986),
    [anon_sym_STAR] = ACTIONS(2986),
    [anon_sym_u8] = ACTIONS(2988),
    [anon_sym_i8] = ACTIONS(2988),
    [anon_sym_u16] = ACTIONS(2988),
    [anon_sym_i16] = ACTIONS(2988),
    [anon_sym_u32] = ACTIONS(2988),
    [anon_sym_i32] = ACTIONS(2988),
    [anon_sym_u64] = ACTIONS(2988),
    [anon_sym_i64] = ACTIONS(2988),
    [anon_sym_u128] = ACTIONS(2988),
    [anon_sym_i128] = ACTIONS(2988),
    [anon_sym_isize] = ACTIONS(2988),
    [anon_sym_usize] = ACTIONS(2988),
    [anon_sym_f32] = ACTIONS(2988),
    [anon_sym_f64] = ACTIONS(2988),
    [anon_sym_bool] = ACTIONS(2988),
    [anon_sym_str] = ACTIONS(2988),
    [anon_sym_char] = ACTIONS(2988),
    [anon_sym_DASH] = ACTIONS(2986),
    [anon_sym_AMP] = ACTIONS(2986),
    [anon_sym_PIPE] = ACTIONS(2986),
    [anon_sym_LT] = ACTIONS(2986),
    [anon_sym_DOT_DOT] = ACTIONS(2986),
    [anon_sym_COLON_COLON] = ACTIONS(2986),
    [anon_sym_POUND] = ACTIONS(2986),
    [anon_sym_SQUOTE] = ACTIONS(2988),
    [anon_sym_async] = ACTIONS(2988),
    [anon_sym_auto] = ACTIONS(2988),
    [anon_sym_await] = ACTIONS(2988),
    [anon_sym_become] = ACTIONS(2988),
    [anon_sym_break] = ACTIONS(2988),
    [anon_sym_const] = ACTIONS(2988),
    [anon_sym_continue] = ACTIONS(2988),
    [anon_sym_default] = ACTIONS(2988),
    [anon_sym_enum] = ACTIONS(2988),
    [anon_sym_fn] = ACTIONS(2988),
    [anon_sym_for] = ACTIONS(2988),
    [anon_sym_gen] = ACTIONS(2988),
    [anon_sym_if] = ACTIONS(2988),
    [anon_sym_impl] = ACTIONS(2988),
    [anon_sym_let] = ACTIONS(2988),
    [anon_sym_loop] = ACTIONS(2988),
    [anon_sym_match] = ACTIONS(2988),
    [anon_sym_mod] = ACTIONS(2988),
    [anon_sym_pub] = ACTIONS(2988),
    [anon_sym_return] = ACTIONS(2988),
    [anon_sym_raw] = ACTIONS(2988),
    [anon_sym_static] = ACTIONS(2988),
    [anon_sym_struct] = ACTIONS(2988),
    [anon_sym_trait] = ACTIONS(2988),
    [anon_sym_type] = ACTIONS(2988),
    [anon_sym_union] = ACTIONS(2988),
    [anon_sym_unsafe] = ACTIONS(2988),
    [anon_sym_use] = ACTIONS(2988),
    [anon_sym_while] = ACTIONS(2988),
    [anon_sym_extern] = ACTIONS(2988),
    [anon_sym_safe] = ACTIONS(2988),
    [anon_sym_yield] = ACTIONS(2988),
    [anon_sym_move] = ACTIONS(2988),
    [anon_sym_try] = ACTIONS(2988),
    [sym_integer_literal] = ACTIONS(2986),
    [aux_sym_string_literal_token1] = ACTIONS(2986),
    [sym_char_literal] = ACTIONS(2986),
    [anon_sym_true] = ACTIONS(2988),
    [anon_sym_false] = ACTIONS(2988),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2988),
    [sym_super] = ACTIONS(2988),
    [sym_crate] = ACTIONS(2988),
    [aux_sym_metavariable_token1] = ACTIONS(2986),
    [sym__raw_string_literal_start] = ACTIONS(2986),
    [sym_float_literal] = ACTIONS(2986),
    [sym__outer_block_doc_comment_start] = ACTIONS(2986),
    [sym__outer_line_doc_comment_start] = ACTIONS(2986),
  },
  [STATE(805)] = {
    [sym_line_comment] = STATE(805),
    [sym_block_comment] = STATE(805),
    [ts_builtin_sym_end] = ACTIONS(2990),
    [sym_identifier] = ACTIONS(2992),
    [anon_sym_SEMI] = ACTIONS(2990),
    [anon_sym_macro] = ACTIONS(2992),
    [anon_sym_LPAREN] = ACTIONS(2990),
    [anon_sym_LBRACE] = ACTIONS(2990),
    [anon_sym_RBRACE] = ACTIONS(2990),
    [anon_sym_macro_rules] = ACTIONS(2992),
    [anon_sym_BANG] = ACTIONS(2990),
    [anon_sym_LBRACK] = ACTIONS(2990),
    [anon_sym_STAR] = ACTIONS(2990),
    [anon_sym_u8] = ACTIONS(2992),
    [anon_sym_i8] = ACTIONS(2992),
    [anon_sym_u16] = ACTIONS(2992),
    [anon_sym_i16] = ACTIONS(2992),
    [anon_sym_u32] = ACTIONS(2992),
    [anon_sym_i32] = ACTIONS(2992),
    [anon_sym_u64] = ACTIONS(2992),
    [anon_sym_i64] = ACTIONS(2992),
    [anon_sym_u128] = ACTIONS(2992),
    [anon_sym_i128] = ACTIONS(2992),
    [anon_sym_isize] = ACTIONS(2992),
    [anon_sym_usize] = ACTIONS(2992),
    [anon_sym_f32] = ACTIONS(2992),
    [anon_sym_f64] = ACTIONS(2992),
    [anon_sym_bool] = ACTIONS(2992),
    [anon_sym_str] = ACTIONS(2992),
    [anon_sym_char] = ACTIONS(2992),
    [anon_sym_DASH] = ACTIONS(2990),
    [anon_sym_AMP] = ACTIONS(2990),
    [anon_sym_PIPE] = ACTIONS(2990),
    [anon_sym_LT] = ACTIONS(2990),
    [anon_sym_DOT_DOT] = ACTIONS(2990),
    [anon_sym_COLON_COLON] = ACTIONS(2990),
    [anon_sym_POUND] = ACTIONS(2990),
    [anon_sym_SQUOTE] = ACTIONS(2992),
    [anon_sym_async] = ACTIONS(2992),
    [anon_sym_auto] = ACTIONS(2992),
    [anon_sym_await] = ACTIONS(2992),
    [anon_sym_become] = ACTIONS(2992),
    [anon_sym_break] = ACTIONS(2992),
    [anon_sym_const] = ACTIONS(2992),
    [anon_sym_continue] = ACTIONS(2992),
    [anon_sym_default] = ACTIONS(2992),
    [anon_sym_enum] = ACTIONS(2992),
    [anon_sym_fn] = ACTIONS(2992),
    [anon_sym_for] = ACTIONS(2992),
    [anon_sym_gen] = ACTIONS(2992),
    [anon_sym_if] = ACTIONS(2992),
    [anon_sym_impl] = ACTIONS(2992),
    [anon_sym_let] = ACTIONS(2992),
    [anon_sym_loop] = ACTIONS(2992),
    [anon_sym_match] = ACTIONS(2992),
    [anon_sym_mod] = ACTIONS(2992),
    [anon_sym_pub] = ACTIONS(2992),
    [anon_sym_return] = ACTIONS(2992),
    [anon_sym_raw] = ACTIONS(2992),
    [anon_sym_static] = ACTIONS(2992),
    [anon_sym_struct] = ACTIONS(2992),
    [anon_sym_trait] = ACTIONS(2992),
    [anon_sym_type] = ACTIONS(2992),
    [anon_sym_union] = ACTIONS(2992),
    [anon_sym_unsafe] = ACTIONS(2992),
    [anon_sym_use] = ACTIONS(2992),
    [anon_sym_while] = ACTIONS(2992),
    [anon_sym_extern] = ACTIONS(2992),
    [anon_sym_safe] = ACTIONS(2992),
    [anon_sym_yield] = ACTIONS(2992),
    [anon_sym_move] = ACTIONS(2992),
    [anon_sym_try] = ACTIONS(2992),
    [sym_integer_literal] = ACTIONS(2990),
    [aux_sym_string_literal_token1] = ACTIONS(2990),
    [sym_char_literal] = ACTIONS(2990),
    [anon_sym_true] = ACTIONS(2992),
    [anon_sym_false] = ACTIONS(2992),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2992),
    [sym_super] = ACTIONS(2992),
    [sym_crate] = ACTIONS(2992),
    [aux_sym_metavariable_token1] = ACTIONS(2990),
    [sym__raw_string_literal_start] = ACTIONS(2990),
    [sym_float_literal] = ACTIONS(2990),
    [sym__outer_block_doc_comment_start] = ACTIONS(2990),
    [sym__outer_line_doc_comment_start] = ACTIONS(2990),
  },
  [STATE(806)] = {
    [sym_line_comment] = STATE(806),
    [sym_block_comment] = STATE(806),
    [ts_builtin_sym_end] = ACTIONS(2994),
    [sym_identifier] = ACTIONS(2996),
    [anon_sym_SEMI] = ACTIONS(2994),
    [anon_sym_macro] = ACTIONS(2996),
    [anon_sym_LPAREN] = ACTIONS(2994),
    [anon_sym_LBRACE] = ACTIONS(2994),
    [anon_sym_RBRACE] = ACTIONS(2994),
    [anon_sym_macro_rules] = ACTIONS(2996),
    [anon_sym_BANG] = ACTIONS(2994),
    [anon_sym_LBRACK] = ACTIONS(2994),
    [anon_sym_STAR] = ACTIONS(2994),
    [anon_sym_u8] = ACTIONS(2996),
    [anon_sym_i8] = ACTIONS(2996),
    [anon_sym_u16] = ACTIONS(2996),
    [anon_sym_i16] = ACTIONS(2996),
    [anon_sym_u32] = ACTIONS(2996),
    [anon_sym_i32] = ACTIONS(2996),
    [anon_sym_u64] = ACTIONS(2996),
    [anon_sym_i64] = ACTIONS(2996),
    [anon_sym_u128] = ACTIONS(2996),
    [anon_sym_i128] = ACTIONS(2996),
    [anon_sym_isize] = ACTIONS(2996),
    [anon_sym_usize] = ACTIONS(2996),
    [anon_sym_f32] = ACTIONS(2996),
    [anon_sym_f64] = ACTIONS(2996),
    [anon_sym_bool] = ACTIONS(2996),
    [anon_sym_str] = ACTIONS(2996),
    [anon_sym_char] = ACTIONS(2996),
    [anon_sym_DASH] = ACTIONS(2994),
    [anon_sym_AMP] = ACTIONS(2994),
    [anon_sym_PIPE] = ACTIONS(2994),
    [anon_sym_LT] = ACTIONS(2994),
    [anon_sym_DOT_DOT] = ACTIONS(2994),
    [anon_sym_COLON_COLON] = ACTIONS(2994),
    [anon_sym_POUND] = ACTIONS(2994),
    [anon_sym_SQUOTE] = ACTIONS(2996),
    [anon_sym_async] = ACTIONS(2996),
    [anon_sym_auto] = ACTIONS(2996),
    [anon_sym_await] = ACTIONS(2996),
    [anon_sym_become] = ACTIONS(2996),
    [anon_sym_break] = ACTIONS(2996),
    [anon_sym_const] = ACTIONS(2996),
    [anon_sym_continue] = ACTIONS(2996),
    [anon_sym_default] = ACTIONS(2996),
    [anon_sym_enum] = ACTIONS(2996),
    [anon_sym_fn] = ACTIONS(2996),
    [anon_sym_for] = ACTIONS(2996),
    [anon_sym_gen] = ACTIONS(2996),
    [anon_sym_if] = ACTIONS(2996),
    [anon_sym_impl] = ACTIONS(2996),
    [anon_sym_let] = ACTIONS(2996),
    [anon_sym_loop] = ACTIONS(2996),
    [anon_sym_match] = ACTIONS(2996),
    [anon_sym_mod] = ACTIONS(2996),
    [anon_sym_pub] = ACTIONS(2996),
    [anon_sym_return] = ACTIONS(2996),
    [anon_sym_raw] = ACTIONS(2996),
    [anon_sym_static] = ACTIONS(2996),
    [anon_sym_struct] = ACTIONS(2996),
    [anon_sym_trait] = ACTIONS(2996),
    [anon_sym_type] = ACTIONS(2996),
    [anon_sym_union] = ACTIONS(2996),
    [anon_sym_unsafe] = ACTIONS(2996),
    [anon_sym_use] = ACTIONS(2996),
    [anon_sym_while] = ACTIONS(2996),
    [anon_sym_extern] = ACTIONS(2996),
    [anon_sym_safe] = ACTIONS(2996),
    [anon_sym_yield] = ACTIONS(2996),
    [anon_sym_move] = ACTIONS(2996),
    [anon_sym_try] = ACTIONS(2996),
    [sym_integer_literal] = ACTIONS(2994),
    [aux_sym_string_literal_token1] = ACTIONS(2994),
    [sym_char_literal] = ACTIONS(2994),
    [anon_sym_true] = ACTIONS(2996),
    [anon_sym_false] = ACTIONS(2996),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2996),
    [sym_super] = ACTIONS(2996),
    [sym_crate] = ACTIONS(2996),
    [aux_sym_metavariable_token1] = ACTIONS(2994),
    [sym__raw_string_literal_start] = ACTIONS(2994),
    [sym_float_literal] = ACTIONS(2994),
    [sym__outer_block_doc_comment_start] = ACTIONS(2994),
    [sym__outer_line_doc_comment_start] = ACTIONS(2994),
  },
  [STATE(807)] = {
    [sym_line_comment] = STATE(807),
    [sym_block_comment] = STATE(807),
    [ts_builtin_sym_end] = ACTIONS(2998),
    [sym_identifier] = ACTIONS(3000),
    [anon_sym_SEMI] = ACTIONS(2998),
    [anon_sym_macro] = ACTIONS(3000),
    [anon_sym_LPAREN] = ACTIONS(2998),
    [anon_sym_LBRACE] = ACTIONS(2998),
    [anon_sym_RBRACE] = ACTIONS(2998),
    [anon_sym_macro_rules] = ACTIONS(3000),
    [anon_sym_BANG] = ACTIONS(2998),
    [anon_sym_LBRACK] = ACTIONS(2998),
    [anon_sym_STAR] = ACTIONS(2998),
    [anon_sym_u8] = ACTIONS(3000),
    [anon_sym_i8] = ACTIONS(3000),
    [anon_sym_u16] = ACTIONS(3000),
    [anon_sym_i16] = ACTIONS(3000),
    [anon_sym_u32] = ACTIONS(3000),
    [anon_sym_i32] = ACTIONS(3000),
    [anon_sym_u64] = ACTIONS(3000),
    [anon_sym_i64] = ACTIONS(3000),
    [anon_sym_u128] = ACTIONS(3000),
    [anon_sym_i128] = ACTIONS(3000),
    [anon_sym_isize] = ACTIONS(3000),
    [anon_sym_usize] = ACTIONS(3000),
    [anon_sym_f32] = ACTIONS(3000),
    [anon_sym_f64] = ACTIONS(3000),
    [anon_sym_bool] = ACTIONS(3000),
    [anon_sym_str] = ACTIONS(3000),
    [anon_sym_char] = ACTIONS(3000),
    [anon_sym_DASH] = ACTIONS(2998),
    [anon_sym_AMP] = ACTIONS(2998),
    [anon_sym_PIPE] = ACTIONS(2998),
    [anon_sym_LT] = ACTIONS(2998),
    [anon_sym_DOT_DOT] = ACTIONS(2998),
    [anon_sym_COLON_COLON] = ACTIONS(2998),
    [anon_sym_POUND] = ACTIONS(2998),
    [anon_sym_SQUOTE] = ACTIONS(3000),
    [anon_sym_async] = ACTIONS(3000),
    [anon_sym_auto] = ACTIONS(3000),
    [anon_sym_await] = ACTIONS(3000),
    [anon_sym_become] = ACTIONS(3000),
    [anon_sym_break] = ACTIONS(3000),
    [anon_sym_const] = ACTIONS(3000),
    [anon_sym_continue] = ACTIONS(3000),
    [anon_sym_default] = ACTIONS(3000),
    [anon_sym_enum] = ACTIONS(3000),
    [anon_sym_fn] = ACTIONS(3000),
    [anon_sym_for] = ACTIONS(3000),
    [anon_sym_gen] = ACTIONS(3000),
    [anon_sym_if] = ACTIONS(3000),
    [anon_sym_impl] = ACTIONS(3000),
    [anon_sym_let] = ACTIONS(3000),
    [anon_sym_loop] = ACTIONS(3000),
    [anon_sym_match] = ACTIONS(3000),
    [anon_sym_mod] = ACTIONS(3000),
    [anon_sym_pub] = ACTIONS(3000),
    [anon_sym_return] = ACTIONS(3000),
    [anon_sym_raw] = ACTIONS(3000),
    [anon_sym_static] = ACTIONS(3000),
    [anon_sym_struct] = ACTIONS(3000),
    [anon_sym_trait] = ACTIONS(3000),
    [anon_sym_type] = ACTIONS(3000),
    [anon_sym_union] = ACTIONS(3000),
    [anon_sym_unsafe] = ACTIONS(3000),
    [anon_sym_use] = ACTIONS(3000),
    [anon_sym_while] = ACTIONS(3000),
    [anon_sym_extern] = ACTIONS(3000),
    [anon_sym_safe] = ACTIONS(3000),
    [anon_sym_yield] = ACTIONS(3000),
    [anon_sym_move] = ACTIONS(3000),
    [anon_sym_try] = ACTIONS(3000),
    [sym_integer_literal] = ACTIONS(2998),
    [aux_sym_string_literal_token1] = ACTIONS(2998),
    [sym_char_literal] = ACTIONS(2998),
    [anon_sym_true] = ACTIONS(3000),
    [anon_sym_false] = ACTIONS(3000),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3000),
    [sym_super] = ACTIONS(3000),
    [sym_crate] = ACTIONS(3000),
    [aux_sym_metavariable_token1] = ACTIONS(2998),
    [sym__raw_string_literal_start] = ACTIONS(2998),
    [sym_float_literal] = ACTIONS(2998),
    [sym__outer_block_doc_comment_start] = ACTIONS(2998),
    [sym__outer_line_doc_comment_start] = ACTIONS(2998),
  },
  [STATE(808)] = {
    [sym_line_comment] = STATE(808),
    [sym_block_comment] = STATE(808),
    [ts_builtin_sym_end] = ACTIONS(3002),
    [sym_identifier] = ACTIONS(3004),
    [anon_sym_SEMI] = ACTIONS(3002),
    [anon_sym_macro] = ACTIONS(3004),
    [anon_sym_LPAREN] = ACTIONS(3002),
    [anon_sym_LBRACE] = ACTIONS(3002),
    [anon_sym_RBRACE] = ACTIONS(3002),
    [anon_sym_macro_rules] = ACTIONS(3004),
    [anon_sym_BANG] = ACTIONS(3002),
    [anon_sym_LBRACK] = ACTIONS(3002),
    [anon_sym_STAR] = ACTIONS(3002),
    [anon_sym_u8] = ACTIONS(3004),
    [anon_sym_i8] = ACTIONS(3004),
    [anon_sym_u16] = ACTIONS(3004),
    [anon_sym_i16] = ACTIONS(3004),
    [anon_sym_u32] = ACTIONS(3004),
    [anon_sym_i32] = ACTIONS(3004),
    [anon_sym_u64] = ACTIONS(3004),
    [anon_sym_i64] = ACTIONS(3004),
    [anon_sym_u128] = ACTIONS(3004),
    [anon_sym_i128] = ACTIONS(3004),
    [anon_sym_isize] = ACTIONS(3004),
    [anon_sym_usize] = ACTIONS(3004),
    [anon_sym_f32] = ACTIONS(3004),
    [anon_sym_f64] = ACTIONS(3004),
    [anon_sym_bool] = ACTIONS(3004),
    [anon_sym_str] = ACTIONS(3004),
    [anon_sym_char] = ACTIONS(3004),
    [anon_sym_DASH] = ACTIONS(3002),
    [anon_sym_AMP] = ACTIONS(3002),
    [anon_sym_PIPE] = ACTIONS(3002),
    [anon_sym_LT] = ACTIONS(3002),
    [anon_sym_DOT_DOT] = ACTIONS(3002),
    [anon_sym_COLON_COLON] = ACTIONS(3002),
    [anon_sym_POUND] = ACTIONS(3002),
    [anon_sym_SQUOTE] = ACTIONS(3004),
    [anon_sym_async] = ACTIONS(3004),
    [anon_sym_auto] = ACTIONS(3004),
    [anon_sym_await] = ACTIONS(3004),
    [anon_sym_become] = ACTIONS(3004),
    [anon_sym_break] = ACTIONS(3004),
    [anon_sym_const] = ACTIONS(3004),
    [anon_sym_continue] = ACTIONS(3004),
    [anon_sym_default] = ACTIONS(3004),
    [anon_sym_enum] = ACTIONS(3004),
    [anon_sym_fn] = ACTIONS(3004),
    [anon_sym_for] = ACTIONS(3004),
    [anon_sym_gen] = ACTIONS(3004),
    [anon_sym_if] = ACTIONS(3004),
    [anon_sym_impl] = ACTIONS(3004),
    [anon_sym_let] = ACTIONS(3004),
    [anon_sym_loop] = ACTIONS(3004),
    [anon_sym_match] = ACTIONS(3004),
    [anon_sym_mod] = ACTIONS(3004),
    [anon_sym_pub] = ACTIONS(3004),
    [anon_sym_return] = ACTIONS(3004),
    [anon_sym_raw] = ACTIONS(3004),
    [anon_sym_static] = ACTIONS(3004),
    [anon_sym_struct] = ACTIONS(3004),
    [anon_sym_trait] = ACTIONS(3004),
    [anon_sym_type] = ACTIONS(3004),
    [anon_sym_union] = ACTIONS(3004),
    [anon_sym_unsafe] = ACTIONS(3004),
    [anon_sym_use] = ACTIONS(3004),
    [anon_sym_while] = ACTIONS(3004),
    [anon_sym_extern] = ACTIONS(3004),
    [anon_sym_safe] = ACTIONS(3004),
    [anon_sym_yield] = ACTIONS(3004),
    [anon_sym_move] = ACTIONS(3004),
    [anon_sym_try] = ACTIONS(3004),
    [sym_integer_literal] = ACTIONS(3002),
    [aux_sym_string_literal_token1] = ACTIONS(3002),
    [sym_char_literal] = ACTIONS(3002),
    [anon_sym_true] = ACTIONS(3004),
    [anon_sym_false] = ACTIONS(3004),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3004),
    [sym_super] = ACTIONS(3004),
    [sym_crate] = ACTIONS(3004),
    [aux_sym_metavariable_token1] = ACTIONS(3002),
    [sym__raw_string_literal_start] = ACTIONS(3002),
    [sym_float_literal] = ACTIONS(3002),
    [sym__outer_block_doc_comment_start] = ACTIONS(3002),
    [sym__outer_line_doc_comment_start] = ACTIONS(3002),
  },
  [STATE(809)] = {
    [sym_line_comment] = STATE(809),
    [sym_block_comment] = STATE(809),
    [ts_builtin_sym_end] = ACTIONS(3006),
    [sym_identifier] = ACTIONS(3008),
    [anon_sym_SEMI] = ACTIONS(3006),
    [anon_sym_macro] = ACTIONS(3008),
    [anon_sym_LPAREN] = ACTIONS(3006),
    [anon_sym_LBRACE] = ACTIONS(3006),
    [anon_sym_RBRACE] = ACTIONS(3006),
    [anon_sym_macro_rules] = ACTIONS(3008),
    [anon_sym_BANG] = ACTIONS(3006),
    [anon_sym_LBRACK] = ACTIONS(3006),
    [anon_sym_STAR] = ACTIONS(3006),
    [anon_sym_u8] = ACTIONS(3008),
    [anon_sym_i8] = ACTIONS(3008),
    [anon_sym_u16] = ACTIONS(3008),
    [anon_sym_i16] = ACTIONS(3008),
    [anon_sym_u32] = ACTIONS(3008),
    [anon_sym_i32] = ACTIONS(3008),
    [anon_sym_u64] = ACTIONS(3008),
    [anon_sym_i64] = ACTIONS(3008),
    [anon_sym_u128] = ACTIONS(3008),
    [anon_sym_i128] = ACTIONS(3008),
    [anon_sym_isize] = ACTIONS(3008),
    [anon_sym_usize] = ACTIONS(3008),
    [anon_sym_f32] = ACTIONS(3008),
    [anon_sym_f64] = ACTIONS(3008),
    [anon_sym_bool] = ACTIONS(3008),
    [anon_sym_str] = ACTIONS(3008),
    [anon_sym_char] = ACTIONS(3008),
    [anon_sym_DASH] = ACTIONS(3006),
    [anon_sym_AMP] = ACTIONS(3006),
    [anon_sym_PIPE] = ACTIONS(3006),
    [anon_sym_LT] = ACTIONS(3006),
    [anon_sym_DOT_DOT] = ACTIONS(3006),
    [anon_sym_COLON_COLON] = ACTIONS(3006),
    [anon_sym_POUND] = ACTIONS(3006),
    [anon_sym_SQUOTE] = ACTIONS(3008),
    [anon_sym_async] = ACTIONS(3008),
    [anon_sym_auto] = ACTIONS(3008),
    [anon_sym_await] = ACTIONS(3008),
    [anon_sym_become] = ACTIONS(3008),
    [anon_sym_break] = ACTIONS(3008),
    [anon_sym_const] = ACTIONS(3008),
    [anon_sym_continue] = ACTIONS(3008),
    [anon_sym_default] = ACTIONS(3008),
    [anon_sym_enum] = ACTIONS(3008),
    [anon_sym_fn] = ACTIONS(3008),
    [anon_sym_for] = ACTIONS(3008),
    [anon_sym_gen] = ACTIONS(3008),
    [anon_sym_if] = ACTIONS(3008),
    [anon_sym_impl] = ACTIONS(3008),
    [anon_sym_let] = ACTIONS(3008),
    [anon_sym_loop] = ACTIONS(3008),
    [anon_sym_match] = ACTIONS(3008),
    [anon_sym_mod] = ACTIONS(3008),
    [anon_sym_pub] = ACTIONS(3008),
    [anon_sym_return] = ACTIONS(3008),
    [anon_sym_raw] = ACTIONS(3008),
    [anon_sym_static] = ACTIONS(3008),
    [anon_sym_struct] = ACTIONS(3008),
    [anon_sym_trait] = ACTIONS(3008),
    [anon_sym_type] = ACTIONS(3008),
    [anon_sym_union] = ACTIONS(3008),
    [anon_sym_unsafe] = ACTIONS(3008),
    [anon_sym_use] = ACTIONS(3008),
    [anon_sym_while] = ACTIONS(3008),
    [anon_sym_extern] = ACTIONS(3008),
    [anon_sym_safe] = ACTIONS(3008),
    [anon_sym_yield] = ACTIONS(3008),
    [anon_sym_move] = ACTIONS(3008),
    [anon_sym_try] = ACTIONS(3008),
    [sym_integer_literal] = ACTIONS(3006),
    [aux_sym_string_literal_token1] = ACTIONS(3006),
    [sym_char_literal] = ACTIONS(3006),
    [anon_sym_true] = ACTIONS(3008),
    [anon_sym_false] = ACTIONS(3008),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3008),
    [sym_super] = ACTIONS(3008),
    [sym_crate] = ACTIONS(3008),
    [aux_sym_metavariable_token1] = ACTIONS(3006),
    [sym__raw_string_literal_start] = ACTIONS(3006),
    [sym_float_literal] = ACTIONS(3006),
    [sym__outer_block_doc_comment_start] = ACTIONS(3006),
    [sym__outer_line_doc_comment_start] = ACTIONS(3006),
  },
  [STATE(810)] = {
    [sym_line_comment] = STATE(810),
    [sym_block_comment] = STATE(810),
    [ts_builtin_sym_end] = ACTIONS(3010),
    [sym_identifier] = ACTIONS(3012),
    [anon_sym_SEMI] = ACTIONS(3010),
    [anon_sym_macro] = ACTIONS(3012),
    [anon_sym_LPAREN] = ACTIONS(3010),
    [anon_sym_LBRACE] = ACTIONS(3010),
    [anon_sym_RBRACE] = ACTIONS(3010),
    [anon_sym_macro_rules] = ACTIONS(3012),
    [anon_sym_BANG] = ACTIONS(3010),
    [anon_sym_LBRACK] = ACTIONS(3010),
    [anon_sym_STAR] = ACTIONS(3010),
    [anon_sym_u8] = ACTIONS(3012),
    [anon_sym_i8] = ACTIONS(3012),
    [anon_sym_u16] = ACTIONS(3012),
    [anon_sym_i16] = ACTIONS(3012),
    [anon_sym_u32] = ACTIONS(3012),
    [anon_sym_i32] = ACTIONS(3012),
    [anon_sym_u64] = ACTIONS(3012),
    [anon_sym_i64] = ACTIONS(3012),
    [anon_sym_u128] = ACTIONS(3012),
    [anon_sym_i128] = ACTIONS(3012),
    [anon_sym_isize] = ACTIONS(3012),
    [anon_sym_usize] = ACTIONS(3012),
    [anon_sym_f32] = ACTIONS(3012),
    [anon_sym_f64] = ACTIONS(3012),
    [anon_sym_bool] = ACTIONS(3012),
    [anon_sym_str] = ACTIONS(3012),
    [anon_sym_char] = ACTIONS(3012),
    [anon_sym_DASH] = ACTIONS(3010),
    [anon_sym_AMP] = ACTIONS(3010),
    [anon_sym_PIPE] = ACTIONS(3010),
    [anon_sym_LT] = ACTIONS(3010),
    [anon_sym_DOT_DOT] = ACTIONS(3010),
    [anon_sym_COLON_COLON] = ACTIONS(3010),
    [anon_sym_POUND] = ACTIONS(3010),
    [anon_sym_SQUOTE] = ACTIONS(3012),
    [anon_sym_async] = ACTIONS(3012),
    [anon_sym_auto] = ACTIONS(3012),
    [anon_sym_await] = ACTIONS(3012),
    [anon_sym_become] = ACTIONS(3012),
    [anon_sym_break] = ACTIONS(3012),
    [anon_sym_const] = ACTIONS(3012),
    [anon_sym_continue] = ACTIONS(3012),
    [anon_sym_default] = ACTIONS(3012),
    [anon_sym_enum] = ACTIONS(3012),
    [anon_sym_fn] = ACTIONS(3012),
    [anon_sym_for] = ACTIONS(3012),
    [anon_sym_gen] = ACTIONS(3012),
    [anon_sym_if] = ACTIONS(3012),
    [anon_sym_impl] = ACTIONS(3012),
    [anon_sym_let] = ACTIONS(3012),
    [anon_sym_loop] = ACTIONS(3012),
    [anon_sym_match] = ACTIONS(3012),
    [anon_sym_mod] = ACTIONS(3012),
    [anon_sym_pub] = ACTIONS(3012),
    [anon_sym_return] = ACTIONS(3012),
    [anon_sym_raw] = ACTIONS(3012),
    [anon_sym_static] = ACTIONS(3012),
    [anon_sym_struct] = ACTIONS(3012),
    [anon_sym_trait] = ACTIONS(3012),
    [anon_sym_type] = ACTIONS(3012),
    [anon_sym_union] = ACTIONS(3012),
    [anon_sym_unsafe] = ACTIONS(3012),
    [anon_sym_use] = ACTIONS(3012),
    [anon_sym_while] = ACTIONS(3012),
    [anon_sym_extern] = ACTIONS(3012),
    [anon_sym_safe] = ACTIONS(3012),
    [anon_sym_yield] = ACTIONS(3012),
    [anon_sym_move] = ACTIONS(3012),
    [anon_sym_try] = ACTIONS(3012),
    [sym_integer_literal] = ACTIONS(3010),
    [aux_sym_string_literal_token1] = ACTIONS(3010),
    [sym_char_literal] = ACTIONS(3010),
    [anon_sym_true] = ACTIONS(3012),
    [anon_sym_false] = ACTIONS(3012),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3012),
    [sym_super] = ACTIONS(3012),
    [sym_crate] = ACTIONS(3012),
    [aux_sym_metavariable_token1] = ACTIONS(3010),
    [sym__raw_string_literal_start] = ACTIONS(3010),
    [sym_float_literal] = ACTIONS(3010),
    [sym__outer_block_doc_comment_start] = ACTIONS(3010),
    [sym__outer_line_doc_comment_start] = ACTIONS(3010),
  },
  [STATE(811)] = {
    [sym_line_comment] = STATE(811),
    [sym_block_comment] = STATE(811),
    [ts_builtin_sym_end] = ACTIONS(3014),
    [sym_identifier] = ACTIONS(3016),
    [anon_sym_SEMI] = ACTIONS(3014),
    [anon_sym_macro] = ACTIONS(3016),
    [anon_sym_LPAREN] = ACTIONS(3014),
    [anon_sym_LBRACE] = ACTIONS(3014),
    [anon_sym_RBRACE] = ACTIONS(3014),
    [anon_sym_macro_rules] = ACTIONS(3016),
    [anon_sym_BANG] = ACTIONS(3014),
    [anon_sym_LBRACK] = ACTIONS(3014),
    [anon_sym_STAR] = ACTIONS(3014),
    [anon_sym_u8] = ACTIONS(3016),
    [anon_sym_i8] = ACTIONS(3016),
    [anon_sym_u16] = ACTIONS(3016),
    [anon_sym_i16] = ACTIONS(3016),
    [anon_sym_u32] = ACTIONS(3016),
    [anon_sym_i32] = ACTIONS(3016),
    [anon_sym_u64] = ACTIONS(3016),
    [anon_sym_i64] = ACTIONS(3016),
    [anon_sym_u128] = ACTIONS(3016),
    [anon_sym_i128] = ACTIONS(3016),
    [anon_sym_isize] = ACTIONS(3016),
    [anon_sym_usize] = ACTIONS(3016),
    [anon_sym_f32] = ACTIONS(3016),
    [anon_sym_f64] = ACTIONS(3016),
    [anon_sym_bool] = ACTIONS(3016),
    [anon_sym_str] = ACTIONS(3016),
    [anon_sym_char] = ACTIONS(3016),
    [anon_sym_DASH] = ACTIONS(3014),
    [anon_sym_AMP] = ACTIONS(3014),
    [anon_sym_PIPE] = ACTIONS(3014),
    [anon_sym_LT] = ACTIONS(3014),
    [anon_sym_DOT_DOT] = ACTIONS(3014),
    [anon_sym_COLON_COLON] = ACTIONS(3014),
    [anon_sym_POUND] = ACTIONS(3014),
    [anon_sym_SQUOTE] = ACTIONS(3016),
    [anon_sym_async] = ACTIONS(3016),
    [anon_sym_auto] = ACTIONS(3016),
    [anon_sym_await] = ACTIONS(3016),
    [anon_sym_become] = ACTIONS(3016),
    [anon_sym_break] = ACTIONS(3016),
    [anon_sym_const] = ACTIONS(3016),
    [anon_sym_continue] = ACTIONS(3016),
    [anon_sym_default] = ACTIONS(3016),
    [anon_sym_enum] = ACTIONS(3016),
    [anon_sym_fn] = ACTIONS(3016),
    [anon_sym_for] = ACTIONS(3016),
    [anon_sym_gen] = ACTIONS(3016),
    [anon_sym_if] = ACTIONS(3016),
    [anon_sym_impl] = ACTIONS(3016),
    [anon_sym_let] = ACTIONS(3016),
    [anon_sym_loop] = ACTIONS(3016),
    [anon_sym_match] = ACTIONS(3016),
    [anon_sym_mod] = ACTIONS(3016),
    [anon_sym_pub] = ACTIONS(3016),
    [anon_sym_return] = ACTIONS(3016),
    [anon_sym_raw] = ACTIONS(3016),
    [anon_sym_static] = ACTIONS(3016),
    [anon_sym_struct] = ACTIONS(3016),
    [anon_sym_trait] = ACTIONS(3016),
    [anon_sym_type] = ACTIONS(3016),
    [anon_sym_union] = ACTIONS(3016),
    [anon_sym_unsafe] = ACTIONS(3016),
    [anon_sym_use] = ACTIONS(3016),
    [anon_sym_while] = ACTIONS(3016),
    [anon_sym_extern] = ACTIONS(3016),
    [anon_sym_safe] = ACTIONS(3016),
    [anon_sym_yield] = ACTIONS(3016),
    [anon_sym_move] = ACTIONS(3016),
    [anon_sym_try] = ACTIONS(3016),
    [sym_integer_literal] = ACTIONS(3014),
    [aux_sym_string_literal_token1] = ACTIONS(3014),
    [sym_char_literal] = ACTIONS(3014),
    [anon_sym_true] = ACTIONS(3016),
    [anon_sym_false] = ACTIONS(3016),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3016),
    [sym_super] = ACTIONS(3016),
    [sym_crate] = ACTIONS(3016),
    [aux_sym_metavariable_token1] = ACTIONS(3014),
    [sym__raw_string_literal_start] = ACTIONS(3014),
    [sym_float_literal] = ACTIONS(3014),
    [sym__outer_block_doc_comment_start] = ACTIONS(3014),
    [sym__outer_line_doc_comment_start] = ACTIONS(3014),
  },
  [STATE(812)] = {
    [sym_line_comment] = STATE(812),
    [sym_block_comment] = STATE(812),
    [ts_builtin_sym_end] = ACTIONS(3018),
    [sym_identifier] = ACTIONS(3020),
    [anon_sym_SEMI] = ACTIONS(3018),
    [anon_sym_macro] = ACTIONS(3020),
    [anon_sym_LPAREN] = ACTIONS(3018),
    [anon_sym_LBRACE] = ACTIONS(3018),
    [anon_sym_RBRACE] = ACTIONS(3018),
    [anon_sym_macro_rules] = ACTIONS(3020),
    [anon_sym_BANG] = ACTIONS(3018),
    [anon_sym_LBRACK] = ACTIONS(3018),
    [anon_sym_STAR] = ACTIONS(3018),
    [anon_sym_u8] = ACTIONS(3020),
    [anon_sym_i8] = ACTIONS(3020),
    [anon_sym_u16] = ACTIONS(3020),
    [anon_sym_i16] = ACTIONS(3020),
    [anon_sym_u32] = ACTIONS(3020),
    [anon_sym_i32] = ACTIONS(3020),
    [anon_sym_u64] = ACTIONS(3020),
    [anon_sym_i64] = ACTIONS(3020),
    [anon_sym_u128] = ACTIONS(3020),
    [anon_sym_i128] = ACTIONS(3020),
    [anon_sym_isize] = ACTIONS(3020),
    [anon_sym_usize] = ACTIONS(3020),
    [anon_sym_f32] = ACTIONS(3020),
    [anon_sym_f64] = ACTIONS(3020),
    [anon_sym_bool] = ACTIONS(3020),
    [anon_sym_str] = ACTIONS(3020),
    [anon_sym_char] = ACTIONS(3020),
    [anon_sym_DASH] = ACTIONS(3018),
    [anon_sym_AMP] = ACTIONS(3018),
    [anon_sym_PIPE] = ACTIONS(3018),
    [anon_sym_LT] = ACTIONS(3018),
    [anon_sym_DOT_DOT] = ACTIONS(3018),
    [anon_sym_COLON_COLON] = ACTIONS(3018),
    [anon_sym_POUND] = ACTIONS(3018),
    [anon_sym_SQUOTE] = ACTIONS(3020),
    [anon_sym_async] = ACTIONS(3020),
    [anon_sym_auto] = ACTIONS(3020),
    [anon_sym_await] = ACTIONS(3020),
    [anon_sym_become] = ACTIONS(3020),
    [anon_sym_break] = ACTIONS(3020),
    [anon_sym_const] = ACTIONS(3020),
    [anon_sym_continue] = ACTIONS(3020),
    [anon_sym_default] = ACTIONS(3020),
    [anon_sym_enum] = ACTIONS(3020),
    [anon_sym_fn] = ACTIONS(3020),
    [anon_sym_for] = ACTIONS(3020),
    [anon_sym_gen] = ACTIONS(3020),
    [anon_sym_if] = ACTIONS(3020),
    [anon_sym_impl] = ACTIONS(3020),
    [anon_sym_let] = ACTIONS(3020),
    [anon_sym_loop] = ACTIONS(3020),
    [anon_sym_match] = ACTIONS(3020),
    [anon_sym_mod] = ACTIONS(3020),
    [anon_sym_pub] = ACTIONS(3020),
    [anon_sym_return] = ACTIONS(3020),
    [anon_sym_raw] = ACTIONS(3020),
    [anon_sym_static] = ACTIONS(3020),
    [anon_sym_struct] = ACTIONS(3020),
    [anon_sym_trait] = ACTIONS(3020),
    [anon_sym_type] = ACTIONS(3020),
    [anon_sym_union] = ACTIONS(3020),
    [anon_sym_unsafe] = ACTIONS(3020),
    [anon_sym_use] = ACTIONS(3020),
    [anon_sym_while] = ACTIONS(3020),
    [anon_sym_extern] = ACTIONS(3020),
    [anon_sym_safe] = ACTIONS(3020),
    [anon_sym_yield] = ACTIONS(3020),
    [anon_sym_move] = ACTIONS(3020),
    [anon_sym_try] = ACTIONS(3020),
    [sym_integer_literal] = ACTIONS(3018),
    [aux_sym_string_literal_token1] = ACTIONS(3018),
    [sym_char_literal] = ACTIONS(3018),
    [anon_sym_true] = ACTIONS(3020),
    [anon_sym_false] = ACTIONS(3020),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3020),
    [sym_super] = ACTIONS(3020),
    [sym_crate] = ACTIONS(3020),
    [aux_sym_metavariable_token1] = ACTIONS(3018),
    [sym__raw_string_literal_start] = ACTIONS(3018),
    [sym_float_literal] = ACTIONS(3018),
    [sym__outer_block_doc_comment_start] = ACTIONS(3018),
    [sym__outer_line_doc_comment_start] = ACTIONS(3018),
  },
  [STATE(813)] = {
    [sym_line_comment] = STATE(813),
    [sym_block_comment] = STATE(813),
    [ts_builtin_sym_end] = ACTIONS(3022),
    [sym_identifier] = ACTIONS(3024),
    [anon_sym_SEMI] = ACTIONS(3022),
    [anon_sym_macro] = ACTIONS(3024),
    [anon_sym_LPAREN] = ACTIONS(3022),
    [anon_sym_LBRACE] = ACTIONS(3022),
    [anon_sym_RBRACE] = ACTIONS(3022),
    [anon_sym_macro_rules] = ACTIONS(3024),
    [anon_sym_BANG] = ACTIONS(3022),
    [anon_sym_LBRACK] = ACTIONS(3022),
    [anon_sym_STAR] = ACTIONS(3022),
    [anon_sym_u8] = ACTIONS(3024),
    [anon_sym_i8] = ACTIONS(3024),
    [anon_sym_u16] = ACTIONS(3024),
    [anon_sym_i16] = ACTIONS(3024),
    [anon_sym_u32] = ACTIONS(3024),
    [anon_sym_i32] = ACTIONS(3024),
    [anon_sym_u64] = ACTIONS(3024),
    [anon_sym_i64] = ACTIONS(3024),
    [anon_sym_u128] = ACTIONS(3024),
    [anon_sym_i128] = ACTIONS(3024),
    [anon_sym_isize] = ACTIONS(3024),
    [anon_sym_usize] = ACTIONS(3024),
    [anon_sym_f32] = ACTIONS(3024),
    [anon_sym_f64] = ACTIONS(3024),
    [anon_sym_bool] = ACTIONS(3024),
    [anon_sym_str] = ACTIONS(3024),
    [anon_sym_char] = ACTIONS(3024),
    [anon_sym_DASH] = ACTIONS(3022),
    [anon_sym_AMP] = ACTIONS(3022),
    [anon_sym_PIPE] = ACTIONS(3022),
    [anon_sym_LT] = ACTIONS(3022),
    [anon_sym_DOT_DOT] = ACTIONS(3022),
    [anon_sym_COLON_COLON] = ACTIONS(3022),
    [anon_sym_POUND] = ACTIONS(3022),
    [anon_sym_SQUOTE] = ACTIONS(3024),
    [anon_sym_async] = ACTIONS(3024),
    [anon_sym_auto] = ACTIONS(3024),
    [anon_sym_await] = ACTIONS(3024),
    [anon_sym_become] = ACTIONS(3024),
    [anon_sym_break] = ACTIONS(3024),
    [anon_sym_const] = ACTIONS(3024),
    [anon_sym_continue] = ACTIONS(3024),
    [anon_sym_default] = ACTIONS(3024),
    [anon_sym_enum] = ACTIONS(3024),
    [anon_sym_fn] = ACTIONS(3024),
    [anon_sym_for] = ACTIONS(3024),
    [anon_sym_gen] = ACTIONS(3024),
    [anon_sym_if] = ACTIONS(3024),
    [anon_sym_impl] = ACTIONS(3024),
    [anon_sym_let] = ACTIONS(3024),
    [anon_sym_loop] = ACTIONS(3024),
    [anon_sym_match] = ACTIONS(3024),
    [anon_sym_mod] = ACTIONS(3024),
    [anon_sym_pub] = ACTIONS(3024),
    [anon_sym_return] = ACTIONS(3024),
    [anon_sym_raw] = ACTIONS(3024),
    [anon_sym_static] = ACTIONS(3024),
    [anon_sym_struct] = ACTIONS(3024),
    [anon_sym_trait] = ACTIONS(3024),
    [anon_sym_type] = ACTIONS(3024),
    [anon_sym_union] = ACTIONS(3024),
    [anon_sym_unsafe] = ACTIONS(3024),
    [anon_sym_use] = ACTIONS(3024),
    [anon_sym_while] = ACTIONS(3024),
    [anon_sym_extern] = ACTIONS(3024),
    [anon_sym_safe] = ACTIONS(3024),
    [anon_sym_yield] = ACTIONS(3024),
    [anon_sym_move] = ACTIONS(3024),
    [anon_sym_try] = ACTIONS(3024),
    [sym_integer_literal] = ACTIONS(3022),
    [aux_sym_string_literal_token1] = ACTIONS(3022),
    [sym_char_literal] = ACTIONS(3022),
    [anon_sym_true] = ACTIONS(3024),
    [anon_sym_false] = ACTIONS(3024),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3024),
    [sym_super] = ACTIONS(3024),
    [sym_crate] = ACTIONS(3024),
    [aux_sym_metavariable_token1] = ACTIONS(3022),
    [sym__raw_string_literal_start] = ACTIONS(3022),
    [sym_float_literal] = ACTIONS(3022),
    [sym__outer_block_doc_comment_start] = ACTIONS(3022),
    [sym__outer_line_doc_comment_start] = ACTIONS(3022),
  },
  [STATE(814)] = {
    [sym_line_comment] = STATE(814),
    [sym_block_comment] = STATE(814),
    [ts_builtin_sym_end] = ACTIONS(3026),
    [sym_identifier] = ACTIONS(3028),
    [anon_sym_SEMI] = ACTIONS(3026),
    [anon_sym_macro] = ACTIONS(3028),
    [anon_sym_LPAREN] = ACTIONS(3026),
    [anon_sym_LBRACE] = ACTIONS(3026),
    [anon_sym_RBRACE] = ACTIONS(3026),
    [anon_sym_macro_rules] = ACTIONS(3028),
    [anon_sym_BANG] = ACTIONS(3026),
    [anon_sym_LBRACK] = ACTIONS(3026),
    [anon_sym_STAR] = ACTIONS(3026),
    [anon_sym_u8] = ACTIONS(3028),
    [anon_sym_i8] = ACTIONS(3028),
    [anon_sym_u16] = ACTIONS(3028),
    [anon_sym_i16] = ACTIONS(3028),
    [anon_sym_u32] = ACTIONS(3028),
    [anon_sym_i32] = ACTIONS(3028),
    [anon_sym_u64] = ACTIONS(3028),
    [anon_sym_i64] = ACTIONS(3028),
    [anon_sym_u128] = ACTIONS(3028),
    [anon_sym_i128] = ACTIONS(3028),
    [anon_sym_isize] = ACTIONS(3028),
    [anon_sym_usize] = ACTIONS(3028),
    [anon_sym_f32] = ACTIONS(3028),
    [anon_sym_f64] = ACTIONS(3028),
    [anon_sym_bool] = ACTIONS(3028),
    [anon_sym_str] = ACTIONS(3028),
    [anon_sym_char] = ACTIONS(3028),
    [anon_sym_DASH] = ACTIONS(3026),
    [anon_sym_AMP] = ACTIONS(3026),
    [anon_sym_PIPE] = ACTIONS(3026),
    [anon_sym_LT] = ACTIONS(3026),
    [anon_sym_DOT_DOT] = ACTIONS(3026),
    [anon_sym_COLON_COLON] = ACTIONS(3026),
    [anon_sym_POUND] = ACTIONS(3026),
    [anon_sym_SQUOTE] = ACTIONS(3028),
    [anon_sym_async] = ACTIONS(3028),
    [anon_sym_auto] = ACTIONS(3028),
    [anon_sym_await] = ACTIONS(3028),
    [anon_sym_become] = ACTIONS(3028),
    [anon_sym_break] = ACTIONS(3028),
    [anon_sym_const] = ACTIONS(3028),
    [anon_sym_continue] = ACTIONS(3028),
    [anon_sym_default] = ACTIONS(3028),
    [anon_sym_enum] = ACTIONS(3028),
    [anon_sym_fn] = ACTIONS(3028),
    [anon_sym_for] = ACTIONS(3028),
    [anon_sym_gen] = ACTIONS(3028),
    [anon_sym_if] = ACTIONS(3028),
    [anon_sym_impl] = ACTIONS(3028),
    [anon_sym_let] = ACTIONS(3028),
    [anon_sym_loop] = ACTIONS(3028),
    [anon_sym_match] = ACTIONS(3028),
    [anon_sym_mod] = ACTIONS(3028),
    [anon_sym_pub] = ACTIONS(3028),
    [anon_sym_return] = ACTIONS(3028),
    [anon_sym_raw] = ACTIONS(3028),
    [anon_sym_static] = ACTIONS(3028),
    [anon_sym_struct] = ACTIONS(3028),
    [anon_sym_trait] = ACTIONS(3028),
    [anon_sym_type] = ACTIONS(3028),
    [anon_sym_union] = ACTIONS(3028),
    [anon_sym_unsafe] = ACTIONS(3028),
    [anon_sym_use] = ACTIONS(3028),
    [anon_sym_while] = ACTIONS(3028),
    [anon_sym_extern] = ACTIONS(3028),
    [anon_sym_safe] = ACTIONS(3028),
    [anon_sym_yield] = ACTIONS(3028),
    [anon_sym_move] = ACTIONS(3028),
    [anon_sym_try] = ACTIONS(3028),
    [sym_integer_literal] = ACTIONS(3026),
    [aux_sym_string_literal_token1] = ACTIONS(3026),
    [sym_char_literal] = ACTIONS(3026),
    [anon_sym_true] = ACTIONS(3028),
    [anon_sym_false] = ACTIONS(3028),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3028),
    [sym_super] = ACTIONS(3028),
    [sym_crate] = ACTIONS(3028),
    [aux_sym_metavariable_token1] = ACTIONS(3026),
    [sym__raw_string_literal_start] = ACTIONS(3026),
    [sym_float_literal] = ACTIONS(3026),
    [sym__outer_block_doc_comment_start] = ACTIONS(3026),
    [sym__outer_line_doc_comment_start] = ACTIONS(3026),
  },
  [STATE(815)] = {
    [sym_line_comment] = STATE(815),
    [sym_block_comment] = STATE(815),
    [ts_builtin_sym_end] = ACTIONS(3030),
    [sym_identifier] = ACTIONS(3032),
    [anon_sym_SEMI] = ACTIONS(3030),
    [anon_sym_macro] = ACTIONS(3032),
    [anon_sym_LPAREN] = ACTIONS(3030),
    [anon_sym_LBRACE] = ACTIONS(3030),
    [anon_sym_RBRACE] = ACTIONS(3030),
    [anon_sym_macro_rules] = ACTIONS(3032),
    [anon_sym_BANG] = ACTIONS(3030),
    [anon_sym_LBRACK] = ACTIONS(3030),
    [anon_sym_STAR] = ACTIONS(3030),
    [anon_sym_u8] = ACTIONS(3032),
    [anon_sym_i8] = ACTIONS(3032),
    [anon_sym_u16] = ACTIONS(3032),
    [anon_sym_i16] = ACTIONS(3032),
    [anon_sym_u32] = ACTIONS(3032),
    [anon_sym_i32] = ACTIONS(3032),
    [anon_sym_u64] = ACTIONS(3032),
    [anon_sym_i64] = ACTIONS(3032),
    [anon_sym_u128] = ACTIONS(3032),
    [anon_sym_i128] = ACTIONS(3032),
    [anon_sym_isize] = ACTIONS(3032),
    [anon_sym_usize] = ACTIONS(3032),
    [anon_sym_f32] = ACTIONS(3032),
    [anon_sym_f64] = ACTIONS(3032),
    [anon_sym_bool] = ACTIONS(3032),
    [anon_sym_str] = ACTIONS(3032),
    [anon_sym_char] = ACTIONS(3032),
    [anon_sym_DASH] = ACTIONS(3030),
    [anon_sym_AMP] = ACTIONS(3030),
    [anon_sym_PIPE] = ACTIONS(3030),
    [anon_sym_LT] = ACTIONS(3030),
    [anon_sym_DOT_DOT] = ACTIONS(3030),
    [anon_sym_COLON_COLON] = ACTIONS(3030),
    [anon_sym_POUND] = ACTIONS(3030),
    [anon_sym_SQUOTE] = ACTIONS(3032),
    [anon_sym_async] = ACTIONS(3032),
    [anon_sym_auto] = ACTIONS(3032),
    [anon_sym_await] = ACTIONS(3032),
    [anon_sym_become] = ACTIONS(3032),
    [anon_sym_break] = ACTIONS(3032),
    [anon_sym_const] = ACTIONS(3032),
    [anon_sym_continue] = ACTIONS(3032),
    [anon_sym_default] = ACTIONS(3032),
    [anon_sym_enum] = ACTIONS(3032),
    [anon_sym_fn] = ACTIONS(3032),
    [anon_sym_for] = ACTIONS(3032),
    [anon_sym_gen] = ACTIONS(3032),
    [anon_sym_if] = ACTIONS(3032),
    [anon_sym_impl] = ACTIONS(3032),
    [anon_sym_let] = ACTIONS(3032),
    [anon_sym_loop] = ACTIONS(3032),
    [anon_sym_match] = ACTIONS(3032),
    [anon_sym_mod] = ACTIONS(3032),
    [anon_sym_pub] = ACTIONS(3032),
    [anon_sym_return] = ACTIONS(3032),
    [anon_sym_raw] = ACTIONS(3032),
    [anon_sym_static] = ACTIONS(3032),
    [anon_sym_struct] = ACTIONS(3032),
    [anon_sym_trait] = ACTIONS(3032),
    [anon_sym_type] = ACTIONS(3032),
    [anon_sym_union] = ACTIONS(3032),
    [anon_sym_unsafe] = ACTIONS(3032),
    [anon_sym_use] = ACTIONS(3032),
    [anon_sym_while] = ACTIONS(3032),
    [anon_sym_extern] = ACTIONS(3032),
    [anon_sym_safe] = ACTIONS(3032),
    [anon_sym_yield] = ACTIONS(3032),
    [anon_sym_move] = ACTIONS(3032),
    [anon_sym_try] = ACTIONS(3032),
    [sym_integer_literal] = ACTIONS(3030),
    [aux_sym_string_literal_token1] = ACTIONS(3030),
    [sym_char_literal] = ACTIONS(3030),
    [anon_sym_true] = ACTIONS(3032),
    [anon_sym_false] = ACTIONS(3032),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3032),
    [sym_super] = ACTIONS(3032),
    [sym_crate] = ACTIONS(3032),
    [aux_sym_metavariable_token1] = ACTIONS(3030),
    [sym__raw_string_literal_start] = ACTIONS(3030),
    [sym_float_literal] = ACTIONS(3030),
    [sym__outer_block_doc_comment_start] = ACTIONS(3030),
    [sym__outer_line_doc_comment_start] = ACTIONS(3030),
  },
  [STATE(816)] = {
    [sym_line_comment] = STATE(816),
    [sym_block_comment] = STATE(816),
    [ts_builtin_sym_end] = ACTIONS(3034),
    [sym_identifier] = ACTIONS(3036),
    [anon_sym_SEMI] = ACTIONS(3034),
    [anon_sym_macro] = ACTIONS(3036),
    [anon_sym_LPAREN] = ACTIONS(3034),
    [anon_sym_LBRACE] = ACTIONS(3034),
    [anon_sym_RBRACE] = ACTIONS(3034),
    [anon_sym_macro_rules] = ACTIONS(3036),
    [anon_sym_BANG] = ACTIONS(3034),
    [anon_sym_LBRACK] = ACTIONS(3034),
    [anon_sym_STAR] = ACTIONS(3034),
    [anon_sym_u8] = ACTIONS(3036),
    [anon_sym_i8] = ACTIONS(3036),
    [anon_sym_u16] = ACTIONS(3036),
    [anon_sym_i16] = ACTIONS(3036),
    [anon_sym_u32] = ACTIONS(3036),
    [anon_sym_i32] = ACTIONS(3036),
    [anon_sym_u64] = ACTIONS(3036),
    [anon_sym_i64] = ACTIONS(3036),
    [anon_sym_u128] = ACTIONS(3036),
    [anon_sym_i128] = ACTIONS(3036),
    [anon_sym_isize] = ACTIONS(3036),
    [anon_sym_usize] = ACTIONS(3036),
    [anon_sym_f32] = ACTIONS(3036),
    [anon_sym_f64] = ACTIONS(3036),
    [anon_sym_bool] = ACTIONS(3036),
    [anon_sym_str] = ACTIONS(3036),
    [anon_sym_char] = ACTIONS(3036),
    [anon_sym_DASH] = ACTIONS(3034),
    [anon_sym_AMP] = ACTIONS(3034),
    [anon_sym_PIPE] = ACTIONS(3034),
    [anon_sym_LT] = ACTIONS(3034),
    [anon_sym_DOT_DOT] = ACTIONS(3034),
    [anon_sym_COLON_COLON] = ACTIONS(3034),
    [anon_sym_POUND] = ACTIONS(3034),
    [anon_sym_SQUOTE] = ACTIONS(3036),
    [anon_sym_async] = ACTIONS(3036),
    [anon_sym_auto] = ACTIONS(3036),
    [anon_sym_await] = ACTIONS(3036),
    [anon_sym_become] = ACTIONS(3036),
    [anon_sym_break] = ACTIONS(3036),
    [anon_sym_const] = ACTIONS(3036),
    [anon_sym_continue] = ACTIONS(3036),
    [anon_sym_default] = ACTIONS(3036),
    [anon_sym_enum] = ACTIONS(3036),
    [anon_sym_fn] = ACTIONS(3036),
    [anon_sym_for] = ACTIONS(3036),
    [anon_sym_gen] = ACTIONS(3036),
    [anon_sym_if] = ACTIONS(3036),
    [anon_sym_impl] = ACTIONS(3036),
    [anon_sym_let] = ACTIONS(3036),
    [anon_sym_loop] = ACTIONS(3036),
    [anon_sym_match] = ACTIONS(3036),
    [anon_sym_mod] = ACTIONS(3036),
    [anon_sym_pub] = ACTIONS(3036),
    [anon_sym_return] = ACTIONS(3036),
    [anon_sym_raw] = ACTIONS(3036),
    [anon_sym_static] = ACTIONS(3036),
    [anon_sym_struct] = ACTIONS(3036),
    [anon_sym_trait] = ACTIONS(3036),
    [anon_sym_type] = ACTIONS(3036),
    [anon_sym_union] = ACTIONS(3036),
    [anon_sym_unsafe] = ACTIONS(3036),
    [anon_sym_use] = ACTIONS(3036),
    [anon_sym_while] = ACTIONS(3036),
    [anon_sym_extern] = ACTIONS(3036),
    [anon_sym_safe] = ACTIONS(3036),
    [anon_sym_yield] = ACTIONS(3036),
    [anon_sym_move] = ACTIONS(3036),
    [anon_sym_try] = ACTIONS(3036),
    [sym_integer_literal] = ACTIONS(3034),
    [aux_sym_string_literal_token1] = ACTIONS(3034),
    [sym_char_literal] = ACTIONS(3034),
    [anon_sym_true] = ACTIONS(3036),
    [anon_sym_false] = ACTIONS(3036),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3036),
    [sym_super] = ACTIONS(3036),
    [sym_crate] = ACTIONS(3036),
    [aux_sym_metavariable_token1] = ACTIONS(3034),
    [sym__raw_string_literal_start] = ACTIONS(3034),
    [sym_float_literal] = ACTIONS(3034),
    [sym__outer_block_doc_comment_start] = ACTIONS(3034),
    [sym__outer_line_doc_comment_start] = ACTIONS(3034),
  },
  [STATE(817)] = {
    [sym_line_comment] = STATE(817),
    [sym_block_comment] = STATE(817),
    [ts_builtin_sym_end] = ACTIONS(3038),
    [sym_identifier] = ACTIONS(3040),
    [anon_sym_SEMI] = ACTIONS(3038),
    [anon_sym_macro] = ACTIONS(3040),
    [anon_sym_LPAREN] = ACTIONS(3038),
    [anon_sym_LBRACE] = ACTIONS(3038),
    [anon_sym_RBRACE] = ACTIONS(3038),
    [anon_sym_macro_rules] = ACTIONS(3040),
    [anon_sym_BANG] = ACTIONS(3038),
    [anon_sym_LBRACK] = ACTIONS(3038),
    [anon_sym_STAR] = ACTIONS(3038),
    [anon_sym_u8] = ACTIONS(3040),
    [anon_sym_i8] = ACTIONS(3040),
    [anon_sym_u16] = ACTIONS(3040),
    [anon_sym_i16] = ACTIONS(3040),
    [anon_sym_u32] = ACTIONS(3040),
    [anon_sym_i32] = ACTIONS(3040),
    [anon_sym_u64] = ACTIONS(3040),
    [anon_sym_i64] = ACTIONS(3040),
    [anon_sym_u128] = ACTIONS(3040),
    [anon_sym_i128] = ACTIONS(3040),
    [anon_sym_isize] = ACTIONS(3040),
    [anon_sym_usize] = ACTIONS(3040),
    [anon_sym_f32] = ACTIONS(3040),
    [anon_sym_f64] = ACTIONS(3040),
    [anon_sym_bool] = ACTIONS(3040),
    [anon_sym_str] = ACTIONS(3040),
    [anon_sym_char] = ACTIONS(3040),
    [anon_sym_DASH] = ACTIONS(3038),
    [anon_sym_AMP] = ACTIONS(3038),
    [anon_sym_PIPE] = ACTIONS(3038),
    [anon_sym_LT] = ACTIONS(3038),
    [anon_sym_DOT_DOT] = ACTIONS(3038),
    [anon_sym_COLON_COLON] = ACTIONS(3038),
    [anon_sym_POUND] = ACTIONS(3038),
    [anon_sym_SQUOTE] = ACTIONS(3040),
    [anon_sym_async] = ACTIONS(3040),
    [anon_sym_auto] = ACTIONS(3040),
    [anon_sym_await] = ACTIONS(3040),
    [anon_sym_become] = ACTIONS(3040),
    [anon_sym_break] = ACTIONS(3040),
    [anon_sym_const] = ACTIONS(3040),
    [anon_sym_continue] = ACTIONS(3040),
    [anon_sym_default] = ACTIONS(3040),
    [anon_sym_enum] = ACTIONS(3040),
    [anon_sym_fn] = ACTIONS(3040),
    [anon_sym_for] = ACTIONS(3040),
    [anon_sym_gen] = ACTIONS(3040),
    [anon_sym_if] = ACTIONS(3040),
    [anon_sym_impl] = ACTIONS(3040),
    [anon_sym_let] = ACTIONS(3040),
    [anon_sym_loop] = ACTIONS(3040),
    [anon_sym_match] = ACTIONS(3040),
    [anon_sym_mod] = ACTIONS(3040),
    [anon_sym_pub] = ACTIONS(3040),
    [anon_sym_return] = ACTIONS(3040),
    [anon_sym_raw] = ACTIONS(3040),
    [anon_sym_static] = ACTIONS(3040),
    [anon_sym_struct] = ACTIONS(3040),
    [anon_sym_trait] = ACTIONS(3040),
    [anon_sym_type] = ACTIONS(3040),
    [anon_sym_union] = ACTIONS(3040),
    [anon_sym_unsafe] = ACTIONS(3040),
    [anon_sym_use] = ACTIONS(3040),
    [anon_sym_while] = ACTIONS(3040),
    [anon_sym_extern] = ACTIONS(3040),
    [anon_sym_safe] = ACTIONS(3040),
    [anon_sym_yield] = ACTIONS(3040),
    [anon_sym_move] = ACTIONS(3040),
    [anon_sym_try] = ACTIONS(3040),
    [sym_integer_literal] = ACTIONS(3038),
    [aux_sym_string_literal_token1] = ACTIONS(3038),
    [sym_char_literal] = ACTIONS(3038),
    [anon_sym_true] = ACTIONS(3040),
    [anon_sym_false] = ACTIONS(3040),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3040),
    [sym_super] = ACTIONS(3040),
    [sym_crate] = ACTIONS(3040),
    [aux_sym_metavariable_token1] = ACTIONS(3038),
    [sym__raw_string_literal_start] = ACTIONS(3038),
    [sym_float_literal] = ACTIONS(3038),
    [sym__outer_block_doc_comment_start] = ACTIONS(3038),
    [sym__outer_line_doc_comment_start] = ACTIONS(3038),
  },
  [STATE(818)] = {
    [sym_line_comment] = STATE(818),
    [sym_block_comment] = STATE(818),
    [ts_builtin_sym_end] = ACTIONS(3042),
    [sym_identifier] = ACTIONS(3044),
    [anon_sym_SEMI] = ACTIONS(3042),
    [anon_sym_macro] = ACTIONS(3044),
    [anon_sym_LPAREN] = ACTIONS(3042),
    [anon_sym_LBRACE] = ACTIONS(3042),
    [anon_sym_RBRACE] = ACTIONS(3042),
    [anon_sym_macro_rules] = ACTIONS(3044),
    [anon_sym_BANG] = ACTIONS(3042),
    [anon_sym_LBRACK] = ACTIONS(3042),
    [anon_sym_STAR] = ACTIONS(3042),
    [anon_sym_u8] = ACTIONS(3044),
    [anon_sym_i8] = ACTIONS(3044),
    [anon_sym_u16] = ACTIONS(3044),
    [anon_sym_i16] = ACTIONS(3044),
    [anon_sym_u32] = ACTIONS(3044),
    [anon_sym_i32] = ACTIONS(3044),
    [anon_sym_u64] = ACTIONS(3044),
    [anon_sym_i64] = ACTIONS(3044),
    [anon_sym_u128] = ACTIONS(3044),
    [anon_sym_i128] = ACTIONS(3044),
    [anon_sym_isize] = ACTIONS(3044),
    [anon_sym_usize] = ACTIONS(3044),
    [anon_sym_f32] = ACTIONS(3044),
    [anon_sym_f64] = ACTIONS(3044),
    [anon_sym_bool] = ACTIONS(3044),
    [anon_sym_str] = ACTIONS(3044),
    [anon_sym_char] = ACTIONS(3044),
    [anon_sym_DASH] = ACTIONS(3042),
    [anon_sym_AMP] = ACTIONS(3042),
    [anon_sym_PIPE] = ACTIONS(3042),
    [anon_sym_LT] = ACTIONS(3042),
    [anon_sym_DOT_DOT] = ACTIONS(3042),
    [anon_sym_COLON_COLON] = ACTIONS(3042),
    [anon_sym_POUND] = ACTIONS(3042),
    [anon_sym_SQUOTE] = ACTIONS(3044),
    [anon_sym_async] = ACTIONS(3044),
    [anon_sym_auto] = ACTIONS(3044),
    [anon_sym_await] = ACTIONS(3044),
    [anon_sym_become] = ACTIONS(3044),
    [anon_sym_break] = ACTIONS(3044),
    [anon_sym_const] = ACTIONS(3044),
    [anon_sym_continue] = ACTIONS(3044),
    [anon_sym_default] = ACTIONS(3044),
    [anon_sym_enum] = ACTIONS(3044),
    [anon_sym_fn] = ACTIONS(3044),
    [anon_sym_for] = ACTIONS(3044),
    [anon_sym_gen] = ACTIONS(3044),
    [anon_sym_if] = ACTIONS(3044),
    [anon_sym_impl] = ACTIONS(3044),
    [anon_sym_let] = ACTIONS(3044),
    [anon_sym_loop] = ACTIONS(3044),
    [anon_sym_match] = ACTIONS(3044),
    [anon_sym_mod] = ACTIONS(3044),
    [anon_sym_pub] = ACTIONS(3044),
    [anon_sym_return] = ACTIONS(3044),
    [anon_sym_raw] = ACTIONS(3044),
    [anon_sym_static] = ACTIONS(3044),
    [anon_sym_struct] = ACTIONS(3044),
    [anon_sym_trait] = ACTIONS(3044),
    [anon_sym_type] = ACTIONS(3044),
    [anon_sym_union] = ACTIONS(3044),
    [anon_sym_unsafe] = ACTIONS(3044),
    [anon_sym_use] = ACTIONS(3044),
    [anon_sym_while] = ACTIONS(3044),
    [anon_sym_extern] = ACTIONS(3044),
    [anon_sym_safe] = ACTIONS(3044),
    [anon_sym_yield] = ACTIONS(3044),
    [anon_sym_move] = ACTIONS(3044),
    [anon_sym_try] = ACTIONS(3044),
    [sym_integer_literal] = ACTIONS(3042),
    [aux_sym_string_literal_token1] = ACTIONS(3042),
    [sym_char_literal] = ACTIONS(3042),
    [anon_sym_true] = ACTIONS(3044),
    [anon_sym_false] = ACTIONS(3044),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3044),
    [sym_super] = ACTIONS(3044),
    [sym_crate] = ACTIONS(3044),
    [aux_sym_metavariable_token1] = ACTIONS(3042),
    [sym__raw_string_literal_start] = ACTIONS(3042),
    [sym_float_literal] = ACTIONS(3042),
    [sym__outer_block_doc_comment_start] = ACTIONS(3042),
    [sym__outer_line_doc_comment_start] = ACTIONS(3042),
  },
  [STATE(819)] = {
    [sym_line_comment] = STATE(819),
    [sym_block_comment] = STATE(819),
    [ts_builtin_sym_end] = ACTIONS(3046),
    [sym_identifier] = ACTIONS(3048),
    [anon_sym_SEMI] = ACTIONS(3046),
    [anon_sym_macro] = ACTIONS(3048),
    [anon_sym_LPAREN] = ACTIONS(3046),
    [anon_sym_LBRACE] = ACTIONS(3046),
    [anon_sym_RBRACE] = ACTIONS(3046),
    [anon_sym_macro_rules] = ACTIONS(3048),
    [anon_sym_BANG] = ACTIONS(3046),
    [anon_sym_LBRACK] = ACTIONS(3046),
    [anon_sym_STAR] = ACTIONS(3046),
    [anon_sym_u8] = ACTIONS(3048),
    [anon_sym_i8] = ACTIONS(3048),
    [anon_sym_u16] = ACTIONS(3048),
    [anon_sym_i16] = ACTIONS(3048),
    [anon_sym_u32] = ACTIONS(3048),
    [anon_sym_i32] = ACTIONS(3048),
    [anon_sym_u64] = ACTIONS(3048),
    [anon_sym_i64] = ACTIONS(3048),
    [anon_sym_u128] = ACTIONS(3048),
    [anon_sym_i128] = ACTIONS(3048),
    [anon_sym_isize] = ACTIONS(3048),
    [anon_sym_usize] = ACTIONS(3048),
    [anon_sym_f32] = ACTIONS(3048),
    [anon_sym_f64] = ACTIONS(3048),
    [anon_sym_bool] = ACTIONS(3048),
    [anon_sym_str] = ACTIONS(3048),
    [anon_sym_char] = ACTIONS(3048),
    [anon_sym_DASH] = ACTIONS(3046),
    [anon_sym_AMP] = ACTIONS(3046),
    [anon_sym_PIPE] = ACTIONS(3046),
    [anon_sym_LT] = ACTIONS(3046),
    [anon_sym_DOT_DOT] = ACTIONS(3046),
    [anon_sym_COLON_COLON] = ACTIONS(3046),
    [anon_sym_POUND] = ACTIONS(3046),
    [anon_sym_SQUOTE] = ACTIONS(3048),
    [anon_sym_async] = ACTIONS(3048),
    [anon_sym_auto] = ACTIONS(3048),
    [anon_sym_await] = ACTIONS(3048),
    [anon_sym_become] = ACTIONS(3048),
    [anon_sym_break] = ACTIONS(3048),
    [anon_sym_const] = ACTIONS(3048),
    [anon_sym_continue] = ACTIONS(3048),
    [anon_sym_default] = ACTIONS(3048),
    [anon_sym_enum] = ACTIONS(3048),
    [anon_sym_fn] = ACTIONS(3048),
    [anon_sym_for] = ACTIONS(3048),
    [anon_sym_gen] = ACTIONS(3048),
    [anon_sym_if] = ACTIONS(3048),
    [anon_sym_impl] = ACTIONS(3048),
    [anon_sym_let] = ACTIONS(3048),
    [anon_sym_loop] = ACTIONS(3048),
    [anon_sym_match] = ACTIONS(3048),
    [anon_sym_mod] = ACTIONS(3048),
    [anon_sym_pub] = ACTIONS(3048),
    [anon_sym_return] = ACTIONS(3048),
    [anon_sym_raw] = ACTIONS(3048),
    [anon_sym_static] = ACTIONS(3048),
    [anon_sym_struct] = ACTIONS(3048),
    [anon_sym_trait] = ACTIONS(3048),
    [anon_sym_type] = ACTIONS(3048),
    [anon_sym_union] = ACTIONS(3048),
    [anon_sym_unsafe] = ACTIONS(3048),
    [anon_sym_use] = ACTIONS(3048),
    [anon_sym_while] = ACTIONS(3048),
    [anon_sym_extern] = ACTIONS(3048),
    [anon_sym_safe] = ACTIONS(3048),
    [anon_sym_yield] = ACTIONS(3048),
    [anon_sym_move] = ACTIONS(3048),
    [anon_sym_try] = ACTIONS(3048),
    [sym_integer_literal] = ACTIONS(3046),
    [aux_sym_string_literal_token1] = ACTIONS(3046),
    [sym_char_literal] = ACTIONS(3046),
    [anon_sym_true] = ACTIONS(3048),
    [anon_sym_false] = ACTIONS(3048),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3048),
    [sym_super] = ACTIONS(3048),
    [sym_crate] = ACTIONS(3048),
    [aux_sym_metavariable_token1] = ACTIONS(3046),
    [sym__raw_string_literal_start] = ACTIONS(3046),
    [sym_float_literal] = ACTIONS(3046),
    [sym__outer_block_doc_comment_start] = ACTIONS(3046),
    [sym__outer_line_doc_comment_start] = ACTIONS(3046),
  },
  [STATE(820)] = {
    [sym_line_comment] = STATE(820),
    [sym_block_comment] = STATE(820),
    [ts_builtin_sym_end] = ACTIONS(3050),
    [sym_identifier] = ACTIONS(3052),
    [anon_sym_SEMI] = ACTIONS(3050),
    [anon_sym_macro] = ACTIONS(3052),
    [anon_sym_LPAREN] = ACTIONS(3050),
    [anon_sym_LBRACE] = ACTIONS(3050),
    [anon_sym_RBRACE] = ACTIONS(3050),
    [anon_sym_macro_rules] = ACTIONS(3052),
    [anon_sym_BANG] = ACTIONS(3050),
    [anon_sym_LBRACK] = ACTIONS(3050),
    [anon_sym_STAR] = ACTIONS(3050),
    [anon_sym_u8] = ACTIONS(3052),
    [anon_sym_i8] = ACTIONS(3052),
    [anon_sym_u16] = ACTIONS(3052),
    [anon_sym_i16] = ACTIONS(3052),
    [anon_sym_u32] = ACTIONS(3052),
    [anon_sym_i32] = ACTIONS(3052),
    [anon_sym_u64] = ACTIONS(3052),
    [anon_sym_i64] = ACTIONS(3052),
    [anon_sym_u128] = ACTIONS(3052),
    [anon_sym_i128] = ACTIONS(3052),
    [anon_sym_isize] = ACTIONS(3052),
    [anon_sym_usize] = ACTIONS(3052),
    [anon_sym_f32] = ACTIONS(3052),
    [anon_sym_f64] = ACTIONS(3052),
    [anon_sym_bool] = ACTIONS(3052),
    [anon_sym_str] = ACTIONS(3052),
    [anon_sym_char] = ACTIONS(3052),
    [anon_sym_DASH] = ACTIONS(3050),
    [anon_sym_AMP] = ACTIONS(3050),
    [anon_sym_PIPE] = ACTIONS(3050),
    [anon_sym_LT] = ACTIONS(3050),
    [anon_sym_DOT_DOT] = ACTIONS(3050),
    [anon_sym_COLON_COLON] = ACTIONS(3050),
    [anon_sym_POUND] = ACTIONS(3050),
    [anon_sym_SQUOTE] = ACTIONS(3052),
    [anon_sym_async] = ACTIONS(3052),
    [anon_sym_auto] = ACTIONS(3052),
    [anon_sym_await] = ACTIONS(3052),
    [anon_sym_become] = ACTIONS(3052),
    [anon_sym_break] = ACTIONS(3052),
    [anon_sym_const] = ACTIONS(3052),
    [anon_sym_continue] = ACTIONS(3052),
    [anon_sym_default] = ACTIONS(3052),
    [anon_sym_enum] = ACTIONS(3052),
    [anon_sym_fn] = ACTIONS(3052),
    [anon_sym_for] = ACTIONS(3052),
    [anon_sym_gen] = ACTIONS(3052),
    [anon_sym_if] = ACTIONS(3052),
    [anon_sym_impl] = ACTIONS(3052),
    [anon_sym_let] = ACTIONS(3052),
    [anon_sym_loop] = ACTIONS(3052),
    [anon_sym_match] = ACTIONS(3052),
    [anon_sym_mod] = ACTIONS(3052),
    [anon_sym_pub] = ACTIONS(3052),
    [anon_sym_return] = ACTIONS(3052),
    [anon_sym_raw] = ACTIONS(3052),
    [anon_sym_static] = ACTIONS(3052),
    [anon_sym_struct] = ACTIONS(3052),
    [anon_sym_trait] = ACTIONS(3052),
    [anon_sym_type] = ACTIONS(3052),
    [anon_sym_union] = ACTIONS(3052),
    [anon_sym_unsafe] = ACTIONS(3052),
    [anon_sym_use] = ACTIONS(3052),
    [anon_sym_while] = ACTIONS(3052),
    [anon_sym_extern] = ACTIONS(3052),
    [anon_sym_safe] = ACTIONS(3052),
    [anon_sym_yield] = ACTIONS(3052),
    [anon_sym_move] = ACTIONS(3052),
    [anon_sym_try] = ACTIONS(3052),
    [sym_integer_literal] = ACTIONS(3050),
    [aux_sym_string_literal_token1] = ACTIONS(3050),
    [sym_char_literal] = ACTIONS(3050),
    [anon_sym_true] = ACTIONS(3052),
    [anon_sym_false] = ACTIONS(3052),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3052),
    [sym_super] = ACTIONS(3052),
    [sym_crate] = ACTIONS(3052),
    [aux_sym_metavariable_token1] = ACTIONS(3050),
    [sym__raw_string_literal_start] = ACTIONS(3050),
    [sym_float_literal] = ACTIONS(3050),
    [sym__outer_block_doc_comment_start] = ACTIONS(3050),
    [sym__outer_line_doc_comment_start] = ACTIONS(3050),
  },
  [STATE(821)] = {
    [sym_line_comment] = STATE(821),
    [sym_block_comment] = STATE(821),
    [ts_builtin_sym_end] = ACTIONS(3054),
    [sym_identifier] = ACTIONS(3056),
    [anon_sym_SEMI] = ACTIONS(3054),
    [anon_sym_macro] = ACTIONS(3056),
    [anon_sym_LPAREN] = ACTIONS(3054),
    [anon_sym_LBRACE] = ACTIONS(3054),
    [anon_sym_RBRACE] = ACTIONS(3054),
    [anon_sym_macro_rules] = ACTIONS(3056),
    [anon_sym_BANG] = ACTIONS(3054),
    [anon_sym_LBRACK] = ACTIONS(3054),
    [anon_sym_STAR] = ACTIONS(3054),
    [anon_sym_u8] = ACTIONS(3056),
    [anon_sym_i8] = ACTIONS(3056),
    [anon_sym_u16] = ACTIONS(3056),
    [anon_sym_i16] = ACTIONS(3056),
    [anon_sym_u32] = ACTIONS(3056),
    [anon_sym_i32] = ACTIONS(3056),
    [anon_sym_u64] = ACTIONS(3056),
    [anon_sym_i64] = ACTIONS(3056),
    [anon_sym_u128] = ACTIONS(3056),
    [anon_sym_i128] = ACTIONS(3056),
    [anon_sym_isize] = ACTIONS(3056),
    [anon_sym_usize] = ACTIONS(3056),
    [anon_sym_f32] = ACTIONS(3056),
    [anon_sym_f64] = ACTIONS(3056),
    [anon_sym_bool] = ACTIONS(3056),
    [anon_sym_str] = ACTIONS(3056),
    [anon_sym_char] = ACTIONS(3056),
    [anon_sym_DASH] = ACTIONS(3054),
    [anon_sym_AMP] = ACTIONS(3054),
    [anon_sym_PIPE] = ACTIONS(3054),
    [anon_sym_LT] = ACTIONS(3054),
    [anon_sym_DOT_DOT] = ACTIONS(3054),
    [anon_sym_COLON_COLON] = ACTIONS(3054),
    [anon_sym_POUND] = ACTIONS(3054),
    [anon_sym_SQUOTE] = ACTIONS(3056),
    [anon_sym_async] = ACTIONS(3056),
    [anon_sym_auto] = ACTIONS(3056),
    [anon_sym_await] = ACTIONS(3056),
    [anon_sym_become] = ACTIONS(3056),
    [anon_sym_break] = ACTIONS(3056),
    [anon_sym_const] = ACTIONS(3056),
    [anon_sym_continue] = ACTIONS(3056),
    [anon_sym_default] = ACTIONS(3056),
    [anon_sym_enum] = ACTIONS(3056),
    [anon_sym_fn] = ACTIONS(3056),
    [anon_sym_for] = ACTIONS(3056),
    [anon_sym_gen] = ACTIONS(3056),
    [anon_sym_if] = ACTIONS(3056),
    [anon_sym_impl] = ACTIONS(3056),
    [anon_sym_let] = ACTIONS(3056),
    [anon_sym_loop] = ACTIONS(3056),
    [anon_sym_match] = ACTIONS(3056),
    [anon_sym_mod] = ACTIONS(3056),
    [anon_sym_pub] = ACTIONS(3056),
    [anon_sym_return] = ACTIONS(3056),
    [anon_sym_raw] = ACTIONS(3056),
    [anon_sym_static] = ACTIONS(3056),
    [anon_sym_struct] = ACTIONS(3056),
    [anon_sym_trait] = ACTIONS(3056),
    [anon_sym_type] = ACTIONS(3056),
    [anon_sym_union] = ACTIONS(3056),
    [anon_sym_unsafe] = ACTIONS(3056),
    [anon_sym_use] = ACTIONS(3056),
    [anon_sym_while] = ACTIONS(3056),
    [anon_sym_extern] = ACTIONS(3056),
    [anon_sym_safe] = ACTIONS(3056),
    [anon_sym_yield] = ACTIONS(3056),
    [anon_sym_move] = ACTIONS(3056),
    [anon_sym_try] = ACTIONS(3056),
    [sym_integer_literal] = ACTIONS(3054),
    [aux_sym_string_literal_token1] = ACTIONS(3054),
    [sym_char_literal] = ACTIONS(3054),
    [anon_sym_true] = ACTIONS(3056),
    [anon_sym_false] = ACTIONS(3056),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3056),
    [sym_super] = ACTIONS(3056),
    [sym_crate] = ACTIONS(3056),
    [aux_sym_metavariable_token1] = ACTIONS(3054),
    [sym__raw_string_literal_start] = ACTIONS(3054),
    [sym_float_literal] = ACTIONS(3054),
    [sym__outer_block_doc_comment_start] = ACTIONS(3054),
    [sym__outer_line_doc_comment_start] = ACTIONS(3054),
  },
  [STATE(822)] = {
    [sym_line_comment] = STATE(822),
    [sym_block_comment] = STATE(822),
    [ts_builtin_sym_end] = ACTIONS(3058),
    [sym_identifier] = ACTIONS(3060),
    [anon_sym_SEMI] = ACTIONS(3058),
    [anon_sym_macro] = ACTIONS(3060),
    [anon_sym_LPAREN] = ACTIONS(3058),
    [anon_sym_LBRACE] = ACTIONS(3058),
    [anon_sym_RBRACE] = ACTIONS(3058),
    [anon_sym_macro_rules] = ACTIONS(3060),
    [anon_sym_BANG] = ACTIONS(3058),
    [anon_sym_LBRACK] = ACTIONS(3058),
    [anon_sym_STAR] = ACTIONS(3058),
    [anon_sym_u8] = ACTIONS(3060),
    [anon_sym_i8] = ACTIONS(3060),
    [anon_sym_u16] = ACTIONS(3060),
    [anon_sym_i16] = ACTIONS(3060),
    [anon_sym_u32] = ACTIONS(3060),
    [anon_sym_i32] = ACTIONS(3060),
    [anon_sym_u64] = ACTIONS(3060),
    [anon_sym_i64] = ACTIONS(3060),
    [anon_sym_u128] = ACTIONS(3060),
    [anon_sym_i128] = ACTIONS(3060),
    [anon_sym_isize] = ACTIONS(3060),
    [anon_sym_usize] = ACTIONS(3060),
    [anon_sym_f32] = ACTIONS(3060),
    [anon_sym_f64] = ACTIONS(3060),
    [anon_sym_bool] = ACTIONS(3060),
    [anon_sym_str] = ACTIONS(3060),
    [anon_sym_char] = ACTIONS(3060),
    [anon_sym_DASH] = ACTIONS(3058),
    [anon_sym_AMP] = ACTIONS(3058),
    [anon_sym_PIPE] = ACTIONS(3058),
    [anon_sym_LT] = ACTIONS(3058),
    [anon_sym_DOT_DOT] = ACTIONS(3058),
    [anon_sym_COLON_COLON] = ACTIONS(3058),
    [anon_sym_POUND] = ACTIONS(3058),
    [anon_sym_SQUOTE] = ACTIONS(3060),
    [anon_sym_async] = ACTIONS(3060),
    [anon_sym_auto] = ACTIONS(3060),
    [anon_sym_await] = ACTIONS(3060),
    [anon_sym_become] = ACTIONS(3060),
    [anon_sym_break] = ACTIONS(3060),
    [anon_sym_const] = ACTIONS(3060),
    [anon_sym_continue] = ACTIONS(3060),
    [anon_sym_default] = ACTIONS(3060),
    [anon_sym_enum] = ACTIONS(3060),
    [anon_sym_fn] = ACTIONS(3060),
    [anon_sym_for] = ACTIONS(3060),
    [anon_sym_gen] = ACTIONS(3060),
    [anon_sym_if] = ACTIONS(3060),
    [anon_sym_impl] = ACTIONS(3060),
    [anon_sym_let] = ACTIONS(3060),
    [anon_sym_loop] = ACTIONS(3060),
    [anon_sym_match] = ACTIONS(3060),
    [anon_sym_mod] = ACTIONS(3060),
    [anon_sym_pub] = ACTIONS(3060),
    [anon_sym_return] = ACTIONS(3060),
    [anon_sym_raw] = ACTIONS(3060),
    [anon_sym_static] = ACTIONS(3060),
    [anon_sym_struct] = ACTIONS(3060),
    [anon_sym_trait] = ACTIONS(3060),
    [anon_sym_type] = ACTIONS(3060),
    [anon_sym_union] = ACTIONS(3060),
    [anon_sym_unsafe] = ACTIONS(3060),
    [anon_sym_use] = ACTIONS(3060),
    [anon_sym_while] = ACTIONS(3060),
    [anon_sym_extern] = ACTIONS(3060),
    [anon_sym_safe] = ACTIONS(3060),
    [anon_sym_yield] = ACTIONS(3060),
    [anon_sym_move] = ACTIONS(3060),
    [anon_sym_try] = ACTIONS(3060),
    [sym_integer_literal] = ACTIONS(3058),
    [aux_sym_string_literal_token1] = ACTIONS(3058),
    [sym_char_literal] = ACTIONS(3058),
    [anon_sym_true] = ACTIONS(3060),
    [anon_sym_false] = ACTIONS(3060),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3060),
    [sym_super] = ACTIONS(3060),
    [sym_crate] = ACTIONS(3060),
    [aux_sym_metavariable_token1] = ACTIONS(3058),
    [sym__raw_string_literal_start] = ACTIONS(3058),
    [sym_float_literal] = ACTIONS(3058),
    [sym__outer_block_doc_comment_start] = ACTIONS(3058),
    [sym__outer_line_doc_comment_start] = ACTIONS(3058),
  },
  [STATE(823)] = {
    [sym_line_comment] = STATE(823),
    [sym_block_comment] = STATE(823),
    [ts_builtin_sym_end] = ACTIONS(3062),
    [sym_identifier] = ACTIONS(3064),
    [anon_sym_SEMI] = ACTIONS(3062),
    [anon_sym_macro] = ACTIONS(3064),
    [anon_sym_LPAREN] = ACTIONS(3062),
    [anon_sym_LBRACE] = ACTIONS(3062),
    [anon_sym_RBRACE] = ACTIONS(3062),
    [anon_sym_macro_rules] = ACTIONS(3064),
    [anon_sym_BANG] = ACTIONS(3062),
    [anon_sym_LBRACK] = ACTIONS(3062),
    [anon_sym_STAR] = ACTIONS(3062),
    [anon_sym_u8] = ACTIONS(3064),
    [anon_sym_i8] = ACTIONS(3064),
    [anon_sym_u16] = ACTIONS(3064),
    [anon_sym_i16] = ACTIONS(3064),
    [anon_sym_u32] = ACTIONS(3064),
    [anon_sym_i32] = ACTIONS(3064),
    [anon_sym_u64] = ACTIONS(3064),
    [anon_sym_i64] = ACTIONS(3064),
    [anon_sym_u128] = ACTIONS(3064),
    [anon_sym_i128] = ACTIONS(3064),
    [anon_sym_isize] = ACTIONS(3064),
    [anon_sym_usize] = ACTIONS(3064),
    [anon_sym_f32] = ACTIONS(3064),
    [anon_sym_f64] = ACTIONS(3064),
    [anon_sym_bool] = ACTIONS(3064),
    [anon_sym_str] = ACTIONS(3064),
    [anon_sym_char] = ACTIONS(3064),
    [anon_sym_DASH] = ACTIONS(3062),
    [anon_sym_AMP] = ACTIONS(3062),
    [anon_sym_PIPE] = ACTIONS(3062),
    [anon_sym_LT] = ACTIONS(3062),
    [anon_sym_DOT_DOT] = ACTIONS(3062),
    [anon_sym_COLON_COLON] = ACTIONS(3062),
    [anon_sym_POUND] = ACTIONS(3062),
    [anon_sym_SQUOTE] = ACTIONS(3064),
    [anon_sym_async] = ACTIONS(3064),
    [anon_sym_auto] = ACTIONS(3064),
    [anon_sym_await] = ACTIONS(3064),
    [anon_sym_become] = ACTIONS(3064),
    [anon_sym_break] = ACTIONS(3064),
    [anon_sym_const] = ACTIONS(3064),
    [anon_sym_continue] = ACTIONS(3064),
    [anon_sym_default] = ACTIONS(3064),
    [anon_sym_enum] = ACTIONS(3064),
    [anon_sym_fn] = ACTIONS(3064),
    [anon_sym_for] = ACTIONS(3064),
    [anon_sym_gen] = ACTIONS(3064),
    [anon_sym_if] = ACTIONS(3064),
    [anon_sym_impl] = ACTIONS(3064),
    [anon_sym_let] = ACTIONS(3064),
    [anon_sym_loop] = ACTIONS(3064),
    [anon_sym_match] = ACTIONS(3064),
    [anon_sym_mod] = ACTIONS(3064),
    [anon_sym_pub] = ACTIONS(3064),
    [anon_sym_return] = ACTIONS(3064),
    [anon_sym_raw] = ACTIONS(3064),
    [anon_sym_static] = ACTIONS(3064),
    [anon_sym_struct] = ACTIONS(3064),
    [anon_sym_trait] = ACTIONS(3064),
    [anon_sym_type] = ACTIONS(3064),
    [anon_sym_union] = ACTIONS(3064),
    [anon_sym_unsafe] = ACTIONS(3064),
    [anon_sym_use] = ACTIONS(3064),
    [anon_sym_while] = ACTIONS(3064),
    [anon_sym_extern] = ACTIONS(3064),
    [anon_sym_safe] = ACTIONS(3064),
    [anon_sym_yield] = ACTIONS(3064),
    [anon_sym_move] = ACTIONS(3064),
    [anon_sym_try] = ACTIONS(3064),
    [sym_integer_literal] = ACTIONS(3062),
    [aux_sym_string_literal_token1] = ACTIONS(3062),
    [sym_char_literal] = ACTIONS(3062),
    [anon_sym_true] = ACTIONS(3064),
    [anon_sym_false] = ACTIONS(3064),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3064),
    [sym_super] = ACTIONS(3064),
    [sym_crate] = ACTIONS(3064),
    [aux_sym_metavariable_token1] = ACTIONS(3062),
    [sym__raw_string_literal_start] = ACTIONS(3062),
    [sym_float_literal] = ACTIONS(3062),
    [sym__outer_block_doc_comment_start] = ACTIONS(3062),
    [sym__outer_line_doc_comment_start] = ACTIONS(3062),
  },
  [STATE(824)] = {
    [sym_line_comment] = STATE(824),
    [sym_block_comment] = STATE(824),
    [ts_builtin_sym_end] = ACTIONS(3066),
    [sym_identifier] = ACTIONS(3068),
    [anon_sym_SEMI] = ACTIONS(3066),
    [anon_sym_macro] = ACTIONS(3068),
    [anon_sym_LPAREN] = ACTIONS(3066),
    [anon_sym_LBRACE] = ACTIONS(3066),
    [anon_sym_RBRACE] = ACTIONS(3066),
    [anon_sym_macro_rules] = ACTIONS(3068),
    [anon_sym_BANG] = ACTIONS(3066),
    [anon_sym_LBRACK] = ACTIONS(3066),
    [anon_sym_STAR] = ACTIONS(3066),
    [anon_sym_u8] = ACTIONS(3068),
    [anon_sym_i8] = ACTIONS(3068),
    [anon_sym_u16] = ACTIONS(3068),
    [anon_sym_i16] = ACTIONS(3068),
    [anon_sym_u32] = ACTIONS(3068),
    [anon_sym_i32] = ACTIONS(3068),
    [anon_sym_u64] = ACTIONS(3068),
    [anon_sym_i64] = ACTIONS(3068),
    [anon_sym_u128] = ACTIONS(3068),
    [anon_sym_i128] = ACTIONS(3068),
    [anon_sym_isize] = ACTIONS(3068),
    [anon_sym_usize] = ACTIONS(3068),
    [anon_sym_f32] = ACTIONS(3068),
    [anon_sym_f64] = ACTIONS(3068),
    [anon_sym_bool] = ACTIONS(3068),
    [anon_sym_str] = ACTIONS(3068),
    [anon_sym_char] = ACTIONS(3068),
    [anon_sym_DASH] = ACTIONS(3066),
    [anon_sym_AMP] = ACTIONS(3066),
    [anon_sym_PIPE] = ACTIONS(3066),
    [anon_sym_LT] = ACTIONS(3066),
    [anon_sym_DOT_DOT] = ACTIONS(3066),
    [anon_sym_COLON_COLON] = ACTIONS(3066),
    [anon_sym_POUND] = ACTIONS(3066),
    [anon_sym_SQUOTE] = ACTIONS(3068),
    [anon_sym_async] = ACTIONS(3068),
    [anon_sym_auto] = ACTIONS(3068),
    [anon_sym_await] = ACTIONS(3068),
    [anon_sym_become] = ACTIONS(3068),
    [anon_sym_break] = ACTIONS(3068),
    [anon_sym_const] = ACTIONS(3068),
    [anon_sym_continue] = ACTIONS(3068),
    [anon_sym_default] = ACTIONS(3068),
    [anon_sym_enum] = ACTIONS(3068),
    [anon_sym_fn] = ACTIONS(3068),
    [anon_sym_for] = ACTIONS(3068),
    [anon_sym_gen] = ACTIONS(3068),
    [anon_sym_if] = ACTIONS(3068),
    [anon_sym_impl] = ACTIONS(3068),
    [anon_sym_let] = ACTIONS(3068),
    [anon_sym_loop] = ACTIONS(3068),
    [anon_sym_match] = ACTIONS(3068),
    [anon_sym_mod] = ACTIONS(3068),
    [anon_sym_pub] = ACTIONS(3068),
    [anon_sym_return] = ACTIONS(3068),
    [anon_sym_raw] = ACTIONS(3068),
    [anon_sym_static] = ACTIONS(3068),
    [anon_sym_struct] = ACTIONS(3068),
    [anon_sym_trait] = ACTIONS(3068),
    [anon_sym_type] = ACTIONS(3068),
    [anon_sym_union] = ACTIONS(3068),
    [anon_sym_unsafe] = ACTIONS(3068),
    [anon_sym_use] = ACTIONS(3068),
    [anon_sym_while] = ACTIONS(3068),
    [anon_sym_extern] = ACTIONS(3068),
    [anon_sym_safe] = ACTIONS(3068),
    [anon_sym_yield] = ACTIONS(3068),
    [anon_sym_move] = ACTIONS(3068),
    [anon_sym_try] = ACTIONS(3068),
    [sym_integer_literal] = ACTIONS(3066),
    [aux_sym_string_literal_token1] = ACTIONS(3066),
    [sym_char_literal] = ACTIONS(3066),
    [anon_sym_true] = ACTIONS(3068),
    [anon_sym_false] = ACTIONS(3068),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3068),
    [sym_super] = ACTIONS(3068),
    [sym_crate] = ACTIONS(3068),
    [aux_sym_metavariable_token1] = ACTIONS(3066),
    [sym__raw_string_literal_start] = ACTIONS(3066),
    [sym_float_literal] = ACTIONS(3066),
    [sym__outer_block_doc_comment_start] = ACTIONS(3066),
    [sym__outer_line_doc_comment_start] = ACTIONS(3066),
  },
  [STATE(825)] = {
    [sym_line_comment] = STATE(825),
    [sym_block_comment] = STATE(825),
    [ts_builtin_sym_end] = ACTIONS(3070),
    [sym_identifier] = ACTIONS(3072),
    [anon_sym_SEMI] = ACTIONS(3070),
    [anon_sym_macro] = ACTIONS(3072),
    [anon_sym_LPAREN] = ACTIONS(3070),
    [anon_sym_LBRACE] = ACTIONS(3070),
    [anon_sym_RBRACE] = ACTIONS(3070),
    [anon_sym_macro_rules] = ACTIONS(3072),
    [anon_sym_BANG] = ACTIONS(3070),
    [anon_sym_LBRACK] = ACTIONS(3070),
    [anon_sym_STAR] = ACTIONS(3070),
    [anon_sym_u8] = ACTIONS(3072),
    [anon_sym_i8] = ACTIONS(3072),
    [anon_sym_u16] = ACTIONS(3072),
    [anon_sym_i16] = ACTIONS(3072),
    [anon_sym_u32] = ACTIONS(3072),
    [anon_sym_i32] = ACTIONS(3072),
    [anon_sym_u64] = ACTIONS(3072),
    [anon_sym_i64] = ACTIONS(3072),
    [anon_sym_u128] = ACTIONS(3072),
    [anon_sym_i128] = ACTIONS(3072),
    [anon_sym_isize] = ACTIONS(3072),
    [anon_sym_usize] = ACTIONS(3072),
    [anon_sym_f32] = ACTIONS(3072),
    [anon_sym_f64] = ACTIONS(3072),
    [anon_sym_bool] = ACTIONS(3072),
    [anon_sym_str] = ACTIONS(3072),
    [anon_sym_char] = ACTIONS(3072),
    [anon_sym_DASH] = ACTIONS(3070),
    [anon_sym_AMP] = ACTIONS(3070),
    [anon_sym_PIPE] = ACTIONS(3070),
    [anon_sym_LT] = ACTIONS(3070),
    [anon_sym_DOT_DOT] = ACTIONS(3070),
    [anon_sym_COLON_COLON] = ACTIONS(3070),
    [anon_sym_POUND] = ACTIONS(3070),
    [anon_sym_SQUOTE] = ACTIONS(3072),
    [anon_sym_async] = ACTIONS(3072),
    [anon_sym_auto] = ACTIONS(3072),
    [anon_sym_await] = ACTIONS(3072),
    [anon_sym_become] = ACTIONS(3072),
    [anon_sym_break] = ACTIONS(3072),
    [anon_sym_const] = ACTIONS(3072),
    [anon_sym_continue] = ACTIONS(3072),
    [anon_sym_default] = ACTIONS(3072),
    [anon_sym_enum] = ACTIONS(3072),
    [anon_sym_fn] = ACTIONS(3072),
    [anon_sym_for] = ACTIONS(3072),
    [anon_sym_gen] = ACTIONS(3072),
    [anon_sym_if] = ACTIONS(3072),
    [anon_sym_impl] = ACTIONS(3072),
    [anon_sym_let] = ACTIONS(3072),
    [anon_sym_loop] = ACTIONS(3072),
    [anon_sym_match] = ACTIONS(3072),
    [anon_sym_mod] = ACTIONS(3072),
    [anon_sym_pub] = ACTIONS(3072),
    [anon_sym_return] = ACTIONS(3072),
    [anon_sym_raw] = ACTIONS(3072),
    [anon_sym_static] = ACTIONS(3072),
    [anon_sym_struct] = ACTIONS(3072),
    [anon_sym_trait] = ACTIONS(3072),
    [anon_sym_type] = ACTIONS(3072),
    [anon_sym_union] = ACTIONS(3072),
    [anon_sym_unsafe] = ACTIONS(3072),
    [anon_sym_use] = ACTIONS(3072),
    [anon_sym_while] = ACTIONS(3072),
    [anon_sym_extern] = ACTIONS(3072),
    [anon_sym_safe] = ACTIONS(3072),
    [anon_sym_yield] = ACTIONS(3072),
    [anon_sym_move] = ACTIONS(3072),
    [anon_sym_try] = ACTIONS(3072),
    [sym_integer_literal] = ACTIONS(3070),
    [aux_sym_string_literal_token1] = ACTIONS(3070),
    [sym_char_literal] = ACTIONS(3070),
    [anon_sym_true] = ACTIONS(3072),
    [anon_sym_false] = ACTIONS(3072),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3072),
    [sym_super] = ACTIONS(3072),
    [sym_crate] = ACTIONS(3072),
    [aux_sym_metavariable_token1] = ACTIONS(3070),
    [sym__raw_string_literal_start] = ACTIONS(3070),
    [sym_float_literal] = ACTIONS(3070),
    [sym__outer_block_doc_comment_start] = ACTIONS(3070),
    [sym__outer_line_doc_comment_start] = ACTIONS(3070),
  },
  [STATE(826)] = {
    [sym_line_comment] = STATE(826),
    [sym_block_comment] = STATE(826),
    [ts_builtin_sym_end] = ACTIONS(3074),
    [sym_identifier] = ACTIONS(3076),
    [anon_sym_SEMI] = ACTIONS(3074),
    [anon_sym_macro] = ACTIONS(3076),
    [anon_sym_LPAREN] = ACTIONS(3074),
    [anon_sym_LBRACE] = ACTIONS(3074),
    [anon_sym_RBRACE] = ACTIONS(3074),
    [anon_sym_macro_rules] = ACTIONS(3076),
    [anon_sym_BANG] = ACTIONS(3074),
    [anon_sym_LBRACK] = ACTIONS(3074),
    [anon_sym_STAR] = ACTIONS(3074),
    [anon_sym_u8] = ACTIONS(3076),
    [anon_sym_i8] = ACTIONS(3076),
    [anon_sym_u16] = ACTIONS(3076),
    [anon_sym_i16] = ACTIONS(3076),
    [anon_sym_u32] = ACTIONS(3076),
    [anon_sym_i32] = ACTIONS(3076),
    [anon_sym_u64] = ACTIONS(3076),
    [anon_sym_i64] = ACTIONS(3076),
    [anon_sym_u128] = ACTIONS(3076),
    [anon_sym_i128] = ACTIONS(3076),
    [anon_sym_isize] = ACTIONS(3076),
    [anon_sym_usize] = ACTIONS(3076),
    [anon_sym_f32] = ACTIONS(3076),
    [anon_sym_f64] = ACTIONS(3076),
    [anon_sym_bool] = ACTIONS(3076),
    [anon_sym_str] = ACTIONS(3076),
    [anon_sym_char] = ACTIONS(3076),
    [anon_sym_DASH] = ACTIONS(3074),
    [anon_sym_AMP] = ACTIONS(3074),
    [anon_sym_PIPE] = ACTIONS(3074),
    [anon_sym_LT] = ACTIONS(3074),
    [anon_sym_DOT_DOT] = ACTIONS(3074),
    [anon_sym_COLON_COLON] = ACTIONS(3074),
    [anon_sym_POUND] = ACTIONS(3074),
    [anon_sym_SQUOTE] = ACTIONS(3076),
    [anon_sym_async] = ACTIONS(3076),
    [anon_sym_auto] = ACTIONS(3076),
    [anon_sym_await] = ACTIONS(3076),
    [anon_sym_become] = ACTIONS(3076),
    [anon_sym_break] = ACTIONS(3076),
    [anon_sym_const] = ACTIONS(3076),
    [anon_sym_continue] = ACTIONS(3076),
    [anon_sym_default] = ACTIONS(3076),
    [anon_sym_enum] = ACTIONS(3076),
    [anon_sym_fn] = ACTIONS(3076),
    [anon_sym_for] = ACTIONS(3076),
    [anon_sym_gen] = ACTIONS(3076),
    [anon_sym_if] = ACTIONS(3076),
    [anon_sym_impl] = ACTIONS(3076),
    [anon_sym_let] = ACTIONS(3076),
    [anon_sym_loop] = ACTIONS(3076),
    [anon_sym_match] = ACTIONS(3076),
    [anon_sym_mod] = ACTIONS(3076),
    [anon_sym_pub] = ACTIONS(3076),
    [anon_sym_return] = ACTIONS(3076),
    [anon_sym_raw] = ACTIONS(3076),
    [anon_sym_static] = ACTIONS(3076),
    [anon_sym_struct] = ACTIONS(3076),
    [anon_sym_trait] = ACTIONS(3076),
    [anon_sym_type] = ACTIONS(3076),
    [anon_sym_union] = ACTIONS(3076),
    [anon_sym_unsafe] = ACTIONS(3076),
    [anon_sym_use] = ACTIONS(3076),
    [anon_sym_while] = ACTIONS(3076),
    [anon_sym_extern] = ACTIONS(3076),
    [anon_sym_safe] = ACTIONS(3076),
    [anon_sym_yield] = ACTIONS(3076),
    [anon_sym_move] = ACTIONS(3076),
    [anon_sym_try] = ACTIONS(3076),
    [sym_integer_literal] = ACTIONS(3074),
    [aux_sym_string_literal_token1] = ACTIONS(3074),
    [sym_char_literal] = ACTIONS(3074),
    [anon_sym_true] = ACTIONS(3076),
    [anon_sym_false] = ACTIONS(3076),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3076),
    [sym_super] = ACTIONS(3076),
    [sym_crate] = ACTIONS(3076),
    [aux_sym_metavariable_token1] = ACTIONS(3074),
    [sym__raw_string_literal_start] = ACTIONS(3074),
    [sym_float_literal] = ACTIONS(3074),
    [sym__outer_block_doc_comment_start] = ACTIONS(3074),
    [sym__outer_line_doc_comment_start] = ACTIONS(3074),
  },
  [STATE(827)] = {
    [sym_line_comment] = STATE(827),
    [sym_block_comment] = STATE(827),
    [ts_builtin_sym_end] = ACTIONS(3078),
    [sym_identifier] = ACTIONS(3080),
    [anon_sym_SEMI] = ACTIONS(3078),
    [anon_sym_macro] = ACTIONS(3080),
    [anon_sym_LPAREN] = ACTIONS(3078),
    [anon_sym_LBRACE] = ACTIONS(3078),
    [anon_sym_RBRACE] = ACTIONS(3078),
    [anon_sym_macro_rules] = ACTIONS(3080),
    [anon_sym_BANG] = ACTIONS(3078),
    [anon_sym_LBRACK] = ACTIONS(3078),
    [anon_sym_STAR] = ACTIONS(3078),
    [anon_sym_u8] = ACTIONS(3080),
    [anon_sym_i8] = ACTIONS(3080),
    [anon_sym_u16] = ACTIONS(3080),
    [anon_sym_i16] = ACTIONS(3080),
    [anon_sym_u32] = ACTIONS(3080),
    [anon_sym_i32] = ACTIONS(3080),
    [anon_sym_u64] = ACTIONS(3080),
    [anon_sym_i64] = ACTIONS(3080),
    [anon_sym_u128] = ACTIONS(3080),
    [anon_sym_i128] = ACTIONS(3080),
    [anon_sym_isize] = ACTIONS(3080),
    [anon_sym_usize] = ACTIONS(3080),
    [anon_sym_f32] = ACTIONS(3080),
    [anon_sym_f64] = ACTIONS(3080),
    [anon_sym_bool] = ACTIONS(3080),
    [anon_sym_str] = ACTIONS(3080),
    [anon_sym_char] = ACTIONS(3080),
    [anon_sym_DASH] = ACTIONS(3078),
    [anon_sym_AMP] = ACTIONS(3078),
    [anon_sym_PIPE] = ACTIONS(3078),
    [anon_sym_LT] = ACTIONS(3078),
    [anon_sym_DOT_DOT] = ACTIONS(3078),
    [anon_sym_COLON_COLON] = ACTIONS(3078),
    [anon_sym_POUND] = ACTIONS(3078),
    [anon_sym_SQUOTE] = ACTIONS(3080),
    [anon_sym_async] = ACTIONS(3080),
    [anon_sym_auto] = ACTIONS(3080),
    [anon_sym_await] = ACTIONS(3080),
    [anon_sym_become] = ACTIONS(3080),
    [anon_sym_break] = ACTIONS(3080),
    [anon_sym_const] = ACTIONS(3080),
    [anon_sym_continue] = ACTIONS(3080),
    [anon_sym_default] = ACTIONS(3080),
    [anon_sym_enum] = ACTIONS(3080),
    [anon_sym_fn] = ACTIONS(3080),
    [anon_sym_for] = ACTIONS(3080),
    [anon_sym_gen] = ACTIONS(3080),
    [anon_sym_if] = ACTIONS(3080),
    [anon_sym_impl] = ACTIONS(3080),
    [anon_sym_let] = ACTIONS(3080),
    [anon_sym_loop] = ACTIONS(3080),
    [anon_sym_match] = ACTIONS(3080),
    [anon_sym_mod] = ACTIONS(3080),
    [anon_sym_pub] = ACTIONS(3080),
    [anon_sym_return] = ACTIONS(3080),
    [anon_sym_raw] = ACTIONS(3080),
    [anon_sym_static] = ACTIONS(3080),
    [anon_sym_struct] = ACTIONS(3080),
    [anon_sym_trait] = ACTIONS(3080),
    [anon_sym_type] = ACTIONS(3080),
    [anon_sym_union] = ACTIONS(3080),
    [anon_sym_unsafe] = ACTIONS(3080),
    [anon_sym_use] = ACTIONS(3080),
    [anon_sym_while] = ACTIONS(3080),
    [anon_sym_extern] = ACTIONS(3080),
    [anon_sym_safe] = ACTIONS(3080),
    [anon_sym_yield] = ACTIONS(3080),
    [anon_sym_move] = ACTIONS(3080),
    [anon_sym_try] = ACTIONS(3080),
    [sym_integer_literal] = ACTIONS(3078),
    [aux_sym_string_literal_token1] = ACTIONS(3078),
    [sym_char_literal] = ACTIONS(3078),
    [anon_sym_true] = ACTIONS(3080),
    [anon_sym_false] = ACTIONS(3080),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3080),
    [sym_super] = ACTIONS(3080),
    [sym_crate] = ACTIONS(3080),
    [aux_sym_metavariable_token1] = ACTIONS(3078),
    [sym__raw_string_literal_start] = ACTIONS(3078),
    [sym_float_literal] = ACTIONS(3078),
    [sym__outer_block_doc_comment_start] = ACTIONS(3078),
    [sym__outer_line_doc_comment_start] = ACTIONS(3078),
  },
  [STATE(828)] = {
    [sym_line_comment] = STATE(828),
    [sym_block_comment] = STATE(828),
    [ts_builtin_sym_end] = ACTIONS(3082),
    [sym_identifier] = ACTIONS(3084),
    [anon_sym_SEMI] = ACTIONS(3082),
    [anon_sym_macro] = ACTIONS(3084),
    [anon_sym_LPAREN] = ACTIONS(3082),
    [anon_sym_LBRACE] = ACTIONS(3082),
    [anon_sym_RBRACE] = ACTIONS(3082),
    [anon_sym_macro_rules] = ACTIONS(3084),
    [anon_sym_BANG] = ACTIONS(3082),
    [anon_sym_LBRACK] = ACTIONS(3082),
    [anon_sym_STAR] = ACTIONS(3082),
    [anon_sym_u8] = ACTIONS(3084),
    [anon_sym_i8] = ACTIONS(3084),
    [anon_sym_u16] = ACTIONS(3084),
    [anon_sym_i16] = ACTIONS(3084),
    [anon_sym_u32] = ACTIONS(3084),
    [anon_sym_i32] = ACTIONS(3084),
    [anon_sym_u64] = ACTIONS(3084),
    [anon_sym_i64] = ACTIONS(3084),
    [anon_sym_u128] = ACTIONS(3084),
    [anon_sym_i128] = ACTIONS(3084),
    [anon_sym_isize] = ACTIONS(3084),
    [anon_sym_usize] = ACTIONS(3084),
    [anon_sym_f32] = ACTIONS(3084),
    [anon_sym_f64] = ACTIONS(3084),
    [anon_sym_bool] = ACTIONS(3084),
    [anon_sym_str] = ACTIONS(3084),
    [anon_sym_char] = ACTIONS(3084),
    [anon_sym_DASH] = ACTIONS(3082),
    [anon_sym_AMP] = ACTIONS(3082),
    [anon_sym_PIPE] = ACTIONS(3082),
    [anon_sym_LT] = ACTIONS(3082),
    [anon_sym_DOT_DOT] = ACTIONS(3082),
    [anon_sym_COLON_COLON] = ACTIONS(3082),
    [anon_sym_POUND] = ACTIONS(3082),
    [anon_sym_SQUOTE] = ACTIONS(3084),
    [anon_sym_async] = ACTIONS(3084),
    [anon_sym_auto] = ACTIONS(3084),
    [anon_sym_await] = ACTIONS(3084),
    [anon_sym_become] = ACTIONS(3084),
    [anon_sym_break] = ACTIONS(3084),
    [anon_sym_const] = ACTIONS(3084),
    [anon_sym_continue] = ACTIONS(3084),
    [anon_sym_default] = ACTIONS(3084),
    [anon_sym_enum] = ACTIONS(3084),
    [anon_sym_fn] = ACTIONS(3084),
    [anon_sym_for] = ACTIONS(3084),
    [anon_sym_gen] = ACTIONS(3084),
    [anon_sym_if] = ACTIONS(3084),
    [anon_sym_impl] = ACTIONS(3084),
    [anon_sym_let] = ACTIONS(3084),
    [anon_sym_loop] = ACTIONS(3084),
    [anon_sym_match] = ACTIONS(3084),
    [anon_sym_mod] = ACTIONS(3084),
    [anon_sym_pub] = ACTIONS(3084),
    [anon_sym_return] = ACTIONS(3084),
    [anon_sym_raw] = ACTIONS(3084),
    [anon_sym_static] = ACTIONS(3084),
    [anon_sym_struct] = ACTIONS(3084),
    [anon_sym_trait] = ACTIONS(3084),
    [anon_sym_type] = ACTIONS(3084),
    [anon_sym_union] = ACTIONS(3084),
    [anon_sym_unsafe] = ACTIONS(3084),
    [anon_sym_use] = ACTIONS(3084),
    [anon_sym_while] = ACTIONS(3084),
    [anon_sym_extern] = ACTIONS(3084),
    [anon_sym_safe] = ACTIONS(3084),
    [anon_sym_yield] = ACTIONS(3084),
    [anon_sym_move] = ACTIONS(3084),
    [anon_sym_try] = ACTIONS(3084),
    [sym_integer_literal] = ACTIONS(3082),
    [aux_sym_string_literal_token1] = ACTIONS(3082),
    [sym_char_literal] = ACTIONS(3082),
    [anon_sym_true] = ACTIONS(3084),
    [anon_sym_false] = ACTIONS(3084),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3084),
    [sym_super] = ACTIONS(3084),
    [sym_crate] = ACTIONS(3084),
    [aux_sym_metavariable_token1] = ACTIONS(3082),
    [sym__raw_string_literal_start] = ACTIONS(3082),
    [sym_float_literal] = ACTIONS(3082),
    [sym__outer_block_doc_comment_start] = ACTIONS(3082),
    [sym__outer_line_doc_comment_start] = ACTIONS(3082),
  },
  [STATE(829)] = {
    [sym_line_comment] = STATE(829),
    [sym_block_comment] = STATE(829),
    [ts_builtin_sym_end] = ACTIONS(3086),
    [sym_identifier] = ACTIONS(3088),
    [anon_sym_SEMI] = ACTIONS(3086),
    [anon_sym_macro] = ACTIONS(3088),
    [anon_sym_LPAREN] = ACTIONS(3086),
    [anon_sym_LBRACE] = ACTIONS(3086),
    [anon_sym_RBRACE] = ACTIONS(3086),
    [anon_sym_macro_rules] = ACTIONS(3088),
    [anon_sym_BANG] = ACTIONS(3086),
    [anon_sym_LBRACK] = ACTIONS(3086),
    [anon_sym_STAR] = ACTIONS(3086),
    [anon_sym_u8] = ACTIONS(3088),
    [anon_sym_i8] = ACTIONS(3088),
    [anon_sym_u16] = ACTIONS(3088),
    [anon_sym_i16] = ACTIONS(3088),
    [anon_sym_u32] = ACTIONS(3088),
    [anon_sym_i32] = ACTIONS(3088),
    [anon_sym_u64] = ACTIONS(3088),
    [anon_sym_i64] = ACTIONS(3088),
    [anon_sym_u128] = ACTIONS(3088),
    [anon_sym_i128] = ACTIONS(3088),
    [anon_sym_isize] = ACTIONS(3088),
    [anon_sym_usize] = ACTIONS(3088),
    [anon_sym_f32] = ACTIONS(3088),
    [anon_sym_f64] = ACTIONS(3088),
    [anon_sym_bool] = ACTIONS(3088),
    [anon_sym_str] = ACTIONS(3088),
    [anon_sym_char] = ACTIONS(3088),
    [anon_sym_DASH] = ACTIONS(3086),
    [anon_sym_AMP] = ACTIONS(3086),
    [anon_sym_PIPE] = ACTIONS(3086),
    [anon_sym_LT] = ACTIONS(3086),
    [anon_sym_DOT_DOT] = ACTIONS(3086),
    [anon_sym_COLON_COLON] = ACTIONS(3086),
    [anon_sym_POUND] = ACTIONS(3086),
    [anon_sym_SQUOTE] = ACTIONS(3088),
    [anon_sym_async] = ACTIONS(3088),
    [anon_sym_auto] = ACTIONS(3088),
    [anon_sym_await] = ACTIONS(3088),
    [anon_sym_become] = ACTIONS(3088),
    [anon_sym_break] = ACTIONS(3088),
    [anon_sym_const] = ACTIONS(3088),
    [anon_sym_continue] = ACTIONS(3088),
    [anon_sym_default] = ACTIONS(3088),
    [anon_sym_enum] = ACTIONS(3088),
    [anon_sym_fn] = ACTIONS(3088),
    [anon_sym_for] = ACTIONS(3088),
    [anon_sym_gen] = ACTIONS(3088),
    [anon_sym_if] = ACTIONS(3088),
    [anon_sym_impl] = ACTIONS(3088),
    [anon_sym_let] = ACTIONS(3088),
    [anon_sym_loop] = ACTIONS(3088),
    [anon_sym_match] = ACTIONS(3088),
    [anon_sym_mod] = ACTIONS(3088),
    [anon_sym_pub] = ACTIONS(3088),
    [anon_sym_return] = ACTIONS(3088),
    [anon_sym_raw] = ACTIONS(3088),
    [anon_sym_static] = ACTIONS(3088),
    [anon_sym_struct] = ACTIONS(3088),
    [anon_sym_trait] = ACTIONS(3088),
    [anon_sym_type] = ACTIONS(3088),
    [anon_sym_union] = ACTIONS(3088),
    [anon_sym_unsafe] = ACTIONS(3088),
    [anon_sym_use] = ACTIONS(3088),
    [anon_sym_while] = ACTIONS(3088),
    [anon_sym_extern] = ACTIONS(3088),
    [anon_sym_safe] = ACTIONS(3088),
    [anon_sym_yield] = ACTIONS(3088),
    [anon_sym_move] = ACTIONS(3088),
    [anon_sym_try] = ACTIONS(3088),
    [sym_integer_literal] = ACTIONS(3086),
    [aux_sym_string_literal_token1] = ACTIONS(3086),
    [sym_char_literal] = ACTIONS(3086),
    [anon_sym_true] = ACTIONS(3088),
    [anon_sym_false] = ACTIONS(3088),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3088),
    [sym_super] = ACTIONS(3088),
    [sym_crate] = ACTIONS(3088),
    [aux_sym_metavariable_token1] = ACTIONS(3086),
    [sym__raw_string_literal_start] = ACTIONS(3086),
    [sym_float_literal] = ACTIONS(3086),
    [sym__outer_block_doc_comment_start] = ACTIONS(3086),
    [sym__outer_line_doc_comment_start] = ACTIONS(3086),
  },
  [STATE(830)] = {
    [sym_line_comment] = STATE(830),
    [sym_block_comment] = STATE(830),
    [ts_builtin_sym_end] = ACTIONS(3090),
    [sym_identifier] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_macro] = ACTIONS(3092),
    [anon_sym_LPAREN] = ACTIONS(3090),
    [anon_sym_LBRACE] = ACTIONS(3090),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_macro_rules] = ACTIONS(3092),
    [anon_sym_BANG] = ACTIONS(3090),
    [anon_sym_LBRACK] = ACTIONS(3090),
    [anon_sym_STAR] = ACTIONS(3090),
    [anon_sym_u8] = ACTIONS(3092),
    [anon_sym_i8] = ACTIONS(3092),
    [anon_sym_u16] = ACTIONS(3092),
    [anon_sym_i16] = ACTIONS(3092),
    [anon_sym_u32] = ACTIONS(3092),
    [anon_sym_i32] = ACTIONS(3092),
    [anon_sym_u64] = ACTIONS(3092),
    [anon_sym_i64] = ACTIONS(3092),
    [anon_sym_u128] = ACTIONS(3092),
    [anon_sym_i128] = ACTIONS(3092),
    [anon_sym_isize] = ACTIONS(3092),
    [anon_sym_usize] = ACTIONS(3092),
    [anon_sym_f32] = ACTIONS(3092),
    [anon_sym_f64] = ACTIONS(3092),
    [anon_sym_bool] = ACTIONS(3092),
    [anon_sym_str] = ACTIONS(3092),
    [anon_sym_char] = ACTIONS(3092),
    [anon_sym_DASH] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3090),
    [anon_sym_LT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3090),
    [anon_sym_COLON_COLON] = ACTIONS(3090),
    [anon_sym_POUND] = ACTIONS(3090),
    [anon_sym_SQUOTE] = ACTIONS(3092),
    [anon_sym_async] = ACTIONS(3092),
    [anon_sym_auto] = ACTIONS(3092),
    [anon_sym_await] = ACTIONS(3092),
    [anon_sym_become] = ACTIONS(3092),
    [anon_sym_break] = ACTIONS(3092),
    [anon_sym_const] = ACTIONS(3092),
    [anon_sym_continue] = ACTIONS(3092),
    [anon_sym_default] = ACTIONS(3092),
    [anon_sym_enum] = ACTIONS(3092),
    [anon_sym_fn] = ACTIONS(3092),
    [anon_sym_for] = ACTIONS(3092),
    [anon_sym_gen] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3092),
    [anon_sym_impl] = ACTIONS(3092),
    [anon_sym_let] = ACTIONS(3092),
    [anon_sym_loop] = ACTIONS(3092),
    [anon_sym_match] = ACTIONS(3092),
    [anon_sym_mod] = ACTIONS(3092),
    [anon_sym_pub] = ACTIONS(3092),
    [anon_sym_return] = ACTIONS(3092),
    [anon_sym_raw] = ACTIONS(3092),
    [anon_sym_static] = ACTIONS(3092),
    [anon_sym_struct] = ACTIONS(3092),
    [anon_sym_trait] = ACTIONS(3092),
    [anon_sym_type] = ACTIONS(3092),
    [anon_sym_union] = ACTIONS(3092),
    [anon_sym_unsafe] = ACTIONS(3092),
    [anon_sym_use] = ACTIONS(3092),
    [anon_sym_while] = ACTIONS(3092),
    [anon_sym_extern] = ACTIONS(3092),
    [anon_sym_safe] = ACTIONS(3092),
    [anon_sym_yield] = ACTIONS(3092),
    [anon_sym_move] = ACTIONS(3092),
    [anon_sym_try] = ACTIONS(3092),
    [sym_integer_literal] = ACTIONS(3090),
    [aux_sym_string_literal_token1] = ACTIONS(3090),
    [sym_char_literal] = ACTIONS(3090),
    [anon_sym_true] = ACTIONS(3092),
    [anon_sym_false] = ACTIONS(3092),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3092),
    [sym_super] = ACTIONS(3092),
    [sym_crate] = ACTIONS(3092),
    [aux_sym_metavariable_token1] = ACTIONS(3090),
    [sym__raw_string_literal_start] = ACTIONS(3090),
    [sym_float_literal] = ACTIONS(3090),
    [sym__outer_block_doc_comment_start] = ACTIONS(3090),
    [sym__outer_line_doc_comment_start] = ACTIONS(3090),
  },
  [STATE(831)] = {
    [sym_line_comment] = STATE(831),
    [sym_block_comment] = STATE(831),
    [ts_builtin_sym_end] = ACTIONS(3094),
    [sym_identifier] = ACTIONS(3096),
    [anon_sym_SEMI] = ACTIONS(3094),
    [anon_sym_macro] = ACTIONS(3096),
    [anon_sym_LPAREN] = ACTIONS(3094),
    [anon_sym_LBRACE] = ACTIONS(3094),
    [anon_sym_RBRACE] = ACTIONS(3094),
    [anon_sym_macro_rules] = ACTIONS(3096),
    [anon_sym_BANG] = ACTIONS(3094),
    [anon_sym_LBRACK] = ACTIONS(3094),
    [anon_sym_STAR] = ACTIONS(3094),
    [anon_sym_u8] = ACTIONS(3096),
    [anon_sym_i8] = ACTIONS(3096),
    [anon_sym_u16] = ACTIONS(3096),
    [anon_sym_i16] = ACTIONS(3096),
    [anon_sym_u32] = ACTIONS(3096),
    [anon_sym_i32] = ACTIONS(3096),
    [anon_sym_u64] = ACTIONS(3096),
    [anon_sym_i64] = ACTIONS(3096),
    [anon_sym_u128] = ACTIONS(3096),
    [anon_sym_i128] = ACTIONS(3096),
    [anon_sym_isize] = ACTIONS(3096),
    [anon_sym_usize] = ACTIONS(3096),
    [anon_sym_f32] = ACTIONS(3096),
    [anon_sym_f64] = ACTIONS(3096),
    [anon_sym_bool] = ACTIONS(3096),
    [anon_sym_str] = ACTIONS(3096),
    [anon_sym_char] = ACTIONS(3096),
    [anon_sym_DASH] = ACTIONS(3094),
    [anon_sym_AMP] = ACTIONS(3094),
    [anon_sym_PIPE] = ACTIONS(3094),
    [anon_sym_LT] = ACTIONS(3094),
    [anon_sym_DOT_DOT] = ACTIONS(3094),
    [anon_sym_COLON_COLON] = ACTIONS(3094),
    [anon_sym_POUND] = ACTIONS(3094),
    [anon_sym_SQUOTE] = ACTIONS(3096),
    [anon_sym_async] = ACTIONS(3096),
    [anon_sym_auto] = ACTIONS(3096),
    [anon_sym_await] = ACTIONS(3096),
    [anon_sym_become] = ACTIONS(3096),
    [anon_sym_break] = ACTIONS(3096),
    [anon_sym_const] = ACTIONS(3096),
    [anon_sym_continue] = ACTIONS(3096),
    [anon_sym_default] = ACTIONS(3096),
    [anon_sym_enum] = ACTIONS(3096),
    [anon_sym_fn] = ACTIONS(3096),
    [anon_sym_for] = ACTIONS(3096),
    [anon_sym_gen] = ACTIONS(3096),
    [anon_sym_if] = ACTIONS(3096),
    [anon_sym_impl] = ACTIONS(3096),
    [anon_sym_let] = ACTIONS(3096),
    [anon_sym_loop] = ACTIONS(3096),
    [anon_sym_match] = ACTIONS(3096),
    [anon_sym_mod] = ACTIONS(3096),
    [anon_sym_pub] = ACTIONS(3096),
    [anon_sym_return] = ACTIONS(3096),
    [anon_sym_raw] = ACTIONS(3096),
    [anon_sym_static] = ACTIONS(3096),
    [anon_sym_struct] = ACTIONS(3096),
    [anon_sym_trait] = ACTIONS(3096),
    [anon_sym_type] = ACTIONS(3096),
    [anon_sym_union] = ACTIONS(3096),
    [anon_sym_unsafe] = ACTIONS(3096),
    [anon_sym_use] = ACTIONS(3096),
    [anon_sym_while] = ACTIONS(3096),
    [anon_sym_extern] = ACTIONS(3096),
    [anon_sym_safe] = ACTIONS(3096),
    [anon_sym_yield] = ACTIONS(3096),
    [anon_sym_move] = ACTIONS(3096),
    [anon_sym_try] = ACTIONS(3096),
    [sym_integer_literal] = ACTIONS(3094),
    [aux_sym_string_literal_token1] = ACTIONS(3094),
    [sym_char_literal] = ACTIONS(3094),
    [anon_sym_true] = ACTIONS(3096),
    [anon_sym_false] = ACTIONS(3096),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3096),
    [sym_super] = ACTIONS(3096),
    [sym_crate] = ACTIONS(3096),
    [aux_sym_metavariable_token1] = ACTIONS(3094),
    [sym__raw_string_literal_start] = ACTIONS(3094),
    [sym_float_literal] = ACTIONS(3094),
    [sym__outer_block_doc_comment_start] = ACTIONS(3094),
    [sym__outer_line_doc_comment_start] = ACTIONS(3094),
  },
  [STATE(832)] = {
    [sym_line_comment] = STATE(832),
    [sym_block_comment] = STATE(832),
    [ts_builtin_sym_end] = ACTIONS(3098),
    [sym_identifier] = ACTIONS(3100),
    [anon_sym_SEMI] = ACTIONS(3098),
    [anon_sym_macro] = ACTIONS(3100),
    [anon_sym_LPAREN] = ACTIONS(3098),
    [anon_sym_LBRACE] = ACTIONS(3098),
    [anon_sym_RBRACE] = ACTIONS(3098),
    [anon_sym_macro_rules] = ACTIONS(3100),
    [anon_sym_BANG] = ACTIONS(3098),
    [anon_sym_LBRACK] = ACTIONS(3098),
    [anon_sym_STAR] = ACTIONS(3098),
    [anon_sym_u8] = ACTIONS(3100),
    [anon_sym_i8] = ACTIONS(3100),
    [anon_sym_u16] = ACTIONS(3100),
    [anon_sym_i16] = ACTIONS(3100),
    [anon_sym_u32] = ACTIONS(3100),
    [anon_sym_i32] = ACTIONS(3100),
    [anon_sym_u64] = ACTIONS(3100),
    [anon_sym_i64] = ACTIONS(3100),
    [anon_sym_u128] = ACTIONS(3100),
    [anon_sym_i128] = ACTIONS(3100),
    [anon_sym_isize] = ACTIONS(3100),
    [anon_sym_usize] = ACTIONS(3100),
    [anon_sym_f32] = ACTIONS(3100),
    [anon_sym_f64] = ACTIONS(3100),
    [anon_sym_bool] = ACTIONS(3100),
    [anon_sym_str] = ACTIONS(3100),
    [anon_sym_char] = ACTIONS(3100),
    [anon_sym_DASH] = ACTIONS(3098),
    [anon_sym_AMP] = ACTIONS(3098),
    [anon_sym_PIPE] = ACTIONS(3098),
    [anon_sym_LT] = ACTIONS(3098),
    [anon_sym_DOT_DOT] = ACTIONS(3098),
    [anon_sym_COLON_COLON] = ACTIONS(3098),
    [anon_sym_POUND] = ACTIONS(3098),
    [anon_sym_SQUOTE] = ACTIONS(3100),
    [anon_sym_async] = ACTIONS(3100),
    [anon_sym_auto] = ACTIONS(3100),
    [anon_sym_await] = ACTIONS(3100),
    [anon_sym_become] = ACTIONS(3100),
    [anon_sym_break] = ACTIONS(3100),
    [anon_sym_const] = ACTIONS(3100),
    [anon_sym_continue] = ACTIONS(3100),
    [anon_sym_default] = ACTIONS(3100),
    [anon_sym_enum] = ACTIONS(3100),
    [anon_sym_fn] = ACTIONS(3100),
    [anon_sym_for] = ACTIONS(3100),
    [anon_sym_gen] = ACTIONS(3100),
    [anon_sym_if] = ACTIONS(3100),
    [anon_sym_impl] = ACTIONS(3100),
    [anon_sym_let] = ACTIONS(3100),
    [anon_sym_loop] = ACTIONS(3100),
    [anon_sym_match] = ACTIONS(3100),
    [anon_sym_mod] = ACTIONS(3100),
    [anon_sym_pub] = ACTIONS(3100),
    [anon_sym_return] = ACTIONS(3100),
    [anon_sym_raw] = ACTIONS(3100),
    [anon_sym_static] = ACTIONS(3100),
    [anon_sym_struct] = ACTIONS(3100),
    [anon_sym_trait] = ACTIONS(3100),
    [anon_sym_type] = ACTIONS(3100),
    [anon_sym_union] = ACTIONS(3100),
    [anon_sym_unsafe] = ACTIONS(3100),
    [anon_sym_use] = ACTIONS(3100),
    [anon_sym_while] = ACTIONS(3100),
    [anon_sym_extern] = ACTIONS(3100),
    [anon_sym_safe] = ACTIONS(3100),
    [anon_sym_yield] = ACTIONS(3100),
    [anon_sym_move] = ACTIONS(3100),
    [anon_sym_try] = ACTIONS(3100),
    [sym_integer_literal] = ACTIONS(3098),
    [aux_sym_string_literal_token1] = ACTIONS(3098),
    [sym_char_literal] = ACTIONS(3098),
    [anon_sym_true] = ACTIONS(3100),
    [anon_sym_false] = ACTIONS(3100),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3100),
    [sym_super] = ACTIONS(3100),
    [sym_crate] = ACTIONS(3100),
    [aux_sym_metavariable_token1] = ACTIONS(3098),
    [sym__raw_string_literal_start] = ACTIONS(3098),
    [sym_float_literal] = ACTIONS(3098),
    [sym__outer_block_doc_comment_start] = ACTIONS(3098),
    [sym__outer_line_doc_comment_start] = ACTIONS(3098),
  },
  [STATE(833)] = {
    [sym_line_comment] = STATE(833),
    [sym_block_comment] = STATE(833),
    [ts_builtin_sym_end] = ACTIONS(3102),
    [sym_identifier] = ACTIONS(3104),
    [anon_sym_SEMI] = ACTIONS(3102),
    [anon_sym_macro] = ACTIONS(3104),
    [anon_sym_LPAREN] = ACTIONS(3102),
    [anon_sym_LBRACE] = ACTIONS(3102),
    [anon_sym_RBRACE] = ACTIONS(3102),
    [anon_sym_macro_rules] = ACTIONS(3104),
    [anon_sym_BANG] = ACTIONS(3102),
    [anon_sym_LBRACK] = ACTIONS(3102),
    [anon_sym_STAR] = ACTIONS(3102),
    [anon_sym_u8] = ACTIONS(3104),
    [anon_sym_i8] = ACTIONS(3104),
    [anon_sym_u16] = ACTIONS(3104),
    [anon_sym_i16] = ACTIONS(3104),
    [anon_sym_u32] = ACTIONS(3104),
    [anon_sym_i32] = ACTIONS(3104),
    [anon_sym_u64] = ACTIONS(3104),
    [anon_sym_i64] = ACTIONS(3104),
    [anon_sym_u128] = ACTIONS(3104),
    [anon_sym_i128] = ACTIONS(3104),
    [anon_sym_isize] = ACTIONS(3104),
    [anon_sym_usize] = ACTIONS(3104),
    [anon_sym_f32] = ACTIONS(3104),
    [anon_sym_f64] = ACTIONS(3104),
    [anon_sym_bool] = ACTIONS(3104),
    [anon_sym_str] = ACTIONS(3104),
    [anon_sym_char] = ACTIONS(3104),
    [anon_sym_DASH] = ACTIONS(3102),
    [anon_sym_AMP] = ACTIONS(3102),
    [anon_sym_PIPE] = ACTIONS(3102),
    [anon_sym_LT] = ACTIONS(3102),
    [anon_sym_DOT_DOT] = ACTIONS(3102),
    [anon_sym_COLON_COLON] = ACTIONS(3102),
    [anon_sym_POUND] = ACTIONS(3102),
    [anon_sym_SQUOTE] = ACTIONS(3104),
    [anon_sym_async] = ACTIONS(3104),
    [anon_sym_auto] = ACTIONS(3104),
    [anon_sym_await] = ACTIONS(3104),
    [anon_sym_become] = ACTIONS(3104),
    [anon_sym_break] = ACTIONS(3104),
    [anon_sym_const] = ACTIONS(3104),
    [anon_sym_continue] = ACTIONS(3104),
    [anon_sym_default] = ACTIONS(3104),
    [anon_sym_enum] = ACTIONS(3104),
    [anon_sym_fn] = ACTIONS(3104),
    [anon_sym_for] = ACTIONS(3104),
    [anon_sym_gen] = ACTIONS(3104),
    [anon_sym_if] = ACTIONS(3104),
    [anon_sym_impl] = ACTIONS(3104),
    [anon_sym_let] = ACTIONS(3104),
    [anon_sym_loop] = ACTIONS(3104),
    [anon_sym_match] = ACTIONS(3104),
    [anon_sym_mod] = ACTIONS(3104),
    [anon_sym_pub] = ACTIONS(3104),
    [anon_sym_return] = ACTIONS(3104),
    [anon_sym_raw] = ACTIONS(3104),
    [anon_sym_static] = ACTIONS(3104),
    [anon_sym_struct] = ACTIONS(3104),
    [anon_sym_trait] = ACTIONS(3104),
    [anon_sym_type] = ACTIONS(3104),
    [anon_sym_union] = ACTIONS(3104),
    [anon_sym_unsafe] = ACTIONS(3104),
    [anon_sym_use] = ACTIONS(3104),
    [anon_sym_while] = ACTIONS(3104),
    [anon_sym_extern] = ACTIONS(3104),
    [anon_sym_safe] = ACTIONS(3104),
    [anon_sym_yield] = ACTIONS(3104),
    [anon_sym_move] = ACTIONS(3104),
    [anon_sym_try] = ACTIONS(3104),
    [sym_integer_literal] = ACTIONS(3102),
    [aux_sym_string_literal_token1] = ACTIONS(3102),
    [sym_char_literal] = ACTIONS(3102),
    [anon_sym_true] = ACTIONS(3104),
    [anon_sym_false] = ACTIONS(3104),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3104),
    [sym_super] = ACTIONS(3104),
    [sym_crate] = ACTIONS(3104),
    [aux_sym_metavariable_token1] = ACTIONS(3102),
    [sym__raw_string_literal_start] = ACTIONS(3102),
    [sym_float_literal] = ACTIONS(3102),
    [sym__outer_block_doc_comment_start] = ACTIONS(3102),
    [sym__outer_line_doc_comment_start] = ACTIONS(3102),
  },
  [STATE(834)] = {
    [sym_line_comment] = STATE(834),
    [sym_block_comment] = STATE(834),
    [ts_builtin_sym_end] = ACTIONS(3106),
    [sym_identifier] = ACTIONS(3108),
    [anon_sym_SEMI] = ACTIONS(3106),
    [anon_sym_macro] = ACTIONS(3108),
    [anon_sym_LPAREN] = ACTIONS(3106),
    [anon_sym_LBRACE] = ACTIONS(3106),
    [anon_sym_RBRACE] = ACTIONS(3106),
    [anon_sym_macro_rules] = ACTIONS(3108),
    [anon_sym_BANG] = ACTIONS(3106),
    [anon_sym_LBRACK] = ACTIONS(3106),
    [anon_sym_STAR] = ACTIONS(3106),
    [anon_sym_u8] = ACTIONS(3108),
    [anon_sym_i8] = ACTIONS(3108),
    [anon_sym_u16] = ACTIONS(3108),
    [anon_sym_i16] = ACTIONS(3108),
    [anon_sym_u32] = ACTIONS(3108),
    [anon_sym_i32] = ACTIONS(3108),
    [anon_sym_u64] = ACTIONS(3108),
    [anon_sym_i64] = ACTIONS(3108),
    [anon_sym_u128] = ACTIONS(3108),
    [anon_sym_i128] = ACTIONS(3108),
    [anon_sym_isize] = ACTIONS(3108),
    [anon_sym_usize] = ACTIONS(3108),
    [anon_sym_f32] = ACTIONS(3108),
    [anon_sym_f64] = ACTIONS(3108),
    [anon_sym_bool] = ACTIONS(3108),
    [anon_sym_str] = ACTIONS(3108),
    [anon_sym_char] = ACTIONS(3108),
    [anon_sym_DASH] = ACTIONS(3106),
    [anon_sym_AMP] = ACTIONS(3106),
    [anon_sym_PIPE] = ACTIONS(3106),
    [anon_sym_LT] = ACTIONS(3106),
    [anon_sym_DOT_DOT] = ACTIONS(3106),
    [anon_sym_COLON_COLON] = ACTIONS(3106),
    [anon_sym_POUND] = ACTIONS(3106),
    [anon_sym_SQUOTE] = ACTIONS(3108),
    [anon_sym_async] = ACTIONS(3108),
    [anon_sym_auto] = ACTIONS(3108),
    [anon_sym_await] = ACTIONS(3108),
    [anon_sym_become] = ACTIONS(3108),
    [anon_sym_break] = ACTIONS(3108),
    [anon_sym_const] = ACTIONS(3108),
    [anon_sym_continue] = ACTIONS(3108),
    [anon_sym_default] = ACTIONS(3108),
    [anon_sym_enum] = ACTIONS(3108),
    [anon_sym_fn] = ACTIONS(3108),
    [anon_sym_for] = ACTIONS(3108),
    [anon_sym_gen] = ACTIONS(3108),
    [anon_sym_if] = ACTIONS(3108),
    [anon_sym_impl] = ACTIONS(3108),
    [anon_sym_let] = ACTIONS(3108),
    [anon_sym_loop] = ACTIONS(3108),
    [anon_sym_match] = ACTIONS(3108),
    [anon_sym_mod] = ACTIONS(3108),
    [anon_sym_pub] = ACTIONS(3108),
    [anon_sym_return] = ACTIONS(3108),
    [anon_sym_raw] = ACTIONS(3108),
    [anon_sym_static] = ACTIONS(3108),
    [anon_sym_struct] = ACTIONS(3108),
    [anon_sym_trait] = ACTIONS(3108),
    [anon_sym_type] = ACTIONS(3108),
    [anon_sym_union] = ACTIONS(3108),
    [anon_sym_unsafe] = ACTIONS(3108),
    [anon_sym_use] = ACTIONS(3108),
    [anon_sym_while] = ACTIONS(3108),
    [anon_sym_extern] = ACTIONS(3108),
    [anon_sym_safe] = ACTIONS(3108),
    [anon_sym_yield] = ACTIONS(3108),
    [anon_sym_move] = ACTIONS(3108),
    [anon_sym_try] = ACTIONS(3108),
    [sym_integer_literal] = ACTIONS(3106),
    [aux_sym_string_literal_token1] = ACTIONS(3106),
    [sym_char_literal] = ACTIONS(3106),
    [anon_sym_true] = ACTIONS(3108),
    [anon_sym_false] = ACTIONS(3108),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3108),
    [sym_super] = ACTIONS(3108),
    [sym_crate] = ACTIONS(3108),
    [aux_sym_metavariable_token1] = ACTIONS(3106),
    [sym__raw_string_literal_start] = ACTIONS(3106),
    [sym_float_literal] = ACTIONS(3106),
    [sym__outer_block_doc_comment_start] = ACTIONS(3106),
    [sym__outer_line_doc_comment_start] = ACTIONS(3106),
  },
  [STATE(835)] = {
    [sym_line_comment] = STATE(835),
    [sym_block_comment] = STATE(835),
    [ts_builtin_sym_end] = ACTIONS(3110),
    [sym_identifier] = ACTIONS(3112),
    [anon_sym_SEMI] = ACTIONS(3110),
    [anon_sym_macro] = ACTIONS(3112),
    [anon_sym_LPAREN] = ACTIONS(3110),
    [anon_sym_LBRACE] = ACTIONS(3110),
    [anon_sym_RBRACE] = ACTIONS(3110),
    [anon_sym_macro_rules] = ACTIONS(3112),
    [anon_sym_BANG] = ACTIONS(3110),
    [anon_sym_LBRACK] = ACTIONS(3110),
    [anon_sym_STAR] = ACTIONS(3110),
    [anon_sym_u8] = ACTIONS(3112),
    [anon_sym_i8] = ACTIONS(3112),
    [anon_sym_u16] = ACTIONS(3112),
    [anon_sym_i16] = ACTIONS(3112),
    [anon_sym_u32] = ACTIONS(3112),
    [anon_sym_i32] = ACTIONS(3112),
    [anon_sym_u64] = ACTIONS(3112),
    [anon_sym_i64] = ACTIONS(3112),
    [anon_sym_u128] = ACTIONS(3112),
    [anon_sym_i128] = ACTIONS(3112),
    [anon_sym_isize] = ACTIONS(3112),
    [anon_sym_usize] = ACTIONS(3112),
    [anon_sym_f32] = ACTIONS(3112),
    [anon_sym_f64] = ACTIONS(3112),
    [anon_sym_bool] = ACTIONS(3112),
    [anon_sym_str] = ACTIONS(3112),
    [anon_sym_char] = ACTIONS(3112),
    [anon_sym_DASH] = ACTIONS(3110),
    [anon_sym_AMP] = ACTIONS(3110),
    [anon_sym_PIPE] = ACTIONS(3110),
    [anon_sym_LT] = ACTIONS(3110),
    [anon_sym_DOT_DOT] = ACTIONS(3110),
    [anon_sym_COLON_COLON] = ACTIONS(3110),
    [anon_sym_POUND] = ACTIONS(3110),
    [anon_sym_SQUOTE] = ACTIONS(3112),
    [anon_sym_async] = ACTIONS(3112),
    [anon_sym_auto] = ACTIONS(3112),
    [anon_sym_await] = ACTIONS(3112),
    [anon_sym_become] = ACTIONS(3112),
    [anon_sym_break] = ACTIONS(3112),
    [anon_sym_const] = ACTIONS(3112),
    [anon_sym_continue] = ACTIONS(3112),
    [anon_sym_default] = ACTIONS(3112),
    [anon_sym_enum] = ACTIONS(3112),
    [anon_sym_fn] = ACTIONS(3112),
    [anon_sym_for] = ACTIONS(3112),
    [anon_sym_gen] = ACTIONS(3112),
    [anon_sym_if] = ACTIONS(3112),
    [anon_sym_impl] = ACTIONS(3112),
    [anon_sym_let] = ACTIONS(3112),
    [anon_sym_loop] = ACTIONS(3112),
    [anon_sym_match] = ACTIONS(3112),
    [anon_sym_mod] = ACTIONS(3112),
    [anon_sym_pub] = ACTIONS(3112),
    [anon_sym_return] = ACTIONS(3112),
    [anon_sym_raw] = ACTIONS(3112),
    [anon_sym_static] = ACTIONS(3112),
    [anon_sym_struct] = ACTIONS(3112),
    [anon_sym_trait] = ACTIONS(3112),
    [anon_sym_type] = ACTIONS(3112),
    [anon_sym_union] = ACTIONS(3112),
    [anon_sym_unsafe] = ACTIONS(3112),
    [anon_sym_use] = ACTIONS(3112),
    [anon_sym_while] = ACTIONS(3112),
    [anon_sym_extern] = ACTIONS(3112),
    [anon_sym_safe] = ACTIONS(3112),
    [anon_sym_yield] = ACTIONS(3112),
    [anon_sym_move] = ACTIONS(3112),
    [anon_sym_try] = ACTIONS(3112),
    [sym_integer_literal] = ACTIONS(3110),
    [aux_sym_string_literal_token1] = ACTIONS(3110),
    [sym_char_literal] = ACTIONS(3110),
    [anon_sym_true] = ACTIONS(3112),
    [anon_sym_false] = ACTIONS(3112),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3112),
    [sym_super] = ACTIONS(3112),
    [sym_crate] = ACTIONS(3112),
    [aux_sym_metavariable_token1] = ACTIONS(3110),
    [sym__raw_string_literal_start] = ACTIONS(3110),
    [sym_float_literal] = ACTIONS(3110),
    [sym__outer_block_doc_comment_start] = ACTIONS(3110),
    [sym__outer_line_doc_comment_start] = ACTIONS(3110),
  },
  [STATE(836)] = {
    [sym_line_comment] = STATE(836),
    [sym_block_comment] = STATE(836),
    [ts_builtin_sym_end] = ACTIONS(3114),
    [sym_identifier] = ACTIONS(3116),
    [anon_sym_SEMI] = ACTIONS(3114),
    [anon_sym_macro] = ACTIONS(3116),
    [anon_sym_LPAREN] = ACTIONS(3114),
    [anon_sym_LBRACE] = ACTIONS(3114),
    [anon_sym_RBRACE] = ACTIONS(3114),
    [anon_sym_macro_rules] = ACTIONS(3116),
    [anon_sym_BANG] = ACTIONS(3114),
    [anon_sym_LBRACK] = ACTIONS(3114),
    [anon_sym_STAR] = ACTIONS(3114),
    [anon_sym_u8] = ACTIONS(3116),
    [anon_sym_i8] = ACTIONS(3116),
    [anon_sym_u16] = ACTIONS(3116),
    [anon_sym_i16] = ACTIONS(3116),
    [anon_sym_u32] = ACTIONS(3116),
    [anon_sym_i32] = ACTIONS(3116),
    [anon_sym_u64] = ACTIONS(3116),
    [anon_sym_i64] = ACTIONS(3116),
    [anon_sym_u128] = ACTIONS(3116),
    [anon_sym_i128] = ACTIONS(3116),
    [anon_sym_isize] = ACTIONS(3116),
    [anon_sym_usize] = ACTIONS(3116),
    [anon_sym_f32] = ACTIONS(3116),
    [anon_sym_f64] = ACTIONS(3116),
    [anon_sym_bool] = ACTIONS(3116),
    [anon_sym_str] = ACTIONS(3116),
    [anon_sym_char] = ACTIONS(3116),
    [anon_sym_DASH] = ACTIONS(3114),
    [anon_sym_AMP] = ACTIONS(3114),
    [anon_sym_PIPE] = ACTIONS(3114),
    [anon_sym_LT] = ACTIONS(3114),
    [anon_sym_DOT_DOT] = ACTIONS(3114),
    [anon_sym_COLON_COLON] = ACTIONS(3114),
    [anon_sym_POUND] = ACTIONS(3114),
    [anon_sym_SQUOTE] = ACTIONS(3116),
    [anon_sym_async] = ACTIONS(3116),
    [anon_sym_auto] = ACTIONS(3116),
    [anon_sym_await] = ACTIONS(3116),
    [anon_sym_become] = ACTIONS(3116),
    [anon_sym_break] = ACTIONS(3116),
    [anon_sym_const] = ACTIONS(3116),
    [anon_sym_continue] = ACTIONS(3116),
    [anon_sym_default] = ACTIONS(3116),
    [anon_sym_enum] = ACTIONS(3116),
    [anon_sym_fn] = ACTIONS(3116),
    [anon_sym_for] = ACTIONS(3116),
    [anon_sym_gen] = ACTIONS(3116),
    [anon_sym_if] = ACTIONS(3116),
    [anon_sym_impl] = ACTIONS(3116),
    [anon_sym_let] = ACTIONS(3116),
    [anon_sym_loop] = ACTIONS(3116),
    [anon_sym_match] = ACTIONS(3116),
    [anon_sym_mod] = ACTIONS(3116),
    [anon_sym_pub] = ACTIONS(3116),
    [anon_sym_return] = ACTIONS(3116),
    [anon_sym_raw] = ACTIONS(3116),
    [anon_sym_static] = ACTIONS(3116),
    [anon_sym_struct] = ACTIONS(3116),
    [anon_sym_trait] = ACTIONS(3116),
    [anon_sym_type] = ACTIONS(3116),
    [anon_sym_union] = ACTIONS(3116),
    [anon_sym_unsafe] = ACTIONS(3116),
    [anon_sym_use] = ACTIONS(3116),
    [anon_sym_while] = ACTIONS(3116),
    [anon_sym_extern] = ACTIONS(3116),
    [anon_sym_safe] = ACTIONS(3116),
    [anon_sym_yield] = ACTIONS(3116),
    [anon_sym_move] = ACTIONS(3116),
    [anon_sym_try] = ACTIONS(3116),
    [sym_integer_literal] = ACTIONS(3114),
    [aux_sym_string_literal_token1] = ACTIONS(3114),
    [sym_char_literal] = ACTIONS(3114),
    [anon_sym_true] = ACTIONS(3116),
    [anon_sym_false] = ACTIONS(3116),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3116),
    [sym_super] = ACTIONS(3116),
    [sym_crate] = ACTIONS(3116),
    [aux_sym_metavariable_token1] = ACTIONS(3114),
    [sym__raw_string_literal_start] = ACTIONS(3114),
    [sym_float_literal] = ACTIONS(3114),
    [sym__outer_block_doc_comment_start] = ACTIONS(3114),
    [sym__outer_line_doc_comment_start] = ACTIONS(3114),
  },
  [STATE(837)] = {
    [sym_line_comment] = STATE(837),
    [sym_block_comment] = STATE(837),
    [ts_builtin_sym_end] = ACTIONS(3118),
    [sym_identifier] = ACTIONS(3120),
    [anon_sym_SEMI] = ACTIONS(3118),
    [anon_sym_macro] = ACTIONS(3120),
    [anon_sym_LPAREN] = ACTIONS(3118),
    [anon_sym_LBRACE] = ACTIONS(3118),
    [anon_sym_RBRACE] = ACTIONS(3118),
    [anon_sym_macro_rules] = ACTIONS(3120),
    [anon_sym_BANG] = ACTIONS(3118),
    [anon_sym_LBRACK] = ACTIONS(3118),
    [anon_sym_STAR] = ACTIONS(3118),
    [anon_sym_u8] = ACTIONS(3120),
    [anon_sym_i8] = ACTIONS(3120),
    [anon_sym_u16] = ACTIONS(3120),
    [anon_sym_i16] = ACTIONS(3120),
    [anon_sym_u32] = ACTIONS(3120),
    [anon_sym_i32] = ACTIONS(3120),
    [anon_sym_u64] = ACTIONS(3120),
    [anon_sym_i64] = ACTIONS(3120),
    [anon_sym_u128] = ACTIONS(3120),
    [anon_sym_i128] = ACTIONS(3120),
    [anon_sym_isize] = ACTIONS(3120),
    [anon_sym_usize] = ACTIONS(3120),
    [anon_sym_f32] = ACTIONS(3120),
    [anon_sym_f64] = ACTIONS(3120),
    [anon_sym_bool] = ACTIONS(3120),
    [anon_sym_str] = ACTIONS(3120),
    [anon_sym_char] = ACTIONS(3120),
    [anon_sym_DASH] = ACTIONS(3118),
    [anon_sym_AMP] = ACTIONS(3118),
    [anon_sym_PIPE] = ACTIONS(3118),
    [anon_sym_LT] = ACTIONS(3118),
    [anon_sym_DOT_DOT] = ACTIONS(3118),
    [anon_sym_COLON_COLON] = ACTIONS(3118),
    [anon_sym_POUND] = ACTIONS(3118),
    [anon_sym_SQUOTE] = ACTIONS(3120),
    [anon_sym_async] = ACTIONS(3120),
    [anon_sym_auto] = ACTIONS(3120),
    [anon_sym_await] = ACTIONS(3120),
    [anon_sym_become] = ACTIONS(3120),
    [anon_sym_break] = ACTIONS(3120),
    [anon_sym_const] = ACTIONS(3120),
    [anon_sym_continue] = ACTIONS(3120),
    [anon_sym_default] = ACTIONS(3120),
    [anon_sym_enum] = ACTIONS(3120),
    [anon_sym_fn] = ACTIONS(3120),
    [anon_sym_for] = ACTIONS(3120),
    [anon_sym_gen] = ACTIONS(3120),
    [anon_sym_if] = ACTIONS(3120),
    [anon_sym_impl] = ACTIONS(3120),
    [anon_sym_let] = ACTIONS(3120),
    [anon_sym_loop] = ACTIONS(3120),
    [anon_sym_match] = ACTIONS(3120),
    [anon_sym_mod] = ACTIONS(3120),
    [anon_sym_pub] = ACTIONS(3120),
    [anon_sym_return] = ACTIONS(3120),
    [anon_sym_raw] = ACTIONS(3120),
    [anon_sym_static] = ACTIONS(3120),
    [anon_sym_struct] = ACTIONS(3120),
    [anon_sym_trait] = ACTIONS(3120),
    [anon_sym_type] = ACTIONS(3120),
    [anon_sym_union] = ACTIONS(3120),
    [anon_sym_unsafe] = ACTIONS(3120),
    [anon_sym_use] = ACTIONS(3120),
    [anon_sym_while] = ACTIONS(3120),
    [anon_sym_extern] = ACTIONS(3120),
    [anon_sym_safe] = ACTIONS(3120),
    [anon_sym_yield] = ACTIONS(3120),
    [anon_sym_move] = ACTIONS(3120),
    [anon_sym_try] = ACTIONS(3120),
    [sym_integer_literal] = ACTIONS(3118),
    [aux_sym_string_literal_token1] = ACTIONS(3118),
    [sym_char_literal] = ACTIONS(3118),
    [anon_sym_true] = ACTIONS(3120),
    [anon_sym_false] = ACTIONS(3120),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3120),
    [sym_super] = ACTIONS(3120),
    [sym_crate] = ACTIONS(3120),
    [aux_sym_metavariable_token1] = ACTIONS(3118),
    [sym__raw_string_literal_start] = ACTIONS(3118),
    [sym_float_literal] = ACTIONS(3118),
    [sym__outer_block_doc_comment_start] = ACTIONS(3118),
    [sym__outer_line_doc_comment_start] = ACTIONS(3118),
  },
  [STATE(838)] = {
    [sym_line_comment] = STATE(838),
    [sym_block_comment] = STATE(838),
    [ts_builtin_sym_end] = ACTIONS(3122),
    [sym_identifier] = ACTIONS(3124),
    [anon_sym_SEMI] = ACTIONS(3122),
    [anon_sym_macro] = ACTIONS(3124),
    [anon_sym_LPAREN] = ACTIONS(3122),
    [anon_sym_LBRACE] = ACTIONS(3122),
    [anon_sym_RBRACE] = ACTIONS(3122),
    [anon_sym_macro_rules] = ACTIONS(3124),
    [anon_sym_BANG] = ACTIONS(3122),
    [anon_sym_LBRACK] = ACTIONS(3122),
    [anon_sym_STAR] = ACTIONS(3122),
    [anon_sym_u8] = ACTIONS(3124),
    [anon_sym_i8] = ACTIONS(3124),
    [anon_sym_u16] = ACTIONS(3124),
    [anon_sym_i16] = ACTIONS(3124),
    [anon_sym_u32] = ACTIONS(3124),
    [anon_sym_i32] = ACTIONS(3124),
    [anon_sym_u64] = ACTIONS(3124),
    [anon_sym_i64] = ACTIONS(3124),
    [anon_sym_u128] = ACTIONS(3124),
    [anon_sym_i128] = ACTIONS(3124),
    [anon_sym_isize] = ACTIONS(3124),
    [anon_sym_usize] = ACTIONS(3124),
    [anon_sym_f32] = ACTIONS(3124),
    [anon_sym_f64] = ACTIONS(3124),
    [anon_sym_bool] = ACTIONS(3124),
    [anon_sym_str] = ACTIONS(3124),
    [anon_sym_char] = ACTIONS(3124),
    [anon_sym_DASH] = ACTIONS(3122),
    [anon_sym_AMP] = ACTIONS(3122),
    [anon_sym_PIPE] = ACTIONS(3122),
    [anon_sym_LT] = ACTIONS(3122),
    [anon_sym_DOT_DOT] = ACTIONS(3122),
    [anon_sym_COLON_COLON] = ACTIONS(3122),
    [anon_sym_POUND] = ACTIONS(3122),
    [anon_sym_SQUOTE] = ACTIONS(3124),
    [anon_sym_async] = ACTIONS(3124),
    [anon_sym_auto] = ACTIONS(3124),
    [anon_sym_await] = ACTIONS(3124),
    [anon_sym_become] = ACTIONS(3124),
    [anon_sym_break] = ACTIONS(3124),
    [anon_sym_const] = ACTIONS(3124),
    [anon_sym_continue] = ACTIONS(3124),
    [anon_sym_default] = ACTIONS(3124),
    [anon_sym_enum] = ACTIONS(3124),
    [anon_sym_fn] = ACTIONS(3124),
    [anon_sym_for] = ACTIONS(3124),
    [anon_sym_gen] = ACTIONS(3124),
    [anon_sym_if] = ACTIONS(3124),
    [anon_sym_impl] = ACTIONS(3124),
    [anon_sym_let] = ACTIONS(3124),
    [anon_sym_loop] = ACTIONS(3124),
    [anon_sym_match] = ACTIONS(3124),
    [anon_sym_mod] = ACTIONS(3124),
    [anon_sym_pub] = ACTIONS(3124),
    [anon_sym_return] = ACTIONS(3124),
    [anon_sym_raw] = ACTIONS(3124),
    [anon_sym_static] = ACTIONS(3124),
    [anon_sym_struct] = ACTIONS(3124),
    [anon_sym_trait] = ACTIONS(3124),
    [anon_sym_type] = ACTIONS(3124),
    [anon_sym_union] = ACTIONS(3124),
    [anon_sym_unsafe] = ACTIONS(3124),
    [anon_sym_use] = ACTIONS(3124),
    [anon_sym_while] = ACTIONS(3124),
    [anon_sym_extern] = ACTIONS(3124),
    [anon_sym_safe] = ACTIONS(3124),
    [anon_sym_yield] = ACTIONS(3124),
    [anon_sym_move] = ACTIONS(3124),
    [anon_sym_try] = ACTIONS(3124),
    [sym_integer_literal] = ACTIONS(3122),
    [aux_sym_string_literal_token1] = ACTIONS(3122),
    [sym_char_literal] = ACTIONS(3122),
    [anon_sym_true] = ACTIONS(3124),
    [anon_sym_false] = ACTIONS(3124),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3124),
    [sym_super] = ACTIONS(3124),
    [sym_crate] = ACTIONS(3124),
    [aux_sym_metavariable_token1] = ACTIONS(3122),
    [sym__raw_string_literal_start] = ACTIONS(3122),
    [sym_float_literal] = ACTIONS(3122),
    [sym__outer_block_doc_comment_start] = ACTIONS(3122),
    [sym__outer_line_doc_comment_start] = ACTIONS(3122),
  },
  [STATE(839)] = {
    [sym_line_comment] = STATE(839),
    [sym_block_comment] = STATE(839),
    [ts_builtin_sym_end] = ACTIONS(3126),
    [sym_identifier] = ACTIONS(3128),
    [anon_sym_SEMI] = ACTIONS(3126),
    [anon_sym_macro] = ACTIONS(3128),
    [anon_sym_LPAREN] = ACTIONS(3126),
    [anon_sym_LBRACE] = ACTIONS(3126),
    [anon_sym_RBRACE] = ACTIONS(3126),
    [anon_sym_macro_rules] = ACTIONS(3128),
    [anon_sym_BANG] = ACTIONS(3126),
    [anon_sym_LBRACK] = ACTIONS(3126),
    [anon_sym_STAR] = ACTIONS(3126),
    [anon_sym_u8] = ACTIONS(3128),
    [anon_sym_i8] = ACTIONS(3128),
    [anon_sym_u16] = ACTIONS(3128),
    [anon_sym_i16] = ACTIONS(3128),
    [anon_sym_u32] = ACTIONS(3128),
    [anon_sym_i32] = ACTIONS(3128),
    [anon_sym_u64] = ACTIONS(3128),
    [anon_sym_i64] = ACTIONS(3128),
    [anon_sym_u128] = ACTIONS(3128),
    [anon_sym_i128] = ACTIONS(3128),
    [anon_sym_isize] = ACTIONS(3128),
    [anon_sym_usize] = ACTIONS(3128),
    [anon_sym_f32] = ACTIONS(3128),
    [anon_sym_f64] = ACTIONS(3128),
    [anon_sym_bool] = ACTIONS(3128),
    [anon_sym_str] = ACTIONS(3128),
    [anon_sym_char] = ACTIONS(3128),
    [anon_sym_DASH] = ACTIONS(3126),
    [anon_sym_AMP] = ACTIONS(3126),
    [anon_sym_PIPE] = ACTIONS(3126),
    [anon_sym_LT] = ACTIONS(3126),
    [anon_sym_DOT_DOT] = ACTIONS(3126),
    [anon_sym_COLON_COLON] = ACTIONS(3126),
    [anon_sym_POUND] = ACTIONS(3126),
    [anon_sym_SQUOTE] = ACTIONS(3128),
    [anon_sym_async] = ACTIONS(3128),
    [anon_sym_auto] = ACTIONS(3128),
    [anon_sym_await] = ACTIONS(3128),
    [anon_sym_become] = ACTIONS(3128),
    [anon_sym_break] = ACTIONS(3128),
    [anon_sym_const] = ACTIONS(3128),
    [anon_sym_continue] = ACTIONS(3128),
    [anon_sym_default] = ACTIONS(3128),
    [anon_sym_enum] = ACTIONS(3128),
    [anon_sym_fn] = ACTIONS(3128),
    [anon_sym_for] = ACTIONS(3128),
    [anon_sym_gen] = ACTIONS(3128),
    [anon_sym_if] = ACTIONS(3128),
    [anon_sym_impl] = ACTIONS(3128),
    [anon_sym_let] = ACTIONS(3128),
    [anon_sym_loop] = ACTIONS(3128),
    [anon_sym_match] = ACTIONS(3128),
    [anon_sym_mod] = ACTIONS(3128),
    [anon_sym_pub] = ACTIONS(3128),
    [anon_sym_return] = ACTIONS(3128),
    [anon_sym_raw] = ACTIONS(3128),
    [anon_sym_static] = ACTIONS(3128),
    [anon_sym_struct] = ACTIONS(3128),
    [anon_sym_trait] = ACTIONS(3128),
    [anon_sym_type] = ACTIONS(3128),
    [anon_sym_union] = ACTIONS(3128),
    [anon_sym_unsafe] = ACTIONS(3128),
    [anon_sym_use] = ACTIONS(3128),
    [anon_sym_while] = ACTIONS(3128),
    [anon_sym_extern] = ACTIONS(3128),
    [anon_sym_safe] = ACTIONS(3128),
    [anon_sym_yield] = ACTIONS(3128),
    [anon_sym_move] = ACTIONS(3128),
    [anon_sym_try] = ACTIONS(3128),
    [sym_integer_literal] = ACTIONS(3126),
    [aux_sym_string_literal_token1] = ACTIONS(3126),
    [sym_char_literal] = ACTIONS(3126),
    [anon_sym_true] = ACTIONS(3128),
    [anon_sym_false] = ACTIONS(3128),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3128),
    [sym_super] = ACTIONS(3128),
    [sym_crate] = ACTIONS(3128),
    [aux_sym_metavariable_token1] = ACTIONS(3126),
    [sym__raw_string_literal_start] = ACTIONS(3126),
    [sym_float_literal] = ACTIONS(3126),
    [sym__outer_block_doc_comment_start] = ACTIONS(3126),
    [sym__outer_line_doc_comment_start] = ACTIONS(3126),
  },
  [STATE(840)] = {
    [sym_line_comment] = STATE(840),
    [sym_block_comment] = STATE(840),
    [ts_builtin_sym_end] = ACTIONS(3130),
    [sym_identifier] = ACTIONS(3132),
    [anon_sym_SEMI] = ACTIONS(3130),
    [anon_sym_macro] = ACTIONS(3132),
    [anon_sym_LPAREN] = ACTIONS(3130),
    [anon_sym_LBRACE] = ACTIONS(3130),
    [anon_sym_RBRACE] = ACTIONS(3130),
    [anon_sym_macro_rules] = ACTIONS(3132),
    [anon_sym_BANG] = ACTIONS(3130),
    [anon_sym_LBRACK] = ACTIONS(3130),
    [anon_sym_STAR] = ACTIONS(3130),
    [anon_sym_u8] = ACTIONS(3132),
    [anon_sym_i8] = ACTIONS(3132),
    [anon_sym_u16] = ACTIONS(3132),
    [anon_sym_i16] = ACTIONS(3132),
    [anon_sym_u32] = ACTIONS(3132),
    [anon_sym_i32] = ACTIONS(3132),
    [anon_sym_u64] = ACTIONS(3132),
    [anon_sym_i64] = ACTIONS(3132),
    [anon_sym_u128] = ACTIONS(3132),
    [anon_sym_i128] = ACTIONS(3132),
    [anon_sym_isize] = ACTIONS(3132),
    [anon_sym_usize] = ACTIONS(3132),
    [anon_sym_f32] = ACTIONS(3132),
    [anon_sym_f64] = ACTIONS(3132),
    [anon_sym_bool] = ACTIONS(3132),
    [anon_sym_str] = ACTIONS(3132),
    [anon_sym_char] = ACTIONS(3132),
    [anon_sym_DASH] = ACTIONS(3130),
    [anon_sym_AMP] = ACTIONS(3130),
    [anon_sym_PIPE] = ACTIONS(3130),
    [anon_sym_LT] = ACTIONS(3130),
    [anon_sym_DOT_DOT] = ACTIONS(3130),
    [anon_sym_COLON_COLON] = ACTIONS(3130),
    [anon_sym_POUND] = ACTIONS(3130),
    [anon_sym_SQUOTE] = ACTIONS(3132),
    [anon_sym_async] = ACTIONS(3132),
    [anon_sym_auto] = ACTIONS(3132),
    [anon_sym_await] = ACTIONS(3132),
    [anon_sym_become] = ACTIONS(3132),
    [anon_sym_break] = ACTIONS(3132),
    [anon_sym_const] = ACTIONS(3132),
    [anon_sym_continue] = ACTIONS(3132),
    [anon_sym_default] = ACTIONS(3132),
    [anon_sym_enum] = ACTIONS(3132),
    [anon_sym_fn] = ACTIONS(3132),
    [anon_sym_for] = ACTIONS(3132),
    [anon_sym_gen] = ACTIONS(3132),
    [anon_sym_if] = ACTIONS(3132),
    [anon_sym_impl] = ACTIONS(3132),
    [anon_sym_let] = ACTIONS(3132),
    [anon_sym_loop] = ACTIONS(3132),
    [anon_sym_match] = ACTIONS(3132),
    [anon_sym_mod] = ACTIONS(3132),
    [anon_sym_pub] = ACTIONS(3132),
    [anon_sym_return] = ACTIONS(3132),
    [anon_sym_raw] = ACTIONS(3132),
    [anon_sym_static] = ACTIONS(3132),
    [anon_sym_struct] = ACTIONS(3132),
    [anon_sym_trait] = ACTIONS(3132),
    [anon_sym_type] = ACTIONS(3132),
    [anon_sym_union] = ACTIONS(3132),
    [anon_sym_unsafe] = ACTIONS(3132),
    [anon_sym_use] = ACTIONS(3132),
    [anon_sym_while] = ACTIONS(3132),
    [anon_sym_extern] = ACTIONS(3132),
    [anon_sym_safe] = ACTIONS(3132),
    [anon_sym_yield] = ACTIONS(3132),
    [anon_sym_move] = ACTIONS(3132),
    [anon_sym_try] = ACTIONS(3132),
    [sym_integer_literal] = ACTIONS(3130),
    [aux_sym_string_literal_token1] = ACTIONS(3130),
    [sym_char_literal] = ACTIONS(3130),
    [anon_sym_true] = ACTIONS(3132),
    [anon_sym_false] = ACTIONS(3132),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3132),
    [sym_super] = ACTIONS(3132),
    [sym_crate] = ACTIONS(3132),
    [aux_sym_metavariable_token1] = ACTIONS(3130),
    [sym__raw_string_literal_start] = ACTIONS(3130),
    [sym_float_literal] = ACTIONS(3130),
    [sym__outer_block_doc_comment_start] = ACTIONS(3130),
    [sym__outer_line_doc_comment_start] = ACTIONS(3130),
  },
  [STATE(841)] = {
    [sym_line_comment] = STATE(841),
    [sym_block_comment] = STATE(841),
    [ts_builtin_sym_end] = ACTIONS(3134),
    [sym_identifier] = ACTIONS(3136),
    [anon_sym_SEMI] = ACTIONS(3134),
    [anon_sym_macro] = ACTIONS(3136),
    [anon_sym_LPAREN] = ACTIONS(3134),
    [anon_sym_LBRACE] = ACTIONS(3134),
    [anon_sym_RBRACE] = ACTIONS(3134),
    [anon_sym_macro_rules] = ACTIONS(3136),
    [anon_sym_BANG] = ACTIONS(3134),
    [anon_sym_LBRACK] = ACTIONS(3134),
    [anon_sym_STAR] = ACTIONS(3134),
    [anon_sym_u8] = ACTIONS(3136),
    [anon_sym_i8] = ACTIONS(3136),
    [anon_sym_u16] = ACTIONS(3136),
    [anon_sym_i16] = ACTIONS(3136),
    [anon_sym_u32] = ACTIONS(3136),
    [anon_sym_i32] = ACTIONS(3136),
    [anon_sym_u64] = ACTIONS(3136),
    [anon_sym_i64] = ACTIONS(3136),
    [anon_sym_u128] = ACTIONS(3136),
    [anon_sym_i128] = ACTIONS(3136),
    [anon_sym_isize] = ACTIONS(3136),
    [anon_sym_usize] = ACTIONS(3136),
    [anon_sym_f32] = ACTIONS(3136),
    [anon_sym_f64] = ACTIONS(3136),
    [anon_sym_bool] = ACTIONS(3136),
    [anon_sym_str] = ACTIONS(3136),
    [anon_sym_char] = ACTIONS(3136),
    [anon_sym_DASH] = ACTIONS(3134),
    [anon_sym_AMP] = ACTIONS(3134),
    [anon_sym_PIPE] = ACTIONS(3134),
    [anon_sym_LT] = ACTIONS(3134),
    [anon_sym_DOT_DOT] = ACTIONS(3134),
    [anon_sym_COLON_COLON] = ACTIONS(3134),
    [anon_sym_POUND] = ACTIONS(3134),
    [anon_sym_SQUOTE] = ACTIONS(3136),
    [anon_sym_async] = ACTIONS(3136),
    [anon_sym_auto] = ACTIONS(3136),
    [anon_sym_await] = ACTIONS(3136),
    [anon_sym_become] = ACTIONS(3136),
    [anon_sym_break] = ACTIONS(3136),
    [anon_sym_const] = ACTIONS(3136),
    [anon_sym_continue] = ACTIONS(3136),
    [anon_sym_default] = ACTIONS(3136),
    [anon_sym_enum] = ACTIONS(3136),
    [anon_sym_fn] = ACTIONS(3136),
    [anon_sym_for] = ACTIONS(3136),
    [anon_sym_gen] = ACTIONS(3136),
    [anon_sym_if] = ACTIONS(3136),
    [anon_sym_impl] = ACTIONS(3136),
    [anon_sym_let] = ACTIONS(3136),
    [anon_sym_loop] = ACTIONS(3136),
    [anon_sym_match] = ACTIONS(3136),
    [anon_sym_mod] = ACTIONS(3136),
    [anon_sym_pub] = ACTIONS(3136),
    [anon_sym_return] = ACTIONS(3136),
    [anon_sym_raw] = ACTIONS(3136),
    [anon_sym_static] = ACTIONS(3136),
    [anon_sym_struct] = ACTIONS(3136),
    [anon_sym_trait] = ACTIONS(3136),
    [anon_sym_type] = ACTIONS(3136),
    [anon_sym_union] = ACTIONS(3136),
    [anon_sym_unsafe] = ACTIONS(3136),
    [anon_sym_use] = ACTIONS(3136),
    [anon_sym_while] = ACTIONS(3136),
    [anon_sym_extern] = ACTIONS(3136),
    [anon_sym_safe] = ACTIONS(3136),
    [anon_sym_yield] = ACTIONS(3136),
    [anon_sym_move] = ACTIONS(3136),
    [anon_sym_try] = ACTIONS(3136),
    [sym_integer_literal] = ACTIONS(3134),
    [aux_sym_string_literal_token1] = ACTIONS(3134),
    [sym_char_literal] = ACTIONS(3134),
    [anon_sym_true] = ACTIONS(3136),
    [anon_sym_false] = ACTIONS(3136),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3136),
    [sym_super] = ACTIONS(3136),
    [sym_crate] = ACTIONS(3136),
    [aux_sym_metavariable_token1] = ACTIONS(3134),
    [sym__raw_string_literal_start] = ACTIONS(3134),
    [sym_float_literal] = ACTIONS(3134),
    [sym__outer_block_doc_comment_start] = ACTIONS(3134),
    [sym__outer_line_doc_comment_start] = ACTIONS(3134),
  },
  [STATE(842)] = {
    [sym_line_comment] = STATE(842),
    [sym_block_comment] = STATE(842),
    [ts_builtin_sym_end] = ACTIONS(3138),
    [sym_identifier] = ACTIONS(3140),
    [anon_sym_SEMI] = ACTIONS(3138),
    [anon_sym_macro] = ACTIONS(3140),
    [anon_sym_LPAREN] = ACTIONS(3138),
    [anon_sym_LBRACE] = ACTIONS(3138),
    [anon_sym_RBRACE] = ACTIONS(3138),
    [anon_sym_macro_rules] = ACTIONS(3140),
    [anon_sym_BANG] = ACTIONS(3138),
    [anon_sym_LBRACK] = ACTIONS(3138),
    [anon_sym_STAR] = ACTIONS(3138),
    [anon_sym_u8] = ACTIONS(3140),
    [anon_sym_i8] = ACTIONS(3140),
    [anon_sym_u16] = ACTIONS(3140),
    [anon_sym_i16] = ACTIONS(3140),
    [anon_sym_u32] = ACTIONS(3140),
    [anon_sym_i32] = ACTIONS(3140),
    [anon_sym_u64] = ACTIONS(3140),
    [anon_sym_i64] = ACTIONS(3140),
    [anon_sym_u128] = ACTIONS(3140),
    [anon_sym_i128] = ACTIONS(3140),
    [anon_sym_isize] = ACTIONS(3140),
    [anon_sym_usize] = ACTIONS(3140),
    [anon_sym_f32] = ACTIONS(3140),
    [anon_sym_f64] = ACTIONS(3140),
    [anon_sym_bool] = ACTIONS(3140),
    [anon_sym_str] = ACTIONS(3140),
    [anon_sym_char] = ACTIONS(3140),
    [anon_sym_DASH] = ACTIONS(3138),
    [anon_sym_AMP] = ACTIONS(3138),
    [anon_sym_PIPE] = ACTIONS(3138),
    [anon_sym_LT] = ACTIONS(3138),
    [anon_sym_DOT_DOT] = ACTIONS(3138),
    [anon_sym_COLON_COLON] = ACTIONS(3138),
    [anon_sym_POUND] = ACTIONS(3138),
    [anon_sym_SQUOTE] = ACTIONS(3140),
    [anon_sym_async] = ACTIONS(3140),
    [anon_sym_auto] = ACTIONS(3140),
    [anon_sym_await] = ACTIONS(3140),
    [anon_sym_become] = ACTIONS(3140),
    [anon_sym_break] = ACTIONS(3140),
    [anon_sym_const] = ACTIONS(3140),
    [anon_sym_continue] = ACTIONS(3140),
    [anon_sym_default] = ACTIONS(3140),
    [anon_sym_enum] = ACTIONS(3140),
    [anon_sym_fn] = ACTIONS(3140),
    [anon_sym_for] = ACTIONS(3140),
    [anon_sym_gen] = ACTIONS(3140),
    [anon_sym_if] = ACTIONS(3140),
    [anon_sym_impl] = ACTIONS(3140),
    [anon_sym_let] = ACTIONS(3140),
    [anon_sym_loop] = ACTIONS(3140),
    [anon_sym_match] = ACTIONS(3140),
    [anon_sym_mod] = ACTIONS(3140),
    [anon_sym_pub] = ACTIONS(3140),
    [anon_sym_return] = ACTIONS(3140),
    [anon_sym_raw] = ACTIONS(3140),
    [anon_sym_static] = ACTIONS(3140),
    [anon_sym_struct] = ACTIONS(3140),
    [anon_sym_trait] = ACTIONS(3140),
    [anon_sym_type] = ACTIONS(3140),
    [anon_sym_union] = ACTIONS(3140),
    [anon_sym_unsafe] = ACTIONS(3140),
    [anon_sym_use] = ACTIONS(3140),
    [anon_sym_while] = ACTIONS(3140),
    [anon_sym_extern] = ACTIONS(3140),
    [anon_sym_safe] = ACTIONS(3140),
    [anon_sym_yield] = ACTIONS(3140),
    [anon_sym_move] = ACTIONS(3140),
    [anon_sym_try] = ACTIONS(3140),
    [sym_integer_literal] = ACTIONS(3138),
    [aux_sym_string_literal_token1] = ACTIONS(3138),
    [sym_char_literal] = ACTIONS(3138),
    [anon_sym_true] = ACTIONS(3140),
    [anon_sym_false] = ACTIONS(3140),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3140),
    [sym_super] = ACTIONS(3140),
    [sym_crate] = ACTIONS(3140),
    [aux_sym_metavariable_token1] = ACTIONS(3138),
    [sym__raw_string_literal_start] = ACTIONS(3138),
    [sym_float_literal] = ACTIONS(3138),
    [sym__outer_block_doc_comment_start] = ACTIONS(3138),
    [sym__outer_line_doc_comment_start] = ACTIONS(3138),
  },
  [STATE(843)] = {
    [sym_line_comment] = STATE(843),
    [sym_block_comment] = STATE(843),
    [ts_builtin_sym_end] = ACTIONS(3142),
    [sym_identifier] = ACTIONS(3144),
    [anon_sym_SEMI] = ACTIONS(3142),
    [anon_sym_macro] = ACTIONS(3144),
    [anon_sym_LPAREN] = ACTIONS(3142),
    [anon_sym_LBRACE] = ACTIONS(3142),
    [anon_sym_RBRACE] = ACTIONS(3142),
    [anon_sym_macro_rules] = ACTIONS(3144),
    [anon_sym_BANG] = ACTIONS(3142),
    [anon_sym_LBRACK] = ACTIONS(3142),
    [anon_sym_STAR] = ACTIONS(3142),
    [anon_sym_u8] = ACTIONS(3144),
    [anon_sym_i8] = ACTIONS(3144),
    [anon_sym_u16] = ACTIONS(3144),
    [anon_sym_i16] = ACTIONS(3144),
    [anon_sym_u32] = ACTIONS(3144),
    [anon_sym_i32] = ACTIONS(3144),
    [anon_sym_u64] = ACTIONS(3144),
    [anon_sym_i64] = ACTIONS(3144),
    [anon_sym_u128] = ACTIONS(3144),
    [anon_sym_i128] = ACTIONS(3144),
    [anon_sym_isize] = ACTIONS(3144),
    [anon_sym_usize] = ACTIONS(3144),
    [anon_sym_f32] = ACTIONS(3144),
    [anon_sym_f64] = ACTIONS(3144),
    [anon_sym_bool] = ACTIONS(3144),
    [anon_sym_str] = ACTIONS(3144),
    [anon_sym_char] = ACTIONS(3144),
    [anon_sym_DASH] = ACTIONS(3142),
    [anon_sym_AMP] = ACTIONS(3142),
    [anon_sym_PIPE] = ACTIONS(3142),
    [anon_sym_LT] = ACTIONS(3142),
    [anon_sym_DOT_DOT] = ACTIONS(3142),
    [anon_sym_COLON_COLON] = ACTIONS(3142),
    [anon_sym_POUND] = ACTIONS(3142),
    [anon_sym_SQUOTE] = ACTIONS(3144),
    [anon_sym_async] = ACTIONS(3144),
    [anon_sym_auto] = ACTIONS(3144),
    [anon_sym_await] = ACTIONS(3144),
    [anon_sym_become] = ACTIONS(3144),
    [anon_sym_break] = ACTIONS(3144),
    [anon_sym_const] = ACTIONS(3144),
    [anon_sym_continue] = ACTIONS(3144),
    [anon_sym_default] = ACTIONS(3144),
    [anon_sym_enum] = ACTIONS(3144),
    [anon_sym_fn] = ACTIONS(3144),
    [anon_sym_for] = ACTIONS(3144),
    [anon_sym_gen] = ACTIONS(3144),
    [anon_sym_if] = ACTIONS(3144),
    [anon_sym_impl] = ACTIONS(3144),
    [anon_sym_let] = ACTIONS(3144),
    [anon_sym_loop] = ACTIONS(3144),
    [anon_sym_match] = ACTIONS(3144),
    [anon_sym_mod] = ACTIONS(3144),
    [anon_sym_pub] = ACTIONS(3144),
    [anon_sym_return] = ACTIONS(3144),
    [anon_sym_raw] = ACTIONS(3144),
    [anon_sym_static] = ACTIONS(3144),
    [anon_sym_struct] = ACTIONS(3144),
    [anon_sym_trait] = ACTIONS(3144),
    [anon_sym_type] = ACTIONS(3144),
    [anon_sym_union] = ACTIONS(3144),
    [anon_sym_unsafe] = ACTIONS(3144),
    [anon_sym_use] = ACTIONS(3144),
    [anon_sym_while] = ACTIONS(3144),
    [anon_sym_extern] = ACTIONS(3144),
    [anon_sym_safe] = ACTIONS(3144),
    [anon_sym_yield] = ACTIONS(3144),
    [anon_sym_move] = ACTIONS(3144),
    [anon_sym_try] = ACTIONS(3144),
    [sym_integer_literal] = ACTIONS(3142),
    [aux_sym_string_literal_token1] = ACTIONS(3142),
    [sym_char_literal] = ACTIONS(3142),
    [anon_sym_true] = ACTIONS(3144),
    [anon_sym_false] = ACTIONS(3144),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3144),
    [sym_super] = ACTIONS(3144),
    [sym_crate] = ACTIONS(3144),
    [aux_sym_metavariable_token1] = ACTIONS(3142),
    [sym__raw_string_literal_start] = ACTIONS(3142),
    [sym_float_literal] = ACTIONS(3142),
    [sym__outer_block_doc_comment_start] = ACTIONS(3142),
    [sym__outer_line_doc_comment_start] = ACTIONS(3142),
  },
  [STATE(844)] = {
    [sym_line_comment] = STATE(844),
    [sym_block_comment] = STATE(844),
    [ts_builtin_sym_end] = ACTIONS(3146),
    [sym_identifier] = ACTIONS(3148),
    [anon_sym_SEMI] = ACTIONS(3146),
    [anon_sym_macro] = ACTIONS(3148),
    [anon_sym_LPAREN] = ACTIONS(3146),
    [anon_sym_LBRACE] = ACTIONS(3146),
    [anon_sym_RBRACE] = ACTIONS(3146),
    [anon_sym_macro_rules] = ACTIONS(3148),
    [anon_sym_BANG] = ACTIONS(3146),
    [anon_sym_LBRACK] = ACTIONS(3146),
    [anon_sym_STAR] = ACTIONS(3146),
    [anon_sym_u8] = ACTIONS(3148),
    [anon_sym_i8] = ACTIONS(3148),
    [anon_sym_u16] = ACTIONS(3148),
    [anon_sym_i16] = ACTIONS(3148),
    [anon_sym_u32] = ACTIONS(3148),
    [anon_sym_i32] = ACTIONS(3148),
    [anon_sym_u64] = ACTIONS(3148),
    [anon_sym_i64] = ACTIONS(3148),
    [anon_sym_u128] = ACTIONS(3148),
    [anon_sym_i128] = ACTIONS(3148),
    [anon_sym_isize] = ACTIONS(3148),
    [anon_sym_usize] = ACTIONS(3148),
    [anon_sym_f32] = ACTIONS(3148),
    [anon_sym_f64] = ACTIONS(3148),
    [anon_sym_bool] = ACTIONS(3148),
    [anon_sym_str] = ACTIONS(3148),
    [anon_sym_char] = ACTIONS(3148),
    [anon_sym_DASH] = ACTIONS(3146),
    [anon_sym_AMP] = ACTIONS(3146),
    [anon_sym_PIPE] = ACTIONS(3146),
    [anon_sym_LT] = ACTIONS(3146),
    [anon_sym_DOT_DOT] = ACTIONS(3146),
    [anon_sym_COLON_COLON] = ACTIONS(3146),
    [anon_sym_POUND] = ACTIONS(3146),
    [anon_sym_SQUOTE] = ACTIONS(3148),
    [anon_sym_async] = ACTIONS(3148),
    [anon_sym_auto] = ACTIONS(3148),
    [anon_sym_await] = ACTIONS(3148),
    [anon_sym_become] = ACTIONS(3148),
    [anon_sym_break] = ACTIONS(3148),
    [anon_sym_const] = ACTIONS(3148),
    [anon_sym_continue] = ACTIONS(3148),
    [anon_sym_default] = ACTIONS(3148),
    [anon_sym_enum] = ACTIONS(3148),
    [anon_sym_fn] = ACTIONS(3148),
    [anon_sym_for] = ACTIONS(3148),
    [anon_sym_gen] = ACTIONS(3148),
    [anon_sym_if] = ACTIONS(3148),
    [anon_sym_impl] = ACTIONS(3148),
    [anon_sym_let] = ACTIONS(3148),
    [anon_sym_loop] = ACTIONS(3148),
    [anon_sym_match] = ACTIONS(3148),
    [anon_sym_mod] = ACTIONS(3148),
    [anon_sym_pub] = ACTIONS(3148),
    [anon_sym_return] = ACTIONS(3148),
    [anon_sym_raw] = ACTIONS(3148),
    [anon_sym_static] = ACTIONS(3148),
    [anon_sym_struct] = ACTIONS(3148),
    [anon_sym_trait] = ACTIONS(3148),
    [anon_sym_type] = ACTIONS(3148),
    [anon_sym_union] = ACTIONS(3148),
    [anon_sym_unsafe] = ACTIONS(3148),
    [anon_sym_use] = ACTIONS(3148),
    [anon_sym_while] = ACTIONS(3148),
    [anon_sym_extern] = ACTIONS(3148),
    [anon_sym_safe] = ACTIONS(3148),
    [anon_sym_yield] = ACTIONS(3148),
    [anon_sym_move] = ACTIONS(3148),
    [anon_sym_try] = ACTIONS(3148),
    [sym_integer_literal] = ACTIONS(3146),
    [aux_sym_string_literal_token1] = ACTIONS(3146),
    [sym_char_literal] = ACTIONS(3146),
    [anon_sym_true] = ACTIONS(3148),
    [anon_sym_false] = ACTIONS(3148),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3148),
    [sym_super] = ACTIONS(3148),
    [sym_crate] = ACTIONS(3148),
    [aux_sym_metavariable_token1] = ACTIONS(3146),
    [sym__raw_string_literal_start] = ACTIONS(3146),
    [sym_float_literal] = ACTIONS(3146),
    [sym__outer_block_doc_comment_start] = ACTIONS(3146),
    [sym__outer_line_doc_comment_start] = ACTIONS(3146),
  },
  [STATE(845)] = {
    [sym_line_comment] = STATE(845),
    [sym_block_comment] = STATE(845),
    [ts_builtin_sym_end] = ACTIONS(3150),
    [sym_identifier] = ACTIONS(3152),
    [anon_sym_SEMI] = ACTIONS(3150),
    [anon_sym_macro] = ACTIONS(3152),
    [anon_sym_LPAREN] = ACTIONS(3150),
    [anon_sym_LBRACE] = ACTIONS(3150),
    [anon_sym_RBRACE] = ACTIONS(3150),
    [anon_sym_macro_rules] = ACTIONS(3152),
    [anon_sym_BANG] = ACTIONS(3150),
    [anon_sym_LBRACK] = ACTIONS(3150),
    [anon_sym_STAR] = ACTIONS(3150),
    [anon_sym_u8] = ACTIONS(3152),
    [anon_sym_i8] = ACTIONS(3152),
    [anon_sym_u16] = ACTIONS(3152),
    [anon_sym_i16] = ACTIONS(3152),
    [anon_sym_u32] = ACTIONS(3152),
    [anon_sym_i32] = ACTIONS(3152),
    [anon_sym_u64] = ACTIONS(3152),
    [anon_sym_i64] = ACTIONS(3152),
    [anon_sym_u128] = ACTIONS(3152),
    [anon_sym_i128] = ACTIONS(3152),
    [anon_sym_isize] = ACTIONS(3152),
    [anon_sym_usize] = ACTIONS(3152),
    [anon_sym_f32] = ACTIONS(3152),
    [anon_sym_f64] = ACTIONS(3152),
    [anon_sym_bool] = ACTIONS(3152),
    [anon_sym_str] = ACTIONS(3152),
    [anon_sym_char] = ACTIONS(3152),
    [anon_sym_DASH] = ACTIONS(3150),
    [anon_sym_AMP] = ACTIONS(3150),
    [anon_sym_PIPE] = ACTIONS(3150),
    [anon_sym_LT] = ACTIONS(3150),
    [anon_sym_DOT_DOT] = ACTIONS(3150),
    [anon_sym_COLON_COLON] = ACTIONS(3150),
    [anon_sym_POUND] = ACTIONS(3150),
    [anon_sym_SQUOTE] = ACTIONS(3152),
    [anon_sym_async] = ACTIONS(3152),
    [anon_sym_auto] = ACTIONS(3152),
    [anon_sym_await] = ACTIONS(3152),
    [anon_sym_become] = ACTIONS(3152),
    [anon_sym_break] = ACTIONS(3152),
    [anon_sym_const] = ACTIONS(3152),
    [anon_sym_continue] = ACTIONS(3152),
    [anon_sym_default] = ACTIONS(3152),
    [anon_sym_enum] = ACTIONS(3152),
    [anon_sym_fn] = ACTIONS(3152),
    [anon_sym_for] = ACTIONS(3152),
    [anon_sym_gen] = ACTIONS(3152),
    [anon_sym_if] = ACTIONS(3152),
    [anon_sym_impl] = ACTIONS(3152),
    [anon_sym_let] = ACTIONS(3152),
    [anon_sym_loop] = ACTIONS(3152),
    [anon_sym_match] = ACTIONS(3152),
    [anon_sym_mod] = ACTIONS(3152),
    [anon_sym_pub] = ACTIONS(3152),
    [anon_sym_return] = ACTIONS(3152),
    [anon_sym_raw] = ACTIONS(3152),
    [anon_sym_static] = ACTIONS(3152),
    [anon_sym_struct] = ACTIONS(3152),
    [anon_sym_trait] = ACTIONS(3152),
    [anon_sym_type] = ACTIONS(3152),
    [anon_sym_union] = ACTIONS(3152),
    [anon_sym_unsafe] = ACTIONS(3152),
    [anon_sym_use] = ACTIONS(3152),
    [anon_sym_while] = ACTIONS(3152),
    [anon_sym_extern] = ACTIONS(3152),
    [anon_sym_safe] = ACTIONS(3152),
    [anon_sym_yield] = ACTIONS(3152),
    [anon_sym_move] = ACTIONS(3152),
    [anon_sym_try] = ACTIONS(3152),
    [sym_integer_literal] = ACTIONS(3150),
    [aux_sym_string_literal_token1] = ACTIONS(3150),
    [sym_char_literal] = ACTIONS(3150),
    [anon_sym_true] = ACTIONS(3152),
    [anon_sym_false] = ACTIONS(3152),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3152),
    [sym_super] = ACTIONS(3152),
    [sym_crate] = ACTIONS(3152),
    [aux_sym_metavariable_token1] = ACTIONS(3150),
    [sym__raw_string_literal_start] = ACTIONS(3150),
    [sym_float_literal] = ACTIONS(3150),
    [sym__outer_block_doc_comment_start] = ACTIONS(3150),
    [sym__outer_line_doc_comment_start] = ACTIONS(3150),
  },
  [STATE(846)] = {
    [sym_line_comment] = STATE(846),
    [sym_block_comment] = STATE(846),
    [ts_builtin_sym_end] = ACTIONS(3154),
    [sym_identifier] = ACTIONS(3156),
    [anon_sym_SEMI] = ACTIONS(3154),
    [anon_sym_macro] = ACTIONS(3156),
    [anon_sym_LPAREN] = ACTIONS(3154),
    [anon_sym_LBRACE] = ACTIONS(3154),
    [anon_sym_RBRACE] = ACTIONS(3154),
    [anon_sym_macro_rules] = ACTIONS(3156),
    [anon_sym_BANG] = ACTIONS(3154),
    [anon_sym_LBRACK] = ACTIONS(3154),
    [anon_sym_STAR] = ACTIONS(3154),
    [anon_sym_u8] = ACTIONS(3156),
    [anon_sym_i8] = ACTIONS(3156),
    [anon_sym_u16] = ACTIONS(3156),
    [anon_sym_i16] = ACTIONS(3156),
    [anon_sym_u32] = ACTIONS(3156),
    [anon_sym_i32] = ACTIONS(3156),
    [anon_sym_u64] = ACTIONS(3156),
    [anon_sym_i64] = ACTIONS(3156),
    [anon_sym_u128] = ACTIONS(3156),
    [anon_sym_i128] = ACTIONS(3156),
    [anon_sym_isize] = ACTIONS(3156),
    [anon_sym_usize] = ACTIONS(3156),
    [anon_sym_f32] = ACTIONS(3156),
    [anon_sym_f64] = ACTIONS(3156),
    [anon_sym_bool] = ACTIONS(3156),
    [anon_sym_str] = ACTIONS(3156),
    [anon_sym_char] = ACTIONS(3156),
    [anon_sym_DASH] = ACTIONS(3154),
    [anon_sym_AMP] = ACTIONS(3154),
    [anon_sym_PIPE] = ACTIONS(3154),
    [anon_sym_LT] = ACTIONS(3154),
    [anon_sym_DOT_DOT] = ACTIONS(3154),
    [anon_sym_COLON_COLON] = ACTIONS(3154),
    [anon_sym_POUND] = ACTIONS(3154),
    [anon_sym_SQUOTE] = ACTIONS(3156),
    [anon_sym_async] = ACTIONS(3156),
    [anon_sym_auto] = ACTIONS(3156),
    [anon_sym_await] = ACTIONS(3156),
    [anon_sym_become] = ACTIONS(3156),
    [anon_sym_break] = ACTIONS(3156),
    [anon_sym_const] = ACTIONS(3156),
    [anon_sym_continue] = ACTIONS(3156),
    [anon_sym_default] = ACTIONS(3156),
    [anon_sym_enum] = ACTIONS(3156),
    [anon_sym_fn] = ACTIONS(3156),
    [anon_sym_for] = ACTIONS(3156),
    [anon_sym_gen] = ACTIONS(3156),
    [anon_sym_if] = ACTIONS(3156),
    [anon_sym_impl] = ACTIONS(3156),
    [anon_sym_let] = ACTIONS(3156),
    [anon_sym_loop] = ACTIONS(3156),
    [anon_sym_match] = ACTIONS(3156),
    [anon_sym_mod] = ACTIONS(3156),
    [anon_sym_pub] = ACTIONS(3156),
    [anon_sym_return] = ACTIONS(3156),
    [anon_sym_raw] = ACTIONS(3156),
    [anon_sym_static] = ACTIONS(3156),
    [anon_sym_struct] = ACTIONS(3156),
    [anon_sym_trait] = ACTIONS(3156),
    [anon_sym_type] = ACTIONS(3156),
    [anon_sym_union] = ACTIONS(3156),
    [anon_sym_unsafe] = ACTIONS(3156),
    [anon_sym_use] = ACTIONS(3156),
    [anon_sym_while] = ACTIONS(3156),
    [anon_sym_extern] = ACTIONS(3156),
    [anon_sym_safe] = ACTIONS(3156),
    [anon_sym_yield] = ACTIONS(3156),
    [anon_sym_move] = ACTIONS(3156),
    [anon_sym_try] = ACTIONS(3156),
    [sym_integer_literal] = ACTIONS(3154),
    [aux_sym_string_literal_token1] = ACTIONS(3154),
    [sym_char_literal] = ACTIONS(3154),
    [anon_sym_true] = ACTIONS(3156),
    [anon_sym_false] = ACTIONS(3156),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3156),
    [sym_super] = ACTIONS(3156),
    [sym_crate] = ACTIONS(3156),
    [aux_sym_metavariable_token1] = ACTIONS(3154),
    [sym__raw_string_literal_start] = ACTIONS(3154),
    [sym_float_literal] = ACTIONS(3154),
    [sym__outer_block_doc_comment_start] = ACTIONS(3154),
    [sym__outer_line_doc_comment_start] = ACTIONS(3154),
  },
  [STATE(847)] = {
    [sym_line_comment] = STATE(847),
    [sym_block_comment] = STATE(847),
    [ts_builtin_sym_end] = ACTIONS(3158),
    [sym_identifier] = ACTIONS(3160),
    [anon_sym_SEMI] = ACTIONS(3158),
    [anon_sym_macro] = ACTIONS(3160),
    [anon_sym_LPAREN] = ACTIONS(3158),
    [anon_sym_LBRACE] = ACTIONS(3158),
    [anon_sym_RBRACE] = ACTIONS(3158),
    [anon_sym_macro_rules] = ACTIONS(3160),
    [anon_sym_BANG] = ACTIONS(3158),
    [anon_sym_LBRACK] = ACTIONS(3158),
    [anon_sym_STAR] = ACTIONS(3158),
    [anon_sym_u8] = ACTIONS(3160),
    [anon_sym_i8] = ACTIONS(3160),
    [anon_sym_u16] = ACTIONS(3160),
    [anon_sym_i16] = ACTIONS(3160),
    [anon_sym_u32] = ACTIONS(3160),
    [anon_sym_i32] = ACTIONS(3160),
    [anon_sym_u64] = ACTIONS(3160),
    [anon_sym_i64] = ACTIONS(3160),
    [anon_sym_u128] = ACTIONS(3160),
    [anon_sym_i128] = ACTIONS(3160),
    [anon_sym_isize] = ACTIONS(3160),
    [anon_sym_usize] = ACTIONS(3160),
    [anon_sym_f32] = ACTIONS(3160),
    [anon_sym_f64] = ACTIONS(3160),
    [anon_sym_bool] = ACTIONS(3160),
    [anon_sym_str] = ACTIONS(3160),
    [anon_sym_char] = ACTIONS(3160),
    [anon_sym_DASH] = ACTIONS(3158),
    [anon_sym_AMP] = ACTIONS(3158),
    [anon_sym_PIPE] = ACTIONS(3158),
    [anon_sym_LT] = ACTIONS(3158),
    [anon_sym_DOT_DOT] = ACTIONS(3158),
    [anon_sym_COLON_COLON] = ACTIONS(3158),
    [anon_sym_POUND] = ACTIONS(3158),
    [anon_sym_SQUOTE] = ACTIONS(3160),
    [anon_sym_async] = ACTIONS(3160),
    [anon_sym_auto] = ACTIONS(3160),
    [anon_sym_await] = ACTIONS(3160),
    [anon_sym_become] = ACTIONS(3160),
    [anon_sym_break] = ACTIONS(3160),
    [anon_sym_const] = ACTIONS(3160),
    [anon_sym_continue] = ACTIONS(3160),
    [anon_sym_default] = ACTIONS(3160),
    [anon_sym_enum] = ACTIONS(3160),
    [anon_sym_fn] = ACTIONS(3160),
    [anon_sym_for] = ACTIONS(3160),
    [anon_sym_gen] = ACTIONS(3160),
    [anon_sym_if] = ACTIONS(3160),
    [anon_sym_impl] = ACTIONS(3160),
    [anon_sym_let] = ACTIONS(3160),
    [anon_sym_loop] = ACTIONS(3160),
    [anon_sym_match] = ACTIONS(3160),
    [anon_sym_mod] = ACTIONS(3160),
    [anon_sym_pub] = ACTIONS(3160),
    [anon_sym_return] = ACTIONS(3160),
    [anon_sym_raw] = ACTIONS(3160),
    [anon_sym_static] = ACTIONS(3160),
    [anon_sym_struct] = ACTIONS(3160),
    [anon_sym_trait] = ACTIONS(3160),
    [anon_sym_type] = ACTIONS(3160),
    [anon_sym_union] = ACTIONS(3160),
    [anon_sym_unsafe] = ACTIONS(3160),
    [anon_sym_use] = ACTIONS(3160),
    [anon_sym_while] = ACTIONS(3160),
    [anon_sym_extern] = ACTIONS(3160),
    [anon_sym_safe] = ACTIONS(3160),
    [anon_sym_yield] = ACTIONS(3160),
    [anon_sym_move] = ACTIONS(3160),
    [anon_sym_try] = ACTIONS(3160),
    [sym_integer_literal] = ACTIONS(3158),
    [aux_sym_string_literal_token1] = ACTIONS(3158),
    [sym_char_literal] = ACTIONS(3158),
    [anon_sym_true] = ACTIONS(3160),
    [anon_sym_false] = ACTIONS(3160),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3160),
    [sym_super] = ACTIONS(3160),
    [sym_crate] = ACTIONS(3160),
    [aux_sym_metavariable_token1] = ACTIONS(3158),
    [sym__raw_string_literal_start] = ACTIONS(3158),
    [sym_float_literal] = ACTIONS(3158),
    [sym__outer_block_doc_comment_start] = ACTIONS(3158),
    [sym__outer_line_doc_comment_start] = ACTIONS(3158),
  },
  [STATE(848)] = {
    [sym_line_comment] = STATE(848),
    [sym_block_comment] = STATE(848),
    [ts_builtin_sym_end] = ACTIONS(3162),
    [sym_identifier] = ACTIONS(3164),
    [anon_sym_SEMI] = ACTIONS(3162),
    [anon_sym_macro] = ACTIONS(3164),
    [anon_sym_LPAREN] = ACTIONS(3162),
    [anon_sym_LBRACE] = ACTIONS(3162),
    [anon_sym_RBRACE] = ACTIONS(3162),
    [anon_sym_macro_rules] = ACTIONS(3164),
    [anon_sym_BANG] = ACTIONS(3162),
    [anon_sym_LBRACK] = ACTIONS(3162),
    [anon_sym_STAR] = ACTIONS(3162),
    [anon_sym_u8] = ACTIONS(3164),
    [anon_sym_i8] = ACTIONS(3164),
    [anon_sym_u16] = ACTIONS(3164),
    [anon_sym_i16] = ACTIONS(3164),
    [anon_sym_u32] = ACTIONS(3164),
    [anon_sym_i32] = ACTIONS(3164),
    [anon_sym_u64] = ACTIONS(3164),
    [anon_sym_i64] = ACTIONS(3164),
    [anon_sym_u128] = ACTIONS(3164),
    [anon_sym_i128] = ACTIONS(3164),
    [anon_sym_isize] = ACTIONS(3164),
    [anon_sym_usize] = ACTIONS(3164),
    [anon_sym_f32] = ACTIONS(3164),
    [anon_sym_f64] = ACTIONS(3164),
    [anon_sym_bool] = ACTIONS(3164),
    [anon_sym_str] = ACTIONS(3164),
    [anon_sym_char] = ACTIONS(3164),
    [anon_sym_DASH] = ACTIONS(3162),
    [anon_sym_AMP] = ACTIONS(3162),
    [anon_sym_PIPE] = ACTIONS(3162),
    [anon_sym_LT] = ACTIONS(3162),
    [anon_sym_DOT_DOT] = ACTIONS(3162),
    [anon_sym_COLON_COLON] = ACTIONS(3162),
    [anon_sym_POUND] = ACTIONS(3162),
    [anon_sym_SQUOTE] = ACTIONS(3164),
    [anon_sym_async] = ACTIONS(3164),
    [anon_sym_auto] = ACTIONS(3164),
    [anon_sym_await] = ACTIONS(3164),
    [anon_sym_become] = ACTIONS(3164),
    [anon_sym_break] = ACTIONS(3164),
    [anon_sym_const] = ACTIONS(3164),
    [anon_sym_continue] = ACTIONS(3164),
    [anon_sym_default] = ACTIONS(3164),
    [anon_sym_enum] = ACTIONS(3164),
    [anon_sym_fn] = ACTIONS(3164),
    [anon_sym_for] = ACTIONS(3164),
    [anon_sym_gen] = ACTIONS(3164),
    [anon_sym_if] = ACTIONS(3164),
    [anon_sym_impl] = ACTIONS(3164),
    [anon_sym_let] = ACTIONS(3164),
    [anon_sym_loop] = ACTIONS(3164),
    [anon_sym_match] = ACTIONS(3164),
    [anon_sym_mod] = ACTIONS(3164),
    [anon_sym_pub] = ACTIONS(3164),
    [anon_sym_return] = ACTIONS(3164),
    [anon_sym_raw] = ACTIONS(3164),
    [anon_sym_static] = ACTIONS(3164),
    [anon_sym_struct] = ACTIONS(3164),
    [anon_sym_trait] = ACTIONS(3164),
    [anon_sym_type] = ACTIONS(3164),
    [anon_sym_union] = ACTIONS(3164),
    [anon_sym_unsafe] = ACTIONS(3164),
    [anon_sym_use] = ACTIONS(3164),
    [anon_sym_while] = ACTIONS(3164),
    [anon_sym_extern] = ACTIONS(3164),
    [anon_sym_safe] = ACTIONS(3164),
    [anon_sym_yield] = ACTIONS(3164),
    [anon_sym_move] = ACTIONS(3164),
    [anon_sym_try] = ACTIONS(3164),
    [sym_integer_literal] = ACTIONS(3162),
    [aux_sym_string_literal_token1] = ACTIONS(3162),
    [sym_char_literal] = ACTIONS(3162),
    [anon_sym_true] = ACTIONS(3164),
    [anon_sym_false] = ACTIONS(3164),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3164),
    [sym_super] = ACTIONS(3164),
    [sym_crate] = ACTIONS(3164),
    [aux_sym_metavariable_token1] = ACTIONS(3162),
    [sym__raw_string_literal_start] = ACTIONS(3162),
    [sym_float_literal] = ACTIONS(3162),
    [sym__outer_block_doc_comment_start] = ACTIONS(3162),
    [sym__outer_line_doc_comment_start] = ACTIONS(3162),
  },
  [STATE(849)] = {
    [sym_line_comment] = STATE(849),
    [sym_block_comment] = STATE(849),
    [ts_builtin_sym_end] = ACTIONS(3166),
    [sym_identifier] = ACTIONS(3168),
    [anon_sym_SEMI] = ACTIONS(3166),
    [anon_sym_macro] = ACTIONS(3168),
    [anon_sym_LPAREN] = ACTIONS(3166),
    [anon_sym_LBRACE] = ACTIONS(3166),
    [anon_sym_RBRACE] = ACTIONS(3166),
    [anon_sym_macro_rules] = ACTIONS(3168),
    [anon_sym_BANG] = ACTIONS(3166),
    [anon_sym_LBRACK] = ACTIONS(3166),
    [anon_sym_STAR] = ACTIONS(3166),
    [anon_sym_u8] = ACTIONS(3168),
    [anon_sym_i8] = ACTIONS(3168),
    [anon_sym_u16] = ACTIONS(3168),
    [anon_sym_i16] = ACTIONS(3168),
    [anon_sym_u32] = ACTIONS(3168),
    [anon_sym_i32] = ACTIONS(3168),
    [anon_sym_u64] = ACTIONS(3168),
    [anon_sym_i64] = ACTIONS(3168),
    [anon_sym_u128] = ACTIONS(3168),
    [anon_sym_i128] = ACTIONS(3168),
    [anon_sym_isize] = ACTIONS(3168),
    [anon_sym_usize] = ACTIONS(3168),
    [anon_sym_f32] = ACTIONS(3168),
    [anon_sym_f64] = ACTIONS(3168),
    [anon_sym_bool] = ACTIONS(3168),
    [anon_sym_str] = ACTIONS(3168),
    [anon_sym_char] = ACTIONS(3168),
    [anon_sym_DASH] = ACTIONS(3166),
    [anon_sym_AMP] = ACTIONS(3166),
    [anon_sym_PIPE] = ACTIONS(3166),
    [anon_sym_LT] = ACTIONS(3166),
    [anon_sym_DOT_DOT] = ACTIONS(3166),
    [anon_sym_COLON_COLON] = ACTIONS(3166),
    [anon_sym_POUND] = ACTIONS(3166),
    [anon_sym_SQUOTE] = ACTIONS(3168),
    [anon_sym_async] = ACTIONS(3168),
    [anon_sym_auto] = ACTIONS(3168),
    [anon_sym_await] = ACTIONS(3168),
    [anon_sym_become] = ACTIONS(3168),
    [anon_sym_break] = ACTIONS(3168),
    [anon_sym_const] = ACTIONS(3168),
    [anon_sym_continue] = ACTIONS(3168),
    [anon_sym_default] = ACTIONS(3168),
    [anon_sym_enum] = ACTIONS(3168),
    [anon_sym_fn] = ACTIONS(3168),
    [anon_sym_for] = ACTIONS(3168),
    [anon_sym_gen] = ACTIONS(3168),
    [anon_sym_if] = ACTIONS(3168),
    [anon_sym_impl] = ACTIONS(3168),
    [anon_sym_let] = ACTIONS(3168),
    [anon_sym_loop] = ACTIONS(3168),
    [anon_sym_match] = ACTIONS(3168),
    [anon_sym_mod] = ACTIONS(3168),
    [anon_sym_pub] = ACTIONS(3168),
    [anon_sym_return] = ACTIONS(3168),
    [anon_sym_raw] = ACTIONS(3168),
    [anon_sym_static] = ACTIONS(3168),
    [anon_sym_struct] = ACTIONS(3168),
    [anon_sym_trait] = ACTIONS(3168),
    [anon_sym_type] = ACTIONS(3168),
    [anon_sym_union] = ACTIONS(3168),
    [anon_sym_unsafe] = ACTIONS(3168),
    [anon_sym_use] = ACTIONS(3168),
    [anon_sym_while] = ACTIONS(3168),
    [anon_sym_extern] = ACTIONS(3168),
    [anon_sym_safe] = ACTIONS(3168),
    [anon_sym_yield] = ACTIONS(3168),
    [anon_sym_move] = ACTIONS(3168),
    [anon_sym_try] = ACTIONS(3168),
    [sym_integer_literal] = ACTIONS(3166),
    [aux_sym_string_literal_token1] = ACTIONS(3166),
    [sym_char_literal] = ACTIONS(3166),
    [anon_sym_true] = ACTIONS(3168),
    [anon_sym_false] = ACTIONS(3168),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3168),
    [sym_super] = ACTIONS(3168),
    [sym_crate] = ACTIONS(3168),
    [aux_sym_metavariable_token1] = ACTIONS(3166),
    [sym__raw_string_literal_start] = ACTIONS(3166),
    [sym_float_literal] = ACTIONS(3166),
    [sym__outer_block_doc_comment_start] = ACTIONS(3166),
    [sym__outer_line_doc_comment_start] = ACTIONS(3166),
  },
  [STATE(850)] = {
    [sym_line_comment] = STATE(850),
    [sym_block_comment] = STATE(850),
    [ts_builtin_sym_end] = ACTIONS(3170),
    [sym_identifier] = ACTIONS(3172),
    [anon_sym_SEMI] = ACTIONS(3170),
    [anon_sym_macro] = ACTIONS(3172),
    [anon_sym_LPAREN] = ACTIONS(3170),
    [anon_sym_LBRACE] = ACTIONS(3170),
    [anon_sym_RBRACE] = ACTIONS(3170),
    [anon_sym_macro_rules] = ACTIONS(3172),
    [anon_sym_BANG] = ACTIONS(3170),
    [anon_sym_LBRACK] = ACTIONS(3170),
    [anon_sym_STAR] = ACTIONS(3170),
    [anon_sym_u8] = ACTIONS(3172),
    [anon_sym_i8] = ACTIONS(3172),
    [anon_sym_u16] = ACTIONS(3172),
    [anon_sym_i16] = ACTIONS(3172),
    [anon_sym_u32] = ACTIONS(3172),
    [anon_sym_i32] = ACTIONS(3172),
    [anon_sym_u64] = ACTIONS(3172),
    [anon_sym_i64] = ACTIONS(3172),
    [anon_sym_u128] = ACTIONS(3172),
    [anon_sym_i128] = ACTIONS(3172),
    [anon_sym_isize] = ACTIONS(3172),
    [anon_sym_usize] = ACTIONS(3172),
    [anon_sym_f32] = ACTIONS(3172),
    [anon_sym_f64] = ACTIONS(3172),
    [anon_sym_bool] = ACTIONS(3172),
    [anon_sym_str] = ACTIONS(3172),
    [anon_sym_char] = ACTIONS(3172),
    [anon_sym_DASH] = ACTIONS(3170),
    [anon_sym_AMP] = ACTIONS(3170),
    [anon_sym_PIPE] = ACTIONS(3170),
    [anon_sym_LT] = ACTIONS(3170),
    [anon_sym_DOT_DOT] = ACTIONS(3170),
    [anon_sym_COLON_COLON] = ACTIONS(3170),
    [anon_sym_POUND] = ACTIONS(3170),
    [anon_sym_SQUOTE] = ACTIONS(3172),
    [anon_sym_async] = ACTIONS(3172),
    [anon_sym_auto] = ACTIONS(3172),
    [anon_sym_await] = ACTIONS(3172),
    [anon_sym_become] = ACTIONS(3172),
    [anon_sym_break] = ACTIONS(3172),
    [anon_sym_const] = ACTIONS(3172),
    [anon_sym_continue] = ACTIONS(3172),
    [anon_sym_default] = ACTIONS(3172),
    [anon_sym_enum] = ACTIONS(3172),
    [anon_sym_fn] = ACTIONS(3172),
    [anon_sym_for] = ACTIONS(3172),
    [anon_sym_gen] = ACTIONS(3172),
    [anon_sym_if] = ACTIONS(3172),
    [anon_sym_impl] = ACTIONS(3172),
    [anon_sym_let] = ACTIONS(3172),
    [anon_sym_loop] = ACTIONS(3172),
    [anon_sym_match] = ACTIONS(3172),
    [anon_sym_mod] = ACTIONS(3172),
    [anon_sym_pub] = ACTIONS(3172),
    [anon_sym_return] = ACTIONS(3172),
    [anon_sym_raw] = ACTIONS(3172),
    [anon_sym_static] = ACTIONS(3172),
    [anon_sym_struct] = ACTIONS(3172),
    [anon_sym_trait] = ACTIONS(3172),
    [anon_sym_type] = ACTIONS(3172),
    [anon_sym_union] = ACTIONS(3172),
    [anon_sym_unsafe] = ACTIONS(3172),
    [anon_sym_use] = ACTIONS(3172),
    [anon_sym_while] = ACTIONS(3172),
    [anon_sym_extern] = ACTIONS(3172),
    [anon_sym_safe] = ACTIONS(3172),
    [anon_sym_yield] = ACTIONS(3172),
    [anon_sym_move] = ACTIONS(3172),
    [anon_sym_try] = ACTIONS(3172),
    [sym_integer_literal] = ACTIONS(3170),
    [aux_sym_string_literal_token1] = ACTIONS(3170),
    [sym_char_literal] = ACTIONS(3170),
    [anon_sym_true] = ACTIONS(3172),
    [anon_sym_false] = ACTIONS(3172),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3172),
    [sym_super] = ACTIONS(3172),
    [sym_crate] = ACTIONS(3172),
    [aux_sym_metavariable_token1] = ACTIONS(3170),
    [sym__raw_string_literal_start] = ACTIONS(3170),
    [sym_float_literal] = ACTIONS(3170),
    [sym__outer_block_doc_comment_start] = ACTIONS(3170),
    [sym__outer_line_doc_comment_start] = ACTIONS(3170),
  },
  [STATE(851)] = {
    [sym_line_comment] = STATE(851),
    [sym_block_comment] = STATE(851),
    [ts_builtin_sym_end] = ACTIONS(3174),
    [sym_identifier] = ACTIONS(3176),
    [anon_sym_SEMI] = ACTIONS(3174),
    [anon_sym_macro] = ACTIONS(3176),
    [anon_sym_LPAREN] = ACTIONS(3174),
    [anon_sym_LBRACE] = ACTIONS(3174),
    [anon_sym_RBRACE] = ACTIONS(3174),
    [anon_sym_macro_rules] = ACTIONS(3176),
    [anon_sym_BANG] = ACTIONS(3174),
    [anon_sym_LBRACK] = ACTIONS(3174),
    [anon_sym_STAR] = ACTIONS(3174),
    [anon_sym_u8] = ACTIONS(3176),
    [anon_sym_i8] = ACTIONS(3176),
    [anon_sym_u16] = ACTIONS(3176),
    [anon_sym_i16] = ACTIONS(3176),
    [anon_sym_u32] = ACTIONS(3176),
    [anon_sym_i32] = ACTIONS(3176),
    [anon_sym_u64] = ACTIONS(3176),
    [anon_sym_i64] = ACTIONS(3176),
    [anon_sym_u128] = ACTIONS(3176),
    [anon_sym_i128] = ACTIONS(3176),
    [anon_sym_isize] = ACTIONS(3176),
    [anon_sym_usize] = ACTIONS(3176),
    [anon_sym_f32] = ACTIONS(3176),
    [anon_sym_f64] = ACTIONS(3176),
    [anon_sym_bool] = ACTIONS(3176),
    [anon_sym_str] = ACTIONS(3176),
    [anon_sym_char] = ACTIONS(3176),
    [anon_sym_DASH] = ACTIONS(3174),
    [anon_sym_AMP] = ACTIONS(3174),
    [anon_sym_PIPE] = ACTIONS(3174),
    [anon_sym_LT] = ACTIONS(3174),
    [anon_sym_DOT_DOT] = ACTIONS(3174),
    [anon_sym_COLON_COLON] = ACTIONS(3174),
    [anon_sym_POUND] = ACTIONS(3174),
    [anon_sym_SQUOTE] = ACTIONS(3176),
    [anon_sym_async] = ACTIONS(3176),
    [anon_sym_auto] = ACTIONS(3176),
    [anon_sym_await] = ACTIONS(3176),
    [anon_sym_become] = ACTIONS(3176),
    [anon_sym_break] = ACTIONS(3176),
    [anon_sym_const] = ACTIONS(3176),
    [anon_sym_continue] = ACTIONS(3176),
    [anon_sym_default] = ACTIONS(3176),
    [anon_sym_enum] = ACTIONS(3176),
    [anon_sym_fn] = ACTIONS(3176),
    [anon_sym_for] = ACTIONS(3176),
    [anon_sym_gen] = ACTIONS(3176),
    [anon_sym_if] = ACTIONS(3176),
    [anon_sym_impl] = ACTIONS(3176),
    [anon_sym_let] = ACTIONS(3176),
    [anon_sym_loop] = ACTIONS(3176),
    [anon_sym_match] = ACTIONS(3176),
    [anon_sym_mod] = ACTIONS(3176),
    [anon_sym_pub] = ACTIONS(3176),
    [anon_sym_return] = ACTIONS(3176),
    [anon_sym_raw] = ACTIONS(3176),
    [anon_sym_static] = ACTIONS(3176),
    [anon_sym_struct] = ACTIONS(3176),
    [anon_sym_trait] = ACTIONS(3176),
    [anon_sym_type] = ACTIONS(3176),
    [anon_sym_union] = ACTIONS(3176),
    [anon_sym_unsafe] = ACTIONS(3176),
    [anon_sym_use] = ACTIONS(3176),
    [anon_sym_while] = ACTIONS(3176),
    [anon_sym_extern] = ACTIONS(3176),
    [anon_sym_safe] = ACTIONS(3176),
    [anon_sym_yield] = ACTIONS(3176),
    [anon_sym_move] = ACTIONS(3176),
    [anon_sym_try] = ACTIONS(3176),
    [sym_integer_literal] = ACTIONS(3174),
    [aux_sym_string_literal_token1] = ACTIONS(3174),
    [sym_char_literal] = ACTIONS(3174),
    [anon_sym_true] = ACTIONS(3176),
    [anon_sym_false] = ACTIONS(3176),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3176),
    [sym_super] = ACTIONS(3176),
    [sym_crate] = ACTIONS(3176),
    [aux_sym_metavariable_token1] = ACTIONS(3174),
    [sym__raw_string_literal_start] = ACTIONS(3174),
    [sym_float_literal] = ACTIONS(3174),
    [sym__outer_block_doc_comment_start] = ACTIONS(3174),
    [sym__outer_line_doc_comment_start] = ACTIONS(3174),
  },
  [STATE(852)] = {
    [sym_line_comment] = STATE(852),
    [sym_block_comment] = STATE(852),
    [ts_builtin_sym_end] = ACTIONS(3178),
    [sym_identifier] = ACTIONS(3180),
    [anon_sym_SEMI] = ACTIONS(3178),
    [anon_sym_macro] = ACTIONS(3180),
    [anon_sym_LPAREN] = ACTIONS(3178),
    [anon_sym_LBRACE] = ACTIONS(3178),
    [anon_sym_RBRACE] = ACTIONS(3178),
    [anon_sym_macro_rules] = ACTIONS(3180),
    [anon_sym_BANG] = ACTIONS(3178),
    [anon_sym_LBRACK] = ACTIONS(3178),
    [anon_sym_STAR] = ACTIONS(3178),
    [anon_sym_u8] = ACTIONS(3180),
    [anon_sym_i8] = ACTIONS(3180),
    [anon_sym_u16] = ACTIONS(3180),
    [anon_sym_i16] = ACTIONS(3180),
    [anon_sym_u32] = ACTIONS(3180),
    [anon_sym_i32] = ACTIONS(3180),
    [anon_sym_u64] = ACTIONS(3180),
    [anon_sym_i64] = ACTIONS(3180),
    [anon_sym_u128] = ACTIONS(3180),
    [anon_sym_i128] = ACTIONS(3180),
    [anon_sym_isize] = ACTIONS(3180),
    [anon_sym_usize] = ACTIONS(3180),
    [anon_sym_f32] = ACTIONS(3180),
    [anon_sym_f64] = ACTIONS(3180),
    [anon_sym_bool] = ACTIONS(3180),
    [anon_sym_str] = ACTIONS(3180),
    [anon_sym_char] = ACTIONS(3180),
    [anon_sym_DASH] = ACTIONS(3178),
    [anon_sym_AMP] = ACTIONS(3178),
    [anon_sym_PIPE] = ACTIONS(3178),
    [anon_sym_LT] = ACTIONS(3178),
    [anon_sym_DOT_DOT] = ACTIONS(3178),
    [anon_sym_COLON_COLON] = ACTIONS(3178),
    [anon_sym_POUND] = ACTIONS(3178),
    [anon_sym_SQUOTE] = ACTIONS(3180),
    [anon_sym_async] = ACTIONS(3180),
    [anon_sym_auto] = ACTIONS(3180),
    [anon_sym_await] = ACTIONS(3180),
    [anon_sym_become] = ACTIONS(3180),
    [anon_sym_break] = ACTIONS(3180),
    [anon_sym_const] = ACTIONS(3180),
    [anon_sym_continue] = ACTIONS(3180),
    [anon_sym_default] = ACTIONS(3180),
    [anon_sym_enum] = ACTIONS(3180),
    [anon_sym_fn] = ACTIONS(3180),
    [anon_sym_for] = ACTIONS(3180),
    [anon_sym_gen] = ACTIONS(3180),
    [anon_sym_if] = ACTIONS(3180),
    [anon_sym_impl] = ACTIONS(3180),
    [anon_sym_let] = ACTIONS(3180),
    [anon_sym_loop] = ACTIONS(3180),
    [anon_sym_match] = ACTIONS(3180),
    [anon_sym_mod] = ACTIONS(3180),
    [anon_sym_pub] = ACTIONS(3180),
    [anon_sym_return] = ACTIONS(3180),
    [anon_sym_raw] = ACTIONS(3180),
    [anon_sym_static] = ACTIONS(3180),
    [anon_sym_struct] = ACTIONS(3180),
    [anon_sym_trait] = ACTIONS(3180),
    [anon_sym_type] = ACTIONS(3180),
    [anon_sym_union] = ACTIONS(3180),
    [anon_sym_unsafe] = ACTIONS(3180),
    [anon_sym_use] = ACTIONS(3180),
    [anon_sym_while] = ACTIONS(3180),
    [anon_sym_extern] = ACTIONS(3180),
    [anon_sym_safe] = ACTIONS(3180),
    [anon_sym_yield] = ACTIONS(3180),
    [anon_sym_move] = ACTIONS(3180),
    [anon_sym_try] = ACTIONS(3180),
    [sym_integer_literal] = ACTIONS(3178),
    [aux_sym_string_literal_token1] = ACTIONS(3178),
    [sym_char_literal] = ACTIONS(3178),
    [anon_sym_true] = ACTIONS(3180),
    [anon_sym_false] = ACTIONS(3180),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3180),
    [sym_super] = ACTIONS(3180),
    [sym_crate] = ACTIONS(3180),
    [aux_sym_metavariable_token1] = ACTIONS(3178),
    [sym__raw_string_literal_start] = ACTIONS(3178),
    [sym_float_literal] = ACTIONS(3178),
    [sym__outer_block_doc_comment_start] = ACTIONS(3178),
    [sym__outer_line_doc_comment_start] = ACTIONS(3178),
  },
  [STATE(853)] = {
    [sym_line_comment] = STATE(853),
    [sym_block_comment] = STATE(853),
    [ts_builtin_sym_end] = ACTIONS(3182),
    [sym_identifier] = ACTIONS(3184),
    [anon_sym_SEMI] = ACTIONS(3182),
    [anon_sym_macro] = ACTIONS(3184),
    [anon_sym_LPAREN] = ACTIONS(3182),
    [anon_sym_LBRACE] = ACTIONS(3182),
    [anon_sym_RBRACE] = ACTIONS(3182),
    [anon_sym_macro_rules] = ACTIONS(3184),
    [anon_sym_BANG] = ACTIONS(3182),
    [anon_sym_LBRACK] = ACTIONS(3182),
    [anon_sym_STAR] = ACTIONS(3182),
    [anon_sym_u8] = ACTIONS(3184),
    [anon_sym_i8] = ACTIONS(3184),
    [anon_sym_u16] = ACTIONS(3184),
    [anon_sym_i16] = ACTIONS(3184),
    [anon_sym_u32] = ACTIONS(3184),
    [anon_sym_i32] = ACTIONS(3184),
    [anon_sym_u64] = ACTIONS(3184),
    [anon_sym_i64] = ACTIONS(3184),
    [anon_sym_u128] = ACTIONS(3184),
    [anon_sym_i128] = ACTIONS(3184),
    [anon_sym_isize] = ACTIONS(3184),
    [anon_sym_usize] = ACTIONS(3184),
    [anon_sym_f32] = ACTIONS(3184),
    [anon_sym_f64] = ACTIONS(3184),
    [anon_sym_bool] = ACTIONS(3184),
    [anon_sym_str] = ACTIONS(3184),
    [anon_sym_char] = ACTIONS(3184),
    [anon_sym_DASH] = ACTIONS(3182),
    [anon_sym_AMP] = ACTIONS(3182),
    [anon_sym_PIPE] = ACTIONS(3182),
    [anon_sym_LT] = ACTIONS(3182),
    [anon_sym_DOT_DOT] = ACTIONS(3182),
    [anon_sym_COLON_COLON] = ACTIONS(3182),
    [anon_sym_POUND] = ACTIONS(3182),
    [anon_sym_SQUOTE] = ACTIONS(3184),
    [anon_sym_async] = ACTIONS(3184),
    [anon_sym_auto] = ACTIONS(3184),
    [anon_sym_await] = ACTIONS(3184),
    [anon_sym_become] = ACTIONS(3184),
    [anon_sym_break] = ACTIONS(3184),
    [anon_sym_const] = ACTIONS(3184),
    [anon_sym_continue] = ACTIONS(3184),
    [anon_sym_default] = ACTIONS(3184),
    [anon_sym_enum] = ACTIONS(3184),
    [anon_sym_fn] = ACTIONS(3184),
    [anon_sym_for] = ACTIONS(3184),
    [anon_sym_gen] = ACTIONS(3184),
    [anon_sym_if] = ACTIONS(3184),
    [anon_sym_impl] = ACTIONS(3184),
    [anon_sym_let] = ACTIONS(3184),
    [anon_sym_loop] = ACTIONS(3184),
    [anon_sym_match] = ACTIONS(3184),
    [anon_sym_mod] = ACTIONS(3184),
    [anon_sym_pub] = ACTIONS(3184),
    [anon_sym_return] = ACTIONS(3184),
    [anon_sym_raw] = ACTIONS(3184),
    [anon_sym_static] = ACTIONS(3184),
    [anon_sym_struct] = ACTIONS(3184),
    [anon_sym_trait] = ACTIONS(3184),
    [anon_sym_type] = ACTIONS(3184),
    [anon_sym_union] = ACTIONS(3184),
    [anon_sym_unsafe] = ACTIONS(3184),
    [anon_sym_use] = ACTIONS(3184),
    [anon_sym_while] = ACTIONS(3184),
    [anon_sym_extern] = ACTIONS(3184),
    [anon_sym_safe] = ACTIONS(3184),
    [anon_sym_yield] = ACTIONS(3184),
    [anon_sym_move] = ACTIONS(3184),
    [anon_sym_try] = ACTIONS(3184),
    [sym_integer_literal] = ACTIONS(3182),
    [aux_sym_string_literal_token1] = ACTIONS(3182),
    [sym_char_literal] = ACTIONS(3182),
    [anon_sym_true] = ACTIONS(3184),
    [anon_sym_false] = ACTIONS(3184),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3184),
    [sym_super] = ACTIONS(3184),
    [sym_crate] = ACTIONS(3184),
    [aux_sym_metavariable_token1] = ACTIONS(3182),
    [sym__raw_string_literal_start] = ACTIONS(3182),
    [sym_float_literal] = ACTIONS(3182),
    [sym__outer_block_doc_comment_start] = ACTIONS(3182),
    [sym__outer_line_doc_comment_start] = ACTIONS(3182),
  },
  [STATE(854)] = {
    [sym_line_comment] = STATE(854),
    [sym_block_comment] = STATE(854),
    [ts_builtin_sym_end] = ACTIONS(3186),
    [sym_identifier] = ACTIONS(3188),
    [anon_sym_SEMI] = ACTIONS(3186),
    [anon_sym_macro] = ACTIONS(3188),
    [anon_sym_LPAREN] = ACTIONS(3186),
    [anon_sym_LBRACE] = ACTIONS(3186),
    [anon_sym_RBRACE] = ACTIONS(3186),
    [anon_sym_macro_rules] = ACTIONS(3188),
    [anon_sym_BANG] = ACTIONS(3186),
    [anon_sym_LBRACK] = ACTIONS(3186),
    [anon_sym_STAR] = ACTIONS(3186),
    [anon_sym_u8] = ACTIONS(3188),
    [anon_sym_i8] = ACTIONS(3188),
    [anon_sym_u16] = ACTIONS(3188),
    [anon_sym_i16] = ACTIONS(3188),
    [anon_sym_u32] = ACTIONS(3188),
    [anon_sym_i32] = ACTIONS(3188),
    [anon_sym_u64] = ACTIONS(3188),
    [anon_sym_i64] = ACTIONS(3188),
    [anon_sym_u128] = ACTIONS(3188),
    [anon_sym_i128] = ACTIONS(3188),
    [anon_sym_isize] = ACTIONS(3188),
    [anon_sym_usize] = ACTIONS(3188),
    [anon_sym_f32] = ACTIONS(3188),
    [anon_sym_f64] = ACTIONS(3188),
    [anon_sym_bool] = ACTIONS(3188),
    [anon_sym_str] = ACTIONS(3188),
    [anon_sym_char] = ACTIONS(3188),
    [anon_sym_DASH] = ACTIONS(3186),
    [anon_sym_AMP] = ACTIONS(3186),
    [anon_sym_PIPE] = ACTIONS(3186),
    [anon_sym_LT] = ACTIONS(3186),
    [anon_sym_DOT_DOT] = ACTIONS(3186),
    [anon_sym_COLON_COLON] = ACTIONS(3186),
    [anon_sym_POUND] = ACTIONS(3186),
    [anon_sym_SQUOTE] = ACTIONS(3188),
    [anon_sym_async] = ACTIONS(3188),
    [anon_sym_auto] = ACTIONS(3188),
    [anon_sym_await] = ACTIONS(3188),
    [anon_sym_become] = ACTIONS(3188),
    [anon_sym_break] = ACTIONS(3188),
    [anon_sym_const] = ACTIONS(3188),
    [anon_sym_continue] = ACTIONS(3188),
    [anon_sym_default] = ACTIONS(3188),
    [anon_sym_enum] = ACTIONS(3188),
    [anon_sym_fn] = ACTIONS(3188),
    [anon_sym_for] = ACTIONS(3188),
    [anon_sym_gen] = ACTIONS(3188),
    [anon_sym_if] = ACTIONS(3188),
    [anon_sym_impl] = ACTIONS(3188),
    [anon_sym_let] = ACTIONS(3188),
    [anon_sym_loop] = ACTIONS(3188),
    [anon_sym_match] = ACTIONS(3188),
    [anon_sym_mod] = ACTIONS(3188),
    [anon_sym_pub] = ACTIONS(3188),
    [anon_sym_return] = ACTIONS(3188),
    [anon_sym_raw] = ACTIONS(3188),
    [anon_sym_static] = ACTIONS(3188),
    [anon_sym_struct] = ACTIONS(3188),
    [anon_sym_trait] = ACTIONS(3188),
    [anon_sym_type] = ACTIONS(3188),
    [anon_sym_union] = ACTIONS(3188),
    [anon_sym_unsafe] = ACTIONS(3188),
    [anon_sym_use] = ACTIONS(3188),
    [anon_sym_while] = ACTIONS(3188),
    [anon_sym_extern] = ACTIONS(3188),
    [anon_sym_safe] = ACTIONS(3188),
    [anon_sym_yield] = ACTIONS(3188),
    [anon_sym_move] = ACTIONS(3188),
    [anon_sym_try] = ACTIONS(3188),
    [sym_integer_literal] = ACTIONS(3186),
    [aux_sym_string_literal_token1] = ACTIONS(3186),
    [sym_char_literal] = ACTIONS(3186),
    [anon_sym_true] = ACTIONS(3188),
    [anon_sym_false] = ACTIONS(3188),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3188),
    [sym_super] = ACTIONS(3188),
    [sym_crate] = ACTIONS(3188),
    [aux_sym_metavariable_token1] = ACTIONS(3186),
    [sym__raw_string_literal_start] = ACTIONS(3186),
    [sym_float_literal] = ACTIONS(3186),
    [sym__outer_block_doc_comment_start] = ACTIONS(3186),
    [sym__outer_line_doc_comment_start] = ACTIONS(3186),
  },
  [STATE(855)] = {
    [sym_line_comment] = STATE(855),
    [sym_block_comment] = STATE(855),
    [ts_builtin_sym_end] = ACTIONS(3190),
    [sym_identifier] = ACTIONS(3192),
    [anon_sym_SEMI] = ACTIONS(3190),
    [anon_sym_macro] = ACTIONS(3192),
    [anon_sym_LPAREN] = ACTIONS(3190),
    [anon_sym_LBRACE] = ACTIONS(3190),
    [anon_sym_RBRACE] = ACTIONS(3190),
    [anon_sym_macro_rules] = ACTIONS(3192),
    [anon_sym_BANG] = ACTIONS(3190),
    [anon_sym_LBRACK] = ACTIONS(3190),
    [anon_sym_STAR] = ACTIONS(3190),
    [anon_sym_u8] = ACTIONS(3192),
    [anon_sym_i8] = ACTIONS(3192),
    [anon_sym_u16] = ACTIONS(3192),
    [anon_sym_i16] = ACTIONS(3192),
    [anon_sym_u32] = ACTIONS(3192),
    [anon_sym_i32] = ACTIONS(3192),
    [anon_sym_u64] = ACTIONS(3192),
    [anon_sym_i64] = ACTIONS(3192),
    [anon_sym_u128] = ACTIONS(3192),
    [anon_sym_i128] = ACTIONS(3192),
    [anon_sym_isize] = ACTIONS(3192),
    [anon_sym_usize] = ACTIONS(3192),
    [anon_sym_f32] = ACTIONS(3192),
    [anon_sym_f64] = ACTIONS(3192),
    [anon_sym_bool] = ACTIONS(3192),
    [anon_sym_str] = ACTIONS(3192),
    [anon_sym_char] = ACTIONS(3192),
    [anon_sym_DASH] = ACTIONS(3190),
    [anon_sym_AMP] = ACTIONS(3190),
    [anon_sym_PIPE] = ACTIONS(3190),
    [anon_sym_LT] = ACTIONS(3190),
    [anon_sym_DOT_DOT] = ACTIONS(3190),
    [anon_sym_COLON_COLON] = ACTIONS(3190),
    [anon_sym_POUND] = ACTIONS(3190),
    [anon_sym_SQUOTE] = ACTIONS(3192),
    [anon_sym_async] = ACTIONS(3192),
    [anon_sym_auto] = ACTIONS(3192),
    [anon_sym_await] = ACTIONS(3192),
    [anon_sym_become] = ACTIONS(3192),
    [anon_sym_break] = ACTIONS(3192),
    [anon_sym_const] = ACTIONS(3192),
    [anon_sym_continue] = ACTIONS(3192),
    [anon_sym_default] = ACTIONS(3192),
    [anon_sym_enum] = ACTIONS(3192),
    [anon_sym_fn] = ACTIONS(3192),
    [anon_sym_for] = ACTIONS(3192),
    [anon_sym_gen] = ACTIONS(3192),
    [anon_sym_if] = ACTIONS(3192),
    [anon_sym_impl] = ACTIONS(3192),
    [anon_sym_let] = ACTIONS(3192),
    [anon_sym_loop] = ACTIONS(3192),
    [anon_sym_match] = ACTIONS(3192),
    [anon_sym_mod] = ACTIONS(3192),
    [anon_sym_pub] = ACTIONS(3192),
    [anon_sym_return] = ACTIONS(3192),
    [anon_sym_raw] = ACTIONS(3192),
    [anon_sym_static] = ACTIONS(3192),
    [anon_sym_struct] = ACTIONS(3192),
    [anon_sym_trait] = ACTIONS(3192),
    [anon_sym_type] = ACTIONS(3192),
    [anon_sym_union] = ACTIONS(3192),
    [anon_sym_unsafe] = ACTIONS(3192),
    [anon_sym_use] = ACTIONS(3192),
    [anon_sym_while] = ACTIONS(3192),
    [anon_sym_extern] = ACTIONS(3192),
    [anon_sym_safe] = ACTIONS(3192),
    [anon_sym_yield] = ACTIONS(3192),
    [anon_sym_move] = ACTIONS(3192),
    [anon_sym_try] = ACTIONS(3192),
    [sym_integer_literal] = ACTIONS(3190),
    [aux_sym_string_literal_token1] = ACTIONS(3190),
    [sym_char_literal] = ACTIONS(3190),
    [anon_sym_true] = ACTIONS(3192),
    [anon_sym_false] = ACTIONS(3192),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3192),
    [sym_super] = ACTIONS(3192),
    [sym_crate] = ACTIONS(3192),
    [aux_sym_metavariable_token1] = ACTIONS(3190),
    [sym__raw_string_literal_start] = ACTIONS(3190),
    [sym_float_literal] = ACTIONS(3190),
    [sym__outer_block_doc_comment_start] = ACTIONS(3190),
    [sym__outer_line_doc_comment_start] = ACTIONS(3190),
  },
  [STATE(856)] = {
    [sym_line_comment] = STATE(856),
    [sym_block_comment] = STATE(856),
    [ts_builtin_sym_end] = ACTIONS(3194),
    [sym_identifier] = ACTIONS(3196),
    [anon_sym_SEMI] = ACTIONS(3194),
    [anon_sym_macro] = ACTIONS(3196),
    [anon_sym_LPAREN] = ACTIONS(3194),
    [anon_sym_LBRACE] = ACTIONS(3194),
    [anon_sym_RBRACE] = ACTIONS(3194),
    [anon_sym_macro_rules] = ACTIONS(3196),
    [anon_sym_BANG] = ACTIONS(3194),
    [anon_sym_LBRACK] = ACTIONS(3194),
    [anon_sym_STAR] = ACTIONS(3194),
    [anon_sym_u8] = ACTIONS(3196),
    [anon_sym_i8] = ACTIONS(3196),
    [anon_sym_u16] = ACTIONS(3196),
    [anon_sym_i16] = ACTIONS(3196),
    [anon_sym_u32] = ACTIONS(3196),
    [anon_sym_i32] = ACTIONS(3196),
    [anon_sym_u64] = ACTIONS(3196),
    [anon_sym_i64] = ACTIONS(3196),
    [anon_sym_u128] = ACTIONS(3196),
    [anon_sym_i128] = ACTIONS(3196),
    [anon_sym_isize] = ACTIONS(3196),
    [anon_sym_usize] = ACTIONS(3196),
    [anon_sym_f32] = ACTIONS(3196),
    [anon_sym_f64] = ACTIONS(3196),
    [anon_sym_bool] = ACTIONS(3196),
    [anon_sym_str] = ACTIONS(3196),
    [anon_sym_char] = ACTIONS(3196),
    [anon_sym_DASH] = ACTIONS(3194),
    [anon_sym_AMP] = ACTIONS(3194),
    [anon_sym_PIPE] = ACTIONS(3194),
    [anon_sym_LT] = ACTIONS(3194),
    [anon_sym_DOT_DOT] = ACTIONS(3194),
    [anon_sym_COLON_COLON] = ACTIONS(3194),
    [anon_sym_POUND] = ACTIONS(3194),
    [anon_sym_SQUOTE] = ACTIONS(3196),
    [anon_sym_async] = ACTIONS(3196),
    [anon_sym_auto] = ACTIONS(3196),
    [anon_sym_await] = ACTIONS(3196),
    [anon_sym_become] = ACTIONS(3196),
    [anon_sym_break] = ACTIONS(3196),
    [anon_sym_const] = ACTIONS(3196),
    [anon_sym_continue] = ACTIONS(3196),
    [anon_sym_default] = ACTIONS(3196),
    [anon_sym_enum] = ACTIONS(3196),
    [anon_sym_fn] = ACTIONS(3196),
    [anon_sym_for] = ACTIONS(3196),
    [anon_sym_gen] = ACTIONS(3196),
    [anon_sym_if] = ACTIONS(3196),
    [anon_sym_impl] = ACTIONS(3196),
    [anon_sym_let] = ACTIONS(3196),
    [anon_sym_loop] = ACTIONS(3196),
    [anon_sym_match] = ACTIONS(3196),
    [anon_sym_mod] = ACTIONS(3196),
    [anon_sym_pub] = ACTIONS(3196),
    [anon_sym_return] = ACTIONS(3196),
    [anon_sym_raw] = ACTIONS(3196),
    [anon_sym_static] = ACTIONS(3196),
    [anon_sym_struct] = ACTIONS(3196),
    [anon_sym_trait] = ACTIONS(3196),
    [anon_sym_type] = ACTIONS(3196),
    [anon_sym_union] = ACTIONS(3196),
    [anon_sym_unsafe] = ACTIONS(3196),
    [anon_sym_use] = ACTIONS(3196),
    [anon_sym_while] = ACTIONS(3196),
    [anon_sym_extern] = ACTIONS(3196),
    [anon_sym_safe] = ACTIONS(3196),
    [anon_sym_yield] = ACTIONS(3196),
    [anon_sym_move] = ACTIONS(3196),
    [anon_sym_try] = ACTIONS(3196),
    [sym_integer_literal] = ACTIONS(3194),
    [aux_sym_string_literal_token1] = ACTIONS(3194),
    [sym_char_literal] = ACTIONS(3194),
    [anon_sym_true] = ACTIONS(3196),
    [anon_sym_false] = ACTIONS(3196),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3196),
    [sym_super] = ACTIONS(3196),
    [sym_crate] = ACTIONS(3196),
    [aux_sym_metavariable_token1] = ACTIONS(3194),
    [sym__raw_string_literal_start] = ACTIONS(3194),
    [sym_float_literal] = ACTIONS(3194),
    [sym__outer_block_doc_comment_start] = ACTIONS(3194),
    [sym__outer_line_doc_comment_start] = ACTIONS(3194),
  },
  [STATE(857)] = {
    [sym_line_comment] = STATE(857),
    [sym_block_comment] = STATE(857),
    [ts_builtin_sym_end] = ACTIONS(3198),
    [sym_identifier] = ACTIONS(3200),
    [anon_sym_SEMI] = ACTIONS(3198),
    [anon_sym_macro] = ACTIONS(3200),
    [anon_sym_LPAREN] = ACTIONS(3198),
    [anon_sym_LBRACE] = ACTIONS(3198),
    [anon_sym_RBRACE] = ACTIONS(3198),
    [anon_sym_macro_rules] = ACTIONS(3200),
    [anon_sym_BANG] = ACTIONS(3198),
    [anon_sym_LBRACK] = ACTIONS(3198),
    [anon_sym_STAR] = ACTIONS(3198),
    [anon_sym_u8] = ACTIONS(3200),
    [anon_sym_i8] = ACTIONS(3200),
    [anon_sym_u16] = ACTIONS(3200),
    [anon_sym_i16] = ACTIONS(3200),
    [anon_sym_u32] = ACTIONS(3200),
    [anon_sym_i32] = ACTIONS(3200),
    [anon_sym_u64] = ACTIONS(3200),
    [anon_sym_i64] = ACTIONS(3200),
    [anon_sym_u128] = ACTIONS(3200),
    [anon_sym_i128] = ACTIONS(3200),
    [anon_sym_isize] = ACTIONS(3200),
    [anon_sym_usize] = ACTIONS(3200),
    [anon_sym_f32] = ACTIONS(3200),
    [anon_sym_f64] = ACTIONS(3200),
    [anon_sym_bool] = ACTIONS(3200),
    [anon_sym_str] = ACTIONS(3200),
    [anon_sym_char] = ACTIONS(3200),
    [anon_sym_DASH] = ACTIONS(3198),
    [anon_sym_AMP] = ACTIONS(3198),
    [anon_sym_PIPE] = ACTIONS(3198),
    [anon_sym_LT] = ACTIONS(3198),
    [anon_sym_DOT_DOT] = ACTIONS(3198),
    [anon_sym_COLON_COLON] = ACTIONS(3198),
    [anon_sym_POUND] = ACTIONS(3198),
    [anon_sym_SQUOTE] = ACTIONS(3200),
    [anon_sym_async] = ACTIONS(3200),
    [anon_sym_auto] = ACTIONS(3200),
    [anon_sym_await] = ACTIONS(3200),
    [anon_sym_become] = ACTIONS(3200),
    [anon_sym_break] = ACTIONS(3200),
    [anon_sym_const] = ACTIONS(3200),
    [anon_sym_continue] = ACTIONS(3200),
    [anon_sym_default] = ACTIONS(3200),
    [anon_sym_enum] = ACTIONS(3200),
    [anon_sym_fn] = ACTIONS(3200),
    [anon_sym_for] = ACTIONS(3200),
    [anon_sym_gen] = ACTIONS(3200),
    [anon_sym_if] = ACTIONS(3200),
    [anon_sym_impl] = ACTIONS(3200),
    [anon_sym_let] = ACTIONS(3200),
    [anon_sym_loop] = ACTIONS(3200),
    [anon_sym_match] = ACTIONS(3200),
    [anon_sym_mod] = ACTIONS(3200),
    [anon_sym_pub] = ACTIONS(3200),
    [anon_sym_return] = ACTIONS(3200),
    [anon_sym_raw] = ACTIONS(3200),
    [anon_sym_static] = ACTIONS(3200),
    [anon_sym_struct] = ACTIONS(3200),
    [anon_sym_trait] = ACTIONS(3200),
    [anon_sym_type] = ACTIONS(3200),
    [anon_sym_union] = ACTIONS(3200),
    [anon_sym_unsafe] = ACTIONS(3200),
    [anon_sym_use] = ACTIONS(3200),
    [anon_sym_while] = ACTIONS(3200),
    [anon_sym_extern] = ACTIONS(3200),
    [anon_sym_safe] = ACTIONS(3200),
    [anon_sym_yield] = ACTIONS(3200),
    [anon_sym_move] = ACTIONS(3200),
    [anon_sym_try] = ACTIONS(3200),
    [sym_integer_literal] = ACTIONS(3198),
    [aux_sym_string_literal_token1] = ACTIONS(3198),
    [sym_char_literal] = ACTIONS(3198),
    [anon_sym_true] = ACTIONS(3200),
    [anon_sym_false] = ACTIONS(3200),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3200),
    [sym_super] = ACTIONS(3200),
    [sym_crate] = ACTIONS(3200),
    [aux_sym_metavariable_token1] = ACTIONS(3198),
    [sym__raw_string_literal_start] = ACTIONS(3198),
    [sym_float_literal] = ACTIONS(3198),
    [sym__outer_block_doc_comment_start] = ACTIONS(3198),
    [sym__outer_line_doc_comment_start] = ACTIONS(3198),
  },
  [STATE(858)] = {
    [sym_line_comment] = STATE(858),
    [sym_block_comment] = STATE(858),
    [ts_builtin_sym_end] = ACTIONS(3202),
    [sym_identifier] = ACTIONS(3204),
    [anon_sym_SEMI] = ACTIONS(3202),
    [anon_sym_macro] = ACTIONS(3204),
    [anon_sym_LPAREN] = ACTIONS(3202),
    [anon_sym_LBRACE] = ACTIONS(3202),
    [anon_sym_RBRACE] = ACTIONS(3202),
    [anon_sym_macro_rules] = ACTIONS(3204),
    [anon_sym_BANG] = ACTIONS(3202),
    [anon_sym_LBRACK] = ACTIONS(3202),
    [anon_sym_STAR] = ACTIONS(3202),
    [anon_sym_u8] = ACTIONS(3204),
    [anon_sym_i8] = ACTIONS(3204),
    [anon_sym_u16] = ACTIONS(3204),
    [anon_sym_i16] = ACTIONS(3204),
    [anon_sym_u32] = ACTIONS(3204),
    [anon_sym_i32] = ACTIONS(3204),
    [anon_sym_u64] = ACTIONS(3204),
    [anon_sym_i64] = ACTIONS(3204),
    [anon_sym_u128] = ACTIONS(3204),
    [anon_sym_i128] = ACTIONS(3204),
    [anon_sym_isize] = ACTIONS(3204),
    [anon_sym_usize] = ACTIONS(3204),
    [anon_sym_f32] = ACTIONS(3204),
    [anon_sym_f64] = ACTIONS(3204),
    [anon_sym_bool] = ACTIONS(3204),
    [anon_sym_str] = ACTIONS(3204),
    [anon_sym_char] = ACTIONS(3204),
    [anon_sym_DASH] = ACTIONS(3202),
    [anon_sym_AMP] = ACTIONS(3202),
    [anon_sym_PIPE] = ACTIONS(3202),
    [anon_sym_LT] = ACTIONS(3202),
    [anon_sym_DOT_DOT] = ACTIONS(3202),
    [anon_sym_COLON_COLON] = ACTIONS(3202),
    [anon_sym_POUND] = ACTIONS(3202),
    [anon_sym_SQUOTE] = ACTIONS(3204),
    [anon_sym_async] = ACTIONS(3204),
    [anon_sym_auto] = ACTIONS(3204),
    [anon_sym_await] = ACTIONS(3204),
    [anon_sym_become] = ACTIONS(3204),
    [anon_sym_break] = ACTIONS(3204),
    [anon_sym_const] = ACTIONS(3204),
    [anon_sym_continue] = ACTIONS(3204),
    [anon_sym_default] = ACTIONS(3204),
    [anon_sym_enum] = ACTIONS(3204),
    [anon_sym_fn] = ACTIONS(3204),
    [anon_sym_for] = ACTIONS(3204),
    [anon_sym_gen] = ACTIONS(3204),
    [anon_sym_if] = ACTIONS(3204),
    [anon_sym_impl] = ACTIONS(3204),
    [anon_sym_let] = ACTIONS(3204),
    [anon_sym_loop] = ACTIONS(3204),
    [anon_sym_match] = ACTIONS(3204),
    [anon_sym_mod] = ACTIONS(3204),
    [anon_sym_pub] = ACTIONS(3204),
    [anon_sym_return] = ACTIONS(3204),
    [anon_sym_raw] = ACTIONS(3204),
    [anon_sym_static] = ACTIONS(3204),
    [anon_sym_struct] = ACTIONS(3204),
    [anon_sym_trait] = ACTIONS(3204),
    [anon_sym_type] = ACTIONS(3204),
    [anon_sym_union] = ACTIONS(3204),
    [anon_sym_unsafe] = ACTIONS(3204),
    [anon_sym_use] = ACTIONS(3204),
    [anon_sym_while] = ACTIONS(3204),
    [anon_sym_extern] = ACTIONS(3204),
    [anon_sym_safe] = ACTIONS(3204),
    [anon_sym_yield] = ACTIONS(3204),
    [anon_sym_move] = ACTIONS(3204),
    [anon_sym_try] = ACTIONS(3204),
    [sym_integer_literal] = ACTIONS(3202),
    [aux_sym_string_literal_token1] = ACTIONS(3202),
    [sym_char_literal] = ACTIONS(3202),
    [anon_sym_true] = ACTIONS(3204),
    [anon_sym_false] = ACTIONS(3204),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3204),
    [sym_super] = ACTIONS(3204),
    [sym_crate] = ACTIONS(3204),
    [aux_sym_metavariable_token1] = ACTIONS(3202),
    [sym__raw_string_literal_start] = ACTIONS(3202),
    [sym_float_literal] = ACTIONS(3202),
    [sym__outer_block_doc_comment_start] = ACTIONS(3202),
    [sym__outer_line_doc_comment_start] = ACTIONS(3202),
  },
  [STATE(859)] = {
    [sym_line_comment] = STATE(859),
    [sym_block_comment] = STATE(859),
    [ts_builtin_sym_end] = ACTIONS(3206),
    [sym_identifier] = ACTIONS(3208),
    [anon_sym_SEMI] = ACTIONS(3206),
    [anon_sym_macro] = ACTIONS(3208),
    [anon_sym_LPAREN] = ACTIONS(3206),
    [anon_sym_LBRACE] = ACTIONS(3206),
    [anon_sym_RBRACE] = ACTIONS(3206),
    [anon_sym_macro_rules] = ACTIONS(3208),
    [anon_sym_BANG] = ACTIONS(3206),
    [anon_sym_LBRACK] = ACTIONS(3206),
    [anon_sym_STAR] = ACTIONS(3206),
    [anon_sym_u8] = ACTIONS(3208),
    [anon_sym_i8] = ACTIONS(3208),
    [anon_sym_u16] = ACTIONS(3208),
    [anon_sym_i16] = ACTIONS(3208),
    [anon_sym_u32] = ACTIONS(3208),
    [anon_sym_i32] = ACTIONS(3208),
    [anon_sym_u64] = ACTIONS(3208),
    [anon_sym_i64] = ACTIONS(3208),
    [anon_sym_u128] = ACTIONS(3208),
    [anon_sym_i128] = ACTIONS(3208),
    [anon_sym_isize] = ACTIONS(3208),
    [anon_sym_usize] = ACTIONS(3208),
    [anon_sym_f32] = ACTIONS(3208),
    [anon_sym_f64] = ACTIONS(3208),
    [anon_sym_bool] = ACTIONS(3208),
    [anon_sym_str] = ACTIONS(3208),
    [anon_sym_char] = ACTIONS(3208),
    [anon_sym_DASH] = ACTIONS(3206),
    [anon_sym_AMP] = ACTIONS(3206),
    [anon_sym_PIPE] = ACTIONS(3206),
    [anon_sym_LT] = ACTIONS(3206),
    [anon_sym_DOT_DOT] = ACTIONS(3206),
    [anon_sym_COLON_COLON] = ACTIONS(3206),
    [anon_sym_POUND] = ACTIONS(3206),
    [anon_sym_SQUOTE] = ACTIONS(3208),
    [anon_sym_async] = ACTIONS(3208),
    [anon_sym_auto] = ACTIONS(3208),
    [anon_sym_await] = ACTIONS(3208),
    [anon_sym_become] = ACTIONS(3208),
    [anon_sym_break] = ACTIONS(3208),
    [anon_sym_const] = ACTIONS(3208),
    [anon_sym_continue] = ACTIONS(3208),
    [anon_sym_default] = ACTIONS(3208),
    [anon_sym_enum] = ACTIONS(3208),
    [anon_sym_fn] = ACTIONS(3208),
    [anon_sym_for] = ACTIONS(3208),
    [anon_sym_gen] = ACTIONS(3208),
    [anon_sym_if] = ACTIONS(3208),
    [anon_sym_impl] = ACTIONS(3208),
    [anon_sym_let] = ACTIONS(3208),
    [anon_sym_loop] = ACTIONS(3208),
    [anon_sym_match] = ACTIONS(3208),
    [anon_sym_mod] = ACTIONS(3208),
    [anon_sym_pub] = ACTIONS(3208),
    [anon_sym_return] = ACTIONS(3208),
    [anon_sym_raw] = ACTIONS(3208),
    [anon_sym_static] = ACTIONS(3208),
    [anon_sym_struct] = ACTIONS(3208),
    [anon_sym_trait] = ACTIONS(3208),
    [anon_sym_type] = ACTIONS(3208),
    [anon_sym_union] = ACTIONS(3208),
    [anon_sym_unsafe] = ACTIONS(3208),
    [anon_sym_use] = ACTIONS(3208),
    [anon_sym_while] = ACTIONS(3208),
    [anon_sym_extern] = ACTIONS(3208),
    [anon_sym_safe] = ACTIONS(3208),
    [anon_sym_yield] = ACTIONS(3208),
    [anon_sym_move] = ACTIONS(3208),
    [anon_sym_try] = ACTIONS(3208),
    [sym_integer_literal] = ACTIONS(3206),
    [aux_sym_string_literal_token1] = ACTIONS(3206),
    [sym_char_literal] = ACTIONS(3206),
    [anon_sym_true] = ACTIONS(3208),
    [anon_sym_false] = ACTIONS(3208),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3208),
    [sym_super] = ACTIONS(3208),
    [sym_crate] = ACTIONS(3208),
    [aux_sym_metavariable_token1] = ACTIONS(3206),
    [sym__raw_string_literal_start] = ACTIONS(3206),
    [sym_float_literal] = ACTIONS(3206),
    [sym__outer_block_doc_comment_start] = ACTIONS(3206),
    [sym__outer_line_doc_comment_start] = ACTIONS(3206),
  },
  [STATE(860)] = {
    [sym_line_comment] = STATE(860),
    [sym_block_comment] = STATE(860),
    [ts_builtin_sym_end] = ACTIONS(3210),
    [sym_identifier] = ACTIONS(3212),
    [anon_sym_SEMI] = ACTIONS(3210),
    [anon_sym_macro] = ACTIONS(3212),
    [anon_sym_LPAREN] = ACTIONS(3210),
    [anon_sym_LBRACE] = ACTIONS(3210),
    [anon_sym_RBRACE] = ACTIONS(3210),
    [anon_sym_macro_rules] = ACTIONS(3212),
    [anon_sym_BANG] = ACTIONS(3210),
    [anon_sym_LBRACK] = ACTIONS(3210),
    [anon_sym_STAR] = ACTIONS(3210),
    [anon_sym_u8] = ACTIONS(3212),
    [anon_sym_i8] = ACTIONS(3212),
    [anon_sym_u16] = ACTIONS(3212),
    [anon_sym_i16] = ACTIONS(3212),
    [anon_sym_u32] = ACTIONS(3212),
    [anon_sym_i32] = ACTIONS(3212),
    [anon_sym_u64] = ACTIONS(3212),
    [anon_sym_i64] = ACTIONS(3212),
    [anon_sym_u128] = ACTIONS(3212),
    [anon_sym_i128] = ACTIONS(3212),
    [anon_sym_isize] = ACTIONS(3212),
    [anon_sym_usize] = ACTIONS(3212),
    [anon_sym_f32] = ACTIONS(3212),
    [anon_sym_f64] = ACTIONS(3212),
    [anon_sym_bool] = ACTIONS(3212),
    [anon_sym_str] = ACTIONS(3212),
    [anon_sym_char] = ACTIONS(3212),
    [anon_sym_DASH] = ACTIONS(3210),
    [anon_sym_AMP] = ACTIONS(3210),
    [anon_sym_PIPE] = ACTIONS(3210),
    [anon_sym_LT] = ACTIONS(3210),
    [anon_sym_DOT_DOT] = ACTIONS(3210),
    [anon_sym_COLON_COLON] = ACTIONS(3210),
    [anon_sym_POUND] = ACTIONS(3210),
    [anon_sym_SQUOTE] = ACTIONS(3212),
    [anon_sym_async] = ACTIONS(3212),
    [anon_sym_auto] = ACTIONS(3212),
    [anon_sym_await] = ACTIONS(3212),
    [anon_sym_become] = ACTIONS(3212),
    [anon_sym_break] = ACTIONS(3212),
    [anon_sym_const] = ACTIONS(3212),
    [anon_sym_continue] = ACTIONS(3212),
    [anon_sym_default] = ACTIONS(3212),
    [anon_sym_enum] = ACTIONS(3212),
    [anon_sym_fn] = ACTIONS(3212),
    [anon_sym_for] = ACTIONS(3212),
    [anon_sym_gen] = ACTIONS(3212),
    [anon_sym_if] = ACTIONS(3212),
    [anon_sym_impl] = ACTIONS(3212),
    [anon_sym_let] = ACTIONS(3212),
    [anon_sym_loop] = ACTIONS(3212),
    [anon_sym_match] = ACTIONS(3212),
    [anon_sym_mod] = ACTIONS(3212),
    [anon_sym_pub] = ACTIONS(3212),
    [anon_sym_return] = ACTIONS(3212),
    [anon_sym_raw] = ACTIONS(3212),
    [anon_sym_static] = ACTIONS(3212),
    [anon_sym_struct] = ACTIONS(3212),
    [anon_sym_trait] = ACTIONS(3212),
    [anon_sym_type] = ACTIONS(3212),
    [anon_sym_union] = ACTIONS(3212),
    [anon_sym_unsafe] = ACTIONS(3212),
    [anon_sym_use] = ACTIONS(3212),
    [anon_sym_while] = ACTIONS(3212),
    [anon_sym_extern] = ACTIONS(3212),
    [anon_sym_safe] = ACTIONS(3212),
    [anon_sym_yield] = ACTIONS(3212),
    [anon_sym_move] = ACTIONS(3212),
    [anon_sym_try] = ACTIONS(3212),
    [sym_integer_literal] = ACTIONS(3210),
    [aux_sym_string_literal_token1] = ACTIONS(3210),
    [sym_char_literal] = ACTIONS(3210),
    [anon_sym_true] = ACTIONS(3212),
    [anon_sym_false] = ACTIONS(3212),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3212),
    [sym_super] = ACTIONS(3212),
    [sym_crate] = ACTIONS(3212),
    [aux_sym_metavariable_token1] = ACTIONS(3210),
    [sym__raw_string_literal_start] = ACTIONS(3210),
    [sym_float_literal] = ACTIONS(3210),
    [sym__outer_block_doc_comment_start] = ACTIONS(3210),
    [sym__outer_line_doc_comment_start] = ACTIONS(3210),
  },
  [STATE(861)] = {
    [sym_line_comment] = STATE(861),
    [sym_block_comment] = STATE(861),
    [ts_builtin_sym_end] = ACTIONS(3214),
    [sym_identifier] = ACTIONS(3216),
    [anon_sym_SEMI] = ACTIONS(3214),
    [anon_sym_macro] = ACTIONS(3216),
    [anon_sym_LPAREN] = ACTIONS(3214),
    [anon_sym_LBRACE] = ACTIONS(3214),
    [anon_sym_RBRACE] = ACTIONS(3214),
    [anon_sym_macro_rules] = ACTIONS(3216),
    [anon_sym_BANG] = ACTIONS(3214),
    [anon_sym_LBRACK] = ACTIONS(3214),
    [anon_sym_STAR] = ACTIONS(3214),
    [anon_sym_u8] = ACTIONS(3216),
    [anon_sym_i8] = ACTIONS(3216),
    [anon_sym_u16] = ACTIONS(3216),
    [anon_sym_i16] = ACTIONS(3216),
    [anon_sym_u32] = ACTIONS(3216),
    [anon_sym_i32] = ACTIONS(3216),
    [anon_sym_u64] = ACTIONS(3216),
    [anon_sym_i64] = ACTIONS(3216),
    [anon_sym_u128] = ACTIONS(3216),
    [anon_sym_i128] = ACTIONS(3216),
    [anon_sym_isize] = ACTIONS(3216),
    [anon_sym_usize] = ACTIONS(3216),
    [anon_sym_f32] = ACTIONS(3216),
    [anon_sym_f64] = ACTIONS(3216),
    [anon_sym_bool] = ACTIONS(3216),
    [anon_sym_str] = ACTIONS(3216),
    [anon_sym_char] = ACTIONS(3216),
    [anon_sym_DASH] = ACTIONS(3214),
    [anon_sym_AMP] = ACTIONS(3214),
    [anon_sym_PIPE] = ACTIONS(3214),
    [anon_sym_LT] = ACTIONS(3214),
    [anon_sym_DOT_DOT] = ACTIONS(3214),
    [anon_sym_COLON_COLON] = ACTIONS(3214),
    [anon_sym_POUND] = ACTIONS(3214),
    [anon_sym_SQUOTE] = ACTIONS(3216),
    [anon_sym_async] = ACTIONS(3216),
    [anon_sym_auto] = ACTIONS(3216),
    [anon_sym_await] = ACTIONS(3216),
    [anon_sym_become] = ACTIONS(3216),
    [anon_sym_break] = ACTIONS(3216),
    [anon_sym_const] = ACTIONS(3216),
    [anon_sym_continue] = ACTIONS(3216),
    [anon_sym_default] = ACTIONS(3216),
    [anon_sym_enum] = ACTIONS(3216),
    [anon_sym_fn] = ACTIONS(3216),
    [anon_sym_for] = ACTIONS(3216),
    [anon_sym_gen] = ACTIONS(3216),
    [anon_sym_if] = ACTIONS(3216),
    [anon_sym_impl] = ACTIONS(3216),
    [anon_sym_let] = ACTIONS(3216),
    [anon_sym_loop] = ACTIONS(3216),
    [anon_sym_match] = ACTIONS(3216),
    [anon_sym_mod] = ACTIONS(3216),
    [anon_sym_pub] = ACTIONS(3216),
    [anon_sym_return] = ACTIONS(3216),
    [anon_sym_raw] = ACTIONS(3216),
    [anon_sym_static] = ACTIONS(3216),
    [anon_sym_struct] = ACTIONS(3216),
    [anon_sym_trait] = ACTIONS(3216),
    [anon_sym_type] = ACTIONS(3216),
    [anon_sym_union] = ACTIONS(3216),
    [anon_sym_unsafe] = ACTIONS(3216),
    [anon_sym_use] = ACTIONS(3216),
    [anon_sym_while] = ACTIONS(3216),
    [anon_sym_extern] = ACTIONS(3216),
    [anon_sym_safe] = ACTIONS(3216),
    [anon_sym_yield] = ACTIONS(3216),
    [anon_sym_move] = ACTIONS(3216),
    [anon_sym_try] = ACTIONS(3216),
    [sym_integer_literal] = ACTIONS(3214),
    [aux_sym_string_literal_token1] = ACTIONS(3214),
    [sym_char_literal] = ACTIONS(3214),
    [anon_sym_true] = ACTIONS(3216),
    [anon_sym_false] = ACTIONS(3216),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3216),
    [sym_super] = ACTIONS(3216),
    [sym_crate] = ACTIONS(3216),
    [aux_sym_metavariable_token1] = ACTIONS(3214),
    [sym__raw_string_literal_start] = ACTIONS(3214),
    [sym_float_literal] = ACTIONS(3214),
    [sym__outer_block_doc_comment_start] = ACTIONS(3214),
    [sym__outer_line_doc_comment_start] = ACTIONS(3214),
  },
  [STATE(862)] = {
    [sym_line_comment] = STATE(862),
    [sym_block_comment] = STATE(862),
    [ts_builtin_sym_end] = ACTIONS(3218),
    [sym_identifier] = ACTIONS(3220),
    [anon_sym_SEMI] = ACTIONS(3218),
    [anon_sym_macro] = ACTIONS(3220),
    [anon_sym_LPAREN] = ACTIONS(3218),
    [anon_sym_LBRACE] = ACTIONS(3218),
    [anon_sym_RBRACE] = ACTIONS(3218),
    [anon_sym_macro_rules] = ACTIONS(3220),
    [anon_sym_BANG] = ACTIONS(3218),
    [anon_sym_LBRACK] = ACTIONS(3218),
    [anon_sym_STAR] = ACTIONS(3218),
    [anon_sym_u8] = ACTIONS(3220),
    [anon_sym_i8] = ACTIONS(3220),
    [anon_sym_u16] = ACTIONS(3220),
    [anon_sym_i16] = ACTIONS(3220),
    [anon_sym_u32] = ACTIONS(3220),
    [anon_sym_i32] = ACTIONS(3220),
    [anon_sym_u64] = ACTIONS(3220),
    [anon_sym_i64] = ACTIONS(3220),
    [anon_sym_u128] = ACTIONS(3220),
    [anon_sym_i128] = ACTIONS(3220),
    [anon_sym_isize] = ACTIONS(3220),
    [anon_sym_usize] = ACTIONS(3220),
    [anon_sym_f32] = ACTIONS(3220),
    [anon_sym_f64] = ACTIONS(3220),
    [anon_sym_bool] = ACTIONS(3220),
    [anon_sym_str] = ACTIONS(3220),
    [anon_sym_char] = ACTIONS(3220),
    [anon_sym_DASH] = ACTIONS(3218),
    [anon_sym_AMP] = ACTIONS(3218),
    [anon_sym_PIPE] = ACTIONS(3218),
    [anon_sym_LT] = ACTIONS(3218),
    [anon_sym_DOT_DOT] = ACTIONS(3218),
    [anon_sym_COLON_COLON] = ACTIONS(3218),
    [anon_sym_POUND] = ACTIONS(3218),
    [anon_sym_SQUOTE] = ACTIONS(3220),
    [anon_sym_async] = ACTIONS(3220),
    [anon_sym_auto] = ACTIONS(3220),
    [anon_sym_await] = ACTIONS(3220),
    [anon_sym_become] = ACTIONS(3220),
    [anon_sym_break] = ACTIONS(3220),
    [anon_sym_const] = ACTIONS(3220),
    [anon_sym_continue] = ACTIONS(3220),
    [anon_sym_default] = ACTIONS(3220),
    [anon_sym_enum] = ACTIONS(3220),
    [anon_sym_fn] = ACTIONS(3220),
    [anon_sym_for] = ACTIONS(3220),
    [anon_sym_gen] = ACTIONS(3220),
    [anon_sym_if] = ACTIONS(3220),
    [anon_sym_impl] = ACTIONS(3220),
    [anon_sym_let] = ACTIONS(3220),
    [anon_sym_loop] = ACTIONS(3220),
    [anon_sym_match] = ACTIONS(3220),
    [anon_sym_mod] = ACTIONS(3220),
    [anon_sym_pub] = ACTIONS(3220),
    [anon_sym_return] = ACTIONS(3220),
    [anon_sym_raw] = ACTIONS(3220),
    [anon_sym_static] = ACTIONS(3220),
    [anon_sym_struct] = ACTIONS(3220),
    [anon_sym_trait] = ACTIONS(3220),
    [anon_sym_type] = ACTIONS(3220),
    [anon_sym_union] = ACTIONS(3220),
    [anon_sym_unsafe] = ACTIONS(3220),
    [anon_sym_use] = ACTIONS(3220),
    [anon_sym_while] = ACTIONS(3220),
    [anon_sym_extern] = ACTIONS(3220),
    [anon_sym_safe] = ACTIONS(3220),
    [anon_sym_yield] = ACTIONS(3220),
    [anon_sym_move] = ACTIONS(3220),
    [anon_sym_try] = ACTIONS(3220),
    [sym_integer_literal] = ACTIONS(3218),
    [aux_sym_string_literal_token1] = ACTIONS(3218),
    [sym_char_literal] = ACTIONS(3218),
    [anon_sym_true] = ACTIONS(3220),
    [anon_sym_false] = ACTIONS(3220),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3220),
    [sym_super] = ACTIONS(3220),
    [sym_crate] = ACTIONS(3220),
    [aux_sym_metavariable_token1] = ACTIONS(3218),
    [sym__raw_string_literal_start] = ACTIONS(3218),
    [sym_float_literal] = ACTIONS(3218),
    [sym__outer_block_doc_comment_start] = ACTIONS(3218),
    [sym__outer_line_doc_comment_start] = ACTIONS(3218),
  },
  [STATE(863)] = {
    [sym_line_comment] = STATE(863),
    [sym_block_comment] = STATE(863),
    [ts_builtin_sym_end] = ACTIONS(3222),
    [sym_identifier] = ACTIONS(3224),
    [anon_sym_SEMI] = ACTIONS(3222),
    [anon_sym_macro] = ACTIONS(3224),
    [anon_sym_LPAREN] = ACTIONS(3222),
    [anon_sym_LBRACE] = ACTIONS(3222),
    [anon_sym_RBRACE] = ACTIONS(3222),
    [anon_sym_macro_rules] = ACTIONS(3224),
    [anon_sym_BANG] = ACTIONS(3222),
    [anon_sym_LBRACK] = ACTIONS(3222),
    [anon_sym_STAR] = ACTIONS(3222),
    [anon_sym_u8] = ACTIONS(3224),
    [anon_sym_i8] = ACTIONS(3224),
    [anon_sym_u16] = ACTIONS(3224),
    [anon_sym_i16] = ACTIONS(3224),
    [anon_sym_u32] = ACTIONS(3224),
    [anon_sym_i32] = ACTIONS(3224),
    [anon_sym_u64] = ACTIONS(3224),
    [anon_sym_i64] = ACTIONS(3224),
    [anon_sym_u128] = ACTIONS(3224),
    [anon_sym_i128] = ACTIONS(3224),
    [anon_sym_isize] = ACTIONS(3224),
    [anon_sym_usize] = ACTIONS(3224),
    [anon_sym_f32] = ACTIONS(3224),
    [anon_sym_f64] = ACTIONS(3224),
    [anon_sym_bool] = ACTIONS(3224),
    [anon_sym_str] = ACTIONS(3224),
    [anon_sym_char] = ACTIONS(3224),
    [anon_sym_DASH] = ACTIONS(3222),
    [anon_sym_AMP] = ACTIONS(3222),
    [anon_sym_PIPE] = ACTIONS(3222),
    [anon_sym_LT] = ACTIONS(3222),
    [anon_sym_DOT_DOT] = ACTIONS(3222),
    [anon_sym_COLON_COLON] = ACTIONS(3222),
    [anon_sym_POUND] = ACTIONS(3222),
    [anon_sym_SQUOTE] = ACTIONS(3224),
    [anon_sym_async] = ACTIONS(3224),
    [anon_sym_auto] = ACTIONS(3224),
    [anon_sym_await] = ACTIONS(3224),
    [anon_sym_become] = ACTIONS(3224),
    [anon_sym_break] = ACTIONS(3224),
    [anon_sym_const] = ACTIONS(3224),
    [anon_sym_continue] = ACTIONS(3224),
    [anon_sym_default] = ACTIONS(3224),
    [anon_sym_enum] = ACTIONS(3224),
    [anon_sym_fn] = ACTIONS(3224),
    [anon_sym_for] = ACTIONS(3224),
    [anon_sym_gen] = ACTIONS(3224),
    [anon_sym_if] = ACTIONS(3224),
    [anon_sym_impl] = ACTIONS(3224),
    [anon_sym_let] = ACTIONS(3224),
    [anon_sym_loop] = ACTIONS(3224),
    [anon_sym_match] = ACTIONS(3224),
    [anon_sym_mod] = ACTIONS(3224),
    [anon_sym_pub] = ACTIONS(3224),
    [anon_sym_return] = ACTIONS(3224),
    [anon_sym_raw] = ACTIONS(3224),
    [anon_sym_static] = ACTIONS(3224),
    [anon_sym_struct] = ACTIONS(3224),
    [anon_sym_trait] = ACTIONS(3224),
    [anon_sym_type] = ACTIONS(3224),
    [anon_sym_union] = ACTIONS(3224),
    [anon_sym_unsafe] = ACTIONS(3224),
    [anon_sym_use] = ACTIONS(3224),
    [anon_sym_while] = ACTIONS(3224),
    [anon_sym_extern] = ACTIONS(3224),
    [anon_sym_safe] = ACTIONS(3224),
    [anon_sym_yield] = ACTIONS(3224),
    [anon_sym_move] = ACTIONS(3224),
    [anon_sym_try] = ACTIONS(3224),
    [sym_integer_literal] = ACTIONS(3222),
    [aux_sym_string_literal_token1] = ACTIONS(3222),
    [sym_char_literal] = ACTIONS(3222),
    [anon_sym_true] = ACTIONS(3224),
    [anon_sym_false] = ACTIONS(3224),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3224),
    [sym_super] = ACTIONS(3224),
    [sym_crate] = ACTIONS(3224),
    [aux_sym_metavariable_token1] = ACTIONS(3222),
    [sym__raw_string_literal_start] = ACTIONS(3222),
    [sym_float_literal] = ACTIONS(3222),
    [sym__outer_block_doc_comment_start] = ACTIONS(3222),
    [sym__outer_line_doc_comment_start] = ACTIONS(3222),
  },
  [STATE(864)] = {
    [sym_line_comment] = STATE(864),
    [sym_block_comment] = STATE(864),
    [ts_builtin_sym_end] = ACTIONS(3226),
    [sym_identifier] = ACTIONS(3228),
    [anon_sym_SEMI] = ACTIONS(3226),
    [anon_sym_macro] = ACTIONS(3228),
    [anon_sym_LPAREN] = ACTIONS(3226),
    [anon_sym_LBRACE] = ACTIONS(3226),
    [anon_sym_RBRACE] = ACTIONS(3226),
    [anon_sym_macro_rules] = ACTIONS(3228),
    [anon_sym_BANG] = ACTIONS(3226),
    [anon_sym_LBRACK] = ACTIONS(3226),
    [anon_sym_STAR] = ACTIONS(3226),
    [anon_sym_u8] = ACTIONS(3228),
    [anon_sym_i8] = ACTIONS(3228),
    [anon_sym_u16] = ACTIONS(3228),
    [anon_sym_i16] = ACTIONS(3228),
    [anon_sym_u32] = ACTIONS(3228),
    [anon_sym_i32] = ACTIONS(3228),
    [anon_sym_u64] = ACTIONS(3228),
    [anon_sym_i64] = ACTIONS(3228),
    [anon_sym_u128] = ACTIONS(3228),
    [anon_sym_i128] = ACTIONS(3228),
    [anon_sym_isize] = ACTIONS(3228),
    [anon_sym_usize] = ACTIONS(3228),
    [anon_sym_f32] = ACTIONS(3228),
    [anon_sym_f64] = ACTIONS(3228),
    [anon_sym_bool] = ACTIONS(3228),
    [anon_sym_str] = ACTIONS(3228),
    [anon_sym_char] = ACTIONS(3228),
    [anon_sym_DASH] = ACTIONS(3226),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_PIPE] = ACTIONS(3226),
    [anon_sym_LT] = ACTIONS(3226),
    [anon_sym_DOT_DOT] = ACTIONS(3226),
    [anon_sym_COLON_COLON] = ACTIONS(3226),
    [anon_sym_POUND] = ACTIONS(3226),
    [anon_sym_SQUOTE] = ACTIONS(3228),
    [anon_sym_async] = ACTIONS(3228),
    [anon_sym_auto] = ACTIONS(3228),
    [anon_sym_await] = ACTIONS(3228),
    [anon_sym_become] = ACTIONS(3228),
    [anon_sym_break] = ACTIONS(3228),
    [anon_sym_const] = ACTIONS(3228),
    [anon_sym_continue] = ACTIONS(3228),
    [anon_sym_default] = ACTIONS(3228),
    [anon_sym_enum] = ACTIONS(3228),
    [anon_sym_fn] = ACTIONS(3228),
    [anon_sym_for] = ACTIONS(3228),
    [anon_sym_gen] = ACTIONS(3228),
    [anon_sym_if] = ACTIONS(3228),
    [anon_sym_impl] = ACTIONS(3228),
    [anon_sym_let] = ACTIONS(3228),
    [anon_sym_loop] = ACTIONS(3228),
    [anon_sym_match] = ACTIONS(3228),
    [anon_sym_mod] = ACTIONS(3228),
    [anon_sym_pub] = ACTIONS(3228),
    [anon_sym_return] = ACTIONS(3228),
    [anon_sym_raw] = ACTIONS(3228),
    [anon_sym_static] = ACTIONS(3228),
    [anon_sym_struct] = ACTIONS(3228),
    [anon_sym_trait] = ACTIONS(3228),
    [anon_sym_type] = ACTIONS(3228),
    [anon_sym_union] = ACTIONS(3228),
    [anon_sym_unsafe] = ACTIONS(3228),
    [anon_sym_use] = ACTIONS(3228),
    [anon_sym_while] = ACTIONS(3228),
    [anon_sym_extern] = ACTIONS(3228),
    [anon_sym_safe] = ACTIONS(3228),
    [anon_sym_yield] = ACTIONS(3228),
    [anon_sym_move] = ACTIONS(3228),
    [anon_sym_try] = ACTIONS(3228),
    [sym_integer_literal] = ACTIONS(3226),
    [aux_sym_string_literal_token1] = ACTIONS(3226),
    [sym_char_literal] = ACTIONS(3226),
    [anon_sym_true] = ACTIONS(3228),
    [anon_sym_false] = ACTIONS(3228),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3228),
    [sym_super] = ACTIONS(3228),
    [sym_crate] = ACTIONS(3228),
    [aux_sym_metavariable_token1] = ACTIONS(3226),
    [sym__raw_string_literal_start] = ACTIONS(3226),
    [sym_float_literal] = ACTIONS(3226),
    [sym__outer_block_doc_comment_start] = ACTIONS(3226),
    [sym__outer_line_doc_comment_start] = ACTIONS(3226),
  },
  [STATE(865)] = {
    [sym_line_comment] = STATE(865),
    [sym_block_comment] = STATE(865),
    [ts_builtin_sym_end] = ACTIONS(3230),
    [sym_identifier] = ACTIONS(3232),
    [anon_sym_SEMI] = ACTIONS(3230),
    [anon_sym_macro] = ACTIONS(3232),
    [anon_sym_LPAREN] = ACTIONS(3230),
    [anon_sym_LBRACE] = ACTIONS(3230),
    [anon_sym_RBRACE] = ACTIONS(3230),
    [anon_sym_macro_rules] = ACTIONS(3232),
    [anon_sym_BANG] = ACTIONS(3230),
    [anon_sym_LBRACK] = ACTIONS(3230),
    [anon_sym_STAR] = ACTIONS(3230),
    [anon_sym_u8] = ACTIONS(3232),
    [anon_sym_i8] = ACTIONS(3232),
    [anon_sym_u16] = ACTIONS(3232),
    [anon_sym_i16] = ACTIONS(3232),
    [anon_sym_u32] = ACTIONS(3232),
    [anon_sym_i32] = ACTIONS(3232),
    [anon_sym_u64] = ACTIONS(3232),
    [anon_sym_i64] = ACTIONS(3232),
    [anon_sym_u128] = ACTIONS(3232),
    [anon_sym_i128] = ACTIONS(3232),
    [anon_sym_isize] = ACTIONS(3232),
    [anon_sym_usize] = ACTIONS(3232),
    [anon_sym_f32] = ACTIONS(3232),
    [anon_sym_f64] = ACTIONS(3232),
    [anon_sym_bool] = ACTIONS(3232),
    [anon_sym_str] = ACTIONS(3232),
    [anon_sym_char] = ACTIONS(3232),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_AMP] = ACTIONS(3230),
    [anon_sym_PIPE] = ACTIONS(3230),
    [anon_sym_LT] = ACTIONS(3230),
    [anon_sym_DOT_DOT] = ACTIONS(3230),
    [anon_sym_COLON_COLON] = ACTIONS(3230),
    [anon_sym_POUND] = ACTIONS(3230),
    [anon_sym_SQUOTE] = ACTIONS(3232),
    [anon_sym_async] = ACTIONS(3232),
    [anon_sym_auto] = ACTIONS(3232),
    [anon_sym_await] = ACTIONS(3232),
    [anon_sym_become] = ACTIONS(3232),
    [anon_sym_break] = ACTIONS(3232),
    [anon_sym_const] = ACTIONS(3232),
    [anon_sym_continue] = ACTIONS(3232),
    [anon_sym_default] = ACTIONS(3232),
    [anon_sym_enum] = ACTIONS(3232),
    [anon_sym_fn] = ACTIONS(3232),
    [anon_sym_for] = ACTIONS(3232),
    [anon_sym_gen] = ACTIONS(3232),
    [anon_sym_if] = ACTIONS(3232),
    [anon_sym_impl] = ACTIONS(3232),
    [anon_sym_let] = ACTIONS(3232),
    [anon_sym_loop] = ACTIONS(3232),
    [anon_sym_match] = ACTIONS(3232),
    [anon_sym_mod] = ACTIONS(3232),
    [anon_sym_pub] = ACTIONS(3232),
    [anon_sym_return] = ACTIONS(3232),
    [anon_sym_raw] = ACTIONS(3232),
    [anon_sym_static] = ACTIONS(3232),
    [anon_sym_struct] = ACTIONS(3232),
    [anon_sym_trait] = ACTIONS(3232),
    [anon_sym_type] = ACTIONS(3232),
    [anon_sym_union] = ACTIONS(3232),
    [anon_sym_unsafe] = ACTIONS(3232),
    [anon_sym_use] = ACTIONS(3232),
    [anon_sym_while] = ACTIONS(3232),
    [anon_sym_extern] = ACTIONS(3232),
    [anon_sym_safe] = ACTIONS(3232),
    [anon_sym_yield] = ACTIONS(3232),
    [anon_sym_move] = ACTIONS(3232),
    [anon_sym_try] = ACTIONS(3232),
    [sym_integer_literal] = ACTIONS(3230),
    [aux_sym_string_literal_token1] = ACTIONS(3230),
    [sym_char_literal] = ACTIONS(3230),
    [anon_sym_true] = ACTIONS(3232),
    [anon_sym_false] = ACTIONS(3232),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3232),
    [sym_super] = ACTIONS(3232),
    [sym_crate] = ACTIONS(3232),
    [aux_sym_metavariable_token1] = ACTIONS(3230),
    [sym__raw_string_literal_start] = ACTIONS(3230),
    [sym_float_literal] = ACTIONS(3230),
    [sym__outer_block_doc_comment_start] = ACTIONS(3230),
    [sym__outer_line_doc_comment_start] = ACTIONS(3230),
  },
  [STATE(866)] = {
    [sym_line_comment] = STATE(866),
    [sym_block_comment] = STATE(866),
    [ts_builtin_sym_end] = ACTIONS(3234),
    [sym_identifier] = ACTIONS(3236),
    [anon_sym_SEMI] = ACTIONS(3234),
    [anon_sym_macro] = ACTIONS(3236),
    [anon_sym_LPAREN] = ACTIONS(3234),
    [anon_sym_LBRACE] = ACTIONS(3234),
    [anon_sym_RBRACE] = ACTIONS(3234),
    [anon_sym_macro_rules] = ACTIONS(3236),
    [anon_sym_BANG] = ACTIONS(3234),
    [anon_sym_LBRACK] = ACTIONS(3234),
    [anon_sym_STAR] = ACTIONS(3234),
    [anon_sym_u8] = ACTIONS(3236),
    [anon_sym_i8] = ACTIONS(3236),
    [anon_sym_u16] = ACTIONS(3236),
    [anon_sym_i16] = ACTIONS(3236),
    [anon_sym_u32] = ACTIONS(3236),
    [anon_sym_i32] = ACTIONS(3236),
    [anon_sym_u64] = ACTIONS(3236),
    [anon_sym_i64] = ACTIONS(3236),
    [anon_sym_u128] = ACTIONS(3236),
    [anon_sym_i128] = ACTIONS(3236),
    [anon_sym_isize] = ACTIONS(3236),
    [anon_sym_usize] = ACTIONS(3236),
    [anon_sym_f32] = ACTIONS(3236),
    [anon_sym_f64] = ACTIONS(3236),
    [anon_sym_bool] = ACTIONS(3236),
    [anon_sym_str] = ACTIONS(3236),
    [anon_sym_char] = ACTIONS(3236),
    [anon_sym_DASH] = ACTIONS(3234),
    [anon_sym_AMP] = ACTIONS(3234),
    [anon_sym_PIPE] = ACTIONS(3234),
    [anon_sym_LT] = ACTIONS(3234),
    [anon_sym_DOT_DOT] = ACTIONS(3234),
    [anon_sym_COLON_COLON] = ACTIONS(3234),
    [anon_sym_POUND] = ACTIONS(3234),
    [anon_sym_SQUOTE] = ACTIONS(3236),
    [anon_sym_async] = ACTIONS(3236),
    [anon_sym_auto] = ACTIONS(3236),
    [anon_sym_await] = ACTIONS(3236),
    [anon_sym_become] = ACTIONS(3236),
    [anon_sym_break] = ACTIONS(3236),
    [anon_sym_const] = ACTIONS(3236),
    [anon_sym_continue] = ACTIONS(3236),
    [anon_sym_default] = ACTIONS(3236),
    [anon_sym_enum] = ACTIONS(3236),
    [anon_sym_fn] = ACTIONS(3236),
    [anon_sym_for] = ACTIONS(3236),
    [anon_sym_gen] = ACTIONS(3236),
    [anon_sym_if] = ACTIONS(3236),
    [anon_sym_impl] = ACTIONS(3236),
    [anon_sym_let] = ACTIONS(3236),
    [anon_sym_loop] = ACTIONS(3236),
    [anon_sym_match] = ACTIONS(3236),
    [anon_sym_mod] = ACTIONS(3236),
    [anon_sym_pub] = ACTIONS(3236),
    [anon_sym_return] = ACTIONS(3236),
    [anon_sym_raw] = ACTIONS(3236),
    [anon_sym_static] = ACTIONS(3236),
    [anon_sym_struct] = ACTIONS(3236),
    [anon_sym_trait] = ACTIONS(3236),
    [anon_sym_type] = ACTIONS(3236),
    [anon_sym_union] = ACTIONS(3236),
    [anon_sym_unsafe] = ACTIONS(3236),
    [anon_sym_use] = ACTIONS(3236),
    [anon_sym_while] = ACTIONS(3236),
    [anon_sym_extern] = ACTIONS(3236),
    [anon_sym_safe] = ACTIONS(3236),
    [anon_sym_yield] = ACTIONS(3236),
    [anon_sym_move] = ACTIONS(3236),
    [anon_sym_try] = ACTIONS(3236),
    [sym_integer_literal] = ACTIONS(3234),
    [aux_sym_string_literal_token1] = ACTIONS(3234),
    [sym_char_literal] = ACTIONS(3234),
    [anon_sym_true] = ACTIONS(3236),
    [anon_sym_false] = ACTIONS(3236),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3236),
    [sym_super] = ACTIONS(3236),
    [sym_crate] = ACTIONS(3236),
    [aux_sym_metavariable_token1] = ACTIONS(3234),
    [sym__raw_string_literal_start] = ACTIONS(3234),
    [sym_float_literal] = ACTIONS(3234),
    [sym__outer_block_doc_comment_start] = ACTIONS(3234),
    [sym__outer_line_doc_comment_start] = ACTIONS(3234),
  },
  [STATE(867)] = {
    [sym_line_comment] = STATE(867),
    [sym_block_comment] = STATE(867),
    [ts_builtin_sym_end] = ACTIONS(3238),
    [sym_identifier] = ACTIONS(3240),
    [anon_sym_SEMI] = ACTIONS(3238),
    [anon_sym_macro] = ACTIONS(3240),
    [anon_sym_LPAREN] = ACTIONS(3238),
    [anon_sym_LBRACE] = ACTIONS(3238),
    [anon_sym_RBRACE] = ACTIONS(3238),
    [anon_sym_macro_rules] = ACTIONS(3240),
    [anon_sym_BANG] = ACTIONS(3238),
    [anon_sym_LBRACK] = ACTIONS(3238),
    [anon_sym_STAR] = ACTIONS(3238),
    [anon_sym_u8] = ACTIONS(3240),
    [anon_sym_i8] = ACTIONS(3240),
    [anon_sym_u16] = ACTIONS(3240),
    [anon_sym_i16] = ACTIONS(3240),
    [anon_sym_u32] = ACTIONS(3240),
    [anon_sym_i32] = ACTIONS(3240),
    [anon_sym_u64] = ACTIONS(3240),
    [anon_sym_i64] = ACTIONS(3240),
    [anon_sym_u128] = ACTIONS(3240),
    [anon_sym_i128] = ACTIONS(3240),
    [anon_sym_isize] = ACTIONS(3240),
    [anon_sym_usize] = ACTIONS(3240),
    [anon_sym_f32] = ACTIONS(3240),
    [anon_sym_f64] = ACTIONS(3240),
    [anon_sym_bool] = ACTIONS(3240),
    [anon_sym_str] = ACTIONS(3240),
    [anon_sym_char] = ACTIONS(3240),
    [anon_sym_DASH] = ACTIONS(3238),
    [anon_sym_AMP] = ACTIONS(3238),
    [anon_sym_PIPE] = ACTIONS(3238),
    [anon_sym_LT] = ACTIONS(3238),
    [anon_sym_DOT_DOT] = ACTIONS(3238),
    [anon_sym_COLON_COLON] = ACTIONS(3238),
    [anon_sym_POUND] = ACTIONS(3238),
    [anon_sym_SQUOTE] = ACTIONS(3240),
    [anon_sym_async] = ACTIONS(3240),
    [anon_sym_auto] = ACTIONS(3240),
    [anon_sym_await] = ACTIONS(3240),
    [anon_sym_become] = ACTIONS(3240),
    [anon_sym_break] = ACTIONS(3240),
    [anon_sym_const] = ACTIONS(3240),
    [anon_sym_continue] = ACTIONS(3240),
    [anon_sym_default] = ACTIONS(3240),
    [anon_sym_enum] = ACTIONS(3240),
    [anon_sym_fn] = ACTIONS(3240),
    [anon_sym_for] = ACTIONS(3240),
    [anon_sym_gen] = ACTIONS(3240),
    [anon_sym_if] = ACTIONS(3240),
    [anon_sym_impl] = ACTIONS(3240),
    [anon_sym_let] = ACTIONS(3240),
    [anon_sym_loop] = ACTIONS(3240),
    [anon_sym_match] = ACTIONS(3240),
    [anon_sym_mod] = ACTIONS(3240),
    [anon_sym_pub] = ACTIONS(3240),
    [anon_sym_return] = ACTIONS(3240),
    [anon_sym_raw] = ACTIONS(3240),
    [anon_sym_static] = ACTIONS(3240),
    [anon_sym_struct] = ACTIONS(3240),
    [anon_sym_trait] = ACTIONS(3240),
    [anon_sym_type] = ACTIONS(3240),
    [anon_sym_union] = ACTIONS(3240),
    [anon_sym_unsafe] = ACTIONS(3240),
    [anon_sym_use] = ACTIONS(3240),
    [anon_sym_while] = ACTIONS(3240),
    [anon_sym_extern] = ACTIONS(3240),
    [anon_sym_safe] = ACTIONS(3240),
    [anon_sym_yield] = ACTIONS(3240),
    [anon_sym_move] = ACTIONS(3240),
    [anon_sym_try] = ACTIONS(3240),
    [sym_integer_literal] = ACTIONS(3238),
    [aux_sym_string_literal_token1] = ACTIONS(3238),
    [sym_char_literal] = ACTIONS(3238),
    [anon_sym_true] = ACTIONS(3240),
    [anon_sym_false] = ACTIONS(3240),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3240),
    [sym_super] = ACTIONS(3240),
    [sym_crate] = ACTIONS(3240),
    [aux_sym_metavariable_token1] = ACTIONS(3238),
    [sym__raw_string_literal_start] = ACTIONS(3238),
    [sym_float_literal] = ACTIONS(3238),
    [sym__outer_block_doc_comment_start] = ACTIONS(3238),
    [sym__outer_line_doc_comment_start] = ACTIONS(3238),
  },
  [STATE(868)] = {
    [sym_line_comment] = STATE(868),
    [sym_block_comment] = STATE(868),
    [ts_builtin_sym_end] = ACTIONS(3242),
    [sym_identifier] = ACTIONS(3244),
    [anon_sym_SEMI] = ACTIONS(3242),
    [anon_sym_macro] = ACTIONS(3244),
    [anon_sym_LPAREN] = ACTIONS(3242),
    [anon_sym_LBRACE] = ACTIONS(3242),
    [anon_sym_RBRACE] = ACTIONS(3242),
    [anon_sym_macro_rules] = ACTIONS(3244),
    [anon_sym_BANG] = ACTIONS(3242),
    [anon_sym_LBRACK] = ACTIONS(3242),
    [anon_sym_STAR] = ACTIONS(3242),
    [anon_sym_u8] = ACTIONS(3244),
    [anon_sym_i8] = ACTIONS(3244),
    [anon_sym_u16] = ACTIONS(3244),
    [anon_sym_i16] = ACTIONS(3244),
    [anon_sym_u32] = ACTIONS(3244),
    [anon_sym_i32] = ACTIONS(3244),
    [anon_sym_u64] = ACTIONS(3244),
    [anon_sym_i64] = ACTIONS(3244),
    [anon_sym_u128] = ACTIONS(3244),
    [anon_sym_i128] = ACTIONS(3244),
    [anon_sym_isize] = ACTIONS(3244),
    [anon_sym_usize] = ACTIONS(3244),
    [anon_sym_f32] = ACTIONS(3244),
    [anon_sym_f64] = ACTIONS(3244),
    [anon_sym_bool] = ACTIONS(3244),
    [anon_sym_str] = ACTIONS(3244),
    [anon_sym_char] = ACTIONS(3244),
    [anon_sym_DASH] = ACTIONS(3242),
    [anon_sym_AMP] = ACTIONS(3242),
    [anon_sym_PIPE] = ACTIONS(3242),
    [anon_sym_LT] = ACTIONS(3242),
    [anon_sym_DOT_DOT] = ACTIONS(3242),
    [anon_sym_COLON_COLON] = ACTIONS(3242),
    [anon_sym_POUND] = ACTIONS(3242),
    [anon_sym_SQUOTE] = ACTIONS(3244),
    [anon_sym_async] = ACTIONS(3244),
    [anon_sym_auto] = ACTIONS(3244),
    [anon_sym_await] = ACTIONS(3244),
    [anon_sym_become] = ACTIONS(3244),
    [anon_sym_break] = ACTIONS(3244),
    [anon_sym_const] = ACTIONS(3244),
    [anon_sym_continue] = ACTIONS(3244),
    [anon_sym_default] = ACTIONS(3244),
    [anon_sym_enum] = ACTIONS(3244),
    [anon_sym_fn] = ACTIONS(3244),
    [anon_sym_for] = ACTIONS(3244),
    [anon_sym_gen] = ACTIONS(3244),
    [anon_sym_if] = ACTIONS(3244),
    [anon_sym_impl] = ACTIONS(3244),
    [anon_sym_let] = ACTIONS(3244),
    [anon_sym_loop] = ACTIONS(3244),
    [anon_sym_match] = ACTIONS(3244),
    [anon_sym_mod] = ACTIONS(3244),
    [anon_sym_pub] = ACTIONS(3244),
    [anon_sym_return] = ACTIONS(3244),
    [anon_sym_raw] = ACTIONS(3244),
    [anon_sym_static] = ACTIONS(3244),
    [anon_sym_struct] = ACTIONS(3244),
    [anon_sym_trait] = ACTIONS(3244),
    [anon_sym_type] = ACTIONS(3244),
    [anon_sym_union] = ACTIONS(3244),
    [anon_sym_unsafe] = ACTIONS(3244),
    [anon_sym_use] = ACTIONS(3244),
    [anon_sym_while] = ACTIONS(3244),
    [anon_sym_extern] = ACTIONS(3244),
    [anon_sym_safe] = ACTIONS(3244),
    [anon_sym_yield] = ACTIONS(3244),
    [anon_sym_move] = ACTIONS(3244),
    [anon_sym_try] = ACTIONS(3244),
    [sym_integer_literal] = ACTIONS(3242),
    [aux_sym_string_literal_token1] = ACTIONS(3242),
    [sym_char_literal] = ACTIONS(3242),
    [anon_sym_true] = ACTIONS(3244),
    [anon_sym_false] = ACTIONS(3244),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3244),
    [sym_super] = ACTIONS(3244),
    [sym_crate] = ACTIONS(3244),
    [aux_sym_metavariable_token1] = ACTIONS(3242),
    [sym__raw_string_literal_start] = ACTIONS(3242),
    [sym_float_literal] = ACTIONS(3242),
    [sym__outer_block_doc_comment_start] = ACTIONS(3242),
    [sym__outer_line_doc_comment_start] = ACTIONS(3242),
  },
  [STATE(869)] = {
    [sym_line_comment] = STATE(869),
    [sym_block_comment] = STATE(869),
    [ts_builtin_sym_end] = ACTIONS(3246),
    [sym_identifier] = ACTIONS(3248),
    [anon_sym_SEMI] = ACTIONS(3246),
    [anon_sym_macro] = ACTIONS(3248),
    [anon_sym_LPAREN] = ACTIONS(3246),
    [anon_sym_LBRACE] = ACTIONS(3246),
    [anon_sym_RBRACE] = ACTIONS(3246),
    [anon_sym_macro_rules] = ACTIONS(3248),
    [anon_sym_BANG] = ACTIONS(3246),
    [anon_sym_LBRACK] = ACTIONS(3246),
    [anon_sym_STAR] = ACTIONS(3246),
    [anon_sym_u8] = ACTIONS(3248),
    [anon_sym_i8] = ACTIONS(3248),
    [anon_sym_u16] = ACTIONS(3248),
    [anon_sym_i16] = ACTIONS(3248),
    [anon_sym_u32] = ACTIONS(3248),
    [anon_sym_i32] = ACTIONS(3248),
    [anon_sym_u64] = ACTIONS(3248),
    [anon_sym_i64] = ACTIONS(3248),
    [anon_sym_u128] = ACTIONS(3248),
    [anon_sym_i128] = ACTIONS(3248),
    [anon_sym_isize] = ACTIONS(3248),
    [anon_sym_usize] = ACTIONS(3248),
    [anon_sym_f32] = ACTIONS(3248),
    [anon_sym_f64] = ACTIONS(3248),
    [anon_sym_bool] = ACTIONS(3248),
    [anon_sym_str] = ACTIONS(3248),
    [anon_sym_char] = ACTIONS(3248),
    [anon_sym_DASH] = ACTIONS(3246),
    [anon_sym_AMP] = ACTIONS(3246),
    [anon_sym_PIPE] = ACTIONS(3246),
    [anon_sym_LT] = ACTIONS(3246),
    [anon_sym_DOT_DOT] = ACTIONS(3246),
    [anon_sym_COLON_COLON] = ACTIONS(3246),
    [anon_sym_POUND] = ACTIONS(3246),
    [anon_sym_SQUOTE] = ACTIONS(3248),
    [anon_sym_async] = ACTIONS(3248),
    [anon_sym_auto] = ACTIONS(3248),
    [anon_sym_await] = ACTIONS(3248),
    [anon_sym_become] = ACTIONS(3248),
    [anon_sym_break] = ACTIONS(3248),
    [anon_sym_const] = ACTIONS(3248),
    [anon_sym_continue] = ACTIONS(3248),
    [anon_sym_default] = ACTIONS(3248),
    [anon_sym_enum] = ACTIONS(3248),
    [anon_sym_fn] = ACTIONS(3248),
    [anon_sym_for] = ACTIONS(3248),
    [anon_sym_gen] = ACTIONS(3248),
    [anon_sym_if] = ACTIONS(3248),
    [anon_sym_impl] = ACTIONS(3248),
    [anon_sym_let] = ACTIONS(3248),
    [anon_sym_loop] = ACTIONS(3248),
    [anon_sym_match] = ACTIONS(3248),
    [anon_sym_mod] = ACTIONS(3248),
    [anon_sym_pub] = ACTIONS(3248),
    [anon_sym_return] = ACTIONS(3248),
    [anon_sym_raw] = ACTIONS(3248),
    [anon_sym_static] = ACTIONS(3248),
    [anon_sym_struct] = ACTIONS(3248),
    [anon_sym_trait] = ACTIONS(3248),
    [anon_sym_type] = ACTIONS(3248),
    [anon_sym_union] = ACTIONS(3248),
    [anon_sym_unsafe] = ACTIONS(3248),
    [anon_sym_use] = ACTIONS(3248),
    [anon_sym_while] = ACTIONS(3248),
    [anon_sym_extern] = ACTIONS(3248),
    [anon_sym_safe] = ACTIONS(3248),
    [anon_sym_yield] = ACTIONS(3248),
    [anon_sym_move] = ACTIONS(3248),
    [anon_sym_try] = ACTIONS(3248),
    [sym_integer_literal] = ACTIONS(3246),
    [aux_sym_string_literal_token1] = ACTIONS(3246),
    [sym_char_literal] = ACTIONS(3246),
    [anon_sym_true] = ACTIONS(3248),
    [anon_sym_false] = ACTIONS(3248),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3248),
    [sym_super] = ACTIONS(3248),
    [sym_crate] = ACTIONS(3248),
    [aux_sym_metavariable_token1] = ACTIONS(3246),
    [sym__raw_string_literal_start] = ACTIONS(3246),
    [sym_float_literal] = ACTIONS(3246),
    [sym__outer_block_doc_comment_start] = ACTIONS(3246),
    [sym__outer_line_doc_comment_start] = ACTIONS(3246),
  },
  [STATE(870)] = {
    [sym_line_comment] = STATE(870),
    [sym_block_comment] = STATE(870),
    [ts_builtin_sym_end] = ACTIONS(3250),
    [sym_identifier] = ACTIONS(3252),
    [anon_sym_SEMI] = ACTIONS(3250),
    [anon_sym_macro] = ACTIONS(3252),
    [anon_sym_LPAREN] = ACTIONS(3250),
    [anon_sym_LBRACE] = ACTIONS(3250),
    [anon_sym_RBRACE] = ACTIONS(3250),
    [anon_sym_macro_rules] = ACTIONS(3252),
    [anon_sym_BANG] = ACTIONS(3250),
    [anon_sym_LBRACK] = ACTIONS(3250),
    [anon_sym_STAR] = ACTIONS(3250),
    [anon_sym_u8] = ACTIONS(3252),
    [anon_sym_i8] = ACTIONS(3252),
    [anon_sym_u16] = ACTIONS(3252),
    [anon_sym_i16] = ACTIONS(3252),
    [anon_sym_u32] = ACTIONS(3252),
    [anon_sym_i32] = ACTIONS(3252),
    [anon_sym_u64] = ACTIONS(3252),
    [anon_sym_i64] = ACTIONS(3252),
    [anon_sym_u128] = ACTIONS(3252),
    [anon_sym_i128] = ACTIONS(3252),
    [anon_sym_isize] = ACTIONS(3252),
    [anon_sym_usize] = ACTIONS(3252),
    [anon_sym_f32] = ACTIONS(3252),
    [anon_sym_f64] = ACTIONS(3252),
    [anon_sym_bool] = ACTIONS(3252),
    [anon_sym_str] = ACTIONS(3252),
    [anon_sym_char] = ACTIONS(3252),
    [anon_sym_DASH] = ACTIONS(3250),
    [anon_sym_AMP] = ACTIONS(3250),
    [anon_sym_PIPE] = ACTIONS(3250),
    [anon_sym_LT] = ACTIONS(3250),
    [anon_sym_DOT_DOT] = ACTIONS(3250),
    [anon_sym_COLON_COLON] = ACTIONS(3250),
    [anon_sym_POUND] = ACTIONS(3250),
    [anon_sym_SQUOTE] = ACTIONS(3252),
    [anon_sym_async] = ACTIONS(3252),
    [anon_sym_auto] = ACTIONS(3252),
    [anon_sym_await] = ACTIONS(3252),
    [anon_sym_become] = ACTIONS(3252),
    [anon_sym_break] = ACTIONS(3252),
    [anon_sym_const] = ACTIONS(3252),
    [anon_sym_continue] = ACTIONS(3252),
    [anon_sym_default] = ACTIONS(3252),
    [anon_sym_enum] = ACTIONS(3252),
    [anon_sym_fn] = ACTIONS(3252),
    [anon_sym_for] = ACTIONS(3252),
    [anon_sym_gen] = ACTIONS(3252),
    [anon_sym_if] = ACTIONS(3252),
    [anon_sym_impl] = ACTIONS(3252),
    [anon_sym_let] = ACTIONS(3252),
    [anon_sym_loop] = ACTIONS(3252),
    [anon_sym_match] = ACTIONS(3252),
    [anon_sym_mod] = ACTIONS(3252),
    [anon_sym_pub] = ACTIONS(3252),
    [anon_sym_return] = ACTIONS(3252),
    [anon_sym_raw] = ACTIONS(3252),
    [anon_sym_static] = ACTIONS(3252),
    [anon_sym_struct] = ACTIONS(3252),
    [anon_sym_trait] = ACTIONS(3252),
    [anon_sym_type] = ACTIONS(3252),
    [anon_sym_union] = ACTIONS(3252),
    [anon_sym_unsafe] = ACTIONS(3252),
    [anon_sym_use] = ACTIONS(3252),
    [anon_sym_while] = ACTIONS(3252),
    [anon_sym_extern] = ACTIONS(3252),
    [anon_sym_safe] = ACTIONS(3252),
    [anon_sym_yield] = ACTIONS(3252),
    [anon_sym_move] = ACTIONS(3252),
    [anon_sym_try] = ACTIONS(3252),
    [sym_integer_literal] = ACTIONS(3250),
    [aux_sym_string_literal_token1] = ACTIONS(3250),
    [sym_char_literal] = ACTIONS(3250),
    [anon_sym_true] = ACTIONS(3252),
    [anon_sym_false] = ACTIONS(3252),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3252),
    [sym_super] = ACTIONS(3252),
    [sym_crate] = ACTIONS(3252),
    [aux_sym_metavariable_token1] = ACTIONS(3250),
    [sym__raw_string_literal_start] = ACTIONS(3250),
    [sym_float_literal] = ACTIONS(3250),
    [sym__outer_block_doc_comment_start] = ACTIONS(3250),
    [sym__outer_line_doc_comment_start] = ACTIONS(3250),
  },
  [STATE(871)] = {
    [sym_line_comment] = STATE(871),
    [sym_block_comment] = STATE(871),
    [ts_builtin_sym_end] = ACTIONS(3254),
    [sym_identifier] = ACTIONS(3256),
    [anon_sym_SEMI] = ACTIONS(3254),
    [anon_sym_macro] = ACTIONS(3256),
    [anon_sym_LPAREN] = ACTIONS(3254),
    [anon_sym_LBRACE] = ACTIONS(3254),
    [anon_sym_RBRACE] = ACTIONS(3254),
    [anon_sym_macro_rules] = ACTIONS(3256),
    [anon_sym_BANG] = ACTIONS(3254),
    [anon_sym_LBRACK] = ACTIONS(3254),
    [anon_sym_STAR] = ACTIONS(3254),
    [anon_sym_u8] = ACTIONS(3256),
    [anon_sym_i8] = ACTIONS(3256),
    [anon_sym_u16] = ACTIONS(3256),
    [anon_sym_i16] = ACTIONS(3256),
    [anon_sym_u32] = ACTIONS(3256),
    [anon_sym_i32] = ACTIONS(3256),
    [anon_sym_u64] = ACTIONS(3256),
    [anon_sym_i64] = ACTIONS(3256),
    [anon_sym_u128] = ACTIONS(3256),
    [anon_sym_i128] = ACTIONS(3256),
    [anon_sym_isize] = ACTIONS(3256),
    [anon_sym_usize] = ACTIONS(3256),
    [anon_sym_f32] = ACTIONS(3256),
    [anon_sym_f64] = ACTIONS(3256),
    [anon_sym_bool] = ACTIONS(3256),
    [anon_sym_str] = ACTIONS(3256),
    [anon_sym_char] = ACTIONS(3256),
    [anon_sym_DASH] = ACTIONS(3254),
    [anon_sym_AMP] = ACTIONS(3254),
    [anon_sym_PIPE] = ACTIONS(3254),
    [anon_sym_LT] = ACTIONS(3254),
    [anon_sym_DOT_DOT] = ACTIONS(3254),
    [anon_sym_COLON_COLON] = ACTIONS(3254),
    [anon_sym_POUND] = ACTIONS(3254),
    [anon_sym_SQUOTE] = ACTIONS(3256),
    [anon_sym_async] = ACTIONS(3256),
    [anon_sym_auto] = ACTIONS(3256),
    [anon_sym_await] = ACTIONS(3256),
    [anon_sym_become] = ACTIONS(3256),
    [anon_sym_break] = ACTIONS(3256),
    [anon_sym_const] = ACTIONS(3256),
    [anon_sym_continue] = ACTIONS(3256),
    [anon_sym_default] = ACTIONS(3256),
    [anon_sym_enum] = ACTIONS(3256),
    [anon_sym_fn] = ACTIONS(3256),
    [anon_sym_for] = ACTIONS(3256),
    [anon_sym_gen] = ACTIONS(3256),
    [anon_sym_if] = ACTIONS(3256),
    [anon_sym_impl] = ACTIONS(3256),
    [anon_sym_let] = ACTIONS(3256),
    [anon_sym_loop] = ACTIONS(3256),
    [anon_sym_match] = ACTIONS(3256),
    [anon_sym_mod] = ACTIONS(3256),
    [anon_sym_pub] = ACTIONS(3256),
    [anon_sym_return] = ACTIONS(3256),
    [anon_sym_raw] = ACTIONS(3256),
    [anon_sym_static] = ACTIONS(3256),
    [anon_sym_struct] = ACTIONS(3256),
    [anon_sym_trait] = ACTIONS(3256),
    [anon_sym_type] = ACTIONS(3256),
    [anon_sym_union] = ACTIONS(3256),
    [anon_sym_unsafe] = ACTIONS(3256),
    [anon_sym_use] = ACTIONS(3256),
    [anon_sym_while] = ACTIONS(3256),
    [anon_sym_extern] = ACTIONS(3256),
    [anon_sym_safe] = ACTIONS(3256),
    [anon_sym_yield] = ACTIONS(3256),
    [anon_sym_move] = ACTIONS(3256),
    [anon_sym_try] = ACTIONS(3256),
    [sym_integer_literal] = ACTIONS(3254),
    [aux_sym_string_literal_token1] = ACTIONS(3254),
    [sym_char_literal] = ACTIONS(3254),
    [anon_sym_true] = ACTIONS(3256),
    [anon_sym_false] = ACTIONS(3256),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3256),
    [sym_super] = ACTIONS(3256),
    [sym_crate] = ACTIONS(3256),
    [aux_sym_metavariable_token1] = ACTIONS(3254),
    [sym__raw_string_literal_start] = ACTIONS(3254),
    [sym_float_literal] = ACTIONS(3254),
    [sym__outer_block_doc_comment_start] = ACTIONS(3254),
    [sym__outer_line_doc_comment_start] = ACTIONS(3254),
  },
  [STATE(872)] = {
    [sym_line_comment] = STATE(872),
    [sym_block_comment] = STATE(872),
    [ts_builtin_sym_end] = ACTIONS(3258),
    [sym_identifier] = ACTIONS(3260),
    [anon_sym_SEMI] = ACTIONS(3258),
    [anon_sym_macro] = ACTIONS(3260),
    [anon_sym_LPAREN] = ACTIONS(3258),
    [anon_sym_LBRACE] = ACTIONS(3258),
    [anon_sym_RBRACE] = ACTIONS(3258),
    [anon_sym_macro_rules] = ACTIONS(3260),
    [anon_sym_BANG] = ACTIONS(3258),
    [anon_sym_LBRACK] = ACTIONS(3258),
    [anon_sym_STAR] = ACTIONS(3258),
    [anon_sym_u8] = ACTIONS(3260),
    [anon_sym_i8] = ACTIONS(3260),
    [anon_sym_u16] = ACTIONS(3260),
    [anon_sym_i16] = ACTIONS(3260),
    [anon_sym_u32] = ACTIONS(3260),
    [anon_sym_i32] = ACTIONS(3260),
    [anon_sym_u64] = ACTIONS(3260),
    [anon_sym_i64] = ACTIONS(3260),
    [anon_sym_u128] = ACTIONS(3260),
    [anon_sym_i128] = ACTIONS(3260),
    [anon_sym_isize] = ACTIONS(3260),
    [anon_sym_usize] = ACTIONS(3260),
    [anon_sym_f32] = ACTIONS(3260),
    [anon_sym_f64] = ACTIONS(3260),
    [anon_sym_bool] = ACTIONS(3260),
    [anon_sym_str] = ACTIONS(3260),
    [anon_sym_char] = ACTIONS(3260),
    [anon_sym_DASH] = ACTIONS(3258),
    [anon_sym_AMP] = ACTIONS(3258),
    [anon_sym_PIPE] = ACTIONS(3258),
    [anon_sym_LT] = ACTIONS(3258),
    [anon_sym_DOT_DOT] = ACTIONS(3258),
    [anon_sym_COLON_COLON] = ACTIONS(3258),
    [anon_sym_POUND] = ACTIONS(3258),
    [anon_sym_SQUOTE] = ACTIONS(3260),
    [anon_sym_async] = ACTIONS(3260),
    [anon_sym_auto] = ACTIONS(3260),
    [anon_sym_await] = ACTIONS(3260),
    [anon_sym_become] = ACTIONS(3260),
    [anon_sym_break] = ACTIONS(3260),
    [anon_sym_const] = ACTIONS(3260),
    [anon_sym_continue] = ACTIONS(3260),
    [anon_sym_default] = ACTIONS(3260),
    [anon_sym_enum] = ACTIONS(3260),
    [anon_sym_fn] = ACTIONS(3260),
    [anon_sym_for] = ACTIONS(3260),
    [anon_sym_gen] = ACTIONS(3260),
    [anon_sym_if] = ACTIONS(3260),
    [anon_sym_impl] = ACTIONS(3260),
    [anon_sym_let] = ACTIONS(3260),
    [anon_sym_loop] = ACTIONS(3260),
    [anon_sym_match] = ACTIONS(3260),
    [anon_sym_mod] = ACTIONS(3260),
    [anon_sym_pub] = ACTIONS(3260),
    [anon_sym_return] = ACTIONS(3260),
    [anon_sym_raw] = ACTIONS(3260),
    [anon_sym_static] = ACTIONS(3260),
    [anon_sym_struct] = ACTIONS(3260),
    [anon_sym_trait] = ACTIONS(3260),
    [anon_sym_type] = ACTIONS(3260),
    [anon_sym_union] = ACTIONS(3260),
    [anon_sym_unsafe] = ACTIONS(3260),
    [anon_sym_use] = ACTIONS(3260),
    [anon_sym_while] = ACTIONS(3260),
    [anon_sym_extern] = ACTIONS(3260),
    [anon_sym_safe] = ACTIONS(3260),
    [anon_sym_yield] = ACTIONS(3260),
    [anon_sym_move] = ACTIONS(3260),
    [anon_sym_try] = ACTIONS(3260),
    [sym_integer_literal] = ACTIONS(3258),
    [aux_sym_string_literal_token1] = ACTIONS(3258),
    [sym_char_literal] = ACTIONS(3258),
    [anon_sym_true] = ACTIONS(3260),
    [anon_sym_false] = ACTIONS(3260),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3260),
    [sym_super] = ACTIONS(3260),
    [sym_crate] = ACTIONS(3260),
    [aux_sym_metavariable_token1] = ACTIONS(3258),
    [sym__raw_string_literal_start] = ACTIONS(3258),
    [sym_float_literal] = ACTIONS(3258),
    [sym__outer_block_doc_comment_start] = ACTIONS(3258),
    [sym__outer_line_doc_comment_start] = ACTIONS(3258),
  },
  [STATE(873)] = {
    [sym_line_comment] = STATE(873),
    [sym_block_comment] = STATE(873),
    [ts_builtin_sym_end] = ACTIONS(3262),
    [sym_identifier] = ACTIONS(3264),
    [anon_sym_SEMI] = ACTIONS(3262),
    [anon_sym_macro] = ACTIONS(3264),
    [anon_sym_LPAREN] = ACTIONS(3262),
    [anon_sym_LBRACE] = ACTIONS(3262),
    [anon_sym_RBRACE] = ACTIONS(3262),
    [anon_sym_macro_rules] = ACTIONS(3264),
    [anon_sym_BANG] = ACTIONS(3262),
    [anon_sym_LBRACK] = ACTIONS(3262),
    [anon_sym_STAR] = ACTIONS(3262),
    [anon_sym_u8] = ACTIONS(3264),
    [anon_sym_i8] = ACTIONS(3264),
    [anon_sym_u16] = ACTIONS(3264),
    [anon_sym_i16] = ACTIONS(3264),
    [anon_sym_u32] = ACTIONS(3264),
    [anon_sym_i32] = ACTIONS(3264),
    [anon_sym_u64] = ACTIONS(3264),
    [anon_sym_i64] = ACTIONS(3264),
    [anon_sym_u128] = ACTIONS(3264),
    [anon_sym_i128] = ACTIONS(3264),
    [anon_sym_isize] = ACTIONS(3264),
    [anon_sym_usize] = ACTIONS(3264),
    [anon_sym_f32] = ACTIONS(3264),
    [anon_sym_f64] = ACTIONS(3264),
    [anon_sym_bool] = ACTIONS(3264),
    [anon_sym_str] = ACTIONS(3264),
    [anon_sym_char] = ACTIONS(3264),
    [anon_sym_DASH] = ACTIONS(3262),
    [anon_sym_AMP] = ACTIONS(3262),
    [anon_sym_PIPE] = ACTIONS(3262),
    [anon_sym_LT] = ACTIONS(3262),
    [anon_sym_DOT_DOT] = ACTIONS(3262),
    [anon_sym_COLON_COLON] = ACTIONS(3262),
    [anon_sym_POUND] = ACTIONS(3262),
    [anon_sym_SQUOTE] = ACTIONS(3264),
    [anon_sym_async] = ACTIONS(3264),
    [anon_sym_auto] = ACTIONS(3264),
    [anon_sym_await] = ACTIONS(3264),
    [anon_sym_become] = ACTIONS(3264),
    [anon_sym_break] = ACTIONS(3264),
    [anon_sym_const] = ACTIONS(3264),
    [anon_sym_continue] = ACTIONS(3264),
    [anon_sym_default] = ACTIONS(3264),
    [anon_sym_enum] = ACTIONS(3264),
    [anon_sym_fn] = ACTIONS(3264),
    [anon_sym_for] = ACTIONS(3264),
    [anon_sym_gen] = ACTIONS(3264),
    [anon_sym_if] = ACTIONS(3264),
    [anon_sym_impl] = ACTIONS(3264),
    [anon_sym_let] = ACTIONS(3264),
    [anon_sym_loop] = ACTIONS(3264),
    [anon_sym_match] = ACTIONS(3264),
    [anon_sym_mod] = ACTIONS(3264),
    [anon_sym_pub] = ACTIONS(3264),
    [anon_sym_return] = ACTIONS(3264),
    [anon_sym_raw] = ACTIONS(3264),
    [anon_sym_static] = ACTIONS(3264),
    [anon_sym_struct] = ACTIONS(3264),
    [anon_sym_trait] = ACTIONS(3264),
    [anon_sym_type] = ACTIONS(3264),
    [anon_sym_union] = ACTIONS(3264),
    [anon_sym_unsafe] = ACTIONS(3264),
    [anon_sym_use] = ACTIONS(3264),
    [anon_sym_while] = ACTIONS(3264),
    [anon_sym_extern] = ACTIONS(3264),
    [anon_sym_safe] = ACTIONS(3264),
    [anon_sym_yield] = ACTIONS(3264),
    [anon_sym_move] = ACTIONS(3264),
    [anon_sym_try] = ACTIONS(3264),
    [sym_integer_literal] = ACTIONS(3262),
    [aux_sym_string_literal_token1] = ACTIONS(3262),
    [sym_char_literal] = ACTIONS(3262),
    [anon_sym_true] = ACTIONS(3264),
    [anon_sym_false] = ACTIONS(3264),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3264),
    [sym_super] = ACTIONS(3264),
    [sym_crate] = ACTIONS(3264),
    [aux_sym_metavariable_token1] = ACTIONS(3262),
    [sym__raw_string_literal_start] = ACTIONS(3262),
    [sym_float_literal] = ACTIONS(3262),
    [sym__outer_block_doc_comment_start] = ACTIONS(3262),
    [sym__outer_line_doc_comment_start] = ACTIONS(3262),
  },
  [STATE(874)] = {
    [sym_line_comment] = STATE(874),
    [sym_block_comment] = STATE(874),
    [ts_builtin_sym_end] = ACTIONS(3266),
    [sym_identifier] = ACTIONS(3268),
    [anon_sym_SEMI] = ACTIONS(3266),
    [anon_sym_macro] = ACTIONS(3268),
    [anon_sym_LPAREN] = ACTIONS(3266),
    [anon_sym_LBRACE] = ACTIONS(3266),
    [anon_sym_RBRACE] = ACTIONS(3266),
    [anon_sym_macro_rules] = ACTIONS(3268),
    [anon_sym_BANG] = ACTIONS(3266),
    [anon_sym_LBRACK] = ACTIONS(3266),
    [anon_sym_STAR] = ACTIONS(3266),
    [anon_sym_u8] = ACTIONS(3268),
    [anon_sym_i8] = ACTIONS(3268),
    [anon_sym_u16] = ACTIONS(3268),
    [anon_sym_i16] = ACTIONS(3268),
    [anon_sym_u32] = ACTIONS(3268),
    [anon_sym_i32] = ACTIONS(3268),
    [anon_sym_u64] = ACTIONS(3268),
    [anon_sym_i64] = ACTIONS(3268),
    [anon_sym_u128] = ACTIONS(3268),
    [anon_sym_i128] = ACTIONS(3268),
    [anon_sym_isize] = ACTIONS(3268),
    [anon_sym_usize] = ACTIONS(3268),
    [anon_sym_f32] = ACTIONS(3268),
    [anon_sym_f64] = ACTIONS(3268),
    [anon_sym_bool] = ACTIONS(3268),
    [anon_sym_str] = ACTIONS(3268),
    [anon_sym_char] = ACTIONS(3268),
    [anon_sym_DASH] = ACTIONS(3266),
    [anon_sym_AMP] = ACTIONS(3266),
    [anon_sym_PIPE] = ACTIONS(3266),
    [anon_sym_LT] = ACTIONS(3266),
    [anon_sym_DOT_DOT] = ACTIONS(3266),
    [anon_sym_COLON_COLON] = ACTIONS(3266),
    [anon_sym_POUND] = ACTIONS(3266),
    [anon_sym_SQUOTE] = ACTIONS(3268),
    [anon_sym_async] = ACTIONS(3268),
    [anon_sym_auto] = ACTIONS(3268),
    [anon_sym_await] = ACTIONS(3268),
    [anon_sym_become] = ACTIONS(3268),
    [anon_sym_break] = ACTIONS(3268),
    [anon_sym_const] = ACTIONS(3268),
    [anon_sym_continue] = ACTIONS(3268),
    [anon_sym_default] = ACTIONS(3268),
    [anon_sym_enum] = ACTIONS(3268),
    [anon_sym_fn] = ACTIONS(3268),
    [anon_sym_for] = ACTIONS(3268),
    [anon_sym_gen] = ACTIONS(3268),
    [anon_sym_if] = ACTIONS(3268),
    [anon_sym_impl] = ACTIONS(3268),
    [anon_sym_let] = ACTIONS(3268),
    [anon_sym_loop] = ACTIONS(3268),
    [anon_sym_match] = ACTIONS(3268),
    [anon_sym_mod] = ACTIONS(3268),
    [anon_sym_pub] = ACTIONS(3268),
    [anon_sym_return] = ACTIONS(3268),
    [anon_sym_raw] = ACTIONS(3268),
    [anon_sym_static] = ACTIONS(3268),
    [anon_sym_struct] = ACTIONS(3268),
    [anon_sym_trait] = ACTIONS(3268),
    [anon_sym_type] = ACTIONS(3268),
    [anon_sym_union] = ACTIONS(3268),
    [anon_sym_unsafe] = ACTIONS(3268),
    [anon_sym_use] = ACTIONS(3268),
    [anon_sym_while] = ACTIONS(3268),
    [anon_sym_extern] = ACTIONS(3268),
    [anon_sym_safe] = ACTIONS(3268),
    [anon_sym_yield] = ACTIONS(3268),
    [anon_sym_move] = ACTIONS(3268),
    [anon_sym_try] = ACTIONS(3268),
    [sym_integer_literal] = ACTIONS(3266),
    [aux_sym_string_literal_token1] = ACTIONS(3266),
    [sym_char_literal] = ACTIONS(3266),
    [anon_sym_true] = ACTIONS(3268),
    [anon_sym_false] = ACTIONS(3268),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3268),
    [sym_super] = ACTIONS(3268),
    [sym_crate] = ACTIONS(3268),
    [aux_sym_metavariable_token1] = ACTIONS(3266),
    [sym__raw_string_literal_start] = ACTIONS(3266),
    [sym_float_literal] = ACTIONS(3266),
    [sym__outer_block_doc_comment_start] = ACTIONS(3266),
    [sym__outer_line_doc_comment_start] = ACTIONS(3266),
  },
  [STATE(875)] = {
    [sym_line_comment] = STATE(875),
    [sym_block_comment] = STATE(875),
    [ts_builtin_sym_end] = ACTIONS(3270),
    [sym_identifier] = ACTIONS(3272),
    [anon_sym_SEMI] = ACTIONS(3270),
    [anon_sym_macro] = ACTIONS(3272),
    [anon_sym_LPAREN] = ACTIONS(3270),
    [anon_sym_LBRACE] = ACTIONS(3270),
    [anon_sym_RBRACE] = ACTIONS(3270),
    [anon_sym_macro_rules] = ACTIONS(3272),
    [anon_sym_BANG] = ACTIONS(3270),
    [anon_sym_LBRACK] = ACTIONS(3270),
    [anon_sym_STAR] = ACTIONS(3270),
    [anon_sym_u8] = ACTIONS(3272),
    [anon_sym_i8] = ACTIONS(3272),
    [anon_sym_u16] = ACTIONS(3272),
    [anon_sym_i16] = ACTIONS(3272),
    [anon_sym_u32] = ACTIONS(3272),
    [anon_sym_i32] = ACTIONS(3272),
    [anon_sym_u64] = ACTIONS(3272),
    [anon_sym_i64] = ACTIONS(3272),
    [anon_sym_u128] = ACTIONS(3272),
    [anon_sym_i128] = ACTIONS(3272),
    [anon_sym_isize] = ACTIONS(3272),
    [anon_sym_usize] = ACTIONS(3272),
    [anon_sym_f32] = ACTIONS(3272),
    [anon_sym_f64] = ACTIONS(3272),
    [anon_sym_bool] = ACTIONS(3272),
    [anon_sym_str] = ACTIONS(3272),
    [anon_sym_char] = ACTIONS(3272),
    [anon_sym_DASH] = ACTIONS(3270),
    [anon_sym_AMP] = ACTIONS(3270),
    [anon_sym_PIPE] = ACTIONS(3270),
    [anon_sym_LT] = ACTIONS(3270),
    [anon_sym_DOT_DOT] = ACTIONS(3270),
    [anon_sym_COLON_COLON] = ACTIONS(3270),
    [anon_sym_POUND] = ACTIONS(3270),
    [anon_sym_SQUOTE] = ACTIONS(3272),
    [anon_sym_async] = ACTIONS(3272),
    [anon_sym_auto] = ACTIONS(3272),
    [anon_sym_await] = ACTIONS(3272),
    [anon_sym_become] = ACTIONS(3272),
    [anon_sym_break] = ACTIONS(3272),
    [anon_sym_const] = ACTIONS(3272),
    [anon_sym_continue] = ACTIONS(3272),
    [anon_sym_default] = ACTIONS(3272),
    [anon_sym_enum] = ACTIONS(3272),
    [anon_sym_fn] = ACTIONS(3272),
    [anon_sym_for] = ACTIONS(3272),
    [anon_sym_gen] = ACTIONS(3272),
    [anon_sym_if] = ACTIONS(3272),
    [anon_sym_impl] = ACTIONS(3272),
    [anon_sym_let] = ACTIONS(3272),
    [anon_sym_loop] = ACTIONS(3272),
    [anon_sym_match] = ACTIONS(3272),
    [anon_sym_mod] = ACTIONS(3272),
    [anon_sym_pub] = ACTIONS(3272),
    [anon_sym_return] = ACTIONS(3272),
    [anon_sym_raw] = ACTIONS(3272),
    [anon_sym_static] = ACTIONS(3272),
    [anon_sym_struct] = ACTIONS(3272),
    [anon_sym_trait] = ACTIONS(3272),
    [anon_sym_type] = ACTIONS(3272),
    [anon_sym_union] = ACTIONS(3272),
    [anon_sym_unsafe] = ACTIONS(3272),
    [anon_sym_use] = ACTIONS(3272),
    [anon_sym_while] = ACTIONS(3272),
    [anon_sym_extern] = ACTIONS(3272),
    [anon_sym_safe] = ACTIONS(3272),
    [anon_sym_yield] = ACTIONS(3272),
    [anon_sym_move] = ACTIONS(3272),
    [anon_sym_try] = ACTIONS(3272),
    [sym_integer_literal] = ACTIONS(3270),
    [aux_sym_string_literal_token1] = ACTIONS(3270),
    [sym_char_literal] = ACTIONS(3270),
    [anon_sym_true] = ACTIONS(3272),
    [anon_sym_false] = ACTIONS(3272),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3272),
    [sym_super] = ACTIONS(3272),
    [sym_crate] = ACTIONS(3272),
    [aux_sym_metavariable_token1] = ACTIONS(3270),
    [sym__raw_string_literal_start] = ACTIONS(3270),
    [sym_float_literal] = ACTIONS(3270),
    [sym__outer_block_doc_comment_start] = ACTIONS(3270),
    [sym__outer_line_doc_comment_start] = ACTIONS(3270),
  },
  [STATE(876)] = {
    [sym_line_comment] = STATE(876),
    [sym_block_comment] = STATE(876),
    [ts_builtin_sym_end] = ACTIONS(3274),
    [sym_identifier] = ACTIONS(3276),
    [anon_sym_SEMI] = ACTIONS(3274),
    [anon_sym_macro] = ACTIONS(3276),
    [anon_sym_LPAREN] = ACTIONS(3274),
    [anon_sym_LBRACE] = ACTIONS(3274),
    [anon_sym_RBRACE] = ACTIONS(3274),
    [anon_sym_macro_rules] = ACTIONS(3276),
    [anon_sym_BANG] = ACTIONS(3274),
    [anon_sym_LBRACK] = ACTIONS(3274),
    [anon_sym_STAR] = ACTIONS(3274),
    [anon_sym_u8] = ACTIONS(3276),
    [anon_sym_i8] = ACTIONS(3276),
    [anon_sym_u16] = ACTIONS(3276),
    [anon_sym_i16] = ACTIONS(3276),
    [anon_sym_u32] = ACTIONS(3276),
    [anon_sym_i32] = ACTIONS(3276),
    [anon_sym_u64] = ACTIONS(3276),
    [anon_sym_i64] = ACTIONS(3276),
    [anon_sym_u128] = ACTIONS(3276),
    [anon_sym_i128] = ACTIONS(3276),
    [anon_sym_isize] = ACTIONS(3276),
    [anon_sym_usize] = ACTIONS(3276),
    [anon_sym_f32] = ACTIONS(3276),
    [anon_sym_f64] = ACTIONS(3276),
    [anon_sym_bool] = ACTIONS(3276),
    [anon_sym_str] = ACTIONS(3276),
    [anon_sym_char] = ACTIONS(3276),
    [anon_sym_DASH] = ACTIONS(3274),
    [anon_sym_AMP] = ACTIONS(3274),
    [anon_sym_PIPE] = ACTIONS(3274),
    [anon_sym_LT] = ACTIONS(3274),
    [anon_sym_DOT_DOT] = ACTIONS(3274),
    [anon_sym_COLON_COLON] = ACTIONS(3274),
    [anon_sym_POUND] = ACTIONS(3274),
    [anon_sym_SQUOTE] = ACTIONS(3276),
    [anon_sym_async] = ACTIONS(3276),
    [anon_sym_auto] = ACTIONS(3276),
    [anon_sym_await] = ACTIONS(3276),
    [anon_sym_become] = ACTIONS(3276),
    [anon_sym_break] = ACTIONS(3276),
    [anon_sym_const] = ACTIONS(3276),
    [anon_sym_continue] = ACTIONS(3276),
    [anon_sym_default] = ACTIONS(3276),
    [anon_sym_enum] = ACTIONS(3276),
    [anon_sym_fn] = ACTIONS(3276),
    [anon_sym_for] = ACTIONS(3276),
    [anon_sym_gen] = ACTIONS(3276),
    [anon_sym_if] = ACTIONS(3276),
    [anon_sym_impl] = ACTIONS(3276),
    [anon_sym_let] = ACTIONS(3276),
    [anon_sym_loop] = ACTIONS(3276),
    [anon_sym_match] = ACTIONS(3276),
    [anon_sym_mod] = ACTIONS(3276),
    [anon_sym_pub] = ACTIONS(3276),
    [anon_sym_return] = ACTIONS(3276),
    [anon_sym_raw] = ACTIONS(3276),
    [anon_sym_static] = ACTIONS(3276),
    [anon_sym_struct] = ACTIONS(3276),
    [anon_sym_trait] = ACTIONS(3276),
    [anon_sym_type] = ACTIONS(3276),
    [anon_sym_union] = ACTIONS(3276),
    [anon_sym_unsafe] = ACTIONS(3276),
    [anon_sym_use] = ACTIONS(3276),
    [anon_sym_while] = ACTIONS(3276),
    [anon_sym_extern] = ACTIONS(3276),
    [anon_sym_safe] = ACTIONS(3276),
    [anon_sym_yield] = ACTIONS(3276),
    [anon_sym_move] = ACTIONS(3276),
    [anon_sym_try] = ACTIONS(3276),
    [sym_integer_literal] = ACTIONS(3274),
    [aux_sym_string_literal_token1] = ACTIONS(3274),
    [sym_char_literal] = ACTIONS(3274),
    [anon_sym_true] = ACTIONS(3276),
    [anon_sym_false] = ACTIONS(3276),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3276),
    [sym_super] = ACTIONS(3276),
    [sym_crate] = ACTIONS(3276),
    [aux_sym_metavariable_token1] = ACTIONS(3274),
    [sym__raw_string_literal_start] = ACTIONS(3274),
    [sym_float_literal] = ACTIONS(3274),
    [sym__outer_block_doc_comment_start] = ACTIONS(3274),
    [sym__outer_line_doc_comment_start] = ACTIONS(3274),
  },
  [STATE(877)] = {
    [sym_line_comment] = STATE(877),
    [sym_block_comment] = STATE(877),
    [ts_builtin_sym_end] = ACTIONS(3278),
    [sym_identifier] = ACTIONS(3280),
    [anon_sym_SEMI] = ACTIONS(3278),
    [anon_sym_macro] = ACTIONS(3280),
    [anon_sym_LPAREN] = ACTIONS(3278),
    [anon_sym_LBRACE] = ACTIONS(3278),
    [anon_sym_RBRACE] = ACTIONS(3278),
    [anon_sym_macro_rules] = ACTIONS(3280),
    [anon_sym_BANG] = ACTIONS(3278),
    [anon_sym_LBRACK] = ACTIONS(3278),
    [anon_sym_STAR] = ACTIONS(3278),
    [anon_sym_u8] = ACTIONS(3280),
    [anon_sym_i8] = ACTIONS(3280),
    [anon_sym_u16] = ACTIONS(3280),
    [anon_sym_i16] = ACTIONS(3280),
    [anon_sym_u32] = ACTIONS(3280),
    [anon_sym_i32] = ACTIONS(3280),
    [anon_sym_u64] = ACTIONS(3280),
    [anon_sym_i64] = ACTIONS(3280),
    [anon_sym_u128] = ACTIONS(3280),
    [anon_sym_i128] = ACTIONS(3280),
    [anon_sym_isize] = ACTIONS(3280),
    [anon_sym_usize] = ACTIONS(3280),
    [anon_sym_f32] = ACTIONS(3280),
    [anon_sym_f64] = ACTIONS(3280),
    [anon_sym_bool] = ACTIONS(3280),
    [anon_sym_str] = ACTIONS(3280),
    [anon_sym_char] = ACTIONS(3280),
    [anon_sym_DASH] = ACTIONS(3278),
    [anon_sym_AMP] = ACTIONS(3278),
    [anon_sym_PIPE] = ACTIONS(3278),
    [anon_sym_LT] = ACTIONS(3278),
    [anon_sym_DOT_DOT] = ACTIONS(3278),
    [anon_sym_COLON_COLON] = ACTIONS(3278),
    [anon_sym_POUND] = ACTIONS(3278),
    [anon_sym_SQUOTE] = ACTIONS(3280),
    [anon_sym_async] = ACTIONS(3280),
    [anon_sym_auto] = ACTIONS(3280),
    [anon_sym_await] = ACTIONS(3280),
    [anon_sym_become] = ACTIONS(3280),
    [anon_sym_break] = ACTIONS(3280),
    [anon_sym_const] = ACTIONS(3280),
    [anon_sym_continue] = ACTIONS(3280),
    [anon_sym_default] = ACTIONS(3280),
    [anon_sym_enum] = ACTIONS(3280),
    [anon_sym_fn] = ACTIONS(3280),
    [anon_sym_for] = ACTIONS(3280),
    [anon_sym_gen] = ACTIONS(3280),
    [anon_sym_if] = ACTIONS(3280),
    [anon_sym_impl] = ACTIONS(3280),
    [anon_sym_let] = ACTIONS(3280),
    [anon_sym_loop] = ACTIONS(3280),
    [anon_sym_match] = ACTIONS(3280),
    [anon_sym_mod] = ACTIONS(3280),
    [anon_sym_pub] = ACTIONS(3280),
    [anon_sym_return] = ACTIONS(3280),
    [anon_sym_raw] = ACTIONS(3280),
    [anon_sym_static] = ACTIONS(3280),
    [anon_sym_struct] = ACTIONS(3280),
    [anon_sym_trait] = ACTIONS(3280),
    [anon_sym_type] = ACTIONS(3280),
    [anon_sym_union] = ACTIONS(3280),
    [anon_sym_unsafe] = ACTIONS(3280),
    [anon_sym_use] = ACTIONS(3280),
    [anon_sym_while] = ACTIONS(3280),
    [anon_sym_extern] = ACTIONS(3280),
    [anon_sym_safe] = ACTIONS(3280),
    [anon_sym_yield] = ACTIONS(3280),
    [anon_sym_move] = ACTIONS(3280),
    [anon_sym_try] = ACTIONS(3280),
    [sym_integer_literal] = ACTIONS(3278),
    [aux_sym_string_literal_token1] = ACTIONS(3278),
    [sym_char_literal] = ACTIONS(3278),
    [anon_sym_true] = ACTIONS(3280),
    [anon_sym_false] = ACTIONS(3280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3280),
    [sym_super] = ACTIONS(3280),
    [sym_crate] = ACTIONS(3280),
    [aux_sym_metavariable_token1] = ACTIONS(3278),
    [sym__raw_string_literal_start] = ACTIONS(3278),
    [sym_float_literal] = ACTIONS(3278),
    [sym__outer_block_doc_comment_start] = ACTIONS(3278),
    [sym__outer_line_doc_comment_start] = ACTIONS(3278),
  },
  [STATE(878)] = {
    [sym_line_comment] = STATE(878),
    [sym_block_comment] = STATE(878),
    [ts_builtin_sym_end] = ACTIONS(3282),
    [sym_identifier] = ACTIONS(3284),
    [anon_sym_SEMI] = ACTIONS(3282),
    [anon_sym_macro] = ACTIONS(3284),
    [anon_sym_LPAREN] = ACTIONS(3282),
    [anon_sym_LBRACE] = ACTIONS(3282),
    [anon_sym_RBRACE] = ACTIONS(3282),
    [anon_sym_macro_rules] = ACTIONS(3284),
    [anon_sym_BANG] = ACTIONS(3282),
    [anon_sym_LBRACK] = ACTIONS(3282),
    [anon_sym_STAR] = ACTIONS(3282),
    [anon_sym_u8] = ACTIONS(3284),
    [anon_sym_i8] = ACTIONS(3284),
    [anon_sym_u16] = ACTIONS(3284),
    [anon_sym_i16] = ACTIONS(3284),
    [anon_sym_u32] = ACTIONS(3284),
    [anon_sym_i32] = ACTIONS(3284),
    [anon_sym_u64] = ACTIONS(3284),
    [anon_sym_i64] = ACTIONS(3284),
    [anon_sym_u128] = ACTIONS(3284),
    [anon_sym_i128] = ACTIONS(3284),
    [anon_sym_isize] = ACTIONS(3284),
    [anon_sym_usize] = ACTIONS(3284),
    [anon_sym_f32] = ACTIONS(3284),
    [anon_sym_f64] = ACTIONS(3284),
    [anon_sym_bool] = ACTIONS(3284),
    [anon_sym_str] = ACTIONS(3284),
    [anon_sym_char] = ACTIONS(3284),
    [anon_sym_DASH] = ACTIONS(3282),
    [anon_sym_AMP] = ACTIONS(3282),
    [anon_sym_PIPE] = ACTIONS(3282),
    [anon_sym_LT] = ACTIONS(3282),
    [anon_sym_DOT_DOT] = ACTIONS(3282),
    [anon_sym_COLON_COLON] = ACTIONS(3282),
    [anon_sym_POUND] = ACTIONS(3282),
    [anon_sym_SQUOTE] = ACTIONS(3284),
    [anon_sym_async] = ACTIONS(3284),
    [anon_sym_auto] = ACTIONS(3284),
    [anon_sym_await] = ACTIONS(3284),
    [anon_sym_become] = ACTIONS(3284),
    [anon_sym_break] = ACTIONS(3284),
    [anon_sym_const] = ACTIONS(3284),
    [anon_sym_continue] = ACTIONS(3284),
    [anon_sym_default] = ACTIONS(3284),
    [anon_sym_enum] = ACTIONS(3284),
    [anon_sym_fn] = ACTIONS(3284),
    [anon_sym_for] = ACTIONS(3284),
    [anon_sym_gen] = ACTIONS(3284),
    [anon_sym_if] = ACTIONS(3284),
    [anon_sym_impl] = ACTIONS(3284),
    [anon_sym_let] = ACTIONS(3284),
    [anon_sym_loop] = ACTIONS(3284),
    [anon_sym_match] = ACTIONS(3284),
    [anon_sym_mod] = ACTIONS(3284),
    [anon_sym_pub] = ACTIONS(3284),
    [anon_sym_return] = ACTIONS(3284),
    [anon_sym_raw] = ACTIONS(3284),
    [anon_sym_static] = ACTIONS(3284),
    [anon_sym_struct] = ACTIONS(3284),
    [anon_sym_trait] = ACTIONS(3284),
    [anon_sym_type] = ACTIONS(3284),
    [anon_sym_union] = ACTIONS(3284),
    [anon_sym_unsafe] = ACTIONS(3284),
    [anon_sym_use] = ACTIONS(3284),
    [anon_sym_while] = ACTIONS(3284),
    [anon_sym_extern] = ACTIONS(3284),
    [anon_sym_safe] = ACTIONS(3284),
    [anon_sym_yield] = ACTIONS(3284),
    [anon_sym_move] = ACTIONS(3284),
    [anon_sym_try] = ACTIONS(3284),
    [sym_integer_literal] = ACTIONS(3282),
    [aux_sym_string_literal_token1] = ACTIONS(3282),
    [sym_char_literal] = ACTIONS(3282),
    [anon_sym_true] = ACTIONS(3284),
    [anon_sym_false] = ACTIONS(3284),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3284),
    [sym_super] = ACTIONS(3284),
    [sym_crate] = ACTIONS(3284),
    [aux_sym_metavariable_token1] = ACTIONS(3282),
    [sym__raw_string_literal_start] = ACTIONS(3282),
    [sym_float_literal] = ACTIONS(3282),
    [sym__outer_block_doc_comment_start] = ACTIONS(3282),
    [sym__outer_line_doc_comment_start] = ACTIONS(3282),
  },
  [STATE(879)] = {
    [sym_line_comment] = STATE(879),
    [sym_block_comment] = STATE(879),
    [ts_builtin_sym_end] = ACTIONS(3286),
    [sym_identifier] = ACTIONS(3288),
    [anon_sym_SEMI] = ACTIONS(3286),
    [anon_sym_macro] = ACTIONS(3288),
    [anon_sym_LPAREN] = ACTIONS(3286),
    [anon_sym_LBRACE] = ACTIONS(3286),
    [anon_sym_RBRACE] = ACTIONS(3286),
    [anon_sym_macro_rules] = ACTIONS(3288),
    [anon_sym_BANG] = ACTIONS(3286),
    [anon_sym_LBRACK] = ACTIONS(3286),
    [anon_sym_STAR] = ACTIONS(3286),
    [anon_sym_u8] = ACTIONS(3288),
    [anon_sym_i8] = ACTIONS(3288),
    [anon_sym_u16] = ACTIONS(3288),
    [anon_sym_i16] = ACTIONS(3288),
    [anon_sym_u32] = ACTIONS(3288),
    [anon_sym_i32] = ACTIONS(3288),
    [anon_sym_u64] = ACTIONS(3288),
    [anon_sym_i64] = ACTIONS(3288),
    [anon_sym_u128] = ACTIONS(3288),
    [anon_sym_i128] = ACTIONS(3288),
    [anon_sym_isize] = ACTIONS(3288),
    [anon_sym_usize] = ACTIONS(3288),
    [anon_sym_f32] = ACTIONS(3288),
    [anon_sym_f64] = ACTIONS(3288),
    [anon_sym_bool] = ACTIONS(3288),
    [anon_sym_str] = ACTIONS(3288),
    [anon_sym_char] = ACTIONS(3288),
    [anon_sym_DASH] = ACTIONS(3286),
    [anon_sym_AMP] = ACTIONS(3286),
    [anon_sym_PIPE] = ACTIONS(3286),
    [anon_sym_LT] = ACTIONS(3286),
    [anon_sym_DOT_DOT] = ACTIONS(3286),
    [anon_sym_COLON_COLON] = ACTIONS(3286),
    [anon_sym_POUND] = ACTIONS(3286),
    [anon_sym_SQUOTE] = ACTIONS(3288),
    [anon_sym_async] = ACTIONS(3288),
    [anon_sym_auto] = ACTIONS(3288),
    [anon_sym_await] = ACTIONS(3288),
    [anon_sym_become] = ACTIONS(3288),
    [anon_sym_break] = ACTIONS(3288),
    [anon_sym_const] = ACTIONS(3288),
    [anon_sym_continue] = ACTIONS(3288),
    [anon_sym_default] = ACTIONS(3288),
    [anon_sym_enum] = ACTIONS(3288),
    [anon_sym_fn] = ACTIONS(3288),
    [anon_sym_for] = ACTIONS(3288),
    [anon_sym_gen] = ACTIONS(3288),
    [anon_sym_if] = ACTIONS(3288),
    [anon_sym_impl] = ACTIONS(3288),
    [anon_sym_let] = ACTIONS(3288),
    [anon_sym_loop] = ACTIONS(3288),
    [anon_sym_match] = ACTIONS(3288),
    [anon_sym_mod] = ACTIONS(3288),
    [anon_sym_pub] = ACTIONS(3288),
    [anon_sym_return] = ACTIONS(3288),
    [anon_sym_raw] = ACTIONS(3288),
    [anon_sym_static] = ACTIONS(3288),
    [anon_sym_struct] = ACTIONS(3288),
    [anon_sym_trait] = ACTIONS(3288),
    [anon_sym_type] = ACTIONS(3288),
    [anon_sym_union] = ACTIONS(3288),
    [anon_sym_unsafe] = ACTIONS(3288),
    [anon_sym_use] = ACTIONS(3288),
    [anon_sym_while] = ACTIONS(3288),
    [anon_sym_extern] = ACTIONS(3288),
    [anon_sym_safe] = ACTIONS(3288),
    [anon_sym_yield] = ACTIONS(3288),
    [anon_sym_move] = ACTIONS(3288),
    [anon_sym_try] = ACTIONS(3288),
    [sym_integer_literal] = ACTIONS(3286),
    [aux_sym_string_literal_token1] = ACTIONS(3286),
    [sym_char_literal] = ACTIONS(3286),
    [anon_sym_true] = ACTIONS(3288),
    [anon_sym_false] = ACTIONS(3288),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3288),
    [sym_super] = ACTIONS(3288),
    [sym_crate] = ACTIONS(3288),
    [aux_sym_metavariable_token1] = ACTIONS(3286),
    [sym__raw_string_literal_start] = ACTIONS(3286),
    [sym_float_literal] = ACTIONS(3286),
    [sym__outer_block_doc_comment_start] = ACTIONS(3286),
    [sym__outer_line_doc_comment_start] = ACTIONS(3286),
  },
  [STATE(880)] = {
    [sym_line_comment] = STATE(880),
    [sym_block_comment] = STATE(880),
    [ts_builtin_sym_end] = ACTIONS(3290),
    [sym_identifier] = ACTIONS(3292),
    [anon_sym_SEMI] = ACTIONS(3290),
    [anon_sym_macro] = ACTIONS(3292),
    [anon_sym_LPAREN] = ACTIONS(3290),
    [anon_sym_LBRACE] = ACTIONS(3290),
    [anon_sym_RBRACE] = ACTIONS(3290),
    [anon_sym_macro_rules] = ACTIONS(3292),
    [anon_sym_BANG] = ACTIONS(3290),
    [anon_sym_LBRACK] = ACTIONS(3290),
    [anon_sym_STAR] = ACTIONS(3290),
    [anon_sym_u8] = ACTIONS(3292),
    [anon_sym_i8] = ACTIONS(3292),
    [anon_sym_u16] = ACTIONS(3292),
    [anon_sym_i16] = ACTIONS(3292),
    [anon_sym_u32] = ACTIONS(3292),
    [anon_sym_i32] = ACTIONS(3292),
    [anon_sym_u64] = ACTIONS(3292),
    [anon_sym_i64] = ACTIONS(3292),
    [anon_sym_u128] = ACTIONS(3292),
    [anon_sym_i128] = ACTIONS(3292),
    [anon_sym_isize] = ACTIONS(3292),
    [anon_sym_usize] = ACTIONS(3292),
    [anon_sym_f32] = ACTIONS(3292),
    [anon_sym_f64] = ACTIONS(3292),
    [anon_sym_bool] = ACTIONS(3292),
    [anon_sym_str] = ACTIONS(3292),
    [anon_sym_char] = ACTIONS(3292),
    [anon_sym_DASH] = ACTIONS(3290),
    [anon_sym_AMP] = ACTIONS(3290),
    [anon_sym_PIPE] = ACTIONS(3290),
    [anon_sym_LT] = ACTIONS(3290),
    [anon_sym_DOT_DOT] = ACTIONS(3290),
    [anon_sym_COLON_COLON] = ACTIONS(3290),
    [anon_sym_POUND] = ACTIONS(3290),
    [anon_sym_SQUOTE] = ACTIONS(3292),
    [anon_sym_async] = ACTIONS(3292),
    [anon_sym_auto] = ACTIONS(3292),
    [anon_sym_await] = ACTIONS(3292),
    [anon_sym_become] = ACTIONS(3292),
    [anon_sym_break] = ACTIONS(3292),
    [anon_sym_const] = ACTIONS(3292),
    [anon_sym_continue] = ACTIONS(3292),
    [anon_sym_default] = ACTIONS(3292),
    [anon_sym_enum] = ACTIONS(3292),
    [anon_sym_fn] = ACTIONS(3292),
    [anon_sym_for] = ACTIONS(3292),
    [anon_sym_gen] = ACTIONS(3292),
    [anon_sym_if] = ACTIONS(3292),
    [anon_sym_impl] = ACTIONS(3292),
    [anon_sym_let] = ACTIONS(3292),
    [anon_sym_loop] = ACTIONS(3292),
    [anon_sym_match] = ACTIONS(3292),
    [anon_sym_mod] = ACTIONS(3292),
    [anon_sym_pub] = ACTIONS(3292),
    [anon_sym_return] = ACTIONS(3292),
    [anon_sym_raw] = ACTIONS(3292),
    [anon_sym_static] = ACTIONS(3292),
    [anon_sym_struct] = ACTIONS(3292),
    [anon_sym_trait] = ACTIONS(3292),
    [anon_sym_type] = ACTIONS(3292),
    [anon_sym_union] = ACTIONS(3292),
    [anon_sym_unsafe] = ACTIONS(3292),
    [anon_sym_use] = ACTIONS(3292),
    [anon_sym_while] = ACTIONS(3292),
    [anon_sym_extern] = ACTIONS(3292),
    [anon_sym_safe] = ACTIONS(3292),
    [anon_sym_yield] = ACTIONS(3292),
    [anon_sym_move] = ACTIONS(3292),
    [anon_sym_try] = ACTIONS(3292),
    [sym_integer_literal] = ACTIONS(3290),
    [aux_sym_string_literal_token1] = ACTIONS(3290),
    [sym_char_literal] = ACTIONS(3290),
    [anon_sym_true] = ACTIONS(3292),
    [anon_sym_false] = ACTIONS(3292),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3292),
    [sym_super] = ACTIONS(3292),
    [sym_crate] = ACTIONS(3292),
    [aux_sym_metavariable_token1] = ACTIONS(3290),
    [sym__raw_string_literal_start] = ACTIONS(3290),
    [sym_float_literal] = ACTIONS(3290),
    [sym__outer_block_doc_comment_start] = ACTIONS(3290),
    [sym__outer_line_doc_comment_start] = ACTIONS(3290),
  },
  [STATE(881)] = {
    [sym_line_comment] = STATE(881),
    [sym_block_comment] = STATE(881),
    [ts_builtin_sym_end] = ACTIONS(3294),
    [sym_identifier] = ACTIONS(3296),
    [anon_sym_SEMI] = ACTIONS(3294),
    [anon_sym_macro] = ACTIONS(3296),
    [anon_sym_LPAREN] = ACTIONS(3294),
    [anon_sym_LBRACE] = ACTIONS(3294),
    [anon_sym_RBRACE] = ACTIONS(3294),
    [anon_sym_macro_rules] = ACTIONS(3296),
    [anon_sym_BANG] = ACTIONS(3294),
    [anon_sym_LBRACK] = ACTIONS(3294),
    [anon_sym_STAR] = ACTIONS(3294),
    [anon_sym_u8] = ACTIONS(3296),
    [anon_sym_i8] = ACTIONS(3296),
    [anon_sym_u16] = ACTIONS(3296),
    [anon_sym_i16] = ACTIONS(3296),
    [anon_sym_u32] = ACTIONS(3296),
    [anon_sym_i32] = ACTIONS(3296),
    [anon_sym_u64] = ACTIONS(3296),
    [anon_sym_i64] = ACTIONS(3296),
    [anon_sym_u128] = ACTIONS(3296),
    [anon_sym_i128] = ACTIONS(3296),
    [anon_sym_isize] = ACTIONS(3296),
    [anon_sym_usize] = ACTIONS(3296),
    [anon_sym_f32] = ACTIONS(3296),
    [anon_sym_f64] = ACTIONS(3296),
    [anon_sym_bool] = ACTIONS(3296),
    [anon_sym_str] = ACTIONS(3296),
    [anon_sym_char] = ACTIONS(3296),
    [anon_sym_DASH] = ACTIONS(3294),
    [anon_sym_AMP] = ACTIONS(3294),
    [anon_sym_PIPE] = ACTIONS(3294),
    [anon_sym_LT] = ACTIONS(3294),
    [anon_sym_DOT_DOT] = ACTIONS(3294),
    [anon_sym_COLON_COLON] = ACTIONS(3294),
    [anon_sym_POUND] = ACTIONS(3294),
    [anon_sym_SQUOTE] = ACTIONS(3296),
    [anon_sym_async] = ACTIONS(3296),
    [anon_sym_auto] = ACTIONS(3296),
    [anon_sym_await] = ACTIONS(3296),
    [anon_sym_become] = ACTIONS(3296),
    [anon_sym_break] = ACTIONS(3296),
    [anon_sym_const] = ACTIONS(3296),
    [anon_sym_continue] = ACTIONS(3296),
    [anon_sym_default] = ACTIONS(3296),
    [anon_sym_enum] = ACTIONS(3296),
    [anon_sym_fn] = ACTIONS(3296),
    [anon_sym_for] = ACTIONS(3296),
    [anon_sym_gen] = ACTIONS(3296),
    [anon_sym_if] = ACTIONS(3296),
    [anon_sym_impl] = ACTIONS(3296),
    [anon_sym_let] = ACTIONS(3296),
    [anon_sym_loop] = ACTIONS(3296),
    [anon_sym_match] = ACTIONS(3296),
    [anon_sym_mod] = ACTIONS(3296),
    [anon_sym_pub] = ACTIONS(3296),
    [anon_sym_return] = ACTIONS(3296),
    [anon_sym_raw] = ACTIONS(3296),
    [anon_sym_static] = ACTIONS(3296),
    [anon_sym_struct] = ACTIONS(3296),
    [anon_sym_trait] = ACTIONS(3296),
    [anon_sym_type] = ACTIONS(3296),
    [anon_sym_union] = ACTIONS(3296),
    [anon_sym_unsafe] = ACTIONS(3296),
    [anon_sym_use] = ACTIONS(3296),
    [anon_sym_while] = ACTIONS(3296),
    [anon_sym_extern] = ACTIONS(3296),
    [anon_sym_safe] = ACTIONS(3296),
    [anon_sym_yield] = ACTIONS(3296),
    [anon_sym_move] = ACTIONS(3296),
    [anon_sym_try] = ACTIONS(3296),
    [sym_integer_literal] = ACTIONS(3294),
    [aux_sym_string_literal_token1] = ACTIONS(3294),
    [sym_char_literal] = ACTIONS(3294),
    [anon_sym_true] = ACTIONS(3296),
    [anon_sym_false] = ACTIONS(3296),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3296),
    [sym_super] = ACTIONS(3296),
    [sym_crate] = ACTIONS(3296),
    [aux_sym_metavariable_token1] = ACTIONS(3294),
    [sym__raw_string_literal_start] = ACTIONS(3294),
    [sym_float_literal] = ACTIONS(3294),
    [sym__outer_block_doc_comment_start] = ACTIONS(3294),
    [sym__outer_line_doc_comment_start] = ACTIONS(3294),
  },
  [STATE(882)] = {
    [sym_line_comment] = STATE(882),
    [sym_block_comment] = STATE(882),
    [ts_builtin_sym_end] = ACTIONS(3298),
    [sym_identifier] = ACTIONS(3300),
    [anon_sym_SEMI] = ACTIONS(3298),
    [anon_sym_macro] = ACTIONS(3300),
    [anon_sym_LPAREN] = ACTIONS(3298),
    [anon_sym_LBRACE] = ACTIONS(3298),
    [anon_sym_RBRACE] = ACTIONS(3298),
    [anon_sym_macro_rules] = ACTIONS(3300),
    [anon_sym_BANG] = ACTIONS(3298),
    [anon_sym_LBRACK] = ACTIONS(3298),
    [anon_sym_STAR] = ACTIONS(3298),
    [anon_sym_u8] = ACTIONS(3300),
    [anon_sym_i8] = ACTIONS(3300),
    [anon_sym_u16] = ACTIONS(3300),
    [anon_sym_i16] = ACTIONS(3300),
    [anon_sym_u32] = ACTIONS(3300),
    [anon_sym_i32] = ACTIONS(3300),
    [anon_sym_u64] = ACTIONS(3300),
    [anon_sym_i64] = ACTIONS(3300),
    [anon_sym_u128] = ACTIONS(3300),
    [anon_sym_i128] = ACTIONS(3300),
    [anon_sym_isize] = ACTIONS(3300),
    [anon_sym_usize] = ACTIONS(3300),
    [anon_sym_f32] = ACTIONS(3300),
    [anon_sym_f64] = ACTIONS(3300),
    [anon_sym_bool] = ACTIONS(3300),
    [anon_sym_str] = ACTIONS(3300),
    [anon_sym_char] = ACTIONS(3300),
    [anon_sym_DASH] = ACTIONS(3298),
    [anon_sym_AMP] = ACTIONS(3298),
    [anon_sym_PIPE] = ACTIONS(3298),
    [anon_sym_LT] = ACTIONS(3298),
    [anon_sym_DOT_DOT] = ACTIONS(3298),
    [anon_sym_COLON_COLON] = ACTIONS(3298),
    [anon_sym_POUND] = ACTIONS(3298),
    [anon_sym_SQUOTE] = ACTIONS(3300),
    [anon_sym_async] = ACTIONS(3300),
    [anon_sym_auto] = ACTIONS(3300),
    [anon_sym_await] = ACTIONS(3300),
    [anon_sym_become] = ACTIONS(3300),
    [anon_sym_break] = ACTIONS(3300),
    [anon_sym_const] = ACTIONS(3300),
    [anon_sym_continue] = ACTIONS(3300),
    [anon_sym_default] = ACTIONS(3300),
    [anon_sym_enum] = ACTIONS(3300),
    [anon_sym_fn] = ACTIONS(3300),
    [anon_sym_for] = ACTIONS(3300),
    [anon_sym_gen] = ACTIONS(3300),
    [anon_sym_if] = ACTIONS(3300),
    [anon_sym_impl] = ACTIONS(3300),
    [anon_sym_let] = ACTIONS(3300),
    [anon_sym_loop] = ACTIONS(3300),
    [anon_sym_match] = ACTIONS(3300),
    [anon_sym_mod] = ACTIONS(3300),
    [anon_sym_pub] = ACTIONS(3300),
    [anon_sym_return] = ACTIONS(3300),
    [anon_sym_raw] = ACTIONS(3300),
    [anon_sym_static] = ACTIONS(3300),
    [anon_sym_struct] = ACTIONS(3300),
    [anon_sym_trait] = ACTIONS(3300),
    [anon_sym_type] = ACTIONS(3300),
    [anon_sym_union] = ACTIONS(3300),
    [anon_sym_unsafe] = ACTIONS(3300),
    [anon_sym_use] = ACTIONS(3300),
    [anon_sym_while] = ACTIONS(3300),
    [anon_sym_extern] = ACTIONS(3300),
    [anon_sym_safe] = ACTIONS(3300),
    [anon_sym_yield] = ACTIONS(3300),
    [anon_sym_move] = ACTIONS(3300),
    [anon_sym_try] = ACTIONS(3300),
    [sym_integer_literal] = ACTIONS(3298),
    [aux_sym_string_literal_token1] = ACTIONS(3298),
    [sym_char_literal] = ACTIONS(3298),
    [anon_sym_true] = ACTIONS(3300),
    [anon_sym_false] = ACTIONS(3300),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3300),
    [sym_super] = ACTIONS(3300),
    [sym_crate] = ACTIONS(3300),
    [aux_sym_metavariable_token1] = ACTIONS(3298),
    [sym__raw_string_literal_start] = ACTIONS(3298),
    [sym_float_literal] = ACTIONS(3298),
    [sym__outer_block_doc_comment_start] = ACTIONS(3298),
    [sym__outer_line_doc_comment_start] = ACTIONS(3298),
  },
  [STATE(883)] = {
    [sym_line_comment] = STATE(883),
    [sym_block_comment] = STATE(883),
    [ts_builtin_sym_end] = ACTIONS(3302),
    [sym_identifier] = ACTIONS(3304),
    [anon_sym_SEMI] = ACTIONS(3302),
    [anon_sym_macro] = ACTIONS(3304),
    [anon_sym_LPAREN] = ACTIONS(3302),
    [anon_sym_LBRACE] = ACTIONS(3302),
    [anon_sym_RBRACE] = ACTIONS(3302),
    [anon_sym_macro_rules] = ACTIONS(3304),
    [anon_sym_BANG] = ACTIONS(3302),
    [anon_sym_LBRACK] = ACTIONS(3302),
    [anon_sym_STAR] = ACTIONS(3302),
    [anon_sym_u8] = ACTIONS(3304),
    [anon_sym_i8] = ACTIONS(3304),
    [anon_sym_u16] = ACTIONS(3304),
    [anon_sym_i16] = ACTIONS(3304),
    [anon_sym_u32] = ACTIONS(3304),
    [anon_sym_i32] = ACTIONS(3304),
    [anon_sym_u64] = ACTIONS(3304),
    [anon_sym_i64] = ACTIONS(3304),
    [anon_sym_u128] = ACTIONS(3304),
    [anon_sym_i128] = ACTIONS(3304),
    [anon_sym_isize] = ACTIONS(3304),
    [anon_sym_usize] = ACTIONS(3304),
    [anon_sym_f32] = ACTIONS(3304),
    [anon_sym_f64] = ACTIONS(3304),
    [anon_sym_bool] = ACTIONS(3304),
    [anon_sym_str] = ACTIONS(3304),
    [anon_sym_char] = ACTIONS(3304),
    [anon_sym_DASH] = ACTIONS(3302),
    [anon_sym_AMP] = ACTIONS(3302),
    [anon_sym_PIPE] = ACTIONS(3302),
    [anon_sym_LT] = ACTIONS(3302),
    [anon_sym_DOT_DOT] = ACTIONS(3302),
    [anon_sym_COLON_COLON] = ACTIONS(3302),
    [anon_sym_POUND] = ACTIONS(3302),
    [anon_sym_SQUOTE] = ACTIONS(3304),
    [anon_sym_async] = ACTIONS(3304),
    [anon_sym_auto] = ACTIONS(3304),
    [anon_sym_await] = ACTIONS(3304),
    [anon_sym_become] = ACTIONS(3304),
    [anon_sym_break] = ACTIONS(3304),
    [anon_sym_const] = ACTIONS(3304),
    [anon_sym_continue] = ACTIONS(3304),
    [anon_sym_default] = ACTIONS(3304),
    [anon_sym_enum] = ACTIONS(3304),
    [anon_sym_fn] = ACTIONS(3304),
    [anon_sym_for] = ACTIONS(3304),
    [anon_sym_gen] = ACTIONS(3304),
    [anon_sym_if] = ACTIONS(3304),
    [anon_sym_impl] = ACTIONS(3304),
    [anon_sym_let] = ACTIONS(3304),
    [anon_sym_loop] = ACTIONS(3304),
    [anon_sym_match] = ACTIONS(3304),
    [anon_sym_mod] = ACTIONS(3304),
    [anon_sym_pub] = ACTIONS(3304),
    [anon_sym_return] = ACTIONS(3304),
    [anon_sym_raw] = ACTIONS(3304),
    [anon_sym_static] = ACTIONS(3304),
    [anon_sym_struct] = ACTIONS(3304),
    [anon_sym_trait] = ACTIONS(3304),
    [anon_sym_type] = ACTIONS(3304),
    [anon_sym_union] = ACTIONS(3304),
    [anon_sym_unsafe] = ACTIONS(3304),
    [anon_sym_use] = ACTIONS(3304),
    [anon_sym_while] = ACTIONS(3304),
    [anon_sym_extern] = ACTIONS(3304),
    [anon_sym_safe] = ACTIONS(3304),
    [anon_sym_yield] = ACTIONS(3304),
    [anon_sym_move] = ACTIONS(3304),
    [anon_sym_try] = ACTIONS(3304),
    [sym_integer_literal] = ACTIONS(3302),
    [aux_sym_string_literal_token1] = ACTIONS(3302),
    [sym_char_literal] = ACTIONS(3302),
    [anon_sym_true] = ACTIONS(3304),
    [anon_sym_false] = ACTIONS(3304),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3304),
    [sym_super] = ACTIONS(3304),
    [sym_crate] = ACTIONS(3304),
    [aux_sym_metavariable_token1] = ACTIONS(3302),
    [sym__raw_string_literal_start] = ACTIONS(3302),
    [sym_float_literal] = ACTIONS(3302),
    [sym__outer_block_doc_comment_start] = ACTIONS(3302),
    [sym__outer_line_doc_comment_start] = ACTIONS(3302),
  },
  [STATE(884)] = {
    [sym_line_comment] = STATE(884),
    [sym_block_comment] = STATE(884),
    [ts_builtin_sym_end] = ACTIONS(3306),
    [sym_identifier] = ACTIONS(3308),
    [anon_sym_SEMI] = ACTIONS(3306),
    [anon_sym_macro] = ACTIONS(3308),
    [anon_sym_LPAREN] = ACTIONS(3306),
    [anon_sym_LBRACE] = ACTIONS(3306),
    [anon_sym_RBRACE] = ACTIONS(3306),
    [anon_sym_macro_rules] = ACTIONS(3308),
    [anon_sym_BANG] = ACTIONS(3306),
    [anon_sym_LBRACK] = ACTIONS(3306),
    [anon_sym_STAR] = ACTIONS(3306),
    [anon_sym_u8] = ACTIONS(3308),
    [anon_sym_i8] = ACTIONS(3308),
    [anon_sym_u16] = ACTIONS(3308),
    [anon_sym_i16] = ACTIONS(3308),
    [anon_sym_u32] = ACTIONS(3308),
    [anon_sym_i32] = ACTIONS(3308),
    [anon_sym_u64] = ACTIONS(3308),
    [anon_sym_i64] = ACTIONS(3308),
    [anon_sym_u128] = ACTIONS(3308),
    [anon_sym_i128] = ACTIONS(3308),
    [anon_sym_isize] = ACTIONS(3308),
    [anon_sym_usize] = ACTIONS(3308),
    [anon_sym_f32] = ACTIONS(3308),
    [anon_sym_f64] = ACTIONS(3308),
    [anon_sym_bool] = ACTIONS(3308),
    [anon_sym_str] = ACTIONS(3308),
    [anon_sym_char] = ACTIONS(3308),
    [anon_sym_DASH] = ACTIONS(3306),
    [anon_sym_AMP] = ACTIONS(3306),
    [anon_sym_PIPE] = ACTIONS(3306),
    [anon_sym_LT] = ACTIONS(3306),
    [anon_sym_DOT_DOT] = ACTIONS(3306),
    [anon_sym_COLON_COLON] = ACTIONS(3306),
    [anon_sym_POUND] = ACTIONS(3306),
    [anon_sym_SQUOTE] = ACTIONS(3308),
    [anon_sym_async] = ACTIONS(3308),
    [anon_sym_auto] = ACTIONS(3308),
    [anon_sym_await] = ACTIONS(3308),
    [anon_sym_become] = ACTIONS(3308),
    [anon_sym_break] = ACTIONS(3308),
    [anon_sym_const] = ACTIONS(3308),
    [anon_sym_continue] = ACTIONS(3308),
    [anon_sym_default] = ACTIONS(3308),
    [anon_sym_enum] = ACTIONS(3308),
    [anon_sym_fn] = ACTIONS(3308),
    [anon_sym_for] = ACTIONS(3308),
    [anon_sym_gen] = ACTIONS(3308),
    [anon_sym_if] = ACTIONS(3308),
    [anon_sym_impl] = ACTIONS(3308),
    [anon_sym_let] = ACTIONS(3308),
    [anon_sym_loop] = ACTIONS(3308),
    [anon_sym_match] = ACTIONS(3308),
    [anon_sym_mod] = ACTIONS(3308),
    [anon_sym_pub] = ACTIONS(3308),
    [anon_sym_return] = ACTIONS(3308),
    [anon_sym_raw] = ACTIONS(3308),
    [anon_sym_static] = ACTIONS(3308),
    [anon_sym_struct] = ACTIONS(3308),
    [anon_sym_trait] = ACTIONS(3308),
    [anon_sym_type] = ACTIONS(3308),
    [anon_sym_union] = ACTIONS(3308),
    [anon_sym_unsafe] = ACTIONS(3308),
    [anon_sym_use] = ACTIONS(3308),
    [anon_sym_while] = ACTIONS(3308),
    [anon_sym_extern] = ACTIONS(3308),
    [anon_sym_safe] = ACTIONS(3308),
    [anon_sym_yield] = ACTIONS(3308),
    [anon_sym_move] = ACTIONS(3308),
    [anon_sym_try] = ACTIONS(3308),
    [sym_integer_literal] = ACTIONS(3306),
    [aux_sym_string_literal_token1] = ACTIONS(3306),
    [sym_char_literal] = ACTIONS(3306),
    [anon_sym_true] = ACTIONS(3308),
    [anon_sym_false] = ACTIONS(3308),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3308),
    [sym_super] = ACTIONS(3308),
    [sym_crate] = ACTIONS(3308),
    [aux_sym_metavariable_token1] = ACTIONS(3306),
    [sym__raw_string_literal_start] = ACTIONS(3306),
    [sym_float_literal] = ACTIONS(3306),
    [sym__outer_block_doc_comment_start] = ACTIONS(3306),
    [sym__outer_line_doc_comment_start] = ACTIONS(3306),
  },
  [STATE(885)] = {
    [sym_line_comment] = STATE(885),
    [sym_block_comment] = STATE(885),
    [ts_builtin_sym_end] = ACTIONS(3310),
    [sym_identifier] = ACTIONS(3312),
    [anon_sym_SEMI] = ACTIONS(3310),
    [anon_sym_macro] = ACTIONS(3312),
    [anon_sym_LPAREN] = ACTIONS(3310),
    [anon_sym_LBRACE] = ACTIONS(3310),
    [anon_sym_RBRACE] = ACTIONS(3310),
    [anon_sym_macro_rules] = ACTIONS(3312),
    [anon_sym_BANG] = ACTIONS(3310),
    [anon_sym_LBRACK] = ACTIONS(3310),
    [anon_sym_STAR] = ACTIONS(3310),
    [anon_sym_u8] = ACTIONS(3312),
    [anon_sym_i8] = ACTIONS(3312),
    [anon_sym_u16] = ACTIONS(3312),
    [anon_sym_i16] = ACTIONS(3312),
    [anon_sym_u32] = ACTIONS(3312),
    [anon_sym_i32] = ACTIONS(3312),
    [anon_sym_u64] = ACTIONS(3312),
    [anon_sym_i64] = ACTIONS(3312),
    [anon_sym_u128] = ACTIONS(3312),
    [anon_sym_i128] = ACTIONS(3312),
    [anon_sym_isize] = ACTIONS(3312),
    [anon_sym_usize] = ACTIONS(3312),
    [anon_sym_f32] = ACTIONS(3312),
    [anon_sym_f64] = ACTIONS(3312),
    [anon_sym_bool] = ACTIONS(3312),
    [anon_sym_str] = ACTIONS(3312),
    [anon_sym_char] = ACTIONS(3312),
    [anon_sym_DASH] = ACTIONS(3310),
    [anon_sym_AMP] = ACTIONS(3310),
    [anon_sym_PIPE] = ACTIONS(3310),
    [anon_sym_LT] = ACTIONS(3310),
    [anon_sym_DOT_DOT] = ACTIONS(3310),
    [anon_sym_COLON_COLON] = ACTIONS(3310),
    [anon_sym_POUND] = ACTIONS(3310),
    [anon_sym_SQUOTE] = ACTIONS(3312),
    [anon_sym_async] = ACTIONS(3312),
    [anon_sym_auto] = ACTIONS(3312),
    [anon_sym_await] = ACTIONS(3312),
    [anon_sym_become] = ACTIONS(3312),
    [anon_sym_break] = ACTIONS(3312),
    [anon_sym_const] = ACTIONS(3312),
    [anon_sym_continue] = ACTIONS(3312),
    [anon_sym_default] = ACTIONS(3312),
    [anon_sym_enum] = ACTIONS(3312),
    [anon_sym_fn] = ACTIONS(3312),
    [anon_sym_for] = ACTIONS(3312),
    [anon_sym_gen] = ACTIONS(3312),
    [anon_sym_if] = ACTIONS(3312),
    [anon_sym_impl] = ACTIONS(3312),
    [anon_sym_let] = ACTIONS(3312),
    [anon_sym_loop] = ACTIONS(3312),
    [anon_sym_match] = ACTIONS(3312),
    [anon_sym_mod] = ACTIONS(3312),
    [anon_sym_pub] = ACTIONS(3312),
    [anon_sym_return] = ACTIONS(3312),
    [anon_sym_raw] = ACTIONS(3312),
    [anon_sym_static] = ACTIONS(3312),
    [anon_sym_struct] = ACTIONS(3312),
    [anon_sym_trait] = ACTIONS(3312),
    [anon_sym_type] = ACTIONS(3312),
    [anon_sym_union] = ACTIONS(3312),
    [anon_sym_unsafe] = ACTIONS(3312),
    [anon_sym_use] = ACTIONS(3312),
    [anon_sym_while] = ACTIONS(3312),
    [anon_sym_extern] = ACTIONS(3312),
    [anon_sym_safe] = ACTIONS(3312),
    [anon_sym_yield] = ACTIONS(3312),
    [anon_sym_move] = ACTIONS(3312),
    [anon_sym_try] = ACTIONS(3312),
    [sym_integer_literal] = ACTIONS(3310),
    [aux_sym_string_literal_token1] = ACTIONS(3310),
    [sym_char_literal] = ACTIONS(3310),
    [anon_sym_true] = ACTIONS(3312),
    [anon_sym_false] = ACTIONS(3312),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3312),
    [sym_super] = ACTIONS(3312),
    [sym_crate] = ACTIONS(3312),
    [aux_sym_metavariable_token1] = ACTIONS(3310),
    [sym__raw_string_literal_start] = ACTIONS(3310),
    [sym_float_literal] = ACTIONS(3310),
    [sym__outer_block_doc_comment_start] = ACTIONS(3310),
    [sym__outer_line_doc_comment_start] = ACTIONS(3310),
  },
  [STATE(886)] = {
    [sym_line_comment] = STATE(886),
    [sym_block_comment] = STATE(886),
    [ts_builtin_sym_end] = ACTIONS(3314),
    [sym_identifier] = ACTIONS(3316),
    [anon_sym_SEMI] = ACTIONS(3314),
    [anon_sym_macro] = ACTIONS(3316),
    [anon_sym_LPAREN] = ACTIONS(3314),
    [anon_sym_LBRACE] = ACTIONS(3314),
    [anon_sym_RBRACE] = ACTIONS(3314),
    [anon_sym_macro_rules] = ACTIONS(3316),
    [anon_sym_BANG] = ACTIONS(3314),
    [anon_sym_LBRACK] = ACTIONS(3314),
    [anon_sym_STAR] = ACTIONS(3314),
    [anon_sym_u8] = ACTIONS(3316),
    [anon_sym_i8] = ACTIONS(3316),
    [anon_sym_u16] = ACTIONS(3316),
    [anon_sym_i16] = ACTIONS(3316),
    [anon_sym_u32] = ACTIONS(3316),
    [anon_sym_i32] = ACTIONS(3316),
    [anon_sym_u64] = ACTIONS(3316),
    [anon_sym_i64] = ACTIONS(3316),
    [anon_sym_u128] = ACTIONS(3316),
    [anon_sym_i128] = ACTIONS(3316),
    [anon_sym_isize] = ACTIONS(3316),
    [anon_sym_usize] = ACTIONS(3316),
    [anon_sym_f32] = ACTIONS(3316),
    [anon_sym_f64] = ACTIONS(3316),
    [anon_sym_bool] = ACTIONS(3316),
    [anon_sym_str] = ACTIONS(3316),
    [anon_sym_char] = ACTIONS(3316),
    [anon_sym_DASH] = ACTIONS(3314),
    [anon_sym_AMP] = ACTIONS(3314),
    [anon_sym_PIPE] = ACTIONS(3314),
    [anon_sym_LT] = ACTIONS(3314),
    [anon_sym_DOT_DOT] = ACTIONS(3314),
    [anon_sym_COLON_COLON] = ACTIONS(3314),
    [anon_sym_POUND] = ACTIONS(3314),
    [anon_sym_SQUOTE] = ACTIONS(3316),
    [anon_sym_async] = ACTIONS(3316),
    [anon_sym_auto] = ACTIONS(3316),
    [anon_sym_await] = ACTIONS(3316),
    [anon_sym_become] = ACTIONS(3316),
    [anon_sym_break] = ACTIONS(3316),
    [anon_sym_const] = ACTIONS(3316),
    [anon_sym_continue] = ACTIONS(3316),
    [anon_sym_default] = ACTIONS(3316),
    [anon_sym_enum] = ACTIONS(3316),
    [anon_sym_fn] = ACTIONS(3316),
    [anon_sym_for] = ACTIONS(3316),
    [anon_sym_gen] = ACTIONS(3316),
    [anon_sym_if] = ACTIONS(3316),
    [anon_sym_impl] = ACTIONS(3316),
    [anon_sym_let] = ACTIONS(3316),
    [anon_sym_loop] = ACTIONS(3316),
    [anon_sym_match] = ACTIONS(3316),
    [anon_sym_mod] = ACTIONS(3316),
    [anon_sym_pub] = ACTIONS(3316),
    [anon_sym_return] = ACTIONS(3316),
    [anon_sym_raw] = ACTIONS(3316),
    [anon_sym_static] = ACTIONS(3316),
    [anon_sym_struct] = ACTIONS(3316),
    [anon_sym_trait] = ACTIONS(3316),
    [anon_sym_type] = ACTIONS(3316),
    [anon_sym_union] = ACTIONS(3316),
    [anon_sym_unsafe] = ACTIONS(3316),
    [anon_sym_use] = ACTIONS(3316),
    [anon_sym_while] = ACTIONS(3316),
    [anon_sym_extern] = ACTIONS(3316),
    [anon_sym_safe] = ACTIONS(3316),
    [anon_sym_yield] = ACTIONS(3316),
    [anon_sym_move] = ACTIONS(3316),
    [anon_sym_try] = ACTIONS(3316),
    [sym_integer_literal] = ACTIONS(3314),
    [aux_sym_string_literal_token1] = ACTIONS(3314),
    [sym_char_literal] = ACTIONS(3314),
    [anon_sym_true] = ACTIONS(3316),
    [anon_sym_false] = ACTIONS(3316),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3316),
    [sym_super] = ACTIONS(3316),
    [sym_crate] = ACTIONS(3316),
    [aux_sym_metavariable_token1] = ACTIONS(3314),
    [sym__raw_string_literal_start] = ACTIONS(3314),
    [sym_float_literal] = ACTIONS(3314),
    [sym__outer_block_doc_comment_start] = ACTIONS(3314),
    [sym__outer_line_doc_comment_start] = ACTIONS(3314),
  },
  [STATE(887)] = {
    [sym_line_comment] = STATE(887),
    [sym_block_comment] = STATE(887),
    [ts_builtin_sym_end] = ACTIONS(3318),
    [sym_identifier] = ACTIONS(3320),
    [anon_sym_SEMI] = ACTIONS(3318),
    [anon_sym_macro] = ACTIONS(3320),
    [anon_sym_LPAREN] = ACTIONS(3318),
    [anon_sym_LBRACE] = ACTIONS(3318),
    [anon_sym_RBRACE] = ACTIONS(3318),
    [anon_sym_macro_rules] = ACTIONS(3320),
    [anon_sym_BANG] = ACTIONS(3318),
    [anon_sym_LBRACK] = ACTIONS(3318),
    [anon_sym_STAR] = ACTIONS(3318),
    [anon_sym_u8] = ACTIONS(3320),
    [anon_sym_i8] = ACTIONS(3320),
    [anon_sym_u16] = ACTIONS(3320),
    [anon_sym_i16] = ACTIONS(3320),
    [anon_sym_u32] = ACTIONS(3320),
    [anon_sym_i32] = ACTIONS(3320),
    [anon_sym_u64] = ACTIONS(3320),
    [anon_sym_i64] = ACTIONS(3320),
    [anon_sym_u128] = ACTIONS(3320),
    [anon_sym_i128] = ACTIONS(3320),
    [anon_sym_isize] = ACTIONS(3320),
    [anon_sym_usize] = ACTIONS(3320),
    [anon_sym_f32] = ACTIONS(3320),
    [anon_sym_f64] = ACTIONS(3320),
    [anon_sym_bool] = ACTIONS(3320),
    [anon_sym_str] = ACTIONS(3320),
    [anon_sym_char] = ACTIONS(3320),
    [anon_sym_DASH] = ACTIONS(3318),
    [anon_sym_AMP] = ACTIONS(3318),
    [anon_sym_PIPE] = ACTIONS(3318),
    [anon_sym_LT] = ACTIONS(3318),
    [anon_sym_DOT_DOT] = ACTIONS(3318),
    [anon_sym_COLON_COLON] = ACTIONS(3318),
    [anon_sym_POUND] = ACTIONS(3318),
    [anon_sym_SQUOTE] = ACTIONS(3320),
    [anon_sym_async] = ACTIONS(3320),
    [anon_sym_auto] = ACTIONS(3320),
    [anon_sym_await] = ACTIONS(3320),
    [anon_sym_become] = ACTIONS(3320),
    [anon_sym_break] = ACTIONS(3320),
    [anon_sym_const] = ACTIONS(3320),
    [anon_sym_continue] = ACTIONS(3320),
    [anon_sym_default] = ACTIONS(3320),
    [anon_sym_enum] = ACTIONS(3320),
    [anon_sym_fn] = ACTIONS(3320),
    [anon_sym_for] = ACTIONS(3320),
    [anon_sym_gen] = ACTIONS(3320),
    [anon_sym_if] = ACTIONS(3320),
    [anon_sym_impl] = ACTIONS(3320),
    [anon_sym_let] = ACTIONS(3320),
    [anon_sym_loop] = ACTIONS(3320),
    [anon_sym_match] = ACTIONS(3320),
    [anon_sym_mod] = ACTIONS(3320),
    [anon_sym_pub] = ACTIONS(3320),
    [anon_sym_return] = ACTIONS(3320),
    [anon_sym_raw] = ACTIONS(3320),
    [anon_sym_static] = ACTIONS(3320),
    [anon_sym_struct] = ACTIONS(3320),
    [anon_sym_trait] = ACTIONS(3320),
    [anon_sym_type] = ACTIONS(3320),
    [anon_sym_union] = ACTIONS(3320),
    [anon_sym_unsafe] = ACTIONS(3320),
    [anon_sym_use] = ACTIONS(3320),
    [anon_sym_while] = ACTIONS(3320),
    [anon_sym_extern] = ACTIONS(3320),
    [anon_sym_safe] = ACTIONS(3320),
    [anon_sym_yield] = ACTIONS(3320),
    [anon_sym_move] = ACTIONS(3320),
    [anon_sym_try] = ACTIONS(3320),
    [sym_integer_literal] = ACTIONS(3318),
    [aux_sym_string_literal_token1] = ACTIONS(3318),
    [sym_char_literal] = ACTIONS(3318),
    [anon_sym_true] = ACTIONS(3320),
    [anon_sym_false] = ACTIONS(3320),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3320),
    [sym_super] = ACTIONS(3320),
    [sym_crate] = ACTIONS(3320),
    [aux_sym_metavariable_token1] = ACTIONS(3318),
    [sym__raw_string_literal_start] = ACTIONS(3318),
    [sym_float_literal] = ACTIONS(3318),
    [sym__outer_block_doc_comment_start] = ACTIONS(3318),
    [sym__outer_line_doc_comment_start] = ACTIONS(3318),
  },
  [STATE(888)] = {
    [sym_line_comment] = STATE(888),
    [sym_block_comment] = STATE(888),
    [ts_builtin_sym_end] = ACTIONS(3322),
    [sym_identifier] = ACTIONS(3324),
    [anon_sym_SEMI] = ACTIONS(3322),
    [anon_sym_macro] = ACTIONS(3324),
    [anon_sym_LPAREN] = ACTIONS(3322),
    [anon_sym_LBRACE] = ACTIONS(3322),
    [anon_sym_RBRACE] = ACTIONS(3322),
    [anon_sym_macro_rules] = ACTIONS(3324),
    [anon_sym_BANG] = ACTIONS(3322),
    [anon_sym_LBRACK] = ACTIONS(3322),
    [anon_sym_STAR] = ACTIONS(3322),
    [anon_sym_u8] = ACTIONS(3324),
    [anon_sym_i8] = ACTIONS(3324),
    [anon_sym_u16] = ACTIONS(3324),
    [anon_sym_i16] = ACTIONS(3324),
    [anon_sym_u32] = ACTIONS(3324),
    [anon_sym_i32] = ACTIONS(3324),
    [anon_sym_u64] = ACTIONS(3324),
    [anon_sym_i64] = ACTIONS(3324),
    [anon_sym_u128] = ACTIONS(3324),
    [anon_sym_i128] = ACTIONS(3324),
    [anon_sym_isize] = ACTIONS(3324),
    [anon_sym_usize] = ACTIONS(3324),
    [anon_sym_f32] = ACTIONS(3324),
    [anon_sym_f64] = ACTIONS(3324),
    [anon_sym_bool] = ACTIONS(3324),
    [anon_sym_str] = ACTIONS(3324),
    [anon_sym_char] = ACTIONS(3324),
    [anon_sym_DASH] = ACTIONS(3322),
    [anon_sym_AMP] = ACTIONS(3322),
    [anon_sym_PIPE] = ACTIONS(3322),
    [anon_sym_LT] = ACTIONS(3322),
    [anon_sym_DOT_DOT] = ACTIONS(3322),
    [anon_sym_COLON_COLON] = ACTIONS(3322),
    [anon_sym_POUND] = ACTIONS(3322),
    [anon_sym_SQUOTE] = ACTIONS(3324),
    [anon_sym_async] = ACTIONS(3324),
    [anon_sym_auto] = ACTIONS(3324),
    [anon_sym_await] = ACTIONS(3324),
    [anon_sym_become] = ACTIONS(3324),
    [anon_sym_break] = ACTIONS(3324),
    [anon_sym_const] = ACTIONS(3324),
    [anon_sym_continue] = ACTIONS(3324),
    [anon_sym_default] = ACTIONS(3324),
    [anon_sym_enum] = ACTIONS(3324),
    [anon_sym_fn] = ACTIONS(3324),
    [anon_sym_for] = ACTIONS(3324),
    [anon_sym_gen] = ACTIONS(3324),
    [anon_sym_if] = ACTIONS(3324),
    [anon_sym_impl] = ACTIONS(3324),
    [anon_sym_let] = ACTIONS(3324),
    [anon_sym_loop] = ACTIONS(3324),
    [anon_sym_match] = ACTIONS(3324),
    [anon_sym_mod] = ACTIONS(3324),
    [anon_sym_pub] = ACTIONS(3324),
    [anon_sym_return] = ACTIONS(3324),
    [anon_sym_raw] = ACTIONS(3324),
    [anon_sym_static] = ACTIONS(3324),
    [anon_sym_struct] = ACTIONS(3324),
    [anon_sym_trait] = ACTIONS(3324),
    [anon_sym_type] = ACTIONS(3324),
    [anon_sym_union] = ACTIONS(3324),
    [anon_sym_unsafe] = ACTIONS(3324),
    [anon_sym_use] = ACTIONS(3324),
    [anon_sym_while] = ACTIONS(3324),
    [anon_sym_extern] = ACTIONS(3324),
    [anon_sym_safe] = ACTIONS(3324),
    [anon_sym_yield] = ACTIONS(3324),
    [anon_sym_move] = ACTIONS(3324),
    [anon_sym_try] = ACTIONS(3324),
    [sym_integer_literal] = ACTIONS(3322),
    [aux_sym_string_literal_token1] = ACTIONS(3322),
    [sym_char_literal] = ACTIONS(3322),
    [anon_sym_true] = ACTIONS(3324),
    [anon_sym_false] = ACTIONS(3324),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3324),
    [sym_super] = ACTIONS(3324),
    [sym_crate] = ACTIONS(3324),
    [aux_sym_metavariable_token1] = ACTIONS(3322),
    [sym__raw_string_literal_start] = ACTIONS(3322),
    [sym_float_literal] = ACTIONS(3322),
    [sym__outer_block_doc_comment_start] = ACTIONS(3322),
    [sym__outer_line_doc_comment_start] = ACTIONS(3322),
  },
  [STATE(889)] = {
    [sym_line_comment] = STATE(889),
    [sym_block_comment] = STATE(889),
    [ts_builtin_sym_end] = ACTIONS(3326),
    [sym_identifier] = ACTIONS(3328),
    [anon_sym_SEMI] = ACTIONS(3326),
    [anon_sym_macro] = ACTIONS(3328),
    [anon_sym_LPAREN] = ACTIONS(3326),
    [anon_sym_LBRACE] = ACTIONS(3326),
    [anon_sym_RBRACE] = ACTIONS(3326),
    [anon_sym_macro_rules] = ACTIONS(3328),
    [anon_sym_BANG] = ACTIONS(3326),
    [anon_sym_LBRACK] = ACTIONS(3326),
    [anon_sym_STAR] = ACTIONS(3326),
    [anon_sym_u8] = ACTIONS(3328),
    [anon_sym_i8] = ACTIONS(3328),
    [anon_sym_u16] = ACTIONS(3328),
    [anon_sym_i16] = ACTIONS(3328),
    [anon_sym_u32] = ACTIONS(3328),
    [anon_sym_i32] = ACTIONS(3328),
    [anon_sym_u64] = ACTIONS(3328),
    [anon_sym_i64] = ACTIONS(3328),
    [anon_sym_u128] = ACTIONS(3328),
    [anon_sym_i128] = ACTIONS(3328),
    [anon_sym_isize] = ACTIONS(3328),
    [anon_sym_usize] = ACTIONS(3328),
    [anon_sym_f32] = ACTIONS(3328),
    [anon_sym_f64] = ACTIONS(3328),
    [anon_sym_bool] = ACTIONS(3328),
    [anon_sym_str] = ACTIONS(3328),
    [anon_sym_char] = ACTIONS(3328),
    [anon_sym_DASH] = ACTIONS(3326),
    [anon_sym_AMP] = ACTIONS(3326),
    [anon_sym_PIPE] = ACTIONS(3326),
    [anon_sym_LT] = ACTIONS(3326),
    [anon_sym_DOT_DOT] = ACTIONS(3326),
    [anon_sym_COLON_COLON] = ACTIONS(3326),
    [anon_sym_POUND] = ACTIONS(3326),
    [anon_sym_SQUOTE] = ACTIONS(3328),
    [anon_sym_async] = ACTIONS(3328),
    [anon_sym_auto] = ACTIONS(3328),
    [anon_sym_await] = ACTIONS(3328),
    [anon_sym_become] = ACTIONS(3328),
    [anon_sym_break] = ACTIONS(3328),
    [anon_sym_const] = ACTIONS(3328),
    [anon_sym_continue] = ACTIONS(3328),
    [anon_sym_default] = ACTIONS(3328),
    [anon_sym_enum] = ACTIONS(3328),
    [anon_sym_fn] = ACTIONS(3328),
    [anon_sym_for] = ACTIONS(3328),
    [anon_sym_gen] = ACTIONS(3328),
    [anon_sym_if] = ACTIONS(3328),
    [anon_sym_impl] = ACTIONS(3328),
    [anon_sym_let] = ACTIONS(3328),
    [anon_sym_loop] = ACTIONS(3328),
    [anon_sym_match] = ACTIONS(3328),
    [anon_sym_mod] = ACTIONS(3328),
    [anon_sym_pub] = ACTIONS(3328),
    [anon_sym_return] = ACTIONS(3328),
    [anon_sym_raw] = ACTIONS(3328),
    [anon_sym_static] = ACTIONS(3328),
    [anon_sym_struct] = ACTIONS(3328),
    [anon_sym_trait] = ACTIONS(3328),
    [anon_sym_type] = ACTIONS(3328),
    [anon_sym_union] = ACTIONS(3328),
    [anon_sym_unsafe] = ACTIONS(3328),
    [anon_sym_use] = ACTIONS(3328),
    [anon_sym_while] = ACTIONS(3328),
    [anon_sym_extern] = ACTIONS(3328),
    [anon_sym_safe] = ACTIONS(3328),
    [anon_sym_yield] = ACTIONS(3328),
    [anon_sym_move] = ACTIONS(3328),
    [anon_sym_try] = ACTIONS(3328),
    [sym_integer_literal] = ACTIONS(3326),
    [aux_sym_string_literal_token1] = ACTIONS(3326),
    [sym_char_literal] = ACTIONS(3326),
    [anon_sym_true] = ACTIONS(3328),
    [anon_sym_false] = ACTIONS(3328),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3328),
    [sym_super] = ACTIONS(3328),
    [sym_crate] = ACTIONS(3328),
    [aux_sym_metavariable_token1] = ACTIONS(3326),
    [sym__raw_string_literal_start] = ACTIONS(3326),
    [sym_float_literal] = ACTIONS(3326),
    [sym__outer_block_doc_comment_start] = ACTIONS(3326),
    [sym__outer_line_doc_comment_start] = ACTIONS(3326),
  },
  [STATE(890)] = {
    [sym_line_comment] = STATE(890),
    [sym_block_comment] = STATE(890),
    [ts_builtin_sym_end] = ACTIONS(3330),
    [sym_identifier] = ACTIONS(3332),
    [anon_sym_SEMI] = ACTIONS(3330),
    [anon_sym_macro] = ACTIONS(3332),
    [anon_sym_LPAREN] = ACTIONS(3330),
    [anon_sym_LBRACE] = ACTIONS(3330),
    [anon_sym_RBRACE] = ACTIONS(3330),
    [anon_sym_macro_rules] = ACTIONS(3332),
    [anon_sym_BANG] = ACTIONS(3330),
    [anon_sym_LBRACK] = ACTIONS(3330),
    [anon_sym_STAR] = ACTIONS(3330),
    [anon_sym_u8] = ACTIONS(3332),
    [anon_sym_i8] = ACTIONS(3332),
    [anon_sym_u16] = ACTIONS(3332),
    [anon_sym_i16] = ACTIONS(3332),
    [anon_sym_u32] = ACTIONS(3332),
    [anon_sym_i32] = ACTIONS(3332),
    [anon_sym_u64] = ACTIONS(3332),
    [anon_sym_i64] = ACTIONS(3332),
    [anon_sym_u128] = ACTIONS(3332),
    [anon_sym_i128] = ACTIONS(3332),
    [anon_sym_isize] = ACTIONS(3332),
    [anon_sym_usize] = ACTIONS(3332),
    [anon_sym_f32] = ACTIONS(3332),
    [anon_sym_f64] = ACTIONS(3332),
    [anon_sym_bool] = ACTIONS(3332),
    [anon_sym_str] = ACTIONS(3332),
    [anon_sym_char] = ACTIONS(3332),
    [anon_sym_DASH] = ACTIONS(3330),
    [anon_sym_AMP] = ACTIONS(3330),
    [anon_sym_PIPE] = ACTIONS(3330),
    [anon_sym_LT] = ACTIONS(3330),
    [anon_sym_DOT_DOT] = ACTIONS(3330),
    [anon_sym_COLON_COLON] = ACTIONS(3330),
    [anon_sym_POUND] = ACTIONS(3330),
    [anon_sym_SQUOTE] = ACTIONS(3332),
    [anon_sym_async] = ACTIONS(3332),
    [anon_sym_auto] = ACTIONS(3332),
    [anon_sym_await] = ACTIONS(3332),
    [anon_sym_become] = ACTIONS(3332),
    [anon_sym_break] = ACTIONS(3332),
    [anon_sym_const] = ACTIONS(3332),
    [anon_sym_continue] = ACTIONS(3332),
    [anon_sym_default] = ACTIONS(3332),
    [anon_sym_enum] = ACTIONS(3332),
    [anon_sym_fn] = ACTIONS(3332),
    [anon_sym_for] = ACTIONS(3332),
    [anon_sym_gen] = ACTIONS(3332),
    [anon_sym_if] = ACTIONS(3332),
    [anon_sym_impl] = ACTIONS(3332),
    [anon_sym_let] = ACTIONS(3332),
    [anon_sym_loop] = ACTIONS(3332),
    [anon_sym_match] = ACTIONS(3332),
    [anon_sym_mod] = ACTIONS(3332),
    [anon_sym_pub] = ACTIONS(3332),
    [anon_sym_return] = ACTIONS(3332),
    [anon_sym_raw] = ACTIONS(3332),
    [anon_sym_static] = ACTIONS(3332),
    [anon_sym_struct] = ACTIONS(3332),
    [anon_sym_trait] = ACTIONS(3332),
    [anon_sym_type] = ACTIONS(3332),
    [anon_sym_union] = ACTIONS(3332),
    [anon_sym_unsafe] = ACTIONS(3332),
    [anon_sym_use] = ACTIONS(3332),
    [anon_sym_while] = ACTIONS(3332),
    [anon_sym_extern] = ACTIONS(3332),
    [anon_sym_safe] = ACTIONS(3332),
    [anon_sym_yield] = ACTIONS(3332),
    [anon_sym_move] = ACTIONS(3332),
    [anon_sym_try] = ACTIONS(3332),
    [sym_integer_literal] = ACTIONS(3330),
    [aux_sym_string_literal_token1] = ACTIONS(3330),
    [sym_char_literal] = ACTIONS(3330),
    [anon_sym_true] = ACTIONS(3332),
    [anon_sym_false] = ACTIONS(3332),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3332),
    [sym_super] = ACTIONS(3332),
    [sym_crate] = ACTIONS(3332),
    [aux_sym_metavariable_token1] = ACTIONS(3330),
    [sym__raw_string_literal_start] = ACTIONS(3330),
    [sym_float_literal] = ACTIONS(3330),
    [sym__outer_block_doc_comment_start] = ACTIONS(3330),
    [sym__outer_line_doc_comment_start] = ACTIONS(3330),
  },
  [STATE(891)] = {
    [sym_line_comment] = STATE(891),
    [sym_block_comment] = STATE(891),
    [ts_builtin_sym_end] = ACTIONS(3334),
    [sym_identifier] = ACTIONS(3336),
    [anon_sym_SEMI] = ACTIONS(3334),
    [anon_sym_macro] = ACTIONS(3336),
    [anon_sym_LPAREN] = ACTIONS(3334),
    [anon_sym_LBRACE] = ACTIONS(3334),
    [anon_sym_RBRACE] = ACTIONS(3334),
    [anon_sym_macro_rules] = ACTIONS(3336),
    [anon_sym_BANG] = ACTIONS(3334),
    [anon_sym_LBRACK] = ACTIONS(3334),
    [anon_sym_STAR] = ACTIONS(3334),
    [anon_sym_u8] = ACTIONS(3336),
    [anon_sym_i8] = ACTIONS(3336),
    [anon_sym_u16] = ACTIONS(3336),
    [anon_sym_i16] = ACTIONS(3336),
    [anon_sym_u32] = ACTIONS(3336),
    [anon_sym_i32] = ACTIONS(3336),
    [anon_sym_u64] = ACTIONS(3336),
    [anon_sym_i64] = ACTIONS(3336),
    [anon_sym_u128] = ACTIONS(3336),
    [anon_sym_i128] = ACTIONS(3336),
    [anon_sym_isize] = ACTIONS(3336),
    [anon_sym_usize] = ACTIONS(3336),
    [anon_sym_f32] = ACTIONS(3336),
    [anon_sym_f64] = ACTIONS(3336),
    [anon_sym_bool] = ACTIONS(3336),
    [anon_sym_str] = ACTIONS(3336),
    [anon_sym_char] = ACTIONS(3336),
    [anon_sym_DASH] = ACTIONS(3334),
    [anon_sym_AMP] = ACTIONS(3334),
    [anon_sym_PIPE] = ACTIONS(3334),
    [anon_sym_LT] = ACTIONS(3334),
    [anon_sym_DOT_DOT] = ACTIONS(3334),
    [anon_sym_COLON_COLON] = ACTIONS(3334),
    [anon_sym_POUND] = ACTIONS(3334),
    [anon_sym_SQUOTE] = ACTIONS(3336),
    [anon_sym_async] = ACTIONS(3336),
    [anon_sym_auto] = ACTIONS(3336),
    [anon_sym_await] = ACTIONS(3336),
    [anon_sym_become] = ACTIONS(3336),
    [anon_sym_break] = ACTIONS(3336),
    [anon_sym_const] = ACTIONS(3336),
    [anon_sym_continue] = ACTIONS(3336),
    [anon_sym_default] = ACTIONS(3336),
    [anon_sym_enum] = ACTIONS(3336),
    [anon_sym_fn] = ACTIONS(3336),
    [anon_sym_for] = ACTIONS(3336),
    [anon_sym_gen] = ACTIONS(3336),
    [anon_sym_if] = ACTIONS(3336),
    [anon_sym_impl] = ACTIONS(3336),
    [anon_sym_let] = ACTIONS(3336),
    [anon_sym_loop] = ACTIONS(3336),
    [anon_sym_match] = ACTIONS(3336),
    [anon_sym_mod] = ACTIONS(3336),
    [anon_sym_pub] = ACTIONS(3336),
    [anon_sym_return] = ACTIONS(3336),
    [anon_sym_raw] = ACTIONS(3336),
    [anon_sym_static] = ACTIONS(3336),
    [anon_sym_struct] = ACTIONS(3336),
    [anon_sym_trait] = ACTIONS(3336),
    [anon_sym_type] = ACTIONS(3336),
    [anon_sym_union] = ACTIONS(3336),
    [anon_sym_unsafe] = ACTIONS(3336),
    [anon_sym_use] = ACTIONS(3336),
    [anon_sym_while] = ACTIONS(3336),
    [anon_sym_extern] = ACTIONS(3336),
    [anon_sym_safe] = ACTIONS(3336),
    [anon_sym_yield] = ACTIONS(3336),
    [anon_sym_move] = ACTIONS(3336),
    [anon_sym_try] = ACTIONS(3336),
    [sym_integer_literal] = ACTIONS(3334),
    [aux_sym_string_literal_token1] = ACTIONS(3334),
    [sym_char_literal] = ACTIONS(3334),
    [anon_sym_true] = ACTIONS(3336),
    [anon_sym_false] = ACTIONS(3336),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3336),
    [sym_super] = ACTIONS(3336),
    [sym_crate] = ACTIONS(3336),
    [aux_sym_metavariable_token1] = ACTIONS(3334),
    [sym__raw_string_literal_start] = ACTIONS(3334),
    [sym_float_literal] = ACTIONS(3334),
    [sym__outer_block_doc_comment_start] = ACTIONS(3334),
    [sym__outer_line_doc_comment_start] = ACTIONS(3334),
  },
  [STATE(892)] = {
    [sym_line_comment] = STATE(892),
    [sym_block_comment] = STATE(892),
    [ts_builtin_sym_end] = ACTIONS(3338),
    [sym_identifier] = ACTIONS(3340),
    [anon_sym_SEMI] = ACTIONS(3338),
    [anon_sym_macro] = ACTIONS(3340),
    [anon_sym_LPAREN] = ACTIONS(3338),
    [anon_sym_LBRACE] = ACTIONS(3338),
    [anon_sym_RBRACE] = ACTIONS(3338),
    [anon_sym_macro_rules] = ACTIONS(3340),
    [anon_sym_BANG] = ACTIONS(3338),
    [anon_sym_LBRACK] = ACTIONS(3338),
    [anon_sym_STAR] = ACTIONS(3338),
    [anon_sym_u8] = ACTIONS(3340),
    [anon_sym_i8] = ACTIONS(3340),
    [anon_sym_u16] = ACTIONS(3340),
    [anon_sym_i16] = ACTIONS(3340),
    [anon_sym_u32] = ACTIONS(3340),
    [anon_sym_i32] = ACTIONS(3340),
    [anon_sym_u64] = ACTIONS(3340),
    [anon_sym_i64] = ACTIONS(3340),
    [anon_sym_u128] = ACTIONS(3340),
    [anon_sym_i128] = ACTIONS(3340),
    [anon_sym_isize] = ACTIONS(3340),
    [anon_sym_usize] = ACTIONS(3340),
    [anon_sym_f32] = ACTIONS(3340),
    [anon_sym_f64] = ACTIONS(3340),
    [anon_sym_bool] = ACTIONS(3340),
    [anon_sym_str] = ACTIONS(3340),
    [anon_sym_char] = ACTIONS(3340),
    [anon_sym_DASH] = ACTIONS(3338),
    [anon_sym_AMP] = ACTIONS(3338),
    [anon_sym_PIPE] = ACTIONS(3338),
    [anon_sym_LT] = ACTIONS(3338),
    [anon_sym_DOT_DOT] = ACTIONS(3338),
    [anon_sym_COLON_COLON] = ACTIONS(3338),
    [anon_sym_POUND] = ACTIONS(3338),
    [anon_sym_SQUOTE] = ACTIONS(3340),
    [anon_sym_async] = ACTIONS(3340),
    [anon_sym_auto] = ACTIONS(3340),
    [anon_sym_await] = ACTIONS(3340),
    [anon_sym_become] = ACTIONS(3340),
    [anon_sym_break] = ACTIONS(3340),
    [anon_sym_const] = ACTIONS(3340),
    [anon_sym_continue] = ACTIONS(3340),
    [anon_sym_default] = ACTIONS(3340),
    [anon_sym_enum] = ACTIONS(3340),
    [anon_sym_fn] = ACTIONS(3340),
    [anon_sym_for] = ACTIONS(3340),
    [anon_sym_gen] = ACTIONS(3340),
    [anon_sym_if] = ACTIONS(3340),
    [anon_sym_impl] = ACTIONS(3340),
    [anon_sym_let] = ACTIONS(3340),
    [anon_sym_loop] = ACTIONS(3340),
    [anon_sym_match] = ACTIONS(3340),
    [anon_sym_mod] = ACTIONS(3340),
    [anon_sym_pub] = ACTIONS(3340),
    [anon_sym_return] = ACTIONS(3340),
    [anon_sym_raw] = ACTIONS(3340),
    [anon_sym_static] = ACTIONS(3340),
    [anon_sym_struct] = ACTIONS(3340),
    [anon_sym_trait] = ACTIONS(3340),
    [anon_sym_type] = ACTIONS(3340),
    [anon_sym_union] = ACTIONS(3340),
    [anon_sym_unsafe] = ACTIONS(3340),
    [anon_sym_use] = ACTIONS(3340),
    [anon_sym_while] = ACTIONS(3340),
    [anon_sym_extern] = ACTIONS(3340),
    [anon_sym_safe] = ACTIONS(3340),
    [anon_sym_yield] = ACTIONS(3340),
    [anon_sym_move] = ACTIONS(3340),
    [anon_sym_try] = ACTIONS(3340),
    [sym_integer_literal] = ACTIONS(3338),
    [aux_sym_string_literal_token1] = ACTIONS(3338),
    [sym_char_literal] = ACTIONS(3338),
    [anon_sym_true] = ACTIONS(3340),
    [anon_sym_false] = ACTIONS(3340),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3340),
    [sym_super] = ACTIONS(3340),
    [sym_crate] = ACTIONS(3340),
    [aux_sym_metavariable_token1] = ACTIONS(3338),
    [sym__raw_string_literal_start] = ACTIONS(3338),
    [sym_float_literal] = ACTIONS(3338),
    [sym__outer_block_doc_comment_start] = ACTIONS(3338),
    [sym__outer_line_doc_comment_start] = ACTIONS(3338),
  },
  [STATE(893)] = {
    [sym_line_comment] = STATE(893),
    [sym_block_comment] = STATE(893),
    [ts_builtin_sym_end] = ACTIONS(3342),
    [sym_identifier] = ACTIONS(3344),
    [anon_sym_SEMI] = ACTIONS(3342),
    [anon_sym_macro] = ACTIONS(3344),
    [anon_sym_LPAREN] = ACTIONS(3342),
    [anon_sym_LBRACE] = ACTIONS(3342),
    [anon_sym_RBRACE] = ACTIONS(3342),
    [anon_sym_macro_rules] = ACTIONS(3344),
    [anon_sym_BANG] = ACTIONS(3342),
    [anon_sym_LBRACK] = ACTIONS(3342),
    [anon_sym_STAR] = ACTIONS(3342),
    [anon_sym_u8] = ACTIONS(3344),
    [anon_sym_i8] = ACTIONS(3344),
    [anon_sym_u16] = ACTIONS(3344),
    [anon_sym_i16] = ACTIONS(3344),
    [anon_sym_u32] = ACTIONS(3344),
    [anon_sym_i32] = ACTIONS(3344),
    [anon_sym_u64] = ACTIONS(3344),
    [anon_sym_i64] = ACTIONS(3344),
    [anon_sym_u128] = ACTIONS(3344),
    [anon_sym_i128] = ACTIONS(3344),
    [anon_sym_isize] = ACTIONS(3344),
    [anon_sym_usize] = ACTIONS(3344),
    [anon_sym_f32] = ACTIONS(3344),
    [anon_sym_f64] = ACTIONS(3344),
    [anon_sym_bool] = ACTIONS(3344),
    [anon_sym_str] = ACTIONS(3344),
    [anon_sym_char] = ACTIONS(3344),
    [anon_sym_DASH] = ACTIONS(3342),
    [anon_sym_AMP] = ACTIONS(3342),
    [anon_sym_PIPE] = ACTIONS(3342),
    [anon_sym_LT] = ACTIONS(3342),
    [anon_sym_DOT_DOT] = ACTIONS(3342),
    [anon_sym_COLON_COLON] = ACTIONS(3342),
    [anon_sym_POUND] = ACTIONS(3342),
    [anon_sym_SQUOTE] = ACTIONS(3344),
    [anon_sym_async] = ACTIONS(3344),
    [anon_sym_auto] = ACTIONS(3344),
    [anon_sym_await] = ACTIONS(3344),
    [anon_sym_become] = ACTIONS(3344),
    [anon_sym_break] = ACTIONS(3344),
    [anon_sym_const] = ACTIONS(3344),
    [anon_sym_continue] = ACTIONS(3344),
    [anon_sym_default] = ACTIONS(3344),
    [anon_sym_enum] = ACTIONS(3344),
    [anon_sym_fn] = ACTIONS(3344),
    [anon_sym_for] = ACTIONS(3344),
    [anon_sym_gen] = ACTIONS(3344),
    [anon_sym_if] = ACTIONS(3344),
    [anon_sym_impl] = ACTIONS(3344),
    [anon_sym_let] = ACTIONS(3344),
    [anon_sym_loop] = ACTIONS(3344),
    [anon_sym_match] = ACTIONS(3344),
    [anon_sym_mod] = ACTIONS(3344),
    [anon_sym_pub] = ACTIONS(3344),
    [anon_sym_return] = ACTIONS(3344),
    [anon_sym_raw] = ACTIONS(3344),
    [anon_sym_static] = ACTIONS(3344),
    [anon_sym_struct] = ACTIONS(3344),
    [anon_sym_trait] = ACTIONS(3344),
    [anon_sym_type] = ACTIONS(3344),
    [anon_sym_union] = ACTIONS(3344),
    [anon_sym_unsafe] = ACTIONS(3344),
    [anon_sym_use] = ACTIONS(3344),
    [anon_sym_while] = ACTIONS(3344),
    [anon_sym_extern] = ACTIONS(3344),
    [anon_sym_safe] = ACTIONS(3344),
    [anon_sym_yield] = ACTIONS(3344),
    [anon_sym_move] = ACTIONS(3344),
    [anon_sym_try] = ACTIONS(3344),
    [sym_integer_literal] = ACTIONS(3342),
    [aux_sym_string_literal_token1] = ACTIONS(3342),
    [sym_char_literal] = ACTIONS(3342),
    [anon_sym_true] = ACTIONS(3344),
    [anon_sym_false] = ACTIONS(3344),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3344),
    [sym_super] = ACTIONS(3344),
    [sym_crate] = ACTIONS(3344),
    [aux_sym_metavariable_token1] = ACTIONS(3342),
    [sym__raw_string_literal_start] = ACTIONS(3342),
    [sym_float_literal] = ACTIONS(3342),
    [sym__outer_block_doc_comment_start] = ACTIONS(3342),
    [sym__outer_line_doc_comment_start] = ACTIONS(3342),
  },
  [STATE(894)] = {
    [sym_line_comment] = STATE(894),
    [sym_block_comment] = STATE(894),
    [ts_builtin_sym_end] = ACTIONS(3346),
    [sym_identifier] = ACTIONS(3348),
    [anon_sym_SEMI] = ACTIONS(3346),
    [anon_sym_macro] = ACTIONS(3348),
    [anon_sym_LPAREN] = ACTIONS(3346),
    [anon_sym_LBRACE] = ACTIONS(3346),
    [anon_sym_RBRACE] = ACTIONS(3346),
    [anon_sym_macro_rules] = ACTIONS(3348),
    [anon_sym_BANG] = ACTIONS(3346),
    [anon_sym_LBRACK] = ACTIONS(3346),
    [anon_sym_STAR] = ACTIONS(3346),
    [anon_sym_u8] = ACTIONS(3348),
    [anon_sym_i8] = ACTIONS(3348),
    [anon_sym_u16] = ACTIONS(3348),
    [anon_sym_i16] = ACTIONS(3348),
    [anon_sym_u32] = ACTIONS(3348),
    [anon_sym_i32] = ACTIONS(3348),
    [anon_sym_u64] = ACTIONS(3348),
    [anon_sym_i64] = ACTIONS(3348),
    [anon_sym_u128] = ACTIONS(3348),
    [anon_sym_i128] = ACTIONS(3348),
    [anon_sym_isize] = ACTIONS(3348),
    [anon_sym_usize] = ACTIONS(3348),
    [anon_sym_f32] = ACTIONS(3348),
    [anon_sym_f64] = ACTIONS(3348),
    [anon_sym_bool] = ACTIONS(3348),
    [anon_sym_str] = ACTIONS(3348),
    [anon_sym_char] = ACTIONS(3348),
    [anon_sym_DASH] = ACTIONS(3346),
    [anon_sym_AMP] = ACTIONS(3346),
    [anon_sym_PIPE] = ACTIONS(3346),
    [anon_sym_LT] = ACTIONS(3346),
    [anon_sym_DOT_DOT] = ACTIONS(3346),
    [anon_sym_COLON_COLON] = ACTIONS(3346),
    [anon_sym_POUND] = ACTIONS(3346),
    [anon_sym_SQUOTE] = ACTIONS(3348),
    [anon_sym_async] = ACTIONS(3348),
    [anon_sym_auto] = ACTIONS(3348),
    [anon_sym_await] = ACTIONS(3348),
    [anon_sym_become] = ACTIONS(3348),
    [anon_sym_break] = ACTIONS(3348),
    [anon_sym_const] = ACTIONS(3348),
    [anon_sym_continue] = ACTIONS(3348),
    [anon_sym_default] = ACTIONS(3348),
    [anon_sym_enum] = ACTIONS(3348),
    [anon_sym_fn] = ACTIONS(3348),
    [anon_sym_for] = ACTIONS(3348),
    [anon_sym_gen] = ACTIONS(3348),
    [anon_sym_if] = ACTIONS(3348),
    [anon_sym_impl] = ACTIONS(3348),
    [anon_sym_let] = ACTIONS(3348),
    [anon_sym_loop] = ACTIONS(3348),
    [anon_sym_match] = ACTIONS(3348),
    [anon_sym_mod] = ACTIONS(3348),
    [anon_sym_pub] = ACTIONS(3348),
    [anon_sym_return] = ACTIONS(3348),
    [anon_sym_raw] = ACTIONS(3348),
    [anon_sym_static] = ACTIONS(3348),
    [anon_sym_struct] = ACTIONS(3348),
    [anon_sym_trait] = ACTIONS(3348),
    [anon_sym_type] = ACTIONS(3348),
    [anon_sym_union] = ACTIONS(3348),
    [anon_sym_unsafe] = ACTIONS(3348),
    [anon_sym_use] = ACTIONS(3348),
    [anon_sym_while] = ACTIONS(3348),
    [anon_sym_extern] = ACTIONS(3348),
    [anon_sym_safe] = ACTIONS(3348),
    [anon_sym_yield] = ACTIONS(3348),
    [anon_sym_move] = ACTIONS(3348),
    [anon_sym_try] = ACTIONS(3348),
    [sym_integer_literal] = ACTIONS(3346),
    [aux_sym_string_literal_token1] = ACTIONS(3346),
    [sym_char_literal] = ACTIONS(3346),
    [anon_sym_true] = ACTIONS(3348),
    [anon_sym_false] = ACTIONS(3348),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3348),
    [sym_super] = ACTIONS(3348),
    [sym_crate] = ACTIONS(3348),
    [aux_sym_metavariable_token1] = ACTIONS(3346),
    [sym__raw_string_literal_start] = ACTIONS(3346),
    [sym_float_literal] = ACTIONS(3346),
    [sym__outer_block_doc_comment_start] = ACTIONS(3346),
    [sym__outer_line_doc_comment_start] = ACTIONS(3346),
  },
  [STATE(895)] = {
    [sym_line_comment] = STATE(895),
    [sym_block_comment] = STATE(895),
    [ts_builtin_sym_end] = ACTIONS(3350),
    [sym_identifier] = ACTIONS(3352),
    [anon_sym_SEMI] = ACTIONS(3350),
    [anon_sym_macro] = ACTIONS(3352),
    [anon_sym_LPAREN] = ACTIONS(3350),
    [anon_sym_LBRACE] = ACTIONS(3350),
    [anon_sym_RBRACE] = ACTIONS(3350),
    [anon_sym_macro_rules] = ACTIONS(3352),
    [anon_sym_BANG] = ACTIONS(3350),
    [anon_sym_LBRACK] = ACTIONS(3350),
    [anon_sym_STAR] = ACTIONS(3350),
    [anon_sym_u8] = ACTIONS(3352),
    [anon_sym_i8] = ACTIONS(3352),
    [anon_sym_u16] = ACTIONS(3352),
    [anon_sym_i16] = ACTIONS(3352),
    [anon_sym_u32] = ACTIONS(3352),
    [anon_sym_i32] = ACTIONS(3352),
    [anon_sym_u64] = ACTIONS(3352),
    [anon_sym_i64] = ACTIONS(3352),
    [anon_sym_u128] = ACTIONS(3352),
    [anon_sym_i128] = ACTIONS(3352),
    [anon_sym_isize] = ACTIONS(3352),
    [anon_sym_usize] = ACTIONS(3352),
    [anon_sym_f32] = ACTIONS(3352),
    [anon_sym_f64] = ACTIONS(3352),
    [anon_sym_bool] = ACTIONS(3352),
    [anon_sym_str] = ACTIONS(3352),
    [anon_sym_char] = ACTIONS(3352),
    [anon_sym_DASH] = ACTIONS(3350),
    [anon_sym_AMP] = ACTIONS(3350),
    [anon_sym_PIPE] = ACTIONS(3350),
    [anon_sym_LT] = ACTIONS(3350),
    [anon_sym_DOT_DOT] = ACTIONS(3350),
    [anon_sym_COLON_COLON] = ACTIONS(3350),
    [anon_sym_POUND] = ACTIONS(3350),
    [anon_sym_SQUOTE] = ACTIONS(3352),
    [anon_sym_async] = ACTIONS(3352),
    [anon_sym_auto] = ACTIONS(3352),
    [anon_sym_await] = ACTIONS(3352),
    [anon_sym_become] = ACTIONS(3352),
    [anon_sym_break] = ACTIONS(3352),
    [anon_sym_const] = ACTIONS(3352),
    [anon_sym_continue] = ACTIONS(3352),
    [anon_sym_default] = ACTIONS(3352),
    [anon_sym_enum] = ACTIONS(3352),
    [anon_sym_fn] = ACTIONS(3352),
    [anon_sym_for] = ACTIONS(3352),
    [anon_sym_gen] = ACTIONS(3352),
    [anon_sym_if] = ACTIONS(3352),
    [anon_sym_impl] = ACTIONS(3352),
    [anon_sym_let] = ACTIONS(3352),
    [anon_sym_loop] = ACTIONS(3352),
    [anon_sym_match] = ACTIONS(3352),
    [anon_sym_mod] = ACTIONS(3352),
    [anon_sym_pub] = ACTIONS(3352),
    [anon_sym_return] = ACTIONS(3352),
    [anon_sym_raw] = ACTIONS(3352),
    [anon_sym_static] = ACTIONS(3352),
    [anon_sym_struct] = ACTIONS(3352),
    [anon_sym_trait] = ACTIONS(3352),
    [anon_sym_type] = ACTIONS(3352),
    [anon_sym_union] = ACTIONS(3352),
    [anon_sym_unsafe] = ACTIONS(3352),
    [anon_sym_use] = ACTIONS(3352),
    [anon_sym_while] = ACTIONS(3352),
    [anon_sym_extern] = ACTIONS(3352),
    [anon_sym_safe] = ACTIONS(3352),
    [anon_sym_yield] = ACTIONS(3352),
    [anon_sym_move] = ACTIONS(3352),
    [anon_sym_try] = ACTIONS(3352),
    [sym_integer_literal] = ACTIONS(3350),
    [aux_sym_string_literal_token1] = ACTIONS(3350),
    [sym_char_literal] = ACTIONS(3350),
    [anon_sym_true] = ACTIONS(3352),
    [anon_sym_false] = ACTIONS(3352),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3352),
    [sym_super] = ACTIONS(3352),
    [sym_crate] = ACTIONS(3352),
    [aux_sym_metavariable_token1] = ACTIONS(3350),
    [sym__raw_string_literal_start] = ACTIONS(3350),
    [sym_float_literal] = ACTIONS(3350),
    [sym__outer_block_doc_comment_start] = ACTIONS(3350),
    [sym__outer_line_doc_comment_start] = ACTIONS(3350),
  },
  [STATE(896)] = {
    [sym_line_comment] = STATE(896),
    [sym_block_comment] = STATE(896),
    [ts_builtin_sym_end] = ACTIONS(3354),
    [sym_identifier] = ACTIONS(3356),
    [anon_sym_SEMI] = ACTIONS(3354),
    [anon_sym_macro] = ACTIONS(3356),
    [anon_sym_LPAREN] = ACTIONS(3354),
    [anon_sym_LBRACE] = ACTIONS(3354),
    [anon_sym_RBRACE] = ACTIONS(3354),
    [anon_sym_macro_rules] = ACTIONS(3356),
    [anon_sym_BANG] = ACTIONS(3354),
    [anon_sym_LBRACK] = ACTIONS(3354),
    [anon_sym_STAR] = ACTIONS(3354),
    [anon_sym_u8] = ACTIONS(3356),
    [anon_sym_i8] = ACTIONS(3356),
    [anon_sym_u16] = ACTIONS(3356),
    [anon_sym_i16] = ACTIONS(3356),
    [anon_sym_u32] = ACTIONS(3356),
    [anon_sym_i32] = ACTIONS(3356),
    [anon_sym_u64] = ACTIONS(3356),
    [anon_sym_i64] = ACTIONS(3356),
    [anon_sym_u128] = ACTIONS(3356),
    [anon_sym_i128] = ACTIONS(3356),
    [anon_sym_isize] = ACTIONS(3356),
    [anon_sym_usize] = ACTIONS(3356),
    [anon_sym_f32] = ACTIONS(3356),
    [anon_sym_f64] = ACTIONS(3356),
    [anon_sym_bool] = ACTIONS(3356),
    [anon_sym_str] = ACTIONS(3356),
    [anon_sym_char] = ACTIONS(3356),
    [anon_sym_DASH] = ACTIONS(3354),
    [anon_sym_AMP] = ACTIONS(3354),
    [anon_sym_PIPE] = ACTIONS(3354),
    [anon_sym_LT] = ACTIONS(3354),
    [anon_sym_DOT_DOT] = ACTIONS(3354),
    [anon_sym_COLON_COLON] = ACTIONS(3354),
    [anon_sym_POUND] = ACTIONS(3354),
    [anon_sym_SQUOTE] = ACTIONS(3356),
    [anon_sym_async] = ACTIONS(3356),
    [anon_sym_auto] = ACTIONS(3356),
    [anon_sym_await] = ACTIONS(3356),
    [anon_sym_become] = ACTIONS(3356),
    [anon_sym_break] = ACTIONS(3356),
    [anon_sym_const] = ACTIONS(3356),
    [anon_sym_continue] = ACTIONS(3356),
    [anon_sym_default] = ACTIONS(3356),
    [anon_sym_enum] = ACTIONS(3356),
    [anon_sym_fn] = ACTIONS(3356),
    [anon_sym_for] = ACTIONS(3356),
    [anon_sym_gen] = ACTIONS(3356),
    [anon_sym_if] = ACTIONS(3356),
    [anon_sym_impl] = ACTIONS(3356),
    [anon_sym_let] = ACTIONS(3356),
    [anon_sym_loop] = ACTIONS(3356),
    [anon_sym_match] = ACTIONS(3356),
    [anon_sym_mod] = ACTIONS(3356),
    [anon_sym_pub] = ACTIONS(3356),
    [anon_sym_return] = ACTIONS(3356),
    [anon_sym_raw] = ACTIONS(3356),
    [anon_sym_static] = ACTIONS(3356),
    [anon_sym_struct] = ACTIONS(3356),
    [anon_sym_trait] = ACTIONS(3356),
    [anon_sym_type] = ACTIONS(3356),
    [anon_sym_union] = ACTIONS(3356),
    [anon_sym_unsafe] = ACTIONS(3356),
    [anon_sym_use] = ACTIONS(3356),
    [anon_sym_while] = ACTIONS(3356),
    [anon_sym_extern] = ACTIONS(3356),
    [anon_sym_safe] = ACTIONS(3356),
    [anon_sym_yield] = ACTIONS(3356),
    [anon_sym_move] = ACTIONS(3356),
    [anon_sym_try] = ACTIONS(3356),
    [sym_integer_literal] = ACTIONS(3354),
    [aux_sym_string_literal_token1] = ACTIONS(3354),
    [sym_char_literal] = ACTIONS(3354),
    [anon_sym_true] = ACTIONS(3356),
    [anon_sym_false] = ACTIONS(3356),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3356),
    [sym_super] = ACTIONS(3356),
    [sym_crate] = ACTIONS(3356),
    [aux_sym_metavariable_token1] = ACTIONS(3354),
    [sym__raw_string_literal_start] = ACTIONS(3354),
    [sym_float_literal] = ACTIONS(3354),
    [sym__outer_block_doc_comment_start] = ACTIONS(3354),
    [sym__outer_line_doc_comment_start] = ACTIONS(3354),
  },
  [STATE(897)] = {
    [sym_line_comment] = STATE(897),
    [sym_block_comment] = STATE(897),
    [ts_builtin_sym_end] = ACTIONS(3358),
    [sym_identifier] = ACTIONS(3360),
    [anon_sym_SEMI] = ACTIONS(3358),
    [anon_sym_macro] = ACTIONS(3360),
    [anon_sym_LPAREN] = ACTIONS(3358),
    [anon_sym_LBRACE] = ACTIONS(3358),
    [anon_sym_RBRACE] = ACTIONS(3358),
    [anon_sym_macro_rules] = ACTIONS(3360),
    [anon_sym_BANG] = ACTIONS(3358),
    [anon_sym_LBRACK] = ACTIONS(3358),
    [anon_sym_STAR] = ACTIONS(3358),
    [anon_sym_u8] = ACTIONS(3360),
    [anon_sym_i8] = ACTIONS(3360),
    [anon_sym_u16] = ACTIONS(3360),
    [anon_sym_i16] = ACTIONS(3360),
    [anon_sym_u32] = ACTIONS(3360),
    [anon_sym_i32] = ACTIONS(3360),
    [anon_sym_u64] = ACTIONS(3360),
    [anon_sym_i64] = ACTIONS(3360),
    [anon_sym_u128] = ACTIONS(3360),
    [anon_sym_i128] = ACTIONS(3360),
    [anon_sym_isize] = ACTIONS(3360),
    [anon_sym_usize] = ACTIONS(3360),
    [anon_sym_f32] = ACTIONS(3360),
    [anon_sym_f64] = ACTIONS(3360),
    [anon_sym_bool] = ACTIONS(3360),
    [anon_sym_str] = ACTIONS(3360),
    [anon_sym_char] = ACTIONS(3360),
    [anon_sym_DASH] = ACTIONS(3358),
    [anon_sym_AMP] = ACTIONS(3358),
    [anon_sym_PIPE] = ACTIONS(3358),
    [anon_sym_LT] = ACTIONS(3358),
    [anon_sym_DOT_DOT] = ACTIONS(3358),
    [anon_sym_COLON_COLON] = ACTIONS(3358),
    [anon_sym_POUND] = ACTIONS(3358),
    [anon_sym_SQUOTE] = ACTIONS(3360),
    [anon_sym_async] = ACTIONS(3360),
    [anon_sym_auto] = ACTIONS(3360),
    [anon_sym_await] = ACTIONS(3360),
    [anon_sym_become] = ACTIONS(3360),
    [anon_sym_break] = ACTIONS(3360),
    [anon_sym_const] = ACTIONS(3360),
    [anon_sym_continue] = ACTIONS(3360),
    [anon_sym_default] = ACTIONS(3360),
    [anon_sym_enum] = ACTIONS(3360),
    [anon_sym_fn] = ACTIONS(3360),
    [anon_sym_for] = ACTIONS(3360),
    [anon_sym_gen] = ACTIONS(3360),
    [anon_sym_if] = ACTIONS(3360),
    [anon_sym_impl] = ACTIONS(3360),
    [anon_sym_let] = ACTIONS(3360),
    [anon_sym_loop] = ACTIONS(3360),
    [anon_sym_match] = ACTIONS(3360),
    [anon_sym_mod] = ACTIONS(3360),
    [anon_sym_pub] = ACTIONS(3360),
    [anon_sym_return] = ACTIONS(3360),
    [anon_sym_raw] = ACTIONS(3360),
    [anon_sym_static] = ACTIONS(3360),
    [anon_sym_struct] = ACTIONS(3360),
    [anon_sym_trait] = ACTIONS(3360),
    [anon_sym_type] = ACTIONS(3360),
    [anon_sym_union] = ACTIONS(3360),
    [anon_sym_unsafe] = ACTIONS(3360),
    [anon_sym_use] = ACTIONS(3360),
    [anon_sym_while] = ACTIONS(3360),
    [anon_sym_extern] = ACTIONS(3360),
    [anon_sym_safe] = ACTIONS(3360),
    [anon_sym_yield] = ACTIONS(3360),
    [anon_sym_move] = ACTIONS(3360),
    [anon_sym_try] = ACTIONS(3360),
    [sym_integer_literal] = ACTIONS(3358),
    [aux_sym_string_literal_token1] = ACTIONS(3358),
    [sym_char_literal] = ACTIONS(3358),
    [anon_sym_true] = ACTIONS(3360),
    [anon_sym_false] = ACTIONS(3360),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3360),
    [sym_super] = ACTIONS(3360),
    [sym_crate] = ACTIONS(3360),
    [aux_sym_metavariable_token1] = ACTIONS(3358),
    [sym__raw_string_literal_start] = ACTIONS(3358),
    [sym_float_literal] = ACTIONS(3358),
    [sym__outer_block_doc_comment_start] = ACTIONS(3358),
    [sym__outer_line_doc_comment_start] = ACTIONS(3358),
  },
  [STATE(898)] = {
    [sym_line_comment] = STATE(898),
    [sym_block_comment] = STATE(898),
    [ts_builtin_sym_end] = ACTIONS(3362),
    [sym_identifier] = ACTIONS(3364),
    [anon_sym_SEMI] = ACTIONS(3362),
    [anon_sym_macro] = ACTIONS(3364),
    [anon_sym_LPAREN] = ACTIONS(3362),
    [anon_sym_LBRACE] = ACTIONS(3362),
    [anon_sym_RBRACE] = ACTIONS(3362),
    [anon_sym_macro_rules] = ACTIONS(3364),
    [anon_sym_BANG] = ACTIONS(3362),
    [anon_sym_LBRACK] = ACTIONS(3362),
    [anon_sym_STAR] = ACTIONS(3362),
    [anon_sym_u8] = ACTIONS(3364),
    [anon_sym_i8] = ACTIONS(3364),
    [anon_sym_u16] = ACTIONS(3364),
    [anon_sym_i16] = ACTIONS(3364),
    [anon_sym_u32] = ACTIONS(3364),
    [anon_sym_i32] = ACTIONS(3364),
    [anon_sym_u64] = ACTIONS(3364),
    [anon_sym_i64] = ACTIONS(3364),
    [anon_sym_u128] = ACTIONS(3364),
    [anon_sym_i128] = ACTIONS(3364),
    [anon_sym_isize] = ACTIONS(3364),
    [anon_sym_usize] = ACTIONS(3364),
    [anon_sym_f32] = ACTIONS(3364),
    [anon_sym_f64] = ACTIONS(3364),
    [anon_sym_bool] = ACTIONS(3364),
    [anon_sym_str] = ACTIONS(3364),
    [anon_sym_char] = ACTIONS(3364),
    [anon_sym_DASH] = ACTIONS(3362),
    [anon_sym_AMP] = ACTIONS(3362),
    [anon_sym_PIPE] = ACTIONS(3362),
    [anon_sym_LT] = ACTIONS(3362),
    [anon_sym_DOT_DOT] = ACTIONS(3362),
    [anon_sym_COLON_COLON] = ACTIONS(3362),
    [anon_sym_POUND] = ACTIONS(3362),
    [anon_sym_SQUOTE] = ACTIONS(3364),
    [anon_sym_async] = ACTIONS(3364),
    [anon_sym_auto] = ACTIONS(3364),
    [anon_sym_await] = ACTIONS(3364),
    [anon_sym_become] = ACTIONS(3364),
    [anon_sym_break] = ACTIONS(3364),
    [anon_sym_const] = ACTIONS(3364),
    [anon_sym_continue] = ACTIONS(3364),
    [anon_sym_default] = ACTIONS(3364),
    [anon_sym_enum] = ACTIONS(3364),
    [anon_sym_fn] = ACTIONS(3364),
    [anon_sym_for] = ACTIONS(3364),
    [anon_sym_gen] = ACTIONS(3364),
    [anon_sym_if] = ACTIONS(3364),
    [anon_sym_impl] = ACTIONS(3364),
    [anon_sym_let] = ACTIONS(3364),
    [anon_sym_loop] = ACTIONS(3364),
    [anon_sym_match] = ACTIONS(3364),
    [anon_sym_mod] = ACTIONS(3364),
    [anon_sym_pub] = ACTIONS(3364),
    [anon_sym_return] = ACTIONS(3364),
    [anon_sym_raw] = ACTIONS(3364),
    [anon_sym_static] = ACTIONS(3364),
    [anon_sym_struct] = ACTIONS(3364),
    [anon_sym_trait] = ACTIONS(3364),
    [anon_sym_type] = ACTIONS(3364),
    [anon_sym_union] = ACTIONS(3364),
    [anon_sym_unsafe] = ACTIONS(3364),
    [anon_sym_use] = ACTIONS(3364),
    [anon_sym_while] = ACTIONS(3364),
    [anon_sym_extern] = ACTIONS(3364),
    [anon_sym_safe] = ACTIONS(3364),
    [anon_sym_yield] = ACTIONS(3364),
    [anon_sym_move] = ACTIONS(3364),
    [anon_sym_try] = ACTIONS(3364),
    [sym_integer_literal] = ACTIONS(3362),
    [aux_sym_string_literal_token1] = ACTIONS(3362),
    [sym_char_literal] = ACTIONS(3362),
    [anon_sym_true] = ACTIONS(3364),
    [anon_sym_false] = ACTIONS(3364),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3364),
    [sym_super] = ACTIONS(3364),
    [sym_crate] = ACTIONS(3364),
    [aux_sym_metavariable_token1] = ACTIONS(3362),
    [sym__raw_string_literal_start] = ACTIONS(3362),
    [sym_float_literal] = ACTIONS(3362),
    [sym__outer_block_doc_comment_start] = ACTIONS(3362),
    [sym__outer_line_doc_comment_start] = ACTIONS(3362),
  },
  [STATE(899)] = {
    [sym_line_comment] = STATE(899),
    [sym_block_comment] = STATE(899),
    [ts_builtin_sym_end] = ACTIONS(3366),
    [sym_identifier] = ACTIONS(3368),
    [anon_sym_SEMI] = ACTIONS(3366),
    [anon_sym_macro] = ACTIONS(3368),
    [anon_sym_LPAREN] = ACTIONS(3366),
    [anon_sym_LBRACE] = ACTIONS(3366),
    [anon_sym_RBRACE] = ACTIONS(3366),
    [anon_sym_macro_rules] = ACTIONS(3368),
    [anon_sym_BANG] = ACTIONS(3366),
    [anon_sym_LBRACK] = ACTIONS(3366),
    [anon_sym_STAR] = ACTIONS(3366),
    [anon_sym_u8] = ACTIONS(3368),
    [anon_sym_i8] = ACTIONS(3368),
    [anon_sym_u16] = ACTIONS(3368),
    [anon_sym_i16] = ACTIONS(3368),
    [anon_sym_u32] = ACTIONS(3368),
    [anon_sym_i32] = ACTIONS(3368),
    [anon_sym_u64] = ACTIONS(3368),
    [anon_sym_i64] = ACTIONS(3368),
    [anon_sym_u128] = ACTIONS(3368),
    [anon_sym_i128] = ACTIONS(3368),
    [anon_sym_isize] = ACTIONS(3368),
    [anon_sym_usize] = ACTIONS(3368),
    [anon_sym_f32] = ACTIONS(3368),
    [anon_sym_f64] = ACTIONS(3368),
    [anon_sym_bool] = ACTIONS(3368),
    [anon_sym_str] = ACTIONS(3368),
    [anon_sym_char] = ACTIONS(3368),
    [anon_sym_DASH] = ACTIONS(3366),
    [anon_sym_AMP] = ACTIONS(3366),
    [anon_sym_PIPE] = ACTIONS(3366),
    [anon_sym_LT] = ACTIONS(3366),
    [anon_sym_DOT_DOT] = ACTIONS(3366),
    [anon_sym_COLON_COLON] = ACTIONS(3366),
    [anon_sym_POUND] = ACTIONS(3366),
    [anon_sym_SQUOTE] = ACTIONS(3368),
    [anon_sym_async] = ACTIONS(3368),
    [anon_sym_auto] = ACTIONS(3368),
    [anon_sym_await] = ACTIONS(3368),
    [anon_sym_become] = ACTIONS(3368),
    [anon_sym_break] = ACTIONS(3368),
    [anon_sym_const] = ACTIONS(3368),
    [anon_sym_continue] = ACTIONS(3368),
    [anon_sym_default] = ACTIONS(3368),
    [anon_sym_enum] = ACTIONS(3368),
    [anon_sym_fn] = ACTIONS(3368),
    [anon_sym_for] = ACTIONS(3368),
    [anon_sym_gen] = ACTIONS(3368),
    [anon_sym_if] = ACTIONS(3368),
    [anon_sym_impl] = ACTIONS(3368),
    [anon_sym_let] = ACTIONS(3368),
    [anon_sym_loop] = ACTIONS(3368),
    [anon_sym_match] = ACTIONS(3368),
    [anon_sym_mod] = ACTIONS(3368),
    [anon_sym_pub] = ACTIONS(3368),
    [anon_sym_return] = ACTIONS(3368),
    [anon_sym_raw] = ACTIONS(3368),
    [anon_sym_static] = ACTIONS(3368),
    [anon_sym_struct] = ACTIONS(3368),
    [anon_sym_trait] = ACTIONS(3368),
    [anon_sym_type] = ACTIONS(3368),
    [anon_sym_union] = ACTIONS(3368),
    [anon_sym_unsafe] = ACTIONS(3368),
    [anon_sym_use] = ACTIONS(3368),
    [anon_sym_while] = ACTIONS(3368),
    [anon_sym_extern] = ACTIONS(3368),
    [anon_sym_safe] = ACTIONS(3368),
    [anon_sym_yield] = ACTIONS(3368),
    [anon_sym_move] = ACTIONS(3368),
    [anon_sym_try] = ACTIONS(3368),
    [sym_integer_literal] = ACTIONS(3366),
    [aux_sym_string_literal_token1] = ACTIONS(3366),
    [sym_char_literal] = ACTIONS(3366),
    [anon_sym_true] = ACTIONS(3368),
    [anon_sym_false] = ACTIONS(3368),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3368),
    [sym_super] = ACTIONS(3368),
    [sym_crate] = ACTIONS(3368),
    [aux_sym_metavariable_token1] = ACTIONS(3366),
    [sym__raw_string_literal_start] = ACTIONS(3366),
    [sym_float_literal] = ACTIONS(3366),
    [sym__outer_block_doc_comment_start] = ACTIONS(3366),
    [sym__outer_line_doc_comment_start] = ACTIONS(3366),
  },
  [STATE(900)] = {
    [sym_line_comment] = STATE(900),
    [sym_block_comment] = STATE(900),
    [ts_builtin_sym_end] = ACTIONS(3370),
    [sym_identifier] = ACTIONS(3372),
    [anon_sym_SEMI] = ACTIONS(3370),
    [anon_sym_macro] = ACTIONS(3372),
    [anon_sym_LPAREN] = ACTIONS(3370),
    [anon_sym_LBRACE] = ACTIONS(3370),
    [anon_sym_RBRACE] = ACTIONS(3370),
    [anon_sym_macro_rules] = ACTIONS(3372),
    [anon_sym_BANG] = ACTIONS(3370),
    [anon_sym_LBRACK] = ACTIONS(3370),
    [anon_sym_STAR] = ACTIONS(3370),
    [anon_sym_u8] = ACTIONS(3372),
    [anon_sym_i8] = ACTIONS(3372),
    [anon_sym_u16] = ACTIONS(3372),
    [anon_sym_i16] = ACTIONS(3372),
    [anon_sym_u32] = ACTIONS(3372),
    [anon_sym_i32] = ACTIONS(3372),
    [anon_sym_u64] = ACTIONS(3372),
    [anon_sym_i64] = ACTIONS(3372),
    [anon_sym_u128] = ACTIONS(3372),
    [anon_sym_i128] = ACTIONS(3372),
    [anon_sym_isize] = ACTIONS(3372),
    [anon_sym_usize] = ACTIONS(3372),
    [anon_sym_f32] = ACTIONS(3372),
    [anon_sym_f64] = ACTIONS(3372),
    [anon_sym_bool] = ACTIONS(3372),
    [anon_sym_str] = ACTIONS(3372),
    [anon_sym_char] = ACTIONS(3372),
    [anon_sym_DASH] = ACTIONS(3370),
    [anon_sym_AMP] = ACTIONS(3370),
    [anon_sym_PIPE] = ACTIONS(3370),
    [anon_sym_LT] = ACTIONS(3370),
    [anon_sym_DOT_DOT] = ACTIONS(3370),
    [anon_sym_COLON_COLON] = ACTIONS(3370),
    [anon_sym_POUND] = ACTIONS(3370),
    [anon_sym_SQUOTE] = ACTIONS(3372),
    [anon_sym_async] = ACTIONS(3372),
    [anon_sym_auto] = ACTIONS(3372),
    [anon_sym_await] = ACTIONS(3372),
    [anon_sym_become] = ACTIONS(3372),
    [anon_sym_break] = ACTIONS(3372),
    [anon_sym_const] = ACTIONS(3372),
    [anon_sym_continue] = ACTIONS(3372),
    [anon_sym_default] = ACTIONS(3372),
    [anon_sym_enum] = ACTIONS(3372),
    [anon_sym_fn] = ACTIONS(3372),
    [anon_sym_for] = ACTIONS(3372),
    [anon_sym_gen] = ACTIONS(3372),
    [anon_sym_if] = ACTIONS(3372),
    [anon_sym_impl] = ACTIONS(3372),
    [anon_sym_let] = ACTIONS(3372),
    [anon_sym_loop] = ACTIONS(3372),
    [anon_sym_match] = ACTIONS(3372),
    [anon_sym_mod] = ACTIONS(3372),
    [anon_sym_pub] = ACTIONS(3372),
    [anon_sym_return] = ACTIONS(3372),
    [anon_sym_raw] = ACTIONS(3372),
    [anon_sym_static] = ACTIONS(3372),
    [anon_sym_struct] = ACTIONS(3372),
    [anon_sym_trait] = ACTIONS(3372),
    [anon_sym_type] = ACTIONS(3372),
    [anon_sym_union] = ACTIONS(3372),
    [anon_sym_unsafe] = ACTIONS(3372),
    [anon_sym_use] = ACTIONS(3372),
    [anon_sym_while] = ACTIONS(3372),
    [anon_sym_extern] = ACTIONS(3372),
    [anon_sym_safe] = ACTIONS(3372),
    [anon_sym_yield] = ACTIONS(3372),
    [anon_sym_move] = ACTIONS(3372),
    [anon_sym_try] = ACTIONS(3372),
    [sym_integer_literal] = ACTIONS(3370),
    [aux_sym_string_literal_token1] = ACTIONS(3370),
    [sym_char_literal] = ACTIONS(3370),
    [anon_sym_true] = ACTIONS(3372),
    [anon_sym_false] = ACTIONS(3372),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3372),
    [sym_super] = ACTIONS(3372),
    [sym_crate] = ACTIONS(3372),
    [aux_sym_metavariable_token1] = ACTIONS(3370),
    [sym__raw_string_literal_start] = ACTIONS(3370),
    [sym_float_literal] = ACTIONS(3370),
    [sym__outer_block_doc_comment_start] = ACTIONS(3370),
    [sym__outer_line_doc_comment_start] = ACTIONS(3370),
  },
  [STATE(901)] = {
    [sym_line_comment] = STATE(901),
    [sym_block_comment] = STATE(901),
    [ts_builtin_sym_end] = ACTIONS(3374),
    [sym_identifier] = ACTIONS(3376),
    [anon_sym_SEMI] = ACTIONS(3374),
    [anon_sym_macro] = ACTIONS(3376),
    [anon_sym_LPAREN] = ACTIONS(3374),
    [anon_sym_LBRACE] = ACTIONS(3374),
    [anon_sym_RBRACE] = ACTIONS(3374),
    [anon_sym_macro_rules] = ACTIONS(3376),
    [anon_sym_BANG] = ACTIONS(3374),
    [anon_sym_LBRACK] = ACTIONS(3374),
    [anon_sym_STAR] = ACTIONS(3374),
    [anon_sym_u8] = ACTIONS(3376),
    [anon_sym_i8] = ACTIONS(3376),
    [anon_sym_u16] = ACTIONS(3376),
    [anon_sym_i16] = ACTIONS(3376),
    [anon_sym_u32] = ACTIONS(3376),
    [anon_sym_i32] = ACTIONS(3376),
    [anon_sym_u64] = ACTIONS(3376),
    [anon_sym_i64] = ACTIONS(3376),
    [anon_sym_u128] = ACTIONS(3376),
    [anon_sym_i128] = ACTIONS(3376),
    [anon_sym_isize] = ACTIONS(3376),
    [anon_sym_usize] = ACTIONS(3376),
    [anon_sym_f32] = ACTIONS(3376),
    [anon_sym_f64] = ACTIONS(3376),
    [anon_sym_bool] = ACTIONS(3376),
    [anon_sym_str] = ACTIONS(3376),
    [anon_sym_char] = ACTIONS(3376),
    [anon_sym_DASH] = ACTIONS(3374),
    [anon_sym_AMP] = ACTIONS(3374),
    [anon_sym_PIPE] = ACTIONS(3374),
    [anon_sym_LT] = ACTIONS(3374),
    [anon_sym_DOT_DOT] = ACTIONS(3374),
    [anon_sym_COLON_COLON] = ACTIONS(3374),
    [anon_sym_POUND] = ACTIONS(3374),
    [anon_sym_SQUOTE] = ACTIONS(3376),
    [anon_sym_async] = ACTIONS(3376),
    [anon_sym_auto] = ACTIONS(3376),
    [anon_sym_await] = ACTIONS(3376),
    [anon_sym_become] = ACTIONS(3376),
    [anon_sym_break] = ACTIONS(3376),
    [anon_sym_const] = ACTIONS(3376),
    [anon_sym_continue] = ACTIONS(3376),
    [anon_sym_default] = ACTIONS(3376),
    [anon_sym_enum] = ACTIONS(3376),
    [anon_sym_fn] = ACTIONS(3376),
    [anon_sym_for] = ACTIONS(3376),
    [anon_sym_gen] = ACTIONS(3376),
    [anon_sym_if] = ACTIONS(3376),
    [anon_sym_impl] = ACTIONS(3376),
    [anon_sym_let] = ACTIONS(3376),
    [anon_sym_loop] = ACTIONS(3376),
    [anon_sym_match] = ACTIONS(3376),
    [anon_sym_mod] = ACTIONS(3376),
    [anon_sym_pub] = ACTIONS(3376),
    [anon_sym_return] = ACTIONS(3376),
    [anon_sym_raw] = ACTIONS(3376),
    [anon_sym_static] = ACTIONS(3376),
    [anon_sym_struct] = ACTIONS(3376),
    [anon_sym_trait] = ACTIONS(3376),
    [anon_sym_type] = ACTIONS(3376),
    [anon_sym_union] = ACTIONS(3376),
    [anon_sym_unsafe] = ACTIONS(3376),
    [anon_sym_use] = ACTIONS(3376),
    [anon_sym_while] = ACTIONS(3376),
    [anon_sym_extern] = ACTIONS(3376),
    [anon_sym_safe] = ACTIONS(3376),
    [anon_sym_yield] = ACTIONS(3376),
    [anon_sym_move] = ACTIONS(3376),
    [anon_sym_try] = ACTIONS(3376),
    [sym_integer_literal] = ACTIONS(3374),
    [aux_sym_string_literal_token1] = ACTIONS(3374),
    [sym_char_literal] = ACTIONS(3374),
    [anon_sym_true] = ACTIONS(3376),
    [anon_sym_false] = ACTIONS(3376),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3376),
    [sym_super] = ACTIONS(3376),
    [sym_crate] = ACTIONS(3376),
    [aux_sym_metavariable_token1] = ACTIONS(3374),
    [sym__raw_string_literal_start] = ACTIONS(3374),
    [sym_float_literal] = ACTIONS(3374),
    [sym__outer_block_doc_comment_start] = ACTIONS(3374),
    [sym__outer_line_doc_comment_start] = ACTIONS(3374),
  },
  [STATE(902)] = {
    [sym_line_comment] = STATE(902),
    [sym_block_comment] = STATE(902),
    [ts_builtin_sym_end] = ACTIONS(3378),
    [sym_identifier] = ACTIONS(3380),
    [anon_sym_SEMI] = ACTIONS(3378),
    [anon_sym_macro] = ACTIONS(3380),
    [anon_sym_LPAREN] = ACTIONS(3378),
    [anon_sym_LBRACE] = ACTIONS(3378),
    [anon_sym_RBRACE] = ACTIONS(3378),
    [anon_sym_macro_rules] = ACTIONS(3380),
    [anon_sym_BANG] = ACTIONS(3378),
    [anon_sym_LBRACK] = ACTIONS(3378),
    [anon_sym_STAR] = ACTIONS(3378),
    [anon_sym_u8] = ACTIONS(3380),
    [anon_sym_i8] = ACTIONS(3380),
    [anon_sym_u16] = ACTIONS(3380),
    [anon_sym_i16] = ACTIONS(3380),
    [anon_sym_u32] = ACTIONS(3380),
    [anon_sym_i32] = ACTIONS(3380),
    [anon_sym_u64] = ACTIONS(3380),
    [anon_sym_i64] = ACTIONS(3380),
    [anon_sym_u128] = ACTIONS(3380),
    [anon_sym_i128] = ACTIONS(3380),
    [anon_sym_isize] = ACTIONS(3380),
    [anon_sym_usize] = ACTIONS(3380),
    [anon_sym_f32] = ACTIONS(3380),
    [anon_sym_f64] = ACTIONS(3380),
    [anon_sym_bool] = ACTIONS(3380),
    [anon_sym_str] = ACTIONS(3380),
    [anon_sym_char] = ACTIONS(3380),
    [anon_sym_DASH] = ACTIONS(3378),
    [anon_sym_AMP] = ACTIONS(3378),
    [anon_sym_PIPE] = ACTIONS(3378),
    [anon_sym_LT] = ACTIONS(3378),
    [anon_sym_DOT_DOT] = ACTIONS(3378),
    [anon_sym_COLON_COLON] = ACTIONS(3378),
    [anon_sym_POUND] = ACTIONS(3378),
    [anon_sym_SQUOTE] = ACTIONS(3380),
    [anon_sym_async] = ACTIONS(3380),
    [anon_sym_auto] = ACTIONS(3380),
    [anon_sym_await] = ACTIONS(3380),
    [anon_sym_become] = ACTIONS(3380),
    [anon_sym_break] = ACTIONS(3380),
    [anon_sym_const] = ACTIONS(3380),
    [anon_sym_continue] = ACTIONS(3380),
    [anon_sym_default] = ACTIONS(3380),
    [anon_sym_enum] = ACTIONS(3380),
    [anon_sym_fn] = ACTIONS(3380),
    [anon_sym_for] = ACTIONS(3380),
    [anon_sym_gen] = ACTIONS(3380),
    [anon_sym_if] = ACTIONS(3380),
    [anon_sym_impl] = ACTIONS(3380),
    [anon_sym_let] = ACTIONS(3380),
    [anon_sym_loop] = ACTIONS(3380),
    [anon_sym_match] = ACTIONS(3380),
    [anon_sym_mod] = ACTIONS(3380),
    [anon_sym_pub] = ACTIONS(3380),
    [anon_sym_return] = ACTIONS(3380),
    [anon_sym_raw] = ACTIONS(3380),
    [anon_sym_static] = ACTIONS(3380),
    [anon_sym_struct] = ACTIONS(3380),
    [anon_sym_trait] = ACTIONS(3380),
    [anon_sym_type] = ACTIONS(3380),
    [anon_sym_union] = ACTIONS(3380),
    [anon_sym_unsafe] = ACTIONS(3380),
    [anon_sym_use] = ACTIONS(3380),
    [anon_sym_while] = ACTIONS(3380),
    [anon_sym_extern] = ACTIONS(3380),
    [anon_sym_safe] = ACTIONS(3380),
    [anon_sym_yield] = ACTIONS(3380),
    [anon_sym_move] = ACTIONS(3380),
    [anon_sym_try] = ACTIONS(3380),
    [sym_integer_literal] = ACTIONS(3378),
    [aux_sym_string_literal_token1] = ACTIONS(3378),
    [sym_char_literal] = ACTIONS(3378),
    [anon_sym_true] = ACTIONS(3380),
    [anon_sym_false] = ACTIONS(3380),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3380),
    [sym_super] = ACTIONS(3380),
    [sym_crate] = ACTIONS(3380),
    [aux_sym_metavariable_token1] = ACTIONS(3378),
    [sym__raw_string_literal_start] = ACTIONS(3378),
    [sym_float_literal] = ACTIONS(3378),
    [sym__outer_block_doc_comment_start] = ACTIONS(3378),
    [sym__outer_line_doc_comment_start] = ACTIONS(3378),
  },
  [STATE(903)] = {
    [sym_line_comment] = STATE(903),
    [sym_block_comment] = STATE(903),
    [ts_builtin_sym_end] = ACTIONS(3382),
    [sym_identifier] = ACTIONS(3384),
    [anon_sym_SEMI] = ACTIONS(3382),
    [anon_sym_macro] = ACTIONS(3384),
    [anon_sym_LPAREN] = ACTIONS(3382),
    [anon_sym_LBRACE] = ACTIONS(3382),
    [anon_sym_RBRACE] = ACTIONS(3382),
    [anon_sym_macro_rules] = ACTIONS(3384),
    [anon_sym_BANG] = ACTIONS(3382),
    [anon_sym_LBRACK] = ACTIONS(3382),
    [anon_sym_STAR] = ACTIONS(3382),
    [anon_sym_u8] = ACTIONS(3384),
    [anon_sym_i8] = ACTIONS(3384),
    [anon_sym_u16] = ACTIONS(3384),
    [anon_sym_i16] = ACTIONS(3384),
    [anon_sym_u32] = ACTIONS(3384),
    [anon_sym_i32] = ACTIONS(3384),
    [anon_sym_u64] = ACTIONS(3384),
    [anon_sym_i64] = ACTIONS(3384),
    [anon_sym_u128] = ACTIONS(3384),
    [anon_sym_i128] = ACTIONS(3384),
    [anon_sym_isize] = ACTIONS(3384),
    [anon_sym_usize] = ACTIONS(3384),
    [anon_sym_f32] = ACTIONS(3384),
    [anon_sym_f64] = ACTIONS(3384),
    [anon_sym_bool] = ACTIONS(3384),
    [anon_sym_str] = ACTIONS(3384),
    [anon_sym_char] = ACTIONS(3384),
    [anon_sym_DASH] = ACTIONS(3382),
    [anon_sym_AMP] = ACTIONS(3382),
    [anon_sym_PIPE] = ACTIONS(3382),
    [anon_sym_LT] = ACTIONS(3382),
    [anon_sym_DOT_DOT] = ACTIONS(3382),
    [anon_sym_COLON_COLON] = ACTIONS(3382),
    [anon_sym_POUND] = ACTIONS(3382),
    [anon_sym_SQUOTE] = ACTIONS(3384),
    [anon_sym_async] = ACTIONS(3384),
    [anon_sym_auto] = ACTIONS(3384),
    [anon_sym_await] = ACTIONS(3384),
    [anon_sym_become] = ACTIONS(3384),
    [anon_sym_break] = ACTIONS(3384),
    [anon_sym_const] = ACTIONS(3384),
    [anon_sym_continue] = ACTIONS(3384),
    [anon_sym_default] = ACTIONS(3384),
    [anon_sym_enum] = ACTIONS(3384),
    [anon_sym_fn] = ACTIONS(3384),
    [anon_sym_for] = ACTIONS(3384),
    [anon_sym_gen] = ACTIONS(3384),
    [anon_sym_if] = ACTIONS(3384),
    [anon_sym_impl] = ACTIONS(3384),
    [anon_sym_let] = ACTIONS(3384),
    [anon_sym_loop] = ACTIONS(3384),
    [anon_sym_match] = ACTIONS(3384),
    [anon_sym_mod] = ACTIONS(3384),
    [anon_sym_pub] = ACTIONS(3384),
    [anon_sym_return] = ACTIONS(3384),
    [anon_sym_raw] = ACTIONS(3384),
    [anon_sym_static] = ACTIONS(3384),
    [anon_sym_struct] = ACTIONS(3384),
    [anon_sym_trait] = ACTIONS(3384),
    [anon_sym_type] = ACTIONS(3384),
    [anon_sym_union] = ACTIONS(3384),
    [anon_sym_unsafe] = ACTIONS(3384),
    [anon_sym_use] = ACTIONS(3384),
    [anon_sym_while] = ACTIONS(3384),
    [anon_sym_extern] = ACTIONS(3384),
    [anon_sym_safe] = ACTIONS(3384),
    [anon_sym_yield] = ACTIONS(3384),
    [anon_sym_move] = ACTIONS(3384),
    [anon_sym_try] = ACTIONS(3384),
    [sym_integer_literal] = ACTIONS(3382),
    [aux_sym_string_literal_token1] = ACTIONS(3382),
    [sym_char_literal] = ACTIONS(3382),
    [anon_sym_true] = ACTIONS(3384),
    [anon_sym_false] = ACTIONS(3384),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3384),
    [sym_super] = ACTIONS(3384),
    [sym_crate] = ACTIONS(3384),
    [aux_sym_metavariable_token1] = ACTIONS(3382),
    [sym__raw_string_literal_start] = ACTIONS(3382),
    [sym_float_literal] = ACTIONS(3382),
    [sym__outer_block_doc_comment_start] = ACTIONS(3382),
    [sym__outer_line_doc_comment_start] = ACTIONS(3382),
  },
  [STATE(904)] = {
    [sym_line_comment] = STATE(904),
    [sym_block_comment] = STATE(904),
    [ts_builtin_sym_end] = ACTIONS(3386),
    [sym_identifier] = ACTIONS(3388),
    [anon_sym_SEMI] = ACTIONS(3386),
    [anon_sym_macro] = ACTIONS(3388),
    [anon_sym_LPAREN] = ACTIONS(3386),
    [anon_sym_LBRACE] = ACTIONS(3386),
    [anon_sym_RBRACE] = ACTIONS(3386),
    [anon_sym_macro_rules] = ACTIONS(3388),
    [anon_sym_BANG] = ACTIONS(3386),
    [anon_sym_LBRACK] = ACTIONS(3386),
    [anon_sym_STAR] = ACTIONS(3386),
    [anon_sym_u8] = ACTIONS(3388),
    [anon_sym_i8] = ACTIONS(3388),
    [anon_sym_u16] = ACTIONS(3388),
    [anon_sym_i16] = ACTIONS(3388),
    [anon_sym_u32] = ACTIONS(3388),
    [anon_sym_i32] = ACTIONS(3388),
    [anon_sym_u64] = ACTIONS(3388),
    [anon_sym_i64] = ACTIONS(3388),
    [anon_sym_u128] = ACTIONS(3388),
    [anon_sym_i128] = ACTIONS(3388),
    [anon_sym_isize] = ACTIONS(3388),
    [anon_sym_usize] = ACTIONS(3388),
    [anon_sym_f32] = ACTIONS(3388),
    [anon_sym_f64] = ACTIONS(3388),
    [anon_sym_bool] = ACTIONS(3388),
    [anon_sym_str] = ACTIONS(3388),
    [anon_sym_char] = ACTIONS(3388),
    [anon_sym_DASH] = ACTIONS(3386),
    [anon_sym_AMP] = ACTIONS(3386),
    [anon_sym_PIPE] = ACTIONS(3386),
    [anon_sym_LT] = ACTIONS(3386),
    [anon_sym_DOT_DOT] = ACTIONS(3386),
    [anon_sym_COLON_COLON] = ACTIONS(3386),
    [anon_sym_POUND] = ACTIONS(3386),
    [anon_sym_SQUOTE] = ACTIONS(3388),
    [anon_sym_async] = ACTIONS(3388),
    [anon_sym_auto] = ACTIONS(3388),
    [anon_sym_await] = ACTIONS(3388),
    [anon_sym_become] = ACTIONS(3388),
    [anon_sym_break] = ACTIONS(3388),
    [anon_sym_const] = ACTIONS(3388),
    [anon_sym_continue] = ACTIONS(3388),
    [anon_sym_default] = ACTIONS(3388),
    [anon_sym_enum] = ACTIONS(3388),
    [anon_sym_fn] = ACTIONS(3388),
    [anon_sym_for] = ACTIONS(3388),
    [anon_sym_gen] = ACTIONS(3388),
    [anon_sym_if] = ACTIONS(3388),
    [anon_sym_impl] = ACTIONS(3388),
    [anon_sym_let] = ACTIONS(3388),
    [anon_sym_loop] = ACTIONS(3388),
    [anon_sym_match] = ACTIONS(3388),
    [anon_sym_mod] = ACTIONS(3388),
    [anon_sym_pub] = ACTIONS(3388),
    [anon_sym_return] = ACTIONS(3388),
    [anon_sym_raw] = ACTIONS(3388),
    [anon_sym_static] = ACTIONS(3388),
    [anon_sym_struct] = ACTIONS(3388),
    [anon_sym_trait] = ACTIONS(3388),
    [anon_sym_type] = ACTIONS(3388),
    [anon_sym_union] = ACTIONS(3388),
    [anon_sym_unsafe] = ACTIONS(3388),
    [anon_sym_use] = ACTIONS(3388),
    [anon_sym_while] = ACTIONS(3388),
    [anon_sym_extern] = ACTIONS(3388),
    [anon_sym_safe] = ACTIONS(3388),
    [anon_sym_yield] = ACTIONS(3388),
    [anon_sym_move] = ACTIONS(3388),
    [anon_sym_try] = ACTIONS(3388),
    [sym_integer_literal] = ACTIONS(3386),
    [aux_sym_string_literal_token1] = ACTIONS(3386),
    [sym_char_literal] = ACTIONS(3386),
    [anon_sym_true] = ACTIONS(3388),
    [anon_sym_false] = ACTIONS(3388),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3388),
    [sym_super] = ACTIONS(3388),
    [sym_crate] = ACTIONS(3388),
    [aux_sym_metavariable_token1] = ACTIONS(3386),
    [sym__raw_string_literal_start] = ACTIONS(3386),
    [sym_float_literal] = ACTIONS(3386),
    [sym__outer_block_doc_comment_start] = ACTIONS(3386),
    [sym__outer_line_doc_comment_start] = ACTIONS(3386),
  },
  [STATE(905)] = {
    [sym_line_comment] = STATE(905),
    [sym_block_comment] = STATE(905),
    [ts_builtin_sym_end] = ACTIONS(3390),
    [sym_identifier] = ACTIONS(3392),
    [anon_sym_SEMI] = ACTIONS(3390),
    [anon_sym_macro] = ACTIONS(3392),
    [anon_sym_LPAREN] = ACTIONS(3390),
    [anon_sym_LBRACE] = ACTIONS(3390),
    [anon_sym_RBRACE] = ACTIONS(3390),
    [anon_sym_macro_rules] = ACTIONS(3392),
    [anon_sym_BANG] = ACTIONS(3390),
    [anon_sym_LBRACK] = ACTIONS(3390),
    [anon_sym_STAR] = ACTIONS(3390),
    [anon_sym_u8] = ACTIONS(3392),
    [anon_sym_i8] = ACTIONS(3392),
    [anon_sym_u16] = ACTIONS(3392),
    [anon_sym_i16] = ACTIONS(3392),
    [anon_sym_u32] = ACTIONS(3392),
    [anon_sym_i32] = ACTIONS(3392),
    [anon_sym_u64] = ACTIONS(3392),
    [anon_sym_i64] = ACTIONS(3392),
    [anon_sym_u128] = ACTIONS(3392),
    [anon_sym_i128] = ACTIONS(3392),
    [anon_sym_isize] = ACTIONS(3392),
    [anon_sym_usize] = ACTIONS(3392),
    [anon_sym_f32] = ACTIONS(3392),
    [anon_sym_f64] = ACTIONS(3392),
    [anon_sym_bool] = ACTIONS(3392),
    [anon_sym_str] = ACTIONS(3392),
    [anon_sym_char] = ACTIONS(3392),
    [anon_sym_DASH] = ACTIONS(3390),
    [anon_sym_AMP] = ACTIONS(3390),
    [anon_sym_PIPE] = ACTIONS(3390),
    [anon_sym_LT] = ACTIONS(3390),
    [anon_sym_DOT_DOT] = ACTIONS(3390),
    [anon_sym_COLON_COLON] = ACTIONS(3390),
    [anon_sym_POUND] = ACTIONS(3390),
    [anon_sym_SQUOTE] = ACTIONS(3392),
    [anon_sym_async] = ACTIONS(3392),
    [anon_sym_auto] = ACTIONS(3392),
    [anon_sym_await] = ACTIONS(3392),
    [anon_sym_become] = ACTIONS(3392),
    [anon_sym_break] = ACTIONS(3392),
    [anon_sym_const] = ACTIONS(3392),
    [anon_sym_continue] = ACTIONS(3392),
    [anon_sym_default] = ACTIONS(3392),
    [anon_sym_enum] = ACTIONS(3392),
    [anon_sym_fn] = ACTIONS(3392),
    [anon_sym_for] = ACTIONS(3392),
    [anon_sym_gen] = ACTIONS(3392),
    [anon_sym_if] = ACTIONS(3392),
    [anon_sym_impl] = ACTIONS(3392),
    [anon_sym_let] = ACTIONS(3392),
    [anon_sym_loop] = ACTIONS(3392),
    [anon_sym_match] = ACTIONS(3392),
    [anon_sym_mod] = ACTIONS(3392),
    [anon_sym_pub] = ACTIONS(3392),
    [anon_sym_return] = ACTIONS(3392),
    [anon_sym_raw] = ACTIONS(3392),
    [anon_sym_static] = ACTIONS(3392),
    [anon_sym_struct] = ACTIONS(3392),
    [anon_sym_trait] = ACTIONS(3392),
    [anon_sym_type] = ACTIONS(3392),
    [anon_sym_union] = ACTIONS(3392),
    [anon_sym_unsafe] = ACTIONS(3392),
    [anon_sym_use] = ACTIONS(3392),
    [anon_sym_while] = ACTIONS(3392),
    [anon_sym_extern] = ACTIONS(3392),
    [anon_sym_safe] = ACTIONS(3392),
    [anon_sym_yield] = ACTIONS(3392),
    [anon_sym_move] = ACTIONS(3392),
    [anon_sym_try] = ACTIONS(3392),
    [sym_integer_literal] = ACTIONS(3390),
    [aux_sym_string_literal_token1] = ACTIONS(3390),
    [sym_char_literal] = ACTIONS(3390),
    [anon_sym_true] = ACTIONS(3392),
    [anon_sym_false] = ACTIONS(3392),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3392),
    [sym_super] = ACTIONS(3392),
    [sym_crate] = ACTIONS(3392),
    [aux_sym_metavariable_token1] = ACTIONS(3390),
    [sym__raw_string_literal_start] = ACTIONS(3390),
    [sym_float_literal] = ACTIONS(3390),
    [sym__outer_block_doc_comment_start] = ACTIONS(3390),
    [sym__outer_line_doc_comment_start] = ACTIONS(3390),
  },
  [STATE(906)] = {
    [sym_line_comment] = STATE(906),
    [sym_block_comment] = STATE(906),
    [ts_builtin_sym_end] = ACTIONS(3394),
    [sym_identifier] = ACTIONS(3396),
    [anon_sym_SEMI] = ACTIONS(3394),
    [anon_sym_macro] = ACTIONS(3396),
    [anon_sym_LPAREN] = ACTIONS(3394),
    [anon_sym_LBRACE] = ACTIONS(3394),
    [anon_sym_RBRACE] = ACTIONS(3394),
    [anon_sym_macro_rules] = ACTIONS(3396),
    [anon_sym_BANG] = ACTIONS(3394),
    [anon_sym_LBRACK] = ACTIONS(3394),
    [anon_sym_STAR] = ACTIONS(3394),
    [anon_sym_u8] = ACTIONS(3396),
    [anon_sym_i8] = ACTIONS(3396),
    [anon_sym_u16] = ACTIONS(3396),
    [anon_sym_i16] = ACTIONS(3396),
    [anon_sym_u32] = ACTIONS(3396),
    [anon_sym_i32] = ACTIONS(3396),
    [anon_sym_u64] = ACTIONS(3396),
    [anon_sym_i64] = ACTIONS(3396),
    [anon_sym_u128] = ACTIONS(3396),
    [anon_sym_i128] = ACTIONS(3396),
    [anon_sym_isize] = ACTIONS(3396),
    [anon_sym_usize] = ACTIONS(3396),
    [anon_sym_f32] = ACTIONS(3396),
    [anon_sym_f64] = ACTIONS(3396),
    [anon_sym_bool] = ACTIONS(3396),
    [anon_sym_str] = ACTIONS(3396),
    [anon_sym_char] = ACTIONS(3396),
    [anon_sym_DASH] = ACTIONS(3394),
    [anon_sym_AMP] = ACTIONS(3394),
    [anon_sym_PIPE] = ACTIONS(3394),
    [anon_sym_LT] = ACTIONS(3394),
    [anon_sym_DOT_DOT] = ACTIONS(3394),
    [anon_sym_COLON_COLON] = ACTIONS(3394),
    [anon_sym_POUND] = ACTIONS(3394),
    [anon_sym_SQUOTE] = ACTIONS(3396),
    [anon_sym_async] = ACTIONS(3396),
    [anon_sym_auto] = ACTIONS(3396),
    [anon_sym_await] = ACTIONS(3396),
    [anon_sym_become] = ACTIONS(3396),
    [anon_sym_break] = ACTIONS(3396),
    [anon_sym_const] = ACTIONS(3396),
    [anon_sym_continue] = ACTIONS(3396),
    [anon_sym_default] = ACTIONS(3396),
    [anon_sym_enum] = ACTIONS(3396),
    [anon_sym_fn] = ACTIONS(3396),
    [anon_sym_for] = ACTIONS(3396),
    [anon_sym_gen] = ACTIONS(3396),
    [anon_sym_if] = ACTIONS(3396),
    [anon_sym_impl] = ACTIONS(3396),
    [anon_sym_let] = ACTIONS(3396),
    [anon_sym_loop] = ACTIONS(3396),
    [anon_sym_match] = ACTIONS(3396),
    [anon_sym_mod] = ACTIONS(3396),
    [anon_sym_pub] = ACTIONS(3396),
    [anon_sym_return] = ACTIONS(3396),
    [anon_sym_raw] = ACTIONS(3396),
    [anon_sym_static] = ACTIONS(3396),
    [anon_sym_struct] = ACTIONS(3396),
    [anon_sym_trait] = ACTIONS(3396),
    [anon_sym_type] = ACTIONS(3396),
    [anon_sym_union] = ACTIONS(3396),
    [anon_sym_unsafe] = ACTIONS(3396),
    [anon_sym_use] = ACTIONS(3396),
    [anon_sym_while] = ACTIONS(3396),
    [anon_sym_extern] = ACTIONS(3396),
    [anon_sym_safe] = ACTIONS(3396),
    [anon_sym_yield] = ACTIONS(3396),
    [anon_sym_move] = ACTIONS(3396),
    [anon_sym_try] = ACTIONS(3396),
    [sym_integer_literal] = ACTIONS(3394),
    [aux_sym_string_literal_token1] = ACTIONS(3394),
    [sym_char_literal] = ACTIONS(3394),
    [anon_sym_true] = ACTIONS(3396),
    [anon_sym_false] = ACTIONS(3396),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3396),
    [sym_super] = ACTIONS(3396),
    [sym_crate] = ACTIONS(3396),
    [aux_sym_metavariable_token1] = ACTIONS(3394),
    [sym__raw_string_literal_start] = ACTIONS(3394),
    [sym_float_literal] = ACTIONS(3394),
    [sym__outer_block_doc_comment_start] = ACTIONS(3394),
    [sym__outer_line_doc_comment_start] = ACTIONS(3394),
  },
  [STATE(907)] = {
    [sym_line_comment] = STATE(907),
    [sym_block_comment] = STATE(907),
    [ts_builtin_sym_end] = ACTIONS(3398),
    [sym_identifier] = ACTIONS(3400),
    [anon_sym_SEMI] = ACTIONS(3398),
    [anon_sym_macro] = ACTIONS(3400),
    [anon_sym_LPAREN] = ACTIONS(3398),
    [anon_sym_LBRACE] = ACTIONS(3398),
    [anon_sym_RBRACE] = ACTIONS(3398),
    [anon_sym_macro_rules] = ACTIONS(3400),
    [anon_sym_BANG] = ACTIONS(3398),
    [anon_sym_LBRACK] = ACTIONS(3398),
    [anon_sym_STAR] = ACTIONS(3398),
    [anon_sym_u8] = ACTIONS(3400),
    [anon_sym_i8] = ACTIONS(3400),
    [anon_sym_u16] = ACTIONS(3400),
    [anon_sym_i16] = ACTIONS(3400),
    [anon_sym_u32] = ACTIONS(3400),
    [anon_sym_i32] = ACTIONS(3400),
    [anon_sym_u64] = ACTIONS(3400),
    [anon_sym_i64] = ACTIONS(3400),
    [anon_sym_u128] = ACTIONS(3400),
    [anon_sym_i128] = ACTIONS(3400),
    [anon_sym_isize] = ACTIONS(3400),
    [anon_sym_usize] = ACTIONS(3400),
    [anon_sym_f32] = ACTIONS(3400),
    [anon_sym_f64] = ACTIONS(3400),
    [anon_sym_bool] = ACTIONS(3400),
    [anon_sym_str] = ACTIONS(3400),
    [anon_sym_char] = ACTIONS(3400),
    [anon_sym_DASH] = ACTIONS(3398),
    [anon_sym_AMP] = ACTIONS(3398),
    [anon_sym_PIPE] = ACTIONS(3398),
    [anon_sym_LT] = ACTIONS(3398),
    [anon_sym_DOT_DOT] = ACTIONS(3398),
    [anon_sym_COLON_COLON] = ACTIONS(3398),
    [anon_sym_POUND] = ACTIONS(3398),
    [anon_sym_SQUOTE] = ACTIONS(3400),
    [anon_sym_async] = ACTIONS(3400),
    [anon_sym_auto] = ACTIONS(3400),
    [anon_sym_await] = ACTIONS(3400),
    [anon_sym_become] = ACTIONS(3400),
    [anon_sym_break] = ACTIONS(3400),
    [anon_sym_const] = ACTIONS(3400),
    [anon_sym_continue] = ACTIONS(3400),
    [anon_sym_default] = ACTIONS(3400),
    [anon_sym_enum] = ACTIONS(3400),
    [anon_sym_fn] = ACTIONS(3400),
    [anon_sym_for] = ACTIONS(3400),
    [anon_sym_gen] = ACTIONS(3400),
    [anon_sym_if] = ACTIONS(3400),
    [anon_sym_impl] = ACTIONS(3400),
    [anon_sym_let] = ACTIONS(3400),
    [anon_sym_loop] = ACTIONS(3400),
    [anon_sym_match] = ACTIONS(3400),
    [anon_sym_mod] = ACTIONS(3400),
    [anon_sym_pub] = ACTIONS(3400),
    [anon_sym_return] = ACTIONS(3400),
    [anon_sym_raw] = ACTIONS(3400),
    [anon_sym_static] = ACTIONS(3400),
    [anon_sym_struct] = ACTIONS(3400),
    [anon_sym_trait] = ACTIONS(3400),
    [anon_sym_type] = ACTIONS(3400),
    [anon_sym_union] = ACTIONS(3400),
    [anon_sym_unsafe] = ACTIONS(3400),
    [anon_sym_use] = ACTIONS(3400),
    [anon_sym_while] = ACTIONS(3400),
    [anon_sym_extern] = ACTIONS(3400),
    [anon_sym_safe] = ACTIONS(3400),
    [anon_sym_yield] = ACTIONS(3400),
    [anon_sym_move] = ACTIONS(3400),
    [anon_sym_try] = ACTIONS(3400),
    [sym_integer_literal] = ACTIONS(3398),
    [aux_sym_string_literal_token1] = ACTIONS(3398),
    [sym_char_literal] = ACTIONS(3398),
    [anon_sym_true] = ACTIONS(3400),
    [anon_sym_false] = ACTIONS(3400),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3400),
    [sym_super] = ACTIONS(3400),
    [sym_crate] = ACTIONS(3400),
    [aux_sym_metavariable_token1] = ACTIONS(3398),
    [sym__raw_string_literal_start] = ACTIONS(3398),
    [sym_float_literal] = ACTIONS(3398),
    [sym__outer_block_doc_comment_start] = ACTIONS(3398),
    [sym__outer_line_doc_comment_start] = ACTIONS(3398),
  },
  [STATE(908)] = {
    [sym_line_comment] = STATE(908),
    [sym_block_comment] = STATE(908),
    [ts_builtin_sym_end] = ACTIONS(3402),
    [sym_identifier] = ACTIONS(3404),
    [anon_sym_SEMI] = ACTIONS(3402),
    [anon_sym_macro] = ACTIONS(3404),
    [anon_sym_LPAREN] = ACTIONS(3402),
    [anon_sym_LBRACE] = ACTIONS(3402),
    [anon_sym_RBRACE] = ACTIONS(3402),
    [anon_sym_macro_rules] = ACTIONS(3404),
    [anon_sym_BANG] = ACTIONS(3402),
    [anon_sym_LBRACK] = ACTIONS(3402),
    [anon_sym_STAR] = ACTIONS(3402),
    [anon_sym_u8] = ACTIONS(3404),
    [anon_sym_i8] = ACTIONS(3404),
    [anon_sym_u16] = ACTIONS(3404),
    [anon_sym_i16] = ACTIONS(3404),
    [anon_sym_u32] = ACTIONS(3404),
    [anon_sym_i32] = ACTIONS(3404),
    [anon_sym_u64] = ACTIONS(3404),
    [anon_sym_i64] = ACTIONS(3404),
    [anon_sym_u128] = ACTIONS(3404),
    [anon_sym_i128] = ACTIONS(3404),
    [anon_sym_isize] = ACTIONS(3404),
    [anon_sym_usize] = ACTIONS(3404),
    [anon_sym_f32] = ACTIONS(3404),
    [anon_sym_f64] = ACTIONS(3404),
    [anon_sym_bool] = ACTIONS(3404),
    [anon_sym_str] = ACTIONS(3404),
    [anon_sym_char] = ACTIONS(3404),
    [anon_sym_DASH] = ACTIONS(3402),
    [anon_sym_AMP] = ACTIONS(3402),
    [anon_sym_PIPE] = ACTIONS(3402),
    [anon_sym_LT] = ACTIONS(3402),
    [anon_sym_DOT_DOT] = ACTIONS(3402),
    [anon_sym_COLON_COLON] = ACTIONS(3402),
    [anon_sym_POUND] = ACTIONS(3402),
    [anon_sym_SQUOTE] = ACTIONS(3404),
    [anon_sym_async] = ACTIONS(3404),
    [anon_sym_auto] = ACTIONS(3404),
    [anon_sym_await] = ACTIONS(3404),
    [anon_sym_become] = ACTIONS(3404),
    [anon_sym_break] = ACTIONS(3404),
    [anon_sym_const] = ACTIONS(3404),
    [anon_sym_continue] = ACTIONS(3404),
    [anon_sym_default] = ACTIONS(3404),
    [anon_sym_enum] = ACTIONS(3404),
    [anon_sym_fn] = ACTIONS(3404),
    [anon_sym_for] = ACTIONS(3404),
    [anon_sym_gen] = ACTIONS(3404),
    [anon_sym_if] = ACTIONS(3404),
    [anon_sym_impl] = ACTIONS(3404),
    [anon_sym_let] = ACTIONS(3404),
    [anon_sym_loop] = ACTIONS(3404),
    [anon_sym_match] = ACTIONS(3404),
    [anon_sym_mod] = ACTIONS(3404),
    [anon_sym_pub] = ACTIONS(3404),
    [anon_sym_return] = ACTIONS(3404),
    [anon_sym_raw] = ACTIONS(3404),
    [anon_sym_static] = ACTIONS(3404),
    [anon_sym_struct] = ACTIONS(3404),
    [anon_sym_trait] = ACTIONS(3404),
    [anon_sym_type] = ACTIONS(3404),
    [anon_sym_union] = ACTIONS(3404),
    [anon_sym_unsafe] = ACTIONS(3404),
    [anon_sym_use] = ACTIONS(3404),
    [anon_sym_while] = ACTIONS(3404),
    [anon_sym_extern] = ACTIONS(3404),
    [anon_sym_safe] = ACTIONS(3404),
    [anon_sym_yield] = ACTIONS(3404),
    [anon_sym_move] = ACTIONS(3404),
    [anon_sym_try] = ACTIONS(3404),
    [sym_integer_literal] = ACTIONS(3402),
    [aux_sym_string_literal_token1] = ACTIONS(3402),
    [sym_char_literal] = ACTIONS(3402),
    [anon_sym_true] = ACTIONS(3404),
    [anon_sym_false] = ACTIONS(3404),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3404),
    [sym_super] = ACTIONS(3404),
    [sym_crate] = ACTIONS(3404),
    [aux_sym_metavariable_token1] = ACTIONS(3402),
    [sym__raw_string_literal_start] = ACTIONS(3402),
    [sym_float_literal] = ACTIONS(3402),
    [sym__outer_block_doc_comment_start] = ACTIONS(3402),
    [sym__outer_line_doc_comment_start] = ACTIONS(3402),
  },
  [STATE(909)] = {
    [sym_line_comment] = STATE(909),
    [sym_block_comment] = STATE(909),
    [ts_builtin_sym_end] = ACTIONS(3406),
    [sym_identifier] = ACTIONS(3408),
    [anon_sym_SEMI] = ACTIONS(3406),
    [anon_sym_macro] = ACTIONS(3408),
    [anon_sym_LPAREN] = ACTIONS(3406),
    [anon_sym_LBRACE] = ACTIONS(3406),
    [anon_sym_RBRACE] = ACTIONS(3406),
    [anon_sym_macro_rules] = ACTIONS(3408),
    [anon_sym_BANG] = ACTIONS(3406),
    [anon_sym_LBRACK] = ACTIONS(3406),
    [anon_sym_STAR] = ACTIONS(3406),
    [anon_sym_u8] = ACTIONS(3408),
    [anon_sym_i8] = ACTIONS(3408),
    [anon_sym_u16] = ACTIONS(3408),
    [anon_sym_i16] = ACTIONS(3408),
    [anon_sym_u32] = ACTIONS(3408),
    [anon_sym_i32] = ACTIONS(3408),
    [anon_sym_u64] = ACTIONS(3408),
    [anon_sym_i64] = ACTIONS(3408),
    [anon_sym_u128] = ACTIONS(3408),
    [anon_sym_i128] = ACTIONS(3408),
    [anon_sym_isize] = ACTIONS(3408),
    [anon_sym_usize] = ACTIONS(3408),
    [anon_sym_f32] = ACTIONS(3408),
    [anon_sym_f64] = ACTIONS(3408),
    [anon_sym_bool] = ACTIONS(3408),
    [anon_sym_str] = ACTIONS(3408),
    [anon_sym_char] = ACTIONS(3408),
    [anon_sym_DASH] = ACTIONS(3406),
    [anon_sym_AMP] = ACTIONS(3406),
    [anon_sym_PIPE] = ACTIONS(3406),
    [anon_sym_LT] = ACTIONS(3406),
    [anon_sym_DOT_DOT] = ACTIONS(3406),
    [anon_sym_COLON_COLON] = ACTIONS(3406),
    [anon_sym_POUND] = ACTIONS(3406),
    [anon_sym_SQUOTE] = ACTIONS(3408),
    [anon_sym_async] = ACTIONS(3408),
    [anon_sym_auto] = ACTIONS(3408),
    [anon_sym_await] = ACTIONS(3408),
    [anon_sym_become] = ACTIONS(3408),
    [anon_sym_break] = ACTIONS(3408),
    [anon_sym_const] = ACTIONS(3408),
    [anon_sym_continue] = ACTIONS(3408),
    [anon_sym_default] = ACTIONS(3408),
    [anon_sym_enum] = ACTIONS(3408),
    [anon_sym_fn] = ACTIONS(3408),
    [anon_sym_for] = ACTIONS(3408),
    [anon_sym_gen] = ACTIONS(3408),
    [anon_sym_if] = ACTIONS(3408),
    [anon_sym_impl] = ACTIONS(3408),
    [anon_sym_let] = ACTIONS(3408),
    [anon_sym_loop] = ACTIONS(3408),
    [anon_sym_match] = ACTIONS(3408),
    [anon_sym_mod] = ACTIONS(3408),
    [anon_sym_pub] = ACTIONS(3408),
    [anon_sym_return] = ACTIONS(3408),
    [anon_sym_raw] = ACTIONS(3408),
    [anon_sym_static] = ACTIONS(3408),
    [anon_sym_struct] = ACTIONS(3408),
    [anon_sym_trait] = ACTIONS(3408),
    [anon_sym_type] = ACTIONS(3408),
    [anon_sym_union] = ACTIONS(3408),
    [anon_sym_unsafe] = ACTIONS(3408),
    [anon_sym_use] = ACTIONS(3408),
    [anon_sym_while] = ACTIONS(3408),
    [anon_sym_extern] = ACTIONS(3408),
    [anon_sym_safe] = ACTIONS(3408),
    [anon_sym_yield] = ACTIONS(3408),
    [anon_sym_move] = ACTIONS(3408),
    [anon_sym_try] = ACTIONS(3408),
    [sym_integer_literal] = ACTIONS(3406),
    [aux_sym_string_literal_token1] = ACTIONS(3406),
    [sym_char_literal] = ACTIONS(3406),
    [anon_sym_true] = ACTIONS(3408),
    [anon_sym_false] = ACTIONS(3408),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3408),
    [sym_super] = ACTIONS(3408),
    [sym_crate] = ACTIONS(3408),
    [aux_sym_metavariable_token1] = ACTIONS(3406),
    [sym__raw_string_literal_start] = ACTIONS(3406),
    [sym_float_literal] = ACTIONS(3406),
    [sym__outer_block_doc_comment_start] = ACTIONS(3406),
    [sym__outer_line_doc_comment_start] = ACTIONS(3406),
  },
  [STATE(910)] = {
    [sym_line_comment] = STATE(910),
    [sym_block_comment] = STATE(910),
    [ts_builtin_sym_end] = ACTIONS(3410),
    [sym_identifier] = ACTIONS(3412),
    [anon_sym_SEMI] = ACTIONS(3410),
    [anon_sym_macro] = ACTIONS(3412),
    [anon_sym_LPAREN] = ACTIONS(3410),
    [anon_sym_LBRACE] = ACTIONS(3410),
    [anon_sym_RBRACE] = ACTIONS(3410),
    [anon_sym_macro_rules] = ACTIONS(3412),
    [anon_sym_BANG] = ACTIONS(3410),
    [anon_sym_LBRACK] = ACTIONS(3410),
    [anon_sym_STAR] = ACTIONS(3410),
    [anon_sym_u8] = ACTIONS(3412),
    [anon_sym_i8] = ACTIONS(3412),
    [anon_sym_u16] = ACTIONS(3412),
    [anon_sym_i16] = ACTIONS(3412),
    [anon_sym_u32] = ACTIONS(3412),
    [anon_sym_i32] = ACTIONS(3412),
    [anon_sym_u64] = ACTIONS(3412),
    [anon_sym_i64] = ACTIONS(3412),
    [anon_sym_u128] = ACTIONS(3412),
    [anon_sym_i128] = ACTIONS(3412),
    [anon_sym_isize] = ACTIONS(3412),
    [anon_sym_usize] = ACTIONS(3412),
    [anon_sym_f32] = ACTIONS(3412),
    [anon_sym_f64] = ACTIONS(3412),
    [anon_sym_bool] = ACTIONS(3412),
    [anon_sym_str] = ACTIONS(3412),
    [anon_sym_char] = ACTIONS(3412),
    [anon_sym_DASH] = ACTIONS(3410),
    [anon_sym_AMP] = ACTIONS(3410),
    [anon_sym_PIPE] = ACTIONS(3410),
    [anon_sym_LT] = ACTIONS(3410),
    [anon_sym_DOT_DOT] = ACTIONS(3410),
    [anon_sym_COLON_COLON] = ACTIONS(3410),
    [anon_sym_POUND] = ACTIONS(3410),
    [anon_sym_SQUOTE] = ACTIONS(3412),
    [anon_sym_async] = ACTIONS(3412),
    [anon_sym_auto] = ACTIONS(3412),
    [anon_sym_await] = ACTIONS(3412),
    [anon_sym_become] = ACTIONS(3412),
    [anon_sym_break] = ACTIONS(3412),
    [anon_sym_const] = ACTIONS(3412),
    [anon_sym_continue] = ACTIONS(3412),
    [anon_sym_default] = ACTIONS(3412),
    [anon_sym_enum] = ACTIONS(3412),
    [anon_sym_fn] = ACTIONS(3412),
    [anon_sym_for] = ACTIONS(3412),
    [anon_sym_gen] = ACTIONS(3412),
    [anon_sym_if] = ACTIONS(3412),
    [anon_sym_impl] = ACTIONS(3412),
    [anon_sym_let] = ACTIONS(3412),
    [anon_sym_loop] = ACTIONS(3412),
    [anon_sym_match] = ACTIONS(3412),
    [anon_sym_mod] = ACTIONS(3412),
    [anon_sym_pub] = ACTIONS(3412),
    [anon_sym_return] = ACTIONS(3412),
    [anon_sym_raw] = ACTIONS(3412),
    [anon_sym_static] = ACTIONS(3412),
    [anon_sym_struct] = ACTIONS(3412),
    [anon_sym_trait] = ACTIONS(3412),
    [anon_sym_type] = ACTIONS(3412),
    [anon_sym_union] = ACTIONS(3412),
    [anon_sym_unsafe] = ACTIONS(3412),
    [anon_sym_use] = ACTIONS(3412),
    [anon_sym_while] = ACTIONS(3412),
    [anon_sym_extern] = ACTIONS(3412),
    [anon_sym_safe] = ACTIONS(3412),
    [anon_sym_yield] = ACTIONS(3412),
    [anon_sym_move] = ACTIONS(3412),
    [anon_sym_try] = ACTIONS(3412),
    [sym_integer_literal] = ACTIONS(3410),
    [aux_sym_string_literal_token1] = ACTIONS(3410),
    [sym_char_literal] = ACTIONS(3410),
    [anon_sym_true] = ACTIONS(3412),
    [anon_sym_false] = ACTIONS(3412),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3412),
    [sym_super] = ACTIONS(3412),
    [sym_crate] = ACTIONS(3412),
    [aux_sym_metavariable_token1] = ACTIONS(3410),
    [sym__raw_string_literal_start] = ACTIONS(3410),
    [sym_float_literal] = ACTIONS(3410),
    [sym__outer_block_doc_comment_start] = ACTIONS(3410),
    [sym__outer_line_doc_comment_start] = ACTIONS(3410),
  },
  [STATE(911)] = {
    [sym_line_comment] = STATE(911),
    [sym_block_comment] = STATE(911),
    [ts_builtin_sym_end] = ACTIONS(3414),
    [sym_identifier] = ACTIONS(3416),
    [anon_sym_SEMI] = ACTIONS(3414),
    [anon_sym_macro] = ACTIONS(3416),
    [anon_sym_LPAREN] = ACTIONS(3414),
    [anon_sym_LBRACE] = ACTIONS(3414),
    [anon_sym_RBRACE] = ACTIONS(3414),
    [anon_sym_macro_rules] = ACTIONS(3416),
    [anon_sym_BANG] = ACTIONS(3414),
    [anon_sym_LBRACK] = ACTIONS(3414),
    [anon_sym_STAR] = ACTIONS(3414),
    [anon_sym_u8] = ACTIONS(3416),
    [anon_sym_i8] = ACTIONS(3416),
    [anon_sym_u16] = ACTIONS(3416),
    [anon_sym_i16] = ACTIONS(3416),
    [anon_sym_u32] = ACTIONS(3416),
    [anon_sym_i32] = ACTIONS(3416),
    [anon_sym_u64] = ACTIONS(3416),
    [anon_sym_i64] = ACTIONS(3416),
    [anon_sym_u128] = ACTIONS(3416),
    [anon_sym_i128] = ACTIONS(3416),
    [anon_sym_isize] = ACTIONS(3416),
    [anon_sym_usize] = ACTIONS(3416),
    [anon_sym_f32] = ACTIONS(3416),
    [anon_sym_f64] = ACTIONS(3416),
    [anon_sym_bool] = ACTIONS(3416),
    [anon_sym_str] = ACTIONS(3416),
    [anon_sym_char] = ACTIONS(3416),
    [anon_sym_DASH] = ACTIONS(3414),
    [anon_sym_AMP] = ACTIONS(3414),
    [anon_sym_PIPE] = ACTIONS(3414),
    [anon_sym_LT] = ACTIONS(3414),
    [anon_sym_DOT_DOT] = ACTIONS(3414),
    [anon_sym_COLON_COLON] = ACTIONS(3414),
    [anon_sym_POUND] = ACTIONS(3414),
    [anon_sym_SQUOTE] = ACTIONS(3416),
    [anon_sym_async] = ACTIONS(3416),
    [anon_sym_auto] = ACTIONS(3416),
    [anon_sym_await] = ACTIONS(3416),
    [anon_sym_become] = ACTIONS(3416),
    [anon_sym_break] = ACTIONS(3416),
    [anon_sym_const] = ACTIONS(3416),
    [anon_sym_continue] = ACTIONS(3416),
    [anon_sym_default] = ACTIONS(3416),
    [anon_sym_enum] = ACTIONS(3416),
    [anon_sym_fn] = ACTIONS(3416),
    [anon_sym_for] = ACTIONS(3416),
    [anon_sym_gen] = ACTIONS(3416),
    [anon_sym_if] = ACTIONS(3416),
    [anon_sym_impl] = ACTIONS(3416),
    [anon_sym_let] = ACTIONS(3416),
    [anon_sym_loop] = ACTIONS(3416),
    [anon_sym_match] = ACTIONS(3416),
    [anon_sym_mod] = ACTIONS(3416),
    [anon_sym_pub] = ACTIONS(3416),
    [anon_sym_return] = ACTIONS(3416),
    [anon_sym_raw] = ACTIONS(3416),
    [anon_sym_static] = ACTIONS(3416),
    [anon_sym_struct] = ACTIONS(3416),
    [anon_sym_trait] = ACTIONS(3416),
    [anon_sym_type] = ACTIONS(3416),
    [anon_sym_union] = ACTIONS(3416),
    [anon_sym_unsafe] = ACTIONS(3416),
    [anon_sym_use] = ACTIONS(3416),
    [anon_sym_while] = ACTIONS(3416),
    [anon_sym_extern] = ACTIONS(3416),
    [anon_sym_safe] = ACTIONS(3416),
    [anon_sym_yield] = ACTIONS(3416),
    [anon_sym_move] = ACTIONS(3416),
    [anon_sym_try] = ACTIONS(3416),
    [sym_integer_literal] = ACTIONS(3414),
    [aux_sym_string_literal_token1] = ACTIONS(3414),
    [sym_char_literal] = ACTIONS(3414),
    [anon_sym_true] = ACTIONS(3416),
    [anon_sym_false] = ACTIONS(3416),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3416),
    [sym_super] = ACTIONS(3416),
    [sym_crate] = ACTIONS(3416),
    [aux_sym_metavariable_token1] = ACTIONS(3414),
    [sym__raw_string_literal_start] = ACTIONS(3414),
    [sym_float_literal] = ACTIONS(3414),
    [sym__outer_block_doc_comment_start] = ACTIONS(3414),
    [sym__outer_line_doc_comment_start] = ACTIONS(3414),
  },
  [STATE(912)] = {
    [sym_line_comment] = STATE(912),
    [sym_block_comment] = STATE(912),
    [ts_builtin_sym_end] = ACTIONS(3418),
    [sym_identifier] = ACTIONS(3420),
    [anon_sym_SEMI] = ACTIONS(3418),
    [anon_sym_macro] = ACTIONS(3420),
    [anon_sym_LPAREN] = ACTIONS(3418),
    [anon_sym_LBRACE] = ACTIONS(3418),
    [anon_sym_RBRACE] = ACTIONS(3418),
    [anon_sym_macro_rules] = ACTIONS(3420),
    [anon_sym_BANG] = ACTIONS(3418),
    [anon_sym_LBRACK] = ACTIONS(3418),
    [anon_sym_STAR] = ACTIONS(3418),
    [anon_sym_u8] = ACTIONS(3420),
    [anon_sym_i8] = ACTIONS(3420),
    [anon_sym_u16] = ACTIONS(3420),
    [anon_sym_i16] = ACTIONS(3420),
    [anon_sym_u32] = ACTIONS(3420),
    [anon_sym_i32] = ACTIONS(3420),
    [anon_sym_u64] = ACTIONS(3420),
    [anon_sym_i64] = ACTIONS(3420),
    [anon_sym_u128] = ACTIONS(3420),
    [anon_sym_i128] = ACTIONS(3420),
    [anon_sym_isize] = ACTIONS(3420),
    [anon_sym_usize] = ACTIONS(3420),
    [anon_sym_f32] = ACTIONS(3420),
    [anon_sym_f64] = ACTIONS(3420),
    [anon_sym_bool] = ACTIONS(3420),
    [anon_sym_str] = ACTIONS(3420),
    [anon_sym_char] = ACTIONS(3420),
    [anon_sym_DASH] = ACTIONS(3418),
    [anon_sym_AMP] = ACTIONS(3418),
    [anon_sym_PIPE] = ACTIONS(3418),
    [anon_sym_LT] = ACTIONS(3418),
    [anon_sym_DOT_DOT] = ACTIONS(3418),
    [anon_sym_COLON_COLON] = ACTIONS(3418),
    [anon_sym_POUND] = ACTIONS(3418),
    [anon_sym_SQUOTE] = ACTIONS(3420),
    [anon_sym_async] = ACTIONS(3420),
    [anon_sym_auto] = ACTIONS(3420),
    [anon_sym_await] = ACTIONS(3420),
    [anon_sym_become] = ACTIONS(3420),
    [anon_sym_break] = ACTIONS(3420),
    [anon_sym_const] = ACTIONS(3420),
    [anon_sym_continue] = ACTIONS(3420),
    [anon_sym_default] = ACTIONS(3420),
    [anon_sym_enum] = ACTIONS(3420),
    [anon_sym_fn] = ACTIONS(3420),
    [anon_sym_for] = ACTIONS(3420),
    [anon_sym_gen] = ACTIONS(3420),
    [anon_sym_if] = ACTIONS(3420),
    [anon_sym_impl] = ACTIONS(3420),
    [anon_sym_let] = ACTIONS(3420),
    [anon_sym_loop] = ACTIONS(3420),
    [anon_sym_match] = ACTIONS(3420),
    [anon_sym_mod] = ACTIONS(3420),
    [anon_sym_pub] = ACTIONS(3420),
    [anon_sym_return] = ACTIONS(3420),
    [anon_sym_raw] = ACTIONS(3420),
    [anon_sym_static] = ACTIONS(3420),
    [anon_sym_struct] = ACTIONS(3420),
    [anon_sym_trait] = ACTIONS(3420),
    [anon_sym_type] = ACTIONS(3420),
    [anon_sym_union] = ACTIONS(3420),
    [anon_sym_unsafe] = ACTIONS(3420),
    [anon_sym_use] = ACTIONS(3420),
    [anon_sym_while] = ACTIONS(3420),
    [anon_sym_extern] = ACTIONS(3420),
    [anon_sym_safe] = ACTIONS(3420),
    [anon_sym_yield] = ACTIONS(3420),
    [anon_sym_move] = ACTIONS(3420),
    [anon_sym_try] = ACTIONS(3420),
    [sym_integer_literal] = ACTIONS(3418),
    [aux_sym_string_literal_token1] = ACTIONS(3418),
    [sym_char_literal] = ACTIONS(3418),
    [anon_sym_true] = ACTIONS(3420),
    [anon_sym_false] = ACTIONS(3420),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3420),
    [sym_super] = ACTIONS(3420),
    [sym_crate] = ACTIONS(3420),
    [aux_sym_metavariable_token1] = ACTIONS(3418),
    [sym__raw_string_literal_start] = ACTIONS(3418),
    [sym_float_literal] = ACTIONS(3418),
    [sym__outer_block_doc_comment_start] = ACTIONS(3418),
    [sym__outer_line_doc_comment_start] = ACTIONS(3418),
  },
  [STATE(913)] = {
    [sym_line_comment] = STATE(913),
    [sym_block_comment] = STATE(913),
    [ts_builtin_sym_end] = ACTIONS(3422),
    [sym_identifier] = ACTIONS(3424),
    [anon_sym_SEMI] = ACTIONS(3422),
    [anon_sym_macro] = ACTIONS(3424),
    [anon_sym_LPAREN] = ACTIONS(3422),
    [anon_sym_LBRACE] = ACTIONS(3422),
    [anon_sym_RBRACE] = ACTIONS(3422),
    [anon_sym_macro_rules] = ACTIONS(3424),
    [anon_sym_BANG] = ACTIONS(3422),
    [anon_sym_LBRACK] = ACTIONS(3422),
    [anon_sym_STAR] = ACTIONS(3422),
    [anon_sym_u8] = ACTIONS(3424),
    [anon_sym_i8] = ACTIONS(3424),
    [anon_sym_u16] = ACTIONS(3424),
    [anon_sym_i16] = ACTIONS(3424),
    [anon_sym_u32] = ACTIONS(3424),
    [anon_sym_i32] = ACTIONS(3424),
    [anon_sym_u64] = ACTIONS(3424),
    [anon_sym_i64] = ACTIONS(3424),
    [anon_sym_u128] = ACTIONS(3424),
    [anon_sym_i128] = ACTIONS(3424),
    [anon_sym_isize] = ACTIONS(3424),
    [anon_sym_usize] = ACTIONS(3424),
    [anon_sym_f32] = ACTIONS(3424),
    [anon_sym_f64] = ACTIONS(3424),
    [anon_sym_bool] = ACTIONS(3424),
    [anon_sym_str] = ACTIONS(3424),
    [anon_sym_char] = ACTIONS(3424),
    [anon_sym_DASH] = ACTIONS(3422),
    [anon_sym_AMP] = ACTIONS(3422),
    [anon_sym_PIPE] = ACTIONS(3422),
    [anon_sym_LT] = ACTIONS(3422),
    [anon_sym_DOT_DOT] = ACTIONS(3422),
    [anon_sym_COLON_COLON] = ACTIONS(3422),
    [anon_sym_POUND] = ACTIONS(3422),
    [anon_sym_SQUOTE] = ACTIONS(3424),
    [anon_sym_async] = ACTIONS(3424),
    [anon_sym_auto] = ACTIONS(3424),
    [anon_sym_await] = ACTIONS(3424),
    [anon_sym_become] = ACTIONS(3424),
    [anon_sym_break] = ACTIONS(3424),
    [anon_sym_const] = ACTIONS(3424),
    [anon_sym_continue] = ACTIONS(3424),
    [anon_sym_default] = ACTIONS(3424),
    [anon_sym_enum] = ACTIONS(3424),
    [anon_sym_fn] = ACTIONS(3424),
    [anon_sym_for] = ACTIONS(3424),
    [anon_sym_gen] = ACTIONS(3424),
    [anon_sym_if] = ACTIONS(3424),
    [anon_sym_impl] = ACTIONS(3424),
    [anon_sym_let] = ACTIONS(3424),
    [anon_sym_loop] = ACTIONS(3424),
    [anon_sym_match] = ACTIONS(3424),
    [anon_sym_mod] = ACTIONS(3424),
    [anon_sym_pub] = ACTIONS(3424),
    [anon_sym_return] = ACTIONS(3424),
    [anon_sym_raw] = ACTIONS(3424),
    [anon_sym_static] = ACTIONS(3424),
    [anon_sym_struct] = ACTIONS(3424),
    [anon_sym_trait] = ACTIONS(3424),
    [anon_sym_type] = ACTIONS(3424),
    [anon_sym_union] = ACTIONS(3424),
    [anon_sym_unsafe] = ACTIONS(3424),
    [anon_sym_use] = ACTIONS(3424),
    [anon_sym_while] = ACTIONS(3424),
    [anon_sym_extern] = ACTIONS(3424),
    [anon_sym_safe] = ACTIONS(3424),
    [anon_sym_yield] = ACTIONS(3424),
    [anon_sym_move] = ACTIONS(3424),
    [anon_sym_try] = ACTIONS(3424),
    [sym_integer_literal] = ACTIONS(3422),
    [aux_sym_string_literal_token1] = ACTIONS(3422),
    [sym_char_literal] = ACTIONS(3422),
    [anon_sym_true] = ACTIONS(3424),
    [anon_sym_false] = ACTIONS(3424),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3424),
    [sym_super] = ACTIONS(3424),
    [sym_crate] = ACTIONS(3424),
    [aux_sym_metavariable_token1] = ACTIONS(3422),
    [sym__raw_string_literal_start] = ACTIONS(3422),
    [sym_float_literal] = ACTIONS(3422),
    [sym__outer_block_doc_comment_start] = ACTIONS(3422),
    [sym__outer_line_doc_comment_start] = ACTIONS(3422),
  },
  [STATE(914)] = {
    [sym_line_comment] = STATE(914),
    [sym_block_comment] = STATE(914),
    [ts_builtin_sym_end] = ACTIONS(3426),
    [sym_identifier] = ACTIONS(3428),
    [anon_sym_SEMI] = ACTIONS(3426),
    [anon_sym_macro] = ACTIONS(3428),
    [anon_sym_LPAREN] = ACTIONS(3426),
    [anon_sym_LBRACE] = ACTIONS(3426),
    [anon_sym_RBRACE] = ACTIONS(3426),
    [anon_sym_macro_rules] = ACTIONS(3428),
    [anon_sym_BANG] = ACTIONS(3426),
    [anon_sym_LBRACK] = ACTIONS(3426),
    [anon_sym_STAR] = ACTIONS(3426),
    [anon_sym_u8] = ACTIONS(3428),
    [anon_sym_i8] = ACTIONS(3428),
    [anon_sym_u16] = ACTIONS(3428),
    [anon_sym_i16] = ACTIONS(3428),
    [anon_sym_u32] = ACTIONS(3428),
    [anon_sym_i32] = ACTIONS(3428),
    [anon_sym_u64] = ACTIONS(3428),
    [anon_sym_i64] = ACTIONS(3428),
    [anon_sym_u128] = ACTIONS(3428),
    [anon_sym_i128] = ACTIONS(3428),
    [anon_sym_isize] = ACTIONS(3428),
    [anon_sym_usize] = ACTIONS(3428),
    [anon_sym_f32] = ACTIONS(3428),
    [anon_sym_f64] = ACTIONS(3428),
    [anon_sym_bool] = ACTIONS(3428),
    [anon_sym_str] = ACTIONS(3428),
    [anon_sym_char] = ACTIONS(3428),
    [anon_sym_DASH] = ACTIONS(3426),
    [anon_sym_AMP] = ACTIONS(3426),
    [anon_sym_PIPE] = ACTIONS(3426),
    [anon_sym_LT] = ACTIONS(3426),
    [anon_sym_DOT_DOT] = ACTIONS(3426),
    [anon_sym_COLON_COLON] = ACTIONS(3426),
    [anon_sym_POUND] = ACTIONS(3426),
    [anon_sym_SQUOTE] = ACTIONS(3428),
    [anon_sym_async] = ACTIONS(3428),
    [anon_sym_auto] = ACTIONS(3428),
    [anon_sym_await] = ACTIONS(3428),
    [anon_sym_become] = ACTIONS(3428),
    [anon_sym_break] = ACTIONS(3428),
    [anon_sym_const] = ACTIONS(3428),
    [anon_sym_continue] = ACTIONS(3428),
    [anon_sym_default] = ACTIONS(3428),
    [anon_sym_enum] = ACTIONS(3428),
    [anon_sym_fn] = ACTIONS(3428),
    [anon_sym_for] = ACTIONS(3428),
    [anon_sym_gen] = ACTIONS(3428),
    [anon_sym_if] = ACTIONS(3428),
    [anon_sym_impl] = ACTIONS(3428),
    [anon_sym_let] = ACTIONS(3428),
    [anon_sym_loop] = ACTIONS(3428),
    [anon_sym_match] = ACTIONS(3428),
    [anon_sym_mod] = ACTIONS(3428),
    [anon_sym_pub] = ACTIONS(3428),
    [anon_sym_return] = ACTIONS(3428),
    [anon_sym_raw] = ACTIONS(3428),
    [anon_sym_static] = ACTIONS(3428),
    [anon_sym_struct] = ACTIONS(3428),
    [anon_sym_trait] = ACTIONS(3428),
    [anon_sym_type] = ACTIONS(3428),
    [anon_sym_union] = ACTIONS(3428),
    [anon_sym_unsafe] = ACTIONS(3428),
    [anon_sym_use] = ACTIONS(3428),
    [anon_sym_while] = ACTIONS(3428),
    [anon_sym_extern] = ACTIONS(3428),
    [anon_sym_safe] = ACTIONS(3428),
    [anon_sym_yield] = ACTIONS(3428),
    [anon_sym_move] = ACTIONS(3428),
    [anon_sym_try] = ACTIONS(3428),
    [sym_integer_literal] = ACTIONS(3426),
    [aux_sym_string_literal_token1] = ACTIONS(3426),
    [sym_char_literal] = ACTIONS(3426),
    [anon_sym_true] = ACTIONS(3428),
    [anon_sym_false] = ACTIONS(3428),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3428),
    [sym_super] = ACTIONS(3428),
    [sym_crate] = ACTIONS(3428),
    [aux_sym_metavariable_token1] = ACTIONS(3426),
    [sym__raw_string_literal_start] = ACTIONS(3426),
    [sym_float_literal] = ACTIONS(3426),
    [sym__outer_block_doc_comment_start] = ACTIONS(3426),
    [sym__outer_line_doc_comment_start] = ACTIONS(3426),
  },
  [STATE(915)] = {
    [sym_line_comment] = STATE(915),
    [sym_block_comment] = STATE(915),
    [ts_builtin_sym_end] = ACTIONS(3430),
    [sym_identifier] = ACTIONS(3432),
    [anon_sym_SEMI] = ACTIONS(3430),
    [anon_sym_macro] = ACTIONS(3432),
    [anon_sym_LPAREN] = ACTIONS(3430),
    [anon_sym_LBRACE] = ACTIONS(3430),
    [anon_sym_RBRACE] = ACTIONS(3430),
    [anon_sym_macro_rules] = ACTIONS(3432),
    [anon_sym_BANG] = ACTIONS(3430),
    [anon_sym_LBRACK] = ACTIONS(3430),
    [anon_sym_STAR] = ACTIONS(3430),
    [anon_sym_u8] = ACTIONS(3432),
    [anon_sym_i8] = ACTIONS(3432),
    [anon_sym_u16] = ACTIONS(3432),
    [anon_sym_i16] = ACTIONS(3432),
    [anon_sym_u32] = ACTIONS(3432),
    [anon_sym_i32] = ACTIONS(3432),
    [anon_sym_u64] = ACTIONS(3432),
    [anon_sym_i64] = ACTIONS(3432),
    [anon_sym_u128] = ACTIONS(3432),
    [anon_sym_i128] = ACTIONS(3432),
    [anon_sym_isize] = ACTIONS(3432),
    [anon_sym_usize] = ACTIONS(3432),
    [anon_sym_f32] = ACTIONS(3432),
    [anon_sym_f64] = ACTIONS(3432),
    [anon_sym_bool] = ACTIONS(3432),
    [anon_sym_str] = ACTIONS(3432),
    [anon_sym_char] = ACTIONS(3432),
    [anon_sym_DASH] = ACTIONS(3430),
    [anon_sym_AMP] = ACTIONS(3430),
    [anon_sym_PIPE] = ACTIONS(3430),
    [anon_sym_LT] = ACTIONS(3430),
    [anon_sym_DOT_DOT] = ACTIONS(3430),
    [anon_sym_COLON_COLON] = ACTIONS(3430),
    [anon_sym_POUND] = ACTIONS(3430),
    [anon_sym_SQUOTE] = ACTIONS(3432),
    [anon_sym_async] = ACTIONS(3432),
    [anon_sym_auto] = ACTIONS(3432),
    [anon_sym_await] = ACTIONS(3432),
    [anon_sym_become] = ACTIONS(3432),
    [anon_sym_break] = ACTIONS(3432),
    [anon_sym_const] = ACTIONS(3432),
    [anon_sym_continue] = ACTIONS(3432),
    [anon_sym_default] = ACTIONS(3432),
    [anon_sym_enum] = ACTIONS(3432),
    [anon_sym_fn] = ACTIONS(3432),
    [anon_sym_for] = ACTIONS(3432),
    [anon_sym_gen] = ACTIONS(3432),
    [anon_sym_if] = ACTIONS(3432),
    [anon_sym_impl] = ACTIONS(3432),
    [anon_sym_let] = ACTIONS(3432),
    [anon_sym_loop] = ACTIONS(3432),
    [anon_sym_match] = ACTIONS(3432),
    [anon_sym_mod] = ACTIONS(3432),
    [anon_sym_pub] = ACTIONS(3432),
    [anon_sym_return] = ACTIONS(3432),
    [anon_sym_raw] = ACTIONS(3432),
    [anon_sym_static] = ACTIONS(3432),
    [anon_sym_struct] = ACTIONS(3432),
    [anon_sym_trait] = ACTIONS(3432),
    [anon_sym_type] = ACTIONS(3432),
    [anon_sym_union] = ACTIONS(3432),
    [anon_sym_unsafe] = ACTIONS(3432),
    [anon_sym_use] = ACTIONS(3432),
    [anon_sym_while] = ACTIONS(3432),
    [anon_sym_extern] = ACTIONS(3432),
    [anon_sym_safe] = ACTIONS(3432),
    [anon_sym_yield] = ACTIONS(3432),
    [anon_sym_move] = ACTIONS(3432),
    [anon_sym_try] = ACTIONS(3432),
    [sym_integer_literal] = ACTIONS(3430),
    [aux_sym_string_literal_token1] = ACTIONS(3430),
    [sym_char_literal] = ACTIONS(3430),
    [anon_sym_true] = ACTIONS(3432),
    [anon_sym_false] = ACTIONS(3432),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3432),
    [sym_super] = ACTIONS(3432),
    [sym_crate] = ACTIONS(3432),
    [aux_sym_metavariable_token1] = ACTIONS(3430),
    [sym__raw_string_literal_start] = ACTIONS(3430),
    [sym_float_literal] = ACTIONS(3430),
    [sym__outer_block_doc_comment_start] = ACTIONS(3430),
    [sym__outer_line_doc_comment_start] = ACTIONS(3430),
  },
  [STATE(916)] = {
    [sym_line_comment] = STATE(916),
    [sym_block_comment] = STATE(916),
    [ts_builtin_sym_end] = ACTIONS(3434),
    [sym_identifier] = ACTIONS(3436),
    [anon_sym_SEMI] = ACTIONS(3434),
    [anon_sym_macro] = ACTIONS(3436),
    [anon_sym_LPAREN] = ACTIONS(3434),
    [anon_sym_LBRACE] = ACTIONS(3434),
    [anon_sym_RBRACE] = ACTIONS(3434),
    [anon_sym_macro_rules] = ACTIONS(3436),
    [anon_sym_BANG] = ACTIONS(3434),
    [anon_sym_LBRACK] = ACTIONS(3434),
    [anon_sym_STAR] = ACTIONS(3434),
    [anon_sym_u8] = ACTIONS(3436),
    [anon_sym_i8] = ACTIONS(3436),
    [anon_sym_u16] = ACTIONS(3436),
    [anon_sym_i16] = ACTIONS(3436),
    [anon_sym_u32] = ACTIONS(3436),
    [anon_sym_i32] = ACTIONS(3436),
    [anon_sym_u64] = ACTIONS(3436),
    [anon_sym_i64] = ACTIONS(3436),
    [anon_sym_u128] = ACTIONS(3436),
    [anon_sym_i128] = ACTIONS(3436),
    [anon_sym_isize] = ACTIONS(3436),
    [anon_sym_usize] = ACTIONS(3436),
    [anon_sym_f32] = ACTIONS(3436),
    [anon_sym_f64] = ACTIONS(3436),
    [anon_sym_bool] = ACTIONS(3436),
    [anon_sym_str] = ACTIONS(3436),
    [anon_sym_char] = ACTIONS(3436),
    [anon_sym_DASH] = ACTIONS(3434),
    [anon_sym_AMP] = ACTIONS(3434),
    [anon_sym_PIPE] = ACTIONS(3434),
    [anon_sym_LT] = ACTIONS(3434),
    [anon_sym_DOT_DOT] = ACTIONS(3434),
    [anon_sym_COLON_COLON] = ACTIONS(3434),
    [anon_sym_POUND] = ACTIONS(3434),
    [anon_sym_SQUOTE] = ACTIONS(3436),
    [anon_sym_async] = ACTIONS(3436),
    [anon_sym_auto] = ACTIONS(3436),
    [anon_sym_await] = ACTIONS(3436),
    [anon_sym_become] = ACTIONS(3436),
    [anon_sym_break] = ACTIONS(3436),
    [anon_sym_const] = ACTIONS(3436),
    [anon_sym_continue] = ACTIONS(3436),
    [anon_sym_default] = ACTIONS(3436),
    [anon_sym_enum] = ACTIONS(3436),
    [anon_sym_fn] = ACTIONS(3436),
    [anon_sym_for] = ACTIONS(3436),
    [anon_sym_gen] = ACTIONS(3436),
    [anon_sym_if] = ACTIONS(3436),
    [anon_sym_impl] = ACTIONS(3436),
    [anon_sym_let] = ACTIONS(3436),
    [anon_sym_loop] = ACTIONS(3436),
    [anon_sym_match] = ACTIONS(3436),
    [anon_sym_mod] = ACTIONS(3436),
    [anon_sym_pub] = ACTIONS(3436),
    [anon_sym_return] = ACTIONS(3436),
    [anon_sym_raw] = ACTIONS(3436),
    [anon_sym_static] = ACTIONS(3436),
    [anon_sym_struct] = ACTIONS(3436),
    [anon_sym_trait] = ACTIONS(3436),
    [anon_sym_type] = ACTIONS(3436),
    [anon_sym_union] = ACTIONS(3436),
    [anon_sym_unsafe] = ACTIONS(3436),
    [anon_sym_use] = ACTIONS(3436),
    [anon_sym_while] = ACTIONS(3436),
    [anon_sym_extern] = ACTIONS(3436),
    [anon_sym_safe] = ACTIONS(3436),
    [anon_sym_yield] = ACTIONS(3436),
    [anon_sym_move] = ACTIONS(3436),
    [anon_sym_try] = ACTIONS(3436),
    [sym_integer_literal] = ACTIONS(3434),
    [aux_sym_string_literal_token1] = ACTIONS(3434),
    [sym_char_literal] = ACTIONS(3434),
    [anon_sym_true] = ACTIONS(3436),
    [anon_sym_false] = ACTIONS(3436),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3436),
    [sym_super] = ACTIONS(3436),
    [sym_crate] = ACTIONS(3436),
    [aux_sym_metavariable_token1] = ACTIONS(3434),
    [sym__raw_string_literal_start] = ACTIONS(3434),
    [sym_float_literal] = ACTIONS(3434),
    [sym__outer_block_doc_comment_start] = ACTIONS(3434),
    [sym__outer_line_doc_comment_start] = ACTIONS(3434),
  },
  [STATE(917)] = {
    [sym_line_comment] = STATE(917),
    [sym_block_comment] = STATE(917),
    [ts_builtin_sym_end] = ACTIONS(3438),
    [sym_identifier] = ACTIONS(3440),
    [anon_sym_SEMI] = ACTIONS(3438),
    [anon_sym_macro] = ACTIONS(3440),
    [anon_sym_LPAREN] = ACTIONS(3438),
    [anon_sym_LBRACE] = ACTIONS(3438),
    [anon_sym_RBRACE] = ACTIONS(3438),
    [anon_sym_macro_rules] = ACTIONS(3440),
    [anon_sym_BANG] = ACTIONS(3438),
    [anon_sym_LBRACK] = ACTIONS(3438),
    [anon_sym_STAR] = ACTIONS(3438),
    [anon_sym_u8] = ACTIONS(3440),
    [anon_sym_i8] = ACTIONS(3440),
    [anon_sym_u16] = ACTIONS(3440),
    [anon_sym_i16] = ACTIONS(3440),
    [anon_sym_u32] = ACTIONS(3440),
    [anon_sym_i32] = ACTIONS(3440),
    [anon_sym_u64] = ACTIONS(3440),
    [anon_sym_i64] = ACTIONS(3440),
    [anon_sym_u128] = ACTIONS(3440),
    [anon_sym_i128] = ACTIONS(3440),
    [anon_sym_isize] = ACTIONS(3440),
    [anon_sym_usize] = ACTIONS(3440),
    [anon_sym_f32] = ACTIONS(3440),
    [anon_sym_f64] = ACTIONS(3440),
    [anon_sym_bool] = ACTIONS(3440),
    [anon_sym_str] = ACTIONS(3440),
    [anon_sym_char] = ACTIONS(3440),
    [anon_sym_DASH] = ACTIONS(3438),
    [anon_sym_AMP] = ACTIONS(3438),
    [anon_sym_PIPE] = ACTIONS(3438),
    [anon_sym_LT] = ACTIONS(3438),
    [anon_sym_DOT_DOT] = ACTIONS(3438),
    [anon_sym_COLON_COLON] = ACTIONS(3438),
    [anon_sym_POUND] = ACTIONS(3438),
    [anon_sym_SQUOTE] = ACTIONS(3440),
    [anon_sym_async] = ACTIONS(3440),
    [anon_sym_auto] = ACTIONS(3440),
    [anon_sym_await] = ACTIONS(3440),
    [anon_sym_become] = ACTIONS(3440),
    [anon_sym_break] = ACTIONS(3440),
    [anon_sym_const] = ACTIONS(3440),
    [anon_sym_continue] = ACTIONS(3440),
    [anon_sym_default] = ACTIONS(3440),
    [anon_sym_enum] = ACTIONS(3440),
    [anon_sym_fn] = ACTIONS(3440),
    [anon_sym_for] = ACTIONS(3440),
    [anon_sym_gen] = ACTIONS(3440),
    [anon_sym_if] = ACTIONS(3440),
    [anon_sym_impl] = ACTIONS(3440),
    [anon_sym_let] = ACTIONS(3440),
    [anon_sym_loop] = ACTIONS(3440),
    [anon_sym_match] = ACTIONS(3440),
    [anon_sym_mod] = ACTIONS(3440),
    [anon_sym_pub] = ACTIONS(3440),
    [anon_sym_return] = ACTIONS(3440),
    [anon_sym_raw] = ACTIONS(3440),
    [anon_sym_static] = ACTIONS(3440),
    [anon_sym_struct] = ACTIONS(3440),
    [anon_sym_trait] = ACTIONS(3440),
    [anon_sym_type] = ACTIONS(3440),
    [anon_sym_union] = ACTIONS(3440),
    [anon_sym_unsafe] = ACTIONS(3440),
    [anon_sym_use] = ACTIONS(3440),
    [anon_sym_while] = ACTIONS(3440),
    [anon_sym_extern] = ACTIONS(3440),
    [anon_sym_safe] = ACTIONS(3440),
    [anon_sym_yield] = ACTIONS(3440),
    [anon_sym_move] = ACTIONS(3440),
    [anon_sym_try] = ACTIONS(3440),
    [sym_integer_literal] = ACTIONS(3438),
    [aux_sym_string_literal_token1] = ACTIONS(3438),
    [sym_char_literal] = ACTIONS(3438),
    [anon_sym_true] = ACTIONS(3440),
    [anon_sym_false] = ACTIONS(3440),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3440),
    [sym_super] = ACTIONS(3440),
    [sym_crate] = ACTIONS(3440),
    [aux_sym_metavariable_token1] = ACTIONS(3438),
    [sym__raw_string_literal_start] = ACTIONS(3438),
    [sym_float_literal] = ACTIONS(3438),
    [sym__outer_block_doc_comment_start] = ACTIONS(3438),
    [sym__outer_line_doc_comment_start] = ACTIONS(3438),
  },
  [STATE(918)] = {
    [sym_line_comment] = STATE(918),
    [sym_block_comment] = STATE(918),
    [ts_builtin_sym_end] = ACTIONS(3442),
    [sym_identifier] = ACTIONS(3444),
    [anon_sym_SEMI] = ACTIONS(3442),
    [anon_sym_macro] = ACTIONS(3444),
    [anon_sym_LPAREN] = ACTIONS(3442),
    [anon_sym_LBRACE] = ACTIONS(3442),
    [anon_sym_RBRACE] = ACTIONS(3442),
    [anon_sym_macro_rules] = ACTIONS(3444),
    [anon_sym_BANG] = ACTIONS(3442),
    [anon_sym_LBRACK] = ACTIONS(3442),
    [anon_sym_STAR] = ACTIONS(3442),
    [anon_sym_u8] = ACTIONS(3444),
    [anon_sym_i8] = ACTIONS(3444),
    [anon_sym_u16] = ACTIONS(3444),
    [anon_sym_i16] = ACTIONS(3444),
    [anon_sym_u32] = ACTIONS(3444),
    [anon_sym_i32] = ACTIONS(3444),
    [anon_sym_u64] = ACTIONS(3444),
    [anon_sym_i64] = ACTIONS(3444),
    [anon_sym_u128] = ACTIONS(3444),
    [anon_sym_i128] = ACTIONS(3444),
    [anon_sym_isize] = ACTIONS(3444),
    [anon_sym_usize] = ACTIONS(3444),
    [anon_sym_f32] = ACTIONS(3444),
    [anon_sym_f64] = ACTIONS(3444),
    [anon_sym_bool] = ACTIONS(3444),
    [anon_sym_str] = ACTIONS(3444),
    [anon_sym_char] = ACTIONS(3444),
    [anon_sym_DASH] = ACTIONS(3442),
    [anon_sym_AMP] = ACTIONS(3442),
    [anon_sym_PIPE] = ACTIONS(3442),
    [anon_sym_LT] = ACTIONS(3442),
    [anon_sym_DOT_DOT] = ACTIONS(3442),
    [anon_sym_COLON_COLON] = ACTIONS(3442),
    [anon_sym_POUND] = ACTIONS(3442),
    [anon_sym_SQUOTE] = ACTIONS(3444),
    [anon_sym_async] = ACTIONS(3444),
    [anon_sym_auto] = ACTIONS(3444),
    [anon_sym_await] = ACTIONS(3444),
    [anon_sym_become] = ACTIONS(3444),
    [anon_sym_break] = ACTIONS(3444),
    [anon_sym_const] = ACTIONS(3444),
    [anon_sym_continue] = ACTIONS(3444),
    [anon_sym_default] = ACTIONS(3444),
    [anon_sym_enum] = ACTIONS(3444),
    [anon_sym_fn] = ACTIONS(3444),
    [anon_sym_for] = ACTIONS(3444),
    [anon_sym_gen] = ACTIONS(3444),
    [anon_sym_if] = ACTIONS(3444),
    [anon_sym_impl] = ACTIONS(3444),
    [anon_sym_let] = ACTIONS(3444),
    [anon_sym_loop] = ACTIONS(3444),
    [anon_sym_match] = ACTIONS(3444),
    [anon_sym_mod] = ACTIONS(3444),
    [anon_sym_pub] = ACTIONS(3444),
    [anon_sym_return] = ACTIONS(3444),
    [anon_sym_raw] = ACTIONS(3444),
    [anon_sym_static] = ACTIONS(3444),
    [anon_sym_struct] = ACTIONS(3444),
    [anon_sym_trait] = ACTIONS(3444),
    [anon_sym_type] = ACTIONS(3444),
    [anon_sym_union] = ACTIONS(3444),
    [anon_sym_unsafe] = ACTIONS(3444),
    [anon_sym_use] = ACTIONS(3444),
    [anon_sym_while] = ACTIONS(3444),
    [anon_sym_extern] = ACTIONS(3444),
    [anon_sym_safe] = ACTIONS(3444),
    [anon_sym_yield] = ACTIONS(3444),
    [anon_sym_move] = ACTIONS(3444),
    [anon_sym_try] = ACTIONS(3444),
    [sym_integer_literal] = ACTIONS(3442),
    [aux_sym_string_literal_token1] = ACTIONS(3442),
    [sym_char_literal] = ACTIONS(3442),
    [anon_sym_true] = ACTIONS(3444),
    [anon_sym_false] = ACTIONS(3444),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3444),
    [sym_super] = ACTIONS(3444),
    [sym_crate] = ACTIONS(3444),
    [aux_sym_metavariable_token1] = ACTIONS(3442),
    [sym__raw_string_literal_start] = ACTIONS(3442),
    [sym_float_literal] = ACTIONS(3442),
    [sym__outer_block_doc_comment_start] = ACTIONS(3442),
    [sym__outer_line_doc_comment_start] = ACTIONS(3442),
  },
  [STATE(919)] = {
    [sym_line_comment] = STATE(919),
    [sym_block_comment] = STATE(919),
    [ts_builtin_sym_end] = ACTIONS(3446),
    [sym_identifier] = ACTIONS(3448),
    [anon_sym_SEMI] = ACTIONS(3446),
    [anon_sym_macro] = ACTIONS(3448),
    [anon_sym_LPAREN] = ACTIONS(3446),
    [anon_sym_LBRACE] = ACTIONS(3446),
    [anon_sym_RBRACE] = ACTIONS(3446),
    [anon_sym_macro_rules] = ACTIONS(3448),
    [anon_sym_BANG] = ACTIONS(3446),
    [anon_sym_LBRACK] = ACTIONS(3446),
    [anon_sym_STAR] = ACTIONS(3446),
    [anon_sym_u8] = ACTIONS(3448),
    [anon_sym_i8] = ACTIONS(3448),
    [anon_sym_u16] = ACTIONS(3448),
    [anon_sym_i16] = ACTIONS(3448),
    [anon_sym_u32] = ACTIONS(3448),
    [anon_sym_i32] = ACTIONS(3448),
    [anon_sym_u64] = ACTIONS(3448),
    [anon_sym_i64] = ACTIONS(3448),
    [anon_sym_u128] = ACTIONS(3448),
    [anon_sym_i128] = ACTIONS(3448),
    [anon_sym_isize] = ACTIONS(3448),
    [anon_sym_usize] = ACTIONS(3448),
    [anon_sym_f32] = ACTIONS(3448),
    [anon_sym_f64] = ACTIONS(3448),
    [anon_sym_bool] = ACTIONS(3448),
    [anon_sym_str] = ACTIONS(3448),
    [anon_sym_char] = ACTIONS(3448),
    [anon_sym_DASH] = ACTIONS(3446),
    [anon_sym_AMP] = ACTIONS(3446),
    [anon_sym_PIPE] = ACTIONS(3446),
    [anon_sym_LT] = ACTIONS(3446),
    [anon_sym_DOT_DOT] = ACTIONS(3446),
    [anon_sym_COLON_COLON] = ACTIONS(3446),
    [anon_sym_POUND] = ACTIONS(3446),
    [anon_sym_SQUOTE] = ACTIONS(3448),
    [anon_sym_async] = ACTIONS(3448),
    [anon_sym_auto] = ACTIONS(3448),
    [anon_sym_await] = ACTIONS(3448),
    [anon_sym_become] = ACTIONS(3448),
    [anon_sym_break] = ACTIONS(3448),
    [anon_sym_const] = ACTIONS(3448),
    [anon_sym_continue] = ACTIONS(3448),
    [anon_sym_default] = ACTIONS(3448),
    [anon_sym_enum] = ACTIONS(3448),
    [anon_sym_fn] = ACTIONS(3448),
    [anon_sym_for] = ACTIONS(3448),
    [anon_sym_gen] = ACTIONS(3448),
    [anon_sym_if] = ACTIONS(3448),
    [anon_sym_impl] = ACTIONS(3448),
    [anon_sym_let] = ACTIONS(3448),
    [anon_sym_loop] = ACTIONS(3448),
    [anon_sym_match] = ACTIONS(3448),
    [anon_sym_mod] = ACTIONS(3448),
    [anon_sym_pub] = ACTIONS(3448),
    [anon_sym_return] = ACTIONS(3448),
    [anon_sym_raw] = ACTIONS(3448),
    [anon_sym_static] = ACTIONS(3448),
    [anon_sym_struct] = ACTIONS(3448),
    [anon_sym_trait] = ACTIONS(3448),
    [anon_sym_type] = ACTIONS(3448),
    [anon_sym_union] = ACTIONS(3448),
    [anon_sym_unsafe] = ACTIONS(3448),
    [anon_sym_use] = ACTIONS(3448),
    [anon_sym_while] = ACTIONS(3448),
    [anon_sym_extern] = ACTIONS(3448),
    [anon_sym_safe] = ACTIONS(3448),
    [anon_sym_yield] = ACTIONS(3448),
    [anon_sym_move] = ACTIONS(3448),
    [anon_sym_try] = ACTIONS(3448),
    [sym_integer_literal] = ACTIONS(3446),
    [aux_sym_string_literal_token1] = ACTIONS(3446),
    [sym_char_literal] = ACTIONS(3446),
    [anon_sym_true] = ACTIONS(3448),
    [anon_sym_false] = ACTIONS(3448),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3448),
    [sym_super] = ACTIONS(3448),
    [sym_crate] = ACTIONS(3448),
    [aux_sym_metavariable_token1] = ACTIONS(3446),
    [sym__raw_string_literal_start] = ACTIONS(3446),
    [sym_float_literal] = ACTIONS(3446),
    [sym__outer_block_doc_comment_start] = ACTIONS(3446),
    [sym__outer_line_doc_comment_start] = ACTIONS(3446),
  },
  [STATE(920)] = {
    [sym_line_comment] = STATE(920),
    [sym_block_comment] = STATE(920),
    [ts_builtin_sym_end] = ACTIONS(3450),
    [sym_identifier] = ACTIONS(3452),
    [anon_sym_SEMI] = ACTIONS(3450),
    [anon_sym_macro] = ACTIONS(3452),
    [anon_sym_LPAREN] = ACTIONS(3450),
    [anon_sym_LBRACE] = ACTIONS(3450),
    [anon_sym_RBRACE] = ACTIONS(3450),
    [anon_sym_macro_rules] = ACTIONS(3452),
    [anon_sym_BANG] = ACTIONS(3450),
    [anon_sym_LBRACK] = ACTIONS(3450),
    [anon_sym_STAR] = ACTIONS(3450),
    [anon_sym_u8] = ACTIONS(3452),
    [anon_sym_i8] = ACTIONS(3452),
    [anon_sym_u16] = ACTIONS(3452),
    [anon_sym_i16] = ACTIONS(3452),
    [anon_sym_u32] = ACTIONS(3452),
    [anon_sym_i32] = ACTIONS(3452),
    [anon_sym_u64] = ACTIONS(3452),
    [anon_sym_i64] = ACTIONS(3452),
    [anon_sym_u128] = ACTIONS(3452),
    [anon_sym_i128] = ACTIONS(3452),
    [anon_sym_isize] = ACTIONS(3452),
    [anon_sym_usize] = ACTIONS(3452),
    [anon_sym_f32] = ACTIONS(3452),
    [anon_sym_f64] = ACTIONS(3452),
    [anon_sym_bool] = ACTIONS(3452),
    [anon_sym_str] = ACTIONS(3452),
    [anon_sym_char] = ACTIONS(3452),
    [anon_sym_DASH] = ACTIONS(3450),
    [anon_sym_AMP] = ACTIONS(3450),
    [anon_sym_PIPE] = ACTIONS(3450),
    [anon_sym_LT] = ACTIONS(3450),
    [anon_sym_DOT_DOT] = ACTIONS(3450),
    [anon_sym_COLON_COLON] = ACTIONS(3450),
    [anon_sym_POUND] = ACTIONS(3450),
    [anon_sym_SQUOTE] = ACTIONS(3452),
    [anon_sym_async] = ACTIONS(3452),
    [anon_sym_auto] = ACTIONS(3452),
    [anon_sym_await] = ACTIONS(3452),
    [anon_sym_become] = ACTIONS(3452),
    [anon_sym_break] = ACTIONS(3452),
    [anon_sym_const] = ACTIONS(3452),
    [anon_sym_continue] = ACTIONS(3452),
    [anon_sym_default] = ACTIONS(3452),
    [anon_sym_enum] = ACTIONS(3452),
    [anon_sym_fn] = ACTIONS(3452),
    [anon_sym_for] = ACTIONS(3452),
    [anon_sym_gen] = ACTIONS(3452),
    [anon_sym_if] = ACTIONS(3452),
    [anon_sym_impl] = ACTIONS(3452),
    [anon_sym_let] = ACTIONS(3452),
    [anon_sym_loop] = ACTIONS(3452),
    [anon_sym_match] = ACTIONS(3452),
    [anon_sym_mod] = ACTIONS(3452),
    [anon_sym_pub] = ACTIONS(3452),
    [anon_sym_return] = ACTIONS(3452),
    [anon_sym_raw] = ACTIONS(3452),
    [anon_sym_static] = ACTIONS(3452),
    [anon_sym_struct] = ACTIONS(3452),
    [anon_sym_trait] = ACTIONS(3452),
    [anon_sym_type] = ACTIONS(3452),
    [anon_sym_union] = ACTIONS(3452),
    [anon_sym_unsafe] = ACTIONS(3452),
    [anon_sym_use] = ACTIONS(3452),
    [anon_sym_while] = ACTIONS(3452),
    [anon_sym_extern] = ACTIONS(3452),
    [anon_sym_safe] = ACTIONS(3452),
    [anon_sym_yield] = ACTIONS(3452),
    [anon_sym_move] = ACTIONS(3452),
    [anon_sym_try] = ACTIONS(3452),
    [sym_integer_literal] = ACTIONS(3450),
    [aux_sym_string_literal_token1] = ACTIONS(3450),
    [sym_char_literal] = ACTIONS(3450),
    [anon_sym_true] = ACTIONS(3452),
    [anon_sym_false] = ACTIONS(3452),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3452),
    [sym_super] = ACTIONS(3452),
    [sym_crate] = ACTIONS(3452),
    [aux_sym_metavariable_token1] = ACTIONS(3450),
    [sym__raw_string_literal_start] = ACTIONS(3450),
    [sym_float_literal] = ACTIONS(3450),
    [sym__outer_block_doc_comment_start] = ACTIONS(3450),
    [sym__outer_line_doc_comment_start] = ACTIONS(3450),
  },
  [STATE(921)] = {
    [sym_line_comment] = STATE(921),
    [sym_block_comment] = STATE(921),
    [ts_builtin_sym_end] = ACTIONS(3454),
    [sym_identifier] = ACTIONS(3456),
    [anon_sym_SEMI] = ACTIONS(3454),
    [anon_sym_macro] = ACTIONS(3456),
    [anon_sym_LPAREN] = ACTIONS(3454),
    [anon_sym_LBRACE] = ACTIONS(3454),
    [anon_sym_RBRACE] = ACTIONS(3454),
    [anon_sym_macro_rules] = ACTIONS(3456),
    [anon_sym_BANG] = ACTIONS(3454),
    [anon_sym_LBRACK] = ACTIONS(3454),
    [anon_sym_STAR] = ACTIONS(3454),
    [anon_sym_u8] = ACTIONS(3456),
    [anon_sym_i8] = ACTIONS(3456),
    [anon_sym_u16] = ACTIONS(3456),
    [anon_sym_i16] = ACTIONS(3456),
    [anon_sym_u32] = ACTIONS(3456),
    [anon_sym_i32] = ACTIONS(3456),
    [anon_sym_u64] = ACTIONS(3456),
    [anon_sym_i64] = ACTIONS(3456),
    [anon_sym_u128] = ACTIONS(3456),
    [anon_sym_i128] = ACTIONS(3456),
    [anon_sym_isize] = ACTIONS(3456),
    [anon_sym_usize] = ACTIONS(3456),
    [anon_sym_f32] = ACTIONS(3456),
    [anon_sym_f64] = ACTIONS(3456),
    [anon_sym_bool] = ACTIONS(3456),
    [anon_sym_str] = ACTIONS(3456),
    [anon_sym_char] = ACTIONS(3456),
    [anon_sym_DASH] = ACTIONS(3454),
    [anon_sym_AMP] = ACTIONS(3454),
    [anon_sym_PIPE] = ACTIONS(3454),
    [anon_sym_LT] = ACTIONS(3454),
    [anon_sym_DOT_DOT] = ACTIONS(3454),
    [anon_sym_COLON_COLON] = ACTIONS(3454),
    [anon_sym_POUND] = ACTIONS(3454),
    [anon_sym_SQUOTE] = ACTIONS(3456),
    [anon_sym_async] = ACTIONS(3456),
    [anon_sym_auto] = ACTIONS(3456),
    [anon_sym_await] = ACTIONS(3456),
    [anon_sym_become] = ACTIONS(3456),
    [anon_sym_break] = ACTIONS(3456),
    [anon_sym_const] = ACTIONS(3456),
    [anon_sym_continue] = ACTIONS(3456),
    [anon_sym_default] = ACTIONS(3456),
    [anon_sym_enum] = ACTIONS(3456),
    [anon_sym_fn] = ACTIONS(3456),
    [anon_sym_for] = ACTIONS(3456),
    [anon_sym_gen] = ACTIONS(3456),
    [anon_sym_if] = ACTIONS(3456),
    [anon_sym_impl] = ACTIONS(3456),
    [anon_sym_let] = ACTIONS(3456),
    [anon_sym_loop] = ACTIONS(3456),
    [anon_sym_match] = ACTIONS(3456),
    [anon_sym_mod] = ACTIONS(3456),
    [anon_sym_pub] = ACTIONS(3456),
    [anon_sym_return] = ACTIONS(3456),
    [anon_sym_raw] = ACTIONS(3456),
    [anon_sym_static] = ACTIONS(3456),
    [anon_sym_struct] = ACTIONS(3456),
    [anon_sym_trait] = ACTIONS(3456),
    [anon_sym_type] = ACTIONS(3456),
    [anon_sym_union] = ACTIONS(3456),
    [anon_sym_unsafe] = ACTIONS(3456),
    [anon_sym_use] = ACTIONS(3456),
    [anon_sym_while] = ACTIONS(3456),
    [anon_sym_extern] = ACTIONS(3456),
    [anon_sym_safe] = ACTIONS(3456),
    [anon_sym_yield] = ACTIONS(3456),
    [anon_sym_move] = ACTIONS(3456),
    [anon_sym_try] = ACTIONS(3456),
    [sym_integer_literal] = ACTIONS(3454),
    [aux_sym_string_literal_token1] = ACTIONS(3454),
    [sym_char_literal] = ACTIONS(3454),
    [anon_sym_true] = ACTIONS(3456),
    [anon_sym_false] = ACTIONS(3456),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3456),
    [sym_super] = ACTIONS(3456),
    [sym_crate] = ACTIONS(3456),
    [aux_sym_metavariable_token1] = ACTIONS(3454),
    [sym__raw_string_literal_start] = ACTIONS(3454),
    [sym_float_literal] = ACTIONS(3454),
    [sym__outer_block_doc_comment_start] = ACTIONS(3454),
    [sym__outer_line_doc_comment_start] = ACTIONS(3454),
  },
  [STATE(922)] = {
    [sym_line_comment] = STATE(922),
    [sym_block_comment] = STATE(922),
    [ts_builtin_sym_end] = ACTIONS(3458),
    [sym_identifier] = ACTIONS(3460),
    [anon_sym_SEMI] = ACTIONS(3458),
    [anon_sym_macro] = ACTIONS(3460),
    [anon_sym_LPAREN] = ACTIONS(3458),
    [anon_sym_LBRACE] = ACTIONS(3458),
    [anon_sym_RBRACE] = ACTIONS(3458),
    [anon_sym_macro_rules] = ACTIONS(3460),
    [anon_sym_BANG] = ACTIONS(3458),
    [anon_sym_LBRACK] = ACTIONS(3458),
    [anon_sym_STAR] = ACTIONS(3458),
    [anon_sym_u8] = ACTIONS(3460),
    [anon_sym_i8] = ACTIONS(3460),
    [anon_sym_u16] = ACTIONS(3460),
    [anon_sym_i16] = ACTIONS(3460),
    [anon_sym_u32] = ACTIONS(3460),
    [anon_sym_i32] = ACTIONS(3460),
    [anon_sym_u64] = ACTIONS(3460),
    [anon_sym_i64] = ACTIONS(3460),
    [anon_sym_u128] = ACTIONS(3460),
    [anon_sym_i128] = ACTIONS(3460),
    [anon_sym_isize] = ACTIONS(3460),
    [anon_sym_usize] = ACTIONS(3460),
    [anon_sym_f32] = ACTIONS(3460),
    [anon_sym_f64] = ACTIONS(3460),
    [anon_sym_bool] = ACTIONS(3460),
    [anon_sym_str] = ACTIONS(3460),
    [anon_sym_char] = ACTIONS(3460),
    [anon_sym_DASH] = ACTIONS(3458),
    [anon_sym_AMP] = ACTIONS(3458),
    [anon_sym_PIPE] = ACTIONS(3458),
    [anon_sym_LT] = ACTIONS(3458),
    [anon_sym_DOT_DOT] = ACTIONS(3458),
    [anon_sym_COLON_COLON] = ACTIONS(3458),
    [anon_sym_POUND] = ACTIONS(3458),
    [anon_sym_SQUOTE] = ACTIONS(3460),
    [anon_sym_async] = ACTIONS(3460),
    [anon_sym_auto] = ACTIONS(3460),
    [anon_sym_await] = ACTIONS(3460),
    [anon_sym_become] = ACTIONS(3460),
    [anon_sym_break] = ACTIONS(3460),
    [anon_sym_const] = ACTIONS(3460),
    [anon_sym_continue] = ACTIONS(3460),
    [anon_sym_default] = ACTIONS(3460),
    [anon_sym_enum] = ACTIONS(3460),
    [anon_sym_fn] = ACTIONS(3460),
    [anon_sym_for] = ACTIONS(3460),
    [anon_sym_gen] = ACTIONS(3460),
    [anon_sym_if] = ACTIONS(3460),
    [anon_sym_impl] = ACTIONS(3460),
    [anon_sym_let] = ACTIONS(3460),
    [anon_sym_loop] = ACTIONS(3460),
    [anon_sym_match] = ACTIONS(3460),
    [anon_sym_mod] = ACTIONS(3460),
    [anon_sym_pub] = ACTIONS(3460),
    [anon_sym_return] = ACTIONS(3460),
    [anon_sym_raw] = ACTIONS(3460),
    [anon_sym_static] = ACTIONS(3460),
    [anon_sym_struct] = ACTIONS(3460),
    [anon_sym_trait] = ACTIONS(3460),
    [anon_sym_type] = ACTIONS(3460),
    [anon_sym_union] = ACTIONS(3460),
    [anon_sym_unsafe] = ACTIONS(3460),
    [anon_sym_use] = ACTIONS(3460),
    [anon_sym_while] = ACTIONS(3460),
    [anon_sym_extern] = ACTIONS(3460),
    [anon_sym_safe] = ACTIONS(3460),
    [anon_sym_yield] = ACTIONS(3460),
    [anon_sym_move] = ACTIONS(3460),
    [anon_sym_try] = ACTIONS(3460),
    [sym_integer_literal] = ACTIONS(3458),
    [aux_sym_string_literal_token1] = ACTIONS(3458),
    [sym_char_literal] = ACTIONS(3458),
    [anon_sym_true] = ACTIONS(3460),
    [anon_sym_false] = ACTIONS(3460),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3460),
    [sym_super] = ACTIONS(3460),
    [sym_crate] = ACTIONS(3460),
    [aux_sym_metavariable_token1] = ACTIONS(3458),
    [sym__raw_string_literal_start] = ACTIONS(3458),
    [sym_float_literal] = ACTIONS(3458),
    [sym__outer_block_doc_comment_start] = ACTIONS(3458),
    [sym__outer_line_doc_comment_start] = ACTIONS(3458),
  },
  [STATE(923)] = {
    [sym_line_comment] = STATE(923),
    [sym_block_comment] = STATE(923),
    [ts_builtin_sym_end] = ACTIONS(3462),
    [sym_identifier] = ACTIONS(3464),
    [anon_sym_SEMI] = ACTIONS(3462),
    [anon_sym_macro] = ACTIONS(3464),
    [anon_sym_LPAREN] = ACTIONS(3462),
    [anon_sym_LBRACE] = ACTIONS(3462),
    [anon_sym_RBRACE] = ACTIONS(3462),
    [anon_sym_macro_rules] = ACTIONS(3464),
    [anon_sym_BANG] = ACTIONS(3462),
    [anon_sym_LBRACK] = ACTIONS(3462),
    [anon_sym_STAR] = ACTIONS(3462),
    [anon_sym_u8] = ACTIONS(3464),
    [anon_sym_i8] = ACTIONS(3464),
    [anon_sym_u16] = ACTIONS(3464),
    [anon_sym_i16] = ACTIONS(3464),
    [anon_sym_u32] = ACTIONS(3464),
    [anon_sym_i32] = ACTIONS(3464),
    [anon_sym_u64] = ACTIONS(3464),
    [anon_sym_i64] = ACTIONS(3464),
    [anon_sym_u128] = ACTIONS(3464),
    [anon_sym_i128] = ACTIONS(3464),
    [anon_sym_isize] = ACTIONS(3464),
    [anon_sym_usize] = ACTIONS(3464),
    [anon_sym_f32] = ACTIONS(3464),
    [anon_sym_f64] = ACTIONS(3464),
    [anon_sym_bool] = ACTIONS(3464),
    [anon_sym_str] = ACTIONS(3464),
    [anon_sym_char] = ACTIONS(3464),
    [anon_sym_DASH] = ACTIONS(3462),
    [anon_sym_AMP] = ACTIONS(3462),
    [anon_sym_PIPE] = ACTIONS(3462),
    [anon_sym_LT] = ACTIONS(3462),
    [anon_sym_DOT_DOT] = ACTIONS(3462),
    [anon_sym_COLON_COLON] = ACTIONS(3462),
    [anon_sym_POUND] = ACTIONS(3462),
    [anon_sym_SQUOTE] = ACTIONS(3464),
    [anon_sym_async] = ACTIONS(3464),
    [anon_sym_auto] = ACTIONS(3464),
    [anon_sym_await] = ACTIONS(3464),
    [anon_sym_become] = ACTIONS(3464),
    [anon_sym_break] = ACTIONS(3464),
    [anon_sym_const] = ACTIONS(3464),
    [anon_sym_continue] = ACTIONS(3464),
    [anon_sym_default] = ACTIONS(3464),
    [anon_sym_enum] = ACTIONS(3464),
    [anon_sym_fn] = ACTIONS(3464),
    [anon_sym_for] = ACTIONS(3464),
    [anon_sym_gen] = ACTIONS(3464),
    [anon_sym_if] = ACTIONS(3464),
    [anon_sym_impl] = ACTIONS(3464),
    [anon_sym_let] = ACTIONS(3464),
    [anon_sym_loop] = ACTIONS(3464),
    [anon_sym_match] = ACTIONS(3464),
    [anon_sym_mod] = ACTIONS(3464),
    [anon_sym_pub] = ACTIONS(3464),
    [anon_sym_return] = ACTIONS(3464),
    [anon_sym_raw] = ACTIONS(3464),
    [anon_sym_static] = ACTIONS(3464),
    [anon_sym_struct] = ACTIONS(3464),
    [anon_sym_trait] = ACTIONS(3464),
    [anon_sym_type] = ACTIONS(3464),
    [anon_sym_union] = ACTIONS(3464),
    [anon_sym_unsafe] = ACTIONS(3464),
    [anon_sym_use] = ACTIONS(3464),
    [anon_sym_while] = ACTIONS(3464),
    [anon_sym_extern] = ACTIONS(3464),
    [anon_sym_safe] = ACTIONS(3464),
    [anon_sym_yield] = ACTIONS(3464),
    [anon_sym_move] = ACTIONS(3464),
    [anon_sym_try] = ACTIONS(3464),
    [sym_integer_literal] = ACTIONS(3462),
    [aux_sym_string_literal_token1] = ACTIONS(3462),
    [sym_char_literal] = ACTIONS(3462),
    [anon_sym_true] = ACTIONS(3464),
    [anon_sym_false] = ACTIONS(3464),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3464),
    [sym_super] = ACTIONS(3464),
    [sym_crate] = ACTIONS(3464),
    [aux_sym_metavariable_token1] = ACTIONS(3462),
    [sym__raw_string_literal_start] = ACTIONS(3462),
    [sym_float_literal] = ACTIONS(3462),
    [sym__outer_block_doc_comment_start] = ACTIONS(3462),
    [sym__outer_line_doc_comment_start] = ACTIONS(3462),
  },
  [STATE(924)] = {
    [sym_line_comment] = STATE(924),
    [sym_block_comment] = STATE(924),
    [ts_builtin_sym_end] = ACTIONS(3466),
    [sym_identifier] = ACTIONS(3468),
    [anon_sym_SEMI] = ACTIONS(3466),
    [anon_sym_macro] = ACTIONS(3468),
    [anon_sym_LPAREN] = ACTIONS(3466),
    [anon_sym_LBRACE] = ACTIONS(3466),
    [anon_sym_RBRACE] = ACTIONS(3466),
    [anon_sym_macro_rules] = ACTIONS(3468),
    [anon_sym_BANG] = ACTIONS(3466),
    [anon_sym_LBRACK] = ACTIONS(3466),
    [anon_sym_STAR] = ACTIONS(3466),
    [anon_sym_u8] = ACTIONS(3468),
    [anon_sym_i8] = ACTIONS(3468),
    [anon_sym_u16] = ACTIONS(3468),
    [anon_sym_i16] = ACTIONS(3468),
    [anon_sym_u32] = ACTIONS(3468),
    [anon_sym_i32] = ACTIONS(3468),
    [anon_sym_u64] = ACTIONS(3468),
    [anon_sym_i64] = ACTIONS(3468),
    [anon_sym_u128] = ACTIONS(3468),
    [anon_sym_i128] = ACTIONS(3468),
    [anon_sym_isize] = ACTIONS(3468),
    [anon_sym_usize] = ACTIONS(3468),
    [anon_sym_f32] = ACTIONS(3468),
    [anon_sym_f64] = ACTIONS(3468),
    [anon_sym_bool] = ACTIONS(3468),
    [anon_sym_str] = ACTIONS(3468),
    [anon_sym_char] = ACTIONS(3468),
    [anon_sym_DASH] = ACTIONS(3466),
    [anon_sym_AMP] = ACTIONS(3466),
    [anon_sym_PIPE] = ACTIONS(3466),
    [anon_sym_LT] = ACTIONS(3466),
    [anon_sym_DOT_DOT] = ACTIONS(3466),
    [anon_sym_COLON_COLON] = ACTIONS(3466),
    [anon_sym_POUND] = ACTIONS(3466),
    [anon_sym_SQUOTE] = ACTIONS(3468),
    [anon_sym_async] = ACTIONS(3468),
    [anon_sym_auto] = ACTIONS(3468),
    [anon_sym_await] = ACTIONS(3468),
    [anon_sym_become] = ACTIONS(3468),
    [anon_sym_break] = ACTIONS(3468),
    [anon_sym_const] = ACTIONS(3468),
    [anon_sym_continue] = ACTIONS(3468),
    [anon_sym_default] = ACTIONS(3468),
    [anon_sym_enum] = ACTIONS(3468),
    [anon_sym_fn] = ACTIONS(3468),
    [anon_sym_for] = ACTIONS(3468),
    [anon_sym_gen] = ACTIONS(3468),
    [anon_sym_if] = ACTIONS(3468),
    [anon_sym_impl] = ACTIONS(3468),
    [anon_sym_let] = ACTIONS(3468),
    [anon_sym_loop] = ACTIONS(3468),
    [anon_sym_match] = ACTIONS(3468),
    [anon_sym_mod] = ACTIONS(3468),
    [anon_sym_pub] = ACTIONS(3468),
    [anon_sym_return] = ACTIONS(3468),
    [anon_sym_raw] = ACTIONS(3468),
    [anon_sym_static] = ACTIONS(3468),
    [anon_sym_struct] = ACTIONS(3468),
    [anon_sym_trait] = ACTIONS(3468),
    [anon_sym_type] = ACTIONS(3468),
    [anon_sym_union] = ACTIONS(3468),
    [anon_sym_unsafe] = ACTIONS(3468),
    [anon_sym_use] = ACTIONS(3468),
    [anon_sym_while] = ACTIONS(3468),
    [anon_sym_extern] = ACTIONS(3468),
    [anon_sym_safe] = ACTIONS(3468),
    [anon_sym_yield] = ACTIONS(3468),
    [anon_sym_move] = ACTIONS(3468),
    [anon_sym_try] = ACTIONS(3468),
    [sym_integer_literal] = ACTIONS(3466),
    [aux_sym_string_literal_token1] = ACTIONS(3466),
    [sym_char_literal] = ACTIONS(3466),
    [anon_sym_true] = ACTIONS(3468),
    [anon_sym_false] = ACTIONS(3468),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3468),
    [sym_super] = ACTIONS(3468),
    [sym_crate] = ACTIONS(3468),
    [aux_sym_metavariable_token1] = ACTIONS(3466),
    [sym__raw_string_literal_start] = ACTIONS(3466),
    [sym_float_literal] = ACTIONS(3466),
    [sym__outer_block_doc_comment_start] = ACTIONS(3466),
    [sym__outer_line_doc_comment_start] = ACTIONS(3466),
  },
  [STATE(925)] = {
    [sym_line_comment] = STATE(925),
    [sym_block_comment] = STATE(925),
    [ts_builtin_sym_end] = ACTIONS(3470),
    [sym_identifier] = ACTIONS(3472),
    [anon_sym_SEMI] = ACTIONS(3470),
    [anon_sym_macro] = ACTIONS(3472),
    [anon_sym_LPAREN] = ACTIONS(3470),
    [anon_sym_LBRACE] = ACTIONS(3470),
    [anon_sym_RBRACE] = ACTIONS(3470),
    [anon_sym_macro_rules] = ACTIONS(3472),
    [anon_sym_BANG] = ACTIONS(3470),
    [anon_sym_LBRACK] = ACTIONS(3470),
    [anon_sym_STAR] = ACTIONS(3470),
    [anon_sym_u8] = ACTIONS(3472),
    [anon_sym_i8] = ACTIONS(3472),
    [anon_sym_u16] = ACTIONS(3472),
    [anon_sym_i16] = ACTIONS(3472),
    [anon_sym_u32] = ACTIONS(3472),
    [anon_sym_i32] = ACTIONS(3472),
    [anon_sym_u64] = ACTIONS(3472),
    [anon_sym_i64] = ACTIONS(3472),
    [anon_sym_u128] = ACTIONS(3472),
    [anon_sym_i128] = ACTIONS(3472),
    [anon_sym_isize] = ACTIONS(3472),
    [anon_sym_usize] = ACTIONS(3472),
    [anon_sym_f32] = ACTIONS(3472),
    [anon_sym_f64] = ACTIONS(3472),
    [anon_sym_bool] = ACTIONS(3472),
    [anon_sym_str] = ACTIONS(3472),
    [anon_sym_char] = ACTIONS(3472),
    [anon_sym_DASH] = ACTIONS(3470),
    [anon_sym_AMP] = ACTIONS(3470),
    [anon_sym_PIPE] = ACTIONS(3470),
    [anon_sym_LT] = ACTIONS(3470),
    [anon_sym_DOT_DOT] = ACTIONS(3470),
    [anon_sym_COLON_COLON] = ACTIONS(3470),
    [anon_sym_POUND] = ACTIONS(3470),
    [anon_sym_SQUOTE] = ACTIONS(3472),
    [anon_sym_async] = ACTIONS(3472),
    [anon_sym_auto] = ACTIONS(3472),
    [anon_sym_await] = ACTIONS(3472),
    [anon_sym_become] = ACTIONS(3472),
    [anon_sym_break] = ACTIONS(3472),
    [anon_sym_const] = ACTIONS(3472),
    [anon_sym_continue] = ACTIONS(3472),
    [anon_sym_default] = ACTIONS(3472),
    [anon_sym_enum] = ACTIONS(3472),
    [anon_sym_fn] = ACTIONS(3472),
    [anon_sym_for] = ACTIONS(3472),
    [anon_sym_gen] = ACTIONS(3472),
    [anon_sym_if] = ACTIONS(3472),
    [anon_sym_impl] = ACTIONS(3472),
    [anon_sym_let] = ACTIONS(3472),
    [anon_sym_loop] = ACTIONS(3472),
    [anon_sym_match] = ACTIONS(3472),
    [anon_sym_mod] = ACTIONS(3472),
    [anon_sym_pub] = ACTIONS(3472),
    [anon_sym_return] = ACTIONS(3472),
    [anon_sym_raw] = ACTIONS(3472),
    [anon_sym_static] = ACTIONS(3472),
    [anon_sym_struct] = ACTIONS(3472),
    [anon_sym_trait] = ACTIONS(3472),
    [anon_sym_type] = ACTIONS(3472),
    [anon_sym_union] = ACTIONS(3472),
    [anon_sym_unsafe] = ACTIONS(3472),
    [anon_sym_use] = ACTIONS(3472),
    [anon_sym_while] = ACTIONS(3472),
    [anon_sym_extern] = ACTIONS(3472),
    [anon_sym_safe] = ACTIONS(3472),
    [anon_sym_yield] = ACTIONS(3472),
    [anon_sym_move] = ACTIONS(3472),
    [anon_sym_try] = ACTIONS(3472),
    [sym_integer_literal] = ACTIONS(3470),
    [aux_sym_string_literal_token1] = ACTIONS(3470),
    [sym_char_literal] = ACTIONS(3470),
    [anon_sym_true] = ACTIONS(3472),
    [anon_sym_false] = ACTIONS(3472),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3472),
    [sym_super] = ACTIONS(3472),
    [sym_crate] = ACTIONS(3472),
    [aux_sym_metavariable_token1] = ACTIONS(3470),
    [sym__raw_string_literal_start] = ACTIONS(3470),
    [sym_float_literal] = ACTIONS(3470),
    [sym__outer_block_doc_comment_start] = ACTIONS(3470),
    [sym__outer_line_doc_comment_start] = ACTIONS(3470),
  },
  [STATE(926)] = {
    [sym_line_comment] = STATE(926),
    [sym_block_comment] = STATE(926),
    [ts_builtin_sym_end] = ACTIONS(3474),
    [sym_identifier] = ACTIONS(3476),
    [anon_sym_SEMI] = ACTIONS(3474),
    [anon_sym_macro] = ACTIONS(3476),
    [anon_sym_LPAREN] = ACTIONS(3474),
    [anon_sym_LBRACE] = ACTIONS(3474),
    [anon_sym_RBRACE] = ACTIONS(3474),
    [anon_sym_macro_rules] = ACTIONS(3476),
    [anon_sym_BANG] = ACTIONS(3474),
    [anon_sym_LBRACK] = ACTIONS(3474),
    [anon_sym_STAR] = ACTIONS(3474),
    [anon_sym_u8] = ACTIONS(3476),
    [anon_sym_i8] = ACTIONS(3476),
    [anon_sym_u16] = ACTIONS(3476),
    [anon_sym_i16] = ACTIONS(3476),
    [anon_sym_u32] = ACTIONS(3476),
    [anon_sym_i32] = ACTIONS(3476),
    [anon_sym_u64] = ACTIONS(3476),
    [anon_sym_i64] = ACTIONS(3476),
    [anon_sym_u128] = ACTIONS(3476),
    [anon_sym_i128] = ACTIONS(3476),
    [anon_sym_isize] = ACTIONS(3476),
    [anon_sym_usize] = ACTIONS(3476),
    [anon_sym_f32] = ACTIONS(3476),
    [anon_sym_f64] = ACTIONS(3476),
    [anon_sym_bool] = ACTIONS(3476),
    [anon_sym_str] = ACTIONS(3476),
    [anon_sym_char] = ACTIONS(3476),
    [anon_sym_DASH] = ACTIONS(3474),
    [anon_sym_AMP] = ACTIONS(3474),
    [anon_sym_PIPE] = ACTIONS(3474),
    [anon_sym_LT] = ACTIONS(3474),
    [anon_sym_DOT_DOT] = ACTIONS(3474),
    [anon_sym_COLON_COLON] = ACTIONS(3474),
    [anon_sym_POUND] = ACTIONS(3474),
    [anon_sym_SQUOTE] = ACTIONS(3476),
    [anon_sym_async] = ACTIONS(3476),
    [anon_sym_auto] = ACTIONS(3476),
    [anon_sym_await] = ACTIONS(3476),
    [anon_sym_become] = ACTIONS(3476),
    [anon_sym_break] = ACTIONS(3476),
    [anon_sym_const] = ACTIONS(3476),
    [anon_sym_continue] = ACTIONS(3476),
    [anon_sym_default] = ACTIONS(3476),
    [anon_sym_enum] = ACTIONS(3476),
    [anon_sym_fn] = ACTIONS(3476),
    [anon_sym_for] = ACTIONS(3476),
    [anon_sym_gen] = ACTIONS(3476),
    [anon_sym_if] = ACTIONS(3476),
    [anon_sym_impl] = ACTIONS(3476),
    [anon_sym_let] = ACTIONS(3476),
    [anon_sym_loop] = ACTIONS(3476),
    [anon_sym_match] = ACTIONS(3476),
    [anon_sym_mod] = ACTIONS(3476),
    [anon_sym_pub] = ACTIONS(3476),
    [anon_sym_return] = ACTIONS(3476),
    [anon_sym_raw] = ACTIONS(3476),
    [anon_sym_static] = ACTIONS(3476),
    [anon_sym_struct] = ACTIONS(3476),
    [anon_sym_trait] = ACTIONS(3476),
    [anon_sym_type] = ACTIONS(3476),
    [anon_sym_union] = ACTIONS(3476),
    [anon_sym_unsafe] = ACTIONS(3476),
    [anon_sym_use] = ACTIONS(3476),
    [anon_sym_while] = ACTIONS(3476),
    [anon_sym_extern] = ACTIONS(3476),
    [anon_sym_safe] = ACTIONS(3476),
    [anon_sym_yield] = ACTIONS(3476),
    [anon_sym_move] = ACTIONS(3476),
    [anon_sym_try] = ACTIONS(3476),
    [sym_integer_literal] = ACTIONS(3474),
    [aux_sym_string_literal_token1] = ACTIONS(3474),
    [sym_char_literal] = ACTIONS(3474),
    [anon_sym_true] = ACTIONS(3476),
    [anon_sym_false] = ACTIONS(3476),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3476),
    [sym_super] = ACTIONS(3476),
    [sym_crate] = ACTIONS(3476),
    [aux_sym_metavariable_token1] = ACTIONS(3474),
    [sym__raw_string_literal_start] = ACTIONS(3474),
    [sym_float_literal] = ACTIONS(3474),
    [sym__outer_block_doc_comment_start] = ACTIONS(3474),
    [sym__outer_line_doc_comment_start] = ACTIONS(3474),
  },
  [STATE(927)] = {
    [sym_line_comment] = STATE(927),
    [sym_block_comment] = STATE(927),
    [ts_builtin_sym_end] = ACTIONS(3478),
    [sym_identifier] = ACTIONS(3480),
    [anon_sym_SEMI] = ACTIONS(3478),
    [anon_sym_macro] = ACTIONS(3480),
    [anon_sym_LPAREN] = ACTIONS(3478),
    [anon_sym_LBRACE] = ACTIONS(3478),
    [anon_sym_RBRACE] = ACTIONS(3478),
    [anon_sym_macro_rules] = ACTIONS(3480),
    [anon_sym_BANG] = ACTIONS(3478),
    [anon_sym_LBRACK] = ACTIONS(3478),
    [anon_sym_STAR] = ACTIONS(3478),
    [anon_sym_u8] = ACTIONS(3480),
    [anon_sym_i8] = ACTIONS(3480),
    [anon_sym_u16] = ACTIONS(3480),
    [anon_sym_i16] = ACTIONS(3480),
    [anon_sym_u32] = ACTIONS(3480),
    [anon_sym_i32] = ACTIONS(3480),
    [anon_sym_u64] = ACTIONS(3480),
    [anon_sym_i64] = ACTIONS(3480),
    [anon_sym_u128] = ACTIONS(3480),
    [anon_sym_i128] = ACTIONS(3480),
    [anon_sym_isize] = ACTIONS(3480),
    [anon_sym_usize] = ACTIONS(3480),
    [anon_sym_f32] = ACTIONS(3480),
    [anon_sym_f64] = ACTIONS(3480),
    [anon_sym_bool] = ACTIONS(3480),
    [anon_sym_str] = ACTIONS(3480),
    [anon_sym_char] = ACTIONS(3480),
    [anon_sym_DASH] = ACTIONS(3478),
    [anon_sym_AMP] = ACTIONS(3478),
    [anon_sym_PIPE] = ACTIONS(3478),
    [anon_sym_LT] = ACTIONS(3478),
    [anon_sym_DOT_DOT] = ACTIONS(3478),
    [anon_sym_COLON_COLON] = ACTIONS(3478),
    [anon_sym_POUND] = ACTIONS(3478),
    [anon_sym_SQUOTE] = ACTIONS(3480),
    [anon_sym_async] = ACTIONS(3480),
    [anon_sym_auto] = ACTIONS(3480),
    [anon_sym_await] = ACTIONS(3480),
    [anon_sym_become] = ACTIONS(3480),
    [anon_sym_break] = ACTIONS(3480),
    [anon_sym_const] = ACTIONS(3480),
    [anon_sym_continue] = ACTIONS(3480),
    [anon_sym_default] = ACTIONS(3480),
    [anon_sym_enum] = ACTIONS(3480),
    [anon_sym_fn] = ACTIONS(3480),
    [anon_sym_for] = ACTIONS(3480),
    [anon_sym_gen] = ACTIONS(3480),
    [anon_sym_if] = ACTIONS(3480),
    [anon_sym_impl] = ACTIONS(3480),
    [anon_sym_let] = ACTIONS(3480),
    [anon_sym_loop] = ACTIONS(3480),
    [anon_sym_match] = ACTIONS(3480),
    [anon_sym_mod] = ACTIONS(3480),
    [anon_sym_pub] = ACTIONS(3480),
    [anon_sym_return] = ACTIONS(3480),
    [anon_sym_raw] = ACTIONS(3480),
    [anon_sym_static] = ACTIONS(3480),
    [anon_sym_struct] = ACTIONS(3480),
    [anon_sym_trait] = ACTIONS(3480),
    [anon_sym_type] = ACTIONS(3480),
    [anon_sym_union] = ACTIONS(3480),
    [anon_sym_unsafe] = ACTIONS(3480),
    [anon_sym_use] = ACTIONS(3480),
    [anon_sym_while] = ACTIONS(3480),
    [anon_sym_extern] = ACTIONS(3480),
    [anon_sym_safe] = ACTIONS(3480),
    [anon_sym_yield] = ACTIONS(3480),
    [anon_sym_move] = ACTIONS(3480),
    [anon_sym_try] = ACTIONS(3480),
    [sym_integer_literal] = ACTIONS(3478),
    [aux_sym_string_literal_token1] = ACTIONS(3478),
    [sym_char_literal] = ACTIONS(3478),
    [anon_sym_true] = ACTIONS(3480),
    [anon_sym_false] = ACTIONS(3480),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3480),
    [sym_super] = ACTIONS(3480),
    [sym_crate] = ACTIONS(3480),
    [aux_sym_metavariable_token1] = ACTIONS(3478),
    [sym__raw_string_literal_start] = ACTIONS(3478),
    [sym_float_literal] = ACTIONS(3478),
    [sym__outer_block_doc_comment_start] = ACTIONS(3478),
    [sym__outer_line_doc_comment_start] = ACTIONS(3478),
  },
  [STATE(928)] = {
    [sym_line_comment] = STATE(928),
    [sym_block_comment] = STATE(928),
    [ts_builtin_sym_end] = ACTIONS(3482),
    [sym_identifier] = ACTIONS(3484),
    [anon_sym_SEMI] = ACTIONS(3482),
    [anon_sym_macro] = ACTIONS(3484),
    [anon_sym_LPAREN] = ACTIONS(3482),
    [anon_sym_LBRACE] = ACTIONS(3482),
    [anon_sym_RBRACE] = ACTIONS(3482),
    [anon_sym_macro_rules] = ACTIONS(3484),
    [anon_sym_BANG] = ACTIONS(3482),
    [anon_sym_LBRACK] = ACTIONS(3482),
    [anon_sym_STAR] = ACTIONS(3482),
    [anon_sym_u8] = ACTIONS(3484),
    [anon_sym_i8] = ACTIONS(3484),
    [anon_sym_u16] = ACTIONS(3484),
    [anon_sym_i16] = ACTIONS(3484),
    [anon_sym_u32] = ACTIONS(3484),
    [anon_sym_i32] = ACTIONS(3484),
    [anon_sym_u64] = ACTIONS(3484),
    [anon_sym_i64] = ACTIONS(3484),
    [anon_sym_u128] = ACTIONS(3484),
    [anon_sym_i128] = ACTIONS(3484),
    [anon_sym_isize] = ACTIONS(3484),
    [anon_sym_usize] = ACTIONS(3484),
    [anon_sym_f32] = ACTIONS(3484),
    [anon_sym_f64] = ACTIONS(3484),
    [anon_sym_bool] = ACTIONS(3484),
    [anon_sym_str] = ACTIONS(3484),
    [anon_sym_char] = ACTIONS(3484),
    [anon_sym_DASH] = ACTIONS(3482),
    [anon_sym_AMP] = ACTIONS(3482),
    [anon_sym_PIPE] = ACTIONS(3482),
    [anon_sym_LT] = ACTIONS(3482),
    [anon_sym_DOT_DOT] = ACTIONS(3482),
    [anon_sym_COLON_COLON] = ACTIONS(3482),
    [anon_sym_POUND] = ACTIONS(3482),
    [anon_sym_SQUOTE] = ACTIONS(3484),
    [anon_sym_async] = ACTIONS(3484),
    [anon_sym_auto] = ACTIONS(3484),
    [anon_sym_await] = ACTIONS(3484),
    [anon_sym_become] = ACTIONS(3484),
    [anon_sym_break] = ACTIONS(3484),
    [anon_sym_const] = ACTIONS(3484),
    [anon_sym_continue] = ACTIONS(3484),
    [anon_sym_default] = ACTIONS(3484),
    [anon_sym_enum] = ACTIONS(3484),
    [anon_sym_fn] = ACTIONS(3484),
    [anon_sym_for] = ACTIONS(3484),
    [anon_sym_gen] = ACTIONS(3484),
    [anon_sym_if] = ACTIONS(3484),
    [anon_sym_impl] = ACTIONS(3484),
    [anon_sym_let] = ACTIONS(3484),
    [anon_sym_loop] = ACTIONS(3484),
    [anon_sym_match] = ACTIONS(3484),
    [anon_sym_mod] = ACTIONS(3484),
    [anon_sym_pub] = ACTIONS(3484),
    [anon_sym_return] = ACTIONS(3484),
    [anon_sym_raw] = ACTIONS(3484),
    [anon_sym_static] = ACTIONS(3484),
    [anon_sym_struct] = ACTIONS(3484),
    [anon_sym_trait] = ACTIONS(3484),
    [anon_sym_type] = ACTIONS(3484),
    [anon_sym_union] = ACTIONS(3484),
    [anon_sym_unsafe] = ACTIONS(3484),
    [anon_sym_use] = ACTIONS(3484),
    [anon_sym_while] = ACTIONS(3484),
    [anon_sym_extern] = ACTIONS(3484),
    [anon_sym_safe] = ACTIONS(3484),
    [anon_sym_yield] = ACTIONS(3484),
    [anon_sym_move] = ACTIONS(3484),
    [anon_sym_try] = ACTIONS(3484),
    [sym_integer_literal] = ACTIONS(3482),
    [aux_sym_string_literal_token1] = ACTIONS(3482),
    [sym_char_literal] = ACTIONS(3482),
    [anon_sym_true] = ACTIONS(3484),
    [anon_sym_false] = ACTIONS(3484),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3484),
    [sym_super] = ACTIONS(3484),
    [sym_crate] = ACTIONS(3484),
    [aux_sym_metavariable_token1] = ACTIONS(3482),
    [sym__raw_string_literal_start] = ACTIONS(3482),
    [sym_float_literal] = ACTIONS(3482),
    [sym__outer_block_doc_comment_start] = ACTIONS(3482),
    [sym__outer_line_doc_comment_start] = ACTIONS(3482),
  },
  [STATE(929)] = {
    [sym_line_comment] = STATE(929),
    [sym_block_comment] = STATE(929),
    [ts_builtin_sym_end] = ACTIONS(3486),
    [sym_identifier] = ACTIONS(3488),
    [anon_sym_SEMI] = ACTIONS(3486),
    [anon_sym_macro] = ACTIONS(3488),
    [anon_sym_LPAREN] = ACTIONS(3486),
    [anon_sym_LBRACE] = ACTIONS(3486),
    [anon_sym_RBRACE] = ACTIONS(3486),
    [anon_sym_macro_rules] = ACTIONS(3488),
    [anon_sym_BANG] = ACTIONS(3486),
    [anon_sym_LBRACK] = ACTIONS(3486),
    [anon_sym_STAR] = ACTIONS(3486),
    [anon_sym_u8] = ACTIONS(3488),
    [anon_sym_i8] = ACTIONS(3488),
    [anon_sym_u16] = ACTIONS(3488),
    [anon_sym_i16] = ACTIONS(3488),
    [anon_sym_u32] = ACTIONS(3488),
    [anon_sym_i32] = ACTIONS(3488),
    [anon_sym_u64] = ACTIONS(3488),
    [anon_sym_i64] = ACTIONS(3488),
    [anon_sym_u128] = ACTIONS(3488),
    [anon_sym_i128] = ACTIONS(3488),
    [anon_sym_isize] = ACTIONS(3488),
    [anon_sym_usize] = ACTIONS(3488),
    [anon_sym_f32] = ACTIONS(3488),
    [anon_sym_f64] = ACTIONS(3488),
    [anon_sym_bool] = ACTIONS(3488),
    [anon_sym_str] = ACTIONS(3488),
    [anon_sym_char] = ACTIONS(3488),
    [anon_sym_DASH] = ACTIONS(3486),
    [anon_sym_AMP] = ACTIONS(3486),
    [anon_sym_PIPE] = ACTIONS(3486),
    [anon_sym_LT] = ACTIONS(3486),
    [anon_sym_DOT_DOT] = ACTIONS(3486),
    [anon_sym_COLON_COLON] = ACTIONS(3486),
    [anon_sym_POUND] = ACTIONS(3486),
    [anon_sym_SQUOTE] = ACTIONS(3488),
    [anon_sym_async] = ACTIONS(3488),
    [anon_sym_auto] = ACTIONS(3488),
    [anon_sym_await] = ACTIONS(3488),
    [anon_sym_become] = ACTIONS(3488),
    [anon_sym_break] = ACTIONS(3488),
    [anon_sym_const] = ACTIONS(3488),
    [anon_sym_continue] = ACTIONS(3488),
    [anon_sym_default] = ACTIONS(3488),
    [anon_sym_enum] = ACTIONS(3488),
    [anon_sym_fn] = ACTIONS(3488),
    [anon_sym_for] = ACTIONS(3488),
    [anon_sym_gen] = ACTIONS(3488),
    [anon_sym_if] = ACTIONS(3488),
    [anon_sym_impl] = ACTIONS(3488),
    [anon_sym_let] = ACTIONS(3488),
    [anon_sym_loop] = ACTIONS(3488),
    [anon_sym_match] = ACTIONS(3488),
    [anon_sym_mod] = ACTIONS(3488),
    [anon_sym_pub] = ACTIONS(3488),
    [anon_sym_return] = ACTIONS(3488),
    [anon_sym_raw] = ACTIONS(3488),
    [anon_sym_static] = ACTIONS(3488),
    [anon_sym_struct] = ACTIONS(3488),
    [anon_sym_trait] = ACTIONS(3488),
    [anon_sym_type] = ACTIONS(3488),
    [anon_sym_union] = ACTIONS(3488),
    [anon_sym_unsafe] = ACTIONS(3488),
    [anon_sym_use] = ACTIONS(3488),
    [anon_sym_while] = ACTIONS(3488),
    [anon_sym_extern] = ACTIONS(3488),
    [anon_sym_safe] = ACTIONS(3488),
    [anon_sym_yield] = ACTIONS(3488),
    [anon_sym_move] = ACTIONS(3488),
    [anon_sym_try] = ACTIONS(3488),
    [sym_integer_literal] = ACTIONS(3486),
    [aux_sym_string_literal_token1] = ACTIONS(3486),
    [sym_char_literal] = ACTIONS(3486),
    [anon_sym_true] = ACTIONS(3488),
    [anon_sym_false] = ACTIONS(3488),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3488),
    [sym_super] = ACTIONS(3488),
    [sym_crate] = ACTIONS(3488),
    [aux_sym_metavariable_token1] = ACTIONS(3486),
    [sym__raw_string_literal_start] = ACTIONS(3486),
    [sym_float_literal] = ACTIONS(3486),
    [sym__outer_block_doc_comment_start] = ACTIONS(3486),
    [sym__outer_line_doc_comment_start] = ACTIONS(3486),
  },
  [STATE(930)] = {
    [sym_line_comment] = STATE(930),
    [sym_block_comment] = STATE(930),
    [ts_builtin_sym_end] = ACTIONS(3490),
    [sym_identifier] = ACTIONS(3492),
    [anon_sym_SEMI] = ACTIONS(3490),
    [anon_sym_macro] = ACTIONS(3492),
    [anon_sym_LPAREN] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3490),
    [anon_sym_RBRACE] = ACTIONS(3490),
    [anon_sym_macro_rules] = ACTIONS(3492),
    [anon_sym_BANG] = ACTIONS(3490),
    [anon_sym_LBRACK] = ACTIONS(3490),
    [anon_sym_STAR] = ACTIONS(3490),
    [anon_sym_u8] = ACTIONS(3492),
    [anon_sym_i8] = ACTIONS(3492),
    [anon_sym_u16] = ACTIONS(3492),
    [anon_sym_i16] = ACTIONS(3492),
    [anon_sym_u32] = ACTIONS(3492),
    [anon_sym_i32] = ACTIONS(3492),
    [anon_sym_u64] = ACTIONS(3492),
    [anon_sym_i64] = ACTIONS(3492),
    [anon_sym_u128] = ACTIONS(3492),
    [anon_sym_i128] = ACTIONS(3492),
    [anon_sym_isize] = ACTIONS(3492),
    [anon_sym_usize] = ACTIONS(3492),
    [anon_sym_f32] = ACTIONS(3492),
    [anon_sym_f64] = ACTIONS(3492),
    [anon_sym_bool] = ACTIONS(3492),
    [anon_sym_str] = ACTIONS(3492),
    [anon_sym_char] = ACTIONS(3492),
    [anon_sym_DASH] = ACTIONS(3490),
    [anon_sym_AMP] = ACTIONS(3490),
    [anon_sym_PIPE] = ACTIONS(3490),
    [anon_sym_LT] = ACTIONS(3490),
    [anon_sym_DOT_DOT] = ACTIONS(3490),
    [anon_sym_COLON_COLON] = ACTIONS(3490),
    [anon_sym_POUND] = ACTIONS(3490),
    [anon_sym_SQUOTE] = ACTIONS(3492),
    [anon_sym_async] = ACTIONS(3492),
    [anon_sym_auto] = ACTIONS(3492),
    [anon_sym_await] = ACTIONS(3492),
    [anon_sym_become] = ACTIONS(3492),
    [anon_sym_break] = ACTIONS(3492),
    [anon_sym_const] = ACTIONS(3492),
    [anon_sym_continue] = ACTIONS(3492),
    [anon_sym_default] = ACTIONS(3492),
    [anon_sym_enum] = ACTIONS(3492),
    [anon_sym_fn] = ACTIONS(3492),
    [anon_sym_for] = ACTIONS(3492),
    [anon_sym_gen] = ACTIONS(3492),
    [anon_sym_if] = ACTIONS(3492),
    [anon_sym_impl] = ACTIONS(3492),
    [anon_sym_let] = ACTIONS(3492),
    [anon_sym_loop] = ACTIONS(3492),
    [anon_sym_match] = ACTIONS(3492),
    [anon_sym_mod] = ACTIONS(3492),
    [anon_sym_pub] = ACTIONS(3492),
    [anon_sym_return] = ACTIONS(3492),
    [anon_sym_raw] = ACTIONS(3492),
    [anon_sym_static] = ACTIONS(3492),
    [anon_sym_struct] = ACTIONS(3492),
    [anon_sym_trait] = ACTIONS(3492),
    [anon_sym_type] = ACTIONS(3492),
    [anon_sym_union] = ACTIONS(3492),
    [anon_sym_unsafe] = ACTIONS(3492),
    [anon_sym_use] = ACTIONS(3492),
    [anon_sym_while] = ACTIONS(3492),
    [anon_sym_extern] = ACTIONS(3492),
    [anon_sym_safe] = ACTIONS(3492),
    [anon_sym_yield] = ACTIONS(3492),
    [anon_sym_move] = ACTIONS(3492),
    [anon_sym_try] = ACTIONS(3492),
    [sym_integer_literal] = ACTIONS(3490),
    [aux_sym_string_literal_token1] = ACTIONS(3490),
    [sym_char_literal] = ACTIONS(3490),
    [anon_sym_true] = ACTIONS(3492),
    [anon_sym_false] = ACTIONS(3492),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3492),
    [sym_super] = ACTIONS(3492),
    [sym_crate] = ACTIONS(3492),
    [aux_sym_metavariable_token1] = ACTIONS(3490),
    [sym__raw_string_literal_start] = ACTIONS(3490),
    [sym_float_literal] = ACTIONS(3490),
    [sym__outer_block_doc_comment_start] = ACTIONS(3490),
    [sym__outer_line_doc_comment_start] = ACTIONS(3490),
  },
  [STATE(931)] = {
    [sym_line_comment] = STATE(931),
    [sym_block_comment] = STATE(931),
    [ts_builtin_sym_end] = ACTIONS(3494),
    [sym_identifier] = ACTIONS(3496),
    [anon_sym_SEMI] = ACTIONS(3494),
    [anon_sym_macro] = ACTIONS(3496),
    [anon_sym_LPAREN] = ACTIONS(3494),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_RBRACE] = ACTIONS(3494),
    [anon_sym_macro_rules] = ACTIONS(3496),
    [anon_sym_BANG] = ACTIONS(3494),
    [anon_sym_LBRACK] = ACTIONS(3494),
    [anon_sym_STAR] = ACTIONS(3494),
    [anon_sym_u8] = ACTIONS(3496),
    [anon_sym_i8] = ACTIONS(3496),
    [anon_sym_u16] = ACTIONS(3496),
    [anon_sym_i16] = ACTIONS(3496),
    [anon_sym_u32] = ACTIONS(3496),
    [anon_sym_i32] = ACTIONS(3496),
    [anon_sym_u64] = ACTIONS(3496),
    [anon_sym_i64] = ACTIONS(3496),
    [anon_sym_u128] = ACTIONS(3496),
    [anon_sym_i128] = ACTIONS(3496),
    [anon_sym_isize] = ACTIONS(3496),
    [anon_sym_usize] = ACTIONS(3496),
    [anon_sym_f32] = ACTIONS(3496),
    [anon_sym_f64] = ACTIONS(3496),
    [anon_sym_bool] = ACTIONS(3496),
    [anon_sym_str] = ACTIONS(3496),
    [anon_sym_char] = ACTIONS(3496),
    [anon_sym_DASH] = ACTIONS(3494),
    [anon_sym_AMP] = ACTIONS(3494),
    [anon_sym_PIPE] = ACTIONS(3494),
    [anon_sym_LT] = ACTIONS(3494),
    [anon_sym_DOT_DOT] = ACTIONS(3494),
    [anon_sym_COLON_COLON] = ACTIONS(3494),
    [anon_sym_POUND] = ACTIONS(3494),
    [anon_sym_SQUOTE] = ACTIONS(3496),
    [anon_sym_async] = ACTIONS(3496),
    [anon_sym_auto] = ACTIONS(3496),
    [anon_sym_await] = ACTIONS(3496),
    [anon_sym_become] = ACTIONS(3496),
    [anon_sym_break] = ACTIONS(3496),
    [anon_sym_const] = ACTIONS(3496),
    [anon_sym_continue] = ACTIONS(3496),
    [anon_sym_default] = ACTIONS(3496),
    [anon_sym_enum] = ACTIONS(3496),
    [anon_sym_fn] = ACTIONS(3496),
    [anon_sym_for] = ACTIONS(3496),
    [anon_sym_gen] = ACTIONS(3496),
    [anon_sym_if] = ACTIONS(3496),
    [anon_sym_impl] = ACTIONS(3496),
    [anon_sym_let] = ACTIONS(3496),
    [anon_sym_loop] = ACTIONS(3496),
    [anon_sym_match] = ACTIONS(3496),
    [anon_sym_mod] = ACTIONS(3496),
    [anon_sym_pub] = ACTIONS(3496),
    [anon_sym_return] = ACTIONS(3496),
    [anon_sym_raw] = ACTIONS(3496),
    [anon_sym_static] = ACTIONS(3496),
    [anon_sym_struct] = ACTIONS(3496),
    [anon_sym_trait] = ACTIONS(3496),
    [anon_sym_type] = ACTIONS(3496),
    [anon_sym_union] = ACTIONS(3496),
    [anon_sym_unsafe] = ACTIONS(3496),
    [anon_sym_use] = ACTIONS(3496),
    [anon_sym_while] = ACTIONS(3496),
    [anon_sym_extern] = ACTIONS(3496),
    [anon_sym_safe] = ACTIONS(3496),
    [anon_sym_yield] = ACTIONS(3496),
    [anon_sym_move] = ACTIONS(3496),
    [anon_sym_try] = ACTIONS(3496),
    [sym_integer_literal] = ACTIONS(3494),
    [aux_sym_string_literal_token1] = ACTIONS(3494),
    [sym_char_literal] = ACTIONS(3494),
    [anon_sym_true] = ACTIONS(3496),
    [anon_sym_false] = ACTIONS(3496),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3496),
    [sym_super] = ACTIONS(3496),
    [sym_crate] = ACTIONS(3496),
    [aux_sym_metavariable_token1] = ACTIONS(3494),
    [sym__raw_string_literal_start] = ACTIONS(3494),
    [sym_float_literal] = ACTIONS(3494),
    [sym__outer_block_doc_comment_start] = ACTIONS(3494),
    [sym__outer_line_doc_comment_start] = ACTIONS(3494),
  },
  [STATE(932)] = {
    [sym_line_comment] = STATE(932),
    [sym_block_comment] = STATE(932),
    [ts_builtin_sym_end] = ACTIONS(3498),
    [sym_identifier] = ACTIONS(3500),
    [anon_sym_SEMI] = ACTIONS(3498),
    [anon_sym_macro] = ACTIONS(3500),
    [anon_sym_LPAREN] = ACTIONS(3498),
    [anon_sym_LBRACE] = ACTIONS(3498),
    [anon_sym_RBRACE] = ACTIONS(3498),
    [anon_sym_macro_rules] = ACTIONS(3500),
    [anon_sym_BANG] = ACTIONS(3498),
    [anon_sym_LBRACK] = ACTIONS(3498),
    [anon_sym_STAR] = ACTIONS(3498),
    [anon_sym_u8] = ACTIONS(3500),
    [anon_sym_i8] = ACTIONS(3500),
    [anon_sym_u16] = ACTIONS(3500),
    [anon_sym_i16] = ACTIONS(3500),
    [anon_sym_u32] = ACTIONS(3500),
    [anon_sym_i32] = ACTIONS(3500),
    [anon_sym_u64] = ACTIONS(3500),
    [anon_sym_i64] = ACTIONS(3500),
    [anon_sym_u128] = ACTIONS(3500),
    [anon_sym_i128] = ACTIONS(3500),
    [anon_sym_isize] = ACTIONS(3500),
    [anon_sym_usize] = ACTIONS(3500),
    [anon_sym_f32] = ACTIONS(3500),
    [anon_sym_f64] = ACTIONS(3500),
    [anon_sym_bool] = ACTIONS(3500),
    [anon_sym_str] = ACTIONS(3500),
    [anon_sym_char] = ACTIONS(3500),
    [anon_sym_DASH] = ACTIONS(3498),
    [anon_sym_AMP] = ACTIONS(3498),
    [anon_sym_PIPE] = ACTIONS(3498),
    [anon_sym_LT] = ACTIONS(3498),
    [anon_sym_DOT_DOT] = ACTIONS(3498),
    [anon_sym_COLON_COLON] = ACTIONS(3498),
    [anon_sym_POUND] = ACTIONS(3498),
    [anon_sym_SQUOTE] = ACTIONS(3500),
    [anon_sym_async] = ACTIONS(3500),
    [anon_sym_auto] = ACTIONS(3500),
    [anon_sym_await] = ACTIONS(3500),
    [anon_sym_become] = ACTIONS(3500),
    [anon_sym_break] = ACTIONS(3500),
    [anon_sym_const] = ACTIONS(3500),
    [anon_sym_continue] = ACTIONS(3500),
    [anon_sym_default] = ACTIONS(3500),
    [anon_sym_enum] = ACTIONS(3500),
    [anon_sym_fn] = ACTIONS(3500),
    [anon_sym_for] = ACTIONS(3500),
    [anon_sym_gen] = ACTIONS(3500),
    [anon_sym_if] = ACTIONS(3500),
    [anon_sym_impl] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3500),
    [anon_sym_loop] = ACTIONS(3500),
    [anon_sym_match] = ACTIONS(3500),
    [anon_sym_mod] = ACTIONS(3500),
    [anon_sym_pub] = ACTIONS(3500),
    [anon_sym_return] = ACTIONS(3500),
    [anon_sym_raw] = ACTIONS(3500),
    [anon_sym_static] = ACTIONS(3500),
    [anon_sym_struct] = ACTIONS(3500),
    [anon_sym_trait] = ACTIONS(3500),
    [anon_sym_type] = ACTIONS(3500),
    [anon_sym_union] = ACTIONS(3500),
    [anon_sym_unsafe] = ACTIONS(3500),
    [anon_sym_use] = ACTIONS(3500),
    [anon_sym_while] = ACTIONS(3500),
    [anon_sym_extern] = ACTIONS(3500),
    [anon_sym_safe] = ACTIONS(3500),
    [anon_sym_yield] = ACTIONS(3500),
    [anon_sym_move] = ACTIONS(3500),
    [anon_sym_try] = ACTIONS(3500),
    [sym_integer_literal] = ACTIONS(3498),
    [aux_sym_string_literal_token1] = ACTIONS(3498),
    [sym_char_literal] = ACTIONS(3498),
    [anon_sym_true] = ACTIONS(3500),
    [anon_sym_false] = ACTIONS(3500),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3500),
    [sym_super] = ACTIONS(3500),
    [sym_crate] = ACTIONS(3500),
    [aux_sym_metavariable_token1] = ACTIONS(3498),
    [sym__raw_string_literal_start] = ACTIONS(3498),
    [sym_float_literal] = ACTIONS(3498),
    [sym__outer_block_doc_comment_start] = ACTIONS(3498),
    [sym__outer_line_doc_comment_start] = ACTIONS(3498),
  },
  [STATE(933)] = {
    [sym_line_comment] = STATE(933),
    [sym_block_comment] = STATE(933),
    [ts_builtin_sym_end] = ACTIONS(3502),
    [sym_identifier] = ACTIONS(3504),
    [anon_sym_SEMI] = ACTIONS(3502),
    [anon_sym_macro] = ACTIONS(3504),
    [anon_sym_LPAREN] = ACTIONS(3502),
    [anon_sym_LBRACE] = ACTIONS(3502),
    [anon_sym_RBRACE] = ACTIONS(3502),
    [anon_sym_macro_rules] = ACTIONS(3504),
    [anon_sym_BANG] = ACTIONS(3502),
    [anon_sym_LBRACK] = ACTIONS(3502),
    [anon_sym_STAR] = ACTIONS(3502),
    [anon_sym_u8] = ACTIONS(3504),
    [anon_sym_i8] = ACTIONS(3504),
    [anon_sym_u16] = ACTIONS(3504),
    [anon_sym_i16] = ACTIONS(3504),
    [anon_sym_u32] = ACTIONS(3504),
    [anon_sym_i32] = ACTIONS(3504),
    [anon_sym_u64] = ACTIONS(3504),
    [anon_sym_i64] = ACTIONS(3504),
    [anon_sym_u128] = ACTIONS(3504),
    [anon_sym_i128] = ACTIONS(3504),
    [anon_sym_isize] = ACTIONS(3504),
    [anon_sym_usize] = ACTIONS(3504),
    [anon_sym_f32] = ACTIONS(3504),
    [anon_sym_f64] = ACTIONS(3504),
    [anon_sym_bool] = ACTIONS(3504),
    [anon_sym_str] = ACTIONS(3504),
    [anon_sym_char] = ACTIONS(3504),
    [anon_sym_DASH] = ACTIONS(3502),
    [anon_sym_AMP] = ACTIONS(3502),
    [anon_sym_PIPE] = ACTIONS(3502),
    [anon_sym_LT] = ACTIONS(3502),
    [anon_sym_DOT_DOT] = ACTIONS(3502),
    [anon_sym_COLON_COLON] = ACTIONS(3502),
    [anon_sym_POUND] = ACTIONS(3502),
    [anon_sym_SQUOTE] = ACTIONS(3504),
    [anon_sym_async] = ACTIONS(3504),
    [anon_sym_auto] = ACTIONS(3504),
    [anon_sym_await] = ACTIONS(3504),
    [anon_sym_become] = ACTIONS(3504),
    [anon_sym_break] = ACTIONS(3504),
    [anon_sym_const] = ACTIONS(3504),
    [anon_sym_continue] = ACTIONS(3504),
    [anon_sym_default] = ACTIONS(3504),
    [anon_sym_enum] = ACTIONS(3504),
    [anon_sym_fn] = ACTIONS(3504),
    [anon_sym_for] = ACTIONS(3504),
    [anon_sym_gen] = ACTIONS(3504),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_impl] = ACTIONS(3504),
    [anon_sym_let] = ACTIONS(3504),
    [anon_sym_loop] = ACTIONS(3504),
    [anon_sym_match] = ACTIONS(3504),
    [anon_sym_mod] = ACTIONS(3504),
    [anon_sym_pub] = ACTIONS(3504),
    [anon_sym_return] = ACTIONS(3504),
    [anon_sym_raw] = ACTIONS(3504),
    [anon_sym_static] = ACTIONS(3504),
    [anon_sym_struct] = ACTIONS(3504),
    [anon_sym_trait] = ACTIONS(3504),
    [anon_sym_type] = ACTIONS(3504),
    [anon_sym_union] = ACTIONS(3504),
    [anon_sym_unsafe] = ACTIONS(3504),
    [anon_sym_use] = ACTIONS(3504),
    [anon_sym_while] = ACTIONS(3504),
    [anon_sym_extern] = ACTIONS(3504),
    [anon_sym_safe] = ACTIONS(3504),
    [anon_sym_yield] = ACTIONS(3504),
    [anon_sym_move] = ACTIONS(3504),
    [anon_sym_try] = ACTIONS(3504),
    [sym_integer_literal] = ACTIONS(3502),
    [aux_sym_string_literal_token1] = ACTIONS(3502),
    [sym_char_literal] = ACTIONS(3502),
    [anon_sym_true] = ACTIONS(3504),
    [anon_sym_false] = ACTIONS(3504),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3504),
    [sym_super] = ACTIONS(3504),
    [sym_crate] = ACTIONS(3504),
    [aux_sym_metavariable_token1] = ACTIONS(3502),
    [sym__raw_string_literal_start] = ACTIONS(3502),
    [sym_float_literal] = ACTIONS(3502),
    [sym__outer_block_doc_comment_start] = ACTIONS(3502),
    [sym__outer_line_doc_comment_start] = ACTIONS(3502),
  },
  [STATE(934)] = {
    [sym_line_comment] = STATE(934),
    [sym_block_comment] = STATE(934),
    [ts_builtin_sym_end] = ACTIONS(3506),
    [sym_identifier] = ACTIONS(3508),
    [anon_sym_SEMI] = ACTIONS(3506),
    [anon_sym_macro] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(3506),
    [anon_sym_LBRACE] = ACTIONS(3506),
    [anon_sym_RBRACE] = ACTIONS(3506),
    [anon_sym_macro_rules] = ACTIONS(3508),
    [anon_sym_BANG] = ACTIONS(3506),
    [anon_sym_LBRACK] = ACTIONS(3506),
    [anon_sym_STAR] = ACTIONS(3506),
    [anon_sym_u8] = ACTIONS(3508),
    [anon_sym_i8] = ACTIONS(3508),
    [anon_sym_u16] = ACTIONS(3508),
    [anon_sym_i16] = ACTIONS(3508),
    [anon_sym_u32] = ACTIONS(3508),
    [anon_sym_i32] = ACTIONS(3508),
    [anon_sym_u64] = ACTIONS(3508),
    [anon_sym_i64] = ACTIONS(3508),
    [anon_sym_u128] = ACTIONS(3508),
    [anon_sym_i128] = ACTIONS(3508),
    [anon_sym_isize] = ACTIONS(3508),
    [anon_sym_usize] = ACTIONS(3508),
    [anon_sym_f32] = ACTIONS(3508),
    [anon_sym_f64] = ACTIONS(3508),
    [anon_sym_bool] = ACTIONS(3508),
    [anon_sym_str] = ACTIONS(3508),
    [anon_sym_char] = ACTIONS(3508),
    [anon_sym_DASH] = ACTIONS(3506),
    [anon_sym_AMP] = ACTIONS(3506),
    [anon_sym_PIPE] = ACTIONS(3506),
    [anon_sym_LT] = ACTIONS(3506),
    [anon_sym_DOT_DOT] = ACTIONS(3506),
    [anon_sym_COLON_COLON] = ACTIONS(3506),
    [anon_sym_POUND] = ACTIONS(3506),
    [anon_sym_SQUOTE] = ACTIONS(3508),
    [anon_sym_async] = ACTIONS(3508),
    [anon_sym_auto] = ACTIONS(3508),
    [anon_sym_await] = ACTIONS(3508),
    [anon_sym_become] = ACTIONS(3508),
    [anon_sym_break] = ACTIONS(3508),
    [anon_sym_const] = ACTIONS(3508),
    [anon_sym_continue] = ACTIONS(3508),
    [anon_sym_default] = ACTIONS(3508),
    [anon_sym_enum] = ACTIONS(3508),
    [anon_sym_fn] = ACTIONS(3508),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_gen] = ACTIONS(3508),
    [anon_sym_if] = ACTIONS(3508),
    [anon_sym_impl] = ACTIONS(3508),
    [anon_sym_let] = ACTIONS(3508),
    [anon_sym_loop] = ACTIONS(3508),
    [anon_sym_match] = ACTIONS(3508),
    [anon_sym_mod] = ACTIONS(3508),
    [anon_sym_pub] = ACTIONS(3508),
    [anon_sym_return] = ACTIONS(3508),
    [anon_sym_raw] = ACTIONS(3508),
    [anon_sym_static] = ACTIONS(3508),
    [anon_sym_struct] = ACTIONS(3508),
    [anon_sym_trait] = ACTIONS(3508),
    [anon_sym_type] = ACTIONS(3508),
    [anon_sym_union] = ACTIONS(3508),
    [anon_sym_unsafe] = ACTIONS(3508),
    [anon_sym_use] = ACTIONS(3508),
    [anon_sym_while] = ACTIONS(3508),
    [anon_sym_extern] = ACTIONS(3508),
    [anon_sym_safe] = ACTIONS(3508),
    [anon_sym_yield] = ACTIONS(3508),
    [anon_sym_move] = ACTIONS(3508),
    [anon_sym_try] = ACTIONS(3508),
    [sym_integer_literal] = ACTIONS(3506),
    [aux_sym_string_literal_token1] = ACTIONS(3506),
    [sym_char_literal] = ACTIONS(3506),
    [anon_sym_true] = ACTIONS(3508),
    [anon_sym_false] = ACTIONS(3508),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3508),
    [sym_super] = ACTIONS(3508),
    [sym_crate] = ACTIONS(3508),
    [aux_sym_metavariable_token1] = ACTIONS(3506),
    [sym__raw_string_literal_start] = ACTIONS(3506),
    [sym_float_literal] = ACTIONS(3506),
    [sym__outer_block_doc_comment_start] = ACTIONS(3506),
    [sym__outer_line_doc_comment_start] = ACTIONS(3506),
  },
  [STATE(935)] = {
    [sym_line_comment] = STATE(935),
    [sym_block_comment] = STATE(935),
    [ts_builtin_sym_end] = ACTIONS(3510),
    [sym_identifier] = ACTIONS(3512),
    [anon_sym_SEMI] = ACTIONS(3510),
    [anon_sym_macro] = ACTIONS(3512),
    [anon_sym_LPAREN] = ACTIONS(3510),
    [anon_sym_LBRACE] = ACTIONS(3510),
    [anon_sym_RBRACE] = ACTIONS(3510),
    [anon_sym_macro_rules] = ACTIONS(3512),
    [anon_sym_BANG] = ACTIONS(3510),
    [anon_sym_LBRACK] = ACTIONS(3510),
    [anon_sym_STAR] = ACTIONS(3510),
    [anon_sym_u8] = ACTIONS(3512),
    [anon_sym_i8] = ACTIONS(3512),
    [anon_sym_u16] = ACTIONS(3512),
    [anon_sym_i16] = ACTIONS(3512),
    [anon_sym_u32] = ACTIONS(3512),
    [anon_sym_i32] = ACTIONS(3512),
    [anon_sym_u64] = ACTIONS(3512),
    [anon_sym_i64] = ACTIONS(3512),
    [anon_sym_u128] = ACTIONS(3512),
    [anon_sym_i128] = ACTIONS(3512),
    [anon_sym_isize] = ACTIONS(3512),
    [anon_sym_usize] = ACTIONS(3512),
    [anon_sym_f32] = ACTIONS(3512),
    [anon_sym_f64] = ACTIONS(3512),
    [anon_sym_bool] = ACTIONS(3512),
    [anon_sym_str] = ACTIONS(3512),
    [anon_sym_char] = ACTIONS(3512),
    [anon_sym_DASH] = ACTIONS(3510),
    [anon_sym_AMP] = ACTIONS(3510),
    [anon_sym_PIPE] = ACTIONS(3510),
    [anon_sym_LT] = ACTIONS(3510),
    [anon_sym_DOT_DOT] = ACTIONS(3510),
    [anon_sym_COLON_COLON] = ACTIONS(3510),
    [anon_sym_POUND] = ACTIONS(3510),
    [anon_sym_SQUOTE] = ACTIONS(3512),
    [anon_sym_async] = ACTIONS(3512),
    [anon_sym_auto] = ACTIONS(3512),
    [anon_sym_await] = ACTIONS(3512),
    [anon_sym_become] = ACTIONS(3512),
    [anon_sym_break] = ACTIONS(3512),
    [anon_sym_const] = ACTIONS(3512),
    [anon_sym_continue] = ACTIONS(3512),
    [anon_sym_default] = ACTIONS(3512),
    [anon_sym_enum] = ACTIONS(3512),
    [anon_sym_fn] = ACTIONS(3512),
    [anon_sym_for] = ACTIONS(3512),
    [anon_sym_gen] = ACTIONS(3512),
    [anon_sym_if] = ACTIONS(3512),
    [anon_sym_impl] = ACTIONS(3512),
    [anon_sym_let] = ACTIONS(3512),
    [anon_sym_loop] = ACTIONS(3512),
    [anon_sym_match] = ACTIONS(3512),
    [anon_sym_mod] = ACTIONS(3512),
    [anon_sym_pub] = ACTIONS(3512),
    [anon_sym_return] = ACTIONS(3512),
    [anon_sym_raw] = ACTIONS(3512),
    [anon_sym_static] = ACTIONS(3512),
    [anon_sym_struct] = ACTIONS(3512),
    [anon_sym_trait] = ACTIONS(3512),
    [anon_sym_type] = ACTIONS(3512),
    [anon_sym_union] = ACTIONS(3512),
    [anon_sym_unsafe] = ACTIONS(3512),
    [anon_sym_use] = ACTIONS(3512),
    [anon_sym_while] = ACTIONS(3512),
    [anon_sym_extern] = ACTIONS(3512),
    [anon_sym_safe] = ACTIONS(3512),
    [anon_sym_yield] = ACTIONS(3512),
    [anon_sym_move] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3512),
    [sym_integer_literal] = ACTIONS(3510),
    [aux_sym_string_literal_token1] = ACTIONS(3510),
    [sym_char_literal] = ACTIONS(3510),
    [anon_sym_true] = ACTIONS(3512),
    [anon_sym_false] = ACTIONS(3512),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3512),
    [sym_super] = ACTIONS(3512),
    [sym_crate] = ACTIONS(3512),
    [aux_sym_metavariable_token1] = ACTIONS(3510),
    [sym__raw_string_literal_start] = ACTIONS(3510),
    [sym_float_literal] = ACTIONS(3510),
    [sym__outer_block_doc_comment_start] = ACTIONS(3510),
    [sym__outer_line_doc_comment_start] = ACTIONS(3510),
  },
  [STATE(936)] = {
    [sym_line_comment] = STATE(936),
    [sym_block_comment] = STATE(936),
    [ts_builtin_sym_end] = ACTIONS(3514),
    [sym_identifier] = ACTIONS(3516),
    [anon_sym_SEMI] = ACTIONS(3514),
    [anon_sym_macro] = ACTIONS(3516),
    [anon_sym_LPAREN] = ACTIONS(3514),
    [anon_sym_LBRACE] = ACTIONS(3514),
    [anon_sym_RBRACE] = ACTIONS(3514),
    [anon_sym_macro_rules] = ACTIONS(3516),
    [anon_sym_BANG] = ACTIONS(3514),
    [anon_sym_LBRACK] = ACTIONS(3514),
    [anon_sym_STAR] = ACTIONS(3514),
    [anon_sym_u8] = ACTIONS(3516),
    [anon_sym_i8] = ACTIONS(3516),
    [anon_sym_u16] = ACTIONS(3516),
    [anon_sym_i16] = ACTIONS(3516),
    [anon_sym_u32] = ACTIONS(3516),
    [anon_sym_i32] = ACTIONS(3516),
    [anon_sym_u64] = ACTIONS(3516),
    [anon_sym_i64] = ACTIONS(3516),
    [anon_sym_u128] = ACTIONS(3516),
    [anon_sym_i128] = ACTIONS(3516),
    [anon_sym_isize] = ACTIONS(3516),
    [anon_sym_usize] = ACTIONS(3516),
    [anon_sym_f32] = ACTIONS(3516),
    [anon_sym_f64] = ACTIONS(3516),
    [anon_sym_bool] = ACTIONS(3516),
    [anon_sym_str] = ACTIONS(3516),
    [anon_sym_char] = ACTIONS(3516),
    [anon_sym_DASH] = ACTIONS(3514),
    [anon_sym_AMP] = ACTIONS(3514),
    [anon_sym_PIPE] = ACTIONS(3514),
    [anon_sym_LT] = ACTIONS(3514),
    [anon_sym_DOT_DOT] = ACTIONS(3514),
    [anon_sym_COLON_COLON] = ACTIONS(3514),
    [anon_sym_POUND] = ACTIONS(3514),
    [anon_sym_SQUOTE] = ACTIONS(3516),
    [anon_sym_async] = ACTIONS(3516),
    [anon_sym_auto] = ACTIONS(3516),
    [anon_sym_await] = ACTIONS(3516),
    [anon_sym_become] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3516),
    [anon_sym_const] = ACTIONS(3516),
    [anon_sym_continue] = ACTIONS(3516),
    [anon_sym_default] = ACTIONS(3516),
    [anon_sym_enum] = ACTIONS(3516),
    [anon_sym_fn] = ACTIONS(3516),
    [anon_sym_for] = ACTIONS(3516),
    [anon_sym_gen] = ACTIONS(3516),
    [anon_sym_if] = ACTIONS(3516),
    [anon_sym_impl] = ACTIONS(3516),
    [anon_sym_let] = ACTIONS(3516),
    [anon_sym_loop] = ACTIONS(3516),
    [anon_sym_match] = ACTIONS(3516),
    [anon_sym_mod] = ACTIONS(3516),
    [anon_sym_pub] = ACTIONS(3516),
    [anon_sym_return] = ACTIONS(3516),
    [anon_sym_raw] = ACTIONS(3516),
    [anon_sym_static] = ACTIONS(3516),
    [anon_sym_struct] = ACTIONS(3516),
    [anon_sym_trait] = ACTIONS(3516),
    [anon_sym_type] = ACTIONS(3516),
    [anon_sym_union] = ACTIONS(3516),
    [anon_sym_unsafe] = ACTIONS(3516),
    [anon_sym_use] = ACTIONS(3516),
    [anon_sym_while] = ACTIONS(3516),
    [anon_sym_extern] = ACTIONS(3516),
    [anon_sym_safe] = ACTIONS(3516),
    [anon_sym_yield] = ACTIONS(3516),
    [anon_sym_move] = ACTIONS(3516),
    [anon_sym_try] = ACTIONS(3516),
    [sym_integer_literal] = ACTIONS(3514),
    [aux_sym_string_literal_token1] = ACTIONS(3514),
    [sym_char_literal] = ACTIONS(3514),
    [anon_sym_true] = ACTIONS(3516),
    [anon_sym_false] = ACTIONS(3516),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3516),
    [sym_super] = ACTIONS(3516),
    [sym_crate] = ACTIONS(3516),
    [aux_sym_metavariable_token1] = ACTIONS(3514),
    [sym__raw_string_literal_start] = ACTIONS(3514),
    [sym_float_literal] = ACTIONS(3514),
    [sym__outer_block_doc_comment_start] = ACTIONS(3514),
    [sym__outer_line_doc_comment_start] = ACTIONS(3514),
  },
  [STATE(937)] = {
    [sym_line_comment] = STATE(937),
    [sym_block_comment] = STATE(937),
    [ts_builtin_sym_end] = ACTIONS(3518),
    [sym_identifier] = ACTIONS(3520),
    [anon_sym_SEMI] = ACTIONS(3518),
    [anon_sym_macro] = ACTIONS(3520),
    [anon_sym_LPAREN] = ACTIONS(3518),
    [anon_sym_LBRACE] = ACTIONS(3518),
    [anon_sym_RBRACE] = ACTIONS(3518),
    [anon_sym_macro_rules] = ACTIONS(3520),
    [anon_sym_BANG] = ACTIONS(3518),
    [anon_sym_LBRACK] = ACTIONS(3518),
    [anon_sym_STAR] = ACTIONS(3518),
    [anon_sym_u8] = ACTIONS(3520),
    [anon_sym_i8] = ACTIONS(3520),
    [anon_sym_u16] = ACTIONS(3520),
    [anon_sym_i16] = ACTIONS(3520),
    [anon_sym_u32] = ACTIONS(3520),
    [anon_sym_i32] = ACTIONS(3520),
    [anon_sym_u64] = ACTIONS(3520),
    [anon_sym_i64] = ACTIONS(3520),
    [anon_sym_u128] = ACTIONS(3520),
    [anon_sym_i128] = ACTIONS(3520),
    [anon_sym_isize] = ACTIONS(3520),
    [anon_sym_usize] = ACTIONS(3520),
    [anon_sym_f32] = ACTIONS(3520),
    [anon_sym_f64] = ACTIONS(3520),
    [anon_sym_bool] = ACTIONS(3520),
    [anon_sym_str] = ACTIONS(3520),
    [anon_sym_char] = ACTIONS(3520),
    [anon_sym_DASH] = ACTIONS(3518),
    [anon_sym_AMP] = ACTIONS(3518),
    [anon_sym_PIPE] = ACTIONS(3518),
    [anon_sym_LT] = ACTIONS(3518),
    [anon_sym_DOT_DOT] = ACTIONS(3518),
    [anon_sym_COLON_COLON] = ACTIONS(3518),
    [anon_sym_POUND] = ACTIONS(3518),
    [anon_sym_SQUOTE] = ACTIONS(3520),
    [anon_sym_async] = ACTIONS(3520),
    [anon_sym_auto] = ACTIONS(3520),
    [anon_sym_await] = ACTIONS(3520),
    [anon_sym_become] = ACTIONS(3520),
    [anon_sym_break] = ACTIONS(3520),
    [anon_sym_const] = ACTIONS(3520),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_default] = ACTIONS(3520),
    [anon_sym_enum] = ACTIONS(3520),
    [anon_sym_fn] = ACTIONS(3520),
    [anon_sym_for] = ACTIONS(3520),
    [anon_sym_gen] = ACTIONS(3520),
    [anon_sym_if] = ACTIONS(3520),
    [anon_sym_impl] = ACTIONS(3520),
    [anon_sym_let] = ACTIONS(3520),
    [anon_sym_loop] = ACTIONS(3520),
    [anon_sym_match] = ACTIONS(3520),
    [anon_sym_mod] = ACTIONS(3520),
    [anon_sym_pub] = ACTIONS(3520),
    [anon_sym_return] = ACTIONS(3520),
    [anon_sym_raw] = ACTIONS(3520),
    [anon_sym_static] = ACTIONS(3520),
    [anon_sym_struct] = ACTIONS(3520),
    [anon_sym_trait] = ACTIONS(3520),
    [anon_sym_type] = ACTIONS(3520),
    [anon_sym_union] = ACTIONS(3520),
    [anon_sym_unsafe] = ACTIONS(3520),
    [anon_sym_use] = ACTIONS(3520),
    [anon_sym_while] = ACTIONS(3520),
    [anon_sym_extern] = ACTIONS(3520),
    [anon_sym_safe] = ACTIONS(3520),
    [anon_sym_yield] = ACTIONS(3520),
    [anon_sym_move] = ACTIONS(3520),
    [anon_sym_try] = ACTIONS(3520),
    [sym_integer_literal] = ACTIONS(3518),
    [aux_sym_string_literal_token1] = ACTIONS(3518),
    [sym_char_literal] = ACTIONS(3518),
    [anon_sym_true] = ACTIONS(3520),
    [anon_sym_false] = ACTIONS(3520),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3520),
    [sym_super] = ACTIONS(3520),
    [sym_crate] = ACTIONS(3520),
    [aux_sym_metavariable_token1] = ACTIONS(3518),
    [sym__raw_string_literal_start] = ACTIONS(3518),
    [sym_float_literal] = ACTIONS(3518),
    [sym__outer_block_doc_comment_start] = ACTIONS(3518),
    [sym__outer_line_doc_comment_start] = ACTIONS(3518),
  },
  [STATE(938)] = {
    [sym_line_comment] = STATE(938),
    [sym_block_comment] = STATE(938),
    [ts_builtin_sym_end] = ACTIONS(3522),
    [sym_identifier] = ACTIONS(3524),
    [anon_sym_SEMI] = ACTIONS(3522),
    [anon_sym_macro] = ACTIONS(3524),
    [anon_sym_LPAREN] = ACTIONS(3522),
    [anon_sym_LBRACE] = ACTIONS(3522),
    [anon_sym_RBRACE] = ACTIONS(3522),
    [anon_sym_macro_rules] = ACTIONS(3524),
    [anon_sym_BANG] = ACTIONS(3522),
    [anon_sym_LBRACK] = ACTIONS(3522),
    [anon_sym_STAR] = ACTIONS(3522),
    [anon_sym_u8] = ACTIONS(3524),
    [anon_sym_i8] = ACTIONS(3524),
    [anon_sym_u16] = ACTIONS(3524),
    [anon_sym_i16] = ACTIONS(3524),
    [anon_sym_u32] = ACTIONS(3524),
    [anon_sym_i32] = ACTIONS(3524),
    [anon_sym_u64] = ACTIONS(3524),
    [anon_sym_i64] = ACTIONS(3524),
    [anon_sym_u128] = ACTIONS(3524),
    [anon_sym_i128] = ACTIONS(3524),
    [anon_sym_isize] = ACTIONS(3524),
    [anon_sym_usize] = ACTIONS(3524),
    [anon_sym_f32] = ACTIONS(3524),
    [anon_sym_f64] = ACTIONS(3524),
    [anon_sym_bool] = ACTIONS(3524),
    [anon_sym_str] = ACTIONS(3524),
    [anon_sym_char] = ACTIONS(3524),
    [anon_sym_DASH] = ACTIONS(3522),
    [anon_sym_AMP] = ACTIONS(3522),
    [anon_sym_PIPE] = ACTIONS(3522),
    [anon_sym_LT] = ACTIONS(3522),
    [anon_sym_DOT_DOT] = ACTIONS(3522),
    [anon_sym_COLON_COLON] = ACTIONS(3522),
    [anon_sym_POUND] = ACTIONS(3522),
    [anon_sym_SQUOTE] = ACTIONS(3524),
    [anon_sym_async] = ACTIONS(3524),
    [anon_sym_auto] = ACTIONS(3524),
    [anon_sym_await] = ACTIONS(3524),
    [anon_sym_become] = ACTIONS(3524),
    [anon_sym_break] = ACTIONS(3524),
    [anon_sym_const] = ACTIONS(3524),
    [anon_sym_continue] = ACTIONS(3524),
    [anon_sym_default] = ACTIONS(3524),
    [anon_sym_enum] = ACTIONS(3524),
    [anon_sym_fn] = ACTIONS(3524),
    [anon_sym_for] = ACTIONS(3524),
    [anon_sym_gen] = ACTIONS(3524),
    [anon_sym_if] = ACTIONS(3524),
    [anon_sym_impl] = ACTIONS(3524),
    [anon_sym_let] = ACTIONS(3524),
    [anon_sym_loop] = ACTIONS(3524),
    [anon_sym_match] = ACTIONS(3524),
    [anon_sym_mod] = ACTIONS(3524),
    [anon_sym_pub] = ACTIONS(3524),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_raw] = ACTIONS(3524),
    [anon_sym_static] = ACTIONS(3524),
    [anon_sym_struct] = ACTIONS(3524),
    [anon_sym_trait] = ACTIONS(3524),
    [anon_sym_type] = ACTIONS(3524),
    [anon_sym_union] = ACTIONS(3524),
    [anon_sym_unsafe] = ACTIONS(3524),
    [anon_sym_use] = ACTIONS(3524),
    [anon_sym_while] = ACTIONS(3524),
    [anon_sym_extern] = ACTIONS(3524),
    [anon_sym_safe] = ACTIONS(3524),
    [anon_sym_yield] = ACTIONS(3524),
    [anon_sym_move] = ACTIONS(3524),
    [anon_sym_try] = ACTIONS(3524),
    [sym_integer_literal] = ACTIONS(3522),
    [aux_sym_string_literal_token1] = ACTIONS(3522),
    [sym_char_literal] = ACTIONS(3522),
    [anon_sym_true] = ACTIONS(3524),
    [anon_sym_false] = ACTIONS(3524),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3524),
    [sym_super] = ACTIONS(3524),
    [sym_crate] = ACTIONS(3524),
    [aux_sym_metavariable_token1] = ACTIONS(3522),
    [sym__raw_string_literal_start] = ACTIONS(3522),
    [sym_float_literal] = ACTIONS(3522),
    [sym__outer_block_doc_comment_start] = ACTIONS(3522),
    [sym__outer_line_doc_comment_start] = ACTIONS(3522),
  },
  [STATE(939)] = {
    [sym_line_comment] = STATE(939),
    [sym_block_comment] = STATE(939),
    [ts_builtin_sym_end] = ACTIONS(3526),
    [sym_identifier] = ACTIONS(3528),
    [anon_sym_SEMI] = ACTIONS(3526),
    [anon_sym_macro] = ACTIONS(3528),
    [anon_sym_LPAREN] = ACTIONS(3526),
    [anon_sym_LBRACE] = ACTIONS(3526),
    [anon_sym_RBRACE] = ACTIONS(3526),
    [anon_sym_macro_rules] = ACTIONS(3528),
    [anon_sym_BANG] = ACTIONS(3526),
    [anon_sym_LBRACK] = ACTIONS(3526),
    [anon_sym_STAR] = ACTIONS(3526),
    [anon_sym_u8] = ACTIONS(3528),
    [anon_sym_i8] = ACTIONS(3528),
    [anon_sym_u16] = ACTIONS(3528),
    [anon_sym_i16] = ACTIONS(3528),
    [anon_sym_u32] = ACTIONS(3528),
    [anon_sym_i32] = ACTIONS(3528),
    [anon_sym_u64] = ACTIONS(3528),
    [anon_sym_i64] = ACTIONS(3528),
    [anon_sym_u128] = ACTIONS(3528),
    [anon_sym_i128] = ACTIONS(3528),
    [anon_sym_isize] = ACTIONS(3528),
    [anon_sym_usize] = ACTIONS(3528),
    [anon_sym_f32] = ACTIONS(3528),
    [anon_sym_f64] = ACTIONS(3528),
    [anon_sym_bool] = ACTIONS(3528),
    [anon_sym_str] = ACTIONS(3528),
    [anon_sym_char] = ACTIONS(3528),
    [anon_sym_DASH] = ACTIONS(3526),
    [anon_sym_AMP] = ACTIONS(3526),
    [anon_sym_PIPE] = ACTIONS(3526),
    [anon_sym_LT] = ACTIONS(3526),
    [anon_sym_DOT_DOT] = ACTIONS(3526),
    [anon_sym_COLON_COLON] = ACTIONS(3526),
    [anon_sym_POUND] = ACTIONS(3526),
    [anon_sym_SQUOTE] = ACTIONS(3528),
    [anon_sym_async] = ACTIONS(3528),
    [anon_sym_auto] = ACTIONS(3528),
    [anon_sym_await] = ACTIONS(3528),
    [anon_sym_become] = ACTIONS(3528),
    [anon_sym_break] = ACTIONS(3528),
    [anon_sym_const] = ACTIONS(3528),
    [anon_sym_continue] = ACTIONS(3528),
    [anon_sym_default] = ACTIONS(3528),
    [anon_sym_enum] = ACTIONS(3528),
    [anon_sym_fn] = ACTIONS(3528),
    [anon_sym_for] = ACTIONS(3528),
    [anon_sym_gen] = ACTIONS(3528),
    [anon_sym_if] = ACTIONS(3528),
    [anon_sym_impl] = ACTIONS(3528),
    [anon_sym_let] = ACTIONS(3528),
    [anon_sym_loop] = ACTIONS(3528),
    [anon_sym_match] = ACTIONS(3528),
    [anon_sym_mod] = ACTIONS(3528),
    [anon_sym_pub] = ACTIONS(3528),
    [anon_sym_return] = ACTIONS(3528),
    [anon_sym_raw] = ACTIONS(3528),
    [anon_sym_static] = ACTIONS(3528),
    [anon_sym_struct] = ACTIONS(3528),
    [anon_sym_trait] = ACTIONS(3528),
    [anon_sym_type] = ACTIONS(3528),
    [anon_sym_union] = ACTIONS(3528),
    [anon_sym_unsafe] = ACTIONS(3528),
    [anon_sym_use] = ACTIONS(3528),
    [anon_sym_while] = ACTIONS(3528),
    [anon_sym_extern] = ACTIONS(3528),
    [anon_sym_safe] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(3528),
    [anon_sym_move] = ACTIONS(3528),
    [anon_sym_try] = ACTIONS(3528),
    [sym_integer_literal] = ACTIONS(3526),
    [aux_sym_string_literal_token1] = ACTIONS(3526),
    [sym_char_literal] = ACTIONS(3526),
    [anon_sym_true] = ACTIONS(3528),
    [anon_sym_false] = ACTIONS(3528),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3528),
    [sym_super] = ACTIONS(3528),
    [sym_crate] = ACTIONS(3528),
    [aux_sym_metavariable_token1] = ACTIONS(3526),
    [sym__raw_string_literal_start] = ACTIONS(3526),
    [sym_float_literal] = ACTIONS(3526),
    [sym__outer_block_doc_comment_start] = ACTIONS(3526),
    [sym__outer_line_doc_comment_start] = ACTIONS(3526),
  },
  [STATE(940)] = {
    [sym_line_comment] = STATE(940),
    [sym_block_comment] = STATE(940),
    [ts_builtin_sym_end] = ACTIONS(3530),
    [sym_identifier] = ACTIONS(3532),
    [anon_sym_SEMI] = ACTIONS(3530),
    [anon_sym_macro] = ACTIONS(3532),
    [anon_sym_LPAREN] = ACTIONS(3530),
    [anon_sym_LBRACE] = ACTIONS(3530),
    [anon_sym_RBRACE] = ACTIONS(3530),
    [anon_sym_macro_rules] = ACTIONS(3532),
    [anon_sym_BANG] = ACTIONS(3530),
    [anon_sym_LBRACK] = ACTIONS(3530),
    [anon_sym_STAR] = ACTIONS(3530),
    [anon_sym_u8] = ACTIONS(3532),
    [anon_sym_i8] = ACTIONS(3532),
    [anon_sym_u16] = ACTIONS(3532),
    [anon_sym_i16] = ACTIONS(3532),
    [anon_sym_u32] = ACTIONS(3532),
    [anon_sym_i32] = ACTIONS(3532),
    [anon_sym_u64] = ACTIONS(3532),
    [anon_sym_i64] = ACTIONS(3532),
    [anon_sym_u128] = ACTIONS(3532),
    [anon_sym_i128] = ACTIONS(3532),
    [anon_sym_isize] = ACTIONS(3532),
    [anon_sym_usize] = ACTIONS(3532),
    [anon_sym_f32] = ACTIONS(3532),
    [anon_sym_f64] = ACTIONS(3532),
    [anon_sym_bool] = ACTIONS(3532),
    [anon_sym_str] = ACTIONS(3532),
    [anon_sym_char] = ACTIONS(3532),
    [anon_sym_DASH] = ACTIONS(3530),
    [anon_sym_AMP] = ACTIONS(3530),
    [anon_sym_PIPE] = ACTIONS(3530),
    [anon_sym_LT] = ACTIONS(3530),
    [anon_sym_DOT_DOT] = ACTIONS(3530),
    [anon_sym_COLON_COLON] = ACTIONS(3530),
    [anon_sym_POUND] = ACTIONS(3530),
    [anon_sym_SQUOTE] = ACTIONS(3532),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_auto] = ACTIONS(3532),
    [anon_sym_await] = ACTIONS(3532),
    [anon_sym_become] = ACTIONS(3532),
    [anon_sym_break] = ACTIONS(3532),
    [anon_sym_const] = ACTIONS(3532),
    [anon_sym_continue] = ACTIONS(3532),
    [anon_sym_default] = ACTIONS(3532),
    [anon_sym_enum] = ACTIONS(3532),
    [anon_sym_fn] = ACTIONS(3532),
    [anon_sym_for] = ACTIONS(3532),
    [anon_sym_gen] = ACTIONS(3532),
    [anon_sym_if] = ACTIONS(3532),
    [anon_sym_impl] = ACTIONS(3532),
    [anon_sym_let] = ACTIONS(3532),
    [anon_sym_loop] = ACTIONS(3532),
    [anon_sym_match] = ACTIONS(3532),
    [anon_sym_mod] = ACTIONS(3532),
    [anon_sym_pub] = ACTIONS(3532),
    [anon_sym_return] = ACTIONS(3532),
    [anon_sym_raw] = ACTIONS(3532),
    [anon_sym_static] = ACTIONS(3532),
    [anon_sym_struct] = ACTIONS(3532),
    [anon_sym_trait] = ACTIONS(3532),
    [anon_sym_type] = ACTIONS(3532),
    [anon_sym_union] = ACTIONS(3532),
    [anon_sym_unsafe] = ACTIONS(3532),
    [anon_sym_use] = ACTIONS(3532),
    [anon_sym_while] = ACTIONS(3532),
    [anon_sym_extern] = ACTIONS(3532),
    [anon_sym_safe] = ACTIONS(3532),
    [anon_sym_yield] = ACTIONS(3532),
    [anon_sym_move] = ACTIONS(3532),
    [anon_sym_try] = ACTIONS(3532),
    [sym_integer_literal] = ACTIONS(3530),
    [aux_sym_string_literal_token1] = ACTIONS(3530),
    [sym_char_literal] = ACTIONS(3530),
    [anon_sym_true] = ACTIONS(3532),
    [anon_sym_false] = ACTIONS(3532),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3532),
    [sym_super] = ACTIONS(3532),
    [sym_crate] = ACTIONS(3532),
    [aux_sym_metavariable_token1] = ACTIONS(3530),
    [sym__raw_string_literal_start] = ACTIONS(3530),
    [sym_float_literal] = ACTIONS(3530),
    [sym__outer_block_doc_comment_start] = ACTIONS(3530),
    [sym__outer_line_doc_comment_start] = ACTIONS(3530),
  },
  [STATE(941)] = {
    [sym_line_comment] = STATE(941),
    [sym_block_comment] = STATE(941),
    [ts_builtin_sym_end] = ACTIONS(3534),
    [sym_identifier] = ACTIONS(3536),
    [anon_sym_SEMI] = ACTIONS(3534),
    [anon_sym_macro] = ACTIONS(3536),
    [anon_sym_LPAREN] = ACTIONS(3534),
    [anon_sym_LBRACE] = ACTIONS(3534),
    [anon_sym_RBRACE] = ACTIONS(3534),
    [anon_sym_macro_rules] = ACTIONS(3536),
    [anon_sym_BANG] = ACTIONS(3534),
    [anon_sym_LBRACK] = ACTIONS(3534),
    [anon_sym_STAR] = ACTIONS(3534),
    [anon_sym_u8] = ACTIONS(3536),
    [anon_sym_i8] = ACTIONS(3536),
    [anon_sym_u16] = ACTIONS(3536),
    [anon_sym_i16] = ACTIONS(3536),
    [anon_sym_u32] = ACTIONS(3536),
    [anon_sym_i32] = ACTIONS(3536),
    [anon_sym_u64] = ACTIONS(3536),
    [anon_sym_i64] = ACTIONS(3536),
    [anon_sym_u128] = ACTIONS(3536),
    [anon_sym_i128] = ACTIONS(3536),
    [anon_sym_isize] = ACTIONS(3536),
    [anon_sym_usize] = ACTIONS(3536),
    [anon_sym_f32] = ACTIONS(3536),
    [anon_sym_f64] = ACTIONS(3536),
    [anon_sym_bool] = ACTIONS(3536),
    [anon_sym_str] = ACTIONS(3536),
    [anon_sym_char] = ACTIONS(3536),
    [anon_sym_DASH] = ACTIONS(3534),
    [anon_sym_AMP] = ACTIONS(3534),
    [anon_sym_PIPE] = ACTIONS(3534),
    [anon_sym_LT] = ACTIONS(3534),
    [anon_sym_DOT_DOT] = ACTIONS(3534),
    [anon_sym_COLON_COLON] = ACTIONS(3534),
    [anon_sym_POUND] = ACTIONS(3534),
    [anon_sym_SQUOTE] = ACTIONS(3536),
    [anon_sym_async] = ACTIONS(3536),
    [anon_sym_auto] = ACTIONS(3536),
    [anon_sym_await] = ACTIONS(3536),
    [anon_sym_become] = ACTIONS(3536),
    [anon_sym_break] = ACTIONS(3536),
    [anon_sym_const] = ACTIONS(3536),
    [anon_sym_continue] = ACTIONS(3536),
    [anon_sym_default] = ACTIONS(3536),
    [anon_sym_enum] = ACTIONS(3536),
    [anon_sym_fn] = ACTIONS(3536),
    [anon_sym_for] = ACTIONS(3536),
    [anon_sym_gen] = ACTIONS(3536),
    [anon_sym_if] = ACTIONS(3536),
    [anon_sym_impl] = ACTIONS(3536),
    [anon_sym_let] = ACTIONS(3536),
    [anon_sym_loop] = ACTIONS(3536),
    [anon_sym_match] = ACTIONS(3536),
    [anon_sym_mod] = ACTIONS(3536),
    [anon_sym_pub] = ACTIONS(3536),
    [anon_sym_return] = ACTIONS(3536),
    [anon_sym_raw] = ACTIONS(3536),
    [anon_sym_static] = ACTIONS(3536),
    [anon_sym_struct] = ACTIONS(3536),
    [anon_sym_trait] = ACTIONS(3536),
    [anon_sym_type] = ACTIONS(3536),
    [anon_sym_union] = ACTIONS(3536),
    [anon_sym_unsafe] = ACTIONS(3536),
    [anon_sym_use] = ACTIONS(3536),
    [anon_sym_while] = ACTIONS(3536),
    [anon_sym_extern] = ACTIONS(3536),
    [anon_sym_safe] = ACTIONS(3536),
    [anon_sym_yield] = ACTIONS(3536),
    [anon_sym_move] = ACTIONS(3536),
    [anon_sym_try] = ACTIONS(3536),
    [sym_integer_literal] = ACTIONS(3534),
    [aux_sym_string_literal_token1] = ACTIONS(3534),
    [sym_char_literal] = ACTIONS(3534),
    [anon_sym_true] = ACTIONS(3536),
    [anon_sym_false] = ACTIONS(3536),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3536),
    [sym_super] = ACTIONS(3536),
    [sym_crate] = ACTIONS(3536),
    [aux_sym_metavariable_token1] = ACTIONS(3534),
    [sym__raw_string_literal_start] = ACTIONS(3534),
    [sym_float_literal] = ACTIONS(3534),
    [sym__outer_block_doc_comment_start] = ACTIONS(3534),
    [sym__outer_line_doc_comment_start] = ACTIONS(3534),
  },
  [STATE(942)] = {
    [sym_line_comment] = STATE(942),
    [sym_block_comment] = STATE(942),
    [ts_builtin_sym_end] = ACTIONS(3538),
    [sym_identifier] = ACTIONS(3540),
    [anon_sym_SEMI] = ACTIONS(3538),
    [anon_sym_macro] = ACTIONS(3540),
    [anon_sym_LPAREN] = ACTIONS(3538),
    [anon_sym_LBRACE] = ACTIONS(3538),
    [anon_sym_RBRACE] = ACTIONS(3538),
    [anon_sym_macro_rules] = ACTIONS(3540),
    [anon_sym_BANG] = ACTIONS(3538),
    [anon_sym_LBRACK] = ACTIONS(3538),
    [anon_sym_STAR] = ACTIONS(3538),
    [anon_sym_u8] = ACTIONS(3540),
    [anon_sym_i8] = ACTIONS(3540),
    [anon_sym_u16] = ACTIONS(3540),
    [anon_sym_i16] = ACTIONS(3540),
    [anon_sym_u32] = ACTIONS(3540),
    [anon_sym_i32] = ACTIONS(3540),
    [anon_sym_u64] = ACTIONS(3540),
    [anon_sym_i64] = ACTIONS(3540),
    [anon_sym_u128] = ACTIONS(3540),
    [anon_sym_i128] = ACTIONS(3540),
    [anon_sym_isize] = ACTIONS(3540),
    [anon_sym_usize] = ACTIONS(3540),
    [anon_sym_f32] = ACTIONS(3540),
    [anon_sym_f64] = ACTIONS(3540),
    [anon_sym_bool] = ACTIONS(3540),
    [anon_sym_str] = ACTIONS(3540),
    [anon_sym_char] = ACTIONS(3540),
    [anon_sym_DASH] = ACTIONS(3538),
    [anon_sym_AMP] = ACTIONS(3538),
    [anon_sym_PIPE] = ACTIONS(3538),
    [anon_sym_LT] = ACTIONS(3538),
    [anon_sym_DOT_DOT] = ACTIONS(3538),
    [anon_sym_COLON_COLON] = ACTIONS(3538),
    [anon_sym_POUND] = ACTIONS(3538),
    [anon_sym_SQUOTE] = ACTIONS(3540),
    [anon_sym_async] = ACTIONS(3540),
    [anon_sym_auto] = ACTIONS(3540),
    [anon_sym_await] = ACTIONS(3540),
    [anon_sym_become] = ACTIONS(3540),
    [anon_sym_break] = ACTIONS(3540),
    [anon_sym_const] = ACTIONS(3540),
    [anon_sym_continue] = ACTIONS(3540),
    [anon_sym_default] = ACTIONS(3540),
    [anon_sym_enum] = ACTIONS(3540),
    [anon_sym_fn] = ACTIONS(3540),
    [anon_sym_for] = ACTIONS(3540),
    [anon_sym_gen] = ACTIONS(3540),
    [anon_sym_if] = ACTIONS(3540),
    [anon_sym_impl] = ACTIONS(3540),
    [anon_sym_let] = ACTIONS(3540),
    [anon_sym_loop] = ACTIONS(3540),
    [anon_sym_match] = ACTIONS(3540),
    [anon_sym_mod] = ACTIONS(3540),
    [anon_sym_pub] = ACTIONS(3540),
    [anon_sym_return] = ACTIONS(3540),
    [anon_sym_raw] = ACTIONS(3540),
    [anon_sym_static] = ACTIONS(3540),
    [anon_sym_struct] = ACTIONS(3540),
    [anon_sym_trait] = ACTIONS(3540),
    [anon_sym_type] = ACTIONS(3540),
    [anon_sym_union] = ACTIONS(3540),
    [anon_sym_unsafe] = ACTIONS(3540),
    [anon_sym_use] = ACTIONS(3540),
    [anon_sym_while] = ACTIONS(3540),
    [anon_sym_extern] = ACTIONS(3540),
    [anon_sym_safe] = ACTIONS(3540),
    [anon_sym_yield] = ACTIONS(3540),
    [anon_sym_move] = ACTIONS(3540),
    [anon_sym_try] = ACTIONS(3540),
    [sym_integer_literal] = ACTIONS(3538),
    [aux_sym_string_literal_token1] = ACTIONS(3538),
    [sym_char_literal] = ACTIONS(3538),
    [anon_sym_true] = ACTIONS(3540),
    [anon_sym_false] = ACTIONS(3540),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3540),
    [sym_super] = ACTIONS(3540),
    [sym_crate] = ACTIONS(3540),
    [aux_sym_metavariable_token1] = ACTIONS(3538),
    [sym__raw_string_literal_start] = ACTIONS(3538),
    [sym_float_literal] = ACTIONS(3538),
    [sym__outer_block_doc_comment_start] = ACTIONS(3538),
    [sym__outer_line_doc_comment_start] = ACTIONS(3538),
  },
  [STATE(943)] = {
    [sym_line_comment] = STATE(943),
    [sym_block_comment] = STATE(943),
    [ts_builtin_sym_end] = ACTIONS(3542),
    [sym_identifier] = ACTIONS(3544),
    [anon_sym_SEMI] = ACTIONS(3542),
    [anon_sym_macro] = ACTIONS(3544),
    [anon_sym_LPAREN] = ACTIONS(3542),
    [anon_sym_LBRACE] = ACTIONS(3542),
    [anon_sym_RBRACE] = ACTIONS(3542),
    [anon_sym_macro_rules] = ACTIONS(3544),
    [anon_sym_BANG] = ACTIONS(3542),
    [anon_sym_LBRACK] = ACTIONS(3542),
    [anon_sym_STAR] = ACTIONS(3542),
    [anon_sym_u8] = ACTIONS(3544),
    [anon_sym_i8] = ACTIONS(3544),
    [anon_sym_u16] = ACTIONS(3544),
    [anon_sym_i16] = ACTIONS(3544),
    [anon_sym_u32] = ACTIONS(3544),
    [anon_sym_i32] = ACTIONS(3544),
    [anon_sym_u64] = ACTIONS(3544),
    [anon_sym_i64] = ACTIONS(3544),
    [anon_sym_u128] = ACTIONS(3544),
    [anon_sym_i128] = ACTIONS(3544),
    [anon_sym_isize] = ACTIONS(3544),
    [anon_sym_usize] = ACTIONS(3544),
    [anon_sym_f32] = ACTIONS(3544),
    [anon_sym_f64] = ACTIONS(3544),
    [anon_sym_bool] = ACTIONS(3544),
    [anon_sym_str] = ACTIONS(3544),
    [anon_sym_char] = ACTIONS(3544),
    [anon_sym_DASH] = ACTIONS(3542),
    [anon_sym_AMP] = ACTIONS(3542),
    [anon_sym_PIPE] = ACTIONS(3542),
    [anon_sym_LT] = ACTIONS(3542),
    [anon_sym_DOT_DOT] = ACTIONS(3542),
    [anon_sym_COLON_COLON] = ACTIONS(3542),
    [anon_sym_POUND] = ACTIONS(3542),
    [anon_sym_SQUOTE] = ACTIONS(3544),
    [anon_sym_async] = ACTIONS(3544),
    [anon_sym_auto] = ACTIONS(3544),
    [anon_sym_await] = ACTIONS(3544),
    [anon_sym_become] = ACTIONS(3544),
    [anon_sym_break] = ACTIONS(3544),
    [anon_sym_const] = ACTIONS(3544),
    [anon_sym_continue] = ACTIONS(3544),
    [anon_sym_default] = ACTIONS(3544),
    [anon_sym_enum] = ACTIONS(3544),
    [anon_sym_fn] = ACTIONS(3544),
    [anon_sym_for] = ACTIONS(3544),
    [anon_sym_gen] = ACTIONS(3544),
    [anon_sym_if] = ACTIONS(3544),
    [anon_sym_impl] = ACTIONS(3544),
    [anon_sym_let] = ACTIONS(3544),
    [anon_sym_loop] = ACTIONS(3544),
    [anon_sym_match] = ACTIONS(3544),
    [anon_sym_mod] = ACTIONS(3544),
    [anon_sym_pub] = ACTIONS(3544),
    [anon_sym_return] = ACTIONS(3544),
    [anon_sym_raw] = ACTIONS(3544),
    [anon_sym_static] = ACTIONS(3544),
    [anon_sym_struct] = ACTIONS(3544),
    [anon_sym_trait] = ACTIONS(3544),
    [anon_sym_type] = ACTIONS(3544),
    [anon_sym_union] = ACTIONS(3544),
    [anon_sym_unsafe] = ACTIONS(3544),
    [anon_sym_use] = ACTIONS(3544),
    [anon_sym_while] = ACTIONS(3544),
    [anon_sym_extern] = ACTIONS(3544),
    [anon_sym_safe] = ACTIONS(3544),
    [anon_sym_yield] = ACTIONS(3544),
    [anon_sym_move] = ACTIONS(3544),
    [anon_sym_try] = ACTIONS(3544),
    [sym_integer_literal] = ACTIONS(3542),
    [aux_sym_string_literal_token1] = ACTIONS(3542),
    [sym_char_literal] = ACTIONS(3542),
    [anon_sym_true] = ACTIONS(3544),
    [anon_sym_false] = ACTIONS(3544),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3544),
    [sym_super] = ACTIONS(3544),
    [sym_crate] = ACTIONS(3544),
    [aux_sym_metavariable_token1] = ACTIONS(3542),
    [sym__raw_string_literal_start] = ACTIONS(3542),
    [sym_float_literal] = ACTIONS(3542),
    [sym__outer_block_doc_comment_start] = ACTIONS(3542),
    [sym__outer_line_doc_comment_start] = ACTIONS(3542),
  },
  [STATE(944)] = {
    [sym_line_comment] = STATE(944),
    [sym_block_comment] = STATE(944),
    [ts_builtin_sym_end] = ACTIONS(3546),
    [sym_identifier] = ACTIONS(3548),
    [anon_sym_SEMI] = ACTIONS(3546),
    [anon_sym_macro] = ACTIONS(3548),
    [anon_sym_LPAREN] = ACTIONS(3546),
    [anon_sym_LBRACE] = ACTIONS(3546),
    [anon_sym_RBRACE] = ACTIONS(3546),
    [anon_sym_macro_rules] = ACTIONS(3548),
    [anon_sym_BANG] = ACTIONS(3546),
    [anon_sym_LBRACK] = ACTIONS(3546),
    [anon_sym_STAR] = ACTIONS(3546),
    [anon_sym_u8] = ACTIONS(3548),
    [anon_sym_i8] = ACTIONS(3548),
    [anon_sym_u16] = ACTIONS(3548),
    [anon_sym_i16] = ACTIONS(3548),
    [anon_sym_u32] = ACTIONS(3548),
    [anon_sym_i32] = ACTIONS(3548),
    [anon_sym_u64] = ACTIONS(3548),
    [anon_sym_i64] = ACTIONS(3548),
    [anon_sym_u128] = ACTIONS(3548),
    [anon_sym_i128] = ACTIONS(3548),
    [anon_sym_isize] = ACTIONS(3548),
    [anon_sym_usize] = ACTIONS(3548),
    [anon_sym_f32] = ACTIONS(3548),
    [anon_sym_f64] = ACTIONS(3548),
    [anon_sym_bool] = ACTIONS(3548),
    [anon_sym_str] = ACTIONS(3548),
    [anon_sym_char] = ACTIONS(3548),
    [anon_sym_DASH] = ACTIONS(3546),
    [anon_sym_AMP] = ACTIONS(3546),
    [anon_sym_PIPE] = ACTIONS(3546),
    [anon_sym_LT] = ACTIONS(3546),
    [anon_sym_DOT_DOT] = ACTIONS(3546),
    [anon_sym_COLON_COLON] = ACTIONS(3546),
    [anon_sym_POUND] = ACTIONS(3546),
    [anon_sym_SQUOTE] = ACTIONS(3548),
    [anon_sym_async] = ACTIONS(3548),
    [anon_sym_auto] = ACTIONS(3548),
    [anon_sym_await] = ACTIONS(3548),
    [anon_sym_become] = ACTIONS(3548),
    [anon_sym_break] = ACTIONS(3548),
    [anon_sym_const] = ACTIONS(3548),
    [anon_sym_continue] = ACTIONS(3548),
    [anon_sym_default] = ACTIONS(3548),
    [anon_sym_enum] = ACTIONS(3548),
    [anon_sym_fn] = ACTIONS(3548),
    [anon_sym_for] = ACTIONS(3548),
    [anon_sym_gen] = ACTIONS(3548),
    [anon_sym_if] = ACTIONS(3548),
    [anon_sym_impl] = ACTIONS(3548),
    [anon_sym_let] = ACTIONS(3548),
    [anon_sym_loop] = ACTIONS(3548),
    [anon_sym_match] = ACTIONS(3548),
    [anon_sym_mod] = ACTIONS(3548),
    [anon_sym_pub] = ACTIONS(3548),
    [anon_sym_return] = ACTIONS(3548),
    [anon_sym_raw] = ACTIONS(3548),
    [anon_sym_static] = ACTIONS(3548),
    [anon_sym_struct] = ACTIONS(3548),
    [anon_sym_trait] = ACTIONS(3548),
    [anon_sym_type] = ACTIONS(3548),
    [anon_sym_union] = ACTIONS(3548),
    [anon_sym_unsafe] = ACTIONS(3548),
    [anon_sym_use] = ACTIONS(3548),
    [anon_sym_while] = ACTIONS(3548),
    [anon_sym_extern] = ACTIONS(3548),
    [anon_sym_safe] = ACTIONS(3548),
    [anon_sym_yield] = ACTIONS(3548),
    [anon_sym_move] = ACTIONS(3548),
    [anon_sym_try] = ACTIONS(3548),
    [sym_integer_literal] = ACTIONS(3546),
    [aux_sym_string_literal_token1] = ACTIONS(3546),
    [sym_char_literal] = ACTIONS(3546),
    [anon_sym_true] = ACTIONS(3548),
    [anon_sym_false] = ACTIONS(3548),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3548),
    [sym_super] = ACTIONS(3548),
    [sym_crate] = ACTIONS(3548),
    [aux_sym_metavariable_token1] = ACTIONS(3546),
    [sym__raw_string_literal_start] = ACTIONS(3546),
    [sym_float_literal] = ACTIONS(3546),
    [sym__outer_block_doc_comment_start] = ACTIONS(3546),
    [sym__outer_line_doc_comment_start] = ACTIONS(3546),
  },
  [STATE(945)] = {
    [sym_line_comment] = STATE(945),
    [sym_block_comment] = STATE(945),
    [ts_builtin_sym_end] = ACTIONS(3550),
    [sym_identifier] = ACTIONS(3552),
    [anon_sym_SEMI] = ACTIONS(3550),
    [anon_sym_macro] = ACTIONS(3552),
    [anon_sym_LPAREN] = ACTIONS(3550),
    [anon_sym_LBRACE] = ACTIONS(3550),
    [anon_sym_RBRACE] = ACTIONS(3550),
    [anon_sym_macro_rules] = ACTIONS(3552),
    [anon_sym_BANG] = ACTIONS(3550),
    [anon_sym_LBRACK] = ACTIONS(3550),
    [anon_sym_STAR] = ACTIONS(3550),
    [anon_sym_u8] = ACTIONS(3552),
    [anon_sym_i8] = ACTIONS(3552),
    [anon_sym_u16] = ACTIONS(3552),
    [anon_sym_i16] = ACTIONS(3552),
    [anon_sym_u32] = ACTIONS(3552),
    [anon_sym_i32] = ACTIONS(3552),
    [anon_sym_u64] = ACTIONS(3552),
    [anon_sym_i64] = ACTIONS(3552),
    [anon_sym_u128] = ACTIONS(3552),
    [anon_sym_i128] = ACTIONS(3552),
    [anon_sym_isize] = ACTIONS(3552),
    [anon_sym_usize] = ACTIONS(3552),
    [anon_sym_f32] = ACTIONS(3552),
    [anon_sym_f64] = ACTIONS(3552),
    [anon_sym_bool] = ACTIONS(3552),
    [anon_sym_str] = ACTIONS(3552),
    [anon_sym_char] = ACTIONS(3552),
    [anon_sym_DASH] = ACTIONS(3550),
    [anon_sym_AMP] = ACTIONS(3550),
    [anon_sym_PIPE] = ACTIONS(3550),
    [anon_sym_LT] = ACTIONS(3550),
    [anon_sym_DOT_DOT] = ACTIONS(3550),
    [anon_sym_COLON_COLON] = ACTIONS(3550),
    [anon_sym_POUND] = ACTIONS(3550),
    [anon_sym_SQUOTE] = ACTIONS(3552),
    [anon_sym_async] = ACTIONS(3552),
    [anon_sym_auto] = ACTIONS(3552),
    [anon_sym_await] = ACTIONS(3552),
    [anon_sym_become] = ACTIONS(3552),
    [anon_sym_break] = ACTIONS(3552),
    [anon_sym_const] = ACTIONS(3552),
    [anon_sym_continue] = ACTIONS(3552),
    [anon_sym_default] = ACTIONS(3552),
    [anon_sym_enum] = ACTIONS(3552),
    [anon_sym_fn] = ACTIONS(3552),
    [anon_sym_for] = ACTIONS(3552),
    [anon_sym_gen] = ACTIONS(3552),
    [anon_sym_if] = ACTIONS(3552),
    [anon_sym_impl] = ACTIONS(3552),
    [anon_sym_let] = ACTIONS(3552),
    [anon_sym_loop] = ACTIONS(3552),
    [anon_sym_match] = ACTIONS(3552),
    [anon_sym_mod] = ACTIONS(3552),
    [anon_sym_pub] = ACTIONS(3552),
    [anon_sym_return] = ACTIONS(3552),
    [anon_sym_raw] = ACTIONS(3552),
    [anon_sym_static] = ACTIONS(3552),
    [anon_sym_struct] = ACTIONS(3552),
    [anon_sym_trait] = ACTIONS(3552),
    [anon_sym_type] = ACTIONS(3552),
    [anon_sym_union] = ACTIONS(3552),
    [anon_sym_unsafe] = ACTIONS(3552),
    [anon_sym_use] = ACTIONS(3552),
    [anon_sym_while] = ACTIONS(3552),
    [anon_sym_extern] = ACTIONS(3552),
    [anon_sym_safe] = ACTIONS(3552),
    [anon_sym_yield] = ACTIONS(3552),
    [anon_sym_move] = ACTIONS(3552),
    [anon_sym_try] = ACTIONS(3552),
    [sym_integer_literal] = ACTIONS(3550),
    [aux_sym_string_literal_token1] = ACTIONS(3550),
    [sym_char_literal] = ACTIONS(3550),
    [anon_sym_true] = ACTIONS(3552),
    [anon_sym_false] = ACTIONS(3552),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3552),
    [sym_super] = ACTIONS(3552),
    [sym_crate] = ACTIONS(3552),
    [aux_sym_metavariable_token1] = ACTIONS(3550),
    [sym__raw_string_literal_start] = ACTIONS(3550),
    [sym_float_literal] = ACTIONS(3550),
    [sym__outer_block_doc_comment_start] = ACTIONS(3550),
    [sym__outer_line_doc_comment_start] = ACTIONS(3550),
  },
  [STATE(946)] = {
    [sym_line_comment] = STATE(946),
    [sym_block_comment] = STATE(946),
    [ts_builtin_sym_end] = ACTIONS(3554),
    [sym_identifier] = ACTIONS(3556),
    [anon_sym_SEMI] = ACTIONS(3554),
    [anon_sym_macro] = ACTIONS(3556),
    [anon_sym_LPAREN] = ACTIONS(3554),
    [anon_sym_LBRACE] = ACTIONS(3554),
    [anon_sym_RBRACE] = ACTIONS(3554),
    [anon_sym_macro_rules] = ACTIONS(3556),
    [anon_sym_BANG] = ACTIONS(3554),
    [anon_sym_LBRACK] = ACTIONS(3554),
    [anon_sym_STAR] = ACTIONS(3554),
    [anon_sym_u8] = ACTIONS(3556),
    [anon_sym_i8] = ACTIONS(3556),
    [anon_sym_u16] = ACTIONS(3556),
    [anon_sym_i16] = ACTIONS(3556),
    [anon_sym_u32] = ACTIONS(3556),
    [anon_sym_i32] = ACTIONS(3556),
    [anon_sym_u64] = ACTIONS(3556),
    [anon_sym_i64] = ACTIONS(3556),
    [anon_sym_u128] = ACTIONS(3556),
    [anon_sym_i128] = ACTIONS(3556),
    [anon_sym_isize] = ACTIONS(3556),
    [anon_sym_usize] = ACTIONS(3556),
    [anon_sym_f32] = ACTIONS(3556),
    [anon_sym_f64] = ACTIONS(3556),
    [anon_sym_bool] = ACTIONS(3556),
    [anon_sym_str] = ACTIONS(3556),
    [anon_sym_char] = ACTIONS(3556),
    [anon_sym_DASH] = ACTIONS(3554),
    [anon_sym_AMP] = ACTIONS(3554),
    [anon_sym_PIPE] = ACTIONS(3554),
    [anon_sym_LT] = ACTIONS(3554),
    [anon_sym_DOT_DOT] = ACTIONS(3554),
    [anon_sym_COLON_COLON] = ACTIONS(3554),
    [anon_sym_POUND] = ACTIONS(3554),
    [anon_sym_SQUOTE] = ACTIONS(3556),
    [anon_sym_async] = ACTIONS(3556),
    [anon_sym_auto] = ACTIONS(3556),
    [anon_sym_await] = ACTIONS(3556),
    [anon_sym_become] = ACTIONS(3556),
    [anon_sym_break] = ACTIONS(3556),
    [anon_sym_const] = ACTIONS(3556),
    [anon_sym_continue] = ACTIONS(3556),
    [anon_sym_default] = ACTIONS(3556),
    [anon_sym_enum] = ACTIONS(3556),
    [anon_sym_fn] = ACTIONS(3556),
    [anon_sym_for] = ACTIONS(3556),
    [anon_sym_gen] = ACTIONS(3556),
    [anon_sym_if] = ACTIONS(3556),
    [anon_sym_impl] = ACTIONS(3556),
    [anon_sym_let] = ACTIONS(3556),
    [anon_sym_loop] = ACTIONS(3556),
    [anon_sym_match] = ACTIONS(3556),
    [anon_sym_mod] = ACTIONS(3556),
    [anon_sym_pub] = ACTIONS(3556),
    [anon_sym_return] = ACTIONS(3556),
    [anon_sym_raw] = ACTIONS(3556),
    [anon_sym_static] = ACTIONS(3556),
    [anon_sym_struct] = ACTIONS(3556),
    [anon_sym_trait] = ACTIONS(3556),
    [anon_sym_type] = ACTIONS(3556),
    [anon_sym_union] = ACTIONS(3556),
    [anon_sym_unsafe] = ACTIONS(3556),
    [anon_sym_use] = ACTIONS(3556),
    [anon_sym_while] = ACTIONS(3556),
    [anon_sym_extern] = ACTIONS(3556),
    [anon_sym_safe] = ACTIONS(3556),
    [anon_sym_yield] = ACTIONS(3556),
    [anon_sym_move] = ACTIONS(3556),
    [anon_sym_try] = ACTIONS(3556),
    [sym_integer_literal] = ACTIONS(3554),
    [aux_sym_string_literal_token1] = ACTIONS(3554),
    [sym_char_literal] = ACTIONS(3554),
    [anon_sym_true] = ACTIONS(3556),
    [anon_sym_false] = ACTIONS(3556),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3556),
    [sym_super] = ACTIONS(3556),
    [sym_crate] = ACTIONS(3556),
    [aux_sym_metavariable_token1] = ACTIONS(3554),
    [sym__raw_string_literal_start] = ACTIONS(3554),
    [sym_float_literal] = ACTIONS(3554),
    [sym__outer_block_doc_comment_start] = ACTIONS(3554),
    [sym__outer_line_doc_comment_start] = ACTIONS(3554),
  },
  [STATE(947)] = {
    [sym_line_comment] = STATE(947),
    [sym_block_comment] = STATE(947),
    [ts_builtin_sym_end] = ACTIONS(3558),
    [sym_identifier] = ACTIONS(3560),
    [anon_sym_SEMI] = ACTIONS(3558),
    [anon_sym_macro] = ACTIONS(3560),
    [anon_sym_LPAREN] = ACTIONS(3558),
    [anon_sym_LBRACE] = ACTIONS(3558),
    [anon_sym_RBRACE] = ACTIONS(3558),
    [anon_sym_macro_rules] = ACTIONS(3560),
    [anon_sym_BANG] = ACTIONS(3558),
    [anon_sym_LBRACK] = ACTIONS(3558),
    [anon_sym_STAR] = ACTIONS(3558),
    [anon_sym_u8] = ACTIONS(3560),
    [anon_sym_i8] = ACTIONS(3560),
    [anon_sym_u16] = ACTIONS(3560),
    [anon_sym_i16] = ACTIONS(3560),
    [anon_sym_u32] = ACTIONS(3560),
    [anon_sym_i32] = ACTIONS(3560),
    [anon_sym_u64] = ACTIONS(3560),
    [anon_sym_i64] = ACTIONS(3560),
    [anon_sym_u128] = ACTIONS(3560),
    [anon_sym_i128] = ACTIONS(3560),
    [anon_sym_isize] = ACTIONS(3560),
    [anon_sym_usize] = ACTIONS(3560),
    [anon_sym_f32] = ACTIONS(3560),
    [anon_sym_f64] = ACTIONS(3560),
    [anon_sym_bool] = ACTIONS(3560),
    [anon_sym_str] = ACTIONS(3560),
    [anon_sym_char] = ACTIONS(3560),
    [anon_sym_DASH] = ACTIONS(3558),
    [anon_sym_AMP] = ACTIONS(3558),
    [anon_sym_PIPE] = ACTIONS(3558),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_DOT_DOT] = ACTIONS(3558),
    [anon_sym_COLON_COLON] = ACTIONS(3558),
    [anon_sym_POUND] = ACTIONS(3558),
    [anon_sym_SQUOTE] = ACTIONS(3560),
    [anon_sym_async] = ACTIONS(3560),
    [anon_sym_auto] = ACTIONS(3560),
    [anon_sym_await] = ACTIONS(3560),
    [anon_sym_become] = ACTIONS(3560),
    [anon_sym_break] = ACTIONS(3560),
    [anon_sym_const] = ACTIONS(3560),
    [anon_sym_continue] = ACTIONS(3560),
    [anon_sym_default] = ACTIONS(3560),
    [anon_sym_enum] = ACTIONS(3560),
    [anon_sym_fn] = ACTIONS(3560),
    [anon_sym_for] = ACTIONS(3560),
    [anon_sym_gen] = ACTIONS(3560),
    [anon_sym_if] = ACTIONS(3560),
    [anon_sym_impl] = ACTIONS(3560),
    [anon_sym_let] = ACTIONS(3560),
    [anon_sym_loop] = ACTIONS(3560),
    [anon_sym_match] = ACTIONS(3560),
    [anon_sym_mod] = ACTIONS(3560),
    [anon_sym_pub] = ACTIONS(3560),
    [anon_sym_return] = ACTIONS(3560),
    [anon_sym_raw] = ACTIONS(3560),
    [anon_sym_static] = ACTIONS(3560),
    [anon_sym_struct] = ACTIONS(3560),
    [anon_sym_trait] = ACTIONS(3560),
    [anon_sym_type] = ACTIONS(3560),
    [anon_sym_union] = ACTIONS(3560),
    [anon_sym_unsafe] = ACTIONS(3560),
    [anon_sym_use] = ACTIONS(3560),
    [anon_sym_while] = ACTIONS(3560),
    [anon_sym_extern] = ACTIONS(3560),
    [anon_sym_safe] = ACTIONS(3560),
    [anon_sym_yield] = ACTIONS(3560),
    [anon_sym_move] = ACTIONS(3560),
    [anon_sym_try] = ACTIONS(3560),
    [sym_integer_literal] = ACTIONS(3558),
    [aux_sym_string_literal_token1] = ACTIONS(3558),
    [sym_char_literal] = ACTIONS(3558),
    [anon_sym_true] = ACTIONS(3560),
    [anon_sym_false] = ACTIONS(3560),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3560),
    [sym_super] = ACTIONS(3560),
    [sym_crate] = ACTIONS(3560),
    [aux_sym_metavariable_token1] = ACTIONS(3558),
    [sym__raw_string_literal_start] = ACTIONS(3558),
    [sym_float_literal] = ACTIONS(3558),
    [sym__outer_block_doc_comment_start] = ACTIONS(3558),
    [sym__outer_line_doc_comment_start] = ACTIONS(3558),
  },
  [STATE(948)] = {
    [sym_line_comment] = STATE(948),
    [sym_block_comment] = STATE(948),
    [ts_builtin_sym_end] = ACTIONS(3562),
    [sym_identifier] = ACTIONS(3564),
    [anon_sym_SEMI] = ACTIONS(3562),
    [anon_sym_macro] = ACTIONS(3564),
    [anon_sym_LPAREN] = ACTIONS(3562),
    [anon_sym_LBRACE] = ACTIONS(3562),
    [anon_sym_RBRACE] = ACTIONS(3562),
    [anon_sym_macro_rules] = ACTIONS(3564),
    [anon_sym_BANG] = ACTIONS(3562),
    [anon_sym_LBRACK] = ACTIONS(3562),
    [anon_sym_STAR] = ACTIONS(3562),
    [anon_sym_u8] = ACTIONS(3564),
    [anon_sym_i8] = ACTIONS(3564),
    [anon_sym_u16] = ACTIONS(3564),
    [anon_sym_i16] = ACTIONS(3564),
    [anon_sym_u32] = ACTIONS(3564),
    [anon_sym_i32] = ACTIONS(3564),
    [anon_sym_u64] = ACTIONS(3564),
    [anon_sym_i64] = ACTIONS(3564),
    [anon_sym_u128] = ACTIONS(3564),
    [anon_sym_i128] = ACTIONS(3564),
    [anon_sym_isize] = ACTIONS(3564),
    [anon_sym_usize] = ACTIONS(3564),
    [anon_sym_f32] = ACTIONS(3564),
    [anon_sym_f64] = ACTIONS(3564),
    [anon_sym_bool] = ACTIONS(3564),
    [anon_sym_str] = ACTIONS(3564),
    [anon_sym_char] = ACTIONS(3564),
    [anon_sym_DASH] = ACTIONS(3562),
    [anon_sym_AMP] = ACTIONS(3562),
    [anon_sym_PIPE] = ACTIONS(3562),
    [anon_sym_LT] = ACTIONS(3562),
    [anon_sym_DOT_DOT] = ACTIONS(3562),
    [anon_sym_COLON_COLON] = ACTIONS(3562),
    [anon_sym_POUND] = ACTIONS(3562),
    [anon_sym_SQUOTE] = ACTIONS(3564),
    [anon_sym_async] = ACTIONS(3564),
    [anon_sym_auto] = ACTIONS(3564),
    [anon_sym_await] = ACTIONS(3564),
    [anon_sym_become] = ACTIONS(3564),
    [anon_sym_break] = ACTIONS(3564),
    [anon_sym_const] = ACTIONS(3564),
    [anon_sym_continue] = ACTIONS(3564),
    [anon_sym_default] = ACTIONS(3564),
    [anon_sym_enum] = ACTIONS(3564),
    [anon_sym_fn] = ACTIONS(3564),
    [anon_sym_for] = ACTIONS(3564),
    [anon_sym_gen] = ACTIONS(3564),
    [anon_sym_if] = ACTIONS(3564),
    [anon_sym_impl] = ACTIONS(3564),
    [anon_sym_let] = ACTIONS(3564),
    [anon_sym_loop] = ACTIONS(3564),
    [anon_sym_match] = ACTIONS(3564),
    [anon_sym_mod] = ACTIONS(3564),
    [anon_sym_pub] = ACTIONS(3564),
    [anon_sym_return] = ACTIONS(3564),
    [anon_sym_raw] = ACTIONS(3564),
    [anon_sym_static] = ACTIONS(3564),
    [anon_sym_struct] = ACTIONS(3564),
    [anon_sym_trait] = ACTIONS(3564),
    [anon_sym_type] = ACTIONS(3564),
    [anon_sym_union] = ACTIONS(3564),
    [anon_sym_unsafe] = ACTIONS(3564),
    [anon_sym_use] = ACTIONS(3564),
    [anon_sym_while] = ACTIONS(3564),
    [anon_sym_extern] = ACTIONS(3564),
    [anon_sym_safe] = ACTIONS(3564),
    [anon_sym_yield] = ACTIONS(3564),
    [anon_sym_move] = ACTIONS(3564),
    [anon_sym_try] = ACTIONS(3564),
    [sym_integer_literal] = ACTIONS(3562),
    [aux_sym_string_literal_token1] = ACTIONS(3562),
    [sym_char_literal] = ACTIONS(3562),
    [anon_sym_true] = ACTIONS(3564),
    [anon_sym_false] = ACTIONS(3564),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3564),
    [sym_super] = ACTIONS(3564),
    [sym_crate] = ACTIONS(3564),
    [aux_sym_metavariable_token1] = ACTIONS(3562),
    [sym__raw_string_literal_start] = ACTIONS(3562),
    [sym_float_literal] = ACTIONS(3562),
    [sym__outer_block_doc_comment_start] = ACTIONS(3562),
    [sym__outer_line_doc_comment_start] = ACTIONS(3562),
  },
  [STATE(949)] = {
    [sym_line_comment] = STATE(949),
    [sym_block_comment] = STATE(949),
    [ts_builtin_sym_end] = ACTIONS(3566),
    [sym_identifier] = ACTIONS(3568),
    [anon_sym_SEMI] = ACTIONS(3566),
    [anon_sym_macro] = ACTIONS(3568),
    [anon_sym_LPAREN] = ACTIONS(3566),
    [anon_sym_LBRACE] = ACTIONS(3566),
    [anon_sym_RBRACE] = ACTIONS(3566),
    [anon_sym_macro_rules] = ACTIONS(3568),
    [anon_sym_BANG] = ACTIONS(3566),
    [anon_sym_LBRACK] = ACTIONS(3566),
    [anon_sym_STAR] = ACTIONS(3566),
    [anon_sym_u8] = ACTIONS(3568),
    [anon_sym_i8] = ACTIONS(3568),
    [anon_sym_u16] = ACTIONS(3568),
    [anon_sym_i16] = ACTIONS(3568),
    [anon_sym_u32] = ACTIONS(3568),
    [anon_sym_i32] = ACTIONS(3568),
    [anon_sym_u64] = ACTIONS(3568),
    [anon_sym_i64] = ACTIONS(3568),
    [anon_sym_u128] = ACTIONS(3568),
    [anon_sym_i128] = ACTIONS(3568),
    [anon_sym_isize] = ACTIONS(3568),
    [anon_sym_usize] = ACTIONS(3568),
    [anon_sym_f32] = ACTIONS(3568),
    [anon_sym_f64] = ACTIONS(3568),
    [anon_sym_bool] = ACTIONS(3568),
    [anon_sym_str] = ACTIONS(3568),
    [anon_sym_char] = ACTIONS(3568),
    [anon_sym_DASH] = ACTIONS(3566),
    [anon_sym_AMP] = ACTIONS(3566),
    [anon_sym_PIPE] = ACTIONS(3566),
    [anon_sym_LT] = ACTIONS(3566),
    [anon_sym_DOT_DOT] = ACTIONS(3566),
    [anon_sym_COLON_COLON] = ACTIONS(3566),
    [anon_sym_POUND] = ACTIONS(3566),
    [anon_sym_SQUOTE] = ACTIONS(3568),
    [anon_sym_async] = ACTIONS(3568),
    [anon_sym_auto] = ACTIONS(3568),
    [anon_sym_await] = ACTIONS(3568),
    [anon_sym_become] = ACTIONS(3568),
    [anon_sym_break] = ACTIONS(3568),
    [anon_sym_const] = ACTIONS(3568),
    [anon_sym_continue] = ACTIONS(3568),
    [anon_sym_default] = ACTIONS(3568),
    [anon_sym_enum] = ACTIONS(3568),
    [anon_sym_fn] = ACTIONS(3568),
    [anon_sym_for] = ACTIONS(3568),
    [anon_sym_gen] = ACTIONS(3568),
    [anon_sym_if] = ACTIONS(3568),
    [anon_sym_impl] = ACTIONS(3568),
    [anon_sym_let] = ACTIONS(3568),
    [anon_sym_loop] = ACTIONS(3568),
    [anon_sym_match] = ACTIONS(3568),
    [anon_sym_mod] = ACTIONS(3568),
    [anon_sym_pub] = ACTIONS(3568),
    [anon_sym_return] = ACTIONS(3568),
    [anon_sym_raw] = ACTIONS(3568),
    [anon_sym_static] = ACTIONS(3568),
    [anon_sym_struct] = ACTIONS(3568),
    [anon_sym_trait] = ACTIONS(3568),
    [anon_sym_type] = ACTIONS(3568),
    [anon_sym_union] = ACTIONS(3568),
    [anon_sym_unsafe] = ACTIONS(3568),
    [anon_sym_use] = ACTIONS(3568),
    [anon_sym_while] = ACTIONS(3568),
    [anon_sym_extern] = ACTIONS(3568),
    [anon_sym_safe] = ACTIONS(3568),
    [anon_sym_yield] = ACTIONS(3568),
    [anon_sym_move] = ACTIONS(3568),
    [anon_sym_try] = ACTIONS(3568),
    [sym_integer_literal] = ACTIONS(3566),
    [aux_sym_string_literal_token1] = ACTIONS(3566),
    [sym_char_literal] = ACTIONS(3566),
    [anon_sym_true] = ACTIONS(3568),
    [anon_sym_false] = ACTIONS(3568),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3568),
    [sym_super] = ACTIONS(3568),
    [sym_crate] = ACTIONS(3568),
    [aux_sym_metavariable_token1] = ACTIONS(3566),
    [sym__raw_string_literal_start] = ACTIONS(3566),
    [sym_float_literal] = ACTIONS(3566),
    [sym__outer_block_doc_comment_start] = ACTIONS(3566),
    [sym__outer_line_doc_comment_start] = ACTIONS(3566),
  },
  [STATE(950)] = {
    [sym_line_comment] = STATE(950),
    [sym_block_comment] = STATE(950),
    [ts_builtin_sym_end] = ACTIONS(3570),
    [sym_identifier] = ACTIONS(3572),
    [anon_sym_SEMI] = ACTIONS(3570),
    [anon_sym_macro] = ACTIONS(3572),
    [anon_sym_LPAREN] = ACTIONS(3570),
    [anon_sym_LBRACE] = ACTIONS(3570),
    [anon_sym_RBRACE] = ACTIONS(3570),
    [anon_sym_macro_rules] = ACTIONS(3572),
    [anon_sym_BANG] = ACTIONS(3570),
    [anon_sym_LBRACK] = ACTIONS(3570),
    [anon_sym_STAR] = ACTIONS(3570),
    [anon_sym_u8] = ACTIONS(3572),
    [anon_sym_i8] = ACTIONS(3572),
    [anon_sym_u16] = ACTIONS(3572),
    [anon_sym_i16] = ACTIONS(3572),
    [anon_sym_u32] = ACTIONS(3572),
    [anon_sym_i32] = ACTIONS(3572),
    [anon_sym_u64] = ACTIONS(3572),
    [anon_sym_i64] = ACTIONS(3572),
    [anon_sym_u128] = ACTIONS(3572),
    [anon_sym_i128] = ACTIONS(3572),
    [anon_sym_isize] = ACTIONS(3572),
    [anon_sym_usize] = ACTIONS(3572),
    [anon_sym_f32] = ACTIONS(3572),
    [anon_sym_f64] = ACTIONS(3572),
    [anon_sym_bool] = ACTIONS(3572),
    [anon_sym_str] = ACTIONS(3572),
    [anon_sym_char] = ACTIONS(3572),
    [anon_sym_DASH] = ACTIONS(3570),
    [anon_sym_AMP] = ACTIONS(3570),
    [anon_sym_PIPE] = ACTIONS(3570),
    [anon_sym_LT] = ACTIONS(3570),
    [anon_sym_DOT_DOT] = ACTIONS(3570),
    [anon_sym_COLON_COLON] = ACTIONS(3570),
    [anon_sym_POUND] = ACTIONS(3570),
    [anon_sym_SQUOTE] = ACTIONS(3572),
    [anon_sym_async] = ACTIONS(3572),
    [anon_sym_auto] = ACTIONS(3572),
    [anon_sym_await] = ACTIONS(3572),
    [anon_sym_become] = ACTIONS(3572),
    [anon_sym_break] = ACTIONS(3572),
    [anon_sym_const] = ACTIONS(3572),
    [anon_sym_continue] = ACTIONS(3572),
    [anon_sym_default] = ACTIONS(3572),
    [anon_sym_enum] = ACTIONS(3572),
    [anon_sym_fn] = ACTIONS(3572),
    [anon_sym_for] = ACTIONS(3572),
    [anon_sym_gen] = ACTIONS(3572),
    [anon_sym_if] = ACTIONS(3572),
    [anon_sym_impl] = ACTIONS(3572),
    [anon_sym_let] = ACTIONS(3572),
    [anon_sym_loop] = ACTIONS(3572),
    [anon_sym_match] = ACTIONS(3572),
    [anon_sym_mod] = ACTIONS(3572),
    [anon_sym_pub] = ACTIONS(3572),
    [anon_sym_return] = ACTIONS(3572),
    [anon_sym_raw] = ACTIONS(3572),
    [anon_sym_static] = ACTIONS(3572),
    [anon_sym_struct] = ACTIONS(3572),
    [anon_sym_trait] = ACTIONS(3572),
    [anon_sym_type] = ACTIONS(3572),
    [anon_sym_union] = ACTIONS(3572),
    [anon_sym_unsafe] = ACTIONS(3572),
    [anon_sym_use] = ACTIONS(3572),
    [anon_sym_while] = ACTIONS(3572),
    [anon_sym_extern] = ACTIONS(3572),
    [anon_sym_safe] = ACTIONS(3572),
    [anon_sym_yield] = ACTIONS(3572),
    [anon_sym_move] = ACTIONS(3572),
    [anon_sym_try] = ACTIONS(3572),
    [sym_integer_literal] = ACTIONS(3570),
    [aux_sym_string_literal_token1] = ACTIONS(3570),
    [sym_char_literal] = ACTIONS(3570),
    [anon_sym_true] = ACTIONS(3572),
    [anon_sym_false] = ACTIONS(3572),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3572),
    [sym_super] = ACTIONS(3572),
    [sym_crate] = ACTIONS(3572),
    [aux_sym_metavariable_token1] = ACTIONS(3570),
    [sym__raw_string_literal_start] = ACTIONS(3570),
    [sym_float_literal] = ACTIONS(3570),
    [sym__outer_block_doc_comment_start] = ACTIONS(3570),
    [sym__outer_line_doc_comment_start] = ACTIONS(3570),
  },
  [STATE(951)] = {
    [sym_line_comment] = STATE(951),
    [sym_block_comment] = STATE(951),
    [ts_builtin_sym_end] = ACTIONS(3574),
    [sym_identifier] = ACTIONS(3576),
    [anon_sym_SEMI] = ACTIONS(3574),
    [anon_sym_macro] = ACTIONS(3576),
    [anon_sym_LPAREN] = ACTIONS(3574),
    [anon_sym_LBRACE] = ACTIONS(3574),
    [anon_sym_RBRACE] = ACTIONS(3574),
    [anon_sym_macro_rules] = ACTIONS(3576),
    [anon_sym_BANG] = ACTIONS(3574),
    [anon_sym_LBRACK] = ACTIONS(3574),
    [anon_sym_STAR] = ACTIONS(3574),
    [anon_sym_u8] = ACTIONS(3576),
    [anon_sym_i8] = ACTIONS(3576),
    [anon_sym_u16] = ACTIONS(3576),
    [anon_sym_i16] = ACTIONS(3576),
    [anon_sym_u32] = ACTIONS(3576),
    [anon_sym_i32] = ACTIONS(3576),
    [anon_sym_u64] = ACTIONS(3576),
    [anon_sym_i64] = ACTIONS(3576),
    [anon_sym_u128] = ACTIONS(3576),
    [anon_sym_i128] = ACTIONS(3576),
    [anon_sym_isize] = ACTIONS(3576),
    [anon_sym_usize] = ACTIONS(3576),
    [anon_sym_f32] = ACTIONS(3576),
    [anon_sym_f64] = ACTIONS(3576),
    [anon_sym_bool] = ACTIONS(3576),
    [anon_sym_str] = ACTIONS(3576),
    [anon_sym_char] = ACTIONS(3576),
    [anon_sym_DASH] = ACTIONS(3574),
    [anon_sym_AMP] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3574),
    [anon_sym_LT] = ACTIONS(3574),
    [anon_sym_DOT_DOT] = ACTIONS(3574),
    [anon_sym_COLON_COLON] = ACTIONS(3574),
    [anon_sym_POUND] = ACTIONS(3574),
    [anon_sym_SQUOTE] = ACTIONS(3576),
    [anon_sym_async] = ACTIONS(3576),
    [anon_sym_auto] = ACTIONS(3576),
    [anon_sym_await] = ACTIONS(3576),
    [anon_sym_become] = ACTIONS(3576),
    [anon_sym_break] = ACTIONS(3576),
    [anon_sym_const] = ACTIONS(3576),
    [anon_sym_continue] = ACTIONS(3576),
    [anon_sym_default] = ACTIONS(3576),
    [anon_sym_enum] = ACTIONS(3576),
    [anon_sym_fn] = ACTIONS(3576),
    [anon_sym_for] = ACTIONS(3576),
    [anon_sym_gen] = ACTIONS(3576),
    [anon_sym_if] = ACTIONS(3576),
    [anon_sym_impl] = ACTIONS(3576),
    [anon_sym_let] = ACTIONS(3576),
    [anon_sym_loop] = ACTIONS(3576),
    [anon_sym_match] = ACTIONS(3576),
    [anon_sym_mod] = ACTIONS(3576),
    [anon_sym_pub] = ACTIONS(3576),
    [anon_sym_return] = ACTIONS(3576),
    [anon_sym_raw] = ACTIONS(3576),
    [anon_sym_static] = ACTIONS(3576),
    [anon_sym_struct] = ACTIONS(3576),
    [anon_sym_trait] = ACTIONS(3576),
    [anon_sym_type] = ACTIONS(3576),
    [anon_sym_union] = ACTIONS(3576),
    [anon_sym_unsafe] = ACTIONS(3576),
    [anon_sym_use] = ACTIONS(3576),
    [anon_sym_while] = ACTIONS(3576),
    [anon_sym_extern] = ACTIONS(3576),
    [anon_sym_safe] = ACTIONS(3576),
    [anon_sym_yield] = ACTIONS(3576),
    [anon_sym_move] = ACTIONS(3576),
    [anon_sym_try] = ACTIONS(3576),
    [sym_integer_literal] = ACTIONS(3574),
    [aux_sym_string_literal_token1] = ACTIONS(3574),
    [sym_char_literal] = ACTIONS(3574),
    [anon_sym_true] = ACTIONS(3576),
    [anon_sym_false] = ACTIONS(3576),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3576),
    [sym_super] = ACTIONS(3576),
    [sym_crate] = ACTIONS(3576),
    [aux_sym_metavariable_token1] = ACTIONS(3574),
    [sym__raw_string_literal_start] = ACTIONS(3574),
    [sym_float_literal] = ACTIONS(3574),
    [sym__outer_block_doc_comment_start] = ACTIONS(3574),
    [sym__outer_line_doc_comment_start] = ACTIONS(3574),
  },
  [STATE(952)] = {
    [sym_line_comment] = STATE(952),
    [sym_block_comment] = STATE(952),
    [ts_builtin_sym_end] = ACTIONS(3578),
    [sym_identifier] = ACTIONS(3580),
    [anon_sym_SEMI] = ACTIONS(3578),
    [anon_sym_macro] = ACTIONS(3580),
    [anon_sym_LPAREN] = ACTIONS(3578),
    [anon_sym_LBRACE] = ACTIONS(3578),
    [anon_sym_RBRACE] = ACTIONS(3578),
    [anon_sym_macro_rules] = ACTIONS(3580),
    [anon_sym_BANG] = ACTIONS(3578),
    [anon_sym_LBRACK] = ACTIONS(3578),
    [anon_sym_STAR] = ACTIONS(3578),
    [anon_sym_u8] = ACTIONS(3580),
    [anon_sym_i8] = ACTIONS(3580),
    [anon_sym_u16] = ACTIONS(3580),
    [anon_sym_i16] = ACTIONS(3580),
    [anon_sym_u32] = ACTIONS(3580),
    [anon_sym_i32] = ACTIONS(3580),
    [anon_sym_u64] = ACTIONS(3580),
    [anon_sym_i64] = ACTIONS(3580),
    [anon_sym_u128] = ACTIONS(3580),
    [anon_sym_i128] = ACTIONS(3580),
    [anon_sym_isize] = ACTIONS(3580),
    [anon_sym_usize] = ACTIONS(3580),
    [anon_sym_f32] = ACTIONS(3580),
    [anon_sym_f64] = ACTIONS(3580),
    [anon_sym_bool] = ACTIONS(3580),
    [anon_sym_str] = ACTIONS(3580),
    [anon_sym_char] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3578),
    [anon_sym_AMP] = ACTIONS(3578),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_LT] = ACTIONS(3578),
    [anon_sym_DOT_DOT] = ACTIONS(3578),
    [anon_sym_COLON_COLON] = ACTIONS(3578),
    [anon_sym_POUND] = ACTIONS(3578),
    [anon_sym_SQUOTE] = ACTIONS(3580),
    [anon_sym_async] = ACTIONS(3580),
    [anon_sym_auto] = ACTIONS(3580),
    [anon_sym_await] = ACTIONS(3580),
    [anon_sym_become] = ACTIONS(3580),
    [anon_sym_break] = ACTIONS(3580),
    [anon_sym_const] = ACTIONS(3580),
    [anon_sym_continue] = ACTIONS(3580),
    [anon_sym_default] = ACTIONS(3580),
    [anon_sym_enum] = ACTIONS(3580),
    [anon_sym_fn] = ACTIONS(3580),
    [anon_sym_for] = ACTIONS(3580),
    [anon_sym_gen] = ACTIONS(3580),
    [anon_sym_if] = ACTIONS(3580),
    [anon_sym_impl] = ACTIONS(3580),
    [anon_sym_let] = ACTIONS(3580),
    [anon_sym_loop] = ACTIONS(3580),
    [anon_sym_match] = ACTIONS(3580),
    [anon_sym_mod] = ACTIONS(3580),
    [anon_sym_pub] = ACTIONS(3580),
    [anon_sym_return] = ACTIONS(3580),
    [anon_sym_raw] = ACTIONS(3580),
    [anon_sym_static] = ACTIONS(3580),
    [anon_sym_struct] = ACTIONS(3580),
    [anon_sym_trait] = ACTIONS(3580),
    [anon_sym_type] = ACTIONS(3580),
    [anon_sym_union] = ACTIONS(3580),
    [anon_sym_unsafe] = ACTIONS(3580),
    [anon_sym_use] = ACTIONS(3580),
    [anon_sym_while] = ACTIONS(3580),
    [anon_sym_extern] = ACTIONS(3580),
    [anon_sym_safe] = ACTIONS(3580),
    [anon_sym_yield] = ACTIONS(3580),
    [anon_sym_move] = ACTIONS(3580),
    [anon_sym_try] = ACTIONS(3580),
    [sym_integer_literal] = ACTIONS(3578),
    [aux_sym_string_literal_token1] = ACTIONS(3578),
    [sym_char_literal] = ACTIONS(3578),
    [anon_sym_true] = ACTIONS(3580),
    [anon_sym_false] = ACTIONS(3580),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3580),
    [sym_super] = ACTIONS(3580),
    [sym_crate] = ACTIONS(3580),
    [aux_sym_metavariable_token1] = ACTIONS(3578),
    [sym__raw_string_literal_start] = ACTIONS(3578),
    [sym_float_literal] = ACTIONS(3578),
    [sym__outer_block_doc_comment_start] = ACTIONS(3578),
    [sym__outer_line_doc_comment_start] = ACTIONS(3578),
  },
  [STATE(953)] = {
    [sym_line_comment] = STATE(953),
    [sym_block_comment] = STATE(953),
    [ts_builtin_sym_end] = ACTIONS(3582),
    [sym_identifier] = ACTIONS(3584),
    [anon_sym_SEMI] = ACTIONS(3582),
    [anon_sym_macro] = ACTIONS(3584),
    [anon_sym_LPAREN] = ACTIONS(3582),
    [anon_sym_LBRACE] = ACTIONS(3582),
    [anon_sym_RBRACE] = ACTIONS(3582),
    [anon_sym_macro_rules] = ACTIONS(3584),
    [anon_sym_BANG] = ACTIONS(3582),
    [anon_sym_LBRACK] = ACTIONS(3582),
    [anon_sym_STAR] = ACTIONS(3582),
    [anon_sym_u8] = ACTIONS(3584),
    [anon_sym_i8] = ACTIONS(3584),
    [anon_sym_u16] = ACTIONS(3584),
    [anon_sym_i16] = ACTIONS(3584),
    [anon_sym_u32] = ACTIONS(3584),
    [anon_sym_i32] = ACTIONS(3584),
    [anon_sym_u64] = ACTIONS(3584),
    [anon_sym_i64] = ACTIONS(3584),
    [anon_sym_u128] = ACTIONS(3584),
    [anon_sym_i128] = ACTIONS(3584),
    [anon_sym_isize] = ACTIONS(3584),
    [anon_sym_usize] = ACTIONS(3584),
    [anon_sym_f32] = ACTIONS(3584),
    [anon_sym_f64] = ACTIONS(3584),
    [anon_sym_bool] = ACTIONS(3584),
    [anon_sym_str] = ACTIONS(3584),
    [anon_sym_char] = ACTIONS(3584),
    [anon_sym_DASH] = ACTIONS(3582),
    [anon_sym_AMP] = ACTIONS(3582),
    [anon_sym_PIPE] = ACTIONS(3582),
    [anon_sym_LT] = ACTIONS(3582),
    [anon_sym_DOT_DOT] = ACTIONS(3582),
    [anon_sym_COLON_COLON] = ACTIONS(3582),
    [anon_sym_POUND] = ACTIONS(3582),
    [anon_sym_SQUOTE] = ACTIONS(3584),
    [anon_sym_async] = ACTIONS(3584),
    [anon_sym_auto] = ACTIONS(3584),
    [anon_sym_await] = ACTIONS(3584),
    [anon_sym_become] = ACTIONS(3584),
    [anon_sym_break] = ACTIONS(3584),
    [anon_sym_const] = ACTIONS(3584),
    [anon_sym_continue] = ACTIONS(3584),
    [anon_sym_default] = ACTIONS(3584),
    [anon_sym_enum] = ACTIONS(3584),
    [anon_sym_fn] = ACTIONS(3584),
    [anon_sym_for] = ACTIONS(3584),
    [anon_sym_gen] = ACTIONS(3584),
    [anon_sym_if] = ACTIONS(3584),
    [anon_sym_impl] = ACTIONS(3584),
    [anon_sym_let] = ACTIONS(3584),
    [anon_sym_loop] = ACTIONS(3584),
    [anon_sym_match] = ACTIONS(3584),
    [anon_sym_mod] = ACTIONS(3584),
    [anon_sym_pub] = ACTIONS(3584),
    [anon_sym_return] = ACTIONS(3584),
    [anon_sym_raw] = ACTIONS(3584),
    [anon_sym_static] = ACTIONS(3584),
    [anon_sym_struct] = ACTIONS(3584),
    [anon_sym_trait] = ACTIONS(3584),
    [anon_sym_type] = ACTIONS(3584),
    [anon_sym_union] = ACTIONS(3584),
    [anon_sym_unsafe] = ACTIONS(3584),
    [anon_sym_use] = ACTIONS(3584),
    [anon_sym_while] = ACTIONS(3584),
    [anon_sym_extern] = ACTIONS(3584),
    [anon_sym_safe] = ACTIONS(3584),
    [anon_sym_yield] = ACTIONS(3584),
    [anon_sym_move] = ACTIONS(3584),
    [anon_sym_try] = ACTIONS(3584),
    [sym_integer_literal] = ACTIONS(3582),
    [aux_sym_string_literal_token1] = ACTIONS(3582),
    [sym_char_literal] = ACTIONS(3582),
    [anon_sym_true] = ACTIONS(3584),
    [anon_sym_false] = ACTIONS(3584),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3584),
    [sym_super] = ACTIONS(3584),
    [sym_crate] = ACTIONS(3584),
    [aux_sym_metavariable_token1] = ACTIONS(3582),
    [sym__raw_string_literal_start] = ACTIONS(3582),
    [sym_float_literal] = ACTIONS(3582),
    [sym__outer_block_doc_comment_start] = ACTIONS(3582),
    [sym__outer_line_doc_comment_start] = ACTIONS(3582),
  },
  [STATE(954)] = {
    [sym_line_comment] = STATE(954),
    [sym_block_comment] = STATE(954),
    [ts_builtin_sym_end] = ACTIONS(3586),
    [sym_identifier] = ACTIONS(3588),
    [anon_sym_SEMI] = ACTIONS(3586),
    [anon_sym_macro] = ACTIONS(3588),
    [anon_sym_LPAREN] = ACTIONS(3586),
    [anon_sym_LBRACE] = ACTIONS(3586),
    [anon_sym_RBRACE] = ACTIONS(3586),
    [anon_sym_macro_rules] = ACTIONS(3588),
    [anon_sym_BANG] = ACTIONS(3586),
    [anon_sym_LBRACK] = ACTIONS(3586),
    [anon_sym_STAR] = ACTIONS(3586),
    [anon_sym_u8] = ACTIONS(3588),
    [anon_sym_i8] = ACTIONS(3588),
    [anon_sym_u16] = ACTIONS(3588),
    [anon_sym_i16] = ACTIONS(3588),
    [anon_sym_u32] = ACTIONS(3588),
    [anon_sym_i32] = ACTIONS(3588),
    [anon_sym_u64] = ACTIONS(3588),
    [anon_sym_i64] = ACTIONS(3588),
    [anon_sym_u128] = ACTIONS(3588),
    [anon_sym_i128] = ACTIONS(3588),
    [anon_sym_isize] = ACTIONS(3588),
    [anon_sym_usize] = ACTIONS(3588),
    [anon_sym_f32] = ACTIONS(3588),
    [anon_sym_f64] = ACTIONS(3588),
    [anon_sym_bool] = ACTIONS(3588),
    [anon_sym_str] = ACTIONS(3588),
    [anon_sym_char] = ACTIONS(3588),
    [anon_sym_DASH] = ACTIONS(3586),
    [anon_sym_AMP] = ACTIONS(3586),
    [anon_sym_PIPE] = ACTIONS(3586),
    [anon_sym_LT] = ACTIONS(3586),
    [anon_sym_DOT_DOT] = ACTIONS(3586),
    [anon_sym_COLON_COLON] = ACTIONS(3586),
    [anon_sym_POUND] = ACTIONS(3586),
    [anon_sym_SQUOTE] = ACTIONS(3588),
    [anon_sym_async] = ACTIONS(3588),
    [anon_sym_auto] = ACTIONS(3588),
    [anon_sym_await] = ACTIONS(3588),
    [anon_sym_become] = ACTIONS(3588),
    [anon_sym_break] = ACTIONS(3588),
    [anon_sym_const] = ACTIONS(3588),
    [anon_sym_continue] = ACTIONS(3588),
    [anon_sym_default] = ACTIONS(3588),
    [anon_sym_enum] = ACTIONS(3588),
    [anon_sym_fn] = ACTIONS(3588),
    [anon_sym_for] = ACTIONS(3588),
    [anon_sym_gen] = ACTIONS(3588),
    [anon_sym_if] = ACTIONS(3588),
    [anon_sym_impl] = ACTIONS(3588),
    [anon_sym_let] = ACTIONS(3588),
    [anon_sym_loop] = ACTIONS(3588),
    [anon_sym_match] = ACTIONS(3588),
    [anon_sym_mod] = ACTIONS(3588),
    [anon_sym_pub] = ACTIONS(3588),
    [anon_sym_return] = ACTIONS(3588),
    [anon_sym_raw] = ACTIONS(3588),
    [anon_sym_static] = ACTIONS(3588),
    [anon_sym_struct] = ACTIONS(3588),
    [anon_sym_trait] = ACTIONS(3588),
    [anon_sym_type] = ACTIONS(3588),
    [anon_sym_union] = ACTIONS(3588),
    [anon_sym_unsafe] = ACTIONS(3588),
    [anon_sym_use] = ACTIONS(3588),
    [anon_sym_while] = ACTIONS(3588),
    [anon_sym_extern] = ACTIONS(3588),
    [anon_sym_safe] = ACTIONS(3588),
    [anon_sym_yield] = ACTIONS(3588),
    [anon_sym_move] = ACTIONS(3588),
    [anon_sym_try] = ACTIONS(3588),
    [sym_integer_literal] = ACTIONS(3586),
    [aux_sym_string_literal_token1] = ACTIONS(3586),
    [sym_char_literal] = ACTIONS(3586),
    [anon_sym_true] = ACTIONS(3588),
    [anon_sym_false] = ACTIONS(3588),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3588),
    [sym_super] = ACTIONS(3588),
    [sym_crate] = ACTIONS(3588),
    [aux_sym_metavariable_token1] = ACTIONS(3586),
    [sym__raw_string_literal_start] = ACTIONS(3586),
    [sym_float_literal] = ACTIONS(3586),
    [sym__outer_block_doc_comment_start] = ACTIONS(3586),
    [sym__outer_line_doc_comment_start] = ACTIONS(3586),
  },
  [STATE(955)] = {
    [sym_line_comment] = STATE(955),
    [sym_block_comment] = STATE(955),
    [ts_builtin_sym_end] = ACTIONS(3590),
    [sym_identifier] = ACTIONS(3592),
    [anon_sym_SEMI] = ACTIONS(3590),
    [anon_sym_macro] = ACTIONS(3592),
    [anon_sym_LPAREN] = ACTIONS(3590),
    [anon_sym_LBRACE] = ACTIONS(3590),
    [anon_sym_RBRACE] = ACTIONS(3590),
    [anon_sym_macro_rules] = ACTIONS(3592),
    [anon_sym_BANG] = ACTIONS(3590),
    [anon_sym_LBRACK] = ACTIONS(3590),
    [anon_sym_STAR] = ACTIONS(3590),
    [anon_sym_u8] = ACTIONS(3592),
    [anon_sym_i8] = ACTIONS(3592),
    [anon_sym_u16] = ACTIONS(3592),
    [anon_sym_i16] = ACTIONS(3592),
    [anon_sym_u32] = ACTIONS(3592),
    [anon_sym_i32] = ACTIONS(3592),
    [anon_sym_u64] = ACTIONS(3592),
    [anon_sym_i64] = ACTIONS(3592),
    [anon_sym_u128] = ACTIONS(3592),
    [anon_sym_i128] = ACTIONS(3592),
    [anon_sym_isize] = ACTIONS(3592),
    [anon_sym_usize] = ACTIONS(3592),
    [anon_sym_f32] = ACTIONS(3592),
    [anon_sym_f64] = ACTIONS(3592),
    [anon_sym_bool] = ACTIONS(3592),
    [anon_sym_str] = ACTIONS(3592),
    [anon_sym_char] = ACTIONS(3592),
    [anon_sym_DASH] = ACTIONS(3590),
    [anon_sym_AMP] = ACTIONS(3590),
    [anon_sym_PIPE] = ACTIONS(3590),
    [anon_sym_LT] = ACTIONS(3590),
    [anon_sym_DOT_DOT] = ACTIONS(3590),
    [anon_sym_COLON_COLON] = ACTIONS(3590),
    [anon_sym_POUND] = ACTIONS(3590),
    [anon_sym_SQUOTE] = ACTIONS(3592),
    [anon_sym_async] = ACTIONS(3592),
    [anon_sym_auto] = ACTIONS(3592),
    [anon_sym_await] = ACTIONS(3592),
    [anon_sym_become] = ACTIONS(3592),
    [anon_sym_break] = ACTIONS(3592),
    [anon_sym_const] = ACTIONS(3592),
    [anon_sym_continue] = ACTIONS(3592),
    [anon_sym_default] = ACTIONS(3592),
    [anon_sym_enum] = ACTIONS(3592),
    [anon_sym_fn] = ACTIONS(3592),
    [anon_sym_for] = ACTIONS(3592),
    [anon_sym_gen] = ACTIONS(3592),
    [anon_sym_if] = ACTIONS(3592),
    [anon_sym_impl] = ACTIONS(3592),
    [anon_sym_let] = ACTIONS(3592),
    [anon_sym_loop] = ACTIONS(3592),
    [anon_sym_match] = ACTIONS(3592),
    [anon_sym_mod] = ACTIONS(3592),
    [anon_sym_pub] = ACTIONS(3592),
    [anon_sym_return] = ACTIONS(3592),
    [anon_sym_raw] = ACTIONS(3592),
    [anon_sym_static] = ACTIONS(3592),
    [anon_sym_struct] = ACTIONS(3592),
    [anon_sym_trait] = ACTIONS(3592),
    [anon_sym_type] = ACTIONS(3592),
    [anon_sym_union] = ACTIONS(3592),
    [anon_sym_unsafe] = ACTIONS(3592),
    [anon_sym_use] = ACTIONS(3592),
    [anon_sym_while] = ACTIONS(3592),
    [anon_sym_extern] = ACTIONS(3592),
    [anon_sym_safe] = ACTIONS(3592),
    [anon_sym_yield] = ACTIONS(3592),
    [anon_sym_move] = ACTIONS(3592),
    [anon_sym_try] = ACTIONS(3592),
    [sym_integer_literal] = ACTIONS(3590),
    [aux_sym_string_literal_token1] = ACTIONS(3590),
    [sym_char_literal] = ACTIONS(3590),
    [anon_sym_true] = ACTIONS(3592),
    [anon_sym_false] = ACTIONS(3592),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3592),
    [sym_super] = ACTIONS(3592),
    [sym_crate] = ACTIONS(3592),
    [aux_sym_metavariable_token1] = ACTIONS(3590),
    [sym__raw_string_literal_start] = ACTIONS(3590),
    [sym_float_literal] = ACTIONS(3590),
    [sym__outer_block_doc_comment_start] = ACTIONS(3590),
    [sym__outer_line_doc_comment_start] = ACTIONS(3590),
  },
  [STATE(956)] = {
    [sym_line_comment] = STATE(956),
    [sym_block_comment] = STATE(956),
    [ts_builtin_sym_end] = ACTIONS(3594),
    [sym_identifier] = ACTIONS(3596),
    [anon_sym_SEMI] = ACTIONS(3594),
    [anon_sym_macro] = ACTIONS(3596),
    [anon_sym_LPAREN] = ACTIONS(3594),
    [anon_sym_LBRACE] = ACTIONS(3594),
    [anon_sym_RBRACE] = ACTIONS(3594),
    [anon_sym_macro_rules] = ACTIONS(3596),
    [anon_sym_BANG] = ACTIONS(3594),
    [anon_sym_LBRACK] = ACTIONS(3594),
    [anon_sym_STAR] = ACTIONS(3594),
    [anon_sym_u8] = ACTIONS(3596),
    [anon_sym_i8] = ACTIONS(3596),
    [anon_sym_u16] = ACTIONS(3596),
    [anon_sym_i16] = ACTIONS(3596),
    [anon_sym_u32] = ACTIONS(3596),
    [anon_sym_i32] = ACTIONS(3596),
    [anon_sym_u64] = ACTIONS(3596),
    [anon_sym_i64] = ACTIONS(3596),
    [anon_sym_u128] = ACTIONS(3596),
    [anon_sym_i128] = ACTIONS(3596),
    [anon_sym_isize] = ACTIONS(3596),
    [anon_sym_usize] = ACTIONS(3596),
    [anon_sym_f32] = ACTIONS(3596),
    [anon_sym_f64] = ACTIONS(3596),
    [anon_sym_bool] = ACTIONS(3596),
    [anon_sym_str] = ACTIONS(3596),
    [anon_sym_char] = ACTIONS(3596),
    [anon_sym_DASH] = ACTIONS(3594),
    [anon_sym_AMP] = ACTIONS(3594),
    [anon_sym_PIPE] = ACTIONS(3594),
    [anon_sym_LT] = ACTIONS(3594),
    [anon_sym_DOT_DOT] = ACTIONS(3594),
    [anon_sym_COLON_COLON] = ACTIONS(3594),
    [anon_sym_POUND] = ACTIONS(3594),
    [anon_sym_SQUOTE] = ACTIONS(3596),
    [anon_sym_async] = ACTIONS(3596),
    [anon_sym_auto] = ACTIONS(3596),
    [anon_sym_await] = ACTIONS(3596),
    [anon_sym_become] = ACTIONS(3596),
    [anon_sym_break] = ACTIONS(3596),
    [anon_sym_const] = ACTIONS(3596),
    [anon_sym_continue] = ACTIONS(3596),
    [anon_sym_default] = ACTIONS(3596),
    [anon_sym_enum] = ACTIONS(3596),
    [anon_sym_fn] = ACTIONS(3596),
    [anon_sym_for] = ACTIONS(3596),
    [anon_sym_gen] = ACTIONS(3596),
    [anon_sym_if] = ACTIONS(3596),
    [anon_sym_impl] = ACTIONS(3596),
    [anon_sym_let] = ACTIONS(3596),
    [anon_sym_loop] = ACTIONS(3596),
    [anon_sym_match] = ACTIONS(3596),
    [anon_sym_mod] = ACTIONS(3596),
    [anon_sym_pub] = ACTIONS(3596),
    [anon_sym_return] = ACTIONS(3596),
    [anon_sym_raw] = ACTIONS(3596),
    [anon_sym_static] = ACTIONS(3596),
    [anon_sym_struct] = ACTIONS(3596),
    [anon_sym_trait] = ACTIONS(3596),
    [anon_sym_type] = ACTIONS(3596),
    [anon_sym_union] = ACTIONS(3596),
    [anon_sym_unsafe] = ACTIONS(3596),
    [anon_sym_use] = ACTIONS(3596),
    [anon_sym_while] = ACTIONS(3596),
    [anon_sym_extern] = ACTIONS(3596),
    [anon_sym_safe] = ACTIONS(3596),
    [anon_sym_yield] = ACTIONS(3596),
    [anon_sym_move] = ACTIONS(3596),
    [anon_sym_try] = ACTIONS(3596),
    [sym_integer_literal] = ACTIONS(3594),
    [aux_sym_string_literal_token1] = ACTIONS(3594),
    [sym_char_literal] = ACTIONS(3594),
    [anon_sym_true] = ACTIONS(3596),
    [anon_sym_false] = ACTIONS(3596),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3596),
    [sym_super] = ACTIONS(3596),
    [sym_crate] = ACTIONS(3596),
    [aux_sym_metavariable_token1] = ACTIONS(3594),
    [sym__raw_string_literal_start] = ACTIONS(3594),
    [sym_float_literal] = ACTIONS(3594),
    [sym__outer_block_doc_comment_start] = ACTIONS(3594),
    [sym__outer_line_doc_comment_start] = ACTIONS(3594),
  },
  [STATE(957)] = {
    [sym_line_comment] = STATE(957),
    [sym_block_comment] = STATE(957),
    [ts_builtin_sym_end] = ACTIONS(3598),
    [sym_identifier] = ACTIONS(3600),
    [anon_sym_SEMI] = ACTIONS(3598),
    [anon_sym_macro] = ACTIONS(3600),
    [anon_sym_LPAREN] = ACTIONS(3598),
    [anon_sym_LBRACE] = ACTIONS(3598),
    [anon_sym_RBRACE] = ACTIONS(3598),
    [anon_sym_macro_rules] = ACTIONS(3600),
    [anon_sym_BANG] = ACTIONS(3598),
    [anon_sym_LBRACK] = ACTIONS(3598),
    [anon_sym_STAR] = ACTIONS(3598),
    [anon_sym_u8] = ACTIONS(3600),
    [anon_sym_i8] = ACTIONS(3600),
    [anon_sym_u16] = ACTIONS(3600),
    [anon_sym_i16] = ACTIONS(3600),
    [anon_sym_u32] = ACTIONS(3600),
    [anon_sym_i32] = ACTIONS(3600),
    [anon_sym_u64] = ACTIONS(3600),
    [anon_sym_i64] = ACTIONS(3600),
    [anon_sym_u128] = ACTIONS(3600),
    [anon_sym_i128] = ACTIONS(3600),
    [anon_sym_isize] = ACTIONS(3600),
    [anon_sym_usize] = ACTIONS(3600),
    [anon_sym_f32] = ACTIONS(3600),
    [anon_sym_f64] = ACTIONS(3600),
    [anon_sym_bool] = ACTIONS(3600),
    [anon_sym_str] = ACTIONS(3600),
    [anon_sym_char] = ACTIONS(3600),
    [anon_sym_DASH] = ACTIONS(3598),
    [anon_sym_AMP] = ACTIONS(3598),
    [anon_sym_PIPE] = ACTIONS(3598),
    [anon_sym_LT] = ACTIONS(3598),
    [anon_sym_DOT_DOT] = ACTIONS(3598),
    [anon_sym_COLON_COLON] = ACTIONS(3598),
    [anon_sym_POUND] = ACTIONS(3598),
    [anon_sym_SQUOTE] = ACTIONS(3600),
    [anon_sym_async] = ACTIONS(3600),
    [anon_sym_auto] = ACTIONS(3600),
    [anon_sym_await] = ACTIONS(3600),
    [anon_sym_become] = ACTIONS(3600),
    [anon_sym_break] = ACTIONS(3600),
    [anon_sym_const] = ACTIONS(3600),
    [anon_sym_continue] = ACTIONS(3600),
    [anon_sym_default] = ACTIONS(3600),
    [anon_sym_enum] = ACTIONS(3600),
    [anon_sym_fn] = ACTIONS(3600),
    [anon_sym_for] = ACTIONS(3600),
    [anon_sym_gen] = ACTIONS(3600),
    [anon_sym_if] = ACTIONS(3600),
    [anon_sym_impl] = ACTIONS(3600),
    [anon_sym_let] = ACTIONS(3600),
    [anon_sym_loop] = ACTIONS(3600),
    [anon_sym_match] = ACTIONS(3600),
    [anon_sym_mod] = ACTIONS(3600),
    [anon_sym_pub] = ACTIONS(3600),
    [anon_sym_return] = ACTIONS(3600),
    [anon_sym_raw] = ACTIONS(3600),
    [anon_sym_static] = ACTIONS(3600),
    [anon_sym_struct] = ACTIONS(3600),
    [anon_sym_trait] = ACTIONS(3600),
    [anon_sym_type] = ACTIONS(3600),
    [anon_sym_union] = ACTIONS(3600),
    [anon_sym_unsafe] = ACTIONS(3600),
    [anon_sym_use] = ACTIONS(3600),
    [anon_sym_while] = ACTIONS(3600),
    [anon_sym_extern] = ACTIONS(3600),
    [anon_sym_safe] = ACTIONS(3600),
    [anon_sym_yield] = ACTIONS(3600),
    [anon_sym_move] = ACTIONS(3600),
    [anon_sym_try] = ACTIONS(3600),
    [sym_integer_literal] = ACTIONS(3598),
    [aux_sym_string_literal_token1] = ACTIONS(3598),
    [sym_char_literal] = ACTIONS(3598),
    [anon_sym_true] = ACTIONS(3600),
    [anon_sym_false] = ACTIONS(3600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3600),
    [sym_super] = ACTIONS(3600),
    [sym_crate] = ACTIONS(3600),
    [aux_sym_metavariable_token1] = ACTIONS(3598),
    [sym__raw_string_literal_start] = ACTIONS(3598),
    [sym_float_literal] = ACTIONS(3598),
    [sym__outer_block_doc_comment_start] = ACTIONS(3598),
    [sym__outer_line_doc_comment_start] = ACTIONS(3598),
  },
  [STATE(958)] = {
    [sym_line_comment] = STATE(958),
    [sym_block_comment] = STATE(958),
    [ts_builtin_sym_end] = ACTIONS(3602),
    [sym_identifier] = ACTIONS(3604),
    [anon_sym_SEMI] = ACTIONS(3602),
    [anon_sym_macro] = ACTIONS(3604),
    [anon_sym_LPAREN] = ACTIONS(3602),
    [anon_sym_LBRACE] = ACTIONS(3602),
    [anon_sym_RBRACE] = ACTIONS(3602),
    [anon_sym_macro_rules] = ACTIONS(3604),
    [anon_sym_BANG] = ACTIONS(3602),
    [anon_sym_LBRACK] = ACTIONS(3602),
    [anon_sym_STAR] = ACTIONS(3602),
    [anon_sym_u8] = ACTIONS(3604),
    [anon_sym_i8] = ACTIONS(3604),
    [anon_sym_u16] = ACTIONS(3604),
    [anon_sym_i16] = ACTIONS(3604),
    [anon_sym_u32] = ACTIONS(3604),
    [anon_sym_i32] = ACTIONS(3604),
    [anon_sym_u64] = ACTIONS(3604),
    [anon_sym_i64] = ACTIONS(3604),
    [anon_sym_u128] = ACTIONS(3604),
    [anon_sym_i128] = ACTIONS(3604),
    [anon_sym_isize] = ACTIONS(3604),
    [anon_sym_usize] = ACTIONS(3604),
    [anon_sym_f32] = ACTIONS(3604),
    [anon_sym_f64] = ACTIONS(3604),
    [anon_sym_bool] = ACTIONS(3604),
    [anon_sym_str] = ACTIONS(3604),
    [anon_sym_char] = ACTIONS(3604),
    [anon_sym_DASH] = ACTIONS(3602),
    [anon_sym_AMP] = ACTIONS(3602),
    [anon_sym_PIPE] = ACTIONS(3602),
    [anon_sym_LT] = ACTIONS(3602),
    [anon_sym_DOT_DOT] = ACTIONS(3602),
    [anon_sym_COLON_COLON] = ACTIONS(3602),
    [anon_sym_POUND] = ACTIONS(3602),
    [anon_sym_SQUOTE] = ACTIONS(3604),
    [anon_sym_async] = ACTIONS(3604),
    [anon_sym_auto] = ACTIONS(3604),
    [anon_sym_await] = ACTIONS(3604),
    [anon_sym_become] = ACTIONS(3604),
    [anon_sym_break] = ACTIONS(3604),
    [anon_sym_const] = ACTIONS(3604),
    [anon_sym_continue] = ACTIONS(3604),
    [anon_sym_default] = ACTIONS(3604),
    [anon_sym_enum] = ACTIONS(3604),
    [anon_sym_fn] = ACTIONS(3604),
    [anon_sym_for] = ACTIONS(3604),
    [anon_sym_gen] = ACTIONS(3604),
    [anon_sym_if] = ACTIONS(3604),
    [anon_sym_impl] = ACTIONS(3604),
    [anon_sym_let] = ACTIONS(3604),
    [anon_sym_loop] = ACTIONS(3604),
    [anon_sym_match] = ACTIONS(3604),
    [anon_sym_mod] = ACTIONS(3604),
    [anon_sym_pub] = ACTIONS(3604),
    [anon_sym_return] = ACTIONS(3604),
    [anon_sym_raw] = ACTIONS(3604),
    [anon_sym_static] = ACTIONS(3604),
    [anon_sym_struct] = ACTIONS(3604),
    [anon_sym_trait] = ACTIONS(3604),
    [anon_sym_type] = ACTIONS(3604),
    [anon_sym_union] = ACTIONS(3604),
    [anon_sym_unsafe] = ACTIONS(3604),
    [anon_sym_use] = ACTIONS(3604),
    [anon_sym_while] = ACTIONS(3604),
    [anon_sym_extern] = ACTIONS(3604),
    [anon_sym_safe] = ACTIONS(3604),
    [anon_sym_yield] = ACTIONS(3604),
    [anon_sym_move] = ACTIONS(3604),
    [anon_sym_try] = ACTIONS(3604),
    [sym_integer_literal] = ACTIONS(3602),
    [aux_sym_string_literal_token1] = ACTIONS(3602),
    [sym_char_literal] = ACTIONS(3602),
    [anon_sym_true] = ACTIONS(3604),
    [anon_sym_false] = ACTIONS(3604),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3604),
    [sym_super] = ACTIONS(3604),
    [sym_crate] = ACTIONS(3604),
    [aux_sym_metavariable_token1] = ACTIONS(3602),
    [sym__raw_string_literal_start] = ACTIONS(3602),
    [sym_float_literal] = ACTIONS(3602),
    [sym__outer_block_doc_comment_start] = ACTIONS(3602),
    [sym__outer_line_doc_comment_start] = ACTIONS(3602),
  },
  [STATE(959)] = {
    [sym_line_comment] = STATE(959),
    [sym_block_comment] = STATE(959),
    [ts_builtin_sym_end] = ACTIONS(3606),
    [sym_identifier] = ACTIONS(3608),
    [anon_sym_SEMI] = ACTIONS(3606),
    [anon_sym_macro] = ACTIONS(3608),
    [anon_sym_LPAREN] = ACTIONS(3606),
    [anon_sym_LBRACE] = ACTIONS(3606),
    [anon_sym_RBRACE] = ACTIONS(3606),
    [anon_sym_macro_rules] = ACTIONS(3608),
    [anon_sym_BANG] = ACTIONS(3606),
    [anon_sym_LBRACK] = ACTIONS(3606),
    [anon_sym_STAR] = ACTIONS(3606),
    [anon_sym_u8] = ACTIONS(3608),
    [anon_sym_i8] = ACTIONS(3608),
    [anon_sym_u16] = ACTIONS(3608),
    [anon_sym_i16] = ACTIONS(3608),
    [anon_sym_u32] = ACTIONS(3608),
    [anon_sym_i32] = ACTIONS(3608),
    [anon_sym_u64] = ACTIONS(3608),
    [anon_sym_i64] = ACTIONS(3608),
    [anon_sym_u128] = ACTIONS(3608),
    [anon_sym_i128] = ACTIONS(3608),
    [anon_sym_isize] = ACTIONS(3608),
    [anon_sym_usize] = ACTIONS(3608),
    [anon_sym_f32] = ACTIONS(3608),
    [anon_sym_f64] = ACTIONS(3608),
    [anon_sym_bool] = ACTIONS(3608),
    [anon_sym_str] = ACTIONS(3608),
    [anon_sym_char] = ACTIONS(3608),
    [anon_sym_DASH] = ACTIONS(3606),
    [anon_sym_AMP] = ACTIONS(3606),
    [anon_sym_PIPE] = ACTIONS(3606),
    [anon_sym_LT] = ACTIONS(3606),
    [anon_sym_DOT_DOT] = ACTIONS(3606),
    [anon_sym_COLON_COLON] = ACTIONS(3606),
    [anon_sym_POUND] = ACTIONS(3606),
    [anon_sym_SQUOTE] = ACTIONS(3608),
    [anon_sym_async] = ACTIONS(3608),
    [anon_sym_auto] = ACTIONS(3608),
    [anon_sym_await] = ACTIONS(3608),
    [anon_sym_become] = ACTIONS(3608),
    [anon_sym_break] = ACTIONS(3608),
    [anon_sym_const] = ACTIONS(3608),
    [anon_sym_continue] = ACTIONS(3608),
    [anon_sym_default] = ACTIONS(3608),
    [anon_sym_enum] = ACTIONS(3608),
    [anon_sym_fn] = ACTIONS(3608),
    [anon_sym_for] = ACTIONS(3608),
    [anon_sym_gen] = ACTIONS(3608),
    [anon_sym_if] = ACTIONS(3608),
    [anon_sym_impl] = ACTIONS(3608),
    [anon_sym_let] = ACTIONS(3608),
    [anon_sym_loop] = ACTIONS(3608),
    [anon_sym_match] = ACTIONS(3608),
    [anon_sym_mod] = ACTIONS(3608),
    [anon_sym_pub] = ACTIONS(3608),
    [anon_sym_return] = ACTIONS(3608),
    [anon_sym_raw] = ACTIONS(3608),
    [anon_sym_static] = ACTIONS(3608),
    [anon_sym_struct] = ACTIONS(3608),
    [anon_sym_trait] = ACTIONS(3608),
    [anon_sym_type] = ACTIONS(3608),
    [anon_sym_union] = ACTIONS(3608),
    [anon_sym_unsafe] = ACTIONS(3608),
    [anon_sym_use] = ACTIONS(3608),
    [anon_sym_while] = ACTIONS(3608),
    [anon_sym_extern] = ACTIONS(3608),
    [anon_sym_safe] = ACTIONS(3608),
    [anon_sym_yield] = ACTIONS(3608),
    [anon_sym_move] = ACTIONS(3608),
    [anon_sym_try] = ACTIONS(3608),
    [sym_integer_literal] = ACTIONS(3606),
    [aux_sym_string_literal_token1] = ACTIONS(3606),
    [sym_char_literal] = ACTIONS(3606),
    [anon_sym_true] = ACTIONS(3608),
    [anon_sym_false] = ACTIONS(3608),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3608),
    [sym_super] = ACTIONS(3608),
    [sym_crate] = ACTIONS(3608),
    [aux_sym_metavariable_token1] = ACTIONS(3606),
    [sym__raw_string_literal_start] = ACTIONS(3606),
    [sym_float_literal] = ACTIONS(3606),
    [sym__outer_block_doc_comment_start] = ACTIONS(3606),
    [sym__outer_line_doc_comment_start] = ACTIONS(3606),
  },
  [STATE(960)] = {
    [sym_line_comment] = STATE(960),
    [sym_block_comment] = STATE(960),
    [ts_builtin_sym_end] = ACTIONS(3610),
    [sym_identifier] = ACTIONS(3612),
    [anon_sym_SEMI] = ACTIONS(3610),
    [anon_sym_macro] = ACTIONS(3612),
    [anon_sym_LPAREN] = ACTIONS(3610),
    [anon_sym_LBRACE] = ACTIONS(3610),
    [anon_sym_RBRACE] = ACTIONS(3610),
    [anon_sym_macro_rules] = ACTIONS(3612),
    [anon_sym_BANG] = ACTIONS(3610),
    [anon_sym_LBRACK] = ACTIONS(3610),
    [anon_sym_STAR] = ACTIONS(3610),
    [anon_sym_u8] = ACTIONS(3612),
    [anon_sym_i8] = ACTIONS(3612),
    [anon_sym_u16] = ACTIONS(3612),
    [anon_sym_i16] = ACTIONS(3612),
    [anon_sym_u32] = ACTIONS(3612),
    [anon_sym_i32] = ACTIONS(3612),
    [anon_sym_u64] = ACTIONS(3612),
    [anon_sym_i64] = ACTIONS(3612),
    [anon_sym_u128] = ACTIONS(3612),
    [anon_sym_i128] = ACTIONS(3612),
    [anon_sym_isize] = ACTIONS(3612),
    [anon_sym_usize] = ACTIONS(3612),
    [anon_sym_f32] = ACTIONS(3612),
    [anon_sym_f64] = ACTIONS(3612),
    [anon_sym_bool] = ACTIONS(3612),
    [anon_sym_str] = ACTIONS(3612),
    [anon_sym_char] = ACTIONS(3612),
    [anon_sym_DASH] = ACTIONS(3610),
    [anon_sym_AMP] = ACTIONS(3610),
    [anon_sym_PIPE] = ACTIONS(3610),
    [anon_sym_LT] = ACTIONS(3610),
    [anon_sym_DOT_DOT] = ACTIONS(3610),
    [anon_sym_COLON_COLON] = ACTIONS(3610),
    [anon_sym_POUND] = ACTIONS(3610),
    [anon_sym_SQUOTE] = ACTIONS(3612),
    [anon_sym_async] = ACTIONS(3612),
    [anon_sym_auto] = ACTIONS(3612),
    [anon_sym_await] = ACTIONS(3612),
    [anon_sym_become] = ACTIONS(3612),
    [anon_sym_break] = ACTIONS(3612),
    [anon_sym_const] = ACTIONS(3612),
    [anon_sym_continue] = ACTIONS(3612),
    [anon_sym_default] = ACTIONS(3612),
    [anon_sym_enum] = ACTIONS(3612),
    [anon_sym_fn] = ACTIONS(3612),
    [anon_sym_for] = ACTIONS(3612),
    [anon_sym_gen] = ACTIONS(3612),
    [anon_sym_if] = ACTIONS(3612),
    [anon_sym_impl] = ACTIONS(3612),
    [anon_sym_let] = ACTIONS(3612),
    [anon_sym_loop] = ACTIONS(3612),
    [anon_sym_match] = ACTIONS(3612),
    [anon_sym_mod] = ACTIONS(3612),
    [anon_sym_pub] = ACTIONS(3612),
    [anon_sym_return] = ACTIONS(3612),
    [anon_sym_raw] = ACTIONS(3612),
    [anon_sym_static] = ACTIONS(3612),
    [anon_sym_struct] = ACTIONS(3612),
    [anon_sym_trait] = ACTIONS(3612),
    [anon_sym_type] = ACTIONS(3612),
    [anon_sym_union] = ACTIONS(3612),
    [anon_sym_unsafe] = ACTIONS(3612),
    [anon_sym_use] = ACTIONS(3612),
    [anon_sym_while] = ACTIONS(3612),
    [anon_sym_extern] = ACTIONS(3612),
    [anon_sym_safe] = ACTIONS(3612),
    [anon_sym_yield] = ACTIONS(3612),
    [anon_sym_move] = ACTIONS(3612),
    [anon_sym_try] = ACTIONS(3612),
    [sym_integer_literal] = ACTIONS(3610),
    [aux_sym_string_literal_token1] = ACTIONS(3610),
    [sym_char_literal] = ACTIONS(3610),
    [anon_sym_true] = ACTIONS(3612),
    [anon_sym_false] = ACTIONS(3612),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3612),
    [sym_super] = ACTIONS(3612),
    [sym_crate] = ACTIONS(3612),
    [aux_sym_metavariable_token1] = ACTIONS(3610),
    [sym__raw_string_literal_start] = ACTIONS(3610),
    [sym_float_literal] = ACTIONS(3610),
    [sym__outer_block_doc_comment_start] = ACTIONS(3610),
    [sym__outer_line_doc_comment_start] = ACTIONS(3610),
  },
  [STATE(961)] = {
    [sym_line_comment] = STATE(961),
    [sym_block_comment] = STATE(961),
    [ts_builtin_sym_end] = ACTIONS(3614),
    [sym_identifier] = ACTIONS(3616),
    [anon_sym_SEMI] = ACTIONS(3614),
    [anon_sym_macro] = ACTIONS(3616),
    [anon_sym_LPAREN] = ACTIONS(3614),
    [anon_sym_LBRACE] = ACTIONS(3614),
    [anon_sym_RBRACE] = ACTIONS(3614),
    [anon_sym_macro_rules] = ACTIONS(3616),
    [anon_sym_BANG] = ACTIONS(3614),
    [anon_sym_LBRACK] = ACTIONS(3614),
    [anon_sym_STAR] = ACTIONS(3614),
    [anon_sym_u8] = ACTIONS(3616),
    [anon_sym_i8] = ACTIONS(3616),
    [anon_sym_u16] = ACTIONS(3616),
    [anon_sym_i16] = ACTIONS(3616),
    [anon_sym_u32] = ACTIONS(3616),
    [anon_sym_i32] = ACTIONS(3616),
    [anon_sym_u64] = ACTIONS(3616),
    [anon_sym_i64] = ACTIONS(3616),
    [anon_sym_u128] = ACTIONS(3616),
    [anon_sym_i128] = ACTIONS(3616),
    [anon_sym_isize] = ACTIONS(3616),
    [anon_sym_usize] = ACTIONS(3616),
    [anon_sym_f32] = ACTIONS(3616),
    [anon_sym_f64] = ACTIONS(3616),
    [anon_sym_bool] = ACTIONS(3616),
    [anon_sym_str] = ACTIONS(3616),
    [anon_sym_char] = ACTIONS(3616),
    [anon_sym_DASH] = ACTIONS(3614),
    [anon_sym_AMP] = ACTIONS(3614),
    [anon_sym_PIPE] = ACTIONS(3614),
    [anon_sym_LT] = ACTIONS(3614),
    [anon_sym_DOT_DOT] = ACTIONS(3614),
    [anon_sym_COLON_COLON] = ACTIONS(3614),
    [anon_sym_POUND] = ACTIONS(3614),
    [anon_sym_SQUOTE] = ACTIONS(3616),
    [anon_sym_async] = ACTIONS(3616),
    [anon_sym_auto] = ACTIONS(3616),
    [anon_sym_await] = ACTIONS(3616),
    [anon_sym_become] = ACTIONS(3616),
    [anon_sym_break] = ACTIONS(3616),
    [anon_sym_const] = ACTIONS(3616),
    [anon_sym_continue] = ACTIONS(3616),
    [anon_sym_default] = ACTIONS(3616),
    [anon_sym_enum] = ACTIONS(3616),
    [anon_sym_fn] = ACTIONS(3616),
    [anon_sym_for] = ACTIONS(3616),
    [anon_sym_gen] = ACTIONS(3616),
    [anon_sym_if] = ACTIONS(3616),
    [anon_sym_impl] = ACTIONS(3616),
    [anon_sym_let] = ACTIONS(3616),
    [anon_sym_loop] = ACTIONS(3616),
    [anon_sym_match] = ACTIONS(3616),
    [anon_sym_mod] = ACTIONS(3616),
    [anon_sym_pub] = ACTIONS(3616),
    [anon_sym_return] = ACTIONS(3616),
    [anon_sym_raw] = ACTIONS(3616),
    [anon_sym_static] = ACTIONS(3616),
    [anon_sym_struct] = ACTIONS(3616),
    [anon_sym_trait] = ACTIONS(3616),
    [anon_sym_type] = ACTIONS(3616),
    [anon_sym_union] = ACTIONS(3616),
    [anon_sym_unsafe] = ACTIONS(3616),
    [anon_sym_use] = ACTIONS(3616),
    [anon_sym_while] = ACTIONS(3616),
    [anon_sym_extern] = ACTIONS(3616),
    [anon_sym_safe] = ACTIONS(3616),
    [anon_sym_yield] = ACTIONS(3616),
    [anon_sym_move] = ACTIONS(3616),
    [anon_sym_try] = ACTIONS(3616),
    [sym_integer_literal] = ACTIONS(3614),
    [aux_sym_string_literal_token1] = ACTIONS(3614),
    [sym_char_literal] = ACTIONS(3614),
    [anon_sym_true] = ACTIONS(3616),
    [anon_sym_false] = ACTIONS(3616),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3616),
    [sym_super] = ACTIONS(3616),
    [sym_crate] = ACTIONS(3616),
    [aux_sym_metavariable_token1] = ACTIONS(3614),
    [sym__raw_string_literal_start] = ACTIONS(3614),
    [sym_float_literal] = ACTIONS(3614),
    [sym__outer_block_doc_comment_start] = ACTIONS(3614),
    [sym__outer_line_doc_comment_start] = ACTIONS(3614),
  },
  [STATE(962)] = {
    [sym_line_comment] = STATE(962),
    [sym_block_comment] = STATE(962),
    [ts_builtin_sym_end] = ACTIONS(3618),
    [sym_identifier] = ACTIONS(3620),
    [anon_sym_SEMI] = ACTIONS(3618),
    [anon_sym_macro] = ACTIONS(3620),
    [anon_sym_LPAREN] = ACTIONS(3618),
    [anon_sym_LBRACE] = ACTIONS(3618),
    [anon_sym_RBRACE] = ACTIONS(3618),
    [anon_sym_macro_rules] = ACTIONS(3620),
    [anon_sym_BANG] = ACTIONS(3618),
    [anon_sym_LBRACK] = ACTIONS(3618),
    [anon_sym_STAR] = ACTIONS(3618),
    [anon_sym_u8] = ACTIONS(3620),
    [anon_sym_i8] = ACTIONS(3620),
    [anon_sym_u16] = ACTIONS(3620),
    [anon_sym_i16] = ACTIONS(3620),
    [anon_sym_u32] = ACTIONS(3620),
    [anon_sym_i32] = ACTIONS(3620),
    [anon_sym_u64] = ACTIONS(3620),
    [anon_sym_i64] = ACTIONS(3620),
    [anon_sym_u128] = ACTIONS(3620),
    [anon_sym_i128] = ACTIONS(3620),
    [anon_sym_isize] = ACTIONS(3620),
    [anon_sym_usize] = ACTIONS(3620),
    [anon_sym_f32] = ACTIONS(3620),
    [anon_sym_f64] = ACTIONS(3620),
    [anon_sym_bool] = ACTIONS(3620),
    [anon_sym_str] = ACTIONS(3620),
    [anon_sym_char] = ACTIONS(3620),
    [anon_sym_DASH] = ACTIONS(3618),
    [anon_sym_AMP] = ACTIONS(3618),
    [anon_sym_PIPE] = ACTIONS(3618),
    [anon_sym_LT] = ACTIONS(3618),
    [anon_sym_DOT_DOT] = ACTIONS(3618),
    [anon_sym_COLON_COLON] = ACTIONS(3618),
    [anon_sym_POUND] = ACTIONS(3618),
    [anon_sym_SQUOTE] = ACTIONS(3620),
    [anon_sym_async] = ACTIONS(3620),
    [anon_sym_auto] = ACTIONS(3620),
    [anon_sym_await] = ACTIONS(3620),
    [anon_sym_become] = ACTIONS(3620),
    [anon_sym_break] = ACTIONS(3620),
    [anon_sym_const] = ACTIONS(3620),
    [anon_sym_continue] = ACTIONS(3620),
    [anon_sym_default] = ACTIONS(3620),
    [anon_sym_enum] = ACTIONS(3620),
    [anon_sym_fn] = ACTIONS(3620),
    [anon_sym_for] = ACTIONS(3620),
    [anon_sym_gen] = ACTIONS(3620),
    [anon_sym_if] = ACTIONS(3620),
    [anon_sym_impl] = ACTIONS(3620),
    [anon_sym_let] = ACTIONS(3620),
    [anon_sym_loop] = ACTIONS(3620),
    [anon_sym_match] = ACTIONS(3620),
    [anon_sym_mod] = ACTIONS(3620),
    [anon_sym_pub] = ACTIONS(3620),
    [anon_sym_return] = ACTIONS(3620),
    [anon_sym_raw] = ACTIONS(3620),
    [anon_sym_static] = ACTIONS(3620),
    [anon_sym_struct] = ACTIONS(3620),
    [anon_sym_trait] = ACTIONS(3620),
    [anon_sym_type] = ACTIONS(3620),
    [anon_sym_union] = ACTIONS(3620),
    [anon_sym_unsafe] = ACTIONS(3620),
    [anon_sym_use] = ACTIONS(3620),
    [anon_sym_while] = ACTIONS(3620),
    [anon_sym_extern] = ACTIONS(3620),
    [anon_sym_safe] = ACTIONS(3620),
    [anon_sym_yield] = ACTIONS(3620),
    [anon_sym_move] = ACTIONS(3620),
    [anon_sym_try] = ACTIONS(3620),
    [sym_integer_literal] = ACTIONS(3618),
    [aux_sym_string_literal_token1] = ACTIONS(3618),
    [sym_char_literal] = ACTIONS(3618),
    [anon_sym_true] = ACTIONS(3620),
    [anon_sym_false] = ACTIONS(3620),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3620),
    [sym_super] = ACTIONS(3620),
    [sym_crate] = ACTIONS(3620),
    [aux_sym_metavariable_token1] = ACTIONS(3618),
    [sym__raw_string_literal_start] = ACTIONS(3618),
    [sym_float_literal] = ACTIONS(3618),
    [sym__outer_block_doc_comment_start] = ACTIONS(3618),
    [sym__outer_line_doc_comment_start] = ACTIONS(3618),
  },
  [STATE(963)] = {
    [sym_line_comment] = STATE(963),
    [sym_block_comment] = STATE(963),
    [ts_builtin_sym_end] = ACTIONS(3622),
    [sym_identifier] = ACTIONS(3624),
    [anon_sym_SEMI] = ACTIONS(3622),
    [anon_sym_macro] = ACTIONS(3624),
    [anon_sym_LPAREN] = ACTIONS(3622),
    [anon_sym_LBRACE] = ACTIONS(3622),
    [anon_sym_RBRACE] = ACTIONS(3622),
    [anon_sym_macro_rules] = ACTIONS(3624),
    [anon_sym_BANG] = ACTIONS(3622),
    [anon_sym_LBRACK] = ACTIONS(3622),
    [anon_sym_STAR] = ACTIONS(3622),
    [anon_sym_u8] = ACTIONS(3624),
    [anon_sym_i8] = ACTIONS(3624),
    [anon_sym_u16] = ACTIONS(3624),
    [anon_sym_i16] = ACTIONS(3624),
    [anon_sym_u32] = ACTIONS(3624),
    [anon_sym_i32] = ACTIONS(3624),
    [anon_sym_u64] = ACTIONS(3624),
    [anon_sym_i64] = ACTIONS(3624),
    [anon_sym_u128] = ACTIONS(3624),
    [anon_sym_i128] = ACTIONS(3624),
    [anon_sym_isize] = ACTIONS(3624),
    [anon_sym_usize] = ACTIONS(3624),
    [anon_sym_f32] = ACTIONS(3624),
    [anon_sym_f64] = ACTIONS(3624),
    [anon_sym_bool] = ACTIONS(3624),
    [anon_sym_str] = ACTIONS(3624),
    [anon_sym_char] = ACTIONS(3624),
    [anon_sym_DASH] = ACTIONS(3622),
    [anon_sym_AMP] = ACTIONS(3622),
    [anon_sym_PIPE] = ACTIONS(3622),
    [anon_sym_LT] = ACTIONS(3622),
    [anon_sym_DOT_DOT] = ACTIONS(3622),
    [anon_sym_COLON_COLON] = ACTIONS(3622),
    [anon_sym_POUND] = ACTIONS(3622),
    [anon_sym_SQUOTE] = ACTIONS(3624),
    [anon_sym_async] = ACTIONS(3624),
    [anon_sym_auto] = ACTIONS(3624),
    [anon_sym_await] = ACTIONS(3624),
    [anon_sym_become] = ACTIONS(3624),
    [anon_sym_break] = ACTIONS(3624),
    [anon_sym_const] = ACTIONS(3624),
    [anon_sym_continue] = ACTIONS(3624),
    [anon_sym_default] = ACTIONS(3624),
    [anon_sym_enum] = ACTIONS(3624),
    [anon_sym_fn] = ACTIONS(3624),
    [anon_sym_for] = ACTIONS(3624),
    [anon_sym_gen] = ACTIONS(3624),
    [anon_sym_if] = ACTIONS(3624),
    [anon_sym_impl] = ACTIONS(3624),
    [anon_sym_let] = ACTIONS(3624),
    [anon_sym_loop] = ACTIONS(3624),
    [anon_sym_match] = ACTIONS(3624),
    [anon_sym_mod] = ACTIONS(3624),
    [anon_sym_pub] = ACTIONS(3624),
    [anon_sym_return] = ACTIONS(3624),
    [anon_sym_raw] = ACTIONS(3624),
    [anon_sym_static] = ACTIONS(3624),
    [anon_sym_struct] = ACTIONS(3624),
    [anon_sym_trait] = ACTIONS(3624),
    [anon_sym_type] = ACTIONS(3624),
    [anon_sym_union] = ACTIONS(3624),
    [anon_sym_unsafe] = ACTIONS(3624),
    [anon_sym_use] = ACTIONS(3624),
    [anon_sym_while] = ACTIONS(3624),
    [anon_sym_extern] = ACTIONS(3624),
    [anon_sym_safe] = ACTIONS(3624),
    [anon_sym_yield] = ACTIONS(3624),
    [anon_sym_move] = ACTIONS(3624),
    [anon_sym_try] = ACTIONS(3624),
    [sym_integer_literal] = ACTIONS(3622),
    [aux_sym_string_literal_token1] = ACTIONS(3622),
    [sym_char_literal] = ACTIONS(3622),
    [anon_sym_true] = ACTIONS(3624),
    [anon_sym_false] = ACTIONS(3624),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3624),
    [sym_super] = ACTIONS(3624),
    [sym_crate] = ACTIONS(3624),
    [aux_sym_metavariable_token1] = ACTIONS(3622),
    [sym__raw_string_literal_start] = ACTIONS(3622),
    [sym_float_literal] = ACTIONS(3622),
    [sym__outer_block_doc_comment_start] = ACTIONS(3622),
    [sym__outer_line_doc_comment_start] = ACTIONS(3622),
  },
  [STATE(964)] = {
    [sym_line_comment] = STATE(964),
    [sym_block_comment] = STATE(964),
    [ts_builtin_sym_end] = ACTIONS(3626),
    [sym_identifier] = ACTIONS(3628),
    [anon_sym_SEMI] = ACTIONS(3626),
    [anon_sym_macro] = ACTIONS(3628),
    [anon_sym_LPAREN] = ACTIONS(3626),
    [anon_sym_LBRACE] = ACTIONS(3626),
    [anon_sym_RBRACE] = ACTIONS(3626),
    [anon_sym_macro_rules] = ACTIONS(3628),
    [anon_sym_BANG] = ACTIONS(3626),
    [anon_sym_LBRACK] = ACTIONS(3626),
    [anon_sym_STAR] = ACTIONS(3626),
    [anon_sym_u8] = ACTIONS(3628),
    [anon_sym_i8] = ACTIONS(3628),
    [anon_sym_u16] = ACTIONS(3628),
    [anon_sym_i16] = ACTIONS(3628),
    [anon_sym_u32] = ACTIONS(3628),
    [anon_sym_i32] = ACTIONS(3628),
    [anon_sym_u64] = ACTIONS(3628),
    [anon_sym_i64] = ACTIONS(3628),
    [anon_sym_u128] = ACTIONS(3628),
    [anon_sym_i128] = ACTIONS(3628),
    [anon_sym_isize] = ACTIONS(3628),
    [anon_sym_usize] = ACTIONS(3628),
    [anon_sym_f32] = ACTIONS(3628),
    [anon_sym_f64] = ACTIONS(3628),
    [anon_sym_bool] = ACTIONS(3628),
    [anon_sym_str] = ACTIONS(3628),
    [anon_sym_char] = ACTIONS(3628),
    [anon_sym_DASH] = ACTIONS(3626),
    [anon_sym_AMP] = ACTIONS(3626),
    [anon_sym_PIPE] = ACTIONS(3626),
    [anon_sym_LT] = ACTIONS(3626),
    [anon_sym_DOT_DOT] = ACTIONS(3626),
    [anon_sym_COLON_COLON] = ACTIONS(3626),
    [anon_sym_POUND] = ACTIONS(3626),
    [anon_sym_SQUOTE] = ACTIONS(3628),
    [anon_sym_async] = ACTIONS(3628),
    [anon_sym_auto] = ACTIONS(3628),
    [anon_sym_await] = ACTIONS(3628),
    [anon_sym_become] = ACTIONS(3628),
    [anon_sym_break] = ACTIONS(3628),
    [anon_sym_const] = ACTIONS(3628),
    [anon_sym_continue] = ACTIONS(3628),
    [anon_sym_default] = ACTIONS(3628),
    [anon_sym_enum] = ACTIONS(3628),
    [anon_sym_fn] = ACTIONS(3628),
    [anon_sym_for] = ACTIONS(3628),
    [anon_sym_gen] = ACTIONS(3628),
    [anon_sym_if] = ACTIONS(3628),
    [anon_sym_impl] = ACTIONS(3628),
    [anon_sym_let] = ACTIONS(3628),
    [anon_sym_loop] = ACTIONS(3628),
    [anon_sym_match] = ACTIONS(3628),
    [anon_sym_mod] = ACTIONS(3628),
    [anon_sym_pub] = ACTIONS(3628),
    [anon_sym_return] = ACTIONS(3628),
    [anon_sym_raw] = ACTIONS(3628),
    [anon_sym_static] = ACTIONS(3628),
    [anon_sym_struct] = ACTIONS(3628),
    [anon_sym_trait] = ACTIONS(3628),
    [anon_sym_type] = ACTIONS(3628),
    [anon_sym_union] = ACTIONS(3628),
    [anon_sym_unsafe] = ACTIONS(3628),
    [anon_sym_use] = ACTIONS(3628),
    [anon_sym_while] = ACTIONS(3628),
    [anon_sym_extern] = ACTIONS(3628),
    [anon_sym_safe] = ACTIONS(3628),
    [anon_sym_yield] = ACTIONS(3628),
    [anon_sym_move] = ACTIONS(3628),
    [anon_sym_try] = ACTIONS(3628),
    [sym_integer_literal] = ACTIONS(3626),
    [aux_sym_string_literal_token1] = ACTIONS(3626),
    [sym_char_literal] = ACTIONS(3626),
    [anon_sym_true] = ACTIONS(3628),
    [anon_sym_false] = ACTIONS(3628),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3628),
    [sym_super] = ACTIONS(3628),
    [sym_crate] = ACTIONS(3628),
    [aux_sym_metavariable_token1] = ACTIONS(3626),
    [sym__raw_string_literal_start] = ACTIONS(3626),
    [sym_float_literal] = ACTIONS(3626),
    [sym__outer_block_doc_comment_start] = ACTIONS(3626),
    [sym__outer_line_doc_comment_start] = ACTIONS(3626),
  },
  [STATE(965)] = {
    [sym_line_comment] = STATE(965),
    [sym_block_comment] = STATE(965),
    [ts_builtin_sym_end] = ACTIONS(3630),
    [sym_identifier] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(3630),
    [anon_sym_macro] = ACTIONS(3632),
    [anon_sym_LPAREN] = ACTIONS(3630),
    [anon_sym_LBRACE] = ACTIONS(3630),
    [anon_sym_RBRACE] = ACTIONS(3630),
    [anon_sym_macro_rules] = ACTIONS(3632),
    [anon_sym_BANG] = ACTIONS(3630),
    [anon_sym_LBRACK] = ACTIONS(3630),
    [anon_sym_STAR] = ACTIONS(3630),
    [anon_sym_u8] = ACTIONS(3632),
    [anon_sym_i8] = ACTIONS(3632),
    [anon_sym_u16] = ACTIONS(3632),
    [anon_sym_i16] = ACTIONS(3632),
    [anon_sym_u32] = ACTIONS(3632),
    [anon_sym_i32] = ACTIONS(3632),
    [anon_sym_u64] = ACTIONS(3632),
    [anon_sym_i64] = ACTIONS(3632),
    [anon_sym_u128] = ACTIONS(3632),
    [anon_sym_i128] = ACTIONS(3632),
    [anon_sym_isize] = ACTIONS(3632),
    [anon_sym_usize] = ACTIONS(3632),
    [anon_sym_f32] = ACTIONS(3632),
    [anon_sym_f64] = ACTIONS(3632),
    [anon_sym_bool] = ACTIONS(3632),
    [anon_sym_str] = ACTIONS(3632),
    [anon_sym_char] = ACTIONS(3632),
    [anon_sym_DASH] = ACTIONS(3630),
    [anon_sym_AMP] = ACTIONS(3630),
    [anon_sym_PIPE] = ACTIONS(3630),
    [anon_sym_LT] = ACTIONS(3630),
    [anon_sym_DOT_DOT] = ACTIONS(3630),
    [anon_sym_COLON_COLON] = ACTIONS(3630),
    [anon_sym_POUND] = ACTIONS(3630),
    [anon_sym_SQUOTE] = ACTIONS(3632),
    [anon_sym_async] = ACTIONS(3632),
    [anon_sym_auto] = ACTIONS(3632),
    [anon_sym_await] = ACTIONS(3632),
    [anon_sym_become] = ACTIONS(3632),
    [anon_sym_break] = ACTIONS(3632),
    [anon_sym_const] = ACTIONS(3632),
    [anon_sym_continue] = ACTIONS(3632),
    [anon_sym_default] = ACTIONS(3632),
    [anon_sym_enum] = ACTIONS(3632),
    [anon_sym_fn] = ACTIONS(3632),
    [anon_sym_for] = ACTIONS(3632),
    [anon_sym_gen] = ACTIONS(3632),
    [anon_sym_if] = ACTIONS(3632),
    [anon_sym_impl] = ACTIONS(3632),
    [anon_sym_let] = ACTIONS(3632),
    [anon_sym_loop] = ACTIONS(3632),
    [anon_sym_match] = ACTIONS(3632),
    [anon_sym_mod] = ACTIONS(3632),
    [anon_sym_pub] = ACTIONS(3632),
    [anon_sym_return] = ACTIONS(3632),
    [anon_sym_raw] = ACTIONS(3632),
    [anon_sym_static] = ACTIONS(3632),
    [anon_sym_struct] = ACTIONS(3632),
    [anon_sym_trait] = ACTIONS(3632),
    [anon_sym_type] = ACTIONS(3632),
    [anon_sym_union] = ACTIONS(3632),
    [anon_sym_unsafe] = ACTIONS(3632),
    [anon_sym_use] = ACTIONS(3632),
    [anon_sym_while] = ACTIONS(3632),
    [anon_sym_extern] = ACTIONS(3632),
    [anon_sym_safe] = ACTIONS(3632),
    [anon_sym_yield] = ACTIONS(3632),
    [anon_sym_move] = ACTIONS(3632),
    [anon_sym_try] = ACTIONS(3632),
    [sym_integer_literal] = ACTIONS(3630),
    [aux_sym_string_literal_token1] = ACTIONS(3630),
    [sym_char_literal] = ACTIONS(3630),
    [anon_sym_true] = ACTIONS(3632),
    [anon_sym_false] = ACTIONS(3632),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3632),
    [sym_super] = ACTIONS(3632),
    [sym_crate] = ACTIONS(3632),
    [aux_sym_metavariable_token1] = ACTIONS(3630),
    [sym__raw_string_literal_start] = ACTIONS(3630),
    [sym_float_literal] = ACTIONS(3630),
    [sym__outer_block_doc_comment_start] = ACTIONS(3630),
    [sym__outer_line_doc_comment_start] = ACTIONS(3630),
  },
  [STATE(966)] = {
    [sym_line_comment] = STATE(966),
    [sym_block_comment] = STATE(966),
    [ts_builtin_sym_end] = ACTIONS(3634),
    [sym_identifier] = ACTIONS(3636),
    [anon_sym_SEMI] = ACTIONS(3634),
    [anon_sym_macro] = ACTIONS(3636),
    [anon_sym_LPAREN] = ACTIONS(3634),
    [anon_sym_LBRACE] = ACTIONS(3634),
    [anon_sym_RBRACE] = ACTIONS(3634),
    [anon_sym_macro_rules] = ACTIONS(3636),
    [anon_sym_BANG] = ACTIONS(3634),
    [anon_sym_LBRACK] = ACTIONS(3634),
    [anon_sym_STAR] = ACTIONS(3634),
    [anon_sym_u8] = ACTIONS(3636),
    [anon_sym_i8] = ACTIONS(3636),
    [anon_sym_u16] = ACTIONS(3636),
    [anon_sym_i16] = ACTIONS(3636),
    [anon_sym_u32] = ACTIONS(3636),
    [anon_sym_i32] = ACTIONS(3636),
    [anon_sym_u64] = ACTIONS(3636),
    [anon_sym_i64] = ACTIONS(3636),
    [anon_sym_u128] = ACTIONS(3636),
    [anon_sym_i128] = ACTIONS(3636),
    [anon_sym_isize] = ACTIONS(3636),
    [anon_sym_usize] = ACTIONS(3636),
    [anon_sym_f32] = ACTIONS(3636),
    [anon_sym_f64] = ACTIONS(3636),
    [anon_sym_bool] = ACTIONS(3636),
    [anon_sym_str] = ACTIONS(3636),
    [anon_sym_char] = ACTIONS(3636),
    [anon_sym_DASH] = ACTIONS(3634),
    [anon_sym_AMP] = ACTIONS(3634),
    [anon_sym_PIPE] = ACTIONS(3634),
    [anon_sym_LT] = ACTIONS(3634),
    [anon_sym_DOT_DOT] = ACTIONS(3634),
    [anon_sym_COLON_COLON] = ACTIONS(3634),
    [anon_sym_POUND] = ACTIONS(3634),
    [anon_sym_SQUOTE] = ACTIONS(3636),
    [anon_sym_async] = ACTIONS(3636),
    [anon_sym_auto] = ACTIONS(3636),
    [anon_sym_await] = ACTIONS(3636),
    [anon_sym_become] = ACTIONS(3636),
    [anon_sym_break] = ACTIONS(3636),
    [anon_sym_const] = ACTIONS(3636),
    [anon_sym_continue] = ACTIONS(3636),
    [anon_sym_default] = ACTIONS(3636),
    [anon_sym_enum] = ACTIONS(3636),
    [anon_sym_fn] = ACTIONS(3636),
    [anon_sym_for] = ACTIONS(3636),
    [anon_sym_gen] = ACTIONS(3636),
    [anon_sym_if] = ACTIONS(3636),
    [anon_sym_impl] = ACTIONS(3636),
    [anon_sym_let] = ACTIONS(3636),
    [anon_sym_loop] = ACTIONS(3636),
    [anon_sym_match] = ACTIONS(3636),
    [anon_sym_mod] = ACTIONS(3636),
    [anon_sym_pub] = ACTIONS(3636),
    [anon_sym_return] = ACTIONS(3636),
    [anon_sym_raw] = ACTIONS(3636),
    [anon_sym_static] = ACTIONS(3636),
    [anon_sym_struct] = ACTIONS(3636),
    [anon_sym_trait] = ACTIONS(3636),
    [anon_sym_type] = ACTIONS(3636),
    [anon_sym_union] = ACTIONS(3636),
    [anon_sym_unsafe] = ACTIONS(3636),
    [anon_sym_use] = ACTIONS(3636),
    [anon_sym_while] = ACTIONS(3636),
    [anon_sym_extern] = ACTIONS(3636),
    [anon_sym_safe] = ACTIONS(3636),
    [anon_sym_yield] = ACTIONS(3636),
    [anon_sym_move] = ACTIONS(3636),
    [anon_sym_try] = ACTIONS(3636),
    [sym_integer_literal] = ACTIONS(3634),
    [aux_sym_string_literal_token1] = ACTIONS(3634),
    [sym_char_literal] = ACTIONS(3634),
    [anon_sym_true] = ACTIONS(3636),
    [anon_sym_false] = ACTIONS(3636),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3636),
    [sym_super] = ACTIONS(3636),
    [sym_crate] = ACTIONS(3636),
    [aux_sym_metavariable_token1] = ACTIONS(3634),
    [sym__raw_string_literal_start] = ACTIONS(3634),
    [sym_float_literal] = ACTIONS(3634),
    [sym__outer_block_doc_comment_start] = ACTIONS(3634),
    [sym__outer_line_doc_comment_start] = ACTIONS(3634),
  },
  [STATE(967)] = {
    [sym_line_comment] = STATE(967),
    [sym_block_comment] = STATE(967),
    [ts_builtin_sym_end] = ACTIONS(3638),
    [sym_identifier] = ACTIONS(3640),
    [anon_sym_SEMI] = ACTIONS(3638),
    [anon_sym_macro] = ACTIONS(3640),
    [anon_sym_LPAREN] = ACTIONS(3638),
    [anon_sym_LBRACE] = ACTIONS(3638),
    [anon_sym_RBRACE] = ACTIONS(3638),
    [anon_sym_macro_rules] = ACTIONS(3640),
    [anon_sym_BANG] = ACTIONS(3638),
    [anon_sym_LBRACK] = ACTIONS(3638),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_u8] = ACTIONS(3640),
    [anon_sym_i8] = ACTIONS(3640),
    [anon_sym_u16] = ACTIONS(3640),
    [anon_sym_i16] = ACTIONS(3640),
    [anon_sym_u32] = ACTIONS(3640),
    [anon_sym_i32] = ACTIONS(3640),
    [anon_sym_u64] = ACTIONS(3640),
    [anon_sym_i64] = ACTIONS(3640),
    [anon_sym_u128] = ACTIONS(3640),
    [anon_sym_i128] = ACTIONS(3640),
    [anon_sym_isize] = ACTIONS(3640),
    [anon_sym_usize] = ACTIONS(3640),
    [anon_sym_f32] = ACTIONS(3640),
    [anon_sym_f64] = ACTIONS(3640),
    [anon_sym_bool] = ACTIONS(3640),
    [anon_sym_str] = ACTIONS(3640),
    [anon_sym_char] = ACTIONS(3640),
    [anon_sym_DASH] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(3638),
    [anon_sym_PIPE] = ACTIONS(3638),
    [anon_sym_LT] = ACTIONS(3638),
    [anon_sym_DOT_DOT] = ACTIONS(3638),
    [anon_sym_COLON_COLON] = ACTIONS(3638),
    [anon_sym_POUND] = ACTIONS(3638),
    [anon_sym_SQUOTE] = ACTIONS(3640),
    [anon_sym_async] = ACTIONS(3640),
    [anon_sym_auto] = ACTIONS(3640),
    [anon_sym_await] = ACTIONS(3640),
    [anon_sym_become] = ACTIONS(3640),
    [anon_sym_break] = ACTIONS(3640),
    [anon_sym_const] = ACTIONS(3640),
    [anon_sym_continue] = ACTIONS(3640),
    [anon_sym_default] = ACTIONS(3640),
    [anon_sym_enum] = ACTIONS(3640),
    [anon_sym_fn] = ACTIONS(3640),
    [anon_sym_for] = ACTIONS(3640),
    [anon_sym_gen] = ACTIONS(3640),
    [anon_sym_if] = ACTIONS(3640),
    [anon_sym_impl] = ACTIONS(3640),
    [anon_sym_let] = ACTIONS(3640),
    [anon_sym_loop] = ACTIONS(3640),
    [anon_sym_match] = ACTIONS(3640),
    [anon_sym_mod] = ACTIONS(3640),
    [anon_sym_pub] = ACTIONS(3640),
    [anon_sym_return] = ACTIONS(3640),
    [anon_sym_raw] = ACTIONS(3640),
    [anon_sym_static] = ACTIONS(3640),
    [anon_sym_struct] = ACTIONS(3640),
    [anon_sym_trait] = ACTIONS(3640),
    [anon_sym_type] = ACTIONS(3640),
    [anon_sym_union] = ACTIONS(3640),
    [anon_sym_unsafe] = ACTIONS(3640),
    [anon_sym_use] = ACTIONS(3640),
    [anon_sym_while] = ACTIONS(3640),
    [anon_sym_extern] = ACTIONS(3640),
    [anon_sym_safe] = ACTIONS(3640),
    [anon_sym_yield] = ACTIONS(3640),
    [anon_sym_move] = ACTIONS(3640),
    [anon_sym_try] = ACTIONS(3640),
    [sym_integer_literal] = ACTIONS(3638),
    [aux_sym_string_literal_token1] = ACTIONS(3638),
    [sym_char_literal] = ACTIONS(3638),
    [anon_sym_true] = ACTIONS(3640),
    [anon_sym_false] = ACTIONS(3640),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3640),
    [sym_super] = ACTIONS(3640),
    [sym_crate] = ACTIONS(3640),
    [aux_sym_metavariable_token1] = ACTIONS(3638),
    [sym__raw_string_literal_start] = ACTIONS(3638),
    [sym_float_literal] = ACTIONS(3638),
    [sym__outer_block_doc_comment_start] = ACTIONS(3638),
    [sym__outer_line_doc_comment_start] = ACTIONS(3638),
  },
  [STATE(968)] = {
    [sym_line_comment] = STATE(968),
    [sym_block_comment] = STATE(968),
    [ts_builtin_sym_end] = ACTIONS(3642),
    [sym_identifier] = ACTIONS(3644),
    [anon_sym_SEMI] = ACTIONS(3642),
    [anon_sym_macro] = ACTIONS(3644),
    [anon_sym_LPAREN] = ACTIONS(3642),
    [anon_sym_LBRACE] = ACTIONS(3642),
    [anon_sym_RBRACE] = ACTIONS(3642),
    [anon_sym_macro_rules] = ACTIONS(3644),
    [anon_sym_BANG] = ACTIONS(3642),
    [anon_sym_LBRACK] = ACTIONS(3642),
    [anon_sym_STAR] = ACTIONS(3642),
    [anon_sym_u8] = ACTIONS(3644),
    [anon_sym_i8] = ACTIONS(3644),
    [anon_sym_u16] = ACTIONS(3644),
    [anon_sym_i16] = ACTIONS(3644),
    [anon_sym_u32] = ACTIONS(3644),
    [anon_sym_i32] = ACTIONS(3644),
    [anon_sym_u64] = ACTIONS(3644),
    [anon_sym_i64] = ACTIONS(3644),
    [anon_sym_u128] = ACTIONS(3644),
    [anon_sym_i128] = ACTIONS(3644),
    [anon_sym_isize] = ACTIONS(3644),
    [anon_sym_usize] = ACTIONS(3644),
    [anon_sym_f32] = ACTIONS(3644),
    [anon_sym_f64] = ACTIONS(3644),
    [anon_sym_bool] = ACTIONS(3644),
    [anon_sym_str] = ACTIONS(3644),
    [anon_sym_char] = ACTIONS(3644),
    [anon_sym_DASH] = ACTIONS(3642),
    [anon_sym_AMP] = ACTIONS(3642),
    [anon_sym_PIPE] = ACTIONS(3642),
    [anon_sym_LT] = ACTIONS(3642),
    [anon_sym_DOT_DOT] = ACTIONS(3642),
    [anon_sym_COLON_COLON] = ACTIONS(3642),
    [anon_sym_POUND] = ACTIONS(3642),
    [anon_sym_SQUOTE] = ACTIONS(3644),
    [anon_sym_async] = ACTIONS(3644),
    [anon_sym_auto] = ACTIONS(3644),
    [anon_sym_await] = ACTIONS(3644),
    [anon_sym_become] = ACTIONS(3644),
    [anon_sym_break] = ACTIONS(3644),
    [anon_sym_const] = ACTIONS(3644),
    [anon_sym_continue] = ACTIONS(3644),
    [anon_sym_default] = ACTIONS(3644),
    [anon_sym_enum] = ACTIONS(3644),
    [anon_sym_fn] = ACTIONS(3644),
    [anon_sym_for] = ACTIONS(3644),
    [anon_sym_gen] = ACTIONS(3644),
    [anon_sym_if] = ACTIONS(3644),
    [anon_sym_impl] = ACTIONS(3644),
    [anon_sym_let] = ACTIONS(3644),
    [anon_sym_loop] = ACTIONS(3644),
    [anon_sym_match] = ACTIONS(3644),
    [anon_sym_mod] = ACTIONS(3644),
    [anon_sym_pub] = ACTIONS(3644),
    [anon_sym_return] = ACTIONS(3644),
    [anon_sym_raw] = ACTIONS(3644),
    [anon_sym_static] = ACTIONS(3644),
    [anon_sym_struct] = ACTIONS(3644),
    [anon_sym_trait] = ACTIONS(3644),
    [anon_sym_type] = ACTIONS(3644),
    [anon_sym_union] = ACTIONS(3644),
    [anon_sym_unsafe] = ACTIONS(3644),
    [anon_sym_use] = ACTIONS(3644),
    [anon_sym_while] = ACTIONS(3644),
    [anon_sym_extern] = ACTIONS(3644),
    [anon_sym_safe] = ACTIONS(3644),
    [anon_sym_yield] = ACTIONS(3644),
    [anon_sym_move] = ACTIONS(3644),
    [anon_sym_try] = ACTIONS(3644),
    [sym_integer_literal] = ACTIONS(3642),
    [aux_sym_string_literal_token1] = ACTIONS(3642),
    [sym_char_literal] = ACTIONS(3642),
    [anon_sym_true] = ACTIONS(3644),
    [anon_sym_false] = ACTIONS(3644),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3644),
    [sym_super] = ACTIONS(3644),
    [sym_crate] = ACTIONS(3644),
    [aux_sym_metavariable_token1] = ACTIONS(3642),
    [sym__raw_string_literal_start] = ACTIONS(3642),
    [sym_float_literal] = ACTIONS(3642),
    [sym__outer_block_doc_comment_start] = ACTIONS(3642),
    [sym__outer_line_doc_comment_start] = ACTIONS(3642),
  },
  [STATE(969)] = {
    [sym_line_comment] = STATE(969),
    [sym_block_comment] = STATE(969),
    [ts_builtin_sym_end] = ACTIONS(3646),
    [sym_identifier] = ACTIONS(3648),
    [anon_sym_SEMI] = ACTIONS(3646),
    [anon_sym_macro] = ACTIONS(3648),
    [anon_sym_LPAREN] = ACTIONS(3646),
    [anon_sym_LBRACE] = ACTIONS(3646),
    [anon_sym_RBRACE] = ACTIONS(3646),
    [anon_sym_macro_rules] = ACTIONS(3648),
    [anon_sym_BANG] = ACTIONS(3646),
    [anon_sym_LBRACK] = ACTIONS(3646),
    [anon_sym_STAR] = ACTIONS(3646),
    [anon_sym_u8] = ACTIONS(3648),
    [anon_sym_i8] = ACTIONS(3648),
    [anon_sym_u16] = ACTIONS(3648),
    [anon_sym_i16] = ACTIONS(3648),
    [anon_sym_u32] = ACTIONS(3648),
    [anon_sym_i32] = ACTIONS(3648),
    [anon_sym_u64] = ACTIONS(3648),
    [anon_sym_i64] = ACTIONS(3648),
    [anon_sym_u128] = ACTIONS(3648),
    [anon_sym_i128] = ACTIONS(3648),
    [anon_sym_isize] = ACTIONS(3648),
    [anon_sym_usize] = ACTIONS(3648),
    [anon_sym_f32] = ACTIONS(3648),
    [anon_sym_f64] = ACTIONS(3648),
    [anon_sym_bool] = ACTIONS(3648),
    [anon_sym_str] = ACTIONS(3648),
    [anon_sym_char] = ACTIONS(3648),
    [anon_sym_DASH] = ACTIONS(3646),
    [anon_sym_AMP] = ACTIONS(3646),
    [anon_sym_PIPE] = ACTIONS(3646),
    [anon_sym_LT] = ACTIONS(3646),
    [anon_sym_DOT_DOT] = ACTIONS(3646),
    [anon_sym_COLON_COLON] = ACTIONS(3646),
    [anon_sym_POUND] = ACTIONS(3646),
    [anon_sym_SQUOTE] = ACTIONS(3648),
    [anon_sym_async] = ACTIONS(3648),
    [anon_sym_auto] = ACTIONS(3648),
    [anon_sym_await] = ACTIONS(3648),
    [anon_sym_become] = ACTIONS(3648),
    [anon_sym_break] = ACTIONS(3648),
    [anon_sym_const] = ACTIONS(3648),
    [anon_sym_continue] = ACTIONS(3648),
    [anon_sym_default] = ACTIONS(3648),
    [anon_sym_enum] = ACTIONS(3648),
    [anon_sym_fn] = ACTIONS(3648),
    [anon_sym_for] = ACTIONS(3648),
    [anon_sym_gen] = ACTIONS(3648),
    [anon_sym_if] = ACTIONS(3648),
    [anon_sym_impl] = ACTIONS(3648),
    [anon_sym_let] = ACTIONS(3648),
    [anon_sym_loop] = ACTIONS(3648),
    [anon_sym_match] = ACTIONS(3648),
    [anon_sym_mod] = ACTIONS(3648),
    [anon_sym_pub] = ACTIONS(3648),
    [anon_sym_return] = ACTIONS(3648),
    [anon_sym_raw] = ACTIONS(3648),
    [anon_sym_static] = ACTIONS(3648),
    [anon_sym_struct] = ACTIONS(3648),
    [anon_sym_trait] = ACTIONS(3648),
    [anon_sym_type] = ACTIONS(3648),
    [anon_sym_union] = ACTIONS(3648),
    [anon_sym_unsafe] = ACTIONS(3648),
    [anon_sym_use] = ACTIONS(3648),
    [anon_sym_while] = ACTIONS(3648),
    [anon_sym_extern] = ACTIONS(3648),
    [anon_sym_safe] = ACTIONS(3648),
    [anon_sym_yield] = ACTIONS(3648),
    [anon_sym_move] = ACTIONS(3648),
    [anon_sym_try] = ACTIONS(3648),
    [sym_integer_literal] = ACTIONS(3646),
    [aux_sym_string_literal_token1] = ACTIONS(3646),
    [sym_char_literal] = ACTIONS(3646),
    [anon_sym_true] = ACTIONS(3648),
    [anon_sym_false] = ACTIONS(3648),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3648),
    [sym_super] = ACTIONS(3648),
    [sym_crate] = ACTIONS(3648),
    [aux_sym_metavariable_token1] = ACTIONS(3646),
    [sym__raw_string_literal_start] = ACTIONS(3646),
    [sym_float_literal] = ACTIONS(3646),
    [sym__outer_block_doc_comment_start] = ACTIONS(3646),
    [sym__outer_line_doc_comment_start] = ACTIONS(3646),
  },
  [STATE(970)] = {
    [sym_line_comment] = STATE(970),
    [sym_block_comment] = STATE(970),
    [ts_builtin_sym_end] = ACTIONS(3650),
    [sym_identifier] = ACTIONS(3652),
    [anon_sym_SEMI] = ACTIONS(3650),
    [anon_sym_macro] = ACTIONS(3652),
    [anon_sym_LPAREN] = ACTIONS(3650),
    [anon_sym_LBRACE] = ACTIONS(3650),
    [anon_sym_RBRACE] = ACTIONS(3650),
    [anon_sym_macro_rules] = ACTIONS(3652),
    [anon_sym_BANG] = ACTIONS(3650),
    [anon_sym_LBRACK] = ACTIONS(3650),
    [anon_sym_STAR] = ACTIONS(3650),
    [anon_sym_u8] = ACTIONS(3652),
    [anon_sym_i8] = ACTIONS(3652),
    [anon_sym_u16] = ACTIONS(3652),
    [anon_sym_i16] = ACTIONS(3652),
    [anon_sym_u32] = ACTIONS(3652),
    [anon_sym_i32] = ACTIONS(3652),
    [anon_sym_u64] = ACTIONS(3652),
    [anon_sym_i64] = ACTIONS(3652),
    [anon_sym_u128] = ACTIONS(3652),
    [anon_sym_i128] = ACTIONS(3652),
    [anon_sym_isize] = ACTIONS(3652),
    [anon_sym_usize] = ACTIONS(3652),
    [anon_sym_f32] = ACTIONS(3652),
    [anon_sym_f64] = ACTIONS(3652),
    [anon_sym_bool] = ACTIONS(3652),
    [anon_sym_str] = ACTIONS(3652),
    [anon_sym_char] = ACTIONS(3652),
    [anon_sym_DASH] = ACTIONS(3650),
    [anon_sym_AMP] = ACTIONS(3650),
    [anon_sym_PIPE] = ACTIONS(3650),
    [anon_sym_LT] = ACTIONS(3650),
    [anon_sym_DOT_DOT] = ACTIONS(3650),
    [anon_sym_COLON_COLON] = ACTIONS(3650),
    [anon_sym_POUND] = ACTIONS(3650),
    [anon_sym_SQUOTE] = ACTIONS(3652),
    [anon_sym_async] = ACTIONS(3652),
    [anon_sym_auto] = ACTIONS(3652),
    [anon_sym_await] = ACTIONS(3652),
    [anon_sym_become] = ACTIONS(3652),
    [anon_sym_break] = ACTIONS(3652),
    [anon_sym_const] = ACTIONS(3652),
    [anon_sym_continue] = ACTIONS(3652),
    [anon_sym_default] = ACTIONS(3652),
    [anon_sym_enum] = ACTIONS(3652),
    [anon_sym_fn] = ACTIONS(3652),
    [anon_sym_for] = ACTIONS(3652),
    [anon_sym_gen] = ACTIONS(3652),
    [anon_sym_if] = ACTIONS(3652),
    [anon_sym_impl] = ACTIONS(3652),
    [anon_sym_let] = ACTIONS(3652),
    [anon_sym_loop] = ACTIONS(3652),
    [anon_sym_match] = ACTIONS(3652),
    [anon_sym_mod] = ACTIONS(3652),
    [anon_sym_pub] = ACTIONS(3652),
    [anon_sym_return] = ACTIONS(3652),
    [anon_sym_raw] = ACTIONS(3652),
    [anon_sym_static] = ACTIONS(3652),
    [anon_sym_struct] = ACTIONS(3652),
    [anon_sym_trait] = ACTIONS(3652),
    [anon_sym_type] = ACTIONS(3652),
    [anon_sym_union] = ACTIONS(3652),
    [anon_sym_unsafe] = ACTIONS(3652),
    [anon_sym_use] = ACTIONS(3652),
    [anon_sym_while] = ACTIONS(3652),
    [anon_sym_extern] = ACTIONS(3652),
    [anon_sym_safe] = ACTIONS(3652),
    [anon_sym_yield] = ACTIONS(3652),
    [anon_sym_move] = ACTIONS(3652),
    [anon_sym_try] = ACTIONS(3652),
    [sym_integer_literal] = ACTIONS(3650),
    [aux_sym_string_literal_token1] = ACTIONS(3650),
    [sym_char_literal] = ACTIONS(3650),
    [anon_sym_true] = ACTIONS(3652),
    [anon_sym_false] = ACTIONS(3652),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3652),
    [sym_super] = ACTIONS(3652),
    [sym_crate] = ACTIONS(3652),
    [aux_sym_metavariable_token1] = ACTIONS(3650),
    [sym__raw_string_literal_start] = ACTIONS(3650),
    [sym_float_literal] = ACTIONS(3650),
    [sym__outer_block_doc_comment_start] = ACTIONS(3650),
    [sym__outer_line_doc_comment_start] = ACTIONS(3650),
  },
  [STATE(971)] = {
    [sym_line_comment] = STATE(971),
    [sym_block_comment] = STATE(971),
    [ts_builtin_sym_end] = ACTIONS(3654),
    [sym_identifier] = ACTIONS(3656),
    [anon_sym_SEMI] = ACTIONS(3654),
    [anon_sym_macro] = ACTIONS(3656),
    [anon_sym_LPAREN] = ACTIONS(3654),
    [anon_sym_LBRACE] = ACTIONS(3654),
    [anon_sym_RBRACE] = ACTIONS(3654),
    [anon_sym_macro_rules] = ACTIONS(3656),
    [anon_sym_BANG] = ACTIONS(3654),
    [anon_sym_LBRACK] = ACTIONS(3654),
    [anon_sym_STAR] = ACTIONS(3654),
    [anon_sym_u8] = ACTIONS(3656),
    [anon_sym_i8] = ACTIONS(3656),
    [anon_sym_u16] = ACTIONS(3656),
    [anon_sym_i16] = ACTIONS(3656),
    [anon_sym_u32] = ACTIONS(3656),
    [anon_sym_i32] = ACTIONS(3656),
    [anon_sym_u64] = ACTIONS(3656),
    [anon_sym_i64] = ACTIONS(3656),
    [anon_sym_u128] = ACTIONS(3656),
    [anon_sym_i128] = ACTIONS(3656),
    [anon_sym_isize] = ACTIONS(3656),
    [anon_sym_usize] = ACTIONS(3656),
    [anon_sym_f32] = ACTIONS(3656),
    [anon_sym_f64] = ACTIONS(3656),
    [anon_sym_bool] = ACTIONS(3656),
    [anon_sym_str] = ACTIONS(3656),
    [anon_sym_char] = ACTIONS(3656),
    [anon_sym_DASH] = ACTIONS(3654),
    [anon_sym_AMP] = ACTIONS(3654),
    [anon_sym_PIPE] = ACTIONS(3654),
    [anon_sym_LT] = ACTIONS(3654),
    [anon_sym_DOT_DOT] = ACTIONS(3654),
    [anon_sym_COLON_COLON] = ACTIONS(3654),
    [anon_sym_POUND] = ACTIONS(3654),
    [anon_sym_SQUOTE] = ACTIONS(3656),
    [anon_sym_async] = ACTIONS(3656),
    [anon_sym_auto] = ACTIONS(3656),
    [anon_sym_await] = ACTIONS(3656),
    [anon_sym_become] = ACTIONS(3656),
    [anon_sym_break] = ACTIONS(3656),
    [anon_sym_const] = ACTIONS(3656),
    [anon_sym_continue] = ACTIONS(3656),
    [anon_sym_default] = ACTIONS(3656),
    [anon_sym_enum] = ACTIONS(3656),
    [anon_sym_fn] = ACTIONS(3656),
    [anon_sym_for] = ACTIONS(3656),
    [anon_sym_gen] = ACTIONS(3656),
    [anon_sym_if] = ACTIONS(3656),
    [anon_sym_impl] = ACTIONS(3656),
    [anon_sym_let] = ACTIONS(3656),
    [anon_sym_loop] = ACTIONS(3656),
    [anon_sym_match] = ACTIONS(3656),
    [anon_sym_mod] = ACTIONS(3656),
    [anon_sym_pub] = ACTIONS(3656),
    [anon_sym_return] = ACTIONS(3656),
    [anon_sym_raw] = ACTIONS(3656),
    [anon_sym_static] = ACTIONS(3656),
    [anon_sym_struct] = ACTIONS(3656),
    [anon_sym_trait] = ACTIONS(3656),
    [anon_sym_type] = ACTIONS(3656),
    [anon_sym_union] = ACTIONS(3656),
    [anon_sym_unsafe] = ACTIONS(3656),
    [anon_sym_use] = ACTIONS(3656),
    [anon_sym_while] = ACTIONS(3656),
    [anon_sym_extern] = ACTIONS(3656),
    [anon_sym_safe] = ACTIONS(3656),
    [anon_sym_yield] = ACTIONS(3656),
    [anon_sym_move] = ACTIONS(3656),
    [anon_sym_try] = ACTIONS(3656),
    [sym_integer_literal] = ACTIONS(3654),
    [aux_sym_string_literal_token1] = ACTIONS(3654),
    [sym_char_literal] = ACTIONS(3654),
    [anon_sym_true] = ACTIONS(3656),
    [anon_sym_false] = ACTIONS(3656),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3656),
    [sym_super] = ACTIONS(3656),
    [sym_crate] = ACTIONS(3656),
    [aux_sym_metavariable_token1] = ACTIONS(3654),
    [sym__raw_string_literal_start] = ACTIONS(3654),
    [sym_float_literal] = ACTIONS(3654),
    [sym__outer_block_doc_comment_start] = ACTIONS(3654),
    [sym__outer_line_doc_comment_start] = ACTIONS(3654),
  },
  [STATE(972)] = {
    [sym_line_comment] = STATE(972),
    [sym_block_comment] = STATE(972),
    [ts_builtin_sym_end] = ACTIONS(3658),
    [sym_identifier] = ACTIONS(3660),
    [anon_sym_SEMI] = ACTIONS(3658),
    [anon_sym_macro] = ACTIONS(3660),
    [anon_sym_LPAREN] = ACTIONS(3658),
    [anon_sym_LBRACE] = ACTIONS(3658),
    [anon_sym_RBRACE] = ACTIONS(3658),
    [anon_sym_macro_rules] = ACTIONS(3660),
    [anon_sym_BANG] = ACTIONS(3658),
    [anon_sym_LBRACK] = ACTIONS(3658),
    [anon_sym_STAR] = ACTIONS(3658),
    [anon_sym_u8] = ACTIONS(3660),
    [anon_sym_i8] = ACTIONS(3660),
    [anon_sym_u16] = ACTIONS(3660),
    [anon_sym_i16] = ACTIONS(3660),
    [anon_sym_u32] = ACTIONS(3660),
    [anon_sym_i32] = ACTIONS(3660),
    [anon_sym_u64] = ACTIONS(3660),
    [anon_sym_i64] = ACTIONS(3660),
    [anon_sym_u128] = ACTIONS(3660),
    [anon_sym_i128] = ACTIONS(3660),
    [anon_sym_isize] = ACTIONS(3660),
    [anon_sym_usize] = ACTIONS(3660),
    [anon_sym_f32] = ACTIONS(3660),
    [anon_sym_f64] = ACTIONS(3660),
    [anon_sym_bool] = ACTIONS(3660),
    [anon_sym_str] = ACTIONS(3660),
    [anon_sym_char] = ACTIONS(3660),
    [anon_sym_DASH] = ACTIONS(3658),
    [anon_sym_AMP] = ACTIONS(3658),
    [anon_sym_PIPE] = ACTIONS(3658),
    [anon_sym_LT] = ACTIONS(3658),
    [anon_sym_DOT_DOT] = ACTIONS(3658),
    [anon_sym_COLON_COLON] = ACTIONS(3658),
    [anon_sym_POUND] = ACTIONS(3658),
    [anon_sym_SQUOTE] = ACTIONS(3660),
    [anon_sym_async] = ACTIONS(3660),
    [anon_sym_auto] = ACTIONS(3660),
    [anon_sym_await] = ACTIONS(3660),
    [anon_sym_become] = ACTIONS(3660),
    [anon_sym_break] = ACTIONS(3660),
    [anon_sym_const] = ACTIONS(3660),
    [anon_sym_continue] = ACTIONS(3660),
    [anon_sym_default] = ACTIONS(3660),
    [anon_sym_enum] = ACTIONS(3660),
    [anon_sym_fn] = ACTIONS(3660),
    [anon_sym_for] = ACTIONS(3660),
    [anon_sym_gen] = ACTIONS(3660),
    [anon_sym_if] = ACTIONS(3660),
    [anon_sym_impl] = ACTIONS(3660),
    [anon_sym_let] = ACTIONS(3660),
    [anon_sym_loop] = ACTIONS(3660),
    [anon_sym_match] = ACTIONS(3660),
    [anon_sym_mod] = ACTIONS(3660),
    [anon_sym_pub] = ACTIONS(3660),
    [anon_sym_return] = ACTIONS(3660),
    [anon_sym_raw] = ACTIONS(3660),
    [anon_sym_static] = ACTIONS(3660),
    [anon_sym_struct] = ACTIONS(3660),
    [anon_sym_trait] = ACTIONS(3660),
    [anon_sym_type] = ACTIONS(3660),
    [anon_sym_union] = ACTIONS(3660),
    [anon_sym_unsafe] = ACTIONS(3660),
    [anon_sym_use] = ACTIONS(3660),
    [anon_sym_while] = ACTIONS(3660),
    [anon_sym_extern] = ACTIONS(3660),
    [anon_sym_safe] = ACTIONS(3660),
    [anon_sym_yield] = ACTIONS(3660),
    [anon_sym_move] = ACTIONS(3660),
    [anon_sym_try] = ACTIONS(3660),
    [sym_integer_literal] = ACTIONS(3658),
    [aux_sym_string_literal_token1] = ACTIONS(3658),
    [sym_char_literal] = ACTIONS(3658),
    [anon_sym_true] = ACTIONS(3660),
    [anon_sym_false] = ACTIONS(3660),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3660),
    [sym_super] = ACTIONS(3660),
    [sym_crate] = ACTIONS(3660),
    [aux_sym_metavariable_token1] = ACTIONS(3658),
    [sym__raw_string_literal_start] = ACTIONS(3658),
    [sym_float_literal] = ACTIONS(3658),
    [sym__outer_block_doc_comment_start] = ACTIONS(3658),
    [sym__outer_line_doc_comment_start] = ACTIONS(3658),
  },
  [STATE(973)] = {
    [sym_line_comment] = STATE(973),
    [sym_block_comment] = STATE(973),
    [ts_builtin_sym_end] = ACTIONS(3662),
    [sym_identifier] = ACTIONS(3664),
    [anon_sym_SEMI] = ACTIONS(3662),
    [anon_sym_macro] = ACTIONS(3664),
    [anon_sym_LPAREN] = ACTIONS(3662),
    [anon_sym_LBRACE] = ACTIONS(3662),
    [anon_sym_RBRACE] = ACTIONS(3662),
    [anon_sym_macro_rules] = ACTIONS(3664),
    [anon_sym_BANG] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(3662),
    [anon_sym_STAR] = ACTIONS(3662),
    [anon_sym_u8] = ACTIONS(3664),
    [anon_sym_i8] = ACTIONS(3664),
    [anon_sym_u16] = ACTIONS(3664),
    [anon_sym_i16] = ACTIONS(3664),
    [anon_sym_u32] = ACTIONS(3664),
    [anon_sym_i32] = ACTIONS(3664),
    [anon_sym_u64] = ACTIONS(3664),
    [anon_sym_i64] = ACTIONS(3664),
    [anon_sym_u128] = ACTIONS(3664),
    [anon_sym_i128] = ACTIONS(3664),
    [anon_sym_isize] = ACTIONS(3664),
    [anon_sym_usize] = ACTIONS(3664),
    [anon_sym_f32] = ACTIONS(3664),
    [anon_sym_f64] = ACTIONS(3664),
    [anon_sym_bool] = ACTIONS(3664),
    [anon_sym_str] = ACTIONS(3664),
    [anon_sym_char] = ACTIONS(3664),
    [anon_sym_DASH] = ACTIONS(3662),
    [anon_sym_AMP] = ACTIONS(3662),
    [anon_sym_PIPE] = ACTIONS(3662),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_DOT_DOT] = ACTIONS(3662),
    [anon_sym_COLON_COLON] = ACTIONS(3662),
    [anon_sym_POUND] = ACTIONS(3662),
    [anon_sym_SQUOTE] = ACTIONS(3664),
    [anon_sym_async] = ACTIONS(3664),
    [anon_sym_auto] = ACTIONS(3664),
    [anon_sym_await] = ACTIONS(3664),
    [anon_sym_become] = ACTIONS(3664),
    [anon_sym_break] = ACTIONS(3664),
    [anon_sym_const] = ACTIONS(3664),
    [anon_sym_continue] = ACTIONS(3664),
    [anon_sym_default] = ACTIONS(3664),
    [anon_sym_enum] = ACTIONS(3664),
    [anon_sym_fn] = ACTIONS(3664),
    [anon_sym_for] = ACTIONS(3664),
    [anon_sym_gen] = ACTIONS(3664),
    [anon_sym_if] = ACTIONS(3664),
    [anon_sym_impl] = ACTIONS(3664),
    [anon_sym_let] = ACTIONS(3664),
    [anon_sym_loop] = ACTIONS(3664),
    [anon_sym_match] = ACTIONS(3664),
    [anon_sym_mod] = ACTIONS(3664),
    [anon_sym_pub] = ACTIONS(3664),
    [anon_sym_return] = ACTIONS(3664),
    [anon_sym_raw] = ACTIONS(3664),
    [anon_sym_static] = ACTIONS(3664),
    [anon_sym_struct] = ACTIONS(3664),
    [anon_sym_trait] = ACTIONS(3664),
    [anon_sym_type] = ACTIONS(3664),
    [anon_sym_union] = ACTIONS(3664),
    [anon_sym_unsafe] = ACTIONS(3664),
    [anon_sym_use] = ACTIONS(3664),
    [anon_sym_while] = ACTIONS(3664),
    [anon_sym_extern] = ACTIONS(3664),
    [anon_sym_safe] = ACTIONS(3664),
    [anon_sym_yield] = ACTIONS(3664),
    [anon_sym_move] = ACTIONS(3664),
    [anon_sym_try] = ACTIONS(3664),
    [sym_integer_literal] = ACTIONS(3662),
    [aux_sym_string_literal_token1] = ACTIONS(3662),
    [sym_char_literal] = ACTIONS(3662),
    [anon_sym_true] = ACTIONS(3664),
    [anon_sym_false] = ACTIONS(3664),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3664),
    [sym_super] = ACTIONS(3664),
    [sym_crate] = ACTIONS(3664),
    [aux_sym_metavariable_token1] = ACTIONS(3662),
    [sym__raw_string_literal_start] = ACTIONS(3662),
    [sym_float_literal] = ACTIONS(3662),
    [sym__outer_block_doc_comment_start] = ACTIONS(3662),
    [sym__outer_line_doc_comment_start] = ACTIONS(3662),
  },
  [STATE(974)] = {
    [sym_line_comment] = STATE(974),
    [sym_block_comment] = STATE(974),
    [ts_builtin_sym_end] = ACTIONS(3666),
    [sym_identifier] = ACTIONS(3668),
    [anon_sym_SEMI] = ACTIONS(3666),
    [anon_sym_macro] = ACTIONS(3668),
    [anon_sym_LPAREN] = ACTIONS(3666),
    [anon_sym_LBRACE] = ACTIONS(3666),
    [anon_sym_RBRACE] = ACTIONS(3666),
    [anon_sym_macro_rules] = ACTIONS(3668),
    [anon_sym_BANG] = ACTIONS(3666),
    [anon_sym_LBRACK] = ACTIONS(3666),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_u8] = ACTIONS(3668),
    [anon_sym_i8] = ACTIONS(3668),
    [anon_sym_u16] = ACTIONS(3668),
    [anon_sym_i16] = ACTIONS(3668),
    [anon_sym_u32] = ACTIONS(3668),
    [anon_sym_i32] = ACTIONS(3668),
    [anon_sym_u64] = ACTIONS(3668),
    [anon_sym_i64] = ACTIONS(3668),
    [anon_sym_u128] = ACTIONS(3668),
    [anon_sym_i128] = ACTIONS(3668),
    [anon_sym_isize] = ACTIONS(3668),
    [anon_sym_usize] = ACTIONS(3668),
    [anon_sym_f32] = ACTIONS(3668),
    [anon_sym_f64] = ACTIONS(3668),
    [anon_sym_bool] = ACTIONS(3668),
    [anon_sym_str] = ACTIONS(3668),
    [anon_sym_char] = ACTIONS(3668),
    [anon_sym_DASH] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3666),
    [anon_sym_PIPE] = ACTIONS(3666),
    [anon_sym_LT] = ACTIONS(3666),
    [anon_sym_DOT_DOT] = ACTIONS(3666),
    [anon_sym_COLON_COLON] = ACTIONS(3666),
    [anon_sym_POUND] = ACTIONS(3666),
    [anon_sym_SQUOTE] = ACTIONS(3668),
    [anon_sym_async] = ACTIONS(3668),
    [anon_sym_auto] = ACTIONS(3668),
    [anon_sym_await] = ACTIONS(3668),
    [anon_sym_become] = ACTIONS(3668),
    [anon_sym_break] = ACTIONS(3668),
    [anon_sym_const] = ACTIONS(3668),
    [anon_sym_continue] = ACTIONS(3668),
    [anon_sym_default] = ACTIONS(3668),
    [anon_sym_enum] = ACTIONS(3668),
    [anon_sym_fn] = ACTIONS(3668),
    [anon_sym_for] = ACTIONS(3668),
    [anon_sym_gen] = ACTIONS(3668),
    [anon_sym_if] = ACTIONS(3668),
    [anon_sym_impl] = ACTIONS(3668),
    [anon_sym_let] = ACTIONS(3668),
    [anon_sym_loop] = ACTIONS(3668),
    [anon_sym_match] = ACTIONS(3668),
    [anon_sym_mod] = ACTIONS(3668),
    [anon_sym_pub] = ACTIONS(3668),
    [anon_sym_return] = ACTIONS(3668),
    [anon_sym_raw] = ACTIONS(3668),
    [anon_sym_static] = ACTIONS(3668),
    [anon_sym_struct] = ACTIONS(3668),
    [anon_sym_trait] = ACTIONS(3668),
    [anon_sym_type] = ACTIONS(3668),
    [anon_sym_union] = ACTIONS(3668),
    [anon_sym_unsafe] = ACTIONS(3668),
    [anon_sym_use] = ACTIONS(3668),
    [anon_sym_while] = ACTIONS(3668),
    [anon_sym_extern] = ACTIONS(3668),
    [anon_sym_safe] = ACTIONS(3668),
    [anon_sym_yield] = ACTIONS(3668),
    [anon_sym_move] = ACTIONS(3668),
    [anon_sym_try] = ACTIONS(3668),
    [sym_integer_literal] = ACTIONS(3666),
    [aux_sym_string_literal_token1] = ACTIONS(3666),
    [sym_char_literal] = ACTIONS(3666),
    [anon_sym_true] = ACTIONS(3668),
    [anon_sym_false] = ACTIONS(3668),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3668),
    [sym_super] = ACTIONS(3668),
    [sym_crate] = ACTIONS(3668),
    [aux_sym_metavariable_token1] = ACTIONS(3666),
    [sym__raw_string_literal_start] = ACTIONS(3666),
    [sym_float_literal] = ACTIONS(3666),
    [sym__outer_block_doc_comment_start] = ACTIONS(3666),
    [sym__outer_line_doc_comment_start] = ACTIONS(3666),
  },
  [STATE(975)] = {
    [sym_line_comment] = STATE(975),
    [sym_block_comment] = STATE(975),
    [ts_builtin_sym_end] = ACTIONS(3670),
    [sym_identifier] = ACTIONS(3672),
    [anon_sym_SEMI] = ACTIONS(3670),
    [anon_sym_macro] = ACTIONS(3672),
    [anon_sym_LPAREN] = ACTIONS(3670),
    [anon_sym_LBRACE] = ACTIONS(3670),
    [anon_sym_RBRACE] = ACTIONS(3670),
    [anon_sym_macro_rules] = ACTIONS(3672),
    [anon_sym_BANG] = ACTIONS(3670),
    [anon_sym_LBRACK] = ACTIONS(3670),
    [anon_sym_STAR] = ACTIONS(3670),
    [anon_sym_u8] = ACTIONS(3672),
    [anon_sym_i8] = ACTIONS(3672),
    [anon_sym_u16] = ACTIONS(3672),
    [anon_sym_i16] = ACTIONS(3672),
    [anon_sym_u32] = ACTIONS(3672),
    [anon_sym_i32] = ACTIONS(3672),
    [anon_sym_u64] = ACTIONS(3672),
    [anon_sym_i64] = ACTIONS(3672),
    [anon_sym_u128] = ACTIONS(3672),
    [anon_sym_i128] = ACTIONS(3672),
    [anon_sym_isize] = ACTIONS(3672),
    [anon_sym_usize] = ACTIONS(3672),
    [anon_sym_f32] = ACTIONS(3672),
    [anon_sym_f64] = ACTIONS(3672),
    [anon_sym_bool] = ACTIONS(3672),
    [anon_sym_str] = ACTIONS(3672),
    [anon_sym_char] = ACTIONS(3672),
    [anon_sym_DASH] = ACTIONS(3670),
    [anon_sym_AMP] = ACTIONS(3670),
    [anon_sym_PIPE] = ACTIONS(3670),
    [anon_sym_LT] = ACTIONS(3670),
    [anon_sym_DOT_DOT] = ACTIONS(3670),
    [anon_sym_COLON_COLON] = ACTIONS(3670),
    [anon_sym_POUND] = ACTIONS(3670),
    [anon_sym_SQUOTE] = ACTIONS(3672),
    [anon_sym_async] = ACTIONS(3672),
    [anon_sym_auto] = ACTIONS(3672),
    [anon_sym_await] = ACTIONS(3672),
    [anon_sym_become] = ACTIONS(3672),
    [anon_sym_break] = ACTIONS(3672),
    [anon_sym_const] = ACTIONS(3672),
    [anon_sym_continue] = ACTIONS(3672),
    [anon_sym_default] = ACTIONS(3672),
    [anon_sym_enum] = ACTIONS(3672),
    [anon_sym_fn] = ACTIONS(3672),
    [anon_sym_for] = ACTIONS(3672),
    [anon_sym_gen] = ACTIONS(3672),
    [anon_sym_if] = ACTIONS(3672),
    [anon_sym_impl] = ACTIONS(3672),
    [anon_sym_let] = ACTIONS(3672),
    [anon_sym_loop] = ACTIONS(3672),
    [anon_sym_match] = ACTIONS(3672),
    [anon_sym_mod] = ACTIONS(3672),
    [anon_sym_pub] = ACTIONS(3672),
    [anon_sym_return] = ACTIONS(3672),
    [anon_sym_raw] = ACTIONS(3672),
    [anon_sym_static] = ACTIONS(3672),
    [anon_sym_struct] = ACTIONS(3672),
    [anon_sym_trait] = ACTIONS(3672),
    [anon_sym_type] = ACTIONS(3672),
    [anon_sym_union] = ACTIONS(3672),
    [anon_sym_unsafe] = ACTIONS(3672),
    [anon_sym_use] = ACTIONS(3672),
    [anon_sym_while] = ACTIONS(3672),
    [anon_sym_extern] = ACTIONS(3672),
    [anon_sym_safe] = ACTIONS(3672),
    [anon_sym_yield] = ACTIONS(3672),
    [anon_sym_move] = ACTIONS(3672),
    [anon_sym_try] = ACTIONS(3672),
    [sym_integer_literal] = ACTIONS(3670),
    [aux_sym_string_literal_token1] = ACTIONS(3670),
    [sym_char_literal] = ACTIONS(3670),
    [anon_sym_true] = ACTIONS(3672),
    [anon_sym_false] = ACTIONS(3672),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3672),
    [sym_super] = ACTIONS(3672),
    [sym_crate] = ACTIONS(3672),
    [aux_sym_metavariable_token1] = ACTIONS(3670),
    [sym__raw_string_literal_start] = ACTIONS(3670),
    [sym_float_literal] = ACTIONS(3670),
    [sym__outer_block_doc_comment_start] = ACTIONS(3670),
    [sym__outer_line_doc_comment_start] = ACTIONS(3670),
  },
  [STATE(976)] = {
    [sym_line_comment] = STATE(976),
    [sym_block_comment] = STATE(976),
    [ts_builtin_sym_end] = ACTIONS(3674),
    [sym_identifier] = ACTIONS(3676),
    [anon_sym_SEMI] = ACTIONS(3674),
    [anon_sym_macro] = ACTIONS(3676),
    [anon_sym_LPAREN] = ACTIONS(3674),
    [anon_sym_LBRACE] = ACTIONS(3674),
    [anon_sym_RBRACE] = ACTIONS(3674),
    [anon_sym_macro_rules] = ACTIONS(3676),
    [anon_sym_BANG] = ACTIONS(3674),
    [anon_sym_LBRACK] = ACTIONS(3674),
    [anon_sym_STAR] = ACTIONS(3674),
    [anon_sym_u8] = ACTIONS(3676),
    [anon_sym_i8] = ACTIONS(3676),
    [anon_sym_u16] = ACTIONS(3676),
    [anon_sym_i16] = ACTIONS(3676),
    [anon_sym_u32] = ACTIONS(3676),
    [anon_sym_i32] = ACTIONS(3676),
    [anon_sym_u64] = ACTIONS(3676),
    [anon_sym_i64] = ACTIONS(3676),
    [anon_sym_u128] = ACTIONS(3676),
    [anon_sym_i128] = ACTIONS(3676),
    [anon_sym_isize] = ACTIONS(3676),
    [anon_sym_usize] = ACTIONS(3676),
    [anon_sym_f32] = ACTIONS(3676),
    [anon_sym_f64] = ACTIONS(3676),
    [anon_sym_bool] = ACTIONS(3676),
    [anon_sym_str] = ACTIONS(3676),
    [anon_sym_char] = ACTIONS(3676),
    [anon_sym_DASH] = ACTIONS(3674),
    [anon_sym_AMP] = ACTIONS(3674),
    [anon_sym_PIPE] = ACTIONS(3674),
    [anon_sym_LT] = ACTIONS(3674),
    [anon_sym_DOT_DOT] = ACTIONS(3674),
    [anon_sym_COLON_COLON] = ACTIONS(3674),
    [anon_sym_POUND] = ACTIONS(3674),
    [anon_sym_SQUOTE] = ACTIONS(3676),
    [anon_sym_async] = ACTIONS(3676),
    [anon_sym_auto] = ACTIONS(3676),
    [anon_sym_await] = ACTIONS(3676),
    [anon_sym_become] = ACTIONS(3676),
    [anon_sym_break] = ACTIONS(3676),
    [anon_sym_const] = ACTIONS(3676),
    [anon_sym_continue] = ACTIONS(3676),
    [anon_sym_default] = ACTIONS(3676),
    [anon_sym_enum] = ACTIONS(3676),
    [anon_sym_fn] = ACTIONS(3676),
    [anon_sym_for] = ACTIONS(3676),
    [anon_sym_gen] = ACTIONS(3676),
    [anon_sym_if] = ACTIONS(3676),
    [anon_sym_impl] = ACTIONS(3676),
    [anon_sym_let] = ACTIONS(3676),
    [anon_sym_loop] = ACTIONS(3676),
    [anon_sym_match] = ACTIONS(3676),
    [anon_sym_mod] = ACTIONS(3676),
    [anon_sym_pub] = ACTIONS(3676),
    [anon_sym_return] = ACTIONS(3676),
    [anon_sym_raw] = ACTIONS(3676),
    [anon_sym_static] = ACTIONS(3676),
    [anon_sym_struct] = ACTIONS(3676),
    [anon_sym_trait] = ACTIONS(3676),
    [anon_sym_type] = ACTIONS(3676),
    [anon_sym_union] = ACTIONS(3676),
    [anon_sym_unsafe] = ACTIONS(3676),
    [anon_sym_use] = ACTIONS(3676),
    [anon_sym_while] = ACTIONS(3676),
    [anon_sym_extern] = ACTIONS(3676),
    [anon_sym_safe] = ACTIONS(3676),
    [anon_sym_yield] = ACTIONS(3676),
    [anon_sym_move] = ACTIONS(3676),
    [anon_sym_try] = ACTIONS(3676),
    [sym_integer_literal] = ACTIONS(3674),
    [aux_sym_string_literal_token1] = ACTIONS(3674),
    [sym_char_literal] = ACTIONS(3674),
    [anon_sym_true] = ACTIONS(3676),
    [anon_sym_false] = ACTIONS(3676),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3676),
    [sym_super] = ACTIONS(3676),
    [sym_crate] = ACTIONS(3676),
    [aux_sym_metavariable_token1] = ACTIONS(3674),
    [sym__raw_string_literal_start] = ACTIONS(3674),
    [sym_float_literal] = ACTIONS(3674),
    [sym__outer_block_doc_comment_start] = ACTIONS(3674),
    [sym__outer_line_doc_comment_start] = ACTIONS(3674),
  },
  [STATE(977)] = {
    [sym_line_comment] = STATE(977),
    [sym_block_comment] = STATE(977),
    [ts_builtin_sym_end] = ACTIONS(3678),
    [sym_identifier] = ACTIONS(3680),
    [anon_sym_SEMI] = ACTIONS(3678),
    [anon_sym_macro] = ACTIONS(3680),
    [anon_sym_LPAREN] = ACTIONS(3678),
    [anon_sym_LBRACE] = ACTIONS(3678),
    [anon_sym_RBRACE] = ACTIONS(3678),
    [anon_sym_macro_rules] = ACTIONS(3680),
    [anon_sym_BANG] = ACTIONS(3678),
    [anon_sym_LBRACK] = ACTIONS(3678),
    [anon_sym_STAR] = ACTIONS(3678),
    [anon_sym_u8] = ACTIONS(3680),
    [anon_sym_i8] = ACTIONS(3680),
    [anon_sym_u16] = ACTIONS(3680),
    [anon_sym_i16] = ACTIONS(3680),
    [anon_sym_u32] = ACTIONS(3680),
    [anon_sym_i32] = ACTIONS(3680),
    [anon_sym_u64] = ACTIONS(3680),
    [anon_sym_i64] = ACTIONS(3680),
    [anon_sym_u128] = ACTIONS(3680),
    [anon_sym_i128] = ACTIONS(3680),
    [anon_sym_isize] = ACTIONS(3680),
    [anon_sym_usize] = ACTIONS(3680),
    [anon_sym_f32] = ACTIONS(3680),
    [anon_sym_f64] = ACTIONS(3680),
    [anon_sym_bool] = ACTIONS(3680),
    [anon_sym_str] = ACTIONS(3680),
    [anon_sym_char] = ACTIONS(3680),
    [anon_sym_DASH] = ACTIONS(3678),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_PIPE] = ACTIONS(3678),
    [anon_sym_LT] = ACTIONS(3678),
    [anon_sym_DOT_DOT] = ACTIONS(3678),
    [anon_sym_COLON_COLON] = ACTIONS(3678),
    [anon_sym_POUND] = ACTIONS(3678),
    [anon_sym_SQUOTE] = ACTIONS(3680),
    [anon_sym_async] = ACTIONS(3680),
    [anon_sym_auto] = ACTIONS(3680),
    [anon_sym_await] = ACTIONS(3680),
    [anon_sym_become] = ACTIONS(3680),
    [anon_sym_break] = ACTIONS(3680),
    [anon_sym_const] = ACTIONS(3680),
    [anon_sym_continue] = ACTIONS(3680),
    [anon_sym_default] = ACTIONS(3680),
    [anon_sym_enum] = ACTIONS(3680),
    [anon_sym_fn] = ACTIONS(3680),
    [anon_sym_for] = ACTIONS(3680),
    [anon_sym_gen] = ACTIONS(3680),
    [anon_sym_if] = ACTIONS(3680),
    [anon_sym_impl] = ACTIONS(3680),
    [anon_sym_let] = ACTIONS(3680),
    [anon_sym_loop] = ACTIONS(3680),
    [anon_sym_match] = ACTIONS(3680),
    [anon_sym_mod] = ACTIONS(3680),
    [anon_sym_pub] = ACTIONS(3680),
    [anon_sym_return] = ACTIONS(3680),
    [anon_sym_raw] = ACTIONS(3680),
    [anon_sym_static] = ACTIONS(3680),
    [anon_sym_struct] = ACTIONS(3680),
    [anon_sym_trait] = ACTIONS(3680),
    [anon_sym_type] = ACTIONS(3680),
    [anon_sym_union] = ACTIONS(3680),
    [anon_sym_unsafe] = ACTIONS(3680),
    [anon_sym_use] = ACTIONS(3680),
    [anon_sym_while] = ACTIONS(3680),
    [anon_sym_extern] = ACTIONS(3680),
    [anon_sym_safe] = ACTIONS(3680),
    [anon_sym_yield] = ACTIONS(3680),
    [anon_sym_move] = ACTIONS(3680),
    [anon_sym_try] = ACTIONS(3680),
    [sym_integer_literal] = ACTIONS(3678),
    [aux_sym_string_literal_token1] = ACTIONS(3678),
    [sym_char_literal] = ACTIONS(3678),
    [anon_sym_true] = ACTIONS(3680),
    [anon_sym_false] = ACTIONS(3680),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3680),
    [sym_super] = ACTIONS(3680),
    [sym_crate] = ACTIONS(3680),
    [aux_sym_metavariable_token1] = ACTIONS(3678),
    [sym__raw_string_literal_start] = ACTIONS(3678),
    [sym_float_literal] = ACTIONS(3678),
    [sym__outer_block_doc_comment_start] = ACTIONS(3678),
    [sym__outer_line_doc_comment_start] = ACTIONS(3678),
  },
  [STATE(978)] = {
    [sym_line_comment] = STATE(978),
    [sym_block_comment] = STATE(978),
    [ts_builtin_sym_end] = ACTIONS(3682),
    [sym_identifier] = ACTIONS(3684),
    [anon_sym_SEMI] = ACTIONS(3682),
    [anon_sym_macro] = ACTIONS(3684),
    [anon_sym_LPAREN] = ACTIONS(3682),
    [anon_sym_LBRACE] = ACTIONS(3682),
    [anon_sym_RBRACE] = ACTIONS(3682),
    [anon_sym_macro_rules] = ACTIONS(3684),
    [anon_sym_BANG] = ACTIONS(3682),
    [anon_sym_LBRACK] = ACTIONS(3682),
    [anon_sym_STAR] = ACTIONS(3682),
    [anon_sym_u8] = ACTIONS(3684),
    [anon_sym_i8] = ACTIONS(3684),
    [anon_sym_u16] = ACTIONS(3684),
    [anon_sym_i16] = ACTIONS(3684),
    [anon_sym_u32] = ACTIONS(3684),
    [anon_sym_i32] = ACTIONS(3684),
    [anon_sym_u64] = ACTIONS(3684),
    [anon_sym_i64] = ACTIONS(3684),
    [anon_sym_u128] = ACTIONS(3684),
    [anon_sym_i128] = ACTIONS(3684),
    [anon_sym_isize] = ACTIONS(3684),
    [anon_sym_usize] = ACTIONS(3684),
    [anon_sym_f32] = ACTIONS(3684),
    [anon_sym_f64] = ACTIONS(3684),
    [anon_sym_bool] = ACTIONS(3684),
    [anon_sym_str] = ACTIONS(3684),
    [anon_sym_char] = ACTIONS(3684),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_AMP] = ACTIONS(3682),
    [anon_sym_PIPE] = ACTIONS(3682),
    [anon_sym_LT] = ACTIONS(3682),
    [anon_sym_DOT_DOT] = ACTIONS(3682),
    [anon_sym_COLON_COLON] = ACTIONS(3682),
    [anon_sym_POUND] = ACTIONS(3682),
    [anon_sym_SQUOTE] = ACTIONS(3684),
    [anon_sym_async] = ACTIONS(3684),
    [anon_sym_auto] = ACTIONS(3684),
    [anon_sym_await] = ACTIONS(3684),
    [anon_sym_become] = ACTIONS(3684),
    [anon_sym_break] = ACTIONS(3684),
    [anon_sym_const] = ACTIONS(3684),
    [anon_sym_continue] = ACTIONS(3684),
    [anon_sym_default] = ACTIONS(3684),
    [anon_sym_enum] = ACTIONS(3684),
    [anon_sym_fn] = ACTIONS(3684),
    [anon_sym_for] = ACTIONS(3684),
    [anon_sym_gen] = ACTIONS(3684),
    [anon_sym_if] = ACTIONS(3684),
    [anon_sym_impl] = ACTIONS(3684),
    [anon_sym_let] = ACTIONS(3684),
    [anon_sym_loop] = ACTIONS(3684),
    [anon_sym_match] = ACTIONS(3684),
    [anon_sym_mod] = ACTIONS(3684),
    [anon_sym_pub] = ACTIONS(3684),
    [anon_sym_return] = ACTIONS(3684),
    [anon_sym_raw] = ACTIONS(3684),
    [anon_sym_static] = ACTIONS(3684),
    [anon_sym_struct] = ACTIONS(3684),
    [anon_sym_trait] = ACTIONS(3684),
    [anon_sym_type] = ACTIONS(3684),
    [anon_sym_union] = ACTIONS(3684),
    [anon_sym_unsafe] = ACTIONS(3684),
    [anon_sym_use] = ACTIONS(3684),
    [anon_sym_while] = ACTIONS(3684),
    [anon_sym_extern] = ACTIONS(3684),
    [anon_sym_safe] = ACTIONS(3684),
    [anon_sym_yield] = ACTIONS(3684),
    [anon_sym_move] = ACTIONS(3684),
    [anon_sym_try] = ACTIONS(3684),
    [sym_integer_literal] = ACTIONS(3682),
    [aux_sym_string_literal_token1] = ACTIONS(3682),
    [sym_char_literal] = ACTIONS(3682),
    [anon_sym_true] = ACTIONS(3684),
    [anon_sym_false] = ACTIONS(3684),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3684),
    [sym_super] = ACTIONS(3684),
    [sym_crate] = ACTIONS(3684),
    [aux_sym_metavariable_token1] = ACTIONS(3682),
    [sym__raw_string_literal_start] = ACTIONS(3682),
    [sym_float_literal] = ACTIONS(3682),
    [sym__outer_block_doc_comment_start] = ACTIONS(3682),
    [sym__outer_line_doc_comment_start] = ACTIONS(3682),
  },
  [STATE(979)] = {
    [sym_line_comment] = STATE(979),
    [sym_block_comment] = STATE(979),
    [ts_builtin_sym_end] = ACTIONS(3686),
    [sym_identifier] = ACTIONS(3688),
    [anon_sym_SEMI] = ACTIONS(3686),
    [anon_sym_macro] = ACTIONS(3688),
    [anon_sym_LPAREN] = ACTIONS(3686),
    [anon_sym_LBRACE] = ACTIONS(3686),
    [anon_sym_RBRACE] = ACTIONS(3686),
    [anon_sym_macro_rules] = ACTIONS(3688),
    [anon_sym_BANG] = ACTIONS(3686),
    [anon_sym_LBRACK] = ACTIONS(3686),
    [anon_sym_STAR] = ACTIONS(3686),
    [anon_sym_u8] = ACTIONS(3688),
    [anon_sym_i8] = ACTIONS(3688),
    [anon_sym_u16] = ACTIONS(3688),
    [anon_sym_i16] = ACTIONS(3688),
    [anon_sym_u32] = ACTIONS(3688),
    [anon_sym_i32] = ACTIONS(3688),
    [anon_sym_u64] = ACTIONS(3688),
    [anon_sym_i64] = ACTIONS(3688),
    [anon_sym_u128] = ACTIONS(3688),
    [anon_sym_i128] = ACTIONS(3688),
    [anon_sym_isize] = ACTIONS(3688),
    [anon_sym_usize] = ACTIONS(3688),
    [anon_sym_f32] = ACTIONS(3688),
    [anon_sym_f64] = ACTIONS(3688),
    [anon_sym_bool] = ACTIONS(3688),
    [anon_sym_str] = ACTIONS(3688),
    [anon_sym_char] = ACTIONS(3688),
    [anon_sym_DASH] = ACTIONS(3686),
    [anon_sym_AMP] = ACTIONS(3686),
    [anon_sym_PIPE] = ACTIONS(3686),
    [anon_sym_LT] = ACTIONS(3686),
    [anon_sym_DOT_DOT] = ACTIONS(3686),
    [anon_sym_COLON_COLON] = ACTIONS(3686),
    [anon_sym_POUND] = ACTIONS(3686),
    [anon_sym_SQUOTE] = ACTIONS(3688),
    [anon_sym_async] = ACTIONS(3688),
    [anon_sym_auto] = ACTIONS(3688),
    [anon_sym_await] = ACTIONS(3688),
    [anon_sym_become] = ACTIONS(3688),
    [anon_sym_break] = ACTIONS(3688),
    [anon_sym_const] = ACTIONS(3688),
    [anon_sym_continue] = ACTIONS(3688),
    [anon_sym_default] = ACTIONS(3688),
    [anon_sym_enum] = ACTIONS(3688),
    [anon_sym_fn] = ACTIONS(3688),
    [anon_sym_for] = ACTIONS(3688),
    [anon_sym_gen] = ACTIONS(3688),
    [anon_sym_if] = ACTIONS(3688),
    [anon_sym_impl] = ACTIONS(3688),
    [anon_sym_let] = ACTIONS(3688),
    [anon_sym_loop] = ACTIONS(3688),
    [anon_sym_match] = ACTIONS(3688),
    [anon_sym_mod] = ACTIONS(3688),
    [anon_sym_pub] = ACTIONS(3688),
    [anon_sym_return] = ACTIONS(3688),
    [anon_sym_raw] = ACTIONS(3688),
    [anon_sym_static] = ACTIONS(3688),
    [anon_sym_struct] = ACTIONS(3688),
    [anon_sym_trait] = ACTIONS(3688),
    [anon_sym_type] = ACTIONS(3688),
    [anon_sym_union] = ACTIONS(3688),
    [anon_sym_unsafe] = ACTIONS(3688),
    [anon_sym_use] = ACTIONS(3688),
    [anon_sym_while] = ACTIONS(3688),
    [anon_sym_extern] = ACTIONS(3688),
    [anon_sym_safe] = ACTIONS(3688),
    [anon_sym_yield] = ACTIONS(3688),
    [anon_sym_move] = ACTIONS(3688),
    [anon_sym_try] = ACTIONS(3688),
    [sym_integer_literal] = ACTIONS(3686),
    [aux_sym_string_literal_token1] = ACTIONS(3686),
    [sym_char_literal] = ACTIONS(3686),
    [anon_sym_true] = ACTIONS(3688),
    [anon_sym_false] = ACTIONS(3688),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3688),
    [sym_super] = ACTIONS(3688),
    [sym_crate] = ACTIONS(3688),
    [aux_sym_metavariable_token1] = ACTIONS(3686),
    [sym__raw_string_literal_start] = ACTIONS(3686),
    [sym_float_literal] = ACTIONS(3686),
    [sym__outer_block_doc_comment_start] = ACTIONS(3686),
    [sym__outer_line_doc_comment_start] = ACTIONS(3686),
  },
  [STATE(980)] = {
    [sym_line_comment] = STATE(980),
    [sym_block_comment] = STATE(980),
    [ts_builtin_sym_end] = ACTIONS(3690),
    [sym_identifier] = ACTIONS(3692),
    [anon_sym_SEMI] = ACTIONS(3690),
    [anon_sym_macro] = ACTIONS(3692),
    [anon_sym_LPAREN] = ACTIONS(3690),
    [anon_sym_LBRACE] = ACTIONS(3690),
    [anon_sym_RBRACE] = ACTIONS(3690),
    [anon_sym_macro_rules] = ACTIONS(3692),
    [anon_sym_BANG] = ACTIONS(3690),
    [anon_sym_LBRACK] = ACTIONS(3690),
    [anon_sym_STAR] = ACTIONS(3690),
    [anon_sym_u8] = ACTIONS(3692),
    [anon_sym_i8] = ACTIONS(3692),
    [anon_sym_u16] = ACTIONS(3692),
    [anon_sym_i16] = ACTIONS(3692),
    [anon_sym_u32] = ACTIONS(3692),
    [anon_sym_i32] = ACTIONS(3692),
    [anon_sym_u64] = ACTIONS(3692),
    [anon_sym_i64] = ACTIONS(3692),
    [anon_sym_u128] = ACTIONS(3692),
    [anon_sym_i128] = ACTIONS(3692),
    [anon_sym_isize] = ACTIONS(3692),
    [anon_sym_usize] = ACTIONS(3692),
    [anon_sym_f32] = ACTIONS(3692),
    [anon_sym_f64] = ACTIONS(3692),
    [anon_sym_bool] = ACTIONS(3692),
    [anon_sym_str] = ACTIONS(3692),
    [anon_sym_char] = ACTIONS(3692),
    [anon_sym_DASH] = ACTIONS(3690),
    [anon_sym_AMP] = ACTIONS(3690),
    [anon_sym_PIPE] = ACTIONS(3690),
    [anon_sym_LT] = ACTIONS(3690),
    [anon_sym_DOT_DOT] = ACTIONS(3690),
    [anon_sym_COLON_COLON] = ACTIONS(3690),
    [anon_sym_POUND] = ACTIONS(3690),
    [anon_sym_SQUOTE] = ACTIONS(3692),
    [anon_sym_async] = ACTIONS(3692),
    [anon_sym_auto] = ACTIONS(3692),
    [anon_sym_await] = ACTIONS(3692),
    [anon_sym_become] = ACTIONS(3692),
    [anon_sym_break] = ACTIONS(3692),
    [anon_sym_const] = ACTIONS(3692),
    [anon_sym_continue] = ACTIONS(3692),
    [anon_sym_default] = ACTIONS(3692),
    [anon_sym_enum] = ACTIONS(3692),
    [anon_sym_fn] = ACTIONS(3692),
    [anon_sym_for] = ACTIONS(3692),
    [anon_sym_gen] = ACTIONS(3692),
    [anon_sym_if] = ACTIONS(3692),
    [anon_sym_impl] = ACTIONS(3692),
    [anon_sym_let] = ACTIONS(3692),
    [anon_sym_loop] = ACTIONS(3692),
    [anon_sym_match] = ACTIONS(3692),
    [anon_sym_mod] = ACTIONS(3692),
    [anon_sym_pub] = ACTIONS(3692),
    [anon_sym_return] = ACTIONS(3692),
    [anon_sym_raw] = ACTIONS(3692),
    [anon_sym_static] = ACTIONS(3692),
    [anon_sym_struct] = ACTIONS(3692),
    [anon_sym_trait] = ACTIONS(3692),
    [anon_sym_type] = ACTIONS(3692),
    [anon_sym_union] = ACTIONS(3692),
    [anon_sym_unsafe] = ACTIONS(3692),
    [anon_sym_use] = ACTIONS(3692),
    [anon_sym_while] = ACTIONS(3692),
    [anon_sym_extern] = ACTIONS(3692),
    [anon_sym_safe] = ACTIONS(3692),
    [anon_sym_yield] = ACTIONS(3692),
    [anon_sym_move] = ACTIONS(3692),
    [anon_sym_try] = ACTIONS(3692),
    [sym_integer_literal] = ACTIONS(3690),
    [aux_sym_string_literal_token1] = ACTIONS(3690),
    [sym_char_literal] = ACTIONS(3690),
    [anon_sym_true] = ACTIONS(3692),
    [anon_sym_false] = ACTIONS(3692),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3692),
    [sym_super] = ACTIONS(3692),
    [sym_crate] = ACTIONS(3692),
    [aux_sym_metavariable_token1] = ACTIONS(3690),
    [sym__raw_string_literal_start] = ACTIONS(3690),
    [sym_float_literal] = ACTIONS(3690),
    [sym__outer_block_doc_comment_start] = ACTIONS(3690),
    [sym__outer_line_doc_comment_start] = ACTIONS(3690),
  },
  [STATE(981)] = {
    [sym_line_comment] = STATE(981),
    [sym_block_comment] = STATE(981),
    [ts_builtin_sym_end] = ACTIONS(3694),
    [sym_identifier] = ACTIONS(3696),
    [anon_sym_SEMI] = ACTIONS(3694),
    [anon_sym_macro] = ACTIONS(3696),
    [anon_sym_LPAREN] = ACTIONS(3694),
    [anon_sym_LBRACE] = ACTIONS(3694),
    [anon_sym_RBRACE] = ACTIONS(3694),
    [anon_sym_macro_rules] = ACTIONS(3696),
    [anon_sym_BANG] = ACTIONS(3694),
    [anon_sym_LBRACK] = ACTIONS(3694),
    [anon_sym_STAR] = ACTIONS(3694),
    [anon_sym_u8] = ACTIONS(3696),
    [anon_sym_i8] = ACTIONS(3696),
    [anon_sym_u16] = ACTIONS(3696),
    [anon_sym_i16] = ACTIONS(3696),
    [anon_sym_u32] = ACTIONS(3696),
    [anon_sym_i32] = ACTIONS(3696),
    [anon_sym_u64] = ACTIONS(3696),
    [anon_sym_i64] = ACTIONS(3696),
    [anon_sym_u128] = ACTIONS(3696),
    [anon_sym_i128] = ACTIONS(3696),
    [anon_sym_isize] = ACTIONS(3696),
    [anon_sym_usize] = ACTIONS(3696),
    [anon_sym_f32] = ACTIONS(3696),
    [anon_sym_f64] = ACTIONS(3696),
    [anon_sym_bool] = ACTIONS(3696),
    [anon_sym_str] = ACTIONS(3696),
    [anon_sym_char] = ACTIONS(3696),
    [anon_sym_DASH] = ACTIONS(3694),
    [anon_sym_AMP] = ACTIONS(3694),
    [anon_sym_PIPE] = ACTIONS(3694),
    [anon_sym_LT] = ACTIONS(3694),
    [anon_sym_DOT_DOT] = ACTIONS(3694),
    [anon_sym_COLON_COLON] = ACTIONS(3694),
    [anon_sym_POUND] = ACTIONS(3694),
    [anon_sym_SQUOTE] = ACTIONS(3696),
    [anon_sym_async] = ACTIONS(3696),
    [anon_sym_auto] = ACTIONS(3696),
    [anon_sym_await] = ACTIONS(3696),
    [anon_sym_become] = ACTIONS(3696),
    [anon_sym_break] = ACTIONS(3696),
    [anon_sym_const] = ACTIONS(3696),
    [anon_sym_continue] = ACTIONS(3696),
    [anon_sym_default] = ACTIONS(3696),
    [anon_sym_enum] = ACTIONS(3696),
    [anon_sym_fn] = ACTIONS(3696),
    [anon_sym_for] = ACTIONS(3696),
    [anon_sym_gen] = ACTIONS(3696),
    [anon_sym_if] = ACTIONS(3696),
    [anon_sym_impl] = ACTIONS(3696),
    [anon_sym_let] = ACTIONS(3696),
    [anon_sym_loop] = ACTIONS(3696),
    [anon_sym_match] = ACTIONS(3696),
    [anon_sym_mod] = ACTIONS(3696),
    [anon_sym_pub] = ACTIONS(3696),
    [anon_sym_return] = ACTIONS(3696),
    [anon_sym_raw] = ACTIONS(3696),
    [anon_sym_static] = ACTIONS(3696),
    [anon_sym_struct] = ACTIONS(3696),
    [anon_sym_trait] = ACTIONS(3696),
    [anon_sym_type] = ACTIONS(3696),
    [anon_sym_union] = ACTIONS(3696),
    [anon_sym_unsafe] = ACTIONS(3696),
    [anon_sym_use] = ACTIONS(3696),
    [anon_sym_while] = ACTIONS(3696),
    [anon_sym_extern] = ACTIONS(3696),
    [anon_sym_safe] = ACTIONS(3696),
    [anon_sym_yield] = ACTIONS(3696),
    [anon_sym_move] = ACTIONS(3696),
    [anon_sym_try] = ACTIONS(3696),
    [sym_integer_literal] = ACTIONS(3694),
    [aux_sym_string_literal_token1] = ACTIONS(3694),
    [sym_char_literal] = ACTIONS(3694),
    [anon_sym_true] = ACTIONS(3696),
    [anon_sym_false] = ACTIONS(3696),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3696),
    [sym_super] = ACTIONS(3696),
    [sym_crate] = ACTIONS(3696),
    [aux_sym_metavariable_token1] = ACTIONS(3694),
    [sym__raw_string_literal_start] = ACTIONS(3694),
    [sym_float_literal] = ACTIONS(3694),
    [sym__outer_block_doc_comment_start] = ACTIONS(3694),
    [sym__outer_line_doc_comment_start] = ACTIONS(3694),
  },
  [STATE(982)] = {
    [sym_line_comment] = STATE(982),
    [sym_block_comment] = STATE(982),
    [ts_builtin_sym_end] = ACTIONS(3698),
    [sym_identifier] = ACTIONS(3700),
    [anon_sym_SEMI] = ACTIONS(3698),
    [anon_sym_macro] = ACTIONS(3700),
    [anon_sym_LPAREN] = ACTIONS(3698),
    [anon_sym_LBRACE] = ACTIONS(3698),
    [anon_sym_RBRACE] = ACTIONS(3698),
    [anon_sym_macro_rules] = ACTIONS(3700),
    [anon_sym_BANG] = ACTIONS(3698),
    [anon_sym_LBRACK] = ACTIONS(3698),
    [anon_sym_STAR] = ACTIONS(3698),
    [anon_sym_u8] = ACTIONS(3700),
    [anon_sym_i8] = ACTIONS(3700),
    [anon_sym_u16] = ACTIONS(3700),
    [anon_sym_i16] = ACTIONS(3700),
    [anon_sym_u32] = ACTIONS(3700),
    [anon_sym_i32] = ACTIONS(3700),
    [anon_sym_u64] = ACTIONS(3700),
    [anon_sym_i64] = ACTIONS(3700),
    [anon_sym_u128] = ACTIONS(3700),
    [anon_sym_i128] = ACTIONS(3700),
    [anon_sym_isize] = ACTIONS(3700),
    [anon_sym_usize] = ACTIONS(3700),
    [anon_sym_f32] = ACTIONS(3700),
    [anon_sym_f64] = ACTIONS(3700),
    [anon_sym_bool] = ACTIONS(3700),
    [anon_sym_str] = ACTIONS(3700),
    [anon_sym_char] = ACTIONS(3700),
    [anon_sym_DASH] = ACTIONS(3698),
    [anon_sym_AMP] = ACTIONS(3698),
    [anon_sym_PIPE] = ACTIONS(3698),
    [anon_sym_LT] = ACTIONS(3698),
    [anon_sym_DOT_DOT] = ACTIONS(3698),
    [anon_sym_COLON_COLON] = ACTIONS(3698),
    [anon_sym_POUND] = ACTIONS(3698),
    [anon_sym_SQUOTE] = ACTIONS(3700),
    [anon_sym_async] = ACTIONS(3700),
    [anon_sym_auto] = ACTIONS(3700),
    [anon_sym_await] = ACTIONS(3700),
    [anon_sym_become] = ACTIONS(3700),
    [anon_sym_break] = ACTIONS(3700),
    [anon_sym_const] = ACTIONS(3700),
    [anon_sym_continue] = ACTIONS(3700),
    [anon_sym_default] = ACTIONS(3700),
    [anon_sym_enum] = ACTIONS(3700),
    [anon_sym_fn] = ACTIONS(3700),
    [anon_sym_for] = ACTIONS(3700),
    [anon_sym_gen] = ACTIONS(3700),
    [anon_sym_if] = ACTIONS(3700),
    [anon_sym_impl] = ACTIONS(3700),
    [anon_sym_let] = ACTIONS(3700),
    [anon_sym_loop] = ACTIONS(3700),
    [anon_sym_match] = ACTIONS(3700),
    [anon_sym_mod] = ACTIONS(3700),
    [anon_sym_pub] = ACTIONS(3700),
    [anon_sym_return] = ACTIONS(3700),
    [anon_sym_raw] = ACTIONS(3700),
    [anon_sym_static] = ACTIONS(3700),
    [anon_sym_struct] = ACTIONS(3700),
    [anon_sym_trait] = ACTIONS(3700),
    [anon_sym_type] = ACTIONS(3700),
    [anon_sym_union] = ACTIONS(3700),
    [anon_sym_unsafe] = ACTIONS(3700),
    [anon_sym_use] = ACTIONS(3700),
    [anon_sym_while] = ACTIONS(3700),
    [anon_sym_extern] = ACTIONS(3700),
    [anon_sym_safe] = ACTIONS(3700),
    [anon_sym_yield] = ACTIONS(3700),
    [anon_sym_move] = ACTIONS(3700),
    [anon_sym_try] = ACTIONS(3700),
    [sym_integer_literal] = ACTIONS(3698),
    [aux_sym_string_literal_token1] = ACTIONS(3698),
    [sym_char_literal] = ACTIONS(3698),
    [anon_sym_true] = ACTIONS(3700),
    [anon_sym_false] = ACTIONS(3700),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3700),
    [sym_super] = ACTIONS(3700),
    [sym_crate] = ACTIONS(3700),
    [aux_sym_metavariable_token1] = ACTIONS(3698),
    [sym__raw_string_literal_start] = ACTIONS(3698),
    [sym_float_literal] = ACTIONS(3698),
    [sym__outer_block_doc_comment_start] = ACTIONS(3698),
    [sym__outer_line_doc_comment_start] = ACTIONS(3698),
  },
  [STATE(983)] = {
    [sym_line_comment] = STATE(983),
    [sym_block_comment] = STATE(983),
    [ts_builtin_sym_end] = ACTIONS(3702),
    [sym_identifier] = ACTIONS(3704),
    [anon_sym_SEMI] = ACTIONS(3702),
    [anon_sym_macro] = ACTIONS(3704),
    [anon_sym_LPAREN] = ACTIONS(3702),
    [anon_sym_LBRACE] = ACTIONS(3702),
    [anon_sym_RBRACE] = ACTIONS(3702),
    [anon_sym_macro_rules] = ACTIONS(3704),
    [anon_sym_BANG] = ACTIONS(3702),
    [anon_sym_LBRACK] = ACTIONS(3702),
    [anon_sym_STAR] = ACTIONS(3702),
    [anon_sym_u8] = ACTIONS(3704),
    [anon_sym_i8] = ACTIONS(3704),
    [anon_sym_u16] = ACTIONS(3704),
    [anon_sym_i16] = ACTIONS(3704),
    [anon_sym_u32] = ACTIONS(3704),
    [anon_sym_i32] = ACTIONS(3704),
    [anon_sym_u64] = ACTIONS(3704),
    [anon_sym_i64] = ACTIONS(3704),
    [anon_sym_u128] = ACTIONS(3704),
    [anon_sym_i128] = ACTIONS(3704),
    [anon_sym_isize] = ACTIONS(3704),
    [anon_sym_usize] = ACTIONS(3704),
    [anon_sym_f32] = ACTIONS(3704),
    [anon_sym_f64] = ACTIONS(3704),
    [anon_sym_bool] = ACTIONS(3704),
    [anon_sym_str] = ACTIONS(3704),
    [anon_sym_char] = ACTIONS(3704),
    [anon_sym_DASH] = ACTIONS(3702),
    [anon_sym_AMP] = ACTIONS(3702),
    [anon_sym_PIPE] = ACTIONS(3702),
    [anon_sym_LT] = ACTIONS(3702),
    [anon_sym_DOT_DOT] = ACTIONS(3702),
    [anon_sym_COLON_COLON] = ACTIONS(3702),
    [anon_sym_POUND] = ACTIONS(3702),
    [anon_sym_SQUOTE] = ACTIONS(3704),
    [anon_sym_async] = ACTIONS(3704),
    [anon_sym_auto] = ACTIONS(3704),
    [anon_sym_await] = ACTIONS(3704),
    [anon_sym_become] = ACTIONS(3704),
    [anon_sym_break] = ACTIONS(3704),
    [anon_sym_const] = ACTIONS(3704),
    [anon_sym_continue] = ACTIONS(3704),
    [anon_sym_default] = ACTIONS(3704),
    [anon_sym_enum] = ACTIONS(3704),
    [anon_sym_fn] = ACTIONS(3704),
    [anon_sym_for] = ACTIONS(3704),
    [anon_sym_gen] = ACTIONS(3704),
    [anon_sym_if] = ACTIONS(3704),
    [anon_sym_impl] = ACTIONS(3704),
    [anon_sym_let] = ACTIONS(3704),
    [anon_sym_loop] = ACTIONS(3704),
    [anon_sym_match] = ACTIONS(3704),
    [anon_sym_mod] = ACTIONS(3704),
    [anon_sym_pub] = ACTIONS(3704),
    [anon_sym_return] = ACTIONS(3704),
    [anon_sym_raw] = ACTIONS(3704),
    [anon_sym_static] = ACTIONS(3704),
    [anon_sym_struct] = ACTIONS(3704),
    [anon_sym_trait] = ACTIONS(3704),
    [anon_sym_type] = ACTIONS(3704),
    [anon_sym_union] = ACTIONS(3704),
    [anon_sym_unsafe] = ACTIONS(3704),
    [anon_sym_use] = ACTIONS(3704),
    [anon_sym_while] = ACTIONS(3704),
    [anon_sym_extern] = ACTIONS(3704),
    [anon_sym_safe] = ACTIONS(3704),
    [anon_sym_yield] = ACTIONS(3704),
    [anon_sym_move] = ACTIONS(3704),
    [anon_sym_try] = ACTIONS(3704),
    [sym_integer_literal] = ACTIONS(3702),
    [aux_sym_string_literal_token1] = ACTIONS(3702),
    [sym_char_literal] = ACTIONS(3702),
    [anon_sym_true] = ACTIONS(3704),
    [anon_sym_false] = ACTIONS(3704),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3704),
    [sym_super] = ACTIONS(3704),
    [sym_crate] = ACTIONS(3704),
    [aux_sym_metavariable_token1] = ACTIONS(3702),
    [sym__raw_string_literal_start] = ACTIONS(3702),
    [sym_float_literal] = ACTIONS(3702),
    [sym__outer_block_doc_comment_start] = ACTIONS(3702),
    [sym__outer_line_doc_comment_start] = ACTIONS(3702),
  },
  [STATE(984)] = {
    [sym_line_comment] = STATE(984),
    [sym_block_comment] = STATE(984),
    [ts_builtin_sym_end] = ACTIONS(3706),
    [sym_identifier] = ACTIONS(3708),
    [anon_sym_SEMI] = ACTIONS(3706),
    [anon_sym_macro] = ACTIONS(3708),
    [anon_sym_LPAREN] = ACTIONS(3706),
    [anon_sym_LBRACE] = ACTIONS(3706),
    [anon_sym_RBRACE] = ACTIONS(3706),
    [anon_sym_macro_rules] = ACTIONS(3708),
    [anon_sym_BANG] = ACTIONS(3706),
    [anon_sym_LBRACK] = ACTIONS(3706),
    [anon_sym_STAR] = ACTIONS(3706),
    [anon_sym_u8] = ACTIONS(3708),
    [anon_sym_i8] = ACTIONS(3708),
    [anon_sym_u16] = ACTIONS(3708),
    [anon_sym_i16] = ACTIONS(3708),
    [anon_sym_u32] = ACTIONS(3708),
    [anon_sym_i32] = ACTIONS(3708),
    [anon_sym_u64] = ACTIONS(3708),
    [anon_sym_i64] = ACTIONS(3708),
    [anon_sym_u128] = ACTIONS(3708),
    [anon_sym_i128] = ACTIONS(3708),
    [anon_sym_isize] = ACTIONS(3708),
    [anon_sym_usize] = ACTIONS(3708),
    [anon_sym_f32] = ACTIONS(3708),
    [anon_sym_f64] = ACTIONS(3708),
    [anon_sym_bool] = ACTIONS(3708),
    [anon_sym_str] = ACTIONS(3708),
    [anon_sym_char] = ACTIONS(3708),
    [anon_sym_DASH] = ACTIONS(3706),
    [anon_sym_AMP] = ACTIONS(3706),
    [anon_sym_PIPE] = ACTIONS(3706),
    [anon_sym_LT] = ACTIONS(3706),
    [anon_sym_DOT_DOT] = ACTIONS(3706),
    [anon_sym_COLON_COLON] = ACTIONS(3706),
    [anon_sym_POUND] = ACTIONS(3706),
    [anon_sym_SQUOTE] = ACTIONS(3708),
    [anon_sym_async] = ACTIONS(3708),
    [anon_sym_auto] = ACTIONS(3708),
    [anon_sym_await] = ACTIONS(3708),
    [anon_sym_become] = ACTIONS(3708),
    [anon_sym_break] = ACTIONS(3708),
    [anon_sym_const] = ACTIONS(3708),
    [anon_sym_continue] = ACTIONS(3708),
    [anon_sym_default] = ACTIONS(3708),
    [anon_sym_enum] = ACTIONS(3708),
    [anon_sym_fn] = ACTIONS(3708),
    [anon_sym_for] = ACTIONS(3708),
    [anon_sym_gen] = ACTIONS(3708),
    [anon_sym_if] = ACTIONS(3708),
    [anon_sym_impl] = ACTIONS(3708),
    [anon_sym_let] = ACTIONS(3708),
    [anon_sym_loop] = ACTIONS(3708),
    [anon_sym_match] = ACTIONS(3708),
    [anon_sym_mod] = ACTIONS(3708),
    [anon_sym_pub] = ACTIONS(3708),
    [anon_sym_return] = ACTIONS(3708),
    [anon_sym_raw] = ACTIONS(3708),
    [anon_sym_static] = ACTIONS(3708),
    [anon_sym_struct] = ACTIONS(3708),
    [anon_sym_trait] = ACTIONS(3708),
    [anon_sym_type] = ACTIONS(3708),
    [anon_sym_union] = ACTIONS(3708),
    [anon_sym_unsafe] = ACTIONS(3708),
    [anon_sym_use] = ACTIONS(3708),
    [anon_sym_while] = ACTIONS(3708),
    [anon_sym_extern] = ACTIONS(3708),
    [anon_sym_safe] = ACTIONS(3708),
    [anon_sym_yield] = ACTIONS(3708),
    [anon_sym_move] = ACTIONS(3708),
    [anon_sym_try] = ACTIONS(3708),
    [sym_integer_literal] = ACTIONS(3706),
    [aux_sym_string_literal_token1] = ACTIONS(3706),
    [sym_char_literal] = ACTIONS(3706),
    [anon_sym_true] = ACTIONS(3708),
    [anon_sym_false] = ACTIONS(3708),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3708),
    [sym_super] = ACTIONS(3708),
    [sym_crate] = ACTIONS(3708),
    [aux_sym_metavariable_token1] = ACTIONS(3706),
    [sym__raw_string_literal_start] = ACTIONS(3706),
    [sym_float_literal] = ACTIONS(3706),
    [sym__outer_block_doc_comment_start] = ACTIONS(3706),
    [sym__outer_line_doc_comment_start] = ACTIONS(3706),
  },
  [STATE(985)] = {
    [sym_line_comment] = STATE(985),
    [sym_block_comment] = STATE(985),
    [ts_builtin_sym_end] = ACTIONS(3710),
    [sym_identifier] = ACTIONS(3712),
    [anon_sym_SEMI] = ACTIONS(3710),
    [anon_sym_macro] = ACTIONS(3712),
    [anon_sym_LPAREN] = ACTIONS(3710),
    [anon_sym_LBRACE] = ACTIONS(3710),
    [anon_sym_RBRACE] = ACTIONS(3710),
    [anon_sym_macro_rules] = ACTIONS(3712),
    [anon_sym_BANG] = ACTIONS(3710),
    [anon_sym_LBRACK] = ACTIONS(3710),
    [anon_sym_STAR] = ACTIONS(3710),
    [anon_sym_u8] = ACTIONS(3712),
    [anon_sym_i8] = ACTIONS(3712),
    [anon_sym_u16] = ACTIONS(3712),
    [anon_sym_i16] = ACTIONS(3712),
    [anon_sym_u32] = ACTIONS(3712),
    [anon_sym_i32] = ACTIONS(3712),
    [anon_sym_u64] = ACTIONS(3712),
    [anon_sym_i64] = ACTIONS(3712),
    [anon_sym_u128] = ACTIONS(3712),
    [anon_sym_i128] = ACTIONS(3712),
    [anon_sym_isize] = ACTIONS(3712),
    [anon_sym_usize] = ACTIONS(3712),
    [anon_sym_f32] = ACTIONS(3712),
    [anon_sym_f64] = ACTIONS(3712),
    [anon_sym_bool] = ACTIONS(3712),
    [anon_sym_str] = ACTIONS(3712),
    [anon_sym_char] = ACTIONS(3712),
    [anon_sym_DASH] = ACTIONS(3710),
    [anon_sym_AMP] = ACTIONS(3710),
    [anon_sym_PIPE] = ACTIONS(3710),
    [anon_sym_LT] = ACTIONS(3710),
    [anon_sym_DOT_DOT] = ACTIONS(3710),
    [anon_sym_COLON_COLON] = ACTIONS(3710),
    [anon_sym_POUND] = ACTIONS(3710),
    [anon_sym_SQUOTE] = ACTIONS(3712),
    [anon_sym_async] = ACTIONS(3712),
    [anon_sym_auto] = ACTIONS(3712),
    [anon_sym_await] = ACTIONS(3712),
    [anon_sym_become] = ACTIONS(3712),
    [anon_sym_break] = ACTIONS(3712),
    [anon_sym_const] = ACTIONS(3712),
    [anon_sym_continue] = ACTIONS(3712),
    [anon_sym_default] = ACTIONS(3712),
    [anon_sym_enum] = ACTIONS(3712),
    [anon_sym_fn] = ACTIONS(3712),
    [anon_sym_for] = ACTIONS(3712),
    [anon_sym_gen] = ACTIONS(3712),
    [anon_sym_if] = ACTIONS(3712),
    [anon_sym_impl] = ACTIONS(3712),
    [anon_sym_let] = ACTIONS(3712),
    [anon_sym_loop] = ACTIONS(3712),
    [anon_sym_match] = ACTIONS(3712),
    [anon_sym_mod] = ACTIONS(3712),
    [anon_sym_pub] = ACTIONS(3712),
    [anon_sym_return] = ACTIONS(3712),
    [anon_sym_raw] = ACTIONS(3712),
    [anon_sym_static] = ACTIONS(3712),
    [anon_sym_struct] = ACTIONS(3712),
    [anon_sym_trait] = ACTIONS(3712),
    [anon_sym_type] = ACTIONS(3712),
    [anon_sym_union] = ACTIONS(3712),
    [anon_sym_unsafe] = ACTIONS(3712),
    [anon_sym_use] = ACTIONS(3712),
    [anon_sym_while] = ACTIONS(3712),
    [anon_sym_extern] = ACTIONS(3712),
    [anon_sym_safe] = ACTIONS(3712),
    [anon_sym_yield] = ACTIONS(3712),
    [anon_sym_move] = ACTIONS(3712),
    [anon_sym_try] = ACTIONS(3712),
    [sym_integer_literal] = ACTIONS(3710),
    [aux_sym_string_literal_token1] = ACTIONS(3710),
    [sym_char_literal] = ACTIONS(3710),
    [anon_sym_true] = ACTIONS(3712),
    [anon_sym_false] = ACTIONS(3712),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3712),
    [sym_super] = ACTIONS(3712),
    [sym_crate] = ACTIONS(3712),
    [aux_sym_metavariable_token1] = ACTIONS(3710),
    [sym__raw_string_literal_start] = ACTIONS(3710),
    [sym_float_literal] = ACTIONS(3710),
    [sym__outer_block_doc_comment_start] = ACTIONS(3710),
    [sym__outer_line_doc_comment_start] = ACTIONS(3710),
  },
  [STATE(986)] = {
    [sym_line_comment] = STATE(986),
    [sym_block_comment] = STATE(986),
    [ts_builtin_sym_end] = ACTIONS(3714),
    [sym_identifier] = ACTIONS(3716),
    [anon_sym_SEMI] = ACTIONS(3714),
    [anon_sym_macro] = ACTIONS(3716),
    [anon_sym_LPAREN] = ACTIONS(3714),
    [anon_sym_LBRACE] = ACTIONS(3714),
    [anon_sym_RBRACE] = ACTIONS(3714),
    [anon_sym_macro_rules] = ACTIONS(3716),
    [anon_sym_BANG] = ACTIONS(3714),
    [anon_sym_LBRACK] = ACTIONS(3714),
    [anon_sym_STAR] = ACTIONS(3714),
    [anon_sym_u8] = ACTIONS(3716),
    [anon_sym_i8] = ACTIONS(3716),
    [anon_sym_u16] = ACTIONS(3716),
    [anon_sym_i16] = ACTIONS(3716),
    [anon_sym_u32] = ACTIONS(3716),
    [anon_sym_i32] = ACTIONS(3716),
    [anon_sym_u64] = ACTIONS(3716),
    [anon_sym_i64] = ACTIONS(3716),
    [anon_sym_u128] = ACTIONS(3716),
    [anon_sym_i128] = ACTIONS(3716),
    [anon_sym_isize] = ACTIONS(3716),
    [anon_sym_usize] = ACTIONS(3716),
    [anon_sym_f32] = ACTIONS(3716),
    [anon_sym_f64] = ACTIONS(3716),
    [anon_sym_bool] = ACTIONS(3716),
    [anon_sym_str] = ACTIONS(3716),
    [anon_sym_char] = ACTIONS(3716),
    [anon_sym_DASH] = ACTIONS(3714),
    [anon_sym_AMP] = ACTIONS(3714),
    [anon_sym_PIPE] = ACTIONS(3714),
    [anon_sym_LT] = ACTIONS(3714),
    [anon_sym_DOT_DOT] = ACTIONS(3714),
    [anon_sym_COLON_COLON] = ACTIONS(3714),
    [anon_sym_POUND] = ACTIONS(3714),
    [anon_sym_SQUOTE] = ACTIONS(3716),
    [anon_sym_async] = ACTIONS(3716),
    [anon_sym_auto] = ACTIONS(3716),
    [anon_sym_await] = ACTIONS(3716),
    [anon_sym_become] = ACTIONS(3716),
    [anon_sym_break] = ACTIONS(3716),
    [anon_sym_const] = ACTIONS(3716),
    [anon_sym_continue] = ACTIONS(3716),
    [anon_sym_default] = ACTIONS(3716),
    [anon_sym_enum] = ACTIONS(3716),
    [anon_sym_fn] = ACTIONS(3716),
    [anon_sym_for] = ACTIONS(3716),
    [anon_sym_gen] = ACTIONS(3716),
    [anon_sym_if] = ACTIONS(3716),
    [anon_sym_impl] = ACTIONS(3716),
    [anon_sym_let] = ACTIONS(3716),
    [anon_sym_loop] = ACTIONS(3716),
    [anon_sym_match] = ACTIONS(3716),
    [anon_sym_mod] = ACTIONS(3716),
    [anon_sym_pub] = ACTIONS(3716),
    [anon_sym_return] = ACTIONS(3716),
    [anon_sym_raw] = ACTIONS(3716),
    [anon_sym_static] = ACTIONS(3716),
    [anon_sym_struct] = ACTIONS(3716),
    [anon_sym_trait] = ACTIONS(3716),
    [anon_sym_type] = ACTIONS(3716),
    [anon_sym_union] = ACTIONS(3716),
    [anon_sym_unsafe] = ACTIONS(3716),
    [anon_sym_use] = ACTIONS(3716),
    [anon_sym_while] = ACTIONS(3716),
    [anon_sym_extern] = ACTIONS(3716),
    [anon_sym_safe] = ACTIONS(3716),
    [anon_sym_yield] = ACTIONS(3716),
    [anon_sym_move] = ACTIONS(3716),
    [anon_sym_try] = ACTIONS(3716),
    [sym_integer_literal] = ACTIONS(3714),
    [aux_sym_string_literal_token1] = ACTIONS(3714),
    [sym_char_literal] = ACTIONS(3714),
    [anon_sym_true] = ACTIONS(3716),
    [anon_sym_false] = ACTIONS(3716),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3716),
    [sym_super] = ACTIONS(3716),
    [sym_crate] = ACTIONS(3716),
    [aux_sym_metavariable_token1] = ACTIONS(3714),
    [sym__raw_string_literal_start] = ACTIONS(3714),
    [sym_float_literal] = ACTIONS(3714),
    [sym__outer_block_doc_comment_start] = ACTIONS(3714),
    [sym__outer_line_doc_comment_start] = ACTIONS(3714),
  },
  [STATE(987)] = {
    [sym_line_comment] = STATE(987),
    [sym_block_comment] = STATE(987),
    [ts_builtin_sym_end] = ACTIONS(3718),
    [sym_identifier] = ACTIONS(3720),
    [anon_sym_SEMI] = ACTIONS(3718),
    [anon_sym_macro] = ACTIONS(3720),
    [anon_sym_LPAREN] = ACTIONS(3718),
    [anon_sym_LBRACE] = ACTIONS(3718),
    [anon_sym_RBRACE] = ACTIONS(3718),
    [anon_sym_macro_rules] = ACTIONS(3720),
    [anon_sym_BANG] = ACTIONS(3718),
    [anon_sym_LBRACK] = ACTIONS(3718),
    [anon_sym_STAR] = ACTIONS(3718),
    [anon_sym_u8] = ACTIONS(3720),
    [anon_sym_i8] = ACTIONS(3720),
    [anon_sym_u16] = ACTIONS(3720),
    [anon_sym_i16] = ACTIONS(3720),
    [anon_sym_u32] = ACTIONS(3720),
    [anon_sym_i32] = ACTIONS(3720),
    [anon_sym_u64] = ACTIONS(3720),
    [anon_sym_i64] = ACTIONS(3720),
    [anon_sym_u128] = ACTIONS(3720),
    [anon_sym_i128] = ACTIONS(3720),
    [anon_sym_isize] = ACTIONS(3720),
    [anon_sym_usize] = ACTIONS(3720),
    [anon_sym_f32] = ACTIONS(3720),
    [anon_sym_f64] = ACTIONS(3720),
    [anon_sym_bool] = ACTIONS(3720),
    [anon_sym_str] = ACTIONS(3720),
    [anon_sym_char] = ACTIONS(3720),
    [anon_sym_DASH] = ACTIONS(3718),
    [anon_sym_AMP] = ACTIONS(3718),
    [anon_sym_PIPE] = ACTIONS(3718),
    [anon_sym_LT] = ACTIONS(3718),
    [anon_sym_DOT_DOT] = ACTIONS(3718),
    [anon_sym_COLON_COLON] = ACTIONS(3718),
    [anon_sym_POUND] = ACTIONS(3718),
    [anon_sym_SQUOTE] = ACTIONS(3720),
    [anon_sym_async] = ACTIONS(3720),
    [anon_sym_auto] = ACTIONS(3720),
    [anon_sym_await] = ACTIONS(3720),
    [anon_sym_become] = ACTIONS(3720),
    [anon_sym_break] = ACTIONS(3720),
    [anon_sym_const] = ACTIONS(3720),
    [anon_sym_continue] = ACTIONS(3720),
    [anon_sym_default] = ACTIONS(3720),
    [anon_sym_enum] = ACTIONS(3720),
    [anon_sym_fn] = ACTIONS(3720),
    [anon_sym_for] = ACTIONS(3720),
    [anon_sym_gen] = ACTIONS(3720),
    [anon_sym_if] = ACTIONS(3720),
    [anon_sym_impl] = ACTIONS(3720),
    [anon_sym_let] = ACTIONS(3720),
    [anon_sym_loop] = ACTIONS(3720),
    [anon_sym_match] = ACTIONS(3720),
    [anon_sym_mod] = ACTIONS(3720),
    [anon_sym_pub] = ACTIONS(3720),
    [anon_sym_return] = ACTIONS(3720),
    [anon_sym_raw] = ACTIONS(3720),
    [anon_sym_static] = ACTIONS(3720),
    [anon_sym_struct] = ACTIONS(3720),
    [anon_sym_trait] = ACTIONS(3720),
    [anon_sym_type] = ACTIONS(3720),
    [anon_sym_union] = ACTIONS(3720),
    [anon_sym_unsafe] = ACTIONS(3720),
    [anon_sym_use] = ACTIONS(3720),
    [anon_sym_while] = ACTIONS(3720),
    [anon_sym_extern] = ACTIONS(3720),
    [anon_sym_safe] = ACTIONS(3720),
    [anon_sym_yield] = ACTIONS(3720),
    [anon_sym_move] = ACTIONS(3720),
    [anon_sym_try] = ACTIONS(3720),
    [sym_integer_literal] = ACTIONS(3718),
    [aux_sym_string_literal_token1] = ACTIONS(3718),
    [sym_char_literal] = ACTIONS(3718),
    [anon_sym_true] = ACTIONS(3720),
    [anon_sym_false] = ACTIONS(3720),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3720),
    [sym_super] = ACTIONS(3720),
    [sym_crate] = ACTIONS(3720),
    [aux_sym_metavariable_token1] = ACTIONS(3718),
    [sym__raw_string_literal_start] = ACTIONS(3718),
    [sym_float_literal] = ACTIONS(3718),
    [sym__outer_block_doc_comment_start] = ACTIONS(3718),
    [sym__outer_line_doc_comment_start] = ACTIONS(3718),
  },
  [STATE(988)] = {
    [sym_line_comment] = STATE(988),
    [sym_block_comment] = STATE(988),
    [ts_builtin_sym_end] = ACTIONS(3722),
    [sym_identifier] = ACTIONS(3724),
    [anon_sym_SEMI] = ACTIONS(3722),
    [anon_sym_macro] = ACTIONS(3724),
    [anon_sym_LPAREN] = ACTIONS(3722),
    [anon_sym_LBRACE] = ACTIONS(3722),
    [anon_sym_RBRACE] = ACTIONS(3722),
    [anon_sym_macro_rules] = ACTIONS(3724),
    [anon_sym_BANG] = ACTIONS(3722),
    [anon_sym_LBRACK] = ACTIONS(3722),
    [anon_sym_STAR] = ACTIONS(3722),
    [anon_sym_u8] = ACTIONS(3724),
    [anon_sym_i8] = ACTIONS(3724),
    [anon_sym_u16] = ACTIONS(3724),
    [anon_sym_i16] = ACTIONS(3724),
    [anon_sym_u32] = ACTIONS(3724),
    [anon_sym_i32] = ACTIONS(3724),
    [anon_sym_u64] = ACTIONS(3724),
    [anon_sym_i64] = ACTIONS(3724),
    [anon_sym_u128] = ACTIONS(3724),
    [anon_sym_i128] = ACTIONS(3724),
    [anon_sym_isize] = ACTIONS(3724),
    [anon_sym_usize] = ACTIONS(3724),
    [anon_sym_f32] = ACTIONS(3724),
    [anon_sym_f64] = ACTIONS(3724),
    [anon_sym_bool] = ACTIONS(3724),
    [anon_sym_str] = ACTIONS(3724),
    [anon_sym_char] = ACTIONS(3724),
    [anon_sym_DASH] = ACTIONS(3722),
    [anon_sym_AMP] = ACTIONS(3722),
    [anon_sym_PIPE] = ACTIONS(3722),
    [anon_sym_LT] = ACTIONS(3722),
    [anon_sym_DOT_DOT] = ACTIONS(3722),
    [anon_sym_COLON_COLON] = ACTIONS(3722),
    [anon_sym_POUND] = ACTIONS(3722),
    [anon_sym_SQUOTE] = ACTIONS(3724),
    [anon_sym_async] = ACTIONS(3724),
    [anon_sym_auto] = ACTIONS(3724),
    [anon_sym_await] = ACTIONS(3724),
    [anon_sym_become] = ACTIONS(3724),
    [anon_sym_break] = ACTIONS(3724),
    [anon_sym_const] = ACTIONS(3724),
    [anon_sym_continue] = ACTIONS(3724),
    [anon_sym_default] = ACTIONS(3724),
    [anon_sym_enum] = ACTIONS(3724),
    [anon_sym_fn] = ACTIONS(3724),
    [anon_sym_for] = ACTIONS(3724),
    [anon_sym_gen] = ACTIONS(3724),
    [anon_sym_if] = ACTIONS(3724),
    [anon_sym_impl] = ACTIONS(3724),
    [anon_sym_let] = ACTIONS(3724),
    [anon_sym_loop] = ACTIONS(3724),
    [anon_sym_match] = ACTIONS(3724),
    [anon_sym_mod] = ACTIONS(3724),
    [anon_sym_pub] = ACTIONS(3724),
    [anon_sym_return] = ACTIONS(3724),
    [anon_sym_raw] = ACTIONS(3724),
    [anon_sym_static] = ACTIONS(3724),
    [anon_sym_struct] = ACTIONS(3724),
    [anon_sym_trait] = ACTIONS(3724),
    [anon_sym_type] = ACTIONS(3724),
    [anon_sym_union] = ACTIONS(3724),
    [anon_sym_unsafe] = ACTIONS(3724),
    [anon_sym_use] = ACTIONS(3724),
    [anon_sym_while] = ACTIONS(3724),
    [anon_sym_extern] = ACTIONS(3724),
    [anon_sym_safe] = ACTIONS(3724),
    [anon_sym_yield] = ACTIONS(3724),
    [anon_sym_move] = ACTIONS(3724),
    [anon_sym_try] = ACTIONS(3724),
    [sym_integer_literal] = ACTIONS(3722),
    [aux_sym_string_literal_token1] = ACTIONS(3722),
    [sym_char_literal] = ACTIONS(3722),
    [anon_sym_true] = ACTIONS(3724),
    [anon_sym_false] = ACTIONS(3724),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3724),
    [sym_super] = ACTIONS(3724),
    [sym_crate] = ACTIONS(3724),
    [aux_sym_metavariable_token1] = ACTIONS(3722),
    [sym__raw_string_literal_start] = ACTIONS(3722),
    [sym_float_literal] = ACTIONS(3722),
    [sym__outer_block_doc_comment_start] = ACTIONS(3722),
    [sym__outer_line_doc_comment_start] = ACTIONS(3722),
  },
  [STATE(989)] = {
    [sym_line_comment] = STATE(989),
    [sym_block_comment] = STATE(989),
    [ts_builtin_sym_end] = ACTIONS(3726),
    [sym_identifier] = ACTIONS(3728),
    [anon_sym_SEMI] = ACTIONS(3726),
    [anon_sym_macro] = ACTIONS(3728),
    [anon_sym_LPAREN] = ACTIONS(3726),
    [anon_sym_LBRACE] = ACTIONS(3726),
    [anon_sym_RBRACE] = ACTIONS(3726),
    [anon_sym_macro_rules] = ACTIONS(3728),
    [anon_sym_BANG] = ACTIONS(3726),
    [anon_sym_LBRACK] = ACTIONS(3726),
    [anon_sym_STAR] = ACTIONS(3726),
    [anon_sym_u8] = ACTIONS(3728),
    [anon_sym_i8] = ACTIONS(3728),
    [anon_sym_u16] = ACTIONS(3728),
    [anon_sym_i16] = ACTIONS(3728),
    [anon_sym_u32] = ACTIONS(3728),
    [anon_sym_i32] = ACTIONS(3728),
    [anon_sym_u64] = ACTIONS(3728),
    [anon_sym_i64] = ACTIONS(3728),
    [anon_sym_u128] = ACTIONS(3728),
    [anon_sym_i128] = ACTIONS(3728),
    [anon_sym_isize] = ACTIONS(3728),
    [anon_sym_usize] = ACTIONS(3728),
    [anon_sym_f32] = ACTIONS(3728),
    [anon_sym_f64] = ACTIONS(3728),
    [anon_sym_bool] = ACTIONS(3728),
    [anon_sym_str] = ACTIONS(3728),
    [anon_sym_char] = ACTIONS(3728),
    [anon_sym_DASH] = ACTIONS(3726),
    [anon_sym_AMP] = ACTIONS(3726),
    [anon_sym_PIPE] = ACTIONS(3726),
    [anon_sym_LT] = ACTIONS(3726),
    [anon_sym_DOT_DOT] = ACTIONS(3726),
    [anon_sym_COLON_COLON] = ACTIONS(3726),
    [anon_sym_POUND] = ACTIONS(3726),
    [anon_sym_SQUOTE] = ACTIONS(3728),
    [anon_sym_async] = ACTIONS(3728),
    [anon_sym_auto] = ACTIONS(3728),
    [anon_sym_await] = ACTIONS(3728),
    [anon_sym_become] = ACTIONS(3728),
    [anon_sym_break] = ACTIONS(3728),
    [anon_sym_const] = ACTIONS(3728),
    [anon_sym_continue] = ACTIONS(3728),
    [anon_sym_default] = ACTIONS(3728),
    [anon_sym_enum] = ACTIONS(3728),
    [anon_sym_fn] = ACTIONS(3728),
    [anon_sym_for] = ACTIONS(3728),
    [anon_sym_gen] = ACTIONS(3728),
    [anon_sym_if] = ACTIONS(3728),
    [anon_sym_impl] = ACTIONS(3728),
    [anon_sym_let] = ACTIONS(3728),
    [anon_sym_loop] = ACTIONS(3728),
    [anon_sym_match] = ACTIONS(3728),
    [anon_sym_mod] = ACTIONS(3728),
    [anon_sym_pub] = ACTIONS(3728),
    [anon_sym_return] = ACTIONS(3728),
    [anon_sym_raw] = ACTIONS(3728),
    [anon_sym_static] = ACTIONS(3728),
    [anon_sym_struct] = ACTIONS(3728),
    [anon_sym_trait] = ACTIONS(3728),
    [anon_sym_type] = ACTIONS(3728),
    [anon_sym_union] = ACTIONS(3728),
    [anon_sym_unsafe] = ACTIONS(3728),
    [anon_sym_use] = ACTIONS(3728),
    [anon_sym_while] = ACTIONS(3728),
    [anon_sym_extern] = ACTIONS(3728),
    [anon_sym_safe] = ACTIONS(3728),
    [anon_sym_yield] = ACTIONS(3728),
    [anon_sym_move] = ACTIONS(3728),
    [anon_sym_try] = ACTIONS(3728),
    [sym_integer_literal] = ACTIONS(3726),
    [aux_sym_string_literal_token1] = ACTIONS(3726),
    [sym_char_literal] = ACTIONS(3726),
    [anon_sym_true] = ACTIONS(3728),
    [anon_sym_false] = ACTIONS(3728),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3728),
    [sym_super] = ACTIONS(3728),
    [sym_crate] = ACTIONS(3728),
    [aux_sym_metavariable_token1] = ACTIONS(3726),
    [sym__raw_string_literal_start] = ACTIONS(3726),
    [sym_float_literal] = ACTIONS(3726),
    [sym__outer_block_doc_comment_start] = ACTIONS(3726),
    [sym__outer_line_doc_comment_start] = ACTIONS(3726),
  },
  [STATE(990)] = {
    [sym_line_comment] = STATE(990),
    [sym_block_comment] = STATE(990),
    [ts_builtin_sym_end] = ACTIONS(3730),
    [sym_identifier] = ACTIONS(3732),
    [anon_sym_SEMI] = ACTIONS(3730),
    [anon_sym_macro] = ACTIONS(3732),
    [anon_sym_LPAREN] = ACTIONS(3730),
    [anon_sym_LBRACE] = ACTIONS(3730),
    [anon_sym_RBRACE] = ACTIONS(3730),
    [anon_sym_macro_rules] = ACTIONS(3732),
    [anon_sym_BANG] = ACTIONS(3730),
    [anon_sym_LBRACK] = ACTIONS(3730),
    [anon_sym_STAR] = ACTIONS(3730),
    [anon_sym_u8] = ACTIONS(3732),
    [anon_sym_i8] = ACTIONS(3732),
    [anon_sym_u16] = ACTIONS(3732),
    [anon_sym_i16] = ACTIONS(3732),
    [anon_sym_u32] = ACTIONS(3732),
    [anon_sym_i32] = ACTIONS(3732),
    [anon_sym_u64] = ACTIONS(3732),
    [anon_sym_i64] = ACTIONS(3732),
    [anon_sym_u128] = ACTIONS(3732),
    [anon_sym_i128] = ACTIONS(3732),
    [anon_sym_isize] = ACTIONS(3732),
    [anon_sym_usize] = ACTIONS(3732),
    [anon_sym_f32] = ACTIONS(3732),
    [anon_sym_f64] = ACTIONS(3732),
    [anon_sym_bool] = ACTIONS(3732),
    [anon_sym_str] = ACTIONS(3732),
    [anon_sym_char] = ACTIONS(3732),
    [anon_sym_DASH] = ACTIONS(3730),
    [anon_sym_AMP] = ACTIONS(3730),
    [anon_sym_PIPE] = ACTIONS(3730),
    [anon_sym_LT] = ACTIONS(3730),
    [anon_sym_DOT_DOT] = ACTIONS(3730),
    [anon_sym_COLON_COLON] = ACTIONS(3730),
    [anon_sym_POUND] = ACTIONS(3730),
    [anon_sym_SQUOTE] = ACTIONS(3732),
    [anon_sym_async] = ACTIONS(3732),
    [anon_sym_auto] = ACTIONS(3732),
    [anon_sym_await] = ACTIONS(3732),
    [anon_sym_become] = ACTIONS(3732),
    [anon_sym_break] = ACTIONS(3732),
    [anon_sym_const] = ACTIONS(3732),
    [anon_sym_continue] = ACTIONS(3732),
    [anon_sym_default] = ACTIONS(3732),
    [anon_sym_enum] = ACTIONS(3732),
    [anon_sym_fn] = ACTIONS(3732),
    [anon_sym_for] = ACTIONS(3732),
    [anon_sym_gen] = ACTIONS(3732),
    [anon_sym_if] = ACTIONS(3732),
    [anon_sym_impl] = ACTIONS(3732),
    [anon_sym_let] = ACTIONS(3732),
    [anon_sym_loop] = ACTIONS(3732),
    [anon_sym_match] = ACTIONS(3732),
    [anon_sym_mod] = ACTIONS(3732),
    [anon_sym_pub] = ACTIONS(3732),
    [anon_sym_return] = ACTIONS(3732),
    [anon_sym_raw] = ACTIONS(3732),
    [anon_sym_static] = ACTIONS(3732),
    [anon_sym_struct] = ACTIONS(3732),
    [anon_sym_trait] = ACTIONS(3732),
    [anon_sym_type] = ACTIONS(3732),
    [anon_sym_union] = ACTIONS(3732),
    [anon_sym_unsafe] = ACTIONS(3732),
    [anon_sym_use] = ACTIONS(3732),
    [anon_sym_while] = ACTIONS(3732),
    [anon_sym_extern] = ACTIONS(3732),
    [anon_sym_safe] = ACTIONS(3732),
    [anon_sym_yield] = ACTIONS(3732),
    [anon_sym_move] = ACTIONS(3732),
    [anon_sym_try] = ACTIONS(3732),
    [sym_integer_literal] = ACTIONS(3730),
    [aux_sym_string_literal_token1] = ACTIONS(3730),
    [sym_char_literal] = ACTIONS(3730),
    [anon_sym_true] = ACTIONS(3732),
    [anon_sym_false] = ACTIONS(3732),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3732),
    [sym_super] = ACTIONS(3732),
    [sym_crate] = ACTIONS(3732),
    [aux_sym_metavariable_token1] = ACTIONS(3730),
    [sym__raw_string_literal_start] = ACTIONS(3730),
    [sym_float_literal] = ACTIONS(3730),
    [sym__outer_block_doc_comment_start] = ACTIONS(3730),
    [sym__outer_line_doc_comment_start] = ACTIONS(3730),
  },
  [STATE(991)] = {
    [sym_line_comment] = STATE(991),
    [sym_block_comment] = STATE(991),
    [ts_builtin_sym_end] = ACTIONS(3734),
    [sym_identifier] = ACTIONS(3736),
    [anon_sym_SEMI] = ACTIONS(3734),
    [anon_sym_macro] = ACTIONS(3736),
    [anon_sym_LPAREN] = ACTIONS(3734),
    [anon_sym_LBRACE] = ACTIONS(3734),
    [anon_sym_RBRACE] = ACTIONS(3734),
    [anon_sym_macro_rules] = ACTIONS(3736),
    [anon_sym_BANG] = ACTIONS(3734),
    [anon_sym_LBRACK] = ACTIONS(3734),
    [anon_sym_STAR] = ACTIONS(3734),
    [anon_sym_u8] = ACTIONS(3736),
    [anon_sym_i8] = ACTIONS(3736),
    [anon_sym_u16] = ACTIONS(3736),
    [anon_sym_i16] = ACTIONS(3736),
    [anon_sym_u32] = ACTIONS(3736),
    [anon_sym_i32] = ACTIONS(3736),
    [anon_sym_u64] = ACTIONS(3736),
    [anon_sym_i64] = ACTIONS(3736),
    [anon_sym_u128] = ACTIONS(3736),
    [anon_sym_i128] = ACTIONS(3736),
    [anon_sym_isize] = ACTIONS(3736),
    [anon_sym_usize] = ACTIONS(3736),
    [anon_sym_f32] = ACTIONS(3736),
    [anon_sym_f64] = ACTIONS(3736),
    [anon_sym_bool] = ACTIONS(3736),
    [anon_sym_str] = ACTIONS(3736),
    [anon_sym_char] = ACTIONS(3736),
    [anon_sym_DASH] = ACTIONS(3734),
    [anon_sym_AMP] = ACTIONS(3734),
    [anon_sym_PIPE] = ACTIONS(3734),
    [anon_sym_LT] = ACTIONS(3734),
    [anon_sym_DOT_DOT] = ACTIONS(3734),
    [anon_sym_COLON_COLON] = ACTIONS(3734),
    [anon_sym_POUND] = ACTIONS(3734),
    [anon_sym_SQUOTE] = ACTIONS(3736),
    [anon_sym_async] = ACTIONS(3736),
    [anon_sym_auto] = ACTIONS(3736),
    [anon_sym_await] = ACTIONS(3736),
    [anon_sym_become] = ACTIONS(3736),
    [anon_sym_break] = ACTIONS(3736),
    [anon_sym_const] = ACTIONS(3736),
    [anon_sym_continue] = ACTIONS(3736),
    [anon_sym_default] = ACTIONS(3736),
    [anon_sym_enum] = ACTIONS(3736),
    [anon_sym_fn] = ACTIONS(3736),
    [anon_sym_for] = ACTIONS(3736),
    [anon_sym_gen] = ACTIONS(3736),
    [anon_sym_if] = ACTIONS(3736),
    [anon_sym_impl] = ACTIONS(3736),
    [anon_sym_let] = ACTIONS(3736),
    [anon_sym_loop] = ACTIONS(3736),
    [anon_sym_match] = ACTIONS(3736),
    [anon_sym_mod] = ACTIONS(3736),
    [anon_sym_pub] = ACTIONS(3736),
    [anon_sym_return] = ACTIONS(3736),
    [anon_sym_raw] = ACTIONS(3736),
    [anon_sym_static] = ACTIONS(3736),
    [anon_sym_struct] = ACTIONS(3736),
    [anon_sym_trait] = ACTIONS(3736),
    [anon_sym_type] = ACTIONS(3736),
    [anon_sym_union] = ACTIONS(3736),
    [anon_sym_unsafe] = ACTIONS(3736),
    [anon_sym_use] = ACTIONS(3736),
    [anon_sym_while] = ACTIONS(3736),
    [anon_sym_extern] = ACTIONS(3736),
    [anon_sym_safe] = ACTIONS(3736),
    [anon_sym_yield] = ACTIONS(3736),
    [anon_sym_move] = ACTIONS(3736),
    [anon_sym_try] = ACTIONS(3736),
    [sym_integer_literal] = ACTIONS(3734),
    [aux_sym_string_literal_token1] = ACTIONS(3734),
    [sym_char_literal] = ACTIONS(3734),
    [anon_sym_true] = ACTIONS(3736),
    [anon_sym_false] = ACTIONS(3736),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3736),
    [sym_super] = ACTIONS(3736),
    [sym_crate] = ACTIONS(3736),
    [aux_sym_metavariable_token1] = ACTIONS(3734),
    [sym__raw_string_literal_start] = ACTIONS(3734),
    [sym_float_literal] = ACTIONS(3734),
    [sym__outer_block_doc_comment_start] = ACTIONS(3734),
    [sym__outer_line_doc_comment_start] = ACTIONS(3734),
  },
  [STATE(992)] = {
    [sym_line_comment] = STATE(992),
    [sym_block_comment] = STATE(992),
    [ts_builtin_sym_end] = ACTIONS(3738),
    [sym_identifier] = ACTIONS(3740),
    [anon_sym_SEMI] = ACTIONS(3738),
    [anon_sym_macro] = ACTIONS(3740),
    [anon_sym_LPAREN] = ACTIONS(3738),
    [anon_sym_LBRACE] = ACTIONS(3738),
    [anon_sym_RBRACE] = ACTIONS(3738),
    [anon_sym_macro_rules] = ACTIONS(3740),
    [anon_sym_BANG] = ACTIONS(3738),
    [anon_sym_LBRACK] = ACTIONS(3738),
    [anon_sym_STAR] = ACTIONS(3738),
    [anon_sym_u8] = ACTIONS(3740),
    [anon_sym_i8] = ACTIONS(3740),
    [anon_sym_u16] = ACTIONS(3740),
    [anon_sym_i16] = ACTIONS(3740),
    [anon_sym_u32] = ACTIONS(3740),
    [anon_sym_i32] = ACTIONS(3740),
    [anon_sym_u64] = ACTIONS(3740),
    [anon_sym_i64] = ACTIONS(3740),
    [anon_sym_u128] = ACTIONS(3740),
    [anon_sym_i128] = ACTIONS(3740),
    [anon_sym_isize] = ACTIONS(3740),
    [anon_sym_usize] = ACTIONS(3740),
    [anon_sym_f32] = ACTIONS(3740),
    [anon_sym_f64] = ACTIONS(3740),
    [anon_sym_bool] = ACTIONS(3740),
    [anon_sym_str] = ACTIONS(3740),
    [anon_sym_char] = ACTIONS(3740),
    [anon_sym_DASH] = ACTIONS(3738),
    [anon_sym_AMP] = ACTIONS(3738),
    [anon_sym_PIPE] = ACTIONS(3738),
    [anon_sym_LT] = ACTIONS(3738),
    [anon_sym_DOT_DOT] = ACTIONS(3738),
    [anon_sym_COLON_COLON] = ACTIONS(3738),
    [anon_sym_POUND] = ACTIONS(3738),
    [anon_sym_SQUOTE] = ACTIONS(3740),
    [anon_sym_async] = ACTIONS(3740),
    [anon_sym_auto] = ACTIONS(3740),
    [anon_sym_await] = ACTIONS(3740),
    [anon_sym_become] = ACTIONS(3740),
    [anon_sym_break] = ACTIONS(3740),
    [anon_sym_const] = ACTIONS(3740),
    [anon_sym_continue] = ACTIONS(3740),
    [anon_sym_default] = ACTIONS(3740),
    [anon_sym_enum] = ACTIONS(3740),
    [anon_sym_fn] = ACTIONS(3740),
    [anon_sym_for] = ACTIONS(3740),
    [anon_sym_gen] = ACTIONS(3740),
    [anon_sym_if] = ACTIONS(3740),
    [anon_sym_impl] = ACTIONS(3740),
    [anon_sym_let] = ACTIONS(3740),
    [anon_sym_loop] = ACTIONS(3740),
    [anon_sym_match] = ACTIONS(3740),
    [anon_sym_mod] = ACTIONS(3740),
    [anon_sym_pub] = ACTIONS(3740),
    [anon_sym_return] = ACTIONS(3740),
    [anon_sym_raw] = ACTIONS(3740),
    [anon_sym_static] = ACTIONS(3740),
    [anon_sym_struct] = ACTIONS(3740),
    [anon_sym_trait] = ACTIONS(3740),
    [anon_sym_type] = ACTIONS(3740),
    [anon_sym_union] = ACTIONS(3740),
    [anon_sym_unsafe] = ACTIONS(3740),
    [anon_sym_use] = ACTIONS(3740),
    [anon_sym_while] = ACTIONS(3740),
    [anon_sym_extern] = ACTIONS(3740),
    [anon_sym_safe] = ACTIONS(3740),
    [anon_sym_yield] = ACTIONS(3740),
    [anon_sym_move] = ACTIONS(3740),
    [anon_sym_try] = ACTIONS(3740),
    [sym_integer_literal] = ACTIONS(3738),
    [aux_sym_string_literal_token1] = ACTIONS(3738),
    [sym_char_literal] = ACTIONS(3738),
    [anon_sym_true] = ACTIONS(3740),
    [anon_sym_false] = ACTIONS(3740),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3740),
    [sym_super] = ACTIONS(3740),
    [sym_crate] = ACTIONS(3740),
    [aux_sym_metavariable_token1] = ACTIONS(3738),
    [sym__raw_string_literal_start] = ACTIONS(3738),
    [sym_float_literal] = ACTIONS(3738),
    [sym__outer_block_doc_comment_start] = ACTIONS(3738),
    [sym__outer_line_doc_comment_start] = ACTIONS(3738),
  },
  [STATE(993)] = {
    [sym_line_comment] = STATE(993),
    [sym_block_comment] = STATE(993),
    [ts_builtin_sym_end] = ACTIONS(3742),
    [sym_identifier] = ACTIONS(3744),
    [anon_sym_SEMI] = ACTIONS(3742),
    [anon_sym_macro] = ACTIONS(3744),
    [anon_sym_LPAREN] = ACTIONS(3742),
    [anon_sym_LBRACE] = ACTIONS(3742),
    [anon_sym_RBRACE] = ACTIONS(3742),
    [anon_sym_macro_rules] = ACTIONS(3744),
    [anon_sym_BANG] = ACTIONS(3742),
    [anon_sym_LBRACK] = ACTIONS(3742),
    [anon_sym_STAR] = ACTIONS(3742),
    [anon_sym_u8] = ACTIONS(3744),
    [anon_sym_i8] = ACTIONS(3744),
    [anon_sym_u16] = ACTIONS(3744),
    [anon_sym_i16] = ACTIONS(3744),
    [anon_sym_u32] = ACTIONS(3744),
    [anon_sym_i32] = ACTIONS(3744),
    [anon_sym_u64] = ACTIONS(3744),
    [anon_sym_i64] = ACTIONS(3744),
    [anon_sym_u128] = ACTIONS(3744),
    [anon_sym_i128] = ACTIONS(3744),
    [anon_sym_isize] = ACTIONS(3744),
    [anon_sym_usize] = ACTIONS(3744),
    [anon_sym_f32] = ACTIONS(3744),
    [anon_sym_f64] = ACTIONS(3744),
    [anon_sym_bool] = ACTIONS(3744),
    [anon_sym_str] = ACTIONS(3744),
    [anon_sym_char] = ACTIONS(3744),
    [anon_sym_DASH] = ACTIONS(3742),
    [anon_sym_AMP] = ACTIONS(3742),
    [anon_sym_PIPE] = ACTIONS(3742),
    [anon_sym_LT] = ACTIONS(3742),
    [anon_sym_DOT_DOT] = ACTIONS(3742),
    [anon_sym_COLON_COLON] = ACTIONS(3742),
    [anon_sym_POUND] = ACTIONS(3742),
    [anon_sym_SQUOTE] = ACTIONS(3744),
    [anon_sym_async] = ACTIONS(3744),
    [anon_sym_auto] = ACTIONS(3744),
    [anon_sym_await] = ACTIONS(3744),
    [anon_sym_become] = ACTIONS(3744),
    [anon_sym_break] = ACTIONS(3744),
    [anon_sym_const] = ACTIONS(3744),
    [anon_sym_continue] = ACTIONS(3744),
    [anon_sym_default] = ACTIONS(3744),
    [anon_sym_enum] = ACTIONS(3744),
    [anon_sym_fn] = ACTIONS(3744),
    [anon_sym_for] = ACTIONS(3744),
    [anon_sym_gen] = ACTIONS(3744),
    [anon_sym_if] = ACTIONS(3744),
    [anon_sym_impl] = ACTIONS(3744),
    [anon_sym_let] = ACTIONS(3744),
    [anon_sym_loop] = ACTIONS(3744),
    [anon_sym_match] = ACTIONS(3744),
    [anon_sym_mod] = ACTIONS(3744),
    [anon_sym_pub] = ACTIONS(3744),
    [anon_sym_return] = ACTIONS(3744),
    [anon_sym_raw] = ACTIONS(3744),
    [anon_sym_static] = ACTIONS(3744),
    [anon_sym_struct] = ACTIONS(3744),
    [anon_sym_trait] = ACTIONS(3744),
    [anon_sym_type] = ACTIONS(3744),
    [anon_sym_union] = ACTIONS(3744),
    [anon_sym_unsafe] = ACTIONS(3744),
    [anon_sym_use] = ACTIONS(3744),
    [anon_sym_while] = ACTIONS(3744),
    [anon_sym_extern] = ACTIONS(3744),
    [anon_sym_safe] = ACTIONS(3744),
    [anon_sym_yield] = ACTIONS(3744),
    [anon_sym_move] = ACTIONS(3744),
    [anon_sym_try] = ACTIONS(3744),
    [sym_integer_literal] = ACTIONS(3742),
    [aux_sym_string_literal_token1] = ACTIONS(3742),
    [sym_char_literal] = ACTIONS(3742),
    [anon_sym_true] = ACTIONS(3744),
    [anon_sym_false] = ACTIONS(3744),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3744),
    [sym_super] = ACTIONS(3744),
    [sym_crate] = ACTIONS(3744),
    [aux_sym_metavariable_token1] = ACTIONS(3742),
    [sym__raw_string_literal_start] = ACTIONS(3742),
    [sym_float_literal] = ACTIONS(3742),
    [sym__outer_block_doc_comment_start] = ACTIONS(3742),
    [sym__outer_line_doc_comment_start] = ACTIONS(3742),
  },
  [STATE(994)] = {
    [sym_line_comment] = STATE(994),
    [sym_block_comment] = STATE(994),
    [ts_builtin_sym_end] = ACTIONS(3746),
    [sym_identifier] = ACTIONS(3748),
    [anon_sym_SEMI] = ACTIONS(3746),
    [anon_sym_macro] = ACTIONS(3748),
    [anon_sym_LPAREN] = ACTIONS(3746),
    [anon_sym_LBRACE] = ACTIONS(3746),
    [anon_sym_RBRACE] = ACTIONS(3746),
    [anon_sym_macro_rules] = ACTIONS(3748),
    [anon_sym_BANG] = ACTIONS(3746),
    [anon_sym_LBRACK] = ACTIONS(3746),
    [anon_sym_STAR] = ACTIONS(3746),
    [anon_sym_u8] = ACTIONS(3748),
    [anon_sym_i8] = ACTIONS(3748),
    [anon_sym_u16] = ACTIONS(3748),
    [anon_sym_i16] = ACTIONS(3748),
    [anon_sym_u32] = ACTIONS(3748),
    [anon_sym_i32] = ACTIONS(3748),
    [anon_sym_u64] = ACTIONS(3748),
    [anon_sym_i64] = ACTIONS(3748),
    [anon_sym_u128] = ACTIONS(3748),
    [anon_sym_i128] = ACTIONS(3748),
    [anon_sym_isize] = ACTIONS(3748),
    [anon_sym_usize] = ACTIONS(3748),
    [anon_sym_f32] = ACTIONS(3748),
    [anon_sym_f64] = ACTIONS(3748),
    [anon_sym_bool] = ACTIONS(3748),
    [anon_sym_str] = ACTIONS(3748),
    [anon_sym_char] = ACTIONS(3748),
    [anon_sym_DASH] = ACTIONS(3746),
    [anon_sym_AMP] = ACTIONS(3746),
    [anon_sym_PIPE] = ACTIONS(3746),
    [anon_sym_LT] = ACTIONS(3746),
    [anon_sym_DOT_DOT] = ACTIONS(3746),
    [anon_sym_COLON_COLON] = ACTIONS(3746),
    [anon_sym_POUND] = ACTIONS(3746),
    [anon_sym_SQUOTE] = ACTIONS(3748),
    [anon_sym_async] = ACTIONS(3748),
    [anon_sym_auto] = ACTIONS(3748),
    [anon_sym_await] = ACTIONS(3748),
    [anon_sym_become] = ACTIONS(3748),
    [anon_sym_break] = ACTIONS(3748),
    [anon_sym_const] = ACTIONS(3748),
    [anon_sym_continue] = ACTIONS(3748),
    [anon_sym_default] = ACTIONS(3748),
    [anon_sym_enum] = ACTIONS(3748),
    [anon_sym_fn] = ACTIONS(3748),
    [anon_sym_for] = ACTIONS(3748),
    [anon_sym_gen] = ACTIONS(3748),
    [anon_sym_if] = ACTIONS(3748),
    [anon_sym_impl] = ACTIONS(3748),
    [anon_sym_let] = ACTIONS(3748),
    [anon_sym_loop] = ACTIONS(3748),
    [anon_sym_match] = ACTIONS(3748),
    [anon_sym_mod] = ACTIONS(3748),
    [anon_sym_pub] = ACTIONS(3748),
    [anon_sym_return] = ACTIONS(3748),
    [anon_sym_raw] = ACTIONS(3748),
    [anon_sym_static] = ACTIONS(3748),
    [anon_sym_struct] = ACTIONS(3748),
    [anon_sym_trait] = ACTIONS(3748),
    [anon_sym_type] = ACTIONS(3748),
    [anon_sym_union] = ACTIONS(3748),
    [anon_sym_unsafe] = ACTIONS(3748),
    [anon_sym_use] = ACTIONS(3748),
    [anon_sym_while] = ACTIONS(3748),
    [anon_sym_extern] = ACTIONS(3748),
    [anon_sym_safe] = ACTIONS(3748),
    [anon_sym_yield] = ACTIONS(3748),
    [anon_sym_move] = ACTIONS(3748),
    [anon_sym_try] = ACTIONS(3748),
    [sym_integer_literal] = ACTIONS(3746),
    [aux_sym_string_literal_token1] = ACTIONS(3746),
    [sym_char_literal] = ACTIONS(3746),
    [anon_sym_true] = ACTIONS(3748),
    [anon_sym_false] = ACTIONS(3748),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3748),
    [sym_super] = ACTIONS(3748),
    [sym_crate] = ACTIONS(3748),
    [aux_sym_metavariable_token1] = ACTIONS(3746),
    [sym__raw_string_literal_start] = ACTIONS(3746),
    [sym_float_literal] = ACTIONS(3746),
    [sym__outer_block_doc_comment_start] = ACTIONS(3746),
    [sym__outer_line_doc_comment_start] = ACTIONS(3746),
  },
  [STATE(995)] = {
    [sym_line_comment] = STATE(995),
    [sym_block_comment] = STATE(995),
    [ts_builtin_sym_end] = ACTIONS(3750),
    [sym_identifier] = ACTIONS(3752),
    [anon_sym_SEMI] = ACTIONS(3750),
    [anon_sym_macro] = ACTIONS(3752),
    [anon_sym_LPAREN] = ACTIONS(3750),
    [anon_sym_LBRACE] = ACTIONS(3750),
    [anon_sym_RBRACE] = ACTIONS(3750),
    [anon_sym_macro_rules] = ACTIONS(3752),
    [anon_sym_BANG] = ACTIONS(3750),
    [anon_sym_LBRACK] = ACTIONS(3750),
    [anon_sym_STAR] = ACTIONS(3750),
    [anon_sym_u8] = ACTIONS(3752),
    [anon_sym_i8] = ACTIONS(3752),
    [anon_sym_u16] = ACTIONS(3752),
    [anon_sym_i16] = ACTIONS(3752),
    [anon_sym_u32] = ACTIONS(3752),
    [anon_sym_i32] = ACTIONS(3752),
    [anon_sym_u64] = ACTIONS(3752),
    [anon_sym_i64] = ACTIONS(3752),
    [anon_sym_u128] = ACTIONS(3752),
    [anon_sym_i128] = ACTIONS(3752),
    [anon_sym_isize] = ACTIONS(3752),
    [anon_sym_usize] = ACTIONS(3752),
    [anon_sym_f32] = ACTIONS(3752),
    [anon_sym_f64] = ACTIONS(3752),
    [anon_sym_bool] = ACTIONS(3752),
    [anon_sym_str] = ACTIONS(3752),
    [anon_sym_char] = ACTIONS(3752),
    [anon_sym_DASH] = ACTIONS(3750),
    [anon_sym_AMP] = ACTIONS(3750),
    [anon_sym_PIPE] = ACTIONS(3750),
    [anon_sym_LT] = ACTIONS(3750),
    [anon_sym_DOT_DOT] = ACTIONS(3750),
    [anon_sym_COLON_COLON] = ACTIONS(3750),
    [anon_sym_POUND] = ACTIONS(3750),
    [anon_sym_SQUOTE] = ACTIONS(3752),
    [anon_sym_async] = ACTIONS(3752),
    [anon_sym_auto] = ACTIONS(3752),
    [anon_sym_await] = ACTIONS(3752),
    [anon_sym_become] = ACTIONS(3752),
    [anon_sym_break] = ACTIONS(3752),
    [anon_sym_const] = ACTIONS(3752),
    [anon_sym_continue] = ACTIONS(3752),
    [anon_sym_default] = ACTIONS(3752),
    [anon_sym_enum] = ACTIONS(3752),
    [anon_sym_fn] = ACTIONS(3752),
    [anon_sym_for] = ACTIONS(3752),
    [anon_sym_gen] = ACTIONS(3752),
    [anon_sym_if] = ACTIONS(3752),
    [anon_sym_impl] = ACTIONS(3752),
    [anon_sym_let] = ACTIONS(3752),
    [anon_sym_loop] = ACTIONS(3752),
    [anon_sym_match] = ACTIONS(3752),
    [anon_sym_mod] = ACTIONS(3752),
    [anon_sym_pub] = ACTIONS(3752),
    [anon_sym_return] = ACTIONS(3752),
    [anon_sym_raw] = ACTIONS(3752),
    [anon_sym_static] = ACTIONS(3752),
    [anon_sym_struct] = ACTIONS(3752),
    [anon_sym_trait] = ACTIONS(3752),
    [anon_sym_type] = ACTIONS(3752),
    [anon_sym_union] = ACTIONS(3752),
    [anon_sym_unsafe] = ACTIONS(3752),
    [anon_sym_use] = ACTIONS(3752),
    [anon_sym_while] = ACTIONS(3752),
    [anon_sym_extern] = ACTIONS(3752),
    [anon_sym_safe] = ACTIONS(3752),
    [anon_sym_yield] = ACTIONS(3752),
    [anon_sym_move] = ACTIONS(3752),
    [anon_sym_try] = ACTIONS(3752),
    [sym_integer_literal] = ACTIONS(3750),
    [aux_sym_string_literal_token1] = ACTIONS(3750),
    [sym_char_literal] = ACTIONS(3750),
    [anon_sym_true] = ACTIONS(3752),
    [anon_sym_false] = ACTIONS(3752),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3752),
    [sym_super] = ACTIONS(3752),
    [sym_crate] = ACTIONS(3752),
    [aux_sym_metavariable_token1] = ACTIONS(3750),
    [sym__raw_string_literal_start] = ACTIONS(3750),
    [sym_float_literal] = ACTIONS(3750),
    [sym__outer_block_doc_comment_start] = ACTIONS(3750),
    [sym__outer_line_doc_comment_start] = ACTIONS(3750),
  },
  [STATE(996)] = {
    [sym_line_comment] = STATE(996),
    [sym_block_comment] = STATE(996),
    [ts_builtin_sym_end] = ACTIONS(3754),
    [sym_identifier] = ACTIONS(3756),
    [anon_sym_SEMI] = ACTIONS(3754),
    [anon_sym_macro] = ACTIONS(3756),
    [anon_sym_LPAREN] = ACTIONS(3754),
    [anon_sym_LBRACE] = ACTIONS(3754),
    [anon_sym_RBRACE] = ACTIONS(3754),
    [anon_sym_macro_rules] = ACTIONS(3756),
    [anon_sym_BANG] = ACTIONS(3754),
    [anon_sym_LBRACK] = ACTIONS(3754),
    [anon_sym_STAR] = ACTIONS(3754),
    [anon_sym_u8] = ACTIONS(3756),
    [anon_sym_i8] = ACTIONS(3756),
    [anon_sym_u16] = ACTIONS(3756),
    [anon_sym_i16] = ACTIONS(3756),
    [anon_sym_u32] = ACTIONS(3756),
    [anon_sym_i32] = ACTIONS(3756),
    [anon_sym_u64] = ACTIONS(3756),
    [anon_sym_i64] = ACTIONS(3756),
    [anon_sym_u128] = ACTIONS(3756),
    [anon_sym_i128] = ACTIONS(3756),
    [anon_sym_isize] = ACTIONS(3756),
    [anon_sym_usize] = ACTIONS(3756),
    [anon_sym_f32] = ACTIONS(3756),
    [anon_sym_f64] = ACTIONS(3756),
    [anon_sym_bool] = ACTIONS(3756),
    [anon_sym_str] = ACTIONS(3756),
    [anon_sym_char] = ACTIONS(3756),
    [anon_sym_DASH] = ACTIONS(3754),
    [anon_sym_AMP] = ACTIONS(3754),
    [anon_sym_PIPE] = ACTIONS(3754),
    [anon_sym_LT] = ACTIONS(3754),
    [anon_sym_DOT_DOT] = ACTIONS(3754),
    [anon_sym_COLON_COLON] = ACTIONS(3754),
    [anon_sym_POUND] = ACTIONS(3754),
    [anon_sym_SQUOTE] = ACTIONS(3756),
    [anon_sym_async] = ACTIONS(3756),
    [anon_sym_auto] = ACTIONS(3756),
    [anon_sym_await] = ACTIONS(3756),
    [anon_sym_become] = ACTIONS(3756),
    [anon_sym_break] = ACTIONS(3756),
    [anon_sym_const] = ACTIONS(3756),
    [anon_sym_continue] = ACTIONS(3756),
    [anon_sym_default] = ACTIONS(3756),
    [anon_sym_enum] = ACTIONS(3756),
    [anon_sym_fn] = ACTIONS(3756),
    [anon_sym_for] = ACTIONS(3756),
    [anon_sym_gen] = ACTIONS(3756),
    [anon_sym_if] = ACTIONS(3756),
    [anon_sym_impl] = ACTIONS(3756),
    [anon_sym_let] = ACTIONS(3756),
    [anon_sym_loop] = ACTIONS(3756),
    [anon_sym_match] = ACTIONS(3756),
    [anon_sym_mod] = ACTIONS(3756),
    [anon_sym_pub] = ACTIONS(3756),
    [anon_sym_return] = ACTIONS(3756),
    [anon_sym_raw] = ACTIONS(3756),
    [anon_sym_static] = ACTIONS(3756),
    [anon_sym_struct] = ACTIONS(3756),
    [anon_sym_trait] = ACTIONS(3756),
    [anon_sym_type] = ACTIONS(3756),
    [anon_sym_union] = ACTIONS(3756),
    [anon_sym_unsafe] = ACTIONS(3756),
    [anon_sym_use] = ACTIONS(3756),
    [anon_sym_while] = ACTIONS(3756),
    [anon_sym_extern] = ACTIONS(3756),
    [anon_sym_safe] = ACTIONS(3756),
    [anon_sym_yield] = ACTIONS(3756),
    [anon_sym_move] = ACTIONS(3756),
    [anon_sym_try] = ACTIONS(3756),
    [sym_integer_literal] = ACTIONS(3754),
    [aux_sym_string_literal_token1] = ACTIONS(3754),
    [sym_char_literal] = ACTIONS(3754),
    [anon_sym_true] = ACTIONS(3756),
    [anon_sym_false] = ACTIONS(3756),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3756),
    [sym_super] = ACTIONS(3756),
    [sym_crate] = ACTIONS(3756),
    [aux_sym_metavariable_token1] = ACTIONS(3754),
    [sym__raw_string_literal_start] = ACTIONS(3754),
    [sym_float_literal] = ACTIONS(3754),
    [sym__outer_block_doc_comment_start] = ACTIONS(3754),
    [sym__outer_line_doc_comment_start] = ACTIONS(3754),
  },
  [STATE(997)] = {
    [sym_line_comment] = STATE(997),
    [sym_block_comment] = STATE(997),
    [ts_builtin_sym_end] = ACTIONS(3758),
    [sym_identifier] = ACTIONS(3760),
    [anon_sym_SEMI] = ACTIONS(3758),
    [anon_sym_macro] = ACTIONS(3760),
    [anon_sym_LPAREN] = ACTIONS(3758),
    [anon_sym_LBRACE] = ACTIONS(3758),
    [anon_sym_RBRACE] = ACTIONS(3758),
    [anon_sym_macro_rules] = ACTIONS(3760),
    [anon_sym_BANG] = ACTIONS(3758),
    [anon_sym_LBRACK] = ACTIONS(3758),
    [anon_sym_STAR] = ACTIONS(3758),
    [anon_sym_u8] = ACTIONS(3760),
    [anon_sym_i8] = ACTIONS(3760),
    [anon_sym_u16] = ACTIONS(3760),
    [anon_sym_i16] = ACTIONS(3760),
    [anon_sym_u32] = ACTIONS(3760),
    [anon_sym_i32] = ACTIONS(3760),
    [anon_sym_u64] = ACTIONS(3760),
    [anon_sym_i64] = ACTIONS(3760),
    [anon_sym_u128] = ACTIONS(3760),
    [anon_sym_i128] = ACTIONS(3760),
    [anon_sym_isize] = ACTIONS(3760),
    [anon_sym_usize] = ACTIONS(3760),
    [anon_sym_f32] = ACTIONS(3760),
    [anon_sym_f64] = ACTIONS(3760),
    [anon_sym_bool] = ACTIONS(3760),
    [anon_sym_str] = ACTIONS(3760),
    [anon_sym_char] = ACTIONS(3760),
    [anon_sym_DASH] = ACTIONS(3758),
    [anon_sym_AMP] = ACTIONS(3758),
    [anon_sym_PIPE] = ACTIONS(3758),
    [anon_sym_LT] = ACTIONS(3758),
    [anon_sym_DOT_DOT] = ACTIONS(3758),
    [anon_sym_COLON_COLON] = ACTIONS(3758),
    [anon_sym_POUND] = ACTIONS(3758),
    [anon_sym_SQUOTE] = ACTIONS(3760),
    [anon_sym_async] = ACTIONS(3760),
    [anon_sym_auto] = ACTIONS(3760),
    [anon_sym_await] = ACTIONS(3760),
    [anon_sym_become] = ACTIONS(3760),
    [anon_sym_break] = ACTIONS(3760),
    [anon_sym_const] = ACTIONS(3760),
    [anon_sym_continue] = ACTIONS(3760),
    [anon_sym_default] = ACTIONS(3760),
    [anon_sym_enum] = ACTIONS(3760),
    [anon_sym_fn] = ACTIONS(3760),
    [anon_sym_for] = ACTIONS(3760),
    [anon_sym_gen] = ACTIONS(3760),
    [anon_sym_if] = ACTIONS(3760),
    [anon_sym_impl] = ACTIONS(3760),
    [anon_sym_let] = ACTIONS(3760),
    [anon_sym_loop] = ACTIONS(3760),
    [anon_sym_match] = ACTIONS(3760),
    [anon_sym_mod] = ACTIONS(3760),
    [anon_sym_pub] = ACTIONS(3760),
    [anon_sym_return] = ACTIONS(3760),
    [anon_sym_raw] = ACTIONS(3760),
    [anon_sym_static] = ACTIONS(3760),
    [anon_sym_struct] = ACTIONS(3760),
    [anon_sym_trait] = ACTIONS(3760),
    [anon_sym_type] = ACTIONS(3760),
    [anon_sym_union] = ACTIONS(3760),
    [anon_sym_unsafe] = ACTIONS(3760),
    [anon_sym_use] = ACTIONS(3760),
    [anon_sym_while] = ACTIONS(3760),
    [anon_sym_extern] = ACTIONS(3760),
    [anon_sym_safe] = ACTIONS(3760),
    [anon_sym_yield] = ACTIONS(3760),
    [anon_sym_move] = ACTIONS(3760),
    [anon_sym_try] = ACTIONS(3760),
    [sym_integer_literal] = ACTIONS(3758),
    [aux_sym_string_literal_token1] = ACTIONS(3758),
    [sym_char_literal] = ACTIONS(3758),
    [anon_sym_true] = ACTIONS(3760),
    [anon_sym_false] = ACTIONS(3760),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3760),
    [sym_super] = ACTIONS(3760),
    [sym_crate] = ACTIONS(3760),
    [aux_sym_metavariable_token1] = ACTIONS(3758),
    [sym__raw_string_literal_start] = ACTIONS(3758),
    [sym_float_literal] = ACTIONS(3758),
    [sym__outer_block_doc_comment_start] = ACTIONS(3758),
    [sym__outer_line_doc_comment_start] = ACTIONS(3758),
  },
  [STATE(998)] = {
    [sym_line_comment] = STATE(998),
    [sym_block_comment] = STATE(998),
    [ts_builtin_sym_end] = ACTIONS(3762),
    [sym_identifier] = ACTIONS(3764),
    [anon_sym_SEMI] = ACTIONS(3762),
    [anon_sym_macro] = ACTIONS(3764),
    [anon_sym_LPAREN] = ACTIONS(3762),
    [anon_sym_LBRACE] = ACTIONS(3762),
    [anon_sym_RBRACE] = ACTIONS(3762),
    [anon_sym_macro_rules] = ACTIONS(3764),
    [anon_sym_BANG] = ACTIONS(3762),
    [anon_sym_LBRACK] = ACTIONS(3762),
    [anon_sym_STAR] = ACTIONS(3762),
    [anon_sym_u8] = ACTIONS(3764),
    [anon_sym_i8] = ACTIONS(3764),
    [anon_sym_u16] = ACTIONS(3764),
    [anon_sym_i16] = ACTIONS(3764),
    [anon_sym_u32] = ACTIONS(3764),
    [anon_sym_i32] = ACTIONS(3764),
    [anon_sym_u64] = ACTIONS(3764),
    [anon_sym_i64] = ACTIONS(3764),
    [anon_sym_u128] = ACTIONS(3764),
    [anon_sym_i128] = ACTIONS(3764),
    [anon_sym_isize] = ACTIONS(3764),
    [anon_sym_usize] = ACTIONS(3764),
    [anon_sym_f32] = ACTIONS(3764),
    [anon_sym_f64] = ACTIONS(3764),
    [anon_sym_bool] = ACTIONS(3764),
    [anon_sym_str] = ACTIONS(3764),
    [anon_sym_char] = ACTIONS(3764),
    [anon_sym_DASH] = ACTIONS(3762),
    [anon_sym_AMP] = ACTIONS(3762),
    [anon_sym_PIPE] = ACTIONS(3762),
    [anon_sym_LT] = ACTIONS(3762),
    [anon_sym_DOT_DOT] = ACTIONS(3762),
    [anon_sym_COLON_COLON] = ACTIONS(3762),
    [anon_sym_POUND] = ACTIONS(3762),
    [anon_sym_SQUOTE] = ACTIONS(3764),
    [anon_sym_async] = ACTIONS(3764),
    [anon_sym_auto] = ACTIONS(3764),
    [anon_sym_await] = ACTIONS(3764),
    [anon_sym_become] = ACTIONS(3764),
    [anon_sym_break] = ACTIONS(3764),
    [anon_sym_const] = ACTIONS(3764),
    [anon_sym_continue] = ACTIONS(3764),
    [anon_sym_default] = ACTIONS(3764),
    [anon_sym_enum] = ACTIONS(3764),
    [anon_sym_fn] = ACTIONS(3764),
    [anon_sym_for] = ACTIONS(3764),
    [anon_sym_gen] = ACTIONS(3764),
    [anon_sym_if] = ACTIONS(3764),
    [anon_sym_impl] = ACTIONS(3764),
    [anon_sym_let] = ACTIONS(3764),
    [anon_sym_loop] = ACTIONS(3764),
    [anon_sym_match] = ACTIONS(3764),
    [anon_sym_mod] = ACTIONS(3764),
    [anon_sym_pub] = ACTIONS(3764),
    [anon_sym_return] = ACTIONS(3764),
    [anon_sym_raw] = ACTIONS(3764),
    [anon_sym_static] = ACTIONS(3764),
    [anon_sym_struct] = ACTIONS(3764),
    [anon_sym_trait] = ACTIONS(3764),
    [anon_sym_type] = ACTIONS(3764),
    [anon_sym_union] = ACTIONS(3764),
    [anon_sym_unsafe] = ACTIONS(3764),
    [anon_sym_use] = ACTIONS(3764),
    [anon_sym_while] = ACTIONS(3764),
    [anon_sym_extern] = ACTIONS(3764),
    [anon_sym_safe] = ACTIONS(3764),
    [anon_sym_yield] = ACTIONS(3764),
    [anon_sym_move] = ACTIONS(3764),
    [anon_sym_try] = ACTIONS(3764),
    [sym_integer_literal] = ACTIONS(3762),
    [aux_sym_string_literal_token1] = ACTIONS(3762),
    [sym_char_literal] = ACTIONS(3762),
    [anon_sym_true] = ACTIONS(3764),
    [anon_sym_false] = ACTIONS(3764),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3764),
    [sym_super] = ACTIONS(3764),
    [sym_crate] = ACTIONS(3764),
    [aux_sym_metavariable_token1] = ACTIONS(3762),
    [sym__raw_string_literal_start] = ACTIONS(3762),
    [sym_float_literal] = ACTIONS(3762),
    [sym__outer_block_doc_comment_start] = ACTIONS(3762),
    [sym__outer_line_doc_comment_start] = ACTIONS(3762),
  },
  [STATE(999)] = {
    [sym_line_comment] = STATE(999),
    [sym_block_comment] = STATE(999),
    [ts_builtin_sym_end] = ACTIONS(3766),
    [sym_identifier] = ACTIONS(3768),
    [anon_sym_SEMI] = ACTIONS(3766),
    [anon_sym_macro] = ACTIONS(3768),
    [anon_sym_LPAREN] = ACTIONS(3766),
    [anon_sym_LBRACE] = ACTIONS(3766),
    [anon_sym_RBRACE] = ACTIONS(3766),
    [anon_sym_macro_rules] = ACTIONS(3768),
    [anon_sym_BANG] = ACTIONS(3766),
    [anon_sym_LBRACK] = ACTIONS(3766),
    [anon_sym_STAR] = ACTIONS(3766),
    [anon_sym_u8] = ACTIONS(3768),
    [anon_sym_i8] = ACTIONS(3768),
    [anon_sym_u16] = ACTIONS(3768),
    [anon_sym_i16] = ACTIONS(3768),
    [anon_sym_u32] = ACTIONS(3768),
    [anon_sym_i32] = ACTIONS(3768),
    [anon_sym_u64] = ACTIONS(3768),
    [anon_sym_i64] = ACTIONS(3768),
    [anon_sym_u128] = ACTIONS(3768),
    [anon_sym_i128] = ACTIONS(3768),
    [anon_sym_isize] = ACTIONS(3768),
    [anon_sym_usize] = ACTIONS(3768),
    [anon_sym_f32] = ACTIONS(3768),
    [anon_sym_f64] = ACTIONS(3768),
    [anon_sym_bool] = ACTIONS(3768),
    [anon_sym_str] = ACTIONS(3768),
    [anon_sym_char] = ACTIONS(3768),
    [anon_sym_DASH] = ACTIONS(3766),
    [anon_sym_AMP] = ACTIONS(3766),
    [anon_sym_PIPE] = ACTIONS(3766),
    [anon_sym_LT] = ACTIONS(3766),
    [anon_sym_DOT_DOT] = ACTIONS(3766),
    [anon_sym_COLON_COLON] = ACTIONS(3766),
    [anon_sym_POUND] = ACTIONS(3766),
    [anon_sym_SQUOTE] = ACTIONS(3768),
    [anon_sym_async] = ACTIONS(3768),
    [anon_sym_auto] = ACTIONS(3768),
    [anon_sym_await] = ACTIONS(3768),
    [anon_sym_become] = ACTIONS(3768),
    [anon_sym_break] = ACTIONS(3768),
    [anon_sym_const] = ACTIONS(3768),
    [anon_sym_continue] = ACTIONS(3768),
    [anon_sym_default] = ACTIONS(3768),
    [anon_sym_enum] = ACTIONS(3768),
    [anon_sym_fn] = ACTIONS(3768),
    [anon_sym_for] = ACTIONS(3768),
    [anon_sym_gen] = ACTIONS(3768),
    [anon_sym_if] = ACTIONS(3768),
    [anon_sym_impl] = ACTIONS(3768),
    [anon_sym_let] = ACTIONS(3768),
    [anon_sym_loop] = ACTIONS(3768),
    [anon_sym_match] = ACTIONS(3768),
    [anon_sym_mod] = ACTIONS(3768),
    [anon_sym_pub] = ACTIONS(3768),
    [anon_sym_return] = ACTIONS(3768),
    [anon_sym_raw] = ACTIONS(3768),
    [anon_sym_static] = ACTIONS(3768),
    [anon_sym_struct] = ACTIONS(3768),
    [anon_sym_trait] = ACTIONS(3768),
    [anon_sym_type] = ACTIONS(3768),
    [anon_sym_union] = ACTIONS(3768),
    [anon_sym_unsafe] = ACTIONS(3768),
    [anon_sym_use] = ACTIONS(3768),
    [anon_sym_while] = ACTIONS(3768),
    [anon_sym_extern] = ACTIONS(3768),
    [anon_sym_safe] = ACTIONS(3768),
    [anon_sym_yield] = ACTIONS(3768),
    [anon_sym_move] = ACTIONS(3768),
    [anon_sym_try] = ACTIONS(3768),
    [sym_integer_literal] = ACTIONS(3766),
    [aux_sym_string_literal_token1] = ACTIONS(3766),
    [sym_char_literal] = ACTIONS(3766),
    [anon_sym_true] = ACTIONS(3768),
    [anon_sym_false] = ACTIONS(3768),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3768),
    [sym_super] = ACTIONS(3768),
    [sym_crate] = ACTIONS(3768),
    [aux_sym_metavariable_token1] = ACTIONS(3766),
    [sym__raw_string_literal_start] = ACTIONS(3766),
    [sym_float_literal] = ACTIONS(3766),
    [sym__outer_block_doc_comment_start] = ACTIONS(3766),
    [sym__outer_line_doc_comment_start] = ACTIONS(3766),
  },
  [STATE(1000)] = {
    [sym_line_comment] = STATE(1000),
    [sym_block_comment] = STATE(1000),
    [ts_builtin_sym_end] = ACTIONS(3770),
    [sym_identifier] = ACTIONS(3772),
    [anon_sym_SEMI] = ACTIONS(3770),
    [anon_sym_macro] = ACTIONS(3772),
    [anon_sym_LPAREN] = ACTIONS(3770),
    [anon_sym_LBRACE] = ACTIONS(3770),
    [anon_sym_RBRACE] = ACTIONS(3770),
    [anon_sym_macro_rules] = ACTIONS(3772),
    [anon_sym_BANG] = ACTIONS(3770),
    [anon_sym_LBRACK] = ACTIONS(3770),
    [anon_sym_STAR] = ACTIONS(3770),
    [anon_sym_u8] = ACTIONS(3772),
    [anon_sym_i8] = ACTIONS(3772),
    [anon_sym_u16] = ACTIONS(3772),
    [anon_sym_i16] = ACTIONS(3772),
    [anon_sym_u32] = ACTIONS(3772),
    [anon_sym_i32] = ACTIONS(3772),
    [anon_sym_u64] = ACTIONS(3772),
    [anon_sym_i64] = ACTIONS(3772),
    [anon_sym_u128] = ACTIONS(3772),
    [anon_sym_i128] = ACTIONS(3772),
    [anon_sym_isize] = ACTIONS(3772),
    [anon_sym_usize] = ACTIONS(3772),
    [anon_sym_f32] = ACTIONS(3772),
    [anon_sym_f64] = ACTIONS(3772),
    [anon_sym_bool] = ACTIONS(3772),
    [anon_sym_str] = ACTIONS(3772),
    [anon_sym_char] = ACTIONS(3772),
    [anon_sym_DASH] = ACTIONS(3770),
    [anon_sym_AMP] = ACTIONS(3770),
    [anon_sym_PIPE] = ACTIONS(3770),
    [anon_sym_LT] = ACTIONS(3770),
    [anon_sym_DOT_DOT] = ACTIONS(3770),
    [anon_sym_COLON_COLON] = ACTIONS(3770),
    [anon_sym_POUND] = ACTIONS(3770),
    [anon_sym_SQUOTE] = ACTIONS(3772),
    [anon_sym_async] = ACTIONS(3772),
    [anon_sym_auto] = ACTIONS(3772),
    [anon_sym_await] = ACTIONS(3772),
    [anon_sym_become] = ACTIONS(3772),
    [anon_sym_break] = ACTIONS(3772),
    [anon_sym_const] = ACTIONS(3772),
    [anon_sym_continue] = ACTIONS(3772),
    [anon_sym_default] = ACTIONS(3772),
    [anon_sym_enum] = ACTIONS(3772),
    [anon_sym_fn] = ACTIONS(3772),
    [anon_sym_for] = ACTIONS(3772),
    [anon_sym_gen] = ACTIONS(3772),
    [anon_sym_if] = ACTIONS(3772),
    [anon_sym_impl] = ACTIONS(3772),
    [anon_sym_let] = ACTIONS(3772),
    [anon_sym_loop] = ACTIONS(3772),
    [anon_sym_match] = ACTIONS(3772),
    [anon_sym_mod] = ACTIONS(3772),
    [anon_sym_pub] = ACTIONS(3772),
    [anon_sym_return] = ACTIONS(3772),
    [anon_sym_raw] = ACTIONS(3772),
    [anon_sym_static] = ACTIONS(3772),
    [anon_sym_struct] = ACTIONS(3772),
    [anon_sym_trait] = ACTIONS(3772),
    [anon_sym_type] = ACTIONS(3772),
    [anon_sym_union] = ACTIONS(3772),
    [anon_sym_unsafe] = ACTIONS(3772),
    [anon_sym_use] = ACTIONS(3772),
    [anon_sym_while] = ACTIONS(3772),
    [anon_sym_extern] = ACTIONS(3772),
    [anon_sym_safe] = ACTIONS(3772),
    [anon_sym_yield] = ACTIONS(3772),
    [anon_sym_move] = ACTIONS(3772),
    [anon_sym_try] = ACTIONS(3772),
    [sym_integer_literal] = ACTIONS(3770),
    [aux_sym_string_literal_token1] = ACTIONS(3770),
    [sym_char_literal] = ACTIONS(3770),
    [anon_sym_true] = ACTIONS(3772),
    [anon_sym_false] = ACTIONS(3772),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3772),
    [sym_super] = ACTIONS(3772),
    [sym_crate] = ACTIONS(3772),
    [aux_sym_metavariable_token1] = ACTIONS(3770),
    [sym__raw_string_literal_start] = ACTIONS(3770),
    [sym_float_literal] = ACTIONS(3770),
    [sym__outer_block_doc_comment_start] = ACTIONS(3770),
    [sym__outer_line_doc_comment_start] = ACTIONS(3770),
  },
  [STATE(1001)] = {
    [sym_line_comment] = STATE(1001),
    [sym_block_comment] = STATE(1001),
    [ts_builtin_sym_end] = ACTIONS(3774),
    [sym_identifier] = ACTIONS(3776),
    [anon_sym_SEMI] = ACTIONS(3774),
    [anon_sym_macro] = ACTIONS(3776),
    [anon_sym_LPAREN] = ACTIONS(3774),
    [anon_sym_LBRACE] = ACTIONS(3774),
    [anon_sym_RBRACE] = ACTIONS(3774),
    [anon_sym_macro_rules] = ACTIONS(3776),
    [anon_sym_BANG] = ACTIONS(3774),
    [anon_sym_LBRACK] = ACTIONS(3774),
    [anon_sym_STAR] = ACTIONS(3774),
    [anon_sym_u8] = ACTIONS(3776),
    [anon_sym_i8] = ACTIONS(3776),
    [anon_sym_u16] = ACTIONS(3776),
    [anon_sym_i16] = ACTIONS(3776),
    [anon_sym_u32] = ACTIONS(3776),
    [anon_sym_i32] = ACTIONS(3776),
    [anon_sym_u64] = ACTIONS(3776),
    [anon_sym_i64] = ACTIONS(3776),
    [anon_sym_u128] = ACTIONS(3776),
    [anon_sym_i128] = ACTIONS(3776),
    [anon_sym_isize] = ACTIONS(3776),
    [anon_sym_usize] = ACTIONS(3776),
    [anon_sym_f32] = ACTIONS(3776),
    [anon_sym_f64] = ACTIONS(3776),
    [anon_sym_bool] = ACTIONS(3776),
    [anon_sym_str] = ACTIONS(3776),
    [anon_sym_char] = ACTIONS(3776),
    [anon_sym_DASH] = ACTIONS(3774),
    [anon_sym_AMP] = ACTIONS(3774),
    [anon_sym_PIPE] = ACTIONS(3774),
    [anon_sym_LT] = ACTIONS(3774),
    [anon_sym_DOT_DOT] = ACTIONS(3774),
    [anon_sym_COLON_COLON] = ACTIONS(3774),
    [anon_sym_POUND] = ACTIONS(3774),
    [anon_sym_SQUOTE] = ACTIONS(3776),
    [anon_sym_async] = ACTIONS(3776),
    [anon_sym_auto] = ACTIONS(3776),
    [anon_sym_await] = ACTIONS(3776),
    [anon_sym_become] = ACTIONS(3776),
    [anon_sym_break] = ACTIONS(3776),
    [anon_sym_const] = ACTIONS(3776),
    [anon_sym_continue] = ACTIONS(3776),
    [anon_sym_default] = ACTIONS(3776),
    [anon_sym_enum] = ACTIONS(3776),
    [anon_sym_fn] = ACTIONS(3776),
    [anon_sym_for] = ACTIONS(3776),
    [anon_sym_gen] = ACTIONS(3776),
    [anon_sym_if] = ACTIONS(3776),
    [anon_sym_impl] = ACTIONS(3776),
    [anon_sym_let] = ACTIONS(3776),
    [anon_sym_loop] = ACTIONS(3776),
    [anon_sym_match] = ACTIONS(3776),
    [anon_sym_mod] = ACTIONS(3776),
    [anon_sym_pub] = ACTIONS(3776),
    [anon_sym_return] = ACTIONS(3776),
    [anon_sym_raw] = ACTIONS(3776),
    [anon_sym_static] = ACTIONS(3776),
    [anon_sym_struct] = ACTIONS(3776),
    [anon_sym_trait] = ACTIONS(3776),
    [anon_sym_type] = ACTIONS(3776),
    [anon_sym_union] = ACTIONS(3776),
    [anon_sym_unsafe] = ACTIONS(3776),
    [anon_sym_use] = ACTIONS(3776),
    [anon_sym_while] = ACTIONS(3776),
    [anon_sym_extern] = ACTIONS(3776),
    [anon_sym_safe] = ACTIONS(3776),
    [anon_sym_yield] = ACTIONS(3776),
    [anon_sym_move] = ACTIONS(3776),
    [anon_sym_try] = ACTIONS(3776),
    [sym_integer_literal] = ACTIONS(3774),
    [aux_sym_string_literal_token1] = ACTIONS(3774),
    [sym_char_literal] = ACTIONS(3774),
    [anon_sym_true] = ACTIONS(3776),
    [anon_sym_false] = ACTIONS(3776),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3776),
    [sym_super] = ACTIONS(3776),
    [sym_crate] = ACTIONS(3776),
    [aux_sym_metavariable_token1] = ACTIONS(3774),
    [sym__raw_string_literal_start] = ACTIONS(3774),
    [sym_float_literal] = ACTIONS(3774),
    [sym__outer_block_doc_comment_start] = ACTIONS(3774),
    [sym__outer_line_doc_comment_start] = ACTIONS(3774),
  },
  [STATE(1002)] = {
    [sym_line_comment] = STATE(1002),
    [sym_block_comment] = STATE(1002),
    [ts_builtin_sym_end] = ACTIONS(3778),
    [sym_identifier] = ACTIONS(3780),
    [anon_sym_SEMI] = ACTIONS(3778),
    [anon_sym_macro] = ACTIONS(3780),
    [anon_sym_LPAREN] = ACTIONS(3778),
    [anon_sym_LBRACE] = ACTIONS(3778),
    [anon_sym_RBRACE] = ACTIONS(3778),
    [anon_sym_macro_rules] = ACTIONS(3780),
    [anon_sym_BANG] = ACTIONS(3778),
    [anon_sym_LBRACK] = ACTIONS(3778),
    [anon_sym_STAR] = ACTIONS(3778),
    [anon_sym_u8] = ACTIONS(3780),
    [anon_sym_i8] = ACTIONS(3780),
    [anon_sym_u16] = ACTIONS(3780),
    [anon_sym_i16] = ACTIONS(3780),
    [anon_sym_u32] = ACTIONS(3780),
    [anon_sym_i32] = ACTIONS(3780),
    [anon_sym_u64] = ACTIONS(3780),
    [anon_sym_i64] = ACTIONS(3780),
    [anon_sym_u128] = ACTIONS(3780),
    [anon_sym_i128] = ACTIONS(3780),
    [anon_sym_isize] = ACTIONS(3780),
    [anon_sym_usize] = ACTIONS(3780),
    [anon_sym_f32] = ACTIONS(3780),
    [anon_sym_f64] = ACTIONS(3780),
    [anon_sym_bool] = ACTIONS(3780),
    [anon_sym_str] = ACTIONS(3780),
    [anon_sym_char] = ACTIONS(3780),
    [anon_sym_DASH] = ACTIONS(3778),
    [anon_sym_AMP] = ACTIONS(3778),
    [anon_sym_PIPE] = ACTIONS(3778),
    [anon_sym_LT] = ACTIONS(3778),
    [anon_sym_DOT_DOT] = ACTIONS(3778),
    [anon_sym_COLON_COLON] = ACTIONS(3778),
    [anon_sym_POUND] = ACTIONS(3778),
    [anon_sym_SQUOTE] = ACTIONS(3780),
    [anon_sym_async] = ACTIONS(3780),
    [anon_sym_auto] = ACTIONS(3780),
    [anon_sym_await] = ACTIONS(3780),
    [anon_sym_become] = ACTIONS(3780),
    [anon_sym_break] = ACTIONS(3780),
    [anon_sym_const] = ACTIONS(3780),
    [anon_sym_continue] = ACTIONS(3780),
    [anon_sym_default] = ACTIONS(3780),
    [anon_sym_enum] = ACTIONS(3780),
    [anon_sym_fn] = ACTIONS(3780),
    [anon_sym_for] = ACTIONS(3780),
    [anon_sym_gen] = ACTIONS(3780),
    [anon_sym_if] = ACTIONS(3780),
    [anon_sym_impl] = ACTIONS(3780),
    [anon_sym_let] = ACTIONS(3780),
    [anon_sym_loop] = ACTIONS(3780),
    [anon_sym_match] = ACTIONS(3780),
    [anon_sym_mod] = ACTIONS(3780),
    [anon_sym_pub] = ACTIONS(3780),
    [anon_sym_return] = ACTIONS(3780),
    [anon_sym_raw] = ACTIONS(3780),
    [anon_sym_static] = ACTIONS(3780),
    [anon_sym_struct] = ACTIONS(3780),
    [anon_sym_trait] = ACTIONS(3780),
    [anon_sym_type] = ACTIONS(3780),
    [anon_sym_union] = ACTIONS(3780),
    [anon_sym_unsafe] = ACTIONS(3780),
    [anon_sym_use] = ACTIONS(3780),
    [anon_sym_while] = ACTIONS(3780),
    [anon_sym_extern] = ACTIONS(3780),
    [anon_sym_safe] = ACTIONS(3780),
    [anon_sym_yield] = ACTIONS(3780),
    [anon_sym_move] = ACTIONS(3780),
    [anon_sym_try] = ACTIONS(3780),
    [sym_integer_literal] = ACTIONS(3778),
    [aux_sym_string_literal_token1] = ACTIONS(3778),
    [sym_char_literal] = ACTIONS(3778),
    [anon_sym_true] = ACTIONS(3780),
    [anon_sym_false] = ACTIONS(3780),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3780),
    [sym_super] = ACTIONS(3780),
    [sym_crate] = ACTIONS(3780),
    [aux_sym_metavariable_token1] = ACTIONS(3778),
    [sym__raw_string_literal_start] = ACTIONS(3778),
    [sym_float_literal] = ACTIONS(3778),
    [sym__outer_block_doc_comment_start] = ACTIONS(3778),
    [sym__outer_line_doc_comment_start] = ACTIONS(3778),
  },
  [STATE(1003)] = {
    [sym_line_comment] = STATE(1003),
    [sym_block_comment] = STATE(1003),
    [ts_builtin_sym_end] = ACTIONS(3782),
    [sym_identifier] = ACTIONS(3784),
    [anon_sym_SEMI] = ACTIONS(3782),
    [anon_sym_macro] = ACTIONS(3784),
    [anon_sym_LPAREN] = ACTIONS(3782),
    [anon_sym_LBRACE] = ACTIONS(3782),
    [anon_sym_RBRACE] = ACTIONS(3782),
    [anon_sym_macro_rules] = ACTIONS(3784),
    [anon_sym_BANG] = ACTIONS(3782),
    [anon_sym_LBRACK] = ACTIONS(3782),
    [anon_sym_STAR] = ACTIONS(3782),
    [anon_sym_u8] = ACTIONS(3784),
    [anon_sym_i8] = ACTIONS(3784),
    [anon_sym_u16] = ACTIONS(3784),
    [anon_sym_i16] = ACTIONS(3784),
    [anon_sym_u32] = ACTIONS(3784),
    [anon_sym_i32] = ACTIONS(3784),
    [anon_sym_u64] = ACTIONS(3784),
    [anon_sym_i64] = ACTIONS(3784),
    [anon_sym_u128] = ACTIONS(3784),
    [anon_sym_i128] = ACTIONS(3784),
    [anon_sym_isize] = ACTIONS(3784),
    [anon_sym_usize] = ACTIONS(3784),
    [anon_sym_f32] = ACTIONS(3784),
    [anon_sym_f64] = ACTIONS(3784),
    [anon_sym_bool] = ACTIONS(3784),
    [anon_sym_str] = ACTIONS(3784),
    [anon_sym_char] = ACTIONS(3784),
    [anon_sym_DASH] = ACTIONS(3782),
    [anon_sym_AMP] = ACTIONS(3782),
    [anon_sym_PIPE] = ACTIONS(3782),
    [anon_sym_LT] = ACTIONS(3782),
    [anon_sym_DOT_DOT] = ACTIONS(3782),
    [anon_sym_COLON_COLON] = ACTIONS(3782),
    [anon_sym_POUND] = ACTIONS(3782),
    [anon_sym_SQUOTE] = ACTIONS(3784),
    [anon_sym_async] = ACTIONS(3784),
    [anon_sym_auto] = ACTIONS(3784),
    [anon_sym_await] = ACTIONS(3784),
    [anon_sym_become] = ACTIONS(3784),
    [anon_sym_break] = ACTIONS(3784),
    [anon_sym_const] = ACTIONS(3784),
    [anon_sym_continue] = ACTIONS(3784),
    [anon_sym_default] = ACTIONS(3784),
    [anon_sym_enum] = ACTIONS(3784),
    [anon_sym_fn] = ACTIONS(3784),
    [anon_sym_for] = ACTIONS(3784),
    [anon_sym_gen] = ACTIONS(3784),
    [anon_sym_if] = ACTIONS(3784),
    [anon_sym_impl] = ACTIONS(3784),
    [anon_sym_let] = ACTIONS(3784),
    [anon_sym_loop] = ACTIONS(3784),
    [anon_sym_match] = ACTIONS(3784),
    [anon_sym_mod] = ACTIONS(3784),
    [anon_sym_pub] = ACTIONS(3784),
    [anon_sym_return] = ACTIONS(3784),
    [anon_sym_raw] = ACTIONS(3784),
    [anon_sym_static] = ACTIONS(3784),
    [anon_sym_struct] = ACTIONS(3784),
    [anon_sym_trait] = ACTIONS(3784),
    [anon_sym_type] = ACTIONS(3784),
    [anon_sym_union] = ACTIONS(3784),
    [anon_sym_unsafe] = ACTIONS(3784),
    [anon_sym_use] = ACTIONS(3784),
    [anon_sym_while] = ACTIONS(3784),
    [anon_sym_extern] = ACTIONS(3784),
    [anon_sym_safe] = ACTIONS(3784),
    [anon_sym_yield] = ACTIONS(3784),
    [anon_sym_move] = ACTIONS(3784),
    [anon_sym_try] = ACTIONS(3784),
    [sym_integer_literal] = ACTIONS(3782),
    [aux_sym_string_literal_token1] = ACTIONS(3782),
    [sym_char_literal] = ACTIONS(3782),
    [anon_sym_true] = ACTIONS(3784),
    [anon_sym_false] = ACTIONS(3784),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3784),
    [sym_super] = ACTIONS(3784),
    [sym_crate] = ACTIONS(3784),
    [aux_sym_metavariable_token1] = ACTIONS(3782),
    [sym__raw_string_literal_start] = ACTIONS(3782),
    [sym_float_literal] = ACTIONS(3782),
    [sym__outer_block_doc_comment_start] = ACTIONS(3782),
    [sym__outer_line_doc_comment_start] = ACTIONS(3782),
  },
  [STATE(1004)] = {
    [sym_line_comment] = STATE(1004),
    [sym_block_comment] = STATE(1004),
    [ts_builtin_sym_end] = ACTIONS(3786),
    [sym_identifier] = ACTIONS(3788),
    [anon_sym_SEMI] = ACTIONS(3786),
    [anon_sym_macro] = ACTIONS(3788),
    [anon_sym_LPAREN] = ACTIONS(3786),
    [anon_sym_LBRACE] = ACTIONS(3786),
    [anon_sym_RBRACE] = ACTIONS(3786),
    [anon_sym_macro_rules] = ACTIONS(3788),
    [anon_sym_BANG] = ACTIONS(3786),
    [anon_sym_LBRACK] = ACTIONS(3786),
    [anon_sym_STAR] = ACTIONS(3786),
    [anon_sym_u8] = ACTIONS(3788),
    [anon_sym_i8] = ACTIONS(3788),
    [anon_sym_u16] = ACTIONS(3788),
    [anon_sym_i16] = ACTIONS(3788),
    [anon_sym_u32] = ACTIONS(3788),
    [anon_sym_i32] = ACTIONS(3788),
    [anon_sym_u64] = ACTIONS(3788),
    [anon_sym_i64] = ACTIONS(3788),
    [anon_sym_u128] = ACTIONS(3788),
    [anon_sym_i128] = ACTIONS(3788),
    [anon_sym_isize] = ACTIONS(3788),
    [anon_sym_usize] = ACTIONS(3788),
    [anon_sym_f32] = ACTIONS(3788),
    [anon_sym_f64] = ACTIONS(3788),
    [anon_sym_bool] = ACTIONS(3788),
    [anon_sym_str] = ACTIONS(3788),
    [anon_sym_char] = ACTIONS(3788),
    [anon_sym_DASH] = ACTIONS(3786),
    [anon_sym_AMP] = ACTIONS(3786),
    [anon_sym_PIPE] = ACTIONS(3786),
    [anon_sym_LT] = ACTIONS(3786),
    [anon_sym_DOT_DOT] = ACTIONS(3786),
    [anon_sym_COLON_COLON] = ACTIONS(3786),
    [anon_sym_POUND] = ACTIONS(3786),
    [anon_sym_SQUOTE] = ACTIONS(3788),
    [anon_sym_async] = ACTIONS(3788),
    [anon_sym_auto] = ACTIONS(3788),
    [anon_sym_await] = ACTIONS(3788),
    [anon_sym_become] = ACTIONS(3788),
    [anon_sym_break] = ACTIONS(3788),
    [anon_sym_const] = ACTIONS(3788),
    [anon_sym_continue] = ACTIONS(3788),
    [anon_sym_default] = ACTIONS(3788),
    [anon_sym_enum] = ACTIONS(3788),
    [anon_sym_fn] = ACTIONS(3788),
    [anon_sym_for] = ACTIONS(3788),
    [anon_sym_gen] = ACTIONS(3788),
    [anon_sym_if] = ACTIONS(3788),
    [anon_sym_impl] = ACTIONS(3788),
    [anon_sym_let] = ACTIONS(3788),
    [anon_sym_loop] = ACTIONS(3788),
    [anon_sym_match] = ACTIONS(3788),
    [anon_sym_mod] = ACTIONS(3788),
    [anon_sym_pub] = ACTIONS(3788),
    [anon_sym_return] = ACTIONS(3788),
    [anon_sym_raw] = ACTIONS(3788),
    [anon_sym_static] = ACTIONS(3788),
    [anon_sym_struct] = ACTIONS(3788),
    [anon_sym_trait] = ACTIONS(3788),
    [anon_sym_type] = ACTIONS(3788),
    [anon_sym_union] = ACTIONS(3788),
    [anon_sym_unsafe] = ACTIONS(3788),
    [anon_sym_use] = ACTIONS(3788),
    [anon_sym_while] = ACTIONS(3788),
    [anon_sym_extern] = ACTIONS(3788),
    [anon_sym_safe] = ACTIONS(3788),
    [anon_sym_yield] = ACTIONS(3788),
    [anon_sym_move] = ACTIONS(3788),
    [anon_sym_try] = ACTIONS(3788),
    [sym_integer_literal] = ACTIONS(3786),
    [aux_sym_string_literal_token1] = ACTIONS(3786),
    [sym_char_literal] = ACTIONS(3786),
    [anon_sym_true] = ACTIONS(3788),
    [anon_sym_false] = ACTIONS(3788),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3788),
    [sym_super] = ACTIONS(3788),
    [sym_crate] = ACTIONS(3788),
    [aux_sym_metavariable_token1] = ACTIONS(3786),
    [sym__raw_string_literal_start] = ACTIONS(3786),
    [sym_float_literal] = ACTIONS(3786),
    [sym__outer_block_doc_comment_start] = ACTIONS(3786),
    [sym__outer_line_doc_comment_start] = ACTIONS(3786),
  },
  [STATE(1005)] = {
    [sym_line_comment] = STATE(1005),
    [sym_block_comment] = STATE(1005),
    [ts_builtin_sym_end] = ACTIONS(3790),
    [sym_identifier] = ACTIONS(3792),
    [anon_sym_SEMI] = ACTIONS(3790),
    [anon_sym_macro] = ACTIONS(3792),
    [anon_sym_LPAREN] = ACTIONS(3790),
    [anon_sym_LBRACE] = ACTIONS(3790),
    [anon_sym_RBRACE] = ACTIONS(3790),
    [anon_sym_macro_rules] = ACTIONS(3792),
    [anon_sym_BANG] = ACTIONS(3790),
    [anon_sym_LBRACK] = ACTIONS(3790),
    [anon_sym_STAR] = ACTIONS(3790),
    [anon_sym_u8] = ACTIONS(3792),
    [anon_sym_i8] = ACTIONS(3792),
    [anon_sym_u16] = ACTIONS(3792),
    [anon_sym_i16] = ACTIONS(3792),
    [anon_sym_u32] = ACTIONS(3792),
    [anon_sym_i32] = ACTIONS(3792),
    [anon_sym_u64] = ACTIONS(3792),
    [anon_sym_i64] = ACTIONS(3792),
    [anon_sym_u128] = ACTIONS(3792),
    [anon_sym_i128] = ACTIONS(3792),
    [anon_sym_isize] = ACTIONS(3792),
    [anon_sym_usize] = ACTIONS(3792),
    [anon_sym_f32] = ACTIONS(3792),
    [anon_sym_f64] = ACTIONS(3792),
    [anon_sym_bool] = ACTIONS(3792),
    [anon_sym_str] = ACTIONS(3792),
    [anon_sym_char] = ACTIONS(3792),
    [anon_sym_DASH] = ACTIONS(3790),
    [anon_sym_AMP] = ACTIONS(3790),
    [anon_sym_PIPE] = ACTIONS(3790),
    [anon_sym_LT] = ACTIONS(3790),
    [anon_sym_DOT_DOT] = ACTIONS(3790),
    [anon_sym_COLON_COLON] = ACTIONS(3790),
    [anon_sym_POUND] = ACTIONS(3790),
    [anon_sym_SQUOTE] = ACTIONS(3792),
    [anon_sym_async] = ACTIONS(3792),
    [anon_sym_auto] = ACTIONS(3792),
    [anon_sym_await] = ACTIONS(3792),
    [anon_sym_become] = ACTIONS(3792),
    [anon_sym_break] = ACTIONS(3792),
    [anon_sym_const] = ACTIONS(3792),
    [anon_sym_continue] = ACTIONS(3792),
    [anon_sym_default] = ACTIONS(3792),
    [anon_sym_enum] = ACTIONS(3792),
    [anon_sym_fn] = ACTIONS(3792),
    [anon_sym_for] = ACTIONS(3792),
    [anon_sym_gen] = ACTIONS(3792),
    [anon_sym_if] = ACTIONS(3792),
    [anon_sym_impl] = ACTIONS(3792),
    [anon_sym_let] = ACTIONS(3792),
    [anon_sym_loop] = ACTIONS(3792),
    [anon_sym_match] = ACTIONS(3792),
    [anon_sym_mod] = ACTIONS(3792),
    [anon_sym_pub] = ACTIONS(3792),
    [anon_sym_return] = ACTIONS(3792),
    [anon_sym_raw] = ACTIONS(3792),
    [anon_sym_static] = ACTIONS(3792),
    [anon_sym_struct] = ACTIONS(3792),
    [anon_sym_trait] = ACTIONS(3792),
    [anon_sym_type] = ACTIONS(3792),
    [anon_sym_union] = ACTIONS(3792),
    [anon_sym_unsafe] = ACTIONS(3792),
    [anon_sym_use] = ACTIONS(3792),
    [anon_sym_while] = ACTIONS(3792),
    [anon_sym_extern] = ACTIONS(3792),
    [anon_sym_safe] = ACTIONS(3792),
    [anon_sym_yield] = ACTIONS(3792),
    [anon_sym_move] = ACTIONS(3792),
    [anon_sym_try] = ACTIONS(3792),
    [sym_integer_literal] = ACTIONS(3790),
    [aux_sym_string_literal_token1] = ACTIONS(3790),
    [sym_char_literal] = ACTIONS(3790),
    [anon_sym_true] = ACTIONS(3792),
    [anon_sym_false] = ACTIONS(3792),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3792),
    [sym_super] = ACTIONS(3792),
    [sym_crate] = ACTIONS(3792),
    [aux_sym_metavariable_token1] = ACTIONS(3790),
    [sym__raw_string_literal_start] = ACTIONS(3790),
    [sym_float_literal] = ACTIONS(3790),
    [sym__outer_block_doc_comment_start] = ACTIONS(3790),
    [sym__outer_line_doc_comment_start] = ACTIONS(3790),
  },
  [STATE(1006)] = {
    [sym_line_comment] = STATE(1006),
    [sym_block_comment] = STATE(1006),
    [ts_builtin_sym_end] = ACTIONS(3794),
    [sym_identifier] = ACTIONS(3796),
    [anon_sym_SEMI] = ACTIONS(3794),
    [anon_sym_macro] = ACTIONS(3796),
    [anon_sym_LPAREN] = ACTIONS(3794),
    [anon_sym_LBRACE] = ACTIONS(3794),
    [anon_sym_RBRACE] = ACTIONS(3794),
    [anon_sym_macro_rules] = ACTIONS(3796),
    [anon_sym_BANG] = ACTIONS(3794),
    [anon_sym_LBRACK] = ACTIONS(3794),
    [anon_sym_STAR] = ACTIONS(3794),
    [anon_sym_u8] = ACTIONS(3796),
    [anon_sym_i8] = ACTIONS(3796),
    [anon_sym_u16] = ACTIONS(3796),
    [anon_sym_i16] = ACTIONS(3796),
    [anon_sym_u32] = ACTIONS(3796),
    [anon_sym_i32] = ACTIONS(3796),
    [anon_sym_u64] = ACTIONS(3796),
    [anon_sym_i64] = ACTIONS(3796),
    [anon_sym_u128] = ACTIONS(3796),
    [anon_sym_i128] = ACTIONS(3796),
    [anon_sym_isize] = ACTIONS(3796),
    [anon_sym_usize] = ACTIONS(3796),
    [anon_sym_f32] = ACTIONS(3796),
    [anon_sym_f64] = ACTIONS(3796),
    [anon_sym_bool] = ACTIONS(3796),
    [anon_sym_str] = ACTIONS(3796),
    [anon_sym_char] = ACTIONS(3796),
    [anon_sym_DASH] = ACTIONS(3794),
    [anon_sym_AMP] = ACTIONS(3794),
    [anon_sym_PIPE] = ACTIONS(3794),
    [anon_sym_LT] = ACTIONS(3794),
    [anon_sym_DOT_DOT] = ACTIONS(3794),
    [anon_sym_COLON_COLON] = ACTIONS(3794),
    [anon_sym_POUND] = ACTIONS(3794),
    [anon_sym_SQUOTE] = ACTIONS(3796),
    [anon_sym_async] = ACTIONS(3796),
    [anon_sym_auto] = ACTIONS(3796),
    [anon_sym_await] = ACTIONS(3796),
    [anon_sym_become] = ACTIONS(3796),
    [anon_sym_break] = ACTIONS(3796),
    [anon_sym_const] = ACTIONS(3796),
    [anon_sym_continue] = ACTIONS(3796),
    [anon_sym_default] = ACTIONS(3796),
    [anon_sym_enum] = ACTIONS(3796),
    [anon_sym_fn] = ACTIONS(3796),
    [anon_sym_for] = ACTIONS(3796),
    [anon_sym_gen] = ACTIONS(3796),
    [anon_sym_if] = ACTIONS(3796),
    [anon_sym_impl] = ACTIONS(3796),
    [anon_sym_let] = ACTIONS(3796),
    [anon_sym_loop] = ACTIONS(3796),
    [anon_sym_match] = ACTIONS(3796),
    [anon_sym_mod] = ACTIONS(3796),
    [anon_sym_pub] = ACTIONS(3796),
    [anon_sym_return] = ACTIONS(3796),
    [anon_sym_raw] = ACTIONS(3796),
    [anon_sym_static] = ACTIONS(3796),
    [anon_sym_struct] = ACTIONS(3796),
    [anon_sym_trait] = ACTIONS(3796),
    [anon_sym_type] = ACTIONS(3796),
    [anon_sym_union] = ACTIONS(3796),
    [anon_sym_unsafe] = ACTIONS(3796),
    [anon_sym_use] = ACTIONS(3796),
    [anon_sym_while] = ACTIONS(3796),
    [anon_sym_extern] = ACTIONS(3796),
    [anon_sym_safe] = ACTIONS(3796),
    [anon_sym_yield] = ACTIONS(3796),
    [anon_sym_move] = ACTIONS(3796),
    [anon_sym_try] = ACTIONS(3796),
    [sym_integer_literal] = ACTIONS(3794),
    [aux_sym_string_literal_token1] = ACTIONS(3794),
    [sym_char_literal] = ACTIONS(3794),
    [anon_sym_true] = ACTIONS(3796),
    [anon_sym_false] = ACTIONS(3796),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3796),
    [sym_super] = ACTIONS(3796),
    [sym_crate] = ACTIONS(3796),
    [aux_sym_metavariable_token1] = ACTIONS(3794),
    [sym__raw_string_literal_start] = ACTIONS(3794),
    [sym_float_literal] = ACTIONS(3794),
    [sym__outer_block_doc_comment_start] = ACTIONS(3794),
    [sym__outer_line_doc_comment_start] = ACTIONS(3794),
  },
  [STATE(1007)] = {
    [sym_line_comment] = STATE(1007),
    [sym_block_comment] = STATE(1007),
    [ts_builtin_sym_end] = ACTIONS(3798),
    [sym_identifier] = ACTIONS(3800),
    [anon_sym_SEMI] = ACTIONS(3798),
    [anon_sym_macro] = ACTIONS(3800),
    [anon_sym_LPAREN] = ACTIONS(3798),
    [anon_sym_LBRACE] = ACTIONS(3798),
    [anon_sym_RBRACE] = ACTIONS(3798),
    [anon_sym_macro_rules] = ACTIONS(3800),
    [anon_sym_BANG] = ACTIONS(3798),
    [anon_sym_LBRACK] = ACTIONS(3798),
    [anon_sym_STAR] = ACTIONS(3798),
    [anon_sym_u8] = ACTIONS(3800),
    [anon_sym_i8] = ACTIONS(3800),
    [anon_sym_u16] = ACTIONS(3800),
    [anon_sym_i16] = ACTIONS(3800),
    [anon_sym_u32] = ACTIONS(3800),
    [anon_sym_i32] = ACTIONS(3800),
    [anon_sym_u64] = ACTIONS(3800),
    [anon_sym_i64] = ACTIONS(3800),
    [anon_sym_u128] = ACTIONS(3800),
    [anon_sym_i128] = ACTIONS(3800),
    [anon_sym_isize] = ACTIONS(3800),
    [anon_sym_usize] = ACTIONS(3800),
    [anon_sym_f32] = ACTIONS(3800),
    [anon_sym_f64] = ACTIONS(3800),
    [anon_sym_bool] = ACTIONS(3800),
    [anon_sym_str] = ACTIONS(3800),
    [anon_sym_char] = ACTIONS(3800),
    [anon_sym_DASH] = ACTIONS(3798),
    [anon_sym_AMP] = ACTIONS(3798),
    [anon_sym_PIPE] = ACTIONS(3798),
    [anon_sym_LT] = ACTIONS(3798),
    [anon_sym_DOT_DOT] = ACTIONS(3798),
    [anon_sym_COLON_COLON] = ACTIONS(3798),
    [anon_sym_POUND] = ACTIONS(3798),
    [anon_sym_SQUOTE] = ACTIONS(3800),
    [anon_sym_async] = ACTIONS(3800),
    [anon_sym_auto] = ACTIONS(3800),
    [anon_sym_await] = ACTIONS(3800),
    [anon_sym_become] = ACTIONS(3800),
    [anon_sym_break] = ACTIONS(3800),
    [anon_sym_const] = ACTIONS(3800),
    [anon_sym_continue] = ACTIONS(3800),
    [anon_sym_default] = ACTIONS(3800),
    [anon_sym_enum] = ACTIONS(3800),
    [anon_sym_fn] = ACTIONS(3800),
    [anon_sym_for] = ACTIONS(3800),
    [anon_sym_gen] = ACTIONS(3800),
    [anon_sym_if] = ACTIONS(3800),
    [anon_sym_impl] = ACTIONS(3800),
    [anon_sym_let] = ACTIONS(3800),
    [anon_sym_loop] = ACTIONS(3800),
    [anon_sym_match] = ACTIONS(3800),
    [anon_sym_mod] = ACTIONS(3800),
    [anon_sym_pub] = ACTIONS(3800),
    [anon_sym_return] = ACTIONS(3800),
    [anon_sym_raw] = ACTIONS(3800),
    [anon_sym_static] = ACTIONS(3800),
    [anon_sym_struct] = ACTIONS(3800),
    [anon_sym_trait] = ACTIONS(3800),
    [anon_sym_type] = ACTIONS(3800),
    [anon_sym_union] = ACTIONS(3800),
    [anon_sym_unsafe] = ACTIONS(3800),
    [anon_sym_use] = ACTIONS(3800),
    [anon_sym_while] = ACTIONS(3800),
    [anon_sym_extern] = ACTIONS(3800),
    [anon_sym_safe] = ACTIONS(3800),
    [anon_sym_yield] = ACTIONS(3800),
    [anon_sym_move] = ACTIONS(3800),
    [anon_sym_try] = ACTIONS(3800),
    [sym_integer_literal] = ACTIONS(3798),
    [aux_sym_string_literal_token1] = ACTIONS(3798),
    [sym_char_literal] = ACTIONS(3798),
    [anon_sym_true] = ACTIONS(3800),
    [anon_sym_false] = ACTIONS(3800),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3800),
    [sym_super] = ACTIONS(3800),
    [sym_crate] = ACTIONS(3800),
    [aux_sym_metavariable_token1] = ACTIONS(3798),
    [sym__raw_string_literal_start] = ACTIONS(3798),
    [sym_float_literal] = ACTIONS(3798),
    [sym__outer_block_doc_comment_start] = ACTIONS(3798),
    [sym__outer_line_doc_comment_start] = ACTIONS(3798),
  },
  [STATE(1008)] = {
    [sym_line_comment] = STATE(1008),
    [sym_block_comment] = STATE(1008),
    [ts_builtin_sym_end] = ACTIONS(3802),
    [sym_identifier] = ACTIONS(3804),
    [anon_sym_SEMI] = ACTIONS(3802),
    [anon_sym_macro] = ACTIONS(3804),
    [anon_sym_LPAREN] = ACTIONS(3802),
    [anon_sym_LBRACE] = ACTIONS(3802),
    [anon_sym_RBRACE] = ACTIONS(3802),
    [anon_sym_macro_rules] = ACTIONS(3804),
    [anon_sym_BANG] = ACTIONS(3802),
    [anon_sym_LBRACK] = ACTIONS(3802),
    [anon_sym_STAR] = ACTIONS(3802),
    [anon_sym_u8] = ACTIONS(3804),
    [anon_sym_i8] = ACTIONS(3804),
    [anon_sym_u16] = ACTIONS(3804),
    [anon_sym_i16] = ACTIONS(3804),
    [anon_sym_u32] = ACTIONS(3804),
    [anon_sym_i32] = ACTIONS(3804),
    [anon_sym_u64] = ACTIONS(3804),
    [anon_sym_i64] = ACTIONS(3804),
    [anon_sym_u128] = ACTIONS(3804),
    [anon_sym_i128] = ACTIONS(3804),
    [anon_sym_isize] = ACTIONS(3804),
    [anon_sym_usize] = ACTIONS(3804),
    [anon_sym_f32] = ACTIONS(3804),
    [anon_sym_f64] = ACTIONS(3804),
    [anon_sym_bool] = ACTIONS(3804),
    [anon_sym_str] = ACTIONS(3804),
    [anon_sym_char] = ACTIONS(3804),
    [anon_sym_DASH] = ACTIONS(3802),
    [anon_sym_AMP] = ACTIONS(3802),
    [anon_sym_PIPE] = ACTIONS(3802),
    [anon_sym_LT] = ACTIONS(3802),
    [anon_sym_DOT_DOT] = ACTIONS(3802),
    [anon_sym_COLON_COLON] = ACTIONS(3802),
    [anon_sym_POUND] = ACTIONS(3802),
    [anon_sym_SQUOTE] = ACTIONS(3804),
    [anon_sym_async] = ACTIONS(3804),
    [anon_sym_auto] = ACTIONS(3804),
    [anon_sym_await] = ACTIONS(3804),
    [anon_sym_become] = ACTIONS(3804),
    [anon_sym_break] = ACTIONS(3804),
    [anon_sym_const] = ACTIONS(3804),
    [anon_sym_continue] = ACTIONS(3804),
    [anon_sym_default] = ACTIONS(3804),
    [anon_sym_enum] = ACTIONS(3804),
    [anon_sym_fn] = ACTIONS(3804),
    [anon_sym_for] = ACTIONS(3804),
    [anon_sym_gen] = ACTIONS(3804),
    [anon_sym_if] = ACTIONS(3804),
    [anon_sym_impl] = ACTIONS(3804),
    [anon_sym_let] = ACTIONS(3804),
    [anon_sym_loop] = ACTIONS(3804),
    [anon_sym_match] = ACTIONS(3804),
    [anon_sym_mod] = ACTIONS(3804),
    [anon_sym_pub] = ACTIONS(3804),
    [anon_sym_return] = ACTIONS(3804),
    [anon_sym_raw] = ACTIONS(3804),
    [anon_sym_static] = ACTIONS(3804),
    [anon_sym_struct] = ACTIONS(3804),
    [anon_sym_trait] = ACTIONS(3804),
    [anon_sym_type] = ACTIONS(3804),
    [anon_sym_union] = ACTIONS(3804),
    [anon_sym_unsafe] = ACTIONS(3804),
    [anon_sym_use] = ACTIONS(3804),
    [anon_sym_while] = ACTIONS(3804),
    [anon_sym_extern] = ACTIONS(3804),
    [anon_sym_safe] = ACTIONS(3804),
    [anon_sym_yield] = ACTIONS(3804),
    [anon_sym_move] = ACTIONS(3804),
    [anon_sym_try] = ACTIONS(3804),
    [sym_integer_literal] = ACTIONS(3802),
    [aux_sym_string_literal_token1] = ACTIONS(3802),
    [sym_char_literal] = ACTIONS(3802),
    [anon_sym_true] = ACTIONS(3804),
    [anon_sym_false] = ACTIONS(3804),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3804),
    [sym_super] = ACTIONS(3804),
    [sym_crate] = ACTIONS(3804),
    [aux_sym_metavariable_token1] = ACTIONS(3802),
    [sym__raw_string_literal_start] = ACTIONS(3802),
    [sym_float_literal] = ACTIONS(3802),
    [sym__outer_block_doc_comment_start] = ACTIONS(3802),
    [sym__outer_line_doc_comment_start] = ACTIONS(3802),
  },
  [STATE(1009)] = {
    [sym_line_comment] = STATE(1009),
    [sym_block_comment] = STATE(1009),
    [ts_builtin_sym_end] = ACTIONS(3806),
    [sym_identifier] = ACTIONS(3808),
    [anon_sym_SEMI] = ACTIONS(3806),
    [anon_sym_macro] = ACTIONS(3808),
    [anon_sym_LPAREN] = ACTIONS(3806),
    [anon_sym_LBRACE] = ACTIONS(3806),
    [anon_sym_RBRACE] = ACTIONS(3806),
    [anon_sym_macro_rules] = ACTIONS(3808),
    [anon_sym_BANG] = ACTIONS(3806),
    [anon_sym_LBRACK] = ACTIONS(3806),
    [anon_sym_STAR] = ACTIONS(3806),
    [anon_sym_u8] = ACTIONS(3808),
    [anon_sym_i8] = ACTIONS(3808),
    [anon_sym_u16] = ACTIONS(3808),
    [anon_sym_i16] = ACTIONS(3808),
    [anon_sym_u32] = ACTIONS(3808),
    [anon_sym_i32] = ACTIONS(3808),
    [anon_sym_u64] = ACTIONS(3808),
    [anon_sym_i64] = ACTIONS(3808),
    [anon_sym_u128] = ACTIONS(3808),
    [anon_sym_i128] = ACTIONS(3808),
    [anon_sym_isize] = ACTIONS(3808),
    [anon_sym_usize] = ACTIONS(3808),
    [anon_sym_f32] = ACTIONS(3808),
    [anon_sym_f64] = ACTIONS(3808),
    [anon_sym_bool] = ACTIONS(3808),
    [anon_sym_str] = ACTIONS(3808),
    [anon_sym_char] = ACTIONS(3808),
    [anon_sym_DASH] = ACTIONS(3806),
    [anon_sym_AMP] = ACTIONS(3806),
    [anon_sym_PIPE] = ACTIONS(3806),
    [anon_sym_LT] = ACTIONS(3806),
    [anon_sym_DOT_DOT] = ACTIONS(3806),
    [anon_sym_COLON_COLON] = ACTIONS(3806),
    [anon_sym_POUND] = ACTIONS(3806),
    [anon_sym_SQUOTE] = ACTIONS(3808),
    [anon_sym_async] = ACTIONS(3808),
    [anon_sym_auto] = ACTIONS(3808),
    [anon_sym_await] = ACTIONS(3808),
    [anon_sym_become] = ACTIONS(3808),
    [anon_sym_break] = ACTIONS(3808),
    [anon_sym_const] = ACTIONS(3808),
    [anon_sym_continue] = ACTIONS(3808),
    [anon_sym_default] = ACTIONS(3808),
    [anon_sym_enum] = ACTIONS(3808),
    [anon_sym_fn] = ACTIONS(3808),
    [anon_sym_for] = ACTIONS(3808),
    [anon_sym_gen] = ACTIONS(3808),
    [anon_sym_if] = ACTIONS(3808),
    [anon_sym_impl] = ACTIONS(3808),
    [anon_sym_let] = ACTIONS(3808),
    [anon_sym_loop] = ACTIONS(3808),
    [anon_sym_match] = ACTIONS(3808),
    [anon_sym_mod] = ACTIONS(3808),
    [anon_sym_pub] = ACTIONS(3808),
    [anon_sym_return] = ACTIONS(3808),
    [anon_sym_raw] = ACTIONS(3808),
    [anon_sym_static] = ACTIONS(3808),
    [anon_sym_struct] = ACTIONS(3808),
    [anon_sym_trait] = ACTIONS(3808),
    [anon_sym_type] = ACTIONS(3808),
    [anon_sym_union] = ACTIONS(3808),
    [anon_sym_unsafe] = ACTIONS(3808),
    [anon_sym_use] = ACTIONS(3808),
    [anon_sym_while] = ACTIONS(3808),
    [anon_sym_extern] = ACTIONS(3808),
    [anon_sym_safe] = ACTIONS(3808),
    [anon_sym_yield] = ACTIONS(3808),
    [anon_sym_move] = ACTIONS(3808),
    [anon_sym_try] = ACTIONS(3808),
    [sym_integer_literal] = ACTIONS(3806),
    [aux_sym_string_literal_token1] = ACTIONS(3806),
    [sym_char_literal] = ACTIONS(3806),
    [anon_sym_true] = ACTIONS(3808),
    [anon_sym_false] = ACTIONS(3808),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3808),
    [sym_super] = ACTIONS(3808),
    [sym_crate] = ACTIONS(3808),
    [aux_sym_metavariable_token1] = ACTIONS(3806),
    [sym__raw_string_literal_start] = ACTIONS(3806),
    [sym_float_literal] = ACTIONS(3806),
    [sym__outer_block_doc_comment_start] = ACTIONS(3806),
    [sym__outer_line_doc_comment_start] = ACTIONS(3806),
  },
  [STATE(1010)] = {
    [sym_line_comment] = STATE(1010),
    [sym_block_comment] = STATE(1010),
    [ts_builtin_sym_end] = ACTIONS(3810),
    [sym_identifier] = ACTIONS(3812),
    [anon_sym_SEMI] = ACTIONS(3810),
    [anon_sym_macro] = ACTIONS(3812),
    [anon_sym_LPAREN] = ACTIONS(3810),
    [anon_sym_LBRACE] = ACTIONS(3810),
    [anon_sym_RBRACE] = ACTIONS(3810),
    [anon_sym_macro_rules] = ACTIONS(3812),
    [anon_sym_BANG] = ACTIONS(3810),
    [anon_sym_LBRACK] = ACTIONS(3810),
    [anon_sym_STAR] = ACTIONS(3810),
    [anon_sym_u8] = ACTIONS(3812),
    [anon_sym_i8] = ACTIONS(3812),
    [anon_sym_u16] = ACTIONS(3812),
    [anon_sym_i16] = ACTIONS(3812),
    [anon_sym_u32] = ACTIONS(3812),
    [anon_sym_i32] = ACTIONS(3812),
    [anon_sym_u64] = ACTIONS(3812),
    [anon_sym_i64] = ACTIONS(3812),
    [anon_sym_u128] = ACTIONS(3812),
    [anon_sym_i128] = ACTIONS(3812),
    [anon_sym_isize] = ACTIONS(3812),
    [anon_sym_usize] = ACTIONS(3812),
    [anon_sym_f32] = ACTIONS(3812),
    [anon_sym_f64] = ACTIONS(3812),
    [anon_sym_bool] = ACTIONS(3812),
    [anon_sym_str] = ACTIONS(3812),
    [anon_sym_char] = ACTIONS(3812),
    [anon_sym_DASH] = ACTIONS(3810),
    [anon_sym_AMP] = ACTIONS(3810),
    [anon_sym_PIPE] = ACTIONS(3810),
    [anon_sym_LT] = ACTIONS(3810),
    [anon_sym_DOT_DOT] = ACTIONS(3810),
    [anon_sym_COLON_COLON] = ACTIONS(3810),
    [anon_sym_POUND] = ACTIONS(3810),
    [anon_sym_SQUOTE] = ACTIONS(3812),
    [anon_sym_async] = ACTIONS(3812),
    [anon_sym_auto] = ACTIONS(3812),
    [anon_sym_await] = ACTIONS(3812),
    [anon_sym_become] = ACTIONS(3812),
    [anon_sym_break] = ACTIONS(3812),
    [anon_sym_const] = ACTIONS(3812),
    [anon_sym_continue] = ACTIONS(3812),
    [anon_sym_default] = ACTIONS(3812),
    [anon_sym_enum] = ACTIONS(3812),
    [anon_sym_fn] = ACTIONS(3812),
    [anon_sym_for] = ACTIONS(3812),
    [anon_sym_gen] = ACTIONS(3812),
    [anon_sym_if] = ACTIONS(3812),
    [anon_sym_impl] = ACTIONS(3812),
    [anon_sym_let] = ACTIONS(3812),
    [anon_sym_loop] = ACTIONS(3812),
    [anon_sym_match] = ACTIONS(3812),
    [anon_sym_mod] = ACTIONS(3812),
    [anon_sym_pub] = ACTIONS(3812),
    [anon_sym_return] = ACTIONS(3812),
    [anon_sym_raw] = ACTIONS(3812),
    [anon_sym_static] = ACTIONS(3812),
    [anon_sym_struct] = ACTIONS(3812),
    [anon_sym_trait] = ACTIONS(3812),
    [anon_sym_type] = ACTIONS(3812),
    [anon_sym_union] = ACTIONS(3812),
    [anon_sym_unsafe] = ACTIONS(3812),
    [anon_sym_use] = ACTIONS(3812),
    [anon_sym_while] = ACTIONS(3812),
    [anon_sym_extern] = ACTIONS(3812),
    [anon_sym_safe] = ACTIONS(3812),
    [anon_sym_yield] = ACTIONS(3812),
    [anon_sym_move] = ACTIONS(3812),
    [anon_sym_try] = ACTIONS(3812),
    [sym_integer_literal] = ACTIONS(3810),
    [aux_sym_string_literal_token1] = ACTIONS(3810),
    [sym_char_literal] = ACTIONS(3810),
    [anon_sym_true] = ACTIONS(3812),
    [anon_sym_false] = ACTIONS(3812),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3812),
    [sym_super] = ACTIONS(3812),
    [sym_crate] = ACTIONS(3812),
    [aux_sym_metavariable_token1] = ACTIONS(3810),
    [sym__raw_string_literal_start] = ACTIONS(3810),
    [sym_float_literal] = ACTIONS(3810),
    [sym__outer_block_doc_comment_start] = ACTIONS(3810),
    [sym__outer_line_doc_comment_start] = ACTIONS(3810),
  },
  [STATE(1011)] = {
    [sym_line_comment] = STATE(1011),
    [sym_block_comment] = STATE(1011),
    [ts_builtin_sym_end] = ACTIONS(3814),
    [sym_identifier] = ACTIONS(3816),
    [anon_sym_SEMI] = ACTIONS(3814),
    [anon_sym_macro] = ACTIONS(3816),
    [anon_sym_LPAREN] = ACTIONS(3814),
    [anon_sym_LBRACE] = ACTIONS(3814),
    [anon_sym_RBRACE] = ACTIONS(3814),
    [anon_sym_macro_rules] = ACTIONS(3816),
    [anon_sym_BANG] = ACTIONS(3814),
    [anon_sym_LBRACK] = ACTIONS(3814),
    [anon_sym_STAR] = ACTIONS(3814),
    [anon_sym_u8] = ACTIONS(3816),
    [anon_sym_i8] = ACTIONS(3816),
    [anon_sym_u16] = ACTIONS(3816),
    [anon_sym_i16] = ACTIONS(3816),
    [anon_sym_u32] = ACTIONS(3816),
    [anon_sym_i32] = ACTIONS(3816),
    [anon_sym_u64] = ACTIONS(3816),
    [anon_sym_i64] = ACTIONS(3816),
    [anon_sym_u128] = ACTIONS(3816),
    [anon_sym_i128] = ACTIONS(3816),
    [anon_sym_isize] = ACTIONS(3816),
    [anon_sym_usize] = ACTIONS(3816),
    [anon_sym_f32] = ACTIONS(3816),
    [anon_sym_f64] = ACTIONS(3816),
    [anon_sym_bool] = ACTIONS(3816),
    [anon_sym_str] = ACTIONS(3816),
    [anon_sym_char] = ACTIONS(3816),
    [anon_sym_DASH] = ACTIONS(3814),
    [anon_sym_AMP] = ACTIONS(3814),
    [anon_sym_PIPE] = ACTIONS(3814),
    [anon_sym_LT] = ACTIONS(3814),
    [anon_sym_DOT_DOT] = ACTIONS(3814),
    [anon_sym_COLON_COLON] = ACTIONS(3814),
    [anon_sym_POUND] = ACTIONS(3814),
    [anon_sym_SQUOTE] = ACTIONS(3816),
    [anon_sym_async] = ACTIONS(3816),
    [anon_sym_auto] = ACTIONS(3816),
    [anon_sym_await] = ACTIONS(3816),
    [anon_sym_become] = ACTIONS(3816),
    [anon_sym_break] = ACTIONS(3816),
    [anon_sym_const] = ACTIONS(3816),
    [anon_sym_continue] = ACTIONS(3816),
    [anon_sym_default] = ACTIONS(3816),
    [anon_sym_enum] = ACTIONS(3816),
    [anon_sym_fn] = ACTIONS(3816),
    [anon_sym_for] = ACTIONS(3816),
    [anon_sym_gen] = ACTIONS(3816),
    [anon_sym_if] = ACTIONS(3816),
    [anon_sym_impl] = ACTIONS(3816),
    [anon_sym_let] = ACTIONS(3816),
    [anon_sym_loop] = ACTIONS(3816),
    [anon_sym_match] = ACTIONS(3816),
    [anon_sym_mod] = ACTIONS(3816),
    [anon_sym_pub] = ACTIONS(3816),
    [anon_sym_return] = ACTIONS(3816),
    [anon_sym_raw] = ACTIONS(3816),
    [anon_sym_static] = ACTIONS(3816),
    [anon_sym_struct] = ACTIONS(3816),
    [anon_sym_trait] = ACTIONS(3816),
    [anon_sym_type] = ACTIONS(3816),
    [anon_sym_union] = ACTIONS(3816),
    [anon_sym_unsafe] = ACTIONS(3816),
    [anon_sym_use] = ACTIONS(3816),
    [anon_sym_while] = ACTIONS(3816),
    [anon_sym_extern] = ACTIONS(3816),
    [anon_sym_safe] = ACTIONS(3816),
    [anon_sym_yield] = ACTIONS(3816),
    [anon_sym_move] = ACTIONS(3816),
    [anon_sym_try] = ACTIONS(3816),
    [sym_integer_literal] = ACTIONS(3814),
    [aux_sym_string_literal_token1] = ACTIONS(3814),
    [sym_char_literal] = ACTIONS(3814),
    [anon_sym_true] = ACTIONS(3816),
    [anon_sym_false] = ACTIONS(3816),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3816),
    [sym_super] = ACTIONS(3816),
    [sym_crate] = ACTIONS(3816),
    [aux_sym_metavariable_token1] = ACTIONS(3814),
    [sym__raw_string_literal_start] = ACTIONS(3814),
    [sym_float_literal] = ACTIONS(3814),
    [sym__outer_block_doc_comment_start] = ACTIONS(3814),
    [sym__outer_line_doc_comment_start] = ACTIONS(3814),
  },
  [STATE(1012)] = {
    [sym_line_comment] = STATE(1012),
    [sym_block_comment] = STATE(1012),
    [ts_builtin_sym_end] = ACTIONS(3818),
    [sym_identifier] = ACTIONS(3820),
    [anon_sym_SEMI] = ACTIONS(3818),
    [anon_sym_macro] = ACTIONS(3820),
    [anon_sym_LPAREN] = ACTIONS(3818),
    [anon_sym_LBRACE] = ACTIONS(3818),
    [anon_sym_RBRACE] = ACTIONS(3818),
    [anon_sym_macro_rules] = ACTIONS(3820),
    [anon_sym_BANG] = ACTIONS(3818),
    [anon_sym_LBRACK] = ACTIONS(3818),
    [anon_sym_STAR] = ACTIONS(3818),
    [anon_sym_u8] = ACTIONS(3820),
    [anon_sym_i8] = ACTIONS(3820),
    [anon_sym_u16] = ACTIONS(3820),
    [anon_sym_i16] = ACTIONS(3820),
    [anon_sym_u32] = ACTIONS(3820),
    [anon_sym_i32] = ACTIONS(3820),
    [anon_sym_u64] = ACTIONS(3820),
    [anon_sym_i64] = ACTIONS(3820),
    [anon_sym_u128] = ACTIONS(3820),
    [anon_sym_i128] = ACTIONS(3820),
    [anon_sym_isize] = ACTIONS(3820),
    [anon_sym_usize] = ACTIONS(3820),
    [anon_sym_f32] = ACTIONS(3820),
    [anon_sym_f64] = ACTIONS(3820),
    [anon_sym_bool] = ACTIONS(3820),
    [anon_sym_str] = ACTIONS(3820),
    [anon_sym_char] = ACTIONS(3820),
    [anon_sym_DASH] = ACTIONS(3818),
    [anon_sym_AMP] = ACTIONS(3818),
    [anon_sym_PIPE] = ACTIONS(3818),
    [anon_sym_LT] = ACTIONS(3818),
    [anon_sym_DOT_DOT] = ACTIONS(3818),
    [anon_sym_COLON_COLON] = ACTIONS(3818),
    [anon_sym_POUND] = ACTIONS(3818),
    [anon_sym_SQUOTE] = ACTIONS(3820),
    [anon_sym_async] = ACTIONS(3820),
    [anon_sym_auto] = ACTIONS(3820),
    [anon_sym_await] = ACTIONS(3820),
    [anon_sym_become] = ACTIONS(3820),
    [anon_sym_break] = ACTIONS(3820),
    [anon_sym_const] = ACTIONS(3820),
    [anon_sym_continue] = ACTIONS(3820),
    [anon_sym_default] = ACTIONS(3820),
    [anon_sym_enum] = ACTIONS(3820),
    [anon_sym_fn] = ACTIONS(3820),
    [anon_sym_for] = ACTIONS(3820),
    [anon_sym_gen] = ACTIONS(3820),
    [anon_sym_if] = ACTIONS(3820),
    [anon_sym_impl] = ACTIONS(3820),
    [anon_sym_let] = ACTIONS(3820),
    [anon_sym_loop] = ACTIONS(3820),
    [anon_sym_match] = ACTIONS(3820),
    [anon_sym_mod] = ACTIONS(3820),
    [anon_sym_pub] = ACTIONS(3820),
    [anon_sym_return] = ACTIONS(3820),
    [anon_sym_raw] = ACTIONS(3820),
    [anon_sym_static] = ACTIONS(3820),
    [anon_sym_struct] = ACTIONS(3820),
    [anon_sym_trait] = ACTIONS(3820),
    [anon_sym_type] = ACTIONS(3820),
    [anon_sym_union] = ACTIONS(3820),
    [anon_sym_unsafe] = ACTIONS(3820),
    [anon_sym_use] = ACTIONS(3820),
    [anon_sym_while] = ACTIONS(3820),
    [anon_sym_extern] = ACTIONS(3820),
    [anon_sym_safe] = ACTIONS(3820),
    [anon_sym_yield] = ACTIONS(3820),
    [anon_sym_move] = ACTIONS(3820),
    [anon_sym_try] = ACTIONS(3820),
    [sym_integer_literal] = ACTIONS(3818),
    [aux_sym_string_literal_token1] = ACTIONS(3818),
    [sym_char_literal] = ACTIONS(3818),
    [anon_sym_true] = ACTIONS(3820),
    [anon_sym_false] = ACTIONS(3820),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3820),
    [sym_super] = ACTIONS(3820),
    [sym_crate] = ACTIONS(3820),
    [aux_sym_metavariable_token1] = ACTIONS(3818),
    [sym__raw_string_literal_start] = ACTIONS(3818),
    [sym_float_literal] = ACTIONS(3818),
    [sym__outer_block_doc_comment_start] = ACTIONS(3818),
    [sym__outer_line_doc_comment_start] = ACTIONS(3818),
  },
  [STATE(1013)] = {
    [sym_line_comment] = STATE(1013),
    [sym_block_comment] = STATE(1013),
    [ts_builtin_sym_end] = ACTIONS(3822),
    [sym_identifier] = ACTIONS(3824),
    [anon_sym_SEMI] = ACTIONS(3822),
    [anon_sym_macro] = ACTIONS(3824),
    [anon_sym_LPAREN] = ACTIONS(3822),
    [anon_sym_LBRACE] = ACTIONS(3822),
    [anon_sym_RBRACE] = ACTIONS(3822),
    [anon_sym_macro_rules] = ACTIONS(3824),
    [anon_sym_BANG] = ACTIONS(3822),
    [anon_sym_LBRACK] = ACTIONS(3822),
    [anon_sym_STAR] = ACTIONS(3822),
    [anon_sym_u8] = ACTIONS(3824),
    [anon_sym_i8] = ACTIONS(3824),
    [anon_sym_u16] = ACTIONS(3824),
    [anon_sym_i16] = ACTIONS(3824),
    [anon_sym_u32] = ACTIONS(3824),
    [anon_sym_i32] = ACTIONS(3824),
    [anon_sym_u64] = ACTIONS(3824),
    [anon_sym_i64] = ACTIONS(3824),
    [anon_sym_u128] = ACTIONS(3824),
    [anon_sym_i128] = ACTIONS(3824),
    [anon_sym_isize] = ACTIONS(3824),
    [anon_sym_usize] = ACTIONS(3824),
    [anon_sym_f32] = ACTIONS(3824),
    [anon_sym_f64] = ACTIONS(3824),
    [anon_sym_bool] = ACTIONS(3824),
    [anon_sym_str] = ACTIONS(3824),
    [anon_sym_char] = ACTIONS(3824),
    [anon_sym_DASH] = ACTIONS(3822),
    [anon_sym_AMP] = ACTIONS(3822),
    [anon_sym_PIPE] = ACTIONS(3822),
    [anon_sym_LT] = ACTIONS(3822),
    [anon_sym_DOT_DOT] = ACTIONS(3822),
    [anon_sym_COLON_COLON] = ACTIONS(3822),
    [anon_sym_POUND] = ACTIONS(3822),
    [anon_sym_SQUOTE] = ACTIONS(3824),
    [anon_sym_async] = ACTIONS(3824),
    [anon_sym_auto] = ACTIONS(3824),
    [anon_sym_await] = ACTIONS(3824),
    [anon_sym_become] = ACTIONS(3824),
    [anon_sym_break] = ACTIONS(3824),
    [anon_sym_const] = ACTIONS(3824),
    [anon_sym_continue] = ACTIONS(3824),
    [anon_sym_default] = ACTIONS(3824),
    [anon_sym_enum] = ACTIONS(3824),
    [anon_sym_fn] = ACTIONS(3824),
    [anon_sym_for] = ACTIONS(3824),
    [anon_sym_gen] = ACTIONS(3824),
    [anon_sym_if] = ACTIONS(3824),
    [anon_sym_impl] = ACTIONS(3824),
    [anon_sym_let] = ACTIONS(3824),
    [anon_sym_loop] = ACTIONS(3824),
    [anon_sym_match] = ACTIONS(3824),
    [anon_sym_mod] = ACTIONS(3824),
    [anon_sym_pub] = ACTIONS(3824),
    [anon_sym_return] = ACTIONS(3824),
    [anon_sym_raw] = ACTIONS(3824),
    [anon_sym_static] = ACTIONS(3824),
    [anon_sym_struct] = ACTIONS(3824),
    [anon_sym_trait] = ACTIONS(3824),
    [anon_sym_type] = ACTIONS(3824),
    [anon_sym_union] = ACTIONS(3824),
    [anon_sym_unsafe] = ACTIONS(3824),
    [anon_sym_use] = ACTIONS(3824),
    [anon_sym_while] = ACTIONS(3824),
    [anon_sym_extern] = ACTIONS(3824),
    [anon_sym_safe] = ACTIONS(3824),
    [anon_sym_yield] = ACTIONS(3824),
    [anon_sym_move] = ACTIONS(3824),
    [anon_sym_try] = ACTIONS(3824),
    [sym_integer_literal] = ACTIONS(3822),
    [aux_sym_string_literal_token1] = ACTIONS(3822),
    [sym_char_literal] = ACTIONS(3822),
    [anon_sym_true] = ACTIONS(3824),
    [anon_sym_false] = ACTIONS(3824),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3824),
    [sym_super] = ACTIONS(3824),
    [sym_crate] = ACTIONS(3824),
    [aux_sym_metavariable_token1] = ACTIONS(3822),
    [sym__raw_string_literal_start] = ACTIONS(3822),
    [sym_float_literal] = ACTIONS(3822),
    [sym__outer_block_doc_comment_start] = ACTIONS(3822),
    [sym__outer_line_doc_comment_start] = ACTIONS(3822),
  },
  [STATE(1014)] = {
    [sym_line_comment] = STATE(1014),
    [sym_block_comment] = STATE(1014),
    [ts_builtin_sym_end] = ACTIONS(3826),
    [sym_identifier] = ACTIONS(3828),
    [anon_sym_SEMI] = ACTIONS(3826),
    [anon_sym_macro] = ACTIONS(3828),
    [anon_sym_LPAREN] = ACTIONS(3826),
    [anon_sym_LBRACE] = ACTIONS(3826),
    [anon_sym_RBRACE] = ACTIONS(3826),
    [anon_sym_macro_rules] = ACTIONS(3828),
    [anon_sym_BANG] = ACTIONS(3826),
    [anon_sym_LBRACK] = ACTIONS(3826),
    [anon_sym_STAR] = ACTIONS(3826),
    [anon_sym_u8] = ACTIONS(3828),
    [anon_sym_i8] = ACTIONS(3828),
    [anon_sym_u16] = ACTIONS(3828),
    [anon_sym_i16] = ACTIONS(3828),
    [anon_sym_u32] = ACTIONS(3828),
    [anon_sym_i32] = ACTIONS(3828),
    [anon_sym_u64] = ACTIONS(3828),
    [anon_sym_i64] = ACTIONS(3828),
    [anon_sym_u128] = ACTIONS(3828),
    [anon_sym_i128] = ACTIONS(3828),
    [anon_sym_isize] = ACTIONS(3828),
    [anon_sym_usize] = ACTIONS(3828),
    [anon_sym_f32] = ACTIONS(3828),
    [anon_sym_f64] = ACTIONS(3828),
    [anon_sym_bool] = ACTIONS(3828),
    [anon_sym_str] = ACTIONS(3828),
    [anon_sym_char] = ACTIONS(3828),
    [anon_sym_DASH] = ACTIONS(3826),
    [anon_sym_AMP] = ACTIONS(3826),
    [anon_sym_PIPE] = ACTIONS(3826),
    [anon_sym_LT] = ACTIONS(3826),
    [anon_sym_DOT_DOT] = ACTIONS(3826),
    [anon_sym_COLON_COLON] = ACTIONS(3826),
    [anon_sym_POUND] = ACTIONS(3826),
    [anon_sym_SQUOTE] = ACTIONS(3828),
    [anon_sym_async] = ACTIONS(3828),
    [anon_sym_auto] = ACTIONS(3828),
    [anon_sym_await] = ACTIONS(3828),
    [anon_sym_become] = ACTIONS(3828),
    [anon_sym_break] = ACTIONS(3828),
    [anon_sym_const] = ACTIONS(3828),
    [anon_sym_continue] = ACTIONS(3828),
    [anon_sym_default] = ACTIONS(3828),
    [anon_sym_enum] = ACTIONS(3828),
    [anon_sym_fn] = ACTIONS(3828),
    [anon_sym_for] = ACTIONS(3828),
    [anon_sym_gen] = ACTIONS(3828),
    [anon_sym_if] = ACTIONS(3828),
    [anon_sym_impl] = ACTIONS(3828),
    [anon_sym_let] = ACTIONS(3828),
    [anon_sym_loop] = ACTIONS(3828),
    [anon_sym_match] = ACTIONS(3828),
    [anon_sym_mod] = ACTIONS(3828),
    [anon_sym_pub] = ACTIONS(3828),
    [anon_sym_return] = ACTIONS(3828),
    [anon_sym_raw] = ACTIONS(3828),
    [anon_sym_static] = ACTIONS(3828),
    [anon_sym_struct] = ACTIONS(3828),
    [anon_sym_trait] = ACTIONS(3828),
    [anon_sym_type] = ACTIONS(3828),
    [anon_sym_union] = ACTIONS(3828),
    [anon_sym_unsafe] = ACTIONS(3828),
    [anon_sym_use] = ACTIONS(3828),
    [anon_sym_while] = ACTIONS(3828),
    [anon_sym_extern] = ACTIONS(3828),
    [anon_sym_safe] = ACTIONS(3828),
    [anon_sym_yield] = ACTIONS(3828),
    [anon_sym_move] = ACTIONS(3828),
    [anon_sym_try] = ACTIONS(3828),
    [sym_integer_literal] = ACTIONS(3826),
    [aux_sym_string_literal_token1] = ACTIONS(3826),
    [sym_char_literal] = ACTIONS(3826),
    [anon_sym_true] = ACTIONS(3828),
    [anon_sym_false] = ACTIONS(3828),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3828),
    [sym_super] = ACTIONS(3828),
    [sym_crate] = ACTIONS(3828),
    [aux_sym_metavariable_token1] = ACTIONS(3826),
    [sym__raw_string_literal_start] = ACTIONS(3826),
    [sym_float_literal] = ACTIONS(3826),
    [sym__outer_block_doc_comment_start] = ACTIONS(3826),
    [sym__outer_line_doc_comment_start] = ACTIONS(3826),
  },
  [STATE(1015)] = {
    [sym_line_comment] = STATE(1015),
    [sym_block_comment] = STATE(1015),
    [ts_builtin_sym_end] = ACTIONS(3830),
    [sym_identifier] = ACTIONS(3832),
    [anon_sym_SEMI] = ACTIONS(3830),
    [anon_sym_macro] = ACTIONS(3832),
    [anon_sym_LPAREN] = ACTIONS(3830),
    [anon_sym_LBRACE] = ACTIONS(3830),
    [anon_sym_RBRACE] = ACTIONS(3830),
    [anon_sym_macro_rules] = ACTIONS(3832),
    [anon_sym_BANG] = ACTIONS(3830),
    [anon_sym_LBRACK] = ACTIONS(3830),
    [anon_sym_STAR] = ACTIONS(3830),
    [anon_sym_u8] = ACTIONS(3832),
    [anon_sym_i8] = ACTIONS(3832),
    [anon_sym_u16] = ACTIONS(3832),
    [anon_sym_i16] = ACTIONS(3832),
    [anon_sym_u32] = ACTIONS(3832),
    [anon_sym_i32] = ACTIONS(3832),
    [anon_sym_u64] = ACTIONS(3832),
    [anon_sym_i64] = ACTIONS(3832),
    [anon_sym_u128] = ACTIONS(3832),
    [anon_sym_i128] = ACTIONS(3832),
    [anon_sym_isize] = ACTIONS(3832),
    [anon_sym_usize] = ACTIONS(3832),
    [anon_sym_f32] = ACTIONS(3832),
    [anon_sym_f64] = ACTIONS(3832),
    [anon_sym_bool] = ACTIONS(3832),
    [anon_sym_str] = ACTIONS(3832),
    [anon_sym_char] = ACTIONS(3832),
    [anon_sym_DASH] = ACTIONS(3830),
    [anon_sym_AMP] = ACTIONS(3830),
    [anon_sym_PIPE] = ACTIONS(3830),
    [anon_sym_LT] = ACTIONS(3830),
    [anon_sym_DOT_DOT] = ACTIONS(3830),
    [anon_sym_COLON_COLON] = ACTIONS(3830),
    [anon_sym_POUND] = ACTIONS(3830),
    [anon_sym_SQUOTE] = ACTIONS(3832),
    [anon_sym_async] = ACTIONS(3832),
    [anon_sym_auto] = ACTIONS(3832),
    [anon_sym_await] = ACTIONS(3832),
    [anon_sym_become] = ACTIONS(3832),
    [anon_sym_break] = ACTIONS(3832),
    [anon_sym_const] = ACTIONS(3832),
    [anon_sym_continue] = ACTIONS(3832),
    [anon_sym_default] = ACTIONS(3832),
    [anon_sym_enum] = ACTIONS(3832),
    [anon_sym_fn] = ACTIONS(3832),
    [anon_sym_for] = ACTIONS(3832),
    [anon_sym_gen] = ACTIONS(3832),
    [anon_sym_if] = ACTIONS(3832),
    [anon_sym_impl] = ACTIONS(3832),
    [anon_sym_let] = ACTIONS(3832),
    [anon_sym_loop] = ACTIONS(3832),
    [anon_sym_match] = ACTIONS(3832),
    [anon_sym_mod] = ACTIONS(3832),
    [anon_sym_pub] = ACTIONS(3832),
    [anon_sym_return] = ACTIONS(3832),
    [anon_sym_raw] = ACTIONS(3832),
    [anon_sym_static] = ACTIONS(3832),
    [anon_sym_struct] = ACTIONS(3832),
    [anon_sym_trait] = ACTIONS(3832),
    [anon_sym_type] = ACTIONS(3832),
    [anon_sym_union] = ACTIONS(3832),
    [anon_sym_unsafe] = ACTIONS(3832),
    [anon_sym_use] = ACTIONS(3832),
    [anon_sym_while] = ACTIONS(3832),
    [anon_sym_extern] = ACTIONS(3832),
    [anon_sym_safe] = ACTIONS(3832),
    [anon_sym_yield] = ACTIONS(3832),
    [anon_sym_move] = ACTIONS(3832),
    [anon_sym_try] = ACTIONS(3832),
    [sym_integer_literal] = ACTIONS(3830),
    [aux_sym_string_literal_token1] = ACTIONS(3830),
    [sym_char_literal] = ACTIONS(3830),
    [anon_sym_true] = ACTIONS(3832),
    [anon_sym_false] = ACTIONS(3832),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3832),
    [sym_super] = ACTIONS(3832),
    [sym_crate] = ACTIONS(3832),
    [aux_sym_metavariable_token1] = ACTIONS(3830),
    [sym__raw_string_literal_start] = ACTIONS(3830),
    [sym_float_literal] = ACTIONS(3830),
    [sym__outer_block_doc_comment_start] = ACTIONS(3830),
    [sym__outer_line_doc_comment_start] = ACTIONS(3830),
  },
  [STATE(1016)] = {
    [sym_line_comment] = STATE(1016),
    [sym_block_comment] = STATE(1016),
    [ts_builtin_sym_end] = ACTIONS(3834),
    [sym_identifier] = ACTIONS(3836),
    [anon_sym_SEMI] = ACTIONS(3834),
    [anon_sym_macro] = ACTIONS(3836),
    [anon_sym_LPAREN] = ACTIONS(3834),
    [anon_sym_LBRACE] = ACTIONS(3834),
    [anon_sym_RBRACE] = ACTIONS(3834),
    [anon_sym_macro_rules] = ACTIONS(3836),
    [anon_sym_BANG] = ACTIONS(3834),
    [anon_sym_LBRACK] = ACTIONS(3834),
    [anon_sym_STAR] = ACTIONS(3834),
    [anon_sym_u8] = ACTIONS(3836),
    [anon_sym_i8] = ACTIONS(3836),
    [anon_sym_u16] = ACTIONS(3836),
    [anon_sym_i16] = ACTIONS(3836),
    [anon_sym_u32] = ACTIONS(3836),
    [anon_sym_i32] = ACTIONS(3836),
    [anon_sym_u64] = ACTIONS(3836),
    [anon_sym_i64] = ACTIONS(3836),
    [anon_sym_u128] = ACTIONS(3836),
    [anon_sym_i128] = ACTIONS(3836),
    [anon_sym_isize] = ACTIONS(3836),
    [anon_sym_usize] = ACTIONS(3836),
    [anon_sym_f32] = ACTIONS(3836),
    [anon_sym_f64] = ACTIONS(3836),
    [anon_sym_bool] = ACTIONS(3836),
    [anon_sym_str] = ACTIONS(3836),
    [anon_sym_char] = ACTIONS(3836),
    [anon_sym_DASH] = ACTIONS(3834),
    [anon_sym_AMP] = ACTIONS(3834),
    [anon_sym_PIPE] = ACTIONS(3834),
    [anon_sym_LT] = ACTIONS(3834),
    [anon_sym_DOT_DOT] = ACTIONS(3834),
    [anon_sym_COLON_COLON] = ACTIONS(3834),
    [anon_sym_POUND] = ACTIONS(3834),
    [anon_sym_SQUOTE] = ACTIONS(3836),
    [anon_sym_async] = ACTIONS(3836),
    [anon_sym_auto] = ACTIONS(3836),
    [anon_sym_await] = ACTIONS(3836),
    [anon_sym_become] = ACTIONS(3836),
    [anon_sym_break] = ACTIONS(3836),
    [anon_sym_const] = ACTIONS(3836),
    [anon_sym_continue] = ACTIONS(3836),
    [anon_sym_default] = ACTIONS(3836),
    [anon_sym_enum] = ACTIONS(3836),
    [anon_sym_fn] = ACTIONS(3836),
    [anon_sym_for] = ACTIONS(3836),
    [anon_sym_gen] = ACTIONS(3836),
    [anon_sym_if] = ACTIONS(3836),
    [anon_sym_impl] = ACTIONS(3836),
    [anon_sym_let] = ACTIONS(3836),
    [anon_sym_loop] = ACTIONS(3836),
    [anon_sym_match] = ACTIONS(3836),
    [anon_sym_mod] = ACTIONS(3836),
    [anon_sym_pub] = ACTIONS(3836),
    [anon_sym_return] = ACTIONS(3836),
    [anon_sym_raw] = ACTIONS(3836),
    [anon_sym_static] = ACTIONS(3836),
    [anon_sym_struct] = ACTIONS(3836),
    [anon_sym_trait] = ACTIONS(3836),
    [anon_sym_type] = ACTIONS(3836),
    [anon_sym_union] = ACTIONS(3836),
    [anon_sym_unsafe] = ACTIONS(3836),
    [anon_sym_use] = ACTIONS(3836),
    [anon_sym_while] = ACTIONS(3836),
    [anon_sym_extern] = ACTIONS(3836),
    [anon_sym_safe] = ACTIONS(3836),
    [anon_sym_yield] = ACTIONS(3836),
    [anon_sym_move] = ACTIONS(3836),
    [anon_sym_try] = ACTIONS(3836),
    [sym_integer_literal] = ACTIONS(3834),
    [aux_sym_string_literal_token1] = ACTIONS(3834),
    [sym_char_literal] = ACTIONS(3834),
    [anon_sym_true] = ACTIONS(3836),
    [anon_sym_false] = ACTIONS(3836),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3836),
    [sym_super] = ACTIONS(3836),
    [sym_crate] = ACTIONS(3836),
    [aux_sym_metavariable_token1] = ACTIONS(3834),
    [sym__raw_string_literal_start] = ACTIONS(3834),
    [sym_float_literal] = ACTIONS(3834),
    [sym__outer_block_doc_comment_start] = ACTIONS(3834),
    [sym__outer_line_doc_comment_start] = ACTIONS(3834),
  },
  [STATE(1017)] = {
    [sym_line_comment] = STATE(1017),
    [sym_block_comment] = STATE(1017),
    [ts_builtin_sym_end] = ACTIONS(3838),
    [sym_identifier] = ACTIONS(3840),
    [anon_sym_SEMI] = ACTIONS(3838),
    [anon_sym_macro] = ACTIONS(3840),
    [anon_sym_LPAREN] = ACTIONS(3838),
    [anon_sym_LBRACE] = ACTIONS(3838),
    [anon_sym_RBRACE] = ACTIONS(3838),
    [anon_sym_macro_rules] = ACTIONS(3840),
    [anon_sym_BANG] = ACTIONS(3838),
    [anon_sym_LBRACK] = ACTIONS(3838),
    [anon_sym_STAR] = ACTIONS(3838),
    [anon_sym_u8] = ACTIONS(3840),
    [anon_sym_i8] = ACTIONS(3840),
    [anon_sym_u16] = ACTIONS(3840),
    [anon_sym_i16] = ACTIONS(3840),
    [anon_sym_u32] = ACTIONS(3840),
    [anon_sym_i32] = ACTIONS(3840),
    [anon_sym_u64] = ACTIONS(3840),
    [anon_sym_i64] = ACTIONS(3840),
    [anon_sym_u128] = ACTIONS(3840),
    [anon_sym_i128] = ACTIONS(3840),
    [anon_sym_isize] = ACTIONS(3840),
    [anon_sym_usize] = ACTIONS(3840),
    [anon_sym_f32] = ACTIONS(3840),
    [anon_sym_f64] = ACTIONS(3840),
    [anon_sym_bool] = ACTIONS(3840),
    [anon_sym_str] = ACTIONS(3840),
    [anon_sym_char] = ACTIONS(3840),
    [anon_sym_DASH] = ACTIONS(3838),
    [anon_sym_AMP] = ACTIONS(3838),
    [anon_sym_PIPE] = ACTIONS(3838),
    [anon_sym_LT] = ACTIONS(3838),
    [anon_sym_DOT_DOT] = ACTIONS(3838),
    [anon_sym_COLON_COLON] = ACTIONS(3838),
    [anon_sym_POUND] = ACTIONS(3838),
    [anon_sym_SQUOTE] = ACTIONS(3840),
    [anon_sym_async] = ACTIONS(3840),
    [anon_sym_auto] = ACTIONS(3840),
    [anon_sym_await] = ACTIONS(3840),
    [anon_sym_become] = ACTIONS(3840),
    [anon_sym_break] = ACTIONS(3840),
    [anon_sym_const] = ACTIONS(3840),
    [anon_sym_continue] = ACTIONS(3840),
    [anon_sym_default] = ACTIONS(3840),
    [anon_sym_enum] = ACTIONS(3840),
    [anon_sym_fn] = ACTIONS(3840),
    [anon_sym_for] = ACTIONS(3840),
    [anon_sym_gen] = ACTIONS(3840),
    [anon_sym_if] = ACTIONS(3840),
    [anon_sym_impl] = ACTIONS(3840),
    [anon_sym_let] = ACTIONS(3840),
    [anon_sym_loop] = ACTIONS(3840),
    [anon_sym_match] = ACTIONS(3840),
    [anon_sym_mod] = ACTIONS(3840),
    [anon_sym_pub] = ACTIONS(3840),
    [anon_sym_return] = ACTIONS(3840),
    [anon_sym_raw] = ACTIONS(3840),
    [anon_sym_static] = ACTIONS(3840),
    [anon_sym_struct] = ACTIONS(3840),
    [anon_sym_trait] = ACTIONS(3840),
    [anon_sym_type] = ACTIONS(3840),
    [anon_sym_union] = ACTIONS(3840),
    [anon_sym_unsafe] = ACTIONS(3840),
    [anon_sym_use] = ACTIONS(3840),
    [anon_sym_while] = ACTIONS(3840),
    [anon_sym_extern] = ACTIONS(3840),
    [anon_sym_safe] = ACTIONS(3840),
    [anon_sym_yield] = ACTIONS(3840),
    [anon_sym_move] = ACTIONS(3840),
    [anon_sym_try] = ACTIONS(3840),
    [sym_integer_literal] = ACTIONS(3838),
    [aux_sym_string_literal_token1] = ACTIONS(3838),
    [sym_char_literal] = ACTIONS(3838),
    [anon_sym_true] = ACTIONS(3840),
    [anon_sym_false] = ACTIONS(3840),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3840),
    [sym_super] = ACTIONS(3840),
    [sym_crate] = ACTIONS(3840),
    [aux_sym_metavariable_token1] = ACTIONS(3838),
    [sym__raw_string_literal_start] = ACTIONS(3838),
    [sym_float_literal] = ACTIONS(3838),
    [sym__outer_block_doc_comment_start] = ACTIONS(3838),
    [sym__outer_line_doc_comment_start] = ACTIONS(3838),
  },
  [STATE(1018)] = {
    [sym_line_comment] = STATE(1018),
    [sym_block_comment] = STATE(1018),
    [ts_builtin_sym_end] = ACTIONS(3842),
    [sym_identifier] = ACTIONS(3844),
    [anon_sym_SEMI] = ACTIONS(3842),
    [anon_sym_macro] = ACTIONS(3844),
    [anon_sym_LPAREN] = ACTIONS(3842),
    [anon_sym_LBRACE] = ACTIONS(3842),
    [anon_sym_RBRACE] = ACTIONS(3842),
    [anon_sym_macro_rules] = ACTIONS(3844),
    [anon_sym_BANG] = ACTIONS(3842),
    [anon_sym_LBRACK] = ACTIONS(3842),
    [anon_sym_STAR] = ACTIONS(3842),
    [anon_sym_u8] = ACTIONS(3844),
    [anon_sym_i8] = ACTIONS(3844),
    [anon_sym_u16] = ACTIONS(3844),
    [anon_sym_i16] = ACTIONS(3844),
    [anon_sym_u32] = ACTIONS(3844),
    [anon_sym_i32] = ACTIONS(3844),
    [anon_sym_u64] = ACTIONS(3844),
    [anon_sym_i64] = ACTIONS(3844),
    [anon_sym_u128] = ACTIONS(3844),
    [anon_sym_i128] = ACTIONS(3844),
    [anon_sym_isize] = ACTIONS(3844),
    [anon_sym_usize] = ACTIONS(3844),
    [anon_sym_f32] = ACTIONS(3844),
    [anon_sym_f64] = ACTIONS(3844),
    [anon_sym_bool] = ACTIONS(3844),
    [anon_sym_str] = ACTIONS(3844),
    [anon_sym_char] = ACTIONS(3844),
    [anon_sym_DASH] = ACTIONS(3842),
    [anon_sym_AMP] = ACTIONS(3842),
    [anon_sym_PIPE] = ACTIONS(3842),
    [anon_sym_LT] = ACTIONS(3842),
    [anon_sym_DOT_DOT] = ACTIONS(3842),
    [anon_sym_COLON_COLON] = ACTIONS(3842),
    [anon_sym_POUND] = ACTIONS(3842),
    [anon_sym_SQUOTE] = ACTIONS(3844),
    [anon_sym_async] = ACTIONS(3844),
    [anon_sym_auto] = ACTIONS(3844),
    [anon_sym_await] = ACTIONS(3844),
    [anon_sym_become] = ACTIONS(3844),
    [anon_sym_break] = ACTIONS(3844),
    [anon_sym_const] = ACTIONS(3844),
    [anon_sym_continue] = ACTIONS(3844),
    [anon_sym_default] = ACTIONS(3844),
    [anon_sym_enum] = ACTIONS(3844),
    [anon_sym_fn] = ACTIONS(3844),
    [anon_sym_for] = ACTIONS(3844),
    [anon_sym_gen] = ACTIONS(3844),
    [anon_sym_if] = ACTIONS(3844),
    [anon_sym_impl] = ACTIONS(3844),
    [anon_sym_let] = ACTIONS(3844),
    [anon_sym_loop] = ACTIONS(3844),
    [anon_sym_match] = ACTIONS(3844),
    [anon_sym_mod] = ACTIONS(3844),
    [anon_sym_pub] = ACTIONS(3844),
    [anon_sym_return] = ACTIONS(3844),
    [anon_sym_raw] = ACTIONS(3844),
    [anon_sym_static] = ACTIONS(3844),
    [anon_sym_struct] = ACTIONS(3844),
    [anon_sym_trait] = ACTIONS(3844),
    [anon_sym_type] = ACTIONS(3844),
    [anon_sym_union] = ACTIONS(3844),
    [anon_sym_unsafe] = ACTIONS(3844),
    [anon_sym_use] = ACTIONS(3844),
    [anon_sym_while] = ACTIONS(3844),
    [anon_sym_extern] = ACTIONS(3844),
    [anon_sym_safe] = ACTIONS(3844),
    [anon_sym_yield] = ACTIONS(3844),
    [anon_sym_move] = ACTIONS(3844),
    [anon_sym_try] = ACTIONS(3844),
    [sym_integer_literal] = ACTIONS(3842),
    [aux_sym_string_literal_token1] = ACTIONS(3842),
    [sym_char_literal] = ACTIONS(3842),
    [anon_sym_true] = ACTIONS(3844),
    [anon_sym_false] = ACTIONS(3844),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3844),
    [sym_super] = ACTIONS(3844),
    [sym_crate] = ACTIONS(3844),
    [aux_sym_metavariable_token1] = ACTIONS(3842),
    [sym__raw_string_literal_start] = ACTIONS(3842),
    [sym_float_literal] = ACTIONS(3842),
    [sym__outer_block_doc_comment_start] = ACTIONS(3842),
    [sym__outer_line_doc_comment_start] = ACTIONS(3842),
  },
  [STATE(1019)] = {
    [sym_line_comment] = STATE(1019),
    [sym_block_comment] = STATE(1019),
    [ts_builtin_sym_end] = ACTIONS(3846),
    [sym_identifier] = ACTIONS(3848),
    [anon_sym_SEMI] = ACTIONS(3846),
    [anon_sym_macro] = ACTIONS(3848),
    [anon_sym_LPAREN] = ACTIONS(3846),
    [anon_sym_LBRACE] = ACTIONS(3846),
    [anon_sym_RBRACE] = ACTIONS(3846),
    [anon_sym_macro_rules] = ACTIONS(3848),
    [anon_sym_BANG] = ACTIONS(3846),
    [anon_sym_LBRACK] = ACTIONS(3846),
    [anon_sym_STAR] = ACTIONS(3846),
    [anon_sym_u8] = ACTIONS(3848),
    [anon_sym_i8] = ACTIONS(3848),
    [anon_sym_u16] = ACTIONS(3848),
    [anon_sym_i16] = ACTIONS(3848),
    [anon_sym_u32] = ACTIONS(3848),
    [anon_sym_i32] = ACTIONS(3848),
    [anon_sym_u64] = ACTIONS(3848),
    [anon_sym_i64] = ACTIONS(3848),
    [anon_sym_u128] = ACTIONS(3848),
    [anon_sym_i128] = ACTIONS(3848),
    [anon_sym_isize] = ACTIONS(3848),
    [anon_sym_usize] = ACTIONS(3848),
    [anon_sym_f32] = ACTIONS(3848),
    [anon_sym_f64] = ACTIONS(3848),
    [anon_sym_bool] = ACTIONS(3848),
    [anon_sym_str] = ACTIONS(3848),
    [anon_sym_char] = ACTIONS(3848),
    [anon_sym_DASH] = ACTIONS(3846),
    [anon_sym_AMP] = ACTIONS(3846),
    [anon_sym_PIPE] = ACTIONS(3846),
    [anon_sym_LT] = ACTIONS(3846),
    [anon_sym_DOT_DOT] = ACTIONS(3846),
    [anon_sym_COLON_COLON] = ACTIONS(3846),
    [anon_sym_POUND] = ACTIONS(3846),
    [anon_sym_SQUOTE] = ACTIONS(3848),
    [anon_sym_async] = ACTIONS(3848),
    [anon_sym_auto] = ACTIONS(3848),
    [anon_sym_await] = ACTIONS(3848),
    [anon_sym_become] = ACTIONS(3848),
    [anon_sym_break] = ACTIONS(3848),
    [anon_sym_const] = ACTIONS(3848),
    [anon_sym_continue] = ACTIONS(3848),
    [anon_sym_default] = ACTIONS(3848),
    [anon_sym_enum] = ACTIONS(3848),
    [anon_sym_fn] = ACTIONS(3848),
    [anon_sym_for] = ACTIONS(3848),
    [anon_sym_gen] = ACTIONS(3848),
    [anon_sym_if] = ACTIONS(3848),
    [anon_sym_impl] = ACTIONS(3848),
    [anon_sym_let] = ACTIONS(3848),
    [anon_sym_loop] = ACTIONS(3848),
    [anon_sym_match] = ACTIONS(3848),
    [anon_sym_mod] = ACTIONS(3848),
    [anon_sym_pub] = ACTIONS(3848),
    [anon_sym_return] = ACTIONS(3848),
    [anon_sym_raw] = ACTIONS(3848),
    [anon_sym_static] = ACTIONS(3848),
    [anon_sym_struct] = ACTIONS(3848),
    [anon_sym_trait] = ACTIONS(3848),
    [anon_sym_type] = ACTIONS(3848),
    [anon_sym_union] = ACTIONS(3848),
    [anon_sym_unsafe] = ACTIONS(3848),
    [anon_sym_use] = ACTIONS(3848),
    [anon_sym_while] = ACTIONS(3848),
    [anon_sym_extern] = ACTIONS(3848),
    [anon_sym_safe] = ACTIONS(3848),
    [anon_sym_yield] = ACTIONS(3848),
    [anon_sym_move] = ACTIONS(3848),
    [anon_sym_try] = ACTIONS(3848),
    [sym_integer_literal] = ACTIONS(3846),
    [aux_sym_string_literal_token1] = ACTIONS(3846),
    [sym_char_literal] = ACTIONS(3846),
    [anon_sym_true] = ACTIONS(3848),
    [anon_sym_false] = ACTIONS(3848),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3848),
    [sym_super] = ACTIONS(3848),
    [sym_crate] = ACTIONS(3848),
    [aux_sym_metavariable_token1] = ACTIONS(3846),
    [sym__raw_string_literal_start] = ACTIONS(3846),
    [sym_float_literal] = ACTIONS(3846),
    [sym__outer_block_doc_comment_start] = ACTIONS(3846),
    [sym__outer_line_doc_comment_start] = ACTIONS(3846),
  },
  [STATE(1020)] = {
    [sym_line_comment] = STATE(1020),
    [sym_block_comment] = STATE(1020),
    [ts_builtin_sym_end] = ACTIONS(3850),
    [sym_identifier] = ACTIONS(3852),
    [anon_sym_SEMI] = ACTIONS(3850),
    [anon_sym_macro] = ACTIONS(3852),
    [anon_sym_LPAREN] = ACTIONS(3850),
    [anon_sym_LBRACE] = ACTIONS(3850),
    [anon_sym_RBRACE] = ACTIONS(3850),
    [anon_sym_macro_rules] = ACTIONS(3852),
    [anon_sym_BANG] = ACTIONS(3850),
    [anon_sym_LBRACK] = ACTIONS(3850),
    [anon_sym_STAR] = ACTIONS(3850),
    [anon_sym_u8] = ACTIONS(3852),
    [anon_sym_i8] = ACTIONS(3852),
    [anon_sym_u16] = ACTIONS(3852),
    [anon_sym_i16] = ACTIONS(3852),
    [anon_sym_u32] = ACTIONS(3852),
    [anon_sym_i32] = ACTIONS(3852),
    [anon_sym_u64] = ACTIONS(3852),
    [anon_sym_i64] = ACTIONS(3852),
    [anon_sym_u128] = ACTIONS(3852),
    [anon_sym_i128] = ACTIONS(3852),
    [anon_sym_isize] = ACTIONS(3852),
    [anon_sym_usize] = ACTIONS(3852),
    [anon_sym_f32] = ACTIONS(3852),
    [anon_sym_f64] = ACTIONS(3852),
    [anon_sym_bool] = ACTIONS(3852),
    [anon_sym_str] = ACTIONS(3852),
    [anon_sym_char] = ACTIONS(3852),
    [anon_sym_DASH] = ACTIONS(3850),
    [anon_sym_AMP] = ACTIONS(3850),
    [anon_sym_PIPE] = ACTIONS(3850),
    [anon_sym_LT] = ACTIONS(3850),
    [anon_sym_DOT_DOT] = ACTIONS(3850),
    [anon_sym_COLON_COLON] = ACTIONS(3850),
    [anon_sym_POUND] = ACTIONS(3850),
    [anon_sym_SQUOTE] = ACTIONS(3852),
    [anon_sym_async] = ACTIONS(3852),
    [anon_sym_auto] = ACTIONS(3852),
    [anon_sym_await] = ACTIONS(3852),
    [anon_sym_become] = ACTIONS(3852),
    [anon_sym_break] = ACTIONS(3852),
    [anon_sym_const] = ACTIONS(3852),
    [anon_sym_continue] = ACTIONS(3852),
    [anon_sym_default] = ACTIONS(3852),
    [anon_sym_enum] = ACTIONS(3852),
    [anon_sym_fn] = ACTIONS(3852),
    [anon_sym_for] = ACTIONS(3852),
    [anon_sym_gen] = ACTIONS(3852),
    [anon_sym_if] = ACTIONS(3852),
    [anon_sym_impl] = ACTIONS(3852),
    [anon_sym_let] = ACTIONS(3852),
    [anon_sym_loop] = ACTIONS(3852),
    [anon_sym_match] = ACTIONS(3852),
    [anon_sym_mod] = ACTIONS(3852),
    [anon_sym_pub] = ACTIONS(3852),
    [anon_sym_return] = ACTIONS(3852),
    [anon_sym_raw] = ACTIONS(3852),
    [anon_sym_static] = ACTIONS(3852),
    [anon_sym_struct] = ACTIONS(3852),
    [anon_sym_trait] = ACTIONS(3852),
    [anon_sym_type] = ACTIONS(3852),
    [anon_sym_union] = ACTIONS(3852),
    [anon_sym_unsafe] = ACTIONS(3852),
    [anon_sym_use] = ACTIONS(3852),
    [anon_sym_while] = ACTIONS(3852),
    [anon_sym_extern] = ACTIONS(3852),
    [anon_sym_safe] = ACTIONS(3852),
    [anon_sym_yield] = ACTIONS(3852),
    [anon_sym_move] = ACTIONS(3852),
    [anon_sym_try] = ACTIONS(3852),
    [sym_integer_literal] = ACTIONS(3850),
    [aux_sym_string_literal_token1] = ACTIONS(3850),
    [sym_char_literal] = ACTIONS(3850),
    [anon_sym_true] = ACTIONS(3852),
    [anon_sym_false] = ACTIONS(3852),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3852),
    [sym_super] = ACTIONS(3852),
    [sym_crate] = ACTIONS(3852),
    [aux_sym_metavariable_token1] = ACTIONS(3850),
    [sym__raw_string_literal_start] = ACTIONS(3850),
    [sym_float_literal] = ACTIONS(3850),
    [sym__outer_block_doc_comment_start] = ACTIONS(3850),
    [sym__outer_line_doc_comment_start] = ACTIONS(3850),
  },
  [STATE(1021)] = {
    [sym_line_comment] = STATE(1021),
    [sym_block_comment] = STATE(1021),
    [ts_builtin_sym_end] = ACTIONS(3854),
    [sym_identifier] = ACTIONS(3856),
    [anon_sym_SEMI] = ACTIONS(3854),
    [anon_sym_macro] = ACTIONS(3856),
    [anon_sym_LPAREN] = ACTIONS(3854),
    [anon_sym_LBRACE] = ACTIONS(3854),
    [anon_sym_RBRACE] = ACTIONS(3854),
    [anon_sym_macro_rules] = ACTIONS(3856),
    [anon_sym_BANG] = ACTIONS(3854),
    [anon_sym_LBRACK] = ACTIONS(3854),
    [anon_sym_STAR] = ACTIONS(3854),
    [anon_sym_u8] = ACTIONS(3856),
    [anon_sym_i8] = ACTIONS(3856),
    [anon_sym_u16] = ACTIONS(3856),
    [anon_sym_i16] = ACTIONS(3856),
    [anon_sym_u32] = ACTIONS(3856),
    [anon_sym_i32] = ACTIONS(3856),
    [anon_sym_u64] = ACTIONS(3856),
    [anon_sym_i64] = ACTIONS(3856),
    [anon_sym_u128] = ACTIONS(3856),
    [anon_sym_i128] = ACTIONS(3856),
    [anon_sym_isize] = ACTIONS(3856),
    [anon_sym_usize] = ACTIONS(3856),
    [anon_sym_f32] = ACTIONS(3856),
    [anon_sym_f64] = ACTIONS(3856),
    [anon_sym_bool] = ACTIONS(3856),
    [anon_sym_str] = ACTIONS(3856),
    [anon_sym_char] = ACTIONS(3856),
    [anon_sym_DASH] = ACTIONS(3854),
    [anon_sym_AMP] = ACTIONS(3854),
    [anon_sym_PIPE] = ACTIONS(3854),
    [anon_sym_LT] = ACTIONS(3854),
    [anon_sym_DOT_DOT] = ACTIONS(3854),
    [anon_sym_COLON_COLON] = ACTIONS(3854),
    [anon_sym_POUND] = ACTIONS(3854),
    [anon_sym_SQUOTE] = ACTIONS(3856),
    [anon_sym_async] = ACTIONS(3856),
    [anon_sym_auto] = ACTIONS(3856),
    [anon_sym_await] = ACTIONS(3856),
    [anon_sym_become] = ACTIONS(3856),
    [anon_sym_break] = ACTIONS(3856),
    [anon_sym_const] = ACTIONS(3856),
    [anon_sym_continue] = ACTIONS(3856),
    [anon_sym_default] = ACTIONS(3856),
    [anon_sym_enum] = ACTIONS(3856),
    [anon_sym_fn] = ACTIONS(3856),
    [anon_sym_for] = ACTIONS(3856),
    [anon_sym_gen] = ACTIONS(3856),
    [anon_sym_if] = ACTIONS(3856),
    [anon_sym_impl] = ACTIONS(3856),
    [anon_sym_let] = ACTIONS(3856),
    [anon_sym_loop] = ACTIONS(3856),
    [anon_sym_match] = ACTIONS(3856),
    [anon_sym_mod] = ACTIONS(3856),
    [anon_sym_pub] = ACTIONS(3856),
    [anon_sym_return] = ACTIONS(3856),
    [anon_sym_raw] = ACTIONS(3856),
    [anon_sym_static] = ACTIONS(3856),
    [anon_sym_struct] = ACTIONS(3856),
    [anon_sym_trait] = ACTIONS(3856),
    [anon_sym_type] = ACTIONS(3856),
    [anon_sym_union] = ACTIONS(3856),
    [anon_sym_unsafe] = ACTIONS(3856),
    [anon_sym_use] = ACTIONS(3856),
    [anon_sym_while] = ACTIONS(3856),
    [anon_sym_extern] = ACTIONS(3856),
    [anon_sym_safe] = ACTIONS(3856),
    [anon_sym_yield] = ACTIONS(3856),
    [anon_sym_move] = ACTIONS(3856),
    [anon_sym_try] = ACTIONS(3856),
    [sym_integer_literal] = ACTIONS(3854),
    [aux_sym_string_literal_token1] = ACTIONS(3854),
    [sym_char_literal] = ACTIONS(3854),
    [anon_sym_true] = ACTIONS(3856),
    [anon_sym_false] = ACTIONS(3856),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3856),
    [sym_super] = ACTIONS(3856),
    [sym_crate] = ACTIONS(3856),
    [aux_sym_metavariable_token1] = ACTIONS(3854),
    [sym__raw_string_literal_start] = ACTIONS(3854),
    [sym_float_literal] = ACTIONS(3854),
    [sym__outer_block_doc_comment_start] = ACTIONS(3854),
    [sym__outer_line_doc_comment_start] = ACTIONS(3854),
  },
  [STATE(1022)] = {
    [sym_line_comment] = STATE(1022),
    [sym_block_comment] = STATE(1022),
    [ts_builtin_sym_end] = ACTIONS(3858),
    [sym_identifier] = ACTIONS(3860),
    [anon_sym_SEMI] = ACTIONS(3858),
    [anon_sym_macro] = ACTIONS(3860),
    [anon_sym_LPAREN] = ACTIONS(3858),
    [anon_sym_LBRACE] = ACTIONS(3858),
    [anon_sym_RBRACE] = ACTIONS(3858),
    [anon_sym_macro_rules] = ACTIONS(3860),
    [anon_sym_BANG] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3858),
    [anon_sym_STAR] = ACTIONS(3858),
    [anon_sym_u8] = ACTIONS(3860),
    [anon_sym_i8] = ACTIONS(3860),
    [anon_sym_u16] = ACTIONS(3860),
    [anon_sym_i16] = ACTIONS(3860),
    [anon_sym_u32] = ACTIONS(3860),
    [anon_sym_i32] = ACTIONS(3860),
    [anon_sym_u64] = ACTIONS(3860),
    [anon_sym_i64] = ACTIONS(3860),
    [anon_sym_u128] = ACTIONS(3860),
    [anon_sym_i128] = ACTIONS(3860),
    [anon_sym_isize] = ACTIONS(3860),
    [anon_sym_usize] = ACTIONS(3860),
    [anon_sym_f32] = ACTIONS(3860),
    [anon_sym_f64] = ACTIONS(3860),
    [anon_sym_bool] = ACTIONS(3860),
    [anon_sym_str] = ACTIONS(3860),
    [anon_sym_char] = ACTIONS(3860),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_AMP] = ACTIONS(3858),
    [anon_sym_PIPE] = ACTIONS(3858),
    [anon_sym_LT] = ACTIONS(3858),
    [anon_sym_DOT_DOT] = ACTIONS(3858),
    [anon_sym_COLON_COLON] = ACTIONS(3858),
    [anon_sym_POUND] = ACTIONS(3858),
    [anon_sym_SQUOTE] = ACTIONS(3860),
    [anon_sym_async] = ACTIONS(3860),
    [anon_sym_auto] = ACTIONS(3860),
    [anon_sym_await] = ACTIONS(3860),
    [anon_sym_become] = ACTIONS(3860),
    [anon_sym_break] = ACTIONS(3860),
    [anon_sym_const] = ACTIONS(3860),
    [anon_sym_continue] = ACTIONS(3860),
    [anon_sym_default] = ACTIONS(3860),
    [anon_sym_enum] = ACTIONS(3860),
    [anon_sym_fn] = ACTIONS(3860),
    [anon_sym_for] = ACTIONS(3860),
    [anon_sym_gen] = ACTIONS(3860),
    [anon_sym_if] = ACTIONS(3860),
    [anon_sym_impl] = ACTIONS(3860),
    [anon_sym_let] = ACTIONS(3860),
    [anon_sym_loop] = ACTIONS(3860),
    [anon_sym_match] = ACTIONS(3860),
    [anon_sym_mod] = ACTIONS(3860),
    [anon_sym_pub] = ACTIONS(3860),
    [anon_sym_return] = ACTIONS(3860),
    [anon_sym_raw] = ACTIONS(3860),
    [anon_sym_static] = ACTIONS(3860),
    [anon_sym_struct] = ACTIONS(3860),
    [anon_sym_trait] = ACTIONS(3860),
    [anon_sym_type] = ACTIONS(3860),
    [anon_sym_union] = ACTIONS(3860),
    [anon_sym_unsafe] = ACTIONS(3860),
    [anon_sym_use] = ACTIONS(3860),
    [anon_sym_while] = ACTIONS(3860),
    [anon_sym_extern] = ACTIONS(3860),
    [anon_sym_safe] = ACTIONS(3860),
    [anon_sym_yield] = ACTIONS(3860),
    [anon_sym_move] = ACTIONS(3860),
    [anon_sym_try] = ACTIONS(3860),
    [sym_integer_literal] = ACTIONS(3858),
    [aux_sym_string_literal_token1] = ACTIONS(3858),
    [sym_char_literal] = ACTIONS(3858),
    [anon_sym_true] = ACTIONS(3860),
    [anon_sym_false] = ACTIONS(3860),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3860),
    [sym_super] = ACTIONS(3860),
    [sym_crate] = ACTIONS(3860),
    [aux_sym_metavariable_token1] = ACTIONS(3858),
    [sym__raw_string_literal_start] = ACTIONS(3858),
    [sym_float_literal] = ACTIONS(3858),
    [sym__outer_block_doc_comment_start] = ACTIONS(3858),
    [sym__outer_line_doc_comment_start] = ACTIONS(3858),
  },
  [STATE(1023)] = {
    [sym_line_comment] = STATE(1023),
    [sym_block_comment] = STATE(1023),
    [ts_builtin_sym_end] = ACTIONS(3862),
    [sym_identifier] = ACTIONS(3864),
    [anon_sym_SEMI] = ACTIONS(3862),
    [anon_sym_macro] = ACTIONS(3864),
    [anon_sym_LPAREN] = ACTIONS(3862),
    [anon_sym_LBRACE] = ACTIONS(3862),
    [anon_sym_RBRACE] = ACTIONS(3862),
    [anon_sym_macro_rules] = ACTIONS(3864),
    [anon_sym_BANG] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3862),
    [anon_sym_STAR] = ACTIONS(3862),
    [anon_sym_u8] = ACTIONS(3864),
    [anon_sym_i8] = ACTIONS(3864),
    [anon_sym_u16] = ACTIONS(3864),
    [anon_sym_i16] = ACTIONS(3864),
    [anon_sym_u32] = ACTIONS(3864),
    [anon_sym_i32] = ACTIONS(3864),
    [anon_sym_u64] = ACTIONS(3864),
    [anon_sym_i64] = ACTIONS(3864),
    [anon_sym_u128] = ACTIONS(3864),
    [anon_sym_i128] = ACTIONS(3864),
    [anon_sym_isize] = ACTIONS(3864),
    [anon_sym_usize] = ACTIONS(3864),
    [anon_sym_f32] = ACTIONS(3864),
    [anon_sym_f64] = ACTIONS(3864),
    [anon_sym_bool] = ACTIONS(3864),
    [anon_sym_str] = ACTIONS(3864),
    [anon_sym_char] = ACTIONS(3864),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_AMP] = ACTIONS(3862),
    [anon_sym_PIPE] = ACTIONS(3862),
    [anon_sym_LT] = ACTIONS(3862),
    [anon_sym_DOT_DOT] = ACTIONS(3862),
    [anon_sym_COLON_COLON] = ACTIONS(3862),
    [anon_sym_POUND] = ACTIONS(3862),
    [anon_sym_SQUOTE] = ACTIONS(3864),
    [anon_sym_async] = ACTIONS(3864),
    [anon_sym_auto] = ACTIONS(3864),
    [anon_sym_await] = ACTIONS(3864),
    [anon_sym_become] = ACTIONS(3864),
    [anon_sym_break] = ACTIONS(3864),
    [anon_sym_const] = ACTIONS(3864),
    [anon_sym_continue] = ACTIONS(3864),
    [anon_sym_default] = ACTIONS(3864),
    [anon_sym_enum] = ACTIONS(3864),
    [anon_sym_fn] = ACTIONS(3864),
    [anon_sym_for] = ACTIONS(3864),
    [anon_sym_gen] = ACTIONS(3864),
    [anon_sym_if] = ACTIONS(3864),
    [anon_sym_impl] = ACTIONS(3864),
    [anon_sym_let] = ACTIONS(3864),
    [anon_sym_loop] = ACTIONS(3864),
    [anon_sym_match] = ACTIONS(3864),
    [anon_sym_mod] = ACTIONS(3864),
    [anon_sym_pub] = ACTIONS(3864),
    [anon_sym_return] = ACTIONS(3864),
    [anon_sym_raw] = ACTIONS(3864),
    [anon_sym_static] = ACTIONS(3864),
    [anon_sym_struct] = ACTIONS(3864),
    [anon_sym_trait] = ACTIONS(3864),
    [anon_sym_type] = ACTIONS(3864),
    [anon_sym_union] = ACTIONS(3864),
    [anon_sym_unsafe] = ACTIONS(3864),
    [anon_sym_use] = ACTIONS(3864),
    [anon_sym_while] = ACTIONS(3864),
    [anon_sym_extern] = ACTIONS(3864),
    [anon_sym_safe] = ACTIONS(3864),
    [anon_sym_yield] = ACTIONS(3864),
    [anon_sym_move] = ACTIONS(3864),
    [anon_sym_try] = ACTIONS(3864),
    [sym_integer_literal] = ACTIONS(3862),
    [aux_sym_string_literal_token1] = ACTIONS(3862),
    [sym_char_literal] = ACTIONS(3862),
    [anon_sym_true] = ACTIONS(3864),
    [anon_sym_false] = ACTIONS(3864),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3864),
    [sym_super] = ACTIONS(3864),
    [sym_crate] = ACTIONS(3864),
    [aux_sym_metavariable_token1] = ACTIONS(3862),
    [sym__raw_string_literal_start] = ACTIONS(3862),
    [sym_float_literal] = ACTIONS(3862),
    [sym__outer_block_doc_comment_start] = ACTIONS(3862),
    [sym__outer_line_doc_comment_start] = ACTIONS(3862),
  },
  [STATE(1024)] = {
    [sym_line_comment] = STATE(1024),
    [sym_block_comment] = STATE(1024),
    [ts_builtin_sym_end] = ACTIONS(3866),
    [sym_identifier] = ACTIONS(3868),
    [anon_sym_SEMI] = ACTIONS(3866),
    [anon_sym_macro] = ACTIONS(3868),
    [anon_sym_LPAREN] = ACTIONS(3866),
    [anon_sym_LBRACE] = ACTIONS(3866),
    [anon_sym_RBRACE] = ACTIONS(3866),
    [anon_sym_macro_rules] = ACTIONS(3868),
    [anon_sym_BANG] = ACTIONS(3866),
    [anon_sym_LBRACK] = ACTIONS(3866),
    [anon_sym_STAR] = ACTIONS(3866),
    [anon_sym_u8] = ACTIONS(3868),
    [anon_sym_i8] = ACTIONS(3868),
    [anon_sym_u16] = ACTIONS(3868),
    [anon_sym_i16] = ACTIONS(3868),
    [anon_sym_u32] = ACTIONS(3868),
    [anon_sym_i32] = ACTIONS(3868),
    [anon_sym_u64] = ACTIONS(3868),
    [anon_sym_i64] = ACTIONS(3868),
    [anon_sym_u128] = ACTIONS(3868),
    [anon_sym_i128] = ACTIONS(3868),
    [anon_sym_isize] = ACTIONS(3868),
    [anon_sym_usize] = ACTIONS(3868),
    [anon_sym_f32] = ACTIONS(3868),
    [anon_sym_f64] = ACTIONS(3868),
    [anon_sym_bool] = ACTIONS(3868),
    [anon_sym_str] = ACTIONS(3868),
    [anon_sym_char] = ACTIONS(3868),
    [anon_sym_DASH] = ACTIONS(3866),
    [anon_sym_AMP] = ACTIONS(3866),
    [anon_sym_PIPE] = ACTIONS(3866),
    [anon_sym_LT] = ACTIONS(3866),
    [anon_sym_DOT_DOT] = ACTIONS(3866),
    [anon_sym_COLON_COLON] = ACTIONS(3866),
    [anon_sym_POUND] = ACTIONS(3866),
    [anon_sym_SQUOTE] = ACTIONS(3868),
    [anon_sym_async] = ACTIONS(3868),
    [anon_sym_auto] = ACTIONS(3868),
    [anon_sym_await] = ACTIONS(3868),
    [anon_sym_become] = ACTIONS(3868),
    [anon_sym_break] = ACTIONS(3868),
    [anon_sym_const] = ACTIONS(3868),
    [anon_sym_continue] = ACTIONS(3868),
    [anon_sym_default] = ACTIONS(3868),
    [anon_sym_enum] = ACTIONS(3868),
    [anon_sym_fn] = ACTIONS(3868),
    [anon_sym_for] = ACTIONS(3868),
    [anon_sym_gen] = ACTIONS(3868),
    [anon_sym_if] = ACTIONS(3868),
    [anon_sym_impl] = ACTIONS(3868),
    [anon_sym_let] = ACTIONS(3868),
    [anon_sym_loop] = ACTIONS(3868),
    [anon_sym_match] = ACTIONS(3868),
    [anon_sym_mod] = ACTIONS(3868),
    [anon_sym_pub] = ACTIONS(3868),
    [anon_sym_return] = ACTIONS(3868),
    [anon_sym_raw] = ACTIONS(3868),
    [anon_sym_static] = ACTIONS(3868),
    [anon_sym_struct] = ACTIONS(3868),
    [anon_sym_trait] = ACTIONS(3868),
    [anon_sym_type] = ACTIONS(3868),
    [anon_sym_union] = ACTIONS(3868),
    [anon_sym_unsafe] = ACTIONS(3868),
    [anon_sym_use] = ACTIONS(3868),
    [anon_sym_while] = ACTIONS(3868),
    [anon_sym_extern] = ACTIONS(3868),
    [anon_sym_safe] = ACTIONS(3868),
    [anon_sym_yield] = ACTIONS(3868),
    [anon_sym_move] = ACTIONS(3868),
    [anon_sym_try] = ACTIONS(3868),
    [sym_integer_literal] = ACTIONS(3866),
    [aux_sym_string_literal_token1] = ACTIONS(3866),
    [sym_char_literal] = ACTIONS(3866),
    [anon_sym_true] = ACTIONS(3868),
    [anon_sym_false] = ACTIONS(3868),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3868),
    [sym_super] = ACTIONS(3868),
    [sym_crate] = ACTIONS(3868),
    [aux_sym_metavariable_token1] = ACTIONS(3866),
    [sym__raw_string_literal_start] = ACTIONS(3866),
    [sym_float_literal] = ACTIONS(3866),
    [sym__outer_block_doc_comment_start] = ACTIONS(3866),
    [sym__outer_line_doc_comment_start] = ACTIONS(3866),
  },
  [STATE(1025)] = {
    [sym_line_comment] = STATE(1025),
    [sym_block_comment] = STATE(1025),
    [ts_builtin_sym_end] = ACTIONS(3870),
    [sym_identifier] = ACTIONS(3872),
    [anon_sym_SEMI] = ACTIONS(3870),
    [anon_sym_macro] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(3870),
    [anon_sym_LBRACE] = ACTIONS(3870),
    [anon_sym_RBRACE] = ACTIONS(3870),
    [anon_sym_macro_rules] = ACTIONS(3872),
    [anon_sym_BANG] = ACTIONS(3870),
    [anon_sym_LBRACK] = ACTIONS(3870),
    [anon_sym_STAR] = ACTIONS(3870),
    [anon_sym_u8] = ACTIONS(3872),
    [anon_sym_i8] = ACTIONS(3872),
    [anon_sym_u16] = ACTIONS(3872),
    [anon_sym_i16] = ACTIONS(3872),
    [anon_sym_u32] = ACTIONS(3872),
    [anon_sym_i32] = ACTIONS(3872),
    [anon_sym_u64] = ACTIONS(3872),
    [anon_sym_i64] = ACTIONS(3872),
    [anon_sym_u128] = ACTIONS(3872),
    [anon_sym_i128] = ACTIONS(3872),
    [anon_sym_isize] = ACTIONS(3872),
    [anon_sym_usize] = ACTIONS(3872),
    [anon_sym_f32] = ACTIONS(3872),
    [anon_sym_f64] = ACTIONS(3872),
    [anon_sym_bool] = ACTIONS(3872),
    [anon_sym_str] = ACTIONS(3872),
    [anon_sym_char] = ACTIONS(3872),
    [anon_sym_DASH] = ACTIONS(3870),
    [anon_sym_AMP] = ACTIONS(3870),
    [anon_sym_PIPE] = ACTIONS(3870),
    [anon_sym_LT] = ACTIONS(3870),
    [anon_sym_DOT_DOT] = ACTIONS(3870),
    [anon_sym_COLON_COLON] = ACTIONS(3870),
    [anon_sym_POUND] = ACTIONS(3870),
    [anon_sym_SQUOTE] = ACTIONS(3872),
    [anon_sym_async] = ACTIONS(3872),
    [anon_sym_auto] = ACTIONS(3872),
    [anon_sym_await] = ACTIONS(3872),
    [anon_sym_become] = ACTIONS(3872),
    [anon_sym_break] = ACTIONS(3872),
    [anon_sym_const] = ACTIONS(3872),
    [anon_sym_continue] = ACTIONS(3872),
    [anon_sym_default] = ACTIONS(3872),
    [anon_sym_enum] = ACTIONS(3872),
    [anon_sym_fn] = ACTIONS(3872),
    [anon_sym_for] = ACTIONS(3872),
    [anon_sym_gen] = ACTIONS(3872),
    [anon_sym_if] = ACTIONS(3872),
    [anon_sym_impl] = ACTIONS(3872),
    [anon_sym_let] = ACTIONS(3872),
    [anon_sym_loop] = ACTIONS(3872),
    [anon_sym_match] = ACTIONS(3872),
    [anon_sym_mod] = ACTIONS(3872),
    [anon_sym_pub] = ACTIONS(3872),
    [anon_sym_return] = ACTIONS(3872),
    [anon_sym_raw] = ACTIONS(3872),
    [anon_sym_static] = ACTIONS(3872),
    [anon_sym_struct] = ACTIONS(3872),
    [anon_sym_trait] = ACTIONS(3872),
    [anon_sym_type] = ACTIONS(3872),
    [anon_sym_union] = ACTIONS(3872),
    [anon_sym_unsafe] = ACTIONS(3872),
    [anon_sym_use] = ACTIONS(3872),
    [anon_sym_while] = ACTIONS(3872),
    [anon_sym_extern] = ACTIONS(3872),
    [anon_sym_safe] = ACTIONS(3872),
    [anon_sym_yield] = ACTIONS(3872),
    [anon_sym_move] = ACTIONS(3872),
    [anon_sym_try] = ACTIONS(3872),
    [sym_integer_literal] = ACTIONS(3870),
    [aux_sym_string_literal_token1] = ACTIONS(3870),
    [sym_char_literal] = ACTIONS(3870),
    [anon_sym_true] = ACTIONS(3872),
    [anon_sym_false] = ACTIONS(3872),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3872),
    [sym_super] = ACTIONS(3872),
    [sym_crate] = ACTIONS(3872),
    [aux_sym_metavariable_token1] = ACTIONS(3870),
    [sym__raw_string_literal_start] = ACTIONS(3870),
    [sym_float_literal] = ACTIONS(3870),
    [sym__outer_block_doc_comment_start] = ACTIONS(3870),
    [sym__outer_line_doc_comment_start] = ACTIONS(3870),
  },
  [STATE(1026)] = {
    [sym_line_comment] = STATE(1026),
    [sym_block_comment] = STATE(1026),
    [ts_builtin_sym_end] = ACTIONS(3874),
    [sym_identifier] = ACTIONS(3876),
    [anon_sym_SEMI] = ACTIONS(3874),
    [anon_sym_macro] = ACTIONS(3876),
    [anon_sym_LPAREN] = ACTIONS(3874),
    [anon_sym_LBRACE] = ACTIONS(3874),
    [anon_sym_RBRACE] = ACTIONS(3874),
    [anon_sym_macro_rules] = ACTIONS(3876),
    [anon_sym_BANG] = ACTIONS(3874),
    [anon_sym_LBRACK] = ACTIONS(3874),
    [anon_sym_STAR] = ACTIONS(3874),
    [anon_sym_u8] = ACTIONS(3876),
    [anon_sym_i8] = ACTIONS(3876),
    [anon_sym_u16] = ACTIONS(3876),
    [anon_sym_i16] = ACTIONS(3876),
    [anon_sym_u32] = ACTIONS(3876),
    [anon_sym_i32] = ACTIONS(3876),
    [anon_sym_u64] = ACTIONS(3876),
    [anon_sym_i64] = ACTIONS(3876),
    [anon_sym_u128] = ACTIONS(3876),
    [anon_sym_i128] = ACTIONS(3876),
    [anon_sym_isize] = ACTIONS(3876),
    [anon_sym_usize] = ACTIONS(3876),
    [anon_sym_f32] = ACTIONS(3876),
    [anon_sym_f64] = ACTIONS(3876),
    [anon_sym_bool] = ACTIONS(3876),
    [anon_sym_str] = ACTIONS(3876),
    [anon_sym_char] = ACTIONS(3876),
    [anon_sym_DASH] = ACTIONS(3874),
    [anon_sym_AMP] = ACTIONS(3874),
    [anon_sym_PIPE] = ACTIONS(3874),
    [anon_sym_LT] = ACTIONS(3874),
    [anon_sym_DOT_DOT] = ACTIONS(3874),
    [anon_sym_COLON_COLON] = ACTIONS(3874),
    [anon_sym_POUND] = ACTIONS(3874),
    [anon_sym_SQUOTE] = ACTIONS(3876),
    [anon_sym_async] = ACTIONS(3876),
    [anon_sym_auto] = ACTIONS(3876),
    [anon_sym_await] = ACTIONS(3876),
    [anon_sym_become] = ACTIONS(3876),
    [anon_sym_break] = ACTIONS(3876),
    [anon_sym_const] = ACTIONS(3876),
    [anon_sym_continue] = ACTIONS(3876),
    [anon_sym_default] = ACTIONS(3876),
    [anon_sym_enum] = ACTIONS(3876),
    [anon_sym_fn] = ACTIONS(3876),
    [anon_sym_for] = ACTIONS(3876),
    [anon_sym_gen] = ACTIONS(3876),
    [anon_sym_if] = ACTIONS(3876),
    [anon_sym_impl] = ACTIONS(3876),
    [anon_sym_let] = ACTIONS(3876),
    [anon_sym_loop] = ACTIONS(3876),
    [anon_sym_match] = ACTIONS(3876),
    [anon_sym_mod] = ACTIONS(3876),
    [anon_sym_pub] = ACTIONS(3876),
    [anon_sym_return] = ACTIONS(3876),
    [anon_sym_raw] = ACTIONS(3876),
    [anon_sym_static] = ACTIONS(3876),
    [anon_sym_struct] = ACTIONS(3876),
    [anon_sym_trait] = ACTIONS(3876),
    [anon_sym_type] = ACTIONS(3876),
    [anon_sym_union] = ACTIONS(3876),
    [anon_sym_unsafe] = ACTIONS(3876),
    [anon_sym_use] = ACTIONS(3876),
    [anon_sym_while] = ACTIONS(3876),
    [anon_sym_extern] = ACTIONS(3876),
    [anon_sym_safe] = ACTIONS(3876),
    [anon_sym_yield] = ACTIONS(3876),
    [anon_sym_move] = ACTIONS(3876),
    [anon_sym_try] = ACTIONS(3876),
    [sym_integer_literal] = ACTIONS(3874),
    [aux_sym_string_literal_token1] = ACTIONS(3874),
    [sym_char_literal] = ACTIONS(3874),
    [anon_sym_true] = ACTIONS(3876),
    [anon_sym_false] = ACTIONS(3876),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3876),
    [sym_super] = ACTIONS(3876),
    [sym_crate] = ACTIONS(3876),
    [aux_sym_metavariable_token1] = ACTIONS(3874),
    [sym__raw_string_literal_start] = ACTIONS(3874),
    [sym_float_literal] = ACTIONS(3874),
    [sym__outer_block_doc_comment_start] = ACTIONS(3874),
    [sym__outer_line_doc_comment_start] = ACTIONS(3874),
  },
  [STATE(1027)] = {
    [sym_line_comment] = STATE(1027),
    [sym_block_comment] = STATE(1027),
    [ts_builtin_sym_end] = ACTIONS(3878),
    [sym_identifier] = ACTIONS(3880),
    [anon_sym_SEMI] = ACTIONS(3878),
    [anon_sym_macro] = ACTIONS(3880),
    [anon_sym_LPAREN] = ACTIONS(3878),
    [anon_sym_LBRACE] = ACTIONS(3878),
    [anon_sym_RBRACE] = ACTIONS(3878),
    [anon_sym_macro_rules] = ACTIONS(3880),
    [anon_sym_BANG] = ACTIONS(3878),
    [anon_sym_LBRACK] = ACTIONS(3878),
    [anon_sym_STAR] = ACTIONS(3878),
    [anon_sym_u8] = ACTIONS(3880),
    [anon_sym_i8] = ACTIONS(3880),
    [anon_sym_u16] = ACTIONS(3880),
    [anon_sym_i16] = ACTIONS(3880),
    [anon_sym_u32] = ACTIONS(3880),
    [anon_sym_i32] = ACTIONS(3880),
    [anon_sym_u64] = ACTIONS(3880),
    [anon_sym_i64] = ACTIONS(3880),
    [anon_sym_u128] = ACTIONS(3880),
    [anon_sym_i128] = ACTIONS(3880),
    [anon_sym_isize] = ACTIONS(3880),
    [anon_sym_usize] = ACTIONS(3880),
    [anon_sym_f32] = ACTIONS(3880),
    [anon_sym_f64] = ACTIONS(3880),
    [anon_sym_bool] = ACTIONS(3880),
    [anon_sym_str] = ACTIONS(3880),
    [anon_sym_char] = ACTIONS(3880),
    [anon_sym_DASH] = ACTIONS(3878),
    [anon_sym_AMP] = ACTIONS(3878),
    [anon_sym_PIPE] = ACTIONS(3878),
    [anon_sym_LT] = ACTIONS(3878),
    [anon_sym_DOT_DOT] = ACTIONS(3878),
    [anon_sym_COLON_COLON] = ACTIONS(3878),
    [anon_sym_POUND] = ACTIONS(3878),
    [anon_sym_SQUOTE] = ACTIONS(3880),
    [anon_sym_async] = ACTIONS(3880),
    [anon_sym_auto] = ACTIONS(3880),
    [anon_sym_await] = ACTIONS(3880),
    [anon_sym_become] = ACTIONS(3880),
    [anon_sym_break] = ACTIONS(3880),
    [anon_sym_const] = ACTIONS(3880),
    [anon_sym_continue] = ACTIONS(3880),
    [anon_sym_default] = ACTIONS(3880),
    [anon_sym_enum] = ACTIONS(3880),
    [anon_sym_fn] = ACTIONS(3880),
    [anon_sym_for] = ACTIONS(3880),
    [anon_sym_gen] = ACTIONS(3880),
    [anon_sym_if] = ACTIONS(3880),
    [anon_sym_impl] = ACTIONS(3880),
    [anon_sym_let] = ACTIONS(3880),
    [anon_sym_loop] = ACTIONS(3880),
    [anon_sym_match] = ACTIONS(3880),
    [anon_sym_mod] = ACTIONS(3880),
    [anon_sym_pub] = ACTIONS(3880),
    [anon_sym_return] = ACTIONS(3880),
    [anon_sym_raw] = ACTIONS(3880),
    [anon_sym_static] = ACTIONS(3880),
    [anon_sym_struct] = ACTIONS(3880),
    [anon_sym_trait] = ACTIONS(3880),
    [anon_sym_type] = ACTIONS(3880),
    [anon_sym_union] = ACTIONS(3880),
    [anon_sym_unsafe] = ACTIONS(3880),
    [anon_sym_use] = ACTIONS(3880),
    [anon_sym_while] = ACTIONS(3880),
    [anon_sym_extern] = ACTIONS(3880),
    [anon_sym_safe] = ACTIONS(3880),
    [anon_sym_yield] = ACTIONS(3880),
    [anon_sym_move] = ACTIONS(3880),
    [anon_sym_try] = ACTIONS(3880),
    [sym_integer_literal] = ACTIONS(3878),
    [aux_sym_string_literal_token1] = ACTIONS(3878),
    [sym_char_literal] = ACTIONS(3878),
    [anon_sym_true] = ACTIONS(3880),
    [anon_sym_false] = ACTIONS(3880),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3880),
    [sym_super] = ACTIONS(3880),
    [sym_crate] = ACTIONS(3880),
    [aux_sym_metavariable_token1] = ACTIONS(3878),
    [sym__raw_string_literal_start] = ACTIONS(3878),
    [sym_float_literal] = ACTIONS(3878),
    [sym__outer_block_doc_comment_start] = ACTIONS(3878),
    [sym__outer_line_doc_comment_start] = ACTIONS(3878),
  },
  [STATE(1028)] = {
    [sym_line_comment] = STATE(1028),
    [sym_block_comment] = STATE(1028),
    [ts_builtin_sym_end] = ACTIONS(3882),
    [sym_identifier] = ACTIONS(3884),
    [anon_sym_SEMI] = ACTIONS(3882),
    [anon_sym_macro] = ACTIONS(3884),
    [anon_sym_LPAREN] = ACTIONS(3882),
    [anon_sym_LBRACE] = ACTIONS(3882),
    [anon_sym_RBRACE] = ACTIONS(3882),
    [anon_sym_macro_rules] = ACTIONS(3884),
    [anon_sym_BANG] = ACTIONS(3882),
    [anon_sym_LBRACK] = ACTIONS(3882),
    [anon_sym_STAR] = ACTIONS(3882),
    [anon_sym_u8] = ACTIONS(3884),
    [anon_sym_i8] = ACTIONS(3884),
    [anon_sym_u16] = ACTIONS(3884),
    [anon_sym_i16] = ACTIONS(3884),
    [anon_sym_u32] = ACTIONS(3884),
    [anon_sym_i32] = ACTIONS(3884),
    [anon_sym_u64] = ACTIONS(3884),
    [anon_sym_i64] = ACTIONS(3884),
    [anon_sym_u128] = ACTIONS(3884),
    [anon_sym_i128] = ACTIONS(3884),
    [anon_sym_isize] = ACTIONS(3884),
    [anon_sym_usize] = ACTIONS(3884),
    [anon_sym_f32] = ACTIONS(3884),
    [anon_sym_f64] = ACTIONS(3884),
    [anon_sym_bool] = ACTIONS(3884),
    [anon_sym_str] = ACTIONS(3884),
    [anon_sym_char] = ACTIONS(3884),
    [anon_sym_DASH] = ACTIONS(3882),
    [anon_sym_AMP] = ACTIONS(3882),
    [anon_sym_PIPE] = ACTIONS(3882),
    [anon_sym_LT] = ACTIONS(3882),
    [anon_sym_DOT_DOT] = ACTIONS(3882),
    [anon_sym_COLON_COLON] = ACTIONS(3882),
    [anon_sym_POUND] = ACTIONS(3882),
    [anon_sym_SQUOTE] = ACTIONS(3884),
    [anon_sym_async] = ACTIONS(3884),
    [anon_sym_auto] = ACTIONS(3884),
    [anon_sym_await] = ACTIONS(3884),
    [anon_sym_become] = ACTIONS(3884),
    [anon_sym_break] = ACTIONS(3884),
    [anon_sym_const] = ACTIONS(3884),
    [anon_sym_continue] = ACTIONS(3884),
    [anon_sym_default] = ACTIONS(3884),
    [anon_sym_enum] = ACTIONS(3884),
    [anon_sym_fn] = ACTIONS(3884),
    [anon_sym_for] = ACTIONS(3884),
    [anon_sym_gen] = ACTIONS(3884),
    [anon_sym_if] = ACTIONS(3884),
    [anon_sym_impl] = ACTIONS(3884),
    [anon_sym_let] = ACTIONS(3884),
    [anon_sym_loop] = ACTIONS(3884),
    [anon_sym_match] = ACTIONS(3884),
    [anon_sym_mod] = ACTIONS(3884),
    [anon_sym_pub] = ACTIONS(3884),
    [anon_sym_return] = ACTIONS(3884),
    [anon_sym_raw] = ACTIONS(3884),
    [anon_sym_static] = ACTIONS(3884),
    [anon_sym_struct] = ACTIONS(3884),
    [anon_sym_trait] = ACTIONS(3884),
    [anon_sym_type] = ACTIONS(3884),
    [anon_sym_union] = ACTIONS(3884),
    [anon_sym_unsafe] = ACTIONS(3884),
    [anon_sym_use] = ACTIONS(3884),
    [anon_sym_while] = ACTIONS(3884),
    [anon_sym_extern] = ACTIONS(3884),
    [anon_sym_safe] = ACTIONS(3884),
    [anon_sym_yield] = ACTIONS(3884),
    [anon_sym_move] = ACTIONS(3884),
    [anon_sym_try] = ACTIONS(3884),
    [sym_integer_literal] = ACTIONS(3882),
    [aux_sym_string_literal_token1] = ACTIONS(3882),
    [sym_char_literal] = ACTIONS(3882),
    [anon_sym_true] = ACTIONS(3884),
    [anon_sym_false] = ACTIONS(3884),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3884),
    [sym_super] = ACTIONS(3884),
    [sym_crate] = ACTIONS(3884),
    [aux_sym_metavariable_token1] = ACTIONS(3882),
    [sym__raw_string_literal_start] = ACTIONS(3882),
    [sym_float_literal] = ACTIONS(3882),
    [sym__outer_block_doc_comment_start] = ACTIONS(3882),
    [sym__outer_line_doc_comment_start] = ACTIONS(3882),
  },
  [STATE(1029)] = {
    [sym_line_comment] = STATE(1029),
    [sym_block_comment] = STATE(1029),
    [ts_builtin_sym_end] = ACTIONS(3886),
    [sym_identifier] = ACTIONS(3888),
    [anon_sym_SEMI] = ACTIONS(3886),
    [anon_sym_macro] = ACTIONS(3888),
    [anon_sym_LPAREN] = ACTIONS(3886),
    [anon_sym_LBRACE] = ACTIONS(3886),
    [anon_sym_RBRACE] = ACTIONS(3886),
    [anon_sym_macro_rules] = ACTIONS(3888),
    [anon_sym_BANG] = ACTIONS(3886),
    [anon_sym_LBRACK] = ACTIONS(3886),
    [anon_sym_STAR] = ACTIONS(3886),
    [anon_sym_u8] = ACTIONS(3888),
    [anon_sym_i8] = ACTIONS(3888),
    [anon_sym_u16] = ACTIONS(3888),
    [anon_sym_i16] = ACTIONS(3888),
    [anon_sym_u32] = ACTIONS(3888),
    [anon_sym_i32] = ACTIONS(3888),
    [anon_sym_u64] = ACTIONS(3888),
    [anon_sym_i64] = ACTIONS(3888),
    [anon_sym_u128] = ACTIONS(3888),
    [anon_sym_i128] = ACTIONS(3888),
    [anon_sym_isize] = ACTIONS(3888),
    [anon_sym_usize] = ACTIONS(3888),
    [anon_sym_f32] = ACTIONS(3888),
    [anon_sym_f64] = ACTIONS(3888),
    [anon_sym_bool] = ACTIONS(3888),
    [anon_sym_str] = ACTIONS(3888),
    [anon_sym_char] = ACTIONS(3888),
    [anon_sym_DASH] = ACTIONS(3886),
    [anon_sym_AMP] = ACTIONS(3886),
    [anon_sym_PIPE] = ACTIONS(3886),
    [anon_sym_LT] = ACTIONS(3886),
    [anon_sym_DOT_DOT] = ACTIONS(3886),
    [anon_sym_COLON_COLON] = ACTIONS(3886),
    [anon_sym_POUND] = ACTIONS(3886),
    [anon_sym_SQUOTE] = ACTIONS(3888),
    [anon_sym_async] = ACTIONS(3888),
    [anon_sym_auto] = ACTIONS(3888),
    [anon_sym_await] = ACTIONS(3888),
    [anon_sym_become] = ACTIONS(3888),
    [anon_sym_break] = ACTIONS(3888),
    [anon_sym_const] = ACTIONS(3888),
    [anon_sym_continue] = ACTIONS(3888),
    [anon_sym_default] = ACTIONS(3888),
    [anon_sym_enum] = ACTIONS(3888),
    [anon_sym_fn] = ACTIONS(3888),
    [anon_sym_for] = ACTIONS(3888),
    [anon_sym_gen] = ACTIONS(3888),
    [anon_sym_if] = ACTIONS(3888),
    [anon_sym_impl] = ACTIONS(3888),
    [anon_sym_let] = ACTIONS(3888),
    [anon_sym_loop] = ACTIONS(3888),
    [anon_sym_match] = ACTIONS(3888),
    [anon_sym_mod] = ACTIONS(3888),
    [anon_sym_pub] = ACTIONS(3888),
    [anon_sym_return] = ACTIONS(3888),
    [anon_sym_raw] = ACTIONS(3888),
    [anon_sym_static] = ACTIONS(3888),
    [anon_sym_struct] = ACTIONS(3888),
    [anon_sym_trait] = ACTIONS(3888),
    [anon_sym_type] = ACTIONS(3888),
    [anon_sym_union] = ACTIONS(3888),
    [anon_sym_unsafe] = ACTIONS(3888),
    [anon_sym_use] = ACTIONS(3888),
    [anon_sym_while] = ACTIONS(3888),
    [anon_sym_extern] = ACTIONS(3888),
    [anon_sym_safe] = ACTIONS(3888),
    [anon_sym_yield] = ACTIONS(3888),
    [anon_sym_move] = ACTIONS(3888),
    [anon_sym_try] = ACTIONS(3888),
    [sym_integer_literal] = ACTIONS(3886),
    [aux_sym_string_literal_token1] = ACTIONS(3886),
    [sym_char_literal] = ACTIONS(3886),
    [anon_sym_true] = ACTIONS(3888),
    [anon_sym_false] = ACTIONS(3888),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3888),
    [sym_super] = ACTIONS(3888),
    [sym_crate] = ACTIONS(3888),
    [aux_sym_metavariable_token1] = ACTIONS(3886),
    [sym__raw_string_literal_start] = ACTIONS(3886),
    [sym_float_literal] = ACTIONS(3886),
    [sym__outer_block_doc_comment_start] = ACTIONS(3886),
    [sym__outer_line_doc_comment_start] = ACTIONS(3886),
  },
  [STATE(1030)] = {
    [sym_line_comment] = STATE(1030),
    [sym_block_comment] = STATE(1030),
    [ts_builtin_sym_end] = ACTIONS(3890),
    [sym_identifier] = ACTIONS(3892),
    [anon_sym_SEMI] = ACTIONS(3890),
    [anon_sym_macro] = ACTIONS(3892),
    [anon_sym_LPAREN] = ACTIONS(3890),
    [anon_sym_LBRACE] = ACTIONS(3890),
    [anon_sym_RBRACE] = ACTIONS(3890),
    [anon_sym_macro_rules] = ACTIONS(3892),
    [anon_sym_BANG] = ACTIONS(3890),
    [anon_sym_LBRACK] = ACTIONS(3890),
    [anon_sym_STAR] = ACTIONS(3890),
    [anon_sym_u8] = ACTIONS(3892),
    [anon_sym_i8] = ACTIONS(3892),
    [anon_sym_u16] = ACTIONS(3892),
    [anon_sym_i16] = ACTIONS(3892),
    [anon_sym_u32] = ACTIONS(3892),
    [anon_sym_i32] = ACTIONS(3892),
    [anon_sym_u64] = ACTIONS(3892),
    [anon_sym_i64] = ACTIONS(3892),
    [anon_sym_u128] = ACTIONS(3892),
    [anon_sym_i128] = ACTIONS(3892),
    [anon_sym_isize] = ACTIONS(3892),
    [anon_sym_usize] = ACTIONS(3892),
    [anon_sym_f32] = ACTIONS(3892),
    [anon_sym_f64] = ACTIONS(3892),
    [anon_sym_bool] = ACTIONS(3892),
    [anon_sym_str] = ACTIONS(3892),
    [anon_sym_char] = ACTIONS(3892),
    [anon_sym_DASH] = ACTIONS(3890),
    [anon_sym_AMP] = ACTIONS(3890),
    [anon_sym_PIPE] = ACTIONS(3890),
    [anon_sym_LT] = ACTIONS(3890),
    [anon_sym_DOT_DOT] = ACTIONS(3890),
    [anon_sym_COLON_COLON] = ACTIONS(3890),
    [anon_sym_POUND] = ACTIONS(3890),
    [anon_sym_SQUOTE] = ACTIONS(3892),
    [anon_sym_async] = ACTIONS(3892),
    [anon_sym_auto] = ACTIONS(3892),
    [anon_sym_await] = ACTIONS(3892),
    [anon_sym_become] = ACTIONS(3892),
    [anon_sym_break] = ACTIONS(3892),
    [anon_sym_const] = ACTIONS(3892),
    [anon_sym_continue] = ACTIONS(3892),
    [anon_sym_default] = ACTIONS(3892),
    [anon_sym_enum] = ACTIONS(3892),
    [anon_sym_fn] = ACTIONS(3892),
    [anon_sym_for] = ACTIONS(3892),
    [anon_sym_gen] = ACTIONS(3892),
    [anon_sym_if] = ACTIONS(3892),
    [anon_sym_impl] = ACTIONS(3892),
    [anon_sym_let] = ACTIONS(3892),
    [anon_sym_loop] = ACTIONS(3892),
    [anon_sym_match] = ACTIONS(3892),
    [anon_sym_mod] = ACTIONS(3892),
    [anon_sym_pub] = ACTIONS(3892),
    [anon_sym_return] = ACTIONS(3892),
    [anon_sym_raw] = ACTIONS(3892),
    [anon_sym_static] = ACTIONS(3892),
    [anon_sym_struct] = ACTIONS(3892),
    [anon_sym_trait] = ACTIONS(3892),
    [anon_sym_type] = ACTIONS(3892),
    [anon_sym_union] = ACTIONS(3892),
    [anon_sym_unsafe] = ACTIONS(3892),
    [anon_sym_use] = ACTIONS(3892),
    [anon_sym_while] = ACTIONS(3892),
    [anon_sym_extern] = ACTIONS(3892),
    [anon_sym_safe] = ACTIONS(3892),
    [anon_sym_yield] = ACTIONS(3892),
    [anon_sym_move] = ACTIONS(3892),
    [anon_sym_try] = ACTIONS(3892),
    [sym_integer_literal] = ACTIONS(3890),
    [aux_sym_string_literal_token1] = ACTIONS(3890),
    [sym_char_literal] = ACTIONS(3890),
    [anon_sym_true] = ACTIONS(3892),
    [anon_sym_false] = ACTIONS(3892),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3892),
    [sym_super] = ACTIONS(3892),
    [sym_crate] = ACTIONS(3892),
    [aux_sym_metavariable_token1] = ACTIONS(3890),
    [sym__raw_string_literal_start] = ACTIONS(3890),
    [sym_float_literal] = ACTIONS(3890),
    [sym__outer_block_doc_comment_start] = ACTIONS(3890),
    [sym__outer_line_doc_comment_start] = ACTIONS(3890),
  },
  [STATE(1031)] = {
    [sym_line_comment] = STATE(1031),
    [sym_block_comment] = STATE(1031),
    [ts_builtin_sym_end] = ACTIONS(3894),
    [sym_identifier] = ACTIONS(3896),
    [anon_sym_SEMI] = ACTIONS(3894),
    [anon_sym_macro] = ACTIONS(3896),
    [anon_sym_LPAREN] = ACTIONS(3894),
    [anon_sym_LBRACE] = ACTIONS(3894),
    [anon_sym_RBRACE] = ACTIONS(3894),
    [anon_sym_macro_rules] = ACTIONS(3896),
    [anon_sym_BANG] = ACTIONS(3894),
    [anon_sym_LBRACK] = ACTIONS(3894),
    [anon_sym_STAR] = ACTIONS(3894),
    [anon_sym_u8] = ACTIONS(3896),
    [anon_sym_i8] = ACTIONS(3896),
    [anon_sym_u16] = ACTIONS(3896),
    [anon_sym_i16] = ACTIONS(3896),
    [anon_sym_u32] = ACTIONS(3896),
    [anon_sym_i32] = ACTIONS(3896),
    [anon_sym_u64] = ACTIONS(3896),
    [anon_sym_i64] = ACTIONS(3896),
    [anon_sym_u128] = ACTIONS(3896),
    [anon_sym_i128] = ACTIONS(3896),
    [anon_sym_isize] = ACTIONS(3896),
    [anon_sym_usize] = ACTIONS(3896),
    [anon_sym_f32] = ACTIONS(3896),
    [anon_sym_f64] = ACTIONS(3896),
    [anon_sym_bool] = ACTIONS(3896),
    [anon_sym_str] = ACTIONS(3896),
    [anon_sym_char] = ACTIONS(3896),
    [anon_sym_DASH] = ACTIONS(3894),
    [anon_sym_AMP] = ACTIONS(3894),
    [anon_sym_PIPE] = ACTIONS(3894),
    [anon_sym_LT] = ACTIONS(3894),
    [anon_sym_DOT_DOT] = ACTIONS(3894),
    [anon_sym_COLON_COLON] = ACTIONS(3894),
    [anon_sym_POUND] = ACTIONS(3894),
    [anon_sym_SQUOTE] = ACTIONS(3896),
    [anon_sym_async] = ACTIONS(3896),
    [anon_sym_auto] = ACTIONS(3896),
    [anon_sym_await] = ACTIONS(3896),
    [anon_sym_become] = ACTIONS(3896),
    [anon_sym_break] = ACTIONS(3896),
    [anon_sym_const] = ACTIONS(3896),
    [anon_sym_continue] = ACTIONS(3896),
    [anon_sym_default] = ACTIONS(3896),
    [anon_sym_enum] = ACTIONS(3896),
    [anon_sym_fn] = ACTIONS(3896),
    [anon_sym_for] = ACTIONS(3896),
    [anon_sym_gen] = ACTIONS(3896),
    [anon_sym_if] = ACTIONS(3896),
    [anon_sym_impl] = ACTIONS(3896),
    [anon_sym_let] = ACTIONS(3896),
    [anon_sym_loop] = ACTIONS(3896),
    [anon_sym_match] = ACTIONS(3896),
    [anon_sym_mod] = ACTIONS(3896),
    [anon_sym_pub] = ACTIONS(3896),
    [anon_sym_return] = ACTIONS(3896),
    [anon_sym_raw] = ACTIONS(3896),
    [anon_sym_static] = ACTIONS(3896),
    [anon_sym_struct] = ACTIONS(3896),
    [anon_sym_trait] = ACTIONS(3896),
    [anon_sym_type] = ACTIONS(3896),
    [anon_sym_union] = ACTIONS(3896),
    [anon_sym_unsafe] = ACTIONS(3896),
    [anon_sym_use] = ACTIONS(3896),
    [anon_sym_while] = ACTIONS(3896),
    [anon_sym_extern] = ACTIONS(3896),
    [anon_sym_safe] = ACTIONS(3896),
    [anon_sym_yield] = ACTIONS(3896),
    [anon_sym_move] = ACTIONS(3896),
    [anon_sym_try] = ACTIONS(3896),
    [sym_integer_literal] = ACTIONS(3894),
    [aux_sym_string_literal_token1] = ACTIONS(3894),
    [sym_char_literal] = ACTIONS(3894),
    [anon_sym_true] = ACTIONS(3896),
    [anon_sym_false] = ACTIONS(3896),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3896),
    [sym_super] = ACTIONS(3896),
    [sym_crate] = ACTIONS(3896),
    [aux_sym_metavariable_token1] = ACTIONS(3894),
    [sym__raw_string_literal_start] = ACTIONS(3894),
    [sym_float_literal] = ACTIONS(3894),
    [sym__outer_block_doc_comment_start] = ACTIONS(3894),
    [sym__outer_line_doc_comment_start] = ACTIONS(3894),
  },
  [STATE(1032)] = {
    [sym_line_comment] = STATE(1032),
    [sym_block_comment] = STATE(1032),
    [ts_builtin_sym_end] = ACTIONS(3898),
    [sym_identifier] = ACTIONS(3900),
    [anon_sym_SEMI] = ACTIONS(3898),
    [anon_sym_macro] = ACTIONS(3900),
    [anon_sym_LPAREN] = ACTIONS(3898),
    [anon_sym_LBRACE] = ACTIONS(3898),
    [anon_sym_RBRACE] = ACTIONS(3898),
    [anon_sym_macro_rules] = ACTIONS(3900),
    [anon_sym_BANG] = ACTIONS(3898),
    [anon_sym_LBRACK] = ACTIONS(3898),
    [anon_sym_STAR] = ACTIONS(3898),
    [anon_sym_u8] = ACTIONS(3900),
    [anon_sym_i8] = ACTIONS(3900),
    [anon_sym_u16] = ACTIONS(3900),
    [anon_sym_i16] = ACTIONS(3900),
    [anon_sym_u32] = ACTIONS(3900),
    [anon_sym_i32] = ACTIONS(3900),
    [anon_sym_u64] = ACTIONS(3900),
    [anon_sym_i64] = ACTIONS(3900),
    [anon_sym_u128] = ACTIONS(3900),
    [anon_sym_i128] = ACTIONS(3900),
    [anon_sym_isize] = ACTIONS(3900),
    [anon_sym_usize] = ACTIONS(3900),
    [anon_sym_f32] = ACTIONS(3900),
    [anon_sym_f64] = ACTIONS(3900),
    [anon_sym_bool] = ACTIONS(3900),
    [anon_sym_str] = ACTIONS(3900),
    [anon_sym_char] = ACTIONS(3900),
    [anon_sym_DASH] = ACTIONS(3898),
    [anon_sym_AMP] = ACTIONS(3898),
    [anon_sym_PIPE] = ACTIONS(3898),
    [anon_sym_LT] = ACTIONS(3898),
    [anon_sym_DOT_DOT] = ACTIONS(3898),
    [anon_sym_COLON_COLON] = ACTIONS(3898),
    [anon_sym_POUND] = ACTIONS(3898),
    [anon_sym_SQUOTE] = ACTIONS(3900),
    [anon_sym_async] = ACTIONS(3900),
    [anon_sym_auto] = ACTIONS(3900),
    [anon_sym_await] = ACTIONS(3900),
    [anon_sym_become] = ACTIONS(3900),
    [anon_sym_break] = ACTIONS(3900),
    [anon_sym_const] = ACTIONS(3900),
    [anon_sym_continue] = ACTIONS(3900),
    [anon_sym_default] = ACTIONS(3900),
    [anon_sym_enum] = ACTIONS(3900),
    [anon_sym_fn] = ACTIONS(3900),
    [anon_sym_for] = ACTIONS(3900),
    [anon_sym_gen] = ACTIONS(3900),
    [anon_sym_if] = ACTIONS(3900),
    [anon_sym_impl] = ACTIONS(3900),
    [anon_sym_let] = ACTIONS(3900),
    [anon_sym_loop] = ACTIONS(3900),
    [anon_sym_match] = ACTIONS(3900),
    [anon_sym_mod] = ACTIONS(3900),
    [anon_sym_pub] = ACTIONS(3900),
    [anon_sym_return] = ACTIONS(3900),
    [anon_sym_raw] = ACTIONS(3900),
    [anon_sym_static] = ACTIONS(3900),
    [anon_sym_struct] = ACTIONS(3900),
    [anon_sym_trait] = ACTIONS(3900),
    [anon_sym_type] = ACTIONS(3900),
    [anon_sym_union] = ACTIONS(3900),
    [anon_sym_unsafe] = ACTIONS(3900),
    [anon_sym_use] = ACTIONS(3900),
    [anon_sym_while] = ACTIONS(3900),
    [anon_sym_extern] = ACTIONS(3900),
    [anon_sym_safe] = ACTIONS(3900),
    [anon_sym_yield] = ACTIONS(3900),
    [anon_sym_move] = ACTIONS(3900),
    [anon_sym_try] = ACTIONS(3900),
    [sym_integer_literal] = ACTIONS(3898),
    [aux_sym_string_literal_token1] = ACTIONS(3898),
    [sym_char_literal] = ACTIONS(3898),
    [anon_sym_true] = ACTIONS(3900),
    [anon_sym_false] = ACTIONS(3900),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3900),
    [sym_super] = ACTIONS(3900),
    [sym_crate] = ACTIONS(3900),
    [aux_sym_metavariable_token1] = ACTIONS(3898),
    [sym__raw_string_literal_start] = ACTIONS(3898),
    [sym_float_literal] = ACTIONS(3898),
    [sym__outer_block_doc_comment_start] = ACTIONS(3898),
    [sym__outer_line_doc_comment_start] = ACTIONS(3898),
  },
  [STATE(1033)] = {
    [sym_line_comment] = STATE(1033),
    [sym_block_comment] = STATE(1033),
    [ts_builtin_sym_end] = ACTIONS(3902),
    [sym_identifier] = ACTIONS(3904),
    [anon_sym_SEMI] = ACTIONS(3902),
    [anon_sym_macro] = ACTIONS(3904),
    [anon_sym_LPAREN] = ACTIONS(3902),
    [anon_sym_LBRACE] = ACTIONS(3902),
    [anon_sym_RBRACE] = ACTIONS(3902),
    [anon_sym_macro_rules] = ACTIONS(3904),
    [anon_sym_BANG] = ACTIONS(3902),
    [anon_sym_LBRACK] = ACTIONS(3902),
    [anon_sym_STAR] = ACTIONS(3902),
    [anon_sym_u8] = ACTIONS(3904),
    [anon_sym_i8] = ACTIONS(3904),
    [anon_sym_u16] = ACTIONS(3904),
    [anon_sym_i16] = ACTIONS(3904),
    [anon_sym_u32] = ACTIONS(3904),
    [anon_sym_i32] = ACTIONS(3904),
    [anon_sym_u64] = ACTIONS(3904),
    [anon_sym_i64] = ACTIONS(3904),
    [anon_sym_u128] = ACTIONS(3904),
    [anon_sym_i128] = ACTIONS(3904),
    [anon_sym_isize] = ACTIONS(3904),
    [anon_sym_usize] = ACTIONS(3904),
    [anon_sym_f32] = ACTIONS(3904),
    [anon_sym_f64] = ACTIONS(3904),
    [anon_sym_bool] = ACTIONS(3904),
    [anon_sym_str] = ACTIONS(3904),
    [anon_sym_char] = ACTIONS(3904),
    [anon_sym_DASH] = ACTIONS(3902),
    [anon_sym_AMP] = ACTIONS(3902),
    [anon_sym_PIPE] = ACTIONS(3902),
    [anon_sym_LT] = ACTIONS(3902),
    [anon_sym_DOT_DOT] = ACTIONS(3902),
    [anon_sym_COLON_COLON] = ACTIONS(3902),
    [anon_sym_POUND] = ACTIONS(3902),
    [anon_sym_SQUOTE] = ACTIONS(3904),
    [anon_sym_async] = ACTIONS(3904),
    [anon_sym_auto] = ACTIONS(3904),
    [anon_sym_await] = ACTIONS(3904),
    [anon_sym_become] = ACTIONS(3904),
    [anon_sym_break] = ACTIONS(3904),
    [anon_sym_const] = ACTIONS(3904),
    [anon_sym_continue] = ACTIONS(3904),
    [anon_sym_default] = ACTIONS(3904),
    [anon_sym_enum] = ACTIONS(3904),
    [anon_sym_fn] = ACTIONS(3904),
    [anon_sym_for] = ACTIONS(3904),
    [anon_sym_gen] = ACTIONS(3904),
    [anon_sym_if] = ACTIONS(3904),
    [anon_sym_impl] = ACTIONS(3904),
    [anon_sym_let] = ACTIONS(3904),
    [anon_sym_loop] = ACTIONS(3904),
    [anon_sym_match] = ACTIONS(3904),
    [anon_sym_mod] = ACTIONS(3904),
    [anon_sym_pub] = ACTIONS(3904),
    [anon_sym_return] = ACTIONS(3904),
    [anon_sym_raw] = ACTIONS(3904),
    [anon_sym_static] = ACTIONS(3904),
    [anon_sym_struct] = ACTIONS(3904),
    [anon_sym_trait] = ACTIONS(3904),
    [anon_sym_type] = ACTIONS(3904),
    [anon_sym_union] = ACTIONS(3904),
    [anon_sym_unsafe] = ACTIONS(3904),
    [anon_sym_use] = ACTIONS(3904),
    [anon_sym_while] = ACTIONS(3904),
    [anon_sym_extern] = ACTIONS(3904),
    [anon_sym_safe] = ACTIONS(3904),
    [anon_sym_yield] = ACTIONS(3904),
    [anon_sym_move] = ACTIONS(3904),
    [anon_sym_try] = ACTIONS(3904),
    [sym_integer_literal] = ACTIONS(3902),
    [aux_sym_string_literal_token1] = ACTIONS(3902),
    [sym_char_literal] = ACTIONS(3902),
    [anon_sym_true] = ACTIONS(3904),
    [anon_sym_false] = ACTIONS(3904),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3904),
    [sym_super] = ACTIONS(3904),
    [sym_crate] = ACTIONS(3904),
    [aux_sym_metavariable_token1] = ACTIONS(3902),
    [sym__raw_string_literal_start] = ACTIONS(3902),
    [sym_float_literal] = ACTIONS(3902),
    [sym__outer_block_doc_comment_start] = ACTIONS(3902),
    [sym__outer_line_doc_comment_start] = ACTIONS(3902),
  },
  [STATE(1034)] = {
    [sym_line_comment] = STATE(1034),
    [sym_block_comment] = STATE(1034),
    [ts_builtin_sym_end] = ACTIONS(3906),
    [sym_identifier] = ACTIONS(3908),
    [anon_sym_SEMI] = ACTIONS(3906),
    [anon_sym_macro] = ACTIONS(3908),
    [anon_sym_LPAREN] = ACTIONS(3906),
    [anon_sym_LBRACE] = ACTIONS(3906),
    [anon_sym_RBRACE] = ACTIONS(3906),
    [anon_sym_macro_rules] = ACTIONS(3908),
    [anon_sym_BANG] = ACTIONS(3906),
    [anon_sym_LBRACK] = ACTIONS(3906),
    [anon_sym_STAR] = ACTIONS(3906),
    [anon_sym_u8] = ACTIONS(3908),
    [anon_sym_i8] = ACTIONS(3908),
    [anon_sym_u16] = ACTIONS(3908),
    [anon_sym_i16] = ACTIONS(3908),
    [anon_sym_u32] = ACTIONS(3908),
    [anon_sym_i32] = ACTIONS(3908),
    [anon_sym_u64] = ACTIONS(3908),
    [anon_sym_i64] = ACTIONS(3908),
    [anon_sym_u128] = ACTIONS(3908),
    [anon_sym_i128] = ACTIONS(3908),
    [anon_sym_isize] = ACTIONS(3908),
    [anon_sym_usize] = ACTIONS(3908),
    [anon_sym_f32] = ACTIONS(3908),
    [anon_sym_f64] = ACTIONS(3908),
    [anon_sym_bool] = ACTIONS(3908),
    [anon_sym_str] = ACTIONS(3908),
    [anon_sym_char] = ACTIONS(3908),
    [anon_sym_DASH] = ACTIONS(3906),
    [anon_sym_AMP] = ACTIONS(3906),
    [anon_sym_PIPE] = ACTIONS(3906),
    [anon_sym_LT] = ACTIONS(3906),
    [anon_sym_DOT_DOT] = ACTIONS(3906),
    [anon_sym_COLON_COLON] = ACTIONS(3906),
    [anon_sym_POUND] = ACTIONS(3906),
    [anon_sym_SQUOTE] = ACTIONS(3908),
    [anon_sym_async] = ACTIONS(3908),
    [anon_sym_auto] = ACTIONS(3908),
    [anon_sym_await] = ACTIONS(3908),
    [anon_sym_become] = ACTIONS(3908),
    [anon_sym_break] = ACTIONS(3908),
    [anon_sym_const] = ACTIONS(3908),
    [anon_sym_continue] = ACTIONS(3908),
    [anon_sym_default] = ACTIONS(3908),
    [anon_sym_enum] = ACTIONS(3908),
    [anon_sym_fn] = ACTIONS(3908),
    [anon_sym_for] = ACTIONS(3908),
    [anon_sym_gen] = ACTIONS(3908),
    [anon_sym_if] = ACTIONS(3908),
    [anon_sym_impl] = ACTIONS(3908),
    [anon_sym_let] = ACTIONS(3908),
    [anon_sym_loop] = ACTIONS(3908),
    [anon_sym_match] = ACTIONS(3908),
    [anon_sym_mod] = ACTIONS(3908),
    [anon_sym_pub] = ACTIONS(3908),
    [anon_sym_return] = ACTIONS(3908),
    [anon_sym_raw] = ACTIONS(3908),
    [anon_sym_static] = ACTIONS(3908),
    [anon_sym_struct] = ACTIONS(3908),
    [anon_sym_trait] = ACTIONS(3908),
    [anon_sym_type] = ACTIONS(3908),
    [anon_sym_union] = ACTIONS(3908),
    [anon_sym_unsafe] = ACTIONS(3908),
    [anon_sym_use] = ACTIONS(3908),
    [anon_sym_while] = ACTIONS(3908),
    [anon_sym_extern] = ACTIONS(3908),
    [anon_sym_safe] = ACTIONS(3908),
    [anon_sym_yield] = ACTIONS(3908),
    [anon_sym_move] = ACTIONS(3908),
    [anon_sym_try] = ACTIONS(3908),
    [sym_integer_literal] = ACTIONS(3906),
    [aux_sym_string_literal_token1] = ACTIONS(3906),
    [sym_char_literal] = ACTIONS(3906),
    [anon_sym_true] = ACTIONS(3908),
    [anon_sym_false] = ACTIONS(3908),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3908),
    [sym_super] = ACTIONS(3908),
    [sym_crate] = ACTIONS(3908),
    [aux_sym_metavariable_token1] = ACTIONS(3906),
    [sym__raw_string_literal_start] = ACTIONS(3906),
    [sym_float_literal] = ACTIONS(3906),
    [sym__outer_block_doc_comment_start] = ACTIONS(3906),
    [sym__outer_line_doc_comment_start] = ACTIONS(3906),
  },
  [STATE(1035)] = {
    [sym_line_comment] = STATE(1035),
    [sym_block_comment] = STATE(1035),
    [ts_builtin_sym_end] = ACTIONS(3910),
    [sym_identifier] = ACTIONS(3912),
    [anon_sym_SEMI] = ACTIONS(3910),
    [anon_sym_macro] = ACTIONS(3912),
    [anon_sym_LPAREN] = ACTIONS(3910),
    [anon_sym_LBRACE] = ACTIONS(3910),
    [anon_sym_RBRACE] = ACTIONS(3910),
    [anon_sym_macro_rules] = ACTIONS(3912),
    [anon_sym_BANG] = ACTIONS(3910),
    [anon_sym_LBRACK] = ACTIONS(3910),
    [anon_sym_STAR] = ACTIONS(3910),
    [anon_sym_u8] = ACTIONS(3912),
    [anon_sym_i8] = ACTIONS(3912),
    [anon_sym_u16] = ACTIONS(3912),
    [anon_sym_i16] = ACTIONS(3912),
    [anon_sym_u32] = ACTIONS(3912),
    [anon_sym_i32] = ACTIONS(3912),
    [anon_sym_u64] = ACTIONS(3912),
    [anon_sym_i64] = ACTIONS(3912),
    [anon_sym_u128] = ACTIONS(3912),
    [anon_sym_i128] = ACTIONS(3912),
    [anon_sym_isize] = ACTIONS(3912),
    [anon_sym_usize] = ACTIONS(3912),
    [anon_sym_f32] = ACTIONS(3912),
    [anon_sym_f64] = ACTIONS(3912),
    [anon_sym_bool] = ACTIONS(3912),
    [anon_sym_str] = ACTIONS(3912),
    [anon_sym_char] = ACTIONS(3912),
    [anon_sym_DASH] = ACTIONS(3910),
    [anon_sym_AMP] = ACTIONS(3910),
    [anon_sym_PIPE] = ACTIONS(3910),
    [anon_sym_LT] = ACTIONS(3910),
    [anon_sym_DOT_DOT] = ACTIONS(3910),
    [anon_sym_COLON_COLON] = ACTIONS(3910),
    [anon_sym_POUND] = ACTIONS(3910),
    [anon_sym_SQUOTE] = ACTIONS(3912),
    [anon_sym_async] = ACTIONS(3912),
    [anon_sym_auto] = ACTIONS(3912),
    [anon_sym_await] = ACTIONS(3912),
    [anon_sym_become] = ACTIONS(3912),
    [anon_sym_break] = ACTIONS(3912),
    [anon_sym_const] = ACTIONS(3912),
    [anon_sym_continue] = ACTIONS(3912),
    [anon_sym_default] = ACTIONS(3912),
    [anon_sym_enum] = ACTIONS(3912),
    [anon_sym_fn] = ACTIONS(3912),
    [anon_sym_for] = ACTIONS(3912),
    [anon_sym_gen] = ACTIONS(3912),
    [anon_sym_if] = ACTIONS(3912),
    [anon_sym_impl] = ACTIONS(3912),
    [anon_sym_let] = ACTIONS(3912),
    [anon_sym_loop] = ACTIONS(3912),
    [anon_sym_match] = ACTIONS(3912),
    [anon_sym_mod] = ACTIONS(3912),
    [anon_sym_pub] = ACTIONS(3912),
    [anon_sym_return] = ACTIONS(3912),
    [anon_sym_raw] = ACTIONS(3912),
    [anon_sym_static] = ACTIONS(3912),
    [anon_sym_struct] = ACTIONS(3912),
    [anon_sym_trait] = ACTIONS(3912),
    [anon_sym_type] = ACTIONS(3912),
    [anon_sym_union] = ACTIONS(3912),
    [anon_sym_unsafe] = ACTIONS(3912),
    [anon_sym_use] = ACTIONS(3912),
    [anon_sym_while] = ACTIONS(3912),
    [anon_sym_extern] = ACTIONS(3912),
    [anon_sym_safe] = ACTIONS(3912),
    [anon_sym_yield] = ACTIONS(3912),
    [anon_sym_move] = ACTIONS(3912),
    [anon_sym_try] = ACTIONS(3912),
    [sym_integer_literal] = ACTIONS(3910),
    [aux_sym_string_literal_token1] = ACTIONS(3910),
    [sym_char_literal] = ACTIONS(3910),
    [anon_sym_true] = ACTIONS(3912),
    [anon_sym_false] = ACTIONS(3912),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3912),
    [sym_super] = ACTIONS(3912),
    [sym_crate] = ACTIONS(3912),
    [aux_sym_metavariable_token1] = ACTIONS(3910),
    [sym__raw_string_literal_start] = ACTIONS(3910),
    [sym_float_literal] = ACTIONS(3910),
    [sym__outer_block_doc_comment_start] = ACTIONS(3910),
    [sym__outer_line_doc_comment_start] = ACTIONS(3910),
  },
  [STATE(1036)] = {
    [sym_line_comment] = STATE(1036),
    [sym_block_comment] = STATE(1036),
    [ts_builtin_sym_end] = ACTIONS(3914),
    [sym_identifier] = ACTIONS(3916),
    [anon_sym_SEMI] = ACTIONS(3914),
    [anon_sym_macro] = ACTIONS(3916),
    [anon_sym_LPAREN] = ACTIONS(3914),
    [anon_sym_LBRACE] = ACTIONS(3914),
    [anon_sym_RBRACE] = ACTIONS(3914),
    [anon_sym_macro_rules] = ACTIONS(3916),
    [anon_sym_BANG] = ACTIONS(3914),
    [anon_sym_LBRACK] = ACTIONS(3914),
    [anon_sym_STAR] = ACTIONS(3914),
    [anon_sym_u8] = ACTIONS(3916),
    [anon_sym_i8] = ACTIONS(3916),
    [anon_sym_u16] = ACTIONS(3916),
    [anon_sym_i16] = ACTIONS(3916),
    [anon_sym_u32] = ACTIONS(3916),
    [anon_sym_i32] = ACTIONS(3916),
    [anon_sym_u64] = ACTIONS(3916),
    [anon_sym_i64] = ACTIONS(3916),
    [anon_sym_u128] = ACTIONS(3916),
    [anon_sym_i128] = ACTIONS(3916),
    [anon_sym_isize] = ACTIONS(3916),
    [anon_sym_usize] = ACTIONS(3916),
    [anon_sym_f32] = ACTIONS(3916),
    [anon_sym_f64] = ACTIONS(3916),
    [anon_sym_bool] = ACTIONS(3916),
    [anon_sym_str] = ACTIONS(3916),
    [anon_sym_char] = ACTIONS(3916),
    [anon_sym_DASH] = ACTIONS(3914),
    [anon_sym_AMP] = ACTIONS(3914),
    [anon_sym_PIPE] = ACTIONS(3914),
    [anon_sym_LT] = ACTIONS(3914),
    [anon_sym_DOT_DOT] = ACTIONS(3914),
    [anon_sym_COLON_COLON] = ACTIONS(3914),
    [anon_sym_POUND] = ACTIONS(3914),
    [anon_sym_SQUOTE] = ACTIONS(3916),
    [anon_sym_async] = ACTIONS(3916),
    [anon_sym_auto] = ACTIONS(3916),
    [anon_sym_await] = ACTIONS(3916),
    [anon_sym_become] = ACTIONS(3916),
    [anon_sym_break] = ACTIONS(3916),
    [anon_sym_const] = ACTIONS(3916),
    [anon_sym_continue] = ACTIONS(3916),
    [anon_sym_default] = ACTIONS(3916),
    [anon_sym_enum] = ACTIONS(3916),
    [anon_sym_fn] = ACTIONS(3916),
    [anon_sym_for] = ACTIONS(3916),
    [anon_sym_gen] = ACTIONS(3916),
    [anon_sym_if] = ACTIONS(3916),
    [anon_sym_impl] = ACTIONS(3916),
    [anon_sym_let] = ACTIONS(3916),
    [anon_sym_loop] = ACTIONS(3916),
    [anon_sym_match] = ACTIONS(3916),
    [anon_sym_mod] = ACTIONS(3916),
    [anon_sym_pub] = ACTIONS(3916),
    [anon_sym_return] = ACTIONS(3916),
    [anon_sym_raw] = ACTIONS(3916),
    [anon_sym_static] = ACTIONS(3916),
    [anon_sym_struct] = ACTIONS(3916),
    [anon_sym_trait] = ACTIONS(3916),
    [anon_sym_type] = ACTIONS(3916),
    [anon_sym_union] = ACTIONS(3916),
    [anon_sym_unsafe] = ACTIONS(3916),
    [anon_sym_use] = ACTIONS(3916),
    [anon_sym_while] = ACTIONS(3916),
    [anon_sym_extern] = ACTIONS(3916),
    [anon_sym_safe] = ACTIONS(3916),
    [anon_sym_yield] = ACTIONS(3916),
    [anon_sym_move] = ACTIONS(3916),
    [anon_sym_try] = ACTIONS(3916),
    [sym_integer_literal] = ACTIONS(3914),
    [aux_sym_string_literal_token1] = ACTIONS(3914),
    [sym_char_literal] = ACTIONS(3914),
    [anon_sym_true] = ACTIONS(3916),
    [anon_sym_false] = ACTIONS(3916),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3916),
    [sym_super] = ACTIONS(3916),
    [sym_crate] = ACTIONS(3916),
    [aux_sym_metavariable_token1] = ACTIONS(3914),
    [sym__raw_string_literal_start] = ACTIONS(3914),
    [sym_float_literal] = ACTIONS(3914),
    [sym__outer_block_doc_comment_start] = ACTIONS(3914),
    [sym__outer_line_doc_comment_start] = ACTIONS(3914),
  },
  [STATE(1037)] = {
    [sym_line_comment] = STATE(1037),
    [sym_block_comment] = STATE(1037),
    [ts_builtin_sym_end] = ACTIONS(3918),
    [sym_identifier] = ACTIONS(3920),
    [anon_sym_SEMI] = ACTIONS(3918),
    [anon_sym_macro] = ACTIONS(3920),
    [anon_sym_LPAREN] = ACTIONS(3918),
    [anon_sym_LBRACE] = ACTIONS(3918),
    [anon_sym_RBRACE] = ACTIONS(3918),
    [anon_sym_macro_rules] = ACTIONS(3920),
    [anon_sym_BANG] = ACTIONS(3918),
    [anon_sym_LBRACK] = ACTIONS(3918),
    [anon_sym_STAR] = ACTIONS(3918),
    [anon_sym_u8] = ACTIONS(3920),
    [anon_sym_i8] = ACTIONS(3920),
    [anon_sym_u16] = ACTIONS(3920),
    [anon_sym_i16] = ACTIONS(3920),
    [anon_sym_u32] = ACTIONS(3920),
    [anon_sym_i32] = ACTIONS(3920),
    [anon_sym_u64] = ACTIONS(3920),
    [anon_sym_i64] = ACTIONS(3920),
    [anon_sym_u128] = ACTIONS(3920),
    [anon_sym_i128] = ACTIONS(3920),
    [anon_sym_isize] = ACTIONS(3920),
    [anon_sym_usize] = ACTIONS(3920),
    [anon_sym_f32] = ACTIONS(3920),
    [anon_sym_f64] = ACTIONS(3920),
    [anon_sym_bool] = ACTIONS(3920),
    [anon_sym_str] = ACTIONS(3920),
    [anon_sym_char] = ACTIONS(3920),
    [anon_sym_DASH] = ACTIONS(3918),
    [anon_sym_AMP] = ACTIONS(3918),
    [anon_sym_PIPE] = ACTIONS(3918),
    [anon_sym_LT] = ACTIONS(3918),
    [anon_sym_DOT_DOT] = ACTIONS(3918),
    [anon_sym_COLON_COLON] = ACTIONS(3918),
    [anon_sym_POUND] = ACTIONS(3918),
    [anon_sym_SQUOTE] = ACTIONS(3920),
    [anon_sym_async] = ACTIONS(3920),
    [anon_sym_auto] = ACTIONS(3920),
    [anon_sym_await] = ACTIONS(3920),
    [anon_sym_become] = ACTIONS(3920),
    [anon_sym_break] = ACTIONS(3920),
    [anon_sym_const] = ACTIONS(3920),
    [anon_sym_continue] = ACTIONS(3920),
    [anon_sym_default] = ACTIONS(3920),
    [anon_sym_enum] = ACTIONS(3920),
    [anon_sym_fn] = ACTIONS(3920),
    [anon_sym_for] = ACTIONS(3920),
    [anon_sym_gen] = ACTIONS(3920),
    [anon_sym_if] = ACTIONS(3920),
    [anon_sym_impl] = ACTIONS(3920),
    [anon_sym_let] = ACTIONS(3920),
    [anon_sym_loop] = ACTIONS(3920),
    [anon_sym_match] = ACTIONS(3920),
    [anon_sym_mod] = ACTIONS(3920),
    [anon_sym_pub] = ACTIONS(3920),
    [anon_sym_return] = ACTIONS(3920),
    [anon_sym_raw] = ACTIONS(3920),
    [anon_sym_static] = ACTIONS(3920),
    [anon_sym_struct] = ACTIONS(3920),
    [anon_sym_trait] = ACTIONS(3920),
    [anon_sym_type] = ACTIONS(3920),
    [anon_sym_union] = ACTIONS(3920),
    [anon_sym_unsafe] = ACTIONS(3920),
    [anon_sym_use] = ACTIONS(3920),
    [anon_sym_while] = ACTIONS(3920),
    [anon_sym_extern] = ACTIONS(3920),
    [anon_sym_safe] = ACTIONS(3920),
    [anon_sym_yield] = ACTIONS(3920),
    [anon_sym_move] = ACTIONS(3920),
    [anon_sym_try] = ACTIONS(3920),
    [sym_integer_literal] = ACTIONS(3918),
    [aux_sym_string_literal_token1] = ACTIONS(3918),
    [sym_char_literal] = ACTIONS(3918),
    [anon_sym_true] = ACTIONS(3920),
    [anon_sym_false] = ACTIONS(3920),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3920),
    [sym_super] = ACTIONS(3920),
    [sym_crate] = ACTIONS(3920),
    [aux_sym_metavariable_token1] = ACTIONS(3918),
    [sym__raw_string_literal_start] = ACTIONS(3918),
    [sym_float_literal] = ACTIONS(3918),
    [sym__outer_block_doc_comment_start] = ACTIONS(3918),
    [sym__outer_line_doc_comment_start] = ACTIONS(3918),
  },
  [STATE(1038)] = {
    [sym_line_comment] = STATE(1038),
    [sym_block_comment] = STATE(1038),
    [ts_builtin_sym_end] = ACTIONS(3922),
    [sym_identifier] = ACTIONS(3924),
    [anon_sym_SEMI] = ACTIONS(3922),
    [anon_sym_macro] = ACTIONS(3924),
    [anon_sym_LPAREN] = ACTIONS(3922),
    [anon_sym_LBRACE] = ACTIONS(3922),
    [anon_sym_RBRACE] = ACTIONS(3922),
    [anon_sym_macro_rules] = ACTIONS(3924),
    [anon_sym_BANG] = ACTIONS(3922),
    [anon_sym_LBRACK] = ACTIONS(3922),
    [anon_sym_STAR] = ACTIONS(3922),
    [anon_sym_u8] = ACTIONS(3924),
    [anon_sym_i8] = ACTIONS(3924),
    [anon_sym_u16] = ACTIONS(3924),
    [anon_sym_i16] = ACTIONS(3924),
    [anon_sym_u32] = ACTIONS(3924),
    [anon_sym_i32] = ACTIONS(3924),
    [anon_sym_u64] = ACTIONS(3924),
    [anon_sym_i64] = ACTIONS(3924),
    [anon_sym_u128] = ACTIONS(3924),
    [anon_sym_i128] = ACTIONS(3924),
    [anon_sym_isize] = ACTIONS(3924),
    [anon_sym_usize] = ACTIONS(3924),
    [anon_sym_f32] = ACTIONS(3924),
    [anon_sym_f64] = ACTIONS(3924),
    [anon_sym_bool] = ACTIONS(3924),
    [anon_sym_str] = ACTIONS(3924),
    [anon_sym_char] = ACTIONS(3924),
    [anon_sym_DASH] = ACTIONS(3922),
    [anon_sym_AMP] = ACTIONS(3922),
    [anon_sym_PIPE] = ACTIONS(3922),
    [anon_sym_LT] = ACTIONS(3922),
    [anon_sym_DOT_DOT] = ACTIONS(3922),
    [anon_sym_COLON_COLON] = ACTIONS(3922),
    [anon_sym_POUND] = ACTIONS(3922),
    [anon_sym_SQUOTE] = ACTIONS(3924),
    [anon_sym_async] = ACTIONS(3924),
    [anon_sym_auto] = ACTIONS(3924),
    [anon_sym_await] = ACTIONS(3924),
    [anon_sym_become] = ACTIONS(3924),
    [anon_sym_break] = ACTIONS(3924),
    [anon_sym_const] = ACTIONS(3924),
    [anon_sym_continue] = ACTIONS(3924),
    [anon_sym_default] = ACTIONS(3924),
    [anon_sym_enum] = ACTIONS(3924),
    [anon_sym_fn] = ACTIONS(3924),
    [anon_sym_for] = ACTIONS(3924),
    [anon_sym_gen] = ACTIONS(3924),
    [anon_sym_if] = ACTIONS(3924),
    [anon_sym_impl] = ACTIONS(3924),
    [anon_sym_let] = ACTIONS(3924),
    [anon_sym_loop] = ACTIONS(3924),
    [anon_sym_match] = ACTIONS(3924),
    [anon_sym_mod] = ACTIONS(3924),
    [anon_sym_pub] = ACTIONS(3924),
    [anon_sym_return] = ACTIONS(3924),
    [anon_sym_raw] = ACTIONS(3924),
    [anon_sym_static] = ACTIONS(3924),
    [anon_sym_struct] = ACTIONS(3924),
    [anon_sym_trait] = ACTIONS(3924),
    [anon_sym_type] = ACTIONS(3924),
    [anon_sym_union] = ACTIONS(3924),
    [anon_sym_unsafe] = ACTIONS(3924),
    [anon_sym_use] = ACTIONS(3924),
    [anon_sym_while] = ACTIONS(3924),
    [anon_sym_extern] = ACTIONS(3924),
    [anon_sym_safe] = ACTIONS(3924),
    [anon_sym_yield] = ACTIONS(3924),
    [anon_sym_move] = ACTIONS(3924),
    [anon_sym_try] = ACTIONS(3924),
    [sym_integer_literal] = ACTIONS(3922),
    [aux_sym_string_literal_token1] = ACTIONS(3922),
    [sym_char_literal] = ACTIONS(3922),
    [anon_sym_true] = ACTIONS(3924),
    [anon_sym_false] = ACTIONS(3924),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3924),
    [sym_super] = ACTIONS(3924),
    [sym_crate] = ACTIONS(3924),
    [aux_sym_metavariable_token1] = ACTIONS(3922),
    [sym__raw_string_literal_start] = ACTIONS(3922),
    [sym_float_literal] = ACTIONS(3922),
    [sym__outer_block_doc_comment_start] = ACTIONS(3922),
    [sym__outer_line_doc_comment_start] = ACTIONS(3922),
  },
  [STATE(1039)] = {
    [sym_line_comment] = STATE(1039),
    [sym_block_comment] = STATE(1039),
    [ts_builtin_sym_end] = ACTIONS(3926),
    [sym_identifier] = ACTIONS(3928),
    [anon_sym_SEMI] = ACTIONS(3926),
    [anon_sym_macro] = ACTIONS(3928),
    [anon_sym_LPAREN] = ACTIONS(3926),
    [anon_sym_LBRACE] = ACTIONS(3926),
    [anon_sym_RBRACE] = ACTIONS(3926),
    [anon_sym_macro_rules] = ACTIONS(3928),
    [anon_sym_BANG] = ACTIONS(3926),
    [anon_sym_LBRACK] = ACTIONS(3926),
    [anon_sym_STAR] = ACTIONS(3926),
    [anon_sym_u8] = ACTIONS(3928),
    [anon_sym_i8] = ACTIONS(3928),
    [anon_sym_u16] = ACTIONS(3928),
    [anon_sym_i16] = ACTIONS(3928),
    [anon_sym_u32] = ACTIONS(3928),
    [anon_sym_i32] = ACTIONS(3928),
    [anon_sym_u64] = ACTIONS(3928),
    [anon_sym_i64] = ACTIONS(3928),
    [anon_sym_u128] = ACTIONS(3928),
    [anon_sym_i128] = ACTIONS(3928),
    [anon_sym_isize] = ACTIONS(3928),
    [anon_sym_usize] = ACTIONS(3928),
    [anon_sym_f32] = ACTIONS(3928),
    [anon_sym_f64] = ACTIONS(3928),
    [anon_sym_bool] = ACTIONS(3928),
    [anon_sym_str] = ACTIONS(3928),
    [anon_sym_char] = ACTIONS(3928),
    [anon_sym_DASH] = ACTIONS(3926),
    [anon_sym_AMP] = ACTIONS(3926),
    [anon_sym_PIPE] = ACTIONS(3926),
    [anon_sym_LT] = ACTIONS(3926),
    [anon_sym_DOT_DOT] = ACTIONS(3926),
    [anon_sym_COLON_COLON] = ACTIONS(3926),
    [anon_sym_POUND] = ACTIONS(3926),
    [anon_sym_SQUOTE] = ACTIONS(3928),
    [anon_sym_async] = ACTIONS(3928),
    [anon_sym_auto] = ACTIONS(3928),
    [anon_sym_await] = ACTIONS(3928),
    [anon_sym_become] = ACTIONS(3928),
    [anon_sym_break] = ACTIONS(3928),
    [anon_sym_const] = ACTIONS(3928),
    [anon_sym_continue] = ACTIONS(3928),
    [anon_sym_default] = ACTIONS(3928),
    [anon_sym_enum] = ACTIONS(3928),
    [anon_sym_fn] = ACTIONS(3928),
    [anon_sym_for] = ACTIONS(3928),
    [anon_sym_gen] = ACTIONS(3928),
    [anon_sym_if] = ACTIONS(3928),
    [anon_sym_impl] = ACTIONS(3928),
    [anon_sym_let] = ACTIONS(3928),
    [anon_sym_loop] = ACTIONS(3928),
    [anon_sym_match] = ACTIONS(3928),
    [anon_sym_mod] = ACTIONS(3928),
    [anon_sym_pub] = ACTIONS(3928),
    [anon_sym_return] = ACTIONS(3928),
    [anon_sym_raw] = ACTIONS(3928),
    [anon_sym_static] = ACTIONS(3928),
    [anon_sym_struct] = ACTIONS(3928),
    [anon_sym_trait] = ACTIONS(3928),
    [anon_sym_type] = ACTIONS(3928),
    [anon_sym_union] = ACTIONS(3928),
    [anon_sym_unsafe] = ACTIONS(3928),
    [anon_sym_use] = ACTIONS(3928),
    [anon_sym_while] = ACTIONS(3928),
    [anon_sym_extern] = ACTIONS(3928),
    [anon_sym_safe] = ACTIONS(3928),
    [anon_sym_yield] = ACTIONS(3928),
    [anon_sym_move] = ACTIONS(3928),
    [anon_sym_try] = ACTIONS(3928),
    [sym_integer_literal] = ACTIONS(3926),
    [aux_sym_string_literal_token1] = ACTIONS(3926),
    [sym_char_literal] = ACTIONS(3926),
    [anon_sym_true] = ACTIONS(3928),
    [anon_sym_false] = ACTIONS(3928),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3928),
    [sym_super] = ACTIONS(3928),
    [sym_crate] = ACTIONS(3928),
    [aux_sym_metavariable_token1] = ACTIONS(3926),
    [sym__raw_string_literal_start] = ACTIONS(3926),
    [sym_float_literal] = ACTIONS(3926),
    [sym__outer_block_doc_comment_start] = ACTIONS(3926),
    [sym__outer_line_doc_comment_start] = ACTIONS(3926),
  },
  [STATE(1040)] = {
    [sym_line_comment] = STATE(1040),
    [sym_block_comment] = STATE(1040),
    [ts_builtin_sym_end] = ACTIONS(3930),
    [sym_identifier] = ACTIONS(3932),
    [anon_sym_SEMI] = ACTIONS(3930),
    [anon_sym_macro] = ACTIONS(3932),
    [anon_sym_LPAREN] = ACTIONS(3930),
    [anon_sym_LBRACE] = ACTIONS(3930),
    [anon_sym_RBRACE] = ACTIONS(3930),
    [anon_sym_macro_rules] = ACTIONS(3932),
    [anon_sym_BANG] = ACTIONS(3930),
    [anon_sym_LBRACK] = ACTIONS(3930),
    [anon_sym_STAR] = ACTIONS(3930),
    [anon_sym_u8] = ACTIONS(3932),
    [anon_sym_i8] = ACTIONS(3932),
    [anon_sym_u16] = ACTIONS(3932),
    [anon_sym_i16] = ACTIONS(3932),
    [anon_sym_u32] = ACTIONS(3932),
    [anon_sym_i32] = ACTIONS(3932),
    [anon_sym_u64] = ACTIONS(3932),
    [anon_sym_i64] = ACTIONS(3932),
    [anon_sym_u128] = ACTIONS(3932),
    [anon_sym_i128] = ACTIONS(3932),
    [anon_sym_isize] = ACTIONS(3932),
    [anon_sym_usize] = ACTIONS(3932),
    [anon_sym_f32] = ACTIONS(3932),
    [anon_sym_f64] = ACTIONS(3932),
    [anon_sym_bool] = ACTIONS(3932),
    [anon_sym_str] = ACTIONS(3932),
    [anon_sym_char] = ACTIONS(3932),
    [anon_sym_DASH] = ACTIONS(3930),
    [anon_sym_AMP] = ACTIONS(3930),
    [anon_sym_PIPE] = ACTIONS(3930),
    [anon_sym_LT] = ACTIONS(3930),
    [anon_sym_DOT_DOT] = ACTIONS(3930),
    [anon_sym_COLON_COLON] = ACTIONS(3930),
    [anon_sym_POUND] = ACTIONS(3930),
    [anon_sym_SQUOTE] = ACTIONS(3932),
    [anon_sym_async] = ACTIONS(3932),
    [anon_sym_auto] = ACTIONS(3932),
    [anon_sym_await] = ACTIONS(3932),
    [anon_sym_become] = ACTIONS(3932),
    [anon_sym_break] = ACTIONS(3932),
    [anon_sym_const] = ACTIONS(3932),
    [anon_sym_continue] = ACTIONS(3932),
    [anon_sym_default] = ACTIONS(3932),
    [anon_sym_enum] = ACTIONS(3932),
    [anon_sym_fn] = ACTIONS(3932),
    [anon_sym_for] = ACTIONS(3932),
    [anon_sym_gen] = ACTIONS(3932),
    [anon_sym_if] = ACTIONS(3932),
    [anon_sym_impl] = ACTIONS(3932),
    [anon_sym_let] = ACTIONS(3932),
    [anon_sym_loop] = ACTIONS(3932),
    [anon_sym_match] = ACTIONS(3932),
    [anon_sym_mod] = ACTIONS(3932),
    [anon_sym_pub] = ACTIONS(3932),
    [anon_sym_return] = ACTIONS(3932),
    [anon_sym_raw] = ACTIONS(3932),
    [anon_sym_static] = ACTIONS(3932),
    [anon_sym_struct] = ACTIONS(3932),
    [anon_sym_trait] = ACTIONS(3932),
    [anon_sym_type] = ACTIONS(3932),
    [anon_sym_union] = ACTIONS(3932),
    [anon_sym_unsafe] = ACTIONS(3932),
    [anon_sym_use] = ACTIONS(3932),
    [anon_sym_while] = ACTIONS(3932),
    [anon_sym_extern] = ACTIONS(3932),
    [anon_sym_safe] = ACTIONS(3932),
    [anon_sym_yield] = ACTIONS(3932),
    [anon_sym_move] = ACTIONS(3932),
    [anon_sym_try] = ACTIONS(3932),
    [sym_integer_literal] = ACTIONS(3930),
    [aux_sym_string_literal_token1] = ACTIONS(3930),
    [sym_char_literal] = ACTIONS(3930),
    [anon_sym_true] = ACTIONS(3932),
    [anon_sym_false] = ACTIONS(3932),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3932),
    [sym_super] = ACTIONS(3932),
    [sym_crate] = ACTIONS(3932),
    [aux_sym_metavariable_token1] = ACTIONS(3930),
    [sym__raw_string_literal_start] = ACTIONS(3930),
    [sym_float_literal] = ACTIONS(3930),
    [sym__outer_block_doc_comment_start] = ACTIONS(3930),
    [sym__outer_line_doc_comment_start] = ACTIONS(3930),
  },
  [STATE(1041)] = {
    [sym_line_comment] = STATE(1041),
    [sym_block_comment] = STATE(1041),
    [ts_builtin_sym_end] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_macro] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_macro_rules] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3934),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_STAR] = ACTIONS(3934),
    [anon_sym_u8] = ACTIONS(3936),
    [anon_sym_i8] = ACTIONS(3936),
    [anon_sym_u16] = ACTIONS(3936),
    [anon_sym_i16] = ACTIONS(3936),
    [anon_sym_u32] = ACTIONS(3936),
    [anon_sym_i32] = ACTIONS(3936),
    [anon_sym_u64] = ACTIONS(3936),
    [anon_sym_i64] = ACTIONS(3936),
    [anon_sym_u128] = ACTIONS(3936),
    [anon_sym_i128] = ACTIONS(3936),
    [anon_sym_isize] = ACTIONS(3936),
    [anon_sym_usize] = ACTIONS(3936),
    [anon_sym_f32] = ACTIONS(3936),
    [anon_sym_f64] = ACTIONS(3936),
    [anon_sym_bool] = ACTIONS(3936),
    [anon_sym_str] = ACTIONS(3936),
    [anon_sym_char] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3934),
    [anon_sym_AMP] = ACTIONS(3934),
    [anon_sym_PIPE] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3934),
    [anon_sym_DOT_DOT] = ACTIONS(3934),
    [anon_sym_COLON_COLON] = ACTIONS(3934),
    [anon_sym_POUND] = ACTIONS(3934),
    [anon_sym_SQUOTE] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_auto] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_become] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_default] = ACTIONS(3936),
    [anon_sym_enum] = ACTIONS(3936),
    [anon_sym_fn] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_gen] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_impl] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_loop] = ACTIONS(3936),
    [anon_sym_match] = ACTIONS(3936),
    [anon_sym_mod] = ACTIONS(3936),
    [anon_sym_pub] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_raw] = ACTIONS(3936),
    [anon_sym_static] = ACTIONS(3936),
    [anon_sym_struct] = ACTIONS(3936),
    [anon_sym_trait] = ACTIONS(3936),
    [anon_sym_type] = ACTIONS(3936),
    [anon_sym_union] = ACTIONS(3936),
    [anon_sym_unsafe] = ACTIONS(3936),
    [anon_sym_use] = ACTIONS(3936),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_extern] = ACTIONS(3936),
    [anon_sym_safe] = ACTIONS(3936),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_move] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [sym_integer_literal] = ACTIONS(3934),
    [aux_sym_string_literal_token1] = ACTIONS(3934),
    [sym_char_literal] = ACTIONS(3934),
    [anon_sym_true] = ACTIONS(3936),
    [anon_sym_false] = ACTIONS(3936),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_crate] = ACTIONS(3936),
    [aux_sym_metavariable_token1] = ACTIONS(3934),
    [sym__raw_string_literal_start] = ACTIONS(3934),
    [sym_float_literal] = ACTIONS(3934),
    [sym__outer_block_doc_comment_start] = ACTIONS(3934),
    [sym__outer_line_doc_comment_start] = ACTIONS(3934),
  },
  [STATE(1042)] = {
    [sym_line_comment] = STATE(1042),
    [sym_block_comment] = STATE(1042),
    [ts_builtin_sym_end] = ACTIONS(3938),
    [sym_identifier] = ACTIONS(3940),
    [anon_sym_SEMI] = ACTIONS(3938),
    [anon_sym_macro] = ACTIONS(3940),
    [anon_sym_LPAREN] = ACTIONS(3938),
    [anon_sym_LBRACE] = ACTIONS(3938),
    [anon_sym_RBRACE] = ACTIONS(3938),
    [anon_sym_macro_rules] = ACTIONS(3940),
    [anon_sym_BANG] = ACTIONS(3938),
    [anon_sym_LBRACK] = ACTIONS(3938),
    [anon_sym_STAR] = ACTIONS(3938),
    [anon_sym_u8] = ACTIONS(3940),
    [anon_sym_i8] = ACTIONS(3940),
    [anon_sym_u16] = ACTIONS(3940),
    [anon_sym_i16] = ACTIONS(3940),
    [anon_sym_u32] = ACTIONS(3940),
    [anon_sym_i32] = ACTIONS(3940),
    [anon_sym_u64] = ACTIONS(3940),
    [anon_sym_i64] = ACTIONS(3940),
    [anon_sym_u128] = ACTIONS(3940),
    [anon_sym_i128] = ACTIONS(3940),
    [anon_sym_isize] = ACTIONS(3940),
    [anon_sym_usize] = ACTIONS(3940),
    [anon_sym_f32] = ACTIONS(3940),
    [anon_sym_f64] = ACTIONS(3940),
    [anon_sym_bool] = ACTIONS(3940),
    [anon_sym_str] = ACTIONS(3940),
    [anon_sym_char] = ACTIONS(3940),
    [anon_sym_DASH] = ACTIONS(3938),
    [anon_sym_AMP] = ACTIONS(3938),
    [anon_sym_PIPE] = ACTIONS(3938),
    [anon_sym_LT] = ACTIONS(3938),
    [anon_sym_DOT_DOT] = ACTIONS(3938),
    [anon_sym_COLON_COLON] = ACTIONS(3938),
    [anon_sym_POUND] = ACTIONS(3938),
    [anon_sym_SQUOTE] = ACTIONS(3940),
    [anon_sym_async] = ACTIONS(3940),
    [anon_sym_auto] = ACTIONS(3940),
    [anon_sym_await] = ACTIONS(3940),
    [anon_sym_become] = ACTIONS(3940),
    [anon_sym_break] = ACTIONS(3940),
    [anon_sym_const] = ACTIONS(3940),
    [anon_sym_continue] = ACTIONS(3940),
    [anon_sym_default] = ACTIONS(3940),
    [anon_sym_enum] = ACTIONS(3940),
    [anon_sym_fn] = ACTIONS(3940),
    [anon_sym_for] = ACTIONS(3940),
    [anon_sym_gen] = ACTIONS(3940),
    [anon_sym_if] = ACTIONS(3940),
    [anon_sym_impl] = ACTIONS(3940),
    [anon_sym_let] = ACTIONS(3940),
    [anon_sym_loop] = ACTIONS(3940),
    [anon_sym_match] = ACTIONS(3940),
    [anon_sym_mod] = ACTIONS(3940),
    [anon_sym_pub] = ACTIONS(3940),
    [anon_sym_return] = ACTIONS(3940),
    [anon_sym_raw] = ACTIONS(3940),
    [anon_sym_static] = ACTIONS(3940),
    [anon_sym_struct] = ACTIONS(3940),
    [anon_sym_trait] = ACTIONS(3940),
    [anon_sym_type] = ACTIONS(3940),
    [anon_sym_union] = ACTIONS(3940),
    [anon_sym_unsafe] = ACTIONS(3940),
    [anon_sym_use] = ACTIONS(3940),
    [anon_sym_while] = ACTIONS(3940),
    [anon_sym_extern] = ACTIONS(3940),
    [anon_sym_safe] = ACTIONS(3940),
    [anon_sym_yield] = ACTIONS(3940),
    [anon_sym_move] = ACTIONS(3940),
    [anon_sym_try] = ACTIONS(3940),
    [sym_integer_literal] = ACTIONS(3938),
    [aux_sym_string_literal_token1] = ACTIONS(3938),
    [sym_char_literal] = ACTIONS(3938),
    [anon_sym_true] = ACTIONS(3940),
    [anon_sym_false] = ACTIONS(3940),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3940),
    [sym_super] = ACTIONS(3940),
    [sym_crate] = ACTIONS(3940),
    [aux_sym_metavariable_token1] = ACTIONS(3938),
    [sym__raw_string_literal_start] = ACTIONS(3938),
    [sym_float_literal] = ACTIONS(3938),
    [sym__outer_block_doc_comment_start] = ACTIONS(3938),
    [sym__outer_line_doc_comment_start] = ACTIONS(3938),
  },
  [STATE(1043)] = {
    [sym_line_comment] = STATE(1043),
    [sym_block_comment] = STATE(1043),
    [ts_builtin_sym_end] = ACTIONS(3942),
    [sym_identifier] = ACTIONS(3944),
    [anon_sym_SEMI] = ACTIONS(3942),
    [anon_sym_macro] = ACTIONS(3944),
    [anon_sym_LPAREN] = ACTIONS(3942),
    [anon_sym_LBRACE] = ACTIONS(3942),
    [anon_sym_RBRACE] = ACTIONS(3942),
    [anon_sym_macro_rules] = ACTIONS(3944),
    [anon_sym_BANG] = ACTIONS(3942),
    [anon_sym_LBRACK] = ACTIONS(3942),
    [anon_sym_STAR] = ACTIONS(3942),
    [anon_sym_u8] = ACTIONS(3944),
    [anon_sym_i8] = ACTIONS(3944),
    [anon_sym_u16] = ACTIONS(3944),
    [anon_sym_i16] = ACTIONS(3944),
    [anon_sym_u32] = ACTIONS(3944),
    [anon_sym_i32] = ACTIONS(3944),
    [anon_sym_u64] = ACTIONS(3944),
    [anon_sym_i64] = ACTIONS(3944),
    [anon_sym_u128] = ACTIONS(3944),
    [anon_sym_i128] = ACTIONS(3944),
    [anon_sym_isize] = ACTIONS(3944),
    [anon_sym_usize] = ACTIONS(3944),
    [anon_sym_f32] = ACTIONS(3944),
    [anon_sym_f64] = ACTIONS(3944),
    [anon_sym_bool] = ACTIONS(3944),
    [anon_sym_str] = ACTIONS(3944),
    [anon_sym_char] = ACTIONS(3944),
    [anon_sym_DASH] = ACTIONS(3942),
    [anon_sym_AMP] = ACTIONS(3942),
    [anon_sym_PIPE] = ACTIONS(3942),
    [anon_sym_LT] = ACTIONS(3942),
    [anon_sym_DOT_DOT] = ACTIONS(3942),
    [anon_sym_COLON_COLON] = ACTIONS(3942),
    [anon_sym_POUND] = ACTIONS(3942),
    [anon_sym_SQUOTE] = ACTIONS(3944),
    [anon_sym_async] = ACTIONS(3944),
    [anon_sym_auto] = ACTIONS(3944),
    [anon_sym_await] = ACTIONS(3944),
    [anon_sym_become] = ACTIONS(3944),
    [anon_sym_break] = ACTIONS(3944),
    [anon_sym_const] = ACTIONS(3944),
    [anon_sym_continue] = ACTIONS(3944),
    [anon_sym_default] = ACTIONS(3944),
    [anon_sym_enum] = ACTIONS(3944),
    [anon_sym_fn] = ACTIONS(3944),
    [anon_sym_for] = ACTIONS(3944),
    [anon_sym_gen] = ACTIONS(3944),
    [anon_sym_if] = ACTIONS(3944),
    [anon_sym_impl] = ACTIONS(3944),
    [anon_sym_let] = ACTIONS(3944),
    [anon_sym_loop] = ACTIONS(3944),
    [anon_sym_match] = ACTIONS(3944),
    [anon_sym_mod] = ACTIONS(3944),
    [anon_sym_pub] = ACTIONS(3944),
    [anon_sym_return] = ACTIONS(3944),
    [anon_sym_raw] = ACTIONS(3944),
    [anon_sym_static] = ACTIONS(3944),
    [anon_sym_struct] = ACTIONS(3944),
    [anon_sym_trait] = ACTIONS(3944),
    [anon_sym_type] = ACTIONS(3944),
    [anon_sym_union] = ACTIONS(3944),
    [anon_sym_unsafe] = ACTIONS(3944),
    [anon_sym_use] = ACTIONS(3944),
    [anon_sym_while] = ACTIONS(3944),
    [anon_sym_extern] = ACTIONS(3944),
    [anon_sym_safe] = ACTIONS(3944),
    [anon_sym_yield] = ACTIONS(3944),
    [anon_sym_move] = ACTIONS(3944),
    [anon_sym_try] = ACTIONS(3944),
    [sym_integer_literal] = ACTIONS(3942),
    [aux_sym_string_literal_token1] = ACTIONS(3942),
    [sym_char_literal] = ACTIONS(3942),
    [anon_sym_true] = ACTIONS(3944),
    [anon_sym_false] = ACTIONS(3944),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3944),
    [sym_super] = ACTIONS(3944),
    [sym_crate] = ACTIONS(3944),
    [aux_sym_metavariable_token1] = ACTIONS(3942),
    [sym__raw_string_literal_start] = ACTIONS(3942),
    [sym_float_literal] = ACTIONS(3942),
    [sym__outer_block_doc_comment_start] = ACTIONS(3942),
    [sym__outer_line_doc_comment_start] = ACTIONS(3942),
  },
  [STATE(1044)] = {
    [sym_line_comment] = STATE(1044),
    [sym_block_comment] = STATE(1044),
    [ts_builtin_sym_end] = ACTIONS(3946),
    [sym_identifier] = ACTIONS(3948),
    [anon_sym_SEMI] = ACTIONS(3946),
    [anon_sym_macro] = ACTIONS(3948),
    [anon_sym_LPAREN] = ACTIONS(3946),
    [anon_sym_LBRACE] = ACTIONS(3946),
    [anon_sym_RBRACE] = ACTIONS(3946),
    [anon_sym_macro_rules] = ACTIONS(3948),
    [anon_sym_BANG] = ACTIONS(3946),
    [anon_sym_LBRACK] = ACTIONS(3946),
    [anon_sym_STAR] = ACTIONS(3946),
    [anon_sym_u8] = ACTIONS(3948),
    [anon_sym_i8] = ACTIONS(3948),
    [anon_sym_u16] = ACTIONS(3948),
    [anon_sym_i16] = ACTIONS(3948),
    [anon_sym_u32] = ACTIONS(3948),
    [anon_sym_i32] = ACTIONS(3948),
    [anon_sym_u64] = ACTIONS(3948),
    [anon_sym_i64] = ACTIONS(3948),
    [anon_sym_u128] = ACTIONS(3948),
    [anon_sym_i128] = ACTIONS(3948),
    [anon_sym_isize] = ACTIONS(3948),
    [anon_sym_usize] = ACTIONS(3948),
    [anon_sym_f32] = ACTIONS(3948),
    [anon_sym_f64] = ACTIONS(3948),
    [anon_sym_bool] = ACTIONS(3948),
    [anon_sym_str] = ACTIONS(3948),
    [anon_sym_char] = ACTIONS(3948),
    [anon_sym_DASH] = ACTIONS(3946),
    [anon_sym_AMP] = ACTIONS(3946),
    [anon_sym_PIPE] = ACTIONS(3946),
    [anon_sym_LT] = ACTIONS(3946),
    [anon_sym_DOT_DOT] = ACTIONS(3946),
    [anon_sym_COLON_COLON] = ACTIONS(3946),
    [anon_sym_POUND] = ACTIONS(3946),
    [anon_sym_SQUOTE] = ACTIONS(3948),
    [anon_sym_async] = ACTIONS(3948),
    [anon_sym_auto] = ACTIONS(3948),
    [anon_sym_await] = ACTIONS(3948),
    [anon_sym_become] = ACTIONS(3948),
    [anon_sym_break] = ACTIONS(3948),
    [anon_sym_const] = ACTIONS(3948),
    [anon_sym_continue] = ACTIONS(3948),
    [anon_sym_default] = ACTIONS(3948),
    [anon_sym_enum] = ACTIONS(3948),
    [anon_sym_fn] = ACTIONS(3948),
    [anon_sym_for] = ACTIONS(3948),
    [anon_sym_gen] = ACTIONS(3948),
    [anon_sym_if] = ACTIONS(3948),
    [anon_sym_impl] = ACTIONS(3948),
    [anon_sym_let] = ACTIONS(3948),
    [anon_sym_loop] = ACTIONS(3948),
    [anon_sym_match] = ACTIONS(3948),
    [anon_sym_mod] = ACTIONS(3948),
    [anon_sym_pub] = ACTIONS(3948),
    [anon_sym_return] = ACTIONS(3948),
    [anon_sym_raw] = ACTIONS(3948),
    [anon_sym_static] = ACTIONS(3948),
    [anon_sym_struct] = ACTIONS(3948),
    [anon_sym_trait] = ACTIONS(3948),
    [anon_sym_type] = ACTIONS(3948),
    [anon_sym_union] = ACTIONS(3948),
    [anon_sym_unsafe] = ACTIONS(3948),
    [anon_sym_use] = ACTIONS(3948),
    [anon_sym_while] = ACTIONS(3948),
    [anon_sym_extern] = ACTIONS(3948),
    [anon_sym_safe] = ACTIONS(3948),
    [anon_sym_yield] = ACTIONS(3948),
    [anon_sym_move] = ACTIONS(3948),
    [anon_sym_try] = ACTIONS(3948),
    [sym_integer_literal] = ACTIONS(3946),
    [aux_sym_string_literal_token1] = ACTIONS(3946),
    [sym_char_literal] = ACTIONS(3946),
    [anon_sym_true] = ACTIONS(3948),
    [anon_sym_false] = ACTIONS(3948),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3948),
    [sym_super] = ACTIONS(3948),
    [sym_crate] = ACTIONS(3948),
    [aux_sym_metavariable_token1] = ACTIONS(3946),
    [sym__raw_string_literal_start] = ACTIONS(3946),
    [sym_float_literal] = ACTIONS(3946),
    [sym__outer_block_doc_comment_start] = ACTIONS(3946),
    [sym__outer_line_doc_comment_start] = ACTIONS(3946),
  },
  [STATE(1045)] = {
    [sym_line_comment] = STATE(1045),
    [sym_block_comment] = STATE(1045),
    [ts_builtin_sym_end] = ACTIONS(3950),
    [sym_identifier] = ACTIONS(3952),
    [anon_sym_SEMI] = ACTIONS(3950),
    [anon_sym_macro] = ACTIONS(3952),
    [anon_sym_LPAREN] = ACTIONS(3950),
    [anon_sym_LBRACE] = ACTIONS(3950),
    [anon_sym_RBRACE] = ACTIONS(3950),
    [anon_sym_macro_rules] = ACTIONS(3952),
    [anon_sym_BANG] = ACTIONS(3950),
    [anon_sym_LBRACK] = ACTIONS(3950),
    [anon_sym_STAR] = ACTIONS(3950),
    [anon_sym_u8] = ACTIONS(3952),
    [anon_sym_i8] = ACTIONS(3952),
    [anon_sym_u16] = ACTIONS(3952),
    [anon_sym_i16] = ACTIONS(3952),
    [anon_sym_u32] = ACTIONS(3952),
    [anon_sym_i32] = ACTIONS(3952),
    [anon_sym_u64] = ACTIONS(3952),
    [anon_sym_i64] = ACTIONS(3952),
    [anon_sym_u128] = ACTIONS(3952),
    [anon_sym_i128] = ACTIONS(3952),
    [anon_sym_isize] = ACTIONS(3952),
    [anon_sym_usize] = ACTIONS(3952),
    [anon_sym_f32] = ACTIONS(3952),
    [anon_sym_f64] = ACTIONS(3952),
    [anon_sym_bool] = ACTIONS(3952),
    [anon_sym_str] = ACTIONS(3952),
    [anon_sym_char] = ACTIONS(3952),
    [anon_sym_DASH] = ACTIONS(3950),
    [anon_sym_AMP] = ACTIONS(3950),
    [anon_sym_PIPE] = ACTIONS(3950),
    [anon_sym_LT] = ACTIONS(3950),
    [anon_sym_DOT_DOT] = ACTIONS(3950),
    [anon_sym_COLON_COLON] = ACTIONS(3950),
    [anon_sym_POUND] = ACTIONS(3950),
    [anon_sym_SQUOTE] = ACTIONS(3952),
    [anon_sym_async] = ACTIONS(3952),
    [anon_sym_auto] = ACTIONS(3952),
    [anon_sym_await] = ACTIONS(3952),
    [anon_sym_become] = ACTIONS(3952),
    [anon_sym_break] = ACTIONS(3952),
    [anon_sym_const] = ACTIONS(3952),
    [anon_sym_continue] = ACTIONS(3952),
    [anon_sym_default] = ACTIONS(3952),
    [anon_sym_enum] = ACTIONS(3952),
    [anon_sym_fn] = ACTIONS(3952),
    [anon_sym_for] = ACTIONS(3952),
    [anon_sym_gen] = ACTIONS(3952),
    [anon_sym_if] = ACTIONS(3952),
    [anon_sym_impl] = ACTIONS(3952),
    [anon_sym_let] = ACTIONS(3952),
    [anon_sym_loop] = ACTIONS(3952),
    [anon_sym_match] = ACTIONS(3952),
    [anon_sym_mod] = ACTIONS(3952),
    [anon_sym_pub] = ACTIONS(3952),
    [anon_sym_return] = ACTIONS(3952),
    [anon_sym_raw] = ACTIONS(3952),
    [anon_sym_static] = ACTIONS(3952),
    [anon_sym_struct] = ACTIONS(3952),
    [anon_sym_trait] = ACTIONS(3952),
    [anon_sym_type] = ACTIONS(3952),
    [anon_sym_union] = ACTIONS(3952),
    [anon_sym_unsafe] = ACTIONS(3952),
    [anon_sym_use] = ACTIONS(3952),
    [anon_sym_while] = ACTIONS(3952),
    [anon_sym_extern] = ACTIONS(3952),
    [anon_sym_safe] = ACTIONS(3952),
    [anon_sym_yield] = ACTIONS(3952),
    [anon_sym_move] = ACTIONS(3952),
    [anon_sym_try] = ACTIONS(3952),
    [sym_integer_literal] = ACTIONS(3950),
    [aux_sym_string_literal_token1] = ACTIONS(3950),
    [sym_char_literal] = ACTIONS(3950),
    [anon_sym_true] = ACTIONS(3952),
    [anon_sym_false] = ACTIONS(3952),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3952),
    [sym_super] = ACTIONS(3952),
    [sym_crate] = ACTIONS(3952),
    [aux_sym_metavariable_token1] = ACTIONS(3950),
    [sym__raw_string_literal_start] = ACTIONS(3950),
    [sym_float_literal] = ACTIONS(3950),
    [sym__outer_block_doc_comment_start] = ACTIONS(3950),
    [sym__outer_line_doc_comment_start] = ACTIONS(3950),
  },
  [STATE(1046)] = {
    [sym_line_comment] = STATE(1046),
    [sym_block_comment] = STATE(1046),
    [ts_builtin_sym_end] = ACTIONS(3954),
    [sym_identifier] = ACTIONS(3956),
    [anon_sym_SEMI] = ACTIONS(3954),
    [anon_sym_macro] = ACTIONS(3956),
    [anon_sym_LPAREN] = ACTIONS(3954),
    [anon_sym_LBRACE] = ACTIONS(3954),
    [anon_sym_RBRACE] = ACTIONS(3954),
    [anon_sym_macro_rules] = ACTIONS(3956),
    [anon_sym_BANG] = ACTIONS(3954),
    [anon_sym_LBRACK] = ACTIONS(3954),
    [anon_sym_STAR] = ACTIONS(3954),
    [anon_sym_u8] = ACTIONS(3956),
    [anon_sym_i8] = ACTIONS(3956),
    [anon_sym_u16] = ACTIONS(3956),
    [anon_sym_i16] = ACTIONS(3956),
    [anon_sym_u32] = ACTIONS(3956),
    [anon_sym_i32] = ACTIONS(3956),
    [anon_sym_u64] = ACTIONS(3956),
    [anon_sym_i64] = ACTIONS(3956),
    [anon_sym_u128] = ACTIONS(3956),
    [anon_sym_i128] = ACTIONS(3956),
    [anon_sym_isize] = ACTIONS(3956),
    [anon_sym_usize] = ACTIONS(3956),
    [anon_sym_f32] = ACTIONS(3956),
    [anon_sym_f64] = ACTIONS(3956),
    [anon_sym_bool] = ACTIONS(3956),
    [anon_sym_str] = ACTIONS(3956),
    [anon_sym_char] = ACTIONS(3956),
    [anon_sym_DASH] = ACTIONS(3954),
    [anon_sym_AMP] = ACTIONS(3954),
    [anon_sym_PIPE] = ACTIONS(3954),
    [anon_sym_LT] = ACTIONS(3954),
    [anon_sym_DOT_DOT] = ACTIONS(3954),
    [anon_sym_COLON_COLON] = ACTIONS(3954),
    [anon_sym_POUND] = ACTIONS(3954),
    [anon_sym_SQUOTE] = ACTIONS(3956),
    [anon_sym_async] = ACTIONS(3956),
    [anon_sym_auto] = ACTIONS(3956),
    [anon_sym_await] = ACTIONS(3956),
    [anon_sym_become] = ACTIONS(3956),
    [anon_sym_break] = ACTIONS(3956),
    [anon_sym_const] = ACTIONS(3956),
    [anon_sym_continue] = ACTIONS(3956),
    [anon_sym_default] = ACTIONS(3956),
    [anon_sym_enum] = ACTIONS(3956),
    [anon_sym_fn] = ACTIONS(3956),
    [anon_sym_for] = ACTIONS(3956),
    [anon_sym_gen] = ACTIONS(3956),
    [anon_sym_if] = ACTIONS(3956),
    [anon_sym_impl] = ACTIONS(3956),
    [anon_sym_let] = ACTIONS(3956),
    [anon_sym_loop] = ACTIONS(3956),
    [anon_sym_match] = ACTIONS(3956),
    [anon_sym_mod] = ACTIONS(3956),
    [anon_sym_pub] = ACTIONS(3956),
    [anon_sym_return] = ACTIONS(3956),
    [anon_sym_raw] = ACTIONS(3956),
    [anon_sym_static] = ACTIONS(3956),
    [anon_sym_struct] = ACTIONS(3956),
    [anon_sym_trait] = ACTIONS(3956),
    [anon_sym_type] = ACTIONS(3956),
    [anon_sym_union] = ACTIONS(3956),
    [anon_sym_unsafe] = ACTIONS(3956),
    [anon_sym_use] = ACTIONS(3956),
    [anon_sym_while] = ACTIONS(3956),
    [anon_sym_extern] = ACTIONS(3956),
    [anon_sym_safe] = ACTIONS(3956),
    [anon_sym_yield] = ACTIONS(3956),
    [anon_sym_move] = ACTIONS(3956),
    [anon_sym_try] = ACTIONS(3956),
    [sym_integer_literal] = ACTIONS(3954),
    [aux_sym_string_literal_token1] = ACTIONS(3954),
    [sym_char_literal] = ACTIONS(3954),
    [anon_sym_true] = ACTIONS(3956),
    [anon_sym_false] = ACTIONS(3956),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3956),
    [sym_super] = ACTIONS(3956),
    [sym_crate] = ACTIONS(3956),
    [aux_sym_metavariable_token1] = ACTIONS(3954),
    [sym__raw_string_literal_start] = ACTIONS(3954),
    [sym_float_literal] = ACTIONS(3954),
    [sym__outer_block_doc_comment_start] = ACTIONS(3954),
    [sym__outer_line_doc_comment_start] = ACTIONS(3954),
  },
  [STATE(1047)] = {
    [sym_line_comment] = STATE(1047),
    [sym_block_comment] = STATE(1047),
    [ts_builtin_sym_end] = ACTIONS(3958),
    [sym_identifier] = ACTIONS(3960),
    [anon_sym_SEMI] = ACTIONS(3958),
    [anon_sym_macro] = ACTIONS(3960),
    [anon_sym_LPAREN] = ACTIONS(3958),
    [anon_sym_LBRACE] = ACTIONS(3958),
    [anon_sym_RBRACE] = ACTIONS(3958),
    [anon_sym_macro_rules] = ACTIONS(3960),
    [anon_sym_BANG] = ACTIONS(3958),
    [anon_sym_LBRACK] = ACTIONS(3958),
    [anon_sym_STAR] = ACTIONS(3958),
    [anon_sym_u8] = ACTIONS(3960),
    [anon_sym_i8] = ACTIONS(3960),
    [anon_sym_u16] = ACTIONS(3960),
    [anon_sym_i16] = ACTIONS(3960),
    [anon_sym_u32] = ACTIONS(3960),
    [anon_sym_i32] = ACTIONS(3960),
    [anon_sym_u64] = ACTIONS(3960),
    [anon_sym_i64] = ACTIONS(3960),
    [anon_sym_u128] = ACTIONS(3960),
    [anon_sym_i128] = ACTIONS(3960),
    [anon_sym_isize] = ACTIONS(3960),
    [anon_sym_usize] = ACTIONS(3960),
    [anon_sym_f32] = ACTIONS(3960),
    [anon_sym_f64] = ACTIONS(3960),
    [anon_sym_bool] = ACTIONS(3960),
    [anon_sym_str] = ACTIONS(3960),
    [anon_sym_char] = ACTIONS(3960),
    [anon_sym_DASH] = ACTIONS(3958),
    [anon_sym_AMP] = ACTIONS(3958),
    [anon_sym_PIPE] = ACTIONS(3958),
    [anon_sym_LT] = ACTIONS(3958),
    [anon_sym_DOT_DOT] = ACTIONS(3958),
    [anon_sym_COLON_COLON] = ACTIONS(3958),
    [anon_sym_POUND] = ACTIONS(3958),
    [anon_sym_SQUOTE] = ACTIONS(3960),
    [anon_sym_async] = ACTIONS(3960),
    [anon_sym_auto] = ACTIONS(3960),
    [anon_sym_await] = ACTIONS(3960),
    [anon_sym_become] = ACTIONS(3960),
    [anon_sym_break] = ACTIONS(3960),
    [anon_sym_const] = ACTIONS(3960),
    [anon_sym_continue] = ACTIONS(3960),
    [anon_sym_default] = ACTIONS(3960),
    [anon_sym_enum] = ACTIONS(3960),
    [anon_sym_fn] = ACTIONS(3960),
    [anon_sym_for] = ACTIONS(3960),
    [anon_sym_gen] = ACTIONS(3960),
    [anon_sym_if] = ACTIONS(3960),
    [anon_sym_impl] = ACTIONS(3960),
    [anon_sym_let] = ACTIONS(3960),
    [anon_sym_loop] = ACTIONS(3960),
    [anon_sym_match] = ACTIONS(3960),
    [anon_sym_mod] = ACTIONS(3960),
    [anon_sym_pub] = ACTIONS(3960),
    [anon_sym_return] = ACTIONS(3960),
    [anon_sym_raw] = ACTIONS(3960),
    [anon_sym_static] = ACTIONS(3960),
    [anon_sym_struct] = ACTIONS(3960),
    [anon_sym_trait] = ACTIONS(3960),
    [anon_sym_type] = ACTIONS(3960),
    [anon_sym_union] = ACTIONS(3960),
    [anon_sym_unsafe] = ACTIONS(3960),
    [anon_sym_use] = ACTIONS(3960),
    [anon_sym_while] = ACTIONS(3960),
    [anon_sym_extern] = ACTIONS(3960),
    [anon_sym_safe] = ACTIONS(3960),
    [anon_sym_yield] = ACTIONS(3960),
    [anon_sym_move] = ACTIONS(3960),
    [anon_sym_try] = ACTIONS(3960),
    [sym_integer_literal] = ACTIONS(3958),
    [aux_sym_string_literal_token1] = ACTIONS(3958),
    [sym_char_literal] = ACTIONS(3958),
    [anon_sym_true] = ACTIONS(3960),
    [anon_sym_false] = ACTIONS(3960),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3960),
    [sym_super] = ACTIONS(3960),
    [sym_crate] = ACTIONS(3960),
    [aux_sym_metavariable_token1] = ACTIONS(3958),
    [sym__raw_string_literal_start] = ACTIONS(3958),
    [sym_float_literal] = ACTIONS(3958),
    [sym__outer_block_doc_comment_start] = ACTIONS(3958),
    [sym__outer_line_doc_comment_start] = ACTIONS(3958),
  },
  [STATE(1048)] = {
    [sym_line_comment] = STATE(1048),
    [sym_block_comment] = STATE(1048),
    [ts_builtin_sym_end] = ACTIONS(3962),
    [sym_identifier] = ACTIONS(3964),
    [anon_sym_SEMI] = ACTIONS(3962),
    [anon_sym_macro] = ACTIONS(3964),
    [anon_sym_LPAREN] = ACTIONS(3962),
    [anon_sym_LBRACE] = ACTIONS(3962),
    [anon_sym_RBRACE] = ACTIONS(3962),
    [anon_sym_macro_rules] = ACTIONS(3964),
    [anon_sym_BANG] = ACTIONS(3962),
    [anon_sym_LBRACK] = ACTIONS(3962),
    [anon_sym_STAR] = ACTIONS(3962),
    [anon_sym_u8] = ACTIONS(3964),
    [anon_sym_i8] = ACTIONS(3964),
    [anon_sym_u16] = ACTIONS(3964),
    [anon_sym_i16] = ACTIONS(3964),
    [anon_sym_u32] = ACTIONS(3964),
    [anon_sym_i32] = ACTIONS(3964),
    [anon_sym_u64] = ACTIONS(3964),
    [anon_sym_i64] = ACTIONS(3964),
    [anon_sym_u128] = ACTIONS(3964),
    [anon_sym_i128] = ACTIONS(3964),
    [anon_sym_isize] = ACTIONS(3964),
    [anon_sym_usize] = ACTIONS(3964),
    [anon_sym_f32] = ACTIONS(3964),
    [anon_sym_f64] = ACTIONS(3964),
    [anon_sym_bool] = ACTIONS(3964),
    [anon_sym_str] = ACTIONS(3964),
    [anon_sym_char] = ACTIONS(3964),
    [anon_sym_DASH] = ACTIONS(3962),
    [anon_sym_AMP] = ACTIONS(3962),
    [anon_sym_PIPE] = ACTIONS(3962),
    [anon_sym_LT] = ACTIONS(3962),
    [anon_sym_DOT_DOT] = ACTIONS(3962),
    [anon_sym_COLON_COLON] = ACTIONS(3962),
    [anon_sym_POUND] = ACTIONS(3962),
    [anon_sym_SQUOTE] = ACTIONS(3964),
    [anon_sym_async] = ACTIONS(3964),
    [anon_sym_auto] = ACTIONS(3964),
    [anon_sym_await] = ACTIONS(3964),
    [anon_sym_become] = ACTIONS(3964),
    [anon_sym_break] = ACTIONS(3964),
    [anon_sym_const] = ACTIONS(3964),
    [anon_sym_continue] = ACTIONS(3964),
    [anon_sym_default] = ACTIONS(3964),
    [anon_sym_enum] = ACTIONS(3964),
    [anon_sym_fn] = ACTIONS(3964),
    [anon_sym_for] = ACTIONS(3964),
    [anon_sym_gen] = ACTIONS(3964),
    [anon_sym_if] = ACTIONS(3964),
    [anon_sym_impl] = ACTIONS(3964),
    [anon_sym_let] = ACTIONS(3964),
    [anon_sym_loop] = ACTIONS(3964),
    [anon_sym_match] = ACTIONS(3964),
    [anon_sym_mod] = ACTIONS(3964),
    [anon_sym_pub] = ACTIONS(3964),
    [anon_sym_return] = ACTIONS(3964),
    [anon_sym_raw] = ACTIONS(3964),
    [anon_sym_static] = ACTIONS(3964),
    [anon_sym_struct] = ACTIONS(3964),
    [anon_sym_trait] = ACTIONS(3964),
    [anon_sym_type] = ACTIONS(3964),
    [anon_sym_union] = ACTIONS(3964),
    [anon_sym_unsafe] = ACTIONS(3964),
    [anon_sym_use] = ACTIONS(3964),
    [anon_sym_while] = ACTIONS(3964),
    [anon_sym_extern] = ACTIONS(3964),
    [anon_sym_safe] = ACTIONS(3964),
    [anon_sym_yield] = ACTIONS(3964),
    [anon_sym_move] = ACTIONS(3964),
    [anon_sym_try] = ACTIONS(3964),
    [sym_integer_literal] = ACTIONS(3962),
    [aux_sym_string_literal_token1] = ACTIONS(3962),
    [sym_char_literal] = ACTIONS(3962),
    [anon_sym_true] = ACTIONS(3964),
    [anon_sym_false] = ACTIONS(3964),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3964),
    [sym_super] = ACTIONS(3964),
    [sym_crate] = ACTIONS(3964),
    [aux_sym_metavariable_token1] = ACTIONS(3962),
    [sym__raw_string_literal_start] = ACTIONS(3962),
    [sym_float_literal] = ACTIONS(3962),
    [sym__outer_block_doc_comment_start] = ACTIONS(3962),
    [sym__outer_line_doc_comment_start] = ACTIONS(3962),
  },
  [STATE(1049)] = {
    [sym_line_comment] = STATE(1049),
    [sym_block_comment] = STATE(1049),
    [ts_builtin_sym_end] = ACTIONS(3966),
    [sym_identifier] = ACTIONS(3968),
    [anon_sym_SEMI] = ACTIONS(3966),
    [anon_sym_macro] = ACTIONS(3968),
    [anon_sym_LPAREN] = ACTIONS(3966),
    [anon_sym_LBRACE] = ACTIONS(3966),
    [anon_sym_RBRACE] = ACTIONS(3966),
    [anon_sym_macro_rules] = ACTIONS(3968),
    [anon_sym_BANG] = ACTIONS(3966),
    [anon_sym_LBRACK] = ACTIONS(3966),
    [anon_sym_STAR] = ACTIONS(3966),
    [anon_sym_u8] = ACTIONS(3968),
    [anon_sym_i8] = ACTIONS(3968),
    [anon_sym_u16] = ACTIONS(3968),
    [anon_sym_i16] = ACTIONS(3968),
    [anon_sym_u32] = ACTIONS(3968),
    [anon_sym_i32] = ACTIONS(3968),
    [anon_sym_u64] = ACTIONS(3968),
    [anon_sym_i64] = ACTIONS(3968),
    [anon_sym_u128] = ACTIONS(3968),
    [anon_sym_i128] = ACTIONS(3968),
    [anon_sym_isize] = ACTIONS(3968),
    [anon_sym_usize] = ACTIONS(3968),
    [anon_sym_f32] = ACTIONS(3968),
    [anon_sym_f64] = ACTIONS(3968),
    [anon_sym_bool] = ACTIONS(3968),
    [anon_sym_str] = ACTIONS(3968),
    [anon_sym_char] = ACTIONS(3968),
    [anon_sym_DASH] = ACTIONS(3966),
    [anon_sym_AMP] = ACTIONS(3966),
    [anon_sym_PIPE] = ACTIONS(3966),
    [anon_sym_LT] = ACTIONS(3966),
    [anon_sym_DOT_DOT] = ACTIONS(3966),
    [anon_sym_COLON_COLON] = ACTIONS(3966),
    [anon_sym_POUND] = ACTIONS(3966),
    [anon_sym_SQUOTE] = ACTIONS(3968),
    [anon_sym_async] = ACTIONS(3968),
    [anon_sym_auto] = ACTIONS(3968),
    [anon_sym_await] = ACTIONS(3968),
    [anon_sym_become] = ACTIONS(3968),
    [anon_sym_break] = ACTIONS(3968),
    [anon_sym_const] = ACTIONS(3968),
    [anon_sym_continue] = ACTIONS(3968),
    [anon_sym_default] = ACTIONS(3968),
    [anon_sym_enum] = ACTIONS(3968),
    [anon_sym_fn] = ACTIONS(3968),
    [anon_sym_for] = ACTIONS(3968),
    [anon_sym_gen] = ACTIONS(3968),
    [anon_sym_if] = ACTIONS(3968),
    [anon_sym_impl] = ACTIONS(3968),
    [anon_sym_let] = ACTIONS(3968),
    [anon_sym_loop] = ACTIONS(3968),
    [anon_sym_match] = ACTIONS(3968),
    [anon_sym_mod] = ACTIONS(3968),
    [anon_sym_pub] = ACTIONS(3968),
    [anon_sym_return] = ACTIONS(3968),
    [anon_sym_raw] = ACTIONS(3968),
    [anon_sym_static] = ACTIONS(3968),
    [anon_sym_struct] = ACTIONS(3968),
    [anon_sym_trait] = ACTIONS(3968),
    [anon_sym_type] = ACTIONS(3968),
    [anon_sym_union] = ACTIONS(3968),
    [anon_sym_unsafe] = ACTIONS(3968),
    [anon_sym_use] = ACTIONS(3968),
    [anon_sym_while] = ACTIONS(3968),
    [anon_sym_extern] = ACTIONS(3968),
    [anon_sym_safe] = ACTIONS(3968),
    [anon_sym_yield] = ACTIONS(3968),
    [anon_sym_move] = ACTIONS(3968),
    [anon_sym_try] = ACTIONS(3968),
    [sym_integer_literal] = ACTIONS(3966),
    [aux_sym_string_literal_token1] = ACTIONS(3966),
    [sym_char_literal] = ACTIONS(3966),
    [anon_sym_true] = ACTIONS(3968),
    [anon_sym_false] = ACTIONS(3968),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3968),
    [sym_super] = ACTIONS(3968),
    [sym_crate] = ACTIONS(3968),
    [aux_sym_metavariable_token1] = ACTIONS(3966),
    [sym__raw_string_literal_start] = ACTIONS(3966),
    [sym_float_literal] = ACTIONS(3966),
    [sym__outer_block_doc_comment_start] = ACTIONS(3966),
    [sym__outer_line_doc_comment_start] = ACTIONS(3966),
  },
  [STATE(1050)] = {
    [sym_line_comment] = STATE(1050),
    [sym_block_comment] = STATE(1050),
    [ts_builtin_sym_end] = ACTIONS(3970),
    [sym_identifier] = ACTIONS(3972),
    [anon_sym_SEMI] = ACTIONS(3970),
    [anon_sym_macro] = ACTIONS(3972),
    [anon_sym_LPAREN] = ACTIONS(3970),
    [anon_sym_LBRACE] = ACTIONS(3970),
    [anon_sym_RBRACE] = ACTIONS(3970),
    [anon_sym_macro_rules] = ACTIONS(3972),
    [anon_sym_BANG] = ACTIONS(3970),
    [anon_sym_LBRACK] = ACTIONS(3970),
    [anon_sym_STAR] = ACTIONS(3970),
    [anon_sym_u8] = ACTIONS(3972),
    [anon_sym_i8] = ACTIONS(3972),
    [anon_sym_u16] = ACTIONS(3972),
    [anon_sym_i16] = ACTIONS(3972),
    [anon_sym_u32] = ACTIONS(3972),
    [anon_sym_i32] = ACTIONS(3972),
    [anon_sym_u64] = ACTIONS(3972),
    [anon_sym_i64] = ACTIONS(3972),
    [anon_sym_u128] = ACTIONS(3972),
    [anon_sym_i128] = ACTIONS(3972),
    [anon_sym_isize] = ACTIONS(3972),
    [anon_sym_usize] = ACTIONS(3972),
    [anon_sym_f32] = ACTIONS(3972),
    [anon_sym_f64] = ACTIONS(3972),
    [anon_sym_bool] = ACTIONS(3972),
    [anon_sym_str] = ACTIONS(3972),
    [anon_sym_char] = ACTIONS(3972),
    [anon_sym_DASH] = ACTIONS(3970),
    [anon_sym_AMP] = ACTIONS(3970),
    [anon_sym_PIPE] = ACTIONS(3970),
    [anon_sym_LT] = ACTIONS(3970),
    [anon_sym_DOT_DOT] = ACTIONS(3970),
    [anon_sym_COLON_COLON] = ACTIONS(3970),
    [anon_sym_POUND] = ACTIONS(3970),
    [anon_sym_SQUOTE] = ACTIONS(3972),
    [anon_sym_async] = ACTIONS(3972),
    [anon_sym_auto] = ACTIONS(3972),
    [anon_sym_await] = ACTIONS(3972),
    [anon_sym_become] = ACTIONS(3972),
    [anon_sym_break] = ACTIONS(3972),
    [anon_sym_const] = ACTIONS(3972),
    [anon_sym_continue] = ACTIONS(3972),
    [anon_sym_default] = ACTIONS(3972),
    [anon_sym_enum] = ACTIONS(3972),
    [anon_sym_fn] = ACTIONS(3972),
    [anon_sym_for] = ACTIONS(3972),
    [anon_sym_gen] = ACTIONS(3972),
    [anon_sym_if] = ACTIONS(3972),
    [anon_sym_impl] = ACTIONS(3972),
    [anon_sym_let] = ACTIONS(3972),
    [anon_sym_loop] = ACTIONS(3972),
    [anon_sym_match] = ACTIONS(3972),
    [anon_sym_mod] = ACTIONS(3972),
    [anon_sym_pub] = ACTIONS(3972),
    [anon_sym_return] = ACTIONS(3972),
    [anon_sym_raw] = ACTIONS(3972),
    [anon_sym_static] = ACTIONS(3972),
    [anon_sym_struct] = ACTIONS(3972),
    [anon_sym_trait] = ACTIONS(3972),
    [anon_sym_type] = ACTIONS(3972),
    [anon_sym_union] = ACTIONS(3972),
    [anon_sym_unsafe] = ACTIONS(3972),
    [anon_sym_use] = ACTIONS(3972),
    [anon_sym_while] = ACTIONS(3972),
    [anon_sym_extern] = ACTIONS(3972),
    [anon_sym_safe] = ACTIONS(3972),
    [anon_sym_yield] = ACTIONS(3972),
    [anon_sym_move] = ACTIONS(3972),
    [anon_sym_try] = ACTIONS(3972),
    [sym_integer_literal] = ACTIONS(3970),
    [aux_sym_string_literal_token1] = ACTIONS(3970),
    [sym_char_literal] = ACTIONS(3970),
    [anon_sym_true] = ACTIONS(3972),
    [anon_sym_false] = ACTIONS(3972),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3972),
    [sym_super] = ACTIONS(3972),
    [sym_crate] = ACTIONS(3972),
    [aux_sym_metavariable_token1] = ACTIONS(3970),
    [sym__raw_string_literal_start] = ACTIONS(3970),
    [sym_float_literal] = ACTIONS(3970),
    [sym__outer_block_doc_comment_start] = ACTIONS(3970),
    [sym__outer_line_doc_comment_start] = ACTIONS(3970),
  },
  [STATE(1051)] = {
    [sym_line_comment] = STATE(1051),
    [sym_block_comment] = STATE(1051),
    [ts_builtin_sym_end] = ACTIONS(3974),
    [sym_identifier] = ACTIONS(3976),
    [anon_sym_SEMI] = ACTIONS(3974),
    [anon_sym_macro] = ACTIONS(3976),
    [anon_sym_LPAREN] = ACTIONS(3974),
    [anon_sym_LBRACE] = ACTIONS(3974),
    [anon_sym_RBRACE] = ACTIONS(3974),
    [anon_sym_macro_rules] = ACTIONS(3976),
    [anon_sym_BANG] = ACTIONS(3974),
    [anon_sym_LBRACK] = ACTIONS(3974),
    [anon_sym_STAR] = ACTIONS(3974),
    [anon_sym_u8] = ACTIONS(3976),
    [anon_sym_i8] = ACTIONS(3976),
    [anon_sym_u16] = ACTIONS(3976),
    [anon_sym_i16] = ACTIONS(3976),
    [anon_sym_u32] = ACTIONS(3976),
    [anon_sym_i32] = ACTIONS(3976),
    [anon_sym_u64] = ACTIONS(3976),
    [anon_sym_i64] = ACTIONS(3976),
    [anon_sym_u128] = ACTIONS(3976),
    [anon_sym_i128] = ACTIONS(3976),
    [anon_sym_isize] = ACTIONS(3976),
    [anon_sym_usize] = ACTIONS(3976),
    [anon_sym_f32] = ACTIONS(3976),
    [anon_sym_f64] = ACTIONS(3976),
    [anon_sym_bool] = ACTIONS(3976),
    [anon_sym_str] = ACTIONS(3976),
    [anon_sym_char] = ACTIONS(3976),
    [anon_sym_DASH] = ACTIONS(3974),
    [anon_sym_AMP] = ACTIONS(3974),
    [anon_sym_PIPE] = ACTIONS(3974),
    [anon_sym_LT] = ACTIONS(3974),
    [anon_sym_DOT_DOT] = ACTIONS(3974),
    [anon_sym_COLON_COLON] = ACTIONS(3974),
    [anon_sym_POUND] = ACTIONS(3974),
    [anon_sym_SQUOTE] = ACTIONS(3976),
    [anon_sym_async] = ACTIONS(3976),
    [anon_sym_auto] = ACTIONS(3976),
    [anon_sym_await] = ACTIONS(3976),
    [anon_sym_become] = ACTIONS(3976),
    [anon_sym_break] = ACTIONS(3976),
    [anon_sym_const] = ACTIONS(3976),
    [anon_sym_continue] = ACTIONS(3976),
    [anon_sym_default] = ACTIONS(3976),
    [anon_sym_enum] = ACTIONS(3976),
    [anon_sym_fn] = ACTIONS(3976),
    [anon_sym_for] = ACTIONS(3976),
    [anon_sym_gen] = ACTIONS(3976),
    [anon_sym_if] = ACTIONS(3976),
    [anon_sym_impl] = ACTIONS(3976),
    [anon_sym_let] = ACTIONS(3976),
    [anon_sym_loop] = ACTIONS(3976),
    [anon_sym_match] = ACTIONS(3976),
    [anon_sym_mod] = ACTIONS(3976),
    [anon_sym_pub] = ACTIONS(3976),
    [anon_sym_return] = ACTIONS(3976),
    [anon_sym_raw] = ACTIONS(3976),
    [anon_sym_static] = ACTIONS(3976),
    [anon_sym_struct] = ACTIONS(3976),
    [anon_sym_trait] = ACTIONS(3976),
    [anon_sym_type] = ACTIONS(3976),
    [anon_sym_union] = ACTIONS(3976),
    [anon_sym_unsafe] = ACTIONS(3976),
    [anon_sym_use] = ACTIONS(3976),
    [anon_sym_while] = ACTIONS(3976),
    [anon_sym_extern] = ACTIONS(3976),
    [anon_sym_safe] = ACTIONS(3976),
    [anon_sym_yield] = ACTIONS(3976),
    [anon_sym_move] = ACTIONS(3976),
    [anon_sym_try] = ACTIONS(3976),
    [sym_integer_literal] = ACTIONS(3974),
    [aux_sym_string_literal_token1] = ACTIONS(3974),
    [sym_char_literal] = ACTIONS(3974),
    [anon_sym_true] = ACTIONS(3976),
    [anon_sym_false] = ACTIONS(3976),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3976),
    [sym_super] = ACTIONS(3976),
    [sym_crate] = ACTIONS(3976),
    [aux_sym_metavariable_token1] = ACTIONS(3974),
    [sym__raw_string_literal_start] = ACTIONS(3974),
    [sym_float_literal] = ACTIONS(3974),
    [sym__outer_block_doc_comment_start] = ACTIONS(3974),
    [sym__outer_line_doc_comment_start] = ACTIONS(3974),
  },
  [STATE(1052)] = {
    [sym_line_comment] = STATE(1052),
    [sym_block_comment] = STATE(1052),
    [ts_builtin_sym_end] = ACTIONS(3978),
    [sym_identifier] = ACTIONS(3980),
    [anon_sym_SEMI] = ACTIONS(3978),
    [anon_sym_macro] = ACTIONS(3980),
    [anon_sym_LPAREN] = ACTIONS(3978),
    [anon_sym_LBRACE] = ACTIONS(3978),
    [anon_sym_RBRACE] = ACTIONS(3978),
    [anon_sym_macro_rules] = ACTIONS(3980),
    [anon_sym_BANG] = ACTIONS(3978),
    [anon_sym_LBRACK] = ACTIONS(3978),
    [anon_sym_STAR] = ACTIONS(3978),
    [anon_sym_u8] = ACTIONS(3980),
    [anon_sym_i8] = ACTIONS(3980),
    [anon_sym_u16] = ACTIONS(3980),
    [anon_sym_i16] = ACTIONS(3980),
    [anon_sym_u32] = ACTIONS(3980),
    [anon_sym_i32] = ACTIONS(3980),
    [anon_sym_u64] = ACTIONS(3980),
    [anon_sym_i64] = ACTIONS(3980),
    [anon_sym_u128] = ACTIONS(3980),
    [anon_sym_i128] = ACTIONS(3980),
    [anon_sym_isize] = ACTIONS(3980),
    [anon_sym_usize] = ACTIONS(3980),
    [anon_sym_f32] = ACTIONS(3980),
    [anon_sym_f64] = ACTIONS(3980),
    [anon_sym_bool] = ACTIONS(3980),
    [anon_sym_str] = ACTIONS(3980),
    [anon_sym_char] = ACTIONS(3980),
    [anon_sym_DASH] = ACTIONS(3978),
    [anon_sym_AMP] = ACTIONS(3978),
    [anon_sym_PIPE] = ACTIONS(3978),
    [anon_sym_LT] = ACTIONS(3978),
    [anon_sym_DOT_DOT] = ACTIONS(3978),
    [anon_sym_COLON_COLON] = ACTIONS(3978),
    [anon_sym_POUND] = ACTIONS(3978),
    [anon_sym_SQUOTE] = ACTIONS(3980),
    [anon_sym_async] = ACTIONS(3980),
    [anon_sym_auto] = ACTIONS(3980),
    [anon_sym_await] = ACTIONS(3980),
    [anon_sym_become] = ACTIONS(3980),
    [anon_sym_break] = ACTIONS(3980),
    [anon_sym_const] = ACTIONS(3980),
    [anon_sym_continue] = ACTIONS(3980),
    [anon_sym_default] = ACTIONS(3980),
    [anon_sym_enum] = ACTIONS(3980),
    [anon_sym_fn] = ACTIONS(3980),
    [anon_sym_for] = ACTIONS(3980),
    [anon_sym_gen] = ACTIONS(3980),
    [anon_sym_if] = ACTIONS(3980),
    [anon_sym_impl] = ACTIONS(3980),
    [anon_sym_let] = ACTIONS(3980),
    [anon_sym_loop] = ACTIONS(3980),
    [anon_sym_match] = ACTIONS(3980),
    [anon_sym_mod] = ACTIONS(3980),
    [anon_sym_pub] = ACTIONS(3980),
    [anon_sym_return] = ACTIONS(3980),
    [anon_sym_raw] = ACTIONS(3980),
    [anon_sym_static] = ACTIONS(3980),
    [anon_sym_struct] = ACTIONS(3980),
    [anon_sym_trait] = ACTIONS(3980),
    [anon_sym_type] = ACTIONS(3980),
    [anon_sym_union] = ACTIONS(3980),
    [anon_sym_unsafe] = ACTIONS(3980),
    [anon_sym_use] = ACTIONS(3980),
    [anon_sym_while] = ACTIONS(3980),
    [anon_sym_extern] = ACTIONS(3980),
    [anon_sym_safe] = ACTIONS(3980),
    [anon_sym_yield] = ACTIONS(3980),
    [anon_sym_move] = ACTIONS(3980),
    [anon_sym_try] = ACTIONS(3980),
    [sym_integer_literal] = ACTIONS(3978),
    [aux_sym_string_literal_token1] = ACTIONS(3978),
    [sym_char_literal] = ACTIONS(3978),
    [anon_sym_true] = ACTIONS(3980),
    [anon_sym_false] = ACTIONS(3980),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3980),
    [sym_super] = ACTIONS(3980),
    [sym_crate] = ACTIONS(3980),
    [aux_sym_metavariable_token1] = ACTIONS(3978),
    [sym__raw_string_literal_start] = ACTIONS(3978),
    [sym_float_literal] = ACTIONS(3978),
    [sym__outer_block_doc_comment_start] = ACTIONS(3978),
    [sym__outer_line_doc_comment_start] = ACTIONS(3978),
  },
  [STATE(1053)] = {
    [sym_line_comment] = STATE(1053),
    [sym_block_comment] = STATE(1053),
    [ts_builtin_sym_end] = ACTIONS(3982),
    [sym_identifier] = ACTIONS(3984),
    [anon_sym_SEMI] = ACTIONS(3982),
    [anon_sym_macro] = ACTIONS(3984),
    [anon_sym_LPAREN] = ACTIONS(3982),
    [anon_sym_LBRACE] = ACTIONS(3982),
    [anon_sym_RBRACE] = ACTIONS(3982),
    [anon_sym_macro_rules] = ACTIONS(3984),
    [anon_sym_BANG] = ACTIONS(3982),
    [anon_sym_LBRACK] = ACTIONS(3982),
    [anon_sym_STAR] = ACTIONS(3982),
    [anon_sym_u8] = ACTIONS(3984),
    [anon_sym_i8] = ACTIONS(3984),
    [anon_sym_u16] = ACTIONS(3984),
    [anon_sym_i16] = ACTIONS(3984),
    [anon_sym_u32] = ACTIONS(3984),
    [anon_sym_i32] = ACTIONS(3984),
    [anon_sym_u64] = ACTIONS(3984),
    [anon_sym_i64] = ACTIONS(3984),
    [anon_sym_u128] = ACTIONS(3984),
    [anon_sym_i128] = ACTIONS(3984),
    [anon_sym_isize] = ACTIONS(3984),
    [anon_sym_usize] = ACTIONS(3984),
    [anon_sym_f32] = ACTIONS(3984),
    [anon_sym_f64] = ACTIONS(3984),
    [anon_sym_bool] = ACTIONS(3984),
    [anon_sym_str] = ACTIONS(3984),
    [anon_sym_char] = ACTIONS(3984),
    [anon_sym_DASH] = ACTIONS(3982),
    [anon_sym_AMP] = ACTIONS(3982),
    [anon_sym_PIPE] = ACTIONS(3982),
    [anon_sym_LT] = ACTIONS(3982),
    [anon_sym_DOT_DOT] = ACTIONS(3982),
    [anon_sym_COLON_COLON] = ACTIONS(3982),
    [anon_sym_POUND] = ACTIONS(3982),
    [anon_sym_SQUOTE] = ACTIONS(3984),
    [anon_sym_async] = ACTIONS(3984),
    [anon_sym_auto] = ACTIONS(3984),
    [anon_sym_await] = ACTIONS(3984),
    [anon_sym_become] = ACTIONS(3984),
    [anon_sym_break] = ACTIONS(3984),
    [anon_sym_const] = ACTIONS(3984),
    [anon_sym_continue] = ACTIONS(3984),
    [anon_sym_default] = ACTIONS(3984),
    [anon_sym_enum] = ACTIONS(3984),
    [anon_sym_fn] = ACTIONS(3984),
    [anon_sym_for] = ACTIONS(3984),
    [anon_sym_gen] = ACTIONS(3984),
    [anon_sym_if] = ACTIONS(3984),
    [anon_sym_impl] = ACTIONS(3984),
    [anon_sym_let] = ACTIONS(3984),
    [anon_sym_loop] = ACTIONS(3984),
    [anon_sym_match] = ACTIONS(3984),
    [anon_sym_mod] = ACTIONS(3984),
    [anon_sym_pub] = ACTIONS(3984),
    [anon_sym_return] = ACTIONS(3984),
    [anon_sym_raw] = ACTIONS(3984),
    [anon_sym_static] = ACTIONS(3984),
    [anon_sym_struct] = ACTIONS(3984),
    [anon_sym_trait] = ACTIONS(3984),
    [anon_sym_type] = ACTIONS(3984),
    [anon_sym_union] = ACTIONS(3984),
    [anon_sym_unsafe] = ACTIONS(3984),
    [anon_sym_use] = ACTIONS(3984),
    [anon_sym_while] = ACTIONS(3984),
    [anon_sym_extern] = ACTIONS(3984),
    [anon_sym_safe] = ACTIONS(3984),
    [anon_sym_yield] = ACTIONS(3984),
    [anon_sym_move] = ACTIONS(3984),
    [anon_sym_try] = ACTIONS(3984),
    [sym_integer_literal] = ACTIONS(3982),
    [aux_sym_string_literal_token1] = ACTIONS(3982),
    [sym_char_literal] = ACTIONS(3982),
    [anon_sym_true] = ACTIONS(3984),
    [anon_sym_false] = ACTIONS(3984),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3984),
    [sym_super] = ACTIONS(3984),
    [sym_crate] = ACTIONS(3984),
    [aux_sym_metavariable_token1] = ACTIONS(3982),
    [sym__raw_string_literal_start] = ACTIONS(3982),
    [sym_float_literal] = ACTIONS(3982),
    [sym__outer_block_doc_comment_start] = ACTIONS(3982),
    [sym__outer_line_doc_comment_start] = ACTIONS(3982),
  },
  [STATE(1054)] = {
    [sym_line_comment] = STATE(1054),
    [sym_block_comment] = STATE(1054),
    [ts_builtin_sym_end] = ACTIONS(3986),
    [sym_identifier] = ACTIONS(3988),
    [anon_sym_SEMI] = ACTIONS(3986),
    [anon_sym_macro] = ACTIONS(3988),
    [anon_sym_LPAREN] = ACTIONS(3986),
    [anon_sym_LBRACE] = ACTIONS(3986),
    [anon_sym_RBRACE] = ACTIONS(3986),
    [anon_sym_macro_rules] = ACTIONS(3988),
    [anon_sym_BANG] = ACTIONS(3986),
    [anon_sym_LBRACK] = ACTIONS(3986),
    [anon_sym_STAR] = ACTIONS(3986),
    [anon_sym_u8] = ACTIONS(3988),
    [anon_sym_i8] = ACTIONS(3988),
    [anon_sym_u16] = ACTIONS(3988),
    [anon_sym_i16] = ACTIONS(3988),
    [anon_sym_u32] = ACTIONS(3988),
    [anon_sym_i32] = ACTIONS(3988),
    [anon_sym_u64] = ACTIONS(3988),
    [anon_sym_i64] = ACTIONS(3988),
    [anon_sym_u128] = ACTIONS(3988),
    [anon_sym_i128] = ACTIONS(3988),
    [anon_sym_isize] = ACTIONS(3988),
    [anon_sym_usize] = ACTIONS(3988),
    [anon_sym_f32] = ACTIONS(3988),
    [anon_sym_f64] = ACTIONS(3988),
    [anon_sym_bool] = ACTIONS(3988),
    [anon_sym_str] = ACTIONS(3988),
    [anon_sym_char] = ACTIONS(3988),
    [anon_sym_DASH] = ACTIONS(3986),
    [anon_sym_AMP] = ACTIONS(3986),
    [anon_sym_PIPE] = ACTIONS(3986),
    [anon_sym_LT] = ACTIONS(3986),
    [anon_sym_DOT_DOT] = ACTIONS(3986),
    [anon_sym_COLON_COLON] = ACTIONS(3986),
    [anon_sym_POUND] = ACTIONS(3986),
    [anon_sym_SQUOTE] = ACTIONS(3988),
    [anon_sym_async] = ACTIONS(3988),
    [anon_sym_auto] = ACTIONS(3988),
    [anon_sym_await] = ACTIONS(3988),
    [anon_sym_become] = ACTIONS(3988),
    [anon_sym_break] = ACTIONS(3988),
    [anon_sym_const] = ACTIONS(3988),
    [anon_sym_continue] = ACTIONS(3988),
    [anon_sym_default] = ACTIONS(3988),
    [anon_sym_enum] = ACTIONS(3988),
    [anon_sym_fn] = ACTIONS(3988),
    [anon_sym_for] = ACTIONS(3988),
    [anon_sym_gen] = ACTIONS(3988),
    [anon_sym_if] = ACTIONS(3988),
    [anon_sym_impl] = ACTIONS(3988),
    [anon_sym_let] = ACTIONS(3988),
    [anon_sym_loop] = ACTIONS(3988),
    [anon_sym_match] = ACTIONS(3988),
    [anon_sym_mod] = ACTIONS(3988),
    [anon_sym_pub] = ACTIONS(3988),
    [anon_sym_return] = ACTIONS(3988),
    [anon_sym_raw] = ACTIONS(3988),
    [anon_sym_static] = ACTIONS(3988),
    [anon_sym_struct] = ACTIONS(3988),
    [anon_sym_trait] = ACTIONS(3988),
    [anon_sym_type] = ACTIONS(3988),
    [anon_sym_union] = ACTIONS(3988),
    [anon_sym_unsafe] = ACTIONS(3988),
    [anon_sym_use] = ACTIONS(3988),
    [anon_sym_while] = ACTIONS(3988),
    [anon_sym_extern] = ACTIONS(3988),
    [anon_sym_safe] = ACTIONS(3988),
    [anon_sym_yield] = ACTIONS(3988),
    [anon_sym_move] = ACTIONS(3988),
    [anon_sym_try] = ACTIONS(3988),
    [sym_integer_literal] = ACTIONS(3986),
    [aux_sym_string_literal_token1] = ACTIONS(3986),
    [sym_char_literal] = ACTIONS(3986),
    [anon_sym_true] = ACTIONS(3988),
    [anon_sym_false] = ACTIONS(3988),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3988),
    [sym_super] = ACTIONS(3988),
    [sym_crate] = ACTIONS(3988),
    [aux_sym_metavariable_token1] = ACTIONS(3986),
    [sym__raw_string_literal_start] = ACTIONS(3986),
    [sym_float_literal] = ACTIONS(3986),
    [sym__outer_block_doc_comment_start] = ACTIONS(3986),
    [sym__outer_line_doc_comment_start] = ACTIONS(3986),
  },
  [STATE(1055)] = {
    [sym_line_comment] = STATE(1055),
    [sym_block_comment] = STATE(1055),
    [ts_builtin_sym_end] = ACTIONS(3990),
    [sym_identifier] = ACTIONS(3992),
    [anon_sym_SEMI] = ACTIONS(3990),
    [anon_sym_macro] = ACTIONS(3992),
    [anon_sym_LPAREN] = ACTIONS(3990),
    [anon_sym_LBRACE] = ACTIONS(3990),
    [anon_sym_RBRACE] = ACTIONS(3990),
    [anon_sym_macro_rules] = ACTIONS(3992),
    [anon_sym_BANG] = ACTIONS(3990),
    [anon_sym_LBRACK] = ACTIONS(3990),
    [anon_sym_STAR] = ACTIONS(3990),
    [anon_sym_u8] = ACTIONS(3992),
    [anon_sym_i8] = ACTIONS(3992),
    [anon_sym_u16] = ACTIONS(3992),
    [anon_sym_i16] = ACTIONS(3992),
    [anon_sym_u32] = ACTIONS(3992),
    [anon_sym_i32] = ACTIONS(3992),
    [anon_sym_u64] = ACTIONS(3992),
    [anon_sym_i64] = ACTIONS(3992),
    [anon_sym_u128] = ACTIONS(3992),
    [anon_sym_i128] = ACTIONS(3992),
    [anon_sym_isize] = ACTIONS(3992),
    [anon_sym_usize] = ACTIONS(3992),
    [anon_sym_f32] = ACTIONS(3992),
    [anon_sym_f64] = ACTIONS(3992),
    [anon_sym_bool] = ACTIONS(3992),
    [anon_sym_str] = ACTIONS(3992),
    [anon_sym_char] = ACTIONS(3992),
    [anon_sym_DASH] = ACTIONS(3990),
    [anon_sym_AMP] = ACTIONS(3990),
    [anon_sym_PIPE] = ACTIONS(3990),
    [anon_sym_LT] = ACTIONS(3990),
    [anon_sym_DOT_DOT] = ACTIONS(3990),
    [anon_sym_COLON_COLON] = ACTIONS(3990),
    [anon_sym_POUND] = ACTIONS(3990),
    [anon_sym_SQUOTE] = ACTIONS(3992),
    [anon_sym_async] = ACTIONS(3992),
    [anon_sym_auto] = ACTIONS(3992),
    [anon_sym_await] = ACTIONS(3992),
    [anon_sym_become] = ACTIONS(3992),
    [anon_sym_break] = ACTIONS(3992),
    [anon_sym_const] = ACTIONS(3992),
    [anon_sym_continue] = ACTIONS(3992),
    [anon_sym_default] = ACTIONS(3992),
    [anon_sym_enum] = ACTIONS(3992),
    [anon_sym_fn] = ACTIONS(3992),
    [anon_sym_for] = ACTIONS(3992),
    [anon_sym_gen] = ACTIONS(3992),
    [anon_sym_if] = ACTIONS(3992),
    [anon_sym_impl] = ACTIONS(3992),
    [anon_sym_let] = ACTIONS(3992),
    [anon_sym_loop] = ACTIONS(3992),
    [anon_sym_match] = ACTIONS(3992),
    [anon_sym_mod] = ACTIONS(3992),
    [anon_sym_pub] = ACTIONS(3992),
    [anon_sym_return] = ACTIONS(3992),
    [anon_sym_raw] = ACTIONS(3992),
    [anon_sym_static] = ACTIONS(3992),
    [anon_sym_struct] = ACTIONS(3992),
    [anon_sym_trait] = ACTIONS(3992),
    [anon_sym_type] = ACTIONS(3992),
    [anon_sym_union] = ACTIONS(3992),
    [anon_sym_unsafe] = ACTIONS(3992),
    [anon_sym_use] = ACTIONS(3992),
    [anon_sym_while] = ACTIONS(3992),
    [anon_sym_extern] = ACTIONS(3992),
    [anon_sym_safe] = ACTIONS(3992),
    [anon_sym_yield] = ACTIONS(3992),
    [anon_sym_move] = ACTIONS(3992),
    [anon_sym_try] = ACTIONS(3992),
    [sym_integer_literal] = ACTIONS(3990),
    [aux_sym_string_literal_token1] = ACTIONS(3990),
    [sym_char_literal] = ACTIONS(3990),
    [anon_sym_true] = ACTIONS(3992),
    [anon_sym_false] = ACTIONS(3992),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3992),
    [sym_super] = ACTIONS(3992),
    [sym_crate] = ACTIONS(3992),
    [aux_sym_metavariable_token1] = ACTIONS(3990),
    [sym__raw_string_literal_start] = ACTIONS(3990),
    [sym_float_literal] = ACTIONS(3990),
    [sym__outer_block_doc_comment_start] = ACTIONS(3990),
    [sym__outer_line_doc_comment_start] = ACTIONS(3990),
  },
  [STATE(1056)] = {
    [sym_line_comment] = STATE(1056),
    [sym_block_comment] = STATE(1056),
    [ts_builtin_sym_end] = ACTIONS(3994),
    [sym_identifier] = ACTIONS(3996),
    [anon_sym_SEMI] = ACTIONS(3994),
    [anon_sym_macro] = ACTIONS(3996),
    [anon_sym_LPAREN] = ACTIONS(3994),
    [anon_sym_LBRACE] = ACTIONS(3994),
    [anon_sym_RBRACE] = ACTIONS(3994),
    [anon_sym_macro_rules] = ACTIONS(3996),
    [anon_sym_BANG] = ACTIONS(3994),
    [anon_sym_LBRACK] = ACTIONS(3994),
    [anon_sym_STAR] = ACTIONS(3994),
    [anon_sym_u8] = ACTIONS(3996),
    [anon_sym_i8] = ACTIONS(3996),
    [anon_sym_u16] = ACTIONS(3996),
    [anon_sym_i16] = ACTIONS(3996),
    [anon_sym_u32] = ACTIONS(3996),
    [anon_sym_i32] = ACTIONS(3996),
    [anon_sym_u64] = ACTIONS(3996),
    [anon_sym_i64] = ACTIONS(3996),
    [anon_sym_u128] = ACTIONS(3996),
    [anon_sym_i128] = ACTIONS(3996),
    [anon_sym_isize] = ACTIONS(3996),
    [anon_sym_usize] = ACTIONS(3996),
    [anon_sym_f32] = ACTIONS(3996),
    [anon_sym_f64] = ACTIONS(3996),
    [anon_sym_bool] = ACTIONS(3996),
    [anon_sym_str] = ACTIONS(3996),
    [anon_sym_char] = ACTIONS(3996),
    [anon_sym_DASH] = ACTIONS(3994),
    [anon_sym_AMP] = ACTIONS(3994),
    [anon_sym_PIPE] = ACTIONS(3994),
    [anon_sym_LT] = ACTIONS(3994),
    [anon_sym_DOT_DOT] = ACTIONS(3994),
    [anon_sym_COLON_COLON] = ACTIONS(3994),
    [anon_sym_POUND] = ACTIONS(3994),
    [anon_sym_SQUOTE] = ACTIONS(3996),
    [anon_sym_async] = ACTIONS(3996),
    [anon_sym_auto] = ACTIONS(3996),
    [anon_sym_await] = ACTIONS(3996),
    [anon_sym_become] = ACTIONS(3996),
    [anon_sym_break] = ACTIONS(3996),
    [anon_sym_const] = ACTIONS(3996),
    [anon_sym_continue] = ACTIONS(3996),
    [anon_sym_default] = ACTIONS(3996),
    [anon_sym_enum] = ACTIONS(3996),
    [anon_sym_fn] = ACTIONS(3996),
    [anon_sym_for] = ACTIONS(3996),
    [anon_sym_gen] = ACTIONS(3996),
    [anon_sym_if] = ACTIONS(3996),
    [anon_sym_impl] = ACTIONS(3996),
    [anon_sym_let] = ACTIONS(3996),
    [anon_sym_loop] = ACTIONS(3996),
    [anon_sym_match] = ACTIONS(3996),
    [anon_sym_mod] = ACTIONS(3996),
    [anon_sym_pub] = ACTIONS(3996),
    [anon_sym_return] = ACTIONS(3996),
    [anon_sym_raw] = ACTIONS(3996),
    [anon_sym_static] = ACTIONS(3996),
    [anon_sym_struct] = ACTIONS(3996),
    [anon_sym_trait] = ACTIONS(3996),
    [anon_sym_type] = ACTIONS(3996),
    [anon_sym_union] = ACTIONS(3996),
    [anon_sym_unsafe] = ACTIONS(3996),
    [anon_sym_use] = ACTIONS(3996),
    [anon_sym_while] = ACTIONS(3996),
    [anon_sym_extern] = ACTIONS(3996),
    [anon_sym_safe] = ACTIONS(3996),
    [anon_sym_yield] = ACTIONS(3996),
    [anon_sym_move] = ACTIONS(3996),
    [anon_sym_try] = ACTIONS(3996),
    [sym_integer_literal] = ACTIONS(3994),
    [aux_sym_string_literal_token1] = ACTIONS(3994),
    [sym_char_literal] = ACTIONS(3994),
    [anon_sym_true] = ACTIONS(3996),
    [anon_sym_false] = ACTIONS(3996),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(3996),
    [sym_super] = ACTIONS(3996),
    [sym_crate] = ACTIONS(3996),
    [aux_sym_metavariable_token1] = ACTIONS(3994),
    [sym__raw_string_literal_start] = ACTIONS(3994),
    [sym_float_literal] = ACTIONS(3994),
    [sym__outer_block_doc_comment_start] = ACTIONS(3994),
    [sym__outer_line_doc_comment_start] = ACTIONS(3994),
  },
  [STATE(1057)] = {
    [sym_line_comment] = STATE(1057),
    [sym_block_comment] = STATE(1057),
    [ts_builtin_sym_end] = ACTIONS(3998),
    [sym_identifier] = ACTIONS(4000),
    [anon_sym_SEMI] = ACTIONS(3998),
    [anon_sym_macro] = ACTIONS(4000),
    [anon_sym_LPAREN] = ACTIONS(3998),
    [anon_sym_LBRACE] = ACTIONS(3998),
    [anon_sym_RBRACE] = ACTIONS(3998),
    [anon_sym_macro_rules] = ACTIONS(4000),
    [anon_sym_BANG] = ACTIONS(3998),
    [anon_sym_LBRACK] = ACTIONS(3998),
    [anon_sym_STAR] = ACTIONS(3998),
    [anon_sym_u8] = ACTIONS(4000),
    [anon_sym_i8] = ACTIONS(4000),
    [anon_sym_u16] = ACTIONS(4000),
    [anon_sym_i16] = ACTIONS(4000),
    [anon_sym_u32] = ACTIONS(4000),
    [anon_sym_i32] = ACTIONS(4000),
    [anon_sym_u64] = ACTIONS(4000),
    [anon_sym_i64] = ACTIONS(4000),
    [anon_sym_u128] = ACTIONS(4000),
    [anon_sym_i128] = ACTIONS(4000),
    [anon_sym_isize] = ACTIONS(4000),
    [anon_sym_usize] = ACTIONS(4000),
    [anon_sym_f32] = ACTIONS(4000),
    [anon_sym_f64] = ACTIONS(4000),
    [anon_sym_bool] = ACTIONS(4000),
    [anon_sym_str] = ACTIONS(4000),
    [anon_sym_char] = ACTIONS(4000),
    [anon_sym_DASH] = ACTIONS(3998),
    [anon_sym_AMP] = ACTIONS(3998),
    [anon_sym_PIPE] = ACTIONS(3998),
    [anon_sym_LT] = ACTIONS(3998),
    [anon_sym_DOT_DOT] = ACTIONS(3998),
    [anon_sym_COLON_COLON] = ACTIONS(3998),
    [anon_sym_POUND] = ACTIONS(3998),
    [anon_sym_SQUOTE] = ACTIONS(4000),
    [anon_sym_async] = ACTIONS(4000),
    [anon_sym_auto] = ACTIONS(4000),
    [anon_sym_await] = ACTIONS(4000),
    [anon_sym_become] = ACTIONS(4000),
    [anon_sym_break] = ACTIONS(4000),
    [anon_sym_const] = ACTIONS(4000),
    [anon_sym_continue] = ACTIONS(4000),
    [anon_sym_default] = ACTIONS(4000),
    [anon_sym_enum] = ACTIONS(4000),
    [anon_sym_fn] = ACTIONS(4000),
    [anon_sym_for] = ACTIONS(4000),
    [anon_sym_gen] = ACTIONS(4000),
    [anon_sym_if] = ACTIONS(4000),
    [anon_sym_impl] = ACTIONS(4000),
    [anon_sym_let] = ACTIONS(4000),
    [anon_sym_loop] = ACTIONS(4000),
    [anon_sym_match] = ACTIONS(4000),
    [anon_sym_mod] = ACTIONS(4000),
    [anon_sym_pub] = ACTIONS(4000),
    [anon_sym_return] = ACTIONS(4000),
    [anon_sym_raw] = ACTIONS(4000),
    [anon_sym_static] = ACTIONS(4000),
    [anon_sym_struct] = ACTIONS(4000),
    [anon_sym_trait] = ACTIONS(4000),
    [anon_sym_type] = ACTIONS(4000),
    [anon_sym_union] = ACTIONS(4000),
    [anon_sym_unsafe] = ACTIONS(4000),
    [anon_sym_use] = ACTIONS(4000),
    [anon_sym_while] = ACTIONS(4000),
    [anon_sym_extern] = ACTIONS(4000),
    [anon_sym_safe] = ACTIONS(4000),
    [anon_sym_yield] = ACTIONS(4000),
    [anon_sym_move] = ACTIONS(4000),
    [anon_sym_try] = ACTIONS(4000),
    [sym_integer_literal] = ACTIONS(3998),
    [aux_sym_string_literal_token1] = ACTIONS(3998),
    [sym_char_literal] = ACTIONS(3998),
    [anon_sym_true] = ACTIONS(4000),
    [anon_sym_false] = ACTIONS(4000),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4000),
    [sym_super] = ACTIONS(4000),
    [sym_crate] = ACTIONS(4000),
    [aux_sym_metavariable_token1] = ACTIONS(3998),
    [sym__raw_string_literal_start] = ACTIONS(3998),
    [sym_float_literal] = ACTIONS(3998),
    [sym__outer_block_doc_comment_start] = ACTIONS(3998),
    [sym__outer_line_doc_comment_start] = ACTIONS(3998),
  },
  [STATE(1058)] = {
    [sym_line_comment] = STATE(1058),
    [sym_block_comment] = STATE(1058),
    [ts_builtin_sym_end] = ACTIONS(4002),
    [sym_identifier] = ACTIONS(4004),
    [anon_sym_SEMI] = ACTIONS(4002),
    [anon_sym_macro] = ACTIONS(4004),
    [anon_sym_LPAREN] = ACTIONS(4002),
    [anon_sym_LBRACE] = ACTIONS(4002),
    [anon_sym_RBRACE] = ACTIONS(4002),
    [anon_sym_macro_rules] = ACTIONS(4004),
    [anon_sym_BANG] = ACTIONS(4002),
    [anon_sym_LBRACK] = ACTIONS(4002),
    [anon_sym_STAR] = ACTIONS(4002),
    [anon_sym_u8] = ACTIONS(4004),
    [anon_sym_i8] = ACTIONS(4004),
    [anon_sym_u16] = ACTIONS(4004),
    [anon_sym_i16] = ACTIONS(4004),
    [anon_sym_u32] = ACTIONS(4004),
    [anon_sym_i32] = ACTIONS(4004),
    [anon_sym_u64] = ACTIONS(4004),
    [anon_sym_i64] = ACTIONS(4004),
    [anon_sym_u128] = ACTIONS(4004),
    [anon_sym_i128] = ACTIONS(4004),
    [anon_sym_isize] = ACTIONS(4004),
    [anon_sym_usize] = ACTIONS(4004),
    [anon_sym_f32] = ACTIONS(4004),
    [anon_sym_f64] = ACTIONS(4004),
    [anon_sym_bool] = ACTIONS(4004),
    [anon_sym_str] = ACTIONS(4004),
    [anon_sym_char] = ACTIONS(4004),
    [anon_sym_DASH] = ACTIONS(4002),
    [anon_sym_AMP] = ACTIONS(4002),
    [anon_sym_PIPE] = ACTIONS(4002),
    [anon_sym_LT] = ACTIONS(4002),
    [anon_sym_DOT_DOT] = ACTIONS(4002),
    [anon_sym_COLON_COLON] = ACTIONS(4002),
    [anon_sym_POUND] = ACTIONS(4002),
    [anon_sym_SQUOTE] = ACTIONS(4004),
    [anon_sym_async] = ACTIONS(4004),
    [anon_sym_auto] = ACTIONS(4004),
    [anon_sym_await] = ACTIONS(4004),
    [anon_sym_become] = ACTIONS(4004),
    [anon_sym_break] = ACTIONS(4004),
    [anon_sym_const] = ACTIONS(4004),
    [anon_sym_continue] = ACTIONS(4004),
    [anon_sym_default] = ACTIONS(4004),
    [anon_sym_enum] = ACTIONS(4004),
    [anon_sym_fn] = ACTIONS(4004),
    [anon_sym_for] = ACTIONS(4004),
    [anon_sym_gen] = ACTIONS(4004),
    [anon_sym_if] = ACTIONS(4004),
    [anon_sym_impl] = ACTIONS(4004),
    [anon_sym_let] = ACTIONS(4004),
    [anon_sym_loop] = ACTIONS(4004),
    [anon_sym_match] = ACTIONS(4004),
    [anon_sym_mod] = ACTIONS(4004),
    [anon_sym_pub] = ACTIONS(4004),
    [anon_sym_return] = ACTIONS(4004),
    [anon_sym_raw] = ACTIONS(4004),
    [anon_sym_static] = ACTIONS(4004),
    [anon_sym_struct] = ACTIONS(4004),
    [anon_sym_trait] = ACTIONS(4004),
    [anon_sym_type] = ACTIONS(4004),
    [anon_sym_union] = ACTIONS(4004),
    [anon_sym_unsafe] = ACTIONS(4004),
    [anon_sym_use] = ACTIONS(4004),
    [anon_sym_while] = ACTIONS(4004),
    [anon_sym_extern] = ACTIONS(4004),
    [anon_sym_safe] = ACTIONS(4004),
    [anon_sym_yield] = ACTIONS(4004),
    [anon_sym_move] = ACTIONS(4004),
    [anon_sym_try] = ACTIONS(4004),
    [sym_integer_literal] = ACTIONS(4002),
    [aux_sym_string_literal_token1] = ACTIONS(4002),
    [sym_char_literal] = ACTIONS(4002),
    [anon_sym_true] = ACTIONS(4004),
    [anon_sym_false] = ACTIONS(4004),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4004),
    [sym_super] = ACTIONS(4004),
    [sym_crate] = ACTIONS(4004),
    [aux_sym_metavariable_token1] = ACTIONS(4002),
    [sym__raw_string_literal_start] = ACTIONS(4002),
    [sym_float_literal] = ACTIONS(4002),
    [sym__outer_block_doc_comment_start] = ACTIONS(4002),
    [sym__outer_line_doc_comment_start] = ACTIONS(4002),
  },
  [STATE(1059)] = {
    [sym_line_comment] = STATE(1059),
    [sym_block_comment] = STATE(1059),
    [ts_builtin_sym_end] = ACTIONS(4006),
    [sym_identifier] = ACTIONS(4008),
    [anon_sym_SEMI] = ACTIONS(4006),
    [anon_sym_macro] = ACTIONS(4008),
    [anon_sym_LPAREN] = ACTIONS(4006),
    [anon_sym_LBRACE] = ACTIONS(4006),
    [anon_sym_RBRACE] = ACTIONS(4006),
    [anon_sym_macro_rules] = ACTIONS(4008),
    [anon_sym_BANG] = ACTIONS(4006),
    [anon_sym_LBRACK] = ACTIONS(4006),
    [anon_sym_STAR] = ACTIONS(4006),
    [anon_sym_u8] = ACTIONS(4008),
    [anon_sym_i8] = ACTIONS(4008),
    [anon_sym_u16] = ACTIONS(4008),
    [anon_sym_i16] = ACTIONS(4008),
    [anon_sym_u32] = ACTIONS(4008),
    [anon_sym_i32] = ACTIONS(4008),
    [anon_sym_u64] = ACTIONS(4008),
    [anon_sym_i64] = ACTIONS(4008),
    [anon_sym_u128] = ACTIONS(4008),
    [anon_sym_i128] = ACTIONS(4008),
    [anon_sym_isize] = ACTIONS(4008),
    [anon_sym_usize] = ACTIONS(4008),
    [anon_sym_f32] = ACTIONS(4008),
    [anon_sym_f64] = ACTIONS(4008),
    [anon_sym_bool] = ACTIONS(4008),
    [anon_sym_str] = ACTIONS(4008),
    [anon_sym_char] = ACTIONS(4008),
    [anon_sym_DASH] = ACTIONS(4006),
    [anon_sym_AMP] = ACTIONS(4006),
    [anon_sym_PIPE] = ACTIONS(4006),
    [anon_sym_LT] = ACTIONS(4006),
    [anon_sym_DOT_DOT] = ACTIONS(4006),
    [anon_sym_COLON_COLON] = ACTIONS(4006),
    [anon_sym_POUND] = ACTIONS(4006),
    [anon_sym_SQUOTE] = ACTIONS(4008),
    [anon_sym_async] = ACTIONS(4008),
    [anon_sym_auto] = ACTIONS(4008),
    [anon_sym_await] = ACTIONS(4008),
    [anon_sym_become] = ACTIONS(4008),
    [anon_sym_break] = ACTIONS(4008),
    [anon_sym_const] = ACTIONS(4008),
    [anon_sym_continue] = ACTIONS(4008),
    [anon_sym_default] = ACTIONS(4008),
    [anon_sym_enum] = ACTIONS(4008),
    [anon_sym_fn] = ACTIONS(4008),
    [anon_sym_for] = ACTIONS(4008),
    [anon_sym_gen] = ACTIONS(4008),
    [anon_sym_if] = ACTIONS(4008),
    [anon_sym_impl] = ACTIONS(4008),
    [anon_sym_let] = ACTIONS(4008),
    [anon_sym_loop] = ACTIONS(4008),
    [anon_sym_match] = ACTIONS(4008),
    [anon_sym_mod] = ACTIONS(4008),
    [anon_sym_pub] = ACTIONS(4008),
    [anon_sym_return] = ACTIONS(4008),
    [anon_sym_raw] = ACTIONS(4008),
    [anon_sym_static] = ACTIONS(4008),
    [anon_sym_struct] = ACTIONS(4008),
    [anon_sym_trait] = ACTIONS(4008),
    [anon_sym_type] = ACTIONS(4008),
    [anon_sym_union] = ACTIONS(4008),
    [anon_sym_unsafe] = ACTIONS(4008),
    [anon_sym_use] = ACTIONS(4008),
    [anon_sym_while] = ACTIONS(4008),
    [anon_sym_extern] = ACTIONS(4008),
    [anon_sym_safe] = ACTIONS(4008),
    [anon_sym_yield] = ACTIONS(4008),
    [anon_sym_move] = ACTIONS(4008),
    [anon_sym_try] = ACTIONS(4008),
    [sym_integer_literal] = ACTIONS(4006),
    [aux_sym_string_literal_token1] = ACTIONS(4006),
    [sym_char_literal] = ACTIONS(4006),
    [anon_sym_true] = ACTIONS(4008),
    [anon_sym_false] = ACTIONS(4008),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4008),
    [sym_super] = ACTIONS(4008),
    [sym_crate] = ACTIONS(4008),
    [aux_sym_metavariable_token1] = ACTIONS(4006),
    [sym__raw_string_literal_start] = ACTIONS(4006),
    [sym_float_literal] = ACTIONS(4006),
    [sym__outer_block_doc_comment_start] = ACTIONS(4006),
    [sym__outer_line_doc_comment_start] = ACTIONS(4006),
  },
  [STATE(1060)] = {
    [sym_line_comment] = STATE(1060),
    [sym_block_comment] = STATE(1060),
    [ts_builtin_sym_end] = ACTIONS(4010),
    [sym_identifier] = ACTIONS(4012),
    [anon_sym_SEMI] = ACTIONS(4010),
    [anon_sym_macro] = ACTIONS(4012),
    [anon_sym_LPAREN] = ACTIONS(4010),
    [anon_sym_LBRACE] = ACTIONS(4010),
    [anon_sym_RBRACE] = ACTIONS(4010),
    [anon_sym_macro_rules] = ACTIONS(4012),
    [anon_sym_BANG] = ACTIONS(4010),
    [anon_sym_LBRACK] = ACTIONS(4010),
    [anon_sym_STAR] = ACTIONS(4010),
    [anon_sym_u8] = ACTIONS(4012),
    [anon_sym_i8] = ACTIONS(4012),
    [anon_sym_u16] = ACTIONS(4012),
    [anon_sym_i16] = ACTIONS(4012),
    [anon_sym_u32] = ACTIONS(4012),
    [anon_sym_i32] = ACTIONS(4012),
    [anon_sym_u64] = ACTIONS(4012),
    [anon_sym_i64] = ACTIONS(4012),
    [anon_sym_u128] = ACTIONS(4012),
    [anon_sym_i128] = ACTIONS(4012),
    [anon_sym_isize] = ACTIONS(4012),
    [anon_sym_usize] = ACTIONS(4012),
    [anon_sym_f32] = ACTIONS(4012),
    [anon_sym_f64] = ACTIONS(4012),
    [anon_sym_bool] = ACTIONS(4012),
    [anon_sym_str] = ACTIONS(4012),
    [anon_sym_char] = ACTIONS(4012),
    [anon_sym_DASH] = ACTIONS(4010),
    [anon_sym_AMP] = ACTIONS(4010),
    [anon_sym_PIPE] = ACTIONS(4010),
    [anon_sym_LT] = ACTIONS(4010),
    [anon_sym_DOT_DOT] = ACTIONS(4010),
    [anon_sym_COLON_COLON] = ACTIONS(4010),
    [anon_sym_POUND] = ACTIONS(4010),
    [anon_sym_SQUOTE] = ACTIONS(4012),
    [anon_sym_async] = ACTIONS(4012),
    [anon_sym_auto] = ACTIONS(4012),
    [anon_sym_await] = ACTIONS(4012),
    [anon_sym_become] = ACTIONS(4012),
    [anon_sym_break] = ACTIONS(4012),
    [anon_sym_const] = ACTIONS(4012),
    [anon_sym_continue] = ACTIONS(4012),
    [anon_sym_default] = ACTIONS(4012),
    [anon_sym_enum] = ACTIONS(4012),
    [anon_sym_fn] = ACTIONS(4012),
    [anon_sym_for] = ACTIONS(4012),
    [anon_sym_gen] = ACTIONS(4012),
    [anon_sym_if] = ACTIONS(4012),
    [anon_sym_impl] = ACTIONS(4012),
    [anon_sym_let] = ACTIONS(4012),
    [anon_sym_loop] = ACTIONS(4012),
    [anon_sym_match] = ACTIONS(4012),
    [anon_sym_mod] = ACTIONS(4012),
    [anon_sym_pub] = ACTIONS(4012),
    [anon_sym_return] = ACTIONS(4012),
    [anon_sym_raw] = ACTIONS(4012),
    [anon_sym_static] = ACTIONS(4012),
    [anon_sym_struct] = ACTIONS(4012),
    [anon_sym_trait] = ACTIONS(4012),
    [anon_sym_type] = ACTIONS(4012),
    [anon_sym_union] = ACTIONS(4012),
    [anon_sym_unsafe] = ACTIONS(4012),
    [anon_sym_use] = ACTIONS(4012),
    [anon_sym_while] = ACTIONS(4012),
    [anon_sym_extern] = ACTIONS(4012),
    [anon_sym_safe] = ACTIONS(4012),
    [anon_sym_yield] = ACTIONS(4012),
    [anon_sym_move] = ACTIONS(4012),
    [anon_sym_try] = ACTIONS(4012),
    [sym_integer_literal] = ACTIONS(4010),
    [aux_sym_string_literal_token1] = ACTIONS(4010),
    [sym_char_literal] = ACTIONS(4010),
    [anon_sym_true] = ACTIONS(4012),
    [anon_sym_false] = ACTIONS(4012),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4012),
    [sym_super] = ACTIONS(4012),
    [sym_crate] = ACTIONS(4012),
    [aux_sym_metavariable_token1] = ACTIONS(4010),
    [sym__raw_string_literal_start] = ACTIONS(4010),
    [sym_float_literal] = ACTIONS(4010),
    [sym__outer_block_doc_comment_start] = ACTIONS(4010),
    [sym__outer_line_doc_comment_start] = ACTIONS(4010),
  },
  [STATE(1061)] = {
    [sym_line_comment] = STATE(1061),
    [sym_block_comment] = STATE(1061),
    [ts_builtin_sym_end] = ACTIONS(4014),
    [sym_identifier] = ACTIONS(4016),
    [anon_sym_SEMI] = ACTIONS(4014),
    [anon_sym_macro] = ACTIONS(4016),
    [anon_sym_LPAREN] = ACTIONS(4014),
    [anon_sym_LBRACE] = ACTIONS(4014),
    [anon_sym_RBRACE] = ACTIONS(4014),
    [anon_sym_macro_rules] = ACTIONS(4016),
    [anon_sym_BANG] = ACTIONS(4014),
    [anon_sym_LBRACK] = ACTIONS(4014),
    [anon_sym_STAR] = ACTIONS(4014),
    [anon_sym_u8] = ACTIONS(4016),
    [anon_sym_i8] = ACTIONS(4016),
    [anon_sym_u16] = ACTIONS(4016),
    [anon_sym_i16] = ACTIONS(4016),
    [anon_sym_u32] = ACTIONS(4016),
    [anon_sym_i32] = ACTIONS(4016),
    [anon_sym_u64] = ACTIONS(4016),
    [anon_sym_i64] = ACTIONS(4016),
    [anon_sym_u128] = ACTIONS(4016),
    [anon_sym_i128] = ACTIONS(4016),
    [anon_sym_isize] = ACTIONS(4016),
    [anon_sym_usize] = ACTIONS(4016),
    [anon_sym_f32] = ACTIONS(4016),
    [anon_sym_f64] = ACTIONS(4016),
    [anon_sym_bool] = ACTIONS(4016),
    [anon_sym_str] = ACTIONS(4016),
    [anon_sym_char] = ACTIONS(4016),
    [anon_sym_DASH] = ACTIONS(4014),
    [anon_sym_AMP] = ACTIONS(4014),
    [anon_sym_PIPE] = ACTIONS(4014),
    [anon_sym_LT] = ACTIONS(4014),
    [anon_sym_DOT_DOT] = ACTIONS(4014),
    [anon_sym_COLON_COLON] = ACTIONS(4014),
    [anon_sym_POUND] = ACTIONS(4014),
    [anon_sym_SQUOTE] = ACTIONS(4016),
    [anon_sym_async] = ACTIONS(4016),
    [anon_sym_auto] = ACTIONS(4016),
    [anon_sym_await] = ACTIONS(4016),
    [anon_sym_become] = ACTIONS(4016),
    [anon_sym_break] = ACTIONS(4016),
    [anon_sym_const] = ACTIONS(4016),
    [anon_sym_continue] = ACTIONS(4016),
    [anon_sym_default] = ACTIONS(4016),
    [anon_sym_enum] = ACTIONS(4016),
    [anon_sym_fn] = ACTIONS(4016),
    [anon_sym_for] = ACTIONS(4016),
    [anon_sym_gen] = ACTIONS(4016),
    [anon_sym_if] = ACTIONS(4016),
    [anon_sym_impl] = ACTIONS(4016),
    [anon_sym_let] = ACTIONS(4016),
    [anon_sym_loop] = ACTIONS(4016),
    [anon_sym_match] = ACTIONS(4016),
    [anon_sym_mod] = ACTIONS(4016),
    [anon_sym_pub] = ACTIONS(4016),
    [anon_sym_return] = ACTIONS(4016),
    [anon_sym_raw] = ACTIONS(4016),
    [anon_sym_static] = ACTIONS(4016),
    [anon_sym_struct] = ACTIONS(4016),
    [anon_sym_trait] = ACTIONS(4016),
    [anon_sym_type] = ACTIONS(4016),
    [anon_sym_union] = ACTIONS(4016),
    [anon_sym_unsafe] = ACTIONS(4016),
    [anon_sym_use] = ACTIONS(4016),
    [anon_sym_while] = ACTIONS(4016),
    [anon_sym_extern] = ACTIONS(4016),
    [anon_sym_safe] = ACTIONS(4016),
    [anon_sym_yield] = ACTIONS(4016),
    [anon_sym_move] = ACTIONS(4016),
    [anon_sym_try] = ACTIONS(4016),
    [sym_integer_literal] = ACTIONS(4014),
    [aux_sym_string_literal_token1] = ACTIONS(4014),
    [sym_char_literal] = ACTIONS(4014),
    [anon_sym_true] = ACTIONS(4016),
    [anon_sym_false] = ACTIONS(4016),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4016),
    [sym_super] = ACTIONS(4016),
    [sym_crate] = ACTIONS(4016),
    [aux_sym_metavariable_token1] = ACTIONS(4014),
    [sym__raw_string_literal_start] = ACTIONS(4014),
    [sym_float_literal] = ACTIONS(4014),
    [sym__outer_block_doc_comment_start] = ACTIONS(4014),
    [sym__outer_line_doc_comment_start] = ACTIONS(4014),
  },
  [STATE(1062)] = {
    [sym_line_comment] = STATE(1062),
    [sym_block_comment] = STATE(1062),
    [ts_builtin_sym_end] = ACTIONS(4018),
    [sym_identifier] = ACTIONS(4020),
    [anon_sym_SEMI] = ACTIONS(4018),
    [anon_sym_macro] = ACTIONS(4020),
    [anon_sym_LPAREN] = ACTIONS(4018),
    [anon_sym_LBRACE] = ACTIONS(4018),
    [anon_sym_RBRACE] = ACTIONS(4018),
    [anon_sym_macro_rules] = ACTIONS(4020),
    [anon_sym_BANG] = ACTIONS(4018),
    [anon_sym_LBRACK] = ACTIONS(4018),
    [anon_sym_STAR] = ACTIONS(4018),
    [anon_sym_u8] = ACTIONS(4020),
    [anon_sym_i8] = ACTIONS(4020),
    [anon_sym_u16] = ACTIONS(4020),
    [anon_sym_i16] = ACTIONS(4020),
    [anon_sym_u32] = ACTIONS(4020),
    [anon_sym_i32] = ACTIONS(4020),
    [anon_sym_u64] = ACTIONS(4020),
    [anon_sym_i64] = ACTIONS(4020),
    [anon_sym_u128] = ACTIONS(4020),
    [anon_sym_i128] = ACTIONS(4020),
    [anon_sym_isize] = ACTIONS(4020),
    [anon_sym_usize] = ACTIONS(4020),
    [anon_sym_f32] = ACTIONS(4020),
    [anon_sym_f64] = ACTIONS(4020),
    [anon_sym_bool] = ACTIONS(4020),
    [anon_sym_str] = ACTIONS(4020),
    [anon_sym_char] = ACTIONS(4020),
    [anon_sym_DASH] = ACTIONS(4018),
    [anon_sym_AMP] = ACTIONS(4018),
    [anon_sym_PIPE] = ACTIONS(4018),
    [anon_sym_LT] = ACTIONS(4018),
    [anon_sym_DOT_DOT] = ACTIONS(4018),
    [anon_sym_COLON_COLON] = ACTIONS(4018),
    [anon_sym_POUND] = ACTIONS(4018),
    [anon_sym_SQUOTE] = ACTIONS(4020),
    [anon_sym_async] = ACTIONS(4020),
    [anon_sym_auto] = ACTIONS(4020),
    [anon_sym_await] = ACTIONS(4020),
    [anon_sym_become] = ACTIONS(4020),
    [anon_sym_break] = ACTIONS(4020),
    [anon_sym_const] = ACTIONS(4020),
    [anon_sym_continue] = ACTIONS(4020),
    [anon_sym_default] = ACTIONS(4020),
    [anon_sym_enum] = ACTIONS(4020),
    [anon_sym_fn] = ACTIONS(4020),
    [anon_sym_for] = ACTIONS(4020),
    [anon_sym_gen] = ACTIONS(4020),
    [anon_sym_if] = ACTIONS(4020),
    [anon_sym_impl] = ACTIONS(4020),
    [anon_sym_let] = ACTIONS(4020),
    [anon_sym_loop] = ACTIONS(4020),
    [anon_sym_match] = ACTIONS(4020),
    [anon_sym_mod] = ACTIONS(4020),
    [anon_sym_pub] = ACTIONS(4020),
    [anon_sym_return] = ACTIONS(4020),
    [anon_sym_raw] = ACTIONS(4020),
    [anon_sym_static] = ACTIONS(4020),
    [anon_sym_struct] = ACTIONS(4020),
    [anon_sym_trait] = ACTIONS(4020),
    [anon_sym_type] = ACTIONS(4020),
    [anon_sym_union] = ACTIONS(4020),
    [anon_sym_unsafe] = ACTIONS(4020),
    [anon_sym_use] = ACTIONS(4020),
    [anon_sym_while] = ACTIONS(4020),
    [anon_sym_extern] = ACTIONS(4020),
    [anon_sym_safe] = ACTIONS(4020),
    [anon_sym_yield] = ACTIONS(4020),
    [anon_sym_move] = ACTIONS(4020),
    [anon_sym_try] = ACTIONS(4020),
    [sym_integer_literal] = ACTIONS(4018),
    [aux_sym_string_literal_token1] = ACTIONS(4018),
    [sym_char_literal] = ACTIONS(4018),
    [anon_sym_true] = ACTIONS(4020),
    [anon_sym_false] = ACTIONS(4020),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4020),
    [sym_super] = ACTIONS(4020),
    [sym_crate] = ACTIONS(4020),
    [aux_sym_metavariable_token1] = ACTIONS(4018),
    [sym__raw_string_literal_start] = ACTIONS(4018),
    [sym_float_literal] = ACTIONS(4018),
    [sym__outer_block_doc_comment_start] = ACTIONS(4018),
    [sym__outer_line_doc_comment_start] = ACTIONS(4018),
  },
  [STATE(1063)] = {
    [sym_line_comment] = STATE(1063),
    [sym_block_comment] = STATE(1063),
    [ts_builtin_sym_end] = ACTIONS(4022),
    [sym_identifier] = ACTIONS(4024),
    [anon_sym_SEMI] = ACTIONS(4022),
    [anon_sym_macro] = ACTIONS(4024),
    [anon_sym_LPAREN] = ACTIONS(4022),
    [anon_sym_LBRACE] = ACTIONS(4022),
    [anon_sym_RBRACE] = ACTIONS(4022),
    [anon_sym_macro_rules] = ACTIONS(4024),
    [anon_sym_BANG] = ACTIONS(4022),
    [anon_sym_LBRACK] = ACTIONS(4022),
    [anon_sym_STAR] = ACTIONS(4022),
    [anon_sym_u8] = ACTIONS(4024),
    [anon_sym_i8] = ACTIONS(4024),
    [anon_sym_u16] = ACTIONS(4024),
    [anon_sym_i16] = ACTIONS(4024),
    [anon_sym_u32] = ACTIONS(4024),
    [anon_sym_i32] = ACTIONS(4024),
    [anon_sym_u64] = ACTIONS(4024),
    [anon_sym_i64] = ACTIONS(4024),
    [anon_sym_u128] = ACTIONS(4024),
    [anon_sym_i128] = ACTIONS(4024),
    [anon_sym_isize] = ACTIONS(4024),
    [anon_sym_usize] = ACTIONS(4024),
    [anon_sym_f32] = ACTIONS(4024),
    [anon_sym_f64] = ACTIONS(4024),
    [anon_sym_bool] = ACTIONS(4024),
    [anon_sym_str] = ACTIONS(4024),
    [anon_sym_char] = ACTIONS(4024),
    [anon_sym_DASH] = ACTIONS(4022),
    [anon_sym_AMP] = ACTIONS(4022),
    [anon_sym_PIPE] = ACTIONS(4022),
    [anon_sym_LT] = ACTIONS(4022),
    [anon_sym_DOT_DOT] = ACTIONS(4022),
    [anon_sym_COLON_COLON] = ACTIONS(4022),
    [anon_sym_POUND] = ACTIONS(4022),
    [anon_sym_SQUOTE] = ACTIONS(4024),
    [anon_sym_async] = ACTIONS(4024),
    [anon_sym_auto] = ACTIONS(4024),
    [anon_sym_await] = ACTIONS(4024),
    [anon_sym_become] = ACTIONS(4024),
    [anon_sym_break] = ACTIONS(4024),
    [anon_sym_const] = ACTIONS(4024),
    [anon_sym_continue] = ACTIONS(4024),
    [anon_sym_default] = ACTIONS(4024),
    [anon_sym_enum] = ACTIONS(4024),
    [anon_sym_fn] = ACTIONS(4024),
    [anon_sym_for] = ACTIONS(4024),
    [anon_sym_gen] = ACTIONS(4024),
    [anon_sym_if] = ACTIONS(4024),
    [anon_sym_impl] = ACTIONS(4024),
    [anon_sym_let] = ACTIONS(4024),
    [anon_sym_loop] = ACTIONS(4024),
    [anon_sym_match] = ACTIONS(4024),
    [anon_sym_mod] = ACTIONS(4024),
    [anon_sym_pub] = ACTIONS(4024),
    [anon_sym_return] = ACTIONS(4024),
    [anon_sym_raw] = ACTIONS(4024),
    [anon_sym_static] = ACTIONS(4024),
    [anon_sym_struct] = ACTIONS(4024),
    [anon_sym_trait] = ACTIONS(4024),
    [anon_sym_type] = ACTIONS(4024),
    [anon_sym_union] = ACTIONS(4024),
    [anon_sym_unsafe] = ACTIONS(4024),
    [anon_sym_use] = ACTIONS(4024),
    [anon_sym_while] = ACTIONS(4024),
    [anon_sym_extern] = ACTIONS(4024),
    [anon_sym_safe] = ACTIONS(4024),
    [anon_sym_yield] = ACTIONS(4024),
    [anon_sym_move] = ACTIONS(4024),
    [anon_sym_try] = ACTIONS(4024),
    [sym_integer_literal] = ACTIONS(4022),
    [aux_sym_string_literal_token1] = ACTIONS(4022),
    [sym_char_literal] = ACTIONS(4022),
    [anon_sym_true] = ACTIONS(4024),
    [anon_sym_false] = ACTIONS(4024),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4024),
    [sym_super] = ACTIONS(4024),
    [sym_crate] = ACTIONS(4024),
    [aux_sym_metavariable_token1] = ACTIONS(4022),
    [sym__raw_string_literal_start] = ACTIONS(4022),
    [sym_float_literal] = ACTIONS(4022),
    [sym__outer_block_doc_comment_start] = ACTIONS(4022),
    [sym__outer_line_doc_comment_start] = ACTIONS(4022),
  },
  [STATE(1064)] = {
    [sym_line_comment] = STATE(1064),
    [sym_block_comment] = STATE(1064),
    [ts_builtin_sym_end] = ACTIONS(4026),
    [sym_identifier] = ACTIONS(4028),
    [anon_sym_SEMI] = ACTIONS(4026),
    [anon_sym_macro] = ACTIONS(4028),
    [anon_sym_LPAREN] = ACTIONS(4026),
    [anon_sym_LBRACE] = ACTIONS(4026),
    [anon_sym_RBRACE] = ACTIONS(4026),
    [anon_sym_macro_rules] = ACTIONS(4028),
    [anon_sym_BANG] = ACTIONS(4026),
    [anon_sym_LBRACK] = ACTIONS(4026),
    [anon_sym_STAR] = ACTIONS(4026),
    [anon_sym_u8] = ACTIONS(4028),
    [anon_sym_i8] = ACTIONS(4028),
    [anon_sym_u16] = ACTIONS(4028),
    [anon_sym_i16] = ACTIONS(4028),
    [anon_sym_u32] = ACTIONS(4028),
    [anon_sym_i32] = ACTIONS(4028),
    [anon_sym_u64] = ACTIONS(4028),
    [anon_sym_i64] = ACTIONS(4028),
    [anon_sym_u128] = ACTIONS(4028),
    [anon_sym_i128] = ACTIONS(4028),
    [anon_sym_isize] = ACTIONS(4028),
    [anon_sym_usize] = ACTIONS(4028),
    [anon_sym_f32] = ACTIONS(4028),
    [anon_sym_f64] = ACTIONS(4028),
    [anon_sym_bool] = ACTIONS(4028),
    [anon_sym_str] = ACTIONS(4028),
    [anon_sym_char] = ACTIONS(4028),
    [anon_sym_DASH] = ACTIONS(4026),
    [anon_sym_AMP] = ACTIONS(4026),
    [anon_sym_PIPE] = ACTIONS(4026),
    [anon_sym_LT] = ACTIONS(4026),
    [anon_sym_DOT_DOT] = ACTIONS(4026),
    [anon_sym_COLON_COLON] = ACTIONS(4026),
    [anon_sym_POUND] = ACTIONS(4026),
    [anon_sym_SQUOTE] = ACTIONS(4028),
    [anon_sym_async] = ACTIONS(4028),
    [anon_sym_auto] = ACTIONS(4028),
    [anon_sym_await] = ACTIONS(4028),
    [anon_sym_become] = ACTIONS(4028),
    [anon_sym_break] = ACTIONS(4028),
    [anon_sym_const] = ACTIONS(4028),
    [anon_sym_continue] = ACTIONS(4028),
    [anon_sym_default] = ACTIONS(4028),
    [anon_sym_enum] = ACTIONS(4028),
    [anon_sym_fn] = ACTIONS(4028),
    [anon_sym_for] = ACTIONS(4028),
    [anon_sym_gen] = ACTIONS(4028),
    [anon_sym_if] = ACTIONS(4028),
    [anon_sym_impl] = ACTIONS(4028),
    [anon_sym_let] = ACTIONS(4028),
    [anon_sym_loop] = ACTIONS(4028),
    [anon_sym_match] = ACTIONS(4028),
    [anon_sym_mod] = ACTIONS(4028),
    [anon_sym_pub] = ACTIONS(4028),
    [anon_sym_return] = ACTIONS(4028),
    [anon_sym_raw] = ACTIONS(4028),
    [anon_sym_static] = ACTIONS(4028),
    [anon_sym_struct] = ACTIONS(4028),
    [anon_sym_trait] = ACTIONS(4028),
    [anon_sym_type] = ACTIONS(4028),
    [anon_sym_union] = ACTIONS(4028),
    [anon_sym_unsafe] = ACTIONS(4028),
    [anon_sym_use] = ACTIONS(4028),
    [anon_sym_while] = ACTIONS(4028),
    [anon_sym_extern] = ACTIONS(4028),
    [anon_sym_safe] = ACTIONS(4028),
    [anon_sym_yield] = ACTIONS(4028),
    [anon_sym_move] = ACTIONS(4028),
    [anon_sym_try] = ACTIONS(4028),
    [sym_integer_literal] = ACTIONS(4026),
    [aux_sym_string_literal_token1] = ACTIONS(4026),
    [sym_char_literal] = ACTIONS(4026),
    [anon_sym_true] = ACTIONS(4028),
    [anon_sym_false] = ACTIONS(4028),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4028),
    [sym_super] = ACTIONS(4028),
    [sym_crate] = ACTIONS(4028),
    [aux_sym_metavariable_token1] = ACTIONS(4026),
    [sym__raw_string_literal_start] = ACTIONS(4026),
    [sym_float_literal] = ACTIONS(4026),
    [sym__outer_block_doc_comment_start] = ACTIONS(4026),
    [sym__outer_line_doc_comment_start] = ACTIONS(4026),
  },
  [STATE(1065)] = {
    [sym_line_comment] = STATE(1065),
    [sym_block_comment] = STATE(1065),
    [ts_builtin_sym_end] = ACTIONS(4030),
    [sym_identifier] = ACTIONS(4032),
    [anon_sym_SEMI] = ACTIONS(4030),
    [anon_sym_macro] = ACTIONS(4032),
    [anon_sym_LPAREN] = ACTIONS(4030),
    [anon_sym_LBRACE] = ACTIONS(4030),
    [anon_sym_RBRACE] = ACTIONS(4030),
    [anon_sym_macro_rules] = ACTIONS(4032),
    [anon_sym_BANG] = ACTIONS(4030),
    [anon_sym_LBRACK] = ACTIONS(4030),
    [anon_sym_STAR] = ACTIONS(4030),
    [anon_sym_u8] = ACTIONS(4032),
    [anon_sym_i8] = ACTIONS(4032),
    [anon_sym_u16] = ACTIONS(4032),
    [anon_sym_i16] = ACTIONS(4032),
    [anon_sym_u32] = ACTIONS(4032),
    [anon_sym_i32] = ACTIONS(4032),
    [anon_sym_u64] = ACTIONS(4032),
    [anon_sym_i64] = ACTIONS(4032),
    [anon_sym_u128] = ACTIONS(4032),
    [anon_sym_i128] = ACTIONS(4032),
    [anon_sym_isize] = ACTIONS(4032),
    [anon_sym_usize] = ACTIONS(4032),
    [anon_sym_f32] = ACTIONS(4032),
    [anon_sym_f64] = ACTIONS(4032),
    [anon_sym_bool] = ACTIONS(4032),
    [anon_sym_str] = ACTIONS(4032),
    [anon_sym_char] = ACTIONS(4032),
    [anon_sym_DASH] = ACTIONS(4030),
    [anon_sym_AMP] = ACTIONS(4030),
    [anon_sym_PIPE] = ACTIONS(4030),
    [anon_sym_LT] = ACTIONS(4030),
    [anon_sym_DOT_DOT] = ACTIONS(4030),
    [anon_sym_COLON_COLON] = ACTIONS(4030),
    [anon_sym_POUND] = ACTIONS(4030),
    [anon_sym_SQUOTE] = ACTIONS(4032),
    [anon_sym_async] = ACTIONS(4032),
    [anon_sym_auto] = ACTIONS(4032),
    [anon_sym_await] = ACTIONS(4032),
    [anon_sym_become] = ACTIONS(4032),
    [anon_sym_break] = ACTIONS(4032),
    [anon_sym_const] = ACTIONS(4032),
    [anon_sym_continue] = ACTIONS(4032),
    [anon_sym_default] = ACTIONS(4032),
    [anon_sym_enum] = ACTIONS(4032),
    [anon_sym_fn] = ACTIONS(4032),
    [anon_sym_for] = ACTIONS(4032),
    [anon_sym_gen] = ACTIONS(4032),
    [anon_sym_if] = ACTIONS(4032),
    [anon_sym_impl] = ACTIONS(4032),
    [anon_sym_let] = ACTIONS(4032),
    [anon_sym_loop] = ACTIONS(4032),
    [anon_sym_match] = ACTIONS(4032),
    [anon_sym_mod] = ACTIONS(4032),
    [anon_sym_pub] = ACTIONS(4032),
    [anon_sym_return] = ACTIONS(4032),
    [anon_sym_raw] = ACTIONS(4032),
    [anon_sym_static] = ACTIONS(4032),
    [anon_sym_struct] = ACTIONS(4032),
    [anon_sym_trait] = ACTIONS(4032),
    [anon_sym_type] = ACTIONS(4032),
    [anon_sym_union] = ACTIONS(4032),
    [anon_sym_unsafe] = ACTIONS(4032),
    [anon_sym_use] = ACTIONS(4032),
    [anon_sym_while] = ACTIONS(4032),
    [anon_sym_extern] = ACTIONS(4032),
    [anon_sym_safe] = ACTIONS(4032),
    [anon_sym_yield] = ACTIONS(4032),
    [anon_sym_move] = ACTIONS(4032),
    [anon_sym_try] = ACTIONS(4032),
    [sym_integer_literal] = ACTIONS(4030),
    [aux_sym_string_literal_token1] = ACTIONS(4030),
    [sym_char_literal] = ACTIONS(4030),
    [anon_sym_true] = ACTIONS(4032),
    [anon_sym_false] = ACTIONS(4032),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4032),
    [sym_super] = ACTIONS(4032),
    [sym_crate] = ACTIONS(4032),
    [aux_sym_metavariable_token1] = ACTIONS(4030),
    [sym__raw_string_literal_start] = ACTIONS(4030),
    [sym_float_literal] = ACTIONS(4030),
    [sym__outer_block_doc_comment_start] = ACTIONS(4030),
    [sym__outer_line_doc_comment_start] = ACTIONS(4030),
  },
  [STATE(1066)] = {
    [sym_line_comment] = STATE(1066),
    [sym_block_comment] = STATE(1066),
    [ts_builtin_sym_end] = ACTIONS(4034),
    [sym_identifier] = ACTIONS(4036),
    [anon_sym_SEMI] = ACTIONS(4034),
    [anon_sym_macro] = ACTIONS(4036),
    [anon_sym_LPAREN] = ACTIONS(4034),
    [anon_sym_LBRACE] = ACTIONS(4034),
    [anon_sym_RBRACE] = ACTIONS(4034),
    [anon_sym_macro_rules] = ACTIONS(4036),
    [anon_sym_BANG] = ACTIONS(4034),
    [anon_sym_LBRACK] = ACTIONS(4034),
    [anon_sym_STAR] = ACTIONS(4034),
    [anon_sym_u8] = ACTIONS(4036),
    [anon_sym_i8] = ACTIONS(4036),
    [anon_sym_u16] = ACTIONS(4036),
    [anon_sym_i16] = ACTIONS(4036),
    [anon_sym_u32] = ACTIONS(4036),
    [anon_sym_i32] = ACTIONS(4036),
    [anon_sym_u64] = ACTIONS(4036),
    [anon_sym_i64] = ACTIONS(4036),
    [anon_sym_u128] = ACTIONS(4036),
    [anon_sym_i128] = ACTIONS(4036),
    [anon_sym_isize] = ACTIONS(4036),
    [anon_sym_usize] = ACTIONS(4036),
    [anon_sym_f32] = ACTIONS(4036),
    [anon_sym_f64] = ACTIONS(4036),
    [anon_sym_bool] = ACTIONS(4036),
    [anon_sym_str] = ACTIONS(4036),
    [anon_sym_char] = ACTIONS(4036),
    [anon_sym_DASH] = ACTIONS(4034),
    [anon_sym_AMP] = ACTIONS(4034),
    [anon_sym_PIPE] = ACTIONS(4034),
    [anon_sym_LT] = ACTIONS(4034),
    [anon_sym_DOT_DOT] = ACTIONS(4034),
    [anon_sym_COLON_COLON] = ACTIONS(4034),
    [anon_sym_POUND] = ACTIONS(4034),
    [anon_sym_SQUOTE] = ACTIONS(4036),
    [anon_sym_async] = ACTIONS(4036),
    [anon_sym_auto] = ACTIONS(4036),
    [anon_sym_await] = ACTIONS(4036),
    [anon_sym_become] = ACTIONS(4036),
    [anon_sym_break] = ACTIONS(4036),
    [anon_sym_const] = ACTIONS(4036),
    [anon_sym_continue] = ACTIONS(4036),
    [anon_sym_default] = ACTIONS(4036),
    [anon_sym_enum] = ACTIONS(4036),
    [anon_sym_fn] = ACTIONS(4036),
    [anon_sym_for] = ACTIONS(4036),
    [anon_sym_gen] = ACTIONS(4036),
    [anon_sym_if] = ACTIONS(4036),
    [anon_sym_impl] = ACTIONS(4036),
    [anon_sym_let] = ACTIONS(4036),
    [anon_sym_loop] = ACTIONS(4036),
    [anon_sym_match] = ACTIONS(4036),
    [anon_sym_mod] = ACTIONS(4036),
    [anon_sym_pub] = ACTIONS(4036),
    [anon_sym_return] = ACTIONS(4036),
    [anon_sym_raw] = ACTIONS(4036),
    [anon_sym_static] = ACTIONS(4036),
    [anon_sym_struct] = ACTIONS(4036),
    [anon_sym_trait] = ACTIONS(4036),
    [anon_sym_type] = ACTIONS(4036),
    [anon_sym_union] = ACTIONS(4036),
    [anon_sym_unsafe] = ACTIONS(4036),
    [anon_sym_use] = ACTIONS(4036),
    [anon_sym_while] = ACTIONS(4036),
    [anon_sym_extern] = ACTIONS(4036),
    [anon_sym_safe] = ACTIONS(4036),
    [anon_sym_yield] = ACTIONS(4036),
    [anon_sym_move] = ACTIONS(4036),
    [anon_sym_try] = ACTIONS(4036),
    [sym_integer_literal] = ACTIONS(4034),
    [aux_sym_string_literal_token1] = ACTIONS(4034),
    [sym_char_literal] = ACTIONS(4034),
    [anon_sym_true] = ACTIONS(4036),
    [anon_sym_false] = ACTIONS(4036),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4036),
    [sym_super] = ACTIONS(4036),
    [sym_crate] = ACTIONS(4036),
    [aux_sym_metavariable_token1] = ACTIONS(4034),
    [sym__raw_string_literal_start] = ACTIONS(4034),
    [sym_float_literal] = ACTIONS(4034),
    [sym__outer_block_doc_comment_start] = ACTIONS(4034),
    [sym__outer_line_doc_comment_start] = ACTIONS(4034),
  },
  [STATE(1067)] = {
    [sym_line_comment] = STATE(1067),
    [sym_block_comment] = STATE(1067),
    [ts_builtin_sym_end] = ACTIONS(4038),
    [sym_identifier] = ACTIONS(4040),
    [anon_sym_SEMI] = ACTIONS(4038),
    [anon_sym_macro] = ACTIONS(4040),
    [anon_sym_LPAREN] = ACTIONS(4038),
    [anon_sym_LBRACE] = ACTIONS(4038),
    [anon_sym_RBRACE] = ACTIONS(4038),
    [anon_sym_macro_rules] = ACTIONS(4040),
    [anon_sym_BANG] = ACTIONS(4038),
    [anon_sym_LBRACK] = ACTIONS(4038),
    [anon_sym_STAR] = ACTIONS(4038),
    [anon_sym_u8] = ACTIONS(4040),
    [anon_sym_i8] = ACTIONS(4040),
    [anon_sym_u16] = ACTIONS(4040),
    [anon_sym_i16] = ACTIONS(4040),
    [anon_sym_u32] = ACTIONS(4040),
    [anon_sym_i32] = ACTIONS(4040),
    [anon_sym_u64] = ACTIONS(4040),
    [anon_sym_i64] = ACTIONS(4040),
    [anon_sym_u128] = ACTIONS(4040),
    [anon_sym_i128] = ACTIONS(4040),
    [anon_sym_isize] = ACTIONS(4040),
    [anon_sym_usize] = ACTIONS(4040),
    [anon_sym_f32] = ACTIONS(4040),
    [anon_sym_f64] = ACTIONS(4040),
    [anon_sym_bool] = ACTIONS(4040),
    [anon_sym_str] = ACTIONS(4040),
    [anon_sym_char] = ACTIONS(4040),
    [anon_sym_DASH] = ACTIONS(4038),
    [anon_sym_AMP] = ACTIONS(4038),
    [anon_sym_PIPE] = ACTIONS(4038),
    [anon_sym_LT] = ACTIONS(4038),
    [anon_sym_DOT_DOT] = ACTIONS(4038),
    [anon_sym_COLON_COLON] = ACTIONS(4038),
    [anon_sym_POUND] = ACTIONS(4038),
    [anon_sym_SQUOTE] = ACTIONS(4040),
    [anon_sym_async] = ACTIONS(4040),
    [anon_sym_auto] = ACTIONS(4040),
    [anon_sym_await] = ACTIONS(4040),
    [anon_sym_become] = ACTIONS(4040),
    [anon_sym_break] = ACTIONS(4040),
    [anon_sym_const] = ACTIONS(4040),
    [anon_sym_continue] = ACTIONS(4040),
    [anon_sym_default] = ACTIONS(4040),
    [anon_sym_enum] = ACTIONS(4040),
    [anon_sym_fn] = ACTIONS(4040),
    [anon_sym_for] = ACTIONS(4040),
    [anon_sym_gen] = ACTIONS(4040),
    [anon_sym_if] = ACTIONS(4040),
    [anon_sym_impl] = ACTIONS(4040),
    [anon_sym_let] = ACTIONS(4040),
    [anon_sym_loop] = ACTIONS(4040),
    [anon_sym_match] = ACTIONS(4040),
    [anon_sym_mod] = ACTIONS(4040),
    [anon_sym_pub] = ACTIONS(4040),
    [anon_sym_return] = ACTIONS(4040),
    [anon_sym_raw] = ACTIONS(4040),
    [anon_sym_static] = ACTIONS(4040),
    [anon_sym_struct] = ACTIONS(4040),
    [anon_sym_trait] = ACTIONS(4040),
    [anon_sym_type] = ACTIONS(4040),
    [anon_sym_union] = ACTIONS(4040),
    [anon_sym_unsafe] = ACTIONS(4040),
    [anon_sym_use] = ACTIONS(4040),
    [anon_sym_while] = ACTIONS(4040),
    [anon_sym_extern] = ACTIONS(4040),
    [anon_sym_safe] = ACTIONS(4040),
    [anon_sym_yield] = ACTIONS(4040),
    [anon_sym_move] = ACTIONS(4040),
    [anon_sym_try] = ACTIONS(4040),
    [sym_integer_literal] = ACTIONS(4038),
    [aux_sym_string_literal_token1] = ACTIONS(4038),
    [sym_char_literal] = ACTIONS(4038),
    [anon_sym_true] = ACTIONS(4040),
    [anon_sym_false] = ACTIONS(4040),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4040),
    [sym_super] = ACTIONS(4040),
    [sym_crate] = ACTIONS(4040),
    [aux_sym_metavariable_token1] = ACTIONS(4038),
    [sym__raw_string_literal_start] = ACTIONS(4038),
    [sym_float_literal] = ACTIONS(4038),
    [sym__outer_block_doc_comment_start] = ACTIONS(4038),
    [sym__outer_line_doc_comment_start] = ACTIONS(4038),
  },
  [STATE(1068)] = {
    [sym_line_comment] = STATE(1068),
    [sym_block_comment] = STATE(1068),
    [ts_builtin_sym_end] = ACTIONS(4042),
    [sym_identifier] = ACTIONS(4044),
    [anon_sym_SEMI] = ACTIONS(4042),
    [anon_sym_macro] = ACTIONS(4044),
    [anon_sym_LPAREN] = ACTIONS(4042),
    [anon_sym_LBRACE] = ACTIONS(4042),
    [anon_sym_RBRACE] = ACTIONS(4042),
    [anon_sym_macro_rules] = ACTIONS(4044),
    [anon_sym_BANG] = ACTIONS(4042),
    [anon_sym_LBRACK] = ACTIONS(4042),
    [anon_sym_STAR] = ACTIONS(4042),
    [anon_sym_u8] = ACTIONS(4044),
    [anon_sym_i8] = ACTIONS(4044),
    [anon_sym_u16] = ACTIONS(4044),
    [anon_sym_i16] = ACTIONS(4044),
    [anon_sym_u32] = ACTIONS(4044),
    [anon_sym_i32] = ACTIONS(4044),
    [anon_sym_u64] = ACTIONS(4044),
    [anon_sym_i64] = ACTIONS(4044),
    [anon_sym_u128] = ACTIONS(4044),
    [anon_sym_i128] = ACTIONS(4044),
    [anon_sym_isize] = ACTIONS(4044),
    [anon_sym_usize] = ACTIONS(4044),
    [anon_sym_f32] = ACTIONS(4044),
    [anon_sym_f64] = ACTIONS(4044),
    [anon_sym_bool] = ACTIONS(4044),
    [anon_sym_str] = ACTIONS(4044),
    [anon_sym_char] = ACTIONS(4044),
    [anon_sym_DASH] = ACTIONS(4042),
    [anon_sym_AMP] = ACTIONS(4042),
    [anon_sym_PIPE] = ACTIONS(4042),
    [anon_sym_LT] = ACTIONS(4042),
    [anon_sym_DOT_DOT] = ACTIONS(4042),
    [anon_sym_COLON_COLON] = ACTIONS(4042),
    [anon_sym_POUND] = ACTIONS(4042),
    [anon_sym_SQUOTE] = ACTIONS(4044),
    [anon_sym_async] = ACTIONS(4044),
    [anon_sym_auto] = ACTIONS(4044),
    [anon_sym_await] = ACTIONS(4044),
    [anon_sym_become] = ACTIONS(4044),
    [anon_sym_break] = ACTIONS(4044),
    [anon_sym_const] = ACTIONS(4044),
    [anon_sym_continue] = ACTIONS(4044),
    [anon_sym_default] = ACTIONS(4044),
    [anon_sym_enum] = ACTIONS(4044),
    [anon_sym_fn] = ACTIONS(4044),
    [anon_sym_for] = ACTIONS(4044),
    [anon_sym_gen] = ACTIONS(4044),
    [anon_sym_if] = ACTIONS(4044),
    [anon_sym_impl] = ACTIONS(4044),
    [anon_sym_let] = ACTIONS(4044),
    [anon_sym_loop] = ACTIONS(4044),
    [anon_sym_match] = ACTIONS(4044),
    [anon_sym_mod] = ACTIONS(4044),
    [anon_sym_pub] = ACTIONS(4044),
    [anon_sym_return] = ACTIONS(4044),
    [anon_sym_raw] = ACTIONS(4044),
    [anon_sym_static] = ACTIONS(4044),
    [anon_sym_struct] = ACTIONS(4044),
    [anon_sym_trait] = ACTIONS(4044),
    [anon_sym_type] = ACTIONS(4044),
    [anon_sym_union] = ACTIONS(4044),
    [anon_sym_unsafe] = ACTIONS(4044),
    [anon_sym_use] = ACTIONS(4044),
    [anon_sym_while] = ACTIONS(4044),
    [anon_sym_extern] = ACTIONS(4044),
    [anon_sym_safe] = ACTIONS(4044),
    [anon_sym_yield] = ACTIONS(4044),
    [anon_sym_move] = ACTIONS(4044),
    [anon_sym_try] = ACTIONS(4044),
    [sym_integer_literal] = ACTIONS(4042),
    [aux_sym_string_literal_token1] = ACTIONS(4042),
    [sym_char_literal] = ACTIONS(4042),
    [anon_sym_true] = ACTIONS(4044),
    [anon_sym_false] = ACTIONS(4044),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4044),
    [sym_super] = ACTIONS(4044),
    [sym_crate] = ACTIONS(4044),
    [aux_sym_metavariable_token1] = ACTIONS(4042),
    [sym__raw_string_literal_start] = ACTIONS(4042),
    [sym_float_literal] = ACTIONS(4042),
    [sym__outer_block_doc_comment_start] = ACTIONS(4042),
    [sym__outer_line_doc_comment_start] = ACTIONS(4042),
  },
  [STATE(1069)] = {
    [sym_line_comment] = STATE(1069),
    [sym_block_comment] = STATE(1069),
    [ts_builtin_sym_end] = ACTIONS(4046),
    [sym_identifier] = ACTIONS(4048),
    [anon_sym_SEMI] = ACTIONS(4046),
    [anon_sym_macro] = ACTIONS(4048),
    [anon_sym_LPAREN] = ACTIONS(4046),
    [anon_sym_LBRACE] = ACTIONS(4046),
    [anon_sym_RBRACE] = ACTIONS(4046),
    [anon_sym_macro_rules] = ACTIONS(4048),
    [anon_sym_BANG] = ACTIONS(4046),
    [anon_sym_LBRACK] = ACTIONS(4046),
    [anon_sym_STAR] = ACTIONS(4046),
    [anon_sym_u8] = ACTIONS(4048),
    [anon_sym_i8] = ACTIONS(4048),
    [anon_sym_u16] = ACTIONS(4048),
    [anon_sym_i16] = ACTIONS(4048),
    [anon_sym_u32] = ACTIONS(4048),
    [anon_sym_i32] = ACTIONS(4048),
    [anon_sym_u64] = ACTIONS(4048),
    [anon_sym_i64] = ACTIONS(4048),
    [anon_sym_u128] = ACTIONS(4048),
    [anon_sym_i128] = ACTIONS(4048),
    [anon_sym_isize] = ACTIONS(4048),
    [anon_sym_usize] = ACTIONS(4048),
    [anon_sym_f32] = ACTIONS(4048),
    [anon_sym_f64] = ACTIONS(4048),
    [anon_sym_bool] = ACTIONS(4048),
    [anon_sym_str] = ACTIONS(4048),
    [anon_sym_char] = ACTIONS(4048),
    [anon_sym_DASH] = ACTIONS(4046),
    [anon_sym_AMP] = ACTIONS(4046),
    [anon_sym_PIPE] = ACTIONS(4046),
    [anon_sym_LT] = ACTIONS(4046),
    [anon_sym_DOT_DOT] = ACTIONS(4046),
    [anon_sym_COLON_COLON] = ACTIONS(4046),
    [anon_sym_POUND] = ACTIONS(4046),
    [anon_sym_SQUOTE] = ACTIONS(4048),
    [anon_sym_async] = ACTIONS(4048),
    [anon_sym_auto] = ACTIONS(4048),
    [anon_sym_await] = ACTIONS(4048),
    [anon_sym_become] = ACTIONS(4048),
    [anon_sym_break] = ACTIONS(4048),
    [anon_sym_const] = ACTIONS(4048),
    [anon_sym_continue] = ACTIONS(4048),
    [anon_sym_default] = ACTIONS(4048),
    [anon_sym_enum] = ACTIONS(4048),
    [anon_sym_fn] = ACTIONS(4048),
    [anon_sym_for] = ACTIONS(4048),
    [anon_sym_gen] = ACTIONS(4048),
    [anon_sym_if] = ACTIONS(4048),
    [anon_sym_impl] = ACTIONS(4048),
    [anon_sym_let] = ACTIONS(4048),
    [anon_sym_loop] = ACTIONS(4048),
    [anon_sym_match] = ACTIONS(4048),
    [anon_sym_mod] = ACTIONS(4048),
    [anon_sym_pub] = ACTIONS(4048),
    [anon_sym_return] = ACTIONS(4048),
    [anon_sym_raw] = ACTIONS(4048),
    [anon_sym_static] = ACTIONS(4048),
    [anon_sym_struct] = ACTIONS(4048),
    [anon_sym_trait] = ACTIONS(4048),
    [anon_sym_type] = ACTIONS(4048),
    [anon_sym_union] = ACTIONS(4048),
    [anon_sym_unsafe] = ACTIONS(4048),
    [anon_sym_use] = ACTIONS(4048),
    [anon_sym_while] = ACTIONS(4048),
    [anon_sym_extern] = ACTIONS(4048),
    [anon_sym_safe] = ACTIONS(4048),
    [anon_sym_yield] = ACTIONS(4048),
    [anon_sym_move] = ACTIONS(4048),
    [anon_sym_try] = ACTIONS(4048),
    [sym_integer_literal] = ACTIONS(4046),
    [aux_sym_string_literal_token1] = ACTIONS(4046),
    [sym_char_literal] = ACTIONS(4046),
    [anon_sym_true] = ACTIONS(4048),
    [anon_sym_false] = ACTIONS(4048),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4048),
    [sym_super] = ACTIONS(4048),
    [sym_crate] = ACTIONS(4048),
    [aux_sym_metavariable_token1] = ACTIONS(4046),
    [sym__raw_string_literal_start] = ACTIONS(4046),
    [sym_float_literal] = ACTIONS(4046),
    [sym__outer_block_doc_comment_start] = ACTIONS(4046),
    [sym__outer_line_doc_comment_start] = ACTIONS(4046),
  },
  [STATE(1070)] = {
    [sym_line_comment] = STATE(1070),
    [sym_block_comment] = STATE(1070),
    [ts_builtin_sym_end] = ACTIONS(4050),
    [sym_identifier] = ACTIONS(4052),
    [anon_sym_SEMI] = ACTIONS(4050),
    [anon_sym_macro] = ACTIONS(4052),
    [anon_sym_LPAREN] = ACTIONS(4050),
    [anon_sym_LBRACE] = ACTIONS(4050),
    [anon_sym_RBRACE] = ACTIONS(4050),
    [anon_sym_macro_rules] = ACTIONS(4052),
    [anon_sym_BANG] = ACTIONS(4050),
    [anon_sym_LBRACK] = ACTIONS(4050),
    [anon_sym_STAR] = ACTIONS(4050),
    [anon_sym_u8] = ACTIONS(4052),
    [anon_sym_i8] = ACTIONS(4052),
    [anon_sym_u16] = ACTIONS(4052),
    [anon_sym_i16] = ACTIONS(4052),
    [anon_sym_u32] = ACTIONS(4052),
    [anon_sym_i32] = ACTIONS(4052),
    [anon_sym_u64] = ACTIONS(4052),
    [anon_sym_i64] = ACTIONS(4052),
    [anon_sym_u128] = ACTIONS(4052),
    [anon_sym_i128] = ACTIONS(4052),
    [anon_sym_isize] = ACTIONS(4052),
    [anon_sym_usize] = ACTIONS(4052),
    [anon_sym_f32] = ACTIONS(4052),
    [anon_sym_f64] = ACTIONS(4052),
    [anon_sym_bool] = ACTIONS(4052),
    [anon_sym_str] = ACTIONS(4052),
    [anon_sym_char] = ACTIONS(4052),
    [anon_sym_DASH] = ACTIONS(4050),
    [anon_sym_AMP] = ACTIONS(4050),
    [anon_sym_PIPE] = ACTIONS(4050),
    [anon_sym_LT] = ACTIONS(4050),
    [anon_sym_DOT_DOT] = ACTIONS(4050),
    [anon_sym_COLON_COLON] = ACTIONS(4050),
    [anon_sym_POUND] = ACTIONS(4050),
    [anon_sym_SQUOTE] = ACTIONS(4052),
    [anon_sym_async] = ACTIONS(4052),
    [anon_sym_auto] = ACTIONS(4052),
    [anon_sym_await] = ACTIONS(4052),
    [anon_sym_become] = ACTIONS(4052),
    [anon_sym_break] = ACTIONS(4052),
    [anon_sym_const] = ACTIONS(4052),
    [anon_sym_continue] = ACTIONS(4052),
    [anon_sym_default] = ACTIONS(4052),
    [anon_sym_enum] = ACTIONS(4052),
    [anon_sym_fn] = ACTIONS(4052),
    [anon_sym_for] = ACTIONS(4052),
    [anon_sym_gen] = ACTIONS(4052),
    [anon_sym_if] = ACTIONS(4052),
    [anon_sym_impl] = ACTIONS(4052),
    [anon_sym_let] = ACTIONS(4052),
    [anon_sym_loop] = ACTIONS(4052),
    [anon_sym_match] = ACTIONS(4052),
    [anon_sym_mod] = ACTIONS(4052),
    [anon_sym_pub] = ACTIONS(4052),
    [anon_sym_return] = ACTIONS(4052),
    [anon_sym_raw] = ACTIONS(4052),
    [anon_sym_static] = ACTIONS(4052),
    [anon_sym_struct] = ACTIONS(4052),
    [anon_sym_trait] = ACTIONS(4052),
    [anon_sym_type] = ACTIONS(4052),
    [anon_sym_union] = ACTIONS(4052),
    [anon_sym_unsafe] = ACTIONS(4052),
    [anon_sym_use] = ACTIONS(4052),
    [anon_sym_while] = ACTIONS(4052),
    [anon_sym_extern] = ACTIONS(4052),
    [anon_sym_safe] = ACTIONS(4052),
    [anon_sym_yield] = ACTIONS(4052),
    [anon_sym_move] = ACTIONS(4052),
    [anon_sym_try] = ACTIONS(4052),
    [sym_integer_literal] = ACTIONS(4050),
    [aux_sym_string_literal_token1] = ACTIONS(4050),
    [sym_char_literal] = ACTIONS(4050),
    [anon_sym_true] = ACTIONS(4052),
    [anon_sym_false] = ACTIONS(4052),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4052),
    [sym_super] = ACTIONS(4052),
    [sym_crate] = ACTIONS(4052),
    [aux_sym_metavariable_token1] = ACTIONS(4050),
    [sym__raw_string_literal_start] = ACTIONS(4050),
    [sym_float_literal] = ACTIONS(4050),
    [sym__outer_block_doc_comment_start] = ACTIONS(4050),
    [sym__outer_line_doc_comment_start] = ACTIONS(4050),
  },
  [STATE(1071)] = {
    [sym_line_comment] = STATE(1071),
    [sym_block_comment] = STATE(1071),
    [ts_builtin_sym_end] = ACTIONS(4054),
    [sym_identifier] = ACTIONS(4056),
    [anon_sym_SEMI] = ACTIONS(4054),
    [anon_sym_macro] = ACTIONS(4056),
    [anon_sym_LPAREN] = ACTIONS(4054),
    [anon_sym_LBRACE] = ACTIONS(4054),
    [anon_sym_RBRACE] = ACTIONS(4054),
    [anon_sym_macro_rules] = ACTIONS(4056),
    [anon_sym_BANG] = ACTIONS(4054),
    [anon_sym_LBRACK] = ACTIONS(4054),
    [anon_sym_STAR] = ACTIONS(4054),
    [anon_sym_u8] = ACTIONS(4056),
    [anon_sym_i8] = ACTIONS(4056),
    [anon_sym_u16] = ACTIONS(4056),
    [anon_sym_i16] = ACTIONS(4056),
    [anon_sym_u32] = ACTIONS(4056),
    [anon_sym_i32] = ACTIONS(4056),
    [anon_sym_u64] = ACTIONS(4056),
    [anon_sym_i64] = ACTIONS(4056),
    [anon_sym_u128] = ACTIONS(4056),
    [anon_sym_i128] = ACTIONS(4056),
    [anon_sym_isize] = ACTIONS(4056),
    [anon_sym_usize] = ACTIONS(4056),
    [anon_sym_f32] = ACTIONS(4056),
    [anon_sym_f64] = ACTIONS(4056),
    [anon_sym_bool] = ACTIONS(4056),
    [anon_sym_str] = ACTIONS(4056),
    [anon_sym_char] = ACTIONS(4056),
    [anon_sym_DASH] = ACTIONS(4054),
    [anon_sym_AMP] = ACTIONS(4054),
    [anon_sym_PIPE] = ACTIONS(4054),
    [anon_sym_LT] = ACTIONS(4054),
    [anon_sym_DOT_DOT] = ACTIONS(4054),
    [anon_sym_COLON_COLON] = ACTIONS(4054),
    [anon_sym_POUND] = ACTIONS(4054),
    [anon_sym_SQUOTE] = ACTIONS(4056),
    [anon_sym_async] = ACTIONS(4056),
    [anon_sym_auto] = ACTIONS(4056),
    [anon_sym_await] = ACTIONS(4056),
    [anon_sym_become] = ACTIONS(4056),
    [anon_sym_break] = ACTIONS(4056),
    [anon_sym_const] = ACTIONS(4056),
    [anon_sym_continue] = ACTIONS(4056),
    [anon_sym_default] = ACTIONS(4056),
    [anon_sym_enum] = ACTIONS(4056),
    [anon_sym_fn] = ACTIONS(4056),
    [anon_sym_for] = ACTIONS(4056),
    [anon_sym_gen] = ACTIONS(4056),
    [anon_sym_if] = ACTIONS(4056),
    [anon_sym_impl] = ACTIONS(4056),
    [anon_sym_let] = ACTIONS(4056),
    [anon_sym_loop] = ACTIONS(4056),
    [anon_sym_match] = ACTIONS(4056),
    [anon_sym_mod] = ACTIONS(4056),
    [anon_sym_pub] = ACTIONS(4056),
    [anon_sym_return] = ACTIONS(4056),
    [anon_sym_raw] = ACTIONS(4056),
    [anon_sym_static] = ACTIONS(4056),
    [anon_sym_struct] = ACTIONS(4056),
    [anon_sym_trait] = ACTIONS(4056),
    [anon_sym_type] = ACTIONS(4056),
    [anon_sym_union] = ACTIONS(4056),
    [anon_sym_unsafe] = ACTIONS(4056),
    [anon_sym_use] = ACTIONS(4056),
    [anon_sym_while] = ACTIONS(4056),
    [anon_sym_extern] = ACTIONS(4056),
    [anon_sym_safe] = ACTIONS(4056),
    [anon_sym_yield] = ACTIONS(4056),
    [anon_sym_move] = ACTIONS(4056),
    [anon_sym_try] = ACTIONS(4056),
    [sym_integer_literal] = ACTIONS(4054),
    [aux_sym_string_literal_token1] = ACTIONS(4054),
    [sym_char_literal] = ACTIONS(4054),
    [anon_sym_true] = ACTIONS(4056),
    [anon_sym_false] = ACTIONS(4056),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4056),
    [sym_super] = ACTIONS(4056),
    [sym_crate] = ACTIONS(4056),
    [aux_sym_metavariable_token1] = ACTIONS(4054),
    [sym__raw_string_literal_start] = ACTIONS(4054),
    [sym_float_literal] = ACTIONS(4054),
    [sym__outer_block_doc_comment_start] = ACTIONS(4054),
    [sym__outer_line_doc_comment_start] = ACTIONS(4054),
  },
  [STATE(1072)] = {
    [sym_line_comment] = STATE(1072),
    [sym_block_comment] = STATE(1072),
    [ts_builtin_sym_end] = ACTIONS(4058),
    [sym_identifier] = ACTIONS(4060),
    [anon_sym_SEMI] = ACTIONS(4058),
    [anon_sym_macro] = ACTIONS(4060),
    [anon_sym_LPAREN] = ACTIONS(4058),
    [anon_sym_LBRACE] = ACTIONS(4058),
    [anon_sym_RBRACE] = ACTIONS(4058),
    [anon_sym_macro_rules] = ACTIONS(4060),
    [anon_sym_BANG] = ACTIONS(4058),
    [anon_sym_LBRACK] = ACTIONS(4058),
    [anon_sym_STAR] = ACTIONS(4058),
    [anon_sym_u8] = ACTIONS(4060),
    [anon_sym_i8] = ACTIONS(4060),
    [anon_sym_u16] = ACTIONS(4060),
    [anon_sym_i16] = ACTIONS(4060),
    [anon_sym_u32] = ACTIONS(4060),
    [anon_sym_i32] = ACTIONS(4060),
    [anon_sym_u64] = ACTIONS(4060),
    [anon_sym_i64] = ACTIONS(4060),
    [anon_sym_u128] = ACTIONS(4060),
    [anon_sym_i128] = ACTIONS(4060),
    [anon_sym_isize] = ACTIONS(4060),
    [anon_sym_usize] = ACTIONS(4060),
    [anon_sym_f32] = ACTIONS(4060),
    [anon_sym_f64] = ACTIONS(4060),
    [anon_sym_bool] = ACTIONS(4060),
    [anon_sym_str] = ACTIONS(4060),
    [anon_sym_char] = ACTIONS(4060),
    [anon_sym_DASH] = ACTIONS(4058),
    [anon_sym_AMP] = ACTIONS(4058),
    [anon_sym_PIPE] = ACTIONS(4058),
    [anon_sym_LT] = ACTIONS(4058),
    [anon_sym_DOT_DOT] = ACTIONS(4058),
    [anon_sym_COLON_COLON] = ACTIONS(4058),
    [anon_sym_POUND] = ACTIONS(4058),
    [anon_sym_SQUOTE] = ACTIONS(4060),
    [anon_sym_async] = ACTIONS(4060),
    [anon_sym_auto] = ACTIONS(4060),
    [anon_sym_await] = ACTIONS(4060),
    [anon_sym_become] = ACTIONS(4060),
    [anon_sym_break] = ACTIONS(4060),
    [anon_sym_const] = ACTIONS(4060),
    [anon_sym_continue] = ACTIONS(4060),
    [anon_sym_default] = ACTIONS(4060),
    [anon_sym_enum] = ACTIONS(4060),
    [anon_sym_fn] = ACTIONS(4060),
    [anon_sym_for] = ACTIONS(4060),
    [anon_sym_gen] = ACTIONS(4060),
    [anon_sym_if] = ACTIONS(4060),
    [anon_sym_impl] = ACTIONS(4060),
    [anon_sym_let] = ACTIONS(4060),
    [anon_sym_loop] = ACTIONS(4060),
    [anon_sym_match] = ACTIONS(4060),
    [anon_sym_mod] = ACTIONS(4060),
    [anon_sym_pub] = ACTIONS(4060),
    [anon_sym_return] = ACTIONS(4060),
    [anon_sym_raw] = ACTIONS(4060),
    [anon_sym_static] = ACTIONS(4060),
    [anon_sym_struct] = ACTIONS(4060),
    [anon_sym_trait] = ACTIONS(4060),
    [anon_sym_type] = ACTIONS(4060),
    [anon_sym_union] = ACTIONS(4060),
    [anon_sym_unsafe] = ACTIONS(4060),
    [anon_sym_use] = ACTIONS(4060),
    [anon_sym_while] = ACTIONS(4060),
    [anon_sym_extern] = ACTIONS(4060),
    [anon_sym_safe] = ACTIONS(4060),
    [anon_sym_yield] = ACTIONS(4060),
    [anon_sym_move] = ACTIONS(4060),
    [anon_sym_try] = ACTIONS(4060),
    [sym_integer_literal] = ACTIONS(4058),
    [aux_sym_string_literal_token1] = ACTIONS(4058),
    [sym_char_literal] = ACTIONS(4058),
    [anon_sym_true] = ACTIONS(4060),
    [anon_sym_false] = ACTIONS(4060),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4060),
    [sym_super] = ACTIONS(4060),
    [sym_crate] = ACTIONS(4060),
    [aux_sym_metavariable_token1] = ACTIONS(4058),
    [sym__raw_string_literal_start] = ACTIONS(4058),
    [sym_float_literal] = ACTIONS(4058),
    [sym__outer_block_doc_comment_start] = ACTIONS(4058),
    [sym__outer_line_doc_comment_start] = ACTIONS(4058),
  },
  [STATE(1073)] = {
    [sym_line_comment] = STATE(1073),
    [sym_block_comment] = STATE(1073),
    [ts_builtin_sym_end] = ACTIONS(4062),
    [sym_identifier] = ACTIONS(4064),
    [anon_sym_SEMI] = ACTIONS(4062),
    [anon_sym_macro] = ACTIONS(4064),
    [anon_sym_LPAREN] = ACTIONS(4062),
    [anon_sym_LBRACE] = ACTIONS(4062),
    [anon_sym_RBRACE] = ACTIONS(4062),
    [anon_sym_macro_rules] = ACTIONS(4064),
    [anon_sym_BANG] = ACTIONS(4062),
    [anon_sym_LBRACK] = ACTIONS(4062),
    [anon_sym_STAR] = ACTIONS(4062),
    [anon_sym_u8] = ACTIONS(4064),
    [anon_sym_i8] = ACTIONS(4064),
    [anon_sym_u16] = ACTIONS(4064),
    [anon_sym_i16] = ACTIONS(4064),
    [anon_sym_u32] = ACTIONS(4064),
    [anon_sym_i32] = ACTIONS(4064),
    [anon_sym_u64] = ACTIONS(4064),
    [anon_sym_i64] = ACTIONS(4064),
    [anon_sym_u128] = ACTIONS(4064),
    [anon_sym_i128] = ACTIONS(4064),
    [anon_sym_isize] = ACTIONS(4064),
    [anon_sym_usize] = ACTIONS(4064),
    [anon_sym_f32] = ACTIONS(4064),
    [anon_sym_f64] = ACTIONS(4064),
    [anon_sym_bool] = ACTIONS(4064),
    [anon_sym_str] = ACTIONS(4064),
    [anon_sym_char] = ACTIONS(4064),
    [anon_sym_DASH] = ACTIONS(4062),
    [anon_sym_AMP] = ACTIONS(4062),
    [anon_sym_PIPE] = ACTIONS(4062),
    [anon_sym_LT] = ACTIONS(4062),
    [anon_sym_DOT_DOT] = ACTIONS(4062),
    [anon_sym_COLON_COLON] = ACTIONS(4062),
    [anon_sym_POUND] = ACTIONS(4062),
    [anon_sym_SQUOTE] = ACTIONS(4064),
    [anon_sym_async] = ACTIONS(4064),
    [anon_sym_auto] = ACTIONS(4064),
    [anon_sym_await] = ACTIONS(4064),
    [anon_sym_become] = ACTIONS(4064),
    [anon_sym_break] = ACTIONS(4064),
    [anon_sym_const] = ACTIONS(4064),
    [anon_sym_continue] = ACTIONS(4064),
    [anon_sym_default] = ACTIONS(4064),
    [anon_sym_enum] = ACTIONS(4064),
    [anon_sym_fn] = ACTIONS(4064),
    [anon_sym_for] = ACTIONS(4064),
    [anon_sym_gen] = ACTIONS(4064),
    [anon_sym_if] = ACTIONS(4064),
    [anon_sym_impl] = ACTIONS(4064),
    [anon_sym_let] = ACTIONS(4064),
    [anon_sym_loop] = ACTIONS(4064),
    [anon_sym_match] = ACTIONS(4064),
    [anon_sym_mod] = ACTIONS(4064),
    [anon_sym_pub] = ACTIONS(4064),
    [anon_sym_return] = ACTIONS(4064),
    [anon_sym_raw] = ACTIONS(4064),
    [anon_sym_static] = ACTIONS(4064),
    [anon_sym_struct] = ACTIONS(4064),
    [anon_sym_trait] = ACTIONS(4064),
    [anon_sym_type] = ACTIONS(4064),
    [anon_sym_union] = ACTIONS(4064),
    [anon_sym_unsafe] = ACTIONS(4064),
    [anon_sym_use] = ACTIONS(4064),
    [anon_sym_while] = ACTIONS(4064),
    [anon_sym_extern] = ACTIONS(4064),
    [anon_sym_safe] = ACTIONS(4064),
    [anon_sym_yield] = ACTIONS(4064),
    [anon_sym_move] = ACTIONS(4064),
    [anon_sym_try] = ACTIONS(4064),
    [sym_integer_literal] = ACTIONS(4062),
    [aux_sym_string_literal_token1] = ACTIONS(4062),
    [sym_char_literal] = ACTIONS(4062),
    [anon_sym_true] = ACTIONS(4064),
    [anon_sym_false] = ACTIONS(4064),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4064),
    [sym_super] = ACTIONS(4064),
    [sym_crate] = ACTIONS(4064),
    [aux_sym_metavariable_token1] = ACTIONS(4062),
    [sym__raw_string_literal_start] = ACTIONS(4062),
    [sym_float_literal] = ACTIONS(4062),
    [sym__outer_block_doc_comment_start] = ACTIONS(4062),
    [sym__outer_line_doc_comment_start] = ACTIONS(4062),
  },
  [STATE(1074)] = {
    [sym_line_comment] = STATE(1074),
    [sym_block_comment] = STATE(1074),
    [ts_builtin_sym_end] = ACTIONS(4066),
    [sym_identifier] = ACTIONS(4068),
    [anon_sym_SEMI] = ACTIONS(4066),
    [anon_sym_macro] = ACTIONS(4068),
    [anon_sym_LPAREN] = ACTIONS(4066),
    [anon_sym_LBRACE] = ACTIONS(4066),
    [anon_sym_RBRACE] = ACTIONS(4066),
    [anon_sym_macro_rules] = ACTIONS(4068),
    [anon_sym_BANG] = ACTIONS(4066),
    [anon_sym_LBRACK] = ACTIONS(4066),
    [anon_sym_STAR] = ACTIONS(4066),
    [anon_sym_u8] = ACTIONS(4068),
    [anon_sym_i8] = ACTIONS(4068),
    [anon_sym_u16] = ACTIONS(4068),
    [anon_sym_i16] = ACTIONS(4068),
    [anon_sym_u32] = ACTIONS(4068),
    [anon_sym_i32] = ACTIONS(4068),
    [anon_sym_u64] = ACTIONS(4068),
    [anon_sym_i64] = ACTIONS(4068),
    [anon_sym_u128] = ACTIONS(4068),
    [anon_sym_i128] = ACTIONS(4068),
    [anon_sym_isize] = ACTIONS(4068),
    [anon_sym_usize] = ACTIONS(4068),
    [anon_sym_f32] = ACTIONS(4068),
    [anon_sym_f64] = ACTIONS(4068),
    [anon_sym_bool] = ACTIONS(4068),
    [anon_sym_str] = ACTIONS(4068),
    [anon_sym_char] = ACTIONS(4068),
    [anon_sym_DASH] = ACTIONS(4066),
    [anon_sym_AMP] = ACTIONS(4066),
    [anon_sym_PIPE] = ACTIONS(4066),
    [anon_sym_LT] = ACTIONS(4066),
    [anon_sym_DOT_DOT] = ACTIONS(4066),
    [anon_sym_COLON_COLON] = ACTIONS(4066),
    [anon_sym_POUND] = ACTIONS(4066),
    [anon_sym_SQUOTE] = ACTIONS(4068),
    [anon_sym_async] = ACTIONS(4068),
    [anon_sym_auto] = ACTIONS(4068),
    [anon_sym_await] = ACTIONS(4068),
    [anon_sym_become] = ACTIONS(4068),
    [anon_sym_break] = ACTIONS(4068),
    [anon_sym_const] = ACTIONS(4068),
    [anon_sym_continue] = ACTIONS(4068),
    [anon_sym_default] = ACTIONS(4068),
    [anon_sym_enum] = ACTIONS(4068),
    [anon_sym_fn] = ACTIONS(4068),
    [anon_sym_for] = ACTIONS(4068),
    [anon_sym_gen] = ACTIONS(4068),
    [anon_sym_if] = ACTIONS(4068),
    [anon_sym_impl] = ACTIONS(4068),
    [anon_sym_let] = ACTIONS(4068),
    [anon_sym_loop] = ACTIONS(4068),
    [anon_sym_match] = ACTIONS(4068),
    [anon_sym_mod] = ACTIONS(4068),
    [anon_sym_pub] = ACTIONS(4068),
    [anon_sym_return] = ACTIONS(4068),
    [anon_sym_raw] = ACTIONS(4068),
    [anon_sym_static] = ACTIONS(4068),
    [anon_sym_struct] = ACTIONS(4068),
    [anon_sym_trait] = ACTIONS(4068),
    [anon_sym_type] = ACTIONS(4068),
    [anon_sym_union] = ACTIONS(4068),
    [anon_sym_unsafe] = ACTIONS(4068),
    [anon_sym_use] = ACTIONS(4068),
    [anon_sym_while] = ACTIONS(4068),
    [anon_sym_extern] = ACTIONS(4068),
    [anon_sym_safe] = ACTIONS(4068),
    [anon_sym_yield] = ACTIONS(4068),
    [anon_sym_move] = ACTIONS(4068),
    [anon_sym_try] = ACTIONS(4068),
    [sym_integer_literal] = ACTIONS(4066),
    [aux_sym_string_literal_token1] = ACTIONS(4066),
    [sym_char_literal] = ACTIONS(4066),
    [anon_sym_true] = ACTIONS(4068),
    [anon_sym_false] = ACTIONS(4068),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4068),
    [sym_super] = ACTIONS(4068),
    [sym_crate] = ACTIONS(4068),
    [aux_sym_metavariable_token1] = ACTIONS(4066),
    [sym__raw_string_literal_start] = ACTIONS(4066),
    [sym_float_literal] = ACTIONS(4066),
    [sym__outer_block_doc_comment_start] = ACTIONS(4066),
    [sym__outer_line_doc_comment_start] = ACTIONS(4066),
  },
  [STATE(1075)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_lifetime] = STATE(5701),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5705),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1075),
    [sym_block_comment] = STATE(1075),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(4070),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(4072),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(4074),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4076),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1076)] = {
    [sym_line_comment] = STATE(1076),
    [sym_block_comment] = STATE(1076),
    [ts_builtin_sym_end] = ACTIONS(4078),
    [sym_identifier] = ACTIONS(4080),
    [anon_sym_SEMI] = ACTIONS(4078),
    [anon_sym_macro] = ACTIONS(4080),
    [anon_sym_LPAREN] = ACTIONS(4078),
    [anon_sym_LBRACE] = ACTIONS(4078),
    [anon_sym_RBRACE] = ACTIONS(4078),
    [anon_sym_macro_rules] = ACTIONS(4080),
    [anon_sym_BANG] = ACTIONS(4078),
    [anon_sym_LBRACK] = ACTIONS(4078),
    [anon_sym_STAR] = ACTIONS(4078),
    [anon_sym_u8] = ACTIONS(4080),
    [anon_sym_i8] = ACTIONS(4080),
    [anon_sym_u16] = ACTIONS(4080),
    [anon_sym_i16] = ACTIONS(4080),
    [anon_sym_u32] = ACTIONS(4080),
    [anon_sym_i32] = ACTIONS(4080),
    [anon_sym_u64] = ACTIONS(4080),
    [anon_sym_i64] = ACTIONS(4080),
    [anon_sym_u128] = ACTIONS(4080),
    [anon_sym_i128] = ACTIONS(4080),
    [anon_sym_isize] = ACTIONS(4080),
    [anon_sym_usize] = ACTIONS(4080),
    [anon_sym_f32] = ACTIONS(4080),
    [anon_sym_f64] = ACTIONS(4080),
    [anon_sym_bool] = ACTIONS(4080),
    [anon_sym_str] = ACTIONS(4080),
    [anon_sym_char] = ACTIONS(4080),
    [anon_sym_DASH] = ACTIONS(4078),
    [anon_sym_AMP] = ACTIONS(4078),
    [anon_sym_PIPE] = ACTIONS(4078),
    [anon_sym_LT] = ACTIONS(4078),
    [anon_sym_DOT_DOT] = ACTIONS(4078),
    [anon_sym_COLON_COLON] = ACTIONS(4078),
    [anon_sym_POUND] = ACTIONS(4078),
    [anon_sym_SQUOTE] = ACTIONS(4080),
    [anon_sym_async] = ACTIONS(4080),
    [anon_sym_auto] = ACTIONS(4080),
    [anon_sym_await] = ACTIONS(4080),
    [anon_sym_become] = ACTIONS(4080),
    [anon_sym_break] = ACTIONS(4080),
    [anon_sym_const] = ACTIONS(4080),
    [anon_sym_continue] = ACTIONS(4080),
    [anon_sym_default] = ACTIONS(4080),
    [anon_sym_enum] = ACTIONS(4080),
    [anon_sym_fn] = ACTIONS(4080),
    [anon_sym_for] = ACTIONS(4080),
    [anon_sym_gen] = ACTIONS(4080),
    [anon_sym_if] = ACTIONS(4080),
    [anon_sym_impl] = ACTIONS(4080),
    [anon_sym_let] = ACTIONS(4080),
    [anon_sym_loop] = ACTIONS(4080),
    [anon_sym_match] = ACTIONS(4080),
    [anon_sym_mod] = ACTIONS(4080),
    [anon_sym_pub] = ACTIONS(4080),
    [anon_sym_return] = ACTIONS(4080),
    [anon_sym_raw] = ACTIONS(4080),
    [anon_sym_static] = ACTIONS(4080),
    [anon_sym_struct] = ACTIONS(4080),
    [anon_sym_trait] = ACTIONS(4080),
    [anon_sym_type] = ACTIONS(4080),
    [anon_sym_union] = ACTIONS(4080),
    [anon_sym_unsafe] = ACTIONS(4080),
    [anon_sym_use] = ACTIONS(4080),
    [anon_sym_while] = ACTIONS(4080),
    [anon_sym_extern] = ACTIONS(4080),
    [anon_sym_safe] = ACTIONS(4080),
    [anon_sym_yield] = ACTIONS(4080),
    [anon_sym_move] = ACTIONS(4080),
    [anon_sym_try] = ACTIONS(4080),
    [sym_integer_literal] = ACTIONS(4078),
    [aux_sym_string_literal_token1] = ACTIONS(4078),
    [sym_char_literal] = ACTIONS(4078),
    [anon_sym_true] = ACTIONS(4080),
    [anon_sym_false] = ACTIONS(4080),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4080),
    [sym_super] = ACTIONS(4080),
    [sym_crate] = ACTIONS(4080),
    [aux_sym_metavariable_token1] = ACTIONS(4078),
    [sym__raw_string_literal_start] = ACTIONS(4078),
    [sym_float_literal] = ACTIONS(4078),
    [sym__outer_block_doc_comment_start] = ACTIONS(4078),
    [sym__outer_line_doc_comment_start] = ACTIONS(4078),
  },
  [STATE(1077)] = {
    [sym_line_comment] = STATE(1077),
    [sym_block_comment] = STATE(1077),
    [ts_builtin_sym_end] = ACTIONS(4082),
    [sym_identifier] = ACTIONS(4084),
    [anon_sym_SEMI] = ACTIONS(4082),
    [anon_sym_macro] = ACTIONS(4084),
    [anon_sym_LPAREN] = ACTIONS(4082),
    [anon_sym_LBRACE] = ACTIONS(4082),
    [anon_sym_RBRACE] = ACTIONS(4082),
    [anon_sym_macro_rules] = ACTIONS(4084),
    [anon_sym_BANG] = ACTIONS(4082),
    [anon_sym_LBRACK] = ACTIONS(4082),
    [anon_sym_STAR] = ACTIONS(4082),
    [anon_sym_u8] = ACTIONS(4084),
    [anon_sym_i8] = ACTIONS(4084),
    [anon_sym_u16] = ACTIONS(4084),
    [anon_sym_i16] = ACTIONS(4084),
    [anon_sym_u32] = ACTIONS(4084),
    [anon_sym_i32] = ACTIONS(4084),
    [anon_sym_u64] = ACTIONS(4084),
    [anon_sym_i64] = ACTIONS(4084),
    [anon_sym_u128] = ACTIONS(4084),
    [anon_sym_i128] = ACTIONS(4084),
    [anon_sym_isize] = ACTIONS(4084),
    [anon_sym_usize] = ACTIONS(4084),
    [anon_sym_f32] = ACTIONS(4084),
    [anon_sym_f64] = ACTIONS(4084),
    [anon_sym_bool] = ACTIONS(4084),
    [anon_sym_str] = ACTIONS(4084),
    [anon_sym_char] = ACTIONS(4084),
    [anon_sym_DASH] = ACTIONS(4082),
    [anon_sym_AMP] = ACTIONS(4082),
    [anon_sym_PIPE] = ACTIONS(4082),
    [anon_sym_LT] = ACTIONS(4082),
    [anon_sym_DOT_DOT] = ACTIONS(4082),
    [anon_sym_COLON_COLON] = ACTIONS(4082),
    [anon_sym_POUND] = ACTIONS(4082),
    [anon_sym_SQUOTE] = ACTIONS(4084),
    [anon_sym_async] = ACTIONS(4084),
    [anon_sym_auto] = ACTIONS(4084),
    [anon_sym_await] = ACTIONS(4084),
    [anon_sym_become] = ACTIONS(4084),
    [anon_sym_break] = ACTIONS(4084),
    [anon_sym_const] = ACTIONS(4084),
    [anon_sym_continue] = ACTIONS(4084),
    [anon_sym_default] = ACTIONS(4084),
    [anon_sym_enum] = ACTIONS(4084),
    [anon_sym_fn] = ACTIONS(4084),
    [anon_sym_for] = ACTIONS(4084),
    [anon_sym_gen] = ACTIONS(4084),
    [anon_sym_if] = ACTIONS(4084),
    [anon_sym_impl] = ACTIONS(4084),
    [anon_sym_let] = ACTIONS(4084),
    [anon_sym_loop] = ACTIONS(4084),
    [anon_sym_match] = ACTIONS(4084),
    [anon_sym_mod] = ACTIONS(4084),
    [anon_sym_pub] = ACTIONS(4084),
    [anon_sym_return] = ACTIONS(4084),
    [anon_sym_raw] = ACTIONS(4084),
    [anon_sym_static] = ACTIONS(4084),
    [anon_sym_struct] = ACTIONS(4084),
    [anon_sym_trait] = ACTIONS(4084),
    [anon_sym_type] = ACTIONS(4084),
    [anon_sym_union] = ACTIONS(4084),
    [anon_sym_unsafe] = ACTIONS(4084),
    [anon_sym_use] = ACTIONS(4084),
    [anon_sym_while] = ACTIONS(4084),
    [anon_sym_extern] = ACTIONS(4084),
    [anon_sym_safe] = ACTIONS(4084),
    [anon_sym_yield] = ACTIONS(4084),
    [anon_sym_move] = ACTIONS(4084),
    [anon_sym_try] = ACTIONS(4084),
    [sym_integer_literal] = ACTIONS(4082),
    [aux_sym_string_literal_token1] = ACTIONS(4082),
    [sym_char_literal] = ACTIONS(4082),
    [anon_sym_true] = ACTIONS(4084),
    [anon_sym_false] = ACTIONS(4084),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4084),
    [sym_super] = ACTIONS(4084),
    [sym_crate] = ACTIONS(4084),
    [aux_sym_metavariable_token1] = ACTIONS(4082),
    [sym__raw_string_literal_start] = ACTIONS(4082),
    [sym_float_literal] = ACTIONS(4082),
    [sym__outer_block_doc_comment_start] = ACTIONS(4082),
    [sym__outer_line_doc_comment_start] = ACTIONS(4082),
  },
  [STATE(1078)] = {
    [sym_line_comment] = STATE(1078),
    [sym_block_comment] = STATE(1078),
    [ts_builtin_sym_end] = ACTIONS(4086),
    [sym_identifier] = ACTIONS(4088),
    [anon_sym_SEMI] = ACTIONS(4086),
    [anon_sym_macro] = ACTIONS(4088),
    [anon_sym_LPAREN] = ACTIONS(4086),
    [anon_sym_LBRACE] = ACTIONS(4086),
    [anon_sym_RBRACE] = ACTIONS(4086),
    [anon_sym_macro_rules] = ACTIONS(4088),
    [anon_sym_BANG] = ACTIONS(4086),
    [anon_sym_LBRACK] = ACTIONS(4086),
    [anon_sym_STAR] = ACTIONS(4086),
    [anon_sym_u8] = ACTIONS(4088),
    [anon_sym_i8] = ACTIONS(4088),
    [anon_sym_u16] = ACTIONS(4088),
    [anon_sym_i16] = ACTIONS(4088),
    [anon_sym_u32] = ACTIONS(4088),
    [anon_sym_i32] = ACTIONS(4088),
    [anon_sym_u64] = ACTIONS(4088),
    [anon_sym_i64] = ACTIONS(4088),
    [anon_sym_u128] = ACTIONS(4088),
    [anon_sym_i128] = ACTIONS(4088),
    [anon_sym_isize] = ACTIONS(4088),
    [anon_sym_usize] = ACTIONS(4088),
    [anon_sym_f32] = ACTIONS(4088),
    [anon_sym_f64] = ACTIONS(4088),
    [anon_sym_bool] = ACTIONS(4088),
    [anon_sym_str] = ACTIONS(4088),
    [anon_sym_char] = ACTIONS(4088),
    [anon_sym_DASH] = ACTIONS(4086),
    [anon_sym_AMP] = ACTIONS(4086),
    [anon_sym_PIPE] = ACTIONS(4086),
    [anon_sym_LT] = ACTIONS(4086),
    [anon_sym_DOT_DOT] = ACTIONS(4086),
    [anon_sym_COLON_COLON] = ACTIONS(4086),
    [anon_sym_POUND] = ACTIONS(4086),
    [anon_sym_SQUOTE] = ACTIONS(4088),
    [anon_sym_async] = ACTIONS(4088),
    [anon_sym_auto] = ACTIONS(4088),
    [anon_sym_await] = ACTIONS(4088),
    [anon_sym_become] = ACTIONS(4088),
    [anon_sym_break] = ACTIONS(4088),
    [anon_sym_const] = ACTIONS(4088),
    [anon_sym_continue] = ACTIONS(4088),
    [anon_sym_default] = ACTIONS(4088),
    [anon_sym_enum] = ACTIONS(4088),
    [anon_sym_fn] = ACTIONS(4088),
    [anon_sym_for] = ACTIONS(4088),
    [anon_sym_gen] = ACTIONS(4088),
    [anon_sym_if] = ACTIONS(4088),
    [anon_sym_impl] = ACTIONS(4088),
    [anon_sym_let] = ACTIONS(4088),
    [anon_sym_loop] = ACTIONS(4088),
    [anon_sym_match] = ACTIONS(4088),
    [anon_sym_mod] = ACTIONS(4088),
    [anon_sym_pub] = ACTIONS(4088),
    [anon_sym_return] = ACTIONS(4088),
    [anon_sym_raw] = ACTIONS(4088),
    [anon_sym_static] = ACTIONS(4088),
    [anon_sym_struct] = ACTIONS(4088),
    [anon_sym_trait] = ACTIONS(4088),
    [anon_sym_type] = ACTIONS(4088),
    [anon_sym_union] = ACTIONS(4088),
    [anon_sym_unsafe] = ACTIONS(4088),
    [anon_sym_use] = ACTIONS(4088),
    [anon_sym_while] = ACTIONS(4088),
    [anon_sym_extern] = ACTIONS(4088),
    [anon_sym_safe] = ACTIONS(4088),
    [anon_sym_yield] = ACTIONS(4088),
    [anon_sym_move] = ACTIONS(4088),
    [anon_sym_try] = ACTIONS(4088),
    [sym_integer_literal] = ACTIONS(4086),
    [aux_sym_string_literal_token1] = ACTIONS(4086),
    [sym_char_literal] = ACTIONS(4086),
    [anon_sym_true] = ACTIONS(4088),
    [anon_sym_false] = ACTIONS(4088),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4088),
    [sym_super] = ACTIONS(4088),
    [sym_crate] = ACTIONS(4088),
    [aux_sym_metavariable_token1] = ACTIONS(4086),
    [sym__raw_string_literal_start] = ACTIONS(4086),
    [sym_float_literal] = ACTIONS(4086),
    [sym__outer_block_doc_comment_start] = ACTIONS(4086),
    [sym__outer_line_doc_comment_start] = ACTIONS(4086),
  },
  [STATE(1079)] = {
    [sym_line_comment] = STATE(1079),
    [sym_block_comment] = STATE(1079),
    [ts_builtin_sym_end] = ACTIONS(4090),
    [sym_identifier] = ACTIONS(4092),
    [anon_sym_SEMI] = ACTIONS(4090),
    [anon_sym_macro] = ACTIONS(4092),
    [anon_sym_LPAREN] = ACTIONS(4090),
    [anon_sym_LBRACE] = ACTIONS(4090),
    [anon_sym_RBRACE] = ACTIONS(4090),
    [anon_sym_macro_rules] = ACTIONS(4092),
    [anon_sym_BANG] = ACTIONS(4090),
    [anon_sym_LBRACK] = ACTIONS(4090),
    [anon_sym_STAR] = ACTIONS(4090),
    [anon_sym_u8] = ACTIONS(4092),
    [anon_sym_i8] = ACTIONS(4092),
    [anon_sym_u16] = ACTIONS(4092),
    [anon_sym_i16] = ACTIONS(4092),
    [anon_sym_u32] = ACTIONS(4092),
    [anon_sym_i32] = ACTIONS(4092),
    [anon_sym_u64] = ACTIONS(4092),
    [anon_sym_i64] = ACTIONS(4092),
    [anon_sym_u128] = ACTIONS(4092),
    [anon_sym_i128] = ACTIONS(4092),
    [anon_sym_isize] = ACTIONS(4092),
    [anon_sym_usize] = ACTIONS(4092),
    [anon_sym_f32] = ACTIONS(4092),
    [anon_sym_f64] = ACTIONS(4092),
    [anon_sym_bool] = ACTIONS(4092),
    [anon_sym_str] = ACTIONS(4092),
    [anon_sym_char] = ACTIONS(4092),
    [anon_sym_DASH] = ACTIONS(4090),
    [anon_sym_AMP] = ACTIONS(4090),
    [anon_sym_PIPE] = ACTIONS(4090),
    [anon_sym_LT] = ACTIONS(4090),
    [anon_sym_DOT_DOT] = ACTIONS(4090),
    [anon_sym_COLON_COLON] = ACTIONS(4090),
    [anon_sym_POUND] = ACTIONS(4090),
    [anon_sym_SQUOTE] = ACTIONS(4092),
    [anon_sym_async] = ACTIONS(4092),
    [anon_sym_auto] = ACTIONS(4092),
    [anon_sym_await] = ACTIONS(4092),
    [anon_sym_become] = ACTIONS(4092),
    [anon_sym_break] = ACTIONS(4092),
    [anon_sym_const] = ACTIONS(4092),
    [anon_sym_continue] = ACTIONS(4092),
    [anon_sym_default] = ACTIONS(4092),
    [anon_sym_enum] = ACTIONS(4092),
    [anon_sym_fn] = ACTIONS(4092),
    [anon_sym_for] = ACTIONS(4092),
    [anon_sym_gen] = ACTIONS(4092),
    [anon_sym_if] = ACTIONS(4092),
    [anon_sym_impl] = ACTIONS(4092),
    [anon_sym_let] = ACTIONS(4092),
    [anon_sym_loop] = ACTIONS(4092),
    [anon_sym_match] = ACTIONS(4092),
    [anon_sym_mod] = ACTIONS(4092),
    [anon_sym_pub] = ACTIONS(4092),
    [anon_sym_return] = ACTIONS(4092),
    [anon_sym_raw] = ACTIONS(4092),
    [anon_sym_static] = ACTIONS(4092),
    [anon_sym_struct] = ACTIONS(4092),
    [anon_sym_trait] = ACTIONS(4092),
    [anon_sym_type] = ACTIONS(4092),
    [anon_sym_union] = ACTIONS(4092),
    [anon_sym_unsafe] = ACTIONS(4092),
    [anon_sym_use] = ACTIONS(4092),
    [anon_sym_while] = ACTIONS(4092),
    [anon_sym_extern] = ACTIONS(4092),
    [anon_sym_safe] = ACTIONS(4092),
    [anon_sym_yield] = ACTIONS(4092),
    [anon_sym_move] = ACTIONS(4092),
    [anon_sym_try] = ACTIONS(4092),
    [sym_integer_literal] = ACTIONS(4090),
    [aux_sym_string_literal_token1] = ACTIONS(4090),
    [sym_char_literal] = ACTIONS(4090),
    [anon_sym_true] = ACTIONS(4092),
    [anon_sym_false] = ACTIONS(4092),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4092),
    [sym_super] = ACTIONS(4092),
    [sym_crate] = ACTIONS(4092),
    [aux_sym_metavariable_token1] = ACTIONS(4090),
    [sym__raw_string_literal_start] = ACTIONS(4090),
    [sym_float_literal] = ACTIONS(4090),
    [sym__outer_block_doc_comment_start] = ACTIONS(4090),
    [sym__outer_line_doc_comment_start] = ACTIONS(4090),
  },
  [STATE(1080)] = {
    [sym_line_comment] = STATE(1080),
    [sym_block_comment] = STATE(1080),
    [ts_builtin_sym_end] = ACTIONS(4094),
    [sym_identifier] = ACTIONS(4096),
    [anon_sym_SEMI] = ACTIONS(4094),
    [anon_sym_macro] = ACTIONS(4096),
    [anon_sym_LPAREN] = ACTIONS(4094),
    [anon_sym_LBRACE] = ACTIONS(4094),
    [anon_sym_RBRACE] = ACTIONS(4094),
    [anon_sym_macro_rules] = ACTIONS(4096),
    [anon_sym_BANG] = ACTIONS(4094),
    [anon_sym_LBRACK] = ACTIONS(4094),
    [anon_sym_STAR] = ACTIONS(4094),
    [anon_sym_u8] = ACTIONS(4096),
    [anon_sym_i8] = ACTIONS(4096),
    [anon_sym_u16] = ACTIONS(4096),
    [anon_sym_i16] = ACTIONS(4096),
    [anon_sym_u32] = ACTIONS(4096),
    [anon_sym_i32] = ACTIONS(4096),
    [anon_sym_u64] = ACTIONS(4096),
    [anon_sym_i64] = ACTIONS(4096),
    [anon_sym_u128] = ACTIONS(4096),
    [anon_sym_i128] = ACTIONS(4096),
    [anon_sym_isize] = ACTIONS(4096),
    [anon_sym_usize] = ACTIONS(4096),
    [anon_sym_f32] = ACTIONS(4096),
    [anon_sym_f64] = ACTIONS(4096),
    [anon_sym_bool] = ACTIONS(4096),
    [anon_sym_str] = ACTIONS(4096),
    [anon_sym_char] = ACTIONS(4096),
    [anon_sym_DASH] = ACTIONS(4094),
    [anon_sym_AMP] = ACTIONS(4094),
    [anon_sym_PIPE] = ACTIONS(4094),
    [anon_sym_LT] = ACTIONS(4094),
    [anon_sym_DOT_DOT] = ACTIONS(4094),
    [anon_sym_COLON_COLON] = ACTIONS(4094),
    [anon_sym_POUND] = ACTIONS(4094),
    [anon_sym_SQUOTE] = ACTIONS(4096),
    [anon_sym_async] = ACTIONS(4096),
    [anon_sym_auto] = ACTIONS(4096),
    [anon_sym_await] = ACTIONS(4096),
    [anon_sym_become] = ACTIONS(4096),
    [anon_sym_break] = ACTIONS(4096),
    [anon_sym_const] = ACTIONS(4096),
    [anon_sym_continue] = ACTIONS(4096),
    [anon_sym_default] = ACTIONS(4096),
    [anon_sym_enum] = ACTIONS(4096),
    [anon_sym_fn] = ACTIONS(4096),
    [anon_sym_for] = ACTIONS(4096),
    [anon_sym_gen] = ACTIONS(4096),
    [anon_sym_if] = ACTIONS(4096),
    [anon_sym_impl] = ACTIONS(4096),
    [anon_sym_let] = ACTIONS(4096),
    [anon_sym_loop] = ACTIONS(4096),
    [anon_sym_match] = ACTIONS(4096),
    [anon_sym_mod] = ACTIONS(4096),
    [anon_sym_pub] = ACTIONS(4096),
    [anon_sym_return] = ACTIONS(4096),
    [anon_sym_raw] = ACTIONS(4096),
    [anon_sym_static] = ACTIONS(4096),
    [anon_sym_struct] = ACTIONS(4096),
    [anon_sym_trait] = ACTIONS(4096),
    [anon_sym_type] = ACTIONS(4096),
    [anon_sym_union] = ACTIONS(4096),
    [anon_sym_unsafe] = ACTIONS(4096),
    [anon_sym_use] = ACTIONS(4096),
    [anon_sym_while] = ACTIONS(4096),
    [anon_sym_extern] = ACTIONS(4096),
    [anon_sym_safe] = ACTIONS(4096),
    [anon_sym_yield] = ACTIONS(4096),
    [anon_sym_move] = ACTIONS(4096),
    [anon_sym_try] = ACTIONS(4096),
    [sym_integer_literal] = ACTIONS(4094),
    [aux_sym_string_literal_token1] = ACTIONS(4094),
    [sym_char_literal] = ACTIONS(4094),
    [anon_sym_true] = ACTIONS(4096),
    [anon_sym_false] = ACTIONS(4096),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4096),
    [sym_super] = ACTIONS(4096),
    [sym_crate] = ACTIONS(4096),
    [aux_sym_metavariable_token1] = ACTIONS(4094),
    [sym__raw_string_literal_start] = ACTIONS(4094),
    [sym_float_literal] = ACTIONS(4094),
    [sym__outer_block_doc_comment_start] = ACTIONS(4094),
    [sym__outer_line_doc_comment_start] = ACTIONS(4094),
  },
  [STATE(1081)] = {
    [sym_line_comment] = STATE(1081),
    [sym_block_comment] = STATE(1081),
    [ts_builtin_sym_end] = ACTIONS(4098),
    [sym_identifier] = ACTIONS(4100),
    [anon_sym_SEMI] = ACTIONS(4098),
    [anon_sym_macro] = ACTIONS(4100),
    [anon_sym_LPAREN] = ACTIONS(4098),
    [anon_sym_LBRACE] = ACTIONS(4098),
    [anon_sym_RBRACE] = ACTIONS(4098),
    [anon_sym_macro_rules] = ACTIONS(4100),
    [anon_sym_BANG] = ACTIONS(4098),
    [anon_sym_LBRACK] = ACTIONS(4098),
    [anon_sym_STAR] = ACTIONS(4098),
    [anon_sym_u8] = ACTIONS(4100),
    [anon_sym_i8] = ACTIONS(4100),
    [anon_sym_u16] = ACTIONS(4100),
    [anon_sym_i16] = ACTIONS(4100),
    [anon_sym_u32] = ACTIONS(4100),
    [anon_sym_i32] = ACTIONS(4100),
    [anon_sym_u64] = ACTIONS(4100),
    [anon_sym_i64] = ACTIONS(4100),
    [anon_sym_u128] = ACTIONS(4100),
    [anon_sym_i128] = ACTIONS(4100),
    [anon_sym_isize] = ACTIONS(4100),
    [anon_sym_usize] = ACTIONS(4100),
    [anon_sym_f32] = ACTIONS(4100),
    [anon_sym_f64] = ACTIONS(4100),
    [anon_sym_bool] = ACTIONS(4100),
    [anon_sym_str] = ACTIONS(4100),
    [anon_sym_char] = ACTIONS(4100),
    [anon_sym_DASH] = ACTIONS(4098),
    [anon_sym_AMP] = ACTIONS(4098),
    [anon_sym_PIPE] = ACTIONS(4098),
    [anon_sym_LT] = ACTIONS(4098),
    [anon_sym_DOT_DOT] = ACTIONS(4098),
    [anon_sym_COLON_COLON] = ACTIONS(4098),
    [anon_sym_POUND] = ACTIONS(4098),
    [anon_sym_SQUOTE] = ACTIONS(4100),
    [anon_sym_async] = ACTIONS(4100),
    [anon_sym_auto] = ACTIONS(4100),
    [anon_sym_await] = ACTIONS(4100),
    [anon_sym_become] = ACTIONS(4100),
    [anon_sym_break] = ACTIONS(4100),
    [anon_sym_const] = ACTIONS(4100),
    [anon_sym_continue] = ACTIONS(4100),
    [anon_sym_default] = ACTIONS(4100),
    [anon_sym_enum] = ACTIONS(4100),
    [anon_sym_fn] = ACTIONS(4100),
    [anon_sym_for] = ACTIONS(4100),
    [anon_sym_gen] = ACTIONS(4100),
    [anon_sym_if] = ACTIONS(4100),
    [anon_sym_impl] = ACTIONS(4100),
    [anon_sym_let] = ACTIONS(4100),
    [anon_sym_loop] = ACTIONS(4100),
    [anon_sym_match] = ACTIONS(4100),
    [anon_sym_mod] = ACTIONS(4100),
    [anon_sym_pub] = ACTIONS(4100),
    [anon_sym_return] = ACTIONS(4100),
    [anon_sym_raw] = ACTIONS(4100),
    [anon_sym_static] = ACTIONS(4100),
    [anon_sym_struct] = ACTIONS(4100),
    [anon_sym_trait] = ACTIONS(4100),
    [anon_sym_type] = ACTIONS(4100),
    [anon_sym_union] = ACTIONS(4100),
    [anon_sym_unsafe] = ACTIONS(4100),
    [anon_sym_use] = ACTIONS(4100),
    [anon_sym_while] = ACTIONS(4100),
    [anon_sym_extern] = ACTIONS(4100),
    [anon_sym_safe] = ACTIONS(4100),
    [anon_sym_yield] = ACTIONS(4100),
    [anon_sym_move] = ACTIONS(4100),
    [anon_sym_try] = ACTIONS(4100),
    [sym_integer_literal] = ACTIONS(4098),
    [aux_sym_string_literal_token1] = ACTIONS(4098),
    [sym_char_literal] = ACTIONS(4098),
    [anon_sym_true] = ACTIONS(4100),
    [anon_sym_false] = ACTIONS(4100),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4100),
    [sym_super] = ACTIONS(4100),
    [sym_crate] = ACTIONS(4100),
    [aux_sym_metavariable_token1] = ACTIONS(4098),
    [sym__raw_string_literal_start] = ACTIONS(4098),
    [sym_float_literal] = ACTIONS(4098),
    [sym__outer_block_doc_comment_start] = ACTIONS(4098),
    [sym__outer_line_doc_comment_start] = ACTIONS(4098),
  },
  [STATE(1082)] = {
    [sym_line_comment] = STATE(1082),
    [sym_block_comment] = STATE(1082),
    [ts_builtin_sym_end] = ACTIONS(4102),
    [sym_identifier] = ACTIONS(4104),
    [anon_sym_SEMI] = ACTIONS(4102),
    [anon_sym_macro] = ACTIONS(4104),
    [anon_sym_LPAREN] = ACTIONS(4102),
    [anon_sym_LBRACE] = ACTIONS(4102),
    [anon_sym_RBRACE] = ACTIONS(4102),
    [anon_sym_macro_rules] = ACTIONS(4104),
    [anon_sym_BANG] = ACTIONS(4102),
    [anon_sym_LBRACK] = ACTIONS(4102),
    [anon_sym_STAR] = ACTIONS(4102),
    [anon_sym_u8] = ACTIONS(4104),
    [anon_sym_i8] = ACTIONS(4104),
    [anon_sym_u16] = ACTIONS(4104),
    [anon_sym_i16] = ACTIONS(4104),
    [anon_sym_u32] = ACTIONS(4104),
    [anon_sym_i32] = ACTIONS(4104),
    [anon_sym_u64] = ACTIONS(4104),
    [anon_sym_i64] = ACTIONS(4104),
    [anon_sym_u128] = ACTIONS(4104),
    [anon_sym_i128] = ACTIONS(4104),
    [anon_sym_isize] = ACTIONS(4104),
    [anon_sym_usize] = ACTIONS(4104),
    [anon_sym_f32] = ACTIONS(4104),
    [anon_sym_f64] = ACTIONS(4104),
    [anon_sym_bool] = ACTIONS(4104),
    [anon_sym_str] = ACTIONS(4104),
    [anon_sym_char] = ACTIONS(4104),
    [anon_sym_DASH] = ACTIONS(4102),
    [anon_sym_AMP] = ACTIONS(4102),
    [anon_sym_PIPE] = ACTIONS(4102),
    [anon_sym_LT] = ACTIONS(4102),
    [anon_sym_DOT_DOT] = ACTIONS(4102),
    [anon_sym_COLON_COLON] = ACTIONS(4102),
    [anon_sym_POUND] = ACTIONS(4102),
    [anon_sym_SQUOTE] = ACTIONS(4104),
    [anon_sym_async] = ACTIONS(4104),
    [anon_sym_auto] = ACTIONS(4104),
    [anon_sym_await] = ACTIONS(4104),
    [anon_sym_become] = ACTIONS(4104),
    [anon_sym_break] = ACTIONS(4104),
    [anon_sym_const] = ACTIONS(4104),
    [anon_sym_continue] = ACTIONS(4104),
    [anon_sym_default] = ACTIONS(4104),
    [anon_sym_enum] = ACTIONS(4104),
    [anon_sym_fn] = ACTIONS(4104),
    [anon_sym_for] = ACTIONS(4104),
    [anon_sym_gen] = ACTIONS(4104),
    [anon_sym_if] = ACTIONS(4104),
    [anon_sym_impl] = ACTIONS(4104),
    [anon_sym_let] = ACTIONS(4104),
    [anon_sym_loop] = ACTIONS(4104),
    [anon_sym_match] = ACTIONS(4104),
    [anon_sym_mod] = ACTIONS(4104),
    [anon_sym_pub] = ACTIONS(4104),
    [anon_sym_return] = ACTIONS(4104),
    [anon_sym_raw] = ACTIONS(4104),
    [anon_sym_static] = ACTIONS(4104),
    [anon_sym_struct] = ACTIONS(4104),
    [anon_sym_trait] = ACTIONS(4104),
    [anon_sym_type] = ACTIONS(4104),
    [anon_sym_union] = ACTIONS(4104),
    [anon_sym_unsafe] = ACTIONS(4104),
    [anon_sym_use] = ACTIONS(4104),
    [anon_sym_while] = ACTIONS(4104),
    [anon_sym_extern] = ACTIONS(4104),
    [anon_sym_safe] = ACTIONS(4104),
    [anon_sym_yield] = ACTIONS(4104),
    [anon_sym_move] = ACTIONS(4104),
    [anon_sym_try] = ACTIONS(4104),
    [sym_integer_literal] = ACTIONS(4102),
    [aux_sym_string_literal_token1] = ACTIONS(4102),
    [sym_char_literal] = ACTIONS(4102),
    [anon_sym_true] = ACTIONS(4104),
    [anon_sym_false] = ACTIONS(4104),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4104),
    [sym_super] = ACTIONS(4104),
    [sym_crate] = ACTIONS(4104),
    [aux_sym_metavariable_token1] = ACTIONS(4102),
    [sym__raw_string_literal_start] = ACTIONS(4102),
    [sym_float_literal] = ACTIONS(4102),
    [sym__outer_block_doc_comment_start] = ACTIONS(4102),
    [sym__outer_line_doc_comment_start] = ACTIONS(4102),
  },
  [STATE(1083)] = {
    [sym_line_comment] = STATE(1083),
    [sym_block_comment] = STATE(1083),
    [ts_builtin_sym_end] = ACTIONS(4106),
    [sym_identifier] = ACTIONS(4108),
    [anon_sym_SEMI] = ACTIONS(4106),
    [anon_sym_macro] = ACTIONS(4108),
    [anon_sym_LPAREN] = ACTIONS(4106),
    [anon_sym_LBRACE] = ACTIONS(4106),
    [anon_sym_RBRACE] = ACTIONS(4106),
    [anon_sym_macro_rules] = ACTIONS(4108),
    [anon_sym_BANG] = ACTIONS(4106),
    [anon_sym_LBRACK] = ACTIONS(4106),
    [anon_sym_STAR] = ACTIONS(4106),
    [anon_sym_u8] = ACTIONS(4108),
    [anon_sym_i8] = ACTIONS(4108),
    [anon_sym_u16] = ACTIONS(4108),
    [anon_sym_i16] = ACTIONS(4108),
    [anon_sym_u32] = ACTIONS(4108),
    [anon_sym_i32] = ACTIONS(4108),
    [anon_sym_u64] = ACTIONS(4108),
    [anon_sym_i64] = ACTIONS(4108),
    [anon_sym_u128] = ACTIONS(4108),
    [anon_sym_i128] = ACTIONS(4108),
    [anon_sym_isize] = ACTIONS(4108),
    [anon_sym_usize] = ACTIONS(4108),
    [anon_sym_f32] = ACTIONS(4108),
    [anon_sym_f64] = ACTIONS(4108),
    [anon_sym_bool] = ACTIONS(4108),
    [anon_sym_str] = ACTIONS(4108),
    [anon_sym_char] = ACTIONS(4108),
    [anon_sym_DASH] = ACTIONS(4106),
    [anon_sym_AMP] = ACTIONS(4106),
    [anon_sym_PIPE] = ACTIONS(4106),
    [anon_sym_LT] = ACTIONS(4106),
    [anon_sym_DOT_DOT] = ACTIONS(4106),
    [anon_sym_COLON_COLON] = ACTIONS(4106),
    [anon_sym_POUND] = ACTIONS(4106),
    [anon_sym_SQUOTE] = ACTIONS(4108),
    [anon_sym_async] = ACTIONS(4108),
    [anon_sym_auto] = ACTIONS(4108),
    [anon_sym_await] = ACTIONS(4108),
    [anon_sym_become] = ACTIONS(4108),
    [anon_sym_break] = ACTIONS(4108),
    [anon_sym_const] = ACTIONS(4108),
    [anon_sym_continue] = ACTIONS(4108),
    [anon_sym_default] = ACTIONS(4108),
    [anon_sym_enum] = ACTIONS(4108),
    [anon_sym_fn] = ACTIONS(4108),
    [anon_sym_for] = ACTIONS(4108),
    [anon_sym_gen] = ACTIONS(4108),
    [anon_sym_if] = ACTIONS(4108),
    [anon_sym_impl] = ACTIONS(4108),
    [anon_sym_let] = ACTIONS(4108),
    [anon_sym_loop] = ACTIONS(4108),
    [anon_sym_match] = ACTIONS(4108),
    [anon_sym_mod] = ACTIONS(4108),
    [anon_sym_pub] = ACTIONS(4108),
    [anon_sym_return] = ACTIONS(4108),
    [anon_sym_raw] = ACTIONS(4108),
    [anon_sym_static] = ACTIONS(4108),
    [anon_sym_struct] = ACTIONS(4108),
    [anon_sym_trait] = ACTIONS(4108),
    [anon_sym_type] = ACTIONS(4108),
    [anon_sym_union] = ACTIONS(4108),
    [anon_sym_unsafe] = ACTIONS(4108),
    [anon_sym_use] = ACTIONS(4108),
    [anon_sym_while] = ACTIONS(4108),
    [anon_sym_extern] = ACTIONS(4108),
    [anon_sym_safe] = ACTIONS(4108),
    [anon_sym_yield] = ACTIONS(4108),
    [anon_sym_move] = ACTIONS(4108),
    [anon_sym_try] = ACTIONS(4108),
    [sym_integer_literal] = ACTIONS(4106),
    [aux_sym_string_literal_token1] = ACTIONS(4106),
    [sym_char_literal] = ACTIONS(4106),
    [anon_sym_true] = ACTIONS(4108),
    [anon_sym_false] = ACTIONS(4108),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4108),
    [sym_super] = ACTIONS(4108),
    [sym_crate] = ACTIONS(4108),
    [aux_sym_metavariable_token1] = ACTIONS(4106),
    [sym__raw_string_literal_start] = ACTIONS(4106),
    [sym_float_literal] = ACTIONS(4106),
    [sym__outer_block_doc_comment_start] = ACTIONS(4106),
    [sym__outer_line_doc_comment_start] = ACTIONS(4106),
  },
  [STATE(1084)] = {
    [sym_line_comment] = STATE(1084),
    [sym_block_comment] = STATE(1084),
    [ts_builtin_sym_end] = ACTIONS(4110),
    [sym_identifier] = ACTIONS(4112),
    [anon_sym_SEMI] = ACTIONS(4110),
    [anon_sym_macro] = ACTIONS(4112),
    [anon_sym_LPAREN] = ACTIONS(4110),
    [anon_sym_LBRACE] = ACTIONS(4110),
    [anon_sym_RBRACE] = ACTIONS(4110),
    [anon_sym_macro_rules] = ACTIONS(4112),
    [anon_sym_BANG] = ACTIONS(4110),
    [anon_sym_LBRACK] = ACTIONS(4110),
    [anon_sym_STAR] = ACTIONS(4110),
    [anon_sym_u8] = ACTIONS(4112),
    [anon_sym_i8] = ACTIONS(4112),
    [anon_sym_u16] = ACTIONS(4112),
    [anon_sym_i16] = ACTIONS(4112),
    [anon_sym_u32] = ACTIONS(4112),
    [anon_sym_i32] = ACTIONS(4112),
    [anon_sym_u64] = ACTIONS(4112),
    [anon_sym_i64] = ACTIONS(4112),
    [anon_sym_u128] = ACTIONS(4112),
    [anon_sym_i128] = ACTIONS(4112),
    [anon_sym_isize] = ACTIONS(4112),
    [anon_sym_usize] = ACTIONS(4112),
    [anon_sym_f32] = ACTIONS(4112),
    [anon_sym_f64] = ACTIONS(4112),
    [anon_sym_bool] = ACTIONS(4112),
    [anon_sym_str] = ACTIONS(4112),
    [anon_sym_char] = ACTIONS(4112),
    [anon_sym_DASH] = ACTIONS(4110),
    [anon_sym_AMP] = ACTIONS(4110),
    [anon_sym_PIPE] = ACTIONS(4110),
    [anon_sym_LT] = ACTIONS(4110),
    [anon_sym_DOT_DOT] = ACTIONS(4110),
    [anon_sym_COLON_COLON] = ACTIONS(4110),
    [anon_sym_POUND] = ACTIONS(4110),
    [anon_sym_SQUOTE] = ACTIONS(4112),
    [anon_sym_async] = ACTIONS(4112),
    [anon_sym_auto] = ACTIONS(4112),
    [anon_sym_await] = ACTIONS(4112),
    [anon_sym_become] = ACTIONS(4112),
    [anon_sym_break] = ACTIONS(4112),
    [anon_sym_const] = ACTIONS(4112),
    [anon_sym_continue] = ACTIONS(4112),
    [anon_sym_default] = ACTIONS(4112),
    [anon_sym_enum] = ACTIONS(4112),
    [anon_sym_fn] = ACTIONS(4112),
    [anon_sym_for] = ACTIONS(4112),
    [anon_sym_gen] = ACTIONS(4112),
    [anon_sym_if] = ACTIONS(4112),
    [anon_sym_impl] = ACTIONS(4112),
    [anon_sym_let] = ACTIONS(4112),
    [anon_sym_loop] = ACTIONS(4112),
    [anon_sym_match] = ACTIONS(4112),
    [anon_sym_mod] = ACTIONS(4112),
    [anon_sym_pub] = ACTIONS(4112),
    [anon_sym_return] = ACTIONS(4112),
    [anon_sym_raw] = ACTIONS(4112),
    [anon_sym_static] = ACTIONS(4112),
    [anon_sym_struct] = ACTIONS(4112),
    [anon_sym_trait] = ACTIONS(4112),
    [anon_sym_type] = ACTIONS(4112),
    [anon_sym_union] = ACTIONS(4112),
    [anon_sym_unsafe] = ACTIONS(4112),
    [anon_sym_use] = ACTIONS(4112),
    [anon_sym_while] = ACTIONS(4112),
    [anon_sym_extern] = ACTIONS(4112),
    [anon_sym_safe] = ACTIONS(4112),
    [anon_sym_yield] = ACTIONS(4112),
    [anon_sym_move] = ACTIONS(4112),
    [anon_sym_try] = ACTIONS(4112),
    [sym_integer_literal] = ACTIONS(4110),
    [aux_sym_string_literal_token1] = ACTIONS(4110),
    [sym_char_literal] = ACTIONS(4110),
    [anon_sym_true] = ACTIONS(4112),
    [anon_sym_false] = ACTIONS(4112),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4112),
    [sym_super] = ACTIONS(4112),
    [sym_crate] = ACTIONS(4112),
    [aux_sym_metavariable_token1] = ACTIONS(4110),
    [sym__raw_string_literal_start] = ACTIONS(4110),
    [sym_float_literal] = ACTIONS(4110),
    [sym__outer_block_doc_comment_start] = ACTIONS(4110),
    [sym__outer_line_doc_comment_start] = ACTIONS(4110),
  },
  [STATE(1085)] = {
    [sym_line_comment] = STATE(1085),
    [sym_block_comment] = STATE(1085),
    [ts_builtin_sym_end] = ACTIONS(4114),
    [sym_identifier] = ACTIONS(4116),
    [anon_sym_SEMI] = ACTIONS(4114),
    [anon_sym_macro] = ACTIONS(4116),
    [anon_sym_LPAREN] = ACTIONS(4114),
    [anon_sym_LBRACE] = ACTIONS(4114),
    [anon_sym_RBRACE] = ACTIONS(4114),
    [anon_sym_macro_rules] = ACTIONS(4116),
    [anon_sym_BANG] = ACTIONS(4114),
    [anon_sym_LBRACK] = ACTIONS(4114),
    [anon_sym_STAR] = ACTIONS(4114),
    [anon_sym_u8] = ACTIONS(4116),
    [anon_sym_i8] = ACTIONS(4116),
    [anon_sym_u16] = ACTIONS(4116),
    [anon_sym_i16] = ACTIONS(4116),
    [anon_sym_u32] = ACTIONS(4116),
    [anon_sym_i32] = ACTIONS(4116),
    [anon_sym_u64] = ACTIONS(4116),
    [anon_sym_i64] = ACTIONS(4116),
    [anon_sym_u128] = ACTIONS(4116),
    [anon_sym_i128] = ACTIONS(4116),
    [anon_sym_isize] = ACTIONS(4116),
    [anon_sym_usize] = ACTIONS(4116),
    [anon_sym_f32] = ACTIONS(4116),
    [anon_sym_f64] = ACTIONS(4116),
    [anon_sym_bool] = ACTIONS(4116),
    [anon_sym_str] = ACTIONS(4116),
    [anon_sym_char] = ACTIONS(4116),
    [anon_sym_DASH] = ACTIONS(4114),
    [anon_sym_AMP] = ACTIONS(4114),
    [anon_sym_PIPE] = ACTIONS(4114),
    [anon_sym_LT] = ACTIONS(4114),
    [anon_sym_DOT_DOT] = ACTIONS(4114),
    [anon_sym_COLON_COLON] = ACTIONS(4114),
    [anon_sym_POUND] = ACTIONS(4114),
    [anon_sym_SQUOTE] = ACTIONS(4116),
    [anon_sym_async] = ACTIONS(4116),
    [anon_sym_auto] = ACTIONS(4116),
    [anon_sym_await] = ACTIONS(4116),
    [anon_sym_become] = ACTIONS(4116),
    [anon_sym_break] = ACTIONS(4116),
    [anon_sym_const] = ACTIONS(4116),
    [anon_sym_continue] = ACTIONS(4116),
    [anon_sym_default] = ACTIONS(4116),
    [anon_sym_enum] = ACTIONS(4116),
    [anon_sym_fn] = ACTIONS(4116),
    [anon_sym_for] = ACTIONS(4116),
    [anon_sym_gen] = ACTIONS(4116),
    [anon_sym_if] = ACTIONS(4116),
    [anon_sym_impl] = ACTIONS(4116),
    [anon_sym_let] = ACTIONS(4116),
    [anon_sym_loop] = ACTIONS(4116),
    [anon_sym_match] = ACTIONS(4116),
    [anon_sym_mod] = ACTIONS(4116),
    [anon_sym_pub] = ACTIONS(4116),
    [anon_sym_return] = ACTIONS(4116),
    [anon_sym_raw] = ACTIONS(4116),
    [anon_sym_static] = ACTIONS(4116),
    [anon_sym_struct] = ACTIONS(4116),
    [anon_sym_trait] = ACTIONS(4116),
    [anon_sym_type] = ACTIONS(4116),
    [anon_sym_union] = ACTIONS(4116),
    [anon_sym_unsafe] = ACTIONS(4116),
    [anon_sym_use] = ACTIONS(4116),
    [anon_sym_while] = ACTIONS(4116),
    [anon_sym_extern] = ACTIONS(4116),
    [anon_sym_safe] = ACTIONS(4116),
    [anon_sym_yield] = ACTIONS(4116),
    [anon_sym_move] = ACTIONS(4116),
    [anon_sym_try] = ACTIONS(4116),
    [sym_integer_literal] = ACTIONS(4114),
    [aux_sym_string_literal_token1] = ACTIONS(4114),
    [sym_char_literal] = ACTIONS(4114),
    [anon_sym_true] = ACTIONS(4116),
    [anon_sym_false] = ACTIONS(4116),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4116),
    [sym_super] = ACTIONS(4116),
    [sym_crate] = ACTIONS(4116),
    [aux_sym_metavariable_token1] = ACTIONS(4114),
    [sym__raw_string_literal_start] = ACTIONS(4114),
    [sym_float_literal] = ACTIONS(4114),
    [sym__outer_block_doc_comment_start] = ACTIONS(4114),
    [sym__outer_line_doc_comment_start] = ACTIONS(4114),
  },
  [STATE(1086)] = {
    [sym_line_comment] = STATE(1086),
    [sym_block_comment] = STATE(1086),
    [ts_builtin_sym_end] = ACTIONS(4118),
    [sym_identifier] = ACTIONS(4120),
    [anon_sym_SEMI] = ACTIONS(4118),
    [anon_sym_macro] = ACTIONS(4120),
    [anon_sym_LPAREN] = ACTIONS(4118),
    [anon_sym_LBRACE] = ACTIONS(4118),
    [anon_sym_RBRACE] = ACTIONS(4118),
    [anon_sym_macro_rules] = ACTIONS(4120),
    [anon_sym_BANG] = ACTIONS(4118),
    [anon_sym_LBRACK] = ACTIONS(4118),
    [anon_sym_STAR] = ACTIONS(4118),
    [anon_sym_u8] = ACTIONS(4120),
    [anon_sym_i8] = ACTIONS(4120),
    [anon_sym_u16] = ACTIONS(4120),
    [anon_sym_i16] = ACTIONS(4120),
    [anon_sym_u32] = ACTIONS(4120),
    [anon_sym_i32] = ACTIONS(4120),
    [anon_sym_u64] = ACTIONS(4120),
    [anon_sym_i64] = ACTIONS(4120),
    [anon_sym_u128] = ACTIONS(4120),
    [anon_sym_i128] = ACTIONS(4120),
    [anon_sym_isize] = ACTIONS(4120),
    [anon_sym_usize] = ACTIONS(4120),
    [anon_sym_f32] = ACTIONS(4120),
    [anon_sym_f64] = ACTIONS(4120),
    [anon_sym_bool] = ACTIONS(4120),
    [anon_sym_str] = ACTIONS(4120),
    [anon_sym_char] = ACTIONS(4120),
    [anon_sym_DASH] = ACTIONS(4118),
    [anon_sym_AMP] = ACTIONS(4118),
    [anon_sym_PIPE] = ACTIONS(4118),
    [anon_sym_LT] = ACTIONS(4118),
    [anon_sym_DOT_DOT] = ACTIONS(4118),
    [anon_sym_COLON_COLON] = ACTIONS(4118),
    [anon_sym_POUND] = ACTIONS(4118),
    [anon_sym_SQUOTE] = ACTIONS(4120),
    [anon_sym_async] = ACTIONS(4120),
    [anon_sym_auto] = ACTIONS(4120),
    [anon_sym_await] = ACTIONS(4120),
    [anon_sym_become] = ACTIONS(4120),
    [anon_sym_break] = ACTIONS(4120),
    [anon_sym_const] = ACTIONS(4120),
    [anon_sym_continue] = ACTIONS(4120),
    [anon_sym_default] = ACTIONS(4120),
    [anon_sym_enum] = ACTIONS(4120),
    [anon_sym_fn] = ACTIONS(4120),
    [anon_sym_for] = ACTIONS(4120),
    [anon_sym_gen] = ACTIONS(4120),
    [anon_sym_if] = ACTIONS(4120),
    [anon_sym_impl] = ACTIONS(4120),
    [anon_sym_let] = ACTIONS(4120),
    [anon_sym_loop] = ACTIONS(4120),
    [anon_sym_match] = ACTIONS(4120),
    [anon_sym_mod] = ACTIONS(4120),
    [anon_sym_pub] = ACTIONS(4120),
    [anon_sym_return] = ACTIONS(4120),
    [anon_sym_raw] = ACTIONS(4120),
    [anon_sym_static] = ACTIONS(4120),
    [anon_sym_struct] = ACTIONS(4120),
    [anon_sym_trait] = ACTIONS(4120),
    [anon_sym_type] = ACTIONS(4120),
    [anon_sym_union] = ACTIONS(4120),
    [anon_sym_unsafe] = ACTIONS(4120),
    [anon_sym_use] = ACTIONS(4120),
    [anon_sym_while] = ACTIONS(4120),
    [anon_sym_extern] = ACTIONS(4120),
    [anon_sym_safe] = ACTIONS(4120),
    [anon_sym_yield] = ACTIONS(4120),
    [anon_sym_move] = ACTIONS(4120),
    [anon_sym_try] = ACTIONS(4120),
    [sym_integer_literal] = ACTIONS(4118),
    [aux_sym_string_literal_token1] = ACTIONS(4118),
    [sym_char_literal] = ACTIONS(4118),
    [anon_sym_true] = ACTIONS(4120),
    [anon_sym_false] = ACTIONS(4120),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4120),
    [sym_super] = ACTIONS(4120),
    [sym_crate] = ACTIONS(4120),
    [aux_sym_metavariable_token1] = ACTIONS(4118),
    [sym__raw_string_literal_start] = ACTIONS(4118),
    [sym_float_literal] = ACTIONS(4118),
    [sym__outer_block_doc_comment_start] = ACTIONS(4118),
    [sym__outer_line_doc_comment_start] = ACTIONS(4118),
  },
  [STATE(1087)] = {
    [sym_line_comment] = STATE(1087),
    [sym_block_comment] = STATE(1087),
    [ts_builtin_sym_end] = ACTIONS(4122),
    [sym_identifier] = ACTIONS(4124),
    [anon_sym_SEMI] = ACTIONS(4122),
    [anon_sym_macro] = ACTIONS(4124),
    [anon_sym_LPAREN] = ACTIONS(4122),
    [anon_sym_LBRACE] = ACTIONS(4122),
    [anon_sym_RBRACE] = ACTIONS(4122),
    [anon_sym_macro_rules] = ACTIONS(4124),
    [anon_sym_BANG] = ACTIONS(4122),
    [anon_sym_LBRACK] = ACTIONS(4122),
    [anon_sym_STAR] = ACTIONS(4122),
    [anon_sym_u8] = ACTIONS(4124),
    [anon_sym_i8] = ACTIONS(4124),
    [anon_sym_u16] = ACTIONS(4124),
    [anon_sym_i16] = ACTIONS(4124),
    [anon_sym_u32] = ACTIONS(4124),
    [anon_sym_i32] = ACTIONS(4124),
    [anon_sym_u64] = ACTIONS(4124),
    [anon_sym_i64] = ACTIONS(4124),
    [anon_sym_u128] = ACTIONS(4124),
    [anon_sym_i128] = ACTIONS(4124),
    [anon_sym_isize] = ACTIONS(4124),
    [anon_sym_usize] = ACTIONS(4124),
    [anon_sym_f32] = ACTIONS(4124),
    [anon_sym_f64] = ACTIONS(4124),
    [anon_sym_bool] = ACTIONS(4124),
    [anon_sym_str] = ACTIONS(4124),
    [anon_sym_char] = ACTIONS(4124),
    [anon_sym_DASH] = ACTIONS(4122),
    [anon_sym_AMP] = ACTIONS(4122),
    [anon_sym_PIPE] = ACTIONS(4122),
    [anon_sym_LT] = ACTIONS(4122),
    [anon_sym_DOT_DOT] = ACTIONS(4122),
    [anon_sym_COLON_COLON] = ACTIONS(4122),
    [anon_sym_POUND] = ACTIONS(4122),
    [anon_sym_SQUOTE] = ACTIONS(4124),
    [anon_sym_async] = ACTIONS(4124),
    [anon_sym_auto] = ACTIONS(4124),
    [anon_sym_await] = ACTIONS(4124),
    [anon_sym_become] = ACTIONS(4124),
    [anon_sym_break] = ACTIONS(4124),
    [anon_sym_const] = ACTIONS(4124),
    [anon_sym_continue] = ACTIONS(4124),
    [anon_sym_default] = ACTIONS(4124),
    [anon_sym_enum] = ACTIONS(4124),
    [anon_sym_fn] = ACTIONS(4124),
    [anon_sym_for] = ACTIONS(4124),
    [anon_sym_gen] = ACTIONS(4124),
    [anon_sym_if] = ACTIONS(4124),
    [anon_sym_impl] = ACTIONS(4124),
    [anon_sym_let] = ACTIONS(4124),
    [anon_sym_loop] = ACTIONS(4124),
    [anon_sym_match] = ACTIONS(4124),
    [anon_sym_mod] = ACTIONS(4124),
    [anon_sym_pub] = ACTIONS(4124),
    [anon_sym_return] = ACTIONS(4124),
    [anon_sym_raw] = ACTIONS(4124),
    [anon_sym_static] = ACTIONS(4124),
    [anon_sym_struct] = ACTIONS(4124),
    [anon_sym_trait] = ACTIONS(4124),
    [anon_sym_type] = ACTIONS(4124),
    [anon_sym_union] = ACTIONS(4124),
    [anon_sym_unsafe] = ACTIONS(4124),
    [anon_sym_use] = ACTIONS(4124),
    [anon_sym_while] = ACTIONS(4124),
    [anon_sym_extern] = ACTIONS(4124),
    [anon_sym_safe] = ACTIONS(4124),
    [anon_sym_yield] = ACTIONS(4124),
    [anon_sym_move] = ACTIONS(4124),
    [anon_sym_try] = ACTIONS(4124),
    [sym_integer_literal] = ACTIONS(4122),
    [aux_sym_string_literal_token1] = ACTIONS(4122),
    [sym_char_literal] = ACTIONS(4122),
    [anon_sym_true] = ACTIONS(4124),
    [anon_sym_false] = ACTIONS(4124),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4124),
    [sym_super] = ACTIONS(4124),
    [sym_crate] = ACTIONS(4124),
    [aux_sym_metavariable_token1] = ACTIONS(4122),
    [sym__raw_string_literal_start] = ACTIONS(4122),
    [sym_float_literal] = ACTIONS(4122),
    [sym__outer_block_doc_comment_start] = ACTIONS(4122),
    [sym__outer_line_doc_comment_start] = ACTIONS(4122),
  },
  [STATE(1088)] = {
    [sym_line_comment] = STATE(1088),
    [sym_block_comment] = STATE(1088),
    [ts_builtin_sym_end] = ACTIONS(4126),
    [sym_identifier] = ACTIONS(4128),
    [anon_sym_SEMI] = ACTIONS(4126),
    [anon_sym_macro] = ACTIONS(4128),
    [anon_sym_LPAREN] = ACTIONS(4126),
    [anon_sym_LBRACE] = ACTIONS(4126),
    [anon_sym_RBRACE] = ACTIONS(4126),
    [anon_sym_macro_rules] = ACTIONS(4128),
    [anon_sym_BANG] = ACTIONS(4126),
    [anon_sym_LBRACK] = ACTIONS(4126),
    [anon_sym_STAR] = ACTIONS(4126),
    [anon_sym_u8] = ACTIONS(4128),
    [anon_sym_i8] = ACTIONS(4128),
    [anon_sym_u16] = ACTIONS(4128),
    [anon_sym_i16] = ACTIONS(4128),
    [anon_sym_u32] = ACTIONS(4128),
    [anon_sym_i32] = ACTIONS(4128),
    [anon_sym_u64] = ACTIONS(4128),
    [anon_sym_i64] = ACTIONS(4128),
    [anon_sym_u128] = ACTIONS(4128),
    [anon_sym_i128] = ACTIONS(4128),
    [anon_sym_isize] = ACTIONS(4128),
    [anon_sym_usize] = ACTIONS(4128),
    [anon_sym_f32] = ACTIONS(4128),
    [anon_sym_f64] = ACTIONS(4128),
    [anon_sym_bool] = ACTIONS(4128),
    [anon_sym_str] = ACTIONS(4128),
    [anon_sym_char] = ACTIONS(4128),
    [anon_sym_DASH] = ACTIONS(4126),
    [anon_sym_AMP] = ACTIONS(4126),
    [anon_sym_PIPE] = ACTIONS(4126),
    [anon_sym_LT] = ACTIONS(4126),
    [anon_sym_DOT_DOT] = ACTIONS(4126),
    [anon_sym_COLON_COLON] = ACTIONS(4126),
    [anon_sym_POUND] = ACTIONS(4126),
    [anon_sym_SQUOTE] = ACTIONS(4128),
    [anon_sym_async] = ACTIONS(4128),
    [anon_sym_auto] = ACTIONS(4128),
    [anon_sym_await] = ACTIONS(4128),
    [anon_sym_become] = ACTIONS(4128),
    [anon_sym_break] = ACTIONS(4128),
    [anon_sym_const] = ACTIONS(4128),
    [anon_sym_continue] = ACTIONS(4128),
    [anon_sym_default] = ACTIONS(4128),
    [anon_sym_enum] = ACTIONS(4128),
    [anon_sym_fn] = ACTIONS(4128),
    [anon_sym_for] = ACTIONS(4128),
    [anon_sym_gen] = ACTIONS(4128),
    [anon_sym_if] = ACTIONS(4128),
    [anon_sym_impl] = ACTIONS(4128),
    [anon_sym_let] = ACTIONS(4128),
    [anon_sym_loop] = ACTIONS(4128),
    [anon_sym_match] = ACTIONS(4128),
    [anon_sym_mod] = ACTIONS(4128),
    [anon_sym_pub] = ACTIONS(4128),
    [anon_sym_return] = ACTIONS(4128),
    [anon_sym_raw] = ACTIONS(4128),
    [anon_sym_static] = ACTIONS(4128),
    [anon_sym_struct] = ACTIONS(4128),
    [anon_sym_trait] = ACTIONS(4128),
    [anon_sym_type] = ACTIONS(4128),
    [anon_sym_union] = ACTIONS(4128),
    [anon_sym_unsafe] = ACTIONS(4128),
    [anon_sym_use] = ACTIONS(4128),
    [anon_sym_while] = ACTIONS(4128),
    [anon_sym_extern] = ACTIONS(4128),
    [anon_sym_safe] = ACTIONS(4128),
    [anon_sym_yield] = ACTIONS(4128),
    [anon_sym_move] = ACTIONS(4128),
    [anon_sym_try] = ACTIONS(4128),
    [sym_integer_literal] = ACTIONS(4126),
    [aux_sym_string_literal_token1] = ACTIONS(4126),
    [sym_char_literal] = ACTIONS(4126),
    [anon_sym_true] = ACTIONS(4128),
    [anon_sym_false] = ACTIONS(4128),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4128),
    [sym_super] = ACTIONS(4128),
    [sym_crate] = ACTIONS(4128),
    [aux_sym_metavariable_token1] = ACTIONS(4126),
    [sym__raw_string_literal_start] = ACTIONS(4126),
    [sym_float_literal] = ACTIONS(4126),
    [sym__outer_block_doc_comment_start] = ACTIONS(4126),
    [sym__outer_line_doc_comment_start] = ACTIONS(4126),
  },
  [STATE(1089)] = {
    [sym_line_comment] = STATE(1089),
    [sym_block_comment] = STATE(1089),
    [ts_builtin_sym_end] = ACTIONS(4130),
    [sym_identifier] = ACTIONS(4132),
    [anon_sym_SEMI] = ACTIONS(4130),
    [anon_sym_macro] = ACTIONS(4132),
    [anon_sym_LPAREN] = ACTIONS(4130),
    [anon_sym_LBRACE] = ACTIONS(4130),
    [anon_sym_RBRACE] = ACTIONS(4130),
    [anon_sym_macro_rules] = ACTIONS(4132),
    [anon_sym_BANG] = ACTIONS(4130),
    [anon_sym_LBRACK] = ACTIONS(4130),
    [anon_sym_STAR] = ACTIONS(4130),
    [anon_sym_u8] = ACTIONS(4132),
    [anon_sym_i8] = ACTIONS(4132),
    [anon_sym_u16] = ACTIONS(4132),
    [anon_sym_i16] = ACTIONS(4132),
    [anon_sym_u32] = ACTIONS(4132),
    [anon_sym_i32] = ACTIONS(4132),
    [anon_sym_u64] = ACTIONS(4132),
    [anon_sym_i64] = ACTIONS(4132),
    [anon_sym_u128] = ACTIONS(4132),
    [anon_sym_i128] = ACTIONS(4132),
    [anon_sym_isize] = ACTIONS(4132),
    [anon_sym_usize] = ACTIONS(4132),
    [anon_sym_f32] = ACTIONS(4132),
    [anon_sym_f64] = ACTIONS(4132),
    [anon_sym_bool] = ACTIONS(4132),
    [anon_sym_str] = ACTIONS(4132),
    [anon_sym_char] = ACTIONS(4132),
    [anon_sym_DASH] = ACTIONS(4130),
    [anon_sym_AMP] = ACTIONS(4130),
    [anon_sym_PIPE] = ACTIONS(4130),
    [anon_sym_LT] = ACTIONS(4130),
    [anon_sym_DOT_DOT] = ACTIONS(4130),
    [anon_sym_COLON_COLON] = ACTIONS(4130),
    [anon_sym_POUND] = ACTIONS(4130),
    [anon_sym_SQUOTE] = ACTIONS(4132),
    [anon_sym_async] = ACTIONS(4132),
    [anon_sym_auto] = ACTIONS(4132),
    [anon_sym_await] = ACTIONS(4132),
    [anon_sym_become] = ACTIONS(4132),
    [anon_sym_break] = ACTIONS(4132),
    [anon_sym_const] = ACTIONS(4132),
    [anon_sym_continue] = ACTIONS(4132),
    [anon_sym_default] = ACTIONS(4132),
    [anon_sym_enum] = ACTIONS(4132),
    [anon_sym_fn] = ACTIONS(4132),
    [anon_sym_for] = ACTIONS(4132),
    [anon_sym_gen] = ACTIONS(4132),
    [anon_sym_if] = ACTIONS(4132),
    [anon_sym_impl] = ACTIONS(4132),
    [anon_sym_let] = ACTIONS(4132),
    [anon_sym_loop] = ACTIONS(4132),
    [anon_sym_match] = ACTIONS(4132),
    [anon_sym_mod] = ACTIONS(4132),
    [anon_sym_pub] = ACTIONS(4132),
    [anon_sym_return] = ACTIONS(4132),
    [anon_sym_raw] = ACTIONS(4132),
    [anon_sym_static] = ACTIONS(4132),
    [anon_sym_struct] = ACTIONS(4132),
    [anon_sym_trait] = ACTIONS(4132),
    [anon_sym_type] = ACTIONS(4132),
    [anon_sym_union] = ACTIONS(4132),
    [anon_sym_unsafe] = ACTIONS(4132),
    [anon_sym_use] = ACTIONS(4132),
    [anon_sym_while] = ACTIONS(4132),
    [anon_sym_extern] = ACTIONS(4132),
    [anon_sym_safe] = ACTIONS(4132),
    [anon_sym_yield] = ACTIONS(4132),
    [anon_sym_move] = ACTIONS(4132),
    [anon_sym_try] = ACTIONS(4132),
    [sym_integer_literal] = ACTIONS(4130),
    [aux_sym_string_literal_token1] = ACTIONS(4130),
    [sym_char_literal] = ACTIONS(4130),
    [anon_sym_true] = ACTIONS(4132),
    [anon_sym_false] = ACTIONS(4132),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4132),
    [sym_super] = ACTIONS(4132),
    [sym_crate] = ACTIONS(4132),
    [aux_sym_metavariable_token1] = ACTIONS(4130),
    [sym__raw_string_literal_start] = ACTIONS(4130),
    [sym_float_literal] = ACTIONS(4130),
    [sym__outer_block_doc_comment_start] = ACTIONS(4130),
    [sym__outer_line_doc_comment_start] = ACTIONS(4130),
  },
  [STATE(1090)] = {
    [sym_line_comment] = STATE(1090),
    [sym_block_comment] = STATE(1090),
    [ts_builtin_sym_end] = ACTIONS(4134),
    [sym_identifier] = ACTIONS(4136),
    [anon_sym_SEMI] = ACTIONS(4134),
    [anon_sym_macro] = ACTIONS(4136),
    [anon_sym_LPAREN] = ACTIONS(4134),
    [anon_sym_LBRACE] = ACTIONS(4134),
    [anon_sym_RBRACE] = ACTIONS(4134),
    [anon_sym_macro_rules] = ACTIONS(4136),
    [anon_sym_BANG] = ACTIONS(4134),
    [anon_sym_LBRACK] = ACTIONS(4134),
    [anon_sym_STAR] = ACTIONS(4134),
    [anon_sym_u8] = ACTIONS(4136),
    [anon_sym_i8] = ACTIONS(4136),
    [anon_sym_u16] = ACTIONS(4136),
    [anon_sym_i16] = ACTIONS(4136),
    [anon_sym_u32] = ACTIONS(4136),
    [anon_sym_i32] = ACTIONS(4136),
    [anon_sym_u64] = ACTIONS(4136),
    [anon_sym_i64] = ACTIONS(4136),
    [anon_sym_u128] = ACTIONS(4136),
    [anon_sym_i128] = ACTIONS(4136),
    [anon_sym_isize] = ACTIONS(4136),
    [anon_sym_usize] = ACTIONS(4136),
    [anon_sym_f32] = ACTIONS(4136),
    [anon_sym_f64] = ACTIONS(4136),
    [anon_sym_bool] = ACTIONS(4136),
    [anon_sym_str] = ACTIONS(4136),
    [anon_sym_char] = ACTIONS(4136),
    [anon_sym_DASH] = ACTIONS(4134),
    [anon_sym_AMP] = ACTIONS(4134),
    [anon_sym_PIPE] = ACTIONS(4134),
    [anon_sym_LT] = ACTIONS(4134),
    [anon_sym_DOT_DOT] = ACTIONS(4134),
    [anon_sym_COLON_COLON] = ACTIONS(4134),
    [anon_sym_POUND] = ACTIONS(4134),
    [anon_sym_SQUOTE] = ACTIONS(4136),
    [anon_sym_async] = ACTIONS(4136),
    [anon_sym_auto] = ACTIONS(4136),
    [anon_sym_await] = ACTIONS(4136),
    [anon_sym_become] = ACTIONS(4136),
    [anon_sym_break] = ACTIONS(4136),
    [anon_sym_const] = ACTIONS(4136),
    [anon_sym_continue] = ACTIONS(4136),
    [anon_sym_default] = ACTIONS(4136),
    [anon_sym_enum] = ACTIONS(4136),
    [anon_sym_fn] = ACTIONS(4136),
    [anon_sym_for] = ACTIONS(4136),
    [anon_sym_gen] = ACTIONS(4136),
    [anon_sym_if] = ACTIONS(4136),
    [anon_sym_impl] = ACTIONS(4136),
    [anon_sym_let] = ACTIONS(4136),
    [anon_sym_loop] = ACTIONS(4136),
    [anon_sym_match] = ACTIONS(4136),
    [anon_sym_mod] = ACTIONS(4136),
    [anon_sym_pub] = ACTIONS(4136),
    [anon_sym_return] = ACTIONS(4136),
    [anon_sym_raw] = ACTIONS(4136),
    [anon_sym_static] = ACTIONS(4136),
    [anon_sym_struct] = ACTIONS(4136),
    [anon_sym_trait] = ACTIONS(4136),
    [anon_sym_type] = ACTIONS(4136),
    [anon_sym_union] = ACTIONS(4136),
    [anon_sym_unsafe] = ACTIONS(4136),
    [anon_sym_use] = ACTIONS(4136),
    [anon_sym_while] = ACTIONS(4136),
    [anon_sym_extern] = ACTIONS(4136),
    [anon_sym_safe] = ACTIONS(4136),
    [anon_sym_yield] = ACTIONS(4136),
    [anon_sym_move] = ACTIONS(4136),
    [anon_sym_try] = ACTIONS(4136),
    [sym_integer_literal] = ACTIONS(4134),
    [aux_sym_string_literal_token1] = ACTIONS(4134),
    [sym_char_literal] = ACTIONS(4134),
    [anon_sym_true] = ACTIONS(4136),
    [anon_sym_false] = ACTIONS(4136),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4136),
    [sym_super] = ACTIONS(4136),
    [sym_crate] = ACTIONS(4136),
    [aux_sym_metavariable_token1] = ACTIONS(4134),
    [sym__raw_string_literal_start] = ACTIONS(4134),
    [sym_float_literal] = ACTIONS(4134),
    [sym__outer_block_doc_comment_start] = ACTIONS(4134),
    [sym__outer_line_doc_comment_start] = ACTIONS(4134),
  },
  [STATE(1091)] = {
    [sym_line_comment] = STATE(1091),
    [sym_block_comment] = STATE(1091),
    [ts_builtin_sym_end] = ACTIONS(4138),
    [sym_identifier] = ACTIONS(4140),
    [anon_sym_SEMI] = ACTIONS(4138),
    [anon_sym_macro] = ACTIONS(4140),
    [anon_sym_LPAREN] = ACTIONS(4138),
    [anon_sym_LBRACE] = ACTIONS(4138),
    [anon_sym_RBRACE] = ACTIONS(4138),
    [anon_sym_macro_rules] = ACTIONS(4140),
    [anon_sym_BANG] = ACTIONS(4138),
    [anon_sym_LBRACK] = ACTIONS(4138),
    [anon_sym_STAR] = ACTIONS(4138),
    [anon_sym_u8] = ACTIONS(4140),
    [anon_sym_i8] = ACTIONS(4140),
    [anon_sym_u16] = ACTIONS(4140),
    [anon_sym_i16] = ACTIONS(4140),
    [anon_sym_u32] = ACTIONS(4140),
    [anon_sym_i32] = ACTIONS(4140),
    [anon_sym_u64] = ACTIONS(4140),
    [anon_sym_i64] = ACTIONS(4140),
    [anon_sym_u128] = ACTIONS(4140),
    [anon_sym_i128] = ACTIONS(4140),
    [anon_sym_isize] = ACTIONS(4140),
    [anon_sym_usize] = ACTIONS(4140),
    [anon_sym_f32] = ACTIONS(4140),
    [anon_sym_f64] = ACTIONS(4140),
    [anon_sym_bool] = ACTIONS(4140),
    [anon_sym_str] = ACTIONS(4140),
    [anon_sym_char] = ACTIONS(4140),
    [anon_sym_DASH] = ACTIONS(4138),
    [anon_sym_AMP] = ACTIONS(4138),
    [anon_sym_PIPE] = ACTIONS(4138),
    [anon_sym_LT] = ACTIONS(4138),
    [anon_sym_DOT_DOT] = ACTIONS(4138),
    [anon_sym_COLON_COLON] = ACTIONS(4138),
    [anon_sym_POUND] = ACTIONS(4138),
    [anon_sym_SQUOTE] = ACTIONS(4140),
    [anon_sym_async] = ACTIONS(4140),
    [anon_sym_auto] = ACTIONS(4140),
    [anon_sym_await] = ACTIONS(4140),
    [anon_sym_become] = ACTIONS(4140),
    [anon_sym_break] = ACTIONS(4140),
    [anon_sym_const] = ACTIONS(4140),
    [anon_sym_continue] = ACTIONS(4140),
    [anon_sym_default] = ACTIONS(4140),
    [anon_sym_enum] = ACTIONS(4140),
    [anon_sym_fn] = ACTIONS(4140),
    [anon_sym_for] = ACTIONS(4140),
    [anon_sym_gen] = ACTIONS(4140),
    [anon_sym_if] = ACTIONS(4140),
    [anon_sym_impl] = ACTIONS(4140),
    [anon_sym_let] = ACTIONS(4140),
    [anon_sym_loop] = ACTIONS(4140),
    [anon_sym_match] = ACTIONS(4140),
    [anon_sym_mod] = ACTIONS(4140),
    [anon_sym_pub] = ACTIONS(4140),
    [anon_sym_return] = ACTIONS(4140),
    [anon_sym_raw] = ACTIONS(4140),
    [anon_sym_static] = ACTIONS(4140),
    [anon_sym_struct] = ACTIONS(4140),
    [anon_sym_trait] = ACTIONS(4140),
    [anon_sym_type] = ACTIONS(4140),
    [anon_sym_union] = ACTIONS(4140),
    [anon_sym_unsafe] = ACTIONS(4140),
    [anon_sym_use] = ACTIONS(4140),
    [anon_sym_while] = ACTIONS(4140),
    [anon_sym_extern] = ACTIONS(4140),
    [anon_sym_safe] = ACTIONS(4140),
    [anon_sym_yield] = ACTIONS(4140),
    [anon_sym_move] = ACTIONS(4140),
    [anon_sym_try] = ACTIONS(4140),
    [sym_integer_literal] = ACTIONS(4138),
    [aux_sym_string_literal_token1] = ACTIONS(4138),
    [sym_char_literal] = ACTIONS(4138),
    [anon_sym_true] = ACTIONS(4140),
    [anon_sym_false] = ACTIONS(4140),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4140),
    [sym_super] = ACTIONS(4140),
    [sym_crate] = ACTIONS(4140),
    [aux_sym_metavariable_token1] = ACTIONS(4138),
    [sym__raw_string_literal_start] = ACTIONS(4138),
    [sym_float_literal] = ACTIONS(4138),
    [sym__outer_block_doc_comment_start] = ACTIONS(4138),
    [sym__outer_line_doc_comment_start] = ACTIONS(4138),
  },
  [STATE(1092)] = {
    [sym_line_comment] = STATE(1092),
    [sym_block_comment] = STATE(1092),
    [ts_builtin_sym_end] = ACTIONS(4142),
    [sym_identifier] = ACTIONS(4144),
    [anon_sym_SEMI] = ACTIONS(4142),
    [anon_sym_macro] = ACTIONS(4144),
    [anon_sym_LPAREN] = ACTIONS(4142),
    [anon_sym_LBRACE] = ACTIONS(4142),
    [anon_sym_RBRACE] = ACTIONS(4142),
    [anon_sym_macro_rules] = ACTIONS(4144),
    [anon_sym_BANG] = ACTIONS(4142),
    [anon_sym_LBRACK] = ACTIONS(4142),
    [anon_sym_STAR] = ACTIONS(4142),
    [anon_sym_u8] = ACTIONS(4144),
    [anon_sym_i8] = ACTIONS(4144),
    [anon_sym_u16] = ACTIONS(4144),
    [anon_sym_i16] = ACTIONS(4144),
    [anon_sym_u32] = ACTIONS(4144),
    [anon_sym_i32] = ACTIONS(4144),
    [anon_sym_u64] = ACTIONS(4144),
    [anon_sym_i64] = ACTIONS(4144),
    [anon_sym_u128] = ACTIONS(4144),
    [anon_sym_i128] = ACTIONS(4144),
    [anon_sym_isize] = ACTIONS(4144),
    [anon_sym_usize] = ACTIONS(4144),
    [anon_sym_f32] = ACTIONS(4144),
    [anon_sym_f64] = ACTIONS(4144),
    [anon_sym_bool] = ACTIONS(4144),
    [anon_sym_str] = ACTIONS(4144),
    [anon_sym_char] = ACTIONS(4144),
    [anon_sym_DASH] = ACTIONS(4142),
    [anon_sym_AMP] = ACTIONS(4142),
    [anon_sym_PIPE] = ACTIONS(4142),
    [anon_sym_LT] = ACTIONS(4142),
    [anon_sym_DOT_DOT] = ACTIONS(4142),
    [anon_sym_COLON_COLON] = ACTIONS(4142),
    [anon_sym_POUND] = ACTIONS(4142),
    [anon_sym_SQUOTE] = ACTIONS(4144),
    [anon_sym_async] = ACTIONS(4144),
    [anon_sym_auto] = ACTIONS(4144),
    [anon_sym_await] = ACTIONS(4144),
    [anon_sym_become] = ACTIONS(4144),
    [anon_sym_break] = ACTIONS(4144),
    [anon_sym_const] = ACTIONS(4144),
    [anon_sym_continue] = ACTIONS(4144),
    [anon_sym_default] = ACTIONS(4144),
    [anon_sym_enum] = ACTIONS(4144),
    [anon_sym_fn] = ACTIONS(4144),
    [anon_sym_for] = ACTIONS(4144),
    [anon_sym_gen] = ACTIONS(4144),
    [anon_sym_if] = ACTIONS(4144),
    [anon_sym_impl] = ACTIONS(4144),
    [anon_sym_let] = ACTIONS(4144),
    [anon_sym_loop] = ACTIONS(4144),
    [anon_sym_match] = ACTIONS(4144),
    [anon_sym_mod] = ACTIONS(4144),
    [anon_sym_pub] = ACTIONS(4144),
    [anon_sym_return] = ACTIONS(4144),
    [anon_sym_raw] = ACTIONS(4144),
    [anon_sym_static] = ACTIONS(4144),
    [anon_sym_struct] = ACTIONS(4144),
    [anon_sym_trait] = ACTIONS(4144),
    [anon_sym_type] = ACTIONS(4144),
    [anon_sym_union] = ACTIONS(4144),
    [anon_sym_unsafe] = ACTIONS(4144),
    [anon_sym_use] = ACTIONS(4144),
    [anon_sym_while] = ACTIONS(4144),
    [anon_sym_extern] = ACTIONS(4144),
    [anon_sym_safe] = ACTIONS(4144),
    [anon_sym_yield] = ACTIONS(4144),
    [anon_sym_move] = ACTIONS(4144),
    [anon_sym_try] = ACTIONS(4144),
    [sym_integer_literal] = ACTIONS(4142),
    [aux_sym_string_literal_token1] = ACTIONS(4142),
    [sym_char_literal] = ACTIONS(4142),
    [anon_sym_true] = ACTIONS(4144),
    [anon_sym_false] = ACTIONS(4144),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4144),
    [sym_super] = ACTIONS(4144),
    [sym_crate] = ACTIONS(4144),
    [aux_sym_metavariable_token1] = ACTIONS(4142),
    [sym__raw_string_literal_start] = ACTIONS(4142),
    [sym_float_literal] = ACTIONS(4142),
    [sym__outer_block_doc_comment_start] = ACTIONS(4142),
    [sym__outer_line_doc_comment_start] = ACTIONS(4142),
  },
  [STATE(1093)] = {
    [sym_line_comment] = STATE(1093),
    [sym_block_comment] = STATE(1093),
    [ts_builtin_sym_end] = ACTIONS(4146),
    [sym_identifier] = ACTIONS(4148),
    [anon_sym_SEMI] = ACTIONS(4146),
    [anon_sym_macro] = ACTIONS(4148),
    [anon_sym_LPAREN] = ACTIONS(4146),
    [anon_sym_LBRACE] = ACTIONS(4146),
    [anon_sym_RBRACE] = ACTIONS(4146),
    [anon_sym_macro_rules] = ACTIONS(4148),
    [anon_sym_BANG] = ACTIONS(4146),
    [anon_sym_LBRACK] = ACTIONS(4146),
    [anon_sym_STAR] = ACTIONS(4146),
    [anon_sym_u8] = ACTIONS(4148),
    [anon_sym_i8] = ACTIONS(4148),
    [anon_sym_u16] = ACTIONS(4148),
    [anon_sym_i16] = ACTIONS(4148),
    [anon_sym_u32] = ACTIONS(4148),
    [anon_sym_i32] = ACTIONS(4148),
    [anon_sym_u64] = ACTIONS(4148),
    [anon_sym_i64] = ACTIONS(4148),
    [anon_sym_u128] = ACTIONS(4148),
    [anon_sym_i128] = ACTIONS(4148),
    [anon_sym_isize] = ACTIONS(4148),
    [anon_sym_usize] = ACTIONS(4148),
    [anon_sym_f32] = ACTIONS(4148),
    [anon_sym_f64] = ACTIONS(4148),
    [anon_sym_bool] = ACTIONS(4148),
    [anon_sym_str] = ACTIONS(4148),
    [anon_sym_char] = ACTIONS(4148),
    [anon_sym_DASH] = ACTIONS(4146),
    [anon_sym_AMP] = ACTIONS(4146),
    [anon_sym_PIPE] = ACTIONS(4146),
    [anon_sym_LT] = ACTIONS(4146),
    [anon_sym_DOT_DOT] = ACTIONS(4146),
    [anon_sym_COLON_COLON] = ACTIONS(4146),
    [anon_sym_POUND] = ACTIONS(4146),
    [anon_sym_SQUOTE] = ACTIONS(4148),
    [anon_sym_async] = ACTIONS(4148),
    [anon_sym_auto] = ACTIONS(4148),
    [anon_sym_await] = ACTIONS(4148),
    [anon_sym_become] = ACTIONS(4148),
    [anon_sym_break] = ACTIONS(4148),
    [anon_sym_const] = ACTIONS(4148),
    [anon_sym_continue] = ACTIONS(4148),
    [anon_sym_default] = ACTIONS(4148),
    [anon_sym_enum] = ACTIONS(4148),
    [anon_sym_fn] = ACTIONS(4148),
    [anon_sym_for] = ACTIONS(4148),
    [anon_sym_gen] = ACTIONS(4148),
    [anon_sym_if] = ACTIONS(4148),
    [anon_sym_impl] = ACTIONS(4148),
    [anon_sym_let] = ACTIONS(4148),
    [anon_sym_loop] = ACTIONS(4148),
    [anon_sym_match] = ACTIONS(4148),
    [anon_sym_mod] = ACTIONS(4148),
    [anon_sym_pub] = ACTIONS(4148),
    [anon_sym_return] = ACTIONS(4148),
    [anon_sym_raw] = ACTIONS(4148),
    [anon_sym_static] = ACTIONS(4148),
    [anon_sym_struct] = ACTIONS(4148),
    [anon_sym_trait] = ACTIONS(4148),
    [anon_sym_type] = ACTIONS(4148),
    [anon_sym_union] = ACTIONS(4148),
    [anon_sym_unsafe] = ACTIONS(4148),
    [anon_sym_use] = ACTIONS(4148),
    [anon_sym_while] = ACTIONS(4148),
    [anon_sym_extern] = ACTIONS(4148),
    [anon_sym_safe] = ACTIONS(4148),
    [anon_sym_yield] = ACTIONS(4148),
    [anon_sym_move] = ACTIONS(4148),
    [anon_sym_try] = ACTIONS(4148),
    [sym_integer_literal] = ACTIONS(4146),
    [aux_sym_string_literal_token1] = ACTIONS(4146),
    [sym_char_literal] = ACTIONS(4146),
    [anon_sym_true] = ACTIONS(4148),
    [anon_sym_false] = ACTIONS(4148),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4148),
    [sym_super] = ACTIONS(4148),
    [sym_crate] = ACTIONS(4148),
    [aux_sym_metavariable_token1] = ACTIONS(4146),
    [sym__raw_string_literal_start] = ACTIONS(4146),
    [sym_float_literal] = ACTIONS(4146),
    [sym__outer_block_doc_comment_start] = ACTIONS(4146),
    [sym__outer_line_doc_comment_start] = ACTIONS(4146),
  },
  [STATE(1094)] = {
    [sym_line_comment] = STATE(1094),
    [sym_block_comment] = STATE(1094),
    [ts_builtin_sym_end] = ACTIONS(4150),
    [sym_identifier] = ACTIONS(4152),
    [anon_sym_SEMI] = ACTIONS(4150),
    [anon_sym_macro] = ACTIONS(4152),
    [anon_sym_LPAREN] = ACTIONS(4150),
    [anon_sym_LBRACE] = ACTIONS(4150),
    [anon_sym_RBRACE] = ACTIONS(4150),
    [anon_sym_macro_rules] = ACTIONS(4152),
    [anon_sym_BANG] = ACTIONS(4150),
    [anon_sym_LBRACK] = ACTIONS(4150),
    [anon_sym_STAR] = ACTIONS(4150),
    [anon_sym_u8] = ACTIONS(4152),
    [anon_sym_i8] = ACTIONS(4152),
    [anon_sym_u16] = ACTIONS(4152),
    [anon_sym_i16] = ACTIONS(4152),
    [anon_sym_u32] = ACTIONS(4152),
    [anon_sym_i32] = ACTIONS(4152),
    [anon_sym_u64] = ACTIONS(4152),
    [anon_sym_i64] = ACTIONS(4152),
    [anon_sym_u128] = ACTIONS(4152),
    [anon_sym_i128] = ACTIONS(4152),
    [anon_sym_isize] = ACTIONS(4152),
    [anon_sym_usize] = ACTIONS(4152),
    [anon_sym_f32] = ACTIONS(4152),
    [anon_sym_f64] = ACTIONS(4152),
    [anon_sym_bool] = ACTIONS(4152),
    [anon_sym_str] = ACTIONS(4152),
    [anon_sym_char] = ACTIONS(4152),
    [anon_sym_DASH] = ACTIONS(4150),
    [anon_sym_AMP] = ACTIONS(4150),
    [anon_sym_PIPE] = ACTIONS(4150),
    [anon_sym_LT] = ACTIONS(4150),
    [anon_sym_DOT_DOT] = ACTIONS(4150),
    [anon_sym_COLON_COLON] = ACTIONS(4150),
    [anon_sym_POUND] = ACTIONS(4150),
    [anon_sym_SQUOTE] = ACTIONS(4152),
    [anon_sym_async] = ACTIONS(4152),
    [anon_sym_auto] = ACTIONS(4152),
    [anon_sym_await] = ACTIONS(4152),
    [anon_sym_become] = ACTIONS(4152),
    [anon_sym_break] = ACTIONS(4152),
    [anon_sym_const] = ACTIONS(4152),
    [anon_sym_continue] = ACTIONS(4152),
    [anon_sym_default] = ACTIONS(4152),
    [anon_sym_enum] = ACTIONS(4152),
    [anon_sym_fn] = ACTIONS(4152),
    [anon_sym_for] = ACTIONS(4152),
    [anon_sym_gen] = ACTIONS(4152),
    [anon_sym_if] = ACTIONS(4152),
    [anon_sym_impl] = ACTIONS(4152),
    [anon_sym_let] = ACTIONS(4152),
    [anon_sym_loop] = ACTIONS(4152),
    [anon_sym_match] = ACTIONS(4152),
    [anon_sym_mod] = ACTIONS(4152),
    [anon_sym_pub] = ACTIONS(4152),
    [anon_sym_return] = ACTIONS(4152),
    [anon_sym_raw] = ACTIONS(4152),
    [anon_sym_static] = ACTIONS(4152),
    [anon_sym_struct] = ACTIONS(4152),
    [anon_sym_trait] = ACTIONS(4152),
    [anon_sym_type] = ACTIONS(4152),
    [anon_sym_union] = ACTIONS(4152),
    [anon_sym_unsafe] = ACTIONS(4152),
    [anon_sym_use] = ACTIONS(4152),
    [anon_sym_while] = ACTIONS(4152),
    [anon_sym_extern] = ACTIONS(4152),
    [anon_sym_safe] = ACTIONS(4152),
    [anon_sym_yield] = ACTIONS(4152),
    [anon_sym_move] = ACTIONS(4152),
    [anon_sym_try] = ACTIONS(4152),
    [sym_integer_literal] = ACTIONS(4150),
    [aux_sym_string_literal_token1] = ACTIONS(4150),
    [sym_char_literal] = ACTIONS(4150),
    [anon_sym_true] = ACTIONS(4152),
    [anon_sym_false] = ACTIONS(4152),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4152),
    [sym_super] = ACTIONS(4152),
    [sym_crate] = ACTIONS(4152),
    [aux_sym_metavariable_token1] = ACTIONS(4150),
    [sym__raw_string_literal_start] = ACTIONS(4150),
    [sym_float_literal] = ACTIONS(4150),
    [sym__outer_block_doc_comment_start] = ACTIONS(4150),
    [sym__outer_line_doc_comment_start] = ACTIONS(4150),
  },
  [STATE(1095)] = {
    [sym_line_comment] = STATE(1095),
    [sym_block_comment] = STATE(1095),
    [ts_builtin_sym_end] = ACTIONS(4154),
    [sym_identifier] = ACTIONS(4156),
    [anon_sym_SEMI] = ACTIONS(4154),
    [anon_sym_macro] = ACTIONS(4156),
    [anon_sym_LPAREN] = ACTIONS(4154),
    [anon_sym_LBRACE] = ACTIONS(4154),
    [anon_sym_RBRACE] = ACTIONS(4154),
    [anon_sym_macro_rules] = ACTIONS(4156),
    [anon_sym_BANG] = ACTIONS(4154),
    [anon_sym_LBRACK] = ACTIONS(4154),
    [anon_sym_STAR] = ACTIONS(4154),
    [anon_sym_u8] = ACTIONS(4156),
    [anon_sym_i8] = ACTIONS(4156),
    [anon_sym_u16] = ACTIONS(4156),
    [anon_sym_i16] = ACTIONS(4156),
    [anon_sym_u32] = ACTIONS(4156),
    [anon_sym_i32] = ACTIONS(4156),
    [anon_sym_u64] = ACTIONS(4156),
    [anon_sym_i64] = ACTIONS(4156),
    [anon_sym_u128] = ACTIONS(4156),
    [anon_sym_i128] = ACTIONS(4156),
    [anon_sym_isize] = ACTIONS(4156),
    [anon_sym_usize] = ACTIONS(4156),
    [anon_sym_f32] = ACTIONS(4156),
    [anon_sym_f64] = ACTIONS(4156),
    [anon_sym_bool] = ACTIONS(4156),
    [anon_sym_str] = ACTIONS(4156),
    [anon_sym_char] = ACTIONS(4156),
    [anon_sym_DASH] = ACTIONS(4154),
    [anon_sym_AMP] = ACTIONS(4154),
    [anon_sym_PIPE] = ACTIONS(4154),
    [anon_sym_LT] = ACTIONS(4154),
    [anon_sym_DOT_DOT] = ACTIONS(4154),
    [anon_sym_COLON_COLON] = ACTIONS(4154),
    [anon_sym_POUND] = ACTIONS(4154),
    [anon_sym_SQUOTE] = ACTIONS(4156),
    [anon_sym_async] = ACTIONS(4156),
    [anon_sym_auto] = ACTIONS(4156),
    [anon_sym_await] = ACTIONS(4156),
    [anon_sym_become] = ACTIONS(4156),
    [anon_sym_break] = ACTIONS(4156),
    [anon_sym_const] = ACTIONS(4156),
    [anon_sym_continue] = ACTIONS(4156),
    [anon_sym_default] = ACTIONS(4156),
    [anon_sym_enum] = ACTIONS(4156),
    [anon_sym_fn] = ACTIONS(4156),
    [anon_sym_for] = ACTIONS(4156),
    [anon_sym_gen] = ACTIONS(4156),
    [anon_sym_if] = ACTIONS(4156),
    [anon_sym_impl] = ACTIONS(4156),
    [anon_sym_let] = ACTIONS(4156),
    [anon_sym_loop] = ACTIONS(4156),
    [anon_sym_match] = ACTIONS(4156),
    [anon_sym_mod] = ACTIONS(4156),
    [anon_sym_pub] = ACTIONS(4156),
    [anon_sym_return] = ACTIONS(4156),
    [anon_sym_raw] = ACTIONS(4156),
    [anon_sym_static] = ACTIONS(4156),
    [anon_sym_struct] = ACTIONS(4156),
    [anon_sym_trait] = ACTIONS(4156),
    [anon_sym_type] = ACTIONS(4156),
    [anon_sym_union] = ACTIONS(4156),
    [anon_sym_unsafe] = ACTIONS(4156),
    [anon_sym_use] = ACTIONS(4156),
    [anon_sym_while] = ACTIONS(4156),
    [anon_sym_extern] = ACTIONS(4156),
    [anon_sym_safe] = ACTIONS(4156),
    [anon_sym_yield] = ACTIONS(4156),
    [anon_sym_move] = ACTIONS(4156),
    [anon_sym_try] = ACTIONS(4156),
    [sym_integer_literal] = ACTIONS(4154),
    [aux_sym_string_literal_token1] = ACTIONS(4154),
    [sym_char_literal] = ACTIONS(4154),
    [anon_sym_true] = ACTIONS(4156),
    [anon_sym_false] = ACTIONS(4156),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4156),
    [sym_super] = ACTIONS(4156),
    [sym_crate] = ACTIONS(4156),
    [aux_sym_metavariable_token1] = ACTIONS(4154),
    [sym__raw_string_literal_start] = ACTIONS(4154),
    [sym_float_literal] = ACTIONS(4154),
    [sym__outer_block_doc_comment_start] = ACTIONS(4154),
    [sym__outer_line_doc_comment_start] = ACTIONS(4154),
  },
  [STATE(1096)] = {
    [sym_line_comment] = STATE(1096),
    [sym_block_comment] = STATE(1096),
    [ts_builtin_sym_end] = ACTIONS(4158),
    [sym_identifier] = ACTIONS(4160),
    [anon_sym_SEMI] = ACTIONS(4158),
    [anon_sym_macro] = ACTIONS(4160),
    [anon_sym_LPAREN] = ACTIONS(4158),
    [anon_sym_LBRACE] = ACTIONS(4158),
    [anon_sym_RBRACE] = ACTIONS(4158),
    [anon_sym_macro_rules] = ACTIONS(4160),
    [anon_sym_BANG] = ACTIONS(4158),
    [anon_sym_LBRACK] = ACTIONS(4158),
    [anon_sym_STAR] = ACTIONS(4158),
    [anon_sym_u8] = ACTIONS(4160),
    [anon_sym_i8] = ACTIONS(4160),
    [anon_sym_u16] = ACTIONS(4160),
    [anon_sym_i16] = ACTIONS(4160),
    [anon_sym_u32] = ACTIONS(4160),
    [anon_sym_i32] = ACTIONS(4160),
    [anon_sym_u64] = ACTIONS(4160),
    [anon_sym_i64] = ACTIONS(4160),
    [anon_sym_u128] = ACTIONS(4160),
    [anon_sym_i128] = ACTIONS(4160),
    [anon_sym_isize] = ACTIONS(4160),
    [anon_sym_usize] = ACTIONS(4160),
    [anon_sym_f32] = ACTIONS(4160),
    [anon_sym_f64] = ACTIONS(4160),
    [anon_sym_bool] = ACTIONS(4160),
    [anon_sym_str] = ACTIONS(4160),
    [anon_sym_char] = ACTIONS(4160),
    [anon_sym_DASH] = ACTIONS(4158),
    [anon_sym_AMP] = ACTIONS(4158),
    [anon_sym_PIPE] = ACTIONS(4158),
    [anon_sym_LT] = ACTIONS(4158),
    [anon_sym_DOT_DOT] = ACTIONS(4158),
    [anon_sym_COLON_COLON] = ACTIONS(4158),
    [anon_sym_POUND] = ACTIONS(4158),
    [anon_sym_SQUOTE] = ACTIONS(4160),
    [anon_sym_async] = ACTIONS(4160),
    [anon_sym_auto] = ACTIONS(4160),
    [anon_sym_await] = ACTIONS(4160),
    [anon_sym_become] = ACTIONS(4160),
    [anon_sym_break] = ACTIONS(4160),
    [anon_sym_const] = ACTIONS(4160),
    [anon_sym_continue] = ACTIONS(4160),
    [anon_sym_default] = ACTIONS(4160),
    [anon_sym_enum] = ACTIONS(4160),
    [anon_sym_fn] = ACTIONS(4160),
    [anon_sym_for] = ACTIONS(4160),
    [anon_sym_gen] = ACTIONS(4160),
    [anon_sym_if] = ACTIONS(4160),
    [anon_sym_impl] = ACTIONS(4160),
    [anon_sym_let] = ACTIONS(4160),
    [anon_sym_loop] = ACTIONS(4160),
    [anon_sym_match] = ACTIONS(4160),
    [anon_sym_mod] = ACTIONS(4160),
    [anon_sym_pub] = ACTIONS(4160),
    [anon_sym_return] = ACTIONS(4160),
    [anon_sym_raw] = ACTIONS(4160),
    [anon_sym_static] = ACTIONS(4160),
    [anon_sym_struct] = ACTIONS(4160),
    [anon_sym_trait] = ACTIONS(4160),
    [anon_sym_type] = ACTIONS(4160),
    [anon_sym_union] = ACTIONS(4160),
    [anon_sym_unsafe] = ACTIONS(4160),
    [anon_sym_use] = ACTIONS(4160),
    [anon_sym_while] = ACTIONS(4160),
    [anon_sym_extern] = ACTIONS(4160),
    [anon_sym_safe] = ACTIONS(4160),
    [anon_sym_yield] = ACTIONS(4160),
    [anon_sym_move] = ACTIONS(4160),
    [anon_sym_try] = ACTIONS(4160),
    [sym_integer_literal] = ACTIONS(4158),
    [aux_sym_string_literal_token1] = ACTIONS(4158),
    [sym_char_literal] = ACTIONS(4158),
    [anon_sym_true] = ACTIONS(4160),
    [anon_sym_false] = ACTIONS(4160),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4160),
    [sym_super] = ACTIONS(4160),
    [sym_crate] = ACTIONS(4160),
    [aux_sym_metavariable_token1] = ACTIONS(4158),
    [sym__raw_string_literal_start] = ACTIONS(4158),
    [sym_float_literal] = ACTIONS(4158),
    [sym__outer_block_doc_comment_start] = ACTIONS(4158),
    [sym__outer_line_doc_comment_start] = ACTIONS(4158),
  },
  [STATE(1097)] = {
    [sym_line_comment] = STATE(1097),
    [sym_block_comment] = STATE(1097),
    [ts_builtin_sym_end] = ACTIONS(4162),
    [sym_identifier] = ACTIONS(4164),
    [anon_sym_SEMI] = ACTIONS(4162),
    [anon_sym_macro] = ACTIONS(4164),
    [anon_sym_LPAREN] = ACTIONS(4162),
    [anon_sym_LBRACE] = ACTIONS(4162),
    [anon_sym_RBRACE] = ACTIONS(4162),
    [anon_sym_macro_rules] = ACTIONS(4164),
    [anon_sym_BANG] = ACTIONS(4162),
    [anon_sym_LBRACK] = ACTIONS(4162),
    [anon_sym_STAR] = ACTIONS(4162),
    [anon_sym_u8] = ACTIONS(4164),
    [anon_sym_i8] = ACTIONS(4164),
    [anon_sym_u16] = ACTIONS(4164),
    [anon_sym_i16] = ACTIONS(4164),
    [anon_sym_u32] = ACTIONS(4164),
    [anon_sym_i32] = ACTIONS(4164),
    [anon_sym_u64] = ACTIONS(4164),
    [anon_sym_i64] = ACTIONS(4164),
    [anon_sym_u128] = ACTIONS(4164),
    [anon_sym_i128] = ACTIONS(4164),
    [anon_sym_isize] = ACTIONS(4164),
    [anon_sym_usize] = ACTIONS(4164),
    [anon_sym_f32] = ACTIONS(4164),
    [anon_sym_f64] = ACTIONS(4164),
    [anon_sym_bool] = ACTIONS(4164),
    [anon_sym_str] = ACTIONS(4164),
    [anon_sym_char] = ACTIONS(4164),
    [anon_sym_DASH] = ACTIONS(4162),
    [anon_sym_AMP] = ACTIONS(4162),
    [anon_sym_PIPE] = ACTIONS(4162),
    [anon_sym_LT] = ACTIONS(4162),
    [anon_sym_DOT_DOT] = ACTIONS(4162),
    [anon_sym_COLON_COLON] = ACTIONS(4162),
    [anon_sym_POUND] = ACTIONS(4162),
    [anon_sym_SQUOTE] = ACTIONS(4164),
    [anon_sym_async] = ACTIONS(4164),
    [anon_sym_auto] = ACTIONS(4164),
    [anon_sym_await] = ACTIONS(4164),
    [anon_sym_become] = ACTIONS(4164),
    [anon_sym_break] = ACTIONS(4164),
    [anon_sym_const] = ACTIONS(4164),
    [anon_sym_continue] = ACTIONS(4164),
    [anon_sym_default] = ACTIONS(4164),
    [anon_sym_enum] = ACTIONS(4164),
    [anon_sym_fn] = ACTIONS(4164),
    [anon_sym_for] = ACTIONS(4164),
    [anon_sym_gen] = ACTIONS(4164),
    [anon_sym_if] = ACTIONS(4164),
    [anon_sym_impl] = ACTIONS(4164),
    [anon_sym_let] = ACTIONS(4164),
    [anon_sym_loop] = ACTIONS(4164),
    [anon_sym_match] = ACTIONS(4164),
    [anon_sym_mod] = ACTIONS(4164),
    [anon_sym_pub] = ACTIONS(4164),
    [anon_sym_return] = ACTIONS(4164),
    [anon_sym_raw] = ACTIONS(4164),
    [anon_sym_static] = ACTIONS(4164),
    [anon_sym_struct] = ACTIONS(4164),
    [anon_sym_trait] = ACTIONS(4164),
    [anon_sym_type] = ACTIONS(4164),
    [anon_sym_union] = ACTIONS(4164),
    [anon_sym_unsafe] = ACTIONS(4164),
    [anon_sym_use] = ACTIONS(4164),
    [anon_sym_while] = ACTIONS(4164),
    [anon_sym_extern] = ACTIONS(4164),
    [anon_sym_safe] = ACTIONS(4164),
    [anon_sym_yield] = ACTIONS(4164),
    [anon_sym_move] = ACTIONS(4164),
    [anon_sym_try] = ACTIONS(4164),
    [sym_integer_literal] = ACTIONS(4162),
    [aux_sym_string_literal_token1] = ACTIONS(4162),
    [sym_char_literal] = ACTIONS(4162),
    [anon_sym_true] = ACTIONS(4164),
    [anon_sym_false] = ACTIONS(4164),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4164),
    [sym_super] = ACTIONS(4164),
    [sym_crate] = ACTIONS(4164),
    [aux_sym_metavariable_token1] = ACTIONS(4162),
    [sym__raw_string_literal_start] = ACTIONS(4162),
    [sym_float_literal] = ACTIONS(4162),
    [sym__outer_block_doc_comment_start] = ACTIONS(4162),
    [sym__outer_line_doc_comment_start] = ACTIONS(4162),
  },
  [STATE(1098)] = {
    [sym_line_comment] = STATE(1098),
    [sym_block_comment] = STATE(1098),
    [ts_builtin_sym_end] = ACTIONS(4166),
    [sym_identifier] = ACTIONS(4168),
    [anon_sym_SEMI] = ACTIONS(4166),
    [anon_sym_macro] = ACTIONS(4168),
    [anon_sym_LPAREN] = ACTIONS(4166),
    [anon_sym_LBRACE] = ACTIONS(4166),
    [anon_sym_RBRACE] = ACTIONS(4166),
    [anon_sym_macro_rules] = ACTIONS(4168),
    [anon_sym_BANG] = ACTIONS(4166),
    [anon_sym_LBRACK] = ACTIONS(4166),
    [anon_sym_STAR] = ACTIONS(4166),
    [anon_sym_u8] = ACTIONS(4168),
    [anon_sym_i8] = ACTIONS(4168),
    [anon_sym_u16] = ACTIONS(4168),
    [anon_sym_i16] = ACTIONS(4168),
    [anon_sym_u32] = ACTIONS(4168),
    [anon_sym_i32] = ACTIONS(4168),
    [anon_sym_u64] = ACTIONS(4168),
    [anon_sym_i64] = ACTIONS(4168),
    [anon_sym_u128] = ACTIONS(4168),
    [anon_sym_i128] = ACTIONS(4168),
    [anon_sym_isize] = ACTIONS(4168),
    [anon_sym_usize] = ACTIONS(4168),
    [anon_sym_f32] = ACTIONS(4168),
    [anon_sym_f64] = ACTIONS(4168),
    [anon_sym_bool] = ACTIONS(4168),
    [anon_sym_str] = ACTIONS(4168),
    [anon_sym_char] = ACTIONS(4168),
    [anon_sym_DASH] = ACTIONS(4166),
    [anon_sym_AMP] = ACTIONS(4166),
    [anon_sym_PIPE] = ACTIONS(4166),
    [anon_sym_LT] = ACTIONS(4166),
    [anon_sym_DOT_DOT] = ACTIONS(4166),
    [anon_sym_COLON_COLON] = ACTIONS(4166),
    [anon_sym_POUND] = ACTIONS(4166),
    [anon_sym_SQUOTE] = ACTIONS(4168),
    [anon_sym_async] = ACTIONS(4168),
    [anon_sym_auto] = ACTIONS(4168),
    [anon_sym_await] = ACTIONS(4168),
    [anon_sym_become] = ACTIONS(4168),
    [anon_sym_break] = ACTIONS(4168),
    [anon_sym_const] = ACTIONS(4168),
    [anon_sym_continue] = ACTIONS(4168),
    [anon_sym_default] = ACTIONS(4168),
    [anon_sym_enum] = ACTIONS(4168),
    [anon_sym_fn] = ACTIONS(4168),
    [anon_sym_for] = ACTIONS(4168),
    [anon_sym_gen] = ACTIONS(4168),
    [anon_sym_if] = ACTIONS(4168),
    [anon_sym_impl] = ACTIONS(4168),
    [anon_sym_let] = ACTIONS(4168),
    [anon_sym_loop] = ACTIONS(4168),
    [anon_sym_match] = ACTIONS(4168),
    [anon_sym_mod] = ACTIONS(4168),
    [anon_sym_pub] = ACTIONS(4168),
    [anon_sym_return] = ACTIONS(4168),
    [anon_sym_raw] = ACTIONS(4168),
    [anon_sym_static] = ACTIONS(4168),
    [anon_sym_struct] = ACTIONS(4168),
    [anon_sym_trait] = ACTIONS(4168),
    [anon_sym_type] = ACTIONS(4168),
    [anon_sym_union] = ACTIONS(4168),
    [anon_sym_unsafe] = ACTIONS(4168),
    [anon_sym_use] = ACTIONS(4168),
    [anon_sym_while] = ACTIONS(4168),
    [anon_sym_extern] = ACTIONS(4168),
    [anon_sym_safe] = ACTIONS(4168),
    [anon_sym_yield] = ACTIONS(4168),
    [anon_sym_move] = ACTIONS(4168),
    [anon_sym_try] = ACTIONS(4168),
    [sym_integer_literal] = ACTIONS(4166),
    [aux_sym_string_literal_token1] = ACTIONS(4166),
    [sym_char_literal] = ACTIONS(4166),
    [anon_sym_true] = ACTIONS(4168),
    [anon_sym_false] = ACTIONS(4168),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4168),
    [sym_super] = ACTIONS(4168),
    [sym_crate] = ACTIONS(4168),
    [aux_sym_metavariable_token1] = ACTIONS(4166),
    [sym__raw_string_literal_start] = ACTIONS(4166),
    [sym_float_literal] = ACTIONS(4166),
    [sym__outer_block_doc_comment_start] = ACTIONS(4166),
    [sym__outer_line_doc_comment_start] = ACTIONS(4166),
  },
  [STATE(1099)] = {
    [sym_line_comment] = STATE(1099),
    [sym_block_comment] = STATE(1099),
    [ts_builtin_sym_end] = ACTIONS(4170),
    [sym_identifier] = ACTIONS(4172),
    [anon_sym_SEMI] = ACTIONS(4170),
    [anon_sym_macro] = ACTIONS(4172),
    [anon_sym_LPAREN] = ACTIONS(4170),
    [anon_sym_LBRACE] = ACTIONS(4170),
    [anon_sym_RBRACE] = ACTIONS(4170),
    [anon_sym_macro_rules] = ACTIONS(4172),
    [anon_sym_BANG] = ACTIONS(4170),
    [anon_sym_LBRACK] = ACTIONS(4170),
    [anon_sym_STAR] = ACTIONS(4170),
    [anon_sym_u8] = ACTIONS(4172),
    [anon_sym_i8] = ACTIONS(4172),
    [anon_sym_u16] = ACTIONS(4172),
    [anon_sym_i16] = ACTIONS(4172),
    [anon_sym_u32] = ACTIONS(4172),
    [anon_sym_i32] = ACTIONS(4172),
    [anon_sym_u64] = ACTIONS(4172),
    [anon_sym_i64] = ACTIONS(4172),
    [anon_sym_u128] = ACTIONS(4172),
    [anon_sym_i128] = ACTIONS(4172),
    [anon_sym_isize] = ACTIONS(4172),
    [anon_sym_usize] = ACTIONS(4172),
    [anon_sym_f32] = ACTIONS(4172),
    [anon_sym_f64] = ACTIONS(4172),
    [anon_sym_bool] = ACTIONS(4172),
    [anon_sym_str] = ACTIONS(4172),
    [anon_sym_char] = ACTIONS(4172),
    [anon_sym_DASH] = ACTIONS(4170),
    [anon_sym_AMP] = ACTIONS(4170),
    [anon_sym_PIPE] = ACTIONS(4170),
    [anon_sym_LT] = ACTIONS(4170),
    [anon_sym_DOT_DOT] = ACTIONS(4170),
    [anon_sym_COLON_COLON] = ACTIONS(4170),
    [anon_sym_POUND] = ACTIONS(4170),
    [anon_sym_SQUOTE] = ACTIONS(4172),
    [anon_sym_async] = ACTIONS(4172),
    [anon_sym_auto] = ACTIONS(4172),
    [anon_sym_await] = ACTIONS(4172),
    [anon_sym_become] = ACTIONS(4172),
    [anon_sym_break] = ACTIONS(4172),
    [anon_sym_const] = ACTIONS(4172),
    [anon_sym_continue] = ACTIONS(4172),
    [anon_sym_default] = ACTIONS(4172),
    [anon_sym_enum] = ACTIONS(4172),
    [anon_sym_fn] = ACTIONS(4172),
    [anon_sym_for] = ACTIONS(4172),
    [anon_sym_gen] = ACTIONS(4172),
    [anon_sym_if] = ACTIONS(4172),
    [anon_sym_impl] = ACTIONS(4172),
    [anon_sym_let] = ACTIONS(4172),
    [anon_sym_loop] = ACTIONS(4172),
    [anon_sym_match] = ACTIONS(4172),
    [anon_sym_mod] = ACTIONS(4172),
    [anon_sym_pub] = ACTIONS(4172),
    [anon_sym_return] = ACTIONS(4172),
    [anon_sym_raw] = ACTIONS(4172),
    [anon_sym_static] = ACTIONS(4172),
    [anon_sym_struct] = ACTIONS(4172),
    [anon_sym_trait] = ACTIONS(4172),
    [anon_sym_type] = ACTIONS(4172),
    [anon_sym_union] = ACTIONS(4172),
    [anon_sym_unsafe] = ACTIONS(4172),
    [anon_sym_use] = ACTIONS(4172),
    [anon_sym_while] = ACTIONS(4172),
    [anon_sym_extern] = ACTIONS(4172),
    [anon_sym_safe] = ACTIONS(4172),
    [anon_sym_yield] = ACTIONS(4172),
    [anon_sym_move] = ACTIONS(4172),
    [anon_sym_try] = ACTIONS(4172),
    [sym_integer_literal] = ACTIONS(4170),
    [aux_sym_string_literal_token1] = ACTIONS(4170),
    [sym_char_literal] = ACTIONS(4170),
    [anon_sym_true] = ACTIONS(4172),
    [anon_sym_false] = ACTIONS(4172),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4172),
    [sym_super] = ACTIONS(4172),
    [sym_crate] = ACTIONS(4172),
    [aux_sym_metavariable_token1] = ACTIONS(4170),
    [sym__raw_string_literal_start] = ACTIONS(4170),
    [sym_float_literal] = ACTIONS(4170),
    [sym__outer_block_doc_comment_start] = ACTIONS(4170),
    [sym__outer_line_doc_comment_start] = ACTIONS(4170),
  },
  [STATE(1100)] = {
    [sym_line_comment] = STATE(1100),
    [sym_block_comment] = STATE(1100),
    [ts_builtin_sym_end] = ACTIONS(4174),
    [sym_identifier] = ACTIONS(4176),
    [anon_sym_SEMI] = ACTIONS(4174),
    [anon_sym_macro] = ACTIONS(4176),
    [anon_sym_LPAREN] = ACTIONS(4174),
    [anon_sym_LBRACE] = ACTIONS(4174),
    [anon_sym_RBRACE] = ACTIONS(4174),
    [anon_sym_macro_rules] = ACTIONS(4176),
    [anon_sym_BANG] = ACTIONS(4174),
    [anon_sym_LBRACK] = ACTIONS(4174),
    [anon_sym_STAR] = ACTIONS(4174),
    [anon_sym_u8] = ACTIONS(4176),
    [anon_sym_i8] = ACTIONS(4176),
    [anon_sym_u16] = ACTIONS(4176),
    [anon_sym_i16] = ACTIONS(4176),
    [anon_sym_u32] = ACTIONS(4176),
    [anon_sym_i32] = ACTIONS(4176),
    [anon_sym_u64] = ACTIONS(4176),
    [anon_sym_i64] = ACTIONS(4176),
    [anon_sym_u128] = ACTIONS(4176),
    [anon_sym_i128] = ACTIONS(4176),
    [anon_sym_isize] = ACTIONS(4176),
    [anon_sym_usize] = ACTIONS(4176),
    [anon_sym_f32] = ACTIONS(4176),
    [anon_sym_f64] = ACTIONS(4176),
    [anon_sym_bool] = ACTIONS(4176),
    [anon_sym_str] = ACTIONS(4176),
    [anon_sym_char] = ACTIONS(4176),
    [anon_sym_DASH] = ACTIONS(4174),
    [anon_sym_AMP] = ACTIONS(4174),
    [anon_sym_PIPE] = ACTIONS(4174),
    [anon_sym_LT] = ACTIONS(4174),
    [anon_sym_DOT_DOT] = ACTIONS(4174),
    [anon_sym_COLON_COLON] = ACTIONS(4174),
    [anon_sym_POUND] = ACTIONS(4174),
    [anon_sym_SQUOTE] = ACTIONS(4176),
    [anon_sym_async] = ACTIONS(4176),
    [anon_sym_auto] = ACTIONS(4176),
    [anon_sym_await] = ACTIONS(4176),
    [anon_sym_become] = ACTIONS(4176),
    [anon_sym_break] = ACTIONS(4176),
    [anon_sym_const] = ACTIONS(4176),
    [anon_sym_continue] = ACTIONS(4176),
    [anon_sym_default] = ACTIONS(4176),
    [anon_sym_enum] = ACTIONS(4176),
    [anon_sym_fn] = ACTIONS(4176),
    [anon_sym_for] = ACTIONS(4176),
    [anon_sym_gen] = ACTIONS(4176),
    [anon_sym_if] = ACTIONS(4176),
    [anon_sym_impl] = ACTIONS(4176),
    [anon_sym_let] = ACTIONS(4176),
    [anon_sym_loop] = ACTIONS(4176),
    [anon_sym_match] = ACTIONS(4176),
    [anon_sym_mod] = ACTIONS(4176),
    [anon_sym_pub] = ACTIONS(4176),
    [anon_sym_return] = ACTIONS(4176),
    [anon_sym_raw] = ACTIONS(4176),
    [anon_sym_static] = ACTIONS(4176),
    [anon_sym_struct] = ACTIONS(4176),
    [anon_sym_trait] = ACTIONS(4176),
    [anon_sym_type] = ACTIONS(4176),
    [anon_sym_union] = ACTIONS(4176),
    [anon_sym_unsafe] = ACTIONS(4176),
    [anon_sym_use] = ACTIONS(4176),
    [anon_sym_while] = ACTIONS(4176),
    [anon_sym_extern] = ACTIONS(4176),
    [anon_sym_safe] = ACTIONS(4176),
    [anon_sym_yield] = ACTIONS(4176),
    [anon_sym_move] = ACTIONS(4176),
    [anon_sym_try] = ACTIONS(4176),
    [sym_integer_literal] = ACTIONS(4174),
    [aux_sym_string_literal_token1] = ACTIONS(4174),
    [sym_char_literal] = ACTIONS(4174),
    [anon_sym_true] = ACTIONS(4176),
    [anon_sym_false] = ACTIONS(4176),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4176),
    [sym_super] = ACTIONS(4176),
    [sym_crate] = ACTIONS(4176),
    [aux_sym_metavariable_token1] = ACTIONS(4174),
    [sym__raw_string_literal_start] = ACTIONS(4174),
    [sym_float_literal] = ACTIONS(4174),
    [sym__outer_block_doc_comment_start] = ACTIONS(4174),
    [sym__outer_line_doc_comment_start] = ACTIONS(4174),
  },
  [STATE(1101)] = {
    [sym_line_comment] = STATE(1101),
    [sym_block_comment] = STATE(1101),
    [ts_builtin_sym_end] = ACTIONS(4178),
    [sym_identifier] = ACTIONS(4180),
    [anon_sym_SEMI] = ACTIONS(4178),
    [anon_sym_macro] = ACTIONS(4180),
    [anon_sym_LPAREN] = ACTIONS(4178),
    [anon_sym_LBRACE] = ACTIONS(4178),
    [anon_sym_RBRACE] = ACTIONS(4178),
    [anon_sym_macro_rules] = ACTIONS(4180),
    [anon_sym_BANG] = ACTIONS(4178),
    [anon_sym_LBRACK] = ACTIONS(4178),
    [anon_sym_STAR] = ACTIONS(4178),
    [anon_sym_u8] = ACTIONS(4180),
    [anon_sym_i8] = ACTIONS(4180),
    [anon_sym_u16] = ACTIONS(4180),
    [anon_sym_i16] = ACTIONS(4180),
    [anon_sym_u32] = ACTIONS(4180),
    [anon_sym_i32] = ACTIONS(4180),
    [anon_sym_u64] = ACTIONS(4180),
    [anon_sym_i64] = ACTIONS(4180),
    [anon_sym_u128] = ACTIONS(4180),
    [anon_sym_i128] = ACTIONS(4180),
    [anon_sym_isize] = ACTIONS(4180),
    [anon_sym_usize] = ACTIONS(4180),
    [anon_sym_f32] = ACTIONS(4180),
    [anon_sym_f64] = ACTIONS(4180),
    [anon_sym_bool] = ACTIONS(4180),
    [anon_sym_str] = ACTIONS(4180),
    [anon_sym_char] = ACTIONS(4180),
    [anon_sym_DASH] = ACTIONS(4178),
    [anon_sym_AMP] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4178),
    [anon_sym_LT] = ACTIONS(4178),
    [anon_sym_DOT_DOT] = ACTIONS(4178),
    [anon_sym_COLON_COLON] = ACTIONS(4178),
    [anon_sym_POUND] = ACTIONS(4178),
    [anon_sym_SQUOTE] = ACTIONS(4180),
    [anon_sym_async] = ACTIONS(4180),
    [anon_sym_auto] = ACTIONS(4180),
    [anon_sym_await] = ACTIONS(4180),
    [anon_sym_become] = ACTIONS(4180),
    [anon_sym_break] = ACTIONS(4180),
    [anon_sym_const] = ACTIONS(4180),
    [anon_sym_continue] = ACTIONS(4180),
    [anon_sym_default] = ACTIONS(4180),
    [anon_sym_enum] = ACTIONS(4180),
    [anon_sym_fn] = ACTIONS(4180),
    [anon_sym_for] = ACTIONS(4180),
    [anon_sym_gen] = ACTIONS(4180),
    [anon_sym_if] = ACTIONS(4180),
    [anon_sym_impl] = ACTIONS(4180),
    [anon_sym_let] = ACTIONS(4180),
    [anon_sym_loop] = ACTIONS(4180),
    [anon_sym_match] = ACTIONS(4180),
    [anon_sym_mod] = ACTIONS(4180),
    [anon_sym_pub] = ACTIONS(4180),
    [anon_sym_return] = ACTIONS(4180),
    [anon_sym_raw] = ACTIONS(4180),
    [anon_sym_static] = ACTIONS(4180),
    [anon_sym_struct] = ACTIONS(4180),
    [anon_sym_trait] = ACTIONS(4180),
    [anon_sym_type] = ACTIONS(4180),
    [anon_sym_union] = ACTIONS(4180),
    [anon_sym_unsafe] = ACTIONS(4180),
    [anon_sym_use] = ACTIONS(4180),
    [anon_sym_while] = ACTIONS(4180),
    [anon_sym_extern] = ACTIONS(4180),
    [anon_sym_safe] = ACTIONS(4180),
    [anon_sym_yield] = ACTIONS(4180),
    [anon_sym_move] = ACTIONS(4180),
    [anon_sym_try] = ACTIONS(4180),
    [sym_integer_literal] = ACTIONS(4178),
    [aux_sym_string_literal_token1] = ACTIONS(4178),
    [sym_char_literal] = ACTIONS(4178),
    [anon_sym_true] = ACTIONS(4180),
    [anon_sym_false] = ACTIONS(4180),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4180),
    [sym_super] = ACTIONS(4180),
    [sym_crate] = ACTIONS(4180),
    [aux_sym_metavariable_token1] = ACTIONS(4178),
    [sym__raw_string_literal_start] = ACTIONS(4178),
    [sym_float_literal] = ACTIONS(4178),
    [sym__outer_block_doc_comment_start] = ACTIONS(4178),
    [sym__outer_line_doc_comment_start] = ACTIONS(4178),
  },
  [STATE(1102)] = {
    [sym_line_comment] = STATE(1102),
    [sym_block_comment] = STATE(1102),
    [ts_builtin_sym_end] = ACTIONS(4182),
    [sym_identifier] = ACTIONS(4184),
    [anon_sym_SEMI] = ACTIONS(4182),
    [anon_sym_macro] = ACTIONS(4184),
    [anon_sym_LPAREN] = ACTIONS(4182),
    [anon_sym_LBRACE] = ACTIONS(4182),
    [anon_sym_RBRACE] = ACTIONS(4182),
    [anon_sym_macro_rules] = ACTIONS(4184),
    [anon_sym_BANG] = ACTIONS(4182),
    [anon_sym_LBRACK] = ACTIONS(4182),
    [anon_sym_STAR] = ACTIONS(4182),
    [anon_sym_u8] = ACTIONS(4184),
    [anon_sym_i8] = ACTIONS(4184),
    [anon_sym_u16] = ACTIONS(4184),
    [anon_sym_i16] = ACTIONS(4184),
    [anon_sym_u32] = ACTIONS(4184),
    [anon_sym_i32] = ACTIONS(4184),
    [anon_sym_u64] = ACTIONS(4184),
    [anon_sym_i64] = ACTIONS(4184),
    [anon_sym_u128] = ACTIONS(4184),
    [anon_sym_i128] = ACTIONS(4184),
    [anon_sym_isize] = ACTIONS(4184),
    [anon_sym_usize] = ACTIONS(4184),
    [anon_sym_f32] = ACTIONS(4184),
    [anon_sym_f64] = ACTIONS(4184),
    [anon_sym_bool] = ACTIONS(4184),
    [anon_sym_str] = ACTIONS(4184),
    [anon_sym_char] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4182),
    [anon_sym_AMP] = ACTIONS(4182),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_LT] = ACTIONS(4182),
    [anon_sym_DOT_DOT] = ACTIONS(4182),
    [anon_sym_COLON_COLON] = ACTIONS(4182),
    [anon_sym_POUND] = ACTIONS(4182),
    [anon_sym_SQUOTE] = ACTIONS(4184),
    [anon_sym_async] = ACTIONS(4184),
    [anon_sym_auto] = ACTIONS(4184),
    [anon_sym_await] = ACTIONS(4184),
    [anon_sym_become] = ACTIONS(4184),
    [anon_sym_break] = ACTIONS(4184),
    [anon_sym_const] = ACTIONS(4184),
    [anon_sym_continue] = ACTIONS(4184),
    [anon_sym_default] = ACTIONS(4184),
    [anon_sym_enum] = ACTIONS(4184),
    [anon_sym_fn] = ACTIONS(4184),
    [anon_sym_for] = ACTIONS(4184),
    [anon_sym_gen] = ACTIONS(4184),
    [anon_sym_if] = ACTIONS(4184),
    [anon_sym_impl] = ACTIONS(4184),
    [anon_sym_let] = ACTIONS(4184),
    [anon_sym_loop] = ACTIONS(4184),
    [anon_sym_match] = ACTIONS(4184),
    [anon_sym_mod] = ACTIONS(4184),
    [anon_sym_pub] = ACTIONS(4184),
    [anon_sym_return] = ACTIONS(4184),
    [anon_sym_raw] = ACTIONS(4184),
    [anon_sym_static] = ACTIONS(4184),
    [anon_sym_struct] = ACTIONS(4184),
    [anon_sym_trait] = ACTIONS(4184),
    [anon_sym_type] = ACTIONS(4184),
    [anon_sym_union] = ACTIONS(4184),
    [anon_sym_unsafe] = ACTIONS(4184),
    [anon_sym_use] = ACTIONS(4184),
    [anon_sym_while] = ACTIONS(4184),
    [anon_sym_extern] = ACTIONS(4184),
    [anon_sym_safe] = ACTIONS(4184),
    [anon_sym_yield] = ACTIONS(4184),
    [anon_sym_move] = ACTIONS(4184),
    [anon_sym_try] = ACTIONS(4184),
    [sym_integer_literal] = ACTIONS(4182),
    [aux_sym_string_literal_token1] = ACTIONS(4182),
    [sym_char_literal] = ACTIONS(4182),
    [anon_sym_true] = ACTIONS(4184),
    [anon_sym_false] = ACTIONS(4184),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4184),
    [sym_super] = ACTIONS(4184),
    [sym_crate] = ACTIONS(4184),
    [aux_sym_metavariable_token1] = ACTIONS(4182),
    [sym__raw_string_literal_start] = ACTIONS(4182),
    [sym_float_literal] = ACTIONS(4182),
    [sym__outer_block_doc_comment_start] = ACTIONS(4182),
    [sym__outer_line_doc_comment_start] = ACTIONS(4182),
  },
  [STATE(1103)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3893),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1103),
    [sym_block_comment] = STATE(1103),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(2358),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(2358),
    [anon_sym_COMMA] = ACTIONS(2358),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(2358),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_EQ] = ACTIONS(2358),
    [anon_sym_GT] = ACTIONS(2358),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(2360),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1104)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3883),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1104),
    [sym_block_comment] = STATE(1104),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(2358),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(2358),
    [anon_sym_COMMA] = ACTIONS(2358),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(2358),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_EQ] = ACTIONS(2358),
    [anon_sym_GT] = ACTIONS(2358),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(2360),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1105)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3898),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1105),
    [sym_block_comment] = STATE(1105),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(2358),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(2358),
    [anon_sym_COMMA] = ACTIONS(2358),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(2358),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_EQ] = ACTIONS(2358),
    [anon_sym_GT] = ACTIONS(2358),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(2360),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1106)] = {
    [sym_line_comment] = STATE(1106),
    [sym_block_comment] = STATE(1106),
    [ts_builtin_sym_end] = ACTIONS(4186),
    [sym_identifier] = ACTIONS(4188),
    [anon_sym_SEMI] = ACTIONS(4186),
    [anon_sym_macro] = ACTIONS(4188),
    [anon_sym_LPAREN] = ACTIONS(4186),
    [anon_sym_LBRACE] = ACTIONS(4186),
    [anon_sym_RBRACE] = ACTIONS(4186),
    [anon_sym_macro_rules] = ACTIONS(4188),
    [anon_sym_BANG] = ACTIONS(4186),
    [anon_sym_LBRACK] = ACTIONS(4186),
    [anon_sym_STAR] = ACTIONS(4186),
    [anon_sym_u8] = ACTIONS(4188),
    [anon_sym_i8] = ACTIONS(4188),
    [anon_sym_u16] = ACTIONS(4188),
    [anon_sym_i16] = ACTIONS(4188),
    [anon_sym_u32] = ACTIONS(4188),
    [anon_sym_i32] = ACTIONS(4188),
    [anon_sym_u64] = ACTIONS(4188),
    [anon_sym_i64] = ACTIONS(4188),
    [anon_sym_u128] = ACTIONS(4188),
    [anon_sym_i128] = ACTIONS(4188),
    [anon_sym_isize] = ACTIONS(4188),
    [anon_sym_usize] = ACTIONS(4188),
    [anon_sym_f32] = ACTIONS(4188),
    [anon_sym_f64] = ACTIONS(4188),
    [anon_sym_bool] = ACTIONS(4188),
    [anon_sym_str] = ACTIONS(4188),
    [anon_sym_char] = ACTIONS(4188),
    [anon_sym_DASH] = ACTIONS(4186),
    [anon_sym_AMP] = ACTIONS(4186),
    [anon_sym_PIPE] = ACTIONS(4186),
    [anon_sym_LT] = ACTIONS(4186),
    [anon_sym_DOT_DOT] = ACTIONS(4186),
    [anon_sym_COLON_COLON] = ACTIONS(4186),
    [anon_sym_POUND] = ACTIONS(4186),
    [anon_sym_SQUOTE] = ACTIONS(4188),
    [anon_sym_async] = ACTIONS(4188),
    [anon_sym_auto] = ACTIONS(4188),
    [anon_sym_await] = ACTIONS(4188),
    [anon_sym_become] = ACTIONS(4188),
    [anon_sym_break] = ACTIONS(4188),
    [anon_sym_const] = ACTIONS(4188),
    [anon_sym_continue] = ACTIONS(4188),
    [anon_sym_default] = ACTIONS(4188),
    [anon_sym_enum] = ACTIONS(4188),
    [anon_sym_fn] = ACTIONS(4188),
    [anon_sym_for] = ACTIONS(4188),
    [anon_sym_gen] = ACTIONS(4188),
    [anon_sym_if] = ACTIONS(4188),
    [anon_sym_impl] = ACTIONS(4188),
    [anon_sym_let] = ACTIONS(4188),
    [anon_sym_loop] = ACTIONS(4188),
    [anon_sym_match] = ACTIONS(4188),
    [anon_sym_mod] = ACTIONS(4188),
    [anon_sym_pub] = ACTIONS(4188),
    [anon_sym_return] = ACTIONS(4188),
    [anon_sym_raw] = ACTIONS(4188),
    [anon_sym_static] = ACTIONS(4188),
    [anon_sym_struct] = ACTIONS(4188),
    [anon_sym_trait] = ACTIONS(4188),
    [anon_sym_type] = ACTIONS(4188),
    [anon_sym_union] = ACTIONS(4188),
    [anon_sym_unsafe] = ACTIONS(4188),
    [anon_sym_use] = ACTIONS(4188),
    [anon_sym_while] = ACTIONS(4188),
    [anon_sym_extern] = ACTIONS(4188),
    [anon_sym_safe] = ACTIONS(4188),
    [anon_sym_yield] = ACTIONS(4188),
    [anon_sym_move] = ACTIONS(4188),
    [anon_sym_try] = ACTIONS(4188),
    [sym_integer_literal] = ACTIONS(4186),
    [aux_sym_string_literal_token1] = ACTIONS(4186),
    [sym_char_literal] = ACTIONS(4186),
    [anon_sym_true] = ACTIONS(4188),
    [anon_sym_false] = ACTIONS(4188),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4188),
    [sym_super] = ACTIONS(4188),
    [sym_crate] = ACTIONS(4188),
    [aux_sym_metavariable_token1] = ACTIONS(4186),
    [sym__raw_string_literal_start] = ACTIONS(4186),
    [sym_float_literal] = ACTIONS(4186),
    [sym__outer_block_doc_comment_start] = ACTIONS(4186),
    [sym__outer_line_doc_comment_start] = ACTIONS(4186),
  },
  [STATE(1107)] = {
    [sym_line_comment] = STATE(1107),
    [sym_block_comment] = STATE(1107),
    [ts_builtin_sym_end] = ACTIONS(4190),
    [sym_identifier] = ACTIONS(4192),
    [anon_sym_SEMI] = ACTIONS(4190),
    [anon_sym_macro] = ACTIONS(4192),
    [anon_sym_LPAREN] = ACTIONS(4190),
    [anon_sym_LBRACE] = ACTIONS(4190),
    [anon_sym_RBRACE] = ACTIONS(4190),
    [anon_sym_macro_rules] = ACTIONS(4192),
    [anon_sym_BANG] = ACTIONS(4190),
    [anon_sym_LBRACK] = ACTIONS(4190),
    [anon_sym_STAR] = ACTIONS(4190),
    [anon_sym_u8] = ACTIONS(4192),
    [anon_sym_i8] = ACTIONS(4192),
    [anon_sym_u16] = ACTIONS(4192),
    [anon_sym_i16] = ACTIONS(4192),
    [anon_sym_u32] = ACTIONS(4192),
    [anon_sym_i32] = ACTIONS(4192),
    [anon_sym_u64] = ACTIONS(4192),
    [anon_sym_i64] = ACTIONS(4192),
    [anon_sym_u128] = ACTIONS(4192),
    [anon_sym_i128] = ACTIONS(4192),
    [anon_sym_isize] = ACTIONS(4192),
    [anon_sym_usize] = ACTIONS(4192),
    [anon_sym_f32] = ACTIONS(4192),
    [anon_sym_f64] = ACTIONS(4192),
    [anon_sym_bool] = ACTIONS(4192),
    [anon_sym_str] = ACTIONS(4192),
    [anon_sym_char] = ACTIONS(4192),
    [anon_sym_DASH] = ACTIONS(4190),
    [anon_sym_AMP] = ACTIONS(4190),
    [anon_sym_PIPE] = ACTIONS(4190),
    [anon_sym_LT] = ACTIONS(4190),
    [anon_sym_DOT_DOT] = ACTIONS(4190),
    [anon_sym_COLON_COLON] = ACTIONS(4190),
    [anon_sym_POUND] = ACTIONS(4190),
    [anon_sym_SQUOTE] = ACTIONS(4192),
    [anon_sym_async] = ACTIONS(4192),
    [anon_sym_auto] = ACTIONS(4192),
    [anon_sym_await] = ACTIONS(4192),
    [anon_sym_become] = ACTIONS(4192),
    [anon_sym_break] = ACTIONS(4192),
    [anon_sym_const] = ACTIONS(4192),
    [anon_sym_continue] = ACTIONS(4192),
    [anon_sym_default] = ACTIONS(4192),
    [anon_sym_enum] = ACTIONS(4192),
    [anon_sym_fn] = ACTIONS(4192),
    [anon_sym_for] = ACTIONS(4192),
    [anon_sym_gen] = ACTIONS(4192),
    [anon_sym_if] = ACTIONS(4192),
    [anon_sym_impl] = ACTIONS(4192),
    [anon_sym_let] = ACTIONS(4192),
    [anon_sym_loop] = ACTIONS(4192),
    [anon_sym_match] = ACTIONS(4192),
    [anon_sym_mod] = ACTIONS(4192),
    [anon_sym_pub] = ACTIONS(4192),
    [anon_sym_return] = ACTIONS(4192),
    [anon_sym_raw] = ACTIONS(4192),
    [anon_sym_static] = ACTIONS(4192),
    [anon_sym_struct] = ACTIONS(4192),
    [anon_sym_trait] = ACTIONS(4192),
    [anon_sym_type] = ACTIONS(4192),
    [anon_sym_union] = ACTIONS(4192),
    [anon_sym_unsafe] = ACTIONS(4192),
    [anon_sym_use] = ACTIONS(4192),
    [anon_sym_while] = ACTIONS(4192),
    [anon_sym_extern] = ACTIONS(4192),
    [anon_sym_safe] = ACTIONS(4192),
    [anon_sym_yield] = ACTIONS(4192),
    [anon_sym_move] = ACTIONS(4192),
    [anon_sym_try] = ACTIONS(4192),
    [sym_integer_literal] = ACTIONS(4190),
    [aux_sym_string_literal_token1] = ACTIONS(4190),
    [sym_char_literal] = ACTIONS(4190),
    [anon_sym_true] = ACTIONS(4192),
    [anon_sym_false] = ACTIONS(4192),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4192),
    [sym_super] = ACTIONS(4192),
    [sym_crate] = ACTIONS(4192),
    [aux_sym_metavariable_token1] = ACTIONS(4190),
    [sym__raw_string_literal_start] = ACTIONS(4190),
    [sym_float_literal] = ACTIONS(4190),
    [sym__outer_block_doc_comment_start] = ACTIONS(4190),
    [sym__outer_line_doc_comment_start] = ACTIONS(4190),
  },
  [STATE(1108)] = {
    [sym_line_comment] = STATE(1108),
    [sym_block_comment] = STATE(1108),
    [ts_builtin_sym_end] = ACTIONS(4194),
    [sym_identifier] = ACTIONS(4196),
    [anon_sym_SEMI] = ACTIONS(4194),
    [anon_sym_macro] = ACTIONS(4196),
    [anon_sym_LPAREN] = ACTIONS(4194),
    [anon_sym_LBRACE] = ACTIONS(4194),
    [anon_sym_RBRACE] = ACTIONS(4194),
    [anon_sym_macro_rules] = ACTIONS(4196),
    [anon_sym_BANG] = ACTIONS(4194),
    [anon_sym_LBRACK] = ACTIONS(4194),
    [anon_sym_STAR] = ACTIONS(4194),
    [anon_sym_u8] = ACTIONS(4196),
    [anon_sym_i8] = ACTIONS(4196),
    [anon_sym_u16] = ACTIONS(4196),
    [anon_sym_i16] = ACTIONS(4196),
    [anon_sym_u32] = ACTIONS(4196),
    [anon_sym_i32] = ACTIONS(4196),
    [anon_sym_u64] = ACTIONS(4196),
    [anon_sym_i64] = ACTIONS(4196),
    [anon_sym_u128] = ACTIONS(4196),
    [anon_sym_i128] = ACTIONS(4196),
    [anon_sym_isize] = ACTIONS(4196),
    [anon_sym_usize] = ACTIONS(4196),
    [anon_sym_f32] = ACTIONS(4196),
    [anon_sym_f64] = ACTIONS(4196),
    [anon_sym_bool] = ACTIONS(4196),
    [anon_sym_str] = ACTIONS(4196),
    [anon_sym_char] = ACTIONS(4196),
    [anon_sym_DASH] = ACTIONS(4194),
    [anon_sym_AMP] = ACTIONS(4194),
    [anon_sym_PIPE] = ACTIONS(4194),
    [anon_sym_LT] = ACTIONS(4194),
    [anon_sym_DOT_DOT] = ACTIONS(4194),
    [anon_sym_COLON_COLON] = ACTIONS(4194),
    [anon_sym_POUND] = ACTIONS(4194),
    [anon_sym_SQUOTE] = ACTIONS(4196),
    [anon_sym_async] = ACTIONS(4196),
    [anon_sym_auto] = ACTIONS(4196),
    [anon_sym_await] = ACTIONS(4196),
    [anon_sym_become] = ACTIONS(4196),
    [anon_sym_break] = ACTIONS(4196),
    [anon_sym_const] = ACTIONS(4196),
    [anon_sym_continue] = ACTIONS(4196),
    [anon_sym_default] = ACTIONS(4196),
    [anon_sym_enum] = ACTIONS(4196),
    [anon_sym_fn] = ACTIONS(4196),
    [anon_sym_for] = ACTIONS(4196),
    [anon_sym_gen] = ACTIONS(4196),
    [anon_sym_if] = ACTIONS(4196),
    [anon_sym_impl] = ACTIONS(4196),
    [anon_sym_let] = ACTIONS(4196),
    [anon_sym_loop] = ACTIONS(4196),
    [anon_sym_match] = ACTIONS(4196),
    [anon_sym_mod] = ACTIONS(4196),
    [anon_sym_pub] = ACTIONS(4196),
    [anon_sym_return] = ACTIONS(4196),
    [anon_sym_raw] = ACTIONS(4196),
    [anon_sym_static] = ACTIONS(4196),
    [anon_sym_struct] = ACTIONS(4196),
    [anon_sym_trait] = ACTIONS(4196),
    [anon_sym_type] = ACTIONS(4196),
    [anon_sym_union] = ACTIONS(4196),
    [anon_sym_unsafe] = ACTIONS(4196),
    [anon_sym_use] = ACTIONS(4196),
    [anon_sym_while] = ACTIONS(4196),
    [anon_sym_extern] = ACTIONS(4196),
    [anon_sym_safe] = ACTIONS(4196),
    [anon_sym_yield] = ACTIONS(4196),
    [anon_sym_move] = ACTIONS(4196),
    [anon_sym_try] = ACTIONS(4196),
    [sym_integer_literal] = ACTIONS(4194),
    [aux_sym_string_literal_token1] = ACTIONS(4194),
    [sym_char_literal] = ACTIONS(4194),
    [anon_sym_true] = ACTIONS(4196),
    [anon_sym_false] = ACTIONS(4196),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4196),
    [sym_super] = ACTIONS(4196),
    [sym_crate] = ACTIONS(4196),
    [aux_sym_metavariable_token1] = ACTIONS(4194),
    [sym__raw_string_literal_start] = ACTIONS(4194),
    [sym_float_literal] = ACTIONS(4194),
    [sym__outer_block_doc_comment_start] = ACTIONS(4194),
    [sym__outer_line_doc_comment_start] = ACTIONS(4194),
  },
  [STATE(1109)] = {
    [sym_line_comment] = STATE(1109),
    [sym_block_comment] = STATE(1109),
    [ts_builtin_sym_end] = ACTIONS(4198),
    [sym_identifier] = ACTIONS(4200),
    [anon_sym_SEMI] = ACTIONS(4198),
    [anon_sym_macro] = ACTIONS(4200),
    [anon_sym_LPAREN] = ACTIONS(4198),
    [anon_sym_LBRACE] = ACTIONS(4198),
    [anon_sym_RBRACE] = ACTIONS(4198),
    [anon_sym_macro_rules] = ACTIONS(4200),
    [anon_sym_BANG] = ACTIONS(4198),
    [anon_sym_LBRACK] = ACTIONS(4198),
    [anon_sym_STAR] = ACTIONS(4198),
    [anon_sym_u8] = ACTIONS(4200),
    [anon_sym_i8] = ACTIONS(4200),
    [anon_sym_u16] = ACTIONS(4200),
    [anon_sym_i16] = ACTIONS(4200),
    [anon_sym_u32] = ACTIONS(4200),
    [anon_sym_i32] = ACTIONS(4200),
    [anon_sym_u64] = ACTIONS(4200),
    [anon_sym_i64] = ACTIONS(4200),
    [anon_sym_u128] = ACTIONS(4200),
    [anon_sym_i128] = ACTIONS(4200),
    [anon_sym_isize] = ACTIONS(4200),
    [anon_sym_usize] = ACTIONS(4200),
    [anon_sym_f32] = ACTIONS(4200),
    [anon_sym_f64] = ACTIONS(4200),
    [anon_sym_bool] = ACTIONS(4200),
    [anon_sym_str] = ACTIONS(4200),
    [anon_sym_char] = ACTIONS(4200),
    [anon_sym_DASH] = ACTIONS(4198),
    [anon_sym_AMP] = ACTIONS(4198),
    [anon_sym_PIPE] = ACTIONS(4198),
    [anon_sym_LT] = ACTIONS(4198),
    [anon_sym_DOT_DOT] = ACTIONS(4198),
    [anon_sym_COLON_COLON] = ACTIONS(4198),
    [anon_sym_POUND] = ACTIONS(4198),
    [anon_sym_SQUOTE] = ACTIONS(4200),
    [anon_sym_async] = ACTIONS(4200),
    [anon_sym_auto] = ACTIONS(4200),
    [anon_sym_await] = ACTIONS(4200),
    [anon_sym_become] = ACTIONS(4200),
    [anon_sym_break] = ACTIONS(4200),
    [anon_sym_const] = ACTIONS(4200),
    [anon_sym_continue] = ACTIONS(4200),
    [anon_sym_default] = ACTIONS(4200),
    [anon_sym_enum] = ACTIONS(4200),
    [anon_sym_fn] = ACTIONS(4200),
    [anon_sym_for] = ACTIONS(4200),
    [anon_sym_gen] = ACTIONS(4200),
    [anon_sym_if] = ACTIONS(4200),
    [anon_sym_impl] = ACTIONS(4200),
    [anon_sym_let] = ACTIONS(4200),
    [anon_sym_loop] = ACTIONS(4200),
    [anon_sym_match] = ACTIONS(4200),
    [anon_sym_mod] = ACTIONS(4200),
    [anon_sym_pub] = ACTIONS(4200),
    [anon_sym_return] = ACTIONS(4200),
    [anon_sym_raw] = ACTIONS(4200),
    [anon_sym_static] = ACTIONS(4200),
    [anon_sym_struct] = ACTIONS(4200),
    [anon_sym_trait] = ACTIONS(4200),
    [anon_sym_type] = ACTIONS(4200),
    [anon_sym_union] = ACTIONS(4200),
    [anon_sym_unsafe] = ACTIONS(4200),
    [anon_sym_use] = ACTIONS(4200),
    [anon_sym_while] = ACTIONS(4200),
    [anon_sym_extern] = ACTIONS(4200),
    [anon_sym_safe] = ACTIONS(4200),
    [anon_sym_yield] = ACTIONS(4200),
    [anon_sym_move] = ACTIONS(4200),
    [anon_sym_try] = ACTIONS(4200),
    [sym_integer_literal] = ACTIONS(4198),
    [aux_sym_string_literal_token1] = ACTIONS(4198),
    [sym_char_literal] = ACTIONS(4198),
    [anon_sym_true] = ACTIONS(4200),
    [anon_sym_false] = ACTIONS(4200),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4200),
    [sym_super] = ACTIONS(4200),
    [sym_crate] = ACTIONS(4200),
    [aux_sym_metavariable_token1] = ACTIONS(4198),
    [sym__raw_string_literal_start] = ACTIONS(4198),
    [sym_float_literal] = ACTIONS(4198),
    [sym__outer_block_doc_comment_start] = ACTIONS(4198),
    [sym__outer_line_doc_comment_start] = ACTIONS(4198),
  },
  [STATE(1110)] = {
    [sym_line_comment] = STATE(1110),
    [sym_block_comment] = STATE(1110),
    [ts_builtin_sym_end] = ACTIONS(4202),
    [sym_identifier] = ACTIONS(4204),
    [anon_sym_SEMI] = ACTIONS(4202),
    [anon_sym_macro] = ACTIONS(4204),
    [anon_sym_LPAREN] = ACTIONS(4202),
    [anon_sym_LBRACE] = ACTIONS(4202),
    [anon_sym_RBRACE] = ACTIONS(4202),
    [anon_sym_macro_rules] = ACTIONS(4204),
    [anon_sym_BANG] = ACTIONS(4202),
    [anon_sym_LBRACK] = ACTIONS(4202),
    [anon_sym_STAR] = ACTIONS(4202),
    [anon_sym_u8] = ACTIONS(4204),
    [anon_sym_i8] = ACTIONS(4204),
    [anon_sym_u16] = ACTIONS(4204),
    [anon_sym_i16] = ACTIONS(4204),
    [anon_sym_u32] = ACTIONS(4204),
    [anon_sym_i32] = ACTIONS(4204),
    [anon_sym_u64] = ACTIONS(4204),
    [anon_sym_i64] = ACTIONS(4204),
    [anon_sym_u128] = ACTIONS(4204),
    [anon_sym_i128] = ACTIONS(4204),
    [anon_sym_isize] = ACTIONS(4204),
    [anon_sym_usize] = ACTIONS(4204),
    [anon_sym_f32] = ACTIONS(4204),
    [anon_sym_f64] = ACTIONS(4204),
    [anon_sym_bool] = ACTIONS(4204),
    [anon_sym_str] = ACTIONS(4204),
    [anon_sym_char] = ACTIONS(4204),
    [anon_sym_DASH] = ACTIONS(4202),
    [anon_sym_AMP] = ACTIONS(4202),
    [anon_sym_PIPE] = ACTIONS(4202),
    [anon_sym_LT] = ACTIONS(4202),
    [anon_sym_DOT_DOT] = ACTIONS(4202),
    [anon_sym_COLON_COLON] = ACTIONS(4202),
    [anon_sym_POUND] = ACTIONS(4202),
    [anon_sym_SQUOTE] = ACTIONS(4204),
    [anon_sym_async] = ACTIONS(4204),
    [anon_sym_auto] = ACTIONS(4204),
    [anon_sym_await] = ACTIONS(4204),
    [anon_sym_become] = ACTIONS(4204),
    [anon_sym_break] = ACTIONS(4204),
    [anon_sym_const] = ACTIONS(4204),
    [anon_sym_continue] = ACTIONS(4204),
    [anon_sym_default] = ACTIONS(4204),
    [anon_sym_enum] = ACTIONS(4204),
    [anon_sym_fn] = ACTIONS(4204),
    [anon_sym_for] = ACTIONS(4204),
    [anon_sym_gen] = ACTIONS(4204),
    [anon_sym_if] = ACTIONS(4204),
    [anon_sym_impl] = ACTIONS(4204),
    [anon_sym_let] = ACTIONS(4204),
    [anon_sym_loop] = ACTIONS(4204),
    [anon_sym_match] = ACTIONS(4204),
    [anon_sym_mod] = ACTIONS(4204),
    [anon_sym_pub] = ACTIONS(4204),
    [anon_sym_return] = ACTIONS(4204),
    [anon_sym_raw] = ACTIONS(4204),
    [anon_sym_static] = ACTIONS(4204),
    [anon_sym_struct] = ACTIONS(4204),
    [anon_sym_trait] = ACTIONS(4204),
    [anon_sym_type] = ACTIONS(4204),
    [anon_sym_union] = ACTIONS(4204),
    [anon_sym_unsafe] = ACTIONS(4204),
    [anon_sym_use] = ACTIONS(4204),
    [anon_sym_while] = ACTIONS(4204),
    [anon_sym_extern] = ACTIONS(4204),
    [anon_sym_safe] = ACTIONS(4204),
    [anon_sym_yield] = ACTIONS(4204),
    [anon_sym_move] = ACTIONS(4204),
    [anon_sym_try] = ACTIONS(4204),
    [sym_integer_literal] = ACTIONS(4202),
    [aux_sym_string_literal_token1] = ACTIONS(4202),
    [sym_char_literal] = ACTIONS(4202),
    [anon_sym_true] = ACTIONS(4204),
    [anon_sym_false] = ACTIONS(4204),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4204),
    [sym_super] = ACTIONS(4204),
    [sym_crate] = ACTIONS(4204),
    [aux_sym_metavariable_token1] = ACTIONS(4202),
    [sym__raw_string_literal_start] = ACTIONS(4202),
    [sym_float_literal] = ACTIONS(4202),
    [sym__outer_block_doc_comment_start] = ACTIONS(4202),
    [sym__outer_line_doc_comment_start] = ACTIONS(4202),
  },
  [STATE(1111)] = {
    [sym_line_comment] = STATE(1111),
    [sym_block_comment] = STATE(1111),
    [ts_builtin_sym_end] = ACTIONS(4206),
    [sym_identifier] = ACTIONS(4208),
    [anon_sym_SEMI] = ACTIONS(4206),
    [anon_sym_macro] = ACTIONS(4208),
    [anon_sym_LPAREN] = ACTIONS(4206),
    [anon_sym_LBRACE] = ACTIONS(4206),
    [anon_sym_RBRACE] = ACTIONS(4206),
    [anon_sym_macro_rules] = ACTIONS(4208),
    [anon_sym_BANG] = ACTIONS(4206),
    [anon_sym_LBRACK] = ACTIONS(4206),
    [anon_sym_STAR] = ACTIONS(4206),
    [anon_sym_u8] = ACTIONS(4208),
    [anon_sym_i8] = ACTIONS(4208),
    [anon_sym_u16] = ACTIONS(4208),
    [anon_sym_i16] = ACTIONS(4208),
    [anon_sym_u32] = ACTIONS(4208),
    [anon_sym_i32] = ACTIONS(4208),
    [anon_sym_u64] = ACTIONS(4208),
    [anon_sym_i64] = ACTIONS(4208),
    [anon_sym_u128] = ACTIONS(4208),
    [anon_sym_i128] = ACTIONS(4208),
    [anon_sym_isize] = ACTIONS(4208),
    [anon_sym_usize] = ACTIONS(4208),
    [anon_sym_f32] = ACTIONS(4208),
    [anon_sym_f64] = ACTIONS(4208),
    [anon_sym_bool] = ACTIONS(4208),
    [anon_sym_str] = ACTIONS(4208),
    [anon_sym_char] = ACTIONS(4208),
    [anon_sym_DASH] = ACTIONS(4206),
    [anon_sym_AMP] = ACTIONS(4206),
    [anon_sym_PIPE] = ACTIONS(4206),
    [anon_sym_LT] = ACTIONS(4206),
    [anon_sym_DOT_DOT] = ACTIONS(4206),
    [anon_sym_COLON_COLON] = ACTIONS(4206),
    [anon_sym_POUND] = ACTIONS(4206),
    [anon_sym_SQUOTE] = ACTIONS(4208),
    [anon_sym_async] = ACTIONS(4208),
    [anon_sym_auto] = ACTIONS(4208),
    [anon_sym_await] = ACTIONS(4208),
    [anon_sym_become] = ACTIONS(4208),
    [anon_sym_break] = ACTIONS(4208),
    [anon_sym_const] = ACTIONS(4208),
    [anon_sym_continue] = ACTIONS(4208),
    [anon_sym_default] = ACTIONS(4208),
    [anon_sym_enum] = ACTIONS(4208),
    [anon_sym_fn] = ACTIONS(4208),
    [anon_sym_for] = ACTIONS(4208),
    [anon_sym_gen] = ACTIONS(4208),
    [anon_sym_if] = ACTIONS(4208),
    [anon_sym_impl] = ACTIONS(4208),
    [anon_sym_let] = ACTIONS(4208),
    [anon_sym_loop] = ACTIONS(4208),
    [anon_sym_match] = ACTIONS(4208),
    [anon_sym_mod] = ACTIONS(4208),
    [anon_sym_pub] = ACTIONS(4208),
    [anon_sym_return] = ACTIONS(4208),
    [anon_sym_raw] = ACTIONS(4208),
    [anon_sym_static] = ACTIONS(4208),
    [anon_sym_struct] = ACTIONS(4208),
    [anon_sym_trait] = ACTIONS(4208),
    [anon_sym_type] = ACTIONS(4208),
    [anon_sym_union] = ACTIONS(4208),
    [anon_sym_unsafe] = ACTIONS(4208),
    [anon_sym_use] = ACTIONS(4208),
    [anon_sym_while] = ACTIONS(4208),
    [anon_sym_extern] = ACTIONS(4208),
    [anon_sym_safe] = ACTIONS(4208),
    [anon_sym_yield] = ACTIONS(4208),
    [anon_sym_move] = ACTIONS(4208),
    [anon_sym_try] = ACTIONS(4208),
    [sym_integer_literal] = ACTIONS(4206),
    [aux_sym_string_literal_token1] = ACTIONS(4206),
    [sym_char_literal] = ACTIONS(4206),
    [anon_sym_true] = ACTIONS(4208),
    [anon_sym_false] = ACTIONS(4208),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4208),
    [sym_super] = ACTIONS(4208),
    [sym_crate] = ACTIONS(4208),
    [aux_sym_metavariable_token1] = ACTIONS(4206),
    [sym__raw_string_literal_start] = ACTIONS(4206),
    [sym_float_literal] = ACTIONS(4206),
    [sym__outer_block_doc_comment_start] = ACTIONS(4206),
    [sym__outer_line_doc_comment_start] = ACTIONS(4206),
  },
  [STATE(1112)] = {
    [sym_line_comment] = STATE(1112),
    [sym_block_comment] = STATE(1112),
    [ts_builtin_sym_end] = ACTIONS(4210),
    [sym_identifier] = ACTIONS(4212),
    [anon_sym_SEMI] = ACTIONS(4210),
    [anon_sym_macro] = ACTIONS(4212),
    [anon_sym_LPAREN] = ACTIONS(4210),
    [anon_sym_LBRACE] = ACTIONS(4210),
    [anon_sym_RBRACE] = ACTIONS(4210),
    [anon_sym_macro_rules] = ACTIONS(4212),
    [anon_sym_BANG] = ACTIONS(4210),
    [anon_sym_LBRACK] = ACTIONS(4210),
    [anon_sym_STAR] = ACTIONS(4210),
    [anon_sym_u8] = ACTIONS(4212),
    [anon_sym_i8] = ACTIONS(4212),
    [anon_sym_u16] = ACTIONS(4212),
    [anon_sym_i16] = ACTIONS(4212),
    [anon_sym_u32] = ACTIONS(4212),
    [anon_sym_i32] = ACTIONS(4212),
    [anon_sym_u64] = ACTIONS(4212),
    [anon_sym_i64] = ACTIONS(4212),
    [anon_sym_u128] = ACTIONS(4212),
    [anon_sym_i128] = ACTIONS(4212),
    [anon_sym_isize] = ACTIONS(4212),
    [anon_sym_usize] = ACTIONS(4212),
    [anon_sym_f32] = ACTIONS(4212),
    [anon_sym_f64] = ACTIONS(4212),
    [anon_sym_bool] = ACTIONS(4212),
    [anon_sym_str] = ACTIONS(4212),
    [anon_sym_char] = ACTIONS(4212),
    [anon_sym_DASH] = ACTIONS(4210),
    [anon_sym_AMP] = ACTIONS(4210),
    [anon_sym_PIPE] = ACTIONS(4210),
    [anon_sym_LT] = ACTIONS(4210),
    [anon_sym_DOT_DOT] = ACTIONS(4210),
    [anon_sym_COLON_COLON] = ACTIONS(4210),
    [anon_sym_POUND] = ACTIONS(4210),
    [anon_sym_SQUOTE] = ACTIONS(4212),
    [anon_sym_async] = ACTIONS(4212),
    [anon_sym_auto] = ACTIONS(4212),
    [anon_sym_await] = ACTIONS(4212),
    [anon_sym_become] = ACTIONS(4212),
    [anon_sym_break] = ACTIONS(4212),
    [anon_sym_const] = ACTIONS(4212),
    [anon_sym_continue] = ACTIONS(4212),
    [anon_sym_default] = ACTIONS(4212),
    [anon_sym_enum] = ACTIONS(4212),
    [anon_sym_fn] = ACTIONS(4212),
    [anon_sym_for] = ACTIONS(4212),
    [anon_sym_gen] = ACTIONS(4212),
    [anon_sym_if] = ACTIONS(4212),
    [anon_sym_impl] = ACTIONS(4212),
    [anon_sym_let] = ACTIONS(4212),
    [anon_sym_loop] = ACTIONS(4212),
    [anon_sym_match] = ACTIONS(4212),
    [anon_sym_mod] = ACTIONS(4212),
    [anon_sym_pub] = ACTIONS(4212),
    [anon_sym_return] = ACTIONS(4212),
    [anon_sym_raw] = ACTIONS(4212),
    [anon_sym_static] = ACTIONS(4212),
    [anon_sym_struct] = ACTIONS(4212),
    [anon_sym_trait] = ACTIONS(4212),
    [anon_sym_type] = ACTIONS(4212),
    [anon_sym_union] = ACTIONS(4212),
    [anon_sym_unsafe] = ACTIONS(4212),
    [anon_sym_use] = ACTIONS(4212),
    [anon_sym_while] = ACTIONS(4212),
    [anon_sym_extern] = ACTIONS(4212),
    [anon_sym_safe] = ACTIONS(4212),
    [anon_sym_yield] = ACTIONS(4212),
    [anon_sym_move] = ACTIONS(4212),
    [anon_sym_try] = ACTIONS(4212),
    [sym_integer_literal] = ACTIONS(4210),
    [aux_sym_string_literal_token1] = ACTIONS(4210),
    [sym_char_literal] = ACTIONS(4210),
    [anon_sym_true] = ACTIONS(4212),
    [anon_sym_false] = ACTIONS(4212),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4212),
    [sym_super] = ACTIONS(4212),
    [sym_crate] = ACTIONS(4212),
    [aux_sym_metavariable_token1] = ACTIONS(4210),
    [sym__raw_string_literal_start] = ACTIONS(4210),
    [sym_float_literal] = ACTIONS(4210),
    [sym__outer_block_doc_comment_start] = ACTIONS(4210),
    [sym__outer_line_doc_comment_start] = ACTIONS(4210),
  },
  [STATE(1113)] = {
    [sym_line_comment] = STATE(1113),
    [sym_block_comment] = STATE(1113),
    [ts_builtin_sym_end] = ACTIONS(4214),
    [sym_identifier] = ACTIONS(4216),
    [anon_sym_SEMI] = ACTIONS(4214),
    [anon_sym_macro] = ACTIONS(4216),
    [anon_sym_LPAREN] = ACTIONS(4214),
    [anon_sym_LBRACE] = ACTIONS(4214),
    [anon_sym_RBRACE] = ACTIONS(4214),
    [anon_sym_macro_rules] = ACTIONS(4216),
    [anon_sym_BANG] = ACTIONS(4214),
    [anon_sym_LBRACK] = ACTIONS(4214),
    [anon_sym_STAR] = ACTIONS(4214),
    [anon_sym_u8] = ACTIONS(4216),
    [anon_sym_i8] = ACTIONS(4216),
    [anon_sym_u16] = ACTIONS(4216),
    [anon_sym_i16] = ACTIONS(4216),
    [anon_sym_u32] = ACTIONS(4216),
    [anon_sym_i32] = ACTIONS(4216),
    [anon_sym_u64] = ACTIONS(4216),
    [anon_sym_i64] = ACTIONS(4216),
    [anon_sym_u128] = ACTIONS(4216),
    [anon_sym_i128] = ACTIONS(4216),
    [anon_sym_isize] = ACTIONS(4216),
    [anon_sym_usize] = ACTIONS(4216),
    [anon_sym_f32] = ACTIONS(4216),
    [anon_sym_f64] = ACTIONS(4216),
    [anon_sym_bool] = ACTIONS(4216),
    [anon_sym_str] = ACTIONS(4216),
    [anon_sym_char] = ACTIONS(4216),
    [anon_sym_DASH] = ACTIONS(4214),
    [anon_sym_AMP] = ACTIONS(4214),
    [anon_sym_PIPE] = ACTIONS(4214),
    [anon_sym_LT] = ACTIONS(4214),
    [anon_sym_DOT_DOT] = ACTIONS(4214),
    [anon_sym_COLON_COLON] = ACTIONS(4214),
    [anon_sym_POUND] = ACTIONS(4214),
    [anon_sym_SQUOTE] = ACTIONS(4216),
    [anon_sym_async] = ACTIONS(4216),
    [anon_sym_auto] = ACTIONS(4216),
    [anon_sym_await] = ACTIONS(4216),
    [anon_sym_become] = ACTIONS(4216),
    [anon_sym_break] = ACTIONS(4216),
    [anon_sym_const] = ACTIONS(4216),
    [anon_sym_continue] = ACTIONS(4216),
    [anon_sym_default] = ACTIONS(4216),
    [anon_sym_enum] = ACTIONS(4216),
    [anon_sym_fn] = ACTIONS(4216),
    [anon_sym_for] = ACTIONS(4216),
    [anon_sym_gen] = ACTIONS(4216),
    [anon_sym_if] = ACTIONS(4216),
    [anon_sym_impl] = ACTIONS(4216),
    [anon_sym_let] = ACTIONS(4216),
    [anon_sym_loop] = ACTIONS(4216),
    [anon_sym_match] = ACTIONS(4216),
    [anon_sym_mod] = ACTIONS(4216),
    [anon_sym_pub] = ACTIONS(4216),
    [anon_sym_return] = ACTIONS(4216),
    [anon_sym_raw] = ACTIONS(4216),
    [anon_sym_static] = ACTIONS(4216),
    [anon_sym_struct] = ACTIONS(4216),
    [anon_sym_trait] = ACTIONS(4216),
    [anon_sym_type] = ACTIONS(4216),
    [anon_sym_union] = ACTIONS(4216),
    [anon_sym_unsafe] = ACTIONS(4216),
    [anon_sym_use] = ACTIONS(4216),
    [anon_sym_while] = ACTIONS(4216),
    [anon_sym_extern] = ACTIONS(4216),
    [anon_sym_safe] = ACTIONS(4216),
    [anon_sym_yield] = ACTIONS(4216),
    [anon_sym_move] = ACTIONS(4216),
    [anon_sym_try] = ACTIONS(4216),
    [sym_integer_literal] = ACTIONS(4214),
    [aux_sym_string_literal_token1] = ACTIONS(4214),
    [sym_char_literal] = ACTIONS(4214),
    [anon_sym_true] = ACTIONS(4216),
    [anon_sym_false] = ACTIONS(4216),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4216),
    [sym_super] = ACTIONS(4216),
    [sym_crate] = ACTIONS(4216),
    [aux_sym_metavariable_token1] = ACTIONS(4214),
    [sym__raw_string_literal_start] = ACTIONS(4214),
    [sym_float_literal] = ACTIONS(4214),
    [sym__outer_block_doc_comment_start] = ACTIONS(4214),
    [sym__outer_line_doc_comment_start] = ACTIONS(4214),
  },
  [STATE(1114)] = {
    [sym_line_comment] = STATE(1114),
    [sym_block_comment] = STATE(1114),
    [ts_builtin_sym_end] = ACTIONS(4218),
    [sym_identifier] = ACTIONS(4220),
    [anon_sym_SEMI] = ACTIONS(4218),
    [anon_sym_macro] = ACTIONS(4220),
    [anon_sym_LPAREN] = ACTIONS(4218),
    [anon_sym_LBRACE] = ACTIONS(4218),
    [anon_sym_RBRACE] = ACTIONS(4218),
    [anon_sym_macro_rules] = ACTIONS(4220),
    [anon_sym_BANG] = ACTIONS(4218),
    [anon_sym_LBRACK] = ACTIONS(4218),
    [anon_sym_STAR] = ACTIONS(4218),
    [anon_sym_u8] = ACTIONS(4220),
    [anon_sym_i8] = ACTIONS(4220),
    [anon_sym_u16] = ACTIONS(4220),
    [anon_sym_i16] = ACTIONS(4220),
    [anon_sym_u32] = ACTIONS(4220),
    [anon_sym_i32] = ACTIONS(4220),
    [anon_sym_u64] = ACTIONS(4220),
    [anon_sym_i64] = ACTIONS(4220),
    [anon_sym_u128] = ACTIONS(4220),
    [anon_sym_i128] = ACTIONS(4220),
    [anon_sym_isize] = ACTIONS(4220),
    [anon_sym_usize] = ACTIONS(4220),
    [anon_sym_f32] = ACTIONS(4220),
    [anon_sym_f64] = ACTIONS(4220),
    [anon_sym_bool] = ACTIONS(4220),
    [anon_sym_str] = ACTIONS(4220),
    [anon_sym_char] = ACTIONS(4220),
    [anon_sym_DASH] = ACTIONS(4218),
    [anon_sym_AMP] = ACTIONS(4218),
    [anon_sym_PIPE] = ACTIONS(4218),
    [anon_sym_LT] = ACTIONS(4218),
    [anon_sym_DOT_DOT] = ACTIONS(4218),
    [anon_sym_COLON_COLON] = ACTIONS(4218),
    [anon_sym_POUND] = ACTIONS(4218),
    [anon_sym_SQUOTE] = ACTIONS(4220),
    [anon_sym_async] = ACTIONS(4220),
    [anon_sym_auto] = ACTIONS(4220),
    [anon_sym_await] = ACTIONS(4220),
    [anon_sym_become] = ACTIONS(4220),
    [anon_sym_break] = ACTIONS(4220),
    [anon_sym_const] = ACTIONS(4220),
    [anon_sym_continue] = ACTIONS(4220),
    [anon_sym_default] = ACTIONS(4220),
    [anon_sym_enum] = ACTIONS(4220),
    [anon_sym_fn] = ACTIONS(4220),
    [anon_sym_for] = ACTIONS(4220),
    [anon_sym_gen] = ACTIONS(4220),
    [anon_sym_if] = ACTIONS(4220),
    [anon_sym_impl] = ACTIONS(4220),
    [anon_sym_let] = ACTIONS(4220),
    [anon_sym_loop] = ACTIONS(4220),
    [anon_sym_match] = ACTIONS(4220),
    [anon_sym_mod] = ACTIONS(4220),
    [anon_sym_pub] = ACTIONS(4220),
    [anon_sym_return] = ACTIONS(4220),
    [anon_sym_raw] = ACTIONS(4220),
    [anon_sym_static] = ACTIONS(4220),
    [anon_sym_struct] = ACTIONS(4220),
    [anon_sym_trait] = ACTIONS(4220),
    [anon_sym_type] = ACTIONS(4220),
    [anon_sym_union] = ACTIONS(4220),
    [anon_sym_unsafe] = ACTIONS(4220),
    [anon_sym_use] = ACTIONS(4220),
    [anon_sym_while] = ACTIONS(4220),
    [anon_sym_extern] = ACTIONS(4220),
    [anon_sym_safe] = ACTIONS(4220),
    [anon_sym_yield] = ACTIONS(4220),
    [anon_sym_move] = ACTIONS(4220),
    [anon_sym_try] = ACTIONS(4220),
    [sym_integer_literal] = ACTIONS(4218),
    [aux_sym_string_literal_token1] = ACTIONS(4218),
    [sym_char_literal] = ACTIONS(4218),
    [anon_sym_true] = ACTIONS(4220),
    [anon_sym_false] = ACTIONS(4220),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4220),
    [sym_super] = ACTIONS(4220),
    [sym_crate] = ACTIONS(4220),
    [aux_sym_metavariable_token1] = ACTIONS(4218),
    [sym__raw_string_literal_start] = ACTIONS(4218),
    [sym_float_literal] = ACTIONS(4218),
    [sym__outer_block_doc_comment_start] = ACTIONS(4218),
    [sym__outer_line_doc_comment_start] = ACTIONS(4218),
  },
  [STATE(1115)] = {
    [sym_line_comment] = STATE(1115),
    [sym_block_comment] = STATE(1115),
    [ts_builtin_sym_end] = ACTIONS(4222),
    [sym_identifier] = ACTIONS(4224),
    [anon_sym_SEMI] = ACTIONS(4222),
    [anon_sym_macro] = ACTIONS(4224),
    [anon_sym_LPAREN] = ACTIONS(4222),
    [anon_sym_LBRACE] = ACTIONS(4222),
    [anon_sym_RBRACE] = ACTIONS(4222),
    [anon_sym_macro_rules] = ACTIONS(4224),
    [anon_sym_BANG] = ACTIONS(4222),
    [anon_sym_LBRACK] = ACTIONS(4222),
    [anon_sym_STAR] = ACTIONS(4222),
    [anon_sym_u8] = ACTIONS(4224),
    [anon_sym_i8] = ACTIONS(4224),
    [anon_sym_u16] = ACTIONS(4224),
    [anon_sym_i16] = ACTIONS(4224),
    [anon_sym_u32] = ACTIONS(4224),
    [anon_sym_i32] = ACTIONS(4224),
    [anon_sym_u64] = ACTIONS(4224),
    [anon_sym_i64] = ACTIONS(4224),
    [anon_sym_u128] = ACTIONS(4224),
    [anon_sym_i128] = ACTIONS(4224),
    [anon_sym_isize] = ACTIONS(4224),
    [anon_sym_usize] = ACTIONS(4224),
    [anon_sym_f32] = ACTIONS(4224),
    [anon_sym_f64] = ACTIONS(4224),
    [anon_sym_bool] = ACTIONS(4224),
    [anon_sym_str] = ACTIONS(4224),
    [anon_sym_char] = ACTIONS(4224),
    [anon_sym_DASH] = ACTIONS(4222),
    [anon_sym_AMP] = ACTIONS(4222),
    [anon_sym_PIPE] = ACTIONS(4222),
    [anon_sym_LT] = ACTIONS(4222),
    [anon_sym_DOT_DOT] = ACTIONS(4222),
    [anon_sym_COLON_COLON] = ACTIONS(4222),
    [anon_sym_POUND] = ACTIONS(4222),
    [anon_sym_SQUOTE] = ACTIONS(4224),
    [anon_sym_async] = ACTIONS(4224),
    [anon_sym_auto] = ACTIONS(4224),
    [anon_sym_await] = ACTIONS(4224),
    [anon_sym_become] = ACTIONS(4224),
    [anon_sym_break] = ACTIONS(4224),
    [anon_sym_const] = ACTIONS(4224),
    [anon_sym_continue] = ACTIONS(4224),
    [anon_sym_default] = ACTIONS(4224),
    [anon_sym_enum] = ACTIONS(4224),
    [anon_sym_fn] = ACTIONS(4224),
    [anon_sym_for] = ACTIONS(4224),
    [anon_sym_gen] = ACTIONS(4224),
    [anon_sym_if] = ACTIONS(4224),
    [anon_sym_impl] = ACTIONS(4224),
    [anon_sym_let] = ACTIONS(4224),
    [anon_sym_loop] = ACTIONS(4224),
    [anon_sym_match] = ACTIONS(4224),
    [anon_sym_mod] = ACTIONS(4224),
    [anon_sym_pub] = ACTIONS(4224),
    [anon_sym_return] = ACTIONS(4224),
    [anon_sym_raw] = ACTIONS(4224),
    [anon_sym_static] = ACTIONS(4224),
    [anon_sym_struct] = ACTIONS(4224),
    [anon_sym_trait] = ACTIONS(4224),
    [anon_sym_type] = ACTIONS(4224),
    [anon_sym_union] = ACTIONS(4224),
    [anon_sym_unsafe] = ACTIONS(4224),
    [anon_sym_use] = ACTIONS(4224),
    [anon_sym_while] = ACTIONS(4224),
    [anon_sym_extern] = ACTIONS(4224),
    [anon_sym_safe] = ACTIONS(4224),
    [anon_sym_yield] = ACTIONS(4224),
    [anon_sym_move] = ACTIONS(4224),
    [anon_sym_try] = ACTIONS(4224),
    [sym_integer_literal] = ACTIONS(4222),
    [aux_sym_string_literal_token1] = ACTIONS(4222),
    [sym_char_literal] = ACTIONS(4222),
    [anon_sym_true] = ACTIONS(4224),
    [anon_sym_false] = ACTIONS(4224),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4224),
    [sym_super] = ACTIONS(4224),
    [sym_crate] = ACTIONS(4224),
    [aux_sym_metavariable_token1] = ACTIONS(4222),
    [sym__raw_string_literal_start] = ACTIONS(4222),
    [sym_float_literal] = ACTIONS(4222),
    [sym__outer_block_doc_comment_start] = ACTIONS(4222),
    [sym__outer_line_doc_comment_start] = ACTIONS(4222),
  },
  [STATE(1116)] = {
    [sym_line_comment] = STATE(1116),
    [sym_block_comment] = STATE(1116),
    [ts_builtin_sym_end] = ACTIONS(4226),
    [sym_identifier] = ACTIONS(4228),
    [anon_sym_SEMI] = ACTIONS(4226),
    [anon_sym_macro] = ACTIONS(4228),
    [anon_sym_LPAREN] = ACTIONS(4226),
    [anon_sym_LBRACE] = ACTIONS(4226),
    [anon_sym_RBRACE] = ACTIONS(4226),
    [anon_sym_macro_rules] = ACTIONS(4228),
    [anon_sym_BANG] = ACTIONS(4226),
    [anon_sym_LBRACK] = ACTIONS(4226),
    [anon_sym_STAR] = ACTIONS(4226),
    [anon_sym_u8] = ACTIONS(4228),
    [anon_sym_i8] = ACTIONS(4228),
    [anon_sym_u16] = ACTIONS(4228),
    [anon_sym_i16] = ACTIONS(4228),
    [anon_sym_u32] = ACTIONS(4228),
    [anon_sym_i32] = ACTIONS(4228),
    [anon_sym_u64] = ACTIONS(4228),
    [anon_sym_i64] = ACTIONS(4228),
    [anon_sym_u128] = ACTIONS(4228),
    [anon_sym_i128] = ACTIONS(4228),
    [anon_sym_isize] = ACTIONS(4228),
    [anon_sym_usize] = ACTIONS(4228),
    [anon_sym_f32] = ACTIONS(4228),
    [anon_sym_f64] = ACTIONS(4228),
    [anon_sym_bool] = ACTIONS(4228),
    [anon_sym_str] = ACTIONS(4228),
    [anon_sym_char] = ACTIONS(4228),
    [anon_sym_DASH] = ACTIONS(4226),
    [anon_sym_AMP] = ACTIONS(4226),
    [anon_sym_PIPE] = ACTIONS(4226),
    [anon_sym_LT] = ACTIONS(4226),
    [anon_sym_DOT_DOT] = ACTIONS(4226),
    [anon_sym_COLON_COLON] = ACTIONS(4226),
    [anon_sym_POUND] = ACTIONS(4226),
    [anon_sym_SQUOTE] = ACTIONS(4228),
    [anon_sym_async] = ACTIONS(4228),
    [anon_sym_auto] = ACTIONS(4228),
    [anon_sym_await] = ACTIONS(4228),
    [anon_sym_become] = ACTIONS(4228),
    [anon_sym_break] = ACTIONS(4228),
    [anon_sym_const] = ACTIONS(4228),
    [anon_sym_continue] = ACTIONS(4228),
    [anon_sym_default] = ACTIONS(4228),
    [anon_sym_enum] = ACTIONS(4228),
    [anon_sym_fn] = ACTIONS(4228),
    [anon_sym_for] = ACTIONS(4228),
    [anon_sym_gen] = ACTIONS(4228),
    [anon_sym_if] = ACTIONS(4228),
    [anon_sym_impl] = ACTIONS(4228),
    [anon_sym_let] = ACTIONS(4228),
    [anon_sym_loop] = ACTIONS(4228),
    [anon_sym_match] = ACTIONS(4228),
    [anon_sym_mod] = ACTIONS(4228),
    [anon_sym_pub] = ACTIONS(4228),
    [anon_sym_return] = ACTIONS(4228),
    [anon_sym_raw] = ACTIONS(4228),
    [anon_sym_static] = ACTIONS(4228),
    [anon_sym_struct] = ACTIONS(4228),
    [anon_sym_trait] = ACTIONS(4228),
    [anon_sym_type] = ACTIONS(4228),
    [anon_sym_union] = ACTIONS(4228),
    [anon_sym_unsafe] = ACTIONS(4228),
    [anon_sym_use] = ACTIONS(4228),
    [anon_sym_while] = ACTIONS(4228),
    [anon_sym_extern] = ACTIONS(4228),
    [anon_sym_safe] = ACTIONS(4228),
    [anon_sym_yield] = ACTIONS(4228),
    [anon_sym_move] = ACTIONS(4228),
    [anon_sym_try] = ACTIONS(4228),
    [sym_integer_literal] = ACTIONS(4226),
    [aux_sym_string_literal_token1] = ACTIONS(4226),
    [sym_char_literal] = ACTIONS(4226),
    [anon_sym_true] = ACTIONS(4228),
    [anon_sym_false] = ACTIONS(4228),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4228),
    [sym_super] = ACTIONS(4228),
    [sym_crate] = ACTIONS(4228),
    [aux_sym_metavariable_token1] = ACTIONS(4226),
    [sym__raw_string_literal_start] = ACTIONS(4226),
    [sym_float_literal] = ACTIONS(4226),
    [sym__outer_block_doc_comment_start] = ACTIONS(4226),
    [sym__outer_line_doc_comment_start] = ACTIONS(4226),
  },
  [STATE(1117)] = {
    [sym_line_comment] = STATE(1117),
    [sym_block_comment] = STATE(1117),
    [ts_builtin_sym_end] = ACTIONS(4230),
    [sym_identifier] = ACTIONS(4232),
    [anon_sym_SEMI] = ACTIONS(4230),
    [anon_sym_macro] = ACTIONS(4232),
    [anon_sym_LPAREN] = ACTIONS(4230),
    [anon_sym_LBRACE] = ACTIONS(4230),
    [anon_sym_RBRACE] = ACTIONS(4230),
    [anon_sym_macro_rules] = ACTIONS(4232),
    [anon_sym_BANG] = ACTIONS(4230),
    [anon_sym_LBRACK] = ACTIONS(4230),
    [anon_sym_STAR] = ACTIONS(4230),
    [anon_sym_u8] = ACTIONS(4232),
    [anon_sym_i8] = ACTIONS(4232),
    [anon_sym_u16] = ACTIONS(4232),
    [anon_sym_i16] = ACTIONS(4232),
    [anon_sym_u32] = ACTIONS(4232),
    [anon_sym_i32] = ACTIONS(4232),
    [anon_sym_u64] = ACTIONS(4232),
    [anon_sym_i64] = ACTIONS(4232),
    [anon_sym_u128] = ACTIONS(4232),
    [anon_sym_i128] = ACTIONS(4232),
    [anon_sym_isize] = ACTIONS(4232),
    [anon_sym_usize] = ACTIONS(4232),
    [anon_sym_f32] = ACTIONS(4232),
    [anon_sym_f64] = ACTIONS(4232),
    [anon_sym_bool] = ACTIONS(4232),
    [anon_sym_str] = ACTIONS(4232),
    [anon_sym_char] = ACTIONS(4232),
    [anon_sym_DASH] = ACTIONS(4230),
    [anon_sym_AMP] = ACTIONS(4230),
    [anon_sym_PIPE] = ACTIONS(4230),
    [anon_sym_LT] = ACTIONS(4230),
    [anon_sym_DOT_DOT] = ACTIONS(4230),
    [anon_sym_COLON_COLON] = ACTIONS(4230),
    [anon_sym_POUND] = ACTIONS(4230),
    [anon_sym_SQUOTE] = ACTIONS(4232),
    [anon_sym_async] = ACTIONS(4232),
    [anon_sym_auto] = ACTIONS(4232),
    [anon_sym_await] = ACTIONS(4232),
    [anon_sym_become] = ACTIONS(4232),
    [anon_sym_break] = ACTIONS(4232),
    [anon_sym_const] = ACTIONS(4232),
    [anon_sym_continue] = ACTIONS(4232),
    [anon_sym_default] = ACTIONS(4232),
    [anon_sym_enum] = ACTIONS(4232),
    [anon_sym_fn] = ACTIONS(4232),
    [anon_sym_for] = ACTIONS(4232),
    [anon_sym_gen] = ACTIONS(4232),
    [anon_sym_if] = ACTIONS(4232),
    [anon_sym_impl] = ACTIONS(4232),
    [anon_sym_let] = ACTIONS(4232),
    [anon_sym_loop] = ACTIONS(4232),
    [anon_sym_match] = ACTIONS(4232),
    [anon_sym_mod] = ACTIONS(4232),
    [anon_sym_pub] = ACTIONS(4232),
    [anon_sym_return] = ACTIONS(4232),
    [anon_sym_raw] = ACTIONS(4232),
    [anon_sym_static] = ACTIONS(4232),
    [anon_sym_struct] = ACTIONS(4232),
    [anon_sym_trait] = ACTIONS(4232),
    [anon_sym_type] = ACTIONS(4232),
    [anon_sym_union] = ACTIONS(4232),
    [anon_sym_unsafe] = ACTIONS(4232),
    [anon_sym_use] = ACTIONS(4232),
    [anon_sym_while] = ACTIONS(4232),
    [anon_sym_extern] = ACTIONS(4232),
    [anon_sym_safe] = ACTIONS(4232),
    [anon_sym_yield] = ACTIONS(4232),
    [anon_sym_move] = ACTIONS(4232),
    [anon_sym_try] = ACTIONS(4232),
    [sym_integer_literal] = ACTIONS(4230),
    [aux_sym_string_literal_token1] = ACTIONS(4230),
    [sym_char_literal] = ACTIONS(4230),
    [anon_sym_true] = ACTIONS(4232),
    [anon_sym_false] = ACTIONS(4232),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4232),
    [sym_super] = ACTIONS(4232),
    [sym_crate] = ACTIONS(4232),
    [aux_sym_metavariable_token1] = ACTIONS(4230),
    [sym__raw_string_literal_start] = ACTIONS(4230),
    [sym_float_literal] = ACTIONS(4230),
    [sym__outer_block_doc_comment_start] = ACTIONS(4230),
    [sym__outer_line_doc_comment_start] = ACTIONS(4230),
  },
  [STATE(1118)] = {
    [sym_line_comment] = STATE(1118),
    [sym_block_comment] = STATE(1118),
    [ts_builtin_sym_end] = ACTIONS(4234),
    [sym_identifier] = ACTIONS(4236),
    [anon_sym_SEMI] = ACTIONS(4234),
    [anon_sym_macro] = ACTIONS(4236),
    [anon_sym_LPAREN] = ACTIONS(4234),
    [anon_sym_LBRACE] = ACTIONS(4234),
    [anon_sym_RBRACE] = ACTIONS(4234),
    [anon_sym_macro_rules] = ACTIONS(4236),
    [anon_sym_BANG] = ACTIONS(4234),
    [anon_sym_LBRACK] = ACTIONS(4234),
    [anon_sym_STAR] = ACTIONS(4234),
    [anon_sym_u8] = ACTIONS(4236),
    [anon_sym_i8] = ACTIONS(4236),
    [anon_sym_u16] = ACTIONS(4236),
    [anon_sym_i16] = ACTIONS(4236),
    [anon_sym_u32] = ACTIONS(4236),
    [anon_sym_i32] = ACTIONS(4236),
    [anon_sym_u64] = ACTIONS(4236),
    [anon_sym_i64] = ACTIONS(4236),
    [anon_sym_u128] = ACTIONS(4236),
    [anon_sym_i128] = ACTIONS(4236),
    [anon_sym_isize] = ACTIONS(4236),
    [anon_sym_usize] = ACTIONS(4236),
    [anon_sym_f32] = ACTIONS(4236),
    [anon_sym_f64] = ACTIONS(4236),
    [anon_sym_bool] = ACTIONS(4236),
    [anon_sym_str] = ACTIONS(4236),
    [anon_sym_char] = ACTIONS(4236),
    [anon_sym_DASH] = ACTIONS(4234),
    [anon_sym_AMP] = ACTIONS(4234),
    [anon_sym_PIPE] = ACTIONS(4234),
    [anon_sym_LT] = ACTIONS(4234),
    [anon_sym_DOT_DOT] = ACTIONS(4234),
    [anon_sym_COLON_COLON] = ACTIONS(4234),
    [anon_sym_POUND] = ACTIONS(4234),
    [anon_sym_SQUOTE] = ACTIONS(4236),
    [anon_sym_async] = ACTIONS(4236),
    [anon_sym_auto] = ACTIONS(4236),
    [anon_sym_await] = ACTIONS(4236),
    [anon_sym_become] = ACTIONS(4236),
    [anon_sym_break] = ACTIONS(4236),
    [anon_sym_const] = ACTIONS(4236),
    [anon_sym_continue] = ACTIONS(4236),
    [anon_sym_default] = ACTIONS(4236),
    [anon_sym_enum] = ACTIONS(4236),
    [anon_sym_fn] = ACTIONS(4236),
    [anon_sym_for] = ACTIONS(4236),
    [anon_sym_gen] = ACTIONS(4236),
    [anon_sym_if] = ACTIONS(4236),
    [anon_sym_impl] = ACTIONS(4236),
    [anon_sym_let] = ACTIONS(4236),
    [anon_sym_loop] = ACTIONS(4236),
    [anon_sym_match] = ACTIONS(4236),
    [anon_sym_mod] = ACTIONS(4236),
    [anon_sym_pub] = ACTIONS(4236),
    [anon_sym_return] = ACTIONS(4236),
    [anon_sym_raw] = ACTIONS(4236),
    [anon_sym_static] = ACTIONS(4236),
    [anon_sym_struct] = ACTIONS(4236),
    [anon_sym_trait] = ACTIONS(4236),
    [anon_sym_type] = ACTIONS(4236),
    [anon_sym_union] = ACTIONS(4236),
    [anon_sym_unsafe] = ACTIONS(4236),
    [anon_sym_use] = ACTIONS(4236),
    [anon_sym_while] = ACTIONS(4236),
    [anon_sym_extern] = ACTIONS(4236),
    [anon_sym_safe] = ACTIONS(4236),
    [anon_sym_yield] = ACTIONS(4236),
    [anon_sym_move] = ACTIONS(4236),
    [anon_sym_try] = ACTIONS(4236),
    [sym_integer_literal] = ACTIONS(4234),
    [aux_sym_string_literal_token1] = ACTIONS(4234),
    [sym_char_literal] = ACTIONS(4234),
    [anon_sym_true] = ACTIONS(4236),
    [anon_sym_false] = ACTIONS(4236),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4236),
    [sym_super] = ACTIONS(4236),
    [sym_crate] = ACTIONS(4236),
    [aux_sym_metavariable_token1] = ACTIONS(4234),
    [sym__raw_string_literal_start] = ACTIONS(4234),
    [sym_float_literal] = ACTIONS(4234),
    [sym__outer_block_doc_comment_start] = ACTIONS(4234),
    [sym__outer_line_doc_comment_start] = ACTIONS(4234),
  },
  [STATE(1119)] = {
    [sym_line_comment] = STATE(1119),
    [sym_block_comment] = STATE(1119),
    [ts_builtin_sym_end] = ACTIONS(4238),
    [sym_identifier] = ACTIONS(4240),
    [anon_sym_SEMI] = ACTIONS(4238),
    [anon_sym_macro] = ACTIONS(4240),
    [anon_sym_LPAREN] = ACTIONS(4238),
    [anon_sym_LBRACE] = ACTIONS(4238),
    [anon_sym_RBRACE] = ACTIONS(4238),
    [anon_sym_macro_rules] = ACTIONS(4240),
    [anon_sym_BANG] = ACTIONS(4238),
    [anon_sym_LBRACK] = ACTIONS(4238),
    [anon_sym_STAR] = ACTIONS(4238),
    [anon_sym_u8] = ACTIONS(4240),
    [anon_sym_i8] = ACTIONS(4240),
    [anon_sym_u16] = ACTIONS(4240),
    [anon_sym_i16] = ACTIONS(4240),
    [anon_sym_u32] = ACTIONS(4240),
    [anon_sym_i32] = ACTIONS(4240),
    [anon_sym_u64] = ACTIONS(4240),
    [anon_sym_i64] = ACTIONS(4240),
    [anon_sym_u128] = ACTIONS(4240),
    [anon_sym_i128] = ACTIONS(4240),
    [anon_sym_isize] = ACTIONS(4240),
    [anon_sym_usize] = ACTIONS(4240),
    [anon_sym_f32] = ACTIONS(4240),
    [anon_sym_f64] = ACTIONS(4240),
    [anon_sym_bool] = ACTIONS(4240),
    [anon_sym_str] = ACTIONS(4240),
    [anon_sym_char] = ACTIONS(4240),
    [anon_sym_DASH] = ACTIONS(4238),
    [anon_sym_AMP] = ACTIONS(4238),
    [anon_sym_PIPE] = ACTIONS(4238),
    [anon_sym_LT] = ACTIONS(4238),
    [anon_sym_DOT_DOT] = ACTIONS(4238),
    [anon_sym_COLON_COLON] = ACTIONS(4238),
    [anon_sym_POUND] = ACTIONS(4238),
    [anon_sym_SQUOTE] = ACTIONS(4240),
    [anon_sym_async] = ACTIONS(4240),
    [anon_sym_auto] = ACTIONS(4240),
    [anon_sym_await] = ACTIONS(4240),
    [anon_sym_become] = ACTIONS(4240),
    [anon_sym_break] = ACTIONS(4240),
    [anon_sym_const] = ACTIONS(4240),
    [anon_sym_continue] = ACTIONS(4240),
    [anon_sym_default] = ACTIONS(4240),
    [anon_sym_enum] = ACTIONS(4240),
    [anon_sym_fn] = ACTIONS(4240),
    [anon_sym_for] = ACTIONS(4240),
    [anon_sym_gen] = ACTIONS(4240),
    [anon_sym_if] = ACTIONS(4240),
    [anon_sym_impl] = ACTIONS(4240),
    [anon_sym_let] = ACTIONS(4240),
    [anon_sym_loop] = ACTIONS(4240),
    [anon_sym_match] = ACTIONS(4240),
    [anon_sym_mod] = ACTIONS(4240),
    [anon_sym_pub] = ACTIONS(4240),
    [anon_sym_return] = ACTIONS(4240),
    [anon_sym_raw] = ACTIONS(4240),
    [anon_sym_static] = ACTIONS(4240),
    [anon_sym_struct] = ACTIONS(4240),
    [anon_sym_trait] = ACTIONS(4240),
    [anon_sym_type] = ACTIONS(4240),
    [anon_sym_union] = ACTIONS(4240),
    [anon_sym_unsafe] = ACTIONS(4240),
    [anon_sym_use] = ACTIONS(4240),
    [anon_sym_while] = ACTIONS(4240),
    [anon_sym_extern] = ACTIONS(4240),
    [anon_sym_safe] = ACTIONS(4240),
    [anon_sym_yield] = ACTIONS(4240),
    [anon_sym_move] = ACTIONS(4240),
    [anon_sym_try] = ACTIONS(4240),
    [sym_integer_literal] = ACTIONS(4238),
    [aux_sym_string_literal_token1] = ACTIONS(4238),
    [sym_char_literal] = ACTIONS(4238),
    [anon_sym_true] = ACTIONS(4240),
    [anon_sym_false] = ACTIONS(4240),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4240),
    [sym_super] = ACTIONS(4240),
    [sym_crate] = ACTIONS(4240),
    [aux_sym_metavariable_token1] = ACTIONS(4238),
    [sym__raw_string_literal_start] = ACTIONS(4238),
    [sym_float_literal] = ACTIONS(4238),
    [sym__outer_block_doc_comment_start] = ACTIONS(4238),
    [sym__outer_line_doc_comment_start] = ACTIONS(4238),
  },
  [STATE(1120)] = {
    [sym_line_comment] = STATE(1120),
    [sym_block_comment] = STATE(1120),
    [ts_builtin_sym_end] = ACTIONS(4242),
    [sym_identifier] = ACTIONS(4244),
    [anon_sym_SEMI] = ACTIONS(4242),
    [anon_sym_macro] = ACTIONS(4244),
    [anon_sym_LPAREN] = ACTIONS(4242),
    [anon_sym_LBRACE] = ACTIONS(4242),
    [anon_sym_RBRACE] = ACTIONS(4242),
    [anon_sym_macro_rules] = ACTIONS(4244),
    [anon_sym_BANG] = ACTIONS(4242),
    [anon_sym_LBRACK] = ACTIONS(4242),
    [anon_sym_STAR] = ACTIONS(4242),
    [anon_sym_u8] = ACTIONS(4244),
    [anon_sym_i8] = ACTIONS(4244),
    [anon_sym_u16] = ACTIONS(4244),
    [anon_sym_i16] = ACTIONS(4244),
    [anon_sym_u32] = ACTIONS(4244),
    [anon_sym_i32] = ACTIONS(4244),
    [anon_sym_u64] = ACTIONS(4244),
    [anon_sym_i64] = ACTIONS(4244),
    [anon_sym_u128] = ACTIONS(4244),
    [anon_sym_i128] = ACTIONS(4244),
    [anon_sym_isize] = ACTIONS(4244),
    [anon_sym_usize] = ACTIONS(4244),
    [anon_sym_f32] = ACTIONS(4244),
    [anon_sym_f64] = ACTIONS(4244),
    [anon_sym_bool] = ACTIONS(4244),
    [anon_sym_str] = ACTIONS(4244),
    [anon_sym_char] = ACTIONS(4244),
    [anon_sym_DASH] = ACTIONS(4242),
    [anon_sym_AMP] = ACTIONS(4242),
    [anon_sym_PIPE] = ACTIONS(4242),
    [anon_sym_LT] = ACTIONS(4242),
    [anon_sym_DOT_DOT] = ACTIONS(4242),
    [anon_sym_COLON_COLON] = ACTIONS(4242),
    [anon_sym_POUND] = ACTIONS(4242),
    [anon_sym_SQUOTE] = ACTIONS(4244),
    [anon_sym_async] = ACTIONS(4244),
    [anon_sym_auto] = ACTIONS(4244),
    [anon_sym_await] = ACTIONS(4244),
    [anon_sym_become] = ACTIONS(4244),
    [anon_sym_break] = ACTIONS(4244),
    [anon_sym_const] = ACTIONS(4244),
    [anon_sym_continue] = ACTIONS(4244),
    [anon_sym_default] = ACTIONS(4244),
    [anon_sym_enum] = ACTIONS(4244),
    [anon_sym_fn] = ACTIONS(4244),
    [anon_sym_for] = ACTIONS(4244),
    [anon_sym_gen] = ACTIONS(4244),
    [anon_sym_if] = ACTIONS(4244),
    [anon_sym_impl] = ACTIONS(4244),
    [anon_sym_let] = ACTIONS(4244),
    [anon_sym_loop] = ACTIONS(4244),
    [anon_sym_match] = ACTIONS(4244),
    [anon_sym_mod] = ACTIONS(4244),
    [anon_sym_pub] = ACTIONS(4244),
    [anon_sym_return] = ACTIONS(4244),
    [anon_sym_raw] = ACTIONS(4244),
    [anon_sym_static] = ACTIONS(4244),
    [anon_sym_struct] = ACTIONS(4244),
    [anon_sym_trait] = ACTIONS(4244),
    [anon_sym_type] = ACTIONS(4244),
    [anon_sym_union] = ACTIONS(4244),
    [anon_sym_unsafe] = ACTIONS(4244),
    [anon_sym_use] = ACTIONS(4244),
    [anon_sym_while] = ACTIONS(4244),
    [anon_sym_extern] = ACTIONS(4244),
    [anon_sym_safe] = ACTIONS(4244),
    [anon_sym_yield] = ACTIONS(4244),
    [anon_sym_move] = ACTIONS(4244),
    [anon_sym_try] = ACTIONS(4244),
    [sym_integer_literal] = ACTIONS(4242),
    [aux_sym_string_literal_token1] = ACTIONS(4242),
    [sym_char_literal] = ACTIONS(4242),
    [anon_sym_true] = ACTIONS(4244),
    [anon_sym_false] = ACTIONS(4244),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4244),
    [sym_super] = ACTIONS(4244),
    [sym_crate] = ACTIONS(4244),
    [aux_sym_metavariable_token1] = ACTIONS(4242),
    [sym__raw_string_literal_start] = ACTIONS(4242),
    [sym_float_literal] = ACTIONS(4242),
    [sym__outer_block_doc_comment_start] = ACTIONS(4242),
    [sym__outer_line_doc_comment_start] = ACTIONS(4242),
  },
  [STATE(1121)] = {
    [sym_line_comment] = STATE(1121),
    [sym_block_comment] = STATE(1121),
    [ts_builtin_sym_end] = ACTIONS(4246),
    [sym_identifier] = ACTIONS(4248),
    [anon_sym_SEMI] = ACTIONS(4246),
    [anon_sym_macro] = ACTIONS(4248),
    [anon_sym_LPAREN] = ACTIONS(4246),
    [anon_sym_LBRACE] = ACTIONS(4246),
    [anon_sym_RBRACE] = ACTIONS(4246),
    [anon_sym_macro_rules] = ACTIONS(4248),
    [anon_sym_BANG] = ACTIONS(4246),
    [anon_sym_LBRACK] = ACTIONS(4246),
    [anon_sym_STAR] = ACTIONS(4246),
    [anon_sym_u8] = ACTIONS(4248),
    [anon_sym_i8] = ACTIONS(4248),
    [anon_sym_u16] = ACTIONS(4248),
    [anon_sym_i16] = ACTIONS(4248),
    [anon_sym_u32] = ACTIONS(4248),
    [anon_sym_i32] = ACTIONS(4248),
    [anon_sym_u64] = ACTIONS(4248),
    [anon_sym_i64] = ACTIONS(4248),
    [anon_sym_u128] = ACTIONS(4248),
    [anon_sym_i128] = ACTIONS(4248),
    [anon_sym_isize] = ACTIONS(4248),
    [anon_sym_usize] = ACTIONS(4248),
    [anon_sym_f32] = ACTIONS(4248),
    [anon_sym_f64] = ACTIONS(4248),
    [anon_sym_bool] = ACTIONS(4248),
    [anon_sym_str] = ACTIONS(4248),
    [anon_sym_char] = ACTIONS(4248),
    [anon_sym_DASH] = ACTIONS(4246),
    [anon_sym_AMP] = ACTIONS(4246),
    [anon_sym_PIPE] = ACTIONS(4246),
    [anon_sym_LT] = ACTIONS(4246),
    [anon_sym_DOT_DOT] = ACTIONS(4246),
    [anon_sym_COLON_COLON] = ACTIONS(4246),
    [anon_sym_POUND] = ACTIONS(4246),
    [anon_sym_SQUOTE] = ACTIONS(4248),
    [anon_sym_async] = ACTIONS(4248),
    [anon_sym_auto] = ACTIONS(4248),
    [anon_sym_await] = ACTIONS(4248),
    [anon_sym_become] = ACTIONS(4248),
    [anon_sym_break] = ACTIONS(4248),
    [anon_sym_const] = ACTIONS(4248),
    [anon_sym_continue] = ACTIONS(4248),
    [anon_sym_default] = ACTIONS(4248),
    [anon_sym_enum] = ACTIONS(4248),
    [anon_sym_fn] = ACTIONS(4248),
    [anon_sym_for] = ACTIONS(4248),
    [anon_sym_gen] = ACTIONS(4248),
    [anon_sym_if] = ACTIONS(4248),
    [anon_sym_impl] = ACTIONS(4248),
    [anon_sym_let] = ACTIONS(4248),
    [anon_sym_loop] = ACTIONS(4248),
    [anon_sym_match] = ACTIONS(4248),
    [anon_sym_mod] = ACTIONS(4248),
    [anon_sym_pub] = ACTIONS(4248),
    [anon_sym_return] = ACTIONS(4248),
    [anon_sym_raw] = ACTIONS(4248),
    [anon_sym_static] = ACTIONS(4248),
    [anon_sym_struct] = ACTIONS(4248),
    [anon_sym_trait] = ACTIONS(4248),
    [anon_sym_type] = ACTIONS(4248),
    [anon_sym_union] = ACTIONS(4248),
    [anon_sym_unsafe] = ACTIONS(4248),
    [anon_sym_use] = ACTIONS(4248),
    [anon_sym_while] = ACTIONS(4248),
    [anon_sym_extern] = ACTIONS(4248),
    [anon_sym_safe] = ACTIONS(4248),
    [anon_sym_yield] = ACTIONS(4248),
    [anon_sym_move] = ACTIONS(4248),
    [anon_sym_try] = ACTIONS(4248),
    [sym_integer_literal] = ACTIONS(4246),
    [aux_sym_string_literal_token1] = ACTIONS(4246),
    [sym_char_literal] = ACTIONS(4246),
    [anon_sym_true] = ACTIONS(4248),
    [anon_sym_false] = ACTIONS(4248),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4248),
    [sym_super] = ACTIONS(4248),
    [sym_crate] = ACTIONS(4248),
    [aux_sym_metavariable_token1] = ACTIONS(4246),
    [sym__raw_string_literal_start] = ACTIONS(4246),
    [sym_float_literal] = ACTIONS(4246),
    [sym__outer_block_doc_comment_start] = ACTIONS(4246),
    [sym__outer_line_doc_comment_start] = ACTIONS(4246),
  },
  [STATE(1122)] = {
    [sym_line_comment] = STATE(1122),
    [sym_block_comment] = STATE(1122),
    [ts_builtin_sym_end] = ACTIONS(4250),
    [sym_identifier] = ACTIONS(4252),
    [anon_sym_SEMI] = ACTIONS(4250),
    [anon_sym_macro] = ACTIONS(4252),
    [anon_sym_LPAREN] = ACTIONS(4250),
    [anon_sym_LBRACE] = ACTIONS(4250),
    [anon_sym_RBRACE] = ACTIONS(4250),
    [anon_sym_macro_rules] = ACTIONS(4252),
    [anon_sym_BANG] = ACTIONS(4250),
    [anon_sym_LBRACK] = ACTIONS(4250),
    [anon_sym_STAR] = ACTIONS(4250),
    [anon_sym_u8] = ACTIONS(4252),
    [anon_sym_i8] = ACTIONS(4252),
    [anon_sym_u16] = ACTIONS(4252),
    [anon_sym_i16] = ACTIONS(4252),
    [anon_sym_u32] = ACTIONS(4252),
    [anon_sym_i32] = ACTIONS(4252),
    [anon_sym_u64] = ACTIONS(4252),
    [anon_sym_i64] = ACTIONS(4252),
    [anon_sym_u128] = ACTIONS(4252),
    [anon_sym_i128] = ACTIONS(4252),
    [anon_sym_isize] = ACTIONS(4252),
    [anon_sym_usize] = ACTIONS(4252),
    [anon_sym_f32] = ACTIONS(4252),
    [anon_sym_f64] = ACTIONS(4252),
    [anon_sym_bool] = ACTIONS(4252),
    [anon_sym_str] = ACTIONS(4252),
    [anon_sym_char] = ACTIONS(4252),
    [anon_sym_DASH] = ACTIONS(4250),
    [anon_sym_AMP] = ACTIONS(4250),
    [anon_sym_PIPE] = ACTIONS(4250),
    [anon_sym_LT] = ACTIONS(4250),
    [anon_sym_DOT_DOT] = ACTIONS(4250),
    [anon_sym_COLON_COLON] = ACTIONS(4250),
    [anon_sym_POUND] = ACTIONS(4250),
    [anon_sym_SQUOTE] = ACTIONS(4252),
    [anon_sym_async] = ACTIONS(4252),
    [anon_sym_auto] = ACTIONS(4252),
    [anon_sym_await] = ACTIONS(4252),
    [anon_sym_become] = ACTIONS(4252),
    [anon_sym_break] = ACTIONS(4252),
    [anon_sym_const] = ACTIONS(4252),
    [anon_sym_continue] = ACTIONS(4252),
    [anon_sym_default] = ACTIONS(4252),
    [anon_sym_enum] = ACTIONS(4252),
    [anon_sym_fn] = ACTIONS(4252),
    [anon_sym_for] = ACTIONS(4252),
    [anon_sym_gen] = ACTIONS(4252),
    [anon_sym_if] = ACTIONS(4252),
    [anon_sym_impl] = ACTIONS(4252),
    [anon_sym_let] = ACTIONS(4252),
    [anon_sym_loop] = ACTIONS(4252),
    [anon_sym_match] = ACTIONS(4252),
    [anon_sym_mod] = ACTIONS(4252),
    [anon_sym_pub] = ACTIONS(4252),
    [anon_sym_return] = ACTIONS(4252),
    [anon_sym_raw] = ACTIONS(4252),
    [anon_sym_static] = ACTIONS(4252),
    [anon_sym_struct] = ACTIONS(4252),
    [anon_sym_trait] = ACTIONS(4252),
    [anon_sym_type] = ACTIONS(4252),
    [anon_sym_union] = ACTIONS(4252),
    [anon_sym_unsafe] = ACTIONS(4252),
    [anon_sym_use] = ACTIONS(4252),
    [anon_sym_while] = ACTIONS(4252),
    [anon_sym_extern] = ACTIONS(4252),
    [anon_sym_safe] = ACTIONS(4252),
    [anon_sym_yield] = ACTIONS(4252),
    [anon_sym_move] = ACTIONS(4252),
    [anon_sym_try] = ACTIONS(4252),
    [sym_integer_literal] = ACTIONS(4250),
    [aux_sym_string_literal_token1] = ACTIONS(4250),
    [sym_char_literal] = ACTIONS(4250),
    [anon_sym_true] = ACTIONS(4252),
    [anon_sym_false] = ACTIONS(4252),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4252),
    [sym_super] = ACTIONS(4252),
    [sym_crate] = ACTIONS(4252),
    [aux_sym_metavariable_token1] = ACTIONS(4250),
    [sym__raw_string_literal_start] = ACTIONS(4250),
    [sym_float_literal] = ACTIONS(4250),
    [sym__outer_block_doc_comment_start] = ACTIONS(4250),
    [sym__outer_line_doc_comment_start] = ACTIONS(4250),
  },
  [STATE(1123)] = {
    [sym_line_comment] = STATE(1123),
    [sym_block_comment] = STATE(1123),
    [ts_builtin_sym_end] = ACTIONS(4254),
    [sym_identifier] = ACTIONS(4256),
    [anon_sym_SEMI] = ACTIONS(4254),
    [anon_sym_macro] = ACTIONS(4256),
    [anon_sym_LPAREN] = ACTIONS(4254),
    [anon_sym_LBRACE] = ACTIONS(4254),
    [anon_sym_RBRACE] = ACTIONS(4254),
    [anon_sym_macro_rules] = ACTIONS(4256),
    [anon_sym_BANG] = ACTIONS(4254),
    [anon_sym_LBRACK] = ACTIONS(4254),
    [anon_sym_STAR] = ACTIONS(4254),
    [anon_sym_u8] = ACTIONS(4256),
    [anon_sym_i8] = ACTIONS(4256),
    [anon_sym_u16] = ACTIONS(4256),
    [anon_sym_i16] = ACTIONS(4256),
    [anon_sym_u32] = ACTIONS(4256),
    [anon_sym_i32] = ACTIONS(4256),
    [anon_sym_u64] = ACTIONS(4256),
    [anon_sym_i64] = ACTIONS(4256),
    [anon_sym_u128] = ACTIONS(4256),
    [anon_sym_i128] = ACTIONS(4256),
    [anon_sym_isize] = ACTIONS(4256),
    [anon_sym_usize] = ACTIONS(4256),
    [anon_sym_f32] = ACTIONS(4256),
    [anon_sym_f64] = ACTIONS(4256),
    [anon_sym_bool] = ACTIONS(4256),
    [anon_sym_str] = ACTIONS(4256),
    [anon_sym_char] = ACTIONS(4256),
    [anon_sym_DASH] = ACTIONS(4254),
    [anon_sym_AMP] = ACTIONS(4254),
    [anon_sym_PIPE] = ACTIONS(4254),
    [anon_sym_LT] = ACTIONS(4254),
    [anon_sym_DOT_DOT] = ACTIONS(4254),
    [anon_sym_COLON_COLON] = ACTIONS(4254),
    [anon_sym_POUND] = ACTIONS(4254),
    [anon_sym_SQUOTE] = ACTIONS(4256),
    [anon_sym_async] = ACTIONS(4256),
    [anon_sym_auto] = ACTIONS(4256),
    [anon_sym_await] = ACTIONS(4256),
    [anon_sym_become] = ACTIONS(4256),
    [anon_sym_break] = ACTIONS(4256),
    [anon_sym_const] = ACTIONS(4256),
    [anon_sym_continue] = ACTIONS(4256),
    [anon_sym_default] = ACTIONS(4256),
    [anon_sym_enum] = ACTIONS(4256),
    [anon_sym_fn] = ACTIONS(4256),
    [anon_sym_for] = ACTIONS(4256),
    [anon_sym_gen] = ACTIONS(4256),
    [anon_sym_if] = ACTIONS(4256),
    [anon_sym_impl] = ACTIONS(4256),
    [anon_sym_let] = ACTIONS(4256),
    [anon_sym_loop] = ACTIONS(4256),
    [anon_sym_match] = ACTIONS(4256),
    [anon_sym_mod] = ACTIONS(4256),
    [anon_sym_pub] = ACTIONS(4256),
    [anon_sym_return] = ACTIONS(4256),
    [anon_sym_raw] = ACTIONS(4256),
    [anon_sym_static] = ACTIONS(4256),
    [anon_sym_struct] = ACTIONS(4256),
    [anon_sym_trait] = ACTIONS(4256),
    [anon_sym_type] = ACTIONS(4256),
    [anon_sym_union] = ACTIONS(4256),
    [anon_sym_unsafe] = ACTIONS(4256),
    [anon_sym_use] = ACTIONS(4256),
    [anon_sym_while] = ACTIONS(4256),
    [anon_sym_extern] = ACTIONS(4256),
    [anon_sym_safe] = ACTIONS(4256),
    [anon_sym_yield] = ACTIONS(4256),
    [anon_sym_move] = ACTIONS(4256),
    [anon_sym_try] = ACTIONS(4256),
    [sym_integer_literal] = ACTIONS(4254),
    [aux_sym_string_literal_token1] = ACTIONS(4254),
    [sym_char_literal] = ACTIONS(4254),
    [anon_sym_true] = ACTIONS(4256),
    [anon_sym_false] = ACTIONS(4256),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4256),
    [sym_super] = ACTIONS(4256),
    [sym_crate] = ACTIONS(4256),
    [aux_sym_metavariable_token1] = ACTIONS(4254),
    [sym__raw_string_literal_start] = ACTIONS(4254),
    [sym_float_literal] = ACTIONS(4254),
    [sym__outer_block_doc_comment_start] = ACTIONS(4254),
    [sym__outer_line_doc_comment_start] = ACTIONS(4254),
  },
  [STATE(1124)] = {
    [sym_line_comment] = STATE(1124),
    [sym_block_comment] = STATE(1124),
    [ts_builtin_sym_end] = ACTIONS(4258),
    [sym_identifier] = ACTIONS(4260),
    [anon_sym_SEMI] = ACTIONS(4258),
    [anon_sym_macro] = ACTIONS(4260),
    [anon_sym_LPAREN] = ACTIONS(4258),
    [anon_sym_LBRACE] = ACTIONS(4258),
    [anon_sym_RBRACE] = ACTIONS(4258),
    [anon_sym_macro_rules] = ACTIONS(4260),
    [anon_sym_BANG] = ACTIONS(4258),
    [anon_sym_LBRACK] = ACTIONS(4258),
    [anon_sym_STAR] = ACTIONS(4258),
    [anon_sym_u8] = ACTIONS(4260),
    [anon_sym_i8] = ACTIONS(4260),
    [anon_sym_u16] = ACTIONS(4260),
    [anon_sym_i16] = ACTIONS(4260),
    [anon_sym_u32] = ACTIONS(4260),
    [anon_sym_i32] = ACTIONS(4260),
    [anon_sym_u64] = ACTIONS(4260),
    [anon_sym_i64] = ACTIONS(4260),
    [anon_sym_u128] = ACTIONS(4260),
    [anon_sym_i128] = ACTIONS(4260),
    [anon_sym_isize] = ACTIONS(4260),
    [anon_sym_usize] = ACTIONS(4260),
    [anon_sym_f32] = ACTIONS(4260),
    [anon_sym_f64] = ACTIONS(4260),
    [anon_sym_bool] = ACTIONS(4260),
    [anon_sym_str] = ACTIONS(4260),
    [anon_sym_char] = ACTIONS(4260),
    [anon_sym_DASH] = ACTIONS(4258),
    [anon_sym_AMP] = ACTIONS(4258),
    [anon_sym_PIPE] = ACTIONS(4258),
    [anon_sym_LT] = ACTIONS(4258),
    [anon_sym_DOT_DOT] = ACTIONS(4258),
    [anon_sym_COLON_COLON] = ACTIONS(4258),
    [anon_sym_POUND] = ACTIONS(4258),
    [anon_sym_SQUOTE] = ACTIONS(4260),
    [anon_sym_async] = ACTIONS(4260),
    [anon_sym_auto] = ACTIONS(4260),
    [anon_sym_await] = ACTIONS(4260),
    [anon_sym_become] = ACTIONS(4260),
    [anon_sym_break] = ACTIONS(4260),
    [anon_sym_const] = ACTIONS(4260),
    [anon_sym_continue] = ACTIONS(4260),
    [anon_sym_default] = ACTIONS(4260),
    [anon_sym_enum] = ACTIONS(4260),
    [anon_sym_fn] = ACTIONS(4260),
    [anon_sym_for] = ACTIONS(4260),
    [anon_sym_gen] = ACTIONS(4260),
    [anon_sym_if] = ACTIONS(4260),
    [anon_sym_impl] = ACTIONS(4260),
    [anon_sym_let] = ACTIONS(4260),
    [anon_sym_loop] = ACTIONS(4260),
    [anon_sym_match] = ACTIONS(4260),
    [anon_sym_mod] = ACTIONS(4260),
    [anon_sym_pub] = ACTIONS(4260),
    [anon_sym_return] = ACTIONS(4260),
    [anon_sym_raw] = ACTIONS(4260),
    [anon_sym_static] = ACTIONS(4260),
    [anon_sym_struct] = ACTIONS(4260),
    [anon_sym_trait] = ACTIONS(4260),
    [anon_sym_type] = ACTIONS(4260),
    [anon_sym_union] = ACTIONS(4260),
    [anon_sym_unsafe] = ACTIONS(4260),
    [anon_sym_use] = ACTIONS(4260),
    [anon_sym_while] = ACTIONS(4260),
    [anon_sym_extern] = ACTIONS(4260),
    [anon_sym_safe] = ACTIONS(4260),
    [anon_sym_yield] = ACTIONS(4260),
    [anon_sym_move] = ACTIONS(4260),
    [anon_sym_try] = ACTIONS(4260),
    [sym_integer_literal] = ACTIONS(4258),
    [aux_sym_string_literal_token1] = ACTIONS(4258),
    [sym_char_literal] = ACTIONS(4258),
    [anon_sym_true] = ACTIONS(4260),
    [anon_sym_false] = ACTIONS(4260),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4260),
    [sym_super] = ACTIONS(4260),
    [sym_crate] = ACTIONS(4260),
    [aux_sym_metavariable_token1] = ACTIONS(4258),
    [sym__raw_string_literal_start] = ACTIONS(4258),
    [sym_float_literal] = ACTIONS(4258),
    [sym__outer_block_doc_comment_start] = ACTIONS(4258),
    [sym__outer_line_doc_comment_start] = ACTIONS(4258),
  },
  [STATE(1125)] = {
    [sym_line_comment] = STATE(1125),
    [sym_block_comment] = STATE(1125),
    [ts_builtin_sym_end] = ACTIONS(4262),
    [sym_identifier] = ACTIONS(4264),
    [anon_sym_SEMI] = ACTIONS(4262),
    [anon_sym_macro] = ACTIONS(4264),
    [anon_sym_LPAREN] = ACTIONS(4262),
    [anon_sym_LBRACE] = ACTIONS(4262),
    [anon_sym_RBRACE] = ACTIONS(4262),
    [anon_sym_macro_rules] = ACTIONS(4264),
    [anon_sym_BANG] = ACTIONS(4262),
    [anon_sym_LBRACK] = ACTIONS(4262),
    [anon_sym_STAR] = ACTIONS(4262),
    [anon_sym_u8] = ACTIONS(4264),
    [anon_sym_i8] = ACTIONS(4264),
    [anon_sym_u16] = ACTIONS(4264),
    [anon_sym_i16] = ACTIONS(4264),
    [anon_sym_u32] = ACTIONS(4264),
    [anon_sym_i32] = ACTIONS(4264),
    [anon_sym_u64] = ACTIONS(4264),
    [anon_sym_i64] = ACTIONS(4264),
    [anon_sym_u128] = ACTIONS(4264),
    [anon_sym_i128] = ACTIONS(4264),
    [anon_sym_isize] = ACTIONS(4264),
    [anon_sym_usize] = ACTIONS(4264),
    [anon_sym_f32] = ACTIONS(4264),
    [anon_sym_f64] = ACTIONS(4264),
    [anon_sym_bool] = ACTIONS(4264),
    [anon_sym_str] = ACTIONS(4264),
    [anon_sym_char] = ACTIONS(4264),
    [anon_sym_DASH] = ACTIONS(4262),
    [anon_sym_AMP] = ACTIONS(4262),
    [anon_sym_PIPE] = ACTIONS(4262),
    [anon_sym_LT] = ACTIONS(4262),
    [anon_sym_DOT_DOT] = ACTIONS(4262),
    [anon_sym_COLON_COLON] = ACTIONS(4262),
    [anon_sym_POUND] = ACTIONS(4262),
    [anon_sym_SQUOTE] = ACTIONS(4264),
    [anon_sym_async] = ACTIONS(4264),
    [anon_sym_auto] = ACTIONS(4264),
    [anon_sym_await] = ACTIONS(4264),
    [anon_sym_become] = ACTIONS(4264),
    [anon_sym_break] = ACTIONS(4264),
    [anon_sym_const] = ACTIONS(4264),
    [anon_sym_continue] = ACTIONS(4264),
    [anon_sym_default] = ACTIONS(4264),
    [anon_sym_enum] = ACTIONS(4264),
    [anon_sym_fn] = ACTIONS(4264),
    [anon_sym_for] = ACTIONS(4264),
    [anon_sym_gen] = ACTIONS(4264),
    [anon_sym_if] = ACTIONS(4264),
    [anon_sym_impl] = ACTIONS(4264),
    [anon_sym_let] = ACTIONS(4264),
    [anon_sym_loop] = ACTIONS(4264),
    [anon_sym_match] = ACTIONS(4264),
    [anon_sym_mod] = ACTIONS(4264),
    [anon_sym_pub] = ACTIONS(4264),
    [anon_sym_return] = ACTIONS(4264),
    [anon_sym_raw] = ACTIONS(4264),
    [anon_sym_static] = ACTIONS(4264),
    [anon_sym_struct] = ACTIONS(4264),
    [anon_sym_trait] = ACTIONS(4264),
    [anon_sym_type] = ACTIONS(4264),
    [anon_sym_union] = ACTIONS(4264),
    [anon_sym_unsafe] = ACTIONS(4264),
    [anon_sym_use] = ACTIONS(4264),
    [anon_sym_while] = ACTIONS(4264),
    [anon_sym_extern] = ACTIONS(4264),
    [anon_sym_safe] = ACTIONS(4264),
    [anon_sym_yield] = ACTIONS(4264),
    [anon_sym_move] = ACTIONS(4264),
    [anon_sym_try] = ACTIONS(4264),
    [sym_integer_literal] = ACTIONS(4262),
    [aux_sym_string_literal_token1] = ACTIONS(4262),
    [sym_char_literal] = ACTIONS(4262),
    [anon_sym_true] = ACTIONS(4264),
    [anon_sym_false] = ACTIONS(4264),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4264),
    [sym_super] = ACTIONS(4264),
    [sym_crate] = ACTIONS(4264),
    [aux_sym_metavariable_token1] = ACTIONS(4262),
    [sym__raw_string_literal_start] = ACTIONS(4262),
    [sym_float_literal] = ACTIONS(4262),
    [sym__outer_block_doc_comment_start] = ACTIONS(4262),
    [sym__outer_line_doc_comment_start] = ACTIONS(4262),
  },
  [STATE(1126)] = {
    [sym_line_comment] = STATE(1126),
    [sym_block_comment] = STATE(1126),
    [ts_builtin_sym_end] = ACTIONS(4266),
    [sym_identifier] = ACTIONS(4268),
    [anon_sym_SEMI] = ACTIONS(4266),
    [anon_sym_macro] = ACTIONS(4268),
    [anon_sym_LPAREN] = ACTIONS(4266),
    [anon_sym_LBRACE] = ACTIONS(4266),
    [anon_sym_RBRACE] = ACTIONS(4266),
    [anon_sym_macro_rules] = ACTIONS(4268),
    [anon_sym_BANG] = ACTIONS(4266),
    [anon_sym_LBRACK] = ACTIONS(4266),
    [anon_sym_STAR] = ACTIONS(4266),
    [anon_sym_u8] = ACTIONS(4268),
    [anon_sym_i8] = ACTIONS(4268),
    [anon_sym_u16] = ACTIONS(4268),
    [anon_sym_i16] = ACTIONS(4268),
    [anon_sym_u32] = ACTIONS(4268),
    [anon_sym_i32] = ACTIONS(4268),
    [anon_sym_u64] = ACTIONS(4268),
    [anon_sym_i64] = ACTIONS(4268),
    [anon_sym_u128] = ACTIONS(4268),
    [anon_sym_i128] = ACTIONS(4268),
    [anon_sym_isize] = ACTIONS(4268),
    [anon_sym_usize] = ACTIONS(4268),
    [anon_sym_f32] = ACTIONS(4268),
    [anon_sym_f64] = ACTIONS(4268),
    [anon_sym_bool] = ACTIONS(4268),
    [anon_sym_str] = ACTIONS(4268),
    [anon_sym_char] = ACTIONS(4268),
    [anon_sym_DASH] = ACTIONS(4266),
    [anon_sym_AMP] = ACTIONS(4266),
    [anon_sym_PIPE] = ACTIONS(4266),
    [anon_sym_LT] = ACTIONS(4266),
    [anon_sym_DOT_DOT] = ACTIONS(4266),
    [anon_sym_COLON_COLON] = ACTIONS(4266),
    [anon_sym_POUND] = ACTIONS(4266),
    [anon_sym_SQUOTE] = ACTIONS(4268),
    [anon_sym_async] = ACTIONS(4268),
    [anon_sym_auto] = ACTIONS(4268),
    [anon_sym_await] = ACTIONS(4268),
    [anon_sym_become] = ACTIONS(4268),
    [anon_sym_break] = ACTIONS(4268),
    [anon_sym_const] = ACTIONS(4268),
    [anon_sym_continue] = ACTIONS(4268),
    [anon_sym_default] = ACTIONS(4268),
    [anon_sym_enum] = ACTIONS(4268),
    [anon_sym_fn] = ACTIONS(4268),
    [anon_sym_for] = ACTIONS(4268),
    [anon_sym_gen] = ACTIONS(4268),
    [anon_sym_if] = ACTIONS(4268),
    [anon_sym_impl] = ACTIONS(4268),
    [anon_sym_let] = ACTIONS(4268),
    [anon_sym_loop] = ACTIONS(4268),
    [anon_sym_match] = ACTIONS(4268),
    [anon_sym_mod] = ACTIONS(4268),
    [anon_sym_pub] = ACTIONS(4268),
    [anon_sym_return] = ACTIONS(4268),
    [anon_sym_raw] = ACTIONS(4268),
    [anon_sym_static] = ACTIONS(4268),
    [anon_sym_struct] = ACTIONS(4268),
    [anon_sym_trait] = ACTIONS(4268),
    [anon_sym_type] = ACTIONS(4268),
    [anon_sym_union] = ACTIONS(4268),
    [anon_sym_unsafe] = ACTIONS(4268),
    [anon_sym_use] = ACTIONS(4268),
    [anon_sym_while] = ACTIONS(4268),
    [anon_sym_extern] = ACTIONS(4268),
    [anon_sym_safe] = ACTIONS(4268),
    [anon_sym_yield] = ACTIONS(4268),
    [anon_sym_move] = ACTIONS(4268),
    [anon_sym_try] = ACTIONS(4268),
    [sym_integer_literal] = ACTIONS(4266),
    [aux_sym_string_literal_token1] = ACTIONS(4266),
    [sym_char_literal] = ACTIONS(4266),
    [anon_sym_true] = ACTIONS(4268),
    [anon_sym_false] = ACTIONS(4268),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4268),
    [sym_super] = ACTIONS(4268),
    [sym_crate] = ACTIONS(4268),
    [aux_sym_metavariable_token1] = ACTIONS(4266),
    [sym__raw_string_literal_start] = ACTIONS(4266),
    [sym_float_literal] = ACTIONS(4266),
    [sym__outer_block_doc_comment_start] = ACTIONS(4266),
    [sym__outer_line_doc_comment_start] = ACTIONS(4266),
  },
  [STATE(1127)] = {
    [sym_line_comment] = STATE(1127),
    [sym_block_comment] = STATE(1127),
    [ts_builtin_sym_end] = ACTIONS(4270),
    [sym_identifier] = ACTIONS(4272),
    [anon_sym_SEMI] = ACTIONS(4270),
    [anon_sym_macro] = ACTIONS(4272),
    [anon_sym_LPAREN] = ACTIONS(4270),
    [anon_sym_LBRACE] = ACTIONS(4270),
    [anon_sym_RBRACE] = ACTIONS(4270),
    [anon_sym_macro_rules] = ACTIONS(4272),
    [anon_sym_BANG] = ACTIONS(4270),
    [anon_sym_LBRACK] = ACTIONS(4270),
    [anon_sym_STAR] = ACTIONS(4270),
    [anon_sym_u8] = ACTIONS(4272),
    [anon_sym_i8] = ACTIONS(4272),
    [anon_sym_u16] = ACTIONS(4272),
    [anon_sym_i16] = ACTIONS(4272),
    [anon_sym_u32] = ACTIONS(4272),
    [anon_sym_i32] = ACTIONS(4272),
    [anon_sym_u64] = ACTIONS(4272),
    [anon_sym_i64] = ACTIONS(4272),
    [anon_sym_u128] = ACTIONS(4272),
    [anon_sym_i128] = ACTIONS(4272),
    [anon_sym_isize] = ACTIONS(4272),
    [anon_sym_usize] = ACTIONS(4272),
    [anon_sym_f32] = ACTIONS(4272),
    [anon_sym_f64] = ACTIONS(4272),
    [anon_sym_bool] = ACTIONS(4272),
    [anon_sym_str] = ACTIONS(4272),
    [anon_sym_char] = ACTIONS(4272),
    [anon_sym_DASH] = ACTIONS(4270),
    [anon_sym_AMP] = ACTIONS(4270),
    [anon_sym_PIPE] = ACTIONS(4270),
    [anon_sym_LT] = ACTIONS(4270),
    [anon_sym_DOT_DOT] = ACTIONS(4270),
    [anon_sym_COLON_COLON] = ACTIONS(4270),
    [anon_sym_POUND] = ACTIONS(4270),
    [anon_sym_SQUOTE] = ACTIONS(4272),
    [anon_sym_async] = ACTIONS(4272),
    [anon_sym_auto] = ACTIONS(4272),
    [anon_sym_await] = ACTIONS(4272),
    [anon_sym_become] = ACTIONS(4272),
    [anon_sym_break] = ACTIONS(4272),
    [anon_sym_const] = ACTIONS(4272),
    [anon_sym_continue] = ACTIONS(4272),
    [anon_sym_default] = ACTIONS(4272),
    [anon_sym_enum] = ACTIONS(4272),
    [anon_sym_fn] = ACTIONS(4272),
    [anon_sym_for] = ACTIONS(4272),
    [anon_sym_gen] = ACTIONS(4272),
    [anon_sym_if] = ACTIONS(4272),
    [anon_sym_impl] = ACTIONS(4272),
    [anon_sym_let] = ACTIONS(4272),
    [anon_sym_loop] = ACTIONS(4272),
    [anon_sym_match] = ACTIONS(4272),
    [anon_sym_mod] = ACTIONS(4272),
    [anon_sym_pub] = ACTIONS(4272),
    [anon_sym_return] = ACTIONS(4272),
    [anon_sym_raw] = ACTIONS(4272),
    [anon_sym_static] = ACTIONS(4272),
    [anon_sym_struct] = ACTIONS(4272),
    [anon_sym_trait] = ACTIONS(4272),
    [anon_sym_type] = ACTIONS(4272),
    [anon_sym_union] = ACTIONS(4272),
    [anon_sym_unsafe] = ACTIONS(4272),
    [anon_sym_use] = ACTIONS(4272),
    [anon_sym_while] = ACTIONS(4272),
    [anon_sym_extern] = ACTIONS(4272),
    [anon_sym_safe] = ACTIONS(4272),
    [anon_sym_yield] = ACTIONS(4272),
    [anon_sym_move] = ACTIONS(4272),
    [anon_sym_try] = ACTIONS(4272),
    [sym_integer_literal] = ACTIONS(4270),
    [aux_sym_string_literal_token1] = ACTIONS(4270),
    [sym_char_literal] = ACTIONS(4270),
    [anon_sym_true] = ACTIONS(4272),
    [anon_sym_false] = ACTIONS(4272),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4272),
    [sym_super] = ACTIONS(4272),
    [sym_crate] = ACTIONS(4272),
    [aux_sym_metavariable_token1] = ACTIONS(4270),
    [sym__raw_string_literal_start] = ACTIONS(4270),
    [sym_float_literal] = ACTIONS(4270),
    [sym__outer_block_doc_comment_start] = ACTIONS(4270),
    [sym__outer_line_doc_comment_start] = ACTIONS(4270),
  },
  [STATE(1128)] = {
    [sym_line_comment] = STATE(1128),
    [sym_block_comment] = STATE(1128),
    [ts_builtin_sym_end] = ACTIONS(4274),
    [sym_identifier] = ACTIONS(4276),
    [anon_sym_SEMI] = ACTIONS(4274),
    [anon_sym_macro] = ACTIONS(4276),
    [anon_sym_LPAREN] = ACTIONS(4274),
    [anon_sym_LBRACE] = ACTIONS(4274),
    [anon_sym_RBRACE] = ACTIONS(4274),
    [anon_sym_macro_rules] = ACTIONS(4276),
    [anon_sym_BANG] = ACTIONS(4274),
    [anon_sym_LBRACK] = ACTIONS(4274),
    [anon_sym_STAR] = ACTIONS(4274),
    [anon_sym_u8] = ACTIONS(4276),
    [anon_sym_i8] = ACTIONS(4276),
    [anon_sym_u16] = ACTIONS(4276),
    [anon_sym_i16] = ACTIONS(4276),
    [anon_sym_u32] = ACTIONS(4276),
    [anon_sym_i32] = ACTIONS(4276),
    [anon_sym_u64] = ACTIONS(4276),
    [anon_sym_i64] = ACTIONS(4276),
    [anon_sym_u128] = ACTIONS(4276),
    [anon_sym_i128] = ACTIONS(4276),
    [anon_sym_isize] = ACTIONS(4276),
    [anon_sym_usize] = ACTIONS(4276),
    [anon_sym_f32] = ACTIONS(4276),
    [anon_sym_f64] = ACTIONS(4276),
    [anon_sym_bool] = ACTIONS(4276),
    [anon_sym_str] = ACTIONS(4276),
    [anon_sym_char] = ACTIONS(4276),
    [anon_sym_DASH] = ACTIONS(4274),
    [anon_sym_AMP] = ACTIONS(4274),
    [anon_sym_PIPE] = ACTIONS(4274),
    [anon_sym_LT] = ACTIONS(4274),
    [anon_sym_DOT_DOT] = ACTIONS(4274),
    [anon_sym_COLON_COLON] = ACTIONS(4274),
    [anon_sym_POUND] = ACTIONS(4274),
    [anon_sym_SQUOTE] = ACTIONS(4276),
    [anon_sym_async] = ACTIONS(4276),
    [anon_sym_auto] = ACTIONS(4276),
    [anon_sym_await] = ACTIONS(4276),
    [anon_sym_become] = ACTIONS(4276),
    [anon_sym_break] = ACTIONS(4276),
    [anon_sym_const] = ACTIONS(4276),
    [anon_sym_continue] = ACTIONS(4276),
    [anon_sym_default] = ACTIONS(4276),
    [anon_sym_enum] = ACTIONS(4276),
    [anon_sym_fn] = ACTIONS(4276),
    [anon_sym_for] = ACTIONS(4276),
    [anon_sym_gen] = ACTIONS(4276),
    [anon_sym_if] = ACTIONS(4276),
    [anon_sym_impl] = ACTIONS(4276),
    [anon_sym_let] = ACTIONS(4276),
    [anon_sym_loop] = ACTIONS(4276),
    [anon_sym_match] = ACTIONS(4276),
    [anon_sym_mod] = ACTIONS(4276),
    [anon_sym_pub] = ACTIONS(4276),
    [anon_sym_return] = ACTIONS(4276),
    [anon_sym_raw] = ACTIONS(4276),
    [anon_sym_static] = ACTIONS(4276),
    [anon_sym_struct] = ACTIONS(4276),
    [anon_sym_trait] = ACTIONS(4276),
    [anon_sym_type] = ACTIONS(4276),
    [anon_sym_union] = ACTIONS(4276),
    [anon_sym_unsafe] = ACTIONS(4276),
    [anon_sym_use] = ACTIONS(4276),
    [anon_sym_while] = ACTIONS(4276),
    [anon_sym_extern] = ACTIONS(4276),
    [anon_sym_safe] = ACTIONS(4276),
    [anon_sym_yield] = ACTIONS(4276),
    [anon_sym_move] = ACTIONS(4276),
    [anon_sym_try] = ACTIONS(4276),
    [sym_integer_literal] = ACTIONS(4274),
    [aux_sym_string_literal_token1] = ACTIONS(4274),
    [sym_char_literal] = ACTIONS(4274),
    [anon_sym_true] = ACTIONS(4276),
    [anon_sym_false] = ACTIONS(4276),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4276),
    [sym_super] = ACTIONS(4276),
    [sym_crate] = ACTIONS(4276),
    [aux_sym_metavariable_token1] = ACTIONS(4274),
    [sym__raw_string_literal_start] = ACTIONS(4274),
    [sym_float_literal] = ACTIONS(4274),
    [sym__outer_block_doc_comment_start] = ACTIONS(4274),
    [sym__outer_line_doc_comment_start] = ACTIONS(4274),
  },
  [STATE(1129)] = {
    [sym_line_comment] = STATE(1129),
    [sym_block_comment] = STATE(1129),
    [ts_builtin_sym_end] = ACTIONS(4278),
    [sym_identifier] = ACTIONS(4280),
    [anon_sym_SEMI] = ACTIONS(4278),
    [anon_sym_macro] = ACTIONS(4280),
    [anon_sym_LPAREN] = ACTIONS(4278),
    [anon_sym_LBRACE] = ACTIONS(4278),
    [anon_sym_RBRACE] = ACTIONS(4278),
    [anon_sym_macro_rules] = ACTIONS(4280),
    [anon_sym_BANG] = ACTIONS(4278),
    [anon_sym_LBRACK] = ACTIONS(4278),
    [anon_sym_STAR] = ACTIONS(4278),
    [anon_sym_u8] = ACTIONS(4280),
    [anon_sym_i8] = ACTIONS(4280),
    [anon_sym_u16] = ACTIONS(4280),
    [anon_sym_i16] = ACTIONS(4280),
    [anon_sym_u32] = ACTIONS(4280),
    [anon_sym_i32] = ACTIONS(4280),
    [anon_sym_u64] = ACTIONS(4280),
    [anon_sym_i64] = ACTIONS(4280),
    [anon_sym_u128] = ACTIONS(4280),
    [anon_sym_i128] = ACTIONS(4280),
    [anon_sym_isize] = ACTIONS(4280),
    [anon_sym_usize] = ACTIONS(4280),
    [anon_sym_f32] = ACTIONS(4280),
    [anon_sym_f64] = ACTIONS(4280),
    [anon_sym_bool] = ACTIONS(4280),
    [anon_sym_str] = ACTIONS(4280),
    [anon_sym_char] = ACTIONS(4280),
    [anon_sym_DASH] = ACTIONS(4278),
    [anon_sym_AMP] = ACTIONS(4278),
    [anon_sym_PIPE] = ACTIONS(4278),
    [anon_sym_LT] = ACTIONS(4278),
    [anon_sym_DOT_DOT] = ACTIONS(4278),
    [anon_sym_COLON_COLON] = ACTIONS(4278),
    [anon_sym_POUND] = ACTIONS(4278),
    [anon_sym_SQUOTE] = ACTIONS(4280),
    [anon_sym_async] = ACTIONS(4280),
    [anon_sym_auto] = ACTIONS(4280),
    [anon_sym_await] = ACTIONS(4280),
    [anon_sym_become] = ACTIONS(4280),
    [anon_sym_break] = ACTIONS(4280),
    [anon_sym_const] = ACTIONS(4280),
    [anon_sym_continue] = ACTIONS(4280),
    [anon_sym_default] = ACTIONS(4280),
    [anon_sym_enum] = ACTIONS(4280),
    [anon_sym_fn] = ACTIONS(4280),
    [anon_sym_for] = ACTIONS(4280),
    [anon_sym_gen] = ACTIONS(4280),
    [anon_sym_if] = ACTIONS(4280),
    [anon_sym_impl] = ACTIONS(4280),
    [anon_sym_let] = ACTIONS(4280),
    [anon_sym_loop] = ACTIONS(4280),
    [anon_sym_match] = ACTIONS(4280),
    [anon_sym_mod] = ACTIONS(4280),
    [anon_sym_pub] = ACTIONS(4280),
    [anon_sym_return] = ACTIONS(4280),
    [anon_sym_raw] = ACTIONS(4280),
    [anon_sym_static] = ACTIONS(4280),
    [anon_sym_struct] = ACTIONS(4280),
    [anon_sym_trait] = ACTIONS(4280),
    [anon_sym_type] = ACTIONS(4280),
    [anon_sym_union] = ACTIONS(4280),
    [anon_sym_unsafe] = ACTIONS(4280),
    [anon_sym_use] = ACTIONS(4280),
    [anon_sym_while] = ACTIONS(4280),
    [anon_sym_extern] = ACTIONS(4280),
    [anon_sym_safe] = ACTIONS(4280),
    [anon_sym_yield] = ACTIONS(4280),
    [anon_sym_move] = ACTIONS(4280),
    [anon_sym_try] = ACTIONS(4280),
    [sym_integer_literal] = ACTIONS(4278),
    [aux_sym_string_literal_token1] = ACTIONS(4278),
    [sym_char_literal] = ACTIONS(4278),
    [anon_sym_true] = ACTIONS(4280),
    [anon_sym_false] = ACTIONS(4280),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4280),
    [sym_super] = ACTIONS(4280),
    [sym_crate] = ACTIONS(4280),
    [aux_sym_metavariable_token1] = ACTIONS(4278),
    [sym__raw_string_literal_start] = ACTIONS(4278),
    [sym_float_literal] = ACTIONS(4278),
    [sym__outer_block_doc_comment_start] = ACTIONS(4278),
    [sym__outer_line_doc_comment_start] = ACTIONS(4278),
  },
  [STATE(1130)] = {
    [sym_line_comment] = STATE(1130),
    [sym_block_comment] = STATE(1130),
    [ts_builtin_sym_end] = ACTIONS(4282),
    [sym_identifier] = ACTIONS(4284),
    [anon_sym_SEMI] = ACTIONS(4282),
    [anon_sym_macro] = ACTIONS(4284),
    [anon_sym_LPAREN] = ACTIONS(4282),
    [anon_sym_LBRACE] = ACTIONS(4282),
    [anon_sym_RBRACE] = ACTIONS(4282),
    [anon_sym_macro_rules] = ACTIONS(4284),
    [anon_sym_BANG] = ACTIONS(4282),
    [anon_sym_LBRACK] = ACTIONS(4282),
    [anon_sym_STAR] = ACTIONS(4282),
    [anon_sym_u8] = ACTIONS(4284),
    [anon_sym_i8] = ACTIONS(4284),
    [anon_sym_u16] = ACTIONS(4284),
    [anon_sym_i16] = ACTIONS(4284),
    [anon_sym_u32] = ACTIONS(4284),
    [anon_sym_i32] = ACTIONS(4284),
    [anon_sym_u64] = ACTIONS(4284),
    [anon_sym_i64] = ACTIONS(4284),
    [anon_sym_u128] = ACTIONS(4284),
    [anon_sym_i128] = ACTIONS(4284),
    [anon_sym_isize] = ACTIONS(4284),
    [anon_sym_usize] = ACTIONS(4284),
    [anon_sym_f32] = ACTIONS(4284),
    [anon_sym_f64] = ACTIONS(4284),
    [anon_sym_bool] = ACTIONS(4284),
    [anon_sym_str] = ACTIONS(4284),
    [anon_sym_char] = ACTIONS(4284),
    [anon_sym_DASH] = ACTIONS(4282),
    [anon_sym_AMP] = ACTIONS(4282),
    [anon_sym_PIPE] = ACTIONS(4282),
    [anon_sym_LT] = ACTIONS(4282),
    [anon_sym_DOT_DOT] = ACTIONS(4282),
    [anon_sym_COLON_COLON] = ACTIONS(4282),
    [anon_sym_POUND] = ACTIONS(4282),
    [anon_sym_SQUOTE] = ACTIONS(4284),
    [anon_sym_async] = ACTIONS(4284),
    [anon_sym_auto] = ACTIONS(4284),
    [anon_sym_await] = ACTIONS(4284),
    [anon_sym_become] = ACTIONS(4284),
    [anon_sym_break] = ACTIONS(4284),
    [anon_sym_const] = ACTIONS(4284),
    [anon_sym_continue] = ACTIONS(4284),
    [anon_sym_default] = ACTIONS(4284),
    [anon_sym_enum] = ACTIONS(4284),
    [anon_sym_fn] = ACTIONS(4284),
    [anon_sym_for] = ACTIONS(4284),
    [anon_sym_gen] = ACTIONS(4284),
    [anon_sym_if] = ACTIONS(4284),
    [anon_sym_impl] = ACTIONS(4284),
    [anon_sym_let] = ACTIONS(4284),
    [anon_sym_loop] = ACTIONS(4284),
    [anon_sym_match] = ACTIONS(4284),
    [anon_sym_mod] = ACTIONS(4284),
    [anon_sym_pub] = ACTIONS(4284),
    [anon_sym_return] = ACTIONS(4284),
    [anon_sym_raw] = ACTIONS(4284),
    [anon_sym_static] = ACTIONS(4284),
    [anon_sym_struct] = ACTIONS(4284),
    [anon_sym_trait] = ACTIONS(4284),
    [anon_sym_type] = ACTIONS(4284),
    [anon_sym_union] = ACTIONS(4284),
    [anon_sym_unsafe] = ACTIONS(4284),
    [anon_sym_use] = ACTIONS(4284),
    [anon_sym_while] = ACTIONS(4284),
    [anon_sym_extern] = ACTIONS(4284),
    [anon_sym_safe] = ACTIONS(4284),
    [anon_sym_yield] = ACTIONS(4284),
    [anon_sym_move] = ACTIONS(4284),
    [anon_sym_try] = ACTIONS(4284),
    [sym_integer_literal] = ACTIONS(4282),
    [aux_sym_string_literal_token1] = ACTIONS(4282),
    [sym_char_literal] = ACTIONS(4282),
    [anon_sym_true] = ACTIONS(4284),
    [anon_sym_false] = ACTIONS(4284),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4284),
    [sym_super] = ACTIONS(4284),
    [sym_crate] = ACTIONS(4284),
    [aux_sym_metavariable_token1] = ACTIONS(4282),
    [sym__raw_string_literal_start] = ACTIONS(4282),
    [sym_float_literal] = ACTIONS(4282),
    [sym__outer_block_doc_comment_start] = ACTIONS(4282),
    [sym__outer_line_doc_comment_start] = ACTIONS(4282),
  },
  [STATE(1131)] = {
    [sym_line_comment] = STATE(1131),
    [sym_block_comment] = STATE(1131),
    [ts_builtin_sym_end] = ACTIONS(4286),
    [sym_identifier] = ACTIONS(4288),
    [anon_sym_SEMI] = ACTIONS(4286),
    [anon_sym_macro] = ACTIONS(4288),
    [anon_sym_LPAREN] = ACTIONS(4286),
    [anon_sym_LBRACE] = ACTIONS(4286),
    [anon_sym_RBRACE] = ACTIONS(4286),
    [anon_sym_macro_rules] = ACTIONS(4288),
    [anon_sym_BANG] = ACTIONS(4286),
    [anon_sym_LBRACK] = ACTIONS(4286),
    [anon_sym_STAR] = ACTIONS(4286),
    [anon_sym_u8] = ACTIONS(4288),
    [anon_sym_i8] = ACTIONS(4288),
    [anon_sym_u16] = ACTIONS(4288),
    [anon_sym_i16] = ACTIONS(4288),
    [anon_sym_u32] = ACTIONS(4288),
    [anon_sym_i32] = ACTIONS(4288),
    [anon_sym_u64] = ACTIONS(4288),
    [anon_sym_i64] = ACTIONS(4288),
    [anon_sym_u128] = ACTIONS(4288),
    [anon_sym_i128] = ACTIONS(4288),
    [anon_sym_isize] = ACTIONS(4288),
    [anon_sym_usize] = ACTIONS(4288),
    [anon_sym_f32] = ACTIONS(4288),
    [anon_sym_f64] = ACTIONS(4288),
    [anon_sym_bool] = ACTIONS(4288),
    [anon_sym_str] = ACTIONS(4288),
    [anon_sym_char] = ACTIONS(4288),
    [anon_sym_DASH] = ACTIONS(4286),
    [anon_sym_AMP] = ACTIONS(4286),
    [anon_sym_PIPE] = ACTIONS(4286),
    [anon_sym_LT] = ACTIONS(4286),
    [anon_sym_DOT_DOT] = ACTIONS(4286),
    [anon_sym_COLON_COLON] = ACTIONS(4286),
    [anon_sym_POUND] = ACTIONS(4286),
    [anon_sym_SQUOTE] = ACTIONS(4288),
    [anon_sym_async] = ACTIONS(4288),
    [anon_sym_auto] = ACTIONS(4288),
    [anon_sym_await] = ACTIONS(4288),
    [anon_sym_become] = ACTIONS(4288),
    [anon_sym_break] = ACTIONS(4288),
    [anon_sym_const] = ACTIONS(4288),
    [anon_sym_continue] = ACTIONS(4288),
    [anon_sym_default] = ACTIONS(4288),
    [anon_sym_enum] = ACTIONS(4288),
    [anon_sym_fn] = ACTIONS(4288),
    [anon_sym_for] = ACTIONS(4288),
    [anon_sym_gen] = ACTIONS(4288),
    [anon_sym_if] = ACTIONS(4288),
    [anon_sym_impl] = ACTIONS(4288),
    [anon_sym_let] = ACTIONS(4288),
    [anon_sym_loop] = ACTIONS(4288),
    [anon_sym_match] = ACTIONS(4288),
    [anon_sym_mod] = ACTIONS(4288),
    [anon_sym_pub] = ACTIONS(4288),
    [anon_sym_return] = ACTIONS(4288),
    [anon_sym_raw] = ACTIONS(4288),
    [anon_sym_static] = ACTIONS(4288),
    [anon_sym_struct] = ACTIONS(4288),
    [anon_sym_trait] = ACTIONS(4288),
    [anon_sym_type] = ACTIONS(4288),
    [anon_sym_union] = ACTIONS(4288),
    [anon_sym_unsafe] = ACTIONS(4288),
    [anon_sym_use] = ACTIONS(4288),
    [anon_sym_while] = ACTIONS(4288),
    [anon_sym_extern] = ACTIONS(4288),
    [anon_sym_safe] = ACTIONS(4288),
    [anon_sym_yield] = ACTIONS(4288),
    [anon_sym_move] = ACTIONS(4288),
    [anon_sym_try] = ACTIONS(4288),
    [sym_integer_literal] = ACTIONS(4286),
    [aux_sym_string_literal_token1] = ACTIONS(4286),
    [sym_char_literal] = ACTIONS(4286),
    [anon_sym_true] = ACTIONS(4288),
    [anon_sym_false] = ACTIONS(4288),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4288),
    [sym_super] = ACTIONS(4288),
    [sym_crate] = ACTIONS(4288),
    [aux_sym_metavariable_token1] = ACTIONS(4286),
    [sym__raw_string_literal_start] = ACTIONS(4286),
    [sym_float_literal] = ACTIONS(4286),
    [sym__outer_block_doc_comment_start] = ACTIONS(4286),
    [sym__outer_line_doc_comment_start] = ACTIONS(4286),
  },
  [STATE(1132)] = {
    [sym_line_comment] = STATE(1132),
    [sym_block_comment] = STATE(1132),
    [ts_builtin_sym_end] = ACTIONS(4290),
    [sym_identifier] = ACTIONS(4292),
    [anon_sym_SEMI] = ACTIONS(4290),
    [anon_sym_macro] = ACTIONS(4292),
    [anon_sym_LPAREN] = ACTIONS(4290),
    [anon_sym_LBRACE] = ACTIONS(4290),
    [anon_sym_RBRACE] = ACTIONS(4290),
    [anon_sym_macro_rules] = ACTIONS(4292),
    [anon_sym_BANG] = ACTIONS(4290),
    [anon_sym_LBRACK] = ACTIONS(4290),
    [anon_sym_STAR] = ACTIONS(4290),
    [anon_sym_u8] = ACTIONS(4292),
    [anon_sym_i8] = ACTIONS(4292),
    [anon_sym_u16] = ACTIONS(4292),
    [anon_sym_i16] = ACTIONS(4292),
    [anon_sym_u32] = ACTIONS(4292),
    [anon_sym_i32] = ACTIONS(4292),
    [anon_sym_u64] = ACTIONS(4292),
    [anon_sym_i64] = ACTIONS(4292),
    [anon_sym_u128] = ACTIONS(4292),
    [anon_sym_i128] = ACTIONS(4292),
    [anon_sym_isize] = ACTIONS(4292),
    [anon_sym_usize] = ACTIONS(4292),
    [anon_sym_f32] = ACTIONS(4292),
    [anon_sym_f64] = ACTIONS(4292),
    [anon_sym_bool] = ACTIONS(4292),
    [anon_sym_str] = ACTIONS(4292),
    [anon_sym_char] = ACTIONS(4292),
    [anon_sym_DASH] = ACTIONS(4290),
    [anon_sym_AMP] = ACTIONS(4290),
    [anon_sym_PIPE] = ACTIONS(4290),
    [anon_sym_LT] = ACTIONS(4290),
    [anon_sym_DOT_DOT] = ACTIONS(4290),
    [anon_sym_COLON_COLON] = ACTIONS(4290),
    [anon_sym_POUND] = ACTIONS(4290),
    [anon_sym_SQUOTE] = ACTIONS(4292),
    [anon_sym_async] = ACTIONS(4292),
    [anon_sym_auto] = ACTIONS(4292),
    [anon_sym_await] = ACTIONS(4292),
    [anon_sym_become] = ACTIONS(4292),
    [anon_sym_break] = ACTIONS(4292),
    [anon_sym_const] = ACTIONS(4292),
    [anon_sym_continue] = ACTIONS(4292),
    [anon_sym_default] = ACTIONS(4292),
    [anon_sym_enum] = ACTIONS(4292),
    [anon_sym_fn] = ACTIONS(4292),
    [anon_sym_for] = ACTIONS(4292),
    [anon_sym_gen] = ACTIONS(4292),
    [anon_sym_if] = ACTIONS(4292),
    [anon_sym_impl] = ACTIONS(4292),
    [anon_sym_let] = ACTIONS(4292),
    [anon_sym_loop] = ACTIONS(4292),
    [anon_sym_match] = ACTIONS(4292),
    [anon_sym_mod] = ACTIONS(4292),
    [anon_sym_pub] = ACTIONS(4292),
    [anon_sym_return] = ACTIONS(4292),
    [anon_sym_raw] = ACTIONS(4292),
    [anon_sym_static] = ACTIONS(4292),
    [anon_sym_struct] = ACTIONS(4292),
    [anon_sym_trait] = ACTIONS(4292),
    [anon_sym_type] = ACTIONS(4292),
    [anon_sym_union] = ACTIONS(4292),
    [anon_sym_unsafe] = ACTIONS(4292),
    [anon_sym_use] = ACTIONS(4292),
    [anon_sym_while] = ACTIONS(4292),
    [anon_sym_extern] = ACTIONS(4292),
    [anon_sym_safe] = ACTIONS(4292),
    [anon_sym_yield] = ACTIONS(4292),
    [anon_sym_move] = ACTIONS(4292),
    [anon_sym_try] = ACTIONS(4292),
    [sym_integer_literal] = ACTIONS(4290),
    [aux_sym_string_literal_token1] = ACTIONS(4290),
    [sym_char_literal] = ACTIONS(4290),
    [anon_sym_true] = ACTIONS(4292),
    [anon_sym_false] = ACTIONS(4292),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4292),
    [sym_super] = ACTIONS(4292),
    [sym_crate] = ACTIONS(4292),
    [aux_sym_metavariable_token1] = ACTIONS(4290),
    [sym__raw_string_literal_start] = ACTIONS(4290),
    [sym_float_literal] = ACTIONS(4290),
    [sym__outer_block_doc_comment_start] = ACTIONS(4290),
    [sym__outer_line_doc_comment_start] = ACTIONS(4290),
  },
  [STATE(1133)] = {
    [sym_line_comment] = STATE(1133),
    [sym_block_comment] = STATE(1133),
    [ts_builtin_sym_end] = ACTIONS(4294),
    [sym_identifier] = ACTIONS(4296),
    [anon_sym_SEMI] = ACTIONS(4294),
    [anon_sym_macro] = ACTIONS(4296),
    [anon_sym_LPAREN] = ACTIONS(4294),
    [anon_sym_LBRACE] = ACTIONS(4294),
    [anon_sym_RBRACE] = ACTIONS(4294),
    [anon_sym_macro_rules] = ACTIONS(4296),
    [anon_sym_BANG] = ACTIONS(4294),
    [anon_sym_LBRACK] = ACTIONS(4294),
    [anon_sym_STAR] = ACTIONS(4294),
    [anon_sym_u8] = ACTIONS(4296),
    [anon_sym_i8] = ACTIONS(4296),
    [anon_sym_u16] = ACTIONS(4296),
    [anon_sym_i16] = ACTIONS(4296),
    [anon_sym_u32] = ACTIONS(4296),
    [anon_sym_i32] = ACTIONS(4296),
    [anon_sym_u64] = ACTIONS(4296),
    [anon_sym_i64] = ACTIONS(4296),
    [anon_sym_u128] = ACTIONS(4296),
    [anon_sym_i128] = ACTIONS(4296),
    [anon_sym_isize] = ACTIONS(4296),
    [anon_sym_usize] = ACTIONS(4296),
    [anon_sym_f32] = ACTIONS(4296),
    [anon_sym_f64] = ACTIONS(4296),
    [anon_sym_bool] = ACTIONS(4296),
    [anon_sym_str] = ACTIONS(4296),
    [anon_sym_char] = ACTIONS(4296),
    [anon_sym_DASH] = ACTIONS(4294),
    [anon_sym_AMP] = ACTIONS(4294),
    [anon_sym_PIPE] = ACTIONS(4294),
    [anon_sym_LT] = ACTIONS(4294),
    [anon_sym_DOT_DOT] = ACTIONS(4294),
    [anon_sym_COLON_COLON] = ACTIONS(4294),
    [anon_sym_POUND] = ACTIONS(4294),
    [anon_sym_SQUOTE] = ACTIONS(4296),
    [anon_sym_async] = ACTIONS(4296),
    [anon_sym_auto] = ACTIONS(4296),
    [anon_sym_await] = ACTIONS(4296),
    [anon_sym_become] = ACTIONS(4296),
    [anon_sym_break] = ACTIONS(4296),
    [anon_sym_const] = ACTIONS(4296),
    [anon_sym_continue] = ACTIONS(4296),
    [anon_sym_default] = ACTIONS(4296),
    [anon_sym_enum] = ACTIONS(4296),
    [anon_sym_fn] = ACTIONS(4296),
    [anon_sym_for] = ACTIONS(4296),
    [anon_sym_gen] = ACTIONS(4296),
    [anon_sym_if] = ACTIONS(4296),
    [anon_sym_impl] = ACTIONS(4296),
    [anon_sym_let] = ACTIONS(4296),
    [anon_sym_loop] = ACTIONS(4296),
    [anon_sym_match] = ACTIONS(4296),
    [anon_sym_mod] = ACTIONS(4296),
    [anon_sym_pub] = ACTIONS(4296),
    [anon_sym_return] = ACTIONS(4296),
    [anon_sym_raw] = ACTIONS(4296),
    [anon_sym_static] = ACTIONS(4296),
    [anon_sym_struct] = ACTIONS(4296),
    [anon_sym_trait] = ACTIONS(4296),
    [anon_sym_type] = ACTIONS(4296),
    [anon_sym_union] = ACTIONS(4296),
    [anon_sym_unsafe] = ACTIONS(4296),
    [anon_sym_use] = ACTIONS(4296),
    [anon_sym_while] = ACTIONS(4296),
    [anon_sym_extern] = ACTIONS(4296),
    [anon_sym_safe] = ACTIONS(4296),
    [anon_sym_yield] = ACTIONS(4296),
    [anon_sym_move] = ACTIONS(4296),
    [anon_sym_try] = ACTIONS(4296),
    [sym_integer_literal] = ACTIONS(4294),
    [aux_sym_string_literal_token1] = ACTIONS(4294),
    [sym_char_literal] = ACTIONS(4294),
    [anon_sym_true] = ACTIONS(4296),
    [anon_sym_false] = ACTIONS(4296),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4296),
    [sym_super] = ACTIONS(4296),
    [sym_crate] = ACTIONS(4296),
    [aux_sym_metavariable_token1] = ACTIONS(4294),
    [sym__raw_string_literal_start] = ACTIONS(4294),
    [sym_float_literal] = ACTIONS(4294),
    [sym__outer_block_doc_comment_start] = ACTIONS(4294),
    [sym__outer_line_doc_comment_start] = ACTIONS(4294),
  },
  [STATE(1134)] = {
    [sym_line_comment] = STATE(1134),
    [sym_block_comment] = STATE(1134),
    [ts_builtin_sym_end] = ACTIONS(4298),
    [sym_identifier] = ACTIONS(4300),
    [anon_sym_SEMI] = ACTIONS(4298),
    [anon_sym_macro] = ACTIONS(4300),
    [anon_sym_LPAREN] = ACTIONS(4298),
    [anon_sym_LBRACE] = ACTIONS(4298),
    [anon_sym_RBRACE] = ACTIONS(4298),
    [anon_sym_macro_rules] = ACTIONS(4300),
    [anon_sym_BANG] = ACTIONS(4298),
    [anon_sym_LBRACK] = ACTIONS(4298),
    [anon_sym_STAR] = ACTIONS(4298),
    [anon_sym_u8] = ACTIONS(4300),
    [anon_sym_i8] = ACTIONS(4300),
    [anon_sym_u16] = ACTIONS(4300),
    [anon_sym_i16] = ACTIONS(4300),
    [anon_sym_u32] = ACTIONS(4300),
    [anon_sym_i32] = ACTIONS(4300),
    [anon_sym_u64] = ACTIONS(4300),
    [anon_sym_i64] = ACTIONS(4300),
    [anon_sym_u128] = ACTIONS(4300),
    [anon_sym_i128] = ACTIONS(4300),
    [anon_sym_isize] = ACTIONS(4300),
    [anon_sym_usize] = ACTIONS(4300),
    [anon_sym_f32] = ACTIONS(4300),
    [anon_sym_f64] = ACTIONS(4300),
    [anon_sym_bool] = ACTIONS(4300),
    [anon_sym_str] = ACTIONS(4300),
    [anon_sym_char] = ACTIONS(4300),
    [anon_sym_DASH] = ACTIONS(4298),
    [anon_sym_AMP] = ACTIONS(4298),
    [anon_sym_PIPE] = ACTIONS(4298),
    [anon_sym_LT] = ACTIONS(4298),
    [anon_sym_DOT_DOT] = ACTIONS(4298),
    [anon_sym_COLON_COLON] = ACTIONS(4298),
    [anon_sym_POUND] = ACTIONS(4298),
    [anon_sym_SQUOTE] = ACTIONS(4300),
    [anon_sym_async] = ACTIONS(4300),
    [anon_sym_auto] = ACTIONS(4300),
    [anon_sym_await] = ACTIONS(4300),
    [anon_sym_become] = ACTIONS(4300),
    [anon_sym_break] = ACTIONS(4300),
    [anon_sym_const] = ACTIONS(4300),
    [anon_sym_continue] = ACTIONS(4300),
    [anon_sym_default] = ACTIONS(4300),
    [anon_sym_enum] = ACTIONS(4300),
    [anon_sym_fn] = ACTIONS(4300),
    [anon_sym_for] = ACTIONS(4300),
    [anon_sym_gen] = ACTIONS(4300),
    [anon_sym_if] = ACTIONS(4300),
    [anon_sym_impl] = ACTIONS(4300),
    [anon_sym_let] = ACTIONS(4300),
    [anon_sym_loop] = ACTIONS(4300),
    [anon_sym_match] = ACTIONS(4300),
    [anon_sym_mod] = ACTIONS(4300),
    [anon_sym_pub] = ACTIONS(4300),
    [anon_sym_return] = ACTIONS(4300),
    [anon_sym_raw] = ACTIONS(4300),
    [anon_sym_static] = ACTIONS(4300),
    [anon_sym_struct] = ACTIONS(4300),
    [anon_sym_trait] = ACTIONS(4300),
    [anon_sym_type] = ACTIONS(4300),
    [anon_sym_union] = ACTIONS(4300),
    [anon_sym_unsafe] = ACTIONS(4300),
    [anon_sym_use] = ACTIONS(4300),
    [anon_sym_while] = ACTIONS(4300),
    [anon_sym_extern] = ACTIONS(4300),
    [anon_sym_safe] = ACTIONS(4300),
    [anon_sym_yield] = ACTIONS(4300),
    [anon_sym_move] = ACTIONS(4300),
    [anon_sym_try] = ACTIONS(4300),
    [sym_integer_literal] = ACTIONS(4298),
    [aux_sym_string_literal_token1] = ACTIONS(4298),
    [sym_char_literal] = ACTIONS(4298),
    [anon_sym_true] = ACTIONS(4300),
    [anon_sym_false] = ACTIONS(4300),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4300),
    [sym_super] = ACTIONS(4300),
    [sym_crate] = ACTIONS(4300),
    [aux_sym_metavariable_token1] = ACTIONS(4298),
    [sym__raw_string_literal_start] = ACTIONS(4298),
    [sym_float_literal] = ACTIONS(4298),
    [sym__outer_block_doc_comment_start] = ACTIONS(4298),
    [sym__outer_line_doc_comment_start] = ACTIONS(4298),
  },
  [STATE(1135)] = {
    [sym_line_comment] = STATE(1135),
    [sym_block_comment] = STATE(1135),
    [ts_builtin_sym_end] = ACTIONS(4302),
    [sym_identifier] = ACTIONS(4304),
    [anon_sym_SEMI] = ACTIONS(4302),
    [anon_sym_macro] = ACTIONS(4304),
    [anon_sym_LPAREN] = ACTIONS(4302),
    [anon_sym_LBRACE] = ACTIONS(4302),
    [anon_sym_RBRACE] = ACTIONS(4302),
    [anon_sym_macro_rules] = ACTIONS(4304),
    [anon_sym_BANG] = ACTIONS(4302),
    [anon_sym_LBRACK] = ACTIONS(4302),
    [anon_sym_STAR] = ACTIONS(4302),
    [anon_sym_u8] = ACTIONS(4304),
    [anon_sym_i8] = ACTIONS(4304),
    [anon_sym_u16] = ACTIONS(4304),
    [anon_sym_i16] = ACTIONS(4304),
    [anon_sym_u32] = ACTIONS(4304),
    [anon_sym_i32] = ACTIONS(4304),
    [anon_sym_u64] = ACTIONS(4304),
    [anon_sym_i64] = ACTIONS(4304),
    [anon_sym_u128] = ACTIONS(4304),
    [anon_sym_i128] = ACTIONS(4304),
    [anon_sym_isize] = ACTIONS(4304),
    [anon_sym_usize] = ACTIONS(4304),
    [anon_sym_f32] = ACTIONS(4304),
    [anon_sym_f64] = ACTIONS(4304),
    [anon_sym_bool] = ACTIONS(4304),
    [anon_sym_str] = ACTIONS(4304),
    [anon_sym_char] = ACTIONS(4304),
    [anon_sym_DASH] = ACTIONS(4302),
    [anon_sym_AMP] = ACTIONS(4302),
    [anon_sym_PIPE] = ACTIONS(4302),
    [anon_sym_LT] = ACTIONS(4302),
    [anon_sym_DOT_DOT] = ACTIONS(4302),
    [anon_sym_COLON_COLON] = ACTIONS(4302),
    [anon_sym_POUND] = ACTIONS(4302),
    [anon_sym_SQUOTE] = ACTIONS(4304),
    [anon_sym_async] = ACTIONS(4304),
    [anon_sym_auto] = ACTIONS(4304),
    [anon_sym_await] = ACTIONS(4304),
    [anon_sym_become] = ACTIONS(4304),
    [anon_sym_break] = ACTIONS(4304),
    [anon_sym_const] = ACTIONS(4304),
    [anon_sym_continue] = ACTIONS(4304),
    [anon_sym_default] = ACTIONS(4304),
    [anon_sym_enum] = ACTIONS(4304),
    [anon_sym_fn] = ACTIONS(4304),
    [anon_sym_for] = ACTIONS(4304),
    [anon_sym_gen] = ACTIONS(4304),
    [anon_sym_if] = ACTIONS(4304),
    [anon_sym_impl] = ACTIONS(4304),
    [anon_sym_let] = ACTIONS(4304),
    [anon_sym_loop] = ACTIONS(4304),
    [anon_sym_match] = ACTIONS(4304),
    [anon_sym_mod] = ACTIONS(4304),
    [anon_sym_pub] = ACTIONS(4304),
    [anon_sym_return] = ACTIONS(4304),
    [anon_sym_raw] = ACTIONS(4304),
    [anon_sym_static] = ACTIONS(4304),
    [anon_sym_struct] = ACTIONS(4304),
    [anon_sym_trait] = ACTIONS(4304),
    [anon_sym_type] = ACTIONS(4304),
    [anon_sym_union] = ACTIONS(4304),
    [anon_sym_unsafe] = ACTIONS(4304),
    [anon_sym_use] = ACTIONS(4304),
    [anon_sym_while] = ACTIONS(4304),
    [anon_sym_extern] = ACTIONS(4304),
    [anon_sym_safe] = ACTIONS(4304),
    [anon_sym_yield] = ACTIONS(4304),
    [anon_sym_move] = ACTIONS(4304),
    [anon_sym_try] = ACTIONS(4304),
    [sym_integer_literal] = ACTIONS(4302),
    [aux_sym_string_literal_token1] = ACTIONS(4302),
    [sym_char_literal] = ACTIONS(4302),
    [anon_sym_true] = ACTIONS(4304),
    [anon_sym_false] = ACTIONS(4304),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4304),
    [sym_super] = ACTIONS(4304),
    [sym_crate] = ACTIONS(4304),
    [aux_sym_metavariable_token1] = ACTIONS(4302),
    [sym__raw_string_literal_start] = ACTIONS(4302),
    [sym_float_literal] = ACTIONS(4302),
    [sym__outer_block_doc_comment_start] = ACTIONS(4302),
    [sym__outer_line_doc_comment_start] = ACTIONS(4302),
  },
  [STATE(1136)] = {
    [sym_line_comment] = STATE(1136),
    [sym_block_comment] = STATE(1136),
    [ts_builtin_sym_end] = ACTIONS(4306),
    [sym_identifier] = ACTIONS(4308),
    [anon_sym_SEMI] = ACTIONS(4306),
    [anon_sym_macro] = ACTIONS(4308),
    [anon_sym_LPAREN] = ACTIONS(4306),
    [anon_sym_LBRACE] = ACTIONS(4306),
    [anon_sym_RBRACE] = ACTIONS(4306),
    [anon_sym_macro_rules] = ACTIONS(4308),
    [anon_sym_BANG] = ACTIONS(4306),
    [anon_sym_LBRACK] = ACTIONS(4306),
    [anon_sym_STAR] = ACTIONS(4306),
    [anon_sym_u8] = ACTIONS(4308),
    [anon_sym_i8] = ACTIONS(4308),
    [anon_sym_u16] = ACTIONS(4308),
    [anon_sym_i16] = ACTIONS(4308),
    [anon_sym_u32] = ACTIONS(4308),
    [anon_sym_i32] = ACTIONS(4308),
    [anon_sym_u64] = ACTIONS(4308),
    [anon_sym_i64] = ACTIONS(4308),
    [anon_sym_u128] = ACTIONS(4308),
    [anon_sym_i128] = ACTIONS(4308),
    [anon_sym_isize] = ACTIONS(4308),
    [anon_sym_usize] = ACTIONS(4308),
    [anon_sym_f32] = ACTIONS(4308),
    [anon_sym_f64] = ACTIONS(4308),
    [anon_sym_bool] = ACTIONS(4308),
    [anon_sym_str] = ACTIONS(4308),
    [anon_sym_char] = ACTIONS(4308),
    [anon_sym_DASH] = ACTIONS(4306),
    [anon_sym_AMP] = ACTIONS(4306),
    [anon_sym_PIPE] = ACTIONS(4306),
    [anon_sym_LT] = ACTIONS(4306),
    [anon_sym_DOT_DOT] = ACTIONS(4306),
    [anon_sym_COLON_COLON] = ACTIONS(4306),
    [anon_sym_POUND] = ACTIONS(4306),
    [anon_sym_SQUOTE] = ACTIONS(4308),
    [anon_sym_async] = ACTIONS(4308),
    [anon_sym_auto] = ACTIONS(4308),
    [anon_sym_await] = ACTIONS(4308),
    [anon_sym_become] = ACTIONS(4308),
    [anon_sym_break] = ACTIONS(4308),
    [anon_sym_const] = ACTIONS(4308),
    [anon_sym_continue] = ACTIONS(4308),
    [anon_sym_default] = ACTIONS(4308),
    [anon_sym_enum] = ACTIONS(4308),
    [anon_sym_fn] = ACTIONS(4308),
    [anon_sym_for] = ACTIONS(4308),
    [anon_sym_gen] = ACTIONS(4308),
    [anon_sym_if] = ACTIONS(4308),
    [anon_sym_impl] = ACTIONS(4308),
    [anon_sym_let] = ACTIONS(4308),
    [anon_sym_loop] = ACTIONS(4308),
    [anon_sym_match] = ACTIONS(4308),
    [anon_sym_mod] = ACTIONS(4308),
    [anon_sym_pub] = ACTIONS(4308),
    [anon_sym_return] = ACTIONS(4308),
    [anon_sym_raw] = ACTIONS(4308),
    [anon_sym_static] = ACTIONS(4308),
    [anon_sym_struct] = ACTIONS(4308),
    [anon_sym_trait] = ACTIONS(4308),
    [anon_sym_type] = ACTIONS(4308),
    [anon_sym_union] = ACTIONS(4308),
    [anon_sym_unsafe] = ACTIONS(4308),
    [anon_sym_use] = ACTIONS(4308),
    [anon_sym_while] = ACTIONS(4308),
    [anon_sym_extern] = ACTIONS(4308),
    [anon_sym_safe] = ACTIONS(4308),
    [anon_sym_yield] = ACTIONS(4308),
    [anon_sym_move] = ACTIONS(4308),
    [anon_sym_try] = ACTIONS(4308),
    [sym_integer_literal] = ACTIONS(4306),
    [aux_sym_string_literal_token1] = ACTIONS(4306),
    [sym_char_literal] = ACTIONS(4306),
    [anon_sym_true] = ACTIONS(4308),
    [anon_sym_false] = ACTIONS(4308),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4308),
    [sym_super] = ACTIONS(4308),
    [sym_crate] = ACTIONS(4308),
    [aux_sym_metavariable_token1] = ACTIONS(4306),
    [sym__raw_string_literal_start] = ACTIONS(4306),
    [sym_float_literal] = ACTIONS(4306),
    [sym__outer_block_doc_comment_start] = ACTIONS(4306),
    [sym__outer_line_doc_comment_start] = ACTIONS(4306),
  },
  [STATE(1137)] = {
    [sym_line_comment] = STATE(1137),
    [sym_block_comment] = STATE(1137),
    [ts_builtin_sym_end] = ACTIONS(4310),
    [sym_identifier] = ACTIONS(4312),
    [anon_sym_SEMI] = ACTIONS(4310),
    [anon_sym_macro] = ACTIONS(4312),
    [anon_sym_LPAREN] = ACTIONS(4310),
    [anon_sym_LBRACE] = ACTIONS(4310),
    [anon_sym_RBRACE] = ACTIONS(4310),
    [anon_sym_macro_rules] = ACTIONS(4312),
    [anon_sym_BANG] = ACTIONS(4310),
    [anon_sym_LBRACK] = ACTIONS(4310),
    [anon_sym_STAR] = ACTIONS(4310),
    [anon_sym_u8] = ACTIONS(4312),
    [anon_sym_i8] = ACTIONS(4312),
    [anon_sym_u16] = ACTIONS(4312),
    [anon_sym_i16] = ACTIONS(4312),
    [anon_sym_u32] = ACTIONS(4312),
    [anon_sym_i32] = ACTIONS(4312),
    [anon_sym_u64] = ACTIONS(4312),
    [anon_sym_i64] = ACTIONS(4312),
    [anon_sym_u128] = ACTIONS(4312),
    [anon_sym_i128] = ACTIONS(4312),
    [anon_sym_isize] = ACTIONS(4312),
    [anon_sym_usize] = ACTIONS(4312),
    [anon_sym_f32] = ACTIONS(4312),
    [anon_sym_f64] = ACTIONS(4312),
    [anon_sym_bool] = ACTIONS(4312),
    [anon_sym_str] = ACTIONS(4312),
    [anon_sym_char] = ACTIONS(4312),
    [anon_sym_DASH] = ACTIONS(4310),
    [anon_sym_AMP] = ACTIONS(4310),
    [anon_sym_PIPE] = ACTIONS(4310),
    [anon_sym_LT] = ACTIONS(4310),
    [anon_sym_DOT_DOT] = ACTIONS(4310),
    [anon_sym_COLON_COLON] = ACTIONS(4310),
    [anon_sym_POUND] = ACTIONS(4310),
    [anon_sym_SQUOTE] = ACTIONS(4312),
    [anon_sym_async] = ACTIONS(4312),
    [anon_sym_auto] = ACTIONS(4312),
    [anon_sym_await] = ACTIONS(4312),
    [anon_sym_become] = ACTIONS(4312),
    [anon_sym_break] = ACTIONS(4312),
    [anon_sym_const] = ACTIONS(4312),
    [anon_sym_continue] = ACTIONS(4312),
    [anon_sym_default] = ACTIONS(4312),
    [anon_sym_enum] = ACTIONS(4312),
    [anon_sym_fn] = ACTIONS(4312),
    [anon_sym_for] = ACTIONS(4312),
    [anon_sym_gen] = ACTIONS(4312),
    [anon_sym_if] = ACTIONS(4312),
    [anon_sym_impl] = ACTIONS(4312),
    [anon_sym_let] = ACTIONS(4312),
    [anon_sym_loop] = ACTIONS(4312),
    [anon_sym_match] = ACTIONS(4312),
    [anon_sym_mod] = ACTIONS(4312),
    [anon_sym_pub] = ACTIONS(4312),
    [anon_sym_return] = ACTIONS(4312),
    [anon_sym_raw] = ACTIONS(4312),
    [anon_sym_static] = ACTIONS(4312),
    [anon_sym_struct] = ACTIONS(4312),
    [anon_sym_trait] = ACTIONS(4312),
    [anon_sym_type] = ACTIONS(4312),
    [anon_sym_union] = ACTIONS(4312),
    [anon_sym_unsafe] = ACTIONS(4312),
    [anon_sym_use] = ACTIONS(4312),
    [anon_sym_while] = ACTIONS(4312),
    [anon_sym_extern] = ACTIONS(4312),
    [anon_sym_safe] = ACTIONS(4312),
    [anon_sym_yield] = ACTIONS(4312),
    [anon_sym_move] = ACTIONS(4312),
    [anon_sym_try] = ACTIONS(4312),
    [sym_integer_literal] = ACTIONS(4310),
    [aux_sym_string_literal_token1] = ACTIONS(4310),
    [sym_char_literal] = ACTIONS(4310),
    [anon_sym_true] = ACTIONS(4312),
    [anon_sym_false] = ACTIONS(4312),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4312),
    [sym_super] = ACTIONS(4312),
    [sym_crate] = ACTIONS(4312),
    [aux_sym_metavariable_token1] = ACTIONS(4310),
    [sym__raw_string_literal_start] = ACTIONS(4310),
    [sym_float_literal] = ACTIONS(4310),
    [sym__outer_block_doc_comment_start] = ACTIONS(4310),
    [sym__outer_line_doc_comment_start] = ACTIONS(4310),
  },
  [STATE(1138)] = {
    [sym_line_comment] = STATE(1138),
    [sym_block_comment] = STATE(1138),
    [ts_builtin_sym_end] = ACTIONS(4314),
    [sym_identifier] = ACTIONS(4316),
    [anon_sym_SEMI] = ACTIONS(4314),
    [anon_sym_macro] = ACTIONS(4316),
    [anon_sym_LPAREN] = ACTIONS(4314),
    [anon_sym_LBRACE] = ACTIONS(4314),
    [anon_sym_RBRACE] = ACTIONS(4314),
    [anon_sym_macro_rules] = ACTIONS(4316),
    [anon_sym_BANG] = ACTIONS(4314),
    [anon_sym_LBRACK] = ACTIONS(4314),
    [anon_sym_STAR] = ACTIONS(4314),
    [anon_sym_u8] = ACTIONS(4316),
    [anon_sym_i8] = ACTIONS(4316),
    [anon_sym_u16] = ACTIONS(4316),
    [anon_sym_i16] = ACTIONS(4316),
    [anon_sym_u32] = ACTIONS(4316),
    [anon_sym_i32] = ACTIONS(4316),
    [anon_sym_u64] = ACTIONS(4316),
    [anon_sym_i64] = ACTIONS(4316),
    [anon_sym_u128] = ACTIONS(4316),
    [anon_sym_i128] = ACTIONS(4316),
    [anon_sym_isize] = ACTIONS(4316),
    [anon_sym_usize] = ACTIONS(4316),
    [anon_sym_f32] = ACTIONS(4316),
    [anon_sym_f64] = ACTIONS(4316),
    [anon_sym_bool] = ACTIONS(4316),
    [anon_sym_str] = ACTIONS(4316),
    [anon_sym_char] = ACTIONS(4316),
    [anon_sym_DASH] = ACTIONS(4314),
    [anon_sym_AMP] = ACTIONS(4314),
    [anon_sym_PIPE] = ACTIONS(4314),
    [anon_sym_LT] = ACTIONS(4314),
    [anon_sym_DOT_DOT] = ACTIONS(4314),
    [anon_sym_COLON_COLON] = ACTIONS(4314),
    [anon_sym_POUND] = ACTIONS(4314),
    [anon_sym_SQUOTE] = ACTIONS(4316),
    [anon_sym_async] = ACTIONS(4316),
    [anon_sym_auto] = ACTIONS(4316),
    [anon_sym_await] = ACTIONS(4316),
    [anon_sym_become] = ACTIONS(4316),
    [anon_sym_break] = ACTIONS(4316),
    [anon_sym_const] = ACTIONS(4316),
    [anon_sym_continue] = ACTIONS(4316),
    [anon_sym_default] = ACTIONS(4316),
    [anon_sym_enum] = ACTIONS(4316),
    [anon_sym_fn] = ACTIONS(4316),
    [anon_sym_for] = ACTIONS(4316),
    [anon_sym_gen] = ACTIONS(4316),
    [anon_sym_if] = ACTIONS(4316),
    [anon_sym_impl] = ACTIONS(4316),
    [anon_sym_let] = ACTIONS(4316),
    [anon_sym_loop] = ACTIONS(4316),
    [anon_sym_match] = ACTIONS(4316),
    [anon_sym_mod] = ACTIONS(4316),
    [anon_sym_pub] = ACTIONS(4316),
    [anon_sym_return] = ACTIONS(4316),
    [anon_sym_raw] = ACTIONS(4316),
    [anon_sym_static] = ACTIONS(4316),
    [anon_sym_struct] = ACTIONS(4316),
    [anon_sym_trait] = ACTIONS(4316),
    [anon_sym_type] = ACTIONS(4316),
    [anon_sym_union] = ACTIONS(4316),
    [anon_sym_unsafe] = ACTIONS(4316),
    [anon_sym_use] = ACTIONS(4316),
    [anon_sym_while] = ACTIONS(4316),
    [anon_sym_extern] = ACTIONS(4316),
    [anon_sym_safe] = ACTIONS(4316),
    [anon_sym_yield] = ACTIONS(4316),
    [anon_sym_move] = ACTIONS(4316),
    [anon_sym_try] = ACTIONS(4316),
    [sym_integer_literal] = ACTIONS(4314),
    [aux_sym_string_literal_token1] = ACTIONS(4314),
    [sym_char_literal] = ACTIONS(4314),
    [anon_sym_true] = ACTIONS(4316),
    [anon_sym_false] = ACTIONS(4316),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4316),
    [sym_super] = ACTIONS(4316),
    [sym_crate] = ACTIONS(4316),
    [aux_sym_metavariable_token1] = ACTIONS(4314),
    [sym__raw_string_literal_start] = ACTIONS(4314),
    [sym_float_literal] = ACTIONS(4314),
    [sym__outer_block_doc_comment_start] = ACTIONS(4314),
    [sym__outer_line_doc_comment_start] = ACTIONS(4314),
  },
  [STATE(1139)] = {
    [sym_line_comment] = STATE(1139),
    [sym_block_comment] = STATE(1139),
    [ts_builtin_sym_end] = ACTIONS(4318),
    [sym_identifier] = ACTIONS(4320),
    [anon_sym_SEMI] = ACTIONS(4318),
    [anon_sym_macro] = ACTIONS(4320),
    [anon_sym_LPAREN] = ACTIONS(4318),
    [anon_sym_LBRACE] = ACTIONS(4318),
    [anon_sym_RBRACE] = ACTIONS(4318),
    [anon_sym_macro_rules] = ACTIONS(4320),
    [anon_sym_BANG] = ACTIONS(4318),
    [anon_sym_LBRACK] = ACTIONS(4318),
    [anon_sym_STAR] = ACTIONS(4318),
    [anon_sym_u8] = ACTIONS(4320),
    [anon_sym_i8] = ACTIONS(4320),
    [anon_sym_u16] = ACTIONS(4320),
    [anon_sym_i16] = ACTIONS(4320),
    [anon_sym_u32] = ACTIONS(4320),
    [anon_sym_i32] = ACTIONS(4320),
    [anon_sym_u64] = ACTIONS(4320),
    [anon_sym_i64] = ACTIONS(4320),
    [anon_sym_u128] = ACTIONS(4320),
    [anon_sym_i128] = ACTIONS(4320),
    [anon_sym_isize] = ACTIONS(4320),
    [anon_sym_usize] = ACTIONS(4320),
    [anon_sym_f32] = ACTIONS(4320),
    [anon_sym_f64] = ACTIONS(4320),
    [anon_sym_bool] = ACTIONS(4320),
    [anon_sym_str] = ACTIONS(4320),
    [anon_sym_char] = ACTIONS(4320),
    [anon_sym_DASH] = ACTIONS(4318),
    [anon_sym_AMP] = ACTIONS(4318),
    [anon_sym_PIPE] = ACTIONS(4318),
    [anon_sym_LT] = ACTIONS(4318),
    [anon_sym_DOT_DOT] = ACTIONS(4318),
    [anon_sym_COLON_COLON] = ACTIONS(4318),
    [anon_sym_POUND] = ACTIONS(4318),
    [anon_sym_SQUOTE] = ACTIONS(4320),
    [anon_sym_async] = ACTIONS(4320),
    [anon_sym_auto] = ACTIONS(4320),
    [anon_sym_await] = ACTIONS(4320),
    [anon_sym_become] = ACTIONS(4320),
    [anon_sym_break] = ACTIONS(4320),
    [anon_sym_const] = ACTIONS(4320),
    [anon_sym_continue] = ACTIONS(4320),
    [anon_sym_default] = ACTIONS(4320),
    [anon_sym_enum] = ACTIONS(4320),
    [anon_sym_fn] = ACTIONS(4320),
    [anon_sym_for] = ACTIONS(4320),
    [anon_sym_gen] = ACTIONS(4320),
    [anon_sym_if] = ACTIONS(4320),
    [anon_sym_impl] = ACTIONS(4320),
    [anon_sym_let] = ACTIONS(4320),
    [anon_sym_loop] = ACTIONS(4320),
    [anon_sym_match] = ACTIONS(4320),
    [anon_sym_mod] = ACTIONS(4320),
    [anon_sym_pub] = ACTIONS(4320),
    [anon_sym_return] = ACTIONS(4320),
    [anon_sym_raw] = ACTIONS(4320),
    [anon_sym_static] = ACTIONS(4320),
    [anon_sym_struct] = ACTIONS(4320),
    [anon_sym_trait] = ACTIONS(4320),
    [anon_sym_type] = ACTIONS(4320),
    [anon_sym_union] = ACTIONS(4320),
    [anon_sym_unsafe] = ACTIONS(4320),
    [anon_sym_use] = ACTIONS(4320),
    [anon_sym_while] = ACTIONS(4320),
    [anon_sym_extern] = ACTIONS(4320),
    [anon_sym_safe] = ACTIONS(4320),
    [anon_sym_yield] = ACTIONS(4320),
    [anon_sym_move] = ACTIONS(4320),
    [anon_sym_try] = ACTIONS(4320),
    [sym_integer_literal] = ACTIONS(4318),
    [aux_sym_string_literal_token1] = ACTIONS(4318),
    [sym_char_literal] = ACTIONS(4318),
    [anon_sym_true] = ACTIONS(4320),
    [anon_sym_false] = ACTIONS(4320),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4320),
    [sym_super] = ACTIONS(4320),
    [sym_crate] = ACTIONS(4320),
    [aux_sym_metavariable_token1] = ACTIONS(4318),
    [sym__raw_string_literal_start] = ACTIONS(4318),
    [sym_float_literal] = ACTIONS(4318),
    [sym__outer_block_doc_comment_start] = ACTIONS(4318),
    [sym__outer_line_doc_comment_start] = ACTIONS(4318),
  },
  [STATE(1140)] = {
    [sym_line_comment] = STATE(1140),
    [sym_block_comment] = STATE(1140),
    [ts_builtin_sym_end] = ACTIONS(4322),
    [sym_identifier] = ACTIONS(4324),
    [anon_sym_SEMI] = ACTIONS(4322),
    [anon_sym_macro] = ACTIONS(4324),
    [anon_sym_LPAREN] = ACTIONS(4322),
    [anon_sym_LBRACE] = ACTIONS(4322),
    [anon_sym_RBRACE] = ACTIONS(4322),
    [anon_sym_macro_rules] = ACTIONS(4324),
    [anon_sym_BANG] = ACTIONS(4322),
    [anon_sym_LBRACK] = ACTIONS(4322),
    [anon_sym_STAR] = ACTIONS(4322),
    [anon_sym_u8] = ACTIONS(4324),
    [anon_sym_i8] = ACTIONS(4324),
    [anon_sym_u16] = ACTIONS(4324),
    [anon_sym_i16] = ACTIONS(4324),
    [anon_sym_u32] = ACTIONS(4324),
    [anon_sym_i32] = ACTIONS(4324),
    [anon_sym_u64] = ACTIONS(4324),
    [anon_sym_i64] = ACTIONS(4324),
    [anon_sym_u128] = ACTIONS(4324),
    [anon_sym_i128] = ACTIONS(4324),
    [anon_sym_isize] = ACTIONS(4324),
    [anon_sym_usize] = ACTIONS(4324),
    [anon_sym_f32] = ACTIONS(4324),
    [anon_sym_f64] = ACTIONS(4324),
    [anon_sym_bool] = ACTIONS(4324),
    [anon_sym_str] = ACTIONS(4324),
    [anon_sym_char] = ACTIONS(4324),
    [anon_sym_DASH] = ACTIONS(4322),
    [anon_sym_AMP] = ACTIONS(4322),
    [anon_sym_PIPE] = ACTIONS(4322),
    [anon_sym_LT] = ACTIONS(4322),
    [anon_sym_DOT_DOT] = ACTIONS(4322),
    [anon_sym_COLON_COLON] = ACTIONS(4322),
    [anon_sym_POUND] = ACTIONS(4322),
    [anon_sym_SQUOTE] = ACTIONS(4324),
    [anon_sym_async] = ACTIONS(4324),
    [anon_sym_auto] = ACTIONS(4324),
    [anon_sym_await] = ACTIONS(4324),
    [anon_sym_become] = ACTIONS(4324),
    [anon_sym_break] = ACTIONS(4324),
    [anon_sym_const] = ACTIONS(4324),
    [anon_sym_continue] = ACTIONS(4324),
    [anon_sym_default] = ACTIONS(4324),
    [anon_sym_enum] = ACTIONS(4324),
    [anon_sym_fn] = ACTIONS(4324),
    [anon_sym_for] = ACTIONS(4324),
    [anon_sym_gen] = ACTIONS(4324),
    [anon_sym_if] = ACTIONS(4324),
    [anon_sym_impl] = ACTIONS(4324),
    [anon_sym_let] = ACTIONS(4324),
    [anon_sym_loop] = ACTIONS(4324),
    [anon_sym_match] = ACTIONS(4324),
    [anon_sym_mod] = ACTIONS(4324),
    [anon_sym_pub] = ACTIONS(4324),
    [anon_sym_return] = ACTIONS(4324),
    [anon_sym_raw] = ACTIONS(4324),
    [anon_sym_static] = ACTIONS(4324),
    [anon_sym_struct] = ACTIONS(4324),
    [anon_sym_trait] = ACTIONS(4324),
    [anon_sym_type] = ACTIONS(4324),
    [anon_sym_union] = ACTIONS(4324),
    [anon_sym_unsafe] = ACTIONS(4324),
    [anon_sym_use] = ACTIONS(4324),
    [anon_sym_while] = ACTIONS(4324),
    [anon_sym_extern] = ACTIONS(4324),
    [anon_sym_safe] = ACTIONS(4324),
    [anon_sym_yield] = ACTIONS(4324),
    [anon_sym_move] = ACTIONS(4324),
    [anon_sym_try] = ACTIONS(4324),
    [sym_integer_literal] = ACTIONS(4322),
    [aux_sym_string_literal_token1] = ACTIONS(4322),
    [sym_char_literal] = ACTIONS(4322),
    [anon_sym_true] = ACTIONS(4324),
    [anon_sym_false] = ACTIONS(4324),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4324),
    [sym_super] = ACTIONS(4324),
    [sym_crate] = ACTIONS(4324),
    [aux_sym_metavariable_token1] = ACTIONS(4322),
    [sym__raw_string_literal_start] = ACTIONS(4322),
    [sym_float_literal] = ACTIONS(4322),
    [sym__outer_block_doc_comment_start] = ACTIONS(4322),
    [sym__outer_line_doc_comment_start] = ACTIONS(4322),
  },
  [STATE(1141)] = {
    [sym_line_comment] = STATE(1141),
    [sym_block_comment] = STATE(1141),
    [ts_builtin_sym_end] = ACTIONS(4326),
    [sym_identifier] = ACTIONS(4328),
    [anon_sym_SEMI] = ACTIONS(4326),
    [anon_sym_macro] = ACTIONS(4328),
    [anon_sym_LPAREN] = ACTIONS(4326),
    [anon_sym_LBRACE] = ACTIONS(4326),
    [anon_sym_RBRACE] = ACTIONS(4326),
    [anon_sym_macro_rules] = ACTIONS(4328),
    [anon_sym_BANG] = ACTIONS(4326),
    [anon_sym_LBRACK] = ACTIONS(4326),
    [anon_sym_STAR] = ACTIONS(4326),
    [anon_sym_u8] = ACTIONS(4328),
    [anon_sym_i8] = ACTIONS(4328),
    [anon_sym_u16] = ACTIONS(4328),
    [anon_sym_i16] = ACTIONS(4328),
    [anon_sym_u32] = ACTIONS(4328),
    [anon_sym_i32] = ACTIONS(4328),
    [anon_sym_u64] = ACTIONS(4328),
    [anon_sym_i64] = ACTIONS(4328),
    [anon_sym_u128] = ACTIONS(4328),
    [anon_sym_i128] = ACTIONS(4328),
    [anon_sym_isize] = ACTIONS(4328),
    [anon_sym_usize] = ACTIONS(4328),
    [anon_sym_f32] = ACTIONS(4328),
    [anon_sym_f64] = ACTIONS(4328),
    [anon_sym_bool] = ACTIONS(4328),
    [anon_sym_str] = ACTIONS(4328),
    [anon_sym_char] = ACTIONS(4328),
    [anon_sym_DASH] = ACTIONS(4326),
    [anon_sym_AMP] = ACTIONS(4326),
    [anon_sym_PIPE] = ACTIONS(4326),
    [anon_sym_LT] = ACTIONS(4326),
    [anon_sym_DOT_DOT] = ACTIONS(4326),
    [anon_sym_COLON_COLON] = ACTIONS(4326),
    [anon_sym_POUND] = ACTIONS(4326),
    [anon_sym_SQUOTE] = ACTIONS(4328),
    [anon_sym_async] = ACTIONS(4328),
    [anon_sym_auto] = ACTIONS(4328),
    [anon_sym_await] = ACTIONS(4328),
    [anon_sym_become] = ACTIONS(4328),
    [anon_sym_break] = ACTIONS(4328),
    [anon_sym_const] = ACTIONS(4328),
    [anon_sym_continue] = ACTIONS(4328),
    [anon_sym_default] = ACTIONS(4328),
    [anon_sym_enum] = ACTIONS(4328),
    [anon_sym_fn] = ACTIONS(4328),
    [anon_sym_for] = ACTIONS(4328),
    [anon_sym_gen] = ACTIONS(4328),
    [anon_sym_if] = ACTIONS(4328),
    [anon_sym_impl] = ACTIONS(4328),
    [anon_sym_let] = ACTIONS(4328),
    [anon_sym_loop] = ACTIONS(4328),
    [anon_sym_match] = ACTIONS(4328),
    [anon_sym_mod] = ACTIONS(4328),
    [anon_sym_pub] = ACTIONS(4328),
    [anon_sym_return] = ACTIONS(4328),
    [anon_sym_raw] = ACTIONS(4328),
    [anon_sym_static] = ACTIONS(4328),
    [anon_sym_struct] = ACTIONS(4328),
    [anon_sym_trait] = ACTIONS(4328),
    [anon_sym_type] = ACTIONS(4328),
    [anon_sym_union] = ACTIONS(4328),
    [anon_sym_unsafe] = ACTIONS(4328),
    [anon_sym_use] = ACTIONS(4328),
    [anon_sym_while] = ACTIONS(4328),
    [anon_sym_extern] = ACTIONS(4328),
    [anon_sym_safe] = ACTIONS(4328),
    [anon_sym_yield] = ACTIONS(4328),
    [anon_sym_move] = ACTIONS(4328),
    [anon_sym_try] = ACTIONS(4328),
    [sym_integer_literal] = ACTIONS(4326),
    [aux_sym_string_literal_token1] = ACTIONS(4326),
    [sym_char_literal] = ACTIONS(4326),
    [anon_sym_true] = ACTIONS(4328),
    [anon_sym_false] = ACTIONS(4328),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4328),
    [sym_super] = ACTIONS(4328),
    [sym_crate] = ACTIONS(4328),
    [aux_sym_metavariable_token1] = ACTIONS(4326),
    [sym__raw_string_literal_start] = ACTIONS(4326),
    [sym_float_literal] = ACTIONS(4326),
    [sym__outer_block_doc_comment_start] = ACTIONS(4326),
    [sym__outer_line_doc_comment_start] = ACTIONS(4326),
  },
  [STATE(1142)] = {
    [sym_line_comment] = STATE(1142),
    [sym_block_comment] = STATE(1142),
    [ts_builtin_sym_end] = ACTIONS(4330),
    [sym_identifier] = ACTIONS(4332),
    [anon_sym_SEMI] = ACTIONS(4330),
    [anon_sym_macro] = ACTIONS(4332),
    [anon_sym_LPAREN] = ACTIONS(4330),
    [anon_sym_LBRACE] = ACTIONS(4330),
    [anon_sym_RBRACE] = ACTIONS(4330),
    [anon_sym_macro_rules] = ACTIONS(4332),
    [anon_sym_BANG] = ACTIONS(4330),
    [anon_sym_LBRACK] = ACTIONS(4330),
    [anon_sym_STAR] = ACTIONS(4330),
    [anon_sym_u8] = ACTIONS(4332),
    [anon_sym_i8] = ACTIONS(4332),
    [anon_sym_u16] = ACTIONS(4332),
    [anon_sym_i16] = ACTIONS(4332),
    [anon_sym_u32] = ACTIONS(4332),
    [anon_sym_i32] = ACTIONS(4332),
    [anon_sym_u64] = ACTIONS(4332),
    [anon_sym_i64] = ACTIONS(4332),
    [anon_sym_u128] = ACTIONS(4332),
    [anon_sym_i128] = ACTIONS(4332),
    [anon_sym_isize] = ACTIONS(4332),
    [anon_sym_usize] = ACTIONS(4332),
    [anon_sym_f32] = ACTIONS(4332),
    [anon_sym_f64] = ACTIONS(4332),
    [anon_sym_bool] = ACTIONS(4332),
    [anon_sym_str] = ACTIONS(4332),
    [anon_sym_char] = ACTIONS(4332),
    [anon_sym_DASH] = ACTIONS(4330),
    [anon_sym_AMP] = ACTIONS(4330),
    [anon_sym_PIPE] = ACTIONS(4330),
    [anon_sym_LT] = ACTIONS(4330),
    [anon_sym_DOT_DOT] = ACTIONS(4330),
    [anon_sym_COLON_COLON] = ACTIONS(4330),
    [anon_sym_POUND] = ACTIONS(4330),
    [anon_sym_SQUOTE] = ACTIONS(4332),
    [anon_sym_async] = ACTIONS(4332),
    [anon_sym_auto] = ACTIONS(4332),
    [anon_sym_await] = ACTIONS(4332),
    [anon_sym_become] = ACTIONS(4332),
    [anon_sym_break] = ACTIONS(4332),
    [anon_sym_const] = ACTIONS(4332),
    [anon_sym_continue] = ACTIONS(4332),
    [anon_sym_default] = ACTIONS(4332),
    [anon_sym_enum] = ACTIONS(4332),
    [anon_sym_fn] = ACTIONS(4332),
    [anon_sym_for] = ACTIONS(4332),
    [anon_sym_gen] = ACTIONS(4332),
    [anon_sym_if] = ACTIONS(4332),
    [anon_sym_impl] = ACTIONS(4332),
    [anon_sym_let] = ACTIONS(4332),
    [anon_sym_loop] = ACTIONS(4332),
    [anon_sym_match] = ACTIONS(4332),
    [anon_sym_mod] = ACTIONS(4332),
    [anon_sym_pub] = ACTIONS(4332),
    [anon_sym_return] = ACTIONS(4332),
    [anon_sym_raw] = ACTIONS(4332),
    [anon_sym_static] = ACTIONS(4332),
    [anon_sym_struct] = ACTIONS(4332),
    [anon_sym_trait] = ACTIONS(4332),
    [anon_sym_type] = ACTIONS(4332),
    [anon_sym_union] = ACTIONS(4332),
    [anon_sym_unsafe] = ACTIONS(4332),
    [anon_sym_use] = ACTIONS(4332),
    [anon_sym_while] = ACTIONS(4332),
    [anon_sym_extern] = ACTIONS(4332),
    [anon_sym_safe] = ACTIONS(4332),
    [anon_sym_yield] = ACTIONS(4332),
    [anon_sym_move] = ACTIONS(4332),
    [anon_sym_try] = ACTIONS(4332),
    [sym_integer_literal] = ACTIONS(4330),
    [aux_sym_string_literal_token1] = ACTIONS(4330),
    [sym_char_literal] = ACTIONS(4330),
    [anon_sym_true] = ACTIONS(4332),
    [anon_sym_false] = ACTIONS(4332),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4332),
    [sym_super] = ACTIONS(4332),
    [sym_crate] = ACTIONS(4332),
    [aux_sym_metavariable_token1] = ACTIONS(4330),
    [sym__raw_string_literal_start] = ACTIONS(4330),
    [sym_float_literal] = ACTIONS(4330),
    [sym__outer_block_doc_comment_start] = ACTIONS(4330),
    [sym__outer_line_doc_comment_start] = ACTIONS(4330),
  },
  [STATE(1143)] = {
    [sym_line_comment] = STATE(1143),
    [sym_block_comment] = STATE(1143),
    [ts_builtin_sym_end] = ACTIONS(4334),
    [sym_identifier] = ACTIONS(4336),
    [anon_sym_SEMI] = ACTIONS(4334),
    [anon_sym_macro] = ACTIONS(4336),
    [anon_sym_LPAREN] = ACTIONS(4334),
    [anon_sym_LBRACE] = ACTIONS(4334),
    [anon_sym_RBRACE] = ACTIONS(4334),
    [anon_sym_macro_rules] = ACTIONS(4336),
    [anon_sym_BANG] = ACTIONS(4334),
    [anon_sym_LBRACK] = ACTIONS(4334),
    [anon_sym_STAR] = ACTIONS(4334),
    [anon_sym_u8] = ACTIONS(4336),
    [anon_sym_i8] = ACTIONS(4336),
    [anon_sym_u16] = ACTIONS(4336),
    [anon_sym_i16] = ACTIONS(4336),
    [anon_sym_u32] = ACTIONS(4336),
    [anon_sym_i32] = ACTIONS(4336),
    [anon_sym_u64] = ACTIONS(4336),
    [anon_sym_i64] = ACTIONS(4336),
    [anon_sym_u128] = ACTIONS(4336),
    [anon_sym_i128] = ACTIONS(4336),
    [anon_sym_isize] = ACTIONS(4336),
    [anon_sym_usize] = ACTIONS(4336),
    [anon_sym_f32] = ACTIONS(4336),
    [anon_sym_f64] = ACTIONS(4336),
    [anon_sym_bool] = ACTIONS(4336),
    [anon_sym_str] = ACTIONS(4336),
    [anon_sym_char] = ACTIONS(4336),
    [anon_sym_DASH] = ACTIONS(4334),
    [anon_sym_AMP] = ACTIONS(4334),
    [anon_sym_PIPE] = ACTIONS(4334),
    [anon_sym_LT] = ACTIONS(4334),
    [anon_sym_DOT_DOT] = ACTIONS(4334),
    [anon_sym_COLON_COLON] = ACTIONS(4334),
    [anon_sym_POUND] = ACTIONS(4334),
    [anon_sym_SQUOTE] = ACTIONS(4336),
    [anon_sym_async] = ACTIONS(4336),
    [anon_sym_auto] = ACTIONS(4336),
    [anon_sym_await] = ACTIONS(4336),
    [anon_sym_become] = ACTIONS(4336),
    [anon_sym_break] = ACTIONS(4336),
    [anon_sym_const] = ACTIONS(4336),
    [anon_sym_continue] = ACTIONS(4336),
    [anon_sym_default] = ACTIONS(4336),
    [anon_sym_enum] = ACTIONS(4336),
    [anon_sym_fn] = ACTIONS(4336),
    [anon_sym_for] = ACTIONS(4336),
    [anon_sym_gen] = ACTIONS(4336),
    [anon_sym_if] = ACTIONS(4336),
    [anon_sym_impl] = ACTIONS(4336),
    [anon_sym_let] = ACTIONS(4336),
    [anon_sym_loop] = ACTIONS(4336),
    [anon_sym_match] = ACTIONS(4336),
    [anon_sym_mod] = ACTIONS(4336),
    [anon_sym_pub] = ACTIONS(4336),
    [anon_sym_return] = ACTIONS(4336),
    [anon_sym_raw] = ACTIONS(4336),
    [anon_sym_static] = ACTIONS(4336),
    [anon_sym_struct] = ACTIONS(4336),
    [anon_sym_trait] = ACTIONS(4336),
    [anon_sym_type] = ACTIONS(4336),
    [anon_sym_union] = ACTIONS(4336),
    [anon_sym_unsafe] = ACTIONS(4336),
    [anon_sym_use] = ACTIONS(4336),
    [anon_sym_while] = ACTIONS(4336),
    [anon_sym_extern] = ACTIONS(4336),
    [anon_sym_safe] = ACTIONS(4336),
    [anon_sym_yield] = ACTIONS(4336),
    [anon_sym_move] = ACTIONS(4336),
    [anon_sym_try] = ACTIONS(4336),
    [sym_integer_literal] = ACTIONS(4334),
    [aux_sym_string_literal_token1] = ACTIONS(4334),
    [sym_char_literal] = ACTIONS(4334),
    [anon_sym_true] = ACTIONS(4336),
    [anon_sym_false] = ACTIONS(4336),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4336),
    [sym_super] = ACTIONS(4336),
    [sym_crate] = ACTIONS(4336),
    [aux_sym_metavariable_token1] = ACTIONS(4334),
    [sym__raw_string_literal_start] = ACTIONS(4334),
    [sym_float_literal] = ACTIONS(4334),
    [sym__outer_block_doc_comment_start] = ACTIONS(4334),
    [sym__outer_line_doc_comment_start] = ACTIONS(4334),
  },
  [STATE(1144)] = {
    [sym_line_comment] = STATE(1144),
    [sym_block_comment] = STATE(1144),
    [ts_builtin_sym_end] = ACTIONS(4338),
    [sym_identifier] = ACTIONS(4340),
    [anon_sym_SEMI] = ACTIONS(4338),
    [anon_sym_macro] = ACTIONS(4340),
    [anon_sym_LPAREN] = ACTIONS(4338),
    [anon_sym_LBRACE] = ACTIONS(4338),
    [anon_sym_RBRACE] = ACTIONS(4338),
    [anon_sym_macro_rules] = ACTIONS(4340),
    [anon_sym_BANG] = ACTIONS(4338),
    [anon_sym_LBRACK] = ACTIONS(4338),
    [anon_sym_STAR] = ACTIONS(4338),
    [anon_sym_u8] = ACTIONS(4340),
    [anon_sym_i8] = ACTIONS(4340),
    [anon_sym_u16] = ACTIONS(4340),
    [anon_sym_i16] = ACTIONS(4340),
    [anon_sym_u32] = ACTIONS(4340),
    [anon_sym_i32] = ACTIONS(4340),
    [anon_sym_u64] = ACTIONS(4340),
    [anon_sym_i64] = ACTIONS(4340),
    [anon_sym_u128] = ACTIONS(4340),
    [anon_sym_i128] = ACTIONS(4340),
    [anon_sym_isize] = ACTIONS(4340),
    [anon_sym_usize] = ACTIONS(4340),
    [anon_sym_f32] = ACTIONS(4340),
    [anon_sym_f64] = ACTIONS(4340),
    [anon_sym_bool] = ACTIONS(4340),
    [anon_sym_str] = ACTIONS(4340),
    [anon_sym_char] = ACTIONS(4340),
    [anon_sym_DASH] = ACTIONS(4338),
    [anon_sym_AMP] = ACTIONS(4338),
    [anon_sym_PIPE] = ACTIONS(4338),
    [anon_sym_LT] = ACTIONS(4338),
    [anon_sym_DOT_DOT] = ACTIONS(4338),
    [anon_sym_COLON_COLON] = ACTIONS(4338),
    [anon_sym_POUND] = ACTIONS(4338),
    [anon_sym_SQUOTE] = ACTIONS(4340),
    [anon_sym_async] = ACTIONS(4340),
    [anon_sym_auto] = ACTIONS(4340),
    [anon_sym_await] = ACTIONS(4340),
    [anon_sym_become] = ACTIONS(4340),
    [anon_sym_break] = ACTIONS(4340),
    [anon_sym_const] = ACTIONS(4340),
    [anon_sym_continue] = ACTIONS(4340),
    [anon_sym_default] = ACTIONS(4340),
    [anon_sym_enum] = ACTIONS(4340),
    [anon_sym_fn] = ACTIONS(4340),
    [anon_sym_for] = ACTIONS(4340),
    [anon_sym_gen] = ACTIONS(4340),
    [anon_sym_if] = ACTIONS(4340),
    [anon_sym_impl] = ACTIONS(4340),
    [anon_sym_let] = ACTIONS(4340),
    [anon_sym_loop] = ACTIONS(4340),
    [anon_sym_match] = ACTIONS(4340),
    [anon_sym_mod] = ACTIONS(4340),
    [anon_sym_pub] = ACTIONS(4340),
    [anon_sym_return] = ACTIONS(4340),
    [anon_sym_raw] = ACTIONS(4340),
    [anon_sym_static] = ACTIONS(4340),
    [anon_sym_struct] = ACTIONS(4340),
    [anon_sym_trait] = ACTIONS(4340),
    [anon_sym_type] = ACTIONS(4340),
    [anon_sym_union] = ACTIONS(4340),
    [anon_sym_unsafe] = ACTIONS(4340),
    [anon_sym_use] = ACTIONS(4340),
    [anon_sym_while] = ACTIONS(4340),
    [anon_sym_extern] = ACTIONS(4340),
    [anon_sym_safe] = ACTIONS(4340),
    [anon_sym_yield] = ACTIONS(4340),
    [anon_sym_move] = ACTIONS(4340),
    [anon_sym_try] = ACTIONS(4340),
    [sym_integer_literal] = ACTIONS(4338),
    [aux_sym_string_literal_token1] = ACTIONS(4338),
    [sym_char_literal] = ACTIONS(4338),
    [anon_sym_true] = ACTIONS(4340),
    [anon_sym_false] = ACTIONS(4340),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4340),
    [sym_super] = ACTIONS(4340),
    [sym_crate] = ACTIONS(4340),
    [aux_sym_metavariable_token1] = ACTIONS(4338),
    [sym__raw_string_literal_start] = ACTIONS(4338),
    [sym_float_literal] = ACTIONS(4338),
    [sym__outer_block_doc_comment_start] = ACTIONS(4338),
    [sym__outer_line_doc_comment_start] = ACTIONS(4338),
  },
  [STATE(1145)] = {
    [sym_line_comment] = STATE(1145),
    [sym_block_comment] = STATE(1145),
    [ts_builtin_sym_end] = ACTIONS(4342),
    [sym_identifier] = ACTIONS(4344),
    [anon_sym_SEMI] = ACTIONS(4342),
    [anon_sym_macro] = ACTIONS(4344),
    [anon_sym_LPAREN] = ACTIONS(4342),
    [anon_sym_LBRACE] = ACTIONS(4342),
    [anon_sym_RBRACE] = ACTIONS(4342),
    [anon_sym_macro_rules] = ACTIONS(4344),
    [anon_sym_BANG] = ACTIONS(4342),
    [anon_sym_LBRACK] = ACTIONS(4342),
    [anon_sym_STAR] = ACTIONS(4342),
    [anon_sym_u8] = ACTIONS(4344),
    [anon_sym_i8] = ACTIONS(4344),
    [anon_sym_u16] = ACTIONS(4344),
    [anon_sym_i16] = ACTIONS(4344),
    [anon_sym_u32] = ACTIONS(4344),
    [anon_sym_i32] = ACTIONS(4344),
    [anon_sym_u64] = ACTIONS(4344),
    [anon_sym_i64] = ACTIONS(4344),
    [anon_sym_u128] = ACTIONS(4344),
    [anon_sym_i128] = ACTIONS(4344),
    [anon_sym_isize] = ACTIONS(4344),
    [anon_sym_usize] = ACTIONS(4344),
    [anon_sym_f32] = ACTIONS(4344),
    [anon_sym_f64] = ACTIONS(4344),
    [anon_sym_bool] = ACTIONS(4344),
    [anon_sym_str] = ACTIONS(4344),
    [anon_sym_char] = ACTIONS(4344),
    [anon_sym_DASH] = ACTIONS(4342),
    [anon_sym_AMP] = ACTIONS(4342),
    [anon_sym_PIPE] = ACTIONS(4342),
    [anon_sym_LT] = ACTIONS(4342),
    [anon_sym_DOT_DOT] = ACTIONS(4342),
    [anon_sym_COLON_COLON] = ACTIONS(4342),
    [anon_sym_POUND] = ACTIONS(4342),
    [anon_sym_SQUOTE] = ACTIONS(4344),
    [anon_sym_async] = ACTIONS(4344),
    [anon_sym_auto] = ACTIONS(4344),
    [anon_sym_await] = ACTIONS(4344),
    [anon_sym_become] = ACTIONS(4344),
    [anon_sym_break] = ACTIONS(4344),
    [anon_sym_const] = ACTIONS(4344),
    [anon_sym_continue] = ACTIONS(4344),
    [anon_sym_default] = ACTIONS(4344),
    [anon_sym_enum] = ACTIONS(4344),
    [anon_sym_fn] = ACTIONS(4344),
    [anon_sym_for] = ACTIONS(4344),
    [anon_sym_gen] = ACTIONS(4344),
    [anon_sym_if] = ACTIONS(4344),
    [anon_sym_impl] = ACTIONS(4344),
    [anon_sym_let] = ACTIONS(4344),
    [anon_sym_loop] = ACTIONS(4344),
    [anon_sym_match] = ACTIONS(4344),
    [anon_sym_mod] = ACTIONS(4344),
    [anon_sym_pub] = ACTIONS(4344),
    [anon_sym_return] = ACTIONS(4344),
    [anon_sym_raw] = ACTIONS(4344),
    [anon_sym_static] = ACTIONS(4344),
    [anon_sym_struct] = ACTIONS(4344),
    [anon_sym_trait] = ACTIONS(4344),
    [anon_sym_type] = ACTIONS(4344),
    [anon_sym_union] = ACTIONS(4344),
    [anon_sym_unsafe] = ACTIONS(4344),
    [anon_sym_use] = ACTIONS(4344),
    [anon_sym_while] = ACTIONS(4344),
    [anon_sym_extern] = ACTIONS(4344),
    [anon_sym_safe] = ACTIONS(4344),
    [anon_sym_yield] = ACTIONS(4344),
    [anon_sym_move] = ACTIONS(4344),
    [anon_sym_try] = ACTIONS(4344),
    [sym_integer_literal] = ACTIONS(4342),
    [aux_sym_string_literal_token1] = ACTIONS(4342),
    [sym_char_literal] = ACTIONS(4342),
    [anon_sym_true] = ACTIONS(4344),
    [anon_sym_false] = ACTIONS(4344),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4344),
    [sym_super] = ACTIONS(4344),
    [sym_crate] = ACTIONS(4344),
    [aux_sym_metavariable_token1] = ACTIONS(4342),
    [sym__raw_string_literal_start] = ACTIONS(4342),
    [sym_float_literal] = ACTIONS(4342),
    [sym__outer_block_doc_comment_start] = ACTIONS(4342),
    [sym__outer_line_doc_comment_start] = ACTIONS(4342),
  },
  [STATE(1146)] = {
    [sym_line_comment] = STATE(1146),
    [sym_block_comment] = STATE(1146),
    [ts_builtin_sym_end] = ACTIONS(1374),
    [sym_identifier] = ACTIONS(1376),
    [anon_sym_SEMI] = ACTIONS(1374),
    [anon_sym_macro] = ACTIONS(1376),
    [anon_sym_LPAREN] = ACTIONS(1374),
    [anon_sym_LBRACE] = ACTIONS(1374),
    [anon_sym_RBRACE] = ACTIONS(1374),
    [anon_sym_macro_rules] = ACTIONS(1376),
    [anon_sym_BANG] = ACTIONS(1374),
    [anon_sym_LBRACK] = ACTIONS(1374),
    [anon_sym_STAR] = ACTIONS(1374),
    [anon_sym_u8] = ACTIONS(1376),
    [anon_sym_i8] = ACTIONS(1376),
    [anon_sym_u16] = ACTIONS(1376),
    [anon_sym_i16] = ACTIONS(1376),
    [anon_sym_u32] = ACTIONS(1376),
    [anon_sym_i32] = ACTIONS(1376),
    [anon_sym_u64] = ACTIONS(1376),
    [anon_sym_i64] = ACTIONS(1376),
    [anon_sym_u128] = ACTIONS(1376),
    [anon_sym_i128] = ACTIONS(1376),
    [anon_sym_isize] = ACTIONS(1376),
    [anon_sym_usize] = ACTIONS(1376),
    [anon_sym_f32] = ACTIONS(1376),
    [anon_sym_f64] = ACTIONS(1376),
    [anon_sym_bool] = ACTIONS(1376),
    [anon_sym_str] = ACTIONS(1376),
    [anon_sym_char] = ACTIONS(1376),
    [anon_sym_DASH] = ACTIONS(1374),
    [anon_sym_AMP] = ACTIONS(1374),
    [anon_sym_PIPE] = ACTIONS(1374),
    [anon_sym_LT] = ACTIONS(1374),
    [anon_sym_DOT_DOT] = ACTIONS(1374),
    [anon_sym_COLON_COLON] = ACTIONS(1374),
    [anon_sym_POUND] = ACTIONS(1374),
    [anon_sym_SQUOTE] = ACTIONS(1376),
    [anon_sym_async] = ACTIONS(1376),
    [anon_sym_auto] = ACTIONS(1376),
    [anon_sym_await] = ACTIONS(1376),
    [anon_sym_become] = ACTIONS(1376),
    [anon_sym_break] = ACTIONS(1376),
    [anon_sym_const] = ACTIONS(1376),
    [anon_sym_continue] = ACTIONS(1376),
    [anon_sym_default] = ACTIONS(1376),
    [anon_sym_enum] = ACTIONS(1376),
    [anon_sym_fn] = ACTIONS(1376),
    [anon_sym_for] = ACTIONS(1376),
    [anon_sym_gen] = ACTIONS(1376),
    [anon_sym_if] = ACTIONS(1376),
    [anon_sym_impl] = ACTIONS(1376),
    [anon_sym_let] = ACTIONS(1376),
    [anon_sym_loop] = ACTIONS(1376),
    [anon_sym_match] = ACTIONS(1376),
    [anon_sym_mod] = ACTIONS(1376),
    [anon_sym_pub] = ACTIONS(1376),
    [anon_sym_return] = ACTIONS(1376),
    [anon_sym_raw] = ACTIONS(1376),
    [anon_sym_static] = ACTIONS(1376),
    [anon_sym_struct] = ACTIONS(1376),
    [anon_sym_trait] = ACTIONS(1376),
    [anon_sym_type] = ACTIONS(1376),
    [anon_sym_union] = ACTIONS(1376),
    [anon_sym_unsafe] = ACTIONS(1376),
    [anon_sym_use] = ACTIONS(1376),
    [anon_sym_while] = ACTIONS(1376),
    [anon_sym_extern] = ACTIONS(1376),
    [anon_sym_safe] = ACTIONS(1376),
    [anon_sym_yield] = ACTIONS(1376),
    [anon_sym_move] = ACTIONS(1376),
    [anon_sym_try] = ACTIONS(1376),
    [sym_integer_literal] = ACTIONS(1374),
    [aux_sym_string_literal_token1] = ACTIONS(1374),
    [sym_char_literal] = ACTIONS(1374),
    [anon_sym_true] = ACTIONS(1376),
    [anon_sym_false] = ACTIONS(1376),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1376),
    [sym_super] = ACTIONS(1376),
    [sym_crate] = ACTIONS(1376),
    [aux_sym_metavariable_token1] = ACTIONS(1374),
    [sym__raw_string_literal_start] = ACTIONS(1374),
    [sym_float_literal] = ACTIONS(1374),
    [sym__outer_block_doc_comment_start] = ACTIONS(1374),
    [sym__outer_line_doc_comment_start] = ACTIONS(1374),
  },
  [STATE(1147)] = {
    [sym_line_comment] = STATE(1147),
    [sym_block_comment] = STATE(1147),
    [ts_builtin_sym_end] = ACTIONS(4346),
    [sym_identifier] = ACTIONS(4348),
    [anon_sym_SEMI] = ACTIONS(4346),
    [anon_sym_macro] = ACTIONS(4348),
    [anon_sym_LPAREN] = ACTIONS(4346),
    [anon_sym_LBRACE] = ACTIONS(4346),
    [anon_sym_RBRACE] = ACTIONS(4346),
    [anon_sym_macro_rules] = ACTIONS(4348),
    [anon_sym_BANG] = ACTIONS(4346),
    [anon_sym_LBRACK] = ACTIONS(4346),
    [anon_sym_STAR] = ACTIONS(4346),
    [anon_sym_u8] = ACTIONS(4348),
    [anon_sym_i8] = ACTIONS(4348),
    [anon_sym_u16] = ACTIONS(4348),
    [anon_sym_i16] = ACTIONS(4348),
    [anon_sym_u32] = ACTIONS(4348),
    [anon_sym_i32] = ACTIONS(4348),
    [anon_sym_u64] = ACTIONS(4348),
    [anon_sym_i64] = ACTIONS(4348),
    [anon_sym_u128] = ACTIONS(4348),
    [anon_sym_i128] = ACTIONS(4348),
    [anon_sym_isize] = ACTIONS(4348),
    [anon_sym_usize] = ACTIONS(4348),
    [anon_sym_f32] = ACTIONS(4348),
    [anon_sym_f64] = ACTIONS(4348),
    [anon_sym_bool] = ACTIONS(4348),
    [anon_sym_str] = ACTIONS(4348),
    [anon_sym_char] = ACTIONS(4348),
    [anon_sym_DASH] = ACTIONS(4346),
    [anon_sym_AMP] = ACTIONS(4346),
    [anon_sym_PIPE] = ACTIONS(4346),
    [anon_sym_LT] = ACTIONS(4346),
    [anon_sym_DOT_DOT] = ACTIONS(4346),
    [anon_sym_COLON_COLON] = ACTIONS(4346),
    [anon_sym_POUND] = ACTIONS(4346),
    [anon_sym_SQUOTE] = ACTIONS(4348),
    [anon_sym_async] = ACTIONS(4348),
    [anon_sym_auto] = ACTIONS(4348),
    [anon_sym_await] = ACTIONS(4348),
    [anon_sym_become] = ACTIONS(4348),
    [anon_sym_break] = ACTIONS(4348),
    [anon_sym_const] = ACTIONS(4348),
    [anon_sym_continue] = ACTIONS(4348),
    [anon_sym_default] = ACTIONS(4348),
    [anon_sym_enum] = ACTIONS(4348),
    [anon_sym_fn] = ACTIONS(4348),
    [anon_sym_for] = ACTIONS(4348),
    [anon_sym_gen] = ACTIONS(4348),
    [anon_sym_if] = ACTIONS(4348),
    [anon_sym_impl] = ACTIONS(4348),
    [anon_sym_let] = ACTIONS(4348),
    [anon_sym_loop] = ACTIONS(4348),
    [anon_sym_match] = ACTIONS(4348),
    [anon_sym_mod] = ACTIONS(4348),
    [anon_sym_pub] = ACTIONS(4348),
    [anon_sym_return] = ACTIONS(4348),
    [anon_sym_raw] = ACTIONS(4348),
    [anon_sym_static] = ACTIONS(4348),
    [anon_sym_struct] = ACTIONS(4348),
    [anon_sym_trait] = ACTIONS(4348),
    [anon_sym_type] = ACTIONS(4348),
    [anon_sym_union] = ACTIONS(4348),
    [anon_sym_unsafe] = ACTIONS(4348),
    [anon_sym_use] = ACTIONS(4348),
    [anon_sym_while] = ACTIONS(4348),
    [anon_sym_extern] = ACTIONS(4348),
    [anon_sym_safe] = ACTIONS(4348),
    [anon_sym_yield] = ACTIONS(4348),
    [anon_sym_move] = ACTIONS(4348),
    [anon_sym_try] = ACTIONS(4348),
    [sym_integer_literal] = ACTIONS(4346),
    [aux_sym_string_literal_token1] = ACTIONS(4346),
    [sym_char_literal] = ACTIONS(4346),
    [anon_sym_true] = ACTIONS(4348),
    [anon_sym_false] = ACTIONS(4348),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4348),
    [sym_super] = ACTIONS(4348),
    [sym_crate] = ACTIONS(4348),
    [aux_sym_metavariable_token1] = ACTIONS(4346),
    [sym__raw_string_literal_start] = ACTIONS(4346),
    [sym_float_literal] = ACTIONS(4346),
    [sym__outer_block_doc_comment_start] = ACTIONS(4346),
    [sym__outer_line_doc_comment_start] = ACTIONS(4346),
  },
  [STATE(1148)] = {
    [sym_line_comment] = STATE(1148),
    [sym_block_comment] = STATE(1148),
    [ts_builtin_sym_end] = ACTIONS(4350),
    [sym_identifier] = ACTIONS(4352),
    [anon_sym_SEMI] = ACTIONS(4350),
    [anon_sym_macro] = ACTIONS(4352),
    [anon_sym_LPAREN] = ACTIONS(4350),
    [anon_sym_LBRACE] = ACTIONS(4350),
    [anon_sym_RBRACE] = ACTIONS(4350),
    [anon_sym_macro_rules] = ACTIONS(4352),
    [anon_sym_BANG] = ACTIONS(4350),
    [anon_sym_LBRACK] = ACTIONS(4350),
    [anon_sym_STAR] = ACTIONS(4350),
    [anon_sym_u8] = ACTIONS(4352),
    [anon_sym_i8] = ACTIONS(4352),
    [anon_sym_u16] = ACTIONS(4352),
    [anon_sym_i16] = ACTIONS(4352),
    [anon_sym_u32] = ACTIONS(4352),
    [anon_sym_i32] = ACTIONS(4352),
    [anon_sym_u64] = ACTIONS(4352),
    [anon_sym_i64] = ACTIONS(4352),
    [anon_sym_u128] = ACTIONS(4352),
    [anon_sym_i128] = ACTIONS(4352),
    [anon_sym_isize] = ACTIONS(4352),
    [anon_sym_usize] = ACTIONS(4352),
    [anon_sym_f32] = ACTIONS(4352),
    [anon_sym_f64] = ACTIONS(4352),
    [anon_sym_bool] = ACTIONS(4352),
    [anon_sym_str] = ACTIONS(4352),
    [anon_sym_char] = ACTIONS(4352),
    [anon_sym_DASH] = ACTIONS(4350),
    [anon_sym_AMP] = ACTIONS(4350),
    [anon_sym_PIPE] = ACTIONS(4350),
    [anon_sym_LT] = ACTIONS(4350),
    [anon_sym_DOT_DOT] = ACTIONS(4350),
    [anon_sym_COLON_COLON] = ACTIONS(4350),
    [anon_sym_POUND] = ACTIONS(4350),
    [anon_sym_SQUOTE] = ACTIONS(4352),
    [anon_sym_async] = ACTIONS(4352),
    [anon_sym_auto] = ACTIONS(4352),
    [anon_sym_await] = ACTIONS(4352),
    [anon_sym_become] = ACTIONS(4352),
    [anon_sym_break] = ACTIONS(4352),
    [anon_sym_const] = ACTIONS(4352),
    [anon_sym_continue] = ACTIONS(4352),
    [anon_sym_default] = ACTIONS(4352),
    [anon_sym_enum] = ACTIONS(4352),
    [anon_sym_fn] = ACTIONS(4352),
    [anon_sym_for] = ACTIONS(4352),
    [anon_sym_gen] = ACTIONS(4352),
    [anon_sym_if] = ACTIONS(4352),
    [anon_sym_impl] = ACTIONS(4352),
    [anon_sym_let] = ACTIONS(4352),
    [anon_sym_loop] = ACTIONS(4352),
    [anon_sym_match] = ACTIONS(4352),
    [anon_sym_mod] = ACTIONS(4352),
    [anon_sym_pub] = ACTIONS(4352),
    [anon_sym_return] = ACTIONS(4352),
    [anon_sym_raw] = ACTIONS(4352),
    [anon_sym_static] = ACTIONS(4352),
    [anon_sym_struct] = ACTIONS(4352),
    [anon_sym_trait] = ACTIONS(4352),
    [anon_sym_type] = ACTIONS(4352),
    [anon_sym_union] = ACTIONS(4352),
    [anon_sym_unsafe] = ACTIONS(4352),
    [anon_sym_use] = ACTIONS(4352),
    [anon_sym_while] = ACTIONS(4352),
    [anon_sym_extern] = ACTIONS(4352),
    [anon_sym_safe] = ACTIONS(4352),
    [anon_sym_yield] = ACTIONS(4352),
    [anon_sym_move] = ACTIONS(4352),
    [anon_sym_try] = ACTIONS(4352),
    [sym_integer_literal] = ACTIONS(4350),
    [aux_sym_string_literal_token1] = ACTIONS(4350),
    [sym_char_literal] = ACTIONS(4350),
    [anon_sym_true] = ACTIONS(4352),
    [anon_sym_false] = ACTIONS(4352),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4352),
    [sym_super] = ACTIONS(4352),
    [sym_crate] = ACTIONS(4352),
    [aux_sym_metavariable_token1] = ACTIONS(4350),
    [sym__raw_string_literal_start] = ACTIONS(4350),
    [sym_float_literal] = ACTIONS(4350),
    [sym__outer_block_doc_comment_start] = ACTIONS(4350),
    [sym__outer_line_doc_comment_start] = ACTIONS(4350),
  },
  [STATE(1149)] = {
    [sym_line_comment] = STATE(1149),
    [sym_block_comment] = STATE(1149),
    [ts_builtin_sym_end] = ACTIONS(4354),
    [sym_identifier] = ACTIONS(4356),
    [anon_sym_SEMI] = ACTIONS(4354),
    [anon_sym_macro] = ACTIONS(4356),
    [anon_sym_LPAREN] = ACTIONS(4354),
    [anon_sym_LBRACE] = ACTIONS(4354),
    [anon_sym_RBRACE] = ACTIONS(4354),
    [anon_sym_macro_rules] = ACTIONS(4356),
    [anon_sym_BANG] = ACTIONS(4354),
    [anon_sym_LBRACK] = ACTIONS(4354),
    [anon_sym_STAR] = ACTIONS(4354),
    [anon_sym_u8] = ACTIONS(4356),
    [anon_sym_i8] = ACTIONS(4356),
    [anon_sym_u16] = ACTIONS(4356),
    [anon_sym_i16] = ACTIONS(4356),
    [anon_sym_u32] = ACTIONS(4356),
    [anon_sym_i32] = ACTIONS(4356),
    [anon_sym_u64] = ACTIONS(4356),
    [anon_sym_i64] = ACTIONS(4356),
    [anon_sym_u128] = ACTIONS(4356),
    [anon_sym_i128] = ACTIONS(4356),
    [anon_sym_isize] = ACTIONS(4356),
    [anon_sym_usize] = ACTIONS(4356),
    [anon_sym_f32] = ACTIONS(4356),
    [anon_sym_f64] = ACTIONS(4356),
    [anon_sym_bool] = ACTIONS(4356),
    [anon_sym_str] = ACTIONS(4356),
    [anon_sym_char] = ACTIONS(4356),
    [anon_sym_DASH] = ACTIONS(4354),
    [anon_sym_AMP] = ACTIONS(4354),
    [anon_sym_PIPE] = ACTIONS(4354),
    [anon_sym_LT] = ACTIONS(4354),
    [anon_sym_DOT_DOT] = ACTIONS(4354),
    [anon_sym_COLON_COLON] = ACTIONS(4354),
    [anon_sym_POUND] = ACTIONS(4354),
    [anon_sym_SQUOTE] = ACTIONS(4356),
    [anon_sym_async] = ACTIONS(4356),
    [anon_sym_auto] = ACTIONS(4356),
    [anon_sym_await] = ACTIONS(4356),
    [anon_sym_become] = ACTIONS(4356),
    [anon_sym_break] = ACTIONS(4356),
    [anon_sym_const] = ACTIONS(4356),
    [anon_sym_continue] = ACTIONS(4356),
    [anon_sym_default] = ACTIONS(4356),
    [anon_sym_enum] = ACTIONS(4356),
    [anon_sym_fn] = ACTIONS(4356),
    [anon_sym_for] = ACTIONS(4356),
    [anon_sym_gen] = ACTIONS(4356),
    [anon_sym_if] = ACTIONS(4356),
    [anon_sym_impl] = ACTIONS(4356),
    [anon_sym_let] = ACTIONS(4356),
    [anon_sym_loop] = ACTIONS(4356),
    [anon_sym_match] = ACTIONS(4356),
    [anon_sym_mod] = ACTIONS(4356),
    [anon_sym_pub] = ACTIONS(4356),
    [anon_sym_return] = ACTIONS(4356),
    [anon_sym_raw] = ACTIONS(4356),
    [anon_sym_static] = ACTIONS(4356),
    [anon_sym_struct] = ACTIONS(4356),
    [anon_sym_trait] = ACTIONS(4356),
    [anon_sym_type] = ACTIONS(4356),
    [anon_sym_union] = ACTIONS(4356),
    [anon_sym_unsafe] = ACTIONS(4356),
    [anon_sym_use] = ACTIONS(4356),
    [anon_sym_while] = ACTIONS(4356),
    [anon_sym_extern] = ACTIONS(4356),
    [anon_sym_safe] = ACTIONS(4356),
    [anon_sym_yield] = ACTIONS(4356),
    [anon_sym_move] = ACTIONS(4356),
    [anon_sym_try] = ACTIONS(4356),
    [sym_integer_literal] = ACTIONS(4354),
    [aux_sym_string_literal_token1] = ACTIONS(4354),
    [sym_char_literal] = ACTIONS(4354),
    [anon_sym_true] = ACTIONS(4356),
    [anon_sym_false] = ACTIONS(4356),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4356),
    [sym_super] = ACTIONS(4356),
    [sym_crate] = ACTIONS(4356),
    [aux_sym_metavariable_token1] = ACTIONS(4354),
    [sym__raw_string_literal_start] = ACTIONS(4354),
    [sym_float_literal] = ACTIONS(4354),
    [sym__outer_block_doc_comment_start] = ACTIONS(4354),
    [sym__outer_line_doc_comment_start] = ACTIONS(4354),
  },
  [STATE(1150)] = {
    [sym_line_comment] = STATE(1150),
    [sym_block_comment] = STATE(1150),
    [ts_builtin_sym_end] = ACTIONS(4358),
    [sym_identifier] = ACTIONS(4360),
    [anon_sym_SEMI] = ACTIONS(4358),
    [anon_sym_macro] = ACTIONS(4360),
    [anon_sym_LPAREN] = ACTIONS(4358),
    [anon_sym_LBRACE] = ACTIONS(4358),
    [anon_sym_RBRACE] = ACTIONS(4358),
    [anon_sym_macro_rules] = ACTIONS(4360),
    [anon_sym_BANG] = ACTIONS(4358),
    [anon_sym_LBRACK] = ACTIONS(4358),
    [anon_sym_STAR] = ACTIONS(4358),
    [anon_sym_u8] = ACTIONS(4360),
    [anon_sym_i8] = ACTIONS(4360),
    [anon_sym_u16] = ACTIONS(4360),
    [anon_sym_i16] = ACTIONS(4360),
    [anon_sym_u32] = ACTIONS(4360),
    [anon_sym_i32] = ACTIONS(4360),
    [anon_sym_u64] = ACTIONS(4360),
    [anon_sym_i64] = ACTIONS(4360),
    [anon_sym_u128] = ACTIONS(4360),
    [anon_sym_i128] = ACTIONS(4360),
    [anon_sym_isize] = ACTIONS(4360),
    [anon_sym_usize] = ACTIONS(4360),
    [anon_sym_f32] = ACTIONS(4360),
    [anon_sym_f64] = ACTIONS(4360),
    [anon_sym_bool] = ACTIONS(4360),
    [anon_sym_str] = ACTIONS(4360),
    [anon_sym_char] = ACTIONS(4360),
    [anon_sym_DASH] = ACTIONS(4358),
    [anon_sym_AMP] = ACTIONS(4358),
    [anon_sym_PIPE] = ACTIONS(4358),
    [anon_sym_LT] = ACTIONS(4358),
    [anon_sym_DOT_DOT] = ACTIONS(4358),
    [anon_sym_COLON_COLON] = ACTIONS(4358),
    [anon_sym_POUND] = ACTIONS(4358),
    [anon_sym_SQUOTE] = ACTIONS(4360),
    [anon_sym_async] = ACTIONS(4360),
    [anon_sym_auto] = ACTIONS(4360),
    [anon_sym_await] = ACTIONS(4360),
    [anon_sym_become] = ACTIONS(4360),
    [anon_sym_break] = ACTIONS(4360),
    [anon_sym_const] = ACTIONS(4360),
    [anon_sym_continue] = ACTIONS(4360),
    [anon_sym_default] = ACTIONS(4360),
    [anon_sym_enum] = ACTIONS(4360),
    [anon_sym_fn] = ACTIONS(4360),
    [anon_sym_for] = ACTIONS(4360),
    [anon_sym_gen] = ACTIONS(4360),
    [anon_sym_if] = ACTIONS(4360),
    [anon_sym_impl] = ACTIONS(4360),
    [anon_sym_let] = ACTIONS(4360),
    [anon_sym_loop] = ACTIONS(4360),
    [anon_sym_match] = ACTIONS(4360),
    [anon_sym_mod] = ACTIONS(4360),
    [anon_sym_pub] = ACTIONS(4360),
    [anon_sym_return] = ACTIONS(4360),
    [anon_sym_raw] = ACTIONS(4360),
    [anon_sym_static] = ACTIONS(4360),
    [anon_sym_struct] = ACTIONS(4360),
    [anon_sym_trait] = ACTIONS(4360),
    [anon_sym_type] = ACTIONS(4360),
    [anon_sym_union] = ACTIONS(4360),
    [anon_sym_unsafe] = ACTIONS(4360),
    [anon_sym_use] = ACTIONS(4360),
    [anon_sym_while] = ACTIONS(4360),
    [anon_sym_extern] = ACTIONS(4360),
    [anon_sym_safe] = ACTIONS(4360),
    [anon_sym_yield] = ACTIONS(4360),
    [anon_sym_move] = ACTIONS(4360),
    [anon_sym_try] = ACTIONS(4360),
    [sym_integer_literal] = ACTIONS(4358),
    [aux_sym_string_literal_token1] = ACTIONS(4358),
    [sym_char_literal] = ACTIONS(4358),
    [anon_sym_true] = ACTIONS(4360),
    [anon_sym_false] = ACTIONS(4360),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4360),
    [sym_super] = ACTIONS(4360),
    [sym_crate] = ACTIONS(4360),
    [aux_sym_metavariable_token1] = ACTIONS(4358),
    [sym__raw_string_literal_start] = ACTIONS(4358),
    [sym_float_literal] = ACTIONS(4358),
    [sym__outer_block_doc_comment_start] = ACTIONS(4358),
    [sym__outer_line_doc_comment_start] = ACTIONS(4358),
  },
  [STATE(1151)] = {
    [sym_line_comment] = STATE(1151),
    [sym_block_comment] = STATE(1151),
    [ts_builtin_sym_end] = ACTIONS(4362),
    [sym_identifier] = ACTIONS(4364),
    [anon_sym_SEMI] = ACTIONS(4362),
    [anon_sym_macro] = ACTIONS(4364),
    [anon_sym_LPAREN] = ACTIONS(4362),
    [anon_sym_LBRACE] = ACTIONS(4362),
    [anon_sym_RBRACE] = ACTIONS(4362),
    [anon_sym_macro_rules] = ACTIONS(4364),
    [anon_sym_BANG] = ACTIONS(4362),
    [anon_sym_LBRACK] = ACTIONS(4362),
    [anon_sym_STAR] = ACTIONS(4362),
    [anon_sym_u8] = ACTIONS(4364),
    [anon_sym_i8] = ACTIONS(4364),
    [anon_sym_u16] = ACTIONS(4364),
    [anon_sym_i16] = ACTIONS(4364),
    [anon_sym_u32] = ACTIONS(4364),
    [anon_sym_i32] = ACTIONS(4364),
    [anon_sym_u64] = ACTIONS(4364),
    [anon_sym_i64] = ACTIONS(4364),
    [anon_sym_u128] = ACTIONS(4364),
    [anon_sym_i128] = ACTIONS(4364),
    [anon_sym_isize] = ACTIONS(4364),
    [anon_sym_usize] = ACTIONS(4364),
    [anon_sym_f32] = ACTIONS(4364),
    [anon_sym_f64] = ACTIONS(4364),
    [anon_sym_bool] = ACTIONS(4364),
    [anon_sym_str] = ACTIONS(4364),
    [anon_sym_char] = ACTIONS(4364),
    [anon_sym_DASH] = ACTIONS(4362),
    [anon_sym_AMP] = ACTIONS(4362),
    [anon_sym_PIPE] = ACTIONS(4362),
    [anon_sym_LT] = ACTIONS(4362),
    [anon_sym_DOT_DOT] = ACTIONS(4362),
    [anon_sym_COLON_COLON] = ACTIONS(4362),
    [anon_sym_POUND] = ACTIONS(4362),
    [anon_sym_SQUOTE] = ACTIONS(4364),
    [anon_sym_async] = ACTIONS(4364),
    [anon_sym_auto] = ACTIONS(4364),
    [anon_sym_await] = ACTIONS(4364),
    [anon_sym_become] = ACTIONS(4364),
    [anon_sym_break] = ACTIONS(4364),
    [anon_sym_const] = ACTIONS(4364),
    [anon_sym_continue] = ACTIONS(4364),
    [anon_sym_default] = ACTIONS(4364),
    [anon_sym_enum] = ACTIONS(4364),
    [anon_sym_fn] = ACTIONS(4364),
    [anon_sym_for] = ACTIONS(4364),
    [anon_sym_gen] = ACTIONS(4364),
    [anon_sym_if] = ACTIONS(4364),
    [anon_sym_impl] = ACTIONS(4364),
    [anon_sym_let] = ACTIONS(4364),
    [anon_sym_loop] = ACTIONS(4364),
    [anon_sym_match] = ACTIONS(4364),
    [anon_sym_mod] = ACTIONS(4364),
    [anon_sym_pub] = ACTIONS(4364),
    [anon_sym_return] = ACTIONS(4364),
    [anon_sym_raw] = ACTIONS(4364),
    [anon_sym_static] = ACTIONS(4364),
    [anon_sym_struct] = ACTIONS(4364),
    [anon_sym_trait] = ACTIONS(4364),
    [anon_sym_type] = ACTIONS(4364),
    [anon_sym_union] = ACTIONS(4364),
    [anon_sym_unsafe] = ACTIONS(4364),
    [anon_sym_use] = ACTIONS(4364),
    [anon_sym_while] = ACTIONS(4364),
    [anon_sym_extern] = ACTIONS(4364),
    [anon_sym_safe] = ACTIONS(4364),
    [anon_sym_yield] = ACTIONS(4364),
    [anon_sym_move] = ACTIONS(4364),
    [anon_sym_try] = ACTIONS(4364),
    [sym_integer_literal] = ACTIONS(4362),
    [aux_sym_string_literal_token1] = ACTIONS(4362),
    [sym_char_literal] = ACTIONS(4362),
    [anon_sym_true] = ACTIONS(4364),
    [anon_sym_false] = ACTIONS(4364),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4364),
    [sym_super] = ACTIONS(4364),
    [sym_crate] = ACTIONS(4364),
    [aux_sym_metavariable_token1] = ACTIONS(4362),
    [sym__raw_string_literal_start] = ACTIONS(4362),
    [sym_float_literal] = ACTIONS(4362),
    [sym__outer_block_doc_comment_start] = ACTIONS(4362),
    [sym__outer_line_doc_comment_start] = ACTIONS(4362),
  },
  [STATE(1152)] = {
    [sym_line_comment] = STATE(1152),
    [sym_block_comment] = STATE(1152),
    [ts_builtin_sym_end] = ACTIONS(4366),
    [sym_identifier] = ACTIONS(4368),
    [anon_sym_SEMI] = ACTIONS(4366),
    [anon_sym_macro] = ACTIONS(4368),
    [anon_sym_LPAREN] = ACTIONS(4366),
    [anon_sym_LBRACE] = ACTIONS(4366),
    [anon_sym_RBRACE] = ACTIONS(4366),
    [anon_sym_macro_rules] = ACTIONS(4368),
    [anon_sym_BANG] = ACTIONS(4366),
    [anon_sym_LBRACK] = ACTIONS(4366),
    [anon_sym_STAR] = ACTIONS(4366),
    [anon_sym_u8] = ACTIONS(4368),
    [anon_sym_i8] = ACTIONS(4368),
    [anon_sym_u16] = ACTIONS(4368),
    [anon_sym_i16] = ACTIONS(4368),
    [anon_sym_u32] = ACTIONS(4368),
    [anon_sym_i32] = ACTIONS(4368),
    [anon_sym_u64] = ACTIONS(4368),
    [anon_sym_i64] = ACTIONS(4368),
    [anon_sym_u128] = ACTIONS(4368),
    [anon_sym_i128] = ACTIONS(4368),
    [anon_sym_isize] = ACTIONS(4368),
    [anon_sym_usize] = ACTIONS(4368),
    [anon_sym_f32] = ACTIONS(4368),
    [anon_sym_f64] = ACTIONS(4368),
    [anon_sym_bool] = ACTIONS(4368),
    [anon_sym_str] = ACTIONS(4368),
    [anon_sym_char] = ACTIONS(4368),
    [anon_sym_DASH] = ACTIONS(4366),
    [anon_sym_AMP] = ACTIONS(4366),
    [anon_sym_PIPE] = ACTIONS(4366),
    [anon_sym_LT] = ACTIONS(4366),
    [anon_sym_DOT_DOT] = ACTIONS(4366),
    [anon_sym_COLON_COLON] = ACTIONS(4366),
    [anon_sym_POUND] = ACTIONS(4366),
    [anon_sym_SQUOTE] = ACTIONS(4368),
    [anon_sym_async] = ACTIONS(4368),
    [anon_sym_auto] = ACTIONS(4368),
    [anon_sym_await] = ACTIONS(4368),
    [anon_sym_become] = ACTIONS(4368),
    [anon_sym_break] = ACTIONS(4368),
    [anon_sym_const] = ACTIONS(4368),
    [anon_sym_continue] = ACTIONS(4368),
    [anon_sym_default] = ACTIONS(4368),
    [anon_sym_enum] = ACTIONS(4368),
    [anon_sym_fn] = ACTIONS(4368),
    [anon_sym_for] = ACTIONS(4368),
    [anon_sym_gen] = ACTIONS(4368),
    [anon_sym_if] = ACTIONS(4368),
    [anon_sym_impl] = ACTIONS(4368),
    [anon_sym_let] = ACTIONS(4368),
    [anon_sym_loop] = ACTIONS(4368),
    [anon_sym_match] = ACTIONS(4368),
    [anon_sym_mod] = ACTIONS(4368),
    [anon_sym_pub] = ACTIONS(4368),
    [anon_sym_return] = ACTIONS(4368),
    [anon_sym_raw] = ACTIONS(4368),
    [anon_sym_static] = ACTIONS(4368),
    [anon_sym_struct] = ACTIONS(4368),
    [anon_sym_trait] = ACTIONS(4368),
    [anon_sym_type] = ACTIONS(4368),
    [anon_sym_union] = ACTIONS(4368),
    [anon_sym_unsafe] = ACTIONS(4368),
    [anon_sym_use] = ACTIONS(4368),
    [anon_sym_while] = ACTIONS(4368),
    [anon_sym_extern] = ACTIONS(4368),
    [anon_sym_safe] = ACTIONS(4368),
    [anon_sym_yield] = ACTIONS(4368),
    [anon_sym_move] = ACTIONS(4368),
    [anon_sym_try] = ACTIONS(4368),
    [sym_integer_literal] = ACTIONS(4366),
    [aux_sym_string_literal_token1] = ACTIONS(4366),
    [sym_char_literal] = ACTIONS(4366),
    [anon_sym_true] = ACTIONS(4368),
    [anon_sym_false] = ACTIONS(4368),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4368),
    [sym_super] = ACTIONS(4368),
    [sym_crate] = ACTIONS(4368),
    [aux_sym_metavariable_token1] = ACTIONS(4366),
    [sym__raw_string_literal_start] = ACTIONS(4366),
    [sym_float_literal] = ACTIONS(4366),
    [sym__outer_block_doc_comment_start] = ACTIONS(4366),
    [sym__outer_line_doc_comment_start] = ACTIONS(4366),
  },
  [STATE(1153)] = {
    [sym_line_comment] = STATE(1153),
    [sym_block_comment] = STATE(1153),
    [ts_builtin_sym_end] = ACTIONS(4370),
    [sym_identifier] = ACTIONS(4372),
    [anon_sym_SEMI] = ACTIONS(4370),
    [anon_sym_macro] = ACTIONS(4372),
    [anon_sym_LPAREN] = ACTIONS(4370),
    [anon_sym_LBRACE] = ACTIONS(4370),
    [anon_sym_RBRACE] = ACTIONS(4370),
    [anon_sym_macro_rules] = ACTIONS(4372),
    [anon_sym_BANG] = ACTIONS(4370),
    [anon_sym_LBRACK] = ACTIONS(4370),
    [anon_sym_STAR] = ACTIONS(4370),
    [anon_sym_u8] = ACTIONS(4372),
    [anon_sym_i8] = ACTIONS(4372),
    [anon_sym_u16] = ACTIONS(4372),
    [anon_sym_i16] = ACTIONS(4372),
    [anon_sym_u32] = ACTIONS(4372),
    [anon_sym_i32] = ACTIONS(4372),
    [anon_sym_u64] = ACTIONS(4372),
    [anon_sym_i64] = ACTIONS(4372),
    [anon_sym_u128] = ACTIONS(4372),
    [anon_sym_i128] = ACTIONS(4372),
    [anon_sym_isize] = ACTIONS(4372),
    [anon_sym_usize] = ACTIONS(4372),
    [anon_sym_f32] = ACTIONS(4372),
    [anon_sym_f64] = ACTIONS(4372),
    [anon_sym_bool] = ACTIONS(4372),
    [anon_sym_str] = ACTIONS(4372),
    [anon_sym_char] = ACTIONS(4372),
    [anon_sym_DASH] = ACTIONS(4370),
    [anon_sym_AMP] = ACTIONS(4370),
    [anon_sym_PIPE] = ACTIONS(4370),
    [anon_sym_LT] = ACTIONS(4370),
    [anon_sym_DOT_DOT] = ACTIONS(4370),
    [anon_sym_COLON_COLON] = ACTIONS(4370),
    [anon_sym_POUND] = ACTIONS(4370),
    [anon_sym_SQUOTE] = ACTIONS(4372),
    [anon_sym_async] = ACTIONS(4372),
    [anon_sym_auto] = ACTIONS(4372),
    [anon_sym_await] = ACTIONS(4372),
    [anon_sym_become] = ACTIONS(4372),
    [anon_sym_break] = ACTIONS(4372),
    [anon_sym_const] = ACTIONS(4372),
    [anon_sym_continue] = ACTIONS(4372),
    [anon_sym_default] = ACTIONS(4372),
    [anon_sym_enum] = ACTIONS(4372),
    [anon_sym_fn] = ACTIONS(4372),
    [anon_sym_for] = ACTIONS(4372),
    [anon_sym_gen] = ACTIONS(4372),
    [anon_sym_if] = ACTIONS(4372),
    [anon_sym_impl] = ACTIONS(4372),
    [anon_sym_let] = ACTIONS(4372),
    [anon_sym_loop] = ACTIONS(4372),
    [anon_sym_match] = ACTIONS(4372),
    [anon_sym_mod] = ACTIONS(4372),
    [anon_sym_pub] = ACTIONS(4372),
    [anon_sym_return] = ACTIONS(4372),
    [anon_sym_raw] = ACTIONS(4372),
    [anon_sym_static] = ACTIONS(4372),
    [anon_sym_struct] = ACTIONS(4372),
    [anon_sym_trait] = ACTIONS(4372),
    [anon_sym_type] = ACTIONS(4372),
    [anon_sym_union] = ACTIONS(4372),
    [anon_sym_unsafe] = ACTIONS(4372),
    [anon_sym_use] = ACTIONS(4372),
    [anon_sym_while] = ACTIONS(4372),
    [anon_sym_extern] = ACTIONS(4372),
    [anon_sym_safe] = ACTIONS(4372),
    [anon_sym_yield] = ACTIONS(4372),
    [anon_sym_move] = ACTIONS(4372),
    [anon_sym_try] = ACTIONS(4372),
    [sym_integer_literal] = ACTIONS(4370),
    [aux_sym_string_literal_token1] = ACTIONS(4370),
    [sym_char_literal] = ACTIONS(4370),
    [anon_sym_true] = ACTIONS(4372),
    [anon_sym_false] = ACTIONS(4372),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4372),
    [sym_super] = ACTIONS(4372),
    [sym_crate] = ACTIONS(4372),
    [aux_sym_metavariable_token1] = ACTIONS(4370),
    [sym__raw_string_literal_start] = ACTIONS(4370),
    [sym_float_literal] = ACTIONS(4370),
    [sym__outer_block_doc_comment_start] = ACTIONS(4370),
    [sym__outer_line_doc_comment_start] = ACTIONS(4370),
  },
  [STATE(1154)] = {
    [sym_line_comment] = STATE(1154),
    [sym_block_comment] = STATE(1154),
    [ts_builtin_sym_end] = ACTIONS(4374),
    [sym_identifier] = ACTIONS(4376),
    [anon_sym_SEMI] = ACTIONS(4374),
    [anon_sym_macro] = ACTIONS(4376),
    [anon_sym_LPAREN] = ACTIONS(4374),
    [anon_sym_LBRACE] = ACTIONS(4374),
    [anon_sym_RBRACE] = ACTIONS(4374),
    [anon_sym_macro_rules] = ACTIONS(4376),
    [anon_sym_BANG] = ACTIONS(4374),
    [anon_sym_LBRACK] = ACTIONS(4374),
    [anon_sym_STAR] = ACTIONS(4374),
    [anon_sym_u8] = ACTIONS(4376),
    [anon_sym_i8] = ACTIONS(4376),
    [anon_sym_u16] = ACTIONS(4376),
    [anon_sym_i16] = ACTIONS(4376),
    [anon_sym_u32] = ACTIONS(4376),
    [anon_sym_i32] = ACTIONS(4376),
    [anon_sym_u64] = ACTIONS(4376),
    [anon_sym_i64] = ACTIONS(4376),
    [anon_sym_u128] = ACTIONS(4376),
    [anon_sym_i128] = ACTIONS(4376),
    [anon_sym_isize] = ACTIONS(4376),
    [anon_sym_usize] = ACTIONS(4376),
    [anon_sym_f32] = ACTIONS(4376),
    [anon_sym_f64] = ACTIONS(4376),
    [anon_sym_bool] = ACTIONS(4376),
    [anon_sym_str] = ACTIONS(4376),
    [anon_sym_char] = ACTIONS(4376),
    [anon_sym_DASH] = ACTIONS(4374),
    [anon_sym_AMP] = ACTIONS(4374),
    [anon_sym_PIPE] = ACTIONS(4374),
    [anon_sym_LT] = ACTIONS(4374),
    [anon_sym_DOT_DOT] = ACTIONS(4374),
    [anon_sym_COLON_COLON] = ACTIONS(4374),
    [anon_sym_POUND] = ACTIONS(4374),
    [anon_sym_SQUOTE] = ACTIONS(4376),
    [anon_sym_async] = ACTIONS(4376),
    [anon_sym_auto] = ACTIONS(4376),
    [anon_sym_await] = ACTIONS(4376),
    [anon_sym_become] = ACTIONS(4376),
    [anon_sym_break] = ACTIONS(4376),
    [anon_sym_const] = ACTIONS(4376),
    [anon_sym_continue] = ACTIONS(4376),
    [anon_sym_default] = ACTIONS(4376),
    [anon_sym_enum] = ACTIONS(4376),
    [anon_sym_fn] = ACTIONS(4376),
    [anon_sym_for] = ACTIONS(4376),
    [anon_sym_gen] = ACTIONS(4376),
    [anon_sym_if] = ACTIONS(4376),
    [anon_sym_impl] = ACTIONS(4376),
    [anon_sym_let] = ACTIONS(4376),
    [anon_sym_loop] = ACTIONS(4376),
    [anon_sym_match] = ACTIONS(4376),
    [anon_sym_mod] = ACTIONS(4376),
    [anon_sym_pub] = ACTIONS(4376),
    [anon_sym_return] = ACTIONS(4376),
    [anon_sym_raw] = ACTIONS(4376),
    [anon_sym_static] = ACTIONS(4376),
    [anon_sym_struct] = ACTIONS(4376),
    [anon_sym_trait] = ACTIONS(4376),
    [anon_sym_type] = ACTIONS(4376),
    [anon_sym_union] = ACTIONS(4376),
    [anon_sym_unsafe] = ACTIONS(4376),
    [anon_sym_use] = ACTIONS(4376),
    [anon_sym_while] = ACTIONS(4376),
    [anon_sym_extern] = ACTIONS(4376),
    [anon_sym_safe] = ACTIONS(4376),
    [anon_sym_yield] = ACTIONS(4376),
    [anon_sym_move] = ACTIONS(4376),
    [anon_sym_try] = ACTIONS(4376),
    [sym_integer_literal] = ACTIONS(4374),
    [aux_sym_string_literal_token1] = ACTIONS(4374),
    [sym_char_literal] = ACTIONS(4374),
    [anon_sym_true] = ACTIONS(4376),
    [anon_sym_false] = ACTIONS(4376),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4376),
    [sym_super] = ACTIONS(4376),
    [sym_crate] = ACTIONS(4376),
    [aux_sym_metavariable_token1] = ACTIONS(4374),
    [sym__raw_string_literal_start] = ACTIONS(4374),
    [sym_float_literal] = ACTIONS(4374),
    [sym__outer_block_doc_comment_start] = ACTIONS(4374),
    [sym__outer_line_doc_comment_start] = ACTIONS(4374),
  },
  [STATE(1155)] = {
    [sym_line_comment] = STATE(1155),
    [sym_block_comment] = STATE(1155),
    [ts_builtin_sym_end] = ACTIONS(4378),
    [sym_identifier] = ACTIONS(4380),
    [anon_sym_SEMI] = ACTIONS(4378),
    [anon_sym_macro] = ACTIONS(4380),
    [anon_sym_LPAREN] = ACTIONS(4378),
    [anon_sym_LBRACE] = ACTIONS(4378),
    [anon_sym_RBRACE] = ACTIONS(4378),
    [anon_sym_macro_rules] = ACTIONS(4380),
    [anon_sym_BANG] = ACTIONS(4378),
    [anon_sym_LBRACK] = ACTIONS(4378),
    [anon_sym_STAR] = ACTIONS(4378),
    [anon_sym_u8] = ACTIONS(4380),
    [anon_sym_i8] = ACTIONS(4380),
    [anon_sym_u16] = ACTIONS(4380),
    [anon_sym_i16] = ACTIONS(4380),
    [anon_sym_u32] = ACTIONS(4380),
    [anon_sym_i32] = ACTIONS(4380),
    [anon_sym_u64] = ACTIONS(4380),
    [anon_sym_i64] = ACTIONS(4380),
    [anon_sym_u128] = ACTIONS(4380),
    [anon_sym_i128] = ACTIONS(4380),
    [anon_sym_isize] = ACTIONS(4380),
    [anon_sym_usize] = ACTIONS(4380),
    [anon_sym_f32] = ACTIONS(4380),
    [anon_sym_f64] = ACTIONS(4380),
    [anon_sym_bool] = ACTIONS(4380),
    [anon_sym_str] = ACTIONS(4380),
    [anon_sym_char] = ACTIONS(4380),
    [anon_sym_DASH] = ACTIONS(4378),
    [anon_sym_AMP] = ACTIONS(4378),
    [anon_sym_PIPE] = ACTIONS(4378),
    [anon_sym_LT] = ACTIONS(4378),
    [anon_sym_DOT_DOT] = ACTIONS(4378),
    [anon_sym_COLON_COLON] = ACTIONS(4378),
    [anon_sym_POUND] = ACTIONS(4378),
    [anon_sym_SQUOTE] = ACTIONS(4380),
    [anon_sym_async] = ACTIONS(4380),
    [anon_sym_auto] = ACTIONS(4380),
    [anon_sym_await] = ACTIONS(4380),
    [anon_sym_become] = ACTIONS(4380),
    [anon_sym_break] = ACTIONS(4380),
    [anon_sym_const] = ACTIONS(4380),
    [anon_sym_continue] = ACTIONS(4380),
    [anon_sym_default] = ACTIONS(4380),
    [anon_sym_enum] = ACTIONS(4380),
    [anon_sym_fn] = ACTIONS(4380),
    [anon_sym_for] = ACTIONS(4380),
    [anon_sym_gen] = ACTIONS(4380),
    [anon_sym_if] = ACTIONS(4380),
    [anon_sym_impl] = ACTIONS(4380),
    [anon_sym_let] = ACTIONS(4380),
    [anon_sym_loop] = ACTIONS(4380),
    [anon_sym_match] = ACTIONS(4380),
    [anon_sym_mod] = ACTIONS(4380),
    [anon_sym_pub] = ACTIONS(4380),
    [anon_sym_return] = ACTIONS(4380),
    [anon_sym_raw] = ACTIONS(4380),
    [anon_sym_static] = ACTIONS(4380),
    [anon_sym_struct] = ACTIONS(4380),
    [anon_sym_trait] = ACTIONS(4380),
    [anon_sym_type] = ACTIONS(4380),
    [anon_sym_union] = ACTIONS(4380),
    [anon_sym_unsafe] = ACTIONS(4380),
    [anon_sym_use] = ACTIONS(4380),
    [anon_sym_while] = ACTIONS(4380),
    [anon_sym_extern] = ACTIONS(4380),
    [anon_sym_safe] = ACTIONS(4380),
    [anon_sym_yield] = ACTIONS(4380),
    [anon_sym_move] = ACTIONS(4380),
    [anon_sym_try] = ACTIONS(4380),
    [sym_integer_literal] = ACTIONS(4378),
    [aux_sym_string_literal_token1] = ACTIONS(4378),
    [sym_char_literal] = ACTIONS(4378),
    [anon_sym_true] = ACTIONS(4380),
    [anon_sym_false] = ACTIONS(4380),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4380),
    [sym_super] = ACTIONS(4380),
    [sym_crate] = ACTIONS(4380),
    [aux_sym_metavariable_token1] = ACTIONS(4378),
    [sym__raw_string_literal_start] = ACTIONS(4378),
    [sym_float_literal] = ACTIONS(4378),
    [sym__outer_block_doc_comment_start] = ACTIONS(4378),
    [sym__outer_line_doc_comment_start] = ACTIONS(4378),
  },
  [STATE(1156)] = {
    [sym_line_comment] = STATE(1156),
    [sym_block_comment] = STATE(1156),
    [ts_builtin_sym_end] = ACTIONS(4382),
    [sym_identifier] = ACTIONS(4384),
    [anon_sym_SEMI] = ACTIONS(4382),
    [anon_sym_macro] = ACTIONS(4384),
    [anon_sym_LPAREN] = ACTIONS(4382),
    [anon_sym_LBRACE] = ACTIONS(4382),
    [anon_sym_RBRACE] = ACTIONS(4382),
    [anon_sym_macro_rules] = ACTIONS(4384),
    [anon_sym_BANG] = ACTIONS(4382),
    [anon_sym_LBRACK] = ACTIONS(4382),
    [anon_sym_STAR] = ACTIONS(4382),
    [anon_sym_u8] = ACTIONS(4384),
    [anon_sym_i8] = ACTIONS(4384),
    [anon_sym_u16] = ACTIONS(4384),
    [anon_sym_i16] = ACTIONS(4384),
    [anon_sym_u32] = ACTIONS(4384),
    [anon_sym_i32] = ACTIONS(4384),
    [anon_sym_u64] = ACTIONS(4384),
    [anon_sym_i64] = ACTIONS(4384),
    [anon_sym_u128] = ACTIONS(4384),
    [anon_sym_i128] = ACTIONS(4384),
    [anon_sym_isize] = ACTIONS(4384),
    [anon_sym_usize] = ACTIONS(4384),
    [anon_sym_f32] = ACTIONS(4384),
    [anon_sym_f64] = ACTIONS(4384),
    [anon_sym_bool] = ACTIONS(4384),
    [anon_sym_str] = ACTIONS(4384),
    [anon_sym_char] = ACTIONS(4384),
    [anon_sym_DASH] = ACTIONS(4382),
    [anon_sym_AMP] = ACTIONS(4382),
    [anon_sym_PIPE] = ACTIONS(4382),
    [anon_sym_LT] = ACTIONS(4382),
    [anon_sym_DOT_DOT] = ACTIONS(4382),
    [anon_sym_COLON_COLON] = ACTIONS(4382),
    [anon_sym_POUND] = ACTIONS(4382),
    [anon_sym_SQUOTE] = ACTIONS(4384),
    [anon_sym_async] = ACTIONS(4384),
    [anon_sym_auto] = ACTIONS(4384),
    [anon_sym_await] = ACTIONS(4384),
    [anon_sym_become] = ACTIONS(4384),
    [anon_sym_break] = ACTIONS(4384),
    [anon_sym_const] = ACTIONS(4384),
    [anon_sym_continue] = ACTIONS(4384),
    [anon_sym_default] = ACTIONS(4384),
    [anon_sym_enum] = ACTIONS(4384),
    [anon_sym_fn] = ACTIONS(4384),
    [anon_sym_for] = ACTIONS(4384),
    [anon_sym_gen] = ACTIONS(4384),
    [anon_sym_if] = ACTIONS(4384),
    [anon_sym_impl] = ACTIONS(4384),
    [anon_sym_let] = ACTIONS(4384),
    [anon_sym_loop] = ACTIONS(4384),
    [anon_sym_match] = ACTIONS(4384),
    [anon_sym_mod] = ACTIONS(4384),
    [anon_sym_pub] = ACTIONS(4384),
    [anon_sym_return] = ACTIONS(4384),
    [anon_sym_raw] = ACTIONS(4384),
    [anon_sym_static] = ACTIONS(4384),
    [anon_sym_struct] = ACTIONS(4384),
    [anon_sym_trait] = ACTIONS(4384),
    [anon_sym_type] = ACTIONS(4384),
    [anon_sym_union] = ACTIONS(4384),
    [anon_sym_unsafe] = ACTIONS(4384),
    [anon_sym_use] = ACTIONS(4384),
    [anon_sym_while] = ACTIONS(4384),
    [anon_sym_extern] = ACTIONS(4384),
    [anon_sym_safe] = ACTIONS(4384),
    [anon_sym_yield] = ACTIONS(4384),
    [anon_sym_move] = ACTIONS(4384),
    [anon_sym_try] = ACTIONS(4384),
    [sym_integer_literal] = ACTIONS(4382),
    [aux_sym_string_literal_token1] = ACTIONS(4382),
    [sym_char_literal] = ACTIONS(4382),
    [anon_sym_true] = ACTIONS(4384),
    [anon_sym_false] = ACTIONS(4384),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4384),
    [sym_super] = ACTIONS(4384),
    [sym_crate] = ACTIONS(4384),
    [aux_sym_metavariable_token1] = ACTIONS(4382),
    [sym__raw_string_literal_start] = ACTIONS(4382),
    [sym_float_literal] = ACTIONS(4382),
    [sym__outer_block_doc_comment_start] = ACTIONS(4382),
    [sym__outer_line_doc_comment_start] = ACTIONS(4382),
  },
  [STATE(1157)] = {
    [sym_line_comment] = STATE(1157),
    [sym_block_comment] = STATE(1157),
    [ts_builtin_sym_end] = ACTIONS(4386),
    [sym_identifier] = ACTIONS(4388),
    [anon_sym_SEMI] = ACTIONS(4386),
    [anon_sym_macro] = ACTIONS(4388),
    [anon_sym_LPAREN] = ACTIONS(4386),
    [anon_sym_LBRACE] = ACTIONS(4386),
    [anon_sym_RBRACE] = ACTIONS(4386),
    [anon_sym_macro_rules] = ACTIONS(4388),
    [anon_sym_BANG] = ACTIONS(4386),
    [anon_sym_LBRACK] = ACTIONS(4386),
    [anon_sym_STAR] = ACTIONS(4386),
    [anon_sym_u8] = ACTIONS(4388),
    [anon_sym_i8] = ACTIONS(4388),
    [anon_sym_u16] = ACTIONS(4388),
    [anon_sym_i16] = ACTIONS(4388),
    [anon_sym_u32] = ACTIONS(4388),
    [anon_sym_i32] = ACTIONS(4388),
    [anon_sym_u64] = ACTIONS(4388),
    [anon_sym_i64] = ACTIONS(4388),
    [anon_sym_u128] = ACTIONS(4388),
    [anon_sym_i128] = ACTIONS(4388),
    [anon_sym_isize] = ACTIONS(4388),
    [anon_sym_usize] = ACTIONS(4388),
    [anon_sym_f32] = ACTIONS(4388),
    [anon_sym_f64] = ACTIONS(4388),
    [anon_sym_bool] = ACTIONS(4388),
    [anon_sym_str] = ACTIONS(4388),
    [anon_sym_char] = ACTIONS(4388),
    [anon_sym_DASH] = ACTIONS(4386),
    [anon_sym_AMP] = ACTIONS(4386),
    [anon_sym_PIPE] = ACTIONS(4386),
    [anon_sym_LT] = ACTIONS(4386),
    [anon_sym_DOT_DOT] = ACTIONS(4386),
    [anon_sym_COLON_COLON] = ACTIONS(4386),
    [anon_sym_POUND] = ACTIONS(4386),
    [anon_sym_SQUOTE] = ACTIONS(4388),
    [anon_sym_async] = ACTIONS(4388),
    [anon_sym_auto] = ACTIONS(4388),
    [anon_sym_await] = ACTIONS(4388),
    [anon_sym_become] = ACTIONS(4388),
    [anon_sym_break] = ACTIONS(4388),
    [anon_sym_const] = ACTIONS(4388),
    [anon_sym_continue] = ACTIONS(4388),
    [anon_sym_default] = ACTIONS(4388),
    [anon_sym_enum] = ACTIONS(4388),
    [anon_sym_fn] = ACTIONS(4388),
    [anon_sym_for] = ACTIONS(4388),
    [anon_sym_gen] = ACTIONS(4388),
    [anon_sym_if] = ACTIONS(4388),
    [anon_sym_impl] = ACTIONS(4388),
    [anon_sym_let] = ACTIONS(4388),
    [anon_sym_loop] = ACTIONS(4388),
    [anon_sym_match] = ACTIONS(4388),
    [anon_sym_mod] = ACTIONS(4388),
    [anon_sym_pub] = ACTIONS(4388),
    [anon_sym_return] = ACTIONS(4388),
    [anon_sym_raw] = ACTIONS(4388),
    [anon_sym_static] = ACTIONS(4388),
    [anon_sym_struct] = ACTIONS(4388),
    [anon_sym_trait] = ACTIONS(4388),
    [anon_sym_type] = ACTIONS(4388),
    [anon_sym_union] = ACTIONS(4388),
    [anon_sym_unsafe] = ACTIONS(4388),
    [anon_sym_use] = ACTIONS(4388),
    [anon_sym_while] = ACTIONS(4388),
    [anon_sym_extern] = ACTIONS(4388),
    [anon_sym_safe] = ACTIONS(4388),
    [anon_sym_yield] = ACTIONS(4388),
    [anon_sym_move] = ACTIONS(4388),
    [anon_sym_try] = ACTIONS(4388),
    [sym_integer_literal] = ACTIONS(4386),
    [aux_sym_string_literal_token1] = ACTIONS(4386),
    [sym_char_literal] = ACTIONS(4386),
    [anon_sym_true] = ACTIONS(4388),
    [anon_sym_false] = ACTIONS(4388),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4388),
    [sym_super] = ACTIONS(4388),
    [sym_crate] = ACTIONS(4388),
    [aux_sym_metavariable_token1] = ACTIONS(4386),
    [sym__raw_string_literal_start] = ACTIONS(4386),
    [sym_float_literal] = ACTIONS(4386),
    [sym__outer_block_doc_comment_start] = ACTIONS(4386),
    [sym__outer_line_doc_comment_start] = ACTIONS(4386),
  },
  [STATE(1158)] = {
    [sym_line_comment] = STATE(1158),
    [sym_block_comment] = STATE(1158),
    [ts_builtin_sym_end] = ACTIONS(4390),
    [sym_identifier] = ACTIONS(4392),
    [anon_sym_SEMI] = ACTIONS(4390),
    [anon_sym_macro] = ACTIONS(4392),
    [anon_sym_LPAREN] = ACTIONS(4390),
    [anon_sym_LBRACE] = ACTIONS(4390),
    [anon_sym_RBRACE] = ACTIONS(4390),
    [anon_sym_macro_rules] = ACTIONS(4392),
    [anon_sym_BANG] = ACTIONS(4390),
    [anon_sym_LBRACK] = ACTIONS(4390),
    [anon_sym_STAR] = ACTIONS(4390),
    [anon_sym_u8] = ACTIONS(4392),
    [anon_sym_i8] = ACTIONS(4392),
    [anon_sym_u16] = ACTIONS(4392),
    [anon_sym_i16] = ACTIONS(4392),
    [anon_sym_u32] = ACTIONS(4392),
    [anon_sym_i32] = ACTIONS(4392),
    [anon_sym_u64] = ACTIONS(4392),
    [anon_sym_i64] = ACTIONS(4392),
    [anon_sym_u128] = ACTIONS(4392),
    [anon_sym_i128] = ACTIONS(4392),
    [anon_sym_isize] = ACTIONS(4392),
    [anon_sym_usize] = ACTIONS(4392),
    [anon_sym_f32] = ACTIONS(4392),
    [anon_sym_f64] = ACTIONS(4392),
    [anon_sym_bool] = ACTIONS(4392),
    [anon_sym_str] = ACTIONS(4392),
    [anon_sym_char] = ACTIONS(4392),
    [anon_sym_DASH] = ACTIONS(4390),
    [anon_sym_AMP] = ACTIONS(4390),
    [anon_sym_PIPE] = ACTIONS(4390),
    [anon_sym_LT] = ACTIONS(4390),
    [anon_sym_DOT_DOT] = ACTIONS(4390),
    [anon_sym_COLON_COLON] = ACTIONS(4390),
    [anon_sym_POUND] = ACTIONS(4390),
    [anon_sym_SQUOTE] = ACTIONS(4392),
    [anon_sym_async] = ACTIONS(4392),
    [anon_sym_auto] = ACTIONS(4392),
    [anon_sym_await] = ACTIONS(4392),
    [anon_sym_become] = ACTIONS(4392),
    [anon_sym_break] = ACTIONS(4392),
    [anon_sym_const] = ACTIONS(4392),
    [anon_sym_continue] = ACTIONS(4392),
    [anon_sym_default] = ACTIONS(4392),
    [anon_sym_enum] = ACTIONS(4392),
    [anon_sym_fn] = ACTIONS(4392),
    [anon_sym_for] = ACTIONS(4392),
    [anon_sym_gen] = ACTIONS(4392),
    [anon_sym_if] = ACTIONS(4392),
    [anon_sym_impl] = ACTIONS(4392),
    [anon_sym_let] = ACTIONS(4392),
    [anon_sym_loop] = ACTIONS(4392),
    [anon_sym_match] = ACTIONS(4392),
    [anon_sym_mod] = ACTIONS(4392),
    [anon_sym_pub] = ACTIONS(4392),
    [anon_sym_return] = ACTIONS(4392),
    [anon_sym_raw] = ACTIONS(4392),
    [anon_sym_static] = ACTIONS(4392),
    [anon_sym_struct] = ACTIONS(4392),
    [anon_sym_trait] = ACTIONS(4392),
    [anon_sym_type] = ACTIONS(4392),
    [anon_sym_union] = ACTIONS(4392),
    [anon_sym_unsafe] = ACTIONS(4392),
    [anon_sym_use] = ACTIONS(4392),
    [anon_sym_while] = ACTIONS(4392),
    [anon_sym_extern] = ACTIONS(4392),
    [anon_sym_safe] = ACTIONS(4392),
    [anon_sym_yield] = ACTIONS(4392),
    [anon_sym_move] = ACTIONS(4392),
    [anon_sym_try] = ACTIONS(4392),
    [sym_integer_literal] = ACTIONS(4390),
    [aux_sym_string_literal_token1] = ACTIONS(4390),
    [sym_char_literal] = ACTIONS(4390),
    [anon_sym_true] = ACTIONS(4392),
    [anon_sym_false] = ACTIONS(4392),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4392),
    [sym_super] = ACTIONS(4392),
    [sym_crate] = ACTIONS(4392),
    [aux_sym_metavariable_token1] = ACTIONS(4390),
    [sym__raw_string_literal_start] = ACTIONS(4390),
    [sym_float_literal] = ACTIONS(4390),
    [sym__outer_block_doc_comment_start] = ACTIONS(4390),
    [sym__outer_line_doc_comment_start] = ACTIONS(4390),
  },
  [STATE(1159)] = {
    [sym_line_comment] = STATE(1159),
    [sym_block_comment] = STATE(1159),
    [ts_builtin_sym_end] = ACTIONS(4394),
    [sym_identifier] = ACTIONS(4396),
    [anon_sym_SEMI] = ACTIONS(4394),
    [anon_sym_macro] = ACTIONS(4396),
    [anon_sym_LPAREN] = ACTIONS(4394),
    [anon_sym_LBRACE] = ACTIONS(4394),
    [anon_sym_RBRACE] = ACTIONS(4394),
    [anon_sym_macro_rules] = ACTIONS(4396),
    [anon_sym_BANG] = ACTIONS(4394),
    [anon_sym_LBRACK] = ACTIONS(4394),
    [anon_sym_STAR] = ACTIONS(4394),
    [anon_sym_u8] = ACTIONS(4396),
    [anon_sym_i8] = ACTIONS(4396),
    [anon_sym_u16] = ACTIONS(4396),
    [anon_sym_i16] = ACTIONS(4396),
    [anon_sym_u32] = ACTIONS(4396),
    [anon_sym_i32] = ACTIONS(4396),
    [anon_sym_u64] = ACTIONS(4396),
    [anon_sym_i64] = ACTIONS(4396),
    [anon_sym_u128] = ACTIONS(4396),
    [anon_sym_i128] = ACTIONS(4396),
    [anon_sym_isize] = ACTIONS(4396),
    [anon_sym_usize] = ACTIONS(4396),
    [anon_sym_f32] = ACTIONS(4396),
    [anon_sym_f64] = ACTIONS(4396),
    [anon_sym_bool] = ACTIONS(4396),
    [anon_sym_str] = ACTIONS(4396),
    [anon_sym_char] = ACTIONS(4396),
    [anon_sym_DASH] = ACTIONS(4394),
    [anon_sym_AMP] = ACTIONS(4394),
    [anon_sym_PIPE] = ACTIONS(4394),
    [anon_sym_LT] = ACTIONS(4394),
    [anon_sym_DOT_DOT] = ACTIONS(4394),
    [anon_sym_COLON_COLON] = ACTIONS(4394),
    [anon_sym_POUND] = ACTIONS(4394),
    [anon_sym_SQUOTE] = ACTIONS(4396),
    [anon_sym_async] = ACTIONS(4396),
    [anon_sym_auto] = ACTIONS(4396),
    [anon_sym_await] = ACTIONS(4396),
    [anon_sym_become] = ACTIONS(4396),
    [anon_sym_break] = ACTIONS(4396),
    [anon_sym_const] = ACTIONS(4396),
    [anon_sym_continue] = ACTIONS(4396),
    [anon_sym_default] = ACTIONS(4396),
    [anon_sym_enum] = ACTIONS(4396),
    [anon_sym_fn] = ACTIONS(4396),
    [anon_sym_for] = ACTIONS(4396),
    [anon_sym_gen] = ACTIONS(4396),
    [anon_sym_if] = ACTIONS(4396),
    [anon_sym_impl] = ACTIONS(4396),
    [anon_sym_let] = ACTIONS(4396),
    [anon_sym_loop] = ACTIONS(4396),
    [anon_sym_match] = ACTIONS(4396),
    [anon_sym_mod] = ACTIONS(4396),
    [anon_sym_pub] = ACTIONS(4396),
    [anon_sym_return] = ACTIONS(4396),
    [anon_sym_raw] = ACTIONS(4396),
    [anon_sym_static] = ACTIONS(4396),
    [anon_sym_struct] = ACTIONS(4396),
    [anon_sym_trait] = ACTIONS(4396),
    [anon_sym_type] = ACTIONS(4396),
    [anon_sym_union] = ACTIONS(4396),
    [anon_sym_unsafe] = ACTIONS(4396),
    [anon_sym_use] = ACTIONS(4396),
    [anon_sym_while] = ACTIONS(4396),
    [anon_sym_extern] = ACTIONS(4396),
    [anon_sym_safe] = ACTIONS(4396),
    [anon_sym_yield] = ACTIONS(4396),
    [anon_sym_move] = ACTIONS(4396),
    [anon_sym_try] = ACTIONS(4396),
    [sym_integer_literal] = ACTIONS(4394),
    [aux_sym_string_literal_token1] = ACTIONS(4394),
    [sym_char_literal] = ACTIONS(4394),
    [anon_sym_true] = ACTIONS(4396),
    [anon_sym_false] = ACTIONS(4396),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4396),
    [sym_super] = ACTIONS(4396),
    [sym_crate] = ACTIONS(4396),
    [aux_sym_metavariable_token1] = ACTIONS(4394),
    [sym__raw_string_literal_start] = ACTIONS(4394),
    [sym_float_literal] = ACTIONS(4394),
    [sym__outer_block_doc_comment_start] = ACTIONS(4394),
    [sym__outer_line_doc_comment_start] = ACTIONS(4394),
  },
  [STATE(1160)] = {
    [sym_line_comment] = STATE(1160),
    [sym_block_comment] = STATE(1160),
    [ts_builtin_sym_end] = ACTIONS(4398),
    [sym_identifier] = ACTIONS(4400),
    [anon_sym_SEMI] = ACTIONS(4398),
    [anon_sym_macro] = ACTIONS(4400),
    [anon_sym_LPAREN] = ACTIONS(4398),
    [anon_sym_LBRACE] = ACTIONS(4398),
    [anon_sym_RBRACE] = ACTIONS(4398),
    [anon_sym_macro_rules] = ACTIONS(4400),
    [anon_sym_BANG] = ACTIONS(4398),
    [anon_sym_LBRACK] = ACTIONS(4398),
    [anon_sym_STAR] = ACTIONS(4398),
    [anon_sym_u8] = ACTIONS(4400),
    [anon_sym_i8] = ACTIONS(4400),
    [anon_sym_u16] = ACTIONS(4400),
    [anon_sym_i16] = ACTIONS(4400),
    [anon_sym_u32] = ACTIONS(4400),
    [anon_sym_i32] = ACTIONS(4400),
    [anon_sym_u64] = ACTIONS(4400),
    [anon_sym_i64] = ACTIONS(4400),
    [anon_sym_u128] = ACTIONS(4400),
    [anon_sym_i128] = ACTIONS(4400),
    [anon_sym_isize] = ACTIONS(4400),
    [anon_sym_usize] = ACTIONS(4400),
    [anon_sym_f32] = ACTIONS(4400),
    [anon_sym_f64] = ACTIONS(4400),
    [anon_sym_bool] = ACTIONS(4400),
    [anon_sym_str] = ACTIONS(4400),
    [anon_sym_char] = ACTIONS(4400),
    [anon_sym_DASH] = ACTIONS(4398),
    [anon_sym_AMP] = ACTIONS(4398),
    [anon_sym_PIPE] = ACTIONS(4398),
    [anon_sym_LT] = ACTIONS(4398),
    [anon_sym_DOT_DOT] = ACTIONS(4398),
    [anon_sym_COLON_COLON] = ACTIONS(4398),
    [anon_sym_POUND] = ACTIONS(4398),
    [anon_sym_SQUOTE] = ACTIONS(4400),
    [anon_sym_async] = ACTIONS(4400),
    [anon_sym_auto] = ACTIONS(4400),
    [anon_sym_await] = ACTIONS(4400),
    [anon_sym_become] = ACTIONS(4400),
    [anon_sym_break] = ACTIONS(4400),
    [anon_sym_const] = ACTIONS(4400),
    [anon_sym_continue] = ACTIONS(4400),
    [anon_sym_default] = ACTIONS(4400),
    [anon_sym_enum] = ACTIONS(4400),
    [anon_sym_fn] = ACTIONS(4400),
    [anon_sym_for] = ACTIONS(4400),
    [anon_sym_gen] = ACTIONS(4400),
    [anon_sym_if] = ACTIONS(4400),
    [anon_sym_impl] = ACTIONS(4400),
    [anon_sym_let] = ACTIONS(4400),
    [anon_sym_loop] = ACTIONS(4400),
    [anon_sym_match] = ACTIONS(4400),
    [anon_sym_mod] = ACTIONS(4400),
    [anon_sym_pub] = ACTIONS(4400),
    [anon_sym_return] = ACTIONS(4400),
    [anon_sym_raw] = ACTIONS(4400),
    [anon_sym_static] = ACTIONS(4400),
    [anon_sym_struct] = ACTIONS(4400),
    [anon_sym_trait] = ACTIONS(4400),
    [anon_sym_type] = ACTIONS(4400),
    [anon_sym_union] = ACTIONS(4400),
    [anon_sym_unsafe] = ACTIONS(4400),
    [anon_sym_use] = ACTIONS(4400),
    [anon_sym_while] = ACTIONS(4400),
    [anon_sym_extern] = ACTIONS(4400),
    [anon_sym_safe] = ACTIONS(4400),
    [anon_sym_yield] = ACTIONS(4400),
    [anon_sym_move] = ACTIONS(4400),
    [anon_sym_try] = ACTIONS(4400),
    [sym_integer_literal] = ACTIONS(4398),
    [aux_sym_string_literal_token1] = ACTIONS(4398),
    [sym_char_literal] = ACTIONS(4398),
    [anon_sym_true] = ACTIONS(4400),
    [anon_sym_false] = ACTIONS(4400),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4400),
    [sym_super] = ACTIONS(4400),
    [sym_crate] = ACTIONS(4400),
    [aux_sym_metavariable_token1] = ACTIONS(4398),
    [sym__raw_string_literal_start] = ACTIONS(4398),
    [sym_float_literal] = ACTIONS(4398),
    [sym__outer_block_doc_comment_start] = ACTIONS(4398),
    [sym__outer_line_doc_comment_start] = ACTIONS(4398),
  },
  [STATE(1161)] = {
    [sym_line_comment] = STATE(1161),
    [sym_block_comment] = STATE(1161),
    [ts_builtin_sym_end] = ACTIONS(4402),
    [sym_identifier] = ACTIONS(4404),
    [anon_sym_SEMI] = ACTIONS(4402),
    [anon_sym_macro] = ACTIONS(4404),
    [anon_sym_LPAREN] = ACTIONS(4402),
    [anon_sym_LBRACE] = ACTIONS(4402),
    [anon_sym_RBRACE] = ACTIONS(4402),
    [anon_sym_macro_rules] = ACTIONS(4404),
    [anon_sym_BANG] = ACTIONS(4402),
    [anon_sym_LBRACK] = ACTIONS(4402),
    [anon_sym_STAR] = ACTIONS(4402),
    [anon_sym_u8] = ACTIONS(4404),
    [anon_sym_i8] = ACTIONS(4404),
    [anon_sym_u16] = ACTIONS(4404),
    [anon_sym_i16] = ACTIONS(4404),
    [anon_sym_u32] = ACTIONS(4404),
    [anon_sym_i32] = ACTIONS(4404),
    [anon_sym_u64] = ACTIONS(4404),
    [anon_sym_i64] = ACTIONS(4404),
    [anon_sym_u128] = ACTIONS(4404),
    [anon_sym_i128] = ACTIONS(4404),
    [anon_sym_isize] = ACTIONS(4404),
    [anon_sym_usize] = ACTIONS(4404),
    [anon_sym_f32] = ACTIONS(4404),
    [anon_sym_f64] = ACTIONS(4404),
    [anon_sym_bool] = ACTIONS(4404),
    [anon_sym_str] = ACTIONS(4404),
    [anon_sym_char] = ACTIONS(4404),
    [anon_sym_DASH] = ACTIONS(4402),
    [anon_sym_AMP] = ACTIONS(4402),
    [anon_sym_PIPE] = ACTIONS(4402),
    [anon_sym_LT] = ACTIONS(4402),
    [anon_sym_DOT_DOT] = ACTIONS(4402),
    [anon_sym_COLON_COLON] = ACTIONS(4402),
    [anon_sym_POUND] = ACTIONS(4402),
    [anon_sym_SQUOTE] = ACTIONS(4404),
    [anon_sym_async] = ACTIONS(4404),
    [anon_sym_auto] = ACTIONS(4404),
    [anon_sym_await] = ACTIONS(4404),
    [anon_sym_become] = ACTIONS(4404),
    [anon_sym_break] = ACTIONS(4404),
    [anon_sym_const] = ACTIONS(4404),
    [anon_sym_continue] = ACTIONS(4404),
    [anon_sym_default] = ACTIONS(4404),
    [anon_sym_enum] = ACTIONS(4404),
    [anon_sym_fn] = ACTIONS(4404),
    [anon_sym_for] = ACTIONS(4404),
    [anon_sym_gen] = ACTIONS(4404),
    [anon_sym_if] = ACTIONS(4404),
    [anon_sym_impl] = ACTIONS(4404),
    [anon_sym_let] = ACTIONS(4404),
    [anon_sym_loop] = ACTIONS(4404),
    [anon_sym_match] = ACTIONS(4404),
    [anon_sym_mod] = ACTIONS(4404),
    [anon_sym_pub] = ACTIONS(4404),
    [anon_sym_return] = ACTIONS(4404),
    [anon_sym_raw] = ACTIONS(4404),
    [anon_sym_static] = ACTIONS(4404),
    [anon_sym_struct] = ACTIONS(4404),
    [anon_sym_trait] = ACTIONS(4404),
    [anon_sym_type] = ACTIONS(4404),
    [anon_sym_union] = ACTIONS(4404),
    [anon_sym_unsafe] = ACTIONS(4404),
    [anon_sym_use] = ACTIONS(4404),
    [anon_sym_while] = ACTIONS(4404),
    [anon_sym_extern] = ACTIONS(4404),
    [anon_sym_safe] = ACTIONS(4404),
    [anon_sym_yield] = ACTIONS(4404),
    [anon_sym_move] = ACTIONS(4404),
    [anon_sym_try] = ACTIONS(4404),
    [sym_integer_literal] = ACTIONS(4402),
    [aux_sym_string_literal_token1] = ACTIONS(4402),
    [sym_char_literal] = ACTIONS(4402),
    [anon_sym_true] = ACTIONS(4404),
    [anon_sym_false] = ACTIONS(4404),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4404),
    [sym_super] = ACTIONS(4404),
    [sym_crate] = ACTIONS(4404),
    [aux_sym_metavariable_token1] = ACTIONS(4402),
    [sym__raw_string_literal_start] = ACTIONS(4402),
    [sym_float_literal] = ACTIONS(4402),
    [sym__outer_block_doc_comment_start] = ACTIONS(4402),
    [sym__outer_line_doc_comment_start] = ACTIONS(4402),
  },
  [STATE(1162)] = {
    [sym_line_comment] = STATE(1162),
    [sym_block_comment] = STATE(1162),
    [ts_builtin_sym_end] = ACTIONS(4406),
    [sym_identifier] = ACTIONS(4408),
    [anon_sym_SEMI] = ACTIONS(4406),
    [anon_sym_macro] = ACTIONS(4408),
    [anon_sym_LPAREN] = ACTIONS(4406),
    [anon_sym_LBRACE] = ACTIONS(4406),
    [anon_sym_RBRACE] = ACTIONS(4406),
    [anon_sym_macro_rules] = ACTIONS(4408),
    [anon_sym_BANG] = ACTIONS(4406),
    [anon_sym_LBRACK] = ACTIONS(4406),
    [anon_sym_STAR] = ACTIONS(4406),
    [anon_sym_u8] = ACTIONS(4408),
    [anon_sym_i8] = ACTIONS(4408),
    [anon_sym_u16] = ACTIONS(4408),
    [anon_sym_i16] = ACTIONS(4408),
    [anon_sym_u32] = ACTIONS(4408),
    [anon_sym_i32] = ACTIONS(4408),
    [anon_sym_u64] = ACTIONS(4408),
    [anon_sym_i64] = ACTIONS(4408),
    [anon_sym_u128] = ACTIONS(4408),
    [anon_sym_i128] = ACTIONS(4408),
    [anon_sym_isize] = ACTIONS(4408),
    [anon_sym_usize] = ACTIONS(4408),
    [anon_sym_f32] = ACTIONS(4408),
    [anon_sym_f64] = ACTIONS(4408),
    [anon_sym_bool] = ACTIONS(4408),
    [anon_sym_str] = ACTIONS(4408),
    [anon_sym_char] = ACTIONS(4408),
    [anon_sym_DASH] = ACTIONS(4406),
    [anon_sym_AMP] = ACTIONS(4406),
    [anon_sym_PIPE] = ACTIONS(4406),
    [anon_sym_LT] = ACTIONS(4406),
    [anon_sym_DOT_DOT] = ACTIONS(4406),
    [anon_sym_COLON_COLON] = ACTIONS(4406),
    [anon_sym_POUND] = ACTIONS(4406),
    [anon_sym_SQUOTE] = ACTIONS(4408),
    [anon_sym_async] = ACTIONS(4408),
    [anon_sym_auto] = ACTIONS(4408),
    [anon_sym_await] = ACTIONS(4408),
    [anon_sym_become] = ACTIONS(4408),
    [anon_sym_break] = ACTIONS(4408),
    [anon_sym_const] = ACTIONS(4408),
    [anon_sym_continue] = ACTIONS(4408),
    [anon_sym_default] = ACTIONS(4408),
    [anon_sym_enum] = ACTIONS(4408),
    [anon_sym_fn] = ACTIONS(4408),
    [anon_sym_for] = ACTIONS(4408),
    [anon_sym_gen] = ACTIONS(4408),
    [anon_sym_if] = ACTIONS(4408),
    [anon_sym_impl] = ACTIONS(4408),
    [anon_sym_let] = ACTIONS(4408),
    [anon_sym_loop] = ACTIONS(4408),
    [anon_sym_match] = ACTIONS(4408),
    [anon_sym_mod] = ACTIONS(4408),
    [anon_sym_pub] = ACTIONS(4408),
    [anon_sym_return] = ACTIONS(4408),
    [anon_sym_raw] = ACTIONS(4408),
    [anon_sym_static] = ACTIONS(4408),
    [anon_sym_struct] = ACTIONS(4408),
    [anon_sym_trait] = ACTIONS(4408),
    [anon_sym_type] = ACTIONS(4408),
    [anon_sym_union] = ACTIONS(4408),
    [anon_sym_unsafe] = ACTIONS(4408),
    [anon_sym_use] = ACTIONS(4408),
    [anon_sym_while] = ACTIONS(4408),
    [anon_sym_extern] = ACTIONS(4408),
    [anon_sym_safe] = ACTIONS(4408),
    [anon_sym_yield] = ACTIONS(4408),
    [anon_sym_move] = ACTIONS(4408),
    [anon_sym_try] = ACTIONS(4408),
    [sym_integer_literal] = ACTIONS(4406),
    [aux_sym_string_literal_token1] = ACTIONS(4406),
    [sym_char_literal] = ACTIONS(4406),
    [anon_sym_true] = ACTIONS(4408),
    [anon_sym_false] = ACTIONS(4408),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4408),
    [sym_super] = ACTIONS(4408),
    [sym_crate] = ACTIONS(4408),
    [aux_sym_metavariable_token1] = ACTIONS(4406),
    [sym__raw_string_literal_start] = ACTIONS(4406),
    [sym_float_literal] = ACTIONS(4406),
    [sym__outer_block_doc_comment_start] = ACTIONS(4406),
    [sym__outer_line_doc_comment_start] = ACTIONS(4406),
  },
  [STATE(1163)] = {
    [sym_line_comment] = STATE(1163),
    [sym_block_comment] = STATE(1163),
    [ts_builtin_sym_end] = ACTIONS(4410),
    [sym_identifier] = ACTIONS(4412),
    [anon_sym_SEMI] = ACTIONS(4410),
    [anon_sym_macro] = ACTIONS(4412),
    [anon_sym_LPAREN] = ACTIONS(4410),
    [anon_sym_LBRACE] = ACTIONS(4410),
    [anon_sym_RBRACE] = ACTIONS(4410),
    [anon_sym_macro_rules] = ACTIONS(4412),
    [anon_sym_BANG] = ACTIONS(4410),
    [anon_sym_LBRACK] = ACTIONS(4410),
    [anon_sym_STAR] = ACTIONS(4410),
    [anon_sym_u8] = ACTIONS(4412),
    [anon_sym_i8] = ACTIONS(4412),
    [anon_sym_u16] = ACTIONS(4412),
    [anon_sym_i16] = ACTIONS(4412),
    [anon_sym_u32] = ACTIONS(4412),
    [anon_sym_i32] = ACTIONS(4412),
    [anon_sym_u64] = ACTIONS(4412),
    [anon_sym_i64] = ACTIONS(4412),
    [anon_sym_u128] = ACTIONS(4412),
    [anon_sym_i128] = ACTIONS(4412),
    [anon_sym_isize] = ACTIONS(4412),
    [anon_sym_usize] = ACTIONS(4412),
    [anon_sym_f32] = ACTIONS(4412),
    [anon_sym_f64] = ACTIONS(4412),
    [anon_sym_bool] = ACTIONS(4412),
    [anon_sym_str] = ACTIONS(4412),
    [anon_sym_char] = ACTIONS(4412),
    [anon_sym_DASH] = ACTIONS(4410),
    [anon_sym_AMP] = ACTIONS(4410),
    [anon_sym_PIPE] = ACTIONS(4410),
    [anon_sym_LT] = ACTIONS(4410),
    [anon_sym_DOT_DOT] = ACTIONS(4410),
    [anon_sym_COLON_COLON] = ACTIONS(4410),
    [anon_sym_POUND] = ACTIONS(4410),
    [anon_sym_SQUOTE] = ACTIONS(4412),
    [anon_sym_async] = ACTIONS(4412),
    [anon_sym_auto] = ACTIONS(4412),
    [anon_sym_await] = ACTIONS(4412),
    [anon_sym_become] = ACTIONS(4412),
    [anon_sym_break] = ACTIONS(4412),
    [anon_sym_const] = ACTIONS(4412),
    [anon_sym_continue] = ACTIONS(4412),
    [anon_sym_default] = ACTIONS(4412),
    [anon_sym_enum] = ACTIONS(4412),
    [anon_sym_fn] = ACTIONS(4412),
    [anon_sym_for] = ACTIONS(4412),
    [anon_sym_gen] = ACTIONS(4412),
    [anon_sym_if] = ACTIONS(4412),
    [anon_sym_impl] = ACTIONS(4412),
    [anon_sym_let] = ACTIONS(4412),
    [anon_sym_loop] = ACTIONS(4412),
    [anon_sym_match] = ACTIONS(4412),
    [anon_sym_mod] = ACTIONS(4412),
    [anon_sym_pub] = ACTIONS(4412),
    [anon_sym_return] = ACTIONS(4412),
    [anon_sym_raw] = ACTIONS(4412),
    [anon_sym_static] = ACTIONS(4412),
    [anon_sym_struct] = ACTIONS(4412),
    [anon_sym_trait] = ACTIONS(4412),
    [anon_sym_type] = ACTIONS(4412),
    [anon_sym_union] = ACTIONS(4412),
    [anon_sym_unsafe] = ACTIONS(4412),
    [anon_sym_use] = ACTIONS(4412),
    [anon_sym_while] = ACTIONS(4412),
    [anon_sym_extern] = ACTIONS(4412),
    [anon_sym_safe] = ACTIONS(4412),
    [anon_sym_yield] = ACTIONS(4412),
    [anon_sym_move] = ACTIONS(4412),
    [anon_sym_try] = ACTIONS(4412),
    [sym_integer_literal] = ACTIONS(4410),
    [aux_sym_string_literal_token1] = ACTIONS(4410),
    [sym_char_literal] = ACTIONS(4410),
    [anon_sym_true] = ACTIONS(4412),
    [anon_sym_false] = ACTIONS(4412),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4412),
    [sym_super] = ACTIONS(4412),
    [sym_crate] = ACTIONS(4412),
    [aux_sym_metavariable_token1] = ACTIONS(4410),
    [sym__raw_string_literal_start] = ACTIONS(4410),
    [sym_float_literal] = ACTIONS(4410),
    [sym__outer_block_doc_comment_start] = ACTIONS(4410),
    [sym__outer_line_doc_comment_start] = ACTIONS(4410),
  },
  [STATE(1164)] = {
    [sym_line_comment] = STATE(1164),
    [sym_block_comment] = STATE(1164),
    [ts_builtin_sym_end] = ACTIONS(4414),
    [sym_identifier] = ACTIONS(4416),
    [anon_sym_SEMI] = ACTIONS(4414),
    [anon_sym_macro] = ACTIONS(4416),
    [anon_sym_LPAREN] = ACTIONS(4414),
    [anon_sym_LBRACE] = ACTIONS(4414),
    [anon_sym_RBRACE] = ACTIONS(4414),
    [anon_sym_macro_rules] = ACTIONS(4416),
    [anon_sym_BANG] = ACTIONS(4414),
    [anon_sym_LBRACK] = ACTIONS(4414),
    [anon_sym_STAR] = ACTIONS(4414),
    [anon_sym_u8] = ACTIONS(4416),
    [anon_sym_i8] = ACTIONS(4416),
    [anon_sym_u16] = ACTIONS(4416),
    [anon_sym_i16] = ACTIONS(4416),
    [anon_sym_u32] = ACTIONS(4416),
    [anon_sym_i32] = ACTIONS(4416),
    [anon_sym_u64] = ACTIONS(4416),
    [anon_sym_i64] = ACTIONS(4416),
    [anon_sym_u128] = ACTIONS(4416),
    [anon_sym_i128] = ACTIONS(4416),
    [anon_sym_isize] = ACTIONS(4416),
    [anon_sym_usize] = ACTIONS(4416),
    [anon_sym_f32] = ACTIONS(4416),
    [anon_sym_f64] = ACTIONS(4416),
    [anon_sym_bool] = ACTIONS(4416),
    [anon_sym_str] = ACTIONS(4416),
    [anon_sym_char] = ACTIONS(4416),
    [anon_sym_DASH] = ACTIONS(4414),
    [anon_sym_AMP] = ACTIONS(4414),
    [anon_sym_PIPE] = ACTIONS(4414),
    [anon_sym_LT] = ACTIONS(4414),
    [anon_sym_DOT_DOT] = ACTIONS(4414),
    [anon_sym_COLON_COLON] = ACTIONS(4414),
    [anon_sym_POUND] = ACTIONS(4414),
    [anon_sym_SQUOTE] = ACTIONS(4416),
    [anon_sym_async] = ACTIONS(4416),
    [anon_sym_auto] = ACTIONS(4416),
    [anon_sym_await] = ACTIONS(4416),
    [anon_sym_become] = ACTIONS(4416),
    [anon_sym_break] = ACTIONS(4416),
    [anon_sym_const] = ACTIONS(4416),
    [anon_sym_continue] = ACTIONS(4416),
    [anon_sym_default] = ACTIONS(4416),
    [anon_sym_enum] = ACTIONS(4416),
    [anon_sym_fn] = ACTIONS(4416),
    [anon_sym_for] = ACTIONS(4416),
    [anon_sym_gen] = ACTIONS(4416),
    [anon_sym_if] = ACTIONS(4416),
    [anon_sym_impl] = ACTIONS(4416),
    [anon_sym_let] = ACTIONS(4416),
    [anon_sym_loop] = ACTIONS(4416),
    [anon_sym_match] = ACTIONS(4416),
    [anon_sym_mod] = ACTIONS(4416),
    [anon_sym_pub] = ACTIONS(4416),
    [anon_sym_return] = ACTIONS(4416),
    [anon_sym_raw] = ACTIONS(4416),
    [anon_sym_static] = ACTIONS(4416),
    [anon_sym_struct] = ACTIONS(4416),
    [anon_sym_trait] = ACTIONS(4416),
    [anon_sym_type] = ACTIONS(4416),
    [anon_sym_union] = ACTIONS(4416),
    [anon_sym_unsafe] = ACTIONS(4416),
    [anon_sym_use] = ACTIONS(4416),
    [anon_sym_while] = ACTIONS(4416),
    [anon_sym_extern] = ACTIONS(4416),
    [anon_sym_safe] = ACTIONS(4416),
    [anon_sym_yield] = ACTIONS(4416),
    [anon_sym_move] = ACTIONS(4416),
    [anon_sym_try] = ACTIONS(4416),
    [sym_integer_literal] = ACTIONS(4414),
    [aux_sym_string_literal_token1] = ACTIONS(4414),
    [sym_char_literal] = ACTIONS(4414),
    [anon_sym_true] = ACTIONS(4416),
    [anon_sym_false] = ACTIONS(4416),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4416),
    [sym_super] = ACTIONS(4416),
    [sym_crate] = ACTIONS(4416),
    [aux_sym_metavariable_token1] = ACTIONS(4414),
    [sym__raw_string_literal_start] = ACTIONS(4414),
    [sym_float_literal] = ACTIONS(4414),
    [sym__outer_block_doc_comment_start] = ACTIONS(4414),
    [sym__outer_line_doc_comment_start] = ACTIONS(4414),
  },
  [STATE(1165)] = {
    [sym_line_comment] = STATE(1165),
    [sym_block_comment] = STATE(1165),
    [ts_builtin_sym_end] = ACTIONS(4418),
    [sym_identifier] = ACTIONS(4420),
    [anon_sym_SEMI] = ACTIONS(4418),
    [anon_sym_macro] = ACTIONS(4420),
    [anon_sym_LPAREN] = ACTIONS(4418),
    [anon_sym_LBRACE] = ACTIONS(4418),
    [anon_sym_RBRACE] = ACTIONS(4418),
    [anon_sym_macro_rules] = ACTIONS(4420),
    [anon_sym_BANG] = ACTIONS(4418),
    [anon_sym_LBRACK] = ACTIONS(4418),
    [anon_sym_STAR] = ACTIONS(4418),
    [anon_sym_u8] = ACTIONS(4420),
    [anon_sym_i8] = ACTIONS(4420),
    [anon_sym_u16] = ACTIONS(4420),
    [anon_sym_i16] = ACTIONS(4420),
    [anon_sym_u32] = ACTIONS(4420),
    [anon_sym_i32] = ACTIONS(4420),
    [anon_sym_u64] = ACTIONS(4420),
    [anon_sym_i64] = ACTIONS(4420),
    [anon_sym_u128] = ACTIONS(4420),
    [anon_sym_i128] = ACTIONS(4420),
    [anon_sym_isize] = ACTIONS(4420),
    [anon_sym_usize] = ACTIONS(4420),
    [anon_sym_f32] = ACTIONS(4420),
    [anon_sym_f64] = ACTIONS(4420),
    [anon_sym_bool] = ACTIONS(4420),
    [anon_sym_str] = ACTIONS(4420),
    [anon_sym_char] = ACTIONS(4420),
    [anon_sym_DASH] = ACTIONS(4418),
    [anon_sym_AMP] = ACTIONS(4418),
    [anon_sym_PIPE] = ACTIONS(4418),
    [anon_sym_LT] = ACTIONS(4418),
    [anon_sym_DOT_DOT] = ACTIONS(4418),
    [anon_sym_COLON_COLON] = ACTIONS(4418),
    [anon_sym_POUND] = ACTIONS(4418),
    [anon_sym_SQUOTE] = ACTIONS(4420),
    [anon_sym_async] = ACTIONS(4420),
    [anon_sym_auto] = ACTIONS(4420),
    [anon_sym_await] = ACTIONS(4420),
    [anon_sym_become] = ACTIONS(4420),
    [anon_sym_break] = ACTIONS(4420),
    [anon_sym_const] = ACTIONS(4420),
    [anon_sym_continue] = ACTIONS(4420),
    [anon_sym_default] = ACTIONS(4420),
    [anon_sym_enum] = ACTIONS(4420),
    [anon_sym_fn] = ACTIONS(4420),
    [anon_sym_for] = ACTIONS(4420),
    [anon_sym_gen] = ACTIONS(4420),
    [anon_sym_if] = ACTIONS(4420),
    [anon_sym_impl] = ACTIONS(4420),
    [anon_sym_let] = ACTIONS(4420),
    [anon_sym_loop] = ACTIONS(4420),
    [anon_sym_match] = ACTIONS(4420),
    [anon_sym_mod] = ACTIONS(4420),
    [anon_sym_pub] = ACTIONS(4420),
    [anon_sym_return] = ACTIONS(4420),
    [anon_sym_raw] = ACTIONS(4420),
    [anon_sym_static] = ACTIONS(4420),
    [anon_sym_struct] = ACTIONS(4420),
    [anon_sym_trait] = ACTIONS(4420),
    [anon_sym_type] = ACTIONS(4420),
    [anon_sym_union] = ACTIONS(4420),
    [anon_sym_unsafe] = ACTIONS(4420),
    [anon_sym_use] = ACTIONS(4420),
    [anon_sym_while] = ACTIONS(4420),
    [anon_sym_extern] = ACTIONS(4420),
    [anon_sym_safe] = ACTIONS(4420),
    [anon_sym_yield] = ACTIONS(4420),
    [anon_sym_move] = ACTIONS(4420),
    [anon_sym_try] = ACTIONS(4420),
    [sym_integer_literal] = ACTIONS(4418),
    [aux_sym_string_literal_token1] = ACTIONS(4418),
    [sym_char_literal] = ACTIONS(4418),
    [anon_sym_true] = ACTIONS(4420),
    [anon_sym_false] = ACTIONS(4420),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4420),
    [sym_super] = ACTIONS(4420),
    [sym_crate] = ACTIONS(4420),
    [aux_sym_metavariable_token1] = ACTIONS(4418),
    [sym__raw_string_literal_start] = ACTIONS(4418),
    [sym_float_literal] = ACTIONS(4418),
    [sym__outer_block_doc_comment_start] = ACTIONS(4418),
    [sym__outer_line_doc_comment_start] = ACTIONS(4418),
  },
  [STATE(1166)] = {
    [sym_line_comment] = STATE(1166),
    [sym_block_comment] = STATE(1166),
    [ts_builtin_sym_end] = ACTIONS(4422),
    [sym_identifier] = ACTIONS(4424),
    [anon_sym_SEMI] = ACTIONS(4422),
    [anon_sym_macro] = ACTIONS(4424),
    [anon_sym_LPAREN] = ACTIONS(4422),
    [anon_sym_LBRACE] = ACTIONS(4422),
    [anon_sym_RBRACE] = ACTIONS(4422),
    [anon_sym_macro_rules] = ACTIONS(4424),
    [anon_sym_BANG] = ACTIONS(4422),
    [anon_sym_LBRACK] = ACTIONS(4422),
    [anon_sym_STAR] = ACTIONS(4422),
    [anon_sym_u8] = ACTIONS(4424),
    [anon_sym_i8] = ACTIONS(4424),
    [anon_sym_u16] = ACTIONS(4424),
    [anon_sym_i16] = ACTIONS(4424),
    [anon_sym_u32] = ACTIONS(4424),
    [anon_sym_i32] = ACTIONS(4424),
    [anon_sym_u64] = ACTIONS(4424),
    [anon_sym_i64] = ACTIONS(4424),
    [anon_sym_u128] = ACTIONS(4424),
    [anon_sym_i128] = ACTIONS(4424),
    [anon_sym_isize] = ACTIONS(4424),
    [anon_sym_usize] = ACTIONS(4424),
    [anon_sym_f32] = ACTIONS(4424),
    [anon_sym_f64] = ACTIONS(4424),
    [anon_sym_bool] = ACTIONS(4424),
    [anon_sym_str] = ACTIONS(4424),
    [anon_sym_char] = ACTIONS(4424),
    [anon_sym_DASH] = ACTIONS(4422),
    [anon_sym_AMP] = ACTIONS(4422),
    [anon_sym_PIPE] = ACTIONS(4422),
    [anon_sym_LT] = ACTIONS(4422),
    [anon_sym_DOT_DOT] = ACTIONS(4422),
    [anon_sym_COLON_COLON] = ACTIONS(4422),
    [anon_sym_POUND] = ACTIONS(4422),
    [anon_sym_SQUOTE] = ACTIONS(4424),
    [anon_sym_async] = ACTIONS(4424),
    [anon_sym_auto] = ACTIONS(4424),
    [anon_sym_await] = ACTIONS(4424),
    [anon_sym_become] = ACTIONS(4424),
    [anon_sym_break] = ACTIONS(4424),
    [anon_sym_const] = ACTIONS(4424),
    [anon_sym_continue] = ACTIONS(4424),
    [anon_sym_default] = ACTIONS(4424),
    [anon_sym_enum] = ACTIONS(4424),
    [anon_sym_fn] = ACTIONS(4424),
    [anon_sym_for] = ACTIONS(4424),
    [anon_sym_gen] = ACTIONS(4424),
    [anon_sym_if] = ACTIONS(4424),
    [anon_sym_impl] = ACTIONS(4424),
    [anon_sym_let] = ACTIONS(4424),
    [anon_sym_loop] = ACTIONS(4424),
    [anon_sym_match] = ACTIONS(4424),
    [anon_sym_mod] = ACTIONS(4424),
    [anon_sym_pub] = ACTIONS(4424),
    [anon_sym_return] = ACTIONS(4424),
    [anon_sym_raw] = ACTIONS(4424),
    [anon_sym_static] = ACTIONS(4424),
    [anon_sym_struct] = ACTIONS(4424),
    [anon_sym_trait] = ACTIONS(4424),
    [anon_sym_type] = ACTIONS(4424),
    [anon_sym_union] = ACTIONS(4424),
    [anon_sym_unsafe] = ACTIONS(4424),
    [anon_sym_use] = ACTIONS(4424),
    [anon_sym_while] = ACTIONS(4424),
    [anon_sym_extern] = ACTIONS(4424),
    [anon_sym_safe] = ACTIONS(4424),
    [anon_sym_yield] = ACTIONS(4424),
    [anon_sym_move] = ACTIONS(4424),
    [anon_sym_try] = ACTIONS(4424),
    [sym_integer_literal] = ACTIONS(4422),
    [aux_sym_string_literal_token1] = ACTIONS(4422),
    [sym_char_literal] = ACTIONS(4422),
    [anon_sym_true] = ACTIONS(4424),
    [anon_sym_false] = ACTIONS(4424),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4424),
    [sym_super] = ACTIONS(4424),
    [sym_crate] = ACTIONS(4424),
    [aux_sym_metavariable_token1] = ACTIONS(4422),
    [sym__raw_string_literal_start] = ACTIONS(4422),
    [sym_float_literal] = ACTIONS(4422),
    [sym__outer_block_doc_comment_start] = ACTIONS(4422),
    [sym__outer_line_doc_comment_start] = ACTIONS(4422),
  },
  [STATE(1167)] = {
    [sym_line_comment] = STATE(1167),
    [sym_block_comment] = STATE(1167),
    [ts_builtin_sym_end] = ACTIONS(1410),
    [sym_identifier] = ACTIONS(1412),
    [anon_sym_SEMI] = ACTIONS(1410),
    [anon_sym_macro] = ACTIONS(1412),
    [anon_sym_LPAREN] = ACTIONS(1410),
    [anon_sym_LBRACE] = ACTIONS(1410),
    [anon_sym_RBRACE] = ACTIONS(1410),
    [anon_sym_macro_rules] = ACTIONS(1412),
    [anon_sym_BANG] = ACTIONS(1410),
    [anon_sym_LBRACK] = ACTIONS(1410),
    [anon_sym_STAR] = ACTIONS(1410),
    [anon_sym_u8] = ACTIONS(1412),
    [anon_sym_i8] = ACTIONS(1412),
    [anon_sym_u16] = ACTIONS(1412),
    [anon_sym_i16] = ACTIONS(1412),
    [anon_sym_u32] = ACTIONS(1412),
    [anon_sym_i32] = ACTIONS(1412),
    [anon_sym_u64] = ACTIONS(1412),
    [anon_sym_i64] = ACTIONS(1412),
    [anon_sym_u128] = ACTIONS(1412),
    [anon_sym_i128] = ACTIONS(1412),
    [anon_sym_isize] = ACTIONS(1412),
    [anon_sym_usize] = ACTIONS(1412),
    [anon_sym_f32] = ACTIONS(1412),
    [anon_sym_f64] = ACTIONS(1412),
    [anon_sym_bool] = ACTIONS(1412),
    [anon_sym_str] = ACTIONS(1412),
    [anon_sym_char] = ACTIONS(1412),
    [anon_sym_DASH] = ACTIONS(1410),
    [anon_sym_AMP] = ACTIONS(1410),
    [anon_sym_PIPE] = ACTIONS(1410),
    [anon_sym_LT] = ACTIONS(1410),
    [anon_sym_DOT_DOT] = ACTIONS(1410),
    [anon_sym_COLON_COLON] = ACTIONS(1410),
    [anon_sym_POUND] = ACTIONS(1410),
    [anon_sym_SQUOTE] = ACTIONS(1412),
    [anon_sym_async] = ACTIONS(1412),
    [anon_sym_auto] = ACTIONS(1412),
    [anon_sym_await] = ACTIONS(1412),
    [anon_sym_become] = ACTIONS(1412),
    [anon_sym_break] = ACTIONS(1412),
    [anon_sym_const] = ACTIONS(1412),
    [anon_sym_continue] = ACTIONS(1412),
    [anon_sym_default] = ACTIONS(1412),
    [anon_sym_enum] = ACTIONS(1412),
    [anon_sym_fn] = ACTIONS(1412),
    [anon_sym_for] = ACTIONS(1412),
    [anon_sym_gen] = ACTIONS(1412),
    [anon_sym_if] = ACTIONS(1412),
    [anon_sym_impl] = ACTIONS(1412),
    [anon_sym_let] = ACTIONS(1412),
    [anon_sym_loop] = ACTIONS(1412),
    [anon_sym_match] = ACTIONS(1412),
    [anon_sym_mod] = ACTIONS(1412),
    [anon_sym_pub] = ACTIONS(1412),
    [anon_sym_return] = ACTIONS(1412),
    [anon_sym_raw] = ACTIONS(1412),
    [anon_sym_static] = ACTIONS(1412),
    [anon_sym_struct] = ACTIONS(1412),
    [anon_sym_trait] = ACTIONS(1412),
    [anon_sym_type] = ACTIONS(1412),
    [anon_sym_union] = ACTIONS(1412),
    [anon_sym_unsafe] = ACTIONS(1412),
    [anon_sym_use] = ACTIONS(1412),
    [anon_sym_while] = ACTIONS(1412),
    [anon_sym_extern] = ACTIONS(1412),
    [anon_sym_safe] = ACTIONS(1412),
    [anon_sym_yield] = ACTIONS(1412),
    [anon_sym_move] = ACTIONS(1412),
    [anon_sym_try] = ACTIONS(1412),
    [sym_integer_literal] = ACTIONS(1410),
    [aux_sym_string_literal_token1] = ACTIONS(1410),
    [sym_char_literal] = ACTIONS(1410),
    [anon_sym_true] = ACTIONS(1412),
    [anon_sym_false] = ACTIONS(1412),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1412),
    [sym_super] = ACTIONS(1412),
    [sym_crate] = ACTIONS(1412),
    [aux_sym_metavariable_token1] = ACTIONS(1410),
    [sym__raw_string_literal_start] = ACTIONS(1410),
    [sym_float_literal] = ACTIONS(1410),
    [sym__outer_block_doc_comment_start] = ACTIONS(1410),
    [sym__outer_line_doc_comment_start] = ACTIONS(1410),
  },
  [STATE(1168)] = {
    [sym_line_comment] = STATE(1168),
    [sym_block_comment] = STATE(1168),
    [ts_builtin_sym_end] = ACTIONS(4426),
    [sym_identifier] = ACTIONS(4428),
    [anon_sym_SEMI] = ACTIONS(4426),
    [anon_sym_macro] = ACTIONS(4428),
    [anon_sym_LPAREN] = ACTIONS(4426),
    [anon_sym_LBRACE] = ACTIONS(4426),
    [anon_sym_RBRACE] = ACTIONS(4426),
    [anon_sym_macro_rules] = ACTIONS(4428),
    [anon_sym_BANG] = ACTIONS(4426),
    [anon_sym_LBRACK] = ACTIONS(4426),
    [anon_sym_STAR] = ACTIONS(4426),
    [anon_sym_u8] = ACTIONS(4428),
    [anon_sym_i8] = ACTIONS(4428),
    [anon_sym_u16] = ACTIONS(4428),
    [anon_sym_i16] = ACTIONS(4428),
    [anon_sym_u32] = ACTIONS(4428),
    [anon_sym_i32] = ACTIONS(4428),
    [anon_sym_u64] = ACTIONS(4428),
    [anon_sym_i64] = ACTIONS(4428),
    [anon_sym_u128] = ACTIONS(4428),
    [anon_sym_i128] = ACTIONS(4428),
    [anon_sym_isize] = ACTIONS(4428),
    [anon_sym_usize] = ACTIONS(4428),
    [anon_sym_f32] = ACTIONS(4428),
    [anon_sym_f64] = ACTIONS(4428),
    [anon_sym_bool] = ACTIONS(4428),
    [anon_sym_str] = ACTIONS(4428),
    [anon_sym_char] = ACTIONS(4428),
    [anon_sym_DASH] = ACTIONS(4426),
    [anon_sym_AMP] = ACTIONS(4426),
    [anon_sym_PIPE] = ACTIONS(4426),
    [anon_sym_LT] = ACTIONS(4426),
    [anon_sym_DOT_DOT] = ACTIONS(4426),
    [anon_sym_COLON_COLON] = ACTIONS(4426),
    [anon_sym_POUND] = ACTIONS(4426),
    [anon_sym_SQUOTE] = ACTIONS(4428),
    [anon_sym_async] = ACTIONS(4428),
    [anon_sym_auto] = ACTIONS(4428),
    [anon_sym_await] = ACTIONS(4428),
    [anon_sym_become] = ACTIONS(4428),
    [anon_sym_break] = ACTIONS(4428),
    [anon_sym_const] = ACTIONS(4428),
    [anon_sym_continue] = ACTIONS(4428),
    [anon_sym_default] = ACTIONS(4428),
    [anon_sym_enum] = ACTIONS(4428),
    [anon_sym_fn] = ACTIONS(4428),
    [anon_sym_for] = ACTIONS(4428),
    [anon_sym_gen] = ACTIONS(4428),
    [anon_sym_if] = ACTIONS(4428),
    [anon_sym_impl] = ACTIONS(4428),
    [anon_sym_let] = ACTIONS(4428),
    [anon_sym_loop] = ACTIONS(4428),
    [anon_sym_match] = ACTIONS(4428),
    [anon_sym_mod] = ACTIONS(4428),
    [anon_sym_pub] = ACTIONS(4428),
    [anon_sym_return] = ACTIONS(4428),
    [anon_sym_raw] = ACTIONS(4428),
    [anon_sym_static] = ACTIONS(4428),
    [anon_sym_struct] = ACTIONS(4428),
    [anon_sym_trait] = ACTIONS(4428),
    [anon_sym_type] = ACTIONS(4428),
    [anon_sym_union] = ACTIONS(4428),
    [anon_sym_unsafe] = ACTIONS(4428),
    [anon_sym_use] = ACTIONS(4428),
    [anon_sym_while] = ACTIONS(4428),
    [anon_sym_extern] = ACTIONS(4428),
    [anon_sym_safe] = ACTIONS(4428),
    [anon_sym_yield] = ACTIONS(4428),
    [anon_sym_move] = ACTIONS(4428),
    [anon_sym_try] = ACTIONS(4428),
    [sym_integer_literal] = ACTIONS(4426),
    [aux_sym_string_literal_token1] = ACTIONS(4426),
    [sym_char_literal] = ACTIONS(4426),
    [anon_sym_true] = ACTIONS(4428),
    [anon_sym_false] = ACTIONS(4428),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4428),
    [sym_super] = ACTIONS(4428),
    [sym_crate] = ACTIONS(4428),
    [aux_sym_metavariable_token1] = ACTIONS(4426),
    [sym__raw_string_literal_start] = ACTIONS(4426),
    [sym_float_literal] = ACTIONS(4426),
    [sym__outer_block_doc_comment_start] = ACTIONS(4426),
    [sym__outer_line_doc_comment_start] = ACTIONS(4426),
  },
  [STATE(1169)] = {
    [sym_line_comment] = STATE(1169),
    [sym_block_comment] = STATE(1169),
    [ts_builtin_sym_end] = ACTIONS(1610),
    [sym_identifier] = ACTIONS(1612),
    [anon_sym_SEMI] = ACTIONS(1610),
    [anon_sym_macro] = ACTIONS(1612),
    [anon_sym_LPAREN] = ACTIONS(1610),
    [anon_sym_LBRACE] = ACTIONS(1610),
    [anon_sym_RBRACE] = ACTIONS(1610),
    [anon_sym_macro_rules] = ACTIONS(1612),
    [anon_sym_BANG] = ACTIONS(1610),
    [anon_sym_LBRACK] = ACTIONS(1610),
    [anon_sym_STAR] = ACTIONS(1610),
    [anon_sym_u8] = ACTIONS(1612),
    [anon_sym_i8] = ACTIONS(1612),
    [anon_sym_u16] = ACTIONS(1612),
    [anon_sym_i16] = ACTIONS(1612),
    [anon_sym_u32] = ACTIONS(1612),
    [anon_sym_i32] = ACTIONS(1612),
    [anon_sym_u64] = ACTIONS(1612),
    [anon_sym_i64] = ACTIONS(1612),
    [anon_sym_u128] = ACTIONS(1612),
    [anon_sym_i128] = ACTIONS(1612),
    [anon_sym_isize] = ACTIONS(1612),
    [anon_sym_usize] = ACTIONS(1612),
    [anon_sym_f32] = ACTIONS(1612),
    [anon_sym_f64] = ACTIONS(1612),
    [anon_sym_bool] = ACTIONS(1612),
    [anon_sym_str] = ACTIONS(1612),
    [anon_sym_char] = ACTIONS(1612),
    [anon_sym_DASH] = ACTIONS(1610),
    [anon_sym_AMP] = ACTIONS(1610),
    [anon_sym_PIPE] = ACTIONS(1610),
    [anon_sym_LT] = ACTIONS(1610),
    [anon_sym_DOT_DOT] = ACTIONS(1610),
    [anon_sym_COLON_COLON] = ACTIONS(1610),
    [anon_sym_POUND] = ACTIONS(1610),
    [anon_sym_SQUOTE] = ACTIONS(1612),
    [anon_sym_async] = ACTIONS(1612),
    [anon_sym_auto] = ACTIONS(1612),
    [anon_sym_await] = ACTIONS(1612),
    [anon_sym_become] = ACTIONS(1612),
    [anon_sym_break] = ACTIONS(1612),
    [anon_sym_const] = ACTIONS(1612),
    [anon_sym_continue] = ACTIONS(1612),
    [anon_sym_default] = ACTIONS(1612),
    [anon_sym_enum] = ACTIONS(1612),
    [anon_sym_fn] = ACTIONS(1612),
    [anon_sym_for] = ACTIONS(1612),
    [anon_sym_gen] = ACTIONS(1612),
    [anon_sym_if] = ACTIONS(1612),
    [anon_sym_impl] = ACTIONS(1612),
    [anon_sym_let] = ACTIONS(1612),
    [anon_sym_loop] = ACTIONS(1612),
    [anon_sym_match] = ACTIONS(1612),
    [anon_sym_mod] = ACTIONS(1612),
    [anon_sym_pub] = ACTIONS(1612),
    [anon_sym_return] = ACTIONS(1612),
    [anon_sym_raw] = ACTIONS(1612),
    [anon_sym_static] = ACTIONS(1612),
    [anon_sym_struct] = ACTIONS(1612),
    [anon_sym_trait] = ACTIONS(1612),
    [anon_sym_type] = ACTIONS(1612),
    [anon_sym_union] = ACTIONS(1612),
    [anon_sym_unsafe] = ACTIONS(1612),
    [anon_sym_use] = ACTIONS(1612),
    [anon_sym_while] = ACTIONS(1612),
    [anon_sym_extern] = ACTIONS(1612),
    [anon_sym_safe] = ACTIONS(1612),
    [anon_sym_yield] = ACTIONS(1612),
    [anon_sym_move] = ACTIONS(1612),
    [anon_sym_try] = ACTIONS(1612),
    [sym_integer_literal] = ACTIONS(1610),
    [aux_sym_string_literal_token1] = ACTIONS(1610),
    [sym_char_literal] = ACTIONS(1610),
    [anon_sym_true] = ACTIONS(1612),
    [anon_sym_false] = ACTIONS(1612),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1612),
    [sym_super] = ACTIONS(1612),
    [sym_crate] = ACTIONS(1612),
    [aux_sym_metavariable_token1] = ACTIONS(1610),
    [sym__raw_string_literal_start] = ACTIONS(1610),
    [sym_float_literal] = ACTIONS(1610),
    [sym__outer_block_doc_comment_start] = ACTIONS(1610),
    [sym__outer_line_doc_comment_start] = ACTIONS(1610),
  },
  [STATE(1170)] = {
    [sym_line_comment] = STATE(1170),
    [sym_block_comment] = STATE(1170),
    [ts_builtin_sym_end] = ACTIONS(4430),
    [sym_identifier] = ACTIONS(4432),
    [anon_sym_SEMI] = ACTIONS(4430),
    [anon_sym_macro] = ACTIONS(4432),
    [anon_sym_LPAREN] = ACTIONS(4430),
    [anon_sym_LBRACE] = ACTIONS(4430),
    [anon_sym_RBRACE] = ACTIONS(4430),
    [anon_sym_macro_rules] = ACTIONS(4432),
    [anon_sym_BANG] = ACTIONS(4430),
    [anon_sym_LBRACK] = ACTIONS(4430),
    [anon_sym_STAR] = ACTIONS(4430),
    [anon_sym_u8] = ACTIONS(4432),
    [anon_sym_i8] = ACTIONS(4432),
    [anon_sym_u16] = ACTIONS(4432),
    [anon_sym_i16] = ACTIONS(4432),
    [anon_sym_u32] = ACTIONS(4432),
    [anon_sym_i32] = ACTIONS(4432),
    [anon_sym_u64] = ACTIONS(4432),
    [anon_sym_i64] = ACTIONS(4432),
    [anon_sym_u128] = ACTIONS(4432),
    [anon_sym_i128] = ACTIONS(4432),
    [anon_sym_isize] = ACTIONS(4432),
    [anon_sym_usize] = ACTIONS(4432),
    [anon_sym_f32] = ACTIONS(4432),
    [anon_sym_f64] = ACTIONS(4432),
    [anon_sym_bool] = ACTIONS(4432),
    [anon_sym_str] = ACTIONS(4432),
    [anon_sym_char] = ACTIONS(4432),
    [anon_sym_DASH] = ACTIONS(4430),
    [anon_sym_AMP] = ACTIONS(4430),
    [anon_sym_PIPE] = ACTIONS(4430),
    [anon_sym_LT] = ACTIONS(4430),
    [anon_sym_DOT_DOT] = ACTIONS(4430),
    [anon_sym_COLON_COLON] = ACTIONS(4430),
    [anon_sym_POUND] = ACTIONS(4430),
    [anon_sym_SQUOTE] = ACTIONS(4432),
    [anon_sym_async] = ACTIONS(4432),
    [anon_sym_auto] = ACTIONS(4432),
    [anon_sym_await] = ACTIONS(4432),
    [anon_sym_become] = ACTIONS(4432),
    [anon_sym_break] = ACTIONS(4432),
    [anon_sym_const] = ACTIONS(4432),
    [anon_sym_continue] = ACTIONS(4432),
    [anon_sym_default] = ACTIONS(4432),
    [anon_sym_enum] = ACTIONS(4432),
    [anon_sym_fn] = ACTIONS(4432),
    [anon_sym_for] = ACTIONS(4432),
    [anon_sym_gen] = ACTIONS(4432),
    [anon_sym_if] = ACTIONS(4432),
    [anon_sym_impl] = ACTIONS(4432),
    [anon_sym_let] = ACTIONS(4432),
    [anon_sym_loop] = ACTIONS(4432),
    [anon_sym_match] = ACTIONS(4432),
    [anon_sym_mod] = ACTIONS(4432),
    [anon_sym_pub] = ACTIONS(4432),
    [anon_sym_return] = ACTIONS(4432),
    [anon_sym_raw] = ACTIONS(4432),
    [anon_sym_static] = ACTIONS(4432),
    [anon_sym_struct] = ACTIONS(4432),
    [anon_sym_trait] = ACTIONS(4432),
    [anon_sym_type] = ACTIONS(4432),
    [anon_sym_union] = ACTIONS(4432),
    [anon_sym_unsafe] = ACTIONS(4432),
    [anon_sym_use] = ACTIONS(4432),
    [anon_sym_while] = ACTIONS(4432),
    [anon_sym_extern] = ACTIONS(4432),
    [anon_sym_safe] = ACTIONS(4432),
    [anon_sym_yield] = ACTIONS(4432),
    [anon_sym_move] = ACTIONS(4432),
    [anon_sym_try] = ACTIONS(4432),
    [sym_integer_literal] = ACTIONS(4430),
    [aux_sym_string_literal_token1] = ACTIONS(4430),
    [sym_char_literal] = ACTIONS(4430),
    [anon_sym_true] = ACTIONS(4432),
    [anon_sym_false] = ACTIONS(4432),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4432),
    [sym_super] = ACTIONS(4432),
    [sym_crate] = ACTIONS(4432),
    [aux_sym_metavariable_token1] = ACTIONS(4430),
    [sym__raw_string_literal_start] = ACTIONS(4430),
    [sym_float_literal] = ACTIONS(4430),
    [sym__outer_block_doc_comment_start] = ACTIONS(4430),
    [sym__outer_line_doc_comment_start] = ACTIONS(4430),
  },
  [STATE(1171)] = {
    [sym_line_comment] = STATE(1171),
    [sym_block_comment] = STATE(1171),
    [ts_builtin_sym_end] = ACTIONS(1458),
    [sym_identifier] = ACTIONS(1460),
    [anon_sym_SEMI] = ACTIONS(1458),
    [anon_sym_macro] = ACTIONS(1460),
    [anon_sym_LPAREN] = ACTIONS(1458),
    [anon_sym_LBRACE] = ACTIONS(1458),
    [anon_sym_RBRACE] = ACTIONS(1458),
    [anon_sym_macro_rules] = ACTIONS(1460),
    [anon_sym_BANG] = ACTIONS(1458),
    [anon_sym_LBRACK] = ACTIONS(1458),
    [anon_sym_STAR] = ACTIONS(1458),
    [anon_sym_u8] = ACTIONS(1460),
    [anon_sym_i8] = ACTIONS(1460),
    [anon_sym_u16] = ACTIONS(1460),
    [anon_sym_i16] = ACTIONS(1460),
    [anon_sym_u32] = ACTIONS(1460),
    [anon_sym_i32] = ACTIONS(1460),
    [anon_sym_u64] = ACTIONS(1460),
    [anon_sym_i64] = ACTIONS(1460),
    [anon_sym_u128] = ACTIONS(1460),
    [anon_sym_i128] = ACTIONS(1460),
    [anon_sym_isize] = ACTIONS(1460),
    [anon_sym_usize] = ACTIONS(1460),
    [anon_sym_f32] = ACTIONS(1460),
    [anon_sym_f64] = ACTIONS(1460),
    [anon_sym_bool] = ACTIONS(1460),
    [anon_sym_str] = ACTIONS(1460),
    [anon_sym_char] = ACTIONS(1460),
    [anon_sym_DASH] = ACTIONS(1458),
    [anon_sym_AMP] = ACTIONS(1458),
    [anon_sym_PIPE] = ACTIONS(1458),
    [anon_sym_LT] = ACTIONS(1458),
    [anon_sym_DOT_DOT] = ACTIONS(1458),
    [anon_sym_COLON_COLON] = ACTIONS(1458),
    [anon_sym_POUND] = ACTIONS(1458),
    [anon_sym_SQUOTE] = ACTIONS(1460),
    [anon_sym_async] = ACTIONS(1460),
    [anon_sym_auto] = ACTIONS(1460),
    [anon_sym_await] = ACTIONS(1460),
    [anon_sym_become] = ACTIONS(1460),
    [anon_sym_break] = ACTIONS(1460),
    [anon_sym_const] = ACTIONS(1460),
    [anon_sym_continue] = ACTIONS(1460),
    [anon_sym_default] = ACTIONS(1460),
    [anon_sym_enum] = ACTIONS(1460),
    [anon_sym_fn] = ACTIONS(1460),
    [anon_sym_for] = ACTIONS(1460),
    [anon_sym_gen] = ACTIONS(1460),
    [anon_sym_if] = ACTIONS(1460),
    [anon_sym_impl] = ACTIONS(1460),
    [anon_sym_let] = ACTIONS(1460),
    [anon_sym_loop] = ACTIONS(1460),
    [anon_sym_match] = ACTIONS(1460),
    [anon_sym_mod] = ACTIONS(1460),
    [anon_sym_pub] = ACTIONS(1460),
    [anon_sym_return] = ACTIONS(1460),
    [anon_sym_raw] = ACTIONS(1460),
    [anon_sym_static] = ACTIONS(1460),
    [anon_sym_struct] = ACTIONS(1460),
    [anon_sym_trait] = ACTIONS(1460),
    [anon_sym_type] = ACTIONS(1460),
    [anon_sym_union] = ACTIONS(1460),
    [anon_sym_unsafe] = ACTIONS(1460),
    [anon_sym_use] = ACTIONS(1460),
    [anon_sym_while] = ACTIONS(1460),
    [anon_sym_extern] = ACTIONS(1460),
    [anon_sym_safe] = ACTIONS(1460),
    [anon_sym_yield] = ACTIONS(1460),
    [anon_sym_move] = ACTIONS(1460),
    [anon_sym_try] = ACTIONS(1460),
    [sym_integer_literal] = ACTIONS(1458),
    [aux_sym_string_literal_token1] = ACTIONS(1458),
    [sym_char_literal] = ACTIONS(1458),
    [anon_sym_true] = ACTIONS(1460),
    [anon_sym_false] = ACTIONS(1460),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1460),
    [sym_super] = ACTIONS(1460),
    [sym_crate] = ACTIONS(1460),
    [aux_sym_metavariable_token1] = ACTIONS(1458),
    [sym__raw_string_literal_start] = ACTIONS(1458),
    [sym_float_literal] = ACTIONS(1458),
    [sym__outer_block_doc_comment_start] = ACTIONS(1458),
    [sym__outer_line_doc_comment_start] = ACTIONS(1458),
  },
  [STATE(1172)] = {
    [sym_line_comment] = STATE(1172),
    [sym_block_comment] = STATE(1172),
    [ts_builtin_sym_end] = ACTIONS(1398),
    [sym_identifier] = ACTIONS(1400),
    [anon_sym_SEMI] = ACTIONS(1398),
    [anon_sym_macro] = ACTIONS(1400),
    [anon_sym_LPAREN] = ACTIONS(1398),
    [anon_sym_LBRACE] = ACTIONS(1398),
    [anon_sym_RBRACE] = ACTIONS(1398),
    [anon_sym_macro_rules] = ACTIONS(1400),
    [anon_sym_BANG] = ACTIONS(1398),
    [anon_sym_LBRACK] = ACTIONS(1398),
    [anon_sym_STAR] = ACTIONS(1398),
    [anon_sym_u8] = ACTIONS(1400),
    [anon_sym_i8] = ACTIONS(1400),
    [anon_sym_u16] = ACTIONS(1400),
    [anon_sym_i16] = ACTIONS(1400),
    [anon_sym_u32] = ACTIONS(1400),
    [anon_sym_i32] = ACTIONS(1400),
    [anon_sym_u64] = ACTIONS(1400),
    [anon_sym_i64] = ACTIONS(1400),
    [anon_sym_u128] = ACTIONS(1400),
    [anon_sym_i128] = ACTIONS(1400),
    [anon_sym_isize] = ACTIONS(1400),
    [anon_sym_usize] = ACTIONS(1400),
    [anon_sym_f32] = ACTIONS(1400),
    [anon_sym_f64] = ACTIONS(1400),
    [anon_sym_bool] = ACTIONS(1400),
    [anon_sym_str] = ACTIONS(1400),
    [anon_sym_char] = ACTIONS(1400),
    [anon_sym_DASH] = ACTIONS(1398),
    [anon_sym_AMP] = ACTIONS(1398),
    [anon_sym_PIPE] = ACTIONS(1398),
    [anon_sym_LT] = ACTIONS(1398),
    [anon_sym_DOT_DOT] = ACTIONS(1398),
    [anon_sym_COLON_COLON] = ACTIONS(1398),
    [anon_sym_POUND] = ACTIONS(1398),
    [anon_sym_SQUOTE] = ACTIONS(1400),
    [anon_sym_async] = ACTIONS(1400),
    [anon_sym_auto] = ACTIONS(1400),
    [anon_sym_await] = ACTIONS(1400),
    [anon_sym_become] = ACTIONS(1400),
    [anon_sym_break] = ACTIONS(1400),
    [anon_sym_const] = ACTIONS(1400),
    [anon_sym_continue] = ACTIONS(1400),
    [anon_sym_default] = ACTIONS(1400),
    [anon_sym_enum] = ACTIONS(1400),
    [anon_sym_fn] = ACTIONS(1400),
    [anon_sym_for] = ACTIONS(1400),
    [anon_sym_gen] = ACTIONS(1400),
    [anon_sym_if] = ACTIONS(1400),
    [anon_sym_impl] = ACTIONS(1400),
    [anon_sym_let] = ACTIONS(1400),
    [anon_sym_loop] = ACTIONS(1400),
    [anon_sym_match] = ACTIONS(1400),
    [anon_sym_mod] = ACTIONS(1400),
    [anon_sym_pub] = ACTIONS(1400),
    [anon_sym_return] = ACTIONS(1400),
    [anon_sym_raw] = ACTIONS(1400),
    [anon_sym_static] = ACTIONS(1400),
    [anon_sym_struct] = ACTIONS(1400),
    [anon_sym_trait] = ACTIONS(1400),
    [anon_sym_type] = ACTIONS(1400),
    [anon_sym_union] = ACTIONS(1400),
    [anon_sym_unsafe] = ACTIONS(1400),
    [anon_sym_use] = ACTIONS(1400),
    [anon_sym_while] = ACTIONS(1400),
    [anon_sym_extern] = ACTIONS(1400),
    [anon_sym_safe] = ACTIONS(1400),
    [anon_sym_yield] = ACTIONS(1400),
    [anon_sym_move] = ACTIONS(1400),
    [anon_sym_try] = ACTIONS(1400),
    [sym_integer_literal] = ACTIONS(1398),
    [aux_sym_string_literal_token1] = ACTIONS(1398),
    [sym_char_literal] = ACTIONS(1398),
    [anon_sym_true] = ACTIONS(1400),
    [anon_sym_false] = ACTIONS(1400),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1400),
    [sym_super] = ACTIONS(1400),
    [sym_crate] = ACTIONS(1400),
    [aux_sym_metavariable_token1] = ACTIONS(1398),
    [sym__raw_string_literal_start] = ACTIONS(1398),
    [sym_float_literal] = ACTIONS(1398),
    [sym__outer_block_doc_comment_start] = ACTIONS(1398),
    [sym__outer_line_doc_comment_start] = ACTIONS(1398),
  },
  [STATE(1173)] = {
    [sym_line_comment] = STATE(1173),
    [sym_block_comment] = STATE(1173),
    [ts_builtin_sym_end] = ACTIONS(1482),
    [sym_identifier] = ACTIONS(1484),
    [anon_sym_SEMI] = ACTIONS(1482),
    [anon_sym_macro] = ACTIONS(1484),
    [anon_sym_LPAREN] = ACTIONS(1482),
    [anon_sym_LBRACE] = ACTIONS(1482),
    [anon_sym_RBRACE] = ACTIONS(1482),
    [anon_sym_macro_rules] = ACTIONS(1484),
    [anon_sym_BANG] = ACTIONS(1482),
    [anon_sym_LBRACK] = ACTIONS(1482),
    [anon_sym_STAR] = ACTIONS(1482),
    [anon_sym_u8] = ACTIONS(1484),
    [anon_sym_i8] = ACTIONS(1484),
    [anon_sym_u16] = ACTIONS(1484),
    [anon_sym_i16] = ACTIONS(1484),
    [anon_sym_u32] = ACTIONS(1484),
    [anon_sym_i32] = ACTIONS(1484),
    [anon_sym_u64] = ACTIONS(1484),
    [anon_sym_i64] = ACTIONS(1484),
    [anon_sym_u128] = ACTIONS(1484),
    [anon_sym_i128] = ACTIONS(1484),
    [anon_sym_isize] = ACTIONS(1484),
    [anon_sym_usize] = ACTIONS(1484),
    [anon_sym_f32] = ACTIONS(1484),
    [anon_sym_f64] = ACTIONS(1484),
    [anon_sym_bool] = ACTIONS(1484),
    [anon_sym_str] = ACTIONS(1484),
    [anon_sym_char] = ACTIONS(1484),
    [anon_sym_DASH] = ACTIONS(1482),
    [anon_sym_AMP] = ACTIONS(1482),
    [anon_sym_PIPE] = ACTIONS(1482),
    [anon_sym_LT] = ACTIONS(1482),
    [anon_sym_DOT_DOT] = ACTIONS(1482),
    [anon_sym_COLON_COLON] = ACTIONS(1482),
    [anon_sym_POUND] = ACTIONS(1482),
    [anon_sym_SQUOTE] = ACTIONS(1484),
    [anon_sym_async] = ACTIONS(1484),
    [anon_sym_auto] = ACTIONS(1484),
    [anon_sym_await] = ACTIONS(1484),
    [anon_sym_become] = ACTIONS(1484),
    [anon_sym_break] = ACTIONS(1484),
    [anon_sym_const] = ACTIONS(1484),
    [anon_sym_continue] = ACTIONS(1484),
    [anon_sym_default] = ACTIONS(1484),
    [anon_sym_enum] = ACTIONS(1484),
    [anon_sym_fn] = ACTIONS(1484),
    [anon_sym_for] = ACTIONS(1484),
    [anon_sym_gen] = ACTIONS(1484),
    [anon_sym_if] = ACTIONS(1484),
    [anon_sym_impl] = ACTIONS(1484),
    [anon_sym_let] = ACTIONS(1484),
    [anon_sym_loop] = ACTIONS(1484),
    [anon_sym_match] = ACTIONS(1484),
    [anon_sym_mod] = ACTIONS(1484),
    [anon_sym_pub] = ACTIONS(1484),
    [anon_sym_return] = ACTIONS(1484),
    [anon_sym_raw] = ACTIONS(1484),
    [anon_sym_static] = ACTIONS(1484),
    [anon_sym_struct] = ACTIONS(1484),
    [anon_sym_trait] = ACTIONS(1484),
    [anon_sym_type] = ACTIONS(1484),
    [anon_sym_union] = ACTIONS(1484),
    [anon_sym_unsafe] = ACTIONS(1484),
    [anon_sym_use] = ACTIONS(1484),
    [anon_sym_while] = ACTIONS(1484),
    [anon_sym_extern] = ACTIONS(1484),
    [anon_sym_safe] = ACTIONS(1484),
    [anon_sym_yield] = ACTIONS(1484),
    [anon_sym_move] = ACTIONS(1484),
    [anon_sym_try] = ACTIONS(1484),
    [sym_integer_literal] = ACTIONS(1482),
    [aux_sym_string_literal_token1] = ACTIONS(1482),
    [sym_char_literal] = ACTIONS(1482),
    [anon_sym_true] = ACTIONS(1484),
    [anon_sym_false] = ACTIONS(1484),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1484),
    [sym_super] = ACTIONS(1484),
    [sym_crate] = ACTIONS(1484),
    [aux_sym_metavariable_token1] = ACTIONS(1482),
    [sym__raw_string_literal_start] = ACTIONS(1482),
    [sym_float_literal] = ACTIONS(1482),
    [sym__outer_block_doc_comment_start] = ACTIONS(1482),
    [sym__outer_line_doc_comment_start] = ACTIONS(1482),
  },
  [STATE(1174)] = {
    [sym_line_comment] = STATE(1174),
    [sym_block_comment] = STATE(1174),
    [ts_builtin_sym_end] = ACTIONS(4434),
    [sym_identifier] = ACTIONS(4436),
    [anon_sym_SEMI] = ACTIONS(4434),
    [anon_sym_macro] = ACTIONS(4436),
    [anon_sym_LPAREN] = ACTIONS(4434),
    [anon_sym_LBRACE] = ACTIONS(4434),
    [anon_sym_RBRACE] = ACTIONS(4434),
    [anon_sym_macro_rules] = ACTIONS(4436),
    [anon_sym_BANG] = ACTIONS(4434),
    [anon_sym_LBRACK] = ACTIONS(4434),
    [anon_sym_STAR] = ACTIONS(4434),
    [anon_sym_u8] = ACTIONS(4436),
    [anon_sym_i8] = ACTIONS(4436),
    [anon_sym_u16] = ACTIONS(4436),
    [anon_sym_i16] = ACTIONS(4436),
    [anon_sym_u32] = ACTIONS(4436),
    [anon_sym_i32] = ACTIONS(4436),
    [anon_sym_u64] = ACTIONS(4436),
    [anon_sym_i64] = ACTIONS(4436),
    [anon_sym_u128] = ACTIONS(4436),
    [anon_sym_i128] = ACTIONS(4436),
    [anon_sym_isize] = ACTIONS(4436),
    [anon_sym_usize] = ACTIONS(4436),
    [anon_sym_f32] = ACTIONS(4436),
    [anon_sym_f64] = ACTIONS(4436),
    [anon_sym_bool] = ACTIONS(4436),
    [anon_sym_str] = ACTIONS(4436),
    [anon_sym_char] = ACTIONS(4436),
    [anon_sym_DASH] = ACTIONS(4434),
    [anon_sym_AMP] = ACTIONS(4434),
    [anon_sym_PIPE] = ACTIONS(4434),
    [anon_sym_LT] = ACTIONS(4434),
    [anon_sym_DOT_DOT] = ACTIONS(4434),
    [anon_sym_COLON_COLON] = ACTIONS(4434),
    [anon_sym_POUND] = ACTIONS(4434),
    [anon_sym_SQUOTE] = ACTIONS(4436),
    [anon_sym_async] = ACTIONS(4436),
    [anon_sym_auto] = ACTIONS(4436),
    [anon_sym_await] = ACTIONS(4436),
    [anon_sym_become] = ACTIONS(4436),
    [anon_sym_break] = ACTIONS(4436),
    [anon_sym_const] = ACTIONS(4436),
    [anon_sym_continue] = ACTIONS(4436),
    [anon_sym_default] = ACTIONS(4436),
    [anon_sym_enum] = ACTIONS(4436),
    [anon_sym_fn] = ACTIONS(4436),
    [anon_sym_for] = ACTIONS(4436),
    [anon_sym_gen] = ACTIONS(4436),
    [anon_sym_if] = ACTIONS(4436),
    [anon_sym_impl] = ACTIONS(4436),
    [anon_sym_let] = ACTIONS(4436),
    [anon_sym_loop] = ACTIONS(4436),
    [anon_sym_match] = ACTIONS(4436),
    [anon_sym_mod] = ACTIONS(4436),
    [anon_sym_pub] = ACTIONS(4436),
    [anon_sym_return] = ACTIONS(4436),
    [anon_sym_raw] = ACTIONS(4436),
    [anon_sym_static] = ACTIONS(4436),
    [anon_sym_struct] = ACTIONS(4436),
    [anon_sym_trait] = ACTIONS(4436),
    [anon_sym_type] = ACTIONS(4436),
    [anon_sym_union] = ACTIONS(4436),
    [anon_sym_unsafe] = ACTIONS(4436),
    [anon_sym_use] = ACTIONS(4436),
    [anon_sym_while] = ACTIONS(4436),
    [anon_sym_extern] = ACTIONS(4436),
    [anon_sym_safe] = ACTIONS(4436),
    [anon_sym_yield] = ACTIONS(4436),
    [anon_sym_move] = ACTIONS(4436),
    [anon_sym_try] = ACTIONS(4436),
    [sym_integer_literal] = ACTIONS(4434),
    [aux_sym_string_literal_token1] = ACTIONS(4434),
    [sym_char_literal] = ACTIONS(4434),
    [anon_sym_true] = ACTIONS(4436),
    [anon_sym_false] = ACTIONS(4436),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4436),
    [sym_super] = ACTIONS(4436),
    [sym_crate] = ACTIONS(4436),
    [aux_sym_metavariable_token1] = ACTIONS(4434),
    [sym__raw_string_literal_start] = ACTIONS(4434),
    [sym_float_literal] = ACTIONS(4434),
    [sym__outer_block_doc_comment_start] = ACTIONS(4434),
    [sym__outer_line_doc_comment_start] = ACTIONS(4434),
  },
  [STATE(1175)] = {
    [sym_line_comment] = STATE(1175),
    [sym_block_comment] = STATE(1175),
    [ts_builtin_sym_end] = ACTIONS(4438),
    [sym_identifier] = ACTIONS(4440),
    [anon_sym_SEMI] = ACTIONS(4438),
    [anon_sym_macro] = ACTIONS(4440),
    [anon_sym_LPAREN] = ACTIONS(4438),
    [anon_sym_LBRACE] = ACTIONS(4438),
    [anon_sym_RBRACE] = ACTIONS(4438),
    [anon_sym_macro_rules] = ACTIONS(4440),
    [anon_sym_BANG] = ACTIONS(4438),
    [anon_sym_LBRACK] = ACTIONS(4438),
    [anon_sym_STAR] = ACTIONS(4438),
    [anon_sym_u8] = ACTIONS(4440),
    [anon_sym_i8] = ACTIONS(4440),
    [anon_sym_u16] = ACTIONS(4440),
    [anon_sym_i16] = ACTIONS(4440),
    [anon_sym_u32] = ACTIONS(4440),
    [anon_sym_i32] = ACTIONS(4440),
    [anon_sym_u64] = ACTIONS(4440),
    [anon_sym_i64] = ACTIONS(4440),
    [anon_sym_u128] = ACTIONS(4440),
    [anon_sym_i128] = ACTIONS(4440),
    [anon_sym_isize] = ACTIONS(4440),
    [anon_sym_usize] = ACTIONS(4440),
    [anon_sym_f32] = ACTIONS(4440),
    [anon_sym_f64] = ACTIONS(4440),
    [anon_sym_bool] = ACTIONS(4440),
    [anon_sym_str] = ACTIONS(4440),
    [anon_sym_char] = ACTIONS(4440),
    [anon_sym_DASH] = ACTIONS(4438),
    [anon_sym_AMP] = ACTIONS(4438),
    [anon_sym_PIPE] = ACTIONS(4438),
    [anon_sym_LT] = ACTIONS(4438),
    [anon_sym_DOT_DOT] = ACTIONS(4438),
    [anon_sym_COLON_COLON] = ACTIONS(4438),
    [anon_sym_POUND] = ACTIONS(4438),
    [anon_sym_SQUOTE] = ACTIONS(4440),
    [anon_sym_async] = ACTIONS(4440),
    [anon_sym_auto] = ACTIONS(4440),
    [anon_sym_await] = ACTIONS(4440),
    [anon_sym_become] = ACTIONS(4440),
    [anon_sym_break] = ACTIONS(4440),
    [anon_sym_const] = ACTIONS(4440),
    [anon_sym_continue] = ACTIONS(4440),
    [anon_sym_default] = ACTIONS(4440),
    [anon_sym_enum] = ACTIONS(4440),
    [anon_sym_fn] = ACTIONS(4440),
    [anon_sym_for] = ACTIONS(4440),
    [anon_sym_gen] = ACTIONS(4440),
    [anon_sym_if] = ACTIONS(4440),
    [anon_sym_impl] = ACTIONS(4440),
    [anon_sym_let] = ACTIONS(4440),
    [anon_sym_loop] = ACTIONS(4440),
    [anon_sym_match] = ACTIONS(4440),
    [anon_sym_mod] = ACTIONS(4440),
    [anon_sym_pub] = ACTIONS(4440),
    [anon_sym_return] = ACTIONS(4440),
    [anon_sym_raw] = ACTIONS(4440),
    [anon_sym_static] = ACTIONS(4440),
    [anon_sym_struct] = ACTIONS(4440),
    [anon_sym_trait] = ACTIONS(4440),
    [anon_sym_type] = ACTIONS(4440),
    [anon_sym_union] = ACTIONS(4440),
    [anon_sym_unsafe] = ACTIONS(4440),
    [anon_sym_use] = ACTIONS(4440),
    [anon_sym_while] = ACTIONS(4440),
    [anon_sym_extern] = ACTIONS(4440),
    [anon_sym_safe] = ACTIONS(4440),
    [anon_sym_yield] = ACTIONS(4440),
    [anon_sym_move] = ACTIONS(4440),
    [anon_sym_try] = ACTIONS(4440),
    [sym_integer_literal] = ACTIONS(4438),
    [aux_sym_string_literal_token1] = ACTIONS(4438),
    [sym_char_literal] = ACTIONS(4438),
    [anon_sym_true] = ACTIONS(4440),
    [anon_sym_false] = ACTIONS(4440),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4440),
    [sym_super] = ACTIONS(4440),
    [sym_crate] = ACTIONS(4440),
    [aux_sym_metavariable_token1] = ACTIONS(4438),
    [sym__raw_string_literal_start] = ACTIONS(4438),
    [sym_float_literal] = ACTIONS(4438),
    [sym__outer_block_doc_comment_start] = ACTIONS(4438),
    [sym__outer_line_doc_comment_start] = ACTIONS(4438),
  },
  [STATE(1176)] = {
    [sym_line_comment] = STATE(1176),
    [sym_block_comment] = STATE(1176),
    [ts_builtin_sym_end] = ACTIONS(4442),
    [sym_identifier] = ACTIONS(4444),
    [anon_sym_SEMI] = ACTIONS(4442),
    [anon_sym_macro] = ACTIONS(4444),
    [anon_sym_LPAREN] = ACTIONS(4442),
    [anon_sym_LBRACE] = ACTIONS(4442),
    [anon_sym_RBRACE] = ACTIONS(4442),
    [anon_sym_macro_rules] = ACTIONS(4444),
    [anon_sym_BANG] = ACTIONS(4442),
    [anon_sym_LBRACK] = ACTIONS(4442),
    [anon_sym_STAR] = ACTIONS(4442),
    [anon_sym_u8] = ACTIONS(4444),
    [anon_sym_i8] = ACTIONS(4444),
    [anon_sym_u16] = ACTIONS(4444),
    [anon_sym_i16] = ACTIONS(4444),
    [anon_sym_u32] = ACTIONS(4444),
    [anon_sym_i32] = ACTIONS(4444),
    [anon_sym_u64] = ACTIONS(4444),
    [anon_sym_i64] = ACTIONS(4444),
    [anon_sym_u128] = ACTIONS(4444),
    [anon_sym_i128] = ACTIONS(4444),
    [anon_sym_isize] = ACTIONS(4444),
    [anon_sym_usize] = ACTIONS(4444),
    [anon_sym_f32] = ACTIONS(4444),
    [anon_sym_f64] = ACTIONS(4444),
    [anon_sym_bool] = ACTIONS(4444),
    [anon_sym_str] = ACTIONS(4444),
    [anon_sym_char] = ACTIONS(4444),
    [anon_sym_DASH] = ACTIONS(4442),
    [anon_sym_AMP] = ACTIONS(4442),
    [anon_sym_PIPE] = ACTIONS(4442),
    [anon_sym_LT] = ACTIONS(4442),
    [anon_sym_DOT_DOT] = ACTIONS(4442),
    [anon_sym_COLON_COLON] = ACTIONS(4442),
    [anon_sym_POUND] = ACTIONS(4442),
    [anon_sym_SQUOTE] = ACTIONS(4444),
    [anon_sym_async] = ACTIONS(4444),
    [anon_sym_auto] = ACTIONS(4444),
    [anon_sym_await] = ACTIONS(4444),
    [anon_sym_become] = ACTIONS(4444),
    [anon_sym_break] = ACTIONS(4444),
    [anon_sym_const] = ACTIONS(4444),
    [anon_sym_continue] = ACTIONS(4444),
    [anon_sym_default] = ACTIONS(4444),
    [anon_sym_enum] = ACTIONS(4444),
    [anon_sym_fn] = ACTIONS(4444),
    [anon_sym_for] = ACTIONS(4444),
    [anon_sym_gen] = ACTIONS(4444),
    [anon_sym_if] = ACTIONS(4444),
    [anon_sym_impl] = ACTIONS(4444),
    [anon_sym_let] = ACTIONS(4444),
    [anon_sym_loop] = ACTIONS(4444),
    [anon_sym_match] = ACTIONS(4444),
    [anon_sym_mod] = ACTIONS(4444),
    [anon_sym_pub] = ACTIONS(4444),
    [anon_sym_return] = ACTIONS(4444),
    [anon_sym_raw] = ACTIONS(4444),
    [anon_sym_static] = ACTIONS(4444),
    [anon_sym_struct] = ACTIONS(4444),
    [anon_sym_trait] = ACTIONS(4444),
    [anon_sym_type] = ACTIONS(4444),
    [anon_sym_union] = ACTIONS(4444),
    [anon_sym_unsafe] = ACTIONS(4444),
    [anon_sym_use] = ACTIONS(4444),
    [anon_sym_while] = ACTIONS(4444),
    [anon_sym_extern] = ACTIONS(4444),
    [anon_sym_safe] = ACTIONS(4444),
    [anon_sym_yield] = ACTIONS(4444),
    [anon_sym_move] = ACTIONS(4444),
    [anon_sym_try] = ACTIONS(4444),
    [sym_integer_literal] = ACTIONS(4442),
    [aux_sym_string_literal_token1] = ACTIONS(4442),
    [sym_char_literal] = ACTIONS(4442),
    [anon_sym_true] = ACTIONS(4444),
    [anon_sym_false] = ACTIONS(4444),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4444),
    [sym_super] = ACTIONS(4444),
    [sym_crate] = ACTIONS(4444),
    [aux_sym_metavariable_token1] = ACTIONS(4442),
    [sym__raw_string_literal_start] = ACTIONS(4442),
    [sym_float_literal] = ACTIONS(4442),
    [sym__outer_block_doc_comment_start] = ACTIONS(4442),
    [sym__outer_line_doc_comment_start] = ACTIONS(4442),
  },
  [STATE(1177)] = {
    [sym_line_comment] = STATE(1177),
    [sym_block_comment] = STATE(1177),
    [ts_builtin_sym_end] = ACTIONS(4446),
    [sym_identifier] = ACTIONS(4448),
    [anon_sym_SEMI] = ACTIONS(4446),
    [anon_sym_macro] = ACTIONS(4448),
    [anon_sym_LPAREN] = ACTIONS(4446),
    [anon_sym_LBRACE] = ACTIONS(4446),
    [anon_sym_RBRACE] = ACTIONS(4446),
    [anon_sym_macro_rules] = ACTIONS(4448),
    [anon_sym_BANG] = ACTIONS(4446),
    [anon_sym_LBRACK] = ACTIONS(4446),
    [anon_sym_STAR] = ACTIONS(4446),
    [anon_sym_u8] = ACTIONS(4448),
    [anon_sym_i8] = ACTIONS(4448),
    [anon_sym_u16] = ACTIONS(4448),
    [anon_sym_i16] = ACTIONS(4448),
    [anon_sym_u32] = ACTIONS(4448),
    [anon_sym_i32] = ACTIONS(4448),
    [anon_sym_u64] = ACTIONS(4448),
    [anon_sym_i64] = ACTIONS(4448),
    [anon_sym_u128] = ACTIONS(4448),
    [anon_sym_i128] = ACTIONS(4448),
    [anon_sym_isize] = ACTIONS(4448),
    [anon_sym_usize] = ACTIONS(4448),
    [anon_sym_f32] = ACTIONS(4448),
    [anon_sym_f64] = ACTIONS(4448),
    [anon_sym_bool] = ACTIONS(4448),
    [anon_sym_str] = ACTIONS(4448),
    [anon_sym_char] = ACTIONS(4448),
    [anon_sym_DASH] = ACTIONS(4446),
    [anon_sym_AMP] = ACTIONS(4446),
    [anon_sym_PIPE] = ACTIONS(4446),
    [anon_sym_LT] = ACTIONS(4446),
    [anon_sym_DOT_DOT] = ACTIONS(4446),
    [anon_sym_COLON_COLON] = ACTIONS(4446),
    [anon_sym_POUND] = ACTIONS(4446),
    [anon_sym_SQUOTE] = ACTIONS(4448),
    [anon_sym_async] = ACTIONS(4448),
    [anon_sym_auto] = ACTIONS(4448),
    [anon_sym_await] = ACTIONS(4448),
    [anon_sym_become] = ACTIONS(4448),
    [anon_sym_break] = ACTIONS(4448),
    [anon_sym_const] = ACTIONS(4448),
    [anon_sym_continue] = ACTIONS(4448),
    [anon_sym_default] = ACTIONS(4448),
    [anon_sym_enum] = ACTIONS(4448),
    [anon_sym_fn] = ACTIONS(4448),
    [anon_sym_for] = ACTIONS(4448),
    [anon_sym_gen] = ACTIONS(4448),
    [anon_sym_if] = ACTIONS(4448),
    [anon_sym_impl] = ACTIONS(4448),
    [anon_sym_let] = ACTIONS(4448),
    [anon_sym_loop] = ACTIONS(4448),
    [anon_sym_match] = ACTIONS(4448),
    [anon_sym_mod] = ACTIONS(4448),
    [anon_sym_pub] = ACTIONS(4448),
    [anon_sym_return] = ACTIONS(4448),
    [anon_sym_raw] = ACTIONS(4448),
    [anon_sym_static] = ACTIONS(4448),
    [anon_sym_struct] = ACTIONS(4448),
    [anon_sym_trait] = ACTIONS(4448),
    [anon_sym_type] = ACTIONS(4448),
    [anon_sym_union] = ACTIONS(4448),
    [anon_sym_unsafe] = ACTIONS(4448),
    [anon_sym_use] = ACTIONS(4448),
    [anon_sym_while] = ACTIONS(4448),
    [anon_sym_extern] = ACTIONS(4448),
    [anon_sym_safe] = ACTIONS(4448),
    [anon_sym_yield] = ACTIONS(4448),
    [anon_sym_move] = ACTIONS(4448),
    [anon_sym_try] = ACTIONS(4448),
    [sym_integer_literal] = ACTIONS(4446),
    [aux_sym_string_literal_token1] = ACTIONS(4446),
    [sym_char_literal] = ACTIONS(4446),
    [anon_sym_true] = ACTIONS(4448),
    [anon_sym_false] = ACTIONS(4448),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4448),
    [sym_super] = ACTIONS(4448),
    [sym_crate] = ACTIONS(4448),
    [aux_sym_metavariable_token1] = ACTIONS(4446),
    [sym__raw_string_literal_start] = ACTIONS(4446),
    [sym_float_literal] = ACTIONS(4446),
    [sym__outer_block_doc_comment_start] = ACTIONS(4446),
    [sym__outer_line_doc_comment_start] = ACTIONS(4446),
  },
  [STATE(1178)] = {
    [sym_line_comment] = STATE(1178),
    [sym_block_comment] = STATE(1178),
    [ts_builtin_sym_end] = ACTIONS(4450),
    [sym_identifier] = ACTIONS(4452),
    [anon_sym_SEMI] = ACTIONS(4450),
    [anon_sym_macro] = ACTIONS(4452),
    [anon_sym_LPAREN] = ACTIONS(4450),
    [anon_sym_LBRACE] = ACTIONS(4450),
    [anon_sym_RBRACE] = ACTIONS(4450),
    [anon_sym_macro_rules] = ACTIONS(4452),
    [anon_sym_BANG] = ACTIONS(4450),
    [anon_sym_LBRACK] = ACTIONS(4450),
    [anon_sym_STAR] = ACTIONS(4450),
    [anon_sym_u8] = ACTIONS(4452),
    [anon_sym_i8] = ACTIONS(4452),
    [anon_sym_u16] = ACTIONS(4452),
    [anon_sym_i16] = ACTIONS(4452),
    [anon_sym_u32] = ACTIONS(4452),
    [anon_sym_i32] = ACTIONS(4452),
    [anon_sym_u64] = ACTIONS(4452),
    [anon_sym_i64] = ACTIONS(4452),
    [anon_sym_u128] = ACTIONS(4452),
    [anon_sym_i128] = ACTIONS(4452),
    [anon_sym_isize] = ACTIONS(4452),
    [anon_sym_usize] = ACTIONS(4452),
    [anon_sym_f32] = ACTIONS(4452),
    [anon_sym_f64] = ACTIONS(4452),
    [anon_sym_bool] = ACTIONS(4452),
    [anon_sym_str] = ACTIONS(4452),
    [anon_sym_char] = ACTIONS(4452),
    [anon_sym_DASH] = ACTIONS(4450),
    [anon_sym_AMP] = ACTIONS(4450),
    [anon_sym_PIPE] = ACTIONS(4450),
    [anon_sym_LT] = ACTIONS(4450),
    [anon_sym_DOT_DOT] = ACTIONS(4450),
    [anon_sym_COLON_COLON] = ACTIONS(4450),
    [anon_sym_POUND] = ACTIONS(4450),
    [anon_sym_SQUOTE] = ACTIONS(4452),
    [anon_sym_async] = ACTIONS(4452),
    [anon_sym_auto] = ACTIONS(4452),
    [anon_sym_await] = ACTIONS(4452),
    [anon_sym_become] = ACTIONS(4452),
    [anon_sym_break] = ACTIONS(4452),
    [anon_sym_const] = ACTIONS(4452),
    [anon_sym_continue] = ACTIONS(4452),
    [anon_sym_default] = ACTIONS(4452),
    [anon_sym_enum] = ACTIONS(4452),
    [anon_sym_fn] = ACTIONS(4452),
    [anon_sym_for] = ACTIONS(4452),
    [anon_sym_gen] = ACTIONS(4452),
    [anon_sym_if] = ACTIONS(4452),
    [anon_sym_impl] = ACTIONS(4452),
    [anon_sym_let] = ACTIONS(4452),
    [anon_sym_loop] = ACTIONS(4452),
    [anon_sym_match] = ACTIONS(4452),
    [anon_sym_mod] = ACTIONS(4452),
    [anon_sym_pub] = ACTIONS(4452),
    [anon_sym_return] = ACTIONS(4452),
    [anon_sym_raw] = ACTIONS(4452),
    [anon_sym_static] = ACTIONS(4452),
    [anon_sym_struct] = ACTIONS(4452),
    [anon_sym_trait] = ACTIONS(4452),
    [anon_sym_type] = ACTIONS(4452),
    [anon_sym_union] = ACTIONS(4452),
    [anon_sym_unsafe] = ACTIONS(4452),
    [anon_sym_use] = ACTIONS(4452),
    [anon_sym_while] = ACTIONS(4452),
    [anon_sym_extern] = ACTIONS(4452),
    [anon_sym_safe] = ACTIONS(4452),
    [anon_sym_yield] = ACTIONS(4452),
    [anon_sym_move] = ACTIONS(4452),
    [anon_sym_try] = ACTIONS(4452),
    [sym_integer_literal] = ACTIONS(4450),
    [aux_sym_string_literal_token1] = ACTIONS(4450),
    [sym_char_literal] = ACTIONS(4450),
    [anon_sym_true] = ACTIONS(4452),
    [anon_sym_false] = ACTIONS(4452),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4452),
    [sym_super] = ACTIONS(4452),
    [sym_crate] = ACTIONS(4452),
    [aux_sym_metavariable_token1] = ACTIONS(4450),
    [sym__raw_string_literal_start] = ACTIONS(4450),
    [sym_float_literal] = ACTIONS(4450),
    [sym__outer_block_doc_comment_start] = ACTIONS(4450),
    [sym__outer_line_doc_comment_start] = ACTIONS(4450),
  },
  [STATE(1179)] = {
    [sym_line_comment] = STATE(1179),
    [sym_block_comment] = STATE(1179),
    [ts_builtin_sym_end] = ACTIONS(4454),
    [sym_identifier] = ACTIONS(4456),
    [anon_sym_SEMI] = ACTIONS(4454),
    [anon_sym_macro] = ACTIONS(4456),
    [anon_sym_LPAREN] = ACTIONS(4454),
    [anon_sym_LBRACE] = ACTIONS(4454),
    [anon_sym_RBRACE] = ACTIONS(4454),
    [anon_sym_macro_rules] = ACTIONS(4456),
    [anon_sym_BANG] = ACTIONS(4454),
    [anon_sym_LBRACK] = ACTIONS(4454),
    [anon_sym_STAR] = ACTIONS(4454),
    [anon_sym_u8] = ACTIONS(4456),
    [anon_sym_i8] = ACTIONS(4456),
    [anon_sym_u16] = ACTIONS(4456),
    [anon_sym_i16] = ACTIONS(4456),
    [anon_sym_u32] = ACTIONS(4456),
    [anon_sym_i32] = ACTIONS(4456),
    [anon_sym_u64] = ACTIONS(4456),
    [anon_sym_i64] = ACTIONS(4456),
    [anon_sym_u128] = ACTIONS(4456),
    [anon_sym_i128] = ACTIONS(4456),
    [anon_sym_isize] = ACTIONS(4456),
    [anon_sym_usize] = ACTIONS(4456),
    [anon_sym_f32] = ACTIONS(4456),
    [anon_sym_f64] = ACTIONS(4456),
    [anon_sym_bool] = ACTIONS(4456),
    [anon_sym_str] = ACTIONS(4456),
    [anon_sym_char] = ACTIONS(4456),
    [anon_sym_DASH] = ACTIONS(4454),
    [anon_sym_AMP] = ACTIONS(4454),
    [anon_sym_PIPE] = ACTIONS(4454),
    [anon_sym_LT] = ACTIONS(4454),
    [anon_sym_DOT_DOT] = ACTIONS(4454),
    [anon_sym_COLON_COLON] = ACTIONS(4454),
    [anon_sym_POUND] = ACTIONS(4454),
    [anon_sym_SQUOTE] = ACTIONS(4456),
    [anon_sym_async] = ACTIONS(4456),
    [anon_sym_auto] = ACTIONS(4456),
    [anon_sym_await] = ACTIONS(4456),
    [anon_sym_become] = ACTIONS(4456),
    [anon_sym_break] = ACTIONS(4456),
    [anon_sym_const] = ACTIONS(4456),
    [anon_sym_continue] = ACTIONS(4456),
    [anon_sym_default] = ACTIONS(4456),
    [anon_sym_enum] = ACTIONS(4456),
    [anon_sym_fn] = ACTIONS(4456),
    [anon_sym_for] = ACTIONS(4456),
    [anon_sym_gen] = ACTIONS(4456),
    [anon_sym_if] = ACTIONS(4456),
    [anon_sym_impl] = ACTIONS(4456),
    [anon_sym_let] = ACTIONS(4456),
    [anon_sym_loop] = ACTIONS(4456),
    [anon_sym_match] = ACTIONS(4456),
    [anon_sym_mod] = ACTIONS(4456),
    [anon_sym_pub] = ACTIONS(4456),
    [anon_sym_return] = ACTIONS(4456),
    [anon_sym_raw] = ACTIONS(4456),
    [anon_sym_static] = ACTIONS(4456),
    [anon_sym_struct] = ACTIONS(4456),
    [anon_sym_trait] = ACTIONS(4456),
    [anon_sym_type] = ACTIONS(4456),
    [anon_sym_union] = ACTIONS(4456),
    [anon_sym_unsafe] = ACTIONS(4456),
    [anon_sym_use] = ACTIONS(4456),
    [anon_sym_while] = ACTIONS(4456),
    [anon_sym_extern] = ACTIONS(4456),
    [anon_sym_safe] = ACTIONS(4456),
    [anon_sym_yield] = ACTIONS(4456),
    [anon_sym_move] = ACTIONS(4456),
    [anon_sym_try] = ACTIONS(4456),
    [sym_integer_literal] = ACTIONS(4454),
    [aux_sym_string_literal_token1] = ACTIONS(4454),
    [sym_char_literal] = ACTIONS(4454),
    [anon_sym_true] = ACTIONS(4456),
    [anon_sym_false] = ACTIONS(4456),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4456),
    [sym_super] = ACTIONS(4456),
    [sym_crate] = ACTIONS(4456),
    [aux_sym_metavariable_token1] = ACTIONS(4454),
    [sym__raw_string_literal_start] = ACTIONS(4454),
    [sym_float_literal] = ACTIONS(4454),
    [sym__outer_block_doc_comment_start] = ACTIONS(4454),
    [sym__outer_line_doc_comment_start] = ACTIONS(4454),
  },
  [STATE(1180)] = {
    [sym_line_comment] = STATE(1180),
    [sym_block_comment] = STATE(1180),
    [ts_builtin_sym_end] = ACTIONS(4458),
    [sym_identifier] = ACTIONS(4460),
    [anon_sym_SEMI] = ACTIONS(4458),
    [anon_sym_macro] = ACTIONS(4460),
    [anon_sym_LPAREN] = ACTIONS(4458),
    [anon_sym_LBRACE] = ACTIONS(4458),
    [anon_sym_RBRACE] = ACTIONS(4458),
    [anon_sym_macro_rules] = ACTIONS(4460),
    [anon_sym_BANG] = ACTIONS(4458),
    [anon_sym_LBRACK] = ACTIONS(4458),
    [anon_sym_STAR] = ACTIONS(4458),
    [anon_sym_u8] = ACTIONS(4460),
    [anon_sym_i8] = ACTIONS(4460),
    [anon_sym_u16] = ACTIONS(4460),
    [anon_sym_i16] = ACTIONS(4460),
    [anon_sym_u32] = ACTIONS(4460),
    [anon_sym_i32] = ACTIONS(4460),
    [anon_sym_u64] = ACTIONS(4460),
    [anon_sym_i64] = ACTIONS(4460),
    [anon_sym_u128] = ACTIONS(4460),
    [anon_sym_i128] = ACTIONS(4460),
    [anon_sym_isize] = ACTIONS(4460),
    [anon_sym_usize] = ACTIONS(4460),
    [anon_sym_f32] = ACTIONS(4460),
    [anon_sym_f64] = ACTIONS(4460),
    [anon_sym_bool] = ACTIONS(4460),
    [anon_sym_str] = ACTIONS(4460),
    [anon_sym_char] = ACTIONS(4460),
    [anon_sym_DASH] = ACTIONS(4458),
    [anon_sym_AMP] = ACTIONS(4458),
    [anon_sym_PIPE] = ACTIONS(4458),
    [anon_sym_LT] = ACTIONS(4458),
    [anon_sym_DOT_DOT] = ACTIONS(4458),
    [anon_sym_COLON_COLON] = ACTIONS(4458),
    [anon_sym_POUND] = ACTIONS(4458),
    [anon_sym_SQUOTE] = ACTIONS(4460),
    [anon_sym_async] = ACTIONS(4460),
    [anon_sym_auto] = ACTIONS(4460),
    [anon_sym_await] = ACTIONS(4460),
    [anon_sym_become] = ACTIONS(4460),
    [anon_sym_break] = ACTIONS(4460),
    [anon_sym_const] = ACTIONS(4460),
    [anon_sym_continue] = ACTIONS(4460),
    [anon_sym_default] = ACTIONS(4460),
    [anon_sym_enum] = ACTIONS(4460),
    [anon_sym_fn] = ACTIONS(4460),
    [anon_sym_for] = ACTIONS(4460),
    [anon_sym_gen] = ACTIONS(4460),
    [anon_sym_if] = ACTIONS(4460),
    [anon_sym_impl] = ACTIONS(4460),
    [anon_sym_let] = ACTIONS(4460),
    [anon_sym_loop] = ACTIONS(4460),
    [anon_sym_match] = ACTIONS(4460),
    [anon_sym_mod] = ACTIONS(4460),
    [anon_sym_pub] = ACTIONS(4460),
    [anon_sym_return] = ACTIONS(4460),
    [anon_sym_raw] = ACTIONS(4460),
    [anon_sym_static] = ACTIONS(4460),
    [anon_sym_struct] = ACTIONS(4460),
    [anon_sym_trait] = ACTIONS(4460),
    [anon_sym_type] = ACTIONS(4460),
    [anon_sym_union] = ACTIONS(4460),
    [anon_sym_unsafe] = ACTIONS(4460),
    [anon_sym_use] = ACTIONS(4460),
    [anon_sym_while] = ACTIONS(4460),
    [anon_sym_extern] = ACTIONS(4460),
    [anon_sym_safe] = ACTIONS(4460),
    [anon_sym_yield] = ACTIONS(4460),
    [anon_sym_move] = ACTIONS(4460),
    [anon_sym_try] = ACTIONS(4460),
    [sym_integer_literal] = ACTIONS(4458),
    [aux_sym_string_literal_token1] = ACTIONS(4458),
    [sym_char_literal] = ACTIONS(4458),
    [anon_sym_true] = ACTIONS(4460),
    [anon_sym_false] = ACTIONS(4460),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4460),
    [sym_super] = ACTIONS(4460),
    [sym_crate] = ACTIONS(4460),
    [aux_sym_metavariable_token1] = ACTIONS(4458),
    [sym__raw_string_literal_start] = ACTIONS(4458),
    [sym_float_literal] = ACTIONS(4458),
    [sym__outer_block_doc_comment_start] = ACTIONS(4458),
    [sym__outer_line_doc_comment_start] = ACTIONS(4458),
  },
  [STATE(1181)] = {
    [sym_line_comment] = STATE(1181),
    [sym_block_comment] = STATE(1181),
    [ts_builtin_sym_end] = ACTIONS(4462),
    [sym_identifier] = ACTIONS(4464),
    [anon_sym_SEMI] = ACTIONS(4462),
    [anon_sym_macro] = ACTIONS(4464),
    [anon_sym_LPAREN] = ACTIONS(4462),
    [anon_sym_LBRACE] = ACTIONS(4462),
    [anon_sym_RBRACE] = ACTIONS(4462),
    [anon_sym_macro_rules] = ACTIONS(4464),
    [anon_sym_BANG] = ACTIONS(4462),
    [anon_sym_LBRACK] = ACTIONS(4462),
    [anon_sym_STAR] = ACTIONS(4462),
    [anon_sym_u8] = ACTIONS(4464),
    [anon_sym_i8] = ACTIONS(4464),
    [anon_sym_u16] = ACTIONS(4464),
    [anon_sym_i16] = ACTIONS(4464),
    [anon_sym_u32] = ACTIONS(4464),
    [anon_sym_i32] = ACTIONS(4464),
    [anon_sym_u64] = ACTIONS(4464),
    [anon_sym_i64] = ACTIONS(4464),
    [anon_sym_u128] = ACTIONS(4464),
    [anon_sym_i128] = ACTIONS(4464),
    [anon_sym_isize] = ACTIONS(4464),
    [anon_sym_usize] = ACTIONS(4464),
    [anon_sym_f32] = ACTIONS(4464),
    [anon_sym_f64] = ACTIONS(4464),
    [anon_sym_bool] = ACTIONS(4464),
    [anon_sym_str] = ACTIONS(4464),
    [anon_sym_char] = ACTIONS(4464),
    [anon_sym_DASH] = ACTIONS(4462),
    [anon_sym_AMP] = ACTIONS(4462),
    [anon_sym_PIPE] = ACTIONS(4462),
    [anon_sym_LT] = ACTIONS(4462),
    [anon_sym_DOT_DOT] = ACTIONS(4462),
    [anon_sym_COLON_COLON] = ACTIONS(4462),
    [anon_sym_POUND] = ACTIONS(4462),
    [anon_sym_SQUOTE] = ACTIONS(4464),
    [anon_sym_async] = ACTIONS(4464),
    [anon_sym_auto] = ACTIONS(4464),
    [anon_sym_await] = ACTIONS(4464),
    [anon_sym_become] = ACTIONS(4464),
    [anon_sym_break] = ACTIONS(4464),
    [anon_sym_const] = ACTIONS(4464),
    [anon_sym_continue] = ACTIONS(4464),
    [anon_sym_default] = ACTIONS(4464),
    [anon_sym_enum] = ACTIONS(4464),
    [anon_sym_fn] = ACTIONS(4464),
    [anon_sym_for] = ACTIONS(4464),
    [anon_sym_gen] = ACTIONS(4464),
    [anon_sym_if] = ACTIONS(4464),
    [anon_sym_impl] = ACTIONS(4464),
    [anon_sym_let] = ACTIONS(4464),
    [anon_sym_loop] = ACTIONS(4464),
    [anon_sym_match] = ACTIONS(4464),
    [anon_sym_mod] = ACTIONS(4464),
    [anon_sym_pub] = ACTIONS(4464),
    [anon_sym_return] = ACTIONS(4464),
    [anon_sym_raw] = ACTIONS(4464),
    [anon_sym_static] = ACTIONS(4464),
    [anon_sym_struct] = ACTIONS(4464),
    [anon_sym_trait] = ACTIONS(4464),
    [anon_sym_type] = ACTIONS(4464),
    [anon_sym_union] = ACTIONS(4464),
    [anon_sym_unsafe] = ACTIONS(4464),
    [anon_sym_use] = ACTIONS(4464),
    [anon_sym_while] = ACTIONS(4464),
    [anon_sym_extern] = ACTIONS(4464),
    [anon_sym_safe] = ACTIONS(4464),
    [anon_sym_yield] = ACTIONS(4464),
    [anon_sym_move] = ACTIONS(4464),
    [anon_sym_try] = ACTIONS(4464),
    [sym_integer_literal] = ACTIONS(4462),
    [aux_sym_string_literal_token1] = ACTIONS(4462),
    [sym_char_literal] = ACTIONS(4462),
    [anon_sym_true] = ACTIONS(4464),
    [anon_sym_false] = ACTIONS(4464),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4464),
    [sym_super] = ACTIONS(4464),
    [sym_crate] = ACTIONS(4464),
    [aux_sym_metavariable_token1] = ACTIONS(4462),
    [sym__raw_string_literal_start] = ACTIONS(4462),
    [sym_float_literal] = ACTIONS(4462),
    [sym__outer_block_doc_comment_start] = ACTIONS(4462),
    [sym__outer_line_doc_comment_start] = ACTIONS(4462),
  },
  [STATE(1182)] = {
    [sym_line_comment] = STATE(1182),
    [sym_block_comment] = STATE(1182),
    [ts_builtin_sym_end] = ACTIONS(4466),
    [sym_identifier] = ACTIONS(4468),
    [anon_sym_SEMI] = ACTIONS(4466),
    [anon_sym_macro] = ACTIONS(4468),
    [anon_sym_LPAREN] = ACTIONS(4466),
    [anon_sym_LBRACE] = ACTIONS(4466),
    [anon_sym_RBRACE] = ACTIONS(4466),
    [anon_sym_macro_rules] = ACTIONS(4468),
    [anon_sym_BANG] = ACTIONS(4466),
    [anon_sym_LBRACK] = ACTIONS(4466),
    [anon_sym_STAR] = ACTIONS(4466),
    [anon_sym_u8] = ACTIONS(4468),
    [anon_sym_i8] = ACTIONS(4468),
    [anon_sym_u16] = ACTIONS(4468),
    [anon_sym_i16] = ACTIONS(4468),
    [anon_sym_u32] = ACTIONS(4468),
    [anon_sym_i32] = ACTIONS(4468),
    [anon_sym_u64] = ACTIONS(4468),
    [anon_sym_i64] = ACTIONS(4468),
    [anon_sym_u128] = ACTIONS(4468),
    [anon_sym_i128] = ACTIONS(4468),
    [anon_sym_isize] = ACTIONS(4468),
    [anon_sym_usize] = ACTIONS(4468),
    [anon_sym_f32] = ACTIONS(4468),
    [anon_sym_f64] = ACTIONS(4468),
    [anon_sym_bool] = ACTIONS(4468),
    [anon_sym_str] = ACTIONS(4468),
    [anon_sym_char] = ACTIONS(4468),
    [anon_sym_DASH] = ACTIONS(4466),
    [anon_sym_AMP] = ACTIONS(4466),
    [anon_sym_PIPE] = ACTIONS(4466),
    [anon_sym_LT] = ACTIONS(4466),
    [anon_sym_DOT_DOT] = ACTIONS(4466),
    [anon_sym_COLON_COLON] = ACTIONS(4466),
    [anon_sym_POUND] = ACTIONS(4466),
    [anon_sym_SQUOTE] = ACTIONS(4468),
    [anon_sym_async] = ACTIONS(4468),
    [anon_sym_auto] = ACTIONS(4468),
    [anon_sym_await] = ACTIONS(4468),
    [anon_sym_become] = ACTIONS(4468),
    [anon_sym_break] = ACTIONS(4468),
    [anon_sym_const] = ACTIONS(4468),
    [anon_sym_continue] = ACTIONS(4468),
    [anon_sym_default] = ACTIONS(4468),
    [anon_sym_enum] = ACTIONS(4468),
    [anon_sym_fn] = ACTIONS(4468),
    [anon_sym_for] = ACTIONS(4468),
    [anon_sym_gen] = ACTIONS(4468),
    [anon_sym_if] = ACTIONS(4468),
    [anon_sym_impl] = ACTIONS(4468),
    [anon_sym_let] = ACTIONS(4468),
    [anon_sym_loop] = ACTIONS(4468),
    [anon_sym_match] = ACTIONS(4468),
    [anon_sym_mod] = ACTIONS(4468),
    [anon_sym_pub] = ACTIONS(4468),
    [anon_sym_return] = ACTIONS(4468),
    [anon_sym_raw] = ACTIONS(4468),
    [anon_sym_static] = ACTIONS(4468),
    [anon_sym_struct] = ACTIONS(4468),
    [anon_sym_trait] = ACTIONS(4468),
    [anon_sym_type] = ACTIONS(4468),
    [anon_sym_union] = ACTIONS(4468),
    [anon_sym_unsafe] = ACTIONS(4468),
    [anon_sym_use] = ACTIONS(4468),
    [anon_sym_while] = ACTIONS(4468),
    [anon_sym_extern] = ACTIONS(4468),
    [anon_sym_safe] = ACTIONS(4468),
    [anon_sym_yield] = ACTIONS(4468),
    [anon_sym_move] = ACTIONS(4468),
    [anon_sym_try] = ACTIONS(4468),
    [sym_integer_literal] = ACTIONS(4466),
    [aux_sym_string_literal_token1] = ACTIONS(4466),
    [sym_char_literal] = ACTIONS(4466),
    [anon_sym_true] = ACTIONS(4468),
    [anon_sym_false] = ACTIONS(4468),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4468),
    [sym_super] = ACTIONS(4468),
    [sym_crate] = ACTIONS(4468),
    [aux_sym_metavariable_token1] = ACTIONS(4466),
    [sym__raw_string_literal_start] = ACTIONS(4466),
    [sym_float_literal] = ACTIONS(4466),
    [sym__outer_block_doc_comment_start] = ACTIONS(4466),
    [sym__outer_line_doc_comment_start] = ACTIONS(4466),
  },
  [STATE(1183)] = {
    [sym_line_comment] = STATE(1183),
    [sym_block_comment] = STATE(1183),
    [ts_builtin_sym_end] = ACTIONS(4470),
    [sym_identifier] = ACTIONS(4472),
    [anon_sym_SEMI] = ACTIONS(4470),
    [anon_sym_macro] = ACTIONS(4472),
    [anon_sym_LPAREN] = ACTIONS(4470),
    [anon_sym_LBRACE] = ACTIONS(4470),
    [anon_sym_RBRACE] = ACTIONS(4470),
    [anon_sym_macro_rules] = ACTIONS(4472),
    [anon_sym_BANG] = ACTIONS(4470),
    [anon_sym_LBRACK] = ACTIONS(4470),
    [anon_sym_STAR] = ACTIONS(4470),
    [anon_sym_u8] = ACTIONS(4472),
    [anon_sym_i8] = ACTIONS(4472),
    [anon_sym_u16] = ACTIONS(4472),
    [anon_sym_i16] = ACTIONS(4472),
    [anon_sym_u32] = ACTIONS(4472),
    [anon_sym_i32] = ACTIONS(4472),
    [anon_sym_u64] = ACTIONS(4472),
    [anon_sym_i64] = ACTIONS(4472),
    [anon_sym_u128] = ACTIONS(4472),
    [anon_sym_i128] = ACTIONS(4472),
    [anon_sym_isize] = ACTIONS(4472),
    [anon_sym_usize] = ACTIONS(4472),
    [anon_sym_f32] = ACTIONS(4472),
    [anon_sym_f64] = ACTIONS(4472),
    [anon_sym_bool] = ACTIONS(4472),
    [anon_sym_str] = ACTIONS(4472),
    [anon_sym_char] = ACTIONS(4472),
    [anon_sym_DASH] = ACTIONS(4470),
    [anon_sym_AMP] = ACTIONS(4470),
    [anon_sym_PIPE] = ACTIONS(4470),
    [anon_sym_LT] = ACTIONS(4470),
    [anon_sym_DOT_DOT] = ACTIONS(4470),
    [anon_sym_COLON_COLON] = ACTIONS(4470),
    [anon_sym_POUND] = ACTIONS(4470),
    [anon_sym_SQUOTE] = ACTIONS(4472),
    [anon_sym_async] = ACTIONS(4472),
    [anon_sym_auto] = ACTIONS(4472),
    [anon_sym_await] = ACTIONS(4472),
    [anon_sym_become] = ACTIONS(4472),
    [anon_sym_break] = ACTIONS(4472),
    [anon_sym_const] = ACTIONS(4472),
    [anon_sym_continue] = ACTIONS(4472),
    [anon_sym_default] = ACTIONS(4472),
    [anon_sym_enum] = ACTIONS(4472),
    [anon_sym_fn] = ACTIONS(4472),
    [anon_sym_for] = ACTIONS(4472),
    [anon_sym_gen] = ACTIONS(4472),
    [anon_sym_if] = ACTIONS(4472),
    [anon_sym_impl] = ACTIONS(4472),
    [anon_sym_let] = ACTIONS(4472),
    [anon_sym_loop] = ACTIONS(4472),
    [anon_sym_match] = ACTIONS(4472),
    [anon_sym_mod] = ACTIONS(4472),
    [anon_sym_pub] = ACTIONS(4472),
    [anon_sym_return] = ACTIONS(4472),
    [anon_sym_raw] = ACTIONS(4472),
    [anon_sym_static] = ACTIONS(4472),
    [anon_sym_struct] = ACTIONS(4472),
    [anon_sym_trait] = ACTIONS(4472),
    [anon_sym_type] = ACTIONS(4472),
    [anon_sym_union] = ACTIONS(4472),
    [anon_sym_unsafe] = ACTIONS(4472),
    [anon_sym_use] = ACTIONS(4472),
    [anon_sym_while] = ACTIONS(4472),
    [anon_sym_extern] = ACTIONS(4472),
    [anon_sym_safe] = ACTIONS(4472),
    [anon_sym_yield] = ACTIONS(4472),
    [anon_sym_move] = ACTIONS(4472),
    [anon_sym_try] = ACTIONS(4472),
    [sym_integer_literal] = ACTIONS(4470),
    [aux_sym_string_literal_token1] = ACTIONS(4470),
    [sym_char_literal] = ACTIONS(4470),
    [anon_sym_true] = ACTIONS(4472),
    [anon_sym_false] = ACTIONS(4472),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4472),
    [sym_super] = ACTIONS(4472),
    [sym_crate] = ACTIONS(4472),
    [aux_sym_metavariable_token1] = ACTIONS(4470),
    [sym__raw_string_literal_start] = ACTIONS(4470),
    [sym_float_literal] = ACTIONS(4470),
    [sym__outer_block_doc_comment_start] = ACTIONS(4470),
    [sym__outer_line_doc_comment_start] = ACTIONS(4470),
  },
  [STATE(1184)] = {
    [sym_line_comment] = STATE(1184),
    [sym_block_comment] = STATE(1184),
    [ts_builtin_sym_end] = ACTIONS(4474),
    [sym_identifier] = ACTIONS(4476),
    [anon_sym_SEMI] = ACTIONS(4474),
    [anon_sym_macro] = ACTIONS(4476),
    [anon_sym_LPAREN] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4474),
    [anon_sym_RBRACE] = ACTIONS(4474),
    [anon_sym_macro_rules] = ACTIONS(4476),
    [anon_sym_BANG] = ACTIONS(4474),
    [anon_sym_LBRACK] = ACTIONS(4474),
    [anon_sym_STAR] = ACTIONS(4474),
    [anon_sym_u8] = ACTIONS(4476),
    [anon_sym_i8] = ACTIONS(4476),
    [anon_sym_u16] = ACTIONS(4476),
    [anon_sym_i16] = ACTIONS(4476),
    [anon_sym_u32] = ACTIONS(4476),
    [anon_sym_i32] = ACTIONS(4476),
    [anon_sym_u64] = ACTIONS(4476),
    [anon_sym_i64] = ACTIONS(4476),
    [anon_sym_u128] = ACTIONS(4476),
    [anon_sym_i128] = ACTIONS(4476),
    [anon_sym_isize] = ACTIONS(4476),
    [anon_sym_usize] = ACTIONS(4476),
    [anon_sym_f32] = ACTIONS(4476),
    [anon_sym_f64] = ACTIONS(4476),
    [anon_sym_bool] = ACTIONS(4476),
    [anon_sym_str] = ACTIONS(4476),
    [anon_sym_char] = ACTIONS(4476),
    [anon_sym_DASH] = ACTIONS(4474),
    [anon_sym_AMP] = ACTIONS(4474),
    [anon_sym_PIPE] = ACTIONS(4474),
    [anon_sym_LT] = ACTIONS(4474),
    [anon_sym_DOT_DOT] = ACTIONS(4474),
    [anon_sym_COLON_COLON] = ACTIONS(4474),
    [anon_sym_POUND] = ACTIONS(4474),
    [anon_sym_SQUOTE] = ACTIONS(4476),
    [anon_sym_async] = ACTIONS(4476),
    [anon_sym_auto] = ACTIONS(4476),
    [anon_sym_await] = ACTIONS(4476),
    [anon_sym_become] = ACTIONS(4476),
    [anon_sym_break] = ACTIONS(4476),
    [anon_sym_const] = ACTIONS(4476),
    [anon_sym_continue] = ACTIONS(4476),
    [anon_sym_default] = ACTIONS(4476),
    [anon_sym_enum] = ACTIONS(4476),
    [anon_sym_fn] = ACTIONS(4476),
    [anon_sym_for] = ACTIONS(4476),
    [anon_sym_gen] = ACTIONS(4476),
    [anon_sym_if] = ACTIONS(4476),
    [anon_sym_impl] = ACTIONS(4476),
    [anon_sym_let] = ACTIONS(4476),
    [anon_sym_loop] = ACTIONS(4476),
    [anon_sym_match] = ACTIONS(4476),
    [anon_sym_mod] = ACTIONS(4476),
    [anon_sym_pub] = ACTIONS(4476),
    [anon_sym_return] = ACTIONS(4476),
    [anon_sym_raw] = ACTIONS(4476),
    [anon_sym_static] = ACTIONS(4476),
    [anon_sym_struct] = ACTIONS(4476),
    [anon_sym_trait] = ACTIONS(4476),
    [anon_sym_type] = ACTIONS(4476),
    [anon_sym_union] = ACTIONS(4476),
    [anon_sym_unsafe] = ACTIONS(4476),
    [anon_sym_use] = ACTIONS(4476),
    [anon_sym_while] = ACTIONS(4476),
    [anon_sym_extern] = ACTIONS(4476),
    [anon_sym_safe] = ACTIONS(4476),
    [anon_sym_yield] = ACTIONS(4476),
    [anon_sym_move] = ACTIONS(4476),
    [anon_sym_try] = ACTIONS(4476),
    [sym_integer_literal] = ACTIONS(4474),
    [aux_sym_string_literal_token1] = ACTIONS(4474),
    [sym_char_literal] = ACTIONS(4474),
    [anon_sym_true] = ACTIONS(4476),
    [anon_sym_false] = ACTIONS(4476),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4476),
    [sym_super] = ACTIONS(4476),
    [sym_crate] = ACTIONS(4476),
    [aux_sym_metavariable_token1] = ACTIONS(4474),
    [sym__raw_string_literal_start] = ACTIONS(4474),
    [sym_float_literal] = ACTIONS(4474),
    [sym__outer_block_doc_comment_start] = ACTIONS(4474),
    [sym__outer_line_doc_comment_start] = ACTIONS(4474),
  },
  [STATE(1185)] = {
    [sym_line_comment] = STATE(1185),
    [sym_block_comment] = STATE(1185),
    [ts_builtin_sym_end] = ACTIONS(4478),
    [sym_identifier] = ACTIONS(4480),
    [anon_sym_SEMI] = ACTIONS(4478),
    [anon_sym_macro] = ACTIONS(4480),
    [anon_sym_LPAREN] = ACTIONS(4478),
    [anon_sym_LBRACE] = ACTIONS(4478),
    [anon_sym_RBRACE] = ACTIONS(4478),
    [anon_sym_macro_rules] = ACTIONS(4480),
    [anon_sym_BANG] = ACTIONS(4478),
    [anon_sym_LBRACK] = ACTIONS(4478),
    [anon_sym_STAR] = ACTIONS(4478),
    [anon_sym_u8] = ACTIONS(4480),
    [anon_sym_i8] = ACTIONS(4480),
    [anon_sym_u16] = ACTIONS(4480),
    [anon_sym_i16] = ACTIONS(4480),
    [anon_sym_u32] = ACTIONS(4480),
    [anon_sym_i32] = ACTIONS(4480),
    [anon_sym_u64] = ACTIONS(4480),
    [anon_sym_i64] = ACTIONS(4480),
    [anon_sym_u128] = ACTIONS(4480),
    [anon_sym_i128] = ACTIONS(4480),
    [anon_sym_isize] = ACTIONS(4480),
    [anon_sym_usize] = ACTIONS(4480),
    [anon_sym_f32] = ACTIONS(4480),
    [anon_sym_f64] = ACTIONS(4480),
    [anon_sym_bool] = ACTIONS(4480),
    [anon_sym_str] = ACTIONS(4480),
    [anon_sym_char] = ACTIONS(4480),
    [anon_sym_DASH] = ACTIONS(4478),
    [anon_sym_AMP] = ACTIONS(4478),
    [anon_sym_PIPE] = ACTIONS(4478),
    [anon_sym_LT] = ACTIONS(4478),
    [anon_sym_DOT_DOT] = ACTIONS(4478),
    [anon_sym_COLON_COLON] = ACTIONS(4478),
    [anon_sym_POUND] = ACTIONS(4478),
    [anon_sym_SQUOTE] = ACTIONS(4480),
    [anon_sym_async] = ACTIONS(4480),
    [anon_sym_auto] = ACTIONS(4480),
    [anon_sym_await] = ACTIONS(4480),
    [anon_sym_become] = ACTIONS(4480),
    [anon_sym_break] = ACTIONS(4480),
    [anon_sym_const] = ACTIONS(4480),
    [anon_sym_continue] = ACTIONS(4480),
    [anon_sym_default] = ACTIONS(4480),
    [anon_sym_enum] = ACTIONS(4480),
    [anon_sym_fn] = ACTIONS(4480),
    [anon_sym_for] = ACTIONS(4480),
    [anon_sym_gen] = ACTIONS(4480),
    [anon_sym_if] = ACTIONS(4480),
    [anon_sym_impl] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4480),
    [anon_sym_loop] = ACTIONS(4480),
    [anon_sym_match] = ACTIONS(4480),
    [anon_sym_mod] = ACTIONS(4480),
    [anon_sym_pub] = ACTIONS(4480),
    [anon_sym_return] = ACTIONS(4480),
    [anon_sym_raw] = ACTIONS(4480),
    [anon_sym_static] = ACTIONS(4480),
    [anon_sym_struct] = ACTIONS(4480),
    [anon_sym_trait] = ACTIONS(4480),
    [anon_sym_type] = ACTIONS(4480),
    [anon_sym_union] = ACTIONS(4480),
    [anon_sym_unsafe] = ACTIONS(4480),
    [anon_sym_use] = ACTIONS(4480),
    [anon_sym_while] = ACTIONS(4480),
    [anon_sym_extern] = ACTIONS(4480),
    [anon_sym_safe] = ACTIONS(4480),
    [anon_sym_yield] = ACTIONS(4480),
    [anon_sym_move] = ACTIONS(4480),
    [anon_sym_try] = ACTIONS(4480),
    [sym_integer_literal] = ACTIONS(4478),
    [aux_sym_string_literal_token1] = ACTIONS(4478),
    [sym_char_literal] = ACTIONS(4478),
    [anon_sym_true] = ACTIONS(4480),
    [anon_sym_false] = ACTIONS(4480),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4480),
    [sym_super] = ACTIONS(4480),
    [sym_crate] = ACTIONS(4480),
    [aux_sym_metavariable_token1] = ACTIONS(4478),
    [sym__raw_string_literal_start] = ACTIONS(4478),
    [sym_float_literal] = ACTIONS(4478),
    [sym__outer_block_doc_comment_start] = ACTIONS(4478),
    [sym__outer_line_doc_comment_start] = ACTIONS(4478),
  },
  [STATE(1186)] = {
    [sym_line_comment] = STATE(1186),
    [sym_block_comment] = STATE(1186),
    [ts_builtin_sym_end] = ACTIONS(4482),
    [sym_identifier] = ACTIONS(4484),
    [anon_sym_SEMI] = ACTIONS(4482),
    [anon_sym_macro] = ACTIONS(4484),
    [anon_sym_LPAREN] = ACTIONS(4482),
    [anon_sym_LBRACE] = ACTIONS(4482),
    [anon_sym_RBRACE] = ACTIONS(4482),
    [anon_sym_macro_rules] = ACTIONS(4484),
    [anon_sym_BANG] = ACTIONS(4482),
    [anon_sym_LBRACK] = ACTIONS(4482),
    [anon_sym_STAR] = ACTIONS(4482),
    [anon_sym_u8] = ACTIONS(4484),
    [anon_sym_i8] = ACTIONS(4484),
    [anon_sym_u16] = ACTIONS(4484),
    [anon_sym_i16] = ACTIONS(4484),
    [anon_sym_u32] = ACTIONS(4484),
    [anon_sym_i32] = ACTIONS(4484),
    [anon_sym_u64] = ACTIONS(4484),
    [anon_sym_i64] = ACTIONS(4484),
    [anon_sym_u128] = ACTIONS(4484),
    [anon_sym_i128] = ACTIONS(4484),
    [anon_sym_isize] = ACTIONS(4484),
    [anon_sym_usize] = ACTIONS(4484),
    [anon_sym_f32] = ACTIONS(4484),
    [anon_sym_f64] = ACTIONS(4484),
    [anon_sym_bool] = ACTIONS(4484),
    [anon_sym_str] = ACTIONS(4484),
    [anon_sym_char] = ACTIONS(4484),
    [anon_sym_DASH] = ACTIONS(4482),
    [anon_sym_AMP] = ACTIONS(4482),
    [anon_sym_PIPE] = ACTIONS(4482),
    [anon_sym_LT] = ACTIONS(4482),
    [anon_sym_DOT_DOT] = ACTIONS(4482),
    [anon_sym_COLON_COLON] = ACTIONS(4482),
    [anon_sym_POUND] = ACTIONS(4482),
    [anon_sym_SQUOTE] = ACTIONS(4484),
    [anon_sym_async] = ACTIONS(4484),
    [anon_sym_auto] = ACTIONS(4484),
    [anon_sym_await] = ACTIONS(4484),
    [anon_sym_become] = ACTIONS(4484),
    [anon_sym_break] = ACTIONS(4484),
    [anon_sym_const] = ACTIONS(4484),
    [anon_sym_continue] = ACTIONS(4484),
    [anon_sym_default] = ACTIONS(4484),
    [anon_sym_enum] = ACTIONS(4484),
    [anon_sym_fn] = ACTIONS(4484),
    [anon_sym_for] = ACTIONS(4484),
    [anon_sym_gen] = ACTIONS(4484),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_impl] = ACTIONS(4484),
    [anon_sym_let] = ACTIONS(4484),
    [anon_sym_loop] = ACTIONS(4484),
    [anon_sym_match] = ACTIONS(4484),
    [anon_sym_mod] = ACTIONS(4484),
    [anon_sym_pub] = ACTIONS(4484),
    [anon_sym_return] = ACTIONS(4484),
    [anon_sym_raw] = ACTIONS(4484),
    [anon_sym_static] = ACTIONS(4484),
    [anon_sym_struct] = ACTIONS(4484),
    [anon_sym_trait] = ACTIONS(4484),
    [anon_sym_type] = ACTIONS(4484),
    [anon_sym_union] = ACTIONS(4484),
    [anon_sym_unsafe] = ACTIONS(4484),
    [anon_sym_use] = ACTIONS(4484),
    [anon_sym_while] = ACTIONS(4484),
    [anon_sym_extern] = ACTIONS(4484),
    [anon_sym_safe] = ACTIONS(4484),
    [anon_sym_yield] = ACTIONS(4484),
    [anon_sym_move] = ACTIONS(4484),
    [anon_sym_try] = ACTIONS(4484),
    [sym_integer_literal] = ACTIONS(4482),
    [aux_sym_string_literal_token1] = ACTIONS(4482),
    [sym_char_literal] = ACTIONS(4482),
    [anon_sym_true] = ACTIONS(4484),
    [anon_sym_false] = ACTIONS(4484),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4484),
    [sym_super] = ACTIONS(4484),
    [sym_crate] = ACTIONS(4484),
    [aux_sym_metavariable_token1] = ACTIONS(4482),
    [sym__raw_string_literal_start] = ACTIONS(4482),
    [sym_float_literal] = ACTIONS(4482),
    [sym__outer_block_doc_comment_start] = ACTIONS(4482),
    [sym__outer_line_doc_comment_start] = ACTIONS(4482),
  },
  [STATE(1187)] = {
    [sym_line_comment] = STATE(1187),
    [sym_block_comment] = STATE(1187),
    [ts_builtin_sym_end] = ACTIONS(4486),
    [sym_identifier] = ACTIONS(4488),
    [anon_sym_SEMI] = ACTIONS(4486),
    [anon_sym_macro] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(4486),
    [anon_sym_LBRACE] = ACTIONS(4486),
    [anon_sym_RBRACE] = ACTIONS(4486),
    [anon_sym_macro_rules] = ACTIONS(4488),
    [anon_sym_BANG] = ACTIONS(4486),
    [anon_sym_LBRACK] = ACTIONS(4486),
    [anon_sym_STAR] = ACTIONS(4486),
    [anon_sym_u8] = ACTIONS(4488),
    [anon_sym_i8] = ACTIONS(4488),
    [anon_sym_u16] = ACTIONS(4488),
    [anon_sym_i16] = ACTIONS(4488),
    [anon_sym_u32] = ACTIONS(4488),
    [anon_sym_i32] = ACTIONS(4488),
    [anon_sym_u64] = ACTIONS(4488),
    [anon_sym_i64] = ACTIONS(4488),
    [anon_sym_u128] = ACTIONS(4488),
    [anon_sym_i128] = ACTIONS(4488),
    [anon_sym_isize] = ACTIONS(4488),
    [anon_sym_usize] = ACTIONS(4488),
    [anon_sym_f32] = ACTIONS(4488),
    [anon_sym_f64] = ACTIONS(4488),
    [anon_sym_bool] = ACTIONS(4488),
    [anon_sym_str] = ACTIONS(4488),
    [anon_sym_char] = ACTIONS(4488),
    [anon_sym_DASH] = ACTIONS(4486),
    [anon_sym_AMP] = ACTIONS(4486),
    [anon_sym_PIPE] = ACTIONS(4486),
    [anon_sym_LT] = ACTIONS(4486),
    [anon_sym_DOT_DOT] = ACTIONS(4486),
    [anon_sym_COLON_COLON] = ACTIONS(4486),
    [anon_sym_POUND] = ACTIONS(4486),
    [anon_sym_SQUOTE] = ACTIONS(4488),
    [anon_sym_async] = ACTIONS(4488),
    [anon_sym_auto] = ACTIONS(4488),
    [anon_sym_await] = ACTIONS(4488),
    [anon_sym_become] = ACTIONS(4488),
    [anon_sym_break] = ACTIONS(4488),
    [anon_sym_const] = ACTIONS(4488),
    [anon_sym_continue] = ACTIONS(4488),
    [anon_sym_default] = ACTIONS(4488),
    [anon_sym_enum] = ACTIONS(4488),
    [anon_sym_fn] = ACTIONS(4488),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_gen] = ACTIONS(4488),
    [anon_sym_if] = ACTIONS(4488),
    [anon_sym_impl] = ACTIONS(4488),
    [anon_sym_let] = ACTIONS(4488),
    [anon_sym_loop] = ACTIONS(4488),
    [anon_sym_match] = ACTIONS(4488),
    [anon_sym_mod] = ACTIONS(4488),
    [anon_sym_pub] = ACTIONS(4488),
    [anon_sym_return] = ACTIONS(4488),
    [anon_sym_raw] = ACTIONS(4488),
    [anon_sym_static] = ACTIONS(4488),
    [anon_sym_struct] = ACTIONS(4488),
    [anon_sym_trait] = ACTIONS(4488),
    [anon_sym_type] = ACTIONS(4488),
    [anon_sym_union] = ACTIONS(4488),
    [anon_sym_unsafe] = ACTIONS(4488),
    [anon_sym_use] = ACTIONS(4488),
    [anon_sym_while] = ACTIONS(4488),
    [anon_sym_extern] = ACTIONS(4488),
    [anon_sym_safe] = ACTIONS(4488),
    [anon_sym_yield] = ACTIONS(4488),
    [anon_sym_move] = ACTIONS(4488),
    [anon_sym_try] = ACTIONS(4488),
    [sym_integer_literal] = ACTIONS(4486),
    [aux_sym_string_literal_token1] = ACTIONS(4486),
    [sym_char_literal] = ACTIONS(4486),
    [anon_sym_true] = ACTIONS(4488),
    [anon_sym_false] = ACTIONS(4488),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4488),
    [sym_super] = ACTIONS(4488),
    [sym_crate] = ACTIONS(4488),
    [aux_sym_metavariable_token1] = ACTIONS(4486),
    [sym__raw_string_literal_start] = ACTIONS(4486),
    [sym_float_literal] = ACTIONS(4486),
    [sym__outer_block_doc_comment_start] = ACTIONS(4486),
    [sym__outer_line_doc_comment_start] = ACTIONS(4486),
  },
  [STATE(1188)] = {
    [sym_line_comment] = STATE(1188),
    [sym_block_comment] = STATE(1188),
    [ts_builtin_sym_end] = ACTIONS(4490),
    [sym_identifier] = ACTIONS(4492),
    [anon_sym_SEMI] = ACTIONS(4490),
    [anon_sym_macro] = ACTIONS(4492),
    [anon_sym_LPAREN] = ACTIONS(4490),
    [anon_sym_LBRACE] = ACTIONS(4490),
    [anon_sym_RBRACE] = ACTIONS(4490),
    [anon_sym_macro_rules] = ACTIONS(4492),
    [anon_sym_BANG] = ACTIONS(4490),
    [anon_sym_LBRACK] = ACTIONS(4490),
    [anon_sym_STAR] = ACTIONS(4490),
    [anon_sym_u8] = ACTIONS(4492),
    [anon_sym_i8] = ACTIONS(4492),
    [anon_sym_u16] = ACTIONS(4492),
    [anon_sym_i16] = ACTIONS(4492),
    [anon_sym_u32] = ACTIONS(4492),
    [anon_sym_i32] = ACTIONS(4492),
    [anon_sym_u64] = ACTIONS(4492),
    [anon_sym_i64] = ACTIONS(4492),
    [anon_sym_u128] = ACTIONS(4492),
    [anon_sym_i128] = ACTIONS(4492),
    [anon_sym_isize] = ACTIONS(4492),
    [anon_sym_usize] = ACTIONS(4492),
    [anon_sym_f32] = ACTIONS(4492),
    [anon_sym_f64] = ACTIONS(4492),
    [anon_sym_bool] = ACTIONS(4492),
    [anon_sym_str] = ACTIONS(4492),
    [anon_sym_char] = ACTIONS(4492),
    [anon_sym_DASH] = ACTIONS(4490),
    [anon_sym_AMP] = ACTIONS(4490),
    [anon_sym_PIPE] = ACTIONS(4490),
    [anon_sym_LT] = ACTIONS(4490),
    [anon_sym_DOT_DOT] = ACTIONS(4490),
    [anon_sym_COLON_COLON] = ACTIONS(4490),
    [anon_sym_POUND] = ACTIONS(4490),
    [anon_sym_SQUOTE] = ACTIONS(4492),
    [anon_sym_async] = ACTIONS(4492),
    [anon_sym_auto] = ACTIONS(4492),
    [anon_sym_await] = ACTIONS(4492),
    [anon_sym_become] = ACTIONS(4492),
    [anon_sym_break] = ACTIONS(4492),
    [anon_sym_const] = ACTIONS(4492),
    [anon_sym_continue] = ACTIONS(4492),
    [anon_sym_default] = ACTIONS(4492),
    [anon_sym_enum] = ACTIONS(4492),
    [anon_sym_fn] = ACTIONS(4492),
    [anon_sym_for] = ACTIONS(4492),
    [anon_sym_gen] = ACTIONS(4492),
    [anon_sym_if] = ACTIONS(4492),
    [anon_sym_impl] = ACTIONS(4492),
    [anon_sym_let] = ACTIONS(4492),
    [anon_sym_loop] = ACTIONS(4492),
    [anon_sym_match] = ACTIONS(4492),
    [anon_sym_mod] = ACTIONS(4492),
    [anon_sym_pub] = ACTIONS(4492),
    [anon_sym_return] = ACTIONS(4492),
    [anon_sym_raw] = ACTIONS(4492),
    [anon_sym_static] = ACTIONS(4492),
    [anon_sym_struct] = ACTIONS(4492),
    [anon_sym_trait] = ACTIONS(4492),
    [anon_sym_type] = ACTIONS(4492),
    [anon_sym_union] = ACTIONS(4492),
    [anon_sym_unsafe] = ACTIONS(4492),
    [anon_sym_use] = ACTIONS(4492),
    [anon_sym_while] = ACTIONS(4492),
    [anon_sym_extern] = ACTIONS(4492),
    [anon_sym_safe] = ACTIONS(4492),
    [anon_sym_yield] = ACTIONS(4492),
    [anon_sym_move] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4492),
    [sym_integer_literal] = ACTIONS(4490),
    [aux_sym_string_literal_token1] = ACTIONS(4490),
    [sym_char_literal] = ACTIONS(4490),
    [anon_sym_true] = ACTIONS(4492),
    [anon_sym_false] = ACTIONS(4492),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4492),
    [sym_super] = ACTIONS(4492),
    [sym_crate] = ACTIONS(4492),
    [aux_sym_metavariable_token1] = ACTIONS(4490),
    [sym__raw_string_literal_start] = ACTIONS(4490),
    [sym_float_literal] = ACTIONS(4490),
    [sym__outer_block_doc_comment_start] = ACTIONS(4490),
    [sym__outer_line_doc_comment_start] = ACTIONS(4490),
  },
  [STATE(1189)] = {
    [sym_line_comment] = STATE(1189),
    [sym_block_comment] = STATE(1189),
    [ts_builtin_sym_end] = ACTIONS(4494),
    [sym_identifier] = ACTIONS(4496),
    [anon_sym_SEMI] = ACTIONS(4494),
    [anon_sym_macro] = ACTIONS(4496),
    [anon_sym_LPAREN] = ACTIONS(4494),
    [anon_sym_LBRACE] = ACTIONS(4494),
    [anon_sym_RBRACE] = ACTIONS(4494),
    [anon_sym_macro_rules] = ACTIONS(4496),
    [anon_sym_BANG] = ACTIONS(4494),
    [anon_sym_LBRACK] = ACTIONS(4494),
    [anon_sym_STAR] = ACTIONS(4494),
    [anon_sym_u8] = ACTIONS(4496),
    [anon_sym_i8] = ACTIONS(4496),
    [anon_sym_u16] = ACTIONS(4496),
    [anon_sym_i16] = ACTIONS(4496),
    [anon_sym_u32] = ACTIONS(4496),
    [anon_sym_i32] = ACTIONS(4496),
    [anon_sym_u64] = ACTIONS(4496),
    [anon_sym_i64] = ACTIONS(4496),
    [anon_sym_u128] = ACTIONS(4496),
    [anon_sym_i128] = ACTIONS(4496),
    [anon_sym_isize] = ACTIONS(4496),
    [anon_sym_usize] = ACTIONS(4496),
    [anon_sym_f32] = ACTIONS(4496),
    [anon_sym_f64] = ACTIONS(4496),
    [anon_sym_bool] = ACTIONS(4496),
    [anon_sym_str] = ACTIONS(4496),
    [anon_sym_char] = ACTIONS(4496),
    [anon_sym_DASH] = ACTIONS(4494),
    [anon_sym_AMP] = ACTIONS(4494),
    [anon_sym_PIPE] = ACTIONS(4494),
    [anon_sym_LT] = ACTIONS(4494),
    [anon_sym_DOT_DOT] = ACTIONS(4494),
    [anon_sym_COLON_COLON] = ACTIONS(4494),
    [anon_sym_POUND] = ACTIONS(4494),
    [anon_sym_SQUOTE] = ACTIONS(4496),
    [anon_sym_async] = ACTIONS(4496),
    [anon_sym_auto] = ACTIONS(4496),
    [anon_sym_await] = ACTIONS(4496),
    [anon_sym_become] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4496),
    [anon_sym_const] = ACTIONS(4496),
    [anon_sym_continue] = ACTIONS(4496),
    [anon_sym_default] = ACTIONS(4496),
    [anon_sym_enum] = ACTIONS(4496),
    [anon_sym_fn] = ACTIONS(4496),
    [anon_sym_for] = ACTIONS(4496),
    [anon_sym_gen] = ACTIONS(4496),
    [anon_sym_if] = ACTIONS(4496),
    [anon_sym_impl] = ACTIONS(4496),
    [anon_sym_let] = ACTIONS(4496),
    [anon_sym_loop] = ACTIONS(4496),
    [anon_sym_match] = ACTIONS(4496),
    [anon_sym_mod] = ACTIONS(4496),
    [anon_sym_pub] = ACTIONS(4496),
    [anon_sym_return] = ACTIONS(4496),
    [anon_sym_raw] = ACTIONS(4496),
    [anon_sym_static] = ACTIONS(4496),
    [anon_sym_struct] = ACTIONS(4496),
    [anon_sym_trait] = ACTIONS(4496),
    [anon_sym_type] = ACTIONS(4496),
    [anon_sym_union] = ACTIONS(4496),
    [anon_sym_unsafe] = ACTIONS(4496),
    [anon_sym_use] = ACTIONS(4496),
    [anon_sym_while] = ACTIONS(4496),
    [anon_sym_extern] = ACTIONS(4496),
    [anon_sym_safe] = ACTIONS(4496),
    [anon_sym_yield] = ACTIONS(4496),
    [anon_sym_move] = ACTIONS(4496),
    [anon_sym_try] = ACTIONS(4496),
    [sym_integer_literal] = ACTIONS(4494),
    [aux_sym_string_literal_token1] = ACTIONS(4494),
    [sym_char_literal] = ACTIONS(4494),
    [anon_sym_true] = ACTIONS(4496),
    [anon_sym_false] = ACTIONS(4496),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4496),
    [sym_super] = ACTIONS(4496),
    [sym_crate] = ACTIONS(4496),
    [aux_sym_metavariable_token1] = ACTIONS(4494),
    [sym__raw_string_literal_start] = ACTIONS(4494),
    [sym_float_literal] = ACTIONS(4494),
    [sym__outer_block_doc_comment_start] = ACTIONS(4494),
    [sym__outer_line_doc_comment_start] = ACTIONS(4494),
  },
  [STATE(1190)] = {
    [sym_line_comment] = STATE(1190),
    [sym_block_comment] = STATE(1190),
    [ts_builtin_sym_end] = ACTIONS(4498),
    [sym_identifier] = ACTIONS(4500),
    [anon_sym_SEMI] = ACTIONS(4498),
    [anon_sym_macro] = ACTIONS(4500),
    [anon_sym_LPAREN] = ACTIONS(4498),
    [anon_sym_LBRACE] = ACTIONS(4498),
    [anon_sym_RBRACE] = ACTIONS(4498),
    [anon_sym_macro_rules] = ACTIONS(4500),
    [anon_sym_BANG] = ACTIONS(4498),
    [anon_sym_LBRACK] = ACTIONS(4498),
    [anon_sym_STAR] = ACTIONS(4498),
    [anon_sym_u8] = ACTIONS(4500),
    [anon_sym_i8] = ACTIONS(4500),
    [anon_sym_u16] = ACTIONS(4500),
    [anon_sym_i16] = ACTIONS(4500),
    [anon_sym_u32] = ACTIONS(4500),
    [anon_sym_i32] = ACTIONS(4500),
    [anon_sym_u64] = ACTIONS(4500),
    [anon_sym_i64] = ACTIONS(4500),
    [anon_sym_u128] = ACTIONS(4500),
    [anon_sym_i128] = ACTIONS(4500),
    [anon_sym_isize] = ACTIONS(4500),
    [anon_sym_usize] = ACTIONS(4500),
    [anon_sym_f32] = ACTIONS(4500),
    [anon_sym_f64] = ACTIONS(4500),
    [anon_sym_bool] = ACTIONS(4500),
    [anon_sym_str] = ACTIONS(4500),
    [anon_sym_char] = ACTIONS(4500),
    [anon_sym_DASH] = ACTIONS(4498),
    [anon_sym_AMP] = ACTIONS(4498),
    [anon_sym_PIPE] = ACTIONS(4498),
    [anon_sym_LT] = ACTIONS(4498),
    [anon_sym_DOT_DOT] = ACTIONS(4498),
    [anon_sym_COLON_COLON] = ACTIONS(4498),
    [anon_sym_POUND] = ACTIONS(4498),
    [anon_sym_SQUOTE] = ACTIONS(4500),
    [anon_sym_async] = ACTIONS(4500),
    [anon_sym_auto] = ACTIONS(4500),
    [anon_sym_await] = ACTIONS(4500),
    [anon_sym_become] = ACTIONS(4500),
    [anon_sym_break] = ACTIONS(4500),
    [anon_sym_const] = ACTIONS(4500),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_default] = ACTIONS(4500),
    [anon_sym_enum] = ACTIONS(4500),
    [anon_sym_fn] = ACTIONS(4500),
    [anon_sym_for] = ACTIONS(4500),
    [anon_sym_gen] = ACTIONS(4500),
    [anon_sym_if] = ACTIONS(4500),
    [anon_sym_impl] = ACTIONS(4500),
    [anon_sym_let] = ACTIONS(4500),
    [anon_sym_loop] = ACTIONS(4500),
    [anon_sym_match] = ACTIONS(4500),
    [anon_sym_mod] = ACTIONS(4500),
    [anon_sym_pub] = ACTIONS(4500),
    [anon_sym_return] = ACTIONS(4500),
    [anon_sym_raw] = ACTIONS(4500),
    [anon_sym_static] = ACTIONS(4500),
    [anon_sym_struct] = ACTIONS(4500),
    [anon_sym_trait] = ACTIONS(4500),
    [anon_sym_type] = ACTIONS(4500),
    [anon_sym_union] = ACTIONS(4500),
    [anon_sym_unsafe] = ACTIONS(4500),
    [anon_sym_use] = ACTIONS(4500),
    [anon_sym_while] = ACTIONS(4500),
    [anon_sym_extern] = ACTIONS(4500),
    [anon_sym_safe] = ACTIONS(4500),
    [anon_sym_yield] = ACTIONS(4500),
    [anon_sym_move] = ACTIONS(4500),
    [anon_sym_try] = ACTIONS(4500),
    [sym_integer_literal] = ACTIONS(4498),
    [aux_sym_string_literal_token1] = ACTIONS(4498),
    [sym_char_literal] = ACTIONS(4498),
    [anon_sym_true] = ACTIONS(4500),
    [anon_sym_false] = ACTIONS(4500),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4500),
    [sym_super] = ACTIONS(4500),
    [sym_crate] = ACTIONS(4500),
    [aux_sym_metavariable_token1] = ACTIONS(4498),
    [sym__raw_string_literal_start] = ACTIONS(4498),
    [sym_float_literal] = ACTIONS(4498),
    [sym__outer_block_doc_comment_start] = ACTIONS(4498),
    [sym__outer_line_doc_comment_start] = ACTIONS(4498),
  },
  [STATE(1191)] = {
    [sym_line_comment] = STATE(1191),
    [sym_block_comment] = STATE(1191),
    [ts_builtin_sym_end] = ACTIONS(4502),
    [sym_identifier] = ACTIONS(4504),
    [anon_sym_SEMI] = ACTIONS(4502),
    [anon_sym_macro] = ACTIONS(4504),
    [anon_sym_LPAREN] = ACTIONS(4502),
    [anon_sym_LBRACE] = ACTIONS(4502),
    [anon_sym_RBRACE] = ACTIONS(4502),
    [anon_sym_macro_rules] = ACTIONS(4504),
    [anon_sym_BANG] = ACTIONS(4502),
    [anon_sym_LBRACK] = ACTIONS(4502),
    [anon_sym_STAR] = ACTIONS(4502),
    [anon_sym_u8] = ACTIONS(4504),
    [anon_sym_i8] = ACTIONS(4504),
    [anon_sym_u16] = ACTIONS(4504),
    [anon_sym_i16] = ACTIONS(4504),
    [anon_sym_u32] = ACTIONS(4504),
    [anon_sym_i32] = ACTIONS(4504),
    [anon_sym_u64] = ACTIONS(4504),
    [anon_sym_i64] = ACTIONS(4504),
    [anon_sym_u128] = ACTIONS(4504),
    [anon_sym_i128] = ACTIONS(4504),
    [anon_sym_isize] = ACTIONS(4504),
    [anon_sym_usize] = ACTIONS(4504),
    [anon_sym_f32] = ACTIONS(4504),
    [anon_sym_f64] = ACTIONS(4504),
    [anon_sym_bool] = ACTIONS(4504),
    [anon_sym_str] = ACTIONS(4504),
    [anon_sym_char] = ACTIONS(4504),
    [anon_sym_DASH] = ACTIONS(4502),
    [anon_sym_AMP] = ACTIONS(4502),
    [anon_sym_PIPE] = ACTIONS(4502),
    [anon_sym_LT] = ACTIONS(4502),
    [anon_sym_DOT_DOT] = ACTIONS(4502),
    [anon_sym_COLON_COLON] = ACTIONS(4502),
    [anon_sym_POUND] = ACTIONS(4502),
    [anon_sym_SQUOTE] = ACTIONS(4504),
    [anon_sym_async] = ACTIONS(4504),
    [anon_sym_auto] = ACTIONS(4504),
    [anon_sym_await] = ACTIONS(4504),
    [anon_sym_become] = ACTIONS(4504),
    [anon_sym_break] = ACTIONS(4504),
    [anon_sym_const] = ACTIONS(4504),
    [anon_sym_continue] = ACTIONS(4504),
    [anon_sym_default] = ACTIONS(4504),
    [anon_sym_enum] = ACTIONS(4504),
    [anon_sym_fn] = ACTIONS(4504),
    [anon_sym_for] = ACTIONS(4504),
    [anon_sym_gen] = ACTIONS(4504),
    [anon_sym_if] = ACTIONS(4504),
    [anon_sym_impl] = ACTIONS(4504),
    [anon_sym_let] = ACTIONS(4504),
    [anon_sym_loop] = ACTIONS(4504),
    [anon_sym_match] = ACTIONS(4504),
    [anon_sym_mod] = ACTIONS(4504),
    [anon_sym_pub] = ACTIONS(4504),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_raw] = ACTIONS(4504),
    [anon_sym_static] = ACTIONS(4504),
    [anon_sym_struct] = ACTIONS(4504),
    [anon_sym_trait] = ACTIONS(4504),
    [anon_sym_type] = ACTIONS(4504),
    [anon_sym_union] = ACTIONS(4504),
    [anon_sym_unsafe] = ACTIONS(4504),
    [anon_sym_use] = ACTIONS(4504),
    [anon_sym_while] = ACTIONS(4504),
    [anon_sym_extern] = ACTIONS(4504),
    [anon_sym_safe] = ACTIONS(4504),
    [anon_sym_yield] = ACTIONS(4504),
    [anon_sym_move] = ACTIONS(4504),
    [anon_sym_try] = ACTIONS(4504),
    [sym_integer_literal] = ACTIONS(4502),
    [aux_sym_string_literal_token1] = ACTIONS(4502),
    [sym_char_literal] = ACTIONS(4502),
    [anon_sym_true] = ACTIONS(4504),
    [anon_sym_false] = ACTIONS(4504),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4504),
    [sym_super] = ACTIONS(4504),
    [sym_crate] = ACTIONS(4504),
    [aux_sym_metavariable_token1] = ACTIONS(4502),
    [sym__raw_string_literal_start] = ACTIONS(4502),
    [sym_float_literal] = ACTIONS(4502),
    [sym__outer_block_doc_comment_start] = ACTIONS(4502),
    [sym__outer_line_doc_comment_start] = ACTIONS(4502),
  },
  [STATE(1192)] = {
    [sym_line_comment] = STATE(1192),
    [sym_block_comment] = STATE(1192),
    [ts_builtin_sym_end] = ACTIONS(4506),
    [sym_identifier] = ACTIONS(4508),
    [anon_sym_SEMI] = ACTIONS(4506),
    [anon_sym_macro] = ACTIONS(4508),
    [anon_sym_LPAREN] = ACTIONS(4506),
    [anon_sym_LBRACE] = ACTIONS(4506),
    [anon_sym_RBRACE] = ACTIONS(4506),
    [anon_sym_macro_rules] = ACTIONS(4508),
    [anon_sym_BANG] = ACTIONS(4506),
    [anon_sym_LBRACK] = ACTIONS(4506),
    [anon_sym_STAR] = ACTIONS(4506),
    [anon_sym_u8] = ACTIONS(4508),
    [anon_sym_i8] = ACTIONS(4508),
    [anon_sym_u16] = ACTIONS(4508),
    [anon_sym_i16] = ACTIONS(4508),
    [anon_sym_u32] = ACTIONS(4508),
    [anon_sym_i32] = ACTIONS(4508),
    [anon_sym_u64] = ACTIONS(4508),
    [anon_sym_i64] = ACTIONS(4508),
    [anon_sym_u128] = ACTIONS(4508),
    [anon_sym_i128] = ACTIONS(4508),
    [anon_sym_isize] = ACTIONS(4508),
    [anon_sym_usize] = ACTIONS(4508),
    [anon_sym_f32] = ACTIONS(4508),
    [anon_sym_f64] = ACTIONS(4508),
    [anon_sym_bool] = ACTIONS(4508),
    [anon_sym_str] = ACTIONS(4508),
    [anon_sym_char] = ACTIONS(4508),
    [anon_sym_DASH] = ACTIONS(4506),
    [anon_sym_AMP] = ACTIONS(4506),
    [anon_sym_PIPE] = ACTIONS(4506),
    [anon_sym_LT] = ACTIONS(4506),
    [anon_sym_DOT_DOT] = ACTIONS(4506),
    [anon_sym_COLON_COLON] = ACTIONS(4506),
    [anon_sym_POUND] = ACTIONS(4506),
    [anon_sym_SQUOTE] = ACTIONS(4508),
    [anon_sym_async] = ACTIONS(4508),
    [anon_sym_auto] = ACTIONS(4508),
    [anon_sym_await] = ACTIONS(4508),
    [anon_sym_become] = ACTIONS(4508),
    [anon_sym_break] = ACTIONS(4508),
    [anon_sym_const] = ACTIONS(4508),
    [anon_sym_continue] = ACTIONS(4508),
    [anon_sym_default] = ACTIONS(4508),
    [anon_sym_enum] = ACTIONS(4508),
    [anon_sym_fn] = ACTIONS(4508),
    [anon_sym_for] = ACTIONS(4508),
    [anon_sym_gen] = ACTIONS(4508),
    [anon_sym_if] = ACTIONS(4508),
    [anon_sym_impl] = ACTIONS(4508),
    [anon_sym_let] = ACTIONS(4508),
    [anon_sym_loop] = ACTIONS(4508),
    [anon_sym_match] = ACTIONS(4508),
    [anon_sym_mod] = ACTIONS(4508),
    [anon_sym_pub] = ACTIONS(4508),
    [anon_sym_return] = ACTIONS(4508),
    [anon_sym_raw] = ACTIONS(4508),
    [anon_sym_static] = ACTIONS(4508),
    [anon_sym_struct] = ACTIONS(4508),
    [anon_sym_trait] = ACTIONS(4508),
    [anon_sym_type] = ACTIONS(4508),
    [anon_sym_union] = ACTIONS(4508),
    [anon_sym_unsafe] = ACTIONS(4508),
    [anon_sym_use] = ACTIONS(4508),
    [anon_sym_while] = ACTIONS(4508),
    [anon_sym_extern] = ACTIONS(4508),
    [anon_sym_safe] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(4508),
    [anon_sym_move] = ACTIONS(4508),
    [anon_sym_try] = ACTIONS(4508),
    [sym_integer_literal] = ACTIONS(4506),
    [aux_sym_string_literal_token1] = ACTIONS(4506),
    [sym_char_literal] = ACTIONS(4506),
    [anon_sym_true] = ACTIONS(4508),
    [anon_sym_false] = ACTIONS(4508),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4508),
    [sym_super] = ACTIONS(4508),
    [sym_crate] = ACTIONS(4508),
    [aux_sym_metavariable_token1] = ACTIONS(4506),
    [sym__raw_string_literal_start] = ACTIONS(4506),
    [sym_float_literal] = ACTIONS(4506),
    [sym__outer_block_doc_comment_start] = ACTIONS(4506),
    [sym__outer_line_doc_comment_start] = ACTIONS(4506),
  },
  [STATE(1193)] = {
    [sym_line_comment] = STATE(1193),
    [sym_block_comment] = STATE(1193),
    [ts_builtin_sym_end] = ACTIONS(4510),
    [sym_identifier] = ACTIONS(4512),
    [anon_sym_SEMI] = ACTIONS(4510),
    [anon_sym_macro] = ACTIONS(4512),
    [anon_sym_LPAREN] = ACTIONS(4510),
    [anon_sym_LBRACE] = ACTIONS(4510),
    [anon_sym_RBRACE] = ACTIONS(4510),
    [anon_sym_macro_rules] = ACTIONS(4512),
    [anon_sym_BANG] = ACTIONS(4510),
    [anon_sym_LBRACK] = ACTIONS(4510),
    [anon_sym_STAR] = ACTIONS(4510),
    [anon_sym_u8] = ACTIONS(4512),
    [anon_sym_i8] = ACTIONS(4512),
    [anon_sym_u16] = ACTIONS(4512),
    [anon_sym_i16] = ACTIONS(4512),
    [anon_sym_u32] = ACTIONS(4512),
    [anon_sym_i32] = ACTIONS(4512),
    [anon_sym_u64] = ACTIONS(4512),
    [anon_sym_i64] = ACTIONS(4512),
    [anon_sym_u128] = ACTIONS(4512),
    [anon_sym_i128] = ACTIONS(4512),
    [anon_sym_isize] = ACTIONS(4512),
    [anon_sym_usize] = ACTIONS(4512),
    [anon_sym_f32] = ACTIONS(4512),
    [anon_sym_f64] = ACTIONS(4512),
    [anon_sym_bool] = ACTIONS(4512),
    [anon_sym_str] = ACTIONS(4512),
    [anon_sym_char] = ACTIONS(4512),
    [anon_sym_DASH] = ACTIONS(4510),
    [anon_sym_AMP] = ACTIONS(4510),
    [anon_sym_PIPE] = ACTIONS(4510),
    [anon_sym_LT] = ACTIONS(4510),
    [anon_sym_DOT_DOT] = ACTIONS(4510),
    [anon_sym_COLON_COLON] = ACTIONS(4510),
    [anon_sym_POUND] = ACTIONS(4510),
    [anon_sym_SQUOTE] = ACTIONS(4512),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_auto] = ACTIONS(4512),
    [anon_sym_await] = ACTIONS(4512),
    [anon_sym_become] = ACTIONS(4512),
    [anon_sym_break] = ACTIONS(4512),
    [anon_sym_const] = ACTIONS(4512),
    [anon_sym_continue] = ACTIONS(4512),
    [anon_sym_default] = ACTIONS(4512),
    [anon_sym_enum] = ACTIONS(4512),
    [anon_sym_fn] = ACTIONS(4512),
    [anon_sym_for] = ACTIONS(4512),
    [anon_sym_gen] = ACTIONS(4512),
    [anon_sym_if] = ACTIONS(4512),
    [anon_sym_impl] = ACTIONS(4512),
    [anon_sym_let] = ACTIONS(4512),
    [anon_sym_loop] = ACTIONS(4512),
    [anon_sym_match] = ACTIONS(4512),
    [anon_sym_mod] = ACTIONS(4512),
    [anon_sym_pub] = ACTIONS(4512),
    [anon_sym_return] = ACTIONS(4512),
    [anon_sym_raw] = ACTIONS(4512),
    [anon_sym_static] = ACTIONS(4512),
    [anon_sym_struct] = ACTIONS(4512),
    [anon_sym_trait] = ACTIONS(4512),
    [anon_sym_type] = ACTIONS(4512),
    [anon_sym_union] = ACTIONS(4512),
    [anon_sym_unsafe] = ACTIONS(4512),
    [anon_sym_use] = ACTIONS(4512),
    [anon_sym_while] = ACTIONS(4512),
    [anon_sym_extern] = ACTIONS(4512),
    [anon_sym_safe] = ACTIONS(4512),
    [anon_sym_yield] = ACTIONS(4512),
    [anon_sym_move] = ACTIONS(4512),
    [anon_sym_try] = ACTIONS(4512),
    [sym_integer_literal] = ACTIONS(4510),
    [aux_sym_string_literal_token1] = ACTIONS(4510),
    [sym_char_literal] = ACTIONS(4510),
    [anon_sym_true] = ACTIONS(4512),
    [anon_sym_false] = ACTIONS(4512),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4512),
    [sym_super] = ACTIONS(4512),
    [sym_crate] = ACTIONS(4512),
    [aux_sym_metavariable_token1] = ACTIONS(4510),
    [sym__raw_string_literal_start] = ACTIONS(4510),
    [sym_float_literal] = ACTIONS(4510),
    [sym__outer_block_doc_comment_start] = ACTIONS(4510),
    [sym__outer_line_doc_comment_start] = ACTIONS(4510),
  },
  [STATE(1194)] = {
    [sym_line_comment] = STATE(1194),
    [sym_block_comment] = STATE(1194),
    [ts_builtin_sym_end] = ACTIONS(4514),
    [sym_identifier] = ACTIONS(4516),
    [anon_sym_SEMI] = ACTIONS(4514),
    [anon_sym_macro] = ACTIONS(4516),
    [anon_sym_LPAREN] = ACTIONS(4514),
    [anon_sym_LBRACE] = ACTIONS(4514),
    [anon_sym_RBRACE] = ACTIONS(4514),
    [anon_sym_macro_rules] = ACTIONS(4516),
    [anon_sym_BANG] = ACTIONS(4514),
    [anon_sym_LBRACK] = ACTIONS(4514),
    [anon_sym_STAR] = ACTIONS(4514),
    [anon_sym_u8] = ACTIONS(4516),
    [anon_sym_i8] = ACTIONS(4516),
    [anon_sym_u16] = ACTIONS(4516),
    [anon_sym_i16] = ACTIONS(4516),
    [anon_sym_u32] = ACTIONS(4516),
    [anon_sym_i32] = ACTIONS(4516),
    [anon_sym_u64] = ACTIONS(4516),
    [anon_sym_i64] = ACTIONS(4516),
    [anon_sym_u128] = ACTIONS(4516),
    [anon_sym_i128] = ACTIONS(4516),
    [anon_sym_isize] = ACTIONS(4516),
    [anon_sym_usize] = ACTIONS(4516),
    [anon_sym_f32] = ACTIONS(4516),
    [anon_sym_f64] = ACTIONS(4516),
    [anon_sym_bool] = ACTIONS(4516),
    [anon_sym_str] = ACTIONS(4516),
    [anon_sym_char] = ACTIONS(4516),
    [anon_sym_DASH] = ACTIONS(4514),
    [anon_sym_AMP] = ACTIONS(4514),
    [anon_sym_PIPE] = ACTIONS(4514),
    [anon_sym_LT] = ACTIONS(4514),
    [anon_sym_DOT_DOT] = ACTIONS(4514),
    [anon_sym_COLON_COLON] = ACTIONS(4514),
    [anon_sym_POUND] = ACTIONS(4514),
    [anon_sym_SQUOTE] = ACTIONS(4516),
    [anon_sym_async] = ACTIONS(4516),
    [anon_sym_auto] = ACTIONS(4516),
    [anon_sym_await] = ACTIONS(4516),
    [anon_sym_become] = ACTIONS(4516),
    [anon_sym_break] = ACTIONS(4516),
    [anon_sym_const] = ACTIONS(4516),
    [anon_sym_continue] = ACTIONS(4516),
    [anon_sym_default] = ACTIONS(4516),
    [anon_sym_enum] = ACTIONS(4516),
    [anon_sym_fn] = ACTIONS(4516),
    [anon_sym_for] = ACTIONS(4516),
    [anon_sym_gen] = ACTIONS(4516),
    [anon_sym_if] = ACTIONS(4516),
    [anon_sym_impl] = ACTIONS(4516),
    [anon_sym_let] = ACTIONS(4516),
    [anon_sym_loop] = ACTIONS(4516),
    [anon_sym_match] = ACTIONS(4516),
    [anon_sym_mod] = ACTIONS(4516),
    [anon_sym_pub] = ACTIONS(4516),
    [anon_sym_return] = ACTIONS(4516),
    [anon_sym_raw] = ACTIONS(4516),
    [anon_sym_static] = ACTIONS(4516),
    [anon_sym_struct] = ACTIONS(4516),
    [anon_sym_trait] = ACTIONS(4516),
    [anon_sym_type] = ACTIONS(4516),
    [anon_sym_union] = ACTIONS(4516),
    [anon_sym_unsafe] = ACTIONS(4516),
    [anon_sym_use] = ACTIONS(4516),
    [anon_sym_while] = ACTIONS(4516),
    [anon_sym_extern] = ACTIONS(4516),
    [anon_sym_safe] = ACTIONS(4516),
    [anon_sym_yield] = ACTIONS(4516),
    [anon_sym_move] = ACTIONS(4516),
    [anon_sym_try] = ACTIONS(4516),
    [sym_integer_literal] = ACTIONS(4514),
    [aux_sym_string_literal_token1] = ACTIONS(4514),
    [sym_char_literal] = ACTIONS(4514),
    [anon_sym_true] = ACTIONS(4516),
    [anon_sym_false] = ACTIONS(4516),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4516),
    [sym_super] = ACTIONS(4516),
    [sym_crate] = ACTIONS(4516),
    [aux_sym_metavariable_token1] = ACTIONS(4514),
    [sym__raw_string_literal_start] = ACTIONS(4514),
    [sym_float_literal] = ACTIONS(4514),
    [sym__outer_block_doc_comment_start] = ACTIONS(4514),
    [sym__outer_line_doc_comment_start] = ACTIONS(4514),
  },
  [STATE(1195)] = {
    [sym_line_comment] = STATE(1195),
    [sym_block_comment] = STATE(1195),
    [ts_builtin_sym_end] = ACTIONS(4518),
    [sym_identifier] = ACTIONS(4520),
    [anon_sym_SEMI] = ACTIONS(4518),
    [anon_sym_macro] = ACTIONS(4520),
    [anon_sym_LPAREN] = ACTIONS(4518),
    [anon_sym_LBRACE] = ACTIONS(4518),
    [anon_sym_RBRACE] = ACTIONS(4518),
    [anon_sym_macro_rules] = ACTIONS(4520),
    [anon_sym_BANG] = ACTIONS(4518),
    [anon_sym_LBRACK] = ACTIONS(4518),
    [anon_sym_STAR] = ACTIONS(4518),
    [anon_sym_u8] = ACTIONS(4520),
    [anon_sym_i8] = ACTIONS(4520),
    [anon_sym_u16] = ACTIONS(4520),
    [anon_sym_i16] = ACTIONS(4520),
    [anon_sym_u32] = ACTIONS(4520),
    [anon_sym_i32] = ACTIONS(4520),
    [anon_sym_u64] = ACTIONS(4520),
    [anon_sym_i64] = ACTIONS(4520),
    [anon_sym_u128] = ACTIONS(4520),
    [anon_sym_i128] = ACTIONS(4520),
    [anon_sym_isize] = ACTIONS(4520),
    [anon_sym_usize] = ACTIONS(4520),
    [anon_sym_f32] = ACTIONS(4520),
    [anon_sym_f64] = ACTIONS(4520),
    [anon_sym_bool] = ACTIONS(4520),
    [anon_sym_str] = ACTIONS(4520),
    [anon_sym_char] = ACTIONS(4520),
    [anon_sym_DASH] = ACTIONS(4518),
    [anon_sym_AMP] = ACTIONS(4518),
    [anon_sym_PIPE] = ACTIONS(4518),
    [anon_sym_LT] = ACTIONS(4518),
    [anon_sym_DOT_DOT] = ACTIONS(4518),
    [anon_sym_COLON_COLON] = ACTIONS(4518),
    [anon_sym_POUND] = ACTIONS(4518),
    [anon_sym_SQUOTE] = ACTIONS(4520),
    [anon_sym_async] = ACTIONS(4520),
    [anon_sym_auto] = ACTIONS(4520),
    [anon_sym_await] = ACTIONS(4520),
    [anon_sym_become] = ACTIONS(4520),
    [anon_sym_break] = ACTIONS(4520),
    [anon_sym_const] = ACTIONS(4520),
    [anon_sym_continue] = ACTIONS(4520),
    [anon_sym_default] = ACTIONS(4520),
    [anon_sym_enum] = ACTIONS(4520),
    [anon_sym_fn] = ACTIONS(4520),
    [anon_sym_for] = ACTIONS(4520),
    [anon_sym_gen] = ACTIONS(4520),
    [anon_sym_if] = ACTIONS(4520),
    [anon_sym_impl] = ACTIONS(4520),
    [anon_sym_let] = ACTIONS(4520),
    [anon_sym_loop] = ACTIONS(4520),
    [anon_sym_match] = ACTIONS(4520),
    [anon_sym_mod] = ACTIONS(4520),
    [anon_sym_pub] = ACTIONS(4520),
    [anon_sym_return] = ACTIONS(4520),
    [anon_sym_raw] = ACTIONS(4520),
    [anon_sym_static] = ACTIONS(4520),
    [anon_sym_struct] = ACTIONS(4520),
    [anon_sym_trait] = ACTIONS(4520),
    [anon_sym_type] = ACTIONS(4520),
    [anon_sym_union] = ACTIONS(4520),
    [anon_sym_unsafe] = ACTIONS(4520),
    [anon_sym_use] = ACTIONS(4520),
    [anon_sym_while] = ACTIONS(4520),
    [anon_sym_extern] = ACTIONS(4520),
    [anon_sym_safe] = ACTIONS(4520),
    [anon_sym_yield] = ACTIONS(4520),
    [anon_sym_move] = ACTIONS(4520),
    [anon_sym_try] = ACTIONS(4520),
    [sym_integer_literal] = ACTIONS(4518),
    [aux_sym_string_literal_token1] = ACTIONS(4518),
    [sym_char_literal] = ACTIONS(4518),
    [anon_sym_true] = ACTIONS(4520),
    [anon_sym_false] = ACTIONS(4520),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4520),
    [sym_super] = ACTIONS(4520),
    [sym_crate] = ACTIONS(4520),
    [aux_sym_metavariable_token1] = ACTIONS(4518),
    [sym__raw_string_literal_start] = ACTIONS(4518),
    [sym_float_literal] = ACTIONS(4518),
    [sym__outer_block_doc_comment_start] = ACTIONS(4518),
    [sym__outer_line_doc_comment_start] = ACTIONS(4518),
  },
  [STATE(1196)] = {
    [sym_line_comment] = STATE(1196),
    [sym_block_comment] = STATE(1196),
    [ts_builtin_sym_end] = ACTIONS(4522),
    [sym_identifier] = ACTIONS(4524),
    [anon_sym_SEMI] = ACTIONS(4522),
    [anon_sym_macro] = ACTIONS(4524),
    [anon_sym_LPAREN] = ACTIONS(4522),
    [anon_sym_LBRACE] = ACTIONS(4522),
    [anon_sym_RBRACE] = ACTIONS(4522),
    [anon_sym_macro_rules] = ACTIONS(4524),
    [anon_sym_BANG] = ACTIONS(4522),
    [anon_sym_LBRACK] = ACTIONS(4522),
    [anon_sym_STAR] = ACTIONS(4522),
    [anon_sym_u8] = ACTIONS(4524),
    [anon_sym_i8] = ACTIONS(4524),
    [anon_sym_u16] = ACTIONS(4524),
    [anon_sym_i16] = ACTIONS(4524),
    [anon_sym_u32] = ACTIONS(4524),
    [anon_sym_i32] = ACTIONS(4524),
    [anon_sym_u64] = ACTIONS(4524),
    [anon_sym_i64] = ACTIONS(4524),
    [anon_sym_u128] = ACTIONS(4524),
    [anon_sym_i128] = ACTIONS(4524),
    [anon_sym_isize] = ACTIONS(4524),
    [anon_sym_usize] = ACTIONS(4524),
    [anon_sym_f32] = ACTIONS(4524),
    [anon_sym_f64] = ACTIONS(4524),
    [anon_sym_bool] = ACTIONS(4524),
    [anon_sym_str] = ACTIONS(4524),
    [anon_sym_char] = ACTIONS(4524),
    [anon_sym_DASH] = ACTIONS(4522),
    [anon_sym_AMP] = ACTIONS(4522),
    [anon_sym_PIPE] = ACTIONS(4522),
    [anon_sym_LT] = ACTIONS(4522),
    [anon_sym_DOT_DOT] = ACTIONS(4522),
    [anon_sym_COLON_COLON] = ACTIONS(4522),
    [anon_sym_POUND] = ACTIONS(4522),
    [anon_sym_SQUOTE] = ACTIONS(4524),
    [anon_sym_async] = ACTIONS(4524),
    [anon_sym_auto] = ACTIONS(4524),
    [anon_sym_await] = ACTIONS(4524),
    [anon_sym_become] = ACTIONS(4524),
    [anon_sym_break] = ACTIONS(4524),
    [anon_sym_const] = ACTIONS(4524),
    [anon_sym_continue] = ACTIONS(4524),
    [anon_sym_default] = ACTIONS(4524),
    [anon_sym_enum] = ACTIONS(4524),
    [anon_sym_fn] = ACTIONS(4524),
    [anon_sym_for] = ACTIONS(4524),
    [anon_sym_gen] = ACTIONS(4524),
    [anon_sym_if] = ACTIONS(4524),
    [anon_sym_impl] = ACTIONS(4524),
    [anon_sym_let] = ACTIONS(4524),
    [anon_sym_loop] = ACTIONS(4524),
    [anon_sym_match] = ACTIONS(4524),
    [anon_sym_mod] = ACTIONS(4524),
    [anon_sym_pub] = ACTIONS(4524),
    [anon_sym_return] = ACTIONS(4524),
    [anon_sym_raw] = ACTIONS(4524),
    [anon_sym_static] = ACTIONS(4524),
    [anon_sym_struct] = ACTIONS(4524),
    [anon_sym_trait] = ACTIONS(4524),
    [anon_sym_type] = ACTIONS(4524),
    [anon_sym_union] = ACTIONS(4524),
    [anon_sym_unsafe] = ACTIONS(4524),
    [anon_sym_use] = ACTIONS(4524),
    [anon_sym_while] = ACTIONS(4524),
    [anon_sym_extern] = ACTIONS(4524),
    [anon_sym_safe] = ACTIONS(4524),
    [anon_sym_yield] = ACTIONS(4524),
    [anon_sym_move] = ACTIONS(4524),
    [anon_sym_try] = ACTIONS(4524),
    [sym_integer_literal] = ACTIONS(4522),
    [aux_sym_string_literal_token1] = ACTIONS(4522),
    [sym_char_literal] = ACTIONS(4522),
    [anon_sym_true] = ACTIONS(4524),
    [anon_sym_false] = ACTIONS(4524),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4524),
    [sym_super] = ACTIONS(4524),
    [sym_crate] = ACTIONS(4524),
    [aux_sym_metavariable_token1] = ACTIONS(4522),
    [sym__raw_string_literal_start] = ACTIONS(4522),
    [sym_float_literal] = ACTIONS(4522),
    [sym__outer_block_doc_comment_start] = ACTIONS(4522),
    [sym__outer_line_doc_comment_start] = ACTIONS(4522),
  },
  [STATE(1197)] = {
    [sym_line_comment] = STATE(1197),
    [sym_block_comment] = STATE(1197),
    [ts_builtin_sym_end] = ACTIONS(4526),
    [sym_identifier] = ACTIONS(4528),
    [anon_sym_SEMI] = ACTIONS(4526),
    [anon_sym_macro] = ACTIONS(4528),
    [anon_sym_LPAREN] = ACTIONS(4526),
    [anon_sym_LBRACE] = ACTIONS(4526),
    [anon_sym_RBRACE] = ACTIONS(4526),
    [anon_sym_macro_rules] = ACTIONS(4528),
    [anon_sym_BANG] = ACTIONS(4526),
    [anon_sym_LBRACK] = ACTIONS(4526),
    [anon_sym_STAR] = ACTIONS(4526),
    [anon_sym_u8] = ACTIONS(4528),
    [anon_sym_i8] = ACTIONS(4528),
    [anon_sym_u16] = ACTIONS(4528),
    [anon_sym_i16] = ACTIONS(4528),
    [anon_sym_u32] = ACTIONS(4528),
    [anon_sym_i32] = ACTIONS(4528),
    [anon_sym_u64] = ACTIONS(4528),
    [anon_sym_i64] = ACTIONS(4528),
    [anon_sym_u128] = ACTIONS(4528),
    [anon_sym_i128] = ACTIONS(4528),
    [anon_sym_isize] = ACTIONS(4528),
    [anon_sym_usize] = ACTIONS(4528),
    [anon_sym_f32] = ACTIONS(4528),
    [anon_sym_f64] = ACTIONS(4528),
    [anon_sym_bool] = ACTIONS(4528),
    [anon_sym_str] = ACTIONS(4528),
    [anon_sym_char] = ACTIONS(4528),
    [anon_sym_DASH] = ACTIONS(4526),
    [anon_sym_AMP] = ACTIONS(4526),
    [anon_sym_PIPE] = ACTIONS(4526),
    [anon_sym_LT] = ACTIONS(4526),
    [anon_sym_DOT_DOT] = ACTIONS(4526),
    [anon_sym_COLON_COLON] = ACTIONS(4526),
    [anon_sym_POUND] = ACTIONS(4526),
    [anon_sym_SQUOTE] = ACTIONS(4528),
    [anon_sym_async] = ACTIONS(4528),
    [anon_sym_auto] = ACTIONS(4528),
    [anon_sym_await] = ACTIONS(4528),
    [anon_sym_become] = ACTIONS(4528),
    [anon_sym_break] = ACTIONS(4528),
    [anon_sym_const] = ACTIONS(4528),
    [anon_sym_continue] = ACTIONS(4528),
    [anon_sym_default] = ACTIONS(4528),
    [anon_sym_enum] = ACTIONS(4528),
    [anon_sym_fn] = ACTIONS(4528),
    [anon_sym_for] = ACTIONS(4528),
    [anon_sym_gen] = ACTIONS(4528),
    [anon_sym_if] = ACTIONS(4528),
    [anon_sym_impl] = ACTIONS(4528),
    [anon_sym_let] = ACTIONS(4528),
    [anon_sym_loop] = ACTIONS(4528),
    [anon_sym_match] = ACTIONS(4528),
    [anon_sym_mod] = ACTIONS(4528),
    [anon_sym_pub] = ACTIONS(4528),
    [anon_sym_return] = ACTIONS(4528),
    [anon_sym_raw] = ACTIONS(4528),
    [anon_sym_static] = ACTIONS(4528),
    [anon_sym_struct] = ACTIONS(4528),
    [anon_sym_trait] = ACTIONS(4528),
    [anon_sym_type] = ACTIONS(4528),
    [anon_sym_union] = ACTIONS(4528),
    [anon_sym_unsafe] = ACTIONS(4528),
    [anon_sym_use] = ACTIONS(4528),
    [anon_sym_while] = ACTIONS(4528),
    [anon_sym_extern] = ACTIONS(4528),
    [anon_sym_safe] = ACTIONS(4528),
    [anon_sym_yield] = ACTIONS(4528),
    [anon_sym_move] = ACTIONS(4528),
    [anon_sym_try] = ACTIONS(4528),
    [sym_integer_literal] = ACTIONS(4526),
    [aux_sym_string_literal_token1] = ACTIONS(4526),
    [sym_char_literal] = ACTIONS(4526),
    [anon_sym_true] = ACTIONS(4528),
    [anon_sym_false] = ACTIONS(4528),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4528),
    [sym_super] = ACTIONS(4528),
    [sym_crate] = ACTIONS(4528),
    [aux_sym_metavariable_token1] = ACTIONS(4526),
    [sym__raw_string_literal_start] = ACTIONS(4526),
    [sym_float_literal] = ACTIONS(4526),
    [sym__outer_block_doc_comment_start] = ACTIONS(4526),
    [sym__outer_line_doc_comment_start] = ACTIONS(4526),
  },
  [STATE(1198)] = {
    [sym_line_comment] = STATE(1198),
    [sym_block_comment] = STATE(1198),
    [ts_builtin_sym_end] = ACTIONS(4530),
    [sym_identifier] = ACTIONS(4532),
    [anon_sym_SEMI] = ACTIONS(4530),
    [anon_sym_macro] = ACTIONS(4532),
    [anon_sym_LPAREN] = ACTIONS(4530),
    [anon_sym_LBRACE] = ACTIONS(4530),
    [anon_sym_RBRACE] = ACTIONS(4530),
    [anon_sym_macro_rules] = ACTIONS(4532),
    [anon_sym_BANG] = ACTIONS(4530),
    [anon_sym_LBRACK] = ACTIONS(4530),
    [anon_sym_STAR] = ACTIONS(4530),
    [anon_sym_u8] = ACTIONS(4532),
    [anon_sym_i8] = ACTIONS(4532),
    [anon_sym_u16] = ACTIONS(4532),
    [anon_sym_i16] = ACTIONS(4532),
    [anon_sym_u32] = ACTIONS(4532),
    [anon_sym_i32] = ACTIONS(4532),
    [anon_sym_u64] = ACTIONS(4532),
    [anon_sym_i64] = ACTIONS(4532),
    [anon_sym_u128] = ACTIONS(4532),
    [anon_sym_i128] = ACTIONS(4532),
    [anon_sym_isize] = ACTIONS(4532),
    [anon_sym_usize] = ACTIONS(4532),
    [anon_sym_f32] = ACTIONS(4532),
    [anon_sym_f64] = ACTIONS(4532),
    [anon_sym_bool] = ACTIONS(4532),
    [anon_sym_str] = ACTIONS(4532),
    [anon_sym_char] = ACTIONS(4532),
    [anon_sym_DASH] = ACTIONS(4530),
    [anon_sym_AMP] = ACTIONS(4530),
    [anon_sym_PIPE] = ACTIONS(4530),
    [anon_sym_LT] = ACTIONS(4530),
    [anon_sym_DOT_DOT] = ACTIONS(4530),
    [anon_sym_COLON_COLON] = ACTIONS(4530),
    [anon_sym_POUND] = ACTIONS(4530),
    [anon_sym_SQUOTE] = ACTIONS(4532),
    [anon_sym_async] = ACTIONS(4532),
    [anon_sym_auto] = ACTIONS(4532),
    [anon_sym_await] = ACTIONS(4532),
    [anon_sym_become] = ACTIONS(4532),
    [anon_sym_break] = ACTIONS(4532),
    [anon_sym_const] = ACTIONS(4532),
    [anon_sym_continue] = ACTIONS(4532),
    [anon_sym_default] = ACTIONS(4532),
    [anon_sym_enum] = ACTIONS(4532),
    [anon_sym_fn] = ACTIONS(4532),
    [anon_sym_for] = ACTIONS(4532),
    [anon_sym_gen] = ACTIONS(4532),
    [anon_sym_if] = ACTIONS(4532),
    [anon_sym_impl] = ACTIONS(4532),
    [anon_sym_let] = ACTIONS(4532),
    [anon_sym_loop] = ACTIONS(4532),
    [anon_sym_match] = ACTIONS(4532),
    [anon_sym_mod] = ACTIONS(4532),
    [anon_sym_pub] = ACTIONS(4532),
    [anon_sym_return] = ACTIONS(4532),
    [anon_sym_raw] = ACTIONS(4532),
    [anon_sym_static] = ACTIONS(4532),
    [anon_sym_struct] = ACTIONS(4532),
    [anon_sym_trait] = ACTIONS(4532),
    [anon_sym_type] = ACTIONS(4532),
    [anon_sym_union] = ACTIONS(4532),
    [anon_sym_unsafe] = ACTIONS(4532),
    [anon_sym_use] = ACTIONS(4532),
    [anon_sym_while] = ACTIONS(4532),
    [anon_sym_extern] = ACTIONS(4532),
    [anon_sym_safe] = ACTIONS(4532),
    [anon_sym_yield] = ACTIONS(4532),
    [anon_sym_move] = ACTIONS(4532),
    [anon_sym_try] = ACTIONS(4532),
    [sym_integer_literal] = ACTIONS(4530),
    [aux_sym_string_literal_token1] = ACTIONS(4530),
    [sym_char_literal] = ACTIONS(4530),
    [anon_sym_true] = ACTIONS(4532),
    [anon_sym_false] = ACTIONS(4532),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4532),
    [sym_super] = ACTIONS(4532),
    [sym_crate] = ACTIONS(4532),
    [aux_sym_metavariable_token1] = ACTIONS(4530),
    [sym__raw_string_literal_start] = ACTIONS(4530),
    [sym_float_literal] = ACTIONS(4530),
    [sym__outer_block_doc_comment_start] = ACTIONS(4530),
    [sym__outer_line_doc_comment_start] = ACTIONS(4530),
  },
  [STATE(1199)] = {
    [sym_line_comment] = STATE(1199),
    [sym_block_comment] = STATE(1199),
    [ts_builtin_sym_end] = ACTIONS(4534),
    [sym_identifier] = ACTIONS(4536),
    [anon_sym_SEMI] = ACTIONS(4534),
    [anon_sym_macro] = ACTIONS(4536),
    [anon_sym_LPAREN] = ACTIONS(4534),
    [anon_sym_LBRACE] = ACTIONS(4534),
    [anon_sym_RBRACE] = ACTIONS(4534),
    [anon_sym_macro_rules] = ACTIONS(4536),
    [anon_sym_BANG] = ACTIONS(4534),
    [anon_sym_LBRACK] = ACTIONS(4534),
    [anon_sym_STAR] = ACTIONS(4534),
    [anon_sym_u8] = ACTIONS(4536),
    [anon_sym_i8] = ACTIONS(4536),
    [anon_sym_u16] = ACTIONS(4536),
    [anon_sym_i16] = ACTIONS(4536),
    [anon_sym_u32] = ACTIONS(4536),
    [anon_sym_i32] = ACTIONS(4536),
    [anon_sym_u64] = ACTIONS(4536),
    [anon_sym_i64] = ACTIONS(4536),
    [anon_sym_u128] = ACTIONS(4536),
    [anon_sym_i128] = ACTIONS(4536),
    [anon_sym_isize] = ACTIONS(4536),
    [anon_sym_usize] = ACTIONS(4536),
    [anon_sym_f32] = ACTIONS(4536),
    [anon_sym_f64] = ACTIONS(4536),
    [anon_sym_bool] = ACTIONS(4536),
    [anon_sym_str] = ACTIONS(4536),
    [anon_sym_char] = ACTIONS(4536),
    [anon_sym_DASH] = ACTIONS(4534),
    [anon_sym_AMP] = ACTIONS(4534),
    [anon_sym_PIPE] = ACTIONS(4534),
    [anon_sym_LT] = ACTIONS(4534),
    [anon_sym_DOT_DOT] = ACTIONS(4534),
    [anon_sym_COLON_COLON] = ACTIONS(4534),
    [anon_sym_POUND] = ACTIONS(4534),
    [anon_sym_SQUOTE] = ACTIONS(4536),
    [anon_sym_async] = ACTIONS(4536),
    [anon_sym_auto] = ACTIONS(4536),
    [anon_sym_await] = ACTIONS(4536),
    [anon_sym_become] = ACTIONS(4536),
    [anon_sym_break] = ACTIONS(4536),
    [anon_sym_const] = ACTIONS(4536),
    [anon_sym_continue] = ACTIONS(4536),
    [anon_sym_default] = ACTIONS(4536),
    [anon_sym_enum] = ACTIONS(4536),
    [anon_sym_fn] = ACTIONS(4536),
    [anon_sym_for] = ACTIONS(4536),
    [anon_sym_gen] = ACTIONS(4536),
    [anon_sym_if] = ACTIONS(4536),
    [anon_sym_impl] = ACTIONS(4536),
    [anon_sym_let] = ACTIONS(4536),
    [anon_sym_loop] = ACTIONS(4536),
    [anon_sym_match] = ACTIONS(4536),
    [anon_sym_mod] = ACTIONS(4536),
    [anon_sym_pub] = ACTIONS(4536),
    [anon_sym_return] = ACTIONS(4536),
    [anon_sym_raw] = ACTIONS(4536),
    [anon_sym_static] = ACTIONS(4536),
    [anon_sym_struct] = ACTIONS(4536),
    [anon_sym_trait] = ACTIONS(4536),
    [anon_sym_type] = ACTIONS(4536),
    [anon_sym_union] = ACTIONS(4536),
    [anon_sym_unsafe] = ACTIONS(4536),
    [anon_sym_use] = ACTIONS(4536),
    [anon_sym_while] = ACTIONS(4536),
    [anon_sym_extern] = ACTIONS(4536),
    [anon_sym_safe] = ACTIONS(4536),
    [anon_sym_yield] = ACTIONS(4536),
    [anon_sym_move] = ACTIONS(4536),
    [anon_sym_try] = ACTIONS(4536),
    [sym_integer_literal] = ACTIONS(4534),
    [aux_sym_string_literal_token1] = ACTIONS(4534),
    [sym_char_literal] = ACTIONS(4534),
    [anon_sym_true] = ACTIONS(4536),
    [anon_sym_false] = ACTIONS(4536),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4536),
    [sym_super] = ACTIONS(4536),
    [sym_crate] = ACTIONS(4536),
    [aux_sym_metavariable_token1] = ACTIONS(4534),
    [sym__raw_string_literal_start] = ACTIONS(4534),
    [sym_float_literal] = ACTIONS(4534),
    [sym__outer_block_doc_comment_start] = ACTIONS(4534),
    [sym__outer_line_doc_comment_start] = ACTIONS(4534),
  },
  [STATE(1200)] = {
    [sym_line_comment] = STATE(1200),
    [sym_block_comment] = STATE(1200),
    [ts_builtin_sym_end] = ACTIONS(4538),
    [sym_identifier] = ACTIONS(4540),
    [anon_sym_SEMI] = ACTIONS(4538),
    [anon_sym_macro] = ACTIONS(4540),
    [anon_sym_LPAREN] = ACTIONS(4538),
    [anon_sym_LBRACE] = ACTIONS(4538),
    [anon_sym_RBRACE] = ACTIONS(4538),
    [anon_sym_macro_rules] = ACTIONS(4540),
    [anon_sym_BANG] = ACTIONS(4538),
    [anon_sym_LBRACK] = ACTIONS(4538),
    [anon_sym_STAR] = ACTIONS(4538),
    [anon_sym_u8] = ACTIONS(4540),
    [anon_sym_i8] = ACTIONS(4540),
    [anon_sym_u16] = ACTIONS(4540),
    [anon_sym_i16] = ACTIONS(4540),
    [anon_sym_u32] = ACTIONS(4540),
    [anon_sym_i32] = ACTIONS(4540),
    [anon_sym_u64] = ACTIONS(4540),
    [anon_sym_i64] = ACTIONS(4540),
    [anon_sym_u128] = ACTIONS(4540),
    [anon_sym_i128] = ACTIONS(4540),
    [anon_sym_isize] = ACTIONS(4540),
    [anon_sym_usize] = ACTIONS(4540),
    [anon_sym_f32] = ACTIONS(4540),
    [anon_sym_f64] = ACTIONS(4540),
    [anon_sym_bool] = ACTIONS(4540),
    [anon_sym_str] = ACTIONS(4540),
    [anon_sym_char] = ACTIONS(4540),
    [anon_sym_DASH] = ACTIONS(4538),
    [anon_sym_AMP] = ACTIONS(4538),
    [anon_sym_PIPE] = ACTIONS(4538),
    [anon_sym_LT] = ACTIONS(4538),
    [anon_sym_DOT_DOT] = ACTIONS(4538),
    [anon_sym_COLON_COLON] = ACTIONS(4538),
    [anon_sym_POUND] = ACTIONS(4538),
    [anon_sym_SQUOTE] = ACTIONS(4540),
    [anon_sym_async] = ACTIONS(4540),
    [anon_sym_auto] = ACTIONS(4540),
    [anon_sym_await] = ACTIONS(4540),
    [anon_sym_become] = ACTIONS(4540),
    [anon_sym_break] = ACTIONS(4540),
    [anon_sym_const] = ACTIONS(4540),
    [anon_sym_continue] = ACTIONS(4540),
    [anon_sym_default] = ACTIONS(4540),
    [anon_sym_enum] = ACTIONS(4540),
    [anon_sym_fn] = ACTIONS(4540),
    [anon_sym_for] = ACTIONS(4540),
    [anon_sym_gen] = ACTIONS(4540),
    [anon_sym_if] = ACTIONS(4540),
    [anon_sym_impl] = ACTIONS(4540),
    [anon_sym_let] = ACTIONS(4540),
    [anon_sym_loop] = ACTIONS(4540),
    [anon_sym_match] = ACTIONS(4540),
    [anon_sym_mod] = ACTIONS(4540),
    [anon_sym_pub] = ACTIONS(4540),
    [anon_sym_return] = ACTIONS(4540),
    [anon_sym_raw] = ACTIONS(4540),
    [anon_sym_static] = ACTIONS(4540),
    [anon_sym_struct] = ACTIONS(4540),
    [anon_sym_trait] = ACTIONS(4540),
    [anon_sym_type] = ACTIONS(4540),
    [anon_sym_union] = ACTIONS(4540),
    [anon_sym_unsafe] = ACTIONS(4540),
    [anon_sym_use] = ACTIONS(4540),
    [anon_sym_while] = ACTIONS(4540),
    [anon_sym_extern] = ACTIONS(4540),
    [anon_sym_safe] = ACTIONS(4540),
    [anon_sym_yield] = ACTIONS(4540),
    [anon_sym_move] = ACTIONS(4540),
    [anon_sym_try] = ACTIONS(4540),
    [sym_integer_literal] = ACTIONS(4538),
    [aux_sym_string_literal_token1] = ACTIONS(4538),
    [sym_char_literal] = ACTIONS(4538),
    [anon_sym_true] = ACTIONS(4540),
    [anon_sym_false] = ACTIONS(4540),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4540),
    [sym_super] = ACTIONS(4540),
    [sym_crate] = ACTIONS(4540),
    [aux_sym_metavariable_token1] = ACTIONS(4538),
    [sym__raw_string_literal_start] = ACTIONS(4538),
    [sym_float_literal] = ACTIONS(4538),
    [sym__outer_block_doc_comment_start] = ACTIONS(4538),
    [sym__outer_line_doc_comment_start] = ACTIONS(4538),
  },
  [STATE(1201)] = {
    [sym_line_comment] = STATE(1201),
    [sym_block_comment] = STATE(1201),
    [ts_builtin_sym_end] = ACTIONS(4542),
    [sym_identifier] = ACTIONS(4544),
    [anon_sym_SEMI] = ACTIONS(4542),
    [anon_sym_macro] = ACTIONS(4544),
    [anon_sym_LPAREN] = ACTIONS(4542),
    [anon_sym_LBRACE] = ACTIONS(4542),
    [anon_sym_RBRACE] = ACTIONS(4542),
    [anon_sym_macro_rules] = ACTIONS(4544),
    [anon_sym_BANG] = ACTIONS(4542),
    [anon_sym_LBRACK] = ACTIONS(4542),
    [anon_sym_STAR] = ACTIONS(4542),
    [anon_sym_u8] = ACTIONS(4544),
    [anon_sym_i8] = ACTIONS(4544),
    [anon_sym_u16] = ACTIONS(4544),
    [anon_sym_i16] = ACTIONS(4544),
    [anon_sym_u32] = ACTIONS(4544),
    [anon_sym_i32] = ACTIONS(4544),
    [anon_sym_u64] = ACTIONS(4544),
    [anon_sym_i64] = ACTIONS(4544),
    [anon_sym_u128] = ACTIONS(4544),
    [anon_sym_i128] = ACTIONS(4544),
    [anon_sym_isize] = ACTIONS(4544),
    [anon_sym_usize] = ACTIONS(4544),
    [anon_sym_f32] = ACTIONS(4544),
    [anon_sym_f64] = ACTIONS(4544),
    [anon_sym_bool] = ACTIONS(4544),
    [anon_sym_str] = ACTIONS(4544),
    [anon_sym_char] = ACTIONS(4544),
    [anon_sym_DASH] = ACTIONS(4542),
    [anon_sym_AMP] = ACTIONS(4542),
    [anon_sym_PIPE] = ACTIONS(4542),
    [anon_sym_LT] = ACTIONS(4542),
    [anon_sym_DOT_DOT] = ACTIONS(4542),
    [anon_sym_COLON_COLON] = ACTIONS(4542),
    [anon_sym_POUND] = ACTIONS(4542),
    [anon_sym_SQUOTE] = ACTIONS(4544),
    [anon_sym_async] = ACTIONS(4544),
    [anon_sym_auto] = ACTIONS(4544),
    [anon_sym_await] = ACTIONS(4544),
    [anon_sym_become] = ACTIONS(4544),
    [anon_sym_break] = ACTIONS(4544),
    [anon_sym_const] = ACTIONS(4544),
    [anon_sym_continue] = ACTIONS(4544),
    [anon_sym_default] = ACTIONS(4544),
    [anon_sym_enum] = ACTIONS(4544),
    [anon_sym_fn] = ACTIONS(4544),
    [anon_sym_for] = ACTIONS(4544),
    [anon_sym_gen] = ACTIONS(4544),
    [anon_sym_if] = ACTIONS(4544),
    [anon_sym_impl] = ACTIONS(4544),
    [anon_sym_let] = ACTIONS(4544),
    [anon_sym_loop] = ACTIONS(4544),
    [anon_sym_match] = ACTIONS(4544),
    [anon_sym_mod] = ACTIONS(4544),
    [anon_sym_pub] = ACTIONS(4544),
    [anon_sym_return] = ACTIONS(4544),
    [anon_sym_raw] = ACTIONS(4544),
    [anon_sym_static] = ACTIONS(4544),
    [anon_sym_struct] = ACTIONS(4544),
    [anon_sym_trait] = ACTIONS(4544),
    [anon_sym_type] = ACTIONS(4544),
    [anon_sym_union] = ACTIONS(4544),
    [anon_sym_unsafe] = ACTIONS(4544),
    [anon_sym_use] = ACTIONS(4544),
    [anon_sym_while] = ACTIONS(4544),
    [anon_sym_extern] = ACTIONS(4544),
    [anon_sym_safe] = ACTIONS(4544),
    [anon_sym_yield] = ACTIONS(4544),
    [anon_sym_move] = ACTIONS(4544),
    [anon_sym_try] = ACTIONS(4544),
    [sym_integer_literal] = ACTIONS(4542),
    [aux_sym_string_literal_token1] = ACTIONS(4542),
    [sym_char_literal] = ACTIONS(4542),
    [anon_sym_true] = ACTIONS(4544),
    [anon_sym_false] = ACTIONS(4544),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4544),
    [sym_super] = ACTIONS(4544),
    [sym_crate] = ACTIONS(4544),
    [aux_sym_metavariable_token1] = ACTIONS(4542),
    [sym__raw_string_literal_start] = ACTIONS(4542),
    [sym_float_literal] = ACTIONS(4542),
    [sym__outer_block_doc_comment_start] = ACTIONS(4542),
    [sym__outer_line_doc_comment_start] = ACTIONS(4542),
  },
  [STATE(1202)] = {
    [sym_line_comment] = STATE(1202),
    [sym_block_comment] = STATE(1202),
    [ts_builtin_sym_end] = ACTIONS(4546),
    [sym_identifier] = ACTIONS(4548),
    [anon_sym_SEMI] = ACTIONS(4546),
    [anon_sym_macro] = ACTIONS(4548),
    [anon_sym_LPAREN] = ACTIONS(4546),
    [anon_sym_LBRACE] = ACTIONS(4546),
    [anon_sym_RBRACE] = ACTIONS(4546),
    [anon_sym_macro_rules] = ACTIONS(4548),
    [anon_sym_BANG] = ACTIONS(4546),
    [anon_sym_LBRACK] = ACTIONS(4546),
    [anon_sym_STAR] = ACTIONS(4546),
    [anon_sym_u8] = ACTIONS(4548),
    [anon_sym_i8] = ACTIONS(4548),
    [anon_sym_u16] = ACTIONS(4548),
    [anon_sym_i16] = ACTIONS(4548),
    [anon_sym_u32] = ACTIONS(4548),
    [anon_sym_i32] = ACTIONS(4548),
    [anon_sym_u64] = ACTIONS(4548),
    [anon_sym_i64] = ACTIONS(4548),
    [anon_sym_u128] = ACTIONS(4548),
    [anon_sym_i128] = ACTIONS(4548),
    [anon_sym_isize] = ACTIONS(4548),
    [anon_sym_usize] = ACTIONS(4548),
    [anon_sym_f32] = ACTIONS(4548),
    [anon_sym_f64] = ACTIONS(4548),
    [anon_sym_bool] = ACTIONS(4548),
    [anon_sym_str] = ACTIONS(4548),
    [anon_sym_char] = ACTIONS(4548),
    [anon_sym_DASH] = ACTIONS(4546),
    [anon_sym_AMP] = ACTIONS(4546),
    [anon_sym_PIPE] = ACTIONS(4546),
    [anon_sym_LT] = ACTIONS(4546),
    [anon_sym_DOT_DOT] = ACTIONS(4546),
    [anon_sym_COLON_COLON] = ACTIONS(4546),
    [anon_sym_POUND] = ACTIONS(4546),
    [anon_sym_SQUOTE] = ACTIONS(4548),
    [anon_sym_async] = ACTIONS(4548),
    [anon_sym_auto] = ACTIONS(4548),
    [anon_sym_await] = ACTIONS(4548),
    [anon_sym_become] = ACTIONS(4548),
    [anon_sym_break] = ACTIONS(4548),
    [anon_sym_const] = ACTIONS(4548),
    [anon_sym_continue] = ACTIONS(4548),
    [anon_sym_default] = ACTIONS(4548),
    [anon_sym_enum] = ACTIONS(4548),
    [anon_sym_fn] = ACTIONS(4548),
    [anon_sym_for] = ACTIONS(4548),
    [anon_sym_gen] = ACTIONS(4548),
    [anon_sym_if] = ACTIONS(4548),
    [anon_sym_impl] = ACTIONS(4548),
    [anon_sym_let] = ACTIONS(4548),
    [anon_sym_loop] = ACTIONS(4548),
    [anon_sym_match] = ACTIONS(4548),
    [anon_sym_mod] = ACTIONS(4548),
    [anon_sym_pub] = ACTIONS(4548),
    [anon_sym_return] = ACTIONS(4548),
    [anon_sym_raw] = ACTIONS(4548),
    [anon_sym_static] = ACTIONS(4548),
    [anon_sym_struct] = ACTIONS(4548),
    [anon_sym_trait] = ACTIONS(4548),
    [anon_sym_type] = ACTIONS(4548),
    [anon_sym_union] = ACTIONS(4548),
    [anon_sym_unsafe] = ACTIONS(4548),
    [anon_sym_use] = ACTIONS(4548),
    [anon_sym_while] = ACTIONS(4548),
    [anon_sym_extern] = ACTIONS(4548),
    [anon_sym_safe] = ACTIONS(4548),
    [anon_sym_yield] = ACTIONS(4548),
    [anon_sym_move] = ACTIONS(4548),
    [anon_sym_try] = ACTIONS(4548),
    [sym_integer_literal] = ACTIONS(4546),
    [aux_sym_string_literal_token1] = ACTIONS(4546),
    [sym_char_literal] = ACTIONS(4546),
    [anon_sym_true] = ACTIONS(4548),
    [anon_sym_false] = ACTIONS(4548),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4548),
    [sym_super] = ACTIONS(4548),
    [sym_crate] = ACTIONS(4548),
    [aux_sym_metavariable_token1] = ACTIONS(4546),
    [sym__raw_string_literal_start] = ACTIONS(4546),
    [sym_float_literal] = ACTIONS(4546),
    [sym__outer_block_doc_comment_start] = ACTIONS(4546),
    [sym__outer_line_doc_comment_start] = ACTIONS(4546),
  },
  [STATE(1203)] = {
    [sym_line_comment] = STATE(1203),
    [sym_block_comment] = STATE(1203),
    [ts_builtin_sym_end] = ACTIONS(4550),
    [sym_identifier] = ACTIONS(4552),
    [anon_sym_SEMI] = ACTIONS(4550),
    [anon_sym_macro] = ACTIONS(4552),
    [anon_sym_LPAREN] = ACTIONS(4550),
    [anon_sym_LBRACE] = ACTIONS(4550),
    [anon_sym_RBRACE] = ACTIONS(4550),
    [anon_sym_macro_rules] = ACTIONS(4552),
    [anon_sym_BANG] = ACTIONS(4550),
    [anon_sym_LBRACK] = ACTIONS(4550),
    [anon_sym_STAR] = ACTIONS(4550),
    [anon_sym_u8] = ACTIONS(4552),
    [anon_sym_i8] = ACTIONS(4552),
    [anon_sym_u16] = ACTIONS(4552),
    [anon_sym_i16] = ACTIONS(4552),
    [anon_sym_u32] = ACTIONS(4552),
    [anon_sym_i32] = ACTIONS(4552),
    [anon_sym_u64] = ACTIONS(4552),
    [anon_sym_i64] = ACTIONS(4552),
    [anon_sym_u128] = ACTIONS(4552),
    [anon_sym_i128] = ACTIONS(4552),
    [anon_sym_isize] = ACTIONS(4552),
    [anon_sym_usize] = ACTIONS(4552),
    [anon_sym_f32] = ACTIONS(4552),
    [anon_sym_f64] = ACTIONS(4552),
    [anon_sym_bool] = ACTIONS(4552),
    [anon_sym_str] = ACTIONS(4552),
    [anon_sym_char] = ACTIONS(4552),
    [anon_sym_DASH] = ACTIONS(4550),
    [anon_sym_AMP] = ACTIONS(4550),
    [anon_sym_PIPE] = ACTIONS(4550),
    [anon_sym_LT] = ACTIONS(4550),
    [anon_sym_DOT_DOT] = ACTIONS(4550),
    [anon_sym_COLON_COLON] = ACTIONS(4550),
    [anon_sym_POUND] = ACTIONS(4550),
    [anon_sym_SQUOTE] = ACTIONS(4552),
    [anon_sym_async] = ACTIONS(4552),
    [anon_sym_auto] = ACTIONS(4552),
    [anon_sym_await] = ACTIONS(4552),
    [anon_sym_become] = ACTIONS(4552),
    [anon_sym_break] = ACTIONS(4552),
    [anon_sym_const] = ACTIONS(4552),
    [anon_sym_continue] = ACTIONS(4552),
    [anon_sym_default] = ACTIONS(4552),
    [anon_sym_enum] = ACTIONS(4552),
    [anon_sym_fn] = ACTIONS(4552),
    [anon_sym_for] = ACTIONS(4552),
    [anon_sym_gen] = ACTIONS(4552),
    [anon_sym_if] = ACTIONS(4552),
    [anon_sym_impl] = ACTIONS(4552),
    [anon_sym_let] = ACTIONS(4552),
    [anon_sym_loop] = ACTIONS(4552),
    [anon_sym_match] = ACTIONS(4552),
    [anon_sym_mod] = ACTIONS(4552),
    [anon_sym_pub] = ACTIONS(4552),
    [anon_sym_return] = ACTIONS(4552),
    [anon_sym_raw] = ACTIONS(4552),
    [anon_sym_static] = ACTIONS(4552),
    [anon_sym_struct] = ACTIONS(4552),
    [anon_sym_trait] = ACTIONS(4552),
    [anon_sym_type] = ACTIONS(4552),
    [anon_sym_union] = ACTIONS(4552),
    [anon_sym_unsafe] = ACTIONS(4552),
    [anon_sym_use] = ACTIONS(4552),
    [anon_sym_while] = ACTIONS(4552),
    [anon_sym_extern] = ACTIONS(4552),
    [anon_sym_safe] = ACTIONS(4552),
    [anon_sym_yield] = ACTIONS(4552),
    [anon_sym_move] = ACTIONS(4552),
    [anon_sym_try] = ACTIONS(4552),
    [sym_integer_literal] = ACTIONS(4550),
    [aux_sym_string_literal_token1] = ACTIONS(4550),
    [sym_char_literal] = ACTIONS(4550),
    [anon_sym_true] = ACTIONS(4552),
    [anon_sym_false] = ACTIONS(4552),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4552),
    [sym_super] = ACTIONS(4552),
    [sym_crate] = ACTIONS(4552),
    [aux_sym_metavariable_token1] = ACTIONS(4550),
    [sym__raw_string_literal_start] = ACTIONS(4550),
    [sym_float_literal] = ACTIONS(4550),
    [sym__outer_block_doc_comment_start] = ACTIONS(4550),
    [sym__outer_line_doc_comment_start] = ACTIONS(4550),
  },
  [STATE(1204)] = {
    [sym_line_comment] = STATE(1204),
    [sym_block_comment] = STATE(1204),
    [ts_builtin_sym_end] = ACTIONS(4554),
    [sym_identifier] = ACTIONS(4556),
    [anon_sym_SEMI] = ACTIONS(4554),
    [anon_sym_macro] = ACTIONS(4556),
    [anon_sym_LPAREN] = ACTIONS(4554),
    [anon_sym_LBRACE] = ACTIONS(4554),
    [anon_sym_RBRACE] = ACTIONS(4554),
    [anon_sym_macro_rules] = ACTIONS(4556),
    [anon_sym_BANG] = ACTIONS(4554),
    [anon_sym_LBRACK] = ACTIONS(4554),
    [anon_sym_STAR] = ACTIONS(4554),
    [anon_sym_u8] = ACTIONS(4556),
    [anon_sym_i8] = ACTIONS(4556),
    [anon_sym_u16] = ACTIONS(4556),
    [anon_sym_i16] = ACTIONS(4556),
    [anon_sym_u32] = ACTIONS(4556),
    [anon_sym_i32] = ACTIONS(4556),
    [anon_sym_u64] = ACTIONS(4556),
    [anon_sym_i64] = ACTIONS(4556),
    [anon_sym_u128] = ACTIONS(4556),
    [anon_sym_i128] = ACTIONS(4556),
    [anon_sym_isize] = ACTIONS(4556),
    [anon_sym_usize] = ACTIONS(4556),
    [anon_sym_f32] = ACTIONS(4556),
    [anon_sym_f64] = ACTIONS(4556),
    [anon_sym_bool] = ACTIONS(4556),
    [anon_sym_str] = ACTIONS(4556),
    [anon_sym_char] = ACTIONS(4556),
    [anon_sym_DASH] = ACTIONS(4554),
    [anon_sym_AMP] = ACTIONS(4554),
    [anon_sym_PIPE] = ACTIONS(4554),
    [anon_sym_LT] = ACTIONS(4554),
    [anon_sym_DOT_DOT] = ACTIONS(4554),
    [anon_sym_COLON_COLON] = ACTIONS(4554),
    [anon_sym_POUND] = ACTIONS(4554),
    [anon_sym_SQUOTE] = ACTIONS(4556),
    [anon_sym_async] = ACTIONS(4556),
    [anon_sym_auto] = ACTIONS(4556),
    [anon_sym_await] = ACTIONS(4556),
    [anon_sym_become] = ACTIONS(4556),
    [anon_sym_break] = ACTIONS(4556),
    [anon_sym_const] = ACTIONS(4556),
    [anon_sym_continue] = ACTIONS(4556),
    [anon_sym_default] = ACTIONS(4556),
    [anon_sym_enum] = ACTIONS(4556),
    [anon_sym_fn] = ACTIONS(4556),
    [anon_sym_for] = ACTIONS(4556),
    [anon_sym_gen] = ACTIONS(4556),
    [anon_sym_if] = ACTIONS(4556),
    [anon_sym_impl] = ACTIONS(4556),
    [anon_sym_let] = ACTIONS(4556),
    [anon_sym_loop] = ACTIONS(4556),
    [anon_sym_match] = ACTIONS(4556),
    [anon_sym_mod] = ACTIONS(4556),
    [anon_sym_pub] = ACTIONS(4556),
    [anon_sym_return] = ACTIONS(4556),
    [anon_sym_raw] = ACTIONS(4556),
    [anon_sym_static] = ACTIONS(4556),
    [anon_sym_struct] = ACTIONS(4556),
    [anon_sym_trait] = ACTIONS(4556),
    [anon_sym_type] = ACTIONS(4556),
    [anon_sym_union] = ACTIONS(4556),
    [anon_sym_unsafe] = ACTIONS(4556),
    [anon_sym_use] = ACTIONS(4556),
    [anon_sym_while] = ACTIONS(4556),
    [anon_sym_extern] = ACTIONS(4556),
    [anon_sym_safe] = ACTIONS(4556),
    [anon_sym_yield] = ACTIONS(4556),
    [anon_sym_move] = ACTIONS(4556),
    [anon_sym_try] = ACTIONS(4556),
    [sym_integer_literal] = ACTIONS(4554),
    [aux_sym_string_literal_token1] = ACTIONS(4554),
    [sym_char_literal] = ACTIONS(4554),
    [anon_sym_true] = ACTIONS(4556),
    [anon_sym_false] = ACTIONS(4556),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4556),
    [sym_super] = ACTIONS(4556),
    [sym_crate] = ACTIONS(4556),
    [aux_sym_metavariable_token1] = ACTIONS(4554),
    [sym__raw_string_literal_start] = ACTIONS(4554),
    [sym_float_literal] = ACTIONS(4554),
    [sym__outer_block_doc_comment_start] = ACTIONS(4554),
    [sym__outer_line_doc_comment_start] = ACTIONS(4554),
  },
  [STATE(1205)] = {
    [sym_line_comment] = STATE(1205),
    [sym_block_comment] = STATE(1205),
    [ts_builtin_sym_end] = ACTIONS(4558),
    [sym_identifier] = ACTIONS(4560),
    [anon_sym_SEMI] = ACTIONS(4558),
    [anon_sym_macro] = ACTIONS(4560),
    [anon_sym_LPAREN] = ACTIONS(4558),
    [anon_sym_LBRACE] = ACTIONS(4558),
    [anon_sym_RBRACE] = ACTIONS(4558),
    [anon_sym_macro_rules] = ACTIONS(4560),
    [anon_sym_BANG] = ACTIONS(4558),
    [anon_sym_LBRACK] = ACTIONS(4558),
    [anon_sym_STAR] = ACTIONS(4558),
    [anon_sym_u8] = ACTIONS(4560),
    [anon_sym_i8] = ACTIONS(4560),
    [anon_sym_u16] = ACTIONS(4560),
    [anon_sym_i16] = ACTIONS(4560),
    [anon_sym_u32] = ACTIONS(4560),
    [anon_sym_i32] = ACTIONS(4560),
    [anon_sym_u64] = ACTIONS(4560),
    [anon_sym_i64] = ACTIONS(4560),
    [anon_sym_u128] = ACTIONS(4560),
    [anon_sym_i128] = ACTIONS(4560),
    [anon_sym_isize] = ACTIONS(4560),
    [anon_sym_usize] = ACTIONS(4560),
    [anon_sym_f32] = ACTIONS(4560),
    [anon_sym_f64] = ACTIONS(4560),
    [anon_sym_bool] = ACTIONS(4560),
    [anon_sym_str] = ACTIONS(4560),
    [anon_sym_char] = ACTIONS(4560),
    [anon_sym_DASH] = ACTIONS(4558),
    [anon_sym_AMP] = ACTIONS(4558),
    [anon_sym_PIPE] = ACTIONS(4558),
    [anon_sym_LT] = ACTIONS(4558),
    [anon_sym_DOT_DOT] = ACTIONS(4558),
    [anon_sym_COLON_COLON] = ACTIONS(4558),
    [anon_sym_POUND] = ACTIONS(4558),
    [anon_sym_SQUOTE] = ACTIONS(4560),
    [anon_sym_async] = ACTIONS(4560),
    [anon_sym_auto] = ACTIONS(4560),
    [anon_sym_await] = ACTIONS(4560),
    [anon_sym_become] = ACTIONS(4560),
    [anon_sym_break] = ACTIONS(4560),
    [anon_sym_const] = ACTIONS(4560),
    [anon_sym_continue] = ACTIONS(4560),
    [anon_sym_default] = ACTIONS(4560),
    [anon_sym_enum] = ACTIONS(4560),
    [anon_sym_fn] = ACTIONS(4560),
    [anon_sym_for] = ACTIONS(4560),
    [anon_sym_gen] = ACTIONS(4560),
    [anon_sym_if] = ACTIONS(4560),
    [anon_sym_impl] = ACTIONS(4560),
    [anon_sym_let] = ACTIONS(4560),
    [anon_sym_loop] = ACTIONS(4560),
    [anon_sym_match] = ACTIONS(4560),
    [anon_sym_mod] = ACTIONS(4560),
    [anon_sym_pub] = ACTIONS(4560),
    [anon_sym_return] = ACTIONS(4560),
    [anon_sym_raw] = ACTIONS(4560),
    [anon_sym_static] = ACTIONS(4560),
    [anon_sym_struct] = ACTIONS(4560),
    [anon_sym_trait] = ACTIONS(4560),
    [anon_sym_type] = ACTIONS(4560),
    [anon_sym_union] = ACTIONS(4560),
    [anon_sym_unsafe] = ACTIONS(4560),
    [anon_sym_use] = ACTIONS(4560),
    [anon_sym_while] = ACTIONS(4560),
    [anon_sym_extern] = ACTIONS(4560),
    [anon_sym_safe] = ACTIONS(4560),
    [anon_sym_yield] = ACTIONS(4560),
    [anon_sym_move] = ACTIONS(4560),
    [anon_sym_try] = ACTIONS(4560),
    [sym_integer_literal] = ACTIONS(4558),
    [aux_sym_string_literal_token1] = ACTIONS(4558),
    [sym_char_literal] = ACTIONS(4558),
    [anon_sym_true] = ACTIONS(4560),
    [anon_sym_false] = ACTIONS(4560),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4560),
    [sym_super] = ACTIONS(4560),
    [sym_crate] = ACTIONS(4560),
    [aux_sym_metavariable_token1] = ACTIONS(4558),
    [sym__raw_string_literal_start] = ACTIONS(4558),
    [sym_float_literal] = ACTIONS(4558),
    [sym__outer_block_doc_comment_start] = ACTIONS(4558),
    [sym__outer_line_doc_comment_start] = ACTIONS(4558),
  },
  [STATE(1206)] = {
    [sym_line_comment] = STATE(1206),
    [sym_block_comment] = STATE(1206),
    [ts_builtin_sym_end] = ACTIONS(4562),
    [sym_identifier] = ACTIONS(4564),
    [anon_sym_SEMI] = ACTIONS(4562),
    [anon_sym_macro] = ACTIONS(4564),
    [anon_sym_LPAREN] = ACTIONS(4562),
    [anon_sym_LBRACE] = ACTIONS(4562),
    [anon_sym_RBRACE] = ACTIONS(4562),
    [anon_sym_macro_rules] = ACTIONS(4564),
    [anon_sym_BANG] = ACTIONS(4562),
    [anon_sym_LBRACK] = ACTIONS(4562),
    [anon_sym_STAR] = ACTIONS(4562),
    [anon_sym_u8] = ACTIONS(4564),
    [anon_sym_i8] = ACTIONS(4564),
    [anon_sym_u16] = ACTIONS(4564),
    [anon_sym_i16] = ACTIONS(4564),
    [anon_sym_u32] = ACTIONS(4564),
    [anon_sym_i32] = ACTIONS(4564),
    [anon_sym_u64] = ACTIONS(4564),
    [anon_sym_i64] = ACTIONS(4564),
    [anon_sym_u128] = ACTIONS(4564),
    [anon_sym_i128] = ACTIONS(4564),
    [anon_sym_isize] = ACTIONS(4564),
    [anon_sym_usize] = ACTIONS(4564),
    [anon_sym_f32] = ACTIONS(4564),
    [anon_sym_f64] = ACTIONS(4564),
    [anon_sym_bool] = ACTIONS(4564),
    [anon_sym_str] = ACTIONS(4564),
    [anon_sym_char] = ACTIONS(4564),
    [anon_sym_DASH] = ACTIONS(4562),
    [anon_sym_AMP] = ACTIONS(4562),
    [anon_sym_PIPE] = ACTIONS(4562),
    [anon_sym_LT] = ACTIONS(4562),
    [anon_sym_DOT_DOT] = ACTIONS(4562),
    [anon_sym_COLON_COLON] = ACTIONS(4562),
    [anon_sym_POUND] = ACTIONS(4562),
    [anon_sym_SQUOTE] = ACTIONS(4564),
    [anon_sym_async] = ACTIONS(4564),
    [anon_sym_auto] = ACTIONS(4564),
    [anon_sym_await] = ACTIONS(4564),
    [anon_sym_become] = ACTIONS(4564),
    [anon_sym_break] = ACTIONS(4564),
    [anon_sym_const] = ACTIONS(4564),
    [anon_sym_continue] = ACTIONS(4564),
    [anon_sym_default] = ACTIONS(4564),
    [anon_sym_enum] = ACTIONS(4564),
    [anon_sym_fn] = ACTIONS(4564),
    [anon_sym_for] = ACTIONS(4564),
    [anon_sym_gen] = ACTIONS(4564),
    [anon_sym_if] = ACTIONS(4564),
    [anon_sym_impl] = ACTIONS(4564),
    [anon_sym_let] = ACTIONS(4564),
    [anon_sym_loop] = ACTIONS(4564),
    [anon_sym_match] = ACTIONS(4564),
    [anon_sym_mod] = ACTIONS(4564),
    [anon_sym_pub] = ACTIONS(4564),
    [anon_sym_return] = ACTIONS(4564),
    [anon_sym_raw] = ACTIONS(4564),
    [anon_sym_static] = ACTIONS(4564),
    [anon_sym_struct] = ACTIONS(4564),
    [anon_sym_trait] = ACTIONS(4564),
    [anon_sym_type] = ACTIONS(4564),
    [anon_sym_union] = ACTIONS(4564),
    [anon_sym_unsafe] = ACTIONS(4564),
    [anon_sym_use] = ACTIONS(4564),
    [anon_sym_while] = ACTIONS(4564),
    [anon_sym_extern] = ACTIONS(4564),
    [anon_sym_safe] = ACTIONS(4564),
    [anon_sym_yield] = ACTIONS(4564),
    [anon_sym_move] = ACTIONS(4564),
    [anon_sym_try] = ACTIONS(4564),
    [sym_integer_literal] = ACTIONS(4562),
    [aux_sym_string_literal_token1] = ACTIONS(4562),
    [sym_char_literal] = ACTIONS(4562),
    [anon_sym_true] = ACTIONS(4564),
    [anon_sym_false] = ACTIONS(4564),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4564),
    [sym_super] = ACTIONS(4564),
    [sym_crate] = ACTIONS(4564),
    [aux_sym_metavariable_token1] = ACTIONS(4562),
    [sym__raw_string_literal_start] = ACTIONS(4562),
    [sym_float_literal] = ACTIONS(4562),
    [sym__outer_block_doc_comment_start] = ACTIONS(4562),
    [sym__outer_line_doc_comment_start] = ACTIONS(4562),
  },
  [STATE(1207)] = {
    [sym_line_comment] = STATE(1207),
    [sym_block_comment] = STATE(1207),
    [ts_builtin_sym_end] = ACTIONS(4566),
    [sym_identifier] = ACTIONS(4568),
    [anon_sym_SEMI] = ACTIONS(4566),
    [anon_sym_macro] = ACTIONS(4568),
    [anon_sym_LPAREN] = ACTIONS(4566),
    [anon_sym_LBRACE] = ACTIONS(4566),
    [anon_sym_RBRACE] = ACTIONS(4566),
    [anon_sym_macro_rules] = ACTIONS(4568),
    [anon_sym_BANG] = ACTIONS(4566),
    [anon_sym_LBRACK] = ACTIONS(4566),
    [anon_sym_STAR] = ACTIONS(4566),
    [anon_sym_u8] = ACTIONS(4568),
    [anon_sym_i8] = ACTIONS(4568),
    [anon_sym_u16] = ACTIONS(4568),
    [anon_sym_i16] = ACTIONS(4568),
    [anon_sym_u32] = ACTIONS(4568),
    [anon_sym_i32] = ACTIONS(4568),
    [anon_sym_u64] = ACTIONS(4568),
    [anon_sym_i64] = ACTIONS(4568),
    [anon_sym_u128] = ACTIONS(4568),
    [anon_sym_i128] = ACTIONS(4568),
    [anon_sym_isize] = ACTIONS(4568),
    [anon_sym_usize] = ACTIONS(4568),
    [anon_sym_f32] = ACTIONS(4568),
    [anon_sym_f64] = ACTIONS(4568),
    [anon_sym_bool] = ACTIONS(4568),
    [anon_sym_str] = ACTIONS(4568),
    [anon_sym_char] = ACTIONS(4568),
    [anon_sym_DASH] = ACTIONS(4566),
    [anon_sym_AMP] = ACTIONS(4566),
    [anon_sym_PIPE] = ACTIONS(4566),
    [anon_sym_LT] = ACTIONS(4566),
    [anon_sym_DOT_DOT] = ACTIONS(4566),
    [anon_sym_COLON_COLON] = ACTIONS(4566),
    [anon_sym_POUND] = ACTIONS(4566),
    [anon_sym_SQUOTE] = ACTIONS(4568),
    [anon_sym_async] = ACTIONS(4568),
    [anon_sym_auto] = ACTIONS(4568),
    [anon_sym_await] = ACTIONS(4568),
    [anon_sym_become] = ACTIONS(4568),
    [anon_sym_break] = ACTIONS(4568),
    [anon_sym_const] = ACTIONS(4568),
    [anon_sym_continue] = ACTIONS(4568),
    [anon_sym_default] = ACTIONS(4568),
    [anon_sym_enum] = ACTIONS(4568),
    [anon_sym_fn] = ACTIONS(4568),
    [anon_sym_for] = ACTIONS(4568),
    [anon_sym_gen] = ACTIONS(4568),
    [anon_sym_if] = ACTIONS(4568),
    [anon_sym_impl] = ACTIONS(4568),
    [anon_sym_let] = ACTIONS(4568),
    [anon_sym_loop] = ACTIONS(4568),
    [anon_sym_match] = ACTIONS(4568),
    [anon_sym_mod] = ACTIONS(4568),
    [anon_sym_pub] = ACTIONS(4568),
    [anon_sym_return] = ACTIONS(4568),
    [anon_sym_raw] = ACTIONS(4568),
    [anon_sym_static] = ACTIONS(4568),
    [anon_sym_struct] = ACTIONS(4568),
    [anon_sym_trait] = ACTIONS(4568),
    [anon_sym_type] = ACTIONS(4568),
    [anon_sym_union] = ACTIONS(4568),
    [anon_sym_unsafe] = ACTIONS(4568),
    [anon_sym_use] = ACTIONS(4568),
    [anon_sym_while] = ACTIONS(4568),
    [anon_sym_extern] = ACTIONS(4568),
    [anon_sym_safe] = ACTIONS(4568),
    [anon_sym_yield] = ACTIONS(4568),
    [anon_sym_move] = ACTIONS(4568),
    [anon_sym_try] = ACTIONS(4568),
    [sym_integer_literal] = ACTIONS(4566),
    [aux_sym_string_literal_token1] = ACTIONS(4566),
    [sym_char_literal] = ACTIONS(4566),
    [anon_sym_true] = ACTIONS(4568),
    [anon_sym_false] = ACTIONS(4568),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4568),
    [sym_super] = ACTIONS(4568),
    [sym_crate] = ACTIONS(4568),
    [aux_sym_metavariable_token1] = ACTIONS(4566),
    [sym__raw_string_literal_start] = ACTIONS(4566),
    [sym_float_literal] = ACTIONS(4566),
    [sym__outer_block_doc_comment_start] = ACTIONS(4566),
    [sym__outer_line_doc_comment_start] = ACTIONS(4566),
  },
  [STATE(1208)] = {
    [sym_line_comment] = STATE(1208),
    [sym_block_comment] = STATE(1208),
    [ts_builtin_sym_end] = ACTIONS(4570),
    [sym_identifier] = ACTIONS(4572),
    [anon_sym_SEMI] = ACTIONS(4570),
    [anon_sym_macro] = ACTIONS(4572),
    [anon_sym_LPAREN] = ACTIONS(4570),
    [anon_sym_LBRACE] = ACTIONS(4570),
    [anon_sym_RBRACE] = ACTIONS(4570),
    [anon_sym_macro_rules] = ACTIONS(4572),
    [anon_sym_BANG] = ACTIONS(4570),
    [anon_sym_LBRACK] = ACTIONS(4570),
    [anon_sym_STAR] = ACTIONS(4570),
    [anon_sym_u8] = ACTIONS(4572),
    [anon_sym_i8] = ACTIONS(4572),
    [anon_sym_u16] = ACTIONS(4572),
    [anon_sym_i16] = ACTIONS(4572),
    [anon_sym_u32] = ACTIONS(4572),
    [anon_sym_i32] = ACTIONS(4572),
    [anon_sym_u64] = ACTIONS(4572),
    [anon_sym_i64] = ACTIONS(4572),
    [anon_sym_u128] = ACTIONS(4572),
    [anon_sym_i128] = ACTIONS(4572),
    [anon_sym_isize] = ACTIONS(4572),
    [anon_sym_usize] = ACTIONS(4572),
    [anon_sym_f32] = ACTIONS(4572),
    [anon_sym_f64] = ACTIONS(4572),
    [anon_sym_bool] = ACTIONS(4572),
    [anon_sym_str] = ACTIONS(4572),
    [anon_sym_char] = ACTIONS(4572),
    [anon_sym_DASH] = ACTIONS(4570),
    [anon_sym_AMP] = ACTIONS(4570),
    [anon_sym_PIPE] = ACTIONS(4570),
    [anon_sym_LT] = ACTIONS(4570),
    [anon_sym_DOT_DOT] = ACTIONS(4570),
    [anon_sym_COLON_COLON] = ACTIONS(4570),
    [anon_sym_POUND] = ACTIONS(4570),
    [anon_sym_SQUOTE] = ACTIONS(4572),
    [anon_sym_async] = ACTIONS(4572),
    [anon_sym_auto] = ACTIONS(4572),
    [anon_sym_await] = ACTIONS(4572),
    [anon_sym_become] = ACTIONS(4572),
    [anon_sym_break] = ACTIONS(4572),
    [anon_sym_const] = ACTIONS(4572),
    [anon_sym_continue] = ACTIONS(4572),
    [anon_sym_default] = ACTIONS(4572),
    [anon_sym_enum] = ACTIONS(4572),
    [anon_sym_fn] = ACTIONS(4572),
    [anon_sym_for] = ACTIONS(4572),
    [anon_sym_gen] = ACTIONS(4572),
    [anon_sym_if] = ACTIONS(4572),
    [anon_sym_impl] = ACTIONS(4572),
    [anon_sym_let] = ACTIONS(4572),
    [anon_sym_loop] = ACTIONS(4572),
    [anon_sym_match] = ACTIONS(4572),
    [anon_sym_mod] = ACTIONS(4572),
    [anon_sym_pub] = ACTIONS(4572),
    [anon_sym_return] = ACTIONS(4572),
    [anon_sym_raw] = ACTIONS(4572),
    [anon_sym_static] = ACTIONS(4572),
    [anon_sym_struct] = ACTIONS(4572),
    [anon_sym_trait] = ACTIONS(4572),
    [anon_sym_type] = ACTIONS(4572),
    [anon_sym_union] = ACTIONS(4572),
    [anon_sym_unsafe] = ACTIONS(4572),
    [anon_sym_use] = ACTIONS(4572),
    [anon_sym_while] = ACTIONS(4572),
    [anon_sym_extern] = ACTIONS(4572),
    [anon_sym_safe] = ACTIONS(4572),
    [anon_sym_yield] = ACTIONS(4572),
    [anon_sym_move] = ACTIONS(4572),
    [anon_sym_try] = ACTIONS(4572),
    [sym_integer_literal] = ACTIONS(4570),
    [aux_sym_string_literal_token1] = ACTIONS(4570),
    [sym_char_literal] = ACTIONS(4570),
    [anon_sym_true] = ACTIONS(4572),
    [anon_sym_false] = ACTIONS(4572),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4572),
    [sym_super] = ACTIONS(4572),
    [sym_crate] = ACTIONS(4572),
    [aux_sym_metavariable_token1] = ACTIONS(4570),
    [sym__raw_string_literal_start] = ACTIONS(4570),
    [sym_float_literal] = ACTIONS(4570),
    [sym__outer_block_doc_comment_start] = ACTIONS(4570),
    [sym__outer_line_doc_comment_start] = ACTIONS(4570),
  },
  [STATE(1209)] = {
    [sym_line_comment] = STATE(1209),
    [sym_block_comment] = STATE(1209),
    [ts_builtin_sym_end] = ACTIONS(4574),
    [sym_identifier] = ACTIONS(4576),
    [anon_sym_SEMI] = ACTIONS(4574),
    [anon_sym_macro] = ACTIONS(4576),
    [anon_sym_LPAREN] = ACTIONS(4574),
    [anon_sym_LBRACE] = ACTIONS(4574),
    [anon_sym_RBRACE] = ACTIONS(4574),
    [anon_sym_macro_rules] = ACTIONS(4576),
    [anon_sym_BANG] = ACTIONS(4574),
    [anon_sym_LBRACK] = ACTIONS(4574),
    [anon_sym_STAR] = ACTIONS(4574),
    [anon_sym_u8] = ACTIONS(4576),
    [anon_sym_i8] = ACTIONS(4576),
    [anon_sym_u16] = ACTIONS(4576),
    [anon_sym_i16] = ACTIONS(4576),
    [anon_sym_u32] = ACTIONS(4576),
    [anon_sym_i32] = ACTIONS(4576),
    [anon_sym_u64] = ACTIONS(4576),
    [anon_sym_i64] = ACTIONS(4576),
    [anon_sym_u128] = ACTIONS(4576),
    [anon_sym_i128] = ACTIONS(4576),
    [anon_sym_isize] = ACTIONS(4576),
    [anon_sym_usize] = ACTIONS(4576),
    [anon_sym_f32] = ACTIONS(4576),
    [anon_sym_f64] = ACTIONS(4576),
    [anon_sym_bool] = ACTIONS(4576),
    [anon_sym_str] = ACTIONS(4576),
    [anon_sym_char] = ACTIONS(4576),
    [anon_sym_DASH] = ACTIONS(4574),
    [anon_sym_AMP] = ACTIONS(4574),
    [anon_sym_PIPE] = ACTIONS(4574),
    [anon_sym_LT] = ACTIONS(4574),
    [anon_sym_DOT_DOT] = ACTIONS(4574),
    [anon_sym_COLON_COLON] = ACTIONS(4574),
    [anon_sym_POUND] = ACTIONS(4574),
    [anon_sym_SQUOTE] = ACTIONS(4576),
    [anon_sym_async] = ACTIONS(4576),
    [anon_sym_auto] = ACTIONS(4576),
    [anon_sym_await] = ACTIONS(4576),
    [anon_sym_become] = ACTIONS(4576),
    [anon_sym_break] = ACTIONS(4576),
    [anon_sym_const] = ACTIONS(4576),
    [anon_sym_continue] = ACTIONS(4576),
    [anon_sym_default] = ACTIONS(4576),
    [anon_sym_enum] = ACTIONS(4576),
    [anon_sym_fn] = ACTIONS(4576),
    [anon_sym_for] = ACTIONS(4576),
    [anon_sym_gen] = ACTIONS(4576),
    [anon_sym_if] = ACTIONS(4576),
    [anon_sym_impl] = ACTIONS(4576),
    [anon_sym_let] = ACTIONS(4576),
    [anon_sym_loop] = ACTIONS(4576),
    [anon_sym_match] = ACTIONS(4576),
    [anon_sym_mod] = ACTIONS(4576),
    [anon_sym_pub] = ACTIONS(4576),
    [anon_sym_return] = ACTIONS(4576),
    [anon_sym_raw] = ACTIONS(4576),
    [anon_sym_static] = ACTIONS(4576),
    [anon_sym_struct] = ACTIONS(4576),
    [anon_sym_trait] = ACTIONS(4576),
    [anon_sym_type] = ACTIONS(4576),
    [anon_sym_union] = ACTIONS(4576),
    [anon_sym_unsafe] = ACTIONS(4576),
    [anon_sym_use] = ACTIONS(4576),
    [anon_sym_while] = ACTIONS(4576),
    [anon_sym_extern] = ACTIONS(4576),
    [anon_sym_safe] = ACTIONS(4576),
    [anon_sym_yield] = ACTIONS(4576),
    [anon_sym_move] = ACTIONS(4576),
    [anon_sym_try] = ACTIONS(4576),
    [sym_integer_literal] = ACTIONS(4574),
    [aux_sym_string_literal_token1] = ACTIONS(4574),
    [sym_char_literal] = ACTIONS(4574),
    [anon_sym_true] = ACTIONS(4576),
    [anon_sym_false] = ACTIONS(4576),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4576),
    [sym_super] = ACTIONS(4576),
    [sym_crate] = ACTIONS(4576),
    [aux_sym_metavariable_token1] = ACTIONS(4574),
    [sym__raw_string_literal_start] = ACTIONS(4574),
    [sym_float_literal] = ACTIONS(4574),
    [sym__outer_block_doc_comment_start] = ACTIONS(4574),
    [sym__outer_line_doc_comment_start] = ACTIONS(4574),
  },
  [STATE(1210)] = {
    [sym_line_comment] = STATE(1210),
    [sym_block_comment] = STATE(1210),
    [ts_builtin_sym_end] = ACTIONS(4578),
    [sym_identifier] = ACTIONS(4580),
    [anon_sym_SEMI] = ACTIONS(4578),
    [anon_sym_macro] = ACTIONS(4580),
    [anon_sym_LPAREN] = ACTIONS(4578),
    [anon_sym_LBRACE] = ACTIONS(4578),
    [anon_sym_RBRACE] = ACTIONS(4578),
    [anon_sym_macro_rules] = ACTIONS(4580),
    [anon_sym_BANG] = ACTIONS(4578),
    [anon_sym_LBRACK] = ACTIONS(4578),
    [anon_sym_STAR] = ACTIONS(4578),
    [anon_sym_u8] = ACTIONS(4580),
    [anon_sym_i8] = ACTIONS(4580),
    [anon_sym_u16] = ACTIONS(4580),
    [anon_sym_i16] = ACTIONS(4580),
    [anon_sym_u32] = ACTIONS(4580),
    [anon_sym_i32] = ACTIONS(4580),
    [anon_sym_u64] = ACTIONS(4580),
    [anon_sym_i64] = ACTIONS(4580),
    [anon_sym_u128] = ACTIONS(4580),
    [anon_sym_i128] = ACTIONS(4580),
    [anon_sym_isize] = ACTIONS(4580),
    [anon_sym_usize] = ACTIONS(4580),
    [anon_sym_f32] = ACTIONS(4580),
    [anon_sym_f64] = ACTIONS(4580),
    [anon_sym_bool] = ACTIONS(4580),
    [anon_sym_str] = ACTIONS(4580),
    [anon_sym_char] = ACTIONS(4580),
    [anon_sym_DASH] = ACTIONS(4578),
    [anon_sym_AMP] = ACTIONS(4578),
    [anon_sym_PIPE] = ACTIONS(4578),
    [anon_sym_LT] = ACTIONS(4578),
    [anon_sym_DOT_DOT] = ACTIONS(4578),
    [anon_sym_COLON_COLON] = ACTIONS(4578),
    [anon_sym_POUND] = ACTIONS(4578),
    [anon_sym_SQUOTE] = ACTIONS(4580),
    [anon_sym_async] = ACTIONS(4580),
    [anon_sym_auto] = ACTIONS(4580),
    [anon_sym_await] = ACTIONS(4580),
    [anon_sym_become] = ACTIONS(4580),
    [anon_sym_break] = ACTIONS(4580),
    [anon_sym_const] = ACTIONS(4580),
    [anon_sym_continue] = ACTIONS(4580),
    [anon_sym_default] = ACTIONS(4580),
    [anon_sym_enum] = ACTIONS(4580),
    [anon_sym_fn] = ACTIONS(4580),
    [anon_sym_for] = ACTIONS(4580),
    [anon_sym_gen] = ACTIONS(4580),
    [anon_sym_if] = ACTIONS(4580),
    [anon_sym_impl] = ACTIONS(4580),
    [anon_sym_let] = ACTIONS(4580),
    [anon_sym_loop] = ACTIONS(4580),
    [anon_sym_match] = ACTIONS(4580),
    [anon_sym_mod] = ACTIONS(4580),
    [anon_sym_pub] = ACTIONS(4580),
    [anon_sym_return] = ACTIONS(4580),
    [anon_sym_raw] = ACTIONS(4580),
    [anon_sym_static] = ACTIONS(4580),
    [anon_sym_struct] = ACTIONS(4580),
    [anon_sym_trait] = ACTIONS(4580),
    [anon_sym_type] = ACTIONS(4580),
    [anon_sym_union] = ACTIONS(4580),
    [anon_sym_unsafe] = ACTIONS(4580),
    [anon_sym_use] = ACTIONS(4580),
    [anon_sym_while] = ACTIONS(4580),
    [anon_sym_extern] = ACTIONS(4580),
    [anon_sym_safe] = ACTIONS(4580),
    [anon_sym_yield] = ACTIONS(4580),
    [anon_sym_move] = ACTIONS(4580),
    [anon_sym_try] = ACTIONS(4580),
    [sym_integer_literal] = ACTIONS(4578),
    [aux_sym_string_literal_token1] = ACTIONS(4578),
    [sym_char_literal] = ACTIONS(4578),
    [anon_sym_true] = ACTIONS(4580),
    [anon_sym_false] = ACTIONS(4580),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4580),
    [sym_super] = ACTIONS(4580),
    [sym_crate] = ACTIONS(4580),
    [aux_sym_metavariable_token1] = ACTIONS(4578),
    [sym__raw_string_literal_start] = ACTIONS(4578),
    [sym_float_literal] = ACTIONS(4578),
    [sym__outer_block_doc_comment_start] = ACTIONS(4578),
    [sym__outer_line_doc_comment_start] = ACTIONS(4578),
  },
  [STATE(1211)] = {
    [sym_line_comment] = STATE(1211),
    [sym_block_comment] = STATE(1211),
    [ts_builtin_sym_end] = ACTIONS(4582),
    [sym_identifier] = ACTIONS(4584),
    [anon_sym_SEMI] = ACTIONS(4582),
    [anon_sym_macro] = ACTIONS(4584),
    [anon_sym_LPAREN] = ACTIONS(4582),
    [anon_sym_LBRACE] = ACTIONS(4582),
    [anon_sym_RBRACE] = ACTIONS(4582),
    [anon_sym_macro_rules] = ACTIONS(4584),
    [anon_sym_BANG] = ACTIONS(4582),
    [anon_sym_LBRACK] = ACTIONS(4582),
    [anon_sym_STAR] = ACTIONS(4582),
    [anon_sym_u8] = ACTIONS(4584),
    [anon_sym_i8] = ACTIONS(4584),
    [anon_sym_u16] = ACTIONS(4584),
    [anon_sym_i16] = ACTIONS(4584),
    [anon_sym_u32] = ACTIONS(4584),
    [anon_sym_i32] = ACTIONS(4584),
    [anon_sym_u64] = ACTIONS(4584),
    [anon_sym_i64] = ACTIONS(4584),
    [anon_sym_u128] = ACTIONS(4584),
    [anon_sym_i128] = ACTIONS(4584),
    [anon_sym_isize] = ACTIONS(4584),
    [anon_sym_usize] = ACTIONS(4584),
    [anon_sym_f32] = ACTIONS(4584),
    [anon_sym_f64] = ACTIONS(4584),
    [anon_sym_bool] = ACTIONS(4584),
    [anon_sym_str] = ACTIONS(4584),
    [anon_sym_char] = ACTIONS(4584),
    [anon_sym_DASH] = ACTIONS(4582),
    [anon_sym_AMP] = ACTIONS(4582),
    [anon_sym_PIPE] = ACTIONS(4582),
    [anon_sym_LT] = ACTIONS(4582),
    [anon_sym_DOT_DOT] = ACTIONS(4582),
    [anon_sym_COLON_COLON] = ACTIONS(4582),
    [anon_sym_POUND] = ACTIONS(4582),
    [anon_sym_SQUOTE] = ACTIONS(4584),
    [anon_sym_async] = ACTIONS(4584),
    [anon_sym_auto] = ACTIONS(4584),
    [anon_sym_await] = ACTIONS(4584),
    [anon_sym_become] = ACTIONS(4584),
    [anon_sym_break] = ACTIONS(4584),
    [anon_sym_const] = ACTIONS(4584),
    [anon_sym_continue] = ACTIONS(4584),
    [anon_sym_default] = ACTIONS(4584),
    [anon_sym_enum] = ACTIONS(4584),
    [anon_sym_fn] = ACTIONS(4584),
    [anon_sym_for] = ACTIONS(4584),
    [anon_sym_gen] = ACTIONS(4584),
    [anon_sym_if] = ACTIONS(4584),
    [anon_sym_impl] = ACTIONS(4584),
    [anon_sym_let] = ACTIONS(4584),
    [anon_sym_loop] = ACTIONS(4584),
    [anon_sym_match] = ACTIONS(4584),
    [anon_sym_mod] = ACTIONS(4584),
    [anon_sym_pub] = ACTIONS(4584),
    [anon_sym_return] = ACTIONS(4584),
    [anon_sym_raw] = ACTIONS(4584),
    [anon_sym_static] = ACTIONS(4584),
    [anon_sym_struct] = ACTIONS(4584),
    [anon_sym_trait] = ACTIONS(4584),
    [anon_sym_type] = ACTIONS(4584),
    [anon_sym_union] = ACTIONS(4584),
    [anon_sym_unsafe] = ACTIONS(4584),
    [anon_sym_use] = ACTIONS(4584),
    [anon_sym_while] = ACTIONS(4584),
    [anon_sym_extern] = ACTIONS(4584),
    [anon_sym_safe] = ACTIONS(4584),
    [anon_sym_yield] = ACTIONS(4584),
    [anon_sym_move] = ACTIONS(4584),
    [anon_sym_try] = ACTIONS(4584),
    [sym_integer_literal] = ACTIONS(4582),
    [aux_sym_string_literal_token1] = ACTIONS(4582),
    [sym_char_literal] = ACTIONS(4582),
    [anon_sym_true] = ACTIONS(4584),
    [anon_sym_false] = ACTIONS(4584),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4584),
    [sym_super] = ACTIONS(4584),
    [sym_crate] = ACTIONS(4584),
    [aux_sym_metavariable_token1] = ACTIONS(4582),
    [sym__raw_string_literal_start] = ACTIONS(4582),
    [sym_float_literal] = ACTIONS(4582),
    [sym__outer_block_doc_comment_start] = ACTIONS(4582),
    [sym__outer_line_doc_comment_start] = ACTIONS(4582),
  },
  [STATE(1212)] = {
    [sym_line_comment] = STATE(1212),
    [sym_block_comment] = STATE(1212),
    [ts_builtin_sym_end] = ACTIONS(4586),
    [sym_identifier] = ACTIONS(4588),
    [anon_sym_SEMI] = ACTIONS(4586),
    [anon_sym_macro] = ACTIONS(4588),
    [anon_sym_LPAREN] = ACTIONS(4586),
    [anon_sym_LBRACE] = ACTIONS(4586),
    [anon_sym_RBRACE] = ACTIONS(4586),
    [anon_sym_macro_rules] = ACTIONS(4588),
    [anon_sym_BANG] = ACTIONS(4586),
    [anon_sym_LBRACK] = ACTIONS(4586),
    [anon_sym_STAR] = ACTIONS(4586),
    [anon_sym_u8] = ACTIONS(4588),
    [anon_sym_i8] = ACTIONS(4588),
    [anon_sym_u16] = ACTIONS(4588),
    [anon_sym_i16] = ACTIONS(4588),
    [anon_sym_u32] = ACTIONS(4588),
    [anon_sym_i32] = ACTIONS(4588),
    [anon_sym_u64] = ACTIONS(4588),
    [anon_sym_i64] = ACTIONS(4588),
    [anon_sym_u128] = ACTIONS(4588),
    [anon_sym_i128] = ACTIONS(4588),
    [anon_sym_isize] = ACTIONS(4588),
    [anon_sym_usize] = ACTIONS(4588),
    [anon_sym_f32] = ACTIONS(4588),
    [anon_sym_f64] = ACTIONS(4588),
    [anon_sym_bool] = ACTIONS(4588),
    [anon_sym_str] = ACTIONS(4588),
    [anon_sym_char] = ACTIONS(4588),
    [anon_sym_DASH] = ACTIONS(4586),
    [anon_sym_AMP] = ACTIONS(4586),
    [anon_sym_PIPE] = ACTIONS(4586),
    [anon_sym_LT] = ACTIONS(4586),
    [anon_sym_DOT_DOT] = ACTIONS(4586),
    [anon_sym_COLON_COLON] = ACTIONS(4586),
    [anon_sym_POUND] = ACTIONS(4586),
    [anon_sym_SQUOTE] = ACTIONS(4588),
    [anon_sym_async] = ACTIONS(4588),
    [anon_sym_auto] = ACTIONS(4588),
    [anon_sym_await] = ACTIONS(4588),
    [anon_sym_become] = ACTIONS(4588),
    [anon_sym_break] = ACTIONS(4588),
    [anon_sym_const] = ACTIONS(4588),
    [anon_sym_continue] = ACTIONS(4588),
    [anon_sym_default] = ACTIONS(4588),
    [anon_sym_enum] = ACTIONS(4588),
    [anon_sym_fn] = ACTIONS(4588),
    [anon_sym_for] = ACTIONS(4588),
    [anon_sym_gen] = ACTIONS(4588),
    [anon_sym_if] = ACTIONS(4588),
    [anon_sym_impl] = ACTIONS(4588),
    [anon_sym_let] = ACTIONS(4588),
    [anon_sym_loop] = ACTIONS(4588),
    [anon_sym_match] = ACTIONS(4588),
    [anon_sym_mod] = ACTIONS(4588),
    [anon_sym_pub] = ACTIONS(4588),
    [anon_sym_return] = ACTIONS(4588),
    [anon_sym_raw] = ACTIONS(4588),
    [anon_sym_static] = ACTIONS(4588),
    [anon_sym_struct] = ACTIONS(4588),
    [anon_sym_trait] = ACTIONS(4588),
    [anon_sym_type] = ACTIONS(4588),
    [anon_sym_union] = ACTIONS(4588),
    [anon_sym_unsafe] = ACTIONS(4588),
    [anon_sym_use] = ACTIONS(4588),
    [anon_sym_while] = ACTIONS(4588),
    [anon_sym_extern] = ACTIONS(4588),
    [anon_sym_safe] = ACTIONS(4588),
    [anon_sym_yield] = ACTIONS(4588),
    [anon_sym_move] = ACTIONS(4588),
    [anon_sym_try] = ACTIONS(4588),
    [sym_integer_literal] = ACTIONS(4586),
    [aux_sym_string_literal_token1] = ACTIONS(4586),
    [sym_char_literal] = ACTIONS(4586),
    [anon_sym_true] = ACTIONS(4588),
    [anon_sym_false] = ACTIONS(4588),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4588),
    [sym_super] = ACTIONS(4588),
    [sym_crate] = ACTIONS(4588),
    [aux_sym_metavariable_token1] = ACTIONS(4586),
    [sym__raw_string_literal_start] = ACTIONS(4586),
    [sym_float_literal] = ACTIONS(4586),
    [sym__outer_block_doc_comment_start] = ACTIONS(4586),
    [sym__outer_line_doc_comment_start] = ACTIONS(4586),
  },
  [STATE(1213)] = {
    [sym_line_comment] = STATE(1213),
    [sym_block_comment] = STATE(1213),
    [ts_builtin_sym_end] = ACTIONS(4590),
    [sym_identifier] = ACTIONS(4592),
    [anon_sym_SEMI] = ACTIONS(4590),
    [anon_sym_macro] = ACTIONS(4592),
    [anon_sym_LPAREN] = ACTIONS(4590),
    [anon_sym_LBRACE] = ACTIONS(4590),
    [anon_sym_RBRACE] = ACTIONS(4590),
    [anon_sym_macro_rules] = ACTIONS(4592),
    [anon_sym_BANG] = ACTIONS(4590),
    [anon_sym_LBRACK] = ACTIONS(4590),
    [anon_sym_STAR] = ACTIONS(4590),
    [anon_sym_u8] = ACTIONS(4592),
    [anon_sym_i8] = ACTIONS(4592),
    [anon_sym_u16] = ACTIONS(4592),
    [anon_sym_i16] = ACTIONS(4592),
    [anon_sym_u32] = ACTIONS(4592),
    [anon_sym_i32] = ACTIONS(4592),
    [anon_sym_u64] = ACTIONS(4592),
    [anon_sym_i64] = ACTIONS(4592),
    [anon_sym_u128] = ACTIONS(4592),
    [anon_sym_i128] = ACTIONS(4592),
    [anon_sym_isize] = ACTIONS(4592),
    [anon_sym_usize] = ACTIONS(4592),
    [anon_sym_f32] = ACTIONS(4592),
    [anon_sym_f64] = ACTIONS(4592),
    [anon_sym_bool] = ACTIONS(4592),
    [anon_sym_str] = ACTIONS(4592),
    [anon_sym_char] = ACTIONS(4592),
    [anon_sym_DASH] = ACTIONS(4590),
    [anon_sym_AMP] = ACTIONS(4590),
    [anon_sym_PIPE] = ACTIONS(4590),
    [anon_sym_LT] = ACTIONS(4590),
    [anon_sym_DOT_DOT] = ACTIONS(4590),
    [anon_sym_COLON_COLON] = ACTIONS(4590),
    [anon_sym_POUND] = ACTIONS(4590),
    [anon_sym_SQUOTE] = ACTIONS(4592),
    [anon_sym_async] = ACTIONS(4592),
    [anon_sym_auto] = ACTIONS(4592),
    [anon_sym_await] = ACTIONS(4592),
    [anon_sym_become] = ACTIONS(4592),
    [anon_sym_break] = ACTIONS(4592),
    [anon_sym_const] = ACTIONS(4592),
    [anon_sym_continue] = ACTIONS(4592),
    [anon_sym_default] = ACTIONS(4592),
    [anon_sym_enum] = ACTIONS(4592),
    [anon_sym_fn] = ACTIONS(4592),
    [anon_sym_for] = ACTIONS(4592),
    [anon_sym_gen] = ACTIONS(4592),
    [anon_sym_if] = ACTIONS(4592),
    [anon_sym_impl] = ACTIONS(4592),
    [anon_sym_let] = ACTIONS(4592),
    [anon_sym_loop] = ACTIONS(4592),
    [anon_sym_match] = ACTIONS(4592),
    [anon_sym_mod] = ACTIONS(4592),
    [anon_sym_pub] = ACTIONS(4592),
    [anon_sym_return] = ACTIONS(4592),
    [anon_sym_raw] = ACTIONS(4592),
    [anon_sym_static] = ACTIONS(4592),
    [anon_sym_struct] = ACTIONS(4592),
    [anon_sym_trait] = ACTIONS(4592),
    [anon_sym_type] = ACTIONS(4592),
    [anon_sym_union] = ACTIONS(4592),
    [anon_sym_unsafe] = ACTIONS(4592),
    [anon_sym_use] = ACTIONS(4592),
    [anon_sym_while] = ACTIONS(4592),
    [anon_sym_extern] = ACTIONS(4592),
    [anon_sym_safe] = ACTIONS(4592),
    [anon_sym_yield] = ACTIONS(4592),
    [anon_sym_move] = ACTIONS(4592),
    [anon_sym_try] = ACTIONS(4592),
    [sym_integer_literal] = ACTIONS(4590),
    [aux_sym_string_literal_token1] = ACTIONS(4590),
    [sym_char_literal] = ACTIONS(4590),
    [anon_sym_true] = ACTIONS(4592),
    [anon_sym_false] = ACTIONS(4592),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4592),
    [sym_super] = ACTIONS(4592),
    [sym_crate] = ACTIONS(4592),
    [aux_sym_metavariable_token1] = ACTIONS(4590),
    [sym__raw_string_literal_start] = ACTIONS(4590),
    [sym_float_literal] = ACTIONS(4590),
    [sym__outer_block_doc_comment_start] = ACTIONS(4590),
    [sym__outer_line_doc_comment_start] = ACTIONS(4590),
  },
  [STATE(1214)] = {
    [sym_line_comment] = STATE(1214),
    [sym_block_comment] = STATE(1214),
    [ts_builtin_sym_end] = ACTIONS(4594),
    [sym_identifier] = ACTIONS(4596),
    [anon_sym_SEMI] = ACTIONS(4594),
    [anon_sym_macro] = ACTIONS(4596),
    [anon_sym_LPAREN] = ACTIONS(4594),
    [anon_sym_LBRACE] = ACTIONS(4594),
    [anon_sym_RBRACE] = ACTIONS(4594),
    [anon_sym_macro_rules] = ACTIONS(4596),
    [anon_sym_BANG] = ACTIONS(4594),
    [anon_sym_LBRACK] = ACTIONS(4594),
    [anon_sym_STAR] = ACTIONS(4594),
    [anon_sym_u8] = ACTIONS(4596),
    [anon_sym_i8] = ACTIONS(4596),
    [anon_sym_u16] = ACTIONS(4596),
    [anon_sym_i16] = ACTIONS(4596),
    [anon_sym_u32] = ACTIONS(4596),
    [anon_sym_i32] = ACTIONS(4596),
    [anon_sym_u64] = ACTIONS(4596),
    [anon_sym_i64] = ACTIONS(4596),
    [anon_sym_u128] = ACTIONS(4596),
    [anon_sym_i128] = ACTIONS(4596),
    [anon_sym_isize] = ACTIONS(4596),
    [anon_sym_usize] = ACTIONS(4596),
    [anon_sym_f32] = ACTIONS(4596),
    [anon_sym_f64] = ACTIONS(4596),
    [anon_sym_bool] = ACTIONS(4596),
    [anon_sym_str] = ACTIONS(4596),
    [anon_sym_char] = ACTIONS(4596),
    [anon_sym_DASH] = ACTIONS(4594),
    [anon_sym_AMP] = ACTIONS(4594),
    [anon_sym_PIPE] = ACTIONS(4594),
    [anon_sym_LT] = ACTIONS(4594),
    [anon_sym_DOT_DOT] = ACTIONS(4594),
    [anon_sym_COLON_COLON] = ACTIONS(4594),
    [anon_sym_POUND] = ACTIONS(4594),
    [anon_sym_SQUOTE] = ACTIONS(4596),
    [anon_sym_async] = ACTIONS(4596),
    [anon_sym_auto] = ACTIONS(4596),
    [anon_sym_await] = ACTIONS(4596),
    [anon_sym_become] = ACTIONS(4596),
    [anon_sym_break] = ACTIONS(4596),
    [anon_sym_const] = ACTIONS(4596),
    [anon_sym_continue] = ACTIONS(4596),
    [anon_sym_default] = ACTIONS(4596),
    [anon_sym_enum] = ACTIONS(4596),
    [anon_sym_fn] = ACTIONS(4596),
    [anon_sym_for] = ACTIONS(4596),
    [anon_sym_gen] = ACTIONS(4596),
    [anon_sym_if] = ACTIONS(4596),
    [anon_sym_impl] = ACTIONS(4596),
    [anon_sym_let] = ACTIONS(4596),
    [anon_sym_loop] = ACTIONS(4596),
    [anon_sym_match] = ACTIONS(4596),
    [anon_sym_mod] = ACTIONS(4596),
    [anon_sym_pub] = ACTIONS(4596),
    [anon_sym_return] = ACTIONS(4596),
    [anon_sym_raw] = ACTIONS(4596),
    [anon_sym_static] = ACTIONS(4596),
    [anon_sym_struct] = ACTIONS(4596),
    [anon_sym_trait] = ACTIONS(4596),
    [anon_sym_type] = ACTIONS(4596),
    [anon_sym_union] = ACTIONS(4596),
    [anon_sym_unsafe] = ACTIONS(4596),
    [anon_sym_use] = ACTIONS(4596),
    [anon_sym_while] = ACTIONS(4596),
    [anon_sym_extern] = ACTIONS(4596),
    [anon_sym_safe] = ACTIONS(4596),
    [anon_sym_yield] = ACTIONS(4596),
    [anon_sym_move] = ACTIONS(4596),
    [anon_sym_try] = ACTIONS(4596),
    [sym_integer_literal] = ACTIONS(4594),
    [aux_sym_string_literal_token1] = ACTIONS(4594),
    [sym_char_literal] = ACTIONS(4594),
    [anon_sym_true] = ACTIONS(4596),
    [anon_sym_false] = ACTIONS(4596),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4596),
    [sym_super] = ACTIONS(4596),
    [sym_crate] = ACTIONS(4596),
    [aux_sym_metavariable_token1] = ACTIONS(4594),
    [sym__raw_string_literal_start] = ACTIONS(4594),
    [sym_float_literal] = ACTIONS(4594),
    [sym__outer_block_doc_comment_start] = ACTIONS(4594),
    [sym__outer_line_doc_comment_start] = ACTIONS(4594),
  },
  [STATE(1215)] = {
    [sym_line_comment] = STATE(1215),
    [sym_block_comment] = STATE(1215),
    [ts_builtin_sym_end] = ACTIONS(4598),
    [sym_identifier] = ACTIONS(4600),
    [anon_sym_SEMI] = ACTIONS(4598),
    [anon_sym_macro] = ACTIONS(4600),
    [anon_sym_LPAREN] = ACTIONS(4598),
    [anon_sym_LBRACE] = ACTIONS(4598),
    [anon_sym_RBRACE] = ACTIONS(4598),
    [anon_sym_macro_rules] = ACTIONS(4600),
    [anon_sym_BANG] = ACTIONS(4598),
    [anon_sym_LBRACK] = ACTIONS(4598),
    [anon_sym_STAR] = ACTIONS(4598),
    [anon_sym_u8] = ACTIONS(4600),
    [anon_sym_i8] = ACTIONS(4600),
    [anon_sym_u16] = ACTIONS(4600),
    [anon_sym_i16] = ACTIONS(4600),
    [anon_sym_u32] = ACTIONS(4600),
    [anon_sym_i32] = ACTIONS(4600),
    [anon_sym_u64] = ACTIONS(4600),
    [anon_sym_i64] = ACTIONS(4600),
    [anon_sym_u128] = ACTIONS(4600),
    [anon_sym_i128] = ACTIONS(4600),
    [anon_sym_isize] = ACTIONS(4600),
    [anon_sym_usize] = ACTIONS(4600),
    [anon_sym_f32] = ACTIONS(4600),
    [anon_sym_f64] = ACTIONS(4600),
    [anon_sym_bool] = ACTIONS(4600),
    [anon_sym_str] = ACTIONS(4600),
    [anon_sym_char] = ACTIONS(4600),
    [anon_sym_DASH] = ACTIONS(4598),
    [anon_sym_AMP] = ACTIONS(4598),
    [anon_sym_PIPE] = ACTIONS(4598),
    [anon_sym_LT] = ACTIONS(4598),
    [anon_sym_DOT_DOT] = ACTIONS(4598),
    [anon_sym_COLON_COLON] = ACTIONS(4598),
    [anon_sym_POUND] = ACTIONS(4598),
    [anon_sym_SQUOTE] = ACTIONS(4600),
    [anon_sym_async] = ACTIONS(4600),
    [anon_sym_auto] = ACTIONS(4600),
    [anon_sym_await] = ACTIONS(4600),
    [anon_sym_become] = ACTIONS(4600),
    [anon_sym_break] = ACTIONS(4600),
    [anon_sym_const] = ACTIONS(4600),
    [anon_sym_continue] = ACTIONS(4600),
    [anon_sym_default] = ACTIONS(4600),
    [anon_sym_enum] = ACTIONS(4600),
    [anon_sym_fn] = ACTIONS(4600),
    [anon_sym_for] = ACTIONS(4600),
    [anon_sym_gen] = ACTIONS(4600),
    [anon_sym_if] = ACTIONS(4600),
    [anon_sym_impl] = ACTIONS(4600),
    [anon_sym_let] = ACTIONS(4600),
    [anon_sym_loop] = ACTIONS(4600),
    [anon_sym_match] = ACTIONS(4600),
    [anon_sym_mod] = ACTIONS(4600),
    [anon_sym_pub] = ACTIONS(4600),
    [anon_sym_return] = ACTIONS(4600),
    [anon_sym_raw] = ACTIONS(4600),
    [anon_sym_static] = ACTIONS(4600),
    [anon_sym_struct] = ACTIONS(4600),
    [anon_sym_trait] = ACTIONS(4600),
    [anon_sym_type] = ACTIONS(4600),
    [anon_sym_union] = ACTIONS(4600),
    [anon_sym_unsafe] = ACTIONS(4600),
    [anon_sym_use] = ACTIONS(4600),
    [anon_sym_while] = ACTIONS(4600),
    [anon_sym_extern] = ACTIONS(4600),
    [anon_sym_safe] = ACTIONS(4600),
    [anon_sym_yield] = ACTIONS(4600),
    [anon_sym_move] = ACTIONS(4600),
    [anon_sym_try] = ACTIONS(4600),
    [sym_integer_literal] = ACTIONS(4598),
    [aux_sym_string_literal_token1] = ACTIONS(4598),
    [sym_char_literal] = ACTIONS(4598),
    [anon_sym_true] = ACTIONS(4600),
    [anon_sym_false] = ACTIONS(4600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4600),
    [sym_super] = ACTIONS(4600),
    [sym_crate] = ACTIONS(4600),
    [aux_sym_metavariable_token1] = ACTIONS(4598),
    [sym__raw_string_literal_start] = ACTIONS(4598),
    [sym_float_literal] = ACTIONS(4598),
    [sym__outer_block_doc_comment_start] = ACTIONS(4598),
    [sym__outer_line_doc_comment_start] = ACTIONS(4598),
  },
  [STATE(1216)] = {
    [sym_line_comment] = STATE(1216),
    [sym_block_comment] = STATE(1216),
    [ts_builtin_sym_end] = ACTIONS(4602),
    [sym_identifier] = ACTIONS(4604),
    [anon_sym_SEMI] = ACTIONS(4602),
    [anon_sym_macro] = ACTIONS(4604),
    [anon_sym_LPAREN] = ACTIONS(4602),
    [anon_sym_LBRACE] = ACTIONS(4602),
    [anon_sym_RBRACE] = ACTIONS(4602),
    [anon_sym_macro_rules] = ACTIONS(4604),
    [anon_sym_BANG] = ACTIONS(4602),
    [anon_sym_LBRACK] = ACTIONS(4602),
    [anon_sym_STAR] = ACTIONS(4602),
    [anon_sym_u8] = ACTIONS(4604),
    [anon_sym_i8] = ACTIONS(4604),
    [anon_sym_u16] = ACTIONS(4604),
    [anon_sym_i16] = ACTIONS(4604),
    [anon_sym_u32] = ACTIONS(4604),
    [anon_sym_i32] = ACTIONS(4604),
    [anon_sym_u64] = ACTIONS(4604),
    [anon_sym_i64] = ACTIONS(4604),
    [anon_sym_u128] = ACTIONS(4604),
    [anon_sym_i128] = ACTIONS(4604),
    [anon_sym_isize] = ACTIONS(4604),
    [anon_sym_usize] = ACTIONS(4604),
    [anon_sym_f32] = ACTIONS(4604),
    [anon_sym_f64] = ACTIONS(4604),
    [anon_sym_bool] = ACTIONS(4604),
    [anon_sym_str] = ACTIONS(4604),
    [anon_sym_char] = ACTIONS(4604),
    [anon_sym_DASH] = ACTIONS(4602),
    [anon_sym_AMP] = ACTIONS(4602),
    [anon_sym_PIPE] = ACTIONS(4602),
    [anon_sym_LT] = ACTIONS(4602),
    [anon_sym_DOT_DOT] = ACTIONS(4602),
    [anon_sym_COLON_COLON] = ACTIONS(4602),
    [anon_sym_POUND] = ACTIONS(4602),
    [anon_sym_SQUOTE] = ACTIONS(4604),
    [anon_sym_async] = ACTIONS(4604),
    [anon_sym_auto] = ACTIONS(4604),
    [anon_sym_await] = ACTIONS(4604),
    [anon_sym_become] = ACTIONS(4604),
    [anon_sym_break] = ACTIONS(4604),
    [anon_sym_const] = ACTIONS(4604),
    [anon_sym_continue] = ACTIONS(4604),
    [anon_sym_default] = ACTIONS(4604),
    [anon_sym_enum] = ACTIONS(4604),
    [anon_sym_fn] = ACTIONS(4604),
    [anon_sym_for] = ACTIONS(4604),
    [anon_sym_gen] = ACTIONS(4604),
    [anon_sym_if] = ACTIONS(4604),
    [anon_sym_impl] = ACTIONS(4604),
    [anon_sym_let] = ACTIONS(4604),
    [anon_sym_loop] = ACTIONS(4604),
    [anon_sym_match] = ACTIONS(4604),
    [anon_sym_mod] = ACTIONS(4604),
    [anon_sym_pub] = ACTIONS(4604),
    [anon_sym_return] = ACTIONS(4604),
    [anon_sym_raw] = ACTIONS(4604),
    [anon_sym_static] = ACTIONS(4604),
    [anon_sym_struct] = ACTIONS(4604),
    [anon_sym_trait] = ACTIONS(4604),
    [anon_sym_type] = ACTIONS(4604),
    [anon_sym_union] = ACTIONS(4604),
    [anon_sym_unsafe] = ACTIONS(4604),
    [anon_sym_use] = ACTIONS(4604),
    [anon_sym_while] = ACTIONS(4604),
    [anon_sym_extern] = ACTIONS(4604),
    [anon_sym_safe] = ACTIONS(4604),
    [anon_sym_yield] = ACTIONS(4604),
    [anon_sym_move] = ACTIONS(4604),
    [anon_sym_try] = ACTIONS(4604),
    [sym_integer_literal] = ACTIONS(4602),
    [aux_sym_string_literal_token1] = ACTIONS(4602),
    [sym_char_literal] = ACTIONS(4602),
    [anon_sym_true] = ACTIONS(4604),
    [anon_sym_false] = ACTIONS(4604),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4604),
    [sym_super] = ACTIONS(4604),
    [sym_crate] = ACTIONS(4604),
    [aux_sym_metavariable_token1] = ACTIONS(4602),
    [sym__raw_string_literal_start] = ACTIONS(4602),
    [sym_float_literal] = ACTIONS(4602),
    [sym__outer_block_doc_comment_start] = ACTIONS(4602),
    [sym__outer_line_doc_comment_start] = ACTIONS(4602),
  },
  [STATE(1217)] = {
    [sym_line_comment] = STATE(1217),
    [sym_block_comment] = STATE(1217),
    [ts_builtin_sym_end] = ACTIONS(4606),
    [sym_identifier] = ACTIONS(4608),
    [anon_sym_SEMI] = ACTIONS(4606),
    [anon_sym_macro] = ACTIONS(4608),
    [anon_sym_LPAREN] = ACTIONS(4606),
    [anon_sym_LBRACE] = ACTIONS(4606),
    [anon_sym_RBRACE] = ACTIONS(4606),
    [anon_sym_macro_rules] = ACTIONS(4608),
    [anon_sym_BANG] = ACTIONS(4606),
    [anon_sym_LBRACK] = ACTIONS(4606),
    [anon_sym_STAR] = ACTIONS(4606),
    [anon_sym_u8] = ACTIONS(4608),
    [anon_sym_i8] = ACTIONS(4608),
    [anon_sym_u16] = ACTIONS(4608),
    [anon_sym_i16] = ACTIONS(4608),
    [anon_sym_u32] = ACTIONS(4608),
    [anon_sym_i32] = ACTIONS(4608),
    [anon_sym_u64] = ACTIONS(4608),
    [anon_sym_i64] = ACTIONS(4608),
    [anon_sym_u128] = ACTIONS(4608),
    [anon_sym_i128] = ACTIONS(4608),
    [anon_sym_isize] = ACTIONS(4608),
    [anon_sym_usize] = ACTIONS(4608),
    [anon_sym_f32] = ACTIONS(4608),
    [anon_sym_f64] = ACTIONS(4608),
    [anon_sym_bool] = ACTIONS(4608),
    [anon_sym_str] = ACTIONS(4608),
    [anon_sym_char] = ACTIONS(4608),
    [anon_sym_DASH] = ACTIONS(4606),
    [anon_sym_AMP] = ACTIONS(4606),
    [anon_sym_PIPE] = ACTIONS(4606),
    [anon_sym_LT] = ACTIONS(4606),
    [anon_sym_DOT_DOT] = ACTIONS(4606),
    [anon_sym_COLON_COLON] = ACTIONS(4606),
    [anon_sym_POUND] = ACTIONS(4606),
    [anon_sym_SQUOTE] = ACTIONS(4608),
    [anon_sym_async] = ACTIONS(4608),
    [anon_sym_auto] = ACTIONS(4608),
    [anon_sym_await] = ACTIONS(4608),
    [anon_sym_become] = ACTIONS(4608),
    [anon_sym_break] = ACTIONS(4608),
    [anon_sym_const] = ACTIONS(4608),
    [anon_sym_continue] = ACTIONS(4608),
    [anon_sym_default] = ACTIONS(4608),
    [anon_sym_enum] = ACTIONS(4608),
    [anon_sym_fn] = ACTIONS(4608),
    [anon_sym_for] = ACTIONS(4608),
    [anon_sym_gen] = ACTIONS(4608),
    [anon_sym_if] = ACTIONS(4608),
    [anon_sym_impl] = ACTIONS(4608),
    [anon_sym_let] = ACTIONS(4608),
    [anon_sym_loop] = ACTIONS(4608),
    [anon_sym_match] = ACTIONS(4608),
    [anon_sym_mod] = ACTIONS(4608),
    [anon_sym_pub] = ACTIONS(4608),
    [anon_sym_return] = ACTIONS(4608),
    [anon_sym_raw] = ACTIONS(4608),
    [anon_sym_static] = ACTIONS(4608),
    [anon_sym_struct] = ACTIONS(4608),
    [anon_sym_trait] = ACTIONS(4608),
    [anon_sym_type] = ACTIONS(4608),
    [anon_sym_union] = ACTIONS(4608),
    [anon_sym_unsafe] = ACTIONS(4608),
    [anon_sym_use] = ACTIONS(4608),
    [anon_sym_while] = ACTIONS(4608),
    [anon_sym_extern] = ACTIONS(4608),
    [anon_sym_safe] = ACTIONS(4608),
    [anon_sym_yield] = ACTIONS(4608),
    [anon_sym_move] = ACTIONS(4608),
    [anon_sym_try] = ACTIONS(4608),
    [sym_integer_literal] = ACTIONS(4606),
    [aux_sym_string_literal_token1] = ACTIONS(4606),
    [sym_char_literal] = ACTIONS(4606),
    [anon_sym_true] = ACTIONS(4608),
    [anon_sym_false] = ACTIONS(4608),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4608),
    [sym_super] = ACTIONS(4608),
    [sym_crate] = ACTIONS(4608),
    [aux_sym_metavariable_token1] = ACTIONS(4606),
    [sym__raw_string_literal_start] = ACTIONS(4606),
    [sym_float_literal] = ACTIONS(4606),
    [sym__outer_block_doc_comment_start] = ACTIONS(4606),
    [sym__outer_line_doc_comment_start] = ACTIONS(4606),
  },
  [STATE(1218)] = {
    [sym_line_comment] = STATE(1218),
    [sym_block_comment] = STATE(1218),
    [ts_builtin_sym_end] = ACTIONS(4610),
    [sym_identifier] = ACTIONS(4612),
    [anon_sym_SEMI] = ACTIONS(4610),
    [anon_sym_macro] = ACTIONS(4612),
    [anon_sym_LPAREN] = ACTIONS(4610),
    [anon_sym_LBRACE] = ACTIONS(4610),
    [anon_sym_RBRACE] = ACTIONS(4610),
    [anon_sym_macro_rules] = ACTIONS(4612),
    [anon_sym_BANG] = ACTIONS(4610),
    [anon_sym_LBRACK] = ACTIONS(4610),
    [anon_sym_STAR] = ACTIONS(4610),
    [anon_sym_u8] = ACTIONS(4612),
    [anon_sym_i8] = ACTIONS(4612),
    [anon_sym_u16] = ACTIONS(4612),
    [anon_sym_i16] = ACTIONS(4612),
    [anon_sym_u32] = ACTIONS(4612),
    [anon_sym_i32] = ACTIONS(4612),
    [anon_sym_u64] = ACTIONS(4612),
    [anon_sym_i64] = ACTIONS(4612),
    [anon_sym_u128] = ACTIONS(4612),
    [anon_sym_i128] = ACTIONS(4612),
    [anon_sym_isize] = ACTIONS(4612),
    [anon_sym_usize] = ACTIONS(4612),
    [anon_sym_f32] = ACTIONS(4612),
    [anon_sym_f64] = ACTIONS(4612),
    [anon_sym_bool] = ACTIONS(4612),
    [anon_sym_str] = ACTIONS(4612),
    [anon_sym_char] = ACTIONS(4612),
    [anon_sym_DASH] = ACTIONS(4610),
    [anon_sym_AMP] = ACTIONS(4610),
    [anon_sym_PIPE] = ACTIONS(4610),
    [anon_sym_LT] = ACTIONS(4610),
    [anon_sym_DOT_DOT] = ACTIONS(4610),
    [anon_sym_COLON_COLON] = ACTIONS(4610),
    [anon_sym_POUND] = ACTIONS(4610),
    [anon_sym_SQUOTE] = ACTIONS(4612),
    [anon_sym_async] = ACTIONS(4612),
    [anon_sym_auto] = ACTIONS(4612),
    [anon_sym_await] = ACTIONS(4612),
    [anon_sym_become] = ACTIONS(4612),
    [anon_sym_break] = ACTIONS(4612),
    [anon_sym_const] = ACTIONS(4612),
    [anon_sym_continue] = ACTIONS(4612),
    [anon_sym_default] = ACTIONS(4612),
    [anon_sym_enum] = ACTIONS(4612),
    [anon_sym_fn] = ACTIONS(4612),
    [anon_sym_for] = ACTIONS(4612),
    [anon_sym_gen] = ACTIONS(4612),
    [anon_sym_if] = ACTIONS(4612),
    [anon_sym_impl] = ACTIONS(4612),
    [anon_sym_let] = ACTIONS(4612),
    [anon_sym_loop] = ACTIONS(4612),
    [anon_sym_match] = ACTIONS(4612),
    [anon_sym_mod] = ACTIONS(4612),
    [anon_sym_pub] = ACTIONS(4612),
    [anon_sym_return] = ACTIONS(4612),
    [anon_sym_raw] = ACTIONS(4612),
    [anon_sym_static] = ACTIONS(4612),
    [anon_sym_struct] = ACTIONS(4612),
    [anon_sym_trait] = ACTIONS(4612),
    [anon_sym_type] = ACTIONS(4612),
    [anon_sym_union] = ACTIONS(4612),
    [anon_sym_unsafe] = ACTIONS(4612),
    [anon_sym_use] = ACTIONS(4612),
    [anon_sym_while] = ACTIONS(4612),
    [anon_sym_extern] = ACTIONS(4612),
    [anon_sym_safe] = ACTIONS(4612),
    [anon_sym_yield] = ACTIONS(4612),
    [anon_sym_move] = ACTIONS(4612),
    [anon_sym_try] = ACTIONS(4612),
    [sym_integer_literal] = ACTIONS(4610),
    [aux_sym_string_literal_token1] = ACTIONS(4610),
    [sym_char_literal] = ACTIONS(4610),
    [anon_sym_true] = ACTIONS(4612),
    [anon_sym_false] = ACTIONS(4612),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4612),
    [sym_super] = ACTIONS(4612),
    [sym_crate] = ACTIONS(4612),
    [aux_sym_metavariable_token1] = ACTIONS(4610),
    [sym__raw_string_literal_start] = ACTIONS(4610),
    [sym_float_literal] = ACTIONS(4610),
    [sym__outer_block_doc_comment_start] = ACTIONS(4610),
    [sym__outer_line_doc_comment_start] = ACTIONS(4610),
  },
  [STATE(1219)] = {
    [sym_line_comment] = STATE(1219),
    [sym_block_comment] = STATE(1219),
    [ts_builtin_sym_end] = ACTIONS(4614),
    [sym_identifier] = ACTIONS(4616),
    [anon_sym_SEMI] = ACTIONS(4614),
    [anon_sym_macro] = ACTIONS(4616),
    [anon_sym_LPAREN] = ACTIONS(4614),
    [anon_sym_LBRACE] = ACTIONS(4614),
    [anon_sym_RBRACE] = ACTIONS(4614),
    [anon_sym_macro_rules] = ACTIONS(4616),
    [anon_sym_BANG] = ACTIONS(4614),
    [anon_sym_LBRACK] = ACTIONS(4614),
    [anon_sym_STAR] = ACTIONS(4614),
    [anon_sym_u8] = ACTIONS(4616),
    [anon_sym_i8] = ACTIONS(4616),
    [anon_sym_u16] = ACTIONS(4616),
    [anon_sym_i16] = ACTIONS(4616),
    [anon_sym_u32] = ACTIONS(4616),
    [anon_sym_i32] = ACTIONS(4616),
    [anon_sym_u64] = ACTIONS(4616),
    [anon_sym_i64] = ACTIONS(4616),
    [anon_sym_u128] = ACTIONS(4616),
    [anon_sym_i128] = ACTIONS(4616),
    [anon_sym_isize] = ACTIONS(4616),
    [anon_sym_usize] = ACTIONS(4616),
    [anon_sym_f32] = ACTIONS(4616),
    [anon_sym_f64] = ACTIONS(4616),
    [anon_sym_bool] = ACTIONS(4616),
    [anon_sym_str] = ACTIONS(4616),
    [anon_sym_char] = ACTIONS(4616),
    [anon_sym_DASH] = ACTIONS(4614),
    [anon_sym_AMP] = ACTIONS(4614),
    [anon_sym_PIPE] = ACTIONS(4614),
    [anon_sym_LT] = ACTIONS(4614),
    [anon_sym_DOT_DOT] = ACTIONS(4614),
    [anon_sym_COLON_COLON] = ACTIONS(4614),
    [anon_sym_POUND] = ACTIONS(4614),
    [anon_sym_SQUOTE] = ACTIONS(4616),
    [anon_sym_async] = ACTIONS(4616),
    [anon_sym_auto] = ACTIONS(4616),
    [anon_sym_await] = ACTIONS(4616),
    [anon_sym_become] = ACTIONS(4616),
    [anon_sym_break] = ACTIONS(4616),
    [anon_sym_const] = ACTIONS(4616),
    [anon_sym_continue] = ACTIONS(4616),
    [anon_sym_default] = ACTIONS(4616),
    [anon_sym_enum] = ACTIONS(4616),
    [anon_sym_fn] = ACTIONS(4616),
    [anon_sym_for] = ACTIONS(4616),
    [anon_sym_gen] = ACTIONS(4616),
    [anon_sym_if] = ACTIONS(4616),
    [anon_sym_impl] = ACTIONS(4616),
    [anon_sym_let] = ACTIONS(4616),
    [anon_sym_loop] = ACTIONS(4616),
    [anon_sym_match] = ACTIONS(4616),
    [anon_sym_mod] = ACTIONS(4616),
    [anon_sym_pub] = ACTIONS(4616),
    [anon_sym_return] = ACTIONS(4616),
    [anon_sym_raw] = ACTIONS(4616),
    [anon_sym_static] = ACTIONS(4616),
    [anon_sym_struct] = ACTIONS(4616),
    [anon_sym_trait] = ACTIONS(4616),
    [anon_sym_type] = ACTIONS(4616),
    [anon_sym_union] = ACTIONS(4616),
    [anon_sym_unsafe] = ACTIONS(4616),
    [anon_sym_use] = ACTIONS(4616),
    [anon_sym_while] = ACTIONS(4616),
    [anon_sym_extern] = ACTIONS(4616),
    [anon_sym_safe] = ACTIONS(4616),
    [anon_sym_yield] = ACTIONS(4616),
    [anon_sym_move] = ACTIONS(4616),
    [anon_sym_try] = ACTIONS(4616),
    [sym_integer_literal] = ACTIONS(4614),
    [aux_sym_string_literal_token1] = ACTIONS(4614),
    [sym_char_literal] = ACTIONS(4614),
    [anon_sym_true] = ACTIONS(4616),
    [anon_sym_false] = ACTIONS(4616),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4616),
    [sym_super] = ACTIONS(4616),
    [sym_crate] = ACTIONS(4616),
    [aux_sym_metavariable_token1] = ACTIONS(4614),
    [sym__raw_string_literal_start] = ACTIONS(4614),
    [sym_float_literal] = ACTIONS(4614),
    [sym__outer_block_doc_comment_start] = ACTIONS(4614),
    [sym__outer_line_doc_comment_start] = ACTIONS(4614),
  },
  [STATE(1220)] = {
    [sym_line_comment] = STATE(1220),
    [sym_block_comment] = STATE(1220),
    [ts_builtin_sym_end] = ACTIONS(4618),
    [sym_identifier] = ACTIONS(4620),
    [anon_sym_SEMI] = ACTIONS(4618),
    [anon_sym_macro] = ACTIONS(4620),
    [anon_sym_LPAREN] = ACTIONS(4618),
    [anon_sym_LBRACE] = ACTIONS(4618),
    [anon_sym_RBRACE] = ACTIONS(4618),
    [anon_sym_macro_rules] = ACTIONS(4620),
    [anon_sym_BANG] = ACTIONS(4618),
    [anon_sym_LBRACK] = ACTIONS(4618),
    [anon_sym_STAR] = ACTIONS(4618),
    [anon_sym_u8] = ACTIONS(4620),
    [anon_sym_i8] = ACTIONS(4620),
    [anon_sym_u16] = ACTIONS(4620),
    [anon_sym_i16] = ACTIONS(4620),
    [anon_sym_u32] = ACTIONS(4620),
    [anon_sym_i32] = ACTIONS(4620),
    [anon_sym_u64] = ACTIONS(4620),
    [anon_sym_i64] = ACTIONS(4620),
    [anon_sym_u128] = ACTIONS(4620),
    [anon_sym_i128] = ACTIONS(4620),
    [anon_sym_isize] = ACTIONS(4620),
    [anon_sym_usize] = ACTIONS(4620),
    [anon_sym_f32] = ACTIONS(4620),
    [anon_sym_f64] = ACTIONS(4620),
    [anon_sym_bool] = ACTIONS(4620),
    [anon_sym_str] = ACTIONS(4620),
    [anon_sym_char] = ACTIONS(4620),
    [anon_sym_DASH] = ACTIONS(4618),
    [anon_sym_AMP] = ACTIONS(4618),
    [anon_sym_PIPE] = ACTIONS(4618),
    [anon_sym_LT] = ACTIONS(4618),
    [anon_sym_DOT_DOT] = ACTIONS(4618),
    [anon_sym_COLON_COLON] = ACTIONS(4618),
    [anon_sym_POUND] = ACTIONS(4618),
    [anon_sym_SQUOTE] = ACTIONS(4620),
    [anon_sym_async] = ACTIONS(4620),
    [anon_sym_auto] = ACTIONS(4620),
    [anon_sym_await] = ACTIONS(4620),
    [anon_sym_become] = ACTIONS(4620),
    [anon_sym_break] = ACTIONS(4620),
    [anon_sym_const] = ACTIONS(4620),
    [anon_sym_continue] = ACTIONS(4620),
    [anon_sym_default] = ACTIONS(4620),
    [anon_sym_enum] = ACTIONS(4620),
    [anon_sym_fn] = ACTIONS(4620),
    [anon_sym_for] = ACTIONS(4620),
    [anon_sym_gen] = ACTIONS(4620),
    [anon_sym_if] = ACTIONS(4620),
    [anon_sym_impl] = ACTIONS(4620),
    [anon_sym_let] = ACTIONS(4620),
    [anon_sym_loop] = ACTIONS(4620),
    [anon_sym_match] = ACTIONS(4620),
    [anon_sym_mod] = ACTIONS(4620),
    [anon_sym_pub] = ACTIONS(4620),
    [anon_sym_return] = ACTIONS(4620),
    [anon_sym_raw] = ACTIONS(4620),
    [anon_sym_static] = ACTIONS(4620),
    [anon_sym_struct] = ACTIONS(4620),
    [anon_sym_trait] = ACTIONS(4620),
    [anon_sym_type] = ACTIONS(4620),
    [anon_sym_union] = ACTIONS(4620),
    [anon_sym_unsafe] = ACTIONS(4620),
    [anon_sym_use] = ACTIONS(4620),
    [anon_sym_while] = ACTIONS(4620),
    [anon_sym_extern] = ACTIONS(4620),
    [anon_sym_safe] = ACTIONS(4620),
    [anon_sym_yield] = ACTIONS(4620),
    [anon_sym_move] = ACTIONS(4620),
    [anon_sym_try] = ACTIONS(4620),
    [sym_integer_literal] = ACTIONS(4618),
    [aux_sym_string_literal_token1] = ACTIONS(4618),
    [sym_char_literal] = ACTIONS(4618),
    [anon_sym_true] = ACTIONS(4620),
    [anon_sym_false] = ACTIONS(4620),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4620),
    [sym_super] = ACTIONS(4620),
    [sym_crate] = ACTIONS(4620),
    [aux_sym_metavariable_token1] = ACTIONS(4618),
    [sym__raw_string_literal_start] = ACTIONS(4618),
    [sym_float_literal] = ACTIONS(4618),
    [sym__outer_block_doc_comment_start] = ACTIONS(4618),
    [sym__outer_line_doc_comment_start] = ACTIONS(4618),
  },
  [STATE(1221)] = {
    [sym_line_comment] = STATE(1221),
    [sym_block_comment] = STATE(1221),
    [ts_builtin_sym_end] = ACTIONS(4622),
    [sym_identifier] = ACTIONS(4624),
    [anon_sym_SEMI] = ACTIONS(4622),
    [anon_sym_macro] = ACTIONS(4624),
    [anon_sym_LPAREN] = ACTIONS(4622),
    [anon_sym_LBRACE] = ACTIONS(4622),
    [anon_sym_RBRACE] = ACTIONS(4622),
    [anon_sym_macro_rules] = ACTIONS(4624),
    [anon_sym_BANG] = ACTIONS(4622),
    [anon_sym_LBRACK] = ACTIONS(4622),
    [anon_sym_STAR] = ACTIONS(4622),
    [anon_sym_u8] = ACTIONS(4624),
    [anon_sym_i8] = ACTIONS(4624),
    [anon_sym_u16] = ACTIONS(4624),
    [anon_sym_i16] = ACTIONS(4624),
    [anon_sym_u32] = ACTIONS(4624),
    [anon_sym_i32] = ACTIONS(4624),
    [anon_sym_u64] = ACTIONS(4624),
    [anon_sym_i64] = ACTIONS(4624),
    [anon_sym_u128] = ACTIONS(4624),
    [anon_sym_i128] = ACTIONS(4624),
    [anon_sym_isize] = ACTIONS(4624),
    [anon_sym_usize] = ACTIONS(4624),
    [anon_sym_f32] = ACTIONS(4624),
    [anon_sym_f64] = ACTIONS(4624),
    [anon_sym_bool] = ACTIONS(4624),
    [anon_sym_str] = ACTIONS(4624),
    [anon_sym_char] = ACTIONS(4624),
    [anon_sym_DASH] = ACTIONS(4622),
    [anon_sym_AMP] = ACTIONS(4622),
    [anon_sym_PIPE] = ACTIONS(4622),
    [anon_sym_LT] = ACTIONS(4622),
    [anon_sym_DOT_DOT] = ACTIONS(4622),
    [anon_sym_COLON_COLON] = ACTIONS(4622),
    [anon_sym_POUND] = ACTIONS(4622),
    [anon_sym_SQUOTE] = ACTIONS(4624),
    [anon_sym_async] = ACTIONS(4624),
    [anon_sym_auto] = ACTIONS(4624),
    [anon_sym_await] = ACTIONS(4624),
    [anon_sym_become] = ACTIONS(4624),
    [anon_sym_break] = ACTIONS(4624),
    [anon_sym_const] = ACTIONS(4624),
    [anon_sym_continue] = ACTIONS(4624),
    [anon_sym_default] = ACTIONS(4624),
    [anon_sym_enum] = ACTIONS(4624),
    [anon_sym_fn] = ACTIONS(4624),
    [anon_sym_for] = ACTIONS(4624),
    [anon_sym_gen] = ACTIONS(4624),
    [anon_sym_if] = ACTIONS(4624),
    [anon_sym_impl] = ACTIONS(4624),
    [anon_sym_let] = ACTIONS(4624),
    [anon_sym_loop] = ACTIONS(4624),
    [anon_sym_match] = ACTIONS(4624),
    [anon_sym_mod] = ACTIONS(4624),
    [anon_sym_pub] = ACTIONS(4624),
    [anon_sym_return] = ACTIONS(4624),
    [anon_sym_raw] = ACTIONS(4624),
    [anon_sym_static] = ACTIONS(4624),
    [anon_sym_struct] = ACTIONS(4624),
    [anon_sym_trait] = ACTIONS(4624),
    [anon_sym_type] = ACTIONS(4624),
    [anon_sym_union] = ACTIONS(4624),
    [anon_sym_unsafe] = ACTIONS(4624),
    [anon_sym_use] = ACTIONS(4624),
    [anon_sym_while] = ACTIONS(4624),
    [anon_sym_extern] = ACTIONS(4624),
    [anon_sym_safe] = ACTIONS(4624),
    [anon_sym_yield] = ACTIONS(4624),
    [anon_sym_move] = ACTIONS(4624),
    [anon_sym_try] = ACTIONS(4624),
    [sym_integer_literal] = ACTIONS(4622),
    [aux_sym_string_literal_token1] = ACTIONS(4622),
    [sym_char_literal] = ACTIONS(4622),
    [anon_sym_true] = ACTIONS(4624),
    [anon_sym_false] = ACTIONS(4624),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4624),
    [sym_super] = ACTIONS(4624),
    [sym_crate] = ACTIONS(4624),
    [aux_sym_metavariable_token1] = ACTIONS(4622),
    [sym__raw_string_literal_start] = ACTIONS(4622),
    [sym_float_literal] = ACTIONS(4622),
    [sym__outer_block_doc_comment_start] = ACTIONS(4622),
    [sym__outer_line_doc_comment_start] = ACTIONS(4622),
  },
  [STATE(1222)] = {
    [sym_line_comment] = STATE(1222),
    [sym_block_comment] = STATE(1222),
    [ts_builtin_sym_end] = ACTIONS(4626),
    [sym_identifier] = ACTIONS(4628),
    [anon_sym_SEMI] = ACTIONS(4626),
    [anon_sym_macro] = ACTIONS(4628),
    [anon_sym_LPAREN] = ACTIONS(4626),
    [anon_sym_LBRACE] = ACTIONS(4626),
    [anon_sym_RBRACE] = ACTIONS(4626),
    [anon_sym_macro_rules] = ACTIONS(4628),
    [anon_sym_BANG] = ACTIONS(4626),
    [anon_sym_LBRACK] = ACTIONS(4626),
    [anon_sym_STAR] = ACTIONS(4626),
    [anon_sym_u8] = ACTIONS(4628),
    [anon_sym_i8] = ACTIONS(4628),
    [anon_sym_u16] = ACTIONS(4628),
    [anon_sym_i16] = ACTIONS(4628),
    [anon_sym_u32] = ACTIONS(4628),
    [anon_sym_i32] = ACTIONS(4628),
    [anon_sym_u64] = ACTIONS(4628),
    [anon_sym_i64] = ACTIONS(4628),
    [anon_sym_u128] = ACTIONS(4628),
    [anon_sym_i128] = ACTIONS(4628),
    [anon_sym_isize] = ACTIONS(4628),
    [anon_sym_usize] = ACTIONS(4628),
    [anon_sym_f32] = ACTIONS(4628),
    [anon_sym_f64] = ACTIONS(4628),
    [anon_sym_bool] = ACTIONS(4628),
    [anon_sym_str] = ACTIONS(4628),
    [anon_sym_char] = ACTIONS(4628),
    [anon_sym_DASH] = ACTIONS(4626),
    [anon_sym_AMP] = ACTIONS(4626),
    [anon_sym_PIPE] = ACTIONS(4626),
    [anon_sym_LT] = ACTIONS(4626),
    [anon_sym_DOT_DOT] = ACTIONS(4626),
    [anon_sym_COLON_COLON] = ACTIONS(4626),
    [anon_sym_POUND] = ACTIONS(4626),
    [anon_sym_SQUOTE] = ACTIONS(4628),
    [anon_sym_async] = ACTIONS(4628),
    [anon_sym_auto] = ACTIONS(4628),
    [anon_sym_await] = ACTIONS(4628),
    [anon_sym_become] = ACTIONS(4628),
    [anon_sym_break] = ACTIONS(4628),
    [anon_sym_const] = ACTIONS(4628),
    [anon_sym_continue] = ACTIONS(4628),
    [anon_sym_default] = ACTIONS(4628),
    [anon_sym_enum] = ACTIONS(4628),
    [anon_sym_fn] = ACTIONS(4628),
    [anon_sym_for] = ACTIONS(4628),
    [anon_sym_gen] = ACTIONS(4628),
    [anon_sym_if] = ACTIONS(4628),
    [anon_sym_impl] = ACTIONS(4628),
    [anon_sym_let] = ACTIONS(4628),
    [anon_sym_loop] = ACTIONS(4628),
    [anon_sym_match] = ACTIONS(4628),
    [anon_sym_mod] = ACTIONS(4628),
    [anon_sym_pub] = ACTIONS(4628),
    [anon_sym_return] = ACTIONS(4628),
    [anon_sym_raw] = ACTIONS(4628),
    [anon_sym_static] = ACTIONS(4628),
    [anon_sym_struct] = ACTIONS(4628),
    [anon_sym_trait] = ACTIONS(4628),
    [anon_sym_type] = ACTIONS(4628),
    [anon_sym_union] = ACTIONS(4628),
    [anon_sym_unsafe] = ACTIONS(4628),
    [anon_sym_use] = ACTIONS(4628),
    [anon_sym_while] = ACTIONS(4628),
    [anon_sym_extern] = ACTIONS(4628),
    [anon_sym_safe] = ACTIONS(4628),
    [anon_sym_yield] = ACTIONS(4628),
    [anon_sym_move] = ACTIONS(4628),
    [anon_sym_try] = ACTIONS(4628),
    [sym_integer_literal] = ACTIONS(4626),
    [aux_sym_string_literal_token1] = ACTIONS(4626),
    [sym_char_literal] = ACTIONS(4626),
    [anon_sym_true] = ACTIONS(4628),
    [anon_sym_false] = ACTIONS(4628),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4628),
    [sym_super] = ACTIONS(4628),
    [sym_crate] = ACTIONS(4628),
    [aux_sym_metavariable_token1] = ACTIONS(4626),
    [sym__raw_string_literal_start] = ACTIONS(4626),
    [sym_float_literal] = ACTIONS(4626),
    [sym__outer_block_doc_comment_start] = ACTIONS(4626),
    [sym__outer_line_doc_comment_start] = ACTIONS(4626),
  },
  [STATE(1223)] = {
    [sym_line_comment] = STATE(1223),
    [sym_block_comment] = STATE(1223),
    [ts_builtin_sym_end] = ACTIONS(4630),
    [sym_identifier] = ACTIONS(4632),
    [anon_sym_SEMI] = ACTIONS(4630),
    [anon_sym_macro] = ACTIONS(4632),
    [anon_sym_LPAREN] = ACTIONS(4630),
    [anon_sym_LBRACE] = ACTIONS(4630),
    [anon_sym_RBRACE] = ACTIONS(4630),
    [anon_sym_macro_rules] = ACTIONS(4632),
    [anon_sym_BANG] = ACTIONS(4630),
    [anon_sym_LBRACK] = ACTIONS(4630),
    [anon_sym_STAR] = ACTIONS(4630),
    [anon_sym_u8] = ACTIONS(4632),
    [anon_sym_i8] = ACTIONS(4632),
    [anon_sym_u16] = ACTIONS(4632),
    [anon_sym_i16] = ACTIONS(4632),
    [anon_sym_u32] = ACTIONS(4632),
    [anon_sym_i32] = ACTIONS(4632),
    [anon_sym_u64] = ACTIONS(4632),
    [anon_sym_i64] = ACTIONS(4632),
    [anon_sym_u128] = ACTIONS(4632),
    [anon_sym_i128] = ACTIONS(4632),
    [anon_sym_isize] = ACTIONS(4632),
    [anon_sym_usize] = ACTIONS(4632),
    [anon_sym_f32] = ACTIONS(4632),
    [anon_sym_f64] = ACTIONS(4632),
    [anon_sym_bool] = ACTIONS(4632),
    [anon_sym_str] = ACTIONS(4632),
    [anon_sym_char] = ACTIONS(4632),
    [anon_sym_DASH] = ACTIONS(4630),
    [anon_sym_AMP] = ACTIONS(4630),
    [anon_sym_PIPE] = ACTIONS(4630),
    [anon_sym_LT] = ACTIONS(4630),
    [anon_sym_DOT_DOT] = ACTIONS(4630),
    [anon_sym_COLON_COLON] = ACTIONS(4630),
    [anon_sym_POUND] = ACTIONS(4630),
    [anon_sym_SQUOTE] = ACTIONS(4632),
    [anon_sym_async] = ACTIONS(4632),
    [anon_sym_auto] = ACTIONS(4632),
    [anon_sym_await] = ACTIONS(4632),
    [anon_sym_become] = ACTIONS(4632),
    [anon_sym_break] = ACTIONS(4632),
    [anon_sym_const] = ACTIONS(4632),
    [anon_sym_continue] = ACTIONS(4632),
    [anon_sym_default] = ACTIONS(4632),
    [anon_sym_enum] = ACTIONS(4632),
    [anon_sym_fn] = ACTIONS(4632),
    [anon_sym_for] = ACTIONS(4632),
    [anon_sym_gen] = ACTIONS(4632),
    [anon_sym_if] = ACTIONS(4632),
    [anon_sym_impl] = ACTIONS(4632),
    [anon_sym_let] = ACTIONS(4632),
    [anon_sym_loop] = ACTIONS(4632),
    [anon_sym_match] = ACTIONS(4632),
    [anon_sym_mod] = ACTIONS(4632),
    [anon_sym_pub] = ACTIONS(4632),
    [anon_sym_return] = ACTIONS(4632),
    [anon_sym_raw] = ACTIONS(4632),
    [anon_sym_static] = ACTIONS(4632),
    [anon_sym_struct] = ACTIONS(4632),
    [anon_sym_trait] = ACTIONS(4632),
    [anon_sym_type] = ACTIONS(4632),
    [anon_sym_union] = ACTIONS(4632),
    [anon_sym_unsafe] = ACTIONS(4632),
    [anon_sym_use] = ACTIONS(4632),
    [anon_sym_while] = ACTIONS(4632),
    [anon_sym_extern] = ACTIONS(4632),
    [anon_sym_safe] = ACTIONS(4632),
    [anon_sym_yield] = ACTIONS(4632),
    [anon_sym_move] = ACTIONS(4632),
    [anon_sym_try] = ACTIONS(4632),
    [sym_integer_literal] = ACTIONS(4630),
    [aux_sym_string_literal_token1] = ACTIONS(4630),
    [sym_char_literal] = ACTIONS(4630),
    [anon_sym_true] = ACTIONS(4632),
    [anon_sym_false] = ACTIONS(4632),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4632),
    [sym_super] = ACTIONS(4632),
    [sym_crate] = ACTIONS(4632),
    [aux_sym_metavariable_token1] = ACTIONS(4630),
    [sym__raw_string_literal_start] = ACTIONS(4630),
    [sym_float_literal] = ACTIONS(4630),
    [sym__outer_block_doc_comment_start] = ACTIONS(4630),
    [sym__outer_line_doc_comment_start] = ACTIONS(4630),
  },
  [STATE(1224)] = {
    [sym_line_comment] = STATE(1224),
    [sym_block_comment] = STATE(1224),
    [ts_builtin_sym_end] = ACTIONS(4634),
    [sym_identifier] = ACTIONS(4636),
    [anon_sym_SEMI] = ACTIONS(4634),
    [anon_sym_macro] = ACTIONS(4636),
    [anon_sym_LPAREN] = ACTIONS(4634),
    [anon_sym_LBRACE] = ACTIONS(4634),
    [anon_sym_RBRACE] = ACTIONS(4634),
    [anon_sym_macro_rules] = ACTIONS(4636),
    [anon_sym_BANG] = ACTIONS(4634),
    [anon_sym_LBRACK] = ACTIONS(4634),
    [anon_sym_STAR] = ACTIONS(4634),
    [anon_sym_u8] = ACTIONS(4636),
    [anon_sym_i8] = ACTIONS(4636),
    [anon_sym_u16] = ACTIONS(4636),
    [anon_sym_i16] = ACTIONS(4636),
    [anon_sym_u32] = ACTIONS(4636),
    [anon_sym_i32] = ACTIONS(4636),
    [anon_sym_u64] = ACTIONS(4636),
    [anon_sym_i64] = ACTIONS(4636),
    [anon_sym_u128] = ACTIONS(4636),
    [anon_sym_i128] = ACTIONS(4636),
    [anon_sym_isize] = ACTIONS(4636),
    [anon_sym_usize] = ACTIONS(4636),
    [anon_sym_f32] = ACTIONS(4636),
    [anon_sym_f64] = ACTIONS(4636),
    [anon_sym_bool] = ACTIONS(4636),
    [anon_sym_str] = ACTIONS(4636),
    [anon_sym_char] = ACTIONS(4636),
    [anon_sym_DASH] = ACTIONS(4634),
    [anon_sym_AMP] = ACTIONS(4634),
    [anon_sym_PIPE] = ACTIONS(4634),
    [anon_sym_LT] = ACTIONS(4634),
    [anon_sym_DOT_DOT] = ACTIONS(4634),
    [anon_sym_COLON_COLON] = ACTIONS(4634),
    [anon_sym_POUND] = ACTIONS(4634),
    [anon_sym_SQUOTE] = ACTIONS(4636),
    [anon_sym_async] = ACTIONS(4636),
    [anon_sym_auto] = ACTIONS(4636),
    [anon_sym_await] = ACTIONS(4636),
    [anon_sym_become] = ACTIONS(4636),
    [anon_sym_break] = ACTIONS(4636),
    [anon_sym_const] = ACTIONS(4636),
    [anon_sym_continue] = ACTIONS(4636),
    [anon_sym_default] = ACTIONS(4636),
    [anon_sym_enum] = ACTIONS(4636),
    [anon_sym_fn] = ACTIONS(4636),
    [anon_sym_for] = ACTIONS(4636),
    [anon_sym_gen] = ACTIONS(4636),
    [anon_sym_if] = ACTIONS(4636),
    [anon_sym_impl] = ACTIONS(4636),
    [anon_sym_let] = ACTIONS(4636),
    [anon_sym_loop] = ACTIONS(4636),
    [anon_sym_match] = ACTIONS(4636),
    [anon_sym_mod] = ACTIONS(4636),
    [anon_sym_pub] = ACTIONS(4636),
    [anon_sym_return] = ACTIONS(4636),
    [anon_sym_raw] = ACTIONS(4636),
    [anon_sym_static] = ACTIONS(4636),
    [anon_sym_struct] = ACTIONS(4636),
    [anon_sym_trait] = ACTIONS(4636),
    [anon_sym_type] = ACTIONS(4636),
    [anon_sym_union] = ACTIONS(4636),
    [anon_sym_unsafe] = ACTIONS(4636),
    [anon_sym_use] = ACTIONS(4636),
    [anon_sym_while] = ACTIONS(4636),
    [anon_sym_extern] = ACTIONS(4636),
    [anon_sym_safe] = ACTIONS(4636),
    [anon_sym_yield] = ACTIONS(4636),
    [anon_sym_move] = ACTIONS(4636),
    [anon_sym_try] = ACTIONS(4636),
    [sym_integer_literal] = ACTIONS(4634),
    [aux_sym_string_literal_token1] = ACTIONS(4634),
    [sym_char_literal] = ACTIONS(4634),
    [anon_sym_true] = ACTIONS(4636),
    [anon_sym_false] = ACTIONS(4636),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4636),
    [sym_super] = ACTIONS(4636),
    [sym_crate] = ACTIONS(4636),
    [aux_sym_metavariable_token1] = ACTIONS(4634),
    [sym__raw_string_literal_start] = ACTIONS(4634),
    [sym_float_literal] = ACTIONS(4634),
    [sym__outer_block_doc_comment_start] = ACTIONS(4634),
    [sym__outer_line_doc_comment_start] = ACTIONS(4634),
  },
  [STATE(1225)] = {
    [sym_line_comment] = STATE(1225),
    [sym_block_comment] = STATE(1225),
    [ts_builtin_sym_end] = ACTIONS(4638),
    [sym_identifier] = ACTIONS(4640),
    [anon_sym_SEMI] = ACTIONS(4638),
    [anon_sym_macro] = ACTIONS(4640),
    [anon_sym_LPAREN] = ACTIONS(4638),
    [anon_sym_LBRACE] = ACTIONS(4638),
    [anon_sym_RBRACE] = ACTIONS(4638),
    [anon_sym_macro_rules] = ACTIONS(4640),
    [anon_sym_BANG] = ACTIONS(4638),
    [anon_sym_LBRACK] = ACTIONS(4638),
    [anon_sym_STAR] = ACTIONS(4638),
    [anon_sym_u8] = ACTIONS(4640),
    [anon_sym_i8] = ACTIONS(4640),
    [anon_sym_u16] = ACTIONS(4640),
    [anon_sym_i16] = ACTIONS(4640),
    [anon_sym_u32] = ACTIONS(4640),
    [anon_sym_i32] = ACTIONS(4640),
    [anon_sym_u64] = ACTIONS(4640),
    [anon_sym_i64] = ACTIONS(4640),
    [anon_sym_u128] = ACTIONS(4640),
    [anon_sym_i128] = ACTIONS(4640),
    [anon_sym_isize] = ACTIONS(4640),
    [anon_sym_usize] = ACTIONS(4640),
    [anon_sym_f32] = ACTIONS(4640),
    [anon_sym_f64] = ACTIONS(4640),
    [anon_sym_bool] = ACTIONS(4640),
    [anon_sym_str] = ACTIONS(4640),
    [anon_sym_char] = ACTIONS(4640),
    [anon_sym_DASH] = ACTIONS(4638),
    [anon_sym_AMP] = ACTIONS(4638),
    [anon_sym_PIPE] = ACTIONS(4638),
    [anon_sym_LT] = ACTIONS(4638),
    [anon_sym_DOT_DOT] = ACTIONS(4638),
    [anon_sym_COLON_COLON] = ACTIONS(4638),
    [anon_sym_POUND] = ACTIONS(4638),
    [anon_sym_SQUOTE] = ACTIONS(4640),
    [anon_sym_async] = ACTIONS(4640),
    [anon_sym_auto] = ACTIONS(4640),
    [anon_sym_await] = ACTIONS(4640),
    [anon_sym_become] = ACTIONS(4640),
    [anon_sym_break] = ACTIONS(4640),
    [anon_sym_const] = ACTIONS(4640),
    [anon_sym_continue] = ACTIONS(4640),
    [anon_sym_default] = ACTIONS(4640),
    [anon_sym_enum] = ACTIONS(4640),
    [anon_sym_fn] = ACTIONS(4640),
    [anon_sym_for] = ACTIONS(4640),
    [anon_sym_gen] = ACTIONS(4640),
    [anon_sym_if] = ACTIONS(4640),
    [anon_sym_impl] = ACTIONS(4640),
    [anon_sym_let] = ACTIONS(4640),
    [anon_sym_loop] = ACTIONS(4640),
    [anon_sym_match] = ACTIONS(4640),
    [anon_sym_mod] = ACTIONS(4640),
    [anon_sym_pub] = ACTIONS(4640),
    [anon_sym_return] = ACTIONS(4640),
    [anon_sym_raw] = ACTIONS(4640),
    [anon_sym_static] = ACTIONS(4640),
    [anon_sym_struct] = ACTIONS(4640),
    [anon_sym_trait] = ACTIONS(4640),
    [anon_sym_type] = ACTIONS(4640),
    [anon_sym_union] = ACTIONS(4640),
    [anon_sym_unsafe] = ACTIONS(4640),
    [anon_sym_use] = ACTIONS(4640),
    [anon_sym_while] = ACTIONS(4640),
    [anon_sym_extern] = ACTIONS(4640),
    [anon_sym_safe] = ACTIONS(4640),
    [anon_sym_yield] = ACTIONS(4640),
    [anon_sym_move] = ACTIONS(4640),
    [anon_sym_try] = ACTIONS(4640),
    [sym_integer_literal] = ACTIONS(4638),
    [aux_sym_string_literal_token1] = ACTIONS(4638),
    [sym_char_literal] = ACTIONS(4638),
    [anon_sym_true] = ACTIONS(4640),
    [anon_sym_false] = ACTIONS(4640),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4640),
    [sym_super] = ACTIONS(4640),
    [sym_crate] = ACTIONS(4640),
    [aux_sym_metavariable_token1] = ACTIONS(4638),
    [sym__raw_string_literal_start] = ACTIONS(4638),
    [sym_float_literal] = ACTIONS(4638),
    [sym__outer_block_doc_comment_start] = ACTIONS(4638),
    [sym__outer_line_doc_comment_start] = ACTIONS(4638),
  },
  [STATE(1226)] = {
    [sym_line_comment] = STATE(1226),
    [sym_block_comment] = STATE(1226),
    [ts_builtin_sym_end] = ACTIONS(4642),
    [sym_identifier] = ACTIONS(4644),
    [anon_sym_SEMI] = ACTIONS(4642),
    [anon_sym_macro] = ACTIONS(4644),
    [anon_sym_LPAREN] = ACTIONS(4642),
    [anon_sym_LBRACE] = ACTIONS(4642),
    [anon_sym_RBRACE] = ACTIONS(4642),
    [anon_sym_macro_rules] = ACTIONS(4644),
    [anon_sym_BANG] = ACTIONS(4642),
    [anon_sym_LBRACK] = ACTIONS(4642),
    [anon_sym_STAR] = ACTIONS(4642),
    [anon_sym_u8] = ACTIONS(4644),
    [anon_sym_i8] = ACTIONS(4644),
    [anon_sym_u16] = ACTIONS(4644),
    [anon_sym_i16] = ACTIONS(4644),
    [anon_sym_u32] = ACTIONS(4644),
    [anon_sym_i32] = ACTIONS(4644),
    [anon_sym_u64] = ACTIONS(4644),
    [anon_sym_i64] = ACTIONS(4644),
    [anon_sym_u128] = ACTIONS(4644),
    [anon_sym_i128] = ACTIONS(4644),
    [anon_sym_isize] = ACTIONS(4644),
    [anon_sym_usize] = ACTIONS(4644),
    [anon_sym_f32] = ACTIONS(4644),
    [anon_sym_f64] = ACTIONS(4644),
    [anon_sym_bool] = ACTIONS(4644),
    [anon_sym_str] = ACTIONS(4644),
    [anon_sym_char] = ACTIONS(4644),
    [anon_sym_DASH] = ACTIONS(4642),
    [anon_sym_AMP] = ACTIONS(4642),
    [anon_sym_PIPE] = ACTIONS(4642),
    [anon_sym_LT] = ACTIONS(4642),
    [anon_sym_DOT_DOT] = ACTIONS(4642),
    [anon_sym_COLON_COLON] = ACTIONS(4642),
    [anon_sym_POUND] = ACTIONS(4642),
    [anon_sym_SQUOTE] = ACTIONS(4644),
    [anon_sym_async] = ACTIONS(4644),
    [anon_sym_auto] = ACTIONS(4644),
    [anon_sym_await] = ACTIONS(4644),
    [anon_sym_become] = ACTIONS(4644),
    [anon_sym_break] = ACTIONS(4644),
    [anon_sym_const] = ACTIONS(4644),
    [anon_sym_continue] = ACTIONS(4644),
    [anon_sym_default] = ACTIONS(4644),
    [anon_sym_enum] = ACTIONS(4644),
    [anon_sym_fn] = ACTIONS(4644),
    [anon_sym_for] = ACTIONS(4644),
    [anon_sym_gen] = ACTIONS(4644),
    [anon_sym_if] = ACTIONS(4644),
    [anon_sym_impl] = ACTIONS(4644),
    [anon_sym_let] = ACTIONS(4644),
    [anon_sym_loop] = ACTIONS(4644),
    [anon_sym_match] = ACTIONS(4644),
    [anon_sym_mod] = ACTIONS(4644),
    [anon_sym_pub] = ACTIONS(4644),
    [anon_sym_return] = ACTIONS(4644),
    [anon_sym_raw] = ACTIONS(4644),
    [anon_sym_static] = ACTIONS(4644),
    [anon_sym_struct] = ACTIONS(4644),
    [anon_sym_trait] = ACTIONS(4644),
    [anon_sym_type] = ACTIONS(4644),
    [anon_sym_union] = ACTIONS(4644),
    [anon_sym_unsafe] = ACTIONS(4644),
    [anon_sym_use] = ACTIONS(4644),
    [anon_sym_while] = ACTIONS(4644),
    [anon_sym_extern] = ACTIONS(4644),
    [anon_sym_safe] = ACTIONS(4644),
    [anon_sym_yield] = ACTIONS(4644),
    [anon_sym_move] = ACTIONS(4644),
    [anon_sym_try] = ACTIONS(4644),
    [sym_integer_literal] = ACTIONS(4642),
    [aux_sym_string_literal_token1] = ACTIONS(4642),
    [sym_char_literal] = ACTIONS(4642),
    [anon_sym_true] = ACTIONS(4644),
    [anon_sym_false] = ACTIONS(4644),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4644),
    [sym_super] = ACTIONS(4644),
    [sym_crate] = ACTIONS(4644),
    [aux_sym_metavariable_token1] = ACTIONS(4642),
    [sym__raw_string_literal_start] = ACTIONS(4642),
    [sym_float_literal] = ACTIONS(4642),
    [sym__outer_block_doc_comment_start] = ACTIONS(4642),
    [sym__outer_line_doc_comment_start] = ACTIONS(4642),
  },
  [STATE(1227)] = {
    [sym_line_comment] = STATE(1227),
    [sym_block_comment] = STATE(1227),
    [ts_builtin_sym_end] = ACTIONS(4646),
    [sym_identifier] = ACTIONS(4648),
    [anon_sym_SEMI] = ACTIONS(4646),
    [anon_sym_macro] = ACTIONS(4648),
    [anon_sym_LPAREN] = ACTIONS(4646),
    [anon_sym_LBRACE] = ACTIONS(4646),
    [anon_sym_RBRACE] = ACTIONS(4646),
    [anon_sym_macro_rules] = ACTIONS(4648),
    [anon_sym_BANG] = ACTIONS(4646),
    [anon_sym_LBRACK] = ACTIONS(4646),
    [anon_sym_STAR] = ACTIONS(4646),
    [anon_sym_u8] = ACTIONS(4648),
    [anon_sym_i8] = ACTIONS(4648),
    [anon_sym_u16] = ACTIONS(4648),
    [anon_sym_i16] = ACTIONS(4648),
    [anon_sym_u32] = ACTIONS(4648),
    [anon_sym_i32] = ACTIONS(4648),
    [anon_sym_u64] = ACTIONS(4648),
    [anon_sym_i64] = ACTIONS(4648),
    [anon_sym_u128] = ACTIONS(4648),
    [anon_sym_i128] = ACTIONS(4648),
    [anon_sym_isize] = ACTIONS(4648),
    [anon_sym_usize] = ACTIONS(4648),
    [anon_sym_f32] = ACTIONS(4648),
    [anon_sym_f64] = ACTIONS(4648),
    [anon_sym_bool] = ACTIONS(4648),
    [anon_sym_str] = ACTIONS(4648),
    [anon_sym_char] = ACTIONS(4648),
    [anon_sym_DASH] = ACTIONS(4646),
    [anon_sym_AMP] = ACTIONS(4646),
    [anon_sym_PIPE] = ACTIONS(4646),
    [anon_sym_LT] = ACTIONS(4646),
    [anon_sym_DOT_DOT] = ACTIONS(4646),
    [anon_sym_COLON_COLON] = ACTIONS(4646),
    [anon_sym_POUND] = ACTIONS(4646),
    [anon_sym_SQUOTE] = ACTIONS(4648),
    [anon_sym_async] = ACTIONS(4648),
    [anon_sym_auto] = ACTIONS(4648),
    [anon_sym_await] = ACTIONS(4648),
    [anon_sym_become] = ACTIONS(4648),
    [anon_sym_break] = ACTIONS(4648),
    [anon_sym_const] = ACTIONS(4648),
    [anon_sym_continue] = ACTIONS(4648),
    [anon_sym_default] = ACTIONS(4648),
    [anon_sym_enum] = ACTIONS(4648),
    [anon_sym_fn] = ACTIONS(4648),
    [anon_sym_for] = ACTIONS(4648),
    [anon_sym_gen] = ACTIONS(4648),
    [anon_sym_if] = ACTIONS(4648),
    [anon_sym_impl] = ACTIONS(4648),
    [anon_sym_let] = ACTIONS(4648),
    [anon_sym_loop] = ACTIONS(4648),
    [anon_sym_match] = ACTIONS(4648),
    [anon_sym_mod] = ACTIONS(4648),
    [anon_sym_pub] = ACTIONS(4648),
    [anon_sym_return] = ACTIONS(4648),
    [anon_sym_raw] = ACTIONS(4648),
    [anon_sym_static] = ACTIONS(4648),
    [anon_sym_struct] = ACTIONS(4648),
    [anon_sym_trait] = ACTIONS(4648),
    [anon_sym_type] = ACTIONS(4648),
    [anon_sym_union] = ACTIONS(4648),
    [anon_sym_unsafe] = ACTIONS(4648),
    [anon_sym_use] = ACTIONS(4648),
    [anon_sym_while] = ACTIONS(4648),
    [anon_sym_extern] = ACTIONS(4648),
    [anon_sym_safe] = ACTIONS(4648),
    [anon_sym_yield] = ACTIONS(4648),
    [anon_sym_move] = ACTIONS(4648),
    [anon_sym_try] = ACTIONS(4648),
    [sym_integer_literal] = ACTIONS(4646),
    [aux_sym_string_literal_token1] = ACTIONS(4646),
    [sym_char_literal] = ACTIONS(4646),
    [anon_sym_true] = ACTIONS(4648),
    [anon_sym_false] = ACTIONS(4648),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4648),
    [sym_super] = ACTIONS(4648),
    [sym_crate] = ACTIONS(4648),
    [aux_sym_metavariable_token1] = ACTIONS(4646),
    [sym__raw_string_literal_start] = ACTIONS(4646),
    [sym_float_literal] = ACTIONS(4646),
    [sym__outer_block_doc_comment_start] = ACTIONS(4646),
    [sym__outer_line_doc_comment_start] = ACTIONS(4646),
  },
  [STATE(1228)] = {
    [sym_line_comment] = STATE(1228),
    [sym_block_comment] = STATE(1228),
    [ts_builtin_sym_end] = ACTIONS(4650),
    [sym_identifier] = ACTIONS(4652),
    [anon_sym_SEMI] = ACTIONS(4650),
    [anon_sym_macro] = ACTIONS(4652),
    [anon_sym_LPAREN] = ACTIONS(4650),
    [anon_sym_LBRACE] = ACTIONS(4650),
    [anon_sym_RBRACE] = ACTIONS(4650),
    [anon_sym_macro_rules] = ACTIONS(4652),
    [anon_sym_BANG] = ACTIONS(4650),
    [anon_sym_LBRACK] = ACTIONS(4650),
    [anon_sym_STAR] = ACTIONS(4650),
    [anon_sym_u8] = ACTIONS(4652),
    [anon_sym_i8] = ACTIONS(4652),
    [anon_sym_u16] = ACTIONS(4652),
    [anon_sym_i16] = ACTIONS(4652),
    [anon_sym_u32] = ACTIONS(4652),
    [anon_sym_i32] = ACTIONS(4652),
    [anon_sym_u64] = ACTIONS(4652),
    [anon_sym_i64] = ACTIONS(4652),
    [anon_sym_u128] = ACTIONS(4652),
    [anon_sym_i128] = ACTIONS(4652),
    [anon_sym_isize] = ACTIONS(4652),
    [anon_sym_usize] = ACTIONS(4652),
    [anon_sym_f32] = ACTIONS(4652),
    [anon_sym_f64] = ACTIONS(4652),
    [anon_sym_bool] = ACTIONS(4652),
    [anon_sym_str] = ACTIONS(4652),
    [anon_sym_char] = ACTIONS(4652),
    [anon_sym_DASH] = ACTIONS(4650),
    [anon_sym_AMP] = ACTIONS(4650),
    [anon_sym_PIPE] = ACTIONS(4650),
    [anon_sym_LT] = ACTIONS(4650),
    [anon_sym_DOT_DOT] = ACTIONS(4650),
    [anon_sym_COLON_COLON] = ACTIONS(4650),
    [anon_sym_POUND] = ACTIONS(4650),
    [anon_sym_SQUOTE] = ACTIONS(4652),
    [anon_sym_async] = ACTIONS(4652),
    [anon_sym_auto] = ACTIONS(4652),
    [anon_sym_await] = ACTIONS(4652),
    [anon_sym_become] = ACTIONS(4652),
    [anon_sym_break] = ACTIONS(4652),
    [anon_sym_const] = ACTIONS(4652),
    [anon_sym_continue] = ACTIONS(4652),
    [anon_sym_default] = ACTIONS(4652),
    [anon_sym_enum] = ACTIONS(4652),
    [anon_sym_fn] = ACTIONS(4652),
    [anon_sym_for] = ACTIONS(4652),
    [anon_sym_gen] = ACTIONS(4652),
    [anon_sym_if] = ACTIONS(4652),
    [anon_sym_impl] = ACTIONS(4652),
    [anon_sym_let] = ACTIONS(4652),
    [anon_sym_loop] = ACTIONS(4652),
    [anon_sym_match] = ACTIONS(4652),
    [anon_sym_mod] = ACTIONS(4652),
    [anon_sym_pub] = ACTIONS(4652),
    [anon_sym_return] = ACTIONS(4652),
    [anon_sym_raw] = ACTIONS(4652),
    [anon_sym_static] = ACTIONS(4652),
    [anon_sym_struct] = ACTIONS(4652),
    [anon_sym_trait] = ACTIONS(4652),
    [anon_sym_type] = ACTIONS(4652),
    [anon_sym_union] = ACTIONS(4652),
    [anon_sym_unsafe] = ACTIONS(4652),
    [anon_sym_use] = ACTIONS(4652),
    [anon_sym_while] = ACTIONS(4652),
    [anon_sym_extern] = ACTIONS(4652),
    [anon_sym_safe] = ACTIONS(4652),
    [anon_sym_yield] = ACTIONS(4652),
    [anon_sym_move] = ACTIONS(4652),
    [anon_sym_try] = ACTIONS(4652),
    [sym_integer_literal] = ACTIONS(4650),
    [aux_sym_string_literal_token1] = ACTIONS(4650),
    [sym_char_literal] = ACTIONS(4650),
    [anon_sym_true] = ACTIONS(4652),
    [anon_sym_false] = ACTIONS(4652),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4652),
    [sym_super] = ACTIONS(4652),
    [sym_crate] = ACTIONS(4652),
    [aux_sym_metavariable_token1] = ACTIONS(4650),
    [sym__raw_string_literal_start] = ACTIONS(4650),
    [sym_float_literal] = ACTIONS(4650),
    [sym__outer_block_doc_comment_start] = ACTIONS(4650),
    [sym__outer_line_doc_comment_start] = ACTIONS(4650),
  },
  [STATE(1229)] = {
    [sym_line_comment] = STATE(1229),
    [sym_block_comment] = STATE(1229),
    [ts_builtin_sym_end] = ACTIONS(4654),
    [sym_identifier] = ACTIONS(4656),
    [anon_sym_SEMI] = ACTIONS(4654),
    [anon_sym_macro] = ACTIONS(4656),
    [anon_sym_LPAREN] = ACTIONS(4654),
    [anon_sym_LBRACE] = ACTIONS(4654),
    [anon_sym_RBRACE] = ACTIONS(4654),
    [anon_sym_macro_rules] = ACTIONS(4656),
    [anon_sym_BANG] = ACTIONS(4654),
    [anon_sym_LBRACK] = ACTIONS(4654),
    [anon_sym_STAR] = ACTIONS(4654),
    [anon_sym_u8] = ACTIONS(4656),
    [anon_sym_i8] = ACTIONS(4656),
    [anon_sym_u16] = ACTIONS(4656),
    [anon_sym_i16] = ACTIONS(4656),
    [anon_sym_u32] = ACTIONS(4656),
    [anon_sym_i32] = ACTIONS(4656),
    [anon_sym_u64] = ACTIONS(4656),
    [anon_sym_i64] = ACTIONS(4656),
    [anon_sym_u128] = ACTIONS(4656),
    [anon_sym_i128] = ACTIONS(4656),
    [anon_sym_isize] = ACTIONS(4656),
    [anon_sym_usize] = ACTIONS(4656),
    [anon_sym_f32] = ACTIONS(4656),
    [anon_sym_f64] = ACTIONS(4656),
    [anon_sym_bool] = ACTIONS(4656),
    [anon_sym_str] = ACTIONS(4656),
    [anon_sym_char] = ACTIONS(4656),
    [anon_sym_DASH] = ACTIONS(4654),
    [anon_sym_AMP] = ACTIONS(4654),
    [anon_sym_PIPE] = ACTIONS(4654),
    [anon_sym_LT] = ACTIONS(4654),
    [anon_sym_DOT_DOT] = ACTIONS(4654),
    [anon_sym_COLON_COLON] = ACTIONS(4654),
    [anon_sym_POUND] = ACTIONS(4654),
    [anon_sym_SQUOTE] = ACTIONS(4656),
    [anon_sym_async] = ACTIONS(4656),
    [anon_sym_auto] = ACTIONS(4656),
    [anon_sym_await] = ACTIONS(4656),
    [anon_sym_become] = ACTIONS(4656),
    [anon_sym_break] = ACTIONS(4656),
    [anon_sym_const] = ACTIONS(4656),
    [anon_sym_continue] = ACTIONS(4656),
    [anon_sym_default] = ACTIONS(4656),
    [anon_sym_enum] = ACTIONS(4656),
    [anon_sym_fn] = ACTIONS(4656),
    [anon_sym_for] = ACTIONS(4656),
    [anon_sym_gen] = ACTIONS(4656),
    [anon_sym_if] = ACTIONS(4656),
    [anon_sym_impl] = ACTIONS(4656),
    [anon_sym_let] = ACTIONS(4656),
    [anon_sym_loop] = ACTIONS(4656),
    [anon_sym_match] = ACTIONS(4656),
    [anon_sym_mod] = ACTIONS(4656),
    [anon_sym_pub] = ACTIONS(4656),
    [anon_sym_return] = ACTIONS(4656),
    [anon_sym_raw] = ACTIONS(4656),
    [anon_sym_static] = ACTIONS(4656),
    [anon_sym_struct] = ACTIONS(4656),
    [anon_sym_trait] = ACTIONS(4656),
    [anon_sym_type] = ACTIONS(4656),
    [anon_sym_union] = ACTIONS(4656),
    [anon_sym_unsafe] = ACTIONS(4656),
    [anon_sym_use] = ACTIONS(4656),
    [anon_sym_while] = ACTIONS(4656),
    [anon_sym_extern] = ACTIONS(4656),
    [anon_sym_safe] = ACTIONS(4656),
    [anon_sym_yield] = ACTIONS(4656),
    [anon_sym_move] = ACTIONS(4656),
    [anon_sym_try] = ACTIONS(4656),
    [sym_integer_literal] = ACTIONS(4654),
    [aux_sym_string_literal_token1] = ACTIONS(4654),
    [sym_char_literal] = ACTIONS(4654),
    [anon_sym_true] = ACTIONS(4656),
    [anon_sym_false] = ACTIONS(4656),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4656),
    [sym_super] = ACTIONS(4656),
    [sym_crate] = ACTIONS(4656),
    [aux_sym_metavariable_token1] = ACTIONS(4654),
    [sym__raw_string_literal_start] = ACTIONS(4654),
    [sym_float_literal] = ACTIONS(4654),
    [sym__outer_block_doc_comment_start] = ACTIONS(4654),
    [sym__outer_line_doc_comment_start] = ACTIONS(4654),
  },
  [STATE(1230)] = {
    [sym_line_comment] = STATE(1230),
    [sym_block_comment] = STATE(1230),
    [ts_builtin_sym_end] = ACTIONS(4658),
    [sym_identifier] = ACTIONS(4660),
    [anon_sym_SEMI] = ACTIONS(4658),
    [anon_sym_macro] = ACTIONS(4660),
    [anon_sym_LPAREN] = ACTIONS(4658),
    [anon_sym_LBRACE] = ACTIONS(4658),
    [anon_sym_RBRACE] = ACTIONS(4658),
    [anon_sym_macro_rules] = ACTIONS(4660),
    [anon_sym_BANG] = ACTIONS(4658),
    [anon_sym_LBRACK] = ACTIONS(4658),
    [anon_sym_STAR] = ACTIONS(4658),
    [anon_sym_u8] = ACTIONS(4660),
    [anon_sym_i8] = ACTIONS(4660),
    [anon_sym_u16] = ACTIONS(4660),
    [anon_sym_i16] = ACTIONS(4660),
    [anon_sym_u32] = ACTIONS(4660),
    [anon_sym_i32] = ACTIONS(4660),
    [anon_sym_u64] = ACTIONS(4660),
    [anon_sym_i64] = ACTIONS(4660),
    [anon_sym_u128] = ACTIONS(4660),
    [anon_sym_i128] = ACTIONS(4660),
    [anon_sym_isize] = ACTIONS(4660),
    [anon_sym_usize] = ACTIONS(4660),
    [anon_sym_f32] = ACTIONS(4660),
    [anon_sym_f64] = ACTIONS(4660),
    [anon_sym_bool] = ACTIONS(4660),
    [anon_sym_str] = ACTIONS(4660),
    [anon_sym_char] = ACTIONS(4660),
    [anon_sym_DASH] = ACTIONS(4658),
    [anon_sym_AMP] = ACTIONS(4658),
    [anon_sym_PIPE] = ACTIONS(4658),
    [anon_sym_LT] = ACTIONS(4658),
    [anon_sym_DOT_DOT] = ACTIONS(4658),
    [anon_sym_COLON_COLON] = ACTIONS(4658),
    [anon_sym_POUND] = ACTIONS(4658),
    [anon_sym_SQUOTE] = ACTIONS(4660),
    [anon_sym_async] = ACTIONS(4660),
    [anon_sym_auto] = ACTIONS(4660),
    [anon_sym_await] = ACTIONS(4660),
    [anon_sym_become] = ACTIONS(4660),
    [anon_sym_break] = ACTIONS(4660),
    [anon_sym_const] = ACTIONS(4660),
    [anon_sym_continue] = ACTIONS(4660),
    [anon_sym_default] = ACTIONS(4660),
    [anon_sym_enum] = ACTIONS(4660),
    [anon_sym_fn] = ACTIONS(4660),
    [anon_sym_for] = ACTIONS(4660),
    [anon_sym_gen] = ACTIONS(4660),
    [anon_sym_if] = ACTIONS(4660),
    [anon_sym_impl] = ACTIONS(4660),
    [anon_sym_let] = ACTIONS(4660),
    [anon_sym_loop] = ACTIONS(4660),
    [anon_sym_match] = ACTIONS(4660),
    [anon_sym_mod] = ACTIONS(4660),
    [anon_sym_pub] = ACTIONS(4660),
    [anon_sym_return] = ACTIONS(4660),
    [anon_sym_raw] = ACTIONS(4660),
    [anon_sym_static] = ACTIONS(4660),
    [anon_sym_struct] = ACTIONS(4660),
    [anon_sym_trait] = ACTIONS(4660),
    [anon_sym_type] = ACTIONS(4660),
    [anon_sym_union] = ACTIONS(4660),
    [anon_sym_unsafe] = ACTIONS(4660),
    [anon_sym_use] = ACTIONS(4660),
    [anon_sym_while] = ACTIONS(4660),
    [anon_sym_extern] = ACTIONS(4660),
    [anon_sym_safe] = ACTIONS(4660),
    [anon_sym_yield] = ACTIONS(4660),
    [anon_sym_move] = ACTIONS(4660),
    [anon_sym_try] = ACTIONS(4660),
    [sym_integer_literal] = ACTIONS(4658),
    [aux_sym_string_literal_token1] = ACTIONS(4658),
    [sym_char_literal] = ACTIONS(4658),
    [anon_sym_true] = ACTIONS(4660),
    [anon_sym_false] = ACTIONS(4660),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4660),
    [sym_super] = ACTIONS(4660),
    [sym_crate] = ACTIONS(4660),
    [aux_sym_metavariable_token1] = ACTIONS(4658),
    [sym__raw_string_literal_start] = ACTIONS(4658),
    [sym_float_literal] = ACTIONS(4658),
    [sym__outer_block_doc_comment_start] = ACTIONS(4658),
    [sym__outer_line_doc_comment_start] = ACTIONS(4658),
  },
  [STATE(1231)] = {
    [sym_line_comment] = STATE(1231),
    [sym_block_comment] = STATE(1231),
    [ts_builtin_sym_end] = ACTIONS(4662),
    [sym_identifier] = ACTIONS(4664),
    [anon_sym_SEMI] = ACTIONS(4662),
    [anon_sym_macro] = ACTIONS(4664),
    [anon_sym_LPAREN] = ACTIONS(4662),
    [anon_sym_LBRACE] = ACTIONS(4662),
    [anon_sym_RBRACE] = ACTIONS(4662),
    [anon_sym_macro_rules] = ACTIONS(4664),
    [anon_sym_BANG] = ACTIONS(4662),
    [anon_sym_LBRACK] = ACTIONS(4662),
    [anon_sym_STAR] = ACTIONS(4662),
    [anon_sym_u8] = ACTIONS(4664),
    [anon_sym_i8] = ACTIONS(4664),
    [anon_sym_u16] = ACTIONS(4664),
    [anon_sym_i16] = ACTIONS(4664),
    [anon_sym_u32] = ACTIONS(4664),
    [anon_sym_i32] = ACTIONS(4664),
    [anon_sym_u64] = ACTIONS(4664),
    [anon_sym_i64] = ACTIONS(4664),
    [anon_sym_u128] = ACTIONS(4664),
    [anon_sym_i128] = ACTIONS(4664),
    [anon_sym_isize] = ACTIONS(4664),
    [anon_sym_usize] = ACTIONS(4664),
    [anon_sym_f32] = ACTIONS(4664),
    [anon_sym_f64] = ACTIONS(4664),
    [anon_sym_bool] = ACTIONS(4664),
    [anon_sym_str] = ACTIONS(4664),
    [anon_sym_char] = ACTIONS(4664),
    [anon_sym_DASH] = ACTIONS(4662),
    [anon_sym_AMP] = ACTIONS(4662),
    [anon_sym_PIPE] = ACTIONS(4662),
    [anon_sym_LT] = ACTIONS(4662),
    [anon_sym_DOT_DOT] = ACTIONS(4662),
    [anon_sym_COLON_COLON] = ACTIONS(4662),
    [anon_sym_POUND] = ACTIONS(4662),
    [anon_sym_SQUOTE] = ACTIONS(4664),
    [anon_sym_async] = ACTIONS(4664),
    [anon_sym_auto] = ACTIONS(4664),
    [anon_sym_await] = ACTIONS(4664),
    [anon_sym_become] = ACTIONS(4664),
    [anon_sym_break] = ACTIONS(4664),
    [anon_sym_const] = ACTIONS(4664),
    [anon_sym_continue] = ACTIONS(4664),
    [anon_sym_default] = ACTIONS(4664),
    [anon_sym_enum] = ACTIONS(4664),
    [anon_sym_fn] = ACTIONS(4664),
    [anon_sym_for] = ACTIONS(4664),
    [anon_sym_gen] = ACTIONS(4664),
    [anon_sym_if] = ACTIONS(4664),
    [anon_sym_impl] = ACTIONS(4664),
    [anon_sym_let] = ACTIONS(4664),
    [anon_sym_loop] = ACTIONS(4664),
    [anon_sym_match] = ACTIONS(4664),
    [anon_sym_mod] = ACTIONS(4664),
    [anon_sym_pub] = ACTIONS(4664),
    [anon_sym_return] = ACTIONS(4664),
    [anon_sym_raw] = ACTIONS(4664),
    [anon_sym_static] = ACTIONS(4664),
    [anon_sym_struct] = ACTIONS(4664),
    [anon_sym_trait] = ACTIONS(4664),
    [anon_sym_type] = ACTIONS(4664),
    [anon_sym_union] = ACTIONS(4664),
    [anon_sym_unsafe] = ACTIONS(4664),
    [anon_sym_use] = ACTIONS(4664),
    [anon_sym_while] = ACTIONS(4664),
    [anon_sym_extern] = ACTIONS(4664),
    [anon_sym_safe] = ACTIONS(4664),
    [anon_sym_yield] = ACTIONS(4664),
    [anon_sym_move] = ACTIONS(4664),
    [anon_sym_try] = ACTIONS(4664),
    [sym_integer_literal] = ACTIONS(4662),
    [aux_sym_string_literal_token1] = ACTIONS(4662),
    [sym_char_literal] = ACTIONS(4662),
    [anon_sym_true] = ACTIONS(4664),
    [anon_sym_false] = ACTIONS(4664),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4664),
    [sym_super] = ACTIONS(4664),
    [sym_crate] = ACTIONS(4664),
    [aux_sym_metavariable_token1] = ACTIONS(4662),
    [sym__raw_string_literal_start] = ACTIONS(4662),
    [sym_float_literal] = ACTIONS(4662),
    [sym__outer_block_doc_comment_start] = ACTIONS(4662),
    [sym__outer_line_doc_comment_start] = ACTIONS(4662),
  },
  [STATE(1232)] = {
    [sym_line_comment] = STATE(1232),
    [sym_block_comment] = STATE(1232),
    [ts_builtin_sym_end] = ACTIONS(4666),
    [sym_identifier] = ACTIONS(4668),
    [anon_sym_SEMI] = ACTIONS(4666),
    [anon_sym_macro] = ACTIONS(4668),
    [anon_sym_LPAREN] = ACTIONS(4666),
    [anon_sym_LBRACE] = ACTIONS(4666),
    [anon_sym_RBRACE] = ACTIONS(4666),
    [anon_sym_macro_rules] = ACTIONS(4668),
    [anon_sym_BANG] = ACTIONS(4666),
    [anon_sym_LBRACK] = ACTIONS(4666),
    [anon_sym_STAR] = ACTIONS(4666),
    [anon_sym_u8] = ACTIONS(4668),
    [anon_sym_i8] = ACTIONS(4668),
    [anon_sym_u16] = ACTIONS(4668),
    [anon_sym_i16] = ACTIONS(4668),
    [anon_sym_u32] = ACTIONS(4668),
    [anon_sym_i32] = ACTIONS(4668),
    [anon_sym_u64] = ACTIONS(4668),
    [anon_sym_i64] = ACTIONS(4668),
    [anon_sym_u128] = ACTIONS(4668),
    [anon_sym_i128] = ACTIONS(4668),
    [anon_sym_isize] = ACTIONS(4668),
    [anon_sym_usize] = ACTIONS(4668),
    [anon_sym_f32] = ACTIONS(4668),
    [anon_sym_f64] = ACTIONS(4668),
    [anon_sym_bool] = ACTIONS(4668),
    [anon_sym_str] = ACTIONS(4668),
    [anon_sym_char] = ACTIONS(4668),
    [anon_sym_DASH] = ACTIONS(4666),
    [anon_sym_AMP] = ACTIONS(4666),
    [anon_sym_PIPE] = ACTIONS(4666),
    [anon_sym_LT] = ACTIONS(4666),
    [anon_sym_DOT_DOT] = ACTIONS(4666),
    [anon_sym_COLON_COLON] = ACTIONS(4666),
    [anon_sym_POUND] = ACTIONS(4666),
    [anon_sym_SQUOTE] = ACTIONS(4668),
    [anon_sym_async] = ACTIONS(4668),
    [anon_sym_auto] = ACTIONS(4668),
    [anon_sym_await] = ACTIONS(4668),
    [anon_sym_become] = ACTIONS(4668),
    [anon_sym_break] = ACTIONS(4668),
    [anon_sym_const] = ACTIONS(4668),
    [anon_sym_continue] = ACTIONS(4668),
    [anon_sym_default] = ACTIONS(4668),
    [anon_sym_enum] = ACTIONS(4668),
    [anon_sym_fn] = ACTIONS(4668),
    [anon_sym_for] = ACTIONS(4668),
    [anon_sym_gen] = ACTIONS(4668),
    [anon_sym_if] = ACTIONS(4668),
    [anon_sym_impl] = ACTIONS(4668),
    [anon_sym_let] = ACTIONS(4668),
    [anon_sym_loop] = ACTIONS(4668),
    [anon_sym_match] = ACTIONS(4668),
    [anon_sym_mod] = ACTIONS(4668),
    [anon_sym_pub] = ACTIONS(4668),
    [anon_sym_return] = ACTIONS(4668),
    [anon_sym_raw] = ACTIONS(4668),
    [anon_sym_static] = ACTIONS(4668),
    [anon_sym_struct] = ACTIONS(4668),
    [anon_sym_trait] = ACTIONS(4668),
    [anon_sym_type] = ACTIONS(4668),
    [anon_sym_union] = ACTIONS(4668),
    [anon_sym_unsafe] = ACTIONS(4668),
    [anon_sym_use] = ACTIONS(4668),
    [anon_sym_while] = ACTIONS(4668),
    [anon_sym_extern] = ACTIONS(4668),
    [anon_sym_safe] = ACTIONS(4668),
    [anon_sym_yield] = ACTIONS(4668),
    [anon_sym_move] = ACTIONS(4668),
    [anon_sym_try] = ACTIONS(4668),
    [sym_integer_literal] = ACTIONS(4666),
    [aux_sym_string_literal_token1] = ACTIONS(4666),
    [sym_char_literal] = ACTIONS(4666),
    [anon_sym_true] = ACTIONS(4668),
    [anon_sym_false] = ACTIONS(4668),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4668),
    [sym_super] = ACTIONS(4668),
    [sym_crate] = ACTIONS(4668),
    [aux_sym_metavariable_token1] = ACTIONS(4666),
    [sym__raw_string_literal_start] = ACTIONS(4666),
    [sym_float_literal] = ACTIONS(4666),
    [sym__outer_block_doc_comment_start] = ACTIONS(4666),
    [sym__outer_line_doc_comment_start] = ACTIONS(4666),
  },
  [STATE(1233)] = {
    [sym_line_comment] = STATE(1233),
    [sym_block_comment] = STATE(1233),
    [ts_builtin_sym_end] = ACTIONS(4670),
    [sym_identifier] = ACTIONS(4672),
    [anon_sym_SEMI] = ACTIONS(4670),
    [anon_sym_macro] = ACTIONS(4672),
    [anon_sym_LPAREN] = ACTIONS(4670),
    [anon_sym_LBRACE] = ACTIONS(4670),
    [anon_sym_RBRACE] = ACTIONS(4670),
    [anon_sym_macro_rules] = ACTIONS(4672),
    [anon_sym_BANG] = ACTIONS(4670),
    [anon_sym_LBRACK] = ACTIONS(4670),
    [anon_sym_STAR] = ACTIONS(4670),
    [anon_sym_u8] = ACTIONS(4672),
    [anon_sym_i8] = ACTIONS(4672),
    [anon_sym_u16] = ACTIONS(4672),
    [anon_sym_i16] = ACTIONS(4672),
    [anon_sym_u32] = ACTIONS(4672),
    [anon_sym_i32] = ACTIONS(4672),
    [anon_sym_u64] = ACTIONS(4672),
    [anon_sym_i64] = ACTIONS(4672),
    [anon_sym_u128] = ACTIONS(4672),
    [anon_sym_i128] = ACTIONS(4672),
    [anon_sym_isize] = ACTIONS(4672),
    [anon_sym_usize] = ACTIONS(4672),
    [anon_sym_f32] = ACTIONS(4672),
    [anon_sym_f64] = ACTIONS(4672),
    [anon_sym_bool] = ACTIONS(4672),
    [anon_sym_str] = ACTIONS(4672),
    [anon_sym_char] = ACTIONS(4672),
    [anon_sym_DASH] = ACTIONS(4670),
    [anon_sym_AMP] = ACTIONS(4670),
    [anon_sym_PIPE] = ACTIONS(4670),
    [anon_sym_LT] = ACTIONS(4670),
    [anon_sym_DOT_DOT] = ACTIONS(4670),
    [anon_sym_COLON_COLON] = ACTIONS(4670),
    [anon_sym_POUND] = ACTIONS(4670),
    [anon_sym_SQUOTE] = ACTIONS(4672),
    [anon_sym_async] = ACTIONS(4672),
    [anon_sym_auto] = ACTIONS(4672),
    [anon_sym_await] = ACTIONS(4672),
    [anon_sym_become] = ACTIONS(4672),
    [anon_sym_break] = ACTIONS(4672),
    [anon_sym_const] = ACTIONS(4672),
    [anon_sym_continue] = ACTIONS(4672),
    [anon_sym_default] = ACTIONS(4672),
    [anon_sym_enum] = ACTIONS(4672),
    [anon_sym_fn] = ACTIONS(4672),
    [anon_sym_for] = ACTIONS(4672),
    [anon_sym_gen] = ACTIONS(4672),
    [anon_sym_if] = ACTIONS(4672),
    [anon_sym_impl] = ACTIONS(4672),
    [anon_sym_let] = ACTIONS(4672),
    [anon_sym_loop] = ACTIONS(4672),
    [anon_sym_match] = ACTIONS(4672),
    [anon_sym_mod] = ACTIONS(4672),
    [anon_sym_pub] = ACTIONS(4672),
    [anon_sym_return] = ACTIONS(4672),
    [anon_sym_raw] = ACTIONS(4672),
    [anon_sym_static] = ACTIONS(4672),
    [anon_sym_struct] = ACTIONS(4672),
    [anon_sym_trait] = ACTIONS(4672),
    [anon_sym_type] = ACTIONS(4672),
    [anon_sym_union] = ACTIONS(4672),
    [anon_sym_unsafe] = ACTIONS(4672),
    [anon_sym_use] = ACTIONS(4672),
    [anon_sym_while] = ACTIONS(4672),
    [anon_sym_extern] = ACTIONS(4672),
    [anon_sym_safe] = ACTIONS(4672),
    [anon_sym_yield] = ACTIONS(4672),
    [anon_sym_move] = ACTIONS(4672),
    [anon_sym_try] = ACTIONS(4672),
    [sym_integer_literal] = ACTIONS(4670),
    [aux_sym_string_literal_token1] = ACTIONS(4670),
    [sym_char_literal] = ACTIONS(4670),
    [anon_sym_true] = ACTIONS(4672),
    [anon_sym_false] = ACTIONS(4672),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4672),
    [sym_super] = ACTIONS(4672),
    [sym_crate] = ACTIONS(4672),
    [aux_sym_metavariable_token1] = ACTIONS(4670),
    [sym__raw_string_literal_start] = ACTIONS(4670),
    [sym_float_literal] = ACTIONS(4670),
    [sym__outer_block_doc_comment_start] = ACTIONS(4670),
    [sym__outer_line_doc_comment_start] = ACTIONS(4670),
  },
  [STATE(1234)] = {
    [sym_line_comment] = STATE(1234),
    [sym_block_comment] = STATE(1234),
    [ts_builtin_sym_end] = ACTIONS(4674),
    [sym_identifier] = ACTIONS(4676),
    [anon_sym_SEMI] = ACTIONS(4674),
    [anon_sym_macro] = ACTIONS(4676),
    [anon_sym_LPAREN] = ACTIONS(4674),
    [anon_sym_LBRACE] = ACTIONS(4674),
    [anon_sym_RBRACE] = ACTIONS(4674),
    [anon_sym_macro_rules] = ACTIONS(4676),
    [anon_sym_BANG] = ACTIONS(4674),
    [anon_sym_LBRACK] = ACTIONS(4674),
    [anon_sym_STAR] = ACTIONS(4674),
    [anon_sym_u8] = ACTIONS(4676),
    [anon_sym_i8] = ACTIONS(4676),
    [anon_sym_u16] = ACTIONS(4676),
    [anon_sym_i16] = ACTIONS(4676),
    [anon_sym_u32] = ACTIONS(4676),
    [anon_sym_i32] = ACTIONS(4676),
    [anon_sym_u64] = ACTIONS(4676),
    [anon_sym_i64] = ACTIONS(4676),
    [anon_sym_u128] = ACTIONS(4676),
    [anon_sym_i128] = ACTIONS(4676),
    [anon_sym_isize] = ACTIONS(4676),
    [anon_sym_usize] = ACTIONS(4676),
    [anon_sym_f32] = ACTIONS(4676),
    [anon_sym_f64] = ACTIONS(4676),
    [anon_sym_bool] = ACTIONS(4676),
    [anon_sym_str] = ACTIONS(4676),
    [anon_sym_char] = ACTIONS(4676),
    [anon_sym_DASH] = ACTIONS(4674),
    [anon_sym_AMP] = ACTIONS(4674),
    [anon_sym_PIPE] = ACTIONS(4674),
    [anon_sym_LT] = ACTIONS(4674),
    [anon_sym_DOT_DOT] = ACTIONS(4674),
    [anon_sym_COLON_COLON] = ACTIONS(4674),
    [anon_sym_POUND] = ACTIONS(4674),
    [anon_sym_SQUOTE] = ACTIONS(4676),
    [anon_sym_async] = ACTIONS(4676),
    [anon_sym_auto] = ACTIONS(4676),
    [anon_sym_await] = ACTIONS(4676),
    [anon_sym_become] = ACTIONS(4676),
    [anon_sym_break] = ACTIONS(4676),
    [anon_sym_const] = ACTIONS(4676),
    [anon_sym_continue] = ACTIONS(4676),
    [anon_sym_default] = ACTIONS(4676),
    [anon_sym_enum] = ACTIONS(4676),
    [anon_sym_fn] = ACTIONS(4676),
    [anon_sym_for] = ACTIONS(4676),
    [anon_sym_gen] = ACTIONS(4676),
    [anon_sym_if] = ACTIONS(4676),
    [anon_sym_impl] = ACTIONS(4676),
    [anon_sym_let] = ACTIONS(4676),
    [anon_sym_loop] = ACTIONS(4676),
    [anon_sym_match] = ACTIONS(4676),
    [anon_sym_mod] = ACTIONS(4676),
    [anon_sym_pub] = ACTIONS(4676),
    [anon_sym_return] = ACTIONS(4676),
    [anon_sym_raw] = ACTIONS(4676),
    [anon_sym_static] = ACTIONS(4676),
    [anon_sym_struct] = ACTIONS(4676),
    [anon_sym_trait] = ACTIONS(4676),
    [anon_sym_type] = ACTIONS(4676),
    [anon_sym_union] = ACTIONS(4676),
    [anon_sym_unsafe] = ACTIONS(4676),
    [anon_sym_use] = ACTIONS(4676),
    [anon_sym_while] = ACTIONS(4676),
    [anon_sym_extern] = ACTIONS(4676),
    [anon_sym_safe] = ACTIONS(4676),
    [anon_sym_yield] = ACTIONS(4676),
    [anon_sym_move] = ACTIONS(4676),
    [anon_sym_try] = ACTIONS(4676),
    [sym_integer_literal] = ACTIONS(4674),
    [aux_sym_string_literal_token1] = ACTIONS(4674),
    [sym_char_literal] = ACTIONS(4674),
    [anon_sym_true] = ACTIONS(4676),
    [anon_sym_false] = ACTIONS(4676),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4676),
    [sym_super] = ACTIONS(4676),
    [sym_crate] = ACTIONS(4676),
    [aux_sym_metavariable_token1] = ACTIONS(4674),
    [sym__raw_string_literal_start] = ACTIONS(4674),
    [sym_float_literal] = ACTIONS(4674),
    [sym__outer_block_doc_comment_start] = ACTIONS(4674),
    [sym__outer_line_doc_comment_start] = ACTIONS(4674),
  },
  [STATE(1235)] = {
    [sym_line_comment] = STATE(1235),
    [sym_block_comment] = STATE(1235),
    [ts_builtin_sym_end] = ACTIONS(4678),
    [sym_identifier] = ACTIONS(4680),
    [anon_sym_SEMI] = ACTIONS(4678),
    [anon_sym_macro] = ACTIONS(4680),
    [anon_sym_LPAREN] = ACTIONS(4678),
    [anon_sym_LBRACE] = ACTIONS(4678),
    [anon_sym_RBRACE] = ACTIONS(4678),
    [anon_sym_macro_rules] = ACTIONS(4680),
    [anon_sym_BANG] = ACTIONS(4678),
    [anon_sym_LBRACK] = ACTIONS(4678),
    [anon_sym_STAR] = ACTIONS(4678),
    [anon_sym_u8] = ACTIONS(4680),
    [anon_sym_i8] = ACTIONS(4680),
    [anon_sym_u16] = ACTIONS(4680),
    [anon_sym_i16] = ACTIONS(4680),
    [anon_sym_u32] = ACTIONS(4680),
    [anon_sym_i32] = ACTIONS(4680),
    [anon_sym_u64] = ACTIONS(4680),
    [anon_sym_i64] = ACTIONS(4680),
    [anon_sym_u128] = ACTIONS(4680),
    [anon_sym_i128] = ACTIONS(4680),
    [anon_sym_isize] = ACTIONS(4680),
    [anon_sym_usize] = ACTIONS(4680),
    [anon_sym_f32] = ACTIONS(4680),
    [anon_sym_f64] = ACTIONS(4680),
    [anon_sym_bool] = ACTIONS(4680),
    [anon_sym_str] = ACTIONS(4680),
    [anon_sym_char] = ACTIONS(4680),
    [anon_sym_DASH] = ACTIONS(4678),
    [anon_sym_AMP] = ACTIONS(4678),
    [anon_sym_PIPE] = ACTIONS(4678),
    [anon_sym_LT] = ACTIONS(4678),
    [anon_sym_DOT_DOT] = ACTIONS(4678),
    [anon_sym_COLON_COLON] = ACTIONS(4678),
    [anon_sym_POUND] = ACTIONS(4678),
    [anon_sym_SQUOTE] = ACTIONS(4680),
    [anon_sym_async] = ACTIONS(4680),
    [anon_sym_auto] = ACTIONS(4680),
    [anon_sym_await] = ACTIONS(4680),
    [anon_sym_become] = ACTIONS(4680),
    [anon_sym_break] = ACTIONS(4680),
    [anon_sym_const] = ACTIONS(4680),
    [anon_sym_continue] = ACTIONS(4680),
    [anon_sym_default] = ACTIONS(4680),
    [anon_sym_enum] = ACTIONS(4680),
    [anon_sym_fn] = ACTIONS(4680),
    [anon_sym_for] = ACTIONS(4680),
    [anon_sym_gen] = ACTIONS(4680),
    [anon_sym_if] = ACTIONS(4680),
    [anon_sym_impl] = ACTIONS(4680),
    [anon_sym_let] = ACTIONS(4680),
    [anon_sym_loop] = ACTIONS(4680),
    [anon_sym_match] = ACTIONS(4680),
    [anon_sym_mod] = ACTIONS(4680),
    [anon_sym_pub] = ACTIONS(4680),
    [anon_sym_return] = ACTIONS(4680),
    [anon_sym_raw] = ACTIONS(4680),
    [anon_sym_static] = ACTIONS(4680),
    [anon_sym_struct] = ACTIONS(4680),
    [anon_sym_trait] = ACTIONS(4680),
    [anon_sym_type] = ACTIONS(4680),
    [anon_sym_union] = ACTIONS(4680),
    [anon_sym_unsafe] = ACTIONS(4680),
    [anon_sym_use] = ACTIONS(4680),
    [anon_sym_while] = ACTIONS(4680),
    [anon_sym_extern] = ACTIONS(4680),
    [anon_sym_safe] = ACTIONS(4680),
    [anon_sym_yield] = ACTIONS(4680),
    [anon_sym_move] = ACTIONS(4680),
    [anon_sym_try] = ACTIONS(4680),
    [sym_integer_literal] = ACTIONS(4678),
    [aux_sym_string_literal_token1] = ACTIONS(4678),
    [sym_char_literal] = ACTIONS(4678),
    [anon_sym_true] = ACTIONS(4680),
    [anon_sym_false] = ACTIONS(4680),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4680),
    [sym_super] = ACTIONS(4680),
    [sym_crate] = ACTIONS(4680),
    [aux_sym_metavariable_token1] = ACTIONS(4678),
    [sym__raw_string_literal_start] = ACTIONS(4678),
    [sym_float_literal] = ACTIONS(4678),
    [sym__outer_block_doc_comment_start] = ACTIONS(4678),
    [sym__outer_line_doc_comment_start] = ACTIONS(4678),
  },
  [STATE(1236)] = {
    [sym_line_comment] = STATE(1236),
    [sym_block_comment] = STATE(1236),
    [ts_builtin_sym_end] = ACTIONS(4682),
    [sym_identifier] = ACTIONS(4684),
    [anon_sym_SEMI] = ACTIONS(4682),
    [anon_sym_macro] = ACTIONS(4684),
    [anon_sym_LPAREN] = ACTIONS(4682),
    [anon_sym_LBRACE] = ACTIONS(4682),
    [anon_sym_RBRACE] = ACTIONS(4682),
    [anon_sym_macro_rules] = ACTIONS(4684),
    [anon_sym_BANG] = ACTIONS(4682),
    [anon_sym_LBRACK] = ACTIONS(4682),
    [anon_sym_STAR] = ACTIONS(4682),
    [anon_sym_u8] = ACTIONS(4684),
    [anon_sym_i8] = ACTIONS(4684),
    [anon_sym_u16] = ACTIONS(4684),
    [anon_sym_i16] = ACTIONS(4684),
    [anon_sym_u32] = ACTIONS(4684),
    [anon_sym_i32] = ACTIONS(4684),
    [anon_sym_u64] = ACTIONS(4684),
    [anon_sym_i64] = ACTIONS(4684),
    [anon_sym_u128] = ACTIONS(4684),
    [anon_sym_i128] = ACTIONS(4684),
    [anon_sym_isize] = ACTIONS(4684),
    [anon_sym_usize] = ACTIONS(4684),
    [anon_sym_f32] = ACTIONS(4684),
    [anon_sym_f64] = ACTIONS(4684),
    [anon_sym_bool] = ACTIONS(4684),
    [anon_sym_str] = ACTIONS(4684),
    [anon_sym_char] = ACTIONS(4684),
    [anon_sym_DASH] = ACTIONS(4682),
    [anon_sym_AMP] = ACTIONS(4682),
    [anon_sym_PIPE] = ACTIONS(4682),
    [anon_sym_LT] = ACTIONS(4682),
    [anon_sym_DOT_DOT] = ACTIONS(4682),
    [anon_sym_COLON_COLON] = ACTIONS(4682),
    [anon_sym_POUND] = ACTIONS(4682),
    [anon_sym_SQUOTE] = ACTIONS(4684),
    [anon_sym_async] = ACTIONS(4684),
    [anon_sym_auto] = ACTIONS(4684),
    [anon_sym_await] = ACTIONS(4684),
    [anon_sym_become] = ACTIONS(4684),
    [anon_sym_break] = ACTIONS(4684),
    [anon_sym_const] = ACTIONS(4684),
    [anon_sym_continue] = ACTIONS(4684),
    [anon_sym_default] = ACTIONS(4684),
    [anon_sym_enum] = ACTIONS(4684),
    [anon_sym_fn] = ACTIONS(4684),
    [anon_sym_for] = ACTIONS(4684),
    [anon_sym_gen] = ACTIONS(4684),
    [anon_sym_if] = ACTIONS(4684),
    [anon_sym_impl] = ACTIONS(4684),
    [anon_sym_let] = ACTIONS(4684),
    [anon_sym_loop] = ACTIONS(4684),
    [anon_sym_match] = ACTIONS(4684),
    [anon_sym_mod] = ACTIONS(4684),
    [anon_sym_pub] = ACTIONS(4684),
    [anon_sym_return] = ACTIONS(4684),
    [anon_sym_raw] = ACTIONS(4684),
    [anon_sym_static] = ACTIONS(4684),
    [anon_sym_struct] = ACTIONS(4684),
    [anon_sym_trait] = ACTIONS(4684),
    [anon_sym_type] = ACTIONS(4684),
    [anon_sym_union] = ACTIONS(4684),
    [anon_sym_unsafe] = ACTIONS(4684),
    [anon_sym_use] = ACTIONS(4684),
    [anon_sym_while] = ACTIONS(4684),
    [anon_sym_extern] = ACTIONS(4684),
    [anon_sym_safe] = ACTIONS(4684),
    [anon_sym_yield] = ACTIONS(4684),
    [anon_sym_move] = ACTIONS(4684),
    [anon_sym_try] = ACTIONS(4684),
    [sym_integer_literal] = ACTIONS(4682),
    [aux_sym_string_literal_token1] = ACTIONS(4682),
    [sym_char_literal] = ACTIONS(4682),
    [anon_sym_true] = ACTIONS(4684),
    [anon_sym_false] = ACTIONS(4684),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4684),
    [sym_super] = ACTIONS(4684),
    [sym_crate] = ACTIONS(4684),
    [aux_sym_metavariable_token1] = ACTIONS(4682),
    [sym__raw_string_literal_start] = ACTIONS(4682),
    [sym_float_literal] = ACTIONS(4682),
    [sym__outer_block_doc_comment_start] = ACTIONS(4682),
    [sym__outer_line_doc_comment_start] = ACTIONS(4682),
  },
  [STATE(1237)] = {
    [sym_line_comment] = STATE(1237),
    [sym_block_comment] = STATE(1237),
    [ts_builtin_sym_end] = ACTIONS(4686),
    [sym_identifier] = ACTIONS(4688),
    [anon_sym_SEMI] = ACTIONS(4686),
    [anon_sym_macro] = ACTIONS(4688),
    [anon_sym_LPAREN] = ACTIONS(4686),
    [anon_sym_LBRACE] = ACTIONS(4686),
    [anon_sym_RBRACE] = ACTIONS(4686),
    [anon_sym_macro_rules] = ACTIONS(4688),
    [anon_sym_BANG] = ACTIONS(4686),
    [anon_sym_LBRACK] = ACTIONS(4686),
    [anon_sym_STAR] = ACTIONS(4686),
    [anon_sym_u8] = ACTIONS(4688),
    [anon_sym_i8] = ACTIONS(4688),
    [anon_sym_u16] = ACTIONS(4688),
    [anon_sym_i16] = ACTIONS(4688),
    [anon_sym_u32] = ACTIONS(4688),
    [anon_sym_i32] = ACTIONS(4688),
    [anon_sym_u64] = ACTIONS(4688),
    [anon_sym_i64] = ACTIONS(4688),
    [anon_sym_u128] = ACTIONS(4688),
    [anon_sym_i128] = ACTIONS(4688),
    [anon_sym_isize] = ACTIONS(4688),
    [anon_sym_usize] = ACTIONS(4688),
    [anon_sym_f32] = ACTIONS(4688),
    [anon_sym_f64] = ACTIONS(4688),
    [anon_sym_bool] = ACTIONS(4688),
    [anon_sym_str] = ACTIONS(4688),
    [anon_sym_char] = ACTIONS(4688),
    [anon_sym_DASH] = ACTIONS(4686),
    [anon_sym_AMP] = ACTIONS(4686),
    [anon_sym_PIPE] = ACTIONS(4686),
    [anon_sym_LT] = ACTIONS(4686),
    [anon_sym_DOT_DOT] = ACTIONS(4686),
    [anon_sym_COLON_COLON] = ACTIONS(4686),
    [anon_sym_POUND] = ACTIONS(4686),
    [anon_sym_SQUOTE] = ACTIONS(4688),
    [anon_sym_async] = ACTIONS(4688),
    [anon_sym_auto] = ACTIONS(4688),
    [anon_sym_await] = ACTIONS(4688),
    [anon_sym_become] = ACTIONS(4688),
    [anon_sym_break] = ACTIONS(4688),
    [anon_sym_const] = ACTIONS(4688),
    [anon_sym_continue] = ACTIONS(4688),
    [anon_sym_default] = ACTIONS(4688),
    [anon_sym_enum] = ACTIONS(4688),
    [anon_sym_fn] = ACTIONS(4688),
    [anon_sym_for] = ACTIONS(4688),
    [anon_sym_gen] = ACTIONS(4688),
    [anon_sym_if] = ACTIONS(4688),
    [anon_sym_impl] = ACTIONS(4688),
    [anon_sym_let] = ACTIONS(4688),
    [anon_sym_loop] = ACTIONS(4688),
    [anon_sym_match] = ACTIONS(4688),
    [anon_sym_mod] = ACTIONS(4688),
    [anon_sym_pub] = ACTIONS(4688),
    [anon_sym_return] = ACTIONS(4688),
    [anon_sym_raw] = ACTIONS(4688),
    [anon_sym_static] = ACTIONS(4688),
    [anon_sym_struct] = ACTIONS(4688),
    [anon_sym_trait] = ACTIONS(4688),
    [anon_sym_type] = ACTIONS(4688),
    [anon_sym_union] = ACTIONS(4688),
    [anon_sym_unsafe] = ACTIONS(4688),
    [anon_sym_use] = ACTIONS(4688),
    [anon_sym_while] = ACTIONS(4688),
    [anon_sym_extern] = ACTIONS(4688),
    [anon_sym_safe] = ACTIONS(4688),
    [anon_sym_yield] = ACTIONS(4688),
    [anon_sym_move] = ACTIONS(4688),
    [anon_sym_try] = ACTIONS(4688),
    [sym_integer_literal] = ACTIONS(4686),
    [aux_sym_string_literal_token1] = ACTIONS(4686),
    [sym_char_literal] = ACTIONS(4686),
    [anon_sym_true] = ACTIONS(4688),
    [anon_sym_false] = ACTIONS(4688),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4688),
    [sym_super] = ACTIONS(4688),
    [sym_crate] = ACTIONS(4688),
    [aux_sym_metavariable_token1] = ACTIONS(4686),
    [sym__raw_string_literal_start] = ACTIONS(4686),
    [sym_float_literal] = ACTIONS(4686),
    [sym__outer_block_doc_comment_start] = ACTIONS(4686),
    [sym__outer_line_doc_comment_start] = ACTIONS(4686),
  },
  [STATE(1238)] = {
    [sym_line_comment] = STATE(1238),
    [sym_block_comment] = STATE(1238),
    [ts_builtin_sym_end] = ACTIONS(4690),
    [sym_identifier] = ACTIONS(4692),
    [anon_sym_SEMI] = ACTIONS(4690),
    [anon_sym_macro] = ACTIONS(4692),
    [anon_sym_LPAREN] = ACTIONS(4690),
    [anon_sym_LBRACE] = ACTIONS(4690),
    [anon_sym_RBRACE] = ACTIONS(4690),
    [anon_sym_macro_rules] = ACTIONS(4692),
    [anon_sym_BANG] = ACTIONS(4690),
    [anon_sym_LBRACK] = ACTIONS(4690),
    [anon_sym_STAR] = ACTIONS(4690),
    [anon_sym_u8] = ACTIONS(4692),
    [anon_sym_i8] = ACTIONS(4692),
    [anon_sym_u16] = ACTIONS(4692),
    [anon_sym_i16] = ACTIONS(4692),
    [anon_sym_u32] = ACTIONS(4692),
    [anon_sym_i32] = ACTIONS(4692),
    [anon_sym_u64] = ACTIONS(4692),
    [anon_sym_i64] = ACTIONS(4692),
    [anon_sym_u128] = ACTIONS(4692),
    [anon_sym_i128] = ACTIONS(4692),
    [anon_sym_isize] = ACTIONS(4692),
    [anon_sym_usize] = ACTIONS(4692),
    [anon_sym_f32] = ACTIONS(4692),
    [anon_sym_f64] = ACTIONS(4692),
    [anon_sym_bool] = ACTIONS(4692),
    [anon_sym_str] = ACTIONS(4692),
    [anon_sym_char] = ACTIONS(4692),
    [anon_sym_DASH] = ACTIONS(4690),
    [anon_sym_AMP] = ACTIONS(4690),
    [anon_sym_PIPE] = ACTIONS(4690),
    [anon_sym_LT] = ACTIONS(4690),
    [anon_sym_DOT_DOT] = ACTIONS(4690),
    [anon_sym_COLON_COLON] = ACTIONS(4690),
    [anon_sym_POUND] = ACTIONS(4690),
    [anon_sym_SQUOTE] = ACTIONS(4692),
    [anon_sym_async] = ACTIONS(4692),
    [anon_sym_auto] = ACTIONS(4692),
    [anon_sym_await] = ACTIONS(4692),
    [anon_sym_become] = ACTIONS(4692),
    [anon_sym_break] = ACTIONS(4692),
    [anon_sym_const] = ACTIONS(4692),
    [anon_sym_continue] = ACTIONS(4692),
    [anon_sym_default] = ACTIONS(4692),
    [anon_sym_enum] = ACTIONS(4692),
    [anon_sym_fn] = ACTIONS(4692),
    [anon_sym_for] = ACTIONS(4692),
    [anon_sym_gen] = ACTIONS(4692),
    [anon_sym_if] = ACTIONS(4692),
    [anon_sym_impl] = ACTIONS(4692),
    [anon_sym_let] = ACTIONS(4692),
    [anon_sym_loop] = ACTIONS(4692),
    [anon_sym_match] = ACTIONS(4692),
    [anon_sym_mod] = ACTIONS(4692),
    [anon_sym_pub] = ACTIONS(4692),
    [anon_sym_return] = ACTIONS(4692),
    [anon_sym_raw] = ACTIONS(4692),
    [anon_sym_static] = ACTIONS(4692),
    [anon_sym_struct] = ACTIONS(4692),
    [anon_sym_trait] = ACTIONS(4692),
    [anon_sym_type] = ACTIONS(4692),
    [anon_sym_union] = ACTIONS(4692),
    [anon_sym_unsafe] = ACTIONS(4692),
    [anon_sym_use] = ACTIONS(4692),
    [anon_sym_while] = ACTIONS(4692),
    [anon_sym_extern] = ACTIONS(4692),
    [anon_sym_safe] = ACTIONS(4692),
    [anon_sym_yield] = ACTIONS(4692),
    [anon_sym_move] = ACTIONS(4692),
    [anon_sym_try] = ACTIONS(4692),
    [sym_integer_literal] = ACTIONS(4690),
    [aux_sym_string_literal_token1] = ACTIONS(4690),
    [sym_char_literal] = ACTIONS(4690),
    [anon_sym_true] = ACTIONS(4692),
    [anon_sym_false] = ACTIONS(4692),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4692),
    [sym_super] = ACTIONS(4692),
    [sym_crate] = ACTIONS(4692),
    [aux_sym_metavariable_token1] = ACTIONS(4690),
    [sym__raw_string_literal_start] = ACTIONS(4690),
    [sym_float_literal] = ACTIONS(4690),
    [sym__outer_block_doc_comment_start] = ACTIONS(4690),
    [sym__outer_line_doc_comment_start] = ACTIONS(4690),
  },
  [STATE(1239)] = {
    [sym_line_comment] = STATE(1239),
    [sym_block_comment] = STATE(1239),
    [ts_builtin_sym_end] = ACTIONS(4694),
    [sym_identifier] = ACTIONS(4696),
    [anon_sym_SEMI] = ACTIONS(4694),
    [anon_sym_macro] = ACTIONS(4696),
    [anon_sym_LPAREN] = ACTIONS(4694),
    [anon_sym_LBRACE] = ACTIONS(4694),
    [anon_sym_RBRACE] = ACTIONS(4694),
    [anon_sym_macro_rules] = ACTIONS(4696),
    [anon_sym_BANG] = ACTIONS(4694),
    [anon_sym_LBRACK] = ACTIONS(4694),
    [anon_sym_STAR] = ACTIONS(4694),
    [anon_sym_u8] = ACTIONS(4696),
    [anon_sym_i8] = ACTIONS(4696),
    [anon_sym_u16] = ACTIONS(4696),
    [anon_sym_i16] = ACTIONS(4696),
    [anon_sym_u32] = ACTIONS(4696),
    [anon_sym_i32] = ACTIONS(4696),
    [anon_sym_u64] = ACTIONS(4696),
    [anon_sym_i64] = ACTIONS(4696),
    [anon_sym_u128] = ACTIONS(4696),
    [anon_sym_i128] = ACTIONS(4696),
    [anon_sym_isize] = ACTIONS(4696),
    [anon_sym_usize] = ACTIONS(4696),
    [anon_sym_f32] = ACTIONS(4696),
    [anon_sym_f64] = ACTIONS(4696),
    [anon_sym_bool] = ACTIONS(4696),
    [anon_sym_str] = ACTIONS(4696),
    [anon_sym_char] = ACTIONS(4696),
    [anon_sym_DASH] = ACTIONS(4694),
    [anon_sym_AMP] = ACTIONS(4694),
    [anon_sym_PIPE] = ACTIONS(4694),
    [anon_sym_LT] = ACTIONS(4694),
    [anon_sym_DOT_DOT] = ACTIONS(4694),
    [anon_sym_COLON_COLON] = ACTIONS(4694),
    [anon_sym_POUND] = ACTIONS(4694),
    [anon_sym_SQUOTE] = ACTIONS(4696),
    [anon_sym_async] = ACTIONS(4696),
    [anon_sym_auto] = ACTIONS(4696),
    [anon_sym_await] = ACTIONS(4696),
    [anon_sym_become] = ACTIONS(4696),
    [anon_sym_break] = ACTIONS(4696),
    [anon_sym_const] = ACTIONS(4696),
    [anon_sym_continue] = ACTIONS(4696),
    [anon_sym_default] = ACTIONS(4696),
    [anon_sym_enum] = ACTIONS(4696),
    [anon_sym_fn] = ACTIONS(4696),
    [anon_sym_for] = ACTIONS(4696),
    [anon_sym_gen] = ACTIONS(4696),
    [anon_sym_if] = ACTIONS(4696),
    [anon_sym_impl] = ACTIONS(4696),
    [anon_sym_let] = ACTIONS(4696),
    [anon_sym_loop] = ACTIONS(4696),
    [anon_sym_match] = ACTIONS(4696),
    [anon_sym_mod] = ACTIONS(4696),
    [anon_sym_pub] = ACTIONS(4696),
    [anon_sym_return] = ACTIONS(4696),
    [anon_sym_raw] = ACTIONS(4696),
    [anon_sym_static] = ACTIONS(4696),
    [anon_sym_struct] = ACTIONS(4696),
    [anon_sym_trait] = ACTIONS(4696),
    [anon_sym_type] = ACTIONS(4696),
    [anon_sym_union] = ACTIONS(4696),
    [anon_sym_unsafe] = ACTIONS(4696),
    [anon_sym_use] = ACTIONS(4696),
    [anon_sym_while] = ACTIONS(4696),
    [anon_sym_extern] = ACTIONS(4696),
    [anon_sym_safe] = ACTIONS(4696),
    [anon_sym_yield] = ACTIONS(4696),
    [anon_sym_move] = ACTIONS(4696),
    [anon_sym_try] = ACTIONS(4696),
    [sym_integer_literal] = ACTIONS(4694),
    [aux_sym_string_literal_token1] = ACTIONS(4694),
    [sym_char_literal] = ACTIONS(4694),
    [anon_sym_true] = ACTIONS(4696),
    [anon_sym_false] = ACTIONS(4696),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4696),
    [sym_super] = ACTIONS(4696),
    [sym_crate] = ACTIONS(4696),
    [aux_sym_metavariable_token1] = ACTIONS(4694),
    [sym__raw_string_literal_start] = ACTIONS(4694),
    [sym_float_literal] = ACTIONS(4694),
    [sym__outer_block_doc_comment_start] = ACTIONS(4694),
    [sym__outer_line_doc_comment_start] = ACTIONS(4694),
  },
  [STATE(1240)] = {
    [sym_line_comment] = STATE(1240),
    [sym_block_comment] = STATE(1240),
    [ts_builtin_sym_end] = ACTIONS(4698),
    [sym_identifier] = ACTIONS(4700),
    [anon_sym_SEMI] = ACTIONS(4698),
    [anon_sym_macro] = ACTIONS(4700),
    [anon_sym_LPAREN] = ACTIONS(4698),
    [anon_sym_LBRACE] = ACTIONS(4698),
    [anon_sym_RBRACE] = ACTIONS(4698),
    [anon_sym_macro_rules] = ACTIONS(4700),
    [anon_sym_BANG] = ACTIONS(4698),
    [anon_sym_LBRACK] = ACTIONS(4698),
    [anon_sym_STAR] = ACTIONS(4698),
    [anon_sym_u8] = ACTIONS(4700),
    [anon_sym_i8] = ACTIONS(4700),
    [anon_sym_u16] = ACTIONS(4700),
    [anon_sym_i16] = ACTIONS(4700),
    [anon_sym_u32] = ACTIONS(4700),
    [anon_sym_i32] = ACTIONS(4700),
    [anon_sym_u64] = ACTIONS(4700),
    [anon_sym_i64] = ACTIONS(4700),
    [anon_sym_u128] = ACTIONS(4700),
    [anon_sym_i128] = ACTIONS(4700),
    [anon_sym_isize] = ACTIONS(4700),
    [anon_sym_usize] = ACTIONS(4700),
    [anon_sym_f32] = ACTIONS(4700),
    [anon_sym_f64] = ACTIONS(4700),
    [anon_sym_bool] = ACTIONS(4700),
    [anon_sym_str] = ACTIONS(4700),
    [anon_sym_char] = ACTIONS(4700),
    [anon_sym_DASH] = ACTIONS(4698),
    [anon_sym_AMP] = ACTIONS(4698),
    [anon_sym_PIPE] = ACTIONS(4698),
    [anon_sym_LT] = ACTIONS(4698),
    [anon_sym_DOT_DOT] = ACTIONS(4698),
    [anon_sym_COLON_COLON] = ACTIONS(4698),
    [anon_sym_POUND] = ACTIONS(4698),
    [anon_sym_SQUOTE] = ACTIONS(4700),
    [anon_sym_async] = ACTIONS(4700),
    [anon_sym_auto] = ACTIONS(4700),
    [anon_sym_await] = ACTIONS(4700),
    [anon_sym_become] = ACTIONS(4700),
    [anon_sym_break] = ACTIONS(4700),
    [anon_sym_const] = ACTIONS(4700),
    [anon_sym_continue] = ACTIONS(4700),
    [anon_sym_default] = ACTIONS(4700),
    [anon_sym_enum] = ACTIONS(4700),
    [anon_sym_fn] = ACTIONS(4700),
    [anon_sym_for] = ACTIONS(4700),
    [anon_sym_gen] = ACTIONS(4700),
    [anon_sym_if] = ACTIONS(4700),
    [anon_sym_impl] = ACTIONS(4700),
    [anon_sym_let] = ACTIONS(4700),
    [anon_sym_loop] = ACTIONS(4700),
    [anon_sym_match] = ACTIONS(4700),
    [anon_sym_mod] = ACTIONS(4700),
    [anon_sym_pub] = ACTIONS(4700),
    [anon_sym_return] = ACTIONS(4700),
    [anon_sym_raw] = ACTIONS(4700),
    [anon_sym_static] = ACTIONS(4700),
    [anon_sym_struct] = ACTIONS(4700),
    [anon_sym_trait] = ACTIONS(4700),
    [anon_sym_type] = ACTIONS(4700),
    [anon_sym_union] = ACTIONS(4700),
    [anon_sym_unsafe] = ACTIONS(4700),
    [anon_sym_use] = ACTIONS(4700),
    [anon_sym_while] = ACTIONS(4700),
    [anon_sym_extern] = ACTIONS(4700),
    [anon_sym_safe] = ACTIONS(4700),
    [anon_sym_yield] = ACTIONS(4700),
    [anon_sym_move] = ACTIONS(4700),
    [anon_sym_try] = ACTIONS(4700),
    [sym_integer_literal] = ACTIONS(4698),
    [aux_sym_string_literal_token1] = ACTIONS(4698),
    [sym_char_literal] = ACTIONS(4698),
    [anon_sym_true] = ACTIONS(4700),
    [anon_sym_false] = ACTIONS(4700),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4700),
    [sym_super] = ACTIONS(4700),
    [sym_crate] = ACTIONS(4700),
    [aux_sym_metavariable_token1] = ACTIONS(4698),
    [sym__raw_string_literal_start] = ACTIONS(4698),
    [sym_float_literal] = ACTIONS(4698),
    [sym__outer_block_doc_comment_start] = ACTIONS(4698),
    [sym__outer_line_doc_comment_start] = ACTIONS(4698),
  },
  [STATE(1241)] = {
    [sym_line_comment] = STATE(1241),
    [sym_block_comment] = STATE(1241),
    [ts_builtin_sym_end] = ACTIONS(4702),
    [sym_identifier] = ACTIONS(4704),
    [anon_sym_SEMI] = ACTIONS(4702),
    [anon_sym_macro] = ACTIONS(4704),
    [anon_sym_LPAREN] = ACTIONS(4702),
    [anon_sym_LBRACE] = ACTIONS(4702),
    [anon_sym_RBRACE] = ACTIONS(4702),
    [anon_sym_macro_rules] = ACTIONS(4704),
    [anon_sym_BANG] = ACTIONS(4702),
    [anon_sym_LBRACK] = ACTIONS(4702),
    [anon_sym_STAR] = ACTIONS(4702),
    [anon_sym_u8] = ACTIONS(4704),
    [anon_sym_i8] = ACTIONS(4704),
    [anon_sym_u16] = ACTIONS(4704),
    [anon_sym_i16] = ACTIONS(4704),
    [anon_sym_u32] = ACTIONS(4704),
    [anon_sym_i32] = ACTIONS(4704),
    [anon_sym_u64] = ACTIONS(4704),
    [anon_sym_i64] = ACTIONS(4704),
    [anon_sym_u128] = ACTIONS(4704),
    [anon_sym_i128] = ACTIONS(4704),
    [anon_sym_isize] = ACTIONS(4704),
    [anon_sym_usize] = ACTIONS(4704),
    [anon_sym_f32] = ACTIONS(4704),
    [anon_sym_f64] = ACTIONS(4704),
    [anon_sym_bool] = ACTIONS(4704),
    [anon_sym_str] = ACTIONS(4704),
    [anon_sym_char] = ACTIONS(4704),
    [anon_sym_DASH] = ACTIONS(4702),
    [anon_sym_AMP] = ACTIONS(4702),
    [anon_sym_PIPE] = ACTIONS(4702),
    [anon_sym_LT] = ACTIONS(4702),
    [anon_sym_DOT_DOT] = ACTIONS(4702),
    [anon_sym_COLON_COLON] = ACTIONS(4702),
    [anon_sym_POUND] = ACTIONS(4702),
    [anon_sym_SQUOTE] = ACTIONS(4704),
    [anon_sym_async] = ACTIONS(4704),
    [anon_sym_auto] = ACTIONS(4704),
    [anon_sym_await] = ACTIONS(4704),
    [anon_sym_become] = ACTIONS(4704),
    [anon_sym_break] = ACTIONS(4704),
    [anon_sym_const] = ACTIONS(4704),
    [anon_sym_continue] = ACTIONS(4704),
    [anon_sym_default] = ACTIONS(4704),
    [anon_sym_enum] = ACTIONS(4704),
    [anon_sym_fn] = ACTIONS(4704),
    [anon_sym_for] = ACTIONS(4704),
    [anon_sym_gen] = ACTIONS(4704),
    [anon_sym_if] = ACTIONS(4704),
    [anon_sym_impl] = ACTIONS(4704),
    [anon_sym_let] = ACTIONS(4704),
    [anon_sym_loop] = ACTIONS(4704),
    [anon_sym_match] = ACTIONS(4704),
    [anon_sym_mod] = ACTIONS(4704),
    [anon_sym_pub] = ACTIONS(4704),
    [anon_sym_return] = ACTIONS(4704),
    [anon_sym_raw] = ACTIONS(4704),
    [anon_sym_static] = ACTIONS(4704),
    [anon_sym_struct] = ACTIONS(4704),
    [anon_sym_trait] = ACTIONS(4704),
    [anon_sym_type] = ACTIONS(4704),
    [anon_sym_union] = ACTIONS(4704),
    [anon_sym_unsafe] = ACTIONS(4704),
    [anon_sym_use] = ACTIONS(4704),
    [anon_sym_while] = ACTIONS(4704),
    [anon_sym_extern] = ACTIONS(4704),
    [anon_sym_safe] = ACTIONS(4704),
    [anon_sym_yield] = ACTIONS(4704),
    [anon_sym_move] = ACTIONS(4704),
    [anon_sym_try] = ACTIONS(4704),
    [sym_integer_literal] = ACTIONS(4702),
    [aux_sym_string_literal_token1] = ACTIONS(4702),
    [sym_char_literal] = ACTIONS(4702),
    [anon_sym_true] = ACTIONS(4704),
    [anon_sym_false] = ACTIONS(4704),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4704),
    [sym_super] = ACTIONS(4704),
    [sym_crate] = ACTIONS(4704),
    [aux_sym_metavariable_token1] = ACTIONS(4702),
    [sym__raw_string_literal_start] = ACTIONS(4702),
    [sym_float_literal] = ACTIONS(4702),
    [sym__outer_block_doc_comment_start] = ACTIONS(4702),
    [sym__outer_line_doc_comment_start] = ACTIONS(4702),
  },
  [STATE(1242)] = {
    [sym_line_comment] = STATE(1242),
    [sym_block_comment] = STATE(1242),
    [ts_builtin_sym_end] = ACTIONS(4706),
    [sym_identifier] = ACTIONS(4708),
    [anon_sym_SEMI] = ACTIONS(4706),
    [anon_sym_macro] = ACTIONS(4708),
    [anon_sym_LPAREN] = ACTIONS(4706),
    [anon_sym_LBRACE] = ACTIONS(4706),
    [anon_sym_RBRACE] = ACTIONS(4706),
    [anon_sym_macro_rules] = ACTIONS(4708),
    [anon_sym_BANG] = ACTIONS(4706),
    [anon_sym_LBRACK] = ACTIONS(4706),
    [anon_sym_STAR] = ACTIONS(4706),
    [anon_sym_u8] = ACTIONS(4708),
    [anon_sym_i8] = ACTIONS(4708),
    [anon_sym_u16] = ACTIONS(4708),
    [anon_sym_i16] = ACTIONS(4708),
    [anon_sym_u32] = ACTIONS(4708),
    [anon_sym_i32] = ACTIONS(4708),
    [anon_sym_u64] = ACTIONS(4708),
    [anon_sym_i64] = ACTIONS(4708),
    [anon_sym_u128] = ACTIONS(4708),
    [anon_sym_i128] = ACTIONS(4708),
    [anon_sym_isize] = ACTIONS(4708),
    [anon_sym_usize] = ACTIONS(4708),
    [anon_sym_f32] = ACTIONS(4708),
    [anon_sym_f64] = ACTIONS(4708),
    [anon_sym_bool] = ACTIONS(4708),
    [anon_sym_str] = ACTIONS(4708),
    [anon_sym_char] = ACTIONS(4708),
    [anon_sym_DASH] = ACTIONS(4706),
    [anon_sym_AMP] = ACTIONS(4706),
    [anon_sym_PIPE] = ACTIONS(4706),
    [anon_sym_LT] = ACTIONS(4706),
    [anon_sym_DOT_DOT] = ACTIONS(4706),
    [anon_sym_COLON_COLON] = ACTIONS(4706),
    [anon_sym_POUND] = ACTIONS(4706),
    [anon_sym_SQUOTE] = ACTIONS(4708),
    [anon_sym_async] = ACTIONS(4708),
    [anon_sym_auto] = ACTIONS(4708),
    [anon_sym_await] = ACTIONS(4708),
    [anon_sym_become] = ACTIONS(4708),
    [anon_sym_break] = ACTIONS(4708),
    [anon_sym_const] = ACTIONS(4708),
    [anon_sym_continue] = ACTIONS(4708),
    [anon_sym_default] = ACTIONS(4708),
    [anon_sym_enum] = ACTIONS(4708),
    [anon_sym_fn] = ACTIONS(4708),
    [anon_sym_for] = ACTIONS(4708),
    [anon_sym_gen] = ACTIONS(4708),
    [anon_sym_if] = ACTIONS(4708),
    [anon_sym_impl] = ACTIONS(4708),
    [anon_sym_let] = ACTIONS(4708),
    [anon_sym_loop] = ACTIONS(4708),
    [anon_sym_match] = ACTIONS(4708),
    [anon_sym_mod] = ACTIONS(4708),
    [anon_sym_pub] = ACTIONS(4708),
    [anon_sym_return] = ACTIONS(4708),
    [anon_sym_raw] = ACTIONS(4708),
    [anon_sym_static] = ACTIONS(4708),
    [anon_sym_struct] = ACTIONS(4708),
    [anon_sym_trait] = ACTIONS(4708),
    [anon_sym_type] = ACTIONS(4708),
    [anon_sym_union] = ACTIONS(4708),
    [anon_sym_unsafe] = ACTIONS(4708),
    [anon_sym_use] = ACTIONS(4708),
    [anon_sym_while] = ACTIONS(4708),
    [anon_sym_extern] = ACTIONS(4708),
    [anon_sym_safe] = ACTIONS(4708),
    [anon_sym_yield] = ACTIONS(4708),
    [anon_sym_move] = ACTIONS(4708),
    [anon_sym_try] = ACTIONS(4708),
    [sym_integer_literal] = ACTIONS(4706),
    [aux_sym_string_literal_token1] = ACTIONS(4706),
    [sym_char_literal] = ACTIONS(4706),
    [anon_sym_true] = ACTIONS(4708),
    [anon_sym_false] = ACTIONS(4708),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4708),
    [sym_super] = ACTIONS(4708),
    [sym_crate] = ACTIONS(4708),
    [aux_sym_metavariable_token1] = ACTIONS(4706),
    [sym__raw_string_literal_start] = ACTIONS(4706),
    [sym_float_literal] = ACTIONS(4706),
    [sym__outer_block_doc_comment_start] = ACTIONS(4706),
    [sym__outer_line_doc_comment_start] = ACTIONS(4706),
  },
  [STATE(1243)] = {
    [sym_line_comment] = STATE(1243),
    [sym_block_comment] = STATE(1243),
    [ts_builtin_sym_end] = ACTIONS(4710),
    [sym_identifier] = ACTIONS(4712),
    [anon_sym_SEMI] = ACTIONS(4710),
    [anon_sym_macro] = ACTIONS(4712),
    [anon_sym_LPAREN] = ACTIONS(4710),
    [anon_sym_LBRACE] = ACTIONS(4710),
    [anon_sym_RBRACE] = ACTIONS(4710),
    [anon_sym_macro_rules] = ACTIONS(4712),
    [anon_sym_BANG] = ACTIONS(4710),
    [anon_sym_LBRACK] = ACTIONS(4710),
    [anon_sym_STAR] = ACTIONS(4710),
    [anon_sym_u8] = ACTIONS(4712),
    [anon_sym_i8] = ACTIONS(4712),
    [anon_sym_u16] = ACTIONS(4712),
    [anon_sym_i16] = ACTIONS(4712),
    [anon_sym_u32] = ACTIONS(4712),
    [anon_sym_i32] = ACTIONS(4712),
    [anon_sym_u64] = ACTIONS(4712),
    [anon_sym_i64] = ACTIONS(4712),
    [anon_sym_u128] = ACTIONS(4712),
    [anon_sym_i128] = ACTIONS(4712),
    [anon_sym_isize] = ACTIONS(4712),
    [anon_sym_usize] = ACTIONS(4712),
    [anon_sym_f32] = ACTIONS(4712),
    [anon_sym_f64] = ACTIONS(4712),
    [anon_sym_bool] = ACTIONS(4712),
    [anon_sym_str] = ACTIONS(4712),
    [anon_sym_char] = ACTIONS(4712),
    [anon_sym_DASH] = ACTIONS(4710),
    [anon_sym_AMP] = ACTIONS(4710),
    [anon_sym_PIPE] = ACTIONS(4710),
    [anon_sym_LT] = ACTIONS(4710),
    [anon_sym_DOT_DOT] = ACTIONS(4710),
    [anon_sym_COLON_COLON] = ACTIONS(4710),
    [anon_sym_POUND] = ACTIONS(4710),
    [anon_sym_SQUOTE] = ACTIONS(4712),
    [anon_sym_async] = ACTIONS(4712),
    [anon_sym_auto] = ACTIONS(4712),
    [anon_sym_await] = ACTIONS(4712),
    [anon_sym_become] = ACTIONS(4712),
    [anon_sym_break] = ACTIONS(4712),
    [anon_sym_const] = ACTIONS(4712),
    [anon_sym_continue] = ACTIONS(4712),
    [anon_sym_default] = ACTIONS(4712),
    [anon_sym_enum] = ACTIONS(4712),
    [anon_sym_fn] = ACTIONS(4712),
    [anon_sym_for] = ACTIONS(4712),
    [anon_sym_gen] = ACTIONS(4712),
    [anon_sym_if] = ACTIONS(4712),
    [anon_sym_impl] = ACTIONS(4712),
    [anon_sym_let] = ACTIONS(4712),
    [anon_sym_loop] = ACTIONS(4712),
    [anon_sym_match] = ACTIONS(4712),
    [anon_sym_mod] = ACTIONS(4712),
    [anon_sym_pub] = ACTIONS(4712),
    [anon_sym_return] = ACTIONS(4712),
    [anon_sym_raw] = ACTIONS(4712),
    [anon_sym_static] = ACTIONS(4712),
    [anon_sym_struct] = ACTIONS(4712),
    [anon_sym_trait] = ACTIONS(4712),
    [anon_sym_type] = ACTIONS(4712),
    [anon_sym_union] = ACTIONS(4712),
    [anon_sym_unsafe] = ACTIONS(4712),
    [anon_sym_use] = ACTIONS(4712),
    [anon_sym_while] = ACTIONS(4712),
    [anon_sym_extern] = ACTIONS(4712),
    [anon_sym_safe] = ACTIONS(4712),
    [anon_sym_yield] = ACTIONS(4712),
    [anon_sym_move] = ACTIONS(4712),
    [anon_sym_try] = ACTIONS(4712),
    [sym_integer_literal] = ACTIONS(4710),
    [aux_sym_string_literal_token1] = ACTIONS(4710),
    [sym_char_literal] = ACTIONS(4710),
    [anon_sym_true] = ACTIONS(4712),
    [anon_sym_false] = ACTIONS(4712),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4712),
    [sym_super] = ACTIONS(4712),
    [sym_crate] = ACTIONS(4712),
    [aux_sym_metavariable_token1] = ACTIONS(4710),
    [sym__raw_string_literal_start] = ACTIONS(4710),
    [sym_float_literal] = ACTIONS(4710),
    [sym__outer_block_doc_comment_start] = ACTIONS(4710),
    [sym__outer_line_doc_comment_start] = ACTIONS(4710),
  },
  [STATE(1244)] = {
    [sym_line_comment] = STATE(1244),
    [sym_block_comment] = STATE(1244),
    [ts_builtin_sym_end] = ACTIONS(4714),
    [sym_identifier] = ACTIONS(4716),
    [anon_sym_SEMI] = ACTIONS(4714),
    [anon_sym_macro] = ACTIONS(4716),
    [anon_sym_LPAREN] = ACTIONS(4714),
    [anon_sym_LBRACE] = ACTIONS(4714),
    [anon_sym_RBRACE] = ACTIONS(4714),
    [anon_sym_macro_rules] = ACTIONS(4716),
    [anon_sym_BANG] = ACTIONS(4714),
    [anon_sym_LBRACK] = ACTIONS(4714),
    [anon_sym_STAR] = ACTIONS(4714),
    [anon_sym_u8] = ACTIONS(4716),
    [anon_sym_i8] = ACTIONS(4716),
    [anon_sym_u16] = ACTIONS(4716),
    [anon_sym_i16] = ACTIONS(4716),
    [anon_sym_u32] = ACTIONS(4716),
    [anon_sym_i32] = ACTIONS(4716),
    [anon_sym_u64] = ACTIONS(4716),
    [anon_sym_i64] = ACTIONS(4716),
    [anon_sym_u128] = ACTIONS(4716),
    [anon_sym_i128] = ACTIONS(4716),
    [anon_sym_isize] = ACTIONS(4716),
    [anon_sym_usize] = ACTIONS(4716),
    [anon_sym_f32] = ACTIONS(4716),
    [anon_sym_f64] = ACTIONS(4716),
    [anon_sym_bool] = ACTIONS(4716),
    [anon_sym_str] = ACTIONS(4716),
    [anon_sym_char] = ACTIONS(4716),
    [anon_sym_DASH] = ACTIONS(4714),
    [anon_sym_AMP] = ACTIONS(4714),
    [anon_sym_PIPE] = ACTIONS(4714),
    [anon_sym_LT] = ACTIONS(4714),
    [anon_sym_DOT_DOT] = ACTIONS(4714),
    [anon_sym_COLON_COLON] = ACTIONS(4714),
    [anon_sym_POUND] = ACTIONS(4714),
    [anon_sym_SQUOTE] = ACTIONS(4716),
    [anon_sym_async] = ACTIONS(4716),
    [anon_sym_auto] = ACTIONS(4716),
    [anon_sym_await] = ACTIONS(4716),
    [anon_sym_become] = ACTIONS(4716),
    [anon_sym_break] = ACTIONS(4716),
    [anon_sym_const] = ACTIONS(4716),
    [anon_sym_continue] = ACTIONS(4716),
    [anon_sym_default] = ACTIONS(4716),
    [anon_sym_enum] = ACTIONS(4716),
    [anon_sym_fn] = ACTIONS(4716),
    [anon_sym_for] = ACTIONS(4716),
    [anon_sym_gen] = ACTIONS(4716),
    [anon_sym_if] = ACTIONS(4716),
    [anon_sym_impl] = ACTIONS(4716),
    [anon_sym_let] = ACTIONS(4716),
    [anon_sym_loop] = ACTIONS(4716),
    [anon_sym_match] = ACTIONS(4716),
    [anon_sym_mod] = ACTIONS(4716),
    [anon_sym_pub] = ACTIONS(4716),
    [anon_sym_return] = ACTIONS(4716),
    [anon_sym_raw] = ACTIONS(4716),
    [anon_sym_static] = ACTIONS(4716),
    [anon_sym_struct] = ACTIONS(4716),
    [anon_sym_trait] = ACTIONS(4716),
    [anon_sym_type] = ACTIONS(4716),
    [anon_sym_union] = ACTIONS(4716),
    [anon_sym_unsafe] = ACTIONS(4716),
    [anon_sym_use] = ACTIONS(4716),
    [anon_sym_while] = ACTIONS(4716),
    [anon_sym_extern] = ACTIONS(4716),
    [anon_sym_safe] = ACTIONS(4716),
    [anon_sym_yield] = ACTIONS(4716),
    [anon_sym_move] = ACTIONS(4716),
    [anon_sym_try] = ACTIONS(4716),
    [sym_integer_literal] = ACTIONS(4714),
    [aux_sym_string_literal_token1] = ACTIONS(4714),
    [sym_char_literal] = ACTIONS(4714),
    [anon_sym_true] = ACTIONS(4716),
    [anon_sym_false] = ACTIONS(4716),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4716),
    [sym_super] = ACTIONS(4716),
    [sym_crate] = ACTIONS(4716),
    [aux_sym_metavariable_token1] = ACTIONS(4714),
    [sym__raw_string_literal_start] = ACTIONS(4714),
    [sym_float_literal] = ACTIONS(4714),
    [sym__outer_block_doc_comment_start] = ACTIONS(4714),
    [sym__outer_line_doc_comment_start] = ACTIONS(4714),
  },
  [STATE(1245)] = {
    [sym_line_comment] = STATE(1245),
    [sym_block_comment] = STATE(1245),
    [ts_builtin_sym_end] = ACTIONS(4718),
    [sym_identifier] = ACTIONS(4720),
    [anon_sym_SEMI] = ACTIONS(4718),
    [anon_sym_macro] = ACTIONS(4720),
    [anon_sym_LPAREN] = ACTIONS(4718),
    [anon_sym_LBRACE] = ACTIONS(4718),
    [anon_sym_RBRACE] = ACTIONS(4718),
    [anon_sym_macro_rules] = ACTIONS(4720),
    [anon_sym_BANG] = ACTIONS(4718),
    [anon_sym_LBRACK] = ACTIONS(4718),
    [anon_sym_STAR] = ACTIONS(4718),
    [anon_sym_u8] = ACTIONS(4720),
    [anon_sym_i8] = ACTIONS(4720),
    [anon_sym_u16] = ACTIONS(4720),
    [anon_sym_i16] = ACTIONS(4720),
    [anon_sym_u32] = ACTIONS(4720),
    [anon_sym_i32] = ACTIONS(4720),
    [anon_sym_u64] = ACTIONS(4720),
    [anon_sym_i64] = ACTIONS(4720),
    [anon_sym_u128] = ACTIONS(4720),
    [anon_sym_i128] = ACTIONS(4720),
    [anon_sym_isize] = ACTIONS(4720),
    [anon_sym_usize] = ACTIONS(4720),
    [anon_sym_f32] = ACTIONS(4720),
    [anon_sym_f64] = ACTIONS(4720),
    [anon_sym_bool] = ACTIONS(4720),
    [anon_sym_str] = ACTIONS(4720),
    [anon_sym_char] = ACTIONS(4720),
    [anon_sym_DASH] = ACTIONS(4718),
    [anon_sym_AMP] = ACTIONS(4718),
    [anon_sym_PIPE] = ACTIONS(4718),
    [anon_sym_LT] = ACTIONS(4718),
    [anon_sym_DOT_DOT] = ACTIONS(4718),
    [anon_sym_COLON_COLON] = ACTIONS(4718),
    [anon_sym_POUND] = ACTIONS(4718),
    [anon_sym_SQUOTE] = ACTIONS(4720),
    [anon_sym_async] = ACTIONS(4720),
    [anon_sym_auto] = ACTIONS(4720),
    [anon_sym_await] = ACTIONS(4720),
    [anon_sym_become] = ACTIONS(4720),
    [anon_sym_break] = ACTIONS(4720),
    [anon_sym_const] = ACTIONS(4720),
    [anon_sym_continue] = ACTIONS(4720),
    [anon_sym_default] = ACTIONS(4720),
    [anon_sym_enum] = ACTIONS(4720),
    [anon_sym_fn] = ACTIONS(4720),
    [anon_sym_for] = ACTIONS(4720),
    [anon_sym_gen] = ACTIONS(4720),
    [anon_sym_if] = ACTIONS(4720),
    [anon_sym_impl] = ACTIONS(4720),
    [anon_sym_let] = ACTIONS(4720),
    [anon_sym_loop] = ACTIONS(4720),
    [anon_sym_match] = ACTIONS(4720),
    [anon_sym_mod] = ACTIONS(4720),
    [anon_sym_pub] = ACTIONS(4720),
    [anon_sym_return] = ACTIONS(4720),
    [anon_sym_raw] = ACTIONS(4720),
    [anon_sym_static] = ACTIONS(4720),
    [anon_sym_struct] = ACTIONS(4720),
    [anon_sym_trait] = ACTIONS(4720),
    [anon_sym_type] = ACTIONS(4720),
    [anon_sym_union] = ACTIONS(4720),
    [anon_sym_unsafe] = ACTIONS(4720),
    [anon_sym_use] = ACTIONS(4720),
    [anon_sym_while] = ACTIONS(4720),
    [anon_sym_extern] = ACTIONS(4720),
    [anon_sym_safe] = ACTIONS(4720),
    [anon_sym_yield] = ACTIONS(4720),
    [anon_sym_move] = ACTIONS(4720),
    [anon_sym_try] = ACTIONS(4720),
    [sym_integer_literal] = ACTIONS(4718),
    [aux_sym_string_literal_token1] = ACTIONS(4718),
    [sym_char_literal] = ACTIONS(4718),
    [anon_sym_true] = ACTIONS(4720),
    [anon_sym_false] = ACTIONS(4720),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4720),
    [sym_super] = ACTIONS(4720),
    [sym_crate] = ACTIONS(4720),
    [aux_sym_metavariable_token1] = ACTIONS(4718),
    [sym__raw_string_literal_start] = ACTIONS(4718),
    [sym_float_literal] = ACTIONS(4718),
    [sym__outer_block_doc_comment_start] = ACTIONS(4718),
    [sym__outer_line_doc_comment_start] = ACTIONS(4718),
  },
  [STATE(1246)] = {
    [sym_line_comment] = STATE(1246),
    [sym_block_comment] = STATE(1246),
    [ts_builtin_sym_end] = ACTIONS(4722),
    [sym_identifier] = ACTIONS(4724),
    [anon_sym_SEMI] = ACTIONS(4722),
    [anon_sym_macro] = ACTIONS(4724),
    [anon_sym_LPAREN] = ACTIONS(4722),
    [anon_sym_LBRACE] = ACTIONS(4722),
    [anon_sym_RBRACE] = ACTIONS(4722),
    [anon_sym_macro_rules] = ACTIONS(4724),
    [anon_sym_BANG] = ACTIONS(4722),
    [anon_sym_LBRACK] = ACTIONS(4722),
    [anon_sym_STAR] = ACTIONS(4722),
    [anon_sym_u8] = ACTIONS(4724),
    [anon_sym_i8] = ACTIONS(4724),
    [anon_sym_u16] = ACTIONS(4724),
    [anon_sym_i16] = ACTIONS(4724),
    [anon_sym_u32] = ACTIONS(4724),
    [anon_sym_i32] = ACTIONS(4724),
    [anon_sym_u64] = ACTIONS(4724),
    [anon_sym_i64] = ACTIONS(4724),
    [anon_sym_u128] = ACTIONS(4724),
    [anon_sym_i128] = ACTIONS(4724),
    [anon_sym_isize] = ACTIONS(4724),
    [anon_sym_usize] = ACTIONS(4724),
    [anon_sym_f32] = ACTIONS(4724),
    [anon_sym_f64] = ACTIONS(4724),
    [anon_sym_bool] = ACTIONS(4724),
    [anon_sym_str] = ACTIONS(4724),
    [anon_sym_char] = ACTIONS(4724),
    [anon_sym_DASH] = ACTIONS(4722),
    [anon_sym_AMP] = ACTIONS(4722),
    [anon_sym_PIPE] = ACTIONS(4722),
    [anon_sym_LT] = ACTIONS(4722),
    [anon_sym_DOT_DOT] = ACTIONS(4722),
    [anon_sym_COLON_COLON] = ACTIONS(4722),
    [anon_sym_POUND] = ACTIONS(4722),
    [anon_sym_SQUOTE] = ACTIONS(4724),
    [anon_sym_async] = ACTIONS(4724),
    [anon_sym_auto] = ACTIONS(4724),
    [anon_sym_await] = ACTIONS(4724),
    [anon_sym_become] = ACTIONS(4724),
    [anon_sym_break] = ACTIONS(4724),
    [anon_sym_const] = ACTIONS(4724),
    [anon_sym_continue] = ACTIONS(4724),
    [anon_sym_default] = ACTIONS(4724),
    [anon_sym_enum] = ACTIONS(4724),
    [anon_sym_fn] = ACTIONS(4724),
    [anon_sym_for] = ACTIONS(4724),
    [anon_sym_gen] = ACTIONS(4724),
    [anon_sym_if] = ACTIONS(4724),
    [anon_sym_impl] = ACTIONS(4724),
    [anon_sym_let] = ACTIONS(4724),
    [anon_sym_loop] = ACTIONS(4724),
    [anon_sym_match] = ACTIONS(4724),
    [anon_sym_mod] = ACTIONS(4724),
    [anon_sym_pub] = ACTIONS(4724),
    [anon_sym_return] = ACTIONS(4724),
    [anon_sym_raw] = ACTIONS(4724),
    [anon_sym_static] = ACTIONS(4724),
    [anon_sym_struct] = ACTIONS(4724),
    [anon_sym_trait] = ACTIONS(4724),
    [anon_sym_type] = ACTIONS(4724),
    [anon_sym_union] = ACTIONS(4724),
    [anon_sym_unsafe] = ACTIONS(4724),
    [anon_sym_use] = ACTIONS(4724),
    [anon_sym_while] = ACTIONS(4724),
    [anon_sym_extern] = ACTIONS(4724),
    [anon_sym_safe] = ACTIONS(4724),
    [anon_sym_yield] = ACTIONS(4724),
    [anon_sym_move] = ACTIONS(4724),
    [anon_sym_try] = ACTIONS(4724),
    [sym_integer_literal] = ACTIONS(4722),
    [aux_sym_string_literal_token1] = ACTIONS(4722),
    [sym_char_literal] = ACTIONS(4722),
    [anon_sym_true] = ACTIONS(4724),
    [anon_sym_false] = ACTIONS(4724),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4724),
    [sym_super] = ACTIONS(4724),
    [sym_crate] = ACTIONS(4724),
    [aux_sym_metavariable_token1] = ACTIONS(4722),
    [sym__raw_string_literal_start] = ACTIONS(4722),
    [sym_float_literal] = ACTIONS(4722),
    [sym__outer_block_doc_comment_start] = ACTIONS(4722),
    [sym__outer_line_doc_comment_start] = ACTIONS(4722),
  },
  [STATE(1247)] = {
    [sym_line_comment] = STATE(1247),
    [sym_block_comment] = STATE(1247),
    [ts_builtin_sym_end] = ACTIONS(4726),
    [sym_identifier] = ACTIONS(4728),
    [anon_sym_SEMI] = ACTIONS(4726),
    [anon_sym_macro] = ACTIONS(4728),
    [anon_sym_LPAREN] = ACTIONS(4726),
    [anon_sym_LBRACE] = ACTIONS(4726),
    [anon_sym_RBRACE] = ACTIONS(4726),
    [anon_sym_macro_rules] = ACTIONS(4728),
    [anon_sym_BANG] = ACTIONS(4726),
    [anon_sym_LBRACK] = ACTIONS(4726),
    [anon_sym_STAR] = ACTIONS(4726),
    [anon_sym_u8] = ACTIONS(4728),
    [anon_sym_i8] = ACTIONS(4728),
    [anon_sym_u16] = ACTIONS(4728),
    [anon_sym_i16] = ACTIONS(4728),
    [anon_sym_u32] = ACTIONS(4728),
    [anon_sym_i32] = ACTIONS(4728),
    [anon_sym_u64] = ACTIONS(4728),
    [anon_sym_i64] = ACTIONS(4728),
    [anon_sym_u128] = ACTIONS(4728),
    [anon_sym_i128] = ACTIONS(4728),
    [anon_sym_isize] = ACTIONS(4728),
    [anon_sym_usize] = ACTIONS(4728),
    [anon_sym_f32] = ACTIONS(4728),
    [anon_sym_f64] = ACTIONS(4728),
    [anon_sym_bool] = ACTIONS(4728),
    [anon_sym_str] = ACTIONS(4728),
    [anon_sym_char] = ACTIONS(4728),
    [anon_sym_DASH] = ACTIONS(4726),
    [anon_sym_AMP] = ACTIONS(4726),
    [anon_sym_PIPE] = ACTIONS(4726),
    [anon_sym_LT] = ACTIONS(4726),
    [anon_sym_DOT_DOT] = ACTIONS(4726),
    [anon_sym_COLON_COLON] = ACTIONS(4726),
    [anon_sym_POUND] = ACTIONS(4726),
    [anon_sym_SQUOTE] = ACTIONS(4728),
    [anon_sym_async] = ACTIONS(4728),
    [anon_sym_auto] = ACTIONS(4728),
    [anon_sym_await] = ACTIONS(4728),
    [anon_sym_become] = ACTIONS(4728),
    [anon_sym_break] = ACTIONS(4728),
    [anon_sym_const] = ACTIONS(4728),
    [anon_sym_continue] = ACTIONS(4728),
    [anon_sym_default] = ACTIONS(4728),
    [anon_sym_enum] = ACTIONS(4728),
    [anon_sym_fn] = ACTIONS(4728),
    [anon_sym_for] = ACTIONS(4728),
    [anon_sym_gen] = ACTIONS(4728),
    [anon_sym_if] = ACTIONS(4728),
    [anon_sym_impl] = ACTIONS(4728),
    [anon_sym_let] = ACTIONS(4728),
    [anon_sym_loop] = ACTIONS(4728),
    [anon_sym_match] = ACTIONS(4728),
    [anon_sym_mod] = ACTIONS(4728),
    [anon_sym_pub] = ACTIONS(4728),
    [anon_sym_return] = ACTIONS(4728),
    [anon_sym_raw] = ACTIONS(4728),
    [anon_sym_static] = ACTIONS(4728),
    [anon_sym_struct] = ACTIONS(4728),
    [anon_sym_trait] = ACTIONS(4728),
    [anon_sym_type] = ACTIONS(4728),
    [anon_sym_union] = ACTIONS(4728),
    [anon_sym_unsafe] = ACTIONS(4728),
    [anon_sym_use] = ACTIONS(4728),
    [anon_sym_while] = ACTIONS(4728),
    [anon_sym_extern] = ACTIONS(4728),
    [anon_sym_safe] = ACTIONS(4728),
    [anon_sym_yield] = ACTIONS(4728),
    [anon_sym_move] = ACTIONS(4728),
    [anon_sym_try] = ACTIONS(4728),
    [sym_integer_literal] = ACTIONS(4726),
    [aux_sym_string_literal_token1] = ACTIONS(4726),
    [sym_char_literal] = ACTIONS(4726),
    [anon_sym_true] = ACTIONS(4728),
    [anon_sym_false] = ACTIONS(4728),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4728),
    [sym_super] = ACTIONS(4728),
    [sym_crate] = ACTIONS(4728),
    [aux_sym_metavariable_token1] = ACTIONS(4726),
    [sym__raw_string_literal_start] = ACTIONS(4726),
    [sym_float_literal] = ACTIONS(4726),
    [sym__outer_block_doc_comment_start] = ACTIONS(4726),
    [sym__outer_line_doc_comment_start] = ACTIONS(4726),
  },
  [STATE(1248)] = {
    [sym_line_comment] = STATE(1248),
    [sym_block_comment] = STATE(1248),
    [ts_builtin_sym_end] = ACTIONS(4730),
    [sym_identifier] = ACTIONS(4732),
    [anon_sym_SEMI] = ACTIONS(4730),
    [anon_sym_macro] = ACTIONS(4732),
    [anon_sym_LPAREN] = ACTIONS(4730),
    [anon_sym_LBRACE] = ACTIONS(4730),
    [anon_sym_RBRACE] = ACTIONS(4730),
    [anon_sym_macro_rules] = ACTIONS(4732),
    [anon_sym_BANG] = ACTIONS(4730),
    [anon_sym_LBRACK] = ACTIONS(4730),
    [anon_sym_STAR] = ACTIONS(4730),
    [anon_sym_u8] = ACTIONS(4732),
    [anon_sym_i8] = ACTIONS(4732),
    [anon_sym_u16] = ACTIONS(4732),
    [anon_sym_i16] = ACTIONS(4732),
    [anon_sym_u32] = ACTIONS(4732),
    [anon_sym_i32] = ACTIONS(4732),
    [anon_sym_u64] = ACTIONS(4732),
    [anon_sym_i64] = ACTIONS(4732),
    [anon_sym_u128] = ACTIONS(4732),
    [anon_sym_i128] = ACTIONS(4732),
    [anon_sym_isize] = ACTIONS(4732),
    [anon_sym_usize] = ACTIONS(4732),
    [anon_sym_f32] = ACTIONS(4732),
    [anon_sym_f64] = ACTIONS(4732),
    [anon_sym_bool] = ACTIONS(4732),
    [anon_sym_str] = ACTIONS(4732),
    [anon_sym_char] = ACTIONS(4732),
    [anon_sym_DASH] = ACTIONS(4730),
    [anon_sym_AMP] = ACTIONS(4730),
    [anon_sym_PIPE] = ACTIONS(4730),
    [anon_sym_LT] = ACTIONS(4730),
    [anon_sym_DOT_DOT] = ACTIONS(4730),
    [anon_sym_COLON_COLON] = ACTIONS(4730),
    [anon_sym_POUND] = ACTIONS(4730),
    [anon_sym_SQUOTE] = ACTIONS(4732),
    [anon_sym_async] = ACTIONS(4732),
    [anon_sym_auto] = ACTIONS(4732),
    [anon_sym_await] = ACTIONS(4732),
    [anon_sym_become] = ACTIONS(4732),
    [anon_sym_break] = ACTIONS(4732),
    [anon_sym_const] = ACTIONS(4732),
    [anon_sym_continue] = ACTIONS(4732),
    [anon_sym_default] = ACTIONS(4732),
    [anon_sym_enum] = ACTIONS(4732),
    [anon_sym_fn] = ACTIONS(4732),
    [anon_sym_for] = ACTIONS(4732),
    [anon_sym_gen] = ACTIONS(4732),
    [anon_sym_if] = ACTIONS(4732),
    [anon_sym_impl] = ACTIONS(4732),
    [anon_sym_let] = ACTIONS(4732),
    [anon_sym_loop] = ACTIONS(4732),
    [anon_sym_match] = ACTIONS(4732),
    [anon_sym_mod] = ACTIONS(4732),
    [anon_sym_pub] = ACTIONS(4732),
    [anon_sym_return] = ACTIONS(4732),
    [anon_sym_raw] = ACTIONS(4732),
    [anon_sym_static] = ACTIONS(4732),
    [anon_sym_struct] = ACTIONS(4732),
    [anon_sym_trait] = ACTIONS(4732),
    [anon_sym_type] = ACTIONS(4732),
    [anon_sym_union] = ACTIONS(4732),
    [anon_sym_unsafe] = ACTIONS(4732),
    [anon_sym_use] = ACTIONS(4732),
    [anon_sym_while] = ACTIONS(4732),
    [anon_sym_extern] = ACTIONS(4732),
    [anon_sym_safe] = ACTIONS(4732),
    [anon_sym_yield] = ACTIONS(4732),
    [anon_sym_move] = ACTIONS(4732),
    [anon_sym_try] = ACTIONS(4732),
    [sym_integer_literal] = ACTIONS(4730),
    [aux_sym_string_literal_token1] = ACTIONS(4730),
    [sym_char_literal] = ACTIONS(4730),
    [anon_sym_true] = ACTIONS(4732),
    [anon_sym_false] = ACTIONS(4732),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4732),
    [sym_super] = ACTIONS(4732),
    [sym_crate] = ACTIONS(4732),
    [aux_sym_metavariable_token1] = ACTIONS(4730),
    [sym__raw_string_literal_start] = ACTIONS(4730),
    [sym_float_literal] = ACTIONS(4730),
    [sym__outer_block_doc_comment_start] = ACTIONS(4730),
    [sym__outer_line_doc_comment_start] = ACTIONS(4730),
  },
  [STATE(1249)] = {
    [sym_line_comment] = STATE(1249),
    [sym_block_comment] = STATE(1249),
    [ts_builtin_sym_end] = ACTIONS(4734),
    [sym_identifier] = ACTIONS(4736),
    [anon_sym_SEMI] = ACTIONS(4734),
    [anon_sym_macro] = ACTIONS(4736),
    [anon_sym_LPAREN] = ACTIONS(4734),
    [anon_sym_LBRACE] = ACTIONS(4734),
    [anon_sym_RBRACE] = ACTIONS(4734),
    [anon_sym_macro_rules] = ACTIONS(4736),
    [anon_sym_BANG] = ACTIONS(4734),
    [anon_sym_LBRACK] = ACTIONS(4734),
    [anon_sym_STAR] = ACTIONS(4734),
    [anon_sym_u8] = ACTIONS(4736),
    [anon_sym_i8] = ACTIONS(4736),
    [anon_sym_u16] = ACTIONS(4736),
    [anon_sym_i16] = ACTIONS(4736),
    [anon_sym_u32] = ACTIONS(4736),
    [anon_sym_i32] = ACTIONS(4736),
    [anon_sym_u64] = ACTIONS(4736),
    [anon_sym_i64] = ACTIONS(4736),
    [anon_sym_u128] = ACTIONS(4736),
    [anon_sym_i128] = ACTIONS(4736),
    [anon_sym_isize] = ACTIONS(4736),
    [anon_sym_usize] = ACTIONS(4736),
    [anon_sym_f32] = ACTIONS(4736),
    [anon_sym_f64] = ACTIONS(4736),
    [anon_sym_bool] = ACTIONS(4736),
    [anon_sym_str] = ACTIONS(4736),
    [anon_sym_char] = ACTIONS(4736),
    [anon_sym_DASH] = ACTIONS(4734),
    [anon_sym_AMP] = ACTIONS(4734),
    [anon_sym_PIPE] = ACTIONS(4734),
    [anon_sym_LT] = ACTIONS(4734),
    [anon_sym_DOT_DOT] = ACTIONS(4734),
    [anon_sym_COLON_COLON] = ACTIONS(4734),
    [anon_sym_POUND] = ACTIONS(4734),
    [anon_sym_SQUOTE] = ACTIONS(4736),
    [anon_sym_async] = ACTIONS(4736),
    [anon_sym_auto] = ACTIONS(4736),
    [anon_sym_await] = ACTIONS(4736),
    [anon_sym_become] = ACTIONS(4736),
    [anon_sym_break] = ACTIONS(4736),
    [anon_sym_const] = ACTIONS(4736),
    [anon_sym_continue] = ACTIONS(4736),
    [anon_sym_default] = ACTIONS(4736),
    [anon_sym_enum] = ACTIONS(4736),
    [anon_sym_fn] = ACTIONS(4736),
    [anon_sym_for] = ACTIONS(4736),
    [anon_sym_gen] = ACTIONS(4736),
    [anon_sym_if] = ACTIONS(4736),
    [anon_sym_impl] = ACTIONS(4736),
    [anon_sym_let] = ACTIONS(4736),
    [anon_sym_loop] = ACTIONS(4736),
    [anon_sym_match] = ACTIONS(4736),
    [anon_sym_mod] = ACTIONS(4736),
    [anon_sym_pub] = ACTIONS(4736),
    [anon_sym_return] = ACTIONS(4736),
    [anon_sym_raw] = ACTIONS(4736),
    [anon_sym_static] = ACTIONS(4736),
    [anon_sym_struct] = ACTIONS(4736),
    [anon_sym_trait] = ACTIONS(4736),
    [anon_sym_type] = ACTIONS(4736),
    [anon_sym_union] = ACTIONS(4736),
    [anon_sym_unsafe] = ACTIONS(4736),
    [anon_sym_use] = ACTIONS(4736),
    [anon_sym_while] = ACTIONS(4736),
    [anon_sym_extern] = ACTIONS(4736),
    [anon_sym_safe] = ACTIONS(4736),
    [anon_sym_yield] = ACTIONS(4736),
    [anon_sym_move] = ACTIONS(4736),
    [anon_sym_try] = ACTIONS(4736),
    [sym_integer_literal] = ACTIONS(4734),
    [aux_sym_string_literal_token1] = ACTIONS(4734),
    [sym_char_literal] = ACTIONS(4734),
    [anon_sym_true] = ACTIONS(4736),
    [anon_sym_false] = ACTIONS(4736),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4736),
    [sym_super] = ACTIONS(4736),
    [sym_crate] = ACTIONS(4736),
    [aux_sym_metavariable_token1] = ACTIONS(4734),
    [sym__raw_string_literal_start] = ACTIONS(4734),
    [sym_float_literal] = ACTIONS(4734),
    [sym__outer_block_doc_comment_start] = ACTIONS(4734),
    [sym__outer_line_doc_comment_start] = ACTIONS(4734),
  },
  [STATE(1250)] = {
    [sym_line_comment] = STATE(1250),
    [sym_block_comment] = STATE(1250),
    [ts_builtin_sym_end] = ACTIONS(4738),
    [sym_identifier] = ACTIONS(4740),
    [anon_sym_SEMI] = ACTIONS(4738),
    [anon_sym_macro] = ACTIONS(4740),
    [anon_sym_LPAREN] = ACTIONS(4738),
    [anon_sym_LBRACE] = ACTIONS(4738),
    [anon_sym_RBRACE] = ACTIONS(4738),
    [anon_sym_macro_rules] = ACTIONS(4740),
    [anon_sym_BANG] = ACTIONS(4738),
    [anon_sym_LBRACK] = ACTIONS(4738),
    [anon_sym_STAR] = ACTIONS(4738),
    [anon_sym_u8] = ACTIONS(4740),
    [anon_sym_i8] = ACTIONS(4740),
    [anon_sym_u16] = ACTIONS(4740),
    [anon_sym_i16] = ACTIONS(4740),
    [anon_sym_u32] = ACTIONS(4740),
    [anon_sym_i32] = ACTIONS(4740),
    [anon_sym_u64] = ACTIONS(4740),
    [anon_sym_i64] = ACTIONS(4740),
    [anon_sym_u128] = ACTIONS(4740),
    [anon_sym_i128] = ACTIONS(4740),
    [anon_sym_isize] = ACTIONS(4740),
    [anon_sym_usize] = ACTIONS(4740),
    [anon_sym_f32] = ACTIONS(4740),
    [anon_sym_f64] = ACTIONS(4740),
    [anon_sym_bool] = ACTIONS(4740),
    [anon_sym_str] = ACTIONS(4740),
    [anon_sym_char] = ACTIONS(4740),
    [anon_sym_DASH] = ACTIONS(4738),
    [anon_sym_AMP] = ACTIONS(4738),
    [anon_sym_PIPE] = ACTIONS(4738),
    [anon_sym_LT] = ACTIONS(4738),
    [anon_sym_DOT_DOT] = ACTIONS(4738),
    [anon_sym_COLON_COLON] = ACTIONS(4738),
    [anon_sym_POUND] = ACTIONS(4738),
    [anon_sym_SQUOTE] = ACTIONS(4740),
    [anon_sym_async] = ACTIONS(4740),
    [anon_sym_auto] = ACTIONS(4740),
    [anon_sym_await] = ACTIONS(4740),
    [anon_sym_become] = ACTIONS(4740),
    [anon_sym_break] = ACTIONS(4740),
    [anon_sym_const] = ACTIONS(4740),
    [anon_sym_continue] = ACTIONS(4740),
    [anon_sym_default] = ACTIONS(4740),
    [anon_sym_enum] = ACTIONS(4740),
    [anon_sym_fn] = ACTIONS(4740),
    [anon_sym_for] = ACTIONS(4740),
    [anon_sym_gen] = ACTIONS(4740),
    [anon_sym_if] = ACTIONS(4740),
    [anon_sym_impl] = ACTIONS(4740),
    [anon_sym_let] = ACTIONS(4740),
    [anon_sym_loop] = ACTIONS(4740),
    [anon_sym_match] = ACTIONS(4740),
    [anon_sym_mod] = ACTIONS(4740),
    [anon_sym_pub] = ACTIONS(4740),
    [anon_sym_return] = ACTIONS(4740),
    [anon_sym_raw] = ACTIONS(4740),
    [anon_sym_static] = ACTIONS(4740),
    [anon_sym_struct] = ACTIONS(4740),
    [anon_sym_trait] = ACTIONS(4740),
    [anon_sym_type] = ACTIONS(4740),
    [anon_sym_union] = ACTIONS(4740),
    [anon_sym_unsafe] = ACTIONS(4740),
    [anon_sym_use] = ACTIONS(4740),
    [anon_sym_while] = ACTIONS(4740),
    [anon_sym_extern] = ACTIONS(4740),
    [anon_sym_safe] = ACTIONS(4740),
    [anon_sym_yield] = ACTIONS(4740),
    [anon_sym_move] = ACTIONS(4740),
    [anon_sym_try] = ACTIONS(4740),
    [sym_integer_literal] = ACTIONS(4738),
    [aux_sym_string_literal_token1] = ACTIONS(4738),
    [sym_char_literal] = ACTIONS(4738),
    [anon_sym_true] = ACTIONS(4740),
    [anon_sym_false] = ACTIONS(4740),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4740),
    [sym_super] = ACTIONS(4740),
    [sym_crate] = ACTIONS(4740),
    [aux_sym_metavariable_token1] = ACTIONS(4738),
    [sym__raw_string_literal_start] = ACTIONS(4738),
    [sym_float_literal] = ACTIONS(4738),
    [sym__outer_block_doc_comment_start] = ACTIONS(4738),
    [sym__outer_line_doc_comment_start] = ACTIONS(4738),
  },
  [STATE(1251)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3829),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1251),
    [sym_block_comment] = STATE(1251),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(2358),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(2358),
    [anon_sym_COMMA] = ACTIONS(2358),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(2358),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_EQ] = ACTIONS(2358),
    [anon_sym_GT] = ACTIONS(2358),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(2360),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1252)] = {
    [sym_line_comment] = STATE(1252),
    [sym_block_comment] = STATE(1252),
    [ts_builtin_sym_end] = ACTIONS(4742),
    [sym_identifier] = ACTIONS(4744),
    [anon_sym_SEMI] = ACTIONS(4742),
    [anon_sym_macro] = ACTIONS(4744),
    [anon_sym_LPAREN] = ACTIONS(4742),
    [anon_sym_LBRACE] = ACTIONS(4742),
    [anon_sym_RBRACE] = ACTIONS(4742),
    [anon_sym_macro_rules] = ACTIONS(4744),
    [anon_sym_BANG] = ACTIONS(4742),
    [anon_sym_LBRACK] = ACTIONS(4742),
    [anon_sym_STAR] = ACTIONS(4742),
    [anon_sym_u8] = ACTIONS(4744),
    [anon_sym_i8] = ACTIONS(4744),
    [anon_sym_u16] = ACTIONS(4744),
    [anon_sym_i16] = ACTIONS(4744),
    [anon_sym_u32] = ACTIONS(4744),
    [anon_sym_i32] = ACTIONS(4744),
    [anon_sym_u64] = ACTIONS(4744),
    [anon_sym_i64] = ACTIONS(4744),
    [anon_sym_u128] = ACTIONS(4744),
    [anon_sym_i128] = ACTIONS(4744),
    [anon_sym_isize] = ACTIONS(4744),
    [anon_sym_usize] = ACTIONS(4744),
    [anon_sym_f32] = ACTIONS(4744),
    [anon_sym_f64] = ACTIONS(4744),
    [anon_sym_bool] = ACTIONS(4744),
    [anon_sym_str] = ACTIONS(4744),
    [anon_sym_char] = ACTIONS(4744),
    [anon_sym_DASH] = ACTIONS(4742),
    [anon_sym_AMP] = ACTIONS(4742),
    [anon_sym_PIPE] = ACTIONS(4742),
    [anon_sym_LT] = ACTIONS(4742),
    [anon_sym_DOT_DOT] = ACTIONS(4742),
    [anon_sym_COLON_COLON] = ACTIONS(4742),
    [anon_sym_POUND] = ACTIONS(4742),
    [anon_sym_SQUOTE] = ACTIONS(4744),
    [anon_sym_async] = ACTIONS(4744),
    [anon_sym_auto] = ACTIONS(4744),
    [anon_sym_await] = ACTIONS(4744),
    [anon_sym_become] = ACTIONS(4744),
    [anon_sym_break] = ACTIONS(4744),
    [anon_sym_const] = ACTIONS(4744),
    [anon_sym_continue] = ACTIONS(4744),
    [anon_sym_default] = ACTIONS(4744),
    [anon_sym_enum] = ACTIONS(4744),
    [anon_sym_fn] = ACTIONS(4744),
    [anon_sym_for] = ACTIONS(4744),
    [anon_sym_gen] = ACTIONS(4744),
    [anon_sym_if] = ACTIONS(4744),
    [anon_sym_impl] = ACTIONS(4744),
    [anon_sym_let] = ACTIONS(4744),
    [anon_sym_loop] = ACTIONS(4744),
    [anon_sym_match] = ACTIONS(4744),
    [anon_sym_mod] = ACTIONS(4744),
    [anon_sym_pub] = ACTIONS(4744),
    [anon_sym_return] = ACTIONS(4744),
    [anon_sym_raw] = ACTIONS(4744),
    [anon_sym_static] = ACTIONS(4744),
    [anon_sym_struct] = ACTIONS(4744),
    [anon_sym_trait] = ACTIONS(4744),
    [anon_sym_type] = ACTIONS(4744),
    [anon_sym_union] = ACTIONS(4744),
    [anon_sym_unsafe] = ACTIONS(4744),
    [anon_sym_use] = ACTIONS(4744),
    [anon_sym_while] = ACTIONS(4744),
    [anon_sym_extern] = ACTIONS(4744),
    [anon_sym_safe] = ACTIONS(4744),
    [anon_sym_yield] = ACTIONS(4744),
    [anon_sym_move] = ACTIONS(4744),
    [anon_sym_try] = ACTIONS(4744),
    [sym_integer_literal] = ACTIONS(4742),
    [aux_sym_string_literal_token1] = ACTIONS(4742),
    [sym_char_literal] = ACTIONS(4742),
    [anon_sym_true] = ACTIONS(4744),
    [anon_sym_false] = ACTIONS(4744),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4744),
    [sym_super] = ACTIONS(4744),
    [sym_crate] = ACTIONS(4744),
    [aux_sym_metavariable_token1] = ACTIONS(4742),
    [sym__raw_string_literal_start] = ACTIONS(4742),
    [sym_float_literal] = ACTIONS(4742),
    [sym__outer_block_doc_comment_start] = ACTIONS(4742),
    [sym__outer_line_doc_comment_start] = ACTIONS(4742),
  },
  [STATE(1253)] = {
    [sym_line_comment] = STATE(1253),
    [sym_block_comment] = STATE(1253),
    [ts_builtin_sym_end] = ACTIONS(4746),
    [sym_identifier] = ACTIONS(4748),
    [anon_sym_SEMI] = ACTIONS(4746),
    [anon_sym_macro] = ACTIONS(4748),
    [anon_sym_LPAREN] = ACTIONS(4746),
    [anon_sym_LBRACE] = ACTIONS(4746),
    [anon_sym_RBRACE] = ACTIONS(4746),
    [anon_sym_macro_rules] = ACTIONS(4748),
    [anon_sym_BANG] = ACTIONS(4746),
    [anon_sym_LBRACK] = ACTIONS(4746),
    [anon_sym_STAR] = ACTIONS(4746),
    [anon_sym_u8] = ACTIONS(4748),
    [anon_sym_i8] = ACTIONS(4748),
    [anon_sym_u16] = ACTIONS(4748),
    [anon_sym_i16] = ACTIONS(4748),
    [anon_sym_u32] = ACTIONS(4748),
    [anon_sym_i32] = ACTIONS(4748),
    [anon_sym_u64] = ACTIONS(4748),
    [anon_sym_i64] = ACTIONS(4748),
    [anon_sym_u128] = ACTIONS(4748),
    [anon_sym_i128] = ACTIONS(4748),
    [anon_sym_isize] = ACTIONS(4748),
    [anon_sym_usize] = ACTIONS(4748),
    [anon_sym_f32] = ACTIONS(4748),
    [anon_sym_f64] = ACTIONS(4748),
    [anon_sym_bool] = ACTIONS(4748),
    [anon_sym_str] = ACTIONS(4748),
    [anon_sym_char] = ACTIONS(4748),
    [anon_sym_DASH] = ACTIONS(4746),
    [anon_sym_AMP] = ACTIONS(4746),
    [anon_sym_PIPE] = ACTIONS(4746),
    [anon_sym_LT] = ACTIONS(4746),
    [anon_sym_DOT_DOT] = ACTIONS(4746),
    [anon_sym_COLON_COLON] = ACTIONS(4746),
    [anon_sym_POUND] = ACTIONS(4746),
    [anon_sym_SQUOTE] = ACTIONS(4748),
    [anon_sym_async] = ACTIONS(4748),
    [anon_sym_auto] = ACTIONS(4748),
    [anon_sym_await] = ACTIONS(4748),
    [anon_sym_become] = ACTIONS(4748),
    [anon_sym_break] = ACTIONS(4748),
    [anon_sym_const] = ACTIONS(4748),
    [anon_sym_continue] = ACTIONS(4748),
    [anon_sym_default] = ACTIONS(4748),
    [anon_sym_enum] = ACTIONS(4748),
    [anon_sym_fn] = ACTIONS(4748),
    [anon_sym_for] = ACTIONS(4748),
    [anon_sym_gen] = ACTIONS(4748),
    [anon_sym_if] = ACTIONS(4748),
    [anon_sym_impl] = ACTIONS(4748),
    [anon_sym_let] = ACTIONS(4748),
    [anon_sym_loop] = ACTIONS(4748),
    [anon_sym_match] = ACTIONS(4748),
    [anon_sym_mod] = ACTIONS(4748),
    [anon_sym_pub] = ACTIONS(4748),
    [anon_sym_return] = ACTIONS(4748),
    [anon_sym_raw] = ACTIONS(4748),
    [anon_sym_static] = ACTIONS(4748),
    [anon_sym_struct] = ACTIONS(4748),
    [anon_sym_trait] = ACTIONS(4748),
    [anon_sym_type] = ACTIONS(4748),
    [anon_sym_union] = ACTIONS(4748),
    [anon_sym_unsafe] = ACTIONS(4748),
    [anon_sym_use] = ACTIONS(4748),
    [anon_sym_while] = ACTIONS(4748),
    [anon_sym_extern] = ACTIONS(4748),
    [anon_sym_safe] = ACTIONS(4748),
    [anon_sym_yield] = ACTIONS(4748),
    [anon_sym_move] = ACTIONS(4748),
    [anon_sym_try] = ACTIONS(4748),
    [sym_integer_literal] = ACTIONS(4746),
    [aux_sym_string_literal_token1] = ACTIONS(4746),
    [sym_char_literal] = ACTIONS(4746),
    [anon_sym_true] = ACTIONS(4748),
    [anon_sym_false] = ACTIONS(4748),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4748),
    [sym_super] = ACTIONS(4748),
    [sym_crate] = ACTIONS(4748),
    [aux_sym_metavariable_token1] = ACTIONS(4746),
    [sym__raw_string_literal_start] = ACTIONS(4746),
    [sym_float_literal] = ACTIONS(4746),
    [sym__outer_block_doc_comment_start] = ACTIONS(4746),
    [sym__outer_line_doc_comment_start] = ACTIONS(4746),
  },
  [STATE(1254)] = {
    [sym_line_comment] = STATE(1254),
    [sym_block_comment] = STATE(1254),
    [ts_builtin_sym_end] = ACTIONS(4750),
    [sym_identifier] = ACTIONS(4752),
    [anon_sym_SEMI] = ACTIONS(4750),
    [anon_sym_macro] = ACTIONS(4752),
    [anon_sym_LPAREN] = ACTIONS(4750),
    [anon_sym_LBRACE] = ACTIONS(4750),
    [anon_sym_RBRACE] = ACTIONS(4750),
    [anon_sym_macro_rules] = ACTIONS(4752),
    [anon_sym_BANG] = ACTIONS(4750),
    [anon_sym_LBRACK] = ACTIONS(4750),
    [anon_sym_STAR] = ACTIONS(4750),
    [anon_sym_u8] = ACTIONS(4752),
    [anon_sym_i8] = ACTIONS(4752),
    [anon_sym_u16] = ACTIONS(4752),
    [anon_sym_i16] = ACTIONS(4752),
    [anon_sym_u32] = ACTIONS(4752),
    [anon_sym_i32] = ACTIONS(4752),
    [anon_sym_u64] = ACTIONS(4752),
    [anon_sym_i64] = ACTIONS(4752),
    [anon_sym_u128] = ACTIONS(4752),
    [anon_sym_i128] = ACTIONS(4752),
    [anon_sym_isize] = ACTIONS(4752),
    [anon_sym_usize] = ACTIONS(4752),
    [anon_sym_f32] = ACTIONS(4752),
    [anon_sym_f64] = ACTIONS(4752),
    [anon_sym_bool] = ACTIONS(4752),
    [anon_sym_str] = ACTIONS(4752),
    [anon_sym_char] = ACTIONS(4752),
    [anon_sym_DASH] = ACTIONS(4750),
    [anon_sym_AMP] = ACTIONS(4750),
    [anon_sym_PIPE] = ACTIONS(4750),
    [anon_sym_LT] = ACTIONS(4750),
    [anon_sym_DOT_DOT] = ACTIONS(4750),
    [anon_sym_COLON_COLON] = ACTIONS(4750),
    [anon_sym_POUND] = ACTIONS(4750),
    [anon_sym_SQUOTE] = ACTIONS(4752),
    [anon_sym_async] = ACTIONS(4752),
    [anon_sym_auto] = ACTIONS(4752),
    [anon_sym_await] = ACTIONS(4752),
    [anon_sym_become] = ACTIONS(4752),
    [anon_sym_break] = ACTIONS(4752),
    [anon_sym_const] = ACTIONS(4752),
    [anon_sym_continue] = ACTIONS(4752),
    [anon_sym_default] = ACTIONS(4752),
    [anon_sym_enum] = ACTIONS(4752),
    [anon_sym_fn] = ACTIONS(4752),
    [anon_sym_for] = ACTIONS(4752),
    [anon_sym_gen] = ACTIONS(4752),
    [anon_sym_if] = ACTIONS(4752),
    [anon_sym_impl] = ACTIONS(4752),
    [anon_sym_let] = ACTIONS(4752),
    [anon_sym_loop] = ACTIONS(4752),
    [anon_sym_match] = ACTIONS(4752),
    [anon_sym_mod] = ACTIONS(4752),
    [anon_sym_pub] = ACTIONS(4752),
    [anon_sym_return] = ACTIONS(4752),
    [anon_sym_raw] = ACTIONS(4752),
    [anon_sym_static] = ACTIONS(4752),
    [anon_sym_struct] = ACTIONS(4752),
    [anon_sym_trait] = ACTIONS(4752),
    [anon_sym_type] = ACTIONS(4752),
    [anon_sym_union] = ACTIONS(4752),
    [anon_sym_unsafe] = ACTIONS(4752),
    [anon_sym_use] = ACTIONS(4752),
    [anon_sym_while] = ACTIONS(4752),
    [anon_sym_extern] = ACTIONS(4752),
    [anon_sym_safe] = ACTIONS(4752),
    [anon_sym_yield] = ACTIONS(4752),
    [anon_sym_move] = ACTIONS(4752),
    [anon_sym_try] = ACTIONS(4752),
    [sym_integer_literal] = ACTIONS(4750),
    [aux_sym_string_literal_token1] = ACTIONS(4750),
    [sym_char_literal] = ACTIONS(4750),
    [anon_sym_true] = ACTIONS(4752),
    [anon_sym_false] = ACTIONS(4752),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4752),
    [sym_super] = ACTIONS(4752),
    [sym_crate] = ACTIONS(4752),
    [aux_sym_metavariable_token1] = ACTIONS(4750),
    [sym__raw_string_literal_start] = ACTIONS(4750),
    [sym_float_literal] = ACTIONS(4750),
    [sym__outer_block_doc_comment_start] = ACTIONS(4750),
    [sym__outer_line_doc_comment_start] = ACTIONS(4750),
  },
  [STATE(1255)] = {
    [sym_line_comment] = STATE(1255),
    [sym_block_comment] = STATE(1255),
    [ts_builtin_sym_end] = ACTIONS(4754),
    [sym_identifier] = ACTIONS(4756),
    [anon_sym_SEMI] = ACTIONS(4754),
    [anon_sym_macro] = ACTIONS(4756),
    [anon_sym_LPAREN] = ACTIONS(4754),
    [anon_sym_LBRACE] = ACTIONS(4754),
    [anon_sym_RBRACE] = ACTIONS(4754),
    [anon_sym_macro_rules] = ACTIONS(4756),
    [anon_sym_BANG] = ACTIONS(4754),
    [anon_sym_LBRACK] = ACTIONS(4754),
    [anon_sym_STAR] = ACTIONS(4754),
    [anon_sym_u8] = ACTIONS(4756),
    [anon_sym_i8] = ACTIONS(4756),
    [anon_sym_u16] = ACTIONS(4756),
    [anon_sym_i16] = ACTIONS(4756),
    [anon_sym_u32] = ACTIONS(4756),
    [anon_sym_i32] = ACTIONS(4756),
    [anon_sym_u64] = ACTIONS(4756),
    [anon_sym_i64] = ACTIONS(4756),
    [anon_sym_u128] = ACTIONS(4756),
    [anon_sym_i128] = ACTIONS(4756),
    [anon_sym_isize] = ACTIONS(4756),
    [anon_sym_usize] = ACTIONS(4756),
    [anon_sym_f32] = ACTIONS(4756),
    [anon_sym_f64] = ACTIONS(4756),
    [anon_sym_bool] = ACTIONS(4756),
    [anon_sym_str] = ACTIONS(4756),
    [anon_sym_char] = ACTIONS(4756),
    [anon_sym_DASH] = ACTIONS(4754),
    [anon_sym_AMP] = ACTIONS(4754),
    [anon_sym_PIPE] = ACTIONS(4754),
    [anon_sym_LT] = ACTIONS(4754),
    [anon_sym_DOT_DOT] = ACTIONS(4754),
    [anon_sym_COLON_COLON] = ACTIONS(4754),
    [anon_sym_POUND] = ACTIONS(4754),
    [anon_sym_SQUOTE] = ACTIONS(4756),
    [anon_sym_async] = ACTIONS(4756),
    [anon_sym_auto] = ACTIONS(4756),
    [anon_sym_await] = ACTIONS(4756),
    [anon_sym_become] = ACTIONS(4756),
    [anon_sym_break] = ACTIONS(4756),
    [anon_sym_const] = ACTIONS(4756),
    [anon_sym_continue] = ACTIONS(4756),
    [anon_sym_default] = ACTIONS(4756),
    [anon_sym_enum] = ACTIONS(4756),
    [anon_sym_fn] = ACTIONS(4756),
    [anon_sym_for] = ACTIONS(4756),
    [anon_sym_gen] = ACTIONS(4756),
    [anon_sym_if] = ACTIONS(4756),
    [anon_sym_impl] = ACTIONS(4756),
    [anon_sym_let] = ACTIONS(4756),
    [anon_sym_loop] = ACTIONS(4756),
    [anon_sym_match] = ACTIONS(4756),
    [anon_sym_mod] = ACTIONS(4756),
    [anon_sym_pub] = ACTIONS(4756),
    [anon_sym_return] = ACTIONS(4756),
    [anon_sym_raw] = ACTIONS(4756),
    [anon_sym_static] = ACTIONS(4756),
    [anon_sym_struct] = ACTIONS(4756),
    [anon_sym_trait] = ACTIONS(4756),
    [anon_sym_type] = ACTIONS(4756),
    [anon_sym_union] = ACTIONS(4756),
    [anon_sym_unsafe] = ACTIONS(4756),
    [anon_sym_use] = ACTIONS(4756),
    [anon_sym_while] = ACTIONS(4756),
    [anon_sym_extern] = ACTIONS(4756),
    [anon_sym_safe] = ACTIONS(4756),
    [anon_sym_yield] = ACTIONS(4756),
    [anon_sym_move] = ACTIONS(4756),
    [anon_sym_try] = ACTIONS(4756),
    [sym_integer_literal] = ACTIONS(4754),
    [aux_sym_string_literal_token1] = ACTIONS(4754),
    [sym_char_literal] = ACTIONS(4754),
    [anon_sym_true] = ACTIONS(4756),
    [anon_sym_false] = ACTIONS(4756),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4756),
    [sym_super] = ACTIONS(4756),
    [sym_crate] = ACTIONS(4756),
    [aux_sym_metavariable_token1] = ACTIONS(4754),
    [sym__raw_string_literal_start] = ACTIONS(4754),
    [sym_float_literal] = ACTIONS(4754),
    [sym__outer_block_doc_comment_start] = ACTIONS(4754),
    [sym__outer_line_doc_comment_start] = ACTIONS(4754),
  },
  [STATE(1256)] = {
    [sym_line_comment] = STATE(1256),
    [sym_block_comment] = STATE(1256),
    [ts_builtin_sym_end] = ACTIONS(4758),
    [sym_identifier] = ACTIONS(4760),
    [anon_sym_SEMI] = ACTIONS(4758),
    [anon_sym_macro] = ACTIONS(4760),
    [anon_sym_LPAREN] = ACTIONS(4758),
    [anon_sym_LBRACE] = ACTIONS(4758),
    [anon_sym_RBRACE] = ACTIONS(4758),
    [anon_sym_macro_rules] = ACTIONS(4760),
    [anon_sym_BANG] = ACTIONS(4758),
    [anon_sym_LBRACK] = ACTIONS(4758),
    [anon_sym_STAR] = ACTIONS(4758),
    [anon_sym_u8] = ACTIONS(4760),
    [anon_sym_i8] = ACTIONS(4760),
    [anon_sym_u16] = ACTIONS(4760),
    [anon_sym_i16] = ACTIONS(4760),
    [anon_sym_u32] = ACTIONS(4760),
    [anon_sym_i32] = ACTIONS(4760),
    [anon_sym_u64] = ACTIONS(4760),
    [anon_sym_i64] = ACTIONS(4760),
    [anon_sym_u128] = ACTIONS(4760),
    [anon_sym_i128] = ACTIONS(4760),
    [anon_sym_isize] = ACTIONS(4760),
    [anon_sym_usize] = ACTIONS(4760),
    [anon_sym_f32] = ACTIONS(4760),
    [anon_sym_f64] = ACTIONS(4760),
    [anon_sym_bool] = ACTIONS(4760),
    [anon_sym_str] = ACTIONS(4760),
    [anon_sym_char] = ACTIONS(4760),
    [anon_sym_DASH] = ACTIONS(4758),
    [anon_sym_AMP] = ACTIONS(4758),
    [anon_sym_PIPE] = ACTIONS(4758),
    [anon_sym_LT] = ACTIONS(4758),
    [anon_sym_DOT_DOT] = ACTIONS(4758),
    [anon_sym_COLON_COLON] = ACTIONS(4758),
    [anon_sym_POUND] = ACTIONS(4758),
    [anon_sym_SQUOTE] = ACTIONS(4760),
    [anon_sym_async] = ACTIONS(4760),
    [anon_sym_auto] = ACTIONS(4760),
    [anon_sym_await] = ACTIONS(4760),
    [anon_sym_become] = ACTIONS(4760),
    [anon_sym_break] = ACTIONS(4760),
    [anon_sym_const] = ACTIONS(4760),
    [anon_sym_continue] = ACTIONS(4760),
    [anon_sym_default] = ACTIONS(4760),
    [anon_sym_enum] = ACTIONS(4760),
    [anon_sym_fn] = ACTIONS(4760),
    [anon_sym_for] = ACTIONS(4760),
    [anon_sym_gen] = ACTIONS(4760),
    [anon_sym_if] = ACTIONS(4760),
    [anon_sym_impl] = ACTIONS(4760),
    [anon_sym_let] = ACTIONS(4760),
    [anon_sym_loop] = ACTIONS(4760),
    [anon_sym_match] = ACTIONS(4760),
    [anon_sym_mod] = ACTIONS(4760),
    [anon_sym_pub] = ACTIONS(4760),
    [anon_sym_return] = ACTIONS(4760),
    [anon_sym_raw] = ACTIONS(4760),
    [anon_sym_static] = ACTIONS(4760),
    [anon_sym_struct] = ACTIONS(4760),
    [anon_sym_trait] = ACTIONS(4760),
    [anon_sym_type] = ACTIONS(4760),
    [anon_sym_union] = ACTIONS(4760),
    [anon_sym_unsafe] = ACTIONS(4760),
    [anon_sym_use] = ACTIONS(4760),
    [anon_sym_while] = ACTIONS(4760),
    [anon_sym_extern] = ACTIONS(4760),
    [anon_sym_safe] = ACTIONS(4760),
    [anon_sym_yield] = ACTIONS(4760),
    [anon_sym_move] = ACTIONS(4760),
    [anon_sym_try] = ACTIONS(4760),
    [sym_integer_literal] = ACTIONS(4758),
    [aux_sym_string_literal_token1] = ACTIONS(4758),
    [sym_char_literal] = ACTIONS(4758),
    [anon_sym_true] = ACTIONS(4760),
    [anon_sym_false] = ACTIONS(4760),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4760),
    [sym_super] = ACTIONS(4760),
    [sym_crate] = ACTIONS(4760),
    [aux_sym_metavariable_token1] = ACTIONS(4758),
    [sym__raw_string_literal_start] = ACTIONS(4758),
    [sym_float_literal] = ACTIONS(4758),
    [sym__outer_block_doc_comment_start] = ACTIONS(4758),
    [sym__outer_line_doc_comment_start] = ACTIONS(4758),
  },
  [STATE(1257)] = {
    [sym_line_comment] = STATE(1257),
    [sym_block_comment] = STATE(1257),
    [ts_builtin_sym_end] = ACTIONS(4762),
    [sym_identifier] = ACTIONS(4764),
    [anon_sym_SEMI] = ACTIONS(4762),
    [anon_sym_macro] = ACTIONS(4764),
    [anon_sym_LPAREN] = ACTIONS(4762),
    [anon_sym_LBRACE] = ACTIONS(4762),
    [anon_sym_RBRACE] = ACTIONS(4762),
    [anon_sym_macro_rules] = ACTIONS(4764),
    [anon_sym_BANG] = ACTIONS(4762),
    [anon_sym_LBRACK] = ACTIONS(4762),
    [anon_sym_STAR] = ACTIONS(4762),
    [anon_sym_u8] = ACTIONS(4764),
    [anon_sym_i8] = ACTIONS(4764),
    [anon_sym_u16] = ACTIONS(4764),
    [anon_sym_i16] = ACTIONS(4764),
    [anon_sym_u32] = ACTIONS(4764),
    [anon_sym_i32] = ACTIONS(4764),
    [anon_sym_u64] = ACTIONS(4764),
    [anon_sym_i64] = ACTIONS(4764),
    [anon_sym_u128] = ACTIONS(4764),
    [anon_sym_i128] = ACTIONS(4764),
    [anon_sym_isize] = ACTIONS(4764),
    [anon_sym_usize] = ACTIONS(4764),
    [anon_sym_f32] = ACTIONS(4764),
    [anon_sym_f64] = ACTIONS(4764),
    [anon_sym_bool] = ACTIONS(4764),
    [anon_sym_str] = ACTIONS(4764),
    [anon_sym_char] = ACTIONS(4764),
    [anon_sym_DASH] = ACTIONS(4762),
    [anon_sym_AMP] = ACTIONS(4762),
    [anon_sym_PIPE] = ACTIONS(4762),
    [anon_sym_LT] = ACTIONS(4762),
    [anon_sym_DOT_DOT] = ACTIONS(4762),
    [anon_sym_COLON_COLON] = ACTIONS(4762),
    [anon_sym_POUND] = ACTIONS(4762),
    [anon_sym_SQUOTE] = ACTIONS(4764),
    [anon_sym_async] = ACTIONS(4764),
    [anon_sym_auto] = ACTIONS(4764),
    [anon_sym_await] = ACTIONS(4764),
    [anon_sym_become] = ACTIONS(4764),
    [anon_sym_break] = ACTIONS(4764),
    [anon_sym_const] = ACTIONS(4764),
    [anon_sym_continue] = ACTIONS(4764),
    [anon_sym_default] = ACTIONS(4764),
    [anon_sym_enum] = ACTIONS(4764),
    [anon_sym_fn] = ACTIONS(4764),
    [anon_sym_for] = ACTIONS(4764),
    [anon_sym_gen] = ACTIONS(4764),
    [anon_sym_if] = ACTIONS(4764),
    [anon_sym_impl] = ACTIONS(4764),
    [anon_sym_let] = ACTIONS(4764),
    [anon_sym_loop] = ACTIONS(4764),
    [anon_sym_match] = ACTIONS(4764),
    [anon_sym_mod] = ACTIONS(4764),
    [anon_sym_pub] = ACTIONS(4764),
    [anon_sym_return] = ACTIONS(4764),
    [anon_sym_raw] = ACTIONS(4764),
    [anon_sym_static] = ACTIONS(4764),
    [anon_sym_struct] = ACTIONS(4764),
    [anon_sym_trait] = ACTIONS(4764),
    [anon_sym_type] = ACTIONS(4764),
    [anon_sym_union] = ACTIONS(4764),
    [anon_sym_unsafe] = ACTIONS(4764),
    [anon_sym_use] = ACTIONS(4764),
    [anon_sym_while] = ACTIONS(4764),
    [anon_sym_extern] = ACTIONS(4764),
    [anon_sym_safe] = ACTIONS(4764),
    [anon_sym_yield] = ACTIONS(4764),
    [anon_sym_move] = ACTIONS(4764),
    [anon_sym_try] = ACTIONS(4764),
    [sym_integer_literal] = ACTIONS(4762),
    [aux_sym_string_literal_token1] = ACTIONS(4762),
    [sym_char_literal] = ACTIONS(4762),
    [anon_sym_true] = ACTIONS(4764),
    [anon_sym_false] = ACTIONS(4764),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4764),
    [sym_super] = ACTIONS(4764),
    [sym_crate] = ACTIONS(4764),
    [aux_sym_metavariable_token1] = ACTIONS(4762),
    [sym__raw_string_literal_start] = ACTIONS(4762),
    [sym_float_literal] = ACTIONS(4762),
    [sym__outer_block_doc_comment_start] = ACTIONS(4762),
    [sym__outer_line_doc_comment_start] = ACTIONS(4762),
  },
  [STATE(1258)] = {
    [sym_line_comment] = STATE(1258),
    [sym_block_comment] = STATE(1258),
    [ts_builtin_sym_end] = ACTIONS(4766),
    [sym_identifier] = ACTIONS(4768),
    [anon_sym_SEMI] = ACTIONS(4766),
    [anon_sym_macro] = ACTIONS(4768),
    [anon_sym_LPAREN] = ACTIONS(4766),
    [anon_sym_LBRACE] = ACTIONS(4766),
    [anon_sym_RBRACE] = ACTIONS(4766),
    [anon_sym_macro_rules] = ACTIONS(4768),
    [anon_sym_BANG] = ACTIONS(4766),
    [anon_sym_LBRACK] = ACTIONS(4766),
    [anon_sym_STAR] = ACTIONS(4766),
    [anon_sym_u8] = ACTIONS(4768),
    [anon_sym_i8] = ACTIONS(4768),
    [anon_sym_u16] = ACTIONS(4768),
    [anon_sym_i16] = ACTIONS(4768),
    [anon_sym_u32] = ACTIONS(4768),
    [anon_sym_i32] = ACTIONS(4768),
    [anon_sym_u64] = ACTIONS(4768),
    [anon_sym_i64] = ACTIONS(4768),
    [anon_sym_u128] = ACTIONS(4768),
    [anon_sym_i128] = ACTIONS(4768),
    [anon_sym_isize] = ACTIONS(4768),
    [anon_sym_usize] = ACTIONS(4768),
    [anon_sym_f32] = ACTIONS(4768),
    [anon_sym_f64] = ACTIONS(4768),
    [anon_sym_bool] = ACTIONS(4768),
    [anon_sym_str] = ACTIONS(4768),
    [anon_sym_char] = ACTIONS(4768),
    [anon_sym_DASH] = ACTIONS(4766),
    [anon_sym_AMP] = ACTIONS(4766),
    [anon_sym_PIPE] = ACTIONS(4766),
    [anon_sym_LT] = ACTIONS(4766),
    [anon_sym_DOT_DOT] = ACTIONS(4766),
    [anon_sym_COLON_COLON] = ACTIONS(4766),
    [anon_sym_POUND] = ACTIONS(4766),
    [anon_sym_SQUOTE] = ACTIONS(4768),
    [anon_sym_async] = ACTIONS(4768),
    [anon_sym_auto] = ACTIONS(4768),
    [anon_sym_await] = ACTIONS(4768),
    [anon_sym_become] = ACTIONS(4768),
    [anon_sym_break] = ACTIONS(4768),
    [anon_sym_const] = ACTIONS(4768),
    [anon_sym_continue] = ACTIONS(4768),
    [anon_sym_default] = ACTIONS(4768),
    [anon_sym_enum] = ACTIONS(4768),
    [anon_sym_fn] = ACTIONS(4768),
    [anon_sym_for] = ACTIONS(4768),
    [anon_sym_gen] = ACTIONS(4768),
    [anon_sym_if] = ACTIONS(4768),
    [anon_sym_impl] = ACTIONS(4768),
    [anon_sym_let] = ACTIONS(4768),
    [anon_sym_loop] = ACTIONS(4768),
    [anon_sym_match] = ACTIONS(4768),
    [anon_sym_mod] = ACTIONS(4768),
    [anon_sym_pub] = ACTIONS(4768),
    [anon_sym_return] = ACTIONS(4768),
    [anon_sym_raw] = ACTIONS(4768),
    [anon_sym_static] = ACTIONS(4768),
    [anon_sym_struct] = ACTIONS(4768),
    [anon_sym_trait] = ACTIONS(4768),
    [anon_sym_type] = ACTIONS(4768),
    [anon_sym_union] = ACTIONS(4768),
    [anon_sym_unsafe] = ACTIONS(4768),
    [anon_sym_use] = ACTIONS(4768),
    [anon_sym_while] = ACTIONS(4768),
    [anon_sym_extern] = ACTIONS(4768),
    [anon_sym_safe] = ACTIONS(4768),
    [anon_sym_yield] = ACTIONS(4768),
    [anon_sym_move] = ACTIONS(4768),
    [anon_sym_try] = ACTIONS(4768),
    [sym_integer_literal] = ACTIONS(4766),
    [aux_sym_string_literal_token1] = ACTIONS(4766),
    [sym_char_literal] = ACTIONS(4766),
    [anon_sym_true] = ACTIONS(4768),
    [anon_sym_false] = ACTIONS(4768),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4768),
    [sym_super] = ACTIONS(4768),
    [sym_crate] = ACTIONS(4768),
    [aux_sym_metavariable_token1] = ACTIONS(4766),
    [sym__raw_string_literal_start] = ACTIONS(4766),
    [sym_float_literal] = ACTIONS(4766),
    [sym__outer_block_doc_comment_start] = ACTIONS(4766),
    [sym__outer_line_doc_comment_start] = ACTIONS(4766),
  },
  [STATE(1259)] = {
    [sym_line_comment] = STATE(1259),
    [sym_block_comment] = STATE(1259),
    [ts_builtin_sym_end] = ACTIONS(4770),
    [sym_identifier] = ACTIONS(4772),
    [anon_sym_SEMI] = ACTIONS(4770),
    [anon_sym_macro] = ACTIONS(4772),
    [anon_sym_LPAREN] = ACTIONS(4770),
    [anon_sym_LBRACE] = ACTIONS(4770),
    [anon_sym_RBRACE] = ACTIONS(4770),
    [anon_sym_macro_rules] = ACTIONS(4772),
    [anon_sym_BANG] = ACTIONS(4770),
    [anon_sym_LBRACK] = ACTIONS(4770),
    [anon_sym_STAR] = ACTIONS(4770),
    [anon_sym_u8] = ACTIONS(4772),
    [anon_sym_i8] = ACTIONS(4772),
    [anon_sym_u16] = ACTIONS(4772),
    [anon_sym_i16] = ACTIONS(4772),
    [anon_sym_u32] = ACTIONS(4772),
    [anon_sym_i32] = ACTIONS(4772),
    [anon_sym_u64] = ACTIONS(4772),
    [anon_sym_i64] = ACTIONS(4772),
    [anon_sym_u128] = ACTIONS(4772),
    [anon_sym_i128] = ACTIONS(4772),
    [anon_sym_isize] = ACTIONS(4772),
    [anon_sym_usize] = ACTIONS(4772),
    [anon_sym_f32] = ACTIONS(4772),
    [anon_sym_f64] = ACTIONS(4772),
    [anon_sym_bool] = ACTIONS(4772),
    [anon_sym_str] = ACTIONS(4772),
    [anon_sym_char] = ACTIONS(4772),
    [anon_sym_DASH] = ACTIONS(4770),
    [anon_sym_AMP] = ACTIONS(4770),
    [anon_sym_PIPE] = ACTIONS(4770),
    [anon_sym_LT] = ACTIONS(4770),
    [anon_sym_DOT_DOT] = ACTIONS(4770),
    [anon_sym_COLON_COLON] = ACTIONS(4770),
    [anon_sym_POUND] = ACTIONS(4770),
    [anon_sym_SQUOTE] = ACTIONS(4772),
    [anon_sym_async] = ACTIONS(4772),
    [anon_sym_auto] = ACTIONS(4772),
    [anon_sym_await] = ACTIONS(4772),
    [anon_sym_become] = ACTIONS(4772),
    [anon_sym_break] = ACTIONS(4772),
    [anon_sym_const] = ACTIONS(4772),
    [anon_sym_continue] = ACTIONS(4772),
    [anon_sym_default] = ACTIONS(4772),
    [anon_sym_enum] = ACTIONS(4772),
    [anon_sym_fn] = ACTIONS(4772),
    [anon_sym_for] = ACTIONS(4772),
    [anon_sym_gen] = ACTIONS(4772),
    [anon_sym_if] = ACTIONS(4772),
    [anon_sym_impl] = ACTIONS(4772),
    [anon_sym_let] = ACTIONS(4772),
    [anon_sym_loop] = ACTIONS(4772),
    [anon_sym_match] = ACTIONS(4772),
    [anon_sym_mod] = ACTIONS(4772),
    [anon_sym_pub] = ACTIONS(4772),
    [anon_sym_return] = ACTIONS(4772),
    [anon_sym_raw] = ACTIONS(4772),
    [anon_sym_static] = ACTIONS(4772),
    [anon_sym_struct] = ACTIONS(4772),
    [anon_sym_trait] = ACTIONS(4772),
    [anon_sym_type] = ACTIONS(4772),
    [anon_sym_union] = ACTIONS(4772),
    [anon_sym_unsafe] = ACTIONS(4772),
    [anon_sym_use] = ACTIONS(4772),
    [anon_sym_while] = ACTIONS(4772),
    [anon_sym_extern] = ACTIONS(4772),
    [anon_sym_safe] = ACTIONS(4772),
    [anon_sym_yield] = ACTIONS(4772),
    [anon_sym_move] = ACTIONS(4772),
    [anon_sym_try] = ACTIONS(4772),
    [sym_integer_literal] = ACTIONS(4770),
    [aux_sym_string_literal_token1] = ACTIONS(4770),
    [sym_char_literal] = ACTIONS(4770),
    [anon_sym_true] = ACTIONS(4772),
    [anon_sym_false] = ACTIONS(4772),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4772),
    [sym_super] = ACTIONS(4772),
    [sym_crate] = ACTIONS(4772),
    [aux_sym_metavariable_token1] = ACTIONS(4770),
    [sym__raw_string_literal_start] = ACTIONS(4770),
    [sym_float_literal] = ACTIONS(4770),
    [sym__outer_block_doc_comment_start] = ACTIONS(4770),
    [sym__outer_line_doc_comment_start] = ACTIONS(4770),
  },
  [STATE(1260)] = {
    [sym_line_comment] = STATE(1260),
    [sym_block_comment] = STATE(1260),
    [ts_builtin_sym_end] = ACTIONS(4774),
    [sym_identifier] = ACTIONS(4776),
    [anon_sym_SEMI] = ACTIONS(4774),
    [anon_sym_macro] = ACTIONS(4776),
    [anon_sym_LPAREN] = ACTIONS(4774),
    [anon_sym_LBRACE] = ACTIONS(4774),
    [anon_sym_RBRACE] = ACTIONS(4774),
    [anon_sym_macro_rules] = ACTIONS(4776),
    [anon_sym_BANG] = ACTIONS(4774),
    [anon_sym_LBRACK] = ACTIONS(4774),
    [anon_sym_STAR] = ACTIONS(4774),
    [anon_sym_u8] = ACTIONS(4776),
    [anon_sym_i8] = ACTIONS(4776),
    [anon_sym_u16] = ACTIONS(4776),
    [anon_sym_i16] = ACTIONS(4776),
    [anon_sym_u32] = ACTIONS(4776),
    [anon_sym_i32] = ACTIONS(4776),
    [anon_sym_u64] = ACTIONS(4776),
    [anon_sym_i64] = ACTIONS(4776),
    [anon_sym_u128] = ACTIONS(4776),
    [anon_sym_i128] = ACTIONS(4776),
    [anon_sym_isize] = ACTIONS(4776),
    [anon_sym_usize] = ACTIONS(4776),
    [anon_sym_f32] = ACTIONS(4776),
    [anon_sym_f64] = ACTIONS(4776),
    [anon_sym_bool] = ACTIONS(4776),
    [anon_sym_str] = ACTIONS(4776),
    [anon_sym_char] = ACTIONS(4776),
    [anon_sym_DASH] = ACTIONS(4774),
    [anon_sym_AMP] = ACTIONS(4774),
    [anon_sym_PIPE] = ACTIONS(4774),
    [anon_sym_LT] = ACTIONS(4774),
    [anon_sym_DOT_DOT] = ACTIONS(4774),
    [anon_sym_COLON_COLON] = ACTIONS(4774),
    [anon_sym_POUND] = ACTIONS(4774),
    [anon_sym_SQUOTE] = ACTIONS(4776),
    [anon_sym_async] = ACTIONS(4776),
    [anon_sym_auto] = ACTIONS(4776),
    [anon_sym_await] = ACTIONS(4776),
    [anon_sym_become] = ACTIONS(4776),
    [anon_sym_break] = ACTIONS(4776),
    [anon_sym_const] = ACTIONS(4776),
    [anon_sym_continue] = ACTIONS(4776),
    [anon_sym_default] = ACTIONS(4776),
    [anon_sym_enum] = ACTIONS(4776),
    [anon_sym_fn] = ACTIONS(4776),
    [anon_sym_for] = ACTIONS(4776),
    [anon_sym_gen] = ACTIONS(4776),
    [anon_sym_if] = ACTIONS(4776),
    [anon_sym_impl] = ACTIONS(4776),
    [anon_sym_let] = ACTIONS(4776),
    [anon_sym_loop] = ACTIONS(4776),
    [anon_sym_match] = ACTIONS(4776),
    [anon_sym_mod] = ACTIONS(4776),
    [anon_sym_pub] = ACTIONS(4776),
    [anon_sym_return] = ACTIONS(4776),
    [anon_sym_raw] = ACTIONS(4776),
    [anon_sym_static] = ACTIONS(4776),
    [anon_sym_struct] = ACTIONS(4776),
    [anon_sym_trait] = ACTIONS(4776),
    [anon_sym_type] = ACTIONS(4776),
    [anon_sym_union] = ACTIONS(4776),
    [anon_sym_unsafe] = ACTIONS(4776),
    [anon_sym_use] = ACTIONS(4776),
    [anon_sym_while] = ACTIONS(4776),
    [anon_sym_extern] = ACTIONS(4776),
    [anon_sym_safe] = ACTIONS(4776),
    [anon_sym_yield] = ACTIONS(4776),
    [anon_sym_move] = ACTIONS(4776),
    [anon_sym_try] = ACTIONS(4776),
    [sym_integer_literal] = ACTIONS(4774),
    [aux_sym_string_literal_token1] = ACTIONS(4774),
    [sym_char_literal] = ACTIONS(4774),
    [anon_sym_true] = ACTIONS(4776),
    [anon_sym_false] = ACTIONS(4776),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4776),
    [sym_super] = ACTIONS(4776),
    [sym_crate] = ACTIONS(4776),
    [aux_sym_metavariable_token1] = ACTIONS(4774),
    [sym__raw_string_literal_start] = ACTIONS(4774),
    [sym_float_literal] = ACTIONS(4774),
    [sym__outer_block_doc_comment_start] = ACTIONS(4774),
    [sym__outer_line_doc_comment_start] = ACTIONS(4774),
  },
  [STATE(1261)] = {
    [sym_line_comment] = STATE(1261),
    [sym_block_comment] = STATE(1261),
    [ts_builtin_sym_end] = ACTIONS(4778),
    [sym_identifier] = ACTIONS(4780),
    [anon_sym_SEMI] = ACTIONS(4778),
    [anon_sym_macro] = ACTIONS(4780),
    [anon_sym_LPAREN] = ACTIONS(4778),
    [anon_sym_LBRACE] = ACTIONS(4778),
    [anon_sym_RBRACE] = ACTIONS(4778),
    [anon_sym_macro_rules] = ACTIONS(4780),
    [anon_sym_BANG] = ACTIONS(4778),
    [anon_sym_LBRACK] = ACTIONS(4778),
    [anon_sym_STAR] = ACTIONS(4778),
    [anon_sym_u8] = ACTIONS(4780),
    [anon_sym_i8] = ACTIONS(4780),
    [anon_sym_u16] = ACTIONS(4780),
    [anon_sym_i16] = ACTIONS(4780),
    [anon_sym_u32] = ACTIONS(4780),
    [anon_sym_i32] = ACTIONS(4780),
    [anon_sym_u64] = ACTIONS(4780),
    [anon_sym_i64] = ACTIONS(4780),
    [anon_sym_u128] = ACTIONS(4780),
    [anon_sym_i128] = ACTIONS(4780),
    [anon_sym_isize] = ACTIONS(4780),
    [anon_sym_usize] = ACTIONS(4780),
    [anon_sym_f32] = ACTIONS(4780),
    [anon_sym_f64] = ACTIONS(4780),
    [anon_sym_bool] = ACTIONS(4780),
    [anon_sym_str] = ACTIONS(4780),
    [anon_sym_char] = ACTIONS(4780),
    [anon_sym_DASH] = ACTIONS(4778),
    [anon_sym_AMP] = ACTIONS(4778),
    [anon_sym_PIPE] = ACTIONS(4778),
    [anon_sym_LT] = ACTIONS(4778),
    [anon_sym_DOT_DOT] = ACTIONS(4778),
    [anon_sym_COLON_COLON] = ACTIONS(4778),
    [anon_sym_POUND] = ACTIONS(4778),
    [anon_sym_SQUOTE] = ACTIONS(4780),
    [anon_sym_async] = ACTIONS(4780),
    [anon_sym_auto] = ACTIONS(4780),
    [anon_sym_await] = ACTIONS(4780),
    [anon_sym_become] = ACTIONS(4780),
    [anon_sym_break] = ACTIONS(4780),
    [anon_sym_const] = ACTIONS(4780),
    [anon_sym_continue] = ACTIONS(4780),
    [anon_sym_default] = ACTIONS(4780),
    [anon_sym_enum] = ACTIONS(4780),
    [anon_sym_fn] = ACTIONS(4780),
    [anon_sym_for] = ACTIONS(4780),
    [anon_sym_gen] = ACTIONS(4780),
    [anon_sym_if] = ACTIONS(4780),
    [anon_sym_impl] = ACTIONS(4780),
    [anon_sym_let] = ACTIONS(4780),
    [anon_sym_loop] = ACTIONS(4780),
    [anon_sym_match] = ACTIONS(4780),
    [anon_sym_mod] = ACTIONS(4780),
    [anon_sym_pub] = ACTIONS(4780),
    [anon_sym_return] = ACTIONS(4780),
    [anon_sym_raw] = ACTIONS(4780),
    [anon_sym_static] = ACTIONS(4780),
    [anon_sym_struct] = ACTIONS(4780),
    [anon_sym_trait] = ACTIONS(4780),
    [anon_sym_type] = ACTIONS(4780),
    [anon_sym_union] = ACTIONS(4780),
    [anon_sym_unsafe] = ACTIONS(4780),
    [anon_sym_use] = ACTIONS(4780),
    [anon_sym_while] = ACTIONS(4780),
    [anon_sym_extern] = ACTIONS(4780),
    [anon_sym_safe] = ACTIONS(4780),
    [anon_sym_yield] = ACTIONS(4780),
    [anon_sym_move] = ACTIONS(4780),
    [anon_sym_try] = ACTIONS(4780),
    [sym_integer_literal] = ACTIONS(4778),
    [aux_sym_string_literal_token1] = ACTIONS(4778),
    [sym_char_literal] = ACTIONS(4778),
    [anon_sym_true] = ACTIONS(4780),
    [anon_sym_false] = ACTIONS(4780),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4780),
    [sym_super] = ACTIONS(4780),
    [sym_crate] = ACTIONS(4780),
    [aux_sym_metavariable_token1] = ACTIONS(4778),
    [sym__raw_string_literal_start] = ACTIONS(4778),
    [sym_float_literal] = ACTIONS(4778),
    [sym__outer_block_doc_comment_start] = ACTIONS(4778),
    [sym__outer_line_doc_comment_start] = ACTIONS(4778),
  },
  [STATE(1262)] = {
    [sym_line_comment] = STATE(1262),
    [sym_block_comment] = STATE(1262),
    [ts_builtin_sym_end] = ACTIONS(4782),
    [sym_identifier] = ACTIONS(4784),
    [anon_sym_SEMI] = ACTIONS(4782),
    [anon_sym_macro] = ACTIONS(4784),
    [anon_sym_LPAREN] = ACTIONS(4782),
    [anon_sym_LBRACE] = ACTIONS(4782),
    [anon_sym_RBRACE] = ACTIONS(4782),
    [anon_sym_macro_rules] = ACTIONS(4784),
    [anon_sym_BANG] = ACTIONS(4782),
    [anon_sym_LBRACK] = ACTIONS(4782),
    [anon_sym_STAR] = ACTIONS(4782),
    [anon_sym_u8] = ACTIONS(4784),
    [anon_sym_i8] = ACTIONS(4784),
    [anon_sym_u16] = ACTIONS(4784),
    [anon_sym_i16] = ACTIONS(4784),
    [anon_sym_u32] = ACTIONS(4784),
    [anon_sym_i32] = ACTIONS(4784),
    [anon_sym_u64] = ACTIONS(4784),
    [anon_sym_i64] = ACTIONS(4784),
    [anon_sym_u128] = ACTIONS(4784),
    [anon_sym_i128] = ACTIONS(4784),
    [anon_sym_isize] = ACTIONS(4784),
    [anon_sym_usize] = ACTIONS(4784),
    [anon_sym_f32] = ACTIONS(4784),
    [anon_sym_f64] = ACTIONS(4784),
    [anon_sym_bool] = ACTIONS(4784),
    [anon_sym_str] = ACTIONS(4784),
    [anon_sym_char] = ACTIONS(4784),
    [anon_sym_DASH] = ACTIONS(4782),
    [anon_sym_AMP] = ACTIONS(4782),
    [anon_sym_PIPE] = ACTIONS(4782),
    [anon_sym_LT] = ACTIONS(4782),
    [anon_sym_DOT_DOT] = ACTIONS(4782),
    [anon_sym_COLON_COLON] = ACTIONS(4782),
    [anon_sym_POUND] = ACTIONS(4782),
    [anon_sym_SQUOTE] = ACTIONS(4784),
    [anon_sym_async] = ACTIONS(4784),
    [anon_sym_auto] = ACTIONS(4784),
    [anon_sym_await] = ACTIONS(4784),
    [anon_sym_become] = ACTIONS(4784),
    [anon_sym_break] = ACTIONS(4784),
    [anon_sym_const] = ACTIONS(4784),
    [anon_sym_continue] = ACTIONS(4784),
    [anon_sym_default] = ACTIONS(4784),
    [anon_sym_enum] = ACTIONS(4784),
    [anon_sym_fn] = ACTIONS(4784),
    [anon_sym_for] = ACTIONS(4784),
    [anon_sym_gen] = ACTIONS(4784),
    [anon_sym_if] = ACTIONS(4784),
    [anon_sym_impl] = ACTIONS(4784),
    [anon_sym_let] = ACTIONS(4784),
    [anon_sym_loop] = ACTIONS(4784),
    [anon_sym_match] = ACTIONS(4784),
    [anon_sym_mod] = ACTIONS(4784),
    [anon_sym_pub] = ACTIONS(4784),
    [anon_sym_return] = ACTIONS(4784),
    [anon_sym_raw] = ACTIONS(4784),
    [anon_sym_static] = ACTIONS(4784),
    [anon_sym_struct] = ACTIONS(4784),
    [anon_sym_trait] = ACTIONS(4784),
    [anon_sym_type] = ACTIONS(4784),
    [anon_sym_union] = ACTIONS(4784),
    [anon_sym_unsafe] = ACTIONS(4784),
    [anon_sym_use] = ACTIONS(4784),
    [anon_sym_while] = ACTIONS(4784),
    [anon_sym_extern] = ACTIONS(4784),
    [anon_sym_safe] = ACTIONS(4784),
    [anon_sym_yield] = ACTIONS(4784),
    [anon_sym_move] = ACTIONS(4784),
    [anon_sym_try] = ACTIONS(4784),
    [sym_integer_literal] = ACTIONS(4782),
    [aux_sym_string_literal_token1] = ACTIONS(4782),
    [sym_char_literal] = ACTIONS(4782),
    [anon_sym_true] = ACTIONS(4784),
    [anon_sym_false] = ACTIONS(4784),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4784),
    [sym_super] = ACTIONS(4784),
    [sym_crate] = ACTIONS(4784),
    [aux_sym_metavariable_token1] = ACTIONS(4782),
    [sym__raw_string_literal_start] = ACTIONS(4782),
    [sym_float_literal] = ACTIONS(4782),
    [sym__outer_block_doc_comment_start] = ACTIONS(4782),
    [sym__outer_line_doc_comment_start] = ACTIONS(4782),
  },
  [STATE(1263)] = {
    [sym_line_comment] = STATE(1263),
    [sym_block_comment] = STATE(1263),
    [ts_builtin_sym_end] = ACTIONS(4786),
    [sym_identifier] = ACTIONS(4788),
    [anon_sym_SEMI] = ACTIONS(4786),
    [anon_sym_macro] = ACTIONS(4788),
    [anon_sym_LPAREN] = ACTIONS(4786),
    [anon_sym_LBRACE] = ACTIONS(4786),
    [anon_sym_RBRACE] = ACTIONS(4786),
    [anon_sym_macro_rules] = ACTIONS(4788),
    [anon_sym_BANG] = ACTIONS(4786),
    [anon_sym_LBRACK] = ACTIONS(4786),
    [anon_sym_STAR] = ACTIONS(4786),
    [anon_sym_u8] = ACTIONS(4788),
    [anon_sym_i8] = ACTIONS(4788),
    [anon_sym_u16] = ACTIONS(4788),
    [anon_sym_i16] = ACTIONS(4788),
    [anon_sym_u32] = ACTIONS(4788),
    [anon_sym_i32] = ACTIONS(4788),
    [anon_sym_u64] = ACTIONS(4788),
    [anon_sym_i64] = ACTIONS(4788),
    [anon_sym_u128] = ACTIONS(4788),
    [anon_sym_i128] = ACTIONS(4788),
    [anon_sym_isize] = ACTIONS(4788),
    [anon_sym_usize] = ACTIONS(4788),
    [anon_sym_f32] = ACTIONS(4788),
    [anon_sym_f64] = ACTIONS(4788),
    [anon_sym_bool] = ACTIONS(4788),
    [anon_sym_str] = ACTIONS(4788),
    [anon_sym_char] = ACTIONS(4788),
    [anon_sym_DASH] = ACTIONS(4786),
    [anon_sym_AMP] = ACTIONS(4786),
    [anon_sym_PIPE] = ACTIONS(4786),
    [anon_sym_LT] = ACTIONS(4786),
    [anon_sym_DOT_DOT] = ACTIONS(4786),
    [anon_sym_COLON_COLON] = ACTIONS(4786),
    [anon_sym_POUND] = ACTIONS(4786),
    [anon_sym_SQUOTE] = ACTIONS(4788),
    [anon_sym_async] = ACTIONS(4788),
    [anon_sym_auto] = ACTIONS(4788),
    [anon_sym_await] = ACTIONS(4788),
    [anon_sym_become] = ACTIONS(4788),
    [anon_sym_break] = ACTIONS(4788),
    [anon_sym_const] = ACTIONS(4788),
    [anon_sym_continue] = ACTIONS(4788),
    [anon_sym_default] = ACTIONS(4788),
    [anon_sym_enum] = ACTIONS(4788),
    [anon_sym_fn] = ACTIONS(4788),
    [anon_sym_for] = ACTIONS(4788),
    [anon_sym_gen] = ACTIONS(4788),
    [anon_sym_if] = ACTIONS(4788),
    [anon_sym_impl] = ACTIONS(4788),
    [anon_sym_let] = ACTIONS(4788),
    [anon_sym_loop] = ACTIONS(4788),
    [anon_sym_match] = ACTIONS(4788),
    [anon_sym_mod] = ACTIONS(4788),
    [anon_sym_pub] = ACTIONS(4788),
    [anon_sym_return] = ACTIONS(4788),
    [anon_sym_raw] = ACTIONS(4788),
    [anon_sym_static] = ACTIONS(4788),
    [anon_sym_struct] = ACTIONS(4788),
    [anon_sym_trait] = ACTIONS(4788),
    [anon_sym_type] = ACTIONS(4788),
    [anon_sym_union] = ACTIONS(4788),
    [anon_sym_unsafe] = ACTIONS(4788),
    [anon_sym_use] = ACTIONS(4788),
    [anon_sym_while] = ACTIONS(4788),
    [anon_sym_extern] = ACTIONS(4788),
    [anon_sym_safe] = ACTIONS(4788),
    [anon_sym_yield] = ACTIONS(4788),
    [anon_sym_move] = ACTIONS(4788),
    [anon_sym_try] = ACTIONS(4788),
    [sym_integer_literal] = ACTIONS(4786),
    [aux_sym_string_literal_token1] = ACTIONS(4786),
    [sym_char_literal] = ACTIONS(4786),
    [anon_sym_true] = ACTIONS(4788),
    [anon_sym_false] = ACTIONS(4788),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4788),
    [sym_super] = ACTIONS(4788),
    [sym_crate] = ACTIONS(4788),
    [aux_sym_metavariable_token1] = ACTIONS(4786),
    [sym__raw_string_literal_start] = ACTIONS(4786),
    [sym_float_literal] = ACTIONS(4786),
    [sym__outer_block_doc_comment_start] = ACTIONS(4786),
    [sym__outer_line_doc_comment_start] = ACTIONS(4786),
  },
  [STATE(1264)] = {
    [sym_line_comment] = STATE(1264),
    [sym_block_comment] = STATE(1264),
    [ts_builtin_sym_end] = ACTIONS(4790),
    [sym_identifier] = ACTIONS(4792),
    [anon_sym_SEMI] = ACTIONS(4790),
    [anon_sym_macro] = ACTIONS(4792),
    [anon_sym_LPAREN] = ACTIONS(4790),
    [anon_sym_LBRACE] = ACTIONS(4790),
    [anon_sym_RBRACE] = ACTIONS(4790),
    [anon_sym_macro_rules] = ACTIONS(4792),
    [anon_sym_BANG] = ACTIONS(4790),
    [anon_sym_LBRACK] = ACTIONS(4790),
    [anon_sym_STAR] = ACTIONS(4790),
    [anon_sym_u8] = ACTIONS(4792),
    [anon_sym_i8] = ACTIONS(4792),
    [anon_sym_u16] = ACTIONS(4792),
    [anon_sym_i16] = ACTIONS(4792),
    [anon_sym_u32] = ACTIONS(4792),
    [anon_sym_i32] = ACTIONS(4792),
    [anon_sym_u64] = ACTIONS(4792),
    [anon_sym_i64] = ACTIONS(4792),
    [anon_sym_u128] = ACTIONS(4792),
    [anon_sym_i128] = ACTIONS(4792),
    [anon_sym_isize] = ACTIONS(4792),
    [anon_sym_usize] = ACTIONS(4792),
    [anon_sym_f32] = ACTIONS(4792),
    [anon_sym_f64] = ACTIONS(4792),
    [anon_sym_bool] = ACTIONS(4792),
    [anon_sym_str] = ACTIONS(4792),
    [anon_sym_char] = ACTIONS(4792),
    [anon_sym_DASH] = ACTIONS(4790),
    [anon_sym_AMP] = ACTIONS(4790),
    [anon_sym_PIPE] = ACTIONS(4790),
    [anon_sym_LT] = ACTIONS(4790),
    [anon_sym_DOT_DOT] = ACTIONS(4790),
    [anon_sym_COLON_COLON] = ACTIONS(4790),
    [anon_sym_POUND] = ACTIONS(4790),
    [anon_sym_SQUOTE] = ACTIONS(4792),
    [anon_sym_async] = ACTIONS(4792),
    [anon_sym_auto] = ACTIONS(4792),
    [anon_sym_await] = ACTIONS(4792),
    [anon_sym_become] = ACTIONS(4792),
    [anon_sym_break] = ACTIONS(4792),
    [anon_sym_const] = ACTIONS(4792),
    [anon_sym_continue] = ACTIONS(4792),
    [anon_sym_default] = ACTIONS(4792),
    [anon_sym_enum] = ACTIONS(4792),
    [anon_sym_fn] = ACTIONS(4792),
    [anon_sym_for] = ACTIONS(4792),
    [anon_sym_gen] = ACTIONS(4792),
    [anon_sym_if] = ACTIONS(4792),
    [anon_sym_impl] = ACTIONS(4792),
    [anon_sym_let] = ACTIONS(4792),
    [anon_sym_loop] = ACTIONS(4792),
    [anon_sym_match] = ACTIONS(4792),
    [anon_sym_mod] = ACTIONS(4792),
    [anon_sym_pub] = ACTIONS(4792),
    [anon_sym_return] = ACTIONS(4792),
    [anon_sym_raw] = ACTIONS(4792),
    [anon_sym_static] = ACTIONS(4792),
    [anon_sym_struct] = ACTIONS(4792),
    [anon_sym_trait] = ACTIONS(4792),
    [anon_sym_type] = ACTIONS(4792),
    [anon_sym_union] = ACTIONS(4792),
    [anon_sym_unsafe] = ACTIONS(4792),
    [anon_sym_use] = ACTIONS(4792),
    [anon_sym_while] = ACTIONS(4792),
    [anon_sym_extern] = ACTIONS(4792),
    [anon_sym_safe] = ACTIONS(4792),
    [anon_sym_yield] = ACTIONS(4792),
    [anon_sym_move] = ACTIONS(4792),
    [anon_sym_try] = ACTIONS(4792),
    [sym_integer_literal] = ACTIONS(4790),
    [aux_sym_string_literal_token1] = ACTIONS(4790),
    [sym_char_literal] = ACTIONS(4790),
    [anon_sym_true] = ACTIONS(4792),
    [anon_sym_false] = ACTIONS(4792),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4792),
    [sym_super] = ACTIONS(4792),
    [sym_crate] = ACTIONS(4792),
    [aux_sym_metavariable_token1] = ACTIONS(4790),
    [sym__raw_string_literal_start] = ACTIONS(4790),
    [sym_float_literal] = ACTIONS(4790),
    [sym__outer_block_doc_comment_start] = ACTIONS(4790),
    [sym__outer_line_doc_comment_start] = ACTIONS(4790),
  },
  [STATE(1265)] = {
    [sym_line_comment] = STATE(1265),
    [sym_block_comment] = STATE(1265),
    [ts_builtin_sym_end] = ACTIONS(4794),
    [sym_identifier] = ACTIONS(4796),
    [anon_sym_SEMI] = ACTIONS(4794),
    [anon_sym_macro] = ACTIONS(4796),
    [anon_sym_LPAREN] = ACTIONS(4794),
    [anon_sym_LBRACE] = ACTIONS(4794),
    [anon_sym_RBRACE] = ACTIONS(4794),
    [anon_sym_macro_rules] = ACTIONS(4796),
    [anon_sym_BANG] = ACTIONS(4794),
    [anon_sym_LBRACK] = ACTIONS(4794),
    [anon_sym_STAR] = ACTIONS(4794),
    [anon_sym_u8] = ACTIONS(4796),
    [anon_sym_i8] = ACTIONS(4796),
    [anon_sym_u16] = ACTIONS(4796),
    [anon_sym_i16] = ACTIONS(4796),
    [anon_sym_u32] = ACTIONS(4796),
    [anon_sym_i32] = ACTIONS(4796),
    [anon_sym_u64] = ACTIONS(4796),
    [anon_sym_i64] = ACTIONS(4796),
    [anon_sym_u128] = ACTIONS(4796),
    [anon_sym_i128] = ACTIONS(4796),
    [anon_sym_isize] = ACTIONS(4796),
    [anon_sym_usize] = ACTIONS(4796),
    [anon_sym_f32] = ACTIONS(4796),
    [anon_sym_f64] = ACTIONS(4796),
    [anon_sym_bool] = ACTIONS(4796),
    [anon_sym_str] = ACTIONS(4796),
    [anon_sym_char] = ACTIONS(4796),
    [anon_sym_DASH] = ACTIONS(4794),
    [anon_sym_AMP] = ACTIONS(4794),
    [anon_sym_PIPE] = ACTIONS(4794),
    [anon_sym_LT] = ACTIONS(4794),
    [anon_sym_DOT_DOT] = ACTIONS(4794),
    [anon_sym_COLON_COLON] = ACTIONS(4794),
    [anon_sym_POUND] = ACTIONS(4794),
    [anon_sym_SQUOTE] = ACTIONS(4796),
    [anon_sym_async] = ACTIONS(4796),
    [anon_sym_auto] = ACTIONS(4796),
    [anon_sym_await] = ACTIONS(4796),
    [anon_sym_become] = ACTIONS(4796),
    [anon_sym_break] = ACTIONS(4796),
    [anon_sym_const] = ACTIONS(4796),
    [anon_sym_continue] = ACTIONS(4796),
    [anon_sym_default] = ACTIONS(4796),
    [anon_sym_enum] = ACTIONS(4796),
    [anon_sym_fn] = ACTIONS(4796),
    [anon_sym_for] = ACTIONS(4796),
    [anon_sym_gen] = ACTIONS(4796),
    [anon_sym_if] = ACTIONS(4796),
    [anon_sym_impl] = ACTIONS(4796),
    [anon_sym_let] = ACTIONS(4796),
    [anon_sym_loop] = ACTIONS(4796),
    [anon_sym_match] = ACTIONS(4796),
    [anon_sym_mod] = ACTIONS(4796),
    [anon_sym_pub] = ACTIONS(4796),
    [anon_sym_return] = ACTIONS(4796),
    [anon_sym_raw] = ACTIONS(4796),
    [anon_sym_static] = ACTIONS(4796),
    [anon_sym_struct] = ACTIONS(4796),
    [anon_sym_trait] = ACTIONS(4796),
    [anon_sym_type] = ACTIONS(4796),
    [anon_sym_union] = ACTIONS(4796),
    [anon_sym_unsafe] = ACTIONS(4796),
    [anon_sym_use] = ACTIONS(4796),
    [anon_sym_while] = ACTIONS(4796),
    [anon_sym_extern] = ACTIONS(4796),
    [anon_sym_safe] = ACTIONS(4796),
    [anon_sym_yield] = ACTIONS(4796),
    [anon_sym_move] = ACTIONS(4796),
    [anon_sym_try] = ACTIONS(4796),
    [sym_integer_literal] = ACTIONS(4794),
    [aux_sym_string_literal_token1] = ACTIONS(4794),
    [sym_char_literal] = ACTIONS(4794),
    [anon_sym_true] = ACTIONS(4796),
    [anon_sym_false] = ACTIONS(4796),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4796),
    [sym_super] = ACTIONS(4796),
    [sym_crate] = ACTIONS(4796),
    [aux_sym_metavariable_token1] = ACTIONS(4794),
    [sym__raw_string_literal_start] = ACTIONS(4794),
    [sym_float_literal] = ACTIONS(4794),
    [sym__outer_block_doc_comment_start] = ACTIONS(4794),
    [sym__outer_line_doc_comment_start] = ACTIONS(4794),
  },
  [STATE(1266)] = {
    [sym_line_comment] = STATE(1266),
    [sym_block_comment] = STATE(1266),
    [ts_builtin_sym_end] = ACTIONS(4798),
    [sym_identifier] = ACTIONS(4800),
    [anon_sym_SEMI] = ACTIONS(4798),
    [anon_sym_macro] = ACTIONS(4800),
    [anon_sym_LPAREN] = ACTIONS(4798),
    [anon_sym_LBRACE] = ACTIONS(4798),
    [anon_sym_RBRACE] = ACTIONS(4798),
    [anon_sym_macro_rules] = ACTIONS(4800),
    [anon_sym_BANG] = ACTIONS(4798),
    [anon_sym_LBRACK] = ACTIONS(4798),
    [anon_sym_STAR] = ACTIONS(4798),
    [anon_sym_u8] = ACTIONS(4800),
    [anon_sym_i8] = ACTIONS(4800),
    [anon_sym_u16] = ACTIONS(4800),
    [anon_sym_i16] = ACTIONS(4800),
    [anon_sym_u32] = ACTIONS(4800),
    [anon_sym_i32] = ACTIONS(4800),
    [anon_sym_u64] = ACTIONS(4800),
    [anon_sym_i64] = ACTIONS(4800),
    [anon_sym_u128] = ACTIONS(4800),
    [anon_sym_i128] = ACTIONS(4800),
    [anon_sym_isize] = ACTIONS(4800),
    [anon_sym_usize] = ACTIONS(4800),
    [anon_sym_f32] = ACTIONS(4800),
    [anon_sym_f64] = ACTIONS(4800),
    [anon_sym_bool] = ACTIONS(4800),
    [anon_sym_str] = ACTIONS(4800),
    [anon_sym_char] = ACTIONS(4800),
    [anon_sym_DASH] = ACTIONS(4798),
    [anon_sym_AMP] = ACTIONS(4798),
    [anon_sym_PIPE] = ACTIONS(4798),
    [anon_sym_LT] = ACTIONS(4798),
    [anon_sym_DOT_DOT] = ACTIONS(4798),
    [anon_sym_COLON_COLON] = ACTIONS(4798),
    [anon_sym_POUND] = ACTIONS(4798),
    [anon_sym_SQUOTE] = ACTIONS(4800),
    [anon_sym_async] = ACTIONS(4800),
    [anon_sym_auto] = ACTIONS(4800),
    [anon_sym_await] = ACTIONS(4800),
    [anon_sym_become] = ACTIONS(4800),
    [anon_sym_break] = ACTIONS(4800),
    [anon_sym_const] = ACTIONS(4800),
    [anon_sym_continue] = ACTIONS(4800),
    [anon_sym_default] = ACTIONS(4800),
    [anon_sym_enum] = ACTIONS(4800),
    [anon_sym_fn] = ACTIONS(4800),
    [anon_sym_for] = ACTIONS(4800),
    [anon_sym_gen] = ACTIONS(4800),
    [anon_sym_if] = ACTIONS(4800),
    [anon_sym_impl] = ACTIONS(4800),
    [anon_sym_let] = ACTIONS(4800),
    [anon_sym_loop] = ACTIONS(4800),
    [anon_sym_match] = ACTIONS(4800),
    [anon_sym_mod] = ACTIONS(4800),
    [anon_sym_pub] = ACTIONS(4800),
    [anon_sym_return] = ACTIONS(4800),
    [anon_sym_raw] = ACTIONS(4800),
    [anon_sym_static] = ACTIONS(4800),
    [anon_sym_struct] = ACTIONS(4800),
    [anon_sym_trait] = ACTIONS(4800),
    [anon_sym_type] = ACTIONS(4800),
    [anon_sym_union] = ACTIONS(4800),
    [anon_sym_unsafe] = ACTIONS(4800),
    [anon_sym_use] = ACTIONS(4800),
    [anon_sym_while] = ACTIONS(4800),
    [anon_sym_extern] = ACTIONS(4800),
    [anon_sym_safe] = ACTIONS(4800),
    [anon_sym_yield] = ACTIONS(4800),
    [anon_sym_move] = ACTIONS(4800),
    [anon_sym_try] = ACTIONS(4800),
    [sym_integer_literal] = ACTIONS(4798),
    [aux_sym_string_literal_token1] = ACTIONS(4798),
    [sym_char_literal] = ACTIONS(4798),
    [anon_sym_true] = ACTIONS(4800),
    [anon_sym_false] = ACTIONS(4800),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4800),
    [sym_super] = ACTIONS(4800),
    [sym_crate] = ACTIONS(4800),
    [aux_sym_metavariable_token1] = ACTIONS(4798),
    [sym__raw_string_literal_start] = ACTIONS(4798),
    [sym_float_literal] = ACTIONS(4798),
    [sym__outer_block_doc_comment_start] = ACTIONS(4798),
    [sym__outer_line_doc_comment_start] = ACTIONS(4798),
  },
  [STATE(1267)] = {
    [sym_line_comment] = STATE(1267),
    [sym_block_comment] = STATE(1267),
    [ts_builtin_sym_end] = ACTIONS(4802),
    [sym_identifier] = ACTIONS(4804),
    [anon_sym_SEMI] = ACTIONS(4802),
    [anon_sym_macro] = ACTIONS(4804),
    [anon_sym_LPAREN] = ACTIONS(4802),
    [anon_sym_LBRACE] = ACTIONS(4802),
    [anon_sym_RBRACE] = ACTIONS(4802),
    [anon_sym_macro_rules] = ACTIONS(4804),
    [anon_sym_BANG] = ACTIONS(4802),
    [anon_sym_LBRACK] = ACTIONS(4802),
    [anon_sym_STAR] = ACTIONS(4802),
    [anon_sym_u8] = ACTIONS(4804),
    [anon_sym_i8] = ACTIONS(4804),
    [anon_sym_u16] = ACTIONS(4804),
    [anon_sym_i16] = ACTIONS(4804),
    [anon_sym_u32] = ACTIONS(4804),
    [anon_sym_i32] = ACTIONS(4804),
    [anon_sym_u64] = ACTIONS(4804),
    [anon_sym_i64] = ACTIONS(4804),
    [anon_sym_u128] = ACTIONS(4804),
    [anon_sym_i128] = ACTIONS(4804),
    [anon_sym_isize] = ACTIONS(4804),
    [anon_sym_usize] = ACTIONS(4804),
    [anon_sym_f32] = ACTIONS(4804),
    [anon_sym_f64] = ACTIONS(4804),
    [anon_sym_bool] = ACTIONS(4804),
    [anon_sym_str] = ACTIONS(4804),
    [anon_sym_char] = ACTIONS(4804),
    [anon_sym_DASH] = ACTIONS(4802),
    [anon_sym_AMP] = ACTIONS(4802),
    [anon_sym_PIPE] = ACTIONS(4802),
    [anon_sym_LT] = ACTIONS(4802),
    [anon_sym_DOT_DOT] = ACTIONS(4802),
    [anon_sym_COLON_COLON] = ACTIONS(4802),
    [anon_sym_POUND] = ACTIONS(4802),
    [anon_sym_SQUOTE] = ACTIONS(4804),
    [anon_sym_async] = ACTIONS(4804),
    [anon_sym_auto] = ACTIONS(4804),
    [anon_sym_await] = ACTIONS(4804),
    [anon_sym_become] = ACTIONS(4804),
    [anon_sym_break] = ACTIONS(4804),
    [anon_sym_const] = ACTIONS(4804),
    [anon_sym_continue] = ACTIONS(4804),
    [anon_sym_default] = ACTIONS(4804),
    [anon_sym_enum] = ACTIONS(4804),
    [anon_sym_fn] = ACTIONS(4804),
    [anon_sym_for] = ACTIONS(4804),
    [anon_sym_gen] = ACTIONS(4804),
    [anon_sym_if] = ACTIONS(4804),
    [anon_sym_impl] = ACTIONS(4804),
    [anon_sym_let] = ACTIONS(4804),
    [anon_sym_loop] = ACTIONS(4804),
    [anon_sym_match] = ACTIONS(4804),
    [anon_sym_mod] = ACTIONS(4804),
    [anon_sym_pub] = ACTIONS(4804),
    [anon_sym_return] = ACTIONS(4804),
    [anon_sym_raw] = ACTIONS(4804),
    [anon_sym_static] = ACTIONS(4804),
    [anon_sym_struct] = ACTIONS(4804),
    [anon_sym_trait] = ACTIONS(4804),
    [anon_sym_type] = ACTIONS(4804),
    [anon_sym_union] = ACTIONS(4804),
    [anon_sym_unsafe] = ACTIONS(4804),
    [anon_sym_use] = ACTIONS(4804),
    [anon_sym_while] = ACTIONS(4804),
    [anon_sym_extern] = ACTIONS(4804),
    [anon_sym_safe] = ACTIONS(4804),
    [anon_sym_yield] = ACTIONS(4804),
    [anon_sym_move] = ACTIONS(4804),
    [anon_sym_try] = ACTIONS(4804),
    [sym_integer_literal] = ACTIONS(4802),
    [aux_sym_string_literal_token1] = ACTIONS(4802),
    [sym_char_literal] = ACTIONS(4802),
    [anon_sym_true] = ACTIONS(4804),
    [anon_sym_false] = ACTIONS(4804),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4804),
    [sym_super] = ACTIONS(4804),
    [sym_crate] = ACTIONS(4804),
    [aux_sym_metavariable_token1] = ACTIONS(4802),
    [sym__raw_string_literal_start] = ACTIONS(4802),
    [sym_float_literal] = ACTIONS(4802),
    [sym__outer_block_doc_comment_start] = ACTIONS(4802),
    [sym__outer_line_doc_comment_start] = ACTIONS(4802),
  },
  [STATE(1268)] = {
    [sym_line_comment] = STATE(1268),
    [sym_block_comment] = STATE(1268),
    [ts_builtin_sym_end] = ACTIONS(4806),
    [sym_identifier] = ACTIONS(4808),
    [anon_sym_SEMI] = ACTIONS(4806),
    [anon_sym_macro] = ACTIONS(4808),
    [anon_sym_LPAREN] = ACTIONS(4806),
    [anon_sym_LBRACE] = ACTIONS(4806),
    [anon_sym_RBRACE] = ACTIONS(4806),
    [anon_sym_macro_rules] = ACTIONS(4808),
    [anon_sym_BANG] = ACTIONS(4806),
    [anon_sym_LBRACK] = ACTIONS(4806),
    [anon_sym_STAR] = ACTIONS(4806),
    [anon_sym_u8] = ACTIONS(4808),
    [anon_sym_i8] = ACTIONS(4808),
    [anon_sym_u16] = ACTIONS(4808),
    [anon_sym_i16] = ACTIONS(4808),
    [anon_sym_u32] = ACTIONS(4808),
    [anon_sym_i32] = ACTIONS(4808),
    [anon_sym_u64] = ACTIONS(4808),
    [anon_sym_i64] = ACTIONS(4808),
    [anon_sym_u128] = ACTIONS(4808),
    [anon_sym_i128] = ACTIONS(4808),
    [anon_sym_isize] = ACTIONS(4808),
    [anon_sym_usize] = ACTIONS(4808),
    [anon_sym_f32] = ACTIONS(4808),
    [anon_sym_f64] = ACTIONS(4808),
    [anon_sym_bool] = ACTIONS(4808),
    [anon_sym_str] = ACTIONS(4808),
    [anon_sym_char] = ACTIONS(4808),
    [anon_sym_DASH] = ACTIONS(4806),
    [anon_sym_AMP] = ACTIONS(4806),
    [anon_sym_PIPE] = ACTIONS(4806),
    [anon_sym_LT] = ACTIONS(4806),
    [anon_sym_DOT_DOT] = ACTIONS(4806),
    [anon_sym_COLON_COLON] = ACTIONS(4806),
    [anon_sym_POUND] = ACTIONS(4806),
    [anon_sym_SQUOTE] = ACTIONS(4808),
    [anon_sym_async] = ACTIONS(4808),
    [anon_sym_auto] = ACTIONS(4808),
    [anon_sym_await] = ACTIONS(4808),
    [anon_sym_become] = ACTIONS(4808),
    [anon_sym_break] = ACTIONS(4808),
    [anon_sym_const] = ACTIONS(4808),
    [anon_sym_continue] = ACTIONS(4808),
    [anon_sym_default] = ACTIONS(4808),
    [anon_sym_enum] = ACTIONS(4808),
    [anon_sym_fn] = ACTIONS(4808),
    [anon_sym_for] = ACTIONS(4808),
    [anon_sym_gen] = ACTIONS(4808),
    [anon_sym_if] = ACTIONS(4808),
    [anon_sym_impl] = ACTIONS(4808),
    [anon_sym_let] = ACTIONS(4808),
    [anon_sym_loop] = ACTIONS(4808),
    [anon_sym_match] = ACTIONS(4808),
    [anon_sym_mod] = ACTIONS(4808),
    [anon_sym_pub] = ACTIONS(4808),
    [anon_sym_return] = ACTIONS(4808),
    [anon_sym_raw] = ACTIONS(4808),
    [anon_sym_static] = ACTIONS(4808),
    [anon_sym_struct] = ACTIONS(4808),
    [anon_sym_trait] = ACTIONS(4808),
    [anon_sym_type] = ACTIONS(4808),
    [anon_sym_union] = ACTIONS(4808),
    [anon_sym_unsafe] = ACTIONS(4808),
    [anon_sym_use] = ACTIONS(4808),
    [anon_sym_while] = ACTIONS(4808),
    [anon_sym_extern] = ACTIONS(4808),
    [anon_sym_safe] = ACTIONS(4808),
    [anon_sym_yield] = ACTIONS(4808),
    [anon_sym_move] = ACTIONS(4808),
    [anon_sym_try] = ACTIONS(4808),
    [sym_integer_literal] = ACTIONS(4806),
    [aux_sym_string_literal_token1] = ACTIONS(4806),
    [sym_char_literal] = ACTIONS(4806),
    [anon_sym_true] = ACTIONS(4808),
    [anon_sym_false] = ACTIONS(4808),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4808),
    [sym_super] = ACTIONS(4808),
    [sym_crate] = ACTIONS(4808),
    [aux_sym_metavariable_token1] = ACTIONS(4806),
    [sym__raw_string_literal_start] = ACTIONS(4806),
    [sym_float_literal] = ACTIONS(4806),
    [sym__outer_block_doc_comment_start] = ACTIONS(4806),
    [sym__outer_line_doc_comment_start] = ACTIONS(4806),
  },
  [STATE(1269)] = {
    [sym_line_comment] = STATE(1269),
    [sym_block_comment] = STATE(1269),
    [ts_builtin_sym_end] = ACTIONS(4810),
    [sym_identifier] = ACTIONS(4812),
    [anon_sym_SEMI] = ACTIONS(4810),
    [anon_sym_macro] = ACTIONS(4812),
    [anon_sym_LPAREN] = ACTIONS(4810),
    [anon_sym_LBRACE] = ACTIONS(4810),
    [anon_sym_RBRACE] = ACTIONS(4810),
    [anon_sym_macro_rules] = ACTIONS(4812),
    [anon_sym_BANG] = ACTIONS(4810),
    [anon_sym_LBRACK] = ACTIONS(4810),
    [anon_sym_STAR] = ACTIONS(4810),
    [anon_sym_u8] = ACTIONS(4812),
    [anon_sym_i8] = ACTIONS(4812),
    [anon_sym_u16] = ACTIONS(4812),
    [anon_sym_i16] = ACTIONS(4812),
    [anon_sym_u32] = ACTIONS(4812),
    [anon_sym_i32] = ACTIONS(4812),
    [anon_sym_u64] = ACTIONS(4812),
    [anon_sym_i64] = ACTIONS(4812),
    [anon_sym_u128] = ACTIONS(4812),
    [anon_sym_i128] = ACTIONS(4812),
    [anon_sym_isize] = ACTIONS(4812),
    [anon_sym_usize] = ACTIONS(4812),
    [anon_sym_f32] = ACTIONS(4812),
    [anon_sym_f64] = ACTIONS(4812),
    [anon_sym_bool] = ACTIONS(4812),
    [anon_sym_str] = ACTIONS(4812),
    [anon_sym_char] = ACTIONS(4812),
    [anon_sym_DASH] = ACTIONS(4810),
    [anon_sym_AMP] = ACTIONS(4810),
    [anon_sym_PIPE] = ACTIONS(4810),
    [anon_sym_LT] = ACTIONS(4810),
    [anon_sym_DOT_DOT] = ACTIONS(4810),
    [anon_sym_COLON_COLON] = ACTIONS(4810),
    [anon_sym_POUND] = ACTIONS(4810),
    [anon_sym_SQUOTE] = ACTIONS(4812),
    [anon_sym_async] = ACTIONS(4812),
    [anon_sym_auto] = ACTIONS(4812),
    [anon_sym_await] = ACTIONS(4812),
    [anon_sym_become] = ACTIONS(4812),
    [anon_sym_break] = ACTIONS(4812),
    [anon_sym_const] = ACTIONS(4812),
    [anon_sym_continue] = ACTIONS(4812),
    [anon_sym_default] = ACTIONS(4812),
    [anon_sym_enum] = ACTIONS(4812),
    [anon_sym_fn] = ACTIONS(4812),
    [anon_sym_for] = ACTIONS(4812),
    [anon_sym_gen] = ACTIONS(4812),
    [anon_sym_if] = ACTIONS(4812),
    [anon_sym_impl] = ACTIONS(4812),
    [anon_sym_let] = ACTIONS(4812),
    [anon_sym_loop] = ACTIONS(4812),
    [anon_sym_match] = ACTIONS(4812),
    [anon_sym_mod] = ACTIONS(4812),
    [anon_sym_pub] = ACTIONS(4812),
    [anon_sym_return] = ACTIONS(4812),
    [anon_sym_raw] = ACTIONS(4812),
    [anon_sym_static] = ACTIONS(4812),
    [anon_sym_struct] = ACTIONS(4812),
    [anon_sym_trait] = ACTIONS(4812),
    [anon_sym_type] = ACTIONS(4812),
    [anon_sym_union] = ACTIONS(4812),
    [anon_sym_unsafe] = ACTIONS(4812),
    [anon_sym_use] = ACTIONS(4812),
    [anon_sym_while] = ACTIONS(4812),
    [anon_sym_extern] = ACTIONS(4812),
    [anon_sym_safe] = ACTIONS(4812),
    [anon_sym_yield] = ACTIONS(4812),
    [anon_sym_move] = ACTIONS(4812),
    [anon_sym_try] = ACTIONS(4812),
    [sym_integer_literal] = ACTIONS(4810),
    [aux_sym_string_literal_token1] = ACTIONS(4810),
    [sym_char_literal] = ACTIONS(4810),
    [anon_sym_true] = ACTIONS(4812),
    [anon_sym_false] = ACTIONS(4812),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4812),
    [sym_super] = ACTIONS(4812),
    [sym_crate] = ACTIONS(4812),
    [aux_sym_metavariable_token1] = ACTIONS(4810),
    [sym__raw_string_literal_start] = ACTIONS(4810),
    [sym_float_literal] = ACTIONS(4810),
    [sym__outer_block_doc_comment_start] = ACTIONS(4810),
    [sym__outer_line_doc_comment_start] = ACTIONS(4810),
  },
  [STATE(1270)] = {
    [sym_line_comment] = STATE(1270),
    [sym_block_comment] = STATE(1270),
    [ts_builtin_sym_end] = ACTIONS(4814),
    [sym_identifier] = ACTIONS(4816),
    [anon_sym_SEMI] = ACTIONS(4814),
    [anon_sym_macro] = ACTIONS(4816),
    [anon_sym_LPAREN] = ACTIONS(4814),
    [anon_sym_LBRACE] = ACTIONS(4814),
    [anon_sym_RBRACE] = ACTIONS(4814),
    [anon_sym_macro_rules] = ACTIONS(4816),
    [anon_sym_BANG] = ACTIONS(4814),
    [anon_sym_LBRACK] = ACTIONS(4814),
    [anon_sym_STAR] = ACTIONS(4814),
    [anon_sym_u8] = ACTIONS(4816),
    [anon_sym_i8] = ACTIONS(4816),
    [anon_sym_u16] = ACTIONS(4816),
    [anon_sym_i16] = ACTIONS(4816),
    [anon_sym_u32] = ACTIONS(4816),
    [anon_sym_i32] = ACTIONS(4816),
    [anon_sym_u64] = ACTIONS(4816),
    [anon_sym_i64] = ACTIONS(4816),
    [anon_sym_u128] = ACTIONS(4816),
    [anon_sym_i128] = ACTIONS(4816),
    [anon_sym_isize] = ACTIONS(4816),
    [anon_sym_usize] = ACTIONS(4816),
    [anon_sym_f32] = ACTIONS(4816),
    [anon_sym_f64] = ACTIONS(4816),
    [anon_sym_bool] = ACTIONS(4816),
    [anon_sym_str] = ACTIONS(4816),
    [anon_sym_char] = ACTIONS(4816),
    [anon_sym_DASH] = ACTIONS(4814),
    [anon_sym_AMP] = ACTIONS(4814),
    [anon_sym_PIPE] = ACTIONS(4814),
    [anon_sym_LT] = ACTIONS(4814),
    [anon_sym_DOT_DOT] = ACTIONS(4814),
    [anon_sym_COLON_COLON] = ACTIONS(4814),
    [anon_sym_POUND] = ACTIONS(4814),
    [anon_sym_SQUOTE] = ACTIONS(4816),
    [anon_sym_async] = ACTIONS(4816),
    [anon_sym_auto] = ACTIONS(4816),
    [anon_sym_await] = ACTIONS(4816),
    [anon_sym_become] = ACTIONS(4816),
    [anon_sym_break] = ACTIONS(4816),
    [anon_sym_const] = ACTIONS(4816),
    [anon_sym_continue] = ACTIONS(4816),
    [anon_sym_default] = ACTIONS(4816),
    [anon_sym_enum] = ACTIONS(4816),
    [anon_sym_fn] = ACTIONS(4816),
    [anon_sym_for] = ACTIONS(4816),
    [anon_sym_gen] = ACTIONS(4816),
    [anon_sym_if] = ACTIONS(4816),
    [anon_sym_impl] = ACTIONS(4816),
    [anon_sym_let] = ACTIONS(4816),
    [anon_sym_loop] = ACTIONS(4816),
    [anon_sym_match] = ACTIONS(4816),
    [anon_sym_mod] = ACTIONS(4816),
    [anon_sym_pub] = ACTIONS(4816),
    [anon_sym_return] = ACTIONS(4816),
    [anon_sym_raw] = ACTIONS(4816),
    [anon_sym_static] = ACTIONS(4816),
    [anon_sym_struct] = ACTIONS(4816),
    [anon_sym_trait] = ACTIONS(4816),
    [anon_sym_type] = ACTIONS(4816),
    [anon_sym_union] = ACTIONS(4816),
    [anon_sym_unsafe] = ACTIONS(4816),
    [anon_sym_use] = ACTIONS(4816),
    [anon_sym_while] = ACTIONS(4816),
    [anon_sym_extern] = ACTIONS(4816),
    [anon_sym_safe] = ACTIONS(4816),
    [anon_sym_yield] = ACTIONS(4816),
    [anon_sym_move] = ACTIONS(4816),
    [anon_sym_try] = ACTIONS(4816),
    [sym_integer_literal] = ACTIONS(4814),
    [aux_sym_string_literal_token1] = ACTIONS(4814),
    [sym_char_literal] = ACTIONS(4814),
    [anon_sym_true] = ACTIONS(4816),
    [anon_sym_false] = ACTIONS(4816),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4816),
    [sym_super] = ACTIONS(4816),
    [sym_crate] = ACTIONS(4816),
    [aux_sym_metavariable_token1] = ACTIONS(4814),
    [sym__raw_string_literal_start] = ACTIONS(4814),
    [sym_float_literal] = ACTIONS(4814),
    [sym__outer_block_doc_comment_start] = ACTIONS(4814),
    [sym__outer_line_doc_comment_start] = ACTIONS(4814),
  },
  [STATE(1271)] = {
    [sym_line_comment] = STATE(1271),
    [sym_block_comment] = STATE(1271),
    [ts_builtin_sym_end] = ACTIONS(4818),
    [sym_identifier] = ACTIONS(4820),
    [anon_sym_SEMI] = ACTIONS(4818),
    [anon_sym_macro] = ACTIONS(4820),
    [anon_sym_LPAREN] = ACTIONS(4818),
    [anon_sym_LBRACE] = ACTIONS(4818),
    [anon_sym_RBRACE] = ACTIONS(4818),
    [anon_sym_macro_rules] = ACTIONS(4820),
    [anon_sym_BANG] = ACTIONS(4818),
    [anon_sym_LBRACK] = ACTIONS(4818),
    [anon_sym_STAR] = ACTIONS(4818),
    [anon_sym_u8] = ACTIONS(4820),
    [anon_sym_i8] = ACTIONS(4820),
    [anon_sym_u16] = ACTIONS(4820),
    [anon_sym_i16] = ACTIONS(4820),
    [anon_sym_u32] = ACTIONS(4820),
    [anon_sym_i32] = ACTIONS(4820),
    [anon_sym_u64] = ACTIONS(4820),
    [anon_sym_i64] = ACTIONS(4820),
    [anon_sym_u128] = ACTIONS(4820),
    [anon_sym_i128] = ACTIONS(4820),
    [anon_sym_isize] = ACTIONS(4820),
    [anon_sym_usize] = ACTIONS(4820),
    [anon_sym_f32] = ACTIONS(4820),
    [anon_sym_f64] = ACTIONS(4820),
    [anon_sym_bool] = ACTIONS(4820),
    [anon_sym_str] = ACTIONS(4820),
    [anon_sym_char] = ACTIONS(4820),
    [anon_sym_DASH] = ACTIONS(4818),
    [anon_sym_AMP] = ACTIONS(4818),
    [anon_sym_PIPE] = ACTIONS(4818),
    [anon_sym_LT] = ACTIONS(4818),
    [anon_sym_DOT_DOT] = ACTIONS(4818),
    [anon_sym_COLON_COLON] = ACTIONS(4818),
    [anon_sym_POUND] = ACTIONS(4818),
    [anon_sym_SQUOTE] = ACTIONS(4820),
    [anon_sym_async] = ACTIONS(4820),
    [anon_sym_auto] = ACTIONS(4820),
    [anon_sym_await] = ACTIONS(4820),
    [anon_sym_become] = ACTIONS(4820),
    [anon_sym_break] = ACTIONS(4820),
    [anon_sym_const] = ACTIONS(4820),
    [anon_sym_continue] = ACTIONS(4820),
    [anon_sym_default] = ACTIONS(4820),
    [anon_sym_enum] = ACTIONS(4820),
    [anon_sym_fn] = ACTIONS(4820),
    [anon_sym_for] = ACTIONS(4820),
    [anon_sym_gen] = ACTIONS(4820),
    [anon_sym_if] = ACTIONS(4820),
    [anon_sym_impl] = ACTIONS(4820),
    [anon_sym_let] = ACTIONS(4820),
    [anon_sym_loop] = ACTIONS(4820),
    [anon_sym_match] = ACTIONS(4820),
    [anon_sym_mod] = ACTIONS(4820),
    [anon_sym_pub] = ACTIONS(4820),
    [anon_sym_return] = ACTIONS(4820),
    [anon_sym_raw] = ACTIONS(4820),
    [anon_sym_static] = ACTIONS(4820),
    [anon_sym_struct] = ACTIONS(4820),
    [anon_sym_trait] = ACTIONS(4820),
    [anon_sym_type] = ACTIONS(4820),
    [anon_sym_union] = ACTIONS(4820),
    [anon_sym_unsafe] = ACTIONS(4820),
    [anon_sym_use] = ACTIONS(4820),
    [anon_sym_while] = ACTIONS(4820),
    [anon_sym_extern] = ACTIONS(4820),
    [anon_sym_safe] = ACTIONS(4820),
    [anon_sym_yield] = ACTIONS(4820),
    [anon_sym_move] = ACTIONS(4820),
    [anon_sym_try] = ACTIONS(4820),
    [sym_integer_literal] = ACTIONS(4818),
    [aux_sym_string_literal_token1] = ACTIONS(4818),
    [sym_char_literal] = ACTIONS(4818),
    [anon_sym_true] = ACTIONS(4820),
    [anon_sym_false] = ACTIONS(4820),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4820),
    [sym_super] = ACTIONS(4820),
    [sym_crate] = ACTIONS(4820),
    [aux_sym_metavariable_token1] = ACTIONS(4818),
    [sym__raw_string_literal_start] = ACTIONS(4818),
    [sym_float_literal] = ACTIONS(4818),
    [sym__outer_block_doc_comment_start] = ACTIONS(4818),
    [sym__outer_line_doc_comment_start] = ACTIONS(4818),
  },
  [STATE(1272)] = {
    [sym_line_comment] = STATE(1272),
    [sym_block_comment] = STATE(1272),
    [ts_builtin_sym_end] = ACTIONS(4822),
    [sym_identifier] = ACTIONS(4824),
    [anon_sym_SEMI] = ACTIONS(4822),
    [anon_sym_macro] = ACTIONS(4824),
    [anon_sym_LPAREN] = ACTIONS(4822),
    [anon_sym_LBRACE] = ACTIONS(4822),
    [anon_sym_RBRACE] = ACTIONS(4822),
    [anon_sym_macro_rules] = ACTIONS(4824),
    [anon_sym_BANG] = ACTIONS(4822),
    [anon_sym_LBRACK] = ACTIONS(4822),
    [anon_sym_STAR] = ACTIONS(4822),
    [anon_sym_u8] = ACTIONS(4824),
    [anon_sym_i8] = ACTIONS(4824),
    [anon_sym_u16] = ACTIONS(4824),
    [anon_sym_i16] = ACTIONS(4824),
    [anon_sym_u32] = ACTIONS(4824),
    [anon_sym_i32] = ACTIONS(4824),
    [anon_sym_u64] = ACTIONS(4824),
    [anon_sym_i64] = ACTIONS(4824),
    [anon_sym_u128] = ACTIONS(4824),
    [anon_sym_i128] = ACTIONS(4824),
    [anon_sym_isize] = ACTIONS(4824),
    [anon_sym_usize] = ACTIONS(4824),
    [anon_sym_f32] = ACTIONS(4824),
    [anon_sym_f64] = ACTIONS(4824),
    [anon_sym_bool] = ACTIONS(4824),
    [anon_sym_str] = ACTIONS(4824),
    [anon_sym_char] = ACTIONS(4824),
    [anon_sym_DASH] = ACTIONS(4822),
    [anon_sym_AMP] = ACTIONS(4822),
    [anon_sym_PIPE] = ACTIONS(4822),
    [anon_sym_LT] = ACTIONS(4822),
    [anon_sym_DOT_DOT] = ACTIONS(4822),
    [anon_sym_COLON_COLON] = ACTIONS(4822),
    [anon_sym_POUND] = ACTIONS(4822),
    [anon_sym_SQUOTE] = ACTIONS(4824),
    [anon_sym_async] = ACTIONS(4824),
    [anon_sym_auto] = ACTIONS(4824),
    [anon_sym_await] = ACTIONS(4824),
    [anon_sym_become] = ACTIONS(4824),
    [anon_sym_break] = ACTIONS(4824),
    [anon_sym_const] = ACTIONS(4824),
    [anon_sym_continue] = ACTIONS(4824),
    [anon_sym_default] = ACTIONS(4824),
    [anon_sym_enum] = ACTIONS(4824),
    [anon_sym_fn] = ACTIONS(4824),
    [anon_sym_for] = ACTIONS(4824),
    [anon_sym_gen] = ACTIONS(4824),
    [anon_sym_if] = ACTIONS(4824),
    [anon_sym_impl] = ACTIONS(4824),
    [anon_sym_let] = ACTIONS(4824),
    [anon_sym_loop] = ACTIONS(4824),
    [anon_sym_match] = ACTIONS(4824),
    [anon_sym_mod] = ACTIONS(4824),
    [anon_sym_pub] = ACTIONS(4824),
    [anon_sym_return] = ACTIONS(4824),
    [anon_sym_raw] = ACTIONS(4824),
    [anon_sym_static] = ACTIONS(4824),
    [anon_sym_struct] = ACTIONS(4824),
    [anon_sym_trait] = ACTIONS(4824),
    [anon_sym_type] = ACTIONS(4824),
    [anon_sym_union] = ACTIONS(4824),
    [anon_sym_unsafe] = ACTIONS(4824),
    [anon_sym_use] = ACTIONS(4824),
    [anon_sym_while] = ACTIONS(4824),
    [anon_sym_extern] = ACTIONS(4824),
    [anon_sym_safe] = ACTIONS(4824),
    [anon_sym_yield] = ACTIONS(4824),
    [anon_sym_move] = ACTIONS(4824),
    [anon_sym_try] = ACTIONS(4824),
    [sym_integer_literal] = ACTIONS(4822),
    [aux_sym_string_literal_token1] = ACTIONS(4822),
    [sym_char_literal] = ACTIONS(4822),
    [anon_sym_true] = ACTIONS(4824),
    [anon_sym_false] = ACTIONS(4824),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4824),
    [sym_super] = ACTIONS(4824),
    [sym_crate] = ACTIONS(4824),
    [aux_sym_metavariable_token1] = ACTIONS(4822),
    [sym__raw_string_literal_start] = ACTIONS(4822),
    [sym_float_literal] = ACTIONS(4822),
    [sym__outer_block_doc_comment_start] = ACTIONS(4822),
    [sym__outer_line_doc_comment_start] = ACTIONS(4822),
  },
  [STATE(1273)] = {
    [sym_line_comment] = STATE(1273),
    [sym_block_comment] = STATE(1273),
    [ts_builtin_sym_end] = ACTIONS(4826),
    [sym_identifier] = ACTIONS(4828),
    [anon_sym_SEMI] = ACTIONS(4826),
    [anon_sym_macro] = ACTIONS(4828),
    [anon_sym_LPAREN] = ACTIONS(4826),
    [anon_sym_LBRACE] = ACTIONS(4826),
    [anon_sym_RBRACE] = ACTIONS(4826),
    [anon_sym_macro_rules] = ACTIONS(4828),
    [anon_sym_BANG] = ACTIONS(4826),
    [anon_sym_LBRACK] = ACTIONS(4826),
    [anon_sym_STAR] = ACTIONS(4826),
    [anon_sym_u8] = ACTIONS(4828),
    [anon_sym_i8] = ACTIONS(4828),
    [anon_sym_u16] = ACTIONS(4828),
    [anon_sym_i16] = ACTIONS(4828),
    [anon_sym_u32] = ACTIONS(4828),
    [anon_sym_i32] = ACTIONS(4828),
    [anon_sym_u64] = ACTIONS(4828),
    [anon_sym_i64] = ACTIONS(4828),
    [anon_sym_u128] = ACTIONS(4828),
    [anon_sym_i128] = ACTIONS(4828),
    [anon_sym_isize] = ACTIONS(4828),
    [anon_sym_usize] = ACTIONS(4828),
    [anon_sym_f32] = ACTIONS(4828),
    [anon_sym_f64] = ACTIONS(4828),
    [anon_sym_bool] = ACTIONS(4828),
    [anon_sym_str] = ACTIONS(4828),
    [anon_sym_char] = ACTIONS(4828),
    [anon_sym_DASH] = ACTIONS(4826),
    [anon_sym_AMP] = ACTIONS(4826),
    [anon_sym_PIPE] = ACTIONS(4826),
    [anon_sym_LT] = ACTIONS(4826),
    [anon_sym_DOT_DOT] = ACTIONS(4826),
    [anon_sym_COLON_COLON] = ACTIONS(4826),
    [anon_sym_POUND] = ACTIONS(4826),
    [anon_sym_SQUOTE] = ACTIONS(4828),
    [anon_sym_async] = ACTIONS(4828),
    [anon_sym_auto] = ACTIONS(4828),
    [anon_sym_await] = ACTIONS(4828),
    [anon_sym_become] = ACTIONS(4828),
    [anon_sym_break] = ACTIONS(4828),
    [anon_sym_const] = ACTIONS(4828),
    [anon_sym_continue] = ACTIONS(4828),
    [anon_sym_default] = ACTIONS(4828),
    [anon_sym_enum] = ACTIONS(4828),
    [anon_sym_fn] = ACTIONS(4828),
    [anon_sym_for] = ACTIONS(4828),
    [anon_sym_gen] = ACTIONS(4828),
    [anon_sym_if] = ACTIONS(4828),
    [anon_sym_impl] = ACTIONS(4828),
    [anon_sym_let] = ACTIONS(4828),
    [anon_sym_loop] = ACTIONS(4828),
    [anon_sym_match] = ACTIONS(4828),
    [anon_sym_mod] = ACTIONS(4828),
    [anon_sym_pub] = ACTIONS(4828),
    [anon_sym_return] = ACTIONS(4828),
    [anon_sym_raw] = ACTIONS(4828),
    [anon_sym_static] = ACTIONS(4828),
    [anon_sym_struct] = ACTIONS(4828),
    [anon_sym_trait] = ACTIONS(4828),
    [anon_sym_type] = ACTIONS(4828),
    [anon_sym_union] = ACTIONS(4828),
    [anon_sym_unsafe] = ACTIONS(4828),
    [anon_sym_use] = ACTIONS(4828),
    [anon_sym_while] = ACTIONS(4828),
    [anon_sym_extern] = ACTIONS(4828),
    [anon_sym_safe] = ACTIONS(4828),
    [anon_sym_yield] = ACTIONS(4828),
    [anon_sym_move] = ACTIONS(4828),
    [anon_sym_try] = ACTIONS(4828),
    [sym_integer_literal] = ACTIONS(4826),
    [aux_sym_string_literal_token1] = ACTIONS(4826),
    [sym_char_literal] = ACTIONS(4826),
    [anon_sym_true] = ACTIONS(4828),
    [anon_sym_false] = ACTIONS(4828),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4828),
    [sym_super] = ACTIONS(4828),
    [sym_crate] = ACTIONS(4828),
    [aux_sym_metavariable_token1] = ACTIONS(4826),
    [sym__raw_string_literal_start] = ACTIONS(4826),
    [sym_float_literal] = ACTIONS(4826),
    [sym__outer_block_doc_comment_start] = ACTIONS(4826),
    [sym__outer_line_doc_comment_start] = ACTIONS(4826),
  },
  [STATE(1274)] = {
    [sym_line_comment] = STATE(1274),
    [sym_block_comment] = STATE(1274),
    [ts_builtin_sym_end] = ACTIONS(4830),
    [sym_identifier] = ACTIONS(4832),
    [anon_sym_SEMI] = ACTIONS(4830),
    [anon_sym_macro] = ACTIONS(4832),
    [anon_sym_LPAREN] = ACTIONS(4830),
    [anon_sym_LBRACE] = ACTIONS(4830),
    [anon_sym_RBRACE] = ACTIONS(4830),
    [anon_sym_macro_rules] = ACTIONS(4832),
    [anon_sym_BANG] = ACTIONS(4830),
    [anon_sym_LBRACK] = ACTIONS(4830),
    [anon_sym_STAR] = ACTIONS(4830),
    [anon_sym_u8] = ACTIONS(4832),
    [anon_sym_i8] = ACTIONS(4832),
    [anon_sym_u16] = ACTIONS(4832),
    [anon_sym_i16] = ACTIONS(4832),
    [anon_sym_u32] = ACTIONS(4832),
    [anon_sym_i32] = ACTIONS(4832),
    [anon_sym_u64] = ACTIONS(4832),
    [anon_sym_i64] = ACTIONS(4832),
    [anon_sym_u128] = ACTIONS(4832),
    [anon_sym_i128] = ACTIONS(4832),
    [anon_sym_isize] = ACTIONS(4832),
    [anon_sym_usize] = ACTIONS(4832),
    [anon_sym_f32] = ACTIONS(4832),
    [anon_sym_f64] = ACTIONS(4832),
    [anon_sym_bool] = ACTIONS(4832),
    [anon_sym_str] = ACTIONS(4832),
    [anon_sym_char] = ACTIONS(4832),
    [anon_sym_DASH] = ACTIONS(4830),
    [anon_sym_AMP] = ACTIONS(4830),
    [anon_sym_PIPE] = ACTIONS(4830),
    [anon_sym_LT] = ACTIONS(4830),
    [anon_sym_DOT_DOT] = ACTIONS(4830),
    [anon_sym_COLON_COLON] = ACTIONS(4830),
    [anon_sym_POUND] = ACTIONS(4830),
    [anon_sym_SQUOTE] = ACTIONS(4832),
    [anon_sym_async] = ACTIONS(4832),
    [anon_sym_auto] = ACTIONS(4832),
    [anon_sym_await] = ACTIONS(4832),
    [anon_sym_become] = ACTIONS(4832),
    [anon_sym_break] = ACTIONS(4832),
    [anon_sym_const] = ACTIONS(4832),
    [anon_sym_continue] = ACTIONS(4832),
    [anon_sym_default] = ACTIONS(4832),
    [anon_sym_enum] = ACTIONS(4832),
    [anon_sym_fn] = ACTIONS(4832),
    [anon_sym_for] = ACTIONS(4832),
    [anon_sym_gen] = ACTIONS(4832),
    [anon_sym_if] = ACTIONS(4832),
    [anon_sym_impl] = ACTIONS(4832),
    [anon_sym_let] = ACTIONS(4832),
    [anon_sym_loop] = ACTIONS(4832),
    [anon_sym_match] = ACTIONS(4832),
    [anon_sym_mod] = ACTIONS(4832),
    [anon_sym_pub] = ACTIONS(4832),
    [anon_sym_return] = ACTIONS(4832),
    [anon_sym_raw] = ACTIONS(4832),
    [anon_sym_static] = ACTIONS(4832),
    [anon_sym_struct] = ACTIONS(4832),
    [anon_sym_trait] = ACTIONS(4832),
    [anon_sym_type] = ACTIONS(4832),
    [anon_sym_union] = ACTIONS(4832),
    [anon_sym_unsafe] = ACTIONS(4832),
    [anon_sym_use] = ACTIONS(4832),
    [anon_sym_while] = ACTIONS(4832),
    [anon_sym_extern] = ACTIONS(4832),
    [anon_sym_safe] = ACTIONS(4832),
    [anon_sym_yield] = ACTIONS(4832),
    [anon_sym_move] = ACTIONS(4832),
    [anon_sym_try] = ACTIONS(4832),
    [sym_integer_literal] = ACTIONS(4830),
    [aux_sym_string_literal_token1] = ACTIONS(4830),
    [sym_char_literal] = ACTIONS(4830),
    [anon_sym_true] = ACTIONS(4832),
    [anon_sym_false] = ACTIONS(4832),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4832),
    [sym_super] = ACTIONS(4832),
    [sym_crate] = ACTIONS(4832),
    [aux_sym_metavariable_token1] = ACTIONS(4830),
    [sym__raw_string_literal_start] = ACTIONS(4830),
    [sym_float_literal] = ACTIONS(4830),
    [sym__outer_block_doc_comment_start] = ACTIONS(4830),
    [sym__outer_line_doc_comment_start] = ACTIONS(4830),
  },
  [STATE(1275)] = {
    [sym_line_comment] = STATE(1275),
    [sym_block_comment] = STATE(1275),
    [ts_builtin_sym_end] = ACTIONS(4834),
    [sym_identifier] = ACTIONS(4836),
    [anon_sym_SEMI] = ACTIONS(4834),
    [anon_sym_macro] = ACTIONS(4836),
    [anon_sym_LPAREN] = ACTIONS(4834),
    [anon_sym_LBRACE] = ACTIONS(4834),
    [anon_sym_RBRACE] = ACTIONS(4834),
    [anon_sym_macro_rules] = ACTIONS(4836),
    [anon_sym_BANG] = ACTIONS(4834),
    [anon_sym_LBRACK] = ACTIONS(4834),
    [anon_sym_STAR] = ACTIONS(4834),
    [anon_sym_u8] = ACTIONS(4836),
    [anon_sym_i8] = ACTIONS(4836),
    [anon_sym_u16] = ACTIONS(4836),
    [anon_sym_i16] = ACTIONS(4836),
    [anon_sym_u32] = ACTIONS(4836),
    [anon_sym_i32] = ACTIONS(4836),
    [anon_sym_u64] = ACTIONS(4836),
    [anon_sym_i64] = ACTIONS(4836),
    [anon_sym_u128] = ACTIONS(4836),
    [anon_sym_i128] = ACTIONS(4836),
    [anon_sym_isize] = ACTIONS(4836),
    [anon_sym_usize] = ACTIONS(4836),
    [anon_sym_f32] = ACTIONS(4836),
    [anon_sym_f64] = ACTIONS(4836),
    [anon_sym_bool] = ACTIONS(4836),
    [anon_sym_str] = ACTIONS(4836),
    [anon_sym_char] = ACTIONS(4836),
    [anon_sym_DASH] = ACTIONS(4834),
    [anon_sym_AMP] = ACTIONS(4834),
    [anon_sym_PIPE] = ACTIONS(4834),
    [anon_sym_LT] = ACTIONS(4834),
    [anon_sym_DOT_DOT] = ACTIONS(4834),
    [anon_sym_COLON_COLON] = ACTIONS(4834),
    [anon_sym_POUND] = ACTIONS(4834),
    [anon_sym_SQUOTE] = ACTIONS(4836),
    [anon_sym_async] = ACTIONS(4836),
    [anon_sym_auto] = ACTIONS(4836),
    [anon_sym_await] = ACTIONS(4836),
    [anon_sym_become] = ACTIONS(4836),
    [anon_sym_break] = ACTIONS(4836),
    [anon_sym_const] = ACTIONS(4836),
    [anon_sym_continue] = ACTIONS(4836),
    [anon_sym_default] = ACTIONS(4836),
    [anon_sym_enum] = ACTIONS(4836),
    [anon_sym_fn] = ACTIONS(4836),
    [anon_sym_for] = ACTIONS(4836),
    [anon_sym_gen] = ACTIONS(4836),
    [anon_sym_if] = ACTIONS(4836),
    [anon_sym_impl] = ACTIONS(4836),
    [anon_sym_let] = ACTIONS(4836),
    [anon_sym_loop] = ACTIONS(4836),
    [anon_sym_match] = ACTIONS(4836),
    [anon_sym_mod] = ACTIONS(4836),
    [anon_sym_pub] = ACTIONS(4836),
    [anon_sym_return] = ACTIONS(4836),
    [anon_sym_raw] = ACTIONS(4836),
    [anon_sym_static] = ACTIONS(4836),
    [anon_sym_struct] = ACTIONS(4836),
    [anon_sym_trait] = ACTIONS(4836),
    [anon_sym_type] = ACTIONS(4836),
    [anon_sym_union] = ACTIONS(4836),
    [anon_sym_unsafe] = ACTIONS(4836),
    [anon_sym_use] = ACTIONS(4836),
    [anon_sym_while] = ACTIONS(4836),
    [anon_sym_extern] = ACTIONS(4836),
    [anon_sym_safe] = ACTIONS(4836),
    [anon_sym_yield] = ACTIONS(4836),
    [anon_sym_move] = ACTIONS(4836),
    [anon_sym_try] = ACTIONS(4836),
    [sym_integer_literal] = ACTIONS(4834),
    [aux_sym_string_literal_token1] = ACTIONS(4834),
    [sym_char_literal] = ACTIONS(4834),
    [anon_sym_true] = ACTIONS(4836),
    [anon_sym_false] = ACTIONS(4836),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4836),
    [sym_super] = ACTIONS(4836),
    [sym_crate] = ACTIONS(4836),
    [aux_sym_metavariable_token1] = ACTIONS(4834),
    [sym__raw_string_literal_start] = ACTIONS(4834),
    [sym_float_literal] = ACTIONS(4834),
    [sym__outer_block_doc_comment_start] = ACTIONS(4834),
    [sym__outer_line_doc_comment_start] = ACTIONS(4834),
  },
  [STATE(1276)] = {
    [sym_line_comment] = STATE(1276),
    [sym_block_comment] = STATE(1276),
    [ts_builtin_sym_end] = ACTIONS(4838),
    [sym_identifier] = ACTIONS(4840),
    [anon_sym_SEMI] = ACTIONS(4838),
    [anon_sym_macro] = ACTIONS(4840),
    [anon_sym_LPAREN] = ACTIONS(4838),
    [anon_sym_LBRACE] = ACTIONS(4838),
    [anon_sym_RBRACE] = ACTIONS(4838),
    [anon_sym_macro_rules] = ACTIONS(4840),
    [anon_sym_BANG] = ACTIONS(4838),
    [anon_sym_LBRACK] = ACTIONS(4838),
    [anon_sym_STAR] = ACTIONS(4838),
    [anon_sym_u8] = ACTIONS(4840),
    [anon_sym_i8] = ACTIONS(4840),
    [anon_sym_u16] = ACTIONS(4840),
    [anon_sym_i16] = ACTIONS(4840),
    [anon_sym_u32] = ACTIONS(4840),
    [anon_sym_i32] = ACTIONS(4840),
    [anon_sym_u64] = ACTIONS(4840),
    [anon_sym_i64] = ACTIONS(4840),
    [anon_sym_u128] = ACTIONS(4840),
    [anon_sym_i128] = ACTIONS(4840),
    [anon_sym_isize] = ACTIONS(4840),
    [anon_sym_usize] = ACTIONS(4840),
    [anon_sym_f32] = ACTIONS(4840),
    [anon_sym_f64] = ACTIONS(4840),
    [anon_sym_bool] = ACTIONS(4840),
    [anon_sym_str] = ACTIONS(4840),
    [anon_sym_char] = ACTIONS(4840),
    [anon_sym_DASH] = ACTIONS(4838),
    [anon_sym_AMP] = ACTIONS(4838),
    [anon_sym_PIPE] = ACTIONS(4838),
    [anon_sym_LT] = ACTIONS(4838),
    [anon_sym_DOT_DOT] = ACTIONS(4838),
    [anon_sym_COLON_COLON] = ACTIONS(4838),
    [anon_sym_POUND] = ACTIONS(4838),
    [anon_sym_SQUOTE] = ACTIONS(4840),
    [anon_sym_async] = ACTIONS(4840),
    [anon_sym_auto] = ACTIONS(4840),
    [anon_sym_await] = ACTIONS(4840),
    [anon_sym_become] = ACTIONS(4840),
    [anon_sym_break] = ACTIONS(4840),
    [anon_sym_const] = ACTIONS(4840),
    [anon_sym_continue] = ACTIONS(4840),
    [anon_sym_default] = ACTIONS(4840),
    [anon_sym_enum] = ACTIONS(4840),
    [anon_sym_fn] = ACTIONS(4840),
    [anon_sym_for] = ACTIONS(4840),
    [anon_sym_gen] = ACTIONS(4840),
    [anon_sym_if] = ACTIONS(4840),
    [anon_sym_impl] = ACTIONS(4840),
    [anon_sym_let] = ACTIONS(4840),
    [anon_sym_loop] = ACTIONS(4840),
    [anon_sym_match] = ACTIONS(4840),
    [anon_sym_mod] = ACTIONS(4840),
    [anon_sym_pub] = ACTIONS(4840),
    [anon_sym_return] = ACTIONS(4840),
    [anon_sym_raw] = ACTIONS(4840),
    [anon_sym_static] = ACTIONS(4840),
    [anon_sym_struct] = ACTIONS(4840),
    [anon_sym_trait] = ACTIONS(4840),
    [anon_sym_type] = ACTIONS(4840),
    [anon_sym_union] = ACTIONS(4840),
    [anon_sym_unsafe] = ACTIONS(4840),
    [anon_sym_use] = ACTIONS(4840),
    [anon_sym_while] = ACTIONS(4840),
    [anon_sym_extern] = ACTIONS(4840),
    [anon_sym_safe] = ACTIONS(4840),
    [anon_sym_yield] = ACTIONS(4840),
    [anon_sym_move] = ACTIONS(4840),
    [anon_sym_try] = ACTIONS(4840),
    [sym_integer_literal] = ACTIONS(4838),
    [aux_sym_string_literal_token1] = ACTIONS(4838),
    [sym_char_literal] = ACTIONS(4838),
    [anon_sym_true] = ACTIONS(4840),
    [anon_sym_false] = ACTIONS(4840),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4840),
    [sym_super] = ACTIONS(4840),
    [sym_crate] = ACTIONS(4840),
    [aux_sym_metavariable_token1] = ACTIONS(4838),
    [sym__raw_string_literal_start] = ACTIONS(4838),
    [sym_float_literal] = ACTIONS(4838),
    [sym__outer_block_doc_comment_start] = ACTIONS(4838),
    [sym__outer_line_doc_comment_start] = ACTIONS(4838),
  },
  [STATE(1277)] = {
    [sym_line_comment] = STATE(1277),
    [sym_block_comment] = STATE(1277),
    [ts_builtin_sym_end] = ACTIONS(4842),
    [sym_identifier] = ACTIONS(4844),
    [anon_sym_SEMI] = ACTIONS(4842),
    [anon_sym_macro] = ACTIONS(4844),
    [anon_sym_LPAREN] = ACTIONS(4842),
    [anon_sym_LBRACE] = ACTIONS(4842),
    [anon_sym_RBRACE] = ACTIONS(4842),
    [anon_sym_macro_rules] = ACTIONS(4844),
    [anon_sym_BANG] = ACTIONS(4842),
    [anon_sym_LBRACK] = ACTIONS(4842),
    [anon_sym_STAR] = ACTIONS(4842),
    [anon_sym_u8] = ACTIONS(4844),
    [anon_sym_i8] = ACTIONS(4844),
    [anon_sym_u16] = ACTIONS(4844),
    [anon_sym_i16] = ACTIONS(4844),
    [anon_sym_u32] = ACTIONS(4844),
    [anon_sym_i32] = ACTIONS(4844),
    [anon_sym_u64] = ACTIONS(4844),
    [anon_sym_i64] = ACTIONS(4844),
    [anon_sym_u128] = ACTIONS(4844),
    [anon_sym_i128] = ACTIONS(4844),
    [anon_sym_isize] = ACTIONS(4844),
    [anon_sym_usize] = ACTIONS(4844),
    [anon_sym_f32] = ACTIONS(4844),
    [anon_sym_f64] = ACTIONS(4844),
    [anon_sym_bool] = ACTIONS(4844),
    [anon_sym_str] = ACTIONS(4844),
    [anon_sym_char] = ACTIONS(4844),
    [anon_sym_DASH] = ACTIONS(4842),
    [anon_sym_AMP] = ACTIONS(4842),
    [anon_sym_PIPE] = ACTIONS(4842),
    [anon_sym_LT] = ACTIONS(4842),
    [anon_sym_DOT_DOT] = ACTIONS(4842),
    [anon_sym_COLON_COLON] = ACTIONS(4842),
    [anon_sym_POUND] = ACTIONS(4842),
    [anon_sym_SQUOTE] = ACTIONS(4844),
    [anon_sym_async] = ACTIONS(4844),
    [anon_sym_auto] = ACTIONS(4844),
    [anon_sym_await] = ACTIONS(4844),
    [anon_sym_become] = ACTIONS(4844),
    [anon_sym_break] = ACTIONS(4844),
    [anon_sym_const] = ACTIONS(4844),
    [anon_sym_continue] = ACTIONS(4844),
    [anon_sym_default] = ACTIONS(4844),
    [anon_sym_enum] = ACTIONS(4844),
    [anon_sym_fn] = ACTIONS(4844),
    [anon_sym_for] = ACTIONS(4844),
    [anon_sym_gen] = ACTIONS(4844),
    [anon_sym_if] = ACTIONS(4844),
    [anon_sym_impl] = ACTIONS(4844),
    [anon_sym_let] = ACTIONS(4844),
    [anon_sym_loop] = ACTIONS(4844),
    [anon_sym_match] = ACTIONS(4844),
    [anon_sym_mod] = ACTIONS(4844),
    [anon_sym_pub] = ACTIONS(4844),
    [anon_sym_return] = ACTIONS(4844),
    [anon_sym_raw] = ACTIONS(4844),
    [anon_sym_static] = ACTIONS(4844),
    [anon_sym_struct] = ACTIONS(4844),
    [anon_sym_trait] = ACTIONS(4844),
    [anon_sym_type] = ACTIONS(4844),
    [anon_sym_union] = ACTIONS(4844),
    [anon_sym_unsafe] = ACTIONS(4844),
    [anon_sym_use] = ACTIONS(4844),
    [anon_sym_while] = ACTIONS(4844),
    [anon_sym_extern] = ACTIONS(4844),
    [anon_sym_safe] = ACTIONS(4844),
    [anon_sym_yield] = ACTIONS(4844),
    [anon_sym_move] = ACTIONS(4844),
    [anon_sym_try] = ACTIONS(4844),
    [sym_integer_literal] = ACTIONS(4842),
    [aux_sym_string_literal_token1] = ACTIONS(4842),
    [sym_char_literal] = ACTIONS(4842),
    [anon_sym_true] = ACTIONS(4844),
    [anon_sym_false] = ACTIONS(4844),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4844),
    [sym_super] = ACTIONS(4844),
    [sym_crate] = ACTIONS(4844),
    [aux_sym_metavariable_token1] = ACTIONS(4842),
    [sym__raw_string_literal_start] = ACTIONS(4842),
    [sym_float_literal] = ACTIONS(4842),
    [sym__outer_block_doc_comment_start] = ACTIONS(4842),
    [sym__outer_line_doc_comment_start] = ACTIONS(4842),
  },
  [STATE(1278)] = {
    [sym_line_comment] = STATE(1278),
    [sym_block_comment] = STATE(1278),
    [ts_builtin_sym_end] = ACTIONS(4846),
    [sym_identifier] = ACTIONS(4848),
    [anon_sym_SEMI] = ACTIONS(4846),
    [anon_sym_macro] = ACTIONS(4848),
    [anon_sym_LPAREN] = ACTIONS(4846),
    [anon_sym_LBRACE] = ACTIONS(4846),
    [anon_sym_RBRACE] = ACTIONS(4846),
    [anon_sym_macro_rules] = ACTIONS(4848),
    [anon_sym_BANG] = ACTIONS(4846),
    [anon_sym_LBRACK] = ACTIONS(4846),
    [anon_sym_STAR] = ACTIONS(4846),
    [anon_sym_u8] = ACTIONS(4848),
    [anon_sym_i8] = ACTIONS(4848),
    [anon_sym_u16] = ACTIONS(4848),
    [anon_sym_i16] = ACTIONS(4848),
    [anon_sym_u32] = ACTIONS(4848),
    [anon_sym_i32] = ACTIONS(4848),
    [anon_sym_u64] = ACTIONS(4848),
    [anon_sym_i64] = ACTIONS(4848),
    [anon_sym_u128] = ACTIONS(4848),
    [anon_sym_i128] = ACTIONS(4848),
    [anon_sym_isize] = ACTIONS(4848),
    [anon_sym_usize] = ACTIONS(4848),
    [anon_sym_f32] = ACTIONS(4848),
    [anon_sym_f64] = ACTIONS(4848),
    [anon_sym_bool] = ACTIONS(4848),
    [anon_sym_str] = ACTIONS(4848),
    [anon_sym_char] = ACTIONS(4848),
    [anon_sym_DASH] = ACTIONS(4846),
    [anon_sym_AMP] = ACTIONS(4846),
    [anon_sym_PIPE] = ACTIONS(4846),
    [anon_sym_LT] = ACTIONS(4846),
    [anon_sym_DOT_DOT] = ACTIONS(4846),
    [anon_sym_COLON_COLON] = ACTIONS(4846),
    [anon_sym_POUND] = ACTIONS(4846),
    [anon_sym_SQUOTE] = ACTIONS(4848),
    [anon_sym_async] = ACTIONS(4848),
    [anon_sym_auto] = ACTIONS(4848),
    [anon_sym_await] = ACTIONS(4848),
    [anon_sym_become] = ACTIONS(4848),
    [anon_sym_break] = ACTIONS(4848),
    [anon_sym_const] = ACTIONS(4848),
    [anon_sym_continue] = ACTIONS(4848),
    [anon_sym_default] = ACTIONS(4848),
    [anon_sym_enum] = ACTIONS(4848),
    [anon_sym_fn] = ACTIONS(4848),
    [anon_sym_for] = ACTIONS(4848),
    [anon_sym_gen] = ACTIONS(4848),
    [anon_sym_if] = ACTIONS(4848),
    [anon_sym_impl] = ACTIONS(4848),
    [anon_sym_let] = ACTIONS(4848),
    [anon_sym_loop] = ACTIONS(4848),
    [anon_sym_match] = ACTIONS(4848),
    [anon_sym_mod] = ACTIONS(4848),
    [anon_sym_pub] = ACTIONS(4848),
    [anon_sym_return] = ACTIONS(4848),
    [anon_sym_raw] = ACTIONS(4848),
    [anon_sym_static] = ACTIONS(4848),
    [anon_sym_struct] = ACTIONS(4848),
    [anon_sym_trait] = ACTIONS(4848),
    [anon_sym_type] = ACTIONS(4848),
    [anon_sym_union] = ACTIONS(4848),
    [anon_sym_unsafe] = ACTIONS(4848),
    [anon_sym_use] = ACTIONS(4848),
    [anon_sym_while] = ACTIONS(4848),
    [anon_sym_extern] = ACTIONS(4848),
    [anon_sym_safe] = ACTIONS(4848),
    [anon_sym_yield] = ACTIONS(4848),
    [anon_sym_move] = ACTIONS(4848),
    [anon_sym_try] = ACTIONS(4848),
    [sym_integer_literal] = ACTIONS(4846),
    [aux_sym_string_literal_token1] = ACTIONS(4846),
    [sym_char_literal] = ACTIONS(4846),
    [anon_sym_true] = ACTIONS(4848),
    [anon_sym_false] = ACTIONS(4848),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4848),
    [sym_super] = ACTIONS(4848),
    [sym_crate] = ACTIONS(4848),
    [aux_sym_metavariable_token1] = ACTIONS(4846),
    [sym__raw_string_literal_start] = ACTIONS(4846),
    [sym_float_literal] = ACTIONS(4846),
    [sym__outer_block_doc_comment_start] = ACTIONS(4846),
    [sym__outer_line_doc_comment_start] = ACTIONS(4846),
  },
  [STATE(1279)] = {
    [sym_line_comment] = STATE(1279),
    [sym_block_comment] = STATE(1279),
    [ts_builtin_sym_end] = ACTIONS(4850),
    [sym_identifier] = ACTIONS(4852),
    [anon_sym_SEMI] = ACTIONS(4850),
    [anon_sym_macro] = ACTIONS(4852),
    [anon_sym_LPAREN] = ACTIONS(4850),
    [anon_sym_LBRACE] = ACTIONS(4850),
    [anon_sym_RBRACE] = ACTIONS(4850),
    [anon_sym_macro_rules] = ACTIONS(4852),
    [anon_sym_BANG] = ACTIONS(4850),
    [anon_sym_LBRACK] = ACTIONS(4850),
    [anon_sym_STAR] = ACTIONS(4850),
    [anon_sym_u8] = ACTIONS(4852),
    [anon_sym_i8] = ACTIONS(4852),
    [anon_sym_u16] = ACTIONS(4852),
    [anon_sym_i16] = ACTIONS(4852),
    [anon_sym_u32] = ACTIONS(4852),
    [anon_sym_i32] = ACTIONS(4852),
    [anon_sym_u64] = ACTIONS(4852),
    [anon_sym_i64] = ACTIONS(4852),
    [anon_sym_u128] = ACTIONS(4852),
    [anon_sym_i128] = ACTIONS(4852),
    [anon_sym_isize] = ACTIONS(4852),
    [anon_sym_usize] = ACTIONS(4852),
    [anon_sym_f32] = ACTIONS(4852),
    [anon_sym_f64] = ACTIONS(4852),
    [anon_sym_bool] = ACTIONS(4852),
    [anon_sym_str] = ACTIONS(4852),
    [anon_sym_char] = ACTIONS(4852),
    [anon_sym_DASH] = ACTIONS(4850),
    [anon_sym_AMP] = ACTIONS(4850),
    [anon_sym_PIPE] = ACTIONS(4850),
    [anon_sym_LT] = ACTIONS(4850),
    [anon_sym_DOT_DOT] = ACTIONS(4850),
    [anon_sym_COLON_COLON] = ACTIONS(4850),
    [anon_sym_POUND] = ACTIONS(4850),
    [anon_sym_SQUOTE] = ACTIONS(4852),
    [anon_sym_async] = ACTIONS(4852),
    [anon_sym_auto] = ACTIONS(4852),
    [anon_sym_await] = ACTIONS(4852),
    [anon_sym_become] = ACTIONS(4852),
    [anon_sym_break] = ACTIONS(4852),
    [anon_sym_const] = ACTIONS(4852),
    [anon_sym_continue] = ACTIONS(4852),
    [anon_sym_default] = ACTIONS(4852),
    [anon_sym_enum] = ACTIONS(4852),
    [anon_sym_fn] = ACTIONS(4852),
    [anon_sym_for] = ACTIONS(4852),
    [anon_sym_gen] = ACTIONS(4852),
    [anon_sym_if] = ACTIONS(4852),
    [anon_sym_impl] = ACTIONS(4852),
    [anon_sym_let] = ACTIONS(4852),
    [anon_sym_loop] = ACTIONS(4852),
    [anon_sym_match] = ACTIONS(4852),
    [anon_sym_mod] = ACTIONS(4852),
    [anon_sym_pub] = ACTIONS(4852),
    [anon_sym_return] = ACTIONS(4852),
    [anon_sym_raw] = ACTIONS(4852),
    [anon_sym_static] = ACTIONS(4852),
    [anon_sym_struct] = ACTIONS(4852),
    [anon_sym_trait] = ACTIONS(4852),
    [anon_sym_type] = ACTIONS(4852),
    [anon_sym_union] = ACTIONS(4852),
    [anon_sym_unsafe] = ACTIONS(4852),
    [anon_sym_use] = ACTIONS(4852),
    [anon_sym_while] = ACTIONS(4852),
    [anon_sym_extern] = ACTIONS(4852),
    [anon_sym_safe] = ACTIONS(4852),
    [anon_sym_yield] = ACTIONS(4852),
    [anon_sym_move] = ACTIONS(4852),
    [anon_sym_try] = ACTIONS(4852),
    [sym_integer_literal] = ACTIONS(4850),
    [aux_sym_string_literal_token1] = ACTIONS(4850),
    [sym_char_literal] = ACTIONS(4850),
    [anon_sym_true] = ACTIONS(4852),
    [anon_sym_false] = ACTIONS(4852),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4852),
    [sym_super] = ACTIONS(4852),
    [sym_crate] = ACTIONS(4852),
    [aux_sym_metavariable_token1] = ACTIONS(4850),
    [sym__raw_string_literal_start] = ACTIONS(4850),
    [sym_float_literal] = ACTIONS(4850),
    [sym__outer_block_doc_comment_start] = ACTIONS(4850),
    [sym__outer_line_doc_comment_start] = ACTIONS(4850),
  },
  [STATE(1280)] = {
    [sym_line_comment] = STATE(1280),
    [sym_block_comment] = STATE(1280),
    [ts_builtin_sym_end] = ACTIONS(4854),
    [sym_identifier] = ACTIONS(4856),
    [anon_sym_SEMI] = ACTIONS(4854),
    [anon_sym_macro] = ACTIONS(4856),
    [anon_sym_LPAREN] = ACTIONS(4854),
    [anon_sym_LBRACE] = ACTIONS(4854),
    [anon_sym_RBRACE] = ACTIONS(4854),
    [anon_sym_macro_rules] = ACTIONS(4856),
    [anon_sym_BANG] = ACTIONS(4854),
    [anon_sym_LBRACK] = ACTIONS(4854),
    [anon_sym_STAR] = ACTIONS(4854),
    [anon_sym_u8] = ACTIONS(4856),
    [anon_sym_i8] = ACTIONS(4856),
    [anon_sym_u16] = ACTIONS(4856),
    [anon_sym_i16] = ACTIONS(4856),
    [anon_sym_u32] = ACTIONS(4856),
    [anon_sym_i32] = ACTIONS(4856),
    [anon_sym_u64] = ACTIONS(4856),
    [anon_sym_i64] = ACTIONS(4856),
    [anon_sym_u128] = ACTIONS(4856),
    [anon_sym_i128] = ACTIONS(4856),
    [anon_sym_isize] = ACTIONS(4856),
    [anon_sym_usize] = ACTIONS(4856),
    [anon_sym_f32] = ACTIONS(4856),
    [anon_sym_f64] = ACTIONS(4856),
    [anon_sym_bool] = ACTIONS(4856),
    [anon_sym_str] = ACTIONS(4856),
    [anon_sym_char] = ACTIONS(4856),
    [anon_sym_DASH] = ACTIONS(4854),
    [anon_sym_AMP] = ACTIONS(4854),
    [anon_sym_PIPE] = ACTIONS(4854),
    [anon_sym_LT] = ACTIONS(4854),
    [anon_sym_DOT_DOT] = ACTIONS(4854),
    [anon_sym_COLON_COLON] = ACTIONS(4854),
    [anon_sym_POUND] = ACTIONS(4854),
    [anon_sym_SQUOTE] = ACTIONS(4856),
    [anon_sym_async] = ACTIONS(4856),
    [anon_sym_auto] = ACTIONS(4856),
    [anon_sym_await] = ACTIONS(4856),
    [anon_sym_become] = ACTIONS(4856),
    [anon_sym_break] = ACTIONS(4856),
    [anon_sym_const] = ACTIONS(4856),
    [anon_sym_continue] = ACTIONS(4856),
    [anon_sym_default] = ACTIONS(4856),
    [anon_sym_enum] = ACTIONS(4856),
    [anon_sym_fn] = ACTIONS(4856),
    [anon_sym_for] = ACTIONS(4856),
    [anon_sym_gen] = ACTIONS(4856),
    [anon_sym_if] = ACTIONS(4856),
    [anon_sym_impl] = ACTIONS(4856),
    [anon_sym_let] = ACTIONS(4856),
    [anon_sym_loop] = ACTIONS(4856),
    [anon_sym_match] = ACTIONS(4856),
    [anon_sym_mod] = ACTIONS(4856),
    [anon_sym_pub] = ACTIONS(4856),
    [anon_sym_return] = ACTIONS(4856),
    [anon_sym_raw] = ACTIONS(4856),
    [anon_sym_static] = ACTIONS(4856),
    [anon_sym_struct] = ACTIONS(4856),
    [anon_sym_trait] = ACTIONS(4856),
    [anon_sym_type] = ACTIONS(4856),
    [anon_sym_union] = ACTIONS(4856),
    [anon_sym_unsafe] = ACTIONS(4856),
    [anon_sym_use] = ACTIONS(4856),
    [anon_sym_while] = ACTIONS(4856),
    [anon_sym_extern] = ACTIONS(4856),
    [anon_sym_safe] = ACTIONS(4856),
    [anon_sym_yield] = ACTIONS(4856),
    [anon_sym_move] = ACTIONS(4856),
    [anon_sym_try] = ACTIONS(4856),
    [sym_integer_literal] = ACTIONS(4854),
    [aux_sym_string_literal_token1] = ACTIONS(4854),
    [sym_char_literal] = ACTIONS(4854),
    [anon_sym_true] = ACTIONS(4856),
    [anon_sym_false] = ACTIONS(4856),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4856),
    [sym_super] = ACTIONS(4856),
    [sym_crate] = ACTIONS(4856),
    [aux_sym_metavariable_token1] = ACTIONS(4854),
    [sym__raw_string_literal_start] = ACTIONS(4854),
    [sym_float_literal] = ACTIONS(4854),
    [sym__outer_block_doc_comment_start] = ACTIONS(4854),
    [sym__outer_line_doc_comment_start] = ACTIONS(4854),
  },
  [STATE(1281)] = {
    [sym_line_comment] = STATE(1281),
    [sym_block_comment] = STATE(1281),
    [ts_builtin_sym_end] = ACTIONS(4858),
    [sym_identifier] = ACTIONS(4860),
    [anon_sym_SEMI] = ACTIONS(4858),
    [anon_sym_macro] = ACTIONS(4860),
    [anon_sym_LPAREN] = ACTIONS(4858),
    [anon_sym_LBRACE] = ACTIONS(4858),
    [anon_sym_RBRACE] = ACTIONS(4858),
    [anon_sym_macro_rules] = ACTIONS(4860),
    [anon_sym_BANG] = ACTIONS(4858),
    [anon_sym_LBRACK] = ACTIONS(4858),
    [anon_sym_STAR] = ACTIONS(4858),
    [anon_sym_u8] = ACTIONS(4860),
    [anon_sym_i8] = ACTIONS(4860),
    [anon_sym_u16] = ACTIONS(4860),
    [anon_sym_i16] = ACTIONS(4860),
    [anon_sym_u32] = ACTIONS(4860),
    [anon_sym_i32] = ACTIONS(4860),
    [anon_sym_u64] = ACTIONS(4860),
    [anon_sym_i64] = ACTIONS(4860),
    [anon_sym_u128] = ACTIONS(4860),
    [anon_sym_i128] = ACTIONS(4860),
    [anon_sym_isize] = ACTIONS(4860),
    [anon_sym_usize] = ACTIONS(4860),
    [anon_sym_f32] = ACTIONS(4860),
    [anon_sym_f64] = ACTIONS(4860),
    [anon_sym_bool] = ACTIONS(4860),
    [anon_sym_str] = ACTIONS(4860),
    [anon_sym_char] = ACTIONS(4860),
    [anon_sym_DASH] = ACTIONS(4858),
    [anon_sym_AMP] = ACTIONS(4858),
    [anon_sym_PIPE] = ACTIONS(4858),
    [anon_sym_LT] = ACTIONS(4858),
    [anon_sym_DOT_DOT] = ACTIONS(4858),
    [anon_sym_COLON_COLON] = ACTIONS(4858),
    [anon_sym_POUND] = ACTIONS(4858),
    [anon_sym_SQUOTE] = ACTIONS(4860),
    [anon_sym_async] = ACTIONS(4860),
    [anon_sym_auto] = ACTIONS(4860),
    [anon_sym_await] = ACTIONS(4860),
    [anon_sym_become] = ACTIONS(4860),
    [anon_sym_break] = ACTIONS(4860),
    [anon_sym_const] = ACTIONS(4860),
    [anon_sym_continue] = ACTIONS(4860),
    [anon_sym_default] = ACTIONS(4860),
    [anon_sym_enum] = ACTIONS(4860),
    [anon_sym_fn] = ACTIONS(4860),
    [anon_sym_for] = ACTIONS(4860),
    [anon_sym_gen] = ACTIONS(4860),
    [anon_sym_if] = ACTIONS(4860),
    [anon_sym_impl] = ACTIONS(4860),
    [anon_sym_let] = ACTIONS(4860),
    [anon_sym_loop] = ACTIONS(4860),
    [anon_sym_match] = ACTIONS(4860),
    [anon_sym_mod] = ACTIONS(4860),
    [anon_sym_pub] = ACTIONS(4860),
    [anon_sym_return] = ACTIONS(4860),
    [anon_sym_raw] = ACTIONS(4860),
    [anon_sym_static] = ACTIONS(4860),
    [anon_sym_struct] = ACTIONS(4860),
    [anon_sym_trait] = ACTIONS(4860),
    [anon_sym_type] = ACTIONS(4860),
    [anon_sym_union] = ACTIONS(4860),
    [anon_sym_unsafe] = ACTIONS(4860),
    [anon_sym_use] = ACTIONS(4860),
    [anon_sym_while] = ACTIONS(4860),
    [anon_sym_extern] = ACTIONS(4860),
    [anon_sym_safe] = ACTIONS(4860),
    [anon_sym_yield] = ACTIONS(4860),
    [anon_sym_move] = ACTIONS(4860),
    [anon_sym_try] = ACTIONS(4860),
    [sym_integer_literal] = ACTIONS(4858),
    [aux_sym_string_literal_token1] = ACTIONS(4858),
    [sym_char_literal] = ACTIONS(4858),
    [anon_sym_true] = ACTIONS(4860),
    [anon_sym_false] = ACTIONS(4860),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4860),
    [sym_super] = ACTIONS(4860),
    [sym_crate] = ACTIONS(4860),
    [aux_sym_metavariable_token1] = ACTIONS(4858),
    [sym__raw_string_literal_start] = ACTIONS(4858),
    [sym_float_literal] = ACTIONS(4858),
    [sym__outer_block_doc_comment_start] = ACTIONS(4858),
    [sym__outer_line_doc_comment_start] = ACTIONS(4858),
  },
  [STATE(1282)] = {
    [sym_line_comment] = STATE(1282),
    [sym_block_comment] = STATE(1282),
    [ts_builtin_sym_end] = ACTIONS(4862),
    [sym_identifier] = ACTIONS(4864),
    [anon_sym_SEMI] = ACTIONS(4862),
    [anon_sym_macro] = ACTIONS(4864),
    [anon_sym_LPAREN] = ACTIONS(4862),
    [anon_sym_LBRACE] = ACTIONS(4862),
    [anon_sym_RBRACE] = ACTIONS(4862),
    [anon_sym_macro_rules] = ACTIONS(4864),
    [anon_sym_BANG] = ACTIONS(4862),
    [anon_sym_LBRACK] = ACTIONS(4862),
    [anon_sym_STAR] = ACTIONS(4862),
    [anon_sym_u8] = ACTIONS(4864),
    [anon_sym_i8] = ACTIONS(4864),
    [anon_sym_u16] = ACTIONS(4864),
    [anon_sym_i16] = ACTIONS(4864),
    [anon_sym_u32] = ACTIONS(4864),
    [anon_sym_i32] = ACTIONS(4864),
    [anon_sym_u64] = ACTIONS(4864),
    [anon_sym_i64] = ACTIONS(4864),
    [anon_sym_u128] = ACTIONS(4864),
    [anon_sym_i128] = ACTIONS(4864),
    [anon_sym_isize] = ACTIONS(4864),
    [anon_sym_usize] = ACTIONS(4864),
    [anon_sym_f32] = ACTIONS(4864),
    [anon_sym_f64] = ACTIONS(4864),
    [anon_sym_bool] = ACTIONS(4864),
    [anon_sym_str] = ACTIONS(4864),
    [anon_sym_char] = ACTIONS(4864),
    [anon_sym_DASH] = ACTIONS(4862),
    [anon_sym_AMP] = ACTIONS(4862),
    [anon_sym_PIPE] = ACTIONS(4862),
    [anon_sym_LT] = ACTIONS(4862),
    [anon_sym_DOT_DOT] = ACTIONS(4862),
    [anon_sym_COLON_COLON] = ACTIONS(4862),
    [anon_sym_POUND] = ACTIONS(4862),
    [anon_sym_SQUOTE] = ACTIONS(4864),
    [anon_sym_async] = ACTIONS(4864),
    [anon_sym_auto] = ACTIONS(4864),
    [anon_sym_await] = ACTIONS(4864),
    [anon_sym_become] = ACTIONS(4864),
    [anon_sym_break] = ACTIONS(4864),
    [anon_sym_const] = ACTIONS(4864),
    [anon_sym_continue] = ACTIONS(4864),
    [anon_sym_default] = ACTIONS(4864),
    [anon_sym_enum] = ACTIONS(4864),
    [anon_sym_fn] = ACTIONS(4864),
    [anon_sym_for] = ACTIONS(4864),
    [anon_sym_gen] = ACTIONS(4864),
    [anon_sym_if] = ACTIONS(4864),
    [anon_sym_impl] = ACTIONS(4864),
    [anon_sym_let] = ACTIONS(4864),
    [anon_sym_loop] = ACTIONS(4864),
    [anon_sym_match] = ACTIONS(4864),
    [anon_sym_mod] = ACTIONS(4864),
    [anon_sym_pub] = ACTIONS(4864),
    [anon_sym_return] = ACTIONS(4864),
    [anon_sym_raw] = ACTIONS(4864),
    [anon_sym_static] = ACTIONS(4864),
    [anon_sym_struct] = ACTIONS(4864),
    [anon_sym_trait] = ACTIONS(4864),
    [anon_sym_type] = ACTIONS(4864),
    [anon_sym_union] = ACTIONS(4864),
    [anon_sym_unsafe] = ACTIONS(4864),
    [anon_sym_use] = ACTIONS(4864),
    [anon_sym_while] = ACTIONS(4864),
    [anon_sym_extern] = ACTIONS(4864),
    [anon_sym_safe] = ACTIONS(4864),
    [anon_sym_yield] = ACTIONS(4864),
    [anon_sym_move] = ACTIONS(4864),
    [anon_sym_try] = ACTIONS(4864),
    [sym_integer_literal] = ACTIONS(4862),
    [aux_sym_string_literal_token1] = ACTIONS(4862),
    [sym_char_literal] = ACTIONS(4862),
    [anon_sym_true] = ACTIONS(4864),
    [anon_sym_false] = ACTIONS(4864),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4864),
    [sym_super] = ACTIONS(4864),
    [sym_crate] = ACTIONS(4864),
    [aux_sym_metavariable_token1] = ACTIONS(4862),
    [sym__raw_string_literal_start] = ACTIONS(4862),
    [sym_float_literal] = ACTIONS(4862),
    [sym__outer_block_doc_comment_start] = ACTIONS(4862),
    [sym__outer_line_doc_comment_start] = ACTIONS(4862),
  },
  [STATE(1283)] = {
    [sym_line_comment] = STATE(1283),
    [sym_block_comment] = STATE(1283),
    [ts_builtin_sym_end] = ACTIONS(4866),
    [sym_identifier] = ACTIONS(4868),
    [anon_sym_SEMI] = ACTIONS(4866),
    [anon_sym_macro] = ACTIONS(4868),
    [anon_sym_LPAREN] = ACTIONS(4866),
    [anon_sym_LBRACE] = ACTIONS(4866),
    [anon_sym_RBRACE] = ACTIONS(4866),
    [anon_sym_macro_rules] = ACTIONS(4868),
    [anon_sym_BANG] = ACTIONS(4866),
    [anon_sym_LBRACK] = ACTIONS(4866),
    [anon_sym_STAR] = ACTIONS(4866),
    [anon_sym_u8] = ACTIONS(4868),
    [anon_sym_i8] = ACTIONS(4868),
    [anon_sym_u16] = ACTIONS(4868),
    [anon_sym_i16] = ACTIONS(4868),
    [anon_sym_u32] = ACTIONS(4868),
    [anon_sym_i32] = ACTIONS(4868),
    [anon_sym_u64] = ACTIONS(4868),
    [anon_sym_i64] = ACTIONS(4868),
    [anon_sym_u128] = ACTIONS(4868),
    [anon_sym_i128] = ACTIONS(4868),
    [anon_sym_isize] = ACTIONS(4868),
    [anon_sym_usize] = ACTIONS(4868),
    [anon_sym_f32] = ACTIONS(4868),
    [anon_sym_f64] = ACTIONS(4868),
    [anon_sym_bool] = ACTIONS(4868),
    [anon_sym_str] = ACTIONS(4868),
    [anon_sym_char] = ACTIONS(4868),
    [anon_sym_DASH] = ACTIONS(4866),
    [anon_sym_AMP] = ACTIONS(4866),
    [anon_sym_PIPE] = ACTIONS(4866),
    [anon_sym_LT] = ACTIONS(4866),
    [anon_sym_DOT_DOT] = ACTIONS(4866),
    [anon_sym_COLON_COLON] = ACTIONS(4866),
    [anon_sym_POUND] = ACTIONS(4866),
    [anon_sym_SQUOTE] = ACTIONS(4868),
    [anon_sym_async] = ACTIONS(4868),
    [anon_sym_auto] = ACTIONS(4868),
    [anon_sym_await] = ACTIONS(4868),
    [anon_sym_become] = ACTIONS(4868),
    [anon_sym_break] = ACTIONS(4868),
    [anon_sym_const] = ACTIONS(4868),
    [anon_sym_continue] = ACTIONS(4868),
    [anon_sym_default] = ACTIONS(4868),
    [anon_sym_enum] = ACTIONS(4868),
    [anon_sym_fn] = ACTIONS(4868),
    [anon_sym_for] = ACTIONS(4868),
    [anon_sym_gen] = ACTIONS(4868),
    [anon_sym_if] = ACTIONS(4868),
    [anon_sym_impl] = ACTIONS(4868),
    [anon_sym_let] = ACTIONS(4868),
    [anon_sym_loop] = ACTIONS(4868),
    [anon_sym_match] = ACTIONS(4868),
    [anon_sym_mod] = ACTIONS(4868),
    [anon_sym_pub] = ACTIONS(4868),
    [anon_sym_return] = ACTIONS(4868),
    [anon_sym_raw] = ACTIONS(4868),
    [anon_sym_static] = ACTIONS(4868),
    [anon_sym_struct] = ACTIONS(4868),
    [anon_sym_trait] = ACTIONS(4868),
    [anon_sym_type] = ACTIONS(4868),
    [anon_sym_union] = ACTIONS(4868),
    [anon_sym_unsafe] = ACTIONS(4868),
    [anon_sym_use] = ACTIONS(4868),
    [anon_sym_while] = ACTIONS(4868),
    [anon_sym_extern] = ACTIONS(4868),
    [anon_sym_safe] = ACTIONS(4868),
    [anon_sym_yield] = ACTIONS(4868),
    [anon_sym_move] = ACTIONS(4868),
    [anon_sym_try] = ACTIONS(4868),
    [sym_integer_literal] = ACTIONS(4866),
    [aux_sym_string_literal_token1] = ACTIONS(4866),
    [sym_char_literal] = ACTIONS(4866),
    [anon_sym_true] = ACTIONS(4868),
    [anon_sym_false] = ACTIONS(4868),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4868),
    [sym_super] = ACTIONS(4868),
    [sym_crate] = ACTIONS(4868),
    [aux_sym_metavariable_token1] = ACTIONS(4866),
    [sym__raw_string_literal_start] = ACTIONS(4866),
    [sym_float_literal] = ACTIONS(4866),
    [sym__outer_block_doc_comment_start] = ACTIONS(4866),
    [sym__outer_line_doc_comment_start] = ACTIONS(4866),
  },
  [STATE(1284)] = {
    [sym_line_comment] = STATE(1284),
    [sym_block_comment] = STATE(1284),
    [ts_builtin_sym_end] = ACTIONS(4870),
    [sym_identifier] = ACTIONS(4872),
    [anon_sym_SEMI] = ACTIONS(4870),
    [anon_sym_macro] = ACTIONS(4872),
    [anon_sym_LPAREN] = ACTIONS(4870),
    [anon_sym_LBRACE] = ACTIONS(4870),
    [anon_sym_RBRACE] = ACTIONS(4870),
    [anon_sym_macro_rules] = ACTIONS(4872),
    [anon_sym_BANG] = ACTIONS(4870),
    [anon_sym_LBRACK] = ACTIONS(4870),
    [anon_sym_STAR] = ACTIONS(4870),
    [anon_sym_u8] = ACTIONS(4872),
    [anon_sym_i8] = ACTIONS(4872),
    [anon_sym_u16] = ACTIONS(4872),
    [anon_sym_i16] = ACTIONS(4872),
    [anon_sym_u32] = ACTIONS(4872),
    [anon_sym_i32] = ACTIONS(4872),
    [anon_sym_u64] = ACTIONS(4872),
    [anon_sym_i64] = ACTIONS(4872),
    [anon_sym_u128] = ACTIONS(4872),
    [anon_sym_i128] = ACTIONS(4872),
    [anon_sym_isize] = ACTIONS(4872),
    [anon_sym_usize] = ACTIONS(4872),
    [anon_sym_f32] = ACTIONS(4872),
    [anon_sym_f64] = ACTIONS(4872),
    [anon_sym_bool] = ACTIONS(4872),
    [anon_sym_str] = ACTIONS(4872),
    [anon_sym_char] = ACTIONS(4872),
    [anon_sym_DASH] = ACTIONS(4870),
    [anon_sym_AMP] = ACTIONS(4870),
    [anon_sym_PIPE] = ACTIONS(4870),
    [anon_sym_LT] = ACTIONS(4870),
    [anon_sym_DOT_DOT] = ACTIONS(4870),
    [anon_sym_COLON_COLON] = ACTIONS(4870),
    [anon_sym_POUND] = ACTIONS(4870),
    [anon_sym_SQUOTE] = ACTIONS(4872),
    [anon_sym_async] = ACTIONS(4872),
    [anon_sym_auto] = ACTIONS(4872),
    [anon_sym_await] = ACTIONS(4872),
    [anon_sym_become] = ACTIONS(4872),
    [anon_sym_break] = ACTIONS(4872),
    [anon_sym_const] = ACTIONS(4872),
    [anon_sym_continue] = ACTIONS(4872),
    [anon_sym_default] = ACTIONS(4872),
    [anon_sym_enum] = ACTIONS(4872),
    [anon_sym_fn] = ACTIONS(4872),
    [anon_sym_for] = ACTIONS(4872),
    [anon_sym_gen] = ACTIONS(4872),
    [anon_sym_if] = ACTIONS(4872),
    [anon_sym_impl] = ACTIONS(4872),
    [anon_sym_let] = ACTIONS(4872),
    [anon_sym_loop] = ACTIONS(4872),
    [anon_sym_match] = ACTIONS(4872),
    [anon_sym_mod] = ACTIONS(4872),
    [anon_sym_pub] = ACTIONS(4872),
    [anon_sym_return] = ACTIONS(4872),
    [anon_sym_raw] = ACTIONS(4872),
    [anon_sym_static] = ACTIONS(4872),
    [anon_sym_struct] = ACTIONS(4872),
    [anon_sym_trait] = ACTIONS(4872),
    [anon_sym_type] = ACTIONS(4872),
    [anon_sym_union] = ACTIONS(4872),
    [anon_sym_unsafe] = ACTIONS(4872),
    [anon_sym_use] = ACTIONS(4872),
    [anon_sym_while] = ACTIONS(4872),
    [anon_sym_extern] = ACTIONS(4872),
    [anon_sym_safe] = ACTIONS(4872),
    [anon_sym_yield] = ACTIONS(4872),
    [anon_sym_move] = ACTIONS(4872),
    [anon_sym_try] = ACTIONS(4872),
    [sym_integer_literal] = ACTIONS(4870),
    [aux_sym_string_literal_token1] = ACTIONS(4870),
    [sym_char_literal] = ACTIONS(4870),
    [anon_sym_true] = ACTIONS(4872),
    [anon_sym_false] = ACTIONS(4872),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4872),
    [sym_super] = ACTIONS(4872),
    [sym_crate] = ACTIONS(4872),
    [aux_sym_metavariable_token1] = ACTIONS(4870),
    [sym__raw_string_literal_start] = ACTIONS(4870),
    [sym_float_literal] = ACTIONS(4870),
    [sym__outer_block_doc_comment_start] = ACTIONS(4870),
    [sym__outer_line_doc_comment_start] = ACTIONS(4870),
  },
  [STATE(1285)] = {
    [sym_line_comment] = STATE(1285),
    [sym_block_comment] = STATE(1285),
    [ts_builtin_sym_end] = ACTIONS(4874),
    [sym_identifier] = ACTIONS(4876),
    [anon_sym_SEMI] = ACTIONS(4874),
    [anon_sym_macro] = ACTIONS(4876),
    [anon_sym_LPAREN] = ACTIONS(4874),
    [anon_sym_LBRACE] = ACTIONS(4874),
    [anon_sym_RBRACE] = ACTIONS(4874),
    [anon_sym_macro_rules] = ACTIONS(4876),
    [anon_sym_BANG] = ACTIONS(4874),
    [anon_sym_LBRACK] = ACTIONS(4874),
    [anon_sym_STAR] = ACTIONS(4874),
    [anon_sym_u8] = ACTIONS(4876),
    [anon_sym_i8] = ACTIONS(4876),
    [anon_sym_u16] = ACTIONS(4876),
    [anon_sym_i16] = ACTIONS(4876),
    [anon_sym_u32] = ACTIONS(4876),
    [anon_sym_i32] = ACTIONS(4876),
    [anon_sym_u64] = ACTIONS(4876),
    [anon_sym_i64] = ACTIONS(4876),
    [anon_sym_u128] = ACTIONS(4876),
    [anon_sym_i128] = ACTIONS(4876),
    [anon_sym_isize] = ACTIONS(4876),
    [anon_sym_usize] = ACTIONS(4876),
    [anon_sym_f32] = ACTIONS(4876),
    [anon_sym_f64] = ACTIONS(4876),
    [anon_sym_bool] = ACTIONS(4876),
    [anon_sym_str] = ACTIONS(4876),
    [anon_sym_char] = ACTIONS(4876),
    [anon_sym_DASH] = ACTIONS(4874),
    [anon_sym_AMP] = ACTIONS(4874),
    [anon_sym_PIPE] = ACTIONS(4874),
    [anon_sym_LT] = ACTIONS(4874),
    [anon_sym_DOT_DOT] = ACTIONS(4874),
    [anon_sym_COLON_COLON] = ACTIONS(4874),
    [anon_sym_POUND] = ACTIONS(4874),
    [anon_sym_SQUOTE] = ACTIONS(4876),
    [anon_sym_async] = ACTIONS(4876),
    [anon_sym_auto] = ACTIONS(4876),
    [anon_sym_await] = ACTIONS(4876),
    [anon_sym_become] = ACTIONS(4876),
    [anon_sym_break] = ACTIONS(4876),
    [anon_sym_const] = ACTIONS(4876),
    [anon_sym_continue] = ACTIONS(4876),
    [anon_sym_default] = ACTIONS(4876),
    [anon_sym_enum] = ACTIONS(4876),
    [anon_sym_fn] = ACTIONS(4876),
    [anon_sym_for] = ACTIONS(4876),
    [anon_sym_gen] = ACTIONS(4876),
    [anon_sym_if] = ACTIONS(4876),
    [anon_sym_impl] = ACTIONS(4876),
    [anon_sym_let] = ACTIONS(4876),
    [anon_sym_loop] = ACTIONS(4876),
    [anon_sym_match] = ACTIONS(4876),
    [anon_sym_mod] = ACTIONS(4876),
    [anon_sym_pub] = ACTIONS(4876),
    [anon_sym_return] = ACTIONS(4876),
    [anon_sym_raw] = ACTIONS(4876),
    [anon_sym_static] = ACTIONS(4876),
    [anon_sym_struct] = ACTIONS(4876),
    [anon_sym_trait] = ACTIONS(4876),
    [anon_sym_type] = ACTIONS(4876),
    [anon_sym_union] = ACTIONS(4876),
    [anon_sym_unsafe] = ACTIONS(4876),
    [anon_sym_use] = ACTIONS(4876),
    [anon_sym_while] = ACTIONS(4876),
    [anon_sym_extern] = ACTIONS(4876),
    [anon_sym_safe] = ACTIONS(4876),
    [anon_sym_yield] = ACTIONS(4876),
    [anon_sym_move] = ACTIONS(4876),
    [anon_sym_try] = ACTIONS(4876),
    [sym_integer_literal] = ACTIONS(4874),
    [aux_sym_string_literal_token1] = ACTIONS(4874),
    [sym_char_literal] = ACTIONS(4874),
    [anon_sym_true] = ACTIONS(4876),
    [anon_sym_false] = ACTIONS(4876),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4876),
    [sym_super] = ACTIONS(4876),
    [sym_crate] = ACTIONS(4876),
    [aux_sym_metavariable_token1] = ACTIONS(4874),
    [sym__raw_string_literal_start] = ACTIONS(4874),
    [sym_float_literal] = ACTIONS(4874),
    [sym__outer_block_doc_comment_start] = ACTIONS(4874),
    [sym__outer_line_doc_comment_start] = ACTIONS(4874),
  },
  [STATE(1286)] = {
    [sym_line_comment] = STATE(1286),
    [sym_block_comment] = STATE(1286),
    [ts_builtin_sym_end] = ACTIONS(4878),
    [sym_identifier] = ACTIONS(4880),
    [anon_sym_SEMI] = ACTIONS(4878),
    [anon_sym_macro] = ACTIONS(4880),
    [anon_sym_LPAREN] = ACTIONS(4878),
    [anon_sym_LBRACE] = ACTIONS(4878),
    [anon_sym_RBRACE] = ACTIONS(4878),
    [anon_sym_macro_rules] = ACTIONS(4880),
    [anon_sym_BANG] = ACTIONS(4878),
    [anon_sym_LBRACK] = ACTIONS(4878),
    [anon_sym_STAR] = ACTIONS(4878),
    [anon_sym_u8] = ACTIONS(4880),
    [anon_sym_i8] = ACTIONS(4880),
    [anon_sym_u16] = ACTIONS(4880),
    [anon_sym_i16] = ACTIONS(4880),
    [anon_sym_u32] = ACTIONS(4880),
    [anon_sym_i32] = ACTIONS(4880),
    [anon_sym_u64] = ACTIONS(4880),
    [anon_sym_i64] = ACTIONS(4880),
    [anon_sym_u128] = ACTIONS(4880),
    [anon_sym_i128] = ACTIONS(4880),
    [anon_sym_isize] = ACTIONS(4880),
    [anon_sym_usize] = ACTIONS(4880),
    [anon_sym_f32] = ACTIONS(4880),
    [anon_sym_f64] = ACTIONS(4880),
    [anon_sym_bool] = ACTIONS(4880),
    [anon_sym_str] = ACTIONS(4880),
    [anon_sym_char] = ACTIONS(4880),
    [anon_sym_DASH] = ACTIONS(4878),
    [anon_sym_AMP] = ACTIONS(4878),
    [anon_sym_PIPE] = ACTIONS(4878),
    [anon_sym_LT] = ACTIONS(4878),
    [anon_sym_DOT_DOT] = ACTIONS(4878),
    [anon_sym_COLON_COLON] = ACTIONS(4878),
    [anon_sym_POUND] = ACTIONS(4878),
    [anon_sym_SQUOTE] = ACTIONS(4880),
    [anon_sym_async] = ACTIONS(4880),
    [anon_sym_auto] = ACTIONS(4880),
    [anon_sym_await] = ACTIONS(4880),
    [anon_sym_become] = ACTIONS(4880),
    [anon_sym_break] = ACTIONS(4880),
    [anon_sym_const] = ACTIONS(4880),
    [anon_sym_continue] = ACTIONS(4880),
    [anon_sym_default] = ACTIONS(4880),
    [anon_sym_enum] = ACTIONS(4880),
    [anon_sym_fn] = ACTIONS(4880),
    [anon_sym_for] = ACTIONS(4880),
    [anon_sym_gen] = ACTIONS(4880),
    [anon_sym_if] = ACTIONS(4880),
    [anon_sym_impl] = ACTIONS(4880),
    [anon_sym_let] = ACTIONS(4880),
    [anon_sym_loop] = ACTIONS(4880),
    [anon_sym_match] = ACTIONS(4880),
    [anon_sym_mod] = ACTIONS(4880),
    [anon_sym_pub] = ACTIONS(4880),
    [anon_sym_return] = ACTIONS(4880),
    [anon_sym_raw] = ACTIONS(4880),
    [anon_sym_static] = ACTIONS(4880),
    [anon_sym_struct] = ACTIONS(4880),
    [anon_sym_trait] = ACTIONS(4880),
    [anon_sym_type] = ACTIONS(4880),
    [anon_sym_union] = ACTIONS(4880),
    [anon_sym_unsafe] = ACTIONS(4880),
    [anon_sym_use] = ACTIONS(4880),
    [anon_sym_while] = ACTIONS(4880),
    [anon_sym_extern] = ACTIONS(4880),
    [anon_sym_safe] = ACTIONS(4880),
    [anon_sym_yield] = ACTIONS(4880),
    [anon_sym_move] = ACTIONS(4880),
    [anon_sym_try] = ACTIONS(4880),
    [sym_integer_literal] = ACTIONS(4878),
    [aux_sym_string_literal_token1] = ACTIONS(4878),
    [sym_char_literal] = ACTIONS(4878),
    [anon_sym_true] = ACTIONS(4880),
    [anon_sym_false] = ACTIONS(4880),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4880),
    [sym_super] = ACTIONS(4880),
    [sym_crate] = ACTIONS(4880),
    [aux_sym_metavariable_token1] = ACTIONS(4878),
    [sym__raw_string_literal_start] = ACTIONS(4878),
    [sym_float_literal] = ACTIONS(4878),
    [sym__outer_block_doc_comment_start] = ACTIONS(4878),
    [sym__outer_line_doc_comment_start] = ACTIONS(4878),
  },
  [STATE(1287)] = {
    [sym_line_comment] = STATE(1287),
    [sym_block_comment] = STATE(1287),
    [ts_builtin_sym_end] = ACTIONS(4882),
    [sym_identifier] = ACTIONS(4884),
    [anon_sym_SEMI] = ACTIONS(4882),
    [anon_sym_macro] = ACTIONS(4884),
    [anon_sym_LPAREN] = ACTIONS(4882),
    [anon_sym_LBRACE] = ACTIONS(4882),
    [anon_sym_RBRACE] = ACTIONS(4882),
    [anon_sym_macro_rules] = ACTIONS(4884),
    [anon_sym_BANG] = ACTIONS(4882),
    [anon_sym_LBRACK] = ACTIONS(4882),
    [anon_sym_STAR] = ACTIONS(4882),
    [anon_sym_u8] = ACTIONS(4884),
    [anon_sym_i8] = ACTIONS(4884),
    [anon_sym_u16] = ACTIONS(4884),
    [anon_sym_i16] = ACTIONS(4884),
    [anon_sym_u32] = ACTIONS(4884),
    [anon_sym_i32] = ACTIONS(4884),
    [anon_sym_u64] = ACTIONS(4884),
    [anon_sym_i64] = ACTIONS(4884),
    [anon_sym_u128] = ACTIONS(4884),
    [anon_sym_i128] = ACTIONS(4884),
    [anon_sym_isize] = ACTIONS(4884),
    [anon_sym_usize] = ACTIONS(4884),
    [anon_sym_f32] = ACTIONS(4884),
    [anon_sym_f64] = ACTIONS(4884),
    [anon_sym_bool] = ACTIONS(4884),
    [anon_sym_str] = ACTIONS(4884),
    [anon_sym_char] = ACTIONS(4884),
    [anon_sym_DASH] = ACTIONS(4882),
    [anon_sym_AMP] = ACTIONS(4882),
    [anon_sym_PIPE] = ACTIONS(4882),
    [anon_sym_LT] = ACTIONS(4882),
    [anon_sym_DOT_DOT] = ACTIONS(4882),
    [anon_sym_COLON_COLON] = ACTIONS(4882),
    [anon_sym_POUND] = ACTIONS(4882),
    [anon_sym_SQUOTE] = ACTIONS(4884),
    [anon_sym_async] = ACTIONS(4884),
    [anon_sym_auto] = ACTIONS(4884),
    [anon_sym_await] = ACTIONS(4884),
    [anon_sym_become] = ACTIONS(4884),
    [anon_sym_break] = ACTIONS(4884),
    [anon_sym_const] = ACTIONS(4884),
    [anon_sym_continue] = ACTIONS(4884),
    [anon_sym_default] = ACTIONS(4884),
    [anon_sym_enum] = ACTIONS(4884),
    [anon_sym_fn] = ACTIONS(4884),
    [anon_sym_for] = ACTIONS(4884),
    [anon_sym_gen] = ACTIONS(4884),
    [anon_sym_if] = ACTIONS(4884),
    [anon_sym_impl] = ACTIONS(4884),
    [anon_sym_let] = ACTIONS(4884),
    [anon_sym_loop] = ACTIONS(4884),
    [anon_sym_match] = ACTIONS(4884),
    [anon_sym_mod] = ACTIONS(4884),
    [anon_sym_pub] = ACTIONS(4884),
    [anon_sym_return] = ACTIONS(4884),
    [anon_sym_raw] = ACTIONS(4884),
    [anon_sym_static] = ACTIONS(4884),
    [anon_sym_struct] = ACTIONS(4884),
    [anon_sym_trait] = ACTIONS(4884),
    [anon_sym_type] = ACTIONS(4884),
    [anon_sym_union] = ACTIONS(4884),
    [anon_sym_unsafe] = ACTIONS(4884),
    [anon_sym_use] = ACTIONS(4884),
    [anon_sym_while] = ACTIONS(4884),
    [anon_sym_extern] = ACTIONS(4884),
    [anon_sym_safe] = ACTIONS(4884),
    [anon_sym_yield] = ACTIONS(4884),
    [anon_sym_move] = ACTIONS(4884),
    [anon_sym_try] = ACTIONS(4884),
    [sym_integer_literal] = ACTIONS(4882),
    [aux_sym_string_literal_token1] = ACTIONS(4882),
    [sym_char_literal] = ACTIONS(4882),
    [anon_sym_true] = ACTIONS(4884),
    [anon_sym_false] = ACTIONS(4884),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4884),
    [sym_super] = ACTIONS(4884),
    [sym_crate] = ACTIONS(4884),
    [aux_sym_metavariable_token1] = ACTIONS(4882),
    [sym__raw_string_literal_start] = ACTIONS(4882),
    [sym_float_literal] = ACTIONS(4882),
    [sym__outer_block_doc_comment_start] = ACTIONS(4882),
    [sym__outer_line_doc_comment_start] = ACTIONS(4882),
  },
  [STATE(1288)] = {
    [sym_line_comment] = STATE(1288),
    [sym_block_comment] = STATE(1288),
    [ts_builtin_sym_end] = ACTIONS(4886),
    [sym_identifier] = ACTIONS(4888),
    [anon_sym_SEMI] = ACTIONS(4886),
    [anon_sym_macro] = ACTIONS(4888),
    [anon_sym_LPAREN] = ACTIONS(4886),
    [anon_sym_LBRACE] = ACTIONS(4886),
    [anon_sym_RBRACE] = ACTIONS(4886),
    [anon_sym_macro_rules] = ACTIONS(4888),
    [anon_sym_BANG] = ACTIONS(4886),
    [anon_sym_LBRACK] = ACTIONS(4886),
    [anon_sym_STAR] = ACTIONS(4886),
    [anon_sym_u8] = ACTIONS(4888),
    [anon_sym_i8] = ACTIONS(4888),
    [anon_sym_u16] = ACTIONS(4888),
    [anon_sym_i16] = ACTIONS(4888),
    [anon_sym_u32] = ACTIONS(4888),
    [anon_sym_i32] = ACTIONS(4888),
    [anon_sym_u64] = ACTIONS(4888),
    [anon_sym_i64] = ACTIONS(4888),
    [anon_sym_u128] = ACTIONS(4888),
    [anon_sym_i128] = ACTIONS(4888),
    [anon_sym_isize] = ACTIONS(4888),
    [anon_sym_usize] = ACTIONS(4888),
    [anon_sym_f32] = ACTIONS(4888),
    [anon_sym_f64] = ACTIONS(4888),
    [anon_sym_bool] = ACTIONS(4888),
    [anon_sym_str] = ACTIONS(4888),
    [anon_sym_char] = ACTIONS(4888),
    [anon_sym_DASH] = ACTIONS(4886),
    [anon_sym_AMP] = ACTIONS(4886),
    [anon_sym_PIPE] = ACTIONS(4886),
    [anon_sym_LT] = ACTIONS(4886),
    [anon_sym_DOT_DOT] = ACTIONS(4886),
    [anon_sym_COLON_COLON] = ACTIONS(4886),
    [anon_sym_POUND] = ACTIONS(4886),
    [anon_sym_SQUOTE] = ACTIONS(4888),
    [anon_sym_async] = ACTIONS(4888),
    [anon_sym_auto] = ACTIONS(4888),
    [anon_sym_await] = ACTIONS(4888),
    [anon_sym_become] = ACTIONS(4888),
    [anon_sym_break] = ACTIONS(4888),
    [anon_sym_const] = ACTIONS(4888),
    [anon_sym_continue] = ACTIONS(4888),
    [anon_sym_default] = ACTIONS(4888),
    [anon_sym_enum] = ACTIONS(4888),
    [anon_sym_fn] = ACTIONS(4888),
    [anon_sym_for] = ACTIONS(4888),
    [anon_sym_gen] = ACTIONS(4888),
    [anon_sym_if] = ACTIONS(4888),
    [anon_sym_impl] = ACTIONS(4888),
    [anon_sym_let] = ACTIONS(4888),
    [anon_sym_loop] = ACTIONS(4888),
    [anon_sym_match] = ACTIONS(4888),
    [anon_sym_mod] = ACTIONS(4888),
    [anon_sym_pub] = ACTIONS(4888),
    [anon_sym_return] = ACTIONS(4888),
    [anon_sym_raw] = ACTIONS(4888),
    [anon_sym_static] = ACTIONS(4888),
    [anon_sym_struct] = ACTIONS(4888),
    [anon_sym_trait] = ACTIONS(4888),
    [anon_sym_type] = ACTIONS(4888),
    [anon_sym_union] = ACTIONS(4888),
    [anon_sym_unsafe] = ACTIONS(4888),
    [anon_sym_use] = ACTIONS(4888),
    [anon_sym_while] = ACTIONS(4888),
    [anon_sym_extern] = ACTIONS(4888),
    [anon_sym_safe] = ACTIONS(4888),
    [anon_sym_yield] = ACTIONS(4888),
    [anon_sym_move] = ACTIONS(4888),
    [anon_sym_try] = ACTIONS(4888),
    [sym_integer_literal] = ACTIONS(4886),
    [aux_sym_string_literal_token1] = ACTIONS(4886),
    [sym_char_literal] = ACTIONS(4886),
    [anon_sym_true] = ACTIONS(4888),
    [anon_sym_false] = ACTIONS(4888),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4888),
    [sym_super] = ACTIONS(4888),
    [sym_crate] = ACTIONS(4888),
    [aux_sym_metavariable_token1] = ACTIONS(4886),
    [sym__raw_string_literal_start] = ACTIONS(4886),
    [sym_float_literal] = ACTIONS(4886),
    [sym__outer_block_doc_comment_start] = ACTIONS(4886),
    [sym__outer_line_doc_comment_start] = ACTIONS(4886),
  },
  [STATE(1289)] = {
    [sym_line_comment] = STATE(1289),
    [sym_block_comment] = STATE(1289),
    [ts_builtin_sym_end] = ACTIONS(4890),
    [sym_identifier] = ACTIONS(4892),
    [anon_sym_SEMI] = ACTIONS(4890),
    [anon_sym_macro] = ACTIONS(4892),
    [anon_sym_LPAREN] = ACTIONS(4890),
    [anon_sym_LBRACE] = ACTIONS(4890),
    [anon_sym_RBRACE] = ACTIONS(4890),
    [anon_sym_macro_rules] = ACTIONS(4892),
    [anon_sym_BANG] = ACTIONS(4890),
    [anon_sym_LBRACK] = ACTIONS(4890),
    [anon_sym_STAR] = ACTIONS(4890),
    [anon_sym_u8] = ACTIONS(4892),
    [anon_sym_i8] = ACTIONS(4892),
    [anon_sym_u16] = ACTIONS(4892),
    [anon_sym_i16] = ACTIONS(4892),
    [anon_sym_u32] = ACTIONS(4892),
    [anon_sym_i32] = ACTIONS(4892),
    [anon_sym_u64] = ACTIONS(4892),
    [anon_sym_i64] = ACTIONS(4892),
    [anon_sym_u128] = ACTIONS(4892),
    [anon_sym_i128] = ACTIONS(4892),
    [anon_sym_isize] = ACTIONS(4892),
    [anon_sym_usize] = ACTIONS(4892),
    [anon_sym_f32] = ACTIONS(4892),
    [anon_sym_f64] = ACTIONS(4892),
    [anon_sym_bool] = ACTIONS(4892),
    [anon_sym_str] = ACTIONS(4892),
    [anon_sym_char] = ACTIONS(4892),
    [anon_sym_DASH] = ACTIONS(4890),
    [anon_sym_AMP] = ACTIONS(4890),
    [anon_sym_PIPE] = ACTIONS(4890),
    [anon_sym_LT] = ACTIONS(4890),
    [anon_sym_DOT_DOT] = ACTIONS(4890),
    [anon_sym_COLON_COLON] = ACTIONS(4890),
    [anon_sym_POUND] = ACTIONS(4890),
    [anon_sym_SQUOTE] = ACTIONS(4892),
    [anon_sym_async] = ACTIONS(4892),
    [anon_sym_auto] = ACTIONS(4892),
    [anon_sym_await] = ACTIONS(4892),
    [anon_sym_become] = ACTIONS(4892),
    [anon_sym_break] = ACTIONS(4892),
    [anon_sym_const] = ACTIONS(4892),
    [anon_sym_continue] = ACTIONS(4892),
    [anon_sym_default] = ACTIONS(4892),
    [anon_sym_enum] = ACTIONS(4892),
    [anon_sym_fn] = ACTIONS(4892),
    [anon_sym_for] = ACTIONS(4892),
    [anon_sym_gen] = ACTIONS(4892),
    [anon_sym_if] = ACTIONS(4892),
    [anon_sym_impl] = ACTIONS(4892),
    [anon_sym_let] = ACTIONS(4892),
    [anon_sym_loop] = ACTIONS(4892),
    [anon_sym_match] = ACTIONS(4892),
    [anon_sym_mod] = ACTIONS(4892),
    [anon_sym_pub] = ACTIONS(4892),
    [anon_sym_return] = ACTIONS(4892),
    [anon_sym_raw] = ACTIONS(4892),
    [anon_sym_static] = ACTIONS(4892),
    [anon_sym_struct] = ACTIONS(4892),
    [anon_sym_trait] = ACTIONS(4892),
    [anon_sym_type] = ACTIONS(4892),
    [anon_sym_union] = ACTIONS(4892),
    [anon_sym_unsafe] = ACTIONS(4892),
    [anon_sym_use] = ACTIONS(4892),
    [anon_sym_while] = ACTIONS(4892),
    [anon_sym_extern] = ACTIONS(4892),
    [anon_sym_safe] = ACTIONS(4892),
    [anon_sym_yield] = ACTIONS(4892),
    [anon_sym_move] = ACTIONS(4892),
    [anon_sym_try] = ACTIONS(4892),
    [sym_integer_literal] = ACTIONS(4890),
    [aux_sym_string_literal_token1] = ACTIONS(4890),
    [sym_char_literal] = ACTIONS(4890),
    [anon_sym_true] = ACTIONS(4892),
    [anon_sym_false] = ACTIONS(4892),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4892),
    [sym_super] = ACTIONS(4892),
    [sym_crate] = ACTIONS(4892),
    [aux_sym_metavariable_token1] = ACTIONS(4890),
    [sym__raw_string_literal_start] = ACTIONS(4890),
    [sym_float_literal] = ACTIONS(4890),
    [sym__outer_block_doc_comment_start] = ACTIONS(4890),
    [sym__outer_line_doc_comment_start] = ACTIONS(4890),
  },
  [STATE(1290)] = {
    [sym_line_comment] = STATE(1290),
    [sym_block_comment] = STATE(1290),
    [ts_builtin_sym_end] = ACTIONS(4894),
    [sym_identifier] = ACTIONS(4896),
    [anon_sym_SEMI] = ACTIONS(4894),
    [anon_sym_macro] = ACTIONS(4896),
    [anon_sym_LPAREN] = ACTIONS(4894),
    [anon_sym_LBRACE] = ACTIONS(4894),
    [anon_sym_RBRACE] = ACTIONS(4894),
    [anon_sym_macro_rules] = ACTIONS(4896),
    [anon_sym_BANG] = ACTIONS(4894),
    [anon_sym_LBRACK] = ACTIONS(4894),
    [anon_sym_STAR] = ACTIONS(4894),
    [anon_sym_u8] = ACTIONS(4896),
    [anon_sym_i8] = ACTIONS(4896),
    [anon_sym_u16] = ACTIONS(4896),
    [anon_sym_i16] = ACTIONS(4896),
    [anon_sym_u32] = ACTIONS(4896),
    [anon_sym_i32] = ACTIONS(4896),
    [anon_sym_u64] = ACTIONS(4896),
    [anon_sym_i64] = ACTIONS(4896),
    [anon_sym_u128] = ACTIONS(4896),
    [anon_sym_i128] = ACTIONS(4896),
    [anon_sym_isize] = ACTIONS(4896),
    [anon_sym_usize] = ACTIONS(4896),
    [anon_sym_f32] = ACTIONS(4896),
    [anon_sym_f64] = ACTIONS(4896),
    [anon_sym_bool] = ACTIONS(4896),
    [anon_sym_str] = ACTIONS(4896),
    [anon_sym_char] = ACTIONS(4896),
    [anon_sym_DASH] = ACTIONS(4894),
    [anon_sym_AMP] = ACTIONS(4894),
    [anon_sym_PIPE] = ACTIONS(4894),
    [anon_sym_LT] = ACTIONS(4894),
    [anon_sym_DOT_DOT] = ACTIONS(4894),
    [anon_sym_COLON_COLON] = ACTIONS(4894),
    [anon_sym_POUND] = ACTIONS(4894),
    [anon_sym_SQUOTE] = ACTIONS(4896),
    [anon_sym_async] = ACTIONS(4896),
    [anon_sym_auto] = ACTIONS(4896),
    [anon_sym_await] = ACTIONS(4896),
    [anon_sym_become] = ACTIONS(4896),
    [anon_sym_break] = ACTIONS(4896),
    [anon_sym_const] = ACTIONS(4896),
    [anon_sym_continue] = ACTIONS(4896),
    [anon_sym_default] = ACTIONS(4896),
    [anon_sym_enum] = ACTIONS(4896),
    [anon_sym_fn] = ACTIONS(4896),
    [anon_sym_for] = ACTIONS(4896),
    [anon_sym_gen] = ACTIONS(4896),
    [anon_sym_if] = ACTIONS(4896),
    [anon_sym_impl] = ACTIONS(4896),
    [anon_sym_let] = ACTIONS(4896),
    [anon_sym_loop] = ACTIONS(4896),
    [anon_sym_match] = ACTIONS(4896),
    [anon_sym_mod] = ACTIONS(4896),
    [anon_sym_pub] = ACTIONS(4896),
    [anon_sym_return] = ACTIONS(4896),
    [anon_sym_raw] = ACTIONS(4896),
    [anon_sym_static] = ACTIONS(4896),
    [anon_sym_struct] = ACTIONS(4896),
    [anon_sym_trait] = ACTIONS(4896),
    [anon_sym_type] = ACTIONS(4896),
    [anon_sym_union] = ACTIONS(4896),
    [anon_sym_unsafe] = ACTIONS(4896),
    [anon_sym_use] = ACTIONS(4896),
    [anon_sym_while] = ACTIONS(4896),
    [anon_sym_extern] = ACTIONS(4896),
    [anon_sym_safe] = ACTIONS(4896),
    [anon_sym_yield] = ACTIONS(4896),
    [anon_sym_move] = ACTIONS(4896),
    [anon_sym_try] = ACTIONS(4896),
    [sym_integer_literal] = ACTIONS(4894),
    [aux_sym_string_literal_token1] = ACTIONS(4894),
    [sym_char_literal] = ACTIONS(4894),
    [anon_sym_true] = ACTIONS(4896),
    [anon_sym_false] = ACTIONS(4896),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4896),
    [sym_super] = ACTIONS(4896),
    [sym_crate] = ACTIONS(4896),
    [aux_sym_metavariable_token1] = ACTIONS(4894),
    [sym__raw_string_literal_start] = ACTIONS(4894),
    [sym_float_literal] = ACTIONS(4894),
    [sym__outer_block_doc_comment_start] = ACTIONS(4894),
    [sym__outer_line_doc_comment_start] = ACTIONS(4894),
  },
  [STATE(1291)] = {
    [sym_line_comment] = STATE(1291),
    [sym_block_comment] = STATE(1291),
    [ts_builtin_sym_end] = ACTIONS(4898),
    [sym_identifier] = ACTIONS(4900),
    [anon_sym_SEMI] = ACTIONS(4898),
    [anon_sym_macro] = ACTIONS(4900),
    [anon_sym_LPAREN] = ACTIONS(4898),
    [anon_sym_LBRACE] = ACTIONS(4898),
    [anon_sym_RBRACE] = ACTIONS(4898),
    [anon_sym_macro_rules] = ACTIONS(4900),
    [anon_sym_BANG] = ACTIONS(4898),
    [anon_sym_LBRACK] = ACTIONS(4898),
    [anon_sym_STAR] = ACTIONS(4898),
    [anon_sym_u8] = ACTIONS(4900),
    [anon_sym_i8] = ACTIONS(4900),
    [anon_sym_u16] = ACTIONS(4900),
    [anon_sym_i16] = ACTIONS(4900),
    [anon_sym_u32] = ACTIONS(4900),
    [anon_sym_i32] = ACTIONS(4900),
    [anon_sym_u64] = ACTIONS(4900),
    [anon_sym_i64] = ACTIONS(4900),
    [anon_sym_u128] = ACTIONS(4900),
    [anon_sym_i128] = ACTIONS(4900),
    [anon_sym_isize] = ACTIONS(4900),
    [anon_sym_usize] = ACTIONS(4900),
    [anon_sym_f32] = ACTIONS(4900),
    [anon_sym_f64] = ACTIONS(4900),
    [anon_sym_bool] = ACTIONS(4900),
    [anon_sym_str] = ACTIONS(4900),
    [anon_sym_char] = ACTIONS(4900),
    [anon_sym_DASH] = ACTIONS(4898),
    [anon_sym_AMP] = ACTIONS(4898),
    [anon_sym_PIPE] = ACTIONS(4898),
    [anon_sym_LT] = ACTIONS(4898),
    [anon_sym_DOT_DOT] = ACTIONS(4898),
    [anon_sym_COLON_COLON] = ACTIONS(4898),
    [anon_sym_POUND] = ACTIONS(4898),
    [anon_sym_SQUOTE] = ACTIONS(4900),
    [anon_sym_async] = ACTIONS(4900),
    [anon_sym_auto] = ACTIONS(4900),
    [anon_sym_await] = ACTIONS(4900),
    [anon_sym_become] = ACTIONS(4900),
    [anon_sym_break] = ACTIONS(4900),
    [anon_sym_const] = ACTIONS(4900),
    [anon_sym_continue] = ACTIONS(4900),
    [anon_sym_default] = ACTIONS(4900),
    [anon_sym_enum] = ACTIONS(4900),
    [anon_sym_fn] = ACTIONS(4900),
    [anon_sym_for] = ACTIONS(4900),
    [anon_sym_gen] = ACTIONS(4900),
    [anon_sym_if] = ACTIONS(4900),
    [anon_sym_impl] = ACTIONS(4900),
    [anon_sym_let] = ACTIONS(4900),
    [anon_sym_loop] = ACTIONS(4900),
    [anon_sym_match] = ACTIONS(4900),
    [anon_sym_mod] = ACTIONS(4900),
    [anon_sym_pub] = ACTIONS(4900),
    [anon_sym_return] = ACTIONS(4900),
    [anon_sym_raw] = ACTIONS(4900),
    [anon_sym_static] = ACTIONS(4900),
    [anon_sym_struct] = ACTIONS(4900),
    [anon_sym_trait] = ACTIONS(4900),
    [anon_sym_type] = ACTIONS(4900),
    [anon_sym_union] = ACTIONS(4900),
    [anon_sym_unsafe] = ACTIONS(4900),
    [anon_sym_use] = ACTIONS(4900),
    [anon_sym_while] = ACTIONS(4900),
    [anon_sym_extern] = ACTIONS(4900),
    [anon_sym_safe] = ACTIONS(4900),
    [anon_sym_yield] = ACTIONS(4900),
    [anon_sym_move] = ACTIONS(4900),
    [anon_sym_try] = ACTIONS(4900),
    [sym_integer_literal] = ACTIONS(4898),
    [aux_sym_string_literal_token1] = ACTIONS(4898),
    [sym_char_literal] = ACTIONS(4898),
    [anon_sym_true] = ACTIONS(4900),
    [anon_sym_false] = ACTIONS(4900),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4900),
    [sym_super] = ACTIONS(4900),
    [sym_crate] = ACTIONS(4900),
    [aux_sym_metavariable_token1] = ACTIONS(4898),
    [sym__raw_string_literal_start] = ACTIONS(4898),
    [sym_float_literal] = ACTIONS(4898),
    [sym__outer_block_doc_comment_start] = ACTIONS(4898),
    [sym__outer_line_doc_comment_start] = ACTIONS(4898),
  },
  [STATE(1292)] = {
    [sym_line_comment] = STATE(1292),
    [sym_block_comment] = STATE(1292),
    [ts_builtin_sym_end] = ACTIONS(4902),
    [sym_identifier] = ACTIONS(4904),
    [anon_sym_SEMI] = ACTIONS(4902),
    [anon_sym_macro] = ACTIONS(4904),
    [anon_sym_LPAREN] = ACTIONS(4902),
    [anon_sym_LBRACE] = ACTIONS(4902),
    [anon_sym_RBRACE] = ACTIONS(4902),
    [anon_sym_macro_rules] = ACTIONS(4904),
    [anon_sym_BANG] = ACTIONS(4902),
    [anon_sym_LBRACK] = ACTIONS(4902),
    [anon_sym_STAR] = ACTIONS(4902),
    [anon_sym_u8] = ACTIONS(4904),
    [anon_sym_i8] = ACTIONS(4904),
    [anon_sym_u16] = ACTIONS(4904),
    [anon_sym_i16] = ACTIONS(4904),
    [anon_sym_u32] = ACTIONS(4904),
    [anon_sym_i32] = ACTIONS(4904),
    [anon_sym_u64] = ACTIONS(4904),
    [anon_sym_i64] = ACTIONS(4904),
    [anon_sym_u128] = ACTIONS(4904),
    [anon_sym_i128] = ACTIONS(4904),
    [anon_sym_isize] = ACTIONS(4904),
    [anon_sym_usize] = ACTIONS(4904),
    [anon_sym_f32] = ACTIONS(4904),
    [anon_sym_f64] = ACTIONS(4904),
    [anon_sym_bool] = ACTIONS(4904),
    [anon_sym_str] = ACTIONS(4904),
    [anon_sym_char] = ACTIONS(4904),
    [anon_sym_DASH] = ACTIONS(4902),
    [anon_sym_AMP] = ACTIONS(4902),
    [anon_sym_PIPE] = ACTIONS(4902),
    [anon_sym_LT] = ACTIONS(4902),
    [anon_sym_DOT_DOT] = ACTIONS(4902),
    [anon_sym_COLON_COLON] = ACTIONS(4902),
    [anon_sym_POUND] = ACTIONS(4902),
    [anon_sym_SQUOTE] = ACTIONS(4904),
    [anon_sym_async] = ACTIONS(4904),
    [anon_sym_auto] = ACTIONS(4904),
    [anon_sym_await] = ACTIONS(4904),
    [anon_sym_become] = ACTIONS(4904),
    [anon_sym_break] = ACTIONS(4904),
    [anon_sym_const] = ACTIONS(4904),
    [anon_sym_continue] = ACTIONS(4904),
    [anon_sym_default] = ACTIONS(4904),
    [anon_sym_enum] = ACTIONS(4904),
    [anon_sym_fn] = ACTIONS(4904),
    [anon_sym_for] = ACTIONS(4904),
    [anon_sym_gen] = ACTIONS(4904),
    [anon_sym_if] = ACTIONS(4904),
    [anon_sym_impl] = ACTIONS(4904),
    [anon_sym_let] = ACTIONS(4904),
    [anon_sym_loop] = ACTIONS(4904),
    [anon_sym_match] = ACTIONS(4904),
    [anon_sym_mod] = ACTIONS(4904),
    [anon_sym_pub] = ACTIONS(4904),
    [anon_sym_return] = ACTIONS(4904),
    [anon_sym_raw] = ACTIONS(4904),
    [anon_sym_static] = ACTIONS(4904),
    [anon_sym_struct] = ACTIONS(4904),
    [anon_sym_trait] = ACTIONS(4904),
    [anon_sym_type] = ACTIONS(4904),
    [anon_sym_union] = ACTIONS(4904),
    [anon_sym_unsafe] = ACTIONS(4904),
    [anon_sym_use] = ACTIONS(4904),
    [anon_sym_while] = ACTIONS(4904),
    [anon_sym_extern] = ACTIONS(4904),
    [anon_sym_safe] = ACTIONS(4904),
    [anon_sym_yield] = ACTIONS(4904),
    [anon_sym_move] = ACTIONS(4904),
    [anon_sym_try] = ACTIONS(4904),
    [sym_integer_literal] = ACTIONS(4902),
    [aux_sym_string_literal_token1] = ACTIONS(4902),
    [sym_char_literal] = ACTIONS(4902),
    [anon_sym_true] = ACTIONS(4904),
    [anon_sym_false] = ACTIONS(4904),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4904),
    [sym_super] = ACTIONS(4904),
    [sym_crate] = ACTIONS(4904),
    [aux_sym_metavariable_token1] = ACTIONS(4902),
    [sym__raw_string_literal_start] = ACTIONS(4902),
    [sym_float_literal] = ACTIONS(4902),
    [sym__outer_block_doc_comment_start] = ACTIONS(4902),
    [sym__outer_line_doc_comment_start] = ACTIONS(4902),
  },
  [STATE(1293)] = {
    [sym_line_comment] = STATE(1293),
    [sym_block_comment] = STATE(1293),
    [ts_builtin_sym_end] = ACTIONS(4906),
    [sym_identifier] = ACTIONS(4908),
    [anon_sym_SEMI] = ACTIONS(4906),
    [anon_sym_macro] = ACTIONS(4908),
    [anon_sym_LPAREN] = ACTIONS(4906),
    [anon_sym_LBRACE] = ACTIONS(4906),
    [anon_sym_RBRACE] = ACTIONS(4906),
    [anon_sym_macro_rules] = ACTIONS(4908),
    [anon_sym_BANG] = ACTIONS(4906),
    [anon_sym_LBRACK] = ACTIONS(4906),
    [anon_sym_STAR] = ACTIONS(4906),
    [anon_sym_u8] = ACTIONS(4908),
    [anon_sym_i8] = ACTIONS(4908),
    [anon_sym_u16] = ACTIONS(4908),
    [anon_sym_i16] = ACTIONS(4908),
    [anon_sym_u32] = ACTIONS(4908),
    [anon_sym_i32] = ACTIONS(4908),
    [anon_sym_u64] = ACTIONS(4908),
    [anon_sym_i64] = ACTIONS(4908),
    [anon_sym_u128] = ACTIONS(4908),
    [anon_sym_i128] = ACTIONS(4908),
    [anon_sym_isize] = ACTIONS(4908),
    [anon_sym_usize] = ACTIONS(4908),
    [anon_sym_f32] = ACTIONS(4908),
    [anon_sym_f64] = ACTIONS(4908),
    [anon_sym_bool] = ACTIONS(4908),
    [anon_sym_str] = ACTIONS(4908),
    [anon_sym_char] = ACTIONS(4908),
    [anon_sym_DASH] = ACTIONS(4906),
    [anon_sym_AMP] = ACTIONS(4906),
    [anon_sym_PIPE] = ACTIONS(4906),
    [anon_sym_LT] = ACTIONS(4906),
    [anon_sym_DOT_DOT] = ACTIONS(4906),
    [anon_sym_COLON_COLON] = ACTIONS(4906),
    [anon_sym_POUND] = ACTIONS(4906),
    [anon_sym_SQUOTE] = ACTIONS(4908),
    [anon_sym_async] = ACTIONS(4908),
    [anon_sym_auto] = ACTIONS(4908),
    [anon_sym_await] = ACTIONS(4908),
    [anon_sym_become] = ACTIONS(4908),
    [anon_sym_break] = ACTIONS(4908),
    [anon_sym_const] = ACTIONS(4908),
    [anon_sym_continue] = ACTIONS(4908),
    [anon_sym_default] = ACTIONS(4908),
    [anon_sym_enum] = ACTIONS(4908),
    [anon_sym_fn] = ACTIONS(4908),
    [anon_sym_for] = ACTIONS(4908),
    [anon_sym_gen] = ACTIONS(4908),
    [anon_sym_if] = ACTIONS(4908),
    [anon_sym_impl] = ACTIONS(4908),
    [anon_sym_let] = ACTIONS(4908),
    [anon_sym_loop] = ACTIONS(4908),
    [anon_sym_match] = ACTIONS(4908),
    [anon_sym_mod] = ACTIONS(4908),
    [anon_sym_pub] = ACTIONS(4908),
    [anon_sym_return] = ACTIONS(4908),
    [anon_sym_raw] = ACTIONS(4908),
    [anon_sym_static] = ACTIONS(4908),
    [anon_sym_struct] = ACTIONS(4908),
    [anon_sym_trait] = ACTIONS(4908),
    [anon_sym_type] = ACTIONS(4908),
    [anon_sym_union] = ACTIONS(4908),
    [anon_sym_unsafe] = ACTIONS(4908),
    [anon_sym_use] = ACTIONS(4908),
    [anon_sym_while] = ACTIONS(4908),
    [anon_sym_extern] = ACTIONS(4908),
    [anon_sym_safe] = ACTIONS(4908),
    [anon_sym_yield] = ACTIONS(4908),
    [anon_sym_move] = ACTIONS(4908),
    [anon_sym_try] = ACTIONS(4908),
    [sym_integer_literal] = ACTIONS(4906),
    [aux_sym_string_literal_token1] = ACTIONS(4906),
    [sym_char_literal] = ACTIONS(4906),
    [anon_sym_true] = ACTIONS(4908),
    [anon_sym_false] = ACTIONS(4908),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4908),
    [sym_super] = ACTIONS(4908),
    [sym_crate] = ACTIONS(4908),
    [aux_sym_metavariable_token1] = ACTIONS(4906),
    [sym__raw_string_literal_start] = ACTIONS(4906),
    [sym_float_literal] = ACTIONS(4906),
    [sym__outer_block_doc_comment_start] = ACTIONS(4906),
    [sym__outer_line_doc_comment_start] = ACTIONS(4906),
  },
  [STATE(1294)] = {
    [sym_line_comment] = STATE(1294),
    [sym_block_comment] = STATE(1294),
    [ts_builtin_sym_end] = ACTIONS(4910),
    [sym_identifier] = ACTIONS(4912),
    [anon_sym_SEMI] = ACTIONS(4910),
    [anon_sym_macro] = ACTIONS(4912),
    [anon_sym_LPAREN] = ACTIONS(4910),
    [anon_sym_LBRACE] = ACTIONS(4910),
    [anon_sym_RBRACE] = ACTIONS(4910),
    [anon_sym_macro_rules] = ACTIONS(4912),
    [anon_sym_BANG] = ACTIONS(4910),
    [anon_sym_LBRACK] = ACTIONS(4910),
    [anon_sym_STAR] = ACTIONS(4910),
    [anon_sym_u8] = ACTIONS(4912),
    [anon_sym_i8] = ACTIONS(4912),
    [anon_sym_u16] = ACTIONS(4912),
    [anon_sym_i16] = ACTIONS(4912),
    [anon_sym_u32] = ACTIONS(4912),
    [anon_sym_i32] = ACTIONS(4912),
    [anon_sym_u64] = ACTIONS(4912),
    [anon_sym_i64] = ACTIONS(4912),
    [anon_sym_u128] = ACTIONS(4912),
    [anon_sym_i128] = ACTIONS(4912),
    [anon_sym_isize] = ACTIONS(4912),
    [anon_sym_usize] = ACTIONS(4912),
    [anon_sym_f32] = ACTIONS(4912),
    [anon_sym_f64] = ACTIONS(4912),
    [anon_sym_bool] = ACTIONS(4912),
    [anon_sym_str] = ACTIONS(4912),
    [anon_sym_char] = ACTIONS(4912),
    [anon_sym_DASH] = ACTIONS(4910),
    [anon_sym_AMP] = ACTIONS(4910),
    [anon_sym_PIPE] = ACTIONS(4910),
    [anon_sym_LT] = ACTIONS(4910),
    [anon_sym_DOT_DOT] = ACTIONS(4910),
    [anon_sym_COLON_COLON] = ACTIONS(4910),
    [anon_sym_POUND] = ACTIONS(4910),
    [anon_sym_SQUOTE] = ACTIONS(4912),
    [anon_sym_async] = ACTIONS(4912),
    [anon_sym_auto] = ACTIONS(4912),
    [anon_sym_await] = ACTIONS(4912),
    [anon_sym_become] = ACTIONS(4912),
    [anon_sym_break] = ACTIONS(4912),
    [anon_sym_const] = ACTIONS(4912),
    [anon_sym_continue] = ACTIONS(4912),
    [anon_sym_default] = ACTIONS(4912),
    [anon_sym_enum] = ACTIONS(4912),
    [anon_sym_fn] = ACTIONS(4912),
    [anon_sym_for] = ACTIONS(4912),
    [anon_sym_gen] = ACTIONS(4912),
    [anon_sym_if] = ACTIONS(4912),
    [anon_sym_impl] = ACTIONS(4912),
    [anon_sym_let] = ACTIONS(4912),
    [anon_sym_loop] = ACTIONS(4912),
    [anon_sym_match] = ACTIONS(4912),
    [anon_sym_mod] = ACTIONS(4912),
    [anon_sym_pub] = ACTIONS(4912),
    [anon_sym_return] = ACTIONS(4912),
    [anon_sym_raw] = ACTIONS(4912),
    [anon_sym_static] = ACTIONS(4912),
    [anon_sym_struct] = ACTIONS(4912),
    [anon_sym_trait] = ACTIONS(4912),
    [anon_sym_type] = ACTIONS(4912),
    [anon_sym_union] = ACTIONS(4912),
    [anon_sym_unsafe] = ACTIONS(4912),
    [anon_sym_use] = ACTIONS(4912),
    [anon_sym_while] = ACTIONS(4912),
    [anon_sym_extern] = ACTIONS(4912),
    [anon_sym_safe] = ACTIONS(4912),
    [anon_sym_yield] = ACTIONS(4912),
    [anon_sym_move] = ACTIONS(4912),
    [anon_sym_try] = ACTIONS(4912),
    [sym_integer_literal] = ACTIONS(4910),
    [aux_sym_string_literal_token1] = ACTIONS(4910),
    [sym_char_literal] = ACTIONS(4910),
    [anon_sym_true] = ACTIONS(4912),
    [anon_sym_false] = ACTIONS(4912),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4912),
    [sym_super] = ACTIONS(4912),
    [sym_crate] = ACTIONS(4912),
    [aux_sym_metavariable_token1] = ACTIONS(4910),
    [sym__raw_string_literal_start] = ACTIONS(4910),
    [sym_float_literal] = ACTIONS(4910),
    [sym__outer_block_doc_comment_start] = ACTIONS(4910),
    [sym__outer_line_doc_comment_start] = ACTIONS(4910),
  },
  [STATE(1295)] = {
    [sym_line_comment] = STATE(1295),
    [sym_block_comment] = STATE(1295),
    [ts_builtin_sym_end] = ACTIONS(4914),
    [sym_identifier] = ACTIONS(4916),
    [anon_sym_SEMI] = ACTIONS(4914),
    [anon_sym_macro] = ACTIONS(4916),
    [anon_sym_LPAREN] = ACTIONS(4914),
    [anon_sym_LBRACE] = ACTIONS(4914),
    [anon_sym_RBRACE] = ACTIONS(4914),
    [anon_sym_macro_rules] = ACTIONS(4916),
    [anon_sym_BANG] = ACTIONS(4914),
    [anon_sym_LBRACK] = ACTIONS(4914),
    [anon_sym_STAR] = ACTIONS(4914),
    [anon_sym_u8] = ACTIONS(4916),
    [anon_sym_i8] = ACTIONS(4916),
    [anon_sym_u16] = ACTIONS(4916),
    [anon_sym_i16] = ACTIONS(4916),
    [anon_sym_u32] = ACTIONS(4916),
    [anon_sym_i32] = ACTIONS(4916),
    [anon_sym_u64] = ACTIONS(4916),
    [anon_sym_i64] = ACTIONS(4916),
    [anon_sym_u128] = ACTIONS(4916),
    [anon_sym_i128] = ACTIONS(4916),
    [anon_sym_isize] = ACTIONS(4916),
    [anon_sym_usize] = ACTIONS(4916),
    [anon_sym_f32] = ACTIONS(4916),
    [anon_sym_f64] = ACTIONS(4916),
    [anon_sym_bool] = ACTIONS(4916),
    [anon_sym_str] = ACTIONS(4916),
    [anon_sym_char] = ACTIONS(4916),
    [anon_sym_DASH] = ACTIONS(4914),
    [anon_sym_AMP] = ACTIONS(4914),
    [anon_sym_PIPE] = ACTIONS(4914),
    [anon_sym_LT] = ACTIONS(4914),
    [anon_sym_DOT_DOT] = ACTIONS(4914),
    [anon_sym_COLON_COLON] = ACTIONS(4914),
    [anon_sym_POUND] = ACTIONS(4914),
    [anon_sym_SQUOTE] = ACTIONS(4916),
    [anon_sym_async] = ACTIONS(4916),
    [anon_sym_auto] = ACTIONS(4916),
    [anon_sym_await] = ACTIONS(4916),
    [anon_sym_become] = ACTIONS(4916),
    [anon_sym_break] = ACTIONS(4916),
    [anon_sym_const] = ACTIONS(4916),
    [anon_sym_continue] = ACTIONS(4916),
    [anon_sym_default] = ACTIONS(4916),
    [anon_sym_enum] = ACTIONS(4916),
    [anon_sym_fn] = ACTIONS(4916),
    [anon_sym_for] = ACTIONS(4916),
    [anon_sym_gen] = ACTIONS(4916),
    [anon_sym_if] = ACTIONS(4916),
    [anon_sym_impl] = ACTIONS(4916),
    [anon_sym_let] = ACTIONS(4916),
    [anon_sym_loop] = ACTIONS(4916),
    [anon_sym_match] = ACTIONS(4916),
    [anon_sym_mod] = ACTIONS(4916),
    [anon_sym_pub] = ACTIONS(4916),
    [anon_sym_return] = ACTIONS(4916),
    [anon_sym_raw] = ACTIONS(4916),
    [anon_sym_static] = ACTIONS(4916),
    [anon_sym_struct] = ACTIONS(4916),
    [anon_sym_trait] = ACTIONS(4916),
    [anon_sym_type] = ACTIONS(4916),
    [anon_sym_union] = ACTIONS(4916),
    [anon_sym_unsafe] = ACTIONS(4916),
    [anon_sym_use] = ACTIONS(4916),
    [anon_sym_while] = ACTIONS(4916),
    [anon_sym_extern] = ACTIONS(4916),
    [anon_sym_safe] = ACTIONS(4916),
    [anon_sym_yield] = ACTIONS(4916),
    [anon_sym_move] = ACTIONS(4916),
    [anon_sym_try] = ACTIONS(4916),
    [sym_integer_literal] = ACTIONS(4914),
    [aux_sym_string_literal_token1] = ACTIONS(4914),
    [sym_char_literal] = ACTIONS(4914),
    [anon_sym_true] = ACTIONS(4916),
    [anon_sym_false] = ACTIONS(4916),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4916),
    [sym_super] = ACTIONS(4916),
    [sym_crate] = ACTIONS(4916),
    [aux_sym_metavariable_token1] = ACTIONS(4914),
    [sym__raw_string_literal_start] = ACTIONS(4914),
    [sym_float_literal] = ACTIONS(4914),
    [sym__outer_block_doc_comment_start] = ACTIONS(4914),
    [sym__outer_line_doc_comment_start] = ACTIONS(4914),
  },
  [STATE(1296)] = {
    [sym_line_comment] = STATE(1296),
    [sym_block_comment] = STATE(1296),
    [ts_builtin_sym_end] = ACTIONS(4918),
    [sym_identifier] = ACTIONS(4920),
    [anon_sym_SEMI] = ACTIONS(4918),
    [anon_sym_macro] = ACTIONS(4920),
    [anon_sym_LPAREN] = ACTIONS(4918),
    [anon_sym_LBRACE] = ACTIONS(4918),
    [anon_sym_RBRACE] = ACTIONS(4918),
    [anon_sym_macro_rules] = ACTIONS(4920),
    [anon_sym_BANG] = ACTIONS(4918),
    [anon_sym_LBRACK] = ACTIONS(4918),
    [anon_sym_STAR] = ACTIONS(4918),
    [anon_sym_u8] = ACTIONS(4920),
    [anon_sym_i8] = ACTIONS(4920),
    [anon_sym_u16] = ACTIONS(4920),
    [anon_sym_i16] = ACTIONS(4920),
    [anon_sym_u32] = ACTIONS(4920),
    [anon_sym_i32] = ACTIONS(4920),
    [anon_sym_u64] = ACTIONS(4920),
    [anon_sym_i64] = ACTIONS(4920),
    [anon_sym_u128] = ACTIONS(4920),
    [anon_sym_i128] = ACTIONS(4920),
    [anon_sym_isize] = ACTIONS(4920),
    [anon_sym_usize] = ACTIONS(4920),
    [anon_sym_f32] = ACTIONS(4920),
    [anon_sym_f64] = ACTIONS(4920),
    [anon_sym_bool] = ACTIONS(4920),
    [anon_sym_str] = ACTIONS(4920),
    [anon_sym_char] = ACTIONS(4920),
    [anon_sym_DASH] = ACTIONS(4918),
    [anon_sym_AMP] = ACTIONS(4918),
    [anon_sym_PIPE] = ACTIONS(4918),
    [anon_sym_LT] = ACTIONS(4918),
    [anon_sym_DOT_DOT] = ACTIONS(4918),
    [anon_sym_COLON_COLON] = ACTIONS(4918),
    [anon_sym_POUND] = ACTIONS(4918),
    [anon_sym_SQUOTE] = ACTIONS(4920),
    [anon_sym_async] = ACTIONS(4920),
    [anon_sym_auto] = ACTIONS(4920),
    [anon_sym_await] = ACTIONS(4920),
    [anon_sym_become] = ACTIONS(4920),
    [anon_sym_break] = ACTIONS(4920),
    [anon_sym_const] = ACTIONS(4920),
    [anon_sym_continue] = ACTIONS(4920),
    [anon_sym_default] = ACTIONS(4920),
    [anon_sym_enum] = ACTIONS(4920),
    [anon_sym_fn] = ACTIONS(4920),
    [anon_sym_for] = ACTIONS(4920),
    [anon_sym_gen] = ACTIONS(4920),
    [anon_sym_if] = ACTIONS(4920),
    [anon_sym_impl] = ACTIONS(4920),
    [anon_sym_let] = ACTIONS(4920),
    [anon_sym_loop] = ACTIONS(4920),
    [anon_sym_match] = ACTIONS(4920),
    [anon_sym_mod] = ACTIONS(4920),
    [anon_sym_pub] = ACTIONS(4920),
    [anon_sym_return] = ACTIONS(4920),
    [anon_sym_raw] = ACTIONS(4920),
    [anon_sym_static] = ACTIONS(4920),
    [anon_sym_struct] = ACTIONS(4920),
    [anon_sym_trait] = ACTIONS(4920),
    [anon_sym_type] = ACTIONS(4920),
    [anon_sym_union] = ACTIONS(4920),
    [anon_sym_unsafe] = ACTIONS(4920),
    [anon_sym_use] = ACTIONS(4920),
    [anon_sym_while] = ACTIONS(4920),
    [anon_sym_extern] = ACTIONS(4920),
    [anon_sym_safe] = ACTIONS(4920),
    [anon_sym_yield] = ACTIONS(4920),
    [anon_sym_move] = ACTIONS(4920),
    [anon_sym_try] = ACTIONS(4920),
    [sym_integer_literal] = ACTIONS(4918),
    [aux_sym_string_literal_token1] = ACTIONS(4918),
    [sym_char_literal] = ACTIONS(4918),
    [anon_sym_true] = ACTIONS(4920),
    [anon_sym_false] = ACTIONS(4920),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4920),
    [sym_super] = ACTIONS(4920),
    [sym_crate] = ACTIONS(4920),
    [aux_sym_metavariable_token1] = ACTIONS(4918),
    [sym__raw_string_literal_start] = ACTIONS(4918),
    [sym_float_literal] = ACTIONS(4918),
    [sym__outer_block_doc_comment_start] = ACTIONS(4918),
    [sym__outer_line_doc_comment_start] = ACTIONS(4918),
  },
  [STATE(1297)] = {
    [sym_line_comment] = STATE(1297),
    [sym_block_comment] = STATE(1297),
    [ts_builtin_sym_end] = ACTIONS(4922),
    [sym_identifier] = ACTIONS(4924),
    [anon_sym_SEMI] = ACTIONS(4922),
    [anon_sym_macro] = ACTIONS(4924),
    [anon_sym_LPAREN] = ACTIONS(4922),
    [anon_sym_LBRACE] = ACTIONS(4922),
    [anon_sym_RBRACE] = ACTIONS(4922),
    [anon_sym_macro_rules] = ACTIONS(4924),
    [anon_sym_BANG] = ACTIONS(4922),
    [anon_sym_LBRACK] = ACTIONS(4922),
    [anon_sym_STAR] = ACTIONS(4922),
    [anon_sym_u8] = ACTIONS(4924),
    [anon_sym_i8] = ACTIONS(4924),
    [anon_sym_u16] = ACTIONS(4924),
    [anon_sym_i16] = ACTIONS(4924),
    [anon_sym_u32] = ACTIONS(4924),
    [anon_sym_i32] = ACTIONS(4924),
    [anon_sym_u64] = ACTIONS(4924),
    [anon_sym_i64] = ACTIONS(4924),
    [anon_sym_u128] = ACTIONS(4924),
    [anon_sym_i128] = ACTIONS(4924),
    [anon_sym_isize] = ACTIONS(4924),
    [anon_sym_usize] = ACTIONS(4924),
    [anon_sym_f32] = ACTIONS(4924),
    [anon_sym_f64] = ACTIONS(4924),
    [anon_sym_bool] = ACTIONS(4924),
    [anon_sym_str] = ACTIONS(4924),
    [anon_sym_char] = ACTIONS(4924),
    [anon_sym_DASH] = ACTIONS(4922),
    [anon_sym_AMP] = ACTIONS(4922),
    [anon_sym_PIPE] = ACTIONS(4922),
    [anon_sym_LT] = ACTIONS(4922),
    [anon_sym_DOT_DOT] = ACTIONS(4922),
    [anon_sym_COLON_COLON] = ACTIONS(4922),
    [anon_sym_POUND] = ACTIONS(4922),
    [anon_sym_SQUOTE] = ACTIONS(4924),
    [anon_sym_async] = ACTIONS(4924),
    [anon_sym_auto] = ACTIONS(4924),
    [anon_sym_await] = ACTIONS(4924),
    [anon_sym_become] = ACTIONS(4924),
    [anon_sym_break] = ACTIONS(4924),
    [anon_sym_const] = ACTIONS(4924),
    [anon_sym_continue] = ACTIONS(4924),
    [anon_sym_default] = ACTIONS(4924),
    [anon_sym_enum] = ACTIONS(4924),
    [anon_sym_fn] = ACTIONS(4924),
    [anon_sym_for] = ACTIONS(4924),
    [anon_sym_gen] = ACTIONS(4924),
    [anon_sym_if] = ACTIONS(4924),
    [anon_sym_impl] = ACTIONS(4924),
    [anon_sym_let] = ACTIONS(4924),
    [anon_sym_loop] = ACTIONS(4924),
    [anon_sym_match] = ACTIONS(4924),
    [anon_sym_mod] = ACTIONS(4924),
    [anon_sym_pub] = ACTIONS(4924),
    [anon_sym_return] = ACTIONS(4924),
    [anon_sym_raw] = ACTIONS(4924),
    [anon_sym_static] = ACTIONS(4924),
    [anon_sym_struct] = ACTIONS(4924),
    [anon_sym_trait] = ACTIONS(4924),
    [anon_sym_type] = ACTIONS(4924),
    [anon_sym_union] = ACTIONS(4924),
    [anon_sym_unsafe] = ACTIONS(4924),
    [anon_sym_use] = ACTIONS(4924),
    [anon_sym_while] = ACTIONS(4924),
    [anon_sym_extern] = ACTIONS(4924),
    [anon_sym_safe] = ACTIONS(4924),
    [anon_sym_yield] = ACTIONS(4924),
    [anon_sym_move] = ACTIONS(4924),
    [anon_sym_try] = ACTIONS(4924),
    [sym_integer_literal] = ACTIONS(4922),
    [aux_sym_string_literal_token1] = ACTIONS(4922),
    [sym_char_literal] = ACTIONS(4922),
    [anon_sym_true] = ACTIONS(4924),
    [anon_sym_false] = ACTIONS(4924),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4924),
    [sym_super] = ACTIONS(4924),
    [sym_crate] = ACTIONS(4924),
    [aux_sym_metavariable_token1] = ACTIONS(4922),
    [sym__raw_string_literal_start] = ACTIONS(4922),
    [sym_float_literal] = ACTIONS(4922),
    [sym__outer_block_doc_comment_start] = ACTIONS(4922),
    [sym__outer_line_doc_comment_start] = ACTIONS(4922),
  },
  [STATE(1298)] = {
    [sym_line_comment] = STATE(1298),
    [sym_block_comment] = STATE(1298),
    [ts_builtin_sym_end] = ACTIONS(4926),
    [sym_identifier] = ACTIONS(4928),
    [anon_sym_SEMI] = ACTIONS(4926),
    [anon_sym_macro] = ACTIONS(4928),
    [anon_sym_LPAREN] = ACTIONS(4926),
    [anon_sym_LBRACE] = ACTIONS(4926),
    [anon_sym_RBRACE] = ACTIONS(4926),
    [anon_sym_macro_rules] = ACTIONS(4928),
    [anon_sym_BANG] = ACTIONS(4926),
    [anon_sym_LBRACK] = ACTIONS(4926),
    [anon_sym_STAR] = ACTIONS(4926),
    [anon_sym_u8] = ACTIONS(4928),
    [anon_sym_i8] = ACTIONS(4928),
    [anon_sym_u16] = ACTIONS(4928),
    [anon_sym_i16] = ACTIONS(4928),
    [anon_sym_u32] = ACTIONS(4928),
    [anon_sym_i32] = ACTIONS(4928),
    [anon_sym_u64] = ACTIONS(4928),
    [anon_sym_i64] = ACTIONS(4928),
    [anon_sym_u128] = ACTIONS(4928),
    [anon_sym_i128] = ACTIONS(4928),
    [anon_sym_isize] = ACTIONS(4928),
    [anon_sym_usize] = ACTIONS(4928),
    [anon_sym_f32] = ACTIONS(4928),
    [anon_sym_f64] = ACTIONS(4928),
    [anon_sym_bool] = ACTIONS(4928),
    [anon_sym_str] = ACTIONS(4928),
    [anon_sym_char] = ACTIONS(4928),
    [anon_sym_DASH] = ACTIONS(4926),
    [anon_sym_AMP] = ACTIONS(4926),
    [anon_sym_PIPE] = ACTIONS(4926),
    [anon_sym_LT] = ACTIONS(4926),
    [anon_sym_DOT_DOT] = ACTIONS(4926),
    [anon_sym_COLON_COLON] = ACTIONS(4926),
    [anon_sym_POUND] = ACTIONS(4926),
    [anon_sym_SQUOTE] = ACTIONS(4928),
    [anon_sym_async] = ACTIONS(4928),
    [anon_sym_auto] = ACTIONS(4928),
    [anon_sym_await] = ACTIONS(4928),
    [anon_sym_become] = ACTIONS(4928),
    [anon_sym_break] = ACTIONS(4928),
    [anon_sym_const] = ACTIONS(4928),
    [anon_sym_continue] = ACTIONS(4928),
    [anon_sym_default] = ACTIONS(4928),
    [anon_sym_enum] = ACTIONS(4928),
    [anon_sym_fn] = ACTIONS(4928),
    [anon_sym_for] = ACTIONS(4928),
    [anon_sym_gen] = ACTIONS(4928),
    [anon_sym_if] = ACTIONS(4928),
    [anon_sym_impl] = ACTIONS(4928),
    [anon_sym_let] = ACTIONS(4928),
    [anon_sym_loop] = ACTIONS(4928),
    [anon_sym_match] = ACTIONS(4928),
    [anon_sym_mod] = ACTIONS(4928),
    [anon_sym_pub] = ACTIONS(4928),
    [anon_sym_return] = ACTIONS(4928),
    [anon_sym_raw] = ACTIONS(4928),
    [anon_sym_static] = ACTIONS(4928),
    [anon_sym_struct] = ACTIONS(4928),
    [anon_sym_trait] = ACTIONS(4928),
    [anon_sym_type] = ACTIONS(4928),
    [anon_sym_union] = ACTIONS(4928),
    [anon_sym_unsafe] = ACTIONS(4928),
    [anon_sym_use] = ACTIONS(4928),
    [anon_sym_while] = ACTIONS(4928),
    [anon_sym_extern] = ACTIONS(4928),
    [anon_sym_safe] = ACTIONS(4928),
    [anon_sym_yield] = ACTIONS(4928),
    [anon_sym_move] = ACTIONS(4928),
    [anon_sym_try] = ACTIONS(4928),
    [sym_integer_literal] = ACTIONS(4926),
    [aux_sym_string_literal_token1] = ACTIONS(4926),
    [sym_char_literal] = ACTIONS(4926),
    [anon_sym_true] = ACTIONS(4928),
    [anon_sym_false] = ACTIONS(4928),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4928),
    [sym_super] = ACTIONS(4928),
    [sym_crate] = ACTIONS(4928),
    [aux_sym_metavariable_token1] = ACTIONS(4926),
    [sym__raw_string_literal_start] = ACTIONS(4926),
    [sym_float_literal] = ACTIONS(4926),
    [sym__outer_block_doc_comment_start] = ACTIONS(4926),
    [sym__outer_line_doc_comment_start] = ACTIONS(4926),
  },
  [STATE(1299)] = {
    [sym_line_comment] = STATE(1299),
    [sym_block_comment] = STATE(1299),
    [ts_builtin_sym_end] = ACTIONS(4930),
    [sym_identifier] = ACTIONS(4932),
    [anon_sym_SEMI] = ACTIONS(4930),
    [anon_sym_macro] = ACTIONS(4932),
    [anon_sym_LPAREN] = ACTIONS(4930),
    [anon_sym_LBRACE] = ACTIONS(4930),
    [anon_sym_RBRACE] = ACTIONS(4930),
    [anon_sym_macro_rules] = ACTIONS(4932),
    [anon_sym_BANG] = ACTIONS(4930),
    [anon_sym_LBRACK] = ACTIONS(4930),
    [anon_sym_STAR] = ACTIONS(4930),
    [anon_sym_u8] = ACTIONS(4932),
    [anon_sym_i8] = ACTIONS(4932),
    [anon_sym_u16] = ACTIONS(4932),
    [anon_sym_i16] = ACTIONS(4932),
    [anon_sym_u32] = ACTIONS(4932),
    [anon_sym_i32] = ACTIONS(4932),
    [anon_sym_u64] = ACTIONS(4932),
    [anon_sym_i64] = ACTIONS(4932),
    [anon_sym_u128] = ACTIONS(4932),
    [anon_sym_i128] = ACTIONS(4932),
    [anon_sym_isize] = ACTIONS(4932),
    [anon_sym_usize] = ACTIONS(4932),
    [anon_sym_f32] = ACTIONS(4932),
    [anon_sym_f64] = ACTIONS(4932),
    [anon_sym_bool] = ACTIONS(4932),
    [anon_sym_str] = ACTIONS(4932),
    [anon_sym_char] = ACTIONS(4932),
    [anon_sym_DASH] = ACTIONS(4930),
    [anon_sym_AMP] = ACTIONS(4930),
    [anon_sym_PIPE] = ACTIONS(4930),
    [anon_sym_LT] = ACTIONS(4930),
    [anon_sym_DOT_DOT] = ACTIONS(4930),
    [anon_sym_COLON_COLON] = ACTIONS(4930),
    [anon_sym_POUND] = ACTIONS(4930),
    [anon_sym_SQUOTE] = ACTIONS(4932),
    [anon_sym_async] = ACTIONS(4932),
    [anon_sym_auto] = ACTIONS(4932),
    [anon_sym_await] = ACTIONS(4932),
    [anon_sym_become] = ACTIONS(4932),
    [anon_sym_break] = ACTIONS(4932),
    [anon_sym_const] = ACTIONS(4932),
    [anon_sym_continue] = ACTIONS(4932),
    [anon_sym_default] = ACTIONS(4932),
    [anon_sym_enum] = ACTIONS(4932),
    [anon_sym_fn] = ACTIONS(4932),
    [anon_sym_for] = ACTIONS(4932),
    [anon_sym_gen] = ACTIONS(4932),
    [anon_sym_if] = ACTIONS(4932),
    [anon_sym_impl] = ACTIONS(4932),
    [anon_sym_let] = ACTIONS(4932),
    [anon_sym_loop] = ACTIONS(4932),
    [anon_sym_match] = ACTIONS(4932),
    [anon_sym_mod] = ACTIONS(4932),
    [anon_sym_pub] = ACTIONS(4932),
    [anon_sym_return] = ACTIONS(4932),
    [anon_sym_raw] = ACTIONS(4932),
    [anon_sym_static] = ACTIONS(4932),
    [anon_sym_struct] = ACTIONS(4932),
    [anon_sym_trait] = ACTIONS(4932),
    [anon_sym_type] = ACTIONS(4932),
    [anon_sym_union] = ACTIONS(4932),
    [anon_sym_unsafe] = ACTIONS(4932),
    [anon_sym_use] = ACTIONS(4932),
    [anon_sym_while] = ACTIONS(4932),
    [anon_sym_extern] = ACTIONS(4932),
    [anon_sym_safe] = ACTIONS(4932),
    [anon_sym_yield] = ACTIONS(4932),
    [anon_sym_move] = ACTIONS(4932),
    [anon_sym_try] = ACTIONS(4932),
    [sym_integer_literal] = ACTIONS(4930),
    [aux_sym_string_literal_token1] = ACTIONS(4930),
    [sym_char_literal] = ACTIONS(4930),
    [anon_sym_true] = ACTIONS(4932),
    [anon_sym_false] = ACTIONS(4932),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4932),
    [sym_super] = ACTIONS(4932),
    [sym_crate] = ACTIONS(4932),
    [aux_sym_metavariable_token1] = ACTIONS(4930),
    [sym__raw_string_literal_start] = ACTIONS(4930),
    [sym_float_literal] = ACTIONS(4930),
    [sym__outer_block_doc_comment_start] = ACTIONS(4930),
    [sym__outer_line_doc_comment_start] = ACTIONS(4930),
  },
  [STATE(1300)] = {
    [sym_line_comment] = STATE(1300),
    [sym_block_comment] = STATE(1300),
    [ts_builtin_sym_end] = ACTIONS(4934),
    [sym_identifier] = ACTIONS(4936),
    [anon_sym_SEMI] = ACTIONS(4934),
    [anon_sym_macro] = ACTIONS(4936),
    [anon_sym_LPAREN] = ACTIONS(4934),
    [anon_sym_LBRACE] = ACTIONS(4934),
    [anon_sym_RBRACE] = ACTIONS(4934),
    [anon_sym_macro_rules] = ACTIONS(4936),
    [anon_sym_BANG] = ACTIONS(4934),
    [anon_sym_LBRACK] = ACTIONS(4934),
    [anon_sym_STAR] = ACTIONS(4934),
    [anon_sym_u8] = ACTIONS(4936),
    [anon_sym_i8] = ACTIONS(4936),
    [anon_sym_u16] = ACTIONS(4936),
    [anon_sym_i16] = ACTIONS(4936),
    [anon_sym_u32] = ACTIONS(4936),
    [anon_sym_i32] = ACTIONS(4936),
    [anon_sym_u64] = ACTIONS(4936),
    [anon_sym_i64] = ACTIONS(4936),
    [anon_sym_u128] = ACTIONS(4936),
    [anon_sym_i128] = ACTIONS(4936),
    [anon_sym_isize] = ACTIONS(4936),
    [anon_sym_usize] = ACTIONS(4936),
    [anon_sym_f32] = ACTIONS(4936),
    [anon_sym_f64] = ACTIONS(4936),
    [anon_sym_bool] = ACTIONS(4936),
    [anon_sym_str] = ACTIONS(4936),
    [anon_sym_char] = ACTIONS(4936),
    [anon_sym_DASH] = ACTIONS(4934),
    [anon_sym_AMP] = ACTIONS(4934),
    [anon_sym_PIPE] = ACTIONS(4934),
    [anon_sym_LT] = ACTIONS(4934),
    [anon_sym_DOT_DOT] = ACTIONS(4934),
    [anon_sym_COLON_COLON] = ACTIONS(4934),
    [anon_sym_POUND] = ACTIONS(4934),
    [anon_sym_SQUOTE] = ACTIONS(4936),
    [anon_sym_async] = ACTIONS(4936),
    [anon_sym_auto] = ACTIONS(4936),
    [anon_sym_await] = ACTIONS(4936),
    [anon_sym_become] = ACTIONS(4936),
    [anon_sym_break] = ACTIONS(4936),
    [anon_sym_const] = ACTIONS(4936),
    [anon_sym_continue] = ACTIONS(4936),
    [anon_sym_default] = ACTIONS(4936),
    [anon_sym_enum] = ACTIONS(4936),
    [anon_sym_fn] = ACTIONS(4936),
    [anon_sym_for] = ACTIONS(4936),
    [anon_sym_gen] = ACTIONS(4936),
    [anon_sym_if] = ACTIONS(4936),
    [anon_sym_impl] = ACTIONS(4936),
    [anon_sym_let] = ACTIONS(4936),
    [anon_sym_loop] = ACTIONS(4936),
    [anon_sym_match] = ACTIONS(4936),
    [anon_sym_mod] = ACTIONS(4936),
    [anon_sym_pub] = ACTIONS(4936),
    [anon_sym_return] = ACTIONS(4936),
    [anon_sym_raw] = ACTIONS(4936),
    [anon_sym_static] = ACTIONS(4936),
    [anon_sym_struct] = ACTIONS(4936),
    [anon_sym_trait] = ACTIONS(4936),
    [anon_sym_type] = ACTIONS(4936),
    [anon_sym_union] = ACTIONS(4936),
    [anon_sym_unsafe] = ACTIONS(4936),
    [anon_sym_use] = ACTIONS(4936),
    [anon_sym_while] = ACTIONS(4936),
    [anon_sym_extern] = ACTIONS(4936),
    [anon_sym_safe] = ACTIONS(4936),
    [anon_sym_yield] = ACTIONS(4936),
    [anon_sym_move] = ACTIONS(4936),
    [anon_sym_try] = ACTIONS(4936),
    [sym_integer_literal] = ACTIONS(4934),
    [aux_sym_string_literal_token1] = ACTIONS(4934),
    [sym_char_literal] = ACTIONS(4934),
    [anon_sym_true] = ACTIONS(4936),
    [anon_sym_false] = ACTIONS(4936),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4936),
    [sym_super] = ACTIONS(4936),
    [sym_crate] = ACTIONS(4936),
    [aux_sym_metavariable_token1] = ACTIONS(4934),
    [sym__raw_string_literal_start] = ACTIONS(4934),
    [sym_float_literal] = ACTIONS(4934),
    [sym__outer_block_doc_comment_start] = ACTIONS(4934),
    [sym__outer_line_doc_comment_start] = ACTIONS(4934),
  },
  [STATE(1301)] = {
    [sym_line_comment] = STATE(1301),
    [sym_block_comment] = STATE(1301),
    [ts_builtin_sym_end] = ACTIONS(4938),
    [sym_identifier] = ACTIONS(4940),
    [anon_sym_SEMI] = ACTIONS(4938),
    [anon_sym_macro] = ACTIONS(4940),
    [anon_sym_LPAREN] = ACTIONS(4938),
    [anon_sym_LBRACE] = ACTIONS(4938),
    [anon_sym_RBRACE] = ACTIONS(4938),
    [anon_sym_macro_rules] = ACTIONS(4940),
    [anon_sym_BANG] = ACTIONS(4938),
    [anon_sym_LBRACK] = ACTIONS(4938),
    [anon_sym_STAR] = ACTIONS(4938),
    [anon_sym_u8] = ACTIONS(4940),
    [anon_sym_i8] = ACTIONS(4940),
    [anon_sym_u16] = ACTIONS(4940),
    [anon_sym_i16] = ACTIONS(4940),
    [anon_sym_u32] = ACTIONS(4940),
    [anon_sym_i32] = ACTIONS(4940),
    [anon_sym_u64] = ACTIONS(4940),
    [anon_sym_i64] = ACTIONS(4940),
    [anon_sym_u128] = ACTIONS(4940),
    [anon_sym_i128] = ACTIONS(4940),
    [anon_sym_isize] = ACTIONS(4940),
    [anon_sym_usize] = ACTIONS(4940),
    [anon_sym_f32] = ACTIONS(4940),
    [anon_sym_f64] = ACTIONS(4940),
    [anon_sym_bool] = ACTIONS(4940),
    [anon_sym_str] = ACTIONS(4940),
    [anon_sym_char] = ACTIONS(4940),
    [anon_sym_DASH] = ACTIONS(4938),
    [anon_sym_AMP] = ACTIONS(4938),
    [anon_sym_PIPE] = ACTIONS(4938),
    [anon_sym_LT] = ACTIONS(4938),
    [anon_sym_DOT_DOT] = ACTIONS(4938),
    [anon_sym_COLON_COLON] = ACTIONS(4938),
    [anon_sym_POUND] = ACTIONS(4938),
    [anon_sym_SQUOTE] = ACTIONS(4940),
    [anon_sym_async] = ACTIONS(4940),
    [anon_sym_auto] = ACTIONS(4940),
    [anon_sym_await] = ACTIONS(4940),
    [anon_sym_become] = ACTIONS(4940),
    [anon_sym_break] = ACTIONS(4940),
    [anon_sym_const] = ACTIONS(4940),
    [anon_sym_continue] = ACTIONS(4940),
    [anon_sym_default] = ACTIONS(4940),
    [anon_sym_enum] = ACTIONS(4940),
    [anon_sym_fn] = ACTIONS(4940),
    [anon_sym_for] = ACTIONS(4940),
    [anon_sym_gen] = ACTIONS(4940),
    [anon_sym_if] = ACTIONS(4940),
    [anon_sym_impl] = ACTIONS(4940),
    [anon_sym_let] = ACTIONS(4940),
    [anon_sym_loop] = ACTIONS(4940),
    [anon_sym_match] = ACTIONS(4940),
    [anon_sym_mod] = ACTIONS(4940),
    [anon_sym_pub] = ACTIONS(4940),
    [anon_sym_return] = ACTIONS(4940),
    [anon_sym_raw] = ACTIONS(4940),
    [anon_sym_static] = ACTIONS(4940),
    [anon_sym_struct] = ACTIONS(4940),
    [anon_sym_trait] = ACTIONS(4940),
    [anon_sym_type] = ACTIONS(4940),
    [anon_sym_union] = ACTIONS(4940),
    [anon_sym_unsafe] = ACTIONS(4940),
    [anon_sym_use] = ACTIONS(4940),
    [anon_sym_while] = ACTIONS(4940),
    [anon_sym_extern] = ACTIONS(4940),
    [anon_sym_safe] = ACTIONS(4940),
    [anon_sym_yield] = ACTIONS(4940),
    [anon_sym_move] = ACTIONS(4940),
    [anon_sym_try] = ACTIONS(4940),
    [sym_integer_literal] = ACTIONS(4938),
    [aux_sym_string_literal_token1] = ACTIONS(4938),
    [sym_char_literal] = ACTIONS(4938),
    [anon_sym_true] = ACTIONS(4940),
    [anon_sym_false] = ACTIONS(4940),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4940),
    [sym_super] = ACTIONS(4940),
    [sym_crate] = ACTIONS(4940),
    [aux_sym_metavariable_token1] = ACTIONS(4938),
    [sym__raw_string_literal_start] = ACTIONS(4938),
    [sym_float_literal] = ACTIONS(4938),
    [sym__outer_block_doc_comment_start] = ACTIONS(4938),
    [sym__outer_line_doc_comment_start] = ACTIONS(4938),
  },
  [STATE(1302)] = {
    [sym_line_comment] = STATE(1302),
    [sym_block_comment] = STATE(1302),
    [ts_builtin_sym_end] = ACTIONS(4942),
    [sym_identifier] = ACTIONS(4944),
    [anon_sym_SEMI] = ACTIONS(4942),
    [anon_sym_macro] = ACTIONS(4944),
    [anon_sym_LPAREN] = ACTIONS(4942),
    [anon_sym_LBRACE] = ACTIONS(4942),
    [anon_sym_RBRACE] = ACTIONS(4942),
    [anon_sym_macro_rules] = ACTIONS(4944),
    [anon_sym_BANG] = ACTIONS(4942),
    [anon_sym_LBRACK] = ACTIONS(4942),
    [anon_sym_STAR] = ACTIONS(4942),
    [anon_sym_u8] = ACTIONS(4944),
    [anon_sym_i8] = ACTIONS(4944),
    [anon_sym_u16] = ACTIONS(4944),
    [anon_sym_i16] = ACTIONS(4944),
    [anon_sym_u32] = ACTIONS(4944),
    [anon_sym_i32] = ACTIONS(4944),
    [anon_sym_u64] = ACTIONS(4944),
    [anon_sym_i64] = ACTIONS(4944),
    [anon_sym_u128] = ACTIONS(4944),
    [anon_sym_i128] = ACTIONS(4944),
    [anon_sym_isize] = ACTIONS(4944),
    [anon_sym_usize] = ACTIONS(4944),
    [anon_sym_f32] = ACTIONS(4944),
    [anon_sym_f64] = ACTIONS(4944),
    [anon_sym_bool] = ACTIONS(4944),
    [anon_sym_str] = ACTIONS(4944),
    [anon_sym_char] = ACTIONS(4944),
    [anon_sym_DASH] = ACTIONS(4942),
    [anon_sym_AMP] = ACTIONS(4942),
    [anon_sym_PIPE] = ACTIONS(4942),
    [anon_sym_LT] = ACTIONS(4942),
    [anon_sym_DOT_DOT] = ACTIONS(4942),
    [anon_sym_COLON_COLON] = ACTIONS(4942),
    [anon_sym_POUND] = ACTIONS(4942),
    [anon_sym_SQUOTE] = ACTIONS(4944),
    [anon_sym_async] = ACTIONS(4944),
    [anon_sym_auto] = ACTIONS(4944),
    [anon_sym_await] = ACTIONS(4944),
    [anon_sym_become] = ACTIONS(4944),
    [anon_sym_break] = ACTIONS(4944),
    [anon_sym_const] = ACTIONS(4944),
    [anon_sym_continue] = ACTIONS(4944),
    [anon_sym_default] = ACTIONS(4944),
    [anon_sym_enum] = ACTIONS(4944),
    [anon_sym_fn] = ACTIONS(4944),
    [anon_sym_for] = ACTIONS(4944),
    [anon_sym_gen] = ACTIONS(4944),
    [anon_sym_if] = ACTIONS(4944),
    [anon_sym_impl] = ACTIONS(4944),
    [anon_sym_let] = ACTIONS(4944),
    [anon_sym_loop] = ACTIONS(4944),
    [anon_sym_match] = ACTIONS(4944),
    [anon_sym_mod] = ACTIONS(4944),
    [anon_sym_pub] = ACTIONS(4944),
    [anon_sym_return] = ACTIONS(4944),
    [anon_sym_raw] = ACTIONS(4944),
    [anon_sym_static] = ACTIONS(4944),
    [anon_sym_struct] = ACTIONS(4944),
    [anon_sym_trait] = ACTIONS(4944),
    [anon_sym_type] = ACTIONS(4944),
    [anon_sym_union] = ACTIONS(4944),
    [anon_sym_unsafe] = ACTIONS(4944),
    [anon_sym_use] = ACTIONS(4944),
    [anon_sym_while] = ACTIONS(4944),
    [anon_sym_extern] = ACTIONS(4944),
    [anon_sym_safe] = ACTIONS(4944),
    [anon_sym_yield] = ACTIONS(4944),
    [anon_sym_move] = ACTIONS(4944),
    [anon_sym_try] = ACTIONS(4944),
    [sym_integer_literal] = ACTIONS(4942),
    [aux_sym_string_literal_token1] = ACTIONS(4942),
    [sym_char_literal] = ACTIONS(4942),
    [anon_sym_true] = ACTIONS(4944),
    [anon_sym_false] = ACTIONS(4944),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4944),
    [sym_super] = ACTIONS(4944),
    [sym_crate] = ACTIONS(4944),
    [aux_sym_metavariable_token1] = ACTIONS(4942),
    [sym__raw_string_literal_start] = ACTIONS(4942),
    [sym_float_literal] = ACTIONS(4942),
    [sym__outer_block_doc_comment_start] = ACTIONS(4942),
    [sym__outer_line_doc_comment_start] = ACTIONS(4942),
  },
  [STATE(1303)] = {
    [sym_line_comment] = STATE(1303),
    [sym_block_comment] = STATE(1303),
    [ts_builtin_sym_end] = ACTIONS(4946),
    [sym_identifier] = ACTIONS(4948),
    [anon_sym_SEMI] = ACTIONS(4946),
    [anon_sym_macro] = ACTIONS(4948),
    [anon_sym_LPAREN] = ACTIONS(4946),
    [anon_sym_LBRACE] = ACTIONS(4946),
    [anon_sym_RBRACE] = ACTIONS(4946),
    [anon_sym_macro_rules] = ACTIONS(4948),
    [anon_sym_BANG] = ACTIONS(4946),
    [anon_sym_LBRACK] = ACTIONS(4946),
    [anon_sym_STAR] = ACTIONS(4946),
    [anon_sym_u8] = ACTIONS(4948),
    [anon_sym_i8] = ACTIONS(4948),
    [anon_sym_u16] = ACTIONS(4948),
    [anon_sym_i16] = ACTIONS(4948),
    [anon_sym_u32] = ACTIONS(4948),
    [anon_sym_i32] = ACTIONS(4948),
    [anon_sym_u64] = ACTIONS(4948),
    [anon_sym_i64] = ACTIONS(4948),
    [anon_sym_u128] = ACTIONS(4948),
    [anon_sym_i128] = ACTIONS(4948),
    [anon_sym_isize] = ACTIONS(4948),
    [anon_sym_usize] = ACTIONS(4948),
    [anon_sym_f32] = ACTIONS(4948),
    [anon_sym_f64] = ACTIONS(4948),
    [anon_sym_bool] = ACTIONS(4948),
    [anon_sym_str] = ACTIONS(4948),
    [anon_sym_char] = ACTIONS(4948),
    [anon_sym_DASH] = ACTIONS(4946),
    [anon_sym_AMP] = ACTIONS(4946),
    [anon_sym_PIPE] = ACTIONS(4946),
    [anon_sym_LT] = ACTIONS(4946),
    [anon_sym_DOT_DOT] = ACTIONS(4946),
    [anon_sym_COLON_COLON] = ACTIONS(4946),
    [anon_sym_POUND] = ACTIONS(4946),
    [anon_sym_SQUOTE] = ACTIONS(4948),
    [anon_sym_async] = ACTIONS(4948),
    [anon_sym_auto] = ACTIONS(4948),
    [anon_sym_await] = ACTIONS(4948),
    [anon_sym_become] = ACTIONS(4948),
    [anon_sym_break] = ACTIONS(4948),
    [anon_sym_const] = ACTIONS(4948),
    [anon_sym_continue] = ACTIONS(4948),
    [anon_sym_default] = ACTIONS(4948),
    [anon_sym_enum] = ACTIONS(4948),
    [anon_sym_fn] = ACTIONS(4948),
    [anon_sym_for] = ACTIONS(4948),
    [anon_sym_gen] = ACTIONS(4948),
    [anon_sym_if] = ACTIONS(4948),
    [anon_sym_impl] = ACTIONS(4948),
    [anon_sym_let] = ACTIONS(4948),
    [anon_sym_loop] = ACTIONS(4948),
    [anon_sym_match] = ACTIONS(4948),
    [anon_sym_mod] = ACTIONS(4948),
    [anon_sym_pub] = ACTIONS(4948),
    [anon_sym_return] = ACTIONS(4948),
    [anon_sym_raw] = ACTIONS(4948),
    [anon_sym_static] = ACTIONS(4948),
    [anon_sym_struct] = ACTIONS(4948),
    [anon_sym_trait] = ACTIONS(4948),
    [anon_sym_type] = ACTIONS(4948),
    [anon_sym_union] = ACTIONS(4948),
    [anon_sym_unsafe] = ACTIONS(4948),
    [anon_sym_use] = ACTIONS(4948),
    [anon_sym_while] = ACTIONS(4948),
    [anon_sym_extern] = ACTIONS(4948),
    [anon_sym_safe] = ACTIONS(4948),
    [anon_sym_yield] = ACTIONS(4948),
    [anon_sym_move] = ACTIONS(4948),
    [anon_sym_try] = ACTIONS(4948),
    [sym_integer_literal] = ACTIONS(4946),
    [aux_sym_string_literal_token1] = ACTIONS(4946),
    [sym_char_literal] = ACTIONS(4946),
    [anon_sym_true] = ACTIONS(4948),
    [anon_sym_false] = ACTIONS(4948),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4948),
    [sym_super] = ACTIONS(4948),
    [sym_crate] = ACTIONS(4948),
    [aux_sym_metavariable_token1] = ACTIONS(4946),
    [sym__raw_string_literal_start] = ACTIONS(4946),
    [sym_float_literal] = ACTIONS(4946),
    [sym__outer_block_doc_comment_start] = ACTIONS(4946),
    [sym__outer_line_doc_comment_start] = ACTIONS(4946),
  },
  [STATE(1304)] = {
    [sym_line_comment] = STATE(1304),
    [sym_block_comment] = STATE(1304),
    [ts_builtin_sym_end] = ACTIONS(4950),
    [sym_identifier] = ACTIONS(4952),
    [anon_sym_SEMI] = ACTIONS(4950),
    [anon_sym_macro] = ACTIONS(4952),
    [anon_sym_LPAREN] = ACTIONS(4950),
    [anon_sym_LBRACE] = ACTIONS(4950),
    [anon_sym_RBRACE] = ACTIONS(4950),
    [anon_sym_macro_rules] = ACTIONS(4952),
    [anon_sym_BANG] = ACTIONS(4950),
    [anon_sym_LBRACK] = ACTIONS(4950),
    [anon_sym_STAR] = ACTIONS(4950),
    [anon_sym_u8] = ACTIONS(4952),
    [anon_sym_i8] = ACTIONS(4952),
    [anon_sym_u16] = ACTIONS(4952),
    [anon_sym_i16] = ACTIONS(4952),
    [anon_sym_u32] = ACTIONS(4952),
    [anon_sym_i32] = ACTIONS(4952),
    [anon_sym_u64] = ACTIONS(4952),
    [anon_sym_i64] = ACTIONS(4952),
    [anon_sym_u128] = ACTIONS(4952),
    [anon_sym_i128] = ACTIONS(4952),
    [anon_sym_isize] = ACTIONS(4952),
    [anon_sym_usize] = ACTIONS(4952),
    [anon_sym_f32] = ACTIONS(4952),
    [anon_sym_f64] = ACTIONS(4952),
    [anon_sym_bool] = ACTIONS(4952),
    [anon_sym_str] = ACTIONS(4952),
    [anon_sym_char] = ACTIONS(4952),
    [anon_sym_DASH] = ACTIONS(4950),
    [anon_sym_AMP] = ACTIONS(4950),
    [anon_sym_PIPE] = ACTIONS(4950),
    [anon_sym_LT] = ACTIONS(4950),
    [anon_sym_DOT_DOT] = ACTIONS(4950),
    [anon_sym_COLON_COLON] = ACTIONS(4950),
    [anon_sym_POUND] = ACTIONS(4950),
    [anon_sym_SQUOTE] = ACTIONS(4952),
    [anon_sym_async] = ACTIONS(4952),
    [anon_sym_auto] = ACTIONS(4952),
    [anon_sym_await] = ACTIONS(4952),
    [anon_sym_become] = ACTIONS(4952),
    [anon_sym_break] = ACTIONS(4952),
    [anon_sym_const] = ACTIONS(4952),
    [anon_sym_continue] = ACTIONS(4952),
    [anon_sym_default] = ACTIONS(4952),
    [anon_sym_enum] = ACTIONS(4952),
    [anon_sym_fn] = ACTIONS(4952),
    [anon_sym_for] = ACTIONS(4952),
    [anon_sym_gen] = ACTIONS(4952),
    [anon_sym_if] = ACTIONS(4952),
    [anon_sym_impl] = ACTIONS(4952),
    [anon_sym_let] = ACTIONS(4952),
    [anon_sym_loop] = ACTIONS(4952),
    [anon_sym_match] = ACTIONS(4952),
    [anon_sym_mod] = ACTIONS(4952),
    [anon_sym_pub] = ACTIONS(4952),
    [anon_sym_return] = ACTIONS(4952),
    [anon_sym_raw] = ACTIONS(4952),
    [anon_sym_static] = ACTIONS(4952),
    [anon_sym_struct] = ACTIONS(4952),
    [anon_sym_trait] = ACTIONS(4952),
    [anon_sym_type] = ACTIONS(4952),
    [anon_sym_union] = ACTIONS(4952),
    [anon_sym_unsafe] = ACTIONS(4952),
    [anon_sym_use] = ACTIONS(4952),
    [anon_sym_while] = ACTIONS(4952),
    [anon_sym_extern] = ACTIONS(4952),
    [anon_sym_safe] = ACTIONS(4952),
    [anon_sym_yield] = ACTIONS(4952),
    [anon_sym_move] = ACTIONS(4952),
    [anon_sym_try] = ACTIONS(4952),
    [sym_integer_literal] = ACTIONS(4950),
    [aux_sym_string_literal_token1] = ACTIONS(4950),
    [sym_char_literal] = ACTIONS(4950),
    [anon_sym_true] = ACTIONS(4952),
    [anon_sym_false] = ACTIONS(4952),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4952),
    [sym_super] = ACTIONS(4952),
    [sym_crate] = ACTIONS(4952),
    [aux_sym_metavariable_token1] = ACTIONS(4950),
    [sym__raw_string_literal_start] = ACTIONS(4950),
    [sym_float_literal] = ACTIONS(4950),
    [sym__outer_block_doc_comment_start] = ACTIONS(4950),
    [sym__outer_line_doc_comment_start] = ACTIONS(4950),
  },
  [STATE(1305)] = {
    [sym_line_comment] = STATE(1305),
    [sym_block_comment] = STATE(1305),
    [ts_builtin_sym_end] = ACTIONS(4954),
    [sym_identifier] = ACTIONS(4956),
    [anon_sym_SEMI] = ACTIONS(4954),
    [anon_sym_macro] = ACTIONS(4956),
    [anon_sym_LPAREN] = ACTIONS(4954),
    [anon_sym_LBRACE] = ACTIONS(4954),
    [anon_sym_RBRACE] = ACTIONS(4954),
    [anon_sym_macro_rules] = ACTIONS(4956),
    [anon_sym_BANG] = ACTIONS(4954),
    [anon_sym_LBRACK] = ACTIONS(4954),
    [anon_sym_STAR] = ACTIONS(4954),
    [anon_sym_u8] = ACTIONS(4956),
    [anon_sym_i8] = ACTIONS(4956),
    [anon_sym_u16] = ACTIONS(4956),
    [anon_sym_i16] = ACTIONS(4956),
    [anon_sym_u32] = ACTIONS(4956),
    [anon_sym_i32] = ACTIONS(4956),
    [anon_sym_u64] = ACTIONS(4956),
    [anon_sym_i64] = ACTIONS(4956),
    [anon_sym_u128] = ACTIONS(4956),
    [anon_sym_i128] = ACTIONS(4956),
    [anon_sym_isize] = ACTIONS(4956),
    [anon_sym_usize] = ACTIONS(4956),
    [anon_sym_f32] = ACTIONS(4956),
    [anon_sym_f64] = ACTIONS(4956),
    [anon_sym_bool] = ACTIONS(4956),
    [anon_sym_str] = ACTIONS(4956),
    [anon_sym_char] = ACTIONS(4956),
    [anon_sym_DASH] = ACTIONS(4954),
    [anon_sym_AMP] = ACTIONS(4954),
    [anon_sym_PIPE] = ACTIONS(4954),
    [anon_sym_LT] = ACTIONS(4954),
    [anon_sym_DOT_DOT] = ACTIONS(4954),
    [anon_sym_COLON_COLON] = ACTIONS(4954),
    [anon_sym_POUND] = ACTIONS(4954),
    [anon_sym_SQUOTE] = ACTIONS(4956),
    [anon_sym_async] = ACTIONS(4956),
    [anon_sym_auto] = ACTIONS(4956),
    [anon_sym_await] = ACTIONS(4956),
    [anon_sym_become] = ACTIONS(4956),
    [anon_sym_break] = ACTIONS(4956),
    [anon_sym_const] = ACTIONS(4956),
    [anon_sym_continue] = ACTIONS(4956),
    [anon_sym_default] = ACTIONS(4956),
    [anon_sym_enum] = ACTIONS(4956),
    [anon_sym_fn] = ACTIONS(4956),
    [anon_sym_for] = ACTIONS(4956),
    [anon_sym_gen] = ACTIONS(4956),
    [anon_sym_if] = ACTIONS(4956),
    [anon_sym_impl] = ACTIONS(4956),
    [anon_sym_let] = ACTIONS(4956),
    [anon_sym_loop] = ACTIONS(4956),
    [anon_sym_match] = ACTIONS(4956),
    [anon_sym_mod] = ACTIONS(4956),
    [anon_sym_pub] = ACTIONS(4956),
    [anon_sym_return] = ACTIONS(4956),
    [anon_sym_raw] = ACTIONS(4956),
    [anon_sym_static] = ACTIONS(4956),
    [anon_sym_struct] = ACTIONS(4956),
    [anon_sym_trait] = ACTIONS(4956),
    [anon_sym_type] = ACTIONS(4956),
    [anon_sym_union] = ACTIONS(4956),
    [anon_sym_unsafe] = ACTIONS(4956),
    [anon_sym_use] = ACTIONS(4956),
    [anon_sym_while] = ACTIONS(4956),
    [anon_sym_extern] = ACTIONS(4956),
    [anon_sym_safe] = ACTIONS(4956),
    [anon_sym_yield] = ACTIONS(4956),
    [anon_sym_move] = ACTIONS(4956),
    [anon_sym_try] = ACTIONS(4956),
    [sym_integer_literal] = ACTIONS(4954),
    [aux_sym_string_literal_token1] = ACTIONS(4954),
    [sym_char_literal] = ACTIONS(4954),
    [anon_sym_true] = ACTIONS(4956),
    [anon_sym_false] = ACTIONS(4956),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4956),
    [sym_super] = ACTIONS(4956),
    [sym_crate] = ACTIONS(4956),
    [aux_sym_metavariable_token1] = ACTIONS(4954),
    [sym__raw_string_literal_start] = ACTIONS(4954),
    [sym_float_literal] = ACTIONS(4954),
    [sym__outer_block_doc_comment_start] = ACTIONS(4954),
    [sym__outer_line_doc_comment_start] = ACTIONS(4954),
  },
  [STATE(1306)] = {
    [sym_line_comment] = STATE(1306),
    [sym_block_comment] = STATE(1306),
    [ts_builtin_sym_end] = ACTIONS(4958),
    [sym_identifier] = ACTIONS(4960),
    [anon_sym_SEMI] = ACTIONS(4958),
    [anon_sym_macro] = ACTIONS(4960),
    [anon_sym_LPAREN] = ACTIONS(4958),
    [anon_sym_LBRACE] = ACTIONS(4958),
    [anon_sym_RBRACE] = ACTIONS(4958),
    [anon_sym_macro_rules] = ACTIONS(4960),
    [anon_sym_BANG] = ACTIONS(4958),
    [anon_sym_LBRACK] = ACTIONS(4958),
    [anon_sym_STAR] = ACTIONS(4958),
    [anon_sym_u8] = ACTIONS(4960),
    [anon_sym_i8] = ACTIONS(4960),
    [anon_sym_u16] = ACTIONS(4960),
    [anon_sym_i16] = ACTIONS(4960),
    [anon_sym_u32] = ACTIONS(4960),
    [anon_sym_i32] = ACTIONS(4960),
    [anon_sym_u64] = ACTIONS(4960),
    [anon_sym_i64] = ACTIONS(4960),
    [anon_sym_u128] = ACTIONS(4960),
    [anon_sym_i128] = ACTIONS(4960),
    [anon_sym_isize] = ACTIONS(4960),
    [anon_sym_usize] = ACTIONS(4960),
    [anon_sym_f32] = ACTIONS(4960),
    [anon_sym_f64] = ACTIONS(4960),
    [anon_sym_bool] = ACTIONS(4960),
    [anon_sym_str] = ACTIONS(4960),
    [anon_sym_char] = ACTIONS(4960),
    [anon_sym_DASH] = ACTIONS(4958),
    [anon_sym_AMP] = ACTIONS(4958),
    [anon_sym_PIPE] = ACTIONS(4958),
    [anon_sym_LT] = ACTIONS(4958),
    [anon_sym_DOT_DOT] = ACTIONS(4958),
    [anon_sym_COLON_COLON] = ACTIONS(4958),
    [anon_sym_POUND] = ACTIONS(4958),
    [anon_sym_SQUOTE] = ACTIONS(4960),
    [anon_sym_async] = ACTIONS(4960),
    [anon_sym_auto] = ACTIONS(4960),
    [anon_sym_await] = ACTIONS(4960),
    [anon_sym_become] = ACTIONS(4960),
    [anon_sym_break] = ACTIONS(4960),
    [anon_sym_const] = ACTIONS(4960),
    [anon_sym_continue] = ACTIONS(4960),
    [anon_sym_default] = ACTIONS(4960),
    [anon_sym_enum] = ACTIONS(4960),
    [anon_sym_fn] = ACTIONS(4960),
    [anon_sym_for] = ACTIONS(4960),
    [anon_sym_gen] = ACTIONS(4960),
    [anon_sym_if] = ACTIONS(4960),
    [anon_sym_impl] = ACTIONS(4960),
    [anon_sym_let] = ACTIONS(4960),
    [anon_sym_loop] = ACTIONS(4960),
    [anon_sym_match] = ACTIONS(4960),
    [anon_sym_mod] = ACTIONS(4960),
    [anon_sym_pub] = ACTIONS(4960),
    [anon_sym_return] = ACTIONS(4960),
    [anon_sym_raw] = ACTIONS(4960),
    [anon_sym_static] = ACTIONS(4960),
    [anon_sym_struct] = ACTIONS(4960),
    [anon_sym_trait] = ACTIONS(4960),
    [anon_sym_type] = ACTIONS(4960),
    [anon_sym_union] = ACTIONS(4960),
    [anon_sym_unsafe] = ACTIONS(4960),
    [anon_sym_use] = ACTIONS(4960),
    [anon_sym_while] = ACTIONS(4960),
    [anon_sym_extern] = ACTIONS(4960),
    [anon_sym_safe] = ACTIONS(4960),
    [anon_sym_yield] = ACTIONS(4960),
    [anon_sym_move] = ACTIONS(4960),
    [anon_sym_try] = ACTIONS(4960),
    [sym_integer_literal] = ACTIONS(4958),
    [aux_sym_string_literal_token1] = ACTIONS(4958),
    [sym_char_literal] = ACTIONS(4958),
    [anon_sym_true] = ACTIONS(4960),
    [anon_sym_false] = ACTIONS(4960),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4960),
    [sym_super] = ACTIONS(4960),
    [sym_crate] = ACTIONS(4960),
    [aux_sym_metavariable_token1] = ACTIONS(4958),
    [sym__raw_string_literal_start] = ACTIONS(4958),
    [sym_float_literal] = ACTIONS(4958),
    [sym__outer_block_doc_comment_start] = ACTIONS(4958),
    [sym__outer_line_doc_comment_start] = ACTIONS(4958),
  },
  [STATE(1307)] = {
    [sym_line_comment] = STATE(1307),
    [sym_block_comment] = STATE(1307),
    [ts_builtin_sym_end] = ACTIONS(4962),
    [sym_identifier] = ACTIONS(4964),
    [anon_sym_SEMI] = ACTIONS(4962),
    [anon_sym_macro] = ACTIONS(4964),
    [anon_sym_LPAREN] = ACTIONS(4962),
    [anon_sym_LBRACE] = ACTIONS(4962),
    [anon_sym_RBRACE] = ACTIONS(4962),
    [anon_sym_macro_rules] = ACTIONS(4964),
    [anon_sym_BANG] = ACTIONS(4962),
    [anon_sym_LBRACK] = ACTIONS(4962),
    [anon_sym_STAR] = ACTIONS(4962),
    [anon_sym_u8] = ACTIONS(4964),
    [anon_sym_i8] = ACTIONS(4964),
    [anon_sym_u16] = ACTIONS(4964),
    [anon_sym_i16] = ACTIONS(4964),
    [anon_sym_u32] = ACTIONS(4964),
    [anon_sym_i32] = ACTIONS(4964),
    [anon_sym_u64] = ACTIONS(4964),
    [anon_sym_i64] = ACTIONS(4964),
    [anon_sym_u128] = ACTIONS(4964),
    [anon_sym_i128] = ACTIONS(4964),
    [anon_sym_isize] = ACTIONS(4964),
    [anon_sym_usize] = ACTIONS(4964),
    [anon_sym_f32] = ACTIONS(4964),
    [anon_sym_f64] = ACTIONS(4964),
    [anon_sym_bool] = ACTIONS(4964),
    [anon_sym_str] = ACTIONS(4964),
    [anon_sym_char] = ACTIONS(4964),
    [anon_sym_DASH] = ACTIONS(4962),
    [anon_sym_AMP] = ACTIONS(4962),
    [anon_sym_PIPE] = ACTIONS(4962),
    [anon_sym_LT] = ACTIONS(4962),
    [anon_sym_DOT_DOT] = ACTIONS(4962),
    [anon_sym_COLON_COLON] = ACTIONS(4962),
    [anon_sym_POUND] = ACTIONS(4962),
    [anon_sym_SQUOTE] = ACTIONS(4964),
    [anon_sym_async] = ACTIONS(4964),
    [anon_sym_auto] = ACTIONS(4964),
    [anon_sym_await] = ACTIONS(4964),
    [anon_sym_become] = ACTIONS(4964),
    [anon_sym_break] = ACTIONS(4964),
    [anon_sym_const] = ACTIONS(4964),
    [anon_sym_continue] = ACTIONS(4964),
    [anon_sym_default] = ACTIONS(4964),
    [anon_sym_enum] = ACTIONS(4964),
    [anon_sym_fn] = ACTIONS(4964),
    [anon_sym_for] = ACTIONS(4964),
    [anon_sym_gen] = ACTIONS(4964),
    [anon_sym_if] = ACTIONS(4964),
    [anon_sym_impl] = ACTIONS(4964),
    [anon_sym_let] = ACTIONS(4964),
    [anon_sym_loop] = ACTIONS(4964),
    [anon_sym_match] = ACTIONS(4964),
    [anon_sym_mod] = ACTIONS(4964),
    [anon_sym_pub] = ACTIONS(4964),
    [anon_sym_return] = ACTIONS(4964),
    [anon_sym_raw] = ACTIONS(4964),
    [anon_sym_static] = ACTIONS(4964),
    [anon_sym_struct] = ACTIONS(4964),
    [anon_sym_trait] = ACTIONS(4964),
    [anon_sym_type] = ACTIONS(4964),
    [anon_sym_union] = ACTIONS(4964),
    [anon_sym_unsafe] = ACTIONS(4964),
    [anon_sym_use] = ACTIONS(4964),
    [anon_sym_while] = ACTIONS(4964),
    [anon_sym_extern] = ACTIONS(4964),
    [anon_sym_safe] = ACTIONS(4964),
    [anon_sym_yield] = ACTIONS(4964),
    [anon_sym_move] = ACTIONS(4964),
    [anon_sym_try] = ACTIONS(4964),
    [sym_integer_literal] = ACTIONS(4962),
    [aux_sym_string_literal_token1] = ACTIONS(4962),
    [sym_char_literal] = ACTIONS(4962),
    [anon_sym_true] = ACTIONS(4964),
    [anon_sym_false] = ACTIONS(4964),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4964),
    [sym_super] = ACTIONS(4964),
    [sym_crate] = ACTIONS(4964),
    [aux_sym_metavariable_token1] = ACTIONS(4962),
    [sym__raw_string_literal_start] = ACTIONS(4962),
    [sym_float_literal] = ACTIONS(4962),
    [sym__outer_block_doc_comment_start] = ACTIONS(4962),
    [sym__outer_line_doc_comment_start] = ACTIONS(4962),
  },
  [STATE(1308)] = {
    [sym_line_comment] = STATE(1308),
    [sym_block_comment] = STATE(1308),
    [ts_builtin_sym_end] = ACTIONS(4966),
    [sym_identifier] = ACTIONS(4968),
    [anon_sym_SEMI] = ACTIONS(4966),
    [anon_sym_macro] = ACTIONS(4968),
    [anon_sym_LPAREN] = ACTIONS(4966),
    [anon_sym_LBRACE] = ACTIONS(4966),
    [anon_sym_RBRACE] = ACTIONS(4966),
    [anon_sym_macro_rules] = ACTIONS(4968),
    [anon_sym_BANG] = ACTIONS(4966),
    [anon_sym_LBRACK] = ACTIONS(4966),
    [anon_sym_STAR] = ACTIONS(4966),
    [anon_sym_u8] = ACTIONS(4968),
    [anon_sym_i8] = ACTIONS(4968),
    [anon_sym_u16] = ACTIONS(4968),
    [anon_sym_i16] = ACTIONS(4968),
    [anon_sym_u32] = ACTIONS(4968),
    [anon_sym_i32] = ACTIONS(4968),
    [anon_sym_u64] = ACTIONS(4968),
    [anon_sym_i64] = ACTIONS(4968),
    [anon_sym_u128] = ACTIONS(4968),
    [anon_sym_i128] = ACTIONS(4968),
    [anon_sym_isize] = ACTIONS(4968),
    [anon_sym_usize] = ACTIONS(4968),
    [anon_sym_f32] = ACTIONS(4968),
    [anon_sym_f64] = ACTIONS(4968),
    [anon_sym_bool] = ACTIONS(4968),
    [anon_sym_str] = ACTIONS(4968),
    [anon_sym_char] = ACTIONS(4968),
    [anon_sym_DASH] = ACTIONS(4966),
    [anon_sym_AMP] = ACTIONS(4966),
    [anon_sym_PIPE] = ACTIONS(4966),
    [anon_sym_LT] = ACTIONS(4966),
    [anon_sym_DOT_DOT] = ACTIONS(4966),
    [anon_sym_COLON_COLON] = ACTIONS(4966),
    [anon_sym_POUND] = ACTIONS(4966),
    [anon_sym_SQUOTE] = ACTIONS(4968),
    [anon_sym_async] = ACTIONS(4968),
    [anon_sym_auto] = ACTIONS(4968),
    [anon_sym_await] = ACTIONS(4968),
    [anon_sym_become] = ACTIONS(4968),
    [anon_sym_break] = ACTIONS(4968),
    [anon_sym_const] = ACTIONS(4968),
    [anon_sym_continue] = ACTIONS(4968),
    [anon_sym_default] = ACTIONS(4968),
    [anon_sym_enum] = ACTIONS(4968),
    [anon_sym_fn] = ACTIONS(4968),
    [anon_sym_for] = ACTIONS(4968),
    [anon_sym_gen] = ACTIONS(4968),
    [anon_sym_if] = ACTIONS(4968),
    [anon_sym_impl] = ACTIONS(4968),
    [anon_sym_let] = ACTIONS(4968),
    [anon_sym_loop] = ACTIONS(4968),
    [anon_sym_match] = ACTIONS(4968),
    [anon_sym_mod] = ACTIONS(4968),
    [anon_sym_pub] = ACTIONS(4968),
    [anon_sym_return] = ACTIONS(4968),
    [anon_sym_raw] = ACTIONS(4968),
    [anon_sym_static] = ACTIONS(4968),
    [anon_sym_struct] = ACTIONS(4968),
    [anon_sym_trait] = ACTIONS(4968),
    [anon_sym_type] = ACTIONS(4968),
    [anon_sym_union] = ACTIONS(4968),
    [anon_sym_unsafe] = ACTIONS(4968),
    [anon_sym_use] = ACTIONS(4968),
    [anon_sym_while] = ACTIONS(4968),
    [anon_sym_extern] = ACTIONS(4968),
    [anon_sym_safe] = ACTIONS(4968),
    [anon_sym_yield] = ACTIONS(4968),
    [anon_sym_move] = ACTIONS(4968),
    [anon_sym_try] = ACTIONS(4968),
    [sym_integer_literal] = ACTIONS(4966),
    [aux_sym_string_literal_token1] = ACTIONS(4966),
    [sym_char_literal] = ACTIONS(4966),
    [anon_sym_true] = ACTIONS(4968),
    [anon_sym_false] = ACTIONS(4968),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4968),
    [sym_super] = ACTIONS(4968),
    [sym_crate] = ACTIONS(4968),
    [aux_sym_metavariable_token1] = ACTIONS(4966),
    [sym__raw_string_literal_start] = ACTIONS(4966),
    [sym_float_literal] = ACTIONS(4966),
    [sym__outer_block_doc_comment_start] = ACTIONS(4966),
    [sym__outer_line_doc_comment_start] = ACTIONS(4966),
  },
  [STATE(1309)] = {
    [sym_line_comment] = STATE(1309),
    [sym_block_comment] = STATE(1309),
    [ts_builtin_sym_end] = ACTIONS(4970),
    [sym_identifier] = ACTIONS(4972),
    [anon_sym_SEMI] = ACTIONS(4970),
    [anon_sym_macro] = ACTIONS(4972),
    [anon_sym_LPAREN] = ACTIONS(4970),
    [anon_sym_LBRACE] = ACTIONS(4970),
    [anon_sym_RBRACE] = ACTIONS(4970),
    [anon_sym_macro_rules] = ACTIONS(4972),
    [anon_sym_BANG] = ACTIONS(4970),
    [anon_sym_LBRACK] = ACTIONS(4970),
    [anon_sym_STAR] = ACTIONS(4970),
    [anon_sym_u8] = ACTIONS(4972),
    [anon_sym_i8] = ACTIONS(4972),
    [anon_sym_u16] = ACTIONS(4972),
    [anon_sym_i16] = ACTIONS(4972),
    [anon_sym_u32] = ACTIONS(4972),
    [anon_sym_i32] = ACTIONS(4972),
    [anon_sym_u64] = ACTIONS(4972),
    [anon_sym_i64] = ACTIONS(4972),
    [anon_sym_u128] = ACTIONS(4972),
    [anon_sym_i128] = ACTIONS(4972),
    [anon_sym_isize] = ACTIONS(4972),
    [anon_sym_usize] = ACTIONS(4972),
    [anon_sym_f32] = ACTIONS(4972),
    [anon_sym_f64] = ACTIONS(4972),
    [anon_sym_bool] = ACTIONS(4972),
    [anon_sym_str] = ACTIONS(4972),
    [anon_sym_char] = ACTIONS(4972),
    [anon_sym_DASH] = ACTIONS(4970),
    [anon_sym_AMP] = ACTIONS(4970),
    [anon_sym_PIPE] = ACTIONS(4970),
    [anon_sym_LT] = ACTIONS(4970),
    [anon_sym_DOT_DOT] = ACTIONS(4970),
    [anon_sym_COLON_COLON] = ACTIONS(4970),
    [anon_sym_POUND] = ACTIONS(4970),
    [anon_sym_SQUOTE] = ACTIONS(4972),
    [anon_sym_async] = ACTIONS(4972),
    [anon_sym_auto] = ACTIONS(4972),
    [anon_sym_await] = ACTIONS(4972),
    [anon_sym_become] = ACTIONS(4972),
    [anon_sym_break] = ACTIONS(4972),
    [anon_sym_const] = ACTIONS(4972),
    [anon_sym_continue] = ACTIONS(4972),
    [anon_sym_default] = ACTIONS(4972),
    [anon_sym_enum] = ACTIONS(4972),
    [anon_sym_fn] = ACTIONS(4972),
    [anon_sym_for] = ACTIONS(4972),
    [anon_sym_gen] = ACTIONS(4972),
    [anon_sym_if] = ACTIONS(4972),
    [anon_sym_impl] = ACTIONS(4972),
    [anon_sym_let] = ACTIONS(4972),
    [anon_sym_loop] = ACTIONS(4972),
    [anon_sym_match] = ACTIONS(4972),
    [anon_sym_mod] = ACTIONS(4972),
    [anon_sym_pub] = ACTIONS(4972),
    [anon_sym_return] = ACTIONS(4972),
    [anon_sym_raw] = ACTIONS(4972),
    [anon_sym_static] = ACTIONS(4972),
    [anon_sym_struct] = ACTIONS(4972),
    [anon_sym_trait] = ACTIONS(4972),
    [anon_sym_type] = ACTIONS(4972),
    [anon_sym_union] = ACTIONS(4972),
    [anon_sym_unsafe] = ACTIONS(4972),
    [anon_sym_use] = ACTIONS(4972),
    [anon_sym_while] = ACTIONS(4972),
    [anon_sym_extern] = ACTIONS(4972),
    [anon_sym_safe] = ACTIONS(4972),
    [anon_sym_yield] = ACTIONS(4972),
    [anon_sym_move] = ACTIONS(4972),
    [anon_sym_try] = ACTIONS(4972),
    [sym_integer_literal] = ACTIONS(4970),
    [aux_sym_string_literal_token1] = ACTIONS(4970),
    [sym_char_literal] = ACTIONS(4970),
    [anon_sym_true] = ACTIONS(4972),
    [anon_sym_false] = ACTIONS(4972),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4972),
    [sym_super] = ACTIONS(4972),
    [sym_crate] = ACTIONS(4972),
    [aux_sym_metavariable_token1] = ACTIONS(4970),
    [sym__raw_string_literal_start] = ACTIONS(4970),
    [sym_float_literal] = ACTIONS(4970),
    [sym__outer_block_doc_comment_start] = ACTIONS(4970),
    [sym__outer_line_doc_comment_start] = ACTIONS(4970),
  },
  [STATE(1310)] = {
    [sym_line_comment] = STATE(1310),
    [sym_block_comment] = STATE(1310),
    [ts_builtin_sym_end] = ACTIONS(4974),
    [sym_identifier] = ACTIONS(4976),
    [anon_sym_SEMI] = ACTIONS(4974),
    [anon_sym_macro] = ACTIONS(4976),
    [anon_sym_LPAREN] = ACTIONS(4974),
    [anon_sym_LBRACE] = ACTIONS(4974),
    [anon_sym_RBRACE] = ACTIONS(4974),
    [anon_sym_macro_rules] = ACTIONS(4976),
    [anon_sym_BANG] = ACTIONS(4974),
    [anon_sym_LBRACK] = ACTIONS(4974),
    [anon_sym_STAR] = ACTIONS(4974),
    [anon_sym_u8] = ACTIONS(4976),
    [anon_sym_i8] = ACTIONS(4976),
    [anon_sym_u16] = ACTIONS(4976),
    [anon_sym_i16] = ACTIONS(4976),
    [anon_sym_u32] = ACTIONS(4976),
    [anon_sym_i32] = ACTIONS(4976),
    [anon_sym_u64] = ACTIONS(4976),
    [anon_sym_i64] = ACTIONS(4976),
    [anon_sym_u128] = ACTIONS(4976),
    [anon_sym_i128] = ACTIONS(4976),
    [anon_sym_isize] = ACTIONS(4976),
    [anon_sym_usize] = ACTIONS(4976),
    [anon_sym_f32] = ACTIONS(4976),
    [anon_sym_f64] = ACTIONS(4976),
    [anon_sym_bool] = ACTIONS(4976),
    [anon_sym_str] = ACTIONS(4976),
    [anon_sym_char] = ACTIONS(4976),
    [anon_sym_DASH] = ACTIONS(4974),
    [anon_sym_AMP] = ACTIONS(4974),
    [anon_sym_PIPE] = ACTIONS(4974),
    [anon_sym_LT] = ACTIONS(4974),
    [anon_sym_DOT_DOT] = ACTIONS(4974),
    [anon_sym_COLON_COLON] = ACTIONS(4974),
    [anon_sym_POUND] = ACTIONS(4974),
    [anon_sym_SQUOTE] = ACTIONS(4976),
    [anon_sym_async] = ACTIONS(4976),
    [anon_sym_auto] = ACTIONS(4976),
    [anon_sym_await] = ACTIONS(4976),
    [anon_sym_become] = ACTIONS(4976),
    [anon_sym_break] = ACTIONS(4976),
    [anon_sym_const] = ACTIONS(4976),
    [anon_sym_continue] = ACTIONS(4976),
    [anon_sym_default] = ACTIONS(4976),
    [anon_sym_enum] = ACTIONS(4976),
    [anon_sym_fn] = ACTIONS(4976),
    [anon_sym_for] = ACTIONS(4976),
    [anon_sym_gen] = ACTIONS(4976),
    [anon_sym_if] = ACTIONS(4976),
    [anon_sym_impl] = ACTIONS(4976),
    [anon_sym_let] = ACTIONS(4976),
    [anon_sym_loop] = ACTIONS(4976),
    [anon_sym_match] = ACTIONS(4976),
    [anon_sym_mod] = ACTIONS(4976),
    [anon_sym_pub] = ACTIONS(4976),
    [anon_sym_return] = ACTIONS(4976),
    [anon_sym_raw] = ACTIONS(4976),
    [anon_sym_static] = ACTIONS(4976),
    [anon_sym_struct] = ACTIONS(4976),
    [anon_sym_trait] = ACTIONS(4976),
    [anon_sym_type] = ACTIONS(4976),
    [anon_sym_union] = ACTIONS(4976),
    [anon_sym_unsafe] = ACTIONS(4976),
    [anon_sym_use] = ACTIONS(4976),
    [anon_sym_while] = ACTIONS(4976),
    [anon_sym_extern] = ACTIONS(4976),
    [anon_sym_safe] = ACTIONS(4976),
    [anon_sym_yield] = ACTIONS(4976),
    [anon_sym_move] = ACTIONS(4976),
    [anon_sym_try] = ACTIONS(4976),
    [sym_integer_literal] = ACTIONS(4974),
    [aux_sym_string_literal_token1] = ACTIONS(4974),
    [sym_char_literal] = ACTIONS(4974),
    [anon_sym_true] = ACTIONS(4976),
    [anon_sym_false] = ACTIONS(4976),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4976),
    [sym_super] = ACTIONS(4976),
    [sym_crate] = ACTIONS(4976),
    [aux_sym_metavariable_token1] = ACTIONS(4974),
    [sym__raw_string_literal_start] = ACTIONS(4974),
    [sym_float_literal] = ACTIONS(4974),
    [sym__outer_block_doc_comment_start] = ACTIONS(4974),
    [sym__outer_line_doc_comment_start] = ACTIONS(4974),
  },
  [STATE(1311)] = {
    [sym_line_comment] = STATE(1311),
    [sym_block_comment] = STATE(1311),
    [ts_builtin_sym_end] = ACTIONS(4978),
    [sym_identifier] = ACTIONS(4980),
    [anon_sym_SEMI] = ACTIONS(4978),
    [anon_sym_macro] = ACTIONS(4980),
    [anon_sym_LPAREN] = ACTIONS(4978),
    [anon_sym_LBRACE] = ACTIONS(4978),
    [anon_sym_RBRACE] = ACTIONS(4978),
    [anon_sym_macro_rules] = ACTIONS(4980),
    [anon_sym_BANG] = ACTIONS(4978),
    [anon_sym_LBRACK] = ACTIONS(4978),
    [anon_sym_STAR] = ACTIONS(4978),
    [anon_sym_u8] = ACTIONS(4980),
    [anon_sym_i8] = ACTIONS(4980),
    [anon_sym_u16] = ACTIONS(4980),
    [anon_sym_i16] = ACTIONS(4980),
    [anon_sym_u32] = ACTIONS(4980),
    [anon_sym_i32] = ACTIONS(4980),
    [anon_sym_u64] = ACTIONS(4980),
    [anon_sym_i64] = ACTIONS(4980),
    [anon_sym_u128] = ACTIONS(4980),
    [anon_sym_i128] = ACTIONS(4980),
    [anon_sym_isize] = ACTIONS(4980),
    [anon_sym_usize] = ACTIONS(4980),
    [anon_sym_f32] = ACTIONS(4980),
    [anon_sym_f64] = ACTIONS(4980),
    [anon_sym_bool] = ACTIONS(4980),
    [anon_sym_str] = ACTIONS(4980),
    [anon_sym_char] = ACTIONS(4980),
    [anon_sym_DASH] = ACTIONS(4978),
    [anon_sym_AMP] = ACTIONS(4978),
    [anon_sym_PIPE] = ACTIONS(4978),
    [anon_sym_LT] = ACTIONS(4978),
    [anon_sym_DOT_DOT] = ACTIONS(4978),
    [anon_sym_COLON_COLON] = ACTIONS(4978),
    [anon_sym_POUND] = ACTIONS(4978),
    [anon_sym_SQUOTE] = ACTIONS(4980),
    [anon_sym_async] = ACTIONS(4980),
    [anon_sym_auto] = ACTIONS(4980),
    [anon_sym_await] = ACTIONS(4980),
    [anon_sym_become] = ACTIONS(4980),
    [anon_sym_break] = ACTIONS(4980),
    [anon_sym_const] = ACTIONS(4980),
    [anon_sym_continue] = ACTIONS(4980),
    [anon_sym_default] = ACTIONS(4980),
    [anon_sym_enum] = ACTIONS(4980),
    [anon_sym_fn] = ACTIONS(4980),
    [anon_sym_for] = ACTIONS(4980),
    [anon_sym_gen] = ACTIONS(4980),
    [anon_sym_if] = ACTIONS(4980),
    [anon_sym_impl] = ACTIONS(4980),
    [anon_sym_let] = ACTIONS(4980),
    [anon_sym_loop] = ACTIONS(4980),
    [anon_sym_match] = ACTIONS(4980),
    [anon_sym_mod] = ACTIONS(4980),
    [anon_sym_pub] = ACTIONS(4980),
    [anon_sym_return] = ACTIONS(4980),
    [anon_sym_raw] = ACTIONS(4980),
    [anon_sym_static] = ACTIONS(4980),
    [anon_sym_struct] = ACTIONS(4980),
    [anon_sym_trait] = ACTIONS(4980),
    [anon_sym_type] = ACTIONS(4980),
    [anon_sym_union] = ACTIONS(4980),
    [anon_sym_unsafe] = ACTIONS(4980),
    [anon_sym_use] = ACTIONS(4980),
    [anon_sym_while] = ACTIONS(4980),
    [anon_sym_extern] = ACTIONS(4980),
    [anon_sym_safe] = ACTIONS(4980),
    [anon_sym_yield] = ACTIONS(4980),
    [anon_sym_move] = ACTIONS(4980),
    [anon_sym_try] = ACTIONS(4980),
    [sym_integer_literal] = ACTIONS(4978),
    [aux_sym_string_literal_token1] = ACTIONS(4978),
    [sym_char_literal] = ACTIONS(4978),
    [anon_sym_true] = ACTIONS(4980),
    [anon_sym_false] = ACTIONS(4980),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4980),
    [sym_super] = ACTIONS(4980),
    [sym_crate] = ACTIONS(4980),
    [aux_sym_metavariable_token1] = ACTIONS(4978),
    [sym__raw_string_literal_start] = ACTIONS(4978),
    [sym_float_literal] = ACTIONS(4978),
    [sym__outer_block_doc_comment_start] = ACTIONS(4978),
    [sym__outer_line_doc_comment_start] = ACTIONS(4978),
  },
  [STATE(1312)] = {
    [sym_line_comment] = STATE(1312),
    [sym_block_comment] = STATE(1312),
    [ts_builtin_sym_end] = ACTIONS(4982),
    [sym_identifier] = ACTIONS(4984),
    [anon_sym_SEMI] = ACTIONS(4982),
    [anon_sym_macro] = ACTIONS(4984),
    [anon_sym_LPAREN] = ACTIONS(4982),
    [anon_sym_LBRACE] = ACTIONS(4982),
    [anon_sym_RBRACE] = ACTIONS(4982),
    [anon_sym_macro_rules] = ACTIONS(4984),
    [anon_sym_BANG] = ACTIONS(4982),
    [anon_sym_LBRACK] = ACTIONS(4982),
    [anon_sym_STAR] = ACTIONS(4982),
    [anon_sym_u8] = ACTIONS(4984),
    [anon_sym_i8] = ACTIONS(4984),
    [anon_sym_u16] = ACTIONS(4984),
    [anon_sym_i16] = ACTIONS(4984),
    [anon_sym_u32] = ACTIONS(4984),
    [anon_sym_i32] = ACTIONS(4984),
    [anon_sym_u64] = ACTIONS(4984),
    [anon_sym_i64] = ACTIONS(4984),
    [anon_sym_u128] = ACTIONS(4984),
    [anon_sym_i128] = ACTIONS(4984),
    [anon_sym_isize] = ACTIONS(4984),
    [anon_sym_usize] = ACTIONS(4984),
    [anon_sym_f32] = ACTIONS(4984),
    [anon_sym_f64] = ACTIONS(4984),
    [anon_sym_bool] = ACTIONS(4984),
    [anon_sym_str] = ACTIONS(4984),
    [anon_sym_char] = ACTIONS(4984),
    [anon_sym_DASH] = ACTIONS(4982),
    [anon_sym_AMP] = ACTIONS(4982),
    [anon_sym_PIPE] = ACTIONS(4982),
    [anon_sym_LT] = ACTIONS(4982),
    [anon_sym_DOT_DOT] = ACTIONS(4982),
    [anon_sym_COLON_COLON] = ACTIONS(4982),
    [anon_sym_POUND] = ACTIONS(4982),
    [anon_sym_SQUOTE] = ACTIONS(4984),
    [anon_sym_async] = ACTIONS(4984),
    [anon_sym_auto] = ACTIONS(4984),
    [anon_sym_await] = ACTIONS(4984),
    [anon_sym_become] = ACTIONS(4984),
    [anon_sym_break] = ACTIONS(4984),
    [anon_sym_const] = ACTIONS(4984),
    [anon_sym_continue] = ACTIONS(4984),
    [anon_sym_default] = ACTIONS(4984),
    [anon_sym_enum] = ACTIONS(4984),
    [anon_sym_fn] = ACTIONS(4984),
    [anon_sym_for] = ACTIONS(4984),
    [anon_sym_gen] = ACTIONS(4984),
    [anon_sym_if] = ACTIONS(4984),
    [anon_sym_impl] = ACTIONS(4984),
    [anon_sym_let] = ACTIONS(4984),
    [anon_sym_loop] = ACTIONS(4984),
    [anon_sym_match] = ACTIONS(4984),
    [anon_sym_mod] = ACTIONS(4984),
    [anon_sym_pub] = ACTIONS(4984),
    [anon_sym_return] = ACTIONS(4984),
    [anon_sym_raw] = ACTIONS(4984),
    [anon_sym_static] = ACTIONS(4984),
    [anon_sym_struct] = ACTIONS(4984),
    [anon_sym_trait] = ACTIONS(4984),
    [anon_sym_type] = ACTIONS(4984),
    [anon_sym_union] = ACTIONS(4984),
    [anon_sym_unsafe] = ACTIONS(4984),
    [anon_sym_use] = ACTIONS(4984),
    [anon_sym_while] = ACTIONS(4984),
    [anon_sym_extern] = ACTIONS(4984),
    [anon_sym_safe] = ACTIONS(4984),
    [anon_sym_yield] = ACTIONS(4984),
    [anon_sym_move] = ACTIONS(4984),
    [anon_sym_try] = ACTIONS(4984),
    [sym_integer_literal] = ACTIONS(4982),
    [aux_sym_string_literal_token1] = ACTIONS(4982),
    [sym_char_literal] = ACTIONS(4982),
    [anon_sym_true] = ACTIONS(4984),
    [anon_sym_false] = ACTIONS(4984),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4984),
    [sym_super] = ACTIONS(4984),
    [sym_crate] = ACTIONS(4984),
    [aux_sym_metavariable_token1] = ACTIONS(4982),
    [sym__raw_string_literal_start] = ACTIONS(4982),
    [sym_float_literal] = ACTIONS(4982),
    [sym__outer_block_doc_comment_start] = ACTIONS(4982),
    [sym__outer_line_doc_comment_start] = ACTIONS(4982),
  },
  [STATE(1313)] = {
    [sym_line_comment] = STATE(1313),
    [sym_block_comment] = STATE(1313),
    [ts_builtin_sym_end] = ACTIONS(4986),
    [sym_identifier] = ACTIONS(4988),
    [anon_sym_SEMI] = ACTIONS(4986),
    [anon_sym_macro] = ACTIONS(4988),
    [anon_sym_LPAREN] = ACTIONS(4986),
    [anon_sym_LBRACE] = ACTIONS(4986),
    [anon_sym_RBRACE] = ACTIONS(4986),
    [anon_sym_macro_rules] = ACTIONS(4988),
    [anon_sym_BANG] = ACTIONS(4986),
    [anon_sym_LBRACK] = ACTIONS(4986),
    [anon_sym_STAR] = ACTIONS(4986),
    [anon_sym_u8] = ACTIONS(4988),
    [anon_sym_i8] = ACTIONS(4988),
    [anon_sym_u16] = ACTIONS(4988),
    [anon_sym_i16] = ACTIONS(4988),
    [anon_sym_u32] = ACTIONS(4988),
    [anon_sym_i32] = ACTIONS(4988),
    [anon_sym_u64] = ACTIONS(4988),
    [anon_sym_i64] = ACTIONS(4988),
    [anon_sym_u128] = ACTIONS(4988),
    [anon_sym_i128] = ACTIONS(4988),
    [anon_sym_isize] = ACTIONS(4988),
    [anon_sym_usize] = ACTIONS(4988),
    [anon_sym_f32] = ACTIONS(4988),
    [anon_sym_f64] = ACTIONS(4988),
    [anon_sym_bool] = ACTIONS(4988),
    [anon_sym_str] = ACTIONS(4988),
    [anon_sym_char] = ACTIONS(4988),
    [anon_sym_DASH] = ACTIONS(4986),
    [anon_sym_AMP] = ACTIONS(4986),
    [anon_sym_PIPE] = ACTIONS(4986),
    [anon_sym_LT] = ACTIONS(4986),
    [anon_sym_DOT_DOT] = ACTIONS(4986),
    [anon_sym_COLON_COLON] = ACTIONS(4986),
    [anon_sym_POUND] = ACTIONS(4986),
    [anon_sym_SQUOTE] = ACTIONS(4988),
    [anon_sym_async] = ACTIONS(4988),
    [anon_sym_auto] = ACTIONS(4988),
    [anon_sym_await] = ACTIONS(4988),
    [anon_sym_become] = ACTIONS(4988),
    [anon_sym_break] = ACTIONS(4988),
    [anon_sym_const] = ACTIONS(4988),
    [anon_sym_continue] = ACTIONS(4988),
    [anon_sym_default] = ACTIONS(4988),
    [anon_sym_enum] = ACTIONS(4988),
    [anon_sym_fn] = ACTIONS(4988),
    [anon_sym_for] = ACTIONS(4988),
    [anon_sym_gen] = ACTIONS(4988),
    [anon_sym_if] = ACTIONS(4988),
    [anon_sym_impl] = ACTIONS(4988),
    [anon_sym_let] = ACTIONS(4988),
    [anon_sym_loop] = ACTIONS(4988),
    [anon_sym_match] = ACTIONS(4988),
    [anon_sym_mod] = ACTIONS(4988),
    [anon_sym_pub] = ACTIONS(4988),
    [anon_sym_return] = ACTIONS(4988),
    [anon_sym_raw] = ACTIONS(4988),
    [anon_sym_static] = ACTIONS(4988),
    [anon_sym_struct] = ACTIONS(4988),
    [anon_sym_trait] = ACTIONS(4988),
    [anon_sym_type] = ACTIONS(4988),
    [anon_sym_union] = ACTIONS(4988),
    [anon_sym_unsafe] = ACTIONS(4988),
    [anon_sym_use] = ACTIONS(4988),
    [anon_sym_while] = ACTIONS(4988),
    [anon_sym_extern] = ACTIONS(4988),
    [anon_sym_safe] = ACTIONS(4988),
    [anon_sym_yield] = ACTIONS(4988),
    [anon_sym_move] = ACTIONS(4988),
    [anon_sym_try] = ACTIONS(4988),
    [sym_integer_literal] = ACTIONS(4986),
    [aux_sym_string_literal_token1] = ACTIONS(4986),
    [sym_char_literal] = ACTIONS(4986),
    [anon_sym_true] = ACTIONS(4988),
    [anon_sym_false] = ACTIONS(4988),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4988),
    [sym_super] = ACTIONS(4988),
    [sym_crate] = ACTIONS(4988),
    [aux_sym_metavariable_token1] = ACTIONS(4986),
    [sym__raw_string_literal_start] = ACTIONS(4986),
    [sym_float_literal] = ACTIONS(4986),
    [sym__outer_block_doc_comment_start] = ACTIONS(4986),
    [sym__outer_line_doc_comment_start] = ACTIONS(4986),
  },
  [STATE(1314)] = {
    [sym_line_comment] = STATE(1314),
    [sym_block_comment] = STATE(1314),
    [ts_builtin_sym_end] = ACTIONS(4990),
    [sym_identifier] = ACTIONS(4992),
    [anon_sym_SEMI] = ACTIONS(4990),
    [anon_sym_macro] = ACTIONS(4992),
    [anon_sym_LPAREN] = ACTIONS(4990),
    [anon_sym_LBRACE] = ACTIONS(4990),
    [anon_sym_RBRACE] = ACTIONS(4990),
    [anon_sym_macro_rules] = ACTIONS(4992),
    [anon_sym_BANG] = ACTIONS(4990),
    [anon_sym_LBRACK] = ACTIONS(4990),
    [anon_sym_STAR] = ACTIONS(4990),
    [anon_sym_u8] = ACTIONS(4992),
    [anon_sym_i8] = ACTIONS(4992),
    [anon_sym_u16] = ACTIONS(4992),
    [anon_sym_i16] = ACTIONS(4992),
    [anon_sym_u32] = ACTIONS(4992),
    [anon_sym_i32] = ACTIONS(4992),
    [anon_sym_u64] = ACTIONS(4992),
    [anon_sym_i64] = ACTIONS(4992),
    [anon_sym_u128] = ACTIONS(4992),
    [anon_sym_i128] = ACTIONS(4992),
    [anon_sym_isize] = ACTIONS(4992),
    [anon_sym_usize] = ACTIONS(4992),
    [anon_sym_f32] = ACTIONS(4992),
    [anon_sym_f64] = ACTIONS(4992),
    [anon_sym_bool] = ACTIONS(4992),
    [anon_sym_str] = ACTIONS(4992),
    [anon_sym_char] = ACTIONS(4992),
    [anon_sym_DASH] = ACTIONS(4990),
    [anon_sym_AMP] = ACTIONS(4990),
    [anon_sym_PIPE] = ACTIONS(4990),
    [anon_sym_LT] = ACTIONS(4990),
    [anon_sym_DOT_DOT] = ACTIONS(4990),
    [anon_sym_COLON_COLON] = ACTIONS(4990),
    [anon_sym_POUND] = ACTIONS(4990),
    [anon_sym_SQUOTE] = ACTIONS(4992),
    [anon_sym_async] = ACTIONS(4992),
    [anon_sym_auto] = ACTIONS(4992),
    [anon_sym_await] = ACTIONS(4992),
    [anon_sym_become] = ACTIONS(4992),
    [anon_sym_break] = ACTIONS(4992),
    [anon_sym_const] = ACTIONS(4992),
    [anon_sym_continue] = ACTIONS(4992),
    [anon_sym_default] = ACTIONS(4992),
    [anon_sym_enum] = ACTIONS(4992),
    [anon_sym_fn] = ACTIONS(4992),
    [anon_sym_for] = ACTIONS(4992),
    [anon_sym_gen] = ACTIONS(4992),
    [anon_sym_if] = ACTIONS(4992),
    [anon_sym_impl] = ACTIONS(4992),
    [anon_sym_let] = ACTIONS(4992),
    [anon_sym_loop] = ACTIONS(4992),
    [anon_sym_match] = ACTIONS(4992),
    [anon_sym_mod] = ACTIONS(4992),
    [anon_sym_pub] = ACTIONS(4992),
    [anon_sym_return] = ACTIONS(4992),
    [anon_sym_raw] = ACTIONS(4992),
    [anon_sym_static] = ACTIONS(4992),
    [anon_sym_struct] = ACTIONS(4992),
    [anon_sym_trait] = ACTIONS(4992),
    [anon_sym_type] = ACTIONS(4992),
    [anon_sym_union] = ACTIONS(4992),
    [anon_sym_unsafe] = ACTIONS(4992),
    [anon_sym_use] = ACTIONS(4992),
    [anon_sym_while] = ACTIONS(4992),
    [anon_sym_extern] = ACTIONS(4992),
    [anon_sym_safe] = ACTIONS(4992),
    [anon_sym_yield] = ACTIONS(4992),
    [anon_sym_move] = ACTIONS(4992),
    [anon_sym_try] = ACTIONS(4992),
    [sym_integer_literal] = ACTIONS(4990),
    [aux_sym_string_literal_token1] = ACTIONS(4990),
    [sym_char_literal] = ACTIONS(4990),
    [anon_sym_true] = ACTIONS(4992),
    [anon_sym_false] = ACTIONS(4992),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4992),
    [sym_super] = ACTIONS(4992),
    [sym_crate] = ACTIONS(4992),
    [aux_sym_metavariable_token1] = ACTIONS(4990),
    [sym__raw_string_literal_start] = ACTIONS(4990),
    [sym_float_literal] = ACTIONS(4990),
    [sym__outer_block_doc_comment_start] = ACTIONS(4990),
    [sym__outer_line_doc_comment_start] = ACTIONS(4990),
  },
  [STATE(1315)] = {
    [sym_line_comment] = STATE(1315),
    [sym_block_comment] = STATE(1315),
    [ts_builtin_sym_end] = ACTIONS(4994),
    [sym_identifier] = ACTIONS(4996),
    [anon_sym_SEMI] = ACTIONS(4994),
    [anon_sym_macro] = ACTIONS(4996),
    [anon_sym_LPAREN] = ACTIONS(4994),
    [anon_sym_LBRACE] = ACTIONS(4994),
    [anon_sym_RBRACE] = ACTIONS(4994),
    [anon_sym_macro_rules] = ACTIONS(4996),
    [anon_sym_BANG] = ACTIONS(4994),
    [anon_sym_LBRACK] = ACTIONS(4994),
    [anon_sym_STAR] = ACTIONS(4994),
    [anon_sym_u8] = ACTIONS(4996),
    [anon_sym_i8] = ACTIONS(4996),
    [anon_sym_u16] = ACTIONS(4996),
    [anon_sym_i16] = ACTIONS(4996),
    [anon_sym_u32] = ACTIONS(4996),
    [anon_sym_i32] = ACTIONS(4996),
    [anon_sym_u64] = ACTIONS(4996),
    [anon_sym_i64] = ACTIONS(4996),
    [anon_sym_u128] = ACTIONS(4996),
    [anon_sym_i128] = ACTIONS(4996),
    [anon_sym_isize] = ACTIONS(4996),
    [anon_sym_usize] = ACTIONS(4996),
    [anon_sym_f32] = ACTIONS(4996),
    [anon_sym_f64] = ACTIONS(4996),
    [anon_sym_bool] = ACTIONS(4996),
    [anon_sym_str] = ACTIONS(4996),
    [anon_sym_char] = ACTIONS(4996),
    [anon_sym_DASH] = ACTIONS(4994),
    [anon_sym_AMP] = ACTIONS(4994),
    [anon_sym_PIPE] = ACTIONS(4994),
    [anon_sym_LT] = ACTIONS(4994),
    [anon_sym_DOT_DOT] = ACTIONS(4994),
    [anon_sym_COLON_COLON] = ACTIONS(4994),
    [anon_sym_POUND] = ACTIONS(4994),
    [anon_sym_SQUOTE] = ACTIONS(4996),
    [anon_sym_async] = ACTIONS(4996),
    [anon_sym_auto] = ACTIONS(4996),
    [anon_sym_await] = ACTIONS(4996),
    [anon_sym_become] = ACTIONS(4996),
    [anon_sym_break] = ACTIONS(4996),
    [anon_sym_const] = ACTIONS(4996),
    [anon_sym_continue] = ACTIONS(4996),
    [anon_sym_default] = ACTIONS(4996),
    [anon_sym_enum] = ACTIONS(4996),
    [anon_sym_fn] = ACTIONS(4996),
    [anon_sym_for] = ACTIONS(4996),
    [anon_sym_gen] = ACTIONS(4996),
    [anon_sym_if] = ACTIONS(4996),
    [anon_sym_impl] = ACTIONS(4996),
    [anon_sym_let] = ACTIONS(4996),
    [anon_sym_loop] = ACTIONS(4996),
    [anon_sym_match] = ACTIONS(4996),
    [anon_sym_mod] = ACTIONS(4996),
    [anon_sym_pub] = ACTIONS(4996),
    [anon_sym_return] = ACTIONS(4996),
    [anon_sym_raw] = ACTIONS(4996),
    [anon_sym_static] = ACTIONS(4996),
    [anon_sym_struct] = ACTIONS(4996),
    [anon_sym_trait] = ACTIONS(4996),
    [anon_sym_type] = ACTIONS(4996),
    [anon_sym_union] = ACTIONS(4996),
    [anon_sym_unsafe] = ACTIONS(4996),
    [anon_sym_use] = ACTIONS(4996),
    [anon_sym_while] = ACTIONS(4996),
    [anon_sym_extern] = ACTIONS(4996),
    [anon_sym_safe] = ACTIONS(4996),
    [anon_sym_yield] = ACTIONS(4996),
    [anon_sym_move] = ACTIONS(4996),
    [anon_sym_try] = ACTIONS(4996),
    [sym_integer_literal] = ACTIONS(4994),
    [aux_sym_string_literal_token1] = ACTIONS(4994),
    [sym_char_literal] = ACTIONS(4994),
    [anon_sym_true] = ACTIONS(4996),
    [anon_sym_false] = ACTIONS(4996),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(4996),
    [sym_super] = ACTIONS(4996),
    [sym_crate] = ACTIONS(4996),
    [aux_sym_metavariable_token1] = ACTIONS(4994),
    [sym__raw_string_literal_start] = ACTIONS(4994),
    [sym_float_literal] = ACTIONS(4994),
    [sym__outer_block_doc_comment_start] = ACTIONS(4994),
    [sym__outer_line_doc_comment_start] = ACTIONS(4994),
  },
  [STATE(1316)] = {
    [sym_line_comment] = STATE(1316),
    [sym_block_comment] = STATE(1316),
    [ts_builtin_sym_end] = ACTIONS(4998),
    [sym_identifier] = ACTIONS(5000),
    [anon_sym_SEMI] = ACTIONS(4998),
    [anon_sym_macro] = ACTIONS(5000),
    [anon_sym_LPAREN] = ACTIONS(4998),
    [anon_sym_LBRACE] = ACTIONS(4998),
    [anon_sym_RBRACE] = ACTIONS(4998),
    [anon_sym_macro_rules] = ACTIONS(5000),
    [anon_sym_BANG] = ACTIONS(4998),
    [anon_sym_LBRACK] = ACTIONS(4998),
    [anon_sym_STAR] = ACTIONS(4998),
    [anon_sym_u8] = ACTIONS(5000),
    [anon_sym_i8] = ACTIONS(5000),
    [anon_sym_u16] = ACTIONS(5000),
    [anon_sym_i16] = ACTIONS(5000),
    [anon_sym_u32] = ACTIONS(5000),
    [anon_sym_i32] = ACTIONS(5000),
    [anon_sym_u64] = ACTIONS(5000),
    [anon_sym_i64] = ACTIONS(5000),
    [anon_sym_u128] = ACTIONS(5000),
    [anon_sym_i128] = ACTIONS(5000),
    [anon_sym_isize] = ACTIONS(5000),
    [anon_sym_usize] = ACTIONS(5000),
    [anon_sym_f32] = ACTIONS(5000),
    [anon_sym_f64] = ACTIONS(5000),
    [anon_sym_bool] = ACTIONS(5000),
    [anon_sym_str] = ACTIONS(5000),
    [anon_sym_char] = ACTIONS(5000),
    [anon_sym_DASH] = ACTIONS(4998),
    [anon_sym_AMP] = ACTIONS(4998),
    [anon_sym_PIPE] = ACTIONS(4998),
    [anon_sym_LT] = ACTIONS(4998),
    [anon_sym_DOT_DOT] = ACTIONS(4998),
    [anon_sym_COLON_COLON] = ACTIONS(4998),
    [anon_sym_POUND] = ACTIONS(4998),
    [anon_sym_SQUOTE] = ACTIONS(5000),
    [anon_sym_async] = ACTIONS(5000),
    [anon_sym_auto] = ACTIONS(5000),
    [anon_sym_await] = ACTIONS(5000),
    [anon_sym_become] = ACTIONS(5000),
    [anon_sym_break] = ACTIONS(5000),
    [anon_sym_const] = ACTIONS(5000),
    [anon_sym_continue] = ACTIONS(5000),
    [anon_sym_default] = ACTIONS(5000),
    [anon_sym_enum] = ACTIONS(5000),
    [anon_sym_fn] = ACTIONS(5000),
    [anon_sym_for] = ACTIONS(5000),
    [anon_sym_gen] = ACTIONS(5000),
    [anon_sym_if] = ACTIONS(5000),
    [anon_sym_impl] = ACTIONS(5000),
    [anon_sym_let] = ACTIONS(5000),
    [anon_sym_loop] = ACTIONS(5000),
    [anon_sym_match] = ACTIONS(5000),
    [anon_sym_mod] = ACTIONS(5000),
    [anon_sym_pub] = ACTIONS(5000),
    [anon_sym_return] = ACTIONS(5000),
    [anon_sym_raw] = ACTIONS(5000),
    [anon_sym_static] = ACTIONS(5000),
    [anon_sym_struct] = ACTIONS(5000),
    [anon_sym_trait] = ACTIONS(5000),
    [anon_sym_type] = ACTIONS(5000),
    [anon_sym_union] = ACTIONS(5000),
    [anon_sym_unsafe] = ACTIONS(5000),
    [anon_sym_use] = ACTIONS(5000),
    [anon_sym_while] = ACTIONS(5000),
    [anon_sym_extern] = ACTIONS(5000),
    [anon_sym_safe] = ACTIONS(5000),
    [anon_sym_yield] = ACTIONS(5000),
    [anon_sym_move] = ACTIONS(5000),
    [anon_sym_try] = ACTIONS(5000),
    [sym_integer_literal] = ACTIONS(4998),
    [aux_sym_string_literal_token1] = ACTIONS(4998),
    [sym_char_literal] = ACTIONS(4998),
    [anon_sym_true] = ACTIONS(5000),
    [anon_sym_false] = ACTIONS(5000),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5000),
    [sym_super] = ACTIONS(5000),
    [sym_crate] = ACTIONS(5000),
    [aux_sym_metavariable_token1] = ACTIONS(4998),
    [sym__raw_string_literal_start] = ACTIONS(4998),
    [sym_float_literal] = ACTIONS(4998),
    [sym__outer_block_doc_comment_start] = ACTIONS(4998),
    [sym__outer_line_doc_comment_start] = ACTIONS(4998),
  },
  [STATE(1317)] = {
    [sym_line_comment] = STATE(1317),
    [sym_block_comment] = STATE(1317),
    [ts_builtin_sym_end] = ACTIONS(5002),
    [sym_identifier] = ACTIONS(5004),
    [anon_sym_SEMI] = ACTIONS(5002),
    [anon_sym_macro] = ACTIONS(5004),
    [anon_sym_LPAREN] = ACTIONS(5002),
    [anon_sym_LBRACE] = ACTIONS(5002),
    [anon_sym_RBRACE] = ACTIONS(5002),
    [anon_sym_macro_rules] = ACTIONS(5004),
    [anon_sym_BANG] = ACTIONS(5002),
    [anon_sym_LBRACK] = ACTIONS(5002),
    [anon_sym_STAR] = ACTIONS(5002),
    [anon_sym_u8] = ACTIONS(5004),
    [anon_sym_i8] = ACTIONS(5004),
    [anon_sym_u16] = ACTIONS(5004),
    [anon_sym_i16] = ACTIONS(5004),
    [anon_sym_u32] = ACTIONS(5004),
    [anon_sym_i32] = ACTIONS(5004),
    [anon_sym_u64] = ACTIONS(5004),
    [anon_sym_i64] = ACTIONS(5004),
    [anon_sym_u128] = ACTIONS(5004),
    [anon_sym_i128] = ACTIONS(5004),
    [anon_sym_isize] = ACTIONS(5004),
    [anon_sym_usize] = ACTIONS(5004),
    [anon_sym_f32] = ACTIONS(5004),
    [anon_sym_f64] = ACTIONS(5004),
    [anon_sym_bool] = ACTIONS(5004),
    [anon_sym_str] = ACTIONS(5004),
    [anon_sym_char] = ACTIONS(5004),
    [anon_sym_DASH] = ACTIONS(5002),
    [anon_sym_AMP] = ACTIONS(5002),
    [anon_sym_PIPE] = ACTIONS(5002),
    [anon_sym_LT] = ACTIONS(5002),
    [anon_sym_DOT_DOT] = ACTIONS(5002),
    [anon_sym_COLON_COLON] = ACTIONS(5002),
    [anon_sym_POUND] = ACTIONS(5002),
    [anon_sym_SQUOTE] = ACTIONS(5004),
    [anon_sym_async] = ACTIONS(5004),
    [anon_sym_auto] = ACTIONS(5004),
    [anon_sym_await] = ACTIONS(5004),
    [anon_sym_become] = ACTIONS(5004),
    [anon_sym_break] = ACTIONS(5004),
    [anon_sym_const] = ACTIONS(5004),
    [anon_sym_continue] = ACTIONS(5004),
    [anon_sym_default] = ACTIONS(5004),
    [anon_sym_enum] = ACTIONS(5004),
    [anon_sym_fn] = ACTIONS(5004),
    [anon_sym_for] = ACTIONS(5004),
    [anon_sym_gen] = ACTIONS(5004),
    [anon_sym_if] = ACTIONS(5004),
    [anon_sym_impl] = ACTIONS(5004),
    [anon_sym_let] = ACTIONS(5004),
    [anon_sym_loop] = ACTIONS(5004),
    [anon_sym_match] = ACTIONS(5004),
    [anon_sym_mod] = ACTIONS(5004),
    [anon_sym_pub] = ACTIONS(5004),
    [anon_sym_return] = ACTIONS(5004),
    [anon_sym_raw] = ACTIONS(5004),
    [anon_sym_static] = ACTIONS(5004),
    [anon_sym_struct] = ACTIONS(5004),
    [anon_sym_trait] = ACTIONS(5004),
    [anon_sym_type] = ACTIONS(5004),
    [anon_sym_union] = ACTIONS(5004),
    [anon_sym_unsafe] = ACTIONS(5004),
    [anon_sym_use] = ACTIONS(5004),
    [anon_sym_while] = ACTIONS(5004),
    [anon_sym_extern] = ACTIONS(5004),
    [anon_sym_safe] = ACTIONS(5004),
    [anon_sym_yield] = ACTIONS(5004),
    [anon_sym_move] = ACTIONS(5004),
    [anon_sym_try] = ACTIONS(5004),
    [sym_integer_literal] = ACTIONS(5002),
    [aux_sym_string_literal_token1] = ACTIONS(5002),
    [sym_char_literal] = ACTIONS(5002),
    [anon_sym_true] = ACTIONS(5004),
    [anon_sym_false] = ACTIONS(5004),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5004),
    [sym_super] = ACTIONS(5004),
    [sym_crate] = ACTIONS(5004),
    [aux_sym_metavariable_token1] = ACTIONS(5002),
    [sym__raw_string_literal_start] = ACTIONS(5002),
    [sym_float_literal] = ACTIONS(5002),
    [sym__outer_block_doc_comment_start] = ACTIONS(5002),
    [sym__outer_line_doc_comment_start] = ACTIONS(5002),
  },
  [STATE(1318)] = {
    [sym_line_comment] = STATE(1318),
    [sym_block_comment] = STATE(1318),
    [ts_builtin_sym_end] = ACTIONS(5006),
    [sym_identifier] = ACTIONS(5008),
    [anon_sym_SEMI] = ACTIONS(5006),
    [anon_sym_macro] = ACTIONS(5008),
    [anon_sym_LPAREN] = ACTIONS(5006),
    [anon_sym_LBRACE] = ACTIONS(5006),
    [anon_sym_RBRACE] = ACTIONS(5006),
    [anon_sym_macro_rules] = ACTIONS(5008),
    [anon_sym_BANG] = ACTIONS(5006),
    [anon_sym_LBRACK] = ACTIONS(5006),
    [anon_sym_STAR] = ACTIONS(5006),
    [anon_sym_u8] = ACTIONS(5008),
    [anon_sym_i8] = ACTIONS(5008),
    [anon_sym_u16] = ACTIONS(5008),
    [anon_sym_i16] = ACTIONS(5008),
    [anon_sym_u32] = ACTIONS(5008),
    [anon_sym_i32] = ACTIONS(5008),
    [anon_sym_u64] = ACTIONS(5008),
    [anon_sym_i64] = ACTIONS(5008),
    [anon_sym_u128] = ACTIONS(5008),
    [anon_sym_i128] = ACTIONS(5008),
    [anon_sym_isize] = ACTIONS(5008),
    [anon_sym_usize] = ACTIONS(5008),
    [anon_sym_f32] = ACTIONS(5008),
    [anon_sym_f64] = ACTIONS(5008),
    [anon_sym_bool] = ACTIONS(5008),
    [anon_sym_str] = ACTIONS(5008),
    [anon_sym_char] = ACTIONS(5008),
    [anon_sym_DASH] = ACTIONS(5006),
    [anon_sym_AMP] = ACTIONS(5006),
    [anon_sym_PIPE] = ACTIONS(5006),
    [anon_sym_LT] = ACTIONS(5006),
    [anon_sym_DOT_DOT] = ACTIONS(5006),
    [anon_sym_COLON_COLON] = ACTIONS(5006),
    [anon_sym_POUND] = ACTIONS(5006),
    [anon_sym_SQUOTE] = ACTIONS(5008),
    [anon_sym_async] = ACTIONS(5008),
    [anon_sym_auto] = ACTIONS(5008),
    [anon_sym_await] = ACTIONS(5008),
    [anon_sym_become] = ACTIONS(5008),
    [anon_sym_break] = ACTIONS(5008),
    [anon_sym_const] = ACTIONS(5008),
    [anon_sym_continue] = ACTIONS(5008),
    [anon_sym_default] = ACTIONS(5008),
    [anon_sym_enum] = ACTIONS(5008),
    [anon_sym_fn] = ACTIONS(5008),
    [anon_sym_for] = ACTIONS(5008),
    [anon_sym_gen] = ACTIONS(5008),
    [anon_sym_if] = ACTIONS(5008),
    [anon_sym_impl] = ACTIONS(5008),
    [anon_sym_let] = ACTIONS(5008),
    [anon_sym_loop] = ACTIONS(5008),
    [anon_sym_match] = ACTIONS(5008),
    [anon_sym_mod] = ACTIONS(5008),
    [anon_sym_pub] = ACTIONS(5008),
    [anon_sym_return] = ACTIONS(5008),
    [anon_sym_raw] = ACTIONS(5008),
    [anon_sym_static] = ACTIONS(5008),
    [anon_sym_struct] = ACTIONS(5008),
    [anon_sym_trait] = ACTIONS(5008),
    [anon_sym_type] = ACTIONS(5008),
    [anon_sym_union] = ACTIONS(5008),
    [anon_sym_unsafe] = ACTIONS(5008),
    [anon_sym_use] = ACTIONS(5008),
    [anon_sym_while] = ACTIONS(5008),
    [anon_sym_extern] = ACTIONS(5008),
    [anon_sym_safe] = ACTIONS(5008),
    [anon_sym_yield] = ACTIONS(5008),
    [anon_sym_move] = ACTIONS(5008),
    [anon_sym_try] = ACTIONS(5008),
    [sym_integer_literal] = ACTIONS(5006),
    [aux_sym_string_literal_token1] = ACTIONS(5006),
    [sym_char_literal] = ACTIONS(5006),
    [anon_sym_true] = ACTIONS(5008),
    [anon_sym_false] = ACTIONS(5008),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5008),
    [sym_super] = ACTIONS(5008),
    [sym_crate] = ACTIONS(5008),
    [aux_sym_metavariable_token1] = ACTIONS(5006),
    [sym__raw_string_literal_start] = ACTIONS(5006),
    [sym_float_literal] = ACTIONS(5006),
    [sym__outer_block_doc_comment_start] = ACTIONS(5006),
    [sym__outer_line_doc_comment_start] = ACTIONS(5006),
  },
  [STATE(1319)] = {
    [sym_line_comment] = STATE(1319),
    [sym_block_comment] = STATE(1319),
    [ts_builtin_sym_end] = ACTIONS(5010),
    [sym_identifier] = ACTIONS(5012),
    [anon_sym_SEMI] = ACTIONS(5010),
    [anon_sym_macro] = ACTIONS(5012),
    [anon_sym_LPAREN] = ACTIONS(5010),
    [anon_sym_LBRACE] = ACTIONS(5010),
    [anon_sym_RBRACE] = ACTIONS(5010),
    [anon_sym_macro_rules] = ACTIONS(5012),
    [anon_sym_BANG] = ACTIONS(5010),
    [anon_sym_LBRACK] = ACTIONS(5010),
    [anon_sym_STAR] = ACTIONS(5010),
    [anon_sym_u8] = ACTIONS(5012),
    [anon_sym_i8] = ACTIONS(5012),
    [anon_sym_u16] = ACTIONS(5012),
    [anon_sym_i16] = ACTIONS(5012),
    [anon_sym_u32] = ACTIONS(5012),
    [anon_sym_i32] = ACTIONS(5012),
    [anon_sym_u64] = ACTIONS(5012),
    [anon_sym_i64] = ACTIONS(5012),
    [anon_sym_u128] = ACTIONS(5012),
    [anon_sym_i128] = ACTIONS(5012),
    [anon_sym_isize] = ACTIONS(5012),
    [anon_sym_usize] = ACTIONS(5012),
    [anon_sym_f32] = ACTIONS(5012),
    [anon_sym_f64] = ACTIONS(5012),
    [anon_sym_bool] = ACTIONS(5012),
    [anon_sym_str] = ACTIONS(5012),
    [anon_sym_char] = ACTIONS(5012),
    [anon_sym_DASH] = ACTIONS(5010),
    [anon_sym_AMP] = ACTIONS(5010),
    [anon_sym_PIPE] = ACTIONS(5010),
    [anon_sym_LT] = ACTIONS(5010),
    [anon_sym_DOT_DOT] = ACTIONS(5010),
    [anon_sym_COLON_COLON] = ACTIONS(5010),
    [anon_sym_POUND] = ACTIONS(5010),
    [anon_sym_SQUOTE] = ACTIONS(5012),
    [anon_sym_async] = ACTIONS(5012),
    [anon_sym_auto] = ACTIONS(5012),
    [anon_sym_await] = ACTIONS(5012),
    [anon_sym_become] = ACTIONS(5012),
    [anon_sym_break] = ACTIONS(5012),
    [anon_sym_const] = ACTIONS(5012),
    [anon_sym_continue] = ACTIONS(5012),
    [anon_sym_default] = ACTIONS(5012),
    [anon_sym_enum] = ACTIONS(5012),
    [anon_sym_fn] = ACTIONS(5012),
    [anon_sym_for] = ACTIONS(5012),
    [anon_sym_gen] = ACTIONS(5012),
    [anon_sym_if] = ACTIONS(5012),
    [anon_sym_impl] = ACTIONS(5012),
    [anon_sym_let] = ACTIONS(5012),
    [anon_sym_loop] = ACTIONS(5012),
    [anon_sym_match] = ACTIONS(5012),
    [anon_sym_mod] = ACTIONS(5012),
    [anon_sym_pub] = ACTIONS(5012),
    [anon_sym_return] = ACTIONS(5012),
    [anon_sym_raw] = ACTIONS(5012),
    [anon_sym_static] = ACTIONS(5012),
    [anon_sym_struct] = ACTIONS(5012),
    [anon_sym_trait] = ACTIONS(5012),
    [anon_sym_type] = ACTIONS(5012),
    [anon_sym_union] = ACTIONS(5012),
    [anon_sym_unsafe] = ACTIONS(5012),
    [anon_sym_use] = ACTIONS(5012),
    [anon_sym_while] = ACTIONS(5012),
    [anon_sym_extern] = ACTIONS(5012),
    [anon_sym_safe] = ACTIONS(5012),
    [anon_sym_yield] = ACTIONS(5012),
    [anon_sym_move] = ACTIONS(5012),
    [anon_sym_try] = ACTIONS(5012),
    [sym_integer_literal] = ACTIONS(5010),
    [aux_sym_string_literal_token1] = ACTIONS(5010),
    [sym_char_literal] = ACTIONS(5010),
    [anon_sym_true] = ACTIONS(5012),
    [anon_sym_false] = ACTIONS(5012),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5012),
    [sym_super] = ACTIONS(5012),
    [sym_crate] = ACTIONS(5012),
    [aux_sym_metavariable_token1] = ACTIONS(5010),
    [sym__raw_string_literal_start] = ACTIONS(5010),
    [sym_float_literal] = ACTIONS(5010),
    [sym__outer_block_doc_comment_start] = ACTIONS(5010),
    [sym__outer_line_doc_comment_start] = ACTIONS(5010),
  },
  [STATE(1320)] = {
    [sym_line_comment] = STATE(1320),
    [sym_block_comment] = STATE(1320),
    [ts_builtin_sym_end] = ACTIONS(5014),
    [sym_identifier] = ACTIONS(5016),
    [anon_sym_SEMI] = ACTIONS(5014),
    [anon_sym_macro] = ACTIONS(5016),
    [anon_sym_LPAREN] = ACTIONS(5014),
    [anon_sym_LBRACE] = ACTIONS(5014),
    [anon_sym_RBRACE] = ACTIONS(5014),
    [anon_sym_macro_rules] = ACTIONS(5016),
    [anon_sym_BANG] = ACTIONS(5014),
    [anon_sym_LBRACK] = ACTIONS(5014),
    [anon_sym_STAR] = ACTIONS(5014),
    [anon_sym_u8] = ACTIONS(5016),
    [anon_sym_i8] = ACTIONS(5016),
    [anon_sym_u16] = ACTIONS(5016),
    [anon_sym_i16] = ACTIONS(5016),
    [anon_sym_u32] = ACTIONS(5016),
    [anon_sym_i32] = ACTIONS(5016),
    [anon_sym_u64] = ACTIONS(5016),
    [anon_sym_i64] = ACTIONS(5016),
    [anon_sym_u128] = ACTIONS(5016),
    [anon_sym_i128] = ACTIONS(5016),
    [anon_sym_isize] = ACTIONS(5016),
    [anon_sym_usize] = ACTIONS(5016),
    [anon_sym_f32] = ACTIONS(5016),
    [anon_sym_f64] = ACTIONS(5016),
    [anon_sym_bool] = ACTIONS(5016),
    [anon_sym_str] = ACTIONS(5016),
    [anon_sym_char] = ACTIONS(5016),
    [anon_sym_DASH] = ACTIONS(5014),
    [anon_sym_AMP] = ACTIONS(5014),
    [anon_sym_PIPE] = ACTIONS(5014),
    [anon_sym_LT] = ACTIONS(5014),
    [anon_sym_DOT_DOT] = ACTIONS(5014),
    [anon_sym_COLON_COLON] = ACTIONS(5014),
    [anon_sym_POUND] = ACTIONS(5014),
    [anon_sym_SQUOTE] = ACTIONS(5016),
    [anon_sym_async] = ACTIONS(5016),
    [anon_sym_auto] = ACTIONS(5016),
    [anon_sym_await] = ACTIONS(5016),
    [anon_sym_become] = ACTIONS(5016),
    [anon_sym_break] = ACTIONS(5016),
    [anon_sym_const] = ACTIONS(5016),
    [anon_sym_continue] = ACTIONS(5016),
    [anon_sym_default] = ACTIONS(5016),
    [anon_sym_enum] = ACTIONS(5016),
    [anon_sym_fn] = ACTIONS(5016),
    [anon_sym_for] = ACTIONS(5016),
    [anon_sym_gen] = ACTIONS(5016),
    [anon_sym_if] = ACTIONS(5016),
    [anon_sym_impl] = ACTIONS(5016),
    [anon_sym_let] = ACTIONS(5016),
    [anon_sym_loop] = ACTIONS(5016),
    [anon_sym_match] = ACTIONS(5016),
    [anon_sym_mod] = ACTIONS(5016),
    [anon_sym_pub] = ACTIONS(5016),
    [anon_sym_return] = ACTIONS(5016),
    [anon_sym_raw] = ACTIONS(5016),
    [anon_sym_static] = ACTIONS(5016),
    [anon_sym_struct] = ACTIONS(5016),
    [anon_sym_trait] = ACTIONS(5016),
    [anon_sym_type] = ACTIONS(5016),
    [anon_sym_union] = ACTIONS(5016),
    [anon_sym_unsafe] = ACTIONS(5016),
    [anon_sym_use] = ACTIONS(5016),
    [anon_sym_while] = ACTIONS(5016),
    [anon_sym_extern] = ACTIONS(5016),
    [anon_sym_safe] = ACTIONS(5016),
    [anon_sym_yield] = ACTIONS(5016),
    [anon_sym_move] = ACTIONS(5016),
    [anon_sym_try] = ACTIONS(5016),
    [sym_integer_literal] = ACTIONS(5014),
    [aux_sym_string_literal_token1] = ACTIONS(5014),
    [sym_char_literal] = ACTIONS(5014),
    [anon_sym_true] = ACTIONS(5016),
    [anon_sym_false] = ACTIONS(5016),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5016),
    [sym_super] = ACTIONS(5016),
    [sym_crate] = ACTIONS(5016),
    [aux_sym_metavariable_token1] = ACTIONS(5014),
    [sym__raw_string_literal_start] = ACTIONS(5014),
    [sym_float_literal] = ACTIONS(5014),
    [sym__outer_block_doc_comment_start] = ACTIONS(5014),
    [sym__outer_line_doc_comment_start] = ACTIONS(5014),
  },
  [STATE(1321)] = {
    [sym_line_comment] = STATE(1321),
    [sym_block_comment] = STATE(1321),
    [ts_builtin_sym_end] = ACTIONS(5018),
    [sym_identifier] = ACTIONS(5020),
    [anon_sym_SEMI] = ACTIONS(5018),
    [anon_sym_macro] = ACTIONS(5020),
    [anon_sym_LPAREN] = ACTIONS(5018),
    [anon_sym_LBRACE] = ACTIONS(5018),
    [anon_sym_RBRACE] = ACTIONS(5018),
    [anon_sym_macro_rules] = ACTIONS(5020),
    [anon_sym_BANG] = ACTIONS(5018),
    [anon_sym_LBRACK] = ACTIONS(5018),
    [anon_sym_STAR] = ACTIONS(5018),
    [anon_sym_u8] = ACTIONS(5020),
    [anon_sym_i8] = ACTIONS(5020),
    [anon_sym_u16] = ACTIONS(5020),
    [anon_sym_i16] = ACTIONS(5020),
    [anon_sym_u32] = ACTIONS(5020),
    [anon_sym_i32] = ACTIONS(5020),
    [anon_sym_u64] = ACTIONS(5020),
    [anon_sym_i64] = ACTIONS(5020),
    [anon_sym_u128] = ACTIONS(5020),
    [anon_sym_i128] = ACTIONS(5020),
    [anon_sym_isize] = ACTIONS(5020),
    [anon_sym_usize] = ACTIONS(5020),
    [anon_sym_f32] = ACTIONS(5020),
    [anon_sym_f64] = ACTIONS(5020),
    [anon_sym_bool] = ACTIONS(5020),
    [anon_sym_str] = ACTIONS(5020),
    [anon_sym_char] = ACTIONS(5020),
    [anon_sym_DASH] = ACTIONS(5018),
    [anon_sym_AMP] = ACTIONS(5018),
    [anon_sym_PIPE] = ACTIONS(5018),
    [anon_sym_LT] = ACTIONS(5018),
    [anon_sym_DOT_DOT] = ACTIONS(5018),
    [anon_sym_COLON_COLON] = ACTIONS(5018),
    [anon_sym_POUND] = ACTIONS(5018),
    [anon_sym_SQUOTE] = ACTIONS(5020),
    [anon_sym_async] = ACTIONS(5020),
    [anon_sym_auto] = ACTIONS(5020),
    [anon_sym_await] = ACTIONS(5020),
    [anon_sym_become] = ACTIONS(5020),
    [anon_sym_break] = ACTIONS(5020),
    [anon_sym_const] = ACTIONS(5020),
    [anon_sym_continue] = ACTIONS(5020),
    [anon_sym_default] = ACTIONS(5020),
    [anon_sym_enum] = ACTIONS(5020),
    [anon_sym_fn] = ACTIONS(5020),
    [anon_sym_for] = ACTIONS(5020),
    [anon_sym_gen] = ACTIONS(5020),
    [anon_sym_if] = ACTIONS(5020),
    [anon_sym_impl] = ACTIONS(5020),
    [anon_sym_let] = ACTIONS(5020),
    [anon_sym_loop] = ACTIONS(5020),
    [anon_sym_match] = ACTIONS(5020),
    [anon_sym_mod] = ACTIONS(5020),
    [anon_sym_pub] = ACTIONS(5020),
    [anon_sym_return] = ACTIONS(5020),
    [anon_sym_raw] = ACTIONS(5020),
    [anon_sym_static] = ACTIONS(5020),
    [anon_sym_struct] = ACTIONS(5020),
    [anon_sym_trait] = ACTIONS(5020),
    [anon_sym_type] = ACTIONS(5020),
    [anon_sym_union] = ACTIONS(5020),
    [anon_sym_unsafe] = ACTIONS(5020),
    [anon_sym_use] = ACTIONS(5020),
    [anon_sym_while] = ACTIONS(5020),
    [anon_sym_extern] = ACTIONS(5020),
    [anon_sym_safe] = ACTIONS(5020),
    [anon_sym_yield] = ACTIONS(5020),
    [anon_sym_move] = ACTIONS(5020),
    [anon_sym_try] = ACTIONS(5020),
    [sym_integer_literal] = ACTIONS(5018),
    [aux_sym_string_literal_token1] = ACTIONS(5018),
    [sym_char_literal] = ACTIONS(5018),
    [anon_sym_true] = ACTIONS(5020),
    [anon_sym_false] = ACTIONS(5020),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5020),
    [sym_super] = ACTIONS(5020),
    [sym_crate] = ACTIONS(5020),
    [aux_sym_metavariable_token1] = ACTIONS(5018),
    [sym__raw_string_literal_start] = ACTIONS(5018),
    [sym_float_literal] = ACTIONS(5018),
    [sym__outer_block_doc_comment_start] = ACTIONS(5018),
    [sym__outer_line_doc_comment_start] = ACTIONS(5018),
  },
  [STATE(1322)] = {
    [sym_line_comment] = STATE(1322),
    [sym_block_comment] = STATE(1322),
    [ts_builtin_sym_end] = ACTIONS(5022),
    [sym_identifier] = ACTIONS(5024),
    [anon_sym_SEMI] = ACTIONS(5022),
    [anon_sym_macro] = ACTIONS(5024),
    [anon_sym_LPAREN] = ACTIONS(5022),
    [anon_sym_LBRACE] = ACTIONS(5022),
    [anon_sym_RBRACE] = ACTIONS(5022),
    [anon_sym_macro_rules] = ACTIONS(5024),
    [anon_sym_BANG] = ACTIONS(5022),
    [anon_sym_LBRACK] = ACTIONS(5022),
    [anon_sym_STAR] = ACTIONS(5022),
    [anon_sym_u8] = ACTIONS(5024),
    [anon_sym_i8] = ACTIONS(5024),
    [anon_sym_u16] = ACTIONS(5024),
    [anon_sym_i16] = ACTIONS(5024),
    [anon_sym_u32] = ACTIONS(5024),
    [anon_sym_i32] = ACTIONS(5024),
    [anon_sym_u64] = ACTIONS(5024),
    [anon_sym_i64] = ACTIONS(5024),
    [anon_sym_u128] = ACTIONS(5024),
    [anon_sym_i128] = ACTIONS(5024),
    [anon_sym_isize] = ACTIONS(5024),
    [anon_sym_usize] = ACTIONS(5024),
    [anon_sym_f32] = ACTIONS(5024),
    [anon_sym_f64] = ACTIONS(5024),
    [anon_sym_bool] = ACTIONS(5024),
    [anon_sym_str] = ACTIONS(5024),
    [anon_sym_char] = ACTIONS(5024),
    [anon_sym_DASH] = ACTIONS(5022),
    [anon_sym_AMP] = ACTIONS(5022),
    [anon_sym_PIPE] = ACTIONS(5022),
    [anon_sym_LT] = ACTIONS(5022),
    [anon_sym_DOT_DOT] = ACTIONS(5022),
    [anon_sym_COLON_COLON] = ACTIONS(5022),
    [anon_sym_POUND] = ACTIONS(5022),
    [anon_sym_SQUOTE] = ACTIONS(5024),
    [anon_sym_async] = ACTIONS(5024),
    [anon_sym_auto] = ACTIONS(5024),
    [anon_sym_await] = ACTIONS(5024),
    [anon_sym_become] = ACTIONS(5024),
    [anon_sym_break] = ACTIONS(5024),
    [anon_sym_const] = ACTIONS(5024),
    [anon_sym_continue] = ACTIONS(5024),
    [anon_sym_default] = ACTIONS(5024),
    [anon_sym_enum] = ACTIONS(5024),
    [anon_sym_fn] = ACTIONS(5024),
    [anon_sym_for] = ACTIONS(5024),
    [anon_sym_gen] = ACTIONS(5024),
    [anon_sym_if] = ACTIONS(5024),
    [anon_sym_impl] = ACTIONS(5024),
    [anon_sym_let] = ACTIONS(5024),
    [anon_sym_loop] = ACTIONS(5024),
    [anon_sym_match] = ACTIONS(5024),
    [anon_sym_mod] = ACTIONS(5024),
    [anon_sym_pub] = ACTIONS(5024),
    [anon_sym_return] = ACTIONS(5024),
    [anon_sym_raw] = ACTIONS(5024),
    [anon_sym_static] = ACTIONS(5024),
    [anon_sym_struct] = ACTIONS(5024),
    [anon_sym_trait] = ACTIONS(5024),
    [anon_sym_type] = ACTIONS(5024),
    [anon_sym_union] = ACTIONS(5024),
    [anon_sym_unsafe] = ACTIONS(5024),
    [anon_sym_use] = ACTIONS(5024),
    [anon_sym_while] = ACTIONS(5024),
    [anon_sym_extern] = ACTIONS(5024),
    [anon_sym_safe] = ACTIONS(5024),
    [anon_sym_yield] = ACTIONS(5024),
    [anon_sym_move] = ACTIONS(5024),
    [anon_sym_try] = ACTIONS(5024),
    [sym_integer_literal] = ACTIONS(5022),
    [aux_sym_string_literal_token1] = ACTIONS(5022),
    [sym_char_literal] = ACTIONS(5022),
    [anon_sym_true] = ACTIONS(5024),
    [anon_sym_false] = ACTIONS(5024),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5024),
    [sym_super] = ACTIONS(5024),
    [sym_crate] = ACTIONS(5024),
    [aux_sym_metavariable_token1] = ACTIONS(5022),
    [sym__raw_string_literal_start] = ACTIONS(5022),
    [sym_float_literal] = ACTIONS(5022),
    [sym__outer_block_doc_comment_start] = ACTIONS(5022),
    [sym__outer_line_doc_comment_start] = ACTIONS(5022),
  },
  [STATE(1323)] = {
    [sym_line_comment] = STATE(1323),
    [sym_block_comment] = STATE(1323),
    [ts_builtin_sym_end] = ACTIONS(5026),
    [sym_identifier] = ACTIONS(5028),
    [anon_sym_SEMI] = ACTIONS(5026),
    [anon_sym_macro] = ACTIONS(5028),
    [anon_sym_LPAREN] = ACTIONS(5026),
    [anon_sym_LBRACE] = ACTIONS(5026),
    [anon_sym_RBRACE] = ACTIONS(5026),
    [anon_sym_macro_rules] = ACTIONS(5028),
    [anon_sym_BANG] = ACTIONS(5026),
    [anon_sym_LBRACK] = ACTIONS(5026),
    [anon_sym_STAR] = ACTIONS(5026),
    [anon_sym_u8] = ACTIONS(5028),
    [anon_sym_i8] = ACTIONS(5028),
    [anon_sym_u16] = ACTIONS(5028),
    [anon_sym_i16] = ACTIONS(5028),
    [anon_sym_u32] = ACTIONS(5028),
    [anon_sym_i32] = ACTIONS(5028),
    [anon_sym_u64] = ACTIONS(5028),
    [anon_sym_i64] = ACTIONS(5028),
    [anon_sym_u128] = ACTIONS(5028),
    [anon_sym_i128] = ACTIONS(5028),
    [anon_sym_isize] = ACTIONS(5028),
    [anon_sym_usize] = ACTIONS(5028),
    [anon_sym_f32] = ACTIONS(5028),
    [anon_sym_f64] = ACTIONS(5028),
    [anon_sym_bool] = ACTIONS(5028),
    [anon_sym_str] = ACTIONS(5028),
    [anon_sym_char] = ACTIONS(5028),
    [anon_sym_DASH] = ACTIONS(5026),
    [anon_sym_AMP] = ACTIONS(5026),
    [anon_sym_PIPE] = ACTIONS(5026),
    [anon_sym_LT] = ACTIONS(5026),
    [anon_sym_DOT_DOT] = ACTIONS(5026),
    [anon_sym_COLON_COLON] = ACTIONS(5026),
    [anon_sym_POUND] = ACTIONS(5026),
    [anon_sym_SQUOTE] = ACTIONS(5028),
    [anon_sym_async] = ACTIONS(5028),
    [anon_sym_auto] = ACTIONS(5028),
    [anon_sym_await] = ACTIONS(5028),
    [anon_sym_become] = ACTIONS(5028),
    [anon_sym_break] = ACTIONS(5028),
    [anon_sym_const] = ACTIONS(5028),
    [anon_sym_continue] = ACTIONS(5028),
    [anon_sym_default] = ACTIONS(5028),
    [anon_sym_enum] = ACTIONS(5028),
    [anon_sym_fn] = ACTIONS(5028),
    [anon_sym_for] = ACTIONS(5028),
    [anon_sym_gen] = ACTIONS(5028),
    [anon_sym_if] = ACTIONS(5028),
    [anon_sym_impl] = ACTIONS(5028),
    [anon_sym_let] = ACTIONS(5028),
    [anon_sym_loop] = ACTIONS(5028),
    [anon_sym_match] = ACTIONS(5028),
    [anon_sym_mod] = ACTIONS(5028),
    [anon_sym_pub] = ACTIONS(5028),
    [anon_sym_return] = ACTIONS(5028),
    [anon_sym_raw] = ACTIONS(5028),
    [anon_sym_static] = ACTIONS(5028),
    [anon_sym_struct] = ACTIONS(5028),
    [anon_sym_trait] = ACTIONS(5028),
    [anon_sym_type] = ACTIONS(5028),
    [anon_sym_union] = ACTIONS(5028),
    [anon_sym_unsafe] = ACTIONS(5028),
    [anon_sym_use] = ACTIONS(5028),
    [anon_sym_while] = ACTIONS(5028),
    [anon_sym_extern] = ACTIONS(5028),
    [anon_sym_safe] = ACTIONS(5028),
    [anon_sym_yield] = ACTIONS(5028),
    [anon_sym_move] = ACTIONS(5028),
    [anon_sym_try] = ACTIONS(5028),
    [sym_integer_literal] = ACTIONS(5026),
    [aux_sym_string_literal_token1] = ACTIONS(5026),
    [sym_char_literal] = ACTIONS(5026),
    [anon_sym_true] = ACTIONS(5028),
    [anon_sym_false] = ACTIONS(5028),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5028),
    [sym_super] = ACTIONS(5028),
    [sym_crate] = ACTIONS(5028),
    [aux_sym_metavariable_token1] = ACTIONS(5026),
    [sym__raw_string_literal_start] = ACTIONS(5026),
    [sym_float_literal] = ACTIONS(5026),
    [sym__outer_block_doc_comment_start] = ACTIONS(5026),
    [sym__outer_line_doc_comment_start] = ACTIONS(5026),
  },
  [STATE(1324)] = {
    [sym_line_comment] = STATE(1324),
    [sym_block_comment] = STATE(1324),
    [ts_builtin_sym_end] = ACTIONS(5030),
    [sym_identifier] = ACTIONS(5032),
    [anon_sym_SEMI] = ACTIONS(5030),
    [anon_sym_macro] = ACTIONS(5032),
    [anon_sym_LPAREN] = ACTIONS(5030),
    [anon_sym_LBRACE] = ACTIONS(5030),
    [anon_sym_RBRACE] = ACTIONS(5030),
    [anon_sym_macro_rules] = ACTIONS(5032),
    [anon_sym_BANG] = ACTIONS(5030),
    [anon_sym_LBRACK] = ACTIONS(5030),
    [anon_sym_STAR] = ACTIONS(5030),
    [anon_sym_u8] = ACTIONS(5032),
    [anon_sym_i8] = ACTIONS(5032),
    [anon_sym_u16] = ACTIONS(5032),
    [anon_sym_i16] = ACTIONS(5032),
    [anon_sym_u32] = ACTIONS(5032),
    [anon_sym_i32] = ACTIONS(5032),
    [anon_sym_u64] = ACTIONS(5032),
    [anon_sym_i64] = ACTIONS(5032),
    [anon_sym_u128] = ACTIONS(5032),
    [anon_sym_i128] = ACTIONS(5032),
    [anon_sym_isize] = ACTIONS(5032),
    [anon_sym_usize] = ACTIONS(5032),
    [anon_sym_f32] = ACTIONS(5032),
    [anon_sym_f64] = ACTIONS(5032),
    [anon_sym_bool] = ACTIONS(5032),
    [anon_sym_str] = ACTIONS(5032),
    [anon_sym_char] = ACTIONS(5032),
    [anon_sym_DASH] = ACTIONS(5030),
    [anon_sym_AMP] = ACTIONS(5030),
    [anon_sym_PIPE] = ACTIONS(5030),
    [anon_sym_LT] = ACTIONS(5030),
    [anon_sym_DOT_DOT] = ACTIONS(5030),
    [anon_sym_COLON_COLON] = ACTIONS(5030),
    [anon_sym_POUND] = ACTIONS(5030),
    [anon_sym_SQUOTE] = ACTIONS(5032),
    [anon_sym_async] = ACTIONS(5032),
    [anon_sym_auto] = ACTIONS(5032),
    [anon_sym_await] = ACTIONS(5032),
    [anon_sym_become] = ACTIONS(5032),
    [anon_sym_break] = ACTIONS(5032),
    [anon_sym_const] = ACTIONS(5032),
    [anon_sym_continue] = ACTIONS(5032),
    [anon_sym_default] = ACTIONS(5032),
    [anon_sym_enum] = ACTIONS(5032),
    [anon_sym_fn] = ACTIONS(5032),
    [anon_sym_for] = ACTIONS(5032),
    [anon_sym_gen] = ACTIONS(5032),
    [anon_sym_if] = ACTIONS(5032),
    [anon_sym_impl] = ACTIONS(5032),
    [anon_sym_let] = ACTIONS(5032),
    [anon_sym_loop] = ACTIONS(5032),
    [anon_sym_match] = ACTIONS(5032),
    [anon_sym_mod] = ACTIONS(5032),
    [anon_sym_pub] = ACTIONS(5032),
    [anon_sym_return] = ACTIONS(5032),
    [anon_sym_raw] = ACTIONS(5032),
    [anon_sym_static] = ACTIONS(5032),
    [anon_sym_struct] = ACTIONS(5032),
    [anon_sym_trait] = ACTIONS(5032),
    [anon_sym_type] = ACTIONS(5032),
    [anon_sym_union] = ACTIONS(5032),
    [anon_sym_unsafe] = ACTIONS(5032),
    [anon_sym_use] = ACTIONS(5032),
    [anon_sym_while] = ACTIONS(5032),
    [anon_sym_extern] = ACTIONS(5032),
    [anon_sym_safe] = ACTIONS(5032),
    [anon_sym_yield] = ACTIONS(5032),
    [anon_sym_move] = ACTIONS(5032),
    [anon_sym_try] = ACTIONS(5032),
    [sym_integer_literal] = ACTIONS(5030),
    [aux_sym_string_literal_token1] = ACTIONS(5030),
    [sym_char_literal] = ACTIONS(5030),
    [anon_sym_true] = ACTIONS(5032),
    [anon_sym_false] = ACTIONS(5032),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5032),
    [sym_super] = ACTIONS(5032),
    [sym_crate] = ACTIONS(5032),
    [aux_sym_metavariable_token1] = ACTIONS(5030),
    [sym__raw_string_literal_start] = ACTIONS(5030),
    [sym_float_literal] = ACTIONS(5030),
    [sym__outer_block_doc_comment_start] = ACTIONS(5030),
    [sym__outer_line_doc_comment_start] = ACTIONS(5030),
  },
  [STATE(1325)] = {
    [sym_line_comment] = STATE(1325),
    [sym_block_comment] = STATE(1325),
    [ts_builtin_sym_end] = ACTIONS(5034),
    [sym_identifier] = ACTIONS(5036),
    [anon_sym_SEMI] = ACTIONS(5034),
    [anon_sym_macro] = ACTIONS(5036),
    [anon_sym_LPAREN] = ACTIONS(5034),
    [anon_sym_LBRACE] = ACTIONS(5034),
    [anon_sym_RBRACE] = ACTIONS(5034),
    [anon_sym_macro_rules] = ACTIONS(5036),
    [anon_sym_BANG] = ACTIONS(5034),
    [anon_sym_LBRACK] = ACTIONS(5034),
    [anon_sym_STAR] = ACTIONS(5034),
    [anon_sym_u8] = ACTIONS(5036),
    [anon_sym_i8] = ACTIONS(5036),
    [anon_sym_u16] = ACTIONS(5036),
    [anon_sym_i16] = ACTIONS(5036),
    [anon_sym_u32] = ACTIONS(5036),
    [anon_sym_i32] = ACTIONS(5036),
    [anon_sym_u64] = ACTIONS(5036),
    [anon_sym_i64] = ACTIONS(5036),
    [anon_sym_u128] = ACTIONS(5036),
    [anon_sym_i128] = ACTIONS(5036),
    [anon_sym_isize] = ACTIONS(5036),
    [anon_sym_usize] = ACTIONS(5036),
    [anon_sym_f32] = ACTIONS(5036),
    [anon_sym_f64] = ACTIONS(5036),
    [anon_sym_bool] = ACTIONS(5036),
    [anon_sym_str] = ACTIONS(5036),
    [anon_sym_char] = ACTIONS(5036),
    [anon_sym_DASH] = ACTIONS(5034),
    [anon_sym_AMP] = ACTIONS(5034),
    [anon_sym_PIPE] = ACTIONS(5034),
    [anon_sym_LT] = ACTIONS(5034),
    [anon_sym_DOT_DOT] = ACTIONS(5034),
    [anon_sym_COLON_COLON] = ACTIONS(5034),
    [anon_sym_POUND] = ACTIONS(5034),
    [anon_sym_SQUOTE] = ACTIONS(5036),
    [anon_sym_async] = ACTIONS(5036),
    [anon_sym_auto] = ACTIONS(5036),
    [anon_sym_await] = ACTIONS(5036),
    [anon_sym_become] = ACTIONS(5036),
    [anon_sym_break] = ACTIONS(5036),
    [anon_sym_const] = ACTIONS(5036),
    [anon_sym_continue] = ACTIONS(5036),
    [anon_sym_default] = ACTIONS(5036),
    [anon_sym_enum] = ACTIONS(5036),
    [anon_sym_fn] = ACTIONS(5036),
    [anon_sym_for] = ACTIONS(5036),
    [anon_sym_gen] = ACTIONS(5036),
    [anon_sym_if] = ACTIONS(5036),
    [anon_sym_impl] = ACTIONS(5036),
    [anon_sym_let] = ACTIONS(5036),
    [anon_sym_loop] = ACTIONS(5036),
    [anon_sym_match] = ACTIONS(5036),
    [anon_sym_mod] = ACTIONS(5036),
    [anon_sym_pub] = ACTIONS(5036),
    [anon_sym_return] = ACTIONS(5036),
    [anon_sym_raw] = ACTIONS(5036),
    [anon_sym_static] = ACTIONS(5036),
    [anon_sym_struct] = ACTIONS(5036),
    [anon_sym_trait] = ACTIONS(5036),
    [anon_sym_type] = ACTIONS(5036),
    [anon_sym_union] = ACTIONS(5036),
    [anon_sym_unsafe] = ACTIONS(5036),
    [anon_sym_use] = ACTIONS(5036),
    [anon_sym_while] = ACTIONS(5036),
    [anon_sym_extern] = ACTIONS(5036),
    [anon_sym_safe] = ACTIONS(5036),
    [anon_sym_yield] = ACTIONS(5036),
    [anon_sym_move] = ACTIONS(5036),
    [anon_sym_try] = ACTIONS(5036),
    [sym_integer_literal] = ACTIONS(5034),
    [aux_sym_string_literal_token1] = ACTIONS(5034),
    [sym_char_literal] = ACTIONS(5034),
    [anon_sym_true] = ACTIONS(5036),
    [anon_sym_false] = ACTIONS(5036),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5036),
    [sym_super] = ACTIONS(5036),
    [sym_crate] = ACTIONS(5036),
    [aux_sym_metavariable_token1] = ACTIONS(5034),
    [sym__raw_string_literal_start] = ACTIONS(5034),
    [sym_float_literal] = ACTIONS(5034),
    [sym__outer_block_doc_comment_start] = ACTIONS(5034),
    [sym__outer_line_doc_comment_start] = ACTIONS(5034),
  },
  [STATE(1326)] = {
    [sym_line_comment] = STATE(1326),
    [sym_block_comment] = STATE(1326),
    [ts_builtin_sym_end] = ACTIONS(5038),
    [sym_identifier] = ACTIONS(5040),
    [anon_sym_SEMI] = ACTIONS(5038),
    [anon_sym_macro] = ACTIONS(5040),
    [anon_sym_LPAREN] = ACTIONS(5038),
    [anon_sym_LBRACE] = ACTIONS(5038),
    [anon_sym_RBRACE] = ACTIONS(5038),
    [anon_sym_macro_rules] = ACTIONS(5040),
    [anon_sym_BANG] = ACTIONS(5038),
    [anon_sym_LBRACK] = ACTIONS(5038),
    [anon_sym_STAR] = ACTIONS(5038),
    [anon_sym_u8] = ACTIONS(5040),
    [anon_sym_i8] = ACTIONS(5040),
    [anon_sym_u16] = ACTIONS(5040),
    [anon_sym_i16] = ACTIONS(5040),
    [anon_sym_u32] = ACTIONS(5040),
    [anon_sym_i32] = ACTIONS(5040),
    [anon_sym_u64] = ACTIONS(5040),
    [anon_sym_i64] = ACTIONS(5040),
    [anon_sym_u128] = ACTIONS(5040),
    [anon_sym_i128] = ACTIONS(5040),
    [anon_sym_isize] = ACTIONS(5040),
    [anon_sym_usize] = ACTIONS(5040),
    [anon_sym_f32] = ACTIONS(5040),
    [anon_sym_f64] = ACTIONS(5040),
    [anon_sym_bool] = ACTIONS(5040),
    [anon_sym_str] = ACTIONS(5040),
    [anon_sym_char] = ACTIONS(5040),
    [anon_sym_DASH] = ACTIONS(5038),
    [anon_sym_AMP] = ACTIONS(5038),
    [anon_sym_PIPE] = ACTIONS(5038),
    [anon_sym_LT] = ACTIONS(5038),
    [anon_sym_DOT_DOT] = ACTIONS(5038),
    [anon_sym_COLON_COLON] = ACTIONS(5038),
    [anon_sym_POUND] = ACTIONS(5038),
    [anon_sym_SQUOTE] = ACTIONS(5040),
    [anon_sym_async] = ACTIONS(5040),
    [anon_sym_auto] = ACTIONS(5040),
    [anon_sym_await] = ACTIONS(5040),
    [anon_sym_become] = ACTIONS(5040),
    [anon_sym_break] = ACTIONS(5040),
    [anon_sym_const] = ACTIONS(5040),
    [anon_sym_continue] = ACTIONS(5040),
    [anon_sym_default] = ACTIONS(5040),
    [anon_sym_enum] = ACTIONS(5040),
    [anon_sym_fn] = ACTIONS(5040),
    [anon_sym_for] = ACTIONS(5040),
    [anon_sym_gen] = ACTIONS(5040),
    [anon_sym_if] = ACTIONS(5040),
    [anon_sym_impl] = ACTIONS(5040),
    [anon_sym_let] = ACTIONS(5040),
    [anon_sym_loop] = ACTIONS(5040),
    [anon_sym_match] = ACTIONS(5040),
    [anon_sym_mod] = ACTIONS(5040),
    [anon_sym_pub] = ACTIONS(5040),
    [anon_sym_return] = ACTIONS(5040),
    [anon_sym_raw] = ACTIONS(5040),
    [anon_sym_static] = ACTIONS(5040),
    [anon_sym_struct] = ACTIONS(5040),
    [anon_sym_trait] = ACTIONS(5040),
    [anon_sym_type] = ACTIONS(5040),
    [anon_sym_union] = ACTIONS(5040),
    [anon_sym_unsafe] = ACTIONS(5040),
    [anon_sym_use] = ACTIONS(5040),
    [anon_sym_while] = ACTIONS(5040),
    [anon_sym_extern] = ACTIONS(5040),
    [anon_sym_safe] = ACTIONS(5040),
    [anon_sym_yield] = ACTIONS(5040),
    [anon_sym_move] = ACTIONS(5040),
    [anon_sym_try] = ACTIONS(5040),
    [sym_integer_literal] = ACTIONS(5038),
    [aux_sym_string_literal_token1] = ACTIONS(5038),
    [sym_char_literal] = ACTIONS(5038),
    [anon_sym_true] = ACTIONS(5040),
    [anon_sym_false] = ACTIONS(5040),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5040),
    [sym_super] = ACTIONS(5040),
    [sym_crate] = ACTIONS(5040),
    [aux_sym_metavariable_token1] = ACTIONS(5038),
    [sym__raw_string_literal_start] = ACTIONS(5038),
    [sym_float_literal] = ACTIONS(5038),
    [sym__outer_block_doc_comment_start] = ACTIONS(5038),
    [sym__outer_line_doc_comment_start] = ACTIONS(5038),
  },
  [STATE(1327)] = {
    [sym_line_comment] = STATE(1327),
    [sym_block_comment] = STATE(1327),
    [ts_builtin_sym_end] = ACTIONS(5042),
    [sym_identifier] = ACTIONS(5044),
    [anon_sym_SEMI] = ACTIONS(5042),
    [anon_sym_macro] = ACTIONS(5044),
    [anon_sym_LPAREN] = ACTIONS(5042),
    [anon_sym_LBRACE] = ACTIONS(5042),
    [anon_sym_RBRACE] = ACTIONS(5042),
    [anon_sym_macro_rules] = ACTIONS(5044),
    [anon_sym_BANG] = ACTIONS(5042),
    [anon_sym_LBRACK] = ACTIONS(5042),
    [anon_sym_STAR] = ACTIONS(5042),
    [anon_sym_u8] = ACTIONS(5044),
    [anon_sym_i8] = ACTIONS(5044),
    [anon_sym_u16] = ACTIONS(5044),
    [anon_sym_i16] = ACTIONS(5044),
    [anon_sym_u32] = ACTIONS(5044),
    [anon_sym_i32] = ACTIONS(5044),
    [anon_sym_u64] = ACTIONS(5044),
    [anon_sym_i64] = ACTIONS(5044),
    [anon_sym_u128] = ACTIONS(5044),
    [anon_sym_i128] = ACTIONS(5044),
    [anon_sym_isize] = ACTIONS(5044),
    [anon_sym_usize] = ACTIONS(5044),
    [anon_sym_f32] = ACTIONS(5044),
    [anon_sym_f64] = ACTIONS(5044),
    [anon_sym_bool] = ACTIONS(5044),
    [anon_sym_str] = ACTIONS(5044),
    [anon_sym_char] = ACTIONS(5044),
    [anon_sym_DASH] = ACTIONS(5042),
    [anon_sym_AMP] = ACTIONS(5042),
    [anon_sym_PIPE] = ACTIONS(5042),
    [anon_sym_LT] = ACTIONS(5042),
    [anon_sym_DOT_DOT] = ACTIONS(5042),
    [anon_sym_COLON_COLON] = ACTIONS(5042),
    [anon_sym_POUND] = ACTIONS(5042),
    [anon_sym_SQUOTE] = ACTIONS(5044),
    [anon_sym_async] = ACTIONS(5044),
    [anon_sym_auto] = ACTIONS(5044),
    [anon_sym_await] = ACTIONS(5044),
    [anon_sym_become] = ACTIONS(5044),
    [anon_sym_break] = ACTIONS(5044),
    [anon_sym_const] = ACTIONS(5044),
    [anon_sym_continue] = ACTIONS(5044),
    [anon_sym_default] = ACTIONS(5044),
    [anon_sym_enum] = ACTIONS(5044),
    [anon_sym_fn] = ACTIONS(5044),
    [anon_sym_for] = ACTIONS(5044),
    [anon_sym_gen] = ACTIONS(5044),
    [anon_sym_if] = ACTIONS(5044),
    [anon_sym_impl] = ACTIONS(5044),
    [anon_sym_let] = ACTIONS(5044),
    [anon_sym_loop] = ACTIONS(5044),
    [anon_sym_match] = ACTIONS(5044),
    [anon_sym_mod] = ACTIONS(5044),
    [anon_sym_pub] = ACTIONS(5044),
    [anon_sym_return] = ACTIONS(5044),
    [anon_sym_raw] = ACTIONS(5044),
    [anon_sym_static] = ACTIONS(5044),
    [anon_sym_struct] = ACTIONS(5044),
    [anon_sym_trait] = ACTIONS(5044),
    [anon_sym_type] = ACTIONS(5044),
    [anon_sym_union] = ACTIONS(5044),
    [anon_sym_unsafe] = ACTIONS(5044),
    [anon_sym_use] = ACTIONS(5044),
    [anon_sym_while] = ACTIONS(5044),
    [anon_sym_extern] = ACTIONS(5044),
    [anon_sym_safe] = ACTIONS(5044),
    [anon_sym_yield] = ACTIONS(5044),
    [anon_sym_move] = ACTIONS(5044),
    [anon_sym_try] = ACTIONS(5044),
    [sym_integer_literal] = ACTIONS(5042),
    [aux_sym_string_literal_token1] = ACTIONS(5042),
    [sym_char_literal] = ACTIONS(5042),
    [anon_sym_true] = ACTIONS(5044),
    [anon_sym_false] = ACTIONS(5044),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5044),
    [sym_super] = ACTIONS(5044),
    [sym_crate] = ACTIONS(5044),
    [aux_sym_metavariable_token1] = ACTIONS(5042),
    [sym__raw_string_literal_start] = ACTIONS(5042),
    [sym_float_literal] = ACTIONS(5042),
    [sym__outer_block_doc_comment_start] = ACTIONS(5042),
    [sym__outer_line_doc_comment_start] = ACTIONS(5042),
  },
  [STATE(1328)] = {
    [sym_line_comment] = STATE(1328),
    [sym_block_comment] = STATE(1328),
    [ts_builtin_sym_end] = ACTIONS(5046),
    [sym_identifier] = ACTIONS(5048),
    [anon_sym_SEMI] = ACTIONS(5046),
    [anon_sym_macro] = ACTIONS(5048),
    [anon_sym_LPAREN] = ACTIONS(5046),
    [anon_sym_LBRACE] = ACTIONS(5046),
    [anon_sym_RBRACE] = ACTIONS(5046),
    [anon_sym_macro_rules] = ACTIONS(5048),
    [anon_sym_BANG] = ACTIONS(5046),
    [anon_sym_LBRACK] = ACTIONS(5046),
    [anon_sym_STAR] = ACTIONS(5046),
    [anon_sym_u8] = ACTIONS(5048),
    [anon_sym_i8] = ACTIONS(5048),
    [anon_sym_u16] = ACTIONS(5048),
    [anon_sym_i16] = ACTIONS(5048),
    [anon_sym_u32] = ACTIONS(5048),
    [anon_sym_i32] = ACTIONS(5048),
    [anon_sym_u64] = ACTIONS(5048),
    [anon_sym_i64] = ACTIONS(5048),
    [anon_sym_u128] = ACTIONS(5048),
    [anon_sym_i128] = ACTIONS(5048),
    [anon_sym_isize] = ACTIONS(5048),
    [anon_sym_usize] = ACTIONS(5048),
    [anon_sym_f32] = ACTIONS(5048),
    [anon_sym_f64] = ACTIONS(5048),
    [anon_sym_bool] = ACTIONS(5048),
    [anon_sym_str] = ACTIONS(5048),
    [anon_sym_char] = ACTIONS(5048),
    [anon_sym_DASH] = ACTIONS(5046),
    [anon_sym_AMP] = ACTIONS(5046),
    [anon_sym_PIPE] = ACTIONS(5046),
    [anon_sym_LT] = ACTIONS(5046),
    [anon_sym_DOT_DOT] = ACTIONS(5046),
    [anon_sym_COLON_COLON] = ACTIONS(5046),
    [anon_sym_POUND] = ACTIONS(5046),
    [anon_sym_SQUOTE] = ACTIONS(5048),
    [anon_sym_async] = ACTIONS(5048),
    [anon_sym_auto] = ACTIONS(5048),
    [anon_sym_await] = ACTIONS(5048),
    [anon_sym_become] = ACTIONS(5048),
    [anon_sym_break] = ACTIONS(5048),
    [anon_sym_const] = ACTIONS(5048),
    [anon_sym_continue] = ACTIONS(5048),
    [anon_sym_default] = ACTIONS(5048),
    [anon_sym_enum] = ACTIONS(5048),
    [anon_sym_fn] = ACTIONS(5048),
    [anon_sym_for] = ACTIONS(5048),
    [anon_sym_gen] = ACTIONS(5048),
    [anon_sym_if] = ACTIONS(5048),
    [anon_sym_impl] = ACTIONS(5048),
    [anon_sym_let] = ACTIONS(5048),
    [anon_sym_loop] = ACTIONS(5048),
    [anon_sym_match] = ACTIONS(5048),
    [anon_sym_mod] = ACTIONS(5048),
    [anon_sym_pub] = ACTIONS(5048),
    [anon_sym_return] = ACTIONS(5048),
    [anon_sym_raw] = ACTIONS(5048),
    [anon_sym_static] = ACTIONS(5048),
    [anon_sym_struct] = ACTIONS(5048),
    [anon_sym_trait] = ACTIONS(5048),
    [anon_sym_type] = ACTIONS(5048),
    [anon_sym_union] = ACTIONS(5048),
    [anon_sym_unsafe] = ACTIONS(5048),
    [anon_sym_use] = ACTIONS(5048),
    [anon_sym_while] = ACTIONS(5048),
    [anon_sym_extern] = ACTIONS(5048),
    [anon_sym_safe] = ACTIONS(5048),
    [anon_sym_yield] = ACTIONS(5048),
    [anon_sym_move] = ACTIONS(5048),
    [anon_sym_try] = ACTIONS(5048),
    [sym_integer_literal] = ACTIONS(5046),
    [aux_sym_string_literal_token1] = ACTIONS(5046),
    [sym_char_literal] = ACTIONS(5046),
    [anon_sym_true] = ACTIONS(5048),
    [anon_sym_false] = ACTIONS(5048),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5048),
    [sym_super] = ACTIONS(5048),
    [sym_crate] = ACTIONS(5048),
    [aux_sym_metavariable_token1] = ACTIONS(5046),
    [sym__raw_string_literal_start] = ACTIONS(5046),
    [sym_float_literal] = ACTIONS(5046),
    [sym__outer_block_doc_comment_start] = ACTIONS(5046),
    [sym__outer_line_doc_comment_start] = ACTIONS(5046),
  },
  [STATE(1329)] = {
    [sym_line_comment] = STATE(1329),
    [sym_block_comment] = STATE(1329),
    [ts_builtin_sym_end] = ACTIONS(5050),
    [sym_identifier] = ACTIONS(5052),
    [anon_sym_SEMI] = ACTIONS(5050),
    [anon_sym_macro] = ACTIONS(5052),
    [anon_sym_LPAREN] = ACTIONS(5050),
    [anon_sym_LBRACE] = ACTIONS(5050),
    [anon_sym_RBRACE] = ACTIONS(5050),
    [anon_sym_macro_rules] = ACTIONS(5052),
    [anon_sym_BANG] = ACTIONS(5050),
    [anon_sym_LBRACK] = ACTIONS(5050),
    [anon_sym_STAR] = ACTIONS(5050),
    [anon_sym_u8] = ACTIONS(5052),
    [anon_sym_i8] = ACTIONS(5052),
    [anon_sym_u16] = ACTIONS(5052),
    [anon_sym_i16] = ACTIONS(5052),
    [anon_sym_u32] = ACTIONS(5052),
    [anon_sym_i32] = ACTIONS(5052),
    [anon_sym_u64] = ACTIONS(5052),
    [anon_sym_i64] = ACTIONS(5052),
    [anon_sym_u128] = ACTIONS(5052),
    [anon_sym_i128] = ACTIONS(5052),
    [anon_sym_isize] = ACTIONS(5052),
    [anon_sym_usize] = ACTIONS(5052),
    [anon_sym_f32] = ACTIONS(5052),
    [anon_sym_f64] = ACTIONS(5052),
    [anon_sym_bool] = ACTIONS(5052),
    [anon_sym_str] = ACTIONS(5052),
    [anon_sym_char] = ACTIONS(5052),
    [anon_sym_DASH] = ACTIONS(5050),
    [anon_sym_AMP] = ACTIONS(5050),
    [anon_sym_PIPE] = ACTIONS(5050),
    [anon_sym_LT] = ACTIONS(5050),
    [anon_sym_DOT_DOT] = ACTIONS(5050),
    [anon_sym_COLON_COLON] = ACTIONS(5050),
    [anon_sym_POUND] = ACTIONS(5050),
    [anon_sym_SQUOTE] = ACTIONS(5052),
    [anon_sym_async] = ACTIONS(5052),
    [anon_sym_auto] = ACTIONS(5052),
    [anon_sym_await] = ACTIONS(5052),
    [anon_sym_become] = ACTIONS(5052),
    [anon_sym_break] = ACTIONS(5052),
    [anon_sym_const] = ACTIONS(5052),
    [anon_sym_continue] = ACTIONS(5052),
    [anon_sym_default] = ACTIONS(5052),
    [anon_sym_enum] = ACTIONS(5052),
    [anon_sym_fn] = ACTIONS(5052),
    [anon_sym_for] = ACTIONS(5052),
    [anon_sym_gen] = ACTIONS(5052),
    [anon_sym_if] = ACTIONS(5052),
    [anon_sym_impl] = ACTIONS(5052),
    [anon_sym_let] = ACTIONS(5052),
    [anon_sym_loop] = ACTIONS(5052),
    [anon_sym_match] = ACTIONS(5052),
    [anon_sym_mod] = ACTIONS(5052),
    [anon_sym_pub] = ACTIONS(5052),
    [anon_sym_return] = ACTIONS(5052),
    [anon_sym_raw] = ACTIONS(5052),
    [anon_sym_static] = ACTIONS(5052),
    [anon_sym_struct] = ACTIONS(5052),
    [anon_sym_trait] = ACTIONS(5052),
    [anon_sym_type] = ACTIONS(5052),
    [anon_sym_union] = ACTIONS(5052),
    [anon_sym_unsafe] = ACTIONS(5052),
    [anon_sym_use] = ACTIONS(5052),
    [anon_sym_while] = ACTIONS(5052),
    [anon_sym_extern] = ACTIONS(5052),
    [anon_sym_safe] = ACTIONS(5052),
    [anon_sym_yield] = ACTIONS(5052),
    [anon_sym_move] = ACTIONS(5052),
    [anon_sym_try] = ACTIONS(5052),
    [sym_integer_literal] = ACTIONS(5050),
    [aux_sym_string_literal_token1] = ACTIONS(5050),
    [sym_char_literal] = ACTIONS(5050),
    [anon_sym_true] = ACTIONS(5052),
    [anon_sym_false] = ACTIONS(5052),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5052),
    [sym_super] = ACTIONS(5052),
    [sym_crate] = ACTIONS(5052),
    [aux_sym_metavariable_token1] = ACTIONS(5050),
    [sym__raw_string_literal_start] = ACTIONS(5050),
    [sym_float_literal] = ACTIONS(5050),
    [sym__outer_block_doc_comment_start] = ACTIONS(5050),
    [sym__outer_line_doc_comment_start] = ACTIONS(5050),
  },
  [STATE(1330)] = {
    [sym_line_comment] = STATE(1330),
    [sym_block_comment] = STATE(1330),
    [ts_builtin_sym_end] = ACTIONS(5054),
    [sym_identifier] = ACTIONS(5056),
    [anon_sym_SEMI] = ACTIONS(5054),
    [anon_sym_macro] = ACTIONS(5056),
    [anon_sym_LPAREN] = ACTIONS(5054),
    [anon_sym_LBRACE] = ACTIONS(5054),
    [anon_sym_RBRACE] = ACTIONS(5054),
    [anon_sym_macro_rules] = ACTIONS(5056),
    [anon_sym_BANG] = ACTIONS(5054),
    [anon_sym_LBRACK] = ACTIONS(5054),
    [anon_sym_STAR] = ACTIONS(5054),
    [anon_sym_u8] = ACTIONS(5056),
    [anon_sym_i8] = ACTIONS(5056),
    [anon_sym_u16] = ACTIONS(5056),
    [anon_sym_i16] = ACTIONS(5056),
    [anon_sym_u32] = ACTIONS(5056),
    [anon_sym_i32] = ACTIONS(5056),
    [anon_sym_u64] = ACTIONS(5056),
    [anon_sym_i64] = ACTIONS(5056),
    [anon_sym_u128] = ACTIONS(5056),
    [anon_sym_i128] = ACTIONS(5056),
    [anon_sym_isize] = ACTIONS(5056),
    [anon_sym_usize] = ACTIONS(5056),
    [anon_sym_f32] = ACTIONS(5056),
    [anon_sym_f64] = ACTIONS(5056),
    [anon_sym_bool] = ACTIONS(5056),
    [anon_sym_str] = ACTIONS(5056),
    [anon_sym_char] = ACTIONS(5056),
    [anon_sym_DASH] = ACTIONS(5054),
    [anon_sym_AMP] = ACTIONS(5054),
    [anon_sym_PIPE] = ACTIONS(5054),
    [anon_sym_LT] = ACTIONS(5054),
    [anon_sym_DOT_DOT] = ACTIONS(5054),
    [anon_sym_COLON_COLON] = ACTIONS(5054),
    [anon_sym_POUND] = ACTIONS(5054),
    [anon_sym_SQUOTE] = ACTIONS(5056),
    [anon_sym_async] = ACTIONS(5056),
    [anon_sym_auto] = ACTIONS(5056),
    [anon_sym_await] = ACTIONS(5056),
    [anon_sym_become] = ACTIONS(5056),
    [anon_sym_break] = ACTIONS(5056),
    [anon_sym_const] = ACTIONS(5056),
    [anon_sym_continue] = ACTIONS(5056),
    [anon_sym_default] = ACTIONS(5056),
    [anon_sym_enum] = ACTIONS(5056),
    [anon_sym_fn] = ACTIONS(5056),
    [anon_sym_for] = ACTIONS(5056),
    [anon_sym_gen] = ACTIONS(5056),
    [anon_sym_if] = ACTIONS(5056),
    [anon_sym_impl] = ACTIONS(5056),
    [anon_sym_let] = ACTIONS(5056),
    [anon_sym_loop] = ACTIONS(5056),
    [anon_sym_match] = ACTIONS(5056),
    [anon_sym_mod] = ACTIONS(5056),
    [anon_sym_pub] = ACTIONS(5056),
    [anon_sym_return] = ACTIONS(5056),
    [anon_sym_raw] = ACTIONS(5056),
    [anon_sym_static] = ACTIONS(5056),
    [anon_sym_struct] = ACTIONS(5056),
    [anon_sym_trait] = ACTIONS(5056),
    [anon_sym_type] = ACTIONS(5056),
    [anon_sym_union] = ACTIONS(5056),
    [anon_sym_unsafe] = ACTIONS(5056),
    [anon_sym_use] = ACTIONS(5056),
    [anon_sym_while] = ACTIONS(5056),
    [anon_sym_extern] = ACTIONS(5056),
    [anon_sym_safe] = ACTIONS(5056),
    [anon_sym_yield] = ACTIONS(5056),
    [anon_sym_move] = ACTIONS(5056),
    [anon_sym_try] = ACTIONS(5056),
    [sym_integer_literal] = ACTIONS(5054),
    [aux_sym_string_literal_token1] = ACTIONS(5054),
    [sym_char_literal] = ACTIONS(5054),
    [anon_sym_true] = ACTIONS(5056),
    [anon_sym_false] = ACTIONS(5056),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5056),
    [sym_super] = ACTIONS(5056),
    [sym_crate] = ACTIONS(5056),
    [aux_sym_metavariable_token1] = ACTIONS(5054),
    [sym__raw_string_literal_start] = ACTIONS(5054),
    [sym_float_literal] = ACTIONS(5054),
    [sym__outer_block_doc_comment_start] = ACTIONS(5054),
    [sym__outer_line_doc_comment_start] = ACTIONS(5054),
  },
  [STATE(1331)] = {
    [sym_line_comment] = STATE(1331),
    [sym_block_comment] = STATE(1331),
    [ts_builtin_sym_end] = ACTIONS(5058),
    [sym_identifier] = ACTIONS(5060),
    [anon_sym_SEMI] = ACTIONS(5058),
    [anon_sym_macro] = ACTIONS(5060),
    [anon_sym_LPAREN] = ACTIONS(5058),
    [anon_sym_LBRACE] = ACTIONS(5058),
    [anon_sym_RBRACE] = ACTIONS(5058),
    [anon_sym_macro_rules] = ACTIONS(5060),
    [anon_sym_BANG] = ACTIONS(5058),
    [anon_sym_LBRACK] = ACTIONS(5058),
    [anon_sym_STAR] = ACTIONS(5058),
    [anon_sym_u8] = ACTIONS(5060),
    [anon_sym_i8] = ACTIONS(5060),
    [anon_sym_u16] = ACTIONS(5060),
    [anon_sym_i16] = ACTIONS(5060),
    [anon_sym_u32] = ACTIONS(5060),
    [anon_sym_i32] = ACTIONS(5060),
    [anon_sym_u64] = ACTIONS(5060),
    [anon_sym_i64] = ACTIONS(5060),
    [anon_sym_u128] = ACTIONS(5060),
    [anon_sym_i128] = ACTIONS(5060),
    [anon_sym_isize] = ACTIONS(5060),
    [anon_sym_usize] = ACTIONS(5060),
    [anon_sym_f32] = ACTIONS(5060),
    [anon_sym_f64] = ACTIONS(5060),
    [anon_sym_bool] = ACTIONS(5060),
    [anon_sym_str] = ACTIONS(5060),
    [anon_sym_char] = ACTIONS(5060),
    [anon_sym_DASH] = ACTIONS(5058),
    [anon_sym_AMP] = ACTIONS(5058),
    [anon_sym_PIPE] = ACTIONS(5058),
    [anon_sym_LT] = ACTIONS(5058),
    [anon_sym_DOT_DOT] = ACTIONS(5058),
    [anon_sym_COLON_COLON] = ACTIONS(5058),
    [anon_sym_POUND] = ACTIONS(5058),
    [anon_sym_SQUOTE] = ACTIONS(5060),
    [anon_sym_async] = ACTIONS(5060),
    [anon_sym_auto] = ACTIONS(5060),
    [anon_sym_await] = ACTIONS(5060),
    [anon_sym_become] = ACTIONS(5060),
    [anon_sym_break] = ACTIONS(5060),
    [anon_sym_const] = ACTIONS(5060),
    [anon_sym_continue] = ACTIONS(5060),
    [anon_sym_default] = ACTIONS(5060),
    [anon_sym_enum] = ACTIONS(5060),
    [anon_sym_fn] = ACTIONS(5060),
    [anon_sym_for] = ACTIONS(5060),
    [anon_sym_gen] = ACTIONS(5060),
    [anon_sym_if] = ACTIONS(5060),
    [anon_sym_impl] = ACTIONS(5060),
    [anon_sym_let] = ACTIONS(5060),
    [anon_sym_loop] = ACTIONS(5060),
    [anon_sym_match] = ACTIONS(5060),
    [anon_sym_mod] = ACTIONS(5060),
    [anon_sym_pub] = ACTIONS(5060),
    [anon_sym_return] = ACTIONS(5060),
    [anon_sym_raw] = ACTIONS(5060),
    [anon_sym_static] = ACTIONS(5060),
    [anon_sym_struct] = ACTIONS(5060),
    [anon_sym_trait] = ACTIONS(5060),
    [anon_sym_type] = ACTIONS(5060),
    [anon_sym_union] = ACTIONS(5060),
    [anon_sym_unsafe] = ACTIONS(5060),
    [anon_sym_use] = ACTIONS(5060),
    [anon_sym_while] = ACTIONS(5060),
    [anon_sym_extern] = ACTIONS(5060),
    [anon_sym_safe] = ACTIONS(5060),
    [anon_sym_yield] = ACTIONS(5060),
    [anon_sym_move] = ACTIONS(5060),
    [anon_sym_try] = ACTIONS(5060),
    [sym_integer_literal] = ACTIONS(5058),
    [aux_sym_string_literal_token1] = ACTIONS(5058),
    [sym_char_literal] = ACTIONS(5058),
    [anon_sym_true] = ACTIONS(5060),
    [anon_sym_false] = ACTIONS(5060),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5060),
    [sym_super] = ACTIONS(5060),
    [sym_crate] = ACTIONS(5060),
    [aux_sym_metavariable_token1] = ACTIONS(5058),
    [sym__raw_string_literal_start] = ACTIONS(5058),
    [sym_float_literal] = ACTIONS(5058),
    [sym__outer_block_doc_comment_start] = ACTIONS(5058),
    [sym__outer_line_doc_comment_start] = ACTIONS(5058),
  },
  [STATE(1332)] = {
    [sym_line_comment] = STATE(1332),
    [sym_block_comment] = STATE(1332),
    [ts_builtin_sym_end] = ACTIONS(5062),
    [sym_identifier] = ACTIONS(5064),
    [anon_sym_SEMI] = ACTIONS(5062),
    [anon_sym_macro] = ACTIONS(5064),
    [anon_sym_LPAREN] = ACTIONS(5062),
    [anon_sym_LBRACE] = ACTIONS(5062),
    [anon_sym_RBRACE] = ACTIONS(5062),
    [anon_sym_macro_rules] = ACTIONS(5064),
    [anon_sym_BANG] = ACTIONS(5062),
    [anon_sym_LBRACK] = ACTIONS(5062),
    [anon_sym_STAR] = ACTIONS(5062),
    [anon_sym_u8] = ACTIONS(5064),
    [anon_sym_i8] = ACTIONS(5064),
    [anon_sym_u16] = ACTIONS(5064),
    [anon_sym_i16] = ACTIONS(5064),
    [anon_sym_u32] = ACTIONS(5064),
    [anon_sym_i32] = ACTIONS(5064),
    [anon_sym_u64] = ACTIONS(5064),
    [anon_sym_i64] = ACTIONS(5064),
    [anon_sym_u128] = ACTIONS(5064),
    [anon_sym_i128] = ACTIONS(5064),
    [anon_sym_isize] = ACTIONS(5064),
    [anon_sym_usize] = ACTIONS(5064),
    [anon_sym_f32] = ACTIONS(5064),
    [anon_sym_f64] = ACTIONS(5064),
    [anon_sym_bool] = ACTIONS(5064),
    [anon_sym_str] = ACTIONS(5064),
    [anon_sym_char] = ACTIONS(5064),
    [anon_sym_DASH] = ACTIONS(5062),
    [anon_sym_AMP] = ACTIONS(5062),
    [anon_sym_PIPE] = ACTIONS(5062),
    [anon_sym_LT] = ACTIONS(5062),
    [anon_sym_DOT_DOT] = ACTIONS(5062),
    [anon_sym_COLON_COLON] = ACTIONS(5062),
    [anon_sym_POUND] = ACTIONS(5062),
    [anon_sym_SQUOTE] = ACTIONS(5064),
    [anon_sym_async] = ACTIONS(5064),
    [anon_sym_auto] = ACTIONS(5064),
    [anon_sym_await] = ACTIONS(5064),
    [anon_sym_become] = ACTIONS(5064),
    [anon_sym_break] = ACTIONS(5064),
    [anon_sym_const] = ACTIONS(5064),
    [anon_sym_continue] = ACTIONS(5064),
    [anon_sym_default] = ACTIONS(5064),
    [anon_sym_enum] = ACTIONS(5064),
    [anon_sym_fn] = ACTIONS(5064),
    [anon_sym_for] = ACTIONS(5064),
    [anon_sym_gen] = ACTIONS(5064),
    [anon_sym_if] = ACTIONS(5064),
    [anon_sym_impl] = ACTIONS(5064),
    [anon_sym_let] = ACTIONS(5064),
    [anon_sym_loop] = ACTIONS(5064),
    [anon_sym_match] = ACTIONS(5064),
    [anon_sym_mod] = ACTIONS(5064),
    [anon_sym_pub] = ACTIONS(5064),
    [anon_sym_return] = ACTIONS(5064),
    [anon_sym_raw] = ACTIONS(5064),
    [anon_sym_static] = ACTIONS(5064),
    [anon_sym_struct] = ACTIONS(5064),
    [anon_sym_trait] = ACTIONS(5064),
    [anon_sym_type] = ACTIONS(5064),
    [anon_sym_union] = ACTIONS(5064),
    [anon_sym_unsafe] = ACTIONS(5064),
    [anon_sym_use] = ACTIONS(5064),
    [anon_sym_while] = ACTIONS(5064),
    [anon_sym_extern] = ACTIONS(5064),
    [anon_sym_safe] = ACTIONS(5064),
    [anon_sym_yield] = ACTIONS(5064),
    [anon_sym_move] = ACTIONS(5064),
    [anon_sym_try] = ACTIONS(5064),
    [sym_integer_literal] = ACTIONS(5062),
    [aux_sym_string_literal_token1] = ACTIONS(5062),
    [sym_char_literal] = ACTIONS(5062),
    [anon_sym_true] = ACTIONS(5064),
    [anon_sym_false] = ACTIONS(5064),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5064),
    [sym_super] = ACTIONS(5064),
    [sym_crate] = ACTIONS(5064),
    [aux_sym_metavariable_token1] = ACTIONS(5062),
    [sym__raw_string_literal_start] = ACTIONS(5062),
    [sym_float_literal] = ACTIONS(5062),
    [sym__outer_block_doc_comment_start] = ACTIONS(5062),
    [sym__outer_line_doc_comment_start] = ACTIONS(5062),
  },
  [STATE(1333)] = {
    [sym_line_comment] = STATE(1333),
    [sym_block_comment] = STATE(1333),
    [ts_builtin_sym_end] = ACTIONS(5066),
    [sym_identifier] = ACTIONS(5068),
    [anon_sym_SEMI] = ACTIONS(5066),
    [anon_sym_macro] = ACTIONS(5068),
    [anon_sym_LPAREN] = ACTIONS(5066),
    [anon_sym_LBRACE] = ACTIONS(5066),
    [anon_sym_RBRACE] = ACTIONS(5066),
    [anon_sym_macro_rules] = ACTIONS(5068),
    [anon_sym_BANG] = ACTIONS(5066),
    [anon_sym_LBRACK] = ACTIONS(5066),
    [anon_sym_STAR] = ACTIONS(5066),
    [anon_sym_u8] = ACTIONS(5068),
    [anon_sym_i8] = ACTIONS(5068),
    [anon_sym_u16] = ACTIONS(5068),
    [anon_sym_i16] = ACTIONS(5068),
    [anon_sym_u32] = ACTIONS(5068),
    [anon_sym_i32] = ACTIONS(5068),
    [anon_sym_u64] = ACTIONS(5068),
    [anon_sym_i64] = ACTIONS(5068),
    [anon_sym_u128] = ACTIONS(5068),
    [anon_sym_i128] = ACTIONS(5068),
    [anon_sym_isize] = ACTIONS(5068),
    [anon_sym_usize] = ACTIONS(5068),
    [anon_sym_f32] = ACTIONS(5068),
    [anon_sym_f64] = ACTIONS(5068),
    [anon_sym_bool] = ACTIONS(5068),
    [anon_sym_str] = ACTIONS(5068),
    [anon_sym_char] = ACTIONS(5068),
    [anon_sym_DASH] = ACTIONS(5066),
    [anon_sym_AMP] = ACTIONS(5066),
    [anon_sym_PIPE] = ACTIONS(5066),
    [anon_sym_LT] = ACTIONS(5066),
    [anon_sym_DOT_DOT] = ACTIONS(5066),
    [anon_sym_COLON_COLON] = ACTIONS(5066),
    [anon_sym_POUND] = ACTIONS(5066),
    [anon_sym_SQUOTE] = ACTIONS(5068),
    [anon_sym_async] = ACTIONS(5068),
    [anon_sym_auto] = ACTIONS(5068),
    [anon_sym_await] = ACTIONS(5068),
    [anon_sym_become] = ACTIONS(5068),
    [anon_sym_break] = ACTIONS(5068),
    [anon_sym_const] = ACTIONS(5068),
    [anon_sym_continue] = ACTIONS(5068),
    [anon_sym_default] = ACTIONS(5068),
    [anon_sym_enum] = ACTIONS(5068),
    [anon_sym_fn] = ACTIONS(5068),
    [anon_sym_for] = ACTIONS(5068),
    [anon_sym_gen] = ACTIONS(5068),
    [anon_sym_if] = ACTIONS(5068),
    [anon_sym_impl] = ACTIONS(5068),
    [anon_sym_let] = ACTIONS(5068),
    [anon_sym_loop] = ACTIONS(5068),
    [anon_sym_match] = ACTIONS(5068),
    [anon_sym_mod] = ACTIONS(5068),
    [anon_sym_pub] = ACTIONS(5068),
    [anon_sym_return] = ACTIONS(5068),
    [anon_sym_raw] = ACTIONS(5068),
    [anon_sym_static] = ACTIONS(5068),
    [anon_sym_struct] = ACTIONS(5068),
    [anon_sym_trait] = ACTIONS(5068),
    [anon_sym_type] = ACTIONS(5068),
    [anon_sym_union] = ACTIONS(5068),
    [anon_sym_unsafe] = ACTIONS(5068),
    [anon_sym_use] = ACTIONS(5068),
    [anon_sym_while] = ACTIONS(5068),
    [anon_sym_extern] = ACTIONS(5068),
    [anon_sym_safe] = ACTIONS(5068),
    [anon_sym_yield] = ACTIONS(5068),
    [anon_sym_move] = ACTIONS(5068),
    [anon_sym_try] = ACTIONS(5068),
    [sym_integer_literal] = ACTIONS(5066),
    [aux_sym_string_literal_token1] = ACTIONS(5066),
    [sym_char_literal] = ACTIONS(5066),
    [anon_sym_true] = ACTIONS(5068),
    [anon_sym_false] = ACTIONS(5068),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5068),
    [sym_super] = ACTIONS(5068),
    [sym_crate] = ACTIONS(5068),
    [aux_sym_metavariable_token1] = ACTIONS(5066),
    [sym__raw_string_literal_start] = ACTIONS(5066),
    [sym_float_literal] = ACTIONS(5066),
    [sym__outer_block_doc_comment_start] = ACTIONS(5066),
    [sym__outer_line_doc_comment_start] = ACTIONS(5066),
  },
  [STATE(1334)] = {
    [sym_line_comment] = STATE(1334),
    [sym_block_comment] = STATE(1334),
    [ts_builtin_sym_end] = ACTIONS(5070),
    [sym_identifier] = ACTIONS(5072),
    [anon_sym_SEMI] = ACTIONS(5070),
    [anon_sym_macro] = ACTIONS(5072),
    [anon_sym_LPAREN] = ACTIONS(5070),
    [anon_sym_LBRACE] = ACTIONS(5070),
    [anon_sym_RBRACE] = ACTIONS(5070),
    [anon_sym_macro_rules] = ACTIONS(5072),
    [anon_sym_BANG] = ACTIONS(5070),
    [anon_sym_LBRACK] = ACTIONS(5070),
    [anon_sym_STAR] = ACTIONS(5070),
    [anon_sym_u8] = ACTIONS(5072),
    [anon_sym_i8] = ACTIONS(5072),
    [anon_sym_u16] = ACTIONS(5072),
    [anon_sym_i16] = ACTIONS(5072),
    [anon_sym_u32] = ACTIONS(5072),
    [anon_sym_i32] = ACTIONS(5072),
    [anon_sym_u64] = ACTIONS(5072),
    [anon_sym_i64] = ACTIONS(5072),
    [anon_sym_u128] = ACTIONS(5072),
    [anon_sym_i128] = ACTIONS(5072),
    [anon_sym_isize] = ACTIONS(5072),
    [anon_sym_usize] = ACTIONS(5072),
    [anon_sym_f32] = ACTIONS(5072),
    [anon_sym_f64] = ACTIONS(5072),
    [anon_sym_bool] = ACTIONS(5072),
    [anon_sym_str] = ACTIONS(5072),
    [anon_sym_char] = ACTIONS(5072),
    [anon_sym_DASH] = ACTIONS(5070),
    [anon_sym_AMP] = ACTIONS(5070),
    [anon_sym_PIPE] = ACTIONS(5070),
    [anon_sym_LT] = ACTIONS(5070),
    [anon_sym_DOT_DOT] = ACTIONS(5070),
    [anon_sym_COLON_COLON] = ACTIONS(5070),
    [anon_sym_POUND] = ACTIONS(5070),
    [anon_sym_SQUOTE] = ACTIONS(5072),
    [anon_sym_async] = ACTIONS(5072),
    [anon_sym_auto] = ACTIONS(5072),
    [anon_sym_await] = ACTIONS(5072),
    [anon_sym_become] = ACTIONS(5072),
    [anon_sym_break] = ACTIONS(5072),
    [anon_sym_const] = ACTIONS(5072),
    [anon_sym_continue] = ACTIONS(5072),
    [anon_sym_default] = ACTIONS(5072),
    [anon_sym_enum] = ACTIONS(5072),
    [anon_sym_fn] = ACTIONS(5072),
    [anon_sym_for] = ACTIONS(5072),
    [anon_sym_gen] = ACTIONS(5072),
    [anon_sym_if] = ACTIONS(5072),
    [anon_sym_impl] = ACTIONS(5072),
    [anon_sym_let] = ACTIONS(5072),
    [anon_sym_loop] = ACTIONS(5072),
    [anon_sym_match] = ACTIONS(5072),
    [anon_sym_mod] = ACTIONS(5072),
    [anon_sym_pub] = ACTIONS(5072),
    [anon_sym_return] = ACTIONS(5072),
    [anon_sym_raw] = ACTIONS(5072),
    [anon_sym_static] = ACTIONS(5072),
    [anon_sym_struct] = ACTIONS(5072),
    [anon_sym_trait] = ACTIONS(5072),
    [anon_sym_type] = ACTIONS(5072),
    [anon_sym_union] = ACTIONS(5072),
    [anon_sym_unsafe] = ACTIONS(5072),
    [anon_sym_use] = ACTIONS(5072),
    [anon_sym_while] = ACTIONS(5072),
    [anon_sym_extern] = ACTIONS(5072),
    [anon_sym_safe] = ACTIONS(5072),
    [anon_sym_yield] = ACTIONS(5072),
    [anon_sym_move] = ACTIONS(5072),
    [anon_sym_try] = ACTIONS(5072),
    [sym_integer_literal] = ACTIONS(5070),
    [aux_sym_string_literal_token1] = ACTIONS(5070),
    [sym_char_literal] = ACTIONS(5070),
    [anon_sym_true] = ACTIONS(5072),
    [anon_sym_false] = ACTIONS(5072),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5072),
    [sym_super] = ACTIONS(5072),
    [sym_crate] = ACTIONS(5072),
    [aux_sym_metavariable_token1] = ACTIONS(5070),
    [sym__raw_string_literal_start] = ACTIONS(5070),
    [sym_float_literal] = ACTIONS(5070),
    [sym__outer_block_doc_comment_start] = ACTIONS(5070),
    [sym__outer_line_doc_comment_start] = ACTIONS(5070),
  },
  [STATE(1335)] = {
    [sym_line_comment] = STATE(1335),
    [sym_block_comment] = STATE(1335),
    [ts_builtin_sym_end] = ACTIONS(5074),
    [sym_identifier] = ACTIONS(5076),
    [anon_sym_SEMI] = ACTIONS(5074),
    [anon_sym_macro] = ACTIONS(5076),
    [anon_sym_LPAREN] = ACTIONS(5074),
    [anon_sym_LBRACE] = ACTIONS(5074),
    [anon_sym_RBRACE] = ACTIONS(5074),
    [anon_sym_macro_rules] = ACTIONS(5076),
    [anon_sym_BANG] = ACTIONS(5074),
    [anon_sym_LBRACK] = ACTIONS(5074),
    [anon_sym_STAR] = ACTIONS(5074),
    [anon_sym_u8] = ACTIONS(5076),
    [anon_sym_i8] = ACTIONS(5076),
    [anon_sym_u16] = ACTIONS(5076),
    [anon_sym_i16] = ACTIONS(5076),
    [anon_sym_u32] = ACTIONS(5076),
    [anon_sym_i32] = ACTIONS(5076),
    [anon_sym_u64] = ACTIONS(5076),
    [anon_sym_i64] = ACTIONS(5076),
    [anon_sym_u128] = ACTIONS(5076),
    [anon_sym_i128] = ACTIONS(5076),
    [anon_sym_isize] = ACTIONS(5076),
    [anon_sym_usize] = ACTIONS(5076),
    [anon_sym_f32] = ACTIONS(5076),
    [anon_sym_f64] = ACTIONS(5076),
    [anon_sym_bool] = ACTIONS(5076),
    [anon_sym_str] = ACTIONS(5076),
    [anon_sym_char] = ACTIONS(5076),
    [anon_sym_DASH] = ACTIONS(5074),
    [anon_sym_AMP] = ACTIONS(5074),
    [anon_sym_PIPE] = ACTIONS(5074),
    [anon_sym_LT] = ACTIONS(5074),
    [anon_sym_DOT_DOT] = ACTIONS(5074),
    [anon_sym_COLON_COLON] = ACTIONS(5074),
    [anon_sym_POUND] = ACTIONS(5074),
    [anon_sym_SQUOTE] = ACTIONS(5076),
    [anon_sym_async] = ACTIONS(5076),
    [anon_sym_auto] = ACTIONS(5076),
    [anon_sym_await] = ACTIONS(5076),
    [anon_sym_become] = ACTIONS(5076),
    [anon_sym_break] = ACTIONS(5076),
    [anon_sym_const] = ACTIONS(5076),
    [anon_sym_continue] = ACTIONS(5076),
    [anon_sym_default] = ACTIONS(5076),
    [anon_sym_enum] = ACTIONS(5076),
    [anon_sym_fn] = ACTIONS(5076),
    [anon_sym_for] = ACTIONS(5076),
    [anon_sym_gen] = ACTIONS(5076),
    [anon_sym_if] = ACTIONS(5076),
    [anon_sym_impl] = ACTIONS(5076),
    [anon_sym_let] = ACTIONS(5076),
    [anon_sym_loop] = ACTIONS(5076),
    [anon_sym_match] = ACTIONS(5076),
    [anon_sym_mod] = ACTIONS(5076),
    [anon_sym_pub] = ACTIONS(5076),
    [anon_sym_return] = ACTIONS(5076),
    [anon_sym_raw] = ACTIONS(5076),
    [anon_sym_static] = ACTIONS(5076),
    [anon_sym_struct] = ACTIONS(5076),
    [anon_sym_trait] = ACTIONS(5076),
    [anon_sym_type] = ACTIONS(5076),
    [anon_sym_union] = ACTIONS(5076),
    [anon_sym_unsafe] = ACTIONS(5076),
    [anon_sym_use] = ACTIONS(5076),
    [anon_sym_while] = ACTIONS(5076),
    [anon_sym_extern] = ACTIONS(5076),
    [anon_sym_safe] = ACTIONS(5076),
    [anon_sym_yield] = ACTIONS(5076),
    [anon_sym_move] = ACTIONS(5076),
    [anon_sym_try] = ACTIONS(5076),
    [sym_integer_literal] = ACTIONS(5074),
    [aux_sym_string_literal_token1] = ACTIONS(5074),
    [sym_char_literal] = ACTIONS(5074),
    [anon_sym_true] = ACTIONS(5076),
    [anon_sym_false] = ACTIONS(5076),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5076),
    [sym_super] = ACTIONS(5076),
    [sym_crate] = ACTIONS(5076),
    [aux_sym_metavariable_token1] = ACTIONS(5074),
    [sym__raw_string_literal_start] = ACTIONS(5074),
    [sym_float_literal] = ACTIONS(5074),
    [sym__outer_block_doc_comment_start] = ACTIONS(5074),
    [sym__outer_line_doc_comment_start] = ACTIONS(5074),
  },
  [STATE(1336)] = {
    [sym_line_comment] = STATE(1336),
    [sym_block_comment] = STATE(1336),
    [ts_builtin_sym_end] = ACTIONS(5078),
    [sym_identifier] = ACTIONS(5080),
    [anon_sym_SEMI] = ACTIONS(5078),
    [anon_sym_macro] = ACTIONS(5080),
    [anon_sym_LPAREN] = ACTIONS(5078),
    [anon_sym_LBRACE] = ACTIONS(5078),
    [anon_sym_RBRACE] = ACTIONS(5078),
    [anon_sym_macro_rules] = ACTIONS(5080),
    [anon_sym_BANG] = ACTIONS(5078),
    [anon_sym_LBRACK] = ACTIONS(5078),
    [anon_sym_STAR] = ACTIONS(5078),
    [anon_sym_u8] = ACTIONS(5080),
    [anon_sym_i8] = ACTIONS(5080),
    [anon_sym_u16] = ACTIONS(5080),
    [anon_sym_i16] = ACTIONS(5080),
    [anon_sym_u32] = ACTIONS(5080),
    [anon_sym_i32] = ACTIONS(5080),
    [anon_sym_u64] = ACTIONS(5080),
    [anon_sym_i64] = ACTIONS(5080),
    [anon_sym_u128] = ACTIONS(5080),
    [anon_sym_i128] = ACTIONS(5080),
    [anon_sym_isize] = ACTIONS(5080),
    [anon_sym_usize] = ACTIONS(5080),
    [anon_sym_f32] = ACTIONS(5080),
    [anon_sym_f64] = ACTIONS(5080),
    [anon_sym_bool] = ACTIONS(5080),
    [anon_sym_str] = ACTIONS(5080),
    [anon_sym_char] = ACTIONS(5080),
    [anon_sym_DASH] = ACTIONS(5078),
    [anon_sym_AMP] = ACTIONS(5078),
    [anon_sym_PIPE] = ACTIONS(5078),
    [anon_sym_LT] = ACTIONS(5078),
    [anon_sym_DOT_DOT] = ACTIONS(5078),
    [anon_sym_COLON_COLON] = ACTIONS(5078),
    [anon_sym_POUND] = ACTIONS(5078),
    [anon_sym_SQUOTE] = ACTIONS(5080),
    [anon_sym_async] = ACTIONS(5080),
    [anon_sym_auto] = ACTIONS(5080),
    [anon_sym_await] = ACTIONS(5080),
    [anon_sym_become] = ACTIONS(5080),
    [anon_sym_break] = ACTIONS(5080),
    [anon_sym_const] = ACTIONS(5080),
    [anon_sym_continue] = ACTIONS(5080),
    [anon_sym_default] = ACTIONS(5080),
    [anon_sym_enum] = ACTIONS(5080),
    [anon_sym_fn] = ACTIONS(5080),
    [anon_sym_for] = ACTIONS(5080),
    [anon_sym_gen] = ACTIONS(5080),
    [anon_sym_if] = ACTIONS(5080),
    [anon_sym_impl] = ACTIONS(5080),
    [anon_sym_let] = ACTIONS(5080),
    [anon_sym_loop] = ACTIONS(5080),
    [anon_sym_match] = ACTIONS(5080),
    [anon_sym_mod] = ACTIONS(5080),
    [anon_sym_pub] = ACTIONS(5080),
    [anon_sym_return] = ACTIONS(5080),
    [anon_sym_raw] = ACTIONS(5080),
    [anon_sym_static] = ACTIONS(5080),
    [anon_sym_struct] = ACTIONS(5080),
    [anon_sym_trait] = ACTIONS(5080),
    [anon_sym_type] = ACTIONS(5080),
    [anon_sym_union] = ACTIONS(5080),
    [anon_sym_unsafe] = ACTIONS(5080),
    [anon_sym_use] = ACTIONS(5080),
    [anon_sym_while] = ACTIONS(5080),
    [anon_sym_extern] = ACTIONS(5080),
    [anon_sym_safe] = ACTIONS(5080),
    [anon_sym_yield] = ACTIONS(5080),
    [anon_sym_move] = ACTIONS(5080),
    [anon_sym_try] = ACTIONS(5080),
    [sym_integer_literal] = ACTIONS(5078),
    [aux_sym_string_literal_token1] = ACTIONS(5078),
    [sym_char_literal] = ACTIONS(5078),
    [anon_sym_true] = ACTIONS(5080),
    [anon_sym_false] = ACTIONS(5080),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5080),
    [sym_super] = ACTIONS(5080),
    [sym_crate] = ACTIONS(5080),
    [aux_sym_metavariable_token1] = ACTIONS(5078),
    [sym__raw_string_literal_start] = ACTIONS(5078),
    [sym_float_literal] = ACTIONS(5078),
    [sym__outer_block_doc_comment_start] = ACTIONS(5078),
    [sym__outer_line_doc_comment_start] = ACTIONS(5078),
  },
  [STATE(1337)] = {
    [sym_line_comment] = STATE(1337),
    [sym_block_comment] = STATE(1337),
    [ts_builtin_sym_end] = ACTIONS(5082),
    [sym_identifier] = ACTIONS(5084),
    [anon_sym_SEMI] = ACTIONS(5082),
    [anon_sym_macro] = ACTIONS(5084),
    [anon_sym_LPAREN] = ACTIONS(5082),
    [anon_sym_LBRACE] = ACTIONS(5082),
    [anon_sym_RBRACE] = ACTIONS(5082),
    [anon_sym_macro_rules] = ACTIONS(5084),
    [anon_sym_BANG] = ACTIONS(5082),
    [anon_sym_LBRACK] = ACTIONS(5082),
    [anon_sym_STAR] = ACTIONS(5082),
    [anon_sym_u8] = ACTIONS(5084),
    [anon_sym_i8] = ACTIONS(5084),
    [anon_sym_u16] = ACTIONS(5084),
    [anon_sym_i16] = ACTIONS(5084),
    [anon_sym_u32] = ACTIONS(5084),
    [anon_sym_i32] = ACTIONS(5084),
    [anon_sym_u64] = ACTIONS(5084),
    [anon_sym_i64] = ACTIONS(5084),
    [anon_sym_u128] = ACTIONS(5084),
    [anon_sym_i128] = ACTIONS(5084),
    [anon_sym_isize] = ACTIONS(5084),
    [anon_sym_usize] = ACTIONS(5084),
    [anon_sym_f32] = ACTIONS(5084),
    [anon_sym_f64] = ACTIONS(5084),
    [anon_sym_bool] = ACTIONS(5084),
    [anon_sym_str] = ACTIONS(5084),
    [anon_sym_char] = ACTIONS(5084),
    [anon_sym_DASH] = ACTIONS(5082),
    [anon_sym_AMP] = ACTIONS(5082),
    [anon_sym_PIPE] = ACTIONS(5082),
    [anon_sym_LT] = ACTIONS(5082),
    [anon_sym_DOT_DOT] = ACTIONS(5082),
    [anon_sym_COLON_COLON] = ACTIONS(5082),
    [anon_sym_POUND] = ACTIONS(5082),
    [anon_sym_SQUOTE] = ACTIONS(5084),
    [anon_sym_async] = ACTIONS(5084),
    [anon_sym_auto] = ACTIONS(5084),
    [anon_sym_await] = ACTIONS(5084),
    [anon_sym_become] = ACTIONS(5084),
    [anon_sym_break] = ACTIONS(5084),
    [anon_sym_const] = ACTIONS(5084),
    [anon_sym_continue] = ACTIONS(5084),
    [anon_sym_default] = ACTIONS(5084),
    [anon_sym_enum] = ACTIONS(5084),
    [anon_sym_fn] = ACTIONS(5084),
    [anon_sym_for] = ACTIONS(5084),
    [anon_sym_gen] = ACTIONS(5084),
    [anon_sym_if] = ACTIONS(5084),
    [anon_sym_impl] = ACTIONS(5084),
    [anon_sym_let] = ACTIONS(5084),
    [anon_sym_loop] = ACTIONS(5084),
    [anon_sym_match] = ACTIONS(5084),
    [anon_sym_mod] = ACTIONS(5084),
    [anon_sym_pub] = ACTIONS(5084),
    [anon_sym_return] = ACTIONS(5084),
    [anon_sym_raw] = ACTIONS(5084),
    [anon_sym_static] = ACTIONS(5084),
    [anon_sym_struct] = ACTIONS(5084),
    [anon_sym_trait] = ACTIONS(5084),
    [anon_sym_type] = ACTIONS(5084),
    [anon_sym_union] = ACTIONS(5084),
    [anon_sym_unsafe] = ACTIONS(5084),
    [anon_sym_use] = ACTIONS(5084),
    [anon_sym_while] = ACTIONS(5084),
    [anon_sym_extern] = ACTIONS(5084),
    [anon_sym_safe] = ACTIONS(5084),
    [anon_sym_yield] = ACTIONS(5084),
    [anon_sym_move] = ACTIONS(5084),
    [anon_sym_try] = ACTIONS(5084),
    [sym_integer_literal] = ACTIONS(5082),
    [aux_sym_string_literal_token1] = ACTIONS(5082),
    [sym_char_literal] = ACTIONS(5082),
    [anon_sym_true] = ACTIONS(5084),
    [anon_sym_false] = ACTIONS(5084),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5084),
    [sym_super] = ACTIONS(5084),
    [sym_crate] = ACTIONS(5084),
    [aux_sym_metavariable_token1] = ACTIONS(5082),
    [sym__raw_string_literal_start] = ACTIONS(5082),
    [sym_float_literal] = ACTIONS(5082),
    [sym__outer_block_doc_comment_start] = ACTIONS(5082),
    [sym__outer_line_doc_comment_start] = ACTIONS(5082),
  },
  [STATE(1338)] = {
    [sym_line_comment] = STATE(1338),
    [sym_block_comment] = STATE(1338),
    [ts_builtin_sym_end] = ACTIONS(5086),
    [sym_identifier] = ACTIONS(5088),
    [anon_sym_SEMI] = ACTIONS(5086),
    [anon_sym_macro] = ACTIONS(5088),
    [anon_sym_LPAREN] = ACTIONS(5086),
    [anon_sym_LBRACE] = ACTIONS(5086),
    [anon_sym_RBRACE] = ACTIONS(5086),
    [anon_sym_macro_rules] = ACTIONS(5088),
    [anon_sym_BANG] = ACTIONS(5086),
    [anon_sym_LBRACK] = ACTIONS(5086),
    [anon_sym_STAR] = ACTIONS(5086),
    [anon_sym_u8] = ACTIONS(5088),
    [anon_sym_i8] = ACTIONS(5088),
    [anon_sym_u16] = ACTIONS(5088),
    [anon_sym_i16] = ACTIONS(5088),
    [anon_sym_u32] = ACTIONS(5088),
    [anon_sym_i32] = ACTIONS(5088),
    [anon_sym_u64] = ACTIONS(5088),
    [anon_sym_i64] = ACTIONS(5088),
    [anon_sym_u128] = ACTIONS(5088),
    [anon_sym_i128] = ACTIONS(5088),
    [anon_sym_isize] = ACTIONS(5088),
    [anon_sym_usize] = ACTIONS(5088),
    [anon_sym_f32] = ACTIONS(5088),
    [anon_sym_f64] = ACTIONS(5088),
    [anon_sym_bool] = ACTIONS(5088),
    [anon_sym_str] = ACTIONS(5088),
    [anon_sym_char] = ACTIONS(5088),
    [anon_sym_DASH] = ACTIONS(5086),
    [anon_sym_AMP] = ACTIONS(5086),
    [anon_sym_PIPE] = ACTIONS(5086),
    [anon_sym_LT] = ACTIONS(5086),
    [anon_sym_DOT_DOT] = ACTIONS(5086),
    [anon_sym_COLON_COLON] = ACTIONS(5086),
    [anon_sym_POUND] = ACTIONS(5086),
    [anon_sym_SQUOTE] = ACTIONS(5088),
    [anon_sym_async] = ACTIONS(5088),
    [anon_sym_auto] = ACTIONS(5088),
    [anon_sym_await] = ACTIONS(5088),
    [anon_sym_become] = ACTIONS(5088),
    [anon_sym_break] = ACTIONS(5088),
    [anon_sym_const] = ACTIONS(5088),
    [anon_sym_continue] = ACTIONS(5088),
    [anon_sym_default] = ACTIONS(5088),
    [anon_sym_enum] = ACTIONS(5088),
    [anon_sym_fn] = ACTIONS(5088),
    [anon_sym_for] = ACTIONS(5088),
    [anon_sym_gen] = ACTIONS(5088),
    [anon_sym_if] = ACTIONS(5088),
    [anon_sym_impl] = ACTIONS(5088),
    [anon_sym_let] = ACTIONS(5088),
    [anon_sym_loop] = ACTIONS(5088),
    [anon_sym_match] = ACTIONS(5088),
    [anon_sym_mod] = ACTIONS(5088),
    [anon_sym_pub] = ACTIONS(5088),
    [anon_sym_return] = ACTIONS(5088),
    [anon_sym_raw] = ACTIONS(5088),
    [anon_sym_static] = ACTIONS(5088),
    [anon_sym_struct] = ACTIONS(5088),
    [anon_sym_trait] = ACTIONS(5088),
    [anon_sym_type] = ACTIONS(5088),
    [anon_sym_union] = ACTIONS(5088),
    [anon_sym_unsafe] = ACTIONS(5088),
    [anon_sym_use] = ACTIONS(5088),
    [anon_sym_while] = ACTIONS(5088),
    [anon_sym_extern] = ACTIONS(5088),
    [anon_sym_safe] = ACTIONS(5088),
    [anon_sym_yield] = ACTIONS(5088),
    [anon_sym_move] = ACTIONS(5088),
    [anon_sym_try] = ACTIONS(5088),
    [sym_integer_literal] = ACTIONS(5086),
    [aux_sym_string_literal_token1] = ACTIONS(5086),
    [sym_char_literal] = ACTIONS(5086),
    [anon_sym_true] = ACTIONS(5088),
    [anon_sym_false] = ACTIONS(5088),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5088),
    [sym_super] = ACTIONS(5088),
    [sym_crate] = ACTIONS(5088),
    [aux_sym_metavariable_token1] = ACTIONS(5086),
    [sym__raw_string_literal_start] = ACTIONS(5086),
    [sym_float_literal] = ACTIONS(5086),
    [sym__outer_block_doc_comment_start] = ACTIONS(5086),
    [sym__outer_line_doc_comment_start] = ACTIONS(5086),
  },
  [STATE(1339)] = {
    [sym_line_comment] = STATE(1339),
    [sym_block_comment] = STATE(1339),
    [ts_builtin_sym_end] = ACTIONS(5090),
    [sym_identifier] = ACTIONS(5092),
    [anon_sym_SEMI] = ACTIONS(5090),
    [anon_sym_macro] = ACTIONS(5092),
    [anon_sym_LPAREN] = ACTIONS(5090),
    [anon_sym_LBRACE] = ACTIONS(5090),
    [anon_sym_RBRACE] = ACTIONS(5090),
    [anon_sym_macro_rules] = ACTIONS(5092),
    [anon_sym_BANG] = ACTIONS(5090),
    [anon_sym_LBRACK] = ACTIONS(5090),
    [anon_sym_STAR] = ACTIONS(5090),
    [anon_sym_u8] = ACTIONS(5092),
    [anon_sym_i8] = ACTIONS(5092),
    [anon_sym_u16] = ACTIONS(5092),
    [anon_sym_i16] = ACTIONS(5092),
    [anon_sym_u32] = ACTIONS(5092),
    [anon_sym_i32] = ACTIONS(5092),
    [anon_sym_u64] = ACTIONS(5092),
    [anon_sym_i64] = ACTIONS(5092),
    [anon_sym_u128] = ACTIONS(5092),
    [anon_sym_i128] = ACTIONS(5092),
    [anon_sym_isize] = ACTIONS(5092),
    [anon_sym_usize] = ACTIONS(5092),
    [anon_sym_f32] = ACTIONS(5092),
    [anon_sym_f64] = ACTIONS(5092),
    [anon_sym_bool] = ACTIONS(5092),
    [anon_sym_str] = ACTIONS(5092),
    [anon_sym_char] = ACTIONS(5092),
    [anon_sym_DASH] = ACTIONS(5090),
    [anon_sym_AMP] = ACTIONS(5090),
    [anon_sym_PIPE] = ACTIONS(5090),
    [anon_sym_LT] = ACTIONS(5090),
    [anon_sym_DOT_DOT] = ACTIONS(5090),
    [anon_sym_COLON_COLON] = ACTIONS(5090),
    [anon_sym_POUND] = ACTIONS(5090),
    [anon_sym_SQUOTE] = ACTIONS(5092),
    [anon_sym_async] = ACTIONS(5092),
    [anon_sym_auto] = ACTIONS(5092),
    [anon_sym_await] = ACTIONS(5092),
    [anon_sym_become] = ACTIONS(5092),
    [anon_sym_break] = ACTIONS(5092),
    [anon_sym_const] = ACTIONS(5092),
    [anon_sym_continue] = ACTIONS(5092),
    [anon_sym_default] = ACTIONS(5092),
    [anon_sym_enum] = ACTIONS(5092),
    [anon_sym_fn] = ACTIONS(5092),
    [anon_sym_for] = ACTIONS(5092),
    [anon_sym_gen] = ACTIONS(5092),
    [anon_sym_if] = ACTIONS(5092),
    [anon_sym_impl] = ACTIONS(5092),
    [anon_sym_let] = ACTIONS(5092),
    [anon_sym_loop] = ACTIONS(5092),
    [anon_sym_match] = ACTIONS(5092),
    [anon_sym_mod] = ACTIONS(5092),
    [anon_sym_pub] = ACTIONS(5092),
    [anon_sym_return] = ACTIONS(5092),
    [anon_sym_raw] = ACTIONS(5092),
    [anon_sym_static] = ACTIONS(5092),
    [anon_sym_struct] = ACTIONS(5092),
    [anon_sym_trait] = ACTIONS(5092),
    [anon_sym_type] = ACTIONS(5092),
    [anon_sym_union] = ACTIONS(5092),
    [anon_sym_unsafe] = ACTIONS(5092),
    [anon_sym_use] = ACTIONS(5092),
    [anon_sym_while] = ACTIONS(5092),
    [anon_sym_extern] = ACTIONS(5092),
    [anon_sym_safe] = ACTIONS(5092),
    [anon_sym_yield] = ACTIONS(5092),
    [anon_sym_move] = ACTIONS(5092),
    [anon_sym_try] = ACTIONS(5092),
    [sym_integer_literal] = ACTIONS(5090),
    [aux_sym_string_literal_token1] = ACTIONS(5090),
    [sym_char_literal] = ACTIONS(5090),
    [anon_sym_true] = ACTIONS(5092),
    [anon_sym_false] = ACTIONS(5092),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5092),
    [sym_super] = ACTIONS(5092),
    [sym_crate] = ACTIONS(5092),
    [aux_sym_metavariable_token1] = ACTIONS(5090),
    [sym__raw_string_literal_start] = ACTIONS(5090),
    [sym_float_literal] = ACTIONS(5090),
    [sym__outer_block_doc_comment_start] = ACTIONS(5090),
    [sym__outer_line_doc_comment_start] = ACTIONS(5090),
  },
  [STATE(1340)] = {
    [sym_line_comment] = STATE(1340),
    [sym_block_comment] = STATE(1340),
    [ts_builtin_sym_end] = ACTIONS(5094),
    [sym_identifier] = ACTIONS(5096),
    [anon_sym_SEMI] = ACTIONS(5094),
    [anon_sym_macro] = ACTIONS(5096),
    [anon_sym_LPAREN] = ACTIONS(5094),
    [anon_sym_LBRACE] = ACTIONS(5094),
    [anon_sym_RBRACE] = ACTIONS(5094),
    [anon_sym_macro_rules] = ACTIONS(5096),
    [anon_sym_BANG] = ACTIONS(5094),
    [anon_sym_LBRACK] = ACTIONS(5094),
    [anon_sym_STAR] = ACTIONS(5094),
    [anon_sym_u8] = ACTIONS(5096),
    [anon_sym_i8] = ACTIONS(5096),
    [anon_sym_u16] = ACTIONS(5096),
    [anon_sym_i16] = ACTIONS(5096),
    [anon_sym_u32] = ACTIONS(5096),
    [anon_sym_i32] = ACTIONS(5096),
    [anon_sym_u64] = ACTIONS(5096),
    [anon_sym_i64] = ACTIONS(5096),
    [anon_sym_u128] = ACTIONS(5096),
    [anon_sym_i128] = ACTIONS(5096),
    [anon_sym_isize] = ACTIONS(5096),
    [anon_sym_usize] = ACTIONS(5096),
    [anon_sym_f32] = ACTIONS(5096),
    [anon_sym_f64] = ACTIONS(5096),
    [anon_sym_bool] = ACTIONS(5096),
    [anon_sym_str] = ACTIONS(5096),
    [anon_sym_char] = ACTIONS(5096),
    [anon_sym_DASH] = ACTIONS(5094),
    [anon_sym_AMP] = ACTIONS(5094),
    [anon_sym_PIPE] = ACTIONS(5094),
    [anon_sym_LT] = ACTIONS(5094),
    [anon_sym_DOT_DOT] = ACTIONS(5094),
    [anon_sym_COLON_COLON] = ACTIONS(5094),
    [anon_sym_POUND] = ACTIONS(5094),
    [anon_sym_SQUOTE] = ACTIONS(5096),
    [anon_sym_async] = ACTIONS(5096),
    [anon_sym_auto] = ACTIONS(5096),
    [anon_sym_await] = ACTIONS(5096),
    [anon_sym_become] = ACTIONS(5096),
    [anon_sym_break] = ACTIONS(5096),
    [anon_sym_const] = ACTIONS(5096),
    [anon_sym_continue] = ACTIONS(5096),
    [anon_sym_default] = ACTIONS(5096),
    [anon_sym_enum] = ACTIONS(5096),
    [anon_sym_fn] = ACTIONS(5096),
    [anon_sym_for] = ACTIONS(5096),
    [anon_sym_gen] = ACTIONS(5096),
    [anon_sym_if] = ACTIONS(5096),
    [anon_sym_impl] = ACTIONS(5096),
    [anon_sym_let] = ACTIONS(5096),
    [anon_sym_loop] = ACTIONS(5096),
    [anon_sym_match] = ACTIONS(5096),
    [anon_sym_mod] = ACTIONS(5096),
    [anon_sym_pub] = ACTIONS(5096),
    [anon_sym_return] = ACTIONS(5096),
    [anon_sym_raw] = ACTIONS(5096),
    [anon_sym_static] = ACTIONS(5096),
    [anon_sym_struct] = ACTIONS(5096),
    [anon_sym_trait] = ACTIONS(5096),
    [anon_sym_type] = ACTIONS(5096),
    [anon_sym_union] = ACTIONS(5096),
    [anon_sym_unsafe] = ACTIONS(5096),
    [anon_sym_use] = ACTIONS(5096),
    [anon_sym_while] = ACTIONS(5096),
    [anon_sym_extern] = ACTIONS(5096),
    [anon_sym_safe] = ACTIONS(5096),
    [anon_sym_yield] = ACTIONS(5096),
    [anon_sym_move] = ACTIONS(5096),
    [anon_sym_try] = ACTIONS(5096),
    [sym_integer_literal] = ACTIONS(5094),
    [aux_sym_string_literal_token1] = ACTIONS(5094),
    [sym_char_literal] = ACTIONS(5094),
    [anon_sym_true] = ACTIONS(5096),
    [anon_sym_false] = ACTIONS(5096),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5096),
    [sym_super] = ACTIONS(5096),
    [sym_crate] = ACTIONS(5096),
    [aux_sym_metavariable_token1] = ACTIONS(5094),
    [sym__raw_string_literal_start] = ACTIONS(5094),
    [sym_float_literal] = ACTIONS(5094),
    [sym__outer_block_doc_comment_start] = ACTIONS(5094),
    [sym__outer_line_doc_comment_start] = ACTIONS(5094),
  },
  [STATE(1341)] = {
    [sym_line_comment] = STATE(1341),
    [sym_block_comment] = STATE(1341),
    [ts_builtin_sym_end] = ACTIONS(5098),
    [sym_identifier] = ACTIONS(5100),
    [anon_sym_SEMI] = ACTIONS(5098),
    [anon_sym_macro] = ACTIONS(5100),
    [anon_sym_LPAREN] = ACTIONS(5098),
    [anon_sym_LBRACE] = ACTIONS(5098),
    [anon_sym_RBRACE] = ACTIONS(5098),
    [anon_sym_macro_rules] = ACTIONS(5100),
    [anon_sym_BANG] = ACTIONS(5098),
    [anon_sym_LBRACK] = ACTIONS(5098),
    [anon_sym_STAR] = ACTIONS(5098),
    [anon_sym_u8] = ACTIONS(5100),
    [anon_sym_i8] = ACTIONS(5100),
    [anon_sym_u16] = ACTIONS(5100),
    [anon_sym_i16] = ACTIONS(5100),
    [anon_sym_u32] = ACTIONS(5100),
    [anon_sym_i32] = ACTIONS(5100),
    [anon_sym_u64] = ACTIONS(5100),
    [anon_sym_i64] = ACTIONS(5100),
    [anon_sym_u128] = ACTIONS(5100),
    [anon_sym_i128] = ACTIONS(5100),
    [anon_sym_isize] = ACTIONS(5100),
    [anon_sym_usize] = ACTIONS(5100),
    [anon_sym_f32] = ACTIONS(5100),
    [anon_sym_f64] = ACTIONS(5100),
    [anon_sym_bool] = ACTIONS(5100),
    [anon_sym_str] = ACTIONS(5100),
    [anon_sym_char] = ACTIONS(5100),
    [anon_sym_DASH] = ACTIONS(5098),
    [anon_sym_AMP] = ACTIONS(5098),
    [anon_sym_PIPE] = ACTIONS(5098),
    [anon_sym_LT] = ACTIONS(5098),
    [anon_sym_DOT_DOT] = ACTIONS(5098),
    [anon_sym_COLON_COLON] = ACTIONS(5098),
    [anon_sym_POUND] = ACTIONS(5098),
    [anon_sym_SQUOTE] = ACTIONS(5100),
    [anon_sym_async] = ACTIONS(5100),
    [anon_sym_auto] = ACTIONS(5100),
    [anon_sym_await] = ACTIONS(5100),
    [anon_sym_become] = ACTIONS(5100),
    [anon_sym_break] = ACTIONS(5100),
    [anon_sym_const] = ACTIONS(5100),
    [anon_sym_continue] = ACTIONS(5100),
    [anon_sym_default] = ACTIONS(5100),
    [anon_sym_enum] = ACTIONS(5100),
    [anon_sym_fn] = ACTIONS(5100),
    [anon_sym_for] = ACTIONS(5100),
    [anon_sym_gen] = ACTIONS(5100),
    [anon_sym_if] = ACTIONS(5100),
    [anon_sym_impl] = ACTIONS(5100),
    [anon_sym_let] = ACTIONS(5100),
    [anon_sym_loop] = ACTIONS(5100),
    [anon_sym_match] = ACTIONS(5100),
    [anon_sym_mod] = ACTIONS(5100),
    [anon_sym_pub] = ACTIONS(5100),
    [anon_sym_return] = ACTIONS(5100),
    [anon_sym_raw] = ACTIONS(5100),
    [anon_sym_static] = ACTIONS(5100),
    [anon_sym_struct] = ACTIONS(5100),
    [anon_sym_trait] = ACTIONS(5100),
    [anon_sym_type] = ACTIONS(5100),
    [anon_sym_union] = ACTIONS(5100),
    [anon_sym_unsafe] = ACTIONS(5100),
    [anon_sym_use] = ACTIONS(5100),
    [anon_sym_while] = ACTIONS(5100),
    [anon_sym_extern] = ACTIONS(5100),
    [anon_sym_safe] = ACTIONS(5100),
    [anon_sym_yield] = ACTIONS(5100),
    [anon_sym_move] = ACTIONS(5100),
    [anon_sym_try] = ACTIONS(5100),
    [sym_integer_literal] = ACTIONS(5098),
    [aux_sym_string_literal_token1] = ACTIONS(5098),
    [sym_char_literal] = ACTIONS(5098),
    [anon_sym_true] = ACTIONS(5100),
    [anon_sym_false] = ACTIONS(5100),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5100),
    [sym_super] = ACTIONS(5100),
    [sym_crate] = ACTIONS(5100),
    [aux_sym_metavariable_token1] = ACTIONS(5098),
    [sym__raw_string_literal_start] = ACTIONS(5098),
    [sym_float_literal] = ACTIONS(5098),
    [sym__outer_block_doc_comment_start] = ACTIONS(5098),
    [sym__outer_line_doc_comment_start] = ACTIONS(5098),
  },
  [STATE(1342)] = {
    [sym_line_comment] = STATE(1342),
    [sym_block_comment] = STATE(1342),
    [ts_builtin_sym_end] = ACTIONS(5102),
    [sym_identifier] = ACTIONS(5104),
    [anon_sym_SEMI] = ACTIONS(5102),
    [anon_sym_macro] = ACTIONS(5104),
    [anon_sym_LPAREN] = ACTIONS(5102),
    [anon_sym_LBRACE] = ACTIONS(5102),
    [anon_sym_RBRACE] = ACTIONS(5102),
    [anon_sym_macro_rules] = ACTIONS(5104),
    [anon_sym_BANG] = ACTIONS(5102),
    [anon_sym_LBRACK] = ACTIONS(5102),
    [anon_sym_STAR] = ACTIONS(5102),
    [anon_sym_u8] = ACTIONS(5104),
    [anon_sym_i8] = ACTIONS(5104),
    [anon_sym_u16] = ACTIONS(5104),
    [anon_sym_i16] = ACTIONS(5104),
    [anon_sym_u32] = ACTIONS(5104),
    [anon_sym_i32] = ACTIONS(5104),
    [anon_sym_u64] = ACTIONS(5104),
    [anon_sym_i64] = ACTIONS(5104),
    [anon_sym_u128] = ACTIONS(5104),
    [anon_sym_i128] = ACTIONS(5104),
    [anon_sym_isize] = ACTIONS(5104),
    [anon_sym_usize] = ACTIONS(5104),
    [anon_sym_f32] = ACTIONS(5104),
    [anon_sym_f64] = ACTIONS(5104),
    [anon_sym_bool] = ACTIONS(5104),
    [anon_sym_str] = ACTIONS(5104),
    [anon_sym_char] = ACTIONS(5104),
    [anon_sym_DASH] = ACTIONS(5102),
    [anon_sym_AMP] = ACTIONS(5102),
    [anon_sym_PIPE] = ACTIONS(5102),
    [anon_sym_LT] = ACTIONS(5102),
    [anon_sym_DOT_DOT] = ACTIONS(5102),
    [anon_sym_COLON_COLON] = ACTIONS(5102),
    [anon_sym_POUND] = ACTIONS(5102),
    [anon_sym_SQUOTE] = ACTIONS(5104),
    [anon_sym_async] = ACTIONS(5104),
    [anon_sym_auto] = ACTIONS(5104),
    [anon_sym_await] = ACTIONS(5104),
    [anon_sym_become] = ACTIONS(5104),
    [anon_sym_break] = ACTIONS(5104),
    [anon_sym_const] = ACTIONS(5104),
    [anon_sym_continue] = ACTIONS(5104),
    [anon_sym_default] = ACTIONS(5104),
    [anon_sym_enum] = ACTIONS(5104),
    [anon_sym_fn] = ACTIONS(5104),
    [anon_sym_for] = ACTIONS(5104),
    [anon_sym_gen] = ACTIONS(5104),
    [anon_sym_if] = ACTIONS(5104),
    [anon_sym_impl] = ACTIONS(5104),
    [anon_sym_let] = ACTIONS(5104),
    [anon_sym_loop] = ACTIONS(5104),
    [anon_sym_match] = ACTIONS(5104),
    [anon_sym_mod] = ACTIONS(5104),
    [anon_sym_pub] = ACTIONS(5104),
    [anon_sym_return] = ACTIONS(5104),
    [anon_sym_raw] = ACTIONS(5104),
    [anon_sym_static] = ACTIONS(5104),
    [anon_sym_struct] = ACTIONS(5104),
    [anon_sym_trait] = ACTIONS(5104),
    [anon_sym_type] = ACTIONS(5104),
    [anon_sym_union] = ACTIONS(5104),
    [anon_sym_unsafe] = ACTIONS(5104),
    [anon_sym_use] = ACTIONS(5104),
    [anon_sym_while] = ACTIONS(5104),
    [anon_sym_extern] = ACTIONS(5104),
    [anon_sym_safe] = ACTIONS(5104),
    [anon_sym_yield] = ACTIONS(5104),
    [anon_sym_move] = ACTIONS(5104),
    [anon_sym_try] = ACTIONS(5104),
    [sym_integer_literal] = ACTIONS(5102),
    [aux_sym_string_literal_token1] = ACTIONS(5102),
    [sym_char_literal] = ACTIONS(5102),
    [anon_sym_true] = ACTIONS(5104),
    [anon_sym_false] = ACTIONS(5104),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5104),
    [sym_super] = ACTIONS(5104),
    [sym_crate] = ACTIONS(5104),
    [aux_sym_metavariable_token1] = ACTIONS(5102),
    [sym__raw_string_literal_start] = ACTIONS(5102),
    [sym_float_literal] = ACTIONS(5102),
    [sym__outer_block_doc_comment_start] = ACTIONS(5102),
    [sym__outer_line_doc_comment_start] = ACTIONS(5102),
  },
  [STATE(1343)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3837),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(3837),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1343),
    [sym_block_comment] = STATE(1343),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(5106),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5106),
    [anon_sym_COMMA] = ACTIONS(5106),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5108),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_EQ] = ACTIONS(5106),
    [anon_sym_GT] = ACTIONS(5106),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(5110),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_where] = ACTIONS(5112),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_TILDEconst] = ACTIONS(5114),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1344)] = {
    [sym_line_comment] = STATE(1344),
    [sym_block_comment] = STATE(1344),
    [ts_builtin_sym_end] = ACTIONS(5116),
    [sym_identifier] = ACTIONS(5118),
    [anon_sym_SEMI] = ACTIONS(5116),
    [anon_sym_macro] = ACTIONS(5118),
    [anon_sym_LPAREN] = ACTIONS(5116),
    [anon_sym_LBRACE] = ACTIONS(5116),
    [anon_sym_RBRACE] = ACTIONS(5116),
    [anon_sym_macro_rules] = ACTIONS(5118),
    [anon_sym_BANG] = ACTIONS(5116),
    [anon_sym_LBRACK] = ACTIONS(5116),
    [anon_sym_STAR] = ACTIONS(5116),
    [anon_sym_u8] = ACTIONS(5118),
    [anon_sym_i8] = ACTIONS(5118),
    [anon_sym_u16] = ACTIONS(5118),
    [anon_sym_i16] = ACTIONS(5118),
    [anon_sym_u32] = ACTIONS(5118),
    [anon_sym_i32] = ACTIONS(5118),
    [anon_sym_u64] = ACTIONS(5118),
    [anon_sym_i64] = ACTIONS(5118),
    [anon_sym_u128] = ACTIONS(5118),
    [anon_sym_i128] = ACTIONS(5118),
    [anon_sym_isize] = ACTIONS(5118),
    [anon_sym_usize] = ACTIONS(5118),
    [anon_sym_f32] = ACTIONS(5118),
    [anon_sym_f64] = ACTIONS(5118),
    [anon_sym_bool] = ACTIONS(5118),
    [anon_sym_str] = ACTIONS(5118),
    [anon_sym_char] = ACTIONS(5118),
    [anon_sym_DASH] = ACTIONS(5116),
    [anon_sym_AMP] = ACTIONS(5116),
    [anon_sym_PIPE] = ACTIONS(5116),
    [anon_sym_LT] = ACTIONS(5116),
    [anon_sym_DOT_DOT] = ACTIONS(5116),
    [anon_sym_COLON_COLON] = ACTIONS(5116),
    [anon_sym_POUND] = ACTIONS(5116),
    [anon_sym_SQUOTE] = ACTIONS(5118),
    [anon_sym_async] = ACTIONS(5118),
    [anon_sym_auto] = ACTIONS(5118),
    [anon_sym_await] = ACTIONS(5118),
    [anon_sym_become] = ACTIONS(5118),
    [anon_sym_break] = ACTIONS(5118),
    [anon_sym_const] = ACTIONS(5118),
    [anon_sym_continue] = ACTIONS(5118),
    [anon_sym_default] = ACTIONS(5118),
    [anon_sym_enum] = ACTIONS(5118),
    [anon_sym_fn] = ACTIONS(5118),
    [anon_sym_for] = ACTIONS(5118),
    [anon_sym_gen] = ACTIONS(5118),
    [anon_sym_if] = ACTIONS(5118),
    [anon_sym_impl] = ACTIONS(5118),
    [anon_sym_let] = ACTIONS(5118),
    [anon_sym_loop] = ACTIONS(5118),
    [anon_sym_match] = ACTIONS(5118),
    [anon_sym_mod] = ACTIONS(5118),
    [anon_sym_pub] = ACTIONS(5118),
    [anon_sym_return] = ACTIONS(5118),
    [anon_sym_raw] = ACTIONS(5118),
    [anon_sym_static] = ACTIONS(5118),
    [anon_sym_struct] = ACTIONS(5118),
    [anon_sym_trait] = ACTIONS(5118),
    [anon_sym_type] = ACTIONS(5118),
    [anon_sym_union] = ACTIONS(5118),
    [anon_sym_unsafe] = ACTIONS(5118),
    [anon_sym_use] = ACTIONS(5118),
    [anon_sym_while] = ACTIONS(5118),
    [anon_sym_extern] = ACTIONS(5118),
    [anon_sym_safe] = ACTIONS(5118),
    [anon_sym_yield] = ACTIONS(5118),
    [anon_sym_move] = ACTIONS(5118),
    [anon_sym_try] = ACTIONS(5118),
    [sym_integer_literal] = ACTIONS(5116),
    [aux_sym_string_literal_token1] = ACTIONS(5116),
    [sym_char_literal] = ACTIONS(5116),
    [anon_sym_true] = ACTIONS(5118),
    [anon_sym_false] = ACTIONS(5118),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5118),
    [sym_super] = ACTIONS(5118),
    [sym_crate] = ACTIONS(5118),
    [aux_sym_metavariable_token1] = ACTIONS(5116),
    [sym__raw_string_literal_start] = ACTIONS(5116),
    [sym_float_literal] = ACTIONS(5116),
    [sym__outer_block_doc_comment_start] = ACTIONS(5116),
    [sym__outer_line_doc_comment_start] = ACTIONS(5116),
  },
  [STATE(1345)] = {
    [sym_line_comment] = STATE(1345),
    [sym_block_comment] = STATE(1345),
    [ts_builtin_sym_end] = ACTIONS(5120),
    [sym_identifier] = ACTIONS(5122),
    [anon_sym_SEMI] = ACTIONS(5120),
    [anon_sym_macro] = ACTIONS(5122),
    [anon_sym_LPAREN] = ACTIONS(5120),
    [anon_sym_LBRACE] = ACTIONS(5120),
    [anon_sym_RBRACE] = ACTIONS(5120),
    [anon_sym_macro_rules] = ACTIONS(5122),
    [anon_sym_BANG] = ACTIONS(5120),
    [anon_sym_LBRACK] = ACTIONS(5120),
    [anon_sym_STAR] = ACTIONS(5120),
    [anon_sym_u8] = ACTIONS(5122),
    [anon_sym_i8] = ACTIONS(5122),
    [anon_sym_u16] = ACTIONS(5122),
    [anon_sym_i16] = ACTIONS(5122),
    [anon_sym_u32] = ACTIONS(5122),
    [anon_sym_i32] = ACTIONS(5122),
    [anon_sym_u64] = ACTIONS(5122),
    [anon_sym_i64] = ACTIONS(5122),
    [anon_sym_u128] = ACTIONS(5122),
    [anon_sym_i128] = ACTIONS(5122),
    [anon_sym_isize] = ACTIONS(5122),
    [anon_sym_usize] = ACTIONS(5122),
    [anon_sym_f32] = ACTIONS(5122),
    [anon_sym_f64] = ACTIONS(5122),
    [anon_sym_bool] = ACTIONS(5122),
    [anon_sym_str] = ACTIONS(5122),
    [anon_sym_char] = ACTIONS(5122),
    [anon_sym_DASH] = ACTIONS(5120),
    [anon_sym_AMP] = ACTIONS(5120),
    [anon_sym_PIPE] = ACTIONS(5120),
    [anon_sym_LT] = ACTIONS(5120),
    [anon_sym_DOT_DOT] = ACTIONS(5120),
    [anon_sym_COLON_COLON] = ACTIONS(5120),
    [anon_sym_POUND] = ACTIONS(5120),
    [anon_sym_SQUOTE] = ACTIONS(5122),
    [anon_sym_async] = ACTIONS(5122),
    [anon_sym_auto] = ACTIONS(5122),
    [anon_sym_await] = ACTIONS(5122),
    [anon_sym_become] = ACTIONS(5122),
    [anon_sym_break] = ACTIONS(5122),
    [anon_sym_const] = ACTIONS(5122),
    [anon_sym_continue] = ACTIONS(5122),
    [anon_sym_default] = ACTIONS(5122),
    [anon_sym_enum] = ACTIONS(5122),
    [anon_sym_fn] = ACTIONS(5122),
    [anon_sym_for] = ACTIONS(5122),
    [anon_sym_gen] = ACTIONS(5122),
    [anon_sym_if] = ACTIONS(5122),
    [anon_sym_impl] = ACTIONS(5122),
    [anon_sym_let] = ACTIONS(5122),
    [anon_sym_loop] = ACTIONS(5122),
    [anon_sym_match] = ACTIONS(5122),
    [anon_sym_mod] = ACTIONS(5122),
    [anon_sym_pub] = ACTIONS(5122),
    [anon_sym_return] = ACTIONS(5122),
    [anon_sym_raw] = ACTIONS(5122),
    [anon_sym_static] = ACTIONS(5122),
    [anon_sym_struct] = ACTIONS(5122),
    [anon_sym_trait] = ACTIONS(5122),
    [anon_sym_type] = ACTIONS(5122),
    [anon_sym_union] = ACTIONS(5122),
    [anon_sym_unsafe] = ACTIONS(5122),
    [anon_sym_use] = ACTIONS(5122),
    [anon_sym_while] = ACTIONS(5122),
    [anon_sym_extern] = ACTIONS(5122),
    [anon_sym_safe] = ACTIONS(5122),
    [anon_sym_yield] = ACTIONS(5122),
    [anon_sym_move] = ACTIONS(5122),
    [anon_sym_try] = ACTIONS(5122),
    [sym_integer_literal] = ACTIONS(5120),
    [aux_sym_string_literal_token1] = ACTIONS(5120),
    [sym_char_literal] = ACTIONS(5120),
    [anon_sym_true] = ACTIONS(5122),
    [anon_sym_false] = ACTIONS(5122),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5122),
    [sym_super] = ACTIONS(5122),
    [sym_crate] = ACTIONS(5122),
    [aux_sym_metavariable_token1] = ACTIONS(5120),
    [sym__raw_string_literal_start] = ACTIONS(5120),
    [sym_float_literal] = ACTIONS(5120),
    [sym__outer_block_doc_comment_start] = ACTIONS(5120),
    [sym__outer_line_doc_comment_start] = ACTIONS(5120),
  },
  [STATE(1346)] = {
    [sym_line_comment] = STATE(1346),
    [sym_block_comment] = STATE(1346),
    [ts_builtin_sym_end] = ACTIONS(5124),
    [sym_identifier] = ACTIONS(5126),
    [anon_sym_SEMI] = ACTIONS(5124),
    [anon_sym_macro] = ACTIONS(5126),
    [anon_sym_LPAREN] = ACTIONS(5124),
    [anon_sym_LBRACE] = ACTIONS(5124),
    [anon_sym_RBRACE] = ACTIONS(5124),
    [anon_sym_macro_rules] = ACTIONS(5126),
    [anon_sym_BANG] = ACTIONS(5124),
    [anon_sym_LBRACK] = ACTIONS(5124),
    [anon_sym_STAR] = ACTIONS(5124),
    [anon_sym_u8] = ACTIONS(5126),
    [anon_sym_i8] = ACTIONS(5126),
    [anon_sym_u16] = ACTIONS(5126),
    [anon_sym_i16] = ACTIONS(5126),
    [anon_sym_u32] = ACTIONS(5126),
    [anon_sym_i32] = ACTIONS(5126),
    [anon_sym_u64] = ACTIONS(5126),
    [anon_sym_i64] = ACTIONS(5126),
    [anon_sym_u128] = ACTIONS(5126),
    [anon_sym_i128] = ACTIONS(5126),
    [anon_sym_isize] = ACTIONS(5126),
    [anon_sym_usize] = ACTIONS(5126),
    [anon_sym_f32] = ACTIONS(5126),
    [anon_sym_f64] = ACTIONS(5126),
    [anon_sym_bool] = ACTIONS(5126),
    [anon_sym_str] = ACTIONS(5126),
    [anon_sym_char] = ACTIONS(5126),
    [anon_sym_DASH] = ACTIONS(5124),
    [anon_sym_AMP] = ACTIONS(5124),
    [anon_sym_PIPE] = ACTIONS(5124),
    [anon_sym_LT] = ACTIONS(5124),
    [anon_sym_DOT_DOT] = ACTIONS(5124),
    [anon_sym_COLON_COLON] = ACTIONS(5124),
    [anon_sym_POUND] = ACTIONS(5124),
    [anon_sym_SQUOTE] = ACTIONS(5126),
    [anon_sym_async] = ACTIONS(5126),
    [anon_sym_auto] = ACTIONS(5126),
    [anon_sym_await] = ACTIONS(5126),
    [anon_sym_become] = ACTIONS(5126),
    [anon_sym_break] = ACTIONS(5126),
    [anon_sym_const] = ACTIONS(5126),
    [anon_sym_continue] = ACTIONS(5126),
    [anon_sym_default] = ACTIONS(5126),
    [anon_sym_enum] = ACTIONS(5126),
    [anon_sym_fn] = ACTIONS(5126),
    [anon_sym_for] = ACTIONS(5126),
    [anon_sym_gen] = ACTIONS(5126),
    [anon_sym_if] = ACTIONS(5126),
    [anon_sym_impl] = ACTIONS(5126),
    [anon_sym_let] = ACTIONS(5126),
    [anon_sym_loop] = ACTIONS(5126),
    [anon_sym_match] = ACTIONS(5126),
    [anon_sym_mod] = ACTIONS(5126),
    [anon_sym_pub] = ACTIONS(5126),
    [anon_sym_return] = ACTIONS(5126),
    [anon_sym_raw] = ACTIONS(5126),
    [anon_sym_static] = ACTIONS(5126),
    [anon_sym_struct] = ACTIONS(5126),
    [anon_sym_trait] = ACTIONS(5126),
    [anon_sym_type] = ACTIONS(5126),
    [anon_sym_union] = ACTIONS(5126),
    [anon_sym_unsafe] = ACTIONS(5126),
    [anon_sym_use] = ACTIONS(5126),
    [anon_sym_while] = ACTIONS(5126),
    [anon_sym_extern] = ACTIONS(5126),
    [anon_sym_safe] = ACTIONS(5126),
    [anon_sym_yield] = ACTIONS(5126),
    [anon_sym_move] = ACTIONS(5126),
    [anon_sym_try] = ACTIONS(5126),
    [sym_integer_literal] = ACTIONS(5124),
    [aux_sym_string_literal_token1] = ACTIONS(5124),
    [sym_char_literal] = ACTIONS(5124),
    [anon_sym_true] = ACTIONS(5126),
    [anon_sym_false] = ACTIONS(5126),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5126),
    [sym_super] = ACTIONS(5126),
    [sym_crate] = ACTIONS(5126),
    [aux_sym_metavariable_token1] = ACTIONS(5124),
    [sym__raw_string_literal_start] = ACTIONS(5124),
    [sym_float_literal] = ACTIONS(5124),
    [sym__outer_block_doc_comment_start] = ACTIONS(5124),
    [sym__outer_line_doc_comment_start] = ACTIONS(5124),
  },
  [STATE(1347)] = {
    [sym_line_comment] = STATE(1347),
    [sym_block_comment] = STATE(1347),
    [ts_builtin_sym_end] = ACTIONS(5128),
    [sym_identifier] = ACTIONS(5130),
    [anon_sym_SEMI] = ACTIONS(5128),
    [anon_sym_macro] = ACTIONS(5130),
    [anon_sym_LPAREN] = ACTIONS(5128),
    [anon_sym_LBRACE] = ACTIONS(5128),
    [anon_sym_RBRACE] = ACTIONS(5128),
    [anon_sym_macro_rules] = ACTIONS(5130),
    [anon_sym_BANG] = ACTIONS(5128),
    [anon_sym_LBRACK] = ACTIONS(5128),
    [anon_sym_STAR] = ACTIONS(5128),
    [anon_sym_u8] = ACTIONS(5130),
    [anon_sym_i8] = ACTIONS(5130),
    [anon_sym_u16] = ACTIONS(5130),
    [anon_sym_i16] = ACTIONS(5130),
    [anon_sym_u32] = ACTIONS(5130),
    [anon_sym_i32] = ACTIONS(5130),
    [anon_sym_u64] = ACTIONS(5130),
    [anon_sym_i64] = ACTIONS(5130),
    [anon_sym_u128] = ACTIONS(5130),
    [anon_sym_i128] = ACTIONS(5130),
    [anon_sym_isize] = ACTIONS(5130),
    [anon_sym_usize] = ACTIONS(5130),
    [anon_sym_f32] = ACTIONS(5130),
    [anon_sym_f64] = ACTIONS(5130),
    [anon_sym_bool] = ACTIONS(5130),
    [anon_sym_str] = ACTIONS(5130),
    [anon_sym_char] = ACTIONS(5130),
    [anon_sym_DASH] = ACTIONS(5128),
    [anon_sym_AMP] = ACTIONS(5128),
    [anon_sym_PIPE] = ACTIONS(5128),
    [anon_sym_LT] = ACTIONS(5128),
    [anon_sym_DOT_DOT] = ACTIONS(5128),
    [anon_sym_COLON_COLON] = ACTIONS(5128),
    [anon_sym_POUND] = ACTIONS(5128),
    [anon_sym_SQUOTE] = ACTIONS(5130),
    [anon_sym_async] = ACTIONS(5130),
    [anon_sym_auto] = ACTIONS(5130),
    [anon_sym_await] = ACTIONS(5130),
    [anon_sym_become] = ACTIONS(5130),
    [anon_sym_break] = ACTIONS(5130),
    [anon_sym_const] = ACTIONS(5130),
    [anon_sym_continue] = ACTIONS(5130),
    [anon_sym_default] = ACTIONS(5130),
    [anon_sym_enum] = ACTIONS(5130),
    [anon_sym_fn] = ACTIONS(5130),
    [anon_sym_for] = ACTIONS(5130),
    [anon_sym_gen] = ACTIONS(5130),
    [anon_sym_if] = ACTIONS(5130),
    [anon_sym_impl] = ACTIONS(5130),
    [anon_sym_let] = ACTIONS(5130),
    [anon_sym_loop] = ACTIONS(5130),
    [anon_sym_match] = ACTIONS(5130),
    [anon_sym_mod] = ACTIONS(5130),
    [anon_sym_pub] = ACTIONS(5130),
    [anon_sym_return] = ACTIONS(5130),
    [anon_sym_raw] = ACTIONS(5130),
    [anon_sym_static] = ACTIONS(5130),
    [anon_sym_struct] = ACTIONS(5130),
    [anon_sym_trait] = ACTIONS(5130),
    [anon_sym_type] = ACTIONS(5130),
    [anon_sym_union] = ACTIONS(5130),
    [anon_sym_unsafe] = ACTIONS(5130),
    [anon_sym_use] = ACTIONS(5130),
    [anon_sym_while] = ACTIONS(5130),
    [anon_sym_extern] = ACTIONS(5130),
    [anon_sym_safe] = ACTIONS(5130),
    [anon_sym_yield] = ACTIONS(5130),
    [anon_sym_move] = ACTIONS(5130),
    [anon_sym_try] = ACTIONS(5130),
    [sym_integer_literal] = ACTIONS(5128),
    [aux_sym_string_literal_token1] = ACTIONS(5128),
    [sym_char_literal] = ACTIONS(5128),
    [anon_sym_true] = ACTIONS(5130),
    [anon_sym_false] = ACTIONS(5130),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5130),
    [sym_super] = ACTIONS(5130),
    [sym_crate] = ACTIONS(5130),
    [aux_sym_metavariable_token1] = ACTIONS(5128),
    [sym__raw_string_literal_start] = ACTIONS(5128),
    [sym_float_literal] = ACTIONS(5128),
    [sym__outer_block_doc_comment_start] = ACTIONS(5128),
    [sym__outer_line_doc_comment_start] = ACTIONS(5128),
  },
  [STATE(1348)] = {
    [sym_line_comment] = STATE(1348),
    [sym_block_comment] = STATE(1348),
    [ts_builtin_sym_end] = ACTIONS(5132),
    [sym_identifier] = ACTIONS(5134),
    [anon_sym_SEMI] = ACTIONS(5132),
    [anon_sym_macro] = ACTIONS(5134),
    [anon_sym_LPAREN] = ACTIONS(5132),
    [anon_sym_LBRACE] = ACTIONS(5132),
    [anon_sym_RBRACE] = ACTIONS(5132),
    [anon_sym_macro_rules] = ACTIONS(5134),
    [anon_sym_BANG] = ACTIONS(5132),
    [anon_sym_LBRACK] = ACTIONS(5132),
    [anon_sym_STAR] = ACTIONS(5132),
    [anon_sym_u8] = ACTIONS(5134),
    [anon_sym_i8] = ACTIONS(5134),
    [anon_sym_u16] = ACTIONS(5134),
    [anon_sym_i16] = ACTIONS(5134),
    [anon_sym_u32] = ACTIONS(5134),
    [anon_sym_i32] = ACTIONS(5134),
    [anon_sym_u64] = ACTIONS(5134),
    [anon_sym_i64] = ACTIONS(5134),
    [anon_sym_u128] = ACTIONS(5134),
    [anon_sym_i128] = ACTIONS(5134),
    [anon_sym_isize] = ACTIONS(5134),
    [anon_sym_usize] = ACTIONS(5134),
    [anon_sym_f32] = ACTIONS(5134),
    [anon_sym_f64] = ACTIONS(5134),
    [anon_sym_bool] = ACTIONS(5134),
    [anon_sym_str] = ACTIONS(5134),
    [anon_sym_char] = ACTIONS(5134),
    [anon_sym_DASH] = ACTIONS(5132),
    [anon_sym_AMP] = ACTIONS(5132),
    [anon_sym_PIPE] = ACTIONS(5132),
    [anon_sym_LT] = ACTIONS(5132),
    [anon_sym_DOT_DOT] = ACTIONS(5132),
    [anon_sym_COLON_COLON] = ACTIONS(5132),
    [anon_sym_POUND] = ACTIONS(5132),
    [anon_sym_SQUOTE] = ACTIONS(5134),
    [anon_sym_async] = ACTIONS(5134),
    [anon_sym_auto] = ACTIONS(5134),
    [anon_sym_await] = ACTIONS(5134),
    [anon_sym_become] = ACTIONS(5134),
    [anon_sym_break] = ACTIONS(5134),
    [anon_sym_const] = ACTIONS(5134),
    [anon_sym_continue] = ACTIONS(5134),
    [anon_sym_default] = ACTIONS(5134),
    [anon_sym_enum] = ACTIONS(5134),
    [anon_sym_fn] = ACTIONS(5134),
    [anon_sym_for] = ACTIONS(5134),
    [anon_sym_gen] = ACTIONS(5134),
    [anon_sym_if] = ACTIONS(5134),
    [anon_sym_impl] = ACTIONS(5134),
    [anon_sym_let] = ACTIONS(5134),
    [anon_sym_loop] = ACTIONS(5134),
    [anon_sym_match] = ACTIONS(5134),
    [anon_sym_mod] = ACTIONS(5134),
    [anon_sym_pub] = ACTIONS(5134),
    [anon_sym_return] = ACTIONS(5134),
    [anon_sym_raw] = ACTIONS(5134),
    [anon_sym_static] = ACTIONS(5134),
    [anon_sym_struct] = ACTIONS(5134),
    [anon_sym_trait] = ACTIONS(5134),
    [anon_sym_type] = ACTIONS(5134),
    [anon_sym_union] = ACTIONS(5134),
    [anon_sym_unsafe] = ACTIONS(5134),
    [anon_sym_use] = ACTIONS(5134),
    [anon_sym_while] = ACTIONS(5134),
    [anon_sym_extern] = ACTIONS(5134),
    [anon_sym_safe] = ACTIONS(5134),
    [anon_sym_yield] = ACTIONS(5134),
    [anon_sym_move] = ACTIONS(5134),
    [anon_sym_try] = ACTIONS(5134),
    [sym_integer_literal] = ACTIONS(5132),
    [aux_sym_string_literal_token1] = ACTIONS(5132),
    [sym_char_literal] = ACTIONS(5132),
    [anon_sym_true] = ACTIONS(5134),
    [anon_sym_false] = ACTIONS(5134),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5134),
    [sym_super] = ACTIONS(5134),
    [sym_crate] = ACTIONS(5134),
    [aux_sym_metavariable_token1] = ACTIONS(5132),
    [sym__raw_string_literal_start] = ACTIONS(5132),
    [sym_float_literal] = ACTIONS(5132),
    [sym__outer_block_doc_comment_start] = ACTIONS(5132),
    [sym__outer_line_doc_comment_start] = ACTIONS(5132),
  },
  [STATE(1349)] = {
    [sym_line_comment] = STATE(1349),
    [sym_block_comment] = STATE(1349),
    [ts_builtin_sym_end] = ACTIONS(5136),
    [sym_identifier] = ACTIONS(5138),
    [anon_sym_SEMI] = ACTIONS(5136),
    [anon_sym_macro] = ACTIONS(5138),
    [anon_sym_LPAREN] = ACTIONS(5136),
    [anon_sym_LBRACE] = ACTIONS(5136),
    [anon_sym_RBRACE] = ACTIONS(5136),
    [anon_sym_macro_rules] = ACTIONS(5138),
    [anon_sym_BANG] = ACTIONS(5136),
    [anon_sym_LBRACK] = ACTIONS(5136),
    [anon_sym_STAR] = ACTIONS(5136),
    [anon_sym_u8] = ACTIONS(5138),
    [anon_sym_i8] = ACTIONS(5138),
    [anon_sym_u16] = ACTIONS(5138),
    [anon_sym_i16] = ACTIONS(5138),
    [anon_sym_u32] = ACTIONS(5138),
    [anon_sym_i32] = ACTIONS(5138),
    [anon_sym_u64] = ACTIONS(5138),
    [anon_sym_i64] = ACTIONS(5138),
    [anon_sym_u128] = ACTIONS(5138),
    [anon_sym_i128] = ACTIONS(5138),
    [anon_sym_isize] = ACTIONS(5138),
    [anon_sym_usize] = ACTIONS(5138),
    [anon_sym_f32] = ACTIONS(5138),
    [anon_sym_f64] = ACTIONS(5138),
    [anon_sym_bool] = ACTIONS(5138),
    [anon_sym_str] = ACTIONS(5138),
    [anon_sym_char] = ACTIONS(5138),
    [anon_sym_DASH] = ACTIONS(5136),
    [anon_sym_AMP] = ACTIONS(5136),
    [anon_sym_PIPE] = ACTIONS(5136),
    [anon_sym_LT] = ACTIONS(5136),
    [anon_sym_DOT_DOT] = ACTIONS(5136),
    [anon_sym_COLON_COLON] = ACTIONS(5136),
    [anon_sym_POUND] = ACTIONS(5136),
    [anon_sym_SQUOTE] = ACTIONS(5138),
    [anon_sym_async] = ACTIONS(5138),
    [anon_sym_auto] = ACTIONS(5138),
    [anon_sym_await] = ACTIONS(5138),
    [anon_sym_become] = ACTIONS(5138),
    [anon_sym_break] = ACTIONS(5138),
    [anon_sym_const] = ACTIONS(5138),
    [anon_sym_continue] = ACTIONS(5138),
    [anon_sym_default] = ACTIONS(5138),
    [anon_sym_enum] = ACTIONS(5138),
    [anon_sym_fn] = ACTIONS(5138),
    [anon_sym_for] = ACTIONS(5138),
    [anon_sym_gen] = ACTIONS(5138),
    [anon_sym_if] = ACTIONS(5138),
    [anon_sym_impl] = ACTIONS(5138),
    [anon_sym_let] = ACTIONS(5138),
    [anon_sym_loop] = ACTIONS(5138),
    [anon_sym_match] = ACTIONS(5138),
    [anon_sym_mod] = ACTIONS(5138),
    [anon_sym_pub] = ACTIONS(5138),
    [anon_sym_return] = ACTIONS(5138),
    [anon_sym_raw] = ACTIONS(5138),
    [anon_sym_static] = ACTIONS(5138),
    [anon_sym_struct] = ACTIONS(5138),
    [anon_sym_trait] = ACTIONS(5138),
    [anon_sym_type] = ACTIONS(5138),
    [anon_sym_union] = ACTIONS(5138),
    [anon_sym_unsafe] = ACTIONS(5138),
    [anon_sym_use] = ACTIONS(5138),
    [anon_sym_while] = ACTIONS(5138),
    [anon_sym_extern] = ACTIONS(5138),
    [anon_sym_safe] = ACTIONS(5138),
    [anon_sym_yield] = ACTIONS(5138),
    [anon_sym_move] = ACTIONS(5138),
    [anon_sym_try] = ACTIONS(5138),
    [sym_integer_literal] = ACTIONS(5136),
    [aux_sym_string_literal_token1] = ACTIONS(5136),
    [sym_char_literal] = ACTIONS(5136),
    [anon_sym_true] = ACTIONS(5138),
    [anon_sym_false] = ACTIONS(5138),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5138),
    [sym_super] = ACTIONS(5138),
    [sym_crate] = ACTIONS(5138),
    [aux_sym_metavariable_token1] = ACTIONS(5136),
    [sym__raw_string_literal_start] = ACTIONS(5136),
    [sym_float_literal] = ACTIONS(5136),
    [sym__outer_block_doc_comment_start] = ACTIONS(5136),
    [sym__outer_line_doc_comment_start] = ACTIONS(5136),
  },
  [STATE(1350)] = {
    [sym_line_comment] = STATE(1350),
    [sym_block_comment] = STATE(1350),
    [ts_builtin_sym_end] = ACTIONS(5140),
    [sym_identifier] = ACTIONS(5142),
    [anon_sym_SEMI] = ACTIONS(5140),
    [anon_sym_macro] = ACTIONS(5142),
    [anon_sym_LPAREN] = ACTIONS(5140),
    [anon_sym_LBRACE] = ACTIONS(5140),
    [anon_sym_RBRACE] = ACTIONS(5140),
    [anon_sym_macro_rules] = ACTIONS(5142),
    [anon_sym_BANG] = ACTIONS(5140),
    [anon_sym_LBRACK] = ACTIONS(5140),
    [anon_sym_STAR] = ACTIONS(5140),
    [anon_sym_u8] = ACTIONS(5142),
    [anon_sym_i8] = ACTIONS(5142),
    [anon_sym_u16] = ACTIONS(5142),
    [anon_sym_i16] = ACTIONS(5142),
    [anon_sym_u32] = ACTIONS(5142),
    [anon_sym_i32] = ACTIONS(5142),
    [anon_sym_u64] = ACTIONS(5142),
    [anon_sym_i64] = ACTIONS(5142),
    [anon_sym_u128] = ACTIONS(5142),
    [anon_sym_i128] = ACTIONS(5142),
    [anon_sym_isize] = ACTIONS(5142),
    [anon_sym_usize] = ACTIONS(5142),
    [anon_sym_f32] = ACTIONS(5142),
    [anon_sym_f64] = ACTIONS(5142),
    [anon_sym_bool] = ACTIONS(5142),
    [anon_sym_str] = ACTIONS(5142),
    [anon_sym_char] = ACTIONS(5142),
    [anon_sym_DASH] = ACTIONS(5140),
    [anon_sym_AMP] = ACTIONS(5140),
    [anon_sym_PIPE] = ACTIONS(5140),
    [anon_sym_LT] = ACTIONS(5140),
    [anon_sym_DOT_DOT] = ACTIONS(5140),
    [anon_sym_COLON_COLON] = ACTIONS(5140),
    [anon_sym_POUND] = ACTIONS(5140),
    [anon_sym_SQUOTE] = ACTIONS(5142),
    [anon_sym_async] = ACTIONS(5142),
    [anon_sym_auto] = ACTIONS(5142),
    [anon_sym_await] = ACTIONS(5142),
    [anon_sym_become] = ACTIONS(5142),
    [anon_sym_break] = ACTIONS(5142),
    [anon_sym_const] = ACTIONS(5142),
    [anon_sym_continue] = ACTIONS(5142),
    [anon_sym_default] = ACTIONS(5142),
    [anon_sym_enum] = ACTIONS(5142),
    [anon_sym_fn] = ACTIONS(5142),
    [anon_sym_for] = ACTIONS(5142),
    [anon_sym_gen] = ACTIONS(5142),
    [anon_sym_if] = ACTIONS(5142),
    [anon_sym_impl] = ACTIONS(5142),
    [anon_sym_let] = ACTIONS(5142),
    [anon_sym_loop] = ACTIONS(5142),
    [anon_sym_match] = ACTIONS(5142),
    [anon_sym_mod] = ACTIONS(5142),
    [anon_sym_pub] = ACTIONS(5142),
    [anon_sym_return] = ACTIONS(5142),
    [anon_sym_raw] = ACTIONS(5142),
    [anon_sym_static] = ACTIONS(5142),
    [anon_sym_struct] = ACTIONS(5142),
    [anon_sym_trait] = ACTIONS(5142),
    [anon_sym_type] = ACTIONS(5142),
    [anon_sym_union] = ACTIONS(5142),
    [anon_sym_unsafe] = ACTIONS(5142),
    [anon_sym_use] = ACTIONS(5142),
    [anon_sym_while] = ACTIONS(5142),
    [anon_sym_extern] = ACTIONS(5142),
    [anon_sym_safe] = ACTIONS(5142),
    [anon_sym_yield] = ACTIONS(5142),
    [anon_sym_move] = ACTIONS(5142),
    [anon_sym_try] = ACTIONS(5142),
    [sym_integer_literal] = ACTIONS(5140),
    [aux_sym_string_literal_token1] = ACTIONS(5140),
    [sym_char_literal] = ACTIONS(5140),
    [anon_sym_true] = ACTIONS(5142),
    [anon_sym_false] = ACTIONS(5142),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5142),
    [sym_super] = ACTIONS(5142),
    [sym_crate] = ACTIONS(5142),
    [aux_sym_metavariable_token1] = ACTIONS(5140),
    [sym__raw_string_literal_start] = ACTIONS(5140),
    [sym_float_literal] = ACTIONS(5140),
    [sym__outer_block_doc_comment_start] = ACTIONS(5140),
    [sym__outer_line_doc_comment_start] = ACTIONS(5140),
  },
  [STATE(1351)] = {
    [sym_line_comment] = STATE(1351),
    [sym_block_comment] = STATE(1351),
    [ts_builtin_sym_end] = ACTIONS(5144),
    [sym_identifier] = ACTIONS(5146),
    [anon_sym_SEMI] = ACTIONS(5144),
    [anon_sym_macro] = ACTIONS(5146),
    [anon_sym_LPAREN] = ACTIONS(5144),
    [anon_sym_LBRACE] = ACTIONS(5144),
    [anon_sym_RBRACE] = ACTIONS(5144),
    [anon_sym_macro_rules] = ACTIONS(5146),
    [anon_sym_BANG] = ACTIONS(5144),
    [anon_sym_LBRACK] = ACTIONS(5144),
    [anon_sym_STAR] = ACTIONS(5144),
    [anon_sym_u8] = ACTIONS(5146),
    [anon_sym_i8] = ACTIONS(5146),
    [anon_sym_u16] = ACTIONS(5146),
    [anon_sym_i16] = ACTIONS(5146),
    [anon_sym_u32] = ACTIONS(5146),
    [anon_sym_i32] = ACTIONS(5146),
    [anon_sym_u64] = ACTIONS(5146),
    [anon_sym_i64] = ACTIONS(5146),
    [anon_sym_u128] = ACTIONS(5146),
    [anon_sym_i128] = ACTIONS(5146),
    [anon_sym_isize] = ACTIONS(5146),
    [anon_sym_usize] = ACTIONS(5146),
    [anon_sym_f32] = ACTIONS(5146),
    [anon_sym_f64] = ACTIONS(5146),
    [anon_sym_bool] = ACTIONS(5146),
    [anon_sym_str] = ACTIONS(5146),
    [anon_sym_char] = ACTIONS(5146),
    [anon_sym_DASH] = ACTIONS(5144),
    [anon_sym_AMP] = ACTIONS(5144),
    [anon_sym_PIPE] = ACTIONS(5144),
    [anon_sym_LT] = ACTIONS(5144),
    [anon_sym_DOT_DOT] = ACTIONS(5144),
    [anon_sym_COLON_COLON] = ACTIONS(5144),
    [anon_sym_POUND] = ACTIONS(5144),
    [anon_sym_SQUOTE] = ACTIONS(5146),
    [anon_sym_async] = ACTIONS(5146),
    [anon_sym_auto] = ACTIONS(5146),
    [anon_sym_await] = ACTIONS(5146),
    [anon_sym_become] = ACTIONS(5146),
    [anon_sym_break] = ACTIONS(5146),
    [anon_sym_const] = ACTIONS(5146),
    [anon_sym_continue] = ACTIONS(5146),
    [anon_sym_default] = ACTIONS(5146),
    [anon_sym_enum] = ACTIONS(5146),
    [anon_sym_fn] = ACTIONS(5146),
    [anon_sym_for] = ACTIONS(5146),
    [anon_sym_gen] = ACTIONS(5146),
    [anon_sym_if] = ACTIONS(5146),
    [anon_sym_impl] = ACTIONS(5146),
    [anon_sym_let] = ACTIONS(5146),
    [anon_sym_loop] = ACTIONS(5146),
    [anon_sym_match] = ACTIONS(5146),
    [anon_sym_mod] = ACTIONS(5146),
    [anon_sym_pub] = ACTIONS(5146),
    [anon_sym_return] = ACTIONS(5146),
    [anon_sym_raw] = ACTIONS(5146),
    [anon_sym_static] = ACTIONS(5146),
    [anon_sym_struct] = ACTIONS(5146),
    [anon_sym_trait] = ACTIONS(5146),
    [anon_sym_type] = ACTIONS(5146),
    [anon_sym_union] = ACTIONS(5146),
    [anon_sym_unsafe] = ACTIONS(5146),
    [anon_sym_use] = ACTIONS(5146),
    [anon_sym_while] = ACTIONS(5146),
    [anon_sym_extern] = ACTIONS(5146),
    [anon_sym_safe] = ACTIONS(5146),
    [anon_sym_yield] = ACTIONS(5146),
    [anon_sym_move] = ACTIONS(5146),
    [anon_sym_try] = ACTIONS(5146),
    [sym_integer_literal] = ACTIONS(5144),
    [aux_sym_string_literal_token1] = ACTIONS(5144),
    [sym_char_literal] = ACTIONS(5144),
    [anon_sym_true] = ACTIONS(5146),
    [anon_sym_false] = ACTIONS(5146),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5146),
    [sym_super] = ACTIONS(5146),
    [sym_crate] = ACTIONS(5146),
    [aux_sym_metavariable_token1] = ACTIONS(5144),
    [sym__raw_string_literal_start] = ACTIONS(5144),
    [sym_float_literal] = ACTIONS(5144),
    [sym__outer_block_doc_comment_start] = ACTIONS(5144),
    [sym__outer_line_doc_comment_start] = ACTIONS(5144),
  },
  [STATE(1352)] = {
    [sym_line_comment] = STATE(1352),
    [sym_block_comment] = STATE(1352),
    [ts_builtin_sym_end] = ACTIONS(5148),
    [sym_identifier] = ACTIONS(5150),
    [anon_sym_SEMI] = ACTIONS(5148),
    [anon_sym_macro] = ACTIONS(5150),
    [anon_sym_LPAREN] = ACTIONS(5148),
    [anon_sym_LBRACE] = ACTIONS(5148),
    [anon_sym_RBRACE] = ACTIONS(5148),
    [anon_sym_macro_rules] = ACTIONS(5150),
    [anon_sym_BANG] = ACTIONS(5148),
    [anon_sym_LBRACK] = ACTIONS(5148),
    [anon_sym_STAR] = ACTIONS(5148),
    [anon_sym_u8] = ACTIONS(5150),
    [anon_sym_i8] = ACTIONS(5150),
    [anon_sym_u16] = ACTIONS(5150),
    [anon_sym_i16] = ACTIONS(5150),
    [anon_sym_u32] = ACTIONS(5150),
    [anon_sym_i32] = ACTIONS(5150),
    [anon_sym_u64] = ACTIONS(5150),
    [anon_sym_i64] = ACTIONS(5150),
    [anon_sym_u128] = ACTIONS(5150),
    [anon_sym_i128] = ACTIONS(5150),
    [anon_sym_isize] = ACTIONS(5150),
    [anon_sym_usize] = ACTIONS(5150),
    [anon_sym_f32] = ACTIONS(5150),
    [anon_sym_f64] = ACTIONS(5150),
    [anon_sym_bool] = ACTIONS(5150),
    [anon_sym_str] = ACTIONS(5150),
    [anon_sym_char] = ACTIONS(5150),
    [anon_sym_DASH] = ACTIONS(5148),
    [anon_sym_AMP] = ACTIONS(5148),
    [anon_sym_PIPE] = ACTIONS(5148),
    [anon_sym_LT] = ACTIONS(5148),
    [anon_sym_DOT_DOT] = ACTIONS(5148),
    [anon_sym_COLON_COLON] = ACTIONS(5148),
    [anon_sym_POUND] = ACTIONS(5148),
    [anon_sym_SQUOTE] = ACTIONS(5150),
    [anon_sym_async] = ACTIONS(5150),
    [anon_sym_auto] = ACTIONS(5150),
    [anon_sym_await] = ACTIONS(5150),
    [anon_sym_become] = ACTIONS(5150),
    [anon_sym_break] = ACTIONS(5150),
    [anon_sym_const] = ACTIONS(5150),
    [anon_sym_continue] = ACTIONS(5150),
    [anon_sym_default] = ACTIONS(5150),
    [anon_sym_enum] = ACTIONS(5150),
    [anon_sym_fn] = ACTIONS(5150),
    [anon_sym_for] = ACTIONS(5150),
    [anon_sym_gen] = ACTIONS(5150),
    [anon_sym_if] = ACTIONS(5150),
    [anon_sym_impl] = ACTIONS(5150),
    [anon_sym_let] = ACTIONS(5150),
    [anon_sym_loop] = ACTIONS(5150),
    [anon_sym_match] = ACTIONS(5150),
    [anon_sym_mod] = ACTIONS(5150),
    [anon_sym_pub] = ACTIONS(5150),
    [anon_sym_return] = ACTIONS(5150),
    [anon_sym_raw] = ACTIONS(5150),
    [anon_sym_static] = ACTIONS(5150),
    [anon_sym_struct] = ACTIONS(5150),
    [anon_sym_trait] = ACTIONS(5150),
    [anon_sym_type] = ACTIONS(5150),
    [anon_sym_union] = ACTIONS(5150),
    [anon_sym_unsafe] = ACTIONS(5150),
    [anon_sym_use] = ACTIONS(5150),
    [anon_sym_while] = ACTIONS(5150),
    [anon_sym_extern] = ACTIONS(5150),
    [anon_sym_safe] = ACTIONS(5150),
    [anon_sym_yield] = ACTIONS(5150),
    [anon_sym_move] = ACTIONS(5150),
    [anon_sym_try] = ACTIONS(5150),
    [sym_integer_literal] = ACTIONS(5148),
    [aux_sym_string_literal_token1] = ACTIONS(5148),
    [sym_char_literal] = ACTIONS(5148),
    [anon_sym_true] = ACTIONS(5150),
    [anon_sym_false] = ACTIONS(5150),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5150),
    [sym_super] = ACTIONS(5150),
    [sym_crate] = ACTIONS(5150),
    [aux_sym_metavariable_token1] = ACTIONS(5148),
    [sym__raw_string_literal_start] = ACTIONS(5148),
    [sym_float_literal] = ACTIONS(5148),
    [sym__outer_block_doc_comment_start] = ACTIONS(5148),
    [sym__outer_line_doc_comment_start] = ACTIONS(5148),
  },
  [STATE(1353)] = {
    [sym_line_comment] = STATE(1353),
    [sym_block_comment] = STATE(1353),
    [ts_builtin_sym_end] = ACTIONS(5152),
    [sym_identifier] = ACTIONS(5154),
    [anon_sym_SEMI] = ACTIONS(5152),
    [anon_sym_macro] = ACTIONS(5154),
    [anon_sym_LPAREN] = ACTIONS(5152),
    [anon_sym_LBRACE] = ACTIONS(5152),
    [anon_sym_RBRACE] = ACTIONS(5152),
    [anon_sym_macro_rules] = ACTIONS(5154),
    [anon_sym_BANG] = ACTIONS(5152),
    [anon_sym_LBRACK] = ACTIONS(5152),
    [anon_sym_STAR] = ACTIONS(5152),
    [anon_sym_u8] = ACTIONS(5154),
    [anon_sym_i8] = ACTIONS(5154),
    [anon_sym_u16] = ACTIONS(5154),
    [anon_sym_i16] = ACTIONS(5154),
    [anon_sym_u32] = ACTIONS(5154),
    [anon_sym_i32] = ACTIONS(5154),
    [anon_sym_u64] = ACTIONS(5154),
    [anon_sym_i64] = ACTIONS(5154),
    [anon_sym_u128] = ACTIONS(5154),
    [anon_sym_i128] = ACTIONS(5154),
    [anon_sym_isize] = ACTIONS(5154),
    [anon_sym_usize] = ACTIONS(5154),
    [anon_sym_f32] = ACTIONS(5154),
    [anon_sym_f64] = ACTIONS(5154),
    [anon_sym_bool] = ACTIONS(5154),
    [anon_sym_str] = ACTIONS(5154),
    [anon_sym_char] = ACTIONS(5154),
    [anon_sym_DASH] = ACTIONS(5152),
    [anon_sym_AMP] = ACTIONS(5152),
    [anon_sym_PIPE] = ACTIONS(5152),
    [anon_sym_LT] = ACTIONS(5152),
    [anon_sym_DOT_DOT] = ACTIONS(5152),
    [anon_sym_COLON_COLON] = ACTIONS(5152),
    [anon_sym_POUND] = ACTIONS(5152),
    [anon_sym_SQUOTE] = ACTIONS(5154),
    [anon_sym_async] = ACTIONS(5154),
    [anon_sym_auto] = ACTIONS(5154),
    [anon_sym_await] = ACTIONS(5154),
    [anon_sym_become] = ACTIONS(5154),
    [anon_sym_break] = ACTIONS(5154),
    [anon_sym_const] = ACTIONS(5154),
    [anon_sym_continue] = ACTIONS(5154),
    [anon_sym_default] = ACTIONS(5154),
    [anon_sym_enum] = ACTIONS(5154),
    [anon_sym_fn] = ACTIONS(5154),
    [anon_sym_for] = ACTIONS(5154),
    [anon_sym_gen] = ACTIONS(5154),
    [anon_sym_if] = ACTIONS(5154),
    [anon_sym_impl] = ACTIONS(5154),
    [anon_sym_let] = ACTIONS(5154),
    [anon_sym_loop] = ACTIONS(5154),
    [anon_sym_match] = ACTIONS(5154),
    [anon_sym_mod] = ACTIONS(5154),
    [anon_sym_pub] = ACTIONS(5154),
    [anon_sym_return] = ACTIONS(5154),
    [anon_sym_raw] = ACTIONS(5154),
    [anon_sym_static] = ACTIONS(5154),
    [anon_sym_struct] = ACTIONS(5154),
    [anon_sym_trait] = ACTIONS(5154),
    [anon_sym_type] = ACTIONS(5154),
    [anon_sym_union] = ACTIONS(5154),
    [anon_sym_unsafe] = ACTIONS(5154),
    [anon_sym_use] = ACTIONS(5154),
    [anon_sym_while] = ACTIONS(5154),
    [anon_sym_extern] = ACTIONS(5154),
    [anon_sym_safe] = ACTIONS(5154),
    [anon_sym_yield] = ACTIONS(5154),
    [anon_sym_move] = ACTIONS(5154),
    [anon_sym_try] = ACTIONS(5154),
    [sym_integer_literal] = ACTIONS(5152),
    [aux_sym_string_literal_token1] = ACTIONS(5152),
    [sym_char_literal] = ACTIONS(5152),
    [anon_sym_true] = ACTIONS(5154),
    [anon_sym_false] = ACTIONS(5154),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5154),
    [sym_super] = ACTIONS(5154),
    [sym_crate] = ACTIONS(5154),
    [aux_sym_metavariable_token1] = ACTIONS(5152),
    [sym__raw_string_literal_start] = ACTIONS(5152),
    [sym_float_literal] = ACTIONS(5152),
    [sym__outer_block_doc_comment_start] = ACTIONS(5152),
    [sym__outer_line_doc_comment_start] = ACTIONS(5152),
  },
  [STATE(1354)] = {
    [sym_line_comment] = STATE(1354),
    [sym_block_comment] = STATE(1354),
    [ts_builtin_sym_end] = ACTIONS(5156),
    [sym_identifier] = ACTIONS(5158),
    [anon_sym_SEMI] = ACTIONS(5156),
    [anon_sym_macro] = ACTIONS(5158),
    [anon_sym_LPAREN] = ACTIONS(5156),
    [anon_sym_LBRACE] = ACTIONS(5156),
    [anon_sym_RBRACE] = ACTIONS(5156),
    [anon_sym_macro_rules] = ACTIONS(5158),
    [anon_sym_BANG] = ACTIONS(5156),
    [anon_sym_LBRACK] = ACTIONS(5156),
    [anon_sym_STAR] = ACTIONS(5156),
    [anon_sym_u8] = ACTIONS(5158),
    [anon_sym_i8] = ACTIONS(5158),
    [anon_sym_u16] = ACTIONS(5158),
    [anon_sym_i16] = ACTIONS(5158),
    [anon_sym_u32] = ACTIONS(5158),
    [anon_sym_i32] = ACTIONS(5158),
    [anon_sym_u64] = ACTIONS(5158),
    [anon_sym_i64] = ACTIONS(5158),
    [anon_sym_u128] = ACTIONS(5158),
    [anon_sym_i128] = ACTIONS(5158),
    [anon_sym_isize] = ACTIONS(5158),
    [anon_sym_usize] = ACTIONS(5158),
    [anon_sym_f32] = ACTIONS(5158),
    [anon_sym_f64] = ACTIONS(5158),
    [anon_sym_bool] = ACTIONS(5158),
    [anon_sym_str] = ACTIONS(5158),
    [anon_sym_char] = ACTIONS(5158),
    [anon_sym_DASH] = ACTIONS(5156),
    [anon_sym_AMP] = ACTIONS(5156),
    [anon_sym_PIPE] = ACTIONS(5156),
    [anon_sym_LT] = ACTIONS(5156),
    [anon_sym_DOT_DOT] = ACTIONS(5156),
    [anon_sym_COLON_COLON] = ACTIONS(5156),
    [anon_sym_POUND] = ACTIONS(5156),
    [anon_sym_SQUOTE] = ACTIONS(5158),
    [anon_sym_async] = ACTIONS(5158),
    [anon_sym_auto] = ACTIONS(5158),
    [anon_sym_await] = ACTIONS(5158),
    [anon_sym_become] = ACTIONS(5158),
    [anon_sym_break] = ACTIONS(5158),
    [anon_sym_const] = ACTIONS(5158),
    [anon_sym_continue] = ACTIONS(5158),
    [anon_sym_default] = ACTIONS(5158),
    [anon_sym_enum] = ACTIONS(5158),
    [anon_sym_fn] = ACTIONS(5158),
    [anon_sym_for] = ACTIONS(5158),
    [anon_sym_gen] = ACTIONS(5158),
    [anon_sym_if] = ACTIONS(5158),
    [anon_sym_impl] = ACTIONS(5158),
    [anon_sym_let] = ACTIONS(5158),
    [anon_sym_loop] = ACTIONS(5158),
    [anon_sym_match] = ACTIONS(5158),
    [anon_sym_mod] = ACTIONS(5158),
    [anon_sym_pub] = ACTIONS(5158),
    [anon_sym_return] = ACTIONS(5158),
    [anon_sym_raw] = ACTIONS(5158),
    [anon_sym_static] = ACTIONS(5158),
    [anon_sym_struct] = ACTIONS(5158),
    [anon_sym_trait] = ACTIONS(5158),
    [anon_sym_type] = ACTIONS(5158),
    [anon_sym_union] = ACTIONS(5158),
    [anon_sym_unsafe] = ACTIONS(5158),
    [anon_sym_use] = ACTIONS(5158),
    [anon_sym_while] = ACTIONS(5158),
    [anon_sym_extern] = ACTIONS(5158),
    [anon_sym_safe] = ACTIONS(5158),
    [anon_sym_yield] = ACTIONS(5158),
    [anon_sym_move] = ACTIONS(5158),
    [anon_sym_try] = ACTIONS(5158),
    [sym_integer_literal] = ACTIONS(5156),
    [aux_sym_string_literal_token1] = ACTIONS(5156),
    [sym_char_literal] = ACTIONS(5156),
    [anon_sym_true] = ACTIONS(5158),
    [anon_sym_false] = ACTIONS(5158),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5158),
    [sym_super] = ACTIONS(5158),
    [sym_crate] = ACTIONS(5158),
    [aux_sym_metavariable_token1] = ACTIONS(5156),
    [sym__raw_string_literal_start] = ACTIONS(5156),
    [sym_float_literal] = ACTIONS(5156),
    [sym__outer_block_doc_comment_start] = ACTIONS(5156),
    [sym__outer_line_doc_comment_start] = ACTIONS(5156),
  },
  [STATE(1355)] = {
    [sym_line_comment] = STATE(1355),
    [sym_block_comment] = STATE(1355),
    [ts_builtin_sym_end] = ACTIONS(5160),
    [sym_identifier] = ACTIONS(5162),
    [anon_sym_SEMI] = ACTIONS(5160),
    [anon_sym_macro] = ACTIONS(5162),
    [anon_sym_LPAREN] = ACTIONS(5160),
    [anon_sym_LBRACE] = ACTIONS(5160),
    [anon_sym_RBRACE] = ACTIONS(5160),
    [anon_sym_macro_rules] = ACTIONS(5162),
    [anon_sym_BANG] = ACTIONS(5160),
    [anon_sym_LBRACK] = ACTIONS(5160),
    [anon_sym_STAR] = ACTIONS(5160),
    [anon_sym_u8] = ACTIONS(5162),
    [anon_sym_i8] = ACTIONS(5162),
    [anon_sym_u16] = ACTIONS(5162),
    [anon_sym_i16] = ACTIONS(5162),
    [anon_sym_u32] = ACTIONS(5162),
    [anon_sym_i32] = ACTIONS(5162),
    [anon_sym_u64] = ACTIONS(5162),
    [anon_sym_i64] = ACTIONS(5162),
    [anon_sym_u128] = ACTIONS(5162),
    [anon_sym_i128] = ACTIONS(5162),
    [anon_sym_isize] = ACTIONS(5162),
    [anon_sym_usize] = ACTIONS(5162),
    [anon_sym_f32] = ACTIONS(5162),
    [anon_sym_f64] = ACTIONS(5162),
    [anon_sym_bool] = ACTIONS(5162),
    [anon_sym_str] = ACTIONS(5162),
    [anon_sym_char] = ACTIONS(5162),
    [anon_sym_DASH] = ACTIONS(5160),
    [anon_sym_AMP] = ACTIONS(5160),
    [anon_sym_PIPE] = ACTIONS(5160),
    [anon_sym_LT] = ACTIONS(5160),
    [anon_sym_DOT_DOT] = ACTIONS(5160),
    [anon_sym_COLON_COLON] = ACTIONS(5160),
    [anon_sym_POUND] = ACTIONS(5160),
    [anon_sym_SQUOTE] = ACTIONS(5162),
    [anon_sym_async] = ACTIONS(5162),
    [anon_sym_auto] = ACTIONS(5162),
    [anon_sym_await] = ACTIONS(5162),
    [anon_sym_become] = ACTIONS(5162),
    [anon_sym_break] = ACTIONS(5162),
    [anon_sym_const] = ACTIONS(5162),
    [anon_sym_continue] = ACTIONS(5162),
    [anon_sym_default] = ACTIONS(5162),
    [anon_sym_enum] = ACTIONS(5162),
    [anon_sym_fn] = ACTIONS(5162),
    [anon_sym_for] = ACTIONS(5162),
    [anon_sym_gen] = ACTIONS(5162),
    [anon_sym_if] = ACTIONS(5162),
    [anon_sym_impl] = ACTIONS(5162),
    [anon_sym_let] = ACTIONS(5162),
    [anon_sym_loop] = ACTIONS(5162),
    [anon_sym_match] = ACTIONS(5162),
    [anon_sym_mod] = ACTIONS(5162),
    [anon_sym_pub] = ACTIONS(5162),
    [anon_sym_return] = ACTIONS(5162),
    [anon_sym_raw] = ACTIONS(5162),
    [anon_sym_static] = ACTIONS(5162),
    [anon_sym_struct] = ACTIONS(5162),
    [anon_sym_trait] = ACTIONS(5162),
    [anon_sym_type] = ACTIONS(5162),
    [anon_sym_union] = ACTIONS(5162),
    [anon_sym_unsafe] = ACTIONS(5162),
    [anon_sym_use] = ACTIONS(5162),
    [anon_sym_while] = ACTIONS(5162),
    [anon_sym_extern] = ACTIONS(5162),
    [anon_sym_safe] = ACTIONS(5162),
    [anon_sym_yield] = ACTIONS(5162),
    [anon_sym_move] = ACTIONS(5162),
    [anon_sym_try] = ACTIONS(5162),
    [sym_integer_literal] = ACTIONS(5160),
    [aux_sym_string_literal_token1] = ACTIONS(5160),
    [sym_char_literal] = ACTIONS(5160),
    [anon_sym_true] = ACTIONS(5162),
    [anon_sym_false] = ACTIONS(5162),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5162),
    [sym_super] = ACTIONS(5162),
    [sym_crate] = ACTIONS(5162),
    [aux_sym_metavariable_token1] = ACTIONS(5160),
    [sym__raw_string_literal_start] = ACTIONS(5160),
    [sym_float_literal] = ACTIONS(5160),
    [sym__outer_block_doc_comment_start] = ACTIONS(5160),
    [sym__outer_line_doc_comment_start] = ACTIONS(5160),
  },
  [STATE(1356)] = {
    [sym_line_comment] = STATE(1356),
    [sym_block_comment] = STATE(1356),
    [ts_builtin_sym_end] = ACTIONS(5164),
    [sym_identifier] = ACTIONS(5166),
    [anon_sym_SEMI] = ACTIONS(5164),
    [anon_sym_macro] = ACTIONS(5166),
    [anon_sym_LPAREN] = ACTIONS(5164),
    [anon_sym_LBRACE] = ACTIONS(5164),
    [anon_sym_RBRACE] = ACTIONS(5164),
    [anon_sym_macro_rules] = ACTIONS(5166),
    [anon_sym_BANG] = ACTIONS(5164),
    [anon_sym_LBRACK] = ACTIONS(5164),
    [anon_sym_STAR] = ACTIONS(5164),
    [anon_sym_u8] = ACTIONS(5166),
    [anon_sym_i8] = ACTIONS(5166),
    [anon_sym_u16] = ACTIONS(5166),
    [anon_sym_i16] = ACTIONS(5166),
    [anon_sym_u32] = ACTIONS(5166),
    [anon_sym_i32] = ACTIONS(5166),
    [anon_sym_u64] = ACTIONS(5166),
    [anon_sym_i64] = ACTIONS(5166),
    [anon_sym_u128] = ACTIONS(5166),
    [anon_sym_i128] = ACTIONS(5166),
    [anon_sym_isize] = ACTIONS(5166),
    [anon_sym_usize] = ACTIONS(5166),
    [anon_sym_f32] = ACTIONS(5166),
    [anon_sym_f64] = ACTIONS(5166),
    [anon_sym_bool] = ACTIONS(5166),
    [anon_sym_str] = ACTIONS(5166),
    [anon_sym_char] = ACTIONS(5166),
    [anon_sym_DASH] = ACTIONS(5164),
    [anon_sym_AMP] = ACTIONS(5164),
    [anon_sym_PIPE] = ACTIONS(5164),
    [anon_sym_LT] = ACTIONS(5164),
    [anon_sym_DOT_DOT] = ACTIONS(5164),
    [anon_sym_COLON_COLON] = ACTIONS(5164),
    [anon_sym_POUND] = ACTIONS(5164),
    [anon_sym_SQUOTE] = ACTIONS(5166),
    [anon_sym_async] = ACTIONS(5166),
    [anon_sym_auto] = ACTIONS(5166),
    [anon_sym_await] = ACTIONS(5166),
    [anon_sym_become] = ACTIONS(5166),
    [anon_sym_break] = ACTIONS(5166),
    [anon_sym_const] = ACTIONS(5166),
    [anon_sym_continue] = ACTIONS(5166),
    [anon_sym_default] = ACTIONS(5166),
    [anon_sym_enum] = ACTIONS(5166),
    [anon_sym_fn] = ACTIONS(5166),
    [anon_sym_for] = ACTIONS(5166),
    [anon_sym_gen] = ACTIONS(5166),
    [anon_sym_if] = ACTIONS(5166),
    [anon_sym_impl] = ACTIONS(5166),
    [anon_sym_let] = ACTIONS(5166),
    [anon_sym_loop] = ACTIONS(5166),
    [anon_sym_match] = ACTIONS(5166),
    [anon_sym_mod] = ACTIONS(5166),
    [anon_sym_pub] = ACTIONS(5166),
    [anon_sym_return] = ACTIONS(5166),
    [anon_sym_raw] = ACTIONS(5166),
    [anon_sym_static] = ACTIONS(5166),
    [anon_sym_struct] = ACTIONS(5166),
    [anon_sym_trait] = ACTIONS(5166),
    [anon_sym_type] = ACTIONS(5166),
    [anon_sym_union] = ACTIONS(5166),
    [anon_sym_unsafe] = ACTIONS(5166),
    [anon_sym_use] = ACTIONS(5166),
    [anon_sym_while] = ACTIONS(5166),
    [anon_sym_extern] = ACTIONS(5166),
    [anon_sym_safe] = ACTIONS(5166),
    [anon_sym_yield] = ACTIONS(5166),
    [anon_sym_move] = ACTIONS(5166),
    [anon_sym_try] = ACTIONS(5166),
    [sym_integer_literal] = ACTIONS(5164),
    [aux_sym_string_literal_token1] = ACTIONS(5164),
    [sym_char_literal] = ACTIONS(5164),
    [anon_sym_true] = ACTIONS(5166),
    [anon_sym_false] = ACTIONS(5166),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5166),
    [sym_super] = ACTIONS(5166),
    [sym_crate] = ACTIONS(5166),
    [aux_sym_metavariable_token1] = ACTIONS(5164),
    [sym__raw_string_literal_start] = ACTIONS(5164),
    [sym_float_literal] = ACTIONS(5164),
    [sym__outer_block_doc_comment_start] = ACTIONS(5164),
    [sym__outer_line_doc_comment_start] = ACTIONS(5164),
  },
  [STATE(1357)] = {
    [sym_line_comment] = STATE(1357),
    [sym_block_comment] = STATE(1357),
    [ts_builtin_sym_end] = ACTIONS(5168),
    [sym_identifier] = ACTIONS(5170),
    [anon_sym_SEMI] = ACTIONS(5168),
    [anon_sym_macro] = ACTIONS(5170),
    [anon_sym_LPAREN] = ACTIONS(5168),
    [anon_sym_LBRACE] = ACTIONS(5168),
    [anon_sym_RBRACE] = ACTIONS(5168),
    [anon_sym_macro_rules] = ACTIONS(5170),
    [anon_sym_BANG] = ACTIONS(5168),
    [anon_sym_LBRACK] = ACTIONS(5168),
    [anon_sym_STAR] = ACTIONS(5168),
    [anon_sym_u8] = ACTIONS(5170),
    [anon_sym_i8] = ACTIONS(5170),
    [anon_sym_u16] = ACTIONS(5170),
    [anon_sym_i16] = ACTIONS(5170),
    [anon_sym_u32] = ACTIONS(5170),
    [anon_sym_i32] = ACTIONS(5170),
    [anon_sym_u64] = ACTIONS(5170),
    [anon_sym_i64] = ACTIONS(5170),
    [anon_sym_u128] = ACTIONS(5170),
    [anon_sym_i128] = ACTIONS(5170),
    [anon_sym_isize] = ACTIONS(5170),
    [anon_sym_usize] = ACTIONS(5170),
    [anon_sym_f32] = ACTIONS(5170),
    [anon_sym_f64] = ACTIONS(5170),
    [anon_sym_bool] = ACTIONS(5170),
    [anon_sym_str] = ACTIONS(5170),
    [anon_sym_char] = ACTIONS(5170),
    [anon_sym_DASH] = ACTIONS(5168),
    [anon_sym_AMP] = ACTIONS(5168),
    [anon_sym_PIPE] = ACTIONS(5168),
    [anon_sym_LT] = ACTIONS(5168),
    [anon_sym_DOT_DOT] = ACTIONS(5168),
    [anon_sym_COLON_COLON] = ACTIONS(5168),
    [anon_sym_POUND] = ACTIONS(5168),
    [anon_sym_SQUOTE] = ACTIONS(5170),
    [anon_sym_async] = ACTIONS(5170),
    [anon_sym_auto] = ACTIONS(5170),
    [anon_sym_await] = ACTIONS(5170),
    [anon_sym_become] = ACTIONS(5170),
    [anon_sym_break] = ACTIONS(5170),
    [anon_sym_const] = ACTIONS(5170),
    [anon_sym_continue] = ACTIONS(5170),
    [anon_sym_default] = ACTIONS(5170),
    [anon_sym_enum] = ACTIONS(5170),
    [anon_sym_fn] = ACTIONS(5170),
    [anon_sym_for] = ACTIONS(5170),
    [anon_sym_gen] = ACTIONS(5170),
    [anon_sym_if] = ACTIONS(5170),
    [anon_sym_impl] = ACTIONS(5170),
    [anon_sym_let] = ACTIONS(5170),
    [anon_sym_loop] = ACTIONS(5170),
    [anon_sym_match] = ACTIONS(5170),
    [anon_sym_mod] = ACTIONS(5170),
    [anon_sym_pub] = ACTIONS(5170),
    [anon_sym_return] = ACTIONS(5170),
    [anon_sym_raw] = ACTIONS(5170),
    [anon_sym_static] = ACTIONS(5170),
    [anon_sym_struct] = ACTIONS(5170),
    [anon_sym_trait] = ACTIONS(5170),
    [anon_sym_type] = ACTIONS(5170),
    [anon_sym_union] = ACTIONS(5170),
    [anon_sym_unsafe] = ACTIONS(5170),
    [anon_sym_use] = ACTIONS(5170),
    [anon_sym_while] = ACTIONS(5170),
    [anon_sym_extern] = ACTIONS(5170),
    [anon_sym_safe] = ACTIONS(5170),
    [anon_sym_yield] = ACTIONS(5170),
    [anon_sym_move] = ACTIONS(5170),
    [anon_sym_try] = ACTIONS(5170),
    [sym_integer_literal] = ACTIONS(5168),
    [aux_sym_string_literal_token1] = ACTIONS(5168),
    [sym_char_literal] = ACTIONS(5168),
    [anon_sym_true] = ACTIONS(5170),
    [anon_sym_false] = ACTIONS(5170),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5170),
    [sym_super] = ACTIONS(5170),
    [sym_crate] = ACTIONS(5170),
    [aux_sym_metavariable_token1] = ACTIONS(5168),
    [sym__raw_string_literal_start] = ACTIONS(5168),
    [sym_float_literal] = ACTIONS(5168),
    [sym__outer_block_doc_comment_start] = ACTIONS(5168),
    [sym__outer_line_doc_comment_start] = ACTIONS(5168),
  },
  [STATE(1358)] = {
    [sym_line_comment] = STATE(1358),
    [sym_block_comment] = STATE(1358),
    [ts_builtin_sym_end] = ACTIONS(5172),
    [sym_identifier] = ACTIONS(5174),
    [anon_sym_SEMI] = ACTIONS(5172),
    [anon_sym_macro] = ACTIONS(5174),
    [anon_sym_LPAREN] = ACTIONS(5172),
    [anon_sym_LBRACE] = ACTIONS(5172),
    [anon_sym_RBRACE] = ACTIONS(5172),
    [anon_sym_macro_rules] = ACTIONS(5174),
    [anon_sym_BANG] = ACTIONS(5172),
    [anon_sym_LBRACK] = ACTIONS(5172),
    [anon_sym_STAR] = ACTIONS(5172),
    [anon_sym_u8] = ACTIONS(5174),
    [anon_sym_i8] = ACTIONS(5174),
    [anon_sym_u16] = ACTIONS(5174),
    [anon_sym_i16] = ACTIONS(5174),
    [anon_sym_u32] = ACTIONS(5174),
    [anon_sym_i32] = ACTIONS(5174),
    [anon_sym_u64] = ACTIONS(5174),
    [anon_sym_i64] = ACTIONS(5174),
    [anon_sym_u128] = ACTIONS(5174),
    [anon_sym_i128] = ACTIONS(5174),
    [anon_sym_isize] = ACTIONS(5174),
    [anon_sym_usize] = ACTIONS(5174),
    [anon_sym_f32] = ACTIONS(5174),
    [anon_sym_f64] = ACTIONS(5174),
    [anon_sym_bool] = ACTIONS(5174),
    [anon_sym_str] = ACTIONS(5174),
    [anon_sym_char] = ACTIONS(5174),
    [anon_sym_DASH] = ACTIONS(5172),
    [anon_sym_AMP] = ACTIONS(5172),
    [anon_sym_PIPE] = ACTIONS(5172),
    [anon_sym_LT] = ACTIONS(5172),
    [anon_sym_DOT_DOT] = ACTIONS(5172),
    [anon_sym_COLON_COLON] = ACTIONS(5172),
    [anon_sym_POUND] = ACTIONS(5172),
    [anon_sym_SQUOTE] = ACTIONS(5174),
    [anon_sym_async] = ACTIONS(5174),
    [anon_sym_auto] = ACTIONS(5174),
    [anon_sym_await] = ACTIONS(5174),
    [anon_sym_become] = ACTIONS(5174),
    [anon_sym_break] = ACTIONS(5174),
    [anon_sym_const] = ACTIONS(5174),
    [anon_sym_continue] = ACTIONS(5174),
    [anon_sym_default] = ACTIONS(5174),
    [anon_sym_enum] = ACTIONS(5174),
    [anon_sym_fn] = ACTIONS(5174),
    [anon_sym_for] = ACTIONS(5174),
    [anon_sym_gen] = ACTIONS(5174),
    [anon_sym_if] = ACTIONS(5174),
    [anon_sym_impl] = ACTIONS(5174),
    [anon_sym_let] = ACTIONS(5174),
    [anon_sym_loop] = ACTIONS(5174),
    [anon_sym_match] = ACTIONS(5174),
    [anon_sym_mod] = ACTIONS(5174),
    [anon_sym_pub] = ACTIONS(5174),
    [anon_sym_return] = ACTIONS(5174),
    [anon_sym_raw] = ACTIONS(5174),
    [anon_sym_static] = ACTIONS(5174),
    [anon_sym_struct] = ACTIONS(5174),
    [anon_sym_trait] = ACTIONS(5174),
    [anon_sym_type] = ACTIONS(5174),
    [anon_sym_union] = ACTIONS(5174),
    [anon_sym_unsafe] = ACTIONS(5174),
    [anon_sym_use] = ACTIONS(5174),
    [anon_sym_while] = ACTIONS(5174),
    [anon_sym_extern] = ACTIONS(5174),
    [anon_sym_safe] = ACTIONS(5174),
    [anon_sym_yield] = ACTIONS(5174),
    [anon_sym_move] = ACTIONS(5174),
    [anon_sym_try] = ACTIONS(5174),
    [sym_integer_literal] = ACTIONS(5172),
    [aux_sym_string_literal_token1] = ACTIONS(5172),
    [sym_char_literal] = ACTIONS(5172),
    [anon_sym_true] = ACTIONS(5174),
    [anon_sym_false] = ACTIONS(5174),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5174),
    [sym_super] = ACTIONS(5174),
    [sym_crate] = ACTIONS(5174),
    [aux_sym_metavariable_token1] = ACTIONS(5172),
    [sym__raw_string_literal_start] = ACTIONS(5172),
    [sym_float_literal] = ACTIONS(5172),
    [sym__outer_block_doc_comment_start] = ACTIONS(5172),
    [sym__outer_line_doc_comment_start] = ACTIONS(5172),
  },
  [STATE(1359)] = {
    [sym_line_comment] = STATE(1359),
    [sym_block_comment] = STATE(1359),
    [ts_builtin_sym_end] = ACTIONS(5176),
    [sym_identifier] = ACTIONS(5178),
    [anon_sym_SEMI] = ACTIONS(5176),
    [anon_sym_macro] = ACTIONS(5178),
    [anon_sym_LPAREN] = ACTIONS(5176),
    [anon_sym_LBRACE] = ACTIONS(5176),
    [anon_sym_RBRACE] = ACTIONS(5176),
    [anon_sym_macro_rules] = ACTIONS(5178),
    [anon_sym_BANG] = ACTIONS(5176),
    [anon_sym_LBRACK] = ACTIONS(5176),
    [anon_sym_STAR] = ACTIONS(5176),
    [anon_sym_u8] = ACTIONS(5178),
    [anon_sym_i8] = ACTIONS(5178),
    [anon_sym_u16] = ACTIONS(5178),
    [anon_sym_i16] = ACTIONS(5178),
    [anon_sym_u32] = ACTIONS(5178),
    [anon_sym_i32] = ACTIONS(5178),
    [anon_sym_u64] = ACTIONS(5178),
    [anon_sym_i64] = ACTIONS(5178),
    [anon_sym_u128] = ACTIONS(5178),
    [anon_sym_i128] = ACTIONS(5178),
    [anon_sym_isize] = ACTIONS(5178),
    [anon_sym_usize] = ACTIONS(5178),
    [anon_sym_f32] = ACTIONS(5178),
    [anon_sym_f64] = ACTIONS(5178),
    [anon_sym_bool] = ACTIONS(5178),
    [anon_sym_str] = ACTIONS(5178),
    [anon_sym_char] = ACTIONS(5178),
    [anon_sym_DASH] = ACTIONS(5176),
    [anon_sym_AMP] = ACTIONS(5176),
    [anon_sym_PIPE] = ACTIONS(5176),
    [anon_sym_LT] = ACTIONS(5176),
    [anon_sym_DOT_DOT] = ACTIONS(5176),
    [anon_sym_COLON_COLON] = ACTIONS(5176),
    [anon_sym_POUND] = ACTIONS(5176),
    [anon_sym_SQUOTE] = ACTIONS(5178),
    [anon_sym_async] = ACTIONS(5178),
    [anon_sym_auto] = ACTIONS(5178),
    [anon_sym_await] = ACTIONS(5178),
    [anon_sym_become] = ACTIONS(5178),
    [anon_sym_break] = ACTIONS(5178),
    [anon_sym_const] = ACTIONS(5178),
    [anon_sym_continue] = ACTIONS(5178),
    [anon_sym_default] = ACTIONS(5178),
    [anon_sym_enum] = ACTIONS(5178),
    [anon_sym_fn] = ACTIONS(5178),
    [anon_sym_for] = ACTIONS(5178),
    [anon_sym_gen] = ACTIONS(5178),
    [anon_sym_if] = ACTIONS(5178),
    [anon_sym_impl] = ACTIONS(5178),
    [anon_sym_let] = ACTIONS(5178),
    [anon_sym_loop] = ACTIONS(5178),
    [anon_sym_match] = ACTIONS(5178),
    [anon_sym_mod] = ACTIONS(5178),
    [anon_sym_pub] = ACTIONS(5178),
    [anon_sym_return] = ACTIONS(5178),
    [anon_sym_raw] = ACTIONS(5178),
    [anon_sym_static] = ACTIONS(5178),
    [anon_sym_struct] = ACTIONS(5178),
    [anon_sym_trait] = ACTIONS(5178),
    [anon_sym_type] = ACTIONS(5178),
    [anon_sym_union] = ACTIONS(5178),
    [anon_sym_unsafe] = ACTIONS(5178),
    [anon_sym_use] = ACTIONS(5178),
    [anon_sym_while] = ACTIONS(5178),
    [anon_sym_extern] = ACTIONS(5178),
    [anon_sym_safe] = ACTIONS(5178),
    [anon_sym_yield] = ACTIONS(5178),
    [anon_sym_move] = ACTIONS(5178),
    [anon_sym_try] = ACTIONS(5178),
    [sym_integer_literal] = ACTIONS(5176),
    [aux_sym_string_literal_token1] = ACTIONS(5176),
    [sym_char_literal] = ACTIONS(5176),
    [anon_sym_true] = ACTIONS(5178),
    [anon_sym_false] = ACTIONS(5178),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5178),
    [sym_super] = ACTIONS(5178),
    [sym_crate] = ACTIONS(5178),
    [aux_sym_metavariable_token1] = ACTIONS(5176),
    [sym__raw_string_literal_start] = ACTIONS(5176),
    [sym_float_literal] = ACTIONS(5176),
    [sym__outer_block_doc_comment_start] = ACTIONS(5176),
    [sym__outer_line_doc_comment_start] = ACTIONS(5176),
  },
  [STATE(1360)] = {
    [sym_line_comment] = STATE(1360),
    [sym_block_comment] = STATE(1360),
    [ts_builtin_sym_end] = ACTIONS(5180),
    [sym_identifier] = ACTIONS(5182),
    [anon_sym_SEMI] = ACTIONS(5180),
    [anon_sym_macro] = ACTIONS(5182),
    [anon_sym_LPAREN] = ACTIONS(5180),
    [anon_sym_LBRACE] = ACTIONS(5180),
    [anon_sym_RBRACE] = ACTIONS(5180),
    [anon_sym_macro_rules] = ACTIONS(5182),
    [anon_sym_BANG] = ACTIONS(5180),
    [anon_sym_LBRACK] = ACTIONS(5180),
    [anon_sym_STAR] = ACTIONS(5180),
    [anon_sym_u8] = ACTIONS(5182),
    [anon_sym_i8] = ACTIONS(5182),
    [anon_sym_u16] = ACTIONS(5182),
    [anon_sym_i16] = ACTIONS(5182),
    [anon_sym_u32] = ACTIONS(5182),
    [anon_sym_i32] = ACTIONS(5182),
    [anon_sym_u64] = ACTIONS(5182),
    [anon_sym_i64] = ACTIONS(5182),
    [anon_sym_u128] = ACTIONS(5182),
    [anon_sym_i128] = ACTIONS(5182),
    [anon_sym_isize] = ACTIONS(5182),
    [anon_sym_usize] = ACTIONS(5182),
    [anon_sym_f32] = ACTIONS(5182),
    [anon_sym_f64] = ACTIONS(5182),
    [anon_sym_bool] = ACTIONS(5182),
    [anon_sym_str] = ACTIONS(5182),
    [anon_sym_char] = ACTIONS(5182),
    [anon_sym_DASH] = ACTIONS(5180),
    [anon_sym_AMP] = ACTIONS(5180),
    [anon_sym_PIPE] = ACTIONS(5180),
    [anon_sym_LT] = ACTIONS(5180),
    [anon_sym_DOT_DOT] = ACTIONS(5180),
    [anon_sym_COLON_COLON] = ACTIONS(5180),
    [anon_sym_POUND] = ACTIONS(5180),
    [anon_sym_SQUOTE] = ACTIONS(5182),
    [anon_sym_async] = ACTIONS(5182),
    [anon_sym_auto] = ACTIONS(5182),
    [anon_sym_await] = ACTIONS(5182),
    [anon_sym_become] = ACTIONS(5182),
    [anon_sym_break] = ACTIONS(5182),
    [anon_sym_const] = ACTIONS(5182),
    [anon_sym_continue] = ACTIONS(5182),
    [anon_sym_default] = ACTIONS(5182),
    [anon_sym_enum] = ACTIONS(5182),
    [anon_sym_fn] = ACTIONS(5182),
    [anon_sym_for] = ACTIONS(5182),
    [anon_sym_gen] = ACTIONS(5182),
    [anon_sym_if] = ACTIONS(5182),
    [anon_sym_impl] = ACTIONS(5182),
    [anon_sym_let] = ACTIONS(5182),
    [anon_sym_loop] = ACTIONS(5182),
    [anon_sym_match] = ACTIONS(5182),
    [anon_sym_mod] = ACTIONS(5182),
    [anon_sym_pub] = ACTIONS(5182),
    [anon_sym_return] = ACTIONS(5182),
    [anon_sym_raw] = ACTIONS(5182),
    [anon_sym_static] = ACTIONS(5182),
    [anon_sym_struct] = ACTIONS(5182),
    [anon_sym_trait] = ACTIONS(5182),
    [anon_sym_type] = ACTIONS(5182),
    [anon_sym_union] = ACTIONS(5182),
    [anon_sym_unsafe] = ACTIONS(5182),
    [anon_sym_use] = ACTIONS(5182),
    [anon_sym_while] = ACTIONS(5182),
    [anon_sym_extern] = ACTIONS(5182),
    [anon_sym_safe] = ACTIONS(5182),
    [anon_sym_yield] = ACTIONS(5182),
    [anon_sym_move] = ACTIONS(5182),
    [anon_sym_try] = ACTIONS(5182),
    [sym_integer_literal] = ACTIONS(5180),
    [aux_sym_string_literal_token1] = ACTIONS(5180),
    [sym_char_literal] = ACTIONS(5180),
    [anon_sym_true] = ACTIONS(5182),
    [anon_sym_false] = ACTIONS(5182),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5182),
    [sym_super] = ACTIONS(5182),
    [sym_crate] = ACTIONS(5182),
    [aux_sym_metavariable_token1] = ACTIONS(5180),
    [sym__raw_string_literal_start] = ACTIONS(5180),
    [sym_float_literal] = ACTIONS(5180),
    [sym__outer_block_doc_comment_start] = ACTIONS(5180),
    [sym__outer_line_doc_comment_start] = ACTIONS(5180),
  },
  [STATE(1361)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4780),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1361),
    [sym_block_comment] = STATE(1361),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_COMMA] = ACTIONS(5184),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_RBRACK] = ACTIONS(5186),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1362)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4531),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1362),
    [sym_block_comment] = STATE(1362),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5188),
    [anon_sym_COMMA] = ACTIONS(5190),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1363)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4784),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1363),
    [sym_block_comment] = STATE(1363),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_COMMA] = ACTIONS(1466),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_RBRACK] = ACTIONS(1470),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1364)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4532),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1364),
    [sym_block_comment] = STATE(1364),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5192),
    [anon_sym_COMMA] = ACTIONS(5194),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1365)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_lifetime] = STATE(5792),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3764),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1365),
    [sym_block_comment] = STATE(1365),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_SQUOTE] = ACTIONS(943),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(5196),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5198),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1366)] = {
    [sym_line_comment] = STATE(1366),
    [sym_block_comment] = STATE(1366),
    [ts_builtin_sym_end] = ACTIONS(5200),
    [sym_identifier] = ACTIONS(5202),
    [anon_sym_SEMI] = ACTIONS(5200),
    [anon_sym_macro] = ACTIONS(5202),
    [anon_sym_LPAREN] = ACTIONS(5200),
    [anon_sym_LBRACE] = ACTIONS(5200),
    [anon_sym_macro_rules] = ACTIONS(5202),
    [anon_sym_BANG] = ACTIONS(5200),
    [anon_sym_LBRACK] = ACTIONS(5200),
    [anon_sym_STAR] = ACTIONS(5200),
    [anon_sym_u8] = ACTIONS(5202),
    [anon_sym_i8] = ACTIONS(5202),
    [anon_sym_u16] = ACTIONS(5202),
    [anon_sym_i16] = ACTIONS(5202),
    [anon_sym_u32] = ACTIONS(5202),
    [anon_sym_i32] = ACTIONS(5202),
    [anon_sym_u64] = ACTIONS(5202),
    [anon_sym_i64] = ACTIONS(5202),
    [anon_sym_u128] = ACTIONS(5202),
    [anon_sym_i128] = ACTIONS(5202),
    [anon_sym_isize] = ACTIONS(5202),
    [anon_sym_usize] = ACTIONS(5202),
    [anon_sym_f32] = ACTIONS(5202),
    [anon_sym_f64] = ACTIONS(5202),
    [anon_sym_bool] = ACTIONS(5202),
    [anon_sym_str] = ACTIONS(5202),
    [anon_sym_char] = ACTIONS(5202),
    [anon_sym_DASH] = ACTIONS(5200),
    [anon_sym_AMP] = ACTIONS(5200),
    [anon_sym_PIPE] = ACTIONS(5200),
    [anon_sym_LT] = ACTIONS(5200),
    [anon_sym_DOT_DOT] = ACTIONS(5200),
    [anon_sym_COLON_COLON] = ACTIONS(5200),
    [anon_sym_POUND] = ACTIONS(5200),
    [anon_sym_SQUOTE] = ACTIONS(5202),
    [anon_sym_async] = ACTIONS(5202),
    [anon_sym_auto] = ACTIONS(5202),
    [anon_sym_await] = ACTIONS(5202),
    [anon_sym_become] = ACTIONS(5202),
    [anon_sym_break] = ACTIONS(5202),
    [anon_sym_const] = ACTIONS(5202),
    [anon_sym_continue] = ACTIONS(5202),
    [anon_sym_default] = ACTIONS(5202),
    [anon_sym_enum] = ACTIONS(5202),
    [anon_sym_fn] = ACTIONS(5202),
    [anon_sym_for] = ACTIONS(5202),
    [anon_sym_gen] = ACTIONS(5202),
    [anon_sym_if] = ACTIONS(5202),
    [anon_sym_impl] = ACTIONS(5202),
    [anon_sym_let] = ACTIONS(5202),
    [anon_sym_loop] = ACTIONS(5202),
    [anon_sym_match] = ACTIONS(5202),
    [anon_sym_mod] = ACTIONS(5202),
    [anon_sym_pub] = ACTIONS(5202),
    [anon_sym_return] = ACTIONS(5202),
    [anon_sym_raw] = ACTIONS(5202),
    [anon_sym_static] = ACTIONS(5202),
    [anon_sym_struct] = ACTIONS(5202),
    [anon_sym_trait] = ACTIONS(5202),
    [anon_sym_type] = ACTIONS(5202),
    [anon_sym_union] = ACTIONS(5202),
    [anon_sym_unsafe] = ACTIONS(5202),
    [anon_sym_use] = ACTIONS(5202),
    [anon_sym_while] = ACTIONS(5202),
    [anon_sym_extern] = ACTIONS(5202),
    [anon_sym_safe] = ACTIONS(5202),
    [anon_sym_yield] = ACTIONS(5202),
    [anon_sym_move] = ACTIONS(5202),
    [anon_sym_try] = ACTIONS(5202),
    [sym_integer_literal] = ACTIONS(5200),
    [aux_sym_string_literal_token1] = ACTIONS(5200),
    [sym_char_literal] = ACTIONS(5200),
    [anon_sym_true] = ACTIONS(5202),
    [anon_sym_false] = ACTIONS(5202),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5202),
    [sym_super] = ACTIONS(5202),
    [sym_crate] = ACTIONS(5202),
    [aux_sym_metavariable_token1] = ACTIONS(5200),
    [sym__raw_string_literal_start] = ACTIONS(5200),
    [sym_float_literal] = ACTIONS(5200),
    [sym__outer_block_doc_comment_start] = ACTIONS(5200),
    [sym__outer_line_doc_comment_start] = ACTIONS(5200),
  },
  [STATE(1367)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4803),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1367),
    [sym_block_comment] = STATE(1367),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5204),
    [anon_sym_COMMA] = ACTIONS(5206),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1368)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_pattern] = STATE(6023),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1368),
    [sym_block_comment] = STATE(1368),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_POUND] = ACTIONS(5208),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1369)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4698),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1369),
    [sym_block_comment] = STATE(1369),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5210),
    [anon_sym_COMMA] = ACTIONS(5212),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1370)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1370),
    [sym_block_comment] = STATE(1370),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5214),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1371)] = {
    [sym_line_comment] = STATE(1371),
    [sym_block_comment] = STATE(1371),
    [sym_identifier] = ACTIONS(5216),
    [anon_sym_SEMI] = ACTIONS(5218),
    [anon_sym_macro] = ACTIONS(5216),
    [anon_sym_LPAREN] = ACTIONS(5218),
    [anon_sym_LBRACE] = ACTIONS(5218),
    [anon_sym_macro_rules] = ACTIONS(5216),
    [anon_sym_BANG] = ACTIONS(5218),
    [anon_sym_LBRACK] = ACTIONS(5218),
    [anon_sym_STAR] = ACTIONS(5218),
    [anon_sym_u8] = ACTIONS(5216),
    [anon_sym_i8] = ACTIONS(5216),
    [anon_sym_u16] = ACTIONS(5216),
    [anon_sym_i16] = ACTIONS(5216),
    [anon_sym_u32] = ACTIONS(5216),
    [anon_sym_i32] = ACTIONS(5216),
    [anon_sym_u64] = ACTIONS(5216),
    [anon_sym_i64] = ACTIONS(5216),
    [anon_sym_u128] = ACTIONS(5216),
    [anon_sym_i128] = ACTIONS(5216),
    [anon_sym_isize] = ACTIONS(5216),
    [anon_sym_usize] = ACTIONS(5216),
    [anon_sym_f32] = ACTIONS(5216),
    [anon_sym_f64] = ACTIONS(5216),
    [anon_sym_bool] = ACTIONS(5216),
    [anon_sym_str] = ACTIONS(5216),
    [anon_sym_char] = ACTIONS(5216),
    [anon_sym_DASH] = ACTIONS(5218),
    [anon_sym_AMP] = ACTIONS(5218),
    [anon_sym_PIPE] = ACTIONS(5218),
    [anon_sym_LT] = ACTIONS(5218),
    [anon_sym_DOT_DOT] = ACTIONS(5218),
    [anon_sym_COLON_COLON] = ACTIONS(5218),
    [anon_sym_POUND] = ACTIONS(5218),
    [anon_sym_SQUOTE] = ACTIONS(5216),
    [anon_sym_async] = ACTIONS(5216),
    [anon_sym_auto] = ACTIONS(5216),
    [anon_sym_await] = ACTIONS(5216),
    [anon_sym_become] = ACTIONS(5216),
    [anon_sym_break] = ACTIONS(5216),
    [anon_sym_const] = ACTIONS(5216),
    [anon_sym_continue] = ACTIONS(5216),
    [anon_sym_default] = ACTIONS(5216),
    [anon_sym_enum] = ACTIONS(5216),
    [anon_sym_fn] = ACTIONS(5216),
    [anon_sym_for] = ACTIONS(5216),
    [anon_sym_gen] = ACTIONS(5216),
    [anon_sym_if] = ACTIONS(5216),
    [anon_sym_impl] = ACTIONS(5216),
    [anon_sym_let] = ACTIONS(5216),
    [anon_sym_loop] = ACTIONS(5216),
    [anon_sym_match] = ACTIONS(5216),
    [anon_sym_mod] = ACTIONS(5216),
    [anon_sym_pub] = ACTIONS(5216),
    [anon_sym_return] = ACTIONS(5216),
    [anon_sym_raw] = ACTIONS(5216),
    [anon_sym_static] = ACTIONS(5216),
    [anon_sym_struct] = ACTIONS(5216),
    [anon_sym_trait] = ACTIONS(5216),
    [anon_sym_type] = ACTIONS(5216),
    [anon_sym_union] = ACTIONS(5216),
    [anon_sym_unsafe] = ACTIONS(5216),
    [anon_sym_use] = ACTIONS(5216),
    [anon_sym_while] = ACTIONS(5216),
    [anon_sym_extern] = ACTIONS(5216),
    [anon_sym_safe] = ACTIONS(5216),
    [anon_sym_yield] = ACTIONS(5216),
    [anon_sym_move] = ACTIONS(5216),
    [anon_sym_try] = ACTIONS(5216),
    [sym_integer_literal] = ACTIONS(5218),
    [aux_sym_string_literal_token1] = ACTIONS(5218),
    [sym_char_literal] = ACTIONS(5218),
    [anon_sym_true] = ACTIONS(5216),
    [anon_sym_false] = ACTIONS(5216),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5216),
    [sym_super] = ACTIONS(5216),
    [sym_crate] = ACTIONS(5216),
    [aux_sym_metavariable_token1] = ACTIONS(5218),
    [sym__raw_string_literal_start] = ACTIONS(5218),
    [sym_float_literal] = ACTIONS(5218),
    [sym__outer_block_doc_comment_start] = ACTIONS(5218),
    [sym__outer_line_doc_comment_start] = ACTIONS(5218),
  },
  [STATE(1372)] = {
    [sym_line_comment] = STATE(1372),
    [sym_block_comment] = STATE(1372),
    [sym_identifier] = ACTIONS(5220),
    [anon_sym_SEMI] = ACTIONS(5222),
    [anon_sym_macro] = ACTIONS(5220),
    [anon_sym_LPAREN] = ACTIONS(5222),
    [anon_sym_LBRACE] = ACTIONS(5222),
    [anon_sym_macro_rules] = ACTIONS(5220),
    [anon_sym_BANG] = ACTIONS(5222),
    [anon_sym_LBRACK] = ACTIONS(5222),
    [anon_sym_STAR] = ACTIONS(5222),
    [anon_sym_u8] = ACTIONS(5220),
    [anon_sym_i8] = ACTIONS(5220),
    [anon_sym_u16] = ACTIONS(5220),
    [anon_sym_i16] = ACTIONS(5220),
    [anon_sym_u32] = ACTIONS(5220),
    [anon_sym_i32] = ACTIONS(5220),
    [anon_sym_u64] = ACTIONS(5220),
    [anon_sym_i64] = ACTIONS(5220),
    [anon_sym_u128] = ACTIONS(5220),
    [anon_sym_i128] = ACTIONS(5220),
    [anon_sym_isize] = ACTIONS(5220),
    [anon_sym_usize] = ACTIONS(5220),
    [anon_sym_f32] = ACTIONS(5220),
    [anon_sym_f64] = ACTIONS(5220),
    [anon_sym_bool] = ACTIONS(5220),
    [anon_sym_str] = ACTIONS(5220),
    [anon_sym_char] = ACTIONS(5220),
    [anon_sym_DASH] = ACTIONS(5222),
    [anon_sym_AMP] = ACTIONS(5222),
    [anon_sym_PIPE] = ACTIONS(5222),
    [anon_sym_LT] = ACTIONS(5222),
    [anon_sym_DOT_DOT] = ACTIONS(5222),
    [anon_sym_COLON_COLON] = ACTIONS(5222),
    [anon_sym_POUND] = ACTIONS(5222),
    [anon_sym_SQUOTE] = ACTIONS(5220),
    [anon_sym_async] = ACTIONS(5220),
    [anon_sym_auto] = ACTIONS(5220),
    [anon_sym_await] = ACTIONS(5220),
    [anon_sym_become] = ACTIONS(5220),
    [anon_sym_break] = ACTIONS(5220),
    [anon_sym_const] = ACTIONS(5220),
    [anon_sym_continue] = ACTIONS(5220),
    [anon_sym_default] = ACTIONS(5220),
    [anon_sym_enum] = ACTIONS(5220),
    [anon_sym_fn] = ACTIONS(5220),
    [anon_sym_for] = ACTIONS(5220),
    [anon_sym_gen] = ACTIONS(5220),
    [anon_sym_if] = ACTIONS(5220),
    [anon_sym_impl] = ACTIONS(5220),
    [anon_sym_let] = ACTIONS(5220),
    [anon_sym_loop] = ACTIONS(5220),
    [anon_sym_match] = ACTIONS(5220),
    [anon_sym_mod] = ACTIONS(5220),
    [anon_sym_pub] = ACTIONS(5220),
    [anon_sym_return] = ACTIONS(5220),
    [anon_sym_raw] = ACTIONS(5220),
    [anon_sym_static] = ACTIONS(5220),
    [anon_sym_struct] = ACTIONS(5220),
    [anon_sym_trait] = ACTIONS(5220),
    [anon_sym_type] = ACTIONS(5220),
    [anon_sym_union] = ACTIONS(5220),
    [anon_sym_unsafe] = ACTIONS(5220),
    [anon_sym_use] = ACTIONS(5220),
    [anon_sym_while] = ACTIONS(5220),
    [anon_sym_extern] = ACTIONS(5220),
    [anon_sym_safe] = ACTIONS(5220),
    [anon_sym_yield] = ACTIONS(5220),
    [anon_sym_move] = ACTIONS(5220),
    [anon_sym_try] = ACTIONS(5220),
    [sym_integer_literal] = ACTIONS(5222),
    [aux_sym_string_literal_token1] = ACTIONS(5222),
    [sym_char_literal] = ACTIONS(5222),
    [anon_sym_true] = ACTIONS(5220),
    [anon_sym_false] = ACTIONS(5220),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5220),
    [sym_super] = ACTIONS(5220),
    [sym_crate] = ACTIONS(5220),
    [aux_sym_metavariable_token1] = ACTIONS(5222),
    [sym__raw_string_literal_start] = ACTIONS(5222),
    [sym_float_literal] = ACTIONS(5222),
    [sym__outer_block_doc_comment_start] = ACTIONS(5222),
    [sym__outer_line_doc_comment_start] = ACTIONS(5222),
  },
  [STATE(1373)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5821),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1373),
    [sym_block_comment] = STATE(1373),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(5224),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5226),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1374)] = {
    [sym_line_comment] = STATE(1374),
    [sym_block_comment] = STATE(1374),
    [sym_identifier] = ACTIONS(5228),
    [anon_sym_SEMI] = ACTIONS(5230),
    [anon_sym_macro] = ACTIONS(5228),
    [anon_sym_LPAREN] = ACTIONS(5230),
    [anon_sym_LBRACE] = ACTIONS(5230),
    [anon_sym_macro_rules] = ACTIONS(5228),
    [anon_sym_BANG] = ACTIONS(5230),
    [anon_sym_LBRACK] = ACTIONS(5230),
    [anon_sym_STAR] = ACTIONS(5230),
    [anon_sym_u8] = ACTIONS(5228),
    [anon_sym_i8] = ACTIONS(5228),
    [anon_sym_u16] = ACTIONS(5228),
    [anon_sym_i16] = ACTIONS(5228),
    [anon_sym_u32] = ACTIONS(5228),
    [anon_sym_i32] = ACTIONS(5228),
    [anon_sym_u64] = ACTIONS(5228),
    [anon_sym_i64] = ACTIONS(5228),
    [anon_sym_u128] = ACTIONS(5228),
    [anon_sym_i128] = ACTIONS(5228),
    [anon_sym_isize] = ACTIONS(5228),
    [anon_sym_usize] = ACTIONS(5228),
    [anon_sym_f32] = ACTIONS(5228),
    [anon_sym_f64] = ACTIONS(5228),
    [anon_sym_bool] = ACTIONS(5228),
    [anon_sym_str] = ACTIONS(5228),
    [anon_sym_char] = ACTIONS(5228),
    [anon_sym_DASH] = ACTIONS(5230),
    [anon_sym_AMP] = ACTIONS(5230),
    [anon_sym_PIPE] = ACTIONS(5230),
    [anon_sym_LT] = ACTIONS(5230),
    [anon_sym_DOT_DOT] = ACTIONS(5230),
    [anon_sym_COLON_COLON] = ACTIONS(5230),
    [anon_sym_POUND] = ACTIONS(5230),
    [anon_sym_SQUOTE] = ACTIONS(5228),
    [anon_sym_async] = ACTIONS(5228),
    [anon_sym_auto] = ACTIONS(5228),
    [anon_sym_await] = ACTIONS(5228),
    [anon_sym_become] = ACTIONS(5228),
    [anon_sym_break] = ACTIONS(5228),
    [anon_sym_const] = ACTIONS(5228),
    [anon_sym_continue] = ACTIONS(5228),
    [anon_sym_default] = ACTIONS(5228),
    [anon_sym_enum] = ACTIONS(5228),
    [anon_sym_fn] = ACTIONS(5228),
    [anon_sym_for] = ACTIONS(5228),
    [anon_sym_gen] = ACTIONS(5228),
    [anon_sym_if] = ACTIONS(5228),
    [anon_sym_impl] = ACTIONS(5228),
    [anon_sym_let] = ACTIONS(5228),
    [anon_sym_loop] = ACTIONS(5228),
    [anon_sym_match] = ACTIONS(5228),
    [anon_sym_mod] = ACTIONS(5228),
    [anon_sym_pub] = ACTIONS(5228),
    [anon_sym_return] = ACTIONS(5228),
    [anon_sym_raw] = ACTIONS(5228),
    [anon_sym_static] = ACTIONS(5228),
    [anon_sym_struct] = ACTIONS(5228),
    [anon_sym_trait] = ACTIONS(5228),
    [anon_sym_type] = ACTIONS(5228),
    [anon_sym_union] = ACTIONS(5228),
    [anon_sym_unsafe] = ACTIONS(5228),
    [anon_sym_use] = ACTIONS(5228),
    [anon_sym_while] = ACTIONS(5228),
    [anon_sym_extern] = ACTIONS(5228),
    [anon_sym_safe] = ACTIONS(5228),
    [anon_sym_yield] = ACTIONS(5228),
    [anon_sym_move] = ACTIONS(5228),
    [anon_sym_try] = ACTIONS(5228),
    [sym_integer_literal] = ACTIONS(5230),
    [aux_sym_string_literal_token1] = ACTIONS(5230),
    [sym_char_literal] = ACTIONS(5230),
    [anon_sym_true] = ACTIONS(5228),
    [anon_sym_false] = ACTIONS(5228),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5228),
    [sym_super] = ACTIONS(5228),
    [sym_crate] = ACTIONS(5228),
    [aux_sym_metavariable_token1] = ACTIONS(5230),
    [sym__raw_string_literal_start] = ACTIONS(5230),
    [sym_float_literal] = ACTIONS(5230),
    [sym__outer_block_doc_comment_start] = ACTIONS(5230),
    [sym__outer_line_doc_comment_start] = ACTIONS(5230),
  },
  [STATE(1375)] = {
    [sym_line_comment] = STATE(1375),
    [sym_block_comment] = STATE(1375),
    [sym_identifier] = ACTIONS(5232),
    [anon_sym_SEMI] = ACTIONS(5234),
    [anon_sym_macro] = ACTIONS(5232),
    [anon_sym_LPAREN] = ACTIONS(5234),
    [anon_sym_LBRACE] = ACTIONS(5234),
    [anon_sym_macro_rules] = ACTIONS(5232),
    [anon_sym_BANG] = ACTIONS(5234),
    [anon_sym_LBRACK] = ACTIONS(5234),
    [anon_sym_STAR] = ACTIONS(5234),
    [anon_sym_u8] = ACTIONS(5232),
    [anon_sym_i8] = ACTIONS(5232),
    [anon_sym_u16] = ACTIONS(5232),
    [anon_sym_i16] = ACTIONS(5232),
    [anon_sym_u32] = ACTIONS(5232),
    [anon_sym_i32] = ACTIONS(5232),
    [anon_sym_u64] = ACTIONS(5232),
    [anon_sym_i64] = ACTIONS(5232),
    [anon_sym_u128] = ACTIONS(5232),
    [anon_sym_i128] = ACTIONS(5232),
    [anon_sym_isize] = ACTIONS(5232),
    [anon_sym_usize] = ACTIONS(5232),
    [anon_sym_f32] = ACTIONS(5232),
    [anon_sym_f64] = ACTIONS(5232),
    [anon_sym_bool] = ACTIONS(5232),
    [anon_sym_str] = ACTIONS(5232),
    [anon_sym_char] = ACTIONS(5232),
    [anon_sym_DASH] = ACTIONS(5234),
    [anon_sym_AMP] = ACTIONS(5234),
    [anon_sym_PIPE] = ACTIONS(5234),
    [anon_sym_LT] = ACTIONS(5234),
    [anon_sym_DOT_DOT] = ACTIONS(5234),
    [anon_sym_COLON_COLON] = ACTIONS(5234),
    [anon_sym_POUND] = ACTIONS(5234),
    [anon_sym_SQUOTE] = ACTIONS(5232),
    [anon_sym_async] = ACTIONS(5232),
    [anon_sym_auto] = ACTIONS(5232),
    [anon_sym_await] = ACTIONS(5232),
    [anon_sym_become] = ACTIONS(5232),
    [anon_sym_break] = ACTIONS(5232),
    [anon_sym_const] = ACTIONS(5232),
    [anon_sym_continue] = ACTIONS(5232),
    [anon_sym_default] = ACTIONS(5232),
    [anon_sym_enum] = ACTIONS(5232),
    [anon_sym_fn] = ACTIONS(5232),
    [anon_sym_for] = ACTIONS(5232),
    [anon_sym_gen] = ACTIONS(5232),
    [anon_sym_if] = ACTIONS(5232),
    [anon_sym_impl] = ACTIONS(5232),
    [anon_sym_let] = ACTIONS(5232),
    [anon_sym_loop] = ACTIONS(5232),
    [anon_sym_match] = ACTIONS(5232),
    [anon_sym_mod] = ACTIONS(5232),
    [anon_sym_pub] = ACTIONS(5232),
    [anon_sym_return] = ACTIONS(5232),
    [anon_sym_raw] = ACTIONS(5232),
    [anon_sym_static] = ACTIONS(5232),
    [anon_sym_struct] = ACTIONS(5232),
    [anon_sym_trait] = ACTIONS(5232),
    [anon_sym_type] = ACTIONS(5232),
    [anon_sym_union] = ACTIONS(5232),
    [anon_sym_unsafe] = ACTIONS(5232),
    [anon_sym_use] = ACTIONS(5232),
    [anon_sym_while] = ACTIONS(5232),
    [anon_sym_extern] = ACTIONS(5232),
    [anon_sym_safe] = ACTIONS(5232),
    [anon_sym_yield] = ACTIONS(5232),
    [anon_sym_move] = ACTIONS(5232),
    [anon_sym_try] = ACTIONS(5232),
    [sym_integer_literal] = ACTIONS(5234),
    [aux_sym_string_literal_token1] = ACTIONS(5234),
    [sym_char_literal] = ACTIONS(5234),
    [anon_sym_true] = ACTIONS(5232),
    [anon_sym_false] = ACTIONS(5232),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5232),
    [sym_super] = ACTIONS(5232),
    [sym_crate] = ACTIONS(5232),
    [aux_sym_metavariable_token1] = ACTIONS(5234),
    [sym__raw_string_literal_start] = ACTIONS(5234),
    [sym_float_literal] = ACTIONS(5234),
    [sym__outer_block_doc_comment_start] = ACTIONS(5234),
    [sym__outer_line_doc_comment_start] = ACTIONS(5234),
  },
  [STATE(1376)] = {
    [sym_line_comment] = STATE(1376),
    [sym_block_comment] = STATE(1376),
    [sym_identifier] = ACTIONS(5236),
    [anon_sym_SEMI] = ACTIONS(5238),
    [anon_sym_macro] = ACTIONS(5236),
    [anon_sym_LPAREN] = ACTIONS(5238),
    [anon_sym_LBRACE] = ACTIONS(5238),
    [anon_sym_macro_rules] = ACTIONS(5236),
    [anon_sym_BANG] = ACTIONS(5238),
    [anon_sym_LBRACK] = ACTIONS(5238),
    [anon_sym_STAR] = ACTIONS(5238),
    [anon_sym_u8] = ACTIONS(5236),
    [anon_sym_i8] = ACTIONS(5236),
    [anon_sym_u16] = ACTIONS(5236),
    [anon_sym_i16] = ACTIONS(5236),
    [anon_sym_u32] = ACTIONS(5236),
    [anon_sym_i32] = ACTIONS(5236),
    [anon_sym_u64] = ACTIONS(5236),
    [anon_sym_i64] = ACTIONS(5236),
    [anon_sym_u128] = ACTIONS(5236),
    [anon_sym_i128] = ACTIONS(5236),
    [anon_sym_isize] = ACTIONS(5236),
    [anon_sym_usize] = ACTIONS(5236),
    [anon_sym_f32] = ACTIONS(5236),
    [anon_sym_f64] = ACTIONS(5236),
    [anon_sym_bool] = ACTIONS(5236),
    [anon_sym_str] = ACTIONS(5236),
    [anon_sym_char] = ACTIONS(5236),
    [anon_sym_DASH] = ACTIONS(5238),
    [anon_sym_AMP] = ACTIONS(5238),
    [anon_sym_PIPE] = ACTIONS(5238),
    [anon_sym_LT] = ACTIONS(5238),
    [anon_sym_DOT_DOT] = ACTIONS(5238),
    [anon_sym_COLON_COLON] = ACTIONS(5238),
    [anon_sym_POUND] = ACTIONS(5238),
    [anon_sym_SQUOTE] = ACTIONS(5236),
    [anon_sym_async] = ACTIONS(5236),
    [anon_sym_auto] = ACTIONS(5236),
    [anon_sym_await] = ACTIONS(5236),
    [anon_sym_become] = ACTIONS(5236),
    [anon_sym_break] = ACTIONS(5236),
    [anon_sym_const] = ACTIONS(5236),
    [anon_sym_continue] = ACTIONS(5236),
    [anon_sym_default] = ACTIONS(5236),
    [anon_sym_enum] = ACTIONS(5236),
    [anon_sym_fn] = ACTIONS(5236),
    [anon_sym_for] = ACTIONS(5236),
    [anon_sym_gen] = ACTIONS(5236),
    [anon_sym_if] = ACTIONS(5236),
    [anon_sym_impl] = ACTIONS(5236),
    [anon_sym_let] = ACTIONS(5236),
    [anon_sym_loop] = ACTIONS(5236),
    [anon_sym_match] = ACTIONS(5236),
    [anon_sym_mod] = ACTIONS(5236),
    [anon_sym_pub] = ACTIONS(5236),
    [anon_sym_return] = ACTIONS(5236),
    [anon_sym_raw] = ACTIONS(5236),
    [anon_sym_static] = ACTIONS(5236),
    [anon_sym_struct] = ACTIONS(5236),
    [anon_sym_trait] = ACTIONS(5236),
    [anon_sym_type] = ACTIONS(5236),
    [anon_sym_union] = ACTIONS(5236),
    [anon_sym_unsafe] = ACTIONS(5236),
    [anon_sym_use] = ACTIONS(5236),
    [anon_sym_while] = ACTIONS(5236),
    [anon_sym_extern] = ACTIONS(5236),
    [anon_sym_safe] = ACTIONS(5236),
    [anon_sym_yield] = ACTIONS(5236),
    [anon_sym_move] = ACTIONS(5236),
    [anon_sym_try] = ACTIONS(5236),
    [sym_integer_literal] = ACTIONS(5238),
    [aux_sym_string_literal_token1] = ACTIONS(5238),
    [sym_char_literal] = ACTIONS(5238),
    [anon_sym_true] = ACTIONS(5236),
    [anon_sym_false] = ACTIONS(5236),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5236),
    [sym_super] = ACTIONS(5236),
    [sym_crate] = ACTIONS(5236),
    [aux_sym_metavariable_token1] = ACTIONS(5238),
    [sym__raw_string_literal_start] = ACTIONS(5238),
    [sym_float_literal] = ACTIONS(5238),
    [sym__outer_block_doc_comment_start] = ACTIONS(5238),
    [sym__outer_line_doc_comment_start] = ACTIONS(5238),
  },
  [STATE(1377)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1377),
    [sym_block_comment] = STATE(1377),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_RBRACK] = ACTIONS(5240),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1378)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1378),
    [sym_block_comment] = STATE(1378),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_RBRACK] = ACTIONS(5242),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1379)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_pattern] = STATE(6341),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1379),
    [sym_block_comment] = STATE(1379),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1380)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1380),
    [sym_block_comment] = STATE(1380),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5244),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1381)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1381),
    [sym_block_comment] = STATE(1381),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_RBRACK] = ACTIONS(5246),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1382)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1382),
    [sym_block_comment] = STATE(1382),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5248),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1383)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1383),
    [sym_block_comment] = STATE(1383),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5250),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1384)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_match_pattern] = STATE(6023),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4999),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1384),
    [sym_block_comment] = STATE(1384),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1385)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1385),
    [sym_block_comment] = STATE(1385),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5252),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1386)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1386),
    [sym_block_comment] = STATE(1386),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5254),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1387)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1387),
    [sym_block_comment] = STATE(1387),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5256),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1388)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4690),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1388),
    [sym_block_comment] = STATE(1388),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(5258),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5260),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1389)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1389),
    [sym_block_comment] = STATE(1389),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_RBRACK] = ACTIONS(5262),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1390)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1390),
    [sym_block_comment] = STATE(1390),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_RPAREN] = ACTIONS(5264),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1391)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4283),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1391),
    [sym_block_comment] = STATE(1391),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(5266),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1392)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4416),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1392),
    [sym_block_comment] = STATE(1392),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1393)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3757),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1393),
    [sym_block_comment] = STATE(1393),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5268),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1394)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4867),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1394),
    [sym_block_comment] = STATE(1394),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1395)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4924),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1395),
    [sym_block_comment] = STATE(1395),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1396)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4859),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1396),
    [sym_block_comment] = STATE(1396),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1397)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3764),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1397),
    [sym_block_comment] = STATE(1397),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(5270),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1398)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(4961),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1398),
    [sym_block_comment] = STATE(1398),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1399)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3769),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1399),
    [sym_block_comment] = STATE(1399),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1400)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4274),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1400),
    [sym_block_comment] = STATE(1400),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(5272),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1401)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5794),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1401),
    [sym_block_comment] = STATE(1401),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1402)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4280),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1402),
    [sym_block_comment] = STATE(1402),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1403)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4372),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1403),
    [sym_block_comment] = STATE(1403),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(5274),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1404)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5338),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1404),
    [sym_block_comment] = STATE(1404),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1405)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3837),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(3837),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1405),
    [sym_block_comment] = STATE(1405),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_SEMI] = ACTIONS(5106),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5106),
    [anon_sym_COMMA] = ACTIONS(5106),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5108),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(5276),
    [anon_sym_async] = ACTIONS(5110),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_TILDEconst] = ACTIONS(5114),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1406)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3773),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1406),
    [sym_block_comment] = STATE(1406),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1407)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5301),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1407),
    [sym_block_comment] = STATE(1407),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1408)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4294),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1408),
    [sym_block_comment] = STATE(1408),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1409)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5302),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1409),
    [sym_block_comment] = STATE(1409),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1410)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3767),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1410),
    [sym_block_comment] = STATE(1410),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1411)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(5272),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1411),
    [sym_block_comment] = STATE(1411),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1412)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(5018),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1412),
    [sym_block_comment] = STATE(1412),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1413)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3757),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1413),
    [sym_block_comment] = STATE(1413),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1414)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(5271),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1414),
    [sym_block_comment] = STATE(1414),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(5279),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1415)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(5329),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1415),
    [sym_block_comment] = STATE(1415),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1416)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4736),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1416),
    [sym_block_comment] = STATE(1416),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1417)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4910),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1417),
    [sym_block_comment] = STATE(1417),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1418)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3795),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1418),
    [sym_block_comment] = STATE(1418),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1419)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3760),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1419),
    [sym_block_comment] = STATE(1419),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1420)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5488),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1420),
    [sym_block_comment] = STATE(1420),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5281),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1421)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5600),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1421),
    [sym_block_comment] = STATE(1421),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1422)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4415),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1422),
    [sym_block_comment] = STATE(1422),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1423)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(3774),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1423),
    [sym_block_comment] = STATE(1423),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1424)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5716),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1424),
    [sym_block_comment] = STATE(1424),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1425)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6289),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5071),
    [sym_macro_invocation] = STATE(5028),
    [sym_scoped_identifier] = STATE(3863),
    [sym_scoped_type_identifier] = STATE(5073),
    [sym_const_block] = STATE(5028),
    [sym__pattern] = STATE(5342),
    [sym_generic_pattern] = STATE(5028),
    [sym_tuple_pattern] = STATE(5028),
    [sym_slice_pattern] = STATE(5028),
    [sym_tuple_struct_pattern] = STATE(5028),
    [sym_struct_pattern] = STATE(5028),
    [sym_remaining_field_pattern] = STATE(5028),
    [sym_mut_pattern] = STATE(5028),
    [sym_range_pattern] = STATE(5028),
    [sym_ref_pattern] = STATE(5028),
    [sym_box_pattern] = STATE(5028),
    [sym_captured_pattern] = STATE(5028),
    [sym_reference_pattern] = STATE(5028),
    [sym_or_pattern] = STATE(5028),
    [sym__literal_pattern] = STATE(4128),
    [sym_negative_literal] = STATE(4104),
    [sym_string_literal] = STATE(4104),
    [sym_raw_string_literal] = STATE(4104),
    [sym_boolean_literal] = STATE(4104),
    [sym_line_comment] = STATE(1425),
    [sym_block_comment] = STATE(1425),
    [sym_metavariable] = STATE(4534),
    [sym_identifier] = ACTIONS(1658),
    [anon_sym_LPAREN] = ACTIONS(1660),
    [anon_sym_macro_rules] = ACTIONS(1664),
    [anon_sym_LBRACK] = ACTIONS(1666),
    [anon_sym_u8] = ACTIONS(1668),
    [anon_sym_i8] = ACTIONS(1668),
    [anon_sym_u16] = ACTIONS(1668),
    [anon_sym_i16] = ACTIONS(1668),
    [anon_sym_u32] = ACTIONS(1668),
    [anon_sym_i32] = ACTIONS(1668),
    [anon_sym_u64] = ACTIONS(1668),
    [anon_sym_i64] = ACTIONS(1668),
    [anon_sym_u128] = ACTIONS(1668),
    [anon_sym_i128] = ACTIONS(1668),
    [anon_sym_isize] = ACTIONS(1668),
    [anon_sym_usize] = ACTIONS(1668),
    [anon_sym_f32] = ACTIONS(1668),
    [anon_sym_f64] = ACTIONS(1668),
    [anon_sym_bool] = ACTIONS(1668),
    [anon_sym_str] = ACTIONS(1668),
    [anon_sym_char] = ACTIONS(1668),
    [anon_sym_DASH] = ACTIONS(1670),
    [anon_sym_AMP] = ACTIONS(1672),
    [anon_sym_PIPE] = ACTIONS(1674),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1676),
    [anon_sym_DOT_DOT] = ACTIONS(1678),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(1680),
    [anon_sym_COLON_COLON] = ACTIONS(1682),
    [anon_sym_auto] = ACTIONS(1664),
    [anon_sym_await] = ACTIONS(1664),
    [anon_sym_const] = ACTIONS(1686),
    [anon_sym_default] = ACTIONS(1664),
    [anon_sym_gen] = ACTIONS(1664),
    [anon_sym_raw] = ACTIONS(1664),
    [anon_sym_union] = ACTIONS(1664),
    [anon_sym_safe] = ACTIONS(1664),
    [anon_sym_ref] = ACTIONS(1688),
    [sym_mutable_specifier] = ACTIONS(1690),
    [anon_sym_try] = ACTIONS(1664),
    [anon_sym_box] = ACTIONS(1692),
    [sym_integer_literal] = ACTIONS(1694),
    [aux_sym_string_literal_token1] = ACTIONS(1696),
    [sym_char_literal] = ACTIONS(1694),
    [anon_sym_true] = ACTIONS(1698),
    [anon_sym_false] = ACTIONS(1698),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1700),
    [sym_super] = ACTIONS(1700),
    [sym_crate] = ACTIONS(1700),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(1702),
    [sym_float_literal] = ACTIONS(1694),
  },
  [STATE(1426)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4434),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1426),
    [sym_block_comment] = STATE(1426),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(5283),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1427)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5829),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1427),
    [sym_block_comment] = STATE(1427),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1428)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5929),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1428),
    [sym_block_comment] = STATE(1428),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1429)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5340),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1429),
    [sym_block_comment] = STATE(1429),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1430)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(4824),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1430),
    [sym_block_comment] = STATE(1430),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5285),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1431)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5871),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1431),
    [sym_block_comment] = STATE(1431),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1432)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5900),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1432),
    [sym_block_comment] = STATE(1432),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1433)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5923),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1433),
    [sym_block_comment] = STATE(1433),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1434)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5927),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1434),
    [sym_block_comment] = STATE(1434),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1435)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5928),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1435),
    [sym_block_comment] = STATE(1435),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(1324),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1978),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1436)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_bracketed_type] = STATE(6156),
    [sym_generic_type] = STATE(6194),
    [sym_generic_type_with_turbofish] = STATE(5300),
    [sym_macro_invocation] = STATE(3786),
    [sym_scoped_identifier] = STATE(3592),
    [sym_scoped_type_identifier] = STATE(5426),
    [sym_const_block] = STATE(3786),
    [sym__pattern] = STATE(5742),
    [sym_generic_pattern] = STATE(3786),
    [sym_tuple_pattern] = STATE(3786),
    [sym_slice_pattern] = STATE(3786),
    [sym_tuple_struct_pattern] = STATE(3786),
    [sym_struct_pattern] = STATE(3786),
    [sym_remaining_field_pattern] = STATE(3786),
    [sym_mut_pattern] = STATE(3786),
    [sym_range_pattern] = STATE(3786),
    [sym_ref_pattern] = STATE(3786),
    [sym_box_pattern] = STATE(3786),
    [sym_captured_pattern] = STATE(3786),
    [sym_reference_pattern] = STATE(3786),
    [sym_or_pattern] = STATE(3786),
    [sym__literal_pattern] = STATE(3678),
    [sym_negative_literal] = STATE(3672),
    [sym_string_literal] = STATE(3672),
    [sym_raw_string_literal] = STATE(3672),
    [sym_boolean_literal] = STATE(3672),
    [sym_line_comment] = STATE(1436),
    [sym_block_comment] = STATE(1436),
    [sym_metavariable] = STATE(4747),
    [sym_identifier] = ACTIONS(1954),
    [anon_sym_LPAREN] = ACTIONS(1956),
    [anon_sym_macro_rules] = ACTIONS(1960),
    [anon_sym_LBRACK] = ACTIONS(1962),
    [anon_sym_u8] = ACTIONS(1964),
    [anon_sym_i8] = ACTIONS(1964),
    [anon_sym_u16] = ACTIONS(1964),
    [anon_sym_i16] = ACTIONS(1964),
    [anon_sym_u32] = ACTIONS(1964),
    [anon_sym_i32] = ACTIONS(1964),
    [anon_sym_u64] = ACTIONS(1964),
    [anon_sym_i64] = ACTIONS(1964),
    [anon_sym_u128] = ACTIONS(1964),
    [anon_sym_i128] = ACTIONS(1964),
    [anon_sym_isize] = ACTIONS(1964),
    [anon_sym_usize] = ACTIONS(1964),
    [anon_sym_f32] = ACTIONS(1964),
    [anon_sym_f64] = ACTIONS(1964),
    [anon_sym_bool] = ACTIONS(1964),
    [anon_sym_str] = ACTIONS(1964),
    [anon_sym_char] = ACTIONS(1964),
    [anon_sym_DASH] = ACTIONS(925),
    [anon_sym_AMP] = ACTIONS(1966),
    [anon_sym_PIPE] = ACTIONS(929),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym__] = ACTIONS(1318),
    [anon_sym_DOT_DOT] = ACTIONS(933),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(937),
    [anon_sym_COLON_COLON] = ACTIONS(1970),
    [anon_sym_auto] = ACTIONS(1960),
    [anon_sym_await] = ACTIONS(1960),
    [anon_sym_const] = ACTIONS(1972),
    [anon_sym_default] = ACTIONS(1960),
    [anon_sym_gen] = ACTIONS(1960),
    [anon_sym_raw] = ACTIONS(1960),
    [anon_sym_union] = ACTIONS(1960),
    [anon_sym_safe] = ACTIONS(1960),
    [anon_sym_ref] = ACTIONS(963),
    [sym_mutable_specifier] = ACTIONS(5287),
    [anon_sym_try] = ACTIONS(1960),
    [anon_sym_box] = ACTIONS(969),
    [sym_integer_literal] = ACTIONS(971),
    [aux_sym_string_literal_token1] = ACTIONS(973),
    [sym_char_literal] = ACTIONS(971),
    [anon_sym_true] = ACTIONS(975),
    [anon_sym_false] = ACTIONS(975),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5289),
    [sym_super] = ACTIONS(1978),
    [sym_crate] = ACTIONS(1978),
    [aux_sym_metavariable_token1] = ACTIONS(121),
    [sym__raw_string_literal_start] = ACTIONS(983),
    [sym_float_literal] = ACTIONS(971),
  },
  [STATE(1437)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1437),
    [sym_block_comment] = STATE(1437),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(5291),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(5293),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1438)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2870),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1438),
    [sym_block_comment] = STATE(1438),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_PLUS] = ACTIONS(5305),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [sym_mutable_specifier] = ACTIONS(5329),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1439)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5671),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1439),
    [sym_block_comment] = STATE(1439),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(5335),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [sym_mutable_specifier] = ACTIONS(5337),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1440)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1595),
    [sym__type] = STATE(5424),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1440),
    [sym_block_comment] = STATE(1440),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1441)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1441),
    [sym_block_comment] = STATE(1441),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(5291),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(5339),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5341),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1442)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3380),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1442),
    [sym_block_comment] = STATE(1442),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_PLUS] = ACTIONS(5353),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [sym_mutable_specifier] = ACTIONS(5377),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1443)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1443),
    [sym_block_comment] = STATE(1443),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_PLUS] = ACTIONS(5335),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [sym_mutable_specifier] = ACTIONS(5381),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1444)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym_visibility_modifier] = STATE(1628),
    [sym__type] = STATE(4654),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1444),
    [sym_block_comment] = STATE(1444),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_pub] = ACTIONS(2103),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(2105),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1445)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1549),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4116),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4149),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3835),
    [sym_line_comment] = STATE(1445),
    [sym_block_comment] = STATE(1445),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5383),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5385),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5389),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1446)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5328),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1446),
    [sym_block_comment] = STATE(1446),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5391),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1447)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5328),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1447),
    [sym_block_comment] = STATE(1447),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5393),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1448)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5127),
    [sym_bracketed_type] = STATE(6234),
    [sym_qualified_type] = STATE(6139),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1448),
    [sym_block_comment] = STATE(1448),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1449)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1727),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4134),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4223),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3868),
    [sym_line_comment] = STATE(1449),
    [sym_block_comment] = STATE(1449),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5395),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5397),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5399),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1450)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4496),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1450),
    [sym_block_comment] = STATE(1450),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(5401),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1451)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1769),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4107),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4246),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3882),
    [sym_line_comment] = STATE(1451),
    [sym_block_comment] = STATE(1451),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5403),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5405),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5407),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1452)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5328),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1452),
    [sym_block_comment] = STATE(1452),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5409),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1453)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4538),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1453),
    [sym_block_comment] = STATE(1453),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5411),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1454)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4567),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1454),
    [sym_block_comment] = STATE(1454),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5413),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1455)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4567),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1455),
    [sym_block_comment] = STATE(1455),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5411),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1456)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3940),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(3941),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(3622),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1456),
    [sym_block_comment] = STATE(1456),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5415),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(5417),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_TILDEconst] = ACTIONS(5419),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1457)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4443),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1457),
    [sym_block_comment] = STATE(1457),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5411),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1458)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5328),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1458),
    [sym_block_comment] = STATE(1458),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5421),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1459)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2959),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(1438),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1459),
    [sym_block_comment] = STATE(1459),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [sym_mutable_specifier] = ACTIONS(5423),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1460)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1532),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4205),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4206),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3862),
    [sym_line_comment] = STATE(1460),
    [sym_block_comment] = STATE(1460),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5425),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5427),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5429),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1461)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3601),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(1437),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1461),
    [sym_block_comment] = STATE(1461),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [sym_mutable_specifier] = ACTIONS(5431),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1462)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3899),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(3900),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1462),
    [sym_block_comment] = STATE(1462),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5415),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(5417),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_TILDEconst] = ACTIONS(5419),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1463)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5328),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1463),
    [sym_block_comment] = STATE(1463),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5433),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1464)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3601),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(1443),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1464),
    [sym_block_comment] = STATE(1464),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [sym_mutable_specifier] = ACTIONS(5435),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1465)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4763),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1465),
    [sym_block_comment] = STATE(1465),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(5437),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1466)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5328),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1466),
    [sym_block_comment] = STATE(1466),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5439),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1467)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4443),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1467),
    [sym_block_comment] = STATE(1467),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_RPAREN] = ACTIONS(5441),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1468)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5552),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(1439),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1468),
    [sym_block_comment] = STATE(1468),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [sym_mutable_specifier] = ACTIONS(5443),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1469)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3327),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(1442),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1469),
    [sym_block_comment] = STATE(1469),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [sym_mutable_specifier] = ACTIONS(5445),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1470)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1582),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4157),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4171),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3840),
    [sym_line_comment] = STATE(1470),
    [sym_block_comment] = STATE(1470),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5447),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5449),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5451),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1471)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4732),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1471),
    [sym_block_comment] = STATE(1471),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(5453),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1472)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4775),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1472),
    [sym_block_comment] = STATE(1472),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(5455),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1473)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1739),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4217),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4232),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3873),
    [sym_line_comment] = STATE(1473),
    [sym_block_comment] = STATE(1473),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5457),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5459),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5461),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1474)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1566),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4141),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4219),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3872),
    [sym_line_comment] = STATE(1474),
    [sym_block_comment] = STATE(1474),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5463),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5465),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5467),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1475)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_type_parameters] = STATE(1740),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4222),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4235),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3875),
    [sym_line_comment] = STATE(1475),
    [sym_block_comment] = STATE(1475),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5469),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5471),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(5387),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5473),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1476)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4170),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1476),
    [sym_block_comment] = STATE(1476),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1477)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5910),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1477),
    [sym_block_comment] = STATE(1477),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1478)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5914),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1478),
    [sym_block_comment] = STATE(1478),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1479)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4659),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1479),
    [sym_block_comment] = STATE(1479),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1480)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4261),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1480),
    [sym_block_comment] = STATE(1480),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1481)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4262),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1481),
    [sym_block_comment] = STATE(1481),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1482)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5906),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1482),
    [sym_block_comment] = STATE(1482),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1483)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3893),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1483),
    [sym_block_comment] = STATE(1483),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5475),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1484)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3893),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1484),
    [sym_block_comment] = STATE(1484),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5475),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(5477),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1485)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5450),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1485),
    [sym_block_comment] = STATE(1485),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1486)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4763),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1486),
    [sym_block_comment] = STATE(1486),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1487)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5454),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1487),
    [sym_block_comment] = STATE(1487),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1488)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5477),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1488),
    [sym_block_comment] = STATE(1488),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1489)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4820),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1489),
    [sym_block_comment] = STATE(1489),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1490)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4440),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1490),
    [sym_block_comment] = STATE(1490),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1491)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4444),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1491),
    [sym_block_comment] = STATE(1491),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1492)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5436),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1492),
    [sym_block_comment] = STATE(1492),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1493)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4913),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1493),
    [sym_block_comment] = STATE(1493),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1494)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3614),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1494),
    [sym_block_comment] = STATE(1494),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1495)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3898),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1495),
    [sym_block_comment] = STATE(1495),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5479),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1496)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3898),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1496),
    [sym_block_comment] = STATE(1496),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5479),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(5481),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1497)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1497),
    [sym_block_comment] = STATE(1497),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1498)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5822),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1498),
    [sym_block_comment] = STATE(1498),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1499)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4505),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1499),
    [sym_block_comment] = STATE(1499),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1500)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5518),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1500),
    [sym_block_comment] = STATE(1500),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1501)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5480),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1501),
    [sym_block_comment] = STATE(1501),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1502)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5315),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1502),
    [sym_block_comment] = STATE(1502),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1503)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5654),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1503),
    [sym_block_comment] = STATE(1503),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1504)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5761),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1504),
    [sym_block_comment] = STATE(1504),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1505)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4143),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1505),
    [sym_block_comment] = STATE(1505),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1506)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4146),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1506),
    [sym_block_comment] = STATE(1506),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1507)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4175),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1507),
    [sym_block_comment] = STATE(1507),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1508)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4182),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1508),
    [sym_block_comment] = STATE(1508),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1509)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4156),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1509),
    [sym_block_comment] = STATE(1509),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1510)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5201),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1510),
    [sym_block_comment] = STATE(1510),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1511)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3625),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1511),
    [sym_block_comment] = STATE(1511),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1512)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4173),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1512),
    [sym_block_comment] = STATE(1512),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1513)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5655),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1513),
    [sym_block_comment] = STATE(1513),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1514)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5662),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1514),
    [sym_block_comment] = STATE(1514),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1515)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4631),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1515),
    [sym_block_comment] = STATE(1515),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1516)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5709),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1516),
    [sym_block_comment] = STATE(1516),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1517)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5510),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1517),
    [sym_block_comment] = STATE(1517),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1518)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4538),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1518),
    [sym_block_comment] = STATE(1518),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1519)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5511),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1519),
    [sym_block_comment] = STATE(1519),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1520)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5512),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1520),
    [sym_block_comment] = STATE(1520),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1521)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5039),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1521),
    [sym_block_comment] = STATE(1521),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1522)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3960),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1522),
    [sym_block_comment] = STATE(1522),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1523)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3622),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(3622),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(3622),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1523),
    [sym_block_comment] = STATE(1523),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1888),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1524)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5807),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1524),
    [sym_block_comment] = STATE(1524),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1525)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5751),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1525),
    [sym_block_comment] = STATE(1525),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1526)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4234),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1526),
    [sym_block_comment] = STATE(1526),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1527)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4071),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1527),
    [sym_block_comment] = STATE(1527),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1528)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4679),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1528),
    [sym_block_comment] = STATE(1528),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1529)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4239),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1529),
    [sym_block_comment] = STATE(1529),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1530)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4242),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1530),
    [sym_block_comment] = STATE(1530),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1531)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5522),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1531),
    [sym_block_comment] = STATE(1531),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1532)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4226),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4228),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3856),
    [sym_line_comment] = STATE(1532),
    [sym_block_comment] = STATE(1532),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5483),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5485),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5487),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1533)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4286),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1533),
    [sym_block_comment] = STATE(1533),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1534)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5226),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1534),
    [sym_block_comment] = STATE(1534),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1535)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5307),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1535),
    [sym_block_comment] = STATE(1535),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1536)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5124),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1536),
    [sym_block_comment] = STATE(1536),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1537)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5602),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1537),
    [sym_block_comment] = STATE(1537),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1538)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5615),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1538),
    [sym_block_comment] = STATE(1538),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1539)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5646),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1539),
    [sym_block_comment] = STATE(1539),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1540)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4809),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1540),
    [sym_block_comment] = STATE(1540),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1541)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4595),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1541),
    [sym_block_comment] = STATE(1541),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1542)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4710),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1542),
    [sym_block_comment] = STATE(1542),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1543)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4715),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1543),
    [sym_block_comment] = STATE(1543),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1544)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5790),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1544),
    [sym_block_comment] = STATE(1544),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1545)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4225),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1545),
    [sym_block_comment] = STATE(1545),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1546)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4072),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1546),
    [sym_block_comment] = STATE(1546),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1547)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3915),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1547),
    [sym_block_comment] = STATE(1547),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1548)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4092),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1548),
    [sym_block_comment] = STATE(1548),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1549)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4190),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4191),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3854),
    [sym_line_comment] = STATE(1549),
    [sym_block_comment] = STATE(1549),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5489),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5491),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5493),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1550)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5532),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1550),
    [sym_block_comment] = STATE(1550),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1551)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4099),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1551),
    [sym_block_comment] = STATE(1551),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1552)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4605),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1552),
    [sym_block_comment] = STATE(1552),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1553)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4524),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1553),
    [sym_block_comment] = STATE(1553),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1554)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4594),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1554),
    [sym_block_comment] = STATE(1554),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1555)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4610),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1555),
    [sym_block_comment] = STATE(1555),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1556)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3955),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1556),
    [sym_block_comment] = STATE(1556),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1557)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5034),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1557),
    [sym_block_comment] = STATE(1557),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1558)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5667),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1558),
    [sym_block_comment] = STATE(1558),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1559)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4721),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1559),
    [sym_block_comment] = STATE(1559),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1560)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4722),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1560),
    [sym_block_comment] = STATE(1560),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1561)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4723),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1561),
    [sym_block_comment] = STATE(1561),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1562)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5739),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1562),
    [sym_block_comment] = STATE(1562),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1563)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4684),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1563),
    [sym_block_comment] = STATE(1563),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1564)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3963),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1564),
    [sym_block_comment] = STATE(1564),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1565)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4671),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1565),
    [sym_block_comment] = STATE(1565),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1566)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4158),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4159),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3846),
    [sym_line_comment] = STATE(1566),
    [sym_block_comment] = STATE(1566),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5495),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5497),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5499),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1567)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2958),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1567),
    [sym_block_comment] = STATE(1567),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1568)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(2960),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(6378),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(2960),
    [sym_unsafe_bound_type] = STATE(2960),
    [sym_tuple_type] = STATE(2960),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(2833),
    [sym_generic_type_with_turbofish] = STATE(5664),
    [sym_bounded_type] = STATE(2960),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5837),
    [sym_scoped_type_identifier] = STATE(2790),
    [sym_line_comment] = STATE(1568),
    [sym_block_comment] = STATE(1568),
    [sym_metavariable] = STATE(5666),
    [sym_identifier] = ACTIONS(5501),
    [anon_sym_LPAREN] = ACTIONS(5503),
    [anon_sym_macro_rules] = ACTIONS(5505),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5507),
    [anon_sym_i8] = ACTIONS(5507),
    [anon_sym_u16] = ACTIONS(5507),
    [anon_sym_i16] = ACTIONS(5507),
    [anon_sym_u32] = ACTIONS(5507),
    [anon_sym_i32] = ACTIONS(5507),
    [anon_sym_u64] = ACTIONS(5507),
    [anon_sym_i64] = ACTIONS(5507),
    [anon_sym_u128] = ACTIONS(5507),
    [anon_sym_i128] = ACTIONS(5507),
    [anon_sym_isize] = ACTIONS(5507),
    [anon_sym_usize] = ACTIONS(5507),
    [anon_sym_f32] = ACTIONS(5507),
    [anon_sym_f64] = ACTIONS(5507),
    [anon_sym_bool] = ACTIONS(5507),
    [anon_sym_str] = ACTIONS(5507),
    [anon_sym_char] = ACTIONS(5507),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5505),
    [anon_sym_await] = ACTIONS(5505),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5509),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5511),
    [anon_sym_gen] = ACTIONS(5509),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(5505),
    [anon_sym_union] = ACTIONS(5505),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5509),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(5505),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1569)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4732),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1569),
    [sym_block_comment] = STATE(1569),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1570)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3620),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1570),
    [sym_block_comment] = STATE(1570),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1571)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3302),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1571),
    [sym_block_comment] = STATE(1571),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1572)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5950),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1572),
    [sym_block_comment] = STATE(1572),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1573)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2868),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1573),
    [sym_block_comment] = STATE(1573),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1574)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2870),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1574),
    [sym_block_comment] = STATE(1574),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1575)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2874),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1575),
    [sym_block_comment] = STATE(1575),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1576)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2879),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(2879),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(2879),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1576),
    [sym_block_comment] = STATE(1576),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(5515),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1577)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2920),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1577),
    [sym_block_comment] = STATE(1577),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1578)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2923),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1578),
    [sym_block_comment] = STATE(1578),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1579)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(2925),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3625),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(2925),
    [sym_unsafe_bound_type] = STATE(2925),
    [sym_tuple_type] = STATE(2925),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(2801),
    [sym_generic_type_with_turbofish] = STATE(5664),
    [sym_bounded_type] = STATE(2925),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5837),
    [sym_scoped_type_identifier] = STATE(2787),
    [sym_line_comment] = STATE(1579),
    [sym_block_comment] = STATE(1579),
    [sym_metavariable] = STATE(5666),
    [sym_identifier] = ACTIONS(5517),
    [anon_sym_LPAREN] = ACTIONS(5503),
    [anon_sym_macro_rules] = ACTIONS(5505),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5507),
    [anon_sym_i8] = ACTIONS(5507),
    [anon_sym_u16] = ACTIONS(5507),
    [anon_sym_i16] = ACTIONS(5507),
    [anon_sym_u32] = ACTIONS(5507),
    [anon_sym_i32] = ACTIONS(5507),
    [anon_sym_u64] = ACTIONS(5507),
    [anon_sym_i64] = ACTIONS(5507),
    [anon_sym_u128] = ACTIONS(5507),
    [anon_sym_i128] = ACTIONS(5507),
    [anon_sym_isize] = ACTIONS(5507),
    [anon_sym_usize] = ACTIONS(5507),
    [anon_sym_f32] = ACTIONS(5507),
    [anon_sym_f64] = ACTIONS(5507),
    [anon_sym_bool] = ACTIONS(5507),
    [anon_sym_str] = ACTIONS(5507),
    [anon_sym_char] = ACTIONS(5507),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5505),
    [anon_sym_await] = ACTIONS(5505),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5509),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(5509),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(5505),
    [anon_sym_union] = ACTIONS(5505),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5509),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(5505),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1580)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2929),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1580),
    [sym_block_comment] = STATE(1580),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1581)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2931),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1581),
    [sym_block_comment] = STATE(1581),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1582)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4187),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4188),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3853),
    [sym_line_comment] = STATE(1582),
    [sym_block_comment] = STATE(1582),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5519),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5521),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5523),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1583)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2980),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1583),
    [sym_block_comment] = STATE(1583),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1584)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2983),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1584),
    [sym_block_comment] = STATE(1584),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1585)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2907),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1585),
    [sym_block_comment] = STATE(1585),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1586)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2970),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1586),
    [sym_block_comment] = STATE(1586),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1587)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3627),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1587),
    [sym_block_comment] = STATE(1587),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1588)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3597),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1588),
    [sym_block_comment] = STATE(1588),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1589)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3374),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1589),
    [sym_block_comment] = STATE(1589),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1590)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3377),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(6154),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3377),
    [sym_unsafe_bound_type] = STATE(3377),
    [sym_tuple_type] = STATE(3377),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3129),
    [sym_generic_type_with_turbofish] = STATE(5836),
    [sym_bounded_type] = STATE(3377),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5863),
    [sym_scoped_type_identifier] = STATE(3062),
    [sym_line_comment] = STATE(1590),
    [sym_block_comment] = STATE(1590),
    [sym_metavariable] = STATE(5838),
    [sym_identifier] = ACTIONS(5525),
    [anon_sym_LPAREN] = ACTIONS(5527),
    [anon_sym_macro_rules] = ACTIONS(5529),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5531),
    [anon_sym_i8] = ACTIONS(5531),
    [anon_sym_u16] = ACTIONS(5531),
    [anon_sym_i16] = ACTIONS(5531),
    [anon_sym_u32] = ACTIONS(5531),
    [anon_sym_i32] = ACTIONS(5531),
    [anon_sym_u64] = ACTIONS(5531),
    [anon_sym_i64] = ACTIONS(5531),
    [anon_sym_u128] = ACTIONS(5531),
    [anon_sym_i128] = ACTIONS(5531),
    [anon_sym_isize] = ACTIONS(5531),
    [anon_sym_usize] = ACTIONS(5531),
    [anon_sym_f32] = ACTIONS(5531),
    [anon_sym_f64] = ACTIONS(5531),
    [anon_sym_bool] = ACTIONS(5531),
    [anon_sym_str] = ACTIONS(5531),
    [anon_sym_char] = ACTIONS(5531),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5529),
    [anon_sym_await] = ACTIONS(5529),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5533),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5535),
    [anon_sym_gen] = ACTIONS(5533),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(5529),
    [anon_sym_union] = ACTIONS(5529),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5533),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(5529),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1591)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5915),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1591),
    [sym_block_comment] = STATE(1591),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1592)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5917),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1592),
    [sym_block_comment] = STATE(1592),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1593)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5941),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1593),
    [sym_block_comment] = STATE(1593),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1594)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3912),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1594),
    [sym_block_comment] = STATE(1594),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1595)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4980),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1595),
    [sym_block_comment] = STATE(1595),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1596)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3379),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1596),
    [sym_block_comment] = STATE(1596),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1597)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3380),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1597),
    [sym_block_comment] = STATE(1597),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1598)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3381),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1598),
    [sym_block_comment] = STATE(1598),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1599)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3418),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(3418),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(3418),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1599),
    [sym_block_comment] = STATE(1599),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(5537),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(5539),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1600)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3896),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1600),
    [sym_block_comment] = STATE(1600),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5541),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1601)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4442),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1601),
    [sym_block_comment] = STATE(1601),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1602)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3420),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1602),
    [sym_block_comment] = STATE(1602),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1603)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3424),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1603),
    [sym_block_comment] = STATE(1603),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1604)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3304),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3625),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3304),
    [sym_unsafe_bound_type] = STATE(3304),
    [sym_tuple_type] = STATE(3304),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3168),
    [sym_generic_type_with_turbofish] = STATE(5836),
    [sym_bounded_type] = STATE(3304),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5863),
    [sym_scoped_type_identifier] = STATE(3058),
    [sym_line_comment] = STATE(1604),
    [sym_block_comment] = STATE(1604),
    [sym_metavariable] = STATE(5838),
    [sym_identifier] = ACTIONS(5543),
    [anon_sym_LPAREN] = ACTIONS(5527),
    [anon_sym_macro_rules] = ACTIONS(5529),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5531),
    [anon_sym_i8] = ACTIONS(5531),
    [anon_sym_u16] = ACTIONS(5531),
    [anon_sym_i16] = ACTIONS(5531),
    [anon_sym_u32] = ACTIONS(5531),
    [anon_sym_i32] = ACTIONS(5531),
    [anon_sym_u64] = ACTIONS(5531),
    [anon_sym_i64] = ACTIONS(5531),
    [anon_sym_u128] = ACTIONS(5531),
    [anon_sym_i128] = ACTIONS(5531),
    [anon_sym_isize] = ACTIONS(5531),
    [anon_sym_usize] = ACTIONS(5531),
    [anon_sym_f32] = ACTIONS(5531),
    [anon_sym_f64] = ACTIONS(5531),
    [anon_sym_bool] = ACTIONS(5531),
    [anon_sym_str] = ACTIONS(5531),
    [anon_sym_char] = ACTIONS(5531),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5529),
    [anon_sym_await] = ACTIONS(5529),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5533),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(5533),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(5529),
    [anon_sym_union] = ACTIONS(5529),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5533),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(5529),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1605)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3426),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1605),
    [sym_block_comment] = STATE(1605),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1606)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3429),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1606),
    [sym_block_comment] = STATE(1606),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1607)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5003),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1607),
    [sym_block_comment] = STATE(1607),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1608)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3209),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1608),
    [sym_block_comment] = STATE(1608),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1609)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3185),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1609),
    [sym_block_comment] = STATE(1609),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1610)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5598),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1610),
    [sym_block_comment] = STATE(1610),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1611)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3191),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1611),
    [sym_block_comment] = STATE(1611),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1612)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5599),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1612),
    [sym_block_comment] = STATE(1612),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1613)] = {
    [sym_primitive_type] = STATE(5468),
    [sym_function_modifiers] = STATE(6228),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3349),
    [sym_removed_trait_bound] = STATE(3349),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3195),
    [sym_bracketed_type] = STATE(6295),
    [sym_lifetime] = STATE(6154),
    [sym_array_type] = STATE(3349),
    [sym_function_type] = STATE(3349),
    [sym_unsafe_bound_type] = STATE(3349),
    [sym_tuple_type] = STATE(3349),
    [sym_unit_type] = STATE(3349),
    [sym_generic_type] = STATE(3130),
    [sym_generic_type_with_turbofish] = STATE(3135),
    [sym_bounded_type] = STATE(3349),
    [sym_use_bounds] = STATE(6154),
    [sym_reference_type] = STATE(3349),
    [sym_pointer_type] = STATE(3349),
    [sym_never_type] = STATE(3349),
    [sym_abstract_type] = STATE(3349),
    [sym_dynamic_type] = STATE(3349),
    [sym_macro_invocation] = STATE(3349),
    [sym_scoped_identifier] = STATE(5857),
    [sym_scoped_type_identifier] = STATE(3074),
    [sym_line_comment] = STATE(1613),
    [sym_block_comment] = STATE(1613),
    [sym_metavariable] = STATE(3140),
    [sym_identifier] = ACTIONS(5343),
    [anon_sym_LPAREN] = ACTIONS(5345),
    [anon_sym_macro_rules] = ACTIONS(5347),
    [anon_sym_BANG] = ACTIONS(5349),
    [anon_sym_LBRACK] = ACTIONS(5351),
    [anon_sym_STAR] = ACTIONS(5355),
    [anon_sym_QMARK] = ACTIONS(5357),
    [anon_sym_u8] = ACTIONS(5359),
    [anon_sym_i8] = ACTIONS(5359),
    [anon_sym_u16] = ACTIONS(5359),
    [anon_sym_i16] = ACTIONS(5359),
    [anon_sym_u32] = ACTIONS(5359),
    [anon_sym_i32] = ACTIONS(5359),
    [anon_sym_u64] = ACTIONS(5359),
    [anon_sym_i64] = ACTIONS(5359),
    [anon_sym_u128] = ACTIONS(5359),
    [anon_sym_i128] = ACTIONS(5359),
    [anon_sym_isize] = ACTIONS(5359),
    [anon_sym_usize] = ACTIONS(5359),
    [anon_sym_f32] = ACTIONS(5359),
    [anon_sym_f64] = ACTIONS(5359),
    [anon_sym_bool] = ACTIONS(5359),
    [anon_sym_str] = ACTIONS(5359),
    [anon_sym_char] = ACTIONS(5359),
    [anon_sym_AMP] = ACTIONS(5361),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5363),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5347),
    [anon_sym_await] = ACTIONS(5347),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5365),
    [anon_sym_fn] = ACTIONS(5367),
    [anon_sym_for] = ACTIONS(5369),
    [anon_sym_gen] = ACTIONS(5365),
    [anon_sym_impl] = ACTIONS(5371),
    [anon_sym_raw] = ACTIONS(5347),
    [anon_sym_union] = ACTIONS(5347),
    [anon_sym_unsafe] = ACTIONS(5373),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5365),
    [anon_sym_dyn] = ACTIONS(5375),
    [anon_sym_try] = ACTIONS(5347),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5379),
    [sym_super] = ACTIONS(5379),
    [sym_crate] = ACTIONS(5379),
    [aux_sym_metavariable_token1] = ACTIONS(512),
  },
  [STATE(1614)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4178),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1614),
    [sym_block_comment] = STATE(1614),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1615)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5601),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1615),
    [sym_block_comment] = STATE(1615),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1616)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4265),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1616),
    [sym_block_comment] = STATE(1616),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1617)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4102),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1617),
    [sym_block_comment] = STATE(1617),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1618)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4111),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1618),
    [sym_block_comment] = STATE(1618),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1619)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4180),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1619),
    [sym_block_comment] = STATE(1619),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1620)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4181),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1620),
    [sym_block_comment] = STATE(1620),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1621)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5689),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1621),
    [sym_block_comment] = STATE(1621),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1622)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5698),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1622),
    [sym_block_comment] = STATE(1622),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1623)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5699),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1623),
    [sym_block_comment] = STATE(1623),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1624)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4105),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1624),
    [sym_block_comment] = STATE(1624),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1625)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4122),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1625),
    [sym_block_comment] = STATE(1625),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1626)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4354),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1626),
    [sym_block_comment] = STATE(1626),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1627)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4862),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1627),
    [sym_block_comment] = STATE(1627),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1628)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4551),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1628),
    [sym_block_comment] = STATE(1628),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1629)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4893),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1629),
    [sym_block_comment] = STATE(1629),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1630)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5815),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1630),
    [sym_block_comment] = STATE(1630),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1631)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3829),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1631),
    [sym_block_comment] = STATE(1631),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5545),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1632)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5555),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1632),
    [sym_block_comment] = STATE(1632),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1633)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4661),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1633),
    [sym_block_comment] = STATE(1633),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1634)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5855),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1634),
    [sym_block_comment] = STATE(1634),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1635)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5562),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1635),
    [sym_block_comment] = STATE(1635),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1636)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5864),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1636),
    [sym_block_comment] = STATE(1636),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1637)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5872),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1637),
    [sym_block_comment] = STATE(1637),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1638)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5563),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1638),
    [sym_block_comment] = STATE(1638),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1639)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4151),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1639),
    [sym_block_comment] = STATE(1639),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1640)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4155),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1640),
    [sym_block_comment] = STATE(1640),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1641)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4167),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1641),
    [sym_block_comment] = STATE(1641),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1642)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4168),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1642),
    [sym_block_comment] = STATE(1642),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1643)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4886),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1643),
    [sym_block_comment] = STATE(1643),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1644)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3905),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1644),
    [sym_block_comment] = STATE(1644),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1645)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4628),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1645),
    [sym_block_comment] = STATE(1645),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1646)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4629),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1646),
    [sym_block_comment] = STATE(1646),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1647)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4640),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1647),
    [sym_block_comment] = STATE(1647),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1648)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5592),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1648),
    [sym_block_comment] = STATE(1648),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1649)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4121),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1649),
    [sym_block_comment] = STATE(1649),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1650)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3614),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1650),
    [sym_block_comment] = STATE(1650),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1651)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3623),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1651),
    [sym_block_comment] = STATE(1651),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1652)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4123),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1652),
    [sym_block_comment] = STATE(1652),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1653)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3625),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1653),
    [sym_block_comment] = STATE(1653),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1654)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4443),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1654),
    [sym_block_comment] = STATE(1654),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1655)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3622),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(3622),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(3622),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1655),
    [sym_block_comment] = STATE(1655),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1656)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4779),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1656),
    [sym_block_comment] = STATE(1656),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1657)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4781),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1657),
    [sym_block_comment] = STATE(1657),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1658)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4783),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1658),
    [sym_block_comment] = STATE(1658),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1659)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5491),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1659),
    [sym_block_comment] = STATE(1659),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1660)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3916),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1660),
    [sym_block_comment] = STATE(1660),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1661)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3617),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1661),
    [sym_block_comment] = STATE(1661),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1662)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3628),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1662),
    [sym_block_comment] = STATE(1662),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1663)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3631),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1663),
    [sym_block_comment] = STATE(1663),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1664)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3638),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1664),
    [sym_block_comment] = STATE(1664),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1665)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5649),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1665),
    [sym_block_comment] = STATE(1665),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1666)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4463),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1666),
    [sym_block_comment] = STATE(1666),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1667)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5541),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1667),
    [sym_block_comment] = STATE(1667),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1668)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3944),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1668),
    [sym_block_comment] = STATE(1668),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1669)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5542),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1669),
    [sym_block_comment] = STATE(1669),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1670)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5547),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1670),
    [sym_block_comment] = STATE(1670),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1671)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4140),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1671),
    [sym_block_comment] = STATE(1671),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1672)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5110),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1672),
    [sym_block_comment] = STATE(1672),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1673)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4142),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1673),
    [sym_block_comment] = STATE(1673),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1674)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3929),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1674),
    [sym_block_comment] = STATE(1674),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1675)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4144),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1675),
    [sym_block_comment] = STATE(1675),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1676)] = {
    [sym_primitive_type] = STATE(5672),
    [sym_function_modifiers] = STATE(6257),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(2906),
    [sym_removed_trait_bound] = STATE(2906),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(2880),
    [sym_bracketed_type] = STATE(6281),
    [sym_lifetime] = STATE(6378),
    [sym_array_type] = STATE(2906),
    [sym_function_type] = STATE(2906),
    [sym_unsafe_bound_type] = STATE(2906),
    [sym_tuple_type] = STATE(2906),
    [sym_unit_type] = STATE(2906),
    [sym_generic_type] = STATE(2835),
    [sym_generic_type_with_turbofish] = STATE(2811),
    [sym_bounded_type] = STATE(2906),
    [sym_use_bounds] = STATE(6378),
    [sym_reference_type] = STATE(2906),
    [sym_pointer_type] = STATE(2906),
    [sym_never_type] = STATE(2906),
    [sym_abstract_type] = STATE(2906),
    [sym_dynamic_type] = STATE(2906),
    [sym_macro_invocation] = STATE(2906),
    [sym_scoped_identifier] = STATE(5665),
    [sym_scoped_type_identifier] = STATE(2786),
    [sym_line_comment] = STATE(1676),
    [sym_block_comment] = STATE(1676),
    [sym_metavariable] = STATE(2840),
    [sym_identifier] = ACTIONS(5295),
    [anon_sym_LPAREN] = ACTIONS(5297),
    [anon_sym_macro_rules] = ACTIONS(5299),
    [anon_sym_BANG] = ACTIONS(5301),
    [anon_sym_LBRACK] = ACTIONS(5303),
    [anon_sym_STAR] = ACTIONS(5307),
    [anon_sym_QMARK] = ACTIONS(5309),
    [anon_sym_u8] = ACTIONS(5311),
    [anon_sym_i8] = ACTIONS(5311),
    [anon_sym_u16] = ACTIONS(5311),
    [anon_sym_i16] = ACTIONS(5311),
    [anon_sym_u32] = ACTIONS(5311),
    [anon_sym_i32] = ACTIONS(5311),
    [anon_sym_u64] = ACTIONS(5311),
    [anon_sym_i64] = ACTIONS(5311),
    [anon_sym_u128] = ACTIONS(5311),
    [anon_sym_i128] = ACTIONS(5311),
    [anon_sym_isize] = ACTIONS(5311),
    [anon_sym_usize] = ACTIONS(5311),
    [anon_sym_f32] = ACTIONS(5311),
    [anon_sym_f64] = ACTIONS(5311),
    [anon_sym_bool] = ACTIONS(5311),
    [anon_sym_str] = ACTIONS(5311),
    [anon_sym_char] = ACTIONS(5311),
    [anon_sym_AMP] = ACTIONS(5313),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5315),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5299),
    [anon_sym_await] = ACTIONS(5299),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5317),
    [anon_sym_fn] = ACTIONS(5319),
    [anon_sym_for] = ACTIONS(5321),
    [anon_sym_gen] = ACTIONS(5317),
    [anon_sym_impl] = ACTIONS(5323),
    [anon_sym_raw] = ACTIONS(5299),
    [anon_sym_union] = ACTIONS(5299),
    [anon_sym_unsafe] = ACTIONS(5325),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5317),
    [anon_sym_dyn] = ACTIONS(5327),
    [anon_sym_try] = ACTIONS(5299),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5331),
    [sym_super] = ACTIONS(5331),
    [sym_crate] = ACTIONS(5331),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1677)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4145),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1677),
    [sym_block_comment] = STATE(1677),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1678)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3627),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1678),
    [sym_block_comment] = STATE(1678),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1679)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3597),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1679),
    [sym_block_comment] = STATE(1679),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1680)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4302),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1680),
    [sym_block_comment] = STATE(1680),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1681)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4694),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1681),
    [sym_block_comment] = STATE(1681),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1682)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5167),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1682),
    [sym_block_comment] = STATE(1682),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1683)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3598),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1683),
    [sym_block_comment] = STATE(1683),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1684)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3620),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1684),
    [sym_block_comment] = STATE(1684),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1685)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5717),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1685),
    [sym_block_comment] = STATE(1685),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1686)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5718),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1686),
    [sym_block_comment] = STATE(1686),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1687)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5721),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1687),
    [sym_block_comment] = STATE(1687),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1688)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5668),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1688),
    [sym_block_comment] = STATE(1688),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1689)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5671),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1689),
    [sym_block_comment] = STATE(1689),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1690)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4775),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1690),
    [sym_block_comment] = STATE(1690),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1691)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5687),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1691),
    [sym_block_comment] = STATE(1691),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1692)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4567),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1692),
    [sym_block_comment] = STATE(1692),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1693)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4725),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1693),
    [sym_block_comment] = STATE(1693),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1694)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4726),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1694),
    [sym_block_comment] = STATE(1694),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1695)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4496),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1695),
    [sym_block_comment] = STATE(1695),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1696)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4727),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1696),
    [sym_block_comment] = STATE(1696),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1697)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5708),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1697),
    [sym_block_comment] = STATE(1697),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1698)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5715),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1698),
    [sym_block_comment] = STATE(1698),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1699)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5735),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1699),
    [sym_block_comment] = STATE(1699),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1700)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5737),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1700),
    [sym_block_comment] = STATE(1700),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1701)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5769),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1701),
    [sym_block_comment] = STATE(1701),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1702)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5771),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1702),
    [sym_block_comment] = STATE(1702),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1703)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5791),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1703),
    [sym_block_comment] = STATE(1703),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1704)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5806),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1704),
    [sym_block_comment] = STATE(1704),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1705)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5206),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1705),
    [sym_block_comment] = STATE(1705),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1706)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5607),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1706),
    [sym_block_comment] = STATE(1706),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1707)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4745),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1707),
    [sym_block_comment] = STATE(1707),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1708)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3617),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1708),
    [sym_block_comment] = STATE(1708),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1709)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3952),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1709),
    [sym_block_comment] = STATE(1709),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1710)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3628),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1710),
    [sym_block_comment] = STATE(1710),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1711)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3630),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3625),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3630),
    [sym_unsafe_bound_type] = STATE(3630),
    [sym_tuple_type] = STATE(3630),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3595),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3630),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3830),
    [sym_line_comment] = STATE(1711),
    [sym_block_comment] = STATE(1711),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5547),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1712)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4769),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1712),
    [sym_block_comment] = STATE(1712),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1713)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5674),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1713),
    [sym_block_comment] = STATE(1713),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1714)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3631),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1714),
    [sym_block_comment] = STATE(1714),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1715)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4770),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1715),
    [sym_block_comment] = STATE(1715),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1716)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4771),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1716),
    [sym_block_comment] = STATE(1716),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1717)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3638),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1717),
    [sym_block_comment] = STATE(1717),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1718)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4986),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1718),
    [sym_block_comment] = STATE(1718),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1719)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3612),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1719),
    [sym_block_comment] = STATE(1719),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1720)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4427),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1720),
    [sym_block_comment] = STATE(1720),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1721)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5949),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1721),
    [sym_block_comment] = STATE(1721),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1722)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3895),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1722),
    [sym_block_comment] = STATE(1722),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1723)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3642),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(6148),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3642),
    [sym_unsafe_bound_type] = STATE(3642),
    [sym_tuple_type] = STATE(3642),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3579),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3642),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3824),
    [sym_line_comment] = STATE(1723),
    [sym_block_comment] = STATE(1723),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5549),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(5551),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1724)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4990),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1724),
    [sym_block_comment] = STATE(1724),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1725)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3612),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1725),
    [sym_block_comment] = STATE(1725),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1726)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3642),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(6258),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3642),
    [sym_unsafe_bound_type] = STATE(3642),
    [sym_tuple_type] = STATE(3642),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3579),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3642),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3552),
    [sym_line_comment] = STATE(1726),
    [sym_block_comment] = STATE(1726),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5553),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(5555),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1727)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4172),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4229),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3871),
    [sym_line_comment] = STATE(1727),
    [sym_block_comment] = STATE(1727),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5557),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5559),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5561),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1728)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4284),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1728),
    [sym_block_comment] = STATE(1728),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1729)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4198),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1729),
    [sym_block_comment] = STATE(1729),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1730)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4288),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1730),
    [sym_block_comment] = STATE(1730),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1731)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4995),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1731),
    [sym_block_comment] = STATE(1731),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1732)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4795),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1732),
    [sym_block_comment] = STATE(1732),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1733)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4091),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1733),
    [sym_block_comment] = STATE(1733),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1734)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4093),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1734),
    [sym_block_comment] = STATE(1734),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1735)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4562),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1735),
    [sym_block_comment] = STATE(1735),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1736)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5052),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1736),
    [sym_block_comment] = STATE(1736),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1737)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5478),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1737),
    [sym_block_comment] = STATE(1737),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1738)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4569),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1738),
    [sym_block_comment] = STATE(1738),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1739)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4097),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4240),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3878),
    [sym_line_comment] = STATE(1739),
    [sym_block_comment] = STATE(1739),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5563),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5565),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5567),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1740)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4103),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4243),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3880),
    [sym_line_comment] = STATE(1740),
    [sym_block_comment] = STATE(1740),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5569),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5571),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5573),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1741)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4970),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1741),
    [sym_block_comment] = STATE(1741),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1742)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5740),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1742),
    [sym_block_comment] = STATE(1742),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1743)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5743),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1743),
    [sym_block_comment] = STATE(1743),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1744)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3630),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3625),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3630),
    [sym_unsafe_bound_type] = STATE(3630),
    [sym_tuple_type] = STATE(3630),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3595),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3630),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3551),
    [sym_line_comment] = STATE(1744),
    [sym_block_comment] = STATE(1744),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5575),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1745)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4298),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1745),
    [sym_block_comment] = STATE(1745),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1746)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5559),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1746),
    [sym_block_comment] = STATE(1746),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1747)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3913),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1747),
    [sym_block_comment] = STATE(1747),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1748)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4135),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1748),
    [sym_block_comment] = STATE(1748),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1749)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4136),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1749),
    [sym_block_comment] = STATE(1749),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1750)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4138),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1750),
    [sym_block_comment] = STATE(1750),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1751)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4139),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1751),
    [sym_block_comment] = STATE(1751),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1752)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4741),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1752),
    [sym_block_comment] = STATE(1752),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1753)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5744),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1753),
    [sym_block_comment] = STATE(1753),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1754)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4897),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1754),
    [sym_block_comment] = STATE(1754),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1755)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5579),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1755),
    [sym_block_comment] = STATE(1755),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1756)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5581),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1756),
    [sym_block_comment] = STATE(1756),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1757)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5582),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1757),
    [sym_block_comment] = STATE(1757),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1758)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4742),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1758),
    [sym_block_comment] = STATE(1758),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1759)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4752),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1759),
    [sym_block_comment] = STATE(1759),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1760)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4757),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1760),
    [sym_block_comment] = STATE(1760),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1761)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4937),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1761),
    [sym_block_comment] = STATE(1761),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1762)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4153),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1762),
    [sym_block_comment] = STATE(1762),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1763)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4154),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1763),
    [sym_block_comment] = STATE(1763),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1764)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4787),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1764),
    [sym_block_comment] = STATE(1764),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1765)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4076),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1765),
    [sym_block_comment] = STATE(1765),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1766)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4955),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1766),
    [sym_block_comment] = STATE(1766),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1767)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5621),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1767),
    [sym_block_comment] = STATE(1767),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1768)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4807),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1768),
    [sym_block_comment] = STATE(1768),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1769)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4161),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(4249),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3884),
    [sym_line_comment] = STATE(1769),
    [sym_block_comment] = STATE(1769),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5577),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(5579),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(5581),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1770)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4987),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1770),
    [sym_block_comment] = STATE(1770),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1771)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4847),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1771),
    [sym_block_comment] = STATE(1771),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1772)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5694),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1772),
    [sym_block_comment] = STATE(1772),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1773)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5696),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1773),
    [sym_block_comment] = STATE(1773),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1774)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5697),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1774),
    [sym_block_comment] = STATE(1774),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1775)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4202),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1775),
    [sym_block_comment] = STATE(1775),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1776)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3923),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1776),
    [sym_block_comment] = STATE(1776),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1777)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4186),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1777),
    [sym_block_comment] = STATE(1777),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1778)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4189),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1778),
    [sym_block_comment] = STATE(1778),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1779)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4194),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1779),
    [sym_block_comment] = STATE(1779),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1780)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4195),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1780),
    [sym_block_comment] = STATE(1780),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1781)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4203),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1781),
    [sym_block_comment] = STATE(1781),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1782)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4317),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1782),
    [sym_block_comment] = STATE(1782),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1783)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5054),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1783),
    [sym_block_comment] = STATE(1783),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1784)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5712),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1784),
    [sym_block_comment] = STATE(1784),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1785)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5713),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1785),
    [sym_block_comment] = STATE(1785),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1786)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5714),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1786),
    [sym_block_comment] = STATE(1786),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1787)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4459),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1787),
    [sym_block_comment] = STATE(1787),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1788)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4460),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1788),
    [sym_block_comment] = STATE(1788),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1789)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4461),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1789),
    [sym_block_comment] = STATE(1789),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1790)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5720),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1790),
    [sym_block_comment] = STATE(1790),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1791)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4196),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1791),
    [sym_block_comment] = STATE(1791),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1792)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4197),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1792),
    [sym_block_comment] = STATE(1792),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1793)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5734),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1793),
    [sym_block_comment] = STATE(1793),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1794)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3924),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1794),
    [sym_block_comment] = STATE(1794),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1795)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4208),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1795),
    [sym_block_comment] = STATE(1795),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1796)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4211),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1796),
    [sym_block_comment] = STATE(1796),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1797)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4213),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1797),
    [sym_block_comment] = STATE(1797),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1798)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4214),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1798),
    [sym_block_comment] = STATE(1798),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1799)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4468),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1799),
    [sym_block_comment] = STATE(1799),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1800)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5117),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1800),
    [sym_block_comment] = STATE(1800),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1801)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5746),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1801),
    [sym_block_comment] = STATE(1801),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1802)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5747),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1802),
    [sym_block_comment] = STATE(1802),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1803)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5748),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1803),
    [sym_block_comment] = STATE(1803),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1804)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4469),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1804),
    [sym_block_comment] = STATE(1804),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1805)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4470),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1805),
    [sym_block_comment] = STATE(1805),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1806)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4471),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1806),
    [sym_block_comment] = STATE(1806),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1807)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5131),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1807),
    [sym_block_comment] = STATE(1807),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1808)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4476),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1808),
    [sym_block_comment] = STATE(1808),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1809)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3927),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1809),
    [sym_block_comment] = STATE(1809),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1810)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4481),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1810),
    [sym_block_comment] = STATE(1810),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1811)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4482),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1811),
    [sym_block_comment] = STATE(1811),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1812)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4483),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1812),
    [sym_block_comment] = STATE(1812),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1813)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5786),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1813),
    [sym_block_comment] = STATE(1813),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1814)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5788),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1814),
    [sym_block_comment] = STATE(1814),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1815)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5789),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1815),
    [sym_block_comment] = STATE(1815),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1816)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4231),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1816),
    [sym_block_comment] = STATE(1816),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1817)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4233),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1817),
    [sym_block_comment] = STATE(1817),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1818)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5328),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1818),
    [sym_block_comment] = STATE(1818),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1819)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5796),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1819),
    [sym_block_comment] = STATE(1819),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1820)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4491),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1820),
    [sym_block_comment] = STATE(1820),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1821)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5797),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1821),
    [sym_block_comment] = STATE(1821),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1822)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5798),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1822),
    [sym_block_comment] = STATE(1822),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1823)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5799),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1823),
    [sym_block_comment] = STATE(1823),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1824)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4237),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1824),
    [sym_block_comment] = STATE(1824),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1825)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4238),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1825),
    [sym_block_comment] = STATE(1825),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1826)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4241),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1826),
    [sym_block_comment] = STATE(1826),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1827)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4244),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1827),
    [sym_block_comment] = STATE(1827),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1828)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5214),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1828),
    [sym_block_comment] = STATE(1828),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1829)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5810),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1829),
    [sym_block_comment] = STATE(1829),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1830)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5811),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1830),
    [sym_block_comment] = STATE(1830),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1831)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5812),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1831),
    [sym_block_comment] = STATE(1831),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1832)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3930),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1832),
    [sym_block_comment] = STATE(1832),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1833)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4247),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1833),
    [sym_block_comment] = STATE(1833),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1834)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4250),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1834),
    [sym_block_comment] = STATE(1834),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1835)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4251),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1835),
    [sym_block_comment] = STATE(1835),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1836)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4256),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1836),
    [sym_block_comment] = STATE(1836),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1837)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3617),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1837),
    [sym_block_comment] = STATE(1837),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5583),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1838)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5237),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1838),
    [sym_block_comment] = STATE(1838),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1839)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5823),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1839),
    [sym_block_comment] = STATE(1839),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1840)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5443),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1840),
    [sym_block_comment] = STATE(1840),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1841)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5824),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1841),
    [sym_block_comment] = STATE(1841),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1842)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4500),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1842),
    [sym_block_comment] = STATE(1842),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1843)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4501),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1843),
    [sym_block_comment] = STATE(1843),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1844)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4502),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1844),
    [sym_block_comment] = STATE(1844),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1845)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5827),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1845),
    [sym_block_comment] = STATE(1845),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1846)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4258),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1846),
    [sym_block_comment] = STATE(1846),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1847)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4263),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1847),
    [sym_block_comment] = STATE(1847),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1848)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3931),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1848),
    [sym_block_comment] = STATE(1848),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1849)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4507),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1849),
    [sym_block_comment] = STATE(1849),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1850)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4508),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1850),
    [sym_block_comment] = STATE(1850),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1851)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4509),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1851),
    [sym_block_comment] = STATE(1851),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1852)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3933),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1852),
    [sym_block_comment] = STATE(1852),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1853)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4515),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1853),
    [sym_block_comment] = STATE(1853),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1854)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4516),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1854),
    [sym_block_comment] = STATE(1854),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1855)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4517),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1855),
    [sym_block_comment] = STATE(1855),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1856)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5862),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1856),
    [sym_block_comment] = STATE(1856),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1857)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3971),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1857),
    [sym_block_comment] = STATE(1857),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1858)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5870),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1858),
    [sym_block_comment] = STATE(1858),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1859)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4095),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1859),
    [sym_block_comment] = STATE(1859),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1860)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5874),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1860),
    [sym_block_comment] = STATE(1860),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1861)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5875),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1861),
    [sym_block_comment] = STATE(1861),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1862)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5876),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1862),
    [sym_block_comment] = STATE(1862),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1863)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4078),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1863),
    [sym_block_comment] = STATE(1863),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1864)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4079),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1864),
    [sym_block_comment] = STATE(1864),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1865)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5885),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1865),
    [sym_block_comment] = STATE(1865),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1866)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5886),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1866),
    [sym_block_comment] = STATE(1866),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1867)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5887),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1867),
    [sym_block_comment] = STATE(1867),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1868)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4080),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1868),
    [sym_block_comment] = STATE(1868),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1869)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4081),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1869),
    [sym_block_comment] = STATE(1869),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1870)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5894),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1870),
    [sym_block_comment] = STATE(1870),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1871)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4523),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1871),
    [sym_block_comment] = STATE(1871),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1872)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5896),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1872),
    [sym_block_comment] = STATE(1872),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1873)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5897),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1873),
    [sym_block_comment] = STATE(1873),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1874)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5898),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1874),
    [sym_block_comment] = STATE(1874),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1875)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4082),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1875),
    [sym_block_comment] = STATE(1875),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1876)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4083),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1876),
    [sym_block_comment] = STATE(1876),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1877)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4084),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1877),
    [sym_block_comment] = STATE(1877),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1878)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4085),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1878),
    [sym_block_comment] = STATE(1878),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1879)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5332),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1879),
    [sym_block_comment] = STATE(1879),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1880)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3936),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1880),
    [sym_block_comment] = STATE(1880),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1881)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4527),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1881),
    [sym_block_comment] = STATE(1881),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1882)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4528),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1882),
    [sym_block_comment] = STATE(1882),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1883)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4529),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1883),
    [sym_block_comment] = STATE(1883),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1884)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5912),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1884),
    [sym_block_comment] = STATE(1884),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1885)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3937),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1885),
    [sym_block_comment] = STATE(1885),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1886)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4533),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1886),
    [sym_block_comment] = STATE(1886),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1887)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5925),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1887),
    [sym_block_comment] = STATE(1887),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1888)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5930),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1888),
    [sym_block_comment] = STATE(1888),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1889)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5931),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1889),
    [sym_block_comment] = STATE(1889),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1890)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5364),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1890),
    [sym_block_comment] = STATE(1890),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1891)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3938),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1891),
    [sym_block_comment] = STATE(1891),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1892)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5935),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1892),
    [sym_block_comment] = STATE(1892),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1893)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6082),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3598),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6148),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6148),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3852),
    [sym_line_comment] = STATE(1893),
    [sym_block_comment] = STATE(1893),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(5333),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(1722),
    [anon_sym_QMARK] = ACTIONS(1724),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(1728),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(1738),
    [anon_sym_for] = ACTIONS(1740),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(1742),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(1744),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(1746),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1894)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5939),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1894),
    [sym_block_comment] = STATE(1894),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1895)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5942),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1895),
    [sym_block_comment] = STATE(1895),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1896)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5943),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1896),
    [sym_block_comment] = STATE(1896),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1897)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5944),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1897),
    [sym_block_comment] = STATE(1897),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1898)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4089),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1898),
    [sym_block_comment] = STATE(1898),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1899)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4090),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1899),
    [sym_block_comment] = STATE(1899),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1900)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4537),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1900),
    [sym_block_comment] = STATE(1900),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1901)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5954),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1901),
    [sym_block_comment] = STATE(1901),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1902)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5955),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1902),
    [sym_block_comment] = STATE(1902),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1903)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5956),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1903),
    [sym_block_comment] = STATE(1903),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1904)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5404),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1904),
    [sym_block_comment] = STATE(1904),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1905)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3939),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1905),
    [sym_block_comment] = STATE(1905),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1906)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5446),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1906),
    [sym_block_comment] = STATE(1906),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1907)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5447),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1907),
    [sym_block_comment] = STATE(1907),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1908)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5957),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1908),
    [sym_block_comment] = STATE(1908),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1909)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5448),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1909),
    [sym_block_comment] = STATE(1909),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1910)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5451),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1910),
    [sym_block_comment] = STATE(1910),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1911)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5452),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1911),
    [sym_block_comment] = STATE(1911),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1912)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5453),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1912),
    [sym_block_comment] = STATE(1912),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1913)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5455),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1913),
    [sym_block_comment] = STATE(1913),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1914)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4688),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1914),
    [sym_block_comment] = STATE(1914),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1915)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5457),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1915),
    [sym_block_comment] = STATE(1915),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1916)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5833),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1916),
    [sym_block_comment] = STATE(1916),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1917)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5840),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1917),
    [sym_block_comment] = STATE(1917),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1918)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5850),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1918),
    [sym_block_comment] = STATE(1918),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1919)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4891),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1919),
    [sym_block_comment] = STATE(1919),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1920)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5133),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1920),
    [sym_block_comment] = STATE(1920),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1921)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(4489),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1921),
    [sym_block_comment] = STATE(1921),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1922)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5858),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1922),
    [sym_block_comment] = STATE(1922),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1923)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(3957),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1923),
    [sym_block_comment] = STATE(1923),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1924)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5860),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1924),
    [sym_block_comment] = STATE(1924),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1925)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5861),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1925),
    [sym_block_comment] = STATE(1925),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1926)] = {
    [sym_primitive_type] = STATE(5787),
    [sym_function_modifiers] = STATE(6439),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_higher_ranked_trait_bound] = STATE(3643),
    [sym_removed_trait_bound] = STATE(3643),
    [sym_extern_modifier] = STATE(6118),
    [sym__type] = STATE(5141),
    [sym_bracketed_type] = STATE(6234),
    [sym_lifetime] = STATE(6258),
    [sym_array_type] = STATE(3643),
    [sym_function_type] = STATE(3643),
    [sym_unsafe_bound_type] = STATE(3643),
    [sym_tuple_type] = STATE(3643),
    [sym_unit_type] = STATE(3643),
    [sym_generic_type] = STATE(3586),
    [sym_generic_type_with_turbofish] = STATE(3567),
    [sym_bounded_type] = STATE(3643),
    [sym_use_bounds] = STATE(6258),
    [sym_reference_type] = STATE(3643),
    [sym_pointer_type] = STATE(3643),
    [sym_never_type] = STATE(3643),
    [sym_abstract_type] = STATE(3643),
    [sym_dynamic_type] = STATE(3643),
    [sym_macro_invocation] = STATE(3643),
    [sym_scoped_identifier] = STATE(5816),
    [sym_scoped_type_identifier] = STATE(3550),
    [sym_line_comment] = STATE(1926),
    [sym_block_comment] = STATE(1926),
    [sym_metavariable] = STATE(3583),
    [sym_identifier] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(1718),
    [anon_sym_BANG] = ACTIONS(915),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(919),
    [anon_sym_QMARK] = ACTIONS(921),
    [anon_sym_u8] = ACTIONS(1726),
    [anon_sym_i8] = ACTIONS(1726),
    [anon_sym_u16] = ACTIONS(1726),
    [anon_sym_i16] = ACTIONS(1726),
    [anon_sym_u32] = ACTIONS(1726),
    [anon_sym_i32] = ACTIONS(1726),
    [anon_sym_u64] = ACTIONS(1726),
    [anon_sym_i64] = ACTIONS(1726),
    [anon_sym_u128] = ACTIONS(1726),
    [anon_sym_i128] = ACTIONS(1726),
    [anon_sym_isize] = ACTIONS(1726),
    [anon_sym_usize] = ACTIONS(1726),
    [anon_sym_f32] = ACTIONS(1726),
    [anon_sym_f64] = ACTIONS(1726),
    [anon_sym_bool] = ACTIONS(1726),
    [anon_sym_str] = ACTIONS(1726),
    [anon_sym_char] = ACTIONS(1726),
    [anon_sym_AMP] = ACTIONS(2097),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(2101),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(1718),
    [anon_sym_await] = ACTIONS(1718),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(1736),
    [anon_sym_fn] = ACTIONS(951),
    [anon_sym_for] = ACTIONS(953),
    [anon_sym_gen] = ACTIONS(1736),
    [anon_sym_impl] = ACTIONS(955),
    [anon_sym_raw] = ACTIONS(1718),
    [anon_sym_union] = ACTIONS(1718),
    [anon_sym_unsafe] = ACTIONS(957),
    [anon_sym_use] = ACTIONS(959),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(1736),
    [anon_sym_dyn] = ACTIONS(965),
    [anon_sym_try] = ACTIONS(1718),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1750),
    [sym_super] = ACTIONS(1750),
    [sym_crate] = ACTIONS(1750),
    [aux_sym_metavariable_token1] = ACTIONS(981),
  },
  [STATE(1927)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_line_comment] = STATE(1927),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(1927),
    [aux_sym_attributes_repeat1] = STATE(1928),
    [sym_identifier] = ACTIONS(2151),
    [anon_sym_LPAREN] = ACTIONS(2153),
    [anon_sym_LBRACE] = ACTIONS(2153),
    [anon_sym_COMMA] = ACTIONS(2153),
    [anon_sym_macro_rules] = ACTIONS(2151),
    [anon_sym_BANG] = ACTIONS(2153),
    [anon_sym_LBRACK] = ACTIONS(2153),
    [anon_sym_RBRACK] = ACTIONS(2153),
    [anon_sym_STAR] = ACTIONS(2153),
    [anon_sym_u8] = ACTIONS(2151),
    [anon_sym_i8] = ACTIONS(2151),
    [anon_sym_u16] = ACTIONS(2151),
    [anon_sym_i16] = ACTIONS(2151),
    [anon_sym_u32] = ACTIONS(2151),
    [anon_sym_i32] = ACTIONS(2151),
    [anon_sym_u64] = ACTIONS(2151),
    [anon_sym_i64] = ACTIONS(2151),
    [anon_sym_u128] = ACTIONS(2151),
    [anon_sym_i128] = ACTIONS(2151),
    [anon_sym_isize] = ACTIONS(2151),
    [anon_sym_usize] = ACTIONS(2151),
    [anon_sym_f32] = ACTIONS(2151),
    [anon_sym_f64] = ACTIONS(2151),
    [anon_sym_bool] = ACTIONS(2151),
    [anon_sym_str] = ACTIONS(2151),
    [anon_sym_char] = ACTIONS(2151),
    [anon_sym_DASH] = ACTIONS(2153),
    [anon_sym_AMP] = ACTIONS(2153),
    [anon_sym_PIPE] = ACTIONS(2153),
    [anon_sym_LT] = ACTIONS(2153),
    [anon_sym_DOT_DOT] = ACTIONS(2153),
    [anon_sym_COLON_COLON] = ACTIONS(2153),
    [anon_sym_POUND] = ACTIONS(396),
    [anon_sym_SQUOTE] = ACTIONS(2151),
    [anon_sym_async] = ACTIONS(2151),
    [anon_sym_auto] = ACTIONS(2151),
    [anon_sym_await] = ACTIONS(2151),
    [anon_sym_become] = ACTIONS(2151),
    [anon_sym_break] = ACTIONS(2151),
    [anon_sym_const] = ACTIONS(2151),
    [anon_sym_continue] = ACTIONS(2151),
    [anon_sym_default] = ACTIONS(2151),
    [anon_sym_for] = ACTIONS(2151),
    [anon_sym_gen] = ACTIONS(2151),
    [anon_sym_if] = ACTIONS(2151),
    [anon_sym_loop] = ACTIONS(2151),
    [anon_sym_match] = ACTIONS(2151),
    [anon_sym_return] = ACTIONS(2151),
    [anon_sym_raw] = ACTIONS(2151),
    [anon_sym_static] = ACTIONS(2151),
    [anon_sym_union] = ACTIONS(2151),
    [anon_sym_unsafe] = ACTIONS(2151),
    [anon_sym_while] = ACTIONS(2151),
    [anon_sym_safe] = ACTIONS(2151),
    [anon_sym_yield] = ACTIONS(2151),
    [anon_sym_move] = ACTIONS(2151),
    [anon_sym_try] = ACTIONS(2151),
    [sym_integer_literal] = ACTIONS(2153),
    [aux_sym_string_literal_token1] = ACTIONS(2153),
    [sym_char_literal] = ACTIONS(2153),
    [anon_sym_true] = ACTIONS(2151),
    [anon_sym_false] = ACTIONS(2151),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2151),
    [sym_super] = ACTIONS(2151),
    [sym_crate] = ACTIONS(2151),
    [aux_sym_metavariable_token1] = ACTIONS(2153),
    [sym__raw_string_literal_start] = ACTIONS(2153),
    [sym_float_literal] = ACTIONS(2153),
    [sym__outer_block_doc_comment_start] = ACTIONS(420),
    [sym__outer_line_doc_comment_start] = ACTIONS(422),
  },
  [STATE(1928)] = {
    [sym_attribute_item] = STATE(1935),
    [sym_line_comment] = STATE(1928),
    [sym_line_outer_doc_comment] = STATE(1935),
    [sym_block_outer_doc_comment] = STATE(1935),
    [sym_block_comment] = STATE(1928),
    [aux_sym_attributes_repeat1] = STATE(1928),
    [sym_identifier] = ACTIONS(2157),
    [anon_sym_LPAREN] = ACTIONS(2159),
    [anon_sym_LBRACE] = ACTIONS(2159),
    [anon_sym_COMMA] = ACTIONS(2159),
    [anon_sym_macro_rules] = ACTIONS(2157),
    [anon_sym_BANG] = ACTIONS(2159),
    [anon_sym_LBRACK] = ACTIONS(2159),
    [anon_sym_RBRACK] = ACTIONS(2159),
    [anon_sym_STAR] = ACTIONS(2159),
    [anon_sym_u8] = ACTIONS(2157),
    [anon_sym_i8] = ACTIONS(2157),
    [anon_sym_u16] = ACTIONS(2157),
    [anon_sym_i16] = ACTIONS(2157),
    [anon_sym_u32] = ACTIONS(2157),
    [anon_sym_i32] = ACTIONS(2157),
    [anon_sym_u64] = ACTIONS(2157),
    [anon_sym_i64] = ACTIONS(2157),
    [anon_sym_u128] = ACTIONS(2157),
    [anon_sym_i128] = ACTIONS(2157),
    [anon_sym_isize] = ACTIONS(2157),
    [anon_sym_usize] = ACTIONS(2157),
    [anon_sym_f32] = ACTIONS(2157),
    [anon_sym_f64] = ACTIONS(2157),
    [anon_sym_bool] = ACTIONS(2157),
    [anon_sym_str] = ACTIONS(2157),
    [anon_sym_char] = ACTIONS(2157),
    [anon_sym_DASH] = ACTIONS(2159),
    [anon_sym_AMP] = ACTIONS(2159),
    [anon_sym_PIPE] = ACTIONS(2159),
    [anon_sym_LT] = ACTIONS(2159),
    [anon_sym_DOT_DOT] = ACTIONS(2159),
    [anon_sym_COLON_COLON] = ACTIONS(2159),
    [anon_sym_POUND] = ACTIONS(5585),
    [anon_sym_SQUOTE] = ACTIONS(2157),
    [anon_sym_async] = ACTIONS(2157),
    [anon_sym_auto] = ACTIONS(2157),
    [anon_sym_await] = ACTIONS(2157),
    [anon_sym_become] = ACTIONS(2157),
    [anon_sym_break] = ACTIONS(2157),
    [anon_sym_const] = ACTIONS(2157),
    [anon_sym_continue] = ACTIONS(2157),
    [anon_sym_default] = ACTIONS(2157),
    [anon_sym_for] = ACTIONS(2157),
    [anon_sym_gen] = ACTIONS(2157),
    [anon_sym_if] = ACTIONS(2157),
    [anon_sym_loop] = ACTIONS(2157),
    [anon_sym_match] = ACTIONS(2157),
    [anon_sym_return] = ACTIONS(2157),
    [anon_sym_raw] = ACTIONS(2157),
    [anon_sym_static] = ACTIONS(2157),
    [anon_sym_union] = ACTIONS(2157),
    [anon_sym_unsafe] = ACTIONS(2157),
    [anon_sym_while] = ACTIONS(2157),
    [anon_sym_safe] = ACTIONS(2157),
    [anon_sym_yield] = ACTIONS(2157),
    [anon_sym_move] = ACTIONS(2157),
    [anon_sym_try] = ACTIONS(2157),
    [sym_integer_literal] = ACTIONS(2159),
    [aux_sym_string_literal_token1] = ACTIONS(2159),
    [sym_char_literal] = ACTIONS(2159),
    [anon_sym_true] = ACTIONS(2157),
    [anon_sym_false] = ACTIONS(2157),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2157),
    [sym_super] = ACTIONS(2157),
    [sym_crate] = ACTIONS(2157),
    [aux_sym_metavariable_token1] = ACTIONS(2159),
    [sym__raw_string_literal_start] = ACTIONS(2159),
    [sym_float_literal] = ACTIONS(2159),
    [sym__outer_block_doc_comment_start] = ACTIONS(5588),
    [sym__outer_line_doc_comment_start] = ACTIONS(5591),
  },
  [STATE(1929)] = {
    [sym_line_comment] = STATE(1929),
    [sym_block_comment] = STATE(1929),
    [sym_identifier] = ACTIONS(1654),
    [anon_sym_LPAREN] = ACTIONS(1656),
    [anon_sym_LBRACE] = ACTIONS(1656),
    [anon_sym_macro_rules] = ACTIONS(1654),
    [anon_sym_BANG] = ACTIONS(1656),
    [anon_sym_LBRACK] = ACTIONS(1656),
    [anon_sym_STAR] = ACTIONS(1656),
    [anon_sym_u8] = ACTIONS(1654),
    [anon_sym_i8] = ACTIONS(1654),
    [anon_sym_u16] = ACTIONS(1654),
    [anon_sym_i16] = ACTIONS(1654),
    [anon_sym_u32] = ACTIONS(1654),
    [anon_sym_i32] = ACTIONS(1654),
    [anon_sym_u64] = ACTIONS(1654),
    [anon_sym_i64] = ACTIONS(1654),
    [anon_sym_u128] = ACTIONS(1654),
    [anon_sym_i128] = ACTIONS(1654),
    [anon_sym_isize] = ACTIONS(1654),
    [anon_sym_usize] = ACTIONS(1654),
    [anon_sym_f32] = ACTIONS(1654),
    [anon_sym_f64] = ACTIONS(1654),
    [anon_sym_bool] = ACTIONS(1654),
    [anon_sym_str] = ACTIONS(1654),
    [anon_sym_char] = ACTIONS(1654),
    [anon_sym_DASH] = ACTIONS(1654),
    [anon_sym_AMP] = ACTIONS(1656),
    [anon_sym_PIPE] = ACTIONS(1656),
    [anon_sym_LT] = ACTIONS(1656),
    [anon_sym__] = ACTIONS(1654),
    [anon_sym_DOT_DOT] = ACTIONS(1656),
    [anon_sym_COLON_COLON] = ACTIONS(1656),
    [anon_sym_DASH_GT] = ACTIONS(1656),
    [anon_sym_POUND] = ACTIONS(1656),
    [anon_sym_SQUOTE] = ACTIONS(1654),
    [anon_sym_async] = ACTIONS(1654),
    [anon_sym_auto] = ACTIONS(1654),
    [anon_sym_await] = ACTIONS(1654),
    [anon_sym_become] = ACTIONS(1654),
    [anon_sym_break] = ACTIONS(1654),
    [anon_sym_const] = ACTIONS(1654),
    [anon_sym_continue] = ACTIONS(1654),
    [anon_sym_default] = ACTIONS(1654),
    [anon_sym_for] = ACTIONS(1654),
    [anon_sym_gen] = ACTIONS(1654),
    [anon_sym_if] = ACTIONS(1654),
    [anon_sym_loop] = ACTIONS(1654),
    [anon_sym_match] = ACTIONS(1654),
    [anon_sym_return] = ACTIONS(1654),
    [anon_sym_raw] = ACTIONS(1654),
    [anon_sym_static] = ACTIONS(1654),
    [anon_sym_union] = ACTIONS(1654),
    [anon_sym_unsafe] = ACTIONS(1654),
    [anon_sym_while] = ACTIONS(1654),
    [anon_sym_safe] = ACTIONS(1654),
    [anon_sym_yield] = ACTIONS(1654),
    [anon_sym_move] = ACTIONS(1654),
    [anon_sym_try] = ACTIONS(1654),
    [sym_integer_literal] = ACTIONS(1656),
    [aux_sym_string_literal_token1] = ACTIONS(1656),
    [sym_char_literal] = ACTIONS(1656),
    [anon_sym_true] = ACTIONS(1654),
    [anon_sym_false] = ACTIONS(1654),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1654),
    [sym_super] = ACTIONS(1654),
    [sym_crate] = ACTIONS(1654),
    [aux_sym_metavariable_token1] = ACTIONS(1656),
    [sym__raw_string_literal_start] = ACTIONS(1656),
    [sym_float_literal] = ACTIONS(1656),
    [sym__outer_block_doc_comment_start] = ACTIONS(1656),
    [sym__outer_line_doc_comment_start] = ACTIONS(1656),
  },
  [STATE(1930)] = {
    [sym_line_comment] = STATE(1930),
    [sym_block_comment] = STATE(1930),
    [sym_identifier] = ACTIONS(5236),
    [anon_sym_LPAREN] = ACTIONS(5238),
    [anon_sym_LBRACE] = ACTIONS(5238),
    [anon_sym_COMMA] = ACTIONS(5238),
    [anon_sym_macro_rules] = ACTIONS(5236),
    [anon_sym_BANG] = ACTIONS(5238),
    [anon_sym_LBRACK] = ACTIONS(5238),
    [anon_sym_RBRACK] = ACTIONS(5238),
    [anon_sym_STAR] = ACTIONS(5238),
    [anon_sym_u8] = ACTIONS(5236),
    [anon_sym_i8] = ACTIONS(5236),
    [anon_sym_u16] = ACTIONS(5236),
    [anon_sym_i16] = ACTIONS(5236),
    [anon_sym_u32] = ACTIONS(5236),
    [anon_sym_i32] = ACTIONS(5236),
    [anon_sym_u64] = ACTIONS(5236),
    [anon_sym_i64] = ACTIONS(5236),
    [anon_sym_u128] = ACTIONS(5236),
    [anon_sym_i128] = ACTIONS(5236),
    [anon_sym_isize] = ACTIONS(5236),
    [anon_sym_usize] = ACTIONS(5236),
    [anon_sym_f32] = ACTIONS(5236),
    [anon_sym_f64] = ACTIONS(5236),
    [anon_sym_bool] = ACTIONS(5236),
    [anon_sym_str] = ACTIONS(5236),
    [anon_sym_char] = ACTIONS(5236),
    [anon_sym_DASH] = ACTIONS(5238),
    [anon_sym_AMP] = ACTIONS(5238),
    [anon_sym_PIPE] = ACTIONS(5238),
    [anon_sym_LT] = ACTIONS(5238),
    [anon_sym_DOT_DOT] = ACTIONS(5238),
    [anon_sym_COLON_COLON] = ACTIONS(5238),
    [anon_sym_POUND] = ACTIONS(5238),
    [anon_sym_SQUOTE] = ACTIONS(5236),
    [anon_sym_async] = ACTIONS(5236),
    [anon_sym_auto] = ACTIONS(5236),
    [anon_sym_await] = ACTIONS(5236),
    [anon_sym_become] = ACTIONS(5236),
    [anon_sym_break] = ACTIONS(5236),
    [anon_sym_const] = ACTIONS(5236),
    [anon_sym_continue] = ACTIONS(5236),
    [anon_sym_default] = ACTIONS(5236),
    [anon_sym_for] = ACTIONS(5236),
    [anon_sym_gen] = ACTIONS(5236),
    [anon_sym_if] = ACTIONS(5236),
    [anon_sym_loop] = ACTIONS(5236),
    [anon_sym_match] = ACTIONS(5236),
    [anon_sym_return] = ACTIONS(5236),
    [anon_sym_raw] = ACTIONS(5236),
    [anon_sym_static] = ACTIONS(5236),
    [anon_sym_union] = ACTIONS(5236),
    [anon_sym_unsafe] = ACTIONS(5236),
    [anon_sym_while] = ACTIONS(5236),
    [anon_sym_safe] = ACTIONS(5236),
    [anon_sym_yield] = ACTIONS(5236),
    [anon_sym_move] = ACTIONS(5236),
    [anon_sym_try] = ACTIONS(5236),
    [sym_integer_literal] = ACTIONS(5238),
    [aux_sym_string_literal_token1] = ACTIONS(5238),
    [sym_char_literal] = ACTIONS(5238),
    [anon_sym_true] = ACTIONS(5236),
    [anon_sym_false] = ACTIONS(5236),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5236),
    [sym_super] = ACTIONS(5236),
    [sym_crate] = ACTIONS(5236),
    [aux_sym_metavariable_token1] = ACTIONS(5238),
    [sym__raw_string_literal_start] = ACTIONS(5238),
    [sym_float_literal] = ACTIONS(5238),
    [sym__outer_block_doc_comment_start] = ACTIONS(5238),
    [sym__outer_line_doc_comment_start] = ACTIONS(5238),
  },
  [STATE(1931)] = {
    [sym_line_comment] = STATE(1931),
    [sym_block_comment] = STATE(1931),
    [sym_identifier] = ACTIONS(5216),
    [anon_sym_LPAREN] = ACTIONS(5218),
    [anon_sym_LBRACE] = ACTIONS(5218),
    [anon_sym_COMMA] = ACTIONS(5218),
    [anon_sym_macro_rules] = ACTIONS(5216),
    [anon_sym_BANG] = ACTIONS(5218),
    [anon_sym_LBRACK] = ACTIONS(5218),
    [anon_sym_RBRACK] = ACTIONS(5218),
    [anon_sym_STAR] = ACTIONS(5218),
    [anon_sym_u8] = ACTIONS(5216),
    [anon_sym_i8] = ACTIONS(5216),
    [anon_sym_u16] = ACTIONS(5216),
    [anon_sym_i16] = ACTIONS(5216),
    [anon_sym_u32] = ACTIONS(5216),
    [anon_sym_i32] = ACTIONS(5216),
    [anon_sym_u64] = ACTIONS(5216),
    [anon_sym_i64] = ACTIONS(5216),
    [anon_sym_u128] = ACTIONS(5216),
    [anon_sym_i128] = ACTIONS(5216),
    [anon_sym_isize] = ACTIONS(5216),
    [anon_sym_usize] = ACTIONS(5216),
    [anon_sym_f32] = ACTIONS(5216),
    [anon_sym_f64] = ACTIONS(5216),
    [anon_sym_bool] = ACTIONS(5216),
    [anon_sym_str] = ACTIONS(5216),
    [anon_sym_char] = ACTIONS(5216),
    [anon_sym_DASH] = ACTIONS(5218),
    [anon_sym_AMP] = ACTIONS(5218),
    [anon_sym_PIPE] = ACTIONS(5218),
    [anon_sym_LT] = ACTIONS(5218),
    [anon_sym_DOT_DOT] = ACTIONS(5218),
    [anon_sym_COLON_COLON] = ACTIONS(5218),
    [anon_sym_POUND] = ACTIONS(5218),
    [anon_sym_SQUOTE] = ACTIONS(5216),
    [anon_sym_async] = ACTIONS(5216),
    [anon_sym_auto] = ACTIONS(5216),
    [anon_sym_await] = ACTIONS(5216),
    [anon_sym_become] = ACTIONS(5216),
    [anon_sym_break] = ACTIONS(5216),
    [anon_sym_const] = ACTIONS(5216),
    [anon_sym_continue] = ACTIONS(5216),
    [anon_sym_default] = ACTIONS(5216),
    [anon_sym_for] = ACTIONS(5216),
    [anon_sym_gen] = ACTIONS(5216),
    [anon_sym_if] = ACTIONS(5216),
    [anon_sym_loop] = ACTIONS(5216),
    [anon_sym_match] = ACTIONS(5216),
    [anon_sym_return] = ACTIONS(5216),
    [anon_sym_raw] = ACTIONS(5216),
    [anon_sym_static] = ACTIONS(5216),
    [anon_sym_union] = ACTIONS(5216),
    [anon_sym_unsafe] = ACTIONS(5216),
    [anon_sym_while] = ACTIONS(5216),
    [anon_sym_safe] = ACTIONS(5216),
    [anon_sym_yield] = ACTIONS(5216),
    [anon_sym_move] = ACTIONS(5216),
    [anon_sym_try] = ACTIONS(5216),
    [sym_integer_literal] = ACTIONS(5218),
    [aux_sym_string_literal_token1] = ACTIONS(5218),
    [sym_char_literal] = ACTIONS(5218),
    [anon_sym_true] = ACTIONS(5216),
    [anon_sym_false] = ACTIONS(5216),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5216),
    [sym_super] = ACTIONS(5216),
    [sym_crate] = ACTIONS(5216),
    [aux_sym_metavariable_token1] = ACTIONS(5218),
    [sym__raw_string_literal_start] = ACTIONS(5218),
    [sym_float_literal] = ACTIONS(5218),
    [sym__outer_block_doc_comment_start] = ACTIONS(5218),
    [sym__outer_line_doc_comment_start] = ACTIONS(5218),
  },
  [STATE(1932)] = {
    [sym_line_comment] = STATE(1932),
    [sym_block_comment] = STATE(1932),
    [sym_identifier] = ACTIONS(1646),
    [anon_sym_LPAREN] = ACTIONS(1648),
    [anon_sym_LBRACE] = ACTIONS(1648),
    [anon_sym_macro_rules] = ACTIONS(1646),
    [anon_sym_BANG] = ACTIONS(1648),
    [anon_sym_LBRACK] = ACTIONS(1648),
    [anon_sym_STAR] = ACTIONS(1648),
    [anon_sym_u8] = ACTIONS(1646),
    [anon_sym_i8] = ACTIONS(1646),
    [anon_sym_u16] = ACTIONS(1646),
    [anon_sym_i16] = ACTIONS(1646),
    [anon_sym_u32] = ACTIONS(1646),
    [anon_sym_i32] = ACTIONS(1646),
    [anon_sym_u64] = ACTIONS(1646),
    [anon_sym_i64] = ACTIONS(1646),
    [anon_sym_u128] = ACTIONS(1646),
    [anon_sym_i128] = ACTIONS(1646),
    [anon_sym_isize] = ACTIONS(1646),
    [anon_sym_usize] = ACTIONS(1646),
    [anon_sym_f32] = ACTIONS(1646),
    [anon_sym_f64] = ACTIONS(1646),
    [anon_sym_bool] = ACTIONS(1646),
    [anon_sym_str] = ACTIONS(1646),
    [anon_sym_char] = ACTIONS(1646),
    [anon_sym_DASH] = ACTIONS(1646),
    [anon_sym_AMP] = ACTIONS(1648),
    [anon_sym_PIPE] = ACTIONS(1648),
    [anon_sym_LT] = ACTIONS(1648),
    [anon_sym__] = ACTIONS(1646),
    [anon_sym_DOT_DOT] = ACTIONS(1648),
    [anon_sym_COLON_COLON] = ACTIONS(1648),
    [anon_sym_DASH_GT] = ACTIONS(1648),
    [anon_sym_POUND] = ACTIONS(1648),
    [anon_sym_SQUOTE] = ACTIONS(1646),
    [anon_sym_async] = ACTIONS(1646),
    [anon_sym_auto] = ACTIONS(1646),
    [anon_sym_await] = ACTIONS(1646),
    [anon_sym_become] = ACTIONS(1646),
    [anon_sym_break] = ACTIONS(1646),
    [anon_sym_const] = ACTIONS(1646),
    [anon_sym_continue] = ACTIONS(1646),
    [anon_sym_default] = ACTIONS(1646),
    [anon_sym_for] = ACTIONS(1646),
    [anon_sym_gen] = ACTIONS(1646),
    [anon_sym_if] = ACTIONS(1646),
    [anon_sym_loop] = ACTIONS(1646),
    [anon_sym_match] = ACTIONS(1646),
    [anon_sym_return] = ACTIONS(1646),
    [anon_sym_raw] = ACTIONS(1646),
    [anon_sym_static] = ACTIONS(1646),
    [anon_sym_union] = ACTIONS(1646),
    [anon_sym_unsafe] = ACTIONS(1646),
    [anon_sym_while] = ACTIONS(1646),
    [anon_sym_safe] = ACTIONS(1646),
    [anon_sym_yield] = ACTIONS(1646),
    [anon_sym_move] = ACTIONS(1646),
    [anon_sym_try] = ACTIONS(1646),
    [sym_integer_literal] = ACTIONS(1648),
    [aux_sym_string_literal_token1] = ACTIONS(1648),
    [sym_char_literal] = ACTIONS(1648),
    [anon_sym_true] = ACTIONS(1646),
    [anon_sym_false] = ACTIONS(1646),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(1646),
    [sym_super] = ACTIONS(1646),
    [sym_crate] = ACTIONS(1646),
    [aux_sym_metavariable_token1] = ACTIONS(1648),
    [sym__raw_string_literal_start] = ACTIONS(1648),
    [sym_float_literal] = ACTIONS(1648),
    [sym__outer_block_doc_comment_start] = ACTIONS(1648),
    [sym__outer_line_doc_comment_start] = ACTIONS(1648),
  },
  [STATE(1933)] = {
    [sym_line_comment] = STATE(1933),
    [sym_block_comment] = STATE(1933),
    [sym_identifier] = ACTIONS(5220),
    [anon_sym_LPAREN] = ACTIONS(5222),
    [anon_sym_LBRACE] = ACTIONS(5222),
    [anon_sym_COMMA] = ACTIONS(5222),
    [anon_sym_macro_rules] = ACTIONS(5220),
    [anon_sym_BANG] = ACTIONS(5222),
    [anon_sym_LBRACK] = ACTIONS(5222),
    [anon_sym_RBRACK] = ACTIONS(5222),
    [anon_sym_STAR] = ACTIONS(5222),
    [anon_sym_u8] = ACTIONS(5220),
    [anon_sym_i8] = ACTIONS(5220),
    [anon_sym_u16] = ACTIONS(5220),
    [anon_sym_i16] = ACTIONS(5220),
    [anon_sym_u32] = ACTIONS(5220),
    [anon_sym_i32] = ACTIONS(5220),
    [anon_sym_u64] = ACTIONS(5220),
    [anon_sym_i64] = ACTIONS(5220),
    [anon_sym_u128] = ACTIONS(5220),
    [anon_sym_i128] = ACTIONS(5220),
    [anon_sym_isize] = ACTIONS(5220),
    [anon_sym_usize] = ACTIONS(5220),
    [anon_sym_f32] = ACTIONS(5220),
    [anon_sym_f64] = ACTIONS(5220),
    [anon_sym_bool] = ACTIONS(5220),
    [anon_sym_str] = ACTIONS(5220),
    [anon_sym_char] = ACTIONS(5220),
    [anon_sym_DASH] = ACTIONS(5222),
    [anon_sym_AMP] = ACTIONS(5222),
    [anon_sym_PIPE] = ACTIONS(5222),
    [anon_sym_LT] = ACTIONS(5222),
    [anon_sym_DOT_DOT] = ACTIONS(5222),
    [anon_sym_COLON_COLON] = ACTIONS(5222),
    [anon_sym_POUND] = ACTIONS(5222),
    [anon_sym_SQUOTE] = ACTIONS(5220),
    [anon_sym_async] = ACTIONS(5220),
    [anon_sym_auto] = ACTIONS(5220),
    [anon_sym_await] = ACTIONS(5220),
    [anon_sym_become] = ACTIONS(5220),
    [anon_sym_break] = ACTIONS(5220),
    [anon_sym_const] = ACTIONS(5220),
    [anon_sym_continue] = ACTIONS(5220),
    [anon_sym_default] = ACTIONS(5220),
    [anon_sym_for] = ACTIONS(5220),
    [anon_sym_gen] = ACTIONS(5220),
    [anon_sym_if] = ACTIONS(5220),
    [anon_sym_loop] = ACTIONS(5220),
    [anon_sym_match] = ACTIONS(5220),
    [anon_sym_return] = ACTIONS(5220),
    [anon_sym_raw] = ACTIONS(5220),
    [anon_sym_static] = ACTIONS(5220),
    [anon_sym_union] = ACTIONS(5220),
    [anon_sym_unsafe] = ACTIONS(5220),
    [anon_sym_while] = ACTIONS(5220),
    [anon_sym_safe] = ACTIONS(5220),
    [anon_sym_yield] = ACTIONS(5220),
    [anon_sym_move] = ACTIONS(5220),
    [anon_sym_try] = ACTIONS(5220),
    [sym_integer_literal] = ACTIONS(5222),
    [aux_sym_string_literal_token1] = ACTIONS(5222),
    [sym_char_literal] = ACTIONS(5222),
    [anon_sym_true] = ACTIONS(5220),
    [anon_sym_false] = ACTIONS(5220),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5220),
    [sym_super] = ACTIONS(5220),
    [sym_crate] = ACTIONS(5220),
    [aux_sym_metavariable_token1] = ACTIONS(5222),
    [sym__raw_string_literal_start] = ACTIONS(5222),
    [sym_float_literal] = ACTIONS(5222),
    [sym__outer_block_doc_comment_start] = ACTIONS(5222),
    [sym__outer_line_doc_comment_start] = ACTIONS(5222),
  },
  [STATE(1934)] = {
    [sym_line_comment] = STATE(1934),
    [sym_block_comment] = STATE(1934),
    [sym_identifier] = ACTIONS(5232),
    [anon_sym_LPAREN] = ACTIONS(5234),
    [anon_sym_LBRACE] = ACTIONS(5234),
    [anon_sym_COMMA] = ACTIONS(5234),
    [anon_sym_macro_rules] = ACTIONS(5232),
    [anon_sym_BANG] = ACTIONS(5234),
    [anon_sym_LBRACK] = ACTIONS(5234),
    [anon_sym_RBRACK] = ACTIONS(5234),
    [anon_sym_STAR] = ACTIONS(5234),
    [anon_sym_u8] = ACTIONS(5232),
    [anon_sym_i8] = ACTIONS(5232),
    [anon_sym_u16] = ACTIONS(5232),
    [anon_sym_i16] = ACTIONS(5232),
    [anon_sym_u32] = ACTIONS(5232),
    [anon_sym_i32] = ACTIONS(5232),
    [anon_sym_u64] = ACTIONS(5232),
    [anon_sym_i64] = ACTIONS(5232),
    [anon_sym_u128] = ACTIONS(5232),
    [anon_sym_i128] = ACTIONS(5232),
    [anon_sym_isize] = ACTIONS(5232),
    [anon_sym_usize] = ACTIONS(5232),
    [anon_sym_f32] = ACTIONS(5232),
    [anon_sym_f64] = ACTIONS(5232),
    [anon_sym_bool] = ACTIONS(5232),
    [anon_sym_str] = ACTIONS(5232),
    [anon_sym_char] = ACTIONS(5232),
    [anon_sym_DASH] = ACTIONS(5234),
    [anon_sym_AMP] = ACTIONS(5234),
    [anon_sym_PIPE] = ACTIONS(5234),
    [anon_sym_LT] = ACTIONS(5234),
    [anon_sym_DOT_DOT] = ACTIONS(5234),
    [anon_sym_COLON_COLON] = ACTIONS(5234),
    [anon_sym_POUND] = ACTIONS(5234),
    [anon_sym_SQUOTE] = ACTIONS(5232),
    [anon_sym_async] = ACTIONS(5232),
    [anon_sym_auto] = ACTIONS(5232),
    [anon_sym_await] = ACTIONS(5232),
    [anon_sym_become] = ACTIONS(5232),
    [anon_sym_break] = ACTIONS(5232),
    [anon_sym_const] = ACTIONS(5232),
    [anon_sym_continue] = ACTIONS(5232),
    [anon_sym_default] = ACTIONS(5232),
    [anon_sym_for] = ACTIONS(5232),
    [anon_sym_gen] = ACTIONS(5232),
    [anon_sym_if] = ACTIONS(5232),
    [anon_sym_loop] = ACTIONS(5232),
    [anon_sym_match] = ACTIONS(5232),
    [anon_sym_return] = ACTIONS(5232),
    [anon_sym_raw] = ACTIONS(5232),
    [anon_sym_static] = ACTIONS(5232),
    [anon_sym_union] = ACTIONS(5232),
    [anon_sym_unsafe] = ACTIONS(5232),
    [anon_sym_while] = ACTIONS(5232),
    [anon_sym_safe] = ACTIONS(5232),
    [anon_sym_yield] = ACTIONS(5232),
    [anon_sym_move] = ACTIONS(5232),
    [anon_sym_try] = ACTIONS(5232),
    [sym_integer_literal] = ACTIONS(5234),
    [aux_sym_string_literal_token1] = ACTIONS(5234),
    [sym_char_literal] = ACTIONS(5234),
    [anon_sym_true] = ACTIONS(5232),
    [anon_sym_false] = ACTIONS(5232),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5232),
    [sym_super] = ACTIONS(5232),
    [sym_crate] = ACTIONS(5232),
    [aux_sym_metavariable_token1] = ACTIONS(5234),
    [sym__raw_string_literal_start] = ACTIONS(5234),
    [sym_float_literal] = ACTIONS(5234),
    [sym__outer_block_doc_comment_start] = ACTIONS(5234),
    [sym__outer_line_doc_comment_start] = ACTIONS(5234),
  },
  [STATE(1935)] = {
    [sym_line_comment] = STATE(1935),
    [sym_block_comment] = STATE(1935),
    [sym_identifier] = ACTIONS(5228),
    [anon_sym_LPAREN] = ACTIONS(5230),
    [anon_sym_LBRACE] = ACTIONS(5230),
    [anon_sym_COMMA] = ACTIONS(5230),
    [anon_sym_macro_rules] = ACTIONS(5228),
    [anon_sym_BANG] = ACTIONS(5230),
    [anon_sym_LBRACK] = ACTIONS(5230),
    [anon_sym_RBRACK] = ACTIONS(5230),
    [anon_sym_STAR] = ACTIONS(5230),
    [anon_sym_u8] = ACTIONS(5228),
    [anon_sym_i8] = ACTIONS(5228),
    [anon_sym_u16] = ACTIONS(5228),
    [anon_sym_i16] = ACTIONS(5228),
    [anon_sym_u32] = ACTIONS(5228),
    [anon_sym_i32] = ACTIONS(5228),
    [anon_sym_u64] = ACTIONS(5228),
    [anon_sym_i64] = ACTIONS(5228),
    [anon_sym_u128] = ACTIONS(5228),
    [anon_sym_i128] = ACTIONS(5228),
    [anon_sym_isize] = ACTIONS(5228),
    [anon_sym_usize] = ACTIONS(5228),
    [anon_sym_f32] = ACTIONS(5228),
    [anon_sym_f64] = ACTIONS(5228),
    [anon_sym_bool] = ACTIONS(5228),
    [anon_sym_str] = ACTIONS(5228),
    [anon_sym_char] = ACTIONS(5228),
    [anon_sym_DASH] = ACTIONS(5230),
    [anon_sym_AMP] = ACTIONS(5230),
    [anon_sym_PIPE] = ACTIONS(5230),
    [anon_sym_LT] = ACTIONS(5230),
    [anon_sym_DOT_DOT] = ACTIONS(5230),
    [anon_sym_COLON_COLON] = ACTIONS(5230),
    [anon_sym_POUND] = ACTIONS(5230),
    [anon_sym_SQUOTE] = ACTIONS(5228),
    [anon_sym_async] = ACTIONS(5228),
    [anon_sym_auto] = ACTIONS(5228),
    [anon_sym_await] = ACTIONS(5228),
    [anon_sym_become] = ACTIONS(5228),
    [anon_sym_break] = ACTIONS(5228),
    [anon_sym_const] = ACTIONS(5228),
    [anon_sym_continue] = ACTIONS(5228),
    [anon_sym_default] = ACTIONS(5228),
    [anon_sym_for] = ACTIONS(5228),
    [anon_sym_gen] = ACTIONS(5228),
    [anon_sym_if] = ACTIONS(5228),
    [anon_sym_loop] = ACTIONS(5228),
    [anon_sym_match] = ACTIONS(5228),
    [anon_sym_return] = ACTIONS(5228),
    [anon_sym_raw] = ACTIONS(5228),
    [anon_sym_static] = ACTIONS(5228),
    [anon_sym_union] = ACTIONS(5228),
    [anon_sym_unsafe] = ACTIONS(5228),
    [anon_sym_while] = ACTIONS(5228),
    [anon_sym_safe] = ACTIONS(5228),
    [anon_sym_yield] = ACTIONS(5228),
    [anon_sym_move] = ACTIONS(5228),
    [anon_sym_try] = ACTIONS(5228),
    [sym_integer_literal] = ACTIONS(5230),
    [aux_sym_string_literal_token1] = ACTIONS(5230),
    [sym_char_literal] = ACTIONS(5230),
    [anon_sym_true] = ACTIONS(5228),
    [anon_sym_false] = ACTIONS(5228),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5228),
    [sym_super] = ACTIONS(5228),
    [sym_crate] = ACTIONS(5228),
    [aux_sym_metavariable_token1] = ACTIONS(5230),
    [sym__raw_string_literal_start] = ACTIONS(5230),
    [sym_float_literal] = ACTIONS(5230),
    [sym__outer_block_doc_comment_start] = ACTIONS(5230),
    [sym__outer_line_doc_comment_start] = ACTIONS(5230),
  },
  [STATE(1936)] = {
    [sym_line_comment] = STATE(1936),
    [sym_block_comment] = STATE(1936),
    [sym_identifier] = ACTIONS(5594),
    [anon_sym_LPAREN] = ACTIONS(722),
    [anon_sym_RPAREN] = ACTIONS(722),
    [anon_sym_LBRACE] = ACTIONS(722),
    [anon_sym_macro_rules] = ACTIONS(5594),
    [anon_sym_BANG] = ACTIONS(722),
    [anon_sym_LBRACK] = ACTIONS(722),
    [anon_sym_STAR] = ACTIONS(722),
    [anon_sym_u8] = ACTIONS(5594),
    [anon_sym_i8] = ACTIONS(5594),
    [anon_sym_u16] = ACTIONS(5594),
    [anon_sym_i16] = ACTIONS(5594),
    [anon_sym_u32] = ACTIONS(5594),
    [anon_sym_i32] = ACTIONS(5594),
    [anon_sym_u64] = ACTIONS(5594),
    [anon_sym_i64] = ACTIONS(5594),
    [anon_sym_u128] = ACTIONS(5594),
    [anon_sym_i128] = ACTIONS(5594),
    [anon_sym_isize] = ACTIONS(5594),
    [anon_sym_usize] = ACTIONS(5594),
    [anon_sym_f32] = ACTIONS(5594),
    [anon_sym_f64] = ACTIONS(5594),
    [anon_sym_bool] = ACTIONS(5594),
    [anon_sym_str] = ACTIONS(5594),
    [anon_sym_char] = ACTIONS(5594),
    [anon_sym_DASH] = ACTIONS(722),
    [anon_sym_AMP] = ACTIONS(722),
    [anon_sym_PIPE] = ACTIONS(722),
    [anon_sym_LT] = ACTIONS(722),
    [anon_sym_DOT_DOT] = ACTIONS(722),
    [anon_sym_COLON_COLON] = ACTIONS(722),
    [anon_sym_POUND] = ACTIONS(722),
    [anon_sym_SQUOTE] = ACTIONS(5594),
    [anon_sym_async] = ACTIONS(5594),
    [anon_sym_auto] = ACTIONS(5594),
    [anon_sym_await] = ACTIONS(5594),
    [anon_sym_become] = ACTIONS(5594),
    [anon_sym_break] = ACTIONS(5594),
    [anon_sym_const] = ACTIONS(5594),
    [anon_sym_continue] = ACTIONS(5594),
    [anon_sym_default] = ACTIONS(5594),
    [anon_sym_for] = ACTIONS(5594),
    [anon_sym_gen] = ACTIONS(5594),
    [anon_sym_if] = ACTIONS(5594),
    [anon_sym_loop] = ACTIONS(5594),
    [anon_sym_match] = ACTIONS(5594),
    [anon_sym_return] = ACTIONS(5594),
    [anon_sym_raw] = ACTIONS(5594),
    [anon_sym_static] = ACTIONS(5594),
    [anon_sym_union] = ACTIONS(5594),
    [anon_sym_unsafe] = ACTIONS(5594),
    [anon_sym_while] = ACTIONS(5594),
    [anon_sym_safe] = ACTIONS(5594),
    [anon_sym_yield] = ACTIONS(5594),
    [anon_sym_move] = ACTIONS(5594),
    [anon_sym_try] = ACTIONS(5594),
    [sym_integer_literal] = ACTIONS(722),
    [aux_sym_string_literal_token1] = ACTIONS(722),
    [sym_char_literal] = ACTIONS(722),
    [anon_sym_true] = ACTIONS(5594),
    [anon_sym_false] = ACTIONS(5594),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5594),
    [sym_super] = ACTIONS(5594),
    [sym_crate] = ACTIONS(5594),
    [aux_sym_metavariable_token1] = ACTIONS(722),
    [sym__raw_string_literal_start] = ACTIONS(722),
    [sym_float_literal] = ACTIONS(722),
    [sym__outer_block_doc_comment_start] = ACTIONS(722),
    [sym__outer_line_doc_comment_start] = ACTIONS(722),
  },
  [STATE(1937)] = {
    [sym_primitive_type] = STATE(4724),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4130),
    [sym_higher_ranked_trait_bound] = STATE(5573),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5573),
    [sym_array_type] = STATE(5573),
    [sym_function_type] = STATE(5573),
    [sym_tuple_type] = STATE(5573),
    [sym_unit_type] = STATE(5573),
    [sym_generic_type] = STATE(4900),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5573),
    [sym_pointer_type] = STATE(5573),
    [sym_dynamic_type] = STATE(5573),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4147),
    [sym_line_comment] = STATE(1937),
    [sym_block_comment] = STATE(1937),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5596),
    [anon_sym_SEMI] = ACTIONS(5598),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5598),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_EQ] = ACTIONS(5598),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_where] = ACTIONS(5616),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1938)] = {
    [sym_primitive_type] = STATE(4724),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4137),
    [sym_higher_ranked_trait_bound] = STATE(5573),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5573),
    [sym_array_type] = STATE(5573),
    [sym_function_type] = STATE(5573),
    [sym_tuple_type] = STATE(5573),
    [sym_unit_type] = STATE(5573),
    [sym_generic_type] = STATE(4900),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5573),
    [sym_pointer_type] = STATE(5573),
    [sym_dynamic_type] = STATE(5573),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4147),
    [sym_line_comment] = STATE(1938),
    [sym_block_comment] = STATE(1938),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5596),
    [anon_sym_SEMI] = ACTIONS(5622),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5622),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_EQ] = ACTIONS(5622),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_where] = ACTIONS(5624),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1939)] = {
    [sym_primitive_type] = STATE(4724),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4130),
    [sym_higher_ranked_trait_bound] = STATE(5573),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5573),
    [sym_array_type] = STATE(5573),
    [sym_function_type] = STATE(5573),
    [sym_tuple_type] = STATE(5573),
    [sym_unit_type] = STATE(5573),
    [sym_generic_type] = STATE(4900),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5573),
    [sym_pointer_type] = STATE(5573),
    [sym_dynamic_type] = STATE(5573),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4147),
    [sym_line_comment] = STATE(1939),
    [sym_block_comment] = STATE(1939),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5596),
    [anon_sym_SEMI] = ACTIONS(5626),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5626),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_EQ] = ACTIONS(5626),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_where] = ACTIONS(5628),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1940)] = {
    [sym_primitive_type] = STATE(4634),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4290),
    [sym_higher_ranked_trait_bound] = STATE(5580),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5580),
    [sym_array_type] = STATE(5580),
    [sym_function_type] = STATE(5580),
    [sym_tuple_type] = STATE(5580),
    [sym_unit_type] = STATE(5580),
    [sym_generic_type] = STATE(5013),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5580),
    [sym_pointer_type] = STATE(5580),
    [sym_dynamic_type] = STATE(5580),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4118),
    [sym_line_comment] = STATE(1940),
    [sym_block_comment] = STATE(1940),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5630),
    [anon_sym_SEMI] = ACTIONS(5622),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5622),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1941)] = {
    [sym_primitive_type] = STATE(4634),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4130),
    [sym_higher_ranked_trait_bound] = STATE(5580),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5580),
    [sym_array_type] = STATE(5580),
    [sym_function_type] = STATE(5580),
    [sym_tuple_type] = STATE(5580),
    [sym_unit_type] = STATE(5580),
    [sym_generic_type] = STATE(5013),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5580),
    [sym_pointer_type] = STATE(5580),
    [sym_dynamic_type] = STATE(5580),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4118),
    [sym_line_comment] = STATE(1941),
    [sym_block_comment] = STATE(1941),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5630),
    [anon_sym_SEMI] = ACTIONS(5598),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5598),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1942)] = {
    [sym_primitive_type] = STATE(4634),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4130),
    [sym_higher_ranked_trait_bound] = STATE(5580),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5580),
    [sym_array_type] = STATE(5580),
    [sym_function_type] = STATE(5580),
    [sym_tuple_type] = STATE(5580),
    [sym_unit_type] = STATE(5580),
    [sym_generic_type] = STATE(5013),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5580),
    [sym_pointer_type] = STATE(5580),
    [sym_dynamic_type] = STATE(5580),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4118),
    [sym_line_comment] = STATE(1942),
    [sym_block_comment] = STATE(1942),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5630),
    [anon_sym_SEMI] = ACTIONS(5626),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_LBRACE] = ACTIONS(5626),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1943)] = {
    [sym_primitive_type] = STATE(4634),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4130),
    [sym_higher_ranked_trait_bound] = STATE(5580),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5580),
    [sym_array_type] = STATE(5580),
    [sym_function_type] = STATE(5580),
    [sym_tuple_type] = STATE(5580),
    [sym_unit_type] = STATE(5580),
    [sym_generic_type] = STATE(5013),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5580),
    [sym_pointer_type] = STATE(5580),
    [sym_dynamic_type] = STATE(5580),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4118),
    [sym_line_comment] = STATE(1943),
    [sym_block_comment] = STATE(1943),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5630),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1944)] = {
    [sym_primitive_type] = STATE(4724),
    [sym_function_modifiers] = STATE(6491),
    [aux_sym__item_modifiers] = STATE(3796),
    [sym_where_predicate] = STATE(4130),
    [sym_higher_ranked_trait_bound] = STATE(5573),
    [sym_extern_modifier] = STATE(6118),
    [sym_bracketed_type] = STATE(6220),
    [sym_lifetime] = STATE(5573),
    [sym_array_type] = STATE(5573),
    [sym_function_type] = STATE(5573),
    [sym_tuple_type] = STATE(5573),
    [sym_unit_type] = STATE(5573),
    [sym_generic_type] = STATE(4900),
    [sym_generic_type_with_turbofish] = STATE(6105),
    [sym_reference_type] = STATE(5573),
    [sym_pointer_type] = STATE(5573),
    [sym_dynamic_type] = STATE(5573),
    [sym_scoped_identifier] = STATE(6229),
    [sym_scoped_type_identifier] = STATE(4147),
    [sym_line_comment] = STATE(1944),
    [sym_block_comment] = STATE(1944),
    [sym_metavariable] = STATE(6220),
    [sym_identifier] = ACTIONS(5596),
    [anon_sym_LPAREN] = ACTIONS(1712),
    [anon_sym_macro_rules] = ACTIONS(5600),
    [anon_sym_LBRACK] = ACTIONS(1720),
    [anon_sym_STAR] = ACTIONS(5602),
    [anon_sym_u8] = ACTIONS(5604),
    [anon_sym_i8] = ACTIONS(5604),
    [anon_sym_u16] = ACTIONS(5604),
    [anon_sym_i16] = ACTIONS(5604),
    [anon_sym_u32] = ACTIONS(5604),
    [anon_sym_i32] = ACTIONS(5604),
    [anon_sym_u64] = ACTIONS(5604),
    [anon_sym_i64] = ACTIONS(5604),
    [anon_sym_u128] = ACTIONS(5604),
    [anon_sym_i128] = ACTIONS(5604),
    [anon_sym_isize] = ACTIONS(5604),
    [anon_sym_usize] = ACTIONS(5604),
    [anon_sym_f32] = ACTIONS(5604),
    [anon_sym_f64] = ACTIONS(5604),
    [anon_sym_bool] = ACTIONS(5604),
    [anon_sym_str] = ACTIONS(5604),
    [anon_sym_char] = ACTIONS(5604),
    [anon_sym_AMP] = ACTIONS(5606),
    [anon_sym_LT] = ACTIONS(31),
    [anon_sym_COLON_COLON] = ACTIONS(5608),
    [anon_sym_SQUOTE] = ACTIONS(5513),
    [anon_sym_async] = ACTIONS(945),
    [anon_sym_auto] = ACTIONS(5600),
    [anon_sym_await] = ACTIONS(5600),
    [anon_sym_const] = ACTIONS(945),
    [anon_sym_default] = ACTIONS(5610),
    [anon_sym_fn] = ACTIONS(5612),
    [anon_sym_for] = ACTIONS(5614),
    [anon_sym_gen] = ACTIONS(5610),
    [anon_sym_raw] = ACTIONS(5600),
    [anon_sym_union] = ACTIONS(5600),
    [anon_sym_unsafe] = ACTIONS(945),
    [anon_sym_extern] = ACTIONS(961),
    [anon_sym_safe] = ACTIONS(5610),
    [anon_sym_dyn] = ACTIONS(5618),
    [anon_sym_try] = ACTIONS(5600),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5620),
    [sym_super] = ACTIONS(5620),
    [sym_crate] = ACTIONS(5620),
    [aux_sym_metavariable_token1] = ACTIONS(121),
  },
  [STATE(1945)] = {
    [sym_attribute_item] = STATE(1967),
    [sym_attributes] = STATE(6067),
    [sym_inner_attribute_item] = STATE(1977),
    [sym_line_comment] = STATE(1945),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(1945),
    [aux_sym_attributes_repeat1] = STATE(1950),
    [aux_sym_match_block_repeat1] = STATE(1945),
    [sym_identifier] = ACTIONS(5632),
    [anon_sym_LPAREN] = ACTIONS(5634),
    [anon_sym_RBRACE] = ACTIONS(5634),
    [anon_sym_macro_rules] = ACTIONS(5632),
    [anon_sym_LBRACK] = ACTIONS(5634),
    [anon_sym_u8] = ACTIONS(5632),
    [anon_sym_i8] = ACTIONS(5632),
    [anon_sym_u16] = ACTIONS(5632),
    [anon_sym_i16] = ACTIONS(5632),
    [anon_sym_u32] = ACTIONS(5632),
    [anon_sym_i32] = ACTIONS(5632),
    [anon_sym_u64] = ACTIONS(5632),
    [anon_sym_i64] = ACTIONS(5632),
    [anon_sym_u128] = ACTIONS(5632),
    [anon_sym_i128] = ACTIONS(5632),
    [anon_sym_isize] = ACTIONS(5632),
    [anon_sym_usize] = ACTIONS(5632),
    [anon_sym_f32] = ACTIONS(5632),
    [anon_sym_f64] = ACTIONS(5632),
    [anon_sym_bool] = ACTIONS(5632),
    [anon_sym_str] = ACTIONS(5632),
    [anon_sym_char] = ACTIONS(5632),
    [anon_sym_DASH] = ACTIONS(5634),
    [anon_sym_AMP] = ACTIONS(5634),
    [anon_sym_PIPE] = ACTIONS(5634),
    [anon_sym_LT] = ACTIONS(5634),
    [anon_sym__] = ACTIONS(5632),
    [anon_sym_DOT_DOT] = ACTIONS(5632),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(5634),
    [anon_sym_COLON_COLON] = ACTIONS(5634),
    [anon_sym_POUND] = ACTIONS(5636),
    [anon_sym_auto] = ACTIONS(5632),
    [anon_sym_await] = ACTIONS(5632),
    [anon_sym_const] = ACTIONS(5632),
    [anon_sym_default] = ACTIONS(5632),
    [anon_sym_gen] = ACTIONS(5632),
    [anon_sym_raw] = ACTIONS(5632),
    [anon_sym_union] = ACTIONS(5632),
    [anon_sym_safe] = ACTIONS(5632),
    [anon_sym_ref] = ACTIONS(5632),
    [sym_mutable_specifier] = ACTIONS(5632),
    [anon_sym_try] = ACTIONS(5632),
    [anon_sym_box] = ACTIONS(5632),
    [sym_integer_literal] = ACTIONS(5634),
    [aux_sym_string_literal_token1] = ACTIONS(5634),
    [sym_char_literal] = ACTIONS(5634),
    [anon_sym_true] = ACTIONS(5632),
    [anon_sym_false] = ACTIONS(5632),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5632),
    [sym_super] = ACTIONS(5632),
    [sym_crate] = ACTIONS(5632),
    [aux_sym_metavariable_token1] = ACTIONS(5634),
    [sym__raw_string_literal_start] = ACTIONS(5634),
    [sym_float_literal] = ACTIONS(5634),
    [sym__outer_block_doc_comment_start] = ACTIONS(5639),
    [sym__outer_line_doc_comment_start] = ACTIONS(5642),
  },
  [STATE(1946)] = {
    [sym_line_comment] = STATE(1946),
    [sym_block_comment] = STATE(1946),
    [sym_identifier] = ACTIONS(5645),
    [anon_sym_SEMI] = ACTIONS(5647),
    [anon_sym_LPAREN] = ACTIONS(5647),
    [anon_sym_RPAREN] = ACTIONS(5647),
    [anon_sym_LBRACE] = ACTIONS(5647),
    [anon_sym_RBRACE] = ACTIONS(5647),
    [anon_sym_COMMA] = ACTIONS(5647),
    [anon_sym_macro_rules] = ACTIONS(5645),
    [anon_sym_BANG] = ACTIONS(5647),
    [anon_sym_LBRACK] = ACTIONS(5647),
    [anon_sym_RBRACK] = ACTIONS(5647),
    [anon_sym_PLUS] = ACTIONS(5647),
    [anon_sym_STAR] = ACTIONS(5647),
    [anon_sym_QMARK] = ACTIONS(5647),
    [anon_sym_u8] = ACTIONS(5645),
    [anon_sym_i8] = ACTIONS(5645),
    [anon_sym_u16] = ACTIONS(5645),
    [anon_sym_i16] = ACTIONS(5645),
    [anon_sym_u32] = ACTIONS(5645),
    [anon_sym_i32] = ACTIONS(5645),
    [anon_sym_u64] = ACTIONS(5645),
    [anon_sym_i64] = ACTIONS(5645),
    [anon_sym_u128] = ACTIONS(5645),
    [anon_sym_i128] = ACTIONS(5645),
    [anon_sym_isize] = ACTIONS(5645),
    [anon_sym_usize] = ACTIONS(5645),
    [anon_sym_f32] = ACTIONS(5645),
    [anon_sym_f64] = ACTIONS(5645),
    [anon_sym_bool] = ACTIONS(5645),
    [anon_sym_str] = ACTIONS(5645),
    [anon_sym_char] = ACTIONS(5645),
    [anon_sym_AMP] = ACTIONS(5647),
    [anon_sym_PIPE] = ACTIONS(5647),
    [anon_sym_EQ] = ACTIONS(5647),
    [anon_sym_GT] = ACTIONS(5647),
    [anon_sym_LT] = ACTIONS(5647),
    [anon_sym_COLON_COLON] = ACTIONS(5647),
    [anon_sym_SQUOTE] = ACTIONS(5647),
    [anon_sym_async] = ACTIONS(5645),
    [anon_sym_auto] = ACTIONS(5645),
    [anon_sym_await] = ACTIONS(5645),
    [anon_sym_const] = ACTIONS(5645),
    [anon_sym_default] = ACTIONS(5645),
    [anon_sym_fn] = ACTIONS(5645),
    [anon_sym_for] = ACTIONS(5645),
    [anon_sym_gen] = ACTIONS(5645),
    [anon_sym_impl] = ACTIONS(5645),
    [anon_sym_raw] = ACTIONS(5645),
    [anon_sym_union] = ACTIONS(5645),
    [anon_sym_unsafe] = ACTIONS(5645),
    [anon_sym_use] = ACTIONS(5645),
    [anon_sym_where] = ACTIONS(5645),
    [anon_sym_extern] = ACTIONS(5645),
    [anon_sym_safe] = ACTIONS(5645),
    [anon_sym_else] = ACTIONS(5645),
    [anon_sym_dyn] = ACTIONS(5645),
    [sym_mutable_specifier] = ACTIONS(5645),
    [anon_sym_try] = ACTIONS(5645),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(5645),
    [sym_super] = ACTIONS(5645),
    [sym_crate] = ACTIONS(5645),
    [aux_sym_metavariable_token1] = ACTIONS(5647),
  },
  [STATE(1947)] = {
    [sym_attribute_item] = STATE(1963),
    [sym_line_comment] = STATE(1947),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(1947),
    [aux_sym_attributes_repeat1] = STATE(1947),
    [sym_identifier] = ACTIONS(2157),
    [anon_sym_LPAREN] = ACTIONS(2159),
    [anon_sym_macro_rules] = ACTIONS(2157),
    [anon_sym_LBRACK] = ACTIONS(2159),
    [anon_sym_u8] = ACTIONS(2157),
    [anon_sym_i8] = ACTIONS(2157),
    [anon_sym_u16] = ACTIONS(2157),
    [anon_sym_i16] = ACTIONS(2157),
    [anon_sym_u32] = ACTIONS(2157),
    [anon_sym_i32] = ACTIONS(2157),
    [anon_sym_u64] = ACTIONS(2157),
    [anon_sym_i64] = ACTIONS(2157),
    [anon_sym_u128] = ACTIONS(2157),
    [anon_sym_i128] = ACTIONS(2157),
    [anon_sym_isize] = ACTIONS(2157),
    [anon_sym_usize] = ACTIONS(2157),
    [anon_sym_f32] = ACTIONS(2157),
    [anon_sym_f64] = ACTIONS(2157),
    [anon_sym_bool] = ACTIONS(2157),
    [anon_sym_str] = ACTIONS(2157),
    [anon_sym_char] = ACTIONS(2157),
    [anon_sym_DASH] = ACTIONS(2159),
    [anon_sym_AMP] = ACTIONS(2159),
    [anon_sym_PIPE] = ACTIONS(2159),
    [anon_sym_LT] = ACTIONS(2159),
    [anon_sym__] = ACTIONS(2157),
    [anon_sym_DOT_DOT] = ACTIONS(2157),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2159),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(2159),
    [anon_sym_COLON_COLON] = ACTIONS(2159),
    [anon_sym_POUND] = ACTIONS(5649),
    [anon_sym_SQUOTE] = ACTIONS(2157),
    [anon_sym_auto] = ACTIONS(2157),
    [anon_sym_await] = ACTIONS(2157),
    [anon_sym_const] = ACTIONS(2157),
    [anon_sym_default] = ACTIONS(2157),
    [anon_sym_gen] = ACTIONS(2157),
    [anon_sym_raw] = ACTIONS(2157),
    [anon_sym_union] = ACTIONS(2157),
    [anon_sym_safe] = ACTIONS(2157),
    [anon_sym_ref] = ACTIONS(2157),
    [sym_mutable_specifier] = ACTIONS(2157),
    [anon_sym_try] = ACTIONS(2157),
    [anon_sym_box] = ACTIONS(2157),
    [sym_integer_literal] = ACTIONS(2159),
    [aux_sym_string_literal_token1] = ACTIONS(2159),
    [sym_char_literal] = ACTIONS(2159),
    [anon_sym_true] = ACTIONS(2157),
    [anon_sym_false] = ACTIONS(2157),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2157),
    [sym_super] = ACTIONS(2157),
    [sym_crate] = ACTIONS(2157),
    [aux_sym_metavariable_token1] = ACTIONS(2159),
    [sym__raw_string_literal_start] = ACTIONS(2159),
    [sym_float_literal] = ACTIONS(2159),
    [sym__outer_block_doc_comment_start] = ACTIONS(5652),
    [sym__outer_line_doc_comment_start] = ACTIONS(5655),
  },
  [STATE(1948)] = {
    [sym_attribute_item] = STATE(1963),
    [sym_line_comment] = STATE(1948),
    [sym_line_outer_doc_comment] = STATE(1963),
    [sym_block_outer_doc_comment] = STATE(1963),
    [sym_block_comment] = STATE(1948),
    [aux_sym_attributes_repeat1] = STATE(1947),
    [sym_identifier] = ACTIONS(2151),
    [anon_sym_LPAREN] = ACTIONS(2153),
    [anon_sym_macro_rules] = ACTIONS(2151),
    [anon_sym_LBRACK] = ACTIONS(2153),
    [anon_sym_u8] = ACTIONS(2151),
    [anon_sym_i8] = ACTIONS(2151),
    [anon_sym_u16] = ACTIONS(2151),
    [anon_sym_i16] = ACTIONS(2151),
    [anon_sym_u32] = ACTIONS(2151),
    [anon_sym_i32] = ACTIONS(2151),
    [anon_sym_u64] = ACTIONS(2151),
    [anon_sym_i64] = ACTIONS(2151),
    [anon_sym_u128] = ACTIONS(2151),
    [anon_sym_i128] = ACTIONS(2151),
    [anon_sym_isize] = ACTIONS(2151),
    [anon_sym_usize] = ACTIONS(2151),
    [anon_sym_f32] = ACTIONS(2151),
    [anon_sym_f64] = ACTIONS(2151),
    [anon_sym_bool] = ACTIONS(2151),
    [anon_sym_str] = ACTIONS(2151),
    [anon_sym_char] = ACTIONS(2151),
    [anon_sym_DASH] = ACTIONS(2153),
    [anon_sym_AMP] = ACTIONS(2153),
    [anon_sym_PIPE] = ACTIONS(2153),
    [anon_sym_LT] = ACTIONS(2153),
    [anon_sym__] = ACTIONS(2151),
    [anon_sym_DOT_DOT] = ACTIONS(2151),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2153),
    [anon_sym_DOT_DOT_EQ] = ACTIONS(2153),
    [anon_sym_COLON_COLON] = ACTIONS(2153),
    [anon_sym_POUND] = ACTIONS(941),
    [anon_sym_SQUOTE] = ACTIONS(2151),
    [anon_sym_auto] = ACTIONS(2151),
    [anon_sym_await] = ACTIONS(2151),
    [anon_sym_const] = ACTIONS(2151),
    [anon_sym_default] = ACTIONS(2151),
    [anon_sym_gen] = ACTIONS(2151),
    [anon_sym_raw] = ACTIONS(2151),
    [anon_sym_union] = ACTIONS(2151),
    [anon_sym_safe] = ACTIONS(2151),
    [anon_sym_ref] = ACTIONS(2151),
    [sym_mutable_specifier] = ACTIONS(2151),
    [anon_sym_try] = ACTIONS(2151),
    [anon_sym_box] = ACTIONS(2151),
    [sym_integer_literal] = ACTIONS(2153),
    [aux_sym_string_literal_token1] = ACTIONS(2153),
    [sym_char_literal] = ACTIONS(2153),
    [anon_sym_true] = ACTIONS(2151),
    [anon_sym_false] = ACTIONS(2151),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2151),
    [sym_super] = ACTIONS(2151),
    [sym_crate] = ACTIONS(2151),
    [aux_sym_metavariable_token1] = ACTIONS(2153),
    [sym__raw_string_literal_start] = ACTIONS(2153),
    [sym_float_literal] = ACTIONS(2153),
    [sym__outer_block_doc_comment_start] = ACTIONS(985),
    [sym__outer_line_doc_comment_start] = ACTIONS(987),
  },
  [STATE(1949)] = {
    [sym_attribute_item] = STATE(1967),
    [sym_line_comment] = STATE(1949),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(1949),
    [aux_sym_attributes_repeat1] = STATE(1949),
    [sym_identifier] = ACTIONS(2157),
    [anon_sym_SEMI] = ACTIONS(2159),
    [anon_sym_macro] = ACTIONS(2157),
    [anon_sym_macro_rules] = ACTIONS(2157),
    [anon_sym_u8] = ACTIONS(2157),
    [anon_sym_i8] = ACTIONS(2157),
    [anon_sym_u16] = ACTIONS(2157),
    [anon_sym_i16] = ACTIONS(2157),
    [anon_sym_u32] = ACTIONS(2157),
    [anon_sym_i32] = ACTIONS(2157),
    [anon_sym_u64] = ACTIONS(2157),
    [anon_sym_i64] = ACTIONS(2157),
    [anon_sym_u128] = ACTIONS(2157),
    [anon_sym_i128] = ACTIONS(2157),
    [anon_sym_isize] = ACTIONS(2157),
    [anon_sym_usize] = ACTIONS(2157),
    [anon_sym_f32] = ACTIONS(2157),
    [anon_sym_f64] = ACTIONS(2157),
    [anon_sym_bool] = ACTIONS(2157),
    [anon_sym_str] = ACTIONS(2157),
    [anon_sym_char] = ACTIONS(2157),
    [anon_sym_LT] = ACTIONS(2159),
    [anon_sym_COLON_COLON] = ACTIONS(2159),
    [anon_sym_POUND] = ACTIONS(5658),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [anon_sym_async] = ACTIONS(2157),
    [anon_sym_auto] = ACTIONS(2157),
    [anon_sym_await] = ACTIONS(2157),
    [anon_sym_const] = ACTIONS(2157),
    [anon_sym_default] = ACTIONS(2157),
    [anon_sym_enum] = ACTIONS(2157),
    [anon_sym_fn] = ACTIONS(2157),
    [anon_sym_gen] = ACTIONS(2157),
    [anon_sym_impl] = ACTIONS(2157),
    [anon_sym_let] = ACTIONS(2157),
    [anon_sym_mod] = ACTIONS(2157),
    [anon_sym_pub] = ACTIONS(2157),
    [anon_sym_raw] = ACTIONS(2157),
    [anon_sym_static] = ACTIONS(2157),
    [anon_sym_struct] = ACTIONS(2157),
    [anon_sym_trait] = ACTIONS(2157),
    [anon_sym_type] = ACTIONS(2157),
    [anon_sym_union] = ACTIONS(2157),
    [anon_sym_unsafe] = ACTIONS(2157),
    [anon_sym_use] = ACTIONS(2157),
    [anon_sym_extern] = ACTIONS(2157),
    [anon_sym_safe] = ACTIONS(2157),
    [anon_sym_ref] = ACTIONS(2157),
    [sym_mutable_specifier] = ACTIONS(2157),
    [anon_sym_try] = ACTIONS(2157),
    [sym_integer_literal] = ACTIONS(2159),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2157),
    [sym_super] = ACTIONS(2157),
    [sym_crate] = ACTIONS(2157),
    [aux_sym_metavariable_token1] = ACTIONS(2159),
    [sym__outer_block_doc_comment_start] = ACTIONS(5661),
    [sym__outer_line_doc_comment_start] = ACTIONS(5664),
  },
  [STATE(1950)] = {
    [sym_attribute_item] = STATE(1967),
    [sym_line_comment] = STATE(1950),
    [sym_line_outer_doc_comment] = STATE(1967),
    [sym_block_outer_doc_comment] = STATE(1967),
    [sym_block_comment] = STATE(1950),
    [aux_sym_attributes_repeat1] = STATE(1949),
    [sym_identifier] = ACTIONS(2151),
    [anon_sym_SEMI] = ACTIONS(2153),
    [anon_sym_macro] = ACTIONS(2151),
    [anon_sym_macro_rules] = ACTIONS(2151),
    [anon_sym_u8] = ACTIONS(2151),
    [anon_sym_i8] = ACTIONS(2151),
    [anon_sym_u16] = ACTIONS(2151),
    [anon_sym_i16] = ACTIONS(2151),
    [anon_sym_u32] = ACTIONS(2151),
    [anon_sym_i32] = ACTIONS(2151),
    [anon_sym_u64] = ACTIONS(2151),
    [anon_sym_i64] = ACTIONS(2151),
    [anon_sym_u128] = ACTIONS(2151),
    [anon_sym_i128] = ACTIONS(2151),
    [anon_sym_isize] = ACTIONS(2151),
    [anon_sym_usize] = ACTIONS(2151),
    [anon_sym_f32] = ACTIONS(2151),
    [anon_sym_f64] = ACTIONS(2151),
    [anon_sym_bool] = ACTIONS(2151),
    [anon_sym_str] = ACTIONS(2151),
    [anon_sym_char] = ACTIONS(2151),
    [anon_sym_LT] = ACTIONS(2153),
    [anon_sym_COLON_COLON] = ACTIONS(2153),
    [anon_sym_POUND] = ACTIONS(1886),
    [anon_sym_SQUOTE] = ACTIONS(2153),
    [anon_sym_async] = ACTIONS(2151),
    [anon_sym_auto] = ACTIONS(2151),
    [anon_sym_await] = ACTIONS(2151),
    [anon_sym_const] = ACTIONS(2151),
    [anon_sym_default] = ACTIONS(2151),
    [anon_sym_enum] = ACTIONS(2151),
    [anon_sym_fn] = ACTIONS(2151),
    [anon_sym_gen] = ACTIONS(2151),
    [anon_sym_impl] = ACTIONS(2151),
    [anon_sym_let] = ACTIONS(2151),
    [anon_sym_mod] = ACTIONS(2151),
    [anon_sym_pub] = ACTIONS(2151),
    [anon_sym_raw] = ACTIONS(2151),
    [anon_sym_static] = ACTIONS(2151),
    [anon_sym_struct] = ACTIONS(2151),
    [anon_sym_trait] = ACTIONS(2151),
    [anon_sym_type] = ACTIONS(2151),
    [anon_sym_union] = ACTIONS(2151),
    [anon_sym_unsafe] = ACTIONS(2151),
    [anon_sym_use] = ACTIONS(2151),
    [anon_sym_extern] = ACTIONS(2151),
    [anon_sym_safe] = ACTIONS(2151),
    [anon_sym_ref] = ACTIONS(2151),
    [sym_mutable_specifier] = ACTIONS(2151),
    [anon_sym_try] = ACTIONS(2151),
    [sym_integer_literal] = ACTIONS(2153),
    [anon_sym_SLASH_SLASH] = ACTIONS(3),
    [anon_sym_SLASH_STAR] = ACTIONS(5),
    [sym_self] = ACTIONS(2151),
    [sym_super] = ACTIONS(2151),
    [sym_crate] = ACTIONS(2151),
    [aux_sym_metavariable_token1] = ACTIONS(2153),
    [sym__outer_block_doc_comment_start] = ACTIONS(1892),
    [sym__outer_line_doc_comment_start] = ACTIONS(1894),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(925), 1,
      anon_sym_DASH,
    ACTIONS(973), 1,
      aux_sym_string_literal_token1,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(983), 1,
      sym__raw_string_literal_start,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5667), 1,
      sym_identifier,
    STATE(3728), 1,
      sym_metavariable,
    STATE(3729), 1,
      sym_scoped_identifier,
    STATE(3772), 1,
      sym__literal_pattern,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(975), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1951), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(971), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5673), 3,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
    ACTIONS(5675), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3672), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5669), 7,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
    ACTIONS(5671), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [109] = 42,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(945), 1,
      anon_sym_async,
    ACTIONS(1896), 1,
      sym_identifier,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(1950), 1,
      sym_crate,
    ACTIONS(5677), 1,
      anon_sym_SEMI,
    ACTIONS(5679), 1,
      anon_sym_macro,
    ACTIONS(5681), 1,
      anon_sym_macro_rules,
    ACTIONS(5683), 1,
      anon_sym_POUND,
    ACTIONS(5685), 1,
      anon_sym_auto,
    ACTIONS(5687), 1,
      anon_sym_const,
    ACTIONS(5689), 1,
      anon_sym_enum,
    ACTIONS(5691), 1,
      anon_sym_fn,
    ACTIONS(5693), 1,
      anon_sym_impl,
    ACTIONS(5695), 1,
      anon_sym_let,
    ACTIONS(5697), 1,
      anon_sym_mod,
    ACTIONS(5699), 1,
      anon_sym_static,
    ACTIONS(5701), 1,
      anon_sym_struct,
    ACTIONS(5703), 1,
      anon_sym_trait,
    ACTIONS(5705), 1,
      anon_sym_type,
    ACTIONS(5707), 1,
      anon_sym_union,
    ACTIONS(5709), 1,
      anon_sym_unsafe,
    ACTIONS(5711), 1,
      anon_sym_use,
    ACTIONS(5713), 1,
      anon_sym_extern,
    ACTIONS(5715), 1,
      anon_sym_safe,
    STATE(2523), 1,
      sym_macro_invocation,
    STATE(3499), 1,
      sym_visibility_modifier,
    STATE(3792), 1,
      aux_sym__item_modifiers,
    STATE(4717), 1,
      sym_extern_modifier,
    STATE(5653), 1,
      sym_scoped_identifier,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(6445), 1,
      sym_function_modifiers,
    ACTIONS(1918), 2,
      anon_sym_default,
      anon_sym_gen,
    ACTIONS(1948), 2,
      sym_self,
      sym_super,
    STATE(1952), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6226), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(1914), 3,
      anon_sym_await,
      anon_sym_raw,
      anon_sym_try,
    ACTIONS(1906), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [258] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(925), 1,
      anon_sym_DASH,
    ACTIONS(973), 1,
      aux_sym_string_literal_token1,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(983), 1,
      sym__raw_string_literal_start,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5717), 1,
      sym_identifier,
    STATE(3724), 1,
      sym_metavariable,
    STATE(3725), 1,
      sym_scoped_identifier,
    STATE(3762), 1,
      sym__literal_pattern,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(975), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1953), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(971), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5723), 3,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
    ACTIONS(5725), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3672), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5719), 7,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
    ACTIONS(5721), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [367] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(925), 1,
      anon_sym_DASH,
    ACTIONS(973), 1,
      aux_sym_string_literal_token1,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(983), 1,
      sym__raw_string_literal_start,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5727), 1,
      sym_identifier,
    STATE(3716), 1,
      sym_metavariable,
    STATE(3717), 1,
      sym_scoped_identifier,
    STATE(3755), 1,
      sym__literal_pattern,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(975), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(1954), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(971), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5733), 3,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
    ACTIONS(5735), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3672), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5729), 7,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
    ACTIONS(5731), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [476] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1704), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1706), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(1800), 1,
      anon_sym_POUND,
    STATE(1956), 1,
      aux_sym_attributes_repeat1,
    STATE(1955), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1981), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    ACTIONS(2153), 14,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(2151), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [560] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5737), 1,
      anon_sym_POUND,
    ACTIONS(5740), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(5743), 1,
      sym__outer_line_doc_comment_start,
    STATE(1956), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_attributes_repeat1,
    STATE(1981), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    ACTIONS(2159), 14,
      sym__raw_string_literal_start,
      sym_float_literal,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(2157), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [642] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5746), 1,
      anon_sym_POUND,
    ACTIONS(5749), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(5752), 1,
      sym__outer_line_doc_comment_start,
    STATE(1957), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_attributes_repeat1,
    STATE(2751), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    ACTIONS(2159), 10,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(2157), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [722] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1958), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5222), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5220), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_SQUOTE,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [794] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1959), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5222), 9,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      sym_integer_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5220), 48,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [866] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1960), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5234), 9,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      sym_integer_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5232), 48,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [938] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1961), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5238), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5236), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_SQUOTE,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1010] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1962), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5234), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5232), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_SQUOTE,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1082] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1963), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5230), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5228), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_SQUOTE,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1154] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1964), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5218), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5216), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_SQUOTE,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1226] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2099), 1,
      anon_sym_POUND,
    ACTIONS(2107), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(2109), 1,
      sym__outer_line_doc_comment_start,
    STATE(1957), 1,
      aux_sym_attributes_repeat1,
    STATE(1965), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(2751), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    ACTIONS(2153), 10,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(2151), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1308] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1966), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5238), 9,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      sym_integer_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5236), 48,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1380] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1967), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5230), 9,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      sym_integer_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5228), 48,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1452] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1968), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5218), 9,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      sym_integer_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5216), 48,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1524] = 28,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(961), 1,
      anon_sym_extern,
    ACTIONS(1738), 1,
      anon_sym_fn,
    ACTIONS(1740), 1,
      anon_sym_for,
    ACTIONS(1744), 1,
      anon_sym_unsafe,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(5608), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5755), 1,
      sym_identifier,
    ACTIONS(5757), 1,
      anon_sym_LPAREN,
    STATE(3575), 1,
      sym_generic_type,
    STATE(3796), 1,
      aux_sym__item_modifiers,
    STATE(3834), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6082), 1,
      sym_function_modifiers,
    STATE(6105), 1,
      sym_generic_type_with_turbofish,
    STATE(6118), 1,
      sym_extern_modifier,
    STATE(6229), 1,
      sym_scoped_identifier,
    ACTIONS(945), 2,
      anon_sym_async,
      anon_sym_const,
    STATE(1969), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6220), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(5610), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_safe,
    ACTIONS(5620), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3618), 5,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_function_type,
      sym_unsafe_bound_type,
      sym_tuple_type,
    ACTIONS(5600), 6,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_try,
    ACTIONS(5604), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [1641] = 28,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(961), 1,
      anon_sym_extern,
    ACTIONS(5319), 1,
      anon_sym_fn,
    ACTIONS(5321), 1,
      anon_sym_for,
    ACTIONS(5325), 1,
      anon_sym_unsafe,
    ACTIONS(5513), 1,
      anon_sym_SQUOTE,
    ACTIONS(5759), 1,
      sym_identifier,
    ACTIONS(5761), 1,
      anon_sym_LPAREN,
    ACTIONS(5767), 1,
      anon_sym_COLON_COLON,
    STATE(2793), 1,
      sym_scoped_type_identifier,
    STATE(2810), 1,
      sym_generic_type,
    STATE(3796), 1,
      aux_sym__item_modifiers,
    STATE(5672), 1,
      sym_primitive_type,
    STATE(6118), 1,
      sym_extern_modifier,
    STATE(6238), 1,
      sym_generic_type_with_turbofish,
    STATE(6239), 1,
      sym_scoped_identifier,
    STATE(6257), 1,
      sym_function_modifiers,
    ACTIONS(945), 2,
      anon_sym_async,
      anon_sym_const,
    STATE(1970), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6291), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(5769), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_safe,
    ACTIONS(5771), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(2961), 5,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_function_type,
      sym_unsafe_bound_type,
      sym_tuple_type,
    ACTIONS(5763), 6,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_try,
    ACTIONS(5765), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [1758] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1971), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4558), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(4560), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [1829] = 28,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(961), 1,
      anon_sym_extern,
    ACTIONS(5367), 1,
      anon_sym_fn,
    ACTIONS(5369), 1,
      anon_sym_for,
    ACTIONS(5373), 1,
      anon_sym_unsafe,
    ACTIONS(5537), 1,
      anon_sym_SQUOTE,
    ACTIONS(5773), 1,
      sym_identifier,
    ACTIONS(5775), 1,
      anon_sym_LPAREN,
    ACTIONS(5781), 1,
      anon_sym_COLON_COLON,
    STATE(3063), 1,
      sym_scoped_type_identifier,
    STATE(3132), 1,
      sym_generic_type,
    STATE(3796), 1,
      aux_sym__item_modifiers,
    STATE(5468), 1,
      sym_primitive_type,
    STATE(6118), 1,
      sym_extern_modifier,
    STATE(6228), 1,
      sym_function_modifiers,
    STATE(6272), 1,
      sym_generic_type_with_turbofish,
    STATE(6273), 1,
      sym_scoped_identifier,
    ACTIONS(945), 2,
      anon_sym_async,
      anon_sym_const,
    STATE(1972), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6296), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(5783), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_safe,
    ACTIONS(5785), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3378), 5,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_function_type,
      sym_unsafe_bound_type,
      sym_tuple_type,
    ACTIONS(5777), 6,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_try,
    ACTIONS(5779), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [1946] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1973), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4362), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(4364), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2017] = 28,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(951), 1,
      anon_sym_fn,
    ACTIONS(953), 1,
      anon_sym_for,
    ACTIONS(957), 1,
      anon_sym_unsafe,
    ACTIONS(961), 1,
      anon_sym_extern,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(5608), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5757), 1,
      anon_sym_LPAREN,
    ACTIONS(5787), 1,
      sym_identifier,
    STATE(3553), 1,
      sym_scoped_type_identifier,
    STATE(3575), 1,
      sym_generic_type,
    STATE(3796), 1,
      aux_sym__item_modifiers,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6105), 1,
      sym_generic_type_with_turbofish,
    STATE(6118), 1,
      sym_extern_modifier,
    STATE(6229), 1,
      sym_scoped_identifier,
    STATE(6439), 1,
      sym_function_modifiers,
    ACTIONS(945), 2,
      anon_sym_async,
      anon_sym_const,
    STATE(1974), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6220), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(5610), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_safe,
    ACTIONS(5620), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3618), 5,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_function_type,
      sym_unsafe_bound_type,
      sym_tuple_type,
    ACTIONS(5600), 6,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_try,
    ACTIONS(5604), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [2134] = 28,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(961), 1,
      anon_sym_extern,
    ACTIONS(5513), 1,
      anon_sym_SQUOTE,
    ACTIONS(5608), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5612), 1,
      anon_sym_fn,
    ACTIONS(5614), 1,
      anon_sym_for,
    ACTIONS(5757), 1,
      anon_sym_LPAREN,
    ACTIONS(5789), 1,
      sym_identifier,
    ACTIONS(5791), 1,
      anon_sym_unsafe,
    STATE(3575), 1,
      sym_generic_type,
    STATE(3796), 1,
      aux_sym__item_modifiers,
    STATE(4435), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6105), 1,
      sym_generic_type_with_turbofish,
    STATE(6118), 1,
      sym_extern_modifier,
    STATE(6229), 1,
      sym_scoped_identifier,
    STATE(6491), 1,
      sym_function_modifiers,
    ACTIONS(945), 2,
      anon_sym_async,
      anon_sym_const,
    STATE(1975), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6220), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(5610), 3,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_safe,
    ACTIONS(5620), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3618), 5,
      sym_higher_ranked_trait_bound,
      sym_lifetime,
      sym_function_type,
      sym_unsafe_bound_type,
      sym_tuple_type,
    ACTIONS(5600), 6,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_try,
    ACTIONS(5604), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [2251] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5797), 1,
      anon_sym_RBRACE,
    STATE(1976), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5795), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5793), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2324] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1977), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5801), 18,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5799), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2395] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5807), 1,
      anon_sym_RBRACE,
    STATE(1978), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5805), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5803), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2468] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1979), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5795), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5793), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2538] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1980), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5811), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5809), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2608] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1981), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5230), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5228), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2678] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1982), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5222), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5220), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2748] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1983), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5234), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5232), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2818] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1984), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2141), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(2139), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2888] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1985), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5238), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5236), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [2958] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1986), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5218), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5216), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3028] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1987), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2137), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(2135), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3098] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1988), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5805), 17,
      sym__raw_string_literal_start,
      sym_float_literal,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
      anon_sym_DASH,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      sym_integer_literal,
      aux_sym_string_literal_token1,
      sym_char_literal,
      aux_sym_metavariable_token1,
    ACTIONS(5803), 38,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym__,
      anon_sym_DOT_DOT,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_ref,
      sym_mutable_specifier,
      anon_sym_try,
      anon_sym_box,
      anon_sym_true,
      anon_sym_false,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3168] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1989), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2458), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2460), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3237] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1990), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2246), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2248), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3306] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1991), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2250), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2252), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3375] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1992), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2254), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2256), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3444] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1993), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2258), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2260), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3513] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1994), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2262), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2264), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3582] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1995), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2266), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2268), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3651] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1996), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2270), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2272), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3720] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1997), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2274), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2276), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3789] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1998), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2278), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2280), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3858] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(1999), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2282), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2284), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3927] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2000), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2286), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2288), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [3996] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2001), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2290), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2292), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4065] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2002), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2294), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2296), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4134] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2003), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2298), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2300), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4203] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2004), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2302), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2304), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4272] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2005), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2306), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2308), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4341] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2006), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2310), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2312), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4410] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2007), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2314), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2316), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4479] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2008), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2318), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2320), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4548] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2009), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2330), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2332), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4617] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2010), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2334), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2336), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4686] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2011), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2362), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2364), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4755] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2012), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2366), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2368), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4824] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2013), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2370), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2372), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4893] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2014), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2374), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2376), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [4962] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2015), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2378), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2380), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5031] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2016), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2382), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2384), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5100] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2017), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2386), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2388), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5169] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2018), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2390), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2392), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5238] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2019), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2394), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2396), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5307] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2020), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2398), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2400), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5376] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2021), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2402), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2404), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5445] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2022), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2406), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2408), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5514] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2023), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2410), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2412), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5583] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2024), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2414), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2416), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5652] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2025), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2418), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2420), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5721] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2026), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2422), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2424), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5790] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2027), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2426), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2428), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5859] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2028), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2430), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2432), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5928] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2029), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2434), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2436), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [5997] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2030), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2438), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2440), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6066] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2031), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2442), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2444), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6135] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2032), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2446), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2448), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6204] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2033), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2450), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2452), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6273] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2034), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2454), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2456), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6342] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2035), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2202), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2204), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6411] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2036), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2462), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2464), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6480] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2037), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2466), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2468), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6549] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2038), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2470), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2472), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6618] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2039), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2474), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2476), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6687] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2040), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2478), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2480), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6756] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2041), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2482), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2484), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6825] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2042), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2486), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2488), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6894] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2043), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2490), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2492), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [6963] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2044), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2494), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2496), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7032] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2045), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2206), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2208), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7101] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2046), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2502), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2504), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7170] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2047), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2506), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2508), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7239] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2048), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2510), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2512), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7308] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2049), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2514), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2516), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7377] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2050), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2518), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2520), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7446] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2051), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2522), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2524), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7515] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2052), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2530), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2532), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7584] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2053), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2534), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2536), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7653] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2054), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2538), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2540), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7722] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2055), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2546), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2548), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7791] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2056), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2550), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2552), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7860] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2057), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2554), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2556), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7929] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2058), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2558), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2560), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [7998] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2059), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2562), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2564), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8067] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2060), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2566), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2568), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8136] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2061), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2570), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2572), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8205] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2062), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2574), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2576), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8274] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2063), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2578), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2580), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8343] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2064), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2582), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2584), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8412] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2065), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2586), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2588), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8481] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2066), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2590), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2592), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8550] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2067), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2594), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2596), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8619] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2068), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2598), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2600), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8688] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2069), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2602), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2604), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8757] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2070), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2606), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2608), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8826] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2071), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2610), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2612), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8895] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2072), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2614), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2616), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [8964] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2073), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2618), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2620), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9033] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2074), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2622), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2624), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9102] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2075), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2626), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2628), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9171] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2076), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2630), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2632), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9240] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2077), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2634), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2636), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9309] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2078), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2642), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2644), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9378] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2079), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2646), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2648), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9447] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2080), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2650), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2652), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9516] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2081), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2654), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2656), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9585] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2082), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2658), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2660), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9654] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2083), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2662), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2664), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9723] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2084), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2666), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2668), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9792] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2085), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2670), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2672), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9861] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2086), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2674), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2676), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9930] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2087), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2678), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2680), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [9999] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2088), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2682), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2684), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10068] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2089), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2686), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2688), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10137] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2090), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2690), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2692), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10206] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2091), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2694), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2696), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10275] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2092), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2698), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2700), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10344] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2093), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2702), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2704), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10413] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2094), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2706), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2708), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10482] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2095), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2710), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2712), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10551] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2096), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2714), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2716), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10620] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2097), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2718), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2720), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10689] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2098), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2722), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2724), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10758] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2099), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2726), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2728), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10827] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2100), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2730), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2732), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10896] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2101), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2734), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2736), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [10965] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2102), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2738), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2740), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11034] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2103), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2742), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2744), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11103] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2104), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2746), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2748), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11172] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2105), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2750), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2752), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11241] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2106), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2754), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2756), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11310] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2107), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2758), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2760), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11379] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2108), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2762), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2764), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11448] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2109), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2766), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2768), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11517] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2110), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2770), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2772), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11586] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2111), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2774), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2776), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11655] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2112), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2782), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2784), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11724] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2113), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2786), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2788), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11793] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2114), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2790), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2792), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11862] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2115), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2794), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2796), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [11931] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2116), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2798), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2800), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12000] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2117), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2802), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2804), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12069] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2118), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2806), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2808), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12138] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2119), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2810), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2812), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12207] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2120), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2814), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2816), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12276] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2121), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2818), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2820), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12345] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2122), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2822), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2824), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12414] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2123), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2826), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2828), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12483] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2124), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2830), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2832), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12552] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2125), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2834), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2836), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12621] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2126), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2838), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2840), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12690] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2127), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2842), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2844), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12759] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2128), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2846), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2848), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12828] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2129), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2850), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2852), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12897] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2130), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2854), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2856), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [12966] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2131), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2858), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2860), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13035] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2132), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2862), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2864), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13104] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2133), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2866), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2868), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13173] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2134), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2870), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2872), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13242] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2135), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2882), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2884), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13311] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2136), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2886), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2888), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13380] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2137), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2890), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2892), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13449] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2138), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2894), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2896), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13518] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2139), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2898), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2900), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13587] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2140), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2902), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2904), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13656] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2141), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2906), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2908), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13725] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2142), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2910), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2912), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13794] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2143), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2914), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2916), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13863] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2144), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2918), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2920), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [13932] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2145), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2922), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2924), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14001] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2146), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2926), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2928), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14070] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2147), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2930), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2932), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14139] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2148), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2934), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2936), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14208] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2149), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2938), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2940), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14277] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2150), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2942), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2944), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14346] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2151), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2946), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2948), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14415] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2152), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2950), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2952), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14484] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2153), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2954), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2956), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14553] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2154), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2958), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2960), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14622] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2155), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2962), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2964), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14691] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2156), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2966), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2968), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14760] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2157), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2974), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2976), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14829] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2158), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2978), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2980), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14898] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2159), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2982), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2984), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [14967] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2160), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2986), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2988), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15036] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2161), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2990), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2992), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15105] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2162), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2994), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2996), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15174] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2163), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2998), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3000), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15243] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2164), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3002), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3004), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15312] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2165), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3006), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3008), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15381] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2166), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3010), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3012), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15450] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2167), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3014), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3016), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15519] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2168), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3018), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3020), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15588] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2169), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3022), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3024), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15657] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2170), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3026), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3028), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15726] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2171), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3030), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3032), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15795] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2172), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3034), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3036), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15864] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2173), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3038), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3040), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [15933] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2174), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3042), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3044), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16002] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2175), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3046), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3048), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16071] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2176), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3050), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3052), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16140] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2177), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3054), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3056), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16209] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2178), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3058), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3060), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16278] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2179), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3062), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3064), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16347] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2180), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3066), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3068), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16416] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2181), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3070), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3072), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16485] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2182), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3074), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3076), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16554] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2183), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3078), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3080), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16623] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2184), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3082), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3084), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16692] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2185), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3086), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3088), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16761] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2186), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3090), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3092), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16830] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2187), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3094), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3096), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16899] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2188), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3102), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3104), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [16968] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2189), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3106), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3108), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17037] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2190), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3110), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3112), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17106] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2191), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3114), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3116), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17175] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2192), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3118), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3120), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17244] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2193), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3122), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3124), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17313] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2194), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3126), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3128), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17382] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2195), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3130), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3132), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17451] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2196), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3134), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3136), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17520] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2197), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3138), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3140), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17589] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2198), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3142), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3144), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17658] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2199), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3146), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3148), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17727] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2200), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3150), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3152), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17796] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2201), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3154), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3156), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17865] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2202), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3158), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3160), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [17934] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2203), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3162), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3164), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18003] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2204), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3166), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3168), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18072] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2205), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3170), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3172), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18141] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2206), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3174), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3176), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18210] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2207), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3178), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3180), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18279] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2208), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3182), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3184), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18348] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2209), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3186), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3188), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18417] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2210), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3190), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3192), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18486] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2211), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3194), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3196), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18555] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2212), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3198), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3200), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18624] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2213), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3202), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3204), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18693] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2214), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3206), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3208), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18762] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2215), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3210), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3212), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18831] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2216), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3214), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3216), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18900] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2217), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3218), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3220), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [18969] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2218), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3222), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3224), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19038] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2219), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3226), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3228), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19107] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2220), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3230), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3232), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19176] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2221), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3234), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3236), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19245] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2222), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3238), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3240), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19314] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2223), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3242), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3244), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19383] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2224), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3246), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3248), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19452] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2225), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3250), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3252), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19521] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2226), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3254), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3256), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19590] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2227), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3258), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3260), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19659] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2228), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3262), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3264), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19728] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2229), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3266), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3268), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19797] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2230), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3270), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3272), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19866] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2231), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3274), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3276), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [19935] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2232), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3278), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3280), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20004] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2233), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3282), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3284), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20073] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2234), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3286), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3288), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20142] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2235), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3290), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3292), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20211] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2236), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3294), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3296), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20280] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2237), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3298), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3300), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20349] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2238), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3302), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3304), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20418] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2239), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3306), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3308), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20487] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2240), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3310), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3312), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20556] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2241), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3314), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3316), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20625] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2242), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3318), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3320), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20694] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2243), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3322), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3324), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20763] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2244), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3326), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3328), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20832] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2245), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3330), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3332), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20901] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2246), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3334), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3336), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [20970] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2247), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3346), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3348), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21039] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2248), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3350), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3352), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21108] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2249), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3354), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3356), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21177] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2250), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3358), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3360), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21246] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2251), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3362), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3364), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21315] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2252), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3366), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3368), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21384] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2253), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3370), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3372), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21453] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2254), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3374), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3376), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21522] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2255), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3378), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3380), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21591] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2256), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3382), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3384), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21660] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2257), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3386), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3388), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21729] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2258), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3390), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3392), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21798] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2259), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3394), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3396), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21867] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2260), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3398), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3400), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [21936] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2261), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3402), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3404), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22005] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2262), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3406), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3408), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22074] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2263), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3410), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3412), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22143] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2264), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3414), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3416), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22212] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2265), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3418), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3420), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22281] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2266), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3422), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3424), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22350] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2267), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3426), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3428), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22419] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2268), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3430), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3432), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22488] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2269), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3434), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3436), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22557] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2270), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3438), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3440), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22626] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2271), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3442), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3444), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22695] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2272), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3446), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3448), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22764] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2273), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3450), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3452), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22833] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2274), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3454), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3456), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22902] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2275), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3458), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3460), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [22971] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2276), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3462), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3464), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23040] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2277), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3466), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3468), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23109] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2278), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3470), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3472), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23178] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2279), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3474), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3476), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23247] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2280), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3478), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3480), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23316] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2281), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3482), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3484), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23385] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2282), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3486), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3488), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23454] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2283), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3490), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3492), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23523] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2284), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3494), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3496), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23592] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2285), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3498), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3500), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23661] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2286), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3502), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3504), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23730] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2287), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3506), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3508), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23799] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2288), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3510), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3512), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23868] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2289), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3514), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3516), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [23937] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2290), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3518), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3520), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24006] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2291), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3522), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3524), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24075] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2292), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3526), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3528), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24144] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2293), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3530), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3532), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24213] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2294), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3534), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3536), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24282] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2295), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3538), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3540), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24351] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2296), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3542), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3544), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24420] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2297), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3546), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3548), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24489] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2298), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3550), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3552), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24558] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2299), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3554), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3556), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24627] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2300), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3558), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3560), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24696] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2301), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3562), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3564), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24765] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2302), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3566), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3568), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24834] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2303), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3570), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3572), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24903] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2304), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3574), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3576), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [24972] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2305), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3578), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3580), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25041] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2306), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3582), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3584), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25110] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2307), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3586), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3588), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25179] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2308), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3590), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3592), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25248] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2309), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3594), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3596), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25317] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2310), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3598), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3600), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25386] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2311), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3602), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3604), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25455] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2312), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3606), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3608), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25524] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2313), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3610), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3612), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25593] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2314), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3614), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3616), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25662] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2315), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3622), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3624), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25731] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2316), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3626), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3628), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25800] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2317), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3630), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3632), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25869] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2318), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3634), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3636), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [25938] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2319), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3638), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3640), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26007] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2320), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3642), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3644), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26076] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2321), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3646), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3648), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26145] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2322), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3650), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3652), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26214] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2323), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3654), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3656), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26283] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2324), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3658), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3660), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26352] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2325), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3662), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3664), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26421] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2326), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3666), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3668), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26490] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2327), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3670), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3672), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26559] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2328), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3674), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3676), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26628] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2329), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3678), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3680), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26697] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2330), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3682), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3684), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26766] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2331), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3686), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3688), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26835] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2332), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3690), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3692), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26904] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2333), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3694), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3696), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [26973] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2334), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3698), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3700), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27042] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2335), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3702), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3704), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27111] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2336), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3706), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3708), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27180] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2337), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3710), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3712), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27249] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2338), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3714), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3716), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27318] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2339), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3718), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3720), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27387] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2340), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3722), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3724), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27456] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2341), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3726), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3728), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27525] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2342), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3730), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3732), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27594] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2343), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3734), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3736), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27663] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2344), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3738), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3740), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27732] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2345), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3742), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3744), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27801] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2346), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3750), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3752), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27870] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2347), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3754), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3756), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [27939] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2348), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3758), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3760), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28008] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2349), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3762), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3764), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28077] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2350), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3766), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3768), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28146] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2351), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3770), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3772), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28215] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2352), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3774), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3776), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28284] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2353), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3778), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3780), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28353] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2354), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3782), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3784), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28422] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2355), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3786), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3788), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28491] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2356), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3790), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3792), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28560] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2357), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3794), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3796), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28629] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2358), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3798), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3800), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28698] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2359), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3802), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3804), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28767] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2360), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3806), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3808), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28836] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2361), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3810), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3812), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28905] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2362), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3814), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3816), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [28974] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2363), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3818), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3820), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29043] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2364), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3822), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3824), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29112] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2365), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3826), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3828), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29181] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2366), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3830), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3832), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29250] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2367), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3834), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3836), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29319] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2368), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3838), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3840), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29388] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2369), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3842), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3844), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29457] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2370), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2170), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2172), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29526] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2371), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3850), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3852), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29595] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2372), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3854), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3856), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29664] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2373), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3858), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3860), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29733] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2374), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3862), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3864), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29802] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2375), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3866), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3868), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29871] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2376), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3870), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3872), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [29940] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2377), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5180), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5182), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30009] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2378), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3874), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3876), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30078] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2379), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3878), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3880), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30147] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2380), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3882), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3884), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30216] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2381), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3886), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3888), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30285] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2382), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3890), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3892), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30354] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2383), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3894), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3896), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30423] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2384), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3898), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3900), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30492] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2385), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3902), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3904), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30561] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2386), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3906), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3908), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30630] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2387), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3910), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3912), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30699] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2388), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3914), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3916), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30768] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2389), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3918), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3920), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30837] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2390), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3922), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3924), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30906] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2391), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3930), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3932), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [30975] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2392), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3934), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3936), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31044] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2393), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3938), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3940), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31113] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2394), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3942), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3944), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31182] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2395), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3946), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3948), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31251] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2396), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3950), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3952), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31320] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2397), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3954), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3956), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31389] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2398), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3958), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3960), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31458] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2399), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3962), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3964), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31527] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2400), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3966), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3968), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31596] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2401), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3970), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3972), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31665] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2402), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3974), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3976), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31734] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2403), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3978), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3980), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31803] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2404), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3982), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3984), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31872] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2405), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3986), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3988), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [31941] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2406), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3990), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3992), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32010] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2407), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3994), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3996), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32079] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2408), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3998), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4000), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32148] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2409), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4002), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4004), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32217] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2410), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4006), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4008), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32286] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2411), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4010), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4012), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32355] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2412), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4014), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4016), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32424] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2413), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4018), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4020), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32493] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2414), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4022), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4024), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32562] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2415), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4026), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4028), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32631] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2416), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4030), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4032), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32700] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2417), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4034), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4036), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32769] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2418), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2210), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2212), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32838] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2419), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2214), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2216), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32907] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2420), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2218), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2220), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [32976] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2421), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2226), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2228), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33045] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2422), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5815), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5813), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33114] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2423), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4850), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4852), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33183] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2424), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2178), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2180), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33252] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2425), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2222), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2224), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33321] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2426), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1550), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1552), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33390] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2427), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4142), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4144), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33459] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2428), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2230), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2232), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33528] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2429), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2234), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2236), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33597] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2430), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4154), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4156), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33666] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2431), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4718), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4720), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33735] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2432), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4174), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4176), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33804] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2433), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2354), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2356), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33873] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2434), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2878), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2880), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [33942] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2435), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4510), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4512), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34011] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2436), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4690), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4692), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34080] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2437), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4706), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4708), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34149] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2438), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4842), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4844), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34218] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2439), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5082), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5084), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34287] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2440), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1538), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1540), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34356] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2441), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5086), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5088), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34425] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2442), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1606), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1608), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34494] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2443), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5090), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5092), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34563] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2444), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5102), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5104), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34632] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2445), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5116), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5118), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34701] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2446), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5124), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5126), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34770] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2447), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5128), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5130), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34839] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2448), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5132), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5134), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34908] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2449), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2338), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2340), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [34977] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2450), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4078), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4080), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35046] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2451), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4114), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4116), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35115] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2452), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4138), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4140), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35184] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2453), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4150), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4152), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35253] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2454), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1494), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1496), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35322] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2455), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1536), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1534), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35391] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2456), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2526), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2528), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35460] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2457), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2638), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2640), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35529] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2458), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1374), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1376), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35598] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2459), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1410), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1412), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35667] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2460), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1458), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1460), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35736] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2461), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1398), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1400), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35805] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2462), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1482), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1484), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35874] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2463), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3846), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3848), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [35943] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2464), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4090), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4092), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36012] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2465), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4094), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4096), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36081] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2466), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4098), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4100), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36150] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2467), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4102), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4104), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36219] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2468), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4106), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4108), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36288] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2469), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4110), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4112), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36357] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2470), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4118), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4120), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36426] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2471), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4122), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4124), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36495] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2472), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4126), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4128), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36564] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2473), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4130), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4132), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36633] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2474), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4134), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4136), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36702] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5829), 1,
      anon_sym_move,
    STATE(472), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(2475), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
    ACTIONS(5821), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(5817), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [36787] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2476), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4158), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4160), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36856] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2477), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4162), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4164), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36925] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2478), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4166), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4168), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [36994] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2479), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4170), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4172), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37063] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2480), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4178), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4180), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37132] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2481), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4190), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4192), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37201] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2482), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4194), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4196), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37270] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2483), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4198), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4200), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37339] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2484), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4202), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4204), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37408] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2485), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4206), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4208), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37477] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2486), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4210), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4212), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37546] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2487), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4214), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4216), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37615] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2488), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4218), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4220), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37684] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2489), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4222), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4224), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37753] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2490), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4226), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4228), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37822] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2491), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4230), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4232), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37891] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2492), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4234), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4236), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [37960] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2493), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4238), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4240), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38029] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2494), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4242), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4244), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38098] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2495), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4246), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4248), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38167] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2496), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4270), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4272), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38236] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2497), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4322), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4324), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38305] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2498), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4326), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4328), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38374] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2499), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4330), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4332), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38443] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2500), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4334), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4336), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38512] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2501), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4342), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4344), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38581] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2502), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4346), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4348), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38650] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2503), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4350), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4352), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38719] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2504), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4354), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4356), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38788] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2505), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4358), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4360), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38857] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2506), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4370), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4372), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38926] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2507), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4374), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4376), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [38995] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2508), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4382), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4384), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39064] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2509), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4386), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4388), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39133] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2510), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4390), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4392), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39202] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2511), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4394), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4396), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39271] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2512), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4398), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4400), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39340] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2513), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4402), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4404), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39409] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2514), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4406), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4408), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39478] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2515), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4410), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4412), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39547] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2516), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4414), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4416), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39616] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2517), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4418), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4420), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39685] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2518), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4426), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4428), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39754] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2519), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4478), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4480), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39823] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2520), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4482), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4484), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39892] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2521), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4498), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4500), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [39961] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2522), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4502), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4504), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40030] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2523), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1502), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(1504), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40099] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2524), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3618), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3620), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40168] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2525), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4558), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4560), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40237] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2526), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4562), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4564), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40306] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2527), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4566), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4568), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40375] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2528), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4570), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4572), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40444] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2529), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4574), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4576), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40513] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2530), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4586), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4588), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40582] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2531), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2238), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2240), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40651] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2532), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4594), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4596), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40720] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2533), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4598), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4600), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40789] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2534), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4602), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4604), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40858] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2535), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4606), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4608), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40927] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2536), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4610), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4612), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [40996] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2537), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4622), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4624), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41065] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2538), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4626), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4628), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41134] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2539), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4630), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4632), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41203] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2540), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4646), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4648), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41272] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2541), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4650), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4652), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41341] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2542), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4654), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4656), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41410] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2543), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4658), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4660), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41479] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2544), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4666), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4668), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41548] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2545), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4670), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4672), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41617] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2546), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4702), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4704), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41686] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2547), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4726), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4728), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41755] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2548), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4730), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4732), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41824] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2549), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4734), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4736), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41893] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2550), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4738), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4740), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [41962] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2551), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4750), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4752), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42031] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2552), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4754), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4756), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42100] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2553), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4762), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4764), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42169] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2554), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4770), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4772), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42238] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2555), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4774), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4776), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42307] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2556), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4778), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4780), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42376] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2557), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4782), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4784), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42445] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2558), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4786), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4788), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42514] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2559), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4798), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4800), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42583] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2560), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4802), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4804), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42652] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2561), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4806), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4808), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42721] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2562), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4810), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4812), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42790] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2563), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4818), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4820), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42859] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2564), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4834), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4836), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42928] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2565), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4838), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4840), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [42997] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2566), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4854), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4856), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43066] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2567), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4858), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4860), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43135] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2568), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4862), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4864), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43204] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2569), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4866), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4868), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43273] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2570), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4870), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4872), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43342] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2571), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4874), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4876), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43411] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2572), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4878), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4880), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43480] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2573), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4882), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4884), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43549] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2574), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4886), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4888), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43618] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2575), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4890), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4892), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43687] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2576), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4894), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4896), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43756] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2577), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4902), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4904), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43825] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2578), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4906), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4908), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43894] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2579), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4910), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4912), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [43963] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2580), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4914), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4916), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44032] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2581), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4918), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4920), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44101] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2582), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4922), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4924), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44170] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2583), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4926), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4928), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44239] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2584), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4930), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4932), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44308] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2585), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4934), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4936), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44377] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2586), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4938), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4940), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44446] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2587), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4942), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4944), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44515] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2588), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4946), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4948), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44584] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2589), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4950), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4952), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44653] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2590), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4954), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4956), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44722] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2591), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4958), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4960), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44791] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2592), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4962), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4964), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44860] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2593), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4966), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4968), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44929] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2594), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4970), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4972), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [44998] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2595), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4974), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4976), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45067] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2596), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4978), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4980), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45136] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2597), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4982), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4984), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45205] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2598), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4986), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4988), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45274] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2599), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4990), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4992), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45343] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2600), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4994), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4996), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45412] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2601), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4998), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5000), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45481] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2602), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5002), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5004), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45550] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2603), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5006), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5008), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45619] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2604), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5010), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5012), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45688] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2605), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5014), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5016), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45757] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2606), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5022), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5024), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45826] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2607), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5026), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5028), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45895] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2608), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5030), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5032), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [45964] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2609), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5038), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5040), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46033] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2610), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5042), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5044), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46102] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2611), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5046), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5048), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46171] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2612), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5050), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5052), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46240] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2613), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5054), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5056), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46309] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2614), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5058), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5060), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46378] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2615), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5062), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5064), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46447] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2616), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5066), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5068), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46516] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2617), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5070), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5072), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46585] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2618), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5074), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5076), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46654] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2619), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5078), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5080), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46723] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2620), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5120), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5122), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46792] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2621), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5152), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5154), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46861] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2622), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5160), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5162), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46930] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2623), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5164), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5166), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [46999] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2624), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2190), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2192), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47068] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2625), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2194), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2196), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47137] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2626), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2198), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2200), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47206] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2627), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2322), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2324), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47275] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2628), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2342), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2344), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47344] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2629), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2346), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2348), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47413] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2630), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2350), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2352), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47482] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2631), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2542), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2544), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47551] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2632), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2778), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2780), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47620] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2633), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2874), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2876), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47689] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2634), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2970), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2972), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47758] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2635), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3098), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3100), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47827] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2636), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3338), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3340), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47896] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2637), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(3926), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(3928), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [47965] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2638), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4038), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4040), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48034] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2639), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4042), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4044), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48103] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2640), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4046), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4048), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48172] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2641), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4050), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4052), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48241] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2642), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4054), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4056), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48310] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2643), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4058), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4060), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48379] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2644), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4062), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4064), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48448] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2645), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4066), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4068), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48517] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2646), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4082), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4084), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48586] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2647), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4086), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4088), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48655] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2648), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4146), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4148), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48724] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2649), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4182), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4184), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48793] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2650), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4186), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4188), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48862] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2651), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4250), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4252), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [48931] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2652), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4254), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4256), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49000] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2653), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4258), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4260), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49069] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2654), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4262), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4264), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49138] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2655), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4266), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4268), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49207] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2656), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4274), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4276), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49276] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2657), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4278), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4280), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49345] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2658), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4282), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4284), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49414] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2659), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4286), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4288), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49483] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2660), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4290), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4292), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49552] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2661), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4294), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4296), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49621] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2662), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4298), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4300), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49690] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2663), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4302), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4304), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49759] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2664), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4306), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4308), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49828] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2665), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4310), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4312), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49897] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2666), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4314), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4316), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [49966] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2667), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4318), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4320), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50035] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2668), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4338), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4340), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50104] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2669), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4362), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4364), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50173] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2670), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4366), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4368), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50242] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2671), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4378), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4380), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50311] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2672), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4422), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4424), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50380] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2673), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4430), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4432), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50449] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2674), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4434), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4436), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50518] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2675), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4438), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4440), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50587] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2676), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4442), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4444), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50656] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2677), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4446), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4448), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50725] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2678), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4450), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4452), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50794] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2679), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4454), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4456), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50863] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2680), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4458), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4460), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [50932] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2681), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4462), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4464), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51001] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2682), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4466), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4468), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51070] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2683), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4470), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4472), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51139] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2684), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4474), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4476), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51208] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2685), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4486), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4488), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51277] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2686), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4490), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4492), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51346] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2687), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4494), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4496), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51415] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2688), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4506), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4508), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51484] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2689), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4514), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4516), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51553] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2690), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4518), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4520), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51622] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2691), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4522), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4524), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51691] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2692), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4526), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4528), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51760] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2693), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4530), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4532), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51829] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2694), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4534), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4536), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51898] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2695), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4538), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4540), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [51967] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2696), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4542), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4544), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52036] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2697), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4546), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4548), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52105] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2698), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4550), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4552), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52174] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2699), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4554), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4556), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52243] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2700), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4578), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4580), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52312] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2701), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4582), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4584), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52381] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2702), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4590), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4592), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52450] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2703), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4614), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4616), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52519] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2704), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4618), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4620), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52588] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2705), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4634), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4636), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52657] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2706), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4638), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4640), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52726] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2707), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4642), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4644), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52795] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2708), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4662), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4664), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52864] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2709), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4674), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4676), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [52933] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2710), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4678), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4680), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53002] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2711), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4682), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4684), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53071] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2712), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4686), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4688), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53140] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2713), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4694), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4696), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53209] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2714), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4698), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4700), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53278] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2715), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4710), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4712), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53347] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2716), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4714), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4716), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53416] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2717), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4722), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4724), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53485] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2718), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4742), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4744), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53554] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2719), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4746), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4748), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53623] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2720), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4758), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4760), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53692] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2721), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4766), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4768), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53761] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2722), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4790), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4792), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53830] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2723), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4794), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4796), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53899] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2724), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4814), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4816), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [53968] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2725), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4822), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4824), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54037] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2726), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4826), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4828), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54106] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2727), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4830), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4832), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54175] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2728), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4846), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4848), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54244] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2729), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4898), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(4900), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54313] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2730), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5018), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5020), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54382] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2731), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5034), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5036), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54451] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2732), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5094), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5096), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54520] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2733), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5098), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5100), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54589] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2734), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5136), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5138), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54658] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2735), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5140), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5142), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54727] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2736), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5144), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5146), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54796] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2737), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5148), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5150), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54865] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2738), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5156), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5158), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [54934] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2739), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5168), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5170), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55003] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2740), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5172), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5174), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55072] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2741), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5176), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(5178), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55141] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2742), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2174), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2176), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55210] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2743), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2182), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2184), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55279] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2744), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2186), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2188), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55348] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2745), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2242), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2244), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55417] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2746), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2498), 8,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      aux_sym_metavariable_token1,
    ACTIONS(2500), 46,
      anon_sym_macro,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_let,
      anon_sym_mod,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55486] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(1670), 1,
      anon_sym_DASH,
    ACTIONS(1696), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1702), 1,
      sym__raw_string_literal_start,
    ACTIONS(5673), 1,
      anon_sym_if,
    ACTIONS(5831), 1,
      sym_identifier,
    ACTIONS(5835), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5839), 1,
      aux_sym_metavariable_token1,
    STATE(4696), 1,
      sym_metavariable,
    STATE(4700), 1,
      sym_scoped_identifier,
    STATE(5313), 1,
      sym__literal_pattern,
    STATE(6216), 1,
      sym_bracketed_type,
    STATE(6233), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1698), 2,
      anon_sym_true,
      anon_sym_false,
    ACTIONS(5669), 2,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
    STATE(2747), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1694), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5837), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(4104), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5833), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [55588] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2748), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5218), 13,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5216), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55656] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2749), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5238), 13,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5236), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55724] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2750), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5222), 13,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5220), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55792] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2751), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5230), 13,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5228), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [55860] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(1670), 1,
      anon_sym_DASH,
    ACTIONS(1696), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1702), 1,
      sym__raw_string_literal_start,
    ACTIONS(5733), 1,
      anon_sym_if,
    ACTIONS(5835), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5839), 1,
      aux_sym_metavariable_token1,
    ACTIONS(5841), 1,
      sym_identifier,
    STATE(4577), 1,
      sym_metavariable,
    STATE(4578), 1,
      sym_scoped_identifier,
    STATE(4914), 1,
      sym__literal_pattern,
    STATE(6216), 1,
      sym_bracketed_type,
    STATE(6233), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1698), 2,
      anon_sym_true,
      anon_sym_false,
    ACTIONS(5729), 2,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
    STATE(2752), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1694), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5845), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(4104), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5843), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [55962] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2753), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5234), 13,
      sym__outer_block_doc_comment_start,
      sym__outer_line_doc_comment_start,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_POUND,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5232), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_pub,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56030] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(1670), 1,
      anon_sym_DASH,
    ACTIONS(1696), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1702), 1,
      sym__raw_string_literal_start,
    ACTIONS(5723), 1,
      anon_sym_if,
    ACTIONS(5835), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5839), 1,
      aux_sym_metavariable_token1,
    ACTIONS(5847), 1,
      sym_identifier,
    STATE(4589), 1,
      sym_metavariable,
    STATE(4592), 1,
      sym_scoped_identifier,
    STATE(4947), 1,
      sym__literal_pattern,
    STATE(6216), 1,
      sym_bracketed_type,
    STATE(6233), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1698), 2,
      anon_sym_true,
      anon_sym_false,
    ACTIONS(5719), 2,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
    STATE(2754), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1694), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5851), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(4104), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5849), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [56132] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2755), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5855), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5853), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56199] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2756), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5859), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5857), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56266] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2757), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5863), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5861), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56333] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2758), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5867), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5865), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56400] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2759), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5871), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5869), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56467] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2760), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5875), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5873), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56534] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2761), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5879), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5877), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56601] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2762), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5883), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5881), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56668] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2763), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5887), 12,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5885), 40,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_where,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [56735] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5889), 1,
      anon_sym_static,
    STATE(2764), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
    ACTIONS(5821), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(5817), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [56808] = 20,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(925), 1,
      anon_sym_DASH,
    ACTIONS(973), 1,
      aux_sym_string_literal_token1,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(983), 1,
      sym__raw_string_literal_start,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5727), 1,
      sym_identifier,
    STATE(3716), 1,
      sym_metavariable,
    STATE(3717), 1,
      sym_scoped_identifier,
    STATE(3755), 1,
      sym__literal_pattern,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(975), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(2765), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(971), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5735), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3672), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5731), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [56903] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5895), 1,
      anon_sym_BANG,
    ACTIONS(5899), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2812), 1,
      sym_parameters,
    STATE(2839), 1,
      sym_type_arguments,
    STATE(2766), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5897), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5891), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [56980] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5903), 1,
      anon_sym_static,
    STATE(2767), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
    ACTIONS(5821), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(5817), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [57053] = 20,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(1670), 1,
      anon_sym_DASH,
    ACTIONS(1696), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1702), 1,
      sym__raw_string_literal_start,
    ACTIONS(5835), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5839), 1,
      aux_sym_metavariable_token1,
    ACTIONS(5841), 1,
      sym_identifier,
    STATE(4577), 1,
      sym_metavariable,
    STATE(4578), 1,
      sym_scoped_identifier,
    STATE(4914), 1,
      sym__literal_pattern,
    STATE(6216), 1,
      sym_bracketed_type,
    STATE(6233), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1698), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(2768), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1694), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5845), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(4104), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5843), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [57148] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5895), 1,
      anon_sym_BANG,
    ACTIONS(5899), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2812), 1,
      sym_parameters,
    STATE(2839), 1,
      sym_type_arguments,
    STATE(2769), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5907), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5905), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [57225] = 20,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(925), 1,
      anon_sym_DASH,
    ACTIONS(973), 1,
      aux_sym_string_literal_token1,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(983), 1,
      sym__raw_string_literal_start,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5667), 1,
      sym_identifier,
    STATE(3728), 1,
      sym_metavariable,
    STATE(3729), 1,
      sym_scoped_identifier,
    STATE(3772), 1,
      sym__literal_pattern,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(975), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(2770), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(971), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5675), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3672), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5671), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [57320] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(5911), 1,
      anon_sym_BANG,
    ACTIONS(5915), 1,
      anon_sym_COLON_COLON,
    STATE(2812), 1,
      sym_parameters,
    STATE(2839), 1,
      sym_type_arguments,
    STATE(2771), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5913), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5909), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [57397] = 20,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(925), 1,
      anon_sym_DASH,
    ACTIONS(973), 1,
      aux_sym_string_literal_token1,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(983), 1,
      sym__raw_string_literal_start,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5717), 1,
      sym_identifier,
    STATE(3724), 1,
      sym_metavariable,
    STATE(3725), 1,
      sym_scoped_identifier,
    STATE(3762), 1,
      sym__literal_pattern,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(975), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(2772), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(971), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5725), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(3672), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5721), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [57492] = 20,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(1670), 1,
      anon_sym_DASH,
    ACTIONS(1696), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1702), 1,
      sym__raw_string_literal_start,
    ACTIONS(5831), 1,
      sym_identifier,
    ACTIONS(5835), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5839), 1,
      aux_sym_metavariable_token1,
    STATE(4696), 1,
      sym_metavariable,
    STATE(4700), 1,
      sym_scoped_identifier,
    STATE(5313), 1,
      sym__literal_pattern,
    STATE(6216), 1,
      sym_bracketed_type,
    STATE(6233), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1698), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(2773), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1694), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5837), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(4104), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5833), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [57587] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(5919), 1,
      anon_sym_move,
    STATE(2930), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(2774), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [57666] = 20,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(1670), 1,
      anon_sym_DASH,
    ACTIONS(1696), 1,
      aux_sym_string_literal_token1,
    ACTIONS(1702), 1,
      sym__raw_string_literal_start,
    ACTIONS(5835), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5839), 1,
      aux_sym_metavariable_token1,
    ACTIONS(5847), 1,
      sym_identifier,
    STATE(4589), 1,
      sym_metavariable,
    STATE(4592), 1,
      sym_scoped_identifier,
    STATE(4947), 1,
      sym__literal_pattern,
    STATE(6216), 1,
      sym_bracketed_type,
    STATE(6233), 1,
      sym_generic_type_with_turbofish,
    ACTIONS(1698), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(2775), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1694), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    ACTIONS(5851), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(4104), 4,
      sym_negative_literal,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    ACTIONS(5849), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [57761] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2776), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5923), 10,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5921), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [57825] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5929), 1,
      anon_sym_COLON_COLON,
    STATE(2777), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5927), 9,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5925), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [57891] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2778), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5855), 10,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5853), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [57955] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    STATE(2934), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(2779), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [58031] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(5936), 1,
      anon_sym_COLON_COLON,
    STATE(2812), 1,
      sym_parameters,
    STATE(2839), 1,
      sym_type_arguments,
    STATE(2780), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5934), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5932), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [58105] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5938), 1,
      anon_sym_LPAREN,
    STATE(2781), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5927), 9,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5925), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [58171] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    STATE(2782), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
    ACTIONS(5821), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(5817), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [58241] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2783), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5943), 10,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5941), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [58305] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2784), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5947), 10,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5945), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [58369] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2785), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5887), 10,
      anon_sym_LPAREN,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_STAR,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      aux_sym_metavariable_token1,
    ACTIONS(5885), 39,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_for,
      anon_sym_gen,
      anon_sym_impl,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      anon_sym_dyn,
      anon_sym_try,
      sym_identifier,
      sym_self,
      sym_super,
      sym_crate,
  [58433] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2809), 1,
      sym_type_arguments,
    STATE(2859), 1,
      sym_parameters,
    STATE(2786), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [58504] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2809), 1,
      sym_type_arguments,
    STATE(2859), 1,
      sym_parameters,
    STATE(2787), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5955), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5953), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [58575] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 1,
      anon_sym_BANG,
    ACTIONS(5963), 1,
      anon_sym_COLON_COLON,
    STATE(2788), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5961), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5957), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [58642] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 1,
      anon_sym_BANG,
    ACTIONS(5971), 1,
      anon_sym_COLON_COLON,
    STATE(2789), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5969), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5965), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [58709] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2809), 1,
      sym_type_arguments,
    STATE(2859), 1,
      sym_parameters,
    STATE(2790), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5975), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5973), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [58780] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 1,
      anon_sym_BANG,
    ACTIONS(5983), 1,
      anon_sym_COLON_COLON,
    STATE(2791), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5981), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5977), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [58847] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 1,
      anon_sym_BANG,
    ACTIONS(5991), 1,
      anon_sym_COLON_COLON,
    STATE(2792), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5989), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5985), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [58914] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2809), 1,
      sym_type_arguments,
    STATE(2859), 1,
      sym_parameters,
    STATE(2793), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5995), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5993), 27,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [58985] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(6001), 1,
      anon_sym_COLON_COLON,
    STATE(2794), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5999), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5997), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59051] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2795), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5967), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5971), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59113] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2796), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5987), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5991), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59175] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5911), 1,
      anon_sym_BANG,
    ACTIONS(6003), 1,
      anon_sym_LBRACE,
    ACTIONS(6005), 1,
      anon_sym_COLON_COLON,
    STATE(2871), 1,
      sym_field_initializer_list,
    STATE(2797), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [59245] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2798), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5979), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5983), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59307] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2799), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5959), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5963), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59369] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(6007), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(2800), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [59437] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6009), 1,
      anon_sym_COLON_COLON,
    STATE(2801), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5955), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5953), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59500] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2802), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5969), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5965), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
      anon_sym_LT2,
  [59561] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2803), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1394), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1396), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_fn,
      anon_sym_else,
  [59622] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2804), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6013), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6011), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59683] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2805), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6017), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6015), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [59744] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6019), 1,
      sym_identifier,
    ACTIONS(6023), 1,
      anon_sym_BANG,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    STATE(5423), 1,
      sym_scoped_type_identifier,
    STATE(5456), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2806), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59831] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2807), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1454), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1456), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_fn,
      anon_sym_else,
  [59892] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6033), 1,
      sym_identifier,
    ACTIONS(6035), 1,
      anon_sym_BANG,
    STATE(5190), 1,
      sym_scoped_type_identifier,
    STATE(5752), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2808), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [59979] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2809), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6039), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6037), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60040] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6009), 1,
      anon_sym_COLON_COLON,
    STATE(2810), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5995), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5993), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60103] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6041), 1,
      anon_sym_COLON_COLON,
    STATE(2811), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60166] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6047), 1,
      anon_sym_DASH_GT,
    STATE(2812), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6045), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6043), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60229] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2891), 1,
      sym_label,
    STATE(2813), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6051), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6049), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [60294] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6053), 1,
      sym_identifier,
    ACTIONS(6055), 1,
      anon_sym_BANG,
    STATE(5083), 1,
      sym_scoped_type_identifier,
    STATE(5546), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2814), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [60381] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2815), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6059), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6057), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60442] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6063), 1,
      anon_sym_LPAREN,
    STATE(2894), 1,
      sym_arguments,
    STATE(2816), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6065), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6061), 29,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60507] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2817), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1378), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1380), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_fn,
      anon_sym_else,
  [60568] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6067), 1,
      anon_sym_LBRACE,
    STATE(2818), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5987), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5991), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [60631] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2819), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6071), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6069), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60692] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6073), 1,
      anon_sym_COLON_COLON,
    STATE(2820), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60755] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6075), 1,
      anon_sym_LBRACE,
    STATE(2821), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5967), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5971), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [60818] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2822), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6079), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6077), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60879] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2823), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6083), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6081), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [60940] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6085), 1,
      sym_identifier,
    ACTIONS(6087), 1,
      anon_sym_BANG,
    STATE(4865), 1,
      sym_scoped_type_identifier,
    STATE(5469), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2824), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [61027] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2825), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6091), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6089), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [61088] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2826), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6095), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6093), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [61149] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6101), 1,
      anon_sym_DASH_GT,
    STATE(2827), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6099), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6097), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [61212] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2828), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6105), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6103), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [61273] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6111), 1,
      anon_sym_RBRACE,
    ACTIONS(6113), 1,
      anon_sym_COMMA,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(4868), 1,
      sym__use_clause,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2829), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [61362] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6123), 1,
      sym_identifier,
    ACTIONS(6125), 1,
      anon_sym_BANG,
    STATE(5195), 1,
      sym_scoped_type_identifier,
    STATE(5754), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2830), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [61449] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6127), 1,
      sym_identifier,
    ACTIONS(6129), 1,
      anon_sym_BANG,
    STATE(4909), 1,
      sym_scoped_type_identifier,
    STATE(5703), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2831), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [61536] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6135), 1,
      anon_sym_DASH_GT,
    STATE(2832), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6133), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6131), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [61599] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6009), 1,
      anon_sym_COLON_COLON,
    STATE(2833), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5975), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5973), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [61662] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(6137), 1,
      anon_sym_move,
    STATE(590), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(2834), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [61737] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6009), 1,
      anon_sym_COLON_COLON,
    STATE(2835), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [61800] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6139), 1,
      anon_sym_else,
    STATE(2885), 1,
      sym_else_clause,
    STATE(2836), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1352), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1350), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [61865] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6141), 1,
      anon_sym_LBRACE,
    STATE(2837), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5959), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5963), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [61928] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6143), 1,
      sym_identifier,
    ACTIONS(6145), 1,
      anon_sym_BANG,
    STATE(5439), 1,
      sym_scoped_type_identifier,
    STATE(5700), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2838), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [62015] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2839), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6149), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6147), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62076] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6151), 1,
      anon_sym_COLON_COLON,
    STATE(2840), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62139] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6153), 1,
      anon_sym_LBRACE,
    STATE(2841), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5979), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5983), 29,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
      anon_sym_else,
  [62202] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6155), 1,
      sym_identifier,
    ACTIONS(6157), 1,
      anon_sym_BANG,
    STATE(5234), 1,
      sym_scoped_type_identifier,
    STATE(5781), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2842), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [62289] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2843), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6161), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6159), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62350] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2844), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6165), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6163), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62411] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2845), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1638), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1640), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62472] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2846), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6169), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6167), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62533] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2847), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5645), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5647), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62594] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2848), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6173), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6171), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62655] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2849), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1356), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1358), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [62716] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6175), 1,
      sym_identifier,
    ACTIONS(6177), 1,
      anon_sym_BANG,
    STATE(5353), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5884), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2850), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [62803] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6179), 1,
      sym_identifier,
    ACTIONS(6181), 1,
      anon_sym_BANG,
    STATE(5370), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5893), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2851), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [62890] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6183), 1,
      sym_identifier,
    ACTIONS(6185), 1,
      anon_sym_BANG,
    STATE(5368), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5892), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2852), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [62977] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6187), 1,
      sym_identifier,
    ACTIONS(6189), 1,
      anon_sym_BANG,
    STATE(5375), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5895), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2853), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [63064] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2854), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6193), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6191), 31,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63125] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6195), 1,
      sym_identifier,
    ACTIONS(6197), 1,
      anon_sym_BANG,
    STATE(5388), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5907), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2855), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [63212] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6199), 1,
      sym_identifier,
    ACTIONS(6201), 1,
      anon_sym_BANG,
    STATE(5392), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5909), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2856), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [63299] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6203), 1,
      sym_identifier,
    ACTIONS(6205), 1,
      anon_sym_BANG,
    STATE(5390), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5908), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2857), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [63386] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6207), 1,
      sym_identifier,
    ACTIONS(6209), 1,
      anon_sym_BANG,
    STATE(5403), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5918), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2858), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6029), 5,
      anon_sym_async,
      anon_sym_const,
      anon_sym_fn,
      anon_sym_unsafe,
      anon_sym_extern,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [63473] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6215), 1,
      anon_sym_DASH_GT,
    STATE(2859), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6213), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6211), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63536] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2860), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1548), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1546), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63596] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2861), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1500), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1498), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63656] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2862), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1460), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1458), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63716] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2863), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1496), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1494), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63776] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2864), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6219), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6217), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63836] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2865), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6223), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6221), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [63896] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6225), 1,
      anon_sym_BANG,
    ACTIONS(6227), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6229), 1,
      anon_sym_move,
    STATE(3262), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(2866), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [63970] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2867), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6233), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6231), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64030] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2868), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6237), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6235), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64090] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    STATE(2869), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6243), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 27,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_else,
  [64158] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2870), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6253), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64218] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2871), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6257), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6255), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64278] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2872), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6261), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6259), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64338] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2873), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6265), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6263), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64398] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2874), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6269), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6267), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64458] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2875), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6273), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6271), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64518] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2876), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6277), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6275), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64578] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2877), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64638] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2878), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6281), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6279), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64698] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2879), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6283), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64758] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2880), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6289), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6287), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64818] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2881), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6065), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6061), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64878] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2882), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1382), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1384), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [64938] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6103), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(2883), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [65000] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2884), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6297), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6295), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65060] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2885), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1528), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1526), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65120] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2886), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65180] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2887), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1532), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1530), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65240] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    STATE(2888), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6301), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6299), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65306] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2889), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6305), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6303), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65366] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2890), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6309), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6307), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65426] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2891), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6313), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6311), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65486] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2892), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6317), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6315), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65546] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2893), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6321), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6319), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65606] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2894), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6325), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6323), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65666] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2895), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1376), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1374), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65726] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    STATE(2896), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6329), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6327), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65792] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2897), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6333), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6331), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65852] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2898), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2360), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(2358), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65912] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2899), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [65972] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2900), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6337), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6335), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66032] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2901), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6341), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6339), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66092] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6343), 1,
      anon_sym_RBRACE,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(5567), 1,
      sym__use_clause,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2902), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [66178] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2903), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6347), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6345), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66238] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2904), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1484), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1482), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66298] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2905), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6351), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6349), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66358] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2906), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66418] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2907), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6355), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6353), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66478] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2908), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6359), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6357), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66538] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2909), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6363), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6361), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66598] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2910), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1544), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1542), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66658] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2911), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6367), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6365), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66718] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2912), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66778] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2913), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6371), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6369), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66838] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2914), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1534), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1536), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66898] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2915), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1440), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1442), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [66958] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    STATE(2916), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6375), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6373), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67024] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2917), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67084] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    STATE(581), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(2918), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [67156] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2919), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6379), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6377), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67216] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2920), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6383), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6381), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67276] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2921), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67336] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2922), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6387), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6385), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67396] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2923), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6391), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6389), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67456] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2924), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6395), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6393), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67516] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2925), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5955), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5953), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67576] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2926), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6399), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6397), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67636] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2927), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6403), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6401), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67696] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2928), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1616), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1614), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67756] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2929), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6407), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6405), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67816] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2930), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1560), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1558), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67876] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2931), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6411), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6409), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67936] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2932), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1580), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1578), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [67996] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2933), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1588), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1586), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68056] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2934), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1604), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1602), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68116] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2935), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6415), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6413), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68176] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2936), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6419), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6417), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68236] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2937), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1572), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1570), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68296] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2938), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1576), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1574), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68356] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2939), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6423), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6421), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68416] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2940), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6427), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6425), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68476] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2941), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68536] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2942), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6431), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6429), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68596] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2943), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1584), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1582), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68656] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    STATE(2944), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [68720] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2945), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6435), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6433), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68780] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5911), 1,
      anon_sym_BANG,
    ACTIONS(6437), 1,
      anon_sym_COLON_COLON,
    STATE(2946), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [68844] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2947), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6441), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6439), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68904] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2948), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6445), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6443), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [68964] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2949), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6449), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6447), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69024] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2950), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6453), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6451), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69084] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2951), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1412), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1410), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69144] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2952), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1540), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1538), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69204] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2953), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6457), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6455), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69264] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2954), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6461), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6459), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69324] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2955), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6465), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6463), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69384] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2956), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6469), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6467), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69444] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2957), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6473), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6471), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69504] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2958), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6477), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6475), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69564] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2959), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6481), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6479), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69624] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2960), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5975), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5973), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69684] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2961), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5995), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5993), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69744] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2962), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1564), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1562), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69804] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6483), 1,
      anon_sym_RBRACE,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(5567), 1,
      sym__use_clause,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2963), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [69890] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2964), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1568), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1566), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [69950] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6089), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(2965), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [70012] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2966), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6487), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6485), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70072] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2967), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1516), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1514), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70132] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2968), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1592), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1590), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70192] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2969), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6491), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6489), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70252] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2970), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6495), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6493), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70312] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(484), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(2971), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [70384] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2972), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1400), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1398), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70444] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2973), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1596), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1594), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70504] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2974), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6499), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6497), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70564] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2975), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6503), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6501), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70624] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2976), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1600), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1598), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70684] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2977), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6507), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6505), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70744] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2978), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6511), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6509), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70804] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2979), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6515), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6513), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70864] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2980), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6519), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6517), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70924] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2981), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6523), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6521), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [70984] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2982), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6527), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6525), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [71044] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2983), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6531), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6529), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [71104] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2984), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1608), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1606), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [71164] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    STATE(2985), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6535), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6533), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [71230] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2986), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6539), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6537), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [71290] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2987), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1556), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1554), 30,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_else,
  [71350] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6033), 1,
      sym_identifier,
    STATE(5190), 1,
      sym_scoped_type_identifier,
    STATE(5752), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2988), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [71435] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6143), 1,
      sym_identifier,
    STATE(5439), 1,
      sym_scoped_type_identifier,
    STATE(5700), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2989), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [71520] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6155), 1,
      sym_identifier,
    STATE(5234), 1,
      sym_scoped_type_identifier,
    STATE(5781), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2990), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [71605] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_COLON_COLON,
    STATE(2991), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [71666] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6127), 1,
      sym_identifier,
    STATE(4909), 1,
      sym_scoped_type_identifier,
    STATE(5703), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(2992), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [71751] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(5567), 1,
      sym__use_clause,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2993), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [71834] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(5960), 1,
      sym__use_clause,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2994), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [71917] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(5919), 1,
      anon_sym_move,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    STATE(2930), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(2995), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [71986] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(6477), 1,
      sym__use_clause,
    STATE(2996), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [72069] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(6009), 1,
      sym__use_clause,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2997), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [72152] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6218), 1,
      sym__use_clause,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2998), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [72235] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6256), 1,
      sym__use_clause,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(2999), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [72318] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6123), 1,
      sym_identifier,
    STATE(5195), 1,
      sym_scoped_type_identifier,
    STATE(5754), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3000), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [72403] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(6382), 1,
      sym__use_clause,
    STATE(3001), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [72486] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6053), 1,
      sym_identifier,
    STATE(5083), 1,
      sym_scoped_type_identifier,
    STATE(5546), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3002), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [72571] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6019), 1,
      sym_identifier,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    STATE(5423), 1,
      sym_scoped_type_identifier,
    STATE(5456), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3003), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [72656] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6085), 1,
      sym_identifier,
    STATE(4865), 1,
      sym_scoped_type_identifier,
    STATE(5469), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3004), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [72741] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6225), 1,
      anon_sym_BANG,
    ACTIONS(6227), 1,
      anon_sym_COLON_COLON,
    STATE(3278), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(3005), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [72812] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(5919), 1,
      anon_sym_move,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    STATE(2930), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(3006), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [72883] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(6325), 1,
      sym__use_clause,
    STATE(3007), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [72966] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6107), 1,
      sym_identifier,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(6117), 1,
      anon_sym_STAR,
    ACTIONS(6119), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 1,
      sym_metavariable,
    STATE(4383), 1,
      sym_scoped_identifier,
    STATE(6086), 1,
      sym__use_clause,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(3008), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6121), 3,
      sym_self,
      sym_super,
      sym_crate,
    STATE(5040), 4,
      sym_scoped_use_list,
      sym_use_list,
      sym_use_as_clause,
      sym_use_wildcard,
    ACTIONS(6115), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [73049] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    STATE(3009), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 28,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [73110] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6175), 1,
      sym_identifier,
    STATE(5353), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5884), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3010), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73195] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6179), 1,
      sym_identifier,
    STATE(5370), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5893), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3011), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73280] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6183), 1,
      sym_identifier,
    STATE(5368), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5892), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3012), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73365] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6187), 1,
      sym_identifier,
    STATE(5375), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5895), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3013), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73450] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6195), 1,
      sym_identifier,
    STATE(5388), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5907), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3014), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73535] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6199), 1,
      sym_identifier,
    STATE(5392), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5909), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3015), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73620] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6203), 1,
      sym_identifier,
    STATE(5390), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5908), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3016), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73705] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(2360), 1,
      anon_sym_where,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6207), 1,
      sym_identifier,
    STATE(5403), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5918), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3017), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(2358), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [73790] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3018), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6275), 7,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_else,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [73882] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3019), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [73954] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3020), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [74030] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6575), 1,
      anon_sym_EQ,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3021), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6573), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_else,
  [74120] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    STATE(3022), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [74188] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3023), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [74266] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(394), 1,
      anon_sym_EQ,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3024), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(384), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_else,
  [74356] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3025), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6315), 7,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_else,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [74448] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6579), 1,
      anon_sym_EQ,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3026), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6577), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_else,
  [74538] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6583), 1,
      anon_sym_EQ,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3027), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6581), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_else,
  [74628] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    STATE(2934), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(3028), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [74696] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3029), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 21,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [74778] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6003), 1,
      anon_sym_LBRACE,
    ACTIONS(6005), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6585), 1,
      anon_sym_BANG,
    STATE(2871), 1,
      sym_field_initializer_list,
    STATE(3030), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [74844] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6589), 1,
      anon_sym_BANG,
    ACTIONS(6591), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    STATE(3134), 1,
      sym_parameters,
    STATE(3177), 1,
      sym_type_arguments,
    STATE(3031), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5913), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5909), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [74914] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5895), 1,
      anon_sym_BANG,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(6595), 1,
      anon_sym_COLON_COLON,
    STATE(3134), 1,
      sym_parameters,
    STATE(3177), 1,
      sym_type_arguments,
    STATE(3032), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5907), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5905), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [74984] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6599), 1,
      anon_sym_EQ,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3033), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6597), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_else,
  [75074] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    STATE(2934), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(3034), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [75140] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5895), 1,
      anon_sym_BANG,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(6595), 1,
      anon_sym_COLON_COLON,
    STATE(3134), 1,
      sym_parameters,
    STATE(3177), 1,
      sym_type_arguments,
    STATE(3035), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5897), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5891), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [75210] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6603), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    STATE(3036), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6601), 19,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [75296] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6607), 1,
      anon_sym_EQ,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3037), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6605), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_else,
  [75386] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3038), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6425), 7,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_else,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [75478] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3039), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 20,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [75562] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(3040), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [75632] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3041), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 25,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [75706] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6611), 1,
      anon_sym_EQ,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3042), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6609), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_else,
  [75796] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6613), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3043), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [75860] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3044), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6369), 7,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_else,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [75952] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(436), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3045), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(434), 19,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
  [76038] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6615), 1,
      anon_sym_trait,
    STATE(3046), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76101] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5911), 1,
      anon_sym_BANG,
    ACTIONS(6617), 1,
      anon_sym_COLON_COLON,
    STATE(2871), 1,
      sym_field_initializer_list,
    STATE(3047), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [76164] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6619), 1,
      sym_identifier,
    STATE(3048), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(5817), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [76227] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6621), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3049), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76290] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(6623), 1,
      anon_sym_COLON_COLON,
    STATE(3134), 1,
      sym_parameters,
    STATE(3177), 1,
      sym_type_arguments,
    STATE(3050), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5934), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5932), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76357] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6225), 1,
      anon_sym_BANG,
    ACTIONS(6227), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6625), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(3051), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76420] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6627), 1,
      sym_identifier,
    STATE(3052), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 16,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_as,
    ACTIONS(5817), 23,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [76483] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6589), 1,
      anon_sym_BANG,
    ACTIONS(6629), 1,
      anon_sym_LBRACE,
    ACTIONS(6631), 1,
      anon_sym_COLON_COLON,
    STATE(3250), 1,
      sym_field_initializer_list,
    STATE(3053), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76548] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6633), 1,
      anon_sym_trait,
    STATE(3054), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76611] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5819), 1,
      anon_sym_BANG,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    STATE(3055), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76671] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6635), 1,
      anon_sym_BANG,
    STATE(3056), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76731] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5823), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6637), 1,
      anon_sym_BANG,
    STATE(3057), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76791] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    STATE(3136), 1,
      sym_parameters,
    STATE(3179), 1,
      sym_type_arguments,
    STATE(3058), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5955), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5953), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [76855] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3059), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5967), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5971), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [76911] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3060), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5959), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5963), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [76967] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6141), 1,
      anon_sym_LBRACE,
    STATE(3061), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5959), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5963), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [77025] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    STATE(3136), 1,
      sym_parameters,
    STATE(3179), 1,
      sym_type_arguments,
    STATE(3062), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5975), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5973), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [77089] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    STATE(3136), 1,
      sym_parameters,
    STATE(3179), 1,
      sym_type_arguments,
    STATE(3063), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5995), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5993), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [77153] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 1,
      anon_sym_BANG,
    ACTIONS(5991), 1,
      anon_sym_COLON_COLON,
    STATE(3064), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5989), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5985), 22,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [77213] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3065), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5987), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5991), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [77269] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5917), 1,
      anon_sym_BANG,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    STATE(3066), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [77329] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 1,
      anon_sym_BANG,
    ACTIONS(5983), 1,
      anon_sym_COLON_COLON,
    STATE(3067), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5981), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5977), 22,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [77389] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6075), 1,
      anon_sym_LBRACE,
    STATE(3068), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5967), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5971), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [77447] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3069), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5979), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5983), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
  [77503] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 1,
      anon_sym_BANG,
    ACTIONS(5971), 1,
      anon_sym_COLON_COLON,
    STATE(3070), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5969), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5965), 22,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [77563] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6067), 1,
      anon_sym_LBRACE,
    STATE(3071), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5987), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5991), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [77621] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6437), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6585), 1,
      anon_sym_BANG,
    STATE(3072), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 24,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [77681] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5911), 1,
      anon_sym_BANG,
    ACTIONS(6639), 1,
      anon_sym_COLON_COLON,
    STATE(3073), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [77741] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    STATE(3136), 1,
      sym_parameters,
    STATE(3179), 1,
      sym_type_arguments,
    STATE(3074), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 20,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [77805] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6153), 1,
      anon_sym_LBRACE,
    STATE(3075), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5979), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5983), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [77863] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 1,
      anon_sym_BANG,
    ACTIONS(5963), 1,
      anon_sym_COLON_COLON,
    STATE(3076), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5961), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5957), 22,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [77923] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3077), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5979), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5983), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [77978] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6641), 1,
      sym_identifier,
    STATE(5113), 1,
      sym_scoped_type_identifier,
    STATE(5690), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3078), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [78055] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6143), 1,
      sym_identifier,
    STATE(5439), 1,
      sym_scoped_type_identifier,
    STATE(5700), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3079), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [78132] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6589), 1,
      anon_sym_BANG,
    ACTIONS(6643), 1,
      anon_sym_COLON_COLON,
    STATE(3080), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [78191] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6645), 1,
      anon_sym_else,
    STATE(3402), 1,
      sym_else_clause,
    STATE(3081), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1352), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1350), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [78250] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3082), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6165), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6163), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [78305] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6647), 1,
      anon_sym_SQUOTE,
    STATE(3232), 1,
      sym_label,
    STATE(3083), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6051), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6049), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [78364] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6543), 1,
      anon_sym_COLON_COLON,
    STATE(3084), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [78421] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(2967), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3085), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [78516] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6103), 1,
      anon_sym_COLON_COLON,
    STATE(3086), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [78573] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(6677), 1,
      anon_sym_COLON_COLON,
    STATE(3087), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5999), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5997), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [78632] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(476), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3088), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [78727] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(2891), 1,
      sym_label,
    STATE(3089), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6051), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6049), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [78784] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6679), 1,
      anon_sym_COLON_COLON,
    STATE(3090), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [78841] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(696), 1,
      anon_sym_RBRACK,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6681), 1,
      anon_sym_SEMI,
    ACTIONS(6683), 1,
      anon_sym_COMMA,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    STATE(4934), 1,
      aux_sym_array_expression_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3091), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [78936] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(584), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3092), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [79031] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1011), 1,
      anon_sym_RBRACK,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6689), 1,
      anon_sym_SEMI,
    ACTIONS(6691), 1,
      anon_sym_COMMA,
    STATE(5359), 1,
      aux_sym_array_expression_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3093), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [79126] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(2937), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3094), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [79221] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6693), 1,
      sym_identifier,
    STATE(5188), 1,
      sym_scoped_type_identifier,
    STATE(5749), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3095), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [79298] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3096), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6079), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6077), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [79353] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3097), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5959), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5963), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [79408] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(684), 1,
      anon_sym_RBRACK,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6695), 1,
      anon_sym_SEMI,
    ACTIONS(6697), 1,
      anon_sym_COMMA,
    STATE(5200), 1,
      aux_sym_array_expression_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3098), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [79503] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6089), 1,
      anon_sym_COLON_COLON,
    STATE(3099), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 24,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
      anon_sym_as,
  [79560] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6225), 1,
      anon_sym_BANG,
    ACTIONS(6227), 1,
      anon_sym_COLON_COLON,
    STATE(3100), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [79619] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(580), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3101), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [79714] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3102), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5967), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5971), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [79769] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6089), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(3103), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [79826] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6699), 1,
      sym_identifier,
    STATE(5129), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5877), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3104), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [79903] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3105), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5987), 16,
      anon_sym_BANG,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5991), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [79958] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6103), 2,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    STATE(3106), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [80015] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6701), 1,
      sym_identifier,
    STATE(5220), 1,
      sym_scoped_type_identifier,
    STATE(5691), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3107), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [80092] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3108), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6193), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6191), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [80147] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6703), 1,
      sym_identifier,
    STATE(5080), 1,
      sym_scoped_type_identifier,
    STATE(5535), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3109), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [80224] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(3430), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3110), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [80319] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6183), 1,
      sym_identifier,
    STATE(5368), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5892), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3111), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [80396] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(3194), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3112), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [80491] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3113), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6169), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6167), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [80546] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6705), 1,
      sym_identifier,
    STATE(5384), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5902), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3114), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [80623] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6707), 1,
      sym_identifier,
    STATE(5386), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5905), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3115), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [80700] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    STATE(452), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3116), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [80795] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6203), 1,
      sym_identifier,
    STATE(5390), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5908), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3117), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [80872] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6019), 1,
      sym_identifier,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    STATE(5423), 1,
      sym_scoped_type_identifier,
    STATE(5456), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3118), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [80949] = 25,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(856), 1,
      anon_sym_RBRACK,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6709), 1,
      anon_sym_SEMI,
    ACTIONS(6711), 1,
      anon_sym_COMMA,
    STATE(4915), 1,
      aux_sym_array_expression_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3119), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [81044] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6713), 1,
      sym_identifier,
    STATE(5397), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5911), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3120), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [81121] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6715), 1,
      sym_identifier,
    STATE(5399), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5913), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3121), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [81198] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6717), 1,
      sym_identifier,
    STATE(5401), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5916), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3122), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [81275] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3123), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6013), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6011), 25,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [81330] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6719), 1,
      sym_identifier,
    STATE(5405), 1,
      sym_scoped_type_identifier,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(5920), 1,
      sym_generic_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3124), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [81407] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6721), 1,
      sym_identifier,
    STATE(5038), 1,
      sym_scoped_type_identifier,
    STATE(5610), 1,
      sym_generic_type,
    STATE(5787), 1,
      sym_primitive_type,
    STATE(6373), 1,
      sym_generic_type_with_turbofish,
    STATE(6374), 1,
      sym_scoped_identifier,
    STATE(3125), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(6267), 2,
      sym_bracketed_type,
      sym_metavariable,
    ACTIONS(6031), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6021), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
    ACTIONS(6025), 17,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
  [81484] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3126), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [81552] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6723), 1,
      anon_sym_DASH_GT,
    STATE(3127), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6133), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6131), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [81608] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(436), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3128), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(434), 15,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [81690] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6725), 1,
      anon_sym_COLON_COLON,
    STATE(3129), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5975), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5973), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [81746] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6725), 1,
      anon_sym_COLON_COLON,
    STATE(3130), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [81802] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6599), 1,
      anon_sym_EQ,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3131), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6597), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [81888] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6725), 1,
      anon_sym_COLON_COLON,
    STATE(3132), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5995), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5993), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [81944] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(394), 1,
      anon_sym_EQ,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3133), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(384), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [82030] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6731), 1,
      anon_sym_DASH_GT,
    STATE(3134), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6045), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6043), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [82086] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6733), 1,
      anon_sym_COLON_COLON,
    STATE(3135), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [82142] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6735), 1,
      anon_sym_DASH_GT,
    STATE(3136), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6213), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6211), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [82198] = 24,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1067), 1,
      anon_sym_RBRACK,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6737), 1,
      anon_sym_COMMA,
    STATE(5077), 1,
      aux_sym_array_expression_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3137), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [82290] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6583), 1,
      anon_sym_EQ,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3138), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6581), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [82376] = 24,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6739), 1,
      anon_sym_RPAREN,
    ACTIONS(6741), 1,
      anon_sym_COMMA,
    STATE(5258), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3139), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [82468] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6743), 1,
      anon_sym_COLON_COLON,
    STATE(3140), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [82524] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3141), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6275), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [82612] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3142), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1400), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1398), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [82666] = 24,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(899), 1,
      anon_sym_RBRACK,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6745), 1,
      anon_sym_COMMA,
    STATE(5298), 1,
      aux_sym_array_expression_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3143), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [82758] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3144), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1412), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1410), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [82812] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6747), 1,
      anon_sym_COLON_COLON,
    STATE(3145), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [82868] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3146), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5969), 17,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_LT_EQ,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5965), 22,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_LT2,
  [82922] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    STATE(3147), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [82986] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6607), 1,
      anon_sym_EQ,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3148), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6605), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [83072] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3149), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6083), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6081), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [83126] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3150), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6071), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6069), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [83180] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3151), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1356), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1358), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [83234] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3152), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [83304] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3153), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6315), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [83392] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3154), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6425), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [83480] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3155), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [83552] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6749), 1,
      anon_sym_DASH_GT,
    STATE(3156), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6099), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6097), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [83608] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3157), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6105), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6103), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [83662] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3158), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 17,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [83740] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6751), 1,
      anon_sym_COLON_COLON,
    STATE(3159), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [83796] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3160), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 16,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [83876] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3161), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1484), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1482), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [83930] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3162), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6173), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6171), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [83984] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3163), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1376), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1374), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [84038] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(3164), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [84104] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3165), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6095), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6093), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [84158] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6611), 1,
      anon_sym_EQ,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3166), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6609), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [84244] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3167), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6059), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6057), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [84298] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6725), 1,
      anon_sym_COLON_COLON,
    STATE(3168), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5955), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5953), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [84354] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3169), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6017), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6015), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_DASH_GT,
      anon_sym_as,
  [84408] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6575), 1,
      anon_sym_EQ,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3170), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6573), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [84494] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6227), 1,
      anon_sym_COLON_COLON,
    STATE(3171), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5821), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5817), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [84550] = 24,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6753), 1,
      anon_sym_RPAREN,
    ACTIONS(6755), 1,
      anon_sym_COMMA,
    STATE(5417), 1,
      aux_sym_arguments_repeat1,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3172), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [84642] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3173), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 21,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [84716] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6579), 1,
      anon_sym_EQ,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3174), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6577), 13,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_SQUOTE,
  [84802] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3175), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6161), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6159), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [84856] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3176), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1460), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1458), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
      anon_sym_else,
  [84910] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3177), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6149), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6147), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [84964] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6757), 1,
      anon_sym_LPAREN,
    STATE(3264), 1,
      sym_arguments,
    STATE(3178), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6065), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6061), 22,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [85022] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3179), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6039), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6037), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [85076] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3180), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6369), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [85164] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6603), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3181), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6601), 15,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_SQUOTE,
  [85246] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3182), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6091), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6089), 24,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_as,
  [85300] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(376), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3183), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [85389] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3184), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6473), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6471), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [85442] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3185), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6531), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6529), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [85495] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3186), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1500), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1498), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [85548] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3187), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6453), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6451), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [85601] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3188), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6539), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6537), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [85654] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6761), 1,
      anon_sym_SEMI,
    ACTIONS(6763), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3189), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [85743] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6765), 1,
      anon_sym_SEMI,
    ACTIONS(6767), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3190), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [85832] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3191), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6355), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6353), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [85885] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6769), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3192), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [85974] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3193), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1616), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1614), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [86027] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3194), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1516), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1514), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [86080] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3195), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6495), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6493), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [86133] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6771), 1,
      anon_sym_RPAREN,
    ACTIONS(6773), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3196), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86222] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6775), 1,
      anon_sym_SEMI,
    ACTIONS(6777), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3197), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86311] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6599), 1,
      anon_sym_EQ,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3198), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6597), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86396] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6811), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3199), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86483] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3200), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6265), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6263), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [86536] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6813), 1,
      anon_sym_SEMI,
    ACTIONS(6815), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3201), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86625] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6817), 1,
      anon_sym_SEMI,
    ACTIONS(6819), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3202), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86714] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3203), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1440), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1442), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [86767] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6821), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3204), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86854] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3205), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6281), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6279), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [86907] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6823), 1,
      anon_sym_SEMI,
    ACTIONS(6825), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3206), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [86996] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6827), 1,
      anon_sym_SEMI,
    ACTIONS(6829), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3207), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [87085] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6831), 1,
      anon_sym_SEMI,
    ACTIONS(6833), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3208), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [87174] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3209), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6519), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6517), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [87227] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3210), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6461), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6459), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [87280] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6835), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(3211), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [87367] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3212), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1382), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1384), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [87420] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3213), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6293), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [87473] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6124), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3214), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [87544] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(336), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3215), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [87633] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6843), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3216), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [87720] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(394), 1,
      anon_sym_EQ,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3217), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(384), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [87805] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3218), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6523), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6521), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [87858] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(436), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3219), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(434), 14,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [87939] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6599), 1,
      anon_sym_EQ,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3220), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6597), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88024] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(394), 1,
      anon_sym_EQ,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3221), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(384), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88109] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6869), 1,
      anon_sym_LBRACE,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6875), 1,
      anon_sym_DOT_DOT,
    STATE(572), 1,
      sym_match_block,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6877), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3222), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88198] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6583), 1,
      anon_sym_EQ,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3223), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6581), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88283] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6607), 1,
      anon_sym_EQ,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3224), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6605), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88368] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(338), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3225), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88457] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3226), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6427), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6425), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [88510] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3227), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6527), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6525), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [88563] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3228), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [88616] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6879), 1,
      anon_sym_SEMI,
    ACTIONS(6881), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3229), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88705] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3230), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1556), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1554), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [88758] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3231), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1378), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1380), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [88811] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3232), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6313), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6311), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [88864] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(6369), 2,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3233), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [88951] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6887), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    STATE(3234), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [89038] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3235), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6515), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6513), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [89091] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3236), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1454), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1456), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [89144] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3237), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1638), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1640), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [89197] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3238), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1540), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1538), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [89250] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(340), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3239), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [89339] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3240), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6503), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6501), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [89392] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(6315), 2,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3241), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [89479] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6889), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3242), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [89568] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6583), 1,
      anon_sym_EQ,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3243), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6581), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [89653] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3244), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1394), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1396), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [89706] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3245), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1608), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1606), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [89759] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6607), 1,
      anon_sym_EQ,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3246), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6605), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [89844] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(344), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3247), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [89933] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6875), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6891), 1,
      anon_sym_LBRACE,
    STATE(3341), 1,
      sym_match_block,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6877), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3248), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90022] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(346), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3249), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90111] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3250), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6257), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6255), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [90164] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3251), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1548), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1546), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [90217] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3252), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6297), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6295), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [90270] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(348), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3253), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90359] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6893), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3254), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90448] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(352), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3255), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90537] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(354), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3256), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90626] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(6425), 2,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3257), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90713] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(356), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3258), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90802] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6895), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3259), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90891] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(360), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3260), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [90980] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(362), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3261), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91069] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3262), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1560), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1558), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [91122] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3263), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6321), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6319), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [91175] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3264), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6325), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6323), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [91228] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(6275), 2,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3265), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91315] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(364), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3266), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91404] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3267), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1580), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1578), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [91457] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6897), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3268), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91546] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6899), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3269), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91633] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(368), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3270), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91722] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(324), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3271), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91811] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(370), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3272), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [91900] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3273), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6333), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6331), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [91953] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6901), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3274), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92042] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3275), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1588), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1586), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [92095] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(372), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3276), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92184] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6903), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3277), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92273] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3278), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1604), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1602), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [92326] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3279), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6379), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6377), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [92379] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(378), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3280), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92468] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(137), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3281), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92557] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6905), 1,
      anon_sym_RBRACE,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3282), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92646] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3283), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6403), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6401), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [92699] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6907), 1,
      anon_sym_SEMI,
    ACTIONS(6909), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3284), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92788] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6911), 1,
      anon_sym_SEMI,
    ACTIONS(6913), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3285), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [92877] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3286), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6277), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6275), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [92930] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6275), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3287), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [93017] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3288), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93070] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6106), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3289), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [93141] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6773), 1,
      anon_sym_COMMA,
    ACTIONS(6915), 1,
      anon_sym_RPAREN,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3290), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [93230] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3291), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6917), 3,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_SQUOTE,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [93315] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3292), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1496), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1494), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93368] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3293), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6359), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6357), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93421] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3294), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1534), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1536), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93474] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(6475), 1,
      sym_attribute,
    STATE(3295), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [93545] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6172), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3296), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [93616] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3297), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6371), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6369), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93669] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3298), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6363), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6361), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93722] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3299), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93775] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6921), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6919), 2,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(3300), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [93862] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3301), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6337), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6335), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93915] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3302), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6289), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6287), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [93968] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3303), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6419), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6417), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94021] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3304), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5955), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5953), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94074] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3305), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6261), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6259), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94127] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3306), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6367), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6365), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94180] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    STATE(3307), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6375), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6373), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94239] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3308), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6347), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6345), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94292] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3309), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6351), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6349), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94345] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3310), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6435), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6433), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94398] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    STATE(3311), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6535), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6533), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94457] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6923), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3312), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [94544] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6925), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3313), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [94631] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6369), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3314), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [94718] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6927), 1,
      anon_sym_SEMI,
    ACTIONS(6929), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3315), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [94807] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3316), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94860] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6038), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3317), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [94931] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3318), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6431), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6429), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [94984] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3319), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6465), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6463), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95037] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6931), 1,
      anon_sym_SEMI,
    ACTIONS(6933), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3320), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [95126] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3321), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6507), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6505), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95179] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6071), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3322), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [95250] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3323), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6065), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6061), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95303] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6935), 1,
      anon_sym_RBRACE,
    ACTIONS(6937), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3324), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [95392] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6939), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3325), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [95479] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3326), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6317), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6315), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95532] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3327), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6481), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6479), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95585] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3328), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95638] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6941), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3329), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [95725] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3330), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6499), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6497), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95778] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3331), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1564), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1562), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [95831] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6943), 1,
      anon_sym_RPAREN,
    ACTIONS(6945), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3332), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [95920] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6875), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6947), 1,
      anon_sym_LBRACE,
    STATE(481), 1,
      sym_match_block,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6877), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3333), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [96009] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(380), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3334), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [96098] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6315), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3335), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [96185] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6579), 1,
      anon_sym_EQ,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3336), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6577), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [96270] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3337), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6423), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6421), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96323] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3338), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1568), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1566), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96376] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3339), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6487), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6485), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96429] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3340), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6511), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6509), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96482] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3341), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1592), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1590), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96535] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3342), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6491), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6489), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96588] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3343), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6441), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6439), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96641] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3344), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1508), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1506), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96694] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3345), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1600), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1598), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96747] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3346), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2360), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(2358), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96800] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3347), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6445), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6443), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96853] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3348), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6399), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6397), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96906] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3349), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5951), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5949), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [96959] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6875), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6949), 1,
      anon_sym_LBRACE,
    STATE(2968), 1,
      sym_match_block,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6877), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3350), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [97048] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    STATE(3351), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [97111] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3352), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [97180] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3353), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [97247] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3354), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [97318] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3355), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6449), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6447), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [97371] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(139), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3356), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [97460] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    STATE(3357), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6301), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6299), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [97519] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6951), 1,
      anon_sym_SEMI,
    ACTIONS(6953), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3358), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [97608] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3359), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6305), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6303), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [97661] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(330), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3360), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [97750] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6955), 1,
      anon_sym_RBRACE,
    ACTIONS(6957), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3361), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [97839] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3362), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 16,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [97916] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3363), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 15,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [97995] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(3364), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [98060] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6611), 1,
      anon_sym_EQ,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3365), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6609), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [98145] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3366), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6415), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6413), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98198] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6575), 1,
      anon_sym_EQ,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3367), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6573), 12,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [98283] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6369), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3368), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [98370] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3369), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [98443] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    STATE(3370), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6329), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6327), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98502] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3371), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6395), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6393), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98555] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6603), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3372), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6601), 14,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [98636] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3373), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6469), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6467), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98689] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3374), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6477), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6475), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98742] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6959), 1,
      anon_sym_RPAREN,
    ACTIONS(6961), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3375), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [98831] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3376), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5645), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5647), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98884] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3377), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5975), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5973), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98937] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3378), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5995), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5993), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [98990] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3379), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6237), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6235), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [99043] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3380), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6253), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [99096] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3381), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6269), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6267), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [99149] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6315), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3382), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [99236] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6579), 1,
      anon_sym_EQ,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3383), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6577), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [99321] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(714), 1,
      anon_sym_RPAREN,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6773), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3384), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [99410] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(702), 1,
      anon_sym_RPAREN,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6773), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3385), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [99499] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3386), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6963), 3,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_SQUOTE,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [99584] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6965), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3387), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [99671] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3388), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6309), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6307), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [99724] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6425), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3389), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [99811] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    STATE(3390), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 11,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [99874] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3391), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 6,
      anon_sym_CARET,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [99943] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3392), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 7,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [100010] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3393), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 5,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [100081] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3394), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 16,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [100158] = 18,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6243), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3395), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6239), 15,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [100237] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(3396), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 9,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [100302] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6611), 1,
      anon_sym_EQ,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3397), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6609), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [100387] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6575), 1,
      anon_sym_EQ,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3398), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6573), 12,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [100472] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    STATE(3399), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6243), 4,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [100545] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6603), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3400), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6601), 14,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [100626] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(704), 1,
      anon_sym_RPAREN,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6773), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3401), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [100715] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3402), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1528), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1526), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [100768] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3403), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6219), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6217), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [100821] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6967), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3404), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [100908] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6969), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3405), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [100995] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3406), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6223), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6221), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101048] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3407), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1532), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1530), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101101] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3408), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6233), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6231), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101154] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    STATE(3409), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6243), 14,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
    ACTIONS(6239), 20,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [101215] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6175), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3410), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [101286] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(6480), 1,
      sym_attribute,
    STATE(3411), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [101357] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3412), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1544), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1542), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101410] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6035), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3413), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [101481] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3414), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6273), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6271), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101534] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3415), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6341), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6339), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101587] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6971), 1,
      anon_sym_SEMI,
    ACTIONS(6973), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3416), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [101676] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6270), 1,
      sym_attribute,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3417), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [101747] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3418), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6285), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6283), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101800] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(6975), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(3419), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [101887] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3420), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6383), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6381), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [101940] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(5982), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3421), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [102011] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1908), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6837), 1,
      sym_identifier,
    STATE(3993), 1,
      sym_metavariable,
    STATE(4000), 1,
      sym_scoped_identifier,
    STATE(6046), 1,
      sym_attribute,
    STATE(6226), 1,
      sym_bracketed_type,
    STATE(6299), 1,
      sym_generic_type_with_turbofish,
    STATE(3422), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6841), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6839), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [102082] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3423), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6387), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6385), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [102135] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3424), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6391), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6389), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [102188] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6977), 1,
      anon_sym_SEMI,
    ACTIONS(6979), 1,
      anon_sym_else,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3425), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [102277] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3426), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6407), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6405), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [102330] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6425), 2,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3427), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [102417] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(332), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3428), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [102506] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3429), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6411), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6409), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [102559] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3430), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1572), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1570), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [102612] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3431), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1576), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1574), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [102665] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3432), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1584), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1582), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [102718] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6275), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3433), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [102805] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(706), 1,
      anon_sym_RPAREN,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6773), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3434), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [102894] = 19,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(436), 2,
      anon_sym_EQ,
      anon_sym_DOT_DOT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    STATE(3435), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(434), 14,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
  [102975] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3436), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1596), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(1594), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [103028] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3437), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6457), 15,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(6455), 23,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_AMP_AMP,
      anon_sym_PIPE_PIPE,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_as,
  [103081] = 23,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(326), 1,
      anon_sym_RBRACE,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3438), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103170] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6981), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3439), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103256] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6983), 1,
      sym_identifier,
    ACTIONS(6987), 1,
      anon_sym_COLON_COLON,
    STATE(5470), 1,
      sym_metavariable,
    STATE(5472), 1,
      sym_scoped_identifier,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(3440), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6989), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(6985), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [103324] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6991), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3441), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103410] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6993), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3442), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103496] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6995), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3443), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103582] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6759), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3444), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103668] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6997), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3445), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103754] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6999), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3446), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103840] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7001), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3447), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [103926] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7003), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3448), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104012] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(7005), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6963), 2,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
    ACTIONS(7007), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3449), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104096] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(31), 1,
      anon_sym_LT,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(6987), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7009), 1,
      sym_identifier,
    STATE(5593), 1,
      sym_metavariable,
    STATE(5594), 1,
      sym_scoped_identifier,
    STATE(6207), 1,
      sym_generic_type_with_turbofish,
    STATE(6279), 1,
      sym_bracketed_type,
    STATE(3450), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7013), 3,
      sym_self,
      sym_super,
      sym_crate,
    ACTIONS(7011), 26,
      anon_sym_macro_rules,
      anon_sym_u8,
      anon_sym_i8,
      anon_sym_u16,
      anon_sym_i16,
      anon_sym_u32,
      anon_sym_i32,
      anon_sym_u64,
      anon_sym_i64,
      anon_sym_u128,
      anon_sym_i128,
      anon_sym_isize,
      anon_sym_usize,
      anon_sym_f32,
      anon_sym_f64,
      anon_sym_bool,
      anon_sym_str,
      anon_sym_char,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [104164] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7015), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3451), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104250] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7017), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3452), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104336] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7019), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3453), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104422] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7021), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3454), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104508] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7023), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3455), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104594] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7025), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3456), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104680] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7027), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3457), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104766] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(7005), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6917), 2,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
    ACTIONS(7007), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3458), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104850] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7029), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3459), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [104936] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7031), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3460), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105022] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7033), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3461), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105108] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7035), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3462), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105194] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7037), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3463), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105280] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7039), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3464), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105366] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7041), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3465), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105452] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7043), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3466), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105538] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7045), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3467), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105624] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7047), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3468), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105710] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7049), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3469), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105796] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7051), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3470), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105882] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7053), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3471), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [105968] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7055), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3472), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106054] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7057), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3473), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106140] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7059), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3474), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106226] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7061), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3475), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106312] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6773), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3476), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106398] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7063), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3477), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106484] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7065), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3478), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106570] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7067), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3479), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106656] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(6919), 1,
      anon_sym_EQ_GT,
    ACTIONS(7005), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7069), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(7007), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3480), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106742] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7071), 1,
      anon_sym_COMMA,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3481), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106828] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7073), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3482), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [106914] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7075), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3483), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107000] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7077), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3484), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107086] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7079), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3485), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107172] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7081), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3486), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107258] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7083), 1,
      anon_sym_SEMI,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3487), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107344] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7085), 1,
      anon_sym_RBRACK,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3488), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107430] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6875), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6877), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3489), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107513] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6727), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6729), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3490), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107596] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(7005), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(7007), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3491), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107679] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6685), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6687), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3492), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107762] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6653), 1,
      anon_sym_CARET,
    ACTIONS(6655), 1,
      anon_sym_AMP,
    ACTIONS(6657), 1,
      anon_sym_PIPE,
    ACTIONS(6659), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6661), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6667), 1,
      anon_sym_EQ,
    ACTIONS(6673), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6649), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6663), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6671), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6675), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3493), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6651), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6669), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6665), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107845] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6849), 1,
      anon_sym_CARET,
    ACTIONS(6851), 1,
      anon_sym_AMP,
    ACTIONS(6853), 1,
      anon_sym_PIPE,
    ACTIONS(6855), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6857), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6865), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6873), 1,
      anon_sym_EQ,
    ACTIONS(6845), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6859), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6863), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6867), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3494), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6847), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6861), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6871), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [107928] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6779), 1,
      anon_sym_LBRACK,
    ACTIONS(6785), 1,
      anon_sym_QMARK,
    ACTIONS(6787), 1,
      anon_sym_CARET,
    ACTIONS(6789), 1,
      anon_sym_AMP,
    ACTIONS(6791), 1,
      anon_sym_PIPE,
    ACTIONS(6793), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6795), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6803), 1,
      anon_sym_DOT,
    ACTIONS(6805), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6809), 1,
      anon_sym_as,
    ACTIONS(6885), 1,
      anon_sym_EQ,
    ACTIONS(6781), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6797), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6801), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6807), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3495), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6783), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6799), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6883), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [108011] = 21,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6241), 1,
      anon_sym_LBRACK,
    ACTIONS(6245), 1,
      anon_sym_QMARK,
    ACTIONS(6247), 1,
      anon_sym_DOT,
    ACTIONS(6249), 1,
      anon_sym_as,
    ACTIONS(6549), 1,
      anon_sym_CARET,
    ACTIONS(6551), 1,
      anon_sym_AMP,
    ACTIONS(6553), 1,
      anon_sym_PIPE,
    ACTIONS(6555), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6557), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6563), 1,
      anon_sym_EQ,
    ACTIONS(6569), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6545), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(6559), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6567), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(6571), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3496), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6547), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6565), 4,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(6561), 10,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
  [108094] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1356), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3497), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1358), 21,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [108132] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5903), 1,
      anon_sym_static,
    ACTIONS(6633), 1,
      anon_sym_trait,
    ACTIONS(7087), 1,
      anon_sym_macro,
    ACTIONS(7091), 1,
      anon_sym_auto,
    ACTIONS(7093), 1,
      anon_sym_const,
    ACTIONS(7095), 1,
      anon_sym_enum,
    ACTIONS(7097), 1,
      anon_sym_fn,
    ACTIONS(7099), 1,
      anon_sym_mod,
    ACTIONS(7101), 1,
      anon_sym_struct,
    ACTIONS(7103), 1,
      anon_sym_type,
    ACTIONS(7105), 1,
      anon_sym_union,
    ACTIONS(7107), 1,
      anon_sym_unsafe,
    ACTIONS(7109), 1,
      anon_sym_use,
    ACTIONS(7111), 1,
      anon_sym_extern,
    ACTIONS(7113), 1,
      anon_sym_safe,
    STATE(3778), 1,
      aux_sym__item_modifiers,
    STATE(4828), 1,
      sym_extern_modifier,
    STATE(6142), 1,
      sym_function_modifiers,
    STATE(3498), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 3,
      anon_sym_async,
      anon_sym_default,
      anon_sym_gen,
  [108202] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7115), 1,
      anon_sym_macro,
    ACTIONS(7117), 1,
      anon_sym_auto,
    ACTIONS(7119), 1,
      anon_sym_const,
    ACTIONS(7121), 1,
      anon_sym_enum,
    ACTIONS(7123), 1,
      anon_sym_fn,
    ACTIONS(7125), 1,
      anon_sym_mod,
    ACTIONS(7127), 1,
      anon_sym_static,
    ACTIONS(7129), 1,
      anon_sym_struct,
    ACTIONS(7131), 1,
      anon_sym_trait,
    ACTIONS(7133), 1,
      anon_sym_type,
    ACTIONS(7135), 1,
      anon_sym_union,
    ACTIONS(7137), 1,
      anon_sym_unsafe,
    ACTIONS(7139), 1,
      anon_sym_use,
    ACTIONS(7141), 1,
      anon_sym_extern,
    ACTIONS(7143), 1,
      anon_sym_safe,
    STATE(3794), 1,
      aux_sym__item_modifiers,
    STATE(4495), 1,
      sym_extern_modifier,
    STATE(6461), 1,
      sym_function_modifiers,
    STATE(3499), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 3,
      anon_sym_async,
      anon_sym_default,
      anon_sym_gen,
  [108272] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7145), 1,
      anon_sym_macro,
    ACTIONS(7147), 1,
      anon_sym_auto,
    ACTIONS(7149), 1,
      anon_sym_const,
    ACTIONS(7151), 1,
      anon_sym_enum,
    ACTIONS(7153), 1,
      anon_sym_fn,
    ACTIONS(7155), 1,
      anon_sym_mod,
    ACTIONS(7157), 1,
      anon_sym_static,
    ACTIONS(7159), 1,
      anon_sym_struct,
    ACTIONS(7161), 1,
      anon_sym_trait,
    ACTIONS(7163), 1,
      anon_sym_type,
    ACTIONS(7165), 1,
      anon_sym_union,
    ACTIONS(7167), 1,
      anon_sym_unsafe,
    ACTIONS(7169), 1,
      anon_sym_use,
    ACTIONS(7171), 1,
      anon_sym_extern,
    ACTIONS(7173), 1,
      anon_sym_safe,
    STATE(3793), 1,
      aux_sym__item_modifiers,
    STATE(4737), 1,
      sym_extern_modifier,
    STATE(6447), 1,
      sym_function_modifiers,
    STATE(3500), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 3,
      anon_sym_async,
      anon_sym_default,
      anon_sym_gen,
  [108342] = 22,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5889), 1,
      anon_sym_static,
    ACTIONS(6615), 1,
      anon_sym_trait,
    ACTIONS(7175), 1,
      anon_sym_macro,
    ACTIONS(7177), 1,
      anon_sym_auto,
    ACTIONS(7179), 1,
      anon_sym_const,
    ACTIONS(7181), 1,
      anon_sym_enum,
    ACTIONS(7183), 1,
      anon_sym_fn,
    ACTIONS(7185), 1,
      anon_sym_mod,
    ACTIONS(7187), 1,
      anon_sym_struct,
    ACTIONS(7189), 1,
      anon_sym_type,
    ACTIONS(7191), 1,
      anon_sym_union,
    ACTIONS(7193), 1,
      anon_sym_unsafe,
    ACTIONS(7195), 1,
      anon_sym_use,
    ACTIONS(7197), 1,
      anon_sym_extern,
    ACTIONS(7199), 1,
      anon_sym_safe,
    STATE(3761), 1,
      aux_sym__item_modifiers,
    STATE(4799), 1,
      sym_extern_modifier,
    STATE(6231), 1,
      sym_function_modifiers,
    STATE(3501), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 3,
      anon_sym_async,
      anon_sym_default,
      anon_sym_gen,
  [108412] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7201), 1,
      anon_sym_LPAREN,
    STATE(3502), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5925), 19,
      anon_sym_macro,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      sym_identifier,
  [108447] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5981), 1,
      anon_sym_COLON,
    ACTIONS(5983), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(3503), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5977), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [108484] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5969), 1,
      anon_sym_COLON,
    ACTIONS(5971), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(3504), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5965), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [108521] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5961), 1,
      anon_sym_COLON,
    ACTIONS(5963), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(3505), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5957), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [108558] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7205), 1,
      anon_sym_LPAREN,
    ACTIONS(7207), 1,
      anon_sym_LBRACE,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7211), 1,
      anon_sym_COLON,
    ACTIONS(7213), 1,
      anon_sym_AT,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7219), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3555), 1,
      sym_type_arguments,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3506), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [108611] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5989), 1,
      anon_sym_COLON,
    ACTIONS(5991), 2,
      anon_sym_BANG,
      anon_sym_COLON_COLON,
    STATE(3507), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5985), 17,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [108648] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3508), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5985), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(5991), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [108684] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3509), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5921), 19,
      anon_sym_macro,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      sym_identifier,
  [108716] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6193), 1,
      anon_sym_COLON,
    STATE(3510), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6191), 18,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [108750] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6013), 1,
      anon_sym_COLON,
    STATE(3511), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6011), 18,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [108784] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3512), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5965), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(5971), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [108820] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3513), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5905), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [108864] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_COLON_COLON,
    STATE(3514), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5927), 18,
      anon_sym_macro,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
  [108898] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6165), 1,
      anon_sym_COLON,
    STATE(3515), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6163), 18,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [108932] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7227), 1,
      anon_sym_COLON_COLON,
    STATE(3516), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5927), 18,
      anon_sym_macro,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
  [108966] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6169), 1,
      anon_sym_COLON,
    STATE(3517), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6167), 18,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [109000] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7239), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3518), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [109048] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3519), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [109092] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3520), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5891), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [109136] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3521), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5957), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(5963), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [109172] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6079), 1,
      anon_sym_COLON,
    STATE(3522), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6077), 18,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [109206] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3523), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5941), 19,
      anon_sym_macro,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      sym_identifier,
  [109238] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3524), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5977), 3,
      anon_sym_LBRACE,
      anon_sym_for,
      anon_sym_LT2,
    ACTIONS(5983), 14,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [109274] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3525), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5945), 19,
      anon_sym_macro,
      anon_sym_async,
      anon_sym_auto,
      anon_sym_const,
      anon_sym_default,
      anon_sym_enum,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_mod,
      anon_sym_static,
      anon_sym_struct,
      anon_sym_trait,
      anon_sym_type,
      anon_sym_union,
      anon_sym_unsafe,
      anon_sym_use,
      anon_sym_extern,
      anon_sym_safe,
      sym_identifier,
  [109306] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1882), 1,
      anon_sym_COMMA,
    ACTIONS(1884), 1,
      anon_sym_GT,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3848), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3526), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5209), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109364] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7245), 1,
      anon_sym_COMMA,
    ACTIONS(7247), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3841), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3527), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5130), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109422] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7249), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3528), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109477] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7251), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3529), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109532] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7253), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3530), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109587] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7255), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3531), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [109632] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7257), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3532), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109687] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3533), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5983), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [109720] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3534), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5971), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [109753] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7259), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3535), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5932), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [109794] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7261), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3536), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109849] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(39), 1,
      anon_sym_SQUOTE,
    ACTIONS(925), 1,
      anon_sym_DASH,
    ACTIONS(973), 1,
      aux_sym_string_literal_token1,
    ACTIONS(983), 1,
      sym__raw_string_literal_start,
    ACTIONS(1714), 1,
      anon_sym_LBRACE,
    ACTIONS(7263), 1,
      sym_identifier,
    STATE(6417), 1,
      sym_label,
    ACTIONS(975), 2,
      anon_sym_true,
      anon_sym_false,
    STATE(3537), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1748), 3,
      sym_float_literal,
      sym_integer_literal,
      sym_char_literal,
    STATE(5065), 3,
      sym_string_literal,
      sym_raw_string_literal,
      sym_boolean_literal,
    STATE(5475), 3,
      sym_block,
      sym__literal,
      sym_negative_literal,
  [109900] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7265), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3538), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [109955] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3539), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5965), 18,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
      anon_sym_LT2,
  [109986] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7267), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3540), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [110041] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3541), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5991), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [110074] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7269), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3542), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [110129] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    ACTIONS(7271), 1,
      anon_sym_GT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3543), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [110184] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3544), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5963), 16,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_else,
      anon_sym_in,
  [110217] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7273), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3545), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [110261] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(5999), 1,
      anon_sym_COLON,
    ACTIONS(7275), 1,
      anon_sym_COLON_COLON,
    STATE(3546), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5997), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [110297] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7279), 1,
      anon_sym_RBRACE,
    ACTIONS(7281), 1,
      anon_sym_COMMA,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4403), 1,
      sym_attributes,
    STATE(3547), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5199), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [110353] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7291), 1,
      anon_sym_RBRACE,
    ACTIONS(7293), 1,
      anon_sym_COMMA,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4426), 1,
      sym_attributes,
    STATE(3548), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5336), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [110409] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7229), 1,
      anon_sym_PIPE,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7295), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3549), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [110455] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3550), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [110493] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3551), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5953), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [110531] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3552), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5973), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [110569] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3553), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5993), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [110607] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7297), 1,
      anon_sym_RBRACE,
    ACTIONS(7299), 1,
      anon_sym_COMMA,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4393), 1,
      sym_attributes,
    STATE(3554), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(4950), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [110663] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6149), 1,
      anon_sym_COLON,
    STATE(3555), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6147), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [110695] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6039), 1,
      anon_sym_COLON,
    STATE(3556), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6037), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [110727] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7301), 1,
      anon_sym_RBRACE,
    ACTIONS(7303), 1,
      anon_sym_COMMA,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4394), 1,
      sym_attributes,
    STATE(3557), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(4964), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [110783] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6161), 1,
      anon_sym_COLON,
    STATE(3558), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6159), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_for,
      anon_sym_where,
      anon_sym_else,
  [110815] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(3822), 1,
      sym_attributes,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3559), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5620), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [110867] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7305), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3560), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [110920] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3561), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6081), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [110949] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3562), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1538), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [110978] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7307), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3563), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [111031] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7309), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3564), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [111084] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3565), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1494), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [111113] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3566), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6015), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111142] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5951), 1,
      anon_sym_COLON,
    ACTIONS(7311), 1,
      anon_sym_COLON_COLON,
    STATE(3567), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111175] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7313), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3568), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [111228] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7203), 1,
      anon_sym_PIPE,
    ACTIONS(7207), 1,
      anon_sym_LBRACE,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7211), 1,
      anon_sym_COLON,
    ACTIONS(7213), 1,
      anon_sym_AT,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7315), 1,
      anon_sym_LPAREN,
    ACTIONS(7317), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3569), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [111281] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5889), 1,
      anon_sym_static,
    ACTIONS(6615), 1,
      anon_sym_trait,
    ACTIONS(7177), 1,
      anon_sym_auto,
    ACTIONS(7319), 1,
      anon_sym_impl,
    STATE(480), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(3570), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [111326] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3571), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6171), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111355] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3572), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1606), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [111384] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7321), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3573), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [111437] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7325), 1,
      anon_sym_RBRACE,
    ACTIONS(7327), 1,
      anon_sym_COMMA,
    ACTIONS(7329), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7331), 1,
      sym_integer_literal,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3574), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5149), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [111488] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5995), 1,
      anon_sym_COLON,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    STATE(3575), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5993), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111521] = 17,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5909), 1,
      anon_sym_PLUS,
    ACTIONS(7203), 1,
      anon_sym_PIPE,
    ACTIONS(7207), 1,
      anon_sym_LBRACE,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7211), 1,
      anon_sym_COLON,
    ACTIONS(7213), 1,
      anon_sym_AT,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7335), 1,
      anon_sym_LPAREN,
    ACTIONS(7340), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(7337), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3576), 2,
      sym_line_comment,
      sym_block_comment,
  [111576] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7207), 1,
      anon_sym_LBRACE,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7213), 1,
      anon_sym_AT,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7337), 1,
      anon_sym_RBRACK,
    ACTIONS(7342), 1,
      anon_sym_LPAREN,
    ACTIONS(7344), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    ACTIONS(5909), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(7203), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3577), 2,
      sym_line_comment,
      sym_block_comment,
  [111629] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(418), 1,
      sym_fragment_specifier,
    ACTIONS(7348), 2,
      anon_sym_expr,
      anon_sym_pat,
    STATE(3578), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7346), 13,
      anon_sym_block,
      anon_sym_expr_2021,
      anon_sym_ident,
      anon_sym_item,
      anon_sym_lifetime,
      anon_sym_literal,
      anon_sym_meta,
      anon_sym_pat_param,
      anon_sym_path,
      anon_sym_stmt,
      anon_sym_tt,
      anon_sym_ty,
      anon_sym_vis,
  [111662] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5975), 1,
      anon_sym_COLON,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    STATE(3579), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5973), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111695] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3580), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1536), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
      anon_sym_in,
  [111724] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7350), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3581), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [111777] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3582), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6069), 16,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DASH_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111806] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5951), 1,
      anon_sym_COLON,
    ACTIONS(7352), 1,
      anon_sym_COLON_COLON,
    STATE(3583), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111839] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7354), 1,
      anon_sym_RBRACE,
    ACTIONS(7356), 1,
      anon_sym_COMMA,
    ACTIONS(7358), 1,
      anon_sym_DOT_DOT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3584), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5242), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [111890] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7360), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3585), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [111943] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5951), 1,
      anon_sym_COLON,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    STATE(3586), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [111976] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6105), 1,
      anon_sym_COLON,
    STATE(3587), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6103), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112007] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7362), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3588), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112060] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7364), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3589), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112113] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7366), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3590), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112166] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(5903), 1,
      anon_sym_static,
    ACTIONS(6633), 1,
      anon_sym_trait,
    ACTIONS(7091), 1,
      anon_sym_auto,
    ACTIONS(7368), 1,
      anon_sym_impl,
    STATE(480), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(3591), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [112211] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7205), 1,
      anon_sym_LPAREN,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7211), 1,
      anon_sym_COLON,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7370), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3592), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [112252] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7372), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3593), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112305] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    ACTIONS(7374), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3594), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112358] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5955), 1,
      anon_sym_COLON,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    STATE(3595), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5953), 14,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112391] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6091), 1,
      anon_sym_COLON,
    STATE(3596), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6089), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_COLON_COLON,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112422] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3597), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6529), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112450] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3598), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6353), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112478] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3599), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6521), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112506] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7376), 1,
      sym_identifier,
    ACTIONS(7378), 1,
      anon_sym_RBRACE,
    ACTIONS(7380), 1,
      anon_sym_COMMA,
    ACTIONS(7382), 1,
      sym_crate,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4701), 1,
      sym_attributes,
    STATE(4998), 1,
      sym_enum_variant,
    STATE(6185), 1,
      sym_visibility_modifier,
    STATE(3600), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112558] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3601), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6479), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112586] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7384), 1,
      anon_sym_RBRACE,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3602), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [112634] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3603), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6357), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112662] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7388), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3604), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [112710] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3605), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6525), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112738] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7392), 1,
      anon_sym_RBRACE,
    ACTIONS(7394), 1,
      anon_sym_COMMA,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4379), 1,
      sym_attributes,
    STATE(5049), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3606), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112790] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7396), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3607), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [112838] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3608), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6509), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112866] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3609), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6221), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [112894] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7273), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3610), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [112932] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7398), 1,
      anon_sym_RBRACE,
    ACTIONS(7400), 1,
      anon_sym_COMMA,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4398), 1,
      sym_attributes,
    STATE(5053), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3611), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [112984] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3612), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6475), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113012] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3613), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6361), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113040] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3614), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6235), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113068] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7402), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3615), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [113116] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7404), 1,
      anon_sym_move,
    STATE(101), 1,
      sym_closure_parameters,
    STATE(469), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(3616), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [113158] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3617), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6381), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113186] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3618), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5993), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113214] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3619), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2358), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113242] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3620), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6493), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113270] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3621), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6397), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113298] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3622), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6283), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113326] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3623), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6251), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113354] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3624), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6401), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113382] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3625), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6267), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113410] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7406), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3626), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [113458] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3627), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6517), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113486] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3628), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6389), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113514] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7283), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4275), 1,
      sym_attributes,
    STATE(3629), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5707), 2,
      sym_field_pattern,
      sym_remaining_field_pattern,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [113564] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3630), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5953), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113592] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3631), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6405), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113620] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7408), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3632), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [113668] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7410), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3633), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [113716] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7376), 1,
      sym_identifier,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7412), 1,
      anon_sym_RBRACE,
    ACTIONS(7414), 1,
      anon_sym_COMMA,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4701), 1,
      sym_attributes,
    STATE(5407), 1,
      sym_enum_variant,
    STATE(6185), 1,
      sym_visibility_modifier,
    STATE(3634), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [113768] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3635), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6271), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113796] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7416), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3636), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [113844] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3637), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6231), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113872] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3638), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6409), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113900] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3639), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6467), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113928] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3640), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6505), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113956] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3641), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6377), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [113984] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3642), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5973), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [114012] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3643), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 15,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_as,
      anon_sym_where,
      anon_sym_else,
  [114040] = 16,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7418), 1,
      anon_sym_RBRACE,
    ACTIONS(7420), 1,
      anon_sym_COMMA,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4424), 1,
      sym_attributes,
    STATE(4926), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3644), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114092] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7422), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3645), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114141] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7424), 1,
      sym_identifier,
    ACTIONS(7426), 1,
      anon_sym_LPAREN,
    ACTIONS(7428), 1,
      anon_sym_LBRACE,
    ACTIONS(7432), 1,
      anon_sym_LBRACK,
    STATE(467), 1,
      sym_delim_token_tree,
    STATE(3646), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7430), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [114178] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7323), 1,
      sym_identifier,
    ACTIONS(7331), 1,
      sym_integer_literal,
    ACTIONS(7386), 1,
      anon_sym_DOT_DOT,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(5534), 1,
      sym_attributes,
    STATE(3647), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
    STATE(5588), 3,
      sym_shorthand_field_initializer,
      sym_field_initializer,
      sym_base_field_initializer,
  [114223] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7434), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3648), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114272] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1378), 1,
      anon_sym_DOT_DOT,
    STATE(3649), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1380), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
      anon_sym_in,
  [114301] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7436), 1,
      sym_identifier,
    ACTIONS(7438), 1,
      anon_sym_LPAREN,
    ACTIONS(7440), 1,
      anon_sym_LBRACE,
    ACTIONS(7444), 1,
      anon_sym_LBRACK,
    STATE(2440), 1,
      sym_delim_token_tree,
    STATE(3650), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7442), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [114338] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1382), 1,
      anon_sym_DOT_DOT,
    STATE(3651), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1384), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
      anon_sym_in,
  [114367] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7446), 1,
      anon_sym_DASH_GT,
    STATE(3652), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6131), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [114396] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7448), 1,
      anon_sym_DASH_GT,
    STATE(3653), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6043), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [114425] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7450), 1,
      anon_sym_DASH_GT,
    STATE(3654), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6211), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [114454] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7438), 1,
      anon_sym_LPAREN,
    ACTIONS(7440), 1,
      anon_sym_LBRACE,
    ACTIONS(7444), 1,
      anon_sym_LBRACK,
    ACTIONS(7452), 1,
      sym_identifier,
    STATE(2440), 1,
      sym_delim_token_tree,
    STATE(3655), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7454), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [114491] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7456), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3656), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114540] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7458), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3657), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114589] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7376), 1,
      sym_identifier,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7460), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4701), 1,
      sym_attributes,
    STATE(5539), 1,
      sym_enum_variant,
    STATE(6185), 1,
      sym_visibility_modifier,
    STATE(3658), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114638] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7376), 1,
      sym_identifier,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7462), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4701), 1,
      sym_attributes,
    STATE(5539), 1,
      sym_enum_variant,
    STATE(6185), 1,
      sym_visibility_modifier,
    STATE(3659), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114687] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7464), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3660), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114736] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1454), 1,
      anon_sym_DOT_DOT,
    STATE(3661), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1456), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
      anon_sym_in,
  [114765] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7466), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3662), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114814] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7468), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3663), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114863] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7470), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3664), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [114912] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1394), 1,
      anon_sym_DOT_DOT,
    STATE(3665), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1396), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
      anon_sym_in,
  [114941] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7472), 1,
      anon_sym_DASH_GT,
    STATE(3666), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6097), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
      anon_sym_else,
  [114970] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7426), 1,
      anon_sym_LPAREN,
    ACTIONS(7428), 1,
      anon_sym_LBRACE,
    ACTIONS(7432), 1,
      anon_sym_LBRACK,
    ACTIONS(7474), 1,
      sym_identifier,
    STATE(467), 1,
      sym_delim_token_tree,
    STATE(3667), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7476), 9,
      anon_sym_macro_rules,
      anon_sym_auto,
      anon_sym_await,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_raw,
      anon_sym_union,
      anon_sym_safe,
      anon_sym_try,
  [115007] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7376), 1,
      sym_identifier,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7478), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4701), 1,
      sym_attributes,
    STATE(5539), 1,
      sym_enum_variant,
    STATE(6185), 1,
      sym_visibility_modifier,
    STATE(3668), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [115056] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7482), 1,
      anon_sym_DOT_DOT,
    STATE(3669), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7480), 13,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
      anon_sym_in,
  [115085] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    ACTIONS(7484), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3670), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [115134] = 15,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7376), 1,
      sym_identifier,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7486), 1,
      anon_sym_RBRACE,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4701), 1,
      sym_attributes,
    STATE(5539), 1,
      sym_enum_variant,
    STATE(6185), 1,
      sym_visibility_modifier,
    STATE(3671), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [115183] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7490), 1,
      anon_sym_DOT_DOT,
    STATE(3672), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7488), 12,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_else,
      anon_sym_in,
  [115211] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7492), 1,
      sym_identifier,
    STATE(471), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(3673), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [115247] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7494), 1,
      anon_sym_LPAREN,
    ACTIONS(5959), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3674), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5957), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5963), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115279] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3675), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5977), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(7497), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(5983), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115311] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7497), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(3676), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5977), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5983), 6,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115343] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7502), 1,
      anon_sym_COLON,
    STATE(3677), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6089), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    ACTIONS(7500), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [115373] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3678), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [115403] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3679), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5985), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(7504), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(5991), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115435] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7507), 1,
      sym_identifier,
    STATE(471), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(3680), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [115471] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7509), 1,
      anon_sym_LPAREN,
    ACTIONS(5967), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3681), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5965), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5971), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115503] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7514), 1,
      anon_sym_COLON,
    ACTIONS(7516), 1,
      anon_sym_EQ,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4112), 1,
      sym_parameters,
    STATE(5236), 1,
      sym_trait_bounds,
    ACTIONS(5909), 2,
      anon_sym_PLUS,
      anon_sym_as,
    ACTIONS(7512), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(3682), 2,
      sym_line_comment,
      sym_block_comment,
  [115549] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7494), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(3683), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5957), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5963), 6,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115581] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7504), 1,
      anon_sym_LPAREN,
    ACTIONS(5987), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3684), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5985), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5991), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115613] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7509), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(3685), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5965), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5971), 6,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115645] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7497), 1,
      anon_sym_LPAREN,
    ACTIONS(5979), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3686), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5977), 5,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5983), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115677] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7376), 1,
      sym_identifier,
    ACTIONS(7382), 1,
      sym_crate,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4701), 1,
      sym_attributes,
    STATE(5539), 1,
      sym_enum_variant,
    STATE(6185), 1,
      sym_visibility_modifier,
    STATE(3687), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [115723] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7502), 1,
      anon_sym_COLON,
    STATE(3688), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6103), 3,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
    ACTIONS(7500), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [115753] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7504), 2,
      anon_sym_LPAREN,
      anon_sym_RBRACK,
    STATE(3689), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5985), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(5991), 6,
      anon_sym_COMMA,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115785] = 14,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(1886), 1,
      anon_sym_POUND,
    ACTIONS(1892), 1,
      sym__outer_block_doc_comment_start,
    ACTIONS(1894), 1,
      sym__outer_line_doc_comment_start,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    STATE(1950), 1,
      aux_sym_attributes_repeat1,
    STATE(4361), 1,
      sym_attributes,
    STATE(5504), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(3690), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(1967), 3,
      sym_attribute_item,
      sym_line_outer_doc_comment,
      sym_block_outer_doc_comment,
  [115831] = 13,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7518), 1,
      anon_sym_LPAREN,
    ACTIONS(7520), 1,
      anon_sym_LBRACE,
    ACTIONS(7522), 1,
      anon_sym_BANG,
    ACTIONS(7524), 1,
      anon_sym_AT,
    ACTIONS(7526), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7530), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    ACTIONS(7528), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3691), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [115875] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3692), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5965), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(7509), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(5971), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115907] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 2,
      anon_sym_COLON,
      anon_sym_DOT_DOT,
    STATE(3693), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5957), 3,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_LT2,
    ACTIONS(7494), 3,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(5963), 5,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
  [115939] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(6001), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    STATE(2843), 1,
      sym_type_arguments,
    STATE(3694), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [115972] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7147), 1,
      anon_sym_auto,
    ACTIONS(7157), 1,
      anon_sym_static,
    ACTIONS(7161), 1,
      anon_sym_trait,
    ACTIONS(7532), 1,
      anon_sym_impl,
    STATE(3695), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116005] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(6677), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    STATE(3175), 1,
      sym_type_arguments,
    STATE(3696), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116038] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(6677), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7534), 1,
      anon_sym_BANG,
    STATE(3175), 1,
      sym_type_arguments,
    STATE(3697), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116071] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(6001), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7536), 1,
      anon_sym_BANG,
    STATE(2843), 1,
      sym_type_arguments,
    STATE(3698), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116104] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7275), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    STATE(3699), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116137] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7117), 1,
      anon_sym_auto,
    ACTIONS(7127), 1,
      anon_sym_static,
    ACTIONS(7131), 1,
      anon_sym_trait,
    ACTIONS(7538), 1,
      anon_sym_impl,
    STATE(3700), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116170] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1714), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3804), 1,
      sym_block,
    STATE(6417), 1,
      sym_label,
    STATE(3701), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116203] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7542), 1,
      anon_sym_COLON,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    STATE(3702), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7540), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116231] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7546), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3703), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [116269] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7548), 1,
      anon_sym_auto,
    ACTIONS(7550), 1,
      anon_sym_static,
    ACTIONS(7552), 1,
      anon_sym_trait,
    STATE(3704), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116299] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7554), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3705), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [116337] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(7556), 1,
      anon_sym_move,
    STATE(120), 1,
      sym_closure_parameters,
    STATE(3706), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116367] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5907), 1,
      anon_sym_COLON,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4112), 1,
      sym_parameters,
    STATE(3707), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5905), 4,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [116405] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7558), 1,
      anon_sym_auto,
    ACTIONS(7560), 1,
      anon_sym_static,
    ACTIONS(7562), 1,
      anon_sym_trait,
    STATE(3708), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116435] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7157), 1,
      anon_sym_static,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    STATE(3709), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116465] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5913), 1,
      anon_sym_COLON,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4112), 1,
      sym_parameters,
    STATE(3710), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [116503] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3711), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1374), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_else,
      anon_sym_in,
  [116527] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3712), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1410), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_else,
      anon_sym_in,
  [116551] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3713), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1458), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_else,
      anon_sym_in,
  [116575] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7568), 1,
      anon_sym_COLON,
    STATE(3714), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7566), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116603] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7572), 1,
      anon_sym_COLON,
    ACTIONS(7574), 1,
      anon_sym_COLON_COLON,
    STATE(3715), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116631] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7227), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7572), 1,
      anon_sym_COLON,
    STATE(3716), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116659] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7572), 1,
      anon_sym_COLON,
    ACTIONS(7576), 1,
      anon_sym_COLON_COLON,
    STATE(3717), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116687] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3718), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1398), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_else,
      anon_sym_in,
  [116711] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3719), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1482), 11,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_else,
      anon_sym_in,
  [116735] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5913), 1,
      anon_sym_COLON,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7578), 1,
      anon_sym_EQ,
    STATE(4075), 1,
      sym_type_arguments,
    STATE(4112), 1,
      sym_parameters,
    STATE(3720), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 3,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_GT,
  [116775] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7580), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3721), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [116813] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7584), 1,
      anon_sym_COLON,
    STATE(3722), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7582), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116841] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7574), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7588), 1,
      anon_sym_COLON,
    STATE(3723), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116869] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7227), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7588), 1,
      anon_sym_COLON,
    STATE(3724), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116897] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7576), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7588), 1,
      anon_sym_COLON,
    STATE(3725), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116925] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7127), 1,
      anon_sym_static,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    STATE(3726), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [116955] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7574), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7592), 1,
      anon_sym_COLON,
    STATE(3727), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [116983] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7227), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7592), 1,
      anon_sym_COLON,
    STATE(3728), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [117011] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7576), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7592), 1,
      anon_sym_COLON,
    STATE(3729), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 9,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [117039] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5977), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(3730), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5983), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [117067] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5957), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(3731), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5963), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [117095] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5965), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(3732), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5971), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [117123] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5985), 2,
      anon_sym_LBRACE,
      anon_sym_LT2,
    STATE(3733), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5991), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [117151] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(7594), 1,
      anon_sym_COLON_COLON,
    STATE(2843), 1,
      sym_type_arguments,
    STATE(3734), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [117181] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5897), 1,
      anon_sym_COLON,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4112), 1,
      sym_parameters,
    STATE(3735), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5891), 4,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [117219] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7596), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    STATE(3736), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [117249] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(7598), 1,
      anon_sym_COLON_COLON,
    STATE(3175), 1,
      sym_type_arguments,
    STATE(3737), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [117279] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7600), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3738), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117317] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7602), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3739), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117355] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7604), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3740), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117393] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7606), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3741), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117431] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7239), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3742), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [117469] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7608), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3743), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117507] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7610), 1,
      anon_sym_auto,
    ACTIONS(7612), 1,
      anon_sym_static,
    ACTIONS(7614), 1,
      anon_sym_trait,
    STATE(3744), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [117537] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7616), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3745), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117575] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7618), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3746), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117613] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7620), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3747), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117651] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7622), 1,
      anon_sym_auto,
    ACTIONS(7624), 1,
      anon_sym_static,
    ACTIONS(7626), 1,
      anon_sym_trait,
    STATE(3748), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [117681] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7628), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3749), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117719] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7630), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3750), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117757] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7632), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3751), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117795] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7634), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3752), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117833] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7225), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7636), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(3653), 1,
      sym_parameters,
    STATE(3753), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5909), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [117871] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3754), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7638), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [117894] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3755), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [117917] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3756), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7640), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [117940] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3757), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7642), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [117963] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3758), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7644), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [117986] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3759), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7646), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118009] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3760), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7648), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118032] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4506), 1,
      sym_extern_modifier,
    STATE(3761), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118063] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3762), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118086] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3763), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7654), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118109] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3764), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7656), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118132] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3765), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7658), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118155] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3766), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7660), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118178] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3767), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7662), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118201] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3768), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7664), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118224] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3769), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7666), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118247] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3770), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7668), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118270] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3771), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7670), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118293] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3772), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118316] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3773), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7672), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118339] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3774), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7674), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118362] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(7229), 1,
      anon_sym_PIPE,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7295), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3775), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5997), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [118397] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7676), 1,
      anon_sym_LBRACE,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3776), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 5,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PLUS,
  [118430] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3777), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7678), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118453] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4849), 1,
      sym_extern_modifier,
    STATE(3778), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118484] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    STATE(3779), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [118511] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7680), 1,
      anon_sym_BANG,
    ACTIONS(7682), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7686), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7684), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3780), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [118546] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3781), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7688), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118569] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4782), 1,
      sym_extern_modifier,
    STATE(3782), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118600] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3783), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7690), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118623] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3784), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7692), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118646] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3785), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7694), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118669] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3786), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118692] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5934), 1,
      anon_sym_COLON,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7259), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4112), 1,
      sym_parameters,
    STATE(3787), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5932), 4,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [118727] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7502), 1,
      anon_sym_COLON,
    STATE(3788), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7500), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
    ACTIONS(6089), 6,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_COLON_COLON,
  [118754] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7502), 1,
      anon_sym_COLON,
    STATE(3789), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7500), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
    ACTIONS(6103), 6,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_COLON_COLON,
  [118781] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(5997), 1,
      anon_sym_PLUS,
    ACTIONS(7229), 1,
      anon_sym_PIPE,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7239), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(7696), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3790), 2,
      sym_line_comment,
      sym_block_comment,
  [118818] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4735), 1,
      sym_extern_modifier,
    STATE(3791), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118849] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4490), 1,
      sym_extern_modifier,
    STATE(3792), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118880] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4511), 1,
      sym_extern_modifier,
    STATE(3793), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118911] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4660), 1,
      sym_extern_modifier,
    STATE(3794), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118942] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3795), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7699), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [118965] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(6403), 1,
      sym_extern_modifier,
    STATE(3796), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [118996] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3797), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7701), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119019] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3798), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7703), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119042] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3799), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7705), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119065] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3800), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7707), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119088] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7205), 1,
      anon_sym_LPAREN,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7211), 1,
      anon_sym_COLON,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7709), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3801), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [119123] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7255), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3802), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
  [119158] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3803), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7711), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119181] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3804), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1554), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119204] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7652), 1,
      anon_sym_extern,
    STATE(3845), 1,
      aux_sym__item_modifiers,
    STATE(4498), 1,
      sym_extern_modifier,
    STATE(3805), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7089), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [119235] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3806), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7713), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119258] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3807), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7715), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119281] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7255), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7696), 1,
      anon_sym_RBRACK,
    ACTIONS(5997), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(7229), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3808), 2,
      sym_line_comment,
      sym_block_comment,
  [119316] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3809), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7717), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119339] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3810), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7719), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119362] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3811), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7721), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119385] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3812), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7723), 10,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_PIPE,
      anon_sym_EQ,
      anon_sym_else,
      anon_sym_in,
  [119408] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7229), 1,
      anon_sym_PIPE,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7233), 1,
      anon_sym_COLON,
    ACTIONS(7235), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7295), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    ACTIONS(7237), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3813), 2,
      sym_line_comment,
      sym_block_comment,
  [119444] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7729), 1,
      anon_sym_EQ,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    STATE(2561), 1,
      sym_declaration_list,
    STATE(4024), 1,
      sym_type_parameters,
    STATE(4304), 1,
      sym_trait_bounds,
    STATE(4911), 1,
      sym_where_clause,
    STATE(3814), 2,
      sym_line_comment,
      sym_block_comment,
  [119482] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7735), 1,
      anon_sym_EQ,
    STATE(2567), 1,
      sym_declaration_list,
    STATE(3984), 1,
      sym_type_parameters,
    STATE(4306), 1,
      sym_trait_bounds,
    STATE(4929), 1,
      sym_where_clause,
    STATE(3815), 2,
      sym_line_comment,
      sym_block_comment,
  [119520] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7737), 1,
      anon_sym_SEMI,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(2598), 1,
      sym_field_declaration_list,
    STATE(4008), 1,
      sym_type_parameters,
    STATE(4977), 1,
      sym_ordered_field_declaration_list,
    STATE(4978), 1,
      sym_where_clause,
    STATE(3816), 2,
      sym_line_comment,
      sym_block_comment,
  [119558] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7745), 1,
      anon_sym_SEMI,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1342), 1,
      sym_field_declaration_list,
    STATE(4056), 1,
      sym_type_parameters,
    STATE(5431), 1,
      sym_ordered_field_declaration_list,
    STATE(5432), 1,
      sym_where_clause,
    STATE(3817), 2,
      sym_line_comment,
      sym_block_comment,
  [119596] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7749), 1,
      anon_sym_EQ,
    STATE(2703), 1,
      sym_declaration_list,
    STATE(4047), 1,
      sym_type_parameters,
    STATE(4322), 1,
      sym_trait_bounds,
    STATE(5125), 1,
      sym_where_clause,
    STATE(3818), 2,
      sym_line_comment,
      sym_block_comment,
  [119634] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7751), 1,
      anon_sym_EQ,
    STATE(2740), 1,
      sym_declaration_list,
    STATE(4052), 1,
      sym_type_parameters,
    STATE(4326), 1,
      sym_trait_bounds,
    STATE(5150), 1,
      sym_where_clause,
    STATE(3819), 2,
      sym_line_comment,
      sym_block_comment,
  [119672] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7755), 1,
      anon_sym_EQ,
    STATE(1344), 1,
      sym_declaration_list,
    STATE(4004), 1,
      sym_type_parameters,
    STATE(4276), 1,
      sym_trait_bounds,
    STATE(5221), 1,
      sym_where_clause,
    STATE(3820), 2,
      sym_line_comment,
      sym_block_comment,
  [119710] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7757), 1,
      anon_sym_EQ,
    STATE(2089), 1,
      sym_declaration_list,
    STATE(4061), 1,
      sym_type_parameters,
    STATE(4335), 1,
      sym_trait_bounds,
    STATE(5264), 1,
      sym_where_clause,
    STATE(3821), 2,
      sym_line_comment,
      sym_block_comment,
  [119748] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3822), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5681), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [119780] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7560), 1,
      anon_sym_static,
    STATE(3823), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [119804] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4120), 1,
      sym_parameters,
    STATE(3824), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5973), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [119834] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7759), 1,
      anon_sym_LT,
    STATE(3825), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [119858] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(7761), 1,
      anon_sym_SEMI,
    STATE(1161), 1,
      sym_field_declaration_list,
    STATE(4059), 1,
      sym_type_parameters,
    STATE(5238), 1,
      sym_ordered_field_declaration_list,
    STATE(5268), 1,
      sym_where_clause,
    STATE(3826), 2,
      sym_line_comment,
      sym_block_comment,
  [119896] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5979), 1,
      anon_sym_DOT_DOT,
    STATE(3827), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5983), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [119920] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7765), 1,
      anon_sym_PLUS,
    STATE(3832), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3828), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7763), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [119946] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7767), 1,
      anon_sym_PLUS,
    STATE(3836), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3829), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7763), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [119972] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4120), 1,
      sym_parameters,
    STATE(3830), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5953), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [120002] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7769), 1,
      sym_identifier,
    STATE(3831), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [120026] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7773), 1,
      anon_sym_PLUS,
    STATE(3832), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_trait_bounds_repeat1,
    ACTIONS(7771), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [120050] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(7776), 1,
      anon_sym_SEMI,
    STATE(1143), 1,
      sym_field_declaration_list,
    STATE(4040), 1,
      sym_type_parameters,
    STATE(5278), 1,
      sym_ordered_field_declaration_list,
    STATE(5280), 1,
      sym_where_clause,
    STATE(3833), 2,
      sym_line_comment,
      sym_block_comment,
  [120088] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4120), 1,
      sym_parameters,
    STATE(3834), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5993), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [120118] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7778), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3835), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [120150] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7765), 1,
      anon_sym_PLUS,
    STATE(3832), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3836), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7780), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [120176] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7767), 1,
      anon_sym_PLUS,
    STATE(3860), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3837), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7782), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [120202] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7784), 1,
      anon_sym_EQ,
    STATE(1079), 1,
      sym_declaration_list,
    STATE(4034), 1,
      sym_type_parameters,
    STATE(4292), 1,
      sym_trait_bounds,
    STATE(5012), 1,
      sym_where_clause,
    STATE(3838), 2,
      sym_line_comment,
      sym_block_comment,
  [120240] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7786), 1,
      anon_sym_LPAREN,
    ACTIONS(7791), 1,
      anon_sym_LBRACE,
    ACTIONS(7794), 1,
      anon_sym_LBRACK,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(6188), 1,
      sym_macro_rule,
    ACTIONS(7789), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
    STATE(3839), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_macro_definition_repeat1,
  [120272] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7797), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3840), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [120304] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3841), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5016), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [120336] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7799), 1,
      anon_sym_EQ,
    STATE(1219), 1,
      sym_declaration_list,
    STATE(4022), 1,
      sym_type_parameters,
    STATE(4402), 1,
      sym_trait_bounds,
    STATE(5184), 1,
      sym_where_clause,
    STATE(3842), 2,
      sym_line_comment,
      sym_block_comment,
  [120374] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7801), 1,
      anon_sym_EQ,
    STATE(1145), 1,
      sym_declaration_list,
    STATE(4041), 1,
      sym_type_parameters,
    STATE(4422), 1,
      sym_trait_bounds,
    STATE(5283), 1,
      sym_where_clause,
    STATE(3843), 2,
      sym_line_comment,
      sym_block_comment,
  [120412] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7803), 1,
      anon_sym_EQ,
    STATE(1268), 1,
      sym_declaration_list,
    STATE(4027), 1,
      sym_type_parameters,
    STATE(4316), 1,
      sym_trait_bounds,
    STATE(5035), 1,
      sym_where_clause,
    STATE(3844), 2,
      sym_line_comment,
      sym_block_comment,
  [120450] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7808), 2,
      anon_sym_fn,
      anon_sym_extern,
    STATE(3845), 3,
      aux_sym__item_modifiers,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7805), 6,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_safe,
  [120474] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7810), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3846), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [120506] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7812), 1,
      sym_identifier,
    STATE(3847), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [120530] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(1888), 1,
      anon_sym_SQUOTE,
    ACTIONS(7241), 1,
      sym_identifier,
    ACTIONS(7243), 1,
      anon_sym_const,
    STATE(4447), 1,
      sym_lifetime,
    STATE(3848), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5273), 4,
      sym_const_parameter,
      sym_type_parameter,
      sym_lifetime_parameter,
      sym_metavariable,
  [120562] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7814), 1,
      anon_sym_EQ,
    STATE(1281), 1,
      sym_declaration_list,
    STATE(4007), 1,
      sym_type_parameters,
    STATE(4340), 1,
      sym_trait_bounds,
    STATE(5291), 1,
      sym_where_clause,
    STATE(3849), 2,
      sym_line_comment,
      sym_block_comment,
  [120600] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7816), 1,
      anon_sym_EQ,
    STATE(728), 1,
      sym_declaration_list,
    STATE(4035), 1,
      sym_type_parameters,
    STATE(4380), 1,
      sym_trait_bounds,
    STATE(5042), 1,
      sym_where_clause,
    STATE(3850), 2,
      sym_line_comment,
      sym_block_comment,
  [120638] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7765), 1,
      anon_sym_PLUS,
    STATE(3832), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3851), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7818), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [120664] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4120), 1,
      sym_parameters,
    STATE(3852), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [120694] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7820), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3853), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [120726] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7822), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3854), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [120758] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7824), 1,
      sym_identifier,
    STATE(3855), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [120782] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7826), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3856), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [120814] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7550), 1,
      anon_sym_static,
    STATE(3857), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [120838] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5959), 1,
      anon_sym_DOT_DOT,
    STATE(3858), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5963), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [120862] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5967), 1,
      anon_sym_DOT_DOT,
    STATE(3859), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5971), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [120886] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7765), 1,
      anon_sym_PLUS,
    STATE(3832), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3860), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7828), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [120912] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7830), 1,
      anon_sym_LT,
    STATE(3861), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [120936] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7832), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3862), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [120968] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7518), 1,
      anon_sym_LPAREN,
    ACTIONS(7522), 1,
      anon_sym_BANG,
    ACTIONS(7526), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7834), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7528), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3863), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [121000] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7205), 1,
      anon_sym_LPAREN,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7836), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3864), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 3,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_PIPE,
  [121032] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(7838), 1,
      anon_sym_SEMI,
    STATE(1313), 1,
      sym_field_declaration_list,
    STATE(4065), 1,
      sym_type_parameters,
    STATE(5231), 1,
      sym_ordered_field_declaration_list,
    STATE(5233), 1,
      sym_where_clause,
    STATE(3865), 2,
      sym_line_comment,
      sym_block_comment,
  [121070] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7840), 1,
      anon_sym_LT,
    STATE(3866), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121094] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5987), 1,
      anon_sym_DOT_DOT,
    STATE(3867), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5991), 8,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_BANG,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [121118] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7842), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3868), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121150] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7844), 1,
      anon_sym_SEMI,
    STATE(2444), 1,
      sym_field_declaration_list,
    STATE(3985), 1,
      sym_type_parameters,
    STATE(5159), 1,
      sym_ordered_field_declaration_list,
    STATE(5162), 1,
      sym_where_clause,
    STATE(3869), 2,
      sym_line_comment,
      sym_block_comment,
  [121188] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7846), 1,
      anon_sym_EQ,
    STATE(2445), 1,
      sym_declaration_list,
    STATE(4003), 1,
      sym_type_parameters,
    STATE(4281), 1,
      sym_trait_bounds,
    STATE(5189), 1,
      sym_where_clause,
    STATE(3870), 2,
      sym_line_comment,
      sym_block_comment,
  [121226] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7848), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3871), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121258] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7850), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3872), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121290] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7852), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3873), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121322] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7624), 1,
      anon_sym_static,
    STATE(3874), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121346] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7854), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3875), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121378] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(7856), 1,
      anon_sym_EQ,
    STATE(1358), 1,
      sym_declaration_list,
    STATE(4037), 1,
      sym_type_parameters,
    STATE(4417), 1,
      sym_trait_bounds,
    STATE(5270), 1,
      sym_where_clause,
    STATE(3876), 2,
      sym_line_comment,
      sym_block_comment,
  [121416] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7858), 1,
      sym_identifier,
    STATE(3877), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121440] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7860), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3878), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121472] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7862), 1,
      anon_sym_EQ,
    STATE(2464), 1,
      sym_declaration_list,
    STATE(4028), 1,
      sym_type_parameters,
    STATE(4289), 1,
      sym_trait_bounds,
    STATE(4992), 1,
      sym_where_clause,
    STATE(3879), 2,
      sym_line_comment,
      sym_block_comment,
  [121510] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7864), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3880), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121542] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7866), 1,
      anon_sym_SEMI,
    STATE(2500), 1,
      sym_field_declaration_list,
    STATE(4049), 1,
      sym_type_parameters,
    STATE(5144), 1,
      sym_ordered_field_declaration_list,
    STATE(5146), 1,
      sym_where_clause,
    STATE(3881), 2,
      sym_line_comment,
      sym_block_comment,
  [121580] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7868), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3882), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121612] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7767), 1,
      anon_sym_PLUS,
    STATE(3851), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3883), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7780), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [121638] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7870), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(3654), 1,
      sym_parameters,
    STATE(3884), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [121670] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7872), 1,
      anon_sym_LT,
    STATE(3885), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121694] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7874), 1,
      anon_sym_LT,
    STATE(3886), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121718] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7876), 1,
      sym_identifier,
    STATE(3887), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121742] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7612), 1,
      anon_sym_static,
    STATE(3888), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121766] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7878), 1,
      sym_identifier,
    STATE(3889), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121790] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7880), 1,
      sym_identifier,
    STATE(3890), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6029), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [121814] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7882), 1,
      anon_sym_EQ,
    STATE(2501), 1,
      sym_declaration_list,
    STATE(4050), 1,
      sym_type_parameters,
    STATE(4295), 1,
      sym_trait_bounds,
    STATE(5148), 1,
      sym_where_clause,
    STATE(3891), 2,
      sym_line_comment,
      sym_block_comment,
  [121852] = 12,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7884), 1,
      anon_sym_SEMI,
    STATE(2513), 1,
      sym_field_declaration_list,
    STATE(4055), 1,
      sym_type_parameters,
    STATE(5240), 1,
      sym_ordered_field_declaration_list,
    STATE(5241), 1,
      sym_where_clause,
    STATE(3892), 2,
      sym_line_comment,
      sym_block_comment,
  [121890] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7767), 1,
      anon_sym_PLUS,
    STATE(3828), 1,
      aux_sym_trait_bounds_repeat1,
    STATE(3893), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7828), 7,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [121916] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7886), 1,
      anon_sym_SEMI,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7890), 1,
      anon_sym_DASH_GT,
    ACTIONS(7892), 1,
      anon_sym_where,
    STATE(1320), 1,
      sym_block,
    STATE(4423), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3894), 2,
      sym_line_comment,
      sym_block_comment,
  [121951] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7894), 1,
      anon_sym_SEMI,
    STATE(751), 1,
      sym_block,
    STATE(4303), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3895), 2,
      sym_line_comment,
      sym_block_comment,
  [121986] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3896), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7896), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [122007] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7898), 1,
      anon_sym_SEMI,
    ACTIONS(7900), 1,
      anon_sym_DASH_GT,
    STATE(600), 1,
      sym_block,
    STATE(4421), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3897), 2,
      sym_line_comment,
      sym_block_comment,
  [122042] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3898), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7902), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [122063] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3899), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7771), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [122084] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3900), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7771), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [122105] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7904), 1,
      anon_sym_SEMI,
    ACTIONS(7906), 1,
      anon_sym_EQ,
    STATE(4126), 1,
      sym_type_parameters,
    STATE(4504), 1,
      sym_trait_bounds,
    STATE(4557), 1,
      sym_where_clause,
    STATE(3901), 2,
      sym_line_comment,
      sym_block_comment,
  [122140] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(7682), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7686), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7684), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3902), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7229), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [122169] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7908), 1,
      anon_sym_SEMI,
    ACTIONS(7910), 1,
      anon_sym_EQ,
    STATE(4204), 1,
      sym_type_parameters,
    STATE(4474), 1,
      sym_where_clause,
    STATE(4475), 1,
      sym_trait_bounds,
    STATE(3903), 2,
      sym_line_comment,
      sym_block_comment,
  [122204] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7912), 1,
      anon_sym_SEMI,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7916), 1,
      anon_sym_DASH_GT,
    STATE(2469), 1,
      sym_block,
    STATE(4291), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3904), 2,
      sym_line_comment,
      sym_block_comment,
  [122239] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7918), 1,
      anon_sym_SEMI,
    STATE(964), 1,
      sym_block,
    STATE(4385), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3905), 2,
      sym_line_comment,
      sym_block_comment,
  [122274] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7920), 1,
      anon_sym_SEMI,
    ACTIONS(7922), 1,
      anon_sym_EQ,
    STATE(4106), 1,
      sym_type_parameters,
    STATE(4621), 1,
      sym_where_clause,
    STATE(4625), 1,
      sym_trait_bounds,
    STATE(3906), 2,
      sym_line_comment,
      sym_block_comment,
  [122309] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7924), 1,
      anon_sym_SEMI,
    ACTIONS(7926), 1,
      anon_sym_EQ,
    STATE(4124), 1,
      sym_type_parameters,
    STATE(4668), 1,
      sym_where_clause,
    STATE(4670), 1,
      sym_trait_bounds,
    STATE(3907), 2,
      sym_line_comment,
      sym_block_comment,
  [122344] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7928), 1,
      anon_sym_SEMI,
    ACTIONS(7930), 1,
      anon_sym_DASH_GT,
    STATE(2534), 1,
      sym_block,
    STATE(4301), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3908), 2,
      sym_line_comment,
      sym_block_comment,
  [122379] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7932), 1,
      anon_sym_SEMI,
    ACTIONS(7934), 1,
      anon_sym_DASH_GT,
    STATE(2571), 1,
      sym_block,
    STATE(4307), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3909), 2,
      sym_line_comment,
      sym_block_comment,
  [122414] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7936), 1,
      anon_sym_SEMI,
    ACTIONS(7938), 1,
      anon_sym_EQ,
    STATE(4164), 1,
      sym_type_parameters,
    STATE(4823), 1,
      sym_where_clause,
    STATE(4825), 1,
      sym_trait_bounds,
    STATE(3910), 2,
      sym_line_comment,
      sym_block_comment,
  [122449] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7940), 1,
      anon_sym_SEMI,
    ACTIONS(7942), 1,
      anon_sym_DASH_GT,
    STATE(2605), 1,
      sym_block,
    STATE(4310), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3911), 2,
      sym_line_comment,
      sym_block_comment,
  [122484] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7944), 1,
      anon_sym_SEMI,
    STATE(615), 1,
      sym_block,
    STATE(4436), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3912), 2,
      sym_line_comment,
      sym_block_comment,
  [122519] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7946), 1,
      anon_sym_SEMI,
    STATE(2632), 1,
      sym_block,
    STATE(4315), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3913), 2,
      sym_line_comment,
      sym_block_comment,
  [122554] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7948), 1,
      anon_sym_SEMI,
    ACTIONS(7950), 1,
      anon_sym_DASH_GT,
    STATE(2676), 1,
      sym_block,
    STATE(4321), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3914), 2,
      sym_line_comment,
      sym_block_comment,
  [122589] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7952), 1,
      anon_sym_SEMI,
    STATE(855), 1,
      sym_block,
    STATE(4305), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3915), 2,
      sym_line_comment,
      sym_block_comment,
  [122624] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7954), 1,
      anon_sym_SEMI,
    STATE(982), 1,
      sym_block,
    STATE(4400), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3916), 2,
      sym_line_comment,
      sym_block_comment,
  [122659] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7956), 1,
      anon_sym_SEMI,
    ACTIONS(7958), 1,
      anon_sym_DASH_GT,
    STATE(2709), 1,
      sym_block,
    STATE(4324), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3917), 2,
      sym_line_comment,
      sym_block_comment,
  [122694] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7960), 1,
      anon_sym_SEMI,
    ACTIONS(7962), 1,
      anon_sym_DASH_GT,
    STATE(2725), 1,
      sym_block,
    STATE(4325), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3918), 2,
      sym_line_comment,
      sym_block_comment,
  [122729] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(7966), 1,
      sym_crate,
    STATE(3919), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [122758] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7968), 1,
      anon_sym_SEMI,
    ACTIONS(7970), 1,
      anon_sym_DASH_GT,
    STATE(2742), 1,
      sym_block,
    STATE(4327), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3920), 2,
      sym_line_comment,
      sym_block_comment,
  [122793] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7972), 1,
      anon_sym_SEMI,
    ACTIONS(7974), 1,
      anon_sym_DASH_GT,
    STATE(1216), 1,
      sym_block,
    STATE(4358), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3921), 2,
      sym_line_comment,
      sym_block_comment,
  [122828] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7976), 1,
      anon_sym_SEMI,
    ACTIONS(7978), 1,
      anon_sym_EQ,
    STATE(4127), 1,
      sym_type_parameters,
    STATE(4512), 1,
      sym_trait_bounds,
    STATE(4614), 1,
      sym_where_clause,
    STATE(3922), 2,
      sym_line_comment,
      sym_block_comment,
  [122863] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7980), 1,
      anon_sym_SEMI,
    STATE(2429), 1,
      sym_block,
    STATE(4328), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3923), 2,
      sym_line_comment,
      sym_block_comment,
  [122898] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7982), 1,
      anon_sym_SEMI,
    STATE(2038), 1,
      sym_block,
    STATE(4331), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3924), 2,
      sym_line_comment,
      sym_block_comment,
  [122933] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7984), 1,
      anon_sym_SEMI,
    ACTIONS(7986), 1,
      anon_sym_DASH_GT,
    STATE(2074), 1,
      sym_block,
    STATE(4334), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3925), 2,
      sym_line_comment,
      sym_block_comment,
  [122968] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7988), 1,
      anon_sym_SEMI,
    ACTIONS(7990), 1,
      anon_sym_DASH_GT,
    STATE(2091), 1,
      sym_block,
    STATE(4336), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3926), 2,
      sym_line_comment,
      sym_block_comment,
  [123003] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7992), 1,
      anon_sym_SEMI,
    STATE(2095), 1,
      sym_block,
    STATE(4337), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3927), 2,
      sym_line_comment,
      sym_block_comment,
  [123038] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(7994), 1,
      anon_sym_SEMI,
    ACTIONS(7996), 1,
      anon_sym_DASH_GT,
    STATE(2109), 1,
      sym_block,
    STATE(4339), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3928), 2,
      sym_line_comment,
      sym_block_comment,
  [123073] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7998), 1,
      anon_sym_SEMI,
    STATE(996), 1,
      sym_block,
    STATE(4404), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3929), 2,
      sym_line_comment,
      sym_block_comment,
  [123108] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8000), 1,
      anon_sym_SEMI,
    STATE(2165), 1,
      sym_block,
    STATE(4341), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3930), 2,
      sym_line_comment,
      sym_block_comment,
  [123143] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8002), 1,
      anon_sym_SEMI,
    STATE(2210), 1,
      sym_block,
    STATE(4344), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3931), 2,
      sym_line_comment,
      sym_block_comment,
  [123178] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8004), 1,
      anon_sym_SEMI,
    ACTIONS(8006), 1,
      anon_sym_DASH_GT,
    STATE(2224), 1,
      sym_block,
    STATE(4346), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3932), 2,
      sym_line_comment,
      sym_block_comment,
  [123213] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8008), 1,
      anon_sym_SEMI,
    STATE(2228), 1,
      sym_block,
    STATE(4347), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3933), 2,
      sym_line_comment,
      sym_block_comment,
  [123248] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3934), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5827), 8,
      anon_sym_async,
      anon_sym_const,
      anon_sym_default,
      anon_sym_fn,
      anon_sym_gen,
      anon_sym_unsafe,
      anon_sym_extern,
      anon_sym_safe,
  [123269] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3935), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8010), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [123290] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8012), 1,
      anon_sym_SEMI,
    STATE(2316), 1,
      sym_block,
    STATE(4350), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3936), 2,
      sym_line_comment,
      sym_block_comment,
  [123325] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8014), 1,
      anon_sym_SEMI,
    STATE(2334), 1,
      sym_block,
    STATE(4351), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3937), 2,
      sym_line_comment,
      sym_block_comment,
  [123360] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8016), 1,
      anon_sym_SEMI,
    STATE(2347), 1,
      sym_block,
    STATE(4352), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3938), 2,
      sym_line_comment,
      sym_block_comment,
  [123395] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8018), 1,
      anon_sym_SEMI,
    STATE(2392), 1,
      sym_block,
    STATE(4353), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3939), 2,
      sym_line_comment,
      sym_block_comment,
  [123430] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3940), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7771), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [123451] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(3941), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7771), 8,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_EQ,
      anon_sym_GT,
      anon_sym_SQUOTE,
      anon_sym_where,
  [123472] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7203), 1,
      anon_sym_PIPE,
    ACTIONS(7205), 1,
      anon_sym_LPAREN,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(7211), 1,
      anon_sym_COLON,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8020), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(3942), 2,
      sym_line_comment,
      sym_block_comment,
  [123505] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7500), 1,
      anon_sym_PIPE,
    ACTIONS(7502), 1,
      anon_sym_COLON,
    ACTIONS(8022), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3943), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6089), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COLON_COLON,
  [123532] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8025), 1,
      anon_sym_SEMI,
    STATE(674), 1,
      sym_block,
    STATE(4431), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3944), 2,
      sym_line_comment,
      sym_block_comment,
  [123567] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7500), 1,
      anon_sym_PIPE,
    ACTIONS(7502), 1,
      anon_sym_COLON,
    ACTIONS(8027), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(3945), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6103), 4,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COLON_COLON,
  [123594] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8030), 1,
      anon_sym_SEMI,
    ACTIONS(8032), 1,
      anon_sym_DASH_GT,
    STATE(1084), 1,
      sym_block,
    STATE(4312), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3946), 2,
      sym_line_comment,
      sym_block_comment,
  [123629] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8034), 1,
      anon_sym_SEMI,
    ACTIONS(8036), 1,
      anon_sym_DASH_GT,
    STATE(1176), 1,
      sym_block,
    STATE(4388), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3947), 2,
      sym_line_comment,
      sym_block_comment,
  [123664] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(8038), 1,
      sym_crate,
    STATE(3948), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [123693] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8040), 1,
      anon_sym_SEMI,
    ACTIONS(8042), 1,
      anon_sym_DASH_GT,
    STATE(712), 1,
      sym_block,
    STATE(4332), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3949), 2,
      sym_line_comment,
      sym_block_comment,
  [123728] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8044), 1,
      anon_sym_SEMI,
    ACTIONS(8046), 1,
      anon_sym_DASH_GT,
    STATE(1285), 1,
      sym_block,
    STATE(4348), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3950), 2,
      sym_line_comment,
      sym_block_comment,
  [123763] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8048), 1,
      anon_sym_SEMI,
    ACTIONS(8050), 1,
      anon_sym_DASH_GT,
    STATE(730), 1,
      sym_block,
    STATE(4382), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3951), 2,
      sym_line_comment,
      sym_block_comment,
  [123798] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8052), 1,
      anon_sym_SEMI,
    STATE(734), 1,
      sym_block,
    STATE(4392), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3952), 2,
      sym_line_comment,
      sym_block_comment,
  [123833] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8054), 1,
      anon_sym_SEMI,
    ACTIONS(8056), 1,
      anon_sym_DASH_GT,
    STATE(869), 1,
      sym_block,
    STATE(4311), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3953), 2,
      sym_line_comment,
      sym_block_comment,
  [123868] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8058), 1,
      anon_sym_SEMI,
    ACTIONS(8060), 1,
      anon_sym_DASH_GT,
    STATE(748), 1,
      sym_block,
    STATE(4412), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3954), 2,
      sym_line_comment,
      sym_block_comment,
  [123903] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8062), 1,
      anon_sym_SEMI,
    STATE(873), 1,
      sym_block,
    STATE(4313), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3955), 2,
      sym_line_comment,
      sym_block_comment,
  [123938] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8064), 1,
      anon_sym_SEMI,
    ACTIONS(8066), 1,
      anon_sym_EQ,
    STATE(4199), 1,
      sym_type_parameters,
    STATE(4791), 1,
      sym_where_clause,
    STATE(4793), 1,
      sym_trait_bounds,
    STATE(3956), 2,
      sym_line_comment,
      sym_block_comment,
  [123973] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8068), 1,
      anon_sym_SEMI,
    STATE(1041), 1,
      sym_block,
    STATE(4425), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3957), 2,
      sym_line_comment,
      sym_block_comment,
  [124008] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(8070), 1,
      sym_crate,
    STATE(3958), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [124037] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7731), 1,
      anon_sym_LT,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8072), 1,
      anon_sym_SEMI,
    ACTIONS(8074), 1,
      anon_sym_EQ,
    STATE(4253), 1,
      sym_type_parameters,
    STATE(4758), 1,
      sym_where_clause,
    STATE(4761), 1,
      sym_trait_bounds,
    STATE(3959), 2,
      sym_line_comment,
      sym_block_comment,
  [124072] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8076), 1,
      anon_sym_SEMI,
    STATE(809), 1,
      sym_block,
    STATE(4282), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3960), 2,
      sym_line_comment,
      sym_block_comment,
  [124107] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(8078), 1,
      sym_crate,
    STATE(3961), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [124136] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(8080), 1,
      sym_crate,
    STATE(3962), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [124165] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8082), 1,
      anon_sym_SEMI,
    STATE(891), 1,
      sym_block,
    STATE(4323), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3963), 2,
      sym_line_comment,
      sym_block_comment,
  [124200] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8084), 1,
      anon_sym_SEMI,
    ACTIONS(8086), 1,
      anon_sym_DASH_GT,
    STATE(1234), 1,
      sym_block,
    STATE(4406), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3964), 2,
      sym_line_comment,
      sym_block_comment,
  [124235] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(8088), 1,
      sym_crate,
    STATE(3965), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [124264] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(8090), 1,
      sym_crate,
    STATE(3966), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [124293] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8022), 1,
      anon_sym_RBRACK,
    ACTIONS(7500), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(3967), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6089), 5,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COLON_COLON,
  [124318] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8027), 1,
      anon_sym_RBRACK,
    ACTIONS(7500), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(3968), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6103), 5,
      anon_sym_SEMI,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_COLON_COLON,
  [124343] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(8092), 1,
      anon_sym_SEMI,
    ACTIONS(8094), 1,
      anon_sym_DASH_GT,
    STATE(1272), 1,
      sym_block,
    STATE(4410), 1,
      sym_where_clause,
    STATE(6427), 1,
      sym_label,
    STATE(3969), 2,
      sym_line_comment,
      sym_block_comment,
  [124378] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    ACTIONS(8096), 1,
      sym_crate,
    STATE(3970), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [124407] = 11,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7892), 1,
      anon_sym_where,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8098), 1,
      anon_sym_SEMI,
    STATE(2246), 1,
      sym_block,
    STATE(4349), 1,
      sym_where_clause,
    STATE(6433), 1,
      sym_label,
    STATE(3971), 2,
      sym_line_comment,
      sym_block_comment,
  [124442] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8104), 1,
      anon_sym_RBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5568), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3972), 2,
      sym_line_comment,
      sym_block_comment,
  [124474] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8108), 1,
      anon_sym_RBRACE,
    STATE(4036), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5727), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3973), 2,
      sym_line_comment,
      sym_block_comment,
  [124506] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8110), 1,
      anon_sym_RBRACE,
    STATE(4038), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5731), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3974), 2,
      sym_line_comment,
      sym_block_comment,
  [124538] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8112), 1,
      anon_sym_move,
    STATE(93), 1,
      sym_closure_parameters,
    STATE(2860), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(3975), 2,
      sym_line_comment,
      sym_block_comment,
  [124570] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8114), 1,
      anon_sym_RPAREN,
    STATE(4053), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5813), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3976), 2,
      sym_line_comment,
      sym_block_comment,
  [124602] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8116), 1,
      anon_sym_RBRACE,
    STATE(3972), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5849), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3977), 2,
      sym_line_comment,
      sym_block_comment,
  [124634] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8114), 1,
      anon_sym_RBRACK,
    STATE(4060), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5919), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3978), 2,
      sym_line_comment,
      sym_block_comment,
  [124666] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8118), 1,
      anon_sym_RPAREN,
    STATE(4063), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5489), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3979), 2,
      sym_line_comment,
      sym_block_comment,
  [124698] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8120), 1,
      anon_sym_RBRACE,
    STATE(4005), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5503), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3980), 2,
      sym_line_comment,
      sym_block_comment,
  [124730] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8122), 1,
      anon_sym_RPAREN,
    STATE(3994), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5622), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3981), 2,
      sym_line_comment,
      sym_block_comment,
  [124762] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8124), 1,
      anon_sym_RBRACE,
    STATE(3995), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5624), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3982), 2,
      sym_line_comment,
      sym_block_comment,
  [124794] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8126), 1,
      anon_sym_move,
    STATE(101), 1,
      sym_closure_parameters,
    STATE(583), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(3983), 2,
      sym_line_comment,
      sym_block_comment,
  [124826] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8128), 1,
      anon_sym_EQ,
    STATE(2672), 1,
      sym_declaration_list,
    STATE(4320), 1,
      sym_trait_bounds,
    STATE(5092), 1,
      sym_where_clause,
    STATE(3984), 2,
      sym_line_comment,
      sym_block_comment,
  [124858] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(8130), 1,
      anon_sym_SEMI,
    STATE(2480), 1,
      sym_field_declaration_list,
    STATE(5056), 1,
      sym_ordered_field_declaration_list,
    STATE(5063), 1,
      sym_where_clause,
    STATE(3985), 2,
      sym_line_comment,
      sym_block_comment,
  [124890] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8122), 1,
      anon_sym_RBRACK,
    STATE(3996), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5626), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3986), 2,
      sym_line_comment,
      sym_block_comment,
  [124922] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8132), 1,
      anon_sym_RPAREN,
    STATE(3997), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5630), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3987), 2,
      sym_line_comment,
      sym_block_comment,
  [124954] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8134), 1,
      anon_sym_RBRACE,
    STATE(3998), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5645), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3988), 2,
      sym_line_comment,
      sym_block_comment,
  [124986] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8136), 1,
      anon_sym_move,
    STATE(127), 1,
      sym_closure_parameters,
    STATE(3251), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(3989), 2,
      sym_line_comment,
      sym_block_comment,
  [125018] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7259), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8138), 1,
      anon_sym_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4255), 1,
      sym_trait_bounds,
    STATE(5844), 1,
      sym_parameters,
    STATE(3990), 2,
      sym_line_comment,
      sym_block_comment,
  [125050] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8140), 1,
      anon_sym_LPAREN,
    ACTIONS(8142), 1,
      anon_sym_LBRACE,
    ACTIONS(8144), 1,
      anon_sym_LBRACK,
    ACTIONS(8146), 1,
      anon_sym_RBRACK,
    ACTIONS(8148), 1,
      anon_sym_EQ,
    STATE(6268), 1,
      sym_delim_token_tree,
    STATE(3991), 2,
      sym_line_comment,
      sym_block_comment,
  [125082] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7574), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8140), 1,
      anon_sym_LPAREN,
    ACTIONS(8142), 1,
      anon_sym_LBRACE,
    ACTIONS(8144), 1,
      anon_sym_LBRACK,
    ACTIONS(8150), 1,
      anon_sym_RBRACK,
    ACTIONS(8152), 1,
      anon_sym_EQ,
    STATE(6305), 1,
      sym_delim_token_tree,
    STATE(3992), 2,
      sym_line_comment,
      sym_block_comment,
  [125114] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7227), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8140), 1,
      anon_sym_LPAREN,
    ACTIONS(8142), 1,
      anon_sym_LBRACE,
    ACTIONS(8144), 1,
      anon_sym_LBRACK,
    ACTIONS(8150), 1,
      anon_sym_RBRACK,
    ACTIONS(8152), 1,
      anon_sym_EQ,
    STATE(6305), 1,
      sym_delim_token_tree,
    STATE(3993), 2,
      sym_line_comment,
      sym_block_comment,
  [125146] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8154), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5531), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3994), 2,
      sym_line_comment,
      sym_block_comment,
  [125178] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8156), 1,
      anon_sym_RBRACE,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5533), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3995), 2,
      sym_line_comment,
      sym_block_comment,
  [125210] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8154), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5543), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3996), 2,
      sym_line_comment,
      sym_block_comment,
  [125242] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8158), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5596), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3997), 2,
      sym_line_comment,
      sym_block_comment,
  [125274] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8160), 1,
      anon_sym_RBRACE,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5603), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3998), 2,
      sym_line_comment,
      sym_block_comment,
  [125306] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8158), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5648), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(3999), 2,
      sym_line_comment,
      sym_block_comment,
  [125338] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7576), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8140), 1,
      anon_sym_LPAREN,
    ACTIONS(8142), 1,
      anon_sym_LBRACE,
    ACTIONS(8144), 1,
      anon_sym_LBRACK,
    ACTIONS(8150), 1,
      anon_sym_RBRACK,
    ACTIONS(8152), 1,
      anon_sym_EQ,
    STATE(6305), 1,
      sym_delim_token_tree,
    STATE(4000), 2,
      sym_line_comment,
      sym_block_comment,
  [125370] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8132), 1,
      anon_sym_RBRACK,
    STATE(3999), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5647), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4001), 2,
      sym_line_comment,
      sym_block_comment,
  [125402] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8118), 1,
      anon_sym_RBRACK,
    STATE(4054), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5513), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4002), 2,
      sym_line_comment,
      sym_block_comment,
  [125434] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8162), 1,
      anon_sym_EQ,
    STATE(2483), 1,
      sym_declaration_list,
    STATE(4293), 1,
      sym_trait_bounds,
    STATE(5070), 1,
      sym_where_clause,
    STATE(4003), 2,
      sym_line_comment,
      sym_block_comment,
  [125466] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8164), 1,
      anon_sym_EQ,
    STATE(1109), 1,
      sym_declaration_list,
    STATE(4391), 1,
      sym_trait_bounds,
    STATE(5105), 1,
      sym_where_clause,
    STATE(4004), 2,
      sym_line_comment,
      sym_block_comment,
  [125498] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8166), 1,
      anon_sym_RBRACE,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5574), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4005), 2,
      sym_line_comment,
      sym_block_comment,
  [125530] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(109), 1,
      aux_sym_string_literal_token1,
    ACTIONS(123), 1,
      sym__raw_string_literal_start,
    STATE(4006), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5078), 2,
      sym_string_literal,
      sym_raw_string_literal,
    ACTIONS(7964), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [125556] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8168), 1,
      anon_sym_EQ,
    STATE(1166), 1,
      sym_declaration_list,
    STATE(4386), 1,
      sym_trait_bounds,
    STATE(5075), 1,
      sym_where_clause,
    STATE(4007), 2,
      sym_line_comment,
      sym_block_comment,
  [125588] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(8170), 1,
      anon_sym_SEMI,
    STATE(2717), 1,
      sym_field_declaration_list,
    STATE(5134), 1,
      sym_ordered_field_declaration_list,
    STATE(5136), 1,
      sym_where_clause,
    STATE(4008), 2,
      sym_line_comment,
      sym_block_comment,
  [125620] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8172), 1,
      anon_sym_RPAREN,
    STATE(4018), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5728), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4009), 2,
      sym_line_comment,
      sym_block_comment,
  [125652] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8172), 1,
      anon_sym_RBRACK,
    STATE(4019), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5729), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4010), 2,
      sym_line_comment,
      sym_block_comment,
  [125684] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8174), 1,
      anon_sym_RPAREN,
    STATE(4020), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5730), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4011), 2,
      sym_line_comment,
      sym_block_comment,
  [125716] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8174), 1,
      anon_sym_RBRACK,
    STATE(4021), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5732), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4012), 2,
      sym_line_comment,
      sym_block_comment,
  [125748] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8178), 1,
      anon_sym_COLON,
    ACTIONS(8180), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8176), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4013), 2,
      sym_line_comment,
      sym_block_comment,
  [125776] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8180), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4014), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [125802] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8178), 1,
      anon_sym_COLON,
    ACTIONS(8182), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8176), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4015), 2,
      sym_line_comment,
      sym_block_comment,
  [125830] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8184), 1,
      anon_sym_RBRACE,
    STATE(4058), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5527), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4016), 2,
      sym_line_comment,
      sym_block_comment,
  [125862] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8186), 1,
      anon_sym_RBRACE,
    STATE(4062), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5528), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4017), 2,
      sym_line_comment,
      sym_block_comment,
  [125894] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8188), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5765), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4018), 2,
      sym_line_comment,
      sym_block_comment,
  [125926] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8188), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5766), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4019), 2,
      sym_line_comment,
      sym_block_comment,
  [125958] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8190), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5767), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4020), 2,
      sym_line_comment,
      sym_block_comment,
  [125990] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8190), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5768), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4021), 2,
      sym_line_comment,
      sym_block_comment,
  [126022] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8192), 1,
      anon_sym_EQ,
    STATE(707), 1,
      sym_declaration_list,
    STATE(4266), 1,
      sym_trait_bounds,
    STATE(5011), 1,
      sym_where_clause,
    STATE(4022), 2,
      sym_line_comment,
      sym_block_comment,
  [126054] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8194), 1,
      anon_sym_RPAREN,
    STATE(4029), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5776), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4023), 2,
      sym_line_comment,
      sym_block_comment,
  [126086] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8196), 1,
      anon_sym_EQ,
    STATE(2663), 1,
      sym_declaration_list,
    STATE(4319), 1,
      sym_trait_bounds,
    STATE(5074), 1,
      sym_where_clause,
    STATE(4024), 2,
      sym_line_comment,
      sym_block_comment,
  [126118] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8198), 1,
      anon_sym_RPAREN,
    STATE(4031), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5779), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4025), 2,
      sym_line_comment,
      sym_block_comment,
  [126150] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8198), 1,
      anon_sym_RBRACK,
    STATE(4032), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5780), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4026), 2,
      sym_line_comment,
      sym_block_comment,
  [126182] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8200), 1,
      anon_sym_EQ,
    STATE(1135), 1,
      sym_declaration_list,
    STATE(4377), 1,
      sym_trait_bounds,
    STATE(5029), 1,
      sym_where_clause,
    STATE(4027), 2,
      sym_line_comment,
      sym_block_comment,
  [126214] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8202), 1,
      anon_sym_EQ,
    STATE(2528), 1,
      sym_declaration_list,
    STATE(4299), 1,
      sym_trait_bounds,
    STATE(4866), 1,
      sym_where_clause,
    STATE(4028), 2,
      sym_line_comment,
      sym_block_comment,
  [126246] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8204), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5800), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4029), 2,
      sym_line_comment,
      sym_block_comment,
  [126278] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8204), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5801), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4030), 2,
      sym_line_comment,
      sym_block_comment,
  [126310] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8206), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5802), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4031), 2,
      sym_line_comment,
      sym_block_comment,
  [126342] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8206), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5803), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4032), 2,
      sym_line_comment,
      sym_block_comment,
  [126374] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8208), 1,
      anon_sym_move,
    STATE(105), 1,
      sym_closure_parameters,
    STATE(2860), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4033), 2,
      sym_line_comment,
      sym_block_comment,
  [126406] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8210), 1,
      anon_sym_EQ,
    STATE(1208), 1,
      sym_declaration_list,
    STATE(4429), 1,
      sym_trait_bounds,
    STATE(5356), 1,
      sym_where_clause,
    STATE(4034), 2,
      sym_line_comment,
      sym_block_comment,
  [126438] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8212), 1,
      anon_sym_EQ,
    STATE(866), 1,
      sym_declaration_list,
    STATE(4309), 1,
      sym_trait_bounds,
    STATE(4982), 1,
      sym_where_clause,
    STATE(4035), 2,
      sym_line_comment,
      sym_block_comment,
  [126470] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8214), 1,
      anon_sym_RBRACE,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5808), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4036), 2,
      sym_line_comment,
      sym_block_comment,
  [126502] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8216), 1,
      anon_sym_EQ,
    STATE(745), 1,
      sym_declaration_list,
    STATE(4409), 1,
      sym_trait_bounds,
    STATE(5228), 1,
      sym_where_clause,
    STATE(4037), 2,
      sym_line_comment,
      sym_block_comment,
  [126534] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8218), 1,
      anon_sym_RBRACE,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5809), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4038), 2,
      sym_line_comment,
      sym_block_comment,
  [126566] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(8220), 1,
      sym_identifier,
    ACTIONS(8222), 1,
      anon_sym_async,
    ACTIONS(8224), 1,
      anon_sym_ref,
    ACTIONS(8226), 1,
      sym_mutable_specifier,
    ACTIONS(8228), 1,
      anon_sym_move,
    STATE(101), 1,
      sym_closure_parameters,
    STATE(4039), 2,
      sym_line_comment,
      sym_block_comment,
  [126598] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(8230), 1,
      anon_sym_SEMI,
    STATE(1296), 1,
      sym_field_declaration_list,
    STATE(5061), 1,
      sym_ordered_field_declaration_list,
    STATE(5062), 1,
      sym_where_clause,
    STATE(4040), 2,
      sym_line_comment,
      sym_block_comment,
  [126630] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8232), 1,
      anon_sym_EQ,
    STATE(1299), 1,
      sym_declaration_list,
    STATE(4389), 1,
      sym_trait_bounds,
    STATE(5091), 1,
      sym_where_clause,
    STATE(4041), 2,
      sym_line_comment,
      sym_block_comment,
  [126662] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8234), 1,
      anon_sym_move,
    STATE(101), 1,
      sym_closure_parameters,
    STATE(2860), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4042), 2,
      sym_line_comment,
      sym_block_comment,
  [126694] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8238), 1,
      anon_sym_COLON,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8236), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4043), 2,
      sym_line_comment,
      sym_block_comment,
  [126722] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8244), 1,
      anon_sym_STAR,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(5181), 1,
      sym_use_list,
    ACTIONS(8242), 2,
      sym_identifier,
      sym_super,
    STATE(4044), 2,
      sym_line_comment,
      sym_block_comment,
  [126752] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8244), 1,
      anon_sym_STAR,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(5181), 1,
      sym_use_list,
    ACTIONS(8242), 2,
      sym_identifier,
      sym_super,
    STATE(4045), 2,
      sym_line_comment,
      sym_block_comment,
  [126782] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8246), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4046), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 3,
      anon_sym_SEMI,
      anon_sym_RBRACK,
      anon_sym_PLUS,
  [126808] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8248), 1,
      anon_sym_EQ,
    STATE(2069), 1,
      sym_declaration_list,
    STATE(4333), 1,
      sym_trait_bounds,
    STATE(5245), 1,
      sym_where_clause,
    STATE(4047), 2,
      sym_line_comment,
      sym_block_comment,
  [126840] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8252), 1,
      anon_sym_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8250), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4048), 2,
      sym_line_comment,
      sym_block_comment,
  [126868] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(8254), 1,
      anon_sym_SEMI,
    STATE(2581), 1,
      sym_field_declaration_list,
    STATE(4956), 1,
      sym_ordered_field_declaration_list,
    STATE(4959), 1,
      sym_where_clause,
    STATE(4049), 2,
      sym_line_comment,
      sym_block_comment,
  [126900] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8256), 1,
      anon_sym_EQ,
    STATE(2584), 1,
      sym_declaration_list,
    STATE(4308), 1,
      sym_trait_bounds,
    STATE(4962), 1,
      sym_where_clause,
    STATE(4050), 2,
      sym_line_comment,
      sym_block_comment,
  [126932] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(8222), 1,
      anon_sym_async,
    ACTIONS(8228), 1,
      anon_sym_move,
    ACTIONS(8258), 1,
      sym_identifier,
    ACTIONS(8260), 1,
      anon_sym_ref,
    ACTIONS(8262), 1,
      sym_mutable_specifier,
    STATE(101), 1,
      sym_closure_parameters,
    STATE(4051), 2,
      sym_line_comment,
      sym_block_comment,
  [126964] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8264), 1,
      anon_sym_EQ,
    STATE(2106), 1,
      sym_declaration_list,
    STATE(4338), 1,
      sym_trait_bounds,
    STATE(5281), 1,
      sym_where_clause,
    STATE(4052), 2,
      sym_line_comment,
      sym_block_comment,
  [126996] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8266), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5566), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4053), 2,
      sym_line_comment,
      sym_block_comment,
  [127028] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8268), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5576), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4054), 2,
      sym_line_comment,
      sym_block_comment,
  [127060] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(8270), 1,
      anon_sym_SEMI,
    STATE(2613), 1,
      sym_field_declaration_list,
    STATE(4988), 1,
      sym_ordered_field_declaration_list,
    STATE(4989), 1,
      sym_where_clause,
    STATE(4055), 2,
      sym_line_comment,
      sym_block_comment,
  [127092] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(8272), 1,
      anon_sym_SEMI,
    STATE(1101), 1,
      sym_field_declaration_list,
    STATE(5064), 1,
      sym_ordered_field_declaration_list,
    STATE(5066), 1,
      sym_where_clause,
    STATE(4056), 2,
      sym_line_comment,
      sym_block_comment,
  [127124] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(8274), 1,
      anon_sym_LBRACE,
    ACTIONS(8278), 1,
      anon_sym_EQ,
    ACTIONS(8276), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4057), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5371), 2,
      sym_field_declaration_list,
      sym_ordered_field_declaration_list,
  [127152] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8280), 1,
      anon_sym_RBRACE,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5679), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4058), 2,
      sym_line_comment,
      sym_block_comment,
  [127184] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(8282), 1,
      anon_sym_SEMI,
    STATE(1330), 1,
      sym_field_declaration_list,
    STATE(5320), 1,
      sym_ordered_field_declaration_list,
    STATE(5321), 1,
      sym_where_clause,
    STATE(4059), 2,
      sym_line_comment,
      sym_block_comment,
  [127216] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8266), 1,
      anon_sym_RBRACK,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5569), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4060), 2,
      sym_line_comment,
      sym_block_comment,
  [127248] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8284), 1,
      anon_sym_EQ,
    STATE(2221), 1,
      sym_declaration_list,
    STATE(4345), 1,
      sym_trait_bounds,
    STATE(5349), 1,
      sym_where_clause,
    STATE(4061), 2,
      sym_line_comment,
      sym_block_comment,
  [127280] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8286), 1,
      anon_sym_RBRACE,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5680), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4062), 2,
      sym_line_comment,
      sym_block_comment,
  [127312] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8268), 1,
      anon_sym_RPAREN,
    STATE(3839), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5572), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4063), 2,
      sym_line_comment,
      sym_block_comment,
  [127344] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(8274), 1,
      anon_sym_LBRACE,
    ACTIONS(8290), 1,
      anon_sym_EQ,
    ACTIONS(8288), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4064), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5317), 2,
      sym_field_declaration_list,
      sym_ordered_field_declaration_list,
  [127372] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(8292), 1,
      anon_sym_SEMI,
    STATE(1246), 1,
      sym_field_declaration_list,
    STATE(5216), 1,
      sym_ordered_field_declaration_list,
    STATE(5217), 1,
      sym_where_clause,
    STATE(4065), 2,
      sym_line_comment,
      sym_block_comment,
  [127404] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7739), 1,
      anon_sym_LPAREN,
    ACTIONS(8274), 1,
      anon_sym_LBRACE,
    ACTIONS(8296), 1,
      anon_sym_EQ,
    ACTIONS(8294), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4066), 2,
      sym_line_comment,
      sym_block_comment,
    STATE(5350), 2,
      sym_field_declaration_list,
      sym_ordered_field_declaration_list,
  [127432] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8182), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4067), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [127458] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7259), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7514), 1,
      anon_sym_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4255), 1,
      sym_trait_bounds,
    STATE(5844), 1,
      sym_parameters,
    STATE(4068), 2,
      sym_line_comment,
      sym_block_comment,
  [127490] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8300), 1,
      anon_sym_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8298), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4069), 2,
      sym_line_comment,
      sym_block_comment,
  [127518] = 10,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8194), 1,
      anon_sym_RBRACK,
    STATE(4030), 1,
      aux_sym_macro_definition_repeat1,
    STATE(5777), 1,
      sym_macro_rule,
    STATE(6174), 1,
      sym_token_tree_pattern,
    STATE(4070), 2,
      sym_line_comment,
      sym_block_comment,
  [127550] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8302), 1,
      anon_sym_SEMI,
    STATE(819), 1,
      sym_declaration_list,
    STATE(4907), 1,
      sym_where_clause,
    STATE(4071), 2,
      sym_line_comment,
      sym_block_comment,
  [127579] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8304), 1,
      anon_sym_SEMI,
    STATE(851), 1,
      sym_declaration_list,
    STATE(4881), 1,
      sym_where_clause,
    STATE(4072), 2,
      sym_line_comment,
      sym_block_comment,
  [127608] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6015), 2,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
    ACTIONS(7711), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(8306), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4073), 2,
      sym_line_comment,
      sym_block_comment,
  [127631] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(2515), 1,
      sym_field_declaration_list,
    STATE(4673), 1,
      sym_type_parameters,
    STATE(5519), 1,
      sym_where_clause,
    STATE(4074), 2,
      sym_line_comment,
      sym_block_comment,
  [127660] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6149), 1,
      anon_sym_COLON,
    ACTIONS(8309), 1,
      anon_sym_EQ,
    STATE(4075), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6147), 4,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_COLON_COLON,
  [127683] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8311), 1,
      anon_sym_SEMI,
    STATE(1069), 1,
      sym_declaration_list,
    STATE(5163), 1,
      sym_where_clause,
    STATE(4076), 2,
      sym_line_comment,
      sym_block_comment,
  [127712] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8313), 1,
      anon_sym_COMMA,
    ACTIONS(8315), 1,
      anon_sym_GT,
    STATE(5333), 1,
      sym_trait_bounds,
    STATE(5334), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4077), 2,
      sym_line_comment,
      sym_block_comment,
  [127741] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8317), 1,
      anon_sym_SEMI,
    STATE(2267), 1,
      sym_declaration_list,
    STATE(5369), 1,
      sym_where_clause,
    STATE(4078), 2,
      sym_line_comment,
      sym_block_comment,
  [127770] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8319), 1,
      anon_sym_SEMI,
    STATE(2269), 1,
      sym_declaration_list,
    STATE(5374), 1,
      sym_where_clause,
    STATE(4079), 2,
      sym_line_comment,
      sym_block_comment,
  [127799] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8321), 1,
      anon_sym_SEMI,
    STATE(2286), 1,
      sym_declaration_list,
    STATE(5376), 1,
      sym_where_clause,
    STATE(4080), 2,
      sym_line_comment,
      sym_block_comment,
  [127828] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8323), 1,
      anon_sym_SEMI,
    STATE(2288), 1,
      sym_declaration_list,
    STATE(5379), 1,
      sym_where_clause,
    STATE(4081), 2,
      sym_line_comment,
      sym_block_comment,
  [127857] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8325), 1,
      anon_sym_SEMI,
    STATE(2303), 1,
      sym_declaration_list,
    STATE(5385), 1,
      sym_where_clause,
    STATE(4082), 2,
      sym_line_comment,
      sym_block_comment,
  [127886] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8327), 1,
      anon_sym_SEMI,
    STATE(2305), 1,
      sym_declaration_list,
    STATE(5387), 1,
      sym_where_clause,
    STATE(4083), 2,
      sym_line_comment,
      sym_block_comment,
  [127915] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8329), 1,
      anon_sym_SEMI,
    STATE(2307), 1,
      sym_declaration_list,
    STATE(5389), 1,
      sym_where_clause,
    STATE(4084), 2,
      sym_line_comment,
      sym_block_comment,
  [127944] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8331), 1,
      anon_sym_SEMI,
    STATE(2309), 1,
      sym_declaration_list,
    STATE(5391), 1,
      sym_where_clause,
    STATE(4085), 2,
      sym_line_comment,
      sym_block_comment,
  [127973] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1382), 1,
      anon_sym_DOT_DOT,
    STATE(4086), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1384), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [127994] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1163), 1,
      sym_field_declaration_list,
    STATE(4638), 1,
      sym_type_parameters,
    STATE(5873), 1,
      sym_where_clause,
    STATE(4087), 2,
      sym_line_comment,
      sym_block_comment,
  [128023] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8333), 1,
      anon_sym_if,
    STATE(6308), 1,
      sym_label,
    STATE(2938), 2,
      sym_if_expression,
      sym_block,
    STATE(4088), 2,
      sym_line_comment,
      sym_block_comment,
  [128050] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8335), 1,
      anon_sym_SEMI,
    STATE(2377), 1,
      sym_declaration_list,
    STATE(5411), 1,
      sym_where_clause,
    STATE(4089), 2,
      sym_line_comment,
      sym_block_comment,
  [128079] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8337), 1,
      anon_sym_SEMI,
    STATE(2379), 1,
      sym_declaration_list,
    STATE(5412), 1,
      sym_where_clause,
    STATE(4090), 2,
      sym_line_comment,
      sym_block_comment,
  [128108] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8339), 1,
      anon_sym_SEMI,
    STATE(2538), 1,
      sym_declaration_list,
    STATE(4882), 1,
      sym_where_clause,
    STATE(4091), 2,
      sym_line_comment,
      sym_block_comment,
  [128137] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8341), 1,
      anon_sym_SEMI,
    STATE(1222), 1,
      sym_declaration_list,
    STATE(5114), 1,
      sym_where_clause,
    STATE(4092), 2,
      sym_line_comment,
      sym_block_comment,
  [128166] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8343), 1,
      anon_sym_SEMI,
    STATE(2542), 1,
      sym_declaration_list,
    STATE(4889), 1,
      sym_where_clause,
    STATE(4093), 2,
      sym_line_comment,
      sym_block_comment,
  [128195] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(112), 1,
      sym_closure_parameters,
    STATE(2962), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4094), 2,
      sym_line_comment,
      sym_block_comment,
  [128224] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8345), 1,
      anon_sym_SEMI,
    STATE(1071), 1,
      sym_declaration_list,
    STATE(5164), 1,
      sym_where_clause,
    STATE(4095), 2,
      sym_line_comment,
      sym_block_comment,
  [128253] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1119), 1,
      sym_enum_variant_list,
    STATE(4748), 1,
      sym_type_parameters,
    STATE(5773), 1,
      sym_where_clause,
    STATE(4096), 2,
      sym_line_comment,
      sym_block_comment,
  [128282] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8349), 1,
      anon_sym_SEMI,
    STATE(2563), 1,
      sym_declaration_list,
    STATE(4917), 1,
      sym_where_clause,
    STATE(4097), 2,
      sym_line_comment,
      sym_block_comment,
  [128311] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8351), 1,
      anon_sym_COMMA,
    ACTIONS(8353), 1,
      anon_sym_GT,
    STATE(4931), 1,
      sym_trait_bounds,
    STATE(4932), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4098), 2,
      sym_line_comment,
      sym_block_comment,
  [128340] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8355), 1,
      anon_sym_SEMI,
    STATE(1229), 1,
      sym_declaration_list,
    STATE(5275), 1,
      sym_where_clause,
    STATE(4099), 2,
      sym_line_comment,
      sym_block_comment,
  [128369] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8357), 1,
      anon_sym_RBRACE,
    STATE(5025), 1,
      sym_macro_rule_v2,
    STATE(5997), 1,
      sym_token_tree_pattern,
    STATE(4100), 2,
      sym_line_comment,
      sym_block_comment,
  [128398] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8359), 1,
      anon_sym_if,
    STATE(6434), 1,
      sym_label,
    STATE(587), 2,
      sym_if_expression,
      sym_block,
    STATE(4101), 2,
      sym_line_comment,
      sym_block_comment,
  [128425] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8361), 1,
      anon_sym_SEMI,
    STATE(914), 1,
      sym_declaration_list,
    STATE(5257), 1,
      sym_where_clause,
    STATE(4102), 2,
      sym_line_comment,
      sym_block_comment,
  [128454] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8363), 1,
      anon_sym_SEMI,
    STATE(2573), 1,
      sym_declaration_list,
    STATE(4945), 1,
      sym_where_clause,
    STATE(4103), 2,
      sym_line_comment,
      sym_block_comment,
  [128483] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7490), 1,
      anon_sym_DOT_DOT,
    STATE(4104), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7488), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [128504] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8365), 1,
      anon_sym_SEMI,
    STATE(933), 1,
      sym_declaration_list,
    STATE(4895), 1,
      sym_where_clause,
    STATE(4105), 2,
      sym_line_comment,
      sym_block_comment,
  [128533] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8367), 1,
      anon_sym_SEMI,
    ACTIONS(8369), 1,
      anon_sym_EQ,
    STATE(4814), 1,
      sym_where_clause,
    STATE(4819), 1,
      sym_trait_bounds,
    STATE(4106), 2,
      sym_line_comment,
      sym_block_comment,
  [128562] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8371), 1,
      anon_sym_SEMI,
    STATE(2591), 1,
      sym_declaration_list,
    STATE(4973), 1,
      sym_where_clause,
    STATE(4107), 2,
      sym_line_comment,
      sym_block_comment,
  [128591] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(104), 1,
      sym_closure_parameters,
    STATE(2962), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4108), 2,
      sym_line_comment,
      sym_block_comment,
  [128620] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2594), 1,
      sym_enum_variant_list,
    STATE(4822), 1,
      sym_type_parameters,
    STATE(5652), 1,
      sym_where_clause,
    STATE(4109), 2,
      sym_line_comment,
      sym_block_comment,
  [128649] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(94), 1,
      sym_closure_parameters,
    STATE(474), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(4110), 2,
      sym_line_comment,
      sym_block_comment,
  [128678] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8375), 1,
      anon_sym_SEMI,
    STATE(916), 1,
      sym_declaration_list,
    STATE(5286), 1,
      sym_where_clause,
    STATE(4111), 2,
      sym_line_comment,
      sym_block_comment,
  [128707] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8377), 1,
      anon_sym_DASH_GT,
    STATE(4112), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6043), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [128728] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8379), 1,
      anon_sym_COMMA,
    ACTIONS(8381), 1,
      anon_sym_GT,
    STATE(4996), 1,
      sym_trait_bounds,
    STATE(4997), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4113), 2,
      sym_line_comment,
      sym_block_comment,
  [128757] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8383), 1,
      anon_sym_if,
    STATE(6435), 1,
      sym_label,
    STATE(3431), 2,
      sym_if_expression,
      sym_block,
    STATE(4114), 2,
      sym_line_comment,
      sym_block_comment,
  [128784] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(2600), 1,
      sym_field_declaration_list,
    STATE(4827), 1,
      sym_type_parameters,
    STATE(5657), 1,
      sym_where_clause,
    STATE(4115), 2,
      sym_line_comment,
      sym_block_comment,
  [128813] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8385), 1,
      anon_sym_SEMI,
    STATE(1242), 1,
      sym_declaration_list,
    STATE(5262), 1,
      sym_where_clause,
    STATE(4116), 2,
      sym_line_comment,
      sym_block_comment,
  [128842] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(94), 1,
      sym_closure_parameters,
    STATE(586), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4117), 2,
      sym_line_comment,
      sym_block_comment,
  [128871] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(8387), 1,
      anon_sym_COLON,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(5845), 1,
      sym_parameters,
    STATE(4118), 2,
      sym_line_comment,
      sym_block_comment,
  [128900] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(129), 1,
      sym_closure_parameters,
    STATE(3331), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4119), 2,
      sym_line_comment,
      sym_block_comment,
  [128929] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8389), 1,
      anon_sym_DASH_GT,
    STATE(4120), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6211), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [128950] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8391), 1,
      anon_sym_SEMI,
    STATE(662), 1,
      sym_declaration_list,
    STATE(5153), 1,
      sym_where_clause,
    STATE(4121), 2,
      sym_line_comment,
      sym_block_comment,
  [128979] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8393), 1,
      anon_sym_SEMI,
    STATE(935), 1,
      sym_declaration_list,
    STATE(4904), 1,
      sym_where_clause,
    STATE(4122), 2,
      sym_line_comment,
      sym_block_comment,
  [129008] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8395), 1,
      anon_sym_SEMI,
    STATE(664), 1,
      sym_declaration_list,
    STATE(5194), 1,
      sym_where_clause,
    STATE(4123), 2,
      sym_line_comment,
      sym_block_comment,
  [129037] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8397), 1,
      anon_sym_SEMI,
    ACTIONS(8399), 1,
      anon_sym_EQ,
    STATE(4853), 1,
      sym_where_clause,
    STATE(4854), 1,
      sym_trait_bounds,
    STATE(4124), 2,
      sym_line_comment,
      sym_block_comment,
  [129066] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7482), 1,
      anon_sym_DOT_DOT,
    STATE(4125), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7480), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [129087] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8401), 1,
      anon_sym_SEMI,
    ACTIONS(8403), 1,
      anon_sym_EQ,
    STATE(4816), 1,
      sym_where_clause,
    STATE(4817), 1,
      sym_trait_bounds,
    STATE(4126), 2,
      sym_line_comment,
      sym_block_comment,
  [129116] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8405), 1,
      anon_sym_SEMI,
    ACTIONS(8407), 1,
      anon_sym_EQ,
    STATE(4705), 1,
      sym_where_clause,
    STATE(4707), 1,
      sym_trait_bounds,
    STATE(4127), 2,
      sym_line_comment,
      sym_block_comment,
  [129145] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7526), 1,
      anon_sym_DOT_DOT,
    ACTIONS(7528), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4128), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [129168] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(94), 1,
      sym_closure_parameters,
    STATE(2962), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4129), 2,
      sym_line_comment,
      sym_block_comment,
  [129197] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4130), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8409), 6,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_SQUOTE,
      anon_sym_where,
  [129216] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8411), 1,
      anon_sym_COMMA,
    STATE(4131), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_where_clause_repeat1,
    ACTIONS(8409), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_where,
  [129237] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1378), 1,
      anon_sym_DOT_DOT,
    STATE(4132), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1380), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [129258] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2435), 1,
      sym_enum_variant_list,
    STATE(4462), 1,
      sym_type_parameters,
    STATE(5793), 1,
      sym_where_clause,
    STATE(4133), 2,
      sym_line_comment,
      sym_block_comment,
  [129287] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8414), 1,
      anon_sym_SEMI,
    STATE(2437), 1,
      sym_declaration_list,
    STATE(5140), 1,
      sym_where_clause,
    STATE(4134), 2,
      sym_line_comment,
      sym_block_comment,
  [129316] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8416), 1,
      anon_sym_SEMI,
    STATE(2634), 1,
      sym_declaration_list,
    STATE(5032), 1,
      sym_where_clause,
    STATE(4135), 2,
      sym_line_comment,
      sym_block_comment,
  [129345] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8418), 1,
      anon_sym_SEMI,
    STATE(2636), 1,
      sym_declaration_list,
    STATE(5033), 1,
      sym_where_clause,
    STATE(4136), 2,
      sym_line_comment,
      sym_block_comment,
  [129374] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8422), 1,
      anon_sym_COMMA,
    STATE(4179), 1,
      aux_sym_where_clause_repeat1,
    STATE(4137), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8420), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_where,
  [129397] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8424), 1,
      anon_sym_SEMI,
    STATE(2640), 1,
      sym_declaration_list,
    STATE(5048), 1,
      sym_where_clause,
    STATE(4138), 2,
      sym_line_comment,
      sym_block_comment,
  [129426] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8426), 1,
      anon_sym_SEMI,
    STATE(2642), 1,
      sym_declaration_list,
    STATE(5051), 1,
      sym_where_clause,
    STATE(4139), 2,
      sym_line_comment,
      sym_block_comment,
  [129455] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8428), 1,
      anon_sym_SEMI,
    STATE(676), 1,
      sym_declaration_list,
    STATE(5373), 1,
      sym_where_clause,
    STATE(4140), 2,
      sym_line_comment,
      sym_block_comment,
  [129484] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8430), 1,
      anon_sym_SEMI,
    STATE(1116), 1,
      sym_declaration_list,
    STATE(5157), 1,
      sym_where_clause,
    STATE(4141), 2,
      sym_line_comment,
      sym_block_comment,
  [129513] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8432), 1,
      anon_sym_SEMI,
    STATE(678), 1,
      sym_declaration_list,
    STATE(5420), 1,
      sym_where_clause,
    STATE(4142), 2,
      sym_line_comment,
      sym_block_comment,
  [129542] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8434), 1,
      anon_sym_SEMI,
    STATE(788), 1,
      sym_declaration_list,
    STATE(5147), 1,
      sym_where_clause,
    STATE(4143), 2,
      sym_line_comment,
      sym_block_comment,
  [129571] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8436), 1,
      anon_sym_SEMI,
    STATE(682), 1,
      sym_declaration_list,
    STATE(5246), 1,
      sym_where_clause,
    STATE(4144), 2,
      sym_line_comment,
      sym_block_comment,
  [129600] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8438), 1,
      anon_sym_SEMI,
    STATE(684), 1,
      sym_declaration_list,
    STATE(5095), 1,
      sym_where_clause,
    STATE(4145), 2,
      sym_line_comment,
      sym_block_comment,
  [129629] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8440), 1,
      anon_sym_SEMI,
    STATE(790), 1,
      sym_declaration_list,
    STATE(5296), 1,
      sym_where_clause,
    STATE(4146), 2,
      sym_line_comment,
      sym_block_comment,
  [129658] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(5845), 1,
      sym_parameters,
    STATE(4147), 2,
      sym_line_comment,
      sym_block_comment,
  [129687] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1454), 1,
      anon_sym_DOT_DOT,
    STATE(4148), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1456), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [129708] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7778), 1,
      anon_sym_for,
    STATE(4149), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [129731] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(2447), 1,
      sym_field_declaration_list,
    STATE(4480), 1,
      sym_type_parameters,
    STATE(5817), 1,
      sym_where_clause,
    STATE(4150), 2,
      sym_line_comment,
      sym_block_comment,
  [129760] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8442), 1,
      anon_sym_SEMI,
    STATE(950), 1,
      sym_declaration_list,
    STATE(5008), 1,
      sym_where_clause,
    STATE(4151), 2,
      sym_line_comment,
      sym_block_comment,
  [129789] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7676), 1,
      anon_sym_LBRACE,
    ACTIONS(8444), 1,
      anon_sym_LPAREN,
    ACTIONS(8446), 1,
      anon_sym_COLON_COLON,
    STATE(4152), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [129814] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8448), 1,
      anon_sym_SEMI,
    STATE(2680), 1,
      sym_declaration_list,
    STATE(5109), 1,
      sym_where_clause,
    STATE(4153), 2,
      sym_line_comment,
      sym_block_comment,
  [129843] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8450), 1,
      anon_sym_SEMI,
    STATE(2684), 1,
      sym_declaration_list,
    STATE(5112), 1,
      sym_where_clause,
    STATE(4154), 2,
      sym_line_comment,
      sym_block_comment,
  [129872] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8452), 1,
      anon_sym_SEMI,
    STATE(952), 1,
      sym_declaration_list,
    STATE(5036), 1,
      sym_where_clause,
    STATE(4155), 2,
      sym_line_comment,
      sym_block_comment,
  [129901] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8454), 1,
      anon_sym_SEMI,
    STATE(1180), 1,
      sym_declaration_list,
    STATE(5108), 1,
      sym_where_clause,
    STATE(4156), 2,
      sym_line_comment,
      sym_block_comment,
  [129930] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8456), 1,
      anon_sym_SEMI,
    STATE(1121), 1,
      sym_declaration_list,
    STATE(5254), 1,
      sym_where_clause,
    STATE(4157), 2,
      sym_line_comment,
      sym_block_comment,
  [129959] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8458), 1,
      anon_sym_SEMI,
    STATE(1271), 1,
      sym_declaration_list,
    STATE(5116), 1,
      sym_where_clause,
    STATE(4158), 2,
      sym_line_comment,
      sym_block_comment,
  [129988] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7810), 1,
      anon_sym_for,
    STATE(4159), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [130011] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    ACTIONS(8460), 1,
      anon_sym_RBRACE,
    STATE(4876), 1,
      sym_macro_rule_v2,
    STATE(5997), 1,
      sym_token_tree_pattern,
    STATE(4160), 2,
      sym_line_comment,
      sym_block_comment,
  [130040] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8462), 1,
      anon_sym_SEMI,
    STATE(2705), 1,
      sym_declaration_list,
    STATE(5128), 1,
      sym_where_clause,
    STATE(4161), 2,
      sym_line_comment,
      sym_block_comment,
  [130069] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8464), 1,
      anon_sym_DASH_GT,
    STATE(4162), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6097), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [130090] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8182), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8466), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4163), 2,
      sym_line_comment,
      sym_block_comment,
  [130115] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8468), 1,
      anon_sym_SEMI,
    ACTIONS(8470), 1,
      anon_sym_EQ,
    STATE(4478), 1,
      sym_where_clause,
    STATE(4479), 1,
      sym_trait_bounds,
    STATE(4164), 2,
      sym_line_comment,
      sym_block_comment,
  [130144] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8180), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8236), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4165), 2,
      sym_line_comment,
      sym_block_comment,
  [130169] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7644), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4166), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6069), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
  [130190] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8472), 1,
      anon_sym_SEMI,
    STATE(954), 1,
      sym_declaration_list,
    STATE(5041), 1,
      sym_where_clause,
    STATE(4167), 2,
      sym_line_comment,
      sym_block_comment,
  [130219] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8474), 1,
      anon_sym_SEMI,
    STATE(956), 1,
      sym_declaration_list,
    STATE(5050), 1,
      sym_where_clause,
    STATE(4168), 2,
      sym_line_comment,
      sym_block_comment,
  [130248] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8298), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4169), 2,
      sym_line_comment,
      sym_block_comment,
  [130273] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8476), 1,
      anon_sym_SEMI,
    STATE(800), 1,
      sym_declaration_list,
    STATE(4918), 1,
      sym_where_clause,
    STATE(4170), 2,
      sym_line_comment,
      sym_block_comment,
  [130302] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7797), 1,
      anon_sym_for,
    STATE(4171), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [130325] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8478), 1,
      anon_sym_SEMI,
    STATE(2471), 1,
      sym_declaration_list,
    STATE(5027), 1,
      sym_where_clause,
    STATE(4172), 2,
      sym_line_comment,
      sym_block_comment,
  [130354] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8480), 1,
      anon_sym_SEMI,
    STATE(1184), 1,
      sym_declaration_list,
    STATE(5120), 1,
      sym_where_clause,
    STATE(4173), 2,
      sym_line_comment,
      sym_block_comment,
  [130383] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8180), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8466), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4174), 2,
      sym_line_comment,
      sym_block_comment,
  [130408] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8482), 1,
      anon_sym_SEMI,
    STATE(792), 1,
      sym_declaration_list,
    STATE(4869), 1,
      sym_where_clause,
    STATE(4175), 2,
      sym_line_comment,
      sym_block_comment,
  [130437] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7352), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8484), 1,
      anon_sym_COMMA,
    ACTIONS(8486), 1,
      anon_sym_GT,
    STATE(5383), 1,
      aux_sym_type_parameters_repeat1,
    ACTIONS(5949), 2,
      anon_sym_PLUS,
      anon_sym_as,
    STATE(4176), 2,
      sym_line_comment,
      sym_block_comment,
  [130464] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1347), 1,
      sym_field_declaration_list,
    STATE(4441), 1,
      sym_type_parameters,
    STATE(5500), 1,
      sym_where_clause,
    STATE(4177), 2,
      sym_line_comment,
      sym_block_comment,
  [130493] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8488), 1,
      anon_sym_SEMI,
    STATE(623), 1,
      sym_declaration_list,
    STATE(5425), 1,
      sym_where_clause,
    STATE(4178), 2,
      sym_line_comment,
      sym_block_comment,
  [130522] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8490), 1,
      anon_sym_COMMA,
    STATE(4131), 1,
      aux_sym_where_clause_repeat1,
    STATE(4179), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5598), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_where,
  [130545] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8492), 1,
      anon_sym_SEMI,
    STATE(627), 1,
      sym_declaration_list,
    STATE(5429), 1,
      sym_where_clause,
    STATE(4180), 2,
      sym_line_comment,
      sym_block_comment,
  [130574] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8494), 1,
      anon_sym_SEMI,
    STATE(629), 1,
      sym_declaration_list,
    STATE(5433), 1,
      sym_where_clause,
    STATE(4181), 2,
      sym_line_comment,
      sym_block_comment,
  [130603] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8496), 1,
      anon_sym_SEMI,
    STATE(794), 1,
      sym_declaration_list,
    STATE(4925), 1,
      sym_where_clause,
    STATE(4182), 2,
      sym_line_comment,
      sym_block_comment,
  [130632] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1394), 1,
      anon_sym_DOT_DOT,
    STATE(4183), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1396), 5,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
      anon_sym_if,
  [130653] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8498), 1,
      anon_sym_DASH_GT,
    STATE(4184), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6131), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [130674] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8182), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8236), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4185), 2,
      sym_line_comment,
      sym_block_comment,
  [130699] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8500), 1,
      anon_sym_SEMI,
    STATE(1995), 1,
      sym_declaration_list,
    STATE(5171), 1,
      sym_where_clause,
    STATE(4186), 2,
      sym_line_comment,
      sym_block_comment,
  [130728] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8502), 1,
      anon_sym_SEMI,
    STATE(1287), 1,
      sym_declaration_list,
    STATE(4965), 1,
      sym_where_clause,
    STATE(4187), 2,
      sym_line_comment,
      sym_block_comment,
  [130757] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7820), 1,
      anon_sym_for,
    STATE(4188), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [130780] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8504), 1,
      anon_sym_SEMI,
    STATE(1997), 1,
      sym_declaration_list,
    STATE(5173), 1,
      sym_where_clause,
    STATE(4189), 2,
      sym_line_comment,
      sym_block_comment,
  [130809] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8506), 1,
      anon_sym_SEMI,
    STATE(1087), 1,
      sym_declaration_list,
    STATE(4887), 1,
      sym_where_clause,
    STATE(4190), 2,
      sym_line_comment,
      sym_block_comment,
  [130838] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7822), 1,
      anon_sym_for,
    STATE(4191), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [130861] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8508), 1,
      anon_sym_RBRACK,
    ACTIONS(7644), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(4192), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6069), 3,
      anon_sym_SEMI,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
  [130884] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6069), 2,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
    ACTIONS(7644), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(8508), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4193), 2,
      sym_line_comment,
      sym_block_comment,
  [130907] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8511), 1,
      anon_sym_SEMI,
    STATE(1999), 1,
      sym_declaration_list,
    STATE(5178), 1,
      sym_where_clause,
    STATE(4194), 2,
      sym_line_comment,
      sym_block_comment,
  [130936] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8513), 1,
      anon_sym_SEMI,
    STATE(2001), 1,
      sym_declaration_list,
    STATE(5179), 1,
      sym_where_clause,
    STATE(4195), 2,
      sym_line_comment,
      sym_block_comment,
  [130965] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8515), 1,
      anon_sym_SEMI,
    STATE(2026), 1,
      sym_declaration_list,
    STATE(5207), 1,
      sym_where_clause,
    STATE(4196), 2,
      sym_line_comment,
      sym_block_comment,
  [130994] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8517), 1,
      anon_sym_SEMI,
    STATE(2028), 1,
      sym_declaration_list,
    STATE(5210), 1,
      sym_where_clause,
    STATE(4197), 2,
      sym_line_comment,
      sym_block_comment,
  [131023] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8519), 1,
      anon_sym_SEMI,
    STATE(892), 1,
      sym_declaration_list,
    STATE(5001), 1,
      sym_where_clause,
    STATE(4198), 2,
      sym_line_comment,
      sym_block_comment,
  [131052] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8521), 1,
      anon_sym_SEMI,
    ACTIONS(8523), 1,
      anon_sym_EQ,
    STATE(4702), 1,
      sym_where_clause,
    STATE(4708), 1,
      sym_trait_bounds,
    STATE(4199), 2,
      sym_line_comment,
      sym_block_comment,
  [131081] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7676), 1,
      anon_sym_LBRACE,
    ACTIONS(8444), 1,
      anon_sym_LPAREN,
    ACTIONS(8525), 1,
      anon_sym_COLON_COLON,
    STATE(4200), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [131106] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8306), 1,
      anon_sym_RBRACK,
    ACTIONS(7711), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(4201), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6015), 3,
      anon_sym_SEMI,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
  [131129] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8527), 1,
      anon_sym_SEMI,
    STATE(1360), 1,
      sym_declaration_list,
    STATE(5290), 1,
      sym_where_clause,
    STATE(4202), 2,
      sym_line_comment,
      sym_block_comment,
  [131158] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8529), 1,
      anon_sym_SEMI,
    STATE(1027), 1,
      sym_declaration_list,
    STATE(5303), 1,
      sym_where_clause,
    STATE(4203), 2,
      sym_line_comment,
      sym_block_comment,
  [131187] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8531), 1,
      anon_sym_SEMI,
    ACTIONS(8533), 1,
      anon_sym_EQ,
    STATE(4579), 1,
      sym_where_clause,
    STATE(4593), 1,
      sym_trait_bounds,
    STATE(4204), 2,
      sym_line_comment,
      sym_block_comment,
  [131216] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8535), 1,
      anon_sym_SEMI,
    STATE(1306), 1,
      sym_declaration_list,
    STATE(5191), 1,
      sym_where_clause,
    STATE(4205), 2,
      sym_line_comment,
      sym_block_comment,
  [131245] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7832), 1,
      anon_sym_for,
    STATE(4206), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [131268] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8537), 1,
      anon_sym_COMMA,
    ACTIONS(8539), 1,
      anon_sym_GT,
    STATE(5285), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5442), 1,
      sym_trait_bounds,
    STATE(4207), 2,
      sym_line_comment,
      sym_block_comment,
  [131297] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8541), 1,
      anon_sym_SEMI,
    STATE(2040), 1,
      sym_declaration_list,
    STATE(5222), 1,
      sym_where_clause,
    STATE(4208), 2,
      sym_line_comment,
      sym_block_comment,
  [131326] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7711), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4209), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6015), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
      anon_sym_DASH_GT,
  [131347] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    ACTIONS(8543), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4210), 2,
      sym_line_comment,
      sym_block_comment,
  [131372] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8545), 1,
      anon_sym_SEMI,
    STATE(2042), 1,
      sym_declaration_list,
    STATE(5223), 1,
      sym_where_clause,
    STATE(4211), 2,
      sym_line_comment,
      sym_block_comment,
  [131401] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7676), 1,
      anon_sym_LBRACE,
    ACTIONS(8444), 1,
      anon_sym_LPAREN,
    ACTIONS(8547), 1,
      anon_sym_COLON_COLON,
    STATE(4212), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 3,
      anon_sym_SEMI,
      anon_sym_RBRACK,
      anon_sym_PLUS,
  [131426] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8549), 1,
      anon_sym_SEMI,
    STATE(2046), 1,
      sym_declaration_list,
    STATE(5232), 1,
      sym_where_clause,
    STATE(4213), 2,
      sym_line_comment,
      sym_block_comment,
  [131455] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8551), 1,
      anon_sym_SEMI,
    STATE(2048), 1,
      sym_declaration_list,
    STATE(5235), 1,
      sym_where_clause,
    STATE(4214), 2,
      sym_line_comment,
      sym_block_comment,
  [131484] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(8553), 1,
      anon_sym_if,
    STATE(6297), 1,
      sym_label,
    STATE(477), 2,
      sym_if_expression,
      sym_block,
    STATE(4215), 2,
      sym_line_comment,
      sym_block_comment,
  [131511] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1309), 1,
      sym_enum_variant_list,
    STATE(4746), 1,
      sym_type_parameters,
    STATE(5444), 1,
      sym_where_clause,
    STATE(4216), 2,
      sym_line_comment,
      sym_block_comment,
  [131540] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8555), 1,
      anon_sym_SEMI,
    STATE(2490), 1,
      sym_declaration_list,
    STATE(5121), 1,
      sym_where_clause,
    STATE(4217), 2,
      sym_line_comment,
      sym_block_comment,
  [131569] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2493), 1,
      sym_enum_variant_list,
    STATE(4599), 1,
      sym_type_parameters,
    STATE(5496), 1,
      sym_where_clause,
    STATE(4218), 2,
      sym_line_comment,
      sym_block_comment,
  [131598] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7850), 1,
      anon_sym_for,
    STATE(4219), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [131621] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1148), 1,
      sym_field_declaration_list,
    STATE(4797), 1,
      sym_type_parameters,
    STATE(5826), 1,
      sym_where_clause,
    STATE(4220), 2,
      sym_line_comment,
      sym_block_comment,
  [131650] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1315), 1,
      sym_field_declaration_list,
    STATE(4764), 1,
      sym_type_parameters,
    STATE(5479), 1,
      sym_where_clause,
    STATE(4221), 2,
      sym_line_comment,
      sym_block_comment,
  [131679] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8557), 1,
      anon_sym_SEMI,
    STATE(2495), 1,
      sym_declaration_list,
    STATE(5139), 1,
      sym_where_clause,
    STATE(4222), 2,
      sym_line_comment,
      sym_block_comment,
  [131708] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7842), 1,
      anon_sym_for,
    STATE(4223), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [131731] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4224), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7789), 6,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
  [131750] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8559), 1,
      anon_sym_SEMI,
    STATE(849), 1,
      sym_declaration_list,
    STATE(4878), 1,
      sym_where_clause,
    STATE(4225), 2,
      sym_line_comment,
      sym_block_comment,
  [131779] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8561), 1,
      anon_sym_SEMI,
    STATE(1224), 1,
      sym_declaration_list,
    STATE(5196), 1,
      sym_where_clause,
    STATE(4226), 2,
      sym_line_comment,
      sym_block_comment,
  [131808] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1157), 1,
      sym_enum_variant_list,
    STATE(4855), 1,
      sym_type_parameters,
    STATE(5953), 1,
      sym_where_clause,
    STATE(4227), 2,
      sym_line_comment,
      sym_block_comment,
  [131837] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7826), 1,
      anon_sym_for,
    STATE(4228), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [131860] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7848), 1,
      anon_sym_for,
    STATE(4229), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [131883] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(2503), 1,
      sym_field_declaration_list,
    STATE(4632), 1,
      sym_type_parameters,
    STATE(5507), 1,
      sym_where_clause,
    STATE(4230), 2,
      sym_line_comment,
      sym_block_comment,
  [131912] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8563), 1,
      anon_sym_SEMI,
    STATE(2128), 1,
      sym_declaration_list,
    STATE(5287), 1,
      sym_where_clause,
    STATE(4231), 2,
      sym_line_comment,
      sym_block_comment,
  [131941] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7852), 1,
      anon_sym_for,
    STATE(4232), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [131964] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8565), 1,
      anon_sym_SEMI,
    STATE(2130), 1,
      sym_declaration_list,
    STATE(5289), 1,
      sym_where_clause,
    STATE(4233), 2,
      sym_line_comment,
      sym_block_comment,
  [131993] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8567), 1,
      anon_sym_SEMI,
    STATE(817), 1,
      sym_declaration_list,
    STATE(4861), 1,
      sym_where_clause,
    STATE(4234), 2,
      sym_line_comment,
      sym_block_comment,
  [132022] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7854), 1,
      anon_sym_for,
    STATE(4235), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [132045] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1193), 1,
      sym_enum_variant_list,
    STATE(4691), 1,
      sym_type_parameters,
    STATE(5695), 1,
      sym_where_clause,
    STATE(4236), 2,
      sym_line_comment,
      sym_block_comment,
  [132074] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8569), 1,
      anon_sym_SEMI,
    STATE(2145), 1,
      sym_declaration_list,
    STATE(5292), 1,
      sym_where_clause,
    STATE(4237), 2,
      sym_line_comment,
      sym_block_comment,
  [132103] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8571), 1,
      anon_sym_SEMI,
    STATE(2147), 1,
      sym_declaration_list,
    STATE(5293), 1,
      sym_where_clause,
    STATE(4238), 2,
      sym_line_comment,
      sym_block_comment,
  [132132] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8573), 1,
      anon_sym_SEMI,
    STATE(821), 1,
      sym_declaration_list,
    STATE(4994), 1,
      sym_where_clause,
    STATE(4239), 2,
      sym_line_comment,
      sym_block_comment,
  [132161] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7860), 1,
      anon_sym_for,
    STATE(4240), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [132184] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8575), 1,
      anon_sym_SEMI,
    STATE(2149), 1,
      sym_declaration_list,
    STATE(5294), 1,
      sym_where_clause,
    STATE(4241), 2,
      sym_line_comment,
      sym_block_comment,
  [132213] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8577), 1,
      anon_sym_SEMI,
    STATE(823), 1,
      sym_declaration_list,
    STATE(5004), 1,
      sym_where_clause,
    STATE(4242), 2,
      sym_line_comment,
      sym_block_comment,
  [132242] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7864), 1,
      anon_sym_for,
    STATE(4243), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [132265] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8579), 1,
      anon_sym_SEMI,
    STATE(2151), 1,
      sym_declaration_list,
    STATE(5295), 1,
      sym_where_clause,
    STATE(4244), 2,
      sym_line_comment,
      sym_block_comment,
  [132294] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7743), 1,
      anon_sym_LT,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2509), 1,
      sym_enum_variant_list,
    STATE(4664), 1,
      sym_type_parameters,
    STATE(5515), 1,
      sym_where_clause,
    STATE(4245), 2,
      sym_line_comment,
      sym_block_comment,
  [132323] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7868), 1,
      anon_sym_for,
    STATE(4246), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [132346] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8581), 1,
      anon_sym_SEMI,
    STATE(2173), 1,
      sym_declaration_list,
    STATE(5304), 1,
      sym_where_clause,
    STATE(4247), 2,
      sym_line_comment,
      sym_block_comment,
  [132375] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4248), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6191), 6,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [132394] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7870), 1,
      anon_sym_for,
    STATE(4249), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 4,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_PLUS,
      anon_sym_where,
  [132417] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8583), 1,
      anon_sym_SEMI,
    STATE(2175), 1,
      sym_declaration_list,
    STATE(5305), 1,
      sym_where_clause,
    STATE(4250), 2,
      sym_line_comment,
      sym_block_comment,
  [132446] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8585), 1,
      anon_sym_SEMI,
    STATE(2177), 1,
      sym_declaration_list,
    STATE(5306), 1,
      sym_where_clause,
    STATE(4251), 2,
      sym_line_comment,
      sym_block_comment,
  [132475] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5934), 1,
      anon_sym_COLON,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7259), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(5844), 1,
      sym_parameters,
    STATE(4252), 2,
      sym_line_comment,
      sym_block_comment,
  [132504] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8587), 1,
      anon_sym_SEMI,
    ACTIONS(8589), 1,
      anon_sym_EQ,
    STATE(4750), 1,
      sym_where_clause,
    STATE(4751), 1,
      sym_trait_bounds,
    STATE(4253), 2,
      sym_line_comment,
      sym_block_comment,
  [132533] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4254), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6011), 6,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [132552] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4255), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8591), 6,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_SQUOTE,
      anon_sym_where,
  [132571] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8593), 1,
      anon_sym_SEMI,
    STATE(2179), 1,
      sym_declaration_list,
    STATE(5312), 1,
      sym_where_clause,
    STATE(4256), 2,
      sym_line_comment,
      sym_block_comment,
  [132600] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4257), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6077), 6,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [132619] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8595), 1,
      anon_sym_SEMI,
    STATE(2204), 1,
      sym_declaration_list,
    STATE(5330), 1,
      sym_where_clause,
    STATE(4258), 2,
      sym_line_comment,
      sym_block_comment,
  [132648] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4259), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6167), 6,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [132667] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4260), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8597), 6,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_SQUOTE,
      anon_sym_where,
  [132686] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8599), 1,
      anon_sym_SEMI,
    STATE(768), 1,
      sym_declaration_list,
    STATE(5378), 1,
      sym_where_clause,
    STATE(4261), 2,
      sym_line_comment,
      sym_block_comment,
  [132715] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8601), 1,
      anon_sym_SEMI,
    STATE(770), 1,
      sym_declaration_list,
    STATE(5393), 1,
      sym_where_clause,
    STATE(4262), 2,
      sym_line_comment,
      sym_block_comment,
  [132744] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8603), 1,
      anon_sym_SEMI,
    STATE(2206), 1,
      sym_declaration_list,
    STATE(5331), 1,
      sym_where_clause,
    STATE(4263), 2,
      sym_line_comment,
      sym_block_comment,
  [132773] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4264), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6163), 6,
      anon_sym_LPAREN,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [132792] = 9,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8605), 1,
      anon_sym_SEMI,
    STATE(625), 1,
      sym_declaration_list,
    STATE(5427), 1,
      sym_where_clause,
    STATE(4265), 2,
      sym_line_comment,
      sym_block_comment,
  [132821] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8607), 1,
      anon_sym_EQ,
    STATE(847), 1,
      sym_declaration_list,
    STATE(5400), 1,
      sym_where_clause,
    STATE(4266), 2,
      sym_line_comment,
      sym_block_comment,
  [132847] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4267), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8609), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [132865] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4268), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8611), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [132883] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4269), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8613), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [132901] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7516), 1,
      anon_sym_EQ,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(5236), 1,
      sym_trait_bounds,
    ACTIONS(7512), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(4270), 2,
      sym_line_comment,
      sym_block_comment,
  [132925] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(5825), 1,
      sym_trait_bounds,
    ACTIONS(8615), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(4271), 2,
      sym_line_comment,
      sym_block_comment,
  [132949] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4272), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8617), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [132967] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4273), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5863), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [132985] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8619), 1,
      anon_sym_SEMI,
    ACTIONS(8621), 1,
      anon_sym_COLON,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(8625), 1,
      anon_sym_EQ,
    ACTIONS(8627), 1,
      anon_sym_else,
    STATE(4274), 2,
      sym_line_comment,
      sym_block_comment,
  [133011] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    STATE(5828), 1,
      sym_field_pattern,
    STATE(4275), 2,
      sym_line_comment,
      sym_block_comment,
  [133037] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8629), 1,
      anon_sym_EQ,
    STATE(1108), 1,
      sym_declaration_list,
    STATE(5089), 1,
      sym_where_clause,
    STATE(4276), 2,
      sym_line_comment,
      sym_block_comment,
  [133063] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8252), 1,
      anon_sym_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4277), 2,
      sym_line_comment,
      sym_block_comment,
  [133087] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8631), 1,
      sym_identifier,
    ACTIONS(8633), 1,
      anon_sym_COMMA,
    ACTIONS(8635), 1,
      anon_sym_GT,
    STATE(4984), 1,
      sym_lifetime,
    STATE(4278), 2,
      sym_line_comment,
      sym_block_comment,
  [133113] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(8637), 1,
      anon_sym_RPAREN,
    ACTIONS(8639), 1,
      anon_sym_COMMA,
    ACTIONS(8641), 1,
      anon_sym_COLON,
    STATE(4979), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4279), 2,
      sym_line_comment,
      sym_block_comment,
  [133139] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7699), 1,
      anon_sym_PIPE,
    ACTIONS(8643), 1,
      anon_sym_SEMI,
    ACTIONS(8645), 1,
      anon_sym_COLON,
    ACTIONS(8647), 1,
      anon_sym_EQ,
    ACTIONS(8649), 1,
      anon_sym_else,
    STATE(4280), 2,
      sym_line_comment,
      sym_block_comment,
  [133165] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8651), 1,
      anon_sym_EQ,
    STATE(2482), 1,
      sym_declaration_list,
    STATE(5069), 1,
      sym_where_clause,
    STATE(4281), 2,
      sym_line_comment,
      sym_block_comment,
  [133191] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8653), 1,
      anon_sym_SEMI,
    STATE(923), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4282), 2,
      sym_line_comment,
      sym_block_comment,
  [133217] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(8655), 1,
      anon_sym_SEMI,
    ACTIONS(8657), 1,
      anon_sym_COLON,
    ACTIONS(8659), 1,
      anon_sym_EQ,
    ACTIONS(8661), 1,
      anon_sym_else,
    STATE(4283), 2,
      sym_line_comment,
      sym_block_comment,
  [133243] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3184), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4284), 2,
      sym_line_comment,
      sym_block_comment,
  [133269] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8313), 1,
      anon_sym_COMMA,
    ACTIONS(8315), 1,
      anon_sym_GT,
    STATE(5333), 1,
      sym_trait_bounds,
    STATE(5334), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4285), 2,
      sym_line_comment,
      sym_block_comment,
  [133295] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2975), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4286), 2,
      sym_line_comment,
      sym_block_comment,
  [133321] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8663), 1,
      anon_sym_RPAREN,
    ACTIONS(8665), 1,
      anon_sym_COMMA,
    STATE(5414), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(7203), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4287), 2,
      sym_line_comment,
      sym_block_comment,
  [133345] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3240), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4288), 2,
      sym_line_comment,
      sym_block_comment,
  [133371] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8667), 1,
      anon_sym_EQ,
    STATE(2527), 1,
      sym_declaration_list,
    STATE(4864), 1,
      sym_where_clause,
    STATE(4289), 2,
      sym_line_comment,
      sym_block_comment,
  [133397] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8669), 1,
      anon_sym_COMMA,
    STATE(4300), 1,
      aux_sym_where_clause_repeat1,
    STATE(4290), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8420), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
  [133419] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8671), 1,
      anon_sym_SEMI,
    STATE(2536), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4291), 2,
      sym_line_comment,
      sym_block_comment,
  [133445] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8673), 1,
      anon_sym_EQ,
    STATE(1207), 1,
      sym_declaration_list,
    STATE(5354), 1,
      sym_where_clause,
    STATE(4292), 2,
      sym_line_comment,
      sym_block_comment,
  [133471] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8675), 1,
      anon_sym_EQ,
    STATE(2554), 1,
      sym_declaration_list,
    STATE(4902), 1,
      sym_where_clause,
    STATE(4293), 2,
      sym_line_comment,
      sym_block_comment,
  [133497] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7699), 1,
      anon_sym_PIPE,
    ACTIONS(8677), 1,
      anon_sym_SEMI,
    ACTIONS(8679), 1,
      anon_sym_COLON,
    ACTIONS(8681), 1,
      anon_sym_EQ,
    ACTIONS(8683), 1,
      anon_sym_else,
    STATE(4294), 2,
      sym_line_comment,
      sym_block_comment,
  [133523] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8685), 1,
      anon_sym_EQ,
    STATE(2583), 1,
      sym_declaration_list,
    STATE(4960), 1,
      sym_where_clause,
    STATE(4295), 2,
      sym_line_comment,
      sym_block_comment,
  [133549] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4296), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8687), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [133567] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4297), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8689), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [133585] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3371), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4298), 2,
      sym_line_comment,
      sym_block_comment,
  [133611] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8691), 1,
      anon_sym_EQ,
    STATE(2627), 1,
      sym_declaration_list,
    STATE(5022), 1,
      sym_where_clause,
    STATE(4299), 2,
      sym_line_comment,
      sym_block_comment,
  [133637] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8693), 1,
      anon_sym_COMMA,
    STATE(4314), 1,
      aux_sym_where_clause_repeat1,
    STATE(4300), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5598), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
  [133659] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8695), 1,
      anon_sym_SEMI,
    STATE(2630), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4301), 2,
      sym_line_comment,
      sym_block_comment,
  [133685] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2957), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4302), 2,
      sym_line_comment,
      sym_block_comment,
  [133711] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8697), 1,
      anon_sym_SEMI,
    STATE(617), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4303), 2,
      sym_line_comment,
      sym_block_comment,
  [133737] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8699), 1,
      anon_sym_EQ,
    STATE(2662), 1,
      sym_declaration_list,
    STATE(5072), 1,
      sym_where_clause,
    STATE(4304), 2,
      sym_line_comment,
      sym_block_comment,
  [133763] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8701), 1,
      anon_sym_SEMI,
    STATE(966), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4305), 2,
      sym_line_comment,
      sym_block_comment,
  [133789] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8703), 1,
      anon_sym_EQ,
    STATE(2671), 1,
      sym_declaration_list,
    STATE(5087), 1,
      sym_where_clause,
    STATE(4306), 2,
      sym_line_comment,
      sym_block_comment,
  [133815] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8705), 1,
      anon_sym_SEMI,
    STATE(2678), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4307), 2,
      sym_line_comment,
      sym_block_comment,
  [133841] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8707), 1,
      anon_sym_EQ,
    STATE(2696), 1,
      sym_declaration_list,
    STATE(5118), 1,
      sym_where_clause,
    STATE(4308), 2,
      sym_line_comment,
      sym_block_comment,
  [133867] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8709), 1,
      anon_sym_EQ,
    STATE(978), 1,
      sym_declaration_list,
    STATE(5132), 1,
      sym_where_clause,
    STATE(4309), 2,
      sym_line_comment,
      sym_block_comment,
  [133893] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8711), 1,
      anon_sym_SEMI,
    STATE(2727), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4310), 2,
      sym_line_comment,
      sym_block_comment,
  [133919] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8713), 1,
      anon_sym_SEMI,
    STATE(980), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4311), 2,
      sym_line_comment,
      sym_block_comment,
  [133945] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8715), 1,
      anon_sym_SEMI,
    STATE(1218), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4312), 2,
      sym_line_comment,
      sym_block_comment,
  [133971] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8717), 1,
      anon_sym_SEMI,
    STATE(984), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4313), 2,
      sym_line_comment,
      sym_block_comment,
  [133997] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8719), 1,
      anon_sym_COMMA,
    ACTIONS(8409), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(4314), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_where_clause_repeat1,
  [134017] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8722), 1,
      anon_sym_SEMI,
    STATE(2745), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4315), 2,
      sym_line_comment,
      sym_block_comment,
  [134043] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8724), 1,
      anon_sym_EQ,
    STATE(1134), 1,
      sym_declaration_list,
    STATE(5023), 1,
      sym_where_clause,
    STATE(4316), 2,
      sym_line_comment,
      sym_block_comment,
  [134069] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3235), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4317), 2,
      sym_line_comment,
      sym_block_comment,
  [134095] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4318), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5883), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [134113] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8726), 1,
      anon_sym_EQ,
    STATE(2024), 1,
      sym_declaration_list,
    STATE(5192), 1,
      sym_where_clause,
    STATE(4319), 2,
      sym_line_comment,
      sym_block_comment,
  [134139] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8728), 1,
      anon_sym_EQ,
    STATE(2034), 1,
      sym_declaration_list,
    STATE(5218), 1,
      sym_where_clause,
    STATE(4320), 2,
      sym_line_comment,
      sym_block_comment,
  [134165] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8730), 1,
      anon_sym_SEMI,
    STATE(2036), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4321), 2,
      sym_line_comment,
      sym_block_comment,
  [134191] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8732), 1,
      anon_sym_EQ,
    STATE(2068), 1,
      sym_declaration_list,
    STATE(5243), 1,
      sym_where_clause,
    STATE(4322), 2,
      sym_line_comment,
      sym_block_comment,
  [134217] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8734), 1,
      anon_sym_SEMI,
    STATE(998), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4323), 2,
      sym_line_comment,
      sym_block_comment,
  [134243] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8736), 1,
      anon_sym_SEMI,
    STATE(2076), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4324), 2,
      sym_line_comment,
      sym_block_comment,
  [134269] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8738), 1,
      anon_sym_SEMI,
    STATE(2093), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4325), 2,
      sym_line_comment,
      sym_block_comment,
  [134295] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8740), 1,
      anon_sym_EQ,
    STATE(2105), 1,
      sym_declaration_list,
    STATE(5279), 1,
      sym_where_clause,
    STATE(4326), 2,
      sym_line_comment,
      sym_block_comment,
  [134321] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8742), 1,
      anon_sym_SEMI,
    STATE(2111), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4327), 2,
      sym_line_comment,
      sym_block_comment,
  [134347] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8744), 1,
      anon_sym_SEMI,
    STATE(2118), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4328), 2,
      sym_line_comment,
      sym_block_comment,
  [134373] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4329), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8746), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [134391] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8750), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8752), 1,
      anon_sym_as,
    STATE(4330), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8748), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [134413] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8754), 1,
      anon_sym_SEMI,
    STATE(2167), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4331), 2,
      sym_line_comment,
      sym_block_comment,
  [134439] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8756), 1,
      anon_sym_SEMI,
    STATE(853), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4332), 2,
      sym_line_comment,
      sym_block_comment,
  [134465] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8758), 1,
      anon_sym_EQ,
    STATE(2202), 1,
      sym_declaration_list,
    STATE(5318), 1,
      sym_where_clause,
    STATE(4333), 2,
      sym_line_comment,
      sym_block_comment,
  [134491] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8760), 1,
      anon_sym_SEMI,
    STATE(2208), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4334), 2,
      sym_line_comment,
      sym_block_comment,
  [134517] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8762), 1,
      anon_sym_EQ,
    STATE(2220), 1,
      sym_declaration_list,
    STATE(5346), 1,
      sym_where_clause,
    STATE(4335), 2,
      sym_line_comment,
      sym_block_comment,
  [134543] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8764), 1,
      anon_sym_SEMI,
    STATE(2226), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4336), 2,
      sym_line_comment,
      sym_block_comment,
  [134569] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8766), 1,
      anon_sym_SEMI,
    STATE(2230), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4337), 2,
      sym_line_comment,
      sym_block_comment,
  [134595] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8768), 1,
      anon_sym_EQ,
    STATE(2242), 1,
      sym_declaration_list,
    STATE(5362), 1,
      sym_where_clause,
    STATE(4338), 2,
      sym_line_comment,
      sym_block_comment,
  [134621] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8770), 1,
      anon_sym_SEMI,
    STATE(2244), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4339), 2,
      sym_line_comment,
      sym_block_comment,
  [134647] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8772), 1,
      anon_sym_EQ,
    STATE(1155), 1,
      sym_declaration_list,
    STATE(5068), 1,
      sym_where_clause,
    STATE(4340), 2,
      sym_line_comment,
      sym_block_comment,
  [134673] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8774), 1,
      anon_sym_SEMI,
    STATE(2276), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4341), 2,
      sym_line_comment,
      sym_block_comment,
  [134699] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4342), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1392), 5,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [134717] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8776), 1,
      anon_sym_RBRACK,
    ACTIONS(5949), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(7203), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(4343), 2,
      sym_line_comment,
      sym_block_comment,
  [134739] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8779), 1,
      anon_sym_SEMI,
    STATE(2318), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4344), 2,
      sym_line_comment,
      sym_block_comment,
  [134765] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8781), 1,
      anon_sym_EQ,
    STATE(2330), 1,
      sym_declaration_list,
    STATE(5402), 1,
      sym_where_clause,
    STATE(4345), 2,
      sym_line_comment,
      sym_block_comment,
  [134791] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8783), 1,
      anon_sym_SEMI,
    STATE(2332), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4346), 2,
      sym_line_comment,
      sym_block_comment,
  [134817] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8785), 1,
      anon_sym_SEMI,
    STATE(2336), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4347), 2,
      sym_line_comment,
      sym_block_comment,
  [134843] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8787), 1,
      anon_sym_SEMI,
    STATE(1178), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4348), 2,
      sym_line_comment,
      sym_block_comment,
  [134869] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8789), 1,
      anon_sym_SEMI,
    STATE(2349), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4349), 2,
      sym_line_comment,
      sym_block_comment,
  [134895] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8791), 1,
      anon_sym_SEMI,
    STATE(2381), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4350), 2,
      sym_line_comment,
      sym_block_comment,
  [134921] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8793), 1,
      anon_sym_SEMI,
    STATE(2394), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4351), 2,
      sym_line_comment,
      sym_block_comment,
  [134947] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8795), 1,
      anon_sym_SEMI,
    STATE(2400), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4352), 2,
      sym_line_comment,
      sym_block_comment,
  [134973] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7914), 1,
      anon_sym_LBRACE,
    ACTIONS(8797), 1,
      anon_sym_SEMI,
    STATE(2413), 1,
      sym_block,
    STATE(6433), 1,
      sym_label,
    STATE(4353), 2,
      sym_line_comment,
      sym_block_comment,
  [134999] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2979), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4354), 2,
      sym_line_comment,
      sym_block_comment,
  [135025] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4355), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8799), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [135043] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4356), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8801), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [135061] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8803), 1,
      sym_identifier,
    ACTIONS(8805), 1,
      anon_sym_COMMA,
    ACTIONS(8807), 1,
      anon_sym_GT,
    STATE(4873), 1,
      sym_lifetime,
    STATE(4357), 2,
      sym_line_comment,
      sym_block_comment,
  [135087] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8809), 1,
      anon_sym_SEMI,
    STATE(644), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4358), 2,
      sym_line_comment,
      sym_block_comment,
  [135113] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4359), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8811), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [135131] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8813), 1,
      sym_identifier,
    ACTIONS(8815), 1,
      anon_sym_COMMA,
    ACTIONS(8817), 1,
      anon_sym_GT,
    STATE(4921), 1,
      sym_lifetime,
    STATE(4360), 2,
      sym_line_comment,
      sym_block_comment,
  [135157] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    STATE(5516), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(4361), 2,
      sym_line_comment,
      sym_block_comment,
  [135183] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8351), 1,
      anon_sym_COMMA,
    ACTIONS(8353), 1,
      anon_sym_GT,
    STATE(4931), 1,
      sym_trait_bounds,
    STATE(4932), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4362), 2,
      sym_line_comment,
      sym_block_comment,
  [135209] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8819), 1,
      anon_sym_RPAREN,
    ACTIONS(8821), 1,
      anon_sym_COMMA,
    STATE(4936), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(7203), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4363), 2,
      sym_line_comment,
      sym_block_comment,
  [135233] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8823), 1,
      anon_sym_RBRACK,
    ACTIONS(6467), 2,
      anon_sym_SEMI,
      anon_sym_PLUS,
    ACTIONS(7715), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(4364), 2,
      sym_line_comment,
      sym_block_comment,
  [135255] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(8828), 1,
      anon_sym_STAR,
    STATE(5165), 1,
      sym_use_list,
    ACTIONS(8826), 2,
      sym_identifier,
      sym_super,
    STATE(4365), 2,
      sym_line_comment,
      sym_block_comment,
  [135279] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8832), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8834), 1,
      anon_sym_as,
    STATE(4366), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8830), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [135301] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7662), 1,
      anon_sym_RPAREN,
    ACTIONS(8836), 1,
      anon_sym_COMMA,
    ACTIONS(8838), 1,
      anon_sym_COLON,
    ACTIONS(8840), 1,
      anon_sym_PIPE,
    STATE(5002), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(4367), 2,
      sym_line_comment,
      sym_block_comment,
  [135327] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8100), 1,
      anon_sym_LPAREN,
    ACTIONS(8102), 1,
      anon_sym_LBRACE,
    ACTIONS(8106), 1,
      anon_sym_LBRACK,
    STATE(5545), 1,
      sym_macro_rule_v2,
    STATE(5997), 1,
      sym_token_tree_pattern,
    STATE(4368), 2,
      sym_line_comment,
      sym_block_comment,
  [135353] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7715), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4369), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6467), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [135373] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8842), 1,
      anon_sym_RPAREN,
    ACTIONS(8845), 1,
      anon_sym_COMMA,
    STATE(5060), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(7203), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4370), 2,
      sym_line_comment,
      sym_block_comment,
  [135397] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5949), 1,
      anon_sym_PLUS,
    ACTIONS(7203), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(8776), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4371), 2,
      sym_line_comment,
      sym_block_comment,
  [135419] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(8848), 1,
      anon_sym_SEMI,
    ACTIONS(8850), 1,
      anon_sym_COLON,
    ACTIONS(8852), 1,
      anon_sym_EQ,
    ACTIONS(8854), 1,
      anon_sym_else,
    STATE(4372), 2,
      sym_line_comment,
      sym_block_comment,
  [135445] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8379), 1,
      anon_sym_COMMA,
    ACTIONS(8381), 1,
      anon_sym_GT,
    STATE(4996), 1,
      sym_trait_bounds,
    STATE(4997), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4373), 2,
      sym_line_comment,
      sym_block_comment,
  [135471] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4374), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8856), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [135489] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8834), 1,
      anon_sym_as,
    ACTIONS(8858), 1,
      anon_sym_COLON_COLON,
    STATE(4375), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8830), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [135511] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4376), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5867), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [135529] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8860), 1,
      anon_sym_EQ,
    STATE(660), 1,
      sym_declaration_list,
    STATE(4991), 1,
      sym_where_clause,
    STATE(4377), 2,
      sym_line_comment,
      sym_block_comment,
  [135555] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4378), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8862), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [135573] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    STATE(5282), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(4379), 2,
      sym_line_comment,
      sym_block_comment,
  [135599] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8864), 1,
      anon_sym_EQ,
    STATE(865), 1,
      sym_declaration_list,
    STATE(4972), 1,
      sym_where_clause,
    STATE(4380), 2,
      sym_line_comment,
      sym_block_comment,
  [135625] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4381), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8866), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [135643] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8868), 1,
      anon_sym_SEMI,
    STATE(871), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4382), 2,
      sym_line_comment,
      sym_block_comment,
  [135669] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8834), 1,
      anon_sym_as,
    ACTIONS(8870), 1,
      anon_sym_COLON_COLON,
    STATE(4383), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8830), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [135691] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    ACTIONS(8537), 1,
      anon_sym_COMMA,
    ACTIONS(8539), 1,
      anon_sym_GT,
    STATE(5285), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5442), 1,
      sym_trait_bounds,
    STATE(4384), 2,
      sym_line_comment,
      sym_block_comment,
  [135717] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8872), 1,
      anon_sym_SEMI,
    STATE(1029), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4385), 2,
      sym_line_comment,
      sym_block_comment,
  [135743] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8874), 1,
      anon_sym_EQ,
    STATE(670), 1,
      sym_declaration_list,
    STATE(5341), 1,
      sym_where_clause,
    STATE(4386), 2,
      sym_line_comment,
      sym_block_comment,
  [135769] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4387), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8876), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [135787] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8878), 1,
      anon_sym_SEMI,
    STATE(672), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4388), 2,
      sym_line_comment,
      sym_block_comment,
  [135813] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8880), 1,
      anon_sym_EQ,
    STATE(1201), 1,
      sym_declaration_list,
    STATE(5158), 1,
      sym_where_clause,
    STATE(4389), 2,
      sym_line_comment,
      sym_block_comment,
  [135839] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6467), 1,
      anon_sym_PLUS,
    ACTIONS(7715), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(8823), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4390), 2,
      sym_line_comment,
      sym_block_comment,
  [135861] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8882), 1,
      anon_sym_EQ,
    STATE(1259), 1,
      sym_declaration_list,
    STATE(4971), 1,
      sym_where_clause,
    STATE(4391), 2,
      sym_line_comment,
      sym_block_comment,
  [135887] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8884), 1,
      anon_sym_SEMI,
    STATE(875), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4392), 2,
      sym_line_comment,
      sym_block_comment,
  [135913] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    STATE(5352), 1,
      sym_field_pattern,
    STATE(4393), 2,
      sym_line_comment,
      sym_block_comment,
  [135939] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    STATE(5361), 1,
      sym_field_pattern,
    STATE(4394), 2,
      sym_line_comment,
      sym_block_comment,
  [135965] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(8888), 1,
      anon_sym_STAR,
    STATE(5175), 1,
      sym_use_list,
    ACTIONS(8886), 2,
      sym_identifier,
      sym_super,
    STATE(4395), 2,
      sym_line_comment,
      sym_block_comment,
  [135989] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4396), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5871), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [136007] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4397), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5875), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [136025] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    STATE(4898), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(4398), 2,
      sym_line_comment,
      sym_block_comment,
  [136051] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4399), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5887), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [136069] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8890), 1,
      anon_sym_SEMI,
    STATE(1043), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4400), 2,
      sym_line_comment,
      sym_block_comment,
  [136095] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4401), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5647), 5,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
      anon_sym_as,
  [136113] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8892), 1,
      anon_sym_EQ,
    STATE(706), 1,
      sym_declaration_list,
    STATE(4952), 1,
      sym_where_clause,
    STATE(4402), 2,
      sym_line_comment,
      sym_block_comment,
  [136139] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    STATE(5185), 1,
      sym_field_pattern,
    STATE(4403), 2,
      sym_line_comment,
      sym_block_comment,
  [136165] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8894), 1,
      anon_sym_SEMI,
    STATE(1049), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4404), 2,
      sym_line_comment,
      sym_block_comment,
  [136191] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8178), 1,
      anon_sym_COLON,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4405), 2,
      sym_line_comment,
      sym_block_comment,
  [136215] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8896), 1,
      anon_sym_SEMI,
    STATE(714), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4406), 2,
      sym_line_comment,
      sym_block_comment,
  [136241] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4407), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8898), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [136259] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(5938), 1,
      sym_trait_bounds,
    ACTIONS(8900), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(4408), 2,
      sym_line_comment,
      sym_block_comment,
  [136283] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8902), 1,
      anon_sym_EQ,
    STATE(887), 1,
      sym_declaration_list,
    STATE(5107), 1,
      sym_where_clause,
    STATE(4409), 2,
      sym_line_comment,
      sym_block_comment,
  [136309] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8904), 1,
      anon_sym_SEMI,
    STATE(732), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4410), 2,
      sym_line_comment,
      sym_block_comment,
  [136335] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4411), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1424), 5,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [136353] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8906), 1,
      anon_sym_SEMI,
    STATE(889), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4412), 2,
      sym_line_comment,
      sym_block_comment,
  [136379] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6109), 1,
      anon_sym_LBRACE,
    ACTIONS(8244), 1,
      anon_sym_STAR,
    STATE(5181), 1,
      sym_use_list,
    ACTIONS(8242), 2,
      sym_identifier,
      sym_super,
    STATE(4413), 2,
      sym_line_comment,
      sym_block_comment,
  [136403] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8300), 1,
      anon_sym_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4414), 2,
      sym_line_comment,
      sym_block_comment,
  [136427] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7699), 1,
      anon_sym_PIPE,
    ACTIONS(8908), 1,
      anon_sym_SEMI,
    ACTIONS(8910), 1,
      anon_sym_COLON,
    ACTIONS(8912), 1,
      anon_sym_EQ,
    ACTIONS(8914), 1,
      anon_sym_else,
    STATE(4415), 2,
      sym_line_comment,
      sym_block_comment,
  [136453] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7699), 1,
      anon_sym_PIPE,
    ACTIONS(8916), 1,
      anon_sym_SEMI,
    ACTIONS(8918), 1,
      anon_sym_COLON,
    ACTIONS(8920), 1,
      anon_sym_EQ,
    ACTIONS(8922), 1,
      anon_sym_else,
    STATE(4416), 2,
      sym_line_comment,
      sym_block_comment,
  [136479] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8924), 1,
      anon_sym_EQ,
    STATE(744), 1,
      sym_declaration_list,
    STATE(5215), 1,
      sym_where_clause,
    STATE(4417), 2,
      sym_line_comment,
      sym_block_comment,
  [136505] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4418), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5859), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [136523] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4419), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5879), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [136541] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4420), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5855), 5,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_where,
  [136559] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8926), 1,
      anon_sym_SEMI,
    STATE(750), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4421), 2,
      sym_line_comment,
      sym_block_comment,
  [136585] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8928), 1,
      anon_sym_EQ,
    STATE(1298), 1,
      sym_declaration_list,
    STATE(5086), 1,
      sym_where_clause,
    STATE(4422), 2,
      sym_line_comment,
      sym_block_comment,
  [136611] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8930), 1,
      anon_sym_SEMI,
    STATE(1274), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4423), 2,
      sym_line_comment,
      sym_block_comment,
  [136637] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(7390), 1,
      sym_identifier,
    STATE(4946), 1,
      sym_field_declaration,
    STATE(6098), 1,
      sym_visibility_modifier,
    STATE(4424), 2,
      sym_line_comment,
      sym_block_comment,
  [136663] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8932), 1,
      anon_sym_SEMI,
    STATE(1062), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4425), 2,
      sym_line_comment,
      sym_block_comment,
  [136689] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7277), 1,
      sym_identifier,
    ACTIONS(7285), 1,
      anon_sym_ref,
    ACTIONS(7287), 1,
      sym_mutable_specifier,
    ACTIONS(7289), 1,
      sym_integer_literal,
    STATE(5297), 1,
      sym_field_pattern,
    STATE(4426), 2,
      sym_line_comment,
      sym_block_comment,
  [136715] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2924), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4427), 2,
      sym_line_comment,
      sym_block_comment,
  [136741] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4428), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8934), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [136759] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8936), 1,
      anon_sym_EQ,
    STATE(637), 1,
      sym_declaration_list,
    STATE(5046), 1,
      sym_where_clause,
    STATE(4429), 2,
      sym_line_comment,
      sym_block_comment,
  [136785] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8238), 1,
      anon_sym_COLON,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4430), 2,
      sym_line_comment,
      sym_block_comment,
  [136809] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8938), 1,
      anon_sym_SEMI,
    STATE(811), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4431), 2,
      sym_line_comment,
      sym_block_comment,
  [136835] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4432), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8940), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [136853] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8942), 1,
      anon_sym_RPAREN,
    ACTIONS(8944), 1,
      anon_sym_COMMA,
    STATE(5060), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(7203), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4433), 2,
      sym_line_comment,
      sym_block_comment,
  [136877] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(8946), 1,
      anon_sym_SEMI,
    ACTIONS(8948), 1,
      anon_sym_COLON,
    ACTIONS(8950), 1,
      anon_sym_EQ,
    ACTIONS(8952), 1,
      anon_sym_else,
    STATE(4434), 2,
      sym_line_comment,
      sym_block_comment,
  [136903] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5993), 1,
      anon_sym_COLON,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5845), 1,
      sym_parameters,
    STATE(4435), 2,
      sym_line_comment,
      sym_block_comment,
  [136929] = 8,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(7888), 1,
      anon_sym_LBRACE,
    ACTIONS(8954), 1,
      anon_sym_SEMI,
    STATE(758), 1,
      sym_block,
    STATE(6427), 1,
      sym_label,
    STATE(4436), 2,
      sym_line_comment,
      sym_block_comment,
  [136955] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7203), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    STATE(4437), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5949), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PLUS,
  [136975] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4438), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8956), 5,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
      anon_sym_where,
  [136993] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3230), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4439), 2,
      sym_line_comment,
      sym_block_comment,
  [137016] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8958), 1,
      anon_sym_SEMI,
    STATE(6375), 1,
      sym_where_clause,
    STATE(4440), 2,
      sym_line_comment,
      sym_block_comment,
  [137039] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1114), 1,
      sym_field_declaration_list,
    STATE(5711), 1,
      sym_where_clause,
    STATE(4441), 2,
      sym_line_comment,
      sym_block_comment,
  [137062] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(8962), 1,
      anon_sym_EQ,
    ACTIONS(8960), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(4442), 2,
      sym_line_comment,
      sym_block_comment,
  [137083] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(8964), 1,
      anon_sym_RPAREN,
    ACTIONS(8966), 1,
      anon_sym_COMMA,
    STATE(4966), 1,
      aux_sym_tuple_type_repeat1,
    STATE(4443), 2,
      sym_line_comment,
      sym_block_comment,
  [137106] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8968), 1,
      anon_sym_SEMI,
    STATE(6414), 1,
      sym_where_clause,
    STATE(4444), 2,
      sym_line_comment,
      sym_block_comment,
  [137129] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6089), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4445), 2,
      sym_line_comment,
      sym_block_comment,
  [137152] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7426), 1,
      anon_sym_LPAREN,
    ACTIONS(7428), 1,
      anon_sym_LBRACE,
    ACTIONS(7432), 1,
      anon_sym_LBRACK,
    STATE(467), 1,
      sym_delim_token_tree,
    STATE(4446), 2,
      sym_line_comment,
      sym_block_comment,
  [137175] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(5938), 1,
      sym_trait_bounds,
    ACTIONS(8900), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(4447), 2,
      sym_line_comment,
      sym_block_comment,
  [137196] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(8972), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4448), 2,
      sym_line_comment,
      sym_block_comment,
  [137219] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(8974), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4449), 2,
      sym_line_comment,
      sym_block_comment,
  [137242] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8976), 1,
      anon_sym_SEMI,
    ACTIONS(8978), 1,
      anon_sym_EQ,
    STATE(5994), 1,
      sym_where_clause,
    STATE(4450), 2,
      sym_line_comment,
      sym_block_comment,
  [137265] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(8980), 1,
      anon_sym_SEMI,
    STATE(2425), 1,
      sym_declaration_list,
    STATE(4451), 2,
      sym_line_comment,
      sym_block_comment,
  [137288] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(8982), 1,
      anon_sym_SEMI,
    STATE(612), 1,
      sym_declaration_list,
    STATE(4452), 2,
      sym_line_comment,
      sym_block_comment,
  [137311] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(8984), 1,
      sym_identifier,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(4453), 2,
      sym_line_comment,
      sym_block_comment,
  [137334] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(8984), 1,
      sym_identifier,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(4454), 2,
      sym_line_comment,
      sym_block_comment,
  [137357] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8988), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4455), 2,
      sym_line_comment,
      sym_block_comment,
  [137380] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4456), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(6917), 4,
      anon_sym_LBRACE,
      anon_sym_EQ_GT,
      anon_sym_AMP_AMP,
      anon_sym_SQUOTE,
  [137397] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(8992), 1,
      anon_sym_LBRACE,
    STATE(1156), 1,
      sym_macro_rules_v2,
    STATE(5945), 1,
      sym_macro_arguments_declaration,
    STATE(4457), 2,
      sym_line_comment,
      sym_block_comment,
  [137420] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6104), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4458), 2,
      sym_line_comment,
      sym_block_comment,
  [137443] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8994), 1,
      anon_sym_SEMI,
    STATE(6129), 1,
      sym_where_clause,
    STATE(4459), 2,
      sym_line_comment,
      sym_block_comment,
  [137466] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8996), 1,
      anon_sym_SEMI,
    STATE(6130), 1,
      sym_where_clause,
    STATE(4460), 2,
      sym_line_comment,
      sym_block_comment,
  [137489] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8998), 1,
      anon_sym_SEMI,
    STATE(6131), 1,
      sym_where_clause,
    STATE(4461), 2,
      sym_line_comment,
      sym_block_comment,
  [137512] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2467), 1,
      sym_enum_variant_list,
    STATE(5445), 1,
      sym_where_clause,
    STATE(4462), 2,
      sym_line_comment,
      sym_block_comment,
  [137535] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9000), 1,
      anon_sym_SEMI,
    STATE(6370), 1,
      sym_where_clause,
    STATE(4463), 2,
      sym_line_comment,
      sym_block_comment,
  [137558] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9002), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4464), 2,
      sym_line_comment,
      sym_block_comment,
  [137581] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(9004), 1,
      anon_sym_LBRACE,
    STATE(2427), 1,
      sym_macro_rules_v2,
    STATE(5736), 1,
      sym_macro_arguments_declaration,
    STATE(4465), 2,
      sym_line_comment,
      sym_block_comment,
  [137604] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9006), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4466), 2,
      sym_line_comment,
      sym_block_comment,
  [137627] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9008), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4467), 2,
      sym_line_comment,
      sym_block_comment,
  [137650] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9010), 1,
      anon_sym_SEMI,
    ACTIONS(9012), 1,
      anon_sym_EQ,
    ACTIONS(9014), 1,
      anon_sym_else,
    STATE(4468), 2,
      sym_line_comment,
      sym_block_comment,
  [137673] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9016), 1,
      anon_sym_SEMI,
    STATE(6191), 1,
      sym_where_clause,
    STATE(4469), 2,
      sym_line_comment,
      sym_block_comment,
  [137696] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9018), 1,
      anon_sym_SEMI,
    STATE(6192), 1,
      sym_where_clause,
    STATE(4470), 2,
      sym_line_comment,
      sym_block_comment,
  [137719] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9020), 1,
      anon_sym_SEMI,
    STATE(6197), 1,
      sym_where_clause,
    STATE(4471), 2,
      sym_line_comment,
      sym_block_comment,
  [137742] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9022), 1,
      anon_sym_SEMI,
    ACTIONS(9024), 1,
      anon_sym_EQ,
    STATE(6201), 1,
      sym_where_clause,
    STATE(4472), 2,
      sym_line_comment,
      sym_block_comment,
  [137765] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1714), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3804), 1,
      sym_block,
    STATE(6417), 1,
      sym_label,
    STATE(4473), 2,
      sym_line_comment,
      sym_block_comment,
  [137788] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9026), 1,
      anon_sym_SEMI,
    ACTIONS(9028), 1,
      anon_sym_EQ,
    STATE(6398), 1,
      sym_where_clause,
    STATE(4474), 2,
      sym_line_comment,
      sym_block_comment,
  [137811] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9030), 1,
      anon_sym_SEMI,
    ACTIONS(9032), 1,
      anon_sym_EQ,
    STATE(4576), 1,
      sym_where_clause,
    STATE(4475), 2,
      sym_line_comment,
      sym_block_comment,
  [137834] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9034), 1,
      anon_sym_SEMI,
    STATE(6217), 1,
      sym_where_clause,
    STATE(4476), 2,
      sym_line_comment,
      sym_block_comment,
  [137857] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9036), 1,
      anon_sym_SEMI,
    ACTIONS(9038), 1,
      anon_sym_EQ,
    STATE(6221), 1,
      sym_where_clause,
    STATE(4477), 2,
      sym_line_comment,
      sym_block_comment,
  [137880] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9040), 1,
      anon_sym_SEMI,
    ACTIONS(9042), 1,
      anon_sym_EQ,
    STATE(6225), 1,
      sym_where_clause,
    STATE(4478), 2,
      sym_line_comment,
      sym_block_comment,
  [137903] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9044), 1,
      anon_sym_SEMI,
    ACTIONS(9046), 1,
      anon_sym_EQ,
    STATE(4510), 1,
      sym_where_clause,
    STATE(4479), 2,
      sym_line_comment,
      sym_block_comment,
  [137926] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2488), 1,
      sym_field_declaration_list,
    STATE(5485), 1,
      sym_where_clause,
    STATE(4480), 2,
      sym_line_comment,
      sym_block_comment,
  [137949] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9048), 1,
      anon_sym_SEMI,
    STATE(6259), 1,
      sym_where_clause,
    STATE(4481), 2,
      sym_line_comment,
      sym_block_comment,
  [137972] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9050), 1,
      anon_sym_SEMI,
    STATE(6261), 1,
      sym_where_clause,
    STATE(4482), 2,
      sym_line_comment,
      sym_block_comment,
  [137995] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9052), 1,
      anon_sym_SEMI,
    STATE(6266), 1,
      sym_where_clause,
    STATE(4483), 2,
      sym_line_comment,
      sym_block_comment,
  [138018] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9054), 1,
      anon_sym_SEMI,
    ACTIONS(9056), 1,
      anon_sym_EQ,
    STATE(6275), 1,
      sym_where_clause,
    STATE(4484), 2,
      sym_line_comment,
      sym_block_comment,
  [138041] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(9004), 1,
      anon_sym_LBRACE,
    STATE(2492), 1,
      sym_macro_rules_v2,
    STATE(5495), 1,
      sym_macro_arguments_declaration,
    STATE(4485), 2,
      sym_line_comment,
      sym_block_comment,
  [138064] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3946), 1,
      sym_parameters,
    STATE(5706), 1,
      sym_type_parameters,
    STATE(4486), 2,
      sym_line_comment,
      sym_block_comment,
  [138087] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(9058), 1,
      anon_sym_async,
    ACTIONS(9060), 1,
      anon_sym_move,
    STATE(105), 1,
      sym_closure_parameters,
    STATE(4487), 2,
      sym_line_comment,
      sym_block_comment,
  [138110] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(576), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4488), 2,
      sym_line_comment,
      sym_block_comment,
  [138133] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9062), 1,
      anon_sym_SEMI,
    ACTIONS(9064), 1,
      anon_sym_EQ,
    ACTIONS(9066), 1,
      anon_sym_else,
    STATE(4489), 2,
      sym_line_comment,
      sym_block_comment,
  [138156] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9068), 1,
      anon_sym_SEMI,
    ACTIONS(9070), 1,
      anon_sym_fn,
    STATE(2506), 1,
      sym_declaration_list,
    STATE(4490), 2,
      sym_line_comment,
      sym_block_comment,
  [138179] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9072), 1,
      anon_sym_SEMI,
    STATE(6316), 1,
      sym_where_clause,
    STATE(4491), 2,
      sym_line_comment,
      sym_block_comment,
  [138202] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(482), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(4492), 2,
      sym_line_comment,
      sym_block_comment,
  [138225] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6186), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4493), 2,
      sym_line_comment,
      sym_block_comment,
  [138248] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3950), 1,
      sym_parameters,
    STATE(5774), 1,
      sym_type_parameters,
    STATE(4494), 2,
      sym_line_comment,
      sym_block_comment,
  [138271] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9068), 1,
      anon_sym_SEMI,
    STATE(2506), 1,
      sym_declaration_list,
    STATE(4495), 2,
      sym_line_comment,
      sym_block_comment,
  [138294] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    STATE(4496), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9074), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [138313] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(9004), 1,
      anon_sym_LBRACE,
    STATE(2508), 1,
      sym_macro_rules_v2,
    STATE(5514), 1,
      sym_macro_arguments_declaration,
    STATE(4497), 2,
      sym_line_comment,
      sym_block_comment,
  [138336] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9070), 1,
      anon_sym_fn,
    ACTIONS(9076), 1,
      anon_sym_SEMI,
    STATE(1091), 1,
      sym_declaration_list,
    STATE(4498), 2,
      sym_line_comment,
      sym_block_comment,
  [138359] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6028), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4499), 2,
      sym_line_comment,
      sym_block_comment,
  [138382] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9078), 1,
      anon_sym_SEMI,
    STATE(6359), 1,
      sym_where_clause,
    STATE(4500), 2,
      sym_line_comment,
      sym_block_comment,
  [138405] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9080), 1,
      anon_sym_SEMI,
    STATE(6360), 1,
      sym_where_clause,
    STATE(4501), 2,
      sym_line_comment,
      sym_block_comment,
  [138428] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9082), 1,
      anon_sym_SEMI,
    STATE(6361), 1,
      sym_where_clause,
    STATE(4502), 2,
      sym_line_comment,
      sym_block_comment,
  [138451] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6351), 1,
      sym_block,
    STATE(4503), 2,
      sym_line_comment,
      sym_block_comment,
  [138474] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9084), 1,
      anon_sym_SEMI,
    ACTIONS(9086), 1,
      anon_sym_EQ,
    STATE(4815), 1,
      sym_where_clause,
    STATE(4504), 2,
      sym_line_comment,
      sym_block_comment,
  [138497] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9088), 1,
      anon_sym_SEMI,
    STATE(6448), 1,
      sym_where_clause,
    STATE(4505), 2,
      sym_line_comment,
      sym_block_comment,
  [138520] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9070), 1,
      anon_sym_fn,
    ACTIONS(9090), 1,
      anon_sym_SEMI,
    STATE(1168), 1,
      sym_declaration_list,
    STATE(4506), 2,
      sym_line_comment,
      sym_block_comment,
  [138543] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9092), 1,
      anon_sym_SEMI,
    STATE(6387), 1,
      sym_where_clause,
    STATE(4507), 2,
      sym_line_comment,
      sym_block_comment,
  [138566] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9094), 1,
      anon_sym_SEMI,
    STATE(6388), 1,
      sym_where_clause,
    STATE(4508), 2,
      sym_line_comment,
      sym_block_comment,
  [138589] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9096), 1,
      anon_sym_SEMI,
    STATE(6392), 1,
      sym_where_clause,
    STATE(4509), 2,
      sym_line_comment,
      sym_block_comment,
  [138612] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9098), 1,
      anon_sym_SEMI,
    ACTIONS(9100), 1,
      anon_sym_EQ,
    STATE(6394), 1,
      sym_where_clause,
    STATE(4510), 2,
      sym_line_comment,
      sym_block_comment,
  [138635] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9070), 1,
      anon_sym_fn,
    ACTIONS(9102), 1,
      anon_sym_SEMI,
    STATE(2518), 1,
      sym_declaration_list,
    STATE(4511), 2,
      sym_line_comment,
      sym_block_comment,
  [138658] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9104), 1,
      anon_sym_SEMI,
    ACTIONS(9106), 1,
      anon_sym_EQ,
    STATE(4699), 1,
      sym_where_clause,
    STATE(4512), 2,
      sym_line_comment,
      sym_block_comment,
  [138681] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(475), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(4513), 2,
      sym_line_comment,
      sym_block_comment,
  [138704] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2910), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4514), 2,
      sym_line_comment,
      sym_block_comment,
  [138727] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9108), 1,
      anon_sym_SEMI,
    STATE(6407), 1,
      sym_where_clause,
    STATE(4515), 2,
      sym_line_comment,
      sym_block_comment,
  [138750] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9110), 1,
      anon_sym_SEMI,
    STATE(6409), 1,
      sym_where_clause,
    STATE(4516), 2,
      sym_line_comment,
      sym_block_comment,
  [138773] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9112), 1,
      anon_sym_SEMI,
    STATE(6415), 1,
      sym_where_clause,
    STATE(4517), 2,
      sym_line_comment,
      sym_block_comment,
  [138796] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9114), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4518), 2,
      sym_line_comment,
      sym_block_comment,
  [138819] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6356), 1,
      sym_block,
    STATE(4519), 2,
      sym_line_comment,
      sym_block_comment,
  [138842] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9116), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4520), 2,
      sym_line_comment,
      sym_block_comment,
  [138865] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9118), 1,
      sym_identifier,
    ACTIONS(9120), 1,
      sym_super,
    STATE(3688), 1,
      sym_type_arguments,
    STATE(4521), 2,
      sym_line_comment,
      sym_block_comment,
  [138888] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4522), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(5647), 4,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PLUS,
      anon_sym_GT,
  [138905] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9122), 1,
      anon_sym_SEMI,
    STATE(6465), 1,
      sym_where_clause,
    STATE(4523), 2,
      sym_line_comment,
      sym_block_comment,
  [138928] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9124), 1,
      anon_sym_SEMI,
    STATE(6057), 1,
      sym_where_clause,
    STATE(4524), 2,
      sym_line_comment,
      sym_block_comment,
  [138951] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9126), 1,
      anon_sym_DQUOTE,
    STATE(4685), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4525), 2,
      sym_line_comment,
      sym_block_comment,
  [138972] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3596), 1,
      sym_type_arguments,
    ACTIONS(9130), 2,
      sym_identifier,
      sym_super,
    STATE(4526), 2,
      sym_line_comment,
      sym_block_comment,
  [138993] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9132), 1,
      anon_sym_SEMI,
    STATE(6486), 1,
      sym_where_clause,
    STATE(4527), 2,
      sym_line_comment,
      sym_block_comment,
  [139016] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9134), 1,
      anon_sym_SEMI,
    STATE(6490), 1,
      sym_where_clause,
    STATE(4528), 2,
      sym_line_comment,
      sym_block_comment,
  [139039] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9136), 1,
      anon_sym_SEMI,
    STATE(6492), 1,
      sym_where_clause,
    STATE(4529), 2,
      sym_line_comment,
      sym_block_comment,
  [139062] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3587), 1,
      sym_type_arguments,
    ACTIONS(9130), 2,
      sym_identifier,
      sym_super,
    STATE(4530), 2,
      sym_line_comment,
      sym_block_comment,
  [139083] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9138), 1,
      anon_sym_RPAREN,
    ACTIONS(9140), 1,
      anon_sym_COMMA,
    STATE(5344), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4531), 2,
      sym_line_comment,
      sym_block_comment,
  [139106] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9142), 1,
      anon_sym_RPAREN,
    ACTIONS(9144), 1,
      anon_sym_COMMA,
    STATE(5360), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4532), 2,
      sym_line_comment,
      sym_block_comment,
  [139129] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9146), 1,
      anon_sym_SEMI,
    STATE(6181), 1,
      sym_where_clause,
    STATE(4533), 2,
      sym_line_comment,
      sym_block_comment,
  [139152] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7526), 1,
      anon_sym_DOT_DOT,
    ACTIONS(9148), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7528), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4534), 2,
      sym_line_comment,
      sym_block_comment,
  [139173] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(6001), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7536), 1,
      anon_sym_BANG,
    STATE(2843), 1,
      sym_type_arguments,
    STATE(4535), 2,
      sym_line_comment,
      sym_block_comment,
  [139196] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(8663), 1,
      anon_sym_RPAREN,
    ACTIONS(8665), 1,
      anon_sym_COMMA,
    STATE(5414), 1,
      aux_sym_parameters_repeat1,
    STATE(4536), 2,
      sym_line_comment,
      sym_block_comment,
  [139219] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9150), 1,
      anon_sym_SEMI,
    STATE(6018), 1,
      sym_where_clause,
    STATE(4537), 2,
      sym_line_comment,
      sym_block_comment,
  [139242] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9152), 1,
      anon_sym_RPAREN,
    ACTIONS(9154), 1,
      anon_sym_COMMA,
    STATE(5348), 1,
      aux_sym_tuple_type_repeat1,
    STATE(4538), 2,
      sym_line_comment,
      sym_block_comment,
  [139265] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8140), 1,
      anon_sym_LPAREN,
    ACTIONS(8142), 1,
      anon_sym_LBRACE,
    ACTIONS(8144), 1,
      anon_sym_LBRACK,
    STATE(2952), 1,
      sym_delim_token_tree,
    STATE(4539), 2,
      sym_line_comment,
      sym_block_comment,
  [139288] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9156), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4540), 2,
      sym_line_comment,
      sym_block_comment,
  [139311] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2964), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4541), 2,
      sym_line_comment,
      sym_block_comment,
  [139334] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9158), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4542), 2,
      sym_line_comment,
      sym_block_comment,
  [139357] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(593), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4543), 2,
      sym_line_comment,
      sym_block_comment,
  [139380] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9160), 1,
      anon_sym_DQUOTE,
    STATE(4559), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4544), 2,
      sym_line_comment,
      sym_block_comment,
  [139401] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    ACTIONS(9162), 1,
      anon_sym_LBRACE,
    STATE(5372), 1,
      sym_block,
    STATE(6431), 1,
      sym_label,
    STATE(4545), 2,
      sym_line_comment,
      sym_block_comment,
  [139424] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(9164), 1,
      anon_sym_async,
    ACTIONS(9166), 1,
      anon_sym_move,
    STATE(93), 1,
      sym_closure_parameters,
    STATE(4546), 2,
      sym_line_comment,
      sym_block_comment,
  [139447] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3275), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4547), 2,
      sym_line_comment,
      sym_block_comment,
  [139470] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2836), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4548), 2,
      sym_line_comment,
      sym_block_comment,
  [139493] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9168), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4549), 2,
      sym_line_comment,
      sym_block_comment,
  [139516] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(6677), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7534), 1,
      anon_sym_BANG,
    STATE(3175), 1,
      sym_type_arguments,
    STATE(4550), 2,
      sym_line_comment,
      sym_block_comment,
  [139539] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9170), 1,
      anon_sym_RPAREN,
    ACTIONS(9172), 1,
      anon_sym_COMMA,
    STATE(5055), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(4551), 2,
      sym_line_comment,
      sym_block_comment,
  [139562] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(8992), 1,
      anon_sym_LBRACE,
    STATE(1092), 1,
      sym_macro_rules_v2,
    STATE(5497), 1,
      sym_macro_arguments_declaration,
    STATE(4552), 2,
      sym_line_comment,
      sym_block_comment,
  [139585] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9174), 1,
      anon_sym_LPAREN,
    ACTIONS(9176), 1,
      anon_sym_LBRACE,
    ACTIONS(9178), 1,
      anon_sym_LBRACK,
    STATE(3562), 1,
      sym_delim_token_tree,
    STATE(4553), 2,
      sym_line_comment,
      sym_block_comment,
  [139608] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(588), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4554), 2,
      sym_line_comment,
      sym_block_comment,
  [139631] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(551), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4555), 2,
      sym_line_comment,
      sym_block_comment,
  [139654] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(5825), 1,
      sym_trait_bounds,
    ACTIONS(8615), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(4556), 2,
      sym_line_comment,
      sym_block_comment,
  [139675] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9180), 1,
      anon_sym_SEMI,
    ACTIONS(9182), 1,
      anon_sym_EQ,
    STATE(6263), 1,
      sym_where_clause,
    STATE(4557), 2,
      sym_line_comment,
      sym_block_comment,
  [139698] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(573), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4558), 2,
      sym_line_comment,
      sym_block_comment,
  [139721] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9184), 1,
      anon_sym_DQUOTE,
    STATE(4818), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4559), 2,
      sym_line_comment,
      sym_block_comment,
  [139742] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9174), 1,
      anon_sym_LPAREN,
    ACTIONS(9176), 1,
      anon_sym_LBRACE,
    ACTIONS(9178), 1,
      anon_sym_LBRACK,
    STATE(3572), 1,
      sym_delim_token_tree,
    STATE(4560), 2,
      sym_line_comment,
      sym_block_comment,
  [139765] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(9186), 1,
      sym_identifier,
    ACTIONS(9188), 1,
      sym_super,
    STATE(3103), 1,
      sym_type_arguments,
    STATE(4561), 2,
      sym_line_comment,
      sym_block_comment,
  [139788] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9190), 1,
      anon_sym_SEMI,
    ACTIONS(9192), 1,
      anon_sym_EQ,
    ACTIONS(9194), 1,
      anon_sym_else,
    STATE(4562), 2,
      sym_line_comment,
      sym_block_comment,
  [139811] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(9186), 1,
      sym_identifier,
    ACTIONS(9188), 1,
      sym_super,
    STATE(3106), 1,
      sym_type_arguments,
    STATE(4563), 2,
      sym_line_comment,
      sym_block_comment,
  [139834] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(8992), 1,
      anon_sym_LBRACE,
    STATE(1118), 1,
      sym_macro_rules_v2,
    STATE(5762), 1,
      sym_macro_arguments_declaration,
    STATE(4564), 2,
      sym_line_comment,
      sym_block_comment,
  [139857] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9130), 1,
      sym_super,
    ACTIONS(9196), 1,
      sym_identifier,
    ACTIONS(9198), 1,
      anon_sym_LT2,
    STATE(3788), 1,
      sym_type_arguments,
    STATE(4565), 2,
      sym_line_comment,
      sym_block_comment,
  [139880] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9130), 1,
      sym_super,
    ACTIONS(9196), 1,
      sym_identifier,
    ACTIONS(9198), 1,
      anon_sym_LT2,
    STATE(3789), 1,
      sym_type_arguments,
    STATE(4566), 2,
      sym_line_comment,
      sym_block_comment,
  [139903] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9200), 1,
      anon_sym_RPAREN,
    ACTIONS(9202), 1,
      anon_sym_COMMA,
    STATE(4912), 1,
      aux_sym_tuple_type_repeat1,
    STATE(4567), 2,
      sym_line_comment,
      sym_block_comment,
  [139926] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2932), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4568), 2,
      sym_line_comment,
      sym_block_comment,
  [139949] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9204), 1,
      anon_sym_SEMI,
    STATE(5971), 1,
      sym_where_clause,
    STATE(4569), 2,
      sym_line_comment,
      sym_block_comment,
  [139972] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9206), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4570), 2,
      sym_line_comment,
      sym_block_comment,
  [139995] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9118), 1,
      sym_identifier,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(4571), 2,
      sym_line_comment,
      sym_block_comment,
  [140018] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9118), 1,
      sym_identifier,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(4572), 2,
      sym_line_comment,
      sym_block_comment,
  [140041] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(582), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4573), 2,
      sym_line_comment,
      sym_block_comment,
  [140064] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9208), 1,
      anon_sym_COLON_COLON,
    STATE(4574), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7566), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140083] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9210), 1,
      anon_sym_COLON_COLON,
    STATE(4575), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140102] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9212), 1,
      anon_sym_SEMI,
    ACTIONS(9214), 1,
      anon_sym_EQ,
    STATE(5979), 1,
      sym_where_clause,
    STATE(4576), 2,
      sym_line_comment,
      sym_block_comment,
  [140125] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9216), 1,
      anon_sym_COLON_COLON,
    STATE(4577), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140144] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9218), 1,
      anon_sym_COLON_COLON,
    STATE(4578), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140163] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9220), 1,
      anon_sym_SEMI,
    ACTIONS(9222), 1,
      anon_sym_EQ,
    STATE(5998), 1,
      sym_where_clause,
    STATE(4579), 2,
      sym_line_comment,
      sym_block_comment,
  [140186] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(8819), 1,
      anon_sym_RPAREN,
    ACTIONS(8821), 1,
      anon_sym_COMMA,
    STATE(4936), 1,
      aux_sym_parameters_repeat1,
    STATE(4580), 2,
      sym_line_comment,
      sym_block_comment,
  [140209] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6248), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4581), 2,
      sym_line_comment,
      sym_block_comment,
  [140232] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9224), 1,
      anon_sym_RPAREN,
    ACTIONS(9226), 1,
      anon_sym_COMMA,
    STATE(4969), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(4582), 2,
      sym_line_comment,
      sym_block_comment,
  [140255] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3596), 1,
      sym_type_arguments,
    ACTIONS(8242), 2,
      sym_identifier,
      sym_super,
    STATE(4583), 2,
      sym_line_comment,
      sym_block_comment,
  [140276] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3587), 1,
      sym_type_arguments,
    ACTIONS(8242), 2,
      sym_identifier,
      sym_super,
    STATE(4584), 2,
      sym_line_comment,
      sym_block_comment,
  [140297] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(592), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4585), 2,
      sym_line_comment,
      sym_block_comment,
  [140320] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(5992), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4586), 2,
      sym_line_comment,
      sym_block_comment,
  [140343] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9208), 1,
      anon_sym_COLON_COLON,
    STATE(4587), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7582), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140362] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9210), 1,
      anon_sym_COLON_COLON,
    STATE(4588), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140381] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9216), 1,
      anon_sym_COLON_COLON,
    STATE(4589), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140400] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9228), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4590), 2,
      sym_line_comment,
      sym_block_comment,
  [140423] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9230), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4591), 2,
      sym_line_comment,
      sym_block_comment,
  [140446] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9218), 1,
      anon_sym_COLON_COLON,
    STATE(4592), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [140465] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9232), 1,
      anon_sym_SEMI,
    ACTIONS(9234), 1,
      anon_sym_EQ,
    STATE(4759), 1,
      sym_where_clause,
    STATE(4593), 2,
      sym_line_comment,
      sym_block_comment,
  [140488] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9236), 1,
      anon_sym_SEMI,
    STATE(6464), 1,
      sym_where_clause,
    STATE(4594), 2,
      sym_line_comment,
      sym_block_comment,
  [140511] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9238), 1,
      anon_sym_SEMI,
    STATE(6200), 1,
      sym_where_clause,
    STATE(4595), 2,
      sym_line_comment,
      sym_block_comment,
  [140534] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9240), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4596), 2,
      sym_line_comment,
      sym_block_comment,
  [140557] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9242), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4597), 2,
      sym_line_comment,
      sym_block_comment,
  [140580] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6495), 1,
      sym_block,
    STATE(4598), 2,
      sym_line_comment,
      sym_block_comment,
  [140603] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2569), 1,
      sym_enum_variant_list,
    STATE(5609), 1,
      sym_where_clause,
    STATE(4599), 2,
      sym_line_comment,
      sym_block_comment,
  [140626] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(5973), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4600), 2,
      sym_line_comment,
      sym_block_comment,
  [140649] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3267), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4601), 2,
      sym_line_comment,
      sym_block_comment,
  [140672] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9244), 1,
      anon_sym_DQUOTE,
    STATE(4612), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4602), 2,
      sym_line_comment,
      sym_block_comment,
  [140693] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(989), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(595), 1,
      sym_block,
    STATE(6434), 1,
      sym_label,
    STATE(4603), 2,
      sym_line_comment,
      sym_block_comment,
  [140716] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(7556), 1,
      anon_sym_move,
    ACTIONS(9246), 1,
      anon_sym_async,
    STATE(120), 1,
      sym_closure_parameters,
    STATE(4604), 2,
      sym_line_comment,
      sym_block_comment,
  [140739] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9248), 1,
      anon_sym_SEMI,
    ACTIONS(9250), 1,
      anon_sym_EQ,
    ACTIONS(9252), 1,
      anon_sym_else,
    STATE(4605), 2,
      sym_line_comment,
      sym_block_comment,
  [140762] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2987), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4606), 2,
      sym_line_comment,
      sym_block_comment,
  [140785] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7438), 1,
      anon_sym_LPAREN,
    ACTIONS(7440), 1,
      anon_sym_LBRACE,
    ACTIONS(7444), 1,
      anon_sym_LBRACK,
    STATE(2440), 1,
      sym_delim_token_tree,
    STATE(4607), 2,
      sym_line_comment,
      sym_block_comment,
  [140808] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3338), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4608), 2,
      sym_line_comment,
      sym_block_comment,
  [140831] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3081), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4609), 2,
      sym_line_comment,
      sym_block_comment,
  [140854] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9254), 1,
      anon_sym_SEMI,
    STATE(6211), 1,
      sym_where_clause,
    STATE(4610), 2,
      sym_line_comment,
      sym_block_comment,
  [140877] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3345), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4611), 2,
      sym_line_comment,
      sym_block_comment,
  [140900] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9256), 1,
      anon_sym_DQUOTE,
    STATE(4818), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4612), 2,
      sym_line_comment,
      sym_block_comment,
  [140921] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7438), 1,
      anon_sym_LPAREN,
    ACTIONS(7440), 1,
      anon_sym_LBRACE,
    ACTIONS(7444), 1,
      anon_sym_LBRACK,
    STATE(2442), 1,
      sym_delim_token_tree,
    STATE(4613), 2,
      sym_line_comment,
      sym_block_comment,
  [140944] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9258), 1,
      anon_sym_SEMI,
    ACTIONS(9260), 1,
      anon_sym_EQ,
    STATE(6408), 1,
      sym_where_clause,
    STATE(4614), 2,
      sym_line_comment,
      sym_block_comment,
  [140967] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9262), 1,
      sym_identifier,
    STATE(2883), 1,
      sym_type_arguments,
    STATE(4615), 2,
      sym_line_comment,
      sym_block_comment,
  [140990] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2976), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4616), 2,
      sym_line_comment,
      sym_block_comment,
  [141013] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9264), 1,
      sym_identifier,
    STATE(3943), 1,
      sym_type_arguments,
    STATE(4617), 2,
      sym_line_comment,
      sym_block_comment,
  [141036] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9264), 1,
      sym_identifier,
    STATE(3945), 1,
      sym_type_arguments,
    STATE(4618), 2,
      sym_line_comment,
      sym_block_comment,
  [141059] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(2825), 1,
      sym_type_arguments,
    STATE(4619), 2,
      sym_line_comment,
      sym_block_comment,
  [141082] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(2828), 1,
      sym_type_arguments,
    STATE(4620), 2,
      sym_line_comment,
      sym_block_comment,
  [141105] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9268), 1,
      anon_sym_SEMI,
    ACTIONS(9270), 1,
      anon_sym_EQ,
    STATE(6133), 1,
      sym_where_clause,
    STATE(4621), 2,
      sym_line_comment,
      sym_block_comment,
  [141128] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3412), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4622), 2,
      sym_line_comment,
      sym_block_comment,
  [141151] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8984), 1,
      sym_identifier,
    ACTIONS(9120), 1,
      sym_super,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(4623), 2,
      sym_line_comment,
      sym_block_comment,
  [141174] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6472), 1,
      sym_block,
    STATE(4624), 2,
      sym_line_comment,
      sym_block_comment,
  [141197] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9272), 1,
      anon_sym_SEMI,
    ACTIONS(9274), 1,
      anon_sym_EQ,
    STATE(4811), 1,
      sym_where_clause,
    STATE(4625), 2,
      sym_line_comment,
      sym_block_comment,
  [141220] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9276), 1,
      anon_sym_SEMI,
    ACTIONS(9278), 1,
      anon_sym_EQ,
    STATE(6013), 1,
      sym_where_clause,
    STATE(4626), 2,
      sym_line_comment,
      sym_block_comment,
  [141243] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(456), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(3436), 1,
      sym_block,
    STATE(6435), 1,
      sym_label,
    STATE(4627), 2,
      sym_line_comment,
      sym_block_comment,
  [141266] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9280), 1,
      anon_sym_SEMI,
    STATE(6369), 1,
      sym_where_clause,
    STATE(4628), 2,
      sym_line_comment,
      sym_block_comment,
  [141289] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9282), 1,
      anon_sym_SEMI,
    STATE(6254), 1,
      sym_where_clause,
    STATE(4629), 2,
      sym_line_comment,
      sym_block_comment,
  [141312] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9284), 1,
      anon_sym_DQUOTE,
    STATE(4636), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4630), 2,
      sym_line_comment,
      sym_block_comment,
  [141333] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9286), 1,
      anon_sym_SEMI,
    ACTIONS(9288), 1,
      anon_sym_EQ,
    ACTIONS(9290), 1,
      anon_sym_else,
    STATE(4631), 2,
      sym_line_comment,
      sym_block_comment,
  [141356] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2589), 1,
      sym_field_declaration_list,
    STATE(5644), 1,
      sym_where_clause,
    STATE(4632), 2,
      sym_line_comment,
      sym_block_comment,
  [141379] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(9292), 1,
      anon_sym_async,
    ACTIONS(9294), 1,
      anon_sym_move,
    STATE(127), 1,
      sym_closure_parameters,
    STATE(4633), 2,
      sym_line_comment,
      sym_block_comment,
  [141402] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8387), 1,
      anon_sym_COLON,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(4634), 2,
      sym_line_comment,
      sym_block_comment,
  [141425] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9296), 1,
      anon_sym_LPAREN,
    ACTIONS(9298), 1,
      anon_sym_LBRACE,
    ACTIONS(9300), 1,
      anon_sym_LBRACK,
    STATE(5380), 1,
      sym_delim_token_tree,
    STATE(4635), 2,
      sym_line_comment,
      sym_block_comment,
  [141448] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9302), 1,
      anon_sym_DQUOTE,
    STATE(4818), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4636), 2,
      sym_line_comment,
      sym_block_comment,
  [141469] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9296), 1,
      anon_sym_LPAREN,
    ACTIONS(9298), 1,
      anon_sym_LBRACE,
    ACTIONS(9300), 1,
      anon_sym_LBRACK,
    STATE(5394), 1,
      sym_delim_token_tree,
    STATE(4637), 2,
      sym_line_comment,
      sym_block_comment,
  [141492] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1335), 1,
      sym_field_declaration_list,
    STATE(5525), 1,
      sym_where_clause,
    STATE(4638), 2,
      sym_line_comment,
      sym_block_comment,
  [141515] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(9004), 1,
      anon_sym_LBRACE,
    STATE(2593), 1,
      sym_macro_rules_v2,
    STATE(5651), 1,
      sym_macro_arguments_declaration,
    STATE(4639), 2,
      sym_line_comment,
      sym_block_comment,
  [141538] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9304), 1,
      anon_sym_SEMI,
    STATE(6162), 1,
      sym_where_clause,
    STATE(4640), 2,
      sym_line_comment,
      sym_block_comment,
  [141561] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9306), 1,
      sym_identifier,
    STATE(3967), 1,
      sym_type_arguments,
    STATE(4641), 2,
      sym_line_comment,
      sym_block_comment,
  [141584] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9306), 1,
      sym_identifier,
    STATE(3968), 1,
      sym_type_arguments,
    STATE(4642), 2,
      sym_line_comment,
      sym_block_comment,
  [141607] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(4643), 2,
      sym_line_comment,
      sym_block_comment,
  [141630] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(4644), 2,
      sym_line_comment,
      sym_block_comment,
  [141653] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9308), 1,
      anon_sym_DQUOTE,
    STATE(4818), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4645), 2,
      sym_line_comment,
      sym_block_comment,
  [141674] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9310), 1,
      anon_sym_DQUOTE,
    STATE(4648), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4646), 2,
      sym_line_comment,
      sym_block_comment,
  [141695] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9312), 1,
      anon_sym_LPAREN,
    ACTIONS(9314), 1,
      anon_sym_LBRACE,
    ACTIONS(9316), 1,
      anon_sym_LBRACK,
    STATE(3238), 1,
      sym_delim_token_tree,
    STATE(4647), 2,
      sym_line_comment,
      sym_block_comment,
  [141718] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9318), 1,
      anon_sym_DQUOTE,
    STATE(4818), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4648), 2,
      sym_line_comment,
      sym_block_comment,
  [141739] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9312), 1,
      anon_sym_LPAREN,
    ACTIONS(9314), 1,
      anon_sym_LBRACE,
    ACTIONS(9316), 1,
      anon_sym_LBRACK,
    STATE(3245), 1,
      sym_delim_token_tree,
    STATE(4649), 2,
      sym_line_comment,
      sym_block_comment,
  [141762] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8984), 1,
      sym_identifier,
    ACTIONS(9120), 1,
      sym_super,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(4650), 2,
      sym_line_comment,
      sym_block_comment,
  [141785] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(9320), 1,
      anon_sym_async,
    ACTIONS(9322), 1,
      anon_sym_move,
    STATE(101), 1,
      sym_closure_parameters,
    STATE(4651), 2,
      sym_line_comment,
      sym_block_comment,
  [141808] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(4652), 2,
      sym_line_comment,
      sym_block_comment,
  [141831] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(4653), 2,
      sym_line_comment,
      sym_block_comment,
  [141854] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9324), 1,
      anon_sym_RPAREN,
    ACTIONS(9326), 1,
      anon_sym_COMMA,
    STATE(4870), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(4654), 2,
      sym_line_comment,
      sym_block_comment,
  [141877] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9328), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4655), 2,
      sym_line_comment,
      sym_block_comment,
  [141900] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9330), 1,
      anon_sym_RPAREN,
    ACTIONS(9332), 1,
      anon_sym_COMMA,
    STATE(5263), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(4656), 2,
      sym_line_comment,
      sym_block_comment,
  [141923] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(3182), 1,
      sym_type_arguments,
    STATE(4657), 2,
      sym_line_comment,
      sym_block_comment,
  [141946] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(3157), 1,
      sym_type_arguments,
    STATE(4658), 2,
      sym_line_comment,
      sym_block_comment,
  [141969] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9336), 1,
      anon_sym_RPAREN,
    ACTIONS(9338), 1,
      anon_sym_COMMA,
    STATE(4877), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(4659), 2,
      sym_line_comment,
      sym_block_comment,
  [141992] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9070), 1,
      anon_sym_fn,
    ACTIONS(9340), 1,
      anon_sym_SEMI,
    STATE(2603), 1,
      sym_declaration_list,
    STATE(4660), 2,
      sym_line_comment,
      sym_block_comment,
  [142015] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9342), 1,
      anon_sym_SEMI,
    STATE(6001), 1,
      sym_where_clause,
    STATE(4661), 2,
      sym_line_comment,
      sym_block_comment,
  [142038] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(4662), 2,
      sym_line_comment,
      sym_block_comment,
  [142061] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(4663), 2,
      sym_line_comment,
      sym_block_comment,
  [142084] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2608), 1,
      sym_enum_variant_list,
    STATE(5663), 1,
      sym_where_clause,
    STATE(4664), 2,
      sym_line_comment,
      sym_block_comment,
  [142107] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(3596), 1,
      sym_type_arguments,
    STATE(4665), 2,
      sym_line_comment,
      sym_block_comment,
  [142130] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(3587), 1,
      sym_type_arguments,
    STATE(4666), 2,
      sym_line_comment,
      sym_block_comment,
  [142153] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8140), 1,
      anon_sym_LPAREN,
    ACTIONS(8142), 1,
      anon_sym_LBRACE,
    ACTIONS(8144), 1,
      anon_sym_LBRACK,
    STATE(2984), 1,
      sym_delim_token_tree,
    STATE(4667), 2,
      sym_line_comment,
      sym_block_comment,
  [142176] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9344), 1,
      anon_sym_SEMI,
    ACTIONS(9346), 1,
      anon_sym_EQ,
    STATE(6303), 1,
      sym_where_clause,
    STATE(4668), 2,
      sym_line_comment,
      sym_block_comment,
  [142199] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8182), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4669), 2,
      sym_line_comment,
      sym_block_comment,
  [142220] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9348), 1,
      anon_sym_SEMI,
    ACTIONS(9350), 1,
      anon_sym_EQ,
    STATE(4848), 1,
      sym_where_clause,
    STATE(4670), 2,
      sym_line_comment,
      sym_block_comment,
  [142243] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    STATE(4671), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9352), 3,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_GT,
  [142262] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9354), 1,
      anon_sym_LBRACE,
    ACTIONS(9356), 1,
      anon_sym_for,
    ACTIONS(9358), 1,
      anon_sym_loop,
    ACTIONS(9360), 1,
      anon_sym_while,
    STATE(4672), 2,
      sym_line_comment,
      sym_block_comment,
  [142285] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2618), 1,
      sym_field_declaration_list,
    STATE(5673), 1,
      sym_where_clause,
    STATE(4673), 2,
      sym_line_comment,
      sym_block_comment,
  [142308] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3904), 1,
      sym_parameters,
    STATE(5686), 1,
      sym_type_parameters,
    STATE(4674), 2,
      sym_line_comment,
      sym_block_comment,
  [142331] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9362), 1,
      anon_sym_in,
    STATE(4675), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9364), 3,
      sym_self,
      sym_super,
      sym_crate,
  [142350] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2973), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4676), 2,
      sym_line_comment,
      sym_block_comment,
  [142373] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(478), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(4677), 2,
      sym_line_comment,
      sym_block_comment,
  [142396] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(2933), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4678), 2,
      sym_line_comment,
      sym_block_comment,
  [142419] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9366), 1,
      anon_sym_SEMI,
    STATE(5988), 1,
      sym_where_clause,
    STATE(4679), 2,
      sym_line_comment,
      sym_block_comment,
  [142442] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9368), 1,
      anon_sym_LBRACE,
    ACTIONS(9370), 1,
      anon_sym_for,
    ACTIONS(9372), 1,
      anon_sym_loop,
    ACTIONS(9374), 1,
      anon_sym_while,
    STATE(4680), 2,
      sym_line_comment,
      sym_block_comment,
  [142465] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9208), 1,
      anon_sym_COLON_COLON,
    STATE(4681), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7540), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [142484] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9210), 1,
      anon_sym_COLON_COLON,
    STATE(4682), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [142503] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3596), 1,
      sym_type_arguments,
    ACTIONS(9120), 2,
      sym_identifier,
      sym_super,
    STATE(4683), 2,
      sym_line_comment,
      sym_block_comment,
  [142524] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9376), 1,
      anon_sym_SEMI,
    STATE(6109), 1,
      sym_where_clause,
    STATE(4684), 2,
      sym_line_comment,
      sym_block_comment,
  [142547] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9378), 1,
      anon_sym_DQUOTE,
    STATE(4818), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4685), 2,
      sym_line_comment,
      sym_block_comment,
  [142568] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9380), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4686), 2,
      sym_line_comment,
      sym_block_comment,
  [142591] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(5962), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4687), 2,
      sym_line_comment,
      sym_block_comment,
  [142614] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9382), 1,
      anon_sym_SEMI,
    STATE(6223), 1,
      sym_where_clause,
    STATE(4688), 2,
      sym_line_comment,
      sym_block_comment,
  [142637] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3909), 1,
      sym_parameters,
    STATE(5719), 1,
      sym_type_parameters,
    STATE(4689), 2,
      sym_line_comment,
      sym_block_comment,
  [142660] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9384), 1,
      anon_sym_COLON,
    STATE(4690), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7699), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [142679] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1082), 1,
      sym_enum_variant_list,
    STATE(5677), 1,
      sym_where_clause,
    STATE(4691), 2,
      sym_line_comment,
      sym_block_comment,
  [142702] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3587), 1,
      sym_type_arguments,
    ACTIONS(9120), 2,
      sym_identifier,
      sym_super,
    STATE(4692), 2,
      sym_line_comment,
      sym_block_comment,
  [142723] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3911), 1,
      sym_parameters,
    STATE(5726), 1,
      sym_type_parameters,
    STATE(4693), 2,
      sym_line_comment,
      sym_block_comment,
  [142746] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9386), 1,
      anon_sym_SEMI,
    ACTIONS(9388), 1,
      anon_sym_EQ,
    ACTIONS(9390), 1,
      anon_sym_else,
    STATE(4694), 2,
      sym_line_comment,
      sym_block_comment,
  [142769] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(414), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(4695), 2,
      sym_line_comment,
      sym_block_comment,
  [142792] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9216), 1,
      anon_sym_COLON_COLON,
    STATE(4696), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [142811] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9392), 1,
      anon_sym_SEMI,
    ACTIONS(9394), 1,
      anon_sym_EQ,
    STATE(6008), 1,
      sym_where_clause,
    STATE(4697), 2,
      sym_line_comment,
      sym_block_comment,
  [142834] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(8637), 1,
      anon_sym_RPAREN,
    ACTIONS(8639), 1,
      anon_sym_COMMA,
    STATE(4979), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4698), 2,
      sym_line_comment,
      sym_block_comment,
  [142857] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9396), 1,
      anon_sym_SEMI,
    ACTIONS(9398), 1,
      anon_sym_EQ,
    STATE(6377), 1,
      sym_where_clause,
    STATE(4699), 2,
      sym_line_comment,
      sym_block_comment,
  [142880] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9218), 1,
      anon_sym_COLON_COLON,
    STATE(4700), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [142899] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(77), 1,
      anon_sym_pub,
    ACTIONS(7382), 1,
      sym_crate,
    ACTIONS(9400), 1,
      sym_identifier,
    STATE(6237), 1,
      sym_visibility_modifier,
    STATE(4701), 2,
      sym_line_comment,
      sym_block_comment,
  [142922] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9402), 1,
      anon_sym_SEMI,
    ACTIONS(9404), 1,
      anon_sym_EQ,
    STATE(6012), 1,
      sym_where_clause,
    STATE(4702), 2,
      sym_line_comment,
      sym_block_comment,
  [142945] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9406), 1,
      anon_sym_LPAREN,
    ACTIONS(9408), 1,
      anon_sym_LBRACE,
    ACTIONS(9410), 1,
      anon_sym_LBRACK,
    STATE(5548), 1,
      sym_token_tree,
    STATE(4703), 2,
      sym_line_comment,
      sym_block_comment,
  [142968] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6169), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4704), 2,
      sym_line_comment,
      sym_block_comment,
  [142991] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9412), 1,
      anon_sym_SEMI,
    ACTIONS(9414), 1,
      anon_sym_EQ,
    STATE(5966), 1,
      sym_where_clause,
    STATE(4705), 2,
      sym_line_comment,
      sym_block_comment,
  [143014] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6212), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4706), 2,
      sym_line_comment,
      sym_block_comment,
  [143037] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9416), 1,
      anon_sym_SEMI,
    ACTIONS(9418), 1,
      anon_sym_EQ,
    STATE(4450), 1,
      sym_where_clause,
    STATE(4707), 2,
      sym_line_comment,
      sym_block_comment,
  [143060] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9420), 1,
      anon_sym_SEMI,
    ACTIONS(9422), 1,
      anon_sym_EQ,
    STATE(4730), 1,
      sym_where_clause,
    STATE(4708), 2,
      sym_line_comment,
      sym_block_comment,
  [143083] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7426), 1,
      anon_sym_LPAREN,
    ACTIONS(7428), 1,
      anon_sym_LBRACE,
    ACTIONS(7432), 1,
      anon_sym_LBRACK,
    STATE(485), 1,
      sym_delim_token_tree,
    STATE(4709), 2,
      sym_line_comment,
      sym_block_comment,
  [143106] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9424), 1,
      anon_sym_SEMI,
    STATE(6209), 1,
      sym_where_clause,
    STATE(4710), 2,
      sym_line_comment,
      sym_block_comment,
  [143129] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9262), 1,
      sym_identifier,
    STATE(2965), 1,
      sym_type_arguments,
    STATE(4711), 2,
      sym_line_comment,
      sym_block_comment,
  [143152] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6466), 1,
      sym_block,
    STATE(4712), 2,
      sym_line_comment,
      sym_block_comment,
  [143175] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9426), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4713), 2,
      sym_line_comment,
      sym_block_comment,
  [143198] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9428), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4714), 2,
      sym_line_comment,
      sym_block_comment,
  [143221] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9430), 1,
      anon_sym_SEMI,
    STATE(6213), 1,
      sym_where_clause,
    STATE(4715), 2,
      sym_line_comment,
      sym_block_comment,
  [143244] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9432), 1,
      sym_identifier,
    STATE(3099), 1,
      sym_type_arguments,
    STATE(4716), 2,
      sym_line_comment,
      sym_block_comment,
  [143267] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9434), 1,
      anon_sym_SEMI,
    STATE(2450), 1,
      sym_declaration_list,
    STATE(4717), 2,
      sym_line_comment,
      sym_block_comment,
  [143290] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6076), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4718), 2,
      sym_line_comment,
      sym_block_comment,
  [143313] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3917), 1,
      sym_parameters,
    STATE(5759), 1,
      sym_type_parameters,
    STATE(4719), 2,
      sym_line_comment,
      sym_block_comment,
  [143336] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3920), 1,
      sym_parameters,
    STATE(5764), 1,
      sym_type_parameters,
    STATE(4720), 2,
      sym_line_comment,
      sym_block_comment,
  [143359] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9436), 1,
      anon_sym_SEMI,
    STATE(6025), 1,
      sym_where_clause,
    STATE(4721), 2,
      sym_line_comment,
      sym_block_comment,
  [143382] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9438), 1,
      anon_sym_SEMI,
    STATE(6066), 1,
      sym_where_clause,
    STATE(4722), 2,
      sym_line_comment,
      sym_block_comment,
  [143405] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9440), 1,
      anon_sym_SEMI,
    STATE(6094), 1,
      sym_where_clause,
    STATE(4723), 2,
      sym_line_comment,
      sym_block_comment,
  [143428] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(4724), 2,
      sym_line_comment,
      sym_block_comment,
  [143451] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9442), 1,
      anon_sym_SEMI,
    STATE(6376), 1,
      sym_where_clause,
    STATE(4725), 2,
      sym_line_comment,
      sym_block_comment,
  [143474] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9444), 1,
      anon_sym_SEMI,
    STATE(6457), 1,
      sym_where_clause,
    STATE(4726), 2,
      sym_line_comment,
      sym_block_comment,
  [143497] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9446), 1,
      anon_sym_SEMI,
    STATE(5985), 1,
      sym_where_clause,
    STATE(4727), 2,
      sym_line_comment,
      sym_block_comment,
  [143520] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6488), 1,
      sym_block,
    STATE(4728), 2,
      sym_line_comment,
      sym_block_comment,
  [143543] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6489), 1,
      sym_block,
    STATE(4729), 2,
      sym_line_comment,
      sym_block_comment,
  [143566] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9448), 1,
      anon_sym_SEMI,
    ACTIONS(9450), 1,
      anon_sym_EQ,
    STATE(6059), 1,
      sym_where_clause,
    STATE(4730), 2,
      sym_line_comment,
      sym_block_comment,
  [143589] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7546), 1,
      anon_sym_for,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4731), 2,
      sym_line_comment,
      sym_block_comment,
  [143612] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    STATE(4732), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9452), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [143631] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9454), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4733), 2,
      sym_line_comment,
      sym_block_comment,
  [143654] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9456), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4734), 2,
      sym_line_comment,
      sym_block_comment,
  [143677] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9070), 1,
      anon_sym_fn,
    ACTIONS(9458), 1,
      anon_sym_SEMI,
    STATE(2452), 1,
      sym_declaration_list,
    STATE(4735), 2,
      sym_line_comment,
      sym_block_comment,
  [143700] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    STATE(4736), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9460), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [143719] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9458), 1,
      anon_sym_SEMI,
    STATE(2452), 1,
      sym_declaration_list,
    STATE(4737), 2,
      sym_line_comment,
      sym_block_comment,
  [143742] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6017), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4738), 2,
      sym_line_comment,
      sym_block_comment,
  [143765] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9432), 1,
      sym_identifier,
    STATE(3086), 1,
      sym_type_arguments,
    STATE(4739), 2,
      sym_line_comment,
      sym_block_comment,
  [143788] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3926), 1,
      sym_parameters,
    STATE(5795), 1,
      sym_type_parameters,
    STATE(4740), 2,
      sym_line_comment,
      sym_block_comment,
  [143811] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9462), 1,
      anon_sym_SEMI,
    ACTIONS(9464), 1,
      anon_sym_EQ,
    ACTIONS(9466), 1,
      anon_sym_else,
    STATE(4741), 2,
      sym_line_comment,
      sym_block_comment,
  [143834] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9468), 1,
      anon_sym_SEMI,
    STATE(5964), 1,
      sym_where_clause,
    STATE(4742), 2,
      sym_line_comment,
      sym_block_comment,
  [143857] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6121), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4743), 2,
      sym_line_comment,
      sym_block_comment,
  [143880] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6125), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4744), 2,
      sym_line_comment,
      sym_block_comment,
  [143903] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9470), 1,
      anon_sym_SEMI,
    STATE(6058), 1,
      sym_where_clause,
    STATE(4745), 2,
      sym_line_comment,
      sym_block_comment,
  [143926] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1237), 1,
      sym_enum_variant_list,
    STATE(5763), 1,
      sym_where_clause,
    STATE(4746), 2,
      sym_line_comment,
      sym_block_comment,
  [143949] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4747), 2,
      sym_line_comment,
      sym_block_comment,
  [143970] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1283), 1,
      sym_enum_variant_list,
    STATE(5675), 1,
      sym_where_clause,
    STATE(4748), 2,
      sym_line_comment,
      sym_block_comment,
  [143993] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9472), 1,
      anon_sym_SEMI,
    ACTIONS(9474), 1,
      anon_sym_EQ,
    STATE(6171), 1,
      sym_where_clause,
    STATE(4749), 2,
      sym_line_comment,
      sym_block_comment,
  [144016] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9476), 1,
      anon_sym_SEMI,
    ACTIONS(9478), 1,
      anon_sym_EQ,
    STATE(6246), 1,
      sym_where_clause,
    STATE(4750), 2,
      sym_line_comment,
      sym_block_comment,
  [144039] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9480), 1,
      anon_sym_SEMI,
    ACTIONS(9482), 1,
      anon_sym_EQ,
    STATE(4626), 1,
      sym_where_clause,
    STATE(4751), 2,
      sym_line_comment,
      sym_block_comment,
  [144062] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9484), 1,
      anon_sym_SEMI,
    STATE(5965), 1,
      sym_where_clause,
    STATE(4752), 2,
      sym_line_comment,
      sym_block_comment,
  [144085] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7606), 1,
      anon_sym_for,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4753), 2,
      sym_line_comment,
      sym_block_comment,
  [144108] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9486), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4754), 2,
      sym_line_comment,
      sym_block_comment,
  [144131] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6179), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4755), 2,
      sym_line_comment,
      sym_block_comment,
  [144154] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6180), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4756), 2,
      sym_line_comment,
      sym_block_comment,
  [144177] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9488), 1,
      anon_sym_SEMI,
    STATE(5970), 1,
      sym_where_clause,
    STATE(4757), 2,
      sym_line_comment,
      sym_block_comment,
  [144200] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9490), 1,
      anon_sym_SEMI,
    ACTIONS(9492), 1,
      anon_sym_EQ,
    STATE(6080), 1,
      sym_where_clause,
    STATE(4758), 2,
      sym_line_comment,
      sym_block_comment,
  [144223] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9494), 1,
      anon_sym_SEMI,
    ACTIONS(9496), 1,
      anon_sym_EQ,
    STATE(5975), 1,
      sym_where_clause,
    STATE(4759), 2,
      sym_line_comment,
      sym_block_comment,
  [144246] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9498), 1,
      anon_sym_in,
    STATE(4760), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9500), 3,
      sym_self,
      sym_super,
      sym_crate,
  [144265] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9502), 1,
      anon_sym_SEMI,
    ACTIONS(9504), 1,
      anon_sym_EQ,
    STATE(4749), 1,
      sym_where_clause,
    STATE(4761), 2,
      sym_line_comment,
      sym_block_comment,
  [144288] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9506), 1,
      anon_sym_COMMA,
    ACTIONS(9460), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
    STATE(4762), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_slice_pattern_repeat1,
  [144307] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    STATE(4763), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9509), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [144326] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1264), 1,
      sym_field_declaration_list,
    STATE(5782), 1,
      sym_where_clause,
    STATE(4764), 2,
      sym_line_comment,
      sym_block_comment,
  [144349] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(468), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(4765), 2,
      sym_line_comment,
      sym_block_comment,
  [144372] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9511), 1,
      anon_sym_DQUOTE,
    STATE(4645), 1,
      aux_sym_string_literal_repeat1,
    ACTIONS(9128), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4766), 2,
      sym_line_comment,
      sym_block_comment,
  [144393] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6312), 1,
      sym_block,
    STATE(4767), 2,
      sym_line_comment,
      sym_block_comment,
  [144416] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3951), 1,
      sym_parameters,
    STATE(5784), 1,
      sym_type_parameters,
    STATE(4768), 2,
      sym_line_comment,
      sym_block_comment,
  [144439] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9513), 1,
      anon_sym_SEMI,
    STATE(6459), 1,
      sym_where_clause,
    STATE(4769), 2,
      sym_line_comment,
      sym_block_comment,
  [144462] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9515), 1,
      anon_sym_SEMI,
    STATE(6462), 1,
      sym_where_clause,
    STATE(4770), 2,
      sym_line_comment,
      sym_block_comment,
  [144485] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9517), 1,
      anon_sym_SEMI,
    STATE(6484), 1,
      sym_where_clause,
    STATE(4771), 2,
      sym_line_comment,
      sym_block_comment,
  [144508] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6039), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4772), 2,
      sym_line_comment,
      sym_block_comment,
  [144531] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6353), 1,
      sym_block,
    STATE(4773), 2,
      sym_line_comment,
      sym_block_comment,
  [144554] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6354), 1,
      sym_block,
    STATE(4774), 2,
      sym_line_comment,
      sym_block_comment,
  [144577] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    STATE(4775), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9519), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [144596] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9521), 1,
      anon_sym_SEMI,
    ACTIONS(9523), 1,
      anon_sym_EQ,
    STATE(6004), 1,
      sym_where_clause,
    STATE(4776), 2,
      sym_line_comment,
      sym_block_comment,
  [144619] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9525), 1,
      anon_sym_RPAREN,
    ACTIONS(9527), 1,
      anon_sym_COMMA,
    STATE(4927), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(4777), 2,
      sym_line_comment,
      sym_block_comment,
  [144642] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    ACTIONS(9529), 1,
      anon_sym_GT,
    STATE(5883), 1,
      sym_lifetime,
    STATE(4778), 2,
      sym_line_comment,
      sym_block_comment,
  [144665] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9531), 1,
      anon_sym_SEMI,
    STATE(6196), 1,
      sym_where_clause,
    STATE(4779), 2,
      sym_line_comment,
      sym_block_comment,
  [144688] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9533), 1,
      anon_sym_COMMA,
    ACTIONS(9535), 1,
      anon_sym_RBRACK,
    STATE(4935), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4780), 2,
      sym_line_comment,
      sym_block_comment,
  [144711] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9537), 1,
      anon_sym_SEMI,
    STATE(6206), 1,
      sym_where_clause,
    STATE(4781), 2,
      sym_line_comment,
      sym_block_comment,
  [144734] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9070), 1,
      anon_sym_fn,
    ACTIONS(9539), 1,
      anon_sym_SEMI,
    STATE(1153), 1,
      sym_declaration_list,
    STATE(4782), 2,
      sym_line_comment,
      sym_block_comment,
  [144757] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9541), 1,
      anon_sym_SEMI,
    STATE(6208), 1,
      sym_where_clause,
    STATE(4783), 2,
      sym_line_comment,
      sym_block_comment,
  [144780] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9543), 1,
      anon_sym_COMMA,
    ACTIONS(9545), 1,
      anon_sym_RBRACK,
    STATE(5085), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4784), 2,
      sym_line_comment,
      sym_block_comment,
  [144803] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6235), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4785), 2,
      sym_line_comment,
      sym_block_comment,
  [144826] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6449), 1,
      sym_block,
    STATE(4786), 2,
      sym_line_comment,
      sym_block_comment,
  [144849] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9547), 1,
      anon_sym_SEMI,
    ACTIONS(9549), 1,
      anon_sym_EQ,
    ACTIONS(9551), 1,
      anon_sym_else,
    STATE(4787), 2,
      sym_line_comment,
      sym_block_comment,
  [144872] = 5,
    ACTIONS(9553), 1,
      aux_sym_token_repetition_pattern_token1,
    ACTIONS(9557), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(9559), 1,
      anon_sym_SLASH_STAR,
    STATE(4788), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9555), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [144891] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6308), 1,
      sym_label,
    STATE(6463), 1,
      sym_block,
    STATE(4789), 2,
      sym_line_comment,
      sym_block_comment,
  [144914] = 5,
    ACTIONS(9557), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(9559), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9561), 1,
      aux_sym_token_repetition_pattern_token1,
    STATE(4790), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9563), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [144933] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9565), 1,
      anon_sym_SEMI,
    ACTIONS(9567), 1,
      anon_sym_EQ,
    STATE(6219), 1,
      sym_where_clause,
    STATE(4791), 2,
      sym_line_comment,
      sym_block_comment,
  [144956] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4792), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9569), 4,
      anon_sym_SEMI,
      anon_sym_RPAREN,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
  [144973] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9571), 1,
      anon_sym_SEMI,
    ACTIONS(9573), 1,
      anon_sym_EQ,
    STATE(4697), 1,
      sym_where_clause,
    STATE(4793), 2,
      sym_line_comment,
      sym_block_comment,
  [144996] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9575), 1,
      sym_identifier,
    STATE(3788), 1,
      sym_type_arguments,
    STATE(4794), 2,
      sym_line_comment,
      sym_block_comment,
  [145019] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    STATE(4795), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9577), 3,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_GT,
  [145038] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(6163), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4796), 2,
      sym_line_comment,
      sym_block_comment,
  [145061] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1304), 1,
      sym_field_declaration_list,
    STATE(5888), 1,
      sym_where_clause,
    STATE(4797), 2,
      sym_line_comment,
      sym_block_comment,
  [145084] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(6001), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    STATE(2843), 1,
      sym_type_arguments,
    STATE(4798), 2,
      sym_line_comment,
      sym_block_comment,
  [145107] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9076), 1,
      anon_sym_SEMI,
    STATE(1091), 1,
      sym_declaration_list,
    STATE(4799), 2,
      sym_line_comment,
      sym_block_comment,
  [145130] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(6677), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    STATE(3175), 1,
      sym_type_arguments,
    STATE(4800), 2,
      sym_line_comment,
      sym_block_comment,
  [145153] = 5,
    ACTIONS(9557), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(9559), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9579), 1,
      aux_sym_token_repetition_pattern_token1,
    STATE(4801), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9581), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [145172] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8180), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4802), 2,
      sym_line_comment,
      sym_block_comment,
  [145193] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9583), 1,
      anon_sym_RPAREN,
    ACTIONS(9585), 1,
      anon_sym_COMMA,
    STATE(5267), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4803), 2,
      sym_line_comment,
      sym_block_comment,
  [145216] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4804), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1358), 4,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_COLON_COLON,
      anon_sym_if,
  [145233] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9587), 1,
      anon_sym_LBRACE,
    ACTIONS(9589), 1,
      anon_sym_for,
    ACTIONS(9591), 1,
      anon_sym_loop,
    ACTIONS(9593), 1,
      anon_sym_while,
    STATE(4805), 2,
      sym_line_comment,
      sym_block_comment,
  [145256] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7231), 1,
      anon_sym_BANG,
    ACTIONS(7275), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    STATE(4806), 2,
      sym_line_comment,
      sym_block_comment,
  [145279] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9595), 1,
      anon_sym_SEMI,
    STATE(6024), 1,
      sym_where_clause,
    STATE(4807), 2,
      sym_line_comment,
      sym_block_comment,
  [145302] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3964), 1,
      sym_parameters,
    STATE(5940), 1,
      sym_type_parameters,
    STATE(4808), 2,
      sym_line_comment,
      sym_block_comment,
  [145325] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9597), 1,
      anon_sym_SEMI,
    STATE(6134), 1,
      sym_where_clause,
    STATE(4809), 2,
      sym_line_comment,
      sym_block_comment,
  [145348] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7215), 1,
      anon_sym_DOT_DOT,
    ACTIONS(8246), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7217), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_DOT_DOT_EQ,
    STATE(4810), 2,
      sym_line_comment,
      sym_block_comment,
  [145369] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9599), 1,
      anon_sym_SEMI,
    ACTIONS(9601), 1,
      anon_sym_EQ,
    STATE(6029), 1,
      sym_where_clause,
    STATE(4811), 2,
      sym_line_comment,
      sym_block_comment,
  [145392] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9603), 1,
      anon_sym_LBRACE,
    ACTIONS(9605), 1,
      anon_sym_for,
    ACTIONS(9607), 1,
      anon_sym_loop,
    ACTIONS(9609), 1,
      anon_sym_while,
    STATE(4812), 2,
      sym_line_comment,
      sym_block_comment,
  [145415] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8990), 1,
      anon_sym_LPAREN,
    ACTIONS(8992), 1,
      anon_sym_LBRACE,
    STATE(1308), 1,
      sym_macro_rules_v2,
    STATE(5946), 1,
      sym_macro_arguments_declaration,
    STATE(4813), 2,
      sym_line_comment,
      sym_block_comment,
  [145438] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9611), 1,
      anon_sym_SEMI,
    ACTIONS(9613), 1,
      anon_sym_EQ,
    STATE(6031), 1,
      sym_where_clause,
    STATE(4814), 2,
      sym_line_comment,
      sym_block_comment,
  [145461] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9615), 1,
      anon_sym_SEMI,
    ACTIONS(9617), 1,
      anon_sym_EQ,
    STATE(6145), 1,
      sym_where_clause,
    STATE(4815), 2,
      sym_line_comment,
      sym_block_comment,
  [145484] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9619), 1,
      anon_sym_SEMI,
    ACTIONS(9621), 1,
      anon_sym_EQ,
    STATE(6149), 1,
      sym_where_clause,
    STATE(4816), 2,
      sym_line_comment,
      sym_block_comment,
  [145507] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9623), 1,
      anon_sym_SEMI,
    ACTIONS(9625), 1,
      anon_sym_EQ,
    STATE(4776), 1,
      sym_where_clause,
    STATE(4817), 2,
      sym_line_comment,
      sym_block_comment,
  [145530] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9627), 1,
      anon_sym_DQUOTE,
    ACTIONS(9629), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(4818), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_string_literal_repeat1,
  [145549] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9632), 1,
      anon_sym_SEMI,
    ACTIONS(9634), 1,
      anon_sym_EQ,
    STATE(4472), 1,
      sym_where_clause,
    STATE(4819), 2,
      sym_line_comment,
      sym_block_comment,
  [145572] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9636), 1,
      anon_sym_SEMI,
    STATE(6364), 1,
      sym_where_clause,
    STATE(4820), 2,
      sym_line_comment,
      sym_block_comment,
  [145595] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7618), 1,
      anon_sym_for,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4821), 2,
      sym_line_comment,
      sym_block_comment,
  [145618] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2712), 1,
      sym_enum_variant_list,
    STATE(5757), 1,
      sym_where_clause,
    STATE(4822), 2,
      sym_line_comment,
      sym_block_comment,
  [145641] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9638), 1,
      anon_sym_SEMI,
    ACTIONS(9640), 1,
      anon_sym_EQ,
    STATE(6055), 1,
      sym_where_clause,
    STATE(4823), 2,
      sym_line_comment,
      sym_block_comment,
  [145664] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9642), 1,
      anon_sym_COLON,
    STATE(4824), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7699), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [145683] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9644), 1,
      anon_sym_SEMI,
    ACTIONS(9646), 1,
      anon_sym_EQ,
    STATE(4477), 1,
      sym_where_clause,
    STATE(4825), 2,
      sym_line_comment,
      sym_block_comment,
  [145706] = 5,
    ACTIONS(9557), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(9559), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9648), 1,
      aux_sym_token_repetition_pattern_token1,
    STATE(4826), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9650), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [145725] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2722), 1,
      sym_field_declaration_list,
    STATE(5770), 1,
      sym_where_clause,
    STATE(4827), 2,
      sym_line_comment,
      sym_block_comment,
  [145748] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9539), 1,
      anon_sym_SEMI,
    STATE(1153), 1,
      sym_declaration_list,
    STATE(4828), 2,
      sym_line_comment,
      sym_block_comment,
  [145771] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9652), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4829), 2,
      sym_line_comment,
      sym_block_comment,
  [145794] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9654), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4830), 2,
      sym_line_comment,
      sym_block_comment,
  [145817] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7632), 1,
      anon_sym_for,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4831), 2,
      sym_line_comment,
      sym_block_comment,
  [145840] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7203), 2,
      anon_sym_COLON,
      anon_sym_PIPE,
    ACTIONS(9656), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4832), 2,
      sym_line_comment,
      sym_block_comment,
  [145859] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3894), 1,
      sym_parameters,
    STATE(5933), 1,
      sym_type_parameters,
    STATE(4833), 2,
      sym_line_comment,
      sym_block_comment,
  [145882] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9658), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4834), 2,
      sym_line_comment,
      sym_block_comment,
  [145905] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9660), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4835), 2,
      sym_line_comment,
      sym_block_comment,
  [145928] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9662), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4836), 2,
      sym_line_comment,
      sym_block_comment,
  [145951] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9664), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4837), 2,
      sym_line_comment,
      sym_block_comment,
  [145974] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9666), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4838), 2,
      sym_line_comment,
      sym_block_comment,
  [145997] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9575), 1,
      sym_identifier,
    STATE(3789), 1,
      sym_type_arguments,
    STATE(4839), 2,
      sym_line_comment,
      sym_block_comment,
  [146020] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9668), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4840), 2,
      sym_line_comment,
      sym_block_comment,
  [146043] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9670), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4841), 2,
      sym_line_comment,
      sym_block_comment,
  [146066] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9672), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4842), 2,
      sym_line_comment,
      sym_block_comment,
  [146089] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9674), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4843), 2,
      sym_line_comment,
      sym_block_comment,
  [146112] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(8986), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9676), 1,
      anon_sym_for,
    STATE(3555), 1,
      sym_type_arguments,
    STATE(4844), 2,
      sym_line_comment,
      sym_block_comment,
  [146135] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(3897), 1,
      sym_parameters,
    STATE(5529), 1,
      sym_type_parameters,
    STATE(4845), 2,
      sym_line_comment,
      sym_block_comment,
  [146158] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9118), 1,
      sym_identifier,
    ACTIONS(9120), 1,
      sym_super,
    STATE(3677), 1,
      sym_type_arguments,
    STATE(4846), 2,
      sym_line_comment,
      sym_block_comment,
  [146181] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9678), 1,
      anon_sym_SEMI,
    STATE(6074), 1,
      sym_where_clause,
    STATE(4847), 2,
      sym_line_comment,
      sym_block_comment,
  [146204] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9680), 1,
      anon_sym_SEMI,
    ACTIONS(9682), 1,
      anon_sym_EQ,
    STATE(6077), 1,
      sym_where_clause,
    STATE(4848), 2,
      sym_line_comment,
      sym_block_comment,
  [146227] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9070), 1,
      anon_sym_fn,
    ACTIONS(9684), 1,
      anon_sym_SEMI,
    STATE(1318), 1,
      sym_declaration_list,
    STATE(4849), 2,
      sym_line_comment,
      sym_block_comment,
  [146250] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(8942), 1,
      anon_sym_RPAREN,
    ACTIONS(8944), 1,
      anon_sym_COMMA,
    STATE(5060), 1,
      aux_sym_parameters_repeat1,
    STATE(4850), 2,
      sym_line_comment,
      sym_block_comment,
  [146273] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(386), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(5976), 1,
      sym_block,
    STATE(6308), 1,
      sym_label,
    STATE(4851), 2,
      sym_line_comment,
      sym_block_comment,
  [146296] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9406), 1,
      anon_sym_LPAREN,
    ACTIONS(9408), 1,
      anon_sym_LBRACE,
    ACTIONS(9410), 1,
      anon_sym_LBRACK,
    STATE(4792), 1,
      sym_token_tree,
    STATE(4852), 2,
      sym_line_comment,
      sym_block_comment,
  [146319] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9686), 1,
      anon_sym_SEMI,
    ACTIONS(9688), 1,
      anon_sym_EQ,
    STATE(6079), 1,
      sym_where_clause,
    STATE(4853), 2,
      sym_line_comment,
      sym_block_comment,
  [146342] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9690), 1,
      anon_sym_SEMI,
    ACTIONS(9692), 1,
      anon_sym_EQ,
    STATE(4484), 1,
      sym_where_clause,
    STATE(4854), 2,
      sym_line_comment,
      sym_block_comment,
  [146365] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1324), 1,
      sym_enum_variant_list,
    STATE(5501), 1,
      sym_where_clause,
    STATE(4855), 2,
      sym_line_comment,
      sym_block_comment,
  [146388] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9694), 1,
      anon_sym_SEMI,
    STATE(1076), 1,
      sym_declaration_list,
    STATE(4856), 2,
      sym_line_comment,
      sym_block_comment,
  [146411] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8836), 1,
      anon_sym_COMMA,
    ACTIONS(8838), 1,
      anon_sym_COLON,
    ACTIONS(8840), 1,
      anon_sym_PIPE,
    STATE(5002), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(4857), 2,
      sym_line_comment,
      sym_block_comment,
  [146434] = 7,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(5825), 1,
      anon_sym_SQUOTE,
    STATE(483), 1,
      sym_block,
    STATE(6297), 1,
      sym_label,
    STATE(4858), 2,
      sym_line_comment,
      sym_block_comment,
  [146457] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4859), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7648), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [146473] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9428), 1,
      anon_sym_GT,
    ACTIONS(9696), 1,
      anon_sym_COMMA,
    STATE(4908), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4860), 2,
      sym_line_comment,
      sym_block_comment,
  [146493] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9698), 1,
      anon_sym_SEMI,
    STATE(925), 1,
      sym_declaration_list,
    STATE(4861), 2,
      sym_line_comment,
      sym_block_comment,
  [146513] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9700), 1,
      anon_sym_SEMI,
    ACTIONS(9702), 1,
      anon_sym_EQ,
    STATE(4862), 2,
      sym_line_comment,
      sym_block_comment,
  [146533] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9704), 1,
      sym_identifier,
    ACTIONS(9706), 1,
      anon_sym_await,
    ACTIONS(9708), 1,
      sym_integer_literal,
    STATE(4863), 2,
      sym_line_comment,
      sym_block_comment,
  [146553] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9710), 1,
      anon_sym_EQ,
    STATE(2625), 1,
      sym_declaration_list,
    STATE(4864), 2,
      sym_line_comment,
      sym_block_comment,
  [146573] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9712), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4865), 2,
      sym_line_comment,
      sym_block_comment,
  [146593] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9714), 1,
      anon_sym_EQ,
    STATE(2626), 1,
      sym_declaration_list,
    STATE(4866), 2,
      sym_line_comment,
      sym_block_comment,
  [146613] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9716), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4867), 2,
      sym_line_comment,
      sym_block_comment,
  [146631] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9718), 1,
      anon_sym_RBRACE,
    ACTIONS(9720), 1,
      anon_sym_COMMA,
    STATE(5172), 1,
      aux_sym_use_list_repeat1,
    STATE(4868), 2,
      sym_line_comment,
      sym_block_comment,
  [146651] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9722), 1,
      anon_sym_SEMI,
    STATE(910), 1,
      sym_declaration_list,
    STATE(4869), 2,
      sym_line_comment,
      sym_block_comment,
  [146671] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9724), 1,
      anon_sym_RPAREN,
    ACTIONS(9726), 1,
      anon_sym_COMMA,
    STATE(4892), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(4870), 2,
      sym_line_comment,
      sym_block_comment,
  [146691] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7486), 1,
      anon_sym_RBRACE,
    ACTIONS(9728), 1,
      anon_sym_COMMA,
    STATE(5398), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(4871), 2,
      sym_line_comment,
      sym_block_comment,
  [146711] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9730), 1,
      anon_sym_COMMA,
    ACTIONS(9733), 1,
      anon_sym_GT,
    STATE(4872), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_type_parameters_repeat1,
  [146729] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9735), 1,
      anon_sym_COMMA,
    ACTIONS(9737), 1,
      anon_sym_GT,
    STATE(5276), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4873), 2,
      sym_line_comment,
      sym_block_comment,
  [146749] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(9739), 1,
      anon_sym_move,
    STATE(94), 1,
      sym_closure_parameters,
    STATE(4874), 2,
      sym_line_comment,
      sym_block_comment,
  [146769] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(9741), 1,
      anon_sym_move,
    STATE(104), 1,
      sym_closure_parameters,
    STATE(4875), 2,
      sym_line_comment,
      sym_block_comment,
  [146789] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9743), 1,
      anon_sym_RBRACE,
    ACTIONS(9745), 1,
      anon_sym_COMMA,
    STATE(5269), 1,
      aux_sym_macro_rules_v2_repeat1,
    STATE(4876), 2,
      sym_line_comment,
      sym_block_comment,
  [146809] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9747), 1,
      anon_sym_RPAREN,
    ACTIONS(9749), 1,
      anon_sym_COMMA,
    STATE(4892), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(4877), 2,
      sym_line_comment,
      sym_block_comment,
  [146829] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9751), 1,
      anon_sym_SEMI,
    STATE(958), 1,
      sym_declaration_list,
    STATE(4878), 2,
      sym_line_comment,
      sym_block_comment,
  [146849] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(9753), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(4879), 2,
      sym_line_comment,
      sym_block_comment,
  [146869] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9755), 1,
      anon_sym_COMMA,
    ACTIONS(9757), 1,
      anon_sym_GT,
    STATE(4872), 1,
      aux_sym_type_parameters_repeat1,
    STATE(4880), 2,
      sym_line_comment,
      sym_block_comment,
  [146889] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9759), 1,
      anon_sym_SEMI,
    STATE(960), 1,
      sym_declaration_list,
    STATE(4881), 2,
      sym_line_comment,
      sym_block_comment,
  [146909] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9761), 1,
      anon_sym_SEMI,
    STATE(2638), 1,
      sym_declaration_list,
    STATE(4882), 2,
      sym_line_comment,
      sym_block_comment,
  [146929] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4883), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(9763), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [146945] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7596), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    STATE(4884), 2,
      sym_line_comment,
      sym_block_comment,
  [146965] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9765), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4885), 2,
      sym_line_comment,
      sym_block_comment,
  [146983] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9767), 1,
      anon_sym_SEMI,
    ACTIONS(9769), 1,
      anon_sym_EQ,
    STATE(4886), 2,
      sym_line_comment,
      sym_block_comment,
  [147003] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9771), 1,
      anon_sym_SEMI,
    STATE(1227), 1,
      sym_declaration_list,
    STATE(4887), 2,
      sym_line_comment,
      sym_block_comment,
  [147023] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4888), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7701), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [147039] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9773), 1,
      anon_sym_SEMI,
    STATE(2644), 1,
      sym_declaration_list,
    STATE(4889), 2,
      sym_line_comment,
      sym_block_comment,
  [147059] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9775), 1,
      anon_sym_COMMA,
    ACTIONS(9778), 1,
      anon_sym_GT,
    STATE(4890), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_unsafe_bound_type_repeat1,
  [147077] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9780), 1,
      anon_sym_SEMI,
    ACTIONS(9782), 1,
      anon_sym_RBRACK,
    STATE(4891), 2,
      sym_line_comment,
      sym_block_comment,
  [147097] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9784), 1,
      anon_sym_RPAREN,
    ACTIONS(9786), 1,
      anon_sym_COMMA,
    STATE(4892), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_ordered_field_declaration_list_repeat1,
  [147115] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9789), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4893), 2,
      sym_line_comment,
      sym_block_comment,
  [147133] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1450), 1,
      anon_sym_COLON,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(7596), 1,
      anon_sym_COLON_COLON,
    STATE(4894), 2,
      sym_line_comment,
      sym_block_comment,
  [147153] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9791), 1,
      anon_sym_SEMI,
    STATE(1008), 1,
      sym_declaration_list,
    STATE(4895), 2,
      sym_line_comment,
      sym_block_comment,
  [147173] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7434), 1,
      anon_sym_RBRACE,
    ACTIONS(9793), 1,
      anon_sym_COMMA,
    STATE(4903), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(4896), 2,
      sym_line_comment,
      sym_block_comment,
  [147193] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9795), 1,
      anon_sym_SEMI,
    ACTIONS(9797), 1,
      anon_sym_EQ,
    STATE(4897), 2,
      sym_line_comment,
      sym_block_comment,
  [147213] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7468), 1,
      anon_sym_RBRACE,
    ACTIONS(9799), 1,
      anon_sym_COMMA,
    STATE(5058), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(4898), 2,
      sym_line_comment,
      sym_block_comment,
  [147233] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    ACTIONS(7594), 1,
      anon_sym_COLON_COLON,
    STATE(2843), 1,
      sym_type_arguments,
    STATE(4899), 2,
      sym_line_comment,
      sym_block_comment,
  [147253] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7514), 1,
      anon_sym_COLON,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(4900), 2,
      sym_line_comment,
      sym_block_comment,
  [147273] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7468), 1,
      anon_sym_RBRACE,
    ACTIONS(9799), 1,
      anon_sym_COMMA,
    STATE(4903), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(4901), 2,
      sym_line_comment,
      sym_block_comment,
  [147293] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9801), 1,
      anon_sym_EQ,
    STATE(2653), 1,
      sym_declaration_list,
    STATE(4902), 2,
      sym_line_comment,
      sym_block_comment,
  [147313] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9803), 1,
      anon_sym_RBRACE,
    ACTIONS(9805), 1,
      anon_sym_COMMA,
    STATE(4903), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_field_declaration_list_repeat1,
  [147331] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9808), 1,
      anon_sym_SEMI,
    STATE(1010), 1,
      sym_declaration_list,
    STATE(4904), 2,
      sym_line_comment,
      sym_block_comment,
  [147351] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7313), 1,
      anon_sym_RBRACE,
    ACTIONS(9810), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(4905), 2,
      sym_line_comment,
      sym_block_comment,
  [147371] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4906), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7717), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [147387] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9812), 1,
      anon_sym_SEMI,
    STATE(927), 1,
      sym_declaration_list,
    STATE(4907), 2,
      sym_line_comment,
      sym_block_comment,
  [147407] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9814), 1,
      anon_sym_COMMA,
    ACTIONS(9817), 1,
      anon_sym_GT,
    STATE(4908), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_use_bounds_repeat1,
  [147425] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(9819), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(4909), 2,
      sym_line_comment,
      sym_block_comment,
  [147445] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(9821), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4910), 2,
      sym_line_comment,
      sym_block_comment,
  [147463] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9823), 1,
      anon_sym_EQ,
    STATE(2661), 1,
      sym_declaration_list,
    STATE(4911), 2,
      sym_line_comment,
      sym_block_comment,
  [147483] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5409), 1,
      anon_sym_RPAREN,
    ACTIONS(9825), 1,
      anon_sym_COMMA,
    STATE(4968), 1,
      aux_sym_tuple_type_repeat1,
    STATE(4912), 2,
      sym_line_comment,
      sym_block_comment,
  [147503] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9827), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4913), 2,
      sym_line_comment,
      sym_block_comment,
  [147521] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4914), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7570), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [147537] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(899), 1,
      anon_sym_RBRACK,
    ACTIONS(6745), 1,
      anon_sym_COMMA,
    STATE(4922), 1,
      aux_sym_array_expression_repeat1,
    STATE(4915), 2,
      sym_line_comment,
      sym_block_comment,
  [147557] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8819), 1,
      anon_sym_RPAREN,
    ACTIONS(8821), 1,
      anon_sym_COMMA,
    STATE(4936), 1,
      aux_sym_parameters_repeat1,
    STATE(4916), 2,
      sym_line_comment,
      sym_block_comment,
  [147577] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9829), 1,
      anon_sym_SEMI,
    STATE(2665), 1,
      sym_declaration_list,
    STATE(4917), 2,
      sym_line_comment,
      sym_block_comment,
  [147597] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9831), 1,
      anon_sym_SEMI,
    STATE(619), 1,
      sym_declaration_list,
    STATE(4918), 2,
      sym_line_comment,
      sym_block_comment,
  [147617] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9833), 1,
      anon_sym_COMMA,
    ACTIONS(9835), 1,
      anon_sym_GT,
    STATE(4940), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(4919), 2,
      sym_line_comment,
      sym_block_comment,
  [147637] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9837), 1,
      anon_sym_COMMA,
    ACTIONS(9839), 1,
      anon_sym_GT,
    STATE(4942), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4920), 2,
      sym_line_comment,
      sym_block_comment,
  [147657] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9841), 1,
      anon_sym_COMMA,
    ACTIONS(9843), 1,
      anon_sym_GT,
    STATE(4944), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4921), 2,
      sym_line_comment,
      sym_block_comment,
  [147677] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6835), 1,
      anon_sym_RBRACK,
    ACTIONS(9845), 1,
      anon_sym_COMMA,
    STATE(4922), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_array_expression_repeat1,
  [147695] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4923), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7640), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [147711] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4924), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7642), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [147727] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9848), 1,
      anon_sym_SEMI,
    STATE(912), 1,
      sym_declaration_list,
    STATE(4925), 2,
      sym_line_comment,
      sym_block_comment,
  [147747] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9850), 1,
      anon_sym_RBRACE,
    ACTIONS(9852), 1,
      anon_sym_COMMA,
    STATE(4948), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(4926), 2,
      sym_line_comment,
      sym_block_comment,
  [147767] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2147), 1,
      anon_sym_RPAREN,
    ACTIONS(9854), 1,
      anon_sym_COMMA,
    STATE(5266), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(4927), 2,
      sym_line_comment,
      sym_block_comment,
  [147787] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9858), 1,
      anon_sym_COLON,
    ACTIONS(9856), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(4928), 2,
      sym_line_comment,
      sym_block_comment,
  [147805] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9860), 1,
      anon_sym_EQ,
    STATE(2670), 1,
      sym_declaration_list,
    STATE(4929), 2,
      sym_line_comment,
      sym_block_comment,
  [147825] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4930), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7644), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [147841] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1788), 1,
      anon_sym_GT,
    ACTIONS(9862), 1,
      anon_sym_COMMA,
    STATE(4951), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4931), 2,
      sym_line_comment,
      sym_block_comment,
  [147861] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1788), 1,
      anon_sym_GT,
    ACTIONS(9862), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4932), 2,
      sym_line_comment,
      sym_block_comment,
  [147881] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4933), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7646), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [147897] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(856), 1,
      anon_sym_RBRACK,
    ACTIONS(6711), 1,
      anon_sym_COMMA,
    STATE(4922), 1,
      aux_sym_array_expression_repeat1,
    STATE(4934), 2,
      sym_line_comment,
      sym_block_comment,
  [147917] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5242), 1,
      anon_sym_RBRACK,
    ACTIONS(9864), 1,
      anon_sym_COMMA,
    STATE(4762), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4935), 2,
      sym_line_comment,
      sym_block_comment,
  [147937] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1101), 1,
      anon_sym_RPAREN,
    ACTIONS(9866), 1,
      anon_sym_COMMA,
    STATE(5005), 1,
      aux_sym_parameters_repeat1,
    STATE(4936), 2,
      sym_line_comment,
      sym_block_comment,
  [147957] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9868), 1,
      anon_sym_SEMI,
    ACTIONS(9870), 1,
      anon_sym_EQ,
    STATE(4937), 2,
      sym_line_comment,
      sym_block_comment,
  [147977] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(9872), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(4938), 2,
      sym_line_comment,
      sym_block_comment,
  [147997] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4939), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7703), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [148013] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9872), 1,
      anon_sym_GT,
    ACTIONS(9874), 1,
      anon_sym_COMMA,
    STATE(4890), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(4940), 2,
      sym_line_comment,
      sym_block_comment,
  [148033] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9525), 1,
      anon_sym_RPAREN,
    ACTIONS(9527), 1,
      anon_sym_COMMA,
    STATE(4927), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(4941), 2,
      sym_line_comment,
      sym_block_comment,
  [148053] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9228), 1,
      anon_sym_GT,
    ACTIONS(9876), 1,
      anon_sym_COMMA,
    STATE(4908), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4942), 2,
      sym_line_comment,
      sym_block_comment,
  [148073] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4943), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1458), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [148089] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9230), 1,
      anon_sym_GT,
    ACTIONS(9878), 1,
      anon_sym_COMMA,
    STATE(4908), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4944), 2,
      sym_line_comment,
      sym_block_comment,
  [148109] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9880), 1,
      anon_sym_SEMI,
    STATE(2682), 1,
      sym_declaration_list,
    STATE(4945), 2,
      sym_line_comment,
      sym_block_comment,
  [148129] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7456), 1,
      anon_sym_RBRACE,
    ACTIONS(9882), 1,
      anon_sym_COMMA,
    STATE(4958), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(4946), 2,
      sym_line_comment,
      sym_block_comment,
  [148149] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4947), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7586), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [148165] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7456), 1,
      anon_sym_RBRACE,
    ACTIONS(9882), 1,
      anon_sym_COMMA,
    STATE(4903), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(4948), 2,
      sym_line_comment,
      sym_block_comment,
  [148185] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    ACTIONS(9884), 1,
      sym_identifier,
    STATE(4949), 2,
      sym_line_comment,
      sym_block_comment,
  [148205] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9886), 1,
      anon_sym_RBRACE,
    ACTIONS(9888), 1,
      anon_sym_COMMA,
    STATE(5358), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(4950), 2,
      sym_line_comment,
      sym_block_comment,
  [148225] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1776), 1,
      anon_sym_GT,
    ACTIONS(9890), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4951), 2,
      sym_line_comment,
      sym_block_comment,
  [148245] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9892), 1,
      anon_sym_EQ,
    STATE(845), 1,
      sym_declaration_list,
    STATE(4952), 2,
      sym_line_comment,
      sym_block_comment,
  [148265] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4953), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7658), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [148281] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(9894), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(4954), 2,
      sym_line_comment,
      sym_block_comment,
  [148301] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9896), 1,
      anon_sym_SEMI,
    ACTIONS(9898), 1,
      anon_sym_EQ,
    STATE(4955), 2,
      sym_line_comment,
      sym_block_comment,
  [148321] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9900), 1,
      anon_sym_SEMI,
    STATE(6019), 1,
      sym_where_clause,
    STATE(4956), 2,
      sym_line_comment,
      sym_block_comment,
  [148341] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4957), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7660), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [148357] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7466), 1,
      anon_sym_RBRACE,
    ACTIONS(9902), 1,
      anon_sym_COMMA,
    STATE(4903), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(4958), 2,
      sym_line_comment,
      sym_block_comment,
  [148377] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(9904), 1,
      anon_sym_SEMI,
    STATE(2692), 1,
      sym_field_declaration_list,
    STATE(4959), 2,
      sym_line_comment,
      sym_block_comment,
  [148397] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9906), 1,
      anon_sym_EQ,
    STATE(2694), 1,
      sym_declaration_list,
    STATE(4960), 2,
      sym_line_comment,
      sym_block_comment,
  [148417] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(4961), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7666), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [148433] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9908), 1,
      anon_sym_EQ,
    STATE(2695), 1,
      sym_declaration_list,
    STATE(4962), 2,
      sym_line_comment,
      sym_block_comment,
  [148453] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7352), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8466), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4963), 2,
      sym_line_comment,
      sym_block_comment,
  [148471] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9910), 1,
      anon_sym_RBRACE,
    ACTIONS(9912), 1,
      anon_sym_COMMA,
    STATE(5367), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(4964), 2,
      sym_line_comment,
      sym_block_comment,
  [148491] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9914), 1,
      anon_sym_SEMI,
    STATE(1182), 1,
      sym_declaration_list,
    STATE(4965), 2,
      sym_line_comment,
      sym_block_comment,
  [148511] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5433), 1,
      anon_sym_RPAREN,
    ACTIONS(9916), 1,
      anon_sym_COMMA,
    STATE(4968), 1,
      aux_sym_tuple_type_repeat1,
    STATE(4966), 2,
      sym_line_comment,
      sym_block_comment,
  [148531] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9918), 1,
      anon_sym_SEMI,
    STATE(1338), 1,
      sym_declaration_list,
    STATE(4967), 2,
      sym_line_comment,
      sym_block_comment,
  [148551] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9920), 1,
      anon_sym_RPAREN,
    ACTIONS(9922), 1,
      anon_sym_COMMA,
    STATE(4968), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_tuple_type_repeat1,
  [148569] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2149), 1,
      anon_sym_RPAREN,
    ACTIONS(9925), 1,
      anon_sym_COMMA,
    STATE(5266), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(4969), 2,
      sym_line_comment,
      sym_block_comment,
  [148589] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9927), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(4970), 2,
      sym_line_comment,
      sym_block_comment,
  [148607] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9929), 1,
      anon_sym_EQ,
    STATE(1124), 1,
      sym_declaration_list,
    STATE(4971), 2,
      sym_line_comment,
      sym_block_comment,
  [148627] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9931), 1,
      anon_sym_EQ,
    STATE(976), 1,
      sym_declaration_list,
    STATE(4972), 2,
      sym_line_comment,
      sym_block_comment,
  [148647] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9933), 1,
      anon_sym_SEMI,
    STATE(2707), 1,
      sym_declaration_list,
    STATE(4973), 2,
      sym_line_comment,
      sym_block_comment,
  [148667] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8663), 1,
      anon_sym_RPAREN,
    ACTIONS(8665), 1,
      anon_sym_COMMA,
    STATE(5414), 1,
      aux_sym_parameters_repeat1,
    STATE(4974), 2,
      sym_line_comment,
      sym_block_comment,
  [148687] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1790), 1,
      anon_sym_GT,
    ACTIONS(9935), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4975), 2,
      sym_line_comment,
      sym_block_comment,
  [148707] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(9937), 1,
      anon_sym_move,
    STATE(123), 1,
      sym_closure_parameters,
    STATE(4976), 2,
      sym_line_comment,
      sym_block_comment,
  [148727] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9939), 1,
      anon_sym_SEMI,
    STATE(6048), 1,
      sym_where_clause,
    STATE(4977), 2,
      sym_line_comment,
      sym_block_comment,
  [148747] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(9941), 1,
      anon_sym_SEMI,
    STATE(2715), 1,
      sym_field_declaration_list,
    STATE(4978), 2,
      sym_line_comment,
      sym_block_comment,
  [148767] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5264), 1,
      anon_sym_RPAREN,
    ACTIONS(9943), 1,
      anon_sym_COMMA,
    STATE(4762), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(4979), 2,
      sym_line_comment,
      sym_block_comment,
  [148787] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9945), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4980), 2,
      sym_line_comment,
      sym_block_comment,
  [148805] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9947), 1,
      anon_sym_COMMA,
    ACTIONS(9949), 1,
      anon_sym_GT,
    STATE(5437), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(4981), 2,
      sym_line_comment,
      sym_block_comment,
  [148825] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9951), 1,
      anon_sym_EQ,
    STATE(977), 1,
      sym_declaration_list,
    STATE(4982), 2,
      sym_line_comment,
      sym_block_comment,
  [148845] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9953), 1,
      anon_sym_COMMA,
    ACTIONS(9955), 1,
      anon_sym_GT,
    STATE(5440), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4983), 2,
      sym_line_comment,
      sym_block_comment,
  [148865] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9957), 1,
      anon_sym_COMMA,
    ACTIONS(9959), 1,
      anon_sym_GT,
    STATE(4860), 1,
      aux_sym_use_bounds_repeat1,
    STATE(4984), 2,
      sym_line_comment,
      sym_block_comment,
  [148885] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    ACTIONS(7598), 1,
      anon_sym_COLON_COLON,
    STATE(3175), 1,
      sym_type_arguments,
    STATE(4985), 2,
      sym_line_comment,
      sym_block_comment,
  [148905] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9961), 1,
      anon_sym_SEMI,
    ACTIONS(9963), 1,
      anon_sym_RBRACK,
    STATE(4986), 2,
      sym_line_comment,
      sym_block_comment,
  [148925] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9965), 1,
      anon_sym_SEMI,
    ACTIONS(9967), 1,
      anon_sym_EQ,
    STATE(4987), 2,
      sym_line_comment,
      sym_block_comment,
  [148945] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(9969), 1,
      anon_sym_SEMI,
    STATE(6073), 1,
      sym_where_clause,
    STATE(4988), 2,
      sym_line_comment,
      sym_block_comment,
  [148965] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(9971), 1,
      anon_sym_SEMI,
    STATE(2733), 1,
      sym_field_declaration_list,
    STATE(4989), 2,
      sym_line_comment,
      sym_block_comment,
  [148985] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9973), 1,
      anon_sym_SEMI,
    ACTIONS(9975), 1,
      anon_sym_RBRACK,
    STATE(4990), 2,
      sym_line_comment,
      sym_block_comment,
  [149005] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9977), 1,
      anon_sym_EQ,
    STATE(786), 1,
      sym_declaration_list,
    STATE(4991), 2,
      sym_line_comment,
      sym_block_comment,
  [149025] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(9979), 1,
      anon_sym_EQ,
    STATE(2526), 1,
      sym_declaration_list,
    STATE(4992), 2,
      sym_line_comment,
      sym_block_comment,
  [149045] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9656), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(4993), 2,
      sym_line_comment,
      sym_block_comment,
  [149063] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(9981), 1,
      anon_sym_SEMI,
    STATE(929), 1,
      sym_declaration_list,
    STATE(4994), 2,
      sym_line_comment,
      sym_block_comment,
  [149083] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9983), 1,
      anon_sym_SEMI,
    ACTIONS(9985), 1,
      anon_sym_EQ,
    STATE(4995), 2,
      sym_line_comment,
      sym_block_comment,
  [149103] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1784), 1,
      anon_sym_GT,
    ACTIONS(9987), 1,
      anon_sym_COMMA,
    STATE(5000), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4996), 2,
      sym_line_comment,
      sym_block_comment,
  [149123] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1784), 1,
      anon_sym_GT,
    ACTIONS(9987), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(4997), 2,
      sym_line_comment,
      sym_block_comment,
  [149143] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9989), 1,
      anon_sym_RBRACE,
    ACTIONS(9991), 1,
      anon_sym_COMMA,
    STATE(4871), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(4998), 2,
      sym_line_comment,
      sym_block_comment,
  [149163] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9993), 1,
      anon_sym_EQ_GT,
    ACTIONS(9995), 1,
      anon_sym_PIPE,
    ACTIONS(9997), 1,
      anon_sym_if,
    STATE(4999), 2,
      sym_line_comment,
      sym_block_comment,
  [149183] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1794), 1,
      anon_sym_GT,
    ACTIONS(9999), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5000), 2,
      sym_line_comment,
      sym_block_comment,
  [149203] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10001), 1,
      anon_sym_SEMI,
    STATE(621), 1,
      sym_declaration_list,
    STATE(5001), 2,
      sym_line_comment,
      sym_block_comment,
  [149223] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10003), 1,
      anon_sym_COMMA,
    ACTIONS(10005), 1,
      anon_sym_PIPE,
    STATE(5308), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(5002), 2,
      sym_line_comment,
      sym_block_comment,
  [149243] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10007), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5003), 2,
      sym_line_comment,
      sym_block_comment,
  [149261] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10009), 1,
      anon_sym_SEMI,
    STATE(931), 1,
      sym_declaration_list,
    STATE(5004), 2,
      sym_line_comment,
      sym_block_comment,
  [149281] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9656), 1,
      anon_sym_RPAREN,
    ACTIONS(10011), 1,
      anon_sym_COMMA,
    STATE(5005), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_parameters_repeat1,
  [149299] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10014), 1,
      anon_sym_SEMI,
    STATE(1141), 1,
      sym_declaration_list,
    STATE(5006), 2,
      sym_line_comment,
      sym_block_comment,
  [149319] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5007), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2878), 3,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
  [149335] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10016), 1,
      anon_sym_SEMI,
    STATE(1018), 1,
      sym_declaration_list,
    STATE(5008), 2,
      sym_line_comment,
      sym_block_comment,
  [149355] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1778), 1,
      anon_sym_GT,
    ACTIONS(10018), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5009), 2,
      sym_line_comment,
      sym_block_comment,
  [149375] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(10020), 1,
      anon_sym_move,
    STATE(129), 1,
      sym_closure_parameters,
    STATE(5010), 2,
      sym_line_comment,
      sym_block_comment,
  [149395] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10022), 1,
      anon_sym_EQ,
    STATE(846), 1,
      sym_declaration_list,
    STATE(5011), 2,
      sym_line_comment,
      sym_block_comment,
  [149415] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10024), 1,
      anon_sym_EQ,
    STATE(1206), 1,
      sym_declaration_list,
    STATE(5012), 2,
      sym_line_comment,
      sym_block_comment,
  [149435] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(8138), 1,
      anon_sym_COLON,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(5013), 2,
      sym_line_comment,
      sym_block_comment,
  [149455] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7366), 1,
      anon_sym_RBRACE,
    ACTIONS(10026), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5014), 2,
      sym_line_comment,
      sym_block_comment,
  [149475] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5015), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7664), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [149491] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10028), 1,
      anon_sym_COMMA,
    ACTIONS(10030), 1,
      anon_sym_GT,
    STATE(4880), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5016), 2,
      sym_line_comment,
      sym_block_comment,
  [149511] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7374), 1,
      anon_sym_RBRACE,
    ACTIONS(10032), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5017), 2,
      sym_line_comment,
      sym_block_comment,
  [149531] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5018), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7699), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [149547] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(10034), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5019), 2,
      sym_line_comment,
      sym_block_comment,
  [149567] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5020), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1398), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [149583] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10036), 1,
      anon_sym_COMMA,
    ACTIONS(10038), 1,
      anon_sym_GT,
    STATE(4872), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5021), 2,
      sym_line_comment,
      sym_block_comment,
  [149603] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10040), 1,
      anon_sym_EQ,
    STATE(2419), 1,
      sym_declaration_list,
    STATE(5022), 2,
      sym_line_comment,
      sym_block_comment,
  [149623] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10042), 1,
      anon_sym_EQ,
    STATE(658), 1,
      sym_declaration_list,
    STATE(5023), 2,
      sym_line_comment,
      sym_block_comment,
  [149643] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10044), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6919), 2,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(5024), 2,
      sym_line_comment,
      sym_block_comment,
  [149661] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9745), 1,
      anon_sym_COMMA,
    ACTIONS(10046), 1,
      anon_sym_RBRACE,
    STATE(5088), 1,
      aux_sym_macro_rules_v2_repeat1,
    STATE(5025), 2,
      sym_line_comment,
      sym_block_comment,
  [149681] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5026), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7668), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [149697] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10048), 1,
      anon_sym_SEMI,
    STATE(2540), 1,
      sym_declaration_list,
    STATE(5027), 2,
      sym_line_comment,
      sym_block_comment,
  [149717] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5028), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7203), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [149733] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10050), 1,
      anon_sym_EQ,
    STATE(659), 1,
      sym_declaration_list,
    STATE(5029), 2,
      sym_line_comment,
      sym_block_comment,
  [149753] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7372), 1,
      anon_sym_RBRACE,
    ACTIONS(10052), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5030), 2,
      sym_line_comment,
      sym_block_comment,
  [149773] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5031), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4154), 3,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
  [149789] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10054), 1,
      anon_sym_SEMI,
    STATE(1991), 1,
      sym_declaration_list,
    STATE(5032), 2,
      sym_line_comment,
      sym_block_comment,
  [149809] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10056), 1,
      anon_sym_SEMI,
    STATE(1993), 1,
      sym_declaration_list,
    STATE(5033), 2,
      sym_line_comment,
      sym_block_comment,
  [149829] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10058), 1,
      anon_sym_SEMI,
    ACTIONS(10060), 1,
      anon_sym_EQ,
    STATE(5034), 2,
      sym_line_comment,
      sym_block_comment,
  [149849] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10062), 1,
      anon_sym_EQ,
    STATE(1133), 1,
      sym_declaration_list,
    STATE(5035), 2,
      sym_line_comment,
      sym_block_comment,
  [149869] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10064), 1,
      anon_sym_SEMI,
    STATE(1020), 1,
      sym_declaration_list,
    STATE(5036), 2,
      sym_line_comment,
      sym_block_comment,
  [149889] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10066), 1,
      anon_sym_COLON_COLON,
    STATE(3558), 1,
      sym_type_arguments,
    STATE(5037), 2,
      sym_line_comment,
      sym_block_comment,
  [149909] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10068), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5038), 2,
      sym_line_comment,
      sym_block_comment,
  [149929] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10070), 1,
      anon_sym_SEMI,
    ACTIONS(10072), 1,
      anon_sym_EQ,
    STATE(5039), 2,
      sym_line_comment,
      sym_block_comment,
  [149949] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5040), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(8830), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [149965] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10074), 1,
      anon_sym_SEMI,
    STATE(1022), 1,
      sym_declaration_list,
    STATE(5041), 2,
      sym_line_comment,
      sym_block_comment,
  [149985] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10076), 1,
      anon_sym_EQ,
    STATE(864), 1,
      sym_declaration_list,
    STATE(5042), 2,
      sym_line_comment,
      sym_block_comment,
  [150005] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10078), 1,
      sym_identifier,
    STATE(4833), 1,
      sym_metavariable,
    STATE(5043), 2,
      sym_line_comment,
      sym_block_comment,
  [150025] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5044), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10080), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [150041] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10044), 1,
      anon_sym_AMP_AMP,
    ACTIONS(10082), 2,
      anon_sym_LBRACE,
      anon_sym_SQUOTE,
    STATE(5045), 2,
      sym_line_comment,
      sym_block_comment,
  [150059] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10084), 1,
      anon_sym_EQ,
    STATE(610), 1,
      sym_declaration_list,
    STATE(5046), 2,
      sym_line_comment,
      sym_block_comment,
  [150079] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10088), 1,
      anon_sym_COLON,
    ACTIONS(10086), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5047), 2,
      sym_line_comment,
      sym_block_comment,
  [150097] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10090), 1,
      anon_sym_SEMI,
    STATE(2003), 1,
      sym_declaration_list,
    STATE(5048), 2,
      sym_line_comment,
      sym_block_comment,
  [150117] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10092), 1,
      anon_sym_RBRACE,
    ACTIONS(10094), 1,
      anon_sym_COMMA,
    STATE(5343), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(5049), 2,
      sym_line_comment,
      sym_block_comment,
  [150137] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10096), 1,
      anon_sym_SEMI,
    STATE(1024), 1,
      sym_declaration_list,
    STATE(5050), 2,
      sym_line_comment,
      sym_block_comment,
  [150157] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10098), 1,
      anon_sym_SEMI,
    STATE(2005), 1,
      sym_declaration_list,
    STATE(5051), 2,
      sym_line_comment,
      sym_block_comment,
  [150177] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10100), 1,
      anon_sym_SEMI,
    ACTIONS(10102), 1,
      anon_sym_EQ,
    STATE(5052), 2,
      sym_line_comment,
      sym_block_comment,
  [150197] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10104), 1,
      anon_sym_RBRACE,
    ACTIONS(10106), 1,
      anon_sym_COMMA,
    STATE(4901), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(5053), 2,
      sym_line_comment,
      sym_block_comment,
  [150217] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10108), 1,
      anon_sym_SEMI,
    ACTIONS(10110), 1,
      anon_sym_EQ,
    STATE(5054), 2,
      sym_line_comment,
      sym_block_comment,
  [150237] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10112), 1,
      anon_sym_RPAREN,
    ACTIONS(10114), 1,
      anon_sym_COMMA,
    STATE(4892), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(5055), 2,
      sym_line_comment,
      sym_block_comment,
  [150257] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10116), 1,
      anon_sym_SEMI,
    STATE(5959), 1,
      sym_where_clause,
    STATE(5056), 2,
      sym_line_comment,
      sym_block_comment,
  [150277] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10118), 1,
      sym_identifier,
    ACTIONS(10120), 1,
      anon_sym_ref,
    ACTIONS(10122), 1,
      sym_mutable_specifier,
    STATE(5057), 2,
      sym_line_comment,
      sym_block_comment,
  [150297] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7464), 1,
      anon_sym_RBRACE,
    ACTIONS(10124), 1,
      anon_sym_COMMA,
    STATE(4903), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(5058), 2,
      sym_line_comment,
      sym_block_comment,
  [150317] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10126), 1,
      sym_identifier,
    STATE(4465), 1,
      sym_metavariable,
    STATE(5059), 2,
      sym_line_comment,
      sym_block_comment,
  [150337] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1093), 1,
      anon_sym_RPAREN,
    ACTIONS(10128), 1,
      anon_sym_COMMA,
    STATE(5005), 1,
      aux_sym_parameters_repeat1,
    STATE(5060), 2,
      sym_line_comment,
      sym_block_comment,
  [150357] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10130), 1,
      anon_sym_SEMI,
    STATE(5967), 1,
      sym_where_clause,
    STATE(5061), 2,
      sym_line_comment,
      sym_block_comment,
  [150377] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10132), 1,
      anon_sym_SEMI,
    STATE(1197), 1,
      sym_field_declaration_list,
    STATE(5062), 2,
      sym_line_comment,
      sym_block_comment,
  [150397] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(10134), 1,
      anon_sym_SEMI,
    STATE(2550), 1,
      sym_field_declaration_list,
    STATE(5063), 2,
      sym_line_comment,
      sym_block_comment,
  [150417] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10136), 1,
      anon_sym_SEMI,
    STATE(6440), 1,
      sym_where_clause,
    STATE(5064), 2,
      sym_line_comment,
      sym_block_comment,
  [150437] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5065), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1442), 3,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_GT,
  [150453] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10138), 1,
      anon_sym_SEMI,
    STATE(1250), 1,
      sym_field_declaration_list,
    STATE(5066), 2,
      sym_line_comment,
      sym_block_comment,
  [150473] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6003), 1,
      anon_sym_LBRACE,
    ACTIONS(10140), 1,
      anon_sym_COLON_COLON,
    STATE(2893), 1,
      sym_field_initializer_list,
    STATE(5067), 2,
      sym_line_comment,
      sym_block_comment,
  [150493] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10142), 1,
      anon_sym_EQ,
    STATE(668), 1,
      sym_declaration_list,
    STATE(5068), 2,
      sym_line_comment,
      sym_block_comment,
  [150513] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10144), 1,
      anon_sym_EQ,
    STATE(2552), 1,
      sym_declaration_list,
    STATE(5069), 2,
      sym_line_comment,
      sym_block_comment,
  [150533] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10146), 1,
      anon_sym_EQ,
    STATE(2553), 1,
      sym_declaration_list,
    STATE(5070), 2,
      sym_line_comment,
      sym_block_comment,
  [150553] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10148), 1,
      anon_sym_LPAREN,
    ACTIONS(10150), 1,
      anon_sym_LBRACE,
    ACTIONS(10152), 1,
      anon_sym_COLON_COLON,
    STATE(5071), 2,
      sym_line_comment,
      sym_block_comment,
  [150573] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10154), 1,
      anon_sym_EQ,
    STATE(2022), 1,
      sym_declaration_list,
    STATE(5072), 2,
      sym_line_comment,
      sym_block_comment,
  [150593] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10150), 1,
      anon_sym_LBRACE,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5073), 2,
      sym_line_comment,
      sym_block_comment,
  [150613] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10156), 1,
      anon_sym_EQ,
    STATE(2023), 1,
      sym_declaration_list,
    STATE(5074), 2,
      sym_line_comment,
      sym_block_comment,
  [150633] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10158), 1,
      anon_sym_EQ,
    STATE(669), 1,
      sym_declaration_list,
    STATE(5075), 2,
      sym_line_comment,
      sym_block_comment,
  [150653] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(5997), 1,
      anon_sym_PLUS,
    ACTIONS(6001), 1,
      anon_sym_COLON_COLON,
    STATE(5076), 2,
      sym_line_comment,
      sym_block_comment,
  [150673] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(903), 1,
      anon_sym_RBRACK,
    ACTIONS(10160), 1,
      anon_sym_COMMA,
    STATE(4922), 1,
      aux_sym_array_expression_repeat1,
    STATE(5077), 2,
      sym_line_comment,
      sym_block_comment,
  [150693] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5078), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10162), 3,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_fn,
  [150709] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10164), 1,
      anon_sym_SEMI,
    STATE(2441), 1,
      sym_declaration_list,
    STATE(5079), 2,
      sym_line_comment,
      sym_block_comment,
  [150729] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10166), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5080), 2,
      sym_line_comment,
      sym_block_comment,
  [150749] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10168), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(10170), 1,
      sym__inner_block_doc_comment_marker,
    ACTIONS(10172), 1,
      sym__block_comment_content,
    STATE(5081), 2,
      sym_line_comment,
      sym_block_comment,
  [150769] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8615), 1,
      anon_sym_GT,
    ACTIONS(10174), 1,
      anon_sym_COMMA,
    STATE(5082), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_type_arguments_repeat1,
  [150787] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10177), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5083), 2,
      sym_line_comment,
      sym_block_comment,
  [150807] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10179), 1,
      sym_identifier,
    STATE(4485), 1,
      sym_metavariable,
    STATE(5084), 2,
      sym_line_comment,
      sym_block_comment,
  [150827] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5240), 1,
      anon_sym_RBRACK,
    ACTIONS(10181), 1,
      anon_sym_COMMA,
    STATE(4762), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(5085), 2,
      sym_line_comment,
      sym_block_comment,
  [150847] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10183), 1,
      anon_sym_EQ,
    STATE(1199), 1,
      sym_declaration_list,
    STATE(5086), 2,
      sym_line_comment,
      sym_block_comment,
  [150867] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10185), 1,
      anon_sym_EQ,
    STATE(2032), 1,
      sym_declaration_list,
    STATE(5087), 2,
      sym_line_comment,
      sym_block_comment,
  [150887] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9745), 1,
      anon_sym_COMMA,
    ACTIONS(10187), 1,
      anon_sym_RBRACE,
    STATE(5377), 1,
      aux_sym_macro_rules_v2_repeat1,
    STATE(5088), 2,
      sym_line_comment,
      sym_block_comment,
  [150907] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10189), 1,
      anon_sym_EQ,
    STATE(1255), 1,
      sym_declaration_list,
    STATE(5089), 2,
      sym_line_comment,
      sym_block_comment,
  [150927] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10191), 1,
      sym_identifier,
    STATE(4497), 1,
      sym_metavariable,
    STATE(5090), 2,
      sym_line_comment,
      sym_block_comment,
  [150947] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10193), 1,
      anon_sym_EQ,
    STATE(1200), 1,
      sym_declaration_list,
    STATE(5091), 2,
      sym_line_comment,
      sym_block_comment,
  [150967] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10195), 1,
      anon_sym_EQ,
    STATE(2033), 1,
      sym_declaration_list,
    STATE(5092), 2,
      sym_line_comment,
      sym_block_comment,
  [150987] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5093), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7670), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [151003] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(10197), 1,
      anon_sym_move,
    STATE(112), 1,
      sym_closure_parameters,
    STATE(5094), 2,
      sym_line_comment,
      sym_block_comment,
  [151023] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10199), 1,
      anon_sym_SEMI,
    STATE(827), 1,
      sym_declaration_list,
    STATE(5095), 2,
      sym_line_comment,
      sym_block_comment,
  [151043] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5096), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7694), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [151059] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10201), 1,
      anon_sym_LPAREN,
    ACTIONS(10203), 1,
      anon_sym_LBRACE,
    ACTIONS(10205), 1,
      anon_sym_LBRACK,
    STATE(5097), 2,
      sym_line_comment,
      sym_block_comment,
  [151079] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10207), 1,
      anon_sym_LPAREN,
    ACTIONS(10209), 1,
      anon_sym_LBRACE,
    ACTIONS(10211), 1,
      anon_sym_LBRACK,
    STATE(5098), 2,
      sym_line_comment,
      sym_block_comment,
  [151099] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5099), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7711), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [151115] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5100), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7713), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [151131] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5101), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7719), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [151147] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5102), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7721), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [151163] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5103), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10213), 3,
      sym_string_content,
      anon_sym_DQUOTE,
      sym_escape_sequence,
  [151179] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10215), 1,
      anon_sym_LPAREN,
    ACTIONS(10217), 1,
      anon_sym_LBRACE,
    ACTIONS(10219), 1,
      anon_sym_LBRACK,
    STATE(5104), 2,
      sym_line_comment,
      sym_block_comment,
  [151199] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10221), 1,
      anon_sym_EQ,
    STATE(1257), 1,
      sym_declaration_list,
    STATE(5105), 2,
      sym_line_comment,
      sym_block_comment,
  [151219] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10223), 1,
      anon_sym_LPAREN,
    ACTIONS(10225), 1,
      anon_sym_LBRACE,
    ACTIONS(10227), 1,
      anon_sym_LBRACK,
    STATE(5106), 2,
      sym_line_comment,
      sym_block_comment,
  [151239] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10229), 1,
      anon_sym_EQ,
    STATE(992), 1,
      sym_declaration_list,
    STATE(5107), 2,
      sym_line_comment,
      sym_block_comment,
  [151259] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10231), 1,
      anon_sym_SEMI,
    STATE(680), 1,
      sym_declaration_list,
    STATE(5108), 2,
      sym_line_comment,
      sym_block_comment,
  [151279] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10233), 1,
      anon_sym_SEMI,
    STATE(2044), 1,
      sym_declaration_list,
    STATE(5109), 2,
      sym_line_comment,
      sym_block_comment,
  [151299] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10235), 1,
      anon_sym_SEMI,
    ACTIONS(10237), 1,
      anon_sym_EQ,
    STATE(5110), 2,
      sym_line_comment,
      sym_block_comment,
  [151319] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10239), 1,
      anon_sym_COMMA,
    ACTIONS(10241), 1,
      anon_sym_GT,
    STATE(4872), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5111), 2,
      sym_line_comment,
      sym_block_comment,
  [151339] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10243), 1,
      anon_sym_SEMI,
    STATE(2050), 1,
      sym_declaration_list,
    STATE(5112), 2,
      sym_line_comment,
      sym_block_comment,
  [151359] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10245), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5113), 2,
      sym_line_comment,
      sym_block_comment,
  [151379] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10247), 1,
      anon_sym_SEMI,
    STATE(1067), 1,
      sym_declaration_list,
    STATE(5114), 2,
      sym_line_comment,
      sym_block_comment,
  [151399] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10249), 1,
      sym_identifier,
    STATE(4552), 1,
      sym_metavariable,
    STATE(5115), 2,
      sym_line_comment,
      sym_block_comment,
  [151419] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10251), 1,
      anon_sym_SEMI,
    STATE(1137), 1,
      sym_declaration_list,
    STATE(5116), 2,
      sym_line_comment,
      sym_block_comment,
  [151439] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10253), 1,
      anon_sym_SEMI,
    ACTIONS(10255), 1,
      anon_sym_EQ,
    STATE(5117), 2,
      sym_line_comment,
      sym_block_comment,
  [151459] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10257), 1,
      anon_sym_EQ,
    STATE(2059), 1,
      sym_declaration_list,
    STATE(5118), 2,
      sym_line_comment,
      sym_block_comment,
  [151479] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10259), 1,
      sym_identifier,
    STATE(4639), 1,
      sym_metavariable,
    STATE(5119), 2,
      sym_line_comment,
      sym_block_comment,
  [151499] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10261), 1,
      anon_sym_SEMI,
    STATE(686), 1,
      sym_declaration_list,
    STATE(5120), 2,
      sym_line_comment,
      sym_block_comment,
  [151519] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10263), 1,
      anon_sym_SEMI,
    STATE(2565), 1,
      sym_declaration_list,
    STATE(5121), 2,
      sym_line_comment,
      sym_block_comment,
  [151539] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5122), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7715), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [151555] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7408), 1,
      anon_sym_RBRACE,
    ACTIONS(10265), 1,
      anon_sym_COMMA,
    STATE(5166), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(5123), 2,
      sym_line_comment,
      sym_block_comment,
  [151575] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10267), 1,
      anon_sym_SEMI,
    ACTIONS(10269), 1,
      anon_sym_EQ,
    STATE(5124), 2,
      sym_line_comment,
      sym_block_comment,
  [151595] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10271), 1,
      anon_sym_EQ,
    STATE(2067), 1,
      sym_declaration_list,
    STATE(5125), 2,
      sym_line_comment,
      sym_block_comment,
  [151615] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7410), 1,
      anon_sym_RBRACE,
    ACTIONS(10273), 1,
      anon_sym_COMMA,
    STATE(5166), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(5126), 2,
      sym_line_comment,
      sym_block_comment,
  [151635] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(10275), 1,
      anon_sym_GT,
    ACTIONS(10277), 1,
      anon_sym_as,
    STATE(5127), 2,
      sym_line_comment,
      sym_block_comment,
  [151655] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10279), 1,
      anon_sym_SEMI,
    STATE(2071), 1,
      sym_declaration_list,
    STATE(5128), 2,
      sym_line_comment,
      sym_block_comment,
  [151675] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10281), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5129), 2,
      sym_line_comment,
      sym_block_comment,
  [151695] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10283), 1,
      anon_sym_COMMA,
    ACTIONS(10285), 1,
      anon_sym_GT,
    STATE(5021), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5130), 2,
      sym_line_comment,
      sym_block_comment,
  [151715] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10287), 1,
      anon_sym_SEMI,
    ACTIONS(10289), 1,
      anon_sym_EQ,
    STATE(5131), 2,
      sym_line_comment,
      sym_block_comment,
  [151735] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10291), 1,
      anon_sym_EQ,
    STATE(1037), 1,
      sym_declaration_list,
    STATE(5132), 2,
      sym_line_comment,
      sym_block_comment,
  [151755] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10293), 1,
      anon_sym_SEMI,
    ACTIONS(10295), 1,
      anon_sym_EQ,
    STATE(5133), 2,
      sym_line_comment,
      sym_block_comment,
  [151775] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10297), 1,
      anon_sym_SEMI,
    STATE(6214), 1,
      sym_where_clause,
    STATE(5134), 2,
      sym_line_comment,
      sym_block_comment,
  [151795] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10299), 1,
      anon_sym_COMMA,
    ACTIONS(10301), 1,
      anon_sym_GT,
    STATE(5170), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(5135), 2,
      sym_line_comment,
      sym_block_comment,
  [151815] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(10303), 1,
      anon_sym_SEMI,
    STATE(2082), 1,
      sym_field_declaration_list,
    STATE(5136), 2,
      sym_line_comment,
      sym_block_comment,
  [151835] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10305), 1,
      sym_identifier,
    ACTIONS(10307), 1,
      anon_sym_await,
    ACTIONS(10309), 1,
      sym_integer_literal,
    STATE(5137), 2,
      sym_line_comment,
      sym_block_comment,
  [151855] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10311), 1,
      anon_sym_SEMI,
    STATE(1159), 1,
      sym_declaration_list,
    STATE(5138), 2,
      sym_line_comment,
      sym_block_comment,
  [151875] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10313), 1,
      anon_sym_SEMI,
    STATE(2575), 1,
      sym_declaration_list,
    STATE(5139), 2,
      sym_line_comment,
      sym_block_comment,
  [151895] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10315), 1,
      anon_sym_SEMI,
    STATE(2473), 1,
      sym_declaration_list,
    STATE(5140), 2,
      sym_line_comment,
      sym_block_comment,
  [151915] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10317), 1,
      anon_sym_SEMI,
    ACTIONS(10319), 1,
      anon_sym_EQ,
    STATE(5141), 2,
      sym_line_comment,
      sym_block_comment,
  [151935] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5142), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10321), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [151951] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5143), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10323), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [151967] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10325), 1,
      anon_sym_SEMI,
    STATE(6091), 1,
      sym_where_clause,
    STATE(5144), 2,
      sym_line_comment,
      sym_block_comment,
  [151987] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10327), 1,
      anon_sym_RBRACE,
    ACTIONS(10329), 1,
      anon_sym_COMMA,
    STATE(5145), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_use_list_repeat1,
  [152005] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(10332), 1,
      anon_sym_SEMI,
    STATE(2579), 1,
      sym_field_declaration_list,
    STATE(5146), 2,
      sym_line_comment,
      sym_block_comment,
  [152025] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10334), 1,
      anon_sym_SEMI,
    STATE(906), 1,
      sym_declaration_list,
    STATE(5147), 2,
      sym_line_comment,
      sym_block_comment,
  [152045] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10336), 1,
      anon_sym_EQ,
    STATE(2582), 1,
      sym_declaration_list,
    STATE(5148), 2,
      sym_line_comment,
      sym_block_comment,
  [152065] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10338), 1,
      anon_sym_RBRACE,
    ACTIONS(10340), 1,
      anon_sym_COMMA,
    STATE(5208), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(5149), 2,
      sym_line_comment,
      sym_block_comment,
  [152085] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10342), 1,
      anon_sym_EQ,
    STATE(2104), 1,
      sym_declaration_list,
    STATE(5150), 2,
      sym_line_comment,
      sym_block_comment,
  [152105] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10344), 1,
      anon_sym_LPAREN,
    ACTIONS(10346), 1,
      anon_sym_LBRACE,
    ACTIONS(10348), 1,
      anon_sym_LBRACK,
    STATE(5151), 2,
      sym_line_comment,
      sym_block_comment,
  [152125] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10350), 1,
      anon_sym_LPAREN,
    ACTIONS(10352), 1,
      anon_sym_LBRACE,
    ACTIONS(10354), 1,
      anon_sym_LBRACK,
    STATE(5152), 2,
      sym_line_comment,
      sym_block_comment,
  [152145] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10356), 1,
      anon_sym_SEMI,
    STATE(796), 1,
      sym_declaration_list,
    STATE(5153), 2,
      sym_line_comment,
      sym_block_comment,
  [152165] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10358), 1,
      sym_identifier,
    ACTIONS(10360), 1,
      sym_mutable_specifier,
    ACTIONS(10362), 1,
      sym_integer_literal,
    STATE(5154), 2,
      sym_line_comment,
      sym_block_comment,
  [152185] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5155), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10364), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152201] = 6,
    ACTIONS(9557), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(9559), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10366), 1,
      aux_sym_line_comment_token1,
    ACTIONS(10368), 1,
      anon_sym_BANG2,
    STATE(6110), 1,
      sym__inner_line_doc_comment_marker,
    STATE(5156), 2,
      sym_line_comment,
      sym_block_comment,
  [152221] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10370), 1,
      anon_sym_SEMI,
    STATE(1276), 1,
      sym_declaration_list,
    STATE(5157), 2,
      sym_line_comment,
      sym_block_comment,
  [152241] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10372), 1,
      anon_sym_EQ,
    STATE(697), 1,
      sym_declaration_list,
    STATE(5158), 2,
      sym_line_comment,
      sym_block_comment,
  [152261] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10374), 1,
      anon_sym_SEMI,
    STATE(6167), 1,
      sym_where_clause,
    STATE(5159), 2,
      sym_line_comment,
      sym_block_comment,
  [152281] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5160), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4718), 3,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
  [152297] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6003), 1,
      anon_sym_LBRACE,
    ACTIONS(10376), 1,
      anon_sym_COLON_COLON,
    STATE(2893), 1,
      sym_field_initializer_list,
    STATE(5161), 2,
      sym_line_comment,
      sym_block_comment,
  [152317] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(10378), 1,
      anon_sym_SEMI,
    STATE(2478), 1,
      sym_field_declaration_list,
    STATE(5162), 2,
      sym_line_comment,
      sym_block_comment,
  [152337] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10380), 1,
      anon_sym_SEMI,
    STATE(631), 1,
      sym_declaration_list,
    STATE(5163), 2,
      sym_line_comment,
      sym_block_comment,
  [152357] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10382), 1,
      anon_sym_SEMI,
    STATE(633), 1,
      sym_declaration_list,
    STATE(5164), 2,
      sym_line_comment,
      sym_block_comment,
  [152377] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5165), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10384), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152393] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10386), 1,
      anon_sym_RBRACE,
    ACTIONS(10388), 1,
      anon_sym_COMMA,
    STATE(5166), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_field_initializer_list_repeat1,
  [152411] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10391), 1,
      anon_sym_SEMI,
    ACTIONS(10393), 1,
      anon_sym_EQ,
    STATE(5167), 2,
      sym_line_comment,
      sym_block_comment,
  [152431] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5168), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10395), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152447] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(10397), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5169), 2,
      sym_line_comment,
      sym_block_comment,
  [152467] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10397), 1,
      anon_sym_GT,
    ACTIONS(10399), 1,
      anon_sym_COMMA,
    STATE(4890), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(5170), 2,
      sym_line_comment,
      sym_block_comment,
  [152487] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10401), 1,
      anon_sym_SEMI,
    STATE(2120), 1,
      sym_declaration_list,
    STATE(5171), 2,
      sym_line_comment,
      sym_block_comment,
  [152507] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6483), 1,
      anon_sym_RBRACE,
    ACTIONS(10403), 1,
      anon_sym_COMMA,
    STATE(5145), 1,
      aux_sym_use_list_repeat1,
    STATE(5172), 2,
      sym_line_comment,
      sym_block_comment,
  [152527] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10405), 1,
      anon_sym_SEMI,
    STATE(2122), 1,
      sym_declaration_list,
    STATE(5173), 2,
      sym_line_comment,
      sym_block_comment,
  [152547] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5174), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10407), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152563] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5175), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10409), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152579] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5176), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10411), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152595] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5177), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10413), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152611] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10415), 1,
      anon_sym_SEMI,
    STATE(2124), 1,
      sym_declaration_list,
    STATE(5178), 2,
      sym_line_comment,
      sym_block_comment,
  [152631] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10417), 1,
      anon_sym_SEMI,
    STATE(2126), 1,
      sym_declaration_list,
    STATE(5179), 2,
      sym_line_comment,
      sym_block_comment,
  [152651] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5180), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1482), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [152667] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5181), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10419), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152683] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5182), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10421), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [152699] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5183), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7705), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [152715] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10423), 1,
      anon_sym_EQ,
    STATE(705), 1,
      sym_declaration_list,
    STATE(5184), 2,
      sym_line_comment,
      sym_block_comment,
  [152735] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7309), 1,
      anon_sym_RBRACE,
    ACTIONS(10425), 1,
      anon_sym_COMMA,
    STATE(5030), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5185), 2,
      sym_line_comment,
      sym_block_comment,
  [152755] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10429), 1,
      anon_sym_COLON,
    ACTIONS(10427), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5186), 2,
      sym_line_comment,
      sym_block_comment,
  [152773] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10431), 1,
      anon_sym_SEMI,
    STATE(2511), 1,
      sym_declaration_list,
    STATE(5187), 2,
      sym_line_comment,
      sym_block_comment,
  [152793] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10433), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5188), 2,
      sym_line_comment,
      sym_block_comment,
  [152813] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10435), 1,
      anon_sym_EQ,
    STATE(2481), 1,
      sym_declaration_list,
    STATE(5189), 2,
      sym_line_comment,
      sym_block_comment,
  [152833] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10437), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5190), 2,
      sym_line_comment,
      sym_block_comment,
  [152853] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10439), 1,
      anon_sym_SEMI,
    STATE(1226), 1,
      sym_declaration_list,
    STATE(5191), 2,
      sym_line_comment,
      sym_block_comment,
  [152873] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10441), 1,
      anon_sym_EQ,
    STATE(2143), 1,
      sym_declaration_list,
    STATE(5192), 2,
      sym_line_comment,
      sym_block_comment,
  [152893] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6629), 1,
      anon_sym_LBRACE,
    ACTIONS(10443), 1,
      anon_sym_COLON_COLON,
    STATE(3263), 1,
      sym_field_initializer_list,
    STATE(5193), 2,
      sym_line_comment,
      sym_block_comment,
  [152913] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10445), 1,
      anon_sym_SEMI,
    STATE(798), 1,
      sym_declaration_list,
    STATE(5194), 2,
      sym_line_comment,
      sym_block_comment,
  [152933] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7850), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5195), 2,
      sym_line_comment,
      sym_block_comment,
  [152953] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10447), 1,
      anon_sym_SEMI,
    STATE(709), 1,
      sym_declaration_list,
    STATE(5196), 2,
      sym_line_comment,
      sym_block_comment,
  [152973] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5197), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(2354), 3,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
  [152989] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7416), 1,
      anon_sym_RBRACE,
    ACTIONS(10449), 1,
      anon_sym_COMMA,
    STATE(5166), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(5198), 2,
      sym_line_comment,
      sym_block_comment,
  [153009] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10451), 1,
      anon_sym_RBRACE,
    ACTIONS(10453), 1,
      anon_sym_COMMA,
    STATE(5203), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5199), 2,
      sym_line_comment,
      sym_block_comment,
  [153029] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1011), 1,
      anon_sym_RBRACK,
    ACTIONS(6691), 1,
      anon_sym_COMMA,
    STATE(4922), 1,
      aux_sym_array_expression_repeat1,
    STATE(5200), 2,
      sym_line_comment,
      sym_block_comment,
  [153049] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10455), 1,
      anon_sym_SEMI,
    ACTIONS(10457), 1,
      anon_sym_EQ,
    STATE(5201), 2,
      sym_line_comment,
      sym_block_comment,
  [153069] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10459), 1,
      sym_identifier,
    STATE(4564), 1,
      sym_metavariable,
    STATE(5202), 2,
      sym_line_comment,
      sym_block_comment,
  [153089] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7309), 1,
      anon_sym_RBRACE,
    ACTIONS(10425), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5203), 2,
      sym_line_comment,
      sym_block_comment,
  [153109] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10463), 1,
      anon_sym_COLON,
    ACTIONS(10461), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5204), 2,
      sym_line_comment,
      sym_block_comment,
  [153127] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10465), 1,
      sym_identifier,
    ACTIONS(10467), 1,
      anon_sym_ref,
    ACTIONS(10469), 1,
      sym_mutable_specifier,
    STATE(5205), 2,
      sym_line_comment,
      sym_block_comment,
  [153147] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10471), 1,
      anon_sym_SEMI,
    ACTIONS(10473), 1,
      anon_sym_EQ,
    STATE(5206), 2,
      sym_line_comment,
      sym_block_comment,
  [153167] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10475), 1,
      anon_sym_SEMI,
    STATE(2153), 1,
      sym_declaration_list,
    STATE(5207), 2,
      sym_line_comment,
      sym_block_comment,
  [153187] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7406), 1,
      anon_sym_RBRACE,
    ACTIONS(10477), 1,
      anon_sym_COMMA,
    STATE(5166), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(5208), 2,
      sym_line_comment,
      sym_block_comment,
  [153207] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8484), 1,
      anon_sym_COMMA,
    ACTIONS(8486), 1,
      anon_sym_GT,
    STATE(5339), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5209), 2,
      sym_line_comment,
      sym_block_comment,
  [153227] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10479), 1,
      anon_sym_SEMI,
    STATE(2155), 1,
      sym_declaration_list,
    STATE(5210), 2,
      sym_line_comment,
      sym_block_comment,
  [153247] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(10481), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5211), 2,
      sym_line_comment,
      sym_block_comment,
  [153267] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10483), 1,
      anon_sym_LPAREN,
    ACTIONS(10485), 1,
      anon_sym_LBRACE,
    ACTIONS(10487), 1,
      anon_sym_LBRACK,
    STATE(5212), 2,
      sym_line_comment,
      sym_block_comment,
  [153287] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10489), 1,
      anon_sym_LPAREN,
    ACTIONS(10491), 1,
      anon_sym_LBRACE,
    ACTIONS(10493), 1,
      anon_sym_LBRACK,
    STATE(5213), 2,
      sym_line_comment,
      sym_block_comment,
  [153307] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10495), 1,
      anon_sym_SEMI,
    ACTIONS(10497), 1,
      anon_sym_EQ,
    STATE(5214), 2,
      sym_line_comment,
      sym_block_comment,
  [153327] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10499), 1,
      anon_sym_EQ,
    STATE(885), 1,
      sym_declaration_list,
    STATE(5215), 2,
      sym_line_comment,
      sym_block_comment,
  [153347] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10501), 1,
      anon_sym_SEMI,
    STATE(6473), 1,
      sym_where_clause,
    STATE(5216), 2,
      sym_line_comment,
      sym_block_comment,
  [153367] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10503), 1,
      anon_sym_SEMI,
    STATE(721), 1,
      sym_field_declaration_list,
    STATE(5217), 2,
      sym_line_comment,
      sym_block_comment,
  [153387] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10505), 1,
      anon_sym_EQ,
    STATE(2162), 1,
      sym_declaration_list,
    STATE(5218), 2,
      sym_line_comment,
      sym_block_comment,
  [153407] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10507), 1,
      anon_sym_SEMI,
    STATE(2596), 1,
      sym_declaration_list,
    STATE(5219), 2,
      sym_line_comment,
      sym_block_comment,
  [153427] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10509), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5220), 2,
      sym_line_comment,
      sym_block_comment,
  [153447] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10511), 1,
      anon_sym_EQ,
    STATE(1107), 1,
      sym_declaration_list,
    STATE(5221), 2,
      sym_line_comment,
      sym_block_comment,
  [153467] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10513), 1,
      anon_sym_SEMI,
    STATE(2169), 1,
      sym_declaration_list,
    STATE(5222), 2,
      sym_line_comment,
      sym_block_comment,
  [153487] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10515), 1,
      anon_sym_SEMI,
    STATE(2171), 1,
      sym_declaration_list,
    STATE(5223), 2,
      sym_line_comment,
      sym_block_comment,
  [153507] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10517), 1,
      anon_sym_RBRACE,
    ACTIONS(10519), 1,
      anon_sym_COMMA,
    STATE(5224), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_struct_pattern_repeat1,
  [153525] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(10522), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5225), 2,
      sym_line_comment,
      sym_block_comment,
  [153545] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10524), 1,
      anon_sym_SEMI,
    ACTIONS(10526), 1,
      anon_sym_EQ,
    STATE(5226), 2,
      sym_line_comment,
      sym_block_comment,
  [153565] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5227), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7707), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [153581] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10528), 1,
      anon_sym_EQ,
    STATE(886), 1,
      sym_declaration_list,
    STATE(5228), 2,
      sym_line_comment,
      sym_block_comment,
  [153601] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10530), 1,
      sym_identifier,
    ACTIONS(10532), 1,
      anon_sym_ref,
    ACTIONS(10534), 1,
      sym_mutable_specifier,
    STATE(5229), 2,
      sym_line_comment,
      sym_block_comment,
  [153621] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10522), 1,
      anon_sym_GT,
    ACTIONS(10536), 1,
      anon_sym_COMMA,
    STATE(4890), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(5230), 2,
      sym_line_comment,
      sym_block_comment,
  [153641] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10538), 1,
      anon_sym_SEMI,
    STATE(6062), 1,
      sym_where_clause,
    STATE(5231), 2,
      sym_line_comment,
      sym_block_comment,
  [153661] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10540), 1,
      anon_sym_SEMI,
    STATE(2181), 1,
      sym_declaration_list,
    STATE(5232), 2,
      sym_line_comment,
      sym_block_comment,
  [153681] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10542), 1,
      anon_sym_SEMI,
    STATE(1243), 1,
      sym_field_declaration_list,
    STATE(5233), 2,
      sym_line_comment,
      sym_block_comment,
  [153701] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7832), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5234), 2,
      sym_line_comment,
      sym_block_comment,
  [153721] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10544), 1,
      anon_sym_SEMI,
    STATE(2183), 1,
      sym_declaration_list,
    STATE(5235), 2,
      sym_line_comment,
      sym_block_comment,
  [153741] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10548), 1,
      anon_sym_EQ,
    ACTIONS(10546), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5236), 2,
      sym_line_comment,
      sym_block_comment,
  [153759] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10550), 1,
      anon_sym_SEMI,
    ACTIONS(10552), 1,
      anon_sym_EQ,
    STATE(5237), 2,
      sym_line_comment,
      sym_block_comment,
  [153779] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10554), 1,
      anon_sym_SEMI,
    STATE(6205), 1,
      sym_where_clause,
    STATE(5238), 2,
      sym_line_comment,
      sym_block_comment,
  [153799] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10556), 1,
      sym_identifier,
    STATE(4808), 1,
      sym_metavariable,
    STATE(5239), 2,
      sym_line_comment,
      sym_block_comment,
  [153819] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10558), 1,
      anon_sym_SEMI,
    STATE(6277), 1,
      sym_where_clause,
    STATE(5240), 2,
      sym_line_comment,
      sym_block_comment,
  [153839] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    ACTIONS(10560), 1,
      anon_sym_SEMI,
    STATE(2611), 1,
      sym_field_declaration_list,
    STATE(5241), 2,
      sym_line_comment,
      sym_block_comment,
  [153859] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10562), 1,
      anon_sym_RBRACE,
    ACTIONS(10564), 1,
      anon_sym_COMMA,
    STATE(5126), 1,
      aux_sym_field_initializer_list_repeat1,
    STATE(5242), 2,
      sym_line_comment,
      sym_block_comment,
  [153879] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10566), 1,
      anon_sym_EQ,
    STATE(2200), 1,
      sym_declaration_list,
    STATE(5243), 2,
      sym_line_comment,
      sym_block_comment,
  [153899] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9486), 1,
      anon_sym_GT,
    ACTIONS(10568), 1,
      anon_sym_COMMA,
    STATE(4908), 1,
      aux_sym_use_bounds_repeat1,
    STATE(5244), 2,
      sym_line_comment,
      sym_block_comment,
  [153919] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10570), 1,
      anon_sym_EQ,
    STATE(2201), 1,
      sym_declaration_list,
    STATE(5245), 2,
      sym_line_comment,
      sym_block_comment,
  [153939] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10572), 1,
      anon_sym_SEMI,
    STATE(825), 1,
      sym_declaration_list,
    STATE(5246), 2,
      sym_line_comment,
      sym_block_comment,
  [153959] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(10574), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5247), 2,
      sym_line_comment,
      sym_block_comment,
  [153977] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5248), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7678), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [153993] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5249), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7688), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154009] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5250), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7638), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154025] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5251), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7723), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154041] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5252), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7690), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154057] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5253), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7692), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154073] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10576), 1,
      anon_sym_SEMI,
    STATE(1289), 1,
      sym_declaration_list,
    STATE(5254), 2,
      sym_line_comment,
      sym_block_comment,
  [154093] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5255), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1410), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154109] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10580), 1,
      anon_sym_COLON,
    ACTIONS(10578), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5256), 2,
      sym_line_comment,
      sym_block_comment,
  [154127] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10582), 1,
      anon_sym_SEMI,
    STATE(1002), 1,
      sym_declaration_list,
    STATE(5257), 2,
      sym_line_comment,
      sym_block_comment,
  [154147] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(901), 1,
      anon_sym_RPAREN,
    ACTIONS(10584), 1,
      anon_sym_COMMA,
    STATE(5357), 1,
      aux_sym_arguments_repeat1,
    STATE(5258), 2,
      sym_line_comment,
      sym_block_comment,
  [154167] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10586), 1,
      anon_sym_SEMI,
    STATE(2498), 1,
      sym_declaration_list,
    STATE(5259), 2,
      sym_line_comment,
      sym_block_comment,
  [154187] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5260), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7654), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154203] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10588), 1,
      sym_identifier,
    STATE(4494), 1,
      sym_metavariable,
    STATE(5261), 2,
      sym_line_comment,
      sym_block_comment,
  [154223] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10590), 1,
      anon_sym_SEMI,
    STATE(1089), 1,
      sym_declaration_list,
    STATE(5262), 2,
      sym_line_comment,
      sym_block_comment,
  [154243] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10592), 1,
      anon_sym_RPAREN,
    ACTIONS(10594), 1,
      anon_sym_COMMA,
    STATE(4892), 1,
      aux_sym_ordered_field_declaration_list_repeat1,
    STATE(5263), 2,
      sym_line_comment,
      sym_block_comment,
  [154263] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10596), 1,
      anon_sym_EQ,
    STATE(2219), 1,
      sym_declaration_list,
    STATE(5264), 2,
      sym_line_comment,
      sym_block_comment,
  [154283] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8836), 1,
      anon_sym_COMMA,
    ACTIONS(10598), 1,
      anon_sym_PIPE,
    STATE(5002), 1,
      aux_sym_closure_parameters_repeat1,
    STATE(5265), 2,
      sym_line_comment,
      sym_block_comment,
  [154303] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10574), 1,
      anon_sym_RPAREN,
    ACTIONS(10600), 1,
      anon_sym_COMMA,
    STATE(5266), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_tuple_pattern_repeat1,
  [154321] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5214), 1,
      anon_sym_RPAREN,
    ACTIONS(10603), 1,
      anon_sym_COMMA,
    STATE(4762), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(5267), 2,
      sym_line_comment,
      sym_block_comment,
  [154341] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10605), 1,
      anon_sym_SEMI,
    STATE(1328), 1,
      sym_field_declaration_list,
    STATE(5268), 2,
      sym_line_comment,
      sym_block_comment,
  [154361] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9745), 1,
      anon_sym_COMMA,
    ACTIONS(10607), 1,
      anon_sym_RBRACE,
    STATE(5377), 1,
      aux_sym_macro_rules_v2_repeat1,
    STATE(5269), 2,
      sym_line_comment,
      sym_block_comment,
  [154381] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10609), 1,
      anon_sym_EQ,
    STATE(743), 1,
      sym_declaration_list,
    STATE(5270), 2,
      sym_line_comment,
      sym_block_comment,
  [154401] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5271), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7656), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154417] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5272), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7662), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [154433] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10611), 1,
      anon_sym_COMMA,
    ACTIONS(10613), 1,
      anon_sym_GT,
    STATE(5111), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5273), 2,
      sym_line_comment,
      sym_block_comment,
  [154453] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10615), 1,
      sym_identifier,
    ACTIONS(10617), 1,
      anon_sym_ref,
    ACTIONS(10619), 1,
      sym_mutable_specifier,
    STATE(5274), 2,
      sym_line_comment,
      sym_block_comment,
  [154473] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10621), 1,
      anon_sym_SEMI,
    STATE(1073), 1,
      sym_declaration_list,
    STATE(5275), 2,
      sym_line_comment,
      sym_block_comment,
  [154493] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9529), 1,
      anon_sym_GT,
    ACTIONS(10623), 1,
      anon_sym_COMMA,
    STATE(4908), 1,
      aux_sym_use_bounds_repeat1,
    STATE(5276), 2,
      sym_line_comment,
      sym_block_comment,
  [154513] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7161), 1,
      anon_sym_trait,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    STATE(5277), 2,
      sym_line_comment,
      sym_block_comment,
  [154533] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10625), 1,
      anon_sym_SEMI,
    STATE(6083), 1,
      sym_where_clause,
    STATE(5278), 2,
      sym_line_comment,
      sym_block_comment,
  [154553] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10627), 1,
      anon_sym_EQ,
    STATE(2240), 1,
      sym_declaration_list,
    STATE(5279), 2,
      sym_line_comment,
      sym_block_comment,
  [154573] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10629), 1,
      anon_sym_SEMI,
    STATE(1294), 1,
      sym_field_declaration_list,
    STATE(5280), 2,
      sym_line_comment,
      sym_block_comment,
  [154593] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10631), 1,
      anon_sym_EQ,
    STATE(2241), 1,
      sym_declaration_list,
    STATE(5281), 2,
      sym_line_comment,
      sym_block_comment,
  [154613] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7484), 1,
      anon_sym_RBRACE,
    ACTIONS(10633), 1,
      anon_sym_COMMA,
    STATE(4896), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(5282), 2,
      sym_line_comment,
      sym_block_comment,
  [154633] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10635), 1,
      anon_sym_EQ,
    STATE(1297), 1,
      sym_declaration_list,
    STATE(5283), 2,
      sym_line_comment,
      sym_block_comment,
  [154653] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5284), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10637), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [154669] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1780), 1,
      anon_sym_GT,
    ACTIONS(10639), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5285), 2,
      sym_line_comment,
      sym_block_comment,
  [154689] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10641), 1,
      anon_sym_SEMI,
    STATE(1004), 1,
      sym_declaration_list,
    STATE(5286), 2,
      sym_line_comment,
      sym_block_comment,
  [154709] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10643), 1,
      anon_sym_SEMI,
    STATE(2249), 1,
      sym_declaration_list,
    STATE(5287), 2,
      sym_line_comment,
      sym_block_comment,
  [154729] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5288), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(4174), 3,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_EQ,
  [154745] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10645), 1,
      anon_sym_SEMI,
    STATE(2251), 1,
      sym_declaration_list,
    STATE(5289), 2,
      sym_line_comment,
      sym_block_comment,
  [154765] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10647), 1,
      anon_sym_SEMI,
    STATE(1053), 1,
      sym_declaration_list,
    STATE(5290), 2,
      sym_line_comment,
      sym_block_comment,
  [154785] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10649), 1,
      anon_sym_EQ,
    STATE(1152), 1,
      sym_declaration_list,
    STATE(5291), 2,
      sym_line_comment,
      sym_block_comment,
  [154805] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10651), 1,
      anon_sym_SEMI,
    STATE(2259), 1,
      sym_declaration_list,
    STATE(5292), 2,
      sym_line_comment,
      sym_block_comment,
  [154825] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10653), 1,
      anon_sym_SEMI,
    STATE(2261), 1,
      sym_declaration_list,
    STATE(5293), 2,
      sym_line_comment,
      sym_block_comment,
  [154845] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10655), 1,
      anon_sym_SEMI,
    STATE(2263), 1,
      sym_declaration_list,
    STATE(5294), 2,
      sym_line_comment,
      sym_block_comment,
  [154865] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10657), 1,
      anon_sym_SEMI,
    STATE(2265), 1,
      sym_declaration_list,
    STATE(5295), 2,
      sym_line_comment,
      sym_block_comment,
  [154885] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10659), 1,
      anon_sym_SEMI,
    STATE(908), 1,
      sym_declaration_list,
    STATE(5296), 2,
      sym_line_comment,
      sym_block_comment,
  [154905] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7321), 1,
      anon_sym_RBRACE,
    ACTIONS(10661), 1,
      anon_sym_COMMA,
    STATE(4905), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5297), 2,
      sym_line_comment,
      sym_block_comment,
  [154925] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1051), 1,
      anon_sym_RBRACK,
    ACTIONS(10663), 1,
      anon_sym_COMMA,
    STATE(4922), 1,
      aux_sym_array_expression_repeat1,
    STATE(5298), 2,
      sym_line_comment,
      sym_block_comment,
  [154945] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8838), 1,
      anon_sym_COLON,
    ACTIONS(10665), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(5299), 2,
      sym_line_comment,
      sym_block_comment,
  [154963] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7676), 1,
      anon_sym_LBRACE,
    ACTIONS(8444), 1,
      anon_sym_LPAREN,
    ACTIONS(10667), 1,
      anon_sym_COLON_COLON,
    STATE(5300), 2,
      sym_line_comment,
      sym_block_comment,
  [154983] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(10669), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5301), 2,
      sym_line_comment,
      sym_block_comment,
  [155001] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(10671), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5302), 2,
      sym_line_comment,
      sym_block_comment,
  [155019] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10673), 1,
      anon_sym_SEMI,
    STATE(1055), 1,
      sym_declaration_list,
    STATE(5303), 2,
      sym_line_comment,
      sym_block_comment,
  [155039] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10675), 1,
      anon_sym_SEMI,
    STATE(2278), 1,
      sym_declaration_list,
    STATE(5304), 2,
      sym_line_comment,
      sym_block_comment,
  [155059] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10677), 1,
      anon_sym_SEMI,
    STATE(2280), 1,
      sym_declaration_list,
    STATE(5305), 2,
      sym_line_comment,
      sym_block_comment,
  [155079] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10679), 1,
      anon_sym_SEMI,
    STATE(2282), 1,
      sym_declaration_list,
    STATE(5306), 2,
      sym_line_comment,
      sym_block_comment,
  [155099] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10681), 1,
      anon_sym_SEMI,
    ACTIONS(10683), 1,
      anon_sym_EQ,
    STATE(5307), 2,
      sym_line_comment,
      sym_block_comment,
  [155119] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10665), 1,
      anon_sym_PIPE,
    ACTIONS(10685), 1,
      anon_sym_COMMA,
    STATE(5308), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_closure_parameters_repeat1,
  [155137] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7352), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10688), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5309), 2,
      sym_line_comment,
      sym_block_comment,
  [155155] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5310), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10690), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [155171] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5311), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10692), 3,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_QMARK,
  [155187] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10694), 1,
      anon_sym_SEMI,
    STATE(2284), 1,
      sym_declaration_list,
    STATE(5312), 2,
      sym_line_comment,
      sym_block_comment,
  [155207] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5313), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7590), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [155223] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10696), 1,
      sym_identifier,
    STATE(4813), 1,
      sym_metavariable,
    STATE(5314), 2,
      sym_line_comment,
      sym_block_comment,
  [155243] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10698), 1,
      anon_sym_SEMI,
    ACTIONS(10700), 1,
      anon_sym_EQ,
    STATE(5315), 2,
      sym_line_comment,
      sym_block_comment,
  [155263] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(5997), 1,
      anon_sym_PLUS,
    ACTIONS(6677), 1,
      anon_sym_COLON_COLON,
    STATE(5316), 2,
      sym_line_comment,
      sym_block_comment,
  [155283] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10704), 1,
      anon_sym_EQ,
    ACTIONS(10702), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5317), 2,
      sym_line_comment,
      sym_block_comment,
  [155301] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10706), 1,
      anon_sym_EQ,
    STATE(2301), 1,
      sym_declaration_list,
    STATE(5318), 2,
      sym_line_comment,
      sym_block_comment,
  [155321] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7321), 1,
      anon_sym_RBRACE,
    ACTIONS(10661), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5319), 2,
      sym_line_comment,
      sym_block_comment,
  [155341] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10708), 1,
      anon_sym_SEMI,
    STATE(6122), 1,
      sym_where_clause,
    STATE(5320), 2,
      sym_line_comment,
      sym_block_comment,
  [155361] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10710), 1,
      anon_sym_SEMI,
    STATE(1341), 1,
      sym_field_declaration_list,
    STATE(5321), 2,
      sym_line_comment,
      sym_block_comment,
  [155381] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10712), 1,
      anon_sym_COMMA,
    ACTIONS(10714), 1,
      anon_sym_GT,
    STATE(5325), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(5322), 2,
      sym_line_comment,
      sym_block_comment,
  [155401] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5323), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(10716), 3,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [155417] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(10718), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5324), 2,
      sym_line_comment,
      sym_block_comment,
  [155437] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10718), 1,
      anon_sym_GT,
    ACTIONS(10720), 1,
      anon_sym_COMMA,
    STATE(4890), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(5325), 2,
      sym_line_comment,
      sym_block_comment,
  [155457] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(8970), 1,
      sym_identifier,
    STATE(5883), 1,
      sym_lifetime,
    STATE(5326), 2,
      sym_line_comment,
      sym_block_comment,
  [155477] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(10722), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5327), 2,
      sym_line_comment,
      sym_block_comment,
  [155497] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(9920), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5328), 2,
      sym_line_comment,
      sym_block_comment,
  [155515] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5329), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7672), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [155531] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10724), 1,
      anon_sym_SEMI,
    STATE(2311), 1,
      sym_declaration_list,
    STATE(5330), 2,
      sym_line_comment,
      sym_block_comment,
  [155551] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10726), 1,
      anon_sym_SEMI,
    STATE(2313), 1,
      sym_declaration_list,
    STATE(5331), 2,
      sym_line_comment,
      sym_block_comment,
  [155571] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10728), 1,
      anon_sym_SEMI,
    ACTIONS(10730), 1,
      anon_sym_EQ,
    STATE(5332), 2,
      sym_line_comment,
      sym_block_comment,
  [155591] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1782), 1,
      anon_sym_GT,
    ACTIONS(10732), 1,
      anon_sym_COMMA,
    STATE(5009), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5333), 2,
      sym_line_comment,
      sym_block_comment,
  [155611] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1782), 1,
      anon_sym_GT,
    ACTIONS(10732), 1,
      anon_sym_COMMA,
    STATE(5082), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5334), 2,
      sym_line_comment,
      sym_block_comment,
  [155631] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10734), 1,
      anon_sym_SEMI,
    STATE(1311), 1,
      sym_declaration_list,
    STATE(5335), 2,
      sym_line_comment,
      sym_block_comment,
  [155651] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10736), 1,
      anon_sym_RBRACE,
    ACTIONS(10738), 1,
      anon_sym_COMMA,
    STATE(5319), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5336), 2,
      sym_line_comment,
      sym_block_comment,
  [155671] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10740), 1,
      sym_identifier,
    STATE(4674), 1,
      sym_metavariable,
    STATE(5337), 2,
      sym_line_comment,
      sym_block_comment,
  [155691] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(10742), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5338), 2,
      sym_line_comment,
      sym_block_comment,
  [155709] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10744), 1,
      anon_sym_COMMA,
    ACTIONS(10746), 1,
      anon_sym_GT,
    STATE(4872), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5339), 2,
      sym_line_comment,
      sym_block_comment,
  [155729] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(10748), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5340), 2,
      sym_line_comment,
      sym_block_comment,
  [155747] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10750), 1,
      anon_sym_EQ,
    STATE(806), 1,
      sym_declaration_list,
    STATE(5341), 2,
      sym_line_comment,
      sym_block_comment,
  [155767] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5342), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(7674), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [155783] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7484), 1,
      anon_sym_RBRACE,
    ACTIONS(10633), 1,
      anon_sym_COMMA,
    STATE(4903), 1,
      aux_sym_field_declaration_list_repeat1,
    STATE(5343), 2,
      sym_line_comment,
      sym_block_comment,
  [155803] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5248), 1,
      anon_sym_RPAREN,
    ACTIONS(10752), 1,
      anon_sym_COMMA,
    STATE(4762), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(5344), 2,
      sym_line_comment,
      sym_block_comment,
  [155823] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10754), 1,
      sym_identifier,
    STATE(4689), 1,
      sym_metavariable,
    STATE(5345), 2,
      sym_line_comment,
      sym_block_comment,
  [155843] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10756), 1,
      anon_sym_EQ,
    STATE(2328), 1,
      sym_declaration_list,
    STATE(5346), 2,
      sym_line_comment,
      sym_block_comment,
  [155863] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10758), 1,
      sym_identifier,
    STATE(4693), 1,
      sym_metavariable,
    STATE(5347), 2,
      sym_line_comment,
      sym_block_comment,
  [155883] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5439), 1,
      anon_sym_RPAREN,
    ACTIONS(10760), 1,
      anon_sym_COMMA,
    STATE(4968), 1,
      aux_sym_tuple_type_repeat1,
    STATE(5348), 2,
      sym_line_comment,
      sym_block_comment,
  [155903] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10762), 1,
      anon_sym_EQ,
    STATE(2329), 1,
      sym_declaration_list,
    STATE(5349), 2,
      sym_line_comment,
      sym_block_comment,
  [155923] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10766), 1,
      anon_sym_EQ,
    ACTIONS(10764), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5350), 2,
      sym_line_comment,
      sym_block_comment,
  [155941] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10768), 1,
      sym_identifier,
    ACTIONS(10770), 1,
      anon_sym_ref,
    ACTIONS(10772), 1,
      sym_mutable_specifier,
    STATE(5351), 2,
      sym_line_comment,
      sym_block_comment,
  [155961] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7305), 1,
      anon_sym_RBRACE,
    ACTIONS(10774), 1,
      anon_sym_COMMA,
    STATE(5014), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5352), 2,
      sym_line_comment,
      sym_block_comment,
  [155981] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7852), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5353), 2,
      sym_line_comment,
      sym_block_comment,
  [156001] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10776), 1,
      anon_sym_EQ,
    STATE(605), 1,
      sym_declaration_list,
    STATE(5354), 2,
      sym_line_comment,
      sym_block_comment,
  [156021] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(121), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10778), 1,
      sym_identifier,
    STATE(4457), 1,
      sym_metavariable,
    STATE(5355), 2,
      sym_line_comment,
      sym_block_comment,
  [156041] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10780), 1,
      anon_sym_EQ,
    STATE(606), 1,
      sym_declaration_list,
    STATE(5356), 2,
      sym_line_comment,
      sym_block_comment,
  [156061] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6899), 1,
      anon_sym_RPAREN,
    ACTIONS(10782), 1,
      anon_sym_COMMA,
    STATE(5357), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_arguments_repeat1,
  [156079] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7305), 1,
      anon_sym_RBRACE,
    ACTIONS(10774), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5358), 2,
      sym_line_comment,
      sym_block_comment,
  [156099] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1067), 1,
      anon_sym_RBRACK,
    ACTIONS(6737), 1,
      anon_sym_COMMA,
    STATE(4922), 1,
      aux_sym_array_expression_repeat1,
    STATE(5359), 2,
      sym_line_comment,
      sym_block_comment,
  [156119] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5252), 1,
      anon_sym_RPAREN,
    ACTIONS(10785), 1,
      anon_sym_COMMA,
    STATE(4762), 1,
      aux_sym_slice_pattern_repeat1,
    STATE(5360), 2,
      sym_line_comment,
      sym_block_comment,
  [156139] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7364), 1,
      anon_sym_RBRACE,
    ACTIONS(10787), 1,
      anon_sym_COMMA,
    STATE(5017), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5361), 2,
      sym_line_comment,
      sym_block_comment,
  [156159] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10789), 1,
      anon_sym_EQ,
    STATE(2344), 1,
      sym_declaration_list,
    STATE(5362), 2,
      sym_line_comment,
      sym_block_comment,
  [156179] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10791), 1,
      sym_identifier,
    STATE(4719), 1,
      sym_metavariable,
    STATE(5363), 2,
      sym_line_comment,
      sym_block_comment,
  [156199] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10793), 1,
      anon_sym_SEMI,
    ACTIONS(10795), 1,
      anon_sym_EQ,
    STATE(5364), 2,
      sym_line_comment,
      sym_block_comment,
  [156219] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10797), 1,
      anon_sym_COMMA,
    ACTIONS(10799), 1,
      anon_sym_GT,
    STATE(5230), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(5365), 2,
      sym_line_comment,
      sym_block_comment,
  [156239] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10801), 1,
      sym_identifier,
    STATE(4720), 1,
      sym_metavariable,
    STATE(5366), 2,
      sym_line_comment,
      sym_block_comment,
  [156259] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7364), 1,
      anon_sym_RBRACE,
    ACTIONS(10787), 1,
      anon_sym_COMMA,
    STATE(5224), 1,
      aux_sym_struct_pattern_repeat1,
    STATE(5367), 2,
      sym_line_comment,
      sym_block_comment,
  [156279] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10803), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5368), 2,
      sym_line_comment,
      sym_block_comment,
  [156299] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10805), 1,
      anon_sym_SEMI,
    STATE(2353), 1,
      sym_declaration_list,
    STATE(5369), 2,
      sym_line_comment,
      sym_block_comment,
  [156319] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10807), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5370), 2,
      sym_line_comment,
      sym_block_comment,
  [156339] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10811), 1,
      anon_sym_EQ,
    ACTIONS(10809), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5371), 2,
      sym_line_comment,
      sym_block_comment,
  [156357] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5372), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1554), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [156373] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10813), 1,
      anon_sym_SEMI,
    STATE(813), 1,
      sym_declaration_list,
    STATE(5373), 2,
      sym_line_comment,
      sym_block_comment,
  [156393] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10815), 1,
      anon_sym_SEMI,
    STATE(2355), 1,
      sym_declaration_list,
    STATE(5374), 2,
      sym_line_comment,
      sym_block_comment,
  [156413] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7868), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5375), 2,
      sym_line_comment,
      sym_block_comment,
  [156433] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10817), 1,
      anon_sym_SEMI,
    STATE(2359), 1,
      sym_declaration_list,
    STATE(5376), 2,
      sym_line_comment,
      sym_block_comment,
  [156453] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10819), 1,
      anon_sym_RBRACE,
    ACTIONS(10821), 1,
      anon_sym_COMMA,
    STATE(5377), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_macro_rules_v2_repeat1,
  [156471] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10824), 1,
      anon_sym_SEMI,
    STATE(896), 1,
      sym_declaration_list,
    STATE(5378), 2,
      sym_line_comment,
      sym_block_comment,
  [156491] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10826), 1,
      anon_sym_SEMI,
    STATE(2361), 1,
      sym_declaration_list,
    STATE(5379), 2,
      sym_line_comment,
      sym_block_comment,
  [156511] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5380), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1538), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [156527] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7478), 1,
      anon_sym_RBRACE,
    ACTIONS(10828), 1,
      anon_sym_COMMA,
    STATE(5398), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(5381), 2,
      sym_line_comment,
      sym_block_comment,
  [156547] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10830), 1,
      sym_identifier,
    STATE(4740), 1,
      sym_metavariable,
    STATE(5382), 2,
      sym_line_comment,
      sym_block_comment,
  [156567] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10832), 1,
      anon_sym_COMMA,
    ACTIONS(10834), 1,
      anon_sym_GT,
    STATE(4872), 1,
      aux_sym_type_parameters_repeat1,
    STATE(5383), 2,
      sym_line_comment,
      sym_block_comment,
  [156587] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10836), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5384), 2,
      sym_line_comment,
      sym_block_comment,
  [156607] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10838), 1,
      anon_sym_SEMI,
    STATE(2369), 1,
      sym_declaration_list,
    STATE(5385), 2,
      sym_line_comment,
      sym_block_comment,
  [156627] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10840), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5386), 2,
      sym_line_comment,
      sym_block_comment,
  [156647] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10842), 1,
      anon_sym_SEMI,
    STATE(2371), 1,
      sym_declaration_list,
    STATE(5387), 2,
      sym_line_comment,
      sym_block_comment,
  [156667] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10844), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5388), 2,
      sym_line_comment,
      sym_block_comment,
  [156687] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10846), 1,
      anon_sym_SEMI,
    STATE(2373), 1,
      sym_declaration_list,
    STATE(5389), 2,
      sym_line_comment,
      sym_block_comment,
  [156707] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10848), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5390), 2,
      sym_line_comment,
      sym_block_comment,
  [156727] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10850), 1,
      anon_sym_SEMI,
    STATE(2375), 1,
      sym_declaration_list,
    STATE(5391), 2,
      sym_line_comment,
      sym_block_comment,
  [156747] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10852), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5392), 2,
      sym_line_comment,
      sym_block_comment,
  [156767] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10854), 1,
      anon_sym_SEMI,
    STATE(898), 1,
      sym_declaration_list,
    STATE(5393), 2,
      sym_line_comment,
      sym_block_comment,
  [156787] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5394), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1606), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [156803] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10856), 1,
      sym_identifier,
    STATE(4486), 1,
      sym_metavariable,
    STATE(5395), 2,
      sym_line_comment,
      sym_block_comment,
  [156823] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(7556), 1,
      anon_sym_move,
    STATE(120), 1,
      sym_closure_parameters,
    STATE(5396), 2,
      sym_line_comment,
      sym_block_comment,
  [156843] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10858), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5397), 2,
      sym_line_comment,
      sym_block_comment,
  [156863] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10860), 1,
      anon_sym_RBRACE,
    ACTIONS(10862), 1,
      anon_sym_COMMA,
    STATE(5398), 3,
      sym_line_comment,
      sym_block_comment,
      aux_sym_enum_variant_list_repeat1,
  [156881] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10865), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5399), 2,
      sym_line_comment,
      sym_block_comment,
  [156901] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10867), 1,
      anon_sym_EQ,
    STATE(948), 1,
      sym_declaration_list,
    STATE(5400), 2,
      sym_line_comment,
      sym_block_comment,
  [156921] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10869), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5401), 2,
      sym_line_comment,
      sym_block_comment,
  [156941] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10871), 1,
      anon_sym_EQ,
    STATE(2389), 1,
      sym_declaration_list,
    STATE(5402), 2,
      sym_line_comment,
      sym_block_comment,
  [156961] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10873), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5403), 2,
      sym_line_comment,
      sym_block_comment,
  [156981] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10875), 1,
      anon_sym_SEMI,
    ACTIONS(10877), 1,
      anon_sym_EQ,
    STATE(5404), 2,
      sym_line_comment,
      sym_block_comment,
  [157001] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10879), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5405), 2,
      sym_line_comment,
      sym_block_comment,
  [157021] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9224), 1,
      anon_sym_RPAREN,
    ACTIONS(9226), 1,
      anon_sym_COMMA,
    STATE(4969), 1,
      aux_sym_tuple_pattern_repeat1,
    STATE(5406), 2,
      sym_line_comment,
      sym_block_comment,
  [157041] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10881), 1,
      anon_sym_RBRACE,
    ACTIONS(10883), 1,
      anon_sym_COMMA,
    STATE(5381), 1,
      aux_sym_enum_variant_list_repeat1,
    STATE(5407), 2,
      sym_line_comment,
      sym_block_comment,
  [157061] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10885), 1,
      anon_sym_COMMA,
    ACTIONS(10887), 1,
      anon_sym_GT,
    STATE(5244), 1,
      aux_sym_use_bounds_repeat1,
    STATE(5408), 2,
      sym_line_comment,
      sym_block_comment,
  [157081] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10889), 1,
      sym_identifier,
    STATE(4768), 1,
      sym_metavariable,
    STATE(5409), 2,
      sym_line_comment,
      sym_block_comment,
  [157101] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8942), 1,
      anon_sym_RPAREN,
    ACTIONS(8944), 1,
      anon_sym_COMMA,
    STATE(5060), 1,
      aux_sym_parameters_repeat1,
    STATE(5410), 2,
      sym_line_comment,
      sym_block_comment,
  [157121] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10891), 1,
      anon_sym_SEMI,
    STATE(2404), 1,
      sym_declaration_list,
    STATE(5411), 2,
      sym_line_comment,
      sym_block_comment,
  [157141] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7725), 1,
      anon_sym_LBRACE,
    ACTIONS(10893), 1,
      anon_sym_SEMI,
    STATE(2406), 1,
      sym_declaration_list,
    STATE(5412), 2,
      sym_line_comment,
      sym_block_comment,
  [157161] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10895), 1,
      sym_identifier,
    ACTIONS(10897), 1,
      anon_sym_ref,
    ACTIONS(10899), 1,
      sym_mutable_specifier,
    STATE(5413), 2,
      sym_line_comment,
      sym_block_comment,
  [157181] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1097), 1,
      anon_sym_RPAREN,
    ACTIONS(10901), 1,
      anon_sym_COMMA,
    STATE(5005), 1,
      aux_sym_parameters_repeat1,
    STATE(5414), 2,
      sym_line_comment,
      sym_block_comment,
  [157201] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10903), 1,
      sym_mutable_specifier,
    ACTIONS(10905), 1,
      sym_self,
    STATE(5415), 2,
      sym_line_comment,
      sym_block_comment,
  [157221] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7131), 1,
      anon_sym_trait,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    STATE(5416), 2,
      sym_line_comment,
      sym_block_comment,
  [157241] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1085), 1,
      anon_sym_RPAREN,
    ACTIONS(10907), 1,
      anon_sym_COMMA,
    STATE(5357), 1,
      aux_sym_arguments_repeat1,
    STATE(5417), 2,
      sym_line_comment,
      sym_block_comment,
  [157261] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10909), 1,
      sym_identifier,
    ACTIONS(10911), 1,
      anon_sym_ref,
    ACTIONS(10913), 1,
      sym_mutable_specifier,
    STATE(5418), 2,
      sym_line_comment,
      sym_block_comment,
  [157281] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10915), 1,
      sym_identifier,
    ACTIONS(10917), 1,
      anon_sym_ref,
    ACTIONS(10919), 1,
      sym_mutable_specifier,
    STATE(5419), 2,
      sym_line_comment,
      sym_block_comment,
  [157301] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10921), 1,
      anon_sym_SEMI,
    STATE(815), 1,
      sym_declaration_list,
    STATE(5420), 2,
      sym_line_comment,
      sym_block_comment,
  [157321] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10923), 1,
      sym_identifier,
    ACTIONS(10925), 1,
      anon_sym_ref,
    ACTIONS(10927), 1,
      sym_mutable_specifier,
    STATE(5421), 2,
      sym_line_comment,
      sym_block_comment,
  [157341] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    ACTIONS(10929), 1,
      sym_identifier,
    STATE(5422), 2,
      sym_line_comment,
      sym_block_comment,
  [157361] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10931), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5423), 2,
      sym_line_comment,
      sym_block_comment,
  [157381] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10933), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5424), 2,
      sym_line_comment,
      sym_block_comment,
  [157399] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10935), 1,
      anon_sym_SEMI,
    STATE(760), 1,
      sym_declaration_list,
    STATE(5425), 2,
      sym_line_comment,
      sym_block_comment,
  [157419] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(7676), 1,
      anon_sym_LBRACE,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5426), 2,
      sym_line_comment,
      sym_block_comment,
  [157439] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10937), 1,
      anon_sym_SEMI,
    STATE(762), 1,
      sym_declaration_list,
    STATE(5427), 2,
      sym_line_comment,
      sym_block_comment,
  [157459] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    ACTIONS(10939), 1,
      anon_sym_GT,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5428), 2,
      sym_line_comment,
      sym_block_comment,
  [157479] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10941), 1,
      anon_sym_SEMI,
    STATE(764), 1,
      sym_declaration_list,
    STATE(5429), 2,
      sym_line_comment,
      sym_block_comment,
  [157499] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10943), 1,
      sym_identifier,
    ACTIONS(10945), 1,
      anon_sym_ref,
    ACTIONS(10947), 1,
      sym_mutable_specifier,
    STATE(5430), 2,
      sym_line_comment,
      sym_block_comment,
  [157519] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7733), 1,
      anon_sym_where,
    ACTIONS(10949), 1,
      anon_sym_SEMI,
    STATE(6108), 1,
      sym_where_clause,
    STATE(5431), 2,
      sym_line_comment,
      sym_block_comment,
  [157539] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    ACTIONS(10951), 1,
      anon_sym_SEMI,
    STATE(1098), 1,
      sym_field_declaration_list,
    STATE(5432), 2,
      sym_line_comment,
      sym_block_comment,
  [157559] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7753), 1,
      anon_sym_LBRACE,
    ACTIONS(10953), 1,
      anon_sym_SEMI,
    STATE(766), 1,
      sym_declaration_list,
    STATE(5433), 2,
      sym_line_comment,
      sym_block_comment,
  [157579] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5434), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1494), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [157595] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5435), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1536), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [157611] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10955), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5436), 2,
      sym_line_comment,
      sym_block_comment,
  [157629] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10939), 1,
      anon_sym_GT,
    ACTIONS(10957), 1,
      anon_sym_COMMA,
    STATE(4890), 1,
      aux_sym_unsafe_bound_type_repeat1,
    STATE(5437), 2,
      sym_line_comment,
      sym_block_comment,
  [157649] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(981), 1,
      aux_sym_metavariable_token1,
    ACTIONS(10959), 1,
      sym_identifier,
    STATE(4845), 1,
      sym_metavariable,
    STATE(5438), 2,
      sym_line_comment,
      sym_block_comment,
  [157669] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    ACTIONS(10961), 1,
      anon_sym_for,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5439), 2,
      sym_line_comment,
      sym_block_comment,
  [157689] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9426), 1,
      anon_sym_GT,
    ACTIONS(10963), 1,
      anon_sym_COMMA,
    STATE(4908), 1,
      aux_sym_use_bounds_repeat1,
    STATE(5440), 2,
      sym_line_comment,
      sym_block_comment,
  [157709] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    STATE(5441), 2,
      sym_line_comment,
      sym_block_comment,
    ACTIONS(1374), 3,
      anon_sym_EQ_GT,
      anon_sym_PIPE,
      anon_sym_if,
  [157725] = 6,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1780), 1,
      anon_sym_GT,
    ACTIONS(10639), 1,
      anon_sym_COMMA,
    STATE(4975), 1,
      aux_sym_type_arguments_repeat1,
    STATE(5442), 2,
      sym_line_comment,
      sym_block_comment,
  [157745] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10965), 1,
      anon_sym_SEMI,
    STATE(5443), 2,
      sym_line_comment,
      sym_block_comment,
  [157762] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1236), 1,
      sym_enum_variant_list,
    STATE(5444), 2,
      sym_line_comment,
      sym_block_comment,
  [157779] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2532), 1,
      sym_enum_variant_list,
    STATE(5445), 2,
      sym_line_comment,
      sym_block_comment,
  [157796] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10967), 1,
      anon_sym_SEMI,
    STATE(5446), 2,
      sym_line_comment,
      sym_block_comment,
  [157813] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10969), 1,
      anon_sym_SEMI,
    STATE(5447), 2,
      sym_line_comment,
      sym_block_comment,
  [157830] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10971), 1,
      anon_sym_SEMI,
    STATE(5448), 2,
      sym_line_comment,
      sym_block_comment,
  [157847] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10973), 1,
      sym_identifier,
    ACTIONS(10975), 1,
      sym_mutable_specifier,
    STATE(5449), 2,
      sym_line_comment,
      sym_block_comment,
  [157864] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10977), 1,
      anon_sym_SEMI,
    STATE(5450), 2,
      sym_line_comment,
      sym_block_comment,
  [157881] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10979), 1,
      anon_sym_SEMI,
    STATE(5451), 2,
      sym_line_comment,
      sym_block_comment,
  [157898] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10981), 1,
      anon_sym_SEMI,
    STATE(5452), 2,
      sym_line_comment,
      sym_block_comment,
  [157915] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10983), 1,
      anon_sym_SEMI,
    STATE(5453), 2,
      sym_line_comment,
      sym_block_comment,
  [157932] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10985), 1,
      anon_sym_SEMI,
    STATE(5454), 2,
      sym_line_comment,
      sym_block_comment,
  [157949] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10987), 1,
      anon_sym_SEMI,
    STATE(5455), 2,
      sym_line_comment,
      sym_block_comment,
  [157966] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10931), 1,
      anon_sym_for,
    STATE(5456), 2,
      sym_line_comment,
      sym_block_comment,
  [157983] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(10989), 1,
      anon_sym_SEMI,
    STATE(5457), 2,
      sym_line_comment,
      sym_block_comment,
  [158000] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8886), 1,
      sym_super,
    ACTIONS(10991), 1,
      sym_identifier,
    STATE(5458), 2,
      sym_line_comment,
      sym_block_comment,
  [158017] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10993), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(10995), 1,
      sym__block_comment_content,
    STATE(5459), 2,
      sym_line_comment,
      sym_block_comment,
  [158034] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10997), 1,
      sym_identifier,
    ACTIONS(10999), 1,
      sym_super,
    STATE(5460), 2,
      sym_line_comment,
      sym_block_comment,
  [158051] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11001), 1,
      anon_sym_BANG,
    ACTIONS(11003), 1,
      anon_sym_LBRACK,
    STATE(5461), 2,
      sym_line_comment,
      sym_block_comment,
  [158068] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(93), 1,
      sym_closure_parameters,
    STATE(5462), 2,
      sym_line_comment,
      sym_block_comment,
  [158085] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11005), 1,
      sym_identifier,
    ACTIONS(11007), 1,
      sym_super,
    STATE(5463), 2,
      sym_line_comment,
      sym_block_comment,
  [158102] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11009), 1,
      anon_sym_RPAREN,
    ACTIONS(11011), 1,
      anon_sym_COLON_COLON,
    STATE(5464), 2,
      sym_line_comment,
      sym_block_comment,
  [158119] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11013), 1,
      sym_identifier,
    ACTIONS(11015), 1,
      sym_super,
    STATE(5465), 2,
      sym_line_comment,
      sym_block_comment,
  [158136] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11017), 1,
      anon_sym_RPAREN,
    ACTIONS(11019), 1,
      anon_sym_COLON_COLON,
    STATE(5466), 2,
      sym_line_comment,
      sym_block_comment,
  [158153] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8826), 1,
      sym_super,
    ACTIONS(11021), 1,
      sym_identifier,
    STATE(5467), 2,
      sym_line_comment,
      sym_block_comment,
  [158170] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    STATE(3179), 1,
      sym_type_arguments,
    STATE(5468), 2,
      sym_line_comment,
      sym_block_comment,
  [158187] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9712), 1,
      anon_sym_for,
    STATE(5469), 2,
      sym_line_comment,
      sym_block_comment,
  [158204] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11017), 1,
      anon_sym_RPAREN,
    ACTIONS(11023), 1,
      anon_sym_COLON_COLON,
    STATE(5470), 2,
      sym_line_comment,
      sym_block_comment,
  [158221] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11025), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(5471), 2,
      sym_line_comment,
      sym_block_comment,
  [158236] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11017), 1,
      anon_sym_RPAREN,
    ACTIONS(11027), 1,
      anon_sym_COLON_COLON,
    STATE(5472), 2,
      sym_line_comment,
      sym_block_comment,
  [158253] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    STATE(3127), 1,
      sym_parameters,
    STATE(5473), 2,
      sym_line_comment,
      sym_block_comment,
  [158270] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1598), 1,
      sym_type_parameters,
    STATE(5474), 2,
      sym_line_comment,
      sym_block_comment,
  [158287] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11029), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5475), 2,
      sym_line_comment,
      sym_block_comment,
  [158302] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    STATE(5365), 1,
      sym_lifetime,
    STATE(5476), 2,
      sym_line_comment,
      sym_block_comment,
  [158319] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11031), 1,
      anon_sym_SEMI,
    STATE(5477), 2,
      sym_line_comment,
      sym_block_comment,
  [158336] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11033), 1,
      anon_sym_SEMI,
    STATE(5478), 2,
      sym_line_comment,
      sym_block_comment,
  [158353] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1258), 1,
      sym_field_declaration_list,
    STATE(5479), 2,
      sym_line_comment,
      sym_block_comment,
  [158370] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11035), 1,
      anon_sym_SEMI,
    STATE(5480), 2,
      sym_line_comment,
      sym_block_comment,
  [158387] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(104), 1,
      sym_closure_parameters,
    STATE(5481), 2,
      sym_line_comment,
      sym_block_comment,
  [158404] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11037), 1,
      sym_identifier,
    ACTIONS(11039), 1,
      sym_mutable_specifier,
    STATE(5482), 2,
      sym_line_comment,
      sym_block_comment,
  [158421] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11041), 1,
      anon_sym_SEMI,
    ACTIONS(11043), 1,
      anon_sym_as,
    STATE(5483), 2,
      sym_line_comment,
      sym_block_comment,
  [158438] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9186), 1,
      sym_identifier,
    ACTIONS(9188), 1,
      sym_super,
    STATE(5484), 2,
      sym_line_comment,
      sym_block_comment,
  [158455] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2560), 1,
      sym_field_declaration_list,
    STATE(5485), 2,
      sym_line_comment,
      sym_block_comment,
  [158472] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9130), 2,
      sym_identifier,
      sym_super,
    STATE(5486), 2,
      sym_line_comment,
      sym_block_comment,
  [158487] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11045), 1,
      sym_identifier,
    ACTIONS(11047), 1,
      sym_super,
    STATE(5487), 2,
      sym_line_comment,
      sym_block_comment,
  [158504] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7699), 1,
      anon_sym_PIPE,
    ACTIONS(11049), 1,
      anon_sym_COLON,
    STATE(5488), 2,
      sym_line_comment,
      sym_block_comment,
  [158521] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8268), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5489), 2,
      sym_line_comment,
      sym_block_comment,
  [158538] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10991), 1,
      sym_identifier,
    ACTIONS(11053), 1,
      sym_super,
    STATE(5490), 2,
      sym_line_comment,
      sym_block_comment,
  [158555] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11055), 1,
      anon_sym_SEMI,
    STATE(5491), 2,
      sym_line_comment,
      sym_block_comment,
  [158572] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10665), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
    STATE(5492), 2,
      sym_line_comment,
      sym_block_comment,
  [158587] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(8984), 1,
      sym_identifier,
    STATE(5493), 2,
      sym_line_comment,
      sym_block_comment,
  [158604] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11007), 1,
      sym_super,
    ACTIONS(11057), 1,
      sym_identifier,
    STATE(5494), 2,
      sym_line_comment,
      sym_block_comment,
  [158621] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11059), 1,
      anon_sym_LBRACE,
    STATE(2566), 1,
      sym_macro_body_v2,
    STATE(5495), 2,
      sym_line_comment,
      sym_block_comment,
  [158638] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2568), 1,
      sym_enum_variant_list,
    STATE(5496), 2,
      sym_line_comment,
      sym_block_comment,
  [158655] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11061), 1,
      anon_sym_LBRACE,
    STATE(716), 1,
      sym_macro_body_v2,
    STATE(5497), 2,
      sym_line_comment,
      sym_block_comment,
  [158672] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1604), 1,
      sym_type_parameters,
    STATE(5498), 2,
      sym_line_comment,
      sym_block_comment,
  [158689] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(101), 1,
      sym_closure_parameters,
    STATE(5499), 2,
      sym_line_comment,
      sym_block_comment,
  [158706] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1113), 1,
      sym_field_declaration_list,
    STATE(5500), 2,
      sym_line_comment,
      sym_block_comment,
  [158723] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1291), 1,
      sym_enum_variant_list,
    STATE(5501), 2,
      sym_line_comment,
      sym_block_comment,
  [158740] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6587), 1,
      anon_sym_LPAREN,
    STATE(3156), 1,
      sym_parameters,
    STATE(5502), 2,
      sym_line_comment,
      sym_block_comment,
  [158757] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8166), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5503), 2,
      sym_line_comment,
      sym_block_comment,
  [158774] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9803), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5504), 2,
      sym_line_comment,
      sym_block_comment,
  [158789] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(106), 1,
      sym_closure_parameters,
    STATE(5505), 2,
      sym_line_comment,
      sym_block_comment,
  [158806] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11063), 2,
      sym_identifier,
      sym_super,
    STATE(5506), 2,
      sym_line_comment,
      sym_block_comment,
  [158821] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2588), 1,
      sym_field_declaration_list,
    STATE(5507), 2,
      sym_line_comment,
      sym_block_comment,
  [158838] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11065), 1,
      anon_sym_SEMI,
    ACTIONS(11067), 1,
      anon_sym_as,
    STATE(5508), 2,
      sym_line_comment,
      sym_block_comment,
  [158855] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11069), 1,
      anon_sym_SEMI,
    ACTIONS(11071), 1,
      anon_sym_as,
    STATE(5509), 2,
      sym_line_comment,
      sym_block_comment,
  [158872] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11073), 1,
      anon_sym_SEMI,
    STATE(5510), 2,
      sym_line_comment,
      sym_block_comment,
  [158889] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11075), 1,
      anon_sym_SEMI,
    STATE(5511), 2,
      sym_line_comment,
      sym_block_comment,
  [158906] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11077), 1,
      anon_sym_SEMI,
    STATE(5512), 2,
      sym_line_comment,
      sym_block_comment,
  [158923] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8268), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5513), 2,
      sym_line_comment,
      sym_block_comment,
  [158940] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11059), 1,
      anon_sym_LBRACE,
    STATE(2606), 1,
      sym_macro_body_v2,
    STATE(5514), 2,
      sym_line_comment,
      sym_block_comment,
  [158957] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2607), 1,
      sym_enum_variant_list,
    STATE(5515), 2,
      sym_line_comment,
      sym_block_comment,
  [158974] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11079), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5516), 2,
      sym_line_comment,
      sym_block_comment,
  [158989] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8886), 2,
      sym_identifier,
      sym_super,
    STATE(5517), 2,
      sym_line_comment,
      sym_block_comment,
  [159004] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11081), 1,
      anon_sym_SEMI,
    STATE(5518), 2,
      sym_line_comment,
      sym_block_comment,
  [159021] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2617), 1,
      sym_field_declaration_list,
    STATE(5519), 2,
      sym_line_comment,
      sym_block_comment,
  [159038] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11083), 1,
      anon_sym_SEMI,
    ACTIONS(11085), 1,
      anon_sym_as,
    STATE(5520), 2,
      sym_line_comment,
      sym_block_comment,
  [159055] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8886), 1,
      sym_super,
    ACTIONS(11087), 1,
      sym_identifier,
    STATE(5521), 2,
      sym_line_comment,
      sym_block_comment,
  [159072] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11089), 1,
      anon_sym_SEMI,
    STATE(5522), 2,
      sym_line_comment,
      sym_block_comment,
  [159089] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11091), 2,
      sym_identifier,
      sym_super,
    STATE(5523), 2,
      sym_line_comment,
      sym_block_comment,
  [159104] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8886), 1,
      sym_super,
    ACTIONS(11093), 1,
      sym_identifier,
    STATE(5524), 2,
      sym_line_comment,
      sym_block_comment,
  [159121] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1357), 1,
      sym_field_declaration_list,
    STATE(5525), 2,
      sym_line_comment,
      sym_block_comment,
  [159138] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8826), 2,
      sym_identifier,
      sym_super,
    STATE(5526), 2,
      sym_line_comment,
      sym_block_comment,
  [159153] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8280), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5527), 2,
      sym_line_comment,
      sym_block_comment,
  [159170] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8286), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5528), 2,
      sym_line_comment,
      sym_block_comment,
  [159187] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3954), 1,
      sym_parameters,
    STATE(5529), 2,
      sym_line_comment,
      sym_block_comment,
  [159204] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11095), 1,
      anon_sym_BANG,
    STATE(5530), 2,
      sym_line_comment,
      sym_block_comment,
  [159221] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11097), 1,
      anon_sym_RPAREN,
    STATE(5531), 2,
      sym_line_comment,
      sym_block_comment,
  [159238] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11099), 1,
      anon_sym_SEMI,
    STATE(5532), 2,
      sym_line_comment,
      sym_block_comment,
  [159255] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11101), 1,
      anon_sym_RBRACE,
    STATE(5533), 2,
      sym_line_comment,
      sym_block_comment,
  [159272] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11103), 1,
      sym_identifier,
    ACTIONS(11105), 1,
      sym_integer_literal,
    STATE(5534), 2,
      sym_line_comment,
      sym_block_comment,
  [159289] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10166), 1,
      anon_sym_for,
    STATE(5535), 2,
      sym_line_comment,
      sym_block_comment,
  [159306] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8886), 1,
      sym_super,
    ACTIONS(11107), 1,
      sym_identifier,
    STATE(5536), 2,
      sym_line_comment,
      sym_block_comment,
  [159323] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11109), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(11111), 1,
      sym__block_comment_content,
    STATE(5537), 2,
      sym_line_comment,
      sym_block_comment,
  [159340] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(120), 1,
      sym_closure_parameters,
    STATE(5538), 2,
      sym_line_comment,
      sym_block_comment,
  [159357] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10860), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5539), 2,
      sym_line_comment,
      sym_block_comment,
  [159372] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8886), 1,
      sym_super,
    ACTIONS(11113), 1,
      sym_identifier,
    STATE(5540), 2,
      sym_line_comment,
      sym_block_comment,
  [159389] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11115), 1,
      anon_sym_SEMI,
    STATE(5541), 2,
      sym_line_comment,
      sym_block_comment,
  [159406] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11117), 1,
      anon_sym_SEMI,
    STATE(5542), 2,
      sym_line_comment,
      sym_block_comment,
  [159423] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11097), 1,
      anon_sym_RBRACK,
    STATE(5543), 2,
      sym_line_comment,
      sym_block_comment,
  [159440] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8826), 1,
      sym_super,
    ACTIONS(11119), 1,
      sym_identifier,
    STATE(5544), 2,
      sym_line_comment,
      sym_block_comment,
  [159457] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10819), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5545), 2,
      sym_line_comment,
      sym_block_comment,
  [159472] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10177), 1,
      anon_sym_for,
    STATE(5546), 2,
      sym_line_comment,
      sym_block_comment,
  [159489] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11121), 1,
      anon_sym_SEMI,
    STATE(5547), 2,
      sym_line_comment,
      sym_block_comment,
  [159506] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11123), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5548), 2,
      sym_line_comment,
      sym_block_comment,
  [159521] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11125), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(11127), 1,
      sym__block_comment_content,
    STATE(5549), 2,
      sym_line_comment,
      sym_block_comment,
  [159538] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(123), 1,
      sym_closure_parameters,
    STATE(5550), 2,
      sym_line_comment,
      sym_block_comment,
  [159555] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11129), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5551), 2,
      sym_line_comment,
      sym_block_comment,
  [159570] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6479), 1,
      anon_sym_COLON,
    STATE(5552), 2,
      sym_line_comment,
      sym_block_comment,
  [159587] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9188), 2,
      sym_identifier,
      sym_super,
    STATE(5553), 2,
      sym_line_comment,
      sym_block_comment,
  [159602] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11053), 1,
      sym_super,
    ACTIONS(11131), 1,
      sym_identifier,
    STATE(5554), 2,
      sym_line_comment,
      sym_block_comment,
  [159619] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11133), 1,
      anon_sym_SEMI,
    STATE(5555), 2,
      sym_line_comment,
      sym_block_comment,
  [159636] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9118), 1,
      sym_identifier,
    STATE(5556), 2,
      sym_line_comment,
      sym_block_comment,
  [159653] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11007), 1,
      sym_super,
    ACTIONS(11135), 1,
      sym_identifier,
    STATE(5557), 2,
      sym_line_comment,
      sym_block_comment,
  [159670] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6003), 1,
      anon_sym_LBRACE,
    STATE(2893), 1,
      sym_field_initializer_list,
    STATE(5558), 2,
      sym_line_comment,
      sym_block_comment,
  [159687] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11137), 1,
      anon_sym_SEMI,
    STATE(5559), 2,
      sym_line_comment,
      sym_block_comment,
  [159704] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7564), 1,
      anon_sym_BANG,
    STATE(5560), 2,
      sym_line_comment,
      sym_block_comment,
  [159721] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(124), 1,
      sym_closure_parameters,
    STATE(5561), 2,
      sym_line_comment,
      sym_block_comment,
  [159738] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11139), 1,
      anon_sym_SEMI,
    STATE(5562), 2,
      sym_line_comment,
      sym_block_comment,
  [159755] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11141), 1,
      anon_sym_SEMI,
    STATE(5563), 2,
      sym_line_comment,
      sym_block_comment,
  [159772] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(4162), 1,
      sym_parameters,
    STATE(5564), 2,
      sym_line_comment,
      sym_block_comment,
  [159789] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10574), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5565), 2,
      sym_line_comment,
      sym_block_comment,
  [159804] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11143), 1,
      anon_sym_RPAREN,
    STATE(5566), 2,
      sym_line_comment,
      sym_block_comment,
  [159821] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10327), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5567), 2,
      sym_line_comment,
      sym_block_comment,
  [159836] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11145), 1,
      anon_sym_RBRACE,
    STATE(5568), 2,
      sym_line_comment,
      sym_block_comment,
  [159853] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11143), 1,
      anon_sym_RBRACK,
    STATE(5569), 2,
      sym_line_comment,
      sym_block_comment,
  [159870] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11147), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(11149), 1,
      sym__block_comment_content,
    STATE(5570), 2,
      sym_line_comment,
      sym_block_comment,
  [159887] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(127), 1,
      sym_closure_parameters,
    STATE(5571), 2,
      sym_line_comment,
      sym_block_comment,
  [159904] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11151), 1,
      anon_sym_RPAREN,
    STATE(5572), 2,
      sym_line_comment,
      sym_block_comment,
  [159921] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7727), 1,
      anon_sym_COLON,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(5573), 2,
      sym_line_comment,
      sym_block_comment,
  [159938] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11153), 1,
      anon_sym_RBRACE,
    STATE(5574), 2,
      sym_line_comment,
      sym_block_comment,
  [159955] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11091), 1,
      sym_super,
    ACTIONS(11155), 1,
      sym_identifier,
    STATE(5575), 2,
      sym_line_comment,
      sym_block_comment,
  [159972] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11151), 1,
      anon_sym_RBRACK,
    STATE(5576), 2,
      sym_line_comment,
      sym_block_comment,
  [159989] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10465), 1,
      sym_identifier,
    ACTIONS(10469), 1,
      sym_mutable_specifier,
    STATE(5577), 2,
      sym_line_comment,
      sym_block_comment,
  [160006] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(129), 1,
      sym_closure_parameters,
    STATE(5578), 2,
      sym_line_comment,
      sym_block_comment,
  [160023] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11157), 1,
      anon_sym_SEMI,
    STATE(5579), 2,
      sym_line_comment,
      sym_block_comment,
  [160040] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8387), 1,
      anon_sym_COLON,
    STATE(4260), 1,
      sym_trait_bounds,
    STATE(5580), 2,
      sym_line_comment,
      sym_block_comment,
  [160057] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11159), 1,
      anon_sym_SEMI,
    STATE(5581), 2,
      sym_line_comment,
      sym_block_comment,
  [160074] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11161), 1,
      anon_sym_SEMI,
    STATE(5582), 2,
      sym_line_comment,
      sym_block_comment,
  [160091] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(8641), 1,
      anon_sym_COLON,
    STATE(5583), 2,
      sym_line_comment,
      sym_block_comment,
  [160108] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11163), 1,
      sym_identifier,
    ACTIONS(11165), 1,
      sym_super,
    STATE(5584), 2,
      sym_line_comment,
      sym_block_comment,
  [160125] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(5585), 2,
      sym_line_comment,
      sym_block_comment,
  [160142] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11167), 1,
      sym_identifier,
    ACTIONS(11169), 1,
      sym_super,
    STATE(5586), 2,
      sym_line_comment,
      sym_block_comment,
  [160159] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8826), 1,
      sym_super,
    ACTIONS(11171), 1,
      sym_identifier,
    STATE(5587), 2,
      sym_line_comment,
      sym_block_comment,
  [160176] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10386), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5588), 2,
      sym_line_comment,
      sym_block_comment,
  [160191] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(92), 1,
      sym_closure_parameters,
    STATE(5589), 2,
      sym_line_comment,
      sym_block_comment,
  [160208] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11011), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11173), 1,
      anon_sym_RPAREN,
    STATE(5590), 2,
      sym_line_comment,
      sym_block_comment,
  [160225] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11019), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11175), 1,
      anon_sym_RPAREN,
    STATE(5591), 2,
      sym_line_comment,
      sym_block_comment,
  [160242] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11177), 1,
      anon_sym_SEMI,
    STATE(5592), 2,
      sym_line_comment,
      sym_block_comment,
  [160259] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11023), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11175), 1,
      anon_sym_RPAREN,
    STATE(5593), 2,
      sym_line_comment,
      sym_block_comment,
  [160276] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11027), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11175), 1,
      anon_sym_RPAREN,
    STATE(5594), 2,
      sym_line_comment,
      sym_block_comment,
  [160293] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11179), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(11181), 1,
      sym__block_comment_content,
    STATE(5595), 2,
      sym_line_comment,
      sym_block_comment,
  [160310] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11183), 1,
      anon_sym_RPAREN,
    STATE(5596), 2,
      sym_line_comment,
      sym_block_comment,
  [160327] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11091), 1,
      sym_super,
    ACTIONS(11185), 1,
      sym_identifier,
    STATE(5597), 2,
      sym_line_comment,
      sym_block_comment,
  [160344] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11187), 1,
      anon_sym_SEMI,
    STATE(5598), 2,
      sym_line_comment,
      sym_block_comment,
  [160361] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11189), 1,
      anon_sym_SEMI,
    STATE(5599), 2,
      sym_line_comment,
      sym_block_comment,
  [160378] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11191), 1,
      anon_sym_in,
    STATE(5600), 2,
      sym_line_comment,
      sym_block_comment,
  [160395] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11193), 1,
      anon_sym_SEMI,
    STATE(5601), 2,
      sym_line_comment,
      sym_block_comment,
  [160412] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11195), 1,
      anon_sym_SEMI,
    STATE(5602), 2,
      sym_line_comment,
      sym_block_comment,
  [160429] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11197), 1,
      anon_sym_RBRACE,
    STATE(5603), 2,
      sym_line_comment,
      sym_block_comment,
  [160446] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11053), 1,
      sym_super,
    ACTIONS(11199), 1,
      sym_identifier,
    STATE(5604), 2,
      sym_line_comment,
      sym_block_comment,
  [160463] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9575), 1,
      sym_identifier,
    STATE(5605), 2,
      sym_line_comment,
      sym_block_comment,
  [160480] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11169), 1,
      sym_super,
    ACTIONS(11201), 1,
      sym_identifier,
    STATE(5606), 2,
      sym_line_comment,
      sym_block_comment,
  [160497] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11203), 1,
      anon_sym_SEMI,
    STATE(5607), 2,
      sym_line_comment,
      sym_block_comment,
  [160514] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11205), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(11207), 1,
      sym__block_comment_content,
    STATE(5608), 2,
      sym_line_comment,
      sym_block_comment,
  [160531] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2674), 1,
      sym_enum_variant_list,
    STATE(5609), 2,
      sym_line_comment,
      sym_block_comment,
  [160548] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10068), 1,
      anon_sym_for,
    STATE(5610), 2,
      sym_line_comment,
      sym_block_comment,
  [160565] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7574), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11209), 1,
      anon_sym_BANG,
    STATE(5611), 2,
      sym_line_comment,
      sym_block_comment,
  [160582] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(4184), 1,
      sym_parameters,
    STATE(5612), 2,
      sym_line_comment,
      sym_block_comment,
  [160599] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11211), 1,
      sym_identifier,
    ACTIONS(11213), 1,
      sym_super,
    STATE(5613), 2,
      sym_line_comment,
      sym_block_comment,
  [160616] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8826), 1,
      sym_super,
    ACTIONS(11215), 1,
      sym_identifier,
    STATE(5614), 2,
      sym_line_comment,
      sym_block_comment,
  [160633] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11217), 1,
      anon_sym_SEMI,
    STATE(5615), 2,
      sym_line_comment,
      sym_block_comment,
  [160650] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11053), 1,
      sym_super,
    ACTIONS(11219), 1,
      sym_identifier,
    STATE(5616), 2,
      sym_line_comment,
      sym_block_comment,
  [160667] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9130), 1,
      sym_super,
    ACTIONS(9196), 1,
      sym_identifier,
    STATE(5617), 2,
      sym_line_comment,
      sym_block_comment,
  [160684] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11063), 1,
      sym_super,
    ACTIONS(11221), 1,
      sym_identifier,
    STATE(5618), 2,
      sym_line_comment,
      sym_block_comment,
  [160701] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(105), 1,
      sym_closure_parameters,
    STATE(5619), 2,
      sym_line_comment,
      sym_block_comment,
  [160718] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11223), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5620), 2,
      sym_line_comment,
      sym_block_comment,
  [160733] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11225), 1,
      anon_sym_SEMI,
    STATE(5621), 2,
      sym_line_comment,
      sym_block_comment,
  [160750] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8154), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5622), 2,
      sym_line_comment,
      sym_block_comment,
  [160767] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1511), 1,
      sym_type_parameters,
    STATE(5623), 2,
      sym_line_comment,
      sym_block_comment,
  [160784] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8156), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5624), 2,
      sym_line_comment,
      sym_block_comment,
  [160801] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8826), 1,
      sym_super,
    ACTIONS(11155), 1,
      sym_identifier,
    STATE(5625), 2,
      sym_line_comment,
      sym_block_comment,
  [160818] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8154), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5626), 2,
      sym_line_comment,
      sym_block_comment,
  [160835] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11053), 1,
      sym_super,
    ACTIONS(11107), 1,
      sym_identifier,
    STATE(5627), 2,
      sym_line_comment,
      sym_block_comment,
  [160852] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9266), 1,
      sym_identifier,
    STATE(5628), 2,
      sym_line_comment,
      sym_block_comment,
  [160869] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11007), 1,
      sym_super,
    ACTIONS(11167), 1,
      sym_identifier,
    STATE(5629), 2,
      sym_line_comment,
      sym_block_comment,
  [160886] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8158), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5630), 2,
      sym_line_comment,
      sym_block_comment,
  [160903] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6629), 1,
      anon_sym_LBRACE,
    STATE(3263), 1,
      sym_field_initializer_list,
    STATE(5631), 2,
      sym_line_comment,
      sym_block_comment,
  [160920] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11091), 1,
      sym_super,
    ACTIONS(11227), 1,
      sym_identifier,
    STATE(5632), 2,
      sym_line_comment,
      sym_block_comment,
  [160937] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9264), 1,
      sym_identifier,
    STATE(5633), 2,
      sym_line_comment,
      sym_block_comment,
  [160954] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11169), 1,
      sym_super,
    ACTIONS(11229), 1,
      sym_identifier,
    STATE(5634), 2,
      sym_line_comment,
      sym_block_comment,
  [160971] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11091), 1,
      sym_super,
    ACTIONS(11231), 1,
      sym_identifier,
    STATE(5635), 2,
      sym_line_comment,
      sym_block_comment,
  [160988] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9306), 1,
      sym_identifier,
    STATE(5636), 2,
      sym_line_comment,
      sym_block_comment,
  [161005] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11169), 1,
      sym_super,
    ACTIONS(11233), 1,
      sym_identifier,
    STATE(5637), 2,
      sym_line_comment,
      sym_block_comment,
  [161022] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11091), 1,
      sym_super,
    ACTIONS(11235), 1,
      sym_identifier,
    STATE(5638), 2,
      sym_line_comment,
      sym_block_comment,
  [161039] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9120), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(5639), 2,
      sym_line_comment,
      sym_block_comment,
  [161056] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11169), 1,
      sym_super,
    ACTIONS(11237), 1,
      sym_identifier,
    STATE(5640), 2,
      sym_line_comment,
      sym_block_comment,
  [161073] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8826), 1,
      sym_super,
    ACTIONS(11235), 1,
      sym_identifier,
    STATE(5641), 2,
      sym_line_comment,
      sym_block_comment,
  [161090] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9334), 1,
      sym_identifier,
    STATE(5642), 2,
      sym_line_comment,
      sym_block_comment,
  [161107] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11007), 1,
      sym_super,
    ACTIONS(11237), 1,
      sym_identifier,
    STATE(5643), 2,
      sym_line_comment,
      sym_block_comment,
  [161124] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2702), 1,
      sym_field_declaration_list,
    STATE(5644), 2,
      sym_line_comment,
      sym_block_comment,
  [161141] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8160), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5645), 2,
      sym_line_comment,
      sym_block_comment,
  [161158] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11239), 1,
      anon_sym_SEMI,
    STATE(5646), 2,
      sym_line_comment,
      sym_block_comment,
  [161175] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8158), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5647), 2,
      sym_line_comment,
      sym_block_comment,
  [161192] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11183), 1,
      anon_sym_RBRACK,
    STATE(5648), 2,
      sym_line_comment,
      sym_block_comment,
  [161209] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11241), 1,
      anon_sym_SEMI,
    STATE(5649), 2,
      sym_line_comment,
      sym_block_comment,
  [161226] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11243), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5650), 2,
      sym_line_comment,
      sym_block_comment,
  [161241] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11059), 1,
      anon_sym_LBRACE,
    STATE(2710), 1,
      sym_macro_body_v2,
    STATE(5651), 2,
      sym_line_comment,
      sym_block_comment,
  [161258] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2711), 1,
      sym_enum_variant_list,
    STATE(5652), 2,
      sym_line_comment,
      sym_block_comment,
  [161275] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7576), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11209), 1,
      anon_sym_BANG,
    STATE(5653), 2,
      sym_line_comment,
      sym_block_comment,
  [161292] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11245), 1,
      anon_sym_SEMI,
    STATE(5654), 2,
      sym_line_comment,
      sym_block_comment,
  [161309] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11247), 1,
      anon_sym_SEMI,
    STATE(5655), 2,
      sym_line_comment,
      sym_block_comment,
  [161326] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8886), 1,
      sym_super,
    ACTIONS(11249), 1,
      sym_identifier,
    STATE(5656), 2,
      sym_line_comment,
      sym_block_comment,
  [161343] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2721), 1,
      sym_field_declaration_list,
    STATE(5657), 2,
      sym_line_comment,
      sym_block_comment,
  [161360] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11251), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(5658), 2,
      sym_line_comment,
      sym_block_comment,
  [161375] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11253), 1,
      anon_sym_SEMI,
    ACTIONS(11255), 1,
      anon_sym_as,
    STATE(5659), 2,
      sym_line_comment,
      sym_block_comment,
  [161392] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9118), 1,
      sym_identifier,
    ACTIONS(9120), 1,
      sym_super,
    STATE(5660), 2,
      sym_line_comment,
      sym_block_comment,
  [161409] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1653), 1,
      sym_type_parameters,
    STATE(5661), 2,
      sym_line_comment,
      sym_block_comment,
  [161426] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11257), 1,
      anon_sym_SEMI,
    STATE(5662), 2,
      sym_line_comment,
      sym_block_comment,
  [161443] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2729), 1,
      sym_enum_variant_list,
    STATE(5663), 2,
      sym_line_comment,
      sym_block_comment,
  [161460] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5949), 1,
      anon_sym_PLUS,
    ACTIONS(6041), 1,
      anon_sym_COLON_COLON,
    STATE(5664), 2,
      sym_line_comment,
      sym_block_comment,
  [161477] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11259), 1,
      anon_sym_BANG,
    ACTIONS(11261), 1,
      anon_sym_COLON_COLON,
    STATE(5665), 2,
      sym_line_comment,
      sym_block_comment,
  [161494] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5949), 1,
      anon_sym_PLUS,
    ACTIONS(6151), 1,
      anon_sym_COLON_COLON,
    STATE(5666), 2,
      sym_line_comment,
      sym_block_comment,
  [161511] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11263), 1,
      anon_sym_SEMI,
    STATE(5667), 2,
      sym_line_comment,
      sym_block_comment,
  [161528] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6235), 1,
      anon_sym_COLON,
    STATE(5668), 2,
      sym_line_comment,
      sym_block_comment,
  [161545] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11265), 2,
      sym_float_literal,
      sym_integer_literal,
    STATE(5669), 2,
      sym_line_comment,
      sym_block_comment,
  [161560] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11021), 1,
      sym_identifier,
    ACTIONS(11091), 1,
      sym_super,
    STATE(5670), 2,
      sym_line_comment,
      sym_block_comment,
  [161577] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6251), 1,
      anon_sym_COLON,
    STATE(5671), 2,
      sym_line_comment,
      sym_block_comment,
  [161594] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2809), 1,
      sym_type_arguments,
    STATE(5672), 2,
      sym_line_comment,
      sym_block_comment,
  [161611] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2739), 1,
      sym_field_declaration_list,
    STATE(5673), 2,
      sym_line_comment,
      sym_block_comment,
  [161628] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11267), 1,
      anon_sym_SEMI,
    STATE(5674), 2,
      sym_line_comment,
      sym_block_comment,
  [161645] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1174), 1,
      sym_enum_variant_list,
    STATE(5675), 2,
      sym_line_comment,
      sym_block_comment,
  [161662] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11269), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(5676), 2,
      sym_line_comment,
      sym_block_comment,
  [161677] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1214), 1,
      sym_enum_variant_list,
    STATE(5677), 2,
      sym_line_comment,
      sym_block_comment,
  [161694] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    STATE(2832), 1,
      sym_parameters,
    STATE(5678), 2,
      sym_line_comment,
      sym_block_comment,
  [161711] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11271), 1,
      anon_sym_RBRACE,
    STATE(5679), 2,
      sym_line_comment,
      sym_block_comment,
  [161728] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11273), 1,
      anon_sym_RBRACE,
    STATE(5680), 2,
      sym_line_comment,
      sym_block_comment,
  [161745] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11275), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5681), 2,
      sym_line_comment,
      sym_block_comment,
  [161760] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    STATE(4981), 1,
      sym_lifetime,
    STATE(5682), 2,
      sym_line_comment,
      sym_block_comment,
  [161777] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(7594), 1,
      anon_sym_COLON_COLON,
    STATE(5683), 2,
      sym_line_comment,
      sym_block_comment,
  [161794] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1575), 1,
      sym_type_parameters,
    STATE(5684), 2,
      sym_line_comment,
      sym_block_comment,
  [161811] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11091), 1,
      sym_super,
    ACTIONS(11119), 1,
      sym_identifier,
    STATE(5685), 2,
      sym_line_comment,
      sym_block_comment,
  [161828] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3908), 1,
      sym_parameters,
    STATE(5686), 2,
      sym_line_comment,
      sym_block_comment,
  [161845] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6267), 1,
      anon_sym_COLON,
    STATE(5687), 2,
      sym_line_comment,
      sym_block_comment,
  [161862] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11277), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5688), 2,
      sym_line_comment,
      sym_block_comment,
  [161877] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11279), 1,
      anon_sym_SEMI,
    STATE(5689), 2,
      sym_line_comment,
      sym_block_comment,
  [161894] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10245), 1,
      anon_sym_for,
    STATE(5690), 2,
      sym_line_comment,
      sym_block_comment,
  [161911] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10509), 1,
      anon_sym_for,
    STATE(5691), 2,
      sym_line_comment,
      sym_block_comment,
  [161928] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6919), 1,
      anon_sym_EQ_GT,
    ACTIONS(11281), 1,
      anon_sym_AMP_AMP,
    STATE(5692), 2,
      sym_line_comment,
      sym_block_comment,
  [161945] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10082), 1,
      anon_sym_EQ_GT,
    ACTIONS(11281), 1,
      anon_sym_AMP_AMP,
    STATE(5693), 2,
      sym_line_comment,
      sym_block_comment,
  [161962] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11283), 1,
      anon_sym_SEMI,
    STATE(5694), 2,
      sym_line_comment,
      sym_block_comment,
  [161979] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1081), 1,
      sym_enum_variant_list,
    STATE(5695), 2,
      sym_line_comment,
      sym_block_comment,
  [161996] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11285), 1,
      anon_sym_SEMI,
    STATE(5696), 2,
      sym_line_comment,
      sym_block_comment,
  [162013] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11287), 1,
      anon_sym_SEMI,
    STATE(5697), 2,
      sym_line_comment,
      sym_block_comment,
  [162030] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11289), 1,
      anon_sym_SEMI,
    STATE(5698), 2,
      sym_line_comment,
      sym_block_comment,
  [162047] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11291), 1,
      anon_sym_SEMI,
    STATE(5699), 2,
      sym_line_comment,
      sym_block_comment,
  [162064] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10961), 1,
      anon_sym_for,
    STATE(5700), 2,
      sym_line_comment,
      sym_block_comment,
  [162081] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11293), 1,
      sym_mutable_specifier,
    ACTIONS(11295), 1,
      sym_self,
    STATE(5701), 2,
      sym_line_comment,
      sym_block_comment,
  [162098] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(112), 1,
      sym_closure_parameters,
    STATE(5702), 2,
      sym_line_comment,
      sym_block_comment,
  [162115] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(9819), 1,
      anon_sym_for,
    STATE(5703), 2,
      sym_line_comment,
      sym_block_comment,
  [162132] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11007), 2,
      sym_identifier,
      sym_super,
    STATE(5704), 2,
      sym_line_comment,
      sym_block_comment,
  [162147] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11297), 1,
      anon_sym_COLON,
    STATE(5705), 2,
      sym_line_comment,
      sym_block_comment,
  [162164] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3921), 1,
      sym_parameters,
    STATE(5706), 2,
      sym_line_comment,
      sym_block_comment,
  [162181] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10517), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5707), 2,
      sym_line_comment,
      sym_block_comment,
  [162196] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6381), 1,
      anon_sym_COLON,
    STATE(5708), 2,
      sym_line_comment,
      sym_block_comment,
  [162213] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11299), 1,
      anon_sym_SEMI,
    STATE(5709), 2,
      sym_line_comment,
      sym_block_comment,
  [162230] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9432), 1,
      sym_identifier,
    STATE(5710), 2,
      sym_line_comment,
      sym_block_comment,
  [162247] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1267), 1,
      sym_field_declaration_list,
    STATE(5711), 2,
      sym_line_comment,
      sym_block_comment,
  [162264] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11301), 1,
      anon_sym_SEMI,
    STATE(5712), 2,
      sym_line_comment,
      sym_block_comment,
  [162281] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11303), 1,
      anon_sym_SEMI,
    STATE(5713), 2,
      sym_line_comment,
      sym_block_comment,
  [162298] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11305), 1,
      anon_sym_SEMI,
    STATE(5714), 2,
      sym_line_comment,
      sym_block_comment,
  [162315] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6389), 1,
      anon_sym_COLON,
    STATE(5715), 2,
      sym_line_comment,
      sym_block_comment,
  [162332] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11307), 1,
      anon_sym_EQ,
    STATE(5716), 2,
      sym_line_comment,
      sym_block_comment,
  [162349] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11309), 1,
      anon_sym_SEMI,
    STATE(5717), 2,
      sym_line_comment,
      sym_block_comment,
  [162366] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11311), 1,
      anon_sym_SEMI,
    STATE(5718), 2,
      sym_line_comment,
      sym_block_comment,
  [162383] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3914), 1,
      sym_parameters,
    STATE(5719), 2,
      sym_line_comment,
      sym_block_comment,
  [162400] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11313), 1,
      anon_sym_SEMI,
    STATE(5720), 2,
      sym_line_comment,
      sym_block_comment,
  [162417] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11315), 1,
      anon_sym_SEMI,
    STATE(5721), 2,
      sym_line_comment,
      sym_block_comment,
  [162434] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8236), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5722), 2,
      sym_line_comment,
      sym_block_comment,
  [162449] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9120), 2,
      sym_identifier,
      sym_super,
    STATE(5723), 2,
      sym_line_comment,
      sym_block_comment,
  [162464] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11007), 1,
      sym_super,
    ACTIONS(11317), 1,
      sym_identifier,
    STATE(5724), 2,
      sym_line_comment,
      sym_block_comment,
  [162481] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10615), 1,
      sym_identifier,
    ACTIONS(10619), 1,
      sym_mutable_specifier,
    STATE(5725), 2,
      sym_line_comment,
      sym_block_comment,
  [162498] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3918), 1,
      sym_parameters,
    STATE(5726), 2,
      sym_line_comment,
      sym_block_comment,
  [162515] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8214), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5727), 2,
      sym_line_comment,
      sym_block_comment,
  [162532] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8188), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5728), 2,
      sym_line_comment,
      sym_block_comment,
  [162549] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8188), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5729), 2,
      sym_line_comment,
      sym_block_comment,
  [162566] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8190), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5730), 2,
      sym_line_comment,
      sym_block_comment,
  [162583] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8218), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5731), 2,
      sym_line_comment,
      sym_block_comment,
  [162600] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8190), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5732), 2,
      sym_line_comment,
      sym_block_comment,
  [162617] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6593), 1,
      anon_sym_LT2,
    STATE(3213), 1,
      sym_type_arguments,
    STATE(5733), 2,
      sym_line_comment,
      sym_block_comment,
  [162634] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11319), 1,
      anon_sym_SEMI,
    STATE(5734), 2,
      sym_line_comment,
      sym_block_comment,
  [162651] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6405), 1,
      anon_sym_COLON,
    STATE(5735), 2,
      sym_line_comment,
      sym_block_comment,
  [162668] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11059), 1,
      anon_sym_LBRACE,
    STATE(2457), 1,
      sym_macro_body_v2,
    STATE(5736), 2,
      sym_line_comment,
      sym_block_comment,
  [162685] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6409), 1,
      anon_sym_COLON,
    STATE(5737), 2,
      sym_line_comment,
      sym_block_comment,
  [162702] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8242), 1,
      sym_super,
    ACTIONS(9262), 1,
      sym_identifier,
    STATE(5738), 2,
      sym_line_comment,
      sym_block_comment,
  [162719] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11321), 1,
      anon_sym_SEMI,
    STATE(5739), 2,
      sym_line_comment,
      sym_block_comment,
  [162736] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11323), 1,
      anon_sym_SEMI,
    STATE(5740), 2,
      sym_line_comment,
      sym_block_comment,
  [162753] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11325), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(5741), 2,
      sym_line_comment,
      sym_block_comment,
  [162768] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11327), 1,
      anon_sym_COLON,
    STATE(5742), 2,
      sym_line_comment,
      sym_block_comment,
  [162785] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11329), 1,
      anon_sym_SEMI,
    STATE(5743), 2,
      sym_line_comment,
      sym_block_comment,
  [162802] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11331), 1,
      anon_sym_SEMI,
    STATE(5744), 2,
      sym_line_comment,
      sym_block_comment,
  [162819] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11333), 1,
      anon_sym_SEMI,
    ACTIONS(11335), 1,
      anon_sym_as,
    STATE(5745), 2,
      sym_line_comment,
      sym_block_comment,
  [162836] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11337), 1,
      anon_sym_SEMI,
    STATE(5746), 2,
      sym_line_comment,
      sym_block_comment,
  [162853] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11339), 1,
      anon_sym_SEMI,
    STATE(5747), 2,
      sym_line_comment,
      sym_block_comment,
  [162870] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11341), 1,
      anon_sym_SEMI,
    STATE(5748), 2,
      sym_line_comment,
      sym_block_comment,
  [162887] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10433), 1,
      anon_sym_for,
    STATE(5749), 2,
      sym_line_comment,
      sym_block_comment,
  [162904] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11053), 1,
      sym_super,
    ACTIONS(11087), 1,
      sym_identifier,
    STATE(5750), 2,
      sym_line_comment,
      sym_block_comment,
  [162921] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11343), 1,
      anon_sym_GT,
    STATE(5751), 2,
      sym_line_comment,
      sym_block_comment,
  [162938] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10437), 1,
      anon_sym_for,
    STATE(5752), 2,
      sym_line_comment,
      sym_block_comment,
  [162955] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11213), 2,
      sym_identifier,
      sym_super,
    STATE(5753), 2,
      sym_line_comment,
      sym_block_comment,
  [162970] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7850), 1,
      anon_sym_for,
    STATE(5754), 2,
      sym_line_comment,
      sym_block_comment,
  [162987] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11345), 2,
      sym_float_literal,
      sym_integer_literal,
    STATE(5755), 2,
      sym_line_comment,
      sym_block_comment,
  [163002] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8984), 1,
      sym_identifier,
    ACTIONS(9120), 1,
      sym_super,
    STATE(5756), 2,
      sym_line_comment,
      sym_block_comment,
  [163019] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2078), 1,
      sym_enum_variant_list,
    STATE(5757), 2,
      sym_line_comment,
      sym_block_comment,
  [163036] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9656), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5758), 2,
      sym_line_comment,
      sym_block_comment,
  [163051] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3925), 1,
      sym_parameters,
    STATE(5759), 2,
      sym_line_comment,
      sym_block_comment,
  [163068] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11057), 1,
      sym_identifier,
    ACTIONS(11169), 1,
      sym_super,
    STATE(5760), 2,
      sym_line_comment,
      sym_block_comment,
  [163085] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11347), 1,
      anon_sym_SEMI,
    STATE(5761), 2,
      sym_line_comment,
      sym_block_comment,
  [163102] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11061), 1,
      anon_sym_LBRACE,
    STATE(1280), 1,
      sym_macro_body_v2,
    STATE(5762), 2,
      sym_line_comment,
      sym_block_comment,
  [163119] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(717), 1,
      sym_enum_variant_list,
    STATE(5763), 2,
      sym_line_comment,
      sym_block_comment,
  [163136] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3928), 1,
      sym_parameters,
    STATE(5764), 2,
      sym_line_comment,
      sym_block_comment,
  [163153] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11349), 1,
      anon_sym_RPAREN,
    STATE(5765), 2,
      sym_line_comment,
      sym_block_comment,
  [163170] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11349), 1,
      anon_sym_RBRACK,
    STATE(5766), 2,
      sym_line_comment,
      sym_block_comment,
  [163187] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11351), 1,
      anon_sym_RPAREN,
    STATE(5767), 2,
      sym_line_comment,
      sym_block_comment,
  [163204] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11351), 1,
      anon_sym_RBRACK,
    STATE(5768), 2,
      sym_line_comment,
      sym_block_comment,
  [163221] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6517), 1,
      anon_sym_COLON,
    STATE(5769), 2,
      sym_line_comment,
      sym_block_comment,
  [163238] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2088), 1,
      sym_field_declaration_list,
    STATE(5770), 2,
      sym_line_comment,
      sym_block_comment,
  [163255] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6529), 1,
      anon_sym_COLON,
    STATE(5771), 2,
      sym_line_comment,
      sym_block_comment,
  [163272] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1579), 1,
      sym_type_parameters,
    STATE(5772), 2,
      sym_line_comment,
      sym_block_comment,
  [163289] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1282), 1,
      sym_enum_variant_list,
    STATE(5773), 2,
      sym_line_comment,
      sym_block_comment,
  [163306] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3947), 1,
      sym_parameters,
    STATE(5774), 2,
      sym_line_comment,
      sym_block_comment,
  [163323] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(10066), 1,
      anon_sym_COLON_COLON,
    STATE(5775), 2,
      sym_line_comment,
      sym_block_comment,
  [163340] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8204), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5776), 2,
      sym_line_comment,
      sym_block_comment,
  [163357] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8204), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5777), 2,
      sym_line_comment,
      sym_block_comment,
  [163374] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5893), 1,
      anon_sym_LPAREN,
    STATE(2827), 1,
      sym_parameters,
    STATE(5778), 2,
      sym_line_comment,
      sym_block_comment,
  [163391] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8206), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5779), 2,
      sym_line_comment,
      sym_block_comment,
  [163408] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8206), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5780), 2,
      sym_line_comment,
      sym_block_comment,
  [163425] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7832), 1,
      anon_sym_for,
    STATE(5781), 2,
      sym_line_comment,
      sym_block_comment,
  [163442] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(727), 1,
      sym_field_declaration_list,
    STATE(5782), 2,
      sym_line_comment,
      sym_block_comment,
  [163459] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1711), 1,
      sym_type_parameters,
    STATE(5783), 2,
      sym_line_comment,
      sym_block_comment,
  [163476] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3953), 1,
      sym_parameters,
    STATE(5784), 2,
      sym_line_comment,
      sym_block_comment,
  [163493] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11353), 1,
      anon_sym_BANG,
    ACTIONS(11355), 1,
      anon_sym_LBRACK,
    STATE(5785), 2,
      sym_line_comment,
      sym_block_comment,
  [163510] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11357), 1,
      anon_sym_SEMI,
    STATE(5786), 2,
      sym_line_comment,
      sym_block_comment,
  [163527] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7221), 1,
      anon_sym_LT2,
    STATE(3556), 1,
      sym_type_arguments,
    STATE(5787), 2,
      sym_line_comment,
      sym_block_comment,
  [163544] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11359), 1,
      anon_sym_SEMI,
    STATE(5788), 2,
      sym_line_comment,
      sym_block_comment,
  [163561] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11361), 1,
      anon_sym_SEMI,
    STATE(5789), 2,
      sym_line_comment,
      sym_block_comment,
  [163578] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11363), 1,
      anon_sym_SEMI,
    STATE(5790), 2,
      sym_line_comment,
      sym_block_comment,
  [163595] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6353), 1,
      anon_sym_COLON,
    STATE(5791), 2,
      sym_line_comment,
      sym_block_comment,
  [163612] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11365), 1,
      sym_mutable_specifier,
    ACTIONS(11367), 1,
      sym_self,
    STATE(5792), 2,
      sym_line_comment,
      sym_block_comment,
  [163629] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8373), 1,
      anon_sym_LBRACE,
    STATE(2466), 1,
      sym_enum_variant_list,
    STATE(5793), 2,
      sym_line_comment,
      sym_block_comment,
  [163646] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11369), 1,
      anon_sym_EQ,
    STATE(5794), 2,
      sym_line_comment,
      sym_block_comment,
  [163663] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3932), 1,
      sym_parameters,
    STATE(5795), 2,
      sym_line_comment,
      sym_block_comment,
  [163680] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11371), 1,
      anon_sym_SEMI,
    STATE(5796), 2,
      sym_line_comment,
      sym_block_comment,
  [163697] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11373), 1,
      anon_sym_SEMI,
    STATE(5797), 2,
      sym_line_comment,
      sym_block_comment,
  [163714] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11375), 1,
      anon_sym_SEMI,
    STATE(5798), 2,
      sym_line_comment,
      sym_block_comment,
  [163731] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11377), 1,
      anon_sym_SEMI,
    STATE(5799), 2,
      sym_line_comment,
      sym_block_comment,
  [163748] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11379), 1,
      anon_sym_RPAREN,
    STATE(5800), 2,
      sym_line_comment,
      sym_block_comment,
  [163765] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11379), 1,
      anon_sym_RBRACK,
    STATE(5801), 2,
      sym_line_comment,
      sym_block_comment,
  [163782] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11381), 1,
      anon_sym_RPAREN,
    STATE(5802), 2,
      sym_line_comment,
      sym_block_comment,
  [163799] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11381), 1,
      anon_sym_RBRACK,
    STATE(5803), 2,
      sym_line_comment,
      sym_block_comment,
  [163816] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11383), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5804), 2,
      sym_line_comment,
      sym_block_comment,
  [163831] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(97), 1,
      sym_closure_parameters,
    STATE(5805), 2,
      sym_line_comment,
      sym_block_comment,
  [163848] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    ACTIONS(6493), 1,
      anon_sym_COLON,
    STATE(5806), 2,
      sym_line_comment,
      sym_block_comment,
  [163865] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11385), 1,
      anon_sym_SEMI,
    STATE(5807), 2,
      sym_line_comment,
      sym_block_comment,
  [163882] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11387), 1,
      anon_sym_RBRACE,
    STATE(5808), 2,
      sym_line_comment,
      sym_block_comment,
  [163899] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    ACTIONS(11389), 1,
      anon_sym_RBRACE,
    STATE(5809), 2,
      sym_line_comment,
      sym_block_comment,
  [163916] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11391), 1,
      anon_sym_SEMI,
    STATE(5810), 2,
      sym_line_comment,
      sym_block_comment,
  [163933] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11393), 1,
      anon_sym_SEMI,
    STATE(5811), 2,
      sym_line_comment,
      sym_block_comment,
  [163950] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11395), 1,
      anon_sym_SEMI,
    STATE(5812), 2,
      sym_line_comment,
      sym_block_comment,
  [163967] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8266), 1,
      anon_sym_RPAREN,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5813), 2,
      sym_line_comment,
      sym_block_comment,
  [163984] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11053), 1,
      sym_super,
    ACTIONS(11113), 1,
      sym_identifier,
    STATE(5814), 2,
      sym_line_comment,
      sym_block_comment,
  [164001] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11397), 1,
      anon_sym_SEMI,
    STATE(5815), 2,
      sym_line_comment,
      sym_block_comment,
  [164018] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(11399), 1,
      anon_sym_COLON_COLON,
    STATE(5816), 2,
      sym_line_comment,
      sym_block_comment,
  [164035] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7741), 1,
      anon_sym_LBRACE,
    STATE(2487), 1,
      sym_field_declaration_list,
    STATE(5817), 2,
      sym_line_comment,
      sym_block_comment,
  [164052] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11053), 2,
      sym_identifier,
      sym_super,
    STATE(5818), 2,
      sym_line_comment,
      sym_block_comment,
  [164067] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11169), 2,
      sym_identifier,
      sym_super,
    STATE(5819), 2,
      sym_line_comment,
      sym_block_comment,
  [164082] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11401), 1,
      anon_sym_SEMI,
    ACTIONS(11403), 1,
      anon_sym_as,
    STATE(5820), 2,
      sym_line_comment,
      sym_block_comment,
  [164099] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7699), 1,
      anon_sym_PIPE,
    ACTIONS(11405), 1,
      anon_sym_COLON,
    STATE(5821), 2,
      sym_line_comment,
      sym_block_comment,
  [164116] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11407), 1,
      anon_sym_SEMI,
    STATE(5822), 2,
      sym_line_comment,
      sym_block_comment,
  [164133] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11409), 1,
      anon_sym_SEMI,
    STATE(5823), 2,
      sym_line_comment,
      sym_block_comment,
  [164150] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11411), 1,
      anon_sym_SEMI,
    STATE(5824), 2,
      sym_line_comment,
      sym_block_comment,
  [164167] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11413), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5825), 2,
      sym_line_comment,
      sym_block_comment,
  [164182] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1303), 1,
      sym_field_declaration_list,
    STATE(5826), 2,
      sym_line_comment,
      sym_block_comment,
  [164199] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11415), 1,
      anon_sym_SEMI,
    STATE(5827), 2,
      sym_line_comment,
      sym_block_comment,
  [164216] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11417), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5828), 2,
      sym_line_comment,
      sym_block_comment,
  [164231] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11419), 1,
      anon_sym_in,
    STATE(5829), 2,
      sym_line_comment,
      sym_block_comment,
  [164248] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11421), 1,
      sym_identifier,
    ACTIONS(11423), 1,
      sym_integer_literal,
    STATE(5830), 2,
      sym_line_comment,
      sym_block_comment,
  [164265] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(130), 1,
      sym_closure_parameters,
    STATE(5831), 2,
      sym_line_comment,
      sym_block_comment,
  [164282] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11425), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5832), 2,
      sym_line_comment,
      sym_block_comment,
  [164297] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11427), 1,
      anon_sym_SEMI,
    STATE(5833), 2,
      sym_line_comment,
      sym_block_comment,
  [164314] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11429), 2,
      anon_sym_const,
      sym_mutable_specifier,
    STATE(5834), 2,
      sym_line_comment,
      sym_block_comment,
  [164329] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1691), 1,
      sym_type_parameters,
    STATE(5835), 2,
      sym_line_comment,
      sym_block_comment,
  [164346] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5949), 1,
      anon_sym_PLUS,
    ACTIONS(6733), 1,
      anon_sym_COLON_COLON,
    STATE(5836), 2,
      sym_line_comment,
      sym_block_comment,
  [164363] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(11431), 1,
      anon_sym_COLON_COLON,
    STATE(5837), 2,
      sym_line_comment,
      sym_block_comment,
  [164380] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5949), 1,
      anon_sym_PLUS,
    ACTIONS(6743), 1,
      anon_sym_COLON_COLON,
    STATE(5838), 2,
      sym_line_comment,
      sym_block_comment,
  [164397] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8298), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5839), 2,
      sym_line_comment,
      sym_block_comment,
  [164412] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11433), 1,
      anon_sym_SEMI,
    STATE(5840), 2,
      sym_line_comment,
      sym_block_comment,
  [164429] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6131), 1,
      anon_sym_COLON,
    ACTIONS(11435), 1,
      anon_sym_DASH_GT,
    STATE(5841), 2,
      sym_line_comment,
      sym_block_comment,
  [164446] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    STATE(4919), 1,
      sym_lifetime,
    STATE(5842), 2,
      sym_line_comment,
      sym_block_comment,
  [164463] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1452), 1,
      anon_sym_LT2,
    ACTIONS(7598), 1,
      anon_sym_COLON_COLON,
    STATE(5843), 2,
      sym_line_comment,
      sym_block_comment,
  [164480] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6043), 1,
      anon_sym_COLON,
    ACTIONS(11437), 1,
      anon_sym_DASH_GT,
    STATE(5844), 2,
      sym_line_comment,
      sym_block_comment,
  [164497] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6211), 1,
      anon_sym_COLON,
    ACTIONS(11439), 1,
      anon_sym_DASH_GT,
    STATE(5845), 2,
      sym_line_comment,
      sym_block_comment,
  [164514] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8543), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5846), 2,
      sym_line_comment,
      sym_block_comment,
  [164529] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11441), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5847), 2,
      sym_line_comment,
      sym_block_comment,
  [164544] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10768), 1,
      sym_identifier,
    ACTIONS(10772), 1,
      sym_mutable_specifier,
    STATE(5848), 2,
      sym_line_comment,
      sym_block_comment,
  [164561] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8104), 1,
      anon_sym_RBRACE,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5849), 2,
      sym_line_comment,
      sym_block_comment,
  [164578] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11443), 1,
      anon_sym_SEMI,
    STATE(5850), 2,
      sym_line_comment,
      sym_block_comment,
  [164595] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6097), 1,
      anon_sym_COLON,
    ACTIONS(11445), 1,
      anon_sym_DASH_GT,
    STATE(5851), 2,
      sym_line_comment,
      sym_block_comment,
  [164612] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8242), 2,
      sym_identifier,
      sym_super,
    STATE(5852), 2,
      sym_line_comment,
      sym_block_comment,
  [164627] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    STATE(5868), 1,
      sym_lifetime,
    STATE(5853), 2,
      sym_line_comment,
      sym_block_comment,
  [164644] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11447), 1,
      anon_sym_SEMI,
    ACTIONS(11449), 1,
      anon_sym_as,
    STATE(5854), 2,
      sym_line_comment,
      sym_block_comment,
  [164661] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11451), 1,
      anon_sym_SEMI,
    STATE(5855), 2,
      sym_line_comment,
      sym_block_comment,
  [164678] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11453), 1,
      anon_sym_STAR_SLASH,
    ACTIONS(11455), 1,
      sym__block_comment_content,
    STATE(5856), 2,
      sym_line_comment,
      sym_block_comment,
  [164695] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11457), 1,
      anon_sym_BANG,
    ACTIONS(11459), 1,
      anon_sym_COLON_COLON,
    STATE(5857), 2,
      sym_line_comment,
      sym_block_comment,
  [164712] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11461), 1,
      anon_sym_SEMI,
    STATE(5858), 2,
      sym_line_comment,
      sym_block_comment,
  [164729] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8466), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5859), 2,
      sym_line_comment,
      sym_block_comment,
  [164744] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11463), 1,
      anon_sym_SEMI,
    STATE(5860), 2,
      sym_line_comment,
      sym_block_comment,
  [164761] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11465), 1,
      anon_sym_SEMI,
    STATE(5861), 2,
      sym_line_comment,
      sym_block_comment,
  [164778] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11467), 1,
      anon_sym_SEMI,
    STATE(5862), 2,
      sym_line_comment,
      sym_block_comment,
  [164795] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7209), 1,
      anon_sym_BANG,
    ACTIONS(11469), 1,
      anon_sym_COLON_COLON,
    STATE(5863), 2,
      sym_line_comment,
      sym_block_comment,
  [164812] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11471), 1,
      anon_sym_SEMI,
    STATE(5864), 2,
      sym_line_comment,
      sym_block_comment,
  [164829] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11473), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5865), 2,
      sym_line_comment,
      sym_block_comment,
  [164844] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11475), 1,
      anon_sym_BANG,
    ACTIONS(11477), 1,
      anon_sym_LBRACK,
    STATE(5866), 2,
      sym_line_comment,
      sym_block_comment,
  [164861] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5901), 1,
      anon_sym_LT2,
    STATE(2886), 1,
      sym_type_arguments,
    STATE(5867), 2,
      sym_line_comment,
      sym_block_comment,
  [164878] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9778), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5868), 2,
      sym_line_comment,
      sym_block_comment,
  [164893] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3652), 1,
      sym_parameters,
    STATE(5869), 2,
      sym_line_comment,
      sym_block_comment,
  [164910] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11479), 1,
      anon_sym_SEMI,
    STATE(5870), 2,
      sym_line_comment,
      sym_block_comment,
  [164927] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11481), 1,
      anon_sym_in,
    STATE(5871), 2,
      sym_line_comment,
      sym_block_comment,
  [164944] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11483), 1,
      anon_sym_SEMI,
    STATE(5872), 2,
      sym_line_comment,
      sym_block_comment,
  [164961] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1334), 1,
      sym_field_declaration_list,
    STATE(5873), 2,
      sym_line_comment,
      sym_block_comment,
  [164978] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11485), 1,
      anon_sym_SEMI,
    STATE(5874), 2,
      sym_line_comment,
      sym_block_comment,
  [164995] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11487), 1,
      anon_sym_SEMI,
    STATE(5875), 2,
      sym_line_comment,
      sym_block_comment,
  [165012] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11489), 1,
      anon_sym_SEMI,
    STATE(5876), 2,
      sym_line_comment,
      sym_block_comment,
  [165029] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10281), 1,
      anon_sym_for,
    STATE(5877), 2,
      sym_line_comment,
      sym_block_comment,
  [165046] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7743), 1,
      anon_sym_LT,
    STATE(1744), 1,
      sym_type_parameters,
    STATE(5878), 2,
      sym_line_comment,
      sym_block_comment,
  [165063] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10118), 1,
      sym_identifier,
    ACTIONS(10122), 1,
      sym_mutable_specifier,
    STATE(5879), 2,
      sym_line_comment,
      sym_block_comment,
  [165080] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11491), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5880), 2,
      sym_line_comment,
      sym_block_comment,
  [165095] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    STATE(5135), 1,
      sym_lifetime,
    STATE(5881), 2,
      sym_line_comment,
      sym_block_comment,
  [165112] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3666), 1,
      sym_parameters,
    STATE(5882), 2,
      sym_line_comment,
      sym_block_comment,
  [165129] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9817), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5883), 2,
      sym_line_comment,
      sym_block_comment,
  [165144] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7852), 1,
      anon_sym_for,
    STATE(5884), 2,
      sym_line_comment,
      sym_block_comment,
  [165161] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11493), 1,
      anon_sym_SEMI,
    STATE(5885), 2,
      sym_line_comment,
      sym_block_comment,
  [165178] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11495), 1,
      anon_sym_SEMI,
    STATE(5886), 2,
      sym_line_comment,
      sym_block_comment,
  [165195] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11497), 1,
      anon_sym_SEMI,
    STATE(5887), 2,
      sym_line_comment,
      sym_block_comment,
  [165212] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7747), 1,
      anon_sym_LBRACE,
    STATE(1213), 1,
      sym_field_declaration_list,
    STATE(5888), 2,
      sym_line_comment,
      sym_block_comment,
  [165229] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11135), 1,
      sym_identifier,
    ACTIONS(11169), 1,
      sym_super,
    STATE(5889), 2,
      sym_line_comment,
      sym_block_comment,
  [165246] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5669), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
    STATE(5890), 2,
      sym_line_comment,
      sym_block_comment,
  [165261] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10362), 1,
      sym_integer_literal,
    ACTIONS(11499), 1,
      sym_identifier,
    STATE(5891), 2,
      sym_line_comment,
      sym_block_comment,
  [165278] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10803), 1,
      anon_sym_for,
    STATE(5892), 2,
      sym_line_comment,
      sym_block_comment,
  [165295] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10807), 1,
      anon_sym_for,
    STATE(5893), 2,
      sym_line_comment,
      sym_block_comment,
  [165312] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11501), 1,
      anon_sym_SEMI,
    STATE(5894), 2,
      sym_line_comment,
      sym_block_comment,
  [165329] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(7868), 1,
      anon_sym_for,
    STATE(5895), 2,
      sym_line_comment,
      sym_block_comment,
  [165346] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11503), 1,
      anon_sym_SEMI,
    STATE(5896), 2,
      sym_line_comment,
      sym_block_comment,
  [165363] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11505), 1,
      anon_sym_SEMI,
    STATE(5897), 2,
      sym_line_comment,
      sym_block_comment,
  [165380] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11507), 1,
      anon_sym_SEMI,
    STATE(5898), 2,
      sym_line_comment,
      sym_block_comment,
  [165397] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    STATE(94), 1,
      sym_closure_parameters,
    STATE(5899), 2,
      sym_line_comment,
      sym_block_comment,
  [165414] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11509), 1,
      anon_sym_in,
    STATE(5900), 2,
      sym_line_comment,
      sym_block_comment,
  [165431] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11511), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5901), 2,
      sym_line_comment,
      sym_block_comment,
  [165446] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10836), 1,
      anon_sym_for,
    STATE(5902), 2,
      sym_line_comment,
      sym_block_comment,
  [165463] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11165), 2,
      sym_identifier,
      sym_super,
    STATE(5903), 2,
      sym_line_comment,
      sym_block_comment,
  [165478] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11513), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    STATE(5904), 2,
      sym_line_comment,
      sym_block_comment,
  [165493] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10840), 1,
      anon_sym_for,
    STATE(5905), 2,
      sym_line_comment,
      sym_block_comment,
  [165510] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11515), 1,
      anon_sym_SEMI,
    STATE(5906), 2,
      sym_line_comment,
      sym_block_comment,
  [165527] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10844), 1,
      anon_sym_for,
    STATE(5907), 2,
      sym_line_comment,
      sym_block_comment,
  [165544] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10848), 1,
      anon_sym_for,
    STATE(5908), 2,
      sym_line_comment,
      sym_block_comment,
  [165561] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10852), 1,
      anon_sym_for,
    STATE(5909), 2,
      sym_line_comment,
      sym_block_comment,
  [165578] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11517), 1,
      anon_sym_SEMI,
    STATE(5910), 2,
      sym_line_comment,
      sym_block_comment,
  [165595] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10858), 1,
      anon_sym_for,
    STATE(5911), 2,
      sym_line_comment,
      sym_block_comment,
  [165612] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11519), 1,
      anon_sym_SEMI,
    STATE(5912), 2,
      sym_line_comment,
      sym_block_comment,
  [165629] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10865), 1,
      anon_sym_for,
    STATE(5913), 2,
      sym_line_comment,
      sym_block_comment,
  [165646] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11521), 1,
      anon_sym_SEMI,
    STATE(5914), 2,
      sym_line_comment,
      sym_block_comment,
  [165663] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11523), 1,
      anon_sym_SEMI,
    STATE(5915), 2,
      sym_line_comment,
      sym_block_comment,
  [165680] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10869), 1,
      anon_sym_for,
    STATE(5916), 2,
      sym_line_comment,
      sym_block_comment,
  [165697] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11525), 1,
      anon_sym_SEMI,
    STATE(5917), 2,
      sym_line_comment,
      sym_block_comment,
  [165714] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10873), 1,
      anon_sym_for,
    STATE(5918), 2,
      sym_line_comment,
      sym_block_comment,
  [165731] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8266), 1,
      anon_sym_RBRACK,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(5919), 2,
      sym_line_comment,
      sym_block_comment,
  [165748] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    ACTIONS(10879), 1,
      anon_sym_for,
    STATE(5920), 2,
      sym_line_comment,
      sym_block_comment,
  [165765] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11527), 1,
      anon_sym_BANG,
    ACTIONS(11529), 1,
      anon_sym_LBRACK,
    STATE(5921), 2,
      sym_line_comment,
      sym_block_comment,
  [165782] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(5841), 1,
      sym_parameters,
    STATE(5922), 2,
      sym_line_comment,
      sym_block_comment,
  [165799] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11531), 1,
      anon_sym_in,
    STATE(5923), 2,
      sym_line_comment,
      sym_block_comment,
  [165816] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(2101), 1,
      anon_sym_SQUOTE,
    STATE(5322), 1,
      sym_lifetime,
    STATE(5924), 2,
      sym_line_comment,
      sym_block_comment,
  [165833] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11533), 1,
      anon_sym_SEMI,
    STATE(5925), 2,
      sym_line_comment,
      sym_block_comment,
  [165850] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(5851), 1,
      sym_parameters,
    STATE(5926), 2,
      sym_line_comment,
      sym_block_comment,
  [165867] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11535), 1,
      anon_sym_in,
    STATE(5927), 2,
      sym_line_comment,
      sym_block_comment,
  [165884] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11537), 1,
      anon_sym_in,
    STATE(5928), 2,
      sym_line_comment,
      sym_block_comment,
  [165901] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8623), 1,
      anon_sym_PIPE,
    ACTIONS(11539), 1,
      anon_sym_in,
    STATE(5929), 2,
      sym_line_comment,
      sym_block_comment,
  [165918] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11541), 1,
      anon_sym_SEMI,
    STATE(5930), 2,
      sym_line_comment,
      sym_block_comment,
  [165935] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11543), 1,
      anon_sym_SEMI,
    STATE(5931), 2,
      sym_line_comment,
      sym_block_comment,
  [165952] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10895), 1,
      sym_identifier,
    ACTIONS(10899), 1,
      sym_mutable_specifier,
    STATE(5932), 2,
      sym_line_comment,
      sym_block_comment,
  [165969] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3969), 1,
      sym_parameters,
    STATE(5933), 2,
      sym_line_comment,
      sym_block_comment,
  [165986] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    ACTIONS(11545), 1,
      anon_sym_BANG,
    STATE(5934), 2,
      sym_line_comment,
      sym_block_comment,
  [166003] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11547), 1,
      anon_sym_SEMI,
    STATE(5935), 2,
      sym_line_comment,
      sym_block_comment,
  [166020] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10923), 1,
      sym_identifier,
    ACTIONS(10927), 1,
      sym_mutable_specifier,
    STATE(5936), 2,
      sym_line_comment,
      sym_block_comment,
  [166037] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10915), 1,
      sym_identifier,
    ACTIONS(10919), 1,
      sym_mutable_specifier,
    STATE(5937), 2,
      sym_line_comment,
      sym_block_comment,
  [166054] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11549), 2,
      anon_sym_COMMA,
      anon_sym_GT,
    STATE(5938), 2,
      sym_line_comment,
      sym_block_comment,
  [166069] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11551), 1,
      anon_sym_SEMI,
    STATE(5939), 2,
      sym_line_comment,
      sym_block_comment,
  [166086] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7223), 1,
      anon_sym_LPAREN,
    STATE(3949), 1,
      sym_parameters,
    STATE(5940), 2,
      sym_line_comment,
      sym_block_comment,
  [166103] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11553), 1,
      anon_sym_SEMI,
    STATE(5941), 2,
      sym_line_comment,
      sym_block_comment,
  [166120] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11555), 1,
      anon_sym_SEMI,
    STATE(5942), 2,
      sym_line_comment,
      sym_block_comment,
  [166137] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11557), 1,
      anon_sym_SEMI,
    STATE(5943), 2,
      sym_line_comment,
      sym_block_comment,
  [166154] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11559), 1,
      anon_sym_SEMI,
    STATE(5944), 2,
      sym_line_comment,
      sym_block_comment,
  [166171] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11061), 1,
      anon_sym_LBRACE,
    STATE(1322), 1,
      sym_macro_body_v2,
    STATE(5945), 2,
      sym_line_comment,
      sym_block_comment,
  [166188] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11061), 1,
      anon_sym_LBRACE,
    STATE(1235), 1,
      sym_macro_body_v2,
    STATE(5946), 2,
      sym_line_comment,
      sym_block_comment,
  [166205] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10943), 1,
      sym_identifier,
    ACTIONS(10947), 1,
      sym_mutable_specifier,
    STATE(5947), 2,
      sym_line_comment,
      sym_block_comment,
  [166222] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11561), 1,
      sym_identifier,
    ACTIONS(11563), 1,
      sym_mutable_specifier,
    STATE(5948), 2,
      sym_line_comment,
      sym_block_comment,
  [166239] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11565), 1,
      anon_sym_SEMI,
    STATE(5949), 2,
      sym_line_comment,
      sym_block_comment,
  [166256] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11567), 1,
      anon_sym_SEMI,
    STATE(5950), 2,
      sym_line_comment,
      sym_block_comment,
  [166273] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11569), 1,
      sym_identifier,
    ACTIONS(11571), 1,
      sym_mutable_specifier,
    STATE(5951), 2,
      sym_line_comment,
      sym_block_comment,
  [166290] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11477), 1,
      anon_sym_LBRACK,
    ACTIONS(11527), 1,
      anon_sym_BANG,
    STATE(5952), 2,
      sym_line_comment,
      sym_block_comment,
  [166307] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8347), 1,
      anon_sym_LBRACE,
    STATE(1323), 1,
      sym_enum_variant_list,
    STATE(5953), 2,
      sym_line_comment,
      sym_block_comment,
  [166324] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11573), 1,
      anon_sym_SEMI,
    STATE(5954), 2,
      sym_line_comment,
      sym_block_comment,
  [166341] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11575), 1,
      anon_sym_SEMI,
    STATE(5955), 2,
      sym_line_comment,
      sym_block_comment,
  [166358] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11577), 1,
      anon_sym_SEMI,
    STATE(5956), 2,
      sym_line_comment,
      sym_block_comment,
  [166375] = 5,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    ACTIONS(11579), 1,
      anon_sym_SEMI,
    STATE(5957), 2,
      sym_line_comment,
      sym_block_comment,
  [166392] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11581), 1,
      anon_sym_COLON,
    STATE(5958), 2,
      sym_line_comment,
      sym_block_comment,
  [166406] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11583), 1,
      anon_sym_SEMI,
    STATE(5959), 2,
      sym_line_comment,
      sym_block_comment,
  [166420] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11585), 1,
      anon_sym_SEMI,
    STATE(5960), 2,
      sym_line_comment,
      sym_block_comment,
  [166434] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1760), 1,
      anon_sym_RBRACE,
    STATE(5961), 2,
      sym_line_comment,
      sym_block_comment,
  [166448] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11587), 1,
      anon_sym_SEMI,
    STATE(5962), 2,
      sym_line_comment,
      sym_block_comment,
  [166462] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9850), 1,
      anon_sym_RBRACE,
    STATE(5963), 2,
      sym_line_comment,
      sym_block_comment,
  [166476] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11589), 1,
      anon_sym_SEMI,
    STATE(5964), 2,
      sym_line_comment,
      sym_block_comment,
  [166490] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11591), 1,
      anon_sym_SEMI,
    STATE(5965), 2,
      sym_line_comment,
      sym_block_comment,
  [166504] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11593), 1,
      anon_sym_SEMI,
    STATE(5966), 2,
      sym_line_comment,
      sym_block_comment,
  [166518] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11595), 1,
      anon_sym_SEMI,
    STATE(5967), 2,
      sym_line_comment,
      sym_block_comment,
  [166532] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6541), 1,
      anon_sym_COLON_COLON,
    STATE(5968), 2,
      sym_line_comment,
      sym_block_comment,
  [166546] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1708), 1,
      anon_sym_RBRACE,
    STATE(5969), 2,
      sym_line_comment,
      sym_block_comment,
  [166560] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11597), 1,
      anon_sym_SEMI,
    STATE(5970), 2,
      sym_line_comment,
      sym_block_comment,
  [166574] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11599), 1,
      anon_sym_SEMI,
    STATE(5971), 2,
      sym_line_comment,
      sym_block_comment,
  [166588] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11601), 1,
      sym_identifier,
    STATE(5972), 2,
      sym_line_comment,
      sym_block_comment,
  [166602] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11603), 1,
      anon_sym_SEMI,
    STATE(5973), 2,
      sym_line_comment,
      sym_block_comment,
  [166616] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11605), 1,
      sym_identifier,
    STATE(5974), 2,
      sym_line_comment,
      sym_block_comment,
  [166630] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11607), 1,
      anon_sym_SEMI,
    STATE(5975), 2,
      sym_line_comment,
      sym_block_comment,
  [166644] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11609), 1,
      anon_sym_SEMI,
    STATE(5976), 2,
      sym_line_comment,
      sym_block_comment,
  [166658] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11611), 1,
      sym_identifier,
    STATE(5977), 2,
      sym_line_comment,
      sym_block_comment,
  [166672] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11613), 1,
      anon_sym_COLON,
    STATE(5978), 2,
      sym_line_comment,
      sym_block_comment,
  [166686] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11615), 1,
      anon_sym_SEMI,
    STATE(5979), 2,
      sym_line_comment,
      sym_block_comment,
  [166700] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11617), 1,
      sym__line_doc_content,
    STATE(5980), 2,
      sym_line_comment,
      sym_block_comment,
  [166714] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11619), 1,
      sym__frontmatter_end,
    STATE(5981), 2,
      sym_line_comment,
      sym_block_comment,
  [166728] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11621), 1,
      anon_sym_RBRACK,
    STATE(5982), 2,
      sym_line_comment,
      sym_block_comment,
  [166742] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11623), 1,
      sym_identifier,
    STATE(5983), 2,
      sym_line_comment,
      sym_block_comment,
  [166756] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11625), 1,
      sym_identifier,
    STATE(5984), 2,
      sym_line_comment,
      sym_block_comment,
  [166770] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11627), 1,
      anon_sym_SEMI,
    STATE(5985), 2,
      sym_line_comment,
      sym_block_comment,
  [166784] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11629), 1,
      sym_identifier,
    STATE(5986), 2,
      sym_line_comment,
      sym_block_comment,
  [166798] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8166), 1,
      anon_sym_SEMI,
    STATE(5987), 2,
      sym_line_comment,
      sym_block_comment,
  [166812] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11631), 1,
      anon_sym_SEMI,
    STATE(5988), 2,
      sym_line_comment,
      sym_block_comment,
  [166826] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8214), 1,
      anon_sym_SEMI,
    STATE(5989), 2,
      sym_line_comment,
      sym_block_comment,
  [166840] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11633), 1,
      sym_identifier,
    STATE(5990), 2,
      sym_line_comment,
      sym_block_comment,
  [166854] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10881), 1,
      anon_sym_RBRACE,
    STATE(5991), 2,
      sym_line_comment,
      sym_block_comment,
  [166868] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11635), 1,
      anon_sym_SEMI,
    STATE(5992), 2,
      sym_line_comment,
      sym_block_comment,
  [166882] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7562), 1,
      anon_sym_trait,
    STATE(5993), 2,
      sym_line_comment,
      sym_block_comment,
  [166896] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11637), 1,
      anon_sym_SEMI,
    STATE(5994), 2,
      sym_line_comment,
      sym_block_comment,
  [166910] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8218), 1,
      anon_sym_SEMI,
    STATE(5995), 2,
      sym_line_comment,
      sym_block_comment,
  [166924] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11639), 1,
      sym_identifier,
    STATE(5996), 2,
      sym_line_comment,
      sym_block_comment,
  [166938] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11641), 1,
      anon_sym_EQ_GT,
    STATE(5997), 2,
      sym_line_comment,
      sym_block_comment,
  [166952] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11643), 1,
      anon_sym_SEMI,
    STATE(5998), 2,
      sym_line_comment,
      sym_block_comment,
  [166966] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11101), 1,
      anon_sym_SEMI,
    STATE(5999), 2,
      sym_line_comment,
      sym_block_comment,
  [166980] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11645), 1,
      anon_sym_STAR_SLASH,
    STATE(6000), 2,
      sym_line_comment,
      sym_block_comment,
  [166994] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11647), 1,
      anon_sym_SEMI,
    STATE(6001), 2,
      sym_line_comment,
      sym_block_comment,
  [167008] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11649), 1,
      sym__line_doc_content,
    STATE(6002), 2,
      sym_line_comment,
      sym_block_comment,
  [167022] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11651), 1,
      anon_sym_COLON_COLON,
    STATE(6003), 2,
      sym_line_comment,
      sym_block_comment,
  [167036] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11653), 1,
      anon_sym_SEMI,
    STATE(6004), 2,
      sym_line_comment,
      sym_block_comment,
  [167050] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11655), 1,
      sym__line_doc_content,
    STATE(6005), 2,
      sym_line_comment,
      sym_block_comment,
  [167064] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11657), 1,
      sym__raw_string_literal_end,
    STATE(6006), 2,
      sym_line_comment,
      sym_block_comment,
  [167078] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11659), 1,
      anon_sym_STAR_SLASH,
    STATE(6007), 2,
      sym_line_comment,
      sym_block_comment,
  [167092] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11661), 1,
      anon_sym_SEMI,
    STATE(6008), 2,
      sym_line_comment,
      sym_block_comment,
  [167106] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11663), 1,
      anon_sym_SEMI,
    STATE(6009), 2,
      sym_line_comment,
      sym_block_comment,
  [167120] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11665), 1,
      anon_sym_EQ_GT,
    STATE(6010), 2,
      sym_line_comment,
      sym_block_comment,
  [167134] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(856), 1,
      anon_sym_RBRACK,
    STATE(6011), 2,
      sym_line_comment,
      sym_block_comment,
  [167148] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11667), 1,
      anon_sym_SEMI,
    STATE(6012), 2,
      sym_line_comment,
      sym_block_comment,
  [167162] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11669), 1,
      anon_sym_SEMI,
    STATE(6013), 2,
      sym_line_comment,
      sym_block_comment,
  [167176] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11671), 1,
      sym_identifier,
    STATE(6014), 2,
      sym_line_comment,
      sym_block_comment,
  [167190] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11673), 1,
      sym_identifier,
    STATE(6015), 2,
      sym_line_comment,
      sym_block_comment,
  [167204] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9525), 1,
      anon_sym_RPAREN,
    STATE(6016), 2,
      sym_line_comment,
      sym_block_comment,
  [167218] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11675), 1,
      anon_sym_SEMI,
    STATE(6017), 2,
      sym_line_comment,
      sym_block_comment,
  [167232] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11677), 1,
      anon_sym_SEMI,
    STATE(6018), 2,
      sym_line_comment,
      sym_block_comment,
  [167246] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11679), 1,
      anon_sym_SEMI,
    STATE(6019), 2,
      sym_line_comment,
      sym_block_comment,
  [167260] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11681), 1,
      anon_sym_SEMI,
    STATE(6020), 2,
      sym_line_comment,
      sym_block_comment,
  [167274] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11683), 1,
      anon_sym_COLON,
    STATE(6021), 2,
      sym_line_comment,
      sym_block_comment,
  [167288] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11685), 1,
      sym_identifier,
    STATE(6022), 2,
      sym_line_comment,
      sym_block_comment,
  [167302] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11687), 1,
      anon_sym_EQ_GT,
    STATE(6023), 2,
      sym_line_comment,
      sym_block_comment,
  [167316] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11689), 1,
      anon_sym_SEMI,
    STATE(6024), 2,
      sym_line_comment,
      sym_block_comment,
  [167330] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11691), 1,
      anon_sym_SEMI,
    STATE(6025), 2,
      sym_line_comment,
      sym_block_comment,
  [167344] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11693), 1,
      anon_sym_RBRACE,
    STATE(6026), 2,
      sym_line_comment,
      sym_block_comment,
  [167358] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8280), 1,
      anon_sym_SEMI,
    STATE(6027), 2,
      sym_line_comment,
      sym_block_comment,
  [167372] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11695), 1,
      anon_sym_SEMI,
    STATE(6028), 2,
      sym_line_comment,
      sym_block_comment,
  [167386] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11697), 1,
      anon_sym_SEMI,
    STATE(6029), 2,
      sym_line_comment,
      sym_block_comment,
  [167400] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9535), 1,
      anon_sym_RBRACK,
    STATE(6030), 2,
      sym_line_comment,
      sym_block_comment,
  [167414] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11699), 1,
      anon_sym_SEMI,
    STATE(6031), 2,
      sym_line_comment,
      sym_block_comment,
  [167428] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11355), 1,
      anon_sym_LBRACK,
    STATE(6032), 2,
      sym_line_comment,
      sym_block_comment,
  [167442] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11701), 1,
      sym_identifier,
    STATE(6033), 2,
      sym_line_comment,
      sym_block_comment,
  [167456] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11703), 1,
      sym_identifier,
    STATE(6034), 2,
      sym_line_comment,
      sym_block_comment,
  [167470] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11705), 1,
      anon_sym_RBRACK,
    STATE(6035), 2,
      sym_line_comment,
      sym_block_comment,
  [167484] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11707), 1,
      sym_identifier,
    STATE(6036), 2,
      sym_line_comment,
      sym_block_comment,
  [167498] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8486), 1,
      anon_sym_GT,
    STATE(6037), 2,
      sym_line_comment,
      sym_block_comment,
  [167512] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11709), 1,
      anon_sym_RBRACK,
    STATE(6038), 2,
      sym_line_comment,
      sym_block_comment,
  [167526] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11711), 1,
      anon_sym_SEMI,
    STATE(6039), 2,
      sym_line_comment,
      sym_block_comment,
  [167540] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11713), 1,
      anon_sym_SEMI,
    STATE(6040), 2,
      sym_line_comment,
      sym_block_comment,
  [167554] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8286), 1,
      anon_sym_SEMI,
    STATE(6041), 2,
      sym_line_comment,
      sym_block_comment,
  [167568] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11715), 1,
      sym_identifier,
    STATE(6042), 2,
      sym_line_comment,
      sym_block_comment,
  [167582] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11717), 1,
      anon_sym_COLON,
    STATE(6043), 2,
      sym_line_comment,
      sym_block_comment,
  [167596] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11719), 1,
      anon_sym_SEMI,
    STATE(6044), 2,
      sym_line_comment,
      sym_block_comment,
  [167610] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11721), 1,
      anon_sym_LBRACE,
    STATE(6045), 2,
      sym_line_comment,
      sym_block_comment,
  [167624] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11723), 1,
      anon_sym_RBRACK,
    STATE(6046), 2,
      sym_line_comment,
      sym_block_comment,
  [167638] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11725), 1,
      anon_sym_COLON,
    STATE(6047), 2,
      sym_line_comment,
      sym_block_comment,
  [167652] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11727), 1,
      anon_sym_SEMI,
    STATE(6048), 2,
      sym_line_comment,
      sym_block_comment,
  [167666] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11729), 1,
      sym__line_doc_content,
    STATE(6049), 2,
      sym_line_comment,
      sym_block_comment,
  [167680] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11731), 1,
      sym_identifier,
    STATE(6050), 2,
      sym_line_comment,
      sym_block_comment,
  [167694] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11733), 1,
      sym__line_doc_content,
    STATE(6051), 2,
      sym_line_comment,
      sym_block_comment,
  [167708] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11735), 1,
      anon_sym_RPAREN,
    STATE(6052), 2,
      sym_line_comment,
      sym_block_comment,
  [167722] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11737), 1,
      anon_sym_COLON,
    STATE(6053), 2,
      sym_line_comment,
      sym_block_comment,
  [167736] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11739), 1,
      sym_identifier,
    STATE(6054), 2,
      sym_line_comment,
      sym_block_comment,
  [167750] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11741), 1,
      anon_sym_SEMI,
    STATE(6055), 2,
      sym_line_comment,
      sym_block_comment,
  [167764] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11743), 1,
      sym_identifier,
    STATE(6056), 2,
      sym_line_comment,
      sym_block_comment,
  [167778] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11745), 1,
      anon_sym_SEMI,
    STATE(6057), 2,
      sym_line_comment,
      sym_block_comment,
  [167792] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11747), 1,
      anon_sym_SEMI,
    STATE(6058), 2,
      sym_line_comment,
      sym_block_comment,
  [167806] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11749), 1,
      anon_sym_SEMI,
    STATE(6059), 2,
      sym_line_comment,
      sym_block_comment,
  [167820] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11751), 1,
      sym_identifier,
    STATE(6060), 2,
      sym_line_comment,
      sym_block_comment,
  [167834] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11753), 1,
      anon_sym_RPAREN,
    STATE(6061), 2,
      sym_line_comment,
      sym_block_comment,
  [167848] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11755), 1,
      anon_sym_SEMI,
    STATE(6062), 2,
      sym_line_comment,
      sym_block_comment,
  [167862] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11757), 1,
      anon_sym_COLON,
    STATE(6063), 2,
      sym_line_comment,
      sym_block_comment,
  [167876] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11759), 1,
      anon_sym_SEMI,
    STATE(6064), 2,
      sym_line_comment,
      sym_block_comment,
  [167890] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11761), 1,
      sym_identifier,
    STATE(6065), 2,
      sym_line_comment,
      sym_block_comment,
  [167904] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11763), 1,
      anon_sym_SEMI,
    STATE(6066), 2,
      sym_line_comment,
      sym_block_comment,
  [167918] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5208), 1,
      anon_sym_POUND,
    STATE(6067), 2,
      sym_line_comment,
      sym_block_comment,
  [167932] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11765), 1,
      sym_identifier,
    STATE(6068), 2,
      sym_line_comment,
      sym_block_comment,
  [167946] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6753), 1,
      anon_sym_RPAREN,
    STATE(6069), 2,
      sym_line_comment,
      sym_block_comment,
  [167960] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11767), 1,
      anon_sym_RBRACE,
    STATE(6070), 2,
      sym_line_comment,
      sym_block_comment,
  [167974] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11769), 1,
      anon_sym_RBRACK,
    STATE(6071), 2,
      sym_line_comment,
      sym_block_comment,
  [167988] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11771), 1,
      sym_identifier,
    STATE(6072), 2,
      sym_line_comment,
      sym_block_comment,
  [168002] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11773), 1,
      anon_sym_SEMI,
    STATE(6073), 2,
      sym_line_comment,
      sym_block_comment,
  [168016] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11775), 1,
      anon_sym_SEMI,
    STATE(6074), 2,
      sym_line_comment,
      sym_block_comment,
  [168030] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11777), 1,
      anon_sym_RPAREN,
    STATE(6075), 2,
      sym_line_comment,
      sym_block_comment,
  [168044] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11779), 1,
      anon_sym_SEMI,
    STATE(6076), 2,
      sym_line_comment,
      sym_block_comment,
  [168058] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11781), 1,
      anon_sym_SEMI,
    STATE(6077), 2,
      sym_line_comment,
      sym_block_comment,
  [168072] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8539), 1,
      anon_sym_GT,
    STATE(6078), 2,
      sym_line_comment,
      sym_block_comment,
  [168086] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11783), 1,
      anon_sym_SEMI,
    STATE(6079), 2,
      sym_line_comment,
      sym_block_comment,
  [168100] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11785), 1,
      anon_sym_SEMI,
    STATE(6080), 2,
      sym_line_comment,
      sym_block_comment,
  [168114] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10092), 1,
      anon_sym_RBRACE,
    STATE(6081), 2,
      sym_line_comment,
      sym_block_comment,
  [168128] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11787), 1,
      anon_sym_fn,
    STATE(6082), 2,
      sym_line_comment,
      sym_block_comment,
  [168142] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11789), 1,
      anon_sym_SEMI,
    STATE(6083), 2,
      sym_line_comment,
      sym_block_comment,
  [168156] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11791), 1,
      anon_sym_RBRACE,
    STATE(6084), 2,
      sym_line_comment,
      sym_block_comment,
  [168170] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11793), 1,
      anon_sym_LPAREN,
    STATE(6085), 2,
      sym_line_comment,
      sym_block_comment,
  [168184] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11795), 1,
      anon_sym_SEMI,
    STATE(6086), 2,
      sym_line_comment,
      sym_block_comment,
  [168198] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11797), 1,
      anon_sym_SEMI,
    STATE(6087), 2,
      sym_line_comment,
      sym_block_comment,
  [168212] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10598), 1,
      anon_sym_PIPE,
    STATE(6088), 2,
      sym_line_comment,
      sym_block_comment,
  [168226] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11799), 1,
      anon_sym_SEMI,
    STATE(6089), 2,
      sym_line_comment,
      sym_block_comment,
  [168240] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11801), 1,
      sym_identifier,
    STATE(6090), 2,
      sym_line_comment,
      sym_block_comment,
  [168254] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11803), 1,
      anon_sym_SEMI,
    STATE(6091), 2,
      sym_line_comment,
      sym_block_comment,
  [168268] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11805), 1,
      anon_sym_COLON,
    STATE(6092), 2,
      sym_line_comment,
      sym_block_comment,
  [168282] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11807), 1,
      sym_identifier,
    STATE(6093), 2,
      sym_line_comment,
      sym_block_comment,
  [168296] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11809), 1,
      anon_sym_SEMI,
    STATE(6094), 2,
      sym_line_comment,
      sym_block_comment,
  [168310] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11811), 1,
      anon_sym_SEMI,
    STATE(6095), 2,
      sym_line_comment,
      sym_block_comment,
  [168324] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11813), 1,
      anon_sym_SEMI,
    STATE(6096), 2,
      sym_line_comment,
      sym_block_comment,
  [168338] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11815), 1,
      anon_sym_RBRACE,
    STATE(6097), 2,
      sym_line_comment,
      sym_block_comment,
  [168352] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11817), 1,
      sym_identifier,
    STATE(6098), 2,
      sym_line_comment,
      sym_block_comment,
  [168366] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11819), 1,
      anon_sym_COLON,
    STATE(6099), 2,
      sym_line_comment,
      sym_block_comment,
  [168380] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11821), 1,
      sym_identifier,
    STATE(6100), 2,
      sym_line_comment,
      sym_block_comment,
  [168394] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11823), 1,
      sym_raw_string_literal_content,
    STATE(6101), 2,
      sym_line_comment,
      sym_block_comment,
  [168408] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11825), 1,
      sym_identifier,
    STATE(6102), 2,
      sym_line_comment,
      sym_block_comment,
  [168422] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8156), 1,
      anon_sym_SEMI,
    STATE(6103), 2,
      sym_line_comment,
      sym_block_comment,
  [168436] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11827), 1,
      anon_sym_SEMI,
    STATE(6104), 2,
      sym_line_comment,
      sym_block_comment,
  [168450] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11829), 1,
      anon_sym_COLON_COLON,
    STATE(6105), 2,
      sym_line_comment,
      sym_block_comment,
  [168464] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11831), 1,
      anon_sym_RBRACK,
    STATE(6106), 2,
      sym_line_comment,
      sym_block_comment,
  [168478] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11833), 1,
      sym_identifier,
    STATE(6107), 2,
      sym_line_comment,
      sym_block_comment,
  [168492] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11835), 1,
      anon_sym_SEMI,
    STATE(6108), 2,
      sym_line_comment,
      sym_block_comment,
  [168506] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11837), 1,
      anon_sym_SEMI,
    STATE(6109), 2,
      sym_line_comment,
      sym_block_comment,
  [168520] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11839), 1,
      sym__line_doc_content,
    STATE(6110), 2,
      sym_line_comment,
      sym_block_comment,
  [168534] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11841), 1,
      sym__line_doc_content,
    STATE(6111), 2,
      sym_line_comment,
      sym_block_comment,
  [168548] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1372), 1,
      anon_sym_EQ_GT,
    STATE(6112), 2,
      sym_line_comment,
      sym_block_comment,
  [168562] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11843), 1,
      anon_sym_COLON,
    STATE(6113), 2,
      sym_line_comment,
      sym_block_comment,
  [168576] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7544), 1,
      anon_sym_COLON_COLON,
    STATE(6114), 2,
      sym_line_comment,
      sym_block_comment,
  [168590] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11845), 1,
      sym_identifier,
    STATE(6115), 2,
      sym_line_comment,
      sym_block_comment,
  [168604] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5927), 1,
      sym_identifier,
    STATE(6116), 2,
      sym_line_comment,
      sym_block_comment,
  [168618] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11847), 1,
      sym_identifier,
    STATE(6117), 2,
      sym_line_comment,
      sym_block_comment,
  [168632] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7650), 1,
      anon_sym_fn,
    STATE(6118), 2,
      sym_line_comment,
      sym_block_comment,
  [168646] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11849), 1,
      sym_identifier,
    STATE(6119), 2,
      sym_line_comment,
      sym_block_comment,
  [168660] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10285), 1,
      anon_sym_GT,
    STATE(6120), 2,
      sym_line_comment,
      sym_block_comment,
  [168674] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11851), 1,
      anon_sym_SEMI,
    STATE(6121), 2,
      sym_line_comment,
      sym_block_comment,
  [168688] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11853), 1,
      anon_sym_SEMI,
    STATE(6122), 2,
      sym_line_comment,
      sym_block_comment,
  [168702] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11855), 1,
      sym_identifier,
    STATE(6123), 2,
      sym_line_comment,
      sym_block_comment,
  [168716] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11857), 1,
      anon_sym_RBRACK,
    STATE(6124), 2,
      sym_line_comment,
      sym_block_comment,
  [168730] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11859), 1,
      anon_sym_SEMI,
    STATE(6125), 2,
      sym_line_comment,
      sym_block_comment,
  [168744] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11861), 1,
      anon_sym_COLON,
    STATE(6126), 2,
      sym_line_comment,
      sym_block_comment,
  [168758] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11863), 1,
      sym_identifier,
    STATE(6127), 2,
      sym_line_comment,
      sym_block_comment,
  [168772] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11865), 1,
      ts_builtin_sym_end,
    STATE(6128), 2,
      sym_line_comment,
      sym_block_comment,
  [168786] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11867), 1,
      anon_sym_SEMI,
    STATE(6129), 2,
      sym_line_comment,
      sym_block_comment,
  [168800] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11869), 1,
      anon_sym_SEMI,
    STATE(6130), 2,
      sym_line_comment,
      sym_block_comment,
  [168814] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11871), 1,
      anon_sym_SEMI,
    STATE(6131), 2,
      sym_line_comment,
      sym_block_comment,
  [168828] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9718), 1,
      anon_sym_RBRACE,
    STATE(6132), 2,
      sym_line_comment,
      sym_block_comment,
  [168842] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11873), 1,
      anon_sym_SEMI,
    STATE(6133), 2,
      sym_line_comment,
      sym_block_comment,
  [168856] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11875), 1,
      anon_sym_SEMI,
    STATE(6134), 2,
      sym_line_comment,
      sym_block_comment,
  [168870] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8160), 1,
      anon_sym_SEMI,
    STATE(6135), 2,
      sym_line_comment,
      sym_block_comment,
  [168884] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8942), 1,
      anon_sym_RPAREN,
    STATE(6136), 2,
      sym_line_comment,
      sym_block_comment,
  [168898] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11877), 1,
      sym_identifier,
    STATE(6137), 2,
      sym_line_comment,
      sym_block_comment,
  [168912] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11879), 1,
      sym_identifier,
    STATE(6138), 2,
      sym_line_comment,
      sym_block_comment,
  [168926] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10275), 1,
      anon_sym_GT,
    STATE(6139), 2,
      sym_line_comment,
      sym_block_comment,
  [168940] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11881), 1,
      sym__raw_string_literal_end,
    STATE(6140), 2,
      sym_line_comment,
      sym_block_comment,
  [168954] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11883), 1,
      anon_sym_STAR_SLASH,
    STATE(6141), 2,
      sym_line_comment,
      sym_block_comment,
  [168968] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11885), 1,
      anon_sym_fn,
    STATE(6142), 2,
      sym_line_comment,
      sym_block_comment,
  [168982] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11887), 1,
      sym_identifier,
    STATE(6143), 2,
      sym_line_comment,
      sym_block_comment,
  [168996] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11889), 1,
      sym__raw_string_literal_end,
    STATE(6144), 2,
      sym_line_comment,
      sym_block_comment,
  [169010] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11891), 1,
      anon_sym_SEMI,
    STATE(6145), 2,
      sym_line_comment,
      sym_block_comment,
  [169024] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11893), 1,
      anon_sym_RPAREN,
    STATE(6146), 2,
      sym_line_comment,
      sym_block_comment,
  [169038] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11895), 1,
      anon_sym_STAR_SLASH,
    STATE(6147), 2,
      sym_line_comment,
      sym_block_comment,
  [169052] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5335), 1,
      anon_sym_PLUS,
    STATE(6148), 2,
      sym_line_comment,
      sym_block_comment,
  [169066] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11897), 1,
      anon_sym_SEMI,
    STATE(6149), 2,
      sym_line_comment,
      sym_block_comment,
  [169080] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11899), 1,
      sym_identifier,
    STATE(6150), 2,
      sym_line_comment,
      sym_block_comment,
  [169094] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11901), 1,
      sym_identifier,
    STATE(6151), 2,
      sym_line_comment,
      sym_block_comment,
  [169108] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11387), 1,
      anon_sym_SEMI,
    STATE(6152), 2,
      sym_line_comment,
      sym_block_comment,
  [169122] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11903), 1,
      sym_identifier,
    STATE(6153), 2,
      sym_line_comment,
      sym_block_comment,
  [169136] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5353), 1,
      anon_sym_PLUS,
    STATE(6154), 2,
      sym_line_comment,
      sym_block_comment,
  [169150] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11905), 1,
      anon_sym_COLON,
    STATE(6155), 2,
      sym_line_comment,
      sym_block_comment,
  [169164] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8240), 1,
      anon_sym_COLON_COLON,
    STATE(6156), 2,
      sym_line_comment,
      sym_block_comment,
  [169178] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11907), 1,
      sym_identifier,
    STATE(6157), 2,
      sym_line_comment,
      sym_block_comment,
  [169192] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11389), 1,
      anon_sym_SEMI,
    STATE(6158), 2,
      sym_line_comment,
      sym_block_comment,
  [169206] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10104), 1,
      anon_sym_RBRACE,
    STATE(6159), 2,
      sym_line_comment,
      sym_block_comment,
  [169220] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11909), 1,
      sym_identifier,
    STATE(6160), 2,
      sym_line_comment,
      sym_block_comment,
  [169234] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11911), 1,
      anon_sym_COLON,
    STATE(6161), 2,
      sym_line_comment,
      sym_block_comment,
  [169248] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11913), 1,
      anon_sym_SEMI,
    STATE(6162), 2,
      sym_line_comment,
      sym_block_comment,
  [169262] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11915), 1,
      anon_sym_SEMI,
    STATE(6163), 2,
      sym_line_comment,
      sym_block_comment,
  [169276] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8104), 1,
      anon_sym_SEMI,
    STATE(6164), 2,
      sym_line_comment,
      sym_block_comment,
  [169290] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11917), 1,
      anon_sym_SEMI,
    STATE(6165), 2,
      sym_line_comment,
      sym_block_comment,
  [169304] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11919), 1,
      anon_sym_LBRACK,
    STATE(6166), 2,
      sym_line_comment,
      sym_block_comment,
  [169318] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11921), 1,
      anon_sym_SEMI,
    STATE(6167), 2,
      sym_line_comment,
      sym_block_comment,
  [169332] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11923), 1,
      sym__raw_string_literal_end,
    STATE(6168), 2,
      sym_line_comment,
      sym_block_comment,
  [169346] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11925), 1,
      anon_sym_SEMI,
    STATE(6169), 2,
      sym_line_comment,
      sym_block_comment,
  [169360] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11927), 1,
      sym_raw_string_literal_content,
    STATE(6170), 2,
      sym_line_comment,
      sym_block_comment,
  [169374] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11929), 1,
      anon_sym_SEMI,
    STATE(6171), 2,
      sym_line_comment,
      sym_block_comment,
  [169388] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11931), 1,
      anon_sym_RBRACK,
    STATE(6172), 2,
      sym_line_comment,
      sym_block_comment,
  [169402] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11933), 1,
      sym_identifier,
    STATE(6173), 2,
      sym_line_comment,
      sym_block_comment,
  [169416] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11935), 1,
      anon_sym_EQ_GT,
    STATE(6174), 2,
      sym_line_comment,
      sym_block_comment,
  [169430] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11937), 1,
      anon_sym_RBRACK,
    STATE(6175), 2,
      sym_line_comment,
      sym_block_comment,
  [169444] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11939), 1,
      sym_identifier,
    STATE(6176), 2,
      sym_line_comment,
      sym_block_comment,
  [169458] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9545), 1,
      anon_sym_RBRACK,
    STATE(6177), 2,
      sym_line_comment,
      sym_block_comment,
  [169472] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10562), 1,
      anon_sym_RBRACE,
    STATE(6178), 2,
      sym_line_comment,
      sym_block_comment,
  [169486] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11941), 1,
      anon_sym_SEMI,
    STATE(6179), 2,
      sym_line_comment,
      sym_block_comment,
  [169500] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11943), 1,
      anon_sym_SEMI,
    STATE(6180), 2,
      sym_line_comment,
      sym_block_comment,
  [169514] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11945), 1,
      anon_sym_SEMI,
    STATE(6181), 2,
      sym_line_comment,
      sym_block_comment,
  [169528] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11947), 1,
      sym_identifier,
    STATE(6182), 2,
      sym_line_comment,
      sym_block_comment,
  [169542] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11949), 1,
      sym_identifier,
    STATE(6183), 2,
      sym_line_comment,
      sym_block_comment,
  [169556] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11951), 1,
      sym_identifier,
    STATE(6184), 2,
      sym_line_comment,
      sym_block_comment,
  [169570] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11953), 1,
      sym_identifier,
    STATE(6185), 2,
      sym_line_comment,
      sym_block_comment,
  [169584] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11955), 1,
      anon_sym_SEMI,
    STATE(6186), 2,
      sym_line_comment,
      sym_block_comment,
  [169598] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11957), 1,
      anon_sym_SEMI,
    STATE(6187), 2,
      sym_line_comment,
      sym_block_comment,
  [169612] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11051), 1,
      anon_sym_SEMI,
    STATE(6188), 2,
      sym_line_comment,
      sym_block_comment,
  [169626] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11959), 1,
      sym_identifier,
    STATE(6189), 2,
      sym_line_comment,
      sym_block_comment,
  [169640] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11961), 1,
      anon_sym_SEMI,
    STATE(6190), 2,
      sym_line_comment,
      sym_block_comment,
  [169654] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11963), 1,
      anon_sym_SEMI,
    STATE(6191), 2,
      sym_line_comment,
      sym_block_comment,
  [169668] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11965), 1,
      anon_sym_SEMI,
    STATE(6192), 2,
      sym_line_comment,
      sym_block_comment,
  [169682] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11967), 1,
      sym_identifier,
    STATE(6193), 2,
      sym_line_comment,
      sym_block_comment,
  [169696] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7333), 1,
      anon_sym_COLON_COLON,
    STATE(6194), 2,
      sym_line_comment,
      sym_block_comment,
  [169710] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11969), 1,
      sym_identifier,
    STATE(6195), 2,
      sym_line_comment,
      sym_block_comment,
  [169724] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11971), 1,
      anon_sym_SEMI,
    STATE(6196), 2,
      sym_line_comment,
      sym_block_comment,
  [169738] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11973), 1,
      anon_sym_SEMI,
    STATE(6197), 2,
      sym_line_comment,
      sym_block_comment,
  [169752] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11367), 1,
      sym_self,
    STATE(6198), 2,
      sym_line_comment,
      sym_block_comment,
  [169766] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11975), 1,
      sym_identifier,
    STATE(6199), 2,
      sym_line_comment,
      sym_block_comment,
  [169780] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11977), 1,
      anon_sym_SEMI,
    STATE(6200), 2,
      sym_line_comment,
      sym_block_comment,
  [169794] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11979), 1,
      anon_sym_SEMI,
    STATE(6201), 2,
      sym_line_comment,
      sym_block_comment,
  [169808] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11981), 1,
      sym_identifier,
    STATE(6202), 2,
      sym_line_comment,
      sym_block_comment,
  [169822] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10338), 1,
      anon_sym_RBRACE,
    STATE(6203), 2,
      sym_line_comment,
      sym_block_comment,
  [169836] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11983), 1,
      anon_sym_RPAREN,
    STATE(6204), 2,
      sym_line_comment,
      sym_block_comment,
  [169850] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11985), 1,
      anon_sym_SEMI,
    STATE(6205), 2,
      sym_line_comment,
      sym_block_comment,
  [169864] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11987), 1,
      anon_sym_SEMI,
    STATE(6206), 2,
      sym_line_comment,
      sym_block_comment,
  [169878] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11989), 1,
      anon_sym_COLON_COLON,
    STATE(6207), 2,
      sym_line_comment,
      sym_block_comment,
  [169892] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11991), 1,
      anon_sym_SEMI,
    STATE(6208), 2,
      sym_line_comment,
      sym_block_comment,
  [169906] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11993), 1,
      anon_sym_SEMI,
    STATE(6209), 2,
      sym_line_comment,
      sym_block_comment,
  [169920] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11995), 1,
      sym_raw_string_literal_content,
    STATE(6210), 2,
      sym_line_comment,
      sym_block_comment,
  [169934] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11997), 1,
      anon_sym_SEMI,
    STATE(6211), 2,
      sym_line_comment,
      sym_block_comment,
  [169948] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11999), 1,
      anon_sym_SEMI,
    STATE(6212), 2,
      sym_line_comment,
      sym_block_comment,
  [169962] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12001), 1,
      anon_sym_SEMI,
    STATE(6213), 2,
      sym_line_comment,
      sym_block_comment,
  [169976] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12003), 1,
      anon_sym_SEMI,
    STATE(6214), 2,
      sym_line_comment,
      sym_block_comment,
  [169990] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6751), 1,
      anon_sym_COLON_COLON,
    STATE(6215), 2,
      sym_line_comment,
      sym_block_comment,
  [170004] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9216), 1,
      anon_sym_COLON_COLON,
    STATE(6216), 2,
      sym_line_comment,
      sym_block_comment,
  [170018] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12005), 1,
      anon_sym_SEMI,
    STATE(6217), 2,
      sym_line_comment,
      sym_block_comment,
  [170032] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12007), 1,
      anon_sym_SEMI,
    STATE(6218), 2,
      sym_line_comment,
      sym_block_comment,
  [170046] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12009), 1,
      anon_sym_SEMI,
    STATE(6219), 2,
      sym_line_comment,
      sym_block_comment,
  [170060] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12011), 1,
      anon_sym_COLON_COLON,
    STATE(6220), 2,
      sym_line_comment,
      sym_block_comment,
  [170074] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12013), 1,
      anon_sym_SEMI,
    STATE(6221), 2,
      sym_line_comment,
      sym_block_comment,
  [170088] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6679), 1,
      anon_sym_COLON_COLON,
    STATE(6222), 2,
      sym_line_comment,
      sym_block_comment,
  [170102] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12015), 1,
      anon_sym_SEMI,
    STATE(6223), 2,
      sym_line_comment,
      sym_block_comment,
  [170116] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12017), 1,
      sym_identifier,
    STATE(6224), 2,
      sym_line_comment,
      sym_block_comment,
  [170130] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12019), 1,
      anon_sym_SEMI,
    STATE(6225), 2,
      sym_line_comment,
      sym_block_comment,
  [170144] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7227), 1,
      anon_sym_COLON_COLON,
    STATE(6226), 2,
      sym_line_comment,
      sym_block_comment,
  [170158] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12021), 1,
      anon_sym_LT2,
    STATE(6227), 2,
      sym_line_comment,
      sym_block_comment,
  [170172] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12023), 1,
      anon_sym_fn,
    STATE(6228), 2,
      sym_line_comment,
      sym_block_comment,
  [170186] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12025), 1,
      anon_sym_COLON_COLON,
    STATE(6229), 2,
      sym_line_comment,
      sym_block_comment,
  [170200] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12027), 1,
      anon_sym_LT2,
    STATE(6230), 2,
      sym_line_comment,
      sym_block_comment,
  [170214] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12029), 1,
      anon_sym_fn,
    STATE(6231), 2,
      sym_line_comment,
      sym_block_comment,
  [170228] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12031), 1,
      anon_sym_COLON,
    STATE(6232), 2,
      sym_line_comment,
      sym_block_comment,
  [170242] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12033), 1,
      anon_sym_COLON_COLON,
    STATE(6233), 2,
      sym_line_comment,
      sym_block_comment,
  [170256] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7352), 1,
      anon_sym_COLON_COLON,
    STATE(6234), 2,
      sym_line_comment,
      sym_block_comment,
  [170270] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12035), 1,
      anon_sym_SEMI,
    STATE(6235), 2,
      sym_line_comment,
      sym_block_comment,
  [170284] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12037), 1,
      sym_identifier,
    STATE(6236), 2,
      sym_line_comment,
      sym_block_comment,
  [170298] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12039), 1,
      sym_identifier,
    STATE(6237), 2,
      sym_line_comment,
      sym_block_comment,
  [170312] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12041), 1,
      anon_sym_COLON_COLON,
    STATE(6238), 2,
      sym_line_comment,
      sym_block_comment,
  [170326] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12043), 1,
      anon_sym_COLON_COLON,
    STATE(6239), 2,
      sym_line_comment,
      sym_block_comment,
  [170340] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(696), 1,
      anon_sym_RBRACK,
    STATE(6240), 2,
      sym_line_comment,
      sym_block_comment,
  [170354] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12045), 1,
      sym_identifier,
    STATE(6241), 2,
      sym_line_comment,
      sym_block_comment,
  [170368] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12047), 1,
      sym_self,
    STATE(6242), 2,
      sym_line_comment,
      sym_block_comment,
  [170382] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12049), 1,
      anon_sym_LBRACE,
    STATE(6243), 2,
      sym_line_comment,
      sym_block_comment,
  [170396] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12051), 1,
      anon_sym_EQ_GT,
    STATE(6244), 2,
      sym_line_comment,
      sym_block_comment,
  [170410] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12053), 1,
      anon_sym_COLON,
    STATE(6245), 2,
      sym_line_comment,
      sym_block_comment,
  [170424] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12055), 1,
      anon_sym_SEMI,
    STATE(6246), 2,
      sym_line_comment,
      sym_block_comment,
  [170438] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12057), 1,
      sym_identifier,
    STATE(6247), 2,
      sym_line_comment,
      sym_block_comment,
  [170452] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12059), 1,
      anon_sym_SEMI,
    STATE(6248), 2,
      sym_line_comment,
      sym_block_comment,
  [170466] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12061), 1,
      anon_sym_SEMI,
    STATE(6249), 2,
      sym_line_comment,
      sym_block_comment,
  [170480] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9583), 1,
      anon_sym_RPAREN,
    STATE(6250), 2,
      sym_line_comment,
      sym_block_comment,
  [170494] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8353), 1,
      anon_sym_GT,
    STATE(6251), 2,
      sym_line_comment,
      sym_block_comment,
  [170508] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12063), 1,
      sym_identifier,
    STATE(6252), 2,
      sym_line_comment,
      sym_block_comment,
  [170522] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12065), 1,
      anon_sym_LT2,
    STATE(6253), 2,
      sym_line_comment,
      sym_block_comment,
  [170536] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12067), 1,
      anon_sym_SEMI,
    STATE(6254), 2,
      sym_line_comment,
      sym_block_comment,
  [170550] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8637), 1,
      anon_sym_RPAREN,
    STATE(6255), 2,
      sym_line_comment,
      sym_block_comment,
  [170564] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12069), 1,
      anon_sym_SEMI,
    STATE(6256), 2,
      sym_line_comment,
      sym_block_comment,
  [170578] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12071), 1,
      anon_sym_fn,
    STATE(6257), 2,
      sym_line_comment,
      sym_block_comment,
  [170592] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5291), 1,
      anon_sym_PLUS,
    STATE(6258), 2,
      sym_line_comment,
      sym_block_comment,
  [170606] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12073), 1,
      anon_sym_SEMI,
    STATE(6259), 2,
      sym_line_comment,
      sym_block_comment,
  [170620] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12075), 1,
      sym_raw_string_literal_content,
    STATE(6260), 2,
      sym_line_comment,
      sym_block_comment,
  [170634] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12077), 1,
      anon_sym_SEMI,
    STATE(6261), 2,
      sym_line_comment,
      sym_block_comment,
  [170648] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12079), 1,
      anon_sym_COLON,
    STATE(6262), 2,
      sym_line_comment,
      sym_block_comment,
  [170662] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12081), 1,
      anon_sym_SEMI,
    STATE(6263), 2,
      sym_line_comment,
      sym_block_comment,
  [170676] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12083), 1,
      anon_sym_COLON_COLON,
    STATE(6264), 2,
      sym_line_comment,
      sym_block_comment,
  [170690] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6739), 1,
      anon_sym_RPAREN,
    STATE(6265), 2,
      sym_line_comment,
      sym_block_comment,
  [170704] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12085), 1,
      anon_sym_SEMI,
    STATE(6266), 2,
      sym_line_comment,
      sym_block_comment,
  [170718] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12087), 1,
      anon_sym_COLON_COLON,
    STATE(6267), 2,
      sym_line_comment,
      sym_block_comment,
  [170732] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12089), 1,
      anon_sym_RBRACK,
    STATE(6268), 2,
      sym_line_comment,
      sym_block_comment,
  [170746] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12091), 1,
      sym__raw_string_literal_end,
    STATE(6269), 2,
      sym_line_comment,
      sym_block_comment,
  [170760] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12093), 1,
      anon_sym_RBRACK,
    STATE(6270), 2,
      sym_line_comment,
      sym_block_comment,
  [170774] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1388), 1,
      anon_sym_EQ_GT,
    STATE(6271), 2,
      sym_line_comment,
      sym_block_comment,
  [170788] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12095), 1,
      anon_sym_COLON_COLON,
    STATE(6272), 2,
      sym_line_comment,
      sym_block_comment,
  [170802] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12097), 1,
      anon_sym_COLON_COLON,
    STATE(6273), 2,
      sym_line_comment,
      sym_block_comment,
  [170816] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12099), 1,
      anon_sym_LBRACE,
    STATE(6274), 2,
      sym_line_comment,
      sym_block_comment,
  [170830] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12101), 1,
      anon_sym_SEMI,
    STATE(6275), 2,
      sym_line_comment,
      sym_block_comment,
  [170844] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12103), 1,
      sym_identifier,
    STATE(6276), 2,
      sym_line_comment,
      sym_block_comment,
  [170858] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12105), 1,
      anon_sym_SEMI,
    STATE(6277), 2,
      sym_line_comment,
      sym_block_comment,
  [170872] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12107), 1,
      anon_sym_COLON,
    STATE(6278), 2,
      sym_line_comment,
      sym_block_comment,
  [170886] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11023), 1,
      anon_sym_COLON_COLON,
    STATE(6279), 2,
      sym_line_comment,
      sym_block_comment,
  [170900] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12109), 1,
      sym_raw_string_literal_content,
    STATE(6280), 2,
      sym_line_comment,
      sym_block_comment,
  [170914] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6151), 1,
      anon_sym_COLON_COLON,
    STATE(6281), 2,
      sym_line_comment,
      sym_block_comment,
  [170928] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7183), 1,
      anon_sym_fn,
    STATE(6282), 2,
      sym_line_comment,
      sym_block_comment,
  [170942] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9354), 1,
      anon_sym_LBRACE,
    STATE(6283), 2,
      sym_line_comment,
      sym_block_comment,
  [170956] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12111), 1,
      anon_sym_LBRACE,
    STATE(6284), 2,
      sym_line_comment,
      sym_block_comment,
  [170970] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12113), 1,
      sym_raw_string_literal_content,
    STATE(6285), 2,
      sym_line_comment,
      sym_block_comment,
  [170984] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8182), 1,
      anon_sym_COLON_COLON,
    STATE(6286), 2,
      sym_line_comment,
      sym_block_comment,
  [170998] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12115), 1,
      anon_sym_COLON,
    STATE(6287), 2,
      sym_line_comment,
      sym_block_comment,
  [171012] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12117), 1,
      anon_sym_LBRACE,
    STATE(6288), 2,
      sym_line_comment,
      sym_block_comment,
  [171026] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9148), 1,
      anon_sym_COLON_COLON,
    STATE(6289), 2,
      sym_line_comment,
      sym_block_comment,
  [171040] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9587), 1,
      anon_sym_LBRACE,
    STATE(6290), 2,
      sym_line_comment,
      sym_block_comment,
  [171054] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12119), 1,
      anon_sym_COLON_COLON,
    STATE(6291), 2,
      sym_line_comment,
      sym_block_comment,
  [171068] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9603), 1,
      anon_sym_LBRACE,
    STATE(6292), 2,
      sym_line_comment,
      sym_block_comment,
  [171082] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8180), 1,
      anon_sym_COLON_COLON,
    STATE(6293), 2,
      sym_line_comment,
      sym_block_comment,
  [171096] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8246), 1,
      anon_sym_COLON_COLON,
    STATE(6294), 2,
      sym_line_comment,
      sym_block_comment,
  [171110] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(6743), 1,
      anon_sym_COLON_COLON,
    STATE(6295), 2,
      sym_line_comment,
      sym_block_comment,
  [171124] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12121), 1,
      anon_sym_COLON_COLON,
    STATE(6296), 2,
      sym_line_comment,
      sym_block_comment,
  [171138] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12123), 1,
      anon_sym_COLON,
    STATE(6297), 2,
      sym_line_comment,
      sym_block_comment,
  [171152] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12125), 1,
      sym_identifier,
    STATE(6298), 2,
      sym_line_comment,
      sym_block_comment,
  [171166] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12127), 1,
      anon_sym_COLON_COLON,
    STATE(6299), 2,
      sym_line_comment,
      sym_block_comment,
  [171180] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12129), 1,
      anon_sym_COLON,
    STATE(6300), 2,
      sym_line_comment,
      sym_block_comment,
  [171194] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11529), 1,
      anon_sym_LBRACK,
    STATE(6301), 2,
      sym_line_comment,
      sym_block_comment,
  [171208] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12131), 1,
      sym_identifier,
    STATE(6302), 2,
      sym_line_comment,
      sym_block_comment,
  [171222] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12133), 1,
      anon_sym_SEMI,
    STATE(6303), 2,
      sym_line_comment,
      sym_block_comment,
  [171236] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12135), 1,
      sym_identifier,
    STATE(6304), 2,
      sym_line_comment,
      sym_block_comment,
  [171250] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12137), 1,
      anon_sym_RBRACK,
    STATE(6305), 2,
      sym_line_comment,
      sym_block_comment,
  [171264] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12139), 1,
      anon_sym_trait,
    STATE(6306), 2,
      sym_line_comment,
      sym_block_comment,
  [171278] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12141), 1,
      anon_sym_LBRACK,
    STATE(6307), 2,
      sym_line_comment,
      sym_block_comment,
  [171292] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12143), 1,
      anon_sym_COLON,
    STATE(6308), 2,
      sym_line_comment,
      sym_block_comment,
  [171306] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12145), 1,
      anon_sym_COLON,
    STATE(6309), 2,
      sym_line_comment,
      sym_block_comment,
  [171320] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12147), 1,
      sym_identifier,
    STATE(6310), 2,
      sym_line_comment,
      sym_block_comment,
  [171334] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12149), 1,
      anon_sym_STAR_SLASH,
    STATE(6311), 2,
      sym_line_comment,
      sym_block_comment,
  [171348] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12151), 1,
      anon_sym_SEMI,
    STATE(6312), 2,
      sym_line_comment,
      sym_block_comment,
  [171362] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12153), 1,
      sym_identifier,
    STATE(6313), 2,
      sym_line_comment,
      sym_block_comment,
  [171376] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12155), 1,
      anon_sym_COLON,
    STATE(6314), 2,
      sym_line_comment,
      sym_block_comment,
  [171390] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7614), 1,
      anon_sym_trait,
    STATE(6315), 2,
      sym_line_comment,
      sym_block_comment,
  [171404] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12157), 1,
      anon_sym_SEMI,
    STATE(6316), 2,
      sym_line_comment,
      sym_block_comment,
  [171418] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12159), 1,
      sym_identifier,
    STATE(6317), 2,
      sym_line_comment,
      sym_block_comment,
  [171432] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12161), 1,
      sym_identifier,
    STATE(6318), 2,
      sym_line_comment,
      sym_block_comment,
  [171446] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12163), 1,
      sym_identifier,
    STATE(6319), 2,
      sym_line_comment,
      sym_block_comment,
  [171460] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(684), 1,
      anon_sym_RBRACK,
    STATE(6320), 2,
      sym_line_comment,
      sym_block_comment,
  [171474] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12165), 1,
      sym_identifier,
    STATE(6321), 2,
      sym_line_comment,
      sym_block_comment,
  [171488] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8381), 1,
      anon_sym_GT,
    STATE(6322), 2,
      sym_line_comment,
      sym_block_comment,
  [171502] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12167), 1,
      anon_sym_COLON,
    STATE(6323), 2,
      sym_line_comment,
      sym_block_comment,
  [171516] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12169), 1,
      sym_identifier,
    STATE(6324), 2,
      sym_line_comment,
      sym_block_comment,
  [171530] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12171), 1,
      anon_sym_SEMI,
    STATE(6325), 2,
      sym_line_comment,
      sym_block_comment,
  [171544] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12173), 1,
      sym_identifier,
    STATE(6326), 2,
      sym_line_comment,
      sym_block_comment,
  [171558] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12175), 1,
      sym_identifier,
    STATE(6327), 2,
      sym_line_comment,
      sym_block_comment,
  [171572] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12177), 1,
      sym_identifier,
    STATE(6328), 2,
      sym_line_comment,
      sym_block_comment,
  [171586] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12179), 1,
      sym_identifier,
    STATE(6329), 2,
      sym_line_comment,
      sym_block_comment,
  [171600] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10451), 1,
      anon_sym_RBRACE,
    STATE(6330), 2,
      sym_line_comment,
      sym_block_comment,
  [171614] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12181), 1,
      sym_identifier,
    STATE(6331), 2,
      sym_line_comment,
      sym_block_comment,
  [171628] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12183), 1,
      anon_sym_SEMI,
    STATE(6332), 2,
      sym_line_comment,
      sym_block_comment,
  [171642] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12185), 1,
      anon_sym_SEMI,
    STATE(6333), 2,
      sym_line_comment,
      sym_block_comment,
  [171656] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12187), 1,
      anon_sym_COLON,
    STATE(6334), 2,
      sym_line_comment,
      sym_block_comment,
  [171670] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12189), 1,
      sym_identifier,
    STATE(6335), 2,
      sym_line_comment,
      sym_block_comment,
  [171684] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11197), 1,
      anon_sym_SEMI,
    STATE(6336), 2,
      sym_line_comment,
      sym_block_comment,
  [171698] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12191), 1,
      anon_sym_SEMI,
    STATE(6337), 2,
      sym_line_comment,
      sym_block_comment,
  [171712] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12193), 1,
      anon_sym_LBRACK,
    STATE(6338), 2,
      sym_line_comment,
      sym_block_comment,
  [171726] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12195), 1,
      anon_sym_COLON,
    STATE(6339), 2,
      sym_line_comment,
      sym_block_comment,
  [171740] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12197), 1,
      sym_identifier,
    STATE(6340), 2,
      sym_line_comment,
      sym_block_comment,
  [171754] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12199), 1,
      anon_sym_EQ_GT,
    STATE(6341), 2,
      sym_line_comment,
      sym_block_comment,
  [171768] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9737), 1,
      anon_sym_GT,
    STATE(6342), 2,
      sym_line_comment,
      sym_block_comment,
  [171782] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12201), 1,
      anon_sym_COLON,
    STATE(6343), 2,
      sym_line_comment,
      sym_block_comment,
  [171796] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7626), 1,
      anon_sym_trait,
    STATE(6344), 2,
      sym_line_comment,
      sym_block_comment,
  [171810] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12203), 1,
      sym_identifier,
    STATE(6345), 2,
      sym_line_comment,
      sym_block_comment,
  [171824] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12205), 1,
      sym_identifier,
    STATE(6346), 2,
      sym_line_comment,
      sym_block_comment,
  [171838] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11145), 1,
      anon_sym_SEMI,
    STATE(6347), 2,
      sym_line_comment,
      sym_block_comment,
  [171852] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12207), 1,
      anon_sym_COLON,
    STATE(6348), 2,
      sym_line_comment,
      sym_block_comment,
  [171866] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12209), 1,
      anon_sym_trait,
    STATE(6349), 2,
      sym_line_comment,
      sym_block_comment,
  [171880] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12211), 1,
      sym_identifier,
    STATE(6350), 2,
      sym_line_comment,
      sym_block_comment,
  [171894] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12213), 1,
      anon_sym_SEMI,
    STATE(6351), 2,
      sym_line_comment,
      sym_block_comment,
  [171908] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11271), 1,
      anon_sym_SEMI,
    STATE(6352), 2,
      sym_line_comment,
      sym_block_comment,
  [171922] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12215), 1,
      anon_sym_SEMI,
    STATE(6353), 2,
      sym_line_comment,
      sym_block_comment,
  [171936] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12217), 1,
      anon_sym_SEMI,
    STATE(6354), 2,
      sym_line_comment,
      sym_block_comment,
  [171950] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1758), 1,
      anon_sym_RBRACE,
    STATE(6355), 2,
      sym_line_comment,
      sym_block_comment,
  [171964] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12219), 1,
      anon_sym_SEMI,
    STATE(6356), 2,
      sym_line_comment,
      sym_block_comment,
  [171978] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12221), 1,
      anon_sym_COLON,
    STATE(6357), 2,
      sym_line_comment,
      sym_block_comment,
  [171992] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11273), 1,
      anon_sym_SEMI,
    STATE(6358), 2,
      sym_line_comment,
      sym_block_comment,
  [172006] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12223), 1,
      anon_sym_SEMI,
    STATE(6359), 2,
      sym_line_comment,
      sym_block_comment,
  [172020] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12225), 1,
      anon_sym_SEMI,
    STATE(6360), 2,
      sym_line_comment,
      sym_block_comment,
  [172034] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12227), 1,
      anon_sym_SEMI,
    STATE(6361), 2,
      sym_line_comment,
      sym_block_comment,
  [172048] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1752), 1,
      anon_sym_RBRACE,
    STATE(6362), 2,
      sym_line_comment,
      sym_block_comment,
  [172062] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12229), 1,
      anon_sym_COLON,
    STATE(6363), 2,
      sym_line_comment,
      sym_block_comment,
  [172076] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12231), 1,
      anon_sym_SEMI,
    STATE(6364), 2,
      sym_line_comment,
      sym_block_comment,
  [172090] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12233), 1,
      anon_sym_EQ_GT,
    STATE(6365), 2,
      sym_line_comment,
      sym_block_comment,
  [172104] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8819), 1,
      anon_sym_RPAREN,
    STATE(6366), 2,
      sym_line_comment,
      sym_block_comment,
  [172118] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12235), 1,
      anon_sym_COLON,
    STATE(6367), 2,
      sym_line_comment,
      sym_block_comment,
  [172132] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8315), 1,
      anon_sym_GT,
    STATE(6368), 2,
      sym_line_comment,
      sym_block_comment,
  [172146] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12237), 1,
      anon_sym_SEMI,
    STATE(6369), 2,
      sym_line_comment,
      sym_block_comment,
  [172160] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12239), 1,
      anon_sym_SEMI,
    STATE(6370), 2,
      sym_line_comment,
      sym_block_comment,
  [172174] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12241), 1,
      anon_sym_COLON,
    STATE(6371), 2,
      sym_line_comment,
      sym_block_comment,
  [172188] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12243), 1,
      anon_sym_trait,
    STATE(6372), 2,
      sym_line_comment,
      sym_block_comment,
  [172202] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12245), 1,
      anon_sym_COLON_COLON,
    STATE(6373), 2,
      sym_line_comment,
      sym_block_comment,
  [172216] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12247), 1,
      anon_sym_COLON_COLON,
    STATE(6374), 2,
      sym_line_comment,
      sym_block_comment,
  [172230] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12249), 1,
      anon_sym_SEMI,
    STATE(6375), 2,
      sym_line_comment,
      sym_block_comment,
  [172244] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12251), 1,
      anon_sym_SEMI,
    STATE(6376), 2,
      sym_line_comment,
      sym_block_comment,
  [172258] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12253), 1,
      anon_sym_SEMI,
    STATE(6377), 2,
      sym_line_comment,
      sym_block_comment,
  [172272] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(5305), 1,
      anon_sym_PLUS,
    STATE(6378), 2,
      sym_line_comment,
      sym_block_comment,
  [172286] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9368), 1,
      anon_sym_LBRACE,
    STATE(6379), 2,
      sym_line_comment,
      sym_block_comment,
  [172300] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12255), 1,
      anon_sym_SEMI,
    STATE(6380), 2,
      sym_line_comment,
      sym_block_comment,
  [172314] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9138), 1,
      anon_sym_RPAREN,
    STATE(6381), 2,
      sym_line_comment,
      sym_block_comment,
  [172328] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12257), 1,
      anon_sym_SEMI,
    STATE(6382), 2,
      sym_line_comment,
      sym_block_comment,
  [172342] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12259), 1,
      anon_sym_COLON,
    STATE(6383), 2,
      sym_line_comment,
      sym_block_comment,
  [172356] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12261), 1,
      anon_sym_COLON,
    STATE(6384), 2,
      sym_line_comment,
      sym_block_comment,
  [172370] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9886), 1,
      anon_sym_RBRACE,
    STATE(6385), 2,
      sym_line_comment,
      sym_block_comment,
  [172384] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9224), 1,
      anon_sym_RPAREN,
    STATE(6386), 2,
      sym_line_comment,
      sym_block_comment,
  [172398] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12263), 1,
      anon_sym_SEMI,
    STATE(6387), 2,
      sym_line_comment,
      sym_block_comment,
  [172412] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12265), 1,
      anon_sym_SEMI,
    STATE(6388), 2,
      sym_line_comment,
      sym_block_comment,
  [172426] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12267), 1,
      anon_sym_RBRACE,
    STATE(6389), 2,
      sym_line_comment,
      sym_block_comment,
  [172440] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12269), 1,
      anon_sym_COLON,
    STATE(6390), 2,
      sym_line_comment,
      sym_block_comment,
  [172454] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12271), 1,
      anon_sym_COLON,
    STATE(6391), 2,
      sym_line_comment,
      sym_block_comment,
  [172468] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12273), 1,
      anon_sym_SEMI,
    STATE(6392), 2,
      sym_line_comment,
      sym_block_comment,
  [172482] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9142), 1,
      anon_sym_RPAREN,
    STATE(6393), 2,
      sym_line_comment,
      sym_block_comment,
  [172496] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12275), 1,
      anon_sym_SEMI,
    STATE(6394), 2,
      sym_line_comment,
      sym_block_comment,
  [172510] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12277), 1,
      anon_sym_STAR_SLASH,
    STATE(6395), 2,
      sym_line_comment,
      sym_block_comment,
  [172524] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7552), 1,
      anon_sym_trait,
    STATE(6396), 2,
      sym_line_comment,
      sym_block_comment,
  [172538] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9910), 1,
      anon_sym_RBRACE,
    STATE(6397), 2,
      sym_line_comment,
      sym_block_comment,
  [172552] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12279), 1,
      anon_sym_SEMI,
    STATE(6398), 2,
      sym_line_comment,
      sym_block_comment,
  [172566] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12281), 1,
      anon_sym_RPAREN,
    STATE(6399), 2,
      sym_line_comment,
      sym_block_comment,
  [172580] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(1011), 1,
      anon_sym_RBRACK,
    STATE(6400), 2,
      sym_line_comment,
      sym_block_comment,
  [172594] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12283), 1,
      sym_frontmatter_content,
    STATE(6401), 2,
      sym_line_comment,
      sym_block_comment,
  [172608] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12285), 1,
      anon_sym_RBRACE,
    STATE(6402), 2,
      sym_line_comment,
      sym_block_comment,
  [172622] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9070), 1,
      anon_sym_fn,
    STATE(6403), 2,
      sym_line_comment,
      sym_block_comment,
  [172636] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12287), 1,
      sym_identifier,
    STATE(6404), 2,
      sym_line_comment,
      sym_block_comment,
  [172650] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12289), 1,
      anon_sym_COLON,
    STATE(6405), 2,
      sym_line_comment,
      sym_block_comment,
  [172664] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12291), 1,
      anon_sym_COLON,
    STATE(6406), 2,
      sym_line_comment,
      sym_block_comment,
  [172678] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12293), 1,
      anon_sym_SEMI,
    STATE(6407), 2,
      sym_line_comment,
      sym_block_comment,
  [172692] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12295), 1,
      anon_sym_SEMI,
    STATE(6408), 2,
      sym_line_comment,
      sym_block_comment,
  [172706] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12297), 1,
      anon_sym_SEMI,
    STATE(6409), 2,
      sym_line_comment,
      sym_block_comment,
  [172720] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12299), 1,
      anon_sym_COLON,
    STATE(6410), 2,
      sym_line_comment,
      sym_block_comment,
  [172734] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12301), 1,
      sym_identifier,
    STATE(6411), 2,
      sym_line_comment,
      sym_block_comment,
  [172748] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12303), 1,
      anon_sym_COLON,
    STATE(6412), 2,
      sym_line_comment,
      sym_block_comment,
  [172762] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11003), 1,
      anon_sym_LBRACK,
    STATE(6413), 2,
      sym_line_comment,
      sym_block_comment,
  [172776] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12305), 1,
      anon_sym_SEMI,
    STATE(6414), 2,
      sym_line_comment,
      sym_block_comment,
  [172790] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12307), 1,
      anon_sym_SEMI,
    STATE(6415), 2,
      sym_line_comment,
      sym_block_comment,
  [172804] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12309), 1,
      anon_sym_LBRACK,
    STATE(6416), 2,
      sym_line_comment,
      sym_block_comment,
  [172818] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12311), 1,
      anon_sym_COLON,
    STATE(6417), 2,
      sym_line_comment,
      sym_block_comment,
  [172832] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12313), 1,
      anon_sym_LBRACK,
    STATE(6418), 2,
      sym_line_comment,
      sym_block_comment,
  [172846] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12315), 1,
      sym__line_doc_content,
    STATE(6419), 2,
      sym_line_comment,
      sym_block_comment,
  [172860] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(8663), 1,
      anon_sym_RPAREN,
    STATE(6420), 2,
      sym_line_comment,
      sym_block_comment,
  [172874] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11153), 1,
      anon_sym_SEMI,
    STATE(6421), 2,
      sym_line_comment,
      sym_block_comment,
  [172888] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12317), 1,
      sym_identifier,
    STATE(6422), 2,
      sym_line_comment,
      sym_block_comment,
  [172902] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12319), 1,
      anon_sym_LBRACK,
    STATE(6423), 2,
      sym_line_comment,
      sym_block_comment,
  [172916] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9843), 1,
      anon_sym_GT,
    STATE(6424), 2,
      sym_line_comment,
      sym_block_comment,
  [172930] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12321), 1,
      anon_sym_COLON,
    STATE(6425), 2,
      sym_line_comment,
      sym_block_comment,
  [172944] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(11477), 1,
      anon_sym_LBRACK,
    STATE(6426), 2,
      sym_line_comment,
      sym_block_comment,
  [172958] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12323), 1,
      anon_sym_COLON,
    STATE(6427), 2,
      sym_line_comment,
      sym_block_comment,
  [172972] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12325), 1,
      sym_identifier,
    STATE(6428), 2,
      sym_line_comment,
      sym_block_comment,
  [172986] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12327), 1,
      sym_identifier,
    STATE(6429), 2,
      sym_line_comment,
      sym_block_comment,
  [173000] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12329), 1,
      anon_sym_LBRACK,
    STATE(6430), 2,
      sym_line_comment,
      sym_block_comment,
  [173014] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12331), 1,
      anon_sym_COLON,
    STATE(6431), 2,
      sym_line_comment,
      sym_block_comment,
  [173028] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12333), 1,
      anon_sym_LBRACK,
    STATE(6432), 2,
      sym_line_comment,
      sym_block_comment,
  [173042] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12335), 1,
      anon_sym_COLON,
    STATE(6433), 2,
      sym_line_comment,
      sym_block_comment,
  [173056] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12337), 1,
      anon_sym_COLON,
    STATE(6434), 2,
      sym_line_comment,
      sym_block_comment,
  [173070] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12339), 1,
      anon_sym_COLON,
    STATE(6435), 2,
      sym_line_comment,
      sym_block_comment,
  [173084] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12341), 1,
      sym_identifier,
    STATE(6436), 2,
      sym_line_comment,
      sym_block_comment,
  [173098] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7153), 1,
      anon_sym_fn,
    STATE(6437), 2,
      sym_line_comment,
      sym_block_comment,
  [173112] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12343), 1,
      sym_identifier,
    STATE(6438), 2,
      sym_line_comment,
      sym_block_comment,
  [173126] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12345), 1,
      anon_sym_fn,
    STATE(6439), 2,
      sym_line_comment,
      sym_block_comment,
  [173140] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12347), 1,
      anon_sym_SEMI,
    STATE(6440), 2,
      sym_line_comment,
      sym_block_comment,
  [173154] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12349), 1,
      sym_identifier,
    STATE(6441), 2,
      sym_line_comment,
      sym_block_comment,
  [173168] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12351), 1,
      sym_identifier,
    STATE(6442), 2,
      sym_line_comment,
      sym_block_comment,
  [173182] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12353), 1,
      sym_identifier,
    STATE(6443), 2,
      sym_line_comment,
      sym_block_comment,
  [173196] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12355), 1,
      sym_identifier,
    STATE(6444), 2,
      sym_line_comment,
      sym_block_comment,
  [173210] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7123), 1,
      anon_sym_fn,
    STATE(6445), 2,
      sym_line_comment,
      sym_block_comment,
  [173224] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12357), 1,
      sym_identifier,
    STATE(6446), 2,
      sym_line_comment,
      sym_block_comment,
  [173238] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12359), 1,
      anon_sym_fn,
    STATE(6447), 2,
      sym_line_comment,
      sym_block_comment,
  [173252] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12361), 1,
      anon_sym_SEMI,
    STATE(6448), 2,
      sym_line_comment,
      sym_block_comment,
  [173266] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12363), 1,
      anon_sym_SEMI,
    STATE(6449), 2,
      sym_line_comment,
      sym_block_comment,
  [173280] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12365), 1,
      sym__raw_string_literal_end,
    STATE(6450), 2,
      sym_line_comment,
      sym_block_comment,
  [173294] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12367), 1,
      sym_identifier,
    STATE(6451), 2,
      sym_line_comment,
      sym_block_comment,
  [173308] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12369), 1,
      anon_sym_BANG,
    STATE(6452), 2,
      sym_line_comment,
      sym_block_comment,
  [173322] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12371), 1,
      anon_sym_STAR_SLASH,
    STATE(6453), 2,
      sym_line_comment,
      sym_block_comment,
  [173336] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12373), 1,
      sym_identifier,
    STATE(6454), 2,
      sym_line_comment,
      sym_block_comment,
  [173350] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12375), 1,
      anon_sym_STAR_SLASH,
    STATE(6455), 2,
      sym_line_comment,
      sym_block_comment,
  [173364] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9959), 1,
      anon_sym_GT,
    STATE(6456), 2,
      sym_line_comment,
      sym_block_comment,
  [173378] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12377), 1,
      anon_sym_SEMI,
    STATE(6457), 2,
      sym_line_comment,
      sym_block_comment,
  [173392] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12379), 1,
      sym_identifier,
    STATE(6458), 2,
      sym_line_comment,
      sym_block_comment,
  [173406] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12381), 1,
      anon_sym_SEMI,
    STATE(6459), 2,
      sym_line_comment,
      sym_block_comment,
  [173420] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12383), 1,
      anon_sym_LBRACE,
    STATE(6460), 2,
      sym_line_comment,
      sym_block_comment,
  [173434] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12385), 1,
      anon_sym_fn,
    STATE(6461), 2,
      sym_line_comment,
      sym_block_comment,
  [173448] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12387), 1,
      anon_sym_SEMI,
    STATE(6462), 2,
      sym_line_comment,
      sym_block_comment,
  [173462] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12389), 1,
      anon_sym_SEMI,
    STATE(6463), 2,
      sym_line_comment,
      sym_block_comment,
  [173476] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12391), 1,
      anon_sym_SEMI,
    STATE(6464), 2,
      sym_line_comment,
      sym_block_comment,
  [173490] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12393), 1,
      anon_sym_SEMI,
    STATE(6465), 2,
      sym_line_comment,
      sym_block_comment,
  [173504] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12395), 1,
      anon_sym_SEMI,
    STATE(6466), 2,
      sym_line_comment,
      sym_block_comment,
  [173518] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12397), 1,
      sym_identifier,
    STATE(6467), 2,
      sym_line_comment,
      sym_block_comment,
  [173532] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(7097), 1,
      anon_sym_fn,
    STATE(6468), 2,
      sym_line_comment,
      sym_block_comment,
  [173546] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12399), 1,
      anon_sym_RBRACE,
    STATE(6469), 2,
      sym_line_comment,
      sym_block_comment,
  [173560] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(10736), 1,
      anon_sym_RBRACE,
    STATE(6470), 2,
      sym_line_comment,
      sym_block_comment,
  [173574] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12401), 1,
      sym_identifier,
    STATE(6471), 2,
      sym_line_comment,
      sym_block_comment,
  [173588] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12403), 1,
      anon_sym_SEMI,
    STATE(6472), 2,
      sym_line_comment,
      sym_block_comment,
  [173602] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12405), 1,
      anon_sym_SEMI,
    STATE(6473), 2,
      sym_line_comment,
      sym_block_comment,
  [173616] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12407), 1,
      anon_sym_trait,
    STATE(6474), 2,
      sym_line_comment,
      sym_block_comment,
  [173630] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12409), 1,
      anon_sym_RBRACK,
    STATE(6475), 2,
      sym_line_comment,
      sym_block_comment,
  [173644] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12411), 1,
      anon_sym_BANG,
    STATE(6476), 2,
      sym_line_comment,
      sym_block_comment,
  [173658] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12413), 1,
      anon_sym_SEMI,
    STATE(6477), 2,
      sym_line_comment,
      sym_block_comment,
  [173672] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12415), 1,
      sym_self,
    STATE(6478), 2,
      sym_line_comment,
      sym_block_comment,
  [173686] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12417), 1,
      sym_identifier,
    STATE(6479), 2,
      sym_line_comment,
      sym_block_comment,
  [173700] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12419), 1,
      anon_sym_RBRACK,
    STATE(6480), 2,
      sym_line_comment,
      sym_block_comment,
  [173714] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12421), 1,
      sym_identifier,
    STATE(6481), 2,
      sym_line_comment,
      sym_block_comment,
  [173728] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12423), 1,
      sym_identifier,
    STATE(6482), 2,
      sym_line_comment,
      sym_block_comment,
  [173742] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12425), 1,
      sym_identifier,
    STATE(6483), 2,
      sym_line_comment,
      sym_block_comment,
  [173756] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12427), 1,
      anon_sym_SEMI,
    STATE(6484), 2,
      sym_line_comment,
      sym_block_comment,
  [173770] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12429), 1,
      sym_identifier,
    STATE(6485), 2,
      sym_line_comment,
      sym_block_comment,
  [173784] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12431), 1,
      anon_sym_SEMI,
    STATE(6486), 2,
      sym_line_comment,
      sym_block_comment,
  [173798] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12433), 1,
      sym_identifier,
    STATE(6487), 2,
      sym_line_comment,
      sym_block_comment,
  [173812] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12435), 1,
      anon_sym_SEMI,
    STATE(6488), 2,
      sym_line_comment,
      sym_block_comment,
  [173826] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12437), 1,
      anon_sym_SEMI,
    STATE(6489), 2,
      sym_line_comment,
      sym_block_comment,
  [173840] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12439), 1,
      anon_sym_SEMI,
    STATE(6490), 2,
      sym_line_comment,
      sym_block_comment,
  [173854] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12441), 1,
      anon_sym_fn,
    STATE(6491), 2,
      sym_line_comment,
      sym_block_comment,
  [173868] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12443), 1,
      anon_sym_SEMI,
    STATE(6492), 2,
      sym_line_comment,
      sym_block_comment,
  [173882] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(9989), 1,
      anon_sym_RBRACE,
    STATE(6493), 2,
      sym_line_comment,
      sym_block_comment,
  [173896] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12445), 1,
      anon_sym_RBRACE,
    STATE(6494), 2,
      sym_line_comment,
      sym_block_comment,
  [173910] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12447), 1,
      anon_sym_SEMI,
    STATE(6495), 2,
      sym_line_comment,
      sym_block_comment,
  [173924] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12449), 1,
      anon_sym_SEMI,
    STATE(6496), 2,
      sym_line_comment,
      sym_block_comment,
  [173938] = 4,
    ACTIONS(3), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(5), 1,
      anon_sym_SLASH_STAR,
    ACTIONS(12451), 1,
      anon_sym_COLON,
    STATE(6497), 2,
      sym_line_comment,
      sym_block_comment,
  [173952] = 1,
    ACTIONS(12453), 1,
      ts_builtin_sym_end,
  [173956] = 1,
    ACTIONS(12455), 1,
      ts_builtin_sym_end,
  [173960] = 1,
    ACTIONS(12457), 1,
      ts_builtin_sym_end,
  [173964] = 1,
    ACTIONS(12459), 1,
      ts_builtin_sym_end,
  [173968] = 1,
    ACTIONS(12461), 1,
      ts_builtin_sym_end,
  [173972] = 1,
    ACTIONS(12463), 1,
      ts_builtin_sym_end,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(1951)] = 0,
  [SMALL_STATE(1952)] = 109,
  [SMALL_STATE(1953)] = 258,
  [SMALL_STATE(1954)] = 367,
  [SMALL_STATE(1955)] = 476,
  [SMALL_STATE(1956)] = 560,
  [SMALL_STATE(1957)] = 642,
  [SMALL_STATE(1958)] = 722,
  [SMALL_STATE(1959)] = 794,
  [SMALL_STATE(1960)] = 866,
  [SMALL_STATE(1961)] = 938,
  [SMALL_STATE(1962)] = 1010,
  [SMALL_STATE(1963)] = 1082,
  [SMALL_STATE(1964)] = 1154,
  [SMALL_STATE(1965)] = 1226,
  [SMALL_STATE(1966)] = 1308,
  [SMALL_STATE(1967)] = 1380,
  [SMALL_STATE(1968)] = 1452,
  [SMALL_STATE(1969)] = 1524,
  [SMALL_STATE(1970)] = 1641,
  [SMALL_STATE(1971)] = 1758,
  [SMALL_STATE(1972)] = 1829,
  [SMALL_STATE(1973)] = 1946,
  [SMALL_STATE(1974)] = 2017,
  [SMALL_STATE(1975)] = 2134,
  [SMALL_STATE(1976)] = 2251,
  [SMALL_STATE(1977)] = 2324,
  [SMALL_STATE(1978)] = 2395,
  [SMALL_STATE(1979)] = 2468,
  [SMALL_STATE(1980)] = 2538,
  [SMALL_STATE(1981)] = 2608,
  [SMALL_STATE(1982)] = 2678,
  [SMALL_STATE(1983)] = 2748,
  [SMALL_STATE(1984)] = 2818,
  [SMALL_STATE(1985)] = 2888,
  [SMALL_STATE(1986)] = 2958,
  [SMALL_STATE(1987)] = 3028,
  [SMALL_STATE(1988)] = 3098,
  [SMALL_STATE(1989)] = 3168,
  [SMALL_STATE(1990)] = 3237,
  [SMALL_STATE(1991)] = 3306,
  [SMALL_STATE(1992)] = 3375,
  [SMALL_STATE(1993)] = 3444,
  [SMALL_STATE(1994)] = 3513,
  [SMALL_STATE(1995)] = 3582,
  [SMALL_STATE(1996)] = 3651,
  [SMALL_STATE(1997)] = 3720,
  [SMALL_STATE(1998)] = 3789,
  [SMALL_STATE(1999)] = 3858,
  [SMALL_STATE(2000)] = 3927,
  [SMALL_STATE(2001)] = 3996,
  [SMALL_STATE(2002)] = 4065,
  [SMALL_STATE(2003)] = 4134,
  [SMALL_STATE(2004)] = 4203,
  [SMALL_STATE(2005)] = 4272,
  [SMALL_STATE(2006)] = 4341,
  [SMALL_STATE(2007)] = 4410,
  [SMALL_STATE(2008)] = 4479,
  [SMALL_STATE(2009)] = 4548,
  [SMALL_STATE(2010)] = 4617,
  [SMALL_STATE(2011)] = 4686,
  [SMALL_STATE(2012)] = 4755,
  [SMALL_STATE(2013)] = 4824,
  [SMALL_STATE(2014)] = 4893,
  [SMALL_STATE(2015)] = 4962,
  [SMALL_STATE(2016)] = 5031,
  [SMALL_STATE(2017)] = 5100,
  [SMALL_STATE(2018)] = 5169,
  [SMALL_STATE(2019)] = 5238,
  [SMALL_STATE(2020)] = 5307,
  [SMALL_STATE(2021)] = 5376,
  [SMALL_STATE(2022)] = 5445,
  [SMALL_STATE(2023)] = 5514,
  [SMALL_STATE(2024)] = 5583,
  [SMALL_STATE(2025)] = 5652,
  [SMALL_STATE(2026)] = 5721,
  [SMALL_STATE(2027)] = 5790,
  [SMALL_STATE(2028)] = 5859,
  [SMALL_STATE(2029)] = 5928,
  [SMALL_STATE(2030)] = 5997,
  [SMALL_STATE(2031)] = 6066,
  [SMALL_STATE(2032)] = 6135,
  [SMALL_STATE(2033)] = 6204,
  [SMALL_STATE(2034)] = 6273,
  [SMALL_STATE(2035)] = 6342,
  [SMALL_STATE(2036)] = 6411,
  [SMALL_STATE(2037)] = 6480,
  [SMALL_STATE(2038)] = 6549,
  [SMALL_STATE(2039)] = 6618,
  [SMALL_STATE(2040)] = 6687,
  [SMALL_STATE(2041)] = 6756,
  [SMALL_STATE(2042)] = 6825,
  [SMALL_STATE(2043)] = 6894,
  [SMALL_STATE(2044)] = 6963,
  [SMALL_STATE(2045)] = 7032,
  [SMALL_STATE(2046)] = 7101,
  [SMALL_STATE(2047)] = 7170,
  [SMALL_STATE(2048)] = 7239,
  [SMALL_STATE(2049)] = 7308,
  [SMALL_STATE(2050)] = 7377,
  [SMALL_STATE(2051)] = 7446,
  [SMALL_STATE(2052)] = 7515,
  [SMALL_STATE(2053)] = 7584,
  [SMALL_STATE(2054)] = 7653,
  [SMALL_STATE(2055)] = 7722,
  [SMALL_STATE(2056)] = 7791,
  [SMALL_STATE(2057)] = 7860,
  [SMALL_STATE(2058)] = 7929,
  [SMALL_STATE(2059)] = 7998,
  [SMALL_STATE(2060)] = 8067,
  [SMALL_STATE(2061)] = 8136,
  [SMALL_STATE(2062)] = 8205,
  [SMALL_STATE(2063)] = 8274,
  [SMALL_STATE(2064)] = 8343,
  [SMALL_STATE(2065)] = 8412,
  [SMALL_STATE(2066)] = 8481,
  [SMALL_STATE(2067)] = 8550,
  [SMALL_STATE(2068)] = 8619,
  [SMALL_STATE(2069)] = 8688,
  [SMALL_STATE(2070)] = 8757,
  [SMALL_STATE(2071)] = 8826,
  [SMALL_STATE(2072)] = 8895,
  [SMALL_STATE(2073)] = 8964,
  [SMALL_STATE(2074)] = 9033,
  [SMALL_STATE(2075)] = 9102,
  [SMALL_STATE(2076)] = 9171,
  [SMALL_STATE(2077)] = 9240,
  [SMALL_STATE(2078)] = 9309,
  [SMALL_STATE(2079)] = 9378,
  [SMALL_STATE(2080)] = 9447,
  [SMALL_STATE(2081)] = 9516,
  [SMALL_STATE(2082)] = 9585,
  [SMALL_STATE(2083)] = 9654,
  [SMALL_STATE(2084)] = 9723,
  [SMALL_STATE(2085)] = 9792,
  [SMALL_STATE(2086)] = 9861,
  [SMALL_STATE(2087)] = 9930,
  [SMALL_STATE(2088)] = 9999,
  [SMALL_STATE(2089)] = 10068,
  [SMALL_STATE(2090)] = 10137,
  [SMALL_STATE(2091)] = 10206,
  [SMALL_STATE(2092)] = 10275,
  [SMALL_STATE(2093)] = 10344,
  [SMALL_STATE(2094)] = 10413,
  [SMALL_STATE(2095)] = 10482,
  [SMALL_STATE(2096)] = 10551,
  [SMALL_STATE(2097)] = 10620,
  [SMALL_STATE(2098)] = 10689,
  [SMALL_STATE(2099)] = 10758,
  [SMALL_STATE(2100)] = 10827,
  [SMALL_STATE(2101)] = 10896,
  [SMALL_STATE(2102)] = 10965,
  [SMALL_STATE(2103)] = 11034,
  [SMALL_STATE(2104)] = 11103,
  [SMALL_STATE(2105)] = 11172,
  [SMALL_STATE(2106)] = 11241,
  [SMALL_STATE(2107)] = 11310,
  [SMALL_STATE(2108)] = 11379,
  [SMALL_STATE(2109)] = 11448,
  [SMALL_STATE(2110)] = 11517,
  [SMALL_STATE(2111)] = 11586,
  [SMALL_STATE(2112)] = 11655,
  [SMALL_STATE(2113)] = 11724,
  [SMALL_STATE(2114)] = 11793,
  [SMALL_STATE(2115)] = 11862,
  [SMALL_STATE(2116)] = 11931,
  [SMALL_STATE(2117)] = 12000,
  [SMALL_STATE(2118)] = 12069,
  [SMALL_STATE(2119)] = 12138,
  [SMALL_STATE(2120)] = 12207,
  [SMALL_STATE(2121)] = 12276,
  [SMALL_STATE(2122)] = 12345,
  [SMALL_STATE(2123)] = 12414,
  [SMALL_STATE(2124)] = 12483,
  [SMALL_STATE(2125)] = 12552,
  [SMALL_STATE(2126)] = 12621,
  [SMALL_STATE(2127)] = 12690,
  [SMALL_STATE(2128)] = 12759,
  [SMALL_STATE(2129)] = 12828,
  [SMALL_STATE(2130)] = 12897,
  [SMALL_STATE(2131)] = 12966,
  [SMALL_STATE(2132)] = 13035,
  [SMALL_STATE(2133)] = 13104,
  [SMALL_STATE(2134)] = 13173,
  [SMALL_STATE(2135)] = 13242,
  [SMALL_STATE(2136)] = 13311,
  [SMALL_STATE(2137)] = 13380,
  [SMALL_STATE(2138)] = 13449,
  [SMALL_STATE(2139)] = 13518,
  [SMALL_STATE(2140)] = 13587,
  [SMALL_STATE(2141)] = 13656,
  [SMALL_STATE(2142)] = 13725,
  [SMALL_STATE(2143)] = 13794,
  [SMALL_STATE(2144)] = 13863,
  [SMALL_STATE(2145)] = 13932,
  [SMALL_STATE(2146)] = 14001,
  [SMALL_STATE(2147)] = 14070,
  [SMALL_STATE(2148)] = 14139,
  [SMALL_STATE(2149)] = 14208,
  [SMALL_STATE(2150)] = 14277,
  [SMALL_STATE(2151)] = 14346,
  [SMALL_STATE(2152)] = 14415,
  [SMALL_STATE(2153)] = 14484,
  [SMALL_STATE(2154)] = 14553,
  [SMALL_STATE(2155)] = 14622,
  [SMALL_STATE(2156)] = 14691,
  [SMALL_STATE(2157)] = 14760,
  [SMALL_STATE(2158)] = 14829,
  [SMALL_STATE(2159)] = 14898,
  [SMALL_STATE(2160)] = 14967,
  [SMALL_STATE(2161)] = 15036,
  [SMALL_STATE(2162)] = 15105,
  [SMALL_STATE(2163)] = 15174,
  [SMALL_STATE(2164)] = 15243,
  [SMALL_STATE(2165)] = 15312,
  [SMALL_STATE(2166)] = 15381,
  [SMALL_STATE(2167)] = 15450,
  [SMALL_STATE(2168)] = 15519,
  [SMALL_STATE(2169)] = 15588,
  [SMALL_STATE(2170)] = 15657,
  [SMALL_STATE(2171)] = 15726,
  [SMALL_STATE(2172)] = 15795,
  [SMALL_STATE(2173)] = 15864,
  [SMALL_STATE(2174)] = 15933,
  [SMALL_STATE(2175)] = 16002,
  [SMALL_STATE(2176)] = 16071,
  [SMALL_STATE(2177)] = 16140,
  [SMALL_STATE(2178)] = 16209,
  [SMALL_STATE(2179)] = 16278,
  [SMALL_STATE(2180)] = 16347,
  [SMALL_STATE(2181)] = 16416,
  [SMALL_STATE(2182)] = 16485,
  [SMALL_STATE(2183)] = 16554,
  [SMALL_STATE(2184)] = 16623,
  [SMALL_STATE(2185)] = 16692,
  [SMALL_STATE(2186)] = 16761,
  [SMALL_STATE(2187)] = 16830,
  [SMALL_STATE(2188)] = 16899,
  [SMALL_STATE(2189)] = 16968,
  [SMALL_STATE(2190)] = 17037,
  [SMALL_STATE(2191)] = 17106,
  [SMALL_STATE(2192)] = 17175,
  [SMALL_STATE(2193)] = 17244,
  [SMALL_STATE(2194)] = 17313,
  [SMALL_STATE(2195)] = 17382,
  [SMALL_STATE(2196)] = 17451,
  [SMALL_STATE(2197)] = 17520,
  [SMALL_STATE(2198)] = 17589,
  [SMALL_STATE(2199)] = 17658,
  [SMALL_STATE(2200)] = 17727,
  [SMALL_STATE(2201)] = 17796,
  [SMALL_STATE(2202)] = 17865,
  [SMALL_STATE(2203)] = 17934,
  [SMALL_STATE(2204)] = 18003,
  [SMALL_STATE(2205)] = 18072,
  [SMALL_STATE(2206)] = 18141,
  [SMALL_STATE(2207)] = 18210,
  [SMALL_STATE(2208)] = 18279,
  [SMALL_STATE(2209)] = 18348,
  [SMALL_STATE(2210)] = 18417,
  [SMALL_STATE(2211)] = 18486,
  [SMALL_STATE(2212)] = 18555,
  [SMALL_STATE(2213)] = 18624,
  [SMALL_STATE(2214)] = 18693,
  [SMALL_STATE(2215)] = 18762,
  [SMALL_STATE(2216)] = 18831,
  [SMALL_STATE(2217)] = 18900,
  [SMALL_STATE(2218)] = 18969,
  [SMALL_STATE(2219)] = 19038,
  [SMALL_STATE(2220)] = 19107,
  [SMALL_STATE(2221)] = 19176,
  [SMALL_STATE(2222)] = 19245,
  [SMALL_STATE(2223)] = 19314,
  [SMALL_STATE(2224)] = 19383,
  [SMALL_STATE(2225)] = 19452,
  [SMALL_STATE(2226)] = 19521,
  [SMALL_STATE(2227)] = 19590,
  [SMALL_STATE(2228)] = 19659,
  [SMALL_STATE(2229)] = 19728,
  [SMALL_STATE(2230)] = 19797,
  [SMALL_STATE(2231)] = 19866,
  [SMALL_STATE(2232)] = 19935,
  [SMALL_STATE(2233)] = 20004,
  [SMALL_STATE(2234)] = 20073,
  [SMALL_STATE(2235)] = 20142,
  [SMALL_STATE(2236)] = 20211,
  [SMALL_STATE(2237)] = 20280,
  [SMALL_STATE(2238)] = 20349,
  [SMALL_STATE(2239)] = 20418,
  [SMALL_STATE(2240)] = 20487,
  [SMALL_STATE(2241)] = 20556,
  [SMALL_STATE(2242)] = 20625,
  [SMALL_STATE(2243)] = 20694,
  [SMALL_STATE(2244)] = 20763,
  [SMALL_STATE(2245)] = 20832,
  [SMALL_STATE(2246)] = 20901,
  [SMALL_STATE(2247)] = 20970,
  [SMALL_STATE(2248)] = 21039,
  [SMALL_STATE(2249)] = 21108,
  [SMALL_STATE(2250)] = 21177,
  [SMALL_STATE(2251)] = 21246,
  [SMALL_STATE(2252)] = 21315,
  [SMALL_STATE(2253)] = 21384,
  [SMALL_STATE(2254)] = 21453,
  [SMALL_STATE(2255)] = 21522,
  [SMALL_STATE(2256)] = 21591,
  [SMALL_STATE(2257)] = 21660,
  [SMALL_STATE(2258)] = 21729,
  [SMALL_STATE(2259)] = 21798,
  [SMALL_STATE(2260)] = 21867,
  [SMALL_STATE(2261)] = 21936,
  [SMALL_STATE(2262)] = 22005,
  [SMALL_STATE(2263)] = 22074,
  [SMALL_STATE(2264)] = 22143,
  [SMALL_STATE(2265)] = 22212,
  [SMALL_STATE(2266)] = 22281,
  [SMALL_STATE(2267)] = 22350,
  [SMALL_STATE(2268)] = 22419,
  [SMALL_STATE(2269)] = 22488,
  [SMALL_STATE(2270)] = 22557,
  [SMALL_STATE(2271)] = 22626,
  [SMALL_STATE(2272)] = 22695,
  [SMALL_STATE(2273)] = 22764,
  [SMALL_STATE(2274)] = 22833,
  [SMALL_STATE(2275)] = 22902,
  [SMALL_STATE(2276)] = 22971,
  [SMALL_STATE(2277)] = 23040,
  [SMALL_STATE(2278)] = 23109,
  [SMALL_STATE(2279)] = 23178,
  [SMALL_STATE(2280)] = 23247,
  [SMALL_STATE(2281)] = 23316,
  [SMALL_STATE(2282)] = 23385,
  [SMALL_STATE(2283)] = 23454,
  [SMALL_STATE(2284)] = 23523,
  [SMALL_STATE(2285)] = 23592,
  [SMALL_STATE(2286)] = 23661,
  [SMALL_STATE(2287)] = 23730,
  [SMALL_STATE(2288)] = 23799,
  [SMALL_STATE(2289)] = 23868,
  [SMALL_STATE(2290)] = 23937,
  [SMALL_STATE(2291)] = 24006,
  [SMALL_STATE(2292)] = 24075,
  [SMALL_STATE(2293)] = 24144,
  [SMALL_STATE(2294)] = 24213,
  [SMALL_STATE(2295)] = 24282,
  [SMALL_STATE(2296)] = 24351,
  [SMALL_STATE(2297)] = 24420,
  [SMALL_STATE(2298)] = 24489,
  [SMALL_STATE(2299)] = 24558,
  [SMALL_STATE(2300)] = 24627,
  [SMALL_STATE(2301)] = 24696,
  [SMALL_STATE(2302)] = 24765,
  [SMALL_STATE(2303)] = 24834,
  [SMALL_STATE(2304)] = 24903,
  [SMALL_STATE(2305)] = 24972,
  [SMALL_STATE(2306)] = 25041,
  [SMALL_STATE(2307)] = 25110,
  [SMALL_STATE(2308)] = 25179,
  [SMALL_STATE(2309)] = 25248,
  [SMALL_STATE(2310)] = 25317,
  [SMALL_STATE(2311)] = 25386,
  [SMALL_STATE(2312)] = 25455,
  [SMALL_STATE(2313)] = 25524,
  [SMALL_STATE(2314)] = 25593,
  [SMALL_STATE(2315)] = 25662,
  [SMALL_STATE(2316)] = 25731,
  [SMALL_STATE(2317)] = 25800,
  [SMALL_STATE(2318)] = 25869,
  [SMALL_STATE(2319)] = 25938,
  [SMALL_STATE(2320)] = 26007,
  [SMALL_STATE(2321)] = 26076,
  [SMALL_STATE(2322)] = 26145,
  [SMALL_STATE(2323)] = 26214,
  [SMALL_STATE(2324)] = 26283,
  [SMALL_STATE(2325)] = 26352,
  [SMALL_STATE(2326)] = 26421,
  [SMALL_STATE(2327)] = 26490,
  [SMALL_STATE(2328)] = 26559,
  [SMALL_STATE(2329)] = 26628,
  [SMALL_STATE(2330)] = 26697,
  [SMALL_STATE(2331)] = 26766,
  [SMALL_STATE(2332)] = 26835,
  [SMALL_STATE(2333)] = 26904,
  [SMALL_STATE(2334)] = 26973,
  [SMALL_STATE(2335)] = 27042,
  [SMALL_STATE(2336)] = 27111,
  [SMALL_STATE(2337)] = 27180,
  [SMALL_STATE(2338)] = 27249,
  [SMALL_STATE(2339)] = 27318,
  [SMALL_STATE(2340)] = 27387,
  [SMALL_STATE(2341)] = 27456,
  [SMALL_STATE(2342)] = 27525,
  [SMALL_STATE(2343)] = 27594,
  [SMALL_STATE(2344)] = 27663,
  [SMALL_STATE(2345)] = 27732,
  [SMALL_STATE(2346)] = 27801,
  [SMALL_STATE(2347)] = 27870,
  [SMALL_STATE(2348)] = 27939,
  [SMALL_STATE(2349)] = 28008,
  [SMALL_STATE(2350)] = 28077,
  [SMALL_STATE(2351)] = 28146,
  [SMALL_STATE(2352)] = 28215,
  [SMALL_STATE(2353)] = 28284,
  [SMALL_STATE(2354)] = 28353,
  [SMALL_STATE(2355)] = 28422,
  [SMALL_STATE(2356)] = 28491,
  [SMALL_STATE(2357)] = 28560,
  [SMALL_STATE(2358)] = 28629,
  [SMALL_STATE(2359)] = 28698,
  [SMALL_STATE(2360)] = 28767,
  [SMALL_STATE(2361)] = 28836,
  [SMALL_STATE(2362)] = 28905,
  [SMALL_STATE(2363)] = 28974,
  [SMALL_STATE(2364)] = 29043,
  [SMALL_STATE(2365)] = 29112,
  [SMALL_STATE(2366)] = 29181,
  [SMALL_STATE(2367)] = 29250,
  [SMALL_STATE(2368)] = 29319,
  [SMALL_STATE(2369)] = 29388,
  [SMALL_STATE(2370)] = 29457,
  [SMALL_STATE(2371)] = 29526,
  [SMALL_STATE(2372)] = 29595,
  [SMALL_STATE(2373)] = 29664,
  [SMALL_STATE(2374)] = 29733,
  [SMALL_STATE(2375)] = 29802,
  [SMALL_STATE(2376)] = 29871,
  [SMALL_STATE(2377)] = 29940,
  [SMALL_STATE(2378)] = 30009,
  [SMALL_STATE(2379)] = 30078,
  [SMALL_STATE(2380)] = 30147,
  [SMALL_STATE(2381)] = 30216,
  [SMALL_STATE(2382)] = 30285,
  [SMALL_STATE(2383)] = 30354,
  [SMALL_STATE(2384)] = 30423,
  [SMALL_STATE(2385)] = 30492,
  [SMALL_STATE(2386)] = 30561,
  [SMALL_STATE(2387)] = 30630,
  [SMALL_STATE(2388)] = 30699,
  [SMALL_STATE(2389)] = 30768,
  [SMALL_STATE(2390)] = 30837,
  [SMALL_STATE(2391)] = 30906,
  [SMALL_STATE(2392)] = 30975,
  [SMALL_STATE(2393)] = 31044,
  [SMALL_STATE(2394)] = 31113,
  [SMALL_STATE(2395)] = 31182,
  [SMALL_STATE(2396)] = 31251,
  [SMALL_STATE(2397)] = 31320,
  [SMALL_STATE(2398)] = 31389,
  [SMALL_STATE(2399)] = 31458,
  [SMALL_STATE(2400)] = 31527,
  [SMALL_STATE(2401)] = 31596,
  [SMALL_STATE(2402)] = 31665,
  [SMALL_STATE(2403)] = 31734,
  [SMALL_STATE(2404)] = 31803,
  [SMALL_STATE(2405)] = 31872,
  [SMALL_STATE(2406)] = 31941,
  [SMALL_STATE(2407)] = 32010,
  [SMALL_STATE(2408)] = 32079,
  [SMALL_STATE(2409)] = 32148,
  [SMALL_STATE(2410)] = 32217,
  [SMALL_STATE(2411)] = 32286,
  [SMALL_STATE(2412)] = 32355,
  [SMALL_STATE(2413)] = 32424,
  [SMALL_STATE(2414)] = 32493,
  [SMALL_STATE(2415)] = 32562,
  [SMALL_STATE(2416)] = 32631,
  [SMALL_STATE(2417)] = 32700,
  [SMALL_STATE(2418)] = 32769,
  [SMALL_STATE(2419)] = 32838,
  [SMALL_STATE(2420)] = 32907,
  [SMALL_STATE(2421)] = 32976,
  [SMALL_STATE(2422)] = 33045,
  [SMALL_STATE(2423)] = 33114,
  [SMALL_STATE(2424)] = 33183,
  [SMALL_STATE(2425)] = 33252,
  [SMALL_STATE(2426)] = 33321,
  [SMALL_STATE(2427)] = 33390,
  [SMALL_STATE(2428)] = 33459,
  [SMALL_STATE(2429)] = 33528,
  [SMALL_STATE(2430)] = 33597,
  [SMALL_STATE(2431)] = 33666,
  [SMALL_STATE(2432)] = 33735,
  [SMALL_STATE(2433)] = 33804,
  [SMALL_STATE(2434)] = 33873,
  [SMALL_STATE(2435)] = 33942,
  [SMALL_STATE(2436)] = 34011,
  [SMALL_STATE(2437)] = 34080,
  [SMALL_STATE(2438)] = 34149,
  [SMALL_STATE(2439)] = 34218,
  [SMALL_STATE(2440)] = 34287,
  [SMALL_STATE(2441)] = 34356,
  [SMALL_STATE(2442)] = 34425,
  [SMALL_STATE(2443)] = 34494,
  [SMALL_STATE(2444)] = 34563,
  [SMALL_STATE(2445)] = 34632,
  [SMALL_STATE(2446)] = 34701,
  [SMALL_STATE(2447)] = 34770,
  [SMALL_STATE(2448)] = 34839,
  [SMALL_STATE(2449)] = 34908,
  [SMALL_STATE(2450)] = 34977,
  [SMALL_STATE(2451)] = 35046,
  [SMALL_STATE(2452)] = 35115,
  [SMALL_STATE(2453)] = 35184,
  [SMALL_STATE(2454)] = 35253,
  [SMALL_STATE(2455)] = 35322,
  [SMALL_STATE(2456)] = 35391,
  [SMALL_STATE(2457)] = 35460,
  [SMALL_STATE(2458)] = 35529,
  [SMALL_STATE(2459)] = 35598,
  [SMALL_STATE(2460)] = 35667,
  [SMALL_STATE(2461)] = 35736,
  [SMALL_STATE(2462)] = 35805,
  [SMALL_STATE(2463)] = 35874,
  [SMALL_STATE(2464)] = 35943,
  [SMALL_STATE(2465)] = 36012,
  [SMALL_STATE(2466)] = 36081,
  [SMALL_STATE(2467)] = 36150,
  [SMALL_STATE(2468)] = 36219,
  [SMALL_STATE(2469)] = 36288,
  [SMALL_STATE(2470)] = 36357,
  [SMALL_STATE(2471)] = 36426,
  [SMALL_STATE(2472)] = 36495,
  [SMALL_STATE(2473)] = 36564,
  [SMALL_STATE(2474)] = 36633,
  [SMALL_STATE(2475)] = 36702,
  [SMALL_STATE(2476)] = 36787,
  [SMALL_STATE(2477)] = 36856,
  [SMALL_STATE(2478)] = 36925,
  [SMALL_STATE(2479)] = 36994,
  [SMALL_STATE(2480)] = 37063,
  [SMALL_STATE(2481)] = 37132,
  [SMALL_STATE(2482)] = 37201,
  [SMALL_STATE(2483)] = 37270,
  [SMALL_STATE(2484)] = 37339,
  [SMALL_STATE(2485)] = 37408,
  [SMALL_STATE(2486)] = 37477,
  [SMALL_STATE(2487)] = 37546,
  [SMALL_STATE(2488)] = 37615,
  [SMALL_STATE(2489)] = 37684,
  [SMALL_STATE(2490)] = 37753,
  [SMALL_STATE(2491)] = 37822,
  [SMALL_STATE(2492)] = 37891,
  [SMALL_STATE(2493)] = 37960,
  [SMALL_STATE(2494)] = 38029,
  [SMALL_STATE(2495)] = 38098,
  [SMALL_STATE(2496)] = 38167,
  [SMALL_STATE(2497)] = 38236,
  [SMALL_STATE(2498)] = 38305,
  [SMALL_STATE(2499)] = 38374,
  [SMALL_STATE(2500)] = 38443,
  [SMALL_STATE(2501)] = 38512,
  [SMALL_STATE(2502)] = 38581,
  [SMALL_STATE(2503)] = 38650,
  [SMALL_STATE(2504)] = 38719,
  [SMALL_STATE(2505)] = 38788,
  [SMALL_STATE(2506)] = 38857,
  [SMALL_STATE(2507)] = 38926,
  [SMALL_STATE(2508)] = 38995,
  [SMALL_STATE(2509)] = 39064,
  [SMALL_STATE(2510)] = 39133,
  [SMALL_STATE(2511)] = 39202,
  [SMALL_STATE(2512)] = 39271,
  [SMALL_STATE(2513)] = 39340,
  [SMALL_STATE(2514)] = 39409,
  [SMALL_STATE(2515)] = 39478,
  [SMALL_STATE(2516)] = 39547,
  [SMALL_STATE(2517)] = 39616,
  [SMALL_STATE(2518)] = 39685,
  [SMALL_STATE(2519)] = 39754,
  [SMALL_STATE(2520)] = 39823,
  [SMALL_STATE(2521)] = 39892,
  [SMALL_STATE(2522)] = 39961,
  [SMALL_STATE(2523)] = 40030,
  [SMALL_STATE(2524)] = 40099,
  [SMALL_STATE(2525)] = 40168,
  [SMALL_STATE(2526)] = 40237,
  [SMALL_STATE(2527)] = 40306,
  [SMALL_STATE(2528)] = 40375,
  [SMALL_STATE(2529)] = 40444,
  [SMALL_STATE(2530)] = 40513,
  [SMALL_STATE(2531)] = 40582,
  [SMALL_STATE(2532)] = 40651,
  [SMALL_STATE(2533)] = 40720,
  [SMALL_STATE(2534)] = 40789,
  [SMALL_STATE(2535)] = 40858,
  [SMALL_STATE(2536)] = 40927,
  [SMALL_STATE(2537)] = 40996,
  [SMALL_STATE(2538)] = 41065,
  [SMALL_STATE(2539)] = 41134,
  [SMALL_STATE(2540)] = 41203,
  [SMALL_STATE(2541)] = 41272,
  [SMALL_STATE(2542)] = 41341,
  [SMALL_STATE(2543)] = 41410,
  [SMALL_STATE(2544)] = 41479,
  [SMALL_STATE(2545)] = 41548,
  [SMALL_STATE(2546)] = 41617,
  [SMALL_STATE(2547)] = 41686,
  [SMALL_STATE(2548)] = 41755,
  [SMALL_STATE(2549)] = 41824,
  [SMALL_STATE(2550)] = 41893,
  [SMALL_STATE(2551)] = 41962,
  [SMALL_STATE(2552)] = 42031,
  [SMALL_STATE(2553)] = 42100,
  [SMALL_STATE(2554)] = 42169,
  [SMALL_STATE(2555)] = 42238,
  [SMALL_STATE(2556)] = 42307,
  [SMALL_STATE(2557)] = 42376,
  [SMALL_STATE(2558)] = 42445,
  [SMALL_STATE(2559)] = 42514,
  [SMALL_STATE(2560)] = 42583,
  [SMALL_STATE(2561)] = 42652,
  [SMALL_STATE(2562)] = 42721,
  [SMALL_STATE(2563)] = 42790,
  [SMALL_STATE(2564)] = 42859,
  [SMALL_STATE(2565)] = 42928,
  [SMALL_STATE(2566)] = 42997,
  [SMALL_STATE(2567)] = 43066,
  [SMALL_STATE(2568)] = 43135,
  [SMALL_STATE(2569)] = 43204,
  [SMALL_STATE(2570)] = 43273,
  [SMALL_STATE(2571)] = 43342,
  [SMALL_STATE(2572)] = 43411,
  [SMALL_STATE(2573)] = 43480,
  [SMALL_STATE(2574)] = 43549,
  [SMALL_STATE(2575)] = 43618,
  [SMALL_STATE(2576)] = 43687,
  [SMALL_STATE(2577)] = 43756,
  [SMALL_STATE(2578)] = 43825,
  [SMALL_STATE(2579)] = 43894,
  [SMALL_STATE(2580)] = 43963,
  [SMALL_STATE(2581)] = 44032,
  [SMALL_STATE(2582)] = 44101,
  [SMALL_STATE(2583)] = 44170,
  [SMALL_STATE(2584)] = 44239,
  [SMALL_STATE(2585)] = 44308,
  [SMALL_STATE(2586)] = 44377,
  [SMALL_STATE(2587)] = 44446,
  [SMALL_STATE(2588)] = 44515,
  [SMALL_STATE(2589)] = 44584,
  [SMALL_STATE(2590)] = 44653,
  [SMALL_STATE(2591)] = 44722,
  [SMALL_STATE(2592)] = 44791,
  [SMALL_STATE(2593)] = 44860,
  [SMALL_STATE(2594)] = 44929,
  [SMALL_STATE(2595)] = 44998,
  [SMALL_STATE(2596)] = 45067,
  [SMALL_STATE(2597)] = 45136,
  [SMALL_STATE(2598)] = 45205,
  [SMALL_STATE(2599)] = 45274,
  [SMALL_STATE(2600)] = 45343,
  [SMALL_STATE(2601)] = 45412,
  [SMALL_STATE(2602)] = 45481,
  [SMALL_STATE(2603)] = 45550,
  [SMALL_STATE(2604)] = 45619,
  [SMALL_STATE(2605)] = 45688,
  [SMALL_STATE(2606)] = 45757,
  [SMALL_STATE(2607)] = 45826,
  [SMALL_STATE(2608)] = 45895,
  [SMALL_STATE(2609)] = 45964,
  [SMALL_STATE(2610)] = 46033,
  [SMALL_STATE(2611)] = 46102,
  [SMALL_STATE(2612)] = 46171,
  [SMALL_STATE(2613)] = 46240,
  [SMALL_STATE(2614)] = 46309,
  [SMALL_STATE(2615)] = 46378,
  [SMALL_STATE(2616)] = 46447,
  [SMALL_STATE(2617)] = 46516,
  [SMALL_STATE(2618)] = 46585,
  [SMALL_STATE(2619)] = 46654,
  [SMALL_STATE(2620)] = 46723,
  [SMALL_STATE(2621)] = 46792,
  [SMALL_STATE(2622)] = 46861,
  [SMALL_STATE(2623)] = 46930,
  [SMALL_STATE(2624)] = 46999,
  [SMALL_STATE(2625)] = 47068,
  [SMALL_STATE(2626)] = 47137,
  [SMALL_STATE(2627)] = 47206,
  [SMALL_STATE(2628)] = 47275,
  [SMALL_STATE(2629)] = 47344,
  [SMALL_STATE(2630)] = 47413,
  [SMALL_STATE(2631)] = 47482,
  [SMALL_STATE(2632)] = 47551,
  [SMALL_STATE(2633)] = 47620,
  [SMALL_STATE(2634)] = 47689,
  [SMALL_STATE(2635)] = 47758,
  [SMALL_STATE(2636)] = 47827,
  [SMALL_STATE(2637)] = 47896,
  [SMALL_STATE(2638)] = 47965,
  [SMALL_STATE(2639)] = 48034,
  [SMALL_STATE(2640)] = 48103,
  [SMALL_STATE(2641)] = 48172,
  [SMALL_STATE(2642)] = 48241,
  [SMALL_STATE(2643)] = 48310,
  [SMALL_STATE(2644)] = 48379,
  [SMALL_STATE(2645)] = 48448,
  [SMALL_STATE(2646)] = 48517,
  [SMALL_STATE(2647)] = 48586,
  [SMALL_STATE(2648)] = 48655,
  [SMALL_STATE(2649)] = 48724,
  [SMALL_STATE(2650)] = 48793,
  [SMALL_STATE(2651)] = 48862,
  [SMALL_STATE(2652)] = 48931,
  [SMALL_STATE(2653)] = 49000,
  [SMALL_STATE(2654)] = 49069,
  [SMALL_STATE(2655)] = 49138,
  [SMALL_STATE(2656)] = 49207,
  [SMALL_STATE(2657)] = 49276,
  [SMALL_STATE(2658)] = 49345,
  [SMALL_STATE(2659)] = 49414,
  [SMALL_STATE(2660)] = 49483,
  [SMALL_STATE(2661)] = 49552,
  [SMALL_STATE(2662)] = 49621,
  [SMALL_STATE(2663)] = 49690,
  [SMALL_STATE(2664)] = 49759,
  [SMALL_STATE(2665)] = 49828,
  [SMALL_STATE(2666)] = 49897,
  [SMALL_STATE(2667)] = 49966,
  [SMALL_STATE(2668)] = 50035,
  [SMALL_STATE(2669)] = 50104,
  [SMALL_STATE(2670)] = 50173,
  [SMALL_STATE(2671)] = 50242,
  [SMALL_STATE(2672)] = 50311,
  [SMALL_STATE(2673)] = 50380,
  [SMALL_STATE(2674)] = 50449,
  [SMALL_STATE(2675)] = 50518,
  [SMALL_STATE(2676)] = 50587,
  [SMALL_STATE(2677)] = 50656,
  [SMALL_STATE(2678)] = 50725,
  [SMALL_STATE(2679)] = 50794,
  [SMALL_STATE(2680)] = 50863,
  [SMALL_STATE(2681)] = 50932,
  [SMALL_STATE(2682)] = 51001,
  [SMALL_STATE(2683)] = 51070,
  [SMALL_STATE(2684)] = 51139,
  [SMALL_STATE(2685)] = 51208,
  [SMALL_STATE(2686)] = 51277,
  [SMALL_STATE(2687)] = 51346,
  [SMALL_STATE(2688)] = 51415,
  [SMALL_STATE(2689)] = 51484,
  [SMALL_STATE(2690)] = 51553,
  [SMALL_STATE(2691)] = 51622,
  [SMALL_STATE(2692)] = 51691,
  [SMALL_STATE(2693)] = 51760,
  [SMALL_STATE(2694)] = 51829,
  [SMALL_STATE(2695)] = 51898,
  [SMALL_STATE(2696)] = 51967,
  [SMALL_STATE(2697)] = 52036,
  [SMALL_STATE(2698)] = 52105,
  [SMALL_STATE(2699)] = 52174,
  [SMALL_STATE(2700)] = 52243,
  [SMALL_STATE(2701)] = 52312,
  [SMALL_STATE(2702)] = 52381,
  [SMALL_STATE(2703)] = 52450,
  [SMALL_STATE(2704)] = 52519,
  [SMALL_STATE(2705)] = 52588,
  [SMALL_STATE(2706)] = 52657,
  [SMALL_STATE(2707)] = 52726,
  [SMALL_STATE(2708)] = 52795,
  [SMALL_STATE(2709)] = 52864,
  [SMALL_STATE(2710)] = 52933,
  [SMALL_STATE(2711)] = 53002,
  [SMALL_STATE(2712)] = 53071,
  [SMALL_STATE(2713)] = 53140,
  [SMALL_STATE(2714)] = 53209,
  [SMALL_STATE(2715)] = 53278,
  [SMALL_STATE(2716)] = 53347,
  [SMALL_STATE(2717)] = 53416,
  [SMALL_STATE(2718)] = 53485,
  [SMALL_STATE(2719)] = 53554,
  [SMALL_STATE(2720)] = 53623,
  [SMALL_STATE(2721)] = 53692,
  [SMALL_STATE(2722)] = 53761,
  [SMALL_STATE(2723)] = 53830,
  [SMALL_STATE(2724)] = 53899,
  [SMALL_STATE(2725)] = 53968,
  [SMALL_STATE(2726)] = 54037,
  [SMALL_STATE(2727)] = 54106,
  [SMALL_STATE(2728)] = 54175,
  [SMALL_STATE(2729)] = 54244,
  [SMALL_STATE(2730)] = 54313,
  [SMALL_STATE(2731)] = 54382,
  [SMALL_STATE(2732)] = 54451,
  [SMALL_STATE(2733)] = 54520,
  [SMALL_STATE(2734)] = 54589,
  [SMALL_STATE(2735)] = 54658,
  [SMALL_STATE(2736)] = 54727,
  [SMALL_STATE(2737)] = 54796,
  [SMALL_STATE(2738)] = 54865,
  [SMALL_STATE(2739)] = 54934,
  [SMALL_STATE(2740)] = 55003,
  [SMALL_STATE(2741)] = 55072,
  [SMALL_STATE(2742)] = 55141,
  [SMALL_STATE(2743)] = 55210,
  [SMALL_STATE(2744)] = 55279,
  [SMALL_STATE(2745)] = 55348,
  [SMALL_STATE(2746)] = 55417,
  [SMALL_STATE(2747)] = 55486,
  [SMALL_STATE(2748)] = 55588,
  [SMALL_STATE(2749)] = 55656,
  [SMALL_STATE(2750)] = 55724,
  [SMALL_STATE(2751)] = 55792,
  [SMALL_STATE(2752)] = 55860,
  [SMALL_STATE(2753)] = 55962,
  [SMALL_STATE(2754)] = 56030,
  [SMALL_STATE(2755)] = 56132,
  [SMALL_STATE(2756)] = 56199,
  [SMALL_STATE(2757)] = 56266,
  [SMALL_STATE(2758)] = 56333,
  [SMALL_STATE(2759)] = 56400,
  [SMALL_STATE(2760)] = 56467,
  [SMALL_STATE(2761)] = 56534,
  [SMALL_STATE(2762)] = 56601,
  [SMALL_STATE(2763)] = 56668,
  [SMALL_STATE(2764)] = 56735,
  [SMALL_STATE(2765)] = 56808,
  [SMALL_STATE(2766)] = 56903,
  [SMALL_STATE(2767)] = 56980,
  [SMALL_STATE(2768)] = 57053,
  [SMALL_STATE(2769)] = 57148,
  [SMALL_STATE(2770)] = 57225,
  [SMALL_STATE(2771)] = 57320,
  [SMALL_STATE(2772)] = 57397,
  [SMALL_STATE(2773)] = 57492,
  [SMALL_STATE(2774)] = 57587,
  [SMALL_STATE(2775)] = 57666,
  [SMALL_STATE(2776)] = 57761,
  [SMALL_STATE(2777)] = 57825,
  [SMALL_STATE(2778)] = 57891,
  [SMALL_STATE(2779)] = 57955,
  [SMALL_STATE(2780)] = 58031,
  [SMALL_STATE(2781)] = 58105,
  [SMALL_STATE(2782)] = 58171,
  [SMALL_STATE(2783)] = 58241,
  [SMALL_STATE(2784)] = 58305,
  [SMALL_STATE(2785)] = 58369,
  [SMALL_STATE(2786)] = 58433,
  [SMALL_STATE(2787)] = 58504,
  [SMALL_STATE(2788)] = 58575,
  [SMALL_STATE(2789)] = 58642,
  [SMALL_STATE(2790)] = 58709,
  [SMALL_STATE(2791)] = 58780,
  [SMALL_STATE(2792)] = 58847,
  [SMALL_STATE(2793)] = 58914,
  [SMALL_STATE(2794)] = 58985,
  [SMALL_STATE(2795)] = 59051,
  [SMALL_STATE(2796)] = 59113,
  [SMALL_STATE(2797)] = 59175,
  [SMALL_STATE(2798)] = 59245,
  [SMALL_STATE(2799)] = 59307,
  [SMALL_STATE(2800)] = 59369,
  [SMALL_STATE(2801)] = 59437,
  [SMALL_STATE(2802)] = 59500,
  [SMALL_STATE(2803)] = 59561,
  [SMALL_STATE(2804)] = 59622,
  [SMALL_STATE(2805)] = 59683,
  [SMALL_STATE(2806)] = 59744,
  [SMALL_STATE(2807)] = 59831,
  [SMALL_STATE(2808)] = 59892,
  [SMALL_STATE(2809)] = 59979,
  [SMALL_STATE(2810)] = 60040,
  [SMALL_STATE(2811)] = 60103,
  [SMALL_STATE(2812)] = 60166,
  [SMALL_STATE(2813)] = 60229,
  [SMALL_STATE(2814)] = 60294,
  [SMALL_STATE(2815)] = 60381,
  [SMALL_STATE(2816)] = 60442,
  [SMALL_STATE(2817)] = 60507,
  [SMALL_STATE(2818)] = 60568,
  [SMALL_STATE(2819)] = 60631,
  [SMALL_STATE(2820)] = 60692,
  [SMALL_STATE(2821)] = 60755,
  [SMALL_STATE(2822)] = 60818,
  [SMALL_STATE(2823)] = 60879,
  [SMALL_STATE(2824)] = 60940,
  [SMALL_STATE(2825)] = 61027,
  [SMALL_STATE(2826)] = 61088,
  [SMALL_STATE(2827)] = 61149,
  [SMALL_STATE(2828)] = 61212,
  [SMALL_STATE(2829)] = 61273,
  [SMALL_STATE(2830)] = 61362,
  [SMALL_STATE(2831)] = 61449,
  [SMALL_STATE(2832)] = 61536,
  [SMALL_STATE(2833)] = 61599,
  [SMALL_STATE(2834)] = 61662,
  [SMALL_STATE(2835)] = 61737,
  [SMALL_STATE(2836)] = 61800,
  [SMALL_STATE(2837)] = 61865,
  [SMALL_STATE(2838)] = 61928,
  [SMALL_STATE(2839)] = 62015,
  [SMALL_STATE(2840)] = 62076,
  [SMALL_STATE(2841)] = 62139,
  [SMALL_STATE(2842)] = 62202,
  [SMALL_STATE(2843)] = 62289,
  [SMALL_STATE(2844)] = 62350,
  [SMALL_STATE(2845)] = 62411,
  [SMALL_STATE(2846)] = 62472,
  [SMALL_STATE(2847)] = 62533,
  [SMALL_STATE(2848)] = 62594,
  [SMALL_STATE(2849)] = 62655,
  [SMALL_STATE(2850)] = 62716,
  [SMALL_STATE(2851)] = 62803,
  [SMALL_STATE(2852)] = 62890,
  [SMALL_STATE(2853)] = 62977,
  [SMALL_STATE(2854)] = 63064,
  [SMALL_STATE(2855)] = 63125,
  [SMALL_STATE(2856)] = 63212,
  [SMALL_STATE(2857)] = 63299,
  [SMALL_STATE(2858)] = 63386,
  [SMALL_STATE(2859)] = 63473,
  [SMALL_STATE(2860)] = 63536,
  [SMALL_STATE(2861)] = 63596,
  [SMALL_STATE(2862)] = 63656,
  [SMALL_STATE(2863)] = 63716,
  [SMALL_STATE(2864)] = 63776,
  [SMALL_STATE(2865)] = 63836,
  [SMALL_STATE(2866)] = 63896,
  [SMALL_STATE(2867)] = 63970,
  [SMALL_STATE(2868)] = 64030,
  [SMALL_STATE(2869)] = 64090,
  [SMALL_STATE(2870)] = 64158,
  [SMALL_STATE(2871)] = 64218,
  [SMALL_STATE(2872)] = 64278,
  [SMALL_STATE(2873)] = 64338,
  [SMALL_STATE(2874)] = 64398,
  [SMALL_STATE(2875)] = 64458,
  [SMALL_STATE(2876)] = 64518,
  [SMALL_STATE(2877)] = 64578,
  [SMALL_STATE(2878)] = 64638,
  [SMALL_STATE(2879)] = 64698,
  [SMALL_STATE(2880)] = 64758,
  [SMALL_STATE(2881)] = 64818,
  [SMALL_STATE(2882)] = 64878,
  [SMALL_STATE(2883)] = 64938,
  [SMALL_STATE(2884)] = 65000,
  [SMALL_STATE(2885)] = 65060,
  [SMALL_STATE(2886)] = 65120,
  [SMALL_STATE(2887)] = 65180,
  [SMALL_STATE(2888)] = 65240,
  [SMALL_STATE(2889)] = 65306,
  [SMALL_STATE(2890)] = 65366,
  [SMALL_STATE(2891)] = 65426,
  [SMALL_STATE(2892)] = 65486,
  [SMALL_STATE(2893)] = 65546,
  [SMALL_STATE(2894)] = 65606,
  [SMALL_STATE(2895)] = 65666,
  [SMALL_STATE(2896)] = 65726,
  [SMALL_STATE(2897)] = 65792,
  [SMALL_STATE(2898)] = 65852,
  [SMALL_STATE(2899)] = 65912,
  [SMALL_STATE(2900)] = 65972,
  [SMALL_STATE(2901)] = 66032,
  [SMALL_STATE(2902)] = 66092,
  [SMALL_STATE(2903)] = 66178,
  [SMALL_STATE(2904)] = 66238,
  [SMALL_STATE(2905)] = 66298,
  [SMALL_STATE(2906)] = 66358,
  [SMALL_STATE(2907)] = 66418,
  [SMALL_STATE(2908)] = 66478,
  [SMALL_STATE(2909)] = 66538,
  [SMALL_STATE(2910)] = 66598,
  [SMALL_STATE(2911)] = 66658,
  [SMALL_STATE(2912)] = 66718,
  [SMALL_STATE(2913)] = 66778,
  [SMALL_STATE(2914)] = 66838,
  [SMALL_STATE(2915)] = 66898,
  [SMALL_STATE(2916)] = 66958,
  [SMALL_STATE(2917)] = 67024,
  [SMALL_STATE(2918)] = 67084,
  [SMALL_STATE(2919)] = 67156,
  [SMALL_STATE(2920)] = 67216,
  [SMALL_STATE(2921)] = 67276,
  [SMALL_STATE(2922)] = 67336,
  [SMALL_STATE(2923)] = 67396,
  [SMALL_STATE(2924)] = 67456,
  [SMALL_STATE(2925)] = 67516,
  [SMALL_STATE(2926)] = 67576,
  [SMALL_STATE(2927)] = 67636,
  [SMALL_STATE(2928)] = 67696,
  [SMALL_STATE(2929)] = 67756,
  [SMALL_STATE(2930)] = 67816,
  [SMALL_STATE(2931)] = 67876,
  [SMALL_STATE(2932)] = 67936,
  [SMALL_STATE(2933)] = 67996,
  [SMALL_STATE(2934)] = 68056,
  [SMALL_STATE(2935)] = 68116,
  [SMALL_STATE(2936)] = 68176,
  [SMALL_STATE(2937)] = 68236,
  [SMALL_STATE(2938)] = 68296,
  [SMALL_STATE(2939)] = 68356,
  [SMALL_STATE(2940)] = 68416,
  [SMALL_STATE(2941)] = 68476,
  [SMALL_STATE(2942)] = 68536,
  [SMALL_STATE(2943)] = 68596,
  [SMALL_STATE(2944)] = 68656,
  [SMALL_STATE(2945)] = 68720,
  [SMALL_STATE(2946)] = 68780,
  [SMALL_STATE(2947)] = 68844,
  [SMALL_STATE(2948)] = 68904,
  [SMALL_STATE(2949)] = 68964,
  [SMALL_STATE(2950)] = 69024,
  [SMALL_STATE(2951)] = 69084,
  [SMALL_STATE(2952)] = 69144,
  [SMALL_STATE(2953)] = 69204,
  [SMALL_STATE(2954)] = 69264,
  [SMALL_STATE(2955)] = 69324,
  [SMALL_STATE(2956)] = 69384,
  [SMALL_STATE(2957)] = 69444,
  [SMALL_STATE(2958)] = 69504,
  [SMALL_STATE(2959)] = 69564,
  [SMALL_STATE(2960)] = 69624,
  [SMALL_STATE(2961)] = 69684,
  [SMALL_STATE(2962)] = 69744,
  [SMALL_STATE(2963)] = 69804,
  [SMALL_STATE(2964)] = 69890,
  [SMALL_STATE(2965)] = 69950,
  [SMALL_STATE(2966)] = 70012,
  [SMALL_STATE(2967)] = 70072,
  [SMALL_STATE(2968)] = 70132,
  [SMALL_STATE(2969)] = 70192,
  [SMALL_STATE(2970)] = 70252,
  [SMALL_STATE(2971)] = 70312,
  [SMALL_STATE(2972)] = 70384,
  [SMALL_STATE(2973)] = 70444,
  [SMALL_STATE(2974)] = 70504,
  [SMALL_STATE(2975)] = 70564,
  [SMALL_STATE(2976)] = 70624,
  [SMALL_STATE(2977)] = 70684,
  [SMALL_STATE(2978)] = 70744,
  [SMALL_STATE(2979)] = 70804,
  [SMALL_STATE(2980)] = 70864,
  [SMALL_STATE(2981)] = 70924,
  [SMALL_STATE(2982)] = 70984,
  [SMALL_STATE(2983)] = 71044,
  [SMALL_STATE(2984)] = 71104,
  [SMALL_STATE(2985)] = 71164,
  [SMALL_STATE(2986)] = 71230,
  [SMALL_STATE(2987)] = 71290,
  [SMALL_STATE(2988)] = 71350,
  [SMALL_STATE(2989)] = 71435,
  [SMALL_STATE(2990)] = 71520,
  [SMALL_STATE(2991)] = 71605,
  [SMALL_STATE(2992)] = 71666,
  [SMALL_STATE(2993)] = 71751,
  [SMALL_STATE(2994)] = 71834,
  [SMALL_STATE(2995)] = 71917,
  [SMALL_STATE(2996)] = 71986,
  [SMALL_STATE(2997)] = 72069,
  [SMALL_STATE(2998)] = 72152,
  [SMALL_STATE(2999)] = 72235,
  [SMALL_STATE(3000)] = 72318,
  [SMALL_STATE(3001)] = 72403,
  [SMALL_STATE(3002)] = 72486,
  [SMALL_STATE(3003)] = 72571,
  [SMALL_STATE(3004)] = 72656,
  [SMALL_STATE(3005)] = 72741,
  [SMALL_STATE(3006)] = 72812,
  [SMALL_STATE(3007)] = 72883,
  [SMALL_STATE(3008)] = 72966,
  [SMALL_STATE(3009)] = 73049,
  [SMALL_STATE(3010)] = 73110,
  [SMALL_STATE(3011)] = 73195,
  [SMALL_STATE(3012)] = 73280,
  [SMALL_STATE(3013)] = 73365,
  [SMALL_STATE(3014)] = 73450,
  [SMALL_STATE(3015)] = 73535,
  [SMALL_STATE(3016)] = 73620,
  [SMALL_STATE(3017)] = 73705,
  [SMALL_STATE(3018)] = 73790,
  [SMALL_STATE(3019)] = 73882,
  [SMALL_STATE(3020)] = 73954,
  [SMALL_STATE(3021)] = 74030,
  [SMALL_STATE(3022)] = 74120,
  [SMALL_STATE(3023)] = 74188,
  [SMALL_STATE(3024)] = 74266,
  [SMALL_STATE(3025)] = 74356,
  [SMALL_STATE(3026)] = 74448,
  [SMALL_STATE(3027)] = 74538,
  [SMALL_STATE(3028)] = 74628,
  [SMALL_STATE(3029)] = 74696,
  [SMALL_STATE(3030)] = 74778,
  [SMALL_STATE(3031)] = 74844,
  [SMALL_STATE(3032)] = 74914,
  [SMALL_STATE(3033)] = 74984,
  [SMALL_STATE(3034)] = 75074,
  [SMALL_STATE(3035)] = 75140,
  [SMALL_STATE(3036)] = 75210,
  [SMALL_STATE(3037)] = 75296,
  [SMALL_STATE(3038)] = 75386,
  [SMALL_STATE(3039)] = 75478,
  [SMALL_STATE(3040)] = 75562,
  [SMALL_STATE(3041)] = 75632,
  [SMALL_STATE(3042)] = 75706,
  [SMALL_STATE(3043)] = 75796,
  [SMALL_STATE(3044)] = 75860,
  [SMALL_STATE(3045)] = 75952,
  [SMALL_STATE(3046)] = 76038,
  [SMALL_STATE(3047)] = 76101,
  [SMALL_STATE(3048)] = 76164,
  [SMALL_STATE(3049)] = 76227,
  [SMALL_STATE(3050)] = 76290,
  [SMALL_STATE(3051)] = 76357,
  [SMALL_STATE(3052)] = 76420,
  [SMALL_STATE(3053)] = 76483,
  [SMALL_STATE(3054)] = 76548,
  [SMALL_STATE(3055)] = 76611,
  [SMALL_STATE(3056)] = 76671,
  [SMALL_STATE(3057)] = 76731,
  [SMALL_STATE(3058)] = 76791,
  [SMALL_STATE(3059)] = 76855,
  [SMALL_STATE(3060)] = 76911,
  [SMALL_STATE(3061)] = 76967,
  [SMALL_STATE(3062)] = 77025,
  [SMALL_STATE(3063)] = 77089,
  [SMALL_STATE(3064)] = 77153,
  [SMALL_STATE(3065)] = 77213,
  [SMALL_STATE(3066)] = 77269,
  [SMALL_STATE(3067)] = 77329,
  [SMALL_STATE(3068)] = 77389,
  [SMALL_STATE(3069)] = 77447,
  [SMALL_STATE(3070)] = 77503,
  [SMALL_STATE(3071)] = 77563,
  [SMALL_STATE(3072)] = 77621,
  [SMALL_STATE(3073)] = 77681,
  [SMALL_STATE(3074)] = 77741,
  [SMALL_STATE(3075)] = 77805,
  [SMALL_STATE(3076)] = 77863,
  [SMALL_STATE(3077)] = 77923,
  [SMALL_STATE(3078)] = 77978,
  [SMALL_STATE(3079)] = 78055,
  [SMALL_STATE(3080)] = 78132,
  [SMALL_STATE(3081)] = 78191,
  [SMALL_STATE(3082)] = 78250,
  [SMALL_STATE(3083)] = 78305,
  [SMALL_STATE(3084)] = 78364,
  [SMALL_STATE(3085)] = 78421,
  [SMALL_STATE(3086)] = 78516,
  [SMALL_STATE(3087)] = 78573,
  [SMALL_STATE(3088)] = 78632,
  [SMALL_STATE(3089)] = 78727,
  [SMALL_STATE(3090)] = 78784,
  [SMALL_STATE(3091)] = 78841,
  [SMALL_STATE(3092)] = 78936,
  [SMALL_STATE(3093)] = 79031,
  [SMALL_STATE(3094)] = 79126,
  [SMALL_STATE(3095)] = 79221,
  [SMALL_STATE(3096)] = 79298,
  [SMALL_STATE(3097)] = 79353,
  [SMALL_STATE(3098)] = 79408,
  [SMALL_STATE(3099)] = 79503,
  [SMALL_STATE(3100)] = 79560,
  [SMALL_STATE(3101)] = 79619,
  [SMALL_STATE(3102)] = 79714,
  [SMALL_STATE(3103)] = 79769,
  [SMALL_STATE(3104)] = 79826,
  [SMALL_STATE(3105)] = 79903,
  [SMALL_STATE(3106)] = 79958,
  [SMALL_STATE(3107)] = 80015,
  [SMALL_STATE(3108)] = 80092,
  [SMALL_STATE(3109)] = 80147,
  [SMALL_STATE(3110)] = 80224,
  [SMALL_STATE(3111)] = 80319,
  [SMALL_STATE(3112)] = 80396,
  [SMALL_STATE(3113)] = 80491,
  [SMALL_STATE(3114)] = 80546,
  [SMALL_STATE(3115)] = 80623,
  [SMALL_STATE(3116)] = 80700,
  [SMALL_STATE(3117)] = 80795,
  [SMALL_STATE(3118)] = 80872,
  [SMALL_STATE(3119)] = 80949,
  [SMALL_STATE(3120)] = 81044,
  [SMALL_STATE(3121)] = 81121,
  [SMALL_STATE(3122)] = 81198,
  [SMALL_STATE(3123)] = 81275,
  [SMALL_STATE(3124)] = 81330,
  [SMALL_STATE(3125)] = 81407,
  [SMALL_STATE(3126)] = 81484,
  [SMALL_STATE(3127)] = 81552,
  [SMALL_STATE(3128)] = 81608,
  [SMALL_STATE(3129)] = 81690,
  [SMALL_STATE(3130)] = 81746,
  [SMALL_STATE(3131)] = 81802,
  [SMALL_STATE(3132)] = 81888,
  [SMALL_STATE(3133)] = 81944,
  [SMALL_STATE(3134)] = 82030,
  [SMALL_STATE(3135)] = 82086,
  [SMALL_STATE(3136)] = 82142,
  [SMALL_STATE(3137)] = 82198,
  [SMALL_STATE(3138)] = 82290,
  [SMALL_STATE(3139)] = 82376,
  [SMALL_STATE(3140)] = 82468,
  [SMALL_STATE(3141)] = 82524,
  [SMALL_STATE(3142)] = 82612,
  [SMALL_STATE(3143)] = 82666,
  [SMALL_STATE(3144)] = 82758,
  [SMALL_STATE(3145)] = 82812,
  [SMALL_STATE(3146)] = 82868,
  [SMALL_STATE(3147)] = 82922,
  [SMALL_STATE(3148)] = 82986,
  [SMALL_STATE(3149)] = 83072,
  [SMALL_STATE(3150)] = 83126,
  [SMALL_STATE(3151)] = 83180,
  [SMALL_STATE(3152)] = 83234,
  [SMALL_STATE(3153)] = 83304,
  [SMALL_STATE(3154)] = 83392,
  [SMALL_STATE(3155)] = 83480,
  [SMALL_STATE(3156)] = 83552,
  [SMALL_STATE(3157)] = 83608,
  [SMALL_STATE(3158)] = 83662,
  [SMALL_STATE(3159)] = 83740,
  [SMALL_STATE(3160)] = 83796,
  [SMALL_STATE(3161)] = 83876,
  [SMALL_STATE(3162)] = 83930,
  [SMALL_STATE(3163)] = 83984,
  [SMALL_STATE(3164)] = 84038,
  [SMALL_STATE(3165)] = 84104,
  [SMALL_STATE(3166)] = 84158,
  [SMALL_STATE(3167)] = 84244,
  [SMALL_STATE(3168)] = 84298,
  [SMALL_STATE(3169)] = 84354,
  [SMALL_STATE(3170)] = 84408,
  [SMALL_STATE(3171)] = 84494,
  [SMALL_STATE(3172)] = 84550,
  [SMALL_STATE(3173)] = 84642,
  [SMALL_STATE(3174)] = 84716,
  [SMALL_STATE(3175)] = 84802,
  [SMALL_STATE(3176)] = 84856,
  [SMALL_STATE(3177)] = 84910,
  [SMALL_STATE(3178)] = 84964,
  [SMALL_STATE(3179)] = 85022,
  [SMALL_STATE(3180)] = 85076,
  [SMALL_STATE(3181)] = 85164,
  [SMALL_STATE(3182)] = 85246,
  [SMALL_STATE(3183)] = 85300,
  [SMALL_STATE(3184)] = 85389,
  [SMALL_STATE(3185)] = 85442,
  [SMALL_STATE(3186)] = 85495,
  [SMALL_STATE(3187)] = 85548,
  [SMALL_STATE(3188)] = 85601,
  [SMALL_STATE(3189)] = 85654,
  [SMALL_STATE(3190)] = 85743,
  [SMALL_STATE(3191)] = 85832,
  [SMALL_STATE(3192)] = 85885,
  [SMALL_STATE(3193)] = 85974,
  [SMALL_STATE(3194)] = 86027,
  [SMALL_STATE(3195)] = 86080,
  [SMALL_STATE(3196)] = 86133,
  [SMALL_STATE(3197)] = 86222,
  [SMALL_STATE(3198)] = 86311,
  [SMALL_STATE(3199)] = 86396,
  [SMALL_STATE(3200)] = 86483,
  [SMALL_STATE(3201)] = 86536,
  [SMALL_STATE(3202)] = 86625,
  [SMALL_STATE(3203)] = 86714,
  [SMALL_STATE(3204)] = 86767,
  [SMALL_STATE(3205)] = 86854,
  [SMALL_STATE(3206)] = 86907,
  [SMALL_STATE(3207)] = 86996,
  [SMALL_STATE(3208)] = 87085,
  [SMALL_STATE(3209)] = 87174,
  [SMALL_STATE(3210)] = 87227,
  [SMALL_STATE(3211)] = 87280,
  [SMALL_STATE(3212)] = 87367,
  [SMALL_STATE(3213)] = 87420,
  [SMALL_STATE(3214)] = 87473,
  [SMALL_STATE(3215)] = 87544,
  [SMALL_STATE(3216)] = 87633,
  [SMALL_STATE(3217)] = 87720,
  [SMALL_STATE(3218)] = 87805,
  [SMALL_STATE(3219)] = 87858,
  [SMALL_STATE(3220)] = 87939,
  [SMALL_STATE(3221)] = 88024,
  [SMALL_STATE(3222)] = 88109,
  [SMALL_STATE(3223)] = 88198,
  [SMALL_STATE(3224)] = 88283,
  [SMALL_STATE(3225)] = 88368,
  [SMALL_STATE(3226)] = 88457,
  [SMALL_STATE(3227)] = 88510,
  [SMALL_STATE(3228)] = 88563,
  [SMALL_STATE(3229)] = 88616,
  [SMALL_STATE(3230)] = 88705,
  [SMALL_STATE(3231)] = 88758,
  [SMALL_STATE(3232)] = 88811,
  [SMALL_STATE(3233)] = 88864,
  [SMALL_STATE(3234)] = 88951,
  [SMALL_STATE(3235)] = 89038,
  [SMALL_STATE(3236)] = 89091,
  [SMALL_STATE(3237)] = 89144,
  [SMALL_STATE(3238)] = 89197,
  [SMALL_STATE(3239)] = 89250,
  [SMALL_STATE(3240)] = 89339,
  [SMALL_STATE(3241)] = 89392,
  [SMALL_STATE(3242)] = 89479,
  [SMALL_STATE(3243)] = 89568,
  [SMALL_STATE(3244)] = 89653,
  [SMALL_STATE(3245)] = 89706,
  [SMALL_STATE(3246)] = 89759,
  [SMALL_STATE(3247)] = 89844,
  [SMALL_STATE(3248)] = 89933,
  [SMALL_STATE(3249)] = 90022,
  [SMALL_STATE(3250)] = 90111,
  [SMALL_STATE(3251)] = 90164,
  [SMALL_STATE(3252)] = 90217,
  [SMALL_STATE(3253)] = 90270,
  [SMALL_STATE(3254)] = 90359,
  [SMALL_STATE(3255)] = 90448,
  [SMALL_STATE(3256)] = 90537,
  [SMALL_STATE(3257)] = 90626,
  [SMALL_STATE(3258)] = 90713,
  [SMALL_STATE(3259)] = 90802,
  [SMALL_STATE(3260)] = 90891,
  [SMALL_STATE(3261)] = 90980,
  [SMALL_STATE(3262)] = 91069,
  [SMALL_STATE(3263)] = 91122,
  [SMALL_STATE(3264)] = 91175,
  [SMALL_STATE(3265)] = 91228,
  [SMALL_STATE(3266)] = 91315,
  [SMALL_STATE(3267)] = 91404,
  [SMALL_STATE(3268)] = 91457,
  [SMALL_STATE(3269)] = 91546,
  [SMALL_STATE(3270)] = 91633,
  [SMALL_STATE(3271)] = 91722,
  [SMALL_STATE(3272)] = 91811,
  [SMALL_STATE(3273)] = 91900,
  [SMALL_STATE(3274)] = 91953,
  [SMALL_STATE(3275)] = 92042,
  [SMALL_STATE(3276)] = 92095,
  [SMALL_STATE(3277)] = 92184,
  [SMALL_STATE(3278)] = 92273,
  [SMALL_STATE(3279)] = 92326,
  [SMALL_STATE(3280)] = 92379,
  [SMALL_STATE(3281)] = 92468,
  [SMALL_STATE(3282)] = 92557,
  [SMALL_STATE(3283)] = 92646,
  [SMALL_STATE(3284)] = 92699,
  [SMALL_STATE(3285)] = 92788,
  [SMALL_STATE(3286)] = 92877,
  [SMALL_STATE(3287)] = 92930,
  [SMALL_STATE(3288)] = 93017,
  [SMALL_STATE(3289)] = 93070,
  [SMALL_STATE(3290)] = 93141,
  [SMALL_STATE(3291)] = 93230,
  [SMALL_STATE(3292)] = 93315,
  [SMALL_STATE(3293)] = 93368,
  [SMALL_STATE(3294)] = 93421,
  [SMALL_STATE(3295)] = 93474,
  [SMALL_STATE(3296)] = 93545,
  [SMALL_STATE(3297)] = 93616,
  [SMALL_STATE(3298)] = 93669,
  [SMALL_STATE(3299)] = 93722,
  [SMALL_STATE(3300)] = 93775,
  [SMALL_STATE(3301)] = 93862,
  [SMALL_STATE(3302)] = 93915,
  [SMALL_STATE(3303)] = 93968,
  [SMALL_STATE(3304)] = 94021,
  [SMALL_STATE(3305)] = 94074,
  [SMALL_STATE(3306)] = 94127,
  [SMALL_STATE(3307)] = 94180,
  [SMALL_STATE(3308)] = 94239,
  [SMALL_STATE(3309)] = 94292,
  [SMALL_STATE(3310)] = 94345,
  [SMALL_STATE(3311)] = 94398,
  [SMALL_STATE(3312)] = 94457,
  [SMALL_STATE(3313)] = 94544,
  [SMALL_STATE(3314)] = 94631,
  [SMALL_STATE(3315)] = 94718,
  [SMALL_STATE(3316)] = 94807,
  [SMALL_STATE(3317)] = 94860,
  [SMALL_STATE(3318)] = 94931,
  [SMALL_STATE(3319)] = 94984,
  [SMALL_STATE(3320)] = 95037,
  [SMALL_STATE(3321)] = 95126,
  [SMALL_STATE(3322)] = 95179,
  [SMALL_STATE(3323)] = 95250,
  [SMALL_STATE(3324)] = 95303,
  [SMALL_STATE(3325)] = 95392,
  [SMALL_STATE(3326)] = 95479,
  [SMALL_STATE(3327)] = 95532,
  [SMALL_STATE(3328)] = 95585,
  [SMALL_STATE(3329)] = 95638,
  [SMALL_STATE(3330)] = 95725,
  [SMALL_STATE(3331)] = 95778,
  [SMALL_STATE(3332)] = 95831,
  [SMALL_STATE(3333)] = 95920,
  [SMALL_STATE(3334)] = 96009,
  [SMALL_STATE(3335)] = 96098,
  [SMALL_STATE(3336)] = 96185,
  [SMALL_STATE(3337)] = 96270,
  [SMALL_STATE(3338)] = 96323,
  [SMALL_STATE(3339)] = 96376,
  [SMALL_STATE(3340)] = 96429,
  [SMALL_STATE(3341)] = 96482,
  [SMALL_STATE(3342)] = 96535,
  [SMALL_STATE(3343)] = 96588,
  [SMALL_STATE(3344)] = 96641,
  [SMALL_STATE(3345)] = 96694,
  [SMALL_STATE(3346)] = 96747,
  [SMALL_STATE(3347)] = 96800,
  [SMALL_STATE(3348)] = 96853,
  [SMALL_STATE(3349)] = 96906,
  [SMALL_STATE(3350)] = 96959,
  [SMALL_STATE(3351)] = 97048,
  [SMALL_STATE(3352)] = 97111,
  [SMALL_STATE(3353)] = 97180,
  [SMALL_STATE(3354)] = 97247,
  [SMALL_STATE(3355)] = 97318,
  [SMALL_STATE(3356)] = 97371,
  [SMALL_STATE(3357)] = 97460,
  [SMALL_STATE(3358)] = 97519,
  [SMALL_STATE(3359)] = 97608,
  [SMALL_STATE(3360)] = 97661,
  [SMALL_STATE(3361)] = 97750,
  [SMALL_STATE(3362)] = 97839,
  [SMALL_STATE(3363)] = 97916,
  [SMALL_STATE(3364)] = 97995,
  [SMALL_STATE(3365)] = 98060,
  [SMALL_STATE(3366)] = 98145,
  [SMALL_STATE(3367)] = 98198,
  [SMALL_STATE(3368)] = 98283,
  [SMALL_STATE(3369)] = 98370,
  [SMALL_STATE(3370)] = 98443,
  [SMALL_STATE(3371)] = 98502,
  [SMALL_STATE(3372)] = 98555,
  [SMALL_STATE(3373)] = 98636,
  [SMALL_STATE(3374)] = 98689,
  [SMALL_STATE(3375)] = 98742,
  [SMALL_STATE(3376)] = 98831,
  [SMALL_STATE(3377)] = 98884,
  [SMALL_STATE(3378)] = 98937,
  [SMALL_STATE(3379)] = 98990,
  [SMALL_STATE(3380)] = 99043,
  [SMALL_STATE(3381)] = 99096,
  [SMALL_STATE(3382)] = 99149,
  [SMALL_STATE(3383)] = 99236,
  [SMALL_STATE(3384)] = 99321,
  [SMALL_STATE(3385)] = 99410,
  [SMALL_STATE(3386)] = 99499,
  [SMALL_STATE(3387)] = 99584,
  [SMALL_STATE(3388)] = 99671,
  [SMALL_STATE(3389)] = 99724,
  [SMALL_STATE(3390)] = 99811,
  [SMALL_STATE(3391)] = 99874,
  [SMALL_STATE(3392)] = 99943,
  [SMALL_STATE(3393)] = 100010,
  [SMALL_STATE(3394)] = 100081,
  [SMALL_STATE(3395)] = 100158,
  [SMALL_STATE(3396)] = 100237,
  [SMALL_STATE(3397)] = 100302,
  [SMALL_STATE(3398)] = 100387,
  [SMALL_STATE(3399)] = 100472,
  [SMALL_STATE(3400)] = 100545,
  [SMALL_STATE(3401)] = 100626,
  [SMALL_STATE(3402)] = 100715,
  [SMALL_STATE(3403)] = 100768,
  [SMALL_STATE(3404)] = 100821,
  [SMALL_STATE(3405)] = 100908,
  [SMALL_STATE(3406)] = 100995,
  [SMALL_STATE(3407)] = 101048,
  [SMALL_STATE(3408)] = 101101,
  [SMALL_STATE(3409)] = 101154,
  [SMALL_STATE(3410)] = 101215,
  [SMALL_STATE(3411)] = 101286,
  [SMALL_STATE(3412)] = 101357,
  [SMALL_STATE(3413)] = 101410,
  [SMALL_STATE(3414)] = 101481,
  [SMALL_STATE(3415)] = 101534,
  [SMALL_STATE(3416)] = 101587,
  [SMALL_STATE(3417)] = 101676,
  [SMALL_STATE(3418)] = 101747,
  [SMALL_STATE(3419)] = 101800,
  [SMALL_STATE(3420)] = 101887,
  [SMALL_STATE(3421)] = 101940,
  [SMALL_STATE(3422)] = 102011,
  [SMALL_STATE(3423)] = 102082,
  [SMALL_STATE(3424)] = 102135,
  [SMALL_STATE(3425)] = 102188,
  [SMALL_STATE(3426)] = 102277,
  [SMALL_STATE(3427)] = 102330,
  [SMALL_STATE(3428)] = 102417,
  [SMALL_STATE(3429)] = 102506,
  [SMALL_STATE(3430)] = 102559,
  [SMALL_STATE(3431)] = 102612,
  [SMALL_STATE(3432)] = 102665,
  [SMALL_STATE(3433)] = 102718,
  [SMALL_STATE(3434)] = 102805,
  [SMALL_STATE(3435)] = 102894,
  [SMALL_STATE(3436)] = 102975,
  [SMALL_STATE(3437)] = 103028,
  [SMALL_STATE(3438)] = 103081,
  [SMALL_STATE(3439)] = 103170,
  [SMALL_STATE(3440)] = 103256,
  [SMALL_STATE(3441)] = 103324,
  [SMALL_STATE(3442)] = 103410,
  [SMALL_STATE(3443)] = 103496,
  [SMALL_STATE(3444)] = 103582,
  [SMALL_STATE(3445)] = 103668,
  [SMALL_STATE(3446)] = 103754,
  [SMALL_STATE(3447)] = 103840,
  [SMALL_STATE(3448)] = 103926,
  [SMALL_STATE(3449)] = 104012,
  [SMALL_STATE(3450)] = 104096,
  [SMALL_STATE(3451)] = 104164,
  [SMALL_STATE(3452)] = 104250,
  [SMALL_STATE(3453)] = 104336,
  [SMALL_STATE(3454)] = 104422,
  [SMALL_STATE(3455)] = 104508,
  [SMALL_STATE(3456)] = 104594,
  [SMALL_STATE(3457)] = 104680,
  [SMALL_STATE(3458)] = 104766,
  [SMALL_STATE(3459)] = 104850,
  [SMALL_STATE(3460)] = 104936,
  [SMALL_STATE(3461)] = 105022,
  [SMALL_STATE(3462)] = 105108,
  [SMALL_STATE(3463)] = 105194,
  [SMALL_STATE(3464)] = 105280,
  [SMALL_STATE(3465)] = 105366,
  [SMALL_STATE(3466)] = 105452,
  [SMALL_STATE(3467)] = 105538,
  [SMALL_STATE(3468)] = 105624,
  [SMALL_STATE(3469)] = 105710,
  [SMALL_STATE(3470)] = 105796,
  [SMALL_STATE(3471)] = 105882,
  [SMALL_STATE(3472)] = 105968,
  [SMALL_STATE(3473)] = 106054,
  [SMALL_STATE(3474)] = 106140,
  [SMALL_STATE(3475)] = 106226,
  [SMALL_STATE(3476)] = 106312,
  [SMALL_STATE(3477)] = 106398,
  [SMALL_STATE(3478)] = 106484,
  [SMALL_STATE(3479)] = 106570,
  [SMALL_STATE(3480)] = 106656,
  [SMALL_STATE(3481)] = 106742,
  [SMALL_STATE(3482)] = 106828,
  [SMALL_STATE(3483)] = 106914,
  [SMALL_STATE(3484)] = 107000,
  [SMALL_STATE(3485)] = 107086,
  [SMALL_STATE(3486)] = 107172,
  [SMALL_STATE(3487)] = 107258,
  [SMALL_STATE(3488)] = 107344,
  [SMALL_STATE(3489)] = 107430,
  [SMALL_STATE(3490)] = 107513,
  [SMALL_STATE(3491)] = 107596,
  [SMALL_STATE(3492)] = 107679,
  [SMALL_STATE(3493)] = 107762,
  [SMALL_STATE(3494)] = 107845,
  [SMALL_STATE(3495)] = 107928,
  [SMALL_STATE(3496)] = 108011,
  [SMALL_STATE(3497)] = 108094,
  [SMALL_STATE(3498)] = 108132,
  [SMALL_STATE(3499)] = 108202,
  [SMALL_STATE(3500)] = 108272,
  [SMALL_STATE(3501)] = 108342,
  [SMALL_STATE(3502)] = 108412,
  [SMALL_STATE(3503)] = 108447,
  [SMALL_STATE(3504)] = 108484,
  [SMALL_STATE(3505)] = 108521,
  [SMALL_STATE(3506)] = 108558,
  [SMALL_STATE(3507)] = 108611,
  [SMALL_STATE(3508)] = 108648,
  [SMALL_STATE(3509)] = 108684,
  [SMALL_STATE(3510)] = 108716,
  [SMALL_STATE(3511)] = 108750,
  [SMALL_STATE(3512)] = 108784,
  [SMALL_STATE(3513)] = 108820,
  [SMALL_STATE(3514)] = 108864,
  [SMALL_STATE(3515)] = 108898,
  [SMALL_STATE(3516)] = 108932,
  [SMALL_STATE(3517)] = 108966,
  [SMALL_STATE(3518)] = 109000,
  [SMALL_STATE(3519)] = 109048,
  [SMALL_STATE(3520)] = 109092,
  [SMALL_STATE(3521)] = 109136,
  [SMALL_STATE(3522)] = 109172,
  [SMALL_STATE(3523)] = 109206,
  [SMALL_STATE(3524)] = 109238,
  [SMALL_STATE(3525)] = 109274,
  [SMALL_STATE(3526)] = 109306,
  [SMALL_STATE(3527)] = 109364,
  [SMALL_STATE(3528)] = 109422,
  [SMALL_STATE(3529)] = 109477,
  [SMALL_STATE(3530)] = 109532,
  [SMALL_STATE(3531)] = 109587,
  [SMALL_STATE(3532)] = 109632,
  [SMALL_STATE(3533)] = 109687,
  [SMALL_STATE(3534)] = 109720,
  [SMALL_STATE(3535)] = 109753,
  [SMALL_STATE(3536)] = 109794,
  [SMALL_STATE(3537)] = 109849,
  [SMALL_STATE(3538)] = 109900,
  [SMALL_STATE(3539)] = 109955,
  [SMALL_STATE(3540)] = 109986,
  [SMALL_STATE(3541)] = 110041,
  [SMALL_STATE(3542)] = 110074,
  [SMALL_STATE(3543)] = 110129,
  [SMALL_STATE(3544)] = 110184,
  [SMALL_STATE(3545)] = 110217,
  [SMALL_STATE(3546)] = 110261,
  [SMALL_STATE(3547)] = 110297,
  [SMALL_STATE(3548)] = 110353,
  [SMALL_STATE(3549)] = 110409,
  [SMALL_STATE(3550)] = 110455,
  [SMALL_STATE(3551)] = 110493,
  [SMALL_STATE(3552)] = 110531,
  [SMALL_STATE(3553)] = 110569,
  [SMALL_STATE(3554)] = 110607,
  [SMALL_STATE(3555)] = 110663,
  [SMALL_STATE(3556)] = 110695,
  [SMALL_STATE(3557)] = 110727,
  [SMALL_STATE(3558)] = 110783,
  [SMALL_STATE(3559)] = 110815,
  [SMALL_STATE(3560)] = 110867,
  [SMALL_STATE(3561)] = 110920,
  [SMALL_STATE(3562)] = 110949,
  [SMALL_STATE(3563)] = 110978,
  [SMALL_STATE(3564)] = 111031,
  [SMALL_STATE(3565)] = 111084,
  [SMALL_STATE(3566)] = 111113,
  [SMALL_STATE(3567)] = 111142,
  [SMALL_STATE(3568)] = 111175,
  [SMALL_STATE(3569)] = 111228,
  [SMALL_STATE(3570)] = 111281,
  [SMALL_STATE(3571)] = 111326,
  [SMALL_STATE(3572)] = 111355,
  [SMALL_STATE(3573)] = 111384,
  [SMALL_STATE(3574)] = 111437,
  [SMALL_STATE(3575)] = 111488,
  [SMALL_STATE(3576)] = 111521,
  [SMALL_STATE(3577)] = 111576,
  [SMALL_STATE(3578)] = 111629,
  [SMALL_STATE(3579)] = 111662,
  [SMALL_STATE(3580)] = 111695,
  [SMALL_STATE(3581)] = 111724,
  [SMALL_STATE(3582)] = 111777,
  [SMALL_STATE(3583)] = 111806,
  [SMALL_STATE(3584)] = 111839,
  [SMALL_STATE(3585)] = 111890,
  [SMALL_STATE(3586)] = 111943,
  [SMALL_STATE(3587)] = 111976,
  [SMALL_STATE(3588)] = 112007,
  [SMALL_STATE(3589)] = 112060,
  [SMALL_STATE(3590)] = 112113,
  [SMALL_STATE(3591)] = 112166,
  [SMALL_STATE(3592)] = 112211,
  [SMALL_STATE(3593)] = 112252,
  [SMALL_STATE(3594)] = 112305,
  [SMALL_STATE(3595)] = 112358,
  [SMALL_STATE(3596)] = 112391,
  [SMALL_STATE(3597)] = 112422,
  [SMALL_STATE(3598)] = 112450,
  [SMALL_STATE(3599)] = 112478,
  [SMALL_STATE(3600)] = 112506,
  [SMALL_STATE(3601)] = 112558,
  [SMALL_STATE(3602)] = 112586,
  [SMALL_STATE(3603)] = 112634,
  [SMALL_STATE(3604)] = 112662,
  [SMALL_STATE(3605)] = 112710,
  [SMALL_STATE(3606)] = 112738,
  [SMALL_STATE(3607)] = 112790,
  [SMALL_STATE(3608)] = 112838,
  [SMALL_STATE(3609)] = 112866,
  [SMALL_STATE(3610)] = 112894,
  [SMALL_STATE(3611)] = 112932,
  [SMALL_STATE(3612)] = 112984,
  [SMALL_STATE(3613)] = 113012,
  [SMALL_STATE(3614)] = 113040,
  [SMALL_STATE(3615)] = 113068,
  [SMALL_STATE(3616)] = 113116,
  [SMALL_STATE(3617)] = 113158,
  [SMALL_STATE(3618)] = 113186,
  [SMALL_STATE(3619)] = 113214,
  [SMALL_STATE(3620)] = 113242,
  [SMALL_STATE(3621)] = 113270,
  [SMALL_STATE(3622)] = 113298,
  [SMALL_STATE(3623)] = 113326,
  [SMALL_STATE(3624)] = 113354,
  [SMALL_STATE(3625)] = 113382,
  [SMALL_STATE(3626)] = 113410,
  [SMALL_STATE(3627)] = 113458,
  [SMALL_STATE(3628)] = 113486,
  [SMALL_STATE(3629)] = 113514,
  [SMALL_STATE(3630)] = 113564,
  [SMALL_STATE(3631)] = 113592,
  [SMALL_STATE(3632)] = 113620,
  [SMALL_STATE(3633)] = 113668,
  [SMALL_STATE(3634)] = 113716,
  [SMALL_STATE(3635)] = 113768,
  [SMALL_STATE(3636)] = 113796,
  [SMALL_STATE(3637)] = 113844,
  [SMALL_STATE(3638)] = 113872,
  [SMALL_STATE(3639)] = 113900,
  [SMALL_STATE(3640)] = 113928,
  [SMALL_STATE(3641)] = 113956,
  [SMALL_STATE(3642)] = 113984,
  [SMALL_STATE(3643)] = 114012,
  [SMALL_STATE(3644)] = 114040,
  [SMALL_STATE(3645)] = 114092,
  [SMALL_STATE(3646)] = 114141,
  [SMALL_STATE(3647)] = 114178,
  [SMALL_STATE(3648)] = 114223,
  [SMALL_STATE(3649)] = 114272,
  [SMALL_STATE(3650)] = 114301,
  [SMALL_STATE(3651)] = 114338,
  [SMALL_STATE(3652)] = 114367,
  [SMALL_STATE(3653)] = 114396,
  [SMALL_STATE(3654)] = 114425,
  [SMALL_STATE(3655)] = 114454,
  [SMALL_STATE(3656)] = 114491,
  [SMALL_STATE(3657)] = 114540,
  [SMALL_STATE(3658)] = 114589,
  [SMALL_STATE(3659)] = 114638,
  [SMALL_STATE(3660)] = 114687,
  [SMALL_STATE(3661)] = 114736,
  [SMALL_STATE(3662)] = 114765,
  [SMALL_STATE(3663)] = 114814,
  [SMALL_STATE(3664)] = 114863,
  [SMALL_STATE(3665)] = 114912,
  [SMALL_STATE(3666)] = 114941,
  [SMALL_STATE(3667)] = 114970,
  [SMALL_STATE(3668)] = 115007,
  [SMALL_STATE(3669)] = 115056,
  [SMALL_STATE(3670)] = 115085,
  [SMALL_STATE(3671)] = 115134,
  [SMALL_STATE(3672)] = 115183,
  [SMALL_STATE(3673)] = 115211,
  [SMALL_STATE(3674)] = 115247,
  [SMALL_STATE(3675)] = 115279,
  [SMALL_STATE(3676)] = 115311,
  [SMALL_STATE(3677)] = 115343,
  [SMALL_STATE(3678)] = 115373,
  [SMALL_STATE(3679)] = 115403,
  [SMALL_STATE(3680)] = 115435,
  [SMALL_STATE(3681)] = 115471,
  [SMALL_STATE(3682)] = 115503,
  [SMALL_STATE(3683)] = 115549,
  [SMALL_STATE(3684)] = 115581,
  [SMALL_STATE(3685)] = 115613,
  [SMALL_STATE(3686)] = 115645,
  [SMALL_STATE(3687)] = 115677,
  [SMALL_STATE(3688)] = 115723,
  [SMALL_STATE(3689)] = 115753,
  [SMALL_STATE(3690)] = 115785,
  [SMALL_STATE(3691)] = 115831,
  [SMALL_STATE(3692)] = 115875,
  [SMALL_STATE(3693)] = 115907,
  [SMALL_STATE(3694)] = 115939,
  [SMALL_STATE(3695)] = 115972,
  [SMALL_STATE(3696)] = 116005,
  [SMALL_STATE(3697)] = 116038,
  [SMALL_STATE(3698)] = 116071,
  [SMALL_STATE(3699)] = 116104,
  [SMALL_STATE(3700)] = 116137,
  [SMALL_STATE(3701)] = 116170,
  [SMALL_STATE(3702)] = 116203,
  [SMALL_STATE(3703)] = 116231,
  [SMALL_STATE(3704)] = 116269,
  [SMALL_STATE(3705)] = 116299,
  [SMALL_STATE(3706)] = 116337,
  [SMALL_STATE(3707)] = 116367,
  [SMALL_STATE(3708)] = 116405,
  [SMALL_STATE(3709)] = 116435,
  [SMALL_STATE(3710)] = 116465,
  [SMALL_STATE(3711)] = 116503,
  [SMALL_STATE(3712)] = 116527,
  [SMALL_STATE(3713)] = 116551,
  [SMALL_STATE(3714)] = 116575,
  [SMALL_STATE(3715)] = 116603,
  [SMALL_STATE(3716)] = 116631,
  [SMALL_STATE(3717)] = 116659,
  [SMALL_STATE(3718)] = 116687,
  [SMALL_STATE(3719)] = 116711,
  [SMALL_STATE(3720)] = 116735,
  [SMALL_STATE(3721)] = 116775,
  [SMALL_STATE(3722)] = 116813,
  [SMALL_STATE(3723)] = 116841,
  [SMALL_STATE(3724)] = 116869,
  [SMALL_STATE(3725)] = 116897,
  [SMALL_STATE(3726)] = 116925,
  [SMALL_STATE(3727)] = 116955,
  [SMALL_STATE(3728)] = 116983,
  [SMALL_STATE(3729)] = 117011,
  [SMALL_STATE(3730)] = 117039,
  [SMALL_STATE(3731)] = 117067,
  [SMALL_STATE(3732)] = 117095,
  [SMALL_STATE(3733)] = 117123,
  [SMALL_STATE(3734)] = 117151,
  [SMALL_STATE(3735)] = 117181,
  [SMALL_STATE(3736)] = 117219,
  [SMALL_STATE(3737)] = 117249,
  [SMALL_STATE(3738)] = 117279,
  [SMALL_STATE(3739)] = 117317,
  [SMALL_STATE(3740)] = 117355,
  [SMALL_STATE(3741)] = 117393,
  [SMALL_STATE(3742)] = 117431,
  [SMALL_STATE(3743)] = 117469,
  [SMALL_STATE(3744)] = 117507,
  [SMALL_STATE(3745)] = 117537,
  [SMALL_STATE(3746)] = 117575,
  [SMALL_STATE(3747)] = 117613,
  [SMALL_STATE(3748)] = 117651,
  [SMALL_STATE(3749)] = 117681,
  [SMALL_STATE(3750)] = 117719,
  [SMALL_STATE(3751)] = 117757,
  [SMALL_STATE(3752)] = 117795,
  [SMALL_STATE(3753)] = 117833,
  [SMALL_STATE(3754)] = 117871,
  [SMALL_STATE(3755)] = 117894,
  [SMALL_STATE(3756)] = 117917,
  [SMALL_STATE(3757)] = 117940,
  [SMALL_STATE(3758)] = 117963,
  [SMALL_STATE(3759)] = 117986,
  [SMALL_STATE(3760)] = 118009,
  [SMALL_STATE(3761)] = 118032,
  [SMALL_STATE(3762)] = 118063,
  [SMALL_STATE(3763)] = 118086,
  [SMALL_STATE(3764)] = 118109,
  [SMALL_STATE(3765)] = 118132,
  [SMALL_STATE(3766)] = 118155,
  [SMALL_STATE(3767)] = 118178,
  [SMALL_STATE(3768)] = 118201,
  [SMALL_STATE(3769)] = 118224,
  [SMALL_STATE(3770)] = 118247,
  [SMALL_STATE(3771)] = 118270,
  [SMALL_STATE(3772)] = 118293,
  [SMALL_STATE(3773)] = 118316,
  [SMALL_STATE(3774)] = 118339,
  [SMALL_STATE(3775)] = 118362,
  [SMALL_STATE(3776)] = 118397,
  [SMALL_STATE(3777)] = 118430,
  [SMALL_STATE(3778)] = 118453,
  [SMALL_STATE(3779)] = 118484,
  [SMALL_STATE(3780)] = 118511,
  [SMALL_STATE(3781)] = 118546,
  [SMALL_STATE(3782)] = 118569,
  [SMALL_STATE(3783)] = 118600,
  [SMALL_STATE(3784)] = 118623,
  [SMALL_STATE(3785)] = 118646,
  [SMALL_STATE(3786)] = 118669,
  [SMALL_STATE(3787)] = 118692,
  [SMALL_STATE(3788)] = 118727,
  [SMALL_STATE(3789)] = 118754,
  [SMALL_STATE(3790)] = 118781,
  [SMALL_STATE(3791)] = 118818,
  [SMALL_STATE(3792)] = 118849,
  [SMALL_STATE(3793)] = 118880,
  [SMALL_STATE(3794)] = 118911,
  [SMALL_STATE(3795)] = 118942,
  [SMALL_STATE(3796)] = 118965,
  [SMALL_STATE(3797)] = 118996,
  [SMALL_STATE(3798)] = 119019,
  [SMALL_STATE(3799)] = 119042,
  [SMALL_STATE(3800)] = 119065,
  [SMALL_STATE(3801)] = 119088,
  [SMALL_STATE(3802)] = 119123,
  [SMALL_STATE(3803)] = 119158,
  [SMALL_STATE(3804)] = 119181,
  [SMALL_STATE(3805)] = 119204,
  [SMALL_STATE(3806)] = 119235,
  [SMALL_STATE(3807)] = 119258,
  [SMALL_STATE(3808)] = 119281,
  [SMALL_STATE(3809)] = 119316,
  [SMALL_STATE(3810)] = 119339,
  [SMALL_STATE(3811)] = 119362,
  [SMALL_STATE(3812)] = 119385,
  [SMALL_STATE(3813)] = 119408,
  [SMALL_STATE(3814)] = 119444,
  [SMALL_STATE(3815)] = 119482,
  [SMALL_STATE(3816)] = 119520,
  [SMALL_STATE(3817)] = 119558,
  [SMALL_STATE(3818)] = 119596,
  [SMALL_STATE(3819)] = 119634,
  [SMALL_STATE(3820)] = 119672,
  [SMALL_STATE(3821)] = 119710,
  [SMALL_STATE(3822)] = 119748,
  [SMALL_STATE(3823)] = 119780,
  [SMALL_STATE(3824)] = 119804,
  [SMALL_STATE(3825)] = 119834,
  [SMALL_STATE(3826)] = 119858,
  [SMALL_STATE(3827)] = 119896,
  [SMALL_STATE(3828)] = 119920,
  [SMALL_STATE(3829)] = 119946,
  [SMALL_STATE(3830)] = 119972,
  [SMALL_STATE(3831)] = 120002,
  [SMALL_STATE(3832)] = 120026,
  [SMALL_STATE(3833)] = 120050,
  [SMALL_STATE(3834)] = 120088,
  [SMALL_STATE(3835)] = 120118,
  [SMALL_STATE(3836)] = 120150,
  [SMALL_STATE(3837)] = 120176,
  [SMALL_STATE(3838)] = 120202,
  [SMALL_STATE(3839)] = 120240,
  [SMALL_STATE(3840)] = 120272,
  [SMALL_STATE(3841)] = 120304,
  [SMALL_STATE(3842)] = 120336,
  [SMALL_STATE(3843)] = 120374,
  [SMALL_STATE(3844)] = 120412,
  [SMALL_STATE(3845)] = 120450,
  [SMALL_STATE(3846)] = 120474,
  [SMALL_STATE(3847)] = 120506,
  [SMALL_STATE(3848)] = 120530,
  [SMALL_STATE(3849)] = 120562,
  [SMALL_STATE(3850)] = 120600,
  [SMALL_STATE(3851)] = 120638,
  [SMALL_STATE(3852)] = 120664,
  [SMALL_STATE(3853)] = 120694,
  [SMALL_STATE(3854)] = 120726,
  [SMALL_STATE(3855)] = 120758,
  [SMALL_STATE(3856)] = 120782,
  [SMALL_STATE(3857)] = 120814,
  [SMALL_STATE(3858)] = 120838,
  [SMALL_STATE(3859)] = 120862,
  [SMALL_STATE(3860)] = 120886,
  [SMALL_STATE(3861)] = 120912,
  [SMALL_STATE(3862)] = 120936,
  [SMALL_STATE(3863)] = 120968,
  [SMALL_STATE(3864)] = 121000,
  [SMALL_STATE(3865)] = 121032,
  [SMALL_STATE(3866)] = 121070,
  [SMALL_STATE(3867)] = 121094,
  [SMALL_STATE(3868)] = 121118,
  [SMALL_STATE(3869)] = 121150,
  [SMALL_STATE(3870)] = 121188,
  [SMALL_STATE(3871)] = 121226,
  [SMALL_STATE(3872)] = 121258,
  [SMALL_STATE(3873)] = 121290,
  [SMALL_STATE(3874)] = 121322,
  [SMALL_STATE(3875)] = 121346,
  [SMALL_STATE(3876)] = 121378,
  [SMALL_STATE(3877)] = 121416,
  [SMALL_STATE(3878)] = 121440,
  [SMALL_STATE(3879)] = 121472,
  [SMALL_STATE(3880)] = 121510,
  [SMALL_STATE(3881)] = 121542,
  [SMALL_STATE(3882)] = 121580,
  [SMALL_STATE(3883)] = 121612,
  [SMALL_STATE(3884)] = 121638,
  [SMALL_STATE(3885)] = 121670,
  [SMALL_STATE(3886)] = 121694,
  [SMALL_STATE(3887)] = 121718,
  [SMALL_STATE(3888)] = 121742,
  [SMALL_STATE(3889)] = 121766,
  [SMALL_STATE(3890)] = 121790,
  [SMALL_STATE(3891)] = 121814,
  [SMALL_STATE(3892)] = 121852,
  [SMALL_STATE(3893)] = 121890,
  [SMALL_STATE(3894)] = 121916,
  [SMALL_STATE(3895)] = 121951,
  [SMALL_STATE(3896)] = 121986,
  [SMALL_STATE(3897)] = 122007,
  [SMALL_STATE(3898)] = 122042,
  [SMALL_STATE(3899)] = 122063,
  [SMALL_STATE(3900)] = 122084,
  [SMALL_STATE(3901)] = 122105,
  [SMALL_STATE(3902)] = 122140,
  [SMALL_STATE(3903)] = 122169,
  [SMALL_STATE(3904)] = 122204,
  [SMALL_STATE(3905)] = 122239,
  [SMALL_STATE(3906)] = 122274,
  [SMALL_STATE(3907)] = 122309,
  [SMALL_STATE(3908)] = 122344,
  [SMALL_STATE(3909)] = 122379,
  [SMALL_STATE(3910)] = 122414,
  [SMALL_STATE(3911)] = 122449,
  [SMALL_STATE(3912)] = 122484,
  [SMALL_STATE(3913)] = 122519,
  [SMALL_STATE(3914)] = 122554,
  [SMALL_STATE(3915)] = 122589,
  [SMALL_STATE(3916)] = 122624,
  [SMALL_STATE(3917)] = 122659,
  [SMALL_STATE(3918)] = 122694,
  [SMALL_STATE(3919)] = 122729,
  [SMALL_STATE(3920)] = 122758,
  [SMALL_STATE(3921)] = 122793,
  [SMALL_STATE(3922)] = 122828,
  [SMALL_STATE(3923)] = 122863,
  [SMALL_STATE(3924)] = 122898,
  [SMALL_STATE(3925)] = 122933,
  [SMALL_STATE(3926)] = 122968,
  [SMALL_STATE(3927)] = 123003,
  [SMALL_STATE(3928)] = 123038,
  [SMALL_STATE(3929)] = 123073,
  [SMALL_STATE(3930)] = 123108,
  [SMALL_STATE(3931)] = 123143,
  [SMALL_STATE(3932)] = 123178,
  [SMALL_STATE(3933)] = 123213,
  [SMALL_STATE(3934)] = 123248,
  [SMALL_STATE(3935)] = 123269,
  [SMALL_STATE(3936)] = 123290,
  [SMALL_STATE(3937)] = 123325,
  [SMALL_STATE(3938)] = 123360,
  [SMALL_STATE(3939)] = 123395,
  [SMALL_STATE(3940)] = 123430,
  [SMALL_STATE(3941)] = 123451,
  [SMALL_STATE(3942)] = 123472,
  [SMALL_STATE(3943)] = 123505,
  [SMALL_STATE(3944)] = 123532,
  [SMALL_STATE(3945)] = 123567,
  [SMALL_STATE(3946)] = 123594,
  [SMALL_STATE(3947)] = 123629,
  [SMALL_STATE(3948)] = 123664,
  [SMALL_STATE(3949)] = 123693,
  [SMALL_STATE(3950)] = 123728,
  [SMALL_STATE(3951)] = 123763,
  [SMALL_STATE(3952)] = 123798,
  [SMALL_STATE(3953)] = 123833,
  [SMALL_STATE(3954)] = 123868,
  [SMALL_STATE(3955)] = 123903,
  [SMALL_STATE(3956)] = 123938,
  [SMALL_STATE(3957)] = 123973,
  [SMALL_STATE(3958)] = 124008,
  [SMALL_STATE(3959)] = 124037,
  [SMALL_STATE(3960)] = 124072,
  [SMALL_STATE(3961)] = 124107,
  [SMALL_STATE(3962)] = 124136,
  [SMALL_STATE(3963)] = 124165,
  [SMALL_STATE(3964)] = 124200,
  [SMALL_STATE(3965)] = 124235,
  [SMALL_STATE(3966)] = 124264,
  [SMALL_STATE(3967)] = 124293,
  [SMALL_STATE(3968)] = 124318,
  [SMALL_STATE(3969)] = 124343,
  [SMALL_STATE(3970)] = 124378,
  [SMALL_STATE(3971)] = 124407,
  [SMALL_STATE(3972)] = 124442,
  [SMALL_STATE(3973)] = 124474,
  [SMALL_STATE(3974)] = 124506,
  [SMALL_STATE(3975)] = 124538,
  [SMALL_STATE(3976)] = 124570,
  [SMALL_STATE(3977)] = 124602,
  [SMALL_STATE(3978)] = 124634,
  [SMALL_STATE(3979)] = 124666,
  [SMALL_STATE(3980)] = 124698,
  [SMALL_STATE(3981)] = 124730,
  [SMALL_STATE(3982)] = 124762,
  [SMALL_STATE(3983)] = 124794,
  [SMALL_STATE(3984)] = 124826,
  [SMALL_STATE(3985)] = 124858,
  [SMALL_STATE(3986)] = 124890,
  [SMALL_STATE(3987)] = 124922,
  [SMALL_STATE(3988)] = 124954,
  [SMALL_STATE(3989)] = 124986,
  [SMALL_STATE(3990)] = 125018,
  [SMALL_STATE(3991)] = 125050,
  [SMALL_STATE(3992)] = 125082,
  [SMALL_STATE(3993)] = 125114,
  [SMALL_STATE(3994)] = 125146,
  [SMALL_STATE(3995)] = 125178,
  [SMALL_STATE(3996)] = 125210,
  [SMALL_STATE(3997)] = 125242,
  [SMALL_STATE(3998)] = 125274,
  [SMALL_STATE(3999)] = 125306,
  [SMALL_STATE(4000)] = 125338,
  [SMALL_STATE(4001)] = 125370,
  [SMALL_STATE(4002)] = 125402,
  [SMALL_STATE(4003)] = 125434,
  [SMALL_STATE(4004)] = 125466,
  [SMALL_STATE(4005)] = 125498,
  [SMALL_STATE(4006)] = 125530,
  [SMALL_STATE(4007)] = 125556,
  [SMALL_STATE(4008)] = 125588,
  [SMALL_STATE(4009)] = 125620,
  [SMALL_STATE(4010)] = 125652,
  [SMALL_STATE(4011)] = 125684,
  [SMALL_STATE(4012)] = 125716,
  [SMALL_STATE(4013)] = 125748,
  [SMALL_STATE(4014)] = 125776,
  [SMALL_STATE(4015)] = 125802,
  [SMALL_STATE(4016)] = 125830,
  [SMALL_STATE(4017)] = 125862,
  [SMALL_STATE(4018)] = 125894,
  [SMALL_STATE(4019)] = 125926,
  [SMALL_STATE(4020)] = 125958,
  [SMALL_STATE(4021)] = 125990,
  [SMALL_STATE(4022)] = 126022,
  [SMALL_STATE(4023)] = 126054,
  [SMALL_STATE(4024)] = 126086,
  [SMALL_STATE(4025)] = 126118,
  [SMALL_STATE(4026)] = 126150,
  [SMALL_STATE(4027)] = 126182,
  [SMALL_STATE(4028)] = 126214,
  [SMALL_STATE(4029)] = 126246,
  [SMALL_STATE(4030)] = 126278,
  [SMALL_STATE(4031)] = 126310,
  [SMALL_STATE(4032)] = 126342,
  [SMALL_STATE(4033)] = 126374,
  [SMALL_STATE(4034)] = 126406,
  [SMALL_STATE(4035)] = 126438,
  [SMALL_STATE(4036)] = 126470,
  [SMALL_STATE(4037)] = 126502,
  [SMALL_STATE(4038)] = 126534,
  [SMALL_STATE(4039)] = 126566,
  [SMALL_STATE(4040)] = 126598,
  [SMALL_STATE(4041)] = 126630,
  [SMALL_STATE(4042)] = 126662,
  [SMALL_STATE(4043)] = 126694,
  [SMALL_STATE(4044)] = 126722,
  [SMALL_STATE(4045)] = 126752,
  [SMALL_STATE(4046)] = 126782,
  [SMALL_STATE(4047)] = 126808,
  [SMALL_STATE(4048)] = 126840,
  [SMALL_STATE(4049)] = 126868,
  [SMALL_STATE(4050)] = 126900,
  [SMALL_STATE(4051)] = 126932,
  [SMALL_STATE(4052)] = 126964,
  [SMALL_STATE(4053)] = 126996,
  [SMALL_STATE(4054)] = 127028,
  [SMALL_STATE(4055)] = 127060,
  [SMALL_STATE(4056)] = 127092,
  [SMALL_STATE(4057)] = 127124,
  [SMALL_STATE(4058)] = 127152,
  [SMALL_STATE(4059)] = 127184,
  [SMALL_STATE(4060)] = 127216,
  [SMALL_STATE(4061)] = 127248,
  [SMALL_STATE(4062)] = 127280,
  [SMALL_STATE(4063)] = 127312,
  [SMALL_STATE(4064)] = 127344,
  [SMALL_STATE(4065)] = 127372,
  [SMALL_STATE(4066)] = 127404,
  [SMALL_STATE(4067)] = 127432,
  [SMALL_STATE(4068)] = 127458,
  [SMALL_STATE(4069)] = 127490,
  [SMALL_STATE(4070)] = 127518,
  [SMALL_STATE(4071)] = 127550,
  [SMALL_STATE(4072)] = 127579,
  [SMALL_STATE(4073)] = 127608,
  [SMALL_STATE(4074)] = 127631,
  [SMALL_STATE(4075)] = 127660,
  [SMALL_STATE(4076)] = 127683,
  [SMALL_STATE(4077)] = 127712,
  [SMALL_STATE(4078)] = 127741,
  [SMALL_STATE(4079)] = 127770,
  [SMALL_STATE(4080)] = 127799,
  [SMALL_STATE(4081)] = 127828,
  [SMALL_STATE(4082)] = 127857,
  [SMALL_STATE(4083)] = 127886,
  [SMALL_STATE(4084)] = 127915,
  [SMALL_STATE(4085)] = 127944,
  [SMALL_STATE(4086)] = 127973,
  [SMALL_STATE(4087)] = 127994,
  [SMALL_STATE(4088)] = 128023,
  [SMALL_STATE(4089)] = 128050,
  [SMALL_STATE(4090)] = 128079,
  [SMALL_STATE(4091)] = 128108,
  [SMALL_STATE(4092)] = 128137,
  [SMALL_STATE(4093)] = 128166,
  [SMALL_STATE(4094)] = 128195,
  [SMALL_STATE(4095)] = 128224,
  [SMALL_STATE(4096)] = 128253,
  [SMALL_STATE(4097)] = 128282,
  [SMALL_STATE(4098)] = 128311,
  [SMALL_STATE(4099)] = 128340,
  [SMALL_STATE(4100)] = 128369,
  [SMALL_STATE(4101)] = 128398,
  [SMALL_STATE(4102)] = 128425,
  [SMALL_STATE(4103)] = 128454,
  [SMALL_STATE(4104)] = 128483,
  [SMALL_STATE(4105)] = 128504,
  [SMALL_STATE(4106)] = 128533,
  [SMALL_STATE(4107)] = 128562,
  [SMALL_STATE(4108)] = 128591,
  [SMALL_STATE(4109)] = 128620,
  [SMALL_STATE(4110)] = 128649,
  [SMALL_STATE(4111)] = 128678,
  [SMALL_STATE(4112)] = 128707,
  [SMALL_STATE(4113)] = 128728,
  [SMALL_STATE(4114)] = 128757,
  [SMALL_STATE(4115)] = 128784,
  [SMALL_STATE(4116)] = 128813,
  [SMALL_STATE(4117)] = 128842,
  [SMALL_STATE(4118)] = 128871,
  [SMALL_STATE(4119)] = 128900,
  [SMALL_STATE(4120)] = 128929,
  [SMALL_STATE(4121)] = 128950,
  [SMALL_STATE(4122)] = 128979,
  [SMALL_STATE(4123)] = 129008,
  [SMALL_STATE(4124)] = 129037,
  [SMALL_STATE(4125)] = 129066,
  [SMALL_STATE(4126)] = 129087,
  [SMALL_STATE(4127)] = 129116,
  [SMALL_STATE(4128)] = 129145,
  [SMALL_STATE(4129)] = 129168,
  [SMALL_STATE(4130)] = 129197,
  [SMALL_STATE(4131)] = 129216,
  [SMALL_STATE(4132)] = 129237,
  [SMALL_STATE(4133)] = 129258,
  [SMALL_STATE(4134)] = 129287,
  [SMALL_STATE(4135)] = 129316,
  [SMALL_STATE(4136)] = 129345,
  [SMALL_STATE(4137)] = 129374,
  [SMALL_STATE(4138)] = 129397,
  [SMALL_STATE(4139)] = 129426,
  [SMALL_STATE(4140)] = 129455,
  [SMALL_STATE(4141)] = 129484,
  [SMALL_STATE(4142)] = 129513,
  [SMALL_STATE(4143)] = 129542,
  [SMALL_STATE(4144)] = 129571,
  [SMALL_STATE(4145)] = 129600,
  [SMALL_STATE(4146)] = 129629,
  [SMALL_STATE(4147)] = 129658,
  [SMALL_STATE(4148)] = 129687,
  [SMALL_STATE(4149)] = 129708,
  [SMALL_STATE(4150)] = 129731,
  [SMALL_STATE(4151)] = 129760,
  [SMALL_STATE(4152)] = 129789,
  [SMALL_STATE(4153)] = 129814,
  [SMALL_STATE(4154)] = 129843,
  [SMALL_STATE(4155)] = 129872,
  [SMALL_STATE(4156)] = 129901,
  [SMALL_STATE(4157)] = 129930,
  [SMALL_STATE(4158)] = 129959,
  [SMALL_STATE(4159)] = 129988,
  [SMALL_STATE(4160)] = 130011,
  [SMALL_STATE(4161)] = 130040,
  [SMALL_STATE(4162)] = 130069,
  [SMALL_STATE(4163)] = 130090,
  [SMALL_STATE(4164)] = 130115,
  [SMALL_STATE(4165)] = 130144,
  [SMALL_STATE(4166)] = 130169,
  [SMALL_STATE(4167)] = 130190,
  [SMALL_STATE(4168)] = 130219,
  [SMALL_STATE(4169)] = 130248,
  [SMALL_STATE(4170)] = 130273,
  [SMALL_STATE(4171)] = 130302,
  [SMALL_STATE(4172)] = 130325,
  [SMALL_STATE(4173)] = 130354,
  [SMALL_STATE(4174)] = 130383,
  [SMALL_STATE(4175)] = 130408,
  [SMALL_STATE(4176)] = 130437,
  [SMALL_STATE(4177)] = 130464,
  [SMALL_STATE(4178)] = 130493,
  [SMALL_STATE(4179)] = 130522,
  [SMALL_STATE(4180)] = 130545,
  [SMALL_STATE(4181)] = 130574,
  [SMALL_STATE(4182)] = 130603,
  [SMALL_STATE(4183)] = 130632,
  [SMALL_STATE(4184)] = 130653,
  [SMALL_STATE(4185)] = 130674,
  [SMALL_STATE(4186)] = 130699,
  [SMALL_STATE(4187)] = 130728,
  [SMALL_STATE(4188)] = 130757,
  [SMALL_STATE(4189)] = 130780,
  [SMALL_STATE(4190)] = 130809,
  [SMALL_STATE(4191)] = 130838,
  [SMALL_STATE(4192)] = 130861,
  [SMALL_STATE(4193)] = 130884,
  [SMALL_STATE(4194)] = 130907,
  [SMALL_STATE(4195)] = 130936,
  [SMALL_STATE(4196)] = 130965,
  [SMALL_STATE(4197)] = 130994,
  [SMALL_STATE(4198)] = 131023,
  [SMALL_STATE(4199)] = 131052,
  [SMALL_STATE(4200)] = 131081,
  [SMALL_STATE(4201)] = 131106,
  [SMALL_STATE(4202)] = 131129,
  [SMALL_STATE(4203)] = 131158,
  [SMALL_STATE(4204)] = 131187,
  [SMALL_STATE(4205)] = 131216,
  [SMALL_STATE(4206)] = 131245,
  [SMALL_STATE(4207)] = 131268,
  [SMALL_STATE(4208)] = 131297,
  [SMALL_STATE(4209)] = 131326,
  [SMALL_STATE(4210)] = 131347,
  [SMALL_STATE(4211)] = 131372,
  [SMALL_STATE(4212)] = 131401,
  [SMALL_STATE(4213)] = 131426,
  [SMALL_STATE(4214)] = 131455,
  [SMALL_STATE(4215)] = 131484,
  [SMALL_STATE(4216)] = 131511,
  [SMALL_STATE(4217)] = 131540,
  [SMALL_STATE(4218)] = 131569,
  [SMALL_STATE(4219)] = 131598,
  [SMALL_STATE(4220)] = 131621,
  [SMALL_STATE(4221)] = 131650,
  [SMALL_STATE(4222)] = 131679,
  [SMALL_STATE(4223)] = 131708,
  [SMALL_STATE(4224)] = 131731,
  [SMALL_STATE(4225)] = 131750,
  [SMALL_STATE(4226)] = 131779,
  [SMALL_STATE(4227)] = 131808,
  [SMALL_STATE(4228)] = 131837,
  [SMALL_STATE(4229)] = 131860,
  [SMALL_STATE(4230)] = 131883,
  [SMALL_STATE(4231)] = 131912,
  [SMALL_STATE(4232)] = 131941,
  [SMALL_STATE(4233)] = 131964,
  [SMALL_STATE(4234)] = 131993,
  [SMALL_STATE(4235)] = 132022,
  [SMALL_STATE(4236)] = 132045,
  [SMALL_STATE(4237)] = 132074,
  [SMALL_STATE(4238)] = 132103,
  [SMALL_STATE(4239)] = 132132,
  [SMALL_STATE(4240)] = 132161,
  [SMALL_STATE(4241)] = 132184,
  [SMALL_STATE(4242)] = 132213,
  [SMALL_STATE(4243)] = 132242,
  [SMALL_STATE(4244)] = 132265,
  [SMALL_STATE(4245)] = 132294,
  [SMALL_STATE(4246)] = 132323,
  [SMALL_STATE(4247)] = 132346,
  [SMALL_STATE(4248)] = 132375,
  [SMALL_STATE(4249)] = 132394,
  [SMALL_STATE(4250)] = 132417,
  [SMALL_STATE(4251)] = 132446,
  [SMALL_STATE(4252)] = 132475,
  [SMALL_STATE(4253)] = 132504,
  [SMALL_STATE(4254)] = 132533,
  [SMALL_STATE(4255)] = 132552,
  [SMALL_STATE(4256)] = 132571,
  [SMALL_STATE(4257)] = 132600,
  [SMALL_STATE(4258)] = 132619,
  [SMALL_STATE(4259)] = 132648,
  [SMALL_STATE(4260)] = 132667,
  [SMALL_STATE(4261)] = 132686,
  [SMALL_STATE(4262)] = 132715,
  [SMALL_STATE(4263)] = 132744,
  [SMALL_STATE(4264)] = 132773,
  [SMALL_STATE(4265)] = 132792,
  [SMALL_STATE(4266)] = 132821,
  [SMALL_STATE(4267)] = 132847,
  [SMALL_STATE(4268)] = 132865,
  [SMALL_STATE(4269)] = 132883,
  [SMALL_STATE(4270)] = 132901,
  [SMALL_STATE(4271)] = 132925,
  [SMALL_STATE(4272)] = 132949,
  [SMALL_STATE(4273)] = 132967,
  [SMALL_STATE(4274)] = 132985,
  [SMALL_STATE(4275)] = 133011,
  [SMALL_STATE(4276)] = 133037,
  [SMALL_STATE(4277)] = 133063,
  [SMALL_STATE(4278)] = 133087,
  [SMALL_STATE(4279)] = 133113,
  [SMALL_STATE(4280)] = 133139,
  [SMALL_STATE(4281)] = 133165,
  [SMALL_STATE(4282)] = 133191,
  [SMALL_STATE(4283)] = 133217,
  [SMALL_STATE(4284)] = 133243,
  [SMALL_STATE(4285)] = 133269,
  [SMALL_STATE(4286)] = 133295,
  [SMALL_STATE(4287)] = 133321,
  [SMALL_STATE(4288)] = 133345,
  [SMALL_STATE(4289)] = 133371,
  [SMALL_STATE(4290)] = 133397,
  [SMALL_STATE(4291)] = 133419,
  [SMALL_STATE(4292)] = 133445,
  [SMALL_STATE(4293)] = 133471,
  [SMALL_STATE(4294)] = 133497,
  [SMALL_STATE(4295)] = 133523,
  [SMALL_STATE(4296)] = 133549,
  [SMALL_STATE(4297)] = 133567,
  [SMALL_STATE(4298)] = 133585,
  [SMALL_STATE(4299)] = 133611,
  [SMALL_STATE(4300)] = 133637,
  [SMALL_STATE(4301)] = 133659,
  [SMALL_STATE(4302)] = 133685,
  [SMALL_STATE(4303)] = 133711,
  [SMALL_STATE(4304)] = 133737,
  [SMALL_STATE(4305)] = 133763,
  [SMALL_STATE(4306)] = 133789,
  [SMALL_STATE(4307)] = 133815,
  [SMALL_STATE(4308)] = 133841,
  [SMALL_STATE(4309)] = 133867,
  [SMALL_STATE(4310)] = 133893,
  [SMALL_STATE(4311)] = 133919,
  [SMALL_STATE(4312)] = 133945,
  [SMALL_STATE(4313)] = 133971,
  [SMALL_STATE(4314)] = 133997,
  [SMALL_STATE(4315)] = 134017,
  [SMALL_STATE(4316)] = 134043,
  [SMALL_STATE(4317)] = 134069,
  [SMALL_STATE(4318)] = 134095,
  [SMALL_STATE(4319)] = 134113,
  [SMALL_STATE(4320)] = 134139,
  [SMALL_STATE(4321)] = 134165,
  [SMALL_STATE(4322)] = 134191,
  [SMALL_STATE(4323)] = 134217,
  [SMALL_STATE(4324)] = 134243,
  [SMALL_STATE(4325)] = 134269,
  [SMALL_STATE(4326)] = 134295,
  [SMALL_STATE(4327)] = 134321,
  [SMALL_STATE(4328)] = 134347,
  [SMALL_STATE(4329)] = 134373,
  [SMALL_STATE(4330)] = 134391,
  [SMALL_STATE(4331)] = 134413,
  [SMALL_STATE(4332)] = 134439,
  [SMALL_STATE(4333)] = 134465,
  [SMALL_STATE(4334)] = 134491,
  [SMALL_STATE(4335)] = 134517,
  [SMALL_STATE(4336)] = 134543,
  [SMALL_STATE(4337)] = 134569,
  [SMALL_STATE(4338)] = 134595,
  [SMALL_STATE(4339)] = 134621,
  [SMALL_STATE(4340)] = 134647,
  [SMALL_STATE(4341)] = 134673,
  [SMALL_STATE(4342)] = 134699,
  [SMALL_STATE(4343)] = 134717,
  [SMALL_STATE(4344)] = 134739,
  [SMALL_STATE(4345)] = 134765,
  [SMALL_STATE(4346)] = 134791,
  [SMALL_STATE(4347)] = 134817,
  [SMALL_STATE(4348)] = 134843,
  [SMALL_STATE(4349)] = 134869,
  [SMALL_STATE(4350)] = 134895,
  [SMALL_STATE(4351)] = 134921,
  [SMALL_STATE(4352)] = 134947,
  [SMALL_STATE(4353)] = 134973,
  [SMALL_STATE(4354)] = 134999,
  [SMALL_STATE(4355)] = 135025,
  [SMALL_STATE(4356)] = 135043,
  [SMALL_STATE(4357)] = 135061,
  [SMALL_STATE(4358)] = 135087,
  [SMALL_STATE(4359)] = 135113,
  [SMALL_STATE(4360)] = 135131,
  [SMALL_STATE(4361)] = 135157,
  [SMALL_STATE(4362)] = 135183,
  [SMALL_STATE(4363)] = 135209,
  [SMALL_STATE(4364)] = 135233,
  [SMALL_STATE(4365)] = 135255,
  [SMALL_STATE(4366)] = 135279,
  [SMALL_STATE(4367)] = 135301,
  [SMALL_STATE(4368)] = 135327,
  [SMALL_STATE(4369)] = 135353,
  [SMALL_STATE(4370)] = 135373,
  [SMALL_STATE(4371)] = 135397,
  [SMALL_STATE(4372)] = 135419,
  [SMALL_STATE(4373)] = 135445,
  [SMALL_STATE(4374)] = 135471,
  [SMALL_STATE(4375)] = 135489,
  [SMALL_STATE(4376)] = 135511,
  [SMALL_STATE(4377)] = 135529,
  [SMALL_STATE(4378)] = 135555,
  [SMALL_STATE(4379)] = 135573,
  [SMALL_STATE(4380)] = 135599,
  [SMALL_STATE(4381)] = 135625,
  [SMALL_STATE(4382)] = 135643,
  [SMALL_STATE(4383)] = 135669,
  [SMALL_STATE(4384)] = 135691,
  [SMALL_STATE(4385)] = 135717,
  [SMALL_STATE(4386)] = 135743,
  [SMALL_STATE(4387)] = 135769,
  [SMALL_STATE(4388)] = 135787,
  [SMALL_STATE(4389)] = 135813,
  [SMALL_STATE(4390)] = 135839,
  [SMALL_STATE(4391)] = 135861,
  [SMALL_STATE(4392)] = 135887,
  [SMALL_STATE(4393)] = 135913,
  [SMALL_STATE(4394)] = 135939,
  [SMALL_STATE(4395)] = 135965,
  [SMALL_STATE(4396)] = 135989,
  [SMALL_STATE(4397)] = 136007,
  [SMALL_STATE(4398)] = 136025,
  [SMALL_STATE(4399)] = 136051,
  [SMALL_STATE(4400)] = 136069,
  [SMALL_STATE(4401)] = 136095,
  [SMALL_STATE(4402)] = 136113,
  [SMALL_STATE(4403)] = 136139,
  [SMALL_STATE(4404)] = 136165,
  [SMALL_STATE(4405)] = 136191,
  [SMALL_STATE(4406)] = 136215,
  [SMALL_STATE(4407)] = 136241,
  [SMALL_STATE(4408)] = 136259,
  [SMALL_STATE(4409)] = 136283,
  [SMALL_STATE(4410)] = 136309,
  [SMALL_STATE(4411)] = 136335,
  [SMALL_STATE(4412)] = 136353,
  [SMALL_STATE(4413)] = 136379,
  [SMALL_STATE(4414)] = 136403,
  [SMALL_STATE(4415)] = 136427,
  [SMALL_STATE(4416)] = 136453,
  [SMALL_STATE(4417)] = 136479,
  [SMALL_STATE(4418)] = 136505,
  [SMALL_STATE(4419)] = 136523,
  [SMALL_STATE(4420)] = 136541,
  [SMALL_STATE(4421)] = 136559,
  [SMALL_STATE(4422)] = 136585,
  [SMALL_STATE(4423)] = 136611,
  [SMALL_STATE(4424)] = 136637,
  [SMALL_STATE(4425)] = 136663,
  [SMALL_STATE(4426)] = 136689,
  [SMALL_STATE(4427)] = 136715,
  [SMALL_STATE(4428)] = 136741,
  [SMALL_STATE(4429)] = 136759,
  [SMALL_STATE(4430)] = 136785,
  [SMALL_STATE(4431)] = 136809,
  [SMALL_STATE(4432)] = 136835,
  [SMALL_STATE(4433)] = 136853,
  [SMALL_STATE(4434)] = 136877,
  [SMALL_STATE(4435)] = 136903,
  [SMALL_STATE(4436)] = 136929,
  [SMALL_STATE(4437)] = 136955,
  [SMALL_STATE(4438)] = 136975,
  [SMALL_STATE(4439)] = 136993,
  [SMALL_STATE(4440)] = 137016,
  [SMALL_STATE(4441)] = 137039,
  [SMALL_STATE(4442)] = 137062,
  [SMALL_STATE(4443)] = 137083,
  [SMALL_STATE(4444)] = 137106,
  [SMALL_STATE(4445)] = 137129,
  [SMALL_STATE(4446)] = 137152,
  [SMALL_STATE(4447)] = 137175,
  [SMALL_STATE(4448)] = 137196,
  [SMALL_STATE(4449)] = 137219,
  [SMALL_STATE(4450)] = 137242,
  [SMALL_STATE(4451)] = 137265,
  [SMALL_STATE(4452)] = 137288,
  [SMALL_STATE(4453)] = 137311,
  [SMALL_STATE(4454)] = 137334,
  [SMALL_STATE(4455)] = 137357,
  [SMALL_STATE(4456)] = 137380,
  [SMALL_STATE(4457)] = 137397,
  [SMALL_STATE(4458)] = 137420,
  [SMALL_STATE(4459)] = 137443,
  [SMALL_STATE(4460)] = 137466,
  [SMALL_STATE(4461)] = 137489,
  [SMALL_STATE(4462)] = 137512,
  [SMALL_STATE(4463)] = 137535,
  [SMALL_STATE(4464)] = 137558,
  [SMALL_STATE(4465)] = 137581,
  [SMALL_STATE(4466)] = 137604,
  [SMALL_STATE(4467)] = 137627,
  [SMALL_STATE(4468)] = 137650,
  [SMALL_STATE(4469)] = 137673,
  [SMALL_STATE(4470)] = 137696,
  [SMALL_STATE(4471)] = 137719,
  [SMALL_STATE(4472)] = 137742,
  [SMALL_STATE(4473)] = 137765,
  [SMALL_STATE(4474)] = 137788,
  [SMALL_STATE(4475)] = 137811,
  [SMALL_STATE(4476)] = 137834,
  [SMALL_STATE(4477)] = 137857,
  [SMALL_STATE(4478)] = 137880,
  [SMALL_STATE(4479)] = 137903,
  [SMALL_STATE(4480)] = 137926,
  [SMALL_STATE(4481)] = 137949,
  [SMALL_STATE(4482)] = 137972,
  [SMALL_STATE(4483)] = 137995,
  [SMALL_STATE(4484)] = 138018,
  [SMALL_STATE(4485)] = 138041,
  [SMALL_STATE(4486)] = 138064,
  [SMALL_STATE(4487)] = 138087,
  [SMALL_STATE(4488)] = 138110,
  [SMALL_STATE(4489)] = 138133,
  [SMALL_STATE(4490)] = 138156,
  [SMALL_STATE(4491)] = 138179,
  [SMALL_STATE(4492)] = 138202,
  [SMALL_STATE(4493)] = 138225,
  [SMALL_STATE(4494)] = 138248,
  [SMALL_STATE(4495)] = 138271,
  [SMALL_STATE(4496)] = 138294,
  [SMALL_STATE(4497)] = 138313,
  [SMALL_STATE(4498)] = 138336,
  [SMALL_STATE(4499)] = 138359,
  [SMALL_STATE(4500)] = 138382,
  [SMALL_STATE(4501)] = 138405,
  [SMALL_STATE(4502)] = 138428,
  [SMALL_STATE(4503)] = 138451,
  [SMALL_STATE(4504)] = 138474,
  [SMALL_STATE(4505)] = 138497,
  [SMALL_STATE(4506)] = 138520,
  [SMALL_STATE(4507)] = 138543,
  [SMALL_STATE(4508)] = 138566,
  [SMALL_STATE(4509)] = 138589,
  [SMALL_STATE(4510)] = 138612,
  [SMALL_STATE(4511)] = 138635,
  [SMALL_STATE(4512)] = 138658,
  [SMALL_STATE(4513)] = 138681,
  [SMALL_STATE(4514)] = 138704,
  [SMALL_STATE(4515)] = 138727,
  [SMALL_STATE(4516)] = 138750,
  [SMALL_STATE(4517)] = 138773,
  [SMALL_STATE(4518)] = 138796,
  [SMALL_STATE(4519)] = 138819,
  [SMALL_STATE(4520)] = 138842,
  [SMALL_STATE(4521)] = 138865,
  [SMALL_STATE(4522)] = 138888,
  [SMALL_STATE(4523)] = 138905,
  [SMALL_STATE(4524)] = 138928,
  [SMALL_STATE(4525)] = 138951,
  [SMALL_STATE(4526)] = 138972,
  [SMALL_STATE(4527)] = 138993,
  [SMALL_STATE(4528)] = 139016,
  [SMALL_STATE(4529)] = 139039,
  [SMALL_STATE(4530)] = 139062,
  [SMALL_STATE(4531)] = 139083,
  [SMALL_STATE(4532)] = 139106,
  [SMALL_STATE(4533)] = 139129,
  [SMALL_STATE(4534)] = 139152,
  [SMALL_STATE(4535)] = 139173,
  [SMALL_STATE(4536)] = 139196,
  [SMALL_STATE(4537)] = 139219,
  [SMALL_STATE(4538)] = 139242,
  [SMALL_STATE(4539)] = 139265,
  [SMALL_STATE(4540)] = 139288,
  [SMALL_STATE(4541)] = 139311,
  [SMALL_STATE(4542)] = 139334,
  [SMALL_STATE(4543)] = 139357,
  [SMALL_STATE(4544)] = 139380,
  [SMALL_STATE(4545)] = 139401,
  [SMALL_STATE(4546)] = 139424,
  [SMALL_STATE(4547)] = 139447,
  [SMALL_STATE(4548)] = 139470,
  [SMALL_STATE(4549)] = 139493,
  [SMALL_STATE(4550)] = 139516,
  [SMALL_STATE(4551)] = 139539,
  [SMALL_STATE(4552)] = 139562,
  [SMALL_STATE(4553)] = 139585,
  [SMALL_STATE(4554)] = 139608,
  [SMALL_STATE(4555)] = 139631,
  [SMALL_STATE(4556)] = 139654,
  [SMALL_STATE(4557)] = 139675,
  [SMALL_STATE(4558)] = 139698,
  [SMALL_STATE(4559)] = 139721,
  [SMALL_STATE(4560)] = 139742,
  [SMALL_STATE(4561)] = 139765,
  [SMALL_STATE(4562)] = 139788,
  [SMALL_STATE(4563)] = 139811,
  [SMALL_STATE(4564)] = 139834,
  [SMALL_STATE(4565)] = 139857,
  [SMALL_STATE(4566)] = 139880,
  [SMALL_STATE(4567)] = 139903,
  [SMALL_STATE(4568)] = 139926,
  [SMALL_STATE(4569)] = 139949,
  [SMALL_STATE(4570)] = 139972,
  [SMALL_STATE(4571)] = 139995,
  [SMALL_STATE(4572)] = 140018,
  [SMALL_STATE(4573)] = 140041,
  [SMALL_STATE(4574)] = 140064,
  [SMALL_STATE(4575)] = 140083,
  [SMALL_STATE(4576)] = 140102,
  [SMALL_STATE(4577)] = 140125,
  [SMALL_STATE(4578)] = 140144,
  [SMALL_STATE(4579)] = 140163,
  [SMALL_STATE(4580)] = 140186,
  [SMALL_STATE(4581)] = 140209,
  [SMALL_STATE(4582)] = 140232,
  [SMALL_STATE(4583)] = 140255,
  [SMALL_STATE(4584)] = 140276,
  [SMALL_STATE(4585)] = 140297,
  [SMALL_STATE(4586)] = 140320,
  [SMALL_STATE(4587)] = 140343,
  [SMALL_STATE(4588)] = 140362,
  [SMALL_STATE(4589)] = 140381,
  [SMALL_STATE(4590)] = 140400,
  [SMALL_STATE(4591)] = 140423,
  [SMALL_STATE(4592)] = 140446,
  [SMALL_STATE(4593)] = 140465,
  [SMALL_STATE(4594)] = 140488,
  [SMALL_STATE(4595)] = 140511,
  [SMALL_STATE(4596)] = 140534,
  [SMALL_STATE(4597)] = 140557,
  [SMALL_STATE(4598)] = 140580,
  [SMALL_STATE(4599)] = 140603,
  [SMALL_STATE(4600)] = 140626,
  [SMALL_STATE(4601)] = 140649,
  [SMALL_STATE(4602)] = 140672,
  [SMALL_STATE(4603)] = 140693,
  [SMALL_STATE(4604)] = 140716,
  [SMALL_STATE(4605)] = 140739,
  [SMALL_STATE(4606)] = 140762,
  [SMALL_STATE(4607)] = 140785,
  [SMALL_STATE(4608)] = 140808,
  [SMALL_STATE(4609)] = 140831,
  [SMALL_STATE(4610)] = 140854,
  [SMALL_STATE(4611)] = 140877,
  [SMALL_STATE(4612)] = 140900,
  [SMALL_STATE(4613)] = 140921,
  [SMALL_STATE(4614)] = 140944,
  [SMALL_STATE(4615)] = 140967,
  [SMALL_STATE(4616)] = 140990,
  [SMALL_STATE(4617)] = 141013,
  [SMALL_STATE(4618)] = 141036,
  [SMALL_STATE(4619)] = 141059,
  [SMALL_STATE(4620)] = 141082,
  [SMALL_STATE(4621)] = 141105,
  [SMALL_STATE(4622)] = 141128,
  [SMALL_STATE(4623)] = 141151,
  [SMALL_STATE(4624)] = 141174,
  [SMALL_STATE(4625)] = 141197,
  [SMALL_STATE(4626)] = 141220,
  [SMALL_STATE(4627)] = 141243,
  [SMALL_STATE(4628)] = 141266,
  [SMALL_STATE(4629)] = 141289,
  [SMALL_STATE(4630)] = 141312,
  [SMALL_STATE(4631)] = 141333,
  [SMALL_STATE(4632)] = 141356,
  [SMALL_STATE(4633)] = 141379,
  [SMALL_STATE(4634)] = 141402,
  [SMALL_STATE(4635)] = 141425,
  [SMALL_STATE(4636)] = 141448,
  [SMALL_STATE(4637)] = 141469,
  [SMALL_STATE(4638)] = 141492,
  [SMALL_STATE(4639)] = 141515,
  [SMALL_STATE(4640)] = 141538,
  [SMALL_STATE(4641)] = 141561,
  [SMALL_STATE(4642)] = 141584,
  [SMALL_STATE(4643)] = 141607,
  [SMALL_STATE(4644)] = 141630,
  [SMALL_STATE(4645)] = 141653,
  [SMALL_STATE(4646)] = 141674,
  [SMALL_STATE(4647)] = 141695,
  [SMALL_STATE(4648)] = 141718,
  [SMALL_STATE(4649)] = 141739,
  [SMALL_STATE(4650)] = 141762,
  [SMALL_STATE(4651)] = 141785,
  [SMALL_STATE(4652)] = 141808,
  [SMALL_STATE(4653)] = 141831,
  [SMALL_STATE(4654)] = 141854,
  [SMALL_STATE(4655)] = 141877,
  [SMALL_STATE(4656)] = 141900,
  [SMALL_STATE(4657)] = 141923,
  [SMALL_STATE(4658)] = 141946,
  [SMALL_STATE(4659)] = 141969,
  [SMALL_STATE(4660)] = 141992,
  [SMALL_STATE(4661)] = 142015,
  [SMALL_STATE(4662)] = 142038,
  [SMALL_STATE(4663)] = 142061,
  [SMALL_STATE(4664)] = 142084,
  [SMALL_STATE(4665)] = 142107,
  [SMALL_STATE(4666)] = 142130,
  [SMALL_STATE(4667)] = 142153,
  [SMALL_STATE(4668)] = 142176,
  [SMALL_STATE(4669)] = 142199,
  [SMALL_STATE(4670)] = 142220,
  [SMALL_STATE(4671)] = 142243,
  [SMALL_STATE(4672)] = 142262,
  [SMALL_STATE(4673)] = 142285,
  [SMALL_STATE(4674)] = 142308,
  [SMALL_STATE(4675)] = 142331,
  [SMALL_STATE(4676)] = 142350,
  [SMALL_STATE(4677)] = 142373,
  [SMALL_STATE(4678)] = 142396,
  [SMALL_STATE(4679)] = 142419,
  [SMALL_STATE(4680)] = 142442,
  [SMALL_STATE(4681)] = 142465,
  [SMALL_STATE(4682)] = 142484,
  [SMALL_STATE(4683)] = 142503,
  [SMALL_STATE(4684)] = 142524,
  [SMALL_STATE(4685)] = 142547,
  [SMALL_STATE(4686)] = 142568,
  [SMALL_STATE(4687)] = 142591,
  [SMALL_STATE(4688)] = 142614,
  [SMALL_STATE(4689)] = 142637,
  [SMALL_STATE(4690)] = 142660,
  [SMALL_STATE(4691)] = 142679,
  [SMALL_STATE(4692)] = 142702,
  [SMALL_STATE(4693)] = 142723,
  [SMALL_STATE(4694)] = 142746,
  [SMALL_STATE(4695)] = 142769,
  [SMALL_STATE(4696)] = 142792,
  [SMALL_STATE(4697)] = 142811,
  [SMALL_STATE(4698)] = 142834,
  [SMALL_STATE(4699)] = 142857,
  [SMALL_STATE(4700)] = 142880,
  [SMALL_STATE(4701)] = 142899,
  [SMALL_STATE(4702)] = 142922,
  [SMALL_STATE(4703)] = 142945,
  [SMALL_STATE(4704)] = 142968,
  [SMALL_STATE(4705)] = 142991,
  [SMALL_STATE(4706)] = 143014,
  [SMALL_STATE(4707)] = 143037,
  [SMALL_STATE(4708)] = 143060,
  [SMALL_STATE(4709)] = 143083,
  [SMALL_STATE(4710)] = 143106,
  [SMALL_STATE(4711)] = 143129,
  [SMALL_STATE(4712)] = 143152,
  [SMALL_STATE(4713)] = 143175,
  [SMALL_STATE(4714)] = 143198,
  [SMALL_STATE(4715)] = 143221,
  [SMALL_STATE(4716)] = 143244,
  [SMALL_STATE(4717)] = 143267,
  [SMALL_STATE(4718)] = 143290,
  [SMALL_STATE(4719)] = 143313,
  [SMALL_STATE(4720)] = 143336,
  [SMALL_STATE(4721)] = 143359,
  [SMALL_STATE(4722)] = 143382,
  [SMALL_STATE(4723)] = 143405,
  [SMALL_STATE(4724)] = 143428,
  [SMALL_STATE(4725)] = 143451,
  [SMALL_STATE(4726)] = 143474,
  [SMALL_STATE(4727)] = 143497,
  [SMALL_STATE(4728)] = 143520,
  [SMALL_STATE(4729)] = 143543,
  [SMALL_STATE(4730)] = 143566,
  [SMALL_STATE(4731)] = 143589,
  [SMALL_STATE(4732)] = 143612,
  [SMALL_STATE(4733)] = 143631,
  [SMALL_STATE(4734)] = 143654,
  [SMALL_STATE(4735)] = 143677,
  [SMALL_STATE(4736)] = 143700,
  [SMALL_STATE(4737)] = 143719,
  [SMALL_STATE(4738)] = 143742,
  [SMALL_STATE(4739)] = 143765,
  [SMALL_STATE(4740)] = 143788,
  [SMALL_STATE(4741)] = 143811,
  [SMALL_STATE(4742)] = 143834,
  [SMALL_STATE(4743)] = 143857,
  [SMALL_STATE(4744)] = 143880,
  [SMALL_STATE(4745)] = 143903,
  [SMALL_STATE(4746)] = 143926,
  [SMALL_STATE(4747)] = 143949,
  [SMALL_STATE(4748)] = 143970,
  [SMALL_STATE(4749)] = 143993,
  [SMALL_STATE(4750)] = 144016,
  [SMALL_STATE(4751)] = 144039,
  [SMALL_STATE(4752)] = 144062,
  [SMALL_STATE(4753)] = 144085,
  [SMALL_STATE(4754)] = 144108,
  [SMALL_STATE(4755)] = 144131,
  [SMALL_STATE(4756)] = 144154,
  [SMALL_STATE(4757)] = 144177,
  [SMALL_STATE(4758)] = 144200,
  [SMALL_STATE(4759)] = 144223,
  [SMALL_STATE(4760)] = 144246,
  [SMALL_STATE(4761)] = 144265,
  [SMALL_STATE(4762)] = 144288,
  [SMALL_STATE(4763)] = 144307,
  [SMALL_STATE(4764)] = 144326,
  [SMALL_STATE(4765)] = 144349,
  [SMALL_STATE(4766)] = 144372,
  [SMALL_STATE(4767)] = 144393,
  [SMALL_STATE(4768)] = 144416,
  [SMALL_STATE(4769)] = 144439,
  [SMALL_STATE(4770)] = 144462,
  [SMALL_STATE(4771)] = 144485,
  [SMALL_STATE(4772)] = 144508,
  [SMALL_STATE(4773)] = 144531,
  [SMALL_STATE(4774)] = 144554,
  [SMALL_STATE(4775)] = 144577,
  [SMALL_STATE(4776)] = 144596,
  [SMALL_STATE(4777)] = 144619,
  [SMALL_STATE(4778)] = 144642,
  [SMALL_STATE(4779)] = 144665,
  [SMALL_STATE(4780)] = 144688,
  [SMALL_STATE(4781)] = 144711,
  [SMALL_STATE(4782)] = 144734,
  [SMALL_STATE(4783)] = 144757,
  [SMALL_STATE(4784)] = 144780,
  [SMALL_STATE(4785)] = 144803,
  [SMALL_STATE(4786)] = 144826,
  [SMALL_STATE(4787)] = 144849,
  [SMALL_STATE(4788)] = 144872,
  [SMALL_STATE(4789)] = 144891,
  [SMALL_STATE(4790)] = 144914,
  [SMALL_STATE(4791)] = 144933,
  [SMALL_STATE(4792)] = 144956,
  [SMALL_STATE(4793)] = 144973,
  [SMALL_STATE(4794)] = 144996,
  [SMALL_STATE(4795)] = 145019,
  [SMALL_STATE(4796)] = 145038,
  [SMALL_STATE(4797)] = 145061,
  [SMALL_STATE(4798)] = 145084,
  [SMALL_STATE(4799)] = 145107,
  [SMALL_STATE(4800)] = 145130,
  [SMALL_STATE(4801)] = 145153,
  [SMALL_STATE(4802)] = 145172,
  [SMALL_STATE(4803)] = 145193,
  [SMALL_STATE(4804)] = 145216,
  [SMALL_STATE(4805)] = 145233,
  [SMALL_STATE(4806)] = 145256,
  [SMALL_STATE(4807)] = 145279,
  [SMALL_STATE(4808)] = 145302,
  [SMALL_STATE(4809)] = 145325,
  [SMALL_STATE(4810)] = 145348,
  [SMALL_STATE(4811)] = 145369,
  [SMALL_STATE(4812)] = 145392,
  [SMALL_STATE(4813)] = 145415,
  [SMALL_STATE(4814)] = 145438,
  [SMALL_STATE(4815)] = 145461,
  [SMALL_STATE(4816)] = 145484,
  [SMALL_STATE(4817)] = 145507,
  [SMALL_STATE(4818)] = 145530,
  [SMALL_STATE(4819)] = 145549,
  [SMALL_STATE(4820)] = 145572,
  [SMALL_STATE(4821)] = 145595,
  [SMALL_STATE(4822)] = 145618,
  [SMALL_STATE(4823)] = 145641,
  [SMALL_STATE(4824)] = 145664,
  [SMALL_STATE(4825)] = 145683,
  [SMALL_STATE(4826)] = 145706,
  [SMALL_STATE(4827)] = 145725,
  [SMALL_STATE(4828)] = 145748,
  [SMALL_STATE(4829)] = 145771,
  [SMALL_STATE(4830)] = 145794,
  [SMALL_STATE(4831)] = 145817,
  [SMALL_STATE(4832)] = 145840,
  [SMALL_STATE(4833)] = 145859,
  [SMALL_STATE(4834)] = 145882,
  [SMALL_STATE(4835)] = 145905,
  [SMALL_STATE(4836)] = 145928,
  [SMALL_STATE(4837)] = 145951,
  [SMALL_STATE(4838)] = 145974,
  [SMALL_STATE(4839)] = 145997,
  [SMALL_STATE(4840)] = 146020,
  [SMALL_STATE(4841)] = 146043,
  [SMALL_STATE(4842)] = 146066,
  [SMALL_STATE(4843)] = 146089,
  [SMALL_STATE(4844)] = 146112,
  [SMALL_STATE(4845)] = 146135,
  [SMALL_STATE(4846)] = 146158,
  [SMALL_STATE(4847)] = 146181,
  [SMALL_STATE(4848)] = 146204,
  [SMALL_STATE(4849)] = 146227,
  [SMALL_STATE(4850)] = 146250,
  [SMALL_STATE(4851)] = 146273,
  [SMALL_STATE(4852)] = 146296,
  [SMALL_STATE(4853)] = 146319,
  [SMALL_STATE(4854)] = 146342,
  [SMALL_STATE(4855)] = 146365,
  [SMALL_STATE(4856)] = 146388,
  [SMALL_STATE(4857)] = 146411,
  [SMALL_STATE(4858)] = 146434,
  [SMALL_STATE(4859)] = 146457,
  [SMALL_STATE(4860)] = 146473,
  [SMALL_STATE(4861)] = 146493,
  [SMALL_STATE(4862)] = 146513,
  [SMALL_STATE(4863)] = 146533,
  [SMALL_STATE(4864)] = 146553,
  [SMALL_STATE(4865)] = 146573,
  [SMALL_STATE(4866)] = 146593,
  [SMALL_STATE(4867)] = 146613,
  [SMALL_STATE(4868)] = 146631,
  [SMALL_STATE(4869)] = 146651,
  [SMALL_STATE(4870)] = 146671,
  [SMALL_STATE(4871)] = 146691,
  [SMALL_STATE(4872)] = 146711,
  [SMALL_STATE(4873)] = 146729,
  [SMALL_STATE(4874)] = 146749,
  [SMALL_STATE(4875)] = 146769,
  [SMALL_STATE(4876)] = 146789,
  [SMALL_STATE(4877)] = 146809,
  [SMALL_STATE(4878)] = 146829,
  [SMALL_STATE(4879)] = 146849,
  [SMALL_STATE(4880)] = 146869,
  [SMALL_STATE(4881)] = 146889,
  [SMALL_STATE(4882)] = 146909,
  [SMALL_STATE(4883)] = 146929,
  [SMALL_STATE(4884)] = 146945,
  [SMALL_STATE(4885)] = 146965,
  [SMALL_STATE(4886)] = 146983,
  [SMALL_STATE(4887)] = 147003,
  [SMALL_STATE(4888)] = 147023,
  [SMALL_STATE(4889)] = 147039,
  [SMALL_STATE(4890)] = 147059,
  [SMALL_STATE(4891)] = 147077,
  [SMALL_STATE(4892)] = 147097,
  [SMALL_STATE(4893)] = 147115,
  [SMALL_STATE(4894)] = 147133,
  [SMALL_STATE(4895)] = 147153,
  [SMALL_STATE(4896)] = 147173,
  [SMALL_STATE(4897)] = 147193,
  [SMALL_STATE(4898)] = 147213,
  [SMALL_STATE(4899)] = 147233,
  [SMALL_STATE(4900)] = 147253,
  [SMALL_STATE(4901)] = 147273,
  [SMALL_STATE(4902)] = 147293,
  [SMALL_STATE(4903)] = 147313,
  [SMALL_STATE(4904)] = 147331,
  [SMALL_STATE(4905)] = 147351,
  [SMALL_STATE(4906)] = 147371,
  [SMALL_STATE(4907)] = 147387,
  [SMALL_STATE(4908)] = 147407,
  [SMALL_STATE(4909)] = 147425,
  [SMALL_STATE(4910)] = 147445,
  [SMALL_STATE(4911)] = 147463,
  [SMALL_STATE(4912)] = 147483,
  [SMALL_STATE(4913)] = 147503,
  [SMALL_STATE(4914)] = 147521,
  [SMALL_STATE(4915)] = 147537,
  [SMALL_STATE(4916)] = 147557,
  [SMALL_STATE(4917)] = 147577,
  [SMALL_STATE(4918)] = 147597,
  [SMALL_STATE(4919)] = 147617,
  [SMALL_STATE(4920)] = 147637,
  [SMALL_STATE(4921)] = 147657,
  [SMALL_STATE(4922)] = 147677,
  [SMALL_STATE(4923)] = 147695,
  [SMALL_STATE(4924)] = 147711,
  [SMALL_STATE(4925)] = 147727,
  [SMALL_STATE(4926)] = 147747,
  [SMALL_STATE(4927)] = 147767,
  [SMALL_STATE(4928)] = 147787,
  [SMALL_STATE(4929)] = 147805,
  [SMALL_STATE(4930)] = 147825,
  [SMALL_STATE(4931)] = 147841,
  [SMALL_STATE(4932)] = 147861,
  [SMALL_STATE(4933)] = 147881,
  [SMALL_STATE(4934)] = 147897,
  [SMALL_STATE(4935)] = 147917,
  [SMALL_STATE(4936)] = 147937,
  [SMALL_STATE(4937)] = 147957,
  [SMALL_STATE(4938)] = 147977,
  [SMALL_STATE(4939)] = 147997,
  [SMALL_STATE(4940)] = 148013,
  [SMALL_STATE(4941)] = 148033,
  [SMALL_STATE(4942)] = 148053,
  [SMALL_STATE(4943)] = 148073,
  [SMALL_STATE(4944)] = 148089,
  [SMALL_STATE(4945)] = 148109,
  [SMALL_STATE(4946)] = 148129,
  [SMALL_STATE(4947)] = 148149,
  [SMALL_STATE(4948)] = 148165,
  [SMALL_STATE(4949)] = 148185,
  [SMALL_STATE(4950)] = 148205,
  [SMALL_STATE(4951)] = 148225,
  [SMALL_STATE(4952)] = 148245,
  [SMALL_STATE(4953)] = 148265,
  [SMALL_STATE(4954)] = 148281,
  [SMALL_STATE(4955)] = 148301,
  [SMALL_STATE(4956)] = 148321,
  [SMALL_STATE(4957)] = 148341,
  [SMALL_STATE(4958)] = 148357,
  [SMALL_STATE(4959)] = 148377,
  [SMALL_STATE(4960)] = 148397,
  [SMALL_STATE(4961)] = 148417,
  [SMALL_STATE(4962)] = 148433,
  [SMALL_STATE(4963)] = 148453,
  [SMALL_STATE(4964)] = 148471,
  [SMALL_STATE(4965)] = 148491,
  [SMALL_STATE(4966)] = 148511,
  [SMALL_STATE(4967)] = 148531,
  [SMALL_STATE(4968)] = 148551,
  [SMALL_STATE(4969)] = 148569,
  [SMALL_STATE(4970)] = 148589,
  [SMALL_STATE(4971)] = 148607,
  [SMALL_STATE(4972)] = 148627,
  [SMALL_STATE(4973)] = 148647,
  [SMALL_STATE(4974)] = 148667,
  [SMALL_STATE(4975)] = 148687,
  [SMALL_STATE(4976)] = 148707,
  [SMALL_STATE(4977)] = 148727,
  [SMALL_STATE(4978)] = 148747,
  [SMALL_STATE(4979)] = 148767,
  [SMALL_STATE(4980)] = 148787,
  [SMALL_STATE(4981)] = 148805,
  [SMALL_STATE(4982)] = 148825,
  [SMALL_STATE(4983)] = 148845,
  [SMALL_STATE(4984)] = 148865,
  [SMALL_STATE(4985)] = 148885,
  [SMALL_STATE(4986)] = 148905,
  [SMALL_STATE(4987)] = 148925,
  [SMALL_STATE(4988)] = 148945,
  [SMALL_STATE(4989)] = 148965,
  [SMALL_STATE(4990)] = 148985,
  [SMALL_STATE(4991)] = 149005,
  [SMALL_STATE(4992)] = 149025,
  [SMALL_STATE(4993)] = 149045,
  [SMALL_STATE(4994)] = 149063,
  [SMALL_STATE(4995)] = 149083,
  [SMALL_STATE(4996)] = 149103,
  [SMALL_STATE(4997)] = 149123,
  [SMALL_STATE(4998)] = 149143,
  [SMALL_STATE(4999)] = 149163,
  [SMALL_STATE(5000)] = 149183,
  [SMALL_STATE(5001)] = 149203,
  [SMALL_STATE(5002)] = 149223,
  [SMALL_STATE(5003)] = 149243,
  [SMALL_STATE(5004)] = 149261,
  [SMALL_STATE(5005)] = 149281,
  [SMALL_STATE(5006)] = 149299,
  [SMALL_STATE(5007)] = 149319,
  [SMALL_STATE(5008)] = 149335,
  [SMALL_STATE(5009)] = 149355,
  [SMALL_STATE(5010)] = 149375,
  [SMALL_STATE(5011)] = 149395,
  [SMALL_STATE(5012)] = 149415,
  [SMALL_STATE(5013)] = 149435,
  [SMALL_STATE(5014)] = 149455,
  [SMALL_STATE(5015)] = 149475,
  [SMALL_STATE(5016)] = 149491,
  [SMALL_STATE(5017)] = 149511,
  [SMALL_STATE(5018)] = 149531,
  [SMALL_STATE(5019)] = 149547,
  [SMALL_STATE(5020)] = 149567,
  [SMALL_STATE(5021)] = 149583,
  [SMALL_STATE(5022)] = 149603,
  [SMALL_STATE(5023)] = 149623,
  [SMALL_STATE(5024)] = 149643,
  [SMALL_STATE(5025)] = 149661,
  [SMALL_STATE(5026)] = 149681,
  [SMALL_STATE(5027)] = 149697,
  [SMALL_STATE(5028)] = 149717,
  [SMALL_STATE(5029)] = 149733,
  [SMALL_STATE(5030)] = 149753,
  [SMALL_STATE(5031)] = 149773,
  [SMALL_STATE(5032)] = 149789,
  [SMALL_STATE(5033)] = 149809,
  [SMALL_STATE(5034)] = 149829,
  [SMALL_STATE(5035)] = 149849,
  [SMALL_STATE(5036)] = 149869,
  [SMALL_STATE(5037)] = 149889,
  [SMALL_STATE(5038)] = 149909,
  [SMALL_STATE(5039)] = 149929,
  [SMALL_STATE(5040)] = 149949,
  [SMALL_STATE(5041)] = 149965,
  [SMALL_STATE(5042)] = 149985,
  [SMALL_STATE(5043)] = 150005,
  [SMALL_STATE(5044)] = 150025,
  [SMALL_STATE(5045)] = 150041,
  [SMALL_STATE(5046)] = 150059,
  [SMALL_STATE(5047)] = 150079,
  [SMALL_STATE(5048)] = 150097,
  [SMALL_STATE(5049)] = 150117,
  [SMALL_STATE(5050)] = 150137,
  [SMALL_STATE(5051)] = 150157,
  [SMALL_STATE(5052)] = 150177,
  [SMALL_STATE(5053)] = 150197,
  [SMALL_STATE(5054)] = 150217,
  [SMALL_STATE(5055)] = 150237,
  [SMALL_STATE(5056)] = 150257,
  [SMALL_STATE(5057)] = 150277,
  [SMALL_STATE(5058)] = 150297,
  [SMALL_STATE(5059)] = 150317,
  [SMALL_STATE(5060)] = 150337,
  [SMALL_STATE(5061)] = 150357,
  [SMALL_STATE(5062)] = 150377,
  [SMALL_STATE(5063)] = 150397,
  [SMALL_STATE(5064)] = 150417,
  [SMALL_STATE(5065)] = 150437,
  [SMALL_STATE(5066)] = 150453,
  [SMALL_STATE(5067)] = 150473,
  [SMALL_STATE(5068)] = 150493,
  [SMALL_STATE(5069)] = 150513,
  [SMALL_STATE(5070)] = 150533,
  [SMALL_STATE(5071)] = 150553,
  [SMALL_STATE(5072)] = 150573,
  [SMALL_STATE(5073)] = 150593,
  [SMALL_STATE(5074)] = 150613,
  [SMALL_STATE(5075)] = 150633,
  [SMALL_STATE(5076)] = 150653,
  [SMALL_STATE(5077)] = 150673,
  [SMALL_STATE(5078)] = 150693,
  [SMALL_STATE(5079)] = 150709,
  [SMALL_STATE(5080)] = 150729,
  [SMALL_STATE(5081)] = 150749,
  [SMALL_STATE(5082)] = 150769,
  [SMALL_STATE(5083)] = 150787,
  [SMALL_STATE(5084)] = 150807,
  [SMALL_STATE(5085)] = 150827,
  [SMALL_STATE(5086)] = 150847,
  [SMALL_STATE(5087)] = 150867,
  [SMALL_STATE(5088)] = 150887,
  [SMALL_STATE(5089)] = 150907,
  [SMALL_STATE(5090)] = 150927,
  [SMALL_STATE(5091)] = 150947,
  [SMALL_STATE(5092)] = 150967,
  [SMALL_STATE(5093)] = 150987,
  [SMALL_STATE(5094)] = 151003,
  [SMALL_STATE(5095)] = 151023,
  [SMALL_STATE(5096)] = 151043,
  [SMALL_STATE(5097)] = 151059,
  [SMALL_STATE(5098)] = 151079,
  [SMALL_STATE(5099)] = 151099,
  [SMALL_STATE(5100)] = 151115,
  [SMALL_STATE(5101)] = 151131,
  [SMALL_STATE(5102)] = 151147,
  [SMALL_STATE(5103)] = 151163,
  [SMALL_STATE(5104)] = 151179,
  [SMALL_STATE(5105)] = 151199,
  [SMALL_STATE(5106)] = 151219,
  [SMALL_STATE(5107)] = 151239,
  [SMALL_STATE(5108)] = 151259,
  [SMALL_STATE(5109)] = 151279,
  [SMALL_STATE(5110)] = 151299,
  [SMALL_STATE(5111)] = 151319,
  [SMALL_STATE(5112)] = 151339,
  [SMALL_STATE(5113)] = 151359,
  [SMALL_STATE(5114)] = 151379,
  [SMALL_STATE(5115)] = 151399,
  [SMALL_STATE(5116)] = 151419,
  [SMALL_STATE(5117)] = 151439,
  [SMALL_STATE(5118)] = 151459,
  [SMALL_STATE(5119)] = 151479,
  [SMALL_STATE(5120)] = 151499,
  [SMALL_STATE(5121)] = 151519,
  [SMALL_STATE(5122)] = 151539,
  [SMALL_STATE(5123)] = 151555,
  [SMALL_STATE(5124)] = 151575,
  [SMALL_STATE(5125)] = 151595,
  [SMALL_STATE(5126)] = 151615,
  [SMALL_STATE(5127)] = 151635,
  [SMALL_STATE(5128)] = 151655,
  [SMALL_STATE(5129)] = 151675,
  [SMALL_STATE(5130)] = 151695,
  [SMALL_STATE(5131)] = 151715,
  [SMALL_STATE(5132)] = 151735,
  [SMALL_STATE(5133)] = 151755,
  [SMALL_STATE(5134)] = 151775,
  [SMALL_STATE(5135)] = 151795,
  [SMALL_STATE(5136)] = 151815,
  [SMALL_STATE(5137)] = 151835,
  [SMALL_STATE(5138)] = 151855,
  [SMALL_STATE(5139)] = 151875,
  [SMALL_STATE(5140)] = 151895,
  [SMALL_STATE(5141)] = 151915,
  [SMALL_STATE(5142)] = 151935,
  [SMALL_STATE(5143)] = 151951,
  [SMALL_STATE(5144)] = 151967,
  [SMALL_STATE(5145)] = 151987,
  [SMALL_STATE(5146)] = 152005,
  [SMALL_STATE(5147)] = 152025,
  [SMALL_STATE(5148)] = 152045,
  [SMALL_STATE(5149)] = 152065,
  [SMALL_STATE(5150)] = 152085,
  [SMALL_STATE(5151)] = 152105,
  [SMALL_STATE(5152)] = 152125,
  [SMALL_STATE(5153)] = 152145,
  [SMALL_STATE(5154)] = 152165,
  [SMALL_STATE(5155)] = 152185,
  [SMALL_STATE(5156)] = 152201,
  [SMALL_STATE(5157)] = 152221,
  [SMALL_STATE(5158)] = 152241,
  [SMALL_STATE(5159)] = 152261,
  [SMALL_STATE(5160)] = 152281,
  [SMALL_STATE(5161)] = 152297,
  [SMALL_STATE(5162)] = 152317,
  [SMALL_STATE(5163)] = 152337,
  [SMALL_STATE(5164)] = 152357,
  [SMALL_STATE(5165)] = 152377,
  [SMALL_STATE(5166)] = 152393,
  [SMALL_STATE(5167)] = 152411,
  [SMALL_STATE(5168)] = 152431,
  [SMALL_STATE(5169)] = 152447,
  [SMALL_STATE(5170)] = 152467,
  [SMALL_STATE(5171)] = 152487,
  [SMALL_STATE(5172)] = 152507,
  [SMALL_STATE(5173)] = 152527,
  [SMALL_STATE(5174)] = 152547,
  [SMALL_STATE(5175)] = 152563,
  [SMALL_STATE(5176)] = 152579,
  [SMALL_STATE(5177)] = 152595,
  [SMALL_STATE(5178)] = 152611,
  [SMALL_STATE(5179)] = 152631,
  [SMALL_STATE(5180)] = 152651,
  [SMALL_STATE(5181)] = 152667,
  [SMALL_STATE(5182)] = 152683,
  [SMALL_STATE(5183)] = 152699,
  [SMALL_STATE(5184)] = 152715,
  [SMALL_STATE(5185)] = 152735,
  [SMALL_STATE(5186)] = 152755,
  [SMALL_STATE(5187)] = 152773,
  [SMALL_STATE(5188)] = 152793,
  [SMALL_STATE(5189)] = 152813,
  [SMALL_STATE(5190)] = 152833,
  [SMALL_STATE(5191)] = 152853,
  [SMALL_STATE(5192)] = 152873,
  [SMALL_STATE(5193)] = 152893,
  [SMALL_STATE(5194)] = 152913,
  [SMALL_STATE(5195)] = 152933,
  [SMALL_STATE(5196)] = 152953,
  [SMALL_STATE(5197)] = 152973,
  [SMALL_STATE(5198)] = 152989,
  [SMALL_STATE(5199)] = 153009,
  [SMALL_STATE(5200)] = 153029,
  [SMALL_STATE(5201)] = 153049,
  [SMALL_STATE(5202)] = 153069,
  [SMALL_STATE(5203)] = 153089,
  [SMALL_STATE(5204)] = 153109,
  [SMALL_STATE(5205)] = 153127,
  [SMALL_STATE(5206)] = 153147,
  [SMALL_STATE(5207)] = 153167,
  [SMALL_STATE(5208)] = 153187,
  [SMALL_STATE(5209)] = 153207,
  [SMALL_STATE(5210)] = 153227,
  [SMALL_STATE(5211)] = 153247,
  [SMALL_STATE(5212)] = 153267,
  [SMALL_STATE(5213)] = 153287,
  [SMALL_STATE(5214)] = 153307,
  [SMALL_STATE(5215)] = 153327,
  [SMALL_STATE(5216)] = 153347,
  [SMALL_STATE(5217)] = 153367,
  [SMALL_STATE(5218)] = 153387,
  [SMALL_STATE(5219)] = 153407,
  [SMALL_STATE(5220)] = 153427,
  [SMALL_STATE(5221)] = 153447,
  [SMALL_STATE(5222)] = 153467,
  [SMALL_STATE(5223)] = 153487,
  [SMALL_STATE(5224)] = 153507,
  [SMALL_STATE(5225)] = 153525,
  [SMALL_STATE(5226)] = 153545,
  [SMALL_STATE(5227)] = 153565,
  [SMALL_STATE(5228)] = 153581,
  [SMALL_STATE(5229)] = 153601,
  [SMALL_STATE(5230)] = 153621,
  [SMALL_STATE(5231)] = 153641,
  [SMALL_STATE(5232)] = 153661,
  [SMALL_STATE(5233)] = 153681,
  [SMALL_STATE(5234)] = 153701,
  [SMALL_STATE(5235)] = 153721,
  [SMALL_STATE(5236)] = 153741,
  [SMALL_STATE(5237)] = 153759,
  [SMALL_STATE(5238)] = 153779,
  [SMALL_STATE(5239)] = 153799,
  [SMALL_STATE(5240)] = 153819,
  [SMALL_STATE(5241)] = 153839,
  [SMALL_STATE(5242)] = 153859,
  [SMALL_STATE(5243)] = 153879,
  [SMALL_STATE(5244)] = 153899,
  [SMALL_STATE(5245)] = 153919,
  [SMALL_STATE(5246)] = 153939,
  [SMALL_STATE(5247)] = 153959,
  [SMALL_STATE(5248)] = 153977,
  [SMALL_STATE(5249)] = 153993,
  [SMALL_STATE(5250)] = 154009,
  [SMALL_STATE(5251)] = 154025,
  [SMALL_STATE(5252)] = 154041,
  [SMALL_STATE(5253)] = 154057,
  [SMALL_STATE(5254)] = 154073,
  [SMALL_STATE(5255)] = 154093,
  [SMALL_STATE(5256)] = 154109,
  [SMALL_STATE(5257)] = 154127,
  [SMALL_STATE(5258)] = 154147,
  [SMALL_STATE(5259)] = 154167,
  [SMALL_STATE(5260)] = 154187,
  [SMALL_STATE(5261)] = 154203,
  [SMALL_STATE(5262)] = 154223,
  [SMALL_STATE(5263)] = 154243,
  [SMALL_STATE(5264)] = 154263,
  [SMALL_STATE(5265)] = 154283,
  [SMALL_STATE(5266)] = 154303,
  [SMALL_STATE(5267)] = 154321,
  [SMALL_STATE(5268)] = 154341,
  [SMALL_STATE(5269)] = 154361,
  [SMALL_STATE(5270)] = 154381,
  [SMALL_STATE(5271)] = 154401,
  [SMALL_STATE(5272)] = 154417,
  [SMALL_STATE(5273)] = 154433,
  [SMALL_STATE(5274)] = 154453,
  [SMALL_STATE(5275)] = 154473,
  [SMALL_STATE(5276)] = 154493,
  [SMALL_STATE(5277)] = 154513,
  [SMALL_STATE(5278)] = 154533,
  [SMALL_STATE(5279)] = 154553,
  [SMALL_STATE(5280)] = 154573,
  [SMALL_STATE(5281)] = 154593,
  [SMALL_STATE(5282)] = 154613,
  [SMALL_STATE(5283)] = 154633,
  [SMALL_STATE(5284)] = 154653,
  [SMALL_STATE(5285)] = 154669,
  [SMALL_STATE(5286)] = 154689,
  [SMALL_STATE(5287)] = 154709,
  [SMALL_STATE(5288)] = 154729,
  [SMALL_STATE(5289)] = 154745,
  [SMALL_STATE(5290)] = 154765,
  [SMALL_STATE(5291)] = 154785,
  [SMALL_STATE(5292)] = 154805,
  [SMALL_STATE(5293)] = 154825,
  [SMALL_STATE(5294)] = 154845,
  [SMALL_STATE(5295)] = 154865,
  [SMALL_STATE(5296)] = 154885,
  [SMALL_STATE(5297)] = 154905,
  [SMALL_STATE(5298)] = 154925,
  [SMALL_STATE(5299)] = 154945,
  [SMALL_STATE(5300)] = 154963,
  [SMALL_STATE(5301)] = 154983,
  [SMALL_STATE(5302)] = 155001,
  [SMALL_STATE(5303)] = 155019,
  [SMALL_STATE(5304)] = 155039,
  [SMALL_STATE(5305)] = 155059,
  [SMALL_STATE(5306)] = 155079,
  [SMALL_STATE(5307)] = 155099,
  [SMALL_STATE(5308)] = 155119,
  [SMALL_STATE(5309)] = 155137,
  [SMALL_STATE(5310)] = 155155,
  [SMALL_STATE(5311)] = 155171,
  [SMALL_STATE(5312)] = 155187,
  [SMALL_STATE(5313)] = 155207,
  [SMALL_STATE(5314)] = 155223,
  [SMALL_STATE(5315)] = 155243,
  [SMALL_STATE(5316)] = 155263,
  [SMALL_STATE(5317)] = 155283,
  [SMALL_STATE(5318)] = 155301,
  [SMALL_STATE(5319)] = 155321,
  [SMALL_STATE(5320)] = 155341,
  [SMALL_STATE(5321)] = 155361,
  [SMALL_STATE(5322)] = 155381,
  [SMALL_STATE(5323)] = 155401,
  [SMALL_STATE(5324)] = 155417,
  [SMALL_STATE(5325)] = 155437,
  [SMALL_STATE(5326)] = 155457,
  [SMALL_STATE(5327)] = 155477,
  [SMALL_STATE(5328)] = 155497,
  [SMALL_STATE(5329)] = 155515,
  [SMALL_STATE(5330)] = 155531,
  [SMALL_STATE(5331)] = 155551,
  [SMALL_STATE(5332)] = 155571,
  [SMALL_STATE(5333)] = 155591,
  [SMALL_STATE(5334)] = 155611,
  [SMALL_STATE(5335)] = 155631,
  [SMALL_STATE(5336)] = 155651,
  [SMALL_STATE(5337)] = 155671,
  [SMALL_STATE(5338)] = 155691,
  [SMALL_STATE(5339)] = 155709,
  [SMALL_STATE(5340)] = 155729,
  [SMALL_STATE(5341)] = 155747,
  [SMALL_STATE(5342)] = 155767,
  [SMALL_STATE(5343)] = 155783,
  [SMALL_STATE(5344)] = 155803,
  [SMALL_STATE(5345)] = 155823,
  [SMALL_STATE(5346)] = 155843,
  [SMALL_STATE(5347)] = 155863,
  [SMALL_STATE(5348)] = 155883,
  [SMALL_STATE(5349)] = 155903,
  [SMALL_STATE(5350)] = 155923,
  [SMALL_STATE(5351)] = 155941,
  [SMALL_STATE(5352)] = 155961,
  [SMALL_STATE(5353)] = 155981,
  [SMALL_STATE(5354)] = 156001,
  [SMALL_STATE(5355)] = 156021,
  [SMALL_STATE(5356)] = 156041,
  [SMALL_STATE(5357)] = 156061,
  [SMALL_STATE(5358)] = 156079,
  [SMALL_STATE(5359)] = 156099,
  [SMALL_STATE(5360)] = 156119,
  [SMALL_STATE(5361)] = 156139,
  [SMALL_STATE(5362)] = 156159,
  [SMALL_STATE(5363)] = 156179,
  [SMALL_STATE(5364)] = 156199,
  [SMALL_STATE(5365)] = 156219,
  [SMALL_STATE(5366)] = 156239,
  [SMALL_STATE(5367)] = 156259,
  [SMALL_STATE(5368)] = 156279,
  [SMALL_STATE(5369)] = 156299,
  [SMALL_STATE(5370)] = 156319,
  [SMALL_STATE(5371)] = 156339,
  [SMALL_STATE(5372)] = 156357,
  [SMALL_STATE(5373)] = 156373,
  [SMALL_STATE(5374)] = 156393,
  [SMALL_STATE(5375)] = 156413,
  [SMALL_STATE(5376)] = 156433,
  [SMALL_STATE(5377)] = 156453,
  [SMALL_STATE(5378)] = 156471,
  [SMALL_STATE(5379)] = 156491,
  [SMALL_STATE(5380)] = 156511,
  [SMALL_STATE(5381)] = 156527,
  [SMALL_STATE(5382)] = 156547,
  [SMALL_STATE(5383)] = 156567,
  [SMALL_STATE(5384)] = 156587,
  [SMALL_STATE(5385)] = 156607,
  [SMALL_STATE(5386)] = 156627,
  [SMALL_STATE(5387)] = 156647,
  [SMALL_STATE(5388)] = 156667,
  [SMALL_STATE(5389)] = 156687,
  [SMALL_STATE(5390)] = 156707,
  [SMALL_STATE(5391)] = 156727,
  [SMALL_STATE(5392)] = 156747,
  [SMALL_STATE(5393)] = 156767,
  [SMALL_STATE(5394)] = 156787,
  [SMALL_STATE(5395)] = 156803,
  [SMALL_STATE(5396)] = 156823,
  [SMALL_STATE(5397)] = 156843,
  [SMALL_STATE(5398)] = 156863,
  [SMALL_STATE(5399)] = 156881,
  [SMALL_STATE(5400)] = 156901,
  [SMALL_STATE(5401)] = 156921,
  [SMALL_STATE(5402)] = 156941,
  [SMALL_STATE(5403)] = 156961,
  [SMALL_STATE(5404)] = 156981,
  [SMALL_STATE(5405)] = 157001,
  [SMALL_STATE(5406)] = 157021,
  [SMALL_STATE(5407)] = 157041,
  [SMALL_STATE(5408)] = 157061,
  [SMALL_STATE(5409)] = 157081,
  [SMALL_STATE(5410)] = 157101,
  [SMALL_STATE(5411)] = 157121,
  [SMALL_STATE(5412)] = 157141,
  [SMALL_STATE(5413)] = 157161,
  [SMALL_STATE(5414)] = 157181,
  [SMALL_STATE(5415)] = 157201,
  [SMALL_STATE(5416)] = 157221,
  [SMALL_STATE(5417)] = 157241,
  [SMALL_STATE(5418)] = 157261,
  [SMALL_STATE(5419)] = 157281,
  [SMALL_STATE(5420)] = 157301,
  [SMALL_STATE(5421)] = 157321,
  [SMALL_STATE(5422)] = 157341,
  [SMALL_STATE(5423)] = 157361,
  [SMALL_STATE(5424)] = 157381,
  [SMALL_STATE(5425)] = 157399,
  [SMALL_STATE(5426)] = 157419,
  [SMALL_STATE(5427)] = 157439,
  [SMALL_STATE(5428)] = 157459,
  [SMALL_STATE(5429)] = 157479,
  [SMALL_STATE(5430)] = 157499,
  [SMALL_STATE(5431)] = 157519,
  [SMALL_STATE(5432)] = 157539,
  [SMALL_STATE(5433)] = 157559,
  [SMALL_STATE(5434)] = 157579,
  [SMALL_STATE(5435)] = 157595,
  [SMALL_STATE(5436)] = 157611,
  [SMALL_STATE(5437)] = 157629,
  [SMALL_STATE(5438)] = 157649,
  [SMALL_STATE(5439)] = 157669,
  [SMALL_STATE(5440)] = 157689,
  [SMALL_STATE(5441)] = 157709,
  [SMALL_STATE(5442)] = 157725,
  [SMALL_STATE(5443)] = 157745,
  [SMALL_STATE(5444)] = 157762,
  [SMALL_STATE(5445)] = 157779,
  [SMALL_STATE(5446)] = 157796,
  [SMALL_STATE(5447)] = 157813,
  [SMALL_STATE(5448)] = 157830,
  [SMALL_STATE(5449)] = 157847,
  [SMALL_STATE(5450)] = 157864,
  [SMALL_STATE(5451)] = 157881,
  [SMALL_STATE(5452)] = 157898,
  [SMALL_STATE(5453)] = 157915,
  [SMALL_STATE(5454)] = 157932,
  [SMALL_STATE(5455)] = 157949,
  [SMALL_STATE(5456)] = 157966,
  [SMALL_STATE(5457)] = 157983,
  [SMALL_STATE(5458)] = 158000,
  [SMALL_STATE(5459)] = 158017,
  [SMALL_STATE(5460)] = 158034,
  [SMALL_STATE(5461)] = 158051,
  [SMALL_STATE(5462)] = 158068,
  [SMALL_STATE(5463)] = 158085,
  [SMALL_STATE(5464)] = 158102,
  [SMALL_STATE(5465)] = 158119,
  [SMALL_STATE(5466)] = 158136,
  [SMALL_STATE(5467)] = 158153,
  [SMALL_STATE(5468)] = 158170,
  [SMALL_STATE(5469)] = 158187,
  [SMALL_STATE(5470)] = 158204,
  [SMALL_STATE(5471)] = 158221,
  [SMALL_STATE(5472)] = 158236,
  [SMALL_STATE(5473)] = 158253,
  [SMALL_STATE(5474)] = 158270,
  [SMALL_STATE(5475)] = 158287,
  [SMALL_STATE(5476)] = 158302,
  [SMALL_STATE(5477)] = 158319,
  [SMALL_STATE(5478)] = 158336,
  [SMALL_STATE(5479)] = 158353,
  [SMALL_STATE(5480)] = 158370,
  [SMALL_STATE(5481)] = 158387,
  [SMALL_STATE(5482)] = 158404,
  [SMALL_STATE(5483)] = 158421,
  [SMALL_STATE(5484)] = 158438,
  [SMALL_STATE(5485)] = 158455,
  [SMALL_STATE(5486)] = 158472,
  [SMALL_STATE(5487)] = 158487,
  [SMALL_STATE(5488)] = 158504,
  [SMALL_STATE(5489)] = 158521,
  [SMALL_STATE(5490)] = 158538,
  [SMALL_STATE(5491)] = 158555,
  [SMALL_STATE(5492)] = 158572,
  [SMALL_STATE(5493)] = 158587,
  [SMALL_STATE(5494)] = 158604,
  [SMALL_STATE(5495)] = 158621,
  [SMALL_STATE(5496)] = 158638,
  [SMALL_STATE(5497)] = 158655,
  [SMALL_STATE(5498)] = 158672,
  [SMALL_STATE(5499)] = 158689,
  [SMALL_STATE(5500)] = 158706,
  [SMALL_STATE(5501)] = 158723,
  [SMALL_STATE(5502)] = 158740,
  [SMALL_STATE(5503)] = 158757,
  [SMALL_STATE(5504)] = 158774,
  [SMALL_STATE(5505)] = 158789,
  [SMALL_STATE(5506)] = 158806,
  [SMALL_STATE(5507)] = 158821,
  [SMALL_STATE(5508)] = 158838,
  [SMALL_STATE(5509)] = 158855,
  [SMALL_STATE(5510)] = 158872,
  [SMALL_STATE(5511)] = 158889,
  [SMALL_STATE(5512)] = 158906,
  [SMALL_STATE(5513)] = 158923,
  [SMALL_STATE(5514)] = 158940,
  [SMALL_STATE(5515)] = 158957,
  [SMALL_STATE(5516)] = 158974,
  [SMALL_STATE(5517)] = 158989,
  [SMALL_STATE(5518)] = 159004,
  [SMALL_STATE(5519)] = 159021,
  [SMALL_STATE(5520)] = 159038,
  [SMALL_STATE(5521)] = 159055,
  [SMALL_STATE(5522)] = 159072,
  [SMALL_STATE(5523)] = 159089,
  [SMALL_STATE(5524)] = 159104,
  [SMALL_STATE(5525)] = 159121,
  [SMALL_STATE(5526)] = 159138,
  [SMALL_STATE(5527)] = 159153,
  [SMALL_STATE(5528)] = 159170,
  [SMALL_STATE(5529)] = 159187,
  [SMALL_STATE(5530)] = 159204,
  [SMALL_STATE(5531)] = 159221,
  [SMALL_STATE(5532)] = 159238,
  [SMALL_STATE(5533)] = 159255,
  [SMALL_STATE(5534)] = 159272,
  [SMALL_STATE(5535)] = 159289,
  [SMALL_STATE(5536)] = 159306,
  [SMALL_STATE(5537)] = 159323,
  [SMALL_STATE(5538)] = 159340,
  [SMALL_STATE(5539)] = 159357,
  [SMALL_STATE(5540)] = 159372,
  [SMALL_STATE(5541)] = 159389,
  [SMALL_STATE(5542)] = 159406,
  [SMALL_STATE(5543)] = 159423,
  [SMALL_STATE(5544)] = 159440,
  [SMALL_STATE(5545)] = 159457,
  [SMALL_STATE(5546)] = 159472,
  [SMALL_STATE(5547)] = 159489,
  [SMALL_STATE(5548)] = 159506,
  [SMALL_STATE(5549)] = 159521,
  [SMALL_STATE(5550)] = 159538,
  [SMALL_STATE(5551)] = 159555,
  [SMALL_STATE(5552)] = 159570,
  [SMALL_STATE(5553)] = 159587,
  [SMALL_STATE(5554)] = 159602,
  [SMALL_STATE(5555)] = 159619,
  [SMALL_STATE(5556)] = 159636,
  [SMALL_STATE(5557)] = 159653,
  [SMALL_STATE(5558)] = 159670,
  [SMALL_STATE(5559)] = 159687,
  [SMALL_STATE(5560)] = 159704,
  [SMALL_STATE(5561)] = 159721,
  [SMALL_STATE(5562)] = 159738,
  [SMALL_STATE(5563)] = 159755,
  [SMALL_STATE(5564)] = 159772,
  [SMALL_STATE(5565)] = 159789,
  [SMALL_STATE(5566)] = 159804,
  [SMALL_STATE(5567)] = 159821,
  [SMALL_STATE(5568)] = 159836,
  [SMALL_STATE(5569)] = 159853,
  [SMALL_STATE(5570)] = 159870,
  [SMALL_STATE(5571)] = 159887,
  [SMALL_STATE(5572)] = 159904,
  [SMALL_STATE(5573)] = 159921,
  [SMALL_STATE(5574)] = 159938,
  [SMALL_STATE(5575)] = 159955,
  [SMALL_STATE(5576)] = 159972,
  [SMALL_STATE(5577)] = 159989,
  [SMALL_STATE(5578)] = 160006,
  [SMALL_STATE(5579)] = 160023,
  [SMALL_STATE(5580)] = 160040,
  [SMALL_STATE(5581)] = 160057,
  [SMALL_STATE(5582)] = 160074,
  [SMALL_STATE(5583)] = 160091,
  [SMALL_STATE(5584)] = 160108,
  [SMALL_STATE(5585)] = 160125,
  [SMALL_STATE(5586)] = 160142,
  [SMALL_STATE(5587)] = 160159,
  [SMALL_STATE(5588)] = 160176,
  [SMALL_STATE(5589)] = 160191,
  [SMALL_STATE(5590)] = 160208,
  [SMALL_STATE(5591)] = 160225,
  [SMALL_STATE(5592)] = 160242,
  [SMALL_STATE(5593)] = 160259,
  [SMALL_STATE(5594)] = 160276,
  [SMALL_STATE(5595)] = 160293,
  [SMALL_STATE(5596)] = 160310,
  [SMALL_STATE(5597)] = 160327,
  [SMALL_STATE(5598)] = 160344,
  [SMALL_STATE(5599)] = 160361,
  [SMALL_STATE(5600)] = 160378,
  [SMALL_STATE(5601)] = 160395,
  [SMALL_STATE(5602)] = 160412,
  [SMALL_STATE(5603)] = 160429,
  [SMALL_STATE(5604)] = 160446,
  [SMALL_STATE(5605)] = 160463,
  [SMALL_STATE(5606)] = 160480,
  [SMALL_STATE(5607)] = 160497,
  [SMALL_STATE(5608)] = 160514,
  [SMALL_STATE(5609)] = 160531,
  [SMALL_STATE(5610)] = 160548,
  [SMALL_STATE(5611)] = 160565,
  [SMALL_STATE(5612)] = 160582,
  [SMALL_STATE(5613)] = 160599,
  [SMALL_STATE(5614)] = 160616,
  [SMALL_STATE(5615)] = 160633,
  [SMALL_STATE(5616)] = 160650,
  [SMALL_STATE(5617)] = 160667,
  [SMALL_STATE(5618)] = 160684,
  [SMALL_STATE(5619)] = 160701,
  [SMALL_STATE(5620)] = 160718,
  [SMALL_STATE(5621)] = 160733,
  [SMALL_STATE(5622)] = 160750,
  [SMALL_STATE(5623)] = 160767,
  [SMALL_STATE(5624)] = 160784,
  [SMALL_STATE(5625)] = 160801,
  [SMALL_STATE(5626)] = 160818,
  [SMALL_STATE(5627)] = 160835,
  [SMALL_STATE(5628)] = 160852,
  [SMALL_STATE(5629)] = 160869,
  [SMALL_STATE(5630)] = 160886,
  [SMALL_STATE(5631)] = 160903,
  [SMALL_STATE(5632)] = 160920,
  [SMALL_STATE(5633)] = 160937,
  [SMALL_STATE(5634)] = 160954,
  [SMALL_STATE(5635)] = 160971,
  [SMALL_STATE(5636)] = 160988,
  [SMALL_STATE(5637)] = 161005,
  [SMALL_STATE(5638)] = 161022,
  [SMALL_STATE(5639)] = 161039,
  [SMALL_STATE(5640)] = 161056,
  [SMALL_STATE(5641)] = 161073,
  [SMALL_STATE(5642)] = 161090,
  [SMALL_STATE(5643)] = 161107,
  [SMALL_STATE(5644)] = 161124,
  [SMALL_STATE(5645)] = 161141,
  [SMALL_STATE(5646)] = 161158,
  [SMALL_STATE(5647)] = 161175,
  [SMALL_STATE(5648)] = 161192,
  [SMALL_STATE(5649)] = 161209,
  [SMALL_STATE(5650)] = 161226,
  [SMALL_STATE(5651)] = 161241,
  [SMALL_STATE(5652)] = 161258,
  [SMALL_STATE(5653)] = 161275,
  [SMALL_STATE(5654)] = 161292,
  [SMALL_STATE(5655)] = 161309,
  [SMALL_STATE(5656)] = 161326,
  [SMALL_STATE(5657)] = 161343,
  [SMALL_STATE(5658)] = 161360,
  [SMALL_STATE(5659)] = 161375,
  [SMALL_STATE(5660)] = 161392,
  [SMALL_STATE(5661)] = 161409,
  [SMALL_STATE(5662)] = 161426,
  [SMALL_STATE(5663)] = 161443,
  [SMALL_STATE(5664)] = 161460,
  [SMALL_STATE(5665)] = 161477,
  [SMALL_STATE(5666)] = 161494,
  [SMALL_STATE(5667)] = 161511,
  [SMALL_STATE(5668)] = 161528,
  [SMALL_STATE(5669)] = 161545,
  [SMALL_STATE(5670)] = 161560,
  [SMALL_STATE(5671)] = 161577,
  [SMALL_STATE(5672)] = 161594,
  [SMALL_STATE(5673)] = 161611,
  [SMALL_STATE(5674)] = 161628,
  [SMALL_STATE(5675)] = 161645,
  [SMALL_STATE(5676)] = 161662,
  [SMALL_STATE(5677)] = 161677,
  [SMALL_STATE(5678)] = 161694,
  [SMALL_STATE(5679)] = 161711,
  [SMALL_STATE(5680)] = 161728,
  [SMALL_STATE(5681)] = 161745,
  [SMALL_STATE(5682)] = 161760,
  [SMALL_STATE(5683)] = 161777,
  [SMALL_STATE(5684)] = 161794,
  [SMALL_STATE(5685)] = 161811,
  [SMALL_STATE(5686)] = 161828,
  [SMALL_STATE(5687)] = 161845,
  [SMALL_STATE(5688)] = 161862,
  [SMALL_STATE(5689)] = 161877,
  [SMALL_STATE(5690)] = 161894,
  [SMALL_STATE(5691)] = 161911,
  [SMALL_STATE(5692)] = 161928,
  [SMALL_STATE(5693)] = 161945,
  [SMALL_STATE(5694)] = 161962,
  [SMALL_STATE(5695)] = 161979,
  [SMALL_STATE(5696)] = 161996,
  [SMALL_STATE(5697)] = 162013,
  [SMALL_STATE(5698)] = 162030,
  [SMALL_STATE(5699)] = 162047,
  [SMALL_STATE(5700)] = 162064,
  [SMALL_STATE(5701)] = 162081,
  [SMALL_STATE(5702)] = 162098,
  [SMALL_STATE(5703)] = 162115,
  [SMALL_STATE(5704)] = 162132,
  [SMALL_STATE(5705)] = 162147,
  [SMALL_STATE(5706)] = 162164,
  [SMALL_STATE(5707)] = 162181,
  [SMALL_STATE(5708)] = 162196,
  [SMALL_STATE(5709)] = 162213,
  [SMALL_STATE(5710)] = 162230,
  [SMALL_STATE(5711)] = 162247,
  [SMALL_STATE(5712)] = 162264,
  [SMALL_STATE(5713)] = 162281,
  [SMALL_STATE(5714)] = 162298,
  [SMALL_STATE(5715)] = 162315,
  [SMALL_STATE(5716)] = 162332,
  [SMALL_STATE(5717)] = 162349,
  [SMALL_STATE(5718)] = 162366,
  [SMALL_STATE(5719)] = 162383,
  [SMALL_STATE(5720)] = 162400,
  [SMALL_STATE(5721)] = 162417,
  [SMALL_STATE(5722)] = 162434,
  [SMALL_STATE(5723)] = 162449,
  [SMALL_STATE(5724)] = 162464,
  [SMALL_STATE(5725)] = 162481,
  [SMALL_STATE(5726)] = 162498,
  [SMALL_STATE(5727)] = 162515,
  [SMALL_STATE(5728)] = 162532,
  [SMALL_STATE(5729)] = 162549,
  [SMALL_STATE(5730)] = 162566,
  [SMALL_STATE(5731)] = 162583,
  [SMALL_STATE(5732)] = 162600,
  [SMALL_STATE(5733)] = 162617,
  [SMALL_STATE(5734)] = 162634,
  [SMALL_STATE(5735)] = 162651,
  [SMALL_STATE(5736)] = 162668,
  [SMALL_STATE(5737)] = 162685,
  [SMALL_STATE(5738)] = 162702,
  [SMALL_STATE(5739)] = 162719,
  [SMALL_STATE(5740)] = 162736,
  [SMALL_STATE(5741)] = 162753,
  [SMALL_STATE(5742)] = 162768,
  [SMALL_STATE(5743)] = 162785,
  [SMALL_STATE(5744)] = 162802,
  [SMALL_STATE(5745)] = 162819,
  [SMALL_STATE(5746)] = 162836,
  [SMALL_STATE(5747)] = 162853,
  [SMALL_STATE(5748)] = 162870,
  [SMALL_STATE(5749)] = 162887,
  [SMALL_STATE(5750)] = 162904,
  [SMALL_STATE(5751)] = 162921,
  [SMALL_STATE(5752)] = 162938,
  [SMALL_STATE(5753)] = 162955,
  [SMALL_STATE(5754)] = 162970,
  [SMALL_STATE(5755)] = 162987,
  [SMALL_STATE(5756)] = 163002,
  [SMALL_STATE(5757)] = 163019,
  [SMALL_STATE(5758)] = 163036,
  [SMALL_STATE(5759)] = 163051,
  [SMALL_STATE(5760)] = 163068,
  [SMALL_STATE(5761)] = 163085,
  [SMALL_STATE(5762)] = 163102,
  [SMALL_STATE(5763)] = 163119,
  [SMALL_STATE(5764)] = 163136,
  [SMALL_STATE(5765)] = 163153,
  [SMALL_STATE(5766)] = 163170,
  [SMALL_STATE(5767)] = 163187,
  [SMALL_STATE(5768)] = 163204,
  [SMALL_STATE(5769)] = 163221,
  [SMALL_STATE(5770)] = 163238,
  [SMALL_STATE(5771)] = 163255,
  [SMALL_STATE(5772)] = 163272,
  [SMALL_STATE(5773)] = 163289,
  [SMALL_STATE(5774)] = 163306,
  [SMALL_STATE(5775)] = 163323,
  [SMALL_STATE(5776)] = 163340,
  [SMALL_STATE(5777)] = 163357,
  [SMALL_STATE(5778)] = 163374,
  [SMALL_STATE(5779)] = 163391,
  [SMALL_STATE(5780)] = 163408,
  [SMALL_STATE(5781)] = 163425,
  [SMALL_STATE(5782)] = 163442,
  [SMALL_STATE(5783)] = 163459,
  [SMALL_STATE(5784)] = 163476,
  [SMALL_STATE(5785)] = 163493,
  [SMALL_STATE(5786)] = 163510,
  [SMALL_STATE(5787)] = 163527,
  [SMALL_STATE(5788)] = 163544,
  [SMALL_STATE(5789)] = 163561,
  [SMALL_STATE(5790)] = 163578,
  [SMALL_STATE(5791)] = 163595,
  [SMALL_STATE(5792)] = 163612,
  [SMALL_STATE(5793)] = 163629,
  [SMALL_STATE(5794)] = 163646,
  [SMALL_STATE(5795)] = 163663,
  [SMALL_STATE(5796)] = 163680,
  [SMALL_STATE(5797)] = 163697,
  [SMALL_STATE(5798)] = 163714,
  [SMALL_STATE(5799)] = 163731,
  [SMALL_STATE(5800)] = 163748,
  [SMALL_STATE(5801)] = 163765,
  [SMALL_STATE(5802)] = 163782,
  [SMALL_STATE(5803)] = 163799,
  [SMALL_STATE(5804)] = 163816,
  [SMALL_STATE(5805)] = 163831,
  [SMALL_STATE(5806)] = 163848,
  [SMALL_STATE(5807)] = 163865,
  [SMALL_STATE(5808)] = 163882,
  [SMALL_STATE(5809)] = 163899,
  [SMALL_STATE(5810)] = 163916,
  [SMALL_STATE(5811)] = 163933,
  [SMALL_STATE(5812)] = 163950,
  [SMALL_STATE(5813)] = 163967,
  [SMALL_STATE(5814)] = 163984,
  [SMALL_STATE(5815)] = 164001,
  [SMALL_STATE(5816)] = 164018,
  [SMALL_STATE(5817)] = 164035,
  [SMALL_STATE(5818)] = 164052,
  [SMALL_STATE(5819)] = 164067,
  [SMALL_STATE(5820)] = 164082,
  [SMALL_STATE(5821)] = 164099,
  [SMALL_STATE(5822)] = 164116,
  [SMALL_STATE(5823)] = 164133,
  [SMALL_STATE(5824)] = 164150,
  [SMALL_STATE(5825)] = 164167,
  [SMALL_STATE(5826)] = 164182,
  [SMALL_STATE(5827)] = 164199,
  [SMALL_STATE(5828)] = 164216,
  [SMALL_STATE(5829)] = 164231,
  [SMALL_STATE(5830)] = 164248,
  [SMALL_STATE(5831)] = 164265,
  [SMALL_STATE(5832)] = 164282,
  [SMALL_STATE(5833)] = 164297,
  [SMALL_STATE(5834)] = 164314,
  [SMALL_STATE(5835)] = 164329,
  [SMALL_STATE(5836)] = 164346,
  [SMALL_STATE(5837)] = 164363,
  [SMALL_STATE(5838)] = 164380,
  [SMALL_STATE(5839)] = 164397,
  [SMALL_STATE(5840)] = 164412,
  [SMALL_STATE(5841)] = 164429,
  [SMALL_STATE(5842)] = 164446,
  [SMALL_STATE(5843)] = 164463,
  [SMALL_STATE(5844)] = 164480,
  [SMALL_STATE(5845)] = 164497,
  [SMALL_STATE(5846)] = 164514,
  [SMALL_STATE(5847)] = 164529,
  [SMALL_STATE(5848)] = 164544,
  [SMALL_STATE(5849)] = 164561,
  [SMALL_STATE(5850)] = 164578,
  [SMALL_STATE(5851)] = 164595,
  [SMALL_STATE(5852)] = 164612,
  [SMALL_STATE(5853)] = 164627,
  [SMALL_STATE(5854)] = 164644,
  [SMALL_STATE(5855)] = 164661,
  [SMALL_STATE(5856)] = 164678,
  [SMALL_STATE(5857)] = 164695,
  [SMALL_STATE(5858)] = 164712,
  [SMALL_STATE(5859)] = 164729,
  [SMALL_STATE(5860)] = 164744,
  [SMALL_STATE(5861)] = 164761,
  [SMALL_STATE(5862)] = 164778,
  [SMALL_STATE(5863)] = 164795,
  [SMALL_STATE(5864)] = 164812,
  [SMALL_STATE(5865)] = 164829,
  [SMALL_STATE(5866)] = 164844,
  [SMALL_STATE(5867)] = 164861,
  [SMALL_STATE(5868)] = 164878,
  [SMALL_STATE(5869)] = 164893,
  [SMALL_STATE(5870)] = 164910,
  [SMALL_STATE(5871)] = 164927,
  [SMALL_STATE(5872)] = 164944,
  [SMALL_STATE(5873)] = 164961,
  [SMALL_STATE(5874)] = 164978,
  [SMALL_STATE(5875)] = 164995,
  [SMALL_STATE(5876)] = 165012,
  [SMALL_STATE(5877)] = 165029,
  [SMALL_STATE(5878)] = 165046,
  [SMALL_STATE(5879)] = 165063,
  [SMALL_STATE(5880)] = 165080,
  [SMALL_STATE(5881)] = 165095,
  [SMALL_STATE(5882)] = 165112,
  [SMALL_STATE(5883)] = 165129,
  [SMALL_STATE(5884)] = 165144,
  [SMALL_STATE(5885)] = 165161,
  [SMALL_STATE(5886)] = 165178,
  [SMALL_STATE(5887)] = 165195,
  [SMALL_STATE(5888)] = 165212,
  [SMALL_STATE(5889)] = 165229,
  [SMALL_STATE(5890)] = 165246,
  [SMALL_STATE(5891)] = 165261,
  [SMALL_STATE(5892)] = 165278,
  [SMALL_STATE(5893)] = 165295,
  [SMALL_STATE(5894)] = 165312,
  [SMALL_STATE(5895)] = 165329,
  [SMALL_STATE(5896)] = 165346,
  [SMALL_STATE(5897)] = 165363,
  [SMALL_STATE(5898)] = 165380,
  [SMALL_STATE(5899)] = 165397,
  [SMALL_STATE(5900)] = 165414,
  [SMALL_STATE(5901)] = 165431,
  [SMALL_STATE(5902)] = 165446,
  [SMALL_STATE(5903)] = 165463,
  [SMALL_STATE(5904)] = 165478,
  [SMALL_STATE(5905)] = 165493,
  [SMALL_STATE(5906)] = 165510,
  [SMALL_STATE(5907)] = 165527,
  [SMALL_STATE(5908)] = 165544,
  [SMALL_STATE(5909)] = 165561,
  [SMALL_STATE(5910)] = 165578,
  [SMALL_STATE(5911)] = 165595,
  [SMALL_STATE(5912)] = 165612,
  [SMALL_STATE(5913)] = 165629,
  [SMALL_STATE(5914)] = 165646,
  [SMALL_STATE(5915)] = 165663,
  [SMALL_STATE(5916)] = 165680,
  [SMALL_STATE(5917)] = 165697,
  [SMALL_STATE(5918)] = 165714,
  [SMALL_STATE(5919)] = 165731,
  [SMALL_STATE(5920)] = 165748,
  [SMALL_STATE(5921)] = 165765,
  [SMALL_STATE(5922)] = 165782,
  [SMALL_STATE(5923)] = 165799,
  [SMALL_STATE(5924)] = 165816,
  [SMALL_STATE(5925)] = 165833,
  [SMALL_STATE(5926)] = 165850,
  [SMALL_STATE(5927)] = 165867,
  [SMALL_STATE(5928)] = 165884,
  [SMALL_STATE(5929)] = 165901,
  [SMALL_STATE(5930)] = 165918,
  [SMALL_STATE(5931)] = 165935,
  [SMALL_STATE(5932)] = 165952,
  [SMALL_STATE(5933)] = 165969,
  [SMALL_STATE(5934)] = 165986,
  [SMALL_STATE(5935)] = 166003,
  [SMALL_STATE(5936)] = 166020,
  [SMALL_STATE(5937)] = 166037,
  [SMALL_STATE(5938)] = 166054,
  [SMALL_STATE(5939)] = 166069,
  [SMALL_STATE(5940)] = 166086,
  [SMALL_STATE(5941)] = 166103,
  [SMALL_STATE(5942)] = 166120,
  [SMALL_STATE(5943)] = 166137,
  [SMALL_STATE(5944)] = 166154,
  [SMALL_STATE(5945)] = 166171,
  [SMALL_STATE(5946)] = 166188,
  [SMALL_STATE(5947)] = 166205,
  [SMALL_STATE(5948)] = 166222,
  [SMALL_STATE(5949)] = 166239,
  [SMALL_STATE(5950)] = 166256,
  [SMALL_STATE(5951)] = 166273,
  [SMALL_STATE(5952)] = 166290,
  [SMALL_STATE(5953)] = 166307,
  [SMALL_STATE(5954)] = 166324,
  [SMALL_STATE(5955)] = 166341,
  [SMALL_STATE(5956)] = 166358,
  [SMALL_STATE(5957)] = 166375,
  [SMALL_STATE(5958)] = 166392,
  [SMALL_STATE(5959)] = 166406,
  [SMALL_STATE(5960)] = 166420,
  [SMALL_STATE(5961)] = 166434,
  [SMALL_STATE(5962)] = 166448,
  [SMALL_STATE(5963)] = 166462,
  [SMALL_STATE(5964)] = 166476,
  [SMALL_STATE(5965)] = 166490,
  [SMALL_STATE(5966)] = 166504,
  [SMALL_STATE(5967)] = 166518,
  [SMALL_STATE(5968)] = 166532,
  [SMALL_STATE(5969)] = 166546,
  [SMALL_STATE(5970)] = 166560,
  [SMALL_STATE(5971)] = 166574,
  [SMALL_STATE(5972)] = 166588,
  [SMALL_STATE(5973)] = 166602,
  [SMALL_STATE(5974)] = 166616,
  [SMALL_STATE(5975)] = 166630,
  [SMALL_STATE(5976)] = 166644,
  [SMALL_STATE(5977)] = 166658,
  [SMALL_STATE(5978)] = 166672,
  [SMALL_STATE(5979)] = 166686,
  [SMALL_STATE(5980)] = 166700,
  [SMALL_STATE(5981)] = 166714,
  [SMALL_STATE(5982)] = 166728,
  [SMALL_STATE(5983)] = 166742,
  [SMALL_STATE(5984)] = 166756,
  [SMALL_STATE(5985)] = 166770,
  [SMALL_STATE(5986)] = 166784,
  [SMALL_STATE(5987)] = 166798,
  [SMALL_STATE(5988)] = 166812,
  [SMALL_STATE(5989)] = 166826,
  [SMALL_STATE(5990)] = 166840,
  [SMALL_STATE(5991)] = 166854,
  [SMALL_STATE(5992)] = 166868,
  [SMALL_STATE(5993)] = 166882,
  [SMALL_STATE(5994)] = 166896,
  [SMALL_STATE(5995)] = 166910,
  [SMALL_STATE(5996)] = 166924,
  [SMALL_STATE(5997)] = 166938,
  [SMALL_STATE(5998)] = 166952,
  [SMALL_STATE(5999)] = 166966,
  [SMALL_STATE(6000)] = 166980,
  [SMALL_STATE(6001)] = 166994,
  [SMALL_STATE(6002)] = 167008,
  [SMALL_STATE(6003)] = 167022,
  [SMALL_STATE(6004)] = 167036,
  [SMALL_STATE(6005)] = 167050,
  [SMALL_STATE(6006)] = 167064,
  [SMALL_STATE(6007)] = 167078,
  [SMALL_STATE(6008)] = 167092,
  [SMALL_STATE(6009)] = 167106,
  [SMALL_STATE(6010)] = 167120,
  [SMALL_STATE(6011)] = 167134,
  [SMALL_STATE(6012)] = 167148,
  [SMALL_STATE(6013)] = 167162,
  [SMALL_STATE(6014)] = 167176,
  [SMALL_STATE(6015)] = 167190,
  [SMALL_STATE(6016)] = 167204,
  [SMALL_STATE(6017)] = 167218,
  [SMALL_STATE(6018)] = 167232,
  [SMALL_STATE(6019)] = 167246,
  [SMALL_STATE(6020)] = 167260,
  [SMALL_STATE(6021)] = 167274,
  [SMALL_STATE(6022)] = 167288,
  [SMALL_STATE(6023)] = 167302,
  [SMALL_STATE(6024)] = 167316,
  [SMALL_STATE(6025)] = 167330,
  [SMALL_STATE(6026)] = 167344,
  [SMALL_STATE(6027)] = 167358,
  [SMALL_STATE(6028)] = 167372,
  [SMALL_STATE(6029)] = 167386,
  [SMALL_STATE(6030)] = 167400,
  [SMALL_STATE(6031)] = 167414,
  [SMALL_STATE(6032)] = 167428,
  [SMALL_STATE(6033)] = 167442,
  [SMALL_STATE(6034)] = 167456,
  [SMALL_STATE(6035)] = 167470,
  [SMALL_STATE(6036)] = 167484,
  [SMALL_STATE(6037)] = 167498,
  [SMALL_STATE(6038)] = 167512,
  [SMALL_STATE(6039)] = 167526,
  [SMALL_STATE(6040)] = 167540,
  [SMALL_STATE(6041)] = 167554,
  [SMALL_STATE(6042)] = 167568,
  [SMALL_STATE(6043)] = 167582,
  [SMALL_STATE(6044)] = 167596,
  [SMALL_STATE(6045)] = 167610,
  [SMALL_STATE(6046)] = 167624,
  [SMALL_STATE(6047)] = 167638,
  [SMALL_STATE(6048)] = 167652,
  [SMALL_STATE(6049)] = 167666,
  [SMALL_STATE(6050)] = 167680,
  [SMALL_STATE(6051)] = 167694,
  [SMALL_STATE(6052)] = 167708,
  [SMALL_STATE(6053)] = 167722,
  [SMALL_STATE(6054)] = 167736,
  [SMALL_STATE(6055)] = 167750,
  [SMALL_STATE(6056)] = 167764,
  [SMALL_STATE(6057)] = 167778,
  [SMALL_STATE(6058)] = 167792,
  [SMALL_STATE(6059)] = 167806,
  [SMALL_STATE(6060)] = 167820,
  [SMALL_STATE(6061)] = 167834,
  [SMALL_STATE(6062)] = 167848,
  [SMALL_STATE(6063)] = 167862,
  [SMALL_STATE(6064)] = 167876,
  [SMALL_STATE(6065)] = 167890,
  [SMALL_STATE(6066)] = 167904,
  [SMALL_STATE(6067)] = 167918,
  [SMALL_STATE(6068)] = 167932,
  [SMALL_STATE(6069)] = 167946,
  [SMALL_STATE(6070)] = 167960,
  [SMALL_STATE(6071)] = 167974,
  [SMALL_STATE(6072)] = 167988,
  [SMALL_STATE(6073)] = 168002,
  [SMALL_STATE(6074)] = 168016,
  [SMALL_STATE(6075)] = 168030,
  [SMALL_STATE(6076)] = 168044,
  [SMALL_STATE(6077)] = 168058,
  [SMALL_STATE(6078)] = 168072,
  [SMALL_STATE(6079)] = 168086,
  [SMALL_STATE(6080)] = 168100,
  [SMALL_STATE(6081)] = 168114,
  [SMALL_STATE(6082)] = 168128,
  [SMALL_STATE(6083)] = 168142,
  [SMALL_STATE(6084)] = 168156,
  [SMALL_STATE(6085)] = 168170,
  [SMALL_STATE(6086)] = 168184,
  [SMALL_STATE(6087)] = 168198,
  [SMALL_STATE(6088)] = 168212,
  [SMALL_STATE(6089)] = 168226,
  [SMALL_STATE(6090)] = 168240,
  [SMALL_STATE(6091)] = 168254,
  [SMALL_STATE(6092)] = 168268,
  [SMALL_STATE(6093)] = 168282,
  [SMALL_STATE(6094)] = 168296,
  [SMALL_STATE(6095)] = 168310,
  [SMALL_STATE(6096)] = 168324,
  [SMALL_STATE(6097)] = 168338,
  [SMALL_STATE(6098)] = 168352,
  [SMALL_STATE(6099)] = 168366,
  [SMALL_STATE(6100)] = 168380,
  [SMALL_STATE(6101)] = 168394,
  [SMALL_STATE(6102)] = 168408,
  [SMALL_STATE(6103)] = 168422,
  [SMALL_STATE(6104)] = 168436,
  [SMALL_STATE(6105)] = 168450,
  [SMALL_STATE(6106)] = 168464,
  [SMALL_STATE(6107)] = 168478,
  [SMALL_STATE(6108)] = 168492,
  [SMALL_STATE(6109)] = 168506,
  [SMALL_STATE(6110)] = 168520,
  [SMALL_STATE(6111)] = 168534,
  [SMALL_STATE(6112)] = 168548,
  [SMALL_STATE(6113)] = 168562,
  [SMALL_STATE(6114)] = 168576,
  [SMALL_STATE(6115)] = 168590,
  [SMALL_STATE(6116)] = 168604,
  [SMALL_STATE(6117)] = 168618,
  [SMALL_STATE(6118)] = 168632,
  [SMALL_STATE(6119)] = 168646,
  [SMALL_STATE(6120)] = 168660,
  [SMALL_STATE(6121)] = 168674,
  [SMALL_STATE(6122)] = 168688,
  [SMALL_STATE(6123)] = 168702,
  [SMALL_STATE(6124)] = 168716,
  [SMALL_STATE(6125)] = 168730,
  [SMALL_STATE(6126)] = 168744,
  [SMALL_STATE(6127)] = 168758,
  [SMALL_STATE(6128)] = 168772,
  [SMALL_STATE(6129)] = 168786,
  [SMALL_STATE(6130)] = 168800,
  [SMALL_STATE(6131)] = 168814,
  [SMALL_STATE(6132)] = 168828,
  [SMALL_STATE(6133)] = 168842,
  [SMALL_STATE(6134)] = 168856,
  [SMALL_STATE(6135)] = 168870,
  [SMALL_STATE(6136)] = 168884,
  [SMALL_STATE(6137)] = 168898,
  [SMALL_STATE(6138)] = 168912,
  [SMALL_STATE(6139)] = 168926,
  [SMALL_STATE(6140)] = 168940,
  [SMALL_STATE(6141)] = 168954,
  [SMALL_STATE(6142)] = 168968,
  [SMALL_STATE(6143)] = 168982,
  [SMALL_STATE(6144)] = 168996,
  [SMALL_STATE(6145)] = 169010,
  [SMALL_STATE(6146)] = 169024,
  [SMALL_STATE(6147)] = 169038,
  [SMALL_STATE(6148)] = 169052,
  [SMALL_STATE(6149)] = 169066,
  [SMALL_STATE(6150)] = 169080,
  [SMALL_STATE(6151)] = 169094,
  [SMALL_STATE(6152)] = 169108,
  [SMALL_STATE(6153)] = 169122,
  [SMALL_STATE(6154)] = 169136,
  [SMALL_STATE(6155)] = 169150,
  [SMALL_STATE(6156)] = 169164,
  [SMALL_STATE(6157)] = 169178,
  [SMALL_STATE(6158)] = 169192,
  [SMALL_STATE(6159)] = 169206,
  [SMALL_STATE(6160)] = 169220,
  [SMALL_STATE(6161)] = 169234,
  [SMALL_STATE(6162)] = 169248,
  [SMALL_STATE(6163)] = 169262,
  [SMALL_STATE(6164)] = 169276,
  [SMALL_STATE(6165)] = 169290,
  [SMALL_STATE(6166)] = 169304,
  [SMALL_STATE(6167)] = 169318,
  [SMALL_STATE(6168)] = 169332,
  [SMALL_STATE(6169)] = 169346,
  [SMALL_STATE(6170)] = 169360,
  [SMALL_STATE(6171)] = 169374,
  [SMALL_STATE(6172)] = 169388,
  [SMALL_STATE(6173)] = 169402,
  [SMALL_STATE(6174)] = 169416,
  [SMALL_STATE(6175)] = 169430,
  [SMALL_STATE(6176)] = 169444,
  [SMALL_STATE(6177)] = 169458,
  [SMALL_STATE(6178)] = 169472,
  [SMALL_STATE(6179)] = 169486,
  [SMALL_STATE(6180)] = 169500,
  [SMALL_STATE(6181)] = 169514,
  [SMALL_STATE(6182)] = 169528,
  [SMALL_STATE(6183)] = 169542,
  [SMALL_STATE(6184)] = 169556,
  [SMALL_STATE(6185)] = 169570,
  [SMALL_STATE(6186)] = 169584,
  [SMALL_STATE(6187)] = 169598,
  [SMALL_STATE(6188)] = 169612,
  [SMALL_STATE(6189)] = 169626,
  [SMALL_STATE(6190)] = 169640,
  [SMALL_STATE(6191)] = 169654,
  [SMALL_STATE(6192)] = 169668,
  [SMALL_STATE(6193)] = 169682,
  [SMALL_STATE(6194)] = 169696,
  [SMALL_STATE(6195)] = 169710,
  [SMALL_STATE(6196)] = 169724,
  [SMALL_STATE(6197)] = 169738,
  [SMALL_STATE(6198)] = 169752,
  [SMALL_STATE(6199)] = 169766,
  [SMALL_STATE(6200)] = 169780,
  [SMALL_STATE(6201)] = 169794,
  [SMALL_STATE(6202)] = 169808,
  [SMALL_STATE(6203)] = 169822,
  [SMALL_STATE(6204)] = 169836,
  [SMALL_STATE(6205)] = 169850,
  [SMALL_STATE(6206)] = 169864,
  [SMALL_STATE(6207)] = 169878,
  [SMALL_STATE(6208)] = 169892,
  [SMALL_STATE(6209)] = 169906,
  [SMALL_STATE(6210)] = 169920,
  [SMALL_STATE(6211)] = 169934,
  [SMALL_STATE(6212)] = 169948,
  [SMALL_STATE(6213)] = 169962,
  [SMALL_STATE(6214)] = 169976,
  [SMALL_STATE(6215)] = 169990,
  [SMALL_STATE(6216)] = 170004,
  [SMALL_STATE(6217)] = 170018,
  [SMALL_STATE(6218)] = 170032,
  [SMALL_STATE(6219)] = 170046,
  [SMALL_STATE(6220)] = 170060,
  [SMALL_STATE(6221)] = 170074,
  [SMALL_STATE(6222)] = 170088,
  [SMALL_STATE(6223)] = 170102,
  [SMALL_STATE(6224)] = 170116,
  [SMALL_STATE(6225)] = 170130,
  [SMALL_STATE(6226)] = 170144,
  [SMALL_STATE(6227)] = 170158,
  [SMALL_STATE(6228)] = 170172,
  [SMALL_STATE(6229)] = 170186,
  [SMALL_STATE(6230)] = 170200,
  [SMALL_STATE(6231)] = 170214,
  [SMALL_STATE(6232)] = 170228,
  [SMALL_STATE(6233)] = 170242,
  [SMALL_STATE(6234)] = 170256,
  [SMALL_STATE(6235)] = 170270,
  [SMALL_STATE(6236)] = 170284,
  [SMALL_STATE(6237)] = 170298,
  [SMALL_STATE(6238)] = 170312,
  [SMALL_STATE(6239)] = 170326,
  [SMALL_STATE(6240)] = 170340,
  [SMALL_STATE(6241)] = 170354,
  [SMALL_STATE(6242)] = 170368,
  [SMALL_STATE(6243)] = 170382,
  [SMALL_STATE(6244)] = 170396,
  [SMALL_STATE(6245)] = 170410,
  [SMALL_STATE(6246)] = 170424,
  [SMALL_STATE(6247)] = 170438,
  [SMALL_STATE(6248)] = 170452,
  [SMALL_STATE(6249)] = 170466,
  [SMALL_STATE(6250)] = 170480,
  [SMALL_STATE(6251)] = 170494,
  [SMALL_STATE(6252)] = 170508,
  [SMALL_STATE(6253)] = 170522,
  [SMALL_STATE(6254)] = 170536,
  [SMALL_STATE(6255)] = 170550,
  [SMALL_STATE(6256)] = 170564,
  [SMALL_STATE(6257)] = 170578,
  [SMALL_STATE(6258)] = 170592,
  [SMALL_STATE(6259)] = 170606,
  [SMALL_STATE(6260)] = 170620,
  [SMALL_STATE(6261)] = 170634,
  [SMALL_STATE(6262)] = 170648,
  [SMALL_STATE(6263)] = 170662,
  [SMALL_STATE(6264)] = 170676,
  [SMALL_STATE(6265)] = 170690,
  [SMALL_STATE(6266)] = 170704,
  [SMALL_STATE(6267)] = 170718,
  [SMALL_STATE(6268)] = 170732,
  [SMALL_STATE(6269)] = 170746,
  [SMALL_STATE(6270)] = 170760,
  [SMALL_STATE(6271)] = 170774,
  [SMALL_STATE(6272)] = 170788,
  [SMALL_STATE(6273)] = 170802,
  [SMALL_STATE(6274)] = 170816,
  [SMALL_STATE(6275)] = 170830,
  [SMALL_STATE(6276)] = 170844,
  [SMALL_STATE(6277)] = 170858,
  [SMALL_STATE(6278)] = 170872,
  [SMALL_STATE(6279)] = 170886,
  [SMALL_STATE(6280)] = 170900,
  [SMALL_STATE(6281)] = 170914,
  [SMALL_STATE(6282)] = 170928,
  [SMALL_STATE(6283)] = 170942,
  [SMALL_STATE(6284)] = 170956,
  [SMALL_STATE(6285)] = 170970,
  [SMALL_STATE(6286)] = 170984,
  [SMALL_STATE(6287)] = 170998,
  [SMALL_STATE(6288)] = 171012,
  [SMALL_STATE(6289)] = 171026,
  [SMALL_STATE(6290)] = 171040,
  [SMALL_STATE(6291)] = 171054,
  [SMALL_STATE(6292)] = 171068,
  [SMALL_STATE(6293)] = 171082,
  [SMALL_STATE(6294)] = 171096,
  [SMALL_STATE(6295)] = 171110,
  [SMALL_STATE(6296)] = 171124,
  [SMALL_STATE(6297)] = 171138,
  [SMALL_STATE(6298)] = 171152,
  [SMALL_STATE(6299)] = 171166,
  [SMALL_STATE(6300)] = 171180,
  [SMALL_STATE(6301)] = 171194,
  [SMALL_STATE(6302)] = 171208,
  [SMALL_STATE(6303)] = 171222,
  [SMALL_STATE(6304)] = 171236,
  [SMALL_STATE(6305)] = 171250,
  [SMALL_STATE(6306)] = 171264,
  [SMALL_STATE(6307)] = 171278,
  [SMALL_STATE(6308)] = 171292,
  [SMALL_STATE(6309)] = 171306,
  [SMALL_STATE(6310)] = 171320,
  [SMALL_STATE(6311)] = 171334,
  [SMALL_STATE(6312)] = 171348,
  [SMALL_STATE(6313)] = 171362,
  [SMALL_STATE(6314)] = 171376,
  [SMALL_STATE(6315)] = 171390,
  [SMALL_STATE(6316)] = 171404,
  [SMALL_STATE(6317)] = 171418,
  [SMALL_STATE(6318)] = 171432,
  [SMALL_STATE(6319)] = 171446,
  [SMALL_STATE(6320)] = 171460,
  [SMALL_STATE(6321)] = 171474,
  [SMALL_STATE(6322)] = 171488,
  [SMALL_STATE(6323)] = 171502,
  [SMALL_STATE(6324)] = 171516,
  [SMALL_STATE(6325)] = 171530,
  [SMALL_STATE(6326)] = 171544,
  [SMALL_STATE(6327)] = 171558,
  [SMALL_STATE(6328)] = 171572,
  [SMALL_STATE(6329)] = 171586,
  [SMALL_STATE(6330)] = 171600,
  [SMALL_STATE(6331)] = 171614,
  [SMALL_STATE(6332)] = 171628,
  [SMALL_STATE(6333)] = 171642,
  [SMALL_STATE(6334)] = 171656,
  [SMALL_STATE(6335)] = 171670,
  [SMALL_STATE(6336)] = 171684,
  [SMALL_STATE(6337)] = 171698,
  [SMALL_STATE(6338)] = 171712,
  [SMALL_STATE(6339)] = 171726,
  [SMALL_STATE(6340)] = 171740,
  [SMALL_STATE(6341)] = 171754,
  [SMALL_STATE(6342)] = 171768,
  [SMALL_STATE(6343)] = 171782,
  [SMALL_STATE(6344)] = 171796,
  [SMALL_STATE(6345)] = 171810,
  [SMALL_STATE(6346)] = 171824,
  [SMALL_STATE(6347)] = 171838,
  [SMALL_STATE(6348)] = 171852,
  [SMALL_STATE(6349)] = 171866,
  [SMALL_STATE(6350)] = 171880,
  [SMALL_STATE(6351)] = 171894,
  [SMALL_STATE(6352)] = 171908,
  [SMALL_STATE(6353)] = 171922,
  [SMALL_STATE(6354)] = 171936,
  [SMALL_STATE(6355)] = 171950,
  [SMALL_STATE(6356)] = 171964,
  [SMALL_STATE(6357)] = 171978,
  [SMALL_STATE(6358)] = 171992,
  [SMALL_STATE(6359)] = 172006,
  [SMALL_STATE(6360)] = 172020,
  [SMALL_STATE(6361)] = 172034,
  [SMALL_STATE(6362)] = 172048,
  [SMALL_STATE(6363)] = 172062,
  [SMALL_STATE(6364)] = 172076,
  [SMALL_STATE(6365)] = 172090,
  [SMALL_STATE(6366)] = 172104,
  [SMALL_STATE(6367)] = 172118,
  [SMALL_STATE(6368)] = 172132,
  [SMALL_STATE(6369)] = 172146,
  [SMALL_STATE(6370)] = 172160,
  [SMALL_STATE(6371)] = 172174,
  [SMALL_STATE(6372)] = 172188,
  [SMALL_STATE(6373)] = 172202,
  [SMALL_STATE(6374)] = 172216,
  [SMALL_STATE(6375)] = 172230,
  [SMALL_STATE(6376)] = 172244,
  [SMALL_STATE(6377)] = 172258,
  [SMALL_STATE(6378)] = 172272,
  [SMALL_STATE(6379)] = 172286,
  [SMALL_STATE(6380)] = 172300,
  [SMALL_STATE(6381)] = 172314,
  [SMALL_STATE(6382)] = 172328,
  [SMALL_STATE(6383)] = 172342,
  [SMALL_STATE(6384)] = 172356,
  [SMALL_STATE(6385)] = 172370,
  [SMALL_STATE(6386)] = 172384,
  [SMALL_STATE(6387)] = 172398,
  [SMALL_STATE(6388)] = 172412,
  [SMALL_STATE(6389)] = 172426,
  [SMALL_STATE(6390)] = 172440,
  [SMALL_STATE(6391)] = 172454,
  [SMALL_STATE(6392)] = 172468,
  [SMALL_STATE(6393)] = 172482,
  [SMALL_STATE(6394)] = 172496,
  [SMALL_STATE(6395)] = 172510,
  [SMALL_STATE(6396)] = 172524,
  [SMALL_STATE(6397)] = 172538,
  [SMALL_STATE(6398)] = 172552,
  [SMALL_STATE(6399)] = 172566,
  [SMALL_STATE(6400)] = 172580,
  [SMALL_STATE(6401)] = 172594,
  [SMALL_STATE(6402)] = 172608,
  [SMALL_STATE(6403)] = 172622,
  [SMALL_STATE(6404)] = 172636,
  [SMALL_STATE(6405)] = 172650,
  [SMALL_STATE(6406)] = 172664,
  [SMALL_STATE(6407)] = 172678,
  [SMALL_STATE(6408)] = 172692,
  [SMALL_STATE(6409)] = 172706,
  [SMALL_STATE(6410)] = 172720,
  [SMALL_STATE(6411)] = 172734,
  [SMALL_STATE(6412)] = 172748,
  [SMALL_STATE(6413)] = 172762,
  [SMALL_STATE(6414)] = 172776,
  [SMALL_STATE(6415)] = 172790,
  [SMALL_STATE(6416)] = 172804,
  [SMALL_STATE(6417)] = 172818,
  [SMALL_STATE(6418)] = 172832,
  [SMALL_STATE(6419)] = 172846,
  [SMALL_STATE(6420)] = 172860,
  [SMALL_STATE(6421)] = 172874,
  [SMALL_STATE(6422)] = 172888,
  [SMALL_STATE(6423)] = 172902,
  [SMALL_STATE(6424)] = 172916,
  [SMALL_STATE(6425)] = 172930,
  [SMALL_STATE(6426)] = 172944,
  [SMALL_STATE(6427)] = 172958,
  [SMALL_STATE(6428)] = 172972,
  [SMALL_STATE(6429)] = 172986,
  [SMALL_STATE(6430)] = 173000,
  [SMALL_STATE(6431)] = 173014,
  [SMALL_STATE(6432)] = 173028,
  [SMALL_STATE(6433)] = 173042,
  [SMALL_STATE(6434)] = 173056,
  [SMALL_STATE(6435)] = 173070,
  [SMALL_STATE(6436)] = 173084,
  [SMALL_STATE(6437)] = 173098,
  [SMALL_STATE(6438)] = 173112,
  [SMALL_STATE(6439)] = 173126,
  [SMALL_STATE(6440)] = 173140,
  [SMALL_STATE(6441)] = 173154,
  [SMALL_STATE(6442)] = 173168,
  [SMALL_STATE(6443)] = 173182,
  [SMALL_STATE(6444)] = 173196,
  [SMALL_STATE(6445)] = 173210,
  [SMALL_STATE(6446)] = 173224,
  [SMALL_STATE(6447)] = 173238,
  [SMALL_STATE(6448)] = 173252,
  [SMALL_STATE(6449)] = 173266,
  [SMALL_STATE(6450)] = 173280,
  [SMALL_STATE(6451)] = 173294,
  [SMALL_STATE(6452)] = 173308,
  [SMALL_STATE(6453)] = 173322,
  [SMALL_STATE(6454)] = 173336,
  [SMALL_STATE(6455)] = 173350,
  [SMALL_STATE(6456)] = 173364,
  [SMALL_STATE(6457)] = 173378,
  [SMALL_STATE(6458)] = 173392,
  [SMALL_STATE(6459)] = 173406,
  [SMALL_STATE(6460)] = 173420,
  [SMALL_STATE(6461)] = 173434,
  [SMALL_STATE(6462)] = 173448,
  [SMALL_STATE(6463)] = 173462,
  [SMALL_STATE(6464)] = 173476,
  [SMALL_STATE(6465)] = 173490,
  [SMALL_STATE(6466)] = 173504,
  [SMALL_STATE(6467)] = 173518,
  [SMALL_STATE(6468)] = 173532,
  [SMALL_STATE(6469)] = 173546,
  [SMALL_STATE(6470)] = 173560,
  [SMALL_STATE(6471)] = 173574,
  [SMALL_STATE(6472)] = 173588,
  [SMALL_STATE(6473)] = 173602,
  [SMALL_STATE(6474)] = 173616,
  [SMALL_STATE(6475)] = 173630,
  [SMALL_STATE(6476)] = 173644,
  [SMALL_STATE(6477)] = 173658,
  [SMALL_STATE(6478)] = 173672,
  [SMALL_STATE(6479)] = 173686,
  [SMALL_STATE(6480)] = 173700,
  [SMALL_STATE(6481)] = 173714,
  [SMALL_STATE(6482)] = 173728,
  [SMALL_STATE(6483)] = 173742,
  [SMALL_STATE(6484)] = 173756,
  [SMALL_STATE(6485)] = 173770,
  [SMALL_STATE(6486)] = 173784,
  [SMALL_STATE(6487)] = 173798,
  [SMALL_STATE(6488)] = 173812,
  [SMALL_STATE(6489)] = 173826,
  [SMALL_STATE(6490)] = 173840,
  [SMALL_STATE(6491)] = 173854,
  [SMALL_STATE(6492)] = 173868,
  [SMALL_STATE(6493)] = 173882,
  [SMALL_STATE(6494)] = 173896,
  [SMALL_STATE(6495)] = 173910,
  [SMALL_STATE(6496)] = 173924,
  [SMALL_STATE(6497)] = 173938,
  [SMALL_STATE(6498)] = 173952,
  [SMALL_STATE(6499)] = 173956,
  [SMALL_STATE(6500)] = 173960,
  [SMALL_STATE(6501)] = 173964,
  [SMALL_STATE(6502)] = 173968,
  [SMALL_STATE(6503)] = 173972,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5156),
  [5] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5081),
  [7] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 0, 0, 0),
  [9] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3030),
  [11] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1019),
  [13] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5115),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(146),
  [17] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3),
  [19] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3056),
  [21] = {.entry = {.count = 1, .reusable = true}}, SHIFT(257),
  [23] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [25] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3009),
  [27] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [29] = {.entry = {.count = 1, .reusable = true}}, SHIFT(547),
  [31] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1448),
  [33] = {.entry = {.count = 1, .reusable = true}}, SHIFT(43),
  [35] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5587),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5461),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6119),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3616),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3046),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3055),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(262),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(44),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3673),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2813),
  [55] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2782),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6247),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5395),
  [61] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1421),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2475),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(81),
  [67] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1445),
  [69] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1426),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4677),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(176),
  [75] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6036),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3502),
  [79] = {.entry = {.count = 1, .reusable = false}}, SHIFT(47),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4051),
  [83] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6107),
  [85] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6151),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5996),
  [89] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3052),
  [91] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3570),
  [93] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2997),
  [95] = {.entry = {.count = 1, .reusable = false}}, SHIFT(77),
  [97] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3948),
  [99] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2764),
  [101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(46),
  [103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5499),
  [105] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2971),
  [107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2915),
  [109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4525),
  [111] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2882),
  [113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2),
  [115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2991),
  [117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5968),
  [119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3514),
  [121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2849),
  [123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6101),
  [125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5549),
  [127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6005),
  [129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6401),
  [131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 1, 0, 0),
  [133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5461),
  [135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(420),
  [137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2461),
  [139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(429),
  [141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 2, 0, 0),
  [143] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0),
  [145] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3030),
  [148] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1019),
  [151] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5115),
  [154] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(146),
  [157] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3),
  [160] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3056),
  [163] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(257),
  [166] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(100),
  [169] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3009),
  [172] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(147),
  [175] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(547),
  [178] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1448),
  [181] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(43),
  [184] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5587),
  [187] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5461),
  [190] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(6119),
  [193] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3616),
  [196] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3046),
  [199] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3055),
  [202] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(262),
  [205] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(44),
  [208] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3673),
  [211] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2813),
  [214] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2782),
  [217] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(6247),
  [220] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5395),
  [223] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1421),
  [226] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2475),
  [229] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(81),
  [232] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1445),
  [235] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1426),
  [238] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(4677),
  [241] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(176),
  [244] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(6036),
  [247] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3502),
  [250] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(47),
  [253] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(4051),
  [256] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(6107),
  [259] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(6151),
  [262] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5996),
  [265] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3052),
  [268] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3570),
  [271] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2997),
  [274] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(77),
  [277] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3948),
  [280] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2764),
  [283] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(46),
  [286] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5499),
  [289] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2971),
  [292] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2915),
  [295] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(4525),
  [298] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2882),
  [301] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2991),
  [304] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5968),
  [307] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(3514),
  [310] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2849),
  [313] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(6101),
  [316] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5549),
  [319] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(6005),
  [322] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 3, 0, 0),
  [324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [326] = {.entry = {.count = 1, .reusable = true}}, SHIFT(426),
  [328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2895),
  [330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2862),
  [332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2972),
  [334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3711),
  [336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3712),
  [338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3713),
  [340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3718),
  [342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1146),
  [344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1167),
  [346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1171),
  [348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1172),
  [350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5441),
  [352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5255),
  [354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4943),
  [356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5020),
  [358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(564),
  [360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(556),
  [364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3163),
  [368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3144),
  [370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3176),
  [372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3142),
  [374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2458),
  [376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2459),
  [378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2460),
  [380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2951),
  [382] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2797),
  [384] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_expression, 2, 0, 0),
  [386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(14),
  [388] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2944),
  [390] = {.entry = {.count = 1, .reusable = false}}, SHIFT(257),
  [392] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4680),
  [394] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_expression, 2, 0, 0),
  [396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6032),
  [398] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4042),
  [400] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4606),
  [402] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1431),
  [404] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2774),
  [406] = {.entry = {.count = 1, .reusable = false}}, SHIFT(82),
  [408] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4568),
  [410] = {.entry = {.count = 1, .reusable = false}}, SHIFT(287),
  [412] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4651),
  [414] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4678),
  [416] = {.entry = {.count = 1, .reusable = false}}, SHIFT(83),
  [418] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2779),
  [420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5608),
  [422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6051),
  [424] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_expression, 1, 0, 0),
  [426] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_expression, 1, 0, 0),
  [428] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_expression, 1, 0, 0),
  [430] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_expression, 1, 0, 0),
  [432] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6068),
  [434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_expression, 2, 0, 0),
  [436] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_expression, 2, 0, 0),
  [438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield_expression, 1, 0, 0),
  [440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_yield_expression, 1, 0, 0),
  [442] = {.entry = {.count = 1, .reusable = false}}, SHIFT(147),
  [444] = {.entry = {.count = 1, .reusable = false}}, SHIFT(547),
  [446] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1448),
  [448] = {.entry = {.count = 1, .reusable = false}}, SHIFT(48),
  [450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_expression, 1, 0, 0),
  [452] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_expression, 1, 0, 0),
  [454] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3053),
  [456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(34),
  [458] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3100),
  [460] = {.entry = {.count = 1, .reusable = false}}, SHIFT(175),
  [462] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4812),
  [464] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3171),
  [466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5460),
  [468] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3989),
  [470] = {.entry = {.count = 1, .reusable = false}}, SHIFT(178),
  [472] = {.entry = {.count = 1, .reusable = false}}, SHIFT(57),
  [474] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4439),
  [476] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3083),
  [478] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1435),
  [480] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2866),
  [482] = {.entry = {.count = 1, .reusable = false}}, SHIFT(87),
  [484] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4601),
  [486] = {.entry = {.count = 1, .reusable = false}}, SHIFT(327),
  [488] = {.entry = {.count = 1, .reusable = false}}, SHIFT(58),
  [490] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4633),
  [492] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4547),
  [494] = {.entry = {.count = 1, .reusable = false}}, SHIFT(88),
  [496] = {.entry = {.count = 1, .reusable = false}}, SHIFT(59),
  [498] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5571),
  [500] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3005),
  [502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3203),
  [504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4646),
  [506] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3212),
  [508] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3159),
  [510] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6215),
  [512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3151),
  [514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6285),
  [516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(155),
  [518] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3047),
  [520] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3066),
  [522] = {.entry = {.count = 1, .reusable = false}}, SHIFT(258),
  [524] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3084),
  [526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5614),
  [528] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4033),
  [530] = {.entry = {.count = 1, .reusable = false}}, SHIFT(259),
  [532] = {.entry = {.count = 1, .reusable = false}}, SHIFT(71),
  [534] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3089),
  [536] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2995),
  [538] = {.entry = {.count = 1, .reusable = false}}, SHIFT(73),
  [540] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4487),
  [542] = {.entry = {.count = 1, .reusable = false}}, SHIFT(72),
  [544] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5619),
  [546] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3034),
  [548] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3090),
  [550] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6222),
  [552] = {.entry = {.count = 1, .reusable = false}}, SHIFT(270),
  [554] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3975),
  [556] = {.entry = {.count = 1, .reusable = false}}, SHIFT(271),
  [558] = {.entry = {.count = 1, .reusable = false}}, SHIFT(63),
  [560] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3006),
  [562] = {.entry = {.count = 1, .reusable = false}}, SHIFT(64),
  [564] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4546),
  [566] = {.entry = {.count = 1, .reusable = false}}, SHIFT(61),
  [568] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5462),
  [570] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3028),
  [572] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6143),
  [574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(99),
  [576] = {.entry = {.count = 1, .reusable = false}}, SHIFT(145),
  [578] = {.entry = {.count = 1, .reusable = false}}, SHIFT(54),
  [580] = {.entry = {.count = 1, .reusable = false}}, SHIFT(154),
  [582] = {.entry = {.count = 1, .reusable = false}}, SHIFT(60),
  [584] = {.entry = {.count = 1, .reusable = false}}, SHIFT(157),
  [586] = {.entry = {.count = 1, .reusable = false}}, SHIFT(67),
  [588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1279),
  [590] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5202),
  [592] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3057),
  [594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5785),
  [596] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3054),
  [598] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3680),
  [600] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6137),
  [602] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5261),
  [604] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1470),
  [606] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1403),
  [608] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6157),
  [610] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4039),
  [612] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6173),
  [614] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6183),
  [616] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6184),
  [618] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3048),
  [620] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3591),
  [622] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3008),
  [624] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3970),
  [626] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2767),
  [628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(157),
  [632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(62),
  [634] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1424),
  [636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(175),
  [638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(145),
  [640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(53),
  [642] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1401),
  [644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2969),
  [646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3636),
  [648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3342),
  [650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3632),
  [652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3301),
  [654] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3286),
  [656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1782),
  [660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(270),
  [662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(154),
  [664] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2892),
  [666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(60),
  [668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1533),
  [670] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2940),
  [672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1720),
  [676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(67),
  [678] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2876),
  [680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1626),
  [682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6400),
  [684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2945),
  [686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6240),
  [688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3359),
  [690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6320),
  [692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2889),
  [694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6011),
  [696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3310),
  [698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3403),
  [700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6265),
  [702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2986),
  [704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3188),
  [706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3343),
  [708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2864),
  [710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6069),
  [712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2900),
  [714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2947),
  [716] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2797),
  [719] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(146),
  [722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0),
  [724] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(14),
  [727] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2944),
  [730] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(257),
  [733] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(100),
  [736] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(3009),
  [739] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(147),
  [742] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(547),
  [745] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1448),
  [748] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(43),
  [751] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(5587),
  [754] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(6032),
  [757] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(6119),
  [760] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(4042),
  [763] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(262),
  [766] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(44),
  [769] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(4606),
  [772] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2813),
  [775] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1431),
  [778] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2774),
  [781] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(82),
  [784] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(4568),
  [787] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(287),
  [790] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(47),
  [793] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(4651),
  [796] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(4678),
  [799] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(83),
  [802] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(46),
  [805] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(5499),
  [808] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2779),
  [811] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2915),
  [814] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(4525),
  [817] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2882),
  [820] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2991),
  [823] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(5968),
  [826] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(2849),
  [829] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(6101),
  [832] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(5608),
  [835] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(6051),
  [838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_initializer_list_repeat1, 2, 0, 106),
  [840] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2913),
  [842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1680),
  [844] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3297),
  [846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1728),
  [848] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3326),
  [850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1730),
  [852] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3226),
  [854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1745),
  [856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3306),
  [858] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(445),
  [861] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(428),
  [864] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(345),
  [867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0),
  [869] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(336),
  [872] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(428),
  [875] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(343),
  [878] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(6085),
  [881] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(439),
  [884] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(436),
  [887] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(4544),
  [890] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(422),
  [893] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(415),
  [896] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(6210),
  [899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3200),
  [901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3337),
  [903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2948),
  [905] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3576),
  [907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(409),
  [909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4166),
  [911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6075),
  [913] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3742),
  [915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3619),
  [917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5658),
  [921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1725),
  [923] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3790),
  [925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5755),
  [927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(495),
  [929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1410),
  [931] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4370),
  [933] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1951),
  [935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5551),
  [937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2770),
  [939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5632),
  [941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6430),
  [943] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6014),
  [945] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3934),
  [947] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3701),
  [949] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3518),
  [951] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5869),
  [953] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5661),
  [955] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1726),
  [957] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3886),
  [959] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6230),
  [961] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4006),
  [963] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1406),
  [965] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1974),
  [967] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1388),
  [969] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1423),
  [971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3672),
  [973] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4766),
  [975] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3651),
  [977] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4013),
  [979] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4802),
  [981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3497),
  [983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6170),
  [985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5570),
  [987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6419),
  [989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [991] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3983),
  [993] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4603),
  [995] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1433),
  [997] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2834),
  [999] = {.entry = {.count = 1, .reusable = false}}, SHIFT(85),
  [1001] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4543),
  [1003] = {.entry = {.count = 1, .reusable = false}}, SHIFT(320),
  [1005] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4488),
  [1007] = {.entry = {.count = 1, .reusable = false}}, SHIFT(86),
  [1009] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2918),
  [1011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2911),
  [1013] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3569),
  [1015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(410),
  [1017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2819),
  [1019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6420),
  [1021] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3813),
  [1023] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3775),
  [1025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(498),
  [1027] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4287),
  [1029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5597),
  [1031] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3549),
  [1033] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4015),
  [1035] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4669),
  [1037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2890),
  [1039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2922),
  [1041] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3051),
  [1043] = {.entry = {.count = 1, .reusable = false}}, SHIFT(160),
  [1045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2974),
  [1047] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2800),
  [1049] = {.entry = {.count = 1, .reusable = false}}, SHIFT(261),
  [1051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3347),
  [1053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4193),
  [1055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6399),
  [1057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3388),
  [1059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3582),
  [1061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6136),
  [1063] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4433),
  [1065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3423),
  [1067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2873),
  [1069] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3049),
  [1071] = {.entry = {.count = 1, .reusable = false}}, SHIFT(204),
  [1073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3330),
  [1075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3150),
  [1077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6366),
  [1079] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4363),
  [1081] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3043),
  [1083] = {.entry = {.count = 1, .reusable = false}}, SHIFT(183),
  [1085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2939),
  [1087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4192),
  [1089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6061),
  [1091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [1093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3561),
  [1095] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4832),
  [1097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2823),
  [1099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2848),
  [1101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3149),
  [1103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3162),
  [1105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3571),
  [1107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(445),
  [1109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [1111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [1113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(336),
  [1115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6112),
  [1117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(428),
  [1119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [1121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6085),
  [1123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(439),
  [1125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [1127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4544),
  [1129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(422),
  [1131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(415),
  [1133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6210),
  [1135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(419),
  [1137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4788),
  [1139] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(440),
  [1142] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(428),
  [1145] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(348),
  [1148] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0),
  [1150] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(356),
  [1153] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(428),
  [1156] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(357),
  [1159] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(438),
  [1162] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(439),
  [1165] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(436),
  [1168] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(4544),
  [1171] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(422),
  [1174] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(415),
  [1177] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(6210),
  [1180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(423),
  [1182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4826),
  [1184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6045),
  [1186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6271),
  [1188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6460),
  [1190] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(453),
  [1193] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(457),
  [1196] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(403),
  [1199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0),
  [1201] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(404),
  [1204] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(457),
  [1207] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(405),
  [1210] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(453),
  [1213] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(458),
  [1216] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(460),
  [1219] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(4602),
  [1222] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(463),
  [1225] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 2, 0, 0), SHIFT_REPEAT(6260),
  [1228] = {.entry = {.count = 1, .reusable = false}}, SHIFT(440),
  [1230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [1232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(432),
  [1234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(356),
  [1236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(357),
  [1238] = {.entry = {.count = 1, .reusable = false}}, SHIFT(438),
  [1240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4342),
  [1242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4801),
  [1244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1186),
  [1246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [1248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4790),
  [1250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1353),
  [1252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2520),
  [1254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4411),
  [1256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2621),
  [1258] = {.entry = {.count = 1, .reusable = false}}, SHIFT(453),
  [1260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [1262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(403),
  [1264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [1266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3292),
  [1268] = {.entry = {.count = 1, .reusable = false}}, SHIFT(457),
  [1270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [1272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(453),
  [1274] = {.entry = {.count = 1, .reusable = false}}, SHIFT(458),
  [1276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(460),
  [1278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4602),
  [1280] = {.entry = {.count = 1, .reusable = false}}, SHIFT(463),
  [1282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6260),
  [1284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5434),
  [1286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3565),
  [1288] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5435),
  [1290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3580),
  [1292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(462),
  [1294] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3294),
  [1296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2863),
  [1298] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2454),
  [1300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2914),
  [1302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(473),
  [1304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2455),
  [1306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [1308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(461),
  [1310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4364),
  [1312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6386),
  [1314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [1316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [1318] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3786),
  [1320] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3706),
  [1322] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4604),
  [1324] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1418),
  [1326] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5538),
  [1328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4390),
  [1330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4369),
  [1332] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_macro_body_v2_repeat1, 1, 0, 0),
  [1334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_macro_body_v2_repeat1, 1, 0, 0),
  [1336] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0),
  [1338] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(428),
  [1341] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0),
  [1343] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(428),
  [1346] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__token_pattern, 1, 0, 0),
  [1348] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__token_pattern, 1, 0, 0),
  [1350] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expression, 3, 0, 41),
  [1352] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expression, 3, 0, 41),
  [1354] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4215),
  [1356] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_metavariable, 1, 0, 0),
  [1358] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_metavariable, 1, 0, 0),
  [1360] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3578),
  [1362] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__non_delim_token, 1, 0, 0),
  [1364] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__non_delim_token, 1, 0, 0),
  [1366] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_binding_pattern, 3, 0, 200),
  [1368] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_binding_pattern, 3, 0, 200),
  [1370] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree_pattern, 3, 0, 0),
  [1372] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree_pattern, 3, 0, 0),
  [1374] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 2, 0, 0),
  [1376] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 2, 0, 0),
  [1378] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 2, 0, 0),
  [1380] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 2, 0, 0),
  [1382] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_boolean_literal, 1, 0, 0),
  [1384] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_boolean_literal, 1, 0, 0),
  [1386] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree_pattern, 2, 0, 0),
  [1388] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree_pattern, 2, 0, 0),
  [1390] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree, 3, 0, 0),
  [1392] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree, 3, 0, 0),
  [1394] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 3, 0, 0),
  [1396] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 3, 0, 0),
  [1398] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 5, 0, 0),
  [1400] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 5, 0, 0),
  [1402] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition, 4, 0, 0),
  [1404] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition, 4, 0, 0),
  [1406] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 1, 0, 0),
  [1408] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 1, 0, 0),
  [1410] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 3, 0, 0),
  [1412] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 3, 0, 0),
  [1414] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition, 6, 0, 0),
  [1416] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition, 6, 0, 0),
  [1418] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fragment_specifier, 1, 0, 0),
  [1420] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fragment_specifier, 1, 0, 0),
  [1422] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_tree, 2, 0, 0),
  [1424] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_tree, 2, 0, 0),
  [1426] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 1, 0, 0),
  [1428] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_macro_arguments_declaration_repeat1, 1, 0, 0),
  [1430] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition_pattern, 5, 0, 0),
  [1432] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition_pattern, 5, 0, 0),
  [1434] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(457),
  [1437] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__non_special_token_repeat1, 2, 0, 0), SHIFT_REPEAT(457),
  [1440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__literal, 1, 0, 0),
  [1442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__literal, 1, 0, 0),
  [1444] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition_pattern, 6, 0, 0),
  [1446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition_pattern, 6, 0, 0),
  [1448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(355),
  [1450] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_primitive_type, 1, 0, 0),
  [1452] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_primitive_type, 1, 0, 0),
  [1454] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_raw_string_literal, 3, 0, 0),
  [1456] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raw_string_literal, 3, 0, 0),
  [1458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 4, 0, 0),
  [1460] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 4, 0, 0),
  [1462] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3577),
  [1464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(408),
  [1466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6177),
  [1468] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3802),
  [1470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3763),
  [1472] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3808),
  [1474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [1476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5635),
  [1478] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3531),
  [1480] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4810),
  [1482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 6, 0, 0),
  [1484] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 6, 0, 0),
  [1486] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition_pattern, 4, 0, 0),
  [1488] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition_pattern, 4, 0, 0),
  [1490] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_token_repetition, 5, 0, 0),
  [1492] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_token_repetition, 5, 0, 0),
  [1494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_delim_token_tree, 2, 0, 0),
  [1496] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_delim_token_tree, 2, 0, 0),
  [1498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_block, 4, 0, 0),
  [1500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_block, 4, 0, 0),
  [1502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_invocation_item, 2, 0, 14),
  [1504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_invocation_item, 2, 0, 14),
  [1506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression_except_range, 1, 0, 0),
  [1508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression_except_range, 1, 0, 0),
  [1510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_expression_with_attribute, 2, 0, 13),
  [1512] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_expression_with_attribute, 2, 0, 13),
  [1514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_expression, 7, 0, 400),
  [1516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_expression, 7, 0, 400),
  [1518] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__delim_tokens, 1, 0, 0),
  [1520] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__delim_tokens, 1, 0, 0),
  [1522] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_delim_token_tree_repeat1, 1, 0, 0),
  [1524] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_delim_token_tree_repeat1, 1, 0, 0),
  [1526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_expression, 4, 0, 91),
  [1528] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_expression, 4, 0, 91),
  [1530] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_block, 2, 0, 0),
  [1532] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_block, 2, 0, 0),
  [1534] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_delim_token_tree, 3, 0, 0),
  [1536] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_delim_token_tree, 3, 0, 0),
  [1538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_invocation, 3, 0, 22),
  [1540] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_invocation, 3, 0, 22),
  [1542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_loop_expression, 4, 0, 121),
  [1544] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_loop_expression, 4, 0, 121),
  [1546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_async_block, 2, 0, 0),
  [1548] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_async_block, 2, 0, 0),
  [1550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_invocation_item, 1, 0, 2),
  [1552] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_invocation_item, 1, 0, 2),
  [1554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_block, 2, 0, 9),
  [1556] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_block, 2, 0, 9),
  [1558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gen_block, 2, 0, 0),
  [1560] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gen_block, 2, 0, 0),
  [1562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_async_block, 3, 0, 0),
  [1564] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_async_block, 3, 0, 0),
  [1566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gen_block, 3, 0, 0),
  [1568] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gen_block, 3, 0, 0),
  [1570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_expression, 5, 0, 147),
  [1572] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_expression, 5, 0, 147),
  [1574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1576] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_loop_expression, 2, 0, 9),
  [1580] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_loop_expression, 2, 0, 9),
  [1582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_block, 3, 0, 0),
  [1584] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_block, 3, 0, 0),
  [1586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unsafe_block, 2, 0, 0),
  [1588] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unsafe_block, 2, 0, 0),
  [1590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_expression, 3, 0, 46),
  [1592] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_expression, 3, 0, 46),
  [1594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_expression, 5, 0, 199),
  [1596] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_expression, 5, 0, 199),
  [1598] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_expression, 3, 0, 50),
  [1600] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_expression, 3, 0, 50),
  [1602] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_block, 2, 0, 0),
  [1604] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_block, 2, 0, 0),
  [1606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_invocation, 3, 0, 51),
  [1608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_invocation, 3, 0, 51),
  [1610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [1612] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [1614] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_block, 5, 0, 0),
  [1616] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_block, 5, 0, 0),
  [1618] = {.entry = {.count = 1, .reusable = false}}, SHIFT(494),
  [1620] = {.entry = {.count = 1, .reusable = false}}, SHIFT(489),
  [1622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(497),
  [1624] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4163),
  [1626] = {.entry = {.count = 1, .reusable = false}}, SHIFT(496),
  [1628] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4165),
  [1630] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4174),
  [1632] = {.entry = {.count = 1, .reusable = false}}, SHIFT(499),
  [1634] = {.entry = {.count = 1, .reusable = false}}, SHIFT(491),
  [1636] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4185),
  [1638] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_label, 2, 0, 0),
  [1640] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_label, 2, 0, 0),
  [1642] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_parameters, 5, 0, 0),
  [1644] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_parameters, 5, 0, 0),
  [1646] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_parameters, 4, 0, 0),
  [1648] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_parameters, 4, 0, 0),
  [1650] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_parameters, 2, 0, 0),
  [1652] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_parameters, 2, 0, 0),
  [1654] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_parameters, 3, 0, 0),
  [1656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_parameters, 3, 0, 0),
  [1658] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3691),
  [1660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(562),
  [1662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2887),
  [1664] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3780),
  [1666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1361),
  [1668] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3902),
  [1670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5669),
  [1672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1414),
  [1674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1411),
  [1676] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5028),
  [1678] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2747),
  [1680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2773),
  [1682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5613),
  [1684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5921),
  [1686] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4545),
  [1688] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1415),
  [1690] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1412),
  [1692] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1425),
  [1694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4104),
  [1696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4630),
  [1698] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4086),
  [1700] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4534),
  [1702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6280),
  [1704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5459),
  [1706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6049),
  [1708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2943),
  [1710] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3720),
  [1712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1453),
  [1714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [1716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6368),
  [1718] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4806),
  [1720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1718),
  [1722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5741),
  [1724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1719),
  [1726] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3546),
  [1728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1464),
  [1730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2854),
  [1732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5670),
  [1734] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6127),
  [1736] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3699),
  [1738] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5612),
  [1740] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5623),
  [1742] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1723),
  [1744] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3825),
  [1746] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1969),
  [1748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5065),
  [1750] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6234),
  [1752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(479),
  [1754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6322),
  [1756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3108),
  [1758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3432),
  [1760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [1762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [1764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6251),
  [1766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4248),
  [1768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(459),
  [1770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3407),
  [1772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6078),
  [1774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3510),
  [1776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4259),
  [1778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2846),
  [1780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3522),
  [1782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2822),
  [1784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3096),
  [1786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2844),
  [1788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4257),
  [1790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3517),
  [1792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4264),
  [1794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3113),
  [1796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3082),
  [1798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3515),
  [1800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6301),
  [1802] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(3691),
  [1805] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(562),
  [1808] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(3780),
  [1811] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1361),
  [1814] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(3902),
  [1817] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(5669),
  [1820] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1414),
  [1823] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1411),
  [1826] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1448),
  [1829] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(5028),
  [1832] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2747),
  [1835] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2773),
  [1838] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(5613),
  [1841] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(6301),
  [1844] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(4545),
  [1847] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1415),
  [1850] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1412),
  [1853] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1425),
  [1856] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(4104),
  [1859] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(4630),
  [1862] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(4086),
  [1865] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(4534),
  [1868] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2849),
  [1871] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(6280),
  [1874] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(5459),
  [1877] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 2, 0, 0), SHIFT_REPEAT(6049),
  [1880] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3682),
  [1882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6037),
  [1884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2762),
  [1886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6426),
  [1888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6340),
  [1890] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3847),
  [1892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5537),
  [1894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6111),
  [1896] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5611),
  [1898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2463),
  [1900] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5059),
  [1902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2453),
  [1904] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5530),
  [1906] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6114),
  [1908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5523),
  [1910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5866),
  [1912] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5277),
  [1914] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5560),
  [1916] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3855),
  [1918] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3779),
  [1920] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6176),
  [1922] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5337),
  [1924] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1449),
  [1926] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1400),
  [1928] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6189),
  [1930] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5229),
  [1932] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6193),
  [1934] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6195),
  [1936] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6202),
  [1938] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4949),
  [1940] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3695),
  [1942] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2996),
  [1944] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3958),
  [1946] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3709),
  [1948] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6226),
  [1950] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3516),
  [1952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1094),
  [1954] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3506),
  [1956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [1958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6088),
  [1960] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3545),
  [1962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1363),
  [1964] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3610),
  [1966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1397),
  [1968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [1970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5685),
  [1972] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4473),
  [1974] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1430),
  [1976] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4405),
  [1978] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4747),
  [1980] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5611),
  [1983] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2463),
  [1986] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5059),
  [1989] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0),
  [1991] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5530),
  [1994] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6114),
  [1997] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1448),
  [2000] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5523),
  [2003] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5866),
  [2006] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3934),
  [2009] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5277),
  [2012] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5560),
  [2015] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3855),
  [2018] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3779),
  [2021] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6176),
  [2024] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5337),
  [2027] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1449),
  [2030] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1400),
  [2033] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6189),
  [2036] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3502),
  [2039] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5229),
  [2042] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6193),
  [2045] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6195),
  [2048] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6202),
  [2051] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(4949),
  [2054] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3695),
  [2057] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2996),
  [2060] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3958),
  [2063] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3709),
  [2066] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6226),
  [2069] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3516),
  [2072] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2849),
  [2075] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(5537),
  [2078] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(6111),
  [2081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1154),
  [2083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2507),
  [2085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(502),
  [2087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [2089] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4101),
  [2091] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3519),
  [2093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4378),
  [2095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6052),
  [2097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1461),
  [2099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6432),
  [2101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6014),
  [2103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2781),
  [2105] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2777),
  [2107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5595),
  [2109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6002),
  [2111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4428),
  [2113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3807),
  [2115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5396),
  [2117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4269),
  [2119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4438),
  [2121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4272),
  [2123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4329),
  [2125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5122),
  [2127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6016),
  [2129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4374),
  [2131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4387),
  [2133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4268),
  [2135] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 3, 0, 236),
  [2137] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 3, 0, 236),
  [2139] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 4, 0, 148),
  [2141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 4, 0, 148),
  [2143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5248),
  [2145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3777),
  [2147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5093),
  [2149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3771),
  [2151] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attributes, 1, 0, 0),
  [2153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attributes, 1, 0, 0),
  [2155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6413),
  [2157] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0),
  [2159] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0),
  [2161] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6413),
  [2164] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(5549),
  [2167] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6005),
  [2170] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 545),
  [2172] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 545),
  [2174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 295),
  [2176] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 295),
  [2178] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 2, 0, 0),
  [2180] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 2, 0, 0),
  [2182] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 7, 0, 123),
  [2184] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 7, 0, 123),
  [2186] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 7, 0, 105),
  [2188] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 7, 0, 105),
  [2190] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 213),
  [2192] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 213),
  [2194] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 214),
  [2196] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 214),
  [2198] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 215),
  [2200] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 215),
  [2202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 301),
  [2204] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 301),
  [2206] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 302),
  [2208] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 302),
  [2210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 303),
  [2212] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 303),
  [2214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 304),
  [2216] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 304),
  [2218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 7, 0, 305),
  [2220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 7, 0, 305),
  [2222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 2, 0, 9),
  [2224] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 2, 0, 9),
  [2226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 5, 0, 0),
  [2228] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 5, 0, 0),
  [2230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 308),
  [2232] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 308),
  [2234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 309),
  [2236] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 309),
  [2238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 219),
  [2240] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 219),
  [2242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 310),
  [2244] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 310),
  [2246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 221),
  [2248] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 221),
  [2250] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 311),
  [2252] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 311),
  [2254] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 223),
  [2256] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 223),
  [2258] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 312),
  [2260] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 312),
  [2262] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 313),
  [2264] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 313),
  [2266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 314),
  [2268] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 314),
  [2270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 315),
  [2272] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 315),
  [2274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 316),
  [2276] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 316),
  [2278] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 317),
  [2280] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 317),
  [2282] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 318),
  [2284] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 318),
  [2286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 319),
  [2288] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 319),
  [2290] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 320),
  [2292] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 320),
  [2294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 228),
  [2296] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 228),
  [2298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 321),
  [2300] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 321),
  [2302] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 230),
  [2304] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 230),
  [2306] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 322),
  [2308] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 322),
  [2310] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 323),
  [2312] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 323),
  [2314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 324),
  [2316] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 324),
  [2318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 325),
  [2320] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 325),
  [2322] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 216),
  [2324] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 216),
  [2326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [2328] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [2330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 7, 0, 327),
  [2332] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 7, 0, 327),
  [2334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 7, 0, 305),
  [2336] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 7, 0, 305),
  [2338] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 3, 0, 12),
  [2340] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 3, 0, 12),
  [2342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 4, 0, 0),
  [2344] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 4, 0, 0),
  [2346] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 144),
  [2348] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 144),
  [2350] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 218),
  [2352] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 218),
  [2354] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 5, 0, 0),
  [2356] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 5, 0, 0),
  [2358] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_never_type, 1, 0, 0),
  [2360] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_never_type, 1, 0, 0),
  [2362] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 332),
  [2364] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 332),
  [2366] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 333),
  [2368] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 333),
  [2370] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 334),
  [2372] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 334),
  [2374] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 247),
  [2376] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 247),
  [2378] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 248),
  [2380] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 248),
  [2382] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 335),
  [2384] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 335),
  [2386] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 249),
  [2388] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 249),
  [2390] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 336),
  [2392] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 336),
  [2394] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 337),
  [2396] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 337),
  [2398] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 165),
  [2400] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 165),
  [2402] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 338),
  [2404] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 338),
  [2406] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 339),
  [2408] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 339),
  [2410] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 340),
  [2412] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 340),
  [2414] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 341),
  [2416] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 341),
  [2418] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 342),
  [2420] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 342),
  [2422] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 343),
  [2424] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 343),
  [2426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 344),
  [2428] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 344),
  [2430] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 345),
  [2432] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 345),
  [2434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 7, 0, 257),
  [2436] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 7, 0, 257),
  [2438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 7, 0, 187),
  [2440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 7, 0, 187),
  [2442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 346),
  [2444] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 346),
  [2446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 347),
  [2448] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 347),
  [2450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 348),
  [2452] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 348),
  [2454] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 349),
  [2456] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 349),
  [2458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 263),
  [2460] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 263),
  [2462] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 350),
  [2464] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 350),
  [2466] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 351),
  [2468] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 351),
  [2470] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 352),
  [2472] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 352),
  [2474] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 353),
  [2476] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 353),
  [2478] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 354),
  [2480] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 354),
  [2482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 355),
  [2484] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 355),
  [2486] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 356),
  [2488] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 356),
  [2490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 266),
  [2492] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 266),
  [2494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 357),
  [2496] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 357),
  [2498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 358),
  [2500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 358),
  [2502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 359),
  [2504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 359),
  [2506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 360),
  [2508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 360),
  [2510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 361),
  [2512] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 361),
  [2514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 269),
  [2516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 269),
  [2518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 362),
  [2520] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 362),
  [2522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 363),
  [2524] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 363),
  [2526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_rules_v2, 2, 0, 0),
  [2528] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_rules_v2, 2, 0, 0),
  [2530] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 364),
  [2532] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 364),
  [2534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 7, 0, 365),
  [2536] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 7, 0, 365),
  [2538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 7, 0, 366),
  [2540] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 7, 0, 366),
  [2542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 219),
  [2544] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 219),
  [2546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 7, 0, 175),
  [2548] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 7, 0, 175),
  [2550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 367),
  [2552] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 367),
  [2554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 368),
  [2556] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 368),
  [2558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 369),
  [2560] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 369),
  [2562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 370),
  [2564] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 370),
  [2566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 276),
  [2568] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 276),
  [2570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 371),
  [2572] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 371),
  [2574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 372),
  [2576] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 372),
  [2578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 184),
  [2580] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 184),
  [2582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 373),
  [2584] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 373),
  [2586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 182),
  [2588] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 182),
  [2590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 277),
  [2592] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 277),
  [2594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 374),
  [2596] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 374),
  [2598] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 375),
  [2600] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 375),
  [2602] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 376),
  [2604] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 376),
  [2606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 279),
  [2608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 279),
  [2610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 7, 0, 377),
  [2612] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 7, 0, 377),
  [2614] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 7, 0, 378),
  [2616] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 7, 0, 378),
  [2618] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 379),
  [2620] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 379),
  [2622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 380),
  [2624] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 380),
  [2626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 282),
  [2628] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 282),
  [2630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 381),
  [2632] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 381),
  [2634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 7, 0, 366),
  [2636] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 7, 0, 366),
  [2638] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 4, 0, 67),
  [2640] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 4, 0, 67),
  [2642] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 7, 0, 348),
  [2644] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 7, 0, 348),
  [2646] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 7, 0, 173),
  [2648] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 7, 0, 173),
  [2650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 7, 0, 260),
  [2652] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 7, 0, 260),
  [2654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 7, 0, 285),
  [2656] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 7, 0, 285),
  [2658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 7, 0, 348),
  [2660] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 7, 0, 348),
  [2662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 382),
  [2664] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 382),
  [2666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 190),
  [2668] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 190),
  [2670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 286),
  [2672] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 286),
  [2674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 285),
  [2676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 285),
  [2678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 383),
  [2680] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 383),
  [2682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 7, 0, 348),
  [2684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 7, 0, 348),
  [2686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 384),
  [2688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 384),
  [2690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 385),
  [2692] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 385),
  [2694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 386),
  [2696] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 386),
  [2698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 288),
  [2700] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 288),
  [2702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 387),
  [2704] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 387),
  [2706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 388),
  [2708] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 388),
  [2710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 389),
  [2712] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 389),
  [2714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 7, 0, 139),
  [2716] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 7, 0, 139),
  [2718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 291),
  [2720] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 291),
  [2722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 390),
  [2724] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 390),
  [2726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 391),
  [2728] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 391),
  [2730] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 197),
  [2732] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 197),
  [2734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 392),
  [2736] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 392),
  [2738] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 196),
  [2740] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 196),
  [2742] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 7, 0, 292),
  [2744] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 7, 0, 292),
  [2746] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 393),
  [2748] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 393),
  [2750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 394),
  [2752] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 394),
  [2754] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 7, 0, 395),
  [2756] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 7, 0, 395),
  [2758] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 7, 0, 396),
  [2760] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 7, 0, 396),
  [2762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 397),
  [2764] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 397),
  [2766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 398),
  [2768] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 398),
  [2770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 7, 0, 294),
  [2772] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 7, 0, 294),
  [2774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 7, 0, 399),
  [2776] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 7, 0, 399),
  [2778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 220),
  [2780] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 220),
  [2782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 8, 0, 123),
  [2784] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 8, 0, 123),
  [2786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 8, 0, 105),
  [2788] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 8, 0, 105),
  [2790] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 404),
  [2792] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 404),
  [2794] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 405),
  [2796] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 405),
  [2798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 406),
  [2800] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 406),
  [2802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 308),
  [2804] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 308),
  [2806] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 409),
  [2808] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 409),
  [2810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 313),
  [2812] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 313),
  [2814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 410),
  [2816] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 410),
  [2818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 315),
  [2820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 315),
  [2822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 411),
  [2824] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 411),
  [2826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 317),
  [2828] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 317),
  [2830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 412),
  [2832] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 412),
  [2834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 319),
  [2836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 319),
  [2838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 413),
  [2840] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 413),
  [2842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 414),
  [2844] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 414),
  [2846] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 415),
  [2848] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 415),
  [2850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 416),
  [2852] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 416),
  [2854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 417),
  [2856] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 417),
  [2858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 418),
  [2860] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 418),
  [2862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 419),
  [2864] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 419),
  [2866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 420),
  [2868] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 420),
  [2870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 8, 0, 421),
  [2872] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 8, 0, 421),
  [2874] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 221),
  [2876] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 221),
  [2878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 6, 0, 0),
  [2880] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 6, 0, 0),
  [2882] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 423),
  [2884] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 423),
  [2886] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 335),
  [2888] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 335),
  [2890] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 336),
  [2892] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 336),
  [2894] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 337),
  [2896] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 337),
  [2898] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 424),
  [2900] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 424),
  [2902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 425),
  [2904] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 425),
  [2906] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 426),
  [2908] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 426),
  [2910] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 427),
  [2912] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 427),
  [2914] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 428),
  [2916] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 428),
  [2918] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 429),
  [2920] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 429),
  [2922] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 430),
  [2924] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 430),
  [2926] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 431),
  [2928] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 431),
  [2930] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 432),
  [2932] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 432),
  [2934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 433),
  [2936] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 433),
  [2938] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 434),
  [2940] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 434),
  [2942] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 435),
  [2944] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 435),
  [2946] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 436),
  [2948] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 436),
  [2950] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 342),
  [2952] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 342),
  [2954] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 437),
  [2956] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 437),
  [2958] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 344),
  [2960] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 344),
  [2962] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 438),
  [2964] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 438),
  [2966] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 8, 0, 439),
  [2968] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 8, 0, 439),
  [2970] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 222),
  [2972] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 222),
  [2974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 8, 0, 257),
  [2976] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 8, 0, 257),
  [2978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 8, 0, 187),
  [2980] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 8, 0, 187),
  [2982] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 440),
  [2984] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 440),
  [2986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 441),
  [2988] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 441),
  [2990] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 442),
  [2992] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 442),
  [2994] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 443),
  [2996] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 443),
  [2998] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 8, 0, 444),
  [3000] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 8, 0, 444),
  [3002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 445),
  [3004] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 445),
  [3006] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 446),
  [3008] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 446),
  [3010] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 351),
  [3012] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 351),
  [3014] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 447),
  [3016] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 447),
  [3018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 353),
  [3020] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 353),
  [3022] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 448),
  [3024] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 448),
  [3026] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 355),
  [3028] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 355),
  [3030] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 449),
  [3032] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 449),
  [3034] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 450),
  [3036] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 450),
  [3038] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 451),
  [3040] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 451),
  [3042] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 452),
  [3044] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 452),
  [3046] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 453),
  [3048] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 453),
  [3050] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 454),
  [3052] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 454),
  [3054] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 455),
  [3056] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 455),
  [3058] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 456),
  [3060] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 456),
  [3062] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 457),
  [3064] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 457),
  [3066] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 358),
  [3068] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 358),
  [3070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 458),
  [3072] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 458),
  [3074] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 360),
  [3076] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 360),
  [3078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 459),
  [3080] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 459),
  [3082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 460),
  [3084] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 460),
  [3086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 461),
  [3088] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 461),
  [3090] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 8, 0, 462),
  [3092] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 8, 0, 462),
  [3094] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 8, 0, 463),
  [3096] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 8, 0, 463),
  [3098] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 223),
  [3100] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 223),
  [3102] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 8, 0, 444),
  [3104] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 8, 0, 444),
  [3106] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 464),
  [3108] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 464),
  [3110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 465),
  [3112] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 465),
  [3114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 466),
  [3116] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 466),
  [3118] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 371),
  [3120] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 371),
  [3122] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 372),
  [3124] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 372),
  [3126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 467),
  [3128] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 467),
  [3130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 373),
  [3132] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 373),
  [3134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 468),
  [3136] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 468),
  [3138] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 469),
  [3140] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 469),
  [3142] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 277),
  [3144] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 277),
  [3146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 470),
  [3148] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 470),
  [3150] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 471),
  [3152] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 471),
  [3154] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 472),
  [3156] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 472),
  [3158] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 473),
  [3160] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 473),
  [3162] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 474),
  [3164] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 474),
  [3166] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 475),
  [3168] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 475),
  [3170] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 476),
  [3172] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 476),
  [3174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 8, 0, 477),
  [3176] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 8, 0, 477),
  [3178] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 379),
  [3180] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 379),
  [3182] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 478),
  [3184] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 478),
  [3186] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 479),
  [3188] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 479),
  [3190] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 480),
  [3192] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 480),
  [3194] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 8, 0, 260),
  [3196] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 8, 0, 260),
  [3198] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 382),
  [3200] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 382),
  [3202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 481),
  [3204] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 481),
  [3206] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 482),
  [3208] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 482),
  [3210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 286),
  [3212] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 286),
  [3214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 483),
  [3216] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 483),
  [3218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 285),
  [3220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 285),
  [3222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 383),
  [3224] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 383),
  [3226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 484),
  [3228] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 484),
  [3230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 485),
  [3232] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 485),
  [3234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 486),
  [3236] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 486),
  [3238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 8, 0, 487),
  [3240] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 8, 0, 487),
  [3242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 488),
  [3244] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 488),
  [3246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 489),
  [3248] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 489),
  [3250] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 385),
  [3252] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 385),
  [3254] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 490),
  [3256] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 490),
  [3258] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 491),
  [3260] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 491),
  [3262] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 492),
  [3264] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 492),
  [3266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 388),
  [3268] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 388),
  [3270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 493),
  [3272] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 493),
  [3274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 8, 0, 421),
  [3276] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 8, 0, 421),
  [3278] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 390),
  [3280] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 390),
  [3282] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 391),
  [3284] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 391),
  [3286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 494),
  [3288] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 494),
  [3290] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 392),
  [3292] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 392),
  [3294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 495),
  [3296] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 495),
  [3298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 496),
  [3300] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 496),
  [3302] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 8, 0, 292),
  [3304] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 8, 0, 292),
  [3306] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 497),
  [3308] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 497),
  [3310] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 498),
  [3312] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 498),
  [3314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 499),
  [3316] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 499),
  [3318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 8, 0, 500),
  [3320] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 8, 0, 500),
  [3322] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 397),
  [3324] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 397),
  [3326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 501),
  [3328] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 501),
  [3330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 8, 0, 502),
  [3332] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 8, 0, 502),
  [3334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 8, 0, 503),
  [3336] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 8, 0, 503),
  [3338] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 224),
  [3340] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 224),
  [3342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 1, 0, 0),
  [3344] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 1, 0, 0),
  [3346] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 504),
  [3348] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 504),
  [3350] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 414),
  [3352] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 414),
  [3354] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 506),
  [3356] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 506),
  [3358] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 416),
  [3360] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 416),
  [3362] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 507),
  [3364] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 507),
  [3366] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 9, 0, 508),
  [3368] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 9, 0, 508),
  [3370] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 9, 0, 509),
  [3372] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 9, 0, 509),
  [3374] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 424),
  [3376] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 424),
  [3378] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 510),
  [3380] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 510),
  [3382] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 511),
  [3384] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 511),
  [3386] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 512),
  [3388] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 512),
  [3390] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 429),
  [3392] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 429),
  [3394] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 513),
  [3396] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 513),
  [3398] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 431),
  [3400] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 431),
  [3402] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 514),
  [3404] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 514),
  [3406] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 433),
  [3408] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 433),
  [3410] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 515),
  [3412] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 515),
  [3414] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 435),
  [3416] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 435),
  [3418] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 516),
  [3420] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 516),
  [3422] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 517),
  [3424] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 517),
  [3426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 518),
  [3428] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 518),
  [3430] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 519),
  [3432] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 519),
  [3434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 520),
  [3436] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 520),
  [3438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 9, 0, 257),
  [3440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 9, 0, 257),
  [3442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 9, 0, 187),
  [3444] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 9, 0, 187),
  [3446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 521),
  [3448] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 521),
  [3450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 522),
  [3452] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 522),
  [3454] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 523),
  [3456] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 523),
  [3458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 445),
  [3460] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 445),
  [3462] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 524),
  [3464] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 524),
  [3466] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 450),
  [3468] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 450),
  [3470] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 525),
  [3472] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 525),
  [3474] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 452),
  [3476] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 452),
  [3478] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 526),
  [3480] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 526),
  [3482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 454),
  [3484] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 454),
  [3486] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 527),
  [3488] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 527),
  [3490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 456),
  [3492] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 456),
  [3494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 528),
  [3496] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 528),
  [3498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 529),
  [3500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 529),
  [3502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 530),
  [3504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 530),
  [3506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 531),
  [3508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 531),
  [3510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 532),
  [3512] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 532),
  [3514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 9, 0, 533),
  [3516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 9, 0, 533),
  [3518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 9, 0, 534),
  [3520] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 9, 0, 534),
  [3522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 9, 0, 535),
  [3524] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 9, 0, 535),
  [3526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 9, 0, 536),
  [3528] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 9, 0, 536),
  [3530] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 537),
  [3532] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 537),
  [3534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 467),
  [3536] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 467),
  [3538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 468),
  [3540] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 468),
  [3542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 469),
  [3544] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 469),
  [3546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 538),
  [3548] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 538),
  [3550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 539),
  [3552] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 539),
  [3554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 540),
  [3556] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 540),
  [3558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 541),
  [3560] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 541),
  [3562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 542),
  [3564] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 542),
  [3566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 543),
  [3568] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 543),
  [3570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 544),
  [3572] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 544),
  [3574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 545),
  [3576] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 545),
  [3578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 546),
  [3580] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 546),
  [3582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 547),
  [3584] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 547),
  [3586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 548),
  [3588] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 548),
  [3590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 549),
  [3592] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 549),
  [3594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 550),
  [3596] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 550),
  [3598] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 474),
  [3600] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 474),
  [3602] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 551),
  [3604] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 551),
  [3606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 476),
  [3608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 476),
  [3610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 9, 0, 552),
  [3612] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 9, 0, 552),
  [3614] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 9, 0, 553),
  [3616] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 9, 0, 553),
  [3618] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_statement, 1, 0, 0),
  [3620] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_statement, 1, 0, 0),
  [3622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 554),
  [3624] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 554),
  [3626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 555),
  [3628] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 555),
  [3630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 479),
  [3632] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 479),
  [3634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 556),
  [3636] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 556),
  [3638] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 9, 0, 536),
  [3640] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 9, 0, 536),
  [3642] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 481),
  [3644] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 481),
  [3646] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 482),
  [3648] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 482),
  [3650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 557),
  [3652] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 557),
  [3654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 483),
  [3656] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 483),
  [3658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 558),
  [3660] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 558),
  [3662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 559),
  [3664] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 559),
  [3666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 383),
  [3668] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 383),
  [3670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 560),
  [3672] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 560),
  [3674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 561),
  [3676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 561),
  [3678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 562),
  [3680] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 562),
  [3682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 563),
  [3684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 563),
  [3686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 488),
  [3688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 488),
  [3690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 564),
  [3692] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 564),
  [3694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 565),
  [3696] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 565),
  [3698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 566),
  [3700] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 566),
  [3702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 491),
  [3704] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 491),
  [3706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 567),
  [3708] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 567),
  [3710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 494),
  [3712] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 494),
  [3714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 495),
  [3716] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 495),
  [3718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 496),
  [3720] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 496),
  [3722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 9, 0, 568),
  [3724] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 9, 0, 568),
  [3726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 569),
  [3728] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 569),
  [3730] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 570),
  [3732] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 570),
  [3734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 571),
  [3736] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 571),
  [3738] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 9, 0, 572),
  [3740] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 9, 0, 572),
  [3742] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 9, 0, 573),
  [3744] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 9, 0, 573),
  [3746] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statement, 1, 0, 0),
  [3748] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__statement, 1, 0, 0),
  [3750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 574),
  [3752] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 574),
  [3754] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 575),
  [3756] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 575),
  [3758] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 9, 0, 502),
  [3760] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 9, 0, 502),
  [3762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 9, 0, 576),
  [3764] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 9, 0, 576),
  [3766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 10, 0, 577),
  [3768] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 10, 0, 577),
  [3770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 578),
  [3772] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 578),
  [3774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 517),
  [3776] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 517),
  [3778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 579),
  [3780] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 579),
  [3782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 519),
  [3784] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 519),
  [3786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 580),
  [3788] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 580),
  [3790] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 10, 0, 581),
  [3792] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 10, 0, 581),
  [3794] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 582),
  [3796] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 582),
  [3798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 529),
  [3800] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 529),
  [3802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 583),
  [3804] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 583),
  [3806] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 531),
  [3808] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 531),
  [3810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 584),
  [3812] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 584),
  [3814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 10, 0, 585),
  [3816] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 10, 0, 585),
  [3818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 10, 0, 586),
  [3820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 10, 0, 586),
  [3822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 10, 0, 538),
  [3824] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 10, 0, 538),
  [3826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 587),
  [3828] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 587),
  [3830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 588),
  [3832] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 588),
  [3834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 589),
  [3836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 589),
  [3838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 543),
  [3840] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 543),
  [3842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 590),
  [3844] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 590),
  [3846] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [3848] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [3850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 591),
  [3852] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 591),
  [3854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 547),
  [3856] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 547),
  [3858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 592),
  [3860] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 592),
  [3862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 549),
  [3864] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 549),
  [3866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 593),
  [3868] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 593),
  [3870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 594),
  [3872] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 594),
  [3874] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 596),
  [3876] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 596),
  [3878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 597),
  [3880] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 597),
  [3882] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 10, 0, 554),
  [3884] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 10, 0, 554),
  [3886] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 10, 0, 598),
  [3888] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 10, 0, 598),
  [3890] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 10, 0, 557),
  [3892] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 10, 0, 557),
  [3894] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 10, 0, 558),
  [3896] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 10, 0, 558),
  [3898] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 10, 0, 559),
  [3900] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 10, 0, 559),
  [3902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 10, 0, 599),
  [3904] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 10, 0, 599),
  [3906] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 600),
  [3908] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 600),
  [3910] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 601),
  [3912] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 601),
  [3914] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 602),
  [3916] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 602),
  [3918] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 603),
  [3920] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 603),
  [3922] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 10, 0, 604),
  [3924] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 10, 0, 604),
  [3926] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 153),
  [3928] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 153),
  [3930] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 10, 0, 605),
  [3932] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 10, 0, 605),
  [3934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 10, 0, 606),
  [3936] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 10, 0, 606),
  [3938] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 10, 0, 565),
  [3940] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 10, 0, 565),
  [3942] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 10, 0, 607),
  [3944] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 10, 0, 607),
  [3946] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 10, 0, 568),
  [3948] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 10, 0, 568),
  [3950] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 608),
  [3952] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 608),
  [3954] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 609),
  [3956] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 609),
  [3958] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 10, 0, 610),
  [3960] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 10, 0, 610),
  [3962] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 10, 0, 574),
  [3964] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 10, 0, 574),
  [3966] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 10, 0, 611),
  [3968] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 10, 0, 611),
  [3970] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 11, 0, 612),
  [3972] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 11, 0, 612),
  [3974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 11, 0, 613),
  [3976] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 11, 0, 613),
  [3978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 11, 0, 594),
  [3980] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 11, 0, 594),
  [3982] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 11, 0, 614),
  [3984] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 11, 0, 614),
  [3986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 11, 0, 596),
  [3988] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 11, 0, 596),
  [3990] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 11, 0, 615),
  [3992] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 11, 0, 615),
  [3994] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 11, 0, 616),
  [3996] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 11, 0, 616),
  [3998] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 11, 0, 599),
  [4000] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 11, 0, 599),
  [4002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 11, 0, 617),
  [4004] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 11, 0, 617),
  [4006] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 11, 0, 618),
  [4008] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 11, 0, 618),
  [4010] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 11, 0, 619),
  [4012] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 11, 0, 619),
  [4014] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 11, 0, 605),
  [4016] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 11, 0, 605),
  [4018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 11, 0, 620),
  [4020] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 11, 0, 620),
  [4022] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 11, 0, 621),
  [4024] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 11, 0, 621),
  [4026] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 11, 0, 622),
  [4028] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 11, 0, 622),
  [4030] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 12, 0, 623),
  [4032] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 12, 0, 623),
  [4034] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 12, 0, 624),
  [4036] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 12, 0, 624),
  [4038] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 227),
  [4040] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 227),
  [4042] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 228),
  [4044] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 228),
  [4046] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 229),
  [4048] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 229),
  [4050] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 230),
  [4052] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 230),
  [4054] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 231),
  [4056] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 231),
  [4058] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 156),
  [4060] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 156),
  [4062] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 232),
  [4064] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 232),
  [4066] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 233),
  [4068] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 233),
  [4070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1365),
  [4072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5688),
  [4074] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1373),
  [4076] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4048),
  [4078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 3, 0, 56),
  [4080] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 3, 0, 56),
  [4082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 234),
  [4084] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 234),
  [4086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 235),
  [4088] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 235),
  [4090] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 86),
  [4092] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 86),
  [4094] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 2, 0, 0),
  [4096] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 2, 0, 0),
  [4098] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 4, 0, 87),
  [4100] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 4, 0, 87),
  [4102] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 4, 0, 88),
  [4104] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 4, 0, 88),
  [4106] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 4, 0, 89),
  [4108] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 4, 0, 89),
  [4110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 4, 0, 90),
  [4112] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 4, 0, 90),
  [4114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 3, 0, 0),
  [4116] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 3, 0, 0),
  [4118] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 80),
  [4120] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 80),
  [4122] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 94),
  [4124] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 94),
  [4126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 33),
  [4128] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 33),
  [4130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 95),
  [4132] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 95),
  [4134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 4, 0, 96),
  [4136] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 4, 0, 96),
  [4138] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 3, 0, 57),
  [4140] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 3, 0, 57),
  [4142] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 3, 0, 21),
  [4144] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 3, 0, 21),
  [4146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 6, 0, 238),
  [4148] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 6, 0, 238),
  [4150] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration_list, 2, 0, 0),
  [4152] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration_list, 2, 0, 0),
  [4154] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [4156] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [4158] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 40),
  [4160] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 40),
  [4162] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 8),
  [4164] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 8),
  [4166] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 87),
  [4168] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 87),
  [4170] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 97),
  [4172] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 97),
  [4174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 4, 0, 0),
  [4176] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 4, 0, 0),
  [4178] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 88),
  [4180] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 88),
  [4182] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 88),
  [4184] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 88),
  [4186] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 243),
  [4188] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 243),
  [4190] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 87),
  [4192] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 87),
  [4194] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 98),
  [4196] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 98),
  [4198] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 88),
  [4200] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 88),
  [4202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 4, 0, 8),
  [4204] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 4, 0, 8),
  [4206] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 4, 0, 99),
  [4208] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 4, 0, 99),
  [4210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 4, 0, 97),
  [4212] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 4, 0, 97),
  [4214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 4, 0, 87),
  [4216] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 4, 0, 87),
  [4218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 4, 0, 88),
  [4220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 4, 0, 88),
  [4222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 79),
  [4224] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 79),
  [4226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 100),
  [4228] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 100),
  [4230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 4, 0, 105),
  [4232] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 4, 0, 105),
  [4234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 4, 0, 107),
  [4236] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 4, 0, 107),
  [4238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 4, 0, 108),
  [4240] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 4, 0, 108),
  [4242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 109),
  [4244] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 109),
  [4246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 4, 0, 110),
  [4248] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 4, 0, 110),
  [4250] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 244),
  [4252] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 244),
  [4254] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 245),
  [4256] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 245),
  [4258] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 246),
  [4260] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 246),
  [4262] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 164),
  [4264] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 164),
  [4266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 247),
  [4268] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 247),
  [4270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 4, 0, 111),
  [4272] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 4, 0, 111),
  [4274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 248),
  [4276] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 248),
  [4278] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 99),
  [4280] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 99),
  [4282] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 249),
  [4284] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 249),
  [4286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 97),
  [4288] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 97),
  [4290] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 165),
  [4292] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 165),
  [4294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 250),
  [4296] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 250),
  [4298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 251),
  [4300] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 251),
  [4302] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 252),
  [4304] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 252),
  [4306] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 167),
  [4308] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 167),
  [4310] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 253),
  [4312] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 253),
  [4314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 6, 0, 254),
  [4316] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 6, 0, 254),
  [4318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 6, 0, 257),
  [4320] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 6, 0, 257),
  [4322] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 4, 0, 112),
  [4324] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 4, 0, 112),
  [4326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 4, 0, 113),
  [4328] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 4, 0, 113),
  [4330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 114),
  [4332] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 114),
  [4334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 108),
  [4336] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 108),
  [4338] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 6, 0, 187),
  [4340] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 6, 0, 187),
  [4342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 4, 0, 108),
  [4344] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 4, 0, 108),
  [4346] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 4, 0, 114),
  [4348] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 4, 0, 114),
  [4350] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 4, 0, 108),
  [4352] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 4, 0, 108),
  [4354] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_declaration, 4, 0, 115),
  [4356] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_declaration, 4, 0, 115),
  [4358] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 4, 0, 12),
  [4360] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 4, 0, 12),
  [4362] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_inner_attribute_item, 6, 0, 12),
  [4364] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_inner_attribute_item, 6, 0, 12),
  [4366] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 258),
  [4368] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 258),
  [4370] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 4, 0, 116),
  [4372] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 4, 0, 116),
  [4374] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration_list, 3, 0, 0),
  [4376] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration_list, 3, 0, 0),
  [4378] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 259),
  [4380] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 259),
  [4382] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 4, 0, 117),
  [4384] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 4, 0, 117),
  [4386] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 4, 0, 86),
  [4388] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 4, 0, 86),
  [4390] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 4, 0, 105),
  [4392] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 4, 0, 105),
  [4394] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 4, 0, 118),
  [4396] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 4, 0, 118),
  [4398] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 119),
  [4400] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 119),
  [4402] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 4, 0, 86),
  [4404] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 4, 0, 86),
  [4406] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 4, 0, 119),
  [4408] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 4, 0, 119),
  [4410] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 4, 0, 86),
  [4412] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 4, 0, 86),
  [4414] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_declaration, 4, 0, 120),
  [4416] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_declaration, 4, 0, 120),
  [4418] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 4, 0, 0),
  [4420] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 4, 0, 0),
  [4422] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 260),
  [4424] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 260),
  [4426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 4, 0, 121),
  [4428] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 4, 0, 121),
  [4430] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 6, 0, 261),
  [4432] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 6, 0, 261),
  [4434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 6, 0, 262),
  [4436] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 6, 0, 262),
  [4438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 263),
  [4440] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 263),
  [4442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 264),
  [4444] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 264),
  [4446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 176),
  [4448] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 176),
  [4450] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 265),
  [4452] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 265),
  [4454] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 266),
  [4456] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 266),
  [4458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 267),
  [4460] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 267),
  [4462] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 178),
  [4464] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 178),
  [4466] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 268),
  [4468] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 268),
  [4470] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 269),
  [4472] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 269),
  [4474] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 270),
  [4476] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 270),
  [4478] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_rules_v2, 3, 0, 0),
  [4480] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_rules_v2, 3, 0, 0),
  [4482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_body_v2, 2, 0, 0),
  [4484] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_body_v2, 2, 0, 0),
  [4486] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 206),
  [4488] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 206),
  [4490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 271),
  [4492] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 271),
  [4494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 6, 0, 272),
  [4496] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 6, 0, 272),
  [4498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 5, 0, 123),
  [4500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 5, 0, 123),
  [4502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 5, 0, 105),
  [4504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 5, 0, 105),
  [4506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 6, 0, 261),
  [4508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 6, 0, 261),
  [4510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 3, 0, 40),
  [4512] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 3, 0, 40),
  [4514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 108),
  [4516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 108),
  [4518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 175),
  [4520] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 175),
  [4522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 182),
  [4524] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 182),
  [4526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 262),
  [4528] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 262),
  [4530] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 273),
  [4532] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 273),
  [4534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 274),
  [4536] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 274),
  [4538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 262),
  [4540] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 262),
  [4542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 275),
  [4544] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 275),
  [4546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 276),
  [4548] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 276),
  [4550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 114),
  [4552] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 114),
  [4554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 184),
  [4556] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 184),
  [4558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_inner_attribute_item, 5, 0, 0),
  [4560] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_inner_attribute_item, 5, 0, 0),
  [4562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 137),
  [4564] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 137),
  [4566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 138),
  [4568] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 138),
  [4570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 139),
  [4572] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 139),
  [4574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 5, 0, 140),
  [4576] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 5, 0, 140),
  [4578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 182),
  [4580] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 182),
  [4582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 277),
  [4584] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 277),
  [4586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant_list, 3, 0, 0),
  [4588] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_variant_list, 3, 0, 0),
  [4590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 6, 0, 262),
  [4592] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 6, 0, 262),
  [4594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 143),
  [4596] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 143),
  [4598] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 5, 0, 144),
  [4600] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 5, 0, 144),
  [4602] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 5, 0, 145),
  [4604] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 5, 0, 145),
  [4606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 5, 0, 89),
  [4608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 5, 0, 89),
  [4610] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 5, 0, 146),
  [4612] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 5, 0, 146),
  [4614] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 278),
  [4616] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 278),
  [4618] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 279),
  [4620] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 279),
  [4622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 153),
  [4624] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 153),
  [4626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 154),
  [4628] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 154),
  [4630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 80),
  [4632] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 80),
  [4634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 280),
  [4636] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 280),
  [4638] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 185),
  [4640] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 185),
  [4642] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 6, 0, 281),
  [4644] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 6, 0, 281),
  [4646] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 155),
  [4648] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 155),
  [4650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 156),
  [4652] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 156),
  [4654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 157),
  [4656] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 157),
  [4658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 5, 0, 125),
  [4660] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 5, 0, 125),
  [4662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 282),
  [4664] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 282),
  [4666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 5, 0, 148),
  [4668] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 5, 0, 148),
  [4670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 5, 0, 158),
  [4672] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 5, 0, 158),
  [4674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 283),
  [4676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 283),
  [4678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 6, 0, 284),
  [4680] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 6, 0, 284),
  [4682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 6, 0, 258),
  [4684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 6, 0, 258),
  [4686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 6, 0, 260),
  [4688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 6, 0, 260),
  [4690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 3, 0, 33),
  [4692] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 3, 0, 33),
  [4694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 173),
  [4696] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 173),
  [4698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 190),
  [4700] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 190),
  [4702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_static_item, 5, 0, 140),
  [4704] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_static_item, 5, 0, 140),
  [4706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 3, 0, 44),
  [4708] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 3, 0, 44),
  [4710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 258),
  [4712] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 258),
  [4714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 285),
  [4716] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 285),
  [4718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [4720] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [4722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 260),
  [4724] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 260),
  [4726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 40),
  [4728] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 40),
  [4730] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 88),
  [4732] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 88),
  [4734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 97),
  [4736] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 97),
  [4738] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 143),
  [4740] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 143),
  [4742] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 190),
  [4744] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 190),
  [4746] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 286),
  [4748] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 286),
  [4750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 161),
  [4752] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 161),
  [4754] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 162),
  [4756] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 162),
  [4758] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 285),
  [4760] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 285),
  [4762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 143),
  [4764] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 143),
  [4766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 6, 0, 258),
  [4768] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 6, 0, 258),
  [4770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 163),
  [4772] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 163),
  [4774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 164),
  [4776] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 164),
  [4778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 8),
  [4780] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 8),
  [4782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 99),
  [4784] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 99),
  [4786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 97),
  [4788] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 97),
  [4790] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 6, 0, 260),
  [4792] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 6, 0, 260),
  [4794] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 6, 0, 287),
  [4796] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 6, 0, 287),
  [4798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 165),
  [4800] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 165),
  [4802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 143),
  [4804] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 143),
  [4806] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 166),
  [4808] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 166),
  [4810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 167),
  [4812] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 167),
  [4814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 288),
  [4816] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 288),
  [4818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 168),
  [4820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 168),
  [4822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 289),
  [4824] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 289),
  [4826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 193),
  [4828] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 193),
  [4830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 6, 0, 290),
  [4832] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 6, 0, 290),
  [4834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 79),
  [4836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 79),
  [4838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 169),
  [4840] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 169),
  [4842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 3, 0, 45),
  [4844] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 3, 0, 45),
  [4846] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_item, 6, 0, 238),
  [4848] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_item, 6, 0, 238),
  [4850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_empty_statement, 2, 0, 12),
  [4852] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_empty_statement, 2, 0, 12),
  [4854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 5, 0, 172),
  [4856] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 5, 0, 172),
  [4858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 173),
  [4860] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 173),
  [4862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 174),
  [4864] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 174),
  [4866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 175),
  [4868] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 175),
  [4870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 5, 0, 176),
  [4872] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 5, 0, 176),
  [4874] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 5, 0, 177),
  [4876] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 5, 0, 177),
  [4878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 178),
  [4880] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 178),
  [4882] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 179),
  [4884] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 179),
  [4886] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 109),
  [4888] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 109),
  [4890] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 180),
  [4892] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 180),
  [4894] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_declaration, 5, 0, 181),
  [4896] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_let_declaration, 5, 0, 181),
  [4898] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 6, 0, 215),
  [4900] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 6, 0, 215),
  [4902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 108),
  [4904] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 108),
  [4906] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 114),
  [4908] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 114),
  [4910] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 174),
  [4912] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 174),
  [4914] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 182),
  [4916] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 182),
  [4918] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 175),
  [4920] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 175),
  [4922] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 174),
  [4924] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 174),
  [4926] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 183),
  [4928] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 183),
  [4930] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 5, 0, 175),
  [4932] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 5, 0, 175),
  [4934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 114),
  [4936] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 114),
  [4938] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 184),
  [4940] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 184),
  [4942] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 182),
  [4944] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 182),
  [4946] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 174),
  [4948] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 174),
  [4950] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 175),
  [4952] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 175),
  [4954] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 185),
  [4956] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 185),
  [4958] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 5, 0, 186),
  [4960] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 5, 0, 186),
  [4962] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 5, 0, 187),
  [4964] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 5, 0, 187),
  [4966] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 5, 0, 188),
  [4968] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 5, 0, 188),
  [4970] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 173),
  [4972] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 173),
  [4974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 5, 0, 187),
  [4976] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 5, 0, 187),
  [4978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 5, 0, 189),
  [4980] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 5, 0, 189),
  [4982] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 190),
  [4984] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 190),
  [4986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 173),
  [4988] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 173),
  [4990] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 190),
  [4992] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 190),
  [4994] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 173),
  [4996] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 173),
  [4998] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_declaration, 5, 0, 191),
  [5000] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_declaration, 5, 0, 191),
  [5002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 5, 0, 12),
  [5004] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 5, 0, 12),
  [5006] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_foreign_mod_item, 5, 0, 192),
  [5008] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_foreign_mod_item, 5, 0, 192),
  [5010] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 5, 0, 193),
  [5012] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 5, 0, 193),
  [5014] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_item, 5, 0, 194),
  [5016] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_item, 5, 0, 194),
  [5018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 86),
  [5020] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 86),
  [5022] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition_v2, 5, 0, 195),
  [5024] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition_v2, 5, 0, 195),
  [5026] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 137),
  [5028] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 137),
  [5030] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_item, 5, 0, 139),
  [5032] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_item, 5, 0, 139),
  [5034] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 139),
  [5036] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 139),
  [5038] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 86),
  [5040] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 86),
  [5042] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 119),
  [5044] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 119),
  [5046] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 137),
  [5048] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 137),
  [5050] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 196),
  [5052] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 196),
  [5054] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 5, 0, 139),
  [5056] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 5, 0, 139),
  [5058] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 119),
  [5060] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 119),
  [5062] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 197),
  [5064] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 197),
  [5066] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 5, 0, 196),
  [5068] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 5, 0, 196),
  [5070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 137),
  [5072] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 137),
  [5074] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 5, 0, 139),
  [5076] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 5, 0, 139),
  [5078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_crate_declaration, 5, 0, 198),
  [5080] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_extern_crate_declaration, 5, 0, 198),
  [5082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 3, 0, 7),
  [5084] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 3, 0, 7),
  [5086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mod_item, 3, 0, 47),
  [5088] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_mod_item, 3, 0, 47),
  [5090] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 3, 0, 8),
  [5092] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 3, 0, 8),
  [5094] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 196),
  [5096] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 196),
  [5098] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 6, 0, 215),
  [5100] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 6, 0, 215),
  [5102] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_item, 3, 0, 40),
  [5104] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_item, 3, 0, 40),
  [5106] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 1, 0, 0),
  [5108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1103),
  [5110] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1483),
  [5112] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_bounds, 1, 0, 0),
  [5114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [5116] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 3, 0, 40),
  [5118] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 3, 0, 40),
  [5120] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_rules_v2, 4, 0, 0),
  [5122] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_rules_v2, 4, 0, 0),
  [5124] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 3, 0, 8),
  [5126] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 3, 0, 8),
  [5128] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 3, 0, 40),
  [5130] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 3, 0, 40),
  [5132] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_declaration, 3, 0, 49),
  [5134] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_declaration, 3, 0, 49),
  [5136] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 291),
  [5138] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 291),
  [5140] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 119),
  [5142] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 119),
  [5144] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 197),
  [5146] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 197),
  [5148] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 196),
  [5150] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 196),
  [5152] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_body_v2, 3, 0, 0),
  [5154] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_body_v2, 3, 0, 0),
  [5156] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_item, 6, 0, 292),
  [5158] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_item, 6, 0, 292),
  [5160] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 6, 0, 123),
  [5162] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 6, 0, 123),
  [5164] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_definition, 6, 0, 105),
  [5166] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_definition, 6, 0, 105),
  [5168] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_item, 6, 0, 215),
  [5170] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_item, 6, 0, 215),
  [5172] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_item, 6, 0, 293),
  [5174] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_trait_item, 6, 0, 293),
  [5176] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_signature_item, 6, 0, 294),
  [5178] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_signature_item, 6, 0, 294),
  [5180] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_impl_item, 10, 0, 595),
  [5182] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_impl_item, 10, 0, 595),
  [5184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6030),
  [5186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5260),
  [5188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4930),
  [5190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6381),
  [5192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4953),
  [5194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6393),
  [5196] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1393),
  [5198] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4169),
  [5200] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_frontmatter, 3, 0, 0),
  [5202] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_frontmatter, 3, 0, 0),
  [5204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3765),
  [5206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6250),
  [5208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6476),
  [5210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3758),
  [5212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6255),
  [5214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3783),
  [5216] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attribute_item, 4, 0, 0),
  [5218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute_item, 4, 0, 0),
  [5220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_outer_doc_comment, 2, 0, 0),
  [5222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_outer_doc_comment, 2, 0, 0),
  [5224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5804),
  [5226] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4069),
  [5228] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_attributes_repeat1, 1, 0, 0),
  [5230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 1, 0, 0),
  [5232] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_line_outer_doc_comment, 2, 0, 3),
  [5234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_outer_doc_comment, 2, 0, 3),
  [5236] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_outer_doc_comment, 3, 0, 20),
  [5238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_outer_doc_comment, 3, 0, 20),
  [5240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3785),
  [5242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5096),
  [5244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4888),
  [5246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5249),
  [5248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5250),
  [5250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3799),
  [5252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5252),
  [5254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3797),
  [5256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5183),
  [5258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5650),
  [5260] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4043),
  [5262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3781),
  [5264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3754),
  [5266] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1408),
  [5268] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4210),
  [5270] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1413),
  [5272] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1402),
  [5274] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1422),
  [5276] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_trait_bounds, 1, 0, 0), SHIFT(6014),
  [5279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1395),
  [5281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4414),
  [5283] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1392),
  [5285] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4430),
  [5287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1420),
  [5289] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4277),
  [5291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1655),
  [5293] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1661),
  [5295] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2771),
  [5297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1467),
  [5299] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4535),
  [5301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2898),
  [5303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1724),
  [5305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1576),
  [5307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5676),
  [5309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1567),
  [5311] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2794),
  [5313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1459),
  [5315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5575),
  [5317] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3698),
  [5319] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5678),
  [5321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5684),
  [5323] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1568),
  [5325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3866),
  [5327] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1970),
  [5329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1577),
  [5331] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6281),
  [5333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3710),
  [5335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1523),
  [5337] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1697),
  [5339] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1837),
  [5341] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4963),
  [5343] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3031),
  [5345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1454),
  [5347] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4550),
  [5349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3346),
  [5351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1919),
  [5353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1599),
  [5355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5471),
  [5357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1589),
  [5359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3087),
  [5361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1469),
  [5363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5638),
  [5365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3697),
  [5367] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5473),
  [5369] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5474),
  [5371] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1590),
  [5373] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3885),
  [5375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1972),
  [5377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1602),
  [5379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6295),
  [5381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1708),
  [5383] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3753),
  [5385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3000),
  [5387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(540),
  [5389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2830),
  [5391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2977),
  [5393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3640),
  [5395] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3743),
  [5397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3010),
  [5399] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2850),
  [5401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5847),
  [5403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3751),
  [5405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3016),
  [5407] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2857),
  [5409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3279),
  [5411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3639),
  [5413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3373),
  [5415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1105),
  [5417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1495),
  [5419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1496),
  [5421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3321),
  [5423] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1574),
  [5425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3741),
  [5427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2989),
  [5429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2838),
  [5431] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1651),
  [5433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2919),
  [5435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1497),
  [5437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5865),
  [5439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3641),
  [5441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2956),
  [5443] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1689),
  [5445] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1597),
  [5447] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3705),
  [5449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2990),
  [5451] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2842),
  [5453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5904),
  [5455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5832),
  [5457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3746),
  [5459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3012),
  [5461] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2852),
  [5463] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3703),
  [5465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3003),
  [5467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2806),
  [5469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3747),
  [5471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3013),
  [5473] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2853),
  [5475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1251),
  [5477] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1631),
  [5479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [5481] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1600),
  [5483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3721),
  [5485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2988),
  [5487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2808),
  [5489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3739),
  [5491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3004),
  [5493] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2824),
  [5495] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3738),
  [5497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3002),
  [5499] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2814),
  [5501] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2766),
  [5503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1457),
  [5505] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4798),
  [5507] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5076),
  [5509] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3694),
  [5511] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5772),
  [5513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6072),
  [5515] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6253),
  [5517] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2769),
  [5519] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3740),
  [5521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2992),
  [5523] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2831),
  [5525] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3035),
  [5527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1455),
  [5529] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4800),
  [5531] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5316),
  [5533] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3696),
  [5535] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5498),
  [5537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6160),
  [5539] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6227),
  [5541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(777),
  [5543] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3032),
  [5545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1104),
  [5547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3707),
  [5549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3735),
  [5551] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5783),
  [5553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3520),
  [5555] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5878),
  [5557] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3745),
  [5559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3011),
  [5561] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2851),
  [5563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3749),
  [5565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3014),
  [5567] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2855),
  [5569] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3750),
  [5571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3015),
  [5573] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2856),
  [5575] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3513),
  [5577] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3752),
  [5579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3017),
  [5581] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2858),
  [5583] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5309),
  [5585] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6032),
  [5588] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(5608),
  [5591] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6051),
  [5594] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_tuple_expression_repeat1, 2, 0, 0),
  [5596] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4068),
  [5598] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_clause, 3, 0, 0),
  [5600] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4884),
  [5602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5834),
  [5604] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4894),
  [5606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1468),
  [5608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5467),
  [5610] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3736),
  [5612] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5922),
  [5614] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5835),
  [5616] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_where_clause, 3, 0, 0),
  [5618] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1975),
  [5620] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6220),
  [5622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_clause, 1, 0, 0),
  [5624] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_where_clause, 1, 0, 0),
  [5626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_clause, 4, 0, 0),
  [5628] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_where_clause, 4, 0, 0),
  [5630] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3990),
  [5632] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0),
  [5634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0),
  [5636] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(5952),
  [5639] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(5537),
  [5642] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 2, 0, 0), SHIFT_REPEAT(6111),
  [5645] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lifetime, 2, 0, 0),
  [5647] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lifetime, 2, 0, 0),
  [5649] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6430),
  [5652] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(5570),
  [5655] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6419),
  [5658] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6426),
  [5661] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(5537),
  [5664] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6111),
  [5667] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3727),
  [5669] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_remaining_field_pattern, 1, 0, 0),
  [5671] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3702),
  [5673] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_remaining_field_pattern, 1, 0, 0),
  [5675] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3728),
  [5677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2423),
  [5679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5084),
  [5681] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5934),
  [5683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6452),
  [5685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5416),
  [5687] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3889),
  [5689] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6304),
  [5691] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5345),
  [5693] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1475),
  [5695] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1391),
  [5697] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6313),
  [5699] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5418),
  [5701] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6317),
  [5703] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6318),
  [5705] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6319),
  [5707] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5422),
  [5709] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3700),
  [5711] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2998),
  [5713] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3961),
  [5715] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3726),
  [5717] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3723),
  [5719] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 2, 0, 30),
  [5721] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3722),
  [5723] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 2, 0, 30),
  [5725] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3724),
  [5727] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3715),
  [5729] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 2, 0, 26),
  [5731] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3714),
  [5733] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 2, 0, 26),
  [5735] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3716),
  [5737] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6301),
  [5740] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(5459),
  [5743] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6049),
  [5746] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6432),
  [5749] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(5595),
  [5752] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributes_repeat1, 2, 0, 0), SHIFT_REPEAT(6002),
  [5755] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3787),
  [5757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1518),
  [5759] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2780),
  [5761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1654),
  [5763] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4899),
  [5765] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5683),
  [5767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5625),
  [5769] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3734),
  [5771] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6291),
  [5773] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3050),
  [5775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1692),
  [5777] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4985),
  [5779] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5843),
  [5781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5641),
  [5783] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3737),
  [5785] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6296),
  [5787] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3535),
  [5789] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4252),
  [5791] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3861),
  [5793] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 4, 0, 236),
  [5795] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 4, 0, 236),
  [5797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 4, 0, 236),
  [5799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_block_repeat1, 1, 0, 0),
  [5801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_block_repeat1, 1, 0, 0),
  [5803] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_match_arm, 5, 0, 148),
  [5805] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_arm, 5, 0, 148),
  [5807] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 5, 0, 148),
  [5809] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_match_block_repeat2, 1, 0, 0),
  [5811] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_match_block_repeat2, 1, 0, 0),
  [5813] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_declaration_list_repeat1, 1, 0, 0),
  [5815] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_list_repeat1, 1, 0, 0),
  [5817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression_except_range, 1, 0, 1),
  [5819] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4446),
  [5821] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression_except_range, 1, 0, 1),
  [5823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5524),
  [5825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6119),
  [5827] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__item_modifiers, 1, 0, 0),
  [5829] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4513),
  [5831] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4682),
  [5833] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4681),
  [5835] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5753),
  [5837] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4696),
  [5839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4804),
  [5841] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4575),
  [5843] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4574),
  [5845] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4577),
  [5847] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4588),
  [5849] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4587),
  [5851] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4589),
  [5853] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 5, 0, 151),
  [5855] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 5, 0, 151),
  [5857] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 5, 0, 150),
  [5859] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 5, 0, 150),
  [5861] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 6, 0, 226),
  [5863] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 6, 0, 226),
  [5865] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 3, 0, 0),
  [5867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 3, 0, 0),
  [5869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 4, 0, 150),
  [5871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 4, 0, 150),
  [5873] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 4, 0, 0),
  [5875] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 4, 0, 0),
  [5877] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 5, 0, 226),
  [5879] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 5, 0, 226),
  [5881] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 2, 0, 0),
  [5883] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 2, 0, 0),
  [5885] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_parameters, 4, 0, 151),
  [5887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameters, 4, 0, 151),
  [5889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5057),
  [5891] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 2, 0, 35),
  [5893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(139),
  [5895] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4560),
  [5897] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 2, 0, 35),
  [5899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4643),
  [5901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(508),
  [5903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5205),
  [5905] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 4, 0, 130),
  [5907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 4, 0, 130),
  [5909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 1, 0, 6),
  [5911] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4667),
  [5913] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type, 1, 0, 6),
  [5915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4619),
  [5917] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4539),
  [5919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4541),
  [5921] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 4, 0, 0),
  [5923] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 4, 0, 0),
  [5925] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 1, 0, 0),
  [5927] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 1, 0, 0),
  [5929] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_visibility_modifier, 1, 0, 0), SHIFT(5756),
  [5932] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dynamic_type, 2, 0, 35),
  [5934] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dynamic_type, 2, 0, 35),
  [5936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4652),
  [5938] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_visibility_modifier, 1, 0, 0), SHIFT(4675),
  [5941] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 5, 0, 0),
  [5943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 5, 0, 0),
  [5945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_visibility_modifier, 5, 0, 159),
  [5947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_visibility_modifier, 5, 0, 159),
  [5949] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 1, 0, 0),
  [5951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type, 1, 0, 0),
  [5953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 4, 0, 131),
  [5955] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 4, 0, 131),
  [5957] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 3, 0, 24),
  [5959] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 3, 0, 23),
  [5961] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 3, 0, 24),
  [5963] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 23),
  [5965] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 3, 0, 53),
  [5967] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 3, 0, 52),
  [5969] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 3, 0, 53),
  [5971] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 52),
  [5973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_type, 2, 0, 36),
  [5975] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_type, 2, 0, 36),
  [5977] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 2, 0, 8),
  [5979] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 2, 0, 7),
  [5981] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 2, 0, 8),
  [5983] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 2, 0, 7),
  [5985] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier, 3, 0, 59),
  [5987] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_identifier, 3, 0, 58),
  [5989] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scoped_type_identifier, 3, 0, 59),
  [5991] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 58),
  [5993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dynamic_type, 2, 0, 36),
  [5995] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dynamic_type, 2, 0, 36),
  [5997] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type, 1, 0, 5),
  [5999] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type, 1, 0, 5),
  [6001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5490),
  [6003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3574),
  [6005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4711),
  [6007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(164),
  [6009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6042),
  [6011] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 3, 0, 0),
  [6013] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 3, 0, 0),
  [6015] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 3, 0, 0),
  [6017] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 3, 0, 0),
  [6019] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4540),
  [6021] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5037),
  [6023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3109),
  [6025] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5775),
  [6027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5544),
  [6029] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__item_modifiers, 1, 0, 0),
  [6031] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6267),
  [6033] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4734),
  [6035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3104),
  [6037] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 0, 32),
  [6039] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 0, 32),
  [6041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5586),
  [6043] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 2, 0, 37),
  [6045] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 2, 0, 37),
  [6047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1580),
  [6049] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_expression, 1, 0, 0),
  [6051] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_expression, 1, 0, 0),
  [6053] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4464),
  [6055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3125),
  [6057] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_expression, 3, 0, 62),
  [6059] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_expression, 3, 0, 62),
  [6061] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression_without_attribute, 1, 0, 0),
  [6063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(113),
  [6065] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression_without_attribute, 1, 0, 0),
  [6067] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 3, 0, 59),
  [6069] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 2, 0, 0),
  [6071] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 2, 0, 0),
  [6073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5867),
  [6075] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 3, 0, 53),
  [6077] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 4, 0, 0),
  [6079] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 4, 0, 0),
  [6081] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 4, 0, 0),
  [6083] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 4, 0, 0),
  [6085] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4549),
  [6087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3107),
  [6089] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 55),
  [6091] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 55),
  [6093] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_expression, 3, 0, 63),
  [6095] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_expression, 3, 0, 63),
  [6097] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 3, 0, 81),
  [6099] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 3, 0, 81),
  [6101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1584),
  [6103] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 65),
  [6105] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 65),
  [6107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4366),
  [6109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2829),
  [6111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5323),
  [6113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6132),
  [6115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4330),
  [6117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4883),
  [6119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4365),
  [6121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4375),
  [6123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4731),
  [6125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3118),
  [6127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4570),
  [6129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3078),
  [6131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 2, 0, 34),
  [6133] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 2, 0, 34),
  [6135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1578),
  [6137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4554),
  [6139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4088),
  [6141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 3, 0, 24),
  [6143] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4542),
  [6145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3095),
  [6147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 0, 31),
  [6149] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 0, 31),
  [6151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5585),
  [6153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_type_identifier_in_expression_position, 2, 0, 8),
  [6155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4753),
  [6157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3079),
  [6159] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_type, 2, 0, 27),
  [6161] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_type, 2, 0, 27),
  [6163] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 6, 0, 0),
  [6165] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 6, 0, 0),
  [6167] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 5, 0, 0),
  [6169] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 5, 0, 0),
  [6171] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 5, 0, 0),
  [6173] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 5, 0, 0),
  [6175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4821),
  [6177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3111),
  [6179] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4830),
  [6181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3114),
  [6183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4829),
  [6185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3115),
  [6187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4831),
  [6189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3117),
  [6191] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_arguments, 2, 0, 0),
  [6193] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_arguments, 2, 0, 0),
  [6195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4836),
  [6197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3120),
  [6199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4838),
  [6201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3121),
  [6203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4837),
  [6205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3122),
  [6207] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4843),
  [6209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3124),
  [6211] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 2, 0, 38),
  [6213] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 2, 0, 38),
  [6215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1581),
  [6217] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 2, 0, 0),
  [6219] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 2, 0, 0),
  [6221] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 3, 0, 0),
  [6223] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 3, 0, 0),
  [6225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4647),
  [6227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5465),
  [6229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4608),
  [6231] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type, 3, 0, 78),
  [6233] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type, 3, 0, 78),
  [6235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_type, 3, 0, 79),
  [6237] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pointer_type, 3, 0, 79),
  [6239] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary_expression, 3, 0, 60),
  [6241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [6243] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary_expression, 3, 0, 60),
  [6245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2897),
  [6247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5137),
  [6249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1676),
  [6251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_type, 3, 0, 79),
  [6253] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_type, 3, 0, 79),
  [6255] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_expression, 2, 0, 11),
  [6257] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_expression, 2, 0, 11),
  [6259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_await_expression, 3, 0, 0),
  [6261] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_await_expression, 3, 0, 0),
  [6263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 5, 0, 0),
  [6265] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 5, 0, 0),
  [6267] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_higher_ranked_trait_bound, 3, 0, 80),
  [6269] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_higher_ranked_trait_bound, 3, 0, 80),
  [6271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_bounds, 3, 0, 0),
  [6273] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_bounds, 3, 0, 0),
  [6275] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 5, 0, 160),
  [6277] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 5, 0, 160),
  [6279] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 5, 0, 124),
  [6281] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 5, 0, 124),
  [6283] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bounded_type, 3, 0, 0),
  [6285] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_bounded_type, 3, 0, 0),
  [6287] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_cast_expression, 3, 0, 64),
  [6289] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_cast_expression, 3, 0, 64),
  [6291] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_function, 3, 0, 54),
  [6293] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_function, 3, 0, 54),
  [6295] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_with_attribute, 2, 0, 13),
  [6297] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_with_attribute, 2, 0, 13),
  [6299] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_expression, 2, 0, 0),
  [6301] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_expression, 2, 0, 0),
  [6303] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 2, 0, 0),
  [6305] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 2, 0, 0),
  [6307] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 5, 0, 0),
  [6309] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 5, 0, 0),
  [6311] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_expression, 2, 0, 0),
  [6313] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_expression, 2, 0, 0),
  [6315] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 3, 0, 39),
  [6317] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 3, 0, 39),
  [6319] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_expression, 2, 0, 15),
  [6321] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_expression, 2, 0, 15),
  [6323] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 2, 0, 16),
  [6325] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 2, 0, 16),
  [6327] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_expression, 2, 0, 4),
  [6329] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_expression, 2, 0, 4),
  [6331] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_expression, 2, 0, 0),
  [6333] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_expression, 2, 0, 0),
  [6335] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 4, 0, 0),
  [6337] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 4, 0, 0),
  [6339] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 7, 0, 0),
  [6341] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 7, 0, 0),
  [6343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5044),
  [6345] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 4, 0, 106),
  [6347] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 4, 0, 106),
  [6349] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 4, 0, 0),
  [6351] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 4, 0, 0),
  [6353] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unsafe_bound_type, 6, 0, 300),
  [6355] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unsafe_bound_type, 6, 0, 300),
  [6357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_bounds, 6, 0, 132),
  [6359] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_bounds, 6, 0, 132),
  [6361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_bounds, 6, 0, 0),
  [6363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_bounds, 6, 0, 0),
  [6365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 4, 0, 0),
  [6367] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 4, 0, 0),
  [6369] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 2, 0, 17),
  [6371] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 2, 0, 17),
  [6373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_expression, 4, 0, 68),
  [6375] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_expression, 4, 0, 68),
  [6377] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 4, 0, 0),
  [6379] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 4, 0, 0),
  [6381] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_type, 4, 0, 128),
  [6383] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_type, 4, 0, 128),
  [6385] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 7, 0, 0),
  [6387] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 7, 0, 0),
  [6389] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 4, 0, 129),
  [6391] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 4, 0, 129),
  [6393] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 6, 0, 212),
  [6395] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 6, 0, 212),
  [6397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_bounds, 4, 0, 0),
  [6399] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_bounds, 4, 0, 0),
  [6401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_bounds, 4, 0, 132),
  [6403] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_bounds, 4, 0, 132),
  [6405] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 4, 0, 133),
  [6407] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 4, 0, 133),
  [6409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 4, 0, 134),
  [6411] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 4, 0, 134),
  [6413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 2, 0, 0),
  [6415] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 2, 0, 0),
  [6417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [6419] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [6421] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 4, 0, 0),
  [6423] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 4, 0, 0),
  [6425] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 4, 0, 85),
  [6427] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 4, 0, 85),
  [6429] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1, 0, 0),
  [6431] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression, 1, 0, 0),
  [6433] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 3, 0, 0),
  [6435] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 3, 0, 0),
  [6437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4615),
  [6439] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 6, 0, 0),
  [6441] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 6, 0, 0),
  [6443] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 6, 0, 0),
  [6445] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 6, 0, 0),
  [6447] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_expression, 6, 0, 201),
  [6449] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_expression, 6, 0, 201),
  [6451] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 5, 0, 106),
  [6453] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 5, 0, 106),
  [6455] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arguments, 3, 0, 0),
  [6457] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_arguments, 3, 0, 0),
  [6459] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 5, 0, 0),
  [6461] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 5, 0, 0),
  [6463] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_index_expression, 4, 0, 0),
  [6465] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_index_expression, 4, 0, 0),
  [6467] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unit_type, 2, 0, 0),
  [6469] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unit_type, 2, 0, 0),
  [6471] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 4, 0, 122),
  [6473] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 4, 0, 122),
  [6475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_removed_trait_bound, 2, 0, 0),
  [6477] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_removed_trait_bound, 2, 0, 0),
  [6479] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_type, 2, 0, 33),
  [6481] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_type, 2, 0, 33),
  [6483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5143),
  [6485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 3, 0, 0),
  [6487] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 3, 0, 0),
  [6489] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer_list, 3, 0, 106),
  [6491] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_initializer_list, 3, 0, 106),
  [6493] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unsafe_bound_type, 7, 0, 403),
  [6495] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unsafe_bound_type, 7, 0, 403),
  [6497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unit_expression, 2, 0, 0),
  [6499] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unit_expression, 2, 0, 0),
  [6501] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 5, 0, 136),
  [6503] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 5, 0, 136),
  [6505] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_type, 5, 0, 0),
  [6507] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_type, 5, 0, 0),
  [6509] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type, 5, 0, 207),
  [6511] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type, 5, 0, 207),
  [6513] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_closure_expression, 7, 0, 326),
  [6515] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_closure_expression, 7, 0, 326),
  [6517] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unsafe_bound_type, 5, 0, 209),
  [6519] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unsafe_bound_type, 5, 0, 209),
  [6521] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_bounds, 5, 0, 132),
  [6523] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_bounds, 5, 0, 132),
  [6525] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_bounds, 5, 0, 0),
  [6527] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_use_bounds, 5, 0, 0),
  [6529] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type, 5, 0, 211),
  [6531] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type, 5, 0, 211),
  [6533] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_expression, 3, 0, 25),
  [6535] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_reference_expression, 3, 0, 25),
  [6537] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_expression, 5, 0, 0),
  [6539] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_expression, 5, 0, 0),
  [6541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5738),
  [6543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5656),
  [6545] = {.entry = {.count = 1, .reusable = false}}, SHIFT(181),
  [6547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(185),
  [6549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(197),
  [6551] = {.entry = {.count = 1, .reusable = false}}, SHIFT(200),
  [6553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(201),
  [6555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(218),
  [6557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [6559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(234),
  [6561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [6563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(247),
  [6565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [6567] = {.entry = {.count = 1, .reusable = false}}, SHIFT(255),
  [6569] = {.entry = {.count = 1, .reusable = false}}, SHIFT(49),
  [6571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(198),
  [6573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_expression, 3, 0, 61),
  [6575] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment_expression, 3, 0, 61),
  [6577] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_expression, 3, 0, 0),
  [6579] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_expression, 3, 0, 0),
  [6581] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_expression, 2, 0, 0),
  [6583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_expression, 2, 0, 0),
  [6585] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4709),
  [6587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(156),
  [6589] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4649),
  [6591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4657),
  [6593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [6595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4662),
  [6597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_become_expression, 2, 0, 0),
  [6599] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_become_expression, 2, 0, 0),
  [6601] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_expression, 3, 0, 0),
  [6603] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_expression, 3, 0, 0),
  [6605] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield_expression, 2, 0, 0),
  [6607] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_yield_expression, 2, 0, 0),
  [6609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_assignment_expr, 3, 0, 60),
  [6611] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_assignment_expr, 3, 0, 60),
  [6613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(199),
  [6615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5977),
  [6617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4716),
  [6619] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4220),
  [6621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [6623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4665),
  [6625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [6627] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4177),
  [6629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3584),
  [6631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4561),
  [6633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6298),
  [6635] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3646),
  [6637] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3667),
  [6639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4739),
  [6641] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4686),
  [6643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4563),
  [6645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4114),
  [6647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6115),
  [6649] = {.entry = {.count = 1, .reusable = false}}, SHIFT(186),
  [6651] = {.entry = {.count = 1, .reusable = false}}, SHIFT(187),
  [6653] = {.entry = {.count = 1, .reusable = false}}, SHIFT(188),
  [6655] = {.entry = {.count = 1, .reusable = false}}, SHIFT(189),
  [6657] = {.entry = {.count = 1, .reusable = false}}, SHIFT(190),
  [6659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(210),
  [6661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(192),
  [6663] = {.entry = {.count = 1, .reusable = false}}, SHIFT(193),
  [6665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [6667] = {.entry = {.count = 1, .reusable = false}}, SHIFT(195),
  [6669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(196),
  [6671] = {.entry = {.count = 1, .reusable = false}}, SHIFT(196),
  [6673] = {.entry = {.count = 1, .reusable = false}}, SHIFT(65),
  [6675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [6677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5627),
  [6679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5710),
  [6681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [6683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(131),
  [6685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(45),
  [6687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [6689] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [6691] = {.entry = {.count = 1, .reusable = true}}, SHIFT(153),
  [6693] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4733),
  [6695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [6697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [6699] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4467),
  [6701] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4466),
  [6703] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4455),
  [6705] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4834),
  [6707] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4835),
  [6709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [6711] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [6713] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4840),
  [6715] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4841),
  [6717] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4842),
  [6719] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4844),
  [6721] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4655),
  [6723] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1603),
  [6725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6065),
  [6727] = {.entry = {.count = 1, .reusable = false}}, SHIFT(70),
  [6729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(242),
  [6731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1605),
  [6733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5640),
  [6735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1606),
  [6737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(135),
  [6739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3437),
  [6741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(134),
  [6743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5639),
  [6745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [6747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5733),
  [6749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1609),
  [6751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5484),
  [6753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2953),
  [6755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [6757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(103),
  [6759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [6761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2646),
  [6763] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4744),
  [6765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2686),
  [6767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4756),
  [6769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2904),
  [6771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3415),
  [6773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1936),
  [6775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1188),
  [6777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4706),
  [6779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(289),
  [6781] = {.entry = {.count = 1, .reusable = false}}, SHIFT(223),
  [6783] = {.entry = {.count = 1, .reusable = false}}, SHIFT(224),
  [6785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3273),
  [6787] = {.entry = {.count = 1, .reusable = false}}, SHIFT(225),
  [6789] = {.entry = {.count = 1, .reusable = false}}, SHIFT(226),
  [6791] = {.entry = {.count = 1, .reusable = false}}, SHIFT(227),
  [6793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [6795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [6797] = {.entry = {.count = 1, .reusable = false}}, SHIFT(230),
  [6799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [6801] = {.entry = {.count = 1, .reusable = false}}, SHIFT(233),
  [6803] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4863),
  [6805] = {.entry = {.count = 1, .reusable = false}}, SHIFT(52),
  [6807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [6809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1571),
  [6811] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 5, 0, 407),
  [6813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2006),
  [6815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4767),
  [6817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2052),
  [6819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4774),
  [6821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 5, 0, 408),
  [6823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2131),
  [6825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4786),
  [6827] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2184),
  [6829] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4789),
  [6831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2289),
  [6833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4796),
  [6835] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_array_expression_repeat1, 2, 0, 0),
  [6837] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3992),
  [6839] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3991),
  [6841] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3993),
  [6843] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 6, 0, 505),
  [6845] = {.entry = {.count = 1, .reusable = false}}, SHIFT(205),
  [6847] = {.entry = {.count = 1, .reusable = false}}, SHIFT(206),
  [6849] = {.entry = {.count = 1, .reusable = false}}, SHIFT(207),
  [6851] = {.entry = {.count = 1, .reusable = false}}, SHIFT(208),
  [6853] = {.entry = {.count = 1, .reusable = false}}, SHIFT(209),
  [6855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(191),
  [6857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(211),
  [6859] = {.entry = {.count = 1, .reusable = false}}, SHIFT(212),
  [6861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [6863] = {.entry = {.count = 1, .reusable = false}}, SHIFT(215),
  [6865] = {.entry = {.count = 1, .reusable = false}}, SHIFT(66),
  [6867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(328),
  [6869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(513),
  [6871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [6873] = {.entry = {.count = 1, .reusable = false}}, SHIFT(214),
  [6875] = {.entry = {.count = 1, .reusable = false}}, SHIFT(69),
  [6877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [6879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [6881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4598),
  [6883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [6885] = {.entry = {.count = 1, .reusable = false}}, SHIFT(232),
  [6887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_array_expression_repeat1, 3, 0, 0),
  [6889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3719),
  [6891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [6893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1173),
  [6895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5180),
  [6897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [6899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_arguments_repeat1, 2, 0, 0),
  [6901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(444),
  [6903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3161),
  [6905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2462),
  [6907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [6909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4581),
  [6911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [6913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4499),
  [6915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2901),
  [6917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__let_chain, 3, 0, 0),
  [6919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__condition, 1, 0, 0),
  [6921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(95),
  [6923] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 4, 0, 255),
  [6925] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 4, 0, 256),
  [6927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(689),
  [6929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4458),
  [6931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(634),
  [6933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4586),
  [6935] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 3, 0, 236),
  [6937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1976),
  [6939] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 4, 0, 306),
  [6941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 4, 0, 307),
  [6943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3303),
  [6945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(91),
  [6947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [6949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(506),
  [6951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(771),
  [6953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4851),
  [6955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_last_match_arm, 4, 0, 148),
  [6957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1978),
  [6959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2936),
  [6961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [6963] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_let_condition, 4, 0, 148),
  [6965] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_base_field_initializer, 2, 0, 0),
  [6967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 3, 0, 170),
  [6969] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_initializer, 3, 0, 171),
  [6971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1232),
  [6973] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4519),
  [6975] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 3, 0, 217),
  [6977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2544),
  [6979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4729),
  [6981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2415),
  [6983] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5466),
  [6985] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5464),
  [6987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5526),
  [6989] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5470),
  [6991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2978),
  [6993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1056),
  [6995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2420),
  [6997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(107),
  [6999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(640),
  [7001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2010),
  [7003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1064),
  [7005] = {.entry = {.count = 1, .reusable = false}}, SHIFT(51),
  [7007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(252),
  [7009] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5591),
  [7011] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5590),
  [7013] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5593),
  [7015] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, 0, 135),
  [7017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1012),
  [7019] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, 0, 25),
  [7021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(967),
  [7023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2134),
  [7025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2163),
  [7027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [7029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1066),
  [7031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2955),
  [7033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2188),
  [7035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1005),
  [7037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3319),
  [7039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2231),
  [7041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2253),
  [7043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2292),
  [7045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1988),
  [7047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2949),
  [7049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [7051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2319),
  [7053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2356),
  [7055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1979),
  [7057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(939),
  [7059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [7061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(900),
  [7063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3340),
  [7065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3205),
  [7067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [7069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [7071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [7073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2363),
  [7075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3608),
  [7077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(611),
  [7079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2407),
  [7081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2878),
  [7083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2417),
  [7085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3355),
  [7087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5314),
  [7089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3934),
  [7091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5993),
  [7093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3831),
  [7095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6331),
  [7097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5239),
  [7099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6444),
  [7101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6479),
  [7103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6481),
  [7105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6482),
  [7107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3708),
  [7109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3007),
  [7111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3966),
  [7113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3823),
  [7115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5119),
  [7117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6344),
  [7119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3890),
  [7121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6428),
  [7123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5363),
  [7125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6429),
  [7127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5419),
  [7129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6436),
  [7131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6411),
  [7133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6438),
  [7135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6442),
  [7137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3748),
  [7139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3001),
  [7141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3965),
  [7143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3874),
  [7145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5090),
  [7147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6315),
  [7149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3887),
  [7151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6324),
  [7153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5347),
  [7155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6327),
  [7157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5413),
  [7159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6328),
  [7161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6276),
  [7163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6329),
  [7165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6335),
  [7167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3744),
  [7169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2999),
  [7171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3962),
  [7173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3888),
  [7175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5355),
  [7177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6396),
  [7179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3877),
  [7181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6310),
  [7183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5043),
  [7185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6321),
  [7187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6015),
  [7189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6182),
  [7191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6224),
  [7193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3704),
  [7195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2994),
  [7197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3919),
  [7199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3857),
  [7201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4760),
  [7203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 0),
  [7205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1369),
  [7207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3547),
  [7209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4560),
  [7211] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern, 1, 0, 0),
  [7213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1419),
  [7215] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1953),
  [7217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2772),
  [7219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4846),
  [7221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [7223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(151),
  [7225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4623),
  [7227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5723),
  [7229] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 1),
  [7231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4553),
  [7233] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern, 1, 0, 1),
  [7235] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1954),
  [7237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2765),
  [7239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5604),
  [7241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4270),
  [7243] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6056),
  [7245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6120),
  [7247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4318),
  [7249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2757),
  [7251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2778),
  [7253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4273),
  [7255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5616),
  [7257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4420),
  [7259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4453),
  [7261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2756),
  [7263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5475),
  [7265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4397),
  [7267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2760),
  [7269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4418),
  [7271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2755),
  [7273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5814),
  [7275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5750),
  [7277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5186),
  [7279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3759),
  [7281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6330),
  [7283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5890),
  [7285] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5154),
  [7287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5891),
  [7289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6390),
  [7291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3766),
  [7293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6470),
  [7295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5554),
  [7297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4933),
  [7299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6385),
  [7301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4957),
  [7303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6397),
  [7305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5251),
  [7307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3770),
  [7309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3812),
  [7311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5760),
  [7313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3800),
  [7315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [7317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4794),
  [7319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1474),
  [7321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3784),
  [7323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4928),
  [7325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2935),
  [7327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6203),
  [7329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(89),
  [7331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6262),
  [7333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6093),
  [7335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [7337] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type, 1, 0, 6), REDUCE(sym__pattern, 1, 0, 0),
  [7340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4617),
  [7342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(159),
  [7344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4641),
  [7346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [7348] = {.entry = {.count = 1, .reusable = false}}, SHIFT(431),
  [7350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5015),
  [7352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5756),
  [7354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3366),
  [7356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6178),
  [7358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(90),
  [7360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5026),
  [7362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3768),
  [7364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5253),
  [7366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4939),
  [7368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1460),
  [7370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4521),
  [7372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3798),
  [7374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5227),
  [7376] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4057),
  [7378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2465),
  [7380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6493),
  [7382] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6116),
  [7384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2950),
  [7386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [7388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3210),
  [7390] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6092),
  [7392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1095),
  [7394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6081),
  [7396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3187),
  [7398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5031),
  [7400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6159),
  [7402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2954),
  [7404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4110),
  [7406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2905),
  [7408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3308),
  [7410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3309),
  [7412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [7414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5991),
  [7416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2903),
  [7418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2430),
  [7420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5963),
  [7422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5007),
  [7424] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5106),
  [7426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(401),
  [7428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(402),
  [7430] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5104),
  [7432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(406),
  [7434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(645),
  [7436] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5152),
  [7438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(387),
  [7440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(393),
  [7442] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5151),
  [7444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(394),
  [7446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1662),
  [7448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1663),
  [7450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1664),
  [7452] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5098),
  [7454] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5097),
  [7456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2432),
  [7458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(776),
  [7460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2421),
  [7462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [7464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5197),
  [7466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2433),
  [7468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5288),
  [7470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2434),
  [7472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1679),
  [7474] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5213),
  [7476] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5212),
  [7478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(642),
  [7480] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_negative_literal, 2, 0, 0),
  [7482] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_negative_literal, 2, 0, 0),
  [7484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1100),
  [7486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2628),
  [7488] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__literal_pattern, 1, 0, 0),
  [7490] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__literal_pattern, 1, 0, 0),
  [7492] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6497),
  [7494] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 23), REDUCE(sym_scoped_type_identifier, 3, 0, 24),
  [7497] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 2, 0, 7), REDUCE(sym_scoped_type_identifier, 2, 0, 8),
  [7500] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_pattern, 3, 0, 75),
  [7502] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_pattern, 3, 0, 75),
  [7504] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 58), REDUCE(sym_scoped_type_identifier, 3, 0, 59),
  [7507] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6363),
  [7509] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_scoped_identifier, 3, 0, 52), REDUCE(sym_scoped_type_identifier, 3, 0, 53),
  [7512] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter, 1, 0, 42),
  [7514] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1343),
  [7516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1741),
  [7518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1362),
  [7520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3554),
  [7522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4637),
  [7524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1396),
  [7526] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2754),
  [7528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2775),
  [7530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4565),
  [7532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1473),
  [7534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4647),
  [7536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4539),
  [7538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1451),
  [7540] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 2, 0, 28),
  [7542] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 2, 0, 28),
  [7544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5818),
  [7546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1476),
  [7548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6474),
  [7550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5274),
  [7552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6117),
  [7554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1509),
  [7556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5550),
  [7558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6306),
  [7560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5351),
  [7562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6199),
  [7564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4607),
  [7566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 69),
  [7568] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 69),
  [7570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 70),
  [7572] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 70),
  [7574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4683),
  [7576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4692),
  [7578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1565),
  [7580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1545),
  [7582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 74),
  [7584] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 74),
  [7586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 3, 0, 61),
  [7588] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 3, 0, 61),
  [7590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range_pattern, 2, 0, 29),
  [7592] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range_pattern, 2, 0, 29),
  [7594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5458),
  [7596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5521),
  [7598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5536),
  [7600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1649),
  [7602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1765),
  [7604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1675),
  [7606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1671),
  [7608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1733),
  [7610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6349),
  [7612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5421),
  [7614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6404),
  [7616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1750),
  [7618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1748),
  [7620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1762),
  [7622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6372),
  [7624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5430),
  [7626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5983),
  [7628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1791),
  [7630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1797),
  [7632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1795),
  [7634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1846),
  [7636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1548),
  [7638] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 5, 0, 71),
  [7640] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 3, 0, 0),
  [7642] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_pattern, 3, 0, 0),
  [7644] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 3, 0, 71),
  [7646] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 3, 0, 72),
  [7648] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_captured_pattern, 3, 0, 0),
  [7650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_modifiers, 1, 0, 0),
  [7652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4006),
  [7654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 2, 0, 0),
  [7656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_reference_pattern, 2, 0, 0),
  [7658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 3, 0, 77),
  [7660] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 3, 0, 71),
  [7662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_or_pattern, 2, 0, 0),
  [7664] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 7, 0, 72),
  [7666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_or_pattern, 3, 0, 0),
  [7668] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 7, 0, 71),
  [7670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 4, 0, 0),
  [7672] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ref_pattern, 2, 0, 0),
  [7674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_box_pattern, 2, 0, 0),
  [7676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3548),
  [7678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 5, 0, 0),
  [7680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4635),
  [7682] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2752),
  [7684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2768),
  [7686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5584),
  [7688] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 5, 0, 0),
  [7690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 5, 0, 77),
  [7692] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 5, 0, 71),
  [7694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice_pattern, 4, 0, 0),
  [7696] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type, 1, 0, 5), REDUCE(sym__pattern, 1, 0, 1),
  [7699] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_mut_pattern, 2, 0, 0),
  [7701] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 6, 0, 71),
  [7703] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 6, 0, 72),
  [7705] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 6, 0, 77),
  [7707] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 6, 0, 71),
  [7709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4618),
  [7711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 4, 0, 71),
  [7713] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 4, 0, 72),
  [7715] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 2, 0, 0),
  [7717] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 3, 0, 0),
  [7719] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_struct_pattern, 4, 0, 77),
  [7721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 4, 0, 71),
  [7723] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_pattern, 5, 0, 72),
  [7725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(541),
  [7727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1343),
  [7729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1790),
  [7731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3527),
  [7733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1938),
  [7735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1793),
  [7737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2597),
  [7739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(552),
  [7741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3644),
  [7743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3526),
  [7745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1339),
  [7747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3606),
  [7749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1845),
  [7751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1856),
  [7753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [7755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1558),
  [7757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1884),
  [7759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5476),
  [7761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1160),
  [7763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 4, 0, 0),
  [7765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [7767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1456),
  [7769] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6278),
  [7771] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_trait_bounds_repeat1, 2, 0, 0),
  [7773] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_trait_bounds_repeat1, 2, 0, 0), SHIFT_REPEAT(1462),
  [7776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1142),
  [7778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1551),
  [7780] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 5, 0, 0),
  [7782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 2, 0, 0),
  [7784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1721),
  [7786] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(340),
  [7789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0),
  [7791] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(341),
  [7794] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(342),
  [7797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1512),
  [7799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1544),
  [7801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1524),
  [7803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1648),
  [7805] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__item_modifiers, 2, 0, 0), SHIFT_REPEAT(3934),
  [7808] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__item_modifiers, 2, 0, 0),
  [7810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1652),
  [7812] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6126),
  [7814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1665),
  [7816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1659),
  [7818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 6, 0, 0),
  [7820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1677),
  [7822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1859),
  [7824] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6309),
  [7826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1546),
  [7828] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_trait_bounds, 3, 0, 0),
  [7830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5924),
  [7832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1673),
  [7834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4566),
  [7836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4642),
  [7838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1312),
  [7840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5682),
  [7842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1734),
  [7844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2443),
  [7846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1737),
  [7848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1751),
  [7850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1729),
  [7852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1749),
  [7854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1763),
  [7856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1562),
  [7858] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6232),
  [7860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1792),
  [7862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1746),
  [7864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1798),
  [7866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2499),
  [7868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1796),
  [7870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1847),
  [7872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5842),
  [7874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5881),
  [7876] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6348),
  [7878] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6339),
  [7880] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6371),
  [7882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1767),
  [7884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2512),
  [7886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1319),
  [7888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [7890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1709),
  [7892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1940),
  [7894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [7896] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_trait_bounds_repeat1, 4, 0, 0),
  [7898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1359),
  [7900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1564),
  [7902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_trait_bounds_repeat1, 3, 0, 0),
  [7904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1162),
  [7906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1540),
  [7908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2446),
  [7910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1738),
  [7912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2468),
  [7914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [7916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1747),
  [7918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(963),
  [7920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2502),
  [7922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1768),
  [7924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2514),
  [7926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1771),
  [7928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2533),
  [7930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1776),
  [7932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2570),
  [7934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1794),
  [7936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2599),
  [7938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1808),
  [7940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2604),
  [7942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1809),
  [7944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [7946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2631),
  [7948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2675),
  [7950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1832),
  [7952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [7954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(981),
  [7956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2708),
  [7958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1848),
  [7960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2724),
  [7962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1852),
  [7964] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_modifier, 1, 0, 0),
  [7966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5974),
  [7968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2741),
  [7970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1857),
  [7972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1215),
  [7974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1594),
  [7976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1346),
  [7978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1563),
  [7980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2428),
  [7982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2037),
  [7984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2073),
  [7986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1880),
  [7988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2090),
  [7990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1885),
  [7992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2094),
  [7994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2108),
  [7996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1891),
  [7998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(995),
  [8000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2164),
  [8002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2209),
  [8004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2223),
  [8006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1905),
  [8008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2227),
  [8010] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_trait_bounds_repeat1, 5, 0, 0),
  [8012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2315),
  [8014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2333),
  [8016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2346),
  [8018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2391),
  [8020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4839),
  [8022] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 55), REDUCE(sym_generic_pattern, 3, 0, 75),
  [8025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(673),
  [8027] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_generic_type_with_turbofish, 3, 0, 65), REDUCE(sym_generic_pattern, 3, 0, 75),
  [8030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1083),
  [8032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1722),
  [8034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [8036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1522),
  [8038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6241),
  [8040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [8042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1644),
  [8044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1284),
  [8046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1668),
  [8048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(729),
  [8050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1660),
  [8052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(733),
  [8054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(868),
  [8056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1923),
  [8058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [8060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1674),
  [8062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [8064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1147),
  [8066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1528),
  [8068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1040),
  [8070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6302),
  [8072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1314),
  [8074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1707),
  [8076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(808),
  [8078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6422),
  [8080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6454),
  [8082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(890),
  [8084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1231),
  [8086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1547),
  [8088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5986),
  [8090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6326),
  [8092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1270),
  [8094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1556),
  [8096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6138),
  [8098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2245),
  [8100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(340),
  [8102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [8104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1355),
  [8106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [8108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2667),
  [8110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2668),
  [8112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4108),
  [8114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6164),
  [8116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1190),
  [8118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5987),
  [8120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1191),
  [8122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6103),
  [8124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1139),
  [8126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4117),
  [8128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1831),
  [8130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2479),
  [8132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6135),
  [8134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1144),
  [8136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4119),
  [8138] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1405),
  [8140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(382),
  [8142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(383),
  [8144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [8146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 1, 0, 1),
  [8148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [8150] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 1, 0, 0),
  [8152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(241),
  [8154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5999),
  [8156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [8158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6336),
  [8160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(666),
  [8162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1757),
  [8164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1488),
  [8166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1356),
  [8168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1516),
  [8170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2716),
  [8172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6027),
  [8174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6041),
  [8176] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 1, 0, 0),
  [8178] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1486),
  [8180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5633),
  [8182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5605),
  [8184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2521),
  [8186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2522),
  [8188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6352),
  [8190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6358),
  [8192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1637),
  [8194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5989),
  [8196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1823),
  [8198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5995),
  [8200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1504),
  [8202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1774),
  [8204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6152),
  [8206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6158),
  [8208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4094),
  [8210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1593),
  [8212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1918),
  [8214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2029),
  [8216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1670),
  [8218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2030),
  [8220] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6113),
  [8222] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4874),
  [8224] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5577),
  [8226] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6090),
  [8228] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5899),
  [8230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1295),
  [8232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1687),
  [8234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4129),
  [8236] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 2, 0, 0),
  [8238] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1690),
  [8240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5660),
  [8242] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2795),
  [8244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5177),
  [8246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5636),
  [8248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1874),
  [8250] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 2, 0, 12),
  [8252] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1695),
  [8254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2580),
  [8256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1803),
  [8258] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5978),
  [8260] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5879),
  [8262] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6350),
  [8264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1889),
  [8266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6347),
  [8268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6421),
  [8270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2612),
  [8272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1099),
  [8274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3611),
  [8276] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 1, 0, 43),
  [8278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(267),
  [8280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2622),
  [8282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1329),
  [8284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1903),
  [8286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2623),
  [8288] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 2, 0, 7),
  [8290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(283),
  [8292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1244),
  [8294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 3, 0, 105),
  [8296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(163),
  [8298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 3, 0, 12),
  [8300] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1569),
  [8302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [8304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [8306] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_parameters, 3, 0, 0), REDUCE(sym_tuple_struct_pattern, 4, 0, 71),
  [8309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1732),
  [8311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1068),
  [8313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [8315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2804),
  [8317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2266),
  [8319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2268),
  [8321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2285),
  [8323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2287),
  [8325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2302),
  [8327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2304),
  [8329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2306),
  [8331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2308),
  [8333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(82),
  [8335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2376),
  [8337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2378),
  [8339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2537),
  [8341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1221),
  [8343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2541),
  [8345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1070),
  [8347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3634),
  [8349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2562),
  [8351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [8353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4254),
  [8355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1228),
  [8357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(688),
  [8359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [8361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(913),
  [8363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2572),
  [8365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(932),
  [8367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2587),
  [8369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1806),
  [8371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2590),
  [8373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3600),
  [8375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(915),
  [8377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1714),
  [8379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [8381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3123),
  [8383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(87),
  [8385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1238),
  [8387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1405),
  [8389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1717),
  [8391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(661),
  [8393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(934),
  [8395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(663),
  [8397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2616),
  [8399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1812),
  [8401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1333),
  [8403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1716),
  [8405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1112),
  [8407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1491),
  [8409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_where_clause_repeat1, 2, 0, 0),
  [8411] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_where_clause_repeat1, 2, 0, 0), SHIFT_REPEAT(1944),
  [8414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2436),
  [8416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2633),
  [8418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2635),
  [8420] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_clause, 2, 0, 0),
  [8422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1937),
  [8424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2639),
  [8426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2641),
  [8428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(675),
  [8430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1115),
  [8432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [8434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [8436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(681),
  [8438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(683),
  [8440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(789),
  [8442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(949),
  [8444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1367),
  [8446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5634),
  [8448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2679),
  [8450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2683),
  [8452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(951),
  [8454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1179),
  [8456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1120),
  [8458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1269),
  [8460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2456),
  [8462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2704),
  [8464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1588),
  [8466] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 3, 0, 0),
  [8468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2720),
  [8470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1851),
  [8472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(953),
  [8474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(955),
  [8476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [8478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2470),
  [8480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1183),
  [8482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [8484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3540),
  [8486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2758),
  [8488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(622),
  [8490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1939),
  [8492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(626),
  [8494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(628),
  [8496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(793),
  [8498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1710),
  [8500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1994),
  [8502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1286),
  [8504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1996),
  [8506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1086),
  [8508] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_parameters, 2, 0, 0), REDUCE(sym_tuple_struct_pattern, 3, 0, 71),
  [8511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1998),
  [8513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2000),
  [8515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2025),
  [8517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2027),
  [8519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(832),
  [8521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1302),
  [8523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1696),
  [8525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5606),
  [8527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1025),
  [8529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1026),
  [8531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2486),
  [8533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1760),
  [8535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1305),
  [8537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [8539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3511),
  [8541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2039),
  [8543] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 4, 0, 12),
  [8545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2041),
  [8547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5637),
  [8549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2746),
  [8551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2047),
  [8553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(81),
  [8555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2489),
  [8557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2494),
  [8559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(848),
  [8561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1220),
  [8563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2127),
  [8565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2129),
  [8567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [8569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2144),
  [8571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2146),
  [8573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(820),
  [8575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2148),
  [8577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [8579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2150),
  [8581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2172),
  [8583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2174),
  [8585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2176),
  [8587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1256),
  [8589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1555),
  [8591] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_predicate, 2, 0, 141),
  [8593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2178),
  [8595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2203),
  [8597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_predicate, 2, 0, 142),
  [8599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(767),
  [8601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(769),
  [8603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2205),
  [8605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(624),
  [8607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1753),
  [8609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 6, 0, 422),
  [8611] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 6, 0, 329),
  [8613] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 6, 0, 330),
  [8615] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 2, 0, 0),
  [8617] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 240),
  [8619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2438),
  [8621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1735),
  [8623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1399),
  [8625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [8627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4687),
  [8629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1487),
  [8631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4983),
  [8633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6456),
  [8635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2875),
  [8637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3803),
  [8639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1390),
  [8641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1465),
  [8643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2474),
  [8645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1752),
  [8647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(297),
  [8649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4712),
  [8651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1756),
  [8653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(922),
  [8655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2496),
  [8657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1764),
  [8659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(298),
  [8661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4718),
  [8663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2805),
  [8665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(265),
  [8667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1773),
  [8669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1941),
  [8671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2535),
  [8673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1592),
  [8675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1786),
  [8677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2576),
  [8679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1799),
  [8681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(302),
  [8683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4738),
  [8685] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1802),
  [8687] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 3, 0, 0),
  [8689] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 3, 0, 33),
  [8691] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1815),
  [8693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1942),
  [8695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2629),
  [8697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(616),
  [8699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1822),
  [8701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [8703] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1830),
  [8705] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2677),
  [8707] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1841),
  [8709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [8711] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2726),
  [8713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(979),
  [8715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1217),
  [8717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [8719] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_where_clause_repeat1, 2, 0, 0), SHIFT_REPEAT(1943),
  [8722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2531),
  [8724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1503),
  [8726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1862),
  [8728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1867),
  [8730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1989),
  [8732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1873),
  [8734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(997),
  [8736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2075),
  [8738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2092),
  [8740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1888),
  [8742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2110),
  [8744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2117),
  [8746] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 7, 0, 422),
  [8748] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__use_clause, 1, 0, 1),
  [8750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4395),
  [8752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6150),
  [8754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2166),
  [8756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [8758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1897),
  [8760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2207),
  [8762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1902),
  [8764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2225),
  [8766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2229),
  [8768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1908),
  [8770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2243),
  [8772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1514),
  [8774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2275),
  [8776] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type, 1, 0, 0), REDUCE(sym__pattern, 1, 0, 0),
  [8779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2317),
  [8781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1912),
  [8783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2331),
  [8785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2335),
  [8787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1177),
  [8789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2348),
  [8791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2380),
  [8793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2393),
  [8795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2399),
  [8797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2412),
  [8799] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 4, 0, 239),
  [8801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 4, 0, 79),
  [8803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5408),
  [8805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6342),
  [8807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3635),
  [8809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(643),
  [8811] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 4, 0, 240),
  [8813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4920),
  [8815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6424),
  [8817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3414),
  [8819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3169),
  [8821] = {.entry = {.count = 1, .reusable = true}}, SHIFT(274),
  [8823] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_unit_type, 2, 0, 0), REDUCE(sym_tuple_pattern, 2, 0, 0),
  [8826] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2798),
  [8828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5155),
  [8830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__use_clause, 1, 0, 0),
  [8832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4044),
  [8834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6236),
  [8836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(548),
  [8838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1486),
  [8840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [8842] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 0), SHIFT(3566),
  [8845] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__pattern, 1, 0, 0), SHIFT(264),
  [8848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1127),
  [8850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1515),
  [8852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [8854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4772),
  [8856] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 6, 0, 328),
  [8858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4413),
  [8860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1615),
  [8862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 2, 0, 0),
  [8864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1917),
  [8866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 328),
  [8868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(870),
  [8870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4045),
  [8872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1028),
  [8874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1623),
  [8876] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 239),
  [8878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [8880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1539),
  [8882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1638),
  [8884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(874),
  [8886] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2799),
  [8888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5174),
  [8890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1042),
  [8892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1636),
  [8894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1048),
  [8896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [8898] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 329),
  [8900] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lifetime_parameter, 1, 0, 43),
  [8902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [8904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [8906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(888),
  [8908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1290),
  [8910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1681),
  [8912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(277),
  [8914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4624),
  [8916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1090),
  [8918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1921),
  [8920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [8922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4600),
  [8924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1669),
  [8926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [8928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1686),
  [8930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1273),
  [8932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1061),
  [8934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 79),
  [8936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1925),
  [8938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [8940] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 5, 0, 330),
  [8942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3566),
  [8944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [8946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1277),
  [8948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1552),
  [8950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(184),
  [8952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4785),
  [8954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [8956] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ordered_field_declaration_list, 4, 0, 33),
  [8958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1128),
  [8960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_parameter, 4, 0, 140),
  [8962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3537),
  [8964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2865),
  [8966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1463),
  [8968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1130),
  [8970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5880),
  [8972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2908),
  [8974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2909),
  [8976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1132),
  [8978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1499),
  [8980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2424),
  [8982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [8984] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3504),
  [8986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4571),
  [8988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1505),
  [8990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(344),
  [8992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4100),
  [8994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2016),
  [8996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2018),
  [8998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2019),
  [9000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(988),
  [9002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1507),
  [9004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4160),
  [9006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1480),
  [9008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1775),
  [9010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2051),
  [9012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(311),
  [9014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4773),
  [9016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2061),
  [9018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2062),
  [9020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2064),
  [9022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2066),
  [9024] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1871),
  [9026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2484),
  [9028] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1758),
  [9030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2485),
  [9032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1759),
  [9034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2083),
  [9036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2085),
  [9038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1881),
  [9040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2086),
  [9042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1882),
  [9044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2087),
  [9046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1883),
  [9048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2098),
  [9050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2099),
  [9052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2101),
  [9054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2103),
  [9056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1886),
  [9058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5094),
  [9060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5702),
  [9062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1074),
  [9064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [9066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4493),
  [9068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2505),
  [9070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_modifiers, 2, 0, 0),
  [9072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2139),
  [9074] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 4, 0, 127),
  [9076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1085),
  [9078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2194),
  [9080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2196),
  [9082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2197),
  [9084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1332),
  [9086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1715),
  [9088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(782),
  [9090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1165),
  [9092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2213),
  [9094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2214),
  [9096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2216),
  [9098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2218),
  [9100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1900),
  [9102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2517),
  [9104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1111),
  [9106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1490),
  [9108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2234),
  [9110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2236),
  [9112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2237),
  [9114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3603),
  [9116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3613),
  [9118] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3512),
  [9120] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3534),
  [9122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2297),
  [9124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [9126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2817),
  [9128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5103),
  [9130] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3859),
  [9132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2322),
  [9134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2324),
  [9136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2325),
  [9138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5099),
  [9140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1382),
  [9142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5101),
  [9144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1385),
  [9146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2340),
  [9148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5617),
  [9150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2385),
  [9152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3609),
  [9154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1466),
  [9156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1614),
  [9158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1526),
  [9160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(421),
  [9162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(26),
  [9164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4875),
  [9166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5481),
  [9168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1619),
  [9170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4381),
  [9172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [9174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(399),
  [9176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(369),
  [9178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(373),
  [9180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1331),
  [9182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1712),
  [9184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(425),
  [9186] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3068),
  [9188] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3102),
  [9190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2543),
  [9192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [9194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4728),
  [9196] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3732),
  [9198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [9200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3406),
  [9202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1452),
  [9204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2555),
  [9206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [9208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5903),
  [9210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4526),
  [9212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2557),
  [9214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1787),
  [9216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5486),
  [9218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4530),
  [9220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2558),
  [9222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1788),
  [9224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3809),
  [9226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [9228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3218),
  [9230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3227),
  [9232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2559),
  [9234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1789),
  [9236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [9238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(839),
  [9240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3293),
  [9242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3298),
  [9244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(464),
  [9246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4976),
  [9248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1230),
  [9250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(269),
  [9252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4503),
  [9254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [9256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [9258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1110),
  [9260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [9262] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2821),
  [9264] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3692),
  [9266] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2789),
  [9268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2585),
  [9270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1804),
  [9272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2586),
  [9274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1805),
  [9276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [9278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1914),
  [9280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [9282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [9284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4132),
  [9286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1187),
  [9288] = {.entry = {.count = 1, .reusable = true}}, SHIFT(319),
  [9290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4704),
  [9292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5010),
  [9294] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5578),
  [9296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(400),
  [9298] = {.entry = {.count = 1, .reusable = true}}, SHIFT(367),
  [9300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(368),
  [9302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4183),
  [9304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(655),
  [9306] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3685),
  [9308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3665),
  [9310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3231),
  [9312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(375),
  [9314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [9316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [9318] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3244),
  [9320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4874),
  [9322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5899),
  [9324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4355),
  [9326] = {.entry = {.count = 1, .reusable = true}}, SHIFT(566),
  [9328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1617),
  [9330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4297),
  [9332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [9334] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3070),
  [9336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4356),
  [9338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [9340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2602),
  [9342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(944),
  [9344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2614),
  [9346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1810),
  [9348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2615),
  [9350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1811),
  [9352] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_binding, 3, 0, 202),
  [9354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [9356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1427),
  [9358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4765),
  [9360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(78),
  [9362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3440),
  [9364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6146),
  [9366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1202),
  [9368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(16),
  [9370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1432),
  [9372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4514),
  [9374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [9376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1260),
  [9378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2803),
  [9380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1624),
  [9382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1033),
  [9384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1472),
  [9386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [9388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(166),
  [9390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4445),
  [9392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1204),
  [9394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1541),
  [9396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1262),
  [9398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1645),
  [9400] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4064),
  [9402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1210),
  [9404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1542),
  [9406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(360),
  [9408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(361),
  [9410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [9412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1263),
  [9414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1646),
  [9416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1266),
  [9418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1647),
  [9420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1211),
  [9422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1543),
  [9424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(841),
  [9426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2981),
  [9428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2982),
  [9430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(842),
  [9432] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3059),
  [9434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2449),
  [9436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(879),
  [9438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [9440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(882),
  [9442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(699),
  [9444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(700),
  [9446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(702),
  [9448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [9450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1633),
  [9452] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 5, 0, 206),
  [9454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1639),
  [9456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1641),
  [9458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2451),
  [9460] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_slice_pattern_repeat1, 2, 0, 0),
  [9462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2645),
  [9464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [9466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4743),
  [9468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2655),
  [9470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(722),
  [9472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [9474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1656),
  [9476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(725),
  [9478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1657),
  [9480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(726),
  [9482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1658),
  [9484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2656),
  [9486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3599),
  [9488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2658),
  [9490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1252),
  [9492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1553),
  [9494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2660),
  [9496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1820),
  [9498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3450),
  [9500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6204),
  [9502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1253),
  [9504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1554),
  [9506] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_slice_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(1416),
  [9509] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 3, 0, 76),
  [9511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3649),
  [9513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(737),
  [9515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [9517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [9519] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 4, 0, 125),
  [9521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(742),
  [9523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1666),
  [9525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4906),
  [9527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [9529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3605),
  [9531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(970),
  [9533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1378),
  [9535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4923),
  [9537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(972),
  [9539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1150),
  [9541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(973),
  [9543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1377),
  [9545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3756),
  [9547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2685),
  [9549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [9551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4755),
  [9553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5310),
  [9555] = {.entry = {.count = 1, .reusable = false}}, SHIFT(434),
  [9557] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5156),
  [9559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5081),
  [9561] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5311),
  [9563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(427),
  [9565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1300),
  [9567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1693),
  [9569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_rule, 3, 0, 61),
  [9571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1301),
  [9573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1694),
  [9575] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3681),
  [9577] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_binding, 4, 0, 296),
  [9579] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5142),
  [9581] = {.entry = {.count = 1, .reusable = false}}, SHIFT(447),
  [9583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3810),
  [9585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1370),
  [9587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(32),
  [9589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1434),
  [9591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4573),
  [9593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(80),
  [9595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2697),
  [9597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [9599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2699),
  [9601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1842),
  [9603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(36),
  [9605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1428),
  [9607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4622),
  [9609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(76),
  [9611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2700),
  [9613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1843),
  [9615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1351),
  [9617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1559),
  [9619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1352),
  [9621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1560),
  [9623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1354),
  [9625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1561),
  [9627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0),
  [9629] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(5103),
  [9632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2701),
  [9634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1844),
  [9636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1126),
  [9638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2718),
  [9640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1849),
  [9642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1690),
  [9644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2719),
  [9646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1850),
  [9648] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5284),
  [9650] = {.entry = {.count = 1, .reusable = false}}, SHIFT(446),
  [9652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1777),
  [9654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1779),
  [9656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0),
  [9658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1816),
  [9660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1824),
  [9662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1826),
  [9664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1833),
  [9666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1835),
  [9668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1863),
  [9670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1868),
  [9672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1875),
  [9674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1877),
  [9676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1898),
  [9678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2734),
  [9680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2736),
  [9682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1853),
  [9684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1317),
  [9686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2737),
  [9688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1854),
  [9690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2738),
  [9692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1855),
  [9694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(641),
  [9696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4449),
  [9698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(924),
  [9700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(639),
  [9702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(165),
  [9704] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3165),
  [9706] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3305),
  [9708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3167),
  [9710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1813),
  [9712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1620),
  [9714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1814),
  [9716] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 3, 0, 204),
  [9718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5168),
  [9720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2963),
  [9722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(909),
  [9724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4407),
  [9726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [9728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3658),
  [9730] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, 0, 152), SHIFT_REPEAT(3559),
  [9733] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, 0, 152),
  [9735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4778),
  [9737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3621),
  [9739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5831),
  [9741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5505),
  [9743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2519),
  [9745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4368),
  [9747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4432),
  [9749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(555),
  [9751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [9753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1570),
  [9755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3530),
  [9757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4419),
  [9759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(959),
  [9761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2637),
  [9763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_wildcard, 1, 0, 0),
  [9765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 2, 0, 33),
  [9767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(961),
  [9769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [9771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1223),
  [9773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2643),
  [9775] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_unsafe_bound_type_repeat1, 2, 0, 0), SHIFT_REPEAT(5853),
  [9778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_unsafe_bound_type_repeat1, 2, 0, 0),
  [9780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(324),
  [9782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3408),
  [9784] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 2, 0, 241),
  [9786] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 2, 0, 241), SHIFT_REPEAT(571),
  [9789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 242),
  [9791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1007),
  [9793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3657),
  [9795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2648),
  [9797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [9799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3660),
  [9801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1819),
  [9803] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0),
  [9805] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3690),
  [9808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1009),
  [9810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3563),
  [9812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(926),
  [9814] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_use_bounds_repeat1, 2, 0, 0), SHIFT_REPEAT(5326),
  [9817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_use_bounds_repeat1, 2, 0, 0),
  [9819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1530),
  [9821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 3, 0, 205),
  [9823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1821),
  [9825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [9827] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 4, 0, 331),
  [9829] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2664),
  [9831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [9833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4938),
  [9835] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1608),
  [9837] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4590),
  [9839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3283),
  [9841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4591),
  [9843] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3348),
  [9845] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_array_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(263),
  [9848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(911),
  [9850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2431),
  [9852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3656),
  [9854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
  [9856] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_shorthand_field_initializer, 1, 0, 0),
  [9858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [9860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1829),
  [9862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [9864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1381),
  [9866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(276),
  [9868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2673),
  [9870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(306),
  [9872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1611),
  [9874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4954),
  [9876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4596),
  [9878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4597),
  [9880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2681),
  [9882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3662),
  [9884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4150),
  [9886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5100),
  [9888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3560),
  [9890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(526),
  [9892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1742),
  [9894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1613),
  [9896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2688),
  [9898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [9900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2690),
  [9902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3664),
  [9904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2691),
  [9906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1839),
  [9908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1840),
  [9910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5102),
  [9912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3589),
  [9914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1181),
  [9916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1446),
  [9918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1337),
  [9920] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2, 0, 0),
  [9922] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_type_repeat1, 2, 0, 0), SHIFT_REPEAT(1818),
  [9925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(574),
  [9927] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter, 3, 0, 149),
  [9929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1498),
  [9931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1517),
  [9933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2706),
  [9935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(529),
  [9937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5561),
  [9939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2713),
  [9941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2714),
  [9943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1386),
  [9945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 4, 0, 328),
  [9947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5428),
  [9949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1583),
  [9951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1519),
  [9953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4713),
  [9955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2927),
  [9957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4714),
  [9959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2926),
  [9961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [9963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3637),
  [9965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2728),
  [9967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(309),
  [9969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2731),
  [9971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2732),
  [9973] = {.entry = {.count = 1, .reusable = true}}, SHIFT(296),
  [9975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2867),
  [9977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1706),
  [9979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1772),
  [9981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(928),
  [9983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2529),
  [9985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [9987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [9989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2530),
  [9991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3671),
  [9993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_pattern, 1, 0, 0),
  [9995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1398),
  [9997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(79),
  [9999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [10001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [10003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [10005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1932),
  [10007] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter, 4, 0, 225),
  [10009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(930),
  [10011] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(329),
  [10014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1140),
  [10016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1017),
  [10018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [10020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5589),
  [10022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1743),
  [10024] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1591),
  [10026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3581),
  [10028] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3542),
  [10030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4396),
  [10032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3585),
  [10034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1586),
  [10036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3532),
  [10038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4399),
  [10040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1858),
  [10042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1610),
  [10044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(96),
  [10046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1185),
  [10048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2539),
  [10050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1612),
  [10052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3588),
  [10054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1990),
  [10056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1992),
  [10058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1241),
  [10060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(272),
  [10062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1500),
  [10064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(599),
  [10066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5540),
  [10068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1618),
  [10070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1192),
  [10072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(321),
  [10074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1021),
  [10076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1916),
  [10078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4833),
  [10080] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 5, 0, 0),
  [10082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__condition, 1, 0, 10),
  [10084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1572),
  [10086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 2, 0, 126),
  [10088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1409),
  [10090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2002),
  [10092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1245),
  [10094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3670),
  [10096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1023),
  [10098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2004),
  [10100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2546),
  [10102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(301),
  [10104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5160),
  [10106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3663),
  [10108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2009),
  [10110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [10112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4267),
  [10114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(561),
  [10116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2548),
  [10118] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5958),
  [10120] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5725),
  [10122] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6252),
  [10124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3645),
  [10126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4465),
  [10128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(281),
  [10130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1195),
  [10132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1196),
  [10134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2549),
  [10136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1248),
  [10138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1249),
  [10140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5724),
  [10142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1621),
  [10144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1784),
  [10146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1785),
  [10148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1364),
  [10150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3557),
  [10152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5618),
  [10154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1860),
  [10156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1861),
  [10158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1622),
  [10160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(141),
  [10162] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extern_modifier, 2, 0, 0),
  [10164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2439),
  [10166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1506),
  [10168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6502),
  [10170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5856),
  [10172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6000),
  [10174] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 2, 0, 0), SHIFT_REPEAT(530),
  [10177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1508),
  [10179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4485),
  [10181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1389),
  [10183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1537),
  [10185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1865),
  [10187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1345),
  [10189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1632),
  [10191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4497),
  [10193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1538),
  [10195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1866),
  [10197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5805),
  [10199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(826),
  [10201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4009),
  [10203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4016),
  [10205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4010),
  [10207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4011),
  [10209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4017),
  [10211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4012),
  [10213] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 1, 0, 0),
  [10215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3976),
  [10217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3977),
  [10219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3978),
  [10221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1635),
  [10223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3979),
  [10225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3980),
  [10227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4002),
  [10229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [10231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [10233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2043),
  [10235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(993),
  [10237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [10239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3528),
  [10241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2761),
  [10243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2049),
  [10245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1625),
  [10247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1039),
  [10249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4552),
  [10251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1136),
  [10253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2054),
  [10255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [10257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1870),
  [10259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4639),
  [10261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(685),
  [10263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2564),
  [10265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3607),
  [10267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1209),
  [10269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [10271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1872),
  [10273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3604),
  [10275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6003),
  [10277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1525),
  [10279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2070),
  [10281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1781),
  [10283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3538),
  [10285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4376),
  [10287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2077),
  [10289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [10291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1550),
  [10293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1038),
  [10295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(174),
  [10297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2080),
  [10299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5169),
  [10301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1678),
  [10303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2081),
  [10305] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2826),
  [10307] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2872),
  [10309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2815),
  [10311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1158),
  [10313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2574),
  [10315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2472),
  [10317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(691),
  [10319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [10321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [10323] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 4, 0, 0),
  [10325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2577),
  [10327] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_use_list_repeat1, 2, 0, 0),
  [10329] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_use_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2993),
  [10332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2578),
  [10334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(905),
  [10336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1801),
  [10338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2966),
  [10340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3626),
  [10342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1887),
  [10344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4023),
  [10346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3973),
  [10348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4070),
  [10350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4025),
  [10352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3974),
  [10354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4026),
  [10356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [10358] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5047),
  [10360] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5830),
  [10362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6245),
  [10364] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_wildcard, 2, 0, 0),
  [10366] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6500),
  [10368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5980),
  [10370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1275),
  [10372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1630),
  [10374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2476),
  [10376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5463),
  [10378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2477),
  [10380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [10382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(632),
  [10384] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_use_list, 2, 0, 48),
  [10386] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_initializer_list_repeat1, 2, 0, 0),
  [10388] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_field_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3647),
  [10391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1093),
  [10393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [10395] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 3, 0, 0),
  [10397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1683),
  [10399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5211),
  [10401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2119),
  [10403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2902),
  [10405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2121),
  [10407] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_wildcard, 3, 0, 1),
  [10409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_use_list, 3, 0, 101),
  [10411] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_as_clause, 3, 0, 102),
  [10413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_wildcard, 3, 0, 0),
  [10415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2123),
  [10417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2125),
  [10419] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scoped_use_list, 3, 0, 103),
  [10421] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_as_clause, 3, 0, 104),
  [10423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [10425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3593),
  [10427] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 1, 0, 73),
  [10429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1417),
  [10431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2510),
  [10433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1640),
  [10435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1755),
  [10437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1642),
  [10439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1225),
  [10441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1892),
  [10443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5487),
  [10445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [10447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(708),
  [10449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3602),
  [10451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3806),
  [10453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3564),
  [10455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [10457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(170),
  [10459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4564),
  [10461] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_shorthand_field_initializer, 2, 0, 0),
  [10463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [10465] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6155),
  [10467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5848),
  [10469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6153),
  [10471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [10473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(168),
  [10475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2152),
  [10477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3615),
  [10479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2154),
  [10481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1684),
  [10483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3981),
  [10485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3982),
  [10487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3986),
  [10489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3987),
  [10491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3988),
  [10493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4001),
  [10495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2156),
  [10497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(314),
  [10499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1482),
  [10501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(719),
  [10503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [10505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1894),
  [10507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2595),
  [10509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1481),
  [10511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1485),
  [10513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2168),
  [10515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2170),
  [10517] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_struct_pattern_repeat1, 2, 0, 0),
  [10519] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_struct_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(3629),
  [10522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1893),
  [10524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(831),
  [10526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(171),
  [10528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1477),
  [10530] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6314),
  [10532] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5932),
  [10534] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6443),
  [10536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4879),
  [10538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1239),
  [10540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2180),
  [10542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1240),
  [10544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2182),
  [10546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_parameter, 2, 0, 92),
  [10548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1607),
  [10550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2187),
  [10552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(315),
  [10554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1326),
  [10556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4808),
  [10558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2609),
  [10560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2610),
  [10562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3339),
  [10564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3633),
  [10566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1895),
  [10568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4518),
  [10570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1896),
  [10572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [10574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_tuple_pattern_repeat1, 2, 0, 0),
  [10576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1288),
  [10578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 3, 0, 203),
  [10580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1429),
  [10582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1001),
  [10584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(150),
  [10586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2497),
  [10588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4494),
  [10590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1088),
  [10592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4359),
  [10594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [10596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1901),
  [10598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1929),
  [10600] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_tuple_pattern_repeat1, 2, 0, 0), SHIFT_REPEAT(598),
  [10603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1383),
  [10605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1327),
  [10607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2620),
  [10609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1667),
  [10611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3536),
  [10613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2759),
  [10615] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6021),
  [10617] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5449),
  [10619] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5984),
  [10621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1072),
  [10623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4520),
  [10625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1292),
  [10627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1906),
  [10629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1293),
  [10631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1907),
  [10633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3648),
  [10635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1685),
  [10637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(434),
  [10639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [10641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1003),
  [10643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2248),
  [10645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2250),
  [10647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1052),
  [10649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1513),
  [10651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2258),
  [10653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2260),
  [10655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2262),
  [10657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2264),
  [10659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(907),
  [10661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3568),
  [10663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(152),
  [10665] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_closure_parameters_repeat1, 2, 0, 0),
  [10667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5889),
  [10669] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 4, 0, 297),
  [10671] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 4, 0, 298),
  [10673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1054),
  [10675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2277),
  [10677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2279),
  [10679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2281),
  [10681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1278),
  [10683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(323),
  [10685] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_closure_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(549),
  [10688] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 4, 0, 0),
  [10690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(437),
  [10692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [10694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2283),
  [10696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4813),
  [10698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1170),
  [10700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(294),
  [10702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 3, 0, 47),
  [10704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(162),
  [10706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1909),
  [10708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1325),
  [10710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1340),
  [10712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5324),
  [10714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1701),
  [10716] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_list, 2, 0, 0),
  [10718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1703),
  [10720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5327),
  [10722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1704),
  [10724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2310),
  [10726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2312),
  [10728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2314),
  [10730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(316),
  [10732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [10734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1310),
  [10736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3811),
  [10738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3573),
  [10740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4674),
  [10742] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 5, 0, 401),
  [10744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3543),
  [10746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2763),
  [10748] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_pattern, 5, 0, 402),
  [10750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1713),
  [10752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1380),
  [10754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4689),
  [10756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1910),
  [10758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4693),
  [10760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1447),
  [10762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1911),
  [10764] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 4, 0, 118),
  [10766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(169),
  [10768] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6053),
  [10770] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5482),
  [10772] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6441),
  [10774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3590),
  [10776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1922),
  [10778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4457),
  [10780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1924),
  [10782] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_arguments_repeat1, 2, 0, 0), SHIFT_REPEAT(326),
  [10785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1387),
  [10787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3594),
  [10789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1913),
  [10791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4719),
  [10793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2345),
  [10795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(317),
  [10797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5225),
  [10799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1587),
  [10801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4720),
  [10803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1778),
  [10805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2352),
  [10807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1780),
  [10809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_variant, 2, 0, 15),
  [10811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(282),
  [10813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [10815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2354),
  [10817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2358),
  [10819] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_macro_rules_v2_repeat1, 2, 0, 0),
  [10821] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_macro_rules_v2_repeat1, 2, 0, 0), SHIFT_REPEAT(4368),
  [10824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(895),
  [10826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2360),
  [10828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3659),
  [10830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4740),
  [10832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3529),
  [10834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2785),
  [10836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1817),
  [10838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2368),
  [10840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1825),
  [10842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2370),
  [10844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1827),
  [10846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2372),
  [10848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1834),
  [10850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2374),
  [10852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1836),
  [10854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [10856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4486),
  [10858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1864),
  [10860] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0),
  [10862] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_variant_list_repeat1, 2, 0, 0), SHIFT_REPEAT(3687),
  [10865] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1869),
  [10867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1501),
  [10869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1876),
  [10871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1915),
  [10873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1878),
  [10875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2390),
  [10877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(318),
  [10879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1899),
  [10881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1212),
  [10883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3668),
  [10885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4754),
  [10887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3624),
  [10889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4768),
  [10891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2403),
  [10893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2405),
  [10895] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6334),
  [10897] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5936),
  [10899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6451),
  [10901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(268),
  [10903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6478),
  [10905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5722),
  [10907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [10909] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6343),
  [10911] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5937),
  [10913] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6458),
  [10915] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6367),
  [10917] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5947),
  [10919] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6471),
  [10921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(814),
  [10923] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6357),
  [10925] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5948),
  [10927] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6467),
  [10929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4230),
  [10931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1616),
  [10933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 3, 0, 239),
  [10935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(759),
  [10937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(761),
  [10939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1585),
  [10941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(763),
  [10943] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6391),
  [10945] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5951),
  [10947] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6483),
  [10949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1096),
  [10951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1097),
  [10953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [10955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_ordered_field_declaration_list_repeat1, 3, 0, 79),
  [10957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5019),
  [10959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4845),
  [10961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1527),
  [10963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4448),
  [10965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2190),
  [10967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2396),
  [10969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2397),
  [10971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2402),
  [10973] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6323),
  [10975] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6022),
  [10977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1106),
  [10979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2409),
  [10981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2410),
  [10983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2411),
  [10985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1122),
  [10987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2414),
  [10989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2416),
  [10991] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2788),
  [10993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1982),
  [10995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6311),
  [10997] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3075),
  [10999] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3077),
  [11001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6418),
  [11003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3296),
  [11005] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2818),
  [11007] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2796),
  [11009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2784),
  [11011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5517),
  [11013] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3061),
  [11015] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3097),
  [11017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2783),
  [11019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4583),
  [11021] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3503),
  [11023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5852),
  [11025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1596),
  [11027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4584),
  [11029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_parameter, 6, 0, 305),
  [11031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1123),
  [11033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2551),
  [11035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1051),
  [11037] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6043),
  [11039] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6100),
  [11041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1265),
  [11043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6102),
  [11045] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3071),
  [11047] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3105),
  [11049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1569),
  [11051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4224),
  [11053] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3544),
  [11055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(975),
  [11057] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3507),
  [11059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [11061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(351),
  [11063] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3867),
  [11065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1336),
  [11067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6033),
  [11069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2592),
  [11071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6050),
  [11073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1058),
  [11075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1059),
  [11077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1060),
  [11079] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_field_declaration_list_repeat1, 3, 0, 0),
  [11081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(783),
  [11083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2619),
  [11085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6060),
  [11087] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3505),
  [11089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1063),
  [11091] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3533),
  [11093] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2837),
  [11095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3655),
  [11097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6337),
  [11099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1065),
  [11101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(801),
  [11103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5204),
  [11105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6047),
  [11107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3076),
  [11109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1959),
  [11111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6141),
  [11113] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3521),
  [11115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(989),
  [11117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(990),
  [11119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3524),
  [11121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(991),
  [11123] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_rule_v2, 3, 0, 61),
  [11125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1372),
  [11127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6395),
  [11129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 1, 0, 0),
  [11131] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3674),
  [11133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(647),
  [11135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3508),
  [11137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2624),
  [11139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [11141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [11143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6187),
  [11145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(602),
  [11147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1958),
  [11149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6453),
  [11151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6190),
  [11153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(603),
  [11155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2791),
  [11157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2650),
  [11159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2651),
  [11161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2652),
  [11163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3731),
  [11165] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3858),
  [11167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2792),
  [11169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3541),
  [11171] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2841),
  [11173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3525),
  [11175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3523),
  [11177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(657),
  [11179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2750),
  [11181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6007),
  [11183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6380),
  [11185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3686),
  [11187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(902),
  [11189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(903),
  [11191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [11193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(904),
  [11195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [11197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [11199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3693),
  [11201] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3684),
  [11203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1000),
  [11205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1933),
  [11207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6147),
  [11209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4613),
  [11211] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3730),
  [11213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3827),
  [11215] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3069),
  [11217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [11219] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3683),
  [11221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3733),
  [11223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 2, 0, 0),
  [11225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2693),
  [11227] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3675),
  [11229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3679),
  [11231] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3676),
  [11233] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3689),
  [11235] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3067),
  [11237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3064),
  [11239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(836),
  [11241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(667),
  [11243] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 2, 0, 0),
  [11245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(784),
  [11247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(803),
  [11249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3060),
  [11251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1650),
  [11253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2723),
  [11255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6123),
  [11257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(804),
  [11259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4667),
  [11261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4620),
  [11263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1254),
  [11265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4125),
  [11267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [11269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1573),
  [11271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2743),
  [11273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2744),
  [11275] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_parameters_repeat1, 3, 0, 150),
  [11277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 2, 0, 12),
  [11279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(919),
  [11281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(125),
  [11283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2035),
  [11285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2045),
  [11287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2418),
  [11289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(920),
  [11291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [11293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6198),
  [11295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5839),
  [11297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1450),
  [11299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [11301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2011),
  [11303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2012),
  [11305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2013),
  [11307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(243),
  [11309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [11311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(695),
  [11313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2021),
  [11315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(696),
  [11317] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3065),
  [11319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2031),
  [11321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [11323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1014),
  [11325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1494),
  [11327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1695),
  [11329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1015),
  [11331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1016),
  [11333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1117),
  [11335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6034),
  [11337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2056),
  [11339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2057),
  [11341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2058),
  [11343] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_qualified_type, 3, 0, 82),
  [11345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3669),
  [11347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(785),
  [11349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6095),
  [11351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6096),
  [11353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6166),
  [11355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3289),
  [11357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2114),
  [11359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2115),
  [11361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2116),
  [11363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(844),
  [11365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6242),
  [11367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5846),
  [11369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [11371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2135),
  [11373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2140),
  [11375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2141),
  [11377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2142),
  [11379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6332),
  [11381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6333),
  [11383] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 3, 0, 12),
  [11385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1198),
  [11387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2157),
  [11389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2158),
  [11391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2159),
  [11393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2160),
  [11395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2161),
  [11397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [11399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4650),
  [11401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2491),
  [11403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5972),
  [11405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1471),
  [11407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(778),
  [11409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2189),
  [11411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2191),
  [11413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_arguments_repeat1, 3, 0, 0),
  [11415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2199),
  [11417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_struct_pattern_repeat1, 3, 0, 0),
  [11419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(279),
  [11421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5256),
  [11423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6287),
  [11425] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 4, 0, 45),
  [11427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1034),
  [11429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1688),
  [11431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4644),
  [11433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1035),
  [11435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1698),
  [11437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1699),
  [11439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1700),
  [11441] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 4, 0, 299),
  [11443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1036),
  [11445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1702),
  [11447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1307),
  [11449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6446),
  [11451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(945),
  [11453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6499),
  [11455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6455),
  [11457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4649),
  [11459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4658),
  [11461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [11463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(755),
  [11465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [11467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2239),
  [11469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4663),
  [11471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(946),
  [11473] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 3, 0, 208),
  [11475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6307),
  [11477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3417),
  [11479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2247),
  [11481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(292),
  [11483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(947),
  [11485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2255),
  [11487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2256),
  [11489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2257),
  [11491] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_use_bounds_repeat1, 2, 0, 210),
  [11493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2272),
  [11495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2273),
  [11497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2274),
  [11499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5047),
  [11501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2293),
  [11503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2298),
  [11505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2299),
  [11507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2300),
  [11509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [11511] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_self_parameter, 5, 0, 12),
  [11513] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 5, 0, 111),
  [11515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1045),
  [11517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1046),
  [11519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2327),
  [11521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1047),
  [11523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [11525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(608),
  [11527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6416),
  [11529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3410),
  [11531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [11533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2341),
  [11535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(325),
  [11537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(161),
  [11539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(248),
  [11541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2342),
  [11543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2343),
  [11545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3650),
  [11547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2351),
  [11549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lifetime_parameter, 2, 0, 93),
  [11551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2357),
  [11553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(609),
  [11555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2365),
  [11557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2366),
  [11559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2367),
  [11561] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6384),
  [11563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6485),
  [11565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(604),
  [11567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(894),
  [11569] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6405),
  [11571] = {.entry = {.count = 1, .reusable = false}}, SHIFT(6487),
  [11573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2386),
  [11575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2387),
  [11577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2388),
  [11579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2398),
  [11581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1682),
  [11583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2649),
  [11585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1164),
  [11587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2545),
  [11589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2014),
  [11591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2015),
  [11593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1131),
  [11595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [11597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2017),
  [11599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2654),
  [11601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6020),
  [11603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1078),
  [11605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5508),
  [11607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2020),
  [11609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(999),
  [11611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3838),
  [11613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1557),
  [11615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2657),
  [11617] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__inner_line_doc_comment_marker, 1, 0, 0),
  [11619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1366),
  [11621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1964),
  [11623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3818),
  [11625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6323),
  [11627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(840),
  [11629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5659),
  [11631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(698),
  [11633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3819),
  [11635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(899),
  [11637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(656),
  [11639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3922),
  [11641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4703),
  [11643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2659),
  [11645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6498),
  [11647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1013),
  [11649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2753),
  [11651] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bracketed_type, 3, 0, 0),
  [11653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [11655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1375),
  [11657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3236),
  [11659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2749),
  [11661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [11663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1348),
  [11665] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_pattern, 3, 0, 237),
  [11667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(703),
  [11669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(974),
  [11671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1946),
  [11673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3826),
  [11675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2053),
  [11677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2408),
  [11679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2055),
  [11681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2666),
  [11683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1627),
  [11685] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6161),
  [11687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [11689] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2060),
  [11691] = {.entry = {.count = 1, .reusable = true}}, SHIFT(985),
  [11693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(591),
  [11695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [11697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2063),
  [11699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2065),
  [11701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6165),
  [11703] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6496),
  [11705] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1973),
  [11707] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4967),
  [11709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2669),
  [11711] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1189),
  [11713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2072),
  [11715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2802),
  [11717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1643),
  [11719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(710),
  [11721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_arguments_declaration, 3, 0, 0),
  [11723] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2748),
  [11725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(273),
  [11727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2079),
  [11729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1983),
  [11731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6040),
  [11733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1934),
  [11735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4296),
  [11737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1534),
  [11739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3821),
  [11741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2084),
  [11743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6126),
  [11745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(968),
  [11747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [11749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(843),
  [11751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6087),
  [11753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4201),
  [11755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [11757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1920),
  [11759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(867),
  [11761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3146),
  [11763] = {.entry = {.count = 1, .reusable = true}}, SHIFT(986),
  [11765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [11767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [11769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1205),
  [11771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2847),
  [11773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2096),
  [11775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2097),
  [11777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4209),
  [11779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2687),
  [11781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2100),
  [11783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2102),
  [11785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [11787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5564),
  [11789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1194),
  [11791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(594),
  [11793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(338),
  [11795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1149),
  [11797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2107),
  [11799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(937),
  [11801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6155),
  [11803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2689),
  [11805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1629),
  [11807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3539),
  [11809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(987),
  [11811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2112),
  [11813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2113),
  [11815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [11817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6383),
  [11819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4672),
  [11821] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6063),
  [11823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6168),
  [11825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6064),
  [11827] = {.entry = {.count = 1, .reusable = true}}, SHIFT(938),
  [11829] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5494),
  [11831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1931),
  [11833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3817),
  [11835] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1247),
  [11837] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1125),
  [11839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6503),
  [11841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1960),
  [11843] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1521),
  [11845] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3237),
  [11847] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3844),
  [11849] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2845),
  [11851] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2132),
  [11853] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [11855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6249),
  [11857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1151),
  [11859] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2133),
  [11861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1601),
  [11863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4522),
  [11865] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [11867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2136),
  [11869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2137),
  [11871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2138),
  [11873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2698),
  [11875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(736),
  [11877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4096),
  [11879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5854),
  [11881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(465),
  [11883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1966),
  [11885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5409),
  [11887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(505),
  [11889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3661),
  [11891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [11893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2776),
  [11895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1930),
  [11897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(741),
  [11899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5176),
  [11901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3820),
  [11903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6053),
  [11905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1926),
  [11907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5006),
  [11909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3376),
  [11911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1672),
  [11913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [11915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2401),
  [11917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [11919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3214),
  [11921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2547),
  [11923] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2807),
  [11925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(829),
  [11927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6144),
  [11929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(860),
  [11931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1371),
  [11933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3833),
  [11935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4852),
  [11937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1986),
  [11939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4133),
  [11941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2185),
  [11943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2186),
  [11945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2395),
  [11947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3901),
  [11949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3843),
  [11951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3956),
  [11953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4064),
  [11955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(772),
  [11957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [11959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5079),
  [11961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [11963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2192),
  [11965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2193),
  [11967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3869),
  [11969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3870),
  [11971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1030),
  [11973] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2195),
  [11975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3842),
  [11977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [11979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2198),
  [11981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3903),
  [11983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3509),
  [11985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1321),
  [11987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1031),
  [11989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5704),
  [11991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1032),
  [11993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(942),
  [11995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6269),
  [11997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(971),
  [11999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [12001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(943),
  [12003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2211),
  [12005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2212),
  [12007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2504),
  [12009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1203),
  [12011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5493),
  [12013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2215),
  [12015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1057),
  [12017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4087),
  [12019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2217),
  [12021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4360),
  [12023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5502),
  [12025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4454),
  [12027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4357),
  [12029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5438),
  [12031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1535),
  [12033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5506),
  [12035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [12037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5182),
  [12039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4066),
  [12041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5629),
  [12043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4653),
  [12045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5745),
  [12047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5901),
  [12049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
  [12051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [12053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1407),
  [12055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(862),
  [12057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4236),
  [12059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1050),
  [12061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2222),
  [12063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6021),
  [12065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4278),
  [12067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(780),
  [12069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2516),
  [12071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5778),
  [12073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2232),
  [12075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6140),
  [12077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2233),
  [12079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [12081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1350),
  [12083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5553),
  [12085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2235),
  [12087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5556),
  [12089] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 2, 0, 83),
  [12091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [12093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1968),
  [12095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5643),
  [12097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4666),
  [12099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(24),
  [12101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2238),
  [12103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3879),
  [12105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2730),
  [12107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1705),
  [12109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6450),
  [12111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(28),
  [12113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6006),
  [12115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1404),
  [12117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [12119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5628),
  [12121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5642),
  [12123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6283),
  [12125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3849),
  [12127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5819),
  [12129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4680),
  [12131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5820),
  [12133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2735),
  [12135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4218),
  [12137] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 2, 0, 84),
  [12139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6345),
  [12141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3295),
  [12143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6379),
  [12145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1731),
  [12147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4227),
  [12149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1985),
  [12151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2252),
  [12153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5259),
  [12155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1736),
  [12157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2254),
  [12159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3881),
  [12161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3891),
  [12163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3906),
  [12165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5138),
  [12167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1510),
  [12169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4245),
  [12171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1316),
  [12173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5483),
  [12175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5187),
  [12177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3892),
  [12179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3907),
  [12181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4216),
  [12183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2270),
  [12185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2271),
  [12187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1754),
  [12189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4074),
  [12191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(917),
  [12193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3317),
  [12195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1761),
  [12197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4401),
  [12199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [12201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1766),
  [12203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3850),
  [12205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3876),
  [12207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1770),
  [12209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5990),
  [12211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5958),
  [12213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(635),
  [12215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2290),
  [12217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2291),
  [12219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(636),
  [12221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1783),
  [12223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2294),
  [12225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2295),
  [12227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2296),
  [12229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1502),
  [12231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [12233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(137),
  [12235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1800),
  [12237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(779),
  [12239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1044),
  [12241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1807),
  [12243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6054),
  [12245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5557),
  [12247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4572),
  [12249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [12251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [12253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1129),
  [12255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(918),
  [12257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2601),
  [12259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1493),
  [12261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1828),
  [12263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2320),
  [12265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2321),
  [12267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3186),
  [12269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1394),
  [12271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1838),
  [12273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2323),
  [12275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2326),
  [12277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1376),
  [12279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2556),
  [12281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4073),
  [12283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5981),
  [12285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3193),
  [12287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3814),
  [12289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1879),
  [12291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1890),
  [12293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2337),
  [12295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1261),
  [12297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2338),
  [12299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1904),
  [12301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3815),
  [12303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4805),
  [12305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(653),
  [12307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2339),
  [12309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3411),
  [12311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6243),
  [12313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3322),
  [12315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1962),
  [12317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5509),
  [12319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3413),
  [12321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4812),
  [12323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6274),
  [12325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4109),
  [12327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5219),
  [12329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3421),
  [12331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6284),
  [12333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3422),
  [12335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6288),
  [12337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6290),
  [12339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6292),
  [12341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3816),
  [12343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3910),
  [12345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5882),
  [12347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1102),
  [12349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6043),
  [12351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4115),
  [12353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6334),
  [12355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5335),
  [12357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6044),
  [12359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5366),
  [12361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(901),
  [12363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2350),
  [12365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4148),
  [12367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6357),
  [12369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6338),
  [12371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1961),
  [12373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5520),
  [12375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6501),
  [12377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(838),
  [12379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6367),
  [12381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(877),
  [12383] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_arguments_declaration, 2, 0, 0),
  [12385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5382),
  [12387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(878),
  [12389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2362),
  [12391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(969),
  [12393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2364),
  [12395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2647),
  [12397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6384),
  [12399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2861),
  [12401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6391),
  [12403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(690),
  [12405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [12407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6346),
  [12409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2525),
  [12411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6423),
  [12413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2448),
  [12415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5859),
  [12417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3865),
  [12419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1971),
  [12421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3959),
  [12423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4221),
  [12425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6405),
  [12427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [12429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6406),
  [12431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2382),
  [12433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6410),
  [12435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2007),
  [12437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2008),
  [12439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2383),
  [12441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5926),
  [12443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2384),
  [12445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2928),
  [12447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1011),
  [12449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1138),
  [12451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1536),
  [12453] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 3, 0, 0),
  [12455] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 3, 0, 19),
  [12457] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_comment, 2, 0, 0),
  [12459] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 4, 0, 66),
  [12461] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_comment, 2, 0, 0),
  [12463] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_line_comment, 3, 0, 18),
};

enum ts_external_scanner_symbol_identifiers {
  ts_external_token_string_content = 0,
  ts_external_token__raw_string_literal_start = 1,
  ts_external_token_raw_string_literal_content = 2,
  ts_external_token__raw_string_literal_end = 3,
  ts_external_token_float_literal = 4,
  ts_external_token__outer_block_doc_comment_start = 5,
  ts_external_token__outer_line_doc_comment_start = 6,
  ts_external_token__inner_block_doc_comment_marker = 7,
  ts_external_token__block_comment_content = 8,
  ts_external_token__line_doc_content = 9,
  ts_external_token__frontmatter_start = 10,
  ts_external_token_frontmatter_content = 11,
  ts_external_token__frontmatter_end = 12,
  ts_external_token__error_sentinel = 13,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token_string_content] = sym_string_content,
  [ts_external_token__raw_string_literal_start] = sym__raw_string_literal_start,
  [ts_external_token_raw_string_literal_content] = sym_raw_string_literal_content,
  [ts_external_token__raw_string_literal_end] = sym__raw_string_literal_end,
  [ts_external_token_float_literal] = sym_float_literal,
  [ts_external_token__outer_block_doc_comment_start] = sym__outer_block_doc_comment_start,
  [ts_external_token__outer_line_doc_comment_start] = sym__outer_line_doc_comment_start,
  [ts_external_token__inner_block_doc_comment_marker] = sym__inner_block_doc_comment_marker,
  [ts_external_token__block_comment_content] = sym__block_comment_content,
  [ts_external_token__line_doc_content] = sym__line_doc_content,
  [ts_external_token__frontmatter_start] = sym__frontmatter_start,
  [ts_external_token_frontmatter_content] = sym_frontmatter_content,
  [ts_external_token__frontmatter_end] = sym__frontmatter_end,
  [ts_external_token__error_sentinel] = sym__error_sentinel,
};

static const bool ts_external_scanner_states[16][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token_string_content] = true,
    [ts_external_token__raw_string_literal_start] = true,
    [ts_external_token_raw_string_literal_content] = true,
    [ts_external_token__raw_string_literal_end] = true,
    [ts_external_token_float_literal] = true,
    [ts_external_token__outer_block_doc_comment_start] = true,
    [ts_external_token__outer_line_doc_comment_start] = true,
    [ts_external_token__inner_block_doc_comment_marker] = true,
    [ts_external_token__block_comment_content] = true,
    [ts_external_token__line_doc_content] = true,
    [ts_external_token__frontmatter_start] = true,
    [ts_external_token_frontmatter_content] = true,
    [ts_external_token__frontmatter_end] = true,
    [ts_external_token__error_sentinel] = true,
  },
  [2] = {
    [ts_external_token__raw_string_literal_start] = true,
    [ts_external_token_float_literal] = true,
    [ts_external_token__outer_block_doc_comment_start] = true,
    [ts_external_token__outer_line_doc_comment_start] = true,
    [ts_external_token__frontmatter_start] = true,
  },
  [3] = {
    [ts_external_token__raw_string_literal_start] = true,
    [ts_external_token_float_literal] = true,
    [ts_external_token__outer_block_doc_comment_start] = true,
    [ts_external_token__outer_line_doc_comment_start] = true,
  },
  [4] = {
    [ts_external_token__raw_string_literal_start] = true,
    [ts_external_token_float_literal] = true,
  },
  [5] = {
    [ts_external_token__outer_block_doc_comment_start] = true,
    [ts_external_token__outer_line_doc_comment_start] = true,
  },
  [6] = {
    [ts_external_token__raw_string_literal_start] = true,
  },
  [7] = {
    [ts_external_token_string_content] = true,
  },
  [8] = {
    [ts_external_token__inner_block_doc_comment_marker] = true,
    [ts_external_token__block_comment_content] = true,
  },
  [9] = {
    [ts_external_token__block_comment_content] = true,
  },
  [10] = {
    [ts_external_token_float_literal] = true,
  },
  [11] = {
    [ts_external_token__line_doc_content] = true,
  },
  [12] = {
    [ts_external_token__frontmatter_end] = true,
  },
  [13] = {
    [ts_external_token__raw_string_literal_end] = true,
  },
  [14] = {
    [ts_external_token_raw_string_literal_content] = true,
  },
  [15] = {
    [ts_external_token_frontmatter_content] = true,
  },
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_rust_orchard_external_scanner_create(void);
void tree_sitter_rust_orchard_external_scanner_destroy(void *);
bool tree_sitter_rust_orchard_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_rust_orchard_external_scanner_serialize(void *, char *);
void tree_sitter_rust_orchard_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#else
#define TS_PUBLIC __attribute__((visibility("default")))
#endif

TS_PUBLIC const TSLanguage *tree_sitter_rust_orchard(void) {
  static const TSLanguage language = {
    .abi_version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = (const void*)ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_rust_orchard_external_scanner_create,
      tree_sitter_rust_orchard_external_scanner_destroy,
      tree_sitter_rust_orchard_external_scanner_scan,
      tree_sitter_rust_orchard_external_scanner_serialize,
      tree_sitter_rust_orchard_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
